function makeExpectedMatch(arr) {
    expectedMatch = {
        x: arr.length
    }
}
var expected = makeExpectedMatch(new Int32Array);
this.toSource();

function testTypedArrayOther() {
    var ar = new Int32Array;
    for (; i < ar; ++i) {
        ar[i] = i;
    }
    for (var i = 0; i<40; i++) {
    }
}
testTypedArrayOther();
