(function(stdlib) {
        "use asm"
        var log = stdlib.Math.log

        function f(x) {
            x = +x
            var y = 3.
            return 0, y
            return +log(x) + y
        }

        return f
})(this)
