
try { new isNaN; } catch (e) {}
new Array;
new Boolean;
new Date;
new Number;
new Object;
new String;
try { new [].push(4); } catch (e) {}
try { new [1,2,3].pop(); } catch (e) {}
try { new "a,b,c".split(","); } catch (e) {}
try { new Array.concat(3); } catch (e) {}
