// Binary: cache/js-dbg-64-d3215d1e985a-linux
// Flags: -m -n -a
//
ForIn_1( { length:4, company:"netscape", year:2000, 0:"zero" } );
function ForIn_1( object ) {
  PropertyArray = new Array();
  for ( PropertyArray[PropertyArray.length] in object ) {
      object[1];
  }
}
