gczeal(2);
for (var x = 0; x < 99; x++) {
    (function() {
        return function() {
            new function(y) {
                return {
                    e: q => q, function() {}
                }
            }
        }
    })()()
}
