// Binary: cache/js-dbg-32-1a1611bb1063-linux
// Flags: -j
//
var e;
function w(f, y) {
    e = 2 * f;
    for (var i = 0; i < 1; i++) {
    }
}
for (var u = 0; u < 4; ++u) {
    w(undefined, 0);
    w(1, (2) * 2);
}
