/* Generated by make_unicode.py DO NOT MODIFY */
/* Unicode version: 12.1.0 */

/*
 * Any copyright is dedicated to the Public Domain.
 * http://creativecommons.org/licenses/publicdomain/
 */
var mapping = [
  ["\u0000", "\u0000"], /* <control> (NULL) */
  ["\u0001", "\u0001"], /* <control> (START OF HEADING) */
  ["\u0002", "\u0002"], /* <control> (START OF TEXT) */
  ["\u0003", "\u0003"], /* <control> (END OF TEXT) */
  ["\u0004", "\u0004"], /* <control> (END OF TRANSMISSION) */
  ["\u0005", "\u0005"], /* <control> (ENQUIRY) */
  ["\u0006", "\u0006"], /* <control> (ACKNOWLEDGE) */
  ["\u0007", "\u0007"], /* <control> (BELL) */
  ["\u0008", "\u0008"], /* <control> (BACKSPACE) */
  ["\u0009", "\u0009"], /* <control> (CHARACTER TABULATION) */
  ["\u000A", "\u000A"], /* <control> (LINE FEED (LF)) */
  ["\u000B", "\u000B"], /* <control> (LINE TABULATION) */
  ["\u000C", "\u000C"], /* <control> (FORM FEED (FF)) */
  ["\u000D", "\u000D"], /* <control> (CARRIAGE RETURN (CR)) */
  ["\u000E", "\u000E"], /* <control> (SHIFT OUT) */
  ["\u000F", "\u000F"], /* <control> (SHIFT IN) */
  ["\u0010", "\u0010"], /* <control> (DATA LINK ESCAPE) */
  ["\u0011", "\u0011"], /* <control> (DEVICE CONTROL ONE) */
  ["\u0012", "\u0012"], /* <control> (DEVICE CONTROL TWO) */
  ["\u0013", "\u0013"], /* <control> (DEVICE CONTROL THREE) */
  ["\u0014", "\u0014"], /* <control> (DEVICE CONTROL FOUR) */
  ["\u0015", "\u0015"], /* <control> (NEGATIVE ACKNOWLEDGE) */
  ["\u0016", "\u0016"], /* <control> (SYNCHRONOUS IDLE) */
  ["\u0017", "\u0017"], /* <control> (END OF TRANSMISSION BLOCK) */
  ["\u0018", "\u0018"], /* <control> (CANCEL) */
  ["\u0019", "\u0019"], /* <control> (END OF MEDIUM) */
  ["\u001A", "\u001A"], /* <control> (SUBSTITUTE) */
  ["\u001B", "\u001B"], /* <control> (ESCAPE) */
  ["\u001C", "\u001C"], /* <control> (INFORMATION SEPARATOR FOUR) */
  ["\u001D", "\u001D"], /* <control> (INFORMATION SEPARATOR THREE) */
  ["\u001E", "\u001E"], /* <control> (INFORMATION SEPARATOR TWO) */
  ["\u001F", "\u001F"], /* <control> (INFORMATION SEPARATOR ONE) */
  ["\u0020", "\u0020"], /* SPACE */
  ["\u0021", "\u0021"], /* EXCLAMATION MARK */
  ["\u0022", "\u0022"], /* QUOTATION MARK */
  ["\u0023", "\u0023"], /* NUMBER SIGN */
  ["\u0024", "\u0024"], /* DOLLAR SIGN */
  ["\u0025", "\u0025"], /* PERCENT SIGN */
  ["\u0026", "\u0026"], /* AMPERSAND */
  ["\u0027", "\u0027"], /* APOSTROPHE (APOSTROPHE-QUOTE) */
  ["\u0028", "\u0028"], /* LEFT PARENTHESIS (OPENING PARENTHESIS) */
  ["\u0029", "\u0029"], /* RIGHT PARENTHESIS (CLOSING PARENTHESIS) */
  ["\u002A", "\u002A"], /* ASTERISK */
  ["\u002B", "\u002B"], /* PLUS SIGN */
  ["\u002C", "\u002C"], /* COMMA */
  ["\u002D", "\u002D"], /* HYPHEN-MINUS */
  ["\u002E", "\u002E"], /* FULL STOP (PERIOD) */
  ["\u002F", "\u002F"], /* SOLIDUS (SLASH) */
  ["\u0030", "\u0030"], /* DIGIT ZERO */
  ["\u0031", "\u0031"], /* DIGIT ONE */
  ["\u0032", "\u0032"], /* DIGIT TWO */
  ["\u0033", "\u0033"], /* DIGIT THREE */
  ["\u0034", "\u0034"], /* DIGIT FOUR */
  ["\u0035", "\u0035"], /* DIGIT FIVE */
  ["\u0036", "\u0036"], /* DIGIT SIX */
  ["\u0037", "\u0037"], /* DIGIT SEVEN */
  ["\u0038", "\u0038"], /* DIGIT EIGHT */
  ["\u0039", "\u0039"], /* DIGIT NINE */
  ["\u003A", "\u003A"], /* COLON */
  ["\u003B", "\u003B"], /* SEMICOLON */
  ["\u003C", "\u003C"], /* LESS-THAN SIGN */
  ["\u003D", "\u003D"], /* EQUALS SIGN */
  ["\u003E", "\u003E"], /* GREATER-THAN SIGN */
  ["\u003F", "\u003F"], /* QUESTION MARK */
  ["\u0040", "\u0040"], /* COMMERCIAL AT */
  ["\u0041", "\u0061"], /* LATIN CAPITAL LETTER A */
  ["\u0042", "\u0062"], /* LATIN CAPITAL LETTER B */
  ["\u0043", "\u0063"], /* LATIN CAPITAL LETTER C */
  ["\u0044", "\u0064"], /* LATIN CAPITAL LETTER D */
  ["\u0045", "\u0065"], /* LATIN CAPITAL LETTER E */
  ["\u0046", "\u0066"], /* LATIN CAPITAL LETTER F */
  ["\u0047", "\u0067"], /* LATIN CAPITAL LETTER G */
  ["\u0048", "\u0068"], /* LATIN CAPITAL LETTER H */
  ["\u0049", "\u0069"], /* LATIN CAPITAL LETTER I */
  ["\u004A", "\u006A"], /* LATIN CAPITAL LETTER J */
  ["\u004B", "\u006B"], /* LATIN CAPITAL LETTER K */
  ["\u004C", "\u006C"], /* LATIN CAPITAL LETTER L */
  ["\u004D", "\u006D"], /* LATIN CAPITAL LETTER M */
  ["\u004E", "\u006E"], /* LATIN CAPITAL LETTER N */
  ["\u004F", "\u006F"], /* LATIN CAPITAL LETTER O */
  ["\u0050", "\u0070"], /* LATIN CAPITAL LETTER P */
  ["\u0051", "\u0071"], /* LATIN CAPITAL LETTER Q */
  ["\u0052", "\u0072"], /* LATIN CAPITAL LETTER R */
  ["\u0053", "\u0073"], /* LATIN CAPITAL LETTER S */
  ["\u0054", "\u0074"], /* LATIN CAPITAL LETTER T */
  ["\u0055", "\u0075"], /* LATIN CAPITAL LETTER U */
  ["\u0056", "\u0076"], /* LATIN CAPITAL LETTER V */
  ["\u0057", "\u0077"], /* LATIN CAPITAL LETTER W */
  ["\u0058", "\u0078"], /* LATIN CAPITAL LETTER X */
  ["\u0059", "\u0079"], /* LATIN CAPITAL LETTER Y */
  ["\u005A", "\u007A"], /* LATIN CAPITAL LETTER Z */
  ["\u005B", "\u005B"], /* LEFT SQUARE BRACKET (OPENING SQUARE BRACKET) */
  ["\u005C", "\u005C"], /* REVERSE SOLIDUS (BACKSLASH) */
  ["\u005D", "\u005D"], /* RIGHT SQUARE BRACKET (CLOSING SQUARE BRACKET) */
  ["\u005E", "\u005E"], /* CIRCUMFLEX ACCENT (SPACING CIRCUMFLEX) */
  ["\u005F", "\u005F"], /* LOW LINE (SPACING UNDERSCORE) */
  ["\u0060", "\u0060"], /* GRAVE ACCENT (SPACING GRAVE) */
  ["\u0041", "\u0061"], /* LATIN SMALL LETTER A */
  ["\u0042", "\u0062"], /* LATIN SMALL LETTER B */
  ["\u0043", "\u0063"], /* LATIN SMALL LETTER C */
  ["\u0044", "\u0064"], /* LATIN SMALL LETTER D */
  ["\u0045", "\u0065"], /* LATIN SMALL LETTER E */
  ["\u0046", "\u0066"], /* LATIN SMALL LETTER F */
  ["\u0047", "\u0067"], /* LATIN SMALL LETTER G */
  ["\u0048", "\u0068"], /* LATIN SMALL LETTER H */
  ["\u0049", "\u0069"], /* LATIN SMALL LETTER I */
  ["\u004A", "\u006A"], /* LATIN SMALL LETTER J */
  ["\u004B", "\u006B"], /* LATIN SMALL LETTER K */
  ["\u004C", "\u006C"], /* LATIN SMALL LETTER L */
  ["\u004D", "\u006D"], /* LATIN SMALL LETTER M */
  ["\u004E", "\u006E"], /* LATIN SMALL LETTER N */
  ["\u004F", "\u006F"], /* LATIN SMALL LETTER O */
  ["\u0050", "\u0070"], /* LATIN SMALL LETTER P */
  ["\u0051", "\u0071"], /* LATIN SMALL LETTER Q */
  ["\u0052", "\u0072"], /* LATIN SMALL LETTER R */
  ["\u0053", "\u0073"], /* LATIN SMALL LETTER S */
  ["\u0054", "\u0074"], /* LATIN SMALL LETTER T */
  ["\u0055", "\u0075"], /* LATIN SMALL LETTER U */
  ["\u0056", "\u0076"], /* LATIN SMALL LETTER V */
  ["\u0057", "\u0077"], /* LATIN SMALL LETTER W */
  ["\u0058", "\u0078"], /* LATIN SMALL LETTER X */
  ["\u0059", "\u0079"], /* LATIN SMALL LETTER Y */
  ["\u005A", "\u007A"], /* LATIN SMALL LETTER Z */
  ["\u007B", "\u007B"], /* LEFT CURLY BRACKET (OPENING CURLY BRACKET) */
  ["\u007C", "\u007C"], /* VERTICAL LINE (VERTICAL BAR) */
  ["\u007D", "\u007D"], /* RIGHT CURLY BRACKET (CLOSING CURLY BRACKET) */
  ["\u007E", "\u007E"], /* TILDE */
  ["\u007F", "\u007F"], /* <control> (DELETE) */
  ["\u0080", "\u0080"], /* <control> */
  ["\u0081", "\u0081"], /* <control> */
  ["\u0082", "\u0082"], /* <control> (BREAK PERMITTED HERE) */
  ["\u0083", "\u0083"], /* <control> (NO BREAK HERE) */
  ["\u0084", "\u0084"], /* <control> */
  ["\u0085", "\u0085"], /* <control> (NEXT LINE (NEL)) */
  ["\u0086", "\u0086"], /* <control> (START OF SELECTED AREA) */
  ["\u0087", "\u0087"], /* <control> (END OF SELECTED AREA) */
  ["\u0088", "\u0088"], /* <control> (CHARACTER TABULATION SET) */
  ["\u0089", "\u0089"], /* <control> (CHARACTER TABULATION WITH JUSTIFICATION) */
  ["\u008A", "\u008A"], /* <control> (LINE TABULATION SET) */
  ["\u008B", "\u008B"], /* <control> (PARTIAL LINE FORWARD) */
  ["\u008C", "\u008C"], /* <control> (PARTIAL LINE BACKWARD) */
  ["\u008D", "\u008D"], /* <control> (REVERSE LINE FEED) */
  ["\u008E", "\u008E"], /* <control> (SINGLE SHIFT TWO) */
  ["\u008F", "\u008F"], /* <control> (SINGLE SHIFT THREE) */
  ["\u0090", "\u0090"], /* <control> (DEVICE CONTROL STRING) */
  ["\u0091", "\u0091"], /* <control> (PRIVATE USE ONE) */
  ["\u0092", "\u0092"], /* <control> (PRIVATE USE TWO) */
  ["\u0093", "\u0093"], /* <control> (SET TRANSMIT STATE) */
  ["\u0094", "\u0094"], /* <control> (CANCEL CHARACTER) */
  ["\u0095", "\u0095"], /* <control> (MESSAGE WAITING) */
  ["\u0096", "\u0096"], /* <control> (START OF GUARDED AREA) */
  ["\u0097", "\u0097"], /* <control> (END OF GUARDED AREA) */
  ["\u0098", "\u0098"], /* <control> (START OF STRING) */
  ["\u0099", "\u0099"], /* <control> */
  ["\u009A", "\u009A"], /* <control> (SINGLE CHARACTER INTRODUCER) */
  ["\u009B", "\u009B"], /* <control> (CONTROL SEQUENCE INTRODUCER) */
  ["\u009C", "\u009C"], /* <control> (STRING TERMINATOR) */
  ["\u009D", "\u009D"], /* <control> (OPERATING SYSTEM COMMAND) */
  ["\u009E", "\u009E"], /* <control> (PRIVACY MESSAGE) */
  ["\u009F", "\u009F"], /* <control> (APPLICATION PROGRAM COMMAND) */
  ["\u00A0", "\u00A0"], /* NO-BREAK SPACE (NON-BREAKING SPACE) */
  ["\u00A1", "\u00A1"], /* INVERTED EXCLAMATION MARK */
  ["\u00A2", "\u00A2"], /* CENT SIGN */
  ["\u00A3", "\u00A3"], /* POUND SIGN */
  ["\u00A4", "\u00A4"], /* CURRENCY SIGN */
  ["\u00A5", "\u00A5"], /* YEN SIGN */
  ["\u00A6", "\u00A6"], /* BROKEN BAR (BROKEN VERTICAL BAR) */
  ["\u00A7", "\u00A7"], /* SECTION SIGN */
  ["\u00A8", "\u00A8"], /* DIAERESIS (SPACING DIAERESIS) */
  ["\u00A9", "\u00A9"], /* COPYRIGHT SIGN */
  ["\u00AA", "\u00AA"], /* FEMININE ORDINAL INDICATOR */
  ["\u00AB", "\u00AB"], /* LEFT-POINTING DOUBLE ANGLE QUOTATION MARK (LEFT POINTING GUILLEMET) */
  ["\u00AC", "\u00AC"], /* NOT SIGN */
  ["\u00AD", "\u00AD"], /* SOFT HYPHEN */
  ["\u00AE", "\u00AE"], /* REGISTERED SIGN (REGISTERED TRADE MARK SIGN) */
  ["\u00AF", "\u00AF"], /* MACRON (SPACING MACRON) */
  ["\u00B0", "\u00B0"], /* DEGREE SIGN */
  ["\u00B1", "\u00B1"], /* PLUS-MINUS SIGN (PLUS-OR-MINUS SIGN) */
  ["\u00B2", "\u00B2"], /* SUPERSCRIPT TWO (SUPERSCRIPT DIGIT TWO) */
  ["\u00B3", "\u00B3"], /* SUPERSCRIPT THREE (SUPERSCRIPT DIGIT THREE) */
  ["\u00B4", "\u00B4"], /* ACUTE ACCENT (SPACING ACUTE) */
  ["\u039C", "\u00B5"], /* MICRO SIGN */
  ["\u00B6", "\u00B6"], /* PILCROW SIGN (PARAGRAPH SIGN) */
  ["\u00B7", "\u00B7"], /* MIDDLE DOT */
  ["\u00B8", "\u00B8"], /* CEDILLA (SPACING CEDILLA) */
  ["\u00B9", "\u00B9"], /* SUPERSCRIPT ONE (SUPERSCRIPT DIGIT ONE) */
  ["\u00BA", "\u00BA"], /* MASCULINE ORDINAL INDICATOR */
  ["\u00BB", "\u00BB"], /* RIGHT-POINTING DOUBLE ANGLE QUOTATION MARK (RIGHT POINTING GUILLEMET) */
  ["\u00BC", "\u00BC"], /* VULGAR FRACTION ONE QUARTER (FRACTION ONE QUARTER) */
  ["\u00BD", "\u00BD"], /* VULGAR FRACTION ONE HALF (FRACTION ONE HALF) */
  ["\u00BE", "\u00BE"], /* VULGAR FRACTION THREE QUARTERS (FRACTION THREE QUARTERS) */
  ["\u00BF", "\u00BF"], /* INVERTED QUESTION MARK */
  ["\u00C0", "\u00E0"], /* LATIN CAPITAL LETTER A WITH GRAVE (LATIN CAPITAL LETTER A GRAVE) */
  ["\u00C1", "\u00E1"], /* LATIN CAPITAL LETTER A WITH ACUTE (LATIN CAPITAL LETTER A ACUTE) */
  ["\u00C2", "\u00E2"], /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX (LATIN CAPITAL LETTER A CIRCUMFLEX) */
  ["\u00C3", "\u00E3"], /* LATIN CAPITAL LETTER A WITH TILDE (LATIN CAPITAL LETTER A TILDE) */
  ["\u00C4", "\u00E4"], /* LATIN CAPITAL LETTER A WITH DIAERESIS (LATIN CAPITAL LETTER A DIAERESIS) */
  ["\u00C5", "\u00E5"], /* LATIN CAPITAL LETTER A WITH RING ABOVE (LATIN CAPITAL LETTER A RING) */
  ["\u00C6", "\u00E6"], /* LATIN CAPITAL LETTER AE (LATIN CAPITAL LETTER A E) */
  ["\u00C7", "\u00E7"], /* LATIN CAPITAL LETTER C WITH CEDILLA (LATIN CAPITAL LETTER C CEDILLA) */
  ["\u00C8", "\u00E8"], /* LATIN CAPITAL LETTER E WITH GRAVE (LATIN CAPITAL LETTER E GRAVE) */
  ["\u00C9", "\u00E9"], /* LATIN CAPITAL LETTER E WITH ACUTE (LATIN CAPITAL LETTER E ACUTE) */
  ["\u00CA", "\u00EA"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX (LATIN CAPITAL LETTER E CIRCUMFLEX) */
  ["\u00CB", "\u00EB"], /* LATIN CAPITAL LETTER E WITH DIAERESIS (LATIN CAPITAL LETTER E DIAERESIS) */
  ["\u00CC", "\u00EC"], /* LATIN CAPITAL LETTER I WITH GRAVE (LATIN CAPITAL LETTER I GRAVE) */
  ["\u00CD", "\u00ED"], /* LATIN CAPITAL LETTER I WITH ACUTE (LATIN CAPITAL LETTER I ACUTE) */
  ["\u00CE", "\u00EE"], /* LATIN CAPITAL LETTER I WITH CIRCUMFLEX (LATIN CAPITAL LETTER I CIRCUMFLEX) */
  ["\u00CF", "\u00EF"], /* LATIN CAPITAL LETTER I WITH DIAERESIS (LATIN CAPITAL LETTER I DIAERESIS) */
  ["\u00D0", "\u00F0"], /* LATIN CAPITAL LETTER ETH */
  ["\u00D1", "\u00F1"], /* LATIN CAPITAL LETTER N WITH TILDE (LATIN CAPITAL LETTER N TILDE) */
  ["\u00D2", "\u00F2"], /* LATIN CAPITAL LETTER O WITH GRAVE (LATIN CAPITAL LETTER O GRAVE) */
  ["\u00D3", "\u00F3"], /* LATIN CAPITAL LETTER O WITH ACUTE (LATIN CAPITAL LETTER O ACUTE) */
  ["\u00D4", "\u00F4"], /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX (LATIN CAPITAL LETTER O CIRCUMFLEX) */
  ["\u00D5", "\u00F5"], /* LATIN CAPITAL LETTER O WITH TILDE (LATIN CAPITAL LETTER O TILDE) */
  ["\u00D6", "\u00F6"], /* LATIN CAPITAL LETTER O WITH DIAERESIS (LATIN CAPITAL LETTER O DIAERESIS) */
  ["\u00D7", "\u00D7"], /* MULTIPLICATION SIGN */
  ["\u00D8", "\u00F8"], /* LATIN CAPITAL LETTER O WITH STROKE (LATIN CAPITAL LETTER O SLASH) */
  ["\u00D9", "\u00F9"], /* LATIN CAPITAL LETTER U WITH GRAVE (LATIN CAPITAL LETTER U GRAVE) */
  ["\u00DA", "\u00FA"], /* LATIN CAPITAL LETTER U WITH ACUTE (LATIN CAPITAL LETTER U ACUTE) */
  ["\u00DB", "\u00FB"], /* LATIN CAPITAL LETTER U WITH CIRCUMFLEX (LATIN CAPITAL LETTER U CIRCUMFLEX) */
  ["\u00DC", "\u00FC"], /* LATIN CAPITAL LETTER U WITH DIAERESIS (LATIN CAPITAL LETTER U DIAERESIS) */
  ["\u00DD", "\u00FD"], /* LATIN CAPITAL LETTER Y WITH ACUTE (LATIN CAPITAL LETTER Y ACUTE) */
  ["\u00DE", "\u00FE"], /* LATIN CAPITAL LETTER THORN */
  ["\u0053\u0053", "\u00DF"], /* LATIN SMALL LETTER SHARP S */
  ["\u00C0", "\u00E0"], /* LATIN SMALL LETTER A WITH GRAVE (LATIN SMALL LETTER A GRAVE) */
  ["\u00C1", "\u00E1"], /* LATIN SMALL LETTER A WITH ACUTE (LATIN SMALL LETTER A ACUTE) */
  ["\u00C2", "\u00E2"], /* LATIN SMALL LETTER A WITH CIRCUMFLEX (LATIN SMALL LETTER A CIRCUMFLEX) */
  ["\u00C3", "\u00E3"], /* LATIN SMALL LETTER A WITH TILDE (LATIN SMALL LETTER A TILDE) */
  ["\u00C4", "\u00E4"], /* LATIN SMALL LETTER A WITH DIAERESIS (LATIN SMALL LETTER A DIAERESIS) */
  ["\u00C5", "\u00E5"], /* LATIN SMALL LETTER A WITH RING ABOVE (LATIN SMALL LETTER A RING) */
  ["\u00C6", "\u00E6"], /* LATIN SMALL LETTER AE (LATIN SMALL LETTER A E) */
  ["\u00C7", "\u00E7"], /* LATIN SMALL LETTER C WITH CEDILLA (LATIN SMALL LETTER C CEDILLA) */
  ["\u00C8", "\u00E8"], /* LATIN SMALL LETTER E WITH GRAVE (LATIN SMALL LETTER E GRAVE) */
  ["\u00C9", "\u00E9"], /* LATIN SMALL LETTER E WITH ACUTE (LATIN SMALL LETTER E ACUTE) */
  ["\u00CA", "\u00EA"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX (LATIN SMALL LETTER E CIRCUMFLEX) */
  ["\u00CB", "\u00EB"], /* LATIN SMALL LETTER E WITH DIAERESIS (LATIN SMALL LETTER E DIAERESIS) */
  ["\u00CC", "\u00EC"], /* LATIN SMALL LETTER I WITH GRAVE (LATIN SMALL LETTER I GRAVE) */
  ["\u00CD", "\u00ED"], /* LATIN SMALL LETTER I WITH ACUTE (LATIN SMALL LETTER I ACUTE) */
  ["\u00CE", "\u00EE"], /* LATIN SMALL LETTER I WITH CIRCUMFLEX (LATIN SMALL LETTER I CIRCUMFLEX) */
  ["\u00CF", "\u00EF"], /* LATIN SMALL LETTER I WITH DIAERESIS (LATIN SMALL LETTER I DIAERESIS) */
  ["\u00D0", "\u00F0"], /* LATIN SMALL LETTER ETH */
  ["\u00D1", "\u00F1"], /* LATIN SMALL LETTER N WITH TILDE (LATIN SMALL LETTER N TILDE) */
  ["\u00D2", "\u00F2"], /* LATIN SMALL LETTER O WITH GRAVE (LATIN SMALL LETTER O GRAVE) */
  ["\u00D3", "\u00F3"], /* LATIN SMALL LETTER O WITH ACUTE (LATIN SMALL LETTER O ACUTE) */
  ["\u00D4", "\u00F4"], /* LATIN SMALL LETTER O WITH CIRCUMFLEX (LATIN SMALL LETTER O CIRCUMFLEX) */
  ["\u00D5", "\u00F5"], /* LATIN SMALL LETTER O WITH TILDE (LATIN SMALL LETTER O TILDE) */
  ["\u00D6", "\u00F6"], /* LATIN SMALL LETTER O WITH DIAERESIS (LATIN SMALL LETTER O DIAERESIS) */
  ["\u00F7", "\u00F7"], /* DIVISION SIGN */
  ["\u00D8", "\u00F8"], /* LATIN SMALL LETTER O WITH STROKE (LATIN SMALL LETTER O SLASH) */
  ["\u00D9", "\u00F9"], /* LATIN SMALL LETTER U WITH GRAVE (LATIN SMALL LETTER U GRAVE) */
  ["\u00DA", "\u00FA"], /* LATIN SMALL LETTER U WITH ACUTE (LATIN SMALL LETTER U ACUTE) */
  ["\u00DB", "\u00FB"], /* LATIN SMALL LETTER U WITH CIRCUMFLEX (LATIN SMALL LETTER U CIRCUMFLEX) */
  ["\u00DC", "\u00FC"], /* LATIN SMALL LETTER U WITH DIAERESIS (LATIN SMALL LETTER U DIAERESIS) */
  ["\u00DD", "\u00FD"], /* LATIN SMALL LETTER Y WITH ACUTE (LATIN SMALL LETTER Y ACUTE) */
  ["\u00DE", "\u00FE"], /* LATIN SMALL LETTER THORN */
  ["\u0178", "\u00FF"], /* LATIN SMALL LETTER Y WITH DIAERESIS (LATIN SMALL LETTER Y DIAERESIS) */
  ["\u0100", "\u0101"], /* LATIN CAPITAL LETTER A WITH MACRON (LATIN CAPITAL LETTER A MACRON) */
  ["\u0100", "\u0101"], /* LATIN SMALL LETTER A WITH MACRON (LATIN SMALL LETTER A MACRON) */
  ["\u0102", "\u0103"], /* LATIN CAPITAL LETTER A WITH BREVE (LATIN CAPITAL LETTER A BREVE) */
  ["\u0102", "\u0103"], /* LATIN SMALL LETTER A WITH BREVE (LATIN SMALL LETTER A BREVE) */
  ["\u0104", "\u0105"], /* LATIN CAPITAL LETTER A WITH OGONEK (LATIN CAPITAL LETTER A OGONEK) */
  ["\u0104", "\u0105"], /* LATIN SMALL LETTER A WITH OGONEK (LATIN SMALL LETTER A OGONEK) */
  ["\u0106", "\u0107"], /* LATIN CAPITAL LETTER C WITH ACUTE (LATIN CAPITAL LETTER C ACUTE) */
  ["\u0106", "\u0107"], /* LATIN SMALL LETTER C WITH ACUTE (LATIN SMALL LETTER C ACUTE) */
  ["\u0108", "\u0109"], /* LATIN CAPITAL LETTER C WITH CIRCUMFLEX (LATIN CAPITAL LETTER C CIRCUMFLEX) */
  ["\u0108", "\u0109"], /* LATIN SMALL LETTER C WITH CIRCUMFLEX (LATIN SMALL LETTER C CIRCUMFLEX) */
  ["\u010A", "\u010B"], /* LATIN CAPITAL LETTER C WITH DOT ABOVE (LATIN CAPITAL LETTER C DOT) */
  ["\u010A", "\u010B"], /* LATIN SMALL LETTER C WITH DOT ABOVE (LATIN SMALL LETTER C DOT) */
  ["\u010C", "\u010D"], /* LATIN CAPITAL LETTER C WITH CARON (LATIN CAPITAL LETTER C HACEK) */
  ["\u010C", "\u010D"], /* LATIN SMALL LETTER C WITH CARON (LATIN SMALL LETTER C HACEK) */
  ["\u010E", "\u010F"], /* LATIN CAPITAL LETTER D WITH CARON (LATIN CAPITAL LETTER D HACEK) */
  ["\u010E", "\u010F"], /* LATIN SMALL LETTER D WITH CARON (LATIN SMALL LETTER D HACEK) */
  ["\u0110", "\u0111"], /* LATIN CAPITAL LETTER D WITH STROKE (LATIN CAPITAL LETTER D BAR) */
  ["\u0110", "\u0111"], /* LATIN SMALL LETTER D WITH STROKE (LATIN SMALL LETTER D BAR) */
  ["\u0112", "\u0113"], /* LATIN CAPITAL LETTER E WITH MACRON (LATIN CAPITAL LETTER E MACRON) */
  ["\u0112", "\u0113"], /* LATIN SMALL LETTER E WITH MACRON (LATIN SMALL LETTER E MACRON) */
  ["\u0114", "\u0115"], /* LATIN CAPITAL LETTER E WITH BREVE (LATIN CAPITAL LETTER E BREVE) */
  ["\u0114", "\u0115"], /* LATIN SMALL LETTER E WITH BREVE (LATIN SMALL LETTER E BREVE) */
  ["\u0116", "\u0117"], /* LATIN CAPITAL LETTER E WITH DOT ABOVE (LATIN CAPITAL LETTER E DOT) */
  ["\u0116", "\u0117"], /* LATIN SMALL LETTER E WITH DOT ABOVE (LATIN SMALL LETTER E DOT) */
  ["\u0118", "\u0119"], /* LATIN CAPITAL LETTER E WITH OGONEK (LATIN CAPITAL LETTER E OGONEK) */
  ["\u0118", "\u0119"], /* LATIN SMALL LETTER E WITH OGONEK (LATIN SMALL LETTER E OGONEK) */
  ["\u011A", "\u011B"], /* LATIN CAPITAL LETTER E WITH CARON (LATIN CAPITAL LETTER E HACEK) */
  ["\u011A", "\u011B"], /* LATIN SMALL LETTER E WITH CARON (LATIN SMALL LETTER E HACEK) */
  ["\u011C", "\u011D"], /* LATIN CAPITAL LETTER G WITH CIRCUMFLEX (LATIN CAPITAL LETTER G CIRCUMFLEX) */
  ["\u011C", "\u011D"], /* LATIN SMALL LETTER G WITH CIRCUMFLEX (LATIN SMALL LETTER G CIRCUMFLEX) */
  ["\u011E", "\u011F"], /* LATIN CAPITAL LETTER G WITH BREVE (LATIN CAPITAL LETTER G BREVE) */
  ["\u011E", "\u011F"], /* LATIN SMALL LETTER G WITH BREVE (LATIN SMALL LETTER G BREVE) */
  ["\u0120", "\u0121"], /* LATIN CAPITAL LETTER G WITH DOT ABOVE (LATIN CAPITAL LETTER G DOT) */
  ["\u0120", "\u0121"], /* LATIN SMALL LETTER G WITH DOT ABOVE (LATIN SMALL LETTER G DOT) */
  ["\u0122", "\u0123"], /* LATIN CAPITAL LETTER G WITH CEDILLA (LATIN CAPITAL LETTER G CEDILLA) */
  ["\u0122", "\u0123"], /* LATIN SMALL LETTER G WITH CEDILLA (LATIN SMALL LETTER G CEDILLA) */
  ["\u0124", "\u0125"], /* LATIN CAPITAL LETTER H WITH CIRCUMFLEX (LATIN CAPITAL LETTER H CIRCUMFLEX) */
  ["\u0124", "\u0125"], /* LATIN SMALL LETTER H WITH CIRCUMFLEX (LATIN SMALL LETTER H CIRCUMFLEX) */
  ["\u0126", "\u0127"], /* LATIN CAPITAL LETTER H WITH STROKE (LATIN CAPITAL LETTER H BAR) */
  ["\u0126", "\u0127"], /* LATIN SMALL LETTER H WITH STROKE (LATIN SMALL LETTER H BAR) */
  ["\u0128", "\u0129"], /* LATIN CAPITAL LETTER I WITH TILDE (LATIN CAPITAL LETTER I TILDE) */
  ["\u0128", "\u0129"], /* LATIN SMALL LETTER I WITH TILDE (LATIN SMALL LETTER I TILDE) */
  ["\u012A", "\u012B"], /* LATIN CAPITAL LETTER I WITH MACRON (LATIN CAPITAL LETTER I MACRON) */
  ["\u012A", "\u012B"], /* LATIN SMALL LETTER I WITH MACRON (LATIN SMALL LETTER I MACRON) */
  ["\u012C", "\u012D"], /* LATIN CAPITAL LETTER I WITH BREVE (LATIN CAPITAL LETTER I BREVE) */
  ["\u012C", "\u012D"], /* LATIN SMALL LETTER I WITH BREVE (LATIN SMALL LETTER I BREVE) */
  ["\u012E", "\u012F"], /* LATIN CAPITAL LETTER I WITH OGONEK (LATIN CAPITAL LETTER I OGONEK) */
  ["\u012E", "\u012F"], /* LATIN SMALL LETTER I WITH OGONEK (LATIN SMALL LETTER I OGONEK) */
  ["\u0130", "\u0069\u0307"], /* LATIN CAPITAL LETTER I WITH DOT ABOVE (LATIN CAPITAL LETTER I DOT) */
  ["\u0049", "\u0131"], /* LATIN SMALL LETTER DOTLESS I */
  ["\u0132", "\u0133"], /* LATIN CAPITAL LIGATURE IJ (LATIN CAPITAL LETTER I J) */
  ["\u0132", "\u0133"], /* LATIN SMALL LIGATURE IJ (LATIN SMALL LETTER I J) */
  ["\u0134", "\u0135"], /* LATIN CAPITAL LETTER J WITH CIRCUMFLEX (LATIN CAPITAL LETTER J CIRCUMFLEX) */
  ["\u0134", "\u0135"], /* LATIN SMALL LETTER J WITH CIRCUMFLEX (LATIN SMALL LETTER J CIRCUMFLEX) */
  ["\u0136", "\u0137"], /* LATIN CAPITAL LETTER K WITH CEDILLA (LATIN CAPITAL LETTER K CEDILLA) */
  ["\u0136", "\u0137"], /* LATIN SMALL LETTER K WITH CEDILLA (LATIN SMALL LETTER K CEDILLA) */
  ["\u0138", "\u0138"], /* LATIN SMALL LETTER KRA */
  ["\u0139", "\u013A"], /* LATIN CAPITAL LETTER L WITH ACUTE (LATIN CAPITAL LETTER L ACUTE) */
  ["\u0139", "\u013A"], /* LATIN SMALL LETTER L WITH ACUTE (LATIN SMALL LETTER L ACUTE) */
  ["\u013B", "\u013C"], /* LATIN CAPITAL LETTER L WITH CEDILLA (LATIN CAPITAL LETTER L CEDILLA) */
  ["\u013B", "\u013C"], /* LATIN SMALL LETTER L WITH CEDILLA (LATIN SMALL LETTER L CEDILLA) */
  ["\u013D", "\u013E"], /* LATIN CAPITAL LETTER L WITH CARON (LATIN CAPITAL LETTER L HACEK) */
  ["\u013D", "\u013E"], /* LATIN SMALL LETTER L WITH CARON (LATIN SMALL LETTER L HACEK) */
  ["\u013F", "\u0140"], /* LATIN CAPITAL LETTER L WITH MIDDLE DOT */
  ["\u013F", "\u0140"], /* LATIN SMALL LETTER L WITH MIDDLE DOT */
  ["\u0141", "\u0142"], /* LATIN CAPITAL LETTER L WITH STROKE (LATIN CAPITAL LETTER L SLASH) */
  ["\u0141", "\u0142"], /* LATIN SMALL LETTER L WITH STROKE (LATIN SMALL LETTER L SLASH) */
  ["\u0143", "\u0144"], /* LATIN CAPITAL LETTER N WITH ACUTE (LATIN CAPITAL LETTER N ACUTE) */
  ["\u0143", "\u0144"], /* LATIN SMALL LETTER N WITH ACUTE (LATIN SMALL LETTER N ACUTE) */
  ["\u0145", "\u0146"], /* LATIN CAPITAL LETTER N WITH CEDILLA (LATIN CAPITAL LETTER N CEDILLA) */
  ["\u0145", "\u0146"], /* LATIN SMALL LETTER N WITH CEDILLA (LATIN SMALL LETTER N CEDILLA) */
  ["\u0147", "\u0148"], /* LATIN CAPITAL LETTER N WITH CARON (LATIN CAPITAL LETTER N HACEK) */
  ["\u0147", "\u0148"], /* LATIN SMALL LETTER N WITH CARON (LATIN SMALL LETTER N HACEK) */
  ["\u02BC\u004E", "\u0149"], /* LATIN SMALL LETTER N PRECEDED BY APOSTROPHE (LATIN SMALL LETTER APOSTROPHE N) */
  ["\u014A", "\u014B"], /* LATIN CAPITAL LETTER ENG */
  ["\u014A", "\u014B"], /* LATIN SMALL LETTER ENG */
  ["\u014C", "\u014D"], /* LATIN CAPITAL LETTER O WITH MACRON (LATIN CAPITAL LETTER O MACRON) */
  ["\u014C", "\u014D"], /* LATIN SMALL LETTER O WITH MACRON (LATIN SMALL LETTER O MACRON) */
  ["\u014E", "\u014F"], /* LATIN CAPITAL LETTER O WITH BREVE (LATIN CAPITAL LETTER O BREVE) */
  ["\u014E", "\u014F"], /* LATIN SMALL LETTER O WITH BREVE (LATIN SMALL LETTER O BREVE) */
  ["\u0150", "\u0151"], /* LATIN CAPITAL LETTER O WITH DOUBLE ACUTE (LATIN CAPITAL LETTER O DOUBLE ACUTE) */
  ["\u0150", "\u0151"], /* LATIN SMALL LETTER O WITH DOUBLE ACUTE (LATIN SMALL LETTER O DOUBLE ACUTE) */
  ["\u0152", "\u0153"], /* LATIN CAPITAL LIGATURE OE (LATIN CAPITAL LETTER O E) */
  ["\u0152", "\u0153"], /* LATIN SMALL LIGATURE OE (LATIN SMALL LETTER O E) */
  ["\u0154", "\u0155"], /* LATIN CAPITAL LETTER R WITH ACUTE (LATIN CAPITAL LETTER R ACUTE) */
  ["\u0154", "\u0155"], /* LATIN SMALL LETTER R WITH ACUTE (LATIN SMALL LETTER R ACUTE) */
  ["\u0156", "\u0157"], /* LATIN CAPITAL LETTER R WITH CEDILLA (LATIN CAPITAL LETTER R CEDILLA) */
  ["\u0156", "\u0157"], /* LATIN SMALL LETTER R WITH CEDILLA (LATIN SMALL LETTER R CEDILLA) */
  ["\u0158", "\u0159"], /* LATIN CAPITAL LETTER R WITH CARON (LATIN CAPITAL LETTER R HACEK) */
  ["\u0158", "\u0159"], /* LATIN SMALL LETTER R WITH CARON (LATIN SMALL LETTER R HACEK) */
  ["\u015A", "\u015B"], /* LATIN CAPITAL LETTER S WITH ACUTE (LATIN CAPITAL LETTER S ACUTE) */
  ["\u015A", "\u015B"], /* LATIN SMALL LETTER S WITH ACUTE (LATIN SMALL LETTER S ACUTE) */
  ["\u015C", "\u015D"], /* LATIN CAPITAL LETTER S WITH CIRCUMFLEX (LATIN CAPITAL LETTER S CIRCUMFLEX) */
  ["\u015C", "\u015D"], /* LATIN SMALL LETTER S WITH CIRCUMFLEX (LATIN SMALL LETTER S CIRCUMFLEX) */
  ["\u015E", "\u015F"], /* LATIN CAPITAL LETTER S WITH CEDILLA (LATIN CAPITAL LETTER S CEDILLA) */
  ["\u015E", "\u015F"], /* LATIN SMALL LETTER S WITH CEDILLA (LATIN SMALL LETTER S CEDILLA) */
  ["\u0160", "\u0161"], /* LATIN CAPITAL LETTER S WITH CARON (LATIN CAPITAL LETTER S HACEK) */
  ["\u0160", "\u0161"], /* LATIN SMALL LETTER S WITH CARON (LATIN SMALL LETTER S HACEK) */
  ["\u0162", "\u0163"], /* LATIN CAPITAL LETTER T WITH CEDILLA (LATIN CAPITAL LETTER T CEDILLA) */
  ["\u0162", "\u0163"], /* LATIN SMALL LETTER T WITH CEDILLA (LATIN SMALL LETTER T CEDILLA) */
  ["\u0164", "\u0165"], /* LATIN CAPITAL LETTER T WITH CARON (LATIN CAPITAL LETTER T HACEK) */
  ["\u0164", "\u0165"], /* LATIN SMALL LETTER T WITH CARON (LATIN SMALL LETTER T HACEK) */
  ["\u0166", "\u0167"], /* LATIN CAPITAL LETTER T WITH STROKE (LATIN CAPITAL LETTER T BAR) */
  ["\u0166", "\u0167"], /* LATIN SMALL LETTER T WITH STROKE (LATIN SMALL LETTER T BAR) */
  ["\u0168", "\u0169"], /* LATIN CAPITAL LETTER U WITH TILDE (LATIN CAPITAL LETTER U TILDE) */
  ["\u0168", "\u0169"], /* LATIN SMALL LETTER U WITH TILDE (LATIN SMALL LETTER U TILDE) */
  ["\u016A", "\u016B"], /* LATIN CAPITAL LETTER U WITH MACRON (LATIN CAPITAL LETTER U MACRON) */
  ["\u016A", "\u016B"], /* LATIN SMALL LETTER U WITH MACRON (LATIN SMALL LETTER U MACRON) */
  ["\u016C", "\u016D"], /* LATIN CAPITAL LETTER U WITH BREVE (LATIN CAPITAL LETTER U BREVE) */
  ["\u016C", "\u016D"], /* LATIN SMALL LETTER U WITH BREVE (LATIN SMALL LETTER U BREVE) */
  ["\u016E", "\u016F"], /* LATIN CAPITAL LETTER U WITH RING ABOVE (LATIN CAPITAL LETTER U RING) */
  ["\u016E", "\u016F"], /* LATIN SMALL LETTER U WITH RING ABOVE (LATIN SMALL LETTER U RING) */
  ["\u0170", "\u0171"], /* LATIN CAPITAL LETTER U WITH DOUBLE ACUTE (LATIN CAPITAL LETTER U DOUBLE ACUTE) */
  ["\u0170", "\u0171"], /* LATIN SMALL LETTER U WITH DOUBLE ACUTE (LATIN SMALL LETTER U DOUBLE ACUTE) */
  ["\u0172", "\u0173"], /* LATIN CAPITAL LETTER U WITH OGONEK (LATIN CAPITAL LETTER U OGONEK) */
  ["\u0172", "\u0173"], /* LATIN SMALL LETTER U WITH OGONEK (LATIN SMALL LETTER U OGONEK) */
  ["\u0174", "\u0175"], /* LATIN CAPITAL LETTER W WITH CIRCUMFLEX (LATIN CAPITAL LETTER W CIRCUMFLEX) */
  ["\u0174", "\u0175"], /* LATIN SMALL LETTER W WITH CIRCUMFLEX (LATIN SMALL LETTER W CIRCUMFLEX) */
  ["\u0176", "\u0177"], /* LATIN CAPITAL LETTER Y WITH CIRCUMFLEX (LATIN CAPITAL LETTER Y CIRCUMFLEX) */
  ["\u0176", "\u0177"], /* LATIN SMALL LETTER Y WITH CIRCUMFLEX (LATIN SMALL LETTER Y CIRCUMFLEX) */
  ["\u0178", "\u00FF"], /* LATIN CAPITAL LETTER Y WITH DIAERESIS (LATIN CAPITAL LETTER Y DIAERESIS) */
  ["\u0179", "\u017A"], /* LATIN CAPITAL LETTER Z WITH ACUTE (LATIN CAPITAL LETTER Z ACUTE) */
  ["\u0179", "\u017A"], /* LATIN SMALL LETTER Z WITH ACUTE (LATIN SMALL LETTER Z ACUTE) */
  ["\u017B", "\u017C"], /* LATIN CAPITAL LETTER Z WITH DOT ABOVE (LATIN CAPITAL LETTER Z DOT) */
  ["\u017B", "\u017C"], /* LATIN SMALL LETTER Z WITH DOT ABOVE (LATIN SMALL LETTER Z DOT) */
  ["\u017D", "\u017E"], /* LATIN CAPITAL LETTER Z WITH CARON (LATIN CAPITAL LETTER Z HACEK) */
  ["\u017D", "\u017E"], /* LATIN SMALL LETTER Z WITH CARON (LATIN SMALL LETTER Z HACEK) */
  ["\u0053", "\u017F"], /* LATIN SMALL LETTER LONG S */
  ["\u0243", "\u0180"], /* LATIN SMALL LETTER B WITH STROKE (LATIN SMALL LETTER B BAR) */
  ["\u0181", "\u0253"], /* LATIN CAPITAL LETTER B WITH HOOK (LATIN CAPITAL LETTER B HOOK) */
  ["\u0182", "\u0183"], /* LATIN CAPITAL LETTER B WITH TOPBAR (LATIN CAPITAL LETTER B TOPBAR) */
  ["\u0182", "\u0183"], /* LATIN SMALL LETTER B WITH TOPBAR (LATIN SMALL LETTER B TOPBAR) */
  ["\u0184", "\u0185"], /* LATIN CAPITAL LETTER TONE SIX */
  ["\u0184", "\u0185"], /* LATIN SMALL LETTER TONE SIX */
  ["\u0186", "\u0254"], /* LATIN CAPITAL LETTER OPEN O */
  ["\u0187", "\u0188"], /* LATIN CAPITAL LETTER C WITH HOOK (LATIN CAPITAL LETTER C HOOK) */
  ["\u0187", "\u0188"], /* LATIN SMALL LETTER C WITH HOOK (LATIN SMALL LETTER C HOOK) */
  ["\u0189", "\u0256"], /* LATIN CAPITAL LETTER AFRICAN D */
  ["\u018A", "\u0257"], /* LATIN CAPITAL LETTER D WITH HOOK (LATIN CAPITAL LETTER D HOOK) */
  ["\u018B", "\u018C"], /* LATIN CAPITAL LETTER D WITH TOPBAR (LATIN CAPITAL LETTER D TOPBAR) */
  ["\u018B", "\u018C"], /* LATIN SMALL LETTER D WITH TOPBAR (LATIN SMALL LETTER D TOPBAR) */
  ["\u018D", "\u018D"], /* LATIN SMALL LETTER TURNED DELTA */
  ["\u018E", "\u01DD"], /* LATIN CAPITAL LETTER REVERSED E (LATIN CAPITAL LETTER TURNED E) */
  ["\u018F", "\u0259"], /* LATIN CAPITAL LETTER SCHWA */
  ["\u0190", "\u025B"], /* LATIN CAPITAL LETTER OPEN E (LATIN CAPITAL LETTER EPSILON) */
  ["\u0191", "\u0192"], /* LATIN CAPITAL LETTER F WITH HOOK (LATIN CAPITAL LETTER F HOOK) */
  ["\u0191", "\u0192"], /* LATIN SMALL LETTER F WITH HOOK (LATIN SMALL LETTER SCRIPT F) */
  ["\u0193", "\u0260"], /* LATIN CAPITAL LETTER G WITH HOOK (LATIN CAPITAL LETTER G HOOK) */
  ["\u0194", "\u0263"], /* LATIN CAPITAL LETTER GAMMA */
  ["\u01F6", "\u0195"], /* LATIN SMALL LETTER HV (LATIN SMALL LETTER H V) */
  ["\u0196", "\u0269"], /* LATIN CAPITAL LETTER IOTA */
  ["\u0197", "\u0268"], /* LATIN CAPITAL LETTER I WITH STROKE (LATIN CAPITAL LETTER BARRED I) */
  ["\u0198", "\u0199"], /* LATIN CAPITAL LETTER K WITH HOOK (LATIN CAPITAL LETTER K HOOK) */
  ["\u0198", "\u0199"], /* LATIN SMALL LETTER K WITH HOOK (LATIN SMALL LETTER K HOOK) */
  ["\u023D", "\u019A"], /* LATIN SMALL LETTER L WITH BAR (LATIN SMALL LETTER BARRED L) */
  ["\u019B", "\u019B"], /* LATIN SMALL LETTER LAMBDA WITH STROKE (LATIN SMALL LETTER BARRED LAMBDA) */
  ["\u019C", "\u026F"], /* LATIN CAPITAL LETTER TURNED M */
  ["\u019D", "\u0272"], /* LATIN CAPITAL LETTER N WITH LEFT HOOK (LATIN CAPITAL LETTER N HOOK) */
  ["\u0220", "\u019E"], /* LATIN SMALL LETTER N WITH LONG RIGHT LEG */
  ["\u019F", "\u0275"], /* LATIN CAPITAL LETTER O WITH MIDDLE TILDE (LATIN CAPITAL LETTER BARRED O) */
  ["\u01A0", "\u01A1"], /* LATIN CAPITAL LETTER O WITH HORN (LATIN CAPITAL LETTER O HORN) */
  ["\u01A0", "\u01A1"], /* LATIN SMALL LETTER O WITH HORN (LATIN SMALL LETTER O HORN) */
  ["\u01A2", "\u01A3"], /* LATIN CAPITAL LETTER OI (LATIN CAPITAL LETTER O I) */
  ["\u01A2", "\u01A3"], /* LATIN SMALL LETTER OI (LATIN SMALL LETTER O I) */
  ["\u01A4", "\u01A5"], /* LATIN CAPITAL LETTER P WITH HOOK (LATIN CAPITAL LETTER P HOOK) */
  ["\u01A4", "\u01A5"], /* LATIN SMALL LETTER P WITH HOOK (LATIN SMALL LETTER P HOOK) */
  ["\u01A6", "\u0280"], /* LATIN LETTER YR (LATIN LETTER Y R) */
  ["\u01A7", "\u01A8"], /* LATIN CAPITAL LETTER TONE TWO */
  ["\u01A7", "\u01A8"], /* LATIN SMALL LETTER TONE TWO */
  ["\u01A9", "\u0283"], /* LATIN CAPITAL LETTER ESH */
  ["\u01AA", "\u01AA"], /* LATIN LETTER REVERSED ESH LOOP */
  ["\u01AB", "\u01AB"], /* LATIN SMALL LETTER T WITH PALATAL HOOK (LATIN SMALL LETTER T PALATAL HOOK) */
  ["\u01AC", "\u01AD"], /* LATIN CAPITAL LETTER T WITH HOOK (LATIN CAPITAL LETTER T HOOK) */
  ["\u01AC", "\u01AD"], /* LATIN SMALL LETTER T WITH HOOK (LATIN SMALL LETTER T HOOK) */
  ["\u01AE", "\u0288"], /* LATIN CAPITAL LETTER T WITH RETROFLEX HOOK (LATIN CAPITAL LETTER T RETROFLEX HOOK) */
  ["\u01AF", "\u01B0"], /* LATIN CAPITAL LETTER U WITH HORN (LATIN CAPITAL LETTER U HORN) */
  ["\u01AF", "\u01B0"], /* LATIN SMALL LETTER U WITH HORN (LATIN SMALL LETTER U HORN) */
  ["\u01B1", "\u028A"], /* LATIN CAPITAL LETTER UPSILON */
  ["\u01B2", "\u028B"], /* LATIN CAPITAL LETTER V WITH HOOK (LATIN CAPITAL LETTER SCRIPT V) */
  ["\u01B3", "\u01B4"], /* LATIN CAPITAL LETTER Y WITH HOOK (LATIN CAPITAL LETTER Y HOOK) */
  ["\u01B3", "\u01B4"], /* LATIN SMALL LETTER Y WITH HOOK (LATIN SMALL LETTER Y HOOK) */
  ["\u01B5", "\u01B6"], /* LATIN CAPITAL LETTER Z WITH STROKE (LATIN CAPITAL LETTER Z BAR) */
  ["\u01B5", "\u01B6"], /* LATIN SMALL LETTER Z WITH STROKE (LATIN SMALL LETTER Z BAR) */
  ["\u01B7", "\u0292"], /* LATIN CAPITAL LETTER EZH (LATIN CAPITAL LETTER YOGH) */
  ["\u01B8", "\u01B9"], /* LATIN CAPITAL LETTER EZH REVERSED (LATIN CAPITAL LETTER REVERSED YOGH) */
  ["\u01B8", "\u01B9"], /* LATIN SMALL LETTER EZH REVERSED (LATIN SMALL LETTER REVERSED YOGH) */
  ["\u01BA", "\u01BA"], /* LATIN SMALL LETTER EZH WITH TAIL (LATIN SMALL LETTER YOGH WITH TAIL) */
  ["\u01BB", "\u01BB"], /* LATIN LETTER TWO WITH STROKE (LATIN LETTER TWO BAR) */
  ["\u01BC", "\u01BD"], /* LATIN CAPITAL LETTER TONE FIVE */
  ["\u01BC", "\u01BD"], /* LATIN SMALL LETTER TONE FIVE */
  ["\u01BE", "\u01BE"], /* LATIN LETTER INVERTED GLOTTAL STOP WITH STROKE (LATIN LETTER INVERTED GLOTTAL STOP BAR) */
  ["\u01F7", "\u01BF"], /* LATIN LETTER WYNN */
  ["\u01C0", "\u01C0"], /* LATIN LETTER DENTAL CLICK (LATIN LETTER PIPE) */
  ["\u01C1", "\u01C1"], /* LATIN LETTER LATERAL CLICK (LATIN LETTER DOUBLE PIPE) */
  ["\u01C2", "\u01C2"], /* LATIN LETTER ALVEOLAR CLICK (LATIN LETTER PIPE DOUBLE BAR) */
  ["\u01C3", "\u01C3"], /* LATIN LETTER RETROFLEX CLICK (LATIN LETTER EXCLAMATION MARK) */
  ["\u01C4", "\u01C6"], /* LATIN CAPITAL LETTER DZ WITH CARON (LATIN CAPITAL LETTER D Z HACEK) */
  ["\u01C4", "\u01C6"], /* LATIN CAPITAL LETTER D WITH SMALL LETTER Z WITH CARON (LATIN LETTER CAPITAL D SMALL Z HACEK) */
  ["\u01C4", "\u01C6"], /* LATIN SMALL LETTER DZ WITH CARON (LATIN SMALL LETTER D Z HACEK) */
  ["\u01C7", "\u01C9"], /* LATIN CAPITAL LETTER LJ (LATIN CAPITAL LETTER L J) */
  ["\u01C7", "\u01C9"], /* LATIN CAPITAL LETTER L WITH SMALL LETTER J (LATIN LETTER CAPITAL L SMALL J) */
  ["\u01C7", "\u01C9"], /* LATIN SMALL LETTER LJ (LATIN SMALL LETTER L J) */
  ["\u01CA", "\u01CC"], /* LATIN CAPITAL LETTER NJ (LATIN CAPITAL LETTER N J) */
  ["\u01CA", "\u01CC"], /* LATIN CAPITAL LETTER N WITH SMALL LETTER J (LATIN LETTER CAPITAL N SMALL J) */
  ["\u01CA", "\u01CC"], /* LATIN SMALL LETTER NJ (LATIN SMALL LETTER N J) */
  ["\u01CD", "\u01CE"], /* LATIN CAPITAL LETTER A WITH CARON (LATIN CAPITAL LETTER A HACEK) */
  ["\u01CD", "\u01CE"], /* LATIN SMALL LETTER A WITH CARON (LATIN SMALL LETTER A HACEK) */
  ["\u01CF", "\u01D0"], /* LATIN CAPITAL LETTER I WITH CARON (LATIN CAPITAL LETTER I HACEK) */
  ["\u01CF", "\u01D0"], /* LATIN SMALL LETTER I WITH CARON (LATIN SMALL LETTER I HACEK) */
  ["\u01D1", "\u01D2"], /* LATIN CAPITAL LETTER O WITH CARON (LATIN CAPITAL LETTER O HACEK) */
  ["\u01D1", "\u01D2"], /* LATIN SMALL LETTER O WITH CARON (LATIN SMALL LETTER O HACEK) */
  ["\u01D3", "\u01D4"], /* LATIN CAPITAL LETTER U WITH CARON (LATIN CAPITAL LETTER U HACEK) */
  ["\u01D3", "\u01D4"], /* LATIN SMALL LETTER U WITH CARON (LATIN SMALL LETTER U HACEK) */
  ["\u01D5", "\u01D6"], /* LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON (LATIN CAPITAL LETTER U DIAERESIS MACRON) */
  ["\u01D5", "\u01D6"], /* LATIN SMALL LETTER U WITH DIAERESIS AND MACRON (LATIN SMALL LETTER U DIAERESIS MACRON) */
  ["\u01D7", "\u01D8"], /* LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE (LATIN CAPITAL LETTER U DIAERESIS ACUTE) */
  ["\u01D7", "\u01D8"], /* LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE (LATIN SMALL LETTER U DIAERESIS ACUTE) */
  ["\u01D9", "\u01DA"], /* LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON (LATIN CAPITAL LETTER U DIAERESIS HACEK) */
  ["\u01D9", "\u01DA"], /* LATIN SMALL LETTER U WITH DIAERESIS AND CARON (LATIN SMALL LETTER U DIAERESIS HACEK) */
  ["\u01DB", "\u01DC"], /* LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE (LATIN CAPITAL LETTER U DIAERESIS GRAVE) */
  ["\u01DB", "\u01DC"], /* LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE (LATIN SMALL LETTER U DIAERESIS GRAVE) */
  ["\u018E", "\u01DD"], /* LATIN SMALL LETTER TURNED E */
  ["\u01DE", "\u01DF"], /* LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON (LATIN CAPITAL LETTER A DIAERESIS MACRON) */
  ["\u01DE", "\u01DF"], /* LATIN SMALL LETTER A WITH DIAERESIS AND MACRON (LATIN SMALL LETTER A DIAERESIS MACRON) */
  ["\u01E0", "\u01E1"], /* LATIN CAPITAL LETTER A WITH DOT ABOVE AND MACRON (LATIN CAPITAL LETTER A DOT MACRON) */
  ["\u01E0", "\u01E1"], /* LATIN SMALL LETTER A WITH DOT ABOVE AND MACRON (LATIN SMALL LETTER A DOT MACRON) */
  ["\u01E2", "\u01E3"], /* LATIN CAPITAL LETTER AE WITH MACRON (LATIN CAPITAL LETTER A E MACRON) */
  ["\u01E2", "\u01E3"], /* LATIN SMALL LETTER AE WITH MACRON (LATIN SMALL LETTER A E MACRON) */
  ["\u01E4", "\u01E5"], /* LATIN CAPITAL LETTER G WITH STROKE (LATIN CAPITAL LETTER G BAR) */
  ["\u01E4", "\u01E5"], /* LATIN SMALL LETTER G WITH STROKE (LATIN SMALL LETTER G BAR) */
  ["\u01E6", "\u01E7"], /* LATIN CAPITAL LETTER G WITH CARON (LATIN CAPITAL LETTER G HACEK) */
  ["\u01E6", "\u01E7"], /* LATIN SMALL LETTER G WITH CARON (LATIN SMALL LETTER G HACEK) */
  ["\u01E8", "\u01E9"], /* LATIN CAPITAL LETTER K WITH CARON (LATIN CAPITAL LETTER K HACEK) */
  ["\u01E8", "\u01E9"], /* LATIN SMALL LETTER K WITH CARON (LATIN SMALL LETTER K HACEK) */
  ["\u01EA", "\u01EB"], /* LATIN CAPITAL LETTER O WITH OGONEK (LATIN CAPITAL LETTER O OGONEK) */
  ["\u01EA", "\u01EB"], /* LATIN SMALL LETTER O WITH OGONEK (LATIN SMALL LETTER O OGONEK) */
  ["\u01EC", "\u01ED"], /* LATIN CAPITAL LETTER O WITH OGONEK AND MACRON (LATIN CAPITAL LETTER O OGONEK MACRON) */
  ["\u01EC", "\u01ED"], /* LATIN SMALL LETTER O WITH OGONEK AND MACRON (LATIN SMALL LETTER O OGONEK MACRON) */
  ["\u01EE", "\u01EF"], /* LATIN CAPITAL LETTER EZH WITH CARON (LATIN CAPITAL LETTER YOGH HACEK) */
  ["\u01EE", "\u01EF"], /* LATIN SMALL LETTER EZH WITH CARON (LATIN SMALL LETTER YOGH HACEK) */
  ["\u004A\u030C", "\u01F0"], /* LATIN SMALL LETTER J WITH CARON (LATIN SMALL LETTER J HACEK) */
  ["\u01F1", "\u01F3"], /* LATIN CAPITAL LETTER DZ */
  ["\u01F1", "\u01F3"], /* LATIN CAPITAL LETTER D WITH SMALL LETTER Z */
  ["\u01F1", "\u01F3"], /* LATIN SMALL LETTER DZ */
  ["\u01F4", "\u01F5"], /* LATIN CAPITAL LETTER G WITH ACUTE */
  ["\u01F4", "\u01F5"], /* LATIN SMALL LETTER G WITH ACUTE */
  ["\u01F6", "\u0195"], /* LATIN CAPITAL LETTER HWAIR */
  ["\u01F7", "\u01BF"], /* LATIN CAPITAL LETTER WYNN */
  ["\u01F8", "\u01F9"], /* LATIN CAPITAL LETTER N WITH GRAVE */
  ["\u01F8", "\u01F9"], /* LATIN SMALL LETTER N WITH GRAVE */
  ["\u01FA", "\u01FB"], /* LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE */
  ["\u01FA", "\u01FB"], /* LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE */
  ["\u01FC", "\u01FD"], /* LATIN CAPITAL LETTER AE WITH ACUTE */
  ["\u01FC", "\u01FD"], /* LATIN SMALL LETTER AE WITH ACUTE */
  ["\u01FE", "\u01FF"], /* LATIN CAPITAL LETTER O WITH STROKE AND ACUTE */
  ["\u01FE", "\u01FF"], /* LATIN SMALL LETTER O WITH STROKE AND ACUTE */
  ["\u0200", "\u0201"], /* LATIN CAPITAL LETTER A WITH DOUBLE GRAVE */
  ["\u0200", "\u0201"], /* LATIN SMALL LETTER A WITH DOUBLE GRAVE */
  ["\u0202", "\u0203"], /* LATIN CAPITAL LETTER A WITH INVERTED BREVE */
  ["\u0202", "\u0203"], /* LATIN SMALL LETTER A WITH INVERTED BREVE */
  ["\u0204", "\u0205"], /* LATIN CAPITAL LETTER E WITH DOUBLE GRAVE */
  ["\u0204", "\u0205"], /* LATIN SMALL LETTER E WITH DOUBLE GRAVE */
  ["\u0206", "\u0207"], /* LATIN CAPITAL LETTER E WITH INVERTED BREVE */
  ["\u0206", "\u0207"], /* LATIN SMALL LETTER E WITH INVERTED BREVE */
  ["\u0208", "\u0209"], /* LATIN CAPITAL LETTER I WITH DOUBLE GRAVE */
  ["\u0208", "\u0209"], /* LATIN SMALL LETTER I WITH DOUBLE GRAVE */
  ["\u020A", "\u020B"], /* LATIN CAPITAL LETTER I WITH INVERTED BREVE */
  ["\u020A", "\u020B"], /* LATIN SMALL LETTER I WITH INVERTED BREVE */
  ["\u020C", "\u020D"], /* LATIN CAPITAL LETTER O WITH DOUBLE GRAVE */
  ["\u020C", "\u020D"], /* LATIN SMALL LETTER O WITH DOUBLE GRAVE */
  ["\u020E", "\u020F"], /* LATIN CAPITAL LETTER O WITH INVERTED BREVE */
  ["\u020E", "\u020F"], /* LATIN SMALL LETTER O WITH INVERTED BREVE */
  ["\u0210", "\u0211"], /* LATIN CAPITAL LETTER R WITH DOUBLE GRAVE */
  ["\u0210", "\u0211"], /* LATIN SMALL LETTER R WITH DOUBLE GRAVE */
  ["\u0212", "\u0213"], /* LATIN CAPITAL LETTER R WITH INVERTED BREVE */
  ["\u0212", "\u0213"], /* LATIN SMALL LETTER R WITH INVERTED BREVE */
  ["\u0214", "\u0215"], /* LATIN CAPITAL LETTER U WITH DOUBLE GRAVE */
  ["\u0214", "\u0215"], /* LATIN SMALL LETTER U WITH DOUBLE GRAVE */
  ["\u0216", "\u0217"], /* LATIN CAPITAL LETTER U WITH INVERTED BREVE */
  ["\u0216", "\u0217"], /* LATIN SMALL LETTER U WITH INVERTED BREVE */
  ["\u0218", "\u0219"], /* LATIN CAPITAL LETTER S WITH COMMA BELOW */
  ["\u0218", "\u0219"], /* LATIN SMALL LETTER S WITH COMMA BELOW */
  ["\u021A", "\u021B"], /* LATIN CAPITAL LETTER T WITH COMMA BELOW */
  ["\u021A", "\u021B"], /* LATIN SMALL LETTER T WITH COMMA BELOW */
  ["\u021C", "\u021D"], /* LATIN CAPITAL LETTER YOGH */
  ["\u021C", "\u021D"], /* LATIN SMALL LETTER YOGH */
  ["\u021E", "\u021F"], /* LATIN CAPITAL LETTER H WITH CARON */
  ["\u021E", "\u021F"], /* LATIN SMALL LETTER H WITH CARON */
  ["\u0220", "\u019E"], /* LATIN CAPITAL LETTER N WITH LONG RIGHT LEG */
  ["\u0221", "\u0221"], /* LATIN SMALL LETTER D WITH CURL */
  ["\u0222", "\u0223"], /* LATIN CAPITAL LETTER OU */
  ["\u0222", "\u0223"], /* LATIN SMALL LETTER OU */
  ["\u0224", "\u0225"], /* LATIN CAPITAL LETTER Z WITH HOOK */
  ["\u0224", "\u0225"], /* LATIN SMALL LETTER Z WITH HOOK */
  ["\u0226", "\u0227"], /* LATIN CAPITAL LETTER A WITH DOT ABOVE */
  ["\u0226", "\u0227"], /* LATIN SMALL LETTER A WITH DOT ABOVE */
  ["\u0228", "\u0229"], /* LATIN CAPITAL LETTER E WITH CEDILLA */
  ["\u0228", "\u0229"], /* LATIN SMALL LETTER E WITH CEDILLA */
  ["\u022A", "\u022B"], /* LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON */
  ["\u022A", "\u022B"], /* LATIN SMALL LETTER O WITH DIAERESIS AND MACRON */
  ["\u022C", "\u022D"], /* LATIN CAPITAL LETTER O WITH TILDE AND MACRON */
  ["\u022C", "\u022D"], /* LATIN SMALL LETTER O WITH TILDE AND MACRON */
  ["\u022E", "\u022F"], /* LATIN CAPITAL LETTER O WITH DOT ABOVE */
  ["\u022E", "\u022F"], /* LATIN SMALL LETTER O WITH DOT ABOVE */
  ["\u0230", "\u0231"], /* LATIN CAPITAL LETTER O WITH DOT ABOVE AND MACRON */
  ["\u0230", "\u0231"], /* LATIN SMALL LETTER O WITH DOT ABOVE AND MACRON */
  ["\u0232", "\u0233"], /* LATIN CAPITAL LETTER Y WITH MACRON */
  ["\u0232", "\u0233"], /* LATIN SMALL LETTER Y WITH MACRON */
  ["\u0234", "\u0234"], /* LATIN SMALL LETTER L WITH CURL */
  ["\u0235", "\u0235"], /* LATIN SMALL LETTER N WITH CURL */
  ["\u0236", "\u0236"], /* LATIN SMALL LETTER T WITH CURL */
  ["\u0237", "\u0237"], /* LATIN SMALL LETTER DOTLESS J */
  ["\u0238", "\u0238"], /* LATIN SMALL LETTER DB DIGRAPH */
  ["\u0239", "\u0239"], /* LATIN SMALL LETTER QP DIGRAPH */
  ["\u023A", "\u2C65"], /* LATIN CAPITAL LETTER A WITH STROKE */
  ["\u023B", "\u023C"], /* LATIN CAPITAL LETTER C WITH STROKE */
  ["\u023B", "\u023C"], /* LATIN SMALL LETTER C WITH STROKE */
  ["\u023D", "\u019A"], /* LATIN CAPITAL LETTER L WITH BAR */
  ["\u023E", "\u2C66"], /* LATIN CAPITAL LETTER T WITH DIAGONAL STROKE */
  ["\u2C7E", "\u023F"], /* LATIN SMALL LETTER S WITH SWASH TAIL */
  ["\u2C7F", "\u0240"], /* LATIN SMALL LETTER Z WITH SWASH TAIL */
  ["\u0241", "\u0242"], /* LATIN CAPITAL LETTER GLOTTAL STOP */
  ["\u0241", "\u0242"], /* LATIN SMALL LETTER GLOTTAL STOP */
  ["\u0243", "\u0180"], /* LATIN CAPITAL LETTER B WITH STROKE */
  ["\u0244", "\u0289"], /* LATIN CAPITAL LETTER U BAR */
  ["\u0245", "\u028C"], /* LATIN CAPITAL LETTER TURNED V */
  ["\u0246", "\u0247"], /* LATIN CAPITAL LETTER E WITH STROKE */
  ["\u0246", "\u0247"], /* LATIN SMALL LETTER E WITH STROKE */
  ["\u0248", "\u0249"], /* LATIN CAPITAL LETTER J WITH STROKE */
  ["\u0248", "\u0249"], /* LATIN SMALL LETTER J WITH STROKE */
  ["\u024A", "\u024B"], /* LATIN CAPITAL LETTER SMALL Q WITH HOOK TAIL */
  ["\u024A", "\u024B"], /* LATIN SMALL LETTER Q WITH HOOK TAIL */
  ["\u024C", "\u024D"], /* LATIN CAPITAL LETTER R WITH STROKE */
  ["\u024C", "\u024D"], /* LATIN SMALL LETTER R WITH STROKE */
  ["\u024E", "\u024F"], /* LATIN CAPITAL LETTER Y WITH STROKE */
  ["\u024E", "\u024F"], /* LATIN SMALL LETTER Y WITH STROKE */
  ["\u2C6F", "\u0250"], /* LATIN SMALL LETTER TURNED A */
  ["\u2C6D", "\u0251"], /* LATIN SMALL LETTER ALPHA (LATIN SMALL LETTER SCRIPT A) */
  ["\u2C70", "\u0252"], /* LATIN SMALL LETTER TURNED ALPHA (LATIN SMALL LETTER TURNED SCRIPT A) */
  ["\u0181", "\u0253"], /* LATIN SMALL LETTER B WITH HOOK (LATIN SMALL LETTER B HOOK) */
  ["\u0186", "\u0254"], /* LATIN SMALL LETTER OPEN O */
  ["\u0255", "\u0255"], /* LATIN SMALL LETTER C WITH CURL (LATIN SMALL LETTER C CURL) */
  ["\u0189", "\u0256"], /* LATIN SMALL LETTER D WITH TAIL (LATIN SMALL LETTER D RETROFLEX HOOK) */
  ["\u018A", "\u0257"], /* LATIN SMALL LETTER D WITH HOOK (LATIN SMALL LETTER D HOOK) */
  ["\u0258", "\u0258"], /* LATIN SMALL LETTER REVERSED E */
  ["\u018F", "\u0259"], /* LATIN SMALL LETTER SCHWA */
  ["\u025A", "\u025A"], /* LATIN SMALL LETTER SCHWA WITH HOOK (LATIN SMALL LETTER SCHWA HOOK) */
  ["\u0190", "\u025B"], /* LATIN SMALL LETTER OPEN E (LATIN SMALL LETTER EPSILON) */
  ["\uA7AB", "\u025C"], /* LATIN SMALL LETTER REVERSED OPEN E (LATIN SMALL LETTER REVERSED EPSILON) */
  ["\u025D", "\u025D"], /* LATIN SMALL LETTER REVERSED OPEN E WITH HOOK (LATIN SMALL LETTER REVERSED EPSILON HOOK) */
  ["\u025E", "\u025E"], /* LATIN SMALL LETTER CLOSED REVERSED OPEN E (LATIN SMALL LETTER CLOSED REVERSED EPSILON) */
  ["\u025F", "\u025F"], /* LATIN SMALL LETTER DOTLESS J WITH STROKE (LATIN SMALL LETTER DOTLESS J BAR) */
  ["\u0193", "\u0260"], /* LATIN SMALL LETTER G WITH HOOK (LATIN SMALL LETTER G HOOK) */
  ["\uA7AC", "\u0261"], /* LATIN SMALL LETTER SCRIPT G */
  ["\u0262", "\u0262"], /* LATIN LETTER SMALL CAPITAL G */
  ["\u0194", "\u0263"], /* LATIN SMALL LETTER GAMMA */
  ["\u0264", "\u0264"], /* LATIN SMALL LETTER RAMS HORN (LATIN SMALL LETTER BABY GAMMA) */
  ["\uA78D", "\u0265"], /* LATIN SMALL LETTER TURNED H */
  ["\uA7AA", "\u0266"], /* LATIN SMALL LETTER H WITH HOOK (LATIN SMALL LETTER H HOOK) */
  ["\u0267", "\u0267"], /* LATIN SMALL LETTER HENG WITH HOOK (LATIN SMALL LETTER HENG HOOK) */
  ["\u0197", "\u0268"], /* LATIN SMALL LETTER I WITH STROKE (LATIN SMALL LETTER BARRED I) */
  ["\u0196", "\u0269"], /* LATIN SMALL LETTER IOTA */
  ["\uA7AE", "\u026A"], /* LATIN LETTER SMALL CAPITAL I */
  ["\u2C62", "\u026B"], /* LATIN SMALL LETTER L WITH MIDDLE TILDE */
  ["\uA7AD", "\u026C"], /* LATIN SMALL LETTER L WITH BELT (LATIN SMALL LETTER L BELT) */
  ["\u026D", "\u026D"], /* LATIN SMALL LETTER L WITH RETROFLEX HOOK (LATIN SMALL LETTER L RETROFLEX HOOK) */
  ["\u026E", "\u026E"], /* LATIN SMALL LETTER LEZH (LATIN SMALL LETTER L YOGH) */
  ["\u019C", "\u026F"], /* LATIN SMALL LETTER TURNED M */
  ["\u0270", "\u0270"], /* LATIN SMALL LETTER TURNED M WITH LONG LEG */
  ["\u2C6E", "\u0271"], /* LATIN SMALL LETTER M WITH HOOK (LATIN SMALL LETTER M HOOK) */
  ["\u019D", "\u0272"], /* LATIN SMALL LETTER N WITH LEFT HOOK (LATIN SMALL LETTER N HOOK) */
  ["\u0273", "\u0273"], /* LATIN SMALL LETTER N WITH RETROFLEX HOOK (LATIN SMALL LETTER N RETROFLEX HOOK) */
  ["\u0274", "\u0274"], /* LATIN LETTER SMALL CAPITAL N */
  ["\u019F", "\u0275"], /* LATIN SMALL LETTER BARRED O */
  ["\u0276", "\u0276"], /* LATIN LETTER SMALL CAPITAL OE (LATIN LETTER SMALL CAPITAL O E) */
  ["\u0277", "\u0277"], /* LATIN SMALL LETTER CLOSED OMEGA */
  ["\u0278", "\u0278"], /* LATIN SMALL LETTER PHI */
  ["\u0279", "\u0279"], /* LATIN SMALL LETTER TURNED R */
  ["\u027A", "\u027A"], /* LATIN SMALL LETTER TURNED R WITH LONG LEG */
  ["\u027B", "\u027B"], /* LATIN SMALL LETTER TURNED R WITH HOOK (LATIN SMALL LETTER TURNED R HOOK) */
  ["\u027C", "\u027C"], /* LATIN SMALL LETTER R WITH LONG LEG */
  ["\u2C64", "\u027D"], /* LATIN SMALL LETTER R WITH TAIL (LATIN SMALL LETTER R HOOK) */
  ["\u027E", "\u027E"], /* LATIN SMALL LETTER R WITH FISHHOOK (LATIN SMALL LETTER FISHHOOK R) */
  ["\u027F", "\u027F"], /* LATIN SMALL LETTER REVERSED R WITH FISHHOOK (LATIN SMALL LETTER REVERSED FISHHOOK R) */
  ["\u01A6", "\u0280"], /* LATIN LETTER SMALL CAPITAL R */
  ["\u0281", "\u0281"], /* LATIN LETTER SMALL CAPITAL INVERTED R */
  ["\uA7C5", "\u0282"], /* LATIN SMALL LETTER S WITH HOOK (LATIN SMALL LETTER S HOOK) */
  ["\u01A9", "\u0283"], /* LATIN SMALL LETTER ESH */
  ["\u0284", "\u0284"], /* LATIN SMALL LETTER DOTLESS J WITH STROKE AND HOOK (LATIN SMALL LETTER DOTLESS J BAR HOOK) */
  ["\u0285", "\u0285"], /* LATIN SMALL LETTER SQUAT REVERSED ESH */
  ["\u0286", "\u0286"], /* LATIN SMALL LETTER ESH WITH CURL (LATIN SMALL LETTER ESH CURL) */
  ["\uA7B1", "\u0287"], /* LATIN SMALL LETTER TURNED T */
  ["\u01AE", "\u0288"], /* LATIN SMALL LETTER T WITH RETROFLEX HOOK (LATIN SMALL LETTER T RETROFLEX HOOK) */
  ["\u0244", "\u0289"], /* LATIN SMALL LETTER U BAR */
  ["\u01B1", "\u028A"], /* LATIN SMALL LETTER UPSILON */
  ["\u01B2", "\u028B"], /* LATIN SMALL LETTER V WITH HOOK (LATIN SMALL LETTER SCRIPT V) */
  ["\u0245", "\u028C"], /* LATIN SMALL LETTER TURNED V */
  ["\u028D", "\u028D"], /* LATIN SMALL LETTER TURNED W */
  ["\u028E", "\u028E"], /* LATIN SMALL LETTER TURNED Y */
  ["\u028F", "\u028F"], /* LATIN LETTER SMALL CAPITAL Y */
  ["\u0290", "\u0290"], /* LATIN SMALL LETTER Z WITH RETROFLEX HOOK (LATIN SMALL LETTER Z RETROFLEX HOOK) */
  ["\u0291", "\u0291"], /* LATIN SMALL LETTER Z WITH CURL (LATIN SMALL LETTER Z CURL) */
  ["\u01B7", "\u0292"], /* LATIN SMALL LETTER EZH (LATIN SMALL LETTER YOGH) */
  ["\u0293", "\u0293"], /* LATIN SMALL LETTER EZH WITH CURL (LATIN SMALL LETTER YOGH CURL) */
  ["\u0294", "\u0294"], /* LATIN LETTER GLOTTAL STOP */
  ["\u0295", "\u0295"], /* LATIN LETTER PHARYNGEAL VOICED FRICATIVE (LATIN LETTER REVERSED GLOTTAL STOP) */
  ["\u0296", "\u0296"], /* LATIN LETTER INVERTED GLOTTAL STOP */
  ["\u0297", "\u0297"], /* LATIN LETTER STRETCHED C */
  ["\u0298", "\u0298"], /* LATIN LETTER BILABIAL CLICK (LATIN LETTER BULLSEYE) */
  ["\u0299", "\u0299"], /* LATIN LETTER SMALL CAPITAL B */
  ["\u029A", "\u029A"], /* LATIN SMALL LETTER CLOSED OPEN E (LATIN SMALL LETTER CLOSED EPSILON) */
  ["\u029B", "\u029B"], /* LATIN LETTER SMALL CAPITAL G WITH HOOK (LATIN LETTER SMALL CAPITAL G HOOK) */
  ["\u029C", "\u029C"], /* LATIN LETTER SMALL CAPITAL H */
  ["\uA7B2", "\u029D"], /* LATIN SMALL LETTER J WITH CROSSED-TAIL (LATIN SMALL LETTER CROSSED-TAIL J) */
  ["\uA7B0", "\u029E"], /* LATIN SMALL LETTER TURNED K */
  ["\u029F", "\u029F"], /* LATIN LETTER SMALL CAPITAL L */
  ["\u02A0", "\u02A0"], /* LATIN SMALL LETTER Q WITH HOOK (LATIN SMALL LETTER Q HOOK) */
  ["\u02A1", "\u02A1"], /* LATIN LETTER GLOTTAL STOP WITH STROKE (LATIN LETTER GLOTTAL STOP BAR) */
  ["\u02A2", "\u02A2"], /* LATIN LETTER REVERSED GLOTTAL STOP WITH STROKE (LATIN LETTER REVERSED GLOTTAL STOP BAR) */
  ["\u02A3", "\u02A3"], /* LATIN SMALL LETTER DZ DIGRAPH (LATIN SMALL LETTER D Z) */
  ["\u02A4", "\u02A4"], /* LATIN SMALL LETTER DEZH DIGRAPH (LATIN SMALL LETTER D YOGH) */
  ["\u02A5", "\u02A5"], /* LATIN SMALL LETTER DZ DIGRAPH WITH CURL (LATIN SMALL LETTER D Z CURL) */
  ["\u02A6", "\u02A6"], /* LATIN SMALL LETTER TS DIGRAPH (LATIN SMALL LETTER T S) */
  ["\u02A7", "\u02A7"], /* LATIN SMALL LETTER TESH DIGRAPH (LATIN SMALL LETTER T ESH) */
  ["\u02A8", "\u02A8"], /* LATIN SMALL LETTER TC DIGRAPH WITH CURL (LATIN SMALL LETTER T C CURL) */
  ["\u02A9", "\u02A9"], /* LATIN SMALL LETTER FENG DIGRAPH */
  ["\u02AA", "\u02AA"], /* LATIN SMALL LETTER LS DIGRAPH */
  ["\u02AB", "\u02AB"], /* LATIN SMALL LETTER LZ DIGRAPH */
  ["\u02AC", "\u02AC"], /* LATIN LETTER BILABIAL PERCUSSIVE */
  ["\u02AD", "\u02AD"], /* LATIN LETTER BIDENTAL PERCUSSIVE */
  ["\u02AE", "\u02AE"], /* LATIN SMALL LETTER TURNED H WITH FISHHOOK */
  ["\u02AF", "\u02AF"], /* LATIN SMALL LETTER TURNED H WITH FISHHOOK AND TAIL */
  ["\u02B0", "\u02B0"], /* MODIFIER LETTER SMALL H */
  ["\u02B1", "\u02B1"], /* MODIFIER LETTER SMALL H WITH HOOK (MODIFIER LETTER SMALL H HOOK) */
  ["\u02B2", "\u02B2"], /* MODIFIER LETTER SMALL J */
  ["\u02B3", "\u02B3"], /* MODIFIER LETTER SMALL R */
  ["\u02B4", "\u02B4"], /* MODIFIER LETTER SMALL TURNED R */
  ["\u02B5", "\u02B5"], /* MODIFIER LETTER SMALL TURNED R WITH HOOK (MODIFIER LETTER SMALL TURNED R HOOK) */
  ["\u02B6", "\u02B6"], /* MODIFIER LETTER SMALL CAPITAL INVERTED R */
  ["\u02B7", "\u02B7"], /* MODIFIER LETTER SMALL W */
  ["\u02B8", "\u02B8"], /* MODIFIER LETTER SMALL Y */
  ["\u02B9", "\u02B9"], /* MODIFIER LETTER PRIME */
  ["\u02BA", "\u02BA"], /* MODIFIER LETTER DOUBLE PRIME */
  ["\u02BB", "\u02BB"], /* MODIFIER LETTER TURNED COMMA */
  ["\u02BC", "\u02BC"], /* MODIFIER LETTER APOSTROPHE */
  ["\u02BD", "\u02BD"], /* MODIFIER LETTER REVERSED COMMA */
  ["\u02BE", "\u02BE"], /* MODIFIER LETTER RIGHT HALF RING */
  ["\u02BF", "\u02BF"], /* MODIFIER LETTER LEFT HALF RING */
  ["\u02C0", "\u02C0"], /* MODIFIER LETTER GLOTTAL STOP */
  ["\u02C1", "\u02C1"], /* MODIFIER LETTER REVERSED GLOTTAL STOP */
  ["\u02C2", "\u02C2"], /* MODIFIER LETTER LEFT ARROWHEAD */
  ["\u02C3", "\u02C3"], /* MODIFIER LETTER RIGHT ARROWHEAD */
  ["\u02C4", "\u02C4"], /* MODIFIER LETTER UP ARROWHEAD */
  ["\u02C5", "\u02C5"], /* MODIFIER LETTER DOWN ARROWHEAD */
  ["\u02C6", "\u02C6"], /* MODIFIER LETTER CIRCUMFLEX ACCENT (MODIFIER LETTER CIRCUMFLEX) */
  ["\u02C7", "\u02C7"], /* CARON (MODIFIER LETTER HACEK) */
  ["\u02C8", "\u02C8"], /* MODIFIER LETTER VERTICAL LINE */
  ["\u02C9", "\u02C9"], /* MODIFIER LETTER MACRON */
  ["\u02CA", "\u02CA"], /* MODIFIER LETTER ACUTE ACCENT (MODIFIER LETTER ACUTE) */
  ["\u02CB", "\u02CB"], /* MODIFIER LETTER GRAVE ACCENT (MODIFIER LETTER GRAVE) */
  ["\u02CC", "\u02CC"], /* MODIFIER LETTER LOW VERTICAL LINE */
  ["\u02CD", "\u02CD"], /* MODIFIER LETTER LOW MACRON */
  ["\u02CE", "\u02CE"], /* MODIFIER LETTER LOW GRAVE ACCENT (MODIFIER LETTER LOW GRAVE) */
  ["\u02CF", "\u02CF"], /* MODIFIER LETTER LOW ACUTE ACCENT (MODIFIER LETTER LOW ACUTE) */
  ["\u02D0", "\u02D0"], /* MODIFIER LETTER TRIANGULAR COLON */
  ["\u02D1", "\u02D1"], /* MODIFIER LETTER HALF TRIANGULAR COLON */
  ["\u02D2", "\u02D2"], /* MODIFIER LETTER CENTRED RIGHT HALF RING (MODIFIER LETTER CENTERED RIGHT HALF RING) */
  ["\u02D3", "\u02D3"], /* MODIFIER LETTER CENTRED LEFT HALF RING (MODIFIER LETTER CENTERED LEFT HALF RING) */
  ["\u02D4", "\u02D4"], /* MODIFIER LETTER UP TACK */
  ["\u02D5", "\u02D5"], /* MODIFIER LETTER DOWN TACK */
  ["\u02D6", "\u02D6"], /* MODIFIER LETTER PLUS SIGN */
  ["\u02D7", "\u02D7"], /* MODIFIER LETTER MINUS SIGN */
  ["\u02D8", "\u02D8"], /* BREVE (SPACING BREVE) */
  ["\u02D9", "\u02D9"], /* DOT ABOVE (SPACING DOT ABOVE) */
  ["\u02DA", "\u02DA"], /* RING ABOVE (SPACING RING ABOVE) */
  ["\u02DB", "\u02DB"], /* OGONEK (SPACING OGONEK) */
  ["\u02DC", "\u02DC"], /* SMALL TILDE (SPACING TILDE) */
  ["\u02DD", "\u02DD"], /* DOUBLE ACUTE ACCENT (SPACING DOUBLE ACUTE) */
  ["\u02DE", "\u02DE"], /* MODIFIER LETTER RHOTIC HOOK */
  ["\u02DF", "\u02DF"], /* MODIFIER LETTER CROSS ACCENT */
  ["\u02E0", "\u02E0"], /* MODIFIER LETTER SMALL GAMMA */
  ["\u02E1", "\u02E1"], /* MODIFIER LETTER SMALL L */
  ["\u02E2", "\u02E2"], /* MODIFIER LETTER SMALL S */
  ["\u02E3", "\u02E3"], /* MODIFIER LETTER SMALL X */
  ["\u02E4", "\u02E4"], /* MODIFIER LETTER SMALL REVERSED GLOTTAL STOP */
  ["\u02E5", "\u02E5"], /* MODIFIER LETTER EXTRA-HIGH TONE BAR */
  ["\u02E6", "\u02E6"], /* MODIFIER LETTER HIGH TONE BAR */
  ["\u02E7", "\u02E7"], /* MODIFIER LETTER MID TONE BAR */
  ["\u02E8", "\u02E8"], /* MODIFIER LETTER LOW TONE BAR */
  ["\u02E9", "\u02E9"], /* MODIFIER LETTER EXTRA-LOW TONE BAR */
  ["\u02EA", "\u02EA"], /* MODIFIER LETTER YIN DEPARTING TONE MARK */
  ["\u02EB", "\u02EB"], /* MODIFIER LETTER YANG DEPARTING TONE MARK */
  ["\u02EC", "\u02EC"], /* MODIFIER LETTER VOICING */
  ["\u02ED", "\u02ED"], /* MODIFIER LETTER UNASPIRATED */
  ["\u02EE", "\u02EE"], /* MODIFIER LETTER DOUBLE APOSTROPHE */
  ["\u02EF", "\u02EF"], /* MODIFIER LETTER LOW DOWN ARROWHEAD */
  ["\u02F0", "\u02F0"], /* MODIFIER LETTER LOW UP ARROWHEAD */
  ["\u02F1", "\u02F1"], /* MODIFIER LETTER LOW LEFT ARROWHEAD */
  ["\u02F2", "\u02F2"], /* MODIFIER LETTER LOW RIGHT ARROWHEAD */
  ["\u02F3", "\u02F3"], /* MODIFIER LETTER LOW RING */
  ["\u02F4", "\u02F4"], /* MODIFIER LETTER MIDDLE GRAVE ACCENT */
  ["\u02F5", "\u02F5"], /* MODIFIER LETTER MIDDLE DOUBLE GRAVE ACCENT */
  ["\u02F6", "\u02F6"], /* MODIFIER LETTER MIDDLE DOUBLE ACUTE ACCENT */
  ["\u02F7", "\u02F7"], /* MODIFIER LETTER LOW TILDE */
  ["\u02F8", "\u02F8"], /* MODIFIER LETTER RAISED COLON */
  ["\u02F9", "\u02F9"], /* MODIFIER LETTER BEGIN HIGH TONE */
  ["\u02FA", "\u02FA"], /* MODIFIER LETTER END HIGH TONE */
  ["\u02FB", "\u02FB"], /* MODIFIER LETTER BEGIN LOW TONE */
  ["\u02FC", "\u02FC"], /* MODIFIER LETTER END LOW TONE */
  ["\u02FD", "\u02FD"], /* MODIFIER LETTER SHELF */
  ["\u02FE", "\u02FE"], /* MODIFIER LETTER OPEN SHELF */
  ["\u02FF", "\u02FF"], /* MODIFIER LETTER LOW LEFT ARROW */
  ["\u0300", "\u0300"], /* COMBINING GRAVE ACCENT (NON-SPACING GRAVE) */
  ["\u0301", "\u0301"], /* COMBINING ACUTE ACCENT (NON-SPACING ACUTE) */
  ["\u0302", "\u0302"], /* COMBINING CIRCUMFLEX ACCENT (NON-SPACING CIRCUMFLEX) */
  ["\u0303", "\u0303"], /* COMBINING TILDE (NON-SPACING TILDE) */
  ["\u0304", "\u0304"], /* COMBINING MACRON (NON-SPACING MACRON) */
  ["\u0305", "\u0305"], /* COMBINING OVERLINE (NON-SPACING OVERSCORE) */
  ["\u0306", "\u0306"], /* COMBINING BREVE (NON-SPACING BREVE) */
  ["\u0307", "\u0307"], /* COMBINING DOT ABOVE (NON-SPACING DOT ABOVE) */
  ["\u0308", "\u0308"], /* COMBINING DIAERESIS (NON-SPACING DIAERESIS) */
  ["\u0309", "\u0309"], /* COMBINING HOOK ABOVE (NON-SPACING HOOK ABOVE) */
  ["\u030A", "\u030A"], /* COMBINING RING ABOVE (NON-SPACING RING ABOVE) */
  ["\u030B", "\u030B"], /* COMBINING DOUBLE ACUTE ACCENT (NON-SPACING DOUBLE ACUTE) */
  ["\u030C", "\u030C"], /* COMBINING CARON (NON-SPACING HACEK) */
  ["\u030D", "\u030D"], /* COMBINING VERTICAL LINE ABOVE (NON-SPACING VERTICAL LINE ABOVE) */
  ["\u030E", "\u030E"], /* COMBINING DOUBLE VERTICAL LINE ABOVE (NON-SPACING DOUBLE VERTICAL LINE ABOVE) */
  ["\u030F", "\u030F"], /* COMBINING DOUBLE GRAVE ACCENT (NON-SPACING DOUBLE GRAVE) */
  ["\u0310", "\u0310"], /* COMBINING CANDRABINDU (NON-SPACING CANDRABINDU) */
  ["\u0311", "\u0311"], /* COMBINING INVERTED BREVE (NON-SPACING INVERTED BREVE) */
  ["\u0312", "\u0312"], /* COMBINING TURNED COMMA ABOVE (NON-SPACING TURNED COMMA ABOVE) */
  ["\u0313", "\u0313"], /* COMBINING COMMA ABOVE (NON-SPACING COMMA ABOVE) */
  ["\u0314", "\u0314"], /* COMBINING REVERSED COMMA ABOVE (NON-SPACING REVERSED COMMA ABOVE) */
  ["\u0315", "\u0315"], /* COMBINING COMMA ABOVE RIGHT (NON-SPACING COMMA ABOVE RIGHT) */
  ["\u0316", "\u0316"], /* COMBINING GRAVE ACCENT BELOW (NON-SPACING GRAVE BELOW) */
  ["\u0317", "\u0317"], /* COMBINING ACUTE ACCENT BELOW (NON-SPACING ACUTE BELOW) */
  ["\u0318", "\u0318"], /* COMBINING LEFT TACK BELOW (NON-SPACING LEFT TACK BELOW) */
  ["\u0319", "\u0319"], /* COMBINING RIGHT TACK BELOW (NON-SPACING RIGHT TACK BELOW) */
  ["\u031A", "\u031A"], /* COMBINING LEFT ANGLE ABOVE (NON-SPACING LEFT ANGLE ABOVE) */
  ["\u031B", "\u031B"], /* COMBINING HORN (NON-SPACING HORN) */
  ["\u031C", "\u031C"], /* COMBINING LEFT HALF RING BELOW (NON-SPACING LEFT HALF RING BELOW) */
  ["\u031D", "\u031D"], /* COMBINING UP TACK BELOW (NON-SPACING UP TACK BELOW) */
  ["\u031E", "\u031E"], /* COMBINING DOWN TACK BELOW (NON-SPACING DOWN TACK BELOW) */
  ["\u031F", "\u031F"], /* COMBINING PLUS SIGN BELOW (NON-SPACING PLUS SIGN BELOW) */
  ["\u0320", "\u0320"], /* COMBINING MINUS SIGN BELOW (NON-SPACING MINUS SIGN BELOW) */
  ["\u0321", "\u0321"], /* COMBINING PALATALIZED HOOK BELOW (NON-SPACING PALATALIZED HOOK BELOW) */
  ["\u0322", "\u0322"], /* COMBINING RETROFLEX HOOK BELOW (NON-SPACING RETROFLEX HOOK BELOW) */
  ["\u0323", "\u0323"], /* COMBINING DOT BELOW (NON-SPACING DOT BELOW) */
  ["\u0324", "\u0324"], /* COMBINING DIAERESIS BELOW (NON-SPACING DOUBLE DOT BELOW) */
  ["\u0325", "\u0325"], /* COMBINING RING BELOW (NON-SPACING RING BELOW) */
  ["\u0326", "\u0326"], /* COMBINING COMMA BELOW (NON-SPACING COMMA BELOW) */
  ["\u0327", "\u0327"], /* COMBINING CEDILLA (NON-SPACING CEDILLA) */
  ["\u0328", "\u0328"], /* COMBINING OGONEK (NON-SPACING OGONEK) */
  ["\u0329", "\u0329"], /* COMBINING VERTICAL LINE BELOW (NON-SPACING VERTICAL LINE BELOW) */
  ["\u032A", "\u032A"], /* COMBINING BRIDGE BELOW (NON-SPACING BRIDGE BELOW) */
  ["\u032B", "\u032B"], /* COMBINING INVERTED DOUBLE ARCH BELOW (NON-SPACING INVERTED DOUBLE ARCH BELOW) */
  ["\u032C", "\u032C"], /* COMBINING CARON BELOW (NON-SPACING HACEK BELOW) */
  ["\u032D", "\u032D"], /* COMBINING CIRCUMFLEX ACCENT BELOW (NON-SPACING CIRCUMFLEX BELOW) */
  ["\u032E", "\u032E"], /* COMBINING BREVE BELOW (NON-SPACING BREVE BELOW) */
  ["\u032F", "\u032F"], /* COMBINING INVERTED BREVE BELOW (NON-SPACING INVERTED BREVE BELOW) */
  ["\u0330", "\u0330"], /* COMBINING TILDE BELOW (NON-SPACING TILDE BELOW) */
  ["\u0331", "\u0331"], /* COMBINING MACRON BELOW (NON-SPACING MACRON BELOW) */
  ["\u0332", "\u0332"], /* COMBINING LOW LINE (NON-SPACING UNDERSCORE) */
  ["\u0333", "\u0333"], /* COMBINING DOUBLE LOW LINE (NON-SPACING DOUBLE UNDERSCORE) */
  ["\u0334", "\u0334"], /* COMBINING TILDE OVERLAY (NON-SPACING TILDE OVERLAY) */
  ["\u0335", "\u0335"], /* COMBINING SHORT STROKE OVERLAY (NON-SPACING SHORT BAR OVERLAY) */
  ["\u0336", "\u0336"], /* COMBINING LONG STROKE OVERLAY (NON-SPACING LONG BAR OVERLAY) */
  ["\u0337", "\u0337"], /* COMBINING SHORT SOLIDUS OVERLAY (NON-SPACING SHORT SLASH OVERLAY) */
  ["\u0338", "\u0338"], /* COMBINING LONG SOLIDUS OVERLAY (NON-SPACING LONG SLASH OVERLAY) */
  ["\u0339", "\u0339"], /* COMBINING RIGHT HALF RING BELOW (NON-SPACING RIGHT HALF RING BELOW) */
  ["\u033A", "\u033A"], /* COMBINING INVERTED BRIDGE BELOW (NON-SPACING INVERTED BRIDGE BELOW) */
  ["\u033B", "\u033B"], /* COMBINING SQUARE BELOW (NON-SPACING SQUARE BELOW) */
  ["\u033C", "\u033C"], /* COMBINING SEAGULL BELOW (NON-SPACING SEAGULL BELOW) */
  ["\u033D", "\u033D"], /* COMBINING X ABOVE (NON-SPACING X ABOVE) */
  ["\u033E", "\u033E"], /* COMBINING VERTICAL TILDE (NON-SPACING VERTICAL TILDE) */
  ["\u033F", "\u033F"], /* COMBINING DOUBLE OVERLINE (NON-SPACING DOUBLE OVERSCORE) */
  ["\u0340", "\u0340"], /* COMBINING GRAVE TONE MARK (NON-SPACING GRAVE TONE MARK) */
  ["\u0341", "\u0341"], /* COMBINING ACUTE TONE MARK (NON-SPACING ACUTE TONE MARK) */
  ["\u0342", "\u0342"], /* COMBINING GREEK PERISPOMENI */
  ["\u0343", "\u0343"], /* COMBINING GREEK KORONIS */
  ["\u0344", "\u0344"], /* COMBINING GREEK DIALYTIKA TONOS (GREEK NON-SPACING DIAERESIS TONOS) */
  ["\u0399", "\u0345"], /* COMBINING GREEK YPOGEGRAMMENI (GREEK NON-SPACING IOTA BELOW) */
  ["\u0346", "\u0346"], /* COMBINING BRIDGE ABOVE */
  ["\u0347", "\u0347"], /* COMBINING EQUALS SIGN BELOW */
  ["\u0348", "\u0348"], /* COMBINING DOUBLE VERTICAL LINE BELOW */
  ["\u0349", "\u0349"], /* COMBINING LEFT ANGLE BELOW */
  ["\u034A", "\u034A"], /* COMBINING NOT TILDE ABOVE */
  ["\u034B", "\u034B"], /* COMBINING HOMOTHETIC ABOVE */
  ["\u034C", "\u034C"], /* COMBINING ALMOST EQUAL TO ABOVE */
  ["\u034D", "\u034D"], /* COMBINING LEFT RIGHT ARROW BELOW */
  ["\u034E", "\u034E"], /* COMBINING UPWARDS ARROW BELOW */
  ["\u034F", "\u034F"], /* COMBINING GRAPHEME JOINER */
  ["\u0350", "\u0350"], /* COMBINING RIGHT ARROWHEAD ABOVE */
  ["\u0351", "\u0351"], /* COMBINING LEFT HALF RING ABOVE */
  ["\u0352", "\u0352"], /* COMBINING FERMATA */
  ["\u0353", "\u0353"], /* COMBINING X BELOW */
  ["\u0354", "\u0354"], /* COMBINING LEFT ARROWHEAD BELOW */
  ["\u0355", "\u0355"], /* COMBINING RIGHT ARROWHEAD BELOW */
  ["\u0356", "\u0356"], /* COMBINING RIGHT ARROWHEAD AND UP ARROWHEAD BELOW */
  ["\u0357", "\u0357"], /* COMBINING RIGHT HALF RING ABOVE */
  ["\u0358", "\u0358"], /* COMBINING DOT ABOVE RIGHT */
  ["\u0359", "\u0359"], /* COMBINING ASTERISK BELOW */
  ["\u035A", "\u035A"], /* COMBINING DOUBLE RING BELOW */
  ["\u035B", "\u035B"], /* COMBINING ZIGZAG ABOVE */
  ["\u035C", "\u035C"], /* COMBINING DOUBLE BREVE BELOW */
  ["\u035D", "\u035D"], /* COMBINING DOUBLE BREVE */
  ["\u035E", "\u035E"], /* COMBINING DOUBLE MACRON */
  ["\u035F", "\u035F"], /* COMBINING DOUBLE MACRON BELOW */
  ["\u0360", "\u0360"], /* COMBINING DOUBLE TILDE */
  ["\u0361", "\u0361"], /* COMBINING DOUBLE INVERTED BREVE */
  ["\u0362", "\u0362"], /* COMBINING DOUBLE RIGHTWARDS ARROW BELOW */
  ["\u0363", "\u0363"], /* COMBINING LATIN SMALL LETTER A */
  ["\u0364", "\u0364"], /* COMBINING LATIN SMALL LETTER E */
  ["\u0365", "\u0365"], /* COMBINING LATIN SMALL LETTER I */
  ["\u0366", "\u0366"], /* COMBINING LATIN SMALL LETTER O */
  ["\u0367", "\u0367"], /* COMBINING LATIN SMALL LETTER U */
  ["\u0368", "\u0368"], /* COMBINING LATIN SMALL LETTER C */
  ["\u0369", "\u0369"], /* COMBINING LATIN SMALL LETTER D */
  ["\u036A", "\u036A"], /* COMBINING LATIN SMALL LETTER H */
  ["\u036B", "\u036B"], /* COMBINING LATIN SMALL LETTER M */
  ["\u036C", "\u036C"], /* COMBINING LATIN SMALL LETTER R */
  ["\u036D", "\u036D"], /* COMBINING LATIN SMALL LETTER T */
  ["\u036E", "\u036E"], /* COMBINING LATIN SMALL LETTER V */
  ["\u036F", "\u036F"], /* COMBINING LATIN SMALL LETTER X */
  ["\u0370", "\u0371"], /* GREEK CAPITAL LETTER HETA */
  ["\u0370", "\u0371"], /* GREEK SMALL LETTER HETA */
  ["\u0372", "\u0373"], /* GREEK CAPITAL LETTER ARCHAIC SAMPI */
  ["\u0372", "\u0373"], /* GREEK SMALL LETTER ARCHAIC SAMPI */
  ["\u0374", "\u0374"], /* GREEK NUMERAL SIGN (GREEK UPPER NUMERAL SIGN) */
  ["\u0375", "\u0375"], /* GREEK LOWER NUMERAL SIGN */
  ["\u0376", "\u0377"], /* GREEK CAPITAL LETTER PAMPHYLIAN DIGAMMA */
  ["\u0376", "\u0377"], /* GREEK SMALL LETTER PAMPHYLIAN DIGAMMA */
  ["\u0378", "\u0378"],
  ["\u0379", "\u0379"],
  ["\u037A", "\u037A"], /* GREEK YPOGEGRAMMENI (GREEK SPACING IOTA BELOW) */
  ["\u03FD", "\u037B"], /* GREEK SMALL REVERSED LUNATE SIGMA SYMBOL */
  ["\u03FE", "\u037C"], /* GREEK SMALL DOTTED LUNATE SIGMA SYMBOL */
  ["\u03FF", "\u037D"], /* GREEK SMALL REVERSED DOTTED LUNATE SIGMA SYMBOL */
  ["\u037E", "\u037E"], /* GREEK QUESTION MARK */
  ["\u037F", "\u03F3"], /* GREEK CAPITAL LETTER YOT */
  ["\u0380", "\u0380"],
  ["\u0381", "\u0381"],
  ["\u0382", "\u0382"],
  ["\u0383", "\u0383"],
  ["\u0384", "\u0384"], /* GREEK TONOS (GREEK SPACING TONOS) */
  ["\u0385", "\u0385"], /* GREEK DIALYTIKA TONOS (GREEK SPACING DIAERESIS TONOS) */
  ["\u0386", "\u03AC"], /* GREEK CAPITAL LETTER ALPHA WITH TONOS (GREEK CAPITAL LETTER ALPHA TONOS) */
  ["\u0387", "\u0387"], /* GREEK ANO TELEIA */
  ["\u0388", "\u03AD"], /* GREEK CAPITAL LETTER EPSILON WITH TONOS (GREEK CAPITAL LETTER EPSILON TONOS) */
  ["\u0389", "\u03AE"], /* GREEK CAPITAL LETTER ETA WITH TONOS (GREEK CAPITAL LETTER ETA TONOS) */
  ["\u038A", "\u03AF"], /* GREEK CAPITAL LETTER IOTA WITH TONOS (GREEK CAPITAL LETTER IOTA TONOS) */
  ["\u038B", "\u038B"],
  ["\u038C", "\u03CC"], /* GREEK CAPITAL LETTER OMICRON WITH TONOS (GREEK CAPITAL LETTER OMICRON TONOS) */
  ["\u038D", "\u038D"],
  ["\u038E", "\u03CD"], /* GREEK CAPITAL LETTER UPSILON WITH TONOS (GREEK CAPITAL LETTER UPSILON TONOS) */
  ["\u038F", "\u03CE"], /* GREEK CAPITAL LETTER OMEGA WITH TONOS (GREEK CAPITAL LETTER OMEGA TONOS) */
  ["\u0399\u0308\u0301", "\u0390"], /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND TONOS (GREEK SMALL LETTER IOTA DIAERESIS TONOS) */
  ["\u0391", "\u03B1"], /* GREEK CAPITAL LETTER ALPHA */
  ["\u0392", "\u03B2"], /* GREEK CAPITAL LETTER BETA */
  ["\u0393", "\u03B3"], /* GREEK CAPITAL LETTER GAMMA */
  ["\u0394", "\u03B4"], /* GREEK CAPITAL LETTER DELTA */
  ["\u0395", "\u03B5"], /* GREEK CAPITAL LETTER EPSILON */
  ["\u0396", "\u03B6"], /* GREEK CAPITAL LETTER ZETA */
  ["\u0397", "\u03B7"], /* GREEK CAPITAL LETTER ETA */
  ["\u0398", "\u03B8"], /* GREEK CAPITAL LETTER THETA */
  ["\u0399", "\u03B9"], /* GREEK CAPITAL LETTER IOTA */
  ["\u039A", "\u03BA"], /* GREEK CAPITAL LETTER KAPPA */
  ["\u039B", "\u03BB"], /* GREEK CAPITAL LETTER LAMDA (GREEK CAPITAL LETTER LAMBDA) */
  ["\u039C", "\u03BC"], /* GREEK CAPITAL LETTER MU */
  ["\u039D", "\u03BD"], /* GREEK CAPITAL LETTER NU */
  ["\u039E", "\u03BE"], /* GREEK CAPITAL LETTER XI */
  ["\u039F", "\u03BF"], /* GREEK CAPITAL LETTER OMICRON */
  ["\u03A0", "\u03C0"], /* GREEK CAPITAL LETTER PI */
  ["\u03A1", "\u03C1"], /* GREEK CAPITAL LETTER RHO */
  ["\u03A2", "\u03A2"],
  ["\u03A3", "\u03C3"], /* GREEK CAPITAL LETTER SIGMA */
  ["\u03A4", "\u03C4"], /* GREEK CAPITAL LETTER TAU */
  ["\u03A5", "\u03C5"], /* GREEK CAPITAL LETTER UPSILON */
  ["\u03A6", "\u03C6"], /* GREEK CAPITAL LETTER PHI */
  ["\u03A7", "\u03C7"], /* GREEK CAPITAL LETTER CHI */
  ["\u03A8", "\u03C8"], /* GREEK CAPITAL LETTER PSI */
  ["\u03A9", "\u03C9"], /* GREEK CAPITAL LETTER OMEGA */
  ["\u03AA", "\u03CA"], /* GREEK CAPITAL LETTER IOTA WITH DIALYTIKA (GREEK CAPITAL LETTER IOTA DIAERESIS) */
  ["\u03AB", "\u03CB"], /* GREEK CAPITAL LETTER UPSILON WITH DIALYTIKA (GREEK CAPITAL LETTER UPSILON DIAERESIS) */
  ["\u0386", "\u03AC"], /* GREEK SMALL LETTER ALPHA WITH TONOS (GREEK SMALL LETTER ALPHA TONOS) */
  ["\u0388", "\u03AD"], /* GREEK SMALL LETTER EPSILON WITH TONOS (GREEK SMALL LETTER EPSILON TONOS) */
  ["\u0389", "\u03AE"], /* GREEK SMALL LETTER ETA WITH TONOS (GREEK SMALL LETTER ETA TONOS) */
  ["\u038A", "\u03AF"], /* GREEK SMALL LETTER IOTA WITH TONOS (GREEK SMALL LETTER IOTA TONOS) */
  ["\u03A5\u0308\u0301", "\u03B0"], /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND TONOS (GREEK SMALL LETTER UPSILON DIAERESIS TONOS) */
  ["\u0391", "\u03B1"], /* GREEK SMALL LETTER ALPHA */
  ["\u0392", "\u03B2"], /* GREEK SMALL LETTER BETA */
  ["\u0393", "\u03B3"], /* GREEK SMALL LETTER GAMMA */
  ["\u0394", "\u03B4"], /* GREEK SMALL LETTER DELTA */
  ["\u0395", "\u03B5"], /* GREEK SMALL LETTER EPSILON */
  ["\u0396", "\u03B6"], /* GREEK SMALL LETTER ZETA */
  ["\u0397", "\u03B7"], /* GREEK SMALL LETTER ETA */
  ["\u0398", "\u03B8"], /* GREEK SMALL LETTER THETA */
  ["\u0399", "\u03B9"], /* GREEK SMALL LETTER IOTA */
  ["\u039A", "\u03BA"], /* GREEK SMALL LETTER KAPPA */
  ["\u039B", "\u03BB"], /* GREEK SMALL LETTER LAMDA (GREEK SMALL LETTER LAMBDA) */
  ["\u039C", "\u03BC"], /* GREEK SMALL LETTER MU */
  ["\u039D", "\u03BD"], /* GREEK SMALL LETTER NU */
  ["\u039E", "\u03BE"], /* GREEK SMALL LETTER XI */
  ["\u039F", "\u03BF"], /* GREEK SMALL LETTER OMICRON */
  ["\u03A0", "\u03C0"], /* GREEK SMALL LETTER PI */
  ["\u03A1", "\u03C1"], /* GREEK SMALL LETTER RHO */
  ["\u03A3", "\u03C2"], /* GREEK SMALL LETTER FINAL SIGMA */
  ["\u03A3", "\u03C3"], /* GREEK SMALL LETTER SIGMA */
  ["\u03A4", "\u03C4"], /* GREEK SMALL LETTER TAU */
  ["\u03A5", "\u03C5"], /* GREEK SMALL LETTER UPSILON */
  ["\u03A6", "\u03C6"], /* GREEK SMALL LETTER PHI */
  ["\u03A7", "\u03C7"], /* GREEK SMALL LETTER CHI */
  ["\u03A8", "\u03C8"], /* GREEK SMALL LETTER PSI */
  ["\u03A9", "\u03C9"], /* GREEK SMALL LETTER OMEGA */
  ["\u03AA", "\u03CA"], /* GREEK SMALL LETTER IOTA WITH DIALYTIKA (GREEK SMALL LETTER IOTA DIAERESIS) */
  ["\u03AB", "\u03CB"], /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA (GREEK SMALL LETTER UPSILON DIAERESIS) */
  ["\u038C", "\u03CC"], /* GREEK SMALL LETTER OMICRON WITH TONOS (GREEK SMALL LETTER OMICRON TONOS) */
  ["\u038E", "\u03CD"], /* GREEK SMALL LETTER UPSILON WITH TONOS (GREEK SMALL LETTER UPSILON TONOS) */
  ["\u038F", "\u03CE"], /* GREEK SMALL LETTER OMEGA WITH TONOS (GREEK SMALL LETTER OMEGA TONOS) */
  ["\u03CF", "\u03D7"], /* GREEK CAPITAL KAI SYMBOL */
  ["\u0392", "\u03D0"], /* GREEK BETA SYMBOL (GREEK SMALL LETTER CURLED BETA) */
  ["\u0398", "\u03D1"], /* GREEK THETA SYMBOL (GREEK SMALL LETTER SCRIPT THETA) */
  ["\u03D2", "\u03D2"], /* GREEK UPSILON WITH HOOK SYMBOL (GREEK CAPITAL LETTER UPSILON HOOK) */
  ["\u03D3", "\u03D3"], /* GREEK UPSILON WITH ACUTE AND HOOK SYMBOL (GREEK CAPITAL LETTER UPSILON HOOK TONOS) */
  ["\u03D4", "\u03D4"], /* GREEK UPSILON WITH DIAERESIS AND HOOK SYMBOL (GREEK CAPITAL LETTER UPSILON HOOK DIAERESIS) */
  ["\u03A6", "\u03D5"], /* GREEK PHI SYMBOL (GREEK SMALL LETTER SCRIPT PHI) */
  ["\u03A0", "\u03D6"], /* GREEK PI SYMBOL (GREEK SMALL LETTER OMEGA PI) */
  ["\u03CF", "\u03D7"], /* GREEK KAI SYMBOL */
  ["\u03D8", "\u03D9"], /* GREEK LETTER ARCHAIC KOPPA */
  ["\u03D8", "\u03D9"], /* GREEK SMALL LETTER ARCHAIC KOPPA */
  ["\u03DA", "\u03DB"], /* GREEK LETTER STIGMA (GREEK CAPITAL LETTER STIGMA) */
  ["\u03DA", "\u03DB"], /* GREEK SMALL LETTER STIGMA */
  ["\u03DC", "\u03DD"], /* GREEK LETTER DIGAMMA (GREEK CAPITAL LETTER DIGAMMA) */
  ["\u03DC", "\u03DD"], /* GREEK SMALL LETTER DIGAMMA */
  ["\u03DE", "\u03DF"], /* GREEK LETTER KOPPA (GREEK CAPITAL LETTER KOPPA) */
  ["\u03DE", "\u03DF"], /* GREEK SMALL LETTER KOPPA */
  ["\u03E0", "\u03E1"], /* GREEK LETTER SAMPI (GREEK CAPITAL LETTER SAMPI) */
  ["\u03E0", "\u03E1"], /* GREEK SMALL LETTER SAMPI */
  ["\u03E2", "\u03E3"], /* COPTIC CAPITAL LETTER SHEI (GREEK CAPITAL LETTER SHEI) */
  ["\u03E2", "\u03E3"], /* COPTIC SMALL LETTER SHEI (GREEK SMALL LETTER SHEI) */
  ["\u03E4", "\u03E5"], /* COPTIC CAPITAL LETTER FEI (GREEK CAPITAL LETTER FEI) */
  ["\u03E4", "\u03E5"], /* COPTIC SMALL LETTER FEI (GREEK SMALL LETTER FEI) */
  ["\u03E6", "\u03E7"], /* COPTIC CAPITAL LETTER KHEI (GREEK CAPITAL LETTER KHEI) */
  ["\u03E6", "\u03E7"], /* COPTIC SMALL LETTER KHEI (GREEK SMALL LETTER KHEI) */
  ["\u03E8", "\u03E9"], /* COPTIC CAPITAL LETTER HORI (GREEK CAPITAL LETTER HORI) */
  ["\u03E8", "\u03E9"], /* COPTIC SMALL LETTER HORI (GREEK SMALL LETTER HORI) */
  ["\u03EA", "\u03EB"], /* COPTIC CAPITAL LETTER GANGIA (GREEK CAPITAL LETTER GANGIA) */
  ["\u03EA", "\u03EB"], /* COPTIC SMALL LETTER GANGIA (GREEK SMALL LETTER GANGIA) */
  ["\u03EC", "\u03ED"], /* COPTIC CAPITAL LETTER SHIMA (GREEK CAPITAL LETTER SHIMA) */
  ["\u03EC", "\u03ED"], /* COPTIC SMALL LETTER SHIMA (GREEK SMALL LETTER SHIMA) */
  ["\u03EE", "\u03EF"], /* COPTIC CAPITAL LETTER DEI (GREEK CAPITAL LETTER DEI) */
  ["\u03EE", "\u03EF"], /* COPTIC SMALL LETTER DEI (GREEK SMALL LETTER DEI) */
  ["\u039A", "\u03F0"], /* GREEK KAPPA SYMBOL (GREEK SMALL LETTER SCRIPT KAPPA) */
  ["\u03A1", "\u03F1"], /* GREEK RHO SYMBOL (GREEK SMALL LETTER TAILED RHO) */
  ["\u03F9", "\u03F2"], /* GREEK LUNATE SIGMA SYMBOL (GREEK SMALL LETTER LUNATE SIGMA) */
  ["\u037F", "\u03F3"], /* GREEK LETTER YOT */
  ["\u03F4", "\u03B8"], /* GREEK CAPITAL THETA SYMBOL */
  ["\u0395", "\u03F5"], /* GREEK LUNATE EPSILON SYMBOL */
  ["\u03F6", "\u03F6"], /* GREEK REVERSED LUNATE EPSILON SYMBOL */
  ["\u03F7", "\u03F8"], /* GREEK CAPITAL LETTER SHO */
  ["\u03F7", "\u03F8"], /* GREEK SMALL LETTER SHO */
  ["\u03F9", "\u03F2"], /* GREEK CAPITAL LUNATE SIGMA SYMBOL */
  ["\u03FA", "\u03FB"], /* GREEK CAPITAL LETTER SAN */
  ["\u03FA", "\u03FB"], /* GREEK SMALL LETTER SAN */
  ["\u03FC", "\u03FC"], /* GREEK RHO WITH STROKE SYMBOL */
  ["\u03FD", "\u037B"], /* GREEK CAPITAL REVERSED LUNATE SIGMA SYMBOL */
  ["\u03FE", "\u037C"], /* GREEK CAPITAL DOTTED LUNATE SIGMA SYMBOL */
  ["\u03FF", "\u037D"], /* GREEK CAPITAL REVERSED DOTTED LUNATE SIGMA SYMBOL */
  ["\u0400", "\u0450"], /* CYRILLIC CAPITAL LETTER IE WITH GRAVE */
  ["\u0401", "\u0451"], /* CYRILLIC CAPITAL LETTER IO */
  ["\u0402", "\u0452"], /* CYRILLIC CAPITAL LETTER DJE */
  ["\u0403", "\u0453"], /* CYRILLIC CAPITAL LETTER GJE */
  ["\u0404", "\u0454"], /* CYRILLIC CAPITAL LETTER UKRAINIAN IE (CYRILLIC CAPITAL LETTER E) */
  ["\u0405", "\u0455"], /* CYRILLIC CAPITAL LETTER DZE */
  ["\u0406", "\u0456"], /* CYRILLIC CAPITAL LETTER BYELORUSSIAN-UKRAINIAN I (CYRILLIC CAPITAL LETTER I) */
  ["\u0407", "\u0457"], /* CYRILLIC CAPITAL LETTER YI */
  ["\u0408", "\u0458"], /* CYRILLIC CAPITAL LETTER JE */
  ["\u0409", "\u0459"], /* CYRILLIC CAPITAL LETTER LJE */
  ["\u040A", "\u045A"], /* CYRILLIC CAPITAL LETTER NJE */
  ["\u040B", "\u045B"], /* CYRILLIC CAPITAL LETTER TSHE */
  ["\u040C", "\u045C"], /* CYRILLIC CAPITAL LETTER KJE */
  ["\u040D", "\u045D"], /* CYRILLIC CAPITAL LETTER I WITH GRAVE */
  ["\u040E", "\u045E"], /* CYRILLIC CAPITAL LETTER SHORT U */
  ["\u040F", "\u045F"], /* CYRILLIC CAPITAL LETTER DZHE */
  ["\u0410", "\u0430"], /* CYRILLIC CAPITAL LETTER A */
  ["\u0411", "\u0431"], /* CYRILLIC CAPITAL LETTER BE */
  ["\u0412", "\u0432"], /* CYRILLIC CAPITAL LETTER VE */
  ["\u0413", "\u0433"], /* CYRILLIC CAPITAL LETTER GHE (CYRILLIC CAPITAL LETTER GE) */
  ["\u0414", "\u0434"], /* CYRILLIC CAPITAL LETTER DE */
  ["\u0415", "\u0435"], /* CYRILLIC CAPITAL LETTER IE */
  ["\u0416", "\u0436"], /* CYRILLIC CAPITAL LETTER ZHE */
  ["\u0417", "\u0437"], /* CYRILLIC CAPITAL LETTER ZE */
  ["\u0418", "\u0438"], /* CYRILLIC CAPITAL LETTER I (CYRILLIC CAPITAL LETTER II) */
  ["\u0419", "\u0439"], /* CYRILLIC CAPITAL LETTER SHORT I (CYRILLIC CAPITAL LETTER SHORT II) */
  ["\u041A", "\u043A"], /* CYRILLIC CAPITAL LETTER KA */
  ["\u041B", "\u043B"], /* CYRILLIC CAPITAL LETTER EL */
  ["\u041C", "\u043C"], /* CYRILLIC CAPITAL LETTER EM */
  ["\u041D", "\u043D"], /* CYRILLIC CAPITAL LETTER EN */
  ["\u041E", "\u043E"], /* CYRILLIC CAPITAL LETTER O */
  ["\u041F", "\u043F"], /* CYRILLIC CAPITAL LETTER PE */
  ["\u0420", "\u0440"], /* CYRILLIC CAPITAL LETTER ER */
  ["\u0421", "\u0441"], /* CYRILLIC CAPITAL LETTER ES */
  ["\u0422", "\u0442"], /* CYRILLIC CAPITAL LETTER TE */
  ["\u0423", "\u0443"], /* CYRILLIC CAPITAL LETTER U */
  ["\u0424", "\u0444"], /* CYRILLIC CAPITAL LETTER EF */
  ["\u0425", "\u0445"], /* CYRILLIC CAPITAL LETTER HA (CYRILLIC CAPITAL LETTER KHA) */
  ["\u0426", "\u0446"], /* CYRILLIC CAPITAL LETTER TSE */
  ["\u0427", "\u0447"], /* CYRILLIC CAPITAL LETTER CHE */
  ["\u0428", "\u0448"], /* CYRILLIC CAPITAL LETTER SHA */
  ["\u0429", "\u0449"], /* CYRILLIC CAPITAL LETTER SHCHA */
  ["\u042A", "\u044A"], /* CYRILLIC CAPITAL LETTER HARD SIGN */
  ["\u042B", "\u044B"], /* CYRILLIC CAPITAL LETTER YERU (CYRILLIC CAPITAL LETTER YERI) */
  ["\u042C", "\u044C"], /* CYRILLIC CAPITAL LETTER SOFT SIGN */
  ["\u042D", "\u044D"], /* CYRILLIC CAPITAL LETTER E (CYRILLIC CAPITAL LETTER REVERSED E) */
  ["\u042E", "\u044E"], /* CYRILLIC CAPITAL LETTER YU (CYRILLIC CAPITAL LETTER IU) */
  ["\u042F", "\u044F"], /* CYRILLIC CAPITAL LETTER YA (CYRILLIC CAPITAL LETTER IA) */
  ["\u0410", "\u0430"], /* CYRILLIC SMALL LETTER A */
  ["\u0411", "\u0431"], /* CYRILLIC SMALL LETTER BE */
  ["\u0412", "\u0432"], /* CYRILLIC SMALL LETTER VE */
  ["\u0413", "\u0433"], /* CYRILLIC SMALL LETTER GHE (CYRILLIC SMALL LETTER GE) */
  ["\u0414", "\u0434"], /* CYRILLIC SMALL LETTER DE */
  ["\u0415", "\u0435"], /* CYRILLIC SMALL LETTER IE */
  ["\u0416", "\u0436"], /* CYRILLIC SMALL LETTER ZHE */
  ["\u0417", "\u0437"], /* CYRILLIC SMALL LETTER ZE */
  ["\u0418", "\u0438"], /* CYRILLIC SMALL LETTER I (CYRILLIC SMALL LETTER II) */
  ["\u0419", "\u0439"], /* CYRILLIC SMALL LETTER SHORT I (CYRILLIC SMALL LETTER SHORT II) */
  ["\u041A", "\u043A"], /* CYRILLIC SMALL LETTER KA */
  ["\u041B", "\u043B"], /* CYRILLIC SMALL LETTER EL */
  ["\u041C", "\u043C"], /* CYRILLIC SMALL LETTER EM */
  ["\u041D", "\u043D"], /* CYRILLIC SMALL LETTER EN */
  ["\u041E", "\u043E"], /* CYRILLIC SMALL LETTER O */
  ["\u041F", "\u043F"], /* CYRILLIC SMALL LETTER PE */
  ["\u0420", "\u0440"], /* CYRILLIC SMALL LETTER ER */
  ["\u0421", "\u0441"], /* CYRILLIC SMALL LETTER ES */
  ["\u0422", "\u0442"], /* CYRILLIC SMALL LETTER TE */
  ["\u0423", "\u0443"], /* CYRILLIC SMALL LETTER U */
  ["\u0424", "\u0444"], /* CYRILLIC SMALL LETTER EF */
  ["\u0425", "\u0445"], /* CYRILLIC SMALL LETTER HA (CYRILLIC SMALL LETTER KHA) */
  ["\u0426", "\u0446"], /* CYRILLIC SMALL LETTER TSE */
  ["\u0427", "\u0447"], /* CYRILLIC SMALL LETTER CHE */
  ["\u0428", "\u0448"], /* CYRILLIC SMALL LETTER SHA */
  ["\u0429", "\u0449"], /* CYRILLIC SMALL LETTER SHCHA */
  ["\u042A", "\u044A"], /* CYRILLIC SMALL LETTER HARD SIGN */
  ["\u042B", "\u044B"], /* CYRILLIC SMALL LETTER YERU (CYRILLIC SMALL LETTER YERI) */
  ["\u042C", "\u044C"], /* CYRILLIC SMALL LETTER SOFT SIGN */
  ["\u042D", "\u044D"], /* CYRILLIC SMALL LETTER E (CYRILLIC SMALL LETTER REVERSED E) */
  ["\u042E", "\u044E"], /* CYRILLIC SMALL LETTER YU (CYRILLIC SMALL LETTER IU) */
  ["\u042F", "\u044F"], /* CYRILLIC SMALL LETTER YA (CYRILLIC SMALL LETTER IA) */
  ["\u0400", "\u0450"], /* CYRILLIC SMALL LETTER IE WITH GRAVE */
  ["\u0401", "\u0451"], /* CYRILLIC SMALL LETTER IO */
  ["\u0402", "\u0452"], /* CYRILLIC SMALL LETTER DJE */
  ["\u0403", "\u0453"], /* CYRILLIC SMALL LETTER GJE */
  ["\u0404", "\u0454"], /* CYRILLIC SMALL LETTER UKRAINIAN IE (CYRILLIC SMALL LETTER E) */
  ["\u0405", "\u0455"], /* CYRILLIC SMALL LETTER DZE */
  ["\u0406", "\u0456"], /* CYRILLIC SMALL LETTER BYELORUSSIAN-UKRAINIAN I (CYRILLIC SMALL LETTER I) */
  ["\u0407", "\u0457"], /* CYRILLIC SMALL LETTER YI */
  ["\u0408", "\u0458"], /* CYRILLIC SMALL LETTER JE */
  ["\u0409", "\u0459"], /* CYRILLIC SMALL LETTER LJE */
  ["\u040A", "\u045A"], /* CYRILLIC SMALL LETTER NJE */
  ["\u040B", "\u045B"], /* CYRILLIC SMALL LETTER TSHE */
  ["\u040C", "\u045C"], /* CYRILLIC SMALL LETTER KJE */
  ["\u040D", "\u045D"], /* CYRILLIC SMALL LETTER I WITH GRAVE */
  ["\u040E", "\u045E"], /* CYRILLIC SMALL LETTER SHORT U */
  ["\u040F", "\u045F"], /* CYRILLIC SMALL LETTER DZHE */
  ["\u0460", "\u0461"], /* CYRILLIC CAPITAL LETTER OMEGA */
  ["\u0460", "\u0461"], /* CYRILLIC SMALL LETTER OMEGA */
  ["\u0462", "\u0463"], /* CYRILLIC CAPITAL LETTER YAT */
  ["\u0462", "\u0463"], /* CYRILLIC SMALL LETTER YAT */
  ["\u0464", "\u0465"], /* CYRILLIC CAPITAL LETTER IOTIFIED E */
  ["\u0464", "\u0465"], /* CYRILLIC SMALL LETTER IOTIFIED E */
  ["\u0466", "\u0467"], /* CYRILLIC CAPITAL LETTER LITTLE YUS */
  ["\u0466", "\u0467"], /* CYRILLIC SMALL LETTER LITTLE YUS */
  ["\u0468", "\u0469"], /* CYRILLIC CAPITAL LETTER IOTIFIED LITTLE YUS */
  ["\u0468", "\u0469"], /* CYRILLIC SMALL LETTER IOTIFIED LITTLE YUS */
  ["\u046A", "\u046B"], /* CYRILLIC CAPITAL LETTER BIG YUS */
  ["\u046A", "\u046B"], /* CYRILLIC SMALL LETTER BIG YUS */
  ["\u046C", "\u046D"], /* CYRILLIC CAPITAL LETTER IOTIFIED BIG YUS */
  ["\u046C", "\u046D"], /* CYRILLIC SMALL LETTER IOTIFIED BIG YUS */
  ["\u046E", "\u046F"], /* CYRILLIC CAPITAL LETTER KSI */
  ["\u046E", "\u046F"], /* CYRILLIC SMALL LETTER KSI */
  ["\u0470", "\u0471"], /* CYRILLIC CAPITAL LETTER PSI */
  ["\u0470", "\u0471"], /* CYRILLIC SMALL LETTER PSI */
  ["\u0472", "\u0473"], /* CYRILLIC CAPITAL LETTER FITA */
  ["\u0472", "\u0473"], /* CYRILLIC SMALL LETTER FITA */
  ["\u0474", "\u0475"], /* CYRILLIC CAPITAL LETTER IZHITSA */
  ["\u0474", "\u0475"], /* CYRILLIC SMALL LETTER IZHITSA */
  ["\u0476", "\u0477"], /* CYRILLIC CAPITAL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT (CYRILLIC CAPITAL LETTER IZHITSA DOUBLE GRAVE) */
  ["\u0476", "\u0477"], /* CYRILLIC SMALL LETTER IZHITSA WITH DOUBLE GRAVE ACCENT (CYRILLIC SMALL LETTER IZHITSA DOUBLE GRAVE) */
  ["\u0478", "\u0479"], /* CYRILLIC CAPITAL LETTER UK (CYRILLIC CAPITAL LETTER UK DIGRAPH) */
  ["\u0478", "\u0479"], /* CYRILLIC SMALL LETTER UK (CYRILLIC SMALL LETTER UK DIGRAPH) */
  ["\u047A", "\u047B"], /* CYRILLIC CAPITAL LETTER ROUND OMEGA */
  ["\u047A", "\u047B"], /* CYRILLIC SMALL LETTER ROUND OMEGA */
  ["\u047C", "\u047D"], /* CYRILLIC CAPITAL LETTER OMEGA WITH TITLO (CYRILLIC CAPITAL LETTER OMEGA TITLO) */
  ["\u047C", "\u047D"], /* CYRILLIC SMALL LETTER OMEGA WITH TITLO (CYRILLIC SMALL LETTER OMEGA TITLO) */
  ["\u047E", "\u047F"], /* CYRILLIC CAPITAL LETTER OT */
  ["\u047E", "\u047F"], /* CYRILLIC SMALL LETTER OT */
  ["\u0480", "\u0481"], /* CYRILLIC CAPITAL LETTER KOPPA */
  ["\u0480", "\u0481"], /* CYRILLIC SMALL LETTER KOPPA */
  ["\u0482", "\u0482"], /* CYRILLIC THOUSANDS SIGN */
  ["\u0483", "\u0483"], /* COMBINING CYRILLIC TITLO (CYRILLIC NON-SPACING TITLO) */
  ["\u0484", "\u0484"], /* COMBINING CYRILLIC PALATALIZATION (CYRILLIC NON-SPACING PALATALIZATION) */
  ["\u0485", "\u0485"], /* COMBINING CYRILLIC DASIA PNEUMATA (CYRILLIC NON-SPACING DASIA PNEUMATA) */
  ["\u0486", "\u0486"], /* COMBINING CYRILLIC PSILI PNEUMATA (CYRILLIC NON-SPACING PSILI PNEUMATA) */
  ["\u0487", "\u0487"], /* COMBINING CYRILLIC POKRYTIE */
  ["\u0488", "\u0488"], /* COMBINING CYRILLIC HUNDRED THOUSANDS SIGN */
  ["\u0489", "\u0489"], /* COMBINING CYRILLIC MILLIONS SIGN */
  ["\u048A", "\u048B"], /* CYRILLIC CAPITAL LETTER SHORT I WITH TAIL */
  ["\u048A", "\u048B"], /* CYRILLIC SMALL LETTER SHORT I WITH TAIL */
  ["\u048C", "\u048D"], /* CYRILLIC CAPITAL LETTER SEMISOFT SIGN */
  ["\u048C", "\u048D"], /* CYRILLIC SMALL LETTER SEMISOFT SIGN */
  ["\u048E", "\u048F"], /* CYRILLIC CAPITAL LETTER ER WITH TICK */
  ["\u048E", "\u048F"], /* CYRILLIC SMALL LETTER ER WITH TICK */
  ["\u0490", "\u0491"], /* CYRILLIC CAPITAL LETTER GHE WITH UPTURN (CYRILLIC CAPITAL LETTER GE WITH UPTURN) */
  ["\u0490", "\u0491"], /* CYRILLIC SMALL LETTER GHE WITH UPTURN (CYRILLIC SMALL LETTER GE WITH UPTURN) */
  ["\u0492", "\u0493"], /* CYRILLIC CAPITAL LETTER GHE WITH STROKE (CYRILLIC CAPITAL LETTER GE BAR) */
  ["\u0492", "\u0493"], /* CYRILLIC SMALL LETTER GHE WITH STROKE (CYRILLIC SMALL LETTER GE BAR) */
  ["\u0494", "\u0495"], /* CYRILLIC CAPITAL LETTER GHE WITH MIDDLE HOOK (CYRILLIC CAPITAL LETTER GE HOOK) */
  ["\u0494", "\u0495"], /* CYRILLIC SMALL LETTER GHE WITH MIDDLE HOOK (CYRILLIC SMALL LETTER GE HOOK) */
  ["\u0496", "\u0497"], /* CYRILLIC CAPITAL LETTER ZHE WITH DESCENDER (CYRILLIC CAPITAL LETTER ZHE WITH RIGHT DESCENDER) */
  ["\u0496", "\u0497"], /* CYRILLIC SMALL LETTER ZHE WITH DESCENDER (CYRILLIC SMALL LETTER ZHE WITH RIGHT DESCENDER) */
  ["\u0498", "\u0499"], /* CYRILLIC CAPITAL LETTER ZE WITH DESCENDER (CYRILLIC CAPITAL LETTER ZE CEDILLA) */
  ["\u0498", "\u0499"], /* CYRILLIC SMALL LETTER ZE WITH DESCENDER (CYRILLIC SMALL LETTER ZE CEDILLA) */
  ["\u049A", "\u049B"], /* CYRILLIC CAPITAL LETTER KA WITH DESCENDER (CYRILLIC CAPITAL LETTER KA WITH RIGHT DESCENDER) */
  ["\u049A", "\u049B"], /* CYRILLIC SMALL LETTER KA WITH DESCENDER (CYRILLIC SMALL LETTER KA WITH RIGHT DESCENDER) */
  ["\u049C", "\u049D"], /* CYRILLIC CAPITAL LETTER KA WITH VERTICAL STROKE (CYRILLIC CAPITAL LETTER KA VERTICAL BAR) */
  ["\u049C", "\u049D"], /* CYRILLIC SMALL LETTER KA WITH VERTICAL STROKE (CYRILLIC SMALL LETTER KA VERTICAL BAR) */
  ["\u049E", "\u049F"], /* CYRILLIC CAPITAL LETTER KA WITH STROKE (CYRILLIC CAPITAL LETTER KA BAR) */
  ["\u049E", "\u049F"], /* CYRILLIC SMALL LETTER KA WITH STROKE (CYRILLIC SMALL LETTER KA BAR) */
  ["\u04A0", "\u04A1"], /* CYRILLIC CAPITAL LETTER BASHKIR KA (CYRILLIC CAPITAL LETTER REVERSED GE KA) */
  ["\u04A0", "\u04A1"], /* CYRILLIC SMALL LETTER BASHKIR KA (CYRILLIC SMALL LETTER REVERSED GE KA) */
  ["\u04A2", "\u04A3"], /* CYRILLIC CAPITAL LETTER EN WITH DESCENDER (CYRILLIC CAPITAL LETTER EN WITH RIGHT DESCENDER) */
  ["\u04A2", "\u04A3"], /* CYRILLIC SMALL LETTER EN WITH DESCENDER (CYRILLIC SMALL LETTER EN WITH RIGHT DESCENDER) */
  ["\u04A4", "\u04A5"], /* CYRILLIC CAPITAL LIGATURE EN GHE (CYRILLIC CAPITAL LETTER EN GE) */
  ["\u04A4", "\u04A5"], /* CYRILLIC SMALL LIGATURE EN GHE (CYRILLIC SMALL LETTER EN GE) */
  ["\u04A6", "\u04A7"], /* CYRILLIC CAPITAL LETTER PE WITH MIDDLE HOOK (CYRILLIC CAPITAL LETTER PE HOOK) */
  ["\u04A6", "\u04A7"], /* CYRILLIC SMALL LETTER PE WITH MIDDLE HOOK (CYRILLIC SMALL LETTER PE HOOK) */
  ["\u04A8", "\u04A9"], /* CYRILLIC CAPITAL LETTER ABKHASIAN HA (CYRILLIC CAPITAL LETTER O HOOK) */
  ["\u04A8", "\u04A9"], /* CYRILLIC SMALL LETTER ABKHASIAN HA (CYRILLIC SMALL LETTER O HOOK) */
  ["\u04AA", "\u04AB"], /* CYRILLIC CAPITAL LETTER ES WITH DESCENDER (CYRILLIC CAPITAL LETTER ES CEDILLA) */
  ["\u04AA", "\u04AB"], /* CYRILLIC SMALL LETTER ES WITH DESCENDER (CYRILLIC SMALL LETTER ES CEDILLA) */
  ["\u04AC", "\u04AD"], /* CYRILLIC CAPITAL LETTER TE WITH DESCENDER (CYRILLIC CAPITAL LETTER TE WITH RIGHT DESCENDER) */
  ["\u04AC", "\u04AD"], /* CYRILLIC SMALL LETTER TE WITH DESCENDER (CYRILLIC SMALL LETTER TE WITH RIGHT DESCENDER) */
  ["\u04AE", "\u04AF"], /* CYRILLIC CAPITAL LETTER STRAIGHT U */
  ["\u04AE", "\u04AF"], /* CYRILLIC SMALL LETTER STRAIGHT U */
  ["\u04B0", "\u04B1"], /* CYRILLIC CAPITAL LETTER STRAIGHT U WITH STROKE (CYRILLIC CAPITAL LETTER STRAIGHT U BAR) */
  ["\u04B0", "\u04B1"], /* CYRILLIC SMALL LETTER STRAIGHT U WITH STROKE (CYRILLIC SMALL LETTER STRAIGHT U BAR) */
  ["\u04B2", "\u04B3"], /* CYRILLIC CAPITAL LETTER HA WITH DESCENDER (CYRILLIC CAPITAL LETTER KHA WITH RIGHT DESCENDER) */
  ["\u04B2", "\u04B3"], /* CYRILLIC SMALL LETTER HA WITH DESCENDER (CYRILLIC SMALL LETTER KHA WITH RIGHT DESCENDER) */
  ["\u04B4", "\u04B5"], /* CYRILLIC CAPITAL LIGATURE TE TSE (CYRILLIC CAPITAL LETTER TE TSE) */
  ["\u04B4", "\u04B5"], /* CYRILLIC SMALL LIGATURE TE TSE (CYRILLIC SMALL LETTER TE TSE) */
  ["\u04B6", "\u04B7"], /* CYRILLIC CAPITAL LETTER CHE WITH DESCENDER (CYRILLIC CAPITAL LETTER CHE WITH RIGHT DESCENDER) */
  ["\u04B6", "\u04B7"], /* CYRILLIC SMALL LETTER CHE WITH DESCENDER (CYRILLIC SMALL LETTER CHE WITH RIGHT DESCENDER) */
  ["\u04B8", "\u04B9"], /* CYRILLIC CAPITAL LETTER CHE WITH VERTICAL STROKE (CYRILLIC CAPITAL LETTER CHE VERTICAL BAR) */
  ["\u04B8", "\u04B9"], /* CYRILLIC SMALL LETTER CHE WITH VERTICAL STROKE (CYRILLIC SMALL LETTER CHE VERTICAL BAR) */
  ["\u04BA", "\u04BB"], /* CYRILLIC CAPITAL LETTER SHHA (CYRILLIC CAPITAL LETTER H) */
  ["\u04BA", "\u04BB"], /* CYRILLIC SMALL LETTER SHHA (CYRILLIC SMALL LETTER H) */
  ["\u04BC", "\u04BD"], /* CYRILLIC CAPITAL LETTER ABKHASIAN CHE (CYRILLIC CAPITAL LETTER IE HOOK) */
  ["\u04BC", "\u04BD"], /* CYRILLIC SMALL LETTER ABKHASIAN CHE (CYRILLIC SMALL LETTER IE HOOK) */
  ["\u04BE", "\u04BF"], /* CYRILLIC CAPITAL LETTER ABKHASIAN CHE WITH DESCENDER (CYRILLIC CAPITAL LETTER IE HOOK OGONEK) */
  ["\u04BE", "\u04BF"], /* CYRILLIC SMALL LETTER ABKHASIAN CHE WITH DESCENDER (CYRILLIC SMALL LETTER IE HOOK OGONEK) */
  ["\u04C0", "\u04CF"], /* CYRILLIC LETTER PALOCHKA (CYRILLIC LETTER I) */
  ["\u04C1", "\u04C2"], /* CYRILLIC CAPITAL LETTER ZHE WITH BREVE (CYRILLIC CAPITAL LETTER SHORT ZHE) */
  ["\u04C1", "\u04C2"], /* CYRILLIC SMALL LETTER ZHE WITH BREVE (CYRILLIC SMALL LETTER SHORT ZHE) */
  ["\u04C3", "\u04C4"], /* CYRILLIC CAPITAL LETTER KA WITH HOOK (CYRILLIC CAPITAL LETTER KA HOOK) */
  ["\u04C3", "\u04C4"], /* CYRILLIC SMALL LETTER KA WITH HOOK (CYRILLIC SMALL LETTER KA HOOK) */
  ["\u04C5", "\u04C6"], /* CYRILLIC CAPITAL LETTER EL WITH TAIL */
  ["\u04C5", "\u04C6"], /* CYRILLIC SMALL LETTER EL WITH TAIL */
  ["\u04C7", "\u04C8"], /* CYRILLIC CAPITAL LETTER EN WITH HOOK (CYRILLIC CAPITAL LETTER EN HOOK) */
  ["\u04C7", "\u04C8"], /* CYRILLIC SMALL LETTER EN WITH HOOK (CYRILLIC SMALL LETTER EN HOOK) */
  ["\u04C9", "\u04CA"], /* CYRILLIC CAPITAL LETTER EN WITH TAIL */
  ["\u04C9", "\u04CA"], /* CYRILLIC SMALL LETTER EN WITH TAIL */
  ["\u04CB", "\u04CC"], /* CYRILLIC CAPITAL LETTER KHAKASSIAN CHE (CYRILLIC CAPITAL LETTER CHE WITH LEFT DESCENDER) */
  ["\u04CB", "\u04CC"], /* CYRILLIC SMALL LETTER KHAKASSIAN CHE (CYRILLIC SMALL LETTER CHE WITH LEFT DESCENDER) */
  ["\u04CD", "\u04CE"], /* CYRILLIC CAPITAL LETTER EM WITH TAIL */
  ["\u04CD", "\u04CE"], /* CYRILLIC SMALL LETTER EM WITH TAIL */
  ["\u04C0", "\u04CF"], /* CYRILLIC SMALL LETTER PALOCHKA */
  ["\u04D0", "\u04D1"], /* CYRILLIC CAPITAL LETTER A WITH BREVE */
  ["\u04D0", "\u04D1"], /* CYRILLIC SMALL LETTER A WITH BREVE */
  ["\u04D2", "\u04D3"], /* CYRILLIC CAPITAL LETTER A WITH DIAERESIS */
  ["\u04D2", "\u04D3"], /* CYRILLIC SMALL LETTER A WITH DIAERESIS */
  ["\u04D4", "\u04D5"], /* CYRILLIC CAPITAL LIGATURE A IE */
  ["\u04D4", "\u04D5"], /* CYRILLIC SMALL LIGATURE A IE */
  ["\u04D6", "\u04D7"], /* CYRILLIC CAPITAL LETTER IE WITH BREVE */
  ["\u04D6", "\u04D7"], /* CYRILLIC SMALL LETTER IE WITH BREVE */
  ["\u04D8", "\u04D9"], /* CYRILLIC CAPITAL LETTER SCHWA */
  ["\u04D8", "\u04D9"], /* CYRILLIC SMALL LETTER SCHWA */
  ["\u04DA", "\u04DB"], /* CYRILLIC CAPITAL LETTER SCHWA WITH DIAERESIS */
  ["\u04DA", "\u04DB"], /* CYRILLIC SMALL LETTER SCHWA WITH DIAERESIS */
  ["\u04DC", "\u04DD"], /* CYRILLIC CAPITAL LETTER ZHE WITH DIAERESIS */
  ["\u04DC", "\u04DD"], /* CYRILLIC SMALL LETTER ZHE WITH DIAERESIS */
  ["\u04DE", "\u04DF"], /* CYRILLIC CAPITAL LETTER ZE WITH DIAERESIS */
  ["\u04DE", "\u04DF"], /* CYRILLIC SMALL LETTER ZE WITH DIAERESIS */
  ["\u04E0", "\u04E1"], /* CYRILLIC CAPITAL LETTER ABKHASIAN DZE */
  ["\u04E0", "\u04E1"], /* CYRILLIC SMALL LETTER ABKHASIAN DZE */
  ["\u04E2", "\u04E3"], /* CYRILLIC CAPITAL LETTER I WITH MACRON */
  ["\u04E2", "\u04E3"], /* CYRILLIC SMALL LETTER I WITH MACRON */
  ["\u04E4", "\u04E5"], /* CYRILLIC CAPITAL LETTER I WITH DIAERESIS */
  ["\u04E4", "\u04E5"], /* CYRILLIC SMALL LETTER I WITH DIAERESIS */
  ["\u04E6", "\u04E7"], /* CYRILLIC CAPITAL LETTER O WITH DIAERESIS */
  ["\u04E6", "\u04E7"], /* CYRILLIC SMALL LETTER O WITH DIAERESIS */
  ["\u04E8", "\u04E9"], /* CYRILLIC CAPITAL LETTER BARRED O */
  ["\u04E8", "\u04E9"], /* CYRILLIC SMALL LETTER BARRED O */
  ["\u04EA", "\u04EB"], /* CYRILLIC CAPITAL LETTER BARRED O WITH DIAERESIS */
  ["\u04EA", "\u04EB"], /* CYRILLIC SMALL LETTER BARRED O WITH DIAERESIS */
  ["\u04EC", "\u04ED"], /* CYRILLIC CAPITAL LETTER E WITH DIAERESIS */
  ["\u04EC", "\u04ED"], /* CYRILLIC SMALL LETTER E WITH DIAERESIS */
  ["\u04EE", "\u04EF"], /* CYRILLIC CAPITAL LETTER U WITH MACRON */
  ["\u04EE", "\u04EF"], /* CYRILLIC SMALL LETTER U WITH MACRON */
  ["\u04F0", "\u04F1"], /* CYRILLIC CAPITAL LETTER U WITH DIAERESIS */
  ["\u04F0", "\u04F1"], /* CYRILLIC SMALL LETTER U WITH DIAERESIS */
  ["\u04F2", "\u04F3"], /* CYRILLIC CAPITAL LETTER U WITH DOUBLE ACUTE */
  ["\u04F2", "\u04F3"], /* CYRILLIC SMALL LETTER U WITH DOUBLE ACUTE */
  ["\u04F4", "\u04F5"], /* CYRILLIC CAPITAL LETTER CHE WITH DIAERESIS */
  ["\u04F4", "\u04F5"], /* CYRILLIC SMALL LETTER CHE WITH DIAERESIS */
  ["\u04F6", "\u04F7"], /* CYRILLIC CAPITAL LETTER GHE WITH DESCENDER */
  ["\u04F6", "\u04F7"], /* CYRILLIC SMALL LETTER GHE WITH DESCENDER */
  ["\u04F8", "\u04F9"], /* CYRILLIC CAPITAL LETTER YERU WITH DIAERESIS */
  ["\u04F8", "\u04F9"], /* CYRILLIC SMALL LETTER YERU WITH DIAERESIS */
  ["\u04FA", "\u04FB"], /* CYRILLIC CAPITAL LETTER GHE WITH STROKE AND HOOK */
  ["\u04FA", "\u04FB"], /* CYRILLIC SMALL LETTER GHE WITH STROKE AND HOOK */
  ["\u04FC", "\u04FD"], /* CYRILLIC CAPITAL LETTER HA WITH HOOK */
  ["\u04FC", "\u04FD"], /* CYRILLIC SMALL LETTER HA WITH HOOK */
  ["\u04FE", "\u04FF"], /* CYRILLIC CAPITAL LETTER HA WITH STROKE */
  ["\u04FE", "\u04FF"], /* CYRILLIC SMALL LETTER HA WITH STROKE */
  ["\u0500", "\u0501"], /* CYRILLIC CAPITAL LETTER KOMI DE */
  ["\u0500", "\u0501"], /* CYRILLIC SMALL LETTER KOMI DE */
  ["\u0502", "\u0503"], /* CYRILLIC CAPITAL LETTER KOMI DJE */
  ["\u0502", "\u0503"], /* CYRILLIC SMALL LETTER KOMI DJE */
  ["\u0504", "\u0505"], /* CYRILLIC CAPITAL LETTER KOMI ZJE */
  ["\u0504", "\u0505"], /* CYRILLIC SMALL LETTER KOMI ZJE */
  ["\u0506", "\u0507"], /* CYRILLIC CAPITAL LETTER KOMI DZJE */
  ["\u0506", "\u0507"], /* CYRILLIC SMALL LETTER KOMI DZJE */
  ["\u0508", "\u0509"], /* CYRILLIC CAPITAL LETTER KOMI LJE */
  ["\u0508", "\u0509"], /* CYRILLIC SMALL LETTER KOMI LJE */
  ["\u050A", "\u050B"], /* CYRILLIC CAPITAL LETTER KOMI NJE */
  ["\u050A", "\u050B"], /* CYRILLIC SMALL LETTER KOMI NJE */
  ["\u050C", "\u050D"], /* CYRILLIC CAPITAL LETTER KOMI SJE */
  ["\u050C", "\u050D"], /* CYRILLIC SMALL LETTER KOMI SJE */
  ["\u050E", "\u050F"], /* CYRILLIC CAPITAL LETTER KOMI TJE */
  ["\u050E", "\u050F"], /* CYRILLIC SMALL LETTER KOMI TJE */
  ["\u0510", "\u0511"], /* CYRILLIC CAPITAL LETTER REVERSED ZE */
  ["\u0510", "\u0511"], /* CYRILLIC SMALL LETTER REVERSED ZE */
  ["\u0512", "\u0513"], /* CYRILLIC CAPITAL LETTER EL WITH HOOK */
  ["\u0512", "\u0513"], /* CYRILLIC SMALL LETTER EL WITH HOOK */
  ["\u0514", "\u0515"], /* CYRILLIC CAPITAL LETTER LHA */
  ["\u0514", "\u0515"], /* CYRILLIC SMALL LETTER LHA */
  ["\u0516", "\u0517"], /* CYRILLIC CAPITAL LETTER RHA */
  ["\u0516", "\u0517"], /* CYRILLIC SMALL LETTER RHA */
  ["\u0518", "\u0519"], /* CYRILLIC CAPITAL LETTER YAE */
  ["\u0518", "\u0519"], /* CYRILLIC SMALL LETTER YAE */
  ["\u051A", "\u051B"], /* CYRILLIC CAPITAL LETTER QA */
  ["\u051A", "\u051B"], /* CYRILLIC SMALL LETTER QA */
  ["\u051C", "\u051D"], /* CYRILLIC CAPITAL LETTER WE */
  ["\u051C", "\u051D"], /* CYRILLIC SMALL LETTER WE */
  ["\u051E", "\u051F"], /* CYRILLIC CAPITAL LETTER ALEUT KA */
  ["\u051E", "\u051F"], /* CYRILLIC SMALL LETTER ALEUT KA */
  ["\u0520", "\u0521"], /* CYRILLIC CAPITAL LETTER EL WITH MIDDLE HOOK */
  ["\u0520", "\u0521"], /* CYRILLIC SMALL LETTER EL WITH MIDDLE HOOK */
  ["\u0522", "\u0523"], /* CYRILLIC CAPITAL LETTER EN WITH MIDDLE HOOK */
  ["\u0522", "\u0523"], /* CYRILLIC SMALL LETTER EN WITH MIDDLE HOOK */
  ["\u0524", "\u0525"], /* CYRILLIC CAPITAL LETTER PE WITH DESCENDER */
  ["\u0524", "\u0525"], /* CYRILLIC SMALL LETTER PE WITH DESCENDER */
  ["\u0526", "\u0527"], /* CYRILLIC CAPITAL LETTER SHHA WITH DESCENDER */
  ["\u0526", "\u0527"], /* CYRILLIC SMALL LETTER SHHA WITH DESCENDER */
  ["\u0528", "\u0529"], /* CYRILLIC CAPITAL LETTER EN WITH LEFT HOOK */
  ["\u0528", "\u0529"], /* CYRILLIC SMALL LETTER EN WITH LEFT HOOK */
  ["\u052A", "\u052B"], /* CYRILLIC CAPITAL LETTER DZZHE */
  ["\u052A", "\u052B"], /* CYRILLIC SMALL LETTER DZZHE */
  ["\u052C", "\u052D"], /* CYRILLIC CAPITAL LETTER DCHE */
  ["\u052C", "\u052D"], /* CYRILLIC SMALL LETTER DCHE */
  ["\u052E", "\u052F"], /* CYRILLIC CAPITAL LETTER EL WITH DESCENDER */
  ["\u052E", "\u052F"], /* CYRILLIC SMALL LETTER EL WITH DESCENDER */
  ["\u0530", "\u0530"],
  ["\u0531", "\u0561"], /* ARMENIAN CAPITAL LETTER AYB */
  ["\u0532", "\u0562"], /* ARMENIAN CAPITAL LETTER BEN */
  ["\u0533", "\u0563"], /* ARMENIAN CAPITAL LETTER GIM */
  ["\u0534", "\u0564"], /* ARMENIAN CAPITAL LETTER DA */
  ["\u0535", "\u0565"], /* ARMENIAN CAPITAL LETTER ECH */
  ["\u0536", "\u0566"], /* ARMENIAN CAPITAL LETTER ZA */
  ["\u0537", "\u0567"], /* ARMENIAN CAPITAL LETTER EH */
  ["\u0538", "\u0568"], /* ARMENIAN CAPITAL LETTER ET */
  ["\u0539", "\u0569"], /* ARMENIAN CAPITAL LETTER TO */
  ["\u053A", "\u056A"], /* ARMENIAN CAPITAL LETTER ZHE */
  ["\u053B", "\u056B"], /* ARMENIAN CAPITAL LETTER INI */
  ["\u053C", "\u056C"], /* ARMENIAN CAPITAL LETTER LIWN */
  ["\u053D", "\u056D"], /* ARMENIAN CAPITAL LETTER XEH */
  ["\u053E", "\u056E"], /* ARMENIAN CAPITAL LETTER CA */
  ["\u053F", "\u056F"], /* ARMENIAN CAPITAL LETTER KEN */
  ["\u0540", "\u0570"], /* ARMENIAN CAPITAL LETTER HO */
  ["\u0541", "\u0571"], /* ARMENIAN CAPITAL LETTER JA */
  ["\u0542", "\u0572"], /* ARMENIAN CAPITAL LETTER GHAD (ARMENIAN CAPITAL LETTER LAD) */
  ["\u0543", "\u0573"], /* ARMENIAN CAPITAL LETTER CHEH */
  ["\u0544", "\u0574"], /* ARMENIAN CAPITAL LETTER MEN */
  ["\u0545", "\u0575"], /* ARMENIAN CAPITAL LETTER YI */
  ["\u0546", "\u0576"], /* ARMENIAN CAPITAL LETTER NOW */
  ["\u0547", "\u0577"], /* ARMENIAN CAPITAL LETTER SHA */
  ["\u0548", "\u0578"], /* ARMENIAN CAPITAL LETTER VO */
  ["\u0549", "\u0579"], /* ARMENIAN CAPITAL LETTER CHA */
  ["\u054A", "\u057A"], /* ARMENIAN CAPITAL LETTER PEH */
  ["\u054B", "\u057B"], /* ARMENIAN CAPITAL LETTER JHEH */
  ["\u054C", "\u057C"], /* ARMENIAN CAPITAL LETTER RA */
  ["\u054D", "\u057D"], /* ARMENIAN CAPITAL LETTER SEH */
  ["\u054E", "\u057E"], /* ARMENIAN CAPITAL LETTER VEW */
  ["\u054F", "\u057F"], /* ARMENIAN CAPITAL LETTER TIWN */
  ["\u0550", "\u0580"], /* ARMENIAN CAPITAL LETTER REH */
  ["\u0551", "\u0581"], /* ARMENIAN CAPITAL LETTER CO */
  ["\u0552", "\u0582"], /* ARMENIAN CAPITAL LETTER YIWN */
  ["\u0553", "\u0583"], /* ARMENIAN CAPITAL LETTER PIWR */
  ["\u0554", "\u0584"], /* ARMENIAN CAPITAL LETTER KEH */
  ["\u0555", "\u0585"], /* ARMENIAN CAPITAL LETTER OH */
  ["\u0556", "\u0586"], /* ARMENIAN CAPITAL LETTER FEH */
  ["\u0557", "\u0557"],
  ["\u0558", "\u0558"],
  ["\u0559", "\u0559"], /* ARMENIAN MODIFIER LETTER LEFT HALF RING */
  ["\u055A", "\u055A"], /* ARMENIAN APOSTROPHE (ARMENIAN MODIFIER LETTER RIGHT HALF RING) */
  ["\u055B", "\u055B"], /* ARMENIAN EMPHASIS MARK */
  ["\u055C", "\u055C"], /* ARMENIAN EXCLAMATION MARK */
  ["\u055D", "\u055D"], /* ARMENIAN COMMA */
  ["\u055E", "\u055E"], /* ARMENIAN QUESTION MARK */
  ["\u055F", "\u055F"], /* ARMENIAN ABBREVIATION MARK */
  ["\u0560", "\u0560"], /* ARMENIAN SMALL LETTER TURNED AYB */
  ["\u0531", "\u0561"], /* ARMENIAN SMALL LETTER AYB */
  ["\u0532", "\u0562"], /* ARMENIAN SMALL LETTER BEN */
  ["\u0533", "\u0563"], /* ARMENIAN SMALL LETTER GIM */
  ["\u0534", "\u0564"], /* ARMENIAN SMALL LETTER DA */
  ["\u0535", "\u0565"], /* ARMENIAN SMALL LETTER ECH */
  ["\u0536", "\u0566"], /* ARMENIAN SMALL LETTER ZA */
  ["\u0537", "\u0567"], /* ARMENIAN SMALL LETTER EH */
  ["\u0538", "\u0568"], /* ARMENIAN SMALL LETTER ET */
  ["\u0539", "\u0569"], /* ARMENIAN SMALL LETTER TO */
  ["\u053A", "\u056A"], /* ARMENIAN SMALL LETTER ZHE */
  ["\u053B", "\u056B"], /* ARMENIAN SMALL LETTER INI */
  ["\u053C", "\u056C"], /* ARMENIAN SMALL LETTER LIWN */
  ["\u053D", "\u056D"], /* ARMENIAN SMALL LETTER XEH */
  ["\u053E", "\u056E"], /* ARMENIAN SMALL LETTER CA */
  ["\u053F", "\u056F"], /* ARMENIAN SMALL LETTER KEN */
  ["\u0540", "\u0570"], /* ARMENIAN SMALL LETTER HO */
  ["\u0541", "\u0571"], /* ARMENIAN SMALL LETTER JA */
  ["\u0542", "\u0572"], /* ARMENIAN SMALL LETTER GHAD (ARMENIAN SMALL LETTER LAD) */
  ["\u0543", "\u0573"], /* ARMENIAN SMALL LETTER CHEH */
  ["\u0544", "\u0574"], /* ARMENIAN SMALL LETTER MEN */
  ["\u0545", "\u0575"], /* ARMENIAN SMALL LETTER YI */
  ["\u0546", "\u0576"], /* ARMENIAN SMALL LETTER NOW */
  ["\u0547", "\u0577"], /* ARMENIAN SMALL LETTER SHA */
  ["\u0548", "\u0578"], /* ARMENIAN SMALL LETTER VO */
  ["\u0549", "\u0579"], /* ARMENIAN SMALL LETTER CHA */
  ["\u054A", "\u057A"], /* ARMENIAN SMALL LETTER PEH */
  ["\u054B", "\u057B"], /* ARMENIAN SMALL LETTER JHEH */
  ["\u054C", "\u057C"], /* ARMENIAN SMALL LETTER RA */
  ["\u054D", "\u057D"], /* ARMENIAN SMALL LETTER SEH */
  ["\u054E", "\u057E"], /* ARMENIAN SMALL LETTER VEW */
  ["\u054F", "\u057F"], /* ARMENIAN SMALL LETTER TIWN */
  ["\u0550", "\u0580"], /* ARMENIAN SMALL LETTER REH */
  ["\u0551", "\u0581"], /* ARMENIAN SMALL LETTER CO */
  ["\u0552", "\u0582"], /* ARMENIAN SMALL LETTER YIWN */
  ["\u0553", "\u0583"], /* ARMENIAN SMALL LETTER PIWR */
  ["\u0554", "\u0584"], /* ARMENIAN SMALL LETTER KEH */
  ["\u0555", "\u0585"], /* ARMENIAN SMALL LETTER OH */
  ["\u0556", "\u0586"], /* ARMENIAN SMALL LETTER FEH */
  ["\u0535\u0552", "\u0587"], /* ARMENIAN SMALL LIGATURE ECH YIWN */
  ["\u0588", "\u0588"], /* ARMENIAN SMALL LETTER YI WITH STROKE */
  ["\u0589", "\u0589"], /* ARMENIAN FULL STOP (ARMENIAN PERIOD) */
  ["\u058A", "\u058A"], /* ARMENIAN HYPHEN */
  ["\u058B", "\u058B"],
  ["\u058C", "\u058C"],
  ["\u058D", "\u058D"], /* RIGHT-FACING ARMENIAN ETERNITY SIGN */
  ["\u058E", "\u058E"], /* LEFT-FACING ARMENIAN ETERNITY SIGN */
  ["\u058F", "\u058F"], /* ARMENIAN DRAM SIGN */
  ["\u0590", "\u0590"],
  ["\u0591", "\u0591"], /* HEBREW ACCENT ETNAHTA */
  ["\u0592", "\u0592"], /* HEBREW ACCENT SEGOL */
  ["\u0593", "\u0593"], /* HEBREW ACCENT SHALSHELET */
  ["\u0594", "\u0594"], /* HEBREW ACCENT ZAQEF QATAN */
  ["\u0595", "\u0595"], /* HEBREW ACCENT ZAQEF GADOL */
  ["\u0596", "\u0596"], /* HEBREW ACCENT TIPEHA */
  ["\u0597", "\u0597"], /* HEBREW ACCENT REVIA */
  ["\u0598", "\u0598"], /* HEBREW ACCENT ZARQA */
  ["\u0599", "\u0599"], /* HEBREW ACCENT PASHTA */
  ["\u059A", "\u059A"], /* HEBREW ACCENT YETIV */
  ["\u059B", "\u059B"], /* HEBREW ACCENT TEVIR */
  ["\u059C", "\u059C"], /* HEBREW ACCENT GERESH */
  ["\u059D", "\u059D"], /* HEBREW ACCENT GERESH MUQDAM */
  ["\u059E", "\u059E"], /* HEBREW ACCENT GERSHAYIM */
  ["\u059F", "\u059F"], /* HEBREW ACCENT QARNEY PARA */
  ["\u05A0", "\u05A0"], /* HEBREW ACCENT TELISHA GEDOLA */
  ["\u05A1", "\u05A1"], /* HEBREW ACCENT PAZER */
  ["\u05A2", "\u05A2"], /* HEBREW ACCENT ATNAH HAFUKH */
  ["\u05A3", "\u05A3"], /* HEBREW ACCENT MUNAH */
  ["\u05A4", "\u05A4"], /* HEBREW ACCENT MAHAPAKH */
  ["\u05A5", "\u05A5"], /* HEBREW ACCENT MERKHA */
  ["\u05A6", "\u05A6"], /* HEBREW ACCENT MERKHA KEFULA */
  ["\u05A7", "\u05A7"], /* HEBREW ACCENT DARGA */
  ["\u05A8", "\u05A8"], /* HEBREW ACCENT QADMA */
  ["\u05A9", "\u05A9"], /* HEBREW ACCENT TELISHA QETANA */
  ["\u05AA", "\u05AA"], /* HEBREW ACCENT YERAH BEN YOMO */
  ["\u05AB", "\u05AB"], /* HEBREW ACCENT OLE */
  ["\u05AC", "\u05AC"], /* HEBREW ACCENT ILUY */
  ["\u05AD", "\u05AD"], /* HEBREW ACCENT DEHI */
  ["\u05AE", "\u05AE"], /* HEBREW ACCENT ZINOR */
  ["\u05AF", "\u05AF"], /* HEBREW MARK MASORA CIRCLE */
  ["\u05B0", "\u05B0"], /* HEBREW POINT SHEVA */
  ["\u05B1", "\u05B1"], /* HEBREW POINT HATAF SEGOL */
  ["\u05B2", "\u05B2"], /* HEBREW POINT HATAF PATAH */
  ["\u05B3", "\u05B3"], /* HEBREW POINT HATAF QAMATS */
  ["\u05B4", "\u05B4"], /* HEBREW POINT HIRIQ */
  ["\u05B5", "\u05B5"], /* HEBREW POINT TSERE */
  ["\u05B6", "\u05B6"], /* HEBREW POINT SEGOL */
  ["\u05B7", "\u05B7"], /* HEBREW POINT PATAH */
  ["\u05B8", "\u05B8"], /* HEBREW POINT QAMATS */
  ["\u05B9", "\u05B9"], /* HEBREW POINT HOLAM */
  ["\u05BA", "\u05BA"], /* HEBREW POINT HOLAM HASER FOR VAV */
  ["\u05BB", "\u05BB"], /* HEBREW POINT QUBUTS */
  ["\u05BC", "\u05BC"], /* HEBREW POINT DAGESH OR MAPIQ (HEBREW POINT DAGESH) */
  ["\u05BD", "\u05BD"], /* HEBREW POINT METEG */
  ["\u05BE", "\u05BE"], /* HEBREW PUNCTUATION MAQAF */
  ["\u05BF", "\u05BF"], /* HEBREW POINT RAFE */
  ["\u05C0", "\u05C0"], /* HEBREW PUNCTUATION PASEQ (HEBREW POINT PASEQ) */
  ["\u05C1", "\u05C1"], /* HEBREW POINT SHIN DOT */
  ["\u05C2", "\u05C2"], /* HEBREW POINT SIN DOT */
  ["\u05C3", "\u05C3"], /* HEBREW PUNCTUATION SOF PASUQ */
  ["\u05C4", "\u05C4"], /* HEBREW MARK UPPER DOT */
  ["\u05C5", "\u05C5"], /* HEBREW MARK LOWER DOT */
  ["\u05C6", "\u05C6"], /* HEBREW PUNCTUATION NUN HAFUKHA */
  ["\u05C7", "\u05C7"], /* HEBREW POINT QAMATS QATAN */
  ["\u05C8", "\u05C8"],
  ["\u05C9", "\u05C9"],
  ["\u05CA", "\u05CA"],
  ["\u05CB", "\u05CB"],
  ["\u05CC", "\u05CC"],
  ["\u05CD", "\u05CD"],
  ["\u05CE", "\u05CE"],
  ["\u05CF", "\u05CF"],
  ["\u05D0", "\u05D0"], /* HEBREW LETTER ALEF */
  ["\u05D1", "\u05D1"], /* HEBREW LETTER BET */
  ["\u05D2", "\u05D2"], /* HEBREW LETTER GIMEL */
  ["\u05D3", "\u05D3"], /* HEBREW LETTER DALET */
  ["\u05D4", "\u05D4"], /* HEBREW LETTER HE */
  ["\u05D5", "\u05D5"], /* HEBREW LETTER VAV */
  ["\u05D6", "\u05D6"], /* HEBREW LETTER ZAYIN */
  ["\u05D7", "\u05D7"], /* HEBREW LETTER HET */
  ["\u05D8", "\u05D8"], /* HEBREW LETTER TET */
  ["\u05D9", "\u05D9"], /* HEBREW LETTER YOD */
  ["\u05DA", "\u05DA"], /* HEBREW LETTER FINAL KAF */
  ["\u05DB", "\u05DB"], /* HEBREW LETTER KAF */
  ["\u05DC", "\u05DC"], /* HEBREW LETTER LAMED */
  ["\u05DD", "\u05DD"], /* HEBREW LETTER FINAL MEM */
  ["\u05DE", "\u05DE"], /* HEBREW LETTER MEM */
  ["\u05DF", "\u05DF"], /* HEBREW LETTER FINAL NUN */
  ["\u05E0", "\u05E0"], /* HEBREW LETTER NUN */
  ["\u05E1", "\u05E1"], /* HEBREW LETTER SAMEKH */
  ["\u05E2", "\u05E2"], /* HEBREW LETTER AYIN */
  ["\u05E3", "\u05E3"], /* HEBREW LETTER FINAL PE */
  ["\u05E4", "\u05E4"], /* HEBREW LETTER PE */
  ["\u05E5", "\u05E5"], /* HEBREW LETTER FINAL TSADI */
  ["\u05E6", "\u05E6"], /* HEBREW LETTER TSADI */
  ["\u05E7", "\u05E7"], /* HEBREW LETTER QOF */
  ["\u05E8", "\u05E8"], /* HEBREW LETTER RESH */
  ["\u05E9", "\u05E9"], /* HEBREW LETTER SHIN */
  ["\u05EA", "\u05EA"], /* HEBREW LETTER TAV */
  ["\u05EB", "\u05EB"],
  ["\u05EC", "\u05EC"],
  ["\u05ED", "\u05ED"],
  ["\u05EE", "\u05EE"],
  ["\u05EF", "\u05EF"], /* HEBREW YOD TRIANGLE */
  ["\u05F0", "\u05F0"], /* HEBREW LIGATURE YIDDISH DOUBLE VAV (HEBREW LETTER DOUBLE VAV) */
  ["\u05F1", "\u05F1"], /* HEBREW LIGATURE YIDDISH VAV YOD (HEBREW LETTER VAV YOD) */
  ["\u05F2", "\u05F2"], /* HEBREW LIGATURE YIDDISH DOUBLE YOD (HEBREW LETTER DOUBLE YOD) */
  ["\u05F3", "\u05F3"], /* HEBREW PUNCTUATION GERESH */
  ["\u05F4", "\u05F4"], /* HEBREW PUNCTUATION GERSHAYIM */
  ["\u05F5", "\u05F5"],
  ["\u05F6", "\u05F6"],
  ["\u05F7", "\u05F7"],
  ["\u05F8", "\u05F8"],
  ["\u05F9", "\u05F9"],
  ["\u05FA", "\u05FA"],
  ["\u05FB", "\u05FB"],
  ["\u05FC", "\u05FC"],
  ["\u05FD", "\u05FD"],
  ["\u05FE", "\u05FE"],
  ["\u05FF", "\u05FF"],
  ["\u0600", "\u0600"], /* ARABIC NUMBER SIGN */
  ["\u0601", "\u0601"], /* ARABIC SIGN SANAH */
  ["\u0602", "\u0602"], /* ARABIC FOOTNOTE MARKER */
  ["\u0603", "\u0603"], /* ARABIC SIGN SAFHA */
  ["\u0604", "\u0604"], /* ARABIC SIGN SAMVAT */
  ["\u0605", "\u0605"], /* ARABIC NUMBER MARK ABOVE */
  ["\u0606", "\u0606"], /* ARABIC-INDIC CUBE ROOT */
  ["\u0607", "\u0607"], /* ARABIC-INDIC FOURTH ROOT */
  ["\u0608", "\u0608"], /* ARABIC RAY */
  ["\u0609", "\u0609"], /* ARABIC-INDIC PER MILLE SIGN */
  ["\u060A", "\u060A"], /* ARABIC-INDIC PER TEN THOUSAND SIGN */
  ["\u060B", "\u060B"], /* AFGHANI SIGN */
  ["\u060C", "\u060C"], /* ARABIC COMMA */
  ["\u060D", "\u060D"], /* ARABIC DATE SEPARATOR */
  ["\u060E", "\u060E"], /* ARABIC POETIC VERSE SIGN */
  ["\u060F", "\u060F"], /* ARABIC SIGN MISRA */
  ["\u0610", "\u0610"], /* ARABIC SIGN SALLALLAHOU ALAYHE WASSALLAM */
  ["\u0611", "\u0611"], /* ARABIC SIGN ALAYHE ASSALLAM */
  ["\u0612", "\u0612"], /* ARABIC SIGN RAHMATULLAH ALAYHE */
  ["\u0613", "\u0613"], /* ARABIC SIGN RADI ALLAHOU ANHU */
  ["\u0614", "\u0614"], /* ARABIC SIGN TAKHALLUS */
  ["\u0615", "\u0615"], /* ARABIC SMALL HIGH TAH */
  ["\u0616", "\u0616"], /* ARABIC SMALL HIGH LIGATURE ALEF WITH LAM WITH YEH */
  ["\u0617", "\u0617"], /* ARABIC SMALL HIGH ZAIN */
  ["\u0618", "\u0618"], /* ARABIC SMALL FATHA */
  ["\u0619", "\u0619"], /* ARABIC SMALL DAMMA */
  ["\u061A", "\u061A"], /* ARABIC SMALL KASRA */
  ["\u061B", "\u061B"], /* ARABIC SEMICOLON */
  ["\u061C", "\u061C"], /* ARABIC LETTER MARK */
  ["\u061D", "\u061D"],
  ["\u061E", "\u061E"], /* ARABIC TRIPLE DOT PUNCTUATION MARK */
  ["\u061F", "\u061F"], /* ARABIC QUESTION MARK */
  ["\u0620", "\u0620"], /* ARABIC LETTER KASHMIRI YEH */
  ["\u0621", "\u0621"], /* ARABIC LETTER HAMZA (ARABIC LETTER HAMZAH) */
  ["\u0622", "\u0622"], /* ARABIC LETTER ALEF WITH MADDA ABOVE (ARABIC LETTER MADDAH ON ALEF) */
  ["\u0623", "\u0623"], /* ARABIC LETTER ALEF WITH HAMZA ABOVE (ARABIC LETTER HAMZAH ON ALEF) */
  ["\u0624", "\u0624"], /* ARABIC LETTER WAW WITH HAMZA ABOVE (ARABIC LETTER HAMZAH ON WAW) */
  ["\u0625", "\u0625"], /* ARABIC LETTER ALEF WITH HAMZA BELOW (ARABIC LETTER HAMZAH UNDER ALEF) */
  ["\u0626", "\u0626"], /* ARABIC LETTER YEH WITH HAMZA ABOVE (ARABIC LETTER HAMZAH ON YA) */
  ["\u0627", "\u0627"], /* ARABIC LETTER ALEF */
  ["\u0628", "\u0628"], /* ARABIC LETTER BEH (ARABIC LETTER BAA) */
  ["\u0629", "\u0629"], /* ARABIC LETTER TEH MARBUTA (ARABIC LETTER TAA MARBUTAH) */
  ["\u062A", "\u062A"], /* ARABIC LETTER TEH (ARABIC LETTER TAA) */
  ["\u062B", "\u062B"], /* ARABIC LETTER THEH (ARABIC LETTER THAA) */
  ["\u062C", "\u062C"], /* ARABIC LETTER JEEM */
  ["\u062D", "\u062D"], /* ARABIC LETTER HAH (ARABIC LETTER HAA) */
  ["\u062E", "\u062E"], /* ARABIC LETTER KHAH (ARABIC LETTER KHAA) */
  ["\u062F", "\u062F"], /* ARABIC LETTER DAL */
  ["\u0630", "\u0630"], /* ARABIC LETTER THAL */
  ["\u0631", "\u0631"], /* ARABIC LETTER REH (ARABIC LETTER RA) */
  ["\u0632", "\u0632"], /* ARABIC LETTER ZAIN */
  ["\u0633", "\u0633"], /* ARABIC LETTER SEEN */
  ["\u0634", "\u0634"], /* ARABIC LETTER SHEEN */
  ["\u0635", "\u0635"], /* ARABIC LETTER SAD */
  ["\u0636", "\u0636"], /* ARABIC LETTER DAD */
  ["\u0637", "\u0637"], /* ARABIC LETTER TAH */
  ["\u0638", "\u0638"], /* ARABIC LETTER ZAH (ARABIC LETTER DHAH) */
  ["\u0639", "\u0639"], /* ARABIC LETTER AIN */
  ["\u063A", "\u063A"], /* ARABIC LETTER GHAIN */
  ["\u063B", "\u063B"], /* ARABIC LETTER KEHEH WITH TWO DOTS ABOVE */
  ["\u063C", "\u063C"], /* ARABIC LETTER KEHEH WITH THREE DOTS BELOW */
  ["\u063D", "\u063D"], /* ARABIC LETTER FARSI YEH WITH INVERTED V */
  ["\u063E", "\u063E"], /* ARABIC LETTER FARSI YEH WITH TWO DOTS ABOVE */
  ["\u063F", "\u063F"], /* ARABIC LETTER FARSI YEH WITH THREE DOTS ABOVE */
  ["\u0640", "\u0640"], /* ARABIC TATWEEL */
  ["\u0641", "\u0641"], /* ARABIC LETTER FEH (ARABIC LETTER FA) */
  ["\u0642", "\u0642"], /* ARABIC LETTER QAF */
  ["\u0643", "\u0643"], /* ARABIC LETTER KAF (ARABIC LETTER CAF) */
  ["\u0644", "\u0644"], /* ARABIC LETTER LAM */
  ["\u0645", "\u0645"], /* ARABIC LETTER MEEM */
  ["\u0646", "\u0646"], /* ARABIC LETTER NOON */
  ["\u0647", "\u0647"], /* ARABIC LETTER HEH (ARABIC LETTER HA) */
  ["\u0648", "\u0648"], /* ARABIC LETTER WAW */
  ["\u0649", "\u0649"], /* ARABIC LETTER ALEF MAKSURA (ARABIC LETTER ALEF MAQSURAH) */
  ["\u064A", "\u064A"], /* ARABIC LETTER YEH (ARABIC LETTER YA) */
  ["\u064B", "\u064B"], /* ARABIC FATHATAN */
  ["\u064C", "\u064C"], /* ARABIC DAMMATAN */
  ["\u064D", "\u064D"], /* ARABIC KASRATAN */
  ["\u064E", "\u064E"], /* ARABIC FATHA (ARABIC FATHAH) */
  ["\u064F", "\u064F"], /* ARABIC DAMMA (ARABIC DAMMAH) */
  ["\u0650", "\u0650"], /* ARABIC KASRA (ARABIC KASRAH) */
  ["\u0651", "\u0651"], /* ARABIC SHADDA (ARABIC SHADDAH) */
  ["\u0652", "\u0652"], /* ARABIC SUKUN */
  ["\u0653", "\u0653"], /* ARABIC MADDAH ABOVE */
  ["\u0654", "\u0654"], /* ARABIC HAMZA ABOVE */
  ["\u0655", "\u0655"], /* ARABIC HAMZA BELOW */
  ["\u0656", "\u0656"], /* ARABIC SUBSCRIPT ALEF */
  ["\u0657", "\u0657"], /* ARABIC INVERTED DAMMA */
  ["\u0658", "\u0658"], /* ARABIC MARK NOON GHUNNA */
  ["\u0659", "\u0659"], /* ARABIC ZWARAKAY */
  ["\u065A", "\u065A"], /* ARABIC VOWEL SIGN SMALL V ABOVE */
  ["\u065B", "\u065B"], /* ARABIC VOWEL SIGN INVERTED SMALL V ABOVE */
  ["\u065C", "\u065C"], /* ARABIC VOWEL SIGN DOT BELOW */
  ["\u065D", "\u065D"], /* ARABIC REVERSED DAMMA */
  ["\u065E", "\u065E"], /* ARABIC FATHA WITH TWO DOTS */
  ["\u065F", "\u065F"], /* ARABIC WAVY HAMZA BELOW */
  ["\u0660", "\u0660"], /* ARABIC-INDIC DIGIT ZERO */
  ["\u0661", "\u0661"], /* ARABIC-INDIC DIGIT ONE */
  ["\u0662", "\u0662"], /* ARABIC-INDIC DIGIT TWO */
  ["\u0663", "\u0663"], /* ARABIC-INDIC DIGIT THREE */
  ["\u0664", "\u0664"], /* ARABIC-INDIC DIGIT FOUR */
  ["\u0665", "\u0665"], /* ARABIC-INDIC DIGIT FIVE */
  ["\u0666", "\u0666"], /* ARABIC-INDIC DIGIT SIX */
  ["\u0667", "\u0667"], /* ARABIC-INDIC DIGIT SEVEN */
  ["\u0668", "\u0668"], /* ARABIC-INDIC DIGIT EIGHT */
  ["\u0669", "\u0669"], /* ARABIC-INDIC DIGIT NINE */
  ["\u066A", "\u066A"], /* ARABIC PERCENT SIGN */
  ["\u066B", "\u066B"], /* ARABIC DECIMAL SEPARATOR */
  ["\u066C", "\u066C"], /* ARABIC THOUSANDS SEPARATOR */
  ["\u066D", "\u066D"], /* ARABIC FIVE POINTED STAR */
  ["\u066E", "\u066E"], /* ARABIC LETTER DOTLESS BEH */
  ["\u066F", "\u066F"], /* ARABIC LETTER DOTLESS QAF */
  ["\u0670", "\u0670"], /* ARABIC LETTER SUPERSCRIPT ALEF (ARABIC ALEF ABOVE) */
  ["\u0671", "\u0671"], /* ARABIC LETTER ALEF WASLA (ARABIC LETTER HAMZAT WASL ON ALEF) */
  ["\u0672", "\u0672"], /* ARABIC LETTER ALEF WITH WAVY HAMZA ABOVE (ARABIC LETTER WAVY HAMZAH ON ALEF) */
  ["\u0673", "\u0673"], /* ARABIC LETTER ALEF WITH WAVY HAMZA BELOW (ARABIC LETTER WAVY HAMZAH UNDER ALEF) */
  ["\u0674", "\u0674"], /* ARABIC LETTER HIGH HAMZA (ARABIC LETTER HIGH HAMZAH) */
  ["\u0675", "\u0675"], /* ARABIC LETTER HIGH HAMZA ALEF (ARABIC LETTER HIGH HAMZAH ALEF) */
  ["\u0676", "\u0676"], /* ARABIC LETTER HIGH HAMZA WAW (ARABIC LETTER HIGH HAMZAH WAW) */
  ["\u0677", "\u0677"], /* ARABIC LETTER U WITH HAMZA ABOVE (ARABIC LETTER HIGH HAMZAH WAW WITH DAMMAH) */
  ["\u0678", "\u0678"], /* ARABIC LETTER HIGH HAMZA YEH (ARABIC LETTER HIGH HAMZAH YA) */
  ["\u0679", "\u0679"], /* ARABIC LETTER TTEH (ARABIC LETTER TAA WITH SMALL TAH) */
  ["\u067A", "\u067A"], /* ARABIC LETTER TTEHEH (ARABIC LETTER TAA WITH TWO DOTS VERTICAL ABOVE) */
  ["\u067B", "\u067B"], /* ARABIC LETTER BEEH (ARABIC LETTER BAA WITH TWO DOTS VERTICAL BELOW) */
  ["\u067C", "\u067C"], /* ARABIC LETTER TEH WITH RING (ARABIC LETTER TAA WITH RING) */
  ["\u067D", "\u067D"], /* ARABIC LETTER TEH WITH THREE DOTS ABOVE DOWNWARDS (ARABIC LETTER TAA WITH THREE DOTS ABOVE DOWNWARD) */
  ["\u067E", "\u067E"], /* ARABIC LETTER PEH (ARABIC LETTER TAA WITH THREE DOTS BELOW) */
  ["\u067F", "\u067F"], /* ARABIC LETTER TEHEH (ARABIC LETTER TAA WITH FOUR DOTS ABOVE) */
  ["\u0680", "\u0680"], /* ARABIC LETTER BEHEH (ARABIC LETTER BAA WITH FOUR DOTS BELOW) */
  ["\u0681", "\u0681"], /* ARABIC LETTER HAH WITH HAMZA ABOVE (ARABIC LETTER HAMZAH ON HAA) */
  ["\u0682", "\u0682"], /* ARABIC LETTER HAH WITH TWO DOTS VERTICAL ABOVE (ARABIC LETTER HAA WITH TWO DOTS VERTICAL ABOVE) */
  ["\u0683", "\u0683"], /* ARABIC LETTER NYEH (ARABIC LETTER HAA WITH MIDDLE TWO DOTS) */
  ["\u0684", "\u0684"], /* ARABIC LETTER DYEH (ARABIC LETTER HAA WITH MIDDLE TWO DOTS VERTICAL) */
  ["\u0685", "\u0685"], /* ARABIC LETTER HAH WITH THREE DOTS ABOVE (ARABIC LETTER HAA WITH THREE DOTS ABOVE) */
  ["\u0686", "\u0686"], /* ARABIC LETTER TCHEH (ARABIC LETTER HAA WITH MIDDLE THREE DOTS DOWNWARD) */
  ["\u0687", "\u0687"], /* ARABIC LETTER TCHEHEH (ARABIC LETTER HAA WITH MIDDLE FOUR DOTS) */
  ["\u0688", "\u0688"], /* ARABIC LETTER DDAL (ARABIC LETTER DAL WITH SMALL TAH) */
  ["\u0689", "\u0689"], /* ARABIC LETTER DAL WITH RING */
  ["\u068A", "\u068A"], /* ARABIC LETTER DAL WITH DOT BELOW */
  ["\u068B", "\u068B"], /* ARABIC LETTER DAL WITH DOT BELOW AND SMALL TAH */
  ["\u068C", "\u068C"], /* ARABIC LETTER DAHAL (ARABIC LETTER DAL WITH TWO DOTS ABOVE) */
  ["\u068D", "\u068D"], /* ARABIC LETTER DDAHAL (ARABIC LETTER DAL WITH TWO DOTS BELOW) */
  ["\u068E", "\u068E"], /* ARABIC LETTER DUL (ARABIC LETTER DAL WITH THREE DOTS ABOVE) */
  ["\u068F", "\u068F"], /* ARABIC LETTER DAL WITH THREE DOTS ABOVE DOWNWARDS (ARABIC LETTER DAL WITH THREE DOTS ABOVE DOWNWARD) */
  ["\u0690", "\u0690"], /* ARABIC LETTER DAL WITH FOUR DOTS ABOVE */
  ["\u0691", "\u0691"], /* ARABIC LETTER RREH (ARABIC LETTER RA WITH SMALL TAH) */
  ["\u0692", "\u0692"], /* ARABIC LETTER REH WITH SMALL V (ARABIC LETTER RA WITH SMALL V) */
  ["\u0693", "\u0693"], /* ARABIC LETTER REH WITH RING (ARABIC LETTER RA WITH RING) */
  ["\u0694", "\u0694"], /* ARABIC LETTER REH WITH DOT BELOW (ARABIC LETTER RA WITH DOT BELOW) */
  ["\u0695", "\u0695"], /* ARABIC LETTER REH WITH SMALL V BELOW (ARABIC LETTER RA WITH SMALL V BELOW) */
  ["\u0696", "\u0696"], /* ARABIC LETTER REH WITH DOT BELOW AND DOT ABOVE (ARABIC LETTER RA WITH DOT BELOW AND DOT ABOVE) */
  ["\u0697", "\u0697"], /* ARABIC LETTER REH WITH TWO DOTS ABOVE (ARABIC LETTER RA WITH TWO DOTS ABOVE) */
  ["\u0698", "\u0698"], /* ARABIC LETTER JEH (ARABIC LETTER RA WITH THREE DOTS ABOVE) */
  ["\u0699", "\u0699"], /* ARABIC LETTER REH WITH FOUR DOTS ABOVE (ARABIC LETTER RA WITH FOUR DOTS ABOVE) */
  ["\u069A", "\u069A"], /* ARABIC LETTER SEEN WITH DOT BELOW AND DOT ABOVE */
  ["\u069B", "\u069B"], /* ARABIC LETTER SEEN WITH THREE DOTS BELOW */
  ["\u069C", "\u069C"], /* ARABIC LETTER SEEN WITH THREE DOTS BELOW AND THREE DOTS ABOVE */
  ["\u069D", "\u069D"], /* ARABIC LETTER SAD WITH TWO DOTS BELOW */
  ["\u069E", "\u069E"], /* ARABIC LETTER SAD WITH THREE DOTS ABOVE */
  ["\u069F", "\u069F"], /* ARABIC LETTER TAH WITH THREE DOTS ABOVE */
  ["\u06A0", "\u06A0"], /* ARABIC LETTER AIN WITH THREE DOTS ABOVE */
  ["\u06A1", "\u06A1"], /* ARABIC LETTER DOTLESS FEH (ARABIC LETTER DOTLESS FA) */
  ["\u06A2", "\u06A2"], /* ARABIC LETTER FEH WITH DOT MOVED BELOW (ARABIC LETTER FA WITH DOT MOVED BELOW) */
  ["\u06A3", "\u06A3"], /* ARABIC LETTER FEH WITH DOT BELOW (ARABIC LETTER FA WITH DOT BELOW) */
  ["\u06A4", "\u06A4"], /* ARABIC LETTER VEH (ARABIC LETTER FA WITH THREE DOTS ABOVE) */
  ["\u06A5", "\u06A5"], /* ARABIC LETTER FEH WITH THREE DOTS BELOW (ARABIC LETTER FA WITH THREE DOTS BELOW) */
  ["\u06A6", "\u06A6"], /* ARABIC LETTER PEHEH (ARABIC LETTER FA WITH FOUR DOTS ABOVE) */
  ["\u06A7", "\u06A7"], /* ARABIC LETTER QAF WITH DOT ABOVE */
  ["\u06A8", "\u06A8"], /* ARABIC LETTER QAF WITH THREE DOTS ABOVE */
  ["\u06A9", "\u06A9"], /* ARABIC LETTER KEHEH (ARABIC LETTER OPEN CAF) */
  ["\u06AA", "\u06AA"], /* ARABIC LETTER SWASH KAF (ARABIC LETTER SWASH CAF) */
  ["\u06AB", "\u06AB"], /* ARABIC LETTER KAF WITH RING (ARABIC LETTER CAF WITH RING) */
  ["\u06AC", "\u06AC"], /* ARABIC LETTER KAF WITH DOT ABOVE (ARABIC LETTER CAF WITH DOT ABOVE) */
  ["\u06AD", "\u06AD"], /* ARABIC LETTER NG (ARABIC LETTER CAF WITH THREE DOTS ABOVE) */
  ["\u06AE", "\u06AE"], /* ARABIC LETTER KAF WITH THREE DOTS BELOW (ARABIC LETTER CAF WITH THREE DOTS BELOW) */
  ["\u06AF", "\u06AF"], /* ARABIC LETTER GAF */
  ["\u06B0", "\u06B0"], /* ARABIC LETTER GAF WITH RING */
  ["\u06B1", "\u06B1"], /* ARABIC LETTER NGOEH (ARABIC LETTER GAF WITH TWO DOTS ABOVE) */
  ["\u06B2", "\u06B2"], /* ARABIC LETTER GAF WITH TWO DOTS BELOW */
  ["\u06B3", "\u06B3"], /* ARABIC LETTER GUEH (ARABIC LETTER GAF WITH TWO DOTS VERTICAL BELOW) */
  ["\u06B4", "\u06B4"], /* ARABIC LETTER GAF WITH THREE DOTS ABOVE */
  ["\u06B5", "\u06B5"], /* ARABIC LETTER LAM WITH SMALL V */
  ["\u06B6", "\u06B6"], /* ARABIC LETTER LAM WITH DOT ABOVE */
  ["\u06B7", "\u06B7"], /* ARABIC LETTER LAM WITH THREE DOTS ABOVE */
  ["\u06B8", "\u06B8"], /* ARABIC LETTER LAM WITH THREE DOTS BELOW */
  ["\u06B9", "\u06B9"], /* ARABIC LETTER NOON WITH DOT BELOW */
  ["\u06BA", "\u06BA"], /* ARABIC LETTER NOON GHUNNA (ARABIC LETTER DOTLESS NOON) */
  ["\u06BB", "\u06BB"], /* ARABIC LETTER RNOON (ARABIC LETTER DOTLESS NOON WITH SMALL TAH) */
  ["\u06BC", "\u06BC"], /* ARABIC LETTER NOON WITH RING */
  ["\u06BD", "\u06BD"], /* ARABIC LETTER NOON WITH THREE DOTS ABOVE */
  ["\u06BE", "\u06BE"], /* ARABIC LETTER HEH DOACHASHMEE (ARABIC LETTER KNOTTED HA) */
  ["\u06BF", "\u06BF"], /* ARABIC LETTER TCHEH WITH DOT ABOVE */
  ["\u06C0", "\u06C0"], /* ARABIC LETTER HEH WITH YEH ABOVE (ARABIC LETTER HAMZAH ON HA) */
  ["\u06C1", "\u06C1"], /* ARABIC LETTER HEH GOAL (ARABIC LETTER HA GOAL) */
  ["\u06C2", "\u06C2"], /* ARABIC LETTER HEH GOAL WITH HAMZA ABOVE (ARABIC LETTER HAMZAH ON HA GOAL) */
  ["\u06C3", "\u06C3"], /* ARABIC LETTER TEH MARBUTA GOAL (ARABIC LETTER TAA MARBUTAH GOAL) */
  ["\u06C4", "\u06C4"], /* ARABIC LETTER WAW WITH RING */
  ["\u06C5", "\u06C5"], /* ARABIC LETTER KIRGHIZ OE (ARABIC LETTER WAW WITH BAR) */
  ["\u06C6", "\u06C6"], /* ARABIC LETTER OE (ARABIC LETTER WAW WITH SMALL V) */
  ["\u06C7", "\u06C7"], /* ARABIC LETTER U (ARABIC LETTER WAW WITH DAMMAH) */
  ["\u06C8", "\u06C8"], /* ARABIC LETTER YU (ARABIC LETTER WAW WITH ALEF ABOVE) */
  ["\u06C9", "\u06C9"], /* ARABIC LETTER KIRGHIZ YU (ARABIC LETTER WAW WITH INVERTED SMALL V) */
  ["\u06CA", "\u06CA"], /* ARABIC LETTER WAW WITH TWO DOTS ABOVE */
  ["\u06CB", "\u06CB"], /* ARABIC LETTER VE (ARABIC LETTER WAW WITH THREE DOTS ABOVE) */
  ["\u06CC", "\u06CC"], /* ARABIC LETTER FARSI YEH (ARABIC LETTER DOTLESS YA) */
  ["\u06CD", "\u06CD"], /* ARABIC LETTER YEH WITH TAIL (ARABIC LETTER YA WITH TAIL) */
  ["\u06CE", "\u06CE"], /* ARABIC LETTER YEH WITH SMALL V (ARABIC LETTER YA WITH SMALL V) */
  ["\u06CF", "\u06CF"], /* ARABIC LETTER WAW WITH DOT ABOVE */
  ["\u06D0", "\u06D0"], /* ARABIC LETTER E (ARABIC LETTER YA WITH TWO DOTS VERTICAL BELOW) */
  ["\u06D1", "\u06D1"], /* ARABIC LETTER YEH WITH THREE DOTS BELOW (ARABIC LETTER YA WITH THREE DOTS BELOW) */
  ["\u06D2", "\u06D2"], /* ARABIC LETTER YEH BARREE (ARABIC LETTER YA BARREE) */
  ["\u06D3", "\u06D3"], /* ARABIC LETTER YEH BARREE WITH HAMZA ABOVE (ARABIC LETTER HAMZAH ON YA BARREE) */
  ["\u06D4", "\u06D4"], /* ARABIC FULL STOP (ARABIC PERIOD) */
  ["\u06D5", "\u06D5"], /* ARABIC LETTER AE */
  ["\u06D6", "\u06D6"], /* ARABIC SMALL HIGH LIGATURE SAD WITH LAM WITH ALEF MAKSURA */
  ["\u06D7", "\u06D7"], /* ARABIC SMALL HIGH LIGATURE QAF WITH LAM WITH ALEF MAKSURA */
  ["\u06D8", "\u06D8"], /* ARABIC SMALL HIGH MEEM INITIAL FORM */
  ["\u06D9", "\u06D9"], /* ARABIC SMALL HIGH LAM ALEF */
  ["\u06DA", "\u06DA"], /* ARABIC SMALL HIGH JEEM */
  ["\u06DB", "\u06DB"], /* ARABIC SMALL HIGH THREE DOTS */
  ["\u06DC", "\u06DC"], /* ARABIC SMALL HIGH SEEN */
  ["\u06DD", "\u06DD"], /* ARABIC END OF AYAH */
  ["\u06DE", "\u06DE"], /* ARABIC START OF RUB EL HIZB */
  ["\u06DF", "\u06DF"], /* ARABIC SMALL HIGH ROUNDED ZERO */
  ["\u06E0", "\u06E0"], /* ARABIC SMALL HIGH UPRIGHT RECTANGULAR ZERO */
  ["\u06E1", "\u06E1"], /* ARABIC SMALL HIGH DOTLESS HEAD OF KHAH */
  ["\u06E2", "\u06E2"], /* ARABIC SMALL HIGH MEEM ISOLATED FORM */
  ["\u06E3", "\u06E3"], /* ARABIC SMALL LOW SEEN */
  ["\u06E4", "\u06E4"], /* ARABIC SMALL HIGH MADDA */
  ["\u06E5", "\u06E5"], /* ARABIC SMALL WAW */
  ["\u06E6", "\u06E6"], /* ARABIC SMALL YEH */
  ["\u06E7", "\u06E7"], /* ARABIC SMALL HIGH YEH */
  ["\u06E8", "\u06E8"], /* ARABIC SMALL HIGH NOON */
  ["\u06E9", "\u06E9"], /* ARABIC PLACE OF SAJDAH */
  ["\u06EA", "\u06EA"], /* ARABIC EMPTY CENTRE LOW STOP */
  ["\u06EB", "\u06EB"], /* ARABIC EMPTY CENTRE HIGH STOP */
  ["\u06EC", "\u06EC"], /* ARABIC ROUNDED HIGH STOP WITH FILLED CENTRE */
  ["\u06ED", "\u06ED"], /* ARABIC SMALL LOW MEEM */
  ["\u06EE", "\u06EE"], /* ARABIC LETTER DAL WITH INVERTED V */
  ["\u06EF", "\u06EF"], /* ARABIC LETTER REH WITH INVERTED V */
  ["\u06F0", "\u06F0"], /* EXTENDED ARABIC-INDIC DIGIT ZERO (EASTERN ARABIC-INDIC DIGIT ZERO) */
  ["\u06F1", "\u06F1"], /* EXTENDED ARABIC-INDIC DIGIT ONE (EASTERN ARABIC-INDIC DIGIT ONE) */
  ["\u06F2", "\u06F2"], /* EXTENDED ARABIC-INDIC DIGIT TWO (EASTERN ARABIC-INDIC DIGIT TWO) */
  ["\u06F3", "\u06F3"], /* EXTENDED ARABIC-INDIC DIGIT THREE (EASTERN ARABIC-INDIC DIGIT THREE) */
  ["\u06F4", "\u06F4"], /* EXTENDED ARABIC-INDIC DIGIT FOUR (EASTERN ARABIC-INDIC DIGIT FOUR) */
  ["\u06F5", "\u06F5"], /* EXTENDED ARABIC-INDIC DIGIT FIVE (EASTERN ARABIC-INDIC DIGIT FIVE) */
  ["\u06F6", "\u06F6"], /* EXTENDED ARABIC-INDIC DIGIT SIX (EASTERN ARABIC-INDIC DIGIT SIX) */
  ["\u06F7", "\u06F7"], /* EXTENDED ARABIC-INDIC DIGIT SEVEN (EASTERN ARABIC-INDIC DIGIT SEVEN) */
  ["\u06F8", "\u06F8"], /* EXTENDED ARABIC-INDIC DIGIT EIGHT (EASTERN ARABIC-INDIC DIGIT EIGHT) */
  ["\u06F9", "\u06F9"], /* EXTENDED ARABIC-INDIC DIGIT NINE (EASTERN ARABIC-INDIC DIGIT NINE) */
  ["\u06FA", "\u06FA"], /* ARABIC LETTER SHEEN WITH DOT BELOW */
  ["\u06FB", "\u06FB"], /* ARABIC LETTER DAD WITH DOT BELOW */
  ["\u06FC", "\u06FC"], /* ARABIC LETTER GHAIN WITH DOT BELOW */
  ["\u06FD", "\u06FD"], /* ARABIC SIGN SINDHI AMPERSAND */
  ["\u06FE", "\u06FE"], /* ARABIC SIGN SINDHI POSTPOSITION MEN */
  ["\u06FF", "\u06FF"], /* ARABIC LETTER HEH WITH INVERTED V */
  ["\u0700", "\u0700"], /* SYRIAC END OF PARAGRAPH */
  ["\u0701", "\u0701"], /* SYRIAC SUPRALINEAR FULL STOP */
  ["\u0702", "\u0702"], /* SYRIAC SUBLINEAR FULL STOP */
  ["\u0703", "\u0703"], /* SYRIAC SUPRALINEAR COLON */
  ["\u0704", "\u0704"], /* SYRIAC SUBLINEAR COLON */
  ["\u0705", "\u0705"], /* SYRIAC HORIZONTAL COLON */
  ["\u0706", "\u0706"], /* SYRIAC COLON SKEWED LEFT */
  ["\u0707", "\u0707"], /* SYRIAC COLON SKEWED RIGHT */
  ["\u0708", "\u0708"], /* SYRIAC SUPRALINEAR COLON SKEWED LEFT */
  ["\u0709", "\u0709"], /* SYRIAC SUBLINEAR COLON SKEWED RIGHT */
  ["\u070A", "\u070A"], /* SYRIAC CONTRACTION */
  ["\u070B", "\u070B"], /* SYRIAC HARKLEAN OBELUS */
  ["\u070C", "\u070C"], /* SYRIAC HARKLEAN METOBELUS */
  ["\u070D", "\u070D"], /* SYRIAC HARKLEAN ASTERISCUS */
  ["\u070E", "\u070E"],
  ["\u070F", "\u070F"], /* SYRIAC ABBREVIATION MARK */
  ["\u0710", "\u0710"], /* SYRIAC LETTER ALAPH */
  ["\u0711", "\u0711"], /* SYRIAC LETTER SUPERSCRIPT ALAPH */
  ["\u0712", "\u0712"], /* SYRIAC LETTER BETH */
  ["\u0713", "\u0713"], /* SYRIAC LETTER GAMAL */
  ["\u0714", "\u0714"], /* SYRIAC LETTER GAMAL GARSHUNI */
  ["\u0715", "\u0715"], /* SYRIAC LETTER DALATH */
  ["\u0716", "\u0716"], /* SYRIAC LETTER DOTLESS DALATH RISH */
  ["\u0717", "\u0717"], /* SYRIAC LETTER HE */
  ["\u0718", "\u0718"], /* SYRIAC LETTER WAW */
  ["\u0719", "\u0719"], /* SYRIAC LETTER ZAIN */
  ["\u071A", "\u071A"], /* SYRIAC LETTER HETH */
  ["\u071B", "\u071B"], /* SYRIAC LETTER TETH */
  ["\u071C", "\u071C"], /* SYRIAC LETTER TETH GARSHUNI */
  ["\u071D", "\u071D"], /* SYRIAC LETTER YUDH */
  ["\u071E", "\u071E"], /* SYRIAC LETTER YUDH HE */
  ["\u071F", "\u071F"], /* SYRIAC LETTER KAPH */
  ["\u0720", "\u0720"], /* SYRIAC LETTER LAMADH */
  ["\u0721", "\u0721"], /* SYRIAC LETTER MIM */
  ["\u0722", "\u0722"], /* SYRIAC LETTER NUN */
  ["\u0723", "\u0723"], /* SYRIAC LETTER SEMKATH */
  ["\u0724", "\u0724"], /* SYRIAC LETTER FINAL SEMKATH */
  ["\u0725", "\u0725"], /* SYRIAC LETTER E */
  ["\u0726", "\u0726"], /* SYRIAC LETTER PE */
  ["\u0727", "\u0727"], /* SYRIAC LETTER REVERSED PE */
  ["\u0728", "\u0728"], /* SYRIAC LETTER SADHE */
  ["\u0729", "\u0729"], /* SYRIAC LETTER QAPH */
  ["\u072A", "\u072A"], /* SYRIAC LETTER RISH */
  ["\u072B", "\u072B"], /* SYRIAC LETTER SHIN */
  ["\u072C", "\u072C"], /* SYRIAC LETTER TAW */
  ["\u072D", "\u072D"], /* SYRIAC LETTER PERSIAN BHETH */
  ["\u072E", "\u072E"], /* SYRIAC LETTER PERSIAN GHAMAL */
  ["\u072F", "\u072F"], /* SYRIAC LETTER PERSIAN DHALATH */
  ["\u0730", "\u0730"], /* SYRIAC PTHAHA ABOVE */
  ["\u0731", "\u0731"], /* SYRIAC PTHAHA BELOW */
  ["\u0732", "\u0732"], /* SYRIAC PTHAHA DOTTED */
  ["\u0733", "\u0733"], /* SYRIAC ZQAPHA ABOVE */
  ["\u0734", "\u0734"], /* SYRIAC ZQAPHA BELOW */
  ["\u0735", "\u0735"], /* SYRIAC ZQAPHA DOTTED */
  ["\u0736", "\u0736"], /* SYRIAC RBASA ABOVE */
  ["\u0737", "\u0737"], /* SYRIAC RBASA BELOW */
  ["\u0738", "\u0738"], /* SYRIAC DOTTED ZLAMA HORIZONTAL */
  ["\u0739", "\u0739"], /* SYRIAC DOTTED ZLAMA ANGULAR */
  ["\u073A", "\u073A"], /* SYRIAC HBASA ABOVE */
  ["\u073B", "\u073B"], /* SYRIAC HBASA BELOW */
  ["\u073C", "\u073C"], /* SYRIAC HBASA-ESASA DOTTED */
  ["\u073D", "\u073D"], /* SYRIAC ESASA ABOVE */
  ["\u073E", "\u073E"], /* SYRIAC ESASA BELOW */
  ["\u073F", "\u073F"], /* SYRIAC RWAHA */
  ["\u0740", "\u0740"], /* SYRIAC FEMININE DOT */
  ["\u0741", "\u0741"], /* SYRIAC QUSHSHAYA */
  ["\u0742", "\u0742"], /* SYRIAC RUKKAKHA */
  ["\u0743", "\u0743"], /* SYRIAC TWO VERTICAL DOTS ABOVE */
  ["\u0744", "\u0744"], /* SYRIAC TWO VERTICAL DOTS BELOW */
  ["\u0745", "\u0745"], /* SYRIAC THREE DOTS ABOVE */
  ["\u0746", "\u0746"], /* SYRIAC THREE DOTS BELOW */
  ["\u0747", "\u0747"], /* SYRIAC OBLIQUE LINE ABOVE */
  ["\u0748", "\u0748"], /* SYRIAC OBLIQUE LINE BELOW */
  ["\u0749", "\u0749"], /* SYRIAC MUSIC */
  ["\u074A", "\u074A"], /* SYRIAC BARREKH */
  ["\u074B", "\u074B"],
  ["\u074C", "\u074C"],
  ["\u074D", "\u074D"], /* SYRIAC LETTER SOGDIAN ZHAIN */
  ["\u074E", "\u074E"], /* SYRIAC LETTER SOGDIAN KHAPH */
  ["\u074F", "\u074F"], /* SYRIAC LETTER SOGDIAN FE */
  ["\u0750", "\u0750"], /* ARABIC LETTER BEH WITH THREE DOTS HORIZONTALLY BELOW */
  ["\u0751", "\u0751"], /* ARABIC LETTER BEH WITH DOT BELOW AND THREE DOTS ABOVE */
  ["\u0752", "\u0752"], /* ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW */
  ["\u0753", "\u0753"], /* ARABIC LETTER BEH WITH THREE DOTS POINTING UPWARDS BELOW AND TWO DOTS ABOVE */
  ["\u0754", "\u0754"], /* ARABIC LETTER BEH WITH TWO DOTS BELOW AND DOT ABOVE */
  ["\u0755", "\u0755"], /* ARABIC LETTER BEH WITH INVERTED SMALL V BELOW */
  ["\u0756", "\u0756"], /* ARABIC LETTER BEH WITH SMALL V */
  ["\u0757", "\u0757"], /* ARABIC LETTER HAH WITH TWO DOTS ABOVE */
  ["\u0758", "\u0758"], /* ARABIC LETTER HAH WITH THREE DOTS POINTING UPWARDS BELOW */
  ["\u0759", "\u0759"], /* ARABIC LETTER DAL WITH TWO DOTS VERTICALLY BELOW AND SMALL TAH */
  ["\u075A", "\u075A"], /* ARABIC LETTER DAL WITH INVERTED SMALL V BELOW */
  ["\u075B", "\u075B"], /* ARABIC LETTER REH WITH STROKE */
  ["\u075C", "\u075C"], /* ARABIC LETTER SEEN WITH FOUR DOTS ABOVE */
  ["\u075D", "\u075D"], /* ARABIC LETTER AIN WITH TWO DOTS ABOVE */
  ["\u075E", "\u075E"], /* ARABIC LETTER AIN WITH THREE DOTS POINTING DOWNWARDS ABOVE */
  ["\u075F", "\u075F"], /* ARABIC LETTER AIN WITH TWO DOTS VERTICALLY ABOVE */
  ["\u0760", "\u0760"], /* ARABIC LETTER FEH WITH TWO DOTS BELOW */
  ["\u0761", "\u0761"], /* ARABIC LETTER FEH WITH THREE DOTS POINTING UPWARDS BELOW */
  ["\u0762", "\u0762"], /* ARABIC LETTER KEHEH WITH DOT ABOVE */
  ["\u0763", "\u0763"], /* ARABIC LETTER KEHEH WITH THREE DOTS ABOVE */
  ["\u0764", "\u0764"], /* ARABIC LETTER KEHEH WITH THREE DOTS POINTING UPWARDS BELOW */
  ["\u0765", "\u0765"], /* ARABIC LETTER MEEM WITH DOT ABOVE */
  ["\u0766", "\u0766"], /* ARABIC LETTER MEEM WITH DOT BELOW */
  ["\u0767", "\u0767"], /* ARABIC LETTER NOON WITH TWO DOTS BELOW */
  ["\u0768", "\u0768"], /* ARABIC LETTER NOON WITH SMALL TAH */
  ["\u0769", "\u0769"], /* ARABIC LETTER NOON WITH SMALL V */
  ["\u076A", "\u076A"], /* ARABIC LETTER LAM WITH BAR */
  ["\u076B", "\u076B"], /* ARABIC LETTER REH WITH TWO DOTS VERTICALLY ABOVE */
  ["\u076C", "\u076C"], /* ARABIC LETTER REH WITH HAMZA ABOVE */
  ["\u076D", "\u076D"], /* ARABIC LETTER SEEN WITH TWO DOTS VERTICALLY ABOVE */
  ["\u076E", "\u076E"], /* ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH BELOW */
  ["\u076F", "\u076F"], /* ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH AND TWO DOTS */
  ["\u0770", "\u0770"], /* ARABIC LETTER SEEN WITH SMALL ARABIC LETTER TAH AND TWO DOTS */
  ["\u0771", "\u0771"], /* ARABIC LETTER REH WITH SMALL ARABIC LETTER TAH AND TWO DOTS */
  ["\u0772", "\u0772"], /* ARABIC LETTER HAH WITH SMALL ARABIC LETTER TAH ABOVE */
  ["\u0773", "\u0773"], /* ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE */
  ["\u0774", "\u0774"], /* ARABIC LETTER ALEF WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE */
  ["\u0775", "\u0775"], /* ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE */
  ["\u0776", "\u0776"], /* ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE */
  ["\u0777", "\u0777"], /* ARABIC LETTER FARSI YEH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW */
  ["\u0778", "\u0778"], /* ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE */
  ["\u0779", "\u0779"], /* ARABIC LETTER WAW WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE */
  ["\u077A", "\u077A"], /* ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT TWO ABOVE */
  ["\u077B", "\u077B"], /* ARABIC LETTER YEH BARREE WITH EXTENDED ARABIC-INDIC DIGIT THREE ABOVE */
  ["\u077C", "\u077C"], /* ARABIC LETTER HAH WITH EXTENDED ARABIC-INDIC DIGIT FOUR BELOW */
  ["\u077D", "\u077D"], /* ARABIC LETTER SEEN WITH EXTENDED ARABIC-INDIC DIGIT FOUR ABOVE */
  ["\u077E", "\u077E"], /* ARABIC LETTER SEEN WITH INVERTED V */
  ["\u077F", "\u077F"], /* ARABIC LETTER KAF WITH TWO DOTS ABOVE */
  ["\u0780", "\u0780"], /* THAANA LETTER HAA */
  ["\u0781", "\u0781"], /* THAANA LETTER SHAVIYANI */
  ["\u0782", "\u0782"], /* THAANA LETTER NOONU */
  ["\u0783", "\u0783"], /* THAANA LETTER RAA */
  ["\u0784", "\u0784"], /* THAANA LETTER BAA */
  ["\u0785", "\u0785"], /* THAANA LETTER LHAVIYANI */
  ["\u0786", "\u0786"], /* THAANA LETTER KAAFU */
  ["\u0787", "\u0787"], /* THAANA LETTER ALIFU */
  ["\u0788", "\u0788"], /* THAANA LETTER VAAVU */
  ["\u0789", "\u0789"], /* THAANA LETTER MEEMU */
  ["\u078A", "\u078A"], /* THAANA LETTER FAAFU */
  ["\u078B", "\u078B"], /* THAANA LETTER DHAALU */
  ["\u078C", "\u078C"], /* THAANA LETTER THAA */
  ["\u078D", "\u078D"], /* THAANA LETTER LAAMU */
  ["\u078E", "\u078E"], /* THAANA LETTER GAAFU */
  ["\u078F", "\u078F"], /* THAANA LETTER GNAVIYANI */
  ["\u0790", "\u0790"], /* THAANA LETTER SEENU */
  ["\u0791", "\u0791"], /* THAANA LETTER DAVIYANI */
  ["\u0792", "\u0792"], /* THAANA LETTER ZAVIYANI */
  ["\u0793", "\u0793"], /* THAANA LETTER TAVIYANI */
  ["\u0794", "\u0794"], /* THAANA LETTER YAA */
  ["\u0795", "\u0795"], /* THAANA LETTER PAVIYANI */
  ["\u0796", "\u0796"], /* THAANA LETTER JAVIYANI */
  ["\u0797", "\u0797"], /* THAANA LETTER CHAVIYANI */
  ["\u0798", "\u0798"], /* THAANA LETTER TTAA */
  ["\u0799", "\u0799"], /* THAANA LETTER HHAA */
  ["\u079A", "\u079A"], /* THAANA LETTER KHAA */
  ["\u079B", "\u079B"], /* THAANA LETTER THAALU */
  ["\u079C", "\u079C"], /* THAANA LETTER ZAA */
  ["\u079D", "\u079D"], /* THAANA LETTER SHEENU */
  ["\u079E", "\u079E"], /* THAANA LETTER SAADHU */
  ["\u079F", "\u079F"], /* THAANA LETTER DAADHU */
  ["\u07A0", "\u07A0"], /* THAANA LETTER TO */
  ["\u07A1", "\u07A1"], /* THAANA LETTER ZO */
  ["\u07A2", "\u07A2"], /* THAANA LETTER AINU */
  ["\u07A3", "\u07A3"], /* THAANA LETTER GHAINU */
  ["\u07A4", "\u07A4"], /* THAANA LETTER QAAFU */
  ["\u07A5", "\u07A5"], /* THAANA LETTER WAAVU */
  ["\u07A6", "\u07A6"], /* THAANA ABAFILI */
  ["\u07A7", "\u07A7"], /* THAANA AABAAFILI */
  ["\u07A8", "\u07A8"], /* THAANA IBIFILI */
  ["\u07A9", "\u07A9"], /* THAANA EEBEEFILI */
  ["\u07AA", "\u07AA"], /* THAANA UBUFILI */
  ["\u07AB", "\u07AB"], /* THAANA OOBOOFILI */
  ["\u07AC", "\u07AC"], /* THAANA EBEFILI */
  ["\u07AD", "\u07AD"], /* THAANA EYBEYFILI */
  ["\u07AE", "\u07AE"], /* THAANA OBOFILI */
  ["\u07AF", "\u07AF"], /* THAANA OABOAFILI */
  ["\u07B0", "\u07B0"], /* THAANA SUKUN */
  ["\u07B1", "\u07B1"], /* THAANA LETTER NAA */
  ["\u07B2", "\u07B2"],
  ["\u07B3", "\u07B3"],
  ["\u07B4", "\u07B4"],
  ["\u07B5", "\u07B5"],
  ["\u07B6", "\u07B6"],
  ["\u07B7", "\u07B7"],
  ["\u07B8", "\u07B8"],
  ["\u07B9", "\u07B9"],
  ["\u07BA", "\u07BA"],
  ["\u07BB", "\u07BB"],
  ["\u07BC", "\u07BC"],
  ["\u07BD", "\u07BD"],
  ["\u07BE", "\u07BE"],
  ["\u07BF", "\u07BF"],
  ["\u07C0", "\u07C0"], /* NKO DIGIT ZERO */
  ["\u07C1", "\u07C1"], /* NKO DIGIT ONE */
  ["\u07C2", "\u07C2"], /* NKO DIGIT TWO */
  ["\u07C3", "\u07C3"], /* NKO DIGIT THREE */
  ["\u07C4", "\u07C4"], /* NKO DIGIT FOUR */
  ["\u07C5", "\u07C5"], /* NKO DIGIT FIVE */
  ["\u07C6", "\u07C6"], /* NKO DIGIT SIX */
  ["\u07C7", "\u07C7"], /* NKO DIGIT SEVEN */
  ["\u07C8", "\u07C8"], /* NKO DIGIT EIGHT */
  ["\u07C9", "\u07C9"], /* NKO DIGIT NINE */
  ["\u07CA", "\u07CA"], /* NKO LETTER A */
  ["\u07CB", "\u07CB"], /* NKO LETTER EE */
  ["\u07CC", "\u07CC"], /* NKO LETTER I */
  ["\u07CD", "\u07CD"], /* NKO LETTER E */
  ["\u07CE", "\u07CE"], /* NKO LETTER U */
  ["\u07CF", "\u07CF"], /* NKO LETTER OO */
  ["\u07D0", "\u07D0"], /* NKO LETTER O */
  ["\u07D1", "\u07D1"], /* NKO LETTER DAGBASINNA */
  ["\u07D2", "\u07D2"], /* NKO LETTER N */
  ["\u07D3", "\u07D3"], /* NKO LETTER BA */
  ["\u07D4", "\u07D4"], /* NKO LETTER PA */
  ["\u07D5", "\u07D5"], /* NKO LETTER TA */
  ["\u07D6", "\u07D6"], /* NKO LETTER JA */
  ["\u07D7", "\u07D7"], /* NKO LETTER CHA */
  ["\u07D8", "\u07D8"], /* NKO LETTER DA */
  ["\u07D9", "\u07D9"], /* NKO LETTER RA */
  ["\u07DA", "\u07DA"], /* NKO LETTER RRA */
  ["\u07DB", "\u07DB"], /* NKO LETTER SA */
  ["\u07DC", "\u07DC"], /* NKO LETTER GBA */
  ["\u07DD", "\u07DD"], /* NKO LETTER FA */
  ["\u07DE", "\u07DE"], /* NKO LETTER KA */
  ["\u07DF", "\u07DF"], /* NKO LETTER LA */
  ["\u07E0", "\u07E0"], /* NKO LETTER NA WOLOSO */
  ["\u07E1", "\u07E1"], /* NKO LETTER MA */
  ["\u07E2", "\u07E2"], /* NKO LETTER NYA */
  ["\u07E3", "\u07E3"], /* NKO LETTER NA */
  ["\u07E4", "\u07E4"], /* NKO LETTER HA */
  ["\u07E5", "\u07E5"], /* NKO LETTER WA */
  ["\u07E6", "\u07E6"], /* NKO LETTER YA */
  ["\u07E7", "\u07E7"], /* NKO LETTER NYA WOLOSO */
  ["\u07E8", "\u07E8"], /* NKO LETTER JONA JA */
  ["\u07E9", "\u07E9"], /* NKO LETTER JONA CHA */
  ["\u07EA", "\u07EA"], /* NKO LETTER JONA RA */
  ["\u07EB", "\u07EB"], /* NKO COMBINING SHORT HIGH TONE */
  ["\u07EC", "\u07EC"], /* NKO COMBINING SHORT LOW TONE */
  ["\u07ED", "\u07ED"], /* NKO COMBINING SHORT RISING TONE */
  ["\u07EE", "\u07EE"], /* NKO COMBINING LONG DESCENDING TONE */
  ["\u07EF", "\u07EF"], /* NKO COMBINING LONG HIGH TONE */
  ["\u07F0", "\u07F0"], /* NKO COMBINING LONG LOW TONE */
  ["\u07F1", "\u07F1"], /* NKO COMBINING LONG RISING TONE */
  ["\u07F2", "\u07F2"], /* NKO COMBINING NASALIZATION MARK */
  ["\u07F3", "\u07F3"], /* NKO COMBINING DOUBLE DOT ABOVE */
  ["\u07F4", "\u07F4"], /* NKO HIGH TONE APOSTROPHE */
  ["\u07F5", "\u07F5"], /* NKO LOW TONE APOSTROPHE */
  ["\u07F6", "\u07F6"], /* NKO SYMBOL OO DENNEN */
  ["\u07F7", "\u07F7"], /* NKO SYMBOL GBAKURUNEN */
  ["\u07F8", "\u07F8"], /* NKO COMMA */
  ["\u07F9", "\u07F9"], /* NKO EXCLAMATION MARK */
  ["\u07FA", "\u07FA"], /* NKO LAJANYALAN */
  ["\u07FB", "\u07FB"],
  ["\u07FC", "\u07FC"],
  ["\u07FD", "\u07FD"], /* NKO DANTAYALAN */
  ["\u07FE", "\u07FE"], /* NKO DOROME SIGN */
  ["\u07FF", "\u07FF"], /* NKO TAMAN SIGN */
  ["\u0800", "\u0800"], /* SAMARITAN LETTER ALAF */
  ["\u0801", "\u0801"], /* SAMARITAN LETTER BIT */
  ["\u0802", "\u0802"], /* SAMARITAN LETTER GAMAN */
  ["\u0803", "\u0803"], /* SAMARITAN LETTER DALAT */
  ["\u0804", "\u0804"], /* SAMARITAN LETTER IY */
  ["\u0805", "\u0805"], /* SAMARITAN LETTER BAA */
  ["\u0806", "\u0806"], /* SAMARITAN LETTER ZEN */
  ["\u0807", "\u0807"], /* SAMARITAN LETTER IT */
  ["\u0808", "\u0808"], /* SAMARITAN LETTER TIT */
  ["\u0809", "\u0809"], /* SAMARITAN LETTER YUT */
  ["\u080A", "\u080A"], /* SAMARITAN LETTER KAAF */
  ["\u080B", "\u080B"], /* SAMARITAN LETTER LABAT */
  ["\u080C", "\u080C"], /* SAMARITAN LETTER MIM */
  ["\u080D", "\u080D"], /* SAMARITAN LETTER NUN */
  ["\u080E", "\u080E"], /* SAMARITAN LETTER SINGAAT */
  ["\u080F", "\u080F"], /* SAMARITAN LETTER IN */
  ["\u0810", "\u0810"], /* SAMARITAN LETTER FI */
  ["\u0811", "\u0811"], /* SAMARITAN LETTER TSAADIY */
  ["\u0812", "\u0812"], /* SAMARITAN LETTER QUF */
  ["\u0813", "\u0813"], /* SAMARITAN LETTER RISH */
  ["\u0814", "\u0814"], /* SAMARITAN LETTER SHAN */
  ["\u0815", "\u0815"], /* SAMARITAN LETTER TAAF */
  ["\u0816", "\u0816"], /* SAMARITAN MARK IN */
  ["\u0817", "\u0817"], /* SAMARITAN MARK IN-ALAF */
  ["\u0818", "\u0818"], /* SAMARITAN MARK OCCLUSION */
  ["\u0819", "\u0819"], /* SAMARITAN MARK DAGESH */
  ["\u081A", "\u081A"], /* SAMARITAN MODIFIER LETTER EPENTHETIC YUT */
  ["\u081B", "\u081B"], /* SAMARITAN MARK EPENTHETIC YUT */
  ["\u081C", "\u081C"], /* SAMARITAN VOWEL SIGN LONG E */
  ["\u081D", "\u081D"], /* SAMARITAN VOWEL SIGN E */
  ["\u081E", "\u081E"], /* SAMARITAN VOWEL SIGN OVERLONG AA */
  ["\u081F", "\u081F"], /* SAMARITAN VOWEL SIGN LONG AA */
  ["\u0820", "\u0820"], /* SAMARITAN VOWEL SIGN AA */
  ["\u0821", "\u0821"], /* SAMARITAN VOWEL SIGN OVERLONG A */
  ["\u0822", "\u0822"], /* SAMARITAN VOWEL SIGN LONG A */
  ["\u0823", "\u0823"], /* SAMARITAN VOWEL SIGN A */
  ["\u0824", "\u0824"], /* SAMARITAN MODIFIER LETTER SHORT A */
  ["\u0825", "\u0825"], /* SAMARITAN VOWEL SIGN SHORT A */
  ["\u0826", "\u0826"], /* SAMARITAN VOWEL SIGN LONG U */
  ["\u0827", "\u0827"], /* SAMARITAN VOWEL SIGN U */
  ["\u0828", "\u0828"], /* SAMARITAN MODIFIER LETTER I */
  ["\u0829", "\u0829"], /* SAMARITAN VOWEL SIGN LONG I */
  ["\u082A", "\u082A"], /* SAMARITAN VOWEL SIGN I */
  ["\u082B", "\u082B"], /* SAMARITAN VOWEL SIGN O */
  ["\u082C", "\u082C"], /* SAMARITAN VOWEL SIGN SUKUN */
  ["\u082D", "\u082D"], /* SAMARITAN MARK NEQUDAA */
  ["\u082E", "\u082E"],
  ["\u082F", "\u082F"],
  ["\u0830", "\u0830"], /* SAMARITAN PUNCTUATION NEQUDAA */
  ["\u0831", "\u0831"], /* SAMARITAN PUNCTUATION AFSAAQ */
  ["\u0832", "\u0832"], /* SAMARITAN PUNCTUATION ANGED */
  ["\u0833", "\u0833"], /* SAMARITAN PUNCTUATION BAU */
  ["\u0834", "\u0834"], /* SAMARITAN PUNCTUATION ATMAAU */
  ["\u0835", "\u0835"], /* SAMARITAN PUNCTUATION SHIYYAALAA */
  ["\u0836", "\u0836"], /* SAMARITAN ABBREVIATION MARK */
  ["\u0837", "\u0837"], /* SAMARITAN PUNCTUATION MELODIC QITSA */
  ["\u0838", "\u0838"], /* SAMARITAN PUNCTUATION ZIQAA */
  ["\u0839", "\u0839"], /* SAMARITAN PUNCTUATION QITSA */
  ["\u083A", "\u083A"], /* SAMARITAN PUNCTUATION ZAEF */
  ["\u083B", "\u083B"], /* SAMARITAN PUNCTUATION TURU */
  ["\u083C", "\u083C"], /* SAMARITAN PUNCTUATION ARKAANU */
  ["\u083D", "\u083D"], /* SAMARITAN PUNCTUATION SOF MASHFAAT */
  ["\u083E", "\u083E"], /* SAMARITAN PUNCTUATION ANNAAU */
  ["\u083F", "\u083F"],
  ["\u0840", "\u0840"], /* MANDAIC LETTER HALQA */
  ["\u0841", "\u0841"], /* MANDAIC LETTER AB */
  ["\u0842", "\u0842"], /* MANDAIC LETTER AG */
  ["\u0843", "\u0843"], /* MANDAIC LETTER AD */
  ["\u0844", "\u0844"], /* MANDAIC LETTER AH */
  ["\u0845", "\u0845"], /* MANDAIC LETTER USHENNA */
  ["\u0846", "\u0846"], /* MANDAIC LETTER AZ */
  ["\u0847", "\u0847"], /* MANDAIC LETTER IT */
  ["\u0848", "\u0848"], /* MANDAIC LETTER ATT */
  ["\u0849", "\u0849"], /* MANDAIC LETTER AKSA */
  ["\u084A", "\u084A"], /* MANDAIC LETTER AK */
  ["\u084B", "\u084B"], /* MANDAIC LETTER AL */
  ["\u084C", "\u084C"], /* MANDAIC LETTER AM */
  ["\u084D", "\u084D"], /* MANDAIC LETTER AN */
  ["\u084E", "\u084E"], /* MANDAIC LETTER AS */
  ["\u084F", "\u084F"], /* MANDAIC LETTER IN */
  ["\u0850", "\u0850"], /* MANDAIC LETTER AP */
  ["\u0851", "\u0851"], /* MANDAIC LETTER ASZ */
  ["\u0852", "\u0852"], /* MANDAIC LETTER AQ */
  ["\u0853", "\u0853"], /* MANDAIC LETTER AR */
  ["\u0854", "\u0854"], /* MANDAIC LETTER ASH */
  ["\u0855", "\u0855"], /* MANDAIC LETTER AT */
  ["\u0856", "\u0856"], /* MANDAIC LETTER DUSHENNA */
  ["\u0857", "\u0857"], /* MANDAIC LETTER KAD */
  ["\u0858", "\u0858"], /* MANDAIC LETTER AIN */
  ["\u0859", "\u0859"], /* MANDAIC AFFRICATION MARK */
  ["\u085A", "\u085A"], /* MANDAIC VOCALIZATION MARK */
  ["\u085B", "\u085B"], /* MANDAIC GEMINATION MARK */
  ["\u085C", "\u085C"],
  ["\u085D", "\u085D"],
  ["\u085E", "\u085E"], /* MANDAIC PUNCTUATION */
  ["\u085F", "\u085F"],
  ["\u0860", "\u0860"], /* SYRIAC LETTER MALAYALAM NGA */
  ["\u0861", "\u0861"], /* SYRIAC LETTER MALAYALAM JA */
  ["\u0862", "\u0862"], /* SYRIAC LETTER MALAYALAM NYA */
  ["\u0863", "\u0863"], /* SYRIAC LETTER MALAYALAM TTA */
  ["\u0864", "\u0864"], /* SYRIAC LETTER MALAYALAM NNA */
  ["\u0865", "\u0865"], /* SYRIAC LETTER MALAYALAM NNNA */
  ["\u0866", "\u0866"], /* SYRIAC LETTER MALAYALAM BHA */
  ["\u0867", "\u0867"], /* SYRIAC LETTER MALAYALAM RA */
  ["\u0868", "\u0868"], /* SYRIAC LETTER MALAYALAM LLA */
  ["\u0869", "\u0869"], /* SYRIAC LETTER MALAYALAM LLLA */
  ["\u086A", "\u086A"], /* SYRIAC LETTER MALAYALAM SSA */
  ["\u086B", "\u086B"],
  ["\u086C", "\u086C"],
  ["\u086D", "\u086D"],
  ["\u086E", "\u086E"],
  ["\u086F", "\u086F"],
  ["\u0870", "\u0870"],
  ["\u0871", "\u0871"],
  ["\u0872", "\u0872"],
  ["\u0873", "\u0873"],
  ["\u0874", "\u0874"],
  ["\u0875", "\u0875"],
  ["\u0876", "\u0876"],
  ["\u0877", "\u0877"],
  ["\u0878", "\u0878"],
  ["\u0879", "\u0879"],
  ["\u087A", "\u087A"],
  ["\u087B", "\u087B"],
  ["\u087C", "\u087C"],
  ["\u087D", "\u087D"],
  ["\u087E", "\u087E"],
  ["\u087F", "\u087F"],
  ["\u0880", "\u0880"],
  ["\u0881", "\u0881"],
  ["\u0882", "\u0882"],
  ["\u0883", "\u0883"],
  ["\u0884", "\u0884"],
  ["\u0885", "\u0885"],
  ["\u0886", "\u0886"],
  ["\u0887", "\u0887"],
  ["\u0888", "\u0888"],
  ["\u0889", "\u0889"],
  ["\u088A", "\u088A"],
  ["\u088B", "\u088B"],
  ["\u088C", "\u088C"],
  ["\u088D", "\u088D"],
  ["\u088E", "\u088E"],
  ["\u088F", "\u088F"],
  ["\u0890", "\u0890"],
  ["\u0891", "\u0891"],
  ["\u0892", "\u0892"],
  ["\u0893", "\u0893"],
  ["\u0894", "\u0894"],
  ["\u0895", "\u0895"],
  ["\u0896", "\u0896"],
  ["\u0897", "\u0897"],
  ["\u0898", "\u0898"],
  ["\u0899", "\u0899"],
  ["\u089A", "\u089A"],
  ["\u089B", "\u089B"],
  ["\u089C", "\u089C"],
  ["\u089D", "\u089D"],
  ["\u089E", "\u089E"],
  ["\u089F", "\u089F"],
  ["\u08A0", "\u08A0"], /* ARABIC LETTER BEH WITH SMALL V BELOW */
  ["\u08A1", "\u08A1"], /* ARABIC LETTER BEH WITH HAMZA ABOVE */
  ["\u08A2", "\u08A2"], /* ARABIC LETTER JEEM WITH TWO DOTS ABOVE */
  ["\u08A3", "\u08A3"], /* ARABIC LETTER TAH WITH TWO DOTS ABOVE */
  ["\u08A4", "\u08A4"], /* ARABIC LETTER FEH WITH DOT BELOW AND THREE DOTS ABOVE */
  ["\u08A5", "\u08A5"], /* ARABIC LETTER QAF WITH DOT BELOW */
  ["\u08A6", "\u08A6"], /* ARABIC LETTER LAM WITH DOUBLE BAR */
  ["\u08A7", "\u08A7"], /* ARABIC LETTER MEEM WITH THREE DOTS ABOVE */
  ["\u08A8", "\u08A8"], /* ARABIC LETTER YEH WITH TWO DOTS BELOW AND HAMZA ABOVE */
  ["\u08A9", "\u08A9"], /* ARABIC LETTER YEH WITH TWO DOTS BELOW AND DOT ABOVE */
  ["\u08AA", "\u08AA"], /* ARABIC LETTER REH WITH LOOP */
  ["\u08AB", "\u08AB"], /* ARABIC LETTER WAW WITH DOT WITHIN */
  ["\u08AC", "\u08AC"], /* ARABIC LETTER ROHINGYA YEH */
  ["\u08AD", "\u08AD"], /* ARABIC LETTER LOW ALEF */
  ["\u08AE", "\u08AE"], /* ARABIC LETTER DAL WITH THREE DOTS BELOW */
  ["\u08AF", "\u08AF"], /* ARABIC LETTER SAD WITH THREE DOTS BELOW */
  ["\u08B0", "\u08B0"], /* ARABIC LETTER GAF WITH INVERTED STROKE */
  ["\u08B1", "\u08B1"], /* ARABIC LETTER STRAIGHT WAW */
  ["\u08B2", "\u08B2"], /* ARABIC LETTER ZAIN WITH INVERTED V ABOVE */
  ["\u08B3", "\u08B3"], /* ARABIC LETTER AIN WITH THREE DOTS BELOW */
  ["\u08B4", "\u08B4"], /* ARABIC LETTER KAF WITH DOT BELOW */
  ["\u08B5", "\u08B5"],
  ["\u08B6", "\u08B6"], /* ARABIC LETTER BEH WITH SMALL MEEM ABOVE */
  ["\u08B7", "\u08B7"], /* ARABIC LETTER PEH WITH SMALL MEEM ABOVE */
  ["\u08B8", "\u08B8"], /* ARABIC LETTER TEH WITH SMALL TEH ABOVE */
  ["\u08B9", "\u08B9"], /* ARABIC LETTER REH WITH SMALL NOON ABOVE */
  ["\u08BA", "\u08BA"], /* ARABIC LETTER YEH WITH TWO DOTS BELOW AND SMALL NOON ABOVE */
  ["\u08BB", "\u08BB"], /* ARABIC LETTER AFRICAN FEH */
  ["\u08BC", "\u08BC"], /* ARABIC LETTER AFRICAN QAF */
  ["\u08BD", "\u08BD"], /* ARABIC LETTER AFRICAN NOON */
  ["\u08BE", "\u08BE"],
  ["\u08BF", "\u08BF"],
  ["\u08C0", "\u08C0"],
  ["\u08C1", "\u08C1"],
  ["\u08C2", "\u08C2"],
  ["\u08C3", "\u08C3"],
  ["\u08C4", "\u08C4"],
  ["\u08C5", "\u08C5"],
  ["\u08C6", "\u08C6"],
  ["\u08C7", "\u08C7"],
  ["\u08C8", "\u08C8"],
  ["\u08C9", "\u08C9"],
  ["\u08CA", "\u08CA"],
  ["\u08CB", "\u08CB"],
  ["\u08CC", "\u08CC"],
  ["\u08CD", "\u08CD"],
  ["\u08CE", "\u08CE"],
  ["\u08CF", "\u08CF"],
  ["\u08D0", "\u08D0"],
  ["\u08D1", "\u08D1"],
  ["\u08D2", "\u08D2"],
  ["\u08D3", "\u08D3"], /* ARABIC SMALL LOW WAW */
  ["\u08D4", "\u08D4"], /* ARABIC SMALL HIGH WORD AR-RUB */
  ["\u08D5", "\u08D5"], /* ARABIC SMALL HIGH SAD */
  ["\u08D6", "\u08D6"], /* ARABIC SMALL HIGH AIN */
  ["\u08D7", "\u08D7"], /* ARABIC SMALL HIGH QAF */
  ["\u08D8", "\u08D8"], /* ARABIC SMALL HIGH NOON WITH KASRA */
  ["\u08D9", "\u08D9"], /* ARABIC SMALL LOW NOON WITH KASRA */
  ["\u08DA", "\u08DA"], /* ARABIC SMALL HIGH WORD ATH-THALATHA */
  ["\u08DB", "\u08DB"], /* ARABIC SMALL HIGH WORD AS-SAJDA */
  ["\u08DC", "\u08DC"], /* ARABIC SMALL HIGH WORD AN-NISF */
  ["\u08DD", "\u08DD"], /* ARABIC SMALL HIGH WORD SAKTA */
  ["\u08DE", "\u08DE"], /* ARABIC SMALL HIGH WORD QIF */
  ["\u08DF", "\u08DF"], /* ARABIC SMALL HIGH WORD WAQFA */
  ["\u08E0", "\u08E0"], /* ARABIC SMALL HIGH FOOTNOTE MARKER */
  ["\u08E1", "\u08E1"], /* ARABIC SMALL HIGH SIGN SAFHA */
  ["\u08E2", "\u08E2"], /* ARABIC DISPUTED END OF AYAH */
  ["\u08E3", "\u08E3"], /* ARABIC TURNED DAMMA BELOW */
  ["\u08E4", "\u08E4"], /* ARABIC CURLY FATHA */
  ["\u08E5", "\u08E5"], /* ARABIC CURLY DAMMA */
  ["\u08E6", "\u08E6"], /* ARABIC CURLY KASRA */
  ["\u08E7", "\u08E7"], /* ARABIC CURLY FATHATAN */
  ["\u08E8", "\u08E8"], /* ARABIC CURLY DAMMATAN */
  ["\u08E9", "\u08E9"], /* ARABIC CURLY KASRATAN */
  ["\u08EA", "\u08EA"], /* ARABIC TONE ONE DOT ABOVE */
  ["\u08EB", "\u08EB"], /* ARABIC TONE TWO DOTS ABOVE */
  ["\u08EC", "\u08EC"], /* ARABIC TONE LOOP ABOVE */
  ["\u08ED", "\u08ED"], /* ARABIC TONE ONE DOT BELOW */
  ["\u08EE", "\u08EE"], /* ARABIC TONE TWO DOTS BELOW */
  ["\u08EF", "\u08EF"], /* ARABIC TONE LOOP BELOW */
  ["\u08F0", "\u08F0"], /* ARABIC OPEN FATHATAN */
  ["\u08F1", "\u08F1"], /* ARABIC OPEN DAMMATAN */
  ["\u08F2", "\u08F2"], /* ARABIC OPEN KASRATAN */
  ["\u08F3", "\u08F3"], /* ARABIC SMALL HIGH WAW */
  ["\u08F4", "\u08F4"], /* ARABIC FATHA WITH RING */
  ["\u08F5", "\u08F5"], /* ARABIC FATHA WITH DOT ABOVE */
  ["\u08F6", "\u08F6"], /* ARABIC KASRA WITH DOT BELOW */
  ["\u08F7", "\u08F7"], /* ARABIC LEFT ARROWHEAD ABOVE */
  ["\u08F8", "\u08F8"], /* ARABIC RIGHT ARROWHEAD ABOVE */
  ["\u08F9", "\u08F9"], /* ARABIC LEFT ARROWHEAD BELOW */
  ["\u08FA", "\u08FA"], /* ARABIC RIGHT ARROWHEAD BELOW */
  ["\u08FB", "\u08FB"], /* ARABIC DOUBLE RIGHT ARROWHEAD ABOVE */
  ["\u08FC", "\u08FC"], /* ARABIC DOUBLE RIGHT ARROWHEAD ABOVE WITH DOT */
  ["\u08FD", "\u08FD"], /* ARABIC RIGHT ARROWHEAD ABOVE WITH DOT */
  ["\u08FE", "\u08FE"], /* ARABIC DAMMA WITH DOT */
  ["\u08FF", "\u08FF"], /* ARABIC MARK SIDEWAYS NOON GHUNNA */
  ["\u0900", "\u0900"], /* DEVANAGARI SIGN INVERTED CANDRABINDU */
  ["\u0901", "\u0901"], /* DEVANAGARI SIGN CANDRABINDU */
  ["\u0902", "\u0902"], /* DEVANAGARI SIGN ANUSVARA */
  ["\u0903", "\u0903"], /* DEVANAGARI SIGN VISARGA */
  ["\u0904", "\u0904"], /* DEVANAGARI LETTER SHORT A */
  ["\u0905", "\u0905"], /* DEVANAGARI LETTER A */
  ["\u0906", "\u0906"], /* DEVANAGARI LETTER AA */
  ["\u0907", "\u0907"], /* DEVANAGARI LETTER I */
  ["\u0908", "\u0908"], /* DEVANAGARI LETTER II */
  ["\u0909", "\u0909"], /* DEVANAGARI LETTER U */
  ["\u090A", "\u090A"], /* DEVANAGARI LETTER UU */
  ["\u090B", "\u090B"], /* DEVANAGARI LETTER VOCALIC R */
  ["\u090C", "\u090C"], /* DEVANAGARI LETTER VOCALIC L */
  ["\u090D", "\u090D"], /* DEVANAGARI LETTER CANDRA E */
  ["\u090E", "\u090E"], /* DEVANAGARI LETTER SHORT E */
  ["\u090F", "\u090F"], /* DEVANAGARI LETTER E */
  ["\u0910", "\u0910"], /* DEVANAGARI LETTER AI */
  ["\u0911", "\u0911"], /* DEVANAGARI LETTER CANDRA O */
  ["\u0912", "\u0912"], /* DEVANAGARI LETTER SHORT O */
  ["\u0913", "\u0913"], /* DEVANAGARI LETTER O */
  ["\u0914", "\u0914"], /* DEVANAGARI LETTER AU */
  ["\u0915", "\u0915"], /* DEVANAGARI LETTER KA */
  ["\u0916", "\u0916"], /* DEVANAGARI LETTER KHA */
  ["\u0917", "\u0917"], /* DEVANAGARI LETTER GA */
  ["\u0918", "\u0918"], /* DEVANAGARI LETTER GHA */
  ["\u0919", "\u0919"], /* DEVANAGARI LETTER NGA */
  ["\u091A", "\u091A"], /* DEVANAGARI LETTER CA */
  ["\u091B", "\u091B"], /* DEVANAGARI LETTER CHA */
  ["\u091C", "\u091C"], /* DEVANAGARI LETTER JA */
  ["\u091D", "\u091D"], /* DEVANAGARI LETTER JHA */
  ["\u091E", "\u091E"], /* DEVANAGARI LETTER NYA */
  ["\u091F", "\u091F"], /* DEVANAGARI LETTER TTA */
  ["\u0920", "\u0920"], /* DEVANAGARI LETTER TTHA */
  ["\u0921", "\u0921"], /* DEVANAGARI LETTER DDA */
  ["\u0922", "\u0922"], /* DEVANAGARI LETTER DDHA */
  ["\u0923", "\u0923"], /* DEVANAGARI LETTER NNA */
  ["\u0924", "\u0924"], /* DEVANAGARI LETTER TA */
  ["\u0925", "\u0925"], /* DEVANAGARI LETTER THA */
  ["\u0926", "\u0926"], /* DEVANAGARI LETTER DA */
  ["\u0927", "\u0927"], /* DEVANAGARI LETTER DHA */
  ["\u0928", "\u0928"], /* DEVANAGARI LETTER NA */
  ["\u0929", "\u0929"], /* DEVANAGARI LETTER NNNA */
  ["\u092A", "\u092A"], /* DEVANAGARI LETTER PA */
  ["\u092B", "\u092B"], /* DEVANAGARI LETTER PHA */
  ["\u092C", "\u092C"], /* DEVANAGARI LETTER BA */
  ["\u092D", "\u092D"], /* DEVANAGARI LETTER BHA */
  ["\u092E", "\u092E"], /* DEVANAGARI LETTER MA */
  ["\u092F", "\u092F"], /* DEVANAGARI LETTER YA */
  ["\u0930", "\u0930"], /* DEVANAGARI LETTER RA */
  ["\u0931", "\u0931"], /* DEVANAGARI LETTER RRA */
  ["\u0932", "\u0932"], /* DEVANAGARI LETTER LA */
  ["\u0933", "\u0933"], /* DEVANAGARI LETTER LLA */
  ["\u0934", "\u0934"], /* DEVANAGARI LETTER LLLA */
  ["\u0935", "\u0935"], /* DEVANAGARI LETTER VA */
  ["\u0936", "\u0936"], /* DEVANAGARI LETTER SHA */
  ["\u0937", "\u0937"], /* DEVANAGARI LETTER SSA */
  ["\u0938", "\u0938"], /* DEVANAGARI LETTER SA */
  ["\u0939", "\u0939"], /* DEVANAGARI LETTER HA */
  ["\u093A", "\u093A"], /* DEVANAGARI VOWEL SIGN OE */
  ["\u093B", "\u093B"], /* DEVANAGARI VOWEL SIGN OOE */
  ["\u093C", "\u093C"], /* DEVANAGARI SIGN NUKTA */
  ["\u093D", "\u093D"], /* DEVANAGARI SIGN AVAGRAHA */
  ["\u093E", "\u093E"], /* DEVANAGARI VOWEL SIGN AA */
  ["\u093F", "\u093F"], /* DEVANAGARI VOWEL SIGN I */
  ["\u0940", "\u0940"], /* DEVANAGARI VOWEL SIGN II */
  ["\u0941", "\u0941"], /* DEVANAGARI VOWEL SIGN U */
  ["\u0942", "\u0942"], /* DEVANAGARI VOWEL SIGN UU */
  ["\u0943", "\u0943"], /* DEVANAGARI VOWEL SIGN VOCALIC R */
  ["\u0944", "\u0944"], /* DEVANAGARI VOWEL SIGN VOCALIC RR */
  ["\u0945", "\u0945"], /* DEVANAGARI VOWEL SIGN CANDRA E */
  ["\u0946", "\u0946"], /* DEVANAGARI VOWEL SIGN SHORT E */
  ["\u0947", "\u0947"], /* DEVANAGARI VOWEL SIGN E */
  ["\u0948", "\u0948"], /* DEVANAGARI VOWEL SIGN AI */
  ["\u0949", "\u0949"], /* DEVANAGARI VOWEL SIGN CANDRA O */
  ["\u094A", "\u094A"], /* DEVANAGARI VOWEL SIGN SHORT O */
  ["\u094B", "\u094B"], /* DEVANAGARI VOWEL SIGN O */
  ["\u094C", "\u094C"], /* DEVANAGARI VOWEL SIGN AU */
  ["\u094D", "\u094D"], /* DEVANAGARI SIGN VIRAMA */
  ["\u094E", "\u094E"], /* DEVANAGARI VOWEL SIGN PRISHTHAMATRA E */
  ["\u094F", "\u094F"], /* DEVANAGARI VOWEL SIGN AW */
  ["\u0950", "\u0950"], /* DEVANAGARI OM */
  ["\u0951", "\u0951"], /* DEVANAGARI STRESS SIGN UDATTA */
  ["\u0952", "\u0952"], /* DEVANAGARI STRESS SIGN ANUDATTA */
  ["\u0953", "\u0953"], /* DEVANAGARI GRAVE ACCENT */
  ["\u0954", "\u0954"], /* DEVANAGARI ACUTE ACCENT */
  ["\u0955", "\u0955"], /* DEVANAGARI VOWEL SIGN CANDRA LONG E */
  ["\u0956", "\u0956"], /* DEVANAGARI VOWEL SIGN UE */
  ["\u0957", "\u0957"], /* DEVANAGARI VOWEL SIGN UUE */
  ["\u0958", "\u0958"], /* DEVANAGARI LETTER QA */
  ["\u0959", "\u0959"], /* DEVANAGARI LETTER KHHA */
  ["\u095A", "\u095A"], /* DEVANAGARI LETTER GHHA */
  ["\u095B", "\u095B"], /* DEVANAGARI LETTER ZA */
  ["\u095C", "\u095C"], /* DEVANAGARI LETTER DDDHA */
  ["\u095D", "\u095D"], /* DEVANAGARI LETTER RHA */
  ["\u095E", "\u095E"], /* DEVANAGARI LETTER FA */
  ["\u095F", "\u095F"], /* DEVANAGARI LETTER YYA */
  ["\u0960", "\u0960"], /* DEVANAGARI LETTER VOCALIC RR */
  ["\u0961", "\u0961"], /* DEVANAGARI LETTER VOCALIC LL */
  ["\u0962", "\u0962"], /* DEVANAGARI VOWEL SIGN VOCALIC L */
  ["\u0963", "\u0963"], /* DEVANAGARI VOWEL SIGN VOCALIC LL */
  ["\u0964", "\u0964"], /* DEVANAGARI DANDA */
  ["\u0965", "\u0965"], /* DEVANAGARI DOUBLE DANDA */
  ["\u0966", "\u0966"], /* DEVANAGARI DIGIT ZERO */
  ["\u0967", "\u0967"], /* DEVANAGARI DIGIT ONE */
  ["\u0968", "\u0968"], /* DEVANAGARI DIGIT TWO */
  ["\u0969", "\u0969"], /* DEVANAGARI DIGIT THREE */
  ["\u096A", "\u096A"], /* DEVANAGARI DIGIT FOUR */
  ["\u096B", "\u096B"], /* DEVANAGARI DIGIT FIVE */
  ["\u096C", "\u096C"], /* DEVANAGARI DIGIT SIX */
  ["\u096D", "\u096D"], /* DEVANAGARI DIGIT SEVEN */
  ["\u096E", "\u096E"], /* DEVANAGARI DIGIT EIGHT */
  ["\u096F", "\u096F"], /* DEVANAGARI DIGIT NINE */
  ["\u0970", "\u0970"], /* DEVANAGARI ABBREVIATION SIGN */
  ["\u0971", "\u0971"], /* DEVANAGARI SIGN HIGH SPACING DOT */
  ["\u0972", "\u0972"], /* DEVANAGARI LETTER CANDRA A */
  ["\u0973", "\u0973"], /* DEVANAGARI LETTER OE */
  ["\u0974", "\u0974"], /* DEVANAGARI LETTER OOE */
  ["\u0975", "\u0975"], /* DEVANAGARI LETTER AW */
  ["\u0976", "\u0976"], /* DEVANAGARI LETTER UE */
  ["\u0977", "\u0977"], /* DEVANAGARI LETTER UUE */
  ["\u0978", "\u0978"], /* DEVANAGARI LETTER MARWARI DDA */
  ["\u0979", "\u0979"], /* DEVANAGARI LETTER ZHA */
  ["\u097A", "\u097A"], /* DEVANAGARI LETTER HEAVY YA */
  ["\u097B", "\u097B"], /* DEVANAGARI LETTER GGA */
  ["\u097C", "\u097C"], /* DEVANAGARI LETTER JJA */
  ["\u097D", "\u097D"], /* DEVANAGARI LETTER GLOTTAL STOP */
  ["\u097E", "\u097E"], /* DEVANAGARI LETTER DDDA */
  ["\u097F", "\u097F"], /* DEVANAGARI LETTER BBA */
  ["\u0980", "\u0980"], /* BENGALI ANJI */
  ["\u0981", "\u0981"], /* BENGALI SIGN CANDRABINDU */
  ["\u0982", "\u0982"], /* BENGALI SIGN ANUSVARA */
  ["\u0983", "\u0983"], /* BENGALI SIGN VISARGA */
  ["\u0984", "\u0984"],
  ["\u0985", "\u0985"], /* BENGALI LETTER A */
  ["\u0986", "\u0986"], /* BENGALI LETTER AA */
  ["\u0987", "\u0987"], /* BENGALI LETTER I */
  ["\u0988", "\u0988"], /* BENGALI LETTER II */
  ["\u0989", "\u0989"], /* BENGALI LETTER U */
  ["\u098A", "\u098A"], /* BENGALI LETTER UU */
  ["\u098B", "\u098B"], /* BENGALI LETTER VOCALIC R */
  ["\u098C", "\u098C"], /* BENGALI LETTER VOCALIC L */
  ["\u098D", "\u098D"],
  ["\u098E", "\u098E"],
  ["\u098F", "\u098F"], /* BENGALI LETTER E */
  ["\u0990", "\u0990"], /* BENGALI LETTER AI */
  ["\u0991", "\u0991"],
  ["\u0992", "\u0992"],
  ["\u0993", "\u0993"], /* BENGALI LETTER O */
  ["\u0994", "\u0994"], /* BENGALI LETTER AU */
  ["\u0995", "\u0995"], /* BENGALI LETTER KA */
  ["\u0996", "\u0996"], /* BENGALI LETTER KHA */
  ["\u0997", "\u0997"], /* BENGALI LETTER GA */
  ["\u0998", "\u0998"], /* BENGALI LETTER GHA */
  ["\u0999", "\u0999"], /* BENGALI LETTER NGA */
  ["\u099A", "\u099A"], /* BENGALI LETTER CA */
  ["\u099B", "\u099B"], /* BENGALI LETTER CHA */
  ["\u099C", "\u099C"], /* BENGALI LETTER JA */
  ["\u099D", "\u099D"], /* BENGALI LETTER JHA */
  ["\u099E", "\u099E"], /* BENGALI LETTER NYA */
  ["\u099F", "\u099F"], /* BENGALI LETTER TTA */
  ["\u09A0", "\u09A0"], /* BENGALI LETTER TTHA */
  ["\u09A1", "\u09A1"], /* BENGALI LETTER DDA */
  ["\u09A2", "\u09A2"], /* BENGALI LETTER DDHA */
  ["\u09A3", "\u09A3"], /* BENGALI LETTER NNA */
  ["\u09A4", "\u09A4"], /* BENGALI LETTER TA */
  ["\u09A5", "\u09A5"], /* BENGALI LETTER THA */
  ["\u09A6", "\u09A6"], /* BENGALI LETTER DA */
  ["\u09A7", "\u09A7"], /* BENGALI LETTER DHA */
  ["\u09A8", "\u09A8"], /* BENGALI LETTER NA */
  ["\u09A9", "\u09A9"],
  ["\u09AA", "\u09AA"], /* BENGALI LETTER PA */
  ["\u09AB", "\u09AB"], /* BENGALI LETTER PHA */
  ["\u09AC", "\u09AC"], /* BENGALI LETTER BA */
  ["\u09AD", "\u09AD"], /* BENGALI LETTER BHA */
  ["\u09AE", "\u09AE"], /* BENGALI LETTER MA */
  ["\u09AF", "\u09AF"], /* BENGALI LETTER YA */
  ["\u09B0", "\u09B0"], /* BENGALI LETTER RA */
  ["\u09B1", "\u09B1"],
  ["\u09B2", "\u09B2"], /* BENGALI LETTER LA */
  ["\u09B3", "\u09B3"],
  ["\u09B4", "\u09B4"],
  ["\u09B5", "\u09B5"],
  ["\u09B6", "\u09B6"], /* BENGALI LETTER SHA */
  ["\u09B7", "\u09B7"], /* BENGALI LETTER SSA */
  ["\u09B8", "\u09B8"], /* BENGALI LETTER SA */
  ["\u09B9", "\u09B9"], /* BENGALI LETTER HA */
  ["\u09BA", "\u09BA"],
  ["\u09BB", "\u09BB"],
  ["\u09BC", "\u09BC"], /* BENGALI SIGN NUKTA */
  ["\u09BD", "\u09BD"], /* BENGALI SIGN AVAGRAHA */
  ["\u09BE", "\u09BE"], /* BENGALI VOWEL SIGN AA */
  ["\u09BF", "\u09BF"], /* BENGALI VOWEL SIGN I */
  ["\u09C0", "\u09C0"], /* BENGALI VOWEL SIGN II */
  ["\u09C1", "\u09C1"], /* BENGALI VOWEL SIGN U */
  ["\u09C2", "\u09C2"], /* BENGALI VOWEL SIGN UU */
  ["\u09C3", "\u09C3"], /* BENGALI VOWEL SIGN VOCALIC R */
  ["\u09C4", "\u09C4"], /* BENGALI VOWEL SIGN VOCALIC RR */
  ["\u09C5", "\u09C5"],
  ["\u09C6", "\u09C6"],
  ["\u09C7", "\u09C7"], /* BENGALI VOWEL SIGN E */
  ["\u09C8", "\u09C8"], /* BENGALI VOWEL SIGN AI */
  ["\u09C9", "\u09C9"],
  ["\u09CA", "\u09CA"],
  ["\u09CB", "\u09CB"], /* BENGALI VOWEL SIGN O */
  ["\u09CC", "\u09CC"], /* BENGALI VOWEL SIGN AU */
  ["\u09CD", "\u09CD"], /* BENGALI SIGN VIRAMA */
  ["\u09CE", "\u09CE"], /* BENGALI LETTER KHANDA TA */
  ["\u09CF", "\u09CF"],
  ["\u09D0", "\u09D0"],
  ["\u09D1", "\u09D1"],
  ["\u09D2", "\u09D2"],
  ["\u09D3", "\u09D3"],
  ["\u09D4", "\u09D4"],
  ["\u09D5", "\u09D5"],
  ["\u09D6", "\u09D6"],
  ["\u09D7", "\u09D7"], /* BENGALI AU LENGTH MARK */
  ["\u09D8", "\u09D8"],
  ["\u09D9", "\u09D9"],
  ["\u09DA", "\u09DA"],
  ["\u09DB", "\u09DB"],
  ["\u09DC", "\u09DC"], /* BENGALI LETTER RRA */
  ["\u09DD", "\u09DD"], /* BENGALI LETTER RHA */
  ["\u09DE", "\u09DE"],
  ["\u09DF", "\u09DF"], /* BENGALI LETTER YYA */
  ["\u09E0", "\u09E0"], /* BENGALI LETTER VOCALIC RR */
  ["\u09E1", "\u09E1"], /* BENGALI LETTER VOCALIC LL */
  ["\u09E2", "\u09E2"], /* BENGALI VOWEL SIGN VOCALIC L */
  ["\u09E3", "\u09E3"], /* BENGALI VOWEL SIGN VOCALIC LL */
  ["\u09E4", "\u09E4"],
  ["\u09E5", "\u09E5"],
  ["\u09E6", "\u09E6"], /* BENGALI DIGIT ZERO */
  ["\u09E7", "\u09E7"], /* BENGALI DIGIT ONE */
  ["\u09E8", "\u09E8"], /* BENGALI DIGIT TWO */
  ["\u09E9", "\u09E9"], /* BENGALI DIGIT THREE */
  ["\u09EA", "\u09EA"], /* BENGALI DIGIT FOUR */
  ["\u09EB", "\u09EB"], /* BENGALI DIGIT FIVE */
  ["\u09EC", "\u09EC"], /* BENGALI DIGIT SIX */
  ["\u09ED", "\u09ED"], /* BENGALI DIGIT SEVEN */
  ["\u09EE", "\u09EE"], /* BENGALI DIGIT EIGHT */
  ["\u09EF", "\u09EF"], /* BENGALI DIGIT NINE */
  ["\u09F0", "\u09F0"], /* BENGALI LETTER RA WITH MIDDLE DIAGONAL */
  ["\u09F1", "\u09F1"], /* BENGALI LETTER RA WITH LOWER DIAGONAL (BENGALI LETTER VA WITH LOWER DIAGONAL) */
  ["\u09F2", "\u09F2"], /* BENGALI RUPEE MARK */
  ["\u09F3", "\u09F3"], /* BENGALI RUPEE SIGN */
  ["\u09F4", "\u09F4"], /* BENGALI CURRENCY NUMERATOR ONE */
  ["\u09F5", "\u09F5"], /* BENGALI CURRENCY NUMERATOR TWO */
  ["\u09F6", "\u09F6"], /* BENGALI CURRENCY NUMERATOR THREE */
  ["\u09F7", "\u09F7"], /* BENGALI CURRENCY NUMERATOR FOUR */
  ["\u09F8", "\u09F8"], /* BENGALI CURRENCY NUMERATOR ONE LESS THAN THE DENOMINATOR */
  ["\u09F9", "\u09F9"], /* BENGALI CURRENCY DENOMINATOR SIXTEEN */
  ["\u09FA", "\u09FA"], /* BENGALI ISSHAR */
  ["\u09FB", "\u09FB"], /* BENGALI GANDA MARK */
  ["\u09FC", "\u09FC"], /* BENGALI LETTER VEDIC ANUSVARA */
  ["\u09FD", "\u09FD"], /* BENGALI ABBREVIATION SIGN */
  ["\u09FE", "\u09FE"], /* BENGALI SANDHI MARK */
  ["\u09FF", "\u09FF"],
  ["\u0A00", "\u0A00"],
  ["\u0A01", "\u0A01"], /* GURMUKHI SIGN ADAK BINDI */
  ["\u0A02", "\u0A02"], /* GURMUKHI SIGN BINDI */
  ["\u0A03", "\u0A03"], /* GURMUKHI SIGN VISARGA */
  ["\u0A04", "\u0A04"],
  ["\u0A05", "\u0A05"], /* GURMUKHI LETTER A */
  ["\u0A06", "\u0A06"], /* GURMUKHI LETTER AA */
  ["\u0A07", "\u0A07"], /* GURMUKHI LETTER I */
  ["\u0A08", "\u0A08"], /* GURMUKHI LETTER II */
  ["\u0A09", "\u0A09"], /* GURMUKHI LETTER U */
  ["\u0A0A", "\u0A0A"], /* GURMUKHI LETTER UU */
  ["\u0A0B", "\u0A0B"],
  ["\u0A0C", "\u0A0C"],
  ["\u0A0D", "\u0A0D"],
  ["\u0A0E", "\u0A0E"],
  ["\u0A0F", "\u0A0F"], /* GURMUKHI LETTER EE */
  ["\u0A10", "\u0A10"], /* GURMUKHI LETTER AI */
  ["\u0A11", "\u0A11"],
  ["\u0A12", "\u0A12"],
  ["\u0A13", "\u0A13"], /* GURMUKHI LETTER OO */
  ["\u0A14", "\u0A14"], /* GURMUKHI LETTER AU */
  ["\u0A15", "\u0A15"], /* GURMUKHI LETTER KA */
  ["\u0A16", "\u0A16"], /* GURMUKHI LETTER KHA */
  ["\u0A17", "\u0A17"], /* GURMUKHI LETTER GA */
  ["\u0A18", "\u0A18"], /* GURMUKHI LETTER GHA */
  ["\u0A19", "\u0A19"], /* GURMUKHI LETTER NGA */
  ["\u0A1A", "\u0A1A"], /* GURMUKHI LETTER CA */
  ["\u0A1B", "\u0A1B"], /* GURMUKHI LETTER CHA */
  ["\u0A1C", "\u0A1C"], /* GURMUKHI LETTER JA */
  ["\u0A1D", "\u0A1D"], /* GURMUKHI LETTER JHA */
  ["\u0A1E", "\u0A1E"], /* GURMUKHI LETTER NYA */
  ["\u0A1F", "\u0A1F"], /* GURMUKHI LETTER TTA */
  ["\u0A20", "\u0A20"], /* GURMUKHI LETTER TTHA */
  ["\u0A21", "\u0A21"], /* GURMUKHI LETTER DDA */
  ["\u0A22", "\u0A22"], /* GURMUKHI LETTER DDHA */
  ["\u0A23", "\u0A23"], /* GURMUKHI LETTER NNA */
  ["\u0A24", "\u0A24"], /* GURMUKHI LETTER TA */
  ["\u0A25", "\u0A25"], /* GURMUKHI LETTER THA */
  ["\u0A26", "\u0A26"], /* GURMUKHI LETTER DA */
  ["\u0A27", "\u0A27"], /* GURMUKHI LETTER DHA */
  ["\u0A28", "\u0A28"], /* GURMUKHI LETTER NA */
  ["\u0A29", "\u0A29"],
  ["\u0A2A", "\u0A2A"], /* GURMUKHI LETTER PA */
  ["\u0A2B", "\u0A2B"], /* GURMUKHI LETTER PHA */
  ["\u0A2C", "\u0A2C"], /* GURMUKHI LETTER BA */
  ["\u0A2D", "\u0A2D"], /* GURMUKHI LETTER BHA */
  ["\u0A2E", "\u0A2E"], /* GURMUKHI LETTER MA */
  ["\u0A2F", "\u0A2F"], /* GURMUKHI LETTER YA */
  ["\u0A30", "\u0A30"], /* GURMUKHI LETTER RA */
  ["\u0A31", "\u0A31"],
  ["\u0A32", "\u0A32"], /* GURMUKHI LETTER LA */
  ["\u0A33", "\u0A33"], /* GURMUKHI LETTER LLA */
  ["\u0A34", "\u0A34"],
  ["\u0A35", "\u0A35"], /* GURMUKHI LETTER VA */
  ["\u0A36", "\u0A36"], /* GURMUKHI LETTER SHA */
  ["\u0A37", "\u0A37"],
  ["\u0A38", "\u0A38"], /* GURMUKHI LETTER SA */
  ["\u0A39", "\u0A39"], /* GURMUKHI LETTER HA */
  ["\u0A3A", "\u0A3A"],
  ["\u0A3B", "\u0A3B"],
  ["\u0A3C", "\u0A3C"], /* GURMUKHI SIGN NUKTA */
  ["\u0A3D", "\u0A3D"],
  ["\u0A3E", "\u0A3E"], /* GURMUKHI VOWEL SIGN AA */
  ["\u0A3F", "\u0A3F"], /* GURMUKHI VOWEL SIGN I */
  ["\u0A40", "\u0A40"], /* GURMUKHI VOWEL SIGN II */
  ["\u0A41", "\u0A41"], /* GURMUKHI VOWEL SIGN U */
  ["\u0A42", "\u0A42"], /* GURMUKHI VOWEL SIGN UU */
  ["\u0A43", "\u0A43"],
  ["\u0A44", "\u0A44"],
  ["\u0A45", "\u0A45"],
  ["\u0A46", "\u0A46"],
  ["\u0A47", "\u0A47"], /* GURMUKHI VOWEL SIGN EE */
  ["\u0A48", "\u0A48"], /* GURMUKHI VOWEL SIGN AI */
  ["\u0A49", "\u0A49"],
  ["\u0A4A", "\u0A4A"],
  ["\u0A4B", "\u0A4B"], /* GURMUKHI VOWEL SIGN OO */
  ["\u0A4C", "\u0A4C"], /* GURMUKHI VOWEL SIGN AU */
  ["\u0A4D", "\u0A4D"], /* GURMUKHI SIGN VIRAMA */
  ["\u0A4E", "\u0A4E"],
  ["\u0A4F", "\u0A4F"],
  ["\u0A50", "\u0A50"],
  ["\u0A51", "\u0A51"], /* GURMUKHI SIGN UDAAT */
  ["\u0A52", "\u0A52"],
  ["\u0A53", "\u0A53"],
  ["\u0A54", "\u0A54"],
  ["\u0A55", "\u0A55"],
  ["\u0A56", "\u0A56"],
  ["\u0A57", "\u0A57"],
  ["\u0A58", "\u0A58"],
  ["\u0A59", "\u0A59"], /* GURMUKHI LETTER KHHA */
  ["\u0A5A", "\u0A5A"], /* GURMUKHI LETTER GHHA */
  ["\u0A5B", "\u0A5B"], /* GURMUKHI LETTER ZA */
  ["\u0A5C", "\u0A5C"], /* GURMUKHI LETTER RRA */
  ["\u0A5D", "\u0A5D"],
  ["\u0A5E", "\u0A5E"], /* GURMUKHI LETTER FA */
  ["\u0A5F", "\u0A5F"],
  ["\u0A60", "\u0A60"],
  ["\u0A61", "\u0A61"],
  ["\u0A62", "\u0A62"],
  ["\u0A63", "\u0A63"],
  ["\u0A64", "\u0A64"],
  ["\u0A65", "\u0A65"],
  ["\u0A66", "\u0A66"], /* GURMUKHI DIGIT ZERO */
  ["\u0A67", "\u0A67"], /* GURMUKHI DIGIT ONE */
  ["\u0A68", "\u0A68"], /* GURMUKHI DIGIT TWO */
  ["\u0A69", "\u0A69"], /* GURMUKHI DIGIT THREE */
  ["\u0A6A", "\u0A6A"], /* GURMUKHI DIGIT FOUR */
  ["\u0A6B", "\u0A6B"], /* GURMUKHI DIGIT FIVE */
  ["\u0A6C", "\u0A6C"], /* GURMUKHI DIGIT SIX */
  ["\u0A6D", "\u0A6D"], /* GURMUKHI DIGIT SEVEN */
  ["\u0A6E", "\u0A6E"], /* GURMUKHI DIGIT EIGHT */
  ["\u0A6F", "\u0A6F"], /* GURMUKHI DIGIT NINE */
  ["\u0A70", "\u0A70"], /* GURMUKHI TIPPI */
  ["\u0A71", "\u0A71"], /* GURMUKHI ADDAK */
  ["\u0A72", "\u0A72"], /* GURMUKHI IRI */
  ["\u0A73", "\u0A73"], /* GURMUKHI URA */
  ["\u0A74", "\u0A74"], /* GURMUKHI EK ONKAR */
  ["\u0A75", "\u0A75"], /* GURMUKHI SIGN YAKASH */
  ["\u0A76", "\u0A76"], /* GURMUKHI ABBREVIATION SIGN */
  ["\u0A77", "\u0A77"],
  ["\u0A78", "\u0A78"],
  ["\u0A79", "\u0A79"],
  ["\u0A7A", "\u0A7A"],
  ["\u0A7B", "\u0A7B"],
  ["\u0A7C", "\u0A7C"],
  ["\u0A7D", "\u0A7D"],
  ["\u0A7E", "\u0A7E"],
  ["\u0A7F", "\u0A7F"],
  ["\u0A80", "\u0A80"],
  ["\u0A81", "\u0A81"], /* GUJARATI SIGN CANDRABINDU */
  ["\u0A82", "\u0A82"], /* GUJARATI SIGN ANUSVARA */
  ["\u0A83", "\u0A83"], /* GUJARATI SIGN VISARGA */
  ["\u0A84", "\u0A84"],
  ["\u0A85", "\u0A85"], /* GUJARATI LETTER A */
  ["\u0A86", "\u0A86"], /* GUJARATI LETTER AA */
  ["\u0A87", "\u0A87"], /* GUJARATI LETTER I */
  ["\u0A88", "\u0A88"], /* GUJARATI LETTER II */
  ["\u0A89", "\u0A89"], /* GUJARATI LETTER U */
  ["\u0A8A", "\u0A8A"], /* GUJARATI LETTER UU */
  ["\u0A8B", "\u0A8B"], /* GUJARATI LETTER VOCALIC R */
  ["\u0A8C", "\u0A8C"], /* GUJARATI LETTER VOCALIC L */
  ["\u0A8D", "\u0A8D"], /* GUJARATI VOWEL CANDRA E */
  ["\u0A8E", "\u0A8E"],
  ["\u0A8F", "\u0A8F"], /* GUJARATI LETTER E */
  ["\u0A90", "\u0A90"], /* GUJARATI LETTER AI */
  ["\u0A91", "\u0A91"], /* GUJARATI VOWEL CANDRA O */
  ["\u0A92", "\u0A92"],
  ["\u0A93", "\u0A93"], /* GUJARATI LETTER O */
  ["\u0A94", "\u0A94"], /* GUJARATI LETTER AU */
  ["\u0A95", "\u0A95"], /* GUJARATI LETTER KA */
  ["\u0A96", "\u0A96"], /* GUJARATI LETTER KHA */
  ["\u0A97", "\u0A97"], /* GUJARATI LETTER GA */
  ["\u0A98", "\u0A98"], /* GUJARATI LETTER GHA */
  ["\u0A99", "\u0A99"], /* GUJARATI LETTER NGA */
  ["\u0A9A", "\u0A9A"], /* GUJARATI LETTER CA */
  ["\u0A9B", "\u0A9B"], /* GUJARATI LETTER CHA */
  ["\u0A9C", "\u0A9C"], /* GUJARATI LETTER JA */
  ["\u0A9D", "\u0A9D"], /* GUJARATI LETTER JHA */
  ["\u0A9E", "\u0A9E"], /* GUJARATI LETTER NYA */
  ["\u0A9F", "\u0A9F"], /* GUJARATI LETTER TTA */
  ["\u0AA0", "\u0AA0"], /* GUJARATI LETTER TTHA */
  ["\u0AA1", "\u0AA1"], /* GUJARATI LETTER DDA */
  ["\u0AA2", "\u0AA2"], /* GUJARATI LETTER DDHA */
  ["\u0AA3", "\u0AA3"], /* GUJARATI LETTER NNA */
  ["\u0AA4", "\u0AA4"], /* GUJARATI LETTER TA */
  ["\u0AA5", "\u0AA5"], /* GUJARATI LETTER THA */
  ["\u0AA6", "\u0AA6"], /* GUJARATI LETTER DA */
  ["\u0AA7", "\u0AA7"], /* GUJARATI LETTER DHA */
  ["\u0AA8", "\u0AA8"], /* GUJARATI LETTER NA */
  ["\u0AA9", "\u0AA9"],
  ["\u0AAA", "\u0AAA"], /* GUJARATI LETTER PA */
  ["\u0AAB", "\u0AAB"], /* GUJARATI LETTER PHA */
  ["\u0AAC", "\u0AAC"], /* GUJARATI LETTER BA */
  ["\u0AAD", "\u0AAD"], /* GUJARATI LETTER BHA */
  ["\u0AAE", "\u0AAE"], /* GUJARATI LETTER MA */
  ["\u0AAF", "\u0AAF"], /* GUJARATI LETTER YA */
  ["\u0AB0", "\u0AB0"], /* GUJARATI LETTER RA */
  ["\u0AB1", "\u0AB1"],
  ["\u0AB2", "\u0AB2"], /* GUJARATI LETTER LA */
  ["\u0AB3", "\u0AB3"], /* GUJARATI LETTER LLA */
  ["\u0AB4", "\u0AB4"],
  ["\u0AB5", "\u0AB5"], /* GUJARATI LETTER VA */
  ["\u0AB6", "\u0AB6"], /* GUJARATI LETTER SHA */
  ["\u0AB7", "\u0AB7"], /* GUJARATI LETTER SSA */
  ["\u0AB8", "\u0AB8"], /* GUJARATI LETTER SA */
  ["\u0AB9", "\u0AB9"], /* GUJARATI LETTER HA */
  ["\u0ABA", "\u0ABA"],
  ["\u0ABB", "\u0ABB"],
  ["\u0ABC", "\u0ABC"], /* GUJARATI SIGN NUKTA */
  ["\u0ABD", "\u0ABD"], /* GUJARATI SIGN AVAGRAHA */
  ["\u0ABE", "\u0ABE"], /* GUJARATI VOWEL SIGN AA */
  ["\u0ABF", "\u0ABF"], /* GUJARATI VOWEL SIGN I */
  ["\u0AC0", "\u0AC0"], /* GUJARATI VOWEL SIGN II */
  ["\u0AC1", "\u0AC1"], /* GUJARATI VOWEL SIGN U */
  ["\u0AC2", "\u0AC2"], /* GUJARATI VOWEL SIGN UU */
  ["\u0AC3", "\u0AC3"], /* GUJARATI VOWEL SIGN VOCALIC R */
  ["\u0AC4", "\u0AC4"], /* GUJARATI VOWEL SIGN VOCALIC RR */
  ["\u0AC5", "\u0AC5"], /* GUJARATI VOWEL SIGN CANDRA E */
  ["\u0AC6", "\u0AC6"],
  ["\u0AC7", "\u0AC7"], /* GUJARATI VOWEL SIGN E */
  ["\u0AC8", "\u0AC8"], /* GUJARATI VOWEL SIGN AI */
  ["\u0AC9", "\u0AC9"], /* GUJARATI VOWEL SIGN CANDRA O */
  ["\u0ACA", "\u0ACA"],
  ["\u0ACB", "\u0ACB"], /* GUJARATI VOWEL SIGN O */
  ["\u0ACC", "\u0ACC"], /* GUJARATI VOWEL SIGN AU */
  ["\u0ACD", "\u0ACD"], /* GUJARATI SIGN VIRAMA */
  ["\u0ACE", "\u0ACE"],
  ["\u0ACF", "\u0ACF"],
  ["\u0AD0", "\u0AD0"], /* GUJARATI OM */
  ["\u0AD1", "\u0AD1"],
  ["\u0AD2", "\u0AD2"],
  ["\u0AD3", "\u0AD3"],
  ["\u0AD4", "\u0AD4"],
  ["\u0AD5", "\u0AD5"],
  ["\u0AD6", "\u0AD6"],
  ["\u0AD7", "\u0AD7"],
  ["\u0AD8", "\u0AD8"],
  ["\u0AD9", "\u0AD9"],
  ["\u0ADA", "\u0ADA"],
  ["\u0ADB", "\u0ADB"],
  ["\u0ADC", "\u0ADC"],
  ["\u0ADD", "\u0ADD"],
  ["\u0ADE", "\u0ADE"],
  ["\u0ADF", "\u0ADF"],
  ["\u0AE0", "\u0AE0"], /* GUJARATI LETTER VOCALIC RR */
  ["\u0AE1", "\u0AE1"], /* GUJARATI LETTER VOCALIC LL */
  ["\u0AE2", "\u0AE2"], /* GUJARATI VOWEL SIGN VOCALIC L */
  ["\u0AE3", "\u0AE3"], /* GUJARATI VOWEL SIGN VOCALIC LL */
  ["\u0AE4", "\u0AE4"],
  ["\u0AE5", "\u0AE5"],
  ["\u0AE6", "\u0AE6"], /* GUJARATI DIGIT ZERO */
  ["\u0AE7", "\u0AE7"], /* GUJARATI DIGIT ONE */
  ["\u0AE8", "\u0AE8"], /* GUJARATI DIGIT TWO */
  ["\u0AE9", "\u0AE9"], /* GUJARATI DIGIT THREE */
  ["\u0AEA", "\u0AEA"], /* GUJARATI DIGIT FOUR */
  ["\u0AEB", "\u0AEB"], /* GUJARATI DIGIT FIVE */
  ["\u0AEC", "\u0AEC"], /* GUJARATI DIGIT SIX */
  ["\u0AED", "\u0AED"], /* GUJARATI DIGIT SEVEN */
  ["\u0AEE", "\u0AEE"], /* GUJARATI DIGIT EIGHT */
  ["\u0AEF", "\u0AEF"], /* GUJARATI DIGIT NINE */
  ["\u0AF0", "\u0AF0"], /* GUJARATI ABBREVIATION SIGN */
  ["\u0AF1", "\u0AF1"], /* GUJARATI RUPEE SIGN */
  ["\u0AF2", "\u0AF2"],
  ["\u0AF3", "\u0AF3"],
  ["\u0AF4", "\u0AF4"],
  ["\u0AF5", "\u0AF5"],
  ["\u0AF6", "\u0AF6"],
  ["\u0AF7", "\u0AF7"],
  ["\u0AF8", "\u0AF8"],
  ["\u0AF9", "\u0AF9"], /* GUJARATI LETTER ZHA */
  ["\u0AFA", "\u0AFA"], /* GUJARATI SIGN SUKUN */
  ["\u0AFB", "\u0AFB"], /* GUJARATI SIGN SHADDA */
  ["\u0AFC", "\u0AFC"], /* GUJARATI SIGN MADDAH */
  ["\u0AFD", "\u0AFD"], /* GUJARATI SIGN THREE-DOT NUKTA ABOVE */
  ["\u0AFE", "\u0AFE"], /* GUJARATI SIGN CIRCLE NUKTA ABOVE */
  ["\u0AFF", "\u0AFF"], /* GUJARATI SIGN TWO-CIRCLE NUKTA ABOVE */
  ["\u0B00", "\u0B00"],
  ["\u0B01", "\u0B01"], /* ORIYA SIGN CANDRABINDU */
  ["\u0B02", "\u0B02"], /* ORIYA SIGN ANUSVARA */
  ["\u0B03", "\u0B03"], /* ORIYA SIGN VISARGA */
  ["\u0B04", "\u0B04"],
  ["\u0B05", "\u0B05"], /* ORIYA LETTER A */
  ["\u0B06", "\u0B06"], /* ORIYA LETTER AA */
  ["\u0B07", "\u0B07"], /* ORIYA LETTER I */
  ["\u0B08", "\u0B08"], /* ORIYA LETTER II */
  ["\u0B09", "\u0B09"], /* ORIYA LETTER U */
  ["\u0B0A", "\u0B0A"], /* ORIYA LETTER UU */
  ["\u0B0B", "\u0B0B"], /* ORIYA LETTER VOCALIC R */
  ["\u0B0C", "\u0B0C"], /* ORIYA LETTER VOCALIC L */
  ["\u0B0D", "\u0B0D"],
  ["\u0B0E", "\u0B0E"],
  ["\u0B0F", "\u0B0F"], /* ORIYA LETTER E */
  ["\u0B10", "\u0B10"], /* ORIYA LETTER AI */
  ["\u0B11", "\u0B11"],
  ["\u0B12", "\u0B12"],
  ["\u0B13", "\u0B13"], /* ORIYA LETTER O */
  ["\u0B14", "\u0B14"], /* ORIYA LETTER AU */
  ["\u0B15", "\u0B15"], /* ORIYA LETTER KA */
  ["\u0B16", "\u0B16"], /* ORIYA LETTER KHA */
  ["\u0B17", "\u0B17"], /* ORIYA LETTER GA */
  ["\u0B18", "\u0B18"], /* ORIYA LETTER GHA */
  ["\u0B19", "\u0B19"], /* ORIYA LETTER NGA */
  ["\u0B1A", "\u0B1A"], /* ORIYA LETTER CA */
  ["\u0B1B", "\u0B1B"], /* ORIYA LETTER CHA */
  ["\u0B1C", "\u0B1C"], /* ORIYA LETTER JA */
  ["\u0B1D", "\u0B1D"], /* ORIYA LETTER JHA */
  ["\u0B1E", "\u0B1E"], /* ORIYA LETTER NYA */
  ["\u0B1F", "\u0B1F"], /* ORIYA LETTER TTA */
  ["\u0B20", "\u0B20"], /* ORIYA LETTER TTHA */
  ["\u0B21", "\u0B21"], /* ORIYA LETTER DDA */
  ["\u0B22", "\u0B22"], /* ORIYA LETTER DDHA */
  ["\u0B23", "\u0B23"], /* ORIYA LETTER NNA */
  ["\u0B24", "\u0B24"], /* ORIYA LETTER TA */
  ["\u0B25", "\u0B25"], /* ORIYA LETTER THA */
  ["\u0B26", "\u0B26"], /* ORIYA LETTER DA */
  ["\u0B27", "\u0B27"], /* ORIYA LETTER DHA */
  ["\u0B28", "\u0B28"], /* ORIYA LETTER NA */
  ["\u0B29", "\u0B29"],
  ["\u0B2A", "\u0B2A"], /* ORIYA LETTER PA */
  ["\u0B2B", "\u0B2B"], /* ORIYA LETTER PHA */
  ["\u0B2C", "\u0B2C"], /* ORIYA LETTER BA */
  ["\u0B2D", "\u0B2D"], /* ORIYA LETTER BHA */
  ["\u0B2E", "\u0B2E"], /* ORIYA LETTER MA */
  ["\u0B2F", "\u0B2F"], /* ORIYA LETTER YA */
  ["\u0B30", "\u0B30"], /* ORIYA LETTER RA */
  ["\u0B31", "\u0B31"],
  ["\u0B32", "\u0B32"], /* ORIYA LETTER LA */
  ["\u0B33", "\u0B33"], /* ORIYA LETTER LLA */
  ["\u0B34", "\u0B34"],
  ["\u0B35", "\u0B35"], /* ORIYA LETTER VA */
  ["\u0B36", "\u0B36"], /* ORIYA LETTER SHA */
  ["\u0B37", "\u0B37"], /* ORIYA LETTER SSA */
  ["\u0B38", "\u0B38"], /* ORIYA LETTER SA */
  ["\u0B39", "\u0B39"], /* ORIYA LETTER HA */
  ["\u0B3A", "\u0B3A"],
  ["\u0B3B", "\u0B3B"],
  ["\u0B3C", "\u0B3C"], /* ORIYA SIGN NUKTA */
  ["\u0B3D", "\u0B3D"], /* ORIYA SIGN AVAGRAHA */
  ["\u0B3E", "\u0B3E"], /* ORIYA VOWEL SIGN AA */
  ["\u0B3F", "\u0B3F"], /* ORIYA VOWEL SIGN I */
  ["\u0B40", "\u0B40"], /* ORIYA VOWEL SIGN II */
  ["\u0B41", "\u0B41"], /* ORIYA VOWEL SIGN U */
  ["\u0B42", "\u0B42"], /* ORIYA VOWEL SIGN UU */
  ["\u0B43", "\u0B43"], /* ORIYA VOWEL SIGN VOCALIC R */
  ["\u0B44", "\u0B44"], /* ORIYA VOWEL SIGN VOCALIC RR */
  ["\u0B45", "\u0B45"],
  ["\u0B46", "\u0B46"],
  ["\u0B47", "\u0B47"], /* ORIYA VOWEL SIGN E */
  ["\u0B48", "\u0B48"], /* ORIYA VOWEL SIGN AI */
  ["\u0B49", "\u0B49"],
  ["\u0B4A", "\u0B4A"],
  ["\u0B4B", "\u0B4B"], /* ORIYA VOWEL SIGN O */
  ["\u0B4C", "\u0B4C"], /* ORIYA VOWEL SIGN AU */
  ["\u0B4D", "\u0B4D"], /* ORIYA SIGN VIRAMA */
  ["\u0B4E", "\u0B4E"],
  ["\u0B4F", "\u0B4F"],
  ["\u0B50", "\u0B50"],
  ["\u0B51", "\u0B51"],
  ["\u0B52", "\u0B52"],
  ["\u0B53", "\u0B53"],
  ["\u0B54", "\u0B54"],
  ["\u0B55", "\u0B55"],
  ["\u0B56", "\u0B56"], /* ORIYA AI LENGTH MARK */
  ["\u0B57", "\u0B57"], /* ORIYA AU LENGTH MARK */
  ["\u0B58", "\u0B58"],
  ["\u0B59", "\u0B59"],
  ["\u0B5A", "\u0B5A"],
  ["\u0B5B", "\u0B5B"],
  ["\u0B5C", "\u0B5C"], /* ORIYA LETTER RRA */
  ["\u0B5D", "\u0B5D"], /* ORIYA LETTER RHA */
  ["\u0B5E", "\u0B5E"],
  ["\u0B5F", "\u0B5F"], /* ORIYA LETTER YYA */
  ["\u0B60", "\u0B60"], /* ORIYA LETTER VOCALIC RR */
  ["\u0B61", "\u0B61"], /* ORIYA LETTER VOCALIC LL */
  ["\u0B62", "\u0B62"], /* ORIYA VOWEL SIGN VOCALIC L */
  ["\u0B63", "\u0B63"], /* ORIYA VOWEL SIGN VOCALIC LL */
  ["\u0B64", "\u0B64"],
  ["\u0B65", "\u0B65"],
  ["\u0B66", "\u0B66"], /* ORIYA DIGIT ZERO */
  ["\u0B67", "\u0B67"], /* ORIYA DIGIT ONE */
  ["\u0B68", "\u0B68"], /* ORIYA DIGIT TWO */
  ["\u0B69", "\u0B69"], /* ORIYA DIGIT THREE */
  ["\u0B6A", "\u0B6A"], /* ORIYA DIGIT FOUR */
  ["\u0B6B", "\u0B6B"], /* ORIYA DIGIT FIVE */
  ["\u0B6C", "\u0B6C"], /* ORIYA DIGIT SIX */
  ["\u0B6D", "\u0B6D"], /* ORIYA DIGIT SEVEN */
  ["\u0B6E", "\u0B6E"], /* ORIYA DIGIT EIGHT */
  ["\u0B6F", "\u0B6F"], /* ORIYA DIGIT NINE */
  ["\u0B70", "\u0B70"], /* ORIYA ISSHAR */
  ["\u0B71", "\u0B71"], /* ORIYA LETTER WA */
  ["\u0B72", "\u0B72"], /* ORIYA FRACTION ONE QUARTER */
  ["\u0B73", "\u0B73"], /* ORIYA FRACTION ONE HALF */
  ["\u0B74", "\u0B74"], /* ORIYA FRACTION THREE QUARTERS */
  ["\u0B75", "\u0B75"], /* ORIYA FRACTION ONE SIXTEENTH */
  ["\u0B76", "\u0B76"], /* ORIYA FRACTION ONE EIGHTH */
  ["\u0B77", "\u0B77"], /* ORIYA FRACTION THREE SIXTEENTHS */
  ["\u0B78", "\u0B78"],
  ["\u0B79", "\u0B79"],
  ["\u0B7A", "\u0B7A"],
  ["\u0B7B", "\u0B7B"],
  ["\u0B7C", "\u0B7C"],
  ["\u0B7D", "\u0B7D"],
  ["\u0B7E", "\u0B7E"],
  ["\u0B7F", "\u0B7F"],
  ["\u0B80", "\u0B80"],
  ["\u0B81", "\u0B81"],
  ["\u0B82", "\u0B82"], /* TAMIL SIGN ANUSVARA */
  ["\u0B83", "\u0B83"], /* TAMIL SIGN VISARGA */
  ["\u0B84", "\u0B84"],
  ["\u0B85", "\u0B85"], /* TAMIL LETTER A */
  ["\u0B86", "\u0B86"], /* TAMIL LETTER AA */
  ["\u0B87", "\u0B87"], /* TAMIL LETTER I */
  ["\u0B88", "\u0B88"], /* TAMIL LETTER II */
  ["\u0B89", "\u0B89"], /* TAMIL LETTER U */
  ["\u0B8A", "\u0B8A"], /* TAMIL LETTER UU */
  ["\u0B8B", "\u0B8B"],
  ["\u0B8C", "\u0B8C"],
  ["\u0B8D", "\u0B8D"],
  ["\u0B8E", "\u0B8E"], /* TAMIL LETTER E */
  ["\u0B8F", "\u0B8F"], /* TAMIL LETTER EE */
  ["\u0B90", "\u0B90"], /* TAMIL LETTER AI */
  ["\u0B91", "\u0B91"],
  ["\u0B92", "\u0B92"], /* TAMIL LETTER O */
  ["\u0B93", "\u0B93"], /* TAMIL LETTER OO */
  ["\u0B94", "\u0B94"], /* TAMIL LETTER AU */
  ["\u0B95", "\u0B95"], /* TAMIL LETTER KA */
  ["\u0B96", "\u0B96"],
  ["\u0B97", "\u0B97"],
  ["\u0B98", "\u0B98"],
  ["\u0B99", "\u0B99"], /* TAMIL LETTER NGA */
  ["\u0B9A", "\u0B9A"], /* TAMIL LETTER CA */
  ["\u0B9B", "\u0B9B"],
  ["\u0B9C", "\u0B9C"], /* TAMIL LETTER JA */
  ["\u0B9D", "\u0B9D"],
  ["\u0B9E", "\u0B9E"], /* TAMIL LETTER NYA */
  ["\u0B9F", "\u0B9F"], /* TAMIL LETTER TTA */
  ["\u0BA0", "\u0BA0"],
  ["\u0BA1", "\u0BA1"],
  ["\u0BA2", "\u0BA2"],
  ["\u0BA3", "\u0BA3"], /* TAMIL LETTER NNA */
  ["\u0BA4", "\u0BA4"], /* TAMIL LETTER TA */
  ["\u0BA5", "\u0BA5"],
  ["\u0BA6", "\u0BA6"],
  ["\u0BA7", "\u0BA7"],
  ["\u0BA8", "\u0BA8"], /* TAMIL LETTER NA */
  ["\u0BA9", "\u0BA9"], /* TAMIL LETTER NNNA */
  ["\u0BAA", "\u0BAA"], /* TAMIL LETTER PA */
  ["\u0BAB", "\u0BAB"],
  ["\u0BAC", "\u0BAC"],
  ["\u0BAD", "\u0BAD"],
  ["\u0BAE", "\u0BAE"], /* TAMIL LETTER MA */
  ["\u0BAF", "\u0BAF"], /* TAMIL LETTER YA */
  ["\u0BB0", "\u0BB0"], /* TAMIL LETTER RA */
  ["\u0BB1", "\u0BB1"], /* TAMIL LETTER RRA */
  ["\u0BB2", "\u0BB2"], /* TAMIL LETTER LA */
  ["\u0BB3", "\u0BB3"], /* TAMIL LETTER LLA */
  ["\u0BB4", "\u0BB4"], /* TAMIL LETTER LLLA */
  ["\u0BB5", "\u0BB5"], /* TAMIL LETTER VA */
  ["\u0BB6", "\u0BB6"], /* TAMIL LETTER SHA */
  ["\u0BB7", "\u0BB7"], /* TAMIL LETTER SSA */
  ["\u0BB8", "\u0BB8"], /* TAMIL LETTER SA */
  ["\u0BB9", "\u0BB9"], /* TAMIL LETTER HA */
  ["\u0BBA", "\u0BBA"],
  ["\u0BBB", "\u0BBB"],
  ["\u0BBC", "\u0BBC"],
  ["\u0BBD", "\u0BBD"],
  ["\u0BBE", "\u0BBE"], /* TAMIL VOWEL SIGN AA */
  ["\u0BBF", "\u0BBF"], /* TAMIL VOWEL SIGN I */
  ["\u0BC0", "\u0BC0"], /* TAMIL VOWEL SIGN II */
  ["\u0BC1", "\u0BC1"], /* TAMIL VOWEL SIGN U */
  ["\u0BC2", "\u0BC2"], /* TAMIL VOWEL SIGN UU */
  ["\u0BC3", "\u0BC3"],
  ["\u0BC4", "\u0BC4"],
  ["\u0BC5", "\u0BC5"],
  ["\u0BC6", "\u0BC6"], /* TAMIL VOWEL SIGN E */
  ["\u0BC7", "\u0BC7"], /* TAMIL VOWEL SIGN EE */
  ["\u0BC8", "\u0BC8"], /* TAMIL VOWEL SIGN AI */
  ["\u0BC9", "\u0BC9"],
  ["\u0BCA", "\u0BCA"], /* TAMIL VOWEL SIGN O */
  ["\u0BCB", "\u0BCB"], /* TAMIL VOWEL SIGN OO */
  ["\u0BCC", "\u0BCC"], /* TAMIL VOWEL SIGN AU */
  ["\u0BCD", "\u0BCD"], /* TAMIL SIGN VIRAMA */
  ["\u0BCE", "\u0BCE"],
  ["\u0BCF", "\u0BCF"],
  ["\u0BD0", "\u0BD0"], /* TAMIL OM */
  ["\u0BD1", "\u0BD1"],
  ["\u0BD2", "\u0BD2"],
  ["\u0BD3", "\u0BD3"],
  ["\u0BD4", "\u0BD4"],
  ["\u0BD5", "\u0BD5"],
  ["\u0BD6", "\u0BD6"],
  ["\u0BD7", "\u0BD7"], /* TAMIL AU LENGTH MARK */
  ["\u0BD8", "\u0BD8"],
  ["\u0BD9", "\u0BD9"],
  ["\u0BDA", "\u0BDA"],
  ["\u0BDB", "\u0BDB"],
  ["\u0BDC", "\u0BDC"],
  ["\u0BDD", "\u0BDD"],
  ["\u0BDE", "\u0BDE"],
  ["\u0BDF", "\u0BDF"],
  ["\u0BE0", "\u0BE0"],
  ["\u0BE1", "\u0BE1"],
  ["\u0BE2", "\u0BE2"],
  ["\u0BE3", "\u0BE3"],
  ["\u0BE4", "\u0BE4"],
  ["\u0BE5", "\u0BE5"],
  ["\u0BE6", "\u0BE6"], /* TAMIL DIGIT ZERO */
  ["\u0BE7", "\u0BE7"], /* TAMIL DIGIT ONE */
  ["\u0BE8", "\u0BE8"], /* TAMIL DIGIT TWO */
  ["\u0BE9", "\u0BE9"], /* TAMIL DIGIT THREE */
  ["\u0BEA", "\u0BEA"], /* TAMIL DIGIT FOUR */
  ["\u0BEB", "\u0BEB"], /* TAMIL DIGIT FIVE */
  ["\u0BEC", "\u0BEC"], /* TAMIL DIGIT SIX */
  ["\u0BED", "\u0BED"], /* TAMIL DIGIT SEVEN */
  ["\u0BEE", "\u0BEE"], /* TAMIL DIGIT EIGHT */
  ["\u0BEF", "\u0BEF"], /* TAMIL DIGIT NINE */
  ["\u0BF0", "\u0BF0"], /* TAMIL NUMBER TEN */
  ["\u0BF1", "\u0BF1"], /* TAMIL NUMBER ONE HUNDRED */
  ["\u0BF2", "\u0BF2"], /* TAMIL NUMBER ONE THOUSAND */
  ["\u0BF3", "\u0BF3"], /* TAMIL DAY SIGN */
  ["\u0BF4", "\u0BF4"], /* TAMIL MONTH SIGN */
  ["\u0BF5", "\u0BF5"], /* TAMIL YEAR SIGN */
  ["\u0BF6", "\u0BF6"], /* TAMIL DEBIT SIGN */
  ["\u0BF7", "\u0BF7"], /* TAMIL CREDIT SIGN */
  ["\u0BF8", "\u0BF8"], /* TAMIL AS ABOVE SIGN */
  ["\u0BF9", "\u0BF9"], /* TAMIL RUPEE SIGN */
  ["\u0BFA", "\u0BFA"], /* TAMIL NUMBER SIGN */
  ["\u0BFB", "\u0BFB"],
  ["\u0BFC", "\u0BFC"],
  ["\u0BFD", "\u0BFD"],
  ["\u0BFE", "\u0BFE"],
  ["\u0BFF", "\u0BFF"],
  ["\u0C00", "\u0C00"], /* TELUGU SIGN COMBINING CANDRABINDU ABOVE */
  ["\u0C01", "\u0C01"], /* TELUGU SIGN CANDRABINDU */
  ["\u0C02", "\u0C02"], /* TELUGU SIGN ANUSVARA */
  ["\u0C03", "\u0C03"], /* TELUGU SIGN VISARGA */
  ["\u0C04", "\u0C04"], /* TELUGU SIGN COMBINING ANUSVARA ABOVE */
  ["\u0C05", "\u0C05"], /* TELUGU LETTER A */
  ["\u0C06", "\u0C06"], /* TELUGU LETTER AA */
  ["\u0C07", "\u0C07"], /* TELUGU LETTER I */
  ["\u0C08", "\u0C08"], /* TELUGU LETTER II */
  ["\u0C09", "\u0C09"], /* TELUGU LETTER U */
  ["\u0C0A", "\u0C0A"], /* TELUGU LETTER UU */
  ["\u0C0B", "\u0C0B"], /* TELUGU LETTER VOCALIC R */
  ["\u0C0C", "\u0C0C"], /* TELUGU LETTER VOCALIC L */
  ["\u0C0D", "\u0C0D"],
  ["\u0C0E", "\u0C0E"], /* TELUGU LETTER E */
  ["\u0C0F", "\u0C0F"], /* TELUGU LETTER EE */
  ["\u0C10", "\u0C10"], /* TELUGU LETTER AI */
  ["\u0C11", "\u0C11"],
  ["\u0C12", "\u0C12"], /* TELUGU LETTER O */
  ["\u0C13", "\u0C13"], /* TELUGU LETTER OO */
  ["\u0C14", "\u0C14"], /* TELUGU LETTER AU */
  ["\u0C15", "\u0C15"], /* TELUGU LETTER KA */
  ["\u0C16", "\u0C16"], /* TELUGU LETTER KHA */
  ["\u0C17", "\u0C17"], /* TELUGU LETTER GA */
  ["\u0C18", "\u0C18"], /* TELUGU LETTER GHA */
  ["\u0C19", "\u0C19"], /* TELUGU LETTER NGA */
  ["\u0C1A", "\u0C1A"], /* TELUGU LETTER CA */
  ["\u0C1B", "\u0C1B"], /* TELUGU LETTER CHA */
  ["\u0C1C", "\u0C1C"], /* TELUGU LETTER JA */
  ["\u0C1D", "\u0C1D"], /* TELUGU LETTER JHA */
  ["\u0C1E", "\u0C1E"], /* TELUGU LETTER NYA */
  ["\u0C1F", "\u0C1F"], /* TELUGU LETTER TTA */
  ["\u0C20", "\u0C20"], /* TELUGU LETTER TTHA */
  ["\u0C21", "\u0C21"], /* TELUGU LETTER DDA */
  ["\u0C22", "\u0C22"], /* TELUGU LETTER DDHA */
  ["\u0C23", "\u0C23"], /* TELUGU LETTER NNA */
  ["\u0C24", "\u0C24"], /* TELUGU LETTER TA */
  ["\u0C25", "\u0C25"], /* TELUGU LETTER THA */
  ["\u0C26", "\u0C26"], /* TELUGU LETTER DA */
  ["\u0C27", "\u0C27"], /* TELUGU LETTER DHA */
  ["\u0C28", "\u0C28"], /* TELUGU LETTER NA */
  ["\u0C29", "\u0C29"],
  ["\u0C2A", "\u0C2A"], /* TELUGU LETTER PA */
  ["\u0C2B", "\u0C2B"], /* TELUGU LETTER PHA */
  ["\u0C2C", "\u0C2C"], /* TELUGU LETTER BA */
  ["\u0C2D", "\u0C2D"], /* TELUGU LETTER BHA */
  ["\u0C2E", "\u0C2E"], /* TELUGU LETTER MA */
  ["\u0C2F", "\u0C2F"], /* TELUGU LETTER YA */
  ["\u0C30", "\u0C30"], /* TELUGU LETTER RA */
  ["\u0C31", "\u0C31"], /* TELUGU LETTER RRA */
  ["\u0C32", "\u0C32"], /* TELUGU LETTER LA */
  ["\u0C33", "\u0C33"], /* TELUGU LETTER LLA */
  ["\u0C34", "\u0C34"], /* TELUGU LETTER LLLA */
  ["\u0C35", "\u0C35"], /* TELUGU LETTER VA */
  ["\u0C36", "\u0C36"], /* TELUGU LETTER SHA */
  ["\u0C37", "\u0C37"], /* TELUGU LETTER SSA */
  ["\u0C38", "\u0C38"], /* TELUGU LETTER SA */
  ["\u0C39", "\u0C39"], /* TELUGU LETTER HA */
  ["\u0C3A", "\u0C3A"],
  ["\u0C3B", "\u0C3B"],
  ["\u0C3C", "\u0C3C"],
  ["\u0C3D", "\u0C3D"], /* TELUGU SIGN AVAGRAHA */
  ["\u0C3E", "\u0C3E"], /* TELUGU VOWEL SIGN AA */
  ["\u0C3F", "\u0C3F"], /* TELUGU VOWEL SIGN I */
  ["\u0C40", "\u0C40"], /* TELUGU VOWEL SIGN II */
  ["\u0C41", "\u0C41"], /* TELUGU VOWEL SIGN U */
  ["\u0C42", "\u0C42"], /* TELUGU VOWEL SIGN UU */
  ["\u0C43", "\u0C43"], /* TELUGU VOWEL SIGN VOCALIC R */
  ["\u0C44", "\u0C44"], /* TELUGU VOWEL SIGN VOCALIC RR */
  ["\u0C45", "\u0C45"],
  ["\u0C46", "\u0C46"], /* TELUGU VOWEL SIGN E */
  ["\u0C47", "\u0C47"], /* TELUGU VOWEL SIGN EE */
  ["\u0C48", "\u0C48"], /* TELUGU VOWEL SIGN AI */
  ["\u0C49", "\u0C49"],
  ["\u0C4A", "\u0C4A"], /* TELUGU VOWEL SIGN O */
  ["\u0C4B", "\u0C4B"], /* TELUGU VOWEL SIGN OO */
  ["\u0C4C", "\u0C4C"], /* TELUGU VOWEL SIGN AU */
  ["\u0C4D", "\u0C4D"], /* TELUGU SIGN VIRAMA */
  ["\u0C4E", "\u0C4E"],
  ["\u0C4F", "\u0C4F"],
  ["\u0C50", "\u0C50"],
  ["\u0C51", "\u0C51"],
  ["\u0C52", "\u0C52"],
  ["\u0C53", "\u0C53"],
  ["\u0C54", "\u0C54"],
  ["\u0C55", "\u0C55"], /* TELUGU LENGTH MARK */
  ["\u0C56", "\u0C56"], /* TELUGU AI LENGTH MARK */
  ["\u0C57", "\u0C57"],
  ["\u0C58", "\u0C58"], /* TELUGU LETTER TSA */
  ["\u0C59", "\u0C59"], /* TELUGU LETTER DZA */
  ["\u0C5A", "\u0C5A"], /* TELUGU LETTER RRRA */
  ["\u0C5B", "\u0C5B"],
  ["\u0C5C", "\u0C5C"],
  ["\u0C5D", "\u0C5D"],
  ["\u0C5E", "\u0C5E"],
  ["\u0C5F", "\u0C5F"],
  ["\u0C60", "\u0C60"], /* TELUGU LETTER VOCALIC RR */
  ["\u0C61", "\u0C61"], /* TELUGU LETTER VOCALIC LL */
  ["\u0C62", "\u0C62"], /* TELUGU VOWEL SIGN VOCALIC L */
  ["\u0C63", "\u0C63"], /* TELUGU VOWEL SIGN VOCALIC LL */
  ["\u0C64", "\u0C64"],
  ["\u0C65", "\u0C65"],
  ["\u0C66", "\u0C66"], /* TELUGU DIGIT ZERO */
  ["\u0C67", "\u0C67"], /* TELUGU DIGIT ONE */
  ["\u0C68", "\u0C68"], /* TELUGU DIGIT TWO */
  ["\u0C69", "\u0C69"], /* TELUGU DIGIT THREE */
  ["\u0C6A", "\u0C6A"], /* TELUGU DIGIT FOUR */
  ["\u0C6B", "\u0C6B"], /* TELUGU DIGIT FIVE */
  ["\u0C6C", "\u0C6C"], /* TELUGU DIGIT SIX */
  ["\u0C6D", "\u0C6D"], /* TELUGU DIGIT SEVEN */
  ["\u0C6E", "\u0C6E"], /* TELUGU DIGIT EIGHT */
  ["\u0C6F", "\u0C6F"], /* TELUGU DIGIT NINE */
  ["\u0C70", "\u0C70"],
  ["\u0C71", "\u0C71"],
  ["\u0C72", "\u0C72"],
  ["\u0C73", "\u0C73"],
  ["\u0C74", "\u0C74"],
  ["\u0C75", "\u0C75"],
  ["\u0C76", "\u0C76"],
  ["\u0C77", "\u0C77"], /* TELUGU SIGN SIDDHAM */
  ["\u0C78", "\u0C78"], /* TELUGU FRACTION DIGIT ZERO FOR ODD POWERS OF FOUR */
  ["\u0C79", "\u0C79"], /* TELUGU FRACTION DIGIT ONE FOR ODD POWERS OF FOUR */
  ["\u0C7A", "\u0C7A"], /* TELUGU FRACTION DIGIT TWO FOR ODD POWERS OF FOUR */
  ["\u0C7B", "\u0C7B"], /* TELUGU FRACTION DIGIT THREE FOR ODD POWERS OF FOUR */
  ["\u0C7C", "\u0C7C"], /* TELUGU FRACTION DIGIT ONE FOR EVEN POWERS OF FOUR */
  ["\u0C7D", "\u0C7D"], /* TELUGU FRACTION DIGIT TWO FOR EVEN POWERS OF FOUR */
  ["\u0C7E", "\u0C7E"], /* TELUGU FRACTION DIGIT THREE FOR EVEN POWERS OF FOUR */
  ["\u0C7F", "\u0C7F"], /* TELUGU SIGN TUUMU */
  ["\u0C80", "\u0C80"], /* KANNADA SIGN SPACING CANDRABINDU */
  ["\u0C81", "\u0C81"], /* KANNADA SIGN CANDRABINDU */
  ["\u0C82", "\u0C82"], /* KANNADA SIGN ANUSVARA */
  ["\u0C83", "\u0C83"], /* KANNADA SIGN VISARGA */
  ["\u0C84", "\u0C84"], /* KANNADA SIGN SIDDHAM */
  ["\u0C85", "\u0C85"], /* KANNADA LETTER A */
  ["\u0C86", "\u0C86"], /* KANNADA LETTER AA */
  ["\u0C87", "\u0C87"], /* KANNADA LETTER I */
  ["\u0C88", "\u0C88"], /* KANNADA LETTER II */
  ["\u0C89", "\u0C89"], /* KANNADA LETTER U */
  ["\u0C8A", "\u0C8A"], /* KANNADA LETTER UU */
  ["\u0C8B", "\u0C8B"], /* KANNADA LETTER VOCALIC R */
  ["\u0C8C", "\u0C8C"], /* KANNADA LETTER VOCALIC L */
  ["\u0C8D", "\u0C8D"],
  ["\u0C8E", "\u0C8E"], /* KANNADA LETTER E */
  ["\u0C8F", "\u0C8F"], /* KANNADA LETTER EE */
  ["\u0C90", "\u0C90"], /* KANNADA LETTER AI */
  ["\u0C91", "\u0C91"],
  ["\u0C92", "\u0C92"], /* KANNADA LETTER O */
  ["\u0C93", "\u0C93"], /* KANNADA LETTER OO */
  ["\u0C94", "\u0C94"], /* KANNADA LETTER AU */
  ["\u0C95", "\u0C95"], /* KANNADA LETTER KA */
  ["\u0C96", "\u0C96"], /* KANNADA LETTER KHA */
  ["\u0C97", "\u0C97"], /* KANNADA LETTER GA */
  ["\u0C98", "\u0C98"], /* KANNADA LETTER GHA */
  ["\u0C99", "\u0C99"], /* KANNADA LETTER NGA */
  ["\u0C9A", "\u0C9A"], /* KANNADA LETTER CA */
  ["\u0C9B", "\u0C9B"], /* KANNADA LETTER CHA */
  ["\u0C9C", "\u0C9C"], /* KANNADA LETTER JA */
  ["\u0C9D", "\u0C9D"], /* KANNADA LETTER JHA */
  ["\u0C9E", "\u0C9E"], /* KANNADA LETTER NYA */
  ["\u0C9F", "\u0C9F"], /* KANNADA LETTER TTA */
  ["\u0CA0", "\u0CA0"], /* KANNADA LETTER TTHA */
  ["\u0CA1", "\u0CA1"], /* KANNADA LETTER DDA */
  ["\u0CA2", "\u0CA2"], /* KANNADA LETTER DDHA */
  ["\u0CA3", "\u0CA3"], /* KANNADA LETTER NNA */
  ["\u0CA4", "\u0CA4"], /* KANNADA LETTER TA */
  ["\u0CA5", "\u0CA5"], /* KANNADA LETTER THA */
  ["\u0CA6", "\u0CA6"], /* KANNADA LETTER DA */
  ["\u0CA7", "\u0CA7"], /* KANNADA LETTER DHA */
  ["\u0CA8", "\u0CA8"], /* KANNADA LETTER NA */
  ["\u0CA9", "\u0CA9"],
  ["\u0CAA", "\u0CAA"], /* KANNADA LETTER PA */
  ["\u0CAB", "\u0CAB"], /* KANNADA LETTER PHA */
  ["\u0CAC", "\u0CAC"], /* KANNADA LETTER BA */
  ["\u0CAD", "\u0CAD"], /* KANNADA LETTER BHA */
  ["\u0CAE", "\u0CAE"], /* KANNADA LETTER MA */
  ["\u0CAF", "\u0CAF"], /* KANNADA LETTER YA */
  ["\u0CB0", "\u0CB0"], /* KANNADA LETTER RA */
  ["\u0CB1", "\u0CB1"], /* KANNADA LETTER RRA */
  ["\u0CB2", "\u0CB2"], /* KANNADA LETTER LA */
  ["\u0CB3", "\u0CB3"], /* KANNADA LETTER LLA */
  ["\u0CB4", "\u0CB4"],
  ["\u0CB5", "\u0CB5"], /* KANNADA LETTER VA */
  ["\u0CB6", "\u0CB6"], /* KANNADA LETTER SHA */
  ["\u0CB7", "\u0CB7"], /* KANNADA LETTER SSA */
  ["\u0CB8", "\u0CB8"], /* KANNADA LETTER SA */
  ["\u0CB9", "\u0CB9"], /* KANNADA LETTER HA */
  ["\u0CBA", "\u0CBA"],
  ["\u0CBB", "\u0CBB"],
  ["\u0CBC", "\u0CBC"], /* KANNADA SIGN NUKTA */
  ["\u0CBD", "\u0CBD"], /* KANNADA SIGN AVAGRAHA */
  ["\u0CBE", "\u0CBE"], /* KANNADA VOWEL SIGN AA */
  ["\u0CBF", "\u0CBF"], /* KANNADA VOWEL SIGN I */
  ["\u0CC0", "\u0CC0"], /* KANNADA VOWEL SIGN II */
  ["\u0CC1", "\u0CC1"], /* KANNADA VOWEL SIGN U */
  ["\u0CC2", "\u0CC2"], /* KANNADA VOWEL SIGN UU */
  ["\u0CC3", "\u0CC3"], /* KANNADA VOWEL SIGN VOCALIC R */
  ["\u0CC4", "\u0CC4"], /* KANNADA VOWEL SIGN VOCALIC RR */
  ["\u0CC5", "\u0CC5"],
  ["\u0CC6", "\u0CC6"], /* KANNADA VOWEL SIGN E */
  ["\u0CC7", "\u0CC7"], /* KANNADA VOWEL SIGN EE */
  ["\u0CC8", "\u0CC8"], /* KANNADA VOWEL SIGN AI */
  ["\u0CC9", "\u0CC9"],
  ["\u0CCA", "\u0CCA"], /* KANNADA VOWEL SIGN O */
  ["\u0CCB", "\u0CCB"], /* KANNADA VOWEL SIGN OO */
  ["\u0CCC", "\u0CCC"], /* KANNADA VOWEL SIGN AU */
  ["\u0CCD", "\u0CCD"], /* KANNADA SIGN VIRAMA */
  ["\u0CCE", "\u0CCE"],
  ["\u0CCF", "\u0CCF"],
  ["\u0CD0", "\u0CD0"],
  ["\u0CD1", "\u0CD1"],
  ["\u0CD2", "\u0CD2"],
  ["\u0CD3", "\u0CD3"],
  ["\u0CD4", "\u0CD4"],
  ["\u0CD5", "\u0CD5"], /* KANNADA LENGTH MARK */
  ["\u0CD6", "\u0CD6"], /* KANNADA AI LENGTH MARK */
  ["\u0CD7", "\u0CD7"],
  ["\u0CD8", "\u0CD8"],
  ["\u0CD9", "\u0CD9"],
  ["\u0CDA", "\u0CDA"],
  ["\u0CDB", "\u0CDB"],
  ["\u0CDC", "\u0CDC"],
  ["\u0CDD", "\u0CDD"],
  ["\u0CDE", "\u0CDE"], /* KANNADA LETTER FA */
  ["\u0CDF", "\u0CDF"],
  ["\u0CE0", "\u0CE0"], /* KANNADA LETTER VOCALIC RR */
  ["\u0CE1", "\u0CE1"], /* KANNADA LETTER VOCALIC LL */
  ["\u0CE2", "\u0CE2"], /* KANNADA VOWEL SIGN VOCALIC L */
  ["\u0CE3", "\u0CE3"], /* KANNADA VOWEL SIGN VOCALIC LL */
  ["\u0CE4", "\u0CE4"],
  ["\u0CE5", "\u0CE5"],
  ["\u0CE6", "\u0CE6"], /* KANNADA DIGIT ZERO */
  ["\u0CE7", "\u0CE7"], /* KANNADA DIGIT ONE */
  ["\u0CE8", "\u0CE8"], /* KANNADA DIGIT TWO */
  ["\u0CE9", "\u0CE9"], /* KANNADA DIGIT THREE */
  ["\u0CEA", "\u0CEA"], /* KANNADA DIGIT FOUR */
  ["\u0CEB", "\u0CEB"], /* KANNADA DIGIT FIVE */
  ["\u0CEC", "\u0CEC"], /* KANNADA DIGIT SIX */
  ["\u0CED", "\u0CED"], /* KANNADA DIGIT SEVEN */
  ["\u0CEE", "\u0CEE"], /* KANNADA DIGIT EIGHT */
  ["\u0CEF", "\u0CEF"], /* KANNADA DIGIT NINE */
  ["\u0CF0", "\u0CF0"],
  ["\u0CF1", "\u0CF1"], /* KANNADA SIGN JIHVAMULIYA */
  ["\u0CF2", "\u0CF2"], /* KANNADA SIGN UPADHMANIYA */
  ["\u0CF3", "\u0CF3"],
  ["\u0CF4", "\u0CF4"],
  ["\u0CF5", "\u0CF5"],
  ["\u0CF6", "\u0CF6"],
  ["\u0CF7", "\u0CF7"],
  ["\u0CF8", "\u0CF8"],
  ["\u0CF9", "\u0CF9"],
  ["\u0CFA", "\u0CFA"],
  ["\u0CFB", "\u0CFB"],
  ["\u0CFC", "\u0CFC"],
  ["\u0CFD", "\u0CFD"],
  ["\u0CFE", "\u0CFE"],
  ["\u0CFF", "\u0CFF"],
  ["\u0D00", "\u0D00"], /* MALAYALAM SIGN COMBINING ANUSVARA ABOVE */
  ["\u0D01", "\u0D01"], /* MALAYALAM SIGN CANDRABINDU */
  ["\u0D02", "\u0D02"], /* MALAYALAM SIGN ANUSVARA */
  ["\u0D03", "\u0D03"], /* MALAYALAM SIGN VISARGA */
  ["\u0D04", "\u0D04"],
  ["\u0D05", "\u0D05"], /* MALAYALAM LETTER A */
  ["\u0D06", "\u0D06"], /* MALAYALAM LETTER AA */
  ["\u0D07", "\u0D07"], /* MALAYALAM LETTER I */
  ["\u0D08", "\u0D08"], /* MALAYALAM LETTER II */
  ["\u0D09", "\u0D09"], /* MALAYALAM LETTER U */
  ["\u0D0A", "\u0D0A"], /* MALAYALAM LETTER UU */
  ["\u0D0B", "\u0D0B"], /* MALAYALAM LETTER VOCALIC R */
  ["\u0D0C", "\u0D0C"], /* MALAYALAM LETTER VOCALIC L */
  ["\u0D0D", "\u0D0D"],
  ["\u0D0E", "\u0D0E"], /* MALAYALAM LETTER E */
  ["\u0D0F", "\u0D0F"], /* MALAYALAM LETTER EE */
  ["\u0D10", "\u0D10"], /* MALAYALAM LETTER AI */
  ["\u0D11", "\u0D11"],
  ["\u0D12", "\u0D12"], /* MALAYALAM LETTER O */
  ["\u0D13", "\u0D13"], /* MALAYALAM LETTER OO */
  ["\u0D14", "\u0D14"], /* MALAYALAM LETTER AU */
  ["\u0D15", "\u0D15"], /* MALAYALAM LETTER KA */
  ["\u0D16", "\u0D16"], /* MALAYALAM LETTER KHA */
  ["\u0D17", "\u0D17"], /* MALAYALAM LETTER GA */
  ["\u0D18", "\u0D18"], /* MALAYALAM LETTER GHA */
  ["\u0D19", "\u0D19"], /* MALAYALAM LETTER NGA */
  ["\u0D1A", "\u0D1A"], /* MALAYALAM LETTER CA */
  ["\u0D1B", "\u0D1B"], /* MALAYALAM LETTER CHA */
  ["\u0D1C", "\u0D1C"], /* MALAYALAM LETTER JA */
  ["\u0D1D", "\u0D1D"], /* MALAYALAM LETTER JHA */
  ["\u0D1E", "\u0D1E"], /* MALAYALAM LETTER NYA */
  ["\u0D1F", "\u0D1F"], /* MALAYALAM LETTER TTA */
  ["\u0D20", "\u0D20"], /* MALAYALAM LETTER TTHA */
  ["\u0D21", "\u0D21"], /* MALAYALAM LETTER DDA */
  ["\u0D22", "\u0D22"], /* MALAYALAM LETTER DDHA */
  ["\u0D23", "\u0D23"], /* MALAYALAM LETTER NNA */
  ["\u0D24", "\u0D24"], /* MALAYALAM LETTER TA */
  ["\u0D25", "\u0D25"], /* MALAYALAM LETTER THA */
  ["\u0D26", "\u0D26"], /* MALAYALAM LETTER DA */
  ["\u0D27", "\u0D27"], /* MALAYALAM LETTER DHA */
  ["\u0D28", "\u0D28"], /* MALAYALAM LETTER NA */
  ["\u0D29", "\u0D29"], /* MALAYALAM LETTER NNNA */
  ["\u0D2A", "\u0D2A"], /* MALAYALAM LETTER PA */
  ["\u0D2B", "\u0D2B"], /* MALAYALAM LETTER PHA */
  ["\u0D2C", "\u0D2C"], /* MALAYALAM LETTER BA */
  ["\u0D2D", "\u0D2D"], /* MALAYALAM LETTER BHA */
  ["\u0D2E", "\u0D2E"], /* MALAYALAM LETTER MA */
  ["\u0D2F", "\u0D2F"], /* MALAYALAM LETTER YA */
  ["\u0D30", "\u0D30"], /* MALAYALAM LETTER RA */
  ["\u0D31", "\u0D31"], /* MALAYALAM LETTER RRA */
  ["\u0D32", "\u0D32"], /* MALAYALAM LETTER LA */
  ["\u0D33", "\u0D33"], /* MALAYALAM LETTER LLA */
  ["\u0D34", "\u0D34"], /* MALAYALAM LETTER LLLA */
  ["\u0D35", "\u0D35"], /* MALAYALAM LETTER VA */
  ["\u0D36", "\u0D36"], /* MALAYALAM LETTER SHA */
  ["\u0D37", "\u0D37"], /* MALAYALAM LETTER SSA */
  ["\u0D38", "\u0D38"], /* MALAYALAM LETTER SA */
  ["\u0D39", "\u0D39"], /* MALAYALAM LETTER HA */
  ["\u0D3A", "\u0D3A"], /* MALAYALAM LETTER TTTA */
  ["\u0D3B", "\u0D3B"], /* MALAYALAM SIGN VERTICAL BAR VIRAMA */
  ["\u0D3C", "\u0D3C"], /* MALAYALAM SIGN CIRCULAR VIRAMA */
  ["\u0D3D", "\u0D3D"], /* MALAYALAM SIGN AVAGRAHA */
  ["\u0D3E", "\u0D3E"], /* MALAYALAM VOWEL SIGN AA */
  ["\u0D3F", "\u0D3F"], /* MALAYALAM VOWEL SIGN I */
  ["\u0D40", "\u0D40"], /* MALAYALAM VOWEL SIGN II */
  ["\u0D41", "\u0D41"], /* MALAYALAM VOWEL SIGN U */
  ["\u0D42", "\u0D42"], /* MALAYALAM VOWEL SIGN UU */
  ["\u0D43", "\u0D43"], /* MALAYALAM VOWEL SIGN VOCALIC R */
  ["\u0D44", "\u0D44"], /* MALAYALAM VOWEL SIGN VOCALIC RR */
  ["\u0D45", "\u0D45"],
  ["\u0D46", "\u0D46"], /* MALAYALAM VOWEL SIGN E */
  ["\u0D47", "\u0D47"], /* MALAYALAM VOWEL SIGN EE */
  ["\u0D48", "\u0D48"], /* MALAYALAM VOWEL SIGN AI */
  ["\u0D49", "\u0D49"],
  ["\u0D4A", "\u0D4A"], /* MALAYALAM VOWEL SIGN O */
  ["\u0D4B", "\u0D4B"], /* MALAYALAM VOWEL SIGN OO */
  ["\u0D4C", "\u0D4C"], /* MALAYALAM VOWEL SIGN AU */
  ["\u0D4D", "\u0D4D"], /* MALAYALAM SIGN VIRAMA */
  ["\u0D4E", "\u0D4E"], /* MALAYALAM LETTER DOT REPH */
  ["\u0D4F", "\u0D4F"], /* MALAYALAM SIGN PARA */
  ["\u0D50", "\u0D50"],
  ["\u0D51", "\u0D51"],
  ["\u0D52", "\u0D52"],
  ["\u0D53", "\u0D53"],
  ["\u0D54", "\u0D54"], /* MALAYALAM LETTER CHILLU M */
  ["\u0D55", "\u0D55"], /* MALAYALAM LETTER CHILLU Y */
  ["\u0D56", "\u0D56"], /* MALAYALAM LETTER CHILLU LLL */
  ["\u0D57", "\u0D57"], /* MALAYALAM AU LENGTH MARK */
  ["\u0D58", "\u0D58"], /* MALAYALAM FRACTION ONE ONE-HUNDRED-AND-SIXTIETH */
  ["\u0D59", "\u0D59"], /* MALAYALAM FRACTION ONE FORTIETH */
  ["\u0D5A", "\u0D5A"], /* MALAYALAM FRACTION THREE EIGHTIETHS */
  ["\u0D5B", "\u0D5B"], /* MALAYALAM FRACTION ONE TWENTIETH */
  ["\u0D5C", "\u0D5C"], /* MALAYALAM FRACTION ONE TENTH */
  ["\u0D5D", "\u0D5D"], /* MALAYALAM FRACTION THREE TWENTIETHS */
  ["\u0D5E", "\u0D5E"], /* MALAYALAM FRACTION ONE FIFTH */
  ["\u0D5F", "\u0D5F"], /* MALAYALAM LETTER ARCHAIC II */
  ["\u0D60", "\u0D60"], /* MALAYALAM LETTER VOCALIC RR */
  ["\u0D61", "\u0D61"], /* MALAYALAM LETTER VOCALIC LL */
  ["\u0D62", "\u0D62"], /* MALAYALAM VOWEL SIGN VOCALIC L */
  ["\u0D63", "\u0D63"], /* MALAYALAM VOWEL SIGN VOCALIC LL */
  ["\u0D64", "\u0D64"],
  ["\u0D65", "\u0D65"],
  ["\u0D66", "\u0D66"], /* MALAYALAM DIGIT ZERO */
  ["\u0D67", "\u0D67"], /* MALAYALAM DIGIT ONE */
  ["\u0D68", "\u0D68"], /* MALAYALAM DIGIT TWO */
  ["\u0D69", "\u0D69"], /* MALAYALAM DIGIT THREE */
  ["\u0D6A", "\u0D6A"], /* MALAYALAM DIGIT FOUR */
  ["\u0D6B", "\u0D6B"], /* MALAYALAM DIGIT FIVE */
  ["\u0D6C", "\u0D6C"], /* MALAYALAM DIGIT SIX */
  ["\u0D6D", "\u0D6D"], /* MALAYALAM DIGIT SEVEN */
  ["\u0D6E", "\u0D6E"], /* MALAYALAM DIGIT EIGHT */
  ["\u0D6F", "\u0D6F"], /* MALAYALAM DIGIT NINE */
  ["\u0D70", "\u0D70"], /* MALAYALAM NUMBER TEN */
  ["\u0D71", "\u0D71"], /* MALAYALAM NUMBER ONE HUNDRED */
  ["\u0D72", "\u0D72"], /* MALAYALAM NUMBER ONE THOUSAND */
  ["\u0D73", "\u0D73"], /* MALAYALAM FRACTION ONE QUARTER */
  ["\u0D74", "\u0D74"], /* MALAYALAM FRACTION ONE HALF */
  ["\u0D75", "\u0D75"], /* MALAYALAM FRACTION THREE QUARTERS */
  ["\u0D76", "\u0D76"], /* MALAYALAM FRACTION ONE SIXTEENTH */
  ["\u0D77", "\u0D77"], /* MALAYALAM FRACTION ONE EIGHTH */
  ["\u0D78", "\u0D78"], /* MALAYALAM FRACTION THREE SIXTEENTHS */
  ["\u0D79", "\u0D79"], /* MALAYALAM DATE MARK */
  ["\u0D7A", "\u0D7A"], /* MALAYALAM LETTER CHILLU NN */
  ["\u0D7B", "\u0D7B"], /* MALAYALAM LETTER CHILLU N */
  ["\u0D7C", "\u0D7C"], /* MALAYALAM LETTER CHILLU RR */
  ["\u0D7D", "\u0D7D"], /* MALAYALAM LETTER CHILLU L */
  ["\u0D7E", "\u0D7E"], /* MALAYALAM LETTER CHILLU LL */
  ["\u0D7F", "\u0D7F"], /* MALAYALAM LETTER CHILLU K */
  ["\u0D80", "\u0D80"],
  ["\u0D81", "\u0D81"],
  ["\u0D82", "\u0D82"], /* SINHALA SIGN ANUSVARAYA */
  ["\u0D83", "\u0D83"], /* SINHALA SIGN VISARGAYA */
  ["\u0D84", "\u0D84"],
  ["\u0D85", "\u0D85"], /* SINHALA LETTER AYANNA */
  ["\u0D86", "\u0D86"], /* SINHALA LETTER AAYANNA */
  ["\u0D87", "\u0D87"], /* SINHALA LETTER AEYANNA */
  ["\u0D88", "\u0D88"], /* SINHALA LETTER AEEYANNA */
  ["\u0D89", "\u0D89"], /* SINHALA LETTER IYANNA */
  ["\u0D8A", "\u0D8A"], /* SINHALA LETTER IIYANNA */
  ["\u0D8B", "\u0D8B"], /* SINHALA LETTER UYANNA */
  ["\u0D8C", "\u0D8C"], /* SINHALA LETTER UUYANNA */
  ["\u0D8D", "\u0D8D"], /* SINHALA LETTER IRUYANNA */
  ["\u0D8E", "\u0D8E"], /* SINHALA LETTER IRUUYANNA */
  ["\u0D8F", "\u0D8F"], /* SINHALA LETTER ILUYANNA */
  ["\u0D90", "\u0D90"], /* SINHALA LETTER ILUUYANNA */
  ["\u0D91", "\u0D91"], /* SINHALA LETTER EYANNA */
  ["\u0D92", "\u0D92"], /* SINHALA LETTER EEYANNA */
  ["\u0D93", "\u0D93"], /* SINHALA LETTER AIYANNA */
  ["\u0D94", "\u0D94"], /* SINHALA LETTER OYANNA */
  ["\u0D95", "\u0D95"], /* SINHALA LETTER OOYANNA */
  ["\u0D96", "\u0D96"], /* SINHALA LETTER AUYANNA */
  ["\u0D97", "\u0D97"],
  ["\u0D98", "\u0D98"],
  ["\u0D99", "\u0D99"],
  ["\u0D9A", "\u0D9A"], /* SINHALA LETTER ALPAPRAANA KAYANNA */
  ["\u0D9B", "\u0D9B"], /* SINHALA LETTER MAHAAPRAANA KAYANNA */
  ["\u0D9C", "\u0D9C"], /* SINHALA LETTER ALPAPRAANA GAYANNA */
  ["\u0D9D", "\u0D9D"], /* SINHALA LETTER MAHAAPRAANA GAYANNA */
  ["\u0D9E", "\u0D9E"], /* SINHALA LETTER KANTAJA NAASIKYAYA */
  ["\u0D9F", "\u0D9F"], /* SINHALA LETTER SANYAKA GAYANNA */
  ["\u0DA0", "\u0DA0"], /* SINHALA LETTER ALPAPRAANA CAYANNA */
  ["\u0DA1", "\u0DA1"], /* SINHALA LETTER MAHAAPRAANA CAYANNA */
  ["\u0DA2", "\u0DA2"], /* SINHALA LETTER ALPAPRAANA JAYANNA */
  ["\u0DA3", "\u0DA3"], /* SINHALA LETTER MAHAAPRAANA JAYANNA */
  ["\u0DA4", "\u0DA4"], /* SINHALA LETTER TAALUJA NAASIKYAYA */
  ["\u0DA5", "\u0DA5"], /* SINHALA LETTER TAALUJA SANYOOGA NAAKSIKYAYA */
  ["\u0DA6", "\u0DA6"], /* SINHALA LETTER SANYAKA JAYANNA */
  ["\u0DA7", "\u0DA7"], /* SINHALA LETTER ALPAPRAANA TTAYANNA */
  ["\u0DA8", "\u0DA8"], /* SINHALA LETTER MAHAAPRAANA TTAYANNA */
  ["\u0DA9", "\u0DA9"], /* SINHALA LETTER ALPAPRAANA DDAYANNA */
  ["\u0DAA", "\u0DAA"], /* SINHALA LETTER MAHAAPRAANA DDAYANNA */
  ["\u0DAB", "\u0DAB"], /* SINHALA LETTER MUURDHAJA NAYANNA */
  ["\u0DAC", "\u0DAC"], /* SINHALA LETTER SANYAKA DDAYANNA */
  ["\u0DAD", "\u0DAD"], /* SINHALA LETTER ALPAPRAANA TAYANNA */
  ["\u0DAE", "\u0DAE"], /* SINHALA LETTER MAHAAPRAANA TAYANNA */
  ["\u0DAF", "\u0DAF"], /* SINHALA LETTER ALPAPRAANA DAYANNA */
  ["\u0DB0", "\u0DB0"], /* SINHALA LETTER MAHAAPRAANA DAYANNA */
  ["\u0DB1", "\u0DB1"], /* SINHALA LETTER DANTAJA NAYANNA */
  ["\u0DB2", "\u0DB2"],
  ["\u0DB3", "\u0DB3"], /* SINHALA LETTER SANYAKA DAYANNA */
  ["\u0DB4", "\u0DB4"], /* SINHALA LETTER ALPAPRAANA PAYANNA */
  ["\u0DB5", "\u0DB5"], /* SINHALA LETTER MAHAAPRAANA PAYANNA */
  ["\u0DB6", "\u0DB6"], /* SINHALA LETTER ALPAPRAANA BAYANNA */
  ["\u0DB7", "\u0DB7"], /* SINHALA LETTER MAHAAPRAANA BAYANNA */
  ["\u0DB8", "\u0DB8"], /* SINHALA LETTER MAYANNA */
  ["\u0DB9", "\u0DB9"], /* SINHALA LETTER AMBA BAYANNA */
  ["\u0DBA", "\u0DBA"], /* SINHALA LETTER YAYANNA */
  ["\u0DBB", "\u0DBB"], /* SINHALA LETTER RAYANNA */
  ["\u0DBC", "\u0DBC"],
  ["\u0DBD", "\u0DBD"], /* SINHALA LETTER DANTAJA LAYANNA */
  ["\u0DBE", "\u0DBE"],
  ["\u0DBF", "\u0DBF"],
  ["\u0DC0", "\u0DC0"], /* SINHALA LETTER VAYANNA */
  ["\u0DC1", "\u0DC1"], /* SINHALA LETTER TAALUJA SAYANNA */
  ["\u0DC2", "\u0DC2"], /* SINHALA LETTER MUURDHAJA SAYANNA */
  ["\u0DC3", "\u0DC3"], /* SINHALA LETTER DANTAJA SAYANNA */
  ["\u0DC4", "\u0DC4"], /* SINHALA LETTER HAYANNA */
  ["\u0DC5", "\u0DC5"], /* SINHALA LETTER MUURDHAJA LAYANNA */
  ["\u0DC6", "\u0DC6"], /* SINHALA LETTER FAYANNA */
  ["\u0DC7", "\u0DC7"],
  ["\u0DC8", "\u0DC8"],
  ["\u0DC9", "\u0DC9"],
  ["\u0DCA", "\u0DCA"], /* SINHALA SIGN AL-LAKUNA */
  ["\u0DCB", "\u0DCB"],
  ["\u0DCC", "\u0DCC"],
  ["\u0DCD", "\u0DCD"],
  ["\u0DCE", "\u0DCE"],
  ["\u0DCF", "\u0DCF"], /* SINHALA VOWEL SIGN AELA-PILLA */
  ["\u0DD0", "\u0DD0"], /* SINHALA VOWEL SIGN KETTI AEDA-PILLA */
  ["\u0DD1", "\u0DD1"], /* SINHALA VOWEL SIGN DIGA AEDA-PILLA */
  ["\u0DD2", "\u0DD2"], /* SINHALA VOWEL SIGN KETTI IS-PILLA */
  ["\u0DD3", "\u0DD3"], /* SINHALA VOWEL SIGN DIGA IS-PILLA */
  ["\u0DD4", "\u0DD4"], /* SINHALA VOWEL SIGN KETTI PAA-PILLA */
  ["\u0DD5", "\u0DD5"],
  ["\u0DD6", "\u0DD6"], /* SINHALA VOWEL SIGN DIGA PAA-PILLA */
  ["\u0DD7", "\u0DD7"],
  ["\u0DD8", "\u0DD8"], /* SINHALA VOWEL SIGN GAETTA-PILLA */
  ["\u0DD9", "\u0DD9"], /* SINHALA VOWEL SIGN KOMBUVA */
  ["\u0DDA", "\u0DDA"], /* SINHALA VOWEL SIGN DIGA KOMBUVA */
  ["\u0DDB", "\u0DDB"], /* SINHALA VOWEL SIGN KOMBU DEKA */
  ["\u0DDC", "\u0DDC"], /* SINHALA VOWEL SIGN KOMBUVA HAA AELA-PILLA */
  ["\u0DDD", "\u0DDD"], /* SINHALA VOWEL SIGN KOMBUVA HAA DIGA AELA-PILLA */
  ["\u0DDE", "\u0DDE"], /* SINHALA VOWEL SIGN KOMBUVA HAA GAYANUKITTA */
  ["\u0DDF", "\u0DDF"], /* SINHALA VOWEL SIGN GAYANUKITTA */
  ["\u0DE0", "\u0DE0"],
  ["\u0DE1", "\u0DE1"],
  ["\u0DE2", "\u0DE2"],
  ["\u0DE3", "\u0DE3"],
  ["\u0DE4", "\u0DE4"],
  ["\u0DE5", "\u0DE5"],
  ["\u0DE6", "\u0DE6"], /* SINHALA LITH DIGIT ZERO */
  ["\u0DE7", "\u0DE7"], /* SINHALA LITH DIGIT ONE */
  ["\u0DE8", "\u0DE8"], /* SINHALA LITH DIGIT TWO */
  ["\u0DE9", "\u0DE9"], /* SINHALA LITH DIGIT THREE */
  ["\u0DEA", "\u0DEA"], /* SINHALA LITH DIGIT FOUR */
  ["\u0DEB", "\u0DEB"], /* SINHALA LITH DIGIT FIVE */
  ["\u0DEC", "\u0DEC"], /* SINHALA LITH DIGIT SIX */
  ["\u0DED", "\u0DED"], /* SINHALA LITH DIGIT SEVEN */
  ["\u0DEE", "\u0DEE"], /* SINHALA LITH DIGIT EIGHT */
  ["\u0DEF", "\u0DEF"], /* SINHALA LITH DIGIT NINE */
  ["\u0DF0", "\u0DF0"],
  ["\u0DF1", "\u0DF1"],
  ["\u0DF2", "\u0DF2"], /* SINHALA VOWEL SIGN DIGA GAETTA-PILLA */
  ["\u0DF3", "\u0DF3"], /* SINHALA VOWEL SIGN DIGA GAYANUKITTA */
  ["\u0DF4", "\u0DF4"], /* SINHALA PUNCTUATION KUNDDALIYA */
  ["\u0DF5", "\u0DF5"],
  ["\u0DF6", "\u0DF6"],
  ["\u0DF7", "\u0DF7"],
  ["\u0DF8", "\u0DF8"],
  ["\u0DF9", "\u0DF9"],
  ["\u0DFA", "\u0DFA"],
  ["\u0DFB", "\u0DFB"],
  ["\u0DFC", "\u0DFC"],
  ["\u0DFD", "\u0DFD"],
  ["\u0DFE", "\u0DFE"],
  ["\u0DFF", "\u0DFF"],
  ["\u0E00", "\u0E00"],
  ["\u0E01", "\u0E01"], /* THAI CHARACTER KO KAI (THAI LETTER KO KAI) */
  ["\u0E02", "\u0E02"], /* THAI CHARACTER KHO KHAI (THAI LETTER KHO KHAI) */
  ["\u0E03", "\u0E03"], /* THAI CHARACTER KHO KHUAT (THAI LETTER KHO KHUAT) */
  ["\u0E04", "\u0E04"], /* THAI CHARACTER KHO KHWAI (THAI LETTER KHO KHWAI) */
  ["\u0E05", "\u0E05"], /* THAI CHARACTER KHO KHON (THAI LETTER KHO KHON) */
  ["\u0E06", "\u0E06"], /* THAI CHARACTER KHO RAKHANG (THAI LETTER KHO RAKHANG) */
  ["\u0E07", "\u0E07"], /* THAI CHARACTER NGO NGU (THAI LETTER NGO NGU) */
  ["\u0E08", "\u0E08"], /* THAI CHARACTER CHO CHAN (THAI LETTER CHO CHAN) */
  ["\u0E09", "\u0E09"], /* THAI CHARACTER CHO CHING (THAI LETTER CHO CHING) */
  ["\u0E0A", "\u0E0A"], /* THAI CHARACTER CHO CHANG (THAI LETTER CHO CHANG) */
  ["\u0E0B", "\u0E0B"], /* THAI CHARACTER SO SO (THAI LETTER SO SO) */
  ["\u0E0C", "\u0E0C"], /* THAI CHARACTER CHO CHOE (THAI LETTER CHO CHOE) */
  ["\u0E0D", "\u0E0D"], /* THAI CHARACTER YO YING (THAI LETTER YO YING) */
  ["\u0E0E", "\u0E0E"], /* THAI CHARACTER DO CHADA (THAI LETTER DO CHADA) */
  ["\u0E0F", "\u0E0F"], /* THAI CHARACTER TO PATAK (THAI LETTER TO PATAK) */
  ["\u0E10", "\u0E10"], /* THAI CHARACTER THO THAN (THAI LETTER THO THAN) */
  ["\u0E11", "\u0E11"], /* THAI CHARACTER THO NANGMONTHO (THAI LETTER THO NANGMONTHO) */
  ["\u0E12", "\u0E12"], /* THAI CHARACTER THO PHUTHAO (THAI LETTER THO PHUTHAO) */
  ["\u0E13", "\u0E13"], /* THAI CHARACTER NO NEN (THAI LETTER NO NEN) */
  ["\u0E14", "\u0E14"], /* THAI CHARACTER DO DEK (THAI LETTER DO DEK) */
  ["\u0E15", "\u0E15"], /* THAI CHARACTER TO TAO (THAI LETTER TO TAO) */
  ["\u0E16", "\u0E16"], /* THAI CHARACTER THO THUNG (THAI LETTER THO THUNG) */
  ["\u0E17", "\u0E17"], /* THAI CHARACTER THO THAHAN (THAI LETTER THO THAHAN) */
  ["\u0E18", "\u0E18"], /* THAI CHARACTER THO THONG (THAI LETTER THO THONG) */
  ["\u0E19", "\u0E19"], /* THAI CHARACTER NO NU (THAI LETTER NO NU) */
  ["\u0E1A", "\u0E1A"], /* THAI CHARACTER BO BAIMAI (THAI LETTER BO BAIMAI) */
  ["\u0E1B", "\u0E1B"], /* THAI CHARACTER PO PLA (THAI LETTER PO PLA) */
  ["\u0E1C", "\u0E1C"], /* THAI CHARACTER PHO PHUNG (THAI LETTER PHO PHUNG) */
  ["\u0E1D", "\u0E1D"], /* THAI CHARACTER FO FA (THAI LETTER FO FA) */
  ["\u0E1E", "\u0E1E"], /* THAI CHARACTER PHO PHAN (THAI LETTER PHO PHAN) */
  ["\u0E1F", "\u0E1F"], /* THAI CHARACTER FO FAN (THAI LETTER FO FAN) */
  ["\u0E20", "\u0E20"], /* THAI CHARACTER PHO SAMPHAO (THAI LETTER PHO SAMPHAO) */
  ["\u0E21", "\u0E21"], /* THAI CHARACTER MO MA (THAI LETTER MO MA) */
  ["\u0E22", "\u0E22"], /* THAI CHARACTER YO YAK (THAI LETTER YO YAK) */
  ["\u0E23", "\u0E23"], /* THAI CHARACTER RO RUA (THAI LETTER RO RUA) */
  ["\u0E24", "\u0E24"], /* THAI CHARACTER RU (THAI LETTER RU) */
  ["\u0E25", "\u0E25"], /* THAI CHARACTER LO LING (THAI LETTER LO LING) */
  ["\u0E26", "\u0E26"], /* THAI CHARACTER LU (THAI LETTER LU) */
  ["\u0E27", "\u0E27"], /* THAI CHARACTER WO WAEN (THAI LETTER WO WAEN) */
  ["\u0E28", "\u0E28"], /* THAI CHARACTER SO SALA (THAI LETTER SO SALA) */
  ["\u0E29", "\u0E29"], /* THAI CHARACTER SO RUSI (THAI LETTER SO RUSI) */
  ["\u0E2A", "\u0E2A"], /* THAI CHARACTER SO SUA (THAI LETTER SO SUA) */
  ["\u0E2B", "\u0E2B"], /* THAI CHARACTER HO HIP (THAI LETTER HO HIP) */
  ["\u0E2C", "\u0E2C"], /* THAI CHARACTER LO CHULA (THAI LETTER LO CHULA) */
  ["\u0E2D", "\u0E2D"], /* THAI CHARACTER O ANG (THAI LETTER O ANG) */
  ["\u0E2E", "\u0E2E"], /* THAI CHARACTER HO NOKHUK (THAI LETTER HO NOK HUK) */
  ["\u0E2F", "\u0E2F"], /* THAI CHARACTER PAIYANNOI (THAI PAI YAN NOI) */
  ["\u0E30", "\u0E30"], /* THAI CHARACTER SARA A (THAI VOWEL SIGN SARA A) */
  ["\u0E31", "\u0E31"], /* THAI CHARACTER MAI HAN-AKAT (THAI VOWEL SIGN MAI HAN-AKAT) */
  ["\u0E32", "\u0E32"], /* THAI CHARACTER SARA AA (THAI VOWEL SIGN SARA AA) */
  ["\u0E33", "\u0E33"], /* THAI CHARACTER SARA AM (THAI VOWEL SIGN SARA AM) */
  ["\u0E34", "\u0E34"], /* THAI CHARACTER SARA I (THAI VOWEL SIGN SARA I) */
  ["\u0E35", "\u0E35"], /* THAI CHARACTER SARA II (THAI VOWEL SIGN SARA II) */
  ["\u0E36", "\u0E36"], /* THAI CHARACTER SARA UE (THAI VOWEL SIGN SARA UE) */
  ["\u0E37", "\u0E37"], /* THAI CHARACTER SARA UEE (THAI VOWEL SIGN SARA UEE) */
  ["\u0E38", "\u0E38"], /* THAI CHARACTER SARA U (THAI VOWEL SIGN SARA U) */
  ["\u0E39", "\u0E39"], /* THAI CHARACTER SARA UU (THAI VOWEL SIGN SARA UU) */
  ["\u0E3A", "\u0E3A"], /* THAI CHARACTER PHINTHU (THAI VOWEL SIGN PHINTHU) */
  ["\u0E3B", "\u0E3B"],
  ["\u0E3C", "\u0E3C"],
  ["\u0E3D", "\u0E3D"],
  ["\u0E3E", "\u0E3E"],
  ["\u0E3F", "\u0E3F"], /* THAI CURRENCY SYMBOL BAHT (THAI BAHT SIGN) */
  ["\u0E40", "\u0E40"], /* THAI CHARACTER SARA E (THAI VOWEL SIGN SARA E) */
  ["\u0E41", "\u0E41"], /* THAI CHARACTER SARA AE (THAI VOWEL SIGN SARA AE) */
  ["\u0E42", "\u0E42"], /* THAI CHARACTER SARA O (THAI VOWEL SIGN SARA O) */
  ["\u0E43", "\u0E43"], /* THAI CHARACTER SARA AI MAIMUAN (THAI VOWEL SIGN SARA MAI MUAN) */
  ["\u0E44", "\u0E44"], /* THAI CHARACTER SARA AI MAIMALAI (THAI VOWEL SIGN SARA MAI MALAI) */
  ["\u0E45", "\u0E45"], /* THAI CHARACTER LAKKHANGYAO (THAI LAK KHANG YAO) */
  ["\u0E46", "\u0E46"], /* THAI CHARACTER MAIYAMOK (THAI MAI YAMOK) */
  ["\u0E47", "\u0E47"], /* THAI CHARACTER MAITAIKHU (THAI VOWEL SIGN MAI TAI KHU) */
  ["\u0E48", "\u0E48"], /* THAI CHARACTER MAI EK (THAI TONE MAI EK) */
  ["\u0E49", "\u0E49"], /* THAI CHARACTER MAI THO (THAI TONE MAI THO) */
  ["\u0E4A", "\u0E4A"], /* THAI CHARACTER MAI TRI (THAI TONE MAI TRI) */
  ["\u0E4B", "\u0E4B"], /* THAI CHARACTER MAI CHATTAWA (THAI TONE MAI CHATTAWA) */
  ["\u0E4C", "\u0E4C"], /* THAI CHARACTER THANTHAKHAT (THAI THANTHAKHAT) */
  ["\u0E4D", "\u0E4D"], /* THAI CHARACTER NIKHAHIT (THAI NIKKHAHIT) */
  ["\u0E4E", "\u0E4E"], /* THAI CHARACTER YAMAKKAN (THAI YAMAKKAN) */
  ["\u0E4F", "\u0E4F"], /* THAI CHARACTER FONGMAN (THAI FONGMAN) */
  ["\u0E50", "\u0E50"], /* THAI DIGIT ZERO */
  ["\u0E51", "\u0E51"], /* THAI DIGIT ONE */
  ["\u0E52", "\u0E52"], /* THAI DIGIT TWO */
  ["\u0E53", "\u0E53"], /* THAI DIGIT THREE */
  ["\u0E54", "\u0E54"], /* THAI DIGIT FOUR */
  ["\u0E55", "\u0E55"], /* THAI DIGIT FIVE */
  ["\u0E56", "\u0E56"], /* THAI DIGIT SIX */
  ["\u0E57", "\u0E57"], /* THAI DIGIT SEVEN */
  ["\u0E58", "\u0E58"], /* THAI DIGIT EIGHT */
  ["\u0E59", "\u0E59"], /* THAI DIGIT NINE */
  ["\u0E5A", "\u0E5A"], /* THAI CHARACTER ANGKHANKHU (THAI ANGKHANKHU) */
  ["\u0E5B", "\u0E5B"], /* THAI CHARACTER KHOMUT (THAI KHOMUT) */
  ["\u0E5C", "\u0E5C"],
  ["\u0E5D", "\u0E5D"],
  ["\u0E5E", "\u0E5E"],
  ["\u0E5F", "\u0E5F"],
  ["\u0E60", "\u0E60"],
  ["\u0E61", "\u0E61"],
  ["\u0E62", "\u0E62"],
  ["\u0E63", "\u0E63"],
  ["\u0E64", "\u0E64"],
  ["\u0E65", "\u0E65"],
  ["\u0E66", "\u0E66"],
  ["\u0E67", "\u0E67"],
  ["\u0E68", "\u0E68"],
  ["\u0E69", "\u0E69"],
  ["\u0E6A", "\u0E6A"],
  ["\u0E6B", "\u0E6B"],
  ["\u0E6C", "\u0E6C"],
  ["\u0E6D", "\u0E6D"],
  ["\u0E6E", "\u0E6E"],
  ["\u0E6F", "\u0E6F"],
  ["\u0E70", "\u0E70"],
  ["\u0E71", "\u0E71"],
  ["\u0E72", "\u0E72"],
  ["\u0E73", "\u0E73"],
  ["\u0E74", "\u0E74"],
  ["\u0E75", "\u0E75"],
  ["\u0E76", "\u0E76"],
  ["\u0E77", "\u0E77"],
  ["\u0E78", "\u0E78"],
  ["\u0E79", "\u0E79"],
  ["\u0E7A", "\u0E7A"],
  ["\u0E7B", "\u0E7B"],
  ["\u0E7C", "\u0E7C"],
  ["\u0E7D", "\u0E7D"],
  ["\u0E7E", "\u0E7E"],
  ["\u0E7F", "\u0E7F"],
  ["\u0E80", "\u0E80"],
  ["\u0E81", "\u0E81"], /* LAO LETTER KO */
  ["\u0E82", "\u0E82"], /* LAO LETTER KHO SUNG */
  ["\u0E83", "\u0E83"],
  ["\u0E84", "\u0E84"], /* LAO LETTER KHO TAM */
  ["\u0E85", "\u0E85"],
  ["\u0E86", "\u0E86"], /* LAO LETTER PALI GHA */
  ["\u0E87", "\u0E87"], /* LAO LETTER NGO */
  ["\u0E88", "\u0E88"], /* LAO LETTER CO */
  ["\u0E89", "\u0E89"], /* LAO LETTER PALI CHA */
  ["\u0E8A", "\u0E8A"], /* LAO LETTER SO TAM */
  ["\u0E8B", "\u0E8B"],
  ["\u0E8C", "\u0E8C"], /* LAO LETTER PALI JHA */
  ["\u0E8D", "\u0E8D"], /* LAO LETTER NYO */
  ["\u0E8E", "\u0E8E"], /* LAO LETTER PALI NYA */
  ["\u0E8F", "\u0E8F"], /* LAO LETTER PALI TTA */
  ["\u0E90", "\u0E90"], /* LAO LETTER PALI TTHA */
  ["\u0E91", "\u0E91"], /* LAO LETTER PALI DDA */
  ["\u0E92", "\u0E92"], /* LAO LETTER PALI DDHA */
  ["\u0E93", "\u0E93"], /* LAO LETTER PALI NNA */
  ["\u0E94", "\u0E94"], /* LAO LETTER DO */
  ["\u0E95", "\u0E95"], /* LAO LETTER TO */
  ["\u0E96", "\u0E96"], /* LAO LETTER THO SUNG */
  ["\u0E97", "\u0E97"], /* LAO LETTER THO TAM */
  ["\u0E98", "\u0E98"], /* LAO LETTER PALI DHA */
  ["\u0E99", "\u0E99"], /* LAO LETTER NO */
  ["\u0E9A", "\u0E9A"], /* LAO LETTER BO */
  ["\u0E9B", "\u0E9B"], /* LAO LETTER PO */
  ["\u0E9C", "\u0E9C"], /* LAO LETTER PHO SUNG */
  ["\u0E9D", "\u0E9D"], /* LAO LETTER FO TAM */
  ["\u0E9E", "\u0E9E"], /* LAO LETTER PHO TAM */
  ["\u0E9F", "\u0E9F"], /* LAO LETTER FO SUNG */
  ["\u0EA0", "\u0EA0"], /* LAO LETTER PALI BHA */
  ["\u0EA1", "\u0EA1"], /* LAO LETTER MO */
  ["\u0EA2", "\u0EA2"], /* LAO LETTER YO */
  ["\u0EA3", "\u0EA3"], /* LAO LETTER LO LING */
  ["\u0EA4", "\u0EA4"],
  ["\u0EA5", "\u0EA5"], /* LAO LETTER LO LOOT */
  ["\u0EA6", "\u0EA6"],
  ["\u0EA7", "\u0EA7"], /* LAO LETTER WO */
  ["\u0EA8", "\u0EA8"], /* LAO LETTER SANSKRIT SHA */
  ["\u0EA9", "\u0EA9"], /* LAO LETTER SANSKRIT SSA */
  ["\u0EAA", "\u0EAA"], /* LAO LETTER SO SUNG */
  ["\u0EAB", "\u0EAB"], /* LAO LETTER HO SUNG */
  ["\u0EAC", "\u0EAC"], /* LAO LETTER PALI LLA */
  ["\u0EAD", "\u0EAD"], /* LAO LETTER O */
  ["\u0EAE", "\u0EAE"], /* LAO LETTER HO TAM */
  ["\u0EAF", "\u0EAF"], /* LAO ELLIPSIS */
  ["\u0EB0", "\u0EB0"], /* LAO VOWEL SIGN A */
  ["\u0EB1", "\u0EB1"], /* LAO VOWEL SIGN MAI KAN */
  ["\u0EB2", "\u0EB2"], /* LAO VOWEL SIGN AA */
  ["\u0EB3", "\u0EB3"], /* LAO VOWEL SIGN AM */
  ["\u0EB4", "\u0EB4"], /* LAO VOWEL SIGN I */
  ["\u0EB5", "\u0EB5"], /* LAO VOWEL SIGN II */
  ["\u0EB6", "\u0EB6"], /* LAO VOWEL SIGN Y */
  ["\u0EB7", "\u0EB7"], /* LAO VOWEL SIGN YY */
  ["\u0EB8", "\u0EB8"], /* LAO VOWEL SIGN U */
  ["\u0EB9", "\u0EB9"], /* LAO VOWEL SIGN UU */
  ["\u0EBA", "\u0EBA"], /* LAO SIGN PALI VIRAMA */
  ["\u0EBB", "\u0EBB"], /* LAO VOWEL SIGN MAI KON */
  ["\u0EBC", "\u0EBC"], /* LAO SEMIVOWEL SIGN LO */
  ["\u0EBD", "\u0EBD"], /* LAO SEMIVOWEL SIGN NYO */
  ["\u0EBE", "\u0EBE"],
  ["\u0EBF", "\u0EBF"],
  ["\u0EC0", "\u0EC0"], /* LAO VOWEL SIGN E */
  ["\u0EC1", "\u0EC1"], /* LAO VOWEL SIGN EI */
  ["\u0EC2", "\u0EC2"], /* LAO VOWEL SIGN O */
  ["\u0EC3", "\u0EC3"], /* LAO VOWEL SIGN AY */
  ["\u0EC4", "\u0EC4"], /* LAO VOWEL SIGN AI */
  ["\u0EC5", "\u0EC5"],
  ["\u0EC6", "\u0EC6"], /* LAO KO LA */
  ["\u0EC7", "\u0EC7"],
  ["\u0EC8", "\u0EC8"], /* LAO TONE MAI EK */
  ["\u0EC9", "\u0EC9"], /* LAO TONE MAI THO */
  ["\u0ECA", "\u0ECA"], /* LAO TONE MAI TI */
  ["\u0ECB", "\u0ECB"], /* LAO TONE MAI CATAWA */
  ["\u0ECC", "\u0ECC"], /* LAO CANCELLATION MARK */
  ["\u0ECD", "\u0ECD"], /* LAO NIGGAHITA */
  ["\u0ECE", "\u0ECE"],
  ["\u0ECF", "\u0ECF"],
  ["\u0ED0", "\u0ED0"], /* LAO DIGIT ZERO */
  ["\u0ED1", "\u0ED1"], /* LAO DIGIT ONE */
  ["\u0ED2", "\u0ED2"], /* LAO DIGIT TWO */
  ["\u0ED3", "\u0ED3"], /* LAO DIGIT THREE */
  ["\u0ED4", "\u0ED4"], /* LAO DIGIT FOUR */
  ["\u0ED5", "\u0ED5"], /* LAO DIGIT FIVE */
  ["\u0ED6", "\u0ED6"], /* LAO DIGIT SIX */
  ["\u0ED7", "\u0ED7"], /* LAO DIGIT SEVEN */
  ["\u0ED8", "\u0ED8"], /* LAO DIGIT EIGHT */
  ["\u0ED9", "\u0ED9"], /* LAO DIGIT NINE */
  ["\u0EDA", "\u0EDA"],
  ["\u0EDB", "\u0EDB"],
  ["\u0EDC", "\u0EDC"], /* LAO HO NO */
  ["\u0EDD", "\u0EDD"], /* LAO HO MO */
  ["\u0EDE", "\u0EDE"], /* LAO LETTER KHMU GO */
  ["\u0EDF", "\u0EDF"], /* LAO LETTER KHMU NYO */
  ["\u0EE0", "\u0EE0"],
  ["\u0EE1", "\u0EE1"],
  ["\u0EE2", "\u0EE2"],
  ["\u0EE3", "\u0EE3"],
  ["\u0EE4", "\u0EE4"],
  ["\u0EE5", "\u0EE5"],
  ["\u0EE6", "\u0EE6"],
  ["\u0EE7", "\u0EE7"],
  ["\u0EE8", "\u0EE8"],
  ["\u0EE9", "\u0EE9"],
  ["\u0EEA", "\u0EEA"],
  ["\u0EEB", "\u0EEB"],
  ["\u0EEC", "\u0EEC"],
  ["\u0EED", "\u0EED"],
  ["\u0EEE", "\u0EEE"],
  ["\u0EEF", "\u0EEF"],
  ["\u0EF0", "\u0EF0"],
  ["\u0EF1", "\u0EF1"],
  ["\u0EF2", "\u0EF2"],
  ["\u0EF3", "\u0EF3"],
  ["\u0EF4", "\u0EF4"],
  ["\u0EF5", "\u0EF5"],
  ["\u0EF6", "\u0EF6"],
  ["\u0EF7", "\u0EF7"],
  ["\u0EF8", "\u0EF8"],
  ["\u0EF9", "\u0EF9"],
  ["\u0EFA", "\u0EFA"],
  ["\u0EFB", "\u0EFB"],
  ["\u0EFC", "\u0EFC"],
  ["\u0EFD", "\u0EFD"],
  ["\u0EFE", "\u0EFE"],
  ["\u0EFF", "\u0EFF"],
  ["\u0F00", "\u0F00"], /* TIBETAN SYLLABLE OM */
  ["\u0F01", "\u0F01"], /* TIBETAN MARK GTER YIG MGO TRUNCATED A */
  ["\u0F02", "\u0F02"], /* TIBETAN MARK GTER YIG MGO -UM RNAM BCAD MA */
  ["\u0F03", "\u0F03"], /* TIBETAN MARK GTER YIG MGO -UM GTER TSHEG MA */
  ["\u0F04", "\u0F04"], /* TIBETAN MARK INITIAL YIG MGO MDUN MA (TIBETAN SINGLE ORNAMENT) */
  ["\u0F05", "\u0F05"], /* TIBETAN MARK CLOSING YIG MGO SGAB MA */
  ["\u0F06", "\u0F06"], /* TIBETAN MARK CARET YIG MGO PHUR SHAD MA */
  ["\u0F07", "\u0F07"], /* TIBETAN MARK YIG MGO TSHEG SHAD MA */
  ["\u0F08", "\u0F08"], /* TIBETAN MARK SBRUL SHAD (TIBETAN RGYANSHAD) */
  ["\u0F09", "\u0F09"], /* TIBETAN MARK BSKUR YIG MGO */
  ["\u0F0A", "\u0F0A"], /* TIBETAN MARK BKA- SHOG YIG MGO */
  ["\u0F0B", "\u0F0B"], /* TIBETAN MARK INTERSYLLABIC TSHEG (TIBETAN TSEG) */
  ["\u0F0C", "\u0F0C"], /* TIBETAN MARK DELIMITER TSHEG BSTAR */
  ["\u0F0D", "\u0F0D"], /* TIBETAN MARK SHAD (TIBETAN SHAD) */
  ["\u0F0E", "\u0F0E"], /* TIBETAN MARK NYIS SHAD (TIBETAN DOUBLE SHAD) */
  ["\u0F0F", "\u0F0F"], /* TIBETAN MARK TSHEG SHAD */
  ["\u0F10", "\u0F10"], /* TIBETAN MARK NYIS TSHEG SHAD */
  ["\u0F11", "\u0F11"], /* TIBETAN MARK RIN CHEN SPUNGS SHAD (TIBETAN RINCHANPHUNGSHAD) */
  ["\u0F12", "\u0F12"], /* TIBETAN MARK RGYA GRAM SHAD */
  ["\u0F13", "\u0F13"], /* TIBETAN MARK CARET -DZUD RTAGS ME LONG CAN */
  ["\u0F14", "\u0F14"], /* TIBETAN MARK GTER TSHEG (TIBETAN COMMA) */
  ["\u0F15", "\u0F15"], /* TIBETAN LOGOTYPE SIGN CHAD RTAGS */
  ["\u0F16", "\u0F16"], /* TIBETAN LOGOTYPE SIGN LHAG RTAGS */
  ["\u0F17", "\u0F17"], /* TIBETAN ASTROLOGICAL SIGN SGRA GCAN -CHAR RTAGS */
  ["\u0F18", "\u0F18"], /* TIBETAN ASTROLOGICAL SIGN -KHYUD PA */
  ["\u0F19", "\u0F19"], /* TIBETAN ASTROLOGICAL SIGN SDONG TSHUGS */
  ["\u0F1A", "\u0F1A"], /* TIBETAN SIGN RDEL DKAR GCIG */
  ["\u0F1B", "\u0F1B"], /* TIBETAN SIGN RDEL DKAR GNYIS */
  ["\u0F1C", "\u0F1C"], /* TIBETAN SIGN RDEL DKAR GSUM */
  ["\u0F1D", "\u0F1D"], /* TIBETAN SIGN RDEL NAG GCIG */
  ["\u0F1E", "\u0F1E"], /* TIBETAN SIGN RDEL NAG GNYIS */
  ["\u0F1F", "\u0F1F"], /* TIBETAN SIGN RDEL DKAR RDEL NAG */
  ["\u0F20", "\u0F20"], /* TIBETAN DIGIT ZERO */
  ["\u0F21", "\u0F21"], /* TIBETAN DIGIT ONE */
  ["\u0F22", "\u0F22"], /* TIBETAN DIGIT TWO */
  ["\u0F23", "\u0F23"], /* TIBETAN DIGIT THREE */
  ["\u0F24", "\u0F24"], /* TIBETAN DIGIT FOUR */
  ["\u0F25", "\u0F25"], /* TIBETAN DIGIT FIVE */
  ["\u0F26", "\u0F26"], /* TIBETAN DIGIT SIX */
  ["\u0F27", "\u0F27"], /* TIBETAN DIGIT SEVEN */
  ["\u0F28", "\u0F28"], /* TIBETAN DIGIT EIGHT */
  ["\u0F29", "\u0F29"], /* TIBETAN DIGIT NINE */
  ["\u0F2A", "\u0F2A"], /* TIBETAN DIGIT HALF ONE */
  ["\u0F2B", "\u0F2B"], /* TIBETAN DIGIT HALF TWO */
  ["\u0F2C", "\u0F2C"], /* TIBETAN DIGIT HALF THREE */
  ["\u0F2D", "\u0F2D"], /* TIBETAN DIGIT HALF FOUR */
  ["\u0F2E", "\u0F2E"], /* TIBETAN DIGIT HALF FIVE */
  ["\u0F2F", "\u0F2F"], /* TIBETAN DIGIT HALF SIX */
  ["\u0F30", "\u0F30"], /* TIBETAN DIGIT HALF SEVEN */
  ["\u0F31", "\u0F31"], /* TIBETAN DIGIT HALF EIGHT */
  ["\u0F32", "\u0F32"], /* TIBETAN DIGIT HALF NINE */
  ["\u0F33", "\u0F33"], /* TIBETAN DIGIT HALF ZERO */
  ["\u0F34", "\u0F34"], /* TIBETAN MARK BSDUS RTAGS */
  ["\u0F35", "\u0F35"], /* TIBETAN MARK NGAS BZUNG NYI ZLA (TIBETAN HONORIFIC UNDER RING) */
  ["\u0F36", "\u0F36"], /* TIBETAN MARK CARET -DZUD RTAGS BZHI MIG CAN */
  ["\u0F37", "\u0F37"], /* TIBETAN MARK NGAS BZUNG SGOR RTAGS (TIBETAN UNDER RING) */
  ["\u0F38", "\u0F38"], /* TIBETAN MARK CHE MGO */
  ["\u0F39", "\u0F39"], /* TIBETAN MARK TSA -PHRU (TIBETAN LENITION MARK) */
  ["\u0F3A", "\u0F3A"], /* TIBETAN MARK GUG RTAGS GYON */
  ["\u0F3B", "\u0F3B"], /* TIBETAN MARK GUG RTAGS GYAS */
  ["\u0F3C", "\u0F3C"], /* TIBETAN MARK ANG KHANG GYON (TIBETAN LEFT BRACE) */
  ["\u0F3D", "\u0F3D"], /* TIBETAN MARK ANG KHANG GYAS (TIBETAN RIGHT BRACE) */
  ["\u0F3E", "\u0F3E"], /* TIBETAN SIGN YAR TSHES */
  ["\u0F3F", "\u0F3F"], /* TIBETAN SIGN MAR TSHES */
  ["\u0F40", "\u0F40"], /* TIBETAN LETTER KA */
  ["\u0F41", "\u0F41"], /* TIBETAN LETTER KHA */
  ["\u0F42", "\u0F42"], /* TIBETAN LETTER GA */
  ["\u0F43", "\u0F43"], /* TIBETAN LETTER GHA */
  ["\u0F44", "\u0F44"], /* TIBETAN LETTER NGA */
  ["\u0F45", "\u0F45"], /* TIBETAN LETTER CA */
  ["\u0F46", "\u0F46"], /* TIBETAN LETTER CHA */
  ["\u0F47", "\u0F47"], /* TIBETAN LETTER JA */
  ["\u0F48", "\u0F48"],
  ["\u0F49", "\u0F49"], /* TIBETAN LETTER NYA */
  ["\u0F4A", "\u0F4A"], /* TIBETAN LETTER TTA (TIBETAN LETTER REVERSED TA) */
  ["\u0F4B", "\u0F4B"], /* TIBETAN LETTER TTHA (TIBETAN LETTER REVERSED THA) */
  ["\u0F4C", "\u0F4C"], /* TIBETAN LETTER DDA (TIBETAN LETTER REVERSED DA) */
  ["\u0F4D", "\u0F4D"], /* TIBETAN LETTER DDHA */
  ["\u0F4E", "\u0F4E"], /* TIBETAN LETTER NNA (TIBETAN LETTER REVERSED NA) */
  ["\u0F4F", "\u0F4F"], /* TIBETAN LETTER TA */
  ["\u0F50", "\u0F50"], /* TIBETAN LETTER THA */
  ["\u0F51", "\u0F51"], /* TIBETAN LETTER DA */
  ["\u0F52", "\u0F52"], /* TIBETAN LETTER DHA */
  ["\u0F53", "\u0F53"], /* TIBETAN LETTER NA */
  ["\u0F54", "\u0F54"], /* TIBETAN LETTER PA */
  ["\u0F55", "\u0F55"], /* TIBETAN LETTER PHA */
  ["\u0F56", "\u0F56"], /* TIBETAN LETTER BA */
  ["\u0F57", "\u0F57"], /* TIBETAN LETTER BHA */
  ["\u0F58", "\u0F58"], /* TIBETAN LETTER MA */
  ["\u0F59", "\u0F59"], /* TIBETAN LETTER TSA */
  ["\u0F5A", "\u0F5A"], /* TIBETAN LETTER TSHA */
  ["\u0F5B", "\u0F5B"], /* TIBETAN LETTER DZA */
  ["\u0F5C", "\u0F5C"], /* TIBETAN LETTER DZHA */
  ["\u0F5D", "\u0F5D"], /* TIBETAN LETTER WA */
  ["\u0F5E", "\u0F5E"], /* TIBETAN LETTER ZHA */
  ["\u0F5F", "\u0F5F"], /* TIBETAN LETTER ZA */
  ["\u0F60", "\u0F60"], /* TIBETAN LETTER -A (TIBETAN LETTER AA) */
  ["\u0F61", "\u0F61"], /* TIBETAN LETTER YA */
  ["\u0F62", "\u0F62"], /* TIBETAN LETTER RA */
  ["\u0F63", "\u0F63"], /* TIBETAN LETTER LA */
  ["\u0F64", "\u0F64"], /* TIBETAN LETTER SHA */
  ["\u0F65", "\u0F65"], /* TIBETAN LETTER SSA (TIBETAN LETTER REVERSED SHA) */
  ["\u0F66", "\u0F66"], /* TIBETAN LETTER SA */
  ["\u0F67", "\u0F67"], /* TIBETAN LETTER HA */
  ["\u0F68", "\u0F68"], /* TIBETAN LETTER A */
  ["\u0F69", "\u0F69"], /* TIBETAN LETTER KSSA */
  ["\u0F6A", "\u0F6A"], /* TIBETAN LETTER FIXED-FORM RA */
  ["\u0F6B", "\u0F6B"], /* TIBETAN LETTER KKA */
  ["\u0F6C", "\u0F6C"], /* TIBETAN LETTER RRA */
  ["\u0F6D", "\u0F6D"],
  ["\u0F6E", "\u0F6E"],
  ["\u0F6F", "\u0F6F"],
  ["\u0F70", "\u0F70"],
  ["\u0F71", "\u0F71"], /* TIBETAN VOWEL SIGN AA */
  ["\u0F72", "\u0F72"], /* TIBETAN VOWEL SIGN I */
  ["\u0F73", "\u0F73"], /* TIBETAN VOWEL SIGN II */
  ["\u0F74", "\u0F74"], /* TIBETAN VOWEL SIGN U */
  ["\u0F75", "\u0F75"], /* TIBETAN VOWEL SIGN UU */
  ["\u0F76", "\u0F76"], /* TIBETAN VOWEL SIGN VOCALIC R */
  ["\u0F77", "\u0F77"], /* TIBETAN VOWEL SIGN VOCALIC RR */
  ["\u0F78", "\u0F78"], /* TIBETAN VOWEL SIGN VOCALIC L */
  ["\u0F79", "\u0F79"], /* TIBETAN VOWEL SIGN VOCALIC LL */
  ["\u0F7A", "\u0F7A"], /* TIBETAN VOWEL SIGN E */
  ["\u0F7B", "\u0F7B"], /* TIBETAN VOWEL SIGN EE (TIBETAN VOWEL SIGN AI) */
  ["\u0F7C", "\u0F7C"], /* TIBETAN VOWEL SIGN O */
  ["\u0F7D", "\u0F7D"], /* TIBETAN VOWEL SIGN OO (TIBETAN VOWEL SIGN AU) */
  ["\u0F7E", "\u0F7E"], /* TIBETAN SIGN RJES SU NGA RO (TIBETAN ANUSVARA) */
  ["\u0F7F", "\u0F7F"], /* TIBETAN SIGN RNAM BCAD (TIBETAN VISARGA) */
  ["\u0F80", "\u0F80"], /* TIBETAN VOWEL SIGN REVERSED I (TIBETAN VOWEL SIGN SHORT I) */
  ["\u0F81", "\u0F81"], /* TIBETAN VOWEL SIGN REVERSED II */
  ["\u0F82", "\u0F82"], /* TIBETAN SIGN NYI ZLA NAA DA (TIBETAN CANDRABINDU WITH ORNAMENT) */
  ["\u0F83", "\u0F83"], /* TIBETAN SIGN SNA LDAN (TIBETAN CANDRABINDU) */
  ["\u0F84", "\u0F84"], /* TIBETAN MARK HALANTA (TIBETAN VIRAMA) */
  ["\u0F85", "\u0F85"], /* TIBETAN MARK PALUTA (TIBETAN CHUCHENYIGE) */
  ["\u0F86", "\u0F86"], /* TIBETAN SIGN LCI RTAGS */
  ["\u0F87", "\u0F87"], /* TIBETAN SIGN YANG RTAGS */
  ["\u0F88", "\u0F88"], /* TIBETAN SIGN LCE TSA CAN */
  ["\u0F89", "\u0F89"], /* TIBETAN SIGN MCHU CAN */
  ["\u0F8A", "\u0F8A"], /* TIBETAN SIGN GRU CAN RGYINGS */
  ["\u0F8B", "\u0F8B"], /* TIBETAN SIGN GRU MED RGYINGS */
  ["\u0F8C", "\u0F8C"], /* TIBETAN SIGN INVERTED MCHU CAN */
  ["\u0F8D", "\u0F8D"], /* TIBETAN SUBJOINED SIGN LCE TSA CAN */
  ["\u0F8E", "\u0F8E"], /* TIBETAN SUBJOINED SIGN MCHU CAN */
  ["\u0F8F", "\u0F8F"], /* TIBETAN SUBJOINED SIGN INVERTED MCHU CAN */
  ["\u0F90", "\u0F90"], /* TIBETAN SUBJOINED LETTER KA */
  ["\u0F91", "\u0F91"], /* TIBETAN SUBJOINED LETTER KHA */
  ["\u0F92", "\u0F92"], /* TIBETAN SUBJOINED LETTER GA */
  ["\u0F93", "\u0F93"], /* TIBETAN SUBJOINED LETTER GHA */
  ["\u0F94", "\u0F94"], /* TIBETAN SUBJOINED LETTER NGA */
  ["\u0F95", "\u0F95"], /* TIBETAN SUBJOINED LETTER CA */
  ["\u0F96", "\u0F96"], /* TIBETAN SUBJOINED LETTER CHA */
  ["\u0F97", "\u0F97"], /* TIBETAN SUBJOINED LETTER JA */
  ["\u0F98", "\u0F98"],
  ["\u0F99", "\u0F99"], /* TIBETAN SUBJOINED LETTER NYA */
  ["\u0F9A", "\u0F9A"], /* TIBETAN SUBJOINED LETTER TTA */
  ["\u0F9B", "\u0F9B"], /* TIBETAN SUBJOINED LETTER TTHA */
  ["\u0F9C", "\u0F9C"], /* TIBETAN SUBJOINED LETTER DDA */
  ["\u0F9D", "\u0F9D"], /* TIBETAN SUBJOINED LETTER DDHA */
  ["\u0F9E", "\u0F9E"], /* TIBETAN SUBJOINED LETTER NNA */
  ["\u0F9F", "\u0F9F"], /* TIBETAN SUBJOINED LETTER TA */
  ["\u0FA0", "\u0FA0"], /* TIBETAN SUBJOINED LETTER THA */
  ["\u0FA1", "\u0FA1"], /* TIBETAN SUBJOINED LETTER DA */
  ["\u0FA2", "\u0FA2"], /* TIBETAN SUBJOINED LETTER DHA */
  ["\u0FA3", "\u0FA3"], /* TIBETAN SUBJOINED LETTER NA */
  ["\u0FA4", "\u0FA4"], /* TIBETAN SUBJOINED LETTER PA */
  ["\u0FA5", "\u0FA5"], /* TIBETAN SUBJOINED LETTER PHA */
  ["\u0FA6", "\u0FA6"], /* TIBETAN SUBJOINED LETTER BA */
  ["\u0FA7", "\u0FA7"], /* TIBETAN SUBJOINED LETTER BHA */
  ["\u0FA8", "\u0FA8"], /* TIBETAN SUBJOINED LETTER MA */
  ["\u0FA9", "\u0FA9"], /* TIBETAN SUBJOINED LETTER TSA */
  ["\u0FAA", "\u0FAA"], /* TIBETAN SUBJOINED LETTER TSHA */
  ["\u0FAB", "\u0FAB"], /* TIBETAN SUBJOINED LETTER DZA */
  ["\u0FAC", "\u0FAC"], /* TIBETAN SUBJOINED LETTER DZHA */
  ["\u0FAD", "\u0FAD"], /* TIBETAN SUBJOINED LETTER WA */
  ["\u0FAE", "\u0FAE"], /* TIBETAN SUBJOINED LETTER ZHA */
  ["\u0FAF", "\u0FAF"], /* TIBETAN SUBJOINED LETTER ZA */
  ["\u0FB0", "\u0FB0"], /* TIBETAN SUBJOINED LETTER -A */
  ["\u0FB1", "\u0FB1"], /* TIBETAN SUBJOINED LETTER YA */
  ["\u0FB2", "\u0FB2"], /* TIBETAN SUBJOINED LETTER RA */
  ["\u0FB3", "\u0FB3"], /* TIBETAN SUBJOINED LETTER LA */
  ["\u0FB4", "\u0FB4"], /* TIBETAN SUBJOINED LETTER SHA */
  ["\u0FB5", "\u0FB5"], /* TIBETAN SUBJOINED LETTER SSA */
  ["\u0FB6", "\u0FB6"], /* TIBETAN SUBJOINED LETTER SA */
  ["\u0FB7", "\u0FB7"], /* TIBETAN SUBJOINED LETTER HA */
  ["\u0FB8", "\u0FB8"], /* TIBETAN SUBJOINED LETTER A */
  ["\u0FB9", "\u0FB9"], /* TIBETAN SUBJOINED LETTER KSSA */
  ["\u0FBA", "\u0FBA"], /* TIBETAN SUBJOINED LETTER FIXED-FORM WA */
  ["\u0FBB", "\u0FBB"], /* TIBETAN SUBJOINED LETTER FIXED-FORM YA */
  ["\u0FBC", "\u0FBC"], /* TIBETAN SUBJOINED LETTER FIXED-FORM RA */
  ["\u0FBD", "\u0FBD"],
  ["\u0FBE", "\u0FBE"], /* TIBETAN KU RU KHA */
  ["\u0FBF", "\u0FBF"], /* TIBETAN KU RU KHA BZHI MIG CAN */
  ["\u0FC0", "\u0FC0"], /* TIBETAN CANTILLATION SIGN HEAVY BEAT */
  ["\u0FC1", "\u0FC1"], /* TIBETAN CANTILLATION SIGN LIGHT BEAT */
  ["\u0FC2", "\u0FC2"], /* TIBETAN CANTILLATION SIGN CANG TE-U */
  ["\u0FC3", "\u0FC3"], /* TIBETAN CANTILLATION SIGN SBUB -CHAL */
  ["\u0FC4", "\u0FC4"], /* TIBETAN SYMBOL DRIL BU */
  ["\u0FC5", "\u0FC5"], /* TIBETAN SYMBOL RDO RJE */
  ["\u0FC6", "\u0FC6"], /* TIBETAN SYMBOL PADMA GDAN */
  ["\u0FC7", "\u0FC7"], /* TIBETAN SYMBOL RDO RJE RGYA GRAM */
  ["\u0FC8", "\u0FC8"], /* TIBETAN SYMBOL PHUR PA */
  ["\u0FC9", "\u0FC9"], /* TIBETAN SYMBOL NOR BU */
  ["\u0FCA", "\u0FCA"], /* TIBETAN SYMBOL NOR BU NYIS -KHYIL */
  ["\u0FCB", "\u0FCB"], /* TIBETAN SYMBOL NOR BU GSUM -KHYIL */
  ["\u0FCC", "\u0FCC"], /* TIBETAN SYMBOL NOR BU BZHI -KHYIL */
  ["\u0FCD", "\u0FCD"],
  ["\u0FCE", "\u0FCE"], /* TIBETAN SIGN RDEL NAG RDEL DKAR */
  ["\u0FCF", "\u0FCF"], /* TIBETAN SIGN RDEL NAG GSUM */
  ["\u0FD0", "\u0FD0"], /* TIBETAN MARK BSKA- SHOG GI MGO RGYAN */
  ["\u0FD1", "\u0FD1"], /* TIBETAN MARK MNYAM YIG GI MGO RGYAN */
  ["\u0FD2", "\u0FD2"], /* TIBETAN MARK NYIS TSHEG */
  ["\u0FD3", "\u0FD3"], /* TIBETAN MARK INITIAL BRDA RNYING YIG MGO MDUN MA */
  ["\u0FD4", "\u0FD4"], /* TIBETAN MARK CLOSING BRDA RNYING YIG MGO SGAB MA */
  ["\u0FD5", "\u0FD5"], /* RIGHT-FACING SVASTI SIGN */
  ["\u0FD6", "\u0FD6"], /* LEFT-FACING SVASTI SIGN */
  ["\u0FD7", "\u0FD7"], /* RIGHT-FACING SVASTI SIGN WITH DOTS */
  ["\u0FD8", "\u0FD8"], /* LEFT-FACING SVASTI SIGN WITH DOTS */
  ["\u0FD9", "\u0FD9"], /* TIBETAN MARK LEADING MCHAN RTAGS */
  ["\u0FDA", "\u0FDA"], /* TIBETAN MARK TRAILING MCHAN RTAGS */
  ["\u0FDB", "\u0FDB"],
  ["\u0FDC", "\u0FDC"],
  ["\u0FDD", "\u0FDD"],
  ["\u0FDE", "\u0FDE"],
  ["\u0FDF", "\u0FDF"],
  ["\u0FE0", "\u0FE0"],
  ["\u0FE1", "\u0FE1"],
  ["\u0FE2", "\u0FE2"],
  ["\u0FE3", "\u0FE3"],
  ["\u0FE4", "\u0FE4"],
  ["\u0FE5", "\u0FE5"],
  ["\u0FE6", "\u0FE6"],
  ["\u0FE7", "\u0FE7"],
  ["\u0FE8", "\u0FE8"],
  ["\u0FE9", "\u0FE9"],
  ["\u0FEA", "\u0FEA"],
  ["\u0FEB", "\u0FEB"],
  ["\u0FEC", "\u0FEC"],
  ["\u0FED", "\u0FED"],
  ["\u0FEE", "\u0FEE"],
  ["\u0FEF", "\u0FEF"],
  ["\u0FF0", "\u0FF0"],
  ["\u0FF1", "\u0FF1"],
  ["\u0FF2", "\u0FF2"],
  ["\u0FF3", "\u0FF3"],
  ["\u0FF4", "\u0FF4"],
  ["\u0FF5", "\u0FF5"],
  ["\u0FF6", "\u0FF6"],
  ["\u0FF7", "\u0FF7"],
  ["\u0FF8", "\u0FF8"],
  ["\u0FF9", "\u0FF9"],
  ["\u0FFA", "\u0FFA"],
  ["\u0FFB", "\u0FFB"],
  ["\u0FFC", "\u0FFC"],
  ["\u0FFD", "\u0FFD"],
  ["\u0FFE", "\u0FFE"],
  ["\u0FFF", "\u0FFF"],
  ["\u1000", "\u1000"], /* MYANMAR LETTER KA */
  ["\u1001", "\u1001"], /* MYANMAR LETTER KHA */
  ["\u1002", "\u1002"], /* MYANMAR LETTER GA */
  ["\u1003", "\u1003"], /* MYANMAR LETTER GHA */
  ["\u1004", "\u1004"], /* MYANMAR LETTER NGA */
  ["\u1005", "\u1005"], /* MYANMAR LETTER CA */
  ["\u1006", "\u1006"], /* MYANMAR LETTER CHA */
  ["\u1007", "\u1007"], /* MYANMAR LETTER JA */
  ["\u1008", "\u1008"], /* MYANMAR LETTER JHA */
  ["\u1009", "\u1009"], /* MYANMAR LETTER NYA */
  ["\u100A", "\u100A"], /* MYANMAR LETTER NNYA */
  ["\u100B", "\u100B"], /* MYANMAR LETTER TTA */
  ["\u100C", "\u100C"], /* MYANMAR LETTER TTHA */
  ["\u100D", "\u100D"], /* MYANMAR LETTER DDA */
  ["\u100E", "\u100E"], /* MYANMAR LETTER DDHA */
  ["\u100F", "\u100F"], /* MYANMAR LETTER NNA */
  ["\u1010", "\u1010"], /* MYANMAR LETTER TA */
  ["\u1011", "\u1011"], /* MYANMAR LETTER THA */
  ["\u1012", "\u1012"], /* MYANMAR LETTER DA */
  ["\u1013", "\u1013"], /* MYANMAR LETTER DHA */
  ["\u1014", "\u1014"], /* MYANMAR LETTER NA */
  ["\u1015", "\u1015"], /* MYANMAR LETTER PA */
  ["\u1016", "\u1016"], /* MYANMAR LETTER PHA */
  ["\u1017", "\u1017"], /* MYANMAR LETTER BA */
  ["\u1018", "\u1018"], /* MYANMAR LETTER BHA */
  ["\u1019", "\u1019"], /* MYANMAR LETTER MA */
  ["\u101A", "\u101A"], /* MYANMAR LETTER YA */
  ["\u101B", "\u101B"], /* MYANMAR LETTER RA */
  ["\u101C", "\u101C"], /* MYANMAR LETTER LA */
  ["\u101D", "\u101D"], /* MYANMAR LETTER WA */
  ["\u101E", "\u101E"], /* MYANMAR LETTER SA */
  ["\u101F", "\u101F"], /* MYANMAR LETTER HA */
  ["\u1020", "\u1020"], /* MYANMAR LETTER LLA */
  ["\u1021", "\u1021"], /* MYANMAR LETTER A */
  ["\u1022", "\u1022"], /* MYANMAR LETTER SHAN A */
  ["\u1023", "\u1023"], /* MYANMAR LETTER I */
  ["\u1024", "\u1024"], /* MYANMAR LETTER II */
  ["\u1025", "\u1025"], /* MYANMAR LETTER U */
  ["\u1026", "\u1026"], /* MYANMAR LETTER UU */
  ["\u1027", "\u1027"], /* MYANMAR LETTER E */
  ["\u1028", "\u1028"], /* MYANMAR LETTER MON E */
  ["\u1029", "\u1029"], /* MYANMAR LETTER O */
  ["\u102A", "\u102A"], /* MYANMAR LETTER AU */
  ["\u102B", "\u102B"], /* MYANMAR VOWEL SIGN TALL AA */
  ["\u102C", "\u102C"], /* MYANMAR VOWEL SIGN AA */
  ["\u102D", "\u102D"], /* MYANMAR VOWEL SIGN I */
  ["\u102E", "\u102E"], /* MYANMAR VOWEL SIGN II */
  ["\u102F", "\u102F"], /* MYANMAR VOWEL SIGN U */
  ["\u1030", "\u1030"], /* MYANMAR VOWEL SIGN UU */
  ["\u1031", "\u1031"], /* MYANMAR VOWEL SIGN E */
  ["\u1032", "\u1032"], /* MYANMAR VOWEL SIGN AI */
  ["\u1033", "\u1033"], /* MYANMAR VOWEL SIGN MON II */
  ["\u1034", "\u1034"], /* MYANMAR VOWEL SIGN MON O */
  ["\u1035", "\u1035"], /* MYANMAR VOWEL SIGN E ABOVE */
  ["\u1036", "\u1036"], /* MYANMAR SIGN ANUSVARA */
  ["\u1037", "\u1037"], /* MYANMAR SIGN DOT BELOW */
  ["\u1038", "\u1038"], /* MYANMAR SIGN VISARGA */
  ["\u1039", "\u1039"], /* MYANMAR SIGN VIRAMA */
  ["\u103A", "\u103A"], /* MYANMAR SIGN ASAT */
  ["\u103B", "\u103B"], /* MYANMAR CONSONANT SIGN MEDIAL YA */
  ["\u103C", "\u103C"], /* MYANMAR CONSONANT SIGN MEDIAL RA */
  ["\u103D", "\u103D"], /* MYANMAR CONSONANT SIGN MEDIAL WA */
  ["\u103E", "\u103E"], /* MYANMAR CONSONANT SIGN MEDIAL HA */
  ["\u103F", "\u103F"], /* MYANMAR LETTER GREAT SA */
  ["\u1040", "\u1040"], /* MYANMAR DIGIT ZERO */
  ["\u1041", "\u1041"], /* MYANMAR DIGIT ONE */
  ["\u1042", "\u1042"], /* MYANMAR DIGIT TWO */
  ["\u1043", "\u1043"], /* MYANMAR DIGIT THREE */
  ["\u1044", "\u1044"], /* MYANMAR DIGIT FOUR */
  ["\u1045", "\u1045"], /* MYANMAR DIGIT FIVE */
  ["\u1046", "\u1046"], /* MYANMAR DIGIT SIX */
  ["\u1047", "\u1047"], /* MYANMAR DIGIT SEVEN */
  ["\u1048", "\u1048"], /* MYANMAR DIGIT EIGHT */
  ["\u1049", "\u1049"], /* MYANMAR DIGIT NINE */
  ["\u104A", "\u104A"], /* MYANMAR SIGN LITTLE SECTION */
  ["\u104B", "\u104B"], /* MYANMAR SIGN SECTION */
  ["\u104C", "\u104C"], /* MYANMAR SYMBOL LOCATIVE */
  ["\u104D", "\u104D"], /* MYANMAR SYMBOL COMPLETED */
  ["\u104E", "\u104E"], /* MYANMAR SYMBOL AFOREMENTIONED */
  ["\u104F", "\u104F"], /* MYANMAR SYMBOL GENITIVE */
  ["\u1050", "\u1050"], /* MYANMAR LETTER SHA */
  ["\u1051", "\u1051"], /* MYANMAR LETTER SSA */
  ["\u1052", "\u1052"], /* MYANMAR LETTER VOCALIC R */
  ["\u1053", "\u1053"], /* MYANMAR LETTER VOCALIC RR */
  ["\u1054", "\u1054"], /* MYANMAR LETTER VOCALIC L */
  ["\u1055", "\u1055"], /* MYANMAR LETTER VOCALIC LL */
  ["\u1056", "\u1056"], /* MYANMAR VOWEL SIGN VOCALIC R */
  ["\u1057", "\u1057"], /* MYANMAR VOWEL SIGN VOCALIC RR */
  ["\u1058", "\u1058"], /* MYANMAR VOWEL SIGN VOCALIC L */
  ["\u1059", "\u1059"], /* MYANMAR VOWEL SIGN VOCALIC LL */
  ["\u105A", "\u105A"], /* MYANMAR LETTER MON NGA */
  ["\u105B", "\u105B"], /* MYANMAR LETTER MON JHA */
  ["\u105C", "\u105C"], /* MYANMAR LETTER MON BBA */
  ["\u105D", "\u105D"], /* MYANMAR LETTER MON BBE */
  ["\u105E", "\u105E"], /* MYANMAR CONSONANT SIGN MON MEDIAL NA */
  ["\u105F", "\u105F"], /* MYANMAR CONSONANT SIGN MON MEDIAL MA */
  ["\u1060", "\u1060"], /* MYANMAR CONSONANT SIGN MON MEDIAL LA */
  ["\u1061", "\u1061"], /* MYANMAR LETTER SGAW KAREN SHA */
  ["\u1062", "\u1062"], /* MYANMAR VOWEL SIGN SGAW KAREN EU */
  ["\u1063", "\u1063"], /* MYANMAR TONE MARK SGAW KAREN HATHI */
  ["\u1064", "\u1064"], /* MYANMAR TONE MARK SGAW KAREN KE PHO */
  ["\u1065", "\u1065"], /* MYANMAR LETTER WESTERN PWO KAREN THA */
  ["\u1066", "\u1066"], /* MYANMAR LETTER WESTERN PWO KAREN PWA */
  ["\u1067", "\u1067"], /* MYANMAR VOWEL SIGN WESTERN PWO KAREN EU */
  ["\u1068", "\u1068"], /* MYANMAR VOWEL SIGN WESTERN PWO KAREN UE */
  ["\u1069", "\u1069"], /* MYANMAR SIGN WESTERN PWO KAREN TONE-1 */
  ["\u106A", "\u106A"], /* MYANMAR SIGN WESTERN PWO KAREN TONE-2 */
  ["\u106B", "\u106B"], /* MYANMAR SIGN WESTERN PWO KAREN TONE-3 */
  ["\u106C", "\u106C"], /* MYANMAR SIGN WESTERN PWO KAREN TONE-4 */
  ["\u106D", "\u106D"], /* MYANMAR SIGN WESTERN PWO KAREN TONE-5 */
  ["\u106E", "\u106E"], /* MYANMAR LETTER EASTERN PWO KAREN NNA */
  ["\u106F", "\u106F"], /* MYANMAR LETTER EASTERN PWO KAREN YWA */
  ["\u1070", "\u1070"], /* MYANMAR LETTER EASTERN PWO KAREN GHWA */
  ["\u1071", "\u1071"], /* MYANMAR VOWEL SIGN GEBA KAREN I */
  ["\u1072", "\u1072"], /* MYANMAR VOWEL SIGN KAYAH OE */
  ["\u1073", "\u1073"], /* MYANMAR VOWEL SIGN KAYAH U */
  ["\u1074", "\u1074"], /* MYANMAR VOWEL SIGN KAYAH EE */
  ["\u1075", "\u1075"], /* MYANMAR LETTER SHAN KA */
  ["\u1076", "\u1076"], /* MYANMAR LETTER SHAN KHA */
  ["\u1077", "\u1077"], /* MYANMAR LETTER SHAN GA */
  ["\u1078", "\u1078"], /* MYANMAR LETTER SHAN CA */
  ["\u1079", "\u1079"], /* MYANMAR LETTER SHAN ZA */
  ["\u107A", "\u107A"], /* MYANMAR LETTER SHAN NYA */
  ["\u107B", "\u107B"], /* MYANMAR LETTER SHAN DA */
  ["\u107C", "\u107C"], /* MYANMAR LETTER SHAN NA */
  ["\u107D", "\u107D"], /* MYANMAR LETTER SHAN PHA */
  ["\u107E", "\u107E"], /* MYANMAR LETTER SHAN FA */
  ["\u107F", "\u107F"], /* MYANMAR LETTER SHAN BA */
  ["\u1080", "\u1080"], /* MYANMAR LETTER SHAN THA */
  ["\u1081", "\u1081"], /* MYANMAR LETTER SHAN HA */
  ["\u1082", "\u1082"], /* MYANMAR CONSONANT SIGN SHAN MEDIAL WA */
  ["\u1083", "\u1083"], /* MYANMAR VOWEL SIGN SHAN AA */
  ["\u1084", "\u1084"], /* MYANMAR VOWEL SIGN SHAN E */
  ["\u1085", "\u1085"], /* MYANMAR VOWEL SIGN SHAN E ABOVE */
  ["\u1086", "\u1086"], /* MYANMAR VOWEL SIGN SHAN FINAL Y */
  ["\u1087", "\u1087"], /* MYANMAR SIGN SHAN TONE-2 */
  ["\u1088", "\u1088"], /* MYANMAR SIGN SHAN TONE-3 */
  ["\u1089", "\u1089"], /* MYANMAR SIGN SHAN TONE-5 */
  ["\u108A", "\u108A"], /* MYANMAR SIGN SHAN TONE-6 */
  ["\u108B", "\u108B"], /* MYANMAR SIGN SHAN COUNCIL TONE-2 */
  ["\u108C", "\u108C"], /* MYANMAR SIGN SHAN COUNCIL TONE-3 */
  ["\u108D", "\u108D"], /* MYANMAR SIGN SHAN COUNCIL EMPHATIC TONE */
  ["\u108E", "\u108E"], /* MYANMAR LETTER RUMAI PALAUNG FA */
  ["\u108F", "\u108F"], /* MYANMAR SIGN RUMAI PALAUNG TONE-5 */
  ["\u1090", "\u1090"], /* MYANMAR SHAN DIGIT ZERO */
  ["\u1091", "\u1091"], /* MYANMAR SHAN DIGIT ONE */
  ["\u1092", "\u1092"], /* MYANMAR SHAN DIGIT TWO */
  ["\u1093", "\u1093"], /* MYANMAR SHAN DIGIT THREE */
  ["\u1094", "\u1094"], /* MYANMAR SHAN DIGIT FOUR */
  ["\u1095", "\u1095"], /* MYANMAR SHAN DIGIT FIVE */
  ["\u1096", "\u1096"], /* MYANMAR SHAN DIGIT SIX */
  ["\u1097", "\u1097"], /* MYANMAR SHAN DIGIT SEVEN */
  ["\u1098", "\u1098"], /* MYANMAR SHAN DIGIT EIGHT */
  ["\u1099", "\u1099"], /* MYANMAR SHAN DIGIT NINE */
  ["\u109A", "\u109A"], /* MYANMAR SIGN KHAMTI TONE-1 */
  ["\u109B", "\u109B"], /* MYANMAR SIGN KHAMTI TONE-3 */
  ["\u109C", "\u109C"], /* MYANMAR VOWEL SIGN AITON A */
  ["\u109D", "\u109D"], /* MYANMAR VOWEL SIGN AITON AI */
  ["\u109E", "\u109E"], /* MYANMAR SYMBOL SHAN ONE */
  ["\u109F", "\u109F"], /* MYANMAR SYMBOL SHAN EXCLAMATION */
  ["\u10A0", "\u2D00"], /* GEORGIAN CAPITAL LETTER AN */
  ["\u10A1", "\u2D01"], /* GEORGIAN CAPITAL LETTER BAN */
  ["\u10A2", "\u2D02"], /* GEORGIAN CAPITAL LETTER GAN */
  ["\u10A3", "\u2D03"], /* GEORGIAN CAPITAL LETTER DON */
  ["\u10A4", "\u2D04"], /* GEORGIAN CAPITAL LETTER EN */
  ["\u10A5", "\u2D05"], /* GEORGIAN CAPITAL LETTER VIN */
  ["\u10A6", "\u2D06"], /* GEORGIAN CAPITAL LETTER ZEN */
  ["\u10A7", "\u2D07"], /* GEORGIAN CAPITAL LETTER TAN */
  ["\u10A8", "\u2D08"], /* GEORGIAN CAPITAL LETTER IN */
  ["\u10A9", "\u2D09"], /* GEORGIAN CAPITAL LETTER KAN */
  ["\u10AA", "\u2D0A"], /* GEORGIAN CAPITAL LETTER LAS */
  ["\u10AB", "\u2D0B"], /* GEORGIAN CAPITAL LETTER MAN */
  ["\u10AC", "\u2D0C"], /* GEORGIAN CAPITAL LETTER NAR */
  ["\u10AD", "\u2D0D"], /* GEORGIAN CAPITAL LETTER ON */
  ["\u10AE", "\u2D0E"], /* GEORGIAN CAPITAL LETTER PAR */
  ["\u10AF", "\u2D0F"], /* GEORGIAN CAPITAL LETTER ZHAR */
  ["\u10B0", "\u2D10"], /* GEORGIAN CAPITAL LETTER RAE */
  ["\u10B1", "\u2D11"], /* GEORGIAN CAPITAL LETTER SAN */
  ["\u10B2", "\u2D12"], /* GEORGIAN CAPITAL LETTER TAR */
  ["\u10B3", "\u2D13"], /* GEORGIAN CAPITAL LETTER UN */
  ["\u10B4", "\u2D14"], /* GEORGIAN CAPITAL LETTER PHAR */
  ["\u10B5", "\u2D15"], /* GEORGIAN CAPITAL LETTER KHAR */
  ["\u10B6", "\u2D16"], /* GEORGIAN CAPITAL LETTER GHAN */
  ["\u10B7", "\u2D17"], /* GEORGIAN CAPITAL LETTER QAR */
  ["\u10B8", "\u2D18"], /* GEORGIAN CAPITAL LETTER SHIN */
  ["\u10B9", "\u2D19"], /* GEORGIAN CAPITAL LETTER CHIN */
  ["\u10BA", "\u2D1A"], /* GEORGIAN CAPITAL LETTER CAN */
  ["\u10BB", "\u2D1B"], /* GEORGIAN CAPITAL LETTER JIL */
  ["\u10BC", "\u2D1C"], /* GEORGIAN CAPITAL LETTER CIL */
  ["\u10BD", "\u2D1D"], /* GEORGIAN CAPITAL LETTER CHAR */
  ["\u10BE", "\u2D1E"], /* GEORGIAN CAPITAL LETTER XAN */
  ["\u10BF", "\u2D1F"], /* GEORGIAN CAPITAL LETTER JHAN */
  ["\u10C0", "\u2D20"], /* GEORGIAN CAPITAL LETTER HAE */
  ["\u10C1", "\u2D21"], /* GEORGIAN CAPITAL LETTER HE */
  ["\u10C2", "\u2D22"], /* GEORGIAN CAPITAL LETTER HIE */
  ["\u10C3", "\u2D23"], /* GEORGIAN CAPITAL LETTER WE */
  ["\u10C4", "\u2D24"], /* GEORGIAN CAPITAL LETTER HAR */
  ["\u10C5", "\u2D25"], /* GEORGIAN CAPITAL LETTER HOE */
  ["\u10C6", "\u10C6"],
  ["\u10C7", "\u2D27"], /* GEORGIAN CAPITAL LETTER YN */
  ["\u10C8", "\u10C8"],
  ["\u10C9", "\u10C9"],
  ["\u10CA", "\u10CA"],
  ["\u10CB", "\u10CB"],
  ["\u10CC", "\u10CC"],
  ["\u10CD", "\u2D2D"], /* GEORGIAN CAPITAL LETTER AEN */
  ["\u10CE", "\u10CE"],
  ["\u10CF", "\u10CF"],
  ["\u1C90", "\u10D0"], /* GEORGIAN LETTER AN (GEORGIAN SMALL LETTER AN) */
  ["\u1C91", "\u10D1"], /* GEORGIAN LETTER BAN (GEORGIAN SMALL LETTER BAN) */
  ["\u1C92", "\u10D2"], /* GEORGIAN LETTER GAN (GEORGIAN SMALL LETTER GAN) */
  ["\u1C93", "\u10D3"], /* GEORGIAN LETTER DON (GEORGIAN SMALL LETTER DON) */
  ["\u1C94", "\u10D4"], /* GEORGIAN LETTER EN (GEORGIAN SMALL LETTER EN) */
  ["\u1C95", "\u10D5"], /* GEORGIAN LETTER VIN (GEORGIAN SMALL LETTER VIN) */
  ["\u1C96", "\u10D6"], /* GEORGIAN LETTER ZEN (GEORGIAN SMALL LETTER ZEN) */
  ["\u1C97", "\u10D7"], /* GEORGIAN LETTER TAN (GEORGIAN SMALL LETTER TAN) */
  ["\u1C98", "\u10D8"], /* GEORGIAN LETTER IN (GEORGIAN SMALL LETTER IN) */
  ["\u1C99", "\u10D9"], /* GEORGIAN LETTER KAN (GEORGIAN SMALL LETTER KAN) */
  ["\u1C9A", "\u10DA"], /* GEORGIAN LETTER LAS (GEORGIAN SMALL LETTER LAS) */
  ["\u1C9B", "\u10DB"], /* GEORGIAN LETTER MAN (GEORGIAN SMALL LETTER MAN) */
  ["\u1C9C", "\u10DC"], /* GEORGIAN LETTER NAR (GEORGIAN SMALL LETTER NAR) */
  ["\u1C9D", "\u10DD"], /* GEORGIAN LETTER ON (GEORGIAN SMALL LETTER ON) */
  ["\u1C9E", "\u10DE"], /* GEORGIAN LETTER PAR (GEORGIAN SMALL LETTER PAR) */
  ["\u1C9F", "\u10DF"], /* GEORGIAN LETTER ZHAR (GEORGIAN SMALL LETTER ZHAR) */
  ["\u1CA0", "\u10E0"], /* GEORGIAN LETTER RAE (GEORGIAN SMALL LETTER RAE) */
  ["\u1CA1", "\u10E1"], /* GEORGIAN LETTER SAN (GEORGIAN SMALL LETTER SAN) */
  ["\u1CA2", "\u10E2"], /* GEORGIAN LETTER TAR (GEORGIAN SMALL LETTER TAR) */
  ["\u1CA3", "\u10E3"], /* GEORGIAN LETTER UN (GEORGIAN SMALL LETTER UN) */
  ["\u1CA4", "\u10E4"], /* GEORGIAN LETTER PHAR (GEORGIAN SMALL LETTER PHAR) */
  ["\u1CA5", "\u10E5"], /* GEORGIAN LETTER KHAR (GEORGIAN SMALL LETTER KHAR) */
  ["\u1CA6", "\u10E6"], /* GEORGIAN LETTER GHAN (GEORGIAN SMALL LETTER GHAN) */
  ["\u1CA7", "\u10E7"], /* GEORGIAN LETTER QAR (GEORGIAN SMALL LETTER QAR) */
  ["\u1CA8", "\u10E8"], /* GEORGIAN LETTER SHIN (GEORGIAN SMALL LETTER SHIN) */
  ["\u1CA9", "\u10E9"], /* GEORGIAN LETTER CHIN (GEORGIAN SMALL LETTER CHIN) */
  ["\u1CAA", "\u10EA"], /* GEORGIAN LETTER CAN (GEORGIAN SMALL LETTER CAN) */
  ["\u1CAB", "\u10EB"], /* GEORGIAN LETTER JIL (GEORGIAN SMALL LETTER JIL) */
  ["\u1CAC", "\u10EC"], /* GEORGIAN LETTER CIL (GEORGIAN SMALL LETTER CIL) */
  ["\u1CAD", "\u10ED"], /* GEORGIAN LETTER CHAR (GEORGIAN SMALL LETTER CHAR) */
  ["\u1CAE", "\u10EE"], /* GEORGIAN LETTER XAN (GEORGIAN SMALL LETTER XAN) */
  ["\u1CAF", "\u10EF"], /* GEORGIAN LETTER JHAN (GEORGIAN SMALL LETTER JHAN) */
  ["\u1CB0", "\u10F0"], /* GEORGIAN LETTER HAE (GEORGIAN SMALL LETTER HAE) */
  ["\u1CB1", "\u10F1"], /* GEORGIAN LETTER HE (GEORGIAN SMALL LETTER HE) */
  ["\u1CB2", "\u10F2"], /* GEORGIAN LETTER HIE (GEORGIAN SMALL LETTER HIE) */
  ["\u1CB3", "\u10F3"], /* GEORGIAN LETTER WE (GEORGIAN SMALL LETTER WE) */
  ["\u1CB4", "\u10F4"], /* GEORGIAN LETTER HAR (GEORGIAN SMALL LETTER HAR) */
  ["\u1CB5", "\u10F5"], /* GEORGIAN LETTER HOE (GEORGIAN SMALL LETTER HOE) */
  ["\u1CB6", "\u10F6"], /* GEORGIAN LETTER FI (GEORGIAN SMALL LETTER FI) */
  ["\u1CB7", "\u10F7"], /* GEORGIAN LETTER YN */
  ["\u1CB8", "\u10F8"], /* GEORGIAN LETTER ELIFI */
  ["\u1CB9", "\u10F9"], /* GEORGIAN LETTER TURNED GAN */
  ["\u1CBA", "\u10FA"], /* GEORGIAN LETTER AIN */
  ["\u10FB", "\u10FB"], /* GEORGIAN PARAGRAPH SEPARATOR */
  ["\u10FC", "\u10FC"], /* MODIFIER LETTER GEORGIAN NAR */
  ["\u1CBD", "\u10FD"], /* GEORGIAN LETTER AEN */
  ["\u1CBE", "\u10FE"], /* GEORGIAN LETTER HARD SIGN */
  ["\u1CBF", "\u10FF"], /* GEORGIAN LETTER LABIAL SIGN */
  ["\u1100", "\u1100"], /* HANGUL CHOSEONG KIYEOK */
  ["\u1101", "\u1101"], /* HANGUL CHOSEONG SSANGKIYEOK */
  ["\u1102", "\u1102"], /* HANGUL CHOSEONG NIEUN */
  ["\u1103", "\u1103"], /* HANGUL CHOSEONG TIKEUT */
  ["\u1104", "\u1104"], /* HANGUL CHOSEONG SSANGTIKEUT */
  ["\u1105", "\u1105"], /* HANGUL CHOSEONG RIEUL */
  ["\u1106", "\u1106"], /* HANGUL CHOSEONG MIEUM */
  ["\u1107", "\u1107"], /* HANGUL CHOSEONG PIEUP */
  ["\u1108", "\u1108"], /* HANGUL CHOSEONG SSANGPIEUP */
  ["\u1109", "\u1109"], /* HANGUL CHOSEONG SIOS */
  ["\u110A", "\u110A"], /* HANGUL CHOSEONG SSANGSIOS */
  ["\u110B", "\u110B"], /* HANGUL CHOSEONG IEUNG */
  ["\u110C", "\u110C"], /* HANGUL CHOSEONG CIEUC */
  ["\u110D", "\u110D"], /* HANGUL CHOSEONG SSANGCIEUC */
  ["\u110E", "\u110E"], /* HANGUL CHOSEONG CHIEUCH */
  ["\u110F", "\u110F"], /* HANGUL CHOSEONG KHIEUKH */
  ["\u1110", "\u1110"], /* HANGUL CHOSEONG THIEUTH */
  ["\u1111", "\u1111"], /* HANGUL CHOSEONG PHIEUPH */
  ["\u1112", "\u1112"], /* HANGUL CHOSEONG HIEUH */
  ["\u1113", "\u1113"], /* HANGUL CHOSEONG NIEUN-KIYEOK */
  ["\u1114", "\u1114"], /* HANGUL CHOSEONG SSANGNIEUN */
  ["\u1115", "\u1115"], /* HANGUL CHOSEONG NIEUN-TIKEUT */
  ["\u1116", "\u1116"], /* HANGUL CHOSEONG NIEUN-PIEUP */
  ["\u1117", "\u1117"], /* HANGUL CHOSEONG TIKEUT-KIYEOK */
  ["\u1118", "\u1118"], /* HANGUL CHOSEONG RIEUL-NIEUN */
  ["\u1119", "\u1119"], /* HANGUL CHOSEONG SSANGRIEUL */
  ["\u111A", "\u111A"], /* HANGUL CHOSEONG RIEUL-HIEUH */
  ["\u111B", "\u111B"], /* HANGUL CHOSEONG KAPYEOUNRIEUL */
  ["\u111C", "\u111C"], /* HANGUL CHOSEONG MIEUM-PIEUP */
  ["\u111D", "\u111D"], /* HANGUL CHOSEONG KAPYEOUNMIEUM */
  ["\u111E", "\u111E"], /* HANGUL CHOSEONG PIEUP-KIYEOK */
  ["\u111F", "\u111F"], /* HANGUL CHOSEONG PIEUP-NIEUN */
  ["\u1120", "\u1120"], /* HANGUL CHOSEONG PIEUP-TIKEUT */
  ["\u1121", "\u1121"], /* HANGUL CHOSEONG PIEUP-SIOS */
  ["\u1122", "\u1122"], /* HANGUL CHOSEONG PIEUP-SIOS-KIYEOK */
  ["\u1123", "\u1123"], /* HANGUL CHOSEONG PIEUP-SIOS-TIKEUT */
  ["\u1124", "\u1124"], /* HANGUL CHOSEONG PIEUP-SIOS-PIEUP */
  ["\u1125", "\u1125"], /* HANGUL CHOSEONG PIEUP-SSANGSIOS */
  ["\u1126", "\u1126"], /* HANGUL CHOSEONG PIEUP-SIOS-CIEUC */
  ["\u1127", "\u1127"], /* HANGUL CHOSEONG PIEUP-CIEUC */
  ["\u1128", "\u1128"], /* HANGUL CHOSEONG PIEUP-CHIEUCH */
  ["\u1129", "\u1129"], /* HANGUL CHOSEONG PIEUP-THIEUTH */
  ["\u112A", "\u112A"], /* HANGUL CHOSEONG PIEUP-PHIEUPH */
  ["\u112B", "\u112B"], /* HANGUL CHOSEONG KAPYEOUNPIEUP */
  ["\u112C", "\u112C"], /* HANGUL CHOSEONG KAPYEOUNSSANGPIEUP */
  ["\u112D", "\u112D"], /* HANGUL CHOSEONG SIOS-KIYEOK */
  ["\u112E", "\u112E"], /* HANGUL CHOSEONG SIOS-NIEUN */
  ["\u112F", "\u112F"], /* HANGUL CHOSEONG SIOS-TIKEUT */
  ["\u1130", "\u1130"], /* HANGUL CHOSEONG SIOS-RIEUL */
  ["\u1131", "\u1131"], /* HANGUL CHOSEONG SIOS-MIEUM */
  ["\u1132", "\u1132"], /* HANGUL CHOSEONG SIOS-PIEUP */
  ["\u1133", "\u1133"], /* HANGUL CHOSEONG SIOS-PIEUP-KIYEOK */
  ["\u1134", "\u1134"], /* HANGUL CHOSEONG SIOS-SSANGSIOS */
  ["\u1135", "\u1135"], /* HANGUL CHOSEONG SIOS-IEUNG */
  ["\u1136", "\u1136"], /* HANGUL CHOSEONG SIOS-CIEUC */
  ["\u1137", "\u1137"], /* HANGUL CHOSEONG SIOS-CHIEUCH */
  ["\u1138", "\u1138"], /* HANGUL CHOSEONG SIOS-KHIEUKH */
  ["\u1139", "\u1139"], /* HANGUL CHOSEONG SIOS-THIEUTH */
  ["\u113A", "\u113A"], /* HANGUL CHOSEONG SIOS-PHIEUPH */
  ["\u113B", "\u113B"], /* HANGUL CHOSEONG SIOS-HIEUH */
  ["\u113C", "\u113C"], /* HANGUL CHOSEONG CHITUEUMSIOS */
  ["\u113D", "\u113D"], /* HANGUL CHOSEONG CHITUEUMSSANGSIOS */
  ["\u113E", "\u113E"], /* HANGUL CHOSEONG CEONGCHIEUMSIOS */
  ["\u113F", "\u113F"], /* HANGUL CHOSEONG CEONGCHIEUMSSANGSIOS */
  ["\u1140", "\u1140"], /* HANGUL CHOSEONG PANSIOS */
  ["\u1141", "\u1141"], /* HANGUL CHOSEONG IEUNG-KIYEOK */
  ["\u1142", "\u1142"], /* HANGUL CHOSEONG IEUNG-TIKEUT */
  ["\u1143", "\u1143"], /* HANGUL CHOSEONG IEUNG-MIEUM */
  ["\u1144", "\u1144"], /* HANGUL CHOSEONG IEUNG-PIEUP */
  ["\u1145", "\u1145"], /* HANGUL CHOSEONG IEUNG-SIOS */
  ["\u1146", "\u1146"], /* HANGUL CHOSEONG IEUNG-PANSIOS */
  ["\u1147", "\u1147"], /* HANGUL CHOSEONG SSANGIEUNG */
  ["\u1148", "\u1148"], /* HANGUL CHOSEONG IEUNG-CIEUC */
  ["\u1149", "\u1149"], /* HANGUL CHOSEONG IEUNG-CHIEUCH */
  ["\u114A", "\u114A"], /* HANGUL CHOSEONG IEUNG-THIEUTH */
  ["\u114B", "\u114B"], /* HANGUL CHOSEONG IEUNG-PHIEUPH */
  ["\u114C", "\u114C"], /* HANGUL CHOSEONG YESIEUNG */
  ["\u114D", "\u114D"], /* HANGUL CHOSEONG CIEUC-IEUNG */
  ["\u114E", "\u114E"], /* HANGUL CHOSEONG CHITUEUMCIEUC */
  ["\u114F", "\u114F"], /* HANGUL CHOSEONG CHITUEUMSSANGCIEUC */
  ["\u1150", "\u1150"], /* HANGUL CHOSEONG CEONGCHIEUMCIEUC */
  ["\u1151", "\u1151"], /* HANGUL CHOSEONG CEONGCHIEUMSSANGCIEUC */
  ["\u1152", "\u1152"], /* HANGUL CHOSEONG CHIEUCH-KHIEUKH */
  ["\u1153", "\u1153"], /* HANGUL CHOSEONG CHIEUCH-HIEUH */
  ["\u1154", "\u1154"], /* HANGUL CHOSEONG CHITUEUMCHIEUCH */
  ["\u1155", "\u1155"], /* HANGUL CHOSEONG CEONGCHIEUMCHIEUCH */
  ["\u1156", "\u1156"], /* HANGUL CHOSEONG PHIEUPH-PIEUP */
  ["\u1157", "\u1157"], /* HANGUL CHOSEONG KAPYEOUNPHIEUPH */
  ["\u1158", "\u1158"], /* HANGUL CHOSEONG SSANGHIEUH */
  ["\u1159", "\u1159"], /* HANGUL CHOSEONG YEORINHIEUH */
  ["\u115A", "\u115A"], /* HANGUL CHOSEONG KIYEOK-TIKEUT */
  ["\u115B", "\u115B"], /* HANGUL CHOSEONG NIEUN-SIOS */
  ["\u115C", "\u115C"], /* HANGUL CHOSEONG NIEUN-CIEUC */
  ["\u115D", "\u115D"], /* HANGUL CHOSEONG NIEUN-HIEUH */
  ["\u115E", "\u115E"], /* HANGUL CHOSEONG TIKEUT-RIEUL */
  ["\u115F", "\u115F"], /* HANGUL CHOSEONG FILLER */
  ["\u1160", "\u1160"], /* HANGUL JUNGSEONG FILLER */
  ["\u1161", "\u1161"], /* HANGUL JUNGSEONG A */
  ["\u1162", "\u1162"], /* HANGUL JUNGSEONG AE */
  ["\u1163", "\u1163"], /* HANGUL JUNGSEONG YA */
  ["\u1164", "\u1164"], /* HANGUL JUNGSEONG YAE */
  ["\u1165", "\u1165"], /* HANGUL JUNGSEONG EO */
  ["\u1166", "\u1166"], /* HANGUL JUNGSEONG E */
  ["\u1167", "\u1167"], /* HANGUL JUNGSEONG YEO */
  ["\u1168", "\u1168"], /* HANGUL JUNGSEONG YE */
  ["\u1169", "\u1169"], /* HANGUL JUNGSEONG O */
  ["\u116A", "\u116A"], /* HANGUL JUNGSEONG WA */
  ["\u116B", "\u116B"], /* HANGUL JUNGSEONG WAE */
  ["\u116C", "\u116C"], /* HANGUL JUNGSEONG OE */
  ["\u116D", "\u116D"], /* HANGUL JUNGSEONG YO */
  ["\u116E", "\u116E"], /* HANGUL JUNGSEONG U */
  ["\u116F", "\u116F"], /* HANGUL JUNGSEONG WEO */
  ["\u1170", "\u1170"], /* HANGUL JUNGSEONG WE */
  ["\u1171", "\u1171"], /* HANGUL JUNGSEONG WI */
  ["\u1172", "\u1172"], /* HANGUL JUNGSEONG YU */
  ["\u1173", "\u1173"], /* HANGUL JUNGSEONG EU */
  ["\u1174", "\u1174"], /* HANGUL JUNGSEONG YI */
  ["\u1175", "\u1175"], /* HANGUL JUNGSEONG I */
  ["\u1176", "\u1176"], /* HANGUL JUNGSEONG A-O */
  ["\u1177", "\u1177"], /* HANGUL JUNGSEONG A-U */
  ["\u1178", "\u1178"], /* HANGUL JUNGSEONG YA-O */
  ["\u1179", "\u1179"], /* HANGUL JUNGSEONG YA-YO */
  ["\u117A", "\u117A"], /* HANGUL JUNGSEONG EO-O */
  ["\u117B", "\u117B"], /* HANGUL JUNGSEONG EO-U */
  ["\u117C", "\u117C"], /* HANGUL JUNGSEONG EO-EU */
  ["\u117D", "\u117D"], /* HANGUL JUNGSEONG YEO-O */
  ["\u117E", "\u117E"], /* HANGUL JUNGSEONG YEO-U */
  ["\u117F", "\u117F"], /* HANGUL JUNGSEONG O-EO */
  ["\u1180", "\u1180"], /* HANGUL JUNGSEONG O-E */
  ["\u1181", "\u1181"], /* HANGUL JUNGSEONG O-YE */
  ["\u1182", "\u1182"], /* HANGUL JUNGSEONG O-O */
  ["\u1183", "\u1183"], /* HANGUL JUNGSEONG O-U */
  ["\u1184", "\u1184"], /* HANGUL JUNGSEONG YO-YA */
  ["\u1185", "\u1185"], /* HANGUL JUNGSEONG YO-YAE */
  ["\u1186", "\u1186"], /* HANGUL JUNGSEONG YO-YEO */
  ["\u1187", "\u1187"], /* HANGUL JUNGSEONG YO-O */
  ["\u1188", "\u1188"], /* HANGUL JUNGSEONG YO-I */
  ["\u1189", "\u1189"], /* HANGUL JUNGSEONG U-A */
  ["\u118A", "\u118A"], /* HANGUL JUNGSEONG U-AE */
  ["\u118B", "\u118B"], /* HANGUL JUNGSEONG U-EO-EU */
  ["\u118C", "\u118C"], /* HANGUL JUNGSEONG U-YE */
  ["\u118D", "\u118D"], /* HANGUL JUNGSEONG U-U */
  ["\u118E", "\u118E"], /* HANGUL JUNGSEONG YU-A */
  ["\u118F", "\u118F"], /* HANGUL JUNGSEONG YU-EO */
  ["\u1190", "\u1190"], /* HANGUL JUNGSEONG YU-E */
  ["\u1191", "\u1191"], /* HANGUL JUNGSEONG YU-YEO */
  ["\u1192", "\u1192"], /* HANGUL JUNGSEONG YU-YE */
  ["\u1193", "\u1193"], /* HANGUL JUNGSEONG YU-U */
  ["\u1194", "\u1194"], /* HANGUL JUNGSEONG YU-I */
  ["\u1195", "\u1195"], /* HANGUL JUNGSEONG EU-U */
  ["\u1196", "\u1196"], /* HANGUL JUNGSEONG EU-EU */
  ["\u1197", "\u1197"], /* HANGUL JUNGSEONG YI-U */
  ["\u1198", "\u1198"], /* HANGUL JUNGSEONG I-A */
  ["\u1199", "\u1199"], /* HANGUL JUNGSEONG I-YA */
  ["\u119A", "\u119A"], /* HANGUL JUNGSEONG I-O */
  ["\u119B", "\u119B"], /* HANGUL JUNGSEONG I-U */
  ["\u119C", "\u119C"], /* HANGUL JUNGSEONG I-EU */
  ["\u119D", "\u119D"], /* HANGUL JUNGSEONG I-ARAEA */
  ["\u119E", "\u119E"], /* HANGUL JUNGSEONG ARAEA */
  ["\u119F", "\u119F"], /* HANGUL JUNGSEONG ARAEA-EO */
  ["\u11A0", "\u11A0"], /* HANGUL JUNGSEONG ARAEA-U */
  ["\u11A1", "\u11A1"], /* HANGUL JUNGSEONG ARAEA-I */
  ["\u11A2", "\u11A2"], /* HANGUL JUNGSEONG SSANGARAEA */
  ["\u11A3", "\u11A3"], /* HANGUL JUNGSEONG A-EU */
  ["\u11A4", "\u11A4"], /* HANGUL JUNGSEONG YA-U */
  ["\u11A5", "\u11A5"], /* HANGUL JUNGSEONG YEO-YA */
  ["\u11A6", "\u11A6"], /* HANGUL JUNGSEONG O-YA */
  ["\u11A7", "\u11A7"], /* HANGUL JUNGSEONG O-YAE */
  ["\u11A8", "\u11A8"], /* HANGUL JONGSEONG KIYEOK */
  ["\u11A9", "\u11A9"], /* HANGUL JONGSEONG SSANGKIYEOK */
  ["\u11AA", "\u11AA"], /* HANGUL JONGSEONG KIYEOK-SIOS */
  ["\u11AB", "\u11AB"], /* HANGUL JONGSEONG NIEUN */
  ["\u11AC", "\u11AC"], /* HANGUL JONGSEONG NIEUN-CIEUC */
  ["\u11AD", "\u11AD"], /* HANGUL JONGSEONG NIEUN-HIEUH */
  ["\u11AE", "\u11AE"], /* HANGUL JONGSEONG TIKEUT */
  ["\u11AF", "\u11AF"], /* HANGUL JONGSEONG RIEUL */
  ["\u11B0", "\u11B0"], /* HANGUL JONGSEONG RIEUL-KIYEOK */
  ["\u11B1", "\u11B1"], /* HANGUL JONGSEONG RIEUL-MIEUM */
  ["\u11B2", "\u11B2"], /* HANGUL JONGSEONG RIEUL-PIEUP */
  ["\u11B3", "\u11B3"], /* HANGUL JONGSEONG RIEUL-SIOS */
  ["\u11B4", "\u11B4"], /* HANGUL JONGSEONG RIEUL-THIEUTH */
  ["\u11B5", "\u11B5"], /* HANGUL JONGSEONG RIEUL-PHIEUPH */
  ["\u11B6", "\u11B6"], /* HANGUL JONGSEONG RIEUL-HIEUH */
  ["\u11B7", "\u11B7"], /* HANGUL JONGSEONG MIEUM */
  ["\u11B8", "\u11B8"], /* HANGUL JONGSEONG PIEUP */
  ["\u11B9", "\u11B9"], /* HANGUL JONGSEONG PIEUP-SIOS */
  ["\u11BA", "\u11BA"], /* HANGUL JONGSEONG SIOS */
  ["\u11BB", "\u11BB"], /* HANGUL JONGSEONG SSANGSIOS */
  ["\u11BC", "\u11BC"], /* HANGUL JONGSEONG IEUNG */
  ["\u11BD", "\u11BD"], /* HANGUL JONGSEONG CIEUC */
  ["\u11BE", "\u11BE"], /* HANGUL JONGSEONG CHIEUCH */
  ["\u11BF", "\u11BF"], /* HANGUL JONGSEONG KHIEUKH */
  ["\u11C0", "\u11C0"], /* HANGUL JONGSEONG THIEUTH */
  ["\u11C1", "\u11C1"], /* HANGUL JONGSEONG PHIEUPH */
  ["\u11C2", "\u11C2"], /* HANGUL JONGSEONG HIEUH */
  ["\u11C3", "\u11C3"], /* HANGUL JONGSEONG KIYEOK-RIEUL */
  ["\u11C4", "\u11C4"], /* HANGUL JONGSEONG KIYEOK-SIOS-KIYEOK */
  ["\u11C5", "\u11C5"], /* HANGUL JONGSEONG NIEUN-KIYEOK */
  ["\u11C6", "\u11C6"], /* HANGUL JONGSEONG NIEUN-TIKEUT */
  ["\u11C7", "\u11C7"], /* HANGUL JONGSEONG NIEUN-SIOS */
  ["\u11C8", "\u11C8"], /* HANGUL JONGSEONG NIEUN-PANSIOS */
  ["\u11C9", "\u11C9"], /* HANGUL JONGSEONG NIEUN-THIEUTH */
  ["\u11CA", "\u11CA"], /* HANGUL JONGSEONG TIKEUT-KIYEOK */
  ["\u11CB", "\u11CB"], /* HANGUL JONGSEONG TIKEUT-RIEUL */
  ["\u11CC", "\u11CC"], /* HANGUL JONGSEONG RIEUL-KIYEOK-SIOS */
  ["\u11CD", "\u11CD"], /* HANGUL JONGSEONG RIEUL-NIEUN */
  ["\u11CE", "\u11CE"], /* HANGUL JONGSEONG RIEUL-TIKEUT */
  ["\u11CF", "\u11CF"], /* HANGUL JONGSEONG RIEUL-TIKEUT-HIEUH */
  ["\u11D0", "\u11D0"], /* HANGUL JONGSEONG SSANGRIEUL */
  ["\u11D1", "\u11D1"], /* HANGUL JONGSEONG RIEUL-MIEUM-KIYEOK */
  ["\u11D2", "\u11D2"], /* HANGUL JONGSEONG RIEUL-MIEUM-SIOS */
  ["\u11D3", "\u11D3"], /* HANGUL JONGSEONG RIEUL-PIEUP-SIOS */
  ["\u11D4", "\u11D4"], /* HANGUL JONGSEONG RIEUL-PIEUP-HIEUH */
  ["\u11D5", "\u11D5"], /* HANGUL JONGSEONG RIEUL-KAPYEOUNPIEUP */
  ["\u11D6", "\u11D6"], /* HANGUL JONGSEONG RIEUL-SSANGSIOS */
  ["\u11D7", "\u11D7"], /* HANGUL JONGSEONG RIEUL-PANSIOS */
  ["\u11D8", "\u11D8"], /* HANGUL JONGSEONG RIEUL-KHIEUKH */
  ["\u11D9", "\u11D9"], /* HANGUL JONGSEONG RIEUL-YEORINHIEUH */
  ["\u11DA", "\u11DA"], /* HANGUL JONGSEONG MIEUM-KIYEOK */
  ["\u11DB", "\u11DB"], /* HANGUL JONGSEONG MIEUM-RIEUL */
  ["\u11DC", "\u11DC"], /* HANGUL JONGSEONG MIEUM-PIEUP */
  ["\u11DD", "\u11DD"], /* HANGUL JONGSEONG MIEUM-SIOS */
  ["\u11DE", "\u11DE"], /* HANGUL JONGSEONG MIEUM-SSANGSIOS */
  ["\u11DF", "\u11DF"], /* HANGUL JONGSEONG MIEUM-PANSIOS */
  ["\u11E0", "\u11E0"], /* HANGUL JONGSEONG MIEUM-CHIEUCH */
  ["\u11E1", "\u11E1"], /* HANGUL JONGSEONG MIEUM-HIEUH */
  ["\u11E2", "\u11E2"], /* HANGUL JONGSEONG KAPYEOUNMIEUM */
  ["\u11E3", "\u11E3"], /* HANGUL JONGSEONG PIEUP-RIEUL */
  ["\u11E4", "\u11E4"], /* HANGUL JONGSEONG PIEUP-PHIEUPH */
  ["\u11E5", "\u11E5"], /* HANGUL JONGSEONG PIEUP-HIEUH */
  ["\u11E6", "\u11E6"], /* HANGUL JONGSEONG KAPYEOUNPIEUP */
  ["\u11E7", "\u11E7"], /* HANGUL JONGSEONG SIOS-KIYEOK */
  ["\u11E8", "\u11E8"], /* HANGUL JONGSEONG SIOS-TIKEUT */
  ["\u11E9", "\u11E9"], /* HANGUL JONGSEONG SIOS-RIEUL */
  ["\u11EA", "\u11EA"], /* HANGUL JONGSEONG SIOS-PIEUP */
  ["\u11EB", "\u11EB"], /* HANGUL JONGSEONG PANSIOS */
  ["\u11EC", "\u11EC"], /* HANGUL JONGSEONG IEUNG-KIYEOK */
  ["\u11ED", "\u11ED"], /* HANGUL JONGSEONG IEUNG-SSANGKIYEOK */
  ["\u11EE", "\u11EE"], /* HANGUL JONGSEONG SSANGIEUNG */
  ["\u11EF", "\u11EF"], /* HANGUL JONGSEONG IEUNG-KHIEUKH */
  ["\u11F0", "\u11F0"], /* HANGUL JONGSEONG YESIEUNG */
  ["\u11F1", "\u11F1"], /* HANGUL JONGSEONG YESIEUNG-SIOS */
  ["\u11F2", "\u11F2"], /* HANGUL JONGSEONG YESIEUNG-PANSIOS */
  ["\u11F3", "\u11F3"], /* HANGUL JONGSEONG PHIEUPH-PIEUP */
  ["\u11F4", "\u11F4"], /* HANGUL JONGSEONG KAPYEOUNPHIEUPH */
  ["\u11F5", "\u11F5"], /* HANGUL JONGSEONG HIEUH-NIEUN */
  ["\u11F6", "\u11F6"], /* HANGUL JONGSEONG HIEUH-RIEUL */
  ["\u11F7", "\u11F7"], /* HANGUL JONGSEONG HIEUH-MIEUM */
  ["\u11F8", "\u11F8"], /* HANGUL JONGSEONG HIEUH-PIEUP */
  ["\u11F9", "\u11F9"], /* HANGUL JONGSEONG YEORINHIEUH */
  ["\u11FA", "\u11FA"], /* HANGUL JONGSEONG KIYEOK-NIEUN */
  ["\u11FB", "\u11FB"], /* HANGUL JONGSEONG KIYEOK-PIEUP */
  ["\u11FC", "\u11FC"], /* HANGUL JONGSEONG KIYEOK-CHIEUCH */
  ["\u11FD", "\u11FD"], /* HANGUL JONGSEONG KIYEOK-KHIEUKH */
  ["\u11FE", "\u11FE"], /* HANGUL JONGSEONG KIYEOK-HIEUH */
  ["\u11FF", "\u11FF"], /* HANGUL JONGSEONG SSANGNIEUN */
  ["\u1200", "\u1200"], /* ETHIOPIC SYLLABLE HA */
  ["\u1201", "\u1201"], /* ETHIOPIC SYLLABLE HU */
  ["\u1202", "\u1202"], /* ETHIOPIC SYLLABLE HI */
  ["\u1203", "\u1203"], /* ETHIOPIC SYLLABLE HAA */
  ["\u1204", "\u1204"], /* ETHIOPIC SYLLABLE HEE */
  ["\u1205", "\u1205"], /* ETHIOPIC SYLLABLE HE */
  ["\u1206", "\u1206"], /* ETHIOPIC SYLLABLE HO */
  ["\u1207", "\u1207"], /* ETHIOPIC SYLLABLE HOA */
  ["\u1208", "\u1208"], /* ETHIOPIC SYLLABLE LA */
  ["\u1209", "\u1209"], /* ETHIOPIC SYLLABLE LU */
  ["\u120A", "\u120A"], /* ETHIOPIC SYLLABLE LI */
  ["\u120B", "\u120B"], /* ETHIOPIC SYLLABLE LAA */
  ["\u120C", "\u120C"], /* ETHIOPIC SYLLABLE LEE */
  ["\u120D", "\u120D"], /* ETHIOPIC SYLLABLE LE */
  ["\u120E", "\u120E"], /* ETHIOPIC SYLLABLE LO */
  ["\u120F", "\u120F"], /* ETHIOPIC SYLLABLE LWA */
  ["\u1210", "\u1210"], /* ETHIOPIC SYLLABLE HHA */
  ["\u1211", "\u1211"], /* ETHIOPIC SYLLABLE HHU */
  ["\u1212", "\u1212"], /* ETHIOPIC SYLLABLE HHI */
  ["\u1213", "\u1213"], /* ETHIOPIC SYLLABLE HHAA */
  ["\u1214", "\u1214"], /* ETHIOPIC SYLLABLE HHEE */
  ["\u1215", "\u1215"], /* ETHIOPIC SYLLABLE HHE */
  ["\u1216", "\u1216"], /* ETHIOPIC SYLLABLE HHO */
  ["\u1217", "\u1217"], /* ETHIOPIC SYLLABLE HHWA */
  ["\u1218", "\u1218"], /* ETHIOPIC SYLLABLE MA */
  ["\u1219", "\u1219"], /* ETHIOPIC SYLLABLE MU */
  ["\u121A", "\u121A"], /* ETHIOPIC SYLLABLE MI */
  ["\u121B", "\u121B"], /* ETHIOPIC SYLLABLE MAA */
  ["\u121C", "\u121C"], /* ETHIOPIC SYLLABLE MEE */
  ["\u121D", "\u121D"], /* ETHIOPIC SYLLABLE ME */
  ["\u121E", "\u121E"], /* ETHIOPIC SYLLABLE MO */
  ["\u121F", "\u121F"], /* ETHIOPIC SYLLABLE MWA */
  ["\u1220", "\u1220"], /* ETHIOPIC SYLLABLE SZA */
  ["\u1221", "\u1221"], /* ETHIOPIC SYLLABLE SZU */
  ["\u1222", "\u1222"], /* ETHIOPIC SYLLABLE SZI */
  ["\u1223", "\u1223"], /* ETHIOPIC SYLLABLE SZAA */
  ["\u1224", "\u1224"], /* ETHIOPIC SYLLABLE SZEE */
  ["\u1225", "\u1225"], /* ETHIOPIC SYLLABLE SZE */
  ["\u1226", "\u1226"], /* ETHIOPIC SYLLABLE SZO */
  ["\u1227", "\u1227"], /* ETHIOPIC SYLLABLE SZWA */
  ["\u1228", "\u1228"], /* ETHIOPIC SYLLABLE RA */
  ["\u1229", "\u1229"], /* ETHIOPIC SYLLABLE RU */
  ["\u122A", "\u122A"], /* ETHIOPIC SYLLABLE RI */
  ["\u122B", "\u122B"], /* ETHIOPIC SYLLABLE RAA */
  ["\u122C", "\u122C"], /* ETHIOPIC SYLLABLE REE */
  ["\u122D", "\u122D"], /* ETHIOPIC SYLLABLE RE */
  ["\u122E", "\u122E"], /* ETHIOPIC SYLLABLE RO */
  ["\u122F", "\u122F"], /* ETHIOPIC SYLLABLE RWA */
  ["\u1230", "\u1230"], /* ETHIOPIC SYLLABLE SA */
  ["\u1231", "\u1231"], /* ETHIOPIC SYLLABLE SU */
  ["\u1232", "\u1232"], /* ETHIOPIC SYLLABLE SI */
  ["\u1233", "\u1233"], /* ETHIOPIC SYLLABLE SAA */
  ["\u1234", "\u1234"], /* ETHIOPIC SYLLABLE SEE */
  ["\u1235", "\u1235"], /* ETHIOPIC SYLLABLE SE */
  ["\u1236", "\u1236"], /* ETHIOPIC SYLLABLE SO */
  ["\u1237", "\u1237"], /* ETHIOPIC SYLLABLE SWA */
  ["\u1238", "\u1238"], /* ETHIOPIC SYLLABLE SHA */
  ["\u1239", "\u1239"], /* ETHIOPIC SYLLABLE SHU */
  ["\u123A", "\u123A"], /* ETHIOPIC SYLLABLE SHI */
  ["\u123B", "\u123B"], /* ETHIOPIC SYLLABLE SHAA */
  ["\u123C", "\u123C"], /* ETHIOPIC SYLLABLE SHEE */
  ["\u123D", "\u123D"], /* ETHIOPIC SYLLABLE SHE */
  ["\u123E", "\u123E"], /* ETHIOPIC SYLLABLE SHO */
  ["\u123F", "\u123F"], /* ETHIOPIC SYLLABLE SHWA */
  ["\u1240", "\u1240"], /* ETHIOPIC SYLLABLE QA */
  ["\u1241", "\u1241"], /* ETHIOPIC SYLLABLE QU */
  ["\u1242", "\u1242"], /* ETHIOPIC SYLLABLE QI */
  ["\u1243", "\u1243"], /* ETHIOPIC SYLLABLE QAA */
  ["\u1244", "\u1244"], /* ETHIOPIC SYLLABLE QEE */
  ["\u1245", "\u1245"], /* ETHIOPIC SYLLABLE QE */
  ["\u1246", "\u1246"], /* ETHIOPIC SYLLABLE QO */
  ["\u1247", "\u1247"], /* ETHIOPIC SYLLABLE QOA */
  ["\u1248", "\u1248"], /* ETHIOPIC SYLLABLE QWA */
  ["\u1249", "\u1249"],
  ["\u124A", "\u124A"], /* ETHIOPIC SYLLABLE QWI */
  ["\u124B", "\u124B"], /* ETHIOPIC SYLLABLE QWAA */
  ["\u124C", "\u124C"], /* ETHIOPIC SYLLABLE QWEE */
  ["\u124D", "\u124D"], /* ETHIOPIC SYLLABLE QWE */
  ["\u124E", "\u124E"],
  ["\u124F", "\u124F"],
  ["\u1250", "\u1250"], /* ETHIOPIC SYLLABLE QHA */
  ["\u1251", "\u1251"], /* ETHIOPIC SYLLABLE QHU */
  ["\u1252", "\u1252"], /* ETHIOPIC SYLLABLE QHI */
  ["\u1253", "\u1253"], /* ETHIOPIC SYLLABLE QHAA */
  ["\u1254", "\u1254"], /* ETHIOPIC SYLLABLE QHEE */
  ["\u1255", "\u1255"], /* ETHIOPIC SYLLABLE QHE */
  ["\u1256", "\u1256"], /* ETHIOPIC SYLLABLE QHO */
  ["\u1257", "\u1257"],
  ["\u1258", "\u1258"], /* ETHIOPIC SYLLABLE QHWA */
  ["\u1259", "\u1259"],
  ["\u125A", "\u125A"], /* ETHIOPIC SYLLABLE QHWI */
  ["\u125B", "\u125B"], /* ETHIOPIC SYLLABLE QHWAA */
  ["\u125C", "\u125C"], /* ETHIOPIC SYLLABLE QHWEE */
  ["\u125D", "\u125D"], /* ETHIOPIC SYLLABLE QHWE */
  ["\u125E", "\u125E"],
  ["\u125F", "\u125F"],
  ["\u1260", "\u1260"], /* ETHIOPIC SYLLABLE BA */
  ["\u1261", "\u1261"], /* ETHIOPIC SYLLABLE BU */
  ["\u1262", "\u1262"], /* ETHIOPIC SYLLABLE BI */
  ["\u1263", "\u1263"], /* ETHIOPIC SYLLABLE BAA */
  ["\u1264", "\u1264"], /* ETHIOPIC SYLLABLE BEE */
  ["\u1265", "\u1265"], /* ETHIOPIC SYLLABLE BE */
  ["\u1266", "\u1266"], /* ETHIOPIC SYLLABLE BO */
  ["\u1267", "\u1267"], /* ETHIOPIC SYLLABLE BWA */
  ["\u1268", "\u1268"], /* ETHIOPIC SYLLABLE VA */
  ["\u1269", "\u1269"], /* ETHIOPIC SYLLABLE VU */
  ["\u126A", "\u126A"], /* ETHIOPIC SYLLABLE VI */
  ["\u126B", "\u126B"], /* ETHIOPIC SYLLABLE VAA */
  ["\u126C", "\u126C"], /* ETHIOPIC SYLLABLE VEE */
  ["\u126D", "\u126D"], /* ETHIOPIC SYLLABLE VE */
  ["\u126E", "\u126E"], /* ETHIOPIC SYLLABLE VO */
  ["\u126F", "\u126F"], /* ETHIOPIC SYLLABLE VWA */
  ["\u1270", "\u1270"], /* ETHIOPIC SYLLABLE TA */
  ["\u1271", "\u1271"], /* ETHIOPIC SYLLABLE TU */
  ["\u1272", "\u1272"], /* ETHIOPIC SYLLABLE TI */
  ["\u1273", "\u1273"], /* ETHIOPIC SYLLABLE TAA */
  ["\u1274", "\u1274"], /* ETHIOPIC SYLLABLE TEE */
  ["\u1275", "\u1275"], /* ETHIOPIC SYLLABLE TE */
  ["\u1276", "\u1276"], /* ETHIOPIC SYLLABLE TO */
  ["\u1277", "\u1277"], /* ETHIOPIC SYLLABLE TWA */
  ["\u1278", "\u1278"], /* ETHIOPIC SYLLABLE CA */
  ["\u1279", "\u1279"], /* ETHIOPIC SYLLABLE CU */
  ["\u127A", "\u127A"], /* ETHIOPIC SYLLABLE CI */
  ["\u127B", "\u127B"], /* ETHIOPIC SYLLABLE CAA */
  ["\u127C", "\u127C"], /* ETHIOPIC SYLLABLE CEE */
  ["\u127D", "\u127D"], /* ETHIOPIC SYLLABLE CE */
  ["\u127E", "\u127E"], /* ETHIOPIC SYLLABLE CO */
  ["\u127F", "\u127F"], /* ETHIOPIC SYLLABLE CWA */
  ["\u1280", "\u1280"], /* ETHIOPIC SYLLABLE XA */
  ["\u1281", "\u1281"], /* ETHIOPIC SYLLABLE XU */
  ["\u1282", "\u1282"], /* ETHIOPIC SYLLABLE XI */
  ["\u1283", "\u1283"], /* ETHIOPIC SYLLABLE XAA */
  ["\u1284", "\u1284"], /* ETHIOPIC SYLLABLE XEE */
  ["\u1285", "\u1285"], /* ETHIOPIC SYLLABLE XE */
  ["\u1286", "\u1286"], /* ETHIOPIC SYLLABLE XO */
  ["\u1287", "\u1287"], /* ETHIOPIC SYLLABLE XOA */
  ["\u1288", "\u1288"], /* ETHIOPIC SYLLABLE XWA */
  ["\u1289", "\u1289"],
  ["\u128A", "\u128A"], /* ETHIOPIC SYLLABLE XWI */
  ["\u128B", "\u128B"], /* ETHIOPIC SYLLABLE XWAA */
  ["\u128C", "\u128C"], /* ETHIOPIC SYLLABLE XWEE */
  ["\u128D", "\u128D"], /* ETHIOPIC SYLLABLE XWE */
  ["\u128E", "\u128E"],
  ["\u128F", "\u128F"],
  ["\u1290", "\u1290"], /* ETHIOPIC SYLLABLE NA */
  ["\u1291", "\u1291"], /* ETHIOPIC SYLLABLE NU */
  ["\u1292", "\u1292"], /* ETHIOPIC SYLLABLE NI */
  ["\u1293", "\u1293"], /* ETHIOPIC SYLLABLE NAA */
  ["\u1294", "\u1294"], /* ETHIOPIC SYLLABLE NEE */
  ["\u1295", "\u1295"], /* ETHIOPIC SYLLABLE NE */
  ["\u1296", "\u1296"], /* ETHIOPIC SYLLABLE NO */
  ["\u1297", "\u1297"], /* ETHIOPIC SYLLABLE NWA */
  ["\u1298", "\u1298"], /* ETHIOPIC SYLLABLE NYA */
  ["\u1299", "\u1299"], /* ETHIOPIC SYLLABLE NYU */
  ["\u129A", "\u129A"], /* ETHIOPIC SYLLABLE NYI */
  ["\u129B", "\u129B"], /* ETHIOPIC SYLLABLE NYAA */
  ["\u129C", "\u129C"], /* ETHIOPIC SYLLABLE NYEE */
  ["\u129D", "\u129D"], /* ETHIOPIC SYLLABLE NYE */
  ["\u129E", "\u129E"], /* ETHIOPIC SYLLABLE NYO */
  ["\u129F", "\u129F"], /* ETHIOPIC SYLLABLE NYWA */
  ["\u12A0", "\u12A0"], /* ETHIOPIC SYLLABLE GLOTTAL A */
  ["\u12A1", "\u12A1"], /* ETHIOPIC SYLLABLE GLOTTAL U */
  ["\u12A2", "\u12A2"], /* ETHIOPIC SYLLABLE GLOTTAL I */
  ["\u12A3", "\u12A3"], /* ETHIOPIC SYLLABLE GLOTTAL AA */
  ["\u12A4", "\u12A4"], /* ETHIOPIC SYLLABLE GLOTTAL EE */
  ["\u12A5", "\u12A5"], /* ETHIOPIC SYLLABLE GLOTTAL E */
  ["\u12A6", "\u12A6"], /* ETHIOPIC SYLLABLE GLOTTAL O */
  ["\u12A7", "\u12A7"], /* ETHIOPIC SYLLABLE GLOTTAL WA */
  ["\u12A8", "\u12A8"], /* ETHIOPIC SYLLABLE KA */
  ["\u12A9", "\u12A9"], /* ETHIOPIC SYLLABLE KU */
  ["\u12AA", "\u12AA"], /* ETHIOPIC SYLLABLE KI */
  ["\u12AB", "\u12AB"], /* ETHIOPIC SYLLABLE KAA */
  ["\u12AC", "\u12AC"], /* ETHIOPIC SYLLABLE KEE */
  ["\u12AD", "\u12AD"], /* ETHIOPIC SYLLABLE KE */
  ["\u12AE", "\u12AE"], /* ETHIOPIC SYLLABLE KO */
  ["\u12AF", "\u12AF"], /* ETHIOPIC SYLLABLE KOA */
  ["\u12B0", "\u12B0"], /* ETHIOPIC SYLLABLE KWA */
  ["\u12B1", "\u12B1"],
  ["\u12B2", "\u12B2"], /* ETHIOPIC SYLLABLE KWI */
  ["\u12B3", "\u12B3"], /* ETHIOPIC SYLLABLE KWAA */
  ["\u12B4", "\u12B4"], /* ETHIOPIC SYLLABLE KWEE */
  ["\u12B5", "\u12B5"], /* ETHIOPIC SYLLABLE KWE */
  ["\u12B6", "\u12B6"],
  ["\u12B7", "\u12B7"],
  ["\u12B8", "\u12B8"], /* ETHIOPIC SYLLABLE KXA */
  ["\u12B9", "\u12B9"], /* ETHIOPIC SYLLABLE KXU */
  ["\u12BA", "\u12BA"], /* ETHIOPIC SYLLABLE KXI */
  ["\u12BB", "\u12BB"], /* ETHIOPIC SYLLABLE KXAA */
  ["\u12BC", "\u12BC"], /* ETHIOPIC SYLLABLE KXEE */
  ["\u12BD", "\u12BD"], /* ETHIOPIC SYLLABLE KXE */
  ["\u12BE", "\u12BE"], /* ETHIOPIC SYLLABLE KXO */
  ["\u12BF", "\u12BF"],
  ["\u12C0", "\u12C0"], /* ETHIOPIC SYLLABLE KXWA */
  ["\u12C1", "\u12C1"],
  ["\u12C2", "\u12C2"], /* ETHIOPIC SYLLABLE KXWI */
  ["\u12C3", "\u12C3"], /* ETHIOPIC SYLLABLE KXWAA */
  ["\u12C4", "\u12C4"], /* ETHIOPIC SYLLABLE KXWEE */
  ["\u12C5", "\u12C5"], /* ETHIOPIC SYLLABLE KXWE */
  ["\u12C6", "\u12C6"],
  ["\u12C7", "\u12C7"],
  ["\u12C8", "\u12C8"], /* ETHIOPIC SYLLABLE WA */
  ["\u12C9", "\u12C9"], /* ETHIOPIC SYLLABLE WU */
  ["\u12CA", "\u12CA"], /* ETHIOPIC SYLLABLE WI */
  ["\u12CB", "\u12CB"], /* ETHIOPIC SYLLABLE WAA */
  ["\u12CC", "\u12CC"], /* ETHIOPIC SYLLABLE WEE */
  ["\u12CD", "\u12CD"], /* ETHIOPIC SYLLABLE WE */
  ["\u12CE", "\u12CE"], /* ETHIOPIC SYLLABLE WO */
  ["\u12CF", "\u12CF"], /* ETHIOPIC SYLLABLE WOA */
  ["\u12D0", "\u12D0"], /* ETHIOPIC SYLLABLE PHARYNGEAL A */
  ["\u12D1", "\u12D1"], /* ETHIOPIC SYLLABLE PHARYNGEAL U */
  ["\u12D2", "\u12D2"], /* ETHIOPIC SYLLABLE PHARYNGEAL I */
  ["\u12D3", "\u12D3"], /* ETHIOPIC SYLLABLE PHARYNGEAL AA */
  ["\u12D4", "\u12D4"], /* ETHIOPIC SYLLABLE PHARYNGEAL EE */
  ["\u12D5", "\u12D5"], /* ETHIOPIC SYLLABLE PHARYNGEAL E */
  ["\u12D6", "\u12D6"], /* ETHIOPIC SYLLABLE PHARYNGEAL O */
  ["\u12D7", "\u12D7"],
  ["\u12D8", "\u12D8"], /* ETHIOPIC SYLLABLE ZA */
  ["\u12D9", "\u12D9"], /* ETHIOPIC SYLLABLE ZU */
  ["\u12DA", "\u12DA"], /* ETHIOPIC SYLLABLE ZI */
  ["\u12DB", "\u12DB"], /* ETHIOPIC SYLLABLE ZAA */
  ["\u12DC", "\u12DC"], /* ETHIOPIC SYLLABLE ZEE */
  ["\u12DD", "\u12DD"], /* ETHIOPIC SYLLABLE ZE */
  ["\u12DE", "\u12DE"], /* ETHIOPIC SYLLABLE ZO */
  ["\u12DF", "\u12DF"], /* ETHIOPIC SYLLABLE ZWA */
  ["\u12E0", "\u12E0"], /* ETHIOPIC SYLLABLE ZHA */
  ["\u12E1", "\u12E1"], /* ETHIOPIC SYLLABLE ZHU */
  ["\u12E2", "\u12E2"], /* ETHIOPIC SYLLABLE ZHI */
  ["\u12E3", "\u12E3"], /* ETHIOPIC SYLLABLE ZHAA */
  ["\u12E4", "\u12E4"], /* ETHIOPIC SYLLABLE ZHEE */
  ["\u12E5", "\u12E5"], /* ETHIOPIC SYLLABLE ZHE */
  ["\u12E6", "\u12E6"], /* ETHIOPIC SYLLABLE ZHO */
  ["\u12E7", "\u12E7"], /* ETHIOPIC SYLLABLE ZHWA */
  ["\u12E8", "\u12E8"], /* ETHIOPIC SYLLABLE YA */
  ["\u12E9", "\u12E9"], /* ETHIOPIC SYLLABLE YU */
  ["\u12EA", "\u12EA"], /* ETHIOPIC SYLLABLE YI */
  ["\u12EB", "\u12EB"], /* ETHIOPIC SYLLABLE YAA */
  ["\u12EC", "\u12EC"], /* ETHIOPIC SYLLABLE YEE */
  ["\u12ED", "\u12ED"], /* ETHIOPIC SYLLABLE YE */
  ["\u12EE", "\u12EE"], /* ETHIOPIC SYLLABLE YO */
  ["\u12EF", "\u12EF"], /* ETHIOPIC SYLLABLE YOA */
  ["\u12F0", "\u12F0"], /* ETHIOPIC SYLLABLE DA */
  ["\u12F1", "\u12F1"], /* ETHIOPIC SYLLABLE DU */
  ["\u12F2", "\u12F2"], /* ETHIOPIC SYLLABLE DI */
  ["\u12F3", "\u12F3"], /* ETHIOPIC SYLLABLE DAA */
  ["\u12F4", "\u12F4"], /* ETHIOPIC SYLLABLE DEE */
  ["\u12F5", "\u12F5"], /* ETHIOPIC SYLLABLE DE */
  ["\u12F6", "\u12F6"], /* ETHIOPIC SYLLABLE DO */
  ["\u12F7", "\u12F7"], /* ETHIOPIC SYLLABLE DWA */
  ["\u12F8", "\u12F8"], /* ETHIOPIC SYLLABLE DDA */
  ["\u12F9", "\u12F9"], /* ETHIOPIC SYLLABLE DDU */
  ["\u12FA", "\u12FA"], /* ETHIOPIC SYLLABLE DDI */
  ["\u12FB", "\u12FB"], /* ETHIOPIC SYLLABLE DDAA */
  ["\u12FC", "\u12FC"], /* ETHIOPIC SYLLABLE DDEE */
  ["\u12FD", "\u12FD"], /* ETHIOPIC SYLLABLE DDE */
  ["\u12FE", "\u12FE"], /* ETHIOPIC SYLLABLE DDO */
  ["\u12FF", "\u12FF"], /* ETHIOPIC SYLLABLE DDWA */
  ["\u1300", "\u1300"], /* ETHIOPIC SYLLABLE JA */
  ["\u1301", "\u1301"], /* ETHIOPIC SYLLABLE JU */
  ["\u1302", "\u1302"], /* ETHIOPIC SYLLABLE JI */
  ["\u1303", "\u1303"], /* ETHIOPIC SYLLABLE JAA */
  ["\u1304", "\u1304"], /* ETHIOPIC SYLLABLE JEE */
  ["\u1305", "\u1305"], /* ETHIOPIC SYLLABLE JE */
  ["\u1306", "\u1306"], /* ETHIOPIC SYLLABLE JO */
  ["\u1307", "\u1307"], /* ETHIOPIC SYLLABLE JWA */
  ["\u1308", "\u1308"], /* ETHIOPIC SYLLABLE GA */
  ["\u1309", "\u1309"], /* ETHIOPIC SYLLABLE GU */
  ["\u130A", "\u130A"], /* ETHIOPIC SYLLABLE GI */
  ["\u130B", "\u130B"], /* ETHIOPIC SYLLABLE GAA */
  ["\u130C", "\u130C"], /* ETHIOPIC SYLLABLE GEE */
  ["\u130D", "\u130D"], /* ETHIOPIC SYLLABLE GE */
  ["\u130E", "\u130E"], /* ETHIOPIC SYLLABLE GO */
  ["\u130F", "\u130F"], /* ETHIOPIC SYLLABLE GOA */
  ["\u1310", "\u1310"], /* ETHIOPIC SYLLABLE GWA */
  ["\u1311", "\u1311"],
  ["\u1312", "\u1312"], /* ETHIOPIC SYLLABLE GWI */
  ["\u1313", "\u1313"], /* ETHIOPIC SYLLABLE GWAA */
  ["\u1314", "\u1314"], /* ETHIOPIC SYLLABLE GWEE */
  ["\u1315", "\u1315"], /* ETHIOPIC SYLLABLE GWE */
  ["\u1316", "\u1316"],
  ["\u1317", "\u1317"],
  ["\u1318", "\u1318"], /* ETHIOPIC SYLLABLE GGA */
  ["\u1319", "\u1319"], /* ETHIOPIC SYLLABLE GGU */
  ["\u131A", "\u131A"], /* ETHIOPIC SYLLABLE GGI */
  ["\u131B", "\u131B"], /* ETHIOPIC SYLLABLE GGAA */
  ["\u131C", "\u131C"], /* ETHIOPIC SYLLABLE GGEE */
  ["\u131D", "\u131D"], /* ETHIOPIC SYLLABLE GGE */
  ["\u131E", "\u131E"], /* ETHIOPIC SYLLABLE GGO */
  ["\u131F", "\u131F"], /* ETHIOPIC SYLLABLE GGWAA */
  ["\u1320", "\u1320"], /* ETHIOPIC SYLLABLE THA */
  ["\u1321", "\u1321"], /* ETHIOPIC SYLLABLE THU */
  ["\u1322", "\u1322"], /* ETHIOPIC SYLLABLE THI */
  ["\u1323", "\u1323"], /* ETHIOPIC SYLLABLE THAA */
  ["\u1324", "\u1324"], /* ETHIOPIC SYLLABLE THEE */
  ["\u1325", "\u1325"], /* ETHIOPIC SYLLABLE THE */
  ["\u1326", "\u1326"], /* ETHIOPIC SYLLABLE THO */
  ["\u1327", "\u1327"], /* ETHIOPIC SYLLABLE THWA */
  ["\u1328", "\u1328"], /* ETHIOPIC SYLLABLE CHA */
  ["\u1329", "\u1329"], /* ETHIOPIC SYLLABLE CHU */
  ["\u132A", "\u132A"], /* ETHIOPIC SYLLABLE CHI */
  ["\u132B", "\u132B"], /* ETHIOPIC SYLLABLE CHAA */
  ["\u132C", "\u132C"], /* ETHIOPIC SYLLABLE CHEE */
  ["\u132D", "\u132D"], /* ETHIOPIC SYLLABLE CHE */
  ["\u132E", "\u132E"], /* ETHIOPIC SYLLABLE CHO */
  ["\u132F", "\u132F"], /* ETHIOPIC SYLLABLE CHWA */
  ["\u1330", "\u1330"], /* ETHIOPIC SYLLABLE PHA */
  ["\u1331", "\u1331"], /* ETHIOPIC SYLLABLE PHU */
  ["\u1332", "\u1332"], /* ETHIOPIC SYLLABLE PHI */
  ["\u1333", "\u1333"], /* ETHIOPIC SYLLABLE PHAA */
  ["\u1334", "\u1334"], /* ETHIOPIC SYLLABLE PHEE */
  ["\u1335", "\u1335"], /* ETHIOPIC SYLLABLE PHE */
  ["\u1336", "\u1336"], /* ETHIOPIC SYLLABLE PHO */
  ["\u1337", "\u1337"], /* ETHIOPIC SYLLABLE PHWA */
  ["\u1338", "\u1338"], /* ETHIOPIC SYLLABLE TSA */
  ["\u1339", "\u1339"], /* ETHIOPIC SYLLABLE TSU */
  ["\u133A", "\u133A"], /* ETHIOPIC SYLLABLE TSI */
  ["\u133B", "\u133B"], /* ETHIOPIC SYLLABLE TSAA */
  ["\u133C", "\u133C"], /* ETHIOPIC SYLLABLE TSEE */
  ["\u133D", "\u133D"], /* ETHIOPIC SYLLABLE TSE */
  ["\u133E", "\u133E"], /* ETHIOPIC SYLLABLE TSO */
  ["\u133F", "\u133F"], /* ETHIOPIC SYLLABLE TSWA */
  ["\u1340", "\u1340"], /* ETHIOPIC SYLLABLE TZA */
  ["\u1341", "\u1341"], /* ETHIOPIC SYLLABLE TZU */
  ["\u1342", "\u1342"], /* ETHIOPIC SYLLABLE TZI */
  ["\u1343", "\u1343"], /* ETHIOPIC SYLLABLE TZAA */
  ["\u1344", "\u1344"], /* ETHIOPIC SYLLABLE TZEE */
  ["\u1345", "\u1345"], /* ETHIOPIC SYLLABLE TZE */
  ["\u1346", "\u1346"], /* ETHIOPIC SYLLABLE TZO */
  ["\u1347", "\u1347"], /* ETHIOPIC SYLLABLE TZOA */
  ["\u1348", "\u1348"], /* ETHIOPIC SYLLABLE FA */
  ["\u1349", "\u1349"], /* ETHIOPIC SYLLABLE FU */
  ["\u134A", "\u134A"], /* ETHIOPIC SYLLABLE FI */
  ["\u134B", "\u134B"], /* ETHIOPIC SYLLABLE FAA */
  ["\u134C", "\u134C"], /* ETHIOPIC SYLLABLE FEE */
  ["\u134D", "\u134D"], /* ETHIOPIC SYLLABLE FE */
  ["\u134E", "\u134E"], /* ETHIOPIC SYLLABLE FO */
  ["\u134F", "\u134F"], /* ETHIOPIC SYLLABLE FWA */
  ["\u1350", "\u1350"], /* ETHIOPIC SYLLABLE PA */
  ["\u1351", "\u1351"], /* ETHIOPIC SYLLABLE PU */
  ["\u1352", "\u1352"], /* ETHIOPIC SYLLABLE PI */
  ["\u1353", "\u1353"], /* ETHIOPIC SYLLABLE PAA */
  ["\u1354", "\u1354"], /* ETHIOPIC SYLLABLE PEE */
  ["\u1355", "\u1355"], /* ETHIOPIC SYLLABLE PE */
  ["\u1356", "\u1356"], /* ETHIOPIC SYLLABLE PO */
  ["\u1357", "\u1357"], /* ETHIOPIC SYLLABLE PWA */
  ["\u1358", "\u1358"], /* ETHIOPIC SYLLABLE RYA */
  ["\u1359", "\u1359"], /* ETHIOPIC SYLLABLE MYA */
  ["\u135A", "\u135A"], /* ETHIOPIC SYLLABLE FYA */
  ["\u135B", "\u135B"],
  ["\u135C", "\u135C"],
  ["\u135D", "\u135D"], /* ETHIOPIC COMBINING GEMINATION AND VOWEL LENGTH MARK */
  ["\u135E", "\u135E"], /* ETHIOPIC COMBINING VOWEL LENGTH MARK */
  ["\u135F", "\u135F"], /* ETHIOPIC COMBINING GEMINATION MARK */
  ["\u1360", "\u1360"], /* ETHIOPIC SECTION MARK */
  ["\u1361", "\u1361"], /* ETHIOPIC WORDSPACE */
  ["\u1362", "\u1362"], /* ETHIOPIC FULL STOP */
  ["\u1363", "\u1363"], /* ETHIOPIC COMMA */
  ["\u1364", "\u1364"], /* ETHIOPIC SEMICOLON */
  ["\u1365", "\u1365"], /* ETHIOPIC COLON */
  ["\u1366", "\u1366"], /* ETHIOPIC PREFACE COLON */
  ["\u1367", "\u1367"], /* ETHIOPIC QUESTION MARK */
  ["\u1368", "\u1368"], /* ETHIOPIC PARAGRAPH SEPARATOR */
  ["\u1369", "\u1369"], /* ETHIOPIC DIGIT ONE */
  ["\u136A", "\u136A"], /* ETHIOPIC DIGIT TWO */
  ["\u136B", "\u136B"], /* ETHIOPIC DIGIT THREE */
  ["\u136C", "\u136C"], /* ETHIOPIC DIGIT FOUR */
  ["\u136D", "\u136D"], /* ETHIOPIC DIGIT FIVE */
  ["\u136E", "\u136E"], /* ETHIOPIC DIGIT SIX */
  ["\u136F", "\u136F"], /* ETHIOPIC DIGIT SEVEN */
  ["\u1370", "\u1370"], /* ETHIOPIC DIGIT EIGHT */
  ["\u1371", "\u1371"], /* ETHIOPIC DIGIT NINE */
  ["\u1372", "\u1372"], /* ETHIOPIC NUMBER TEN */
  ["\u1373", "\u1373"], /* ETHIOPIC NUMBER TWENTY */
  ["\u1374", "\u1374"], /* ETHIOPIC NUMBER THIRTY */
  ["\u1375", "\u1375"], /* ETHIOPIC NUMBER FORTY */
  ["\u1376", "\u1376"], /* ETHIOPIC NUMBER FIFTY */
  ["\u1377", "\u1377"], /* ETHIOPIC NUMBER SIXTY */
  ["\u1378", "\u1378"], /* ETHIOPIC NUMBER SEVENTY */
  ["\u1379", "\u1379"], /* ETHIOPIC NUMBER EIGHTY */
  ["\u137A", "\u137A"], /* ETHIOPIC NUMBER NINETY */
  ["\u137B", "\u137B"], /* ETHIOPIC NUMBER HUNDRED */
  ["\u137C", "\u137C"], /* ETHIOPIC NUMBER TEN THOUSAND */
  ["\u137D", "\u137D"],
  ["\u137E", "\u137E"],
  ["\u137F", "\u137F"],
  ["\u1380", "\u1380"], /* ETHIOPIC SYLLABLE SEBATBEIT MWA */
  ["\u1381", "\u1381"], /* ETHIOPIC SYLLABLE MWI */
  ["\u1382", "\u1382"], /* ETHIOPIC SYLLABLE MWEE */
  ["\u1383", "\u1383"], /* ETHIOPIC SYLLABLE MWE */
  ["\u1384", "\u1384"], /* ETHIOPIC SYLLABLE SEBATBEIT BWA */
  ["\u1385", "\u1385"], /* ETHIOPIC SYLLABLE BWI */
  ["\u1386", "\u1386"], /* ETHIOPIC SYLLABLE BWEE */
  ["\u1387", "\u1387"], /* ETHIOPIC SYLLABLE BWE */
  ["\u1388", "\u1388"], /* ETHIOPIC SYLLABLE SEBATBEIT FWA */
  ["\u1389", "\u1389"], /* ETHIOPIC SYLLABLE FWI */
  ["\u138A", "\u138A"], /* ETHIOPIC SYLLABLE FWEE */
  ["\u138B", "\u138B"], /* ETHIOPIC SYLLABLE FWE */
  ["\u138C", "\u138C"], /* ETHIOPIC SYLLABLE SEBATBEIT PWA */
  ["\u138D", "\u138D"], /* ETHIOPIC SYLLABLE PWI */
  ["\u138E", "\u138E"], /* ETHIOPIC SYLLABLE PWEE */
  ["\u138F", "\u138F"], /* ETHIOPIC SYLLABLE PWE */
  ["\u1390", "\u1390"], /* ETHIOPIC TONAL MARK YIZET */
  ["\u1391", "\u1391"], /* ETHIOPIC TONAL MARK DERET */
  ["\u1392", "\u1392"], /* ETHIOPIC TONAL MARK RIKRIK */
  ["\u1393", "\u1393"], /* ETHIOPIC TONAL MARK SHORT RIKRIK */
  ["\u1394", "\u1394"], /* ETHIOPIC TONAL MARK DIFAT */
  ["\u1395", "\u1395"], /* ETHIOPIC TONAL MARK KENAT */
  ["\u1396", "\u1396"], /* ETHIOPIC TONAL MARK CHIRET */
  ["\u1397", "\u1397"], /* ETHIOPIC TONAL MARK HIDET */
  ["\u1398", "\u1398"], /* ETHIOPIC TONAL MARK DERET-HIDET */
  ["\u1399", "\u1399"], /* ETHIOPIC TONAL MARK KURT */
  ["\u139A", "\u139A"],
  ["\u139B", "\u139B"],
  ["\u139C", "\u139C"],
  ["\u139D", "\u139D"],
  ["\u139E", "\u139E"],
  ["\u139F", "\u139F"],
  ["\u13A0", "\uAB70"], /* CHEROKEE LETTER A */
  ["\u13A1", "\uAB71"], /* CHEROKEE LETTER E */
  ["\u13A2", "\uAB72"], /* CHEROKEE LETTER I */
  ["\u13A3", "\uAB73"], /* CHEROKEE LETTER O */
  ["\u13A4", "\uAB74"], /* CHEROKEE LETTER U */
  ["\u13A5", "\uAB75"], /* CHEROKEE LETTER V */
  ["\u13A6", "\uAB76"], /* CHEROKEE LETTER GA */
  ["\u13A7", "\uAB77"], /* CHEROKEE LETTER KA */
  ["\u13A8", "\uAB78"], /* CHEROKEE LETTER GE */
  ["\u13A9", "\uAB79"], /* CHEROKEE LETTER GI */
  ["\u13AA", "\uAB7A"], /* CHEROKEE LETTER GO */
  ["\u13AB", "\uAB7B"], /* CHEROKEE LETTER GU */
  ["\u13AC", "\uAB7C"], /* CHEROKEE LETTER GV */
  ["\u13AD", "\uAB7D"], /* CHEROKEE LETTER HA */
  ["\u13AE", "\uAB7E"], /* CHEROKEE LETTER HE */
  ["\u13AF", "\uAB7F"], /* CHEROKEE LETTER HI */
  ["\u13B0", "\uAB80"], /* CHEROKEE LETTER HO */
  ["\u13B1", "\uAB81"], /* CHEROKEE LETTER HU */
  ["\u13B2", "\uAB82"], /* CHEROKEE LETTER HV */
  ["\u13B3", "\uAB83"], /* CHEROKEE LETTER LA */
  ["\u13B4", "\uAB84"], /* CHEROKEE LETTER LE */
  ["\u13B5", "\uAB85"], /* CHEROKEE LETTER LI */
  ["\u13B6", "\uAB86"], /* CHEROKEE LETTER LO */
  ["\u13B7", "\uAB87"], /* CHEROKEE LETTER LU */
  ["\u13B8", "\uAB88"], /* CHEROKEE LETTER LV */
  ["\u13B9", "\uAB89"], /* CHEROKEE LETTER MA */
  ["\u13BA", "\uAB8A"], /* CHEROKEE LETTER ME */
  ["\u13BB", "\uAB8B"], /* CHEROKEE LETTER MI */
  ["\u13BC", "\uAB8C"], /* CHEROKEE LETTER MO */
  ["\u13BD", "\uAB8D"], /* CHEROKEE LETTER MU */
  ["\u13BE", "\uAB8E"], /* CHEROKEE LETTER NA */
  ["\u13BF", "\uAB8F"], /* CHEROKEE LETTER HNA */
  ["\u13C0", "\uAB90"], /* CHEROKEE LETTER NAH */
  ["\u13C1", "\uAB91"], /* CHEROKEE LETTER NE */
  ["\u13C2", "\uAB92"], /* CHEROKEE LETTER NI */
  ["\u13C3", "\uAB93"], /* CHEROKEE LETTER NO */
  ["\u13C4", "\uAB94"], /* CHEROKEE LETTER NU */
  ["\u13C5", "\uAB95"], /* CHEROKEE LETTER NV */
  ["\u13C6", "\uAB96"], /* CHEROKEE LETTER QUA */
  ["\u13C7", "\uAB97"], /* CHEROKEE LETTER QUE */
  ["\u13C8", "\uAB98"], /* CHEROKEE LETTER QUI */
  ["\u13C9", "\uAB99"], /* CHEROKEE LETTER QUO */
  ["\u13CA", "\uAB9A"], /* CHEROKEE LETTER QUU */
  ["\u13CB", "\uAB9B"], /* CHEROKEE LETTER QUV */
  ["\u13CC", "\uAB9C"], /* CHEROKEE LETTER SA */
  ["\u13CD", "\uAB9D"], /* CHEROKEE LETTER S */
  ["\u13CE", "\uAB9E"], /* CHEROKEE LETTER SE */
  ["\u13CF", "\uAB9F"], /* CHEROKEE LETTER SI */
  ["\u13D0", "\uABA0"], /* CHEROKEE LETTER SO */
  ["\u13D1", "\uABA1"], /* CHEROKEE LETTER SU */
  ["\u13D2", "\uABA2"], /* CHEROKEE LETTER SV */
  ["\u13D3", "\uABA3"], /* CHEROKEE LETTER DA */
  ["\u13D4", "\uABA4"], /* CHEROKEE LETTER TA */
  ["\u13D5", "\uABA5"], /* CHEROKEE LETTER DE */
  ["\u13D6", "\uABA6"], /* CHEROKEE LETTER TE */
  ["\u13D7", "\uABA7"], /* CHEROKEE LETTER DI */
  ["\u13D8", "\uABA8"], /* CHEROKEE LETTER TI */
  ["\u13D9", "\uABA9"], /* CHEROKEE LETTER DO */
  ["\u13DA", "\uABAA"], /* CHEROKEE LETTER DU */
  ["\u13DB", "\uABAB"], /* CHEROKEE LETTER DV */
  ["\u13DC", "\uABAC"], /* CHEROKEE LETTER DLA */
  ["\u13DD", "\uABAD"], /* CHEROKEE LETTER TLA */
  ["\u13DE", "\uABAE"], /* CHEROKEE LETTER TLE */
  ["\u13DF", "\uABAF"], /* CHEROKEE LETTER TLI */
  ["\u13E0", "\uABB0"], /* CHEROKEE LETTER TLO */
  ["\u13E1", "\uABB1"], /* CHEROKEE LETTER TLU */
  ["\u13E2", "\uABB2"], /* CHEROKEE LETTER TLV */
  ["\u13E3", "\uABB3"], /* CHEROKEE LETTER TSA */
  ["\u13E4", "\uABB4"], /* CHEROKEE LETTER TSE */
  ["\u13E5", "\uABB5"], /* CHEROKEE LETTER TSI */
  ["\u13E6", "\uABB6"], /* CHEROKEE LETTER TSO */
  ["\u13E7", "\uABB7"], /* CHEROKEE LETTER TSU */
  ["\u13E8", "\uABB8"], /* CHEROKEE LETTER TSV */
  ["\u13E9", "\uABB9"], /* CHEROKEE LETTER WA */
  ["\u13EA", "\uABBA"], /* CHEROKEE LETTER WE */
  ["\u13EB", "\uABBB"], /* CHEROKEE LETTER WI */
  ["\u13EC", "\uABBC"], /* CHEROKEE LETTER WO */
  ["\u13ED", "\uABBD"], /* CHEROKEE LETTER WU */
  ["\u13EE", "\uABBE"], /* CHEROKEE LETTER WV */
  ["\u13EF", "\uABBF"], /* CHEROKEE LETTER YA */
  ["\u13F0", "\u13F8"], /* CHEROKEE LETTER YE */
  ["\u13F1", "\u13F9"], /* CHEROKEE LETTER YI */
  ["\u13F2", "\u13FA"], /* CHEROKEE LETTER YO */
  ["\u13F3", "\u13FB"], /* CHEROKEE LETTER YU */
  ["\u13F4", "\u13FC"], /* CHEROKEE LETTER YV */
  ["\u13F5", "\u13FD"], /* CHEROKEE LETTER MV */
  ["\u13F6", "\u13F6"],
  ["\u13F7", "\u13F7"],
  ["\u13F0", "\u13F8"], /* CHEROKEE SMALL LETTER YE */
  ["\u13F1", "\u13F9"], /* CHEROKEE SMALL LETTER YI */
  ["\u13F2", "\u13FA"], /* CHEROKEE SMALL LETTER YO */
  ["\u13F3", "\u13FB"], /* CHEROKEE SMALL LETTER YU */
  ["\u13F4", "\u13FC"], /* CHEROKEE SMALL LETTER YV */
  ["\u13F5", "\u13FD"], /* CHEROKEE SMALL LETTER MV */
  ["\u13FE", "\u13FE"],
  ["\u13FF", "\u13FF"],
  ["\u1400", "\u1400"], /* CANADIAN SYLLABICS HYPHEN */
  ["\u1401", "\u1401"], /* CANADIAN SYLLABICS E */
  ["\u1402", "\u1402"], /* CANADIAN SYLLABICS AAI */
  ["\u1403", "\u1403"], /* CANADIAN SYLLABICS I */
  ["\u1404", "\u1404"], /* CANADIAN SYLLABICS II */
  ["\u1405", "\u1405"], /* CANADIAN SYLLABICS O */
  ["\u1406", "\u1406"], /* CANADIAN SYLLABICS OO */
  ["\u1407", "\u1407"], /* CANADIAN SYLLABICS Y-CREE OO */
  ["\u1408", "\u1408"], /* CANADIAN SYLLABICS CARRIER EE */
  ["\u1409", "\u1409"], /* CANADIAN SYLLABICS CARRIER I */
  ["\u140A", "\u140A"], /* CANADIAN SYLLABICS A */
  ["\u140B", "\u140B"], /* CANADIAN SYLLABICS AA */
  ["\u140C", "\u140C"], /* CANADIAN SYLLABICS WE */
  ["\u140D", "\u140D"], /* CANADIAN SYLLABICS WEST-CREE WE */
  ["\u140E", "\u140E"], /* CANADIAN SYLLABICS WI */
  ["\u140F", "\u140F"], /* CANADIAN SYLLABICS WEST-CREE WI */
  ["\u1410", "\u1410"], /* CANADIAN SYLLABICS WII */
  ["\u1411", "\u1411"], /* CANADIAN SYLLABICS WEST-CREE WII */
  ["\u1412", "\u1412"], /* CANADIAN SYLLABICS WO */
  ["\u1413", "\u1413"], /* CANADIAN SYLLABICS WEST-CREE WO */
  ["\u1414", "\u1414"], /* CANADIAN SYLLABICS WOO */
  ["\u1415", "\u1415"], /* CANADIAN SYLLABICS WEST-CREE WOO */
  ["\u1416", "\u1416"], /* CANADIAN SYLLABICS NASKAPI WOO */
  ["\u1417", "\u1417"], /* CANADIAN SYLLABICS WA */
  ["\u1418", "\u1418"], /* CANADIAN SYLLABICS WEST-CREE WA */
  ["\u1419", "\u1419"], /* CANADIAN SYLLABICS WAA */
  ["\u141A", "\u141A"], /* CANADIAN SYLLABICS WEST-CREE WAA */
  ["\u141B", "\u141B"], /* CANADIAN SYLLABICS NASKAPI WAA */
  ["\u141C", "\u141C"], /* CANADIAN SYLLABICS AI */
  ["\u141D", "\u141D"], /* CANADIAN SYLLABICS Y-CREE W */
  ["\u141E", "\u141E"], /* CANADIAN SYLLABICS GLOTTAL STOP */
  ["\u141F", "\u141F"], /* CANADIAN SYLLABICS FINAL ACUTE */
  ["\u1420", "\u1420"], /* CANADIAN SYLLABICS FINAL GRAVE */
  ["\u1421", "\u1421"], /* CANADIAN SYLLABICS FINAL BOTTOM HALF RING */
  ["\u1422", "\u1422"], /* CANADIAN SYLLABICS FINAL TOP HALF RING */
  ["\u1423", "\u1423"], /* CANADIAN SYLLABICS FINAL RIGHT HALF RING */
  ["\u1424", "\u1424"], /* CANADIAN SYLLABICS FINAL RING */
  ["\u1425", "\u1425"], /* CANADIAN SYLLABICS FINAL DOUBLE ACUTE */
  ["\u1426", "\u1426"], /* CANADIAN SYLLABICS FINAL DOUBLE SHORT VERTICAL STROKES */
  ["\u1427", "\u1427"], /* CANADIAN SYLLABICS FINAL MIDDLE DOT */
  ["\u1428", "\u1428"], /* CANADIAN SYLLABICS FINAL SHORT HORIZONTAL STROKE */
  ["\u1429", "\u1429"], /* CANADIAN SYLLABICS FINAL PLUS */
  ["\u142A", "\u142A"], /* CANADIAN SYLLABICS FINAL DOWN TACK */
  ["\u142B", "\u142B"], /* CANADIAN SYLLABICS EN */
  ["\u142C", "\u142C"], /* CANADIAN SYLLABICS IN */
  ["\u142D", "\u142D"], /* CANADIAN SYLLABICS ON */
  ["\u142E", "\u142E"], /* CANADIAN SYLLABICS AN */
  ["\u142F", "\u142F"], /* CANADIAN SYLLABICS PE */
  ["\u1430", "\u1430"], /* CANADIAN SYLLABICS PAAI */
  ["\u1431", "\u1431"], /* CANADIAN SYLLABICS PI */
  ["\u1432", "\u1432"], /* CANADIAN SYLLABICS PII */
  ["\u1433", "\u1433"], /* CANADIAN SYLLABICS PO */
  ["\u1434", "\u1434"], /* CANADIAN SYLLABICS POO */
  ["\u1435", "\u1435"], /* CANADIAN SYLLABICS Y-CREE POO */
  ["\u1436", "\u1436"], /* CANADIAN SYLLABICS CARRIER HEE */
  ["\u1437", "\u1437"], /* CANADIAN SYLLABICS CARRIER HI */
  ["\u1438", "\u1438"], /* CANADIAN SYLLABICS PA */
  ["\u1439", "\u1439"], /* CANADIAN SYLLABICS PAA */
  ["\u143A", "\u143A"], /* CANADIAN SYLLABICS PWE */
  ["\u143B", "\u143B"], /* CANADIAN SYLLABICS WEST-CREE PWE */
  ["\u143C", "\u143C"], /* CANADIAN SYLLABICS PWI */
  ["\u143D", "\u143D"], /* CANADIAN SYLLABICS WEST-CREE PWI */
  ["\u143E", "\u143E"], /* CANADIAN SYLLABICS PWII */
  ["\u143F", "\u143F"], /* CANADIAN SYLLABICS WEST-CREE PWII */
  ["\u1440", "\u1440"], /* CANADIAN SYLLABICS PWO */
  ["\u1441", "\u1441"], /* CANADIAN SYLLABICS WEST-CREE PWO */
  ["\u1442", "\u1442"], /* CANADIAN SYLLABICS PWOO */
  ["\u1443", "\u1443"], /* CANADIAN SYLLABICS WEST-CREE PWOO */
  ["\u1444", "\u1444"], /* CANADIAN SYLLABICS PWA */
  ["\u1445", "\u1445"], /* CANADIAN SYLLABICS WEST-CREE PWA */
  ["\u1446", "\u1446"], /* CANADIAN SYLLABICS PWAA */
  ["\u1447", "\u1447"], /* CANADIAN SYLLABICS WEST-CREE PWAA */
  ["\u1448", "\u1448"], /* CANADIAN SYLLABICS Y-CREE PWAA */
  ["\u1449", "\u1449"], /* CANADIAN SYLLABICS P */
  ["\u144A", "\u144A"], /* CANADIAN SYLLABICS WEST-CREE P */
  ["\u144B", "\u144B"], /* CANADIAN SYLLABICS CARRIER H */
  ["\u144C", "\u144C"], /* CANADIAN SYLLABICS TE */
  ["\u144D", "\u144D"], /* CANADIAN SYLLABICS TAAI */
  ["\u144E", "\u144E"], /* CANADIAN SYLLABICS TI */
  ["\u144F", "\u144F"], /* CANADIAN SYLLABICS TII */
  ["\u1450", "\u1450"], /* CANADIAN SYLLABICS TO */
  ["\u1451", "\u1451"], /* CANADIAN SYLLABICS TOO */
  ["\u1452", "\u1452"], /* CANADIAN SYLLABICS Y-CREE TOO */
  ["\u1453", "\u1453"], /* CANADIAN SYLLABICS CARRIER DEE */
  ["\u1454", "\u1454"], /* CANADIAN SYLLABICS CARRIER DI */
  ["\u1455", "\u1455"], /* CANADIAN SYLLABICS TA */
  ["\u1456", "\u1456"], /* CANADIAN SYLLABICS TAA */
  ["\u1457", "\u1457"], /* CANADIAN SYLLABICS TWE */
  ["\u1458", "\u1458"], /* CANADIAN SYLLABICS WEST-CREE TWE */
  ["\u1459", "\u1459"], /* CANADIAN SYLLABICS TWI */
  ["\u145A", "\u145A"], /* CANADIAN SYLLABICS WEST-CREE TWI */
  ["\u145B", "\u145B"], /* CANADIAN SYLLABICS TWII */
  ["\u145C", "\u145C"], /* CANADIAN SYLLABICS WEST-CREE TWII */
  ["\u145D", "\u145D"], /* CANADIAN SYLLABICS TWO */
  ["\u145E", "\u145E"], /* CANADIAN SYLLABICS WEST-CREE TWO */
  ["\u145F", "\u145F"], /* CANADIAN SYLLABICS TWOO */
  ["\u1460", "\u1460"], /* CANADIAN SYLLABICS WEST-CREE TWOO */
  ["\u1461", "\u1461"], /* CANADIAN SYLLABICS TWA */
  ["\u1462", "\u1462"], /* CANADIAN SYLLABICS WEST-CREE TWA */
  ["\u1463", "\u1463"], /* CANADIAN SYLLABICS TWAA */
  ["\u1464", "\u1464"], /* CANADIAN SYLLABICS WEST-CREE TWAA */
  ["\u1465", "\u1465"], /* CANADIAN SYLLABICS NASKAPI TWAA */
  ["\u1466", "\u1466"], /* CANADIAN SYLLABICS T */
  ["\u1467", "\u1467"], /* CANADIAN SYLLABICS TTE */
  ["\u1468", "\u1468"], /* CANADIAN SYLLABICS TTI */
  ["\u1469", "\u1469"], /* CANADIAN SYLLABICS TTO */
  ["\u146A", "\u146A"], /* CANADIAN SYLLABICS TTA */
  ["\u146B", "\u146B"], /* CANADIAN SYLLABICS KE */
  ["\u146C", "\u146C"], /* CANADIAN SYLLABICS KAAI */
  ["\u146D", "\u146D"], /* CANADIAN SYLLABICS KI */
  ["\u146E", "\u146E"], /* CANADIAN SYLLABICS KII */
  ["\u146F", "\u146F"], /* CANADIAN SYLLABICS KO */
  ["\u1470", "\u1470"], /* CANADIAN SYLLABICS KOO */
  ["\u1471", "\u1471"], /* CANADIAN SYLLABICS Y-CREE KOO */
  ["\u1472", "\u1472"], /* CANADIAN SYLLABICS KA */
  ["\u1473", "\u1473"], /* CANADIAN SYLLABICS KAA */
  ["\u1474", "\u1474"], /* CANADIAN SYLLABICS KWE */
  ["\u1475", "\u1475"], /* CANADIAN SYLLABICS WEST-CREE KWE */
  ["\u1476", "\u1476"], /* CANADIAN SYLLABICS KWI */
  ["\u1477", "\u1477"], /* CANADIAN SYLLABICS WEST-CREE KWI */
  ["\u1478", "\u1478"], /* CANADIAN SYLLABICS KWII */
  ["\u1479", "\u1479"], /* CANADIAN SYLLABICS WEST-CREE KWII */
  ["\u147A", "\u147A"], /* CANADIAN SYLLABICS KWO */
  ["\u147B", "\u147B"], /* CANADIAN SYLLABICS WEST-CREE KWO */
  ["\u147C", "\u147C"], /* CANADIAN SYLLABICS KWOO */
  ["\u147D", "\u147D"], /* CANADIAN SYLLABICS WEST-CREE KWOO */
  ["\u147E", "\u147E"], /* CANADIAN SYLLABICS KWA */
  ["\u147F", "\u147F"], /* CANADIAN SYLLABICS WEST-CREE KWA */
  ["\u1480", "\u1480"], /* CANADIAN SYLLABICS KWAA */
  ["\u1481", "\u1481"], /* CANADIAN SYLLABICS WEST-CREE KWAA */
  ["\u1482", "\u1482"], /* CANADIAN SYLLABICS NASKAPI KWAA */
  ["\u1483", "\u1483"], /* CANADIAN SYLLABICS K */
  ["\u1484", "\u1484"], /* CANADIAN SYLLABICS KW */
  ["\u1485", "\u1485"], /* CANADIAN SYLLABICS SOUTH-SLAVEY KEH */
  ["\u1486", "\u1486"], /* CANADIAN SYLLABICS SOUTH-SLAVEY KIH */
  ["\u1487", "\u1487"], /* CANADIAN SYLLABICS SOUTH-SLAVEY KOH */
  ["\u1488", "\u1488"], /* CANADIAN SYLLABICS SOUTH-SLAVEY KAH */
  ["\u1489", "\u1489"], /* CANADIAN SYLLABICS CE */
  ["\u148A", "\u148A"], /* CANADIAN SYLLABICS CAAI */
  ["\u148B", "\u148B"], /* CANADIAN SYLLABICS CI */
  ["\u148C", "\u148C"], /* CANADIAN SYLLABICS CII */
  ["\u148D", "\u148D"], /* CANADIAN SYLLABICS CO */
  ["\u148E", "\u148E"], /* CANADIAN SYLLABICS COO */
  ["\u148F", "\u148F"], /* CANADIAN SYLLABICS Y-CREE COO */
  ["\u1490", "\u1490"], /* CANADIAN SYLLABICS CA */
  ["\u1491", "\u1491"], /* CANADIAN SYLLABICS CAA */
  ["\u1492", "\u1492"], /* CANADIAN SYLLABICS CWE */
  ["\u1493", "\u1493"], /* CANADIAN SYLLABICS WEST-CREE CWE */
  ["\u1494", "\u1494"], /* CANADIAN SYLLABICS CWI */
  ["\u1495", "\u1495"], /* CANADIAN SYLLABICS WEST-CREE CWI */
  ["\u1496", "\u1496"], /* CANADIAN SYLLABICS CWII */
  ["\u1497", "\u1497"], /* CANADIAN SYLLABICS WEST-CREE CWII */
  ["\u1498", "\u1498"], /* CANADIAN SYLLABICS CWO */
  ["\u1499", "\u1499"], /* CANADIAN SYLLABICS WEST-CREE CWO */
  ["\u149A", "\u149A"], /* CANADIAN SYLLABICS CWOO */
  ["\u149B", "\u149B"], /* CANADIAN SYLLABICS WEST-CREE CWOO */
  ["\u149C", "\u149C"], /* CANADIAN SYLLABICS CWA */
  ["\u149D", "\u149D"], /* CANADIAN SYLLABICS WEST-CREE CWA */
  ["\u149E", "\u149E"], /* CANADIAN SYLLABICS CWAA */
  ["\u149F", "\u149F"], /* CANADIAN SYLLABICS WEST-CREE CWAA */
  ["\u14A0", "\u14A0"], /* CANADIAN SYLLABICS NASKAPI CWAA */
  ["\u14A1", "\u14A1"], /* CANADIAN SYLLABICS C */
  ["\u14A2", "\u14A2"], /* CANADIAN SYLLABICS SAYISI TH */
  ["\u14A3", "\u14A3"], /* CANADIAN SYLLABICS ME */
  ["\u14A4", "\u14A4"], /* CANADIAN SYLLABICS MAAI */
  ["\u14A5", "\u14A5"], /* CANADIAN SYLLABICS MI */
  ["\u14A6", "\u14A6"], /* CANADIAN SYLLABICS MII */
  ["\u14A7", "\u14A7"], /* CANADIAN SYLLABICS MO */
  ["\u14A8", "\u14A8"], /* CANADIAN SYLLABICS MOO */
  ["\u14A9", "\u14A9"], /* CANADIAN SYLLABICS Y-CREE MOO */
  ["\u14AA", "\u14AA"], /* CANADIAN SYLLABICS MA */
  ["\u14AB", "\u14AB"], /* CANADIAN SYLLABICS MAA */
  ["\u14AC", "\u14AC"], /* CANADIAN SYLLABICS MWE */
  ["\u14AD", "\u14AD"], /* CANADIAN SYLLABICS WEST-CREE MWE */
  ["\u14AE", "\u14AE"], /* CANADIAN SYLLABICS MWI */
  ["\u14AF", "\u14AF"], /* CANADIAN SYLLABICS WEST-CREE MWI */
  ["\u14B0", "\u14B0"], /* CANADIAN SYLLABICS MWII */
  ["\u14B1", "\u14B1"], /* CANADIAN SYLLABICS WEST-CREE MWII */
  ["\u14B2", "\u14B2"], /* CANADIAN SYLLABICS MWO */
  ["\u14B3", "\u14B3"], /* CANADIAN SYLLABICS WEST-CREE MWO */
  ["\u14B4", "\u14B4"], /* CANADIAN SYLLABICS MWOO */
  ["\u14B5", "\u14B5"], /* CANADIAN SYLLABICS WEST-CREE MWOO */
  ["\u14B6", "\u14B6"], /* CANADIAN SYLLABICS MWA */
  ["\u14B7", "\u14B7"], /* CANADIAN SYLLABICS WEST-CREE MWA */
  ["\u14B8", "\u14B8"], /* CANADIAN SYLLABICS MWAA */
  ["\u14B9", "\u14B9"], /* CANADIAN SYLLABICS WEST-CREE MWAA */
  ["\u14BA", "\u14BA"], /* CANADIAN SYLLABICS NASKAPI MWAA */
  ["\u14BB", "\u14BB"], /* CANADIAN SYLLABICS M */
  ["\u14BC", "\u14BC"], /* CANADIAN SYLLABICS WEST-CREE M */
  ["\u14BD", "\u14BD"], /* CANADIAN SYLLABICS MH */
  ["\u14BE", "\u14BE"], /* CANADIAN SYLLABICS ATHAPASCAN M */
  ["\u14BF", "\u14BF"], /* CANADIAN SYLLABICS SAYISI M */
  ["\u14C0", "\u14C0"], /* CANADIAN SYLLABICS NE */
  ["\u14C1", "\u14C1"], /* CANADIAN SYLLABICS NAAI */
  ["\u14C2", "\u14C2"], /* CANADIAN SYLLABICS NI */
  ["\u14C3", "\u14C3"], /* CANADIAN SYLLABICS NII */
  ["\u14C4", "\u14C4"], /* CANADIAN SYLLABICS NO */
  ["\u14C5", "\u14C5"], /* CANADIAN SYLLABICS NOO */
  ["\u14C6", "\u14C6"], /* CANADIAN SYLLABICS Y-CREE NOO */
  ["\u14C7", "\u14C7"], /* CANADIAN SYLLABICS NA */
  ["\u14C8", "\u14C8"], /* CANADIAN SYLLABICS NAA */
  ["\u14C9", "\u14C9"], /* CANADIAN SYLLABICS NWE */
  ["\u14CA", "\u14CA"], /* CANADIAN SYLLABICS WEST-CREE NWE */
  ["\u14CB", "\u14CB"], /* CANADIAN SYLLABICS NWA */
  ["\u14CC", "\u14CC"], /* CANADIAN SYLLABICS WEST-CREE NWA */
  ["\u14CD", "\u14CD"], /* CANADIAN SYLLABICS NWAA */
  ["\u14CE", "\u14CE"], /* CANADIAN SYLLABICS WEST-CREE NWAA */
  ["\u14CF", "\u14CF"], /* CANADIAN SYLLABICS NASKAPI NWAA */
  ["\u14D0", "\u14D0"], /* CANADIAN SYLLABICS N */
  ["\u14D1", "\u14D1"], /* CANADIAN SYLLABICS CARRIER NG */
  ["\u14D2", "\u14D2"], /* CANADIAN SYLLABICS NH */
  ["\u14D3", "\u14D3"], /* CANADIAN SYLLABICS LE */
  ["\u14D4", "\u14D4"], /* CANADIAN SYLLABICS LAAI */
  ["\u14D5", "\u14D5"], /* CANADIAN SYLLABICS LI */
  ["\u14D6", "\u14D6"], /* CANADIAN SYLLABICS LII */
  ["\u14D7", "\u14D7"], /* CANADIAN SYLLABICS LO */
  ["\u14D8", "\u14D8"], /* CANADIAN SYLLABICS LOO */
  ["\u14D9", "\u14D9"], /* CANADIAN SYLLABICS Y-CREE LOO */
  ["\u14DA", "\u14DA"], /* CANADIAN SYLLABICS LA */
  ["\u14DB", "\u14DB"], /* CANADIAN SYLLABICS LAA */
  ["\u14DC", "\u14DC"], /* CANADIAN SYLLABICS LWE */
  ["\u14DD", "\u14DD"], /* CANADIAN SYLLABICS WEST-CREE LWE */
  ["\u14DE", "\u14DE"], /* CANADIAN SYLLABICS LWI */
  ["\u14DF", "\u14DF"], /* CANADIAN SYLLABICS WEST-CREE LWI */
  ["\u14E0", "\u14E0"], /* CANADIAN SYLLABICS LWII */
  ["\u14E1", "\u14E1"], /* CANADIAN SYLLABICS WEST-CREE LWII */
  ["\u14E2", "\u14E2"], /* CANADIAN SYLLABICS LWO */
  ["\u14E3", "\u14E3"], /* CANADIAN SYLLABICS WEST-CREE LWO */
  ["\u14E4", "\u14E4"], /* CANADIAN SYLLABICS LWOO */
  ["\u14E5", "\u14E5"], /* CANADIAN SYLLABICS WEST-CREE LWOO */
  ["\u14E6", "\u14E6"], /* CANADIAN SYLLABICS LWA */
  ["\u14E7", "\u14E7"], /* CANADIAN SYLLABICS WEST-CREE LWA */
  ["\u14E8", "\u14E8"], /* CANADIAN SYLLABICS LWAA */
  ["\u14E9", "\u14E9"], /* CANADIAN SYLLABICS WEST-CREE LWAA */
  ["\u14EA", "\u14EA"], /* CANADIAN SYLLABICS L */
  ["\u14EB", "\u14EB"], /* CANADIAN SYLLABICS WEST-CREE L */
  ["\u14EC", "\u14EC"], /* CANADIAN SYLLABICS MEDIAL L */
  ["\u14ED", "\u14ED"], /* CANADIAN SYLLABICS SE */
  ["\u14EE", "\u14EE"], /* CANADIAN SYLLABICS SAAI */
  ["\u14EF", "\u14EF"], /* CANADIAN SYLLABICS SI */
  ["\u14F0", "\u14F0"], /* CANADIAN SYLLABICS SII */
  ["\u14F1", "\u14F1"], /* CANADIAN SYLLABICS SO */
  ["\u14F2", "\u14F2"], /* CANADIAN SYLLABICS SOO */
  ["\u14F3", "\u14F3"], /* CANADIAN SYLLABICS Y-CREE SOO */
  ["\u14F4", "\u14F4"], /* CANADIAN SYLLABICS SA */
  ["\u14F5", "\u14F5"], /* CANADIAN SYLLABICS SAA */
  ["\u14F6", "\u14F6"], /* CANADIAN SYLLABICS SWE */
  ["\u14F7", "\u14F7"], /* CANADIAN SYLLABICS WEST-CREE SWE */
  ["\u14F8", "\u14F8"], /* CANADIAN SYLLABICS SWI */
  ["\u14F9", "\u14F9"], /* CANADIAN SYLLABICS WEST-CREE SWI */
  ["\u14FA", "\u14FA"], /* CANADIAN SYLLABICS SWII */
  ["\u14FB", "\u14FB"], /* CANADIAN SYLLABICS WEST-CREE SWII */
  ["\u14FC", "\u14FC"], /* CANADIAN SYLLABICS SWO */
  ["\u14FD", "\u14FD"], /* CANADIAN SYLLABICS WEST-CREE SWO */
  ["\u14FE", "\u14FE"], /* CANADIAN SYLLABICS SWOO */
  ["\u14FF", "\u14FF"], /* CANADIAN SYLLABICS WEST-CREE SWOO */
  ["\u1500", "\u1500"], /* CANADIAN SYLLABICS SWA */
  ["\u1501", "\u1501"], /* CANADIAN SYLLABICS WEST-CREE SWA */
  ["\u1502", "\u1502"], /* CANADIAN SYLLABICS SWAA */
  ["\u1503", "\u1503"], /* CANADIAN SYLLABICS WEST-CREE SWAA */
  ["\u1504", "\u1504"], /* CANADIAN SYLLABICS NASKAPI SWAA */
  ["\u1505", "\u1505"], /* CANADIAN SYLLABICS S */
  ["\u1506", "\u1506"], /* CANADIAN SYLLABICS ATHAPASCAN S */
  ["\u1507", "\u1507"], /* CANADIAN SYLLABICS SW */
  ["\u1508", "\u1508"], /* CANADIAN SYLLABICS BLACKFOOT S */
  ["\u1509", "\u1509"], /* CANADIAN SYLLABICS MOOSE-CREE SK */
  ["\u150A", "\u150A"], /* CANADIAN SYLLABICS NASKAPI SKW */
  ["\u150B", "\u150B"], /* CANADIAN SYLLABICS NASKAPI S-W */
  ["\u150C", "\u150C"], /* CANADIAN SYLLABICS NASKAPI SPWA */
  ["\u150D", "\u150D"], /* CANADIAN SYLLABICS NASKAPI STWA */
  ["\u150E", "\u150E"], /* CANADIAN SYLLABICS NASKAPI SKWA */
  ["\u150F", "\u150F"], /* CANADIAN SYLLABICS NASKAPI SCWA */
  ["\u1510", "\u1510"], /* CANADIAN SYLLABICS SHE */
  ["\u1511", "\u1511"], /* CANADIAN SYLLABICS SHI */
  ["\u1512", "\u1512"], /* CANADIAN SYLLABICS SHII */
  ["\u1513", "\u1513"], /* CANADIAN SYLLABICS SHO */
  ["\u1514", "\u1514"], /* CANADIAN SYLLABICS SHOO */
  ["\u1515", "\u1515"], /* CANADIAN SYLLABICS SHA */
  ["\u1516", "\u1516"], /* CANADIAN SYLLABICS SHAA */
  ["\u1517", "\u1517"], /* CANADIAN SYLLABICS SHWE */
  ["\u1518", "\u1518"], /* CANADIAN SYLLABICS WEST-CREE SHWE */
  ["\u1519", "\u1519"], /* CANADIAN SYLLABICS SHWI */
  ["\u151A", "\u151A"], /* CANADIAN SYLLABICS WEST-CREE SHWI */
  ["\u151B", "\u151B"], /* CANADIAN SYLLABICS SHWII */
  ["\u151C", "\u151C"], /* CANADIAN SYLLABICS WEST-CREE SHWII */
  ["\u151D", "\u151D"], /* CANADIAN SYLLABICS SHWO */
  ["\u151E", "\u151E"], /* CANADIAN SYLLABICS WEST-CREE SHWO */
  ["\u151F", "\u151F"], /* CANADIAN SYLLABICS SHWOO */
  ["\u1520", "\u1520"], /* CANADIAN SYLLABICS WEST-CREE SHWOO */
  ["\u1521", "\u1521"], /* CANADIAN SYLLABICS SHWA */
  ["\u1522", "\u1522"], /* CANADIAN SYLLABICS WEST-CREE SHWA */
  ["\u1523", "\u1523"], /* CANADIAN SYLLABICS SHWAA */
  ["\u1524", "\u1524"], /* CANADIAN SYLLABICS WEST-CREE SHWAA */
  ["\u1525", "\u1525"], /* CANADIAN SYLLABICS SH */
  ["\u1526", "\u1526"], /* CANADIAN SYLLABICS YE */
  ["\u1527", "\u1527"], /* CANADIAN SYLLABICS YAAI */
  ["\u1528", "\u1528"], /* CANADIAN SYLLABICS YI */
  ["\u1529", "\u1529"], /* CANADIAN SYLLABICS YII */
  ["\u152A", "\u152A"], /* CANADIAN SYLLABICS YO */
  ["\u152B", "\u152B"], /* CANADIAN SYLLABICS YOO */
  ["\u152C", "\u152C"], /* CANADIAN SYLLABICS Y-CREE YOO */
  ["\u152D", "\u152D"], /* CANADIAN SYLLABICS YA */
  ["\u152E", "\u152E"], /* CANADIAN SYLLABICS YAA */
  ["\u152F", "\u152F"], /* CANADIAN SYLLABICS YWE */
  ["\u1530", "\u1530"], /* CANADIAN SYLLABICS WEST-CREE YWE */
  ["\u1531", "\u1531"], /* CANADIAN SYLLABICS YWI */
  ["\u1532", "\u1532"], /* CANADIAN SYLLABICS WEST-CREE YWI */
  ["\u1533", "\u1533"], /* CANADIAN SYLLABICS YWII */
  ["\u1534", "\u1534"], /* CANADIAN SYLLABICS WEST-CREE YWII */
  ["\u1535", "\u1535"], /* CANADIAN SYLLABICS YWO */
  ["\u1536", "\u1536"], /* CANADIAN SYLLABICS WEST-CREE YWO */
  ["\u1537", "\u1537"], /* CANADIAN SYLLABICS YWOO */
  ["\u1538", "\u1538"], /* CANADIAN SYLLABICS WEST-CREE YWOO */
  ["\u1539", "\u1539"], /* CANADIAN SYLLABICS YWA */
  ["\u153A", "\u153A"], /* CANADIAN SYLLABICS WEST-CREE YWA */
  ["\u153B", "\u153B"], /* CANADIAN SYLLABICS YWAA */
  ["\u153C", "\u153C"], /* CANADIAN SYLLABICS WEST-CREE YWAA */
  ["\u153D", "\u153D"], /* CANADIAN SYLLABICS NASKAPI YWAA */
  ["\u153E", "\u153E"], /* CANADIAN SYLLABICS Y */
  ["\u153F", "\u153F"], /* CANADIAN SYLLABICS BIBLE-CREE Y */
  ["\u1540", "\u1540"], /* CANADIAN SYLLABICS WEST-CREE Y */
  ["\u1541", "\u1541"], /* CANADIAN SYLLABICS SAYISI YI */
  ["\u1542", "\u1542"], /* CANADIAN SYLLABICS RE */
  ["\u1543", "\u1543"], /* CANADIAN SYLLABICS R-CREE RE */
  ["\u1544", "\u1544"], /* CANADIAN SYLLABICS WEST-CREE LE */
  ["\u1545", "\u1545"], /* CANADIAN SYLLABICS RAAI */
  ["\u1546", "\u1546"], /* CANADIAN SYLLABICS RI */
  ["\u1547", "\u1547"], /* CANADIAN SYLLABICS RII */
  ["\u1548", "\u1548"], /* CANADIAN SYLLABICS RO */
  ["\u1549", "\u1549"], /* CANADIAN SYLLABICS ROO */
  ["\u154A", "\u154A"], /* CANADIAN SYLLABICS WEST-CREE LO */
  ["\u154B", "\u154B"], /* CANADIAN SYLLABICS RA */
  ["\u154C", "\u154C"], /* CANADIAN SYLLABICS RAA */
  ["\u154D", "\u154D"], /* CANADIAN SYLLABICS WEST-CREE LA */
  ["\u154E", "\u154E"], /* CANADIAN SYLLABICS RWAA */
  ["\u154F", "\u154F"], /* CANADIAN SYLLABICS WEST-CREE RWAA */
  ["\u1550", "\u1550"], /* CANADIAN SYLLABICS R */
  ["\u1551", "\u1551"], /* CANADIAN SYLLABICS WEST-CREE R */
  ["\u1552", "\u1552"], /* CANADIAN SYLLABICS MEDIAL R */
  ["\u1553", "\u1553"], /* CANADIAN SYLLABICS FE */
  ["\u1554", "\u1554"], /* CANADIAN SYLLABICS FAAI */
  ["\u1555", "\u1555"], /* CANADIAN SYLLABICS FI */
  ["\u1556", "\u1556"], /* CANADIAN SYLLABICS FII */
  ["\u1557", "\u1557"], /* CANADIAN SYLLABICS FO */
  ["\u1558", "\u1558"], /* CANADIAN SYLLABICS FOO */
  ["\u1559", "\u1559"], /* CANADIAN SYLLABICS FA */
  ["\u155A", "\u155A"], /* CANADIAN SYLLABICS FAA */
  ["\u155B", "\u155B"], /* CANADIAN SYLLABICS FWAA */
  ["\u155C", "\u155C"], /* CANADIAN SYLLABICS WEST-CREE FWAA */
  ["\u155D", "\u155D"], /* CANADIAN SYLLABICS F */
  ["\u155E", "\u155E"], /* CANADIAN SYLLABICS THE */
  ["\u155F", "\u155F"], /* CANADIAN SYLLABICS N-CREE THE */
  ["\u1560", "\u1560"], /* CANADIAN SYLLABICS THI */
  ["\u1561", "\u1561"], /* CANADIAN SYLLABICS N-CREE THI */
  ["\u1562", "\u1562"], /* CANADIAN SYLLABICS THII */
  ["\u1563", "\u1563"], /* CANADIAN SYLLABICS N-CREE THII */
  ["\u1564", "\u1564"], /* CANADIAN SYLLABICS THO */
  ["\u1565", "\u1565"], /* CANADIAN SYLLABICS THOO */
  ["\u1566", "\u1566"], /* CANADIAN SYLLABICS THA */
  ["\u1567", "\u1567"], /* CANADIAN SYLLABICS THAA */
  ["\u1568", "\u1568"], /* CANADIAN SYLLABICS THWAA */
  ["\u1569", "\u1569"], /* CANADIAN SYLLABICS WEST-CREE THWAA */
  ["\u156A", "\u156A"], /* CANADIAN SYLLABICS TH */
  ["\u156B", "\u156B"], /* CANADIAN SYLLABICS TTHE */
  ["\u156C", "\u156C"], /* CANADIAN SYLLABICS TTHI */
  ["\u156D", "\u156D"], /* CANADIAN SYLLABICS TTHO */
  ["\u156E", "\u156E"], /* CANADIAN SYLLABICS TTHA */
  ["\u156F", "\u156F"], /* CANADIAN SYLLABICS TTH */
  ["\u1570", "\u1570"], /* CANADIAN SYLLABICS TYE */
  ["\u1571", "\u1571"], /* CANADIAN SYLLABICS TYI */
  ["\u1572", "\u1572"], /* CANADIAN SYLLABICS TYO */
  ["\u1573", "\u1573"], /* CANADIAN SYLLABICS TYA */
  ["\u1574", "\u1574"], /* CANADIAN SYLLABICS NUNAVIK HE */
  ["\u1575", "\u1575"], /* CANADIAN SYLLABICS NUNAVIK HI */
  ["\u1576", "\u1576"], /* CANADIAN SYLLABICS NUNAVIK HII */
  ["\u1577", "\u1577"], /* CANADIAN SYLLABICS NUNAVIK HO */
  ["\u1578", "\u1578"], /* CANADIAN SYLLABICS NUNAVIK HOO */
  ["\u1579", "\u1579"], /* CANADIAN SYLLABICS NUNAVIK HA */
  ["\u157A", "\u157A"], /* CANADIAN SYLLABICS NUNAVIK HAA */
  ["\u157B", "\u157B"], /* CANADIAN SYLLABICS NUNAVIK H */
  ["\u157C", "\u157C"], /* CANADIAN SYLLABICS NUNAVUT H */
  ["\u157D", "\u157D"], /* CANADIAN SYLLABICS HK */
  ["\u157E", "\u157E"], /* CANADIAN SYLLABICS QAAI */
  ["\u157F", "\u157F"], /* CANADIAN SYLLABICS QI */
  ["\u1580", "\u1580"], /* CANADIAN SYLLABICS QII */
  ["\u1581", "\u1581"], /* CANADIAN SYLLABICS QO */
  ["\u1582", "\u1582"], /* CANADIAN SYLLABICS QOO */
  ["\u1583", "\u1583"], /* CANADIAN SYLLABICS QA */
  ["\u1584", "\u1584"], /* CANADIAN SYLLABICS QAA */
  ["\u1585", "\u1585"], /* CANADIAN SYLLABICS Q */
  ["\u1586", "\u1586"], /* CANADIAN SYLLABICS TLHE */
  ["\u1587", "\u1587"], /* CANADIAN SYLLABICS TLHI */
  ["\u1588", "\u1588"], /* CANADIAN SYLLABICS TLHO */
  ["\u1589", "\u1589"], /* CANADIAN SYLLABICS TLHA */
  ["\u158A", "\u158A"], /* CANADIAN SYLLABICS WEST-CREE RE */
  ["\u158B", "\u158B"], /* CANADIAN SYLLABICS WEST-CREE RI */
  ["\u158C", "\u158C"], /* CANADIAN SYLLABICS WEST-CREE RO */
  ["\u158D", "\u158D"], /* CANADIAN SYLLABICS WEST-CREE RA */
  ["\u158E", "\u158E"], /* CANADIAN SYLLABICS NGAAI */
  ["\u158F", "\u158F"], /* CANADIAN SYLLABICS NGI */
  ["\u1590", "\u1590"], /* CANADIAN SYLLABICS NGII */
  ["\u1591", "\u1591"], /* CANADIAN SYLLABICS NGO */
  ["\u1592", "\u1592"], /* CANADIAN SYLLABICS NGOO */
  ["\u1593", "\u1593"], /* CANADIAN SYLLABICS NGA */
  ["\u1594", "\u1594"], /* CANADIAN SYLLABICS NGAA */
  ["\u1595", "\u1595"], /* CANADIAN SYLLABICS NG */
  ["\u1596", "\u1596"], /* CANADIAN SYLLABICS NNG */
  ["\u1597", "\u1597"], /* CANADIAN SYLLABICS SAYISI SHE */
  ["\u1598", "\u1598"], /* CANADIAN SYLLABICS SAYISI SHI */
  ["\u1599", "\u1599"], /* CANADIAN SYLLABICS SAYISI SHO */
  ["\u159A", "\u159A"], /* CANADIAN SYLLABICS SAYISI SHA */
  ["\u159B", "\u159B"], /* CANADIAN SYLLABICS WOODS-CREE THE */
  ["\u159C", "\u159C"], /* CANADIAN SYLLABICS WOODS-CREE THI */
  ["\u159D", "\u159D"], /* CANADIAN SYLLABICS WOODS-CREE THO */
  ["\u159E", "\u159E"], /* CANADIAN SYLLABICS WOODS-CREE THA */
  ["\u159F", "\u159F"], /* CANADIAN SYLLABICS WOODS-CREE TH */
  ["\u15A0", "\u15A0"], /* CANADIAN SYLLABICS LHI */
  ["\u15A1", "\u15A1"], /* CANADIAN SYLLABICS LHII */
  ["\u15A2", "\u15A2"], /* CANADIAN SYLLABICS LHO */
  ["\u15A3", "\u15A3"], /* CANADIAN SYLLABICS LHOO */
  ["\u15A4", "\u15A4"], /* CANADIAN SYLLABICS LHA */
  ["\u15A5", "\u15A5"], /* CANADIAN SYLLABICS LHAA */
  ["\u15A6", "\u15A6"], /* CANADIAN SYLLABICS LH */
  ["\u15A7", "\u15A7"], /* CANADIAN SYLLABICS TH-CREE THE */
  ["\u15A8", "\u15A8"], /* CANADIAN SYLLABICS TH-CREE THI */
  ["\u15A9", "\u15A9"], /* CANADIAN SYLLABICS TH-CREE THII */
  ["\u15AA", "\u15AA"], /* CANADIAN SYLLABICS TH-CREE THO */
  ["\u15AB", "\u15AB"], /* CANADIAN SYLLABICS TH-CREE THOO */
  ["\u15AC", "\u15AC"], /* CANADIAN SYLLABICS TH-CREE THA */
  ["\u15AD", "\u15AD"], /* CANADIAN SYLLABICS TH-CREE THAA */
  ["\u15AE", "\u15AE"], /* CANADIAN SYLLABICS TH-CREE TH */
  ["\u15AF", "\u15AF"], /* CANADIAN SYLLABICS AIVILIK B */
  ["\u15B0", "\u15B0"], /* CANADIAN SYLLABICS BLACKFOOT E */
  ["\u15B1", "\u15B1"], /* CANADIAN SYLLABICS BLACKFOOT I */
  ["\u15B2", "\u15B2"], /* CANADIAN SYLLABICS BLACKFOOT O */
  ["\u15B3", "\u15B3"], /* CANADIAN SYLLABICS BLACKFOOT A */
  ["\u15B4", "\u15B4"], /* CANADIAN SYLLABICS BLACKFOOT WE */
  ["\u15B5", "\u15B5"], /* CANADIAN SYLLABICS BLACKFOOT WI */
  ["\u15B6", "\u15B6"], /* CANADIAN SYLLABICS BLACKFOOT WO */
  ["\u15B7", "\u15B7"], /* CANADIAN SYLLABICS BLACKFOOT WA */
  ["\u15B8", "\u15B8"], /* CANADIAN SYLLABICS BLACKFOOT NE */
  ["\u15B9", "\u15B9"], /* CANADIAN SYLLABICS BLACKFOOT NI */
  ["\u15BA", "\u15BA"], /* CANADIAN SYLLABICS BLACKFOOT NO */
  ["\u15BB", "\u15BB"], /* CANADIAN SYLLABICS BLACKFOOT NA */
  ["\u15BC", "\u15BC"], /* CANADIAN SYLLABICS BLACKFOOT KE */
  ["\u15BD", "\u15BD"], /* CANADIAN SYLLABICS BLACKFOOT KI */
  ["\u15BE", "\u15BE"], /* CANADIAN SYLLABICS BLACKFOOT KO */
  ["\u15BF", "\u15BF"], /* CANADIAN SYLLABICS BLACKFOOT KA */
  ["\u15C0", "\u15C0"], /* CANADIAN SYLLABICS SAYISI HE */
  ["\u15C1", "\u15C1"], /* CANADIAN SYLLABICS SAYISI HI */
  ["\u15C2", "\u15C2"], /* CANADIAN SYLLABICS SAYISI HO */
  ["\u15C3", "\u15C3"], /* CANADIAN SYLLABICS SAYISI HA */
  ["\u15C4", "\u15C4"], /* CANADIAN SYLLABICS CARRIER GHU */
  ["\u15C5", "\u15C5"], /* CANADIAN SYLLABICS CARRIER GHO */
  ["\u15C6", "\u15C6"], /* CANADIAN SYLLABICS CARRIER GHE */
  ["\u15C7", "\u15C7"], /* CANADIAN SYLLABICS CARRIER GHEE */
  ["\u15C8", "\u15C8"], /* CANADIAN SYLLABICS CARRIER GHI */
  ["\u15C9", "\u15C9"], /* CANADIAN SYLLABICS CARRIER GHA */
  ["\u15CA", "\u15CA"], /* CANADIAN SYLLABICS CARRIER RU */
  ["\u15CB", "\u15CB"], /* CANADIAN SYLLABICS CARRIER RO */
  ["\u15CC", "\u15CC"], /* CANADIAN SYLLABICS CARRIER RE */
  ["\u15CD", "\u15CD"], /* CANADIAN SYLLABICS CARRIER REE */
  ["\u15CE", "\u15CE"], /* CANADIAN SYLLABICS CARRIER RI */
  ["\u15CF", "\u15CF"], /* CANADIAN SYLLABICS CARRIER RA */
  ["\u15D0", "\u15D0"], /* CANADIAN SYLLABICS CARRIER WU */
  ["\u15D1", "\u15D1"], /* CANADIAN SYLLABICS CARRIER WO */
  ["\u15D2", "\u15D2"], /* CANADIAN SYLLABICS CARRIER WE */
  ["\u15D3", "\u15D3"], /* CANADIAN SYLLABICS CARRIER WEE */
  ["\u15D4", "\u15D4"], /* CANADIAN SYLLABICS CARRIER WI */
  ["\u15D5", "\u15D5"], /* CANADIAN SYLLABICS CARRIER WA */
  ["\u15D6", "\u15D6"], /* CANADIAN SYLLABICS CARRIER HWU */
  ["\u15D7", "\u15D7"], /* CANADIAN SYLLABICS CARRIER HWO */
  ["\u15D8", "\u15D8"], /* CANADIAN SYLLABICS CARRIER HWE */
  ["\u15D9", "\u15D9"], /* CANADIAN SYLLABICS CARRIER HWEE */
  ["\u15DA", "\u15DA"], /* CANADIAN SYLLABICS CARRIER HWI */
  ["\u15DB", "\u15DB"], /* CANADIAN SYLLABICS CARRIER HWA */
  ["\u15DC", "\u15DC"], /* CANADIAN SYLLABICS CARRIER THU */
  ["\u15DD", "\u15DD"], /* CANADIAN SYLLABICS CARRIER THO */
  ["\u15DE", "\u15DE"], /* CANADIAN SYLLABICS CARRIER THE */
  ["\u15DF", "\u15DF"], /* CANADIAN SYLLABICS CARRIER THEE */
  ["\u15E0", "\u15E0"], /* CANADIAN SYLLABICS CARRIER THI */
  ["\u15E1", "\u15E1"], /* CANADIAN SYLLABICS CARRIER THA */
  ["\u15E2", "\u15E2"], /* CANADIAN SYLLABICS CARRIER TTU */
  ["\u15E3", "\u15E3"], /* CANADIAN SYLLABICS CARRIER TTO */
  ["\u15E4", "\u15E4"], /* CANADIAN SYLLABICS CARRIER TTE */
  ["\u15E5", "\u15E5"], /* CANADIAN SYLLABICS CARRIER TTEE */
  ["\u15E6", "\u15E6"], /* CANADIAN SYLLABICS CARRIER TTI */
  ["\u15E7", "\u15E7"], /* CANADIAN SYLLABICS CARRIER TTA */
  ["\u15E8", "\u15E8"], /* CANADIAN SYLLABICS CARRIER PU */
  ["\u15E9", "\u15E9"], /* CANADIAN SYLLABICS CARRIER PO */
  ["\u15EA", "\u15EA"], /* CANADIAN SYLLABICS CARRIER PE */
  ["\u15EB", "\u15EB"], /* CANADIAN SYLLABICS CARRIER PEE */
  ["\u15EC", "\u15EC"], /* CANADIAN SYLLABICS CARRIER PI */
  ["\u15ED", "\u15ED"], /* CANADIAN SYLLABICS CARRIER PA */
  ["\u15EE", "\u15EE"], /* CANADIAN SYLLABICS CARRIER P */
  ["\u15EF", "\u15EF"], /* CANADIAN SYLLABICS CARRIER GU */
  ["\u15F0", "\u15F0"], /* CANADIAN SYLLABICS CARRIER GO */
  ["\u15F1", "\u15F1"], /* CANADIAN SYLLABICS CARRIER GE */
  ["\u15F2", "\u15F2"], /* CANADIAN SYLLABICS CARRIER GEE */
  ["\u15F3", "\u15F3"], /* CANADIAN SYLLABICS CARRIER GI */
  ["\u15F4", "\u15F4"], /* CANADIAN SYLLABICS CARRIER GA */
  ["\u15F5", "\u15F5"], /* CANADIAN SYLLABICS CARRIER KHU */
  ["\u15F6", "\u15F6"], /* CANADIAN SYLLABICS CARRIER KHO */
  ["\u15F7", "\u15F7"], /* CANADIAN SYLLABICS CARRIER KHE */
  ["\u15F8", "\u15F8"], /* CANADIAN SYLLABICS CARRIER KHEE */
  ["\u15F9", "\u15F9"], /* CANADIAN SYLLABICS CARRIER KHI */
  ["\u15FA", "\u15FA"], /* CANADIAN SYLLABICS CARRIER KHA */
  ["\u15FB", "\u15FB"], /* CANADIAN SYLLABICS CARRIER KKU */
  ["\u15FC", "\u15FC"], /* CANADIAN SYLLABICS CARRIER KKO */
  ["\u15FD", "\u15FD"], /* CANADIAN SYLLABICS CARRIER KKE */
  ["\u15FE", "\u15FE"], /* CANADIAN SYLLABICS CARRIER KKEE */
  ["\u15FF", "\u15FF"], /* CANADIAN SYLLABICS CARRIER KKI */
  ["\u1600", "\u1600"], /* CANADIAN SYLLABICS CARRIER KKA */
  ["\u1601", "\u1601"], /* CANADIAN SYLLABICS CARRIER KK */
  ["\u1602", "\u1602"], /* CANADIAN SYLLABICS CARRIER NU */
  ["\u1603", "\u1603"], /* CANADIAN SYLLABICS CARRIER NO */
  ["\u1604", "\u1604"], /* CANADIAN SYLLABICS CARRIER NE */
  ["\u1605", "\u1605"], /* CANADIAN SYLLABICS CARRIER NEE */
  ["\u1606", "\u1606"], /* CANADIAN SYLLABICS CARRIER NI */
  ["\u1607", "\u1607"], /* CANADIAN SYLLABICS CARRIER NA */
  ["\u1608", "\u1608"], /* CANADIAN SYLLABICS CARRIER MU */
  ["\u1609", "\u1609"], /* CANADIAN SYLLABICS CARRIER MO */
  ["\u160A", "\u160A"], /* CANADIAN SYLLABICS CARRIER ME */
  ["\u160B", "\u160B"], /* CANADIAN SYLLABICS CARRIER MEE */
  ["\u160C", "\u160C"], /* CANADIAN SYLLABICS CARRIER MI */
  ["\u160D", "\u160D"], /* CANADIAN SYLLABICS CARRIER MA */
  ["\u160E", "\u160E"], /* CANADIAN SYLLABICS CARRIER YU */
  ["\u160F", "\u160F"], /* CANADIAN SYLLABICS CARRIER YO */
  ["\u1610", "\u1610"], /* CANADIAN SYLLABICS CARRIER YE */
  ["\u1611", "\u1611"], /* CANADIAN SYLLABICS CARRIER YEE */
  ["\u1612", "\u1612"], /* CANADIAN SYLLABICS CARRIER YI */
  ["\u1613", "\u1613"], /* CANADIAN SYLLABICS CARRIER YA */
  ["\u1614", "\u1614"], /* CANADIAN SYLLABICS CARRIER JU */
  ["\u1615", "\u1615"], /* CANADIAN SYLLABICS SAYISI JU */
  ["\u1616", "\u1616"], /* CANADIAN SYLLABICS CARRIER JO */
  ["\u1617", "\u1617"], /* CANADIAN SYLLABICS CARRIER JE */
  ["\u1618", "\u1618"], /* CANADIAN SYLLABICS CARRIER JEE */
  ["\u1619", "\u1619"], /* CANADIAN SYLLABICS CARRIER JI */
  ["\u161A", "\u161A"], /* CANADIAN SYLLABICS SAYISI JI */
  ["\u161B", "\u161B"], /* CANADIAN SYLLABICS CARRIER JA */
  ["\u161C", "\u161C"], /* CANADIAN SYLLABICS CARRIER JJU */
  ["\u161D", "\u161D"], /* CANADIAN SYLLABICS CARRIER JJO */
  ["\u161E", "\u161E"], /* CANADIAN SYLLABICS CARRIER JJE */
  ["\u161F", "\u161F"], /* CANADIAN SYLLABICS CARRIER JJEE */
  ["\u1620", "\u1620"], /* CANADIAN SYLLABICS CARRIER JJI */
  ["\u1621", "\u1621"], /* CANADIAN SYLLABICS CARRIER JJA */
  ["\u1622", "\u1622"], /* CANADIAN SYLLABICS CARRIER LU */
  ["\u1623", "\u1623"], /* CANADIAN SYLLABICS CARRIER LO */
  ["\u1624", "\u1624"], /* CANADIAN SYLLABICS CARRIER LE */
  ["\u1625", "\u1625"], /* CANADIAN SYLLABICS CARRIER LEE */
  ["\u1626", "\u1626"], /* CANADIAN SYLLABICS CARRIER LI */
  ["\u1627", "\u1627"], /* CANADIAN SYLLABICS CARRIER LA */
  ["\u1628", "\u1628"], /* CANADIAN SYLLABICS CARRIER DLU */
  ["\u1629", "\u1629"], /* CANADIAN SYLLABICS CARRIER DLO */
  ["\u162A", "\u162A"], /* CANADIAN SYLLABICS CARRIER DLE */
  ["\u162B", "\u162B"], /* CANADIAN SYLLABICS CARRIER DLEE */
  ["\u162C", "\u162C"], /* CANADIAN SYLLABICS CARRIER DLI */
  ["\u162D", "\u162D"], /* CANADIAN SYLLABICS CARRIER DLA */
  ["\u162E", "\u162E"], /* CANADIAN SYLLABICS CARRIER LHU */
  ["\u162F", "\u162F"], /* CANADIAN SYLLABICS CARRIER LHO */
  ["\u1630", "\u1630"], /* CANADIAN SYLLABICS CARRIER LHE */
  ["\u1631", "\u1631"], /* CANADIAN SYLLABICS CARRIER LHEE */
  ["\u1632", "\u1632"], /* CANADIAN SYLLABICS CARRIER LHI */
  ["\u1633", "\u1633"], /* CANADIAN SYLLABICS CARRIER LHA */
  ["\u1634", "\u1634"], /* CANADIAN SYLLABICS CARRIER TLHU */
  ["\u1635", "\u1635"], /* CANADIAN SYLLABICS CARRIER TLHO */
  ["\u1636", "\u1636"], /* CANADIAN SYLLABICS CARRIER TLHE */
  ["\u1637", "\u1637"], /* CANADIAN SYLLABICS CARRIER TLHEE */
  ["\u1638", "\u1638"], /* CANADIAN SYLLABICS CARRIER TLHI */
  ["\u1639", "\u1639"], /* CANADIAN SYLLABICS CARRIER TLHA */
  ["\u163A", "\u163A"], /* CANADIAN SYLLABICS CARRIER TLU */
  ["\u163B", "\u163B"], /* CANADIAN SYLLABICS CARRIER TLO */
  ["\u163C", "\u163C"], /* CANADIAN SYLLABICS CARRIER TLE */
  ["\u163D", "\u163D"], /* CANADIAN SYLLABICS CARRIER TLEE */
  ["\u163E", "\u163E"], /* CANADIAN SYLLABICS CARRIER TLI */
  ["\u163F", "\u163F"], /* CANADIAN SYLLABICS CARRIER TLA */
  ["\u1640", "\u1640"], /* CANADIAN SYLLABICS CARRIER ZU */
  ["\u1641", "\u1641"], /* CANADIAN SYLLABICS CARRIER ZO */
  ["\u1642", "\u1642"], /* CANADIAN SYLLABICS CARRIER ZE */
  ["\u1643", "\u1643"], /* CANADIAN SYLLABICS CARRIER ZEE */
  ["\u1644", "\u1644"], /* CANADIAN SYLLABICS CARRIER ZI */
  ["\u1645", "\u1645"], /* CANADIAN SYLLABICS CARRIER ZA */
  ["\u1646", "\u1646"], /* CANADIAN SYLLABICS CARRIER Z */
  ["\u1647", "\u1647"], /* CANADIAN SYLLABICS CARRIER INITIAL Z */
  ["\u1648", "\u1648"], /* CANADIAN SYLLABICS CARRIER DZU */
  ["\u1649", "\u1649"], /* CANADIAN SYLLABICS CARRIER DZO */
  ["\u164A", "\u164A"], /* CANADIAN SYLLABICS CARRIER DZE */
  ["\u164B", "\u164B"], /* CANADIAN SYLLABICS CARRIER DZEE */
  ["\u164C", "\u164C"], /* CANADIAN SYLLABICS CARRIER DZI */
  ["\u164D", "\u164D"], /* CANADIAN SYLLABICS CARRIER DZA */
  ["\u164E", "\u164E"], /* CANADIAN SYLLABICS CARRIER SU */
  ["\u164F", "\u164F"], /* CANADIAN SYLLABICS CARRIER SO */
  ["\u1650", "\u1650"], /* CANADIAN SYLLABICS CARRIER SE */
  ["\u1651", "\u1651"], /* CANADIAN SYLLABICS CARRIER SEE */
  ["\u1652", "\u1652"], /* CANADIAN SYLLABICS CARRIER SI */
  ["\u1653", "\u1653"], /* CANADIAN SYLLABICS CARRIER SA */
  ["\u1654", "\u1654"], /* CANADIAN SYLLABICS CARRIER SHU */
  ["\u1655", "\u1655"], /* CANADIAN SYLLABICS CARRIER SHO */
  ["\u1656", "\u1656"], /* CANADIAN SYLLABICS CARRIER SHE */
  ["\u1657", "\u1657"], /* CANADIAN SYLLABICS CARRIER SHEE */
  ["\u1658", "\u1658"], /* CANADIAN SYLLABICS CARRIER SHI */
  ["\u1659", "\u1659"], /* CANADIAN SYLLABICS CARRIER SHA */
  ["\u165A", "\u165A"], /* CANADIAN SYLLABICS CARRIER SH */
  ["\u165B", "\u165B"], /* CANADIAN SYLLABICS CARRIER TSU */
  ["\u165C", "\u165C"], /* CANADIAN SYLLABICS CARRIER TSO */
  ["\u165D", "\u165D"], /* CANADIAN SYLLABICS CARRIER TSE */
  ["\u165E", "\u165E"], /* CANADIAN SYLLABICS CARRIER TSEE */
  ["\u165F", "\u165F"], /* CANADIAN SYLLABICS CARRIER TSI */
  ["\u1660", "\u1660"], /* CANADIAN SYLLABICS CARRIER TSA */
  ["\u1661", "\u1661"], /* CANADIAN SYLLABICS CARRIER CHU */
  ["\u1662", "\u1662"], /* CANADIAN SYLLABICS CARRIER CHO */
  ["\u1663", "\u1663"], /* CANADIAN SYLLABICS CARRIER CHE */
  ["\u1664", "\u1664"], /* CANADIAN SYLLABICS CARRIER CHEE */
  ["\u1665", "\u1665"], /* CANADIAN SYLLABICS CARRIER CHI */
  ["\u1666", "\u1666"], /* CANADIAN SYLLABICS CARRIER CHA */
  ["\u1667", "\u1667"], /* CANADIAN SYLLABICS CARRIER TTSU */
  ["\u1668", "\u1668"], /* CANADIAN SYLLABICS CARRIER TTSO */
  ["\u1669", "\u1669"], /* CANADIAN SYLLABICS CARRIER TTSE */
  ["\u166A", "\u166A"], /* CANADIAN SYLLABICS CARRIER TTSEE */
  ["\u166B", "\u166B"], /* CANADIAN SYLLABICS CARRIER TTSI */
  ["\u166C", "\u166C"], /* CANADIAN SYLLABICS CARRIER TTSA */
  ["\u166D", "\u166D"], /* CANADIAN SYLLABICS CHI SIGN */
  ["\u166E", "\u166E"], /* CANADIAN SYLLABICS FULL STOP */
  ["\u166F", "\u166F"], /* CANADIAN SYLLABICS QAI */
  ["\u1670", "\u1670"], /* CANADIAN SYLLABICS NGAI */
  ["\u1671", "\u1671"], /* CANADIAN SYLLABICS NNGI */
  ["\u1672", "\u1672"], /* CANADIAN SYLLABICS NNGII */
  ["\u1673", "\u1673"], /* CANADIAN SYLLABICS NNGO */
  ["\u1674", "\u1674"], /* CANADIAN SYLLABICS NNGOO */
  ["\u1675", "\u1675"], /* CANADIAN SYLLABICS NNGA */
  ["\u1676", "\u1676"], /* CANADIAN SYLLABICS NNGAA */
  ["\u1677", "\u1677"], /* CANADIAN SYLLABICS WOODS-CREE THWEE */
  ["\u1678", "\u1678"], /* CANADIAN SYLLABICS WOODS-CREE THWI */
  ["\u1679", "\u1679"], /* CANADIAN SYLLABICS WOODS-CREE THWII */
  ["\u167A", "\u167A"], /* CANADIAN SYLLABICS WOODS-CREE THWO */
  ["\u167B", "\u167B"], /* CANADIAN SYLLABICS WOODS-CREE THWOO */
  ["\u167C", "\u167C"], /* CANADIAN SYLLABICS WOODS-CREE THWA */
  ["\u167D", "\u167D"], /* CANADIAN SYLLABICS WOODS-CREE THWAA */
  ["\u167E", "\u167E"], /* CANADIAN SYLLABICS WOODS-CREE FINAL TH */
  ["\u167F", "\u167F"], /* CANADIAN SYLLABICS BLACKFOOT W */
  ["\u1680", "\u1680"], /* OGHAM SPACE MARK */
  ["\u1681", "\u1681"], /* OGHAM LETTER BEITH */
  ["\u1682", "\u1682"], /* OGHAM LETTER LUIS */
  ["\u1683", "\u1683"], /* OGHAM LETTER FEARN */
  ["\u1684", "\u1684"], /* OGHAM LETTER SAIL */
  ["\u1685", "\u1685"], /* OGHAM LETTER NION */
  ["\u1686", "\u1686"], /* OGHAM LETTER UATH */
  ["\u1687", "\u1687"], /* OGHAM LETTER DAIR */
  ["\u1688", "\u1688"], /* OGHAM LETTER TINNE */
  ["\u1689", "\u1689"], /* OGHAM LETTER COLL */
  ["\u168A", "\u168A"], /* OGHAM LETTER CEIRT */
  ["\u168B", "\u168B"], /* OGHAM LETTER MUIN */
  ["\u168C", "\u168C"], /* OGHAM LETTER GORT */
  ["\u168D", "\u168D"], /* OGHAM LETTER NGEADAL */
  ["\u168E", "\u168E"], /* OGHAM LETTER STRAIF */
  ["\u168F", "\u168F"], /* OGHAM LETTER RUIS */
  ["\u1690", "\u1690"], /* OGHAM LETTER AILM */
  ["\u1691", "\u1691"], /* OGHAM LETTER ONN */
  ["\u1692", "\u1692"], /* OGHAM LETTER UR */
  ["\u1693", "\u1693"], /* OGHAM LETTER EADHADH */
  ["\u1694", "\u1694"], /* OGHAM LETTER IODHADH */
  ["\u1695", "\u1695"], /* OGHAM LETTER EABHADH */
  ["\u1696", "\u1696"], /* OGHAM LETTER OR */
  ["\u1697", "\u1697"], /* OGHAM LETTER UILLEANN */
  ["\u1698", "\u1698"], /* OGHAM LETTER IFIN */
  ["\u1699", "\u1699"], /* OGHAM LETTER EAMHANCHOLL */
  ["\u169A", "\u169A"], /* OGHAM LETTER PEITH */
  ["\u169B", "\u169B"], /* OGHAM FEATHER MARK */
  ["\u169C", "\u169C"], /* OGHAM REVERSED FEATHER MARK */
  ["\u169D", "\u169D"],
  ["\u169E", "\u169E"],
  ["\u169F", "\u169F"],
  ["\u16A0", "\u16A0"], /* RUNIC LETTER FEHU FEOH FE F */
  ["\u16A1", "\u16A1"], /* RUNIC LETTER V */
  ["\u16A2", "\u16A2"], /* RUNIC LETTER URUZ UR U */
  ["\u16A3", "\u16A3"], /* RUNIC LETTER YR */
  ["\u16A4", "\u16A4"], /* RUNIC LETTER Y */
  ["\u16A5", "\u16A5"], /* RUNIC LETTER W */
  ["\u16A6", "\u16A6"], /* RUNIC LETTER THURISAZ THURS THORN */
  ["\u16A7", "\u16A7"], /* RUNIC LETTER ETH */
  ["\u16A8", "\u16A8"], /* RUNIC LETTER ANSUZ A */
  ["\u16A9", "\u16A9"], /* RUNIC LETTER OS O */
  ["\u16AA", "\u16AA"], /* RUNIC LETTER AC A */
  ["\u16AB", "\u16AB"], /* RUNIC LETTER AESC */
  ["\u16AC", "\u16AC"], /* RUNIC LETTER LONG-BRANCH-OSS O */
  ["\u16AD", "\u16AD"], /* RUNIC LETTER SHORT-TWIG-OSS O */
  ["\u16AE", "\u16AE"], /* RUNIC LETTER O */
  ["\u16AF", "\u16AF"], /* RUNIC LETTER OE */
  ["\u16B0", "\u16B0"], /* RUNIC LETTER ON */
  ["\u16B1", "\u16B1"], /* RUNIC LETTER RAIDO RAD REID R */
  ["\u16B2", "\u16B2"], /* RUNIC LETTER KAUNA */
  ["\u16B3", "\u16B3"], /* RUNIC LETTER CEN */
  ["\u16B4", "\u16B4"], /* RUNIC LETTER KAUN K */
  ["\u16B5", "\u16B5"], /* RUNIC LETTER G */
  ["\u16B6", "\u16B6"], /* RUNIC LETTER ENG */
  ["\u16B7", "\u16B7"], /* RUNIC LETTER GEBO GYFU G */
  ["\u16B8", "\u16B8"], /* RUNIC LETTER GAR */
  ["\u16B9", "\u16B9"], /* RUNIC LETTER WUNJO WYNN W */
  ["\u16BA", "\u16BA"], /* RUNIC LETTER HAGLAZ H */
  ["\u16BB", "\u16BB"], /* RUNIC LETTER HAEGL H */
  ["\u16BC", "\u16BC"], /* RUNIC LETTER LONG-BRANCH-HAGALL H */
  ["\u16BD", "\u16BD"], /* RUNIC LETTER SHORT-TWIG-HAGALL H */
  ["\u16BE", "\u16BE"], /* RUNIC LETTER NAUDIZ NYD NAUD N */
  ["\u16BF", "\u16BF"], /* RUNIC LETTER SHORT-TWIG-NAUD N */
  ["\u16C0", "\u16C0"], /* RUNIC LETTER DOTTED-N */
  ["\u16C1", "\u16C1"], /* RUNIC LETTER ISAZ IS ISS I */
  ["\u16C2", "\u16C2"], /* RUNIC LETTER E */
  ["\u16C3", "\u16C3"], /* RUNIC LETTER JERAN J */
  ["\u16C4", "\u16C4"], /* RUNIC LETTER GER */
  ["\u16C5", "\u16C5"], /* RUNIC LETTER LONG-BRANCH-AR AE */
  ["\u16C6", "\u16C6"], /* RUNIC LETTER SHORT-TWIG-AR A */
  ["\u16C7", "\u16C7"], /* RUNIC LETTER IWAZ EOH */
  ["\u16C8", "\u16C8"], /* RUNIC LETTER PERTHO PEORTH P */
  ["\u16C9", "\u16C9"], /* RUNIC LETTER ALGIZ EOLHX */
  ["\u16CA", "\u16CA"], /* RUNIC LETTER SOWILO S */
  ["\u16CB", "\u16CB"], /* RUNIC LETTER SIGEL LONG-BRANCH-SOL S */
  ["\u16CC", "\u16CC"], /* RUNIC LETTER SHORT-TWIG-SOL S */
  ["\u16CD", "\u16CD"], /* RUNIC LETTER C */
  ["\u16CE", "\u16CE"], /* RUNIC LETTER Z */
  ["\u16CF", "\u16CF"], /* RUNIC LETTER TIWAZ TIR TYR T */
  ["\u16D0", "\u16D0"], /* RUNIC LETTER SHORT-TWIG-TYR T */
  ["\u16D1", "\u16D1"], /* RUNIC LETTER D */
  ["\u16D2", "\u16D2"], /* RUNIC LETTER BERKANAN BEORC BJARKAN B */
  ["\u16D3", "\u16D3"], /* RUNIC LETTER SHORT-TWIG-BJARKAN B */
  ["\u16D4", "\u16D4"], /* RUNIC LETTER DOTTED-P */
  ["\u16D5", "\u16D5"], /* RUNIC LETTER OPEN-P */
  ["\u16D6", "\u16D6"], /* RUNIC LETTER EHWAZ EH E */
  ["\u16D7", "\u16D7"], /* RUNIC LETTER MANNAZ MAN M */
  ["\u16D8", "\u16D8"], /* RUNIC LETTER LONG-BRANCH-MADR M */
  ["\u16D9", "\u16D9"], /* RUNIC LETTER SHORT-TWIG-MADR M */
  ["\u16DA", "\u16DA"], /* RUNIC LETTER LAUKAZ LAGU LOGR L */
  ["\u16DB", "\u16DB"], /* RUNIC LETTER DOTTED-L */
  ["\u16DC", "\u16DC"], /* RUNIC LETTER INGWAZ */
  ["\u16DD", "\u16DD"], /* RUNIC LETTER ING */
  ["\u16DE", "\u16DE"], /* RUNIC LETTER DAGAZ DAEG D */
  ["\u16DF", "\u16DF"], /* RUNIC LETTER OTHALAN ETHEL O */
  ["\u16E0", "\u16E0"], /* RUNIC LETTER EAR */
  ["\u16E1", "\u16E1"], /* RUNIC LETTER IOR */
  ["\u16E2", "\u16E2"], /* RUNIC LETTER CWEORTH */
  ["\u16E3", "\u16E3"], /* RUNIC LETTER CALC */
  ["\u16E4", "\u16E4"], /* RUNIC LETTER CEALC */
  ["\u16E5", "\u16E5"], /* RUNIC LETTER STAN */
  ["\u16E6", "\u16E6"], /* RUNIC LETTER LONG-BRANCH-YR */
  ["\u16E7", "\u16E7"], /* RUNIC LETTER SHORT-TWIG-YR */
  ["\u16E8", "\u16E8"], /* RUNIC LETTER ICELANDIC-YR */
  ["\u16E9", "\u16E9"], /* RUNIC LETTER Q */
  ["\u16EA", "\u16EA"], /* RUNIC LETTER X */
  ["\u16EB", "\u16EB"], /* RUNIC SINGLE PUNCTUATION */
  ["\u16EC", "\u16EC"], /* RUNIC MULTIPLE PUNCTUATION */
  ["\u16ED", "\u16ED"], /* RUNIC CROSS PUNCTUATION */
  ["\u16EE", "\u16EE"], /* RUNIC ARLAUG SYMBOL */
  ["\u16EF", "\u16EF"], /* RUNIC TVIMADUR SYMBOL */
  ["\u16F0", "\u16F0"], /* RUNIC BELGTHOR SYMBOL */
  ["\u16F1", "\u16F1"], /* RUNIC LETTER K */
  ["\u16F2", "\u16F2"], /* RUNIC LETTER SH */
  ["\u16F3", "\u16F3"], /* RUNIC LETTER OO */
  ["\u16F4", "\u16F4"], /* RUNIC LETTER FRANKS CASKET OS */
  ["\u16F5", "\u16F5"], /* RUNIC LETTER FRANKS CASKET IS */
  ["\u16F6", "\u16F6"], /* RUNIC LETTER FRANKS CASKET EH */
  ["\u16F7", "\u16F7"], /* RUNIC LETTER FRANKS CASKET AC */
  ["\u16F8", "\u16F8"], /* RUNIC LETTER FRANKS CASKET AESC */
  ["\u16F9", "\u16F9"],
  ["\u16FA", "\u16FA"],
  ["\u16FB", "\u16FB"],
  ["\u16FC", "\u16FC"],
  ["\u16FD", "\u16FD"],
  ["\u16FE", "\u16FE"],
  ["\u16FF", "\u16FF"],
  ["\u1700", "\u1700"], /* TAGALOG LETTER A */
  ["\u1701", "\u1701"], /* TAGALOG LETTER I */
  ["\u1702", "\u1702"], /* TAGALOG LETTER U */
  ["\u1703", "\u1703"], /* TAGALOG LETTER KA */
  ["\u1704", "\u1704"], /* TAGALOG LETTER GA */
  ["\u1705", "\u1705"], /* TAGALOG LETTER NGA */
  ["\u1706", "\u1706"], /* TAGALOG LETTER TA */
  ["\u1707", "\u1707"], /* TAGALOG LETTER DA */
  ["\u1708", "\u1708"], /* TAGALOG LETTER NA */
  ["\u1709", "\u1709"], /* TAGALOG LETTER PA */
  ["\u170A", "\u170A"], /* TAGALOG LETTER BA */
  ["\u170B", "\u170B"], /* TAGALOG LETTER MA */
  ["\u170C", "\u170C"], /* TAGALOG LETTER YA */
  ["\u170D", "\u170D"],
  ["\u170E", "\u170E"], /* TAGALOG LETTER LA */
  ["\u170F", "\u170F"], /* TAGALOG LETTER WA */
  ["\u1710", "\u1710"], /* TAGALOG LETTER SA */
  ["\u1711", "\u1711"], /* TAGALOG LETTER HA */
  ["\u1712", "\u1712"], /* TAGALOG VOWEL SIGN I */
  ["\u1713", "\u1713"], /* TAGALOG VOWEL SIGN U */
  ["\u1714", "\u1714"], /* TAGALOG SIGN VIRAMA */
  ["\u1715", "\u1715"],
  ["\u1716", "\u1716"],
  ["\u1717", "\u1717"],
  ["\u1718", "\u1718"],
  ["\u1719", "\u1719"],
  ["\u171A", "\u171A"],
  ["\u171B", "\u171B"],
  ["\u171C", "\u171C"],
  ["\u171D", "\u171D"],
  ["\u171E", "\u171E"],
  ["\u171F", "\u171F"],
  ["\u1720", "\u1720"], /* HANUNOO LETTER A */
  ["\u1721", "\u1721"], /* HANUNOO LETTER I */
  ["\u1722", "\u1722"], /* HANUNOO LETTER U */
  ["\u1723", "\u1723"], /* HANUNOO LETTER KA */
  ["\u1724", "\u1724"], /* HANUNOO LETTER GA */
  ["\u1725", "\u1725"], /* HANUNOO LETTER NGA */
  ["\u1726", "\u1726"], /* HANUNOO LETTER TA */
  ["\u1727", "\u1727"], /* HANUNOO LETTER DA */
  ["\u1728", "\u1728"], /* HANUNOO LETTER NA */
  ["\u1729", "\u1729"], /* HANUNOO LETTER PA */
  ["\u172A", "\u172A"], /* HANUNOO LETTER BA */
  ["\u172B", "\u172B"], /* HANUNOO LETTER MA */
  ["\u172C", "\u172C"], /* HANUNOO LETTER YA */
  ["\u172D", "\u172D"], /* HANUNOO LETTER RA */
  ["\u172E", "\u172E"], /* HANUNOO LETTER LA */
  ["\u172F", "\u172F"], /* HANUNOO LETTER WA */
  ["\u1730", "\u1730"], /* HANUNOO LETTER SA */
  ["\u1731", "\u1731"], /* HANUNOO LETTER HA */
  ["\u1732", "\u1732"], /* HANUNOO VOWEL SIGN I */
  ["\u1733", "\u1733"], /* HANUNOO VOWEL SIGN U */
  ["\u1734", "\u1734"], /* HANUNOO SIGN PAMUDPOD */
  ["\u1735", "\u1735"], /* PHILIPPINE SINGLE PUNCTUATION */
  ["\u1736", "\u1736"], /* PHILIPPINE DOUBLE PUNCTUATION */
  ["\u1737", "\u1737"],
  ["\u1738", "\u1738"],
  ["\u1739", "\u1739"],
  ["\u173A", "\u173A"],
  ["\u173B", "\u173B"],
  ["\u173C", "\u173C"],
  ["\u173D", "\u173D"],
  ["\u173E", "\u173E"],
  ["\u173F", "\u173F"],
  ["\u1740", "\u1740"], /* BUHID LETTER A */
  ["\u1741", "\u1741"], /* BUHID LETTER I */
  ["\u1742", "\u1742"], /* BUHID LETTER U */
  ["\u1743", "\u1743"], /* BUHID LETTER KA */
  ["\u1744", "\u1744"], /* BUHID LETTER GA */
  ["\u1745", "\u1745"], /* BUHID LETTER NGA */
  ["\u1746", "\u1746"], /* BUHID LETTER TA */
  ["\u1747", "\u1747"], /* BUHID LETTER DA */
  ["\u1748", "\u1748"], /* BUHID LETTER NA */
  ["\u1749", "\u1749"], /* BUHID LETTER PA */
  ["\u174A", "\u174A"], /* BUHID LETTER BA */
  ["\u174B", "\u174B"], /* BUHID LETTER MA */
  ["\u174C", "\u174C"], /* BUHID LETTER YA */
  ["\u174D", "\u174D"], /* BUHID LETTER RA */
  ["\u174E", "\u174E"], /* BUHID LETTER LA */
  ["\u174F", "\u174F"], /* BUHID LETTER WA */
  ["\u1750", "\u1750"], /* BUHID LETTER SA */
  ["\u1751", "\u1751"], /* BUHID LETTER HA */
  ["\u1752", "\u1752"], /* BUHID VOWEL SIGN I */
  ["\u1753", "\u1753"], /* BUHID VOWEL SIGN U */
  ["\u1754", "\u1754"],
  ["\u1755", "\u1755"],
  ["\u1756", "\u1756"],
  ["\u1757", "\u1757"],
  ["\u1758", "\u1758"],
  ["\u1759", "\u1759"],
  ["\u175A", "\u175A"],
  ["\u175B", "\u175B"],
  ["\u175C", "\u175C"],
  ["\u175D", "\u175D"],
  ["\u175E", "\u175E"],
  ["\u175F", "\u175F"],
  ["\u1760", "\u1760"], /* TAGBANWA LETTER A */
  ["\u1761", "\u1761"], /* TAGBANWA LETTER I */
  ["\u1762", "\u1762"], /* TAGBANWA LETTER U */
  ["\u1763", "\u1763"], /* TAGBANWA LETTER KA */
  ["\u1764", "\u1764"], /* TAGBANWA LETTER GA */
  ["\u1765", "\u1765"], /* TAGBANWA LETTER NGA */
  ["\u1766", "\u1766"], /* TAGBANWA LETTER TA */
  ["\u1767", "\u1767"], /* TAGBANWA LETTER DA */
  ["\u1768", "\u1768"], /* TAGBANWA LETTER NA */
  ["\u1769", "\u1769"], /* TAGBANWA LETTER PA */
  ["\u176A", "\u176A"], /* TAGBANWA LETTER BA */
  ["\u176B", "\u176B"], /* TAGBANWA LETTER MA */
  ["\u176C", "\u176C"], /* TAGBANWA LETTER YA */
  ["\u176D", "\u176D"],
  ["\u176E", "\u176E"], /* TAGBANWA LETTER LA */
  ["\u176F", "\u176F"], /* TAGBANWA LETTER WA */
  ["\u1770", "\u1770"], /* TAGBANWA LETTER SA */
  ["\u1771", "\u1771"],
  ["\u1772", "\u1772"], /* TAGBANWA VOWEL SIGN I */
  ["\u1773", "\u1773"], /* TAGBANWA VOWEL SIGN U */
  ["\u1774", "\u1774"],
  ["\u1775", "\u1775"],
  ["\u1776", "\u1776"],
  ["\u1777", "\u1777"],
  ["\u1778", "\u1778"],
  ["\u1779", "\u1779"],
  ["\u177A", "\u177A"],
  ["\u177B", "\u177B"],
  ["\u177C", "\u177C"],
  ["\u177D", "\u177D"],
  ["\u177E", "\u177E"],
  ["\u177F", "\u177F"],
  ["\u1780", "\u1780"], /* KHMER LETTER KA */
  ["\u1781", "\u1781"], /* KHMER LETTER KHA */
  ["\u1782", "\u1782"], /* KHMER LETTER KO */
  ["\u1783", "\u1783"], /* KHMER LETTER KHO */
  ["\u1784", "\u1784"], /* KHMER LETTER NGO */
  ["\u1785", "\u1785"], /* KHMER LETTER CA */
  ["\u1786", "\u1786"], /* KHMER LETTER CHA */
  ["\u1787", "\u1787"], /* KHMER LETTER CO */
  ["\u1788", "\u1788"], /* KHMER LETTER CHO */
  ["\u1789", "\u1789"], /* KHMER LETTER NYO */
  ["\u178A", "\u178A"], /* KHMER LETTER DA */
  ["\u178B", "\u178B"], /* KHMER LETTER TTHA */
  ["\u178C", "\u178C"], /* KHMER LETTER DO */
  ["\u178D", "\u178D"], /* KHMER LETTER TTHO */
  ["\u178E", "\u178E"], /* KHMER LETTER NNO */
  ["\u178F", "\u178F"], /* KHMER LETTER TA */
  ["\u1790", "\u1790"], /* KHMER LETTER THA */
  ["\u1791", "\u1791"], /* KHMER LETTER TO */
  ["\u1792", "\u1792"], /* KHMER LETTER THO */
  ["\u1793", "\u1793"], /* KHMER LETTER NO */
  ["\u1794", "\u1794"], /* KHMER LETTER BA */
  ["\u1795", "\u1795"], /* KHMER LETTER PHA */
  ["\u1796", "\u1796"], /* KHMER LETTER PO */
  ["\u1797", "\u1797"], /* KHMER LETTER PHO */
  ["\u1798", "\u1798"], /* KHMER LETTER MO */
  ["\u1799", "\u1799"], /* KHMER LETTER YO */
  ["\u179A", "\u179A"], /* KHMER LETTER RO */
  ["\u179B", "\u179B"], /* KHMER LETTER LO */
  ["\u179C", "\u179C"], /* KHMER LETTER VO */
  ["\u179D", "\u179D"], /* KHMER LETTER SHA */
  ["\u179E", "\u179E"], /* KHMER LETTER SSO */
  ["\u179F", "\u179F"], /* KHMER LETTER SA */
  ["\u17A0", "\u17A0"], /* KHMER LETTER HA */
  ["\u17A1", "\u17A1"], /* KHMER LETTER LA */
  ["\u17A2", "\u17A2"], /* KHMER LETTER QA */
  ["\u17A3", "\u17A3"], /* KHMER INDEPENDENT VOWEL QAQ */
  ["\u17A4", "\u17A4"], /* KHMER INDEPENDENT VOWEL QAA */
  ["\u17A5", "\u17A5"], /* KHMER INDEPENDENT VOWEL QI */
  ["\u17A6", "\u17A6"], /* KHMER INDEPENDENT VOWEL QII */
  ["\u17A7", "\u17A7"], /* KHMER INDEPENDENT VOWEL QU */
  ["\u17A8", "\u17A8"], /* KHMER INDEPENDENT VOWEL QUK */
  ["\u17A9", "\u17A9"], /* KHMER INDEPENDENT VOWEL QUU */
  ["\u17AA", "\u17AA"], /* KHMER INDEPENDENT VOWEL QUUV */
  ["\u17AB", "\u17AB"], /* KHMER INDEPENDENT VOWEL RY */
  ["\u17AC", "\u17AC"], /* KHMER INDEPENDENT VOWEL RYY */
  ["\u17AD", "\u17AD"], /* KHMER INDEPENDENT VOWEL LY */
  ["\u17AE", "\u17AE"], /* KHMER INDEPENDENT VOWEL LYY */
  ["\u17AF", "\u17AF"], /* KHMER INDEPENDENT VOWEL QE */
  ["\u17B0", "\u17B0"], /* KHMER INDEPENDENT VOWEL QAI */
  ["\u17B1", "\u17B1"], /* KHMER INDEPENDENT VOWEL QOO TYPE ONE */
  ["\u17B2", "\u17B2"], /* KHMER INDEPENDENT VOWEL QOO TYPE TWO */
  ["\u17B3", "\u17B3"], /* KHMER INDEPENDENT VOWEL QAU */
  ["\u17B4", "\u17B4"], /* KHMER VOWEL INHERENT AQ */
  ["\u17B5", "\u17B5"], /* KHMER VOWEL INHERENT AA */
  ["\u17B6", "\u17B6"], /* KHMER VOWEL SIGN AA */
  ["\u17B7", "\u17B7"], /* KHMER VOWEL SIGN I */
  ["\u17B8", "\u17B8"], /* KHMER VOWEL SIGN II */
  ["\u17B9", "\u17B9"], /* KHMER VOWEL SIGN Y */
  ["\u17BA", "\u17BA"], /* KHMER VOWEL SIGN YY */
  ["\u17BB", "\u17BB"], /* KHMER VOWEL SIGN U */
  ["\u17BC", "\u17BC"], /* KHMER VOWEL SIGN UU */
  ["\u17BD", "\u17BD"], /* KHMER VOWEL SIGN UA */
  ["\u17BE", "\u17BE"], /* KHMER VOWEL SIGN OE */
  ["\u17BF", "\u17BF"], /* KHMER VOWEL SIGN YA */
  ["\u17C0", "\u17C0"], /* KHMER VOWEL SIGN IE */
  ["\u17C1", "\u17C1"], /* KHMER VOWEL SIGN E */
  ["\u17C2", "\u17C2"], /* KHMER VOWEL SIGN AE */
  ["\u17C3", "\u17C3"], /* KHMER VOWEL SIGN AI */
  ["\u17C4", "\u17C4"], /* KHMER VOWEL SIGN OO */
  ["\u17C5", "\u17C5"], /* KHMER VOWEL SIGN AU */
  ["\u17C6", "\u17C6"], /* KHMER SIGN NIKAHIT */
  ["\u17C7", "\u17C7"], /* KHMER SIGN REAHMUK */
  ["\u17C8", "\u17C8"], /* KHMER SIGN YUUKALEAPINTU */
  ["\u17C9", "\u17C9"], /* KHMER SIGN MUUSIKATOAN */
  ["\u17CA", "\u17CA"], /* KHMER SIGN TRIISAP */
  ["\u17CB", "\u17CB"], /* KHMER SIGN BANTOC */
  ["\u17CC", "\u17CC"], /* KHMER SIGN ROBAT */
  ["\u17CD", "\u17CD"], /* KHMER SIGN TOANDAKHIAT */
  ["\u17CE", "\u17CE"], /* KHMER SIGN KAKABAT */
  ["\u17CF", "\u17CF"], /* KHMER SIGN AHSDA */
  ["\u17D0", "\u17D0"], /* KHMER SIGN SAMYOK SANNYA */
  ["\u17D1", "\u17D1"], /* KHMER SIGN VIRIAM */
  ["\u17D2", "\u17D2"], /* KHMER SIGN COENG */
  ["\u17D3", "\u17D3"], /* KHMER SIGN BATHAMASAT */
  ["\u17D4", "\u17D4"], /* KHMER SIGN KHAN */
  ["\u17D5", "\u17D5"], /* KHMER SIGN BARIYOOSAN */
  ["\u17D6", "\u17D6"], /* KHMER SIGN CAMNUC PII KUUH */
  ["\u17D7", "\u17D7"], /* KHMER SIGN LEK TOO */
  ["\u17D8", "\u17D8"], /* KHMER SIGN BEYYAL */
  ["\u17D9", "\u17D9"], /* KHMER SIGN PHNAEK MUAN */
  ["\u17DA", "\u17DA"], /* KHMER SIGN KOOMUUT */
  ["\u17DB", "\u17DB"], /* KHMER CURRENCY SYMBOL RIEL */
  ["\u17DC", "\u17DC"], /* KHMER SIGN AVAKRAHASANYA */
  ["\u17DD", "\u17DD"], /* KHMER SIGN ATTHACAN */
  ["\u17DE", "\u17DE"],
  ["\u17DF", "\u17DF"],
  ["\u17E0", "\u17E0"], /* KHMER DIGIT ZERO */
  ["\u17E1", "\u17E1"], /* KHMER DIGIT ONE */
  ["\u17E2", "\u17E2"], /* KHMER DIGIT TWO */
  ["\u17E3", "\u17E3"], /* KHMER DIGIT THREE */
  ["\u17E4", "\u17E4"], /* KHMER DIGIT FOUR */
  ["\u17E5", "\u17E5"], /* KHMER DIGIT FIVE */
  ["\u17E6", "\u17E6"], /* KHMER DIGIT SIX */
  ["\u17E7", "\u17E7"], /* KHMER DIGIT SEVEN */
  ["\u17E8", "\u17E8"], /* KHMER DIGIT EIGHT */
  ["\u17E9", "\u17E9"], /* KHMER DIGIT NINE */
  ["\u17EA", "\u17EA"],
  ["\u17EB", "\u17EB"],
  ["\u17EC", "\u17EC"],
  ["\u17ED", "\u17ED"],
  ["\u17EE", "\u17EE"],
  ["\u17EF", "\u17EF"],
  ["\u17F0", "\u17F0"], /* KHMER SYMBOL LEK ATTAK SON */
  ["\u17F1", "\u17F1"], /* KHMER SYMBOL LEK ATTAK MUOY */
  ["\u17F2", "\u17F2"], /* KHMER SYMBOL LEK ATTAK PII */
  ["\u17F3", "\u17F3"], /* KHMER SYMBOL LEK ATTAK BEI */
  ["\u17F4", "\u17F4"], /* KHMER SYMBOL LEK ATTAK BUON */
  ["\u17F5", "\u17F5"], /* KHMER SYMBOL LEK ATTAK PRAM */
  ["\u17F6", "\u17F6"], /* KHMER SYMBOL LEK ATTAK PRAM-MUOY */
  ["\u17F7", "\u17F7"], /* KHMER SYMBOL LEK ATTAK PRAM-PII */
  ["\u17F8", "\u17F8"], /* KHMER SYMBOL LEK ATTAK PRAM-BEI */
  ["\u17F9", "\u17F9"], /* KHMER SYMBOL LEK ATTAK PRAM-BUON */
  ["\u17FA", "\u17FA"],
  ["\u17FB", "\u17FB"],
  ["\u17FC", "\u17FC"],
  ["\u17FD", "\u17FD"],
  ["\u17FE", "\u17FE"],
  ["\u17FF", "\u17FF"],
  ["\u1800", "\u1800"], /* MONGOLIAN BIRGA */
  ["\u1801", "\u1801"], /* MONGOLIAN ELLIPSIS */
  ["\u1802", "\u1802"], /* MONGOLIAN COMMA */
  ["\u1803", "\u1803"], /* MONGOLIAN FULL STOP */
  ["\u1804", "\u1804"], /* MONGOLIAN COLON */
  ["\u1805", "\u1805"], /* MONGOLIAN FOUR DOTS */
  ["\u1806", "\u1806"], /* MONGOLIAN TODO SOFT HYPHEN */
  ["\u1807", "\u1807"], /* MONGOLIAN SIBE SYLLABLE BOUNDARY MARKER */
  ["\u1808", "\u1808"], /* MONGOLIAN MANCHU COMMA */
  ["\u1809", "\u1809"], /* MONGOLIAN MANCHU FULL STOP */
  ["\u180A", "\u180A"], /* MONGOLIAN NIRUGU */
  ["\u180B", "\u180B"], /* MONGOLIAN FREE VARIATION SELECTOR ONE */
  ["\u180C", "\u180C"], /* MONGOLIAN FREE VARIATION SELECTOR TWO */
  ["\u180D", "\u180D"], /* MONGOLIAN FREE VARIATION SELECTOR THREE */
  ["\u180E", "\u180E"], /* MONGOLIAN VOWEL SEPARATOR */
  ["\u180F", "\u180F"],
  ["\u1810", "\u1810"], /* MONGOLIAN DIGIT ZERO */
  ["\u1811", "\u1811"], /* MONGOLIAN DIGIT ONE */
  ["\u1812", "\u1812"], /* MONGOLIAN DIGIT TWO */
  ["\u1813", "\u1813"], /* MONGOLIAN DIGIT THREE */
  ["\u1814", "\u1814"], /* MONGOLIAN DIGIT FOUR */
  ["\u1815", "\u1815"], /* MONGOLIAN DIGIT FIVE */
  ["\u1816", "\u1816"], /* MONGOLIAN DIGIT SIX */
  ["\u1817", "\u1817"], /* MONGOLIAN DIGIT SEVEN */
  ["\u1818", "\u1818"], /* MONGOLIAN DIGIT EIGHT */
  ["\u1819", "\u1819"], /* MONGOLIAN DIGIT NINE */
  ["\u181A", "\u181A"],
  ["\u181B", "\u181B"],
  ["\u181C", "\u181C"],
  ["\u181D", "\u181D"],
  ["\u181E", "\u181E"],
  ["\u181F", "\u181F"],
  ["\u1820", "\u1820"], /* MONGOLIAN LETTER A */
  ["\u1821", "\u1821"], /* MONGOLIAN LETTER E */
  ["\u1822", "\u1822"], /* MONGOLIAN LETTER I */
  ["\u1823", "\u1823"], /* MONGOLIAN LETTER O */
  ["\u1824", "\u1824"], /* MONGOLIAN LETTER U */
  ["\u1825", "\u1825"], /* MONGOLIAN LETTER OE */
  ["\u1826", "\u1826"], /* MONGOLIAN LETTER UE */
  ["\u1827", "\u1827"], /* MONGOLIAN LETTER EE */
  ["\u1828", "\u1828"], /* MONGOLIAN LETTER NA */
  ["\u1829", "\u1829"], /* MONGOLIAN LETTER ANG */
  ["\u182A", "\u182A"], /* MONGOLIAN LETTER BA */
  ["\u182B", "\u182B"], /* MONGOLIAN LETTER PA */
  ["\u182C", "\u182C"], /* MONGOLIAN LETTER QA */
  ["\u182D", "\u182D"], /* MONGOLIAN LETTER GA */
  ["\u182E", "\u182E"], /* MONGOLIAN LETTER MA */
  ["\u182F", "\u182F"], /* MONGOLIAN LETTER LA */
  ["\u1830", "\u1830"], /* MONGOLIAN LETTER SA */
  ["\u1831", "\u1831"], /* MONGOLIAN LETTER SHA */
  ["\u1832", "\u1832"], /* MONGOLIAN LETTER TA */
  ["\u1833", "\u1833"], /* MONGOLIAN LETTER DA */
  ["\u1834", "\u1834"], /* MONGOLIAN LETTER CHA */
  ["\u1835", "\u1835"], /* MONGOLIAN LETTER JA */
  ["\u1836", "\u1836"], /* MONGOLIAN LETTER YA */
  ["\u1837", "\u1837"], /* MONGOLIAN LETTER RA */
  ["\u1838", "\u1838"], /* MONGOLIAN LETTER WA */
  ["\u1839", "\u1839"], /* MONGOLIAN LETTER FA */
  ["\u183A", "\u183A"], /* MONGOLIAN LETTER KA */
  ["\u183B", "\u183B"], /* MONGOLIAN LETTER KHA */
  ["\u183C", "\u183C"], /* MONGOLIAN LETTER TSA */
  ["\u183D", "\u183D"], /* MONGOLIAN LETTER ZA */
  ["\u183E", "\u183E"], /* MONGOLIAN LETTER HAA */
  ["\u183F", "\u183F"], /* MONGOLIAN LETTER ZRA */
  ["\u1840", "\u1840"], /* MONGOLIAN LETTER LHA */
  ["\u1841", "\u1841"], /* MONGOLIAN LETTER ZHI */
  ["\u1842", "\u1842"], /* MONGOLIAN LETTER CHI */
  ["\u1843", "\u1843"], /* MONGOLIAN LETTER TODO LONG VOWEL SIGN */
  ["\u1844", "\u1844"], /* MONGOLIAN LETTER TODO E */
  ["\u1845", "\u1845"], /* MONGOLIAN LETTER TODO I */
  ["\u1846", "\u1846"], /* MONGOLIAN LETTER TODO O */
  ["\u1847", "\u1847"], /* MONGOLIAN LETTER TODO U */
  ["\u1848", "\u1848"], /* MONGOLIAN LETTER TODO OE */
  ["\u1849", "\u1849"], /* MONGOLIAN LETTER TODO UE */
  ["\u184A", "\u184A"], /* MONGOLIAN LETTER TODO ANG */
  ["\u184B", "\u184B"], /* MONGOLIAN LETTER TODO BA */
  ["\u184C", "\u184C"], /* MONGOLIAN LETTER TODO PA */
  ["\u184D", "\u184D"], /* MONGOLIAN LETTER TODO QA */
  ["\u184E", "\u184E"], /* MONGOLIAN LETTER TODO GA */
  ["\u184F", "\u184F"], /* MONGOLIAN LETTER TODO MA */
  ["\u1850", "\u1850"], /* MONGOLIAN LETTER TODO TA */
  ["\u1851", "\u1851"], /* MONGOLIAN LETTER TODO DA */
  ["\u1852", "\u1852"], /* MONGOLIAN LETTER TODO CHA */
  ["\u1853", "\u1853"], /* MONGOLIAN LETTER TODO JA */
  ["\u1854", "\u1854"], /* MONGOLIAN LETTER TODO TSA */
  ["\u1855", "\u1855"], /* MONGOLIAN LETTER TODO YA */
  ["\u1856", "\u1856"], /* MONGOLIAN LETTER TODO WA */
  ["\u1857", "\u1857"], /* MONGOLIAN LETTER TODO KA */
  ["\u1858", "\u1858"], /* MONGOLIAN LETTER TODO GAA */
  ["\u1859", "\u1859"], /* MONGOLIAN LETTER TODO HAA */
  ["\u185A", "\u185A"], /* MONGOLIAN LETTER TODO JIA */
  ["\u185B", "\u185B"], /* MONGOLIAN LETTER TODO NIA */
  ["\u185C", "\u185C"], /* MONGOLIAN LETTER TODO DZA */
  ["\u185D", "\u185D"], /* MONGOLIAN LETTER SIBE E */
  ["\u185E", "\u185E"], /* MONGOLIAN LETTER SIBE I */
  ["\u185F", "\u185F"], /* MONGOLIAN LETTER SIBE IY */
  ["\u1860", "\u1860"], /* MONGOLIAN LETTER SIBE UE */
  ["\u1861", "\u1861"], /* MONGOLIAN LETTER SIBE U */
  ["\u1862", "\u1862"], /* MONGOLIAN LETTER SIBE ANG */
  ["\u1863", "\u1863"], /* MONGOLIAN LETTER SIBE KA */
  ["\u1864", "\u1864"], /* MONGOLIAN LETTER SIBE GA */
  ["\u1865", "\u1865"], /* MONGOLIAN LETTER SIBE HA */
  ["\u1866", "\u1866"], /* MONGOLIAN LETTER SIBE PA */
  ["\u1867", "\u1867"], /* MONGOLIAN LETTER SIBE SHA */
  ["\u1868", "\u1868"], /* MONGOLIAN LETTER SIBE TA */
  ["\u1869", "\u1869"], /* MONGOLIAN LETTER SIBE DA */
  ["\u186A", "\u186A"], /* MONGOLIAN LETTER SIBE JA */
  ["\u186B", "\u186B"], /* MONGOLIAN LETTER SIBE FA */
  ["\u186C", "\u186C"], /* MONGOLIAN LETTER SIBE GAA */
  ["\u186D", "\u186D"], /* MONGOLIAN LETTER SIBE HAA */
  ["\u186E", "\u186E"], /* MONGOLIAN LETTER SIBE TSA */
  ["\u186F", "\u186F"], /* MONGOLIAN LETTER SIBE ZA */
  ["\u1870", "\u1870"], /* MONGOLIAN LETTER SIBE RAA */
  ["\u1871", "\u1871"], /* MONGOLIAN LETTER SIBE CHA */
  ["\u1872", "\u1872"], /* MONGOLIAN LETTER SIBE ZHA */
  ["\u1873", "\u1873"], /* MONGOLIAN LETTER MANCHU I */
  ["\u1874", "\u1874"], /* MONGOLIAN LETTER MANCHU KA */
  ["\u1875", "\u1875"], /* MONGOLIAN LETTER MANCHU RA */
  ["\u1876", "\u1876"], /* MONGOLIAN LETTER MANCHU FA */
  ["\u1877", "\u1877"], /* MONGOLIAN LETTER MANCHU ZHA */
  ["\u1878", "\u1878"], /* MONGOLIAN LETTER CHA WITH TWO DOTS */
  ["\u1879", "\u1879"],
  ["\u187A", "\u187A"],
  ["\u187B", "\u187B"],
  ["\u187C", "\u187C"],
  ["\u187D", "\u187D"],
  ["\u187E", "\u187E"],
  ["\u187F", "\u187F"],
  ["\u1880", "\u1880"], /* MONGOLIAN LETTER ALI GALI ANUSVARA ONE */
  ["\u1881", "\u1881"], /* MONGOLIAN LETTER ALI GALI VISARGA ONE */
  ["\u1882", "\u1882"], /* MONGOLIAN LETTER ALI GALI DAMARU */
  ["\u1883", "\u1883"], /* MONGOLIAN LETTER ALI GALI UBADAMA */
  ["\u1884", "\u1884"], /* MONGOLIAN LETTER ALI GALI INVERTED UBADAMA */
  ["\u1885", "\u1885"], /* MONGOLIAN LETTER ALI GALI BALUDA */
  ["\u1886", "\u1886"], /* MONGOLIAN LETTER ALI GALI THREE BALUDA */
  ["\u1887", "\u1887"], /* MONGOLIAN LETTER ALI GALI A */
  ["\u1888", "\u1888"], /* MONGOLIAN LETTER ALI GALI I */
  ["\u1889", "\u1889"], /* MONGOLIAN LETTER ALI GALI KA */
  ["\u188A", "\u188A"], /* MONGOLIAN LETTER ALI GALI NGA */
  ["\u188B", "\u188B"], /* MONGOLIAN LETTER ALI GALI CA */
  ["\u188C", "\u188C"], /* MONGOLIAN LETTER ALI GALI TTA */
  ["\u188D", "\u188D"], /* MONGOLIAN LETTER ALI GALI TTHA */
  ["\u188E", "\u188E"], /* MONGOLIAN LETTER ALI GALI DDA */
  ["\u188F", "\u188F"], /* MONGOLIAN LETTER ALI GALI NNA */
  ["\u1890", "\u1890"], /* MONGOLIAN LETTER ALI GALI TA */
  ["\u1891", "\u1891"], /* MONGOLIAN LETTER ALI GALI DA */
  ["\u1892", "\u1892"], /* MONGOLIAN LETTER ALI GALI PA */
  ["\u1893", "\u1893"], /* MONGOLIAN LETTER ALI GALI PHA */
  ["\u1894", "\u1894"], /* MONGOLIAN LETTER ALI GALI SSA */
  ["\u1895", "\u1895"], /* MONGOLIAN LETTER ALI GALI ZHA */
  ["\u1896", "\u1896"], /* MONGOLIAN LETTER ALI GALI ZA */
  ["\u1897", "\u1897"], /* MONGOLIAN LETTER ALI GALI AH */
  ["\u1898", "\u1898"], /* MONGOLIAN LETTER TODO ALI GALI TA */
  ["\u1899", "\u1899"], /* MONGOLIAN LETTER TODO ALI GALI ZHA */
  ["\u189A", "\u189A"], /* MONGOLIAN LETTER MANCHU ALI GALI GHA */
  ["\u189B", "\u189B"], /* MONGOLIAN LETTER MANCHU ALI GALI NGA */
  ["\u189C", "\u189C"], /* MONGOLIAN LETTER MANCHU ALI GALI CA */
  ["\u189D", "\u189D"], /* MONGOLIAN LETTER MANCHU ALI GALI JHA */
  ["\u189E", "\u189E"], /* MONGOLIAN LETTER MANCHU ALI GALI TTA */
  ["\u189F", "\u189F"], /* MONGOLIAN LETTER MANCHU ALI GALI DDHA */
  ["\u18A0", "\u18A0"], /* MONGOLIAN LETTER MANCHU ALI GALI TA */
  ["\u18A1", "\u18A1"], /* MONGOLIAN LETTER MANCHU ALI GALI DHA */
  ["\u18A2", "\u18A2"], /* MONGOLIAN LETTER MANCHU ALI GALI SSA */
  ["\u18A3", "\u18A3"], /* MONGOLIAN LETTER MANCHU ALI GALI CYA */
  ["\u18A4", "\u18A4"], /* MONGOLIAN LETTER MANCHU ALI GALI ZHA */
  ["\u18A5", "\u18A5"], /* MONGOLIAN LETTER MANCHU ALI GALI ZA */
  ["\u18A6", "\u18A6"], /* MONGOLIAN LETTER ALI GALI HALF U */
  ["\u18A7", "\u18A7"], /* MONGOLIAN LETTER ALI GALI HALF YA */
  ["\u18A8", "\u18A8"], /* MONGOLIAN LETTER MANCHU ALI GALI BHA */
  ["\u18A9", "\u18A9"], /* MONGOLIAN LETTER ALI GALI DAGALGA */
  ["\u18AA", "\u18AA"], /* MONGOLIAN LETTER MANCHU ALI GALI LHA */
  ["\u18AB", "\u18AB"],
  ["\u18AC", "\u18AC"],
  ["\u18AD", "\u18AD"],
  ["\u18AE", "\u18AE"],
  ["\u18AF", "\u18AF"],
  ["\u18B0", "\u18B0"], /* CANADIAN SYLLABICS OY */
  ["\u18B1", "\u18B1"], /* CANADIAN SYLLABICS AY */
  ["\u18B2", "\u18B2"], /* CANADIAN SYLLABICS AAY */
  ["\u18B3", "\u18B3"], /* CANADIAN SYLLABICS WAY */
  ["\u18B4", "\u18B4"], /* CANADIAN SYLLABICS POY */
  ["\u18B5", "\u18B5"], /* CANADIAN SYLLABICS PAY */
  ["\u18B6", "\u18B6"], /* CANADIAN SYLLABICS PWOY */
  ["\u18B7", "\u18B7"], /* CANADIAN SYLLABICS TAY */
  ["\u18B8", "\u18B8"], /* CANADIAN SYLLABICS KAY */
  ["\u18B9", "\u18B9"], /* CANADIAN SYLLABICS KWAY */
  ["\u18BA", "\u18BA"], /* CANADIAN SYLLABICS MAY */
  ["\u18BB", "\u18BB"], /* CANADIAN SYLLABICS NOY */
  ["\u18BC", "\u18BC"], /* CANADIAN SYLLABICS NAY */
  ["\u18BD", "\u18BD"], /* CANADIAN SYLLABICS LAY */
  ["\u18BE", "\u18BE"], /* CANADIAN SYLLABICS SOY */
  ["\u18BF", "\u18BF"], /* CANADIAN SYLLABICS SAY */
  ["\u18C0", "\u18C0"], /* CANADIAN SYLLABICS SHOY */
  ["\u18C1", "\u18C1"], /* CANADIAN SYLLABICS SHAY */
  ["\u18C2", "\u18C2"], /* CANADIAN SYLLABICS SHWOY */
  ["\u18C3", "\u18C3"], /* CANADIAN SYLLABICS YOY */
  ["\u18C4", "\u18C4"], /* CANADIAN SYLLABICS YAY */
  ["\u18C5", "\u18C5"], /* CANADIAN SYLLABICS RAY */
  ["\u18C6", "\u18C6"], /* CANADIAN SYLLABICS NWI */
  ["\u18C7", "\u18C7"], /* CANADIAN SYLLABICS OJIBWAY NWI */
  ["\u18C8", "\u18C8"], /* CANADIAN SYLLABICS NWII */
  ["\u18C9", "\u18C9"], /* CANADIAN SYLLABICS OJIBWAY NWII */
  ["\u18CA", "\u18CA"], /* CANADIAN SYLLABICS NWO */
  ["\u18CB", "\u18CB"], /* CANADIAN SYLLABICS OJIBWAY NWO */
  ["\u18CC", "\u18CC"], /* CANADIAN SYLLABICS NWOO */
  ["\u18CD", "\u18CD"], /* CANADIAN SYLLABICS OJIBWAY NWOO */
  ["\u18CE", "\u18CE"], /* CANADIAN SYLLABICS RWEE */
  ["\u18CF", "\u18CF"], /* CANADIAN SYLLABICS RWI */
  ["\u18D0", "\u18D0"], /* CANADIAN SYLLABICS RWII */
  ["\u18D1", "\u18D1"], /* CANADIAN SYLLABICS RWO */
  ["\u18D2", "\u18D2"], /* CANADIAN SYLLABICS RWOO */
  ["\u18D3", "\u18D3"], /* CANADIAN SYLLABICS RWA */
  ["\u18D4", "\u18D4"], /* CANADIAN SYLLABICS OJIBWAY P */
  ["\u18D5", "\u18D5"], /* CANADIAN SYLLABICS OJIBWAY T */
  ["\u18D6", "\u18D6"], /* CANADIAN SYLLABICS OJIBWAY K */
  ["\u18D7", "\u18D7"], /* CANADIAN SYLLABICS OJIBWAY C */
  ["\u18D8", "\u18D8"], /* CANADIAN SYLLABICS OJIBWAY M */
  ["\u18D9", "\u18D9"], /* CANADIAN SYLLABICS OJIBWAY N */
  ["\u18DA", "\u18DA"], /* CANADIAN SYLLABICS OJIBWAY S */
  ["\u18DB", "\u18DB"], /* CANADIAN SYLLABICS OJIBWAY SH */
  ["\u18DC", "\u18DC"], /* CANADIAN SYLLABICS EASTERN W */
  ["\u18DD", "\u18DD"], /* CANADIAN SYLLABICS WESTERN W */
  ["\u18DE", "\u18DE"], /* CANADIAN SYLLABICS FINAL SMALL RING */
  ["\u18DF", "\u18DF"], /* CANADIAN SYLLABICS FINAL RAISED DOT */
  ["\u18E0", "\u18E0"], /* CANADIAN SYLLABICS R-CREE RWE */
  ["\u18E1", "\u18E1"], /* CANADIAN SYLLABICS WEST-CREE LOO */
  ["\u18E2", "\u18E2"], /* CANADIAN SYLLABICS WEST-CREE LAA */
  ["\u18E3", "\u18E3"], /* CANADIAN SYLLABICS THWE */
  ["\u18E4", "\u18E4"], /* CANADIAN SYLLABICS THWA */
  ["\u18E5", "\u18E5"], /* CANADIAN SYLLABICS TTHWE */
  ["\u18E6", "\u18E6"], /* CANADIAN SYLLABICS TTHOO */
  ["\u18E7", "\u18E7"], /* CANADIAN SYLLABICS TTHAA */
  ["\u18E8", "\u18E8"], /* CANADIAN SYLLABICS TLHWE */
  ["\u18E9", "\u18E9"], /* CANADIAN SYLLABICS TLHOO */
  ["\u18EA", "\u18EA"], /* CANADIAN SYLLABICS SAYISI SHWE */
  ["\u18EB", "\u18EB"], /* CANADIAN SYLLABICS SAYISI SHOO */
  ["\u18EC", "\u18EC"], /* CANADIAN SYLLABICS SAYISI HOO */
  ["\u18ED", "\u18ED"], /* CANADIAN SYLLABICS CARRIER GWU */
  ["\u18EE", "\u18EE"], /* CANADIAN SYLLABICS CARRIER DENE GEE */
  ["\u18EF", "\u18EF"], /* CANADIAN SYLLABICS CARRIER GAA */
  ["\u18F0", "\u18F0"], /* CANADIAN SYLLABICS CARRIER GWA */
  ["\u18F1", "\u18F1"], /* CANADIAN SYLLABICS SAYISI JUU */
  ["\u18F2", "\u18F2"], /* CANADIAN SYLLABICS CARRIER JWA */
  ["\u18F3", "\u18F3"], /* CANADIAN SYLLABICS BEAVER DENE L */
  ["\u18F4", "\u18F4"], /* CANADIAN SYLLABICS BEAVER DENE R */
  ["\u18F5", "\u18F5"], /* CANADIAN SYLLABICS CARRIER DENTAL S */
  ["\u18F6", "\u18F6"],
  ["\u18F7", "\u18F7"],
  ["\u18F8", "\u18F8"],
  ["\u18F9", "\u18F9"],
  ["\u18FA", "\u18FA"],
  ["\u18FB", "\u18FB"],
  ["\u18FC", "\u18FC"],
  ["\u18FD", "\u18FD"],
  ["\u18FE", "\u18FE"],
  ["\u18FF", "\u18FF"],
  ["\u1900", "\u1900"], /* LIMBU VOWEL-CARRIER LETTER */
  ["\u1901", "\u1901"], /* LIMBU LETTER KA */
  ["\u1902", "\u1902"], /* LIMBU LETTER KHA */
  ["\u1903", "\u1903"], /* LIMBU LETTER GA */
  ["\u1904", "\u1904"], /* LIMBU LETTER GHA */
  ["\u1905", "\u1905"], /* LIMBU LETTER NGA */
  ["\u1906", "\u1906"], /* LIMBU LETTER CA */
  ["\u1907", "\u1907"], /* LIMBU LETTER CHA */
  ["\u1908", "\u1908"], /* LIMBU LETTER JA */
  ["\u1909", "\u1909"], /* LIMBU LETTER JHA */
  ["\u190A", "\u190A"], /* LIMBU LETTER YAN */
  ["\u190B", "\u190B"], /* LIMBU LETTER TA */
  ["\u190C", "\u190C"], /* LIMBU LETTER THA */
  ["\u190D", "\u190D"], /* LIMBU LETTER DA */
  ["\u190E", "\u190E"], /* LIMBU LETTER DHA */
  ["\u190F", "\u190F"], /* LIMBU LETTER NA */
  ["\u1910", "\u1910"], /* LIMBU LETTER PA */
  ["\u1911", "\u1911"], /* LIMBU LETTER PHA */
  ["\u1912", "\u1912"], /* LIMBU LETTER BA */
  ["\u1913", "\u1913"], /* LIMBU LETTER BHA */
  ["\u1914", "\u1914"], /* LIMBU LETTER MA */
  ["\u1915", "\u1915"], /* LIMBU LETTER YA */
  ["\u1916", "\u1916"], /* LIMBU LETTER RA */
  ["\u1917", "\u1917"], /* LIMBU LETTER LA */
  ["\u1918", "\u1918"], /* LIMBU LETTER WA */
  ["\u1919", "\u1919"], /* LIMBU LETTER SHA */
  ["\u191A", "\u191A"], /* LIMBU LETTER SSA */
  ["\u191B", "\u191B"], /* LIMBU LETTER SA */
  ["\u191C", "\u191C"], /* LIMBU LETTER HA */
  ["\u191D", "\u191D"], /* LIMBU LETTER GYAN */
  ["\u191E", "\u191E"], /* LIMBU LETTER TRA */
  ["\u191F", "\u191F"],
  ["\u1920", "\u1920"], /* LIMBU VOWEL SIGN A */
  ["\u1921", "\u1921"], /* LIMBU VOWEL SIGN I */
  ["\u1922", "\u1922"], /* LIMBU VOWEL SIGN U */
  ["\u1923", "\u1923"], /* LIMBU VOWEL SIGN EE */
  ["\u1924", "\u1924"], /* LIMBU VOWEL SIGN AI */
  ["\u1925", "\u1925"], /* LIMBU VOWEL SIGN OO */
  ["\u1926", "\u1926"], /* LIMBU VOWEL SIGN AU */
  ["\u1927", "\u1927"], /* LIMBU VOWEL SIGN E */
  ["\u1928", "\u1928"], /* LIMBU VOWEL SIGN O */
  ["\u1929", "\u1929"], /* LIMBU SUBJOINED LETTER YA */
  ["\u192A", "\u192A"], /* LIMBU SUBJOINED LETTER RA */
  ["\u192B", "\u192B"], /* LIMBU SUBJOINED LETTER WA */
  ["\u192C", "\u192C"],
  ["\u192D", "\u192D"],
  ["\u192E", "\u192E"],
  ["\u192F", "\u192F"],
  ["\u1930", "\u1930"], /* LIMBU SMALL LETTER KA */
  ["\u1931", "\u1931"], /* LIMBU SMALL LETTER NGA */
  ["\u1932", "\u1932"], /* LIMBU SMALL LETTER ANUSVARA */
  ["\u1933", "\u1933"], /* LIMBU SMALL LETTER TA */
  ["\u1934", "\u1934"], /* LIMBU SMALL LETTER NA */
  ["\u1935", "\u1935"], /* LIMBU SMALL LETTER PA */
  ["\u1936", "\u1936"], /* LIMBU SMALL LETTER MA */
  ["\u1937", "\u1937"], /* LIMBU SMALL LETTER RA */
  ["\u1938", "\u1938"], /* LIMBU SMALL LETTER LA */
  ["\u1939", "\u1939"], /* LIMBU SIGN MUKPHRENG */
  ["\u193A", "\u193A"], /* LIMBU SIGN KEMPHRENG */
  ["\u193B", "\u193B"], /* LIMBU SIGN SA-I */
  ["\u193C", "\u193C"],
  ["\u193D", "\u193D"],
  ["\u193E", "\u193E"],
  ["\u193F", "\u193F"],
  ["\u1940", "\u1940"], /* LIMBU SIGN LOO */
  ["\u1941", "\u1941"],
  ["\u1942", "\u1942"],
  ["\u1943", "\u1943"],
  ["\u1944", "\u1944"], /* LIMBU EXCLAMATION MARK */
  ["\u1945", "\u1945"], /* LIMBU QUESTION MARK */
  ["\u1946", "\u1946"], /* LIMBU DIGIT ZERO */
  ["\u1947", "\u1947"], /* LIMBU DIGIT ONE */
  ["\u1948", "\u1948"], /* LIMBU DIGIT TWO */
  ["\u1949", "\u1949"], /* LIMBU DIGIT THREE */
  ["\u194A", "\u194A"], /* LIMBU DIGIT FOUR */
  ["\u194B", "\u194B"], /* LIMBU DIGIT FIVE */
  ["\u194C", "\u194C"], /* LIMBU DIGIT SIX */
  ["\u194D", "\u194D"], /* LIMBU DIGIT SEVEN */
  ["\u194E", "\u194E"], /* LIMBU DIGIT EIGHT */
  ["\u194F", "\u194F"], /* LIMBU DIGIT NINE */
  ["\u1950", "\u1950"], /* TAI LE LETTER KA */
  ["\u1951", "\u1951"], /* TAI LE LETTER XA */
  ["\u1952", "\u1952"], /* TAI LE LETTER NGA */
  ["\u1953", "\u1953"], /* TAI LE LETTER TSA */
  ["\u1954", "\u1954"], /* TAI LE LETTER SA */
  ["\u1955", "\u1955"], /* TAI LE LETTER YA */
  ["\u1956", "\u1956"], /* TAI LE LETTER TA */
  ["\u1957", "\u1957"], /* TAI LE LETTER THA */
  ["\u1958", "\u1958"], /* TAI LE LETTER LA */
  ["\u1959", "\u1959"], /* TAI LE LETTER PA */
  ["\u195A", "\u195A"], /* TAI LE LETTER PHA */
  ["\u195B", "\u195B"], /* TAI LE LETTER MA */
  ["\u195C", "\u195C"], /* TAI LE LETTER FA */
  ["\u195D", "\u195D"], /* TAI LE LETTER VA */
  ["\u195E", "\u195E"], /* TAI LE LETTER HA */
  ["\u195F", "\u195F"], /* TAI LE LETTER QA */
  ["\u1960", "\u1960"], /* TAI LE LETTER KHA */
  ["\u1961", "\u1961"], /* TAI LE LETTER TSHA */
  ["\u1962", "\u1962"], /* TAI LE LETTER NA */
  ["\u1963", "\u1963"], /* TAI LE LETTER A */
  ["\u1964", "\u1964"], /* TAI LE LETTER I */
  ["\u1965", "\u1965"], /* TAI LE LETTER EE */
  ["\u1966", "\u1966"], /* TAI LE LETTER EH */
  ["\u1967", "\u1967"], /* TAI LE LETTER U */
  ["\u1968", "\u1968"], /* TAI LE LETTER OO */
  ["\u1969", "\u1969"], /* TAI LE LETTER O */
  ["\u196A", "\u196A"], /* TAI LE LETTER UE */
  ["\u196B", "\u196B"], /* TAI LE LETTER E */
  ["\u196C", "\u196C"], /* TAI LE LETTER AUE */
  ["\u196D", "\u196D"], /* TAI LE LETTER AI */
  ["\u196E", "\u196E"],
  ["\u196F", "\u196F"],
  ["\u1970", "\u1970"], /* TAI LE LETTER TONE-2 */
  ["\u1971", "\u1971"], /* TAI LE LETTER TONE-3 */
  ["\u1972", "\u1972"], /* TAI LE LETTER TONE-4 */
  ["\u1973", "\u1973"], /* TAI LE LETTER TONE-5 */
  ["\u1974", "\u1974"], /* TAI LE LETTER TONE-6 */
  ["\u1975", "\u1975"],
  ["\u1976", "\u1976"],
  ["\u1977", "\u1977"],
  ["\u1978", "\u1978"],
  ["\u1979", "\u1979"],
  ["\u197A", "\u197A"],
  ["\u197B", "\u197B"],
  ["\u197C", "\u197C"],
  ["\u197D", "\u197D"],
  ["\u197E", "\u197E"],
  ["\u197F", "\u197F"],
  ["\u1980", "\u1980"], /* NEW TAI LUE LETTER HIGH QA */
  ["\u1981", "\u1981"], /* NEW TAI LUE LETTER LOW QA */
  ["\u1982", "\u1982"], /* NEW TAI LUE LETTER HIGH KA */
  ["\u1983", "\u1983"], /* NEW TAI LUE LETTER HIGH XA */
  ["\u1984", "\u1984"], /* NEW TAI LUE LETTER HIGH NGA */
  ["\u1985", "\u1985"], /* NEW TAI LUE LETTER LOW KA */
  ["\u1986", "\u1986"], /* NEW TAI LUE LETTER LOW XA */
  ["\u1987", "\u1987"], /* NEW TAI LUE LETTER LOW NGA */
  ["\u1988", "\u1988"], /* NEW TAI LUE LETTER HIGH TSA */
  ["\u1989", "\u1989"], /* NEW TAI LUE LETTER HIGH SA */
  ["\u198A", "\u198A"], /* NEW TAI LUE LETTER HIGH YA */
  ["\u198B", "\u198B"], /* NEW TAI LUE LETTER LOW TSA */
  ["\u198C", "\u198C"], /* NEW TAI LUE LETTER LOW SA */
  ["\u198D", "\u198D"], /* NEW TAI LUE LETTER LOW YA */
  ["\u198E", "\u198E"], /* NEW TAI LUE LETTER HIGH TA */
  ["\u198F", "\u198F"], /* NEW TAI LUE LETTER HIGH THA */
  ["\u1990", "\u1990"], /* NEW TAI LUE LETTER HIGH NA */
  ["\u1991", "\u1991"], /* NEW TAI LUE LETTER LOW TA */
  ["\u1992", "\u1992"], /* NEW TAI LUE LETTER LOW THA */
  ["\u1993", "\u1993"], /* NEW TAI LUE LETTER LOW NA */
  ["\u1994", "\u1994"], /* NEW TAI LUE LETTER HIGH PA */
  ["\u1995", "\u1995"], /* NEW TAI LUE LETTER HIGH PHA */
  ["\u1996", "\u1996"], /* NEW TAI LUE LETTER HIGH MA */
  ["\u1997", "\u1997"], /* NEW TAI LUE LETTER LOW PA */
  ["\u1998", "\u1998"], /* NEW TAI LUE LETTER LOW PHA */
  ["\u1999", "\u1999"], /* NEW TAI LUE LETTER LOW MA */
  ["\u199A", "\u199A"], /* NEW TAI LUE LETTER HIGH FA */
  ["\u199B", "\u199B"], /* NEW TAI LUE LETTER HIGH VA */
  ["\u199C", "\u199C"], /* NEW TAI LUE LETTER HIGH LA */
  ["\u199D", "\u199D"], /* NEW TAI LUE LETTER LOW FA */
  ["\u199E", "\u199E"], /* NEW TAI LUE LETTER LOW VA */
  ["\u199F", "\u199F"], /* NEW TAI LUE LETTER LOW LA */
  ["\u19A0", "\u19A0"], /* NEW TAI LUE LETTER HIGH HA */
  ["\u19A1", "\u19A1"], /* NEW TAI LUE LETTER HIGH DA */
  ["\u19A2", "\u19A2"], /* NEW TAI LUE LETTER HIGH BA */
  ["\u19A3", "\u19A3"], /* NEW TAI LUE LETTER LOW HA */
  ["\u19A4", "\u19A4"], /* NEW TAI LUE LETTER LOW DA */
  ["\u19A5", "\u19A5"], /* NEW TAI LUE LETTER LOW BA */
  ["\u19A6", "\u19A6"], /* NEW TAI LUE LETTER HIGH KVA */
  ["\u19A7", "\u19A7"], /* NEW TAI LUE LETTER HIGH XVA */
  ["\u19A8", "\u19A8"], /* NEW TAI LUE LETTER LOW KVA */
  ["\u19A9", "\u19A9"], /* NEW TAI LUE LETTER LOW XVA */
  ["\u19AA", "\u19AA"], /* NEW TAI LUE LETTER HIGH SUA */
  ["\u19AB", "\u19AB"], /* NEW TAI LUE LETTER LOW SUA */
  ["\u19AC", "\u19AC"],
  ["\u19AD", "\u19AD"],
  ["\u19AE", "\u19AE"],
  ["\u19AF", "\u19AF"],
  ["\u19B0", "\u19B0"], /* NEW TAI LUE VOWEL SIGN VOWEL SHORTENER */
  ["\u19B1", "\u19B1"], /* NEW TAI LUE VOWEL SIGN AA */
  ["\u19B2", "\u19B2"], /* NEW TAI LUE VOWEL SIGN II */
  ["\u19B3", "\u19B3"], /* NEW TAI LUE VOWEL SIGN U */
  ["\u19B4", "\u19B4"], /* NEW TAI LUE VOWEL SIGN UU */
  ["\u19B5", "\u19B5"], /* NEW TAI LUE VOWEL SIGN E */
  ["\u19B6", "\u19B6"], /* NEW TAI LUE VOWEL SIGN AE */
  ["\u19B7", "\u19B7"], /* NEW TAI LUE VOWEL SIGN O */
  ["\u19B8", "\u19B8"], /* NEW TAI LUE VOWEL SIGN OA */
  ["\u19B9", "\u19B9"], /* NEW TAI LUE VOWEL SIGN UE */
  ["\u19BA", "\u19BA"], /* NEW TAI LUE VOWEL SIGN AY */
  ["\u19BB", "\u19BB"], /* NEW TAI LUE VOWEL SIGN AAY */
  ["\u19BC", "\u19BC"], /* NEW TAI LUE VOWEL SIGN UY */
  ["\u19BD", "\u19BD"], /* NEW TAI LUE VOWEL SIGN OY */
  ["\u19BE", "\u19BE"], /* NEW TAI LUE VOWEL SIGN OAY */
  ["\u19BF", "\u19BF"], /* NEW TAI LUE VOWEL SIGN UEY */
  ["\u19C0", "\u19C0"], /* NEW TAI LUE VOWEL SIGN IY */
  ["\u19C1", "\u19C1"], /* NEW TAI LUE LETTER FINAL V */
  ["\u19C2", "\u19C2"], /* NEW TAI LUE LETTER FINAL NG */
  ["\u19C3", "\u19C3"], /* NEW TAI LUE LETTER FINAL N */
  ["\u19C4", "\u19C4"], /* NEW TAI LUE LETTER FINAL M */
  ["\u19C5", "\u19C5"], /* NEW TAI LUE LETTER FINAL K */
  ["\u19C6", "\u19C6"], /* NEW TAI LUE LETTER FINAL D */
  ["\u19C7", "\u19C7"], /* NEW TAI LUE LETTER FINAL B */
  ["\u19C8", "\u19C8"], /* NEW TAI LUE TONE MARK-1 */
  ["\u19C9", "\u19C9"], /* NEW TAI LUE TONE MARK-2 */
  ["\u19CA", "\u19CA"],
  ["\u19CB", "\u19CB"],
  ["\u19CC", "\u19CC"],
  ["\u19CD", "\u19CD"],
  ["\u19CE", "\u19CE"],
  ["\u19CF", "\u19CF"],
  ["\u19D0", "\u19D0"], /* NEW TAI LUE DIGIT ZERO */
  ["\u19D1", "\u19D1"], /* NEW TAI LUE DIGIT ONE */
  ["\u19D2", "\u19D2"], /* NEW TAI LUE DIGIT TWO */
  ["\u19D3", "\u19D3"], /* NEW TAI LUE DIGIT THREE */
  ["\u19D4", "\u19D4"], /* NEW TAI LUE DIGIT FOUR */
  ["\u19D5", "\u19D5"], /* NEW TAI LUE DIGIT FIVE */
  ["\u19D6", "\u19D6"], /* NEW TAI LUE DIGIT SIX */
  ["\u19D7", "\u19D7"], /* NEW TAI LUE DIGIT SEVEN */
  ["\u19D8", "\u19D8"], /* NEW TAI LUE DIGIT EIGHT */
  ["\u19D9", "\u19D9"], /* NEW TAI LUE DIGIT NINE */
  ["\u19DA", "\u19DA"], /* NEW TAI LUE THAM DIGIT ONE */
  ["\u19DB", "\u19DB"],
  ["\u19DC", "\u19DC"],
  ["\u19DD", "\u19DD"],
  ["\u19DE", "\u19DE"], /* NEW TAI LUE SIGN LAE */
  ["\u19DF", "\u19DF"], /* NEW TAI LUE SIGN LAEV */
  ["\u19E0", "\u19E0"], /* KHMER SYMBOL PATHAMASAT */
  ["\u19E1", "\u19E1"], /* KHMER SYMBOL MUOY KOET */
  ["\u19E2", "\u19E2"], /* KHMER SYMBOL PII KOET */
  ["\u19E3", "\u19E3"], /* KHMER SYMBOL BEI KOET */
  ["\u19E4", "\u19E4"], /* KHMER SYMBOL BUON KOET */
  ["\u19E5", "\u19E5"], /* KHMER SYMBOL PRAM KOET */
  ["\u19E6", "\u19E6"], /* KHMER SYMBOL PRAM-MUOY KOET */
  ["\u19E7", "\u19E7"], /* KHMER SYMBOL PRAM-PII KOET */
  ["\u19E8", "\u19E8"], /* KHMER SYMBOL PRAM-BEI KOET */
  ["\u19E9", "\u19E9"], /* KHMER SYMBOL PRAM-BUON KOET */
  ["\u19EA", "\u19EA"], /* KHMER SYMBOL DAP KOET */
  ["\u19EB", "\u19EB"], /* KHMER SYMBOL DAP-MUOY KOET */
  ["\u19EC", "\u19EC"], /* KHMER SYMBOL DAP-PII KOET */
  ["\u19ED", "\u19ED"], /* KHMER SYMBOL DAP-BEI KOET */
  ["\u19EE", "\u19EE"], /* KHMER SYMBOL DAP-BUON KOET */
  ["\u19EF", "\u19EF"], /* KHMER SYMBOL DAP-PRAM KOET */
  ["\u19F0", "\u19F0"], /* KHMER SYMBOL TUTEYASAT */
  ["\u19F1", "\u19F1"], /* KHMER SYMBOL MUOY ROC */
  ["\u19F2", "\u19F2"], /* KHMER SYMBOL PII ROC */
  ["\u19F3", "\u19F3"], /* KHMER SYMBOL BEI ROC */
  ["\u19F4", "\u19F4"], /* KHMER SYMBOL BUON ROC */
  ["\u19F5", "\u19F5"], /* KHMER SYMBOL PRAM ROC */
  ["\u19F6", "\u19F6"], /* KHMER SYMBOL PRAM-MUOY ROC */
  ["\u19F7", "\u19F7"], /* KHMER SYMBOL PRAM-PII ROC */
  ["\u19F8", "\u19F8"], /* KHMER SYMBOL PRAM-BEI ROC */
  ["\u19F9", "\u19F9"], /* KHMER SYMBOL PRAM-BUON ROC */
  ["\u19FA", "\u19FA"], /* KHMER SYMBOL DAP ROC */
  ["\u19FB", "\u19FB"], /* KHMER SYMBOL DAP-MUOY ROC */
  ["\u19FC", "\u19FC"], /* KHMER SYMBOL DAP-PII ROC */
  ["\u19FD", "\u19FD"], /* KHMER SYMBOL DAP-BEI ROC */
  ["\u19FE", "\u19FE"], /* KHMER SYMBOL DAP-BUON ROC */
  ["\u19FF", "\u19FF"], /* KHMER SYMBOL DAP-PRAM ROC */
  ["\u1A00", "\u1A00"], /* BUGINESE LETTER KA */
  ["\u1A01", "\u1A01"], /* BUGINESE LETTER GA */
  ["\u1A02", "\u1A02"], /* BUGINESE LETTER NGA */
  ["\u1A03", "\u1A03"], /* BUGINESE LETTER NGKA */
  ["\u1A04", "\u1A04"], /* BUGINESE LETTER PA */
  ["\u1A05", "\u1A05"], /* BUGINESE LETTER BA */
  ["\u1A06", "\u1A06"], /* BUGINESE LETTER MA */
  ["\u1A07", "\u1A07"], /* BUGINESE LETTER MPA */
  ["\u1A08", "\u1A08"], /* BUGINESE LETTER TA */
  ["\u1A09", "\u1A09"], /* BUGINESE LETTER DA */
  ["\u1A0A", "\u1A0A"], /* BUGINESE LETTER NA */
  ["\u1A0B", "\u1A0B"], /* BUGINESE LETTER NRA */
  ["\u1A0C", "\u1A0C"], /* BUGINESE LETTER CA */
  ["\u1A0D", "\u1A0D"], /* BUGINESE LETTER JA */
  ["\u1A0E", "\u1A0E"], /* BUGINESE LETTER NYA */
  ["\u1A0F", "\u1A0F"], /* BUGINESE LETTER NYCA */
  ["\u1A10", "\u1A10"], /* BUGINESE LETTER YA */
  ["\u1A11", "\u1A11"], /* BUGINESE LETTER RA */
  ["\u1A12", "\u1A12"], /* BUGINESE LETTER LA */
  ["\u1A13", "\u1A13"], /* BUGINESE LETTER VA */
  ["\u1A14", "\u1A14"], /* BUGINESE LETTER SA */
  ["\u1A15", "\u1A15"], /* BUGINESE LETTER A */
  ["\u1A16", "\u1A16"], /* BUGINESE LETTER HA */
  ["\u1A17", "\u1A17"], /* BUGINESE VOWEL SIGN I */
  ["\u1A18", "\u1A18"], /* BUGINESE VOWEL SIGN U */
  ["\u1A19", "\u1A19"], /* BUGINESE VOWEL SIGN E */
  ["\u1A1A", "\u1A1A"], /* BUGINESE VOWEL SIGN O */
  ["\u1A1B", "\u1A1B"], /* BUGINESE VOWEL SIGN AE */
  ["\u1A1C", "\u1A1C"],
  ["\u1A1D", "\u1A1D"],
  ["\u1A1E", "\u1A1E"], /* BUGINESE PALLAWA */
  ["\u1A1F", "\u1A1F"], /* BUGINESE END OF SECTION */
  ["\u1A20", "\u1A20"], /* TAI THAM LETTER HIGH KA */
  ["\u1A21", "\u1A21"], /* TAI THAM LETTER HIGH KHA */
  ["\u1A22", "\u1A22"], /* TAI THAM LETTER HIGH KXA */
  ["\u1A23", "\u1A23"], /* TAI THAM LETTER LOW KA */
  ["\u1A24", "\u1A24"], /* TAI THAM LETTER LOW KXA */
  ["\u1A25", "\u1A25"], /* TAI THAM LETTER LOW KHA */
  ["\u1A26", "\u1A26"], /* TAI THAM LETTER NGA */
  ["\u1A27", "\u1A27"], /* TAI THAM LETTER HIGH CA */
  ["\u1A28", "\u1A28"], /* TAI THAM LETTER HIGH CHA */
  ["\u1A29", "\u1A29"], /* TAI THAM LETTER LOW CA */
  ["\u1A2A", "\u1A2A"], /* TAI THAM LETTER LOW SA */
  ["\u1A2B", "\u1A2B"], /* TAI THAM LETTER LOW CHA */
  ["\u1A2C", "\u1A2C"], /* TAI THAM LETTER NYA */
  ["\u1A2D", "\u1A2D"], /* TAI THAM LETTER RATA */
  ["\u1A2E", "\u1A2E"], /* TAI THAM LETTER HIGH RATHA */
  ["\u1A2F", "\u1A2F"], /* TAI THAM LETTER DA */
  ["\u1A30", "\u1A30"], /* TAI THAM LETTER LOW RATHA */
  ["\u1A31", "\u1A31"], /* TAI THAM LETTER RANA */
  ["\u1A32", "\u1A32"], /* TAI THAM LETTER HIGH TA */
  ["\u1A33", "\u1A33"], /* TAI THAM LETTER HIGH THA */
  ["\u1A34", "\u1A34"], /* TAI THAM LETTER LOW TA */
  ["\u1A35", "\u1A35"], /* TAI THAM LETTER LOW THA */
  ["\u1A36", "\u1A36"], /* TAI THAM LETTER NA */
  ["\u1A37", "\u1A37"], /* TAI THAM LETTER BA */
  ["\u1A38", "\u1A38"], /* TAI THAM LETTER HIGH PA */
  ["\u1A39", "\u1A39"], /* TAI THAM LETTER HIGH PHA */
  ["\u1A3A", "\u1A3A"], /* TAI THAM LETTER HIGH FA */
  ["\u1A3B", "\u1A3B"], /* TAI THAM LETTER LOW PA */
  ["\u1A3C", "\u1A3C"], /* TAI THAM LETTER LOW FA */
  ["\u1A3D", "\u1A3D"], /* TAI THAM LETTER LOW PHA */
  ["\u1A3E", "\u1A3E"], /* TAI THAM LETTER MA */
  ["\u1A3F", "\u1A3F"], /* TAI THAM LETTER LOW YA */
  ["\u1A40", "\u1A40"], /* TAI THAM LETTER HIGH YA */
  ["\u1A41", "\u1A41"], /* TAI THAM LETTER RA */
  ["\u1A42", "\u1A42"], /* TAI THAM LETTER RUE */
  ["\u1A43", "\u1A43"], /* TAI THAM LETTER LA */
  ["\u1A44", "\u1A44"], /* TAI THAM LETTER LUE */
  ["\u1A45", "\u1A45"], /* TAI THAM LETTER WA */
  ["\u1A46", "\u1A46"], /* TAI THAM LETTER HIGH SHA */
  ["\u1A47", "\u1A47"], /* TAI THAM LETTER HIGH SSA */
  ["\u1A48", "\u1A48"], /* TAI THAM LETTER HIGH SA */
  ["\u1A49", "\u1A49"], /* TAI THAM LETTER HIGH HA */
  ["\u1A4A", "\u1A4A"], /* TAI THAM LETTER LLA */
  ["\u1A4B", "\u1A4B"], /* TAI THAM LETTER A */
  ["\u1A4C", "\u1A4C"], /* TAI THAM LETTER LOW HA */
  ["\u1A4D", "\u1A4D"], /* TAI THAM LETTER I */
  ["\u1A4E", "\u1A4E"], /* TAI THAM LETTER II */
  ["\u1A4F", "\u1A4F"], /* TAI THAM LETTER U */
  ["\u1A50", "\u1A50"], /* TAI THAM LETTER UU */
  ["\u1A51", "\u1A51"], /* TAI THAM LETTER EE */
  ["\u1A52", "\u1A52"], /* TAI THAM LETTER OO */
  ["\u1A53", "\u1A53"], /* TAI THAM LETTER LAE */
  ["\u1A54", "\u1A54"], /* TAI THAM LETTER GREAT SA */
  ["\u1A55", "\u1A55"], /* TAI THAM CONSONANT SIGN MEDIAL RA */
  ["\u1A56", "\u1A56"], /* TAI THAM CONSONANT SIGN MEDIAL LA */
  ["\u1A57", "\u1A57"], /* TAI THAM CONSONANT SIGN LA TANG LAI */
  ["\u1A58", "\u1A58"], /* TAI THAM SIGN MAI KANG LAI */
  ["\u1A59", "\u1A59"], /* TAI THAM CONSONANT SIGN FINAL NGA */
  ["\u1A5A", "\u1A5A"], /* TAI THAM CONSONANT SIGN LOW PA */
  ["\u1A5B", "\u1A5B"], /* TAI THAM CONSONANT SIGN HIGH RATHA OR LOW PA */
  ["\u1A5C", "\u1A5C"], /* TAI THAM CONSONANT SIGN MA */
  ["\u1A5D", "\u1A5D"], /* TAI THAM CONSONANT SIGN BA */
  ["\u1A5E", "\u1A5E"], /* TAI THAM CONSONANT SIGN SA */
  ["\u1A5F", "\u1A5F"],
  ["\u1A60", "\u1A60"], /* TAI THAM SIGN SAKOT */
  ["\u1A61", "\u1A61"], /* TAI THAM VOWEL SIGN A */
  ["\u1A62", "\u1A62"], /* TAI THAM VOWEL SIGN MAI SAT */
  ["\u1A63", "\u1A63"], /* TAI THAM VOWEL SIGN AA */
  ["\u1A64", "\u1A64"], /* TAI THAM VOWEL SIGN TALL AA */
  ["\u1A65", "\u1A65"], /* TAI THAM VOWEL SIGN I */
  ["\u1A66", "\u1A66"], /* TAI THAM VOWEL SIGN II */
  ["\u1A67", "\u1A67"], /* TAI THAM VOWEL SIGN UE */
  ["\u1A68", "\u1A68"], /* TAI THAM VOWEL SIGN UUE */
  ["\u1A69", "\u1A69"], /* TAI THAM VOWEL SIGN U */
  ["\u1A6A", "\u1A6A"], /* TAI THAM VOWEL SIGN UU */
  ["\u1A6B", "\u1A6B"], /* TAI THAM VOWEL SIGN O */
  ["\u1A6C", "\u1A6C"], /* TAI THAM VOWEL SIGN OA BELOW */
  ["\u1A6D", "\u1A6D"], /* TAI THAM VOWEL SIGN OY */
  ["\u1A6E", "\u1A6E"], /* TAI THAM VOWEL SIGN E */
  ["\u1A6F", "\u1A6F"], /* TAI THAM VOWEL SIGN AE */
  ["\u1A70", "\u1A70"], /* TAI THAM VOWEL SIGN OO */
  ["\u1A71", "\u1A71"], /* TAI THAM VOWEL SIGN AI */
  ["\u1A72", "\u1A72"], /* TAI THAM VOWEL SIGN THAM AI */
  ["\u1A73", "\u1A73"], /* TAI THAM VOWEL SIGN OA ABOVE */
  ["\u1A74", "\u1A74"], /* TAI THAM SIGN MAI KANG */
  ["\u1A75", "\u1A75"], /* TAI THAM SIGN TONE-1 */
  ["\u1A76", "\u1A76"], /* TAI THAM SIGN TONE-2 */
  ["\u1A77", "\u1A77"], /* TAI THAM SIGN KHUEN TONE-3 */
  ["\u1A78", "\u1A78"], /* TAI THAM SIGN KHUEN TONE-4 */
  ["\u1A79", "\u1A79"], /* TAI THAM SIGN KHUEN TONE-5 */
  ["\u1A7A", "\u1A7A"], /* TAI THAM SIGN RA HAAM */
  ["\u1A7B", "\u1A7B"], /* TAI THAM SIGN MAI SAM */
  ["\u1A7C", "\u1A7C"], /* TAI THAM SIGN KHUEN-LUE KARAN */
  ["\u1A7D", "\u1A7D"],
  ["\u1A7E", "\u1A7E"],
  ["\u1A7F", "\u1A7F"], /* TAI THAM COMBINING CRYPTOGRAMMIC DOT */
  ["\u1A80", "\u1A80"], /* TAI THAM HORA DIGIT ZERO */
  ["\u1A81", "\u1A81"], /* TAI THAM HORA DIGIT ONE */
  ["\u1A82", "\u1A82"], /* TAI THAM HORA DIGIT TWO */
  ["\u1A83", "\u1A83"], /* TAI THAM HORA DIGIT THREE */
  ["\u1A84", "\u1A84"], /* TAI THAM HORA DIGIT FOUR */
  ["\u1A85", "\u1A85"], /* TAI THAM HORA DIGIT FIVE */
  ["\u1A86", "\u1A86"], /* TAI THAM HORA DIGIT SIX */
  ["\u1A87", "\u1A87"], /* TAI THAM HORA DIGIT SEVEN */
  ["\u1A88", "\u1A88"], /* TAI THAM HORA DIGIT EIGHT */
  ["\u1A89", "\u1A89"], /* TAI THAM HORA DIGIT NINE */
  ["\u1A8A", "\u1A8A"],
  ["\u1A8B", "\u1A8B"],
  ["\u1A8C", "\u1A8C"],
  ["\u1A8D", "\u1A8D"],
  ["\u1A8E", "\u1A8E"],
  ["\u1A8F", "\u1A8F"],
  ["\u1A90", "\u1A90"], /* TAI THAM THAM DIGIT ZERO */
  ["\u1A91", "\u1A91"], /* TAI THAM THAM DIGIT ONE */
  ["\u1A92", "\u1A92"], /* TAI THAM THAM DIGIT TWO */
  ["\u1A93", "\u1A93"], /* TAI THAM THAM DIGIT THREE */
  ["\u1A94", "\u1A94"], /* TAI THAM THAM DIGIT FOUR */
  ["\u1A95", "\u1A95"], /* TAI THAM THAM DIGIT FIVE */
  ["\u1A96", "\u1A96"], /* TAI THAM THAM DIGIT SIX */
  ["\u1A97", "\u1A97"], /* TAI THAM THAM DIGIT SEVEN */
  ["\u1A98", "\u1A98"], /* TAI THAM THAM DIGIT EIGHT */
  ["\u1A99", "\u1A99"], /* TAI THAM THAM DIGIT NINE */
  ["\u1A9A", "\u1A9A"],
  ["\u1A9B", "\u1A9B"],
  ["\u1A9C", "\u1A9C"],
  ["\u1A9D", "\u1A9D"],
  ["\u1A9E", "\u1A9E"],
  ["\u1A9F", "\u1A9F"],
  ["\u1AA0", "\u1AA0"], /* TAI THAM SIGN WIANG */
  ["\u1AA1", "\u1AA1"], /* TAI THAM SIGN WIANGWAAK */
  ["\u1AA2", "\u1AA2"], /* TAI THAM SIGN SAWAN */
  ["\u1AA3", "\u1AA3"], /* TAI THAM SIGN KEOW */
  ["\u1AA4", "\u1AA4"], /* TAI THAM SIGN HOY */
  ["\u1AA5", "\u1AA5"], /* TAI THAM SIGN DOKMAI */
  ["\u1AA6", "\u1AA6"], /* TAI THAM SIGN REVERSED ROTATED RANA */
  ["\u1AA7", "\u1AA7"], /* TAI THAM SIGN MAI YAMOK */
  ["\u1AA8", "\u1AA8"], /* TAI THAM SIGN KAAN */
  ["\u1AA9", "\u1AA9"], /* TAI THAM SIGN KAANKUU */
  ["\u1AAA", "\u1AAA"], /* TAI THAM SIGN SATKAAN */
  ["\u1AAB", "\u1AAB"], /* TAI THAM SIGN SATKAANKUU */
  ["\u1AAC", "\u1AAC"], /* TAI THAM SIGN HANG */
  ["\u1AAD", "\u1AAD"], /* TAI THAM SIGN CAANG */
  ["\u1AAE", "\u1AAE"],
  ["\u1AAF", "\u1AAF"],
  ["\u1AB0", "\u1AB0"], /* COMBINING DOUBLED CIRCUMFLEX ACCENT */
  ["\u1AB1", "\u1AB1"], /* COMBINING DIAERESIS-RING */
  ["\u1AB2", "\u1AB2"], /* COMBINING INFINITY */
  ["\u1AB3", "\u1AB3"], /* COMBINING DOWNWARDS ARROW */
  ["\u1AB4", "\u1AB4"], /* COMBINING TRIPLE DOT */
  ["\u1AB5", "\u1AB5"], /* COMBINING X-X BELOW */
  ["\u1AB6", "\u1AB6"], /* COMBINING WIGGLY LINE BELOW */
  ["\u1AB7", "\u1AB7"], /* COMBINING OPEN MARK BELOW */
  ["\u1AB8", "\u1AB8"], /* COMBINING DOUBLE OPEN MARK BELOW */
  ["\u1AB9", "\u1AB9"], /* COMBINING LIGHT CENTRALIZATION STROKE BELOW */
  ["\u1ABA", "\u1ABA"], /* COMBINING STRONG CENTRALIZATION STROKE BELOW */
  ["\u1ABB", "\u1ABB"], /* COMBINING PARENTHESES ABOVE */
  ["\u1ABC", "\u1ABC"], /* COMBINING DOUBLE PARENTHESES ABOVE */
  ["\u1ABD", "\u1ABD"], /* COMBINING PARENTHESES BELOW */
  ["\u1ABE", "\u1ABE"], /* COMBINING PARENTHESES OVERLAY */
  ["\u1ABF", "\u1ABF"],
  ["\u1AC0", "\u1AC0"],
  ["\u1AC1", "\u1AC1"],
  ["\u1AC2", "\u1AC2"],
  ["\u1AC3", "\u1AC3"],
  ["\u1AC4", "\u1AC4"],
  ["\u1AC5", "\u1AC5"],
  ["\u1AC6", "\u1AC6"],
  ["\u1AC7", "\u1AC7"],
  ["\u1AC8", "\u1AC8"],
  ["\u1AC9", "\u1AC9"],
  ["\u1ACA", "\u1ACA"],
  ["\u1ACB", "\u1ACB"],
  ["\u1ACC", "\u1ACC"],
  ["\u1ACD", "\u1ACD"],
  ["\u1ACE", "\u1ACE"],
  ["\u1ACF", "\u1ACF"],
  ["\u1AD0", "\u1AD0"],
  ["\u1AD1", "\u1AD1"],
  ["\u1AD2", "\u1AD2"],
  ["\u1AD3", "\u1AD3"],
  ["\u1AD4", "\u1AD4"],
  ["\u1AD5", "\u1AD5"],
  ["\u1AD6", "\u1AD6"],
  ["\u1AD7", "\u1AD7"],
  ["\u1AD8", "\u1AD8"],
  ["\u1AD9", "\u1AD9"],
  ["\u1ADA", "\u1ADA"],
  ["\u1ADB", "\u1ADB"],
  ["\u1ADC", "\u1ADC"],
  ["\u1ADD", "\u1ADD"],
  ["\u1ADE", "\u1ADE"],
  ["\u1ADF", "\u1ADF"],
  ["\u1AE0", "\u1AE0"],
  ["\u1AE1", "\u1AE1"],
  ["\u1AE2", "\u1AE2"],
  ["\u1AE3", "\u1AE3"],
  ["\u1AE4", "\u1AE4"],
  ["\u1AE5", "\u1AE5"],
  ["\u1AE6", "\u1AE6"],
  ["\u1AE7", "\u1AE7"],
  ["\u1AE8", "\u1AE8"],
  ["\u1AE9", "\u1AE9"],
  ["\u1AEA", "\u1AEA"],
  ["\u1AEB", "\u1AEB"],
  ["\u1AEC", "\u1AEC"],
  ["\u1AED", "\u1AED"],
  ["\u1AEE", "\u1AEE"],
  ["\u1AEF", "\u1AEF"],
  ["\u1AF0", "\u1AF0"],
  ["\u1AF1", "\u1AF1"],
  ["\u1AF2", "\u1AF2"],
  ["\u1AF3", "\u1AF3"],
  ["\u1AF4", "\u1AF4"],
  ["\u1AF5", "\u1AF5"],
  ["\u1AF6", "\u1AF6"],
  ["\u1AF7", "\u1AF7"],
  ["\u1AF8", "\u1AF8"],
  ["\u1AF9", "\u1AF9"],
  ["\u1AFA", "\u1AFA"],
  ["\u1AFB", "\u1AFB"],
  ["\u1AFC", "\u1AFC"],
  ["\u1AFD", "\u1AFD"],
  ["\u1AFE", "\u1AFE"],
  ["\u1AFF", "\u1AFF"],
  ["\u1B00", "\u1B00"], /* BALINESE SIGN ULU RICEM */
  ["\u1B01", "\u1B01"], /* BALINESE SIGN ULU CANDRA */
  ["\u1B02", "\u1B02"], /* BALINESE SIGN CECEK */
  ["\u1B03", "\u1B03"], /* BALINESE SIGN SURANG */
  ["\u1B04", "\u1B04"], /* BALINESE SIGN BISAH */
  ["\u1B05", "\u1B05"], /* BALINESE LETTER AKARA */
  ["\u1B06", "\u1B06"], /* BALINESE LETTER AKARA TEDUNG */
  ["\u1B07", "\u1B07"], /* BALINESE LETTER IKARA */
  ["\u1B08", "\u1B08"], /* BALINESE LETTER IKARA TEDUNG */
  ["\u1B09", "\u1B09"], /* BALINESE LETTER UKARA */
  ["\u1B0A", "\u1B0A"], /* BALINESE LETTER UKARA TEDUNG */
  ["\u1B0B", "\u1B0B"], /* BALINESE LETTER RA REPA */
  ["\u1B0C", "\u1B0C"], /* BALINESE LETTER RA REPA TEDUNG */
  ["\u1B0D", "\u1B0D"], /* BALINESE LETTER LA LENGA */
  ["\u1B0E", "\u1B0E"], /* BALINESE LETTER LA LENGA TEDUNG */
  ["\u1B0F", "\u1B0F"], /* BALINESE LETTER EKARA */
  ["\u1B10", "\u1B10"], /* BALINESE LETTER AIKARA */
  ["\u1B11", "\u1B11"], /* BALINESE LETTER OKARA */
  ["\u1B12", "\u1B12"], /* BALINESE LETTER OKARA TEDUNG */
  ["\u1B13", "\u1B13"], /* BALINESE LETTER KA */
  ["\u1B14", "\u1B14"], /* BALINESE LETTER KA MAHAPRANA */
  ["\u1B15", "\u1B15"], /* BALINESE LETTER GA */
  ["\u1B16", "\u1B16"], /* BALINESE LETTER GA GORA */
  ["\u1B17", "\u1B17"], /* BALINESE LETTER NGA */
  ["\u1B18", "\u1B18"], /* BALINESE LETTER CA */
  ["\u1B19", "\u1B19"], /* BALINESE LETTER CA LACA */
  ["\u1B1A", "\u1B1A"], /* BALINESE LETTER JA */
  ["\u1B1B", "\u1B1B"], /* BALINESE LETTER JA JERA */
  ["\u1B1C", "\u1B1C"], /* BALINESE LETTER NYA */
  ["\u1B1D", "\u1B1D"], /* BALINESE LETTER TA LATIK */
  ["\u1B1E", "\u1B1E"], /* BALINESE LETTER TA MURDA MAHAPRANA */
  ["\u1B1F", "\u1B1F"], /* BALINESE LETTER DA MURDA ALPAPRANA */
  ["\u1B20", "\u1B20"], /* BALINESE LETTER DA MURDA MAHAPRANA */
  ["\u1B21", "\u1B21"], /* BALINESE LETTER NA RAMBAT */
  ["\u1B22", "\u1B22"], /* BALINESE LETTER TA */
  ["\u1B23", "\u1B23"], /* BALINESE LETTER TA TAWA */
  ["\u1B24", "\u1B24"], /* BALINESE LETTER DA */
  ["\u1B25", "\u1B25"], /* BALINESE LETTER DA MADU */
  ["\u1B26", "\u1B26"], /* BALINESE LETTER NA */
  ["\u1B27", "\u1B27"], /* BALINESE LETTER PA */
  ["\u1B28", "\u1B28"], /* BALINESE LETTER PA KAPAL */
  ["\u1B29", "\u1B29"], /* BALINESE LETTER BA */
  ["\u1B2A", "\u1B2A"], /* BALINESE LETTER BA KEMBANG */
  ["\u1B2B", "\u1B2B"], /* BALINESE LETTER MA */
  ["\u1B2C", "\u1B2C"], /* BALINESE LETTER YA */
  ["\u1B2D", "\u1B2D"], /* BALINESE LETTER RA */
  ["\u1B2E", "\u1B2E"], /* BALINESE LETTER LA */
  ["\u1B2F", "\u1B2F"], /* BALINESE LETTER WA */
  ["\u1B30", "\u1B30"], /* BALINESE LETTER SA SAGA */
  ["\u1B31", "\u1B31"], /* BALINESE LETTER SA SAPA */
  ["\u1B32", "\u1B32"], /* BALINESE LETTER SA */
  ["\u1B33", "\u1B33"], /* BALINESE LETTER HA */
  ["\u1B34", "\u1B34"], /* BALINESE SIGN REREKAN */
  ["\u1B35", "\u1B35"], /* BALINESE VOWEL SIGN TEDUNG */
  ["\u1B36", "\u1B36"], /* BALINESE VOWEL SIGN ULU */
  ["\u1B37", "\u1B37"], /* BALINESE VOWEL SIGN ULU SARI */
  ["\u1B38", "\u1B38"], /* BALINESE VOWEL SIGN SUKU */
  ["\u1B39", "\u1B39"], /* BALINESE VOWEL SIGN SUKU ILUT */
  ["\u1B3A", "\u1B3A"], /* BALINESE VOWEL SIGN RA REPA */
  ["\u1B3B", "\u1B3B"], /* BALINESE VOWEL SIGN RA REPA TEDUNG */
  ["\u1B3C", "\u1B3C"], /* BALINESE VOWEL SIGN LA LENGA */
  ["\u1B3D", "\u1B3D"], /* BALINESE VOWEL SIGN LA LENGA TEDUNG */
  ["\u1B3E", "\u1B3E"], /* BALINESE VOWEL SIGN TALING */
  ["\u1B3F", "\u1B3F"], /* BALINESE VOWEL SIGN TALING REPA */
  ["\u1B40", "\u1B40"], /* BALINESE VOWEL SIGN TALING TEDUNG */
  ["\u1B41", "\u1B41"], /* BALINESE VOWEL SIGN TALING REPA TEDUNG */
  ["\u1B42", "\u1B42"], /* BALINESE VOWEL SIGN PEPET */
  ["\u1B43", "\u1B43"], /* BALINESE VOWEL SIGN PEPET TEDUNG */
  ["\u1B44", "\u1B44"], /* BALINESE ADEG ADEG */
  ["\u1B45", "\u1B45"], /* BALINESE LETTER KAF SASAK */
  ["\u1B46", "\u1B46"], /* BALINESE LETTER KHOT SASAK */
  ["\u1B47", "\u1B47"], /* BALINESE LETTER TZIR SASAK */
  ["\u1B48", "\u1B48"], /* BALINESE LETTER EF SASAK */
  ["\u1B49", "\u1B49"], /* BALINESE LETTER VE SASAK */
  ["\u1B4A", "\u1B4A"], /* BALINESE LETTER ZAL SASAK */
  ["\u1B4B", "\u1B4B"], /* BALINESE LETTER ASYURA SASAK */
  ["\u1B4C", "\u1B4C"],
  ["\u1B4D", "\u1B4D"],
  ["\u1B4E", "\u1B4E"],
  ["\u1B4F", "\u1B4F"],
  ["\u1B50", "\u1B50"], /* BALINESE DIGIT ZERO */
  ["\u1B51", "\u1B51"], /* BALINESE DIGIT ONE */
  ["\u1B52", "\u1B52"], /* BALINESE DIGIT TWO */
  ["\u1B53", "\u1B53"], /* BALINESE DIGIT THREE */
  ["\u1B54", "\u1B54"], /* BALINESE DIGIT FOUR */
  ["\u1B55", "\u1B55"], /* BALINESE DIGIT FIVE */
  ["\u1B56", "\u1B56"], /* BALINESE DIGIT SIX */
  ["\u1B57", "\u1B57"], /* BALINESE DIGIT SEVEN */
  ["\u1B58", "\u1B58"], /* BALINESE DIGIT EIGHT */
  ["\u1B59", "\u1B59"], /* BALINESE DIGIT NINE */
  ["\u1B5A", "\u1B5A"], /* BALINESE PANTI */
  ["\u1B5B", "\u1B5B"], /* BALINESE PAMADA */
  ["\u1B5C", "\u1B5C"], /* BALINESE WINDU */
  ["\u1B5D", "\u1B5D"], /* BALINESE CARIK PAMUNGKAH */
  ["\u1B5E", "\u1B5E"], /* BALINESE CARIK SIKI */
  ["\u1B5F", "\u1B5F"], /* BALINESE CARIK PAREREN */
  ["\u1B60", "\u1B60"], /* BALINESE PAMENENG */
  ["\u1B61", "\u1B61"], /* BALINESE MUSICAL SYMBOL DONG */
  ["\u1B62", "\u1B62"], /* BALINESE MUSICAL SYMBOL DENG */
  ["\u1B63", "\u1B63"], /* BALINESE MUSICAL SYMBOL DUNG */
  ["\u1B64", "\u1B64"], /* BALINESE MUSICAL SYMBOL DANG */
  ["\u1B65", "\u1B65"], /* BALINESE MUSICAL SYMBOL DANG SURANG */
  ["\u1B66", "\u1B66"], /* BALINESE MUSICAL SYMBOL DING */
  ["\u1B67", "\u1B67"], /* BALINESE MUSICAL SYMBOL DAENG */
  ["\u1B68", "\u1B68"], /* BALINESE MUSICAL SYMBOL DEUNG */
  ["\u1B69", "\u1B69"], /* BALINESE MUSICAL SYMBOL DAING */
  ["\u1B6A", "\u1B6A"], /* BALINESE MUSICAL SYMBOL DANG GEDE */
  ["\u1B6B", "\u1B6B"], /* BALINESE MUSICAL SYMBOL COMBINING TEGEH */
  ["\u1B6C", "\u1B6C"], /* BALINESE MUSICAL SYMBOL COMBINING ENDEP */
  ["\u1B6D", "\u1B6D"], /* BALINESE MUSICAL SYMBOL COMBINING KEMPUL */
  ["\u1B6E", "\u1B6E"], /* BALINESE MUSICAL SYMBOL COMBINING KEMPLI */
  ["\u1B6F", "\u1B6F"], /* BALINESE MUSICAL SYMBOL COMBINING JEGOGAN */
  ["\u1B70", "\u1B70"], /* BALINESE MUSICAL SYMBOL COMBINING KEMPUL WITH JEGOGAN */
  ["\u1B71", "\u1B71"], /* BALINESE MUSICAL SYMBOL COMBINING KEMPLI WITH JEGOGAN */
  ["\u1B72", "\u1B72"], /* BALINESE MUSICAL SYMBOL COMBINING BENDE */
  ["\u1B73", "\u1B73"], /* BALINESE MUSICAL SYMBOL COMBINING GONG */
  ["\u1B74", "\u1B74"], /* BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DUG */
  ["\u1B75", "\u1B75"], /* BALINESE MUSICAL SYMBOL RIGHT-HAND OPEN DAG */
  ["\u1B76", "\u1B76"], /* BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TUK */
  ["\u1B77", "\u1B77"], /* BALINESE MUSICAL SYMBOL RIGHT-HAND CLOSED TAK */
  ["\u1B78", "\u1B78"], /* BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PANG */
  ["\u1B79", "\u1B79"], /* BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PUNG */
  ["\u1B7A", "\u1B7A"], /* BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLAK */
  ["\u1B7B", "\u1B7B"], /* BALINESE MUSICAL SYMBOL LEFT-HAND CLOSED PLUK */
  ["\u1B7C", "\u1B7C"], /* BALINESE MUSICAL SYMBOL LEFT-HAND OPEN PING */
  ["\u1B7D", "\u1B7D"],
  ["\u1B7E", "\u1B7E"],
  ["\u1B7F", "\u1B7F"],
  ["\u1B80", "\u1B80"], /* SUNDANESE SIGN PANYECEK */
  ["\u1B81", "\u1B81"], /* SUNDANESE SIGN PANGLAYAR */
  ["\u1B82", "\u1B82"], /* SUNDANESE SIGN PANGWISAD */
  ["\u1B83", "\u1B83"], /* SUNDANESE LETTER A */
  ["\u1B84", "\u1B84"], /* SUNDANESE LETTER I */
  ["\u1B85", "\u1B85"], /* SUNDANESE LETTER U */
  ["\u1B86", "\u1B86"], /* SUNDANESE LETTER AE */
  ["\u1B87", "\u1B87"], /* SUNDANESE LETTER O */
  ["\u1B88", "\u1B88"], /* SUNDANESE LETTER E */
  ["\u1B89", "\u1B89"], /* SUNDANESE LETTER EU */
  ["\u1B8A", "\u1B8A"], /* SUNDANESE LETTER KA */
  ["\u1B8B", "\u1B8B"], /* SUNDANESE LETTER QA */
  ["\u1B8C", "\u1B8C"], /* SUNDANESE LETTER GA */
  ["\u1B8D", "\u1B8D"], /* SUNDANESE LETTER NGA */
  ["\u1B8E", "\u1B8E"], /* SUNDANESE LETTER CA */
  ["\u1B8F", "\u1B8F"], /* SUNDANESE LETTER JA */
  ["\u1B90", "\u1B90"], /* SUNDANESE LETTER ZA */
  ["\u1B91", "\u1B91"], /* SUNDANESE LETTER NYA */
  ["\u1B92", "\u1B92"], /* SUNDANESE LETTER TA */
  ["\u1B93", "\u1B93"], /* SUNDANESE LETTER DA */
  ["\u1B94", "\u1B94"], /* SUNDANESE LETTER NA */
  ["\u1B95", "\u1B95"], /* SUNDANESE LETTER PA */
  ["\u1B96", "\u1B96"], /* SUNDANESE LETTER FA */
  ["\u1B97", "\u1B97"], /* SUNDANESE LETTER VA */
  ["\u1B98", "\u1B98"], /* SUNDANESE LETTER BA */
  ["\u1B99", "\u1B99"], /* SUNDANESE LETTER MA */
  ["\u1B9A", "\u1B9A"], /* SUNDANESE LETTER YA */
  ["\u1B9B", "\u1B9B"], /* SUNDANESE LETTER RA */
  ["\u1B9C", "\u1B9C"], /* SUNDANESE LETTER LA */
  ["\u1B9D", "\u1B9D"], /* SUNDANESE LETTER WA */
  ["\u1B9E", "\u1B9E"], /* SUNDANESE LETTER SA */
  ["\u1B9F", "\u1B9F"], /* SUNDANESE LETTER XA */
  ["\u1BA0", "\u1BA0"], /* SUNDANESE LETTER HA */
  ["\u1BA1", "\u1BA1"], /* SUNDANESE CONSONANT SIGN PAMINGKAL */
  ["\u1BA2", "\u1BA2"], /* SUNDANESE CONSONANT SIGN PANYAKRA */
  ["\u1BA3", "\u1BA3"], /* SUNDANESE CONSONANT SIGN PANYIKU */
  ["\u1BA4", "\u1BA4"], /* SUNDANESE VOWEL SIGN PANGHULU */
  ["\u1BA5", "\u1BA5"], /* SUNDANESE VOWEL SIGN PANYUKU */
  ["\u1BA6", "\u1BA6"], /* SUNDANESE VOWEL SIGN PANAELAENG */
  ["\u1BA7", "\u1BA7"], /* SUNDANESE VOWEL SIGN PANOLONG */
  ["\u1BA8", "\u1BA8"], /* SUNDANESE VOWEL SIGN PAMEPET */
  ["\u1BA9", "\u1BA9"], /* SUNDANESE VOWEL SIGN PANEULEUNG */
  ["\u1BAA", "\u1BAA"], /* SUNDANESE SIGN PAMAAEH */
  ["\u1BAB", "\u1BAB"], /* SUNDANESE SIGN VIRAMA */
  ["\u1BAC", "\u1BAC"], /* SUNDANESE CONSONANT SIGN PASANGAN MA */
  ["\u1BAD", "\u1BAD"], /* SUNDANESE CONSONANT SIGN PASANGAN WA */
  ["\u1BAE", "\u1BAE"], /* SUNDANESE LETTER KHA */
  ["\u1BAF", "\u1BAF"], /* SUNDANESE LETTER SYA */
  ["\u1BB0", "\u1BB0"], /* SUNDANESE DIGIT ZERO */
  ["\u1BB1", "\u1BB1"], /* SUNDANESE DIGIT ONE */
  ["\u1BB2", "\u1BB2"], /* SUNDANESE DIGIT TWO */
  ["\u1BB3", "\u1BB3"], /* SUNDANESE DIGIT THREE */
  ["\u1BB4", "\u1BB4"], /* SUNDANESE DIGIT FOUR */
  ["\u1BB5", "\u1BB5"], /* SUNDANESE DIGIT FIVE */
  ["\u1BB6", "\u1BB6"], /* SUNDANESE DIGIT SIX */
  ["\u1BB7", "\u1BB7"], /* SUNDANESE DIGIT SEVEN */
  ["\u1BB8", "\u1BB8"], /* SUNDANESE DIGIT EIGHT */
  ["\u1BB9", "\u1BB9"], /* SUNDANESE DIGIT NINE */
  ["\u1BBA", "\u1BBA"], /* SUNDANESE AVAGRAHA */
  ["\u1BBB", "\u1BBB"], /* SUNDANESE LETTER REU */
  ["\u1BBC", "\u1BBC"], /* SUNDANESE LETTER LEU */
  ["\u1BBD", "\u1BBD"], /* SUNDANESE LETTER BHA */
  ["\u1BBE", "\u1BBE"], /* SUNDANESE LETTER FINAL K */
  ["\u1BBF", "\u1BBF"], /* SUNDANESE LETTER FINAL M */
  ["\u1BC0", "\u1BC0"], /* BATAK LETTER A */
  ["\u1BC1", "\u1BC1"], /* BATAK LETTER SIMALUNGUN A */
  ["\u1BC2", "\u1BC2"], /* BATAK LETTER HA */
  ["\u1BC3", "\u1BC3"], /* BATAK LETTER SIMALUNGUN HA */
  ["\u1BC4", "\u1BC4"], /* BATAK LETTER MANDAILING HA */
  ["\u1BC5", "\u1BC5"], /* BATAK LETTER BA */
  ["\u1BC6", "\u1BC6"], /* BATAK LETTER KARO BA */
  ["\u1BC7", "\u1BC7"], /* BATAK LETTER PA */
  ["\u1BC8", "\u1BC8"], /* BATAK LETTER SIMALUNGUN PA */
  ["\u1BC9", "\u1BC9"], /* BATAK LETTER NA */
  ["\u1BCA", "\u1BCA"], /* BATAK LETTER MANDAILING NA */
  ["\u1BCB", "\u1BCB"], /* BATAK LETTER WA */
  ["\u1BCC", "\u1BCC"], /* BATAK LETTER SIMALUNGUN WA */
  ["\u1BCD", "\u1BCD"], /* BATAK LETTER PAKPAK WA */
  ["\u1BCE", "\u1BCE"], /* BATAK LETTER GA */
  ["\u1BCF", "\u1BCF"], /* BATAK LETTER SIMALUNGUN GA */
  ["\u1BD0", "\u1BD0"], /* BATAK LETTER JA */
  ["\u1BD1", "\u1BD1"], /* BATAK LETTER DA */
  ["\u1BD2", "\u1BD2"], /* BATAK LETTER RA */
  ["\u1BD3", "\u1BD3"], /* BATAK LETTER SIMALUNGUN RA */
  ["\u1BD4", "\u1BD4"], /* BATAK LETTER MA */
  ["\u1BD5", "\u1BD5"], /* BATAK LETTER SIMALUNGUN MA */
  ["\u1BD6", "\u1BD6"], /* BATAK LETTER SOUTHERN TA */
  ["\u1BD7", "\u1BD7"], /* BATAK LETTER NORTHERN TA */
  ["\u1BD8", "\u1BD8"], /* BATAK LETTER SA */
  ["\u1BD9", "\u1BD9"], /* BATAK LETTER SIMALUNGUN SA */
  ["\u1BDA", "\u1BDA"], /* BATAK LETTER MANDAILING SA */
  ["\u1BDB", "\u1BDB"], /* BATAK LETTER YA */
  ["\u1BDC", "\u1BDC"], /* BATAK LETTER SIMALUNGUN YA */
  ["\u1BDD", "\u1BDD"], /* BATAK LETTER NGA */
  ["\u1BDE", "\u1BDE"], /* BATAK LETTER LA */
  ["\u1BDF", "\u1BDF"], /* BATAK LETTER SIMALUNGUN LA */
  ["\u1BE0", "\u1BE0"], /* BATAK LETTER NYA */
  ["\u1BE1", "\u1BE1"], /* BATAK LETTER CA */
  ["\u1BE2", "\u1BE2"], /* BATAK LETTER NDA */
  ["\u1BE3", "\u1BE3"], /* BATAK LETTER MBA */
  ["\u1BE4", "\u1BE4"], /* BATAK LETTER I */
  ["\u1BE5", "\u1BE5"], /* BATAK LETTER U */
  ["\u1BE6", "\u1BE6"], /* BATAK SIGN TOMPI */
  ["\u1BE7", "\u1BE7"], /* BATAK VOWEL SIGN E */
  ["\u1BE8", "\u1BE8"], /* BATAK VOWEL SIGN PAKPAK E */
  ["\u1BE9", "\u1BE9"], /* BATAK VOWEL SIGN EE */
  ["\u1BEA", "\u1BEA"], /* BATAK VOWEL SIGN I */
  ["\u1BEB", "\u1BEB"], /* BATAK VOWEL SIGN KARO I */
  ["\u1BEC", "\u1BEC"], /* BATAK VOWEL SIGN O */
  ["\u1BED", "\u1BED"], /* BATAK VOWEL SIGN KARO O */
  ["\u1BEE", "\u1BEE"], /* BATAK VOWEL SIGN U */
  ["\u1BEF", "\u1BEF"], /* BATAK VOWEL SIGN U FOR SIMALUNGUN SA */
  ["\u1BF0", "\u1BF0"], /* BATAK CONSONANT SIGN NG */
  ["\u1BF1", "\u1BF1"], /* BATAK CONSONANT SIGN H */
  ["\u1BF2", "\u1BF2"], /* BATAK PANGOLAT */
  ["\u1BF3", "\u1BF3"], /* BATAK PANONGONAN */
  ["\u1BF4", "\u1BF4"],
  ["\u1BF5", "\u1BF5"],
  ["\u1BF6", "\u1BF6"],
  ["\u1BF7", "\u1BF7"],
  ["\u1BF8", "\u1BF8"],
  ["\u1BF9", "\u1BF9"],
  ["\u1BFA", "\u1BFA"],
  ["\u1BFB", "\u1BFB"],
  ["\u1BFC", "\u1BFC"], /* BATAK SYMBOL BINDU NA METEK */
  ["\u1BFD", "\u1BFD"], /* BATAK SYMBOL BINDU PINARBORAS */
  ["\u1BFE", "\u1BFE"], /* BATAK SYMBOL BINDU JUDUL */
  ["\u1BFF", "\u1BFF"], /* BATAK SYMBOL BINDU PANGOLAT */
  ["\u1C00", "\u1C00"], /* LEPCHA LETTER KA */
  ["\u1C01", "\u1C01"], /* LEPCHA LETTER KLA */
  ["\u1C02", "\u1C02"], /* LEPCHA LETTER KHA */
  ["\u1C03", "\u1C03"], /* LEPCHA LETTER GA */
  ["\u1C04", "\u1C04"], /* LEPCHA LETTER GLA */
  ["\u1C05", "\u1C05"], /* LEPCHA LETTER NGA */
  ["\u1C06", "\u1C06"], /* LEPCHA LETTER CA */
  ["\u1C07", "\u1C07"], /* LEPCHA LETTER CHA */
  ["\u1C08", "\u1C08"], /* LEPCHA LETTER JA */
  ["\u1C09", "\u1C09"], /* LEPCHA LETTER NYA */
  ["\u1C0A", "\u1C0A"], /* LEPCHA LETTER TA */
  ["\u1C0B", "\u1C0B"], /* LEPCHA LETTER THA */
  ["\u1C0C", "\u1C0C"], /* LEPCHA LETTER DA */
  ["\u1C0D", "\u1C0D"], /* LEPCHA LETTER NA */
  ["\u1C0E", "\u1C0E"], /* LEPCHA LETTER PA */
  ["\u1C0F", "\u1C0F"], /* LEPCHA LETTER PLA */
  ["\u1C10", "\u1C10"], /* LEPCHA LETTER PHA */
  ["\u1C11", "\u1C11"], /* LEPCHA LETTER FA */
  ["\u1C12", "\u1C12"], /* LEPCHA LETTER FLA */
  ["\u1C13", "\u1C13"], /* LEPCHA LETTER BA */
  ["\u1C14", "\u1C14"], /* LEPCHA LETTER BLA */
  ["\u1C15", "\u1C15"], /* LEPCHA LETTER MA */
  ["\u1C16", "\u1C16"], /* LEPCHA LETTER MLA */
  ["\u1C17", "\u1C17"], /* LEPCHA LETTER TSA */
  ["\u1C18", "\u1C18"], /* LEPCHA LETTER TSHA */
  ["\u1C19", "\u1C19"], /* LEPCHA LETTER DZA */
  ["\u1C1A", "\u1C1A"], /* LEPCHA LETTER YA */
  ["\u1C1B", "\u1C1B"], /* LEPCHA LETTER RA */
  ["\u1C1C", "\u1C1C"], /* LEPCHA LETTER LA */
  ["\u1C1D", "\u1C1D"], /* LEPCHA LETTER HA */
  ["\u1C1E", "\u1C1E"], /* LEPCHA LETTER HLA */
  ["\u1C1F", "\u1C1F"], /* LEPCHA LETTER VA */
  ["\u1C20", "\u1C20"], /* LEPCHA LETTER SA */
  ["\u1C21", "\u1C21"], /* LEPCHA LETTER SHA */
  ["\u1C22", "\u1C22"], /* LEPCHA LETTER WA */
  ["\u1C23", "\u1C23"], /* LEPCHA LETTER A */
  ["\u1C24", "\u1C24"], /* LEPCHA SUBJOINED LETTER YA */
  ["\u1C25", "\u1C25"], /* LEPCHA SUBJOINED LETTER RA */
  ["\u1C26", "\u1C26"], /* LEPCHA VOWEL SIGN AA */
  ["\u1C27", "\u1C27"], /* LEPCHA VOWEL SIGN I */
  ["\u1C28", "\u1C28"], /* LEPCHA VOWEL SIGN O */
  ["\u1C29", "\u1C29"], /* LEPCHA VOWEL SIGN OO */
  ["\u1C2A", "\u1C2A"], /* LEPCHA VOWEL SIGN U */
  ["\u1C2B", "\u1C2B"], /* LEPCHA VOWEL SIGN UU */
  ["\u1C2C", "\u1C2C"], /* LEPCHA VOWEL SIGN E */
  ["\u1C2D", "\u1C2D"], /* LEPCHA CONSONANT SIGN K */
  ["\u1C2E", "\u1C2E"], /* LEPCHA CONSONANT SIGN M */
  ["\u1C2F", "\u1C2F"], /* LEPCHA CONSONANT SIGN L */
  ["\u1C30", "\u1C30"], /* LEPCHA CONSONANT SIGN N */
  ["\u1C31", "\u1C31"], /* LEPCHA CONSONANT SIGN P */
  ["\u1C32", "\u1C32"], /* LEPCHA CONSONANT SIGN R */
  ["\u1C33", "\u1C33"], /* LEPCHA CONSONANT SIGN T */
  ["\u1C34", "\u1C34"], /* LEPCHA CONSONANT SIGN NYIN-DO */
  ["\u1C35", "\u1C35"], /* LEPCHA CONSONANT SIGN KANG */
  ["\u1C36", "\u1C36"], /* LEPCHA SIGN RAN */
  ["\u1C37", "\u1C37"], /* LEPCHA SIGN NUKTA */
  ["\u1C38", "\u1C38"],
  ["\u1C39", "\u1C39"],
  ["\u1C3A", "\u1C3A"],
  ["\u1C3B", "\u1C3B"], /* LEPCHA PUNCTUATION TA-ROL */
  ["\u1C3C", "\u1C3C"], /* LEPCHA PUNCTUATION NYET THYOOM TA-ROL */
  ["\u1C3D", "\u1C3D"], /* LEPCHA PUNCTUATION CER-WA */
  ["\u1C3E", "\u1C3E"], /* LEPCHA PUNCTUATION TSHOOK CER-WA */
  ["\u1C3F", "\u1C3F"], /* LEPCHA PUNCTUATION TSHOOK */
  ["\u1C40", "\u1C40"], /* LEPCHA DIGIT ZERO */
  ["\u1C41", "\u1C41"], /* LEPCHA DIGIT ONE */
  ["\u1C42", "\u1C42"], /* LEPCHA DIGIT TWO */
  ["\u1C43", "\u1C43"], /* LEPCHA DIGIT THREE */
  ["\u1C44", "\u1C44"], /* LEPCHA DIGIT FOUR */
  ["\u1C45", "\u1C45"], /* LEPCHA DIGIT FIVE */
  ["\u1C46", "\u1C46"], /* LEPCHA DIGIT SIX */
  ["\u1C47", "\u1C47"], /* LEPCHA DIGIT SEVEN */
  ["\u1C48", "\u1C48"], /* LEPCHA DIGIT EIGHT */
  ["\u1C49", "\u1C49"], /* LEPCHA DIGIT NINE */
  ["\u1C4A", "\u1C4A"],
  ["\u1C4B", "\u1C4B"],
  ["\u1C4C", "\u1C4C"],
  ["\u1C4D", "\u1C4D"], /* LEPCHA LETTER TTA */
  ["\u1C4E", "\u1C4E"], /* LEPCHA LETTER TTHA */
  ["\u1C4F", "\u1C4F"], /* LEPCHA LETTER DDA */
  ["\u1C50", "\u1C50"], /* OL CHIKI DIGIT ZERO */
  ["\u1C51", "\u1C51"], /* OL CHIKI DIGIT ONE */
  ["\u1C52", "\u1C52"], /* OL CHIKI DIGIT TWO */
  ["\u1C53", "\u1C53"], /* OL CHIKI DIGIT THREE */
  ["\u1C54", "\u1C54"], /* OL CHIKI DIGIT FOUR */
  ["\u1C55", "\u1C55"], /* OL CHIKI DIGIT FIVE */
  ["\u1C56", "\u1C56"], /* OL CHIKI DIGIT SIX */
  ["\u1C57", "\u1C57"], /* OL CHIKI DIGIT SEVEN */
  ["\u1C58", "\u1C58"], /* OL CHIKI DIGIT EIGHT */
  ["\u1C59", "\u1C59"], /* OL CHIKI DIGIT NINE */
  ["\u1C5A", "\u1C5A"], /* OL CHIKI LETTER LA */
  ["\u1C5B", "\u1C5B"], /* OL CHIKI LETTER AT */
  ["\u1C5C", "\u1C5C"], /* OL CHIKI LETTER AG */
  ["\u1C5D", "\u1C5D"], /* OL CHIKI LETTER ANG */
  ["\u1C5E", "\u1C5E"], /* OL CHIKI LETTER AL */
  ["\u1C5F", "\u1C5F"], /* OL CHIKI LETTER LAA */
  ["\u1C60", "\u1C60"], /* OL CHIKI LETTER AAK */
  ["\u1C61", "\u1C61"], /* OL CHIKI LETTER AAJ */
  ["\u1C62", "\u1C62"], /* OL CHIKI LETTER AAM */
  ["\u1C63", "\u1C63"], /* OL CHIKI LETTER AAW */
  ["\u1C64", "\u1C64"], /* OL CHIKI LETTER LI */
  ["\u1C65", "\u1C65"], /* OL CHIKI LETTER IS */
  ["\u1C66", "\u1C66"], /* OL CHIKI LETTER IH */
  ["\u1C67", "\u1C67"], /* OL CHIKI LETTER INY */
  ["\u1C68", "\u1C68"], /* OL CHIKI LETTER IR */
  ["\u1C69", "\u1C69"], /* OL CHIKI LETTER LU */
  ["\u1C6A", "\u1C6A"], /* OL CHIKI LETTER UC */
  ["\u1C6B", "\u1C6B"], /* OL CHIKI LETTER UD */
  ["\u1C6C", "\u1C6C"], /* OL CHIKI LETTER UNN */
  ["\u1C6D", "\u1C6D"], /* OL CHIKI LETTER UY */
  ["\u1C6E", "\u1C6E"], /* OL CHIKI LETTER LE */
  ["\u1C6F", "\u1C6F"], /* OL CHIKI LETTER EP */
  ["\u1C70", "\u1C70"], /* OL CHIKI LETTER EDD */
  ["\u1C71", "\u1C71"], /* OL CHIKI LETTER EN */
  ["\u1C72", "\u1C72"], /* OL CHIKI LETTER ERR */
  ["\u1C73", "\u1C73"], /* OL CHIKI LETTER LO */
  ["\u1C74", "\u1C74"], /* OL CHIKI LETTER OTT */
  ["\u1C75", "\u1C75"], /* OL CHIKI LETTER OB */
  ["\u1C76", "\u1C76"], /* OL CHIKI LETTER OV */
  ["\u1C77", "\u1C77"], /* OL CHIKI LETTER OH */
  ["\u1C78", "\u1C78"], /* OL CHIKI MU TTUDDAG */
  ["\u1C79", "\u1C79"], /* OL CHIKI GAAHLAA TTUDDAAG */
  ["\u1C7A", "\u1C7A"], /* OL CHIKI MU-GAAHLAA TTUDDAAG */
  ["\u1C7B", "\u1C7B"], /* OL CHIKI RELAA */
  ["\u1C7C", "\u1C7C"], /* OL CHIKI PHAARKAA */
  ["\u1C7D", "\u1C7D"], /* OL CHIKI AHAD */
  ["\u1C7E", "\u1C7E"], /* OL CHIKI PUNCTUATION MUCAAD */
  ["\u1C7F", "\u1C7F"], /* OL CHIKI PUNCTUATION DOUBLE MUCAAD */
  ["\u0412", "\u1C80"], /* CYRILLIC SMALL LETTER ROUNDED VE */
  ["\u0414", "\u1C81"], /* CYRILLIC SMALL LETTER LONG-LEGGED DE */
  ["\u041E", "\u1C82"], /* CYRILLIC SMALL LETTER NARROW O */
  ["\u0421", "\u1C83"], /* CYRILLIC SMALL LETTER WIDE ES */
  ["\u0422", "\u1C84"], /* CYRILLIC SMALL LETTER TALL TE */
  ["\u0422", "\u1C85"], /* CYRILLIC SMALL LETTER THREE-LEGGED TE */
  ["\u042A", "\u1C86"], /* CYRILLIC SMALL LETTER TALL HARD SIGN */
  ["\u0462", "\u1C87"], /* CYRILLIC SMALL LETTER TALL YAT */
  ["\uA64A", "\u1C88"], /* CYRILLIC SMALL LETTER UNBLENDED UK */
  ["\u1C89", "\u1C89"],
  ["\u1C8A", "\u1C8A"],
  ["\u1C8B", "\u1C8B"],
  ["\u1C8C", "\u1C8C"],
  ["\u1C8D", "\u1C8D"],
  ["\u1C8E", "\u1C8E"],
  ["\u1C8F", "\u1C8F"],
  ["\u1C90", "\u10D0"], /* GEORGIAN MTAVRULI CAPITAL LETTER AN */
  ["\u1C91", "\u10D1"], /* GEORGIAN MTAVRULI CAPITAL LETTER BAN */
  ["\u1C92", "\u10D2"], /* GEORGIAN MTAVRULI CAPITAL LETTER GAN */
  ["\u1C93", "\u10D3"], /* GEORGIAN MTAVRULI CAPITAL LETTER DON */
  ["\u1C94", "\u10D4"], /* GEORGIAN MTAVRULI CAPITAL LETTER EN */
  ["\u1C95", "\u10D5"], /* GEORGIAN MTAVRULI CAPITAL LETTER VIN */
  ["\u1C96", "\u10D6"], /* GEORGIAN MTAVRULI CAPITAL LETTER ZEN */
  ["\u1C97", "\u10D7"], /* GEORGIAN MTAVRULI CAPITAL LETTER TAN */
  ["\u1C98", "\u10D8"], /* GEORGIAN MTAVRULI CAPITAL LETTER IN */
  ["\u1C99", "\u10D9"], /* GEORGIAN MTAVRULI CAPITAL LETTER KAN */
  ["\u1C9A", "\u10DA"], /* GEORGIAN MTAVRULI CAPITAL LETTER LAS */
  ["\u1C9B", "\u10DB"], /* GEORGIAN MTAVRULI CAPITAL LETTER MAN */
  ["\u1C9C", "\u10DC"], /* GEORGIAN MTAVRULI CAPITAL LETTER NAR */
  ["\u1C9D", "\u10DD"], /* GEORGIAN MTAVRULI CAPITAL LETTER ON */
  ["\u1C9E", "\u10DE"], /* GEORGIAN MTAVRULI CAPITAL LETTER PAR */
  ["\u1C9F", "\u10DF"], /* GEORGIAN MTAVRULI CAPITAL LETTER ZHAR */
  ["\u1CA0", "\u10E0"], /* GEORGIAN MTAVRULI CAPITAL LETTER RAE */
  ["\u1CA1", "\u10E1"], /* GEORGIAN MTAVRULI CAPITAL LETTER SAN */
  ["\u1CA2", "\u10E2"], /* GEORGIAN MTAVRULI CAPITAL LETTER TAR */
  ["\u1CA3", "\u10E3"], /* GEORGIAN MTAVRULI CAPITAL LETTER UN */
  ["\u1CA4", "\u10E4"], /* GEORGIAN MTAVRULI CAPITAL LETTER PHAR */
  ["\u1CA5", "\u10E5"], /* GEORGIAN MTAVRULI CAPITAL LETTER KHAR */
  ["\u1CA6", "\u10E6"], /* GEORGIAN MTAVRULI CAPITAL LETTER GHAN */
  ["\u1CA7", "\u10E7"], /* GEORGIAN MTAVRULI CAPITAL LETTER QAR */
  ["\u1CA8", "\u10E8"], /* GEORGIAN MTAVRULI CAPITAL LETTER SHIN */
  ["\u1CA9", "\u10E9"], /* GEORGIAN MTAVRULI CAPITAL LETTER CHIN */
  ["\u1CAA", "\u10EA"], /* GEORGIAN MTAVRULI CAPITAL LETTER CAN */
  ["\u1CAB", "\u10EB"], /* GEORGIAN MTAVRULI CAPITAL LETTER JIL */
  ["\u1CAC", "\u10EC"], /* GEORGIAN MTAVRULI CAPITAL LETTER CIL */
  ["\u1CAD", "\u10ED"], /* GEORGIAN MTAVRULI CAPITAL LETTER CHAR */
  ["\u1CAE", "\u10EE"], /* GEORGIAN MTAVRULI CAPITAL LETTER XAN */
  ["\u1CAF", "\u10EF"], /* GEORGIAN MTAVRULI CAPITAL LETTER JHAN */
  ["\u1CB0", "\u10F0"], /* GEORGIAN MTAVRULI CAPITAL LETTER HAE */
  ["\u1CB1", "\u10F1"], /* GEORGIAN MTAVRULI CAPITAL LETTER HE */
  ["\u1CB2", "\u10F2"], /* GEORGIAN MTAVRULI CAPITAL LETTER HIE */
  ["\u1CB3", "\u10F3"], /* GEORGIAN MTAVRULI CAPITAL LETTER WE */
  ["\u1CB4", "\u10F4"], /* GEORGIAN MTAVRULI CAPITAL LETTER HAR */
  ["\u1CB5", "\u10F5"], /* GEORGIAN MTAVRULI CAPITAL LETTER HOE */
  ["\u1CB6", "\u10F6"], /* GEORGIAN MTAVRULI CAPITAL LETTER FI */
  ["\u1CB7", "\u10F7"], /* GEORGIAN MTAVRULI CAPITAL LETTER YN */
  ["\u1CB8", "\u10F8"], /* GEORGIAN MTAVRULI CAPITAL LETTER ELIFI */
  ["\u1CB9", "\u10F9"], /* GEORGIAN MTAVRULI CAPITAL LETTER TURNED GAN */
  ["\u1CBA", "\u10FA"], /* GEORGIAN MTAVRULI CAPITAL LETTER AIN */
  ["\u1CBB", "\u1CBB"],
  ["\u1CBC", "\u1CBC"],
  ["\u1CBD", "\u10FD"], /* GEORGIAN MTAVRULI CAPITAL LETTER AEN */
  ["\u1CBE", "\u10FE"], /* GEORGIAN MTAVRULI CAPITAL LETTER HARD SIGN */
  ["\u1CBF", "\u10FF"], /* GEORGIAN MTAVRULI CAPITAL LETTER LABIAL SIGN */
  ["\u1CC0", "\u1CC0"], /* SUNDANESE PUNCTUATION BINDU SURYA */
  ["\u1CC1", "\u1CC1"], /* SUNDANESE PUNCTUATION BINDU PANGLONG */
  ["\u1CC2", "\u1CC2"], /* SUNDANESE PUNCTUATION BINDU PURNAMA */
  ["\u1CC3", "\u1CC3"], /* SUNDANESE PUNCTUATION BINDU CAKRA */
  ["\u1CC4", "\u1CC4"], /* SUNDANESE PUNCTUATION BINDU LEU SATANGA */
  ["\u1CC5", "\u1CC5"], /* SUNDANESE PUNCTUATION BINDU KA SATANGA */
  ["\u1CC6", "\u1CC6"], /* SUNDANESE PUNCTUATION BINDU DA SATANGA */
  ["\u1CC7", "\u1CC7"], /* SUNDANESE PUNCTUATION BINDU BA SATANGA */
  ["\u1CC8", "\u1CC8"],
  ["\u1CC9", "\u1CC9"],
  ["\u1CCA", "\u1CCA"],
  ["\u1CCB", "\u1CCB"],
  ["\u1CCC", "\u1CCC"],
  ["\u1CCD", "\u1CCD"],
  ["\u1CCE", "\u1CCE"],
  ["\u1CCF", "\u1CCF"],
  ["\u1CD0", "\u1CD0"], /* VEDIC TONE KARSHANA */
  ["\u1CD1", "\u1CD1"], /* VEDIC TONE SHARA */
  ["\u1CD2", "\u1CD2"], /* VEDIC TONE PRENKHA */
  ["\u1CD3", "\u1CD3"], /* VEDIC SIGN NIHSHVASA */
  ["\u1CD4", "\u1CD4"], /* VEDIC SIGN YAJURVEDIC MIDLINE SVARITA */
  ["\u1CD5", "\u1CD5"], /* VEDIC TONE YAJURVEDIC AGGRAVATED INDEPENDENT SVARITA */
  ["\u1CD6", "\u1CD6"], /* VEDIC TONE YAJURVEDIC INDEPENDENT SVARITA */
  ["\u1CD7", "\u1CD7"], /* VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA */
  ["\u1CD8", "\u1CD8"], /* VEDIC TONE CANDRA BELOW */
  ["\u1CD9", "\u1CD9"], /* VEDIC TONE YAJURVEDIC KATHAKA INDEPENDENT SVARITA SCHROEDER */
  ["\u1CDA", "\u1CDA"], /* VEDIC TONE DOUBLE SVARITA */
  ["\u1CDB", "\u1CDB"], /* VEDIC TONE TRIPLE SVARITA */
  ["\u1CDC", "\u1CDC"], /* VEDIC TONE KATHAKA ANUDATTA */
  ["\u1CDD", "\u1CDD"], /* VEDIC TONE DOT BELOW */
  ["\u1CDE", "\u1CDE"], /* VEDIC TONE TWO DOTS BELOW */
  ["\u1CDF", "\u1CDF"], /* VEDIC TONE THREE DOTS BELOW */
  ["\u1CE0", "\u1CE0"], /* VEDIC TONE RIGVEDIC KASHMIRI INDEPENDENT SVARITA */
  ["\u1CE1", "\u1CE1"], /* VEDIC TONE ATHARVAVEDIC INDEPENDENT SVARITA */
  ["\u1CE2", "\u1CE2"], /* VEDIC SIGN VISARGA SVARITA */
  ["\u1CE3", "\u1CE3"], /* VEDIC SIGN VISARGA UDATTA */
  ["\u1CE4", "\u1CE4"], /* VEDIC SIGN REVERSED VISARGA UDATTA */
  ["\u1CE5", "\u1CE5"], /* VEDIC SIGN VISARGA ANUDATTA */
  ["\u1CE6", "\u1CE6"], /* VEDIC SIGN REVERSED VISARGA ANUDATTA */
  ["\u1CE7", "\u1CE7"], /* VEDIC SIGN VISARGA UDATTA WITH TAIL */
  ["\u1CE8", "\u1CE8"], /* VEDIC SIGN VISARGA ANUDATTA WITH TAIL */
  ["\u1CE9", "\u1CE9"], /* VEDIC SIGN ANUSVARA ANTARGOMUKHA */
  ["\u1CEA", "\u1CEA"], /* VEDIC SIGN ANUSVARA BAHIRGOMUKHA */
  ["\u1CEB", "\u1CEB"], /* VEDIC SIGN ANUSVARA VAMAGOMUKHA */
  ["\u1CEC", "\u1CEC"], /* VEDIC SIGN ANUSVARA VAMAGOMUKHA WITH TAIL */
  ["\u1CED", "\u1CED"], /* VEDIC SIGN TIRYAK */
  ["\u1CEE", "\u1CEE"], /* VEDIC SIGN HEXIFORM LONG ANUSVARA */
  ["\u1CEF", "\u1CEF"], /* VEDIC SIGN LONG ANUSVARA */
  ["\u1CF0", "\u1CF0"], /* VEDIC SIGN RTHANG LONG ANUSVARA */
  ["\u1CF1", "\u1CF1"], /* VEDIC SIGN ANUSVARA UBHAYATO MUKHA */
  ["\u1CF2", "\u1CF2"], /* VEDIC SIGN ARDHAVISARGA */
  ["\u1CF3", "\u1CF3"], /* VEDIC SIGN ROTATED ARDHAVISARGA */
  ["\u1CF4", "\u1CF4"], /* VEDIC TONE CANDRA ABOVE */
  ["\u1CF5", "\u1CF5"], /* VEDIC SIGN JIHVAMULIYA */
  ["\u1CF6", "\u1CF6"], /* VEDIC SIGN UPADHMANIYA */
  ["\u1CF7", "\u1CF7"], /* VEDIC SIGN ATIKRAMA */
  ["\u1CF8", "\u1CF8"], /* VEDIC TONE RING ABOVE */
  ["\u1CF9", "\u1CF9"], /* VEDIC TONE DOUBLE RING ABOVE */
  ["\u1CFA", "\u1CFA"], /* VEDIC SIGN DOUBLE ANUSVARA ANTARGOMUKHA */
  ["\u1CFB", "\u1CFB"],
  ["\u1CFC", "\u1CFC"],
  ["\u1CFD", "\u1CFD"],
  ["\u1CFE", "\u1CFE"],
  ["\u1CFF", "\u1CFF"],
  ["\u1D00", "\u1D00"], /* LATIN LETTER SMALL CAPITAL A */
  ["\u1D01", "\u1D01"], /* LATIN LETTER SMALL CAPITAL AE */
  ["\u1D02", "\u1D02"], /* LATIN SMALL LETTER TURNED AE */
  ["\u1D03", "\u1D03"], /* LATIN LETTER SMALL CAPITAL BARRED B */
  ["\u1D04", "\u1D04"], /* LATIN LETTER SMALL CAPITAL C */
  ["\u1D05", "\u1D05"], /* LATIN LETTER SMALL CAPITAL D */
  ["\u1D06", "\u1D06"], /* LATIN LETTER SMALL CAPITAL ETH */
  ["\u1D07", "\u1D07"], /* LATIN LETTER SMALL CAPITAL E */
  ["\u1D08", "\u1D08"], /* LATIN SMALL LETTER TURNED OPEN E */
  ["\u1D09", "\u1D09"], /* LATIN SMALL LETTER TURNED I */
  ["\u1D0A", "\u1D0A"], /* LATIN LETTER SMALL CAPITAL J */
  ["\u1D0B", "\u1D0B"], /* LATIN LETTER SMALL CAPITAL K */
  ["\u1D0C", "\u1D0C"], /* LATIN LETTER SMALL CAPITAL L WITH STROKE */
  ["\u1D0D", "\u1D0D"], /* LATIN LETTER SMALL CAPITAL M */
  ["\u1D0E", "\u1D0E"], /* LATIN LETTER SMALL CAPITAL REVERSED N */
  ["\u1D0F", "\u1D0F"], /* LATIN LETTER SMALL CAPITAL O */
  ["\u1D10", "\u1D10"], /* LATIN LETTER SMALL CAPITAL OPEN O */
  ["\u1D11", "\u1D11"], /* LATIN SMALL LETTER SIDEWAYS O */
  ["\u1D12", "\u1D12"], /* LATIN SMALL LETTER SIDEWAYS OPEN O */
  ["\u1D13", "\u1D13"], /* LATIN SMALL LETTER SIDEWAYS O WITH STROKE */
  ["\u1D14", "\u1D14"], /* LATIN SMALL LETTER TURNED OE */
  ["\u1D15", "\u1D15"], /* LATIN LETTER SMALL CAPITAL OU */
  ["\u1D16", "\u1D16"], /* LATIN SMALL LETTER TOP HALF O */
  ["\u1D17", "\u1D17"], /* LATIN SMALL LETTER BOTTOM HALF O */
  ["\u1D18", "\u1D18"], /* LATIN LETTER SMALL CAPITAL P */
  ["\u1D19", "\u1D19"], /* LATIN LETTER SMALL CAPITAL REVERSED R */
  ["\u1D1A", "\u1D1A"], /* LATIN LETTER SMALL CAPITAL TURNED R */
  ["\u1D1B", "\u1D1B"], /* LATIN LETTER SMALL CAPITAL T */
  ["\u1D1C", "\u1D1C"], /* LATIN LETTER SMALL CAPITAL U */
  ["\u1D1D", "\u1D1D"], /* LATIN SMALL LETTER SIDEWAYS U */
  ["\u1D1E", "\u1D1E"], /* LATIN SMALL LETTER SIDEWAYS DIAERESIZED U */
  ["\u1D1F", "\u1D1F"], /* LATIN SMALL LETTER SIDEWAYS TURNED M */
  ["\u1D20", "\u1D20"], /* LATIN LETTER SMALL CAPITAL V */
  ["\u1D21", "\u1D21"], /* LATIN LETTER SMALL CAPITAL W */
  ["\u1D22", "\u1D22"], /* LATIN LETTER SMALL CAPITAL Z */
  ["\u1D23", "\u1D23"], /* LATIN LETTER SMALL CAPITAL EZH */
  ["\u1D24", "\u1D24"], /* LATIN LETTER VOICED LARYNGEAL SPIRANT */
  ["\u1D25", "\u1D25"], /* LATIN LETTER AIN */
  ["\u1D26", "\u1D26"], /* GREEK LETTER SMALL CAPITAL GAMMA */
  ["\u1D27", "\u1D27"], /* GREEK LETTER SMALL CAPITAL LAMDA */
  ["\u1D28", "\u1D28"], /* GREEK LETTER SMALL CAPITAL PI */
  ["\u1D29", "\u1D29"], /* GREEK LETTER SMALL CAPITAL RHO */
  ["\u1D2A", "\u1D2A"], /* GREEK LETTER SMALL CAPITAL PSI */
  ["\u1D2B", "\u1D2B"], /* CYRILLIC LETTER SMALL CAPITAL EL */
  ["\u1D2C", "\u1D2C"], /* MODIFIER LETTER CAPITAL A */
  ["\u1D2D", "\u1D2D"], /* MODIFIER LETTER CAPITAL AE */
  ["\u1D2E", "\u1D2E"], /* MODIFIER LETTER CAPITAL B */
  ["\u1D2F", "\u1D2F"], /* MODIFIER LETTER CAPITAL BARRED B */
  ["\u1D30", "\u1D30"], /* MODIFIER LETTER CAPITAL D */
  ["\u1D31", "\u1D31"], /* MODIFIER LETTER CAPITAL E */
  ["\u1D32", "\u1D32"], /* MODIFIER LETTER CAPITAL REVERSED E */
  ["\u1D33", "\u1D33"], /* MODIFIER LETTER CAPITAL G */
  ["\u1D34", "\u1D34"], /* MODIFIER LETTER CAPITAL H */
  ["\u1D35", "\u1D35"], /* MODIFIER LETTER CAPITAL I */
  ["\u1D36", "\u1D36"], /* MODIFIER LETTER CAPITAL J */
  ["\u1D37", "\u1D37"], /* MODIFIER LETTER CAPITAL K */
  ["\u1D38", "\u1D38"], /* MODIFIER LETTER CAPITAL L */
  ["\u1D39", "\u1D39"], /* MODIFIER LETTER CAPITAL M */
  ["\u1D3A", "\u1D3A"], /* MODIFIER LETTER CAPITAL N */
  ["\u1D3B", "\u1D3B"], /* MODIFIER LETTER CAPITAL REVERSED N */
  ["\u1D3C", "\u1D3C"], /* MODIFIER LETTER CAPITAL O */
  ["\u1D3D", "\u1D3D"], /* MODIFIER LETTER CAPITAL OU */
  ["\u1D3E", "\u1D3E"], /* MODIFIER LETTER CAPITAL P */
  ["\u1D3F", "\u1D3F"], /* MODIFIER LETTER CAPITAL R */
  ["\u1D40", "\u1D40"], /* MODIFIER LETTER CAPITAL T */
  ["\u1D41", "\u1D41"], /* MODIFIER LETTER CAPITAL U */
  ["\u1D42", "\u1D42"], /* MODIFIER LETTER CAPITAL W */
  ["\u1D43", "\u1D43"], /* MODIFIER LETTER SMALL A */
  ["\u1D44", "\u1D44"], /* MODIFIER LETTER SMALL TURNED A */
  ["\u1D45", "\u1D45"], /* MODIFIER LETTER SMALL ALPHA */
  ["\u1D46", "\u1D46"], /* MODIFIER LETTER SMALL TURNED AE */
  ["\u1D47", "\u1D47"], /* MODIFIER LETTER SMALL B */
  ["\u1D48", "\u1D48"], /* MODIFIER LETTER SMALL D */
  ["\u1D49", "\u1D49"], /* MODIFIER LETTER SMALL E */
  ["\u1D4A", "\u1D4A"], /* MODIFIER LETTER SMALL SCHWA */
  ["\u1D4B", "\u1D4B"], /* MODIFIER LETTER SMALL OPEN E */
  ["\u1D4C", "\u1D4C"], /* MODIFIER LETTER SMALL TURNED OPEN E */
  ["\u1D4D", "\u1D4D"], /* MODIFIER LETTER SMALL G */
  ["\u1D4E", "\u1D4E"], /* MODIFIER LETTER SMALL TURNED I */
  ["\u1D4F", "\u1D4F"], /* MODIFIER LETTER SMALL K */
  ["\u1D50", "\u1D50"], /* MODIFIER LETTER SMALL M */
  ["\u1D51", "\u1D51"], /* MODIFIER LETTER SMALL ENG */
  ["\u1D52", "\u1D52"], /* MODIFIER LETTER SMALL O */
  ["\u1D53", "\u1D53"], /* MODIFIER LETTER SMALL OPEN O */
  ["\u1D54", "\u1D54"], /* MODIFIER LETTER SMALL TOP HALF O */
  ["\u1D55", "\u1D55"], /* MODIFIER LETTER SMALL BOTTOM HALF O */
  ["\u1D56", "\u1D56"], /* MODIFIER LETTER SMALL P */
  ["\u1D57", "\u1D57"], /* MODIFIER LETTER SMALL T */
  ["\u1D58", "\u1D58"], /* MODIFIER LETTER SMALL U */
  ["\u1D59", "\u1D59"], /* MODIFIER LETTER SMALL SIDEWAYS U */
  ["\u1D5A", "\u1D5A"], /* MODIFIER LETTER SMALL TURNED M */
  ["\u1D5B", "\u1D5B"], /* MODIFIER LETTER SMALL V */
  ["\u1D5C", "\u1D5C"], /* MODIFIER LETTER SMALL AIN */
  ["\u1D5D", "\u1D5D"], /* MODIFIER LETTER SMALL BETA */
  ["\u1D5E", "\u1D5E"], /* MODIFIER LETTER SMALL GREEK GAMMA */
  ["\u1D5F", "\u1D5F"], /* MODIFIER LETTER SMALL DELTA */
  ["\u1D60", "\u1D60"], /* MODIFIER LETTER SMALL GREEK PHI */
  ["\u1D61", "\u1D61"], /* MODIFIER LETTER SMALL CHI */
  ["\u1D62", "\u1D62"], /* LATIN SUBSCRIPT SMALL LETTER I */
  ["\u1D63", "\u1D63"], /* LATIN SUBSCRIPT SMALL LETTER R */
  ["\u1D64", "\u1D64"], /* LATIN SUBSCRIPT SMALL LETTER U */
  ["\u1D65", "\u1D65"], /* LATIN SUBSCRIPT SMALL LETTER V */
  ["\u1D66", "\u1D66"], /* GREEK SUBSCRIPT SMALL LETTER BETA */
  ["\u1D67", "\u1D67"], /* GREEK SUBSCRIPT SMALL LETTER GAMMA */
  ["\u1D68", "\u1D68"], /* GREEK SUBSCRIPT SMALL LETTER RHO */
  ["\u1D69", "\u1D69"], /* GREEK SUBSCRIPT SMALL LETTER PHI */
  ["\u1D6A", "\u1D6A"], /* GREEK SUBSCRIPT SMALL LETTER CHI */
  ["\u1D6B", "\u1D6B"], /* LATIN SMALL LETTER UE */
  ["\u1D6C", "\u1D6C"], /* LATIN SMALL LETTER B WITH MIDDLE TILDE */
  ["\u1D6D", "\u1D6D"], /* LATIN SMALL LETTER D WITH MIDDLE TILDE */
  ["\u1D6E", "\u1D6E"], /* LATIN SMALL LETTER F WITH MIDDLE TILDE */
  ["\u1D6F", "\u1D6F"], /* LATIN SMALL LETTER M WITH MIDDLE TILDE */
  ["\u1D70", "\u1D70"], /* LATIN SMALL LETTER N WITH MIDDLE TILDE */
  ["\u1D71", "\u1D71"], /* LATIN SMALL LETTER P WITH MIDDLE TILDE */
  ["\u1D72", "\u1D72"], /* LATIN SMALL LETTER R WITH MIDDLE TILDE */
  ["\u1D73", "\u1D73"], /* LATIN SMALL LETTER R WITH FISHHOOK AND MIDDLE TILDE */
  ["\u1D74", "\u1D74"], /* LATIN SMALL LETTER S WITH MIDDLE TILDE */
  ["\u1D75", "\u1D75"], /* LATIN SMALL LETTER T WITH MIDDLE TILDE */
  ["\u1D76", "\u1D76"], /* LATIN SMALL LETTER Z WITH MIDDLE TILDE */
  ["\u1D77", "\u1D77"], /* LATIN SMALL LETTER TURNED G */
  ["\u1D78", "\u1D78"], /* MODIFIER LETTER CYRILLIC EN */
  ["\uA77D", "\u1D79"], /* LATIN SMALL LETTER INSULAR G */
  ["\u1D7A", "\u1D7A"], /* LATIN SMALL LETTER TH WITH STRIKETHROUGH */
  ["\u1D7B", "\u1D7B"], /* LATIN SMALL CAPITAL LETTER I WITH STROKE */
  ["\u1D7C", "\u1D7C"], /* LATIN SMALL LETTER IOTA WITH STROKE */
  ["\u2C63", "\u1D7D"], /* LATIN SMALL LETTER P WITH STROKE */
  ["\u1D7E", "\u1D7E"], /* LATIN SMALL CAPITAL LETTER U WITH STROKE */
  ["\u1D7F", "\u1D7F"], /* LATIN SMALL LETTER UPSILON WITH STROKE */
  ["\u1D80", "\u1D80"], /* LATIN SMALL LETTER B WITH PALATAL HOOK */
  ["\u1D81", "\u1D81"], /* LATIN SMALL LETTER D WITH PALATAL HOOK */
  ["\u1D82", "\u1D82"], /* LATIN SMALL LETTER F WITH PALATAL HOOK */
  ["\u1D83", "\u1D83"], /* LATIN SMALL LETTER G WITH PALATAL HOOK */
  ["\u1D84", "\u1D84"], /* LATIN SMALL LETTER K WITH PALATAL HOOK */
  ["\u1D85", "\u1D85"], /* LATIN SMALL LETTER L WITH PALATAL HOOK */
  ["\u1D86", "\u1D86"], /* LATIN SMALL LETTER M WITH PALATAL HOOK */
  ["\u1D87", "\u1D87"], /* LATIN SMALL LETTER N WITH PALATAL HOOK */
  ["\u1D88", "\u1D88"], /* LATIN SMALL LETTER P WITH PALATAL HOOK */
  ["\u1D89", "\u1D89"], /* LATIN SMALL LETTER R WITH PALATAL HOOK */
  ["\u1D8A", "\u1D8A"], /* LATIN SMALL LETTER S WITH PALATAL HOOK */
  ["\u1D8B", "\u1D8B"], /* LATIN SMALL LETTER ESH WITH PALATAL HOOK */
  ["\u1D8C", "\u1D8C"], /* LATIN SMALL LETTER V WITH PALATAL HOOK */
  ["\u1D8D", "\u1D8D"], /* LATIN SMALL LETTER X WITH PALATAL HOOK */
  ["\uA7C6", "\u1D8E"], /* LATIN SMALL LETTER Z WITH PALATAL HOOK */
  ["\u1D8F", "\u1D8F"], /* LATIN SMALL LETTER A WITH RETROFLEX HOOK */
  ["\u1D90", "\u1D90"], /* LATIN SMALL LETTER ALPHA WITH RETROFLEX HOOK */
  ["\u1D91", "\u1D91"], /* LATIN SMALL LETTER D WITH HOOK AND TAIL */
  ["\u1D92", "\u1D92"], /* LATIN SMALL LETTER E WITH RETROFLEX HOOK */
  ["\u1D93", "\u1D93"], /* LATIN SMALL LETTER OPEN E WITH RETROFLEX HOOK */
  ["\u1D94", "\u1D94"], /* LATIN SMALL LETTER REVERSED OPEN E WITH RETROFLEX HOOK */
  ["\u1D95", "\u1D95"], /* LATIN SMALL LETTER SCHWA WITH RETROFLEX HOOK */
  ["\u1D96", "\u1D96"], /* LATIN SMALL LETTER I WITH RETROFLEX HOOK */
  ["\u1D97", "\u1D97"], /* LATIN SMALL LETTER OPEN O WITH RETROFLEX HOOK */
  ["\u1D98", "\u1D98"], /* LATIN SMALL LETTER ESH WITH RETROFLEX HOOK */
  ["\u1D99", "\u1D99"], /* LATIN SMALL LETTER U WITH RETROFLEX HOOK */
  ["\u1D9A", "\u1D9A"], /* LATIN SMALL LETTER EZH WITH RETROFLEX HOOK */
  ["\u1D9B", "\u1D9B"], /* MODIFIER LETTER SMALL TURNED ALPHA */
  ["\u1D9C", "\u1D9C"], /* MODIFIER LETTER SMALL C */
  ["\u1D9D", "\u1D9D"], /* MODIFIER LETTER SMALL C WITH CURL */
  ["\u1D9E", "\u1D9E"], /* MODIFIER LETTER SMALL ETH */
  ["\u1D9F", "\u1D9F"], /* MODIFIER LETTER SMALL REVERSED OPEN E */
  ["\u1DA0", "\u1DA0"], /* MODIFIER LETTER SMALL F */
  ["\u1DA1", "\u1DA1"], /* MODIFIER LETTER SMALL DOTLESS J WITH STROKE */
  ["\u1DA2", "\u1DA2"], /* MODIFIER LETTER SMALL SCRIPT G */
  ["\u1DA3", "\u1DA3"], /* MODIFIER LETTER SMALL TURNED H */
  ["\u1DA4", "\u1DA4"], /* MODIFIER LETTER SMALL I WITH STROKE */
  ["\u1DA5", "\u1DA5"], /* MODIFIER LETTER SMALL IOTA */
  ["\u1DA6", "\u1DA6"], /* MODIFIER LETTER SMALL CAPITAL I */
  ["\u1DA7", "\u1DA7"], /* MODIFIER LETTER SMALL CAPITAL I WITH STROKE */
  ["\u1DA8", "\u1DA8"], /* MODIFIER LETTER SMALL J WITH CROSSED-TAIL */
  ["\u1DA9", "\u1DA9"], /* MODIFIER LETTER SMALL L WITH RETROFLEX HOOK */
  ["\u1DAA", "\u1DAA"], /* MODIFIER LETTER SMALL L WITH PALATAL HOOK */
  ["\u1DAB", "\u1DAB"], /* MODIFIER LETTER SMALL CAPITAL L */
  ["\u1DAC", "\u1DAC"], /* MODIFIER LETTER SMALL M WITH HOOK */
  ["\u1DAD", "\u1DAD"], /* MODIFIER LETTER SMALL TURNED M WITH LONG LEG */
  ["\u1DAE", "\u1DAE"], /* MODIFIER LETTER SMALL N WITH LEFT HOOK */
  ["\u1DAF", "\u1DAF"], /* MODIFIER LETTER SMALL N WITH RETROFLEX HOOK */
  ["\u1DB0", "\u1DB0"], /* MODIFIER LETTER SMALL CAPITAL N */
  ["\u1DB1", "\u1DB1"], /* MODIFIER LETTER SMALL BARRED O */
  ["\u1DB2", "\u1DB2"], /* MODIFIER LETTER SMALL PHI */
  ["\u1DB3", "\u1DB3"], /* MODIFIER LETTER SMALL S WITH HOOK */
  ["\u1DB4", "\u1DB4"], /* MODIFIER LETTER SMALL ESH */
  ["\u1DB5", "\u1DB5"], /* MODIFIER LETTER SMALL T WITH PALATAL HOOK */
  ["\u1DB6", "\u1DB6"], /* MODIFIER LETTER SMALL U BAR */
  ["\u1DB7", "\u1DB7"], /* MODIFIER LETTER SMALL UPSILON */
  ["\u1DB8", "\u1DB8"], /* MODIFIER LETTER SMALL CAPITAL U */
  ["\u1DB9", "\u1DB9"], /* MODIFIER LETTER SMALL V WITH HOOK */
  ["\u1DBA", "\u1DBA"], /* MODIFIER LETTER SMALL TURNED V */
  ["\u1DBB", "\u1DBB"], /* MODIFIER LETTER SMALL Z */
  ["\u1DBC", "\u1DBC"], /* MODIFIER LETTER SMALL Z WITH RETROFLEX HOOK */
  ["\u1DBD", "\u1DBD"], /* MODIFIER LETTER SMALL Z WITH CURL */
  ["\u1DBE", "\u1DBE"], /* MODIFIER LETTER SMALL EZH */
  ["\u1DBF", "\u1DBF"], /* MODIFIER LETTER SMALL THETA */
  ["\u1DC0", "\u1DC0"], /* COMBINING DOTTED GRAVE ACCENT */
  ["\u1DC1", "\u1DC1"], /* COMBINING DOTTED ACUTE ACCENT */
  ["\u1DC2", "\u1DC2"], /* COMBINING SNAKE BELOW */
  ["\u1DC3", "\u1DC3"], /* COMBINING SUSPENSION MARK */
  ["\u1DC4", "\u1DC4"], /* COMBINING MACRON-ACUTE */
  ["\u1DC5", "\u1DC5"], /* COMBINING GRAVE-MACRON */
  ["\u1DC6", "\u1DC6"], /* COMBINING MACRON-GRAVE */
  ["\u1DC7", "\u1DC7"], /* COMBINING ACUTE-MACRON */
  ["\u1DC8", "\u1DC8"], /* COMBINING GRAVE-ACUTE-GRAVE */
  ["\u1DC9", "\u1DC9"], /* COMBINING ACUTE-GRAVE-ACUTE */
  ["\u1DCA", "\u1DCA"], /* COMBINING LATIN SMALL LETTER R BELOW */
  ["\u1DCB", "\u1DCB"], /* COMBINING BREVE-MACRON */
  ["\u1DCC", "\u1DCC"], /* COMBINING MACRON-BREVE */
  ["\u1DCD", "\u1DCD"], /* COMBINING DOUBLE CIRCUMFLEX ABOVE */
  ["\u1DCE", "\u1DCE"], /* COMBINING OGONEK ABOVE */
  ["\u1DCF", "\u1DCF"], /* COMBINING ZIGZAG BELOW */
  ["\u1DD0", "\u1DD0"], /* COMBINING IS BELOW */
  ["\u1DD1", "\u1DD1"], /* COMBINING UR ABOVE */
  ["\u1DD2", "\u1DD2"], /* COMBINING US ABOVE */
  ["\u1DD3", "\u1DD3"], /* COMBINING LATIN SMALL LETTER FLATTENED OPEN A ABOVE */
  ["\u1DD4", "\u1DD4"], /* COMBINING LATIN SMALL LETTER AE */
  ["\u1DD5", "\u1DD5"], /* COMBINING LATIN SMALL LETTER AO */
  ["\u1DD6", "\u1DD6"], /* COMBINING LATIN SMALL LETTER AV */
  ["\u1DD7", "\u1DD7"], /* COMBINING LATIN SMALL LETTER C CEDILLA */
  ["\u1DD8", "\u1DD8"], /* COMBINING LATIN SMALL LETTER INSULAR D */
  ["\u1DD9", "\u1DD9"], /* COMBINING LATIN SMALL LETTER ETH */
  ["\u1DDA", "\u1DDA"], /* COMBINING LATIN SMALL LETTER G */
  ["\u1DDB", "\u1DDB"], /* COMBINING LATIN LETTER SMALL CAPITAL G */
  ["\u1DDC", "\u1DDC"], /* COMBINING LATIN SMALL LETTER K */
  ["\u1DDD", "\u1DDD"], /* COMBINING LATIN SMALL LETTER L */
  ["\u1DDE", "\u1DDE"], /* COMBINING LATIN LETTER SMALL CAPITAL L */
  ["\u1DDF", "\u1DDF"], /* COMBINING LATIN LETTER SMALL CAPITAL M */
  ["\u1DE0", "\u1DE0"], /* COMBINING LATIN SMALL LETTER N */
  ["\u1DE1", "\u1DE1"], /* COMBINING LATIN LETTER SMALL CAPITAL N */
  ["\u1DE2", "\u1DE2"], /* COMBINING LATIN LETTER SMALL CAPITAL R */
  ["\u1DE3", "\u1DE3"], /* COMBINING LATIN SMALL LETTER R ROTUNDA */
  ["\u1DE4", "\u1DE4"], /* COMBINING LATIN SMALL LETTER S */
  ["\u1DE5", "\u1DE5"], /* COMBINING LATIN SMALL LETTER LONG S */
  ["\u1DE6", "\u1DE6"], /* COMBINING LATIN SMALL LETTER Z */
  ["\u1DE7", "\u1DE7"], /* COMBINING LATIN SMALL LETTER ALPHA */
  ["\u1DE8", "\u1DE8"], /* COMBINING LATIN SMALL LETTER B */
  ["\u1DE9", "\u1DE9"], /* COMBINING LATIN SMALL LETTER BETA */
  ["\u1DEA", "\u1DEA"], /* COMBINING LATIN SMALL LETTER SCHWA */
  ["\u1DEB", "\u1DEB"], /* COMBINING LATIN SMALL LETTER F */
  ["\u1DEC", "\u1DEC"], /* COMBINING LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE */
  ["\u1DED", "\u1DED"], /* COMBINING LATIN SMALL LETTER O WITH LIGHT CENTRALIZATION STROKE */
  ["\u1DEE", "\u1DEE"], /* COMBINING LATIN SMALL LETTER P */
  ["\u1DEF", "\u1DEF"], /* COMBINING LATIN SMALL LETTER ESH */
  ["\u1DF0", "\u1DF0"], /* COMBINING LATIN SMALL LETTER U WITH LIGHT CENTRALIZATION STROKE */
  ["\u1DF1", "\u1DF1"], /* COMBINING LATIN SMALL LETTER W */
  ["\u1DF2", "\u1DF2"], /* COMBINING LATIN SMALL LETTER A WITH DIAERESIS */
  ["\u1DF3", "\u1DF3"], /* COMBINING LATIN SMALL LETTER O WITH DIAERESIS */
  ["\u1DF4", "\u1DF4"], /* COMBINING LATIN SMALL LETTER U WITH DIAERESIS */
  ["\u1DF5", "\u1DF5"], /* COMBINING UP TACK ABOVE */
  ["\u1DF6", "\u1DF6"], /* COMBINING KAVYKA ABOVE RIGHT */
  ["\u1DF7", "\u1DF7"], /* COMBINING KAVYKA ABOVE LEFT */
  ["\u1DF8", "\u1DF8"], /* COMBINING DOT ABOVE LEFT */
  ["\u1DF9", "\u1DF9"], /* COMBINING WIDE INVERTED BRIDGE BELOW */
  ["\u1DFA", "\u1DFA"],
  ["\u1DFB", "\u1DFB"], /* COMBINING DELETION MARK */
  ["\u1DFC", "\u1DFC"], /* COMBINING DOUBLE INVERTED BREVE BELOW */
  ["\u1DFD", "\u1DFD"], /* COMBINING ALMOST EQUAL TO BELOW */
  ["\u1DFE", "\u1DFE"], /* COMBINING LEFT ARROWHEAD ABOVE */
  ["\u1DFF", "\u1DFF"], /* COMBINING RIGHT ARROWHEAD AND DOWN ARROWHEAD BELOW */
  ["\u1E00", "\u1E01"], /* LATIN CAPITAL LETTER A WITH RING BELOW */
  ["\u1E00", "\u1E01"], /* LATIN SMALL LETTER A WITH RING BELOW */
  ["\u1E02", "\u1E03"], /* LATIN CAPITAL LETTER B WITH DOT ABOVE */
  ["\u1E02", "\u1E03"], /* LATIN SMALL LETTER B WITH DOT ABOVE */
  ["\u1E04", "\u1E05"], /* LATIN CAPITAL LETTER B WITH DOT BELOW */
  ["\u1E04", "\u1E05"], /* LATIN SMALL LETTER B WITH DOT BELOW */
  ["\u1E06", "\u1E07"], /* LATIN CAPITAL LETTER B WITH LINE BELOW */
  ["\u1E06", "\u1E07"], /* LATIN SMALL LETTER B WITH LINE BELOW */
  ["\u1E08", "\u1E09"], /* LATIN CAPITAL LETTER C WITH CEDILLA AND ACUTE */
  ["\u1E08", "\u1E09"], /* LATIN SMALL LETTER C WITH CEDILLA AND ACUTE */
  ["\u1E0A", "\u1E0B"], /* LATIN CAPITAL LETTER D WITH DOT ABOVE */
  ["\u1E0A", "\u1E0B"], /* LATIN SMALL LETTER D WITH DOT ABOVE */
  ["\u1E0C", "\u1E0D"], /* LATIN CAPITAL LETTER D WITH DOT BELOW */
  ["\u1E0C", "\u1E0D"], /* LATIN SMALL LETTER D WITH DOT BELOW */
  ["\u1E0E", "\u1E0F"], /* LATIN CAPITAL LETTER D WITH LINE BELOW */
  ["\u1E0E", "\u1E0F"], /* LATIN SMALL LETTER D WITH LINE BELOW */
  ["\u1E10", "\u1E11"], /* LATIN CAPITAL LETTER D WITH CEDILLA */
  ["\u1E10", "\u1E11"], /* LATIN SMALL LETTER D WITH CEDILLA */
  ["\u1E12", "\u1E13"], /* LATIN CAPITAL LETTER D WITH CIRCUMFLEX BELOW */
  ["\u1E12", "\u1E13"], /* LATIN SMALL LETTER D WITH CIRCUMFLEX BELOW */
  ["\u1E14", "\u1E15"], /* LATIN CAPITAL LETTER E WITH MACRON AND GRAVE */
  ["\u1E14", "\u1E15"], /* LATIN SMALL LETTER E WITH MACRON AND GRAVE */
  ["\u1E16", "\u1E17"], /* LATIN CAPITAL LETTER E WITH MACRON AND ACUTE */
  ["\u1E16", "\u1E17"], /* LATIN SMALL LETTER E WITH MACRON AND ACUTE */
  ["\u1E18", "\u1E19"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX BELOW */
  ["\u1E18", "\u1E19"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX BELOW */
  ["\u1E1A", "\u1E1B"], /* LATIN CAPITAL LETTER E WITH TILDE BELOW */
  ["\u1E1A", "\u1E1B"], /* LATIN SMALL LETTER E WITH TILDE BELOW */
  ["\u1E1C", "\u1E1D"], /* LATIN CAPITAL LETTER E WITH CEDILLA AND BREVE */
  ["\u1E1C", "\u1E1D"], /* LATIN SMALL LETTER E WITH CEDILLA AND BREVE */
  ["\u1E1E", "\u1E1F"], /* LATIN CAPITAL LETTER F WITH DOT ABOVE */
  ["\u1E1E", "\u1E1F"], /* LATIN SMALL LETTER F WITH DOT ABOVE */
  ["\u1E20", "\u1E21"], /* LATIN CAPITAL LETTER G WITH MACRON */
  ["\u1E20", "\u1E21"], /* LATIN SMALL LETTER G WITH MACRON */
  ["\u1E22", "\u1E23"], /* LATIN CAPITAL LETTER H WITH DOT ABOVE */
  ["\u1E22", "\u1E23"], /* LATIN SMALL LETTER H WITH DOT ABOVE */
  ["\u1E24", "\u1E25"], /* LATIN CAPITAL LETTER H WITH DOT BELOW */
  ["\u1E24", "\u1E25"], /* LATIN SMALL LETTER H WITH DOT BELOW */
  ["\u1E26", "\u1E27"], /* LATIN CAPITAL LETTER H WITH DIAERESIS */
  ["\u1E26", "\u1E27"], /* LATIN SMALL LETTER H WITH DIAERESIS */
  ["\u1E28", "\u1E29"], /* LATIN CAPITAL LETTER H WITH CEDILLA */
  ["\u1E28", "\u1E29"], /* LATIN SMALL LETTER H WITH CEDILLA */
  ["\u1E2A", "\u1E2B"], /* LATIN CAPITAL LETTER H WITH BREVE BELOW */
  ["\u1E2A", "\u1E2B"], /* LATIN SMALL LETTER H WITH BREVE BELOW */
  ["\u1E2C", "\u1E2D"], /* LATIN CAPITAL LETTER I WITH TILDE BELOW */
  ["\u1E2C", "\u1E2D"], /* LATIN SMALL LETTER I WITH TILDE BELOW */
  ["\u1E2E", "\u1E2F"], /* LATIN CAPITAL LETTER I WITH DIAERESIS AND ACUTE */
  ["\u1E2E", "\u1E2F"], /* LATIN SMALL LETTER I WITH DIAERESIS AND ACUTE */
  ["\u1E30", "\u1E31"], /* LATIN CAPITAL LETTER K WITH ACUTE */
  ["\u1E30", "\u1E31"], /* LATIN SMALL LETTER K WITH ACUTE */
  ["\u1E32", "\u1E33"], /* LATIN CAPITAL LETTER K WITH DOT BELOW */
  ["\u1E32", "\u1E33"], /* LATIN SMALL LETTER K WITH DOT BELOW */
  ["\u1E34", "\u1E35"], /* LATIN CAPITAL LETTER K WITH LINE BELOW */
  ["\u1E34", "\u1E35"], /* LATIN SMALL LETTER K WITH LINE BELOW */
  ["\u1E36", "\u1E37"], /* LATIN CAPITAL LETTER L WITH DOT BELOW */
  ["\u1E36", "\u1E37"], /* LATIN SMALL LETTER L WITH DOT BELOW */
  ["\u1E38", "\u1E39"], /* LATIN CAPITAL LETTER L WITH DOT BELOW AND MACRON */
  ["\u1E38", "\u1E39"], /* LATIN SMALL LETTER L WITH DOT BELOW AND MACRON */
  ["\u1E3A", "\u1E3B"], /* LATIN CAPITAL LETTER L WITH LINE BELOW */
  ["\u1E3A", "\u1E3B"], /* LATIN SMALL LETTER L WITH LINE BELOW */
  ["\u1E3C", "\u1E3D"], /* LATIN CAPITAL LETTER L WITH CIRCUMFLEX BELOW */
  ["\u1E3C", "\u1E3D"], /* LATIN SMALL LETTER L WITH CIRCUMFLEX BELOW */
  ["\u1E3E", "\u1E3F"], /* LATIN CAPITAL LETTER M WITH ACUTE */
  ["\u1E3E", "\u1E3F"], /* LATIN SMALL LETTER M WITH ACUTE */
  ["\u1E40", "\u1E41"], /* LATIN CAPITAL LETTER M WITH DOT ABOVE */
  ["\u1E40", "\u1E41"], /* LATIN SMALL LETTER M WITH DOT ABOVE */
  ["\u1E42", "\u1E43"], /* LATIN CAPITAL LETTER M WITH DOT BELOW */
  ["\u1E42", "\u1E43"], /* LATIN SMALL LETTER M WITH DOT BELOW */
  ["\u1E44", "\u1E45"], /* LATIN CAPITAL LETTER N WITH DOT ABOVE */
  ["\u1E44", "\u1E45"], /* LATIN SMALL LETTER N WITH DOT ABOVE */
  ["\u1E46", "\u1E47"], /* LATIN CAPITAL LETTER N WITH DOT BELOW */
  ["\u1E46", "\u1E47"], /* LATIN SMALL LETTER N WITH DOT BELOW */
  ["\u1E48", "\u1E49"], /* LATIN CAPITAL LETTER N WITH LINE BELOW */
  ["\u1E48", "\u1E49"], /* LATIN SMALL LETTER N WITH LINE BELOW */
  ["\u1E4A", "\u1E4B"], /* LATIN CAPITAL LETTER N WITH CIRCUMFLEX BELOW */
  ["\u1E4A", "\u1E4B"], /* LATIN SMALL LETTER N WITH CIRCUMFLEX BELOW */
  ["\u1E4C", "\u1E4D"], /* LATIN CAPITAL LETTER O WITH TILDE AND ACUTE */
  ["\u1E4C", "\u1E4D"], /* LATIN SMALL LETTER O WITH TILDE AND ACUTE */
  ["\u1E4E", "\u1E4F"], /* LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS */
  ["\u1E4E", "\u1E4F"], /* LATIN SMALL LETTER O WITH TILDE AND DIAERESIS */
  ["\u1E50", "\u1E51"], /* LATIN CAPITAL LETTER O WITH MACRON AND GRAVE */
  ["\u1E50", "\u1E51"], /* LATIN SMALL LETTER O WITH MACRON AND GRAVE */
  ["\u1E52", "\u1E53"], /* LATIN CAPITAL LETTER O WITH MACRON AND ACUTE */
  ["\u1E52", "\u1E53"], /* LATIN SMALL LETTER O WITH MACRON AND ACUTE */
  ["\u1E54", "\u1E55"], /* LATIN CAPITAL LETTER P WITH ACUTE */
  ["\u1E54", "\u1E55"], /* LATIN SMALL LETTER P WITH ACUTE */
  ["\u1E56", "\u1E57"], /* LATIN CAPITAL LETTER P WITH DOT ABOVE */
  ["\u1E56", "\u1E57"], /* LATIN SMALL LETTER P WITH DOT ABOVE */
  ["\u1E58", "\u1E59"], /* LATIN CAPITAL LETTER R WITH DOT ABOVE */
  ["\u1E58", "\u1E59"], /* LATIN SMALL LETTER R WITH DOT ABOVE */
  ["\u1E5A", "\u1E5B"], /* LATIN CAPITAL LETTER R WITH DOT BELOW */
  ["\u1E5A", "\u1E5B"], /* LATIN SMALL LETTER R WITH DOT BELOW */
  ["\u1E5C", "\u1E5D"], /* LATIN CAPITAL LETTER R WITH DOT BELOW AND MACRON */
  ["\u1E5C", "\u1E5D"], /* LATIN SMALL LETTER R WITH DOT BELOW AND MACRON */
  ["\u1E5E", "\u1E5F"], /* LATIN CAPITAL LETTER R WITH LINE BELOW */
  ["\u1E5E", "\u1E5F"], /* LATIN SMALL LETTER R WITH LINE BELOW */
  ["\u1E60", "\u1E61"], /* LATIN CAPITAL LETTER S WITH DOT ABOVE */
  ["\u1E60", "\u1E61"], /* LATIN SMALL LETTER S WITH DOT ABOVE */
  ["\u1E62", "\u1E63"], /* LATIN CAPITAL LETTER S WITH DOT BELOW */
  ["\u1E62", "\u1E63"], /* LATIN SMALL LETTER S WITH DOT BELOW */
  ["\u1E64", "\u1E65"], /* LATIN CAPITAL LETTER S WITH ACUTE AND DOT ABOVE */
  ["\u1E64", "\u1E65"], /* LATIN SMALL LETTER S WITH ACUTE AND DOT ABOVE */
  ["\u1E66", "\u1E67"], /* LATIN CAPITAL LETTER S WITH CARON AND DOT ABOVE */
  ["\u1E66", "\u1E67"], /* LATIN SMALL LETTER S WITH CARON AND DOT ABOVE */
  ["\u1E68", "\u1E69"], /* LATIN CAPITAL LETTER S WITH DOT BELOW AND DOT ABOVE */
  ["\u1E68", "\u1E69"], /* LATIN SMALL LETTER S WITH DOT BELOW AND DOT ABOVE */
  ["\u1E6A", "\u1E6B"], /* LATIN CAPITAL LETTER T WITH DOT ABOVE */
  ["\u1E6A", "\u1E6B"], /* LATIN SMALL LETTER T WITH DOT ABOVE */
  ["\u1E6C", "\u1E6D"], /* LATIN CAPITAL LETTER T WITH DOT BELOW */
  ["\u1E6C", "\u1E6D"], /* LATIN SMALL LETTER T WITH DOT BELOW */
  ["\u1E6E", "\u1E6F"], /* LATIN CAPITAL LETTER T WITH LINE BELOW */
  ["\u1E6E", "\u1E6F"], /* LATIN SMALL LETTER T WITH LINE BELOW */
  ["\u1E70", "\u1E71"], /* LATIN CAPITAL LETTER T WITH CIRCUMFLEX BELOW */
  ["\u1E70", "\u1E71"], /* LATIN SMALL LETTER T WITH CIRCUMFLEX BELOW */
  ["\u1E72", "\u1E73"], /* LATIN CAPITAL LETTER U WITH DIAERESIS BELOW */
  ["\u1E72", "\u1E73"], /* LATIN SMALL LETTER U WITH DIAERESIS BELOW */
  ["\u1E74", "\u1E75"], /* LATIN CAPITAL LETTER U WITH TILDE BELOW */
  ["\u1E74", "\u1E75"], /* LATIN SMALL LETTER U WITH TILDE BELOW */
  ["\u1E76", "\u1E77"], /* LATIN CAPITAL LETTER U WITH CIRCUMFLEX BELOW */
  ["\u1E76", "\u1E77"], /* LATIN SMALL LETTER U WITH CIRCUMFLEX BELOW */
  ["\u1E78", "\u1E79"], /* LATIN CAPITAL LETTER U WITH TILDE AND ACUTE */
  ["\u1E78", "\u1E79"], /* LATIN SMALL LETTER U WITH TILDE AND ACUTE */
  ["\u1E7A", "\u1E7B"], /* LATIN CAPITAL LETTER U WITH MACRON AND DIAERESIS */
  ["\u1E7A", "\u1E7B"], /* LATIN SMALL LETTER U WITH MACRON AND DIAERESIS */
  ["\u1E7C", "\u1E7D"], /* LATIN CAPITAL LETTER V WITH TILDE */
  ["\u1E7C", "\u1E7D"], /* LATIN SMALL LETTER V WITH TILDE */
  ["\u1E7E", "\u1E7F"], /* LATIN CAPITAL LETTER V WITH DOT BELOW */
  ["\u1E7E", "\u1E7F"], /* LATIN SMALL LETTER V WITH DOT BELOW */
  ["\u1E80", "\u1E81"], /* LATIN CAPITAL LETTER W WITH GRAVE */
  ["\u1E80", "\u1E81"], /* LATIN SMALL LETTER W WITH GRAVE */
  ["\u1E82", "\u1E83"], /* LATIN CAPITAL LETTER W WITH ACUTE */
  ["\u1E82", "\u1E83"], /* LATIN SMALL LETTER W WITH ACUTE */
  ["\u1E84", "\u1E85"], /* LATIN CAPITAL LETTER W WITH DIAERESIS */
  ["\u1E84", "\u1E85"], /* LATIN SMALL LETTER W WITH DIAERESIS */
  ["\u1E86", "\u1E87"], /* LATIN CAPITAL LETTER W WITH DOT ABOVE */
  ["\u1E86", "\u1E87"], /* LATIN SMALL LETTER W WITH DOT ABOVE */
  ["\u1E88", "\u1E89"], /* LATIN CAPITAL LETTER W WITH DOT BELOW */
  ["\u1E88", "\u1E89"], /* LATIN SMALL LETTER W WITH DOT BELOW */
  ["\u1E8A", "\u1E8B"], /* LATIN CAPITAL LETTER X WITH DOT ABOVE */
  ["\u1E8A", "\u1E8B"], /* LATIN SMALL LETTER X WITH DOT ABOVE */
  ["\u1E8C", "\u1E8D"], /* LATIN CAPITAL LETTER X WITH DIAERESIS */
  ["\u1E8C", "\u1E8D"], /* LATIN SMALL LETTER X WITH DIAERESIS */
  ["\u1E8E", "\u1E8F"], /* LATIN CAPITAL LETTER Y WITH DOT ABOVE */
  ["\u1E8E", "\u1E8F"], /* LATIN SMALL LETTER Y WITH DOT ABOVE */
  ["\u1E90", "\u1E91"], /* LATIN CAPITAL LETTER Z WITH CIRCUMFLEX */
  ["\u1E90", "\u1E91"], /* LATIN SMALL LETTER Z WITH CIRCUMFLEX */
  ["\u1E92", "\u1E93"], /* LATIN CAPITAL LETTER Z WITH DOT BELOW */
  ["\u1E92", "\u1E93"], /* LATIN SMALL LETTER Z WITH DOT BELOW */
  ["\u1E94", "\u1E95"], /* LATIN CAPITAL LETTER Z WITH LINE BELOW */
  ["\u1E94", "\u1E95"], /* LATIN SMALL LETTER Z WITH LINE BELOW */
  ["\u0048\u0331", "\u1E96"], /* LATIN SMALL LETTER H WITH LINE BELOW */
  ["\u0054\u0308", "\u1E97"], /* LATIN SMALL LETTER T WITH DIAERESIS */
  ["\u0057\u030A", "\u1E98"], /* LATIN SMALL LETTER W WITH RING ABOVE */
  ["\u0059\u030A", "\u1E99"], /* LATIN SMALL LETTER Y WITH RING ABOVE */
  ["\u0041\u02BE", "\u1E9A"], /* LATIN SMALL LETTER A WITH RIGHT HALF RING */
  ["\u1E60", "\u1E9B"], /* LATIN SMALL LETTER LONG S WITH DOT ABOVE */
  ["\u1E9C", "\u1E9C"], /* LATIN SMALL LETTER LONG S WITH DIAGONAL STROKE */
  ["\u1E9D", "\u1E9D"], /* LATIN SMALL LETTER LONG S WITH HIGH STROKE */
  ["\u1E9E", "\u00DF"], /* LATIN CAPITAL LETTER SHARP S */
  ["\u1E9F", "\u1E9F"], /* LATIN SMALL LETTER DELTA */
  ["\u1EA0", "\u1EA1"], /* LATIN CAPITAL LETTER A WITH DOT BELOW */
  ["\u1EA0", "\u1EA1"], /* LATIN SMALL LETTER A WITH DOT BELOW */
  ["\u1EA2", "\u1EA3"], /* LATIN CAPITAL LETTER A WITH HOOK ABOVE */
  ["\u1EA2", "\u1EA3"], /* LATIN SMALL LETTER A WITH HOOK ABOVE */
  ["\u1EA4", "\u1EA5"], /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND ACUTE */
  ["\u1EA4", "\u1EA5"], /* LATIN SMALL LETTER A WITH CIRCUMFLEX AND ACUTE */
  ["\u1EA6", "\u1EA7"], /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND GRAVE */
  ["\u1EA6", "\u1EA7"], /* LATIN SMALL LETTER A WITH CIRCUMFLEX AND GRAVE */
  ["\u1EA8", "\u1EA9"], /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE */
  ["\u1EA8", "\u1EA9"], /* LATIN SMALL LETTER A WITH CIRCUMFLEX AND HOOK ABOVE */
  ["\u1EAA", "\u1EAB"], /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND TILDE */
  ["\u1EAA", "\u1EAB"], /* LATIN SMALL LETTER A WITH CIRCUMFLEX AND TILDE */
  ["\u1EAC", "\u1EAD"], /* LATIN CAPITAL LETTER A WITH CIRCUMFLEX AND DOT BELOW */
  ["\u1EAC", "\u1EAD"], /* LATIN SMALL LETTER A WITH CIRCUMFLEX AND DOT BELOW */
  ["\u1EAE", "\u1EAF"], /* LATIN CAPITAL LETTER A WITH BREVE AND ACUTE */
  ["\u1EAE", "\u1EAF"], /* LATIN SMALL LETTER A WITH BREVE AND ACUTE */
  ["\u1EB0", "\u1EB1"], /* LATIN CAPITAL LETTER A WITH BREVE AND GRAVE */
  ["\u1EB0", "\u1EB1"], /* LATIN SMALL LETTER A WITH BREVE AND GRAVE */
  ["\u1EB2", "\u1EB3"], /* LATIN CAPITAL LETTER A WITH BREVE AND HOOK ABOVE */
  ["\u1EB2", "\u1EB3"], /* LATIN SMALL LETTER A WITH BREVE AND HOOK ABOVE */
  ["\u1EB4", "\u1EB5"], /* LATIN CAPITAL LETTER A WITH BREVE AND TILDE */
  ["\u1EB4", "\u1EB5"], /* LATIN SMALL LETTER A WITH BREVE AND TILDE */
  ["\u1EB6", "\u1EB7"], /* LATIN CAPITAL LETTER A WITH BREVE AND DOT BELOW */
  ["\u1EB6", "\u1EB7"], /* LATIN SMALL LETTER A WITH BREVE AND DOT BELOW */
  ["\u1EB8", "\u1EB9"], /* LATIN CAPITAL LETTER E WITH DOT BELOW */
  ["\u1EB8", "\u1EB9"], /* LATIN SMALL LETTER E WITH DOT BELOW */
  ["\u1EBA", "\u1EBB"], /* LATIN CAPITAL LETTER E WITH HOOK ABOVE */
  ["\u1EBA", "\u1EBB"], /* LATIN SMALL LETTER E WITH HOOK ABOVE */
  ["\u1EBC", "\u1EBD"], /* LATIN CAPITAL LETTER E WITH TILDE */
  ["\u1EBC", "\u1EBD"], /* LATIN SMALL LETTER E WITH TILDE */
  ["\u1EBE", "\u1EBF"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND ACUTE */
  ["\u1EBE", "\u1EBF"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX AND ACUTE */
  ["\u1EC0", "\u1EC1"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND GRAVE */
  ["\u1EC0", "\u1EC1"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX AND GRAVE */
  ["\u1EC2", "\u1EC3"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE */
  ["\u1EC2", "\u1EC3"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX AND HOOK ABOVE */
  ["\u1EC4", "\u1EC5"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND TILDE */
  ["\u1EC4", "\u1EC5"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX AND TILDE */
  ["\u1EC6", "\u1EC7"], /* LATIN CAPITAL LETTER E WITH CIRCUMFLEX AND DOT BELOW */
  ["\u1EC6", "\u1EC7"], /* LATIN SMALL LETTER E WITH CIRCUMFLEX AND DOT BELOW */
  ["\u1EC8", "\u1EC9"], /* LATIN CAPITAL LETTER I WITH HOOK ABOVE */
  ["\u1EC8", "\u1EC9"], /* LATIN SMALL LETTER I WITH HOOK ABOVE */
  ["\u1ECA", "\u1ECB"], /* LATIN CAPITAL LETTER I WITH DOT BELOW */
  ["\u1ECA", "\u1ECB"], /* LATIN SMALL LETTER I WITH DOT BELOW */
  ["\u1ECC", "\u1ECD"], /* LATIN CAPITAL LETTER O WITH DOT BELOW */
  ["\u1ECC", "\u1ECD"], /* LATIN SMALL LETTER O WITH DOT BELOW */
  ["\u1ECE", "\u1ECF"], /* LATIN CAPITAL LETTER O WITH HOOK ABOVE */
  ["\u1ECE", "\u1ECF"], /* LATIN SMALL LETTER O WITH HOOK ABOVE */
  ["\u1ED0", "\u1ED1"], /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND ACUTE */
  ["\u1ED0", "\u1ED1"], /* LATIN SMALL LETTER O WITH CIRCUMFLEX AND ACUTE */
  ["\u1ED2", "\u1ED3"], /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND GRAVE */
  ["\u1ED2", "\u1ED3"], /* LATIN SMALL LETTER O WITH CIRCUMFLEX AND GRAVE */
  ["\u1ED4", "\u1ED5"], /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE */
  ["\u1ED4", "\u1ED5"], /* LATIN SMALL LETTER O WITH CIRCUMFLEX AND HOOK ABOVE */
  ["\u1ED6", "\u1ED7"], /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND TILDE */
  ["\u1ED6", "\u1ED7"], /* LATIN SMALL LETTER O WITH CIRCUMFLEX AND TILDE */
  ["\u1ED8", "\u1ED9"], /* LATIN CAPITAL LETTER O WITH CIRCUMFLEX AND DOT BELOW */
  ["\u1ED8", "\u1ED9"], /* LATIN SMALL LETTER O WITH CIRCUMFLEX AND DOT BELOW */
  ["\u1EDA", "\u1EDB"], /* LATIN CAPITAL LETTER O WITH HORN AND ACUTE */
  ["\u1EDA", "\u1EDB"], /* LATIN SMALL LETTER O WITH HORN AND ACUTE */
  ["\u1EDC", "\u1EDD"], /* LATIN CAPITAL LETTER O WITH HORN AND GRAVE */
  ["\u1EDC", "\u1EDD"], /* LATIN SMALL LETTER O WITH HORN AND GRAVE */
  ["\u1EDE", "\u1EDF"], /* LATIN CAPITAL LETTER O WITH HORN AND HOOK ABOVE */
  ["\u1EDE", "\u1EDF"], /* LATIN SMALL LETTER O WITH HORN AND HOOK ABOVE */
  ["\u1EE0", "\u1EE1"], /* LATIN CAPITAL LETTER O WITH HORN AND TILDE */
  ["\u1EE0", "\u1EE1"], /* LATIN SMALL LETTER O WITH HORN AND TILDE */
  ["\u1EE2", "\u1EE3"], /* LATIN CAPITAL LETTER O WITH HORN AND DOT BELOW */
  ["\u1EE2", "\u1EE3"], /* LATIN SMALL LETTER O WITH HORN AND DOT BELOW */
  ["\u1EE4", "\u1EE5"], /* LATIN CAPITAL LETTER U WITH DOT BELOW */
  ["\u1EE4", "\u1EE5"], /* LATIN SMALL LETTER U WITH DOT BELOW */
  ["\u1EE6", "\u1EE7"], /* LATIN CAPITAL LETTER U WITH HOOK ABOVE */
  ["\u1EE6", "\u1EE7"], /* LATIN SMALL LETTER U WITH HOOK ABOVE */
  ["\u1EE8", "\u1EE9"], /* LATIN CAPITAL LETTER U WITH HORN AND ACUTE */
  ["\u1EE8", "\u1EE9"], /* LATIN SMALL LETTER U WITH HORN AND ACUTE */
  ["\u1EEA", "\u1EEB"], /* LATIN CAPITAL LETTER U WITH HORN AND GRAVE */
  ["\u1EEA", "\u1EEB"], /* LATIN SMALL LETTER U WITH HORN AND GRAVE */
  ["\u1EEC", "\u1EED"], /* LATIN CAPITAL LETTER U WITH HORN AND HOOK ABOVE */
  ["\u1EEC", "\u1EED"], /* LATIN SMALL LETTER U WITH HORN AND HOOK ABOVE */
  ["\u1EEE", "\u1EEF"], /* LATIN CAPITAL LETTER U WITH HORN AND TILDE */
  ["\u1EEE", "\u1EEF"], /* LATIN SMALL LETTER U WITH HORN AND TILDE */
  ["\u1EF0", "\u1EF1"], /* LATIN CAPITAL LETTER U WITH HORN AND DOT BELOW */
  ["\u1EF0", "\u1EF1"], /* LATIN SMALL LETTER U WITH HORN AND DOT BELOW */
  ["\u1EF2", "\u1EF3"], /* LATIN CAPITAL LETTER Y WITH GRAVE */
  ["\u1EF2", "\u1EF3"], /* LATIN SMALL LETTER Y WITH GRAVE */
  ["\u1EF4", "\u1EF5"], /* LATIN CAPITAL LETTER Y WITH DOT BELOW */
  ["\u1EF4", "\u1EF5"], /* LATIN SMALL LETTER Y WITH DOT BELOW */
  ["\u1EF6", "\u1EF7"], /* LATIN CAPITAL LETTER Y WITH HOOK ABOVE */
  ["\u1EF6", "\u1EF7"], /* LATIN SMALL LETTER Y WITH HOOK ABOVE */
  ["\u1EF8", "\u1EF9"], /* LATIN CAPITAL LETTER Y WITH TILDE */
  ["\u1EF8", "\u1EF9"], /* LATIN SMALL LETTER Y WITH TILDE */
  ["\u1EFA", "\u1EFB"], /* LATIN CAPITAL LETTER MIDDLE-WELSH LL */
  ["\u1EFA", "\u1EFB"], /* LATIN SMALL LETTER MIDDLE-WELSH LL */
  ["\u1EFC", "\u1EFD"], /* LATIN CAPITAL LETTER MIDDLE-WELSH V */
  ["\u1EFC", "\u1EFD"], /* LATIN SMALL LETTER MIDDLE-WELSH V */
  ["\u1EFE", "\u1EFF"], /* LATIN CAPITAL LETTER Y WITH LOOP */
  ["\u1EFE", "\u1EFF"], /* LATIN SMALL LETTER Y WITH LOOP */
  ["\u1F08", "\u1F00"], /* GREEK SMALL LETTER ALPHA WITH PSILI */
  ["\u1F09", "\u1F01"], /* GREEK SMALL LETTER ALPHA WITH DASIA */
  ["\u1F0A", "\u1F02"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA */
  ["\u1F0B", "\u1F03"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA */
  ["\u1F0C", "\u1F04"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA */
  ["\u1F0D", "\u1F05"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA */
  ["\u1F0E", "\u1F06"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI */
  ["\u1F0F", "\u1F07"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI */
  ["\u1F08", "\u1F00"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI */
  ["\u1F09", "\u1F01"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA */
  ["\u1F0A", "\u1F02"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA */
  ["\u1F0B", "\u1F03"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA */
  ["\u1F0C", "\u1F04"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA */
  ["\u1F0D", "\u1F05"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA */
  ["\u1F0E", "\u1F06"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI */
  ["\u1F0F", "\u1F07"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI */
  ["\u1F18", "\u1F10"], /* GREEK SMALL LETTER EPSILON WITH PSILI */
  ["\u1F19", "\u1F11"], /* GREEK SMALL LETTER EPSILON WITH DASIA */
  ["\u1F1A", "\u1F12"], /* GREEK SMALL LETTER EPSILON WITH PSILI AND VARIA */
  ["\u1F1B", "\u1F13"], /* GREEK SMALL LETTER EPSILON WITH DASIA AND VARIA */
  ["\u1F1C", "\u1F14"], /* GREEK SMALL LETTER EPSILON WITH PSILI AND OXIA */
  ["\u1F1D", "\u1F15"], /* GREEK SMALL LETTER EPSILON WITH DASIA AND OXIA */
  ["\u1F16", "\u1F16"],
  ["\u1F17", "\u1F17"],
  ["\u1F18", "\u1F10"], /* GREEK CAPITAL LETTER EPSILON WITH PSILI */
  ["\u1F19", "\u1F11"], /* GREEK CAPITAL LETTER EPSILON WITH DASIA */
  ["\u1F1A", "\u1F12"], /* GREEK CAPITAL LETTER EPSILON WITH PSILI AND VARIA */
  ["\u1F1B", "\u1F13"], /* GREEK CAPITAL LETTER EPSILON WITH DASIA AND VARIA */
  ["\u1F1C", "\u1F14"], /* GREEK CAPITAL LETTER EPSILON WITH PSILI AND OXIA */
  ["\u1F1D", "\u1F15"], /* GREEK CAPITAL LETTER EPSILON WITH DASIA AND OXIA */
  ["\u1F1E", "\u1F1E"],
  ["\u1F1F", "\u1F1F"],
  ["\u1F28", "\u1F20"], /* GREEK SMALL LETTER ETA WITH PSILI */
  ["\u1F29", "\u1F21"], /* GREEK SMALL LETTER ETA WITH DASIA */
  ["\u1F2A", "\u1F22"], /* GREEK SMALL LETTER ETA WITH PSILI AND VARIA */
  ["\u1F2B", "\u1F23"], /* GREEK SMALL LETTER ETA WITH DASIA AND VARIA */
  ["\u1F2C", "\u1F24"], /* GREEK SMALL LETTER ETA WITH PSILI AND OXIA */
  ["\u1F2D", "\u1F25"], /* GREEK SMALL LETTER ETA WITH DASIA AND OXIA */
  ["\u1F2E", "\u1F26"], /* GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI */
  ["\u1F2F", "\u1F27"], /* GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI */
  ["\u1F28", "\u1F20"], /* GREEK CAPITAL LETTER ETA WITH PSILI */
  ["\u1F29", "\u1F21"], /* GREEK CAPITAL LETTER ETA WITH DASIA */
  ["\u1F2A", "\u1F22"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA */
  ["\u1F2B", "\u1F23"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA */
  ["\u1F2C", "\u1F24"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA */
  ["\u1F2D", "\u1F25"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA */
  ["\u1F2E", "\u1F26"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI */
  ["\u1F2F", "\u1F27"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI */
  ["\u1F38", "\u1F30"], /* GREEK SMALL LETTER IOTA WITH PSILI */
  ["\u1F39", "\u1F31"], /* GREEK SMALL LETTER IOTA WITH DASIA */
  ["\u1F3A", "\u1F32"], /* GREEK SMALL LETTER IOTA WITH PSILI AND VARIA */
  ["\u1F3B", "\u1F33"], /* GREEK SMALL LETTER IOTA WITH DASIA AND VARIA */
  ["\u1F3C", "\u1F34"], /* GREEK SMALL LETTER IOTA WITH PSILI AND OXIA */
  ["\u1F3D", "\u1F35"], /* GREEK SMALL LETTER IOTA WITH DASIA AND OXIA */
  ["\u1F3E", "\u1F36"], /* GREEK SMALL LETTER IOTA WITH PSILI AND PERISPOMENI */
  ["\u1F3F", "\u1F37"], /* GREEK SMALL LETTER IOTA WITH DASIA AND PERISPOMENI */
  ["\u1F38", "\u1F30"], /* GREEK CAPITAL LETTER IOTA WITH PSILI */
  ["\u1F39", "\u1F31"], /* GREEK CAPITAL LETTER IOTA WITH DASIA */
  ["\u1F3A", "\u1F32"], /* GREEK CAPITAL LETTER IOTA WITH PSILI AND VARIA */
  ["\u1F3B", "\u1F33"], /* GREEK CAPITAL LETTER IOTA WITH DASIA AND VARIA */
  ["\u1F3C", "\u1F34"], /* GREEK CAPITAL LETTER IOTA WITH PSILI AND OXIA */
  ["\u1F3D", "\u1F35"], /* GREEK CAPITAL LETTER IOTA WITH DASIA AND OXIA */
  ["\u1F3E", "\u1F36"], /* GREEK CAPITAL LETTER IOTA WITH PSILI AND PERISPOMENI */
  ["\u1F3F", "\u1F37"], /* GREEK CAPITAL LETTER IOTA WITH DASIA AND PERISPOMENI */
  ["\u1F48", "\u1F40"], /* GREEK SMALL LETTER OMICRON WITH PSILI */
  ["\u1F49", "\u1F41"], /* GREEK SMALL LETTER OMICRON WITH DASIA */
  ["\u1F4A", "\u1F42"], /* GREEK SMALL LETTER OMICRON WITH PSILI AND VARIA */
  ["\u1F4B", "\u1F43"], /* GREEK SMALL LETTER OMICRON WITH DASIA AND VARIA */
  ["\u1F4C", "\u1F44"], /* GREEK SMALL LETTER OMICRON WITH PSILI AND OXIA */
  ["\u1F4D", "\u1F45"], /* GREEK SMALL LETTER OMICRON WITH DASIA AND OXIA */
  ["\u1F46", "\u1F46"],
  ["\u1F47", "\u1F47"],
  ["\u1F48", "\u1F40"], /* GREEK CAPITAL LETTER OMICRON WITH PSILI */
  ["\u1F49", "\u1F41"], /* GREEK CAPITAL LETTER OMICRON WITH DASIA */
  ["\u1F4A", "\u1F42"], /* GREEK CAPITAL LETTER OMICRON WITH PSILI AND VARIA */
  ["\u1F4B", "\u1F43"], /* GREEK CAPITAL LETTER OMICRON WITH DASIA AND VARIA */
  ["\u1F4C", "\u1F44"], /* GREEK CAPITAL LETTER OMICRON WITH PSILI AND OXIA */
  ["\u1F4D", "\u1F45"], /* GREEK CAPITAL LETTER OMICRON WITH DASIA AND OXIA */
  ["\u1F4E", "\u1F4E"],
  ["\u1F4F", "\u1F4F"],
  ["\u03A5\u0313", "\u1F50"], /* GREEK SMALL LETTER UPSILON WITH PSILI */
  ["\u1F59", "\u1F51"], /* GREEK SMALL LETTER UPSILON WITH DASIA */
  ["\u03A5\u0313\u0300", "\u1F52"], /* GREEK SMALL LETTER UPSILON WITH PSILI AND VARIA */
  ["\u1F5B", "\u1F53"], /* GREEK SMALL LETTER UPSILON WITH DASIA AND VARIA */
  ["\u03A5\u0313\u0301", "\u1F54"], /* GREEK SMALL LETTER UPSILON WITH PSILI AND OXIA */
  ["\u1F5D", "\u1F55"], /* GREEK SMALL LETTER UPSILON WITH DASIA AND OXIA */
  ["\u03A5\u0313\u0342", "\u1F56"], /* GREEK SMALL LETTER UPSILON WITH PSILI AND PERISPOMENI */
  ["\u1F5F", "\u1F57"], /* GREEK SMALL LETTER UPSILON WITH DASIA AND PERISPOMENI */
  ["\u1F58", "\u1F58"],
  ["\u1F59", "\u1F51"], /* GREEK CAPITAL LETTER UPSILON WITH DASIA */
  ["\u1F5A", "\u1F5A"],
  ["\u1F5B", "\u1F53"], /* GREEK CAPITAL LETTER UPSILON WITH DASIA AND VARIA */
  ["\u1F5C", "\u1F5C"],
  ["\u1F5D", "\u1F55"], /* GREEK CAPITAL LETTER UPSILON WITH DASIA AND OXIA */
  ["\u1F5E", "\u1F5E"],
  ["\u1F5F", "\u1F57"], /* GREEK CAPITAL LETTER UPSILON WITH DASIA AND PERISPOMENI */
  ["\u1F68", "\u1F60"], /* GREEK SMALL LETTER OMEGA WITH PSILI */
  ["\u1F69", "\u1F61"], /* GREEK SMALL LETTER OMEGA WITH DASIA */
  ["\u1F6A", "\u1F62"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA */
  ["\u1F6B", "\u1F63"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA */
  ["\u1F6C", "\u1F64"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA */
  ["\u1F6D", "\u1F65"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA */
  ["\u1F6E", "\u1F66"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI */
  ["\u1F6F", "\u1F67"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI */
  ["\u1F68", "\u1F60"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI */
  ["\u1F69", "\u1F61"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA */
  ["\u1F6A", "\u1F62"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA */
  ["\u1F6B", "\u1F63"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA */
  ["\u1F6C", "\u1F64"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA */
  ["\u1F6D", "\u1F65"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA */
  ["\u1F6E", "\u1F66"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI */
  ["\u1F6F", "\u1F67"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI */
  ["\u1FBA", "\u1F70"], /* GREEK SMALL LETTER ALPHA WITH VARIA */
  ["\u1FBB", "\u1F71"], /* GREEK SMALL LETTER ALPHA WITH OXIA */
  ["\u1FC8", "\u1F72"], /* GREEK SMALL LETTER EPSILON WITH VARIA */
  ["\u1FC9", "\u1F73"], /* GREEK SMALL LETTER EPSILON WITH OXIA */
  ["\u1FCA", "\u1F74"], /* GREEK SMALL LETTER ETA WITH VARIA */
  ["\u1FCB", "\u1F75"], /* GREEK SMALL LETTER ETA WITH OXIA */
  ["\u1FDA", "\u1F76"], /* GREEK SMALL LETTER IOTA WITH VARIA */
  ["\u1FDB", "\u1F77"], /* GREEK SMALL LETTER IOTA WITH OXIA */
  ["\u1FF8", "\u1F78"], /* GREEK SMALL LETTER OMICRON WITH VARIA */
  ["\u1FF9", "\u1F79"], /* GREEK SMALL LETTER OMICRON WITH OXIA */
  ["\u1FEA", "\u1F7A"], /* GREEK SMALL LETTER UPSILON WITH VARIA */
  ["\u1FEB", "\u1F7B"], /* GREEK SMALL LETTER UPSILON WITH OXIA */
  ["\u1FFA", "\u1F7C"], /* GREEK SMALL LETTER OMEGA WITH VARIA */
  ["\u1FFB", "\u1F7D"], /* GREEK SMALL LETTER OMEGA WITH OXIA */
  ["\u1F7E", "\u1F7E"],
  ["\u1F7F", "\u1F7F"],
  ["\u1F08\u0399", "\u1F80"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND YPOGEGRAMMENI */
  ["\u1F09\u0399", "\u1F81"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND YPOGEGRAMMENI */
  ["\u1F0A\u0399", "\u1F82"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND VARIA AND YPOGEGRAMMENI */
  ["\u1F0B\u0399", "\u1F83"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND VARIA AND YPOGEGRAMMENI */
  ["\u1F0C\u0399", "\u1F84"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND OXIA AND YPOGEGRAMMENI */
  ["\u1F0D\u0399", "\u1F85"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND OXIA AND YPOGEGRAMMENI */
  ["\u1F0E\u0399", "\u1F86"], /* GREEK SMALL LETTER ALPHA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1F0F\u0399", "\u1F87"], /* GREEK SMALL LETTER ALPHA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1F08\u0399", "\u1F80"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND PROSGEGRAMMENI */
  ["\u1F09\u0399", "\u1F81"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND PROSGEGRAMMENI */
  ["\u1F0A\u0399", "\u1F82"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND VARIA AND PROSGEGRAMMENI */
  ["\u1F0B\u0399", "\u1F83"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND VARIA AND PROSGEGRAMMENI */
  ["\u1F0C\u0399", "\u1F84"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND OXIA AND PROSGEGRAMMENI */
  ["\u1F0D\u0399", "\u1F85"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND OXIA AND PROSGEGRAMMENI */
  ["\u1F0E\u0399", "\u1F86"], /* GREEK CAPITAL LETTER ALPHA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI */
  ["\u1F0F\u0399", "\u1F87"], /* GREEK CAPITAL LETTER ALPHA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI */
  ["\u1F28\u0399", "\u1F90"], /* GREEK SMALL LETTER ETA WITH PSILI AND YPOGEGRAMMENI */
  ["\u1F29\u0399", "\u1F91"], /* GREEK SMALL LETTER ETA WITH DASIA AND YPOGEGRAMMENI */
  ["\u1F2A\u0399", "\u1F92"], /* GREEK SMALL LETTER ETA WITH PSILI AND VARIA AND YPOGEGRAMMENI */
  ["\u1F2B\u0399", "\u1F93"], /* GREEK SMALL LETTER ETA WITH DASIA AND VARIA AND YPOGEGRAMMENI */
  ["\u1F2C\u0399", "\u1F94"], /* GREEK SMALL LETTER ETA WITH PSILI AND OXIA AND YPOGEGRAMMENI */
  ["\u1F2D\u0399", "\u1F95"], /* GREEK SMALL LETTER ETA WITH DASIA AND OXIA AND YPOGEGRAMMENI */
  ["\u1F2E\u0399", "\u1F96"], /* GREEK SMALL LETTER ETA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1F2F\u0399", "\u1F97"], /* GREEK SMALL LETTER ETA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1F28\u0399", "\u1F90"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND PROSGEGRAMMENI */
  ["\u1F29\u0399", "\u1F91"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND PROSGEGRAMMENI */
  ["\u1F2A\u0399", "\u1F92"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND VARIA AND PROSGEGRAMMENI */
  ["\u1F2B\u0399", "\u1F93"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND VARIA AND PROSGEGRAMMENI */
  ["\u1F2C\u0399", "\u1F94"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND OXIA AND PROSGEGRAMMENI */
  ["\u1F2D\u0399", "\u1F95"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND OXIA AND PROSGEGRAMMENI */
  ["\u1F2E\u0399", "\u1F96"], /* GREEK CAPITAL LETTER ETA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI */
  ["\u1F2F\u0399", "\u1F97"], /* GREEK CAPITAL LETTER ETA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI */
  ["\u1F68\u0399", "\u1FA0"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND YPOGEGRAMMENI */
  ["\u1F69\u0399", "\u1FA1"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND YPOGEGRAMMENI */
  ["\u1F6A\u0399", "\u1FA2"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND VARIA AND YPOGEGRAMMENI */
  ["\u1F6B\u0399", "\u1FA3"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND VARIA AND YPOGEGRAMMENI */
  ["\u1F6C\u0399", "\u1FA4"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND OXIA AND YPOGEGRAMMENI */
  ["\u1F6D\u0399", "\u1FA5"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND OXIA AND YPOGEGRAMMENI */
  ["\u1F6E\u0399", "\u1FA6"], /* GREEK SMALL LETTER OMEGA WITH PSILI AND PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1F6F\u0399", "\u1FA7"], /* GREEK SMALL LETTER OMEGA WITH DASIA AND PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1F68\u0399", "\u1FA0"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND PROSGEGRAMMENI */
  ["\u1F69\u0399", "\u1FA1"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND PROSGEGRAMMENI */
  ["\u1F6A\u0399", "\u1FA2"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND VARIA AND PROSGEGRAMMENI */
  ["\u1F6B\u0399", "\u1FA3"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND VARIA AND PROSGEGRAMMENI */
  ["\u1F6C\u0399", "\u1FA4"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND OXIA AND PROSGEGRAMMENI */
  ["\u1F6D\u0399", "\u1FA5"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND OXIA AND PROSGEGRAMMENI */
  ["\u1F6E\u0399", "\u1FA6"], /* GREEK CAPITAL LETTER OMEGA WITH PSILI AND PERISPOMENI AND PROSGEGRAMMENI */
  ["\u1F6F\u0399", "\u1FA7"], /* GREEK CAPITAL LETTER OMEGA WITH DASIA AND PERISPOMENI AND PROSGEGRAMMENI */
  ["\u1FB8", "\u1FB0"], /* GREEK SMALL LETTER ALPHA WITH VRACHY */
  ["\u1FB9", "\u1FB1"], /* GREEK SMALL LETTER ALPHA WITH MACRON */
  ["\u1FBA\u0399", "\u1FB2"], /* GREEK SMALL LETTER ALPHA WITH VARIA AND YPOGEGRAMMENI */
  ["\u0391\u0399", "\u1FB3"], /* GREEK SMALL LETTER ALPHA WITH YPOGEGRAMMENI */
  ["\u0386\u0399", "\u1FB4"], /* GREEK SMALL LETTER ALPHA WITH OXIA AND YPOGEGRAMMENI */
  ["\u1FB5", "\u1FB5"],
  ["\u0391\u0342", "\u1FB6"], /* GREEK SMALL LETTER ALPHA WITH PERISPOMENI */
  ["\u0391\u0342\u0399", "\u1FB7"], /* GREEK SMALL LETTER ALPHA WITH PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1FB8", "\u1FB0"], /* GREEK CAPITAL LETTER ALPHA WITH VRACHY */
  ["\u1FB9", "\u1FB1"], /* GREEK CAPITAL LETTER ALPHA WITH MACRON */
  ["\u1FBA", "\u1F70"], /* GREEK CAPITAL LETTER ALPHA WITH VARIA */
  ["\u1FBB", "\u1F71"], /* GREEK CAPITAL LETTER ALPHA WITH OXIA */
  ["\u0391\u0399", "\u1FB3"], /* GREEK CAPITAL LETTER ALPHA WITH PROSGEGRAMMENI */
  ["\u1FBD", "\u1FBD"], /* GREEK KORONIS */
  ["\u0399", "\u1FBE"], /* GREEK PROSGEGRAMMENI */
  ["\u1FBF", "\u1FBF"], /* GREEK PSILI */
  ["\u1FC0", "\u1FC0"], /* GREEK PERISPOMENI */
  ["\u1FC1", "\u1FC1"], /* GREEK DIALYTIKA AND PERISPOMENI */
  ["\u1FCA\u0399", "\u1FC2"], /* GREEK SMALL LETTER ETA WITH VARIA AND YPOGEGRAMMENI */
  ["\u0397\u0399", "\u1FC3"], /* GREEK SMALL LETTER ETA WITH YPOGEGRAMMENI */
  ["\u0389\u0399", "\u1FC4"], /* GREEK SMALL LETTER ETA WITH OXIA AND YPOGEGRAMMENI */
  ["\u1FC5", "\u1FC5"],
  ["\u0397\u0342", "\u1FC6"], /* GREEK SMALL LETTER ETA WITH PERISPOMENI */
  ["\u0397\u0342\u0399", "\u1FC7"], /* GREEK SMALL LETTER ETA WITH PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1FC8", "\u1F72"], /* GREEK CAPITAL LETTER EPSILON WITH VARIA */
  ["\u1FC9", "\u1F73"], /* GREEK CAPITAL LETTER EPSILON WITH OXIA */
  ["\u1FCA", "\u1F74"], /* GREEK CAPITAL LETTER ETA WITH VARIA */
  ["\u1FCB", "\u1F75"], /* GREEK CAPITAL LETTER ETA WITH OXIA */
  ["\u0397\u0399", "\u1FC3"], /* GREEK CAPITAL LETTER ETA WITH PROSGEGRAMMENI */
  ["\u1FCD", "\u1FCD"], /* GREEK PSILI AND VARIA */
  ["\u1FCE", "\u1FCE"], /* GREEK PSILI AND OXIA */
  ["\u1FCF", "\u1FCF"], /* GREEK PSILI AND PERISPOMENI */
  ["\u1FD8", "\u1FD0"], /* GREEK SMALL LETTER IOTA WITH VRACHY */
  ["\u1FD9", "\u1FD1"], /* GREEK SMALL LETTER IOTA WITH MACRON */
  ["\u0399\u0308\u0300", "\u1FD2"], /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND VARIA */
  ["\u0399\u0308\u0301", "\u1FD3"], /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND OXIA */
  ["\u1FD4", "\u1FD4"],
  ["\u1FD5", "\u1FD5"],
  ["\u0399\u0342", "\u1FD6"], /* GREEK SMALL LETTER IOTA WITH PERISPOMENI */
  ["\u0399\u0308\u0342", "\u1FD7"], /* GREEK SMALL LETTER IOTA WITH DIALYTIKA AND PERISPOMENI */
  ["\u1FD8", "\u1FD0"], /* GREEK CAPITAL LETTER IOTA WITH VRACHY */
  ["\u1FD9", "\u1FD1"], /* GREEK CAPITAL LETTER IOTA WITH MACRON */
  ["\u1FDA", "\u1F76"], /* GREEK CAPITAL LETTER IOTA WITH VARIA */
  ["\u1FDB", "\u1F77"], /* GREEK CAPITAL LETTER IOTA WITH OXIA */
  ["\u1FDC", "\u1FDC"],
  ["\u1FDD", "\u1FDD"], /* GREEK DASIA AND VARIA */
  ["\u1FDE", "\u1FDE"], /* GREEK DASIA AND OXIA */
  ["\u1FDF", "\u1FDF"], /* GREEK DASIA AND PERISPOMENI */
  ["\u1FE8", "\u1FE0"], /* GREEK SMALL LETTER UPSILON WITH VRACHY */
  ["\u1FE9", "\u1FE1"], /* GREEK SMALL LETTER UPSILON WITH MACRON */
  ["\u03A5\u0308\u0300", "\u1FE2"], /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND VARIA */
  ["\u03A5\u0308\u0301", "\u1FE3"], /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND OXIA */
  ["\u03A1\u0313", "\u1FE4"], /* GREEK SMALL LETTER RHO WITH PSILI */
  ["\u1FEC", "\u1FE5"], /* GREEK SMALL LETTER RHO WITH DASIA */
  ["\u03A5\u0342", "\u1FE6"], /* GREEK SMALL LETTER UPSILON WITH PERISPOMENI */
  ["\u03A5\u0308\u0342", "\u1FE7"], /* GREEK SMALL LETTER UPSILON WITH DIALYTIKA AND PERISPOMENI */
  ["\u1FE8", "\u1FE0"], /* GREEK CAPITAL LETTER UPSILON WITH VRACHY */
  ["\u1FE9", "\u1FE1"], /* GREEK CAPITAL LETTER UPSILON WITH MACRON */
  ["\u1FEA", "\u1F7A"], /* GREEK CAPITAL LETTER UPSILON WITH VARIA */
  ["\u1FEB", "\u1F7B"], /* GREEK CAPITAL LETTER UPSILON WITH OXIA */
  ["\u1FEC", "\u1FE5"], /* GREEK CAPITAL LETTER RHO WITH DASIA */
  ["\u1FED", "\u1FED"], /* GREEK DIALYTIKA AND VARIA */
  ["\u1FEE", "\u1FEE"], /* GREEK DIALYTIKA AND OXIA */
  ["\u1FEF", "\u1FEF"], /* GREEK VARIA */
  ["\u1FF0", "\u1FF0"],
  ["\u1FF1", "\u1FF1"],
  ["\u1FFA\u0399", "\u1FF2"], /* GREEK SMALL LETTER OMEGA WITH VARIA AND YPOGEGRAMMENI */
  ["\u03A9\u0399", "\u1FF3"], /* GREEK SMALL LETTER OMEGA WITH YPOGEGRAMMENI */
  ["\u038F\u0399", "\u1FF4"], /* GREEK SMALL LETTER OMEGA WITH OXIA AND YPOGEGRAMMENI */
  ["\u1FF5", "\u1FF5"],
  ["\u03A9\u0342", "\u1FF6"], /* GREEK SMALL LETTER OMEGA WITH PERISPOMENI */
  ["\u03A9\u0342\u0399", "\u1FF7"], /* GREEK SMALL LETTER OMEGA WITH PERISPOMENI AND YPOGEGRAMMENI */
  ["\u1FF8", "\u1F78"], /* GREEK CAPITAL LETTER OMICRON WITH VARIA */
  ["\u1FF9", "\u1F79"], /* GREEK CAPITAL LETTER OMICRON WITH OXIA */
  ["\u1FFA", "\u1F7C"], /* GREEK CAPITAL LETTER OMEGA WITH VARIA */
  ["\u1FFB", "\u1F7D"], /* GREEK CAPITAL LETTER OMEGA WITH OXIA */
  ["\u03A9\u0399", "\u1FF3"], /* GREEK CAPITAL LETTER OMEGA WITH PROSGEGRAMMENI */
  ["\u1FFD", "\u1FFD"], /* GREEK OXIA */
  ["\u1FFE", "\u1FFE"], /* GREEK DASIA */
  ["\u1FFF", "\u1FFF"],
  ["\u2000", "\u2000"], /* EN QUAD */
  ["\u2001", "\u2001"], /* EM QUAD */
  ["\u2002", "\u2002"], /* EN SPACE */
  ["\u2003", "\u2003"], /* EM SPACE */
  ["\u2004", "\u2004"], /* THREE-PER-EM SPACE */
  ["\u2005", "\u2005"], /* FOUR-PER-EM SPACE */
  ["\u2006", "\u2006"], /* SIX-PER-EM SPACE */
  ["\u2007", "\u2007"], /* FIGURE SPACE */
  ["\u2008", "\u2008"], /* PUNCTUATION SPACE */
  ["\u2009", "\u2009"], /* THIN SPACE */
  ["\u200A", "\u200A"], /* HAIR SPACE */
  ["\u200B", "\u200B"], /* ZERO WIDTH SPACE */
  ["\u200C", "\u200C"], /* ZERO WIDTH NON-JOINER */
  ["\u200D", "\u200D"], /* ZERO WIDTH JOINER */
  ["\u200E", "\u200E"], /* LEFT-TO-RIGHT MARK */
  ["\u200F", "\u200F"], /* RIGHT-TO-LEFT MARK */
  ["\u2010", "\u2010"], /* HYPHEN */
  ["\u2011", "\u2011"], /* NON-BREAKING HYPHEN */
  ["\u2012", "\u2012"], /* FIGURE DASH */
  ["\u2013", "\u2013"], /* EN DASH */
  ["\u2014", "\u2014"], /* EM DASH */
  ["\u2015", "\u2015"], /* HORIZONTAL BAR (QUOTATION DASH) */
  ["\u2016", "\u2016"], /* DOUBLE VERTICAL LINE (DOUBLE VERTICAL BAR) */
  ["\u2017", "\u2017"], /* DOUBLE LOW LINE (SPACING DOUBLE UNDERSCORE) */
  ["\u2018", "\u2018"], /* LEFT SINGLE QUOTATION MARK (SINGLE TURNED COMMA QUOTATION MARK) */
  ["\u2019", "\u2019"], /* RIGHT SINGLE QUOTATION MARK (SINGLE COMMA QUOTATION MARK) */
  ["\u201A", "\u201A"], /* SINGLE LOW-9 QUOTATION MARK (LOW SINGLE COMMA QUOTATION MARK) */
  ["\u201B", "\u201B"], /* SINGLE HIGH-REVERSED-9 QUOTATION MARK (SINGLE REVERSED COMMA QUOTATION MARK) */
  ["\u201C", "\u201C"], /* LEFT DOUBLE QUOTATION MARK (DOUBLE TURNED COMMA QUOTATION MARK) */
  ["\u201D", "\u201D"], /* RIGHT DOUBLE QUOTATION MARK (DOUBLE COMMA QUOTATION MARK) */
  ["\u201E", "\u201E"], /* DOUBLE LOW-9 QUOTATION MARK (LOW DOUBLE COMMA QUOTATION MARK) */
  ["\u201F", "\u201F"], /* DOUBLE HIGH-REVERSED-9 QUOTATION MARK (DOUBLE REVERSED COMMA QUOTATION MARK) */
  ["\u2020", "\u2020"], /* DAGGER */
  ["\u2021", "\u2021"], /* DOUBLE DAGGER */
  ["\u2022", "\u2022"], /* BULLET */
  ["\u2023", "\u2023"], /* TRIANGULAR BULLET */
  ["\u2024", "\u2024"], /* ONE DOT LEADER */
  ["\u2025", "\u2025"], /* TWO DOT LEADER */
  ["\u2026", "\u2026"], /* HORIZONTAL ELLIPSIS */
  ["\u2027", "\u2027"], /* HYPHENATION POINT */
  ["\u2028", "\u2028"], /* LINE SEPARATOR */
  ["\u2029", "\u2029"], /* PARAGRAPH SEPARATOR */
  ["\u202A", "\u202A"], /* LEFT-TO-RIGHT EMBEDDING */
  ["\u202B", "\u202B"], /* RIGHT-TO-LEFT EMBEDDING */
  ["\u202C", "\u202C"], /* POP DIRECTIONAL FORMATTING */
  ["\u202D", "\u202D"], /* LEFT-TO-RIGHT OVERRIDE */
  ["\u202E", "\u202E"], /* RIGHT-TO-LEFT OVERRIDE */
  ["\u202F", "\u202F"], /* NARROW NO-BREAK SPACE */
  ["\u2030", "\u2030"], /* PER MILLE SIGN */
  ["\u2031", "\u2031"], /* PER TEN THOUSAND SIGN */
  ["\u2032", "\u2032"], /* PRIME */
  ["\u2033", "\u2033"], /* DOUBLE PRIME */
  ["\u2034", "\u2034"], /* TRIPLE PRIME */
  ["\u2035", "\u2035"], /* REVERSED PRIME */
  ["\u2036", "\u2036"], /* REVERSED DOUBLE PRIME */
  ["\u2037", "\u2037"], /* REVERSED TRIPLE PRIME */
  ["\u2038", "\u2038"], /* CARET */
  ["\u2039", "\u2039"], /* SINGLE LEFT-POINTING ANGLE QUOTATION MARK (LEFT POINTING SINGLE GUILLEMET) */
  ["\u203A", "\u203A"], /* SINGLE RIGHT-POINTING ANGLE QUOTATION MARK (RIGHT POINTING SINGLE GUILLEMET) */
  ["\u203B", "\u203B"], /* REFERENCE MARK */
  ["\u203C", "\u203C"], /* DOUBLE EXCLAMATION MARK */
  ["\u203D", "\u203D"], /* INTERROBANG */
  ["\u203E", "\u203E"], /* OVERLINE (SPACING OVERSCORE) */
  ["\u203F", "\u203F"], /* UNDERTIE */
  ["\u2040", "\u2040"], /* CHARACTER TIE */
  ["\u2041", "\u2041"], /* CARET INSERTION POINT */
  ["\u2042", "\u2042"], /* ASTERISM */
  ["\u2043", "\u2043"], /* HYPHEN BULLET */
  ["\u2044", "\u2044"], /* FRACTION SLASH */
  ["\u2045", "\u2045"], /* LEFT SQUARE BRACKET WITH QUILL */
  ["\u2046", "\u2046"], /* RIGHT SQUARE BRACKET WITH QUILL */
  ["\u2047", "\u2047"], /* DOUBLE QUESTION MARK */
  ["\u2048", "\u2048"], /* QUESTION EXCLAMATION MARK */
  ["\u2049", "\u2049"], /* EXCLAMATION QUESTION MARK */
  ["\u204A", "\u204A"], /* TIRONIAN SIGN ET */
  ["\u204B", "\u204B"], /* REVERSED PILCROW SIGN */
  ["\u204C", "\u204C"], /* BLACK LEFTWARDS BULLET */
  ["\u204D", "\u204D"], /* BLACK RIGHTWARDS BULLET */
  ["\u204E", "\u204E"], /* LOW ASTERISK */
  ["\u204F", "\u204F"], /* REVERSED SEMICOLON */
  ["\u2050", "\u2050"], /* CLOSE UP */
  ["\u2051", "\u2051"], /* TWO ASTERISKS ALIGNED VERTICALLY */
  ["\u2052", "\u2052"], /* COMMERCIAL MINUS SIGN */
  ["\u2053", "\u2053"], /* SWUNG DASH */
  ["\u2054", "\u2054"], /* INVERTED UNDERTIE */
  ["\u2055", "\u2055"], /* FLOWER PUNCTUATION MARK */
  ["\u2056", "\u2056"], /* THREE DOT PUNCTUATION */
  ["\u2057", "\u2057"], /* QUADRUPLE PRIME */
  ["\u2058", "\u2058"], /* FOUR DOT PUNCTUATION */
  ["\u2059", "\u2059"], /* FIVE DOT PUNCTUATION */
  ["\u205A", "\u205A"], /* TWO DOT PUNCTUATION */
  ["\u205B", "\u205B"], /* FOUR DOT MARK */
  ["\u205C", "\u205C"], /* DOTTED CROSS */
  ["\u205D", "\u205D"], /* TRICOLON */
  ["\u205E", "\u205E"], /* VERTICAL FOUR DOTS */
  ["\u205F", "\u205F"], /* MEDIUM MATHEMATICAL SPACE */
  ["\u2060", "\u2060"], /* WORD JOINER */
  ["\u2061", "\u2061"], /* FUNCTION APPLICATION */
  ["\u2062", "\u2062"], /* INVISIBLE TIMES */
  ["\u2063", "\u2063"], /* INVISIBLE SEPARATOR */
  ["\u2064", "\u2064"], /* INVISIBLE PLUS */
  ["\u2065", "\u2065"],
  ["\u2066", "\u2066"], /* LEFT-TO-RIGHT ISOLATE */
  ["\u2067", "\u2067"], /* RIGHT-TO-LEFT ISOLATE */
  ["\u2068", "\u2068"], /* FIRST STRONG ISOLATE */
  ["\u2069", "\u2069"], /* POP DIRECTIONAL ISOLATE */
  ["\u206A", "\u206A"], /* INHIBIT SYMMETRIC SWAPPING */
  ["\u206B", "\u206B"], /* ACTIVATE SYMMETRIC SWAPPING */
  ["\u206C", "\u206C"], /* INHIBIT ARABIC FORM SHAPING */
  ["\u206D", "\u206D"], /* ACTIVATE ARABIC FORM SHAPING */
  ["\u206E", "\u206E"], /* NATIONAL DIGIT SHAPES */
  ["\u206F", "\u206F"], /* NOMINAL DIGIT SHAPES */
  ["\u2070", "\u2070"], /* SUPERSCRIPT ZERO (SUPERSCRIPT DIGIT ZERO) */
  ["\u2071", "\u2071"], /* SUPERSCRIPT LATIN SMALL LETTER I */
  ["\u2072", "\u2072"],
  ["\u2073", "\u2073"],
  ["\u2074", "\u2074"], /* SUPERSCRIPT FOUR (SUPERSCRIPT DIGIT FOUR) */
  ["\u2075", "\u2075"], /* SUPERSCRIPT FIVE (SUPERSCRIPT DIGIT FIVE) */
  ["\u2076", "\u2076"], /* SUPERSCRIPT SIX (SUPERSCRIPT DIGIT SIX) */
  ["\u2077", "\u2077"], /* SUPERSCRIPT SEVEN (SUPERSCRIPT DIGIT SEVEN) */
  ["\u2078", "\u2078"], /* SUPERSCRIPT EIGHT (SUPERSCRIPT DIGIT EIGHT) */
  ["\u2079", "\u2079"], /* SUPERSCRIPT NINE (SUPERSCRIPT DIGIT NINE) */
  ["\u207A", "\u207A"], /* SUPERSCRIPT PLUS SIGN */
  ["\u207B", "\u207B"], /* SUPERSCRIPT MINUS (SUPERSCRIPT HYPHEN-MINUS) */
  ["\u207C", "\u207C"], /* SUPERSCRIPT EQUALS SIGN */
  ["\u207D", "\u207D"], /* SUPERSCRIPT LEFT PARENTHESIS (SUPERSCRIPT OPENING PARENTHESIS) */
  ["\u207E", "\u207E"], /* SUPERSCRIPT RIGHT PARENTHESIS (SUPERSCRIPT CLOSING PARENTHESIS) */
  ["\u207F", "\u207F"], /* SUPERSCRIPT LATIN SMALL LETTER N */
  ["\u2080", "\u2080"], /* SUBSCRIPT ZERO (SUBSCRIPT DIGIT ZERO) */
  ["\u2081", "\u2081"], /* SUBSCRIPT ONE (SUBSCRIPT DIGIT ONE) */
  ["\u2082", "\u2082"], /* SUBSCRIPT TWO (SUBSCRIPT DIGIT TWO) */
  ["\u2083", "\u2083"], /* SUBSCRIPT THREE (SUBSCRIPT DIGIT THREE) */
  ["\u2084", "\u2084"], /* SUBSCRIPT FOUR (SUBSCRIPT DIGIT FOUR) */
  ["\u2085", "\u2085"], /* SUBSCRIPT FIVE (SUBSCRIPT DIGIT FIVE) */
  ["\u2086", "\u2086"], /* SUBSCRIPT SIX (SUBSCRIPT DIGIT SIX) */
  ["\u2087", "\u2087"], /* SUBSCRIPT SEVEN (SUBSCRIPT DIGIT SEVEN) */
  ["\u2088", "\u2088"], /* SUBSCRIPT EIGHT (SUBSCRIPT DIGIT EIGHT) */
  ["\u2089", "\u2089"], /* SUBSCRIPT NINE (SUBSCRIPT DIGIT NINE) */
  ["\u208A", "\u208A"], /* SUBSCRIPT PLUS SIGN */
  ["\u208B", "\u208B"], /* SUBSCRIPT MINUS (SUBSCRIPT HYPHEN-MINUS) */
  ["\u208C", "\u208C"], /* SUBSCRIPT EQUALS SIGN */
  ["\u208D", "\u208D"], /* SUBSCRIPT LEFT PARENTHESIS (SUBSCRIPT OPENING PARENTHESIS) */
  ["\u208E", "\u208E"], /* SUBSCRIPT RIGHT PARENTHESIS (SUBSCRIPT CLOSING PARENTHESIS) */
  ["\u208F", "\u208F"],
  ["\u2090", "\u2090"], /* LATIN SUBSCRIPT SMALL LETTER A */
  ["\u2091", "\u2091"], /* LATIN SUBSCRIPT SMALL LETTER E */
  ["\u2092", "\u2092"], /* LATIN SUBSCRIPT SMALL LETTER O */
  ["\u2093", "\u2093"], /* LATIN SUBSCRIPT SMALL LETTER X */
  ["\u2094", "\u2094"], /* LATIN SUBSCRIPT SMALL LETTER SCHWA */
  ["\u2095", "\u2095"], /* LATIN SUBSCRIPT SMALL LETTER H */
  ["\u2096", "\u2096"], /* LATIN SUBSCRIPT SMALL LETTER K */
  ["\u2097", "\u2097"], /* LATIN SUBSCRIPT SMALL LETTER L */
  ["\u2098", "\u2098"], /* LATIN SUBSCRIPT SMALL LETTER M */
  ["\u2099", "\u2099"], /* LATIN SUBSCRIPT SMALL LETTER N */
  ["\u209A", "\u209A"], /* LATIN SUBSCRIPT SMALL LETTER P */
  ["\u209B", "\u209B"], /* LATIN SUBSCRIPT SMALL LETTER S */
  ["\u209C", "\u209C"], /* LATIN SUBSCRIPT SMALL LETTER T */
  ["\u209D", "\u209D"],
  ["\u209E", "\u209E"],
  ["\u209F", "\u209F"],
  ["\u20A0", "\u20A0"], /* EURO-CURRENCY SIGN */
  ["\u20A1", "\u20A1"], /* COLON SIGN */
  ["\u20A2", "\u20A2"], /* CRUZEIRO SIGN */
  ["\u20A3", "\u20A3"], /* FRENCH FRANC SIGN */
  ["\u20A4", "\u20A4"], /* LIRA SIGN */
  ["\u20A5", "\u20A5"], /* MILL SIGN */
  ["\u20A6", "\u20A6"], /* NAIRA SIGN */
  ["\u20A7", "\u20A7"], /* PESETA SIGN */
  ["\u20A8", "\u20A8"], /* RUPEE SIGN */
  ["\u20A9", "\u20A9"], /* WON SIGN */
  ["\u20AA", "\u20AA"], /* NEW SHEQEL SIGN */
  ["\u20AB", "\u20AB"], /* DONG SIGN */
  ["\u20AC", "\u20AC"], /* EURO SIGN */
  ["\u20AD", "\u20AD"], /* KIP SIGN */
  ["\u20AE", "\u20AE"], /* TUGRIK SIGN */
  ["\u20AF", "\u20AF"], /* DRACHMA SIGN */
  ["\u20B0", "\u20B0"], /* GERMAN PENNY SIGN */
  ["\u20B1", "\u20B1"], /* PESO SIGN */
  ["\u20B2", "\u20B2"], /* GUARANI SIGN */
  ["\u20B3", "\u20B3"], /* AUSTRAL SIGN */
  ["\u20B4", "\u20B4"], /* HRYVNIA SIGN */
  ["\u20B5", "\u20B5"], /* CEDI SIGN */
  ["\u20B6", "\u20B6"], /* LIVRE TOURNOIS SIGN */
  ["\u20B7", "\u20B7"], /* SPESMILO SIGN */
  ["\u20B8", "\u20B8"], /* TENGE SIGN */
  ["\u20B9", "\u20B9"], /* INDIAN RUPEE SIGN */
  ["\u20BA", "\u20BA"], /* TURKISH LIRA SIGN */
  ["\u20BB", "\u20BB"], /* NORDIC MARK SIGN */
  ["\u20BC", "\u20BC"], /* MANAT SIGN */
  ["\u20BD", "\u20BD"], /* RUBLE SIGN */
  ["\u20BE", "\u20BE"], /* LARI SIGN */
  ["\u20BF", "\u20BF"], /* BITCOIN SIGN */
  ["\u20C0", "\u20C0"],
  ["\u20C1", "\u20C1"],
  ["\u20C2", "\u20C2"],
  ["\u20C3", "\u20C3"],
  ["\u20C4", "\u20C4"],
  ["\u20C5", "\u20C5"],
  ["\u20C6", "\u20C6"],
  ["\u20C7", "\u20C7"],
  ["\u20C8", "\u20C8"],
  ["\u20C9", "\u20C9"],
  ["\u20CA", "\u20CA"],
  ["\u20CB", "\u20CB"],
  ["\u20CC", "\u20CC"],
  ["\u20CD", "\u20CD"],
  ["\u20CE", "\u20CE"],
  ["\u20CF", "\u20CF"],
  ["\u20D0", "\u20D0"], /* COMBINING LEFT HARPOON ABOVE (NON-SPACING LEFT HARPOON ABOVE) */
  ["\u20D1", "\u20D1"], /* COMBINING RIGHT HARPOON ABOVE (NON-SPACING RIGHT HARPOON ABOVE) */
  ["\u20D2", "\u20D2"], /* COMBINING LONG VERTICAL LINE OVERLAY (NON-SPACING LONG VERTICAL BAR OVERLAY) */
  ["\u20D3", "\u20D3"], /* COMBINING SHORT VERTICAL LINE OVERLAY (NON-SPACING SHORT VERTICAL BAR OVERLAY) */
  ["\u20D4", "\u20D4"], /* COMBINING ANTICLOCKWISE ARROW ABOVE (NON-SPACING ANTICLOCKWISE ARROW ABOVE) */
  ["\u20D5", "\u20D5"], /* COMBINING CLOCKWISE ARROW ABOVE (NON-SPACING CLOCKWISE ARROW ABOVE) */
  ["\u20D6", "\u20D6"], /* COMBINING LEFT ARROW ABOVE (NON-SPACING LEFT ARROW ABOVE) */
  ["\u20D7", "\u20D7"], /* COMBINING RIGHT ARROW ABOVE (NON-SPACING RIGHT ARROW ABOVE) */
  ["\u20D8", "\u20D8"], /* COMBINING RING OVERLAY (NON-SPACING RING OVERLAY) */
  ["\u20D9", "\u20D9"], /* COMBINING CLOCKWISE RING OVERLAY (NON-SPACING CLOCKWISE RING OVERLAY) */
  ["\u20DA", "\u20DA"], /* COMBINING ANTICLOCKWISE RING OVERLAY (NON-SPACING ANTICLOCKWISE RING OVERLAY) */
  ["\u20DB", "\u20DB"], /* COMBINING THREE DOTS ABOVE (NON-SPACING THREE DOTS ABOVE) */
  ["\u20DC", "\u20DC"], /* COMBINING FOUR DOTS ABOVE (NON-SPACING FOUR DOTS ABOVE) */
  ["\u20DD", "\u20DD"], /* COMBINING ENCLOSING CIRCLE (ENCLOSING CIRCLE) */
  ["\u20DE", "\u20DE"], /* COMBINING ENCLOSING SQUARE (ENCLOSING SQUARE) */
  ["\u20DF", "\u20DF"], /* COMBINING ENCLOSING DIAMOND (ENCLOSING DIAMOND) */
  ["\u20E0", "\u20E0"], /* COMBINING ENCLOSING CIRCLE BACKSLASH (ENCLOSING CIRCLE SLASH) */
  ["\u20E1", "\u20E1"], /* COMBINING LEFT RIGHT ARROW ABOVE (NON-SPACING LEFT RIGHT ARROW ABOVE) */
  ["\u20E2", "\u20E2"], /* COMBINING ENCLOSING SCREEN */
  ["\u20E3", "\u20E3"], /* COMBINING ENCLOSING KEYCAP */
  ["\u20E4", "\u20E4"], /* COMBINING ENCLOSING UPWARD POINTING TRIANGLE */
  ["\u20E5", "\u20E5"], /* COMBINING REVERSE SOLIDUS OVERLAY */
  ["\u20E6", "\u20E6"], /* COMBINING DOUBLE VERTICAL STROKE OVERLAY */
  ["\u20E7", "\u20E7"], /* COMBINING ANNUITY SYMBOL */
  ["\u20E8", "\u20E8"], /* COMBINING TRIPLE UNDERDOT */
  ["\u20E9", "\u20E9"], /* COMBINING WIDE BRIDGE ABOVE */
  ["\u20EA", "\u20EA"], /* COMBINING LEFTWARDS ARROW OVERLAY */
  ["\u20EB", "\u20EB"], /* COMBINING LONG DOUBLE SOLIDUS OVERLAY */
  ["\u20EC", "\u20EC"], /* COMBINING RIGHTWARDS HARPOON WITH BARB DOWNWARDS */
  ["\u20ED", "\u20ED"], /* COMBINING LEFTWARDS HARPOON WITH BARB DOWNWARDS */
  ["\u20EE", "\u20EE"], /* COMBINING LEFT ARROW BELOW */
  ["\u20EF", "\u20EF"], /* COMBINING RIGHT ARROW BELOW */
  ["\u20F0", "\u20F0"], /* COMBINING ASTERISK ABOVE */
  ["\u20F1", "\u20F1"],
  ["\u20F2", "\u20F2"],
  ["\u20F3", "\u20F3"],
  ["\u20F4", "\u20F4"],
  ["\u20F5", "\u20F5"],
  ["\u20F6", "\u20F6"],
  ["\u20F7", "\u20F7"],
  ["\u20F8", "\u20F8"],
  ["\u20F9", "\u20F9"],
  ["\u20FA", "\u20FA"],
  ["\u20FB", "\u20FB"],
  ["\u20FC", "\u20FC"],
  ["\u20FD", "\u20FD"],
  ["\u20FE", "\u20FE"],
  ["\u20FF", "\u20FF"],
  ["\u2100", "\u2100"], /* ACCOUNT OF */
  ["\u2101", "\u2101"], /* ADDRESSED TO THE SUBJECT */
  ["\u2102", "\u2102"], /* DOUBLE-STRUCK CAPITAL C (DOUBLE-STRUCK C) */
  ["\u2103", "\u2103"], /* DEGREE CELSIUS (DEGREES CENTIGRADE) */
  ["\u2104", "\u2104"], /* CENTRE LINE SYMBOL (C L SYMBOL) */
  ["\u2105", "\u2105"], /* CARE OF */
  ["\u2106", "\u2106"], /* CADA UNA */
  ["\u2107", "\u2107"], /* EULER CONSTANT (EULERS) */
  ["\u2108", "\u2108"], /* SCRUPLE */
  ["\u2109", "\u2109"], /* DEGREE FAHRENHEIT (DEGREES FAHRENHEIT) */
  ["\u210A", "\u210A"], /* SCRIPT SMALL G */
  ["\u210B", "\u210B"], /* SCRIPT CAPITAL H (SCRIPT H) */
  ["\u210C", "\u210C"], /* BLACK-LETTER CAPITAL H (BLACK-LETTER H) */
  ["\u210D", "\u210D"], /* DOUBLE-STRUCK CAPITAL H (DOUBLE-STRUCK H) */
  ["\u210E", "\u210E"], /* PLANCK CONSTANT */
  ["\u210F", "\u210F"], /* PLANCK CONSTANT OVER TWO PI (PLANCK CONSTANT OVER 2 PI) */
  ["\u2110", "\u2110"], /* SCRIPT CAPITAL I (SCRIPT I) */
  ["\u2111", "\u2111"], /* BLACK-LETTER CAPITAL I (BLACK-LETTER I) */
  ["\u2112", "\u2112"], /* SCRIPT CAPITAL L (SCRIPT L) */
  ["\u2113", "\u2113"], /* SCRIPT SMALL L */
  ["\u2114", "\u2114"], /* L B BAR SYMBOL */
  ["\u2115", "\u2115"], /* DOUBLE-STRUCK CAPITAL N (DOUBLE-STRUCK N) */
  ["\u2116", "\u2116"], /* NUMERO SIGN (NUMERO) */
  ["\u2117", "\u2117"], /* SOUND RECORDING COPYRIGHT */
  ["\u2118", "\u2118"], /* SCRIPT CAPITAL P (SCRIPT P) */
  ["\u2119", "\u2119"], /* DOUBLE-STRUCK CAPITAL P (DOUBLE-STRUCK P) */
  ["\u211A", "\u211A"], /* DOUBLE-STRUCK CAPITAL Q (DOUBLE-STRUCK Q) */
  ["\u211B", "\u211B"], /* SCRIPT CAPITAL R (SCRIPT R) */
  ["\u211C", "\u211C"], /* BLACK-LETTER CAPITAL R (BLACK-LETTER R) */
  ["\u211D", "\u211D"], /* DOUBLE-STRUCK CAPITAL R (DOUBLE-STRUCK R) */
  ["\u211E", "\u211E"], /* PRESCRIPTION TAKE */
  ["\u211F", "\u211F"], /* RESPONSE */
  ["\u2120", "\u2120"], /* SERVICE MARK */
  ["\u2121", "\u2121"], /* TELEPHONE SIGN (T E L SYMBOL) */
  ["\u2122", "\u2122"], /* TRADE MARK SIGN (TRADEMARK) */
  ["\u2123", "\u2123"], /* VERSICLE */
  ["\u2124", "\u2124"], /* DOUBLE-STRUCK CAPITAL Z (DOUBLE-STRUCK Z) */
  ["\u2125", "\u2125"], /* OUNCE SIGN (OUNCE) */
  ["\u2126", "\u03C9"], /* OHM SIGN (OHM) */
  ["\u2127", "\u2127"], /* INVERTED OHM SIGN (MHO) */
  ["\u2128", "\u2128"], /* BLACK-LETTER CAPITAL Z (BLACK-LETTER Z) */
  ["\u2129", "\u2129"], /* TURNED GREEK SMALL LETTER IOTA */
  ["\u212A", "\u006B"], /* KELVIN SIGN (DEGREES KELVIN) */
  ["\u212B", "\u00E5"], /* ANGSTROM SIGN (ANGSTROM UNIT) */
  ["\u212C", "\u212C"], /* SCRIPT CAPITAL B (SCRIPT B) */
  ["\u212D", "\u212D"], /* BLACK-LETTER CAPITAL C (BLACK-LETTER C) */
  ["\u212E", "\u212E"], /* ESTIMATED SYMBOL */
  ["\u212F", "\u212F"], /* SCRIPT SMALL E */
  ["\u2130", "\u2130"], /* SCRIPT CAPITAL E (SCRIPT E) */
  ["\u2131", "\u2131"], /* SCRIPT CAPITAL F (SCRIPT F) */
  ["\u2132", "\u214E"], /* TURNED CAPITAL F (TURNED F) */
  ["\u2133", "\u2133"], /* SCRIPT CAPITAL M (SCRIPT M) */
  ["\u2134", "\u2134"], /* SCRIPT SMALL O */
  ["\u2135", "\u2135"], /* ALEF SYMBOL (FIRST TRANSFINITE CARDINAL) */
  ["\u2136", "\u2136"], /* BET SYMBOL (SECOND TRANSFINITE CARDINAL) */
  ["\u2137", "\u2137"], /* GIMEL SYMBOL (THIRD TRANSFINITE CARDINAL) */
  ["\u2138", "\u2138"], /* DALET SYMBOL (FOURTH TRANSFINITE CARDINAL) */
  ["\u2139", "\u2139"], /* INFORMATION SOURCE */
  ["\u213A", "\u213A"], /* ROTATED CAPITAL Q */
  ["\u213B", "\u213B"], /* FACSIMILE SIGN */
  ["\u213C", "\u213C"], /* DOUBLE-STRUCK SMALL PI */
  ["\u213D", "\u213D"], /* DOUBLE-STRUCK SMALL GAMMA */
  ["\u213E", "\u213E"], /* DOUBLE-STRUCK CAPITAL GAMMA */
  ["\u213F", "\u213F"], /* DOUBLE-STRUCK CAPITAL PI */
  ["\u2140", "\u2140"], /* DOUBLE-STRUCK N-ARY SUMMATION */
  ["\u2141", "\u2141"], /* TURNED SANS-SERIF CAPITAL G */
  ["\u2142", "\u2142"], /* TURNED SANS-SERIF CAPITAL L */
  ["\u2143", "\u2143"], /* REVERSED SANS-SERIF CAPITAL L */
  ["\u2144", "\u2144"], /* TURNED SANS-SERIF CAPITAL Y */
  ["\u2145", "\u2145"], /* DOUBLE-STRUCK ITALIC CAPITAL D */
  ["\u2146", "\u2146"], /* DOUBLE-STRUCK ITALIC SMALL D */
  ["\u2147", "\u2147"], /* DOUBLE-STRUCK ITALIC SMALL E */
  ["\u2148", "\u2148"], /* DOUBLE-STRUCK ITALIC SMALL I */
  ["\u2149", "\u2149"], /* DOUBLE-STRUCK ITALIC SMALL J */
  ["\u214A", "\u214A"], /* PROPERTY LINE */
  ["\u214B", "\u214B"], /* TURNED AMPERSAND */
  ["\u214C", "\u214C"], /* PER SIGN */
  ["\u214D", "\u214D"], /* AKTIESELSKAB */
  ["\u2132", "\u214E"], /* TURNED SMALL F */
  ["\u214F", "\u214F"], /* SYMBOL FOR SAMARITAN SOURCE */
  ["\u2150", "\u2150"], /* VULGAR FRACTION ONE SEVENTH */
  ["\u2151", "\u2151"], /* VULGAR FRACTION ONE NINTH */
  ["\u2152", "\u2152"], /* VULGAR FRACTION ONE TENTH */
  ["\u2153", "\u2153"], /* VULGAR FRACTION ONE THIRD (FRACTION ONE THIRD) */
  ["\u2154", "\u2154"], /* VULGAR FRACTION TWO THIRDS (FRACTION TWO THIRDS) */
  ["\u2155", "\u2155"], /* VULGAR FRACTION ONE FIFTH (FRACTION ONE FIFTH) */
  ["\u2156", "\u2156"], /* VULGAR FRACTION TWO FIFTHS (FRACTION TWO FIFTHS) */
  ["\u2157", "\u2157"], /* VULGAR FRACTION THREE FIFTHS (FRACTION THREE FIFTHS) */
  ["\u2158", "\u2158"], /* VULGAR FRACTION FOUR FIFTHS (FRACTION FOUR FIFTHS) */
  ["\u2159", "\u2159"], /* VULGAR FRACTION ONE SIXTH (FRACTION ONE SIXTH) */
  ["\u215A", "\u215A"], /* VULGAR FRACTION FIVE SIXTHS (FRACTION FIVE SIXTHS) */
  ["\u215B", "\u215B"], /* VULGAR FRACTION ONE EIGHTH (FRACTION ONE EIGHTH) */
  ["\u215C", "\u215C"], /* VULGAR FRACTION THREE EIGHTHS (FRACTION THREE EIGHTHS) */
  ["\u215D", "\u215D"], /* VULGAR FRACTION FIVE EIGHTHS (FRACTION FIVE EIGHTHS) */
  ["\u215E", "\u215E"], /* VULGAR FRACTION SEVEN EIGHTHS (FRACTION SEVEN EIGHTHS) */
  ["\u215F", "\u215F"], /* FRACTION NUMERATOR ONE */
  ["\u2160", "\u2170"], /* ROMAN NUMERAL ONE */
  ["\u2161", "\u2171"], /* ROMAN NUMERAL TWO */
  ["\u2162", "\u2172"], /* ROMAN NUMERAL THREE */
  ["\u2163", "\u2173"], /* ROMAN NUMERAL FOUR */
  ["\u2164", "\u2174"], /* ROMAN NUMERAL FIVE */
  ["\u2165", "\u2175"], /* ROMAN NUMERAL SIX */
  ["\u2166", "\u2176"], /* ROMAN NUMERAL SEVEN */
  ["\u2167", "\u2177"], /* ROMAN NUMERAL EIGHT */
  ["\u2168", "\u2178"], /* ROMAN NUMERAL NINE */
  ["\u2169", "\u2179"], /* ROMAN NUMERAL TEN */
  ["\u216A", "\u217A"], /* ROMAN NUMERAL ELEVEN */
  ["\u216B", "\u217B"], /* ROMAN NUMERAL TWELVE */
  ["\u216C", "\u217C"], /* ROMAN NUMERAL FIFTY */
  ["\u216D", "\u217D"], /* ROMAN NUMERAL ONE HUNDRED */
  ["\u216E", "\u217E"], /* ROMAN NUMERAL FIVE HUNDRED */
  ["\u216F", "\u217F"], /* ROMAN NUMERAL ONE THOUSAND */
  ["\u2160", "\u2170"], /* SMALL ROMAN NUMERAL ONE */
  ["\u2161", "\u2171"], /* SMALL ROMAN NUMERAL TWO */
  ["\u2162", "\u2172"], /* SMALL ROMAN NUMERAL THREE */
  ["\u2163", "\u2173"], /* SMALL ROMAN NUMERAL FOUR */
  ["\u2164", "\u2174"], /* SMALL ROMAN NUMERAL FIVE */
  ["\u2165", "\u2175"], /* SMALL ROMAN NUMERAL SIX */
  ["\u2166", "\u2176"], /* SMALL ROMAN NUMERAL SEVEN */
  ["\u2167", "\u2177"], /* SMALL ROMAN NUMERAL EIGHT */
  ["\u2168", "\u2178"], /* SMALL ROMAN NUMERAL NINE */
  ["\u2169", "\u2179"], /* SMALL ROMAN NUMERAL TEN */
  ["\u216A", "\u217A"], /* SMALL ROMAN NUMERAL ELEVEN */
  ["\u216B", "\u217B"], /* SMALL ROMAN NUMERAL TWELVE */
  ["\u216C", "\u217C"], /* SMALL ROMAN NUMERAL FIFTY */
  ["\u216D", "\u217D"], /* SMALL ROMAN NUMERAL ONE HUNDRED */
  ["\u216E", "\u217E"], /* SMALL ROMAN NUMERAL FIVE HUNDRED */
  ["\u216F", "\u217F"], /* SMALL ROMAN NUMERAL ONE THOUSAND */
  ["\u2180", "\u2180"], /* ROMAN NUMERAL ONE THOUSAND C D */
  ["\u2181", "\u2181"], /* ROMAN NUMERAL FIVE THOUSAND */
  ["\u2182", "\u2182"], /* ROMAN NUMERAL TEN THOUSAND */
  ["\u2183", "\u2184"], /* ROMAN NUMERAL REVERSED ONE HUNDRED */
  ["\u2183", "\u2184"], /* LATIN SMALL LETTER REVERSED C */
  ["\u2185", "\u2185"], /* ROMAN NUMERAL SIX LATE FORM */
  ["\u2186", "\u2186"], /* ROMAN NUMERAL FIFTY EARLY FORM */
  ["\u2187", "\u2187"], /* ROMAN NUMERAL FIFTY THOUSAND */
  ["\u2188", "\u2188"], /* ROMAN NUMERAL ONE HUNDRED THOUSAND */
  ["\u2189", "\u2189"], /* VULGAR FRACTION ZERO THIRDS */
  ["\u218A", "\u218A"], /* TURNED DIGIT TWO */
  ["\u218B", "\u218B"], /* TURNED DIGIT THREE */
  ["\u218C", "\u218C"],
  ["\u218D", "\u218D"],
  ["\u218E", "\u218E"],
  ["\u218F", "\u218F"],
  ["\u2190", "\u2190"], /* LEFTWARDS ARROW (LEFT ARROW) */
  ["\u2191", "\u2191"], /* UPWARDS ARROW (UP ARROW) */
  ["\u2192", "\u2192"], /* RIGHTWARDS ARROW (RIGHT ARROW) */
  ["\u2193", "\u2193"], /* DOWNWARDS ARROW (DOWN ARROW) */
  ["\u2194", "\u2194"], /* LEFT RIGHT ARROW */
  ["\u2195", "\u2195"], /* UP DOWN ARROW */
  ["\u2196", "\u2196"], /* NORTH WEST ARROW (UPPER LEFT ARROW) */
  ["\u2197", "\u2197"], /* NORTH EAST ARROW (UPPER RIGHT ARROW) */
  ["\u2198", "\u2198"], /* SOUTH EAST ARROW (LOWER RIGHT ARROW) */
  ["\u2199", "\u2199"], /* SOUTH WEST ARROW (LOWER LEFT ARROW) */
  ["\u219A", "\u219A"], /* LEFTWARDS ARROW WITH STROKE (LEFT ARROW WITH STROKE) */
  ["\u219B", "\u219B"], /* RIGHTWARDS ARROW WITH STROKE (RIGHT ARROW WITH STROKE) */
  ["\u219C", "\u219C"], /* LEFTWARDS WAVE ARROW (LEFT WAVE ARROW) */
  ["\u219D", "\u219D"], /* RIGHTWARDS WAVE ARROW (RIGHT WAVE ARROW) */
  ["\u219E", "\u219E"], /* LEFTWARDS TWO HEADED ARROW (LEFT TWO HEADED ARROW) */
  ["\u219F", "\u219F"], /* UPWARDS TWO HEADED ARROW (UP TWO HEADED ARROW) */
  ["\u21A0", "\u21A0"], /* RIGHTWARDS TWO HEADED ARROW (RIGHT TWO HEADED ARROW) */
  ["\u21A1", "\u21A1"], /* DOWNWARDS TWO HEADED ARROW (DOWN TWO HEADED ARROW) */
  ["\u21A2", "\u21A2"], /* LEFTWARDS ARROW WITH TAIL (LEFT ARROW WITH TAIL) */
  ["\u21A3", "\u21A3"], /* RIGHTWARDS ARROW WITH TAIL (RIGHT ARROW WITH TAIL) */
  ["\u21A4", "\u21A4"], /* LEFTWARDS ARROW FROM BAR (LEFT ARROW FROM BAR) */
  ["\u21A5", "\u21A5"], /* UPWARDS ARROW FROM BAR (UP ARROW FROM BAR) */
  ["\u21A6", "\u21A6"], /* RIGHTWARDS ARROW FROM BAR (RIGHT ARROW FROM BAR) */
  ["\u21A7", "\u21A7"], /* DOWNWARDS ARROW FROM BAR (DOWN ARROW FROM BAR) */
  ["\u21A8", "\u21A8"], /* UP DOWN ARROW WITH BASE */
  ["\u21A9", "\u21A9"], /* LEFTWARDS ARROW WITH HOOK (LEFT ARROW WITH HOOK) */
  ["\u21AA", "\u21AA"], /* RIGHTWARDS ARROW WITH HOOK (RIGHT ARROW WITH HOOK) */
  ["\u21AB", "\u21AB"], /* LEFTWARDS ARROW WITH LOOP (LEFT ARROW WITH LOOP) */
  ["\u21AC", "\u21AC"], /* RIGHTWARDS ARROW WITH LOOP (RIGHT ARROW WITH LOOP) */
  ["\u21AD", "\u21AD"], /* LEFT RIGHT WAVE ARROW */
  ["\u21AE", "\u21AE"], /* LEFT RIGHT ARROW WITH STROKE */
  ["\u21AF", "\u21AF"], /* DOWNWARDS ZIGZAG ARROW (DOWN ZIGZAG ARROW) */
  ["\u21B0", "\u21B0"], /* UPWARDS ARROW WITH TIP LEFTWARDS (UP ARROW WITH TIP LEFT) */
  ["\u21B1", "\u21B1"], /* UPWARDS ARROW WITH TIP RIGHTWARDS (UP ARROW WITH TIP RIGHT) */
  ["\u21B2", "\u21B2"], /* DOWNWARDS ARROW WITH TIP LEFTWARDS (DOWN ARROW WITH TIP LEFT) */
  ["\u21B3", "\u21B3"], /* DOWNWARDS ARROW WITH TIP RIGHTWARDS (DOWN ARROW WITH TIP RIGHT) */
  ["\u21B4", "\u21B4"], /* RIGHTWARDS ARROW WITH CORNER DOWNWARDS (RIGHT ARROW WITH CORNER DOWN) */
  ["\u21B5", "\u21B5"], /* DOWNWARDS ARROW WITH CORNER LEFTWARDS (DOWN ARROW WITH CORNER LEFT) */
  ["\u21B6", "\u21B6"], /* ANTICLOCKWISE TOP SEMICIRCLE ARROW */
  ["\u21B7", "\u21B7"], /* CLOCKWISE TOP SEMICIRCLE ARROW */
  ["\u21B8", "\u21B8"], /* NORTH WEST ARROW TO LONG BAR (UPPER LEFT ARROW TO LONG BAR) */
  ["\u21B9", "\u21B9"], /* LEFTWARDS ARROW TO BAR OVER RIGHTWARDS ARROW TO BAR (LEFT ARROW TO BAR OVER RIGHT ARROW TO BAR) */
  ["\u21BA", "\u21BA"], /* ANTICLOCKWISE OPEN CIRCLE ARROW */
  ["\u21BB", "\u21BB"], /* CLOCKWISE OPEN CIRCLE ARROW */
  ["\u21BC", "\u21BC"], /* LEFTWARDS HARPOON WITH BARB UPWARDS (LEFT HARPOON WITH BARB UP) */
  ["\u21BD", "\u21BD"], /* LEFTWARDS HARPOON WITH BARB DOWNWARDS (LEFT HARPOON WITH BARB DOWN) */
  ["\u21BE", "\u21BE"], /* UPWARDS HARPOON WITH BARB RIGHTWARDS (UP HARPOON WITH BARB RIGHT) */
  ["\u21BF", "\u21BF"], /* UPWARDS HARPOON WITH BARB LEFTWARDS (UP HARPOON WITH BARB LEFT) */
  ["\u21C0", "\u21C0"], /* RIGHTWARDS HARPOON WITH BARB UPWARDS (RIGHT HARPOON WITH BARB UP) */
  ["\u21C1", "\u21C1"], /* RIGHTWARDS HARPOON WITH BARB DOWNWARDS (RIGHT HARPOON WITH BARB DOWN) */
  ["\u21C2", "\u21C2"], /* DOWNWARDS HARPOON WITH BARB RIGHTWARDS (DOWN HARPOON WITH BARB RIGHT) */
  ["\u21C3", "\u21C3"], /* DOWNWARDS HARPOON WITH BARB LEFTWARDS (DOWN HARPOON WITH BARB LEFT) */
  ["\u21C4", "\u21C4"], /* RIGHTWARDS ARROW OVER LEFTWARDS ARROW (RIGHT ARROW OVER LEFT ARROW) */
  ["\u21C5", "\u21C5"], /* UPWARDS ARROW LEFTWARDS OF DOWNWARDS ARROW (UP ARROW LEFT OF DOWN ARROW) */
  ["\u21C6", "\u21C6"], /* LEFTWARDS ARROW OVER RIGHTWARDS ARROW (LEFT ARROW OVER RIGHT ARROW) */
  ["\u21C7", "\u21C7"], /* LEFTWARDS PAIRED ARROWS (LEFT PAIRED ARROWS) */
  ["\u21C8", "\u21C8"], /* UPWARDS PAIRED ARROWS (UP PAIRED ARROWS) */
  ["\u21C9", "\u21C9"], /* RIGHTWARDS PAIRED ARROWS (RIGHT PAIRED ARROWS) */
  ["\u21CA", "\u21CA"], /* DOWNWARDS PAIRED ARROWS (DOWN PAIRED ARROWS) */
  ["\u21CB", "\u21CB"], /* LEFTWARDS HARPOON OVER RIGHTWARDS HARPOON (LEFT HARPOON OVER RIGHT HARPOON) */
  ["\u21CC", "\u21CC"], /* RIGHTWARDS HARPOON OVER LEFTWARDS HARPOON (RIGHT HARPOON OVER LEFT HARPOON) */
  ["\u21CD", "\u21CD"], /* LEFTWARDS DOUBLE ARROW WITH STROKE (LEFT DOUBLE ARROW WITH STROKE) */
  ["\u21CE", "\u21CE"], /* LEFT RIGHT DOUBLE ARROW WITH STROKE */
  ["\u21CF", "\u21CF"], /* RIGHTWARDS DOUBLE ARROW WITH STROKE (RIGHT DOUBLE ARROW WITH STROKE) */
  ["\u21D0", "\u21D0"], /* LEFTWARDS DOUBLE ARROW (LEFT DOUBLE ARROW) */
  ["\u21D1", "\u21D1"], /* UPWARDS DOUBLE ARROW (UP DOUBLE ARROW) */
  ["\u21D2", "\u21D2"], /* RIGHTWARDS DOUBLE ARROW (RIGHT DOUBLE ARROW) */
  ["\u21D3", "\u21D3"], /* DOWNWARDS DOUBLE ARROW (DOWN DOUBLE ARROW) */
  ["\u21D4", "\u21D4"], /* LEFT RIGHT DOUBLE ARROW */
  ["\u21D5", "\u21D5"], /* UP DOWN DOUBLE ARROW */
  ["\u21D6", "\u21D6"], /* NORTH WEST DOUBLE ARROW (UPPER LEFT DOUBLE ARROW) */
  ["\u21D7", "\u21D7"], /* NORTH EAST DOUBLE ARROW (UPPER RIGHT DOUBLE ARROW) */
  ["\u21D8", "\u21D8"], /* SOUTH EAST DOUBLE ARROW (LOWER RIGHT DOUBLE ARROW) */
  ["\u21D9", "\u21D9"], /* SOUTH WEST DOUBLE ARROW (LOWER LEFT DOUBLE ARROW) */
  ["\u21DA", "\u21DA"], /* LEFTWARDS TRIPLE ARROW (LEFT TRIPLE ARROW) */
  ["\u21DB", "\u21DB"], /* RIGHTWARDS TRIPLE ARROW (RIGHT TRIPLE ARROW) */
  ["\u21DC", "\u21DC"], /* LEFTWARDS SQUIGGLE ARROW (LEFT SQUIGGLE ARROW) */
  ["\u21DD", "\u21DD"], /* RIGHTWARDS SQUIGGLE ARROW (RIGHT SQUIGGLE ARROW) */
  ["\u21DE", "\u21DE"], /* UPWARDS ARROW WITH DOUBLE STROKE (UP ARROW WITH DOUBLE STROKE) */
  ["\u21DF", "\u21DF"], /* DOWNWARDS ARROW WITH DOUBLE STROKE (DOWN ARROW WITH DOUBLE STROKE) */
  ["\u21E0", "\u21E0"], /* LEFTWARDS DASHED ARROW (LEFT DASHED ARROW) */
  ["\u21E1", "\u21E1"], /* UPWARDS DASHED ARROW (UP DASHED ARROW) */
  ["\u21E2", "\u21E2"], /* RIGHTWARDS DASHED ARROW (RIGHT DASHED ARROW) */
  ["\u21E3", "\u21E3"], /* DOWNWARDS DASHED ARROW (DOWN DASHED ARROW) */
  ["\u21E4", "\u21E4"], /* LEFTWARDS ARROW TO BAR (LEFT ARROW TO BAR) */
  ["\u21E5", "\u21E5"], /* RIGHTWARDS ARROW TO BAR (RIGHT ARROW TO BAR) */
  ["\u21E6", "\u21E6"], /* LEFTWARDS WHITE ARROW (WHITE LEFT ARROW) */
  ["\u21E7", "\u21E7"], /* UPWARDS WHITE ARROW (WHITE UP ARROW) */
  ["\u21E8", "\u21E8"], /* RIGHTWARDS WHITE ARROW (WHITE RIGHT ARROW) */
  ["\u21E9", "\u21E9"], /* DOWNWARDS WHITE ARROW (WHITE DOWN ARROW) */
  ["\u21EA", "\u21EA"], /* UPWARDS WHITE ARROW FROM BAR (WHITE UP ARROW FROM BAR) */
  ["\u21EB", "\u21EB"], /* UPWARDS WHITE ARROW ON PEDESTAL */
  ["\u21EC", "\u21EC"], /* UPWARDS WHITE ARROW ON PEDESTAL WITH HORIZONTAL BAR */
  ["\u21ED", "\u21ED"], /* UPWARDS WHITE ARROW ON PEDESTAL WITH VERTICAL BAR */
  ["\u21EE", "\u21EE"], /* UPWARDS WHITE DOUBLE ARROW */
  ["\u21EF", "\u21EF"], /* UPWARDS WHITE DOUBLE ARROW ON PEDESTAL */
  ["\u21F0", "\u21F0"], /* RIGHTWARDS WHITE ARROW FROM WALL */
  ["\u21F1", "\u21F1"], /* NORTH WEST ARROW TO CORNER */
  ["\u21F2", "\u21F2"], /* SOUTH EAST ARROW TO CORNER */
  ["\u21F3", "\u21F3"], /* UP DOWN WHITE ARROW */
  ["\u21F4", "\u21F4"], /* RIGHT ARROW WITH SMALL CIRCLE */
  ["\u21F5", "\u21F5"], /* DOWNWARDS ARROW LEFTWARDS OF UPWARDS ARROW */
  ["\u21F6", "\u21F6"], /* THREE RIGHTWARDS ARROWS */
  ["\u21F7", "\u21F7"], /* LEFTWARDS ARROW WITH VERTICAL STROKE */
  ["\u21F8", "\u21F8"], /* RIGHTWARDS ARROW WITH VERTICAL STROKE */
  ["\u21F9", "\u21F9"], /* LEFT RIGHT ARROW WITH VERTICAL STROKE */
  ["\u21FA", "\u21FA"], /* LEFTWARDS ARROW WITH DOUBLE VERTICAL STROKE */
  ["\u21FB", "\u21FB"], /* RIGHTWARDS ARROW WITH DOUBLE VERTICAL STROKE */
  ["\u21FC", "\u21FC"], /* LEFT RIGHT ARROW WITH DOUBLE VERTICAL STROKE */
  ["\u21FD", "\u21FD"], /* LEFTWARDS OPEN-HEADED ARROW */
  ["\u21FE", "\u21FE"], /* RIGHTWARDS OPEN-HEADED ARROW */
  ["\u21FF", "\u21FF"], /* LEFT RIGHT OPEN-HEADED ARROW */
  ["\u2200", "\u2200"], /* FOR ALL */
  ["\u2201", "\u2201"], /* COMPLEMENT */
  ["\u2202", "\u2202"], /* PARTIAL DIFFERENTIAL */
  ["\u2203", "\u2203"], /* THERE EXISTS */
  ["\u2204", "\u2204"], /* THERE DOES NOT EXIST */
  ["\u2205", "\u2205"], /* EMPTY SET */
  ["\u2206", "\u2206"], /* INCREMENT */
  ["\u2207", "\u2207"], /* NABLA */
  ["\u2208", "\u2208"], /* ELEMENT OF */
  ["\u2209", "\u2209"], /* NOT AN ELEMENT OF */
  ["\u220A", "\u220A"], /* SMALL ELEMENT OF */
  ["\u220B", "\u220B"], /* CONTAINS AS MEMBER */
  ["\u220C", "\u220C"], /* DOES NOT CONTAIN AS MEMBER */
  ["\u220D", "\u220D"], /* SMALL CONTAINS AS MEMBER */
  ["\u220E", "\u220E"], /* END OF PROOF */
  ["\u220F", "\u220F"], /* N-ARY PRODUCT */
  ["\u2210", "\u2210"], /* N-ARY COPRODUCT */
  ["\u2211", "\u2211"], /* N-ARY SUMMATION */
  ["\u2212", "\u2212"], /* MINUS SIGN */
  ["\u2213", "\u2213"], /* MINUS-OR-PLUS SIGN */
  ["\u2214", "\u2214"], /* DOT PLUS */
  ["\u2215", "\u2215"], /* DIVISION SLASH */
  ["\u2216", "\u2216"], /* SET MINUS */
  ["\u2217", "\u2217"], /* ASTERISK OPERATOR */
  ["\u2218", "\u2218"], /* RING OPERATOR */
  ["\u2219", "\u2219"], /* BULLET OPERATOR */
  ["\u221A", "\u221A"], /* SQUARE ROOT */
  ["\u221B", "\u221B"], /* CUBE ROOT */
  ["\u221C", "\u221C"], /* FOURTH ROOT */
  ["\u221D", "\u221D"], /* PROPORTIONAL TO */
  ["\u221E", "\u221E"], /* INFINITY */
  ["\u221F", "\u221F"], /* RIGHT ANGLE */
  ["\u2220", "\u2220"], /* ANGLE */
  ["\u2221", "\u2221"], /* MEASURED ANGLE */
  ["\u2222", "\u2222"], /* SPHERICAL ANGLE */
  ["\u2223", "\u2223"], /* DIVIDES */
  ["\u2224", "\u2224"], /* DOES NOT DIVIDE */
  ["\u2225", "\u2225"], /* PARALLEL TO */
  ["\u2226", "\u2226"], /* NOT PARALLEL TO */
  ["\u2227", "\u2227"], /* LOGICAL AND */
  ["\u2228", "\u2228"], /* LOGICAL OR */
  ["\u2229", "\u2229"], /* INTERSECTION */
  ["\u222A", "\u222A"], /* UNION */
  ["\u222B", "\u222B"], /* INTEGRAL */
  ["\u222C", "\u222C"], /* DOUBLE INTEGRAL */
  ["\u222D", "\u222D"], /* TRIPLE INTEGRAL */
  ["\u222E", "\u222E"], /* CONTOUR INTEGRAL */
  ["\u222F", "\u222F"], /* SURFACE INTEGRAL */
  ["\u2230", "\u2230"], /* VOLUME INTEGRAL */
  ["\u2231", "\u2231"], /* CLOCKWISE INTEGRAL */
  ["\u2232", "\u2232"], /* CLOCKWISE CONTOUR INTEGRAL */
  ["\u2233", "\u2233"], /* ANTICLOCKWISE CONTOUR INTEGRAL */
  ["\u2234", "\u2234"], /* THEREFORE */
  ["\u2235", "\u2235"], /* BECAUSE */
  ["\u2236", "\u2236"], /* RATIO */
  ["\u2237", "\u2237"], /* PROPORTION */
  ["\u2238", "\u2238"], /* DOT MINUS */
  ["\u2239", "\u2239"], /* EXCESS */
  ["\u223A", "\u223A"], /* GEOMETRIC PROPORTION */
  ["\u223B", "\u223B"], /* HOMOTHETIC */
  ["\u223C", "\u223C"], /* TILDE OPERATOR */
  ["\u223D", "\u223D"], /* REVERSED TILDE */
  ["\u223E", "\u223E"], /* INVERTED LAZY S */
  ["\u223F", "\u223F"], /* SINE WAVE */
  ["\u2240", "\u2240"], /* WREATH PRODUCT */
  ["\u2241", "\u2241"], /* NOT TILDE */
  ["\u2242", "\u2242"], /* MINUS TILDE */
  ["\u2243", "\u2243"], /* ASYMPTOTICALLY EQUAL TO */
  ["\u2244", "\u2244"], /* NOT ASYMPTOTICALLY EQUAL TO */
  ["\u2245", "\u2245"], /* APPROXIMATELY EQUAL TO */
  ["\u2246", "\u2246"], /* APPROXIMATELY BUT NOT ACTUALLY EQUAL TO */
  ["\u2247", "\u2247"], /* NEITHER APPROXIMATELY NOR ACTUALLY EQUAL TO */
  ["\u2248", "\u2248"], /* ALMOST EQUAL TO */
  ["\u2249", "\u2249"], /* NOT ALMOST EQUAL TO */
  ["\u224A", "\u224A"], /* ALMOST EQUAL OR EQUAL TO */
  ["\u224B", "\u224B"], /* TRIPLE TILDE */
  ["\u224C", "\u224C"], /* ALL EQUAL TO */
  ["\u224D", "\u224D"], /* EQUIVALENT TO */
  ["\u224E", "\u224E"], /* GEOMETRICALLY EQUIVALENT TO */
  ["\u224F", "\u224F"], /* DIFFERENCE BETWEEN */
  ["\u2250", "\u2250"], /* APPROACHES THE LIMIT */
  ["\u2251", "\u2251"], /* GEOMETRICALLY EQUAL TO */
  ["\u2252", "\u2252"], /* APPROXIMATELY EQUAL TO OR THE IMAGE OF */
  ["\u2253", "\u2253"], /* IMAGE OF OR APPROXIMATELY EQUAL TO */
  ["\u2254", "\u2254"], /* COLON EQUALS (COLON EQUAL) */
  ["\u2255", "\u2255"], /* EQUALS COLON (EQUAL COLON) */
  ["\u2256", "\u2256"], /* RING IN EQUAL TO */
  ["\u2257", "\u2257"], /* RING EQUAL TO */
  ["\u2258", "\u2258"], /* CORRESPONDS TO */
  ["\u2259", "\u2259"], /* ESTIMATES */
  ["\u225A", "\u225A"], /* EQUIANGULAR TO */
  ["\u225B", "\u225B"], /* STAR EQUALS */
  ["\u225C", "\u225C"], /* DELTA EQUAL TO */
  ["\u225D", "\u225D"], /* EQUAL TO BY DEFINITION */
  ["\u225E", "\u225E"], /* MEASURED BY */
  ["\u225F", "\u225F"], /* QUESTIONED EQUAL TO */
  ["\u2260", "\u2260"], /* NOT EQUAL TO */
  ["\u2261", "\u2261"], /* IDENTICAL TO */
  ["\u2262", "\u2262"], /* NOT IDENTICAL TO */
  ["\u2263", "\u2263"], /* STRICTLY EQUIVALENT TO */
  ["\u2264", "\u2264"], /* LESS-THAN OR EQUAL TO (LESS THAN OR EQUAL TO) */
  ["\u2265", "\u2265"], /* GREATER-THAN OR EQUAL TO (GREATER THAN OR EQUAL TO) */
  ["\u2266", "\u2266"], /* LESS-THAN OVER EQUAL TO (LESS THAN OVER EQUAL TO) */
  ["\u2267", "\u2267"], /* GREATER-THAN OVER EQUAL TO (GREATER THAN OVER EQUAL TO) */
  ["\u2268", "\u2268"], /* LESS-THAN BUT NOT EQUAL TO (LESS THAN BUT NOT EQUAL TO) */
  ["\u2269", "\u2269"], /* GREATER-THAN BUT NOT EQUAL TO (GREATER THAN BUT NOT EQUAL TO) */
  ["\u226A", "\u226A"], /* MUCH LESS-THAN (MUCH LESS THAN) */
  ["\u226B", "\u226B"], /* MUCH GREATER-THAN (MUCH GREATER THAN) */
  ["\u226C", "\u226C"], /* BETWEEN */
  ["\u226D", "\u226D"], /* NOT EQUIVALENT TO */
  ["\u226E", "\u226E"], /* NOT LESS-THAN (NOT LESS THAN) */
  ["\u226F", "\u226F"], /* NOT GREATER-THAN (NOT GREATER THAN) */
  ["\u2270", "\u2270"], /* NEITHER LESS-THAN NOR EQUAL TO (NEITHER LESS THAN NOR EQUAL TO) */
  ["\u2271", "\u2271"], /* NEITHER GREATER-THAN NOR EQUAL TO (NEITHER GREATER THAN NOR EQUAL TO) */
  ["\u2272", "\u2272"], /* LESS-THAN OR EQUIVALENT TO (LESS THAN OR EQUIVALENT TO) */
  ["\u2273", "\u2273"], /* GREATER-THAN OR EQUIVALENT TO (GREATER THAN OR EQUIVALENT TO) */
  ["\u2274", "\u2274"], /* NEITHER LESS-THAN NOR EQUIVALENT TO (NEITHER LESS THAN NOR EQUIVALENT TO) */
  ["\u2275", "\u2275"], /* NEITHER GREATER-THAN NOR EQUIVALENT TO (NEITHER GREATER THAN NOR EQUIVALENT TO) */
  ["\u2276", "\u2276"], /* LESS-THAN OR GREATER-THAN (LESS THAN OR GREATER THAN) */
  ["\u2277", "\u2277"], /* GREATER-THAN OR LESS-THAN (GREATER THAN OR LESS THAN) */
  ["\u2278", "\u2278"], /* NEITHER LESS-THAN NOR GREATER-THAN (NEITHER LESS THAN NOR GREATER THAN) */
  ["\u2279", "\u2279"], /* NEITHER GREATER-THAN NOR LESS-THAN (NEITHER GREATER THAN NOR LESS THAN) */
  ["\u227A", "\u227A"], /* PRECEDES */
  ["\u227B", "\u227B"], /* SUCCEEDS */
  ["\u227C", "\u227C"], /* PRECEDES OR EQUAL TO */
  ["\u227D", "\u227D"], /* SUCCEEDS OR EQUAL TO */
  ["\u227E", "\u227E"], /* PRECEDES OR EQUIVALENT TO */
  ["\u227F", "\u227F"], /* SUCCEEDS OR EQUIVALENT TO */
  ["\u2280", "\u2280"], /* DOES NOT PRECEDE */
  ["\u2281", "\u2281"], /* DOES NOT SUCCEED */
  ["\u2282", "\u2282"], /* SUBSET OF */
  ["\u2283", "\u2283"], /* SUPERSET OF */
  ["\u2284", "\u2284"], /* NOT A SUBSET OF */
  ["\u2285", "\u2285"], /* NOT A SUPERSET OF */
  ["\u2286", "\u2286"], /* SUBSET OF OR EQUAL TO */
  ["\u2287", "\u2287"], /* SUPERSET OF OR EQUAL TO */
  ["\u2288", "\u2288"], /* NEITHER A SUBSET OF NOR EQUAL TO */
  ["\u2289", "\u2289"], /* NEITHER A SUPERSET OF NOR EQUAL TO */
  ["\u228A", "\u228A"], /* SUBSET OF WITH NOT EQUAL TO (SUBSET OF OR NOT EQUAL TO) */
  ["\u228B", "\u228B"], /* SUPERSET OF WITH NOT EQUAL TO (SUPERSET OF OR NOT EQUAL TO) */
  ["\u228C", "\u228C"], /* MULTISET */
  ["\u228D", "\u228D"], /* MULTISET MULTIPLICATION */
  ["\u228E", "\u228E"], /* MULTISET UNION */
  ["\u228F", "\u228F"], /* SQUARE IMAGE OF */
  ["\u2290", "\u2290"], /* SQUARE ORIGINAL OF */
  ["\u2291", "\u2291"], /* SQUARE IMAGE OF OR EQUAL TO */
  ["\u2292", "\u2292"], /* SQUARE ORIGINAL OF OR EQUAL TO */
  ["\u2293", "\u2293"], /* SQUARE CAP */
  ["\u2294", "\u2294"], /* SQUARE CUP */
  ["\u2295", "\u2295"], /* CIRCLED PLUS */
  ["\u2296", "\u2296"], /* CIRCLED MINUS */
  ["\u2297", "\u2297"], /* CIRCLED TIMES */
  ["\u2298", "\u2298"], /* CIRCLED DIVISION SLASH */
  ["\u2299", "\u2299"], /* CIRCLED DOT OPERATOR */
  ["\u229A", "\u229A"], /* CIRCLED RING OPERATOR */
  ["\u229B", "\u229B"], /* CIRCLED ASTERISK OPERATOR */
  ["\u229C", "\u229C"], /* CIRCLED EQUALS */
  ["\u229D", "\u229D"], /* CIRCLED DASH */
  ["\u229E", "\u229E"], /* SQUARED PLUS */
  ["\u229F", "\u229F"], /* SQUARED MINUS */
  ["\u22A0", "\u22A0"], /* SQUARED TIMES */
  ["\u22A1", "\u22A1"], /* SQUARED DOT OPERATOR */
  ["\u22A2", "\u22A2"], /* RIGHT TACK */
  ["\u22A3", "\u22A3"], /* LEFT TACK */
  ["\u22A4", "\u22A4"], /* DOWN TACK */
  ["\u22A5", "\u22A5"], /* UP TACK */
  ["\u22A6", "\u22A6"], /* ASSERTION */
  ["\u22A7", "\u22A7"], /* MODELS */
  ["\u22A8", "\u22A8"], /* TRUE */
  ["\u22A9", "\u22A9"], /* FORCES */
  ["\u22AA", "\u22AA"], /* TRIPLE VERTICAL BAR RIGHT TURNSTILE */
  ["\u22AB", "\u22AB"], /* DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE */
  ["\u22AC", "\u22AC"], /* DOES NOT PROVE */
  ["\u22AD", "\u22AD"], /* NOT TRUE */
  ["\u22AE", "\u22AE"], /* DOES NOT FORCE */
  ["\u22AF", "\u22AF"], /* NEGATED DOUBLE VERTICAL BAR DOUBLE RIGHT TURNSTILE */
  ["\u22B0", "\u22B0"], /* PRECEDES UNDER RELATION */
  ["\u22B1", "\u22B1"], /* SUCCEEDS UNDER RELATION */
  ["\u22B2", "\u22B2"], /* NORMAL SUBGROUP OF */
  ["\u22B3", "\u22B3"], /* CONTAINS AS NORMAL SUBGROUP */
  ["\u22B4", "\u22B4"], /* NORMAL SUBGROUP OF OR EQUAL TO */
  ["\u22B5", "\u22B5"], /* CONTAINS AS NORMAL SUBGROUP OR EQUAL TO */
  ["\u22B6", "\u22B6"], /* ORIGINAL OF */
  ["\u22B7", "\u22B7"], /* IMAGE OF */
  ["\u22B8", "\u22B8"], /* MULTIMAP */
  ["\u22B9", "\u22B9"], /* HERMITIAN CONJUGATE MATRIX */
  ["\u22BA", "\u22BA"], /* INTERCALATE */
  ["\u22BB", "\u22BB"], /* XOR */
  ["\u22BC", "\u22BC"], /* NAND */
  ["\u22BD", "\u22BD"], /* NOR */
  ["\u22BE", "\u22BE"], /* RIGHT ANGLE WITH ARC */
  ["\u22BF", "\u22BF"], /* RIGHT TRIANGLE */
  ["\u22C0", "\u22C0"], /* N-ARY LOGICAL AND */
  ["\u22C1", "\u22C1"], /* N-ARY LOGICAL OR */
  ["\u22C2", "\u22C2"], /* N-ARY INTERSECTION */
  ["\u22C3", "\u22C3"], /* N-ARY UNION */
  ["\u22C4", "\u22C4"], /* DIAMOND OPERATOR */
  ["\u22C5", "\u22C5"], /* DOT OPERATOR */
  ["\u22C6", "\u22C6"], /* STAR OPERATOR */
  ["\u22C7", "\u22C7"], /* DIVISION TIMES */
  ["\u22C8", "\u22C8"], /* BOWTIE */
  ["\u22C9", "\u22C9"], /* LEFT NORMAL FACTOR SEMIDIRECT PRODUCT */
  ["\u22CA", "\u22CA"], /* RIGHT NORMAL FACTOR SEMIDIRECT PRODUCT */
  ["\u22CB", "\u22CB"], /* LEFT SEMIDIRECT PRODUCT */
  ["\u22CC", "\u22CC"], /* RIGHT SEMIDIRECT PRODUCT */
  ["\u22CD", "\u22CD"], /* REVERSED TILDE EQUALS */
  ["\u22CE", "\u22CE"], /* CURLY LOGICAL OR */
  ["\u22CF", "\u22CF"], /* CURLY LOGICAL AND */
  ["\u22D0", "\u22D0"], /* DOUBLE SUBSET */
  ["\u22D1", "\u22D1"], /* DOUBLE SUPERSET */
  ["\u22D2", "\u22D2"], /* DOUBLE INTERSECTION */
  ["\u22D3", "\u22D3"], /* DOUBLE UNION */
  ["\u22D4", "\u22D4"], /* PITCHFORK */
  ["\u22D5", "\u22D5"], /* EQUAL AND PARALLEL TO */
  ["\u22D6", "\u22D6"], /* LESS-THAN WITH DOT (LESS THAN WITH DOT) */
  ["\u22D7", "\u22D7"], /* GREATER-THAN WITH DOT (GREATER THAN WITH DOT) */
  ["\u22D8", "\u22D8"], /* VERY MUCH LESS-THAN (VERY MUCH LESS THAN) */
  ["\u22D9", "\u22D9"], /* VERY MUCH GREATER-THAN (VERY MUCH GREATER THAN) */
  ["\u22DA", "\u22DA"], /* LESS-THAN EQUAL TO OR GREATER-THAN (LESS THAN EQUAL TO OR GREATER THAN) */
  ["\u22DB", "\u22DB"], /* GREATER-THAN EQUAL TO OR LESS-THAN (GREATER THAN EQUAL TO OR LESS THAN) */
  ["\u22DC", "\u22DC"], /* EQUAL TO OR LESS-THAN (EQUAL TO OR LESS THAN) */
  ["\u22DD", "\u22DD"], /* EQUAL TO OR GREATER-THAN (EQUAL TO OR GREATER THAN) */
  ["\u22DE", "\u22DE"], /* EQUAL TO OR PRECEDES */
  ["\u22DF", "\u22DF"], /* EQUAL TO OR SUCCEEDS */
  ["\u22E0", "\u22E0"], /* DOES NOT PRECEDE OR EQUAL */
  ["\u22E1", "\u22E1"], /* DOES NOT SUCCEED OR EQUAL */
  ["\u22E2", "\u22E2"], /* NOT SQUARE IMAGE OF OR EQUAL TO */
  ["\u22E3", "\u22E3"], /* NOT SQUARE ORIGINAL OF OR EQUAL TO */
  ["\u22E4", "\u22E4"], /* SQUARE IMAGE OF OR NOT EQUAL TO */
  ["\u22E5", "\u22E5"], /* SQUARE ORIGINAL OF OR NOT EQUAL TO */
  ["\u22E6", "\u22E6"], /* LESS-THAN BUT NOT EQUIVALENT TO (LESS THAN BUT NOT EQUIVALENT TO) */
  ["\u22E7", "\u22E7"], /* GREATER-THAN BUT NOT EQUIVALENT TO (GREATER THAN BUT NOT EQUIVALENT TO) */
  ["\u22E8", "\u22E8"], /* PRECEDES BUT NOT EQUIVALENT TO */
  ["\u22E9", "\u22E9"], /* SUCCEEDS BUT NOT EQUIVALENT TO */
  ["\u22EA", "\u22EA"], /* NOT NORMAL SUBGROUP OF */
  ["\u22EB", "\u22EB"], /* DOES NOT CONTAIN AS NORMAL SUBGROUP */
  ["\u22EC", "\u22EC"], /* NOT NORMAL SUBGROUP OF OR EQUAL TO */
  ["\u22ED", "\u22ED"], /* DOES NOT CONTAIN AS NORMAL SUBGROUP OR EQUAL */
  ["\u22EE", "\u22EE"], /* VERTICAL ELLIPSIS */
  ["\u22EF", "\u22EF"], /* MIDLINE HORIZONTAL ELLIPSIS */
  ["\u22F0", "\u22F0"], /* UP RIGHT DIAGONAL ELLIPSIS */
  ["\u22F1", "\u22F1"], /* DOWN RIGHT DIAGONAL ELLIPSIS */
  ["\u22F2", "\u22F2"], /* ELEMENT OF WITH LONG HORIZONTAL STROKE */
  ["\u22F3", "\u22F3"], /* ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE */
  ["\u22F4", "\u22F4"], /* SMALL ELEMENT OF WITH VERTICAL BAR AT END OF HORIZONTAL STROKE */
  ["\u22F5", "\u22F5"], /* ELEMENT OF WITH DOT ABOVE */
  ["\u22F6", "\u22F6"], /* ELEMENT OF WITH OVERBAR */
  ["\u22F7", "\u22F7"], /* SMALL ELEMENT OF WITH OVERBAR */
  ["\u22F8", "\u22F8"], /* ELEMENT OF WITH UNDERBAR */
  ["\u22F9", "\u22F9"], /* ELEMENT OF WITH TWO HORIZONTAL STROKES */
  ["\u22FA", "\u22FA"], /* CONTAINS WITH LONG HORIZONTAL STROKE */
  ["\u22FB", "\u22FB"], /* CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE */
  ["\u22FC", "\u22FC"], /* SMALL CONTAINS WITH VERTICAL BAR AT END OF HORIZONTAL STROKE */
  ["\u22FD", "\u22FD"], /* CONTAINS WITH OVERBAR */
  ["\u22FE", "\u22FE"], /* SMALL CONTAINS WITH OVERBAR */
  ["\u22FF", "\u22FF"], /* Z NOTATION BAG MEMBERSHIP */
  ["\u2300", "\u2300"], /* DIAMETER SIGN */
  ["\u2301", "\u2301"], /* ELECTRIC ARROW */
  ["\u2302", "\u2302"], /* HOUSE */
  ["\u2303", "\u2303"], /* UP ARROWHEAD */
  ["\u2304", "\u2304"], /* DOWN ARROWHEAD */
  ["\u2305", "\u2305"], /* PROJECTIVE */
  ["\u2306", "\u2306"], /* PERSPECTIVE */
  ["\u2307", "\u2307"], /* WAVY LINE */
  ["\u2308", "\u2308"], /* LEFT CEILING */
  ["\u2309", "\u2309"], /* RIGHT CEILING */
  ["\u230A", "\u230A"], /* LEFT FLOOR */
  ["\u230B", "\u230B"], /* RIGHT FLOOR */
  ["\u230C", "\u230C"], /* BOTTOM RIGHT CROP */
  ["\u230D", "\u230D"], /* BOTTOM LEFT CROP */
  ["\u230E", "\u230E"], /* TOP RIGHT CROP */
  ["\u230F", "\u230F"], /* TOP LEFT CROP */
  ["\u2310", "\u2310"], /* REVERSED NOT SIGN */
  ["\u2311", "\u2311"], /* SQUARE LOZENGE */
  ["\u2312", "\u2312"], /* ARC */
  ["\u2313", "\u2313"], /* SEGMENT */
  ["\u2314", "\u2314"], /* SECTOR */
  ["\u2315", "\u2315"], /* TELEPHONE RECORDER */
  ["\u2316", "\u2316"], /* POSITION INDICATOR */
  ["\u2317", "\u2317"], /* VIEWDATA SQUARE */
  ["\u2318", "\u2318"], /* PLACE OF INTEREST SIGN (COMMAND KEY) */
  ["\u2319", "\u2319"], /* TURNED NOT SIGN */
  ["\u231A", "\u231A"], /* WATCH */
  ["\u231B", "\u231B"], /* HOURGLASS */
  ["\u231C", "\u231C"], /* TOP LEFT CORNER */
  ["\u231D", "\u231D"], /* TOP RIGHT CORNER */
  ["\u231E", "\u231E"], /* BOTTOM LEFT CORNER */
  ["\u231F", "\u231F"], /* BOTTOM RIGHT CORNER */
  ["\u2320", "\u2320"], /* TOP HALF INTEGRAL */
  ["\u2321", "\u2321"], /* BOTTOM HALF INTEGRAL */
  ["\u2322", "\u2322"], /* FROWN */
  ["\u2323", "\u2323"], /* SMILE */
  ["\u2324", "\u2324"], /* UP ARROWHEAD BETWEEN TWO HORIZONTAL BARS (ENTER KEY) */
  ["\u2325", "\u2325"], /* OPTION KEY */
  ["\u2326", "\u2326"], /* ERASE TO THE RIGHT (DELETE TO THE RIGHT KEY) */
  ["\u2327", "\u2327"], /* X IN A RECTANGLE BOX (CLEAR KEY) */
  ["\u2328", "\u2328"], /* KEYBOARD */
  ["\u2329", "\u2329"], /* LEFT-POINTING ANGLE BRACKET (BRA) */
  ["\u232A", "\u232A"], /* RIGHT-POINTING ANGLE BRACKET (KET) */
  ["\u232B", "\u232B"], /* ERASE TO THE LEFT (DELETE TO THE LEFT KEY) */
  ["\u232C", "\u232C"], /* BENZENE RING */
  ["\u232D", "\u232D"], /* CYLINDRICITY */
  ["\u232E", "\u232E"], /* ALL AROUND-PROFILE */
  ["\u232F", "\u232F"], /* SYMMETRY */
  ["\u2330", "\u2330"], /* TOTAL RUNOUT */
  ["\u2331", "\u2331"], /* DIMENSION ORIGIN */
  ["\u2332", "\u2332"], /* CONICAL TAPER */
  ["\u2333", "\u2333"], /* SLOPE */
  ["\u2334", "\u2334"], /* COUNTERBORE */
  ["\u2335", "\u2335"], /* COUNTERSINK */
  ["\u2336", "\u2336"], /* APL FUNCTIONAL SYMBOL I-BEAM */
  ["\u2337", "\u2337"], /* APL FUNCTIONAL SYMBOL SQUISH QUAD */
  ["\u2338", "\u2338"], /* APL FUNCTIONAL SYMBOL QUAD EQUAL */
  ["\u2339", "\u2339"], /* APL FUNCTIONAL SYMBOL QUAD DIVIDE */
  ["\u233A", "\u233A"], /* APL FUNCTIONAL SYMBOL QUAD DIAMOND */
  ["\u233B", "\u233B"], /* APL FUNCTIONAL SYMBOL QUAD JOT */
  ["\u233C", "\u233C"], /* APL FUNCTIONAL SYMBOL QUAD CIRCLE */
  ["\u233D", "\u233D"], /* APL FUNCTIONAL SYMBOL CIRCLE STILE */
  ["\u233E", "\u233E"], /* APL FUNCTIONAL SYMBOL CIRCLE JOT */
  ["\u233F", "\u233F"], /* APL FUNCTIONAL SYMBOL SLASH BAR */
  ["\u2340", "\u2340"], /* APL FUNCTIONAL SYMBOL BACKSLASH BAR */
  ["\u2341", "\u2341"], /* APL FUNCTIONAL SYMBOL QUAD SLASH */
  ["\u2342", "\u2342"], /* APL FUNCTIONAL SYMBOL QUAD BACKSLASH */
  ["\u2343", "\u2343"], /* APL FUNCTIONAL SYMBOL QUAD LESS-THAN */
  ["\u2344", "\u2344"], /* APL FUNCTIONAL SYMBOL QUAD GREATER-THAN */
  ["\u2345", "\u2345"], /* APL FUNCTIONAL SYMBOL LEFTWARDS VANE */
  ["\u2346", "\u2346"], /* APL FUNCTIONAL SYMBOL RIGHTWARDS VANE */
  ["\u2347", "\u2347"], /* APL FUNCTIONAL SYMBOL QUAD LEFTWARDS ARROW */
  ["\u2348", "\u2348"], /* APL FUNCTIONAL SYMBOL QUAD RIGHTWARDS ARROW */
  ["\u2349", "\u2349"], /* APL FUNCTIONAL SYMBOL CIRCLE BACKSLASH */
  ["\u234A", "\u234A"], /* APL FUNCTIONAL SYMBOL DOWN TACK UNDERBAR */
  ["\u234B", "\u234B"], /* APL FUNCTIONAL SYMBOL DELTA STILE */
  ["\u234C", "\u234C"], /* APL FUNCTIONAL SYMBOL QUAD DOWN CARET */
  ["\u234D", "\u234D"], /* APL FUNCTIONAL SYMBOL QUAD DELTA */
  ["\u234E", "\u234E"], /* APL FUNCTIONAL SYMBOL DOWN TACK JOT */
  ["\u234F", "\u234F"], /* APL FUNCTIONAL SYMBOL UPWARDS VANE */
  ["\u2350", "\u2350"], /* APL FUNCTIONAL SYMBOL QUAD UPWARDS ARROW */
  ["\u2351", "\u2351"], /* APL FUNCTIONAL SYMBOL UP TACK OVERBAR */
  ["\u2352", "\u2352"], /* APL FUNCTIONAL SYMBOL DEL STILE */
  ["\u2353", "\u2353"], /* APL FUNCTIONAL SYMBOL QUAD UP CARET */
  ["\u2354", "\u2354"], /* APL FUNCTIONAL SYMBOL QUAD DEL */
  ["\u2355", "\u2355"], /* APL FUNCTIONAL SYMBOL UP TACK JOT */
  ["\u2356", "\u2356"], /* APL FUNCTIONAL SYMBOL DOWNWARDS VANE */
  ["\u2357", "\u2357"], /* APL FUNCTIONAL SYMBOL QUAD DOWNWARDS ARROW */
  ["\u2358", "\u2358"], /* APL FUNCTIONAL SYMBOL QUOTE UNDERBAR */
  ["\u2359", "\u2359"], /* APL FUNCTIONAL SYMBOL DELTA UNDERBAR */
  ["\u235A", "\u235A"], /* APL FUNCTIONAL SYMBOL DIAMOND UNDERBAR */
  ["\u235B", "\u235B"], /* APL FUNCTIONAL SYMBOL JOT UNDERBAR */
  ["\u235C", "\u235C"], /* APL FUNCTIONAL SYMBOL CIRCLE UNDERBAR */
  ["\u235D", "\u235D"], /* APL FUNCTIONAL SYMBOL UP SHOE JOT */
  ["\u235E", "\u235E"], /* APL FUNCTIONAL SYMBOL QUOTE QUAD */
  ["\u235F", "\u235F"], /* APL FUNCTIONAL SYMBOL CIRCLE STAR */
  ["\u2360", "\u2360"], /* APL FUNCTIONAL SYMBOL QUAD COLON */
  ["\u2361", "\u2361"], /* APL FUNCTIONAL SYMBOL UP TACK DIAERESIS */
  ["\u2362", "\u2362"], /* APL FUNCTIONAL SYMBOL DEL DIAERESIS */
  ["\u2363", "\u2363"], /* APL FUNCTIONAL SYMBOL STAR DIAERESIS */
  ["\u2364", "\u2364"], /* APL FUNCTIONAL SYMBOL JOT DIAERESIS */
  ["\u2365", "\u2365"], /* APL FUNCTIONAL SYMBOL CIRCLE DIAERESIS */
  ["\u2366", "\u2366"], /* APL FUNCTIONAL SYMBOL DOWN SHOE STILE */
  ["\u2367", "\u2367"], /* APL FUNCTIONAL SYMBOL LEFT SHOE STILE */
  ["\u2368", "\u2368"], /* APL FUNCTIONAL SYMBOL TILDE DIAERESIS */
  ["\u2369", "\u2369"], /* APL FUNCTIONAL SYMBOL GREATER-THAN DIAERESIS */
  ["\u236A", "\u236A"], /* APL FUNCTIONAL SYMBOL COMMA BAR */
  ["\u236B", "\u236B"], /* APL FUNCTIONAL SYMBOL DEL TILDE */
  ["\u236C", "\u236C"], /* APL FUNCTIONAL SYMBOL ZILDE */
  ["\u236D", "\u236D"], /* APL FUNCTIONAL SYMBOL STILE TILDE */
  ["\u236E", "\u236E"], /* APL FUNCTIONAL SYMBOL SEMICOLON UNDERBAR */
  ["\u236F", "\u236F"], /* APL FUNCTIONAL SYMBOL QUAD NOT EQUAL */
  ["\u2370", "\u2370"], /* APL FUNCTIONAL SYMBOL QUAD QUESTION */
  ["\u2371", "\u2371"], /* APL FUNCTIONAL SYMBOL DOWN CARET TILDE */
  ["\u2372", "\u2372"], /* APL FUNCTIONAL SYMBOL UP CARET TILDE */
  ["\u2373", "\u2373"], /* APL FUNCTIONAL SYMBOL IOTA */
  ["\u2374", "\u2374"], /* APL FUNCTIONAL SYMBOL RHO */
  ["\u2375", "\u2375"], /* APL FUNCTIONAL SYMBOL OMEGA */
  ["\u2376", "\u2376"], /* APL FUNCTIONAL SYMBOL ALPHA UNDERBAR */
  ["\u2377", "\u2377"], /* APL FUNCTIONAL SYMBOL EPSILON UNDERBAR */
  ["\u2378", "\u2378"], /* APL FUNCTIONAL SYMBOL IOTA UNDERBAR */
  ["\u2379", "\u2379"], /* APL FUNCTIONAL SYMBOL OMEGA UNDERBAR */
  ["\u237A", "\u237A"], /* APL FUNCTIONAL SYMBOL ALPHA */
  ["\u237B", "\u237B"], /* NOT CHECK MARK */
  ["\u237C", "\u237C"], /* RIGHT ANGLE WITH DOWNWARDS ZIGZAG ARROW */
  ["\u237D", "\u237D"], /* SHOULDERED OPEN BOX */
  ["\u237E", "\u237E"], /* BELL SYMBOL */
  ["\u237F", "\u237F"], /* VERTICAL LINE WITH MIDDLE DOT */
  ["\u2380", "\u2380"], /* INSERTION SYMBOL */
  ["\u2381", "\u2381"], /* CONTINUOUS UNDERLINE SYMBOL */
  ["\u2382", "\u2382"], /* DISCONTINUOUS UNDERLINE SYMBOL */
  ["\u2383", "\u2383"], /* EMPHASIS SYMBOL */
  ["\u2384", "\u2384"], /* COMPOSITION SYMBOL */
  ["\u2385", "\u2385"], /* WHITE SQUARE WITH CENTRE VERTICAL LINE */
  ["\u2386", "\u2386"], /* ENTER SYMBOL */
  ["\u2387", "\u2387"], /* ALTERNATIVE KEY SYMBOL */
  ["\u2388", "\u2388"], /* HELM SYMBOL */
  ["\u2389", "\u2389"], /* CIRCLED HORIZONTAL BAR WITH NOTCH */
  ["\u238A", "\u238A"], /* CIRCLED TRIANGLE DOWN */
  ["\u238B", "\u238B"], /* BROKEN CIRCLE WITH NORTHWEST ARROW */
  ["\u238C", "\u238C"], /* UNDO SYMBOL */
  ["\u238D", "\u238D"], /* MONOSTABLE SYMBOL */
  ["\u238E", "\u238E"], /* HYSTERESIS SYMBOL */
  ["\u238F", "\u238F"], /* OPEN-CIRCUIT-OUTPUT H-TYPE SYMBOL */
  ["\u2390", "\u2390"], /* OPEN-CIRCUIT-OUTPUT L-TYPE SYMBOL */
  ["\u2391", "\u2391"], /* PASSIVE-PULL-DOWN-OUTPUT SYMBOL */
  ["\u2392", "\u2392"], /* PASSIVE-PULL-UP-OUTPUT SYMBOL */
  ["\u2393", "\u2393"], /* DIRECT CURRENT SYMBOL FORM TWO */
  ["\u2394", "\u2394"], /* SOFTWARE-FUNCTION SYMBOL */
  ["\u2395", "\u2395"], /* APL FUNCTIONAL SYMBOL QUAD */
  ["\u2396", "\u2396"], /* DECIMAL SEPARATOR KEY SYMBOL */
  ["\u2397", "\u2397"], /* PREVIOUS PAGE */
  ["\u2398", "\u2398"], /* NEXT PAGE */
  ["\u2399", "\u2399"], /* PRINT SCREEN SYMBOL */
  ["\u239A", "\u239A"], /* CLEAR SCREEN SYMBOL */
  ["\u239B", "\u239B"], /* LEFT PARENTHESIS UPPER HOOK */
  ["\u239C", "\u239C"], /* LEFT PARENTHESIS EXTENSION */
  ["\u239D", "\u239D"], /* LEFT PARENTHESIS LOWER HOOK */
  ["\u239E", "\u239E"], /* RIGHT PARENTHESIS UPPER HOOK */
  ["\u239F", "\u239F"], /* RIGHT PARENTHESIS EXTENSION */
  ["\u23A0", "\u23A0"], /* RIGHT PARENTHESIS LOWER HOOK */
  ["\u23A1", "\u23A1"], /* LEFT SQUARE BRACKET UPPER CORNER */
  ["\u23A2", "\u23A2"], /* LEFT SQUARE BRACKET EXTENSION */
  ["\u23A3", "\u23A3"], /* LEFT SQUARE BRACKET LOWER CORNER */
  ["\u23A4", "\u23A4"], /* RIGHT SQUARE BRACKET UPPER CORNER */
  ["\u23A5", "\u23A5"], /* RIGHT SQUARE BRACKET EXTENSION */
  ["\u23A6", "\u23A6"], /* RIGHT SQUARE BRACKET LOWER CORNER */
  ["\u23A7", "\u23A7"], /* LEFT CURLY BRACKET UPPER HOOK */
  ["\u23A8", "\u23A8"], /* LEFT CURLY BRACKET MIDDLE PIECE */
  ["\u23A9", "\u23A9"], /* LEFT CURLY BRACKET LOWER HOOK */
  ["\u23AA", "\u23AA"], /* CURLY BRACKET EXTENSION */
  ["\u23AB", "\u23AB"], /* RIGHT CURLY BRACKET UPPER HOOK */
  ["\u23AC", "\u23AC"], /* RIGHT CURLY BRACKET MIDDLE PIECE */
  ["\u23AD", "\u23AD"], /* RIGHT CURLY BRACKET LOWER HOOK */
  ["\u23AE", "\u23AE"], /* INTEGRAL EXTENSION */
  ["\u23AF", "\u23AF"], /* HORIZONTAL LINE EXTENSION */
  ["\u23B0", "\u23B0"], /* UPPER LEFT OR LOWER RIGHT CURLY BRACKET SECTION */
  ["\u23B1", "\u23B1"], /* UPPER RIGHT OR LOWER LEFT CURLY BRACKET SECTION */
  ["\u23B2", "\u23B2"], /* SUMMATION TOP */
  ["\u23B3", "\u23B3"], /* SUMMATION BOTTOM */
  ["\u23B4", "\u23B4"], /* TOP SQUARE BRACKET */
  ["\u23B5", "\u23B5"], /* BOTTOM SQUARE BRACKET */
  ["\u23B6", "\u23B6"], /* BOTTOM SQUARE BRACKET OVER TOP SQUARE BRACKET */
  ["\u23B7", "\u23B7"], /* RADICAL SYMBOL BOTTOM */
  ["\u23B8", "\u23B8"], /* LEFT VERTICAL BOX LINE */
  ["\u23B9", "\u23B9"], /* RIGHT VERTICAL BOX LINE */
  ["\u23BA", "\u23BA"], /* HORIZONTAL SCAN LINE-1 */
  ["\u23BB", "\u23BB"], /* HORIZONTAL SCAN LINE-3 */
  ["\u23BC", "\u23BC"], /* HORIZONTAL SCAN LINE-7 */
  ["\u23BD", "\u23BD"], /* HORIZONTAL SCAN LINE-9 */
  ["\u23BE", "\u23BE"], /* DENTISTRY SYMBOL LIGHT VERTICAL AND TOP RIGHT */
  ["\u23BF", "\u23BF"], /* DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM RIGHT */
  ["\u23C0", "\u23C0"], /* DENTISTRY SYMBOL LIGHT VERTICAL WITH CIRCLE */
  ["\u23C1", "\u23C1"], /* DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH CIRCLE */
  ["\u23C2", "\u23C2"], /* DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH CIRCLE */
  ["\u23C3", "\u23C3"], /* DENTISTRY SYMBOL LIGHT VERTICAL WITH TRIANGLE */
  ["\u23C4", "\u23C4"], /* DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH TRIANGLE */
  ["\u23C5", "\u23C5"], /* DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH TRIANGLE */
  ["\u23C6", "\u23C6"], /* DENTISTRY SYMBOL LIGHT VERTICAL AND WAVE */
  ["\u23C7", "\u23C7"], /* DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL WITH WAVE */
  ["\u23C8", "\u23C8"], /* DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL WITH WAVE */
  ["\u23C9", "\u23C9"], /* DENTISTRY SYMBOL LIGHT DOWN AND HORIZONTAL */
  ["\u23CA", "\u23CA"], /* DENTISTRY SYMBOL LIGHT UP AND HORIZONTAL */
  ["\u23CB", "\u23CB"], /* DENTISTRY SYMBOL LIGHT VERTICAL AND TOP LEFT */
  ["\u23CC", "\u23CC"], /* DENTISTRY SYMBOL LIGHT VERTICAL AND BOTTOM LEFT */
  ["\u23CD", "\u23CD"], /* SQUARE FOOT */
  ["\u23CE", "\u23CE"], /* RETURN SYMBOL */
  ["\u23CF", "\u23CF"], /* EJECT SYMBOL */
  ["\u23D0", "\u23D0"], /* VERTICAL LINE EXTENSION */
  ["\u23D1", "\u23D1"], /* METRICAL BREVE */
  ["\u23D2", "\u23D2"], /* METRICAL LONG OVER SHORT */
  ["\u23D3", "\u23D3"], /* METRICAL SHORT OVER LONG */
  ["\u23D4", "\u23D4"], /* METRICAL LONG OVER TWO SHORTS */
  ["\u23D5", "\u23D5"], /* METRICAL TWO SHORTS OVER LONG */
  ["\u23D6", "\u23D6"], /* METRICAL TWO SHORTS JOINED */
  ["\u23D7", "\u23D7"], /* METRICAL TRISEME */
  ["\u23D8", "\u23D8"], /* METRICAL TETRASEME */
  ["\u23D9", "\u23D9"], /* METRICAL PENTASEME */
  ["\u23DA", "\u23DA"], /* EARTH GROUND */
  ["\u23DB", "\u23DB"], /* FUSE */
  ["\u23DC", "\u23DC"], /* TOP PARENTHESIS */
  ["\u23DD", "\u23DD"], /* BOTTOM PARENTHESIS */
  ["\u23DE", "\u23DE"], /* TOP CURLY BRACKET */
  ["\u23DF", "\u23DF"], /* BOTTOM CURLY BRACKET */
  ["\u23E0", "\u23E0"], /* TOP TORTOISE SHELL BRACKET */
  ["\u23E1", "\u23E1"], /* BOTTOM TORTOISE SHELL BRACKET */
  ["\u23E2", "\u23E2"], /* WHITE TRAPEZIUM */
  ["\u23E3", "\u23E3"], /* BENZENE RING WITH CIRCLE */
  ["\u23E4", "\u23E4"], /* STRAIGHTNESS */
  ["\u23E5", "\u23E5"], /* FLATNESS */
  ["\u23E6", "\u23E6"], /* AC CURRENT */
  ["\u23E7", "\u23E7"], /* ELECTRICAL INTERSECTION */
  ["\u23E8", "\u23E8"], /* DECIMAL EXPONENT SYMBOL */
  ["\u23E9", "\u23E9"], /* BLACK RIGHT-POINTING DOUBLE TRIANGLE */
  ["\u23EA", "\u23EA"], /* BLACK LEFT-POINTING DOUBLE TRIANGLE */
  ["\u23EB", "\u23EB"], /* BLACK UP-POINTING DOUBLE TRIANGLE */
  ["\u23EC", "\u23EC"], /* BLACK DOWN-POINTING DOUBLE TRIANGLE */
  ["\u23ED", "\u23ED"], /* BLACK RIGHT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR */
  ["\u23EE", "\u23EE"], /* BLACK LEFT-POINTING DOUBLE TRIANGLE WITH VERTICAL BAR */
  ["\u23EF", "\u23EF"], /* BLACK RIGHT-POINTING TRIANGLE WITH DOUBLE VERTICAL BAR */
  ["\u23F0", "\u23F0"], /* ALARM CLOCK */
  ["\u23F1", "\u23F1"], /* STOPWATCH */
  ["\u23F2", "\u23F2"], /* TIMER CLOCK */
  ["\u23F3", "\u23F3"], /* HOURGLASS WITH FLOWING SAND */
  ["\u23F4", "\u23F4"], /* BLACK MEDIUM LEFT-POINTING TRIANGLE */
  ["\u23F5", "\u23F5"], /* BLACK MEDIUM RIGHT-POINTING TRIANGLE */
  ["\u23F6", "\u23F6"], /* BLACK MEDIUM UP-POINTING TRIANGLE */
  ["\u23F7", "\u23F7"], /* BLACK MEDIUM DOWN-POINTING TRIANGLE */
  ["\u23F8", "\u23F8"], /* DOUBLE VERTICAL BAR */
  ["\u23F9", "\u23F9"], /* BLACK SQUARE FOR STOP */
  ["\u23FA", "\u23FA"], /* BLACK CIRCLE FOR RECORD */
  ["\u23FB", "\u23FB"], /* POWER SYMBOL */
  ["\u23FC", "\u23FC"], /* POWER ON-OFF SYMBOL */
  ["\u23FD", "\u23FD"], /* POWER ON SYMBOL */
  ["\u23FE", "\u23FE"], /* POWER SLEEP SYMBOL */
  ["\u23FF", "\u23FF"], /* OBSERVER EYE SYMBOL */
  ["\u2400", "\u2400"], /* SYMBOL FOR NULL (GRAPHIC FOR NULL) */
  ["\u2401", "\u2401"], /* SYMBOL FOR START OF HEADING (GRAPHIC FOR START OF HEADING) */
  ["\u2402", "\u2402"], /* SYMBOL FOR START OF TEXT (GRAPHIC FOR START OF TEXT) */
  ["\u2403", "\u2403"], /* SYMBOL FOR END OF TEXT (GRAPHIC FOR END OF TEXT) */
  ["\u2404", "\u2404"], /* SYMBOL FOR END OF TRANSMISSION (GRAPHIC FOR END OF TRANSMISSION) */
  ["\u2405", "\u2405"], /* SYMBOL FOR ENQUIRY (GRAPHIC FOR ENQUIRY) */
  ["\u2406", "\u2406"], /* SYMBOL FOR ACKNOWLEDGE (GRAPHIC FOR ACKNOWLEDGE) */
  ["\u2407", "\u2407"], /* SYMBOL FOR BELL (GRAPHIC FOR BELL) */
  ["\u2408", "\u2408"], /* SYMBOL FOR BACKSPACE (GRAPHIC FOR BACKSPACE) */
  ["\u2409", "\u2409"], /* SYMBOL FOR HORIZONTAL TABULATION (GRAPHIC FOR HORIZONTAL TABULATION) */
  ["\u240A", "\u240A"], /* SYMBOL FOR LINE FEED (GRAPHIC FOR LINE FEED) */
  ["\u240B", "\u240B"], /* SYMBOL FOR VERTICAL TABULATION (GRAPHIC FOR VERTICAL TABULATION) */
  ["\u240C", "\u240C"], /* SYMBOL FOR FORM FEED (GRAPHIC FOR FORM FEED) */
  ["\u240D", "\u240D"], /* SYMBOL FOR CARRIAGE RETURN (GRAPHIC FOR CARRIAGE RETURN) */
  ["\u240E", "\u240E"], /* SYMBOL FOR SHIFT OUT (GRAPHIC FOR SHIFT OUT) */
  ["\u240F", "\u240F"], /* SYMBOL FOR SHIFT IN (GRAPHIC FOR SHIFT IN) */
  ["\u2410", "\u2410"], /* SYMBOL FOR DATA LINK ESCAPE (GRAPHIC FOR DATA LINK ESCAPE) */
  ["\u2411", "\u2411"], /* SYMBOL FOR DEVICE CONTROL ONE (GRAPHIC FOR DEVICE CONTROL ONE) */
  ["\u2412", "\u2412"], /* SYMBOL FOR DEVICE CONTROL TWO (GRAPHIC FOR DEVICE CONTROL TWO) */
  ["\u2413", "\u2413"], /* SYMBOL FOR DEVICE CONTROL THREE (GRAPHIC FOR DEVICE CONTROL THREE) */
  ["\u2414", "\u2414"], /* SYMBOL FOR DEVICE CONTROL FOUR (GRAPHIC FOR DEVICE CONTROL FOUR) */
  ["\u2415", "\u2415"], /* SYMBOL FOR NEGATIVE ACKNOWLEDGE (GRAPHIC FOR NEGATIVE ACKNOWLEDGE) */
  ["\u2416", "\u2416"], /* SYMBOL FOR SYNCHRONOUS IDLE (GRAPHIC FOR SYNCHRONOUS IDLE) */
  ["\u2417", "\u2417"], /* SYMBOL FOR END OF TRANSMISSION BLOCK (GRAPHIC FOR END OF TRANSMISSION BLOCK) */
  ["\u2418", "\u2418"], /* SYMBOL FOR CANCEL (GRAPHIC FOR CANCEL) */
  ["\u2419", "\u2419"], /* SYMBOL FOR END OF MEDIUM (GRAPHIC FOR END OF MEDIUM) */
  ["\u241A", "\u241A"], /* SYMBOL FOR SUBSTITUTE (GRAPHIC FOR SUBSTITUTE) */
  ["\u241B", "\u241B"], /* SYMBOL FOR ESCAPE (GRAPHIC FOR ESCAPE) */
  ["\u241C", "\u241C"], /* SYMBOL FOR FILE SEPARATOR (GRAPHIC FOR FILE SEPARATOR) */
  ["\u241D", "\u241D"], /* SYMBOL FOR GROUP SEPARATOR (GRAPHIC FOR GROUP SEPARATOR) */
  ["\u241E", "\u241E"], /* SYMBOL FOR RECORD SEPARATOR (GRAPHIC FOR RECORD SEPARATOR) */
  ["\u241F", "\u241F"], /* SYMBOL FOR UNIT SEPARATOR (GRAPHIC FOR UNIT SEPARATOR) */
  ["\u2420", "\u2420"], /* SYMBOL FOR SPACE (GRAPHIC FOR SPACE) */
  ["\u2421", "\u2421"], /* SYMBOL FOR DELETE (GRAPHIC FOR DELETE) */
  ["\u2422", "\u2422"], /* BLANK SYMBOL (BLANK) */
  ["\u2423", "\u2423"], /* OPEN BOX */
  ["\u2424", "\u2424"], /* SYMBOL FOR NEWLINE (GRAPHIC FOR NEWLINE) */
  ["\u2425", "\u2425"], /* SYMBOL FOR DELETE FORM TWO */
  ["\u2426", "\u2426"], /* SYMBOL FOR SUBSTITUTE FORM TWO */
  ["\u2427", "\u2427"],
  ["\u2428", "\u2428"],
  ["\u2429", "\u2429"],
  ["\u242A", "\u242A"],
  ["\u242B", "\u242B"],
  ["\u242C", "\u242C"],
  ["\u242D", "\u242D"],
  ["\u242E", "\u242E"],
  ["\u242F", "\u242F"],
  ["\u2430", "\u2430"],
  ["\u2431", "\u2431"],
  ["\u2432", "\u2432"],
  ["\u2433", "\u2433"],
  ["\u2434", "\u2434"],
  ["\u2435", "\u2435"],
  ["\u2436", "\u2436"],
  ["\u2437", "\u2437"],
  ["\u2438", "\u2438"],
  ["\u2439", "\u2439"],
  ["\u243A", "\u243A"],
  ["\u243B", "\u243B"],
  ["\u243C", "\u243C"],
  ["\u243D", "\u243D"],
  ["\u243E", "\u243E"],
  ["\u243F", "\u243F"],
  ["\u2440", "\u2440"], /* OCR HOOK */
  ["\u2441", "\u2441"], /* OCR CHAIR */
  ["\u2442", "\u2442"], /* OCR FORK */
  ["\u2443", "\u2443"], /* OCR INVERTED FORK */
  ["\u2444", "\u2444"], /* OCR BELT BUCKLE */
  ["\u2445", "\u2445"], /* OCR BOW TIE */
  ["\u2446", "\u2446"], /* OCR BRANCH BANK IDENTIFICATION */
  ["\u2447", "\u2447"], /* OCR AMOUNT OF CHECK */
  ["\u2448", "\u2448"], /* OCR DASH */
  ["\u2449", "\u2449"], /* OCR CUSTOMER ACCOUNT NUMBER */
  ["\u244A", "\u244A"], /* OCR DOUBLE BACKSLASH */
  ["\u244B", "\u244B"],
  ["\u244C", "\u244C"],
  ["\u244D", "\u244D"],
  ["\u244E", "\u244E"],
  ["\u244F", "\u244F"],
  ["\u2450", "\u2450"],
  ["\u2451", "\u2451"],
  ["\u2452", "\u2452"],
  ["\u2453", "\u2453"],
  ["\u2454", "\u2454"],
  ["\u2455", "\u2455"],
  ["\u2456", "\u2456"],
  ["\u2457", "\u2457"],
  ["\u2458", "\u2458"],
  ["\u2459", "\u2459"],
  ["\u245A", "\u245A"],
  ["\u245B", "\u245B"],
  ["\u245C", "\u245C"],
  ["\u245D", "\u245D"],
  ["\u245E", "\u245E"],
  ["\u245F", "\u245F"],
  ["\u2460", "\u2460"], /* CIRCLED DIGIT ONE */
  ["\u2461", "\u2461"], /* CIRCLED DIGIT TWO */
  ["\u2462", "\u2462"], /* CIRCLED DIGIT THREE */
  ["\u2463", "\u2463"], /* CIRCLED DIGIT FOUR */
  ["\u2464", "\u2464"], /* CIRCLED DIGIT FIVE */
  ["\u2465", "\u2465"], /* CIRCLED DIGIT SIX */
  ["\u2466", "\u2466"], /* CIRCLED DIGIT SEVEN */
  ["\u2467", "\u2467"], /* CIRCLED DIGIT EIGHT */
  ["\u2468", "\u2468"], /* CIRCLED DIGIT NINE */
  ["\u2469", "\u2469"], /* CIRCLED NUMBER TEN */
  ["\u246A", "\u246A"], /* CIRCLED NUMBER ELEVEN */
  ["\u246B", "\u246B"], /* CIRCLED NUMBER TWELVE */
  ["\u246C", "\u246C"], /* CIRCLED NUMBER THIRTEEN */
  ["\u246D", "\u246D"], /* CIRCLED NUMBER FOURTEEN */
  ["\u246E", "\u246E"], /* CIRCLED NUMBER FIFTEEN */
  ["\u246F", "\u246F"], /* CIRCLED NUMBER SIXTEEN */
  ["\u2470", "\u2470"], /* CIRCLED NUMBER SEVENTEEN */
  ["\u2471", "\u2471"], /* CIRCLED NUMBER EIGHTEEN */
  ["\u2472", "\u2472"], /* CIRCLED NUMBER NINETEEN */
  ["\u2473", "\u2473"], /* CIRCLED NUMBER TWENTY */
  ["\u2474", "\u2474"], /* PARENTHESIZED DIGIT ONE */
  ["\u2475", "\u2475"], /* PARENTHESIZED DIGIT TWO */
  ["\u2476", "\u2476"], /* PARENTHESIZED DIGIT THREE */
  ["\u2477", "\u2477"], /* PARENTHESIZED DIGIT FOUR */
  ["\u2478", "\u2478"], /* PARENTHESIZED DIGIT FIVE */
  ["\u2479", "\u2479"], /* PARENTHESIZED DIGIT SIX */
  ["\u247A", "\u247A"], /* PARENTHESIZED DIGIT SEVEN */
  ["\u247B", "\u247B"], /* PARENTHESIZED DIGIT EIGHT */
  ["\u247C", "\u247C"], /* PARENTHESIZED DIGIT NINE */
  ["\u247D", "\u247D"], /* PARENTHESIZED NUMBER TEN */
  ["\u247E", "\u247E"], /* PARENTHESIZED NUMBER ELEVEN */
  ["\u247F", "\u247F"], /* PARENTHESIZED NUMBER TWELVE */
  ["\u2480", "\u2480"], /* PARENTHESIZED NUMBER THIRTEEN */
  ["\u2481", "\u2481"], /* PARENTHESIZED NUMBER FOURTEEN */
  ["\u2482", "\u2482"], /* PARENTHESIZED NUMBER FIFTEEN */
  ["\u2483", "\u2483"], /* PARENTHESIZED NUMBER SIXTEEN */
  ["\u2484", "\u2484"], /* PARENTHESIZED NUMBER SEVENTEEN */
  ["\u2485", "\u2485"], /* PARENTHESIZED NUMBER EIGHTEEN */
  ["\u2486", "\u2486"], /* PARENTHESIZED NUMBER NINETEEN */
  ["\u2487", "\u2487"], /* PARENTHESIZED NUMBER TWENTY */
  ["\u2488", "\u2488"], /* DIGIT ONE FULL STOP (DIGIT ONE PERIOD) */
  ["\u2489", "\u2489"], /* DIGIT TWO FULL STOP (DIGIT TWO PERIOD) */
  ["\u248A", "\u248A"], /* DIGIT THREE FULL STOP (DIGIT THREE PERIOD) */
  ["\u248B", "\u248B"], /* DIGIT FOUR FULL STOP (DIGIT FOUR PERIOD) */
  ["\u248C", "\u248C"], /* DIGIT FIVE FULL STOP (DIGIT FIVE PERIOD) */
  ["\u248D", "\u248D"], /* DIGIT SIX FULL STOP (DIGIT SIX PERIOD) */
  ["\u248E", "\u248E"], /* DIGIT SEVEN FULL STOP (DIGIT SEVEN PERIOD) */
  ["\u248F", "\u248F"], /* DIGIT EIGHT FULL STOP (DIGIT EIGHT PERIOD) */
  ["\u2490", "\u2490"], /* DIGIT NINE FULL STOP (DIGIT NINE PERIOD) */
  ["\u2491", "\u2491"], /* NUMBER TEN FULL STOP (NUMBER TEN PERIOD) */
  ["\u2492", "\u2492"], /* NUMBER ELEVEN FULL STOP (NUMBER ELEVEN PERIOD) */
  ["\u2493", "\u2493"], /* NUMBER TWELVE FULL STOP (NUMBER TWELVE PERIOD) */
  ["\u2494", "\u2494"], /* NUMBER THIRTEEN FULL STOP (NUMBER THIRTEEN PERIOD) */
  ["\u2495", "\u2495"], /* NUMBER FOURTEEN FULL STOP (NUMBER FOURTEEN PERIOD) */
  ["\u2496", "\u2496"], /* NUMBER FIFTEEN FULL STOP (NUMBER FIFTEEN PERIOD) */
  ["\u2497", "\u2497"], /* NUMBER SIXTEEN FULL STOP (NUMBER SIXTEEN PERIOD) */
  ["\u2498", "\u2498"], /* NUMBER SEVENTEEN FULL STOP (NUMBER SEVENTEEN PERIOD) */
  ["\u2499", "\u2499"], /* NUMBER EIGHTEEN FULL STOP (NUMBER EIGHTEEN PERIOD) */
  ["\u249A", "\u249A"], /* NUMBER NINETEEN FULL STOP (NUMBER NINETEEN PERIOD) */
  ["\u249B", "\u249B"], /* NUMBER TWENTY FULL STOP (NUMBER TWENTY PERIOD) */
  ["\u249C", "\u249C"], /* PARENTHESIZED LATIN SMALL LETTER A */
  ["\u249D", "\u249D"], /* PARENTHESIZED LATIN SMALL LETTER B */
  ["\u249E", "\u249E"], /* PARENTHESIZED LATIN SMALL LETTER C */
  ["\u249F", "\u249F"], /* PARENTHESIZED LATIN SMALL LETTER D */
  ["\u24A0", "\u24A0"], /* PARENTHESIZED LATIN SMALL LETTER E */
  ["\u24A1", "\u24A1"], /* PARENTHESIZED LATIN SMALL LETTER F */
  ["\u24A2", "\u24A2"], /* PARENTHESIZED LATIN SMALL LETTER G */
  ["\u24A3", "\u24A3"], /* PARENTHESIZED LATIN SMALL LETTER H */
  ["\u24A4", "\u24A4"], /* PARENTHESIZED LATIN SMALL LETTER I */
  ["\u24A5", "\u24A5"], /* PARENTHESIZED LATIN SMALL LETTER J */
  ["\u24A6", "\u24A6"], /* PARENTHESIZED LATIN SMALL LETTER K */
  ["\u24A7", "\u24A7"], /* PARENTHESIZED LATIN SMALL LETTER L */
  ["\u24A8", "\u24A8"], /* PARENTHESIZED LATIN SMALL LETTER M */
  ["\u24A9", "\u24A9"], /* PARENTHESIZED LATIN SMALL LETTER N */
  ["\u24AA", "\u24AA"], /* PARENTHESIZED LATIN SMALL LETTER O */
  ["\u24AB", "\u24AB"], /* PARENTHESIZED LATIN SMALL LETTER P */
  ["\u24AC", "\u24AC"], /* PARENTHESIZED LATIN SMALL LETTER Q */
  ["\u24AD", "\u24AD"], /* PARENTHESIZED LATIN SMALL LETTER R */
  ["\u24AE", "\u24AE"], /* PARENTHESIZED LATIN SMALL LETTER S */
  ["\u24AF", "\u24AF"], /* PARENTHESIZED LATIN SMALL LETTER T */
  ["\u24B0", "\u24B0"], /* PARENTHESIZED LATIN SMALL LETTER U */
  ["\u24B1", "\u24B1"], /* PARENTHESIZED LATIN SMALL LETTER V */
  ["\u24B2", "\u24B2"], /* PARENTHESIZED LATIN SMALL LETTER W */
  ["\u24B3", "\u24B3"], /* PARENTHESIZED LATIN SMALL LETTER X */
  ["\u24B4", "\u24B4"], /* PARENTHESIZED LATIN SMALL LETTER Y */
  ["\u24B5", "\u24B5"], /* PARENTHESIZED LATIN SMALL LETTER Z */
  ["\u24B6", "\u24D0"], /* CIRCLED LATIN CAPITAL LETTER A */
  ["\u24B7", "\u24D1"], /* CIRCLED LATIN CAPITAL LETTER B */
  ["\u24B8", "\u24D2"], /* CIRCLED LATIN CAPITAL LETTER C */
  ["\u24B9", "\u24D3"], /* CIRCLED LATIN CAPITAL LETTER D */
  ["\u24BA", "\u24D4"], /* CIRCLED LATIN CAPITAL LETTER E */
  ["\u24BB", "\u24D5"], /* CIRCLED LATIN CAPITAL LETTER F */
  ["\u24BC", "\u24D6"], /* CIRCLED LATIN CAPITAL LETTER G */
  ["\u24BD", "\u24D7"], /* CIRCLED LATIN CAPITAL LETTER H */
  ["\u24BE", "\u24D8"], /* CIRCLED LATIN CAPITAL LETTER I */
  ["\u24BF", "\u24D9"], /* CIRCLED LATIN CAPITAL LETTER J */
  ["\u24C0", "\u24DA"], /* CIRCLED LATIN CAPITAL LETTER K */
  ["\u24C1", "\u24DB"], /* CIRCLED LATIN CAPITAL LETTER L */
  ["\u24C2", "\u24DC"], /* CIRCLED LATIN CAPITAL LETTER M */
  ["\u24C3", "\u24DD"], /* CIRCLED LATIN CAPITAL LETTER N */
  ["\u24C4", "\u24DE"], /* CIRCLED LATIN CAPITAL LETTER O */
  ["\u24C5", "\u24DF"], /* CIRCLED LATIN CAPITAL LETTER P */
  ["\u24C6", "\u24E0"], /* CIRCLED LATIN CAPITAL LETTER Q */
  ["\u24C7", "\u24E1"], /* CIRCLED LATIN CAPITAL LETTER R */
  ["\u24C8", "\u24E2"], /* CIRCLED LATIN CAPITAL LETTER S */
  ["\u24C9", "\u24E3"], /* CIRCLED LATIN CAPITAL LETTER T */
  ["\u24CA", "\u24E4"], /* CIRCLED LATIN CAPITAL LETTER U */
  ["\u24CB", "\u24E5"], /* CIRCLED LATIN CAPITAL LETTER V */
  ["\u24CC", "\u24E6"], /* CIRCLED LATIN CAPITAL LETTER W */
  ["\u24CD", "\u24E7"], /* CIRCLED LATIN CAPITAL LETTER X */
  ["\u24CE", "\u24E8"], /* CIRCLED LATIN CAPITAL LETTER Y */
  ["\u24CF", "\u24E9"], /* CIRCLED LATIN CAPITAL LETTER Z */
  ["\u24B6", "\u24D0"], /* CIRCLED LATIN SMALL LETTER A */
  ["\u24B7", "\u24D1"], /* CIRCLED LATIN SMALL LETTER B */
  ["\u24B8", "\u24D2"], /* CIRCLED LATIN SMALL LETTER C */
  ["\u24B9", "\u24D3"], /* CIRCLED LATIN SMALL LETTER D */
  ["\u24BA", "\u24D4"], /* CIRCLED LATIN SMALL LETTER E */
  ["\u24BB", "\u24D5"], /* CIRCLED LATIN SMALL LETTER F */
  ["\u24BC", "\u24D6"], /* CIRCLED LATIN SMALL LETTER G */
  ["\u24BD", "\u24D7"], /* CIRCLED LATIN SMALL LETTER H */
  ["\u24BE", "\u24D8"], /* CIRCLED LATIN SMALL LETTER I */
  ["\u24BF", "\u24D9"], /* CIRCLED LATIN SMALL LETTER J */
  ["\u24C0", "\u24DA"], /* CIRCLED LATIN SMALL LETTER K */
  ["\u24C1", "\u24DB"], /* CIRCLED LATIN SMALL LETTER L */
  ["\u24C2", "\u24DC"], /* CIRCLED LATIN SMALL LETTER M */
  ["\u24C3", "\u24DD"], /* CIRCLED LATIN SMALL LETTER N */
  ["\u24C4", "\u24DE"], /* CIRCLED LATIN SMALL LETTER O */
  ["\u24C5", "\u24DF"], /* CIRCLED LATIN SMALL LETTER P */
  ["\u24C6", "\u24E0"], /* CIRCLED LATIN SMALL LETTER Q */
  ["\u24C7", "\u24E1"], /* CIRCLED LATIN SMALL LETTER R */
  ["\u24C8", "\u24E2"], /* CIRCLED LATIN SMALL LETTER S */
  ["\u24C9", "\u24E3"], /* CIRCLED LATIN SMALL LETTER T */
  ["\u24CA", "\u24E4"], /* CIRCLED LATIN SMALL LETTER U */
  ["\u24CB", "\u24E5"], /* CIRCLED LATIN SMALL LETTER V */
  ["\u24CC", "\u24E6"], /* CIRCLED LATIN SMALL LETTER W */
  ["\u24CD", "\u24E7"], /* CIRCLED LATIN SMALL LETTER X */
  ["\u24CE", "\u24E8"], /* CIRCLED LATIN SMALL LETTER Y */
  ["\u24CF", "\u24E9"], /* CIRCLED LATIN SMALL LETTER Z */
  ["\u24EA", "\u24EA"], /* CIRCLED DIGIT ZERO */
  ["\u24EB", "\u24EB"], /* NEGATIVE CIRCLED NUMBER ELEVEN */
  ["\u24EC", "\u24EC"], /* NEGATIVE CIRCLED NUMBER TWELVE */
  ["\u24ED", "\u24ED"], /* NEGATIVE CIRCLED NUMBER THIRTEEN */
  ["\u24EE", "\u24EE"], /* NEGATIVE CIRCLED NUMBER FOURTEEN */
  ["\u24EF", "\u24EF"], /* NEGATIVE CIRCLED NUMBER FIFTEEN */
  ["\u24F0", "\u24F0"], /* NEGATIVE CIRCLED NUMBER SIXTEEN */
  ["\u24F1", "\u24F1"], /* NEGATIVE CIRCLED NUMBER SEVENTEEN */
  ["\u24F2", "\u24F2"], /* NEGATIVE CIRCLED NUMBER EIGHTEEN */
  ["\u24F3", "\u24F3"], /* NEGATIVE CIRCLED NUMBER NINETEEN */
  ["\u24F4", "\u24F4"], /* NEGATIVE CIRCLED NUMBER TWENTY */
  ["\u24F5", "\u24F5"], /* DOUBLE CIRCLED DIGIT ONE */
  ["\u24F6", "\u24F6"], /* DOUBLE CIRCLED DIGIT TWO */
  ["\u24F7", "\u24F7"], /* DOUBLE CIRCLED DIGIT THREE */
  ["\u24F8", "\u24F8"], /* DOUBLE CIRCLED DIGIT FOUR */
  ["\u24F9", "\u24F9"], /* DOUBLE CIRCLED DIGIT FIVE */
  ["\u24FA", "\u24FA"], /* DOUBLE CIRCLED DIGIT SIX */
  ["\u24FB", "\u24FB"], /* DOUBLE CIRCLED DIGIT SEVEN */
  ["\u24FC", "\u24FC"], /* DOUBLE CIRCLED DIGIT EIGHT */
  ["\u24FD", "\u24FD"], /* DOUBLE CIRCLED DIGIT NINE */
  ["\u24FE", "\u24FE"], /* DOUBLE CIRCLED NUMBER TEN */
  ["\u24FF", "\u24FF"], /* NEGATIVE CIRCLED DIGIT ZERO */
  ["\u2500", "\u2500"], /* BOX DRAWINGS LIGHT HORIZONTAL (FORMS LIGHT HORIZONTAL) */
  ["\u2501", "\u2501"], /* BOX DRAWINGS HEAVY HORIZONTAL (FORMS HEAVY HORIZONTAL) */
  ["\u2502", "\u2502"], /* BOX DRAWINGS LIGHT VERTICAL (FORMS LIGHT VERTICAL) */
  ["\u2503", "\u2503"], /* BOX DRAWINGS HEAVY VERTICAL (FORMS HEAVY VERTICAL) */
  ["\u2504", "\u2504"], /* BOX DRAWINGS LIGHT TRIPLE DASH HORIZONTAL (FORMS LIGHT TRIPLE DASH HORIZONTAL) */
  ["\u2505", "\u2505"], /* BOX DRAWINGS HEAVY TRIPLE DASH HORIZONTAL (FORMS HEAVY TRIPLE DASH HORIZONTAL) */
  ["\u2506", "\u2506"], /* BOX DRAWINGS LIGHT TRIPLE DASH VERTICAL (FORMS LIGHT TRIPLE DASH VERTICAL) */
  ["\u2507", "\u2507"], /* BOX DRAWINGS HEAVY TRIPLE DASH VERTICAL (FORMS HEAVY TRIPLE DASH VERTICAL) */
  ["\u2508", "\u2508"], /* BOX DRAWINGS LIGHT QUADRUPLE DASH HORIZONTAL (FORMS LIGHT QUADRUPLE DASH HORIZONTAL) */
  ["\u2509", "\u2509"], /* BOX DRAWINGS HEAVY QUADRUPLE DASH HORIZONTAL (FORMS HEAVY QUADRUPLE DASH HORIZONTAL) */
  ["\u250A", "\u250A"], /* BOX DRAWINGS LIGHT QUADRUPLE DASH VERTICAL (FORMS LIGHT QUADRUPLE DASH VERTICAL) */
  ["\u250B", "\u250B"], /* BOX DRAWINGS HEAVY QUADRUPLE DASH VERTICAL (FORMS HEAVY QUADRUPLE DASH VERTICAL) */
  ["\u250C", "\u250C"], /* BOX DRAWINGS LIGHT DOWN AND RIGHT (FORMS LIGHT DOWN AND RIGHT) */
  ["\u250D", "\u250D"], /* BOX DRAWINGS DOWN LIGHT AND RIGHT HEAVY (FORMS DOWN LIGHT AND RIGHT HEAVY) */
  ["\u250E", "\u250E"], /* BOX DRAWINGS DOWN HEAVY AND RIGHT LIGHT (FORMS DOWN HEAVY AND RIGHT LIGHT) */
  ["\u250F", "\u250F"], /* BOX DRAWINGS HEAVY DOWN AND RIGHT (FORMS HEAVY DOWN AND RIGHT) */
  ["\u2510", "\u2510"], /* BOX DRAWINGS LIGHT DOWN AND LEFT (FORMS LIGHT DOWN AND LEFT) */
  ["\u2511", "\u2511"], /* BOX DRAWINGS DOWN LIGHT AND LEFT HEAVY (FORMS DOWN LIGHT AND LEFT HEAVY) */
  ["\u2512", "\u2512"], /* BOX DRAWINGS DOWN HEAVY AND LEFT LIGHT (FORMS DOWN HEAVY AND LEFT LIGHT) */
  ["\u2513", "\u2513"], /* BOX DRAWINGS HEAVY DOWN AND LEFT (FORMS HEAVY DOWN AND LEFT) */
  ["\u2514", "\u2514"], /* BOX DRAWINGS LIGHT UP AND RIGHT (FORMS LIGHT UP AND RIGHT) */
  ["\u2515", "\u2515"], /* BOX DRAWINGS UP LIGHT AND RIGHT HEAVY (FORMS UP LIGHT AND RIGHT HEAVY) */
  ["\u2516", "\u2516"], /* BOX DRAWINGS UP HEAVY AND RIGHT LIGHT (FORMS UP HEAVY AND RIGHT LIGHT) */
  ["\u2517", "\u2517"], /* BOX DRAWINGS HEAVY UP AND RIGHT (FORMS HEAVY UP AND RIGHT) */
  ["\u2518", "\u2518"], /* BOX DRAWINGS LIGHT UP AND LEFT (FORMS LIGHT UP AND LEFT) */
  ["\u2519", "\u2519"], /* BOX DRAWINGS UP LIGHT AND LEFT HEAVY (FORMS UP LIGHT AND LEFT HEAVY) */
  ["\u251A", "\u251A"], /* BOX DRAWINGS UP HEAVY AND LEFT LIGHT (FORMS UP HEAVY AND LEFT LIGHT) */
  ["\u251B", "\u251B"], /* BOX DRAWINGS HEAVY UP AND LEFT (FORMS HEAVY UP AND LEFT) */
  ["\u251C", "\u251C"], /* BOX DRAWINGS LIGHT VERTICAL AND RIGHT (FORMS LIGHT VERTICAL AND RIGHT) */
  ["\u251D", "\u251D"], /* BOX DRAWINGS VERTICAL LIGHT AND RIGHT HEAVY (FORMS VERTICAL LIGHT AND RIGHT HEAVY) */
  ["\u251E", "\u251E"], /* BOX DRAWINGS UP HEAVY AND RIGHT DOWN LIGHT (FORMS UP HEAVY AND RIGHT DOWN LIGHT) */
  ["\u251F", "\u251F"], /* BOX DRAWINGS DOWN HEAVY AND RIGHT UP LIGHT (FORMS DOWN HEAVY AND RIGHT UP LIGHT) */
  ["\u2520", "\u2520"], /* BOX DRAWINGS VERTICAL HEAVY AND RIGHT LIGHT (FORMS VERTICAL HEAVY AND RIGHT LIGHT) */
  ["\u2521", "\u2521"], /* BOX DRAWINGS DOWN LIGHT AND RIGHT UP HEAVY (FORMS DOWN LIGHT AND RIGHT UP HEAVY) */
  ["\u2522", "\u2522"], /* BOX DRAWINGS UP LIGHT AND RIGHT DOWN HEAVY (FORMS UP LIGHT AND RIGHT DOWN HEAVY) */
  ["\u2523", "\u2523"], /* BOX DRAWINGS HEAVY VERTICAL AND RIGHT (FORMS HEAVY VERTICAL AND RIGHT) */
  ["\u2524", "\u2524"], /* BOX DRAWINGS LIGHT VERTICAL AND LEFT (FORMS LIGHT VERTICAL AND LEFT) */
  ["\u2525", "\u2525"], /* BOX DRAWINGS VERTICAL LIGHT AND LEFT HEAVY (FORMS VERTICAL LIGHT AND LEFT HEAVY) */
  ["\u2526", "\u2526"], /* BOX DRAWINGS UP HEAVY AND LEFT DOWN LIGHT (FORMS UP HEAVY AND LEFT DOWN LIGHT) */
  ["\u2527", "\u2527"], /* BOX DRAWINGS DOWN HEAVY AND LEFT UP LIGHT (FORMS DOWN HEAVY AND LEFT UP LIGHT) */
  ["\u2528", "\u2528"], /* BOX DRAWINGS VERTICAL HEAVY AND LEFT LIGHT (FORMS VERTICAL HEAVY AND LEFT LIGHT) */
  ["\u2529", "\u2529"], /* BOX DRAWINGS DOWN LIGHT AND LEFT UP HEAVY (FORMS DOWN LIGHT AND LEFT UP HEAVY) */
  ["\u252A", "\u252A"], /* BOX DRAWINGS UP LIGHT AND LEFT DOWN HEAVY (FORMS UP LIGHT AND LEFT DOWN HEAVY) */
  ["\u252B", "\u252B"], /* BOX DRAWINGS HEAVY VERTICAL AND LEFT (FORMS HEAVY VERTICAL AND LEFT) */
  ["\u252C", "\u252C"], /* BOX DRAWINGS LIGHT DOWN AND HORIZONTAL (FORMS LIGHT DOWN AND HORIZONTAL) */
  ["\u252D", "\u252D"], /* BOX DRAWINGS LEFT HEAVY AND RIGHT DOWN LIGHT (FORMS LEFT HEAVY AND RIGHT DOWN LIGHT) */
  ["\u252E", "\u252E"], /* BOX DRAWINGS RIGHT HEAVY AND LEFT DOWN LIGHT (FORMS RIGHT HEAVY AND LEFT DOWN LIGHT) */
  ["\u252F", "\u252F"], /* BOX DRAWINGS DOWN LIGHT AND HORIZONTAL HEAVY (FORMS DOWN LIGHT AND HORIZONTAL HEAVY) */
  ["\u2530", "\u2530"], /* BOX DRAWINGS DOWN HEAVY AND HORIZONTAL LIGHT (FORMS DOWN HEAVY AND HORIZONTAL LIGHT) */
  ["\u2531", "\u2531"], /* BOX DRAWINGS RIGHT LIGHT AND LEFT DOWN HEAVY (FORMS RIGHT LIGHT AND LEFT DOWN HEAVY) */
  ["\u2532", "\u2532"], /* BOX DRAWINGS LEFT LIGHT AND RIGHT DOWN HEAVY (FORMS LEFT LIGHT AND RIGHT DOWN HEAVY) */
  ["\u2533", "\u2533"], /* BOX DRAWINGS HEAVY DOWN AND HORIZONTAL (FORMS HEAVY DOWN AND HORIZONTAL) */
  ["\u2534", "\u2534"], /* BOX DRAWINGS LIGHT UP AND HORIZONTAL (FORMS LIGHT UP AND HORIZONTAL) */
  ["\u2535", "\u2535"], /* BOX DRAWINGS LEFT HEAVY AND RIGHT UP LIGHT (FORMS LEFT HEAVY AND RIGHT UP LIGHT) */
  ["\u2536", "\u2536"], /* BOX DRAWINGS RIGHT HEAVY AND LEFT UP LIGHT (FORMS RIGHT HEAVY AND LEFT UP LIGHT) */
  ["\u2537", "\u2537"], /* BOX DRAWINGS UP LIGHT AND HORIZONTAL HEAVY (FORMS UP LIGHT AND HORIZONTAL HEAVY) */
  ["\u2538", "\u2538"], /* BOX DRAWINGS UP HEAVY AND HORIZONTAL LIGHT (FORMS UP HEAVY AND HORIZONTAL LIGHT) */
  ["\u2539", "\u2539"], /* BOX DRAWINGS RIGHT LIGHT AND LEFT UP HEAVY (FORMS RIGHT LIGHT AND LEFT UP HEAVY) */
  ["\u253A", "\u253A"], /* BOX DRAWINGS LEFT LIGHT AND RIGHT UP HEAVY (FORMS LEFT LIGHT AND RIGHT UP HEAVY) */
  ["\u253B", "\u253B"], /* BOX DRAWINGS HEAVY UP AND HORIZONTAL (FORMS HEAVY UP AND HORIZONTAL) */
  ["\u253C", "\u253C"], /* BOX DRAWINGS LIGHT VERTICAL AND HORIZONTAL (FORMS LIGHT VERTICAL AND HORIZONTAL) */
  ["\u253D", "\u253D"], /* BOX DRAWINGS LEFT HEAVY AND RIGHT VERTICAL LIGHT (FORMS LEFT HEAVY AND RIGHT VERTICAL LIGHT) */
  ["\u253E", "\u253E"], /* BOX DRAWINGS RIGHT HEAVY AND LEFT VERTICAL LIGHT (FORMS RIGHT HEAVY AND LEFT VERTICAL LIGHT) */
  ["\u253F", "\u253F"], /* BOX DRAWINGS VERTICAL LIGHT AND HORIZONTAL HEAVY (FORMS VERTICAL LIGHT AND HORIZONTAL HEAVY) */
  ["\u2540", "\u2540"], /* BOX DRAWINGS UP HEAVY AND DOWN HORIZONTAL LIGHT (FORMS UP HEAVY AND DOWN HORIZONTAL LIGHT) */
  ["\u2541", "\u2541"], /* BOX DRAWINGS DOWN HEAVY AND UP HORIZONTAL LIGHT (FORMS DOWN HEAVY AND UP HORIZONTAL LIGHT) */
  ["\u2542", "\u2542"], /* BOX DRAWINGS VERTICAL HEAVY AND HORIZONTAL LIGHT (FORMS VERTICAL HEAVY AND HORIZONTAL LIGHT) */
  ["\u2543", "\u2543"], /* BOX DRAWINGS LEFT UP HEAVY AND RIGHT DOWN LIGHT (FORMS LEFT UP HEAVY AND RIGHT DOWN LIGHT) */
  ["\u2544", "\u2544"], /* BOX DRAWINGS RIGHT UP HEAVY AND LEFT DOWN LIGHT (FORMS RIGHT UP HEAVY AND LEFT DOWN LIGHT) */
  ["\u2545", "\u2545"], /* BOX DRAWINGS LEFT DOWN HEAVY AND RIGHT UP LIGHT (FORMS LEFT DOWN HEAVY AND RIGHT UP LIGHT) */
  ["\u2546", "\u2546"], /* BOX DRAWINGS RIGHT DOWN HEAVY AND LEFT UP LIGHT (FORMS RIGHT DOWN HEAVY AND LEFT UP LIGHT) */
  ["\u2547", "\u2547"], /* BOX DRAWINGS DOWN LIGHT AND UP HORIZONTAL HEAVY (FORMS DOWN LIGHT AND UP HORIZONTAL HEAVY) */
  ["\u2548", "\u2548"], /* BOX DRAWINGS UP LIGHT AND DOWN HORIZONTAL HEAVY (FORMS UP LIGHT AND DOWN HORIZONTAL HEAVY) */
  ["\u2549", "\u2549"], /* BOX DRAWINGS RIGHT LIGHT AND LEFT VERTICAL HEAVY (FORMS RIGHT LIGHT AND LEFT VERTICAL HEAVY) */
  ["\u254A", "\u254A"], /* BOX DRAWINGS LEFT LIGHT AND RIGHT VERTICAL HEAVY (FORMS LEFT LIGHT AND RIGHT VERTICAL HEAVY) */
  ["\u254B", "\u254B"], /* BOX DRAWINGS HEAVY VERTICAL AND HORIZONTAL (FORMS HEAVY VERTICAL AND HORIZONTAL) */
  ["\u254C", "\u254C"], /* BOX DRAWINGS LIGHT DOUBLE DASH HORIZONTAL (FORMS LIGHT DOUBLE DASH HORIZONTAL) */
  ["\u254D", "\u254D"], /* BOX DRAWINGS HEAVY DOUBLE DASH HORIZONTAL (FORMS HEAVY DOUBLE DASH HORIZONTAL) */
  ["\u254E", "\u254E"], /* BOX DRAWINGS LIGHT DOUBLE DASH VERTICAL (FORMS LIGHT DOUBLE DASH VERTICAL) */
  ["\u254F", "\u254F"], /* BOX DRAWINGS HEAVY DOUBLE DASH VERTICAL (FORMS HEAVY DOUBLE DASH VERTICAL) */
  ["\u2550", "\u2550"], /* BOX DRAWINGS DOUBLE HORIZONTAL (FORMS DOUBLE HORIZONTAL) */
  ["\u2551", "\u2551"], /* BOX DRAWINGS DOUBLE VERTICAL (FORMS DOUBLE VERTICAL) */
  ["\u2552", "\u2552"], /* BOX DRAWINGS DOWN SINGLE AND RIGHT DOUBLE (FORMS DOWN SINGLE AND RIGHT DOUBLE) */
  ["\u2553", "\u2553"], /* BOX DRAWINGS DOWN DOUBLE AND RIGHT SINGLE (FORMS DOWN DOUBLE AND RIGHT SINGLE) */
  ["\u2554", "\u2554"], /* BOX DRAWINGS DOUBLE DOWN AND RIGHT (FORMS DOUBLE DOWN AND RIGHT) */
  ["\u2555", "\u2555"], /* BOX DRAWINGS DOWN SINGLE AND LEFT DOUBLE (FORMS DOWN SINGLE AND LEFT DOUBLE) */
  ["\u2556", "\u2556"], /* BOX DRAWINGS DOWN DOUBLE AND LEFT SINGLE (FORMS DOWN DOUBLE AND LEFT SINGLE) */
  ["\u2557", "\u2557"], /* BOX DRAWINGS DOUBLE DOWN AND LEFT (FORMS DOUBLE DOWN AND LEFT) */
  ["\u2558", "\u2558"], /* BOX DRAWINGS UP SINGLE AND RIGHT DOUBLE (FORMS UP SINGLE AND RIGHT DOUBLE) */
  ["\u2559", "\u2559"], /* BOX DRAWINGS UP DOUBLE AND RIGHT SINGLE (FORMS UP DOUBLE AND RIGHT SINGLE) */
  ["\u255A", "\u255A"], /* BOX DRAWINGS DOUBLE UP AND RIGHT (FORMS DOUBLE UP AND RIGHT) */
  ["\u255B", "\u255B"], /* BOX DRAWINGS UP SINGLE AND LEFT DOUBLE (FORMS UP SINGLE AND LEFT DOUBLE) */
  ["\u255C", "\u255C"], /* BOX DRAWINGS UP DOUBLE AND LEFT SINGLE (FORMS UP DOUBLE AND LEFT SINGLE) */
  ["\u255D", "\u255D"], /* BOX DRAWINGS DOUBLE UP AND LEFT (FORMS DOUBLE UP AND LEFT) */
  ["\u255E", "\u255E"], /* BOX DRAWINGS VERTICAL SINGLE AND RIGHT DOUBLE (FORMS VERTICAL SINGLE AND RIGHT DOUBLE) */
  ["\u255F", "\u255F"], /* BOX DRAWINGS VERTICAL DOUBLE AND RIGHT SINGLE (FORMS VERTICAL DOUBLE AND RIGHT SINGLE) */
  ["\u2560", "\u2560"], /* BOX DRAWINGS DOUBLE VERTICAL AND RIGHT (FORMS DOUBLE VERTICAL AND RIGHT) */
  ["\u2561", "\u2561"], /* BOX DRAWINGS VERTICAL SINGLE AND LEFT DOUBLE (FORMS VERTICAL SINGLE AND LEFT DOUBLE) */
  ["\u2562", "\u2562"], /* BOX DRAWINGS VERTICAL DOUBLE AND LEFT SINGLE (FORMS VERTICAL DOUBLE AND LEFT SINGLE) */
  ["\u2563", "\u2563"], /* BOX DRAWINGS DOUBLE VERTICAL AND LEFT (FORMS DOUBLE VERTICAL AND LEFT) */
  ["\u2564", "\u2564"], /* BOX DRAWINGS DOWN SINGLE AND HORIZONTAL DOUBLE (FORMS DOWN SINGLE AND HORIZONTAL DOUBLE) */
  ["\u2565", "\u2565"], /* BOX DRAWINGS DOWN DOUBLE AND HORIZONTAL SINGLE (FORMS DOWN DOUBLE AND HORIZONTAL SINGLE) */
  ["\u2566", "\u2566"], /* BOX DRAWINGS DOUBLE DOWN AND HORIZONTAL (FORMS DOUBLE DOWN AND HORIZONTAL) */
  ["\u2567", "\u2567"], /* BOX DRAWINGS UP SINGLE AND HORIZONTAL DOUBLE (FORMS UP SINGLE AND HORIZONTAL DOUBLE) */
  ["\u2568", "\u2568"], /* BOX DRAWINGS UP DOUBLE AND HORIZONTAL SINGLE (FORMS UP DOUBLE AND HORIZONTAL SINGLE) */
  ["\u2569", "\u2569"], /* BOX DRAWINGS DOUBLE UP AND HORIZONTAL (FORMS DOUBLE UP AND HORIZONTAL) */
  ["\u256A", "\u256A"], /* BOX DRAWINGS VERTICAL SINGLE AND HORIZONTAL DOUBLE (FORMS VERTICAL SINGLE AND HORIZONTAL DOUBLE) */
  ["\u256B", "\u256B"], /* BOX DRAWINGS VERTICAL DOUBLE AND HORIZONTAL SINGLE (FORMS VERTICAL DOUBLE AND HORIZONTAL SINGLE) */
  ["\u256C", "\u256C"], /* BOX DRAWINGS DOUBLE VERTICAL AND HORIZONTAL (FORMS DOUBLE VERTICAL AND HORIZONTAL) */
  ["\u256D", "\u256D"], /* BOX DRAWINGS LIGHT ARC DOWN AND RIGHT (FORMS LIGHT ARC DOWN AND RIGHT) */
  ["\u256E", "\u256E"], /* BOX DRAWINGS LIGHT ARC DOWN AND LEFT (FORMS LIGHT ARC DOWN AND LEFT) */
  ["\u256F", "\u256F"], /* BOX DRAWINGS LIGHT ARC UP AND LEFT (FORMS LIGHT ARC UP AND LEFT) */
  ["\u2570", "\u2570"], /* BOX DRAWINGS LIGHT ARC UP AND RIGHT (FORMS LIGHT ARC UP AND RIGHT) */
  ["\u2571", "\u2571"], /* BOX DRAWINGS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT (FORMS LIGHT DIAGONAL UPPER RIGHT TO LOWER LEFT) */
  ["\u2572", "\u2572"], /* BOX DRAWINGS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT (FORMS LIGHT DIAGONAL UPPER LEFT TO LOWER RIGHT) */
  ["\u2573", "\u2573"], /* BOX DRAWINGS LIGHT DIAGONAL CROSS (FORMS LIGHT DIAGONAL CROSS) */
  ["\u2574", "\u2574"], /* BOX DRAWINGS LIGHT LEFT (FORMS LIGHT LEFT) */
  ["\u2575", "\u2575"], /* BOX DRAWINGS LIGHT UP (FORMS LIGHT UP) */
  ["\u2576", "\u2576"], /* BOX DRAWINGS LIGHT RIGHT (FORMS LIGHT RIGHT) */
  ["\u2577", "\u2577"], /* BOX DRAWINGS LIGHT DOWN (FORMS LIGHT DOWN) */
  ["\u2578", "\u2578"], /* BOX DRAWINGS HEAVY LEFT (FORMS HEAVY LEFT) */
  ["\u2579", "\u2579"], /* BOX DRAWINGS HEAVY UP (FORMS HEAVY UP) */
  ["\u257A", "\u257A"], /* BOX DRAWINGS HEAVY RIGHT (FORMS HEAVY RIGHT) */
  ["\u257B", "\u257B"], /* BOX DRAWINGS HEAVY DOWN (FORMS HEAVY DOWN) */
  ["\u257C", "\u257C"], /* BOX DRAWINGS LIGHT LEFT AND HEAVY RIGHT (FORMS LIGHT LEFT AND HEAVY RIGHT) */
  ["\u257D", "\u257D"], /* BOX DRAWINGS LIGHT UP AND HEAVY DOWN (FORMS LIGHT UP AND HEAVY DOWN) */
  ["\u257E", "\u257E"], /* BOX DRAWINGS HEAVY LEFT AND LIGHT RIGHT (FORMS HEAVY LEFT AND LIGHT RIGHT) */
  ["\u257F", "\u257F"], /* BOX DRAWINGS HEAVY UP AND LIGHT DOWN (FORMS HEAVY UP AND LIGHT DOWN) */
  ["\u2580", "\u2580"], /* UPPER HALF BLOCK */
  ["\u2581", "\u2581"], /* LOWER ONE EIGHTH BLOCK */
  ["\u2582", "\u2582"], /* LOWER ONE QUARTER BLOCK */
  ["\u2583", "\u2583"], /* LOWER THREE EIGHTHS BLOCK */
  ["\u2584", "\u2584"], /* LOWER HALF BLOCK */
  ["\u2585", "\u2585"], /* LOWER FIVE EIGHTHS BLOCK */
  ["\u2586", "\u2586"], /* LOWER THREE QUARTERS BLOCK (LOWER THREE QUARTER BLOCK) */
  ["\u2587", "\u2587"], /* LOWER SEVEN EIGHTHS BLOCK */
  ["\u2588", "\u2588"], /* FULL BLOCK */
  ["\u2589", "\u2589"], /* LEFT SEVEN EIGHTHS BLOCK */
  ["\u258A", "\u258A"], /* LEFT THREE QUARTERS BLOCK (LEFT THREE QUARTER BLOCK) */
  ["\u258B", "\u258B"], /* LEFT FIVE EIGHTHS BLOCK */
  ["\u258C", "\u258C"], /* LEFT HALF BLOCK */
  ["\u258D", "\u258D"], /* LEFT THREE EIGHTHS BLOCK */
  ["\u258E", "\u258E"], /* LEFT ONE QUARTER BLOCK */
  ["\u258F", "\u258F"], /* LEFT ONE EIGHTH BLOCK */
  ["\u2590", "\u2590"], /* RIGHT HALF BLOCK */
  ["\u2591", "\u2591"], /* LIGHT SHADE */
  ["\u2592", "\u2592"], /* MEDIUM SHADE */
  ["\u2593", "\u2593"], /* DARK SHADE */
  ["\u2594", "\u2594"], /* UPPER ONE EIGHTH BLOCK */
  ["\u2595", "\u2595"], /* RIGHT ONE EIGHTH BLOCK */
  ["\u2596", "\u2596"], /* QUADRANT LOWER LEFT */
  ["\u2597", "\u2597"], /* QUADRANT LOWER RIGHT */
  ["\u2598", "\u2598"], /* QUADRANT UPPER LEFT */
  ["\u2599", "\u2599"], /* QUADRANT UPPER LEFT AND LOWER LEFT AND LOWER RIGHT */
  ["\u259A", "\u259A"], /* QUADRANT UPPER LEFT AND LOWER RIGHT */
  ["\u259B", "\u259B"], /* QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER LEFT */
  ["\u259C", "\u259C"], /* QUADRANT UPPER LEFT AND UPPER RIGHT AND LOWER RIGHT */
  ["\u259D", "\u259D"], /* QUADRANT UPPER RIGHT */
  ["\u259E", "\u259E"], /* QUADRANT UPPER RIGHT AND LOWER LEFT */
  ["\u259F", "\u259F"], /* QUADRANT UPPER RIGHT AND LOWER LEFT AND LOWER RIGHT */
  ["\u25A0", "\u25A0"], /* BLACK SQUARE */
  ["\u25A1", "\u25A1"], /* WHITE SQUARE */
  ["\u25A2", "\u25A2"], /* WHITE SQUARE WITH ROUNDED CORNERS */
  ["\u25A3", "\u25A3"], /* WHITE SQUARE CONTAINING BLACK SMALL SQUARE */
  ["\u25A4", "\u25A4"], /* SQUARE WITH HORIZONTAL FILL */
  ["\u25A5", "\u25A5"], /* SQUARE WITH VERTICAL FILL */
  ["\u25A6", "\u25A6"], /* SQUARE WITH ORTHOGONAL CROSSHATCH FILL */
  ["\u25A7", "\u25A7"], /* SQUARE WITH UPPER LEFT TO LOWER RIGHT FILL */
  ["\u25A8", "\u25A8"], /* SQUARE WITH UPPER RIGHT TO LOWER LEFT FILL */
  ["\u25A9", "\u25A9"], /* SQUARE WITH DIAGONAL CROSSHATCH FILL */
  ["\u25AA", "\u25AA"], /* BLACK SMALL SQUARE */
  ["\u25AB", "\u25AB"], /* WHITE SMALL SQUARE */
  ["\u25AC", "\u25AC"], /* BLACK RECTANGLE */
  ["\u25AD", "\u25AD"], /* WHITE RECTANGLE */
  ["\u25AE", "\u25AE"], /* BLACK VERTICAL RECTANGLE */
  ["\u25AF", "\u25AF"], /* WHITE VERTICAL RECTANGLE */
  ["\u25B0", "\u25B0"], /* BLACK PARALLELOGRAM */
  ["\u25B1", "\u25B1"], /* WHITE PARALLELOGRAM */
  ["\u25B2", "\u25B2"], /* BLACK UP-POINTING TRIANGLE (BLACK UP POINTING TRIANGLE) */
  ["\u25B3", "\u25B3"], /* WHITE UP-POINTING TRIANGLE (WHITE UP POINTING TRIANGLE) */
  ["\u25B4", "\u25B4"], /* BLACK UP-POINTING SMALL TRIANGLE (BLACK UP POINTING SMALL TRIANGLE) */
  ["\u25B5", "\u25B5"], /* WHITE UP-POINTING SMALL TRIANGLE (WHITE UP POINTING SMALL TRIANGLE) */
  ["\u25B6", "\u25B6"], /* BLACK RIGHT-POINTING TRIANGLE (BLACK RIGHT POINTING TRIANGLE) */
  ["\u25B7", "\u25B7"], /* WHITE RIGHT-POINTING TRIANGLE (WHITE RIGHT POINTING TRIANGLE) */
  ["\u25B8", "\u25B8"], /* BLACK RIGHT-POINTING SMALL TRIANGLE (BLACK RIGHT POINTING SMALL TRIANGLE) */
  ["\u25B9", "\u25B9"], /* WHITE RIGHT-POINTING SMALL TRIANGLE (WHITE RIGHT POINTING SMALL TRIANGLE) */
  ["\u25BA", "\u25BA"], /* BLACK RIGHT-POINTING POINTER (BLACK RIGHT POINTING POINTER) */
  ["\u25BB", "\u25BB"], /* WHITE RIGHT-POINTING POINTER (WHITE RIGHT POINTING POINTER) */
  ["\u25BC", "\u25BC"], /* BLACK DOWN-POINTING TRIANGLE (BLACK DOWN POINTING TRIANGLE) */
  ["\u25BD", "\u25BD"], /* WHITE DOWN-POINTING TRIANGLE (WHITE DOWN POINTING TRIANGLE) */
  ["\u25BE", "\u25BE"], /* BLACK DOWN-POINTING SMALL TRIANGLE (BLACK DOWN POINTING SMALL TRIANGLE) */
  ["\u25BF", "\u25BF"], /* WHITE DOWN-POINTING SMALL TRIANGLE (WHITE DOWN POINTING SMALL TRIANGLE) */
  ["\u25C0", "\u25C0"], /* BLACK LEFT-POINTING TRIANGLE (BLACK LEFT POINTING TRIANGLE) */
  ["\u25C1", "\u25C1"], /* WHITE LEFT-POINTING TRIANGLE (WHITE LEFT POINTING TRIANGLE) */
  ["\u25C2", "\u25C2"], /* BLACK LEFT-POINTING SMALL TRIANGLE (BLACK LEFT POINTING SMALL TRIANGLE) */
  ["\u25C3", "\u25C3"], /* WHITE LEFT-POINTING SMALL TRIANGLE (WHITE LEFT POINTING SMALL TRIANGLE) */
  ["\u25C4", "\u25C4"], /* BLACK LEFT-POINTING POINTER (BLACK LEFT POINTING POINTER) */
  ["\u25C5", "\u25C5"], /* WHITE LEFT-POINTING POINTER (WHITE LEFT POINTING POINTER) */
  ["\u25C6", "\u25C6"], /* BLACK DIAMOND */
  ["\u25C7", "\u25C7"], /* WHITE DIAMOND */
  ["\u25C8", "\u25C8"], /* WHITE DIAMOND CONTAINING BLACK SMALL DIAMOND */
  ["\u25C9", "\u25C9"], /* FISHEYE */
  ["\u25CA", "\u25CA"], /* LOZENGE */
  ["\u25CB", "\u25CB"], /* WHITE CIRCLE */
  ["\u25CC", "\u25CC"], /* DOTTED CIRCLE */
  ["\u25CD", "\u25CD"], /* CIRCLE WITH VERTICAL FILL */
  ["\u25CE", "\u25CE"], /* BULLSEYE */
  ["\u25CF", "\u25CF"], /* BLACK CIRCLE */
  ["\u25D0", "\u25D0"], /* CIRCLE WITH LEFT HALF BLACK */
  ["\u25D1", "\u25D1"], /* CIRCLE WITH RIGHT HALF BLACK */
  ["\u25D2", "\u25D2"], /* CIRCLE WITH LOWER HALF BLACK */
  ["\u25D3", "\u25D3"], /* CIRCLE WITH UPPER HALF BLACK */
  ["\u25D4", "\u25D4"], /* CIRCLE WITH UPPER RIGHT QUADRANT BLACK */
  ["\u25D5", "\u25D5"], /* CIRCLE WITH ALL BUT UPPER LEFT QUADRANT BLACK */
  ["\u25D6", "\u25D6"], /* LEFT HALF BLACK CIRCLE */
  ["\u25D7", "\u25D7"], /* RIGHT HALF BLACK CIRCLE */
  ["\u25D8", "\u25D8"], /* INVERSE BULLET */
  ["\u25D9", "\u25D9"], /* INVERSE WHITE CIRCLE */
  ["\u25DA", "\u25DA"], /* UPPER HALF INVERSE WHITE CIRCLE */
  ["\u25DB", "\u25DB"], /* LOWER HALF INVERSE WHITE CIRCLE */
  ["\u25DC", "\u25DC"], /* UPPER LEFT QUADRANT CIRCULAR ARC */
  ["\u25DD", "\u25DD"], /* UPPER RIGHT QUADRANT CIRCULAR ARC */
  ["\u25DE", "\u25DE"], /* LOWER RIGHT QUADRANT CIRCULAR ARC */
  ["\u25DF", "\u25DF"], /* LOWER LEFT QUADRANT CIRCULAR ARC */
  ["\u25E0", "\u25E0"], /* UPPER HALF CIRCLE */
  ["\u25E1", "\u25E1"], /* LOWER HALF CIRCLE */
  ["\u25E2", "\u25E2"], /* BLACK LOWER RIGHT TRIANGLE */
  ["\u25E3", "\u25E3"], /* BLACK LOWER LEFT TRIANGLE */
  ["\u25E4", "\u25E4"], /* BLACK UPPER LEFT TRIANGLE */
  ["\u25E5", "\u25E5"], /* BLACK UPPER RIGHT TRIANGLE */
  ["\u25E6", "\u25E6"], /* WHITE BULLET */
  ["\u25E7", "\u25E7"], /* SQUARE WITH LEFT HALF BLACK */
  ["\u25E8", "\u25E8"], /* SQUARE WITH RIGHT HALF BLACK */
  ["\u25E9", "\u25E9"], /* SQUARE WITH UPPER LEFT DIAGONAL HALF BLACK */
  ["\u25EA", "\u25EA"], /* SQUARE WITH LOWER RIGHT DIAGONAL HALF BLACK */
  ["\u25EB", "\u25EB"], /* WHITE SQUARE WITH VERTICAL BISECTING LINE */
  ["\u25EC", "\u25EC"], /* WHITE UP-POINTING TRIANGLE WITH DOT (WHITE UP POINTING TRIANGLE WITH DOT) */
  ["\u25ED", "\u25ED"], /* UP-POINTING TRIANGLE WITH LEFT HALF BLACK (UP POINTING TRIANGLE WITH LEFT HALF BLACK) */
  ["\u25EE", "\u25EE"], /* UP-POINTING TRIANGLE WITH RIGHT HALF BLACK (UP POINTING TRIANGLE WITH RIGHT HALF BLACK) */
  ["\u25EF", "\u25EF"], /* LARGE CIRCLE */
  ["\u25F0", "\u25F0"], /* WHITE SQUARE WITH UPPER LEFT QUADRANT */
  ["\u25F1", "\u25F1"], /* WHITE SQUARE WITH LOWER LEFT QUADRANT */
  ["\u25F2", "\u25F2"], /* WHITE SQUARE WITH LOWER RIGHT QUADRANT */
  ["\u25F3", "\u25F3"], /* WHITE SQUARE WITH UPPER RIGHT QUADRANT */
  ["\u25F4", "\u25F4"], /* WHITE CIRCLE WITH UPPER LEFT QUADRANT */
  ["\u25F5", "\u25F5"], /* WHITE CIRCLE WITH LOWER LEFT QUADRANT */
  ["\u25F6", "\u25F6"], /* WHITE CIRCLE WITH LOWER RIGHT QUADRANT */
  ["\u25F7", "\u25F7"], /* WHITE CIRCLE WITH UPPER RIGHT QUADRANT */
  ["\u25F8", "\u25F8"], /* UPPER LEFT TRIANGLE */
  ["\u25F9", "\u25F9"], /* UPPER RIGHT TRIANGLE */
  ["\u25FA", "\u25FA"], /* LOWER LEFT TRIANGLE */
  ["\u25FB", "\u25FB"], /* WHITE MEDIUM SQUARE */
  ["\u25FC", "\u25FC"], /* BLACK MEDIUM SQUARE */
  ["\u25FD", "\u25FD"], /* WHITE MEDIUM SMALL SQUARE */
  ["\u25FE", "\u25FE"], /* BLACK MEDIUM SMALL SQUARE */
  ["\u25FF", "\u25FF"], /* LOWER RIGHT TRIANGLE */
  ["\u2600", "\u2600"], /* BLACK SUN WITH RAYS */
  ["\u2601", "\u2601"], /* CLOUD */
  ["\u2602", "\u2602"], /* UMBRELLA */
  ["\u2603", "\u2603"], /* SNOWMAN */
  ["\u2604", "\u2604"], /* COMET */
  ["\u2605", "\u2605"], /* BLACK STAR */
  ["\u2606", "\u2606"], /* WHITE STAR */
  ["\u2607", "\u2607"], /* LIGHTNING */
  ["\u2608", "\u2608"], /* THUNDERSTORM */
  ["\u2609", "\u2609"], /* SUN */
  ["\u260A", "\u260A"], /* ASCENDING NODE */
  ["\u260B", "\u260B"], /* DESCENDING NODE */
  ["\u260C", "\u260C"], /* CONJUNCTION */
  ["\u260D", "\u260D"], /* OPPOSITION */
  ["\u260E", "\u260E"], /* BLACK TELEPHONE */
  ["\u260F", "\u260F"], /* WHITE TELEPHONE */
  ["\u2610", "\u2610"], /* BALLOT BOX */
  ["\u2611", "\u2611"], /* BALLOT BOX WITH CHECK */
  ["\u2612", "\u2612"], /* BALLOT BOX WITH X */
  ["\u2613", "\u2613"], /* SALTIRE */
  ["\u2614", "\u2614"], /* UMBRELLA WITH RAIN DROPS */
  ["\u2615", "\u2615"], /* HOT BEVERAGE */
  ["\u2616", "\u2616"], /* WHITE SHOGI PIECE */
  ["\u2617", "\u2617"], /* BLACK SHOGI PIECE */
  ["\u2618", "\u2618"], /* SHAMROCK */
  ["\u2619", "\u2619"], /* REVERSED ROTATED FLORAL HEART BULLET */
  ["\u261A", "\u261A"], /* BLACK LEFT POINTING INDEX */
  ["\u261B", "\u261B"], /* BLACK RIGHT POINTING INDEX */
  ["\u261C", "\u261C"], /* WHITE LEFT POINTING INDEX */
  ["\u261D", "\u261D"], /* WHITE UP POINTING INDEX */
  ["\u261E", "\u261E"], /* WHITE RIGHT POINTING INDEX */
  ["\u261F", "\u261F"], /* WHITE DOWN POINTING INDEX */
  ["\u2620", "\u2620"], /* SKULL AND CROSSBONES */
  ["\u2621", "\u2621"], /* CAUTION SIGN */
  ["\u2622", "\u2622"], /* RADIOACTIVE SIGN */
  ["\u2623", "\u2623"], /* BIOHAZARD SIGN */
  ["\u2624", "\u2624"], /* CADUCEUS */
  ["\u2625", "\u2625"], /* ANKH */
  ["\u2626", "\u2626"], /* ORTHODOX CROSS */
  ["\u2627", "\u2627"], /* CHI RHO */
  ["\u2628", "\u2628"], /* CROSS OF LORRAINE */
  ["\u2629", "\u2629"], /* CROSS OF JERUSALEM */
  ["\u262A", "\u262A"], /* STAR AND CRESCENT */
  ["\u262B", "\u262B"], /* FARSI SYMBOL (SYMBOL OF IRAN) */
  ["\u262C", "\u262C"], /* ADI SHAKTI */
  ["\u262D", "\u262D"], /* HAMMER AND SICKLE */
  ["\u262E", "\u262E"], /* PEACE SYMBOL */
  ["\u262F", "\u262F"], /* YIN YANG */
  ["\u2630", "\u2630"], /* TRIGRAM FOR HEAVEN */
  ["\u2631", "\u2631"], /* TRIGRAM FOR LAKE */
  ["\u2632", "\u2632"], /* TRIGRAM FOR FIRE */
  ["\u2633", "\u2633"], /* TRIGRAM FOR THUNDER */
  ["\u2634", "\u2634"], /* TRIGRAM FOR WIND */
  ["\u2635", "\u2635"], /* TRIGRAM FOR WATER */
  ["\u2636", "\u2636"], /* TRIGRAM FOR MOUNTAIN */
  ["\u2637", "\u2637"], /* TRIGRAM FOR EARTH */
  ["\u2638", "\u2638"], /* WHEEL OF DHARMA */
  ["\u2639", "\u2639"], /* WHITE FROWNING FACE */
  ["\u263A", "\u263A"], /* WHITE SMILING FACE */
  ["\u263B", "\u263B"], /* BLACK SMILING FACE */
  ["\u263C", "\u263C"], /* WHITE SUN WITH RAYS */
  ["\u263D", "\u263D"], /* FIRST QUARTER MOON */
  ["\u263E", "\u263E"], /* LAST QUARTER MOON */
  ["\u263F", "\u263F"], /* MERCURY */
  ["\u2640", "\u2640"], /* FEMALE SIGN */
  ["\u2641", "\u2641"], /* EARTH */
  ["\u2642", "\u2642"], /* MALE SIGN */
  ["\u2643", "\u2643"], /* JUPITER */
  ["\u2644", "\u2644"], /* SATURN */
  ["\u2645", "\u2645"], /* URANUS */
  ["\u2646", "\u2646"], /* NEPTUNE */
  ["\u2647", "\u2647"], /* PLUTO */
  ["\u2648", "\u2648"], /* ARIES */
  ["\u2649", "\u2649"], /* TAURUS */
  ["\u264A", "\u264A"], /* GEMINI */
  ["\u264B", "\u264B"], /* CANCER */
  ["\u264C", "\u264C"], /* LEO */
  ["\u264D", "\u264D"], /* VIRGO */
  ["\u264E", "\u264E"], /* LIBRA */
  ["\u264F", "\u264F"], /* SCORPIUS */
  ["\u2650", "\u2650"], /* SAGITTARIUS */
  ["\u2651", "\u2651"], /* CAPRICORN */
  ["\u2652", "\u2652"], /* AQUARIUS */
  ["\u2653", "\u2653"], /* PISCES */
  ["\u2654", "\u2654"], /* WHITE CHESS KING */
  ["\u2655", "\u2655"], /* WHITE CHESS QUEEN */
  ["\u2656", "\u2656"], /* WHITE CHESS ROOK */
  ["\u2657", "\u2657"], /* WHITE CHESS BISHOP */
  ["\u2658", "\u2658"], /* WHITE CHESS KNIGHT */
  ["\u2659", "\u2659"], /* WHITE CHESS PAWN */
  ["\u265A", "\u265A"], /* BLACK CHESS KING */
  ["\u265B", "\u265B"], /* BLACK CHESS QUEEN */
  ["\u265C", "\u265C"], /* BLACK CHESS ROOK */
  ["\u265D", "\u265D"], /* BLACK CHESS BISHOP */
  ["\u265E", "\u265E"], /* BLACK CHESS KNIGHT */
  ["\u265F", "\u265F"], /* BLACK CHESS PAWN */
  ["\u2660", "\u2660"], /* BLACK SPADE SUIT */
  ["\u2661", "\u2661"], /* WHITE HEART SUIT */
  ["\u2662", "\u2662"], /* WHITE DIAMOND SUIT */
  ["\u2663", "\u2663"], /* BLACK CLUB SUIT */
  ["\u2664", "\u2664"], /* WHITE SPADE SUIT */
  ["\u2665", "\u2665"], /* BLACK HEART SUIT */
  ["\u2666", "\u2666"], /* BLACK DIAMOND SUIT */
  ["\u2667", "\u2667"], /* WHITE CLUB SUIT */
  ["\u2668", "\u2668"], /* HOT SPRINGS */
  ["\u2669", "\u2669"], /* QUARTER NOTE */
  ["\u266A", "\u266A"], /* EIGHTH NOTE */
  ["\u266B", "\u266B"], /* BEAMED EIGHTH NOTES (BARRED EIGHTH NOTES) */
  ["\u266C", "\u266C"], /* BEAMED SIXTEENTH NOTES (BARRED SIXTEENTH NOTES) */
  ["\u266D", "\u266D"], /* MUSIC FLAT SIGN (FLAT) */
  ["\u266E", "\u266E"], /* MUSIC NATURAL SIGN (NATURAL) */
  ["\u266F", "\u266F"], /* MUSIC SHARP SIGN (SHARP) */
  ["\u2670", "\u2670"], /* WEST SYRIAC CROSS */
  ["\u2671", "\u2671"], /* EAST SYRIAC CROSS */
  ["\u2672", "\u2672"], /* UNIVERSAL RECYCLING SYMBOL */
  ["\u2673", "\u2673"], /* RECYCLING SYMBOL FOR TYPE-1 PLASTICS */
  ["\u2674", "\u2674"], /* RECYCLING SYMBOL FOR TYPE-2 PLASTICS */
  ["\u2675", "\u2675"], /* RECYCLING SYMBOL FOR TYPE-3 PLASTICS */
  ["\u2676", "\u2676"], /* RECYCLING SYMBOL FOR TYPE-4 PLASTICS */
  ["\u2677", "\u2677"], /* RECYCLING SYMBOL FOR TYPE-5 PLASTICS */
  ["\u2678", "\u2678"], /* RECYCLING SYMBOL FOR TYPE-6 PLASTICS */
  ["\u2679", "\u2679"], /* RECYCLING SYMBOL FOR TYPE-7 PLASTICS */
  ["\u267A", "\u267A"], /* RECYCLING SYMBOL FOR GENERIC MATERIALS */
  ["\u267B", "\u267B"], /* BLACK UNIVERSAL RECYCLING SYMBOL */
  ["\u267C", "\u267C"], /* RECYCLED PAPER SYMBOL */
  ["\u267D", "\u267D"], /* PARTIALLY-RECYCLED PAPER SYMBOL */
  ["\u267E", "\u267E"], /* PERMANENT PAPER SIGN */
  ["\u267F", "\u267F"], /* WHEELCHAIR SYMBOL */
  ["\u2680", "\u2680"], /* DIE FACE-1 */
  ["\u2681", "\u2681"], /* DIE FACE-2 */
  ["\u2682", "\u2682"], /* DIE FACE-3 */
  ["\u2683", "\u2683"], /* DIE FACE-4 */
  ["\u2684", "\u2684"], /* DIE FACE-5 */
  ["\u2685", "\u2685"], /* DIE FACE-6 */
  ["\u2686", "\u2686"], /* WHITE CIRCLE WITH DOT RIGHT */
  ["\u2687", "\u2687"], /* WHITE CIRCLE WITH TWO DOTS */
  ["\u2688", "\u2688"], /* BLACK CIRCLE WITH WHITE DOT RIGHT */
  ["\u2689", "\u2689"], /* BLACK CIRCLE WITH TWO WHITE DOTS */
  ["\u268A", "\u268A"], /* MONOGRAM FOR YANG */
  ["\u268B", "\u268B"], /* MONOGRAM FOR YIN */
  ["\u268C", "\u268C"], /* DIGRAM FOR GREATER YANG */
  ["\u268D", "\u268D"], /* DIGRAM FOR LESSER YIN */
  ["\u268E", "\u268E"], /* DIGRAM FOR LESSER YANG */
  ["\u268F", "\u268F"], /* DIGRAM FOR GREATER YIN */
  ["\u2690", "\u2690"], /* WHITE FLAG */
  ["\u2691", "\u2691"], /* BLACK FLAG */
  ["\u2692", "\u2692"], /* HAMMER AND PICK */
  ["\u2693", "\u2693"], /* ANCHOR */
  ["\u2694", "\u2694"], /* CROSSED SWORDS */
  ["\u2695", "\u2695"], /* STAFF OF AESCULAPIUS */
  ["\u2696", "\u2696"], /* SCALES */
  ["\u2697", "\u2697"], /* ALEMBIC */
  ["\u2698", "\u2698"], /* FLOWER */
  ["\u2699", "\u2699"], /* GEAR */
  ["\u269A", "\u269A"], /* STAFF OF HERMES */
  ["\u269B", "\u269B"], /* ATOM SYMBOL */
  ["\u269C", "\u269C"], /* FLEUR-DE-LIS */
  ["\u269D", "\u269D"], /* OUTLINED WHITE STAR */
  ["\u269E", "\u269E"], /* THREE LINES CONVERGING RIGHT */
  ["\u269F", "\u269F"], /* THREE LINES CONVERGING LEFT */
  ["\u26A0", "\u26A0"], /* WARNING SIGN */
  ["\u26A1", "\u26A1"], /* HIGH VOLTAGE SIGN */
  ["\u26A2", "\u26A2"], /* DOUBLED FEMALE SIGN */
  ["\u26A3", "\u26A3"], /* DOUBLED MALE SIGN */
  ["\u26A4", "\u26A4"], /* INTERLOCKED FEMALE AND MALE SIGN */
  ["\u26A5", "\u26A5"], /* MALE AND FEMALE SIGN */
  ["\u26A6", "\u26A6"], /* MALE WITH STROKE SIGN */
  ["\u26A7", "\u26A7"], /* MALE WITH STROKE AND MALE AND FEMALE SIGN */
  ["\u26A8", "\u26A8"], /* VERTICAL MALE WITH STROKE SIGN */
  ["\u26A9", "\u26A9"], /* HORIZONTAL MALE WITH STROKE SIGN */
  ["\u26AA", "\u26AA"], /* MEDIUM WHITE CIRCLE */
  ["\u26AB", "\u26AB"], /* MEDIUM BLACK CIRCLE */
  ["\u26AC", "\u26AC"], /* MEDIUM SMALL WHITE CIRCLE */
  ["\u26AD", "\u26AD"], /* MARRIAGE SYMBOL */
  ["\u26AE", "\u26AE"], /* DIVORCE SYMBOL */
  ["\u26AF", "\u26AF"], /* UNMARRIED PARTNERSHIP SYMBOL */
  ["\u26B0", "\u26B0"], /* COFFIN */
  ["\u26B1", "\u26B1"], /* FUNERAL URN */
  ["\u26B2", "\u26B2"], /* NEUTER */
  ["\u26B3", "\u26B3"], /* CERES */
  ["\u26B4", "\u26B4"], /* PALLAS */
  ["\u26B5", "\u26B5"], /* JUNO */
  ["\u26B6", "\u26B6"], /* VESTA */
  ["\u26B7", "\u26B7"], /* CHIRON */
  ["\u26B8", "\u26B8"], /* BLACK MOON LILITH */
  ["\u26B9", "\u26B9"], /* SEXTILE */
  ["\u26BA", "\u26BA"], /* SEMISEXTILE */
  ["\u26BB", "\u26BB"], /* QUINCUNX */
  ["\u26BC", "\u26BC"], /* SESQUIQUADRATE */
  ["\u26BD", "\u26BD"], /* SOCCER BALL */
  ["\u26BE", "\u26BE"], /* BASEBALL */
  ["\u26BF", "\u26BF"], /* SQUARED KEY */
  ["\u26C0", "\u26C0"], /* WHITE DRAUGHTS MAN */
  ["\u26C1", "\u26C1"], /* WHITE DRAUGHTS KING */
  ["\u26C2", "\u26C2"], /* BLACK DRAUGHTS MAN */
  ["\u26C3", "\u26C3"], /* BLACK DRAUGHTS KING */
  ["\u26C4", "\u26C4"], /* SNOWMAN WITHOUT SNOW */
  ["\u26C5", "\u26C5"], /* SUN BEHIND CLOUD */
  ["\u26C6", "\u26C6"], /* RAIN */
  ["\u26C7", "\u26C7"], /* BLACK SNOWMAN */
  ["\u26C8", "\u26C8"], /* THUNDER CLOUD AND RAIN */
  ["\u26C9", "\u26C9"], /* TURNED WHITE SHOGI PIECE */
  ["\u26CA", "\u26CA"], /* TURNED BLACK SHOGI PIECE */
  ["\u26CB", "\u26CB"], /* WHITE DIAMOND IN SQUARE */
  ["\u26CC", "\u26CC"], /* CROSSING LANES */
  ["\u26CD", "\u26CD"], /* DISABLED CAR */
  ["\u26CE", "\u26CE"], /* OPHIUCHUS */
  ["\u26CF", "\u26CF"], /* PICK */
  ["\u26D0", "\u26D0"], /* CAR SLIDING */
  ["\u26D1", "\u26D1"], /* HELMET WITH WHITE CROSS */
  ["\u26D2", "\u26D2"], /* CIRCLED CROSSING LANES */
  ["\u26D3", "\u26D3"], /* CHAINS */
  ["\u26D4", "\u26D4"], /* NO ENTRY */
  ["\u26D5", "\u26D5"], /* ALTERNATE ONE-WAY LEFT WAY TRAFFIC */
  ["\u26D6", "\u26D6"], /* BLACK TWO-WAY LEFT WAY TRAFFIC */
  ["\u26D7", "\u26D7"], /* WHITE TWO-WAY LEFT WAY TRAFFIC */
  ["\u26D8", "\u26D8"], /* BLACK LEFT LANE MERGE */
  ["\u26D9", "\u26D9"], /* WHITE LEFT LANE MERGE */
  ["\u26DA", "\u26DA"], /* DRIVE SLOW SIGN */
  ["\u26DB", "\u26DB"], /* HEAVY WHITE DOWN-POINTING TRIANGLE */
  ["\u26DC", "\u26DC"], /* LEFT CLOSED ENTRY */
  ["\u26DD", "\u26DD"], /* SQUARED SALTIRE */
  ["\u26DE", "\u26DE"], /* FALLING DIAGONAL IN WHITE CIRCLE IN BLACK SQUARE */
  ["\u26DF", "\u26DF"], /* BLACK TRUCK */
  ["\u26E0", "\u26E0"], /* RESTRICTED LEFT ENTRY-1 */
  ["\u26E1", "\u26E1"], /* RESTRICTED LEFT ENTRY-2 */
  ["\u26E2", "\u26E2"], /* ASTRONOMICAL SYMBOL FOR URANUS */
  ["\u26E3", "\u26E3"], /* HEAVY CIRCLE WITH STROKE AND TWO DOTS ABOVE */
  ["\u26E4", "\u26E4"], /* PENTAGRAM */
  ["\u26E5", "\u26E5"], /* RIGHT-HANDED INTERLACED PENTAGRAM */
  ["\u26E6", "\u26E6"], /* LEFT-HANDED INTERLACED PENTAGRAM */
  ["\u26E7", "\u26E7"], /* INVERTED PENTAGRAM */
  ["\u26E8", "\u26E8"], /* BLACK CROSS ON SHIELD */
  ["\u26E9", "\u26E9"], /* SHINTO SHRINE */
  ["\u26EA", "\u26EA"], /* CHURCH */
  ["\u26EB", "\u26EB"], /* CASTLE */
  ["\u26EC", "\u26EC"], /* HISTORIC SITE */
  ["\u26ED", "\u26ED"], /* GEAR WITHOUT HUB */
  ["\u26EE", "\u26EE"], /* GEAR WITH HANDLES */
  ["\u26EF", "\u26EF"], /* MAP SYMBOL FOR LIGHTHOUSE */
  ["\u26F0", "\u26F0"], /* MOUNTAIN */
  ["\u26F1", "\u26F1"], /* UMBRELLA ON GROUND */
  ["\u26F2", "\u26F2"], /* FOUNTAIN */
  ["\u26F3", "\u26F3"], /* FLAG IN HOLE */
  ["\u26F4", "\u26F4"], /* FERRY */
  ["\u26F5", "\u26F5"], /* SAILBOAT */
  ["\u26F6", "\u26F6"], /* SQUARE FOUR CORNERS */
  ["\u26F7", "\u26F7"], /* SKIER */
  ["\u26F8", "\u26F8"], /* ICE SKATE */
  ["\u26F9", "\u26F9"], /* PERSON WITH BALL */
  ["\u26FA", "\u26FA"], /* TENT */
  ["\u26FB", "\u26FB"], /* JAPANESE BANK SYMBOL */
  ["\u26FC", "\u26FC"], /* HEADSTONE GRAVEYARD SYMBOL */
  ["\u26FD", "\u26FD"], /* FUEL PUMP */
  ["\u26FE", "\u26FE"], /* CUP ON BLACK SQUARE */
  ["\u26FF", "\u26FF"], /* WHITE FLAG WITH HORIZONTAL MIDDLE BLACK STRIPE */
  ["\u2700", "\u2700"], /* BLACK SAFETY SCISSORS */
  ["\u2701", "\u2701"], /* UPPER BLADE SCISSORS */
  ["\u2702", "\u2702"], /* BLACK SCISSORS */
  ["\u2703", "\u2703"], /* LOWER BLADE SCISSORS */
  ["\u2704", "\u2704"], /* WHITE SCISSORS */
  ["\u2705", "\u2705"], /* WHITE HEAVY CHECK MARK */
  ["\u2706", "\u2706"], /* TELEPHONE LOCATION SIGN */
  ["\u2707", "\u2707"], /* TAPE DRIVE */
  ["\u2708", "\u2708"], /* AIRPLANE */
  ["\u2709", "\u2709"], /* ENVELOPE */
  ["\u270A", "\u270A"], /* RAISED FIST */
  ["\u270B", "\u270B"], /* RAISED HAND */
  ["\u270C", "\u270C"], /* VICTORY HAND */
  ["\u270D", "\u270D"], /* WRITING HAND */
  ["\u270E", "\u270E"], /* LOWER RIGHT PENCIL */
  ["\u270F", "\u270F"], /* PENCIL */
  ["\u2710", "\u2710"], /* UPPER RIGHT PENCIL */
  ["\u2711", "\u2711"], /* WHITE NIB */
  ["\u2712", "\u2712"], /* BLACK NIB */
  ["\u2713", "\u2713"], /* CHECK MARK */
  ["\u2714", "\u2714"], /* HEAVY CHECK MARK */
  ["\u2715", "\u2715"], /* MULTIPLICATION X */
  ["\u2716", "\u2716"], /* HEAVY MULTIPLICATION X */
  ["\u2717", "\u2717"], /* BALLOT X */
  ["\u2718", "\u2718"], /* HEAVY BALLOT X */
  ["\u2719", "\u2719"], /* OUTLINED GREEK CROSS */
  ["\u271A", "\u271A"], /* HEAVY GREEK CROSS */
  ["\u271B", "\u271B"], /* OPEN CENTRE CROSS (OPEN CENTER CROSS) */
  ["\u271C", "\u271C"], /* HEAVY OPEN CENTRE CROSS (HEAVY OPEN CENTER CROSS) */
  ["\u271D", "\u271D"], /* LATIN CROSS */
  ["\u271E", "\u271E"], /* SHADOWED WHITE LATIN CROSS */
  ["\u271F", "\u271F"], /* OUTLINED LATIN CROSS */
  ["\u2720", "\u2720"], /* MALTESE CROSS */
  ["\u2721", "\u2721"], /* STAR OF DAVID */
  ["\u2722", "\u2722"], /* FOUR TEARDROP-SPOKED ASTERISK */
  ["\u2723", "\u2723"], /* FOUR BALLOON-SPOKED ASTERISK */
  ["\u2724", "\u2724"], /* HEAVY FOUR BALLOON-SPOKED ASTERISK */
  ["\u2725", "\u2725"], /* FOUR CLUB-SPOKED ASTERISK */
  ["\u2726", "\u2726"], /* BLACK FOUR POINTED STAR */
  ["\u2727", "\u2727"], /* WHITE FOUR POINTED STAR */
  ["\u2728", "\u2728"], /* SPARKLES */
  ["\u2729", "\u2729"], /* STRESS OUTLINED WHITE STAR */
  ["\u272A", "\u272A"], /* CIRCLED WHITE STAR */
  ["\u272B", "\u272B"], /* OPEN CENTRE BLACK STAR (OPEN CENTER BLACK STAR) */
  ["\u272C", "\u272C"], /* BLACK CENTRE WHITE STAR (BLACK CENTER WHITE STAR) */
  ["\u272D", "\u272D"], /* OUTLINED BLACK STAR */
  ["\u272E", "\u272E"], /* HEAVY OUTLINED BLACK STAR */
  ["\u272F", "\u272F"], /* PINWHEEL STAR */
  ["\u2730", "\u2730"], /* SHADOWED WHITE STAR */
  ["\u2731", "\u2731"], /* HEAVY ASTERISK */
  ["\u2732", "\u2732"], /* OPEN CENTRE ASTERISK (OPEN CENTER ASTERISK) */
  ["\u2733", "\u2733"], /* EIGHT SPOKED ASTERISK */
  ["\u2734", "\u2734"], /* EIGHT POINTED BLACK STAR */
  ["\u2735", "\u2735"], /* EIGHT POINTED PINWHEEL STAR */
  ["\u2736", "\u2736"], /* SIX POINTED BLACK STAR */
  ["\u2737", "\u2737"], /* EIGHT POINTED RECTILINEAR BLACK STAR */
  ["\u2738", "\u2738"], /* HEAVY EIGHT POINTED RECTILINEAR BLACK STAR */
  ["\u2739", "\u2739"], /* TWELVE POINTED BLACK STAR */
  ["\u273A", "\u273A"], /* SIXTEEN POINTED ASTERISK */
  ["\u273B", "\u273B"], /* TEARDROP-SPOKED ASTERISK */
  ["\u273C", "\u273C"], /* OPEN CENTRE TEARDROP-SPOKED ASTERISK (OPEN CENTER TEARDROP-SPOKED ASTERISK) */
  ["\u273D", "\u273D"], /* HEAVY TEARDROP-SPOKED ASTERISK */
  ["\u273E", "\u273E"], /* SIX PETALLED BLACK AND WHITE FLORETTE */
  ["\u273F", "\u273F"], /* BLACK FLORETTE */
  ["\u2740", "\u2740"], /* WHITE FLORETTE */
  ["\u2741", "\u2741"], /* EIGHT PETALLED OUTLINED BLACK FLORETTE */
  ["\u2742", "\u2742"], /* CIRCLED OPEN CENTRE EIGHT POINTED STAR (CIRCLED OPEN CENTER EIGHT POINTED STAR) */
  ["\u2743", "\u2743"], /* HEAVY TEARDROP-SPOKED PINWHEEL ASTERISK */
  ["\u2744", "\u2744"], /* SNOWFLAKE */
  ["\u2745", "\u2745"], /* TIGHT TRIFOLIATE SNOWFLAKE */
  ["\u2746", "\u2746"], /* HEAVY CHEVRON SNOWFLAKE */
  ["\u2747", "\u2747"], /* SPARKLE */
  ["\u2748", "\u2748"], /* HEAVY SPARKLE */
  ["\u2749", "\u2749"], /* BALLOON-SPOKED ASTERISK */
  ["\u274A", "\u274A"], /* EIGHT TEARDROP-SPOKED PROPELLER ASTERISK */
  ["\u274B", "\u274B"], /* HEAVY EIGHT TEARDROP-SPOKED PROPELLER ASTERISK */
  ["\u274C", "\u274C"], /* CROSS MARK */
  ["\u274D", "\u274D"], /* SHADOWED WHITE CIRCLE */
  ["\u274E", "\u274E"], /* NEGATIVE SQUARED CROSS MARK */
  ["\u274F", "\u274F"], /* LOWER RIGHT DROP-SHADOWED WHITE SQUARE */
  ["\u2750", "\u2750"], /* UPPER RIGHT DROP-SHADOWED WHITE SQUARE */
  ["\u2751", "\u2751"], /* LOWER RIGHT SHADOWED WHITE SQUARE */
  ["\u2752", "\u2752"], /* UPPER RIGHT SHADOWED WHITE SQUARE */
  ["\u2753", "\u2753"], /* BLACK QUESTION MARK ORNAMENT */
  ["\u2754", "\u2754"], /* WHITE QUESTION MARK ORNAMENT */
  ["\u2755", "\u2755"], /* WHITE EXCLAMATION MARK ORNAMENT */
  ["\u2756", "\u2756"], /* BLACK DIAMOND MINUS WHITE X */
  ["\u2757", "\u2757"], /* HEAVY EXCLAMATION MARK SYMBOL */
  ["\u2758", "\u2758"], /* LIGHT VERTICAL BAR */
  ["\u2759", "\u2759"], /* MEDIUM VERTICAL BAR */
  ["\u275A", "\u275A"], /* HEAVY VERTICAL BAR */
  ["\u275B", "\u275B"], /* HEAVY SINGLE TURNED COMMA QUOTATION MARK ORNAMENT */
  ["\u275C", "\u275C"], /* HEAVY SINGLE COMMA QUOTATION MARK ORNAMENT */
  ["\u275D", "\u275D"], /* HEAVY DOUBLE TURNED COMMA QUOTATION MARK ORNAMENT */
  ["\u275E", "\u275E"], /* HEAVY DOUBLE COMMA QUOTATION MARK ORNAMENT */
  ["\u275F", "\u275F"], /* HEAVY LOW SINGLE COMMA QUOTATION MARK ORNAMENT */
  ["\u2760", "\u2760"], /* HEAVY LOW DOUBLE COMMA QUOTATION MARK ORNAMENT */
  ["\u2761", "\u2761"], /* CURVED STEM PARAGRAPH SIGN ORNAMENT */
  ["\u2762", "\u2762"], /* HEAVY EXCLAMATION MARK ORNAMENT */
  ["\u2763", "\u2763"], /* HEAVY HEART EXCLAMATION MARK ORNAMENT */
  ["\u2764", "\u2764"], /* HEAVY BLACK HEART */
  ["\u2765", "\u2765"], /* ROTATED HEAVY BLACK HEART BULLET */
  ["\u2766", "\u2766"], /* FLORAL HEART */
  ["\u2767", "\u2767"], /* ROTATED FLORAL HEART BULLET */
  ["\u2768", "\u2768"], /* MEDIUM LEFT PARENTHESIS ORNAMENT */
  ["\u2769", "\u2769"], /* MEDIUM RIGHT PARENTHESIS ORNAMENT */
  ["\u276A", "\u276A"], /* MEDIUM FLATTENED LEFT PARENTHESIS ORNAMENT */
  ["\u276B", "\u276B"], /* MEDIUM FLATTENED RIGHT PARENTHESIS ORNAMENT */
  ["\u276C", "\u276C"], /* MEDIUM LEFT-POINTING ANGLE BRACKET ORNAMENT */
  ["\u276D", "\u276D"], /* MEDIUM RIGHT-POINTING ANGLE BRACKET ORNAMENT */
  ["\u276E", "\u276E"], /* HEAVY LEFT-POINTING ANGLE QUOTATION MARK ORNAMENT */
  ["\u276F", "\u276F"], /* HEAVY RIGHT-POINTING ANGLE QUOTATION MARK ORNAMENT */
  ["\u2770", "\u2770"], /* HEAVY LEFT-POINTING ANGLE BRACKET ORNAMENT */
  ["\u2771", "\u2771"], /* HEAVY RIGHT-POINTING ANGLE BRACKET ORNAMENT */
  ["\u2772", "\u2772"], /* LIGHT LEFT TORTOISE SHELL BRACKET ORNAMENT */
  ["\u2773", "\u2773"], /* LIGHT RIGHT TORTOISE SHELL BRACKET ORNAMENT */
  ["\u2774", "\u2774"], /* MEDIUM LEFT CURLY BRACKET ORNAMENT */
  ["\u2775", "\u2775"], /* MEDIUM RIGHT CURLY BRACKET ORNAMENT */
  ["\u2776", "\u2776"], /* DINGBAT NEGATIVE CIRCLED DIGIT ONE (INVERSE CIRCLED DIGIT ONE) */
  ["\u2777", "\u2777"], /* DINGBAT NEGATIVE CIRCLED DIGIT TWO (INVERSE CIRCLED DIGIT TWO) */
  ["\u2778", "\u2778"], /* DINGBAT NEGATIVE CIRCLED DIGIT THREE (INVERSE CIRCLED DIGIT THREE) */
  ["\u2779", "\u2779"], /* DINGBAT NEGATIVE CIRCLED DIGIT FOUR (INVERSE CIRCLED DIGIT FOUR) */
  ["\u277A", "\u277A"], /* DINGBAT NEGATIVE CIRCLED DIGIT FIVE (INVERSE CIRCLED DIGIT FIVE) */
  ["\u277B", "\u277B"], /* DINGBAT NEGATIVE CIRCLED DIGIT SIX (INVERSE CIRCLED DIGIT SIX) */
  ["\u277C", "\u277C"], /* DINGBAT NEGATIVE CIRCLED DIGIT SEVEN (INVERSE CIRCLED DIGIT SEVEN) */
  ["\u277D", "\u277D"], /* DINGBAT NEGATIVE CIRCLED DIGIT EIGHT (INVERSE CIRCLED DIGIT EIGHT) */
  ["\u277E", "\u277E"], /* DINGBAT NEGATIVE CIRCLED DIGIT NINE (INVERSE CIRCLED DIGIT NINE) */
  ["\u277F", "\u277F"], /* DINGBAT NEGATIVE CIRCLED NUMBER TEN (INVERSE CIRCLED NUMBER TEN) */
  ["\u2780", "\u2780"], /* DINGBAT CIRCLED SANS-SERIF DIGIT ONE (CIRCLED SANS-SERIF DIGIT ONE) */
  ["\u2781", "\u2781"], /* DINGBAT CIRCLED SANS-SERIF DIGIT TWO (CIRCLED SANS-SERIF DIGIT TWO) */
  ["\u2782", "\u2782"], /* DINGBAT CIRCLED SANS-SERIF DIGIT THREE (CIRCLED SANS-SERIF DIGIT THREE) */
  ["\u2783", "\u2783"], /* DINGBAT CIRCLED SANS-SERIF DIGIT FOUR (CIRCLED SANS-SERIF DIGIT FOUR) */
  ["\u2784", "\u2784"], /* DINGBAT CIRCLED SANS-SERIF DIGIT FIVE (CIRCLED SANS-SERIF DIGIT FIVE) */
  ["\u2785", "\u2785"], /* DINGBAT CIRCLED SANS-SERIF DIGIT SIX (CIRCLED SANS-SERIF DIGIT SIX) */
  ["\u2786", "\u2786"], /* DINGBAT CIRCLED SANS-SERIF DIGIT SEVEN (CIRCLED SANS-SERIF DIGIT SEVEN) */
  ["\u2787", "\u2787"], /* DINGBAT CIRCLED SANS-SERIF DIGIT EIGHT (CIRCLED SANS-SERIF DIGIT EIGHT) */
  ["\u2788", "\u2788"], /* DINGBAT CIRCLED SANS-SERIF DIGIT NINE (CIRCLED SANS-SERIF DIGIT NINE) */
  ["\u2789", "\u2789"], /* DINGBAT CIRCLED SANS-SERIF NUMBER TEN (CIRCLED SANS-SERIF NUMBER TEN) */
  ["\u278A", "\u278A"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT ONE (INVERSE CIRCLED SANS-SERIF DIGIT ONE) */
  ["\u278B", "\u278B"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT TWO (INVERSE CIRCLED SANS-SERIF DIGIT TWO) */
  ["\u278C", "\u278C"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT THREE (INVERSE CIRCLED SANS-SERIF DIGIT THREE) */
  ["\u278D", "\u278D"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FOUR (INVERSE CIRCLED SANS-SERIF DIGIT FOUR) */
  ["\u278E", "\u278E"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT FIVE (INVERSE CIRCLED SANS-SERIF DIGIT FIVE) */
  ["\u278F", "\u278F"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SIX (INVERSE CIRCLED SANS-SERIF DIGIT SIX) */
  ["\u2790", "\u2790"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT SEVEN (INVERSE CIRCLED SANS-SERIF DIGIT SEVEN) */
  ["\u2791", "\u2791"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT EIGHT (INVERSE CIRCLED SANS-SERIF DIGIT EIGHT) */
  ["\u2792", "\u2792"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF DIGIT NINE (INVERSE CIRCLED SANS-SERIF DIGIT NINE) */
  ["\u2793", "\u2793"], /* DINGBAT NEGATIVE CIRCLED SANS-SERIF NUMBER TEN (INVERSE CIRCLED SANS-SERIF NUMBER TEN) */
  ["\u2794", "\u2794"], /* HEAVY WIDE-HEADED RIGHTWARDS ARROW (HEAVY WIDE-HEADED RIGHT ARROW) */
  ["\u2795", "\u2795"], /* HEAVY PLUS SIGN */
  ["\u2796", "\u2796"], /* HEAVY MINUS SIGN */
  ["\u2797", "\u2797"], /* HEAVY DIVISION SIGN */
  ["\u2798", "\u2798"], /* HEAVY SOUTH EAST ARROW (HEAVY LOWER RIGHT ARROW) */
  ["\u2799", "\u2799"], /* HEAVY RIGHTWARDS ARROW (HEAVY RIGHT ARROW) */
  ["\u279A", "\u279A"], /* HEAVY NORTH EAST ARROW (HEAVY UPPER RIGHT ARROW) */
  ["\u279B", "\u279B"], /* DRAFTING POINT RIGHTWARDS ARROW (DRAFTING POINT RIGHT ARROW) */
  ["\u279C", "\u279C"], /* HEAVY ROUND-TIPPED RIGHTWARDS ARROW (HEAVY ROUND-TIPPED RIGHT ARROW) */
  ["\u279D", "\u279D"], /* TRIANGLE-HEADED RIGHTWARDS ARROW (TRIANGLE-HEADED RIGHT ARROW) */
  ["\u279E", "\u279E"], /* HEAVY TRIANGLE-HEADED RIGHTWARDS ARROW (HEAVY TRIANGLE-HEADED RIGHT ARROW) */
  ["\u279F", "\u279F"], /* DASHED TRIANGLE-HEADED RIGHTWARDS ARROW (DASHED TRIANGLE-HEADED RIGHT ARROW) */
  ["\u27A0", "\u27A0"], /* HEAVY DASHED TRIANGLE-HEADED RIGHTWARDS ARROW (HEAVY DASHED TRIANGLE-HEADED RIGHT ARROW) */
  ["\u27A1", "\u27A1"], /* BLACK RIGHTWARDS ARROW (BLACK RIGHT ARROW) */
  ["\u27A2", "\u27A2"], /* THREE-D TOP-LIGHTED RIGHTWARDS ARROWHEAD (THREE-D TOP-LIGHTED RIGHT ARROWHEAD) */
  ["\u27A3", "\u27A3"], /* THREE-D BOTTOM-LIGHTED RIGHTWARDS ARROWHEAD (THREE-D BOTTOM-LIGHTED RIGHT ARROWHEAD) */
  ["\u27A4", "\u27A4"], /* BLACK RIGHTWARDS ARROWHEAD (BLACK RIGHT ARROWHEAD) */
  ["\u27A5", "\u27A5"], /* HEAVY BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW (HEAVY BLACK CURVED DOWN AND RIGHT ARROW) */
  ["\u27A6", "\u27A6"], /* HEAVY BLACK CURVED UPWARDS AND RIGHTWARDS ARROW (HEAVY BLACK CURVED UP AND RIGHT ARROW) */
  ["\u27A7", "\u27A7"], /* SQUAT BLACK RIGHTWARDS ARROW (SQUAT BLACK RIGHT ARROW) */
  ["\u27A8", "\u27A8"], /* HEAVY CONCAVE-POINTED BLACK RIGHTWARDS ARROW (HEAVY CONCAVE-POINTED BLACK RIGHT ARROW) */
  ["\u27A9", "\u27A9"], /* RIGHT-SHADED WHITE RIGHTWARDS ARROW (RIGHT-SHADED WHITE RIGHT ARROW) */
  ["\u27AA", "\u27AA"], /* LEFT-SHADED WHITE RIGHTWARDS ARROW (LEFT-SHADED WHITE RIGHT ARROW) */
  ["\u27AB", "\u27AB"], /* BACK-TILTED SHADOWED WHITE RIGHTWARDS ARROW (BACK-TILTED SHADOWED WHITE RIGHT ARROW) */
  ["\u27AC", "\u27AC"], /* FRONT-TILTED SHADOWED WHITE RIGHTWARDS ARROW (FRONT-TILTED SHADOWED WHITE RIGHT ARROW) */
  ["\u27AD", "\u27AD"], /* HEAVY LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW (HEAVY LOWER RIGHT-SHADOWED WHITE RIGHT ARROW) */
  ["\u27AE", "\u27AE"], /* HEAVY UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW (HEAVY UPPER RIGHT-SHADOWED WHITE RIGHT ARROW) */
  ["\u27AF", "\u27AF"], /* NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW (NOTCHED LOWER RIGHT-SHADOWED WHITE RIGHT ARROW) */
  ["\u27B0", "\u27B0"], /* CURLY LOOP */
  ["\u27B1", "\u27B1"], /* NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHTWARDS ARROW (NOTCHED UPPER RIGHT-SHADOWED WHITE RIGHT ARROW) */
  ["\u27B2", "\u27B2"], /* CIRCLED HEAVY WHITE RIGHTWARDS ARROW (CIRCLED HEAVY WHITE RIGHT ARROW) */
  ["\u27B3", "\u27B3"], /* WHITE-FEATHERED RIGHTWARDS ARROW (WHITE-FEATHERED RIGHT ARROW) */
  ["\u27B4", "\u27B4"], /* BLACK-FEATHERED SOUTH EAST ARROW (BLACK-FEATHERED LOWER RIGHT ARROW) */
  ["\u27B5", "\u27B5"], /* BLACK-FEATHERED RIGHTWARDS ARROW (BLACK-FEATHERED RIGHT ARROW) */
  ["\u27B6", "\u27B6"], /* BLACK-FEATHERED NORTH EAST ARROW (BLACK-FEATHERED UPPER RIGHT ARROW) */
  ["\u27B7", "\u27B7"], /* HEAVY BLACK-FEATHERED SOUTH EAST ARROW (HEAVY BLACK-FEATHERED LOWER RIGHT ARROW) */
  ["\u27B8", "\u27B8"], /* HEAVY BLACK-FEATHERED RIGHTWARDS ARROW (HEAVY BLACK-FEATHERED RIGHT ARROW) */
  ["\u27B9", "\u27B9"], /* HEAVY BLACK-FEATHERED NORTH EAST ARROW (HEAVY BLACK-FEATHERED UPPER RIGHT ARROW) */
  ["\u27BA", "\u27BA"], /* TEARDROP-BARBED RIGHTWARDS ARROW (TEARDROP-BARBED RIGHT ARROW) */
  ["\u27BB", "\u27BB"], /* HEAVY TEARDROP-SHANKED RIGHTWARDS ARROW (HEAVY TEARDROP-SHANKED RIGHT ARROW) */
  ["\u27BC", "\u27BC"], /* WEDGE-TAILED RIGHTWARDS ARROW (WEDGE-TAILED RIGHT ARROW) */
  ["\u27BD", "\u27BD"], /* HEAVY WEDGE-TAILED RIGHTWARDS ARROW (HEAVY WEDGE-TAILED RIGHT ARROW) */
  ["\u27BE", "\u27BE"], /* OPEN-OUTLINED RIGHTWARDS ARROW (OPEN-OUTLINED RIGHT ARROW) */
  ["\u27BF", "\u27BF"], /* DOUBLE CURLY LOOP */
  ["\u27C0", "\u27C0"], /* THREE DIMENSIONAL ANGLE */
  ["\u27C1", "\u27C1"], /* WHITE TRIANGLE CONTAINING SMALL WHITE TRIANGLE */
  ["\u27C2", "\u27C2"], /* PERPENDICULAR */
  ["\u27C3", "\u27C3"], /* OPEN SUBSET */
  ["\u27C4", "\u27C4"], /* OPEN SUPERSET */
  ["\u27C5", "\u27C5"], /* LEFT S-SHAPED BAG DELIMITER */
  ["\u27C6", "\u27C6"], /* RIGHT S-SHAPED BAG DELIMITER */
  ["\u27C7", "\u27C7"], /* OR WITH DOT INSIDE */
  ["\u27C8", "\u27C8"], /* REVERSE SOLIDUS PRECEDING SUBSET */
  ["\u27C9", "\u27C9"], /* SUPERSET PRECEDING SOLIDUS */
  ["\u27CA", "\u27CA"], /* VERTICAL BAR WITH HORIZONTAL STROKE */
  ["\u27CB", "\u27CB"], /* MATHEMATICAL RISING DIAGONAL */
  ["\u27CC", "\u27CC"], /* LONG DIVISION */
  ["\u27CD", "\u27CD"], /* MATHEMATICAL FALLING DIAGONAL */
  ["\u27CE", "\u27CE"], /* SQUARED LOGICAL AND */
  ["\u27CF", "\u27CF"], /* SQUARED LOGICAL OR */
  ["\u27D0", "\u27D0"], /* WHITE DIAMOND WITH CENTRED DOT */
  ["\u27D1", "\u27D1"], /* AND WITH DOT */
  ["\u27D2", "\u27D2"], /* ELEMENT OF OPENING UPWARDS */
  ["\u27D3", "\u27D3"], /* LOWER RIGHT CORNER WITH DOT */
  ["\u27D4", "\u27D4"], /* UPPER LEFT CORNER WITH DOT */
  ["\u27D5", "\u27D5"], /* LEFT OUTER JOIN */
  ["\u27D6", "\u27D6"], /* RIGHT OUTER JOIN */
  ["\u27D7", "\u27D7"], /* FULL OUTER JOIN */
  ["\u27D8", "\u27D8"], /* LARGE UP TACK */
  ["\u27D9", "\u27D9"], /* LARGE DOWN TACK */
  ["\u27DA", "\u27DA"], /* LEFT AND RIGHT DOUBLE TURNSTILE */
  ["\u27DB", "\u27DB"], /* LEFT AND RIGHT TACK */
  ["\u27DC", "\u27DC"], /* LEFT MULTIMAP */
  ["\u27DD", "\u27DD"], /* LONG RIGHT TACK */
  ["\u27DE", "\u27DE"], /* LONG LEFT TACK */
  ["\u27DF", "\u27DF"], /* UP TACK WITH CIRCLE ABOVE */
  ["\u27E0", "\u27E0"], /* LOZENGE DIVIDED BY HORIZONTAL RULE */
  ["\u27E1", "\u27E1"], /* WHITE CONCAVE-SIDED DIAMOND */
  ["\u27E2", "\u27E2"], /* WHITE CONCAVE-SIDED DIAMOND WITH LEFTWARDS TICK */
  ["\u27E3", "\u27E3"], /* WHITE CONCAVE-SIDED DIAMOND WITH RIGHTWARDS TICK */
  ["\u27E4", "\u27E4"], /* WHITE SQUARE WITH LEFTWARDS TICK */
  ["\u27E5", "\u27E5"], /* WHITE SQUARE WITH RIGHTWARDS TICK */
  ["\u27E6", "\u27E6"], /* MATHEMATICAL LEFT WHITE SQUARE BRACKET */
  ["\u27E7", "\u27E7"], /* MATHEMATICAL RIGHT WHITE SQUARE BRACKET */
  ["\u27E8", "\u27E8"], /* MATHEMATICAL LEFT ANGLE BRACKET */
  ["\u27E9", "\u27E9"], /* MATHEMATICAL RIGHT ANGLE BRACKET */
  ["\u27EA", "\u27EA"], /* MATHEMATICAL LEFT DOUBLE ANGLE BRACKET */
  ["\u27EB", "\u27EB"], /* MATHEMATICAL RIGHT DOUBLE ANGLE BRACKET */
  ["\u27EC", "\u27EC"], /* MATHEMATICAL LEFT WHITE TORTOISE SHELL BRACKET */
  ["\u27ED", "\u27ED"], /* MATHEMATICAL RIGHT WHITE TORTOISE SHELL BRACKET */
  ["\u27EE", "\u27EE"], /* MATHEMATICAL LEFT FLATTENED PARENTHESIS */
  ["\u27EF", "\u27EF"], /* MATHEMATICAL RIGHT FLATTENED PARENTHESIS */
  ["\u27F0", "\u27F0"], /* UPWARDS QUADRUPLE ARROW */
  ["\u27F1", "\u27F1"], /* DOWNWARDS QUADRUPLE ARROW */
  ["\u27F2", "\u27F2"], /* ANTICLOCKWISE GAPPED CIRCLE ARROW */
  ["\u27F3", "\u27F3"], /* CLOCKWISE GAPPED CIRCLE ARROW */
  ["\u27F4", "\u27F4"], /* RIGHT ARROW WITH CIRCLED PLUS */
  ["\u27F5", "\u27F5"], /* LONG LEFTWARDS ARROW */
  ["\u27F6", "\u27F6"], /* LONG RIGHTWARDS ARROW */
  ["\u27F7", "\u27F7"], /* LONG LEFT RIGHT ARROW */
  ["\u27F8", "\u27F8"], /* LONG LEFTWARDS DOUBLE ARROW */
  ["\u27F9", "\u27F9"], /* LONG RIGHTWARDS DOUBLE ARROW */
  ["\u27FA", "\u27FA"], /* LONG LEFT RIGHT DOUBLE ARROW */
  ["\u27FB", "\u27FB"], /* LONG LEFTWARDS ARROW FROM BAR */
  ["\u27FC", "\u27FC"], /* LONG RIGHTWARDS ARROW FROM BAR */
  ["\u27FD", "\u27FD"], /* LONG LEFTWARDS DOUBLE ARROW FROM BAR */
  ["\u27FE", "\u27FE"], /* LONG RIGHTWARDS DOUBLE ARROW FROM BAR */
  ["\u27FF", "\u27FF"], /* LONG RIGHTWARDS SQUIGGLE ARROW */
  ["\u2800", "\u2800"], /* BRAILLE PATTERN BLANK */
  ["\u2801", "\u2801"], /* BRAILLE PATTERN DOTS-1 */
  ["\u2802", "\u2802"], /* BRAILLE PATTERN DOTS-2 */
  ["\u2803", "\u2803"], /* BRAILLE PATTERN DOTS-12 */
  ["\u2804", "\u2804"], /* BRAILLE PATTERN DOTS-3 */
  ["\u2805", "\u2805"], /* BRAILLE PATTERN DOTS-13 */
  ["\u2806", "\u2806"], /* BRAILLE PATTERN DOTS-23 */
  ["\u2807", "\u2807"], /* BRAILLE PATTERN DOTS-123 */
  ["\u2808", "\u2808"], /* BRAILLE PATTERN DOTS-4 */
  ["\u2809", "\u2809"], /* BRAILLE PATTERN DOTS-14 */
  ["\u280A", "\u280A"], /* BRAILLE PATTERN DOTS-24 */
  ["\u280B", "\u280B"], /* BRAILLE PATTERN DOTS-124 */
  ["\u280C", "\u280C"], /* BRAILLE PATTERN DOTS-34 */
  ["\u280D", "\u280D"], /* BRAILLE PATTERN DOTS-134 */
  ["\u280E", "\u280E"], /* BRAILLE PATTERN DOTS-234 */
  ["\u280F", "\u280F"], /* BRAILLE PATTERN DOTS-1234 */
  ["\u2810", "\u2810"], /* BRAILLE PATTERN DOTS-5 */
  ["\u2811", "\u2811"], /* BRAILLE PATTERN DOTS-15 */
  ["\u2812", "\u2812"], /* BRAILLE PATTERN DOTS-25 */
  ["\u2813", "\u2813"], /* BRAILLE PATTERN DOTS-125 */
  ["\u2814", "\u2814"], /* BRAILLE PATTERN DOTS-35 */
  ["\u2815", "\u2815"], /* BRAILLE PATTERN DOTS-135 */
  ["\u2816", "\u2816"], /* BRAILLE PATTERN DOTS-235 */
  ["\u2817", "\u2817"], /* BRAILLE PATTERN DOTS-1235 */
  ["\u2818", "\u2818"], /* BRAILLE PATTERN DOTS-45 */
  ["\u2819", "\u2819"], /* BRAILLE PATTERN DOTS-145 */
  ["\u281A", "\u281A"], /* BRAILLE PATTERN DOTS-245 */
  ["\u281B", "\u281B"], /* BRAILLE PATTERN DOTS-1245 */
  ["\u281C", "\u281C"], /* BRAILLE PATTERN DOTS-345 */
  ["\u281D", "\u281D"], /* BRAILLE PATTERN DOTS-1345 */
  ["\u281E", "\u281E"], /* BRAILLE PATTERN DOTS-2345 */
  ["\u281F", "\u281F"], /* BRAILLE PATTERN DOTS-12345 */
  ["\u2820", "\u2820"], /* BRAILLE PATTERN DOTS-6 */
  ["\u2821", "\u2821"], /* BRAILLE PATTERN DOTS-16 */
  ["\u2822", "\u2822"], /* BRAILLE PATTERN DOTS-26 */
  ["\u2823", "\u2823"], /* BRAILLE PATTERN DOTS-126 */
  ["\u2824", "\u2824"], /* BRAILLE PATTERN DOTS-36 */
  ["\u2825", "\u2825"], /* BRAILLE PATTERN DOTS-136 */
  ["\u2826", "\u2826"], /* BRAILLE PATTERN DOTS-236 */
  ["\u2827", "\u2827"], /* BRAILLE PATTERN DOTS-1236 */
  ["\u2828", "\u2828"], /* BRAILLE PATTERN DOTS-46 */
  ["\u2829", "\u2829"], /* BRAILLE PATTERN DOTS-146 */
  ["\u282A", "\u282A"], /* BRAILLE PATTERN DOTS-246 */
  ["\u282B", "\u282B"], /* BRAILLE PATTERN DOTS-1246 */
  ["\u282C", "\u282C"], /* BRAILLE PATTERN DOTS-346 */
  ["\u282D", "\u282D"], /* BRAILLE PATTERN DOTS-1346 */
  ["\u282E", "\u282E"], /* BRAILLE PATTERN DOTS-2346 */
  ["\u282F", "\u282F"], /* BRAILLE PATTERN DOTS-12346 */
  ["\u2830", "\u2830"], /* BRAILLE PATTERN DOTS-56 */
  ["\u2831", "\u2831"], /* BRAILLE PATTERN DOTS-156 */
  ["\u2832", "\u2832"], /* BRAILLE PATTERN DOTS-256 */
  ["\u2833", "\u2833"], /* BRAILLE PATTERN DOTS-1256 */
  ["\u2834", "\u2834"], /* BRAILLE PATTERN DOTS-356 */
  ["\u2835", "\u2835"], /* BRAILLE PATTERN DOTS-1356 */
  ["\u2836", "\u2836"], /* BRAILLE PATTERN DOTS-2356 */
  ["\u2837", "\u2837"], /* BRAILLE PATTERN DOTS-12356 */
  ["\u2838", "\u2838"], /* BRAILLE PATTERN DOTS-456 */
  ["\u2839", "\u2839"], /* BRAILLE PATTERN DOTS-1456 */
  ["\u283A", "\u283A"], /* BRAILLE PATTERN DOTS-2456 */
  ["\u283B", "\u283B"], /* BRAILLE PATTERN DOTS-12456 */
  ["\u283C", "\u283C"], /* BRAILLE PATTERN DOTS-3456 */
  ["\u283D", "\u283D"], /* BRAILLE PATTERN DOTS-13456 */
  ["\u283E", "\u283E"], /* BRAILLE PATTERN DOTS-23456 */
  ["\u283F", "\u283F"], /* BRAILLE PATTERN DOTS-123456 */
  ["\u2840", "\u2840"], /* BRAILLE PATTERN DOTS-7 */
  ["\u2841", "\u2841"], /* BRAILLE PATTERN DOTS-17 */
  ["\u2842", "\u2842"], /* BRAILLE PATTERN DOTS-27 */
  ["\u2843", "\u2843"], /* BRAILLE PATTERN DOTS-127 */
  ["\u2844", "\u2844"], /* BRAILLE PATTERN DOTS-37 */
  ["\u2845", "\u2845"], /* BRAILLE PATTERN DOTS-137 */
  ["\u2846", "\u2846"], /* BRAILLE PATTERN DOTS-237 */
  ["\u2847", "\u2847"], /* BRAILLE PATTERN DOTS-1237 */
  ["\u2848", "\u2848"], /* BRAILLE PATTERN DOTS-47 */
  ["\u2849", "\u2849"], /* BRAILLE PATTERN DOTS-147 */
  ["\u284A", "\u284A"], /* BRAILLE PATTERN DOTS-247 */
  ["\u284B", "\u284B"], /* BRAILLE PATTERN DOTS-1247 */
  ["\u284C", "\u284C"], /* BRAILLE PATTERN DOTS-347 */
  ["\u284D", "\u284D"], /* BRAILLE PATTERN DOTS-1347 */
  ["\u284E", "\u284E"], /* BRAILLE PATTERN DOTS-2347 */
  ["\u284F", "\u284F"], /* BRAILLE PATTERN DOTS-12347 */
  ["\u2850", "\u2850"], /* BRAILLE PATTERN DOTS-57 */
  ["\u2851", "\u2851"], /* BRAILLE PATTERN DOTS-157 */
  ["\u2852", "\u2852"], /* BRAILLE PATTERN DOTS-257 */
  ["\u2853", "\u2853"], /* BRAILLE PATTERN DOTS-1257 */
  ["\u2854", "\u2854"], /* BRAILLE PATTERN DOTS-357 */
  ["\u2855", "\u2855"], /* BRAILLE PATTERN DOTS-1357 */
  ["\u2856", "\u2856"], /* BRAILLE PATTERN DOTS-2357 */
  ["\u2857", "\u2857"], /* BRAILLE PATTERN DOTS-12357 */
  ["\u2858", "\u2858"], /* BRAILLE PATTERN DOTS-457 */
  ["\u2859", "\u2859"], /* BRAILLE PATTERN DOTS-1457 */
  ["\u285A", "\u285A"], /* BRAILLE PATTERN DOTS-2457 */
  ["\u285B", "\u285B"], /* BRAILLE PATTERN DOTS-12457 */
  ["\u285C", "\u285C"], /* BRAILLE PATTERN DOTS-3457 */
  ["\u285D", "\u285D"], /* BRAILLE PATTERN DOTS-13457 */
  ["\u285E", "\u285E"], /* BRAILLE PATTERN DOTS-23457 */
  ["\u285F", "\u285F"], /* BRAILLE PATTERN DOTS-123457 */
  ["\u2860", "\u2860"], /* BRAILLE PATTERN DOTS-67 */
  ["\u2861", "\u2861"], /* BRAILLE PATTERN DOTS-167 */
  ["\u2862", "\u2862"], /* BRAILLE PATTERN DOTS-267 */
  ["\u2863", "\u2863"], /* BRAILLE PATTERN DOTS-1267 */
  ["\u2864", "\u2864"], /* BRAILLE PATTERN DOTS-367 */
  ["\u2865", "\u2865"], /* BRAILLE PATTERN DOTS-1367 */
  ["\u2866", "\u2866"], /* BRAILLE PATTERN DOTS-2367 */
  ["\u2867", "\u2867"], /* BRAILLE PATTERN DOTS-12367 */
  ["\u2868", "\u2868"], /* BRAILLE PATTERN DOTS-467 */
  ["\u2869", "\u2869"], /* BRAILLE PATTERN DOTS-1467 */
  ["\u286A", "\u286A"], /* BRAILLE PATTERN DOTS-2467 */
  ["\u286B", "\u286B"], /* BRAILLE PATTERN DOTS-12467 */
  ["\u286C", "\u286C"], /* BRAILLE PATTERN DOTS-3467 */
  ["\u286D", "\u286D"], /* BRAILLE PATTERN DOTS-13467 */
  ["\u286E", "\u286E"], /* BRAILLE PATTERN DOTS-23467 */
  ["\u286F", "\u286F"], /* BRAILLE PATTERN DOTS-123467 */
  ["\u2870", "\u2870"], /* BRAILLE PATTERN DOTS-567 */
  ["\u2871", "\u2871"], /* BRAILLE PATTERN DOTS-1567 */
  ["\u2872", "\u2872"], /* BRAILLE PATTERN DOTS-2567 */
  ["\u2873", "\u2873"], /* BRAILLE PATTERN DOTS-12567 */
  ["\u2874", "\u2874"], /* BRAILLE PATTERN DOTS-3567 */
  ["\u2875", "\u2875"], /* BRAILLE PATTERN DOTS-13567 */
  ["\u2876", "\u2876"], /* BRAILLE PATTERN DOTS-23567 */
  ["\u2877", "\u2877"], /* BRAILLE PATTERN DOTS-123567 */
  ["\u2878", "\u2878"], /* BRAILLE PATTERN DOTS-4567 */
  ["\u2879", "\u2879"], /* BRAILLE PATTERN DOTS-14567 */
  ["\u287A", "\u287A"], /* BRAILLE PATTERN DOTS-24567 */
  ["\u287B", "\u287B"], /* BRAILLE PATTERN DOTS-124567 */
  ["\u287C", "\u287C"], /* BRAILLE PATTERN DOTS-34567 */
  ["\u287D", "\u287D"], /* BRAILLE PATTERN DOTS-134567 */
  ["\u287E", "\u287E"], /* BRAILLE PATTERN DOTS-234567 */
  ["\u287F", "\u287F"], /* BRAILLE PATTERN DOTS-1234567 */
  ["\u2880", "\u2880"], /* BRAILLE PATTERN DOTS-8 */
  ["\u2881", "\u2881"], /* BRAILLE PATTERN DOTS-18 */
  ["\u2882", "\u2882"], /* BRAILLE PATTERN DOTS-28 */
  ["\u2883", "\u2883"], /* BRAILLE PATTERN DOTS-128 */
  ["\u2884", "\u2884"], /* BRAILLE PATTERN DOTS-38 */
  ["\u2885", "\u2885"], /* BRAILLE PATTERN DOTS-138 */
  ["\u2886", "\u2886"], /* BRAILLE PATTERN DOTS-238 */
  ["\u2887", "\u2887"], /* BRAILLE PATTERN DOTS-1238 */
  ["\u2888", "\u2888"], /* BRAILLE PATTERN DOTS-48 */
  ["\u2889", "\u2889"], /* BRAILLE PATTERN DOTS-148 */
  ["\u288A", "\u288A"], /* BRAILLE PATTERN DOTS-248 */
  ["\u288B", "\u288B"], /* BRAILLE PATTERN DOTS-1248 */
  ["\u288C", "\u288C"], /* BRAILLE PATTERN DOTS-348 */
  ["\u288D", "\u288D"], /* BRAILLE PATTERN DOTS-1348 */
  ["\u288E", "\u288E"], /* BRAILLE PATTERN DOTS-2348 */
  ["\u288F", "\u288F"], /* BRAILLE PATTERN DOTS-12348 */
  ["\u2890", "\u2890"], /* BRAILLE PATTERN DOTS-58 */
  ["\u2891", "\u2891"], /* BRAILLE PATTERN DOTS-158 */
  ["\u2892", "\u2892"], /* BRAILLE PATTERN DOTS-258 */
  ["\u2893", "\u2893"], /* BRAILLE PATTERN DOTS-1258 */
  ["\u2894", "\u2894"], /* BRAILLE PATTERN DOTS-358 */
  ["\u2895", "\u2895"], /* BRAILLE PATTERN DOTS-1358 */
  ["\u2896", "\u2896"], /* BRAILLE PATTERN DOTS-2358 */
  ["\u2897", "\u2897"], /* BRAILLE PATTERN DOTS-12358 */
  ["\u2898", "\u2898"], /* BRAILLE PATTERN DOTS-458 */
  ["\u2899", "\u2899"], /* BRAILLE PATTERN DOTS-1458 */
  ["\u289A", "\u289A"], /* BRAILLE PATTERN DOTS-2458 */
  ["\u289B", "\u289B"], /* BRAILLE PATTERN DOTS-12458 */
  ["\u289C", "\u289C"], /* BRAILLE PATTERN DOTS-3458 */
  ["\u289D", "\u289D"], /* BRAILLE PATTERN DOTS-13458 */
  ["\u289E", "\u289E"], /* BRAILLE PATTERN DOTS-23458 */
  ["\u289F", "\u289F"], /* BRAILLE PATTERN DOTS-123458 */
  ["\u28A0", "\u28A0"], /* BRAILLE PATTERN DOTS-68 */
  ["\u28A1", "\u28A1"], /* BRAILLE PATTERN DOTS-168 */
  ["\u28A2", "\u28A2"], /* BRAILLE PATTERN DOTS-268 */
  ["\u28A3", "\u28A3"], /* BRAILLE PATTERN DOTS-1268 */
  ["\u28A4", "\u28A4"], /* BRAILLE PATTERN DOTS-368 */
  ["\u28A5", "\u28A5"], /* BRAILLE PATTERN DOTS-1368 */
  ["\u28A6", "\u28A6"], /* BRAILLE PATTERN DOTS-2368 */
  ["\u28A7", "\u28A7"], /* BRAILLE PATTERN DOTS-12368 */
  ["\u28A8", "\u28A8"], /* BRAILLE PATTERN DOTS-468 */
  ["\u28A9", "\u28A9"], /* BRAILLE PATTERN DOTS-1468 */
  ["\u28AA", "\u28AA"], /* BRAILLE PATTERN DOTS-2468 */
  ["\u28AB", "\u28AB"], /* BRAILLE PATTERN DOTS-12468 */
  ["\u28AC", "\u28AC"], /* BRAILLE PATTERN DOTS-3468 */
  ["\u28AD", "\u28AD"], /* BRAILLE PATTERN DOTS-13468 */
  ["\u28AE", "\u28AE"], /* BRAILLE PATTERN DOTS-23468 */
  ["\u28AF", "\u28AF"], /* BRAILLE PATTERN DOTS-123468 */
  ["\u28B0", "\u28B0"], /* BRAILLE PATTERN DOTS-568 */
  ["\u28B1", "\u28B1"], /* BRAILLE PATTERN DOTS-1568 */
  ["\u28B2", "\u28B2"], /* BRAILLE PATTERN DOTS-2568 */
  ["\u28B3", "\u28B3"], /* BRAILLE PATTERN DOTS-12568 */
  ["\u28B4", "\u28B4"], /* BRAILLE PATTERN DOTS-3568 */
  ["\u28B5", "\u28B5"], /* BRAILLE PATTERN DOTS-13568 */
  ["\u28B6", "\u28B6"], /* BRAILLE PATTERN DOTS-23568 */
  ["\u28B7", "\u28B7"], /* BRAILLE PATTERN DOTS-123568 */
  ["\u28B8", "\u28B8"], /* BRAILLE PATTERN DOTS-4568 */
  ["\u28B9", "\u28B9"], /* BRAILLE PATTERN DOTS-14568 */
  ["\u28BA", "\u28BA"], /* BRAILLE PATTERN DOTS-24568 */
  ["\u28BB", "\u28BB"], /* BRAILLE PATTERN DOTS-124568 */
  ["\u28BC", "\u28BC"], /* BRAILLE PATTERN DOTS-34568 */
  ["\u28BD", "\u28BD"], /* BRAILLE PATTERN DOTS-134568 */
  ["\u28BE", "\u28BE"], /* BRAILLE PATTERN DOTS-234568 */
  ["\u28BF", "\u28BF"], /* BRAILLE PATTERN DOTS-1234568 */
  ["\u28C0", "\u28C0"], /* BRAILLE PATTERN DOTS-78 */
  ["\u28C1", "\u28C1"], /* BRAILLE PATTERN DOTS-178 */
  ["\u28C2", "\u28C2"], /* BRAILLE PATTERN DOTS-278 */
  ["\u28C3", "\u28C3"], /* BRAILLE PATTERN DOTS-1278 */
  ["\u28C4", "\u28C4"], /* BRAILLE PATTERN DOTS-378 */
  ["\u28C5", "\u28C5"], /* BRAILLE PATTERN DOTS-1378 */
  ["\u28C6", "\u28C6"], /* BRAILLE PATTERN DOTS-2378 */
  ["\u28C7", "\u28C7"], /* BRAILLE PATTERN DOTS-12378 */
  ["\u28C8", "\u28C8"], /* BRAILLE PATTERN DOTS-478 */
  ["\u28C9", "\u28C9"], /* BRAILLE PATTERN DOTS-1478 */
  ["\u28CA", "\u28CA"], /* BRAILLE PATTERN DOTS-2478 */
  ["\u28CB", "\u28CB"], /* BRAILLE PATTERN DOTS-12478 */
  ["\u28CC", "\u28CC"], /* BRAILLE PATTERN DOTS-3478 */
  ["\u28CD", "\u28CD"], /* BRAILLE PATTERN DOTS-13478 */
  ["\u28CE", "\u28CE"], /* BRAILLE PATTERN DOTS-23478 */
  ["\u28CF", "\u28CF"], /* BRAILLE PATTERN DOTS-123478 */
  ["\u28D0", "\u28D0"], /* BRAILLE PATTERN DOTS-578 */
  ["\u28D1", "\u28D1"], /* BRAILLE PATTERN DOTS-1578 */
  ["\u28D2", "\u28D2"], /* BRAILLE PATTERN DOTS-2578 */
  ["\u28D3", "\u28D3"], /* BRAILLE PATTERN DOTS-12578 */
  ["\u28D4", "\u28D4"], /* BRAILLE PATTERN DOTS-3578 */
  ["\u28D5", "\u28D5"], /* BRAILLE PATTERN DOTS-13578 */
  ["\u28D6", "\u28D6"], /* BRAILLE PATTERN DOTS-23578 */
  ["\u28D7", "\u28D7"], /* BRAILLE PATTERN DOTS-123578 */
  ["\u28D8", "\u28D8"], /* BRAILLE PATTERN DOTS-4578 */
  ["\u28D9", "\u28D9"], /* BRAILLE PATTERN DOTS-14578 */
  ["\u28DA", "\u28DA"], /* BRAILLE PATTERN DOTS-24578 */
  ["\u28DB", "\u28DB"], /* BRAILLE PATTERN DOTS-124578 */
  ["\u28DC", "\u28DC"], /* BRAILLE PATTERN DOTS-34578 */
  ["\u28DD", "\u28DD"], /* BRAILLE PATTERN DOTS-134578 */
  ["\u28DE", "\u28DE"], /* BRAILLE PATTERN DOTS-234578 */
  ["\u28DF", "\u28DF"], /* BRAILLE PATTERN DOTS-1234578 */
  ["\u28E0", "\u28E0"], /* BRAILLE PATTERN DOTS-678 */
  ["\u28E1", "\u28E1"], /* BRAILLE PATTERN DOTS-1678 */
  ["\u28E2", "\u28E2"], /* BRAILLE PATTERN DOTS-2678 */
  ["\u28E3", "\u28E3"], /* BRAILLE PATTERN DOTS-12678 */
  ["\u28E4", "\u28E4"], /* BRAILLE PATTERN DOTS-3678 */
  ["\u28E5", "\u28E5"], /* BRAILLE PATTERN DOTS-13678 */
  ["\u28E6", "\u28E6"], /* BRAILLE PATTERN DOTS-23678 */
  ["\u28E7", "\u28E7"], /* BRAILLE PATTERN DOTS-123678 */
  ["\u28E8", "\u28E8"], /* BRAILLE PATTERN DOTS-4678 */
  ["\u28E9", "\u28E9"], /* BRAILLE PATTERN DOTS-14678 */
  ["\u28EA", "\u28EA"], /* BRAILLE PATTERN DOTS-24678 */
  ["\u28EB", "\u28EB"], /* BRAILLE PATTERN DOTS-124678 */
  ["\u28EC", "\u28EC"], /* BRAILLE PATTERN DOTS-34678 */
  ["\u28ED", "\u28ED"], /* BRAILLE PATTERN DOTS-134678 */
  ["\u28EE", "\u28EE"], /* BRAILLE PATTERN DOTS-234678 */
  ["\u28EF", "\u28EF"], /* BRAILLE PATTERN DOTS-1234678 */
  ["\u28F0", "\u28F0"], /* BRAILLE PATTERN DOTS-5678 */
  ["\u28F1", "\u28F1"], /* BRAILLE PATTERN DOTS-15678 */
  ["\u28F2", "\u28F2"], /* BRAILLE PATTERN DOTS-25678 */
  ["\u28F3", "\u28F3"], /* BRAILLE PATTERN DOTS-125678 */
  ["\u28F4", "\u28F4"], /* BRAILLE PATTERN DOTS-35678 */
  ["\u28F5", "\u28F5"], /* BRAILLE PATTERN DOTS-135678 */
  ["\u28F6", "\u28F6"], /* BRAILLE PATTERN DOTS-235678 */
  ["\u28F7", "\u28F7"], /* BRAILLE PATTERN DOTS-1235678 */
  ["\u28F8", "\u28F8"], /* BRAILLE PATTERN DOTS-45678 */
  ["\u28F9", "\u28F9"], /* BRAILLE PATTERN DOTS-145678 */
  ["\u28FA", "\u28FA"], /* BRAILLE PATTERN DOTS-245678 */
  ["\u28FB", "\u28FB"], /* BRAILLE PATTERN DOTS-1245678 */
  ["\u28FC", "\u28FC"], /* BRAILLE PATTERN DOTS-345678 */
  ["\u28FD", "\u28FD"], /* BRAILLE PATTERN DOTS-1345678 */
  ["\u28FE", "\u28FE"], /* BRAILLE PATTERN DOTS-2345678 */
  ["\u28FF", "\u28FF"], /* BRAILLE PATTERN DOTS-12345678 */
  ["\u2900", "\u2900"], /* RIGHTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE */
  ["\u2901", "\u2901"], /* RIGHTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE */
  ["\u2902", "\u2902"], /* LEFTWARDS DOUBLE ARROW WITH VERTICAL STROKE */
  ["\u2903", "\u2903"], /* RIGHTWARDS DOUBLE ARROW WITH VERTICAL STROKE */
  ["\u2904", "\u2904"], /* LEFT RIGHT DOUBLE ARROW WITH VERTICAL STROKE */
  ["\u2905", "\u2905"], /* RIGHTWARDS TWO-HEADED ARROW FROM BAR */
  ["\u2906", "\u2906"], /* LEFTWARDS DOUBLE ARROW FROM BAR */
  ["\u2907", "\u2907"], /* RIGHTWARDS DOUBLE ARROW FROM BAR */
  ["\u2908", "\u2908"], /* DOWNWARDS ARROW WITH HORIZONTAL STROKE */
  ["\u2909", "\u2909"], /* UPWARDS ARROW WITH HORIZONTAL STROKE */
  ["\u290A", "\u290A"], /* UPWARDS TRIPLE ARROW */
  ["\u290B", "\u290B"], /* DOWNWARDS TRIPLE ARROW */
  ["\u290C", "\u290C"], /* LEFTWARDS DOUBLE DASH ARROW */
  ["\u290D", "\u290D"], /* RIGHTWARDS DOUBLE DASH ARROW */
  ["\u290E", "\u290E"], /* LEFTWARDS TRIPLE DASH ARROW */
  ["\u290F", "\u290F"], /* RIGHTWARDS TRIPLE DASH ARROW */
  ["\u2910", "\u2910"], /* RIGHTWARDS TWO-HEADED TRIPLE DASH ARROW */
  ["\u2911", "\u2911"], /* RIGHTWARDS ARROW WITH DOTTED STEM */
  ["\u2912", "\u2912"], /* UPWARDS ARROW TO BAR */
  ["\u2913", "\u2913"], /* DOWNWARDS ARROW TO BAR */
  ["\u2914", "\u2914"], /* RIGHTWARDS ARROW WITH TAIL WITH VERTICAL STROKE */
  ["\u2915", "\u2915"], /* RIGHTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE */
  ["\u2916", "\u2916"], /* RIGHTWARDS TWO-HEADED ARROW WITH TAIL */
  ["\u2917", "\u2917"], /* RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE */
  ["\u2918", "\u2918"], /* RIGHTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE */
  ["\u2919", "\u2919"], /* LEFTWARDS ARROW-TAIL */
  ["\u291A", "\u291A"], /* RIGHTWARDS ARROW-TAIL */
  ["\u291B", "\u291B"], /* LEFTWARDS DOUBLE ARROW-TAIL */
  ["\u291C", "\u291C"], /* RIGHTWARDS DOUBLE ARROW-TAIL */
  ["\u291D", "\u291D"], /* LEFTWARDS ARROW TO BLACK DIAMOND */
  ["\u291E", "\u291E"], /* RIGHTWARDS ARROW TO BLACK DIAMOND */
  ["\u291F", "\u291F"], /* LEFTWARDS ARROW FROM BAR TO BLACK DIAMOND */
  ["\u2920", "\u2920"], /* RIGHTWARDS ARROW FROM BAR TO BLACK DIAMOND */
  ["\u2921", "\u2921"], /* NORTH WEST AND SOUTH EAST ARROW */
  ["\u2922", "\u2922"], /* NORTH EAST AND SOUTH WEST ARROW */
  ["\u2923", "\u2923"], /* NORTH WEST ARROW WITH HOOK */
  ["\u2924", "\u2924"], /* NORTH EAST ARROW WITH HOOK */
  ["\u2925", "\u2925"], /* SOUTH EAST ARROW WITH HOOK */
  ["\u2926", "\u2926"], /* SOUTH WEST ARROW WITH HOOK */
  ["\u2927", "\u2927"], /* NORTH WEST ARROW AND NORTH EAST ARROW */
  ["\u2928", "\u2928"], /* NORTH EAST ARROW AND SOUTH EAST ARROW */
  ["\u2929", "\u2929"], /* SOUTH EAST ARROW AND SOUTH WEST ARROW */
  ["\u292A", "\u292A"], /* SOUTH WEST ARROW AND NORTH WEST ARROW */
  ["\u292B", "\u292B"], /* RISING DIAGONAL CROSSING FALLING DIAGONAL */
  ["\u292C", "\u292C"], /* FALLING DIAGONAL CROSSING RISING DIAGONAL */
  ["\u292D", "\u292D"], /* SOUTH EAST ARROW CROSSING NORTH EAST ARROW */
  ["\u292E", "\u292E"], /* NORTH EAST ARROW CROSSING SOUTH EAST ARROW */
  ["\u292F", "\u292F"], /* FALLING DIAGONAL CROSSING NORTH EAST ARROW */
  ["\u2930", "\u2930"], /* RISING DIAGONAL CROSSING SOUTH EAST ARROW */
  ["\u2931", "\u2931"], /* NORTH EAST ARROW CROSSING NORTH WEST ARROW */
  ["\u2932", "\u2932"], /* NORTH WEST ARROW CROSSING NORTH EAST ARROW */
  ["\u2933", "\u2933"], /* WAVE ARROW POINTING DIRECTLY RIGHT */
  ["\u2934", "\u2934"], /* ARROW POINTING RIGHTWARDS THEN CURVING UPWARDS */
  ["\u2935", "\u2935"], /* ARROW POINTING RIGHTWARDS THEN CURVING DOWNWARDS */
  ["\u2936", "\u2936"], /* ARROW POINTING DOWNWARDS THEN CURVING LEFTWARDS */
  ["\u2937", "\u2937"], /* ARROW POINTING DOWNWARDS THEN CURVING RIGHTWARDS */
  ["\u2938", "\u2938"], /* RIGHT-SIDE ARC CLOCKWISE ARROW */
  ["\u2939", "\u2939"], /* LEFT-SIDE ARC ANTICLOCKWISE ARROW */
  ["\u293A", "\u293A"], /* TOP ARC ANTICLOCKWISE ARROW */
  ["\u293B", "\u293B"], /* BOTTOM ARC ANTICLOCKWISE ARROW */
  ["\u293C", "\u293C"], /* TOP ARC CLOCKWISE ARROW WITH MINUS */
  ["\u293D", "\u293D"], /* TOP ARC ANTICLOCKWISE ARROW WITH PLUS */
  ["\u293E", "\u293E"], /* LOWER RIGHT SEMICIRCULAR CLOCKWISE ARROW */
  ["\u293F", "\u293F"], /* LOWER LEFT SEMICIRCULAR ANTICLOCKWISE ARROW */
  ["\u2940", "\u2940"], /* ANTICLOCKWISE CLOSED CIRCLE ARROW */
  ["\u2941", "\u2941"], /* CLOCKWISE CLOSED CIRCLE ARROW */
  ["\u2942", "\u2942"], /* RIGHTWARDS ARROW ABOVE SHORT LEFTWARDS ARROW */
  ["\u2943", "\u2943"], /* LEFTWARDS ARROW ABOVE SHORT RIGHTWARDS ARROW */
  ["\u2944", "\u2944"], /* SHORT RIGHTWARDS ARROW ABOVE LEFTWARDS ARROW */
  ["\u2945", "\u2945"], /* RIGHTWARDS ARROW WITH PLUS BELOW */
  ["\u2946", "\u2946"], /* LEFTWARDS ARROW WITH PLUS BELOW */
  ["\u2947", "\u2947"], /* RIGHTWARDS ARROW THROUGH X */
  ["\u2948", "\u2948"], /* LEFT RIGHT ARROW THROUGH SMALL CIRCLE */
  ["\u2949", "\u2949"], /* UPWARDS TWO-HEADED ARROW FROM SMALL CIRCLE */
  ["\u294A", "\u294A"], /* LEFT BARB UP RIGHT BARB DOWN HARPOON */
  ["\u294B", "\u294B"], /* LEFT BARB DOWN RIGHT BARB UP HARPOON */
  ["\u294C", "\u294C"], /* UP BARB RIGHT DOWN BARB LEFT HARPOON */
  ["\u294D", "\u294D"], /* UP BARB LEFT DOWN BARB RIGHT HARPOON */
  ["\u294E", "\u294E"], /* LEFT BARB UP RIGHT BARB UP HARPOON */
  ["\u294F", "\u294F"], /* UP BARB RIGHT DOWN BARB RIGHT HARPOON */
  ["\u2950", "\u2950"], /* LEFT BARB DOWN RIGHT BARB DOWN HARPOON */
  ["\u2951", "\u2951"], /* UP BARB LEFT DOWN BARB LEFT HARPOON */
  ["\u2952", "\u2952"], /* LEFTWARDS HARPOON WITH BARB UP TO BAR */
  ["\u2953", "\u2953"], /* RIGHTWARDS HARPOON WITH BARB UP TO BAR */
  ["\u2954", "\u2954"], /* UPWARDS HARPOON WITH BARB RIGHT TO BAR */
  ["\u2955", "\u2955"], /* DOWNWARDS HARPOON WITH BARB RIGHT TO BAR */
  ["\u2956", "\u2956"], /* LEFTWARDS HARPOON WITH BARB DOWN TO BAR */
  ["\u2957", "\u2957"], /* RIGHTWARDS HARPOON WITH BARB DOWN TO BAR */
  ["\u2958", "\u2958"], /* UPWARDS HARPOON WITH BARB LEFT TO BAR */
  ["\u2959", "\u2959"], /* DOWNWARDS HARPOON WITH BARB LEFT TO BAR */
  ["\u295A", "\u295A"], /* LEFTWARDS HARPOON WITH BARB UP FROM BAR */
  ["\u295B", "\u295B"], /* RIGHTWARDS HARPOON WITH BARB UP FROM BAR */
  ["\u295C", "\u295C"], /* UPWARDS HARPOON WITH BARB RIGHT FROM BAR */
  ["\u295D", "\u295D"], /* DOWNWARDS HARPOON WITH BARB RIGHT FROM BAR */
  ["\u295E", "\u295E"], /* LEFTWARDS HARPOON WITH BARB DOWN FROM BAR */
  ["\u295F", "\u295F"], /* RIGHTWARDS HARPOON WITH BARB DOWN FROM BAR */
  ["\u2960", "\u2960"], /* UPWARDS HARPOON WITH BARB LEFT FROM BAR */
  ["\u2961", "\u2961"], /* DOWNWARDS HARPOON WITH BARB LEFT FROM BAR */
  ["\u2962", "\u2962"], /* LEFTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB DOWN */
  ["\u2963", "\u2963"], /* UPWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT */
  ["\u2964", "\u2964"], /* RIGHTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB DOWN */
  ["\u2965", "\u2965"], /* DOWNWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT */
  ["\u2966", "\u2966"], /* LEFTWARDS HARPOON WITH BARB UP ABOVE RIGHTWARDS HARPOON WITH BARB UP */
  ["\u2967", "\u2967"], /* LEFTWARDS HARPOON WITH BARB DOWN ABOVE RIGHTWARDS HARPOON WITH BARB DOWN */
  ["\u2968", "\u2968"], /* RIGHTWARDS HARPOON WITH BARB UP ABOVE LEFTWARDS HARPOON WITH BARB UP */
  ["\u2969", "\u2969"], /* RIGHTWARDS HARPOON WITH BARB DOWN ABOVE LEFTWARDS HARPOON WITH BARB DOWN */
  ["\u296A", "\u296A"], /* LEFTWARDS HARPOON WITH BARB UP ABOVE LONG DASH */
  ["\u296B", "\u296B"], /* LEFTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH */
  ["\u296C", "\u296C"], /* RIGHTWARDS HARPOON WITH BARB UP ABOVE LONG DASH */
  ["\u296D", "\u296D"], /* RIGHTWARDS HARPOON WITH BARB DOWN BELOW LONG DASH */
  ["\u296E", "\u296E"], /* UPWARDS HARPOON WITH BARB LEFT BESIDE DOWNWARDS HARPOON WITH BARB RIGHT */
  ["\u296F", "\u296F"], /* DOWNWARDS HARPOON WITH BARB LEFT BESIDE UPWARDS HARPOON WITH BARB RIGHT */
  ["\u2970", "\u2970"], /* RIGHT DOUBLE ARROW WITH ROUNDED HEAD */
  ["\u2971", "\u2971"], /* EQUALS SIGN ABOVE RIGHTWARDS ARROW */
  ["\u2972", "\u2972"], /* TILDE OPERATOR ABOVE RIGHTWARDS ARROW */
  ["\u2973", "\u2973"], /* LEFTWARDS ARROW ABOVE TILDE OPERATOR */
  ["\u2974", "\u2974"], /* RIGHTWARDS ARROW ABOVE TILDE OPERATOR */
  ["\u2975", "\u2975"], /* RIGHTWARDS ARROW ABOVE ALMOST EQUAL TO */
  ["\u2976", "\u2976"], /* LESS-THAN ABOVE LEFTWARDS ARROW */
  ["\u2977", "\u2977"], /* LEFTWARDS ARROW THROUGH LESS-THAN */
  ["\u2978", "\u2978"], /* GREATER-THAN ABOVE RIGHTWARDS ARROW */
  ["\u2979", "\u2979"], /* SUBSET ABOVE RIGHTWARDS ARROW */
  ["\u297A", "\u297A"], /* LEFTWARDS ARROW THROUGH SUBSET */
  ["\u297B", "\u297B"], /* SUPERSET ABOVE LEFTWARDS ARROW */
  ["\u297C", "\u297C"], /* LEFT FISH TAIL */
  ["\u297D", "\u297D"], /* RIGHT FISH TAIL */
  ["\u297E", "\u297E"], /* UP FISH TAIL */
  ["\u297F", "\u297F"], /* DOWN FISH TAIL */
  ["\u2980", "\u2980"], /* TRIPLE VERTICAL BAR DELIMITER */
  ["\u2981", "\u2981"], /* Z NOTATION SPOT */
  ["\u2982", "\u2982"], /* Z NOTATION TYPE COLON */
  ["\u2983", "\u2983"], /* LEFT WHITE CURLY BRACKET */
  ["\u2984", "\u2984"], /* RIGHT WHITE CURLY BRACKET */
  ["\u2985", "\u2985"], /* LEFT WHITE PARENTHESIS */
  ["\u2986", "\u2986"], /* RIGHT WHITE PARENTHESIS */
  ["\u2987", "\u2987"], /* Z NOTATION LEFT IMAGE BRACKET */
  ["\u2988", "\u2988"], /* Z NOTATION RIGHT IMAGE BRACKET */
  ["\u2989", "\u2989"], /* Z NOTATION LEFT BINDING BRACKET */
  ["\u298A", "\u298A"], /* Z NOTATION RIGHT BINDING BRACKET */
  ["\u298B", "\u298B"], /* LEFT SQUARE BRACKET WITH UNDERBAR */
  ["\u298C", "\u298C"], /* RIGHT SQUARE BRACKET WITH UNDERBAR */
  ["\u298D", "\u298D"], /* LEFT SQUARE BRACKET WITH TICK IN TOP CORNER */
  ["\u298E", "\u298E"], /* RIGHT SQUARE BRACKET WITH TICK IN BOTTOM CORNER */
  ["\u298F", "\u298F"], /* LEFT SQUARE BRACKET WITH TICK IN BOTTOM CORNER */
  ["\u2990", "\u2990"], /* RIGHT SQUARE BRACKET WITH TICK IN TOP CORNER */
  ["\u2991", "\u2991"], /* LEFT ANGLE BRACKET WITH DOT */
  ["\u2992", "\u2992"], /* RIGHT ANGLE BRACKET WITH DOT */
  ["\u2993", "\u2993"], /* LEFT ARC LESS-THAN BRACKET */
  ["\u2994", "\u2994"], /* RIGHT ARC GREATER-THAN BRACKET */
  ["\u2995", "\u2995"], /* DOUBLE LEFT ARC GREATER-THAN BRACKET */
  ["\u2996", "\u2996"], /* DOUBLE RIGHT ARC LESS-THAN BRACKET */
  ["\u2997", "\u2997"], /* LEFT BLACK TORTOISE SHELL BRACKET */
  ["\u2998", "\u2998"], /* RIGHT BLACK TORTOISE SHELL BRACKET */
  ["\u2999", "\u2999"], /* DOTTED FENCE */
  ["\u299A", "\u299A"], /* VERTICAL ZIGZAG LINE */
  ["\u299B", "\u299B"], /* MEASURED ANGLE OPENING LEFT */
  ["\u299C", "\u299C"], /* RIGHT ANGLE VARIANT WITH SQUARE */
  ["\u299D", "\u299D"], /* MEASURED RIGHT ANGLE WITH DOT */
  ["\u299E", "\u299E"], /* ANGLE WITH S INSIDE */
  ["\u299F", "\u299F"], /* ACUTE ANGLE */
  ["\u29A0", "\u29A0"], /* SPHERICAL ANGLE OPENING LEFT */
  ["\u29A1", "\u29A1"], /* SPHERICAL ANGLE OPENING UP */
  ["\u29A2", "\u29A2"], /* TURNED ANGLE */
  ["\u29A3", "\u29A3"], /* REVERSED ANGLE */
  ["\u29A4", "\u29A4"], /* ANGLE WITH UNDERBAR */
  ["\u29A5", "\u29A5"], /* REVERSED ANGLE WITH UNDERBAR */
  ["\u29A6", "\u29A6"], /* OBLIQUE ANGLE OPENING UP */
  ["\u29A7", "\u29A7"], /* OBLIQUE ANGLE OPENING DOWN */
  ["\u29A8", "\u29A8"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND RIGHT */
  ["\u29A9", "\u29A9"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING UP AND LEFT */
  ["\u29AA", "\u29AA"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND RIGHT */
  ["\u29AB", "\u29AB"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING DOWN AND LEFT */
  ["\u29AC", "\u29AC"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND UP */
  ["\u29AD", "\u29AD"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND UP */
  ["\u29AE", "\u29AE"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING RIGHT AND DOWN */
  ["\u29AF", "\u29AF"], /* MEASURED ANGLE WITH OPEN ARM ENDING IN ARROW POINTING LEFT AND DOWN */
  ["\u29B0", "\u29B0"], /* REVERSED EMPTY SET */
  ["\u29B1", "\u29B1"], /* EMPTY SET WITH OVERBAR */
  ["\u29B2", "\u29B2"], /* EMPTY SET WITH SMALL CIRCLE ABOVE */
  ["\u29B3", "\u29B3"], /* EMPTY SET WITH RIGHT ARROW ABOVE */
  ["\u29B4", "\u29B4"], /* EMPTY SET WITH LEFT ARROW ABOVE */
  ["\u29B5", "\u29B5"], /* CIRCLE WITH HORIZONTAL BAR */
  ["\u29B6", "\u29B6"], /* CIRCLED VERTICAL BAR */
  ["\u29B7", "\u29B7"], /* CIRCLED PARALLEL */
  ["\u29B8", "\u29B8"], /* CIRCLED REVERSE SOLIDUS */
  ["\u29B9", "\u29B9"], /* CIRCLED PERPENDICULAR */
  ["\u29BA", "\u29BA"], /* CIRCLE DIVIDED BY HORIZONTAL BAR AND TOP HALF DIVIDED BY VERTICAL BAR */
  ["\u29BB", "\u29BB"], /* CIRCLE WITH SUPERIMPOSED X */
  ["\u29BC", "\u29BC"], /* CIRCLED ANTICLOCKWISE-ROTATED DIVISION SIGN */
  ["\u29BD", "\u29BD"], /* UP ARROW THROUGH CIRCLE */
  ["\u29BE", "\u29BE"], /* CIRCLED WHITE BULLET */
  ["\u29BF", "\u29BF"], /* CIRCLED BULLET */
  ["\u29C0", "\u29C0"], /* CIRCLED LESS-THAN */
  ["\u29C1", "\u29C1"], /* CIRCLED GREATER-THAN */
  ["\u29C2", "\u29C2"], /* CIRCLE WITH SMALL CIRCLE TO THE RIGHT */
  ["\u29C3", "\u29C3"], /* CIRCLE WITH TWO HORIZONTAL STROKES TO THE RIGHT */
  ["\u29C4", "\u29C4"], /* SQUARED RISING DIAGONAL SLASH */
  ["\u29C5", "\u29C5"], /* SQUARED FALLING DIAGONAL SLASH */
  ["\u29C6", "\u29C6"], /* SQUARED ASTERISK */
  ["\u29C7", "\u29C7"], /* SQUARED SMALL CIRCLE */
  ["\u29C8", "\u29C8"], /* SQUARED SQUARE */
  ["\u29C9", "\u29C9"], /* TWO JOINED SQUARES */
  ["\u29CA", "\u29CA"], /* TRIANGLE WITH DOT ABOVE */
  ["\u29CB", "\u29CB"], /* TRIANGLE WITH UNDERBAR */
  ["\u29CC", "\u29CC"], /* S IN TRIANGLE */
  ["\u29CD", "\u29CD"], /* TRIANGLE WITH SERIFS AT BOTTOM */
  ["\u29CE", "\u29CE"], /* RIGHT TRIANGLE ABOVE LEFT TRIANGLE */
  ["\u29CF", "\u29CF"], /* LEFT TRIANGLE BESIDE VERTICAL BAR */
  ["\u29D0", "\u29D0"], /* VERTICAL BAR BESIDE RIGHT TRIANGLE */
  ["\u29D1", "\u29D1"], /* BOWTIE WITH LEFT HALF BLACK */
  ["\u29D2", "\u29D2"], /* BOWTIE WITH RIGHT HALF BLACK */
  ["\u29D3", "\u29D3"], /* BLACK BOWTIE */
  ["\u29D4", "\u29D4"], /* TIMES WITH LEFT HALF BLACK */
  ["\u29D5", "\u29D5"], /* TIMES WITH RIGHT HALF BLACK */
  ["\u29D6", "\u29D6"], /* WHITE HOURGLASS */
  ["\u29D7", "\u29D7"], /* BLACK HOURGLASS */
  ["\u29D8", "\u29D8"], /* LEFT WIGGLY FENCE */
  ["\u29D9", "\u29D9"], /* RIGHT WIGGLY FENCE */
  ["\u29DA", "\u29DA"], /* LEFT DOUBLE WIGGLY FENCE */
  ["\u29DB", "\u29DB"], /* RIGHT DOUBLE WIGGLY FENCE */
  ["\u29DC", "\u29DC"], /* INCOMPLETE INFINITY */
  ["\u29DD", "\u29DD"], /* TIE OVER INFINITY */
  ["\u29DE", "\u29DE"], /* INFINITY NEGATED WITH VERTICAL BAR */
  ["\u29DF", "\u29DF"], /* DOUBLE-ENDED MULTIMAP */
  ["\u29E0", "\u29E0"], /* SQUARE WITH CONTOURED OUTLINE */
  ["\u29E1", "\u29E1"], /* INCREASES AS */
  ["\u29E2", "\u29E2"], /* SHUFFLE PRODUCT */
  ["\u29E3", "\u29E3"], /* EQUALS SIGN AND SLANTED PARALLEL */
  ["\u29E4", "\u29E4"], /* EQUALS SIGN AND SLANTED PARALLEL WITH TILDE ABOVE */
  ["\u29E5", "\u29E5"], /* IDENTICAL TO AND SLANTED PARALLEL */
  ["\u29E6", "\u29E6"], /* GLEICH STARK */
  ["\u29E7", "\u29E7"], /* THERMODYNAMIC */
  ["\u29E8", "\u29E8"], /* DOWN-POINTING TRIANGLE WITH LEFT HALF BLACK */
  ["\u29E9", "\u29E9"], /* DOWN-POINTING TRIANGLE WITH RIGHT HALF BLACK */
  ["\u29EA", "\u29EA"], /* BLACK DIAMOND WITH DOWN ARROW */
  ["\u29EB", "\u29EB"], /* BLACK LOZENGE */
  ["\u29EC", "\u29EC"], /* WHITE CIRCLE WITH DOWN ARROW */
  ["\u29ED", "\u29ED"], /* BLACK CIRCLE WITH DOWN ARROW */
  ["\u29EE", "\u29EE"], /* ERROR-BARRED WHITE SQUARE */
  ["\u29EF", "\u29EF"], /* ERROR-BARRED BLACK SQUARE */
  ["\u29F0", "\u29F0"], /* ERROR-BARRED WHITE DIAMOND */
  ["\u29F1", "\u29F1"], /* ERROR-BARRED BLACK DIAMOND */
  ["\u29F2", "\u29F2"], /* ERROR-BARRED WHITE CIRCLE */
  ["\u29F3", "\u29F3"], /* ERROR-BARRED BLACK CIRCLE */
  ["\u29F4", "\u29F4"], /* RULE-DELAYED */
  ["\u29F5", "\u29F5"], /* REVERSE SOLIDUS OPERATOR */
  ["\u29F6", "\u29F6"], /* SOLIDUS WITH OVERBAR */
  ["\u29F7", "\u29F7"], /* REVERSE SOLIDUS WITH HORIZONTAL STROKE */
  ["\u29F8", "\u29F8"], /* BIG SOLIDUS */
  ["\u29F9", "\u29F9"], /* BIG REVERSE SOLIDUS */
  ["\u29FA", "\u29FA"], /* DOUBLE PLUS */
  ["\u29FB", "\u29FB"], /* TRIPLE PLUS */
  ["\u29FC", "\u29FC"], /* LEFT-POINTING CURVED ANGLE BRACKET */
  ["\u29FD", "\u29FD"], /* RIGHT-POINTING CURVED ANGLE BRACKET */
  ["\u29FE", "\u29FE"], /* TINY */
  ["\u29FF", "\u29FF"], /* MINY */
  ["\u2A00", "\u2A00"], /* N-ARY CIRCLED DOT OPERATOR */
  ["\u2A01", "\u2A01"], /* N-ARY CIRCLED PLUS OPERATOR */
  ["\u2A02", "\u2A02"], /* N-ARY CIRCLED TIMES OPERATOR */
  ["\u2A03", "\u2A03"], /* N-ARY UNION OPERATOR WITH DOT */
  ["\u2A04", "\u2A04"], /* N-ARY UNION OPERATOR WITH PLUS */
  ["\u2A05", "\u2A05"], /* N-ARY SQUARE INTERSECTION OPERATOR */
  ["\u2A06", "\u2A06"], /* N-ARY SQUARE UNION OPERATOR */
  ["\u2A07", "\u2A07"], /* TWO LOGICAL AND OPERATOR */
  ["\u2A08", "\u2A08"], /* TWO LOGICAL OR OPERATOR */
  ["\u2A09", "\u2A09"], /* N-ARY TIMES OPERATOR */
  ["\u2A0A", "\u2A0A"], /* MODULO TWO SUM */
  ["\u2A0B", "\u2A0B"], /* SUMMATION WITH INTEGRAL */
  ["\u2A0C", "\u2A0C"], /* QUADRUPLE INTEGRAL OPERATOR */
  ["\u2A0D", "\u2A0D"], /* FINITE PART INTEGRAL */
  ["\u2A0E", "\u2A0E"], /* INTEGRAL WITH DOUBLE STROKE */
  ["\u2A0F", "\u2A0F"], /* INTEGRAL AVERAGE WITH SLASH */
  ["\u2A10", "\u2A10"], /* CIRCULATION FUNCTION */
  ["\u2A11", "\u2A11"], /* ANTICLOCKWISE INTEGRATION */
  ["\u2A12", "\u2A12"], /* LINE INTEGRATION WITH RECTANGULAR PATH AROUND POLE */
  ["\u2A13", "\u2A13"], /* LINE INTEGRATION WITH SEMICIRCULAR PATH AROUND POLE */
  ["\u2A14", "\u2A14"], /* LINE INTEGRATION NOT INCLUDING THE POLE */
  ["\u2A15", "\u2A15"], /* INTEGRAL AROUND A POINT OPERATOR */
  ["\u2A16", "\u2A16"], /* QUATERNION INTEGRAL OPERATOR */
  ["\u2A17", "\u2A17"], /* INTEGRAL WITH LEFTWARDS ARROW WITH HOOK */
  ["\u2A18", "\u2A18"], /* INTEGRAL WITH TIMES SIGN */
  ["\u2A19", "\u2A19"], /* INTEGRAL WITH INTERSECTION */
  ["\u2A1A", "\u2A1A"], /* INTEGRAL WITH UNION */
  ["\u2A1B", "\u2A1B"], /* INTEGRAL WITH OVERBAR */
  ["\u2A1C", "\u2A1C"], /* INTEGRAL WITH UNDERBAR */
  ["\u2A1D", "\u2A1D"], /* JOIN */
  ["\u2A1E", "\u2A1E"], /* LARGE LEFT TRIANGLE OPERATOR */
  ["\u2A1F", "\u2A1F"], /* Z NOTATION SCHEMA COMPOSITION */
  ["\u2A20", "\u2A20"], /* Z NOTATION SCHEMA PIPING */
  ["\u2A21", "\u2A21"], /* Z NOTATION SCHEMA PROJECTION */
  ["\u2A22", "\u2A22"], /* PLUS SIGN WITH SMALL CIRCLE ABOVE */
  ["\u2A23", "\u2A23"], /* PLUS SIGN WITH CIRCUMFLEX ACCENT ABOVE */
  ["\u2A24", "\u2A24"], /* PLUS SIGN WITH TILDE ABOVE */
  ["\u2A25", "\u2A25"], /* PLUS SIGN WITH DOT BELOW */
  ["\u2A26", "\u2A26"], /* PLUS SIGN WITH TILDE BELOW */
  ["\u2A27", "\u2A27"], /* PLUS SIGN WITH SUBSCRIPT TWO */
  ["\u2A28", "\u2A28"], /* PLUS SIGN WITH BLACK TRIANGLE */
  ["\u2A29", "\u2A29"], /* MINUS SIGN WITH COMMA ABOVE */
  ["\u2A2A", "\u2A2A"], /* MINUS SIGN WITH DOT BELOW */
  ["\u2A2B", "\u2A2B"], /* MINUS SIGN WITH FALLING DOTS */
  ["\u2A2C", "\u2A2C"], /* MINUS SIGN WITH RISING DOTS */
  ["\u2A2D", "\u2A2D"], /* PLUS SIGN IN LEFT HALF CIRCLE */
  ["\u2A2E", "\u2A2E"], /* PLUS SIGN IN RIGHT HALF CIRCLE */
  ["\u2A2F", "\u2A2F"], /* VECTOR OR CROSS PRODUCT */
  ["\u2A30", "\u2A30"], /* MULTIPLICATION SIGN WITH DOT ABOVE */
  ["\u2A31", "\u2A31"], /* MULTIPLICATION SIGN WITH UNDERBAR */
  ["\u2A32", "\u2A32"], /* SEMIDIRECT PRODUCT WITH BOTTOM CLOSED */
  ["\u2A33", "\u2A33"], /* SMASH PRODUCT */
  ["\u2A34", "\u2A34"], /* MULTIPLICATION SIGN IN LEFT HALF CIRCLE */
  ["\u2A35", "\u2A35"], /* MULTIPLICATION SIGN IN RIGHT HALF CIRCLE */
  ["\u2A36", "\u2A36"], /* CIRCLED MULTIPLICATION SIGN WITH CIRCUMFLEX ACCENT */
  ["\u2A37", "\u2A37"], /* MULTIPLICATION SIGN IN DOUBLE CIRCLE */
  ["\u2A38", "\u2A38"], /* CIRCLED DIVISION SIGN */
  ["\u2A39", "\u2A39"], /* PLUS SIGN IN TRIANGLE */
  ["\u2A3A", "\u2A3A"], /* MINUS SIGN IN TRIANGLE */
  ["\u2A3B", "\u2A3B"], /* MULTIPLICATION SIGN IN TRIANGLE */
  ["\u2A3C", "\u2A3C"], /* INTERIOR PRODUCT */
  ["\u2A3D", "\u2A3D"], /* RIGHTHAND INTERIOR PRODUCT */
  ["\u2A3E", "\u2A3E"], /* Z NOTATION RELATIONAL COMPOSITION */
  ["\u2A3F", "\u2A3F"], /* AMALGAMATION OR COPRODUCT */
  ["\u2A40", "\u2A40"], /* INTERSECTION WITH DOT */
  ["\u2A41", "\u2A41"], /* UNION WITH MINUS SIGN */
  ["\u2A42", "\u2A42"], /* UNION WITH OVERBAR */
  ["\u2A43", "\u2A43"], /* INTERSECTION WITH OVERBAR */
  ["\u2A44", "\u2A44"], /* INTERSECTION WITH LOGICAL AND */
  ["\u2A45", "\u2A45"], /* UNION WITH LOGICAL OR */
  ["\u2A46", "\u2A46"], /* UNION ABOVE INTERSECTION */
  ["\u2A47", "\u2A47"], /* INTERSECTION ABOVE UNION */
  ["\u2A48", "\u2A48"], /* UNION ABOVE BAR ABOVE INTERSECTION */
  ["\u2A49", "\u2A49"], /* INTERSECTION ABOVE BAR ABOVE UNION */
  ["\u2A4A", "\u2A4A"], /* UNION BESIDE AND JOINED WITH UNION */
  ["\u2A4B", "\u2A4B"], /* INTERSECTION BESIDE AND JOINED WITH INTERSECTION */
  ["\u2A4C", "\u2A4C"], /* CLOSED UNION WITH SERIFS */
  ["\u2A4D", "\u2A4D"], /* CLOSED INTERSECTION WITH SERIFS */
  ["\u2A4E", "\u2A4E"], /* DOUBLE SQUARE INTERSECTION */
  ["\u2A4F", "\u2A4F"], /* DOUBLE SQUARE UNION */
  ["\u2A50", "\u2A50"], /* CLOSED UNION WITH SERIFS AND SMASH PRODUCT */
  ["\u2A51", "\u2A51"], /* LOGICAL AND WITH DOT ABOVE */
  ["\u2A52", "\u2A52"], /* LOGICAL OR WITH DOT ABOVE */
  ["\u2A53", "\u2A53"], /* DOUBLE LOGICAL AND */
  ["\u2A54", "\u2A54"], /* DOUBLE LOGICAL OR */
  ["\u2A55", "\u2A55"], /* TWO INTERSECTING LOGICAL AND */
  ["\u2A56", "\u2A56"], /* TWO INTERSECTING LOGICAL OR */
  ["\u2A57", "\u2A57"], /* SLOPING LARGE OR */
  ["\u2A58", "\u2A58"], /* SLOPING LARGE AND */
  ["\u2A59", "\u2A59"], /* LOGICAL OR OVERLAPPING LOGICAL AND */
  ["\u2A5A", "\u2A5A"], /* LOGICAL AND WITH MIDDLE STEM */
  ["\u2A5B", "\u2A5B"], /* LOGICAL OR WITH MIDDLE STEM */
  ["\u2A5C", "\u2A5C"], /* LOGICAL AND WITH HORIZONTAL DASH */
  ["\u2A5D", "\u2A5D"], /* LOGICAL OR WITH HORIZONTAL DASH */
  ["\u2A5E", "\u2A5E"], /* LOGICAL AND WITH DOUBLE OVERBAR */
  ["\u2A5F", "\u2A5F"], /* LOGICAL AND WITH UNDERBAR */
  ["\u2A60", "\u2A60"], /* LOGICAL AND WITH DOUBLE UNDERBAR */
  ["\u2A61", "\u2A61"], /* SMALL VEE WITH UNDERBAR */
  ["\u2A62", "\u2A62"], /* LOGICAL OR WITH DOUBLE OVERBAR */
  ["\u2A63", "\u2A63"], /* LOGICAL OR WITH DOUBLE UNDERBAR */
  ["\u2A64", "\u2A64"], /* Z NOTATION DOMAIN ANTIRESTRICTION */
  ["\u2A65", "\u2A65"], /* Z NOTATION RANGE ANTIRESTRICTION */
  ["\u2A66", "\u2A66"], /* EQUALS SIGN WITH DOT BELOW */
  ["\u2A67", "\u2A67"], /* IDENTICAL WITH DOT ABOVE */
  ["\u2A68", "\u2A68"], /* TRIPLE HORIZONTAL BAR WITH DOUBLE VERTICAL STROKE */
  ["\u2A69", "\u2A69"], /* TRIPLE HORIZONTAL BAR WITH TRIPLE VERTICAL STROKE */
  ["\u2A6A", "\u2A6A"], /* TILDE OPERATOR WITH DOT ABOVE */
  ["\u2A6B", "\u2A6B"], /* TILDE OPERATOR WITH RISING DOTS */
  ["\u2A6C", "\u2A6C"], /* SIMILAR MINUS SIMILAR */
  ["\u2A6D", "\u2A6D"], /* CONGRUENT WITH DOT ABOVE */
  ["\u2A6E", "\u2A6E"], /* EQUALS WITH ASTERISK */
  ["\u2A6F", "\u2A6F"], /* ALMOST EQUAL TO WITH CIRCUMFLEX ACCENT */
  ["\u2A70", "\u2A70"], /* APPROXIMATELY EQUAL OR EQUAL TO */
  ["\u2A71", "\u2A71"], /* EQUALS SIGN ABOVE PLUS SIGN */
  ["\u2A72", "\u2A72"], /* PLUS SIGN ABOVE EQUALS SIGN */
  ["\u2A73", "\u2A73"], /* EQUALS SIGN ABOVE TILDE OPERATOR */
  ["\u2A74", "\u2A74"], /* DOUBLE COLON EQUAL */
  ["\u2A75", "\u2A75"], /* TWO CONSECUTIVE EQUALS SIGNS */
  ["\u2A76", "\u2A76"], /* THREE CONSECUTIVE EQUALS SIGNS */
  ["\u2A77", "\u2A77"], /* EQUALS SIGN WITH TWO DOTS ABOVE AND TWO DOTS BELOW */
  ["\u2A78", "\u2A78"], /* EQUIVALENT WITH FOUR DOTS ABOVE */
  ["\u2A79", "\u2A79"], /* LESS-THAN WITH CIRCLE INSIDE */
  ["\u2A7A", "\u2A7A"], /* GREATER-THAN WITH CIRCLE INSIDE */
  ["\u2A7B", "\u2A7B"], /* LESS-THAN WITH QUESTION MARK ABOVE */
  ["\u2A7C", "\u2A7C"], /* GREATER-THAN WITH QUESTION MARK ABOVE */
  ["\u2A7D", "\u2A7D"], /* LESS-THAN OR SLANTED EQUAL TO */
  ["\u2A7E", "\u2A7E"], /* GREATER-THAN OR SLANTED EQUAL TO */
  ["\u2A7F", "\u2A7F"], /* LESS-THAN OR SLANTED EQUAL TO WITH DOT INSIDE */
  ["\u2A80", "\u2A80"], /* GREATER-THAN OR SLANTED EQUAL TO WITH DOT INSIDE */
  ["\u2A81", "\u2A81"], /* LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE */
  ["\u2A82", "\u2A82"], /* GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE */
  ["\u2A83", "\u2A83"], /* LESS-THAN OR SLANTED EQUAL TO WITH DOT ABOVE RIGHT */
  ["\u2A84", "\u2A84"], /* GREATER-THAN OR SLANTED EQUAL TO WITH DOT ABOVE LEFT */
  ["\u2A85", "\u2A85"], /* LESS-THAN OR APPROXIMATE */
  ["\u2A86", "\u2A86"], /* GREATER-THAN OR APPROXIMATE */
  ["\u2A87", "\u2A87"], /* LESS-THAN AND SINGLE-LINE NOT EQUAL TO */
  ["\u2A88", "\u2A88"], /* GREATER-THAN AND SINGLE-LINE NOT EQUAL TO */
  ["\u2A89", "\u2A89"], /* LESS-THAN AND NOT APPROXIMATE */
  ["\u2A8A", "\u2A8A"], /* GREATER-THAN AND NOT APPROXIMATE */
  ["\u2A8B", "\u2A8B"], /* LESS-THAN ABOVE DOUBLE-LINE EQUAL ABOVE GREATER-THAN */
  ["\u2A8C", "\u2A8C"], /* GREATER-THAN ABOVE DOUBLE-LINE EQUAL ABOVE LESS-THAN */
  ["\u2A8D", "\u2A8D"], /* LESS-THAN ABOVE SIMILAR OR EQUAL */
  ["\u2A8E", "\u2A8E"], /* GREATER-THAN ABOVE SIMILAR OR EQUAL */
  ["\u2A8F", "\u2A8F"], /* LESS-THAN ABOVE SIMILAR ABOVE GREATER-THAN */
  ["\u2A90", "\u2A90"], /* GREATER-THAN ABOVE SIMILAR ABOVE LESS-THAN */
  ["\u2A91", "\u2A91"], /* LESS-THAN ABOVE GREATER-THAN ABOVE DOUBLE-LINE EQUAL */
  ["\u2A92", "\u2A92"], /* GREATER-THAN ABOVE LESS-THAN ABOVE DOUBLE-LINE EQUAL */
  ["\u2A93", "\u2A93"], /* LESS-THAN ABOVE SLANTED EQUAL ABOVE GREATER-THAN ABOVE SLANTED EQUAL */
  ["\u2A94", "\u2A94"], /* GREATER-THAN ABOVE SLANTED EQUAL ABOVE LESS-THAN ABOVE SLANTED EQUAL */
  ["\u2A95", "\u2A95"], /* SLANTED EQUAL TO OR LESS-THAN */
  ["\u2A96", "\u2A96"], /* SLANTED EQUAL TO OR GREATER-THAN */
  ["\u2A97", "\u2A97"], /* SLANTED EQUAL TO OR LESS-THAN WITH DOT INSIDE */
  ["\u2A98", "\u2A98"], /* SLANTED EQUAL TO OR GREATER-THAN WITH DOT INSIDE */
  ["\u2A99", "\u2A99"], /* DOUBLE-LINE EQUAL TO OR LESS-THAN */
  ["\u2A9A", "\u2A9A"], /* DOUBLE-LINE EQUAL TO OR GREATER-THAN */
  ["\u2A9B", "\u2A9B"], /* DOUBLE-LINE SLANTED EQUAL TO OR LESS-THAN */
  ["\u2A9C", "\u2A9C"], /* DOUBLE-LINE SLANTED EQUAL TO OR GREATER-THAN */
  ["\u2A9D", "\u2A9D"], /* SIMILAR OR LESS-THAN */
  ["\u2A9E", "\u2A9E"], /* SIMILAR OR GREATER-THAN */
  ["\u2A9F", "\u2A9F"], /* SIMILAR ABOVE LESS-THAN ABOVE EQUALS SIGN */
  ["\u2AA0", "\u2AA0"], /* SIMILAR ABOVE GREATER-THAN ABOVE EQUALS SIGN */
  ["\u2AA1", "\u2AA1"], /* DOUBLE NESTED LESS-THAN */
  ["\u2AA2", "\u2AA2"], /* DOUBLE NESTED GREATER-THAN */
  ["\u2AA3", "\u2AA3"], /* DOUBLE NESTED LESS-THAN WITH UNDERBAR */
  ["\u2AA4", "\u2AA4"], /* GREATER-THAN OVERLAPPING LESS-THAN */
  ["\u2AA5", "\u2AA5"], /* GREATER-THAN BESIDE LESS-THAN */
  ["\u2AA6", "\u2AA6"], /* LESS-THAN CLOSED BY CURVE */
  ["\u2AA7", "\u2AA7"], /* GREATER-THAN CLOSED BY CURVE */
  ["\u2AA8", "\u2AA8"], /* LESS-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL */
  ["\u2AA9", "\u2AA9"], /* GREATER-THAN CLOSED BY CURVE ABOVE SLANTED EQUAL */
  ["\u2AAA", "\u2AAA"], /* SMALLER THAN */
  ["\u2AAB", "\u2AAB"], /* LARGER THAN */
  ["\u2AAC", "\u2AAC"], /* SMALLER THAN OR EQUAL TO */
  ["\u2AAD", "\u2AAD"], /* LARGER THAN OR EQUAL TO */
  ["\u2AAE", "\u2AAE"], /* EQUALS SIGN WITH BUMPY ABOVE */
  ["\u2AAF", "\u2AAF"], /* PRECEDES ABOVE SINGLE-LINE EQUALS SIGN */
  ["\u2AB0", "\u2AB0"], /* SUCCEEDS ABOVE SINGLE-LINE EQUALS SIGN */
  ["\u2AB1", "\u2AB1"], /* PRECEDES ABOVE SINGLE-LINE NOT EQUAL TO */
  ["\u2AB2", "\u2AB2"], /* SUCCEEDS ABOVE SINGLE-LINE NOT EQUAL TO */
  ["\u2AB3", "\u2AB3"], /* PRECEDES ABOVE EQUALS SIGN */
  ["\u2AB4", "\u2AB4"], /* SUCCEEDS ABOVE EQUALS SIGN */
  ["\u2AB5", "\u2AB5"], /* PRECEDES ABOVE NOT EQUAL TO */
  ["\u2AB6", "\u2AB6"], /* SUCCEEDS ABOVE NOT EQUAL TO */
  ["\u2AB7", "\u2AB7"], /* PRECEDES ABOVE ALMOST EQUAL TO */
  ["\u2AB8", "\u2AB8"], /* SUCCEEDS ABOVE ALMOST EQUAL TO */
  ["\u2AB9", "\u2AB9"], /* PRECEDES ABOVE NOT ALMOST EQUAL TO */
  ["\u2ABA", "\u2ABA"], /* SUCCEEDS ABOVE NOT ALMOST EQUAL TO */
  ["\u2ABB", "\u2ABB"], /* DOUBLE PRECEDES */
  ["\u2ABC", "\u2ABC"], /* DOUBLE SUCCEEDS */
  ["\u2ABD", "\u2ABD"], /* SUBSET WITH DOT */
  ["\u2ABE", "\u2ABE"], /* SUPERSET WITH DOT */
  ["\u2ABF", "\u2ABF"], /* SUBSET WITH PLUS SIGN BELOW */
  ["\u2AC0", "\u2AC0"], /* SUPERSET WITH PLUS SIGN BELOW */
  ["\u2AC1", "\u2AC1"], /* SUBSET WITH MULTIPLICATION SIGN BELOW */
  ["\u2AC2", "\u2AC2"], /* SUPERSET WITH MULTIPLICATION SIGN BELOW */
  ["\u2AC3", "\u2AC3"], /* SUBSET OF OR EQUAL TO WITH DOT ABOVE */
  ["\u2AC4", "\u2AC4"], /* SUPERSET OF OR EQUAL TO WITH DOT ABOVE */
  ["\u2AC5", "\u2AC5"], /* SUBSET OF ABOVE EQUALS SIGN */
  ["\u2AC6", "\u2AC6"], /* SUPERSET OF ABOVE EQUALS SIGN */
  ["\u2AC7", "\u2AC7"], /* SUBSET OF ABOVE TILDE OPERATOR */
  ["\u2AC8", "\u2AC8"], /* SUPERSET OF ABOVE TILDE OPERATOR */
  ["\u2AC9", "\u2AC9"], /* SUBSET OF ABOVE ALMOST EQUAL TO */
  ["\u2ACA", "\u2ACA"], /* SUPERSET OF ABOVE ALMOST EQUAL TO */
  ["\u2ACB", "\u2ACB"], /* SUBSET OF ABOVE NOT EQUAL TO */
  ["\u2ACC", "\u2ACC"], /* SUPERSET OF ABOVE NOT EQUAL TO */
  ["\u2ACD", "\u2ACD"], /* SQUARE LEFT OPEN BOX OPERATOR */
  ["\u2ACE", "\u2ACE"], /* SQUARE RIGHT OPEN BOX OPERATOR */
  ["\u2ACF", "\u2ACF"], /* CLOSED SUBSET */
  ["\u2AD0", "\u2AD0"], /* CLOSED SUPERSET */
  ["\u2AD1", "\u2AD1"], /* CLOSED SUBSET OR EQUAL TO */
  ["\u2AD2", "\u2AD2"], /* CLOSED SUPERSET OR EQUAL TO */
  ["\u2AD3", "\u2AD3"], /* SUBSET ABOVE SUPERSET */
  ["\u2AD4", "\u2AD4"], /* SUPERSET ABOVE SUBSET */
  ["\u2AD5", "\u2AD5"], /* SUBSET ABOVE SUBSET */
  ["\u2AD6", "\u2AD6"], /* SUPERSET ABOVE SUPERSET */
  ["\u2AD7", "\u2AD7"], /* SUPERSET BESIDE SUBSET */
  ["\u2AD8", "\u2AD8"], /* SUPERSET BESIDE AND JOINED BY DASH WITH SUBSET */
  ["\u2AD9", "\u2AD9"], /* ELEMENT OF OPENING DOWNWARDS */
  ["\u2ADA", "\u2ADA"], /* PITCHFORK WITH TEE TOP */
  ["\u2ADB", "\u2ADB"], /* TRANSVERSAL INTERSECTION */
  ["\u2ADC", "\u2ADC"], /* FORKING */
  ["\u2ADD", "\u2ADD"], /* NONFORKING */
  ["\u2ADE", "\u2ADE"], /* SHORT LEFT TACK */
  ["\u2ADF", "\u2ADF"], /* SHORT DOWN TACK */
  ["\u2AE0", "\u2AE0"], /* SHORT UP TACK */
  ["\u2AE1", "\u2AE1"], /* PERPENDICULAR WITH S */
  ["\u2AE2", "\u2AE2"], /* VERTICAL BAR TRIPLE RIGHT TURNSTILE */
  ["\u2AE3", "\u2AE3"], /* DOUBLE VERTICAL BAR LEFT TURNSTILE */
  ["\u2AE4", "\u2AE4"], /* VERTICAL BAR DOUBLE LEFT TURNSTILE */
  ["\u2AE5", "\u2AE5"], /* DOUBLE VERTICAL BAR DOUBLE LEFT TURNSTILE */
  ["\u2AE6", "\u2AE6"], /* LONG DASH FROM LEFT MEMBER OF DOUBLE VERTICAL */
  ["\u2AE7", "\u2AE7"], /* SHORT DOWN TACK WITH OVERBAR */
  ["\u2AE8", "\u2AE8"], /* SHORT UP TACK WITH UNDERBAR */
  ["\u2AE9", "\u2AE9"], /* SHORT UP TACK ABOVE SHORT DOWN TACK */
  ["\u2AEA", "\u2AEA"], /* DOUBLE DOWN TACK */
  ["\u2AEB", "\u2AEB"], /* DOUBLE UP TACK */
  ["\u2AEC", "\u2AEC"], /* DOUBLE STROKE NOT SIGN */
  ["\u2AED", "\u2AED"], /* REVERSED DOUBLE STROKE NOT SIGN */
  ["\u2AEE", "\u2AEE"], /* DOES NOT DIVIDE WITH REVERSED NEGATION SLASH */
  ["\u2AEF", "\u2AEF"], /* VERTICAL LINE WITH CIRCLE ABOVE */
  ["\u2AF0", "\u2AF0"], /* VERTICAL LINE WITH CIRCLE BELOW */
  ["\u2AF1", "\u2AF1"], /* DOWN TACK WITH CIRCLE BELOW */
  ["\u2AF2", "\u2AF2"], /* PARALLEL WITH HORIZONTAL STROKE */
  ["\u2AF3", "\u2AF3"], /* PARALLEL WITH TILDE OPERATOR */
  ["\u2AF4", "\u2AF4"], /* TRIPLE VERTICAL BAR BINARY RELATION */
  ["\u2AF5", "\u2AF5"], /* TRIPLE VERTICAL BAR WITH HORIZONTAL STROKE */
  ["\u2AF6", "\u2AF6"], /* TRIPLE COLON OPERATOR */
  ["\u2AF7", "\u2AF7"], /* TRIPLE NESTED LESS-THAN */
  ["\u2AF8", "\u2AF8"], /* TRIPLE NESTED GREATER-THAN */
  ["\u2AF9", "\u2AF9"], /* DOUBLE-LINE SLANTED LESS-THAN OR EQUAL TO */
  ["\u2AFA", "\u2AFA"], /* DOUBLE-LINE SLANTED GREATER-THAN OR EQUAL TO */
  ["\u2AFB", "\u2AFB"], /* TRIPLE SOLIDUS BINARY RELATION */
  ["\u2AFC", "\u2AFC"], /* LARGE TRIPLE VERTICAL BAR OPERATOR */
  ["\u2AFD", "\u2AFD"], /* DOUBLE SOLIDUS OPERATOR */
  ["\u2AFE", "\u2AFE"], /* WHITE VERTICAL BAR */
  ["\u2AFF", "\u2AFF"], /* N-ARY WHITE VERTICAL BAR */
  ["\u2B00", "\u2B00"], /* NORTH EAST WHITE ARROW */
  ["\u2B01", "\u2B01"], /* NORTH WEST WHITE ARROW */
  ["\u2B02", "\u2B02"], /* SOUTH EAST WHITE ARROW */
  ["\u2B03", "\u2B03"], /* SOUTH WEST WHITE ARROW */
  ["\u2B04", "\u2B04"], /* LEFT RIGHT WHITE ARROW */
  ["\u2B05", "\u2B05"], /* LEFTWARDS BLACK ARROW */
  ["\u2B06", "\u2B06"], /* UPWARDS BLACK ARROW */
  ["\u2B07", "\u2B07"], /* DOWNWARDS BLACK ARROW */
  ["\u2B08", "\u2B08"], /* NORTH EAST BLACK ARROW */
  ["\u2B09", "\u2B09"], /* NORTH WEST BLACK ARROW */
  ["\u2B0A", "\u2B0A"], /* SOUTH EAST BLACK ARROW */
  ["\u2B0B", "\u2B0B"], /* SOUTH WEST BLACK ARROW */
  ["\u2B0C", "\u2B0C"], /* LEFT RIGHT BLACK ARROW */
  ["\u2B0D", "\u2B0D"], /* UP DOWN BLACK ARROW */
  ["\u2B0E", "\u2B0E"], /* RIGHTWARDS ARROW WITH TIP DOWNWARDS */
  ["\u2B0F", "\u2B0F"], /* RIGHTWARDS ARROW WITH TIP UPWARDS */
  ["\u2B10", "\u2B10"], /* LEFTWARDS ARROW WITH TIP DOWNWARDS */
  ["\u2B11", "\u2B11"], /* LEFTWARDS ARROW WITH TIP UPWARDS */
  ["\u2B12", "\u2B12"], /* SQUARE WITH TOP HALF BLACK */
  ["\u2B13", "\u2B13"], /* SQUARE WITH BOTTOM HALF BLACK */
  ["\u2B14", "\u2B14"], /* SQUARE WITH UPPER RIGHT DIAGONAL HALF BLACK */
  ["\u2B15", "\u2B15"], /* SQUARE WITH LOWER LEFT DIAGONAL HALF BLACK */
  ["\u2B16", "\u2B16"], /* DIAMOND WITH LEFT HALF BLACK */
  ["\u2B17", "\u2B17"], /* DIAMOND WITH RIGHT HALF BLACK */
  ["\u2B18", "\u2B18"], /* DIAMOND WITH TOP HALF BLACK */
  ["\u2B19", "\u2B19"], /* DIAMOND WITH BOTTOM HALF BLACK */
  ["\u2B1A", "\u2B1A"], /* DOTTED SQUARE */
  ["\u2B1B", "\u2B1B"], /* BLACK LARGE SQUARE */
  ["\u2B1C", "\u2B1C"], /* WHITE LARGE SQUARE */
  ["\u2B1D", "\u2B1D"], /* BLACK VERY SMALL SQUARE */
  ["\u2B1E", "\u2B1E"], /* WHITE VERY SMALL SQUARE */
  ["\u2B1F", "\u2B1F"], /* BLACK PENTAGON */
  ["\u2B20", "\u2B20"], /* WHITE PENTAGON */
  ["\u2B21", "\u2B21"], /* WHITE HEXAGON */
  ["\u2B22", "\u2B22"], /* BLACK HEXAGON */
  ["\u2B23", "\u2B23"], /* HORIZONTAL BLACK HEXAGON */
  ["\u2B24", "\u2B24"], /* BLACK LARGE CIRCLE */
  ["\u2B25", "\u2B25"], /* BLACK MEDIUM DIAMOND */
  ["\u2B26", "\u2B26"], /* WHITE MEDIUM DIAMOND */
  ["\u2B27", "\u2B27"], /* BLACK MEDIUM LOZENGE */
  ["\u2B28", "\u2B28"], /* WHITE MEDIUM LOZENGE */
  ["\u2B29", "\u2B29"], /* BLACK SMALL DIAMOND */
  ["\u2B2A", "\u2B2A"], /* BLACK SMALL LOZENGE */
  ["\u2B2B", "\u2B2B"], /* WHITE SMALL LOZENGE */
  ["\u2B2C", "\u2B2C"], /* BLACK HORIZONTAL ELLIPSE */
  ["\u2B2D", "\u2B2D"], /* WHITE HORIZONTAL ELLIPSE */
  ["\u2B2E", "\u2B2E"], /* BLACK VERTICAL ELLIPSE */
  ["\u2B2F", "\u2B2F"], /* WHITE VERTICAL ELLIPSE */
  ["\u2B30", "\u2B30"], /* LEFT ARROW WITH SMALL CIRCLE */
  ["\u2B31", "\u2B31"], /* THREE LEFTWARDS ARROWS */
  ["\u2B32", "\u2B32"], /* LEFT ARROW WITH CIRCLED PLUS */
  ["\u2B33", "\u2B33"], /* LONG LEFTWARDS SQUIGGLE ARROW */
  ["\u2B34", "\u2B34"], /* LEFTWARDS TWO-HEADED ARROW WITH VERTICAL STROKE */
  ["\u2B35", "\u2B35"], /* LEFTWARDS TWO-HEADED ARROW WITH DOUBLE VERTICAL STROKE */
  ["\u2B36", "\u2B36"], /* LEFTWARDS TWO-HEADED ARROW FROM BAR */
  ["\u2B37", "\u2B37"], /* LEFTWARDS TWO-HEADED TRIPLE DASH ARROW */
  ["\u2B38", "\u2B38"], /* LEFTWARDS ARROW WITH DOTTED STEM */
  ["\u2B39", "\u2B39"], /* LEFTWARDS ARROW WITH TAIL WITH VERTICAL STROKE */
  ["\u2B3A", "\u2B3A"], /* LEFTWARDS ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE */
  ["\u2B3B", "\u2B3B"], /* LEFTWARDS TWO-HEADED ARROW WITH TAIL */
  ["\u2B3C", "\u2B3C"], /* LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH VERTICAL STROKE */
  ["\u2B3D", "\u2B3D"], /* LEFTWARDS TWO-HEADED ARROW WITH TAIL WITH DOUBLE VERTICAL STROKE */
  ["\u2B3E", "\u2B3E"], /* LEFTWARDS ARROW THROUGH X */
  ["\u2B3F", "\u2B3F"], /* WAVE ARROW POINTING DIRECTLY LEFT */
  ["\u2B40", "\u2B40"], /* EQUALS SIGN ABOVE LEFTWARDS ARROW */
  ["\u2B41", "\u2B41"], /* REVERSE TILDE OPERATOR ABOVE LEFTWARDS ARROW */
  ["\u2B42", "\u2B42"], /* LEFTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO */
  ["\u2B43", "\u2B43"], /* RIGHTWARDS ARROW THROUGH GREATER-THAN */
  ["\u2B44", "\u2B44"], /* RIGHTWARDS ARROW THROUGH SUPERSET */
  ["\u2B45", "\u2B45"], /* LEFTWARDS QUADRUPLE ARROW */
  ["\u2B46", "\u2B46"], /* RIGHTWARDS QUADRUPLE ARROW */
  ["\u2B47", "\u2B47"], /* REVERSE TILDE OPERATOR ABOVE RIGHTWARDS ARROW */
  ["\u2B48", "\u2B48"], /* RIGHTWARDS ARROW ABOVE REVERSE ALMOST EQUAL TO */
  ["\u2B49", "\u2B49"], /* TILDE OPERATOR ABOVE LEFTWARDS ARROW */
  ["\u2B4A", "\u2B4A"], /* LEFTWARDS ARROW ABOVE ALMOST EQUAL TO */
  ["\u2B4B", "\u2B4B"], /* LEFTWARDS ARROW ABOVE REVERSE TILDE OPERATOR */
  ["\u2B4C", "\u2B4C"], /* RIGHTWARDS ARROW ABOVE REVERSE TILDE OPERATOR */
  ["\u2B4D", "\u2B4D"], /* DOWNWARDS TRIANGLE-HEADED ZIGZAG ARROW */
  ["\u2B4E", "\u2B4E"], /* SHORT SLANTED NORTH ARROW */
  ["\u2B4F", "\u2B4F"], /* SHORT BACKSLANTED SOUTH ARROW */
  ["\u2B50", "\u2B50"], /* WHITE MEDIUM STAR */
  ["\u2B51", "\u2B51"], /* BLACK SMALL STAR */
  ["\u2B52", "\u2B52"], /* WHITE SMALL STAR */
  ["\u2B53", "\u2B53"], /* BLACK RIGHT-POINTING PENTAGON */
  ["\u2B54", "\u2B54"], /* WHITE RIGHT-POINTING PENTAGON */
  ["\u2B55", "\u2B55"], /* HEAVY LARGE CIRCLE */
  ["\u2B56", "\u2B56"], /* HEAVY OVAL WITH OVAL INSIDE */
  ["\u2B57", "\u2B57"], /* HEAVY CIRCLE WITH CIRCLE INSIDE */
  ["\u2B58", "\u2B58"], /* HEAVY CIRCLE */
  ["\u2B59", "\u2B59"], /* HEAVY CIRCLED SALTIRE */
  ["\u2B5A", "\u2B5A"], /* SLANTED NORTH ARROW WITH HOOKED HEAD */
  ["\u2B5B", "\u2B5B"], /* BACKSLANTED SOUTH ARROW WITH HOOKED TAIL */
  ["\u2B5C", "\u2B5C"], /* SLANTED NORTH ARROW WITH HORIZONTAL TAIL */
  ["\u2B5D", "\u2B5D"], /* BACKSLANTED SOUTH ARROW WITH HORIZONTAL TAIL */
  ["\u2B5E", "\u2B5E"], /* BENT ARROW POINTING DOWNWARDS THEN NORTH EAST */
  ["\u2B5F", "\u2B5F"], /* SHORT BENT ARROW POINTING DOWNWARDS THEN NORTH EAST */
  ["\u2B60", "\u2B60"], /* LEFTWARDS TRIANGLE-HEADED ARROW */
  ["\u2B61", "\u2B61"], /* UPWARDS TRIANGLE-HEADED ARROW */
  ["\u2B62", "\u2B62"], /* RIGHTWARDS TRIANGLE-HEADED ARROW */
  ["\u2B63", "\u2B63"], /* DOWNWARDS TRIANGLE-HEADED ARROW */
  ["\u2B64", "\u2B64"], /* LEFT RIGHT TRIANGLE-HEADED ARROW */
  ["\u2B65", "\u2B65"], /* UP DOWN TRIANGLE-HEADED ARROW */
  ["\u2B66", "\u2B66"], /* NORTH WEST TRIANGLE-HEADED ARROW */
  ["\u2B67", "\u2B67"], /* NORTH EAST TRIANGLE-HEADED ARROW */
  ["\u2B68", "\u2B68"], /* SOUTH EAST TRIANGLE-HEADED ARROW */
  ["\u2B69", "\u2B69"], /* SOUTH WEST TRIANGLE-HEADED ARROW */
  ["\u2B6A", "\u2B6A"], /* LEFTWARDS TRIANGLE-HEADED DASHED ARROW */
  ["\u2B6B", "\u2B6B"], /* UPWARDS TRIANGLE-HEADED DASHED ARROW */
  ["\u2B6C", "\u2B6C"], /* RIGHTWARDS TRIANGLE-HEADED DASHED ARROW */
  ["\u2B6D", "\u2B6D"], /* DOWNWARDS TRIANGLE-HEADED DASHED ARROW */
  ["\u2B6E", "\u2B6E"], /* CLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW */
  ["\u2B6F", "\u2B6F"], /* ANTICLOCKWISE TRIANGLE-HEADED OPEN CIRCLE ARROW */
  ["\u2B70", "\u2B70"], /* LEFTWARDS TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B71", "\u2B71"], /* UPWARDS TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B72", "\u2B72"], /* RIGHTWARDS TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B73", "\u2B73"], /* DOWNWARDS TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B74", "\u2B74"],
  ["\u2B75", "\u2B75"],
  ["\u2B76", "\u2B76"], /* NORTH WEST TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B77", "\u2B77"], /* NORTH EAST TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B78", "\u2B78"], /* SOUTH EAST TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B79", "\u2B79"], /* SOUTH WEST TRIANGLE-HEADED ARROW TO BAR */
  ["\u2B7A", "\u2B7A"], /* LEFTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE */
  ["\u2B7B", "\u2B7B"], /* UPWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE */
  ["\u2B7C", "\u2B7C"], /* RIGHTWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE */
  ["\u2B7D", "\u2B7D"], /* DOWNWARDS TRIANGLE-HEADED ARROW WITH DOUBLE HORIZONTAL STROKE */
  ["\u2B7E", "\u2B7E"], /* HORIZONTAL TAB KEY */
  ["\u2B7F", "\u2B7F"], /* VERTICAL TAB KEY */
  ["\u2B80", "\u2B80"], /* LEFTWARDS TRIANGLE-HEADED ARROW OVER RIGHTWARDS TRIANGLE-HEADED ARROW */
  ["\u2B81", "\u2B81"], /* UPWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF DOWNWARDS TRIANGLE-HEADED ARROW */
  ["\u2B82", "\u2B82"], /* RIGHTWARDS TRIANGLE-HEADED ARROW OVER LEFTWARDS TRIANGLE-HEADED ARROW */
  ["\u2B83", "\u2B83"], /* DOWNWARDS TRIANGLE-HEADED ARROW LEFTWARDS OF UPWARDS TRIANGLE-HEADED ARROW */
  ["\u2B84", "\u2B84"], /* LEFTWARDS TRIANGLE-HEADED PAIRED ARROWS */
  ["\u2B85", "\u2B85"], /* UPWARDS TRIANGLE-HEADED PAIRED ARROWS */
  ["\u2B86", "\u2B86"], /* RIGHTWARDS TRIANGLE-HEADED PAIRED ARROWS */
  ["\u2B87", "\u2B87"], /* DOWNWARDS TRIANGLE-HEADED PAIRED ARROWS */
  ["\u2B88", "\u2B88"], /* LEFTWARDS BLACK CIRCLED WHITE ARROW */
  ["\u2B89", "\u2B89"], /* UPWARDS BLACK CIRCLED WHITE ARROW */
  ["\u2B8A", "\u2B8A"], /* RIGHTWARDS BLACK CIRCLED WHITE ARROW */
  ["\u2B8B", "\u2B8B"], /* DOWNWARDS BLACK CIRCLED WHITE ARROW */
  ["\u2B8C", "\u2B8C"], /* ANTICLOCKWISE TRIANGLE-HEADED RIGHT U-SHAPED ARROW */
  ["\u2B8D", "\u2B8D"], /* ANTICLOCKWISE TRIANGLE-HEADED BOTTOM U-SHAPED ARROW */
  ["\u2B8E", "\u2B8E"], /* ANTICLOCKWISE TRIANGLE-HEADED LEFT U-SHAPED ARROW */
  ["\u2B8F", "\u2B8F"], /* ANTICLOCKWISE TRIANGLE-HEADED TOP U-SHAPED ARROW */
  ["\u2B90", "\u2B90"], /* RETURN LEFT */
  ["\u2B91", "\u2B91"], /* RETURN RIGHT */
  ["\u2B92", "\u2B92"], /* NEWLINE LEFT */
  ["\u2B93", "\u2B93"], /* NEWLINE RIGHT */
  ["\u2B94", "\u2B94"], /* FOUR CORNER ARROWS CIRCLING ANTICLOCKWISE */
  ["\u2B95", "\u2B95"], /* RIGHTWARDS BLACK ARROW */
  ["\u2B96", "\u2B96"],
  ["\u2B97", "\u2B97"],
  ["\u2B98", "\u2B98"], /* THREE-D TOP-LIGHTED LEFTWARDS EQUILATERAL ARROWHEAD */
  ["\u2B99", "\u2B99"], /* THREE-D RIGHT-LIGHTED UPWARDS EQUILATERAL ARROWHEAD */
  ["\u2B9A", "\u2B9A"], /* THREE-D TOP-LIGHTED RIGHTWARDS EQUILATERAL ARROWHEAD */
  ["\u2B9B", "\u2B9B"], /* THREE-D LEFT-LIGHTED DOWNWARDS EQUILATERAL ARROWHEAD */
  ["\u2B9C", "\u2B9C"], /* BLACK LEFTWARDS EQUILATERAL ARROWHEAD */
  ["\u2B9D", "\u2B9D"], /* BLACK UPWARDS EQUILATERAL ARROWHEAD */
  ["\u2B9E", "\u2B9E"], /* BLACK RIGHTWARDS EQUILATERAL ARROWHEAD */
  ["\u2B9F", "\u2B9F"], /* BLACK DOWNWARDS EQUILATERAL ARROWHEAD */
  ["\u2BA0", "\u2BA0"], /* DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS */
  ["\u2BA1", "\u2BA1"], /* DOWNWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS */
  ["\u2BA2", "\u2BA2"], /* UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP LEFTWARDS */
  ["\u2BA3", "\u2BA3"], /* UPWARDS TRIANGLE-HEADED ARROW WITH LONG TIP RIGHTWARDS */
  ["\u2BA4", "\u2BA4"], /* LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS */
  ["\u2BA5", "\u2BA5"], /* RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP UPWARDS */
  ["\u2BA6", "\u2BA6"], /* LEFTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS */
  ["\u2BA7", "\u2BA7"], /* RIGHTWARDS TRIANGLE-HEADED ARROW WITH LONG TIP DOWNWARDS */
  ["\u2BA8", "\u2BA8"], /* BLACK CURVED DOWNWARDS AND LEFTWARDS ARROW */
  ["\u2BA9", "\u2BA9"], /* BLACK CURVED DOWNWARDS AND RIGHTWARDS ARROW */
  ["\u2BAA", "\u2BAA"], /* BLACK CURVED UPWARDS AND LEFTWARDS ARROW */
  ["\u2BAB", "\u2BAB"], /* BLACK CURVED UPWARDS AND RIGHTWARDS ARROW */
  ["\u2BAC", "\u2BAC"], /* BLACK CURVED LEFTWARDS AND UPWARDS ARROW */
  ["\u2BAD", "\u2BAD"], /* BLACK CURVED RIGHTWARDS AND UPWARDS ARROW */
  ["\u2BAE", "\u2BAE"], /* BLACK CURVED LEFTWARDS AND DOWNWARDS ARROW */
  ["\u2BAF", "\u2BAF"], /* BLACK CURVED RIGHTWARDS AND DOWNWARDS ARROW */
  ["\u2BB0", "\u2BB0"], /* RIBBON ARROW DOWN LEFT */
  ["\u2BB1", "\u2BB1"], /* RIBBON ARROW DOWN RIGHT */
  ["\u2BB2", "\u2BB2"], /* RIBBON ARROW UP LEFT */
  ["\u2BB3", "\u2BB3"], /* RIBBON ARROW UP RIGHT */
  ["\u2BB4", "\u2BB4"], /* RIBBON ARROW LEFT UP */
  ["\u2BB5", "\u2BB5"], /* RIBBON ARROW RIGHT UP */
  ["\u2BB6", "\u2BB6"], /* RIBBON ARROW LEFT DOWN */
  ["\u2BB7", "\u2BB7"], /* RIBBON ARROW RIGHT DOWN */
  ["\u2BB8", "\u2BB8"], /* UPWARDS WHITE ARROW FROM BAR WITH HORIZONTAL BAR */
  ["\u2BB9", "\u2BB9"], /* UP ARROWHEAD IN A RECTANGLE BOX */
  ["\u2BBA", "\u2BBA"], /* OVERLAPPING WHITE SQUARES */
  ["\u2BBB", "\u2BBB"], /* OVERLAPPING WHITE AND BLACK SQUARES */
  ["\u2BBC", "\u2BBC"], /* OVERLAPPING BLACK SQUARES */
  ["\u2BBD", "\u2BBD"], /* BALLOT BOX WITH LIGHT X */
  ["\u2BBE", "\u2BBE"], /* CIRCLED X */
  ["\u2BBF", "\u2BBF"], /* CIRCLED BOLD X */
  ["\u2BC0", "\u2BC0"], /* BLACK SQUARE CENTRED */
  ["\u2BC1", "\u2BC1"], /* BLACK DIAMOND CENTRED */
  ["\u2BC2", "\u2BC2"], /* TURNED BLACK PENTAGON */
  ["\u2BC3", "\u2BC3"], /* HORIZONTAL BLACK OCTAGON */
  ["\u2BC4", "\u2BC4"], /* BLACK OCTAGON */
  ["\u2BC5", "\u2BC5"], /* BLACK MEDIUM UP-POINTING TRIANGLE CENTRED */
  ["\u2BC6", "\u2BC6"], /* BLACK MEDIUM DOWN-POINTING TRIANGLE CENTRED */
  ["\u2BC7", "\u2BC7"], /* BLACK MEDIUM LEFT-POINTING TRIANGLE CENTRED */
  ["\u2BC8", "\u2BC8"], /* BLACK MEDIUM RIGHT-POINTING TRIANGLE CENTRED */
  ["\u2BC9", "\u2BC9"], /* NEPTUNE FORM TWO */
  ["\u2BCA", "\u2BCA"], /* TOP HALF BLACK CIRCLE */
  ["\u2BCB", "\u2BCB"], /* BOTTOM HALF BLACK CIRCLE */
  ["\u2BCC", "\u2BCC"], /* LIGHT FOUR POINTED BLACK CUSP */
  ["\u2BCD", "\u2BCD"], /* ROTATED LIGHT FOUR POINTED BLACK CUSP */
  ["\u2BCE", "\u2BCE"], /* WHITE FOUR POINTED CUSP */
  ["\u2BCF", "\u2BCF"], /* ROTATED WHITE FOUR POINTED CUSP */
  ["\u2BD0", "\u2BD0"], /* SQUARE POSITION INDICATOR */
  ["\u2BD1", "\u2BD1"], /* UNCERTAINTY SIGN */
  ["\u2BD2", "\u2BD2"], /* GROUP MARK */
  ["\u2BD3", "\u2BD3"], /* PLUTO FORM TWO */
  ["\u2BD4", "\u2BD4"], /* PLUTO FORM THREE */
  ["\u2BD5", "\u2BD5"], /* PLUTO FORM FOUR */
  ["\u2BD6", "\u2BD6"], /* PLUTO FORM FIVE */
  ["\u2BD7", "\u2BD7"], /* TRANSPLUTO */
  ["\u2BD8", "\u2BD8"], /* PROSERPINA */
  ["\u2BD9", "\u2BD9"], /* ASTRAEA */
  ["\u2BDA", "\u2BDA"], /* HYGIEA */
  ["\u2BDB", "\u2BDB"], /* PHOLUS */
  ["\u2BDC", "\u2BDC"], /* NESSUS */
  ["\u2BDD", "\u2BDD"], /* WHITE MOON SELENA */
  ["\u2BDE", "\u2BDE"], /* BLACK DIAMOND ON CROSS */
  ["\u2BDF", "\u2BDF"], /* TRUE LIGHT MOON ARTA */
  ["\u2BE0", "\u2BE0"], /* CUPIDO */
  ["\u2BE1", "\u2BE1"], /* HADES */
  ["\u2BE2", "\u2BE2"], /* ZEUS */
  ["\u2BE3", "\u2BE3"], /* KRONOS */
  ["\u2BE4", "\u2BE4"], /* APOLLON */
  ["\u2BE5", "\u2BE5"], /* ADMETOS */
  ["\u2BE6", "\u2BE6"], /* VULCANUS */
  ["\u2BE7", "\u2BE7"], /* POSEIDON */
  ["\u2BE8", "\u2BE8"], /* LEFT HALF BLACK STAR */
  ["\u2BE9", "\u2BE9"], /* RIGHT HALF BLACK STAR */
  ["\u2BEA", "\u2BEA"], /* STAR WITH LEFT HALF BLACK */
  ["\u2BEB", "\u2BEB"], /* STAR WITH RIGHT HALF BLACK */
  ["\u2BEC", "\u2BEC"], /* LEFTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS */
  ["\u2BED", "\u2BED"], /* UPWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS */
  ["\u2BEE", "\u2BEE"], /* RIGHTWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS */
  ["\u2BEF", "\u2BEF"], /* DOWNWARDS TWO-HEADED ARROW WITH TRIANGLE ARROWHEADS */
  ["\u2BF0", "\u2BF0"], /* ERIS FORM ONE */
  ["\u2BF1", "\u2BF1"], /* ERIS FORM TWO */
  ["\u2BF2", "\u2BF2"], /* SEDNA */
  ["\u2BF3", "\u2BF3"], /* RUSSIAN ASTROLOGICAL SYMBOL VIGINTILE */
  ["\u2BF4", "\u2BF4"], /* RUSSIAN ASTROLOGICAL SYMBOL NOVILE */
  ["\u2BF5", "\u2BF5"], /* RUSSIAN ASTROLOGICAL SYMBOL QUINTILE */
  ["\u2BF6", "\u2BF6"], /* RUSSIAN ASTROLOGICAL SYMBOL BINOVILE */
  ["\u2BF7", "\u2BF7"], /* RUSSIAN ASTROLOGICAL SYMBOL SENTAGON */
  ["\u2BF8", "\u2BF8"], /* RUSSIAN ASTROLOGICAL SYMBOL TREDECILE */
  ["\u2BF9", "\u2BF9"], /* EQUALS SIGN WITH INFINITY BELOW */
  ["\u2BFA", "\u2BFA"], /* UNITED SYMBOL */
  ["\u2BFB", "\u2BFB"], /* SEPARATED SYMBOL */
  ["\u2BFC", "\u2BFC"], /* DOUBLED SYMBOL */
  ["\u2BFD", "\u2BFD"], /* PASSED SYMBOL */
  ["\u2BFE", "\u2BFE"], /* REVERSED RIGHT ANGLE */
  ["\u2BFF", "\u2BFF"], /* HELLSCHREIBER PAUSE SYMBOL */
  ["\u2C00", "\u2C30"], /* GLAGOLITIC CAPITAL LETTER AZU */
  ["\u2C01", "\u2C31"], /* GLAGOLITIC CAPITAL LETTER BUKY */
  ["\u2C02", "\u2C32"], /* GLAGOLITIC CAPITAL LETTER VEDE */
  ["\u2C03", "\u2C33"], /* GLAGOLITIC CAPITAL LETTER GLAGOLI */
  ["\u2C04", "\u2C34"], /* GLAGOLITIC CAPITAL LETTER DOBRO */
  ["\u2C05", "\u2C35"], /* GLAGOLITIC CAPITAL LETTER YESTU */
  ["\u2C06", "\u2C36"], /* GLAGOLITIC CAPITAL LETTER ZHIVETE */
  ["\u2C07", "\u2C37"], /* GLAGOLITIC CAPITAL LETTER DZELO */
  ["\u2C08", "\u2C38"], /* GLAGOLITIC CAPITAL LETTER ZEMLJA */
  ["\u2C09", "\u2C39"], /* GLAGOLITIC CAPITAL LETTER IZHE */
  ["\u2C0A", "\u2C3A"], /* GLAGOLITIC CAPITAL LETTER INITIAL IZHE */
  ["\u2C0B", "\u2C3B"], /* GLAGOLITIC CAPITAL LETTER I */
  ["\u2C0C", "\u2C3C"], /* GLAGOLITIC CAPITAL LETTER DJERVI */
  ["\u2C0D", "\u2C3D"], /* GLAGOLITIC CAPITAL LETTER KAKO */
  ["\u2C0E", "\u2C3E"], /* GLAGOLITIC CAPITAL LETTER LJUDIJE */
  ["\u2C0F", "\u2C3F"], /* GLAGOLITIC CAPITAL LETTER MYSLITE */
  ["\u2C10", "\u2C40"], /* GLAGOLITIC CAPITAL LETTER NASHI */
  ["\u2C11", "\u2C41"], /* GLAGOLITIC CAPITAL LETTER ONU */
  ["\u2C12", "\u2C42"], /* GLAGOLITIC CAPITAL LETTER POKOJI */
  ["\u2C13", "\u2C43"], /* GLAGOLITIC CAPITAL LETTER RITSI */
  ["\u2C14", "\u2C44"], /* GLAGOLITIC CAPITAL LETTER SLOVO */
  ["\u2C15", "\u2C45"], /* GLAGOLITIC CAPITAL LETTER TVRIDO */
  ["\u2C16", "\u2C46"], /* GLAGOLITIC CAPITAL LETTER UKU */
  ["\u2C17", "\u2C47"], /* GLAGOLITIC CAPITAL LETTER FRITU */
  ["\u2C18", "\u2C48"], /* GLAGOLITIC CAPITAL LETTER HERU */
  ["\u2C19", "\u2C49"], /* GLAGOLITIC CAPITAL LETTER OTU */
  ["\u2C1A", "\u2C4A"], /* GLAGOLITIC CAPITAL LETTER PE */
  ["\u2C1B", "\u2C4B"], /* GLAGOLITIC CAPITAL LETTER SHTA */
  ["\u2C1C", "\u2C4C"], /* GLAGOLITIC CAPITAL LETTER TSI */
  ["\u2C1D", "\u2C4D"], /* GLAGOLITIC CAPITAL LETTER CHRIVI */
  ["\u2C1E", "\u2C4E"], /* GLAGOLITIC CAPITAL LETTER SHA */
  ["\u2C1F", "\u2C4F"], /* GLAGOLITIC CAPITAL LETTER YERU */
  ["\u2C20", "\u2C50"], /* GLAGOLITIC CAPITAL LETTER YERI */
  ["\u2C21", "\u2C51"], /* GLAGOLITIC CAPITAL LETTER YATI */
  ["\u2C22", "\u2C52"], /* GLAGOLITIC CAPITAL LETTER SPIDERY HA */
  ["\u2C23", "\u2C53"], /* GLAGOLITIC CAPITAL LETTER YU */
  ["\u2C24", "\u2C54"], /* GLAGOLITIC CAPITAL LETTER SMALL YUS */
  ["\u2C25", "\u2C55"], /* GLAGOLITIC CAPITAL LETTER SMALL YUS WITH TAIL */
  ["\u2C26", "\u2C56"], /* GLAGOLITIC CAPITAL LETTER YO */
  ["\u2C27", "\u2C57"], /* GLAGOLITIC CAPITAL LETTER IOTATED SMALL YUS */
  ["\u2C28", "\u2C58"], /* GLAGOLITIC CAPITAL LETTER BIG YUS */
  ["\u2C29", "\u2C59"], /* GLAGOLITIC CAPITAL LETTER IOTATED BIG YUS */
  ["\u2C2A", "\u2C5A"], /* GLAGOLITIC CAPITAL LETTER FITA */
  ["\u2C2B", "\u2C5B"], /* GLAGOLITIC CAPITAL LETTER IZHITSA */
  ["\u2C2C", "\u2C5C"], /* GLAGOLITIC CAPITAL LETTER SHTAPIC */
  ["\u2C2D", "\u2C5D"], /* GLAGOLITIC CAPITAL LETTER TROKUTASTI A */
  ["\u2C2E", "\u2C5E"], /* GLAGOLITIC CAPITAL LETTER LATINATE MYSLITE */
  ["\u2C2F", "\u2C2F"],
  ["\u2C00", "\u2C30"], /* GLAGOLITIC SMALL LETTER AZU */
  ["\u2C01", "\u2C31"], /* GLAGOLITIC SMALL LETTER BUKY */
  ["\u2C02", "\u2C32"], /* GLAGOLITIC SMALL LETTER VEDE */
  ["\u2C03", "\u2C33"], /* GLAGOLITIC SMALL LETTER GLAGOLI */
  ["\u2C04", "\u2C34"], /* GLAGOLITIC SMALL LETTER DOBRO */
  ["\u2C05", "\u2C35"], /* GLAGOLITIC SMALL LETTER YESTU */
  ["\u2C06", "\u2C36"], /* GLAGOLITIC SMALL LETTER ZHIVETE */
  ["\u2C07", "\u2C37"], /* GLAGOLITIC SMALL LETTER DZELO */
  ["\u2C08", "\u2C38"], /* GLAGOLITIC SMALL LETTER ZEMLJA */
  ["\u2C09", "\u2C39"], /* GLAGOLITIC SMALL LETTER IZHE */
  ["\u2C0A", "\u2C3A"], /* GLAGOLITIC SMALL LETTER INITIAL IZHE */
  ["\u2C0B", "\u2C3B"], /* GLAGOLITIC SMALL LETTER I */
  ["\u2C0C", "\u2C3C"], /* GLAGOLITIC SMALL LETTER DJERVI */
  ["\u2C0D", "\u2C3D"], /* GLAGOLITIC SMALL LETTER KAKO */
  ["\u2C0E", "\u2C3E"], /* GLAGOLITIC SMALL LETTER LJUDIJE */
  ["\u2C0F", "\u2C3F"], /* GLAGOLITIC SMALL LETTER MYSLITE */
  ["\u2C10", "\u2C40"], /* GLAGOLITIC SMALL LETTER NASHI */
  ["\u2C11", "\u2C41"], /* GLAGOLITIC SMALL LETTER ONU */
  ["\u2C12", "\u2C42"], /* GLAGOLITIC SMALL LETTER POKOJI */
  ["\u2C13", "\u2C43"], /* GLAGOLITIC SMALL LETTER RITSI */
  ["\u2C14", "\u2C44"], /* GLAGOLITIC SMALL LETTER SLOVO */
  ["\u2C15", "\u2C45"], /* GLAGOLITIC SMALL LETTER TVRIDO */
  ["\u2C16", "\u2C46"], /* GLAGOLITIC SMALL LETTER UKU */
  ["\u2C17", "\u2C47"], /* GLAGOLITIC SMALL LETTER FRITU */
  ["\u2C18", "\u2C48"], /* GLAGOLITIC SMALL LETTER HERU */
  ["\u2C19", "\u2C49"], /* GLAGOLITIC SMALL LETTER OTU */
  ["\u2C1A", "\u2C4A"], /* GLAGOLITIC SMALL LETTER PE */
  ["\u2C1B", "\u2C4B"], /* GLAGOLITIC SMALL LETTER SHTA */
  ["\u2C1C", "\u2C4C"], /* GLAGOLITIC SMALL LETTER TSI */
  ["\u2C1D", "\u2C4D"], /* GLAGOLITIC SMALL LETTER CHRIVI */
  ["\u2C1E", "\u2C4E"], /* GLAGOLITIC SMALL LETTER SHA */
  ["\u2C1F", "\u2C4F"], /* GLAGOLITIC SMALL LETTER YERU */
  ["\u2C20", "\u2C50"], /* GLAGOLITIC SMALL LETTER YERI */
  ["\u2C21", "\u2C51"], /* GLAGOLITIC SMALL LETTER YATI */
  ["\u2C22", "\u2C52"], /* GLAGOLITIC SMALL LETTER SPIDERY HA */
  ["\u2C23", "\u2C53"], /* GLAGOLITIC SMALL LETTER YU */
  ["\u2C24", "\u2C54"], /* GLAGOLITIC SMALL LETTER SMALL YUS */
  ["\u2C25", "\u2C55"], /* GLAGOLITIC SMALL LETTER SMALL YUS WITH TAIL */
  ["\u2C26", "\u2C56"], /* GLAGOLITIC SMALL LETTER YO */
  ["\u2C27", "\u2C57"], /* GLAGOLITIC SMALL LETTER IOTATED SMALL YUS */
  ["\u2C28", "\u2C58"], /* GLAGOLITIC SMALL LETTER BIG YUS */
  ["\u2C29", "\u2C59"], /* GLAGOLITIC SMALL LETTER IOTATED BIG YUS */
  ["\u2C2A", "\u2C5A"], /* GLAGOLITIC SMALL LETTER FITA */
  ["\u2C2B", "\u2C5B"], /* GLAGOLITIC SMALL LETTER IZHITSA */
  ["\u2C2C", "\u2C5C"], /* GLAGOLITIC SMALL LETTER SHTAPIC */
  ["\u2C2D", "\u2C5D"], /* GLAGOLITIC SMALL LETTER TROKUTASTI A */
  ["\u2C2E", "\u2C5E"], /* GLAGOLITIC SMALL LETTER LATINATE MYSLITE */
  ["\u2C5F", "\u2C5F"],
  ["\u2C60", "\u2C61"], /* LATIN CAPITAL LETTER L WITH DOUBLE BAR */
  ["\u2C60", "\u2C61"], /* LATIN SMALL LETTER L WITH DOUBLE BAR */
  ["\u2C62", "\u026B"], /* LATIN CAPITAL LETTER L WITH MIDDLE TILDE */
  ["\u2C63", "\u1D7D"], /* LATIN CAPITAL LETTER P WITH STROKE */
  ["\u2C64", "\u027D"], /* LATIN CAPITAL LETTER R WITH TAIL */
  ["\u023A", "\u2C65"], /* LATIN SMALL LETTER A WITH STROKE */
  ["\u023E", "\u2C66"], /* LATIN SMALL LETTER T WITH DIAGONAL STROKE */
  ["\u2C67", "\u2C68"], /* LATIN CAPITAL LETTER H WITH DESCENDER */
  ["\u2C67", "\u2C68"], /* LATIN SMALL LETTER H WITH DESCENDER */
  ["\u2C69", "\u2C6A"], /* LATIN CAPITAL LETTER K WITH DESCENDER */
  ["\u2C69", "\u2C6A"], /* LATIN SMALL LETTER K WITH DESCENDER */
  ["\u2C6B", "\u2C6C"], /* LATIN CAPITAL LETTER Z WITH DESCENDER */
  ["\u2C6B", "\u2C6C"], /* LATIN SMALL LETTER Z WITH DESCENDER */
  ["\u2C6D", "\u0251"], /* LATIN CAPITAL LETTER ALPHA */
  ["\u2C6E", "\u0271"], /* LATIN CAPITAL LETTER M WITH HOOK */
  ["\u2C6F", "\u0250"], /* LATIN CAPITAL LETTER TURNED A */
  ["\u2C70", "\u0252"], /* LATIN CAPITAL LETTER TURNED ALPHA */
  ["\u2C71", "\u2C71"], /* LATIN SMALL LETTER V WITH RIGHT HOOK */
  ["\u2C72", "\u2C73"], /* LATIN CAPITAL LETTER W WITH HOOK */
  ["\u2C72", "\u2C73"], /* LATIN SMALL LETTER W WITH HOOK */
  ["\u2C74", "\u2C74"], /* LATIN SMALL LETTER V WITH CURL */
  ["\u2C75", "\u2C76"], /* LATIN CAPITAL LETTER HALF H */
  ["\u2C75", "\u2C76"], /* LATIN SMALL LETTER HALF H */
  ["\u2C77", "\u2C77"], /* LATIN SMALL LETTER TAILLESS PHI */
  ["\u2C78", "\u2C78"], /* LATIN SMALL LETTER E WITH NOTCH */
  ["\u2C79", "\u2C79"], /* LATIN SMALL LETTER TURNED R WITH TAIL */
  ["\u2C7A", "\u2C7A"], /* LATIN SMALL LETTER O WITH LOW RING INSIDE */
  ["\u2C7B", "\u2C7B"], /* LATIN LETTER SMALL CAPITAL TURNED E */
  ["\u2C7C", "\u2C7C"], /* LATIN SUBSCRIPT SMALL LETTER J */
  ["\u2C7D", "\u2C7D"], /* MODIFIER LETTER CAPITAL V */
  ["\u2C7E", "\u023F"], /* LATIN CAPITAL LETTER S WITH SWASH TAIL */
  ["\u2C7F", "\u0240"], /* LATIN CAPITAL LETTER Z WITH SWASH TAIL */
  ["\u2C80", "\u2C81"], /* COPTIC CAPITAL LETTER ALFA */
  ["\u2C80", "\u2C81"], /* COPTIC SMALL LETTER ALFA */
  ["\u2C82", "\u2C83"], /* COPTIC CAPITAL LETTER VIDA */
  ["\u2C82", "\u2C83"], /* COPTIC SMALL LETTER VIDA */
  ["\u2C84", "\u2C85"], /* COPTIC CAPITAL LETTER GAMMA */
  ["\u2C84", "\u2C85"], /* COPTIC SMALL LETTER GAMMA */
  ["\u2C86", "\u2C87"], /* COPTIC CAPITAL LETTER DALDA */
  ["\u2C86", "\u2C87"], /* COPTIC SMALL LETTER DALDA */
  ["\u2C88", "\u2C89"], /* COPTIC CAPITAL LETTER EIE */
  ["\u2C88", "\u2C89"], /* COPTIC SMALL LETTER EIE */
  ["\u2C8A", "\u2C8B"], /* COPTIC CAPITAL LETTER SOU */
  ["\u2C8A", "\u2C8B"], /* COPTIC SMALL LETTER SOU */
  ["\u2C8C", "\u2C8D"], /* COPTIC CAPITAL LETTER ZATA */
  ["\u2C8C", "\u2C8D"], /* COPTIC SMALL LETTER ZATA */
  ["\u2C8E", "\u2C8F"], /* COPTIC CAPITAL LETTER HATE */
  ["\u2C8E", "\u2C8F"], /* COPTIC SMALL LETTER HATE */
  ["\u2C90", "\u2C91"], /* COPTIC CAPITAL LETTER THETHE */
  ["\u2C90", "\u2C91"], /* COPTIC SMALL LETTER THETHE */
  ["\u2C92", "\u2C93"], /* COPTIC CAPITAL LETTER IAUDA */
  ["\u2C92", "\u2C93"], /* COPTIC SMALL LETTER IAUDA */
  ["\u2C94", "\u2C95"], /* COPTIC CAPITAL LETTER KAPA */
  ["\u2C94", "\u2C95"], /* COPTIC SMALL LETTER KAPA */
  ["\u2C96", "\u2C97"], /* COPTIC CAPITAL LETTER LAULA */
  ["\u2C96", "\u2C97"], /* COPTIC SMALL LETTER LAULA */
  ["\u2C98", "\u2C99"], /* COPTIC CAPITAL LETTER MI */
  ["\u2C98", "\u2C99"], /* COPTIC SMALL LETTER MI */
  ["\u2C9A", "\u2C9B"], /* COPTIC CAPITAL LETTER NI */
  ["\u2C9A", "\u2C9B"], /* COPTIC SMALL LETTER NI */
  ["\u2C9C", "\u2C9D"], /* COPTIC CAPITAL LETTER KSI */
  ["\u2C9C", "\u2C9D"], /* COPTIC SMALL LETTER KSI */
  ["\u2C9E", "\u2C9F"], /* COPTIC CAPITAL LETTER O */
  ["\u2C9E", "\u2C9F"], /* COPTIC SMALL LETTER O */
  ["\u2CA0", "\u2CA1"], /* COPTIC CAPITAL LETTER PI */
  ["\u2CA0", "\u2CA1"], /* COPTIC SMALL LETTER PI */
  ["\u2CA2", "\u2CA3"], /* COPTIC CAPITAL LETTER RO */
  ["\u2CA2", "\u2CA3"], /* COPTIC SMALL LETTER RO */
  ["\u2CA4", "\u2CA5"], /* COPTIC CAPITAL LETTER SIMA */
  ["\u2CA4", "\u2CA5"], /* COPTIC SMALL LETTER SIMA */
  ["\u2CA6", "\u2CA7"], /* COPTIC CAPITAL LETTER TAU */
  ["\u2CA6", "\u2CA7"], /* COPTIC SMALL LETTER TAU */
  ["\u2CA8", "\u2CA9"], /* COPTIC CAPITAL LETTER UA */
  ["\u2CA8", "\u2CA9"], /* COPTIC SMALL LETTER UA */
  ["\u2CAA", "\u2CAB"], /* COPTIC CAPITAL LETTER FI */
  ["\u2CAA", "\u2CAB"], /* COPTIC SMALL LETTER FI */
  ["\u2CAC", "\u2CAD"], /* COPTIC CAPITAL LETTER KHI */
  ["\u2CAC", "\u2CAD"], /* COPTIC SMALL LETTER KHI */
  ["\u2CAE", "\u2CAF"], /* COPTIC CAPITAL LETTER PSI */
  ["\u2CAE", "\u2CAF"], /* COPTIC SMALL LETTER PSI */
  ["\u2CB0", "\u2CB1"], /* COPTIC CAPITAL LETTER OOU */
  ["\u2CB0", "\u2CB1"], /* COPTIC SMALL LETTER OOU */
  ["\u2CB2", "\u2CB3"], /* COPTIC CAPITAL LETTER DIALECT-P ALEF */
  ["\u2CB2", "\u2CB3"], /* COPTIC SMALL LETTER DIALECT-P ALEF */
  ["\u2CB4", "\u2CB5"], /* COPTIC CAPITAL LETTER OLD COPTIC AIN */
  ["\u2CB4", "\u2CB5"], /* COPTIC SMALL LETTER OLD COPTIC AIN */
  ["\u2CB6", "\u2CB7"], /* COPTIC CAPITAL LETTER CRYPTOGRAMMIC EIE */
  ["\u2CB6", "\u2CB7"], /* COPTIC SMALL LETTER CRYPTOGRAMMIC EIE */
  ["\u2CB8", "\u2CB9"], /* COPTIC CAPITAL LETTER DIALECT-P KAPA */
  ["\u2CB8", "\u2CB9"], /* COPTIC SMALL LETTER DIALECT-P KAPA */
  ["\u2CBA", "\u2CBB"], /* COPTIC CAPITAL LETTER DIALECT-P NI */
  ["\u2CBA", "\u2CBB"], /* COPTIC SMALL LETTER DIALECT-P NI */
  ["\u2CBC", "\u2CBD"], /* COPTIC CAPITAL LETTER CRYPTOGRAMMIC NI */
  ["\u2CBC", "\u2CBD"], /* COPTIC SMALL LETTER CRYPTOGRAMMIC NI */
  ["\u2CBE", "\u2CBF"], /* COPTIC CAPITAL LETTER OLD COPTIC OOU */
  ["\u2CBE", "\u2CBF"], /* COPTIC SMALL LETTER OLD COPTIC OOU */
  ["\u2CC0", "\u2CC1"], /* COPTIC CAPITAL LETTER SAMPI */
  ["\u2CC0", "\u2CC1"], /* COPTIC SMALL LETTER SAMPI */
  ["\u2CC2", "\u2CC3"], /* COPTIC CAPITAL LETTER CROSSED SHEI */
  ["\u2CC2", "\u2CC3"], /* COPTIC SMALL LETTER CROSSED SHEI */
  ["\u2CC4", "\u2CC5"], /* COPTIC CAPITAL LETTER OLD COPTIC SHEI */
  ["\u2CC4", "\u2CC5"], /* COPTIC SMALL LETTER OLD COPTIC SHEI */
  ["\u2CC6", "\u2CC7"], /* COPTIC CAPITAL LETTER OLD COPTIC ESH */
  ["\u2CC6", "\u2CC7"], /* COPTIC SMALL LETTER OLD COPTIC ESH */
  ["\u2CC8", "\u2CC9"], /* COPTIC CAPITAL LETTER AKHMIMIC KHEI */
  ["\u2CC8", "\u2CC9"], /* COPTIC SMALL LETTER AKHMIMIC KHEI */
  ["\u2CCA", "\u2CCB"], /* COPTIC CAPITAL LETTER DIALECT-P HORI */
  ["\u2CCA", "\u2CCB"], /* COPTIC SMALL LETTER DIALECT-P HORI */
  ["\u2CCC", "\u2CCD"], /* COPTIC CAPITAL LETTER OLD COPTIC HORI */
  ["\u2CCC", "\u2CCD"], /* COPTIC SMALL LETTER OLD COPTIC HORI */
  ["\u2CCE", "\u2CCF"], /* COPTIC CAPITAL LETTER OLD COPTIC HA */
  ["\u2CCE", "\u2CCF"], /* COPTIC SMALL LETTER OLD COPTIC HA */
  ["\u2CD0", "\u2CD1"], /* COPTIC CAPITAL LETTER L-SHAPED HA */
  ["\u2CD0", "\u2CD1"], /* COPTIC SMALL LETTER L-SHAPED HA */
  ["\u2CD2", "\u2CD3"], /* COPTIC CAPITAL LETTER OLD COPTIC HEI */
  ["\u2CD2", "\u2CD3"], /* COPTIC SMALL LETTER OLD COPTIC HEI */
  ["\u2CD4", "\u2CD5"], /* COPTIC CAPITAL LETTER OLD COPTIC HAT */
  ["\u2CD4", "\u2CD5"], /* COPTIC SMALL LETTER OLD COPTIC HAT */
  ["\u2CD6", "\u2CD7"], /* COPTIC CAPITAL LETTER OLD COPTIC GANGIA */
  ["\u2CD6", "\u2CD7"], /* COPTIC SMALL LETTER OLD COPTIC GANGIA */
  ["\u2CD8", "\u2CD9"], /* COPTIC CAPITAL LETTER OLD COPTIC DJA */
  ["\u2CD8", "\u2CD9"], /* COPTIC SMALL LETTER OLD COPTIC DJA */
  ["\u2CDA", "\u2CDB"], /* COPTIC CAPITAL LETTER OLD COPTIC SHIMA */
  ["\u2CDA", "\u2CDB"], /* COPTIC SMALL LETTER OLD COPTIC SHIMA */
  ["\u2CDC", "\u2CDD"], /* COPTIC CAPITAL LETTER OLD NUBIAN SHIMA */
  ["\u2CDC", "\u2CDD"], /* COPTIC SMALL LETTER OLD NUBIAN SHIMA */
  ["\u2CDE", "\u2CDF"], /* COPTIC CAPITAL LETTER OLD NUBIAN NGI */
  ["\u2CDE", "\u2CDF"], /* COPTIC SMALL LETTER OLD NUBIAN NGI */
  ["\u2CE0", "\u2CE1"], /* COPTIC CAPITAL LETTER OLD NUBIAN NYI */
  ["\u2CE0", "\u2CE1"], /* COPTIC SMALL LETTER OLD NUBIAN NYI */
  ["\u2CE2", "\u2CE3"], /* COPTIC CAPITAL LETTER OLD NUBIAN WAU */
  ["\u2CE2", "\u2CE3"], /* COPTIC SMALL LETTER OLD NUBIAN WAU */
  ["\u2CE4", "\u2CE4"], /* COPTIC SYMBOL KAI */
  ["\u2CE5", "\u2CE5"], /* COPTIC SYMBOL MI RO */
  ["\u2CE6", "\u2CE6"], /* COPTIC SYMBOL PI RO */
  ["\u2CE7", "\u2CE7"], /* COPTIC SYMBOL STAUROS */
  ["\u2CE8", "\u2CE8"], /* COPTIC SYMBOL TAU RO */
  ["\u2CE9", "\u2CE9"], /* COPTIC SYMBOL KHI RO */
  ["\u2CEA", "\u2CEA"], /* COPTIC SYMBOL SHIMA SIMA */
  ["\u2CEB", "\u2CEC"], /* COPTIC CAPITAL LETTER CRYPTOGRAMMIC SHEI */
  ["\u2CEB", "\u2CEC"], /* COPTIC SMALL LETTER CRYPTOGRAMMIC SHEI */
  ["\u2CED", "\u2CEE"], /* COPTIC CAPITAL LETTER CRYPTOGRAMMIC GANGIA */
  ["\u2CED", "\u2CEE"], /* COPTIC SMALL LETTER CRYPTOGRAMMIC GANGIA */
  ["\u2CEF", "\u2CEF"], /* COPTIC COMBINING NI ABOVE */
  ["\u2CF0", "\u2CF0"], /* COPTIC COMBINING SPIRITUS ASPER */
  ["\u2CF1", "\u2CF1"], /* COPTIC COMBINING SPIRITUS LENIS */
  ["\u2CF2", "\u2CF3"], /* COPTIC CAPITAL LETTER BOHAIRIC KHEI */
  ["\u2CF2", "\u2CF3"], /* COPTIC SMALL LETTER BOHAIRIC KHEI */
  ["\u2CF4", "\u2CF4"],
  ["\u2CF5", "\u2CF5"],
  ["\u2CF6", "\u2CF6"],
  ["\u2CF7", "\u2CF7"],
  ["\u2CF8", "\u2CF8"],
  ["\u2CF9", "\u2CF9"], /* COPTIC OLD NUBIAN FULL STOP */
  ["\u2CFA", "\u2CFA"], /* COPTIC OLD NUBIAN DIRECT QUESTION MARK */
  ["\u2CFB", "\u2CFB"], /* COPTIC OLD NUBIAN INDIRECT QUESTION MARK */
  ["\u2CFC", "\u2CFC"], /* COPTIC OLD NUBIAN VERSE DIVIDER */
  ["\u2CFD", "\u2CFD"], /* COPTIC FRACTION ONE HALF */
  ["\u2CFE", "\u2CFE"], /* COPTIC FULL STOP */
  ["\u2CFF", "\u2CFF"], /* COPTIC MORPHOLOGICAL DIVIDER */
  ["\u10A0", "\u2D00"], /* GEORGIAN SMALL LETTER AN */
  ["\u10A1", "\u2D01"], /* GEORGIAN SMALL LETTER BAN */
  ["\u10A2", "\u2D02"], /* GEORGIAN SMALL LETTER GAN */
  ["\u10A3", "\u2D03"], /* GEORGIAN SMALL LETTER DON */
  ["\u10A4", "\u2D04"], /* GEORGIAN SMALL LETTER EN */
  ["\u10A5", "\u2D05"], /* GEORGIAN SMALL LETTER VIN */
  ["\u10A6", "\u2D06"], /* GEORGIAN SMALL LETTER ZEN */
  ["\u10A7", "\u2D07"], /* GEORGIAN SMALL LETTER TAN */
  ["\u10A8", "\u2D08"], /* GEORGIAN SMALL LETTER IN */
  ["\u10A9", "\u2D09"], /* GEORGIAN SMALL LETTER KAN */
  ["\u10AA", "\u2D0A"], /* GEORGIAN SMALL LETTER LAS */
  ["\u10AB", "\u2D0B"], /* GEORGIAN SMALL LETTER MAN */
  ["\u10AC", "\u2D0C"], /* GEORGIAN SMALL LETTER NAR */
  ["\u10AD", "\u2D0D"], /* GEORGIAN SMALL LETTER ON */
  ["\u10AE", "\u2D0E"], /* GEORGIAN SMALL LETTER PAR */
  ["\u10AF", "\u2D0F"], /* GEORGIAN SMALL LETTER ZHAR */
  ["\u10B0", "\u2D10"], /* GEORGIAN SMALL LETTER RAE */
  ["\u10B1", "\u2D11"], /* GEORGIAN SMALL LETTER SAN */
  ["\u10B2", "\u2D12"], /* GEORGIAN SMALL LETTER TAR */
  ["\u10B3", "\u2D13"], /* GEORGIAN SMALL LETTER UN */
  ["\u10B4", "\u2D14"], /* GEORGIAN SMALL LETTER PHAR */
  ["\u10B5", "\u2D15"], /* GEORGIAN SMALL LETTER KHAR */
  ["\u10B6", "\u2D16"], /* GEORGIAN SMALL LETTER GHAN */
  ["\u10B7", "\u2D17"], /* GEORGIAN SMALL LETTER QAR */
  ["\u10B8", "\u2D18"], /* GEORGIAN SMALL LETTER SHIN */
  ["\u10B9", "\u2D19"], /* GEORGIAN SMALL LETTER CHIN */
  ["\u10BA", "\u2D1A"], /* GEORGIAN SMALL LETTER CAN */
  ["\u10BB", "\u2D1B"], /* GEORGIAN SMALL LETTER JIL */
  ["\u10BC", "\u2D1C"], /* GEORGIAN SMALL LETTER CIL */
  ["\u10BD", "\u2D1D"], /* GEORGIAN SMALL LETTER CHAR */
  ["\u10BE", "\u2D1E"], /* GEORGIAN SMALL LETTER XAN */
  ["\u10BF", "\u2D1F"], /* GEORGIAN SMALL LETTER JHAN */
  ["\u10C0", "\u2D20"], /* GEORGIAN SMALL LETTER HAE */
  ["\u10C1", "\u2D21"], /* GEORGIAN SMALL LETTER HE */
  ["\u10C2", "\u2D22"], /* GEORGIAN SMALL LETTER HIE */
  ["\u10C3", "\u2D23"], /* GEORGIAN SMALL LETTER WE */
  ["\u10C4", "\u2D24"], /* GEORGIAN SMALL LETTER HAR */
  ["\u10C5", "\u2D25"], /* GEORGIAN SMALL LETTER HOE */
  ["\u2D26", "\u2D26"],
  ["\u10C7", "\u2D27"], /* GEORGIAN SMALL LETTER YN */
  ["\u2D28", "\u2D28"],
  ["\u2D29", "\u2D29"],
  ["\u2D2A", "\u2D2A"],
  ["\u2D2B", "\u2D2B"],
  ["\u2D2C", "\u2D2C"],
  ["\u10CD", "\u2D2D"], /* GEORGIAN SMALL LETTER AEN */
  ["\u2D2E", "\u2D2E"],
  ["\u2D2F", "\u2D2F"],
  ["\u2D30", "\u2D30"], /* TIFINAGH LETTER YA */
  ["\u2D31", "\u2D31"], /* TIFINAGH LETTER YAB */
  ["\u2D32", "\u2D32"], /* TIFINAGH LETTER YABH */
  ["\u2D33", "\u2D33"], /* TIFINAGH LETTER YAG */
  ["\u2D34", "\u2D34"], /* TIFINAGH LETTER YAGHH */
  ["\u2D35", "\u2D35"], /* TIFINAGH LETTER BERBER ACADEMY YAJ */
  ["\u2D36", "\u2D36"], /* TIFINAGH LETTER YAJ */
  ["\u2D37", "\u2D37"], /* TIFINAGH LETTER YAD */
  ["\u2D38", "\u2D38"], /* TIFINAGH LETTER YADH */
  ["\u2D39", "\u2D39"], /* TIFINAGH LETTER YADD */
  ["\u2D3A", "\u2D3A"], /* TIFINAGH LETTER YADDH */
  ["\u2D3B", "\u2D3B"], /* TIFINAGH LETTER YEY */
  ["\u2D3C", "\u2D3C"], /* TIFINAGH LETTER YAF */
  ["\u2D3D", "\u2D3D"], /* TIFINAGH LETTER YAK */
  ["\u2D3E", "\u2D3E"], /* TIFINAGH LETTER TUAREG YAK */
  ["\u2D3F", "\u2D3F"], /* TIFINAGH LETTER YAKHH */
  ["\u2D40", "\u2D40"], /* TIFINAGH LETTER YAH */
  ["\u2D41", "\u2D41"], /* TIFINAGH LETTER BERBER ACADEMY YAH */
  ["\u2D42", "\u2D42"], /* TIFINAGH LETTER TUAREG YAH */
  ["\u2D43", "\u2D43"], /* TIFINAGH LETTER YAHH */
  ["\u2D44", "\u2D44"], /* TIFINAGH LETTER YAA */
  ["\u2D45", "\u2D45"], /* TIFINAGH LETTER YAKH */
  ["\u2D46", "\u2D46"], /* TIFINAGH LETTER TUAREG YAKH */
  ["\u2D47", "\u2D47"], /* TIFINAGH LETTER YAQ */
  ["\u2D48", "\u2D48"], /* TIFINAGH LETTER TUAREG YAQ */
  ["\u2D49", "\u2D49"], /* TIFINAGH LETTER YI */
  ["\u2D4A", "\u2D4A"], /* TIFINAGH LETTER YAZH */
  ["\u2D4B", "\u2D4B"], /* TIFINAGH LETTER AHAGGAR YAZH */
  ["\u2D4C", "\u2D4C"], /* TIFINAGH LETTER TUAREG YAZH */
  ["\u2D4D", "\u2D4D"], /* TIFINAGH LETTER YAL */
  ["\u2D4E", "\u2D4E"], /* TIFINAGH LETTER YAM */
  ["\u2D4F", "\u2D4F"], /* TIFINAGH LETTER YAN */
  ["\u2D50", "\u2D50"], /* TIFINAGH LETTER TUAREG YAGN */
  ["\u2D51", "\u2D51"], /* TIFINAGH LETTER TUAREG YANG */
  ["\u2D52", "\u2D52"], /* TIFINAGH LETTER YAP */
  ["\u2D53", "\u2D53"], /* TIFINAGH LETTER YU */
  ["\u2D54", "\u2D54"], /* TIFINAGH LETTER YAR */
  ["\u2D55", "\u2D55"], /* TIFINAGH LETTER YARR */
  ["\u2D56", "\u2D56"], /* TIFINAGH LETTER YAGH */
  ["\u2D57", "\u2D57"], /* TIFINAGH LETTER TUAREG YAGH */
  ["\u2D58", "\u2D58"], /* TIFINAGH LETTER AYER YAGH */
  ["\u2D59", "\u2D59"], /* TIFINAGH LETTER YAS */
  ["\u2D5A", "\u2D5A"], /* TIFINAGH LETTER YASS */
  ["\u2D5B", "\u2D5B"], /* TIFINAGH LETTER YASH */
  ["\u2D5C", "\u2D5C"], /* TIFINAGH LETTER YAT */
  ["\u2D5D", "\u2D5D"], /* TIFINAGH LETTER YATH */
  ["\u2D5E", "\u2D5E"], /* TIFINAGH LETTER YACH */
  ["\u2D5F", "\u2D5F"], /* TIFINAGH LETTER YATT */
  ["\u2D60", "\u2D60"], /* TIFINAGH LETTER YAV */
  ["\u2D61", "\u2D61"], /* TIFINAGH LETTER YAW */
  ["\u2D62", "\u2D62"], /* TIFINAGH LETTER YAY */
  ["\u2D63", "\u2D63"], /* TIFINAGH LETTER YAZ */
  ["\u2D64", "\u2D64"], /* TIFINAGH LETTER TAWELLEMET YAZ */
  ["\u2D65", "\u2D65"], /* TIFINAGH LETTER YAZZ */
  ["\u2D66", "\u2D66"], /* TIFINAGH LETTER YE */
  ["\u2D67", "\u2D67"], /* TIFINAGH LETTER YO */
  ["\u2D68", "\u2D68"],
  ["\u2D69", "\u2D69"],
  ["\u2D6A", "\u2D6A"],
  ["\u2D6B", "\u2D6B"],
  ["\u2D6C", "\u2D6C"],
  ["\u2D6D", "\u2D6D"],
  ["\u2D6E", "\u2D6E"],
  ["\u2D6F", "\u2D6F"], /* TIFINAGH MODIFIER LETTER LABIALIZATION MARK */
  ["\u2D70", "\u2D70"], /* TIFINAGH SEPARATOR MARK */
  ["\u2D71", "\u2D71"],
  ["\u2D72", "\u2D72"],
  ["\u2D73", "\u2D73"],
  ["\u2D74", "\u2D74"],
  ["\u2D75", "\u2D75"],
  ["\u2D76", "\u2D76"],
  ["\u2D77", "\u2D77"],
  ["\u2D78", "\u2D78"],
  ["\u2D79", "\u2D79"],
  ["\u2D7A", "\u2D7A"],
  ["\u2D7B", "\u2D7B"],
  ["\u2D7C", "\u2D7C"],
  ["\u2D7D", "\u2D7D"],
  ["\u2D7E", "\u2D7E"],
  ["\u2D7F", "\u2D7F"], /* TIFINAGH CONSONANT JOINER */
  ["\u2D80", "\u2D80"], /* ETHIOPIC SYLLABLE LOA */
  ["\u2D81", "\u2D81"], /* ETHIOPIC SYLLABLE MOA */
  ["\u2D82", "\u2D82"], /* ETHIOPIC SYLLABLE ROA */
  ["\u2D83", "\u2D83"], /* ETHIOPIC SYLLABLE SOA */
  ["\u2D84", "\u2D84"], /* ETHIOPIC SYLLABLE SHOA */
  ["\u2D85", "\u2D85"], /* ETHIOPIC SYLLABLE BOA */
  ["\u2D86", "\u2D86"], /* ETHIOPIC SYLLABLE TOA */
  ["\u2D87", "\u2D87"], /* ETHIOPIC SYLLABLE COA */
  ["\u2D88", "\u2D88"], /* ETHIOPIC SYLLABLE NOA */
  ["\u2D89", "\u2D89"], /* ETHIOPIC SYLLABLE NYOA */
  ["\u2D8A", "\u2D8A"], /* ETHIOPIC SYLLABLE GLOTTAL OA */
  ["\u2D8B", "\u2D8B"], /* ETHIOPIC SYLLABLE ZOA */
  ["\u2D8C", "\u2D8C"], /* ETHIOPIC SYLLABLE DOA */
  ["\u2D8D", "\u2D8D"], /* ETHIOPIC SYLLABLE DDOA */
  ["\u2D8E", "\u2D8E"], /* ETHIOPIC SYLLABLE JOA */
  ["\u2D8F", "\u2D8F"], /* ETHIOPIC SYLLABLE THOA */
  ["\u2D90", "\u2D90"], /* ETHIOPIC SYLLABLE CHOA */
  ["\u2D91", "\u2D91"], /* ETHIOPIC SYLLABLE PHOA */
  ["\u2D92", "\u2D92"], /* ETHIOPIC SYLLABLE POA */
  ["\u2D93", "\u2D93"], /* ETHIOPIC SYLLABLE GGWA */
  ["\u2D94", "\u2D94"], /* ETHIOPIC SYLLABLE GGWI */
  ["\u2D95", "\u2D95"], /* ETHIOPIC SYLLABLE GGWEE */
  ["\u2D96", "\u2D96"], /* ETHIOPIC SYLLABLE GGWE */
  ["\u2D97", "\u2D97"],
  ["\u2D98", "\u2D98"],
  ["\u2D99", "\u2D99"],
  ["\u2D9A", "\u2D9A"],
  ["\u2D9B", "\u2D9B"],
  ["\u2D9C", "\u2D9C"],
  ["\u2D9D", "\u2D9D"],
  ["\u2D9E", "\u2D9E"],
  ["\u2D9F", "\u2D9F"],
  ["\u2DA0", "\u2DA0"], /* ETHIOPIC SYLLABLE SSA */
  ["\u2DA1", "\u2DA1"], /* ETHIOPIC SYLLABLE SSU */
  ["\u2DA2", "\u2DA2"], /* ETHIOPIC SYLLABLE SSI */
  ["\u2DA3", "\u2DA3"], /* ETHIOPIC SYLLABLE SSAA */
  ["\u2DA4", "\u2DA4"], /* ETHIOPIC SYLLABLE SSEE */
  ["\u2DA5", "\u2DA5"], /* ETHIOPIC SYLLABLE SSE */
  ["\u2DA6", "\u2DA6"], /* ETHIOPIC SYLLABLE SSO */
  ["\u2DA7", "\u2DA7"],
  ["\u2DA8", "\u2DA8"], /* ETHIOPIC SYLLABLE CCA */
  ["\u2DA9", "\u2DA9"], /* ETHIOPIC SYLLABLE CCU */
  ["\u2DAA", "\u2DAA"], /* ETHIOPIC SYLLABLE CCI */
  ["\u2DAB", "\u2DAB"], /* ETHIOPIC SYLLABLE CCAA */
  ["\u2DAC", "\u2DAC"], /* ETHIOPIC SYLLABLE CCEE */
  ["\u2DAD", "\u2DAD"], /* ETHIOPIC SYLLABLE CCE */
  ["\u2DAE", "\u2DAE"], /* ETHIOPIC SYLLABLE CCO */
  ["\u2DAF", "\u2DAF"],
  ["\u2DB0", "\u2DB0"], /* ETHIOPIC SYLLABLE ZZA */
  ["\u2DB1", "\u2DB1"], /* ETHIOPIC SYLLABLE ZZU */
  ["\u2DB2", "\u2DB2"], /* ETHIOPIC SYLLABLE ZZI */
  ["\u2DB3", "\u2DB3"], /* ETHIOPIC SYLLABLE ZZAA */
  ["\u2DB4", "\u2DB4"], /* ETHIOPIC SYLLABLE ZZEE */
  ["\u2DB5", "\u2DB5"], /* ETHIOPIC SYLLABLE ZZE */
  ["\u2DB6", "\u2DB6"], /* ETHIOPIC SYLLABLE ZZO */
  ["\u2DB7", "\u2DB7"],
  ["\u2DB8", "\u2DB8"], /* ETHIOPIC SYLLABLE CCHA */
  ["\u2DB9", "\u2DB9"], /* ETHIOPIC SYLLABLE CCHU */
  ["\u2DBA", "\u2DBA"], /* ETHIOPIC SYLLABLE CCHI */
  ["\u2DBB", "\u2DBB"], /* ETHIOPIC SYLLABLE CCHAA */
  ["\u2DBC", "\u2DBC"], /* ETHIOPIC SYLLABLE CCHEE */
  ["\u2DBD", "\u2DBD"], /* ETHIOPIC SYLLABLE CCHE */
  ["\u2DBE", "\u2DBE"], /* ETHIOPIC SYLLABLE CCHO */
  ["\u2DBF", "\u2DBF"],
  ["\u2DC0", "\u2DC0"], /* ETHIOPIC SYLLABLE QYA */
  ["\u2DC1", "\u2DC1"], /* ETHIOPIC SYLLABLE QYU */
  ["\u2DC2", "\u2DC2"], /* ETHIOPIC SYLLABLE QYI */
  ["\u2DC3", "\u2DC3"], /* ETHIOPIC SYLLABLE QYAA */
  ["\u2DC4", "\u2DC4"], /* ETHIOPIC SYLLABLE QYEE */
  ["\u2DC5", "\u2DC5"], /* ETHIOPIC SYLLABLE QYE */
  ["\u2DC6", "\u2DC6"], /* ETHIOPIC SYLLABLE QYO */
  ["\u2DC7", "\u2DC7"],
  ["\u2DC8", "\u2DC8"], /* ETHIOPIC SYLLABLE KYA */
  ["\u2DC9", "\u2DC9"], /* ETHIOPIC SYLLABLE KYU */
  ["\u2DCA", "\u2DCA"], /* ETHIOPIC SYLLABLE KYI */
  ["\u2DCB", "\u2DCB"], /* ETHIOPIC SYLLABLE KYAA */
  ["\u2DCC", "\u2DCC"], /* ETHIOPIC SYLLABLE KYEE */
  ["\u2DCD", "\u2DCD"], /* ETHIOPIC SYLLABLE KYE */
  ["\u2DCE", "\u2DCE"], /* ETHIOPIC SYLLABLE KYO */
  ["\u2DCF", "\u2DCF"],
  ["\u2DD0", "\u2DD0"], /* ETHIOPIC SYLLABLE XYA */
  ["\u2DD1", "\u2DD1"], /* ETHIOPIC SYLLABLE XYU */
  ["\u2DD2", "\u2DD2"], /* ETHIOPIC SYLLABLE XYI */
  ["\u2DD3", "\u2DD3"], /* ETHIOPIC SYLLABLE XYAA */
  ["\u2DD4", "\u2DD4"], /* ETHIOPIC SYLLABLE XYEE */
  ["\u2DD5", "\u2DD5"], /* ETHIOPIC SYLLABLE XYE */
  ["\u2DD6", "\u2DD6"], /* ETHIOPIC SYLLABLE XYO */
  ["\u2DD7", "\u2DD7"],
  ["\u2DD8", "\u2DD8"], /* ETHIOPIC SYLLABLE GYA */
  ["\u2DD9", "\u2DD9"], /* ETHIOPIC SYLLABLE GYU */
  ["\u2DDA", "\u2DDA"], /* ETHIOPIC SYLLABLE GYI */
  ["\u2DDB", "\u2DDB"], /* ETHIOPIC SYLLABLE GYAA */
  ["\u2DDC", "\u2DDC"], /* ETHIOPIC SYLLABLE GYEE */
  ["\u2DDD", "\u2DDD"], /* ETHIOPIC SYLLABLE GYE */
  ["\u2DDE", "\u2DDE"], /* ETHIOPIC SYLLABLE GYO */
  ["\u2DDF", "\u2DDF"],
  ["\u2DE0", "\u2DE0"], /* COMBINING CYRILLIC LETTER BE */
  ["\u2DE1", "\u2DE1"], /* COMBINING CYRILLIC LETTER VE */
  ["\u2DE2", "\u2DE2"], /* COMBINING CYRILLIC LETTER GHE */
  ["\u2DE3", "\u2DE3"], /* COMBINING CYRILLIC LETTER DE */
  ["\u2DE4", "\u2DE4"], /* COMBINING CYRILLIC LETTER ZHE */
  ["\u2DE5", "\u2DE5"], /* COMBINING CYRILLIC LETTER ZE */
  ["\u2DE6", "\u2DE6"], /* COMBINING CYRILLIC LETTER KA */
  ["\u2DE7", "\u2DE7"], /* COMBINING CYRILLIC LETTER EL */
  ["\u2DE8", "\u2DE8"], /* COMBINING CYRILLIC LETTER EM */
  ["\u2DE9", "\u2DE9"], /* COMBINING CYRILLIC LETTER EN */
  ["\u2DEA", "\u2DEA"], /* COMBINING CYRILLIC LETTER O */
  ["\u2DEB", "\u2DEB"], /* COMBINING CYRILLIC LETTER PE */
  ["\u2DEC", "\u2DEC"], /* COMBINING CYRILLIC LETTER ER */
  ["\u2DED", "\u2DED"], /* COMBINING CYRILLIC LETTER ES */
  ["\u2DEE", "\u2DEE"], /* COMBINING CYRILLIC LETTER TE */
  ["\u2DEF", "\u2DEF"], /* COMBINING CYRILLIC LETTER HA */
  ["\u2DF0", "\u2DF0"], /* COMBINING CYRILLIC LETTER TSE */
  ["\u2DF1", "\u2DF1"], /* COMBINING CYRILLIC LETTER CHE */
  ["\u2DF2", "\u2DF2"], /* COMBINING CYRILLIC LETTER SHA */
  ["\u2DF3", "\u2DF3"], /* COMBINING CYRILLIC LETTER SHCHA */
  ["\u2DF4", "\u2DF4"], /* COMBINING CYRILLIC LETTER FITA */
  ["\u2DF5", "\u2DF5"], /* COMBINING CYRILLIC LETTER ES-TE */
  ["\u2DF6", "\u2DF6"], /* COMBINING CYRILLIC LETTER A */
  ["\u2DF7", "\u2DF7"], /* COMBINING CYRILLIC LETTER IE */
  ["\u2DF8", "\u2DF8"], /* COMBINING CYRILLIC LETTER DJERV */
  ["\u2DF9", "\u2DF9"], /* COMBINING CYRILLIC LETTER MONOGRAPH UK */
  ["\u2DFA", "\u2DFA"], /* COMBINING CYRILLIC LETTER YAT */
  ["\u2DFB", "\u2DFB"], /* COMBINING CYRILLIC LETTER YU */
  ["\u2DFC", "\u2DFC"], /* COMBINING CYRILLIC LETTER IOTIFIED A */
  ["\u2DFD", "\u2DFD"], /* COMBINING CYRILLIC LETTER LITTLE YUS */
  ["\u2DFE", "\u2DFE"], /* COMBINING CYRILLIC LETTER BIG YUS */
  ["\u2DFF", "\u2DFF"], /* COMBINING CYRILLIC LETTER IOTIFIED BIG YUS */
  ["\u2E00", "\u2E00"], /* RIGHT ANGLE SUBSTITUTION MARKER */
  ["\u2E01", "\u2E01"], /* RIGHT ANGLE DOTTED SUBSTITUTION MARKER */
  ["\u2E02", "\u2E02"], /* LEFT SUBSTITUTION BRACKET */
  ["\u2E03", "\u2E03"], /* RIGHT SUBSTITUTION BRACKET */
  ["\u2E04", "\u2E04"], /* LEFT DOTTED SUBSTITUTION BRACKET */
  ["\u2E05", "\u2E05"], /* RIGHT DOTTED SUBSTITUTION BRACKET */
  ["\u2E06", "\u2E06"], /* RAISED INTERPOLATION MARKER */
  ["\u2E07", "\u2E07"], /* RAISED DOTTED INTERPOLATION MARKER */
  ["\u2E08", "\u2E08"], /* DOTTED TRANSPOSITION MARKER */
  ["\u2E09", "\u2E09"], /* LEFT TRANSPOSITION BRACKET */
  ["\u2E0A", "\u2E0A"], /* RIGHT TRANSPOSITION BRACKET */
  ["\u2E0B", "\u2E0B"], /* RAISED SQUARE */
  ["\u2E0C", "\u2E0C"], /* LEFT RAISED OMISSION BRACKET */
  ["\u2E0D", "\u2E0D"], /* RIGHT RAISED OMISSION BRACKET */
  ["\u2E0E", "\u2E0E"], /* EDITORIAL CORONIS */
  ["\u2E0F", "\u2E0F"], /* PARAGRAPHOS */
  ["\u2E10", "\u2E10"], /* FORKED PARAGRAPHOS */
  ["\u2E11", "\u2E11"], /* REVERSED FORKED PARAGRAPHOS */
  ["\u2E12", "\u2E12"], /* HYPODIASTOLE */
  ["\u2E13", "\u2E13"], /* DOTTED OBELOS */
  ["\u2E14", "\u2E14"], /* DOWNWARDS ANCORA */
  ["\u2E15", "\u2E15"], /* UPWARDS ANCORA */
  ["\u2E16", "\u2E16"], /* DOTTED RIGHT-POINTING ANGLE */
  ["\u2E17", "\u2E17"], /* DOUBLE OBLIQUE HYPHEN */
  ["\u2E18", "\u2E18"], /* INVERTED INTERROBANG */
  ["\u2E19", "\u2E19"], /* PALM BRANCH */
  ["\u2E1A", "\u2E1A"], /* HYPHEN WITH DIAERESIS */
  ["\u2E1B", "\u2E1B"], /* TILDE WITH RING ABOVE */
  ["\u2E1C", "\u2E1C"], /* LEFT LOW PARAPHRASE BRACKET */
  ["\u2E1D", "\u2E1D"], /* RIGHT LOW PARAPHRASE BRACKET */
  ["\u2E1E", "\u2E1E"], /* TILDE WITH DOT ABOVE */
  ["\u2E1F", "\u2E1F"], /* TILDE WITH DOT BELOW */
  ["\u2E20", "\u2E20"], /* LEFT VERTICAL BAR WITH QUILL */
  ["\u2E21", "\u2E21"], /* RIGHT VERTICAL BAR WITH QUILL */
  ["\u2E22", "\u2E22"], /* TOP LEFT HALF BRACKET */
  ["\u2E23", "\u2E23"], /* TOP RIGHT HALF BRACKET */
  ["\u2E24", "\u2E24"], /* BOTTOM LEFT HALF BRACKET */
  ["\u2E25", "\u2E25"], /* BOTTOM RIGHT HALF BRACKET */
  ["\u2E26", "\u2E26"], /* LEFT SIDEWAYS U BRACKET */
  ["\u2E27", "\u2E27"], /* RIGHT SIDEWAYS U BRACKET */
  ["\u2E28", "\u2E28"], /* LEFT DOUBLE PARENTHESIS */
  ["\u2E29", "\u2E29"], /* RIGHT DOUBLE PARENTHESIS */
  ["\u2E2A", "\u2E2A"], /* TWO DOTS OVER ONE DOT PUNCTUATION */
  ["\u2E2B", "\u2E2B"], /* ONE DOT OVER TWO DOTS PUNCTUATION */
  ["\u2E2C", "\u2E2C"], /* SQUARED FOUR DOT PUNCTUATION */
  ["\u2E2D", "\u2E2D"], /* FIVE DOT MARK */
  ["\u2E2E", "\u2E2E"], /* REVERSED QUESTION MARK */
  ["\u2E2F", "\u2E2F"], /* VERTICAL TILDE */
  ["\u2E30", "\u2E30"], /* RING POINT */
  ["\u2E31", "\u2E31"], /* WORD SEPARATOR MIDDLE DOT */
  ["\u2E32", "\u2E32"], /* TURNED COMMA */
  ["\u2E33", "\u2E33"], /* RAISED DOT */
  ["\u2E34", "\u2E34"], /* RAISED COMMA */
  ["\u2E35", "\u2E35"], /* TURNED SEMICOLON */
  ["\u2E36", "\u2E36"], /* DAGGER WITH LEFT GUARD */
  ["\u2E37", "\u2E37"], /* DAGGER WITH RIGHT GUARD */
  ["\u2E38", "\u2E38"], /* TURNED DAGGER */
  ["\u2E39", "\u2E39"], /* TOP HALF SECTION SIGN */
  ["\u2E3A", "\u2E3A"], /* TWO-EM DASH */
  ["\u2E3B", "\u2E3B"], /* THREE-EM DASH */
  ["\u2E3C", "\u2E3C"], /* STENOGRAPHIC FULL STOP */
  ["\u2E3D", "\u2E3D"], /* VERTICAL SIX DOTS */
  ["\u2E3E", "\u2E3E"], /* WIGGLY VERTICAL LINE */
  ["\u2E3F", "\u2E3F"], /* CAPITULUM */
  ["\u2E40", "\u2E40"], /* DOUBLE HYPHEN */
  ["\u2E41", "\u2E41"], /* REVERSED COMMA */
  ["\u2E42", "\u2E42"], /* DOUBLE LOW-REVERSED-9 QUOTATION MARK */
  ["\u2E43", "\u2E43"], /* DASH WITH LEFT UPTURN */
  ["\u2E44", "\u2E44"], /* DOUBLE SUSPENSION MARK */
  ["\u2E45", "\u2E45"], /* INVERTED LOW KAVYKA */
  ["\u2E46", "\u2E46"], /* INVERTED LOW KAVYKA WITH KAVYKA ABOVE */
  ["\u2E47", "\u2E47"], /* LOW KAVYKA */
  ["\u2E48", "\u2E48"], /* LOW KAVYKA WITH DOT */
  ["\u2E49", "\u2E49"], /* DOUBLE STACKED COMMA */
  ["\u2E4A", "\u2E4A"], /* DOTTED SOLIDUS */
  ["\u2E4B", "\u2E4B"], /* TRIPLE DAGGER */
  ["\u2E4C", "\u2E4C"], /* MEDIEVAL COMMA */
  ["\u2E4D", "\u2E4D"], /* PARAGRAPHUS MARK */
  ["\u2E4E", "\u2E4E"], /* PUNCTUS ELEVATUS MARK */
  ["\u2E4F", "\u2E4F"], /* CORNISH VERSE DIVIDER */
  ["\u2E50", "\u2E50"],
  ["\u2E51", "\u2E51"],
  ["\u2E52", "\u2E52"],
  ["\u2E53", "\u2E53"],
  ["\u2E54", "\u2E54"],
  ["\u2E55", "\u2E55"],
  ["\u2E56", "\u2E56"],
  ["\u2E57", "\u2E57"],
  ["\u2E58", "\u2E58"],
  ["\u2E59", "\u2E59"],
  ["\u2E5A", "\u2E5A"],
  ["\u2E5B", "\u2E5B"],
  ["\u2E5C", "\u2E5C"],
  ["\u2E5D", "\u2E5D"],
  ["\u2E5E", "\u2E5E"],
  ["\u2E5F", "\u2E5F"],
  ["\u2E60", "\u2E60"],
  ["\u2E61", "\u2E61"],
  ["\u2E62", "\u2E62"],
  ["\u2E63", "\u2E63"],
  ["\u2E64", "\u2E64"],
  ["\u2E65", "\u2E65"],
  ["\u2E66", "\u2E66"],
  ["\u2E67", "\u2E67"],
  ["\u2E68", "\u2E68"],
  ["\u2E69", "\u2E69"],
  ["\u2E6A", "\u2E6A"],
  ["\u2E6B", "\u2E6B"],
  ["\u2E6C", "\u2E6C"],
  ["\u2E6D", "\u2E6D"],
  ["\u2E6E", "\u2E6E"],
  ["\u2E6F", "\u2E6F"],
  ["\u2E70", "\u2E70"],
  ["\u2E71", "\u2E71"],
  ["\u2E72", "\u2E72"],
  ["\u2E73", "\u2E73"],
  ["\u2E74", "\u2E74"],
  ["\u2E75", "\u2E75"],
  ["\u2E76", "\u2E76"],
  ["\u2E77", "\u2E77"],
  ["\u2E78", "\u2E78"],
  ["\u2E79", "\u2E79"],
  ["\u2E7A", "\u2E7A"],
  ["\u2E7B", "\u2E7B"],
  ["\u2E7C", "\u2E7C"],
  ["\u2E7D", "\u2E7D"],
  ["\u2E7E", "\u2E7E"],
  ["\u2E7F", "\u2E7F"],
  ["\u2E80", "\u2E80"], /* CJK RADICAL REPEAT */
  ["\u2E81", "\u2E81"], /* CJK RADICAL CLIFF */
  ["\u2E82", "\u2E82"], /* CJK RADICAL SECOND ONE */
  ["\u2E83", "\u2E83"], /* CJK RADICAL SECOND TWO */
  ["\u2E84", "\u2E84"], /* CJK RADICAL SECOND THREE */
  ["\u2E85", "\u2E85"], /* CJK RADICAL PERSON */
  ["\u2E86", "\u2E86"], /* CJK RADICAL BOX */
  ["\u2E87", "\u2E87"], /* CJK RADICAL TABLE */
  ["\u2E88", "\u2E88"], /* CJK RADICAL KNIFE ONE */
  ["\u2E89", "\u2E89"], /* CJK RADICAL KNIFE TWO */
  ["\u2E8A", "\u2E8A"], /* CJK RADICAL DIVINATION */
  ["\u2E8B", "\u2E8B"], /* CJK RADICAL SEAL */
  ["\u2E8C", "\u2E8C"], /* CJK RADICAL SMALL ONE */
  ["\u2E8D", "\u2E8D"], /* CJK RADICAL SMALL TWO */
  ["\u2E8E", "\u2E8E"], /* CJK RADICAL LAME ONE */
  ["\u2E8F", "\u2E8F"], /* CJK RADICAL LAME TWO */
  ["\u2E90", "\u2E90"], /* CJK RADICAL LAME THREE */
  ["\u2E91", "\u2E91"], /* CJK RADICAL LAME FOUR */
  ["\u2E92", "\u2E92"], /* CJK RADICAL SNAKE */
  ["\u2E93", "\u2E93"], /* CJK RADICAL THREAD */
  ["\u2E94", "\u2E94"], /* CJK RADICAL SNOUT ONE */
  ["\u2E95", "\u2E95"], /* CJK RADICAL SNOUT TWO */
  ["\u2E96", "\u2E96"], /* CJK RADICAL HEART ONE */
  ["\u2E97", "\u2E97"], /* CJK RADICAL HEART TWO */
  ["\u2E98", "\u2E98"], /* CJK RADICAL HAND */
  ["\u2E99", "\u2E99"], /* CJK RADICAL RAP */
  ["\u2E9A", "\u2E9A"],
  ["\u2E9B", "\u2E9B"], /* CJK RADICAL CHOKE */
  ["\u2E9C", "\u2E9C"], /* CJK RADICAL SUN */
  ["\u2E9D", "\u2E9D"], /* CJK RADICAL MOON */
  ["\u2E9E", "\u2E9E"], /* CJK RADICAL DEATH */
  ["\u2E9F", "\u2E9F"], /* CJK RADICAL MOTHER */
  ["\u2EA0", "\u2EA0"], /* CJK RADICAL CIVILIAN */
  ["\u2EA1", "\u2EA1"], /* CJK RADICAL WATER ONE */
  ["\u2EA2", "\u2EA2"], /* CJK RADICAL WATER TWO */
  ["\u2EA3", "\u2EA3"], /* CJK RADICAL FIRE */
  ["\u2EA4", "\u2EA4"], /* CJK RADICAL PAW ONE */
  ["\u2EA5", "\u2EA5"], /* CJK RADICAL PAW TWO */
  ["\u2EA6", "\u2EA6"], /* CJK RADICAL SIMPLIFIED HALF TREE TRUNK */
  ["\u2EA7", "\u2EA7"], /* CJK RADICAL COW */
  ["\u2EA8", "\u2EA8"], /* CJK RADICAL DOG */
  ["\u2EA9", "\u2EA9"], /* CJK RADICAL JADE */
  ["\u2EAA", "\u2EAA"], /* CJK RADICAL BOLT OF CLOTH */
  ["\u2EAB", "\u2EAB"], /* CJK RADICAL EYE */
  ["\u2EAC", "\u2EAC"], /* CJK RADICAL SPIRIT ONE */
  ["\u2EAD", "\u2EAD"], /* CJK RADICAL SPIRIT TWO */
  ["\u2EAE", "\u2EAE"], /* CJK RADICAL BAMBOO */
  ["\u2EAF", "\u2EAF"], /* CJK RADICAL SILK */
  ["\u2EB0", "\u2EB0"], /* CJK RADICAL C-SIMPLIFIED SILK */
  ["\u2EB1", "\u2EB1"], /* CJK RADICAL NET ONE */
  ["\u2EB2", "\u2EB2"], /* CJK RADICAL NET TWO */
  ["\u2EB3", "\u2EB3"], /* CJK RADICAL NET THREE */
  ["\u2EB4", "\u2EB4"], /* CJK RADICAL NET FOUR */
  ["\u2EB5", "\u2EB5"], /* CJK RADICAL MESH */
  ["\u2EB6", "\u2EB6"], /* CJK RADICAL SHEEP */
  ["\u2EB7", "\u2EB7"], /* CJK RADICAL RAM */
  ["\u2EB8", "\u2EB8"], /* CJK RADICAL EWE */
  ["\u2EB9", "\u2EB9"], /* CJK RADICAL OLD */
  ["\u2EBA", "\u2EBA"], /* CJK RADICAL BRUSH ONE */
  ["\u2EBB", "\u2EBB"], /* CJK RADICAL BRUSH TWO */
  ["\u2EBC", "\u2EBC"], /* CJK RADICAL MEAT */
  ["\u2EBD", "\u2EBD"], /* CJK RADICAL MORTAR */
  ["\u2EBE", "\u2EBE"], /* CJK RADICAL GRASS ONE */
  ["\u2EBF", "\u2EBF"], /* CJK RADICAL GRASS TWO */
  ["\u2EC0", "\u2EC0"], /* CJK RADICAL GRASS THREE */
  ["\u2EC1", "\u2EC1"], /* CJK RADICAL TIGER */
  ["\u2EC2", "\u2EC2"], /* CJK RADICAL CLOTHES */
  ["\u2EC3", "\u2EC3"], /* CJK RADICAL WEST ONE */
  ["\u2EC4", "\u2EC4"], /* CJK RADICAL WEST TWO */
  ["\u2EC5", "\u2EC5"], /* CJK RADICAL C-SIMPLIFIED SEE */
  ["\u2EC6", "\u2EC6"], /* CJK RADICAL SIMPLIFIED HORN */
  ["\u2EC7", "\u2EC7"], /* CJK RADICAL HORN */
  ["\u2EC8", "\u2EC8"], /* CJK RADICAL C-SIMPLIFIED SPEECH */
  ["\u2EC9", "\u2EC9"], /* CJK RADICAL C-SIMPLIFIED SHELL */
  ["\u2ECA", "\u2ECA"], /* CJK RADICAL FOOT */
  ["\u2ECB", "\u2ECB"], /* CJK RADICAL C-SIMPLIFIED CART */
  ["\u2ECC", "\u2ECC"], /* CJK RADICAL SIMPLIFIED WALK */
  ["\u2ECD", "\u2ECD"], /* CJK RADICAL WALK ONE */
  ["\u2ECE", "\u2ECE"], /* CJK RADICAL WALK TWO */
  ["\u2ECF", "\u2ECF"], /* CJK RADICAL CITY */
  ["\u2ED0", "\u2ED0"], /* CJK RADICAL C-SIMPLIFIED GOLD */
  ["\u2ED1", "\u2ED1"], /* CJK RADICAL LONG ONE */
  ["\u2ED2", "\u2ED2"], /* CJK RADICAL LONG TWO */
  ["\u2ED3", "\u2ED3"], /* CJK RADICAL C-SIMPLIFIED LONG */
  ["\u2ED4", "\u2ED4"], /* CJK RADICAL C-SIMPLIFIED GATE */
  ["\u2ED5", "\u2ED5"], /* CJK RADICAL MOUND ONE */
  ["\u2ED6", "\u2ED6"], /* CJK RADICAL MOUND TWO */
  ["\u2ED7", "\u2ED7"], /* CJK RADICAL RAIN */
  ["\u2ED8", "\u2ED8"], /* CJK RADICAL BLUE */
  ["\u2ED9", "\u2ED9"], /* CJK RADICAL C-SIMPLIFIED TANNED LEATHER */
  ["\u2EDA", "\u2EDA"], /* CJK RADICAL C-SIMPLIFIED LEAF */
  ["\u2EDB", "\u2EDB"], /* CJK RADICAL C-SIMPLIFIED WIND */
  ["\u2EDC", "\u2EDC"], /* CJK RADICAL C-SIMPLIFIED FLY */
  ["\u2EDD", "\u2EDD"], /* CJK RADICAL EAT ONE */
  ["\u2EDE", "\u2EDE"], /* CJK RADICAL EAT TWO */
  ["\u2EDF", "\u2EDF"], /* CJK RADICAL EAT THREE */
  ["\u2EE0", "\u2EE0"], /* CJK RADICAL C-SIMPLIFIED EAT */
  ["\u2EE1", "\u2EE1"], /* CJK RADICAL HEAD */
  ["\u2EE2", "\u2EE2"], /* CJK RADICAL C-SIMPLIFIED HORSE */
  ["\u2EE3", "\u2EE3"], /* CJK RADICAL BONE */
  ["\u2EE4", "\u2EE4"], /* CJK RADICAL GHOST */
  ["\u2EE5", "\u2EE5"], /* CJK RADICAL C-SIMPLIFIED FISH */
  ["\u2EE6", "\u2EE6"], /* CJK RADICAL C-SIMPLIFIED BIRD */
  ["\u2EE7", "\u2EE7"], /* CJK RADICAL C-SIMPLIFIED SALT */
  ["\u2EE8", "\u2EE8"], /* CJK RADICAL SIMPLIFIED WHEAT */
  ["\u2EE9", "\u2EE9"], /* CJK RADICAL SIMPLIFIED YELLOW */
  ["\u2EEA", "\u2EEA"], /* CJK RADICAL C-SIMPLIFIED FROG */
  ["\u2EEB", "\u2EEB"], /* CJK RADICAL J-SIMPLIFIED EVEN */
  ["\u2EEC", "\u2EEC"], /* CJK RADICAL C-SIMPLIFIED EVEN */
  ["\u2EED", "\u2EED"], /* CJK RADICAL J-SIMPLIFIED TOOTH */
  ["\u2EEE", "\u2EEE"], /* CJK RADICAL C-SIMPLIFIED TOOTH */
  ["\u2EEF", "\u2EEF"], /* CJK RADICAL J-SIMPLIFIED DRAGON */
  ["\u2EF0", "\u2EF0"], /* CJK RADICAL C-SIMPLIFIED DRAGON */
  ["\u2EF1", "\u2EF1"], /* CJK RADICAL TURTLE */
  ["\u2EF2", "\u2EF2"], /* CJK RADICAL J-SIMPLIFIED TURTLE */
  ["\u2EF3", "\u2EF3"], /* CJK RADICAL C-SIMPLIFIED TURTLE */
  ["\u2EF4", "\u2EF4"],
  ["\u2EF5", "\u2EF5"],
  ["\u2EF6", "\u2EF6"],
  ["\u2EF7", "\u2EF7"],
  ["\u2EF8", "\u2EF8"],
  ["\u2EF9", "\u2EF9"],
  ["\u2EFA", "\u2EFA"],
  ["\u2EFB", "\u2EFB"],
  ["\u2EFC", "\u2EFC"],
  ["\u2EFD", "\u2EFD"],
  ["\u2EFE", "\u2EFE"],
  ["\u2EFF", "\u2EFF"],
  ["\u2F00", "\u2F00"], /* KANGXI RADICAL ONE */
  ["\u2F01", "\u2F01"], /* KANGXI RADICAL LINE */
  ["\u2F02", "\u2F02"], /* KANGXI RADICAL DOT */
  ["\u2F03", "\u2F03"], /* KANGXI RADICAL SLASH */
  ["\u2F04", "\u2F04"], /* KANGXI RADICAL SECOND */
  ["\u2F05", "\u2F05"], /* KANGXI RADICAL HOOK */
  ["\u2F06", "\u2F06"], /* KANGXI RADICAL TWO */
  ["\u2F07", "\u2F07"], /* KANGXI RADICAL LID */
  ["\u2F08", "\u2F08"], /* KANGXI RADICAL MAN */
  ["\u2F09", "\u2F09"], /* KANGXI RADICAL LEGS */
  ["\u2F0A", "\u2F0A"], /* KANGXI RADICAL ENTER */
  ["\u2F0B", "\u2F0B"], /* KANGXI RADICAL EIGHT */
  ["\u2F0C", "\u2F0C"], /* KANGXI RADICAL DOWN BOX */
  ["\u2F0D", "\u2F0D"], /* KANGXI RADICAL COVER */
  ["\u2F0E", "\u2F0E"], /* KANGXI RADICAL ICE */
  ["\u2F0F", "\u2F0F"], /* KANGXI RADICAL TABLE */
  ["\u2F10", "\u2F10"], /* KANGXI RADICAL OPEN BOX */
  ["\u2F11", "\u2F11"], /* KANGXI RADICAL KNIFE */
  ["\u2F12", "\u2F12"], /* KANGXI RADICAL POWER */
  ["\u2F13", "\u2F13"], /* KANGXI RADICAL WRAP */
  ["\u2F14", "\u2F14"], /* KANGXI RADICAL SPOON */
  ["\u2F15", "\u2F15"], /* KANGXI RADICAL RIGHT OPEN BOX */
  ["\u2F16", "\u2F16"], /* KANGXI RADICAL HIDING ENCLOSURE */
  ["\u2F17", "\u2F17"], /* KANGXI RADICAL TEN */
  ["\u2F18", "\u2F18"], /* KANGXI RADICAL DIVINATION */
  ["\u2F19", "\u2F19"], /* KANGXI RADICAL SEAL */
  ["\u2F1A", "\u2F1A"], /* KANGXI RADICAL CLIFF */
  ["\u2F1B", "\u2F1B"], /* KANGXI RADICAL PRIVATE */
  ["\u2F1C", "\u2F1C"], /* KANGXI RADICAL AGAIN */
  ["\u2F1D", "\u2F1D"], /* KANGXI RADICAL MOUTH */
  ["\u2F1E", "\u2F1E"], /* KANGXI RADICAL ENCLOSURE */
  ["\u2F1F", "\u2F1F"], /* KANGXI RADICAL EARTH */
  ["\u2F20", "\u2F20"], /* KANGXI RADICAL SCHOLAR */
  ["\u2F21", "\u2F21"], /* KANGXI RADICAL GO */
  ["\u2F22", "\u2F22"], /* KANGXI RADICAL GO SLOWLY */
  ["\u2F23", "\u2F23"], /* KANGXI RADICAL EVENING */
  ["\u2F24", "\u2F24"], /* KANGXI RADICAL BIG */
  ["\u2F25", "\u2F25"], /* KANGXI RADICAL WOMAN */
  ["\u2F26", "\u2F26"], /* KANGXI RADICAL CHILD */
  ["\u2F27", "\u2F27"], /* KANGXI RADICAL ROOF */
  ["\u2F28", "\u2F28"], /* KANGXI RADICAL INCH */
  ["\u2F29", "\u2F29"], /* KANGXI RADICAL SMALL */
  ["\u2F2A", "\u2F2A"], /* KANGXI RADICAL LAME */
  ["\u2F2B", "\u2F2B"], /* KANGXI RADICAL CORPSE */
  ["\u2F2C", "\u2F2C"], /* KANGXI RADICAL SPROUT */
  ["\u2F2D", "\u2F2D"], /* KANGXI RADICAL MOUNTAIN */
  ["\u2F2E", "\u2F2E"], /* KANGXI RADICAL RIVER */
  ["\u2F2F", "\u2F2F"], /* KANGXI RADICAL WORK */
  ["\u2F30", "\u2F30"], /* KANGXI RADICAL ONESELF */
  ["\u2F31", "\u2F31"], /* KANGXI RADICAL TURBAN */
  ["\u2F32", "\u2F32"], /* KANGXI RADICAL DRY */
  ["\u2F33", "\u2F33"], /* KANGXI RADICAL SHORT THREAD */
  ["\u2F34", "\u2F34"], /* KANGXI RADICAL DOTTED CLIFF */
  ["\u2F35", "\u2F35"], /* KANGXI RADICAL LONG STRIDE */
  ["\u2F36", "\u2F36"], /* KANGXI RADICAL TWO HANDS */
  ["\u2F37", "\u2F37"], /* KANGXI RADICAL SHOOT */
  ["\u2F38", "\u2F38"], /* KANGXI RADICAL BOW */
  ["\u2F39", "\u2F39"], /* KANGXI RADICAL SNOUT */
  ["\u2F3A", "\u2F3A"], /* KANGXI RADICAL BRISTLE */
  ["\u2F3B", "\u2F3B"], /* KANGXI RADICAL STEP */
  ["\u2F3C", "\u2F3C"], /* KANGXI RADICAL HEART */
  ["\u2F3D", "\u2F3D"], /* KANGXI RADICAL HALBERD */
  ["\u2F3E", "\u2F3E"], /* KANGXI RADICAL DOOR */
  ["\u2F3F", "\u2F3F"], /* KANGXI RADICAL HAND */
  ["\u2F40", "\u2F40"], /* KANGXI RADICAL BRANCH */
  ["\u2F41", "\u2F41"], /* KANGXI RADICAL RAP */
  ["\u2F42", "\u2F42"], /* KANGXI RADICAL SCRIPT */
  ["\u2F43", "\u2F43"], /* KANGXI RADICAL DIPPER */
  ["\u2F44", "\u2F44"], /* KANGXI RADICAL AXE */
  ["\u2F45", "\u2F45"], /* KANGXI RADICAL SQUARE */
  ["\u2F46", "\u2F46"], /* KANGXI RADICAL NOT */
  ["\u2F47", "\u2F47"], /* KANGXI RADICAL SUN */
  ["\u2F48", "\u2F48"], /* KANGXI RADICAL SAY */
  ["\u2F49", "\u2F49"], /* KANGXI RADICAL MOON */
  ["\u2F4A", "\u2F4A"], /* KANGXI RADICAL TREE */
  ["\u2F4B", "\u2F4B"], /* KANGXI RADICAL LACK */
  ["\u2F4C", "\u2F4C"], /* KANGXI RADICAL STOP */
  ["\u2F4D", "\u2F4D"], /* KANGXI RADICAL DEATH */
  ["\u2F4E", "\u2F4E"], /* KANGXI RADICAL WEAPON */
  ["\u2F4F", "\u2F4F"], /* KANGXI RADICAL DO NOT */
  ["\u2F50", "\u2F50"], /* KANGXI RADICAL COMPARE */
  ["\u2F51", "\u2F51"], /* KANGXI RADICAL FUR */
  ["\u2F52", "\u2F52"], /* KANGXI RADICAL CLAN */
  ["\u2F53", "\u2F53"], /* KANGXI RADICAL STEAM */
  ["\u2F54", "\u2F54"], /* KANGXI RADICAL WATER */
  ["\u2F55", "\u2F55"], /* KANGXI RADICAL FIRE */
  ["\u2F56", "\u2F56"], /* KANGXI RADICAL CLAW */
  ["\u2F57", "\u2F57"], /* KANGXI RADICAL FATHER */
  ["\u2F58", "\u2F58"], /* KANGXI RADICAL DOUBLE X */
  ["\u2F59", "\u2F59"], /* KANGXI RADICAL HALF TREE TRUNK */
  ["\u2F5A", "\u2F5A"], /* KANGXI RADICAL SLICE */
  ["\u2F5B", "\u2F5B"], /* KANGXI RADICAL FANG */
  ["\u2F5C", "\u2F5C"], /* KANGXI RADICAL COW */
  ["\u2F5D", "\u2F5D"], /* KANGXI RADICAL DOG */
  ["\u2F5E", "\u2F5E"], /* KANGXI RADICAL PROFOUND */
  ["\u2F5F", "\u2F5F"], /* KANGXI RADICAL JADE */
  ["\u2F60", "\u2F60"], /* KANGXI RADICAL MELON */
  ["\u2F61", "\u2F61"], /* KANGXI RADICAL TILE */
  ["\u2F62", "\u2F62"], /* KANGXI RADICAL SWEET */
  ["\u2F63", "\u2F63"], /* KANGXI RADICAL LIFE */
  ["\u2F64", "\u2F64"], /* KANGXI RADICAL USE */
  ["\u2F65", "\u2F65"], /* KANGXI RADICAL FIELD */
  ["\u2F66", "\u2F66"], /* KANGXI RADICAL BOLT OF CLOTH */
  ["\u2F67", "\u2F67"], /* KANGXI RADICAL SICKNESS */
  ["\u2F68", "\u2F68"], /* KANGXI RADICAL DOTTED TENT */
  ["\u2F69", "\u2F69"], /* KANGXI RADICAL WHITE */
  ["\u2F6A", "\u2F6A"], /* KANGXI RADICAL SKIN */
  ["\u2F6B", "\u2F6B"], /* KANGXI RADICAL DISH */
  ["\u2F6C", "\u2F6C"], /* KANGXI RADICAL EYE */
  ["\u2F6D", "\u2F6D"], /* KANGXI RADICAL SPEAR */
  ["\u2F6E", "\u2F6E"], /* KANGXI RADICAL ARROW */
  ["\u2F6F", "\u2F6F"], /* KANGXI RADICAL STONE */
  ["\u2F70", "\u2F70"], /* KANGXI RADICAL SPIRIT */
  ["\u2F71", "\u2F71"], /* KANGXI RADICAL TRACK */
  ["\u2F72", "\u2F72"], /* KANGXI RADICAL GRAIN */
  ["\u2F73", "\u2F73"], /* KANGXI RADICAL CAVE */
  ["\u2F74", "\u2F74"], /* KANGXI RADICAL STAND */
  ["\u2F75", "\u2F75"], /* KANGXI RADICAL BAMBOO */
  ["\u2F76", "\u2F76"], /* KANGXI RADICAL RICE */
  ["\u2F77", "\u2F77"], /* KANGXI RADICAL SILK */
  ["\u2F78", "\u2F78"], /* KANGXI RADICAL JAR */
  ["\u2F79", "\u2F79"], /* KANGXI RADICAL NET */
  ["\u2F7A", "\u2F7A"], /* KANGXI RADICAL SHEEP */
  ["\u2F7B", "\u2F7B"], /* KANGXI RADICAL FEATHER */
  ["\u2F7C", "\u2F7C"], /* KANGXI RADICAL OLD */
  ["\u2F7D", "\u2F7D"], /* KANGXI RADICAL AND */
  ["\u2F7E", "\u2F7E"], /* KANGXI RADICAL PLOW */
  ["\u2F7F", "\u2F7F"], /* KANGXI RADICAL EAR */
  ["\u2F80", "\u2F80"], /* KANGXI RADICAL BRUSH */
  ["\u2F81", "\u2F81"], /* KANGXI RADICAL MEAT */
  ["\u2F82", "\u2F82"], /* KANGXI RADICAL MINISTER */
  ["\u2F83", "\u2F83"], /* KANGXI RADICAL SELF */
  ["\u2F84", "\u2F84"], /* KANGXI RADICAL ARRIVE */
  ["\u2F85", "\u2F85"], /* KANGXI RADICAL MORTAR */
  ["\u2F86", "\u2F86"], /* KANGXI RADICAL TONGUE */
  ["\u2F87", "\u2F87"], /* KANGXI RADICAL OPPOSE */
  ["\u2F88", "\u2F88"], /* KANGXI RADICAL BOAT */
  ["\u2F89", "\u2F89"], /* KANGXI RADICAL STOPPING */
  ["\u2F8A", "\u2F8A"], /* KANGXI RADICAL COLOR */
  ["\u2F8B", "\u2F8B"], /* KANGXI RADICAL GRASS */
  ["\u2F8C", "\u2F8C"], /* KANGXI RADICAL TIGER */
  ["\u2F8D", "\u2F8D"], /* KANGXI RADICAL INSECT */
  ["\u2F8E", "\u2F8E"], /* KANGXI RADICAL BLOOD */
  ["\u2F8F", "\u2F8F"], /* KANGXI RADICAL WALK ENCLOSURE */
  ["\u2F90", "\u2F90"], /* KANGXI RADICAL CLOTHES */
  ["\u2F91", "\u2F91"], /* KANGXI RADICAL WEST */
  ["\u2F92", "\u2F92"], /* KANGXI RADICAL SEE */
  ["\u2F93", "\u2F93"], /* KANGXI RADICAL HORN */
  ["\u2F94", "\u2F94"], /* KANGXI RADICAL SPEECH */
  ["\u2F95", "\u2F95"], /* KANGXI RADICAL VALLEY */
  ["\u2F96", "\u2F96"], /* KANGXI RADICAL BEAN */
  ["\u2F97", "\u2F97"], /* KANGXI RADICAL PIG */
  ["\u2F98", "\u2F98"], /* KANGXI RADICAL BADGER */
  ["\u2F99", "\u2F99"], /* KANGXI RADICAL SHELL */
  ["\u2F9A", "\u2F9A"], /* KANGXI RADICAL RED */
  ["\u2F9B", "\u2F9B"], /* KANGXI RADICAL RUN */
  ["\u2F9C", "\u2F9C"], /* KANGXI RADICAL FOOT */
  ["\u2F9D", "\u2F9D"], /* KANGXI RADICAL BODY */
  ["\u2F9E", "\u2F9E"], /* KANGXI RADICAL CART */
  ["\u2F9F", "\u2F9F"], /* KANGXI RADICAL BITTER */
  ["\u2FA0", "\u2FA0"], /* KANGXI RADICAL MORNING */
  ["\u2FA1", "\u2FA1"], /* KANGXI RADICAL WALK */
  ["\u2FA2", "\u2FA2"], /* KANGXI RADICAL CITY */
  ["\u2FA3", "\u2FA3"], /* KANGXI RADICAL WINE */
  ["\u2FA4", "\u2FA4"], /* KANGXI RADICAL DISTINGUISH */
  ["\u2FA5", "\u2FA5"], /* KANGXI RADICAL VILLAGE */
  ["\u2FA6", "\u2FA6"], /* KANGXI RADICAL GOLD */
  ["\u2FA7", "\u2FA7"], /* KANGXI RADICAL LONG */
  ["\u2FA8", "\u2FA8"], /* KANGXI RADICAL GATE */
  ["\u2FA9", "\u2FA9"], /* KANGXI RADICAL MOUND */
  ["\u2FAA", "\u2FAA"], /* KANGXI RADICAL SLAVE */
  ["\u2FAB", "\u2FAB"], /* KANGXI RADICAL SHORT TAILED BIRD */
  ["\u2FAC", "\u2FAC"], /* KANGXI RADICAL RAIN */
  ["\u2FAD", "\u2FAD"], /* KANGXI RADICAL BLUE */
  ["\u2FAE", "\u2FAE"], /* KANGXI RADICAL WRONG */
  ["\u2FAF", "\u2FAF"], /* KANGXI RADICAL FACE */
  ["\u2FB0", "\u2FB0"], /* KANGXI RADICAL LEATHER */
  ["\u2FB1", "\u2FB1"], /* KANGXI RADICAL TANNED LEATHER */
  ["\u2FB2", "\u2FB2"], /* KANGXI RADICAL LEEK */
  ["\u2FB3", "\u2FB3"], /* KANGXI RADICAL SOUND */
  ["\u2FB4", "\u2FB4"], /* KANGXI RADICAL LEAF */
  ["\u2FB5", "\u2FB5"], /* KANGXI RADICAL WIND */
  ["\u2FB6", "\u2FB6"], /* KANGXI RADICAL FLY */
  ["\u2FB7", "\u2FB7"], /* KANGXI RADICAL EAT */
  ["\u2FB8", "\u2FB8"], /* KANGXI RADICAL HEAD */
  ["\u2FB9", "\u2FB9"], /* KANGXI RADICAL FRAGRANT */
  ["\u2FBA", "\u2FBA"], /* KANGXI RADICAL HORSE */
  ["\u2FBB", "\u2FBB"], /* KANGXI RADICAL BONE */
  ["\u2FBC", "\u2FBC"], /* KANGXI RADICAL TALL */
  ["\u2FBD", "\u2FBD"], /* KANGXI RADICAL HAIR */
  ["\u2FBE", "\u2FBE"], /* KANGXI RADICAL FIGHT */
  ["\u2FBF", "\u2FBF"], /* KANGXI RADICAL SACRIFICIAL WINE */
  ["\u2FC0", "\u2FC0"], /* KANGXI RADICAL CAULDRON */
  ["\u2FC1", "\u2FC1"], /* KANGXI RADICAL GHOST */
  ["\u2FC2", "\u2FC2"], /* KANGXI RADICAL FISH */
  ["\u2FC3", "\u2FC3"], /* KANGXI RADICAL BIRD */
  ["\u2FC4", "\u2FC4"], /* KANGXI RADICAL SALT */
  ["\u2FC5", "\u2FC5"], /* KANGXI RADICAL DEER */
  ["\u2FC6", "\u2FC6"], /* KANGXI RADICAL WHEAT */
  ["\u2FC7", "\u2FC7"], /* KANGXI RADICAL HEMP */
  ["\u2FC8", "\u2FC8"], /* KANGXI RADICAL YELLOW */
  ["\u2FC9", "\u2FC9"], /* KANGXI RADICAL MILLET */
  ["\u2FCA", "\u2FCA"], /* KANGXI RADICAL BLACK */
  ["\u2FCB", "\u2FCB"], /* KANGXI RADICAL EMBROIDERY */
  ["\u2FCC", "\u2FCC"], /* KANGXI RADICAL FROG */
  ["\u2FCD", "\u2FCD"], /* KANGXI RADICAL TRIPOD */
  ["\u2FCE", "\u2FCE"], /* KANGXI RADICAL DRUM */
  ["\u2FCF", "\u2FCF"], /* KANGXI RADICAL RAT */
  ["\u2FD0", "\u2FD0"], /* KANGXI RADICAL NOSE */
  ["\u2FD1", "\u2FD1"], /* KANGXI RADICAL EVEN */
  ["\u2FD2", "\u2FD2"], /* KANGXI RADICAL TOOTH */
  ["\u2FD3", "\u2FD3"], /* KANGXI RADICAL DRAGON */
  ["\u2FD4", "\u2FD4"], /* KANGXI RADICAL TURTLE */
  ["\u2FD5", "\u2FD5"], /* KANGXI RADICAL FLUTE */
  ["\u2FD6", "\u2FD6"],
  ["\u2FD7", "\u2FD7"],
  ["\u2FD8", "\u2FD8"],
  ["\u2FD9", "\u2FD9"],
  ["\u2FDA", "\u2FDA"],
  ["\u2FDB", "\u2FDB"],
  ["\u2FDC", "\u2FDC"],
  ["\u2FDD", "\u2FDD"],
  ["\u2FDE", "\u2FDE"],
  ["\u2FDF", "\u2FDF"],
  ["\u2FE0", "\u2FE0"],
  ["\u2FE1", "\u2FE1"],
  ["\u2FE2", "\u2FE2"],
  ["\u2FE3", "\u2FE3"],
  ["\u2FE4", "\u2FE4"],
  ["\u2FE5", "\u2FE5"],
  ["\u2FE6", "\u2FE6"],
  ["\u2FE7", "\u2FE7"],
  ["\u2FE8", "\u2FE8"],
  ["\u2FE9", "\u2FE9"],
  ["\u2FEA", "\u2FEA"],
  ["\u2FEB", "\u2FEB"],
  ["\u2FEC", "\u2FEC"],
  ["\u2FED", "\u2FED"],
  ["\u2FEE", "\u2FEE"],
  ["\u2FEF", "\u2FEF"],
  ["\u2FF0", "\u2FF0"], /* IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO RIGHT */
  ["\u2FF1", "\u2FF1"], /* IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO BELOW */
  ["\u2FF2", "\u2FF2"], /* IDEOGRAPHIC DESCRIPTION CHARACTER LEFT TO MIDDLE AND RIGHT */
  ["\u2FF3", "\u2FF3"], /* IDEOGRAPHIC DESCRIPTION CHARACTER ABOVE TO MIDDLE AND BELOW */
  ["\u2FF4", "\u2FF4"], /* IDEOGRAPHIC DESCRIPTION CHARACTER FULL SURROUND */
  ["\u2FF5", "\u2FF5"], /* IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM ABOVE */
  ["\u2FF6", "\u2FF6"], /* IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM BELOW */
  ["\u2FF7", "\u2FF7"], /* IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LEFT */
  ["\u2FF8", "\u2FF8"], /* IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER LEFT */
  ["\u2FF9", "\u2FF9"], /* IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM UPPER RIGHT */
  ["\u2FFA", "\u2FFA"], /* IDEOGRAPHIC DESCRIPTION CHARACTER SURROUND FROM LOWER LEFT */
  ["\u2FFB", "\u2FFB"], /* IDEOGRAPHIC DESCRIPTION CHARACTER OVERLAID */
  ["\u2FFC", "\u2FFC"],
  ["\u2FFD", "\u2FFD"],
  ["\u2FFE", "\u2FFE"],
  ["\u2FFF", "\u2FFF"],
  ["\u3000", "\u3000"], /* IDEOGRAPHIC SPACE */
  ["\u3001", "\u3001"], /* IDEOGRAPHIC COMMA */
  ["\u3002", "\u3002"], /* IDEOGRAPHIC FULL STOP (IDEOGRAPHIC PERIOD) */
  ["\u3003", "\u3003"], /* DITTO MARK */
  ["\u3004", "\u3004"], /* JAPANESE INDUSTRIAL STANDARD SYMBOL */
  ["\u3005", "\u3005"], /* IDEOGRAPHIC ITERATION MARK */
  ["\u3006", "\u3006"], /* IDEOGRAPHIC CLOSING MARK */
  ["\u3007", "\u3007"], /* IDEOGRAPHIC NUMBER ZERO */
  ["\u3008", "\u3008"], /* LEFT ANGLE BRACKET (OPENING ANGLE BRACKET) */
  ["\u3009", "\u3009"], /* RIGHT ANGLE BRACKET (CLOSING ANGLE BRACKET) */
  ["\u300A", "\u300A"], /* LEFT DOUBLE ANGLE BRACKET (OPENING DOUBLE ANGLE BRACKET) */
  ["\u300B", "\u300B"], /* RIGHT DOUBLE ANGLE BRACKET (CLOSING DOUBLE ANGLE BRACKET) */
  ["\u300C", "\u300C"], /* LEFT CORNER BRACKET (OPENING CORNER BRACKET) */
  ["\u300D", "\u300D"], /* RIGHT CORNER BRACKET (CLOSING CORNER BRACKET) */
  ["\u300E", "\u300E"], /* LEFT WHITE CORNER BRACKET (OPENING WHITE CORNER BRACKET) */
  ["\u300F", "\u300F"], /* RIGHT WHITE CORNER BRACKET (CLOSING WHITE CORNER BRACKET) */
  ["\u3010", "\u3010"], /* LEFT BLACK LENTICULAR BRACKET (OPENING BLACK LENTICULAR BRACKET) */
  ["\u3011", "\u3011"], /* RIGHT BLACK LENTICULAR BRACKET (CLOSING BLACK LENTICULAR BRACKET) */
  ["\u3012", "\u3012"], /* POSTAL MARK */
  ["\u3013", "\u3013"], /* GETA MARK */
  ["\u3014", "\u3014"], /* LEFT TORTOISE SHELL BRACKET (OPENING TORTOISE SHELL BRACKET) */
  ["\u3015", "\u3015"], /* RIGHT TORTOISE SHELL BRACKET (CLOSING TORTOISE SHELL BRACKET) */
  ["\u3016", "\u3016"], /* LEFT WHITE LENTICULAR BRACKET (OPENING WHITE LENTICULAR BRACKET) */
  ["\u3017", "\u3017"], /* RIGHT WHITE LENTICULAR BRACKET (CLOSING WHITE LENTICULAR BRACKET) */
  ["\u3018", "\u3018"], /* LEFT WHITE TORTOISE SHELL BRACKET (OPENING WHITE TORTOISE SHELL BRACKET) */
  ["\u3019", "\u3019"], /* RIGHT WHITE TORTOISE SHELL BRACKET (CLOSING WHITE TORTOISE SHELL BRACKET) */
  ["\u301A", "\u301A"], /* LEFT WHITE SQUARE BRACKET (OPENING WHITE SQUARE BRACKET) */
  ["\u301B", "\u301B"], /* RIGHT WHITE SQUARE BRACKET (CLOSING WHITE SQUARE BRACKET) */
  ["\u301C", "\u301C"], /* WAVE DASH */
  ["\u301D", "\u301D"], /* REVERSED DOUBLE PRIME QUOTATION MARK */
  ["\u301E", "\u301E"], /* DOUBLE PRIME QUOTATION MARK */
  ["\u301F", "\u301F"], /* LOW DOUBLE PRIME QUOTATION MARK */
  ["\u3020", "\u3020"], /* POSTAL MARK FACE */
  ["\u3021", "\u3021"], /* HANGZHOU NUMERAL ONE */
  ["\u3022", "\u3022"], /* HANGZHOU NUMERAL TWO */
  ["\u3023", "\u3023"], /* HANGZHOU NUMERAL THREE */
  ["\u3024", "\u3024"], /* HANGZHOU NUMERAL FOUR */
  ["\u3025", "\u3025"], /* HANGZHOU NUMERAL FIVE */
  ["\u3026", "\u3026"], /* HANGZHOU NUMERAL SIX */
  ["\u3027", "\u3027"], /* HANGZHOU NUMERAL SEVEN */
  ["\u3028", "\u3028"], /* HANGZHOU NUMERAL EIGHT */
  ["\u3029", "\u3029"], /* HANGZHOU NUMERAL NINE */
  ["\u302A", "\u302A"], /* IDEOGRAPHIC LEVEL TONE MARK */
  ["\u302B", "\u302B"], /* IDEOGRAPHIC RISING TONE MARK */
  ["\u302C", "\u302C"], /* IDEOGRAPHIC DEPARTING TONE MARK */
  ["\u302D", "\u302D"], /* IDEOGRAPHIC ENTERING TONE MARK */
  ["\u302E", "\u302E"], /* HANGUL SINGLE DOT TONE MARK */
  ["\u302F", "\u302F"], /* HANGUL DOUBLE DOT TONE MARK */
  ["\u3030", "\u3030"], /* WAVY DASH */
  ["\u3031", "\u3031"], /* VERTICAL KANA REPEAT MARK */
  ["\u3032", "\u3032"], /* VERTICAL KANA REPEAT WITH VOICED SOUND MARK */
  ["\u3033", "\u3033"], /* VERTICAL KANA REPEAT MARK UPPER HALF */
  ["\u3034", "\u3034"], /* VERTICAL KANA REPEAT WITH VOICED SOUND MARK UPPER HALF */
  ["\u3035", "\u3035"], /* VERTICAL KANA REPEAT MARK LOWER HALF */
  ["\u3036", "\u3036"], /* CIRCLED POSTAL MARK */
  ["\u3037", "\u3037"], /* IDEOGRAPHIC TELEGRAPH LINE FEED SEPARATOR SYMBOL */
  ["\u3038", "\u3038"], /* HANGZHOU NUMERAL TEN */
  ["\u3039", "\u3039"], /* HANGZHOU NUMERAL TWENTY */
  ["\u303A", "\u303A"], /* HANGZHOU NUMERAL THIRTY */
  ["\u303B", "\u303B"], /* VERTICAL IDEOGRAPHIC ITERATION MARK */
  ["\u303C", "\u303C"], /* MASU MARK */
  ["\u303D", "\u303D"], /* PART ALTERNATION MARK */
  ["\u303E", "\u303E"], /* IDEOGRAPHIC VARIATION INDICATOR */
  ["\u303F", "\u303F"], /* IDEOGRAPHIC HALF FILL SPACE */
  ["\u3040", "\u3040"],
  ["\u3041", "\u3041"], /* HIRAGANA LETTER SMALL A */
  ["\u3042", "\u3042"], /* HIRAGANA LETTER A */
  ["\u3043", "\u3043"], /* HIRAGANA LETTER SMALL I */
  ["\u3044", "\u3044"], /* HIRAGANA LETTER I */
  ["\u3045", "\u3045"], /* HIRAGANA LETTER SMALL U */
  ["\u3046", "\u3046"], /* HIRAGANA LETTER U */
  ["\u3047", "\u3047"], /* HIRAGANA LETTER SMALL E */
  ["\u3048", "\u3048"], /* HIRAGANA LETTER E */
  ["\u3049", "\u3049"], /* HIRAGANA LETTER SMALL O */
  ["\u304A", "\u304A"], /* HIRAGANA LETTER O */
  ["\u304B", "\u304B"], /* HIRAGANA LETTER KA */
  ["\u304C", "\u304C"], /* HIRAGANA LETTER GA */
  ["\u304D", "\u304D"], /* HIRAGANA LETTER KI */
  ["\u304E", "\u304E"], /* HIRAGANA LETTER GI */
  ["\u304F", "\u304F"], /* HIRAGANA LETTER KU */
  ["\u3050", "\u3050"], /* HIRAGANA LETTER GU */
  ["\u3051", "\u3051"], /* HIRAGANA LETTER KE */
  ["\u3052", "\u3052"], /* HIRAGANA LETTER GE */
  ["\u3053", "\u3053"], /* HIRAGANA LETTER KO */
  ["\u3054", "\u3054"], /* HIRAGANA LETTER GO */
  ["\u3055", "\u3055"], /* HIRAGANA LETTER SA */
  ["\u3056", "\u3056"], /* HIRAGANA LETTER ZA */
  ["\u3057", "\u3057"], /* HIRAGANA LETTER SI */
  ["\u3058", "\u3058"], /* HIRAGANA LETTER ZI */
  ["\u3059", "\u3059"], /* HIRAGANA LETTER SU */
  ["\u305A", "\u305A"], /* HIRAGANA LETTER ZU */
  ["\u305B", "\u305B"], /* HIRAGANA LETTER SE */
  ["\u305C", "\u305C"], /* HIRAGANA LETTER ZE */
  ["\u305D", "\u305D"], /* HIRAGANA LETTER SO */
  ["\u305E", "\u305E"], /* HIRAGANA LETTER ZO */
  ["\u305F", "\u305F"], /* HIRAGANA LETTER TA */
  ["\u3060", "\u3060"], /* HIRAGANA LETTER DA */
  ["\u3061", "\u3061"], /* HIRAGANA LETTER TI */
  ["\u3062", "\u3062"], /* HIRAGANA LETTER DI */
  ["\u3063", "\u3063"], /* HIRAGANA LETTER SMALL TU */
  ["\u3064", "\u3064"], /* HIRAGANA LETTER TU */
  ["\u3065", "\u3065"], /* HIRAGANA LETTER DU */
  ["\u3066", "\u3066"], /* HIRAGANA LETTER TE */
  ["\u3067", "\u3067"], /* HIRAGANA LETTER DE */
  ["\u3068", "\u3068"], /* HIRAGANA LETTER TO */
  ["\u3069", "\u3069"], /* HIRAGANA LETTER DO */
  ["\u306A", "\u306A"], /* HIRAGANA LETTER NA */
  ["\u306B", "\u306B"], /* HIRAGANA LETTER NI */
  ["\u306C", "\u306C"], /* HIRAGANA LETTER NU */
  ["\u306D", "\u306D"], /* HIRAGANA LETTER NE */
  ["\u306E", "\u306E"], /* HIRAGANA LETTER NO */
  ["\u306F", "\u306F"], /* HIRAGANA LETTER HA */
  ["\u3070", "\u3070"], /* HIRAGANA LETTER BA */
  ["\u3071", "\u3071"], /* HIRAGANA LETTER PA */
  ["\u3072", "\u3072"], /* HIRAGANA LETTER HI */
  ["\u3073", "\u3073"], /* HIRAGANA LETTER BI */
  ["\u3074", "\u3074"], /* HIRAGANA LETTER PI */
  ["\u3075", "\u3075"], /* HIRAGANA LETTER HU */
  ["\u3076", "\u3076"], /* HIRAGANA LETTER BU */
  ["\u3077", "\u3077"], /* HIRAGANA LETTER PU */
  ["\u3078", "\u3078"], /* HIRAGANA LETTER HE */
  ["\u3079", "\u3079"], /* HIRAGANA LETTER BE */
  ["\u307A", "\u307A"], /* HIRAGANA LETTER PE */
  ["\u307B", "\u307B"], /* HIRAGANA LETTER HO */
  ["\u307C", "\u307C"], /* HIRAGANA LETTER BO */
  ["\u307D", "\u307D"], /* HIRAGANA LETTER PO */
  ["\u307E", "\u307E"], /* HIRAGANA LETTER MA */
  ["\u307F", "\u307F"], /* HIRAGANA LETTER MI */
  ["\u3080", "\u3080"], /* HIRAGANA LETTER MU */
  ["\u3081", "\u3081"], /* HIRAGANA LETTER ME */
  ["\u3082", "\u3082"], /* HIRAGANA LETTER MO */
  ["\u3083", "\u3083"], /* HIRAGANA LETTER SMALL YA */
  ["\u3084", "\u3084"], /* HIRAGANA LETTER YA */
  ["\u3085", "\u3085"], /* HIRAGANA LETTER SMALL YU */
  ["\u3086", "\u3086"], /* HIRAGANA LETTER YU */
  ["\u3087", "\u3087"], /* HIRAGANA LETTER SMALL YO */
  ["\u3088", "\u3088"], /* HIRAGANA LETTER YO */
  ["\u3089", "\u3089"], /* HIRAGANA LETTER RA */
  ["\u308A", "\u308A"], /* HIRAGANA LETTER RI */
  ["\u308B", "\u308B"], /* HIRAGANA LETTER RU */
  ["\u308C", "\u308C"], /* HIRAGANA LETTER RE */
  ["\u308D", "\u308D"], /* HIRAGANA LETTER RO */
  ["\u308E", "\u308E"], /* HIRAGANA LETTER SMALL WA */
  ["\u308F", "\u308F"], /* HIRAGANA LETTER WA */
  ["\u3090", "\u3090"], /* HIRAGANA LETTER WI */
  ["\u3091", "\u3091"], /* HIRAGANA LETTER WE */
  ["\u3092", "\u3092"], /* HIRAGANA LETTER WO */
  ["\u3093", "\u3093"], /* HIRAGANA LETTER N */
  ["\u3094", "\u3094"], /* HIRAGANA LETTER VU */
  ["\u3095", "\u3095"], /* HIRAGANA LETTER SMALL KA */
  ["\u3096", "\u3096"], /* HIRAGANA LETTER SMALL KE */
  ["\u3097", "\u3097"],
  ["\u3098", "\u3098"],
  ["\u3099", "\u3099"], /* COMBINING KATAKANA-HIRAGANA VOICED SOUND MARK (NON-SPACING KATAKANA-HIRAGANA VOICED SOUND MARK) */
  ["\u309A", "\u309A"], /* COMBINING KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK (NON-SPACING KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK) */
  ["\u309B", "\u309B"], /* KATAKANA-HIRAGANA VOICED SOUND MARK */
  ["\u309C", "\u309C"], /* KATAKANA-HIRAGANA SEMI-VOICED SOUND MARK */
  ["\u309D", "\u309D"], /* HIRAGANA ITERATION MARK */
  ["\u309E", "\u309E"], /* HIRAGANA VOICED ITERATION MARK */
  ["\u309F", "\u309F"], /* HIRAGANA DIGRAPH YORI */
  ["\u30A0", "\u30A0"], /* KATAKANA-HIRAGANA DOUBLE HYPHEN */
  ["\u30A1", "\u30A1"], /* KATAKANA LETTER SMALL A */
  ["\u30A2", "\u30A2"], /* KATAKANA LETTER A */
  ["\u30A3", "\u30A3"], /* KATAKANA LETTER SMALL I */
  ["\u30A4", "\u30A4"], /* KATAKANA LETTER I */
  ["\u30A5", "\u30A5"], /* KATAKANA LETTER SMALL U */
  ["\u30A6", "\u30A6"], /* KATAKANA LETTER U */
  ["\u30A7", "\u30A7"], /* KATAKANA LETTER SMALL E */
  ["\u30A8", "\u30A8"], /* KATAKANA LETTER E */
  ["\u30A9", "\u30A9"], /* KATAKANA LETTER SMALL O */
  ["\u30AA", "\u30AA"], /* KATAKANA LETTER O */
  ["\u30AB", "\u30AB"], /* KATAKANA LETTER KA */
  ["\u30AC", "\u30AC"], /* KATAKANA LETTER GA */
  ["\u30AD", "\u30AD"], /* KATAKANA LETTER KI */
  ["\u30AE", "\u30AE"], /* KATAKANA LETTER GI */
  ["\u30AF", "\u30AF"], /* KATAKANA LETTER KU */
  ["\u30B0", "\u30B0"], /* KATAKANA LETTER GU */
  ["\u30B1", "\u30B1"], /* KATAKANA LETTER KE */
  ["\u30B2", "\u30B2"], /* KATAKANA LETTER GE */
  ["\u30B3", "\u30B3"], /* KATAKANA LETTER KO */
  ["\u30B4", "\u30B4"], /* KATAKANA LETTER GO */
  ["\u30B5", "\u30B5"], /* KATAKANA LETTER SA */
  ["\u30B6", "\u30B6"], /* KATAKANA LETTER ZA */
  ["\u30B7", "\u30B7"], /* KATAKANA LETTER SI */
  ["\u30B8", "\u30B8"], /* KATAKANA LETTER ZI */
  ["\u30B9", "\u30B9"], /* KATAKANA LETTER SU */
  ["\u30BA", "\u30BA"], /* KATAKANA LETTER ZU */
  ["\u30BB", "\u30BB"], /* KATAKANA LETTER SE */
  ["\u30BC", "\u30BC"], /* KATAKANA LETTER ZE */
  ["\u30BD", "\u30BD"], /* KATAKANA LETTER SO */
  ["\u30BE", "\u30BE"], /* KATAKANA LETTER ZO */
  ["\u30BF", "\u30BF"], /* KATAKANA LETTER TA */
  ["\u30C0", "\u30C0"], /* KATAKANA LETTER DA */
  ["\u30C1", "\u30C1"], /* KATAKANA LETTER TI */
  ["\u30C2", "\u30C2"], /* KATAKANA LETTER DI */
  ["\u30C3", "\u30C3"], /* KATAKANA LETTER SMALL TU */
  ["\u30C4", "\u30C4"], /* KATAKANA LETTER TU */
  ["\u30C5", "\u30C5"], /* KATAKANA LETTER DU */
  ["\u30C6", "\u30C6"], /* KATAKANA LETTER TE */
  ["\u30C7", "\u30C7"], /* KATAKANA LETTER DE */
  ["\u30C8", "\u30C8"], /* KATAKANA LETTER TO */
  ["\u30C9", "\u30C9"], /* KATAKANA LETTER DO */
  ["\u30CA", "\u30CA"], /* KATAKANA LETTER NA */
  ["\u30CB", "\u30CB"], /* KATAKANA LETTER NI */
  ["\u30CC", "\u30CC"], /* KATAKANA LETTER NU */
  ["\u30CD", "\u30CD"], /* KATAKANA LETTER NE */
  ["\u30CE", "\u30CE"], /* KATAKANA LETTER NO */
  ["\u30CF", "\u30CF"], /* KATAKANA LETTER HA */
  ["\u30D0", "\u30D0"], /* KATAKANA LETTER BA */
  ["\u30D1", "\u30D1"], /* KATAKANA LETTER PA */
  ["\u30D2", "\u30D2"], /* KATAKANA LETTER HI */
  ["\u30D3", "\u30D3"], /* KATAKANA LETTER BI */
  ["\u30D4", "\u30D4"], /* KATAKANA LETTER PI */
  ["\u30D5", "\u30D5"], /* KATAKANA LETTER HU */
  ["\u30D6", "\u30D6"], /* KATAKANA LETTER BU */
  ["\u30D7", "\u30D7"], /* KATAKANA LETTER PU */
  ["\u30D8", "\u30D8"], /* KATAKANA LETTER HE */
  ["\u30D9", "\u30D9"], /* KATAKANA LETTER BE */
  ["\u30DA", "\u30DA"], /* KATAKANA LETTER PE */
  ["\u30DB", "\u30DB"], /* KATAKANA LETTER HO */
  ["\u30DC", "\u30DC"], /* KATAKANA LETTER BO */
  ["\u30DD", "\u30DD"], /* KATAKANA LETTER PO */
  ["\u30DE", "\u30DE"], /* KATAKANA LETTER MA */
  ["\u30DF", "\u30DF"], /* KATAKANA LETTER MI */
  ["\u30E0", "\u30E0"], /* KATAKANA LETTER MU */
  ["\u30E1", "\u30E1"], /* KATAKANA LETTER ME */
  ["\u30E2", "\u30E2"], /* KATAKANA LETTER MO */
  ["\u30E3", "\u30E3"], /* KATAKANA LETTER SMALL YA */
  ["\u30E4", "\u30E4"], /* KATAKANA LETTER YA */
  ["\u30E5", "\u30E5"], /* KATAKANA LETTER SMALL YU */
  ["\u30E6", "\u30E6"], /* KATAKANA LETTER YU */
  ["\u30E7", "\u30E7"], /* KATAKANA LETTER SMALL YO */
  ["\u30E8", "\u30E8"], /* KATAKANA LETTER YO */
  ["\u30E9", "\u30E9"], /* KATAKANA LETTER RA */
  ["\u30EA", "\u30EA"], /* KATAKANA LETTER RI */
  ["\u30EB", "\u30EB"], /* KATAKANA LETTER RU */
  ["\u30EC", "\u30EC"], /* KATAKANA LETTER RE */
  ["\u30ED", "\u30ED"], /* KATAKANA LETTER RO */
  ["\u30EE", "\u30EE"], /* KATAKANA LETTER SMALL WA */
  ["\u30EF", "\u30EF"], /* KATAKANA LETTER WA */
  ["\u30F0", "\u30F0"], /* KATAKANA LETTER WI */
  ["\u30F1", "\u30F1"], /* KATAKANA LETTER WE */
  ["\u30F2", "\u30F2"], /* KATAKANA LETTER WO */
  ["\u30F3", "\u30F3"], /* KATAKANA LETTER N */
  ["\u30F4", "\u30F4"], /* KATAKANA LETTER VU */
  ["\u30F5", "\u30F5"], /* KATAKANA LETTER SMALL KA */
  ["\u30F6", "\u30F6"], /* KATAKANA LETTER SMALL KE */
  ["\u30F7", "\u30F7"], /* KATAKANA LETTER VA */
  ["\u30F8", "\u30F8"], /* KATAKANA LETTER VI */
  ["\u30F9", "\u30F9"], /* KATAKANA LETTER VE */
  ["\u30FA", "\u30FA"], /* KATAKANA LETTER VO */
  ["\u30FB", "\u30FB"], /* KATAKANA MIDDLE DOT */
  ["\u30FC", "\u30FC"], /* KATAKANA-HIRAGANA PROLONGED SOUND MARK */
  ["\u30FD", "\u30FD"], /* KATAKANA ITERATION MARK */
  ["\u30FE", "\u30FE"], /* KATAKANA VOICED ITERATION MARK */
  ["\u30FF", "\u30FF"], /* KATAKANA DIGRAPH KOTO */
  ["\u3100", "\u3100"],
  ["\u3101", "\u3101"],
  ["\u3102", "\u3102"],
  ["\u3103", "\u3103"],
  ["\u3104", "\u3104"],
  ["\u3105", "\u3105"], /* BOPOMOFO LETTER B */
  ["\u3106", "\u3106"], /* BOPOMOFO LETTER P */
  ["\u3107", "\u3107"], /* BOPOMOFO LETTER M */
  ["\u3108", "\u3108"], /* BOPOMOFO LETTER F */
  ["\u3109", "\u3109"], /* BOPOMOFO LETTER D */
  ["\u310A", "\u310A"], /* BOPOMOFO LETTER T */
  ["\u310B", "\u310B"], /* BOPOMOFO LETTER N */
  ["\u310C", "\u310C"], /* BOPOMOFO LETTER L */
  ["\u310D", "\u310D"], /* BOPOMOFO LETTER G */
  ["\u310E", "\u310E"], /* BOPOMOFO LETTER K */
  ["\u310F", "\u310F"], /* BOPOMOFO LETTER H */
  ["\u3110", "\u3110"], /* BOPOMOFO LETTER J */
  ["\u3111", "\u3111"], /* BOPOMOFO LETTER Q */
  ["\u3112", "\u3112"], /* BOPOMOFO LETTER X */
  ["\u3113", "\u3113"], /* BOPOMOFO LETTER ZH */
  ["\u3114", "\u3114"], /* BOPOMOFO LETTER CH */
  ["\u3115", "\u3115"], /* BOPOMOFO LETTER SH */
  ["\u3116", "\u3116"], /* BOPOMOFO LETTER R */
  ["\u3117", "\u3117"], /* BOPOMOFO LETTER Z */
  ["\u3118", "\u3118"], /* BOPOMOFO LETTER C */
  ["\u3119", "\u3119"], /* BOPOMOFO LETTER S */
  ["\u311A", "\u311A"], /* BOPOMOFO LETTER A */
  ["\u311B", "\u311B"], /* BOPOMOFO LETTER O */
  ["\u311C", "\u311C"], /* BOPOMOFO LETTER E */
  ["\u311D", "\u311D"], /* BOPOMOFO LETTER EH */
  ["\u311E", "\u311E"], /* BOPOMOFO LETTER AI */
  ["\u311F", "\u311F"], /* BOPOMOFO LETTER EI */
  ["\u3120", "\u3120"], /* BOPOMOFO LETTER AU */
  ["\u3121", "\u3121"], /* BOPOMOFO LETTER OU */
  ["\u3122", "\u3122"], /* BOPOMOFO LETTER AN */
  ["\u3123", "\u3123"], /* BOPOMOFO LETTER EN */
  ["\u3124", "\u3124"], /* BOPOMOFO LETTER ANG */
  ["\u3125", "\u3125"], /* BOPOMOFO LETTER ENG */
  ["\u3126", "\u3126"], /* BOPOMOFO LETTER ER */
  ["\u3127", "\u3127"], /* BOPOMOFO LETTER I */
  ["\u3128", "\u3128"], /* BOPOMOFO LETTER U */
  ["\u3129", "\u3129"], /* BOPOMOFO LETTER IU */
  ["\u312A", "\u312A"], /* BOPOMOFO LETTER V */
  ["\u312B", "\u312B"], /* BOPOMOFO LETTER NG */
  ["\u312C", "\u312C"], /* BOPOMOFO LETTER GN */
  ["\u312D", "\u312D"], /* BOPOMOFO LETTER IH */
  ["\u312E", "\u312E"], /* BOPOMOFO LETTER O WITH DOT ABOVE */
  ["\u312F", "\u312F"], /* BOPOMOFO LETTER NN */
  ["\u3130", "\u3130"],
  ["\u3131", "\u3131"], /* HANGUL LETTER KIYEOK (HANGUL LETTER GIYEOG) */
  ["\u3132", "\u3132"], /* HANGUL LETTER SSANGKIYEOK (HANGUL LETTER SSANG GIYEOG) */
  ["\u3133", "\u3133"], /* HANGUL LETTER KIYEOK-SIOS (HANGUL LETTER GIYEOG SIOS) */
  ["\u3134", "\u3134"], /* HANGUL LETTER NIEUN */
  ["\u3135", "\u3135"], /* HANGUL LETTER NIEUN-CIEUC (HANGUL LETTER NIEUN JIEUJ) */
  ["\u3136", "\u3136"], /* HANGUL LETTER NIEUN-HIEUH (HANGUL LETTER NIEUN HIEUH) */
  ["\u3137", "\u3137"], /* HANGUL LETTER TIKEUT (HANGUL LETTER DIGEUD) */
  ["\u3138", "\u3138"], /* HANGUL LETTER SSANGTIKEUT (HANGUL LETTER SSANG DIGEUD) */
  ["\u3139", "\u3139"], /* HANGUL LETTER RIEUL (HANGUL LETTER LIEUL) */
  ["\u313A", "\u313A"], /* HANGUL LETTER RIEUL-KIYEOK (HANGUL LETTER LIEUL GIYEOG) */
  ["\u313B", "\u313B"], /* HANGUL LETTER RIEUL-MIEUM (HANGUL LETTER LIEUL MIEUM) */
  ["\u313C", "\u313C"], /* HANGUL LETTER RIEUL-PIEUP (HANGUL LETTER LIEUL BIEUB) */
  ["\u313D", "\u313D"], /* HANGUL LETTER RIEUL-SIOS (HANGUL LETTER LIEUL SIOS) */
  ["\u313E", "\u313E"], /* HANGUL LETTER RIEUL-THIEUTH (HANGUL LETTER LIEUL TIEUT) */
  ["\u313F", "\u313F"], /* HANGUL LETTER RIEUL-PHIEUPH (HANGUL LETTER LIEUL PIEUP) */
  ["\u3140", "\u3140"], /* HANGUL LETTER RIEUL-HIEUH (HANGUL LETTER LIEUL HIEUH) */
  ["\u3141", "\u3141"], /* HANGUL LETTER MIEUM */
  ["\u3142", "\u3142"], /* HANGUL LETTER PIEUP (HANGUL LETTER BIEUB) */
  ["\u3143", "\u3143"], /* HANGUL LETTER SSANGPIEUP (HANGUL LETTER SSANG BIEUB) */
  ["\u3144", "\u3144"], /* HANGUL LETTER PIEUP-SIOS (HANGUL LETTER BIEUB SIOS) */
  ["\u3145", "\u3145"], /* HANGUL LETTER SIOS */
  ["\u3146", "\u3146"], /* HANGUL LETTER SSANGSIOS (HANGUL LETTER SSANG SIOS) */
  ["\u3147", "\u3147"], /* HANGUL LETTER IEUNG */
  ["\u3148", "\u3148"], /* HANGUL LETTER CIEUC (HANGUL LETTER JIEUJ) */
  ["\u3149", "\u3149"], /* HANGUL LETTER SSANGCIEUC (HANGUL LETTER SSANG JIEUJ) */
  ["\u314A", "\u314A"], /* HANGUL LETTER CHIEUCH (HANGUL LETTER CIEUC) */
  ["\u314B", "\u314B"], /* HANGUL LETTER KHIEUKH (HANGUL LETTER KIYEOK) */
  ["\u314C", "\u314C"], /* HANGUL LETTER THIEUTH (HANGUL LETTER TIEUT) */
  ["\u314D", "\u314D"], /* HANGUL LETTER PHIEUPH (HANGUL LETTER PIEUP) */
  ["\u314E", "\u314E"], /* HANGUL LETTER HIEUH */
  ["\u314F", "\u314F"], /* HANGUL LETTER A */
  ["\u3150", "\u3150"], /* HANGUL LETTER AE */
  ["\u3151", "\u3151"], /* HANGUL LETTER YA */
  ["\u3152", "\u3152"], /* HANGUL LETTER YAE */
  ["\u3153", "\u3153"], /* HANGUL LETTER EO */
  ["\u3154", "\u3154"], /* HANGUL LETTER E */
  ["\u3155", "\u3155"], /* HANGUL LETTER YEO */
  ["\u3156", "\u3156"], /* HANGUL LETTER YE */
  ["\u3157", "\u3157"], /* HANGUL LETTER O */
  ["\u3158", "\u3158"], /* HANGUL LETTER WA */
  ["\u3159", "\u3159"], /* HANGUL LETTER WAE */
  ["\u315A", "\u315A"], /* HANGUL LETTER OE */
  ["\u315B", "\u315B"], /* HANGUL LETTER YO */
  ["\u315C", "\u315C"], /* HANGUL LETTER U */
  ["\u315D", "\u315D"], /* HANGUL LETTER WEO */
  ["\u315E", "\u315E"], /* HANGUL LETTER WE */
  ["\u315F", "\u315F"], /* HANGUL LETTER WI */
  ["\u3160", "\u3160"], /* HANGUL LETTER YU */
  ["\u3161", "\u3161"], /* HANGUL LETTER EU */
  ["\u3162", "\u3162"], /* HANGUL LETTER YI */
  ["\u3163", "\u3163"], /* HANGUL LETTER I */
  ["\u3164", "\u3164"], /* HANGUL FILLER (HANGUL CAE OM) */
  ["\u3165", "\u3165"], /* HANGUL LETTER SSANGNIEUN (HANGUL LETTER SSANG NIEUN) */
  ["\u3166", "\u3166"], /* HANGUL LETTER NIEUN-TIKEUT (HANGUL LETTER NIEUN DIGEUD) */
  ["\u3167", "\u3167"], /* HANGUL LETTER NIEUN-SIOS (HANGUL LETTER NIEUN SIOS) */
  ["\u3168", "\u3168"], /* HANGUL LETTER NIEUN-PANSIOS (HANGUL LETTER NIEUN BAN CHI EUM) */
  ["\u3169", "\u3169"], /* HANGUL LETTER RIEUL-KIYEOK-SIOS (HANGUL LETTER LIEUL GIYEOG SIOS) */
  ["\u316A", "\u316A"], /* HANGUL LETTER RIEUL-TIKEUT (HANGUL LETTER LIEUL DIGEUD) */
  ["\u316B", "\u316B"], /* HANGUL LETTER RIEUL-PIEUP-SIOS (HANGUL LETTER LIEUL BIEUB SIOS) */
  ["\u316C", "\u316C"], /* HANGUL LETTER RIEUL-PANSIOS (HANGUL LETTER LIEUL BAN CHI EUM) */
  ["\u316D", "\u316D"], /* HANGUL LETTER RIEUL-YEORINHIEUH (HANGUL LETTER LIEUL YEOLIN HIEUH) */
  ["\u316E", "\u316E"], /* HANGUL LETTER MIEUM-PIEUP (HANGUL LETTER MIEUM BIEUB) */
  ["\u316F", "\u316F"], /* HANGUL LETTER MIEUM-SIOS (HANGUL LETTER MIEUM SIOS) */
  ["\u3170", "\u3170"], /* HANGUL LETTER MIEUM-PANSIOS (HANGUL LETTER BIEUB BAN CHI EUM) */
  ["\u3171", "\u3171"], /* HANGUL LETTER KAPYEOUNMIEUM (HANGUL LETTER MIEUM SUN GYEONG EUM) */
  ["\u3172", "\u3172"], /* HANGUL LETTER PIEUP-KIYEOK (HANGUL LETTER BIEUB GIYEOG) */
  ["\u3173", "\u3173"], /* HANGUL LETTER PIEUP-TIKEUT (HANGUL LETTER BIEUB DIGEUD) */
  ["\u3174", "\u3174"], /* HANGUL LETTER PIEUP-SIOS-KIYEOK (HANGUL LETTER BIEUB SIOS GIYEOG) */
  ["\u3175", "\u3175"], /* HANGUL LETTER PIEUP-SIOS-TIKEUT (HANGUL LETTER BIEUB SIOS DIGEUD) */
  ["\u3176", "\u3176"], /* HANGUL LETTER PIEUP-CIEUC (HANGUL LETTER BIEUB JIEUJ) */
  ["\u3177", "\u3177"], /* HANGUL LETTER PIEUP-THIEUTH (HANGUL LETTER BIEUB TIEUT) */
  ["\u3178", "\u3178"], /* HANGUL LETTER KAPYEOUNPIEUP (HANGUL LETTER BIEUB SUN GYEONG EUM) */
  ["\u3179", "\u3179"], /* HANGUL LETTER KAPYEOUNSSANGPIEUP (HANGUL LETTER SSANG BIEUB SUN GYEONG EUM) */
  ["\u317A", "\u317A"], /* HANGUL LETTER SIOS-KIYEOK (HANGUL LETTER SIOS GIYEOG) */
  ["\u317B", "\u317B"], /* HANGUL LETTER SIOS-NIEUN (HANGUL LETTER SIOS NIEUN) */
  ["\u317C", "\u317C"], /* HANGUL LETTER SIOS-TIKEUT (HANGUL LETTER SIOS DIGEUD) */
  ["\u317D", "\u317D"], /* HANGUL LETTER SIOS-PIEUP (HANGUL LETTER SIOS BIEUB) */
  ["\u317E", "\u317E"], /* HANGUL LETTER SIOS-CIEUC (HANGUL LETTER SIOS JIEUJ) */
  ["\u317F", "\u317F"], /* HANGUL LETTER PANSIOS (HANGUL LETTER BAN CHI EUM) */
  ["\u3180", "\u3180"], /* HANGUL LETTER SSANGIEUNG (HANGUL LETTER SSANG IEUNG) */
  ["\u3181", "\u3181"], /* HANGUL LETTER YESIEUNG (HANGUL LETTER NGIEUNG) */
  ["\u3182", "\u3182"], /* HANGUL LETTER YESIEUNG-SIOS (HANGUL LETTER NGIEUNG SIOS) */
  ["\u3183", "\u3183"], /* HANGUL LETTER YESIEUNG-PANSIOS (HANGUL LETTER NGIEUNG BAN CHI EUM) */
  ["\u3184", "\u3184"], /* HANGUL LETTER KAPYEOUNPHIEUPH (HANGUL LETTER PIEUP SUN GYEONG EUM) */
  ["\u3185", "\u3185"], /* HANGUL LETTER SSANGHIEUH (HANGUL LETTER SSANG HIEUH) */
  ["\u3186", "\u3186"], /* HANGUL LETTER YEORINHIEUH (HANGUL LETTER YEOLIN HIEUH) */
  ["\u3187", "\u3187"], /* HANGUL LETTER YO-YA (HANGUL LETTER YOYA) */
  ["\u3188", "\u3188"], /* HANGUL LETTER YO-YAE (HANGUL LETTER YOYAE) */
  ["\u3189", "\u3189"], /* HANGUL LETTER YO-I (HANGUL LETTER YOI) */
  ["\u318A", "\u318A"], /* HANGUL LETTER YU-YEO (HANGUL LETTER YUYEO) */
  ["\u318B", "\u318B"], /* HANGUL LETTER YU-YE (HANGUL LETTER YUYE) */
  ["\u318C", "\u318C"], /* HANGUL LETTER YU-I (HANGUL LETTER YUI) */
  ["\u318D", "\u318D"], /* HANGUL LETTER ARAEA (HANGUL LETTER ALAE A) */
  ["\u318E", "\u318E"], /* HANGUL LETTER ARAEAE (HANGUL LETTER ALAE AE) */
  ["\u318F", "\u318F"],
  ["\u3190", "\u3190"], /* IDEOGRAPHIC ANNOTATION LINKING MARK (KANBUN TATETEN) */
  ["\u3191", "\u3191"], /* IDEOGRAPHIC ANNOTATION REVERSE MARK (KAERITEN RE) */
  ["\u3192", "\u3192"], /* IDEOGRAPHIC ANNOTATION ONE MARK (KAERITEN ITI) */
  ["\u3193", "\u3193"], /* IDEOGRAPHIC ANNOTATION TWO MARK (KAERITEN NI) */
  ["\u3194", "\u3194"], /* IDEOGRAPHIC ANNOTATION THREE MARK (KAERITEN SAN) */
  ["\u3195", "\u3195"], /* IDEOGRAPHIC ANNOTATION FOUR MARK (KAERITEN SI) */
  ["\u3196", "\u3196"], /* IDEOGRAPHIC ANNOTATION TOP MARK (KAERITEN ZYOU) */
  ["\u3197", "\u3197"], /* IDEOGRAPHIC ANNOTATION MIDDLE MARK (KAERITEN TYUU) */
  ["\u3198", "\u3198"], /* IDEOGRAPHIC ANNOTATION BOTTOM MARK (KAERITEN GE) */
  ["\u3199", "\u3199"], /* IDEOGRAPHIC ANNOTATION FIRST MARK (KAERITEN KOU) */
  ["\u319A", "\u319A"], /* IDEOGRAPHIC ANNOTATION SECOND MARK (KAERITEN OTU) */
  ["\u319B", "\u319B"], /* IDEOGRAPHIC ANNOTATION THIRD MARK (KAERITEN HEI) */
  ["\u319C", "\u319C"], /* IDEOGRAPHIC ANNOTATION FOURTH MARK (KAERITEN TEI) */
  ["\u319D", "\u319D"], /* IDEOGRAPHIC ANNOTATION HEAVEN MARK (KAERITEN TEN) */
  ["\u319E", "\u319E"], /* IDEOGRAPHIC ANNOTATION EARTH MARK (KAERITEN TI) */
  ["\u319F", "\u319F"], /* IDEOGRAPHIC ANNOTATION MAN MARK (KAERITEN ZIN) */
  ["\u31A0", "\u31A0"], /* BOPOMOFO LETTER BU */
  ["\u31A1", "\u31A1"], /* BOPOMOFO LETTER ZI */
  ["\u31A2", "\u31A2"], /* BOPOMOFO LETTER JI */
  ["\u31A3", "\u31A3"], /* BOPOMOFO LETTER GU */
  ["\u31A4", "\u31A4"], /* BOPOMOFO LETTER EE */
  ["\u31A5", "\u31A5"], /* BOPOMOFO LETTER ENN */
  ["\u31A6", "\u31A6"], /* BOPOMOFO LETTER OO */
  ["\u31A7", "\u31A7"], /* BOPOMOFO LETTER ONN */
  ["\u31A8", "\u31A8"], /* BOPOMOFO LETTER IR */
  ["\u31A9", "\u31A9"], /* BOPOMOFO LETTER ANN */
  ["\u31AA", "\u31AA"], /* BOPOMOFO LETTER INN */
  ["\u31AB", "\u31AB"], /* BOPOMOFO LETTER UNN */
  ["\u31AC", "\u31AC"], /* BOPOMOFO LETTER IM */
  ["\u31AD", "\u31AD"], /* BOPOMOFO LETTER NGG */
  ["\u31AE", "\u31AE"], /* BOPOMOFO LETTER AINN */
  ["\u31AF", "\u31AF"], /* BOPOMOFO LETTER AUNN */
  ["\u31B0", "\u31B0"], /* BOPOMOFO LETTER AM */
  ["\u31B1", "\u31B1"], /* BOPOMOFO LETTER OM */
  ["\u31B2", "\u31B2"], /* BOPOMOFO LETTER ONG */
  ["\u31B3", "\u31B3"], /* BOPOMOFO LETTER INNN */
  ["\u31B4", "\u31B4"], /* BOPOMOFO FINAL LETTER P */
  ["\u31B5", "\u31B5"], /* BOPOMOFO FINAL LETTER T */
  ["\u31B6", "\u31B6"], /* BOPOMOFO FINAL LETTER K */
  ["\u31B7", "\u31B7"], /* BOPOMOFO FINAL LETTER H */
  ["\u31B8", "\u31B8"], /* BOPOMOFO LETTER GH */
  ["\u31B9", "\u31B9"], /* BOPOMOFO LETTER LH */
  ["\u31BA", "\u31BA"], /* BOPOMOFO LETTER ZY */
  ["\u31BB", "\u31BB"],
  ["\u31BC", "\u31BC"],
  ["\u31BD", "\u31BD"],
  ["\u31BE", "\u31BE"],
  ["\u31BF", "\u31BF"],
  ["\u31C0", "\u31C0"], /* CJK STROKE T */
  ["\u31C1", "\u31C1"], /* CJK STROKE WG */
  ["\u31C2", "\u31C2"], /* CJK STROKE XG */
  ["\u31C3", "\u31C3"], /* CJK STROKE BXG */
  ["\u31C4", "\u31C4"], /* CJK STROKE SW */
  ["\u31C5", "\u31C5"], /* CJK STROKE HZZ */
  ["\u31C6", "\u31C6"], /* CJK STROKE HZG */
  ["\u31C7", "\u31C7"], /* CJK STROKE HP */
  ["\u31C8", "\u31C8"], /* CJK STROKE HZWG */
  ["\u31C9", "\u31C9"], /* CJK STROKE SZWG */
  ["\u31CA", "\u31CA"], /* CJK STROKE HZT */
  ["\u31CB", "\u31CB"], /* CJK STROKE HZZP */
  ["\u31CC", "\u31CC"], /* CJK STROKE HPWG */
  ["\u31CD", "\u31CD"], /* CJK STROKE HZW */
  ["\u31CE", "\u31CE"], /* CJK STROKE HZZZ */
  ["\u31CF", "\u31CF"], /* CJK STROKE N */
  ["\u31D0", "\u31D0"], /* CJK STROKE H */
  ["\u31D1", "\u31D1"], /* CJK STROKE S */
  ["\u31D2", "\u31D2"], /* CJK STROKE P */
  ["\u31D3", "\u31D3"], /* CJK STROKE SP */
  ["\u31D4", "\u31D4"], /* CJK STROKE D */
  ["\u31D5", "\u31D5"], /* CJK STROKE HZ */
  ["\u31D6", "\u31D6"], /* CJK STROKE HG */
  ["\u31D7", "\u31D7"], /* CJK STROKE SZ */
  ["\u31D8", "\u31D8"], /* CJK STROKE SWZ */
  ["\u31D9", "\u31D9"], /* CJK STROKE ST */
  ["\u31DA", "\u31DA"], /* CJK STROKE SG */
  ["\u31DB", "\u31DB"], /* CJK STROKE PD */
  ["\u31DC", "\u31DC"], /* CJK STROKE PZ */
  ["\u31DD", "\u31DD"], /* CJK STROKE TN */
  ["\u31DE", "\u31DE"], /* CJK STROKE SZZ */
  ["\u31DF", "\u31DF"], /* CJK STROKE SWG */
  ["\u31E0", "\u31E0"], /* CJK STROKE HXWG */
  ["\u31E1", "\u31E1"], /* CJK STROKE HZZZG */
  ["\u31E2", "\u31E2"], /* CJK STROKE PG */
  ["\u31E3", "\u31E3"], /* CJK STROKE Q */
  ["\u31E4", "\u31E4"],
  ["\u31E5", "\u31E5"],
  ["\u31E6", "\u31E6"],
  ["\u31E7", "\u31E7"],
  ["\u31E8", "\u31E8"],
  ["\u31E9", "\u31E9"],
  ["\u31EA", "\u31EA"],
  ["\u31EB", "\u31EB"],
  ["\u31EC", "\u31EC"],
  ["\u31ED", "\u31ED"],
  ["\u31EE", "\u31EE"],
  ["\u31EF", "\u31EF"],
  ["\u31F0", "\u31F0"], /* KATAKANA LETTER SMALL KU */
  ["\u31F1", "\u31F1"], /* KATAKANA LETTER SMALL SI */
  ["\u31F2", "\u31F2"], /* KATAKANA LETTER SMALL SU */
  ["\u31F3", "\u31F3"], /* KATAKANA LETTER SMALL TO */
  ["\u31F4", "\u31F4"], /* KATAKANA LETTER SMALL NU */
  ["\u31F5", "\u31F5"], /* KATAKANA LETTER SMALL HA */
  ["\u31F6", "\u31F6"], /* KATAKANA LETTER SMALL HI */
  ["\u31F7", "\u31F7"], /* KATAKANA LETTER SMALL HU */
  ["\u31F8", "\u31F8"], /* KATAKANA LETTER SMALL HE */
  ["\u31F9", "\u31F9"], /* KATAKANA LETTER SMALL HO */
  ["\u31FA", "\u31FA"], /* KATAKANA LETTER SMALL MU */
  ["\u31FB", "\u31FB"], /* KATAKANA LETTER SMALL RA */
  ["\u31FC", "\u31FC"], /* KATAKANA LETTER SMALL RI */
  ["\u31FD", "\u31FD"], /* KATAKANA LETTER SMALL RU */
  ["\u31FE", "\u31FE"], /* KATAKANA LETTER SMALL RE */
  ["\u31FF", "\u31FF"], /* KATAKANA LETTER SMALL RO */
  ["\u3200", "\u3200"], /* PARENTHESIZED HANGUL KIYEOK (PARENTHESIZED HANGUL GIYEOG) */
  ["\u3201", "\u3201"], /* PARENTHESIZED HANGUL NIEUN */
  ["\u3202", "\u3202"], /* PARENTHESIZED HANGUL TIKEUT (PARENTHESIZED HANGUL DIGEUD) */
  ["\u3203", "\u3203"], /* PARENTHESIZED HANGUL RIEUL (PARENTHESIZED HANGUL LIEUL) */
  ["\u3204", "\u3204"], /* PARENTHESIZED HANGUL MIEUM */
  ["\u3205", "\u3205"], /* PARENTHESIZED HANGUL PIEUP (PARENTHESIZED HANGUL BIEUB) */
  ["\u3206", "\u3206"], /* PARENTHESIZED HANGUL SIOS */
  ["\u3207", "\u3207"], /* PARENTHESIZED HANGUL IEUNG */
  ["\u3208", "\u3208"], /* PARENTHESIZED HANGUL CIEUC (PARENTHESIZED HANGUL JIEUJ) */
  ["\u3209", "\u3209"], /* PARENTHESIZED HANGUL CHIEUCH (PARENTHESIZED HANGUL CIEUC) */
  ["\u320A", "\u320A"], /* PARENTHESIZED HANGUL KHIEUKH (PARENTHESIZED HANGUL KIYEOK) */
  ["\u320B", "\u320B"], /* PARENTHESIZED HANGUL THIEUTH (PARENTHESIZED HANGUL TIEUT) */
  ["\u320C", "\u320C"], /* PARENTHESIZED HANGUL PHIEUPH (PARENTHESIZED HANGUL PIEUP) */
  ["\u320D", "\u320D"], /* PARENTHESIZED HANGUL HIEUH */
  ["\u320E", "\u320E"], /* PARENTHESIZED HANGUL KIYEOK A (PARENTHESIZED HANGUL GA) */
  ["\u320F", "\u320F"], /* PARENTHESIZED HANGUL NIEUN A (PARENTHESIZED HANGUL NA) */
  ["\u3210", "\u3210"], /* PARENTHESIZED HANGUL TIKEUT A (PARENTHESIZED HANGUL DA) */
  ["\u3211", "\u3211"], /* PARENTHESIZED HANGUL RIEUL A (PARENTHESIZED HANGUL LA) */
  ["\u3212", "\u3212"], /* PARENTHESIZED HANGUL MIEUM A (PARENTHESIZED HANGUL MA) */
  ["\u3213", "\u3213"], /* PARENTHESIZED HANGUL PIEUP A (PARENTHESIZED HANGUL BA) */
  ["\u3214", "\u3214"], /* PARENTHESIZED HANGUL SIOS A (PARENTHESIZED HANGUL SA) */
  ["\u3215", "\u3215"], /* PARENTHESIZED HANGUL IEUNG A (PARENTHESIZED HANGUL A) */
  ["\u3216", "\u3216"], /* PARENTHESIZED HANGUL CIEUC A (PARENTHESIZED HANGUL JA) */
  ["\u3217", "\u3217"], /* PARENTHESIZED HANGUL CHIEUCH A (PARENTHESIZED HANGUL CA) */
  ["\u3218", "\u3218"], /* PARENTHESIZED HANGUL KHIEUKH A (PARENTHESIZED HANGUL KA) */
  ["\u3219", "\u3219"], /* PARENTHESIZED HANGUL THIEUTH A (PARENTHESIZED HANGUL TA) */
  ["\u321A", "\u321A"], /* PARENTHESIZED HANGUL PHIEUPH A (PARENTHESIZED HANGUL PA) */
  ["\u321B", "\u321B"], /* PARENTHESIZED HANGUL HIEUH A (PARENTHESIZED HANGUL HA) */
  ["\u321C", "\u321C"], /* PARENTHESIZED HANGUL CIEUC U (PARENTHESIZED HANGUL JU) */
  ["\u321D", "\u321D"], /* PARENTHESIZED KOREAN CHARACTER OJEON */
  ["\u321E", "\u321E"], /* PARENTHESIZED KOREAN CHARACTER O HU */
  ["\u321F", "\u321F"],
  ["\u3220", "\u3220"], /* PARENTHESIZED IDEOGRAPH ONE */
  ["\u3221", "\u3221"], /* PARENTHESIZED IDEOGRAPH TWO */
  ["\u3222", "\u3222"], /* PARENTHESIZED IDEOGRAPH THREE */
  ["\u3223", "\u3223"], /* PARENTHESIZED IDEOGRAPH FOUR */
  ["\u3224", "\u3224"], /* PARENTHESIZED IDEOGRAPH FIVE */
  ["\u3225", "\u3225"], /* PARENTHESIZED IDEOGRAPH SIX */
  ["\u3226", "\u3226"], /* PARENTHESIZED IDEOGRAPH SEVEN */
  ["\u3227", "\u3227"], /* PARENTHESIZED IDEOGRAPH EIGHT */
  ["\u3228", "\u3228"], /* PARENTHESIZED IDEOGRAPH NINE */
  ["\u3229", "\u3229"], /* PARENTHESIZED IDEOGRAPH TEN */
  ["\u322A", "\u322A"], /* PARENTHESIZED IDEOGRAPH MOON */
  ["\u322B", "\u322B"], /* PARENTHESIZED IDEOGRAPH FIRE */
  ["\u322C", "\u322C"], /* PARENTHESIZED IDEOGRAPH WATER */
  ["\u322D", "\u322D"], /* PARENTHESIZED IDEOGRAPH WOOD */
  ["\u322E", "\u322E"], /* PARENTHESIZED IDEOGRAPH METAL */
  ["\u322F", "\u322F"], /* PARENTHESIZED IDEOGRAPH EARTH */
  ["\u3230", "\u3230"], /* PARENTHESIZED IDEOGRAPH SUN */
  ["\u3231", "\u3231"], /* PARENTHESIZED IDEOGRAPH STOCK */
  ["\u3232", "\u3232"], /* PARENTHESIZED IDEOGRAPH HAVE */
  ["\u3233", "\u3233"], /* PARENTHESIZED IDEOGRAPH SOCIETY */
  ["\u3234", "\u3234"], /* PARENTHESIZED IDEOGRAPH NAME */
  ["\u3235", "\u3235"], /* PARENTHESIZED IDEOGRAPH SPECIAL */
  ["\u3236", "\u3236"], /* PARENTHESIZED IDEOGRAPH FINANCIAL */
  ["\u3237", "\u3237"], /* PARENTHESIZED IDEOGRAPH CONGRATULATION */
  ["\u3238", "\u3238"], /* PARENTHESIZED IDEOGRAPH LABOR */
  ["\u3239", "\u3239"], /* PARENTHESIZED IDEOGRAPH REPRESENT */
  ["\u323A", "\u323A"], /* PARENTHESIZED IDEOGRAPH CALL */
  ["\u323B", "\u323B"], /* PARENTHESIZED IDEOGRAPH STUDY */
  ["\u323C", "\u323C"], /* PARENTHESIZED IDEOGRAPH SUPERVISE */
  ["\u323D", "\u323D"], /* PARENTHESIZED IDEOGRAPH ENTERPRISE */
  ["\u323E", "\u323E"], /* PARENTHESIZED IDEOGRAPH RESOURCE */
  ["\u323F", "\u323F"], /* PARENTHESIZED IDEOGRAPH ALLIANCE */
  ["\u3240", "\u3240"], /* PARENTHESIZED IDEOGRAPH FESTIVAL */
  ["\u3241", "\u3241"], /* PARENTHESIZED IDEOGRAPH REST */
  ["\u3242", "\u3242"], /* PARENTHESIZED IDEOGRAPH SELF */
  ["\u3243", "\u3243"], /* PARENTHESIZED IDEOGRAPH REACH */
  ["\u3244", "\u3244"], /* CIRCLED IDEOGRAPH QUESTION */
  ["\u3245", "\u3245"], /* CIRCLED IDEOGRAPH KINDERGARTEN */
  ["\u3246", "\u3246"], /* CIRCLED IDEOGRAPH SCHOOL */
  ["\u3247", "\u3247"], /* CIRCLED IDEOGRAPH KOTO */
  ["\u3248", "\u3248"], /* CIRCLED NUMBER TEN ON BLACK SQUARE */
  ["\u3249", "\u3249"], /* CIRCLED NUMBER TWENTY ON BLACK SQUARE */
  ["\u324A", "\u324A"], /* CIRCLED NUMBER THIRTY ON BLACK SQUARE */
  ["\u324B", "\u324B"], /* CIRCLED NUMBER FORTY ON BLACK SQUARE */
  ["\u324C", "\u324C"], /* CIRCLED NUMBER FIFTY ON BLACK SQUARE */
  ["\u324D", "\u324D"], /* CIRCLED NUMBER SIXTY ON BLACK SQUARE */
  ["\u324E", "\u324E"], /* CIRCLED NUMBER SEVENTY ON BLACK SQUARE */
  ["\u324F", "\u324F"], /* CIRCLED NUMBER EIGHTY ON BLACK SQUARE */
  ["\u3250", "\u3250"], /* PARTNERSHIP SIGN */
  ["\u3251", "\u3251"], /* CIRCLED NUMBER TWENTY ONE */
  ["\u3252", "\u3252"], /* CIRCLED NUMBER TWENTY TWO */
  ["\u3253", "\u3253"], /* CIRCLED NUMBER TWENTY THREE */
  ["\u3254", "\u3254"], /* CIRCLED NUMBER TWENTY FOUR */
  ["\u3255", "\u3255"], /* CIRCLED NUMBER TWENTY FIVE */
  ["\u3256", "\u3256"], /* CIRCLED NUMBER TWENTY SIX */
  ["\u3257", "\u3257"], /* CIRCLED NUMBER TWENTY SEVEN */
  ["\u3258", "\u3258"], /* CIRCLED NUMBER TWENTY EIGHT */
  ["\u3259", "\u3259"], /* CIRCLED NUMBER TWENTY NINE */
  ["\u325A", "\u325A"], /* CIRCLED NUMBER THIRTY */
  ["\u325B", "\u325B"], /* CIRCLED NUMBER THIRTY ONE */
  ["\u325C", "\u325C"], /* CIRCLED NUMBER THIRTY TWO */
  ["\u325D", "\u325D"], /* CIRCLED NUMBER THIRTY THREE */
  ["\u325E", "\u325E"], /* CIRCLED NUMBER THIRTY FOUR */
  ["\u325F", "\u325F"], /* CIRCLED NUMBER THIRTY FIVE */
  ["\u3260", "\u3260"], /* CIRCLED HANGUL KIYEOK (CIRCLED HANGUL GIYEOG) */
  ["\u3261", "\u3261"], /* CIRCLED HANGUL NIEUN */
  ["\u3262", "\u3262"], /* CIRCLED HANGUL TIKEUT (CIRCLED HANGUL DIGEUD) */
  ["\u3263", "\u3263"], /* CIRCLED HANGUL RIEUL (CIRCLED HANGUL LIEUL) */
  ["\u3264", "\u3264"], /* CIRCLED HANGUL MIEUM */
  ["\u3265", "\u3265"], /* CIRCLED HANGUL PIEUP (CIRCLED HANGUL BIEUB) */
  ["\u3266", "\u3266"], /* CIRCLED HANGUL SIOS */
  ["\u3267", "\u3267"], /* CIRCLED HANGUL IEUNG */
  ["\u3268", "\u3268"], /* CIRCLED HANGUL CIEUC (CIRCLED HANGUL JIEUJ) */
  ["\u3269", "\u3269"], /* CIRCLED HANGUL CHIEUCH (CIRCLED HANGUL CIEUC) */
  ["\u326A", "\u326A"], /* CIRCLED HANGUL KHIEUKH (CIRCLED HANGUL KIYEOK) */
  ["\u326B", "\u326B"], /* CIRCLED HANGUL THIEUTH (CIRCLED HANGUL TIEUT) */
  ["\u326C", "\u326C"], /* CIRCLED HANGUL PHIEUPH (CIRCLED HANGUL PIEUP) */
  ["\u326D", "\u326D"], /* CIRCLED HANGUL HIEUH */
  ["\u326E", "\u326E"], /* CIRCLED HANGUL KIYEOK A (CIRCLED HANGUL GA) */
  ["\u326F", "\u326F"], /* CIRCLED HANGUL NIEUN A (CIRCLED HANGUL NA) */
  ["\u3270", "\u3270"], /* CIRCLED HANGUL TIKEUT A (CIRCLED HANGUL DA) */
  ["\u3271", "\u3271"], /* CIRCLED HANGUL RIEUL A (CIRCLED HANGUL LA) */
  ["\u3272", "\u3272"], /* CIRCLED HANGUL MIEUM A (CIRCLED HANGUL MA) */
  ["\u3273", "\u3273"], /* CIRCLED HANGUL PIEUP A (CIRCLED HANGUL BA) */
  ["\u3274", "\u3274"], /* CIRCLED HANGUL SIOS A (CIRCLED HANGUL SA) */
  ["\u3275", "\u3275"], /* CIRCLED HANGUL IEUNG A (CIRCLED HANGUL A) */
  ["\u3276", "\u3276"], /* CIRCLED HANGUL CIEUC A (CIRCLED HANGUL JA) */
  ["\u3277", "\u3277"], /* CIRCLED HANGUL CHIEUCH A (CIRCLED HANGUL CA) */
  ["\u3278", "\u3278"], /* CIRCLED HANGUL KHIEUKH A (CIRCLED HANGUL KA) */
  ["\u3279", "\u3279"], /* CIRCLED HANGUL THIEUTH A (CIRCLED HANGUL TA) */
  ["\u327A", "\u327A"], /* CIRCLED HANGUL PHIEUPH A (CIRCLED HANGUL PA) */
  ["\u327B", "\u327B"], /* CIRCLED HANGUL HIEUH A (CIRCLED HANGUL HA) */
  ["\u327C", "\u327C"], /* CIRCLED KOREAN CHARACTER CHAMKO */
  ["\u327D", "\u327D"], /* CIRCLED KOREAN CHARACTER JUEUI */
  ["\u327E", "\u327E"], /* CIRCLED HANGUL IEUNG U */
  ["\u327F", "\u327F"], /* KOREAN STANDARD SYMBOL */
  ["\u3280", "\u3280"], /* CIRCLED IDEOGRAPH ONE */
  ["\u3281", "\u3281"], /* CIRCLED IDEOGRAPH TWO */
  ["\u3282", "\u3282"], /* CIRCLED IDEOGRAPH THREE */
  ["\u3283", "\u3283"], /* CIRCLED IDEOGRAPH FOUR */
  ["\u3284", "\u3284"], /* CIRCLED IDEOGRAPH FIVE */
  ["\u3285", "\u3285"], /* CIRCLED IDEOGRAPH SIX */
  ["\u3286", "\u3286"], /* CIRCLED IDEOGRAPH SEVEN */
  ["\u3287", "\u3287"], /* CIRCLED IDEOGRAPH EIGHT */
  ["\u3288", "\u3288"], /* CIRCLED IDEOGRAPH NINE */
  ["\u3289", "\u3289"], /* CIRCLED IDEOGRAPH TEN */
  ["\u328A", "\u328A"], /* CIRCLED IDEOGRAPH MOON */
  ["\u328B", "\u328B"], /* CIRCLED IDEOGRAPH FIRE */
  ["\u328C", "\u328C"], /* CIRCLED IDEOGRAPH WATER */
  ["\u328D", "\u328D"], /* CIRCLED IDEOGRAPH WOOD */
  ["\u328E", "\u328E"], /* CIRCLED IDEOGRAPH METAL */
  ["\u328F", "\u328F"], /* CIRCLED IDEOGRAPH EARTH */
  ["\u3290", "\u3290"], /* CIRCLED IDEOGRAPH SUN */
  ["\u3291", "\u3291"], /* CIRCLED IDEOGRAPH STOCK */
  ["\u3292", "\u3292"], /* CIRCLED IDEOGRAPH HAVE */
  ["\u3293", "\u3293"], /* CIRCLED IDEOGRAPH SOCIETY */
  ["\u3294", "\u3294"], /* CIRCLED IDEOGRAPH NAME */
  ["\u3295", "\u3295"], /* CIRCLED IDEOGRAPH SPECIAL */
  ["\u3296", "\u3296"], /* CIRCLED IDEOGRAPH FINANCIAL */
  ["\u3297", "\u3297"], /* CIRCLED IDEOGRAPH CONGRATULATION */
  ["\u3298", "\u3298"], /* CIRCLED IDEOGRAPH LABOR */
  ["\u3299", "\u3299"], /* CIRCLED IDEOGRAPH SECRET */
  ["\u329A", "\u329A"], /* CIRCLED IDEOGRAPH MALE */
  ["\u329B", "\u329B"], /* CIRCLED IDEOGRAPH FEMALE */
  ["\u329C", "\u329C"], /* CIRCLED IDEOGRAPH SUITABLE */
  ["\u329D", "\u329D"], /* CIRCLED IDEOGRAPH EXCELLENT */
  ["\u329E", "\u329E"], /* CIRCLED IDEOGRAPH PRINT */
  ["\u329F", "\u329F"], /* CIRCLED IDEOGRAPH ATTENTION */
  ["\u32A0", "\u32A0"], /* CIRCLED IDEOGRAPH ITEM */
  ["\u32A1", "\u32A1"], /* CIRCLED IDEOGRAPH REST */
  ["\u32A2", "\u32A2"], /* CIRCLED IDEOGRAPH COPY */
  ["\u32A3", "\u32A3"], /* CIRCLED IDEOGRAPH CORRECT */
  ["\u32A4", "\u32A4"], /* CIRCLED IDEOGRAPH HIGH */
  ["\u32A5", "\u32A5"], /* CIRCLED IDEOGRAPH CENTRE (CIRCLED IDEOGRAPH CENTER) */
  ["\u32A6", "\u32A6"], /* CIRCLED IDEOGRAPH LOW */
  ["\u32A7", "\u32A7"], /* CIRCLED IDEOGRAPH LEFT */
  ["\u32A8", "\u32A8"], /* CIRCLED IDEOGRAPH RIGHT */
  ["\u32A9", "\u32A9"], /* CIRCLED IDEOGRAPH MEDICINE */
  ["\u32AA", "\u32AA"], /* CIRCLED IDEOGRAPH RELIGION */
  ["\u32AB", "\u32AB"], /* CIRCLED IDEOGRAPH STUDY */
  ["\u32AC", "\u32AC"], /* CIRCLED IDEOGRAPH SUPERVISE */
  ["\u32AD", "\u32AD"], /* CIRCLED IDEOGRAPH ENTERPRISE */
  ["\u32AE", "\u32AE"], /* CIRCLED IDEOGRAPH RESOURCE */
  ["\u32AF", "\u32AF"], /* CIRCLED IDEOGRAPH ALLIANCE */
  ["\u32B0", "\u32B0"], /* CIRCLED IDEOGRAPH NIGHT */
  ["\u32B1", "\u32B1"], /* CIRCLED NUMBER THIRTY SIX */
  ["\u32B2", "\u32B2"], /* CIRCLED NUMBER THIRTY SEVEN */
  ["\u32B3", "\u32B3"], /* CIRCLED NUMBER THIRTY EIGHT */
  ["\u32B4", "\u32B4"], /* CIRCLED NUMBER THIRTY NINE */
  ["\u32B5", "\u32B5"], /* CIRCLED NUMBER FORTY */
  ["\u32B6", "\u32B6"], /* CIRCLED NUMBER FORTY ONE */
  ["\u32B7", "\u32B7"], /* CIRCLED NUMBER FORTY TWO */
  ["\u32B8", "\u32B8"], /* CIRCLED NUMBER FORTY THREE */
  ["\u32B9", "\u32B9"], /* CIRCLED NUMBER FORTY FOUR */
  ["\u32BA", "\u32BA"], /* CIRCLED NUMBER FORTY FIVE */
  ["\u32BB", "\u32BB"], /* CIRCLED NUMBER FORTY SIX */
  ["\u32BC", "\u32BC"], /* CIRCLED NUMBER FORTY SEVEN */
  ["\u32BD", "\u32BD"], /* CIRCLED NUMBER FORTY EIGHT */
  ["\u32BE", "\u32BE"], /* CIRCLED NUMBER FORTY NINE */
  ["\u32BF", "\u32BF"], /* CIRCLED NUMBER FIFTY */
  ["\u32C0", "\u32C0"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR JANUARY */
  ["\u32C1", "\u32C1"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR FEBRUARY */
  ["\u32C2", "\u32C2"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR MARCH */
  ["\u32C3", "\u32C3"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR APRIL */
  ["\u32C4", "\u32C4"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR MAY */
  ["\u32C5", "\u32C5"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR JUNE */
  ["\u32C6", "\u32C6"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR JULY */
  ["\u32C7", "\u32C7"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR AUGUST */
  ["\u32C8", "\u32C8"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR SEPTEMBER */
  ["\u32C9", "\u32C9"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR OCTOBER */
  ["\u32CA", "\u32CA"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR NOVEMBER */
  ["\u32CB", "\u32CB"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DECEMBER */
  ["\u32CC", "\u32CC"], /* SQUARE HG */
  ["\u32CD", "\u32CD"], /* SQUARE ERG */
  ["\u32CE", "\u32CE"], /* SQUARE EV */
  ["\u32CF", "\u32CF"], /* LIMITED LIABILITY SIGN */
  ["\u32D0", "\u32D0"], /* CIRCLED KATAKANA A */
  ["\u32D1", "\u32D1"], /* CIRCLED KATAKANA I */
  ["\u32D2", "\u32D2"], /* CIRCLED KATAKANA U */
  ["\u32D3", "\u32D3"], /* CIRCLED KATAKANA E */
  ["\u32D4", "\u32D4"], /* CIRCLED KATAKANA O */
  ["\u32D5", "\u32D5"], /* CIRCLED KATAKANA KA */
  ["\u32D6", "\u32D6"], /* CIRCLED KATAKANA KI */
  ["\u32D7", "\u32D7"], /* CIRCLED KATAKANA KU */
  ["\u32D8", "\u32D8"], /* CIRCLED KATAKANA KE */
  ["\u32D9", "\u32D9"], /* CIRCLED KATAKANA KO */
  ["\u32DA", "\u32DA"], /* CIRCLED KATAKANA SA */
  ["\u32DB", "\u32DB"], /* CIRCLED KATAKANA SI */
  ["\u32DC", "\u32DC"], /* CIRCLED KATAKANA SU */
  ["\u32DD", "\u32DD"], /* CIRCLED KATAKANA SE */
  ["\u32DE", "\u32DE"], /* CIRCLED KATAKANA SO */
  ["\u32DF", "\u32DF"], /* CIRCLED KATAKANA TA */
  ["\u32E0", "\u32E0"], /* CIRCLED KATAKANA TI */
  ["\u32E1", "\u32E1"], /* CIRCLED KATAKANA TU */
  ["\u32E2", "\u32E2"], /* CIRCLED KATAKANA TE */
  ["\u32E3", "\u32E3"], /* CIRCLED KATAKANA TO */
  ["\u32E4", "\u32E4"], /* CIRCLED KATAKANA NA */
  ["\u32E5", "\u32E5"], /* CIRCLED KATAKANA NI */
  ["\u32E6", "\u32E6"], /* CIRCLED KATAKANA NU */
  ["\u32E7", "\u32E7"], /* CIRCLED KATAKANA NE */
  ["\u32E8", "\u32E8"], /* CIRCLED KATAKANA NO */
  ["\u32E9", "\u32E9"], /* CIRCLED KATAKANA HA */
  ["\u32EA", "\u32EA"], /* CIRCLED KATAKANA HI */
  ["\u32EB", "\u32EB"], /* CIRCLED KATAKANA HU */
  ["\u32EC", "\u32EC"], /* CIRCLED KATAKANA HE */
  ["\u32ED", "\u32ED"], /* CIRCLED KATAKANA HO */
  ["\u32EE", "\u32EE"], /* CIRCLED KATAKANA MA */
  ["\u32EF", "\u32EF"], /* CIRCLED KATAKANA MI */
  ["\u32F0", "\u32F0"], /* CIRCLED KATAKANA MU */
  ["\u32F1", "\u32F1"], /* CIRCLED KATAKANA ME */
  ["\u32F2", "\u32F2"], /* CIRCLED KATAKANA MO */
  ["\u32F3", "\u32F3"], /* CIRCLED KATAKANA YA */
  ["\u32F4", "\u32F4"], /* CIRCLED KATAKANA YU */
  ["\u32F5", "\u32F5"], /* CIRCLED KATAKANA YO */
  ["\u32F6", "\u32F6"], /* CIRCLED KATAKANA RA */
  ["\u32F7", "\u32F7"], /* CIRCLED KATAKANA RI */
  ["\u32F8", "\u32F8"], /* CIRCLED KATAKANA RU */
  ["\u32F9", "\u32F9"], /* CIRCLED KATAKANA RE */
  ["\u32FA", "\u32FA"], /* CIRCLED KATAKANA RO */
  ["\u32FB", "\u32FB"], /* CIRCLED KATAKANA WA */
  ["\u32FC", "\u32FC"], /* CIRCLED KATAKANA WI */
  ["\u32FD", "\u32FD"], /* CIRCLED KATAKANA WE */
  ["\u32FE", "\u32FE"], /* CIRCLED KATAKANA WO */
  ["\u32FF", "\u32FF"], /* SQUARE ERA NAME REIWA */
  ["\u3300", "\u3300"], /* SQUARE APAATO (SQUARED APAATO) */
  ["\u3301", "\u3301"], /* SQUARE ARUHUA (SQUARED ARUHUA) */
  ["\u3302", "\u3302"], /* SQUARE ANPEA (SQUARED ANPEA) */
  ["\u3303", "\u3303"], /* SQUARE AARU (SQUARED AARU) */
  ["\u3304", "\u3304"], /* SQUARE ININGU (SQUARED ININGU) */
  ["\u3305", "\u3305"], /* SQUARE INTI (SQUARED INTI) */
  ["\u3306", "\u3306"], /* SQUARE UON (SQUARED UON) */
  ["\u3307", "\u3307"], /* SQUARE ESUKUUDO (SQUARED ESUKUUDO) */
  ["\u3308", "\u3308"], /* SQUARE EEKAA (SQUARED EEKAA) */
  ["\u3309", "\u3309"], /* SQUARE ONSU (SQUARED ONSU) */
  ["\u330A", "\u330A"], /* SQUARE OOMU (SQUARED OOMU) */
  ["\u330B", "\u330B"], /* SQUARE KAIRI (SQUARED KAIRI) */
  ["\u330C", "\u330C"], /* SQUARE KARATTO (SQUARED KARATTO) */
  ["\u330D", "\u330D"], /* SQUARE KARORII (SQUARED KARORII) */
  ["\u330E", "\u330E"], /* SQUARE GARON (SQUARED GARON) */
  ["\u330F", "\u330F"], /* SQUARE GANMA (SQUARED GANMA) */
  ["\u3310", "\u3310"], /* SQUARE GIGA (SQUARED GIGA) */
  ["\u3311", "\u3311"], /* SQUARE GINII (SQUARED GINII) */
  ["\u3312", "\u3312"], /* SQUARE KYURII (SQUARED KYURII) */
  ["\u3313", "\u3313"], /* SQUARE GIRUDAA (SQUARED GIRUDAA) */
  ["\u3314", "\u3314"], /* SQUARE KIRO (SQUARED KIRO) */
  ["\u3315", "\u3315"], /* SQUARE KIROGURAMU (SQUARED KIROGURAMU) */
  ["\u3316", "\u3316"], /* SQUARE KIROMEETORU (SQUARED KIROMEETORU) */
  ["\u3317", "\u3317"], /* SQUARE KIROWATTO (SQUARED KIROWATTO) */
  ["\u3318", "\u3318"], /* SQUARE GURAMU (SQUARED GURAMU) */
  ["\u3319", "\u3319"], /* SQUARE GURAMUTON (SQUARED GURAMUTON) */
  ["\u331A", "\u331A"], /* SQUARE KURUZEIRO (SQUARED KURUZEIRO) */
  ["\u331B", "\u331B"], /* SQUARE KUROONE (SQUARED KUROONE) */
  ["\u331C", "\u331C"], /* SQUARE KEESU (SQUARED KEESU) */
  ["\u331D", "\u331D"], /* SQUARE KORUNA (SQUARED KORUNA) */
  ["\u331E", "\u331E"], /* SQUARE KOOPO (SQUARED KOOPO) */
  ["\u331F", "\u331F"], /* SQUARE SAIKURU (SQUARED SAIKURU) */
  ["\u3320", "\u3320"], /* SQUARE SANTIIMU (SQUARED SANTIIMU) */
  ["\u3321", "\u3321"], /* SQUARE SIRINGU (SQUARED SIRINGU) */
  ["\u3322", "\u3322"], /* SQUARE SENTI (SQUARED SENTI) */
  ["\u3323", "\u3323"], /* SQUARE SENTO (SQUARED SENTO) */
  ["\u3324", "\u3324"], /* SQUARE DAASU (SQUARED DAASU) */
  ["\u3325", "\u3325"], /* SQUARE DESI (SQUARED DESI) */
  ["\u3326", "\u3326"], /* SQUARE DORU (SQUARED DORU) */
  ["\u3327", "\u3327"], /* SQUARE TON (SQUARED TON) */
  ["\u3328", "\u3328"], /* SQUARE NANO (SQUARED NANO) */
  ["\u3329", "\u3329"], /* SQUARE NOTTO (SQUARED NOTTO) */
  ["\u332A", "\u332A"], /* SQUARE HAITU (SQUARED HAITU) */
  ["\u332B", "\u332B"], /* SQUARE PAASENTO (SQUARED PAASENTO) */
  ["\u332C", "\u332C"], /* SQUARE PAATU (SQUARED PAATU) */
  ["\u332D", "\u332D"], /* SQUARE BAARERU (SQUARED BAARERU) */
  ["\u332E", "\u332E"], /* SQUARE PIASUTORU (SQUARED PIASUTORU) */
  ["\u332F", "\u332F"], /* SQUARE PIKURU (SQUARED PIKURU) */
  ["\u3330", "\u3330"], /* SQUARE PIKO (SQUARED PIKO) */
  ["\u3331", "\u3331"], /* SQUARE BIRU (SQUARED BIRU) */
  ["\u3332", "\u3332"], /* SQUARE HUARADDO (SQUARED HUARADDO) */
  ["\u3333", "\u3333"], /* SQUARE HUIITO (SQUARED HUIITO) */
  ["\u3334", "\u3334"], /* SQUARE BUSSYERU (SQUARED BUSSYERU) */
  ["\u3335", "\u3335"], /* SQUARE HURAN (SQUARED HURAN) */
  ["\u3336", "\u3336"], /* SQUARE HEKUTAARU (SQUARED HEKUTAARU) */
  ["\u3337", "\u3337"], /* SQUARE PESO (SQUARED PESO) */
  ["\u3338", "\u3338"], /* SQUARE PENIHI (SQUARED PENIHI) */
  ["\u3339", "\u3339"], /* SQUARE HERUTU (SQUARED HERUTU) */
  ["\u333A", "\u333A"], /* SQUARE PENSU (SQUARED PENSU) */
  ["\u333B", "\u333B"], /* SQUARE PEEZI (SQUARED PEEZI) */
  ["\u333C", "\u333C"], /* SQUARE BEETA (SQUARED BEETA) */
  ["\u333D", "\u333D"], /* SQUARE POINTO (SQUARED POINTO) */
  ["\u333E", "\u333E"], /* SQUARE BORUTO (SQUARED BORUTO) */
  ["\u333F", "\u333F"], /* SQUARE HON (SQUARED HON) */
  ["\u3340", "\u3340"], /* SQUARE PONDO (SQUARED PONDO) */
  ["\u3341", "\u3341"], /* SQUARE HOORU (SQUARED HOORU) */
  ["\u3342", "\u3342"], /* SQUARE HOON (SQUARED HOON) */
  ["\u3343", "\u3343"], /* SQUARE MAIKURO (SQUARED MAIKURO) */
  ["\u3344", "\u3344"], /* SQUARE MAIRU (SQUARED MAIRU) */
  ["\u3345", "\u3345"], /* SQUARE MAHHA (SQUARED MAHHA) */
  ["\u3346", "\u3346"], /* SQUARE MARUKU (SQUARED MARUKU) */
  ["\u3347", "\u3347"], /* SQUARE MANSYON (SQUARED MANSYON) */
  ["\u3348", "\u3348"], /* SQUARE MIKURON (SQUARED MIKURON) */
  ["\u3349", "\u3349"], /* SQUARE MIRI (SQUARED MIRI) */
  ["\u334A", "\u334A"], /* SQUARE MIRIBAARU (SQUARED MIRIBAARU) */
  ["\u334B", "\u334B"], /* SQUARE MEGA (SQUARED MEGA) */
  ["\u334C", "\u334C"], /* SQUARE MEGATON (SQUARED MEGATON) */
  ["\u334D", "\u334D"], /* SQUARE MEETORU (SQUARED MEETORU) */
  ["\u334E", "\u334E"], /* SQUARE YAADO (SQUARED YAADO) */
  ["\u334F", "\u334F"], /* SQUARE YAARU (SQUARED YAARU) */
  ["\u3350", "\u3350"], /* SQUARE YUAN (SQUARED YUAN) */
  ["\u3351", "\u3351"], /* SQUARE RITTORU (SQUARED RITTORU) */
  ["\u3352", "\u3352"], /* SQUARE RIRA (SQUARED RIRA) */
  ["\u3353", "\u3353"], /* SQUARE RUPII (SQUARED RUPII) */
  ["\u3354", "\u3354"], /* SQUARE RUUBURU (SQUARED RUUBURU) */
  ["\u3355", "\u3355"], /* SQUARE REMU (SQUARED REMU) */
  ["\u3356", "\u3356"], /* SQUARE RENTOGEN (SQUARED RENTOGEN) */
  ["\u3357", "\u3357"], /* SQUARE WATTO (SQUARED WATTO) */
  ["\u3358", "\u3358"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ZERO */
  ["\u3359", "\u3359"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ONE */
  ["\u335A", "\u335A"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWO */
  ["\u335B", "\u335B"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THREE */
  ["\u335C", "\u335C"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOUR */
  ["\u335D", "\u335D"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIVE */
  ["\u335E", "\u335E"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIX */
  ["\u335F", "\u335F"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVEN */
  ["\u3360", "\u3360"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHT */
  ["\u3361", "\u3361"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINE */
  ["\u3362", "\u3362"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TEN */
  ["\u3363", "\u3363"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR ELEVEN */
  ["\u3364", "\u3364"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWELVE */
  ["\u3365", "\u3365"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR THIRTEEN */
  ["\u3366", "\u3366"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FOURTEEN */
  ["\u3367", "\u3367"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR FIFTEEN */
  ["\u3368", "\u3368"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SIXTEEN */
  ["\u3369", "\u3369"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR SEVENTEEN */
  ["\u336A", "\u336A"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR EIGHTEEN */
  ["\u336B", "\u336B"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR NINETEEN */
  ["\u336C", "\u336C"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY */
  ["\u336D", "\u336D"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-ONE */
  ["\u336E", "\u336E"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-TWO */
  ["\u336F", "\u336F"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-THREE */
  ["\u3370", "\u3370"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR HOUR TWENTY-FOUR */
  ["\u3371", "\u3371"], /* SQUARE HPA */
  ["\u3372", "\u3372"], /* SQUARE DA */
  ["\u3373", "\u3373"], /* SQUARE AU */
  ["\u3374", "\u3374"], /* SQUARE BAR */
  ["\u3375", "\u3375"], /* SQUARE OV */
  ["\u3376", "\u3376"], /* SQUARE PC */
  ["\u3377", "\u3377"], /* SQUARE DM */
  ["\u3378", "\u3378"], /* SQUARE DM SQUARED */
  ["\u3379", "\u3379"], /* SQUARE DM CUBED */
  ["\u337A", "\u337A"], /* SQUARE IU */
  ["\u337B", "\u337B"], /* SQUARE ERA NAME HEISEI (SQUARED TWO IDEOGRAPHS ERA NAME HEISEI) */
  ["\u337C", "\u337C"], /* SQUARE ERA NAME SYOUWA (SQUARED TWO IDEOGRAPHS ERA NAME SYOUWA) */
  ["\u337D", "\u337D"], /* SQUARE ERA NAME TAISYOU (SQUARED TWO IDEOGRAPHS ERA NAME TAISYOU) */
  ["\u337E", "\u337E"], /* SQUARE ERA NAME MEIZI (SQUARED TWO IDEOGRAPHS ERA NAME MEIZI) */
  ["\u337F", "\u337F"], /* SQUARE CORPORATION (SQUARED FOUR IDEOGRAPHS CORPORATION) */
  ["\u3380", "\u3380"], /* SQUARE PA AMPS (SQUARED PA AMPS) */
  ["\u3381", "\u3381"], /* SQUARE NA (SQUARED NA) */
  ["\u3382", "\u3382"], /* SQUARE MU A (SQUARED MU A) */
  ["\u3383", "\u3383"], /* SQUARE MA (SQUARED MA) */
  ["\u3384", "\u3384"], /* SQUARE KA (SQUARED KA) */
  ["\u3385", "\u3385"], /* SQUARE KB (SQUARED KB) */
  ["\u3386", "\u3386"], /* SQUARE MB (SQUARED MB) */
  ["\u3387", "\u3387"], /* SQUARE GB (SQUARED GB) */
  ["\u3388", "\u3388"], /* SQUARE CAL (SQUARED CAL) */
  ["\u3389", "\u3389"], /* SQUARE KCAL (SQUARED KCAL) */
  ["\u338A", "\u338A"], /* SQUARE PF (SQUARED PF) */
  ["\u338B", "\u338B"], /* SQUARE NF (SQUARED NF) */
  ["\u338C", "\u338C"], /* SQUARE MU F (SQUARED MU F) */
  ["\u338D", "\u338D"], /* SQUARE MU G (SQUARED MU G) */
  ["\u338E", "\u338E"], /* SQUARE MG (SQUARED MG) */
  ["\u338F", "\u338F"], /* SQUARE KG (SQUARED KG) */
  ["\u3390", "\u3390"], /* SQUARE HZ (SQUARED HZ) */
  ["\u3391", "\u3391"], /* SQUARE KHZ (SQUARED KHZ) */
  ["\u3392", "\u3392"], /* SQUARE MHZ (SQUARED MHZ) */
  ["\u3393", "\u3393"], /* SQUARE GHZ (SQUARED GHZ) */
  ["\u3394", "\u3394"], /* SQUARE THZ (SQUARED THZ) */
  ["\u3395", "\u3395"], /* SQUARE MU L (SQUARED MU L) */
  ["\u3396", "\u3396"], /* SQUARE ML (SQUARED ML) */
  ["\u3397", "\u3397"], /* SQUARE DL (SQUARED DL) */
  ["\u3398", "\u3398"], /* SQUARE KL (SQUARED KL) */
  ["\u3399", "\u3399"], /* SQUARE FM (SQUARED FM) */
  ["\u339A", "\u339A"], /* SQUARE NM (SQUARED NM) */
  ["\u339B", "\u339B"], /* SQUARE MU M (SQUARED MU M) */
  ["\u339C", "\u339C"], /* SQUARE MM (SQUARED MM) */
  ["\u339D", "\u339D"], /* SQUARE CM (SQUARED CM) */
  ["\u339E", "\u339E"], /* SQUARE KM (SQUARED KM) */
  ["\u339F", "\u339F"], /* SQUARE MM SQUARED (SQUARED MM SQUARED) */
  ["\u33A0", "\u33A0"], /* SQUARE CM SQUARED (SQUARED CM SQUARED) */
  ["\u33A1", "\u33A1"], /* SQUARE M SQUARED (SQUARED M SQUARED) */
  ["\u33A2", "\u33A2"], /* SQUARE KM SQUARED (SQUARED KM SQUARED) */
  ["\u33A3", "\u33A3"], /* SQUARE MM CUBED (SQUARED MM CUBED) */
  ["\u33A4", "\u33A4"], /* SQUARE CM CUBED (SQUARED CM CUBED) */
  ["\u33A5", "\u33A5"], /* SQUARE M CUBED (SQUARED M CUBED) */
  ["\u33A6", "\u33A6"], /* SQUARE KM CUBED (SQUARED KM CUBED) */
  ["\u33A7", "\u33A7"], /* SQUARE M OVER S (SQUARED M OVER S) */
  ["\u33A8", "\u33A8"], /* SQUARE M OVER S SQUARED (SQUARED M OVER S SQUARED) */
  ["\u33A9", "\u33A9"], /* SQUARE PA (SQUARED PA) */
  ["\u33AA", "\u33AA"], /* SQUARE KPA (SQUARED KPA) */
  ["\u33AB", "\u33AB"], /* SQUARE MPA (SQUARED MPA) */
  ["\u33AC", "\u33AC"], /* SQUARE GPA (SQUARED GPA) */
  ["\u33AD", "\u33AD"], /* SQUARE RAD (SQUARED RAD) */
  ["\u33AE", "\u33AE"], /* SQUARE RAD OVER S (SQUARED RAD OVER S) */
  ["\u33AF", "\u33AF"], /* SQUARE RAD OVER S SQUARED (SQUARED RAD OVER S SQUARED) */
  ["\u33B0", "\u33B0"], /* SQUARE PS (SQUARED PS) */
  ["\u33B1", "\u33B1"], /* SQUARE NS (SQUARED NS) */
  ["\u33B2", "\u33B2"], /* SQUARE MU S (SQUARED MU S) */
  ["\u33B3", "\u33B3"], /* SQUARE MS (SQUARED MS) */
  ["\u33B4", "\u33B4"], /* SQUARE PV (SQUARED PV) */
  ["\u33B5", "\u33B5"], /* SQUARE NV (SQUARED NV) */
  ["\u33B6", "\u33B6"], /* SQUARE MU V (SQUARED MU V) */
  ["\u33B7", "\u33B7"], /* SQUARE MV (SQUARED MV) */
  ["\u33B8", "\u33B8"], /* SQUARE KV (SQUARED KV) */
  ["\u33B9", "\u33B9"], /* SQUARE MV MEGA (SQUARED MV MEGA) */
  ["\u33BA", "\u33BA"], /* SQUARE PW (SQUARED PW) */
  ["\u33BB", "\u33BB"], /* SQUARE NW (SQUARED NW) */
  ["\u33BC", "\u33BC"], /* SQUARE MU W (SQUARED MU W) */
  ["\u33BD", "\u33BD"], /* SQUARE MW (SQUARED MW) */
  ["\u33BE", "\u33BE"], /* SQUARE KW (SQUARED KW) */
  ["\u33BF", "\u33BF"], /* SQUARE MW MEGA (SQUARED MW MEGA) */
  ["\u33C0", "\u33C0"], /* SQUARE K OHM (SQUARED K OHM) */
  ["\u33C1", "\u33C1"], /* SQUARE M OHM (SQUARED M OHM) */
  ["\u33C2", "\u33C2"], /* SQUARE AM (SQUARED AM) */
  ["\u33C3", "\u33C3"], /* SQUARE BQ (SQUARED BQ) */
  ["\u33C4", "\u33C4"], /* SQUARE CC (SQUARED CC) */
  ["\u33C5", "\u33C5"], /* SQUARE CD (SQUARED CD) */
  ["\u33C6", "\u33C6"], /* SQUARE C OVER KG (SQUARED C OVER KG) */
  ["\u33C7", "\u33C7"], /* SQUARE CO (SQUARED CO) */
  ["\u33C8", "\u33C8"], /* SQUARE DB (SQUARED DB) */
  ["\u33C9", "\u33C9"], /* SQUARE GY (SQUARED GY) */
  ["\u33CA", "\u33CA"], /* SQUARE HA (SQUARED HA) */
  ["\u33CB", "\u33CB"], /* SQUARE HP (SQUARED HP) */
  ["\u33CC", "\u33CC"], /* SQUARE IN (SQUARED IN) */
  ["\u33CD", "\u33CD"], /* SQUARE KK (SQUARED KK) */
  ["\u33CE", "\u33CE"], /* SQUARE KM CAPITAL (SQUARED KM CAPITAL) */
  ["\u33CF", "\u33CF"], /* SQUARE KT (SQUARED KT) */
  ["\u33D0", "\u33D0"], /* SQUARE LM (SQUARED LM) */
  ["\u33D1", "\u33D1"], /* SQUARE LN (SQUARED LN) */
  ["\u33D2", "\u33D2"], /* SQUARE LOG (SQUARED LOG) */
  ["\u33D3", "\u33D3"], /* SQUARE LX (SQUARED LX) */
  ["\u33D4", "\u33D4"], /* SQUARE MB SMALL (SQUARED MB SMALL) */
  ["\u33D5", "\u33D5"], /* SQUARE MIL (SQUARED MIL) */
  ["\u33D6", "\u33D6"], /* SQUARE MOL (SQUARED MOL) */
  ["\u33D7", "\u33D7"], /* SQUARE PH (SQUARED PH) */
  ["\u33D8", "\u33D8"], /* SQUARE PM (SQUARED PM) */
  ["\u33D9", "\u33D9"], /* SQUARE PPM (SQUARED PPM) */
  ["\u33DA", "\u33DA"], /* SQUARE PR (SQUARED PR) */
  ["\u33DB", "\u33DB"], /* SQUARE SR (SQUARED SR) */
  ["\u33DC", "\u33DC"], /* SQUARE SV (SQUARED SV) */
  ["\u33DD", "\u33DD"], /* SQUARE WB (SQUARED WB) */
  ["\u33DE", "\u33DE"], /* SQUARE V OVER M */
  ["\u33DF", "\u33DF"], /* SQUARE A OVER M */
  ["\u33E0", "\u33E0"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ONE */
  ["\u33E1", "\u33E1"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWO */
  ["\u33E2", "\u33E2"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THREE */
  ["\u33E3", "\u33E3"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOUR */
  ["\u33E4", "\u33E4"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIVE */
  ["\u33E5", "\u33E5"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIX */
  ["\u33E6", "\u33E6"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVEN */
  ["\u33E7", "\u33E7"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHT */
  ["\u33E8", "\u33E8"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINE */
  ["\u33E9", "\u33E9"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TEN */
  ["\u33EA", "\u33EA"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY ELEVEN */
  ["\u33EB", "\u33EB"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWELVE */
  ["\u33EC", "\u33EC"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTEEN */
  ["\u33ED", "\u33ED"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FOURTEEN */
  ["\u33EE", "\u33EE"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY FIFTEEN */
  ["\u33EF", "\u33EF"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SIXTEEN */
  ["\u33F0", "\u33F0"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY SEVENTEEN */
  ["\u33F1", "\u33F1"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY EIGHTEEN */
  ["\u33F2", "\u33F2"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY NINETEEN */
  ["\u33F3", "\u33F3"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY */
  ["\u33F4", "\u33F4"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-ONE */
  ["\u33F5", "\u33F5"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-TWO */
  ["\u33F6", "\u33F6"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-THREE */
  ["\u33F7", "\u33F7"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FOUR */
  ["\u33F8", "\u33F8"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-FIVE */
  ["\u33F9", "\u33F9"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SIX */
  ["\u33FA", "\u33FA"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-SEVEN */
  ["\u33FB", "\u33FB"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-EIGHT */
  ["\u33FC", "\u33FC"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY TWENTY-NINE */
  ["\u33FD", "\u33FD"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY */
  ["\u33FE", "\u33FE"], /* IDEOGRAPHIC TELEGRAPH SYMBOL FOR DAY THIRTY-ONE */
  ["\u33FF", "\u33FF"], /* SQUARE GAL */
  ["\u3400", "\u3400"], /* CJK Ideograph Extension A */
  ["\u3401", "\u3401"], /* CJK Ideograph Extension A */
  ["\u3402", "\u3402"], /* CJK Ideograph Extension A */
  ["\u3403", "\u3403"], /* CJK Ideograph Extension A */
  ["\u3404", "\u3404"], /* CJK Ideograph Extension A */
  ["\u3405", "\u3405"], /* CJK Ideograph Extension A */
  ["\u3406", "\u3406"], /* CJK Ideograph Extension A */
  ["\u3407", "\u3407"], /* CJK Ideograph Extension A */
  ["\u3408", "\u3408"], /* CJK Ideograph Extension A */
  ["\u3409", "\u3409"], /* CJK Ideograph Extension A */
  ["\u340A", "\u340A"], /* CJK Ideograph Extension A */
  ["\u340B", "\u340B"], /* CJK Ideograph Extension A */
  ["\u340C", "\u340C"], /* CJK Ideograph Extension A */
  ["\u340D", "\u340D"], /* CJK Ideograph Extension A */
  ["\u340E", "\u340E"], /* CJK Ideograph Extension A */
  ["\u340F", "\u340F"], /* CJK Ideograph Extension A */
  ["\u3410", "\u3410"], /* CJK Ideograph Extension A */
  ["\u3411", "\u3411"], /* CJK Ideograph Extension A */
  ["\u3412", "\u3412"], /* CJK Ideograph Extension A */
  ["\u3413", "\u3413"], /* CJK Ideograph Extension A */
  ["\u3414", "\u3414"], /* CJK Ideograph Extension A */
  ["\u3415", "\u3415"], /* CJK Ideograph Extension A */
  ["\u3416", "\u3416"], /* CJK Ideograph Extension A */
  ["\u3417", "\u3417"], /* CJK Ideograph Extension A */
  ["\u3418", "\u3418"], /* CJK Ideograph Extension A */
  ["\u3419", "\u3419"], /* CJK Ideograph Extension A */
  ["\u341A", "\u341A"], /* CJK Ideograph Extension A */
  ["\u341B", "\u341B"], /* CJK Ideograph Extension A */
  ["\u341C", "\u341C"], /* CJK Ideograph Extension A */
  ["\u341D", "\u341D"], /* CJK Ideograph Extension A */
  ["\u341E", "\u341E"], /* CJK Ideograph Extension A */
  ["\u341F", "\u341F"], /* CJK Ideograph Extension A */
  ["\u3420", "\u3420"], /* CJK Ideograph Extension A */
  ["\u3421", "\u3421"], /* CJK Ideograph Extension A */
  ["\u3422", "\u3422"], /* CJK Ideograph Extension A */
  ["\u3423", "\u3423"], /* CJK Ideograph Extension A */
  ["\u3424", "\u3424"], /* CJK Ideograph Extension A */
  ["\u3425", "\u3425"], /* CJK Ideograph Extension A */
  ["\u3426", "\u3426"], /* CJK Ideograph Extension A */
  ["\u3427", "\u3427"], /* CJK Ideograph Extension A */
  ["\u3428", "\u3428"], /* CJK Ideograph Extension A */
  ["\u3429", "\u3429"], /* CJK Ideograph Extension A */
  ["\u342A", "\u342A"], /* CJK Ideograph Extension A */
  ["\u342B", "\u342B"], /* CJK Ideograph Extension A */
  ["\u342C", "\u342C"], /* CJK Ideograph Extension A */
  ["\u342D", "\u342D"], /* CJK Ideograph Extension A */
  ["\u342E", "\u342E"], /* CJK Ideograph Extension A */
  ["\u342F", "\u342F"], /* CJK Ideograph Extension A */
  ["\u3430", "\u3430"], /* CJK Ideograph Extension A */
  ["\u3431", "\u3431"], /* CJK Ideograph Extension A */
  ["\u3432", "\u3432"], /* CJK Ideograph Extension A */
  ["\u3433", "\u3433"], /* CJK Ideograph Extension A */
  ["\u3434", "\u3434"], /* CJK Ideograph Extension A */
  ["\u3435", "\u3435"], /* CJK Ideograph Extension A */
  ["\u3436", "\u3436"], /* CJK Ideograph Extension A */
  ["\u3437", "\u3437"], /* CJK Ideograph Extension A */
  ["\u3438", "\u3438"], /* CJK Ideograph Extension A */
  ["\u3439", "\u3439"], /* CJK Ideograph Extension A */
  ["\u343A", "\u343A"], /* CJK Ideograph Extension A */
  ["\u343B", "\u343B"], /* CJK Ideograph Extension A */
  ["\u343C", "\u343C"], /* CJK Ideograph Extension A */
  ["\u343D", "\u343D"], /* CJK Ideograph Extension A */
  ["\u343E", "\u343E"], /* CJK Ideograph Extension A */
  ["\u343F", "\u343F"], /* CJK Ideograph Extension A */
  ["\u3440", "\u3440"], /* CJK Ideograph Extension A */
  ["\u3441", "\u3441"], /* CJK Ideograph Extension A */
  ["\u3442", "\u3442"], /* CJK Ideograph Extension A */
  ["\u3443", "\u3443"], /* CJK Ideograph Extension A */
  ["\u3444", "\u3444"], /* CJK Ideograph Extension A */
  ["\u3445", "\u3445"], /* CJK Ideograph Extension A */
  ["\u3446", "\u3446"], /* CJK Ideograph Extension A */
  ["\u3447", "\u3447"], /* CJK Ideograph Extension A */
  ["\u3448", "\u3448"], /* CJK Ideograph Extension A */
  ["\u3449", "\u3449"], /* CJK Ideograph Extension A */
  ["\u344A", "\u344A"], /* CJK Ideograph Extension A */
  ["\u344B", "\u344B"], /* CJK Ideograph Extension A */
  ["\u344C", "\u344C"], /* CJK Ideograph Extension A */
  ["\u344D", "\u344D"], /* CJK Ideograph Extension A */
  ["\u344E", "\u344E"], /* CJK Ideograph Extension A */
  ["\u344F", "\u344F"], /* CJK Ideograph Extension A */
  ["\u3450", "\u3450"], /* CJK Ideograph Extension A */
  ["\u3451", "\u3451"], /* CJK Ideograph Extension A */
  ["\u3452", "\u3452"], /* CJK Ideograph Extension A */
  ["\u3453", "\u3453"], /* CJK Ideograph Extension A */
  ["\u3454", "\u3454"], /* CJK Ideograph Extension A */
  ["\u3455", "\u3455"], /* CJK Ideograph Extension A */
  ["\u3456", "\u3456"], /* CJK Ideograph Extension A */
  ["\u3457", "\u3457"], /* CJK Ideograph Extension A */
  ["\u3458", "\u3458"], /* CJK Ideograph Extension A */
  ["\u3459", "\u3459"], /* CJK Ideograph Extension A */
  ["\u345A", "\u345A"], /* CJK Ideograph Extension A */
  ["\u345B", "\u345B"], /* CJK Ideograph Extension A */
  ["\u345C", "\u345C"], /* CJK Ideograph Extension A */
  ["\u345D", "\u345D"], /* CJK Ideograph Extension A */
  ["\u345E", "\u345E"], /* CJK Ideograph Extension A */
  ["\u345F", "\u345F"], /* CJK Ideograph Extension A */
  ["\u3460", "\u3460"], /* CJK Ideograph Extension A */
  ["\u3461", "\u3461"], /* CJK Ideograph Extension A */
  ["\u3462", "\u3462"], /* CJK Ideograph Extension A */
  ["\u3463", "\u3463"], /* CJK Ideograph Extension A */
  ["\u3464", "\u3464"], /* CJK Ideograph Extension A */
  ["\u3465", "\u3465"], /* CJK Ideograph Extension A */
  ["\u3466", "\u3466"], /* CJK Ideograph Extension A */
  ["\u3467", "\u3467"], /* CJK Ideograph Extension A */
  ["\u3468", "\u3468"], /* CJK Ideograph Extension A */
  ["\u3469", "\u3469"], /* CJK Ideograph Extension A */
  ["\u346A", "\u346A"], /* CJK Ideograph Extension A */
  ["\u346B", "\u346B"], /* CJK Ideograph Extension A */
  ["\u346C", "\u346C"], /* CJK Ideograph Extension A */
  ["\u346D", "\u346D"], /* CJK Ideograph Extension A */
  ["\u346E", "\u346E"], /* CJK Ideograph Extension A */
  ["\u346F", "\u346F"], /* CJK Ideograph Extension A */
  ["\u3470", "\u3470"], /* CJK Ideograph Extension A */
  ["\u3471", "\u3471"], /* CJK Ideograph Extension A */
  ["\u3472", "\u3472"], /* CJK Ideograph Extension A */
  ["\u3473", "\u3473"], /* CJK Ideograph Extension A */
  ["\u3474", "\u3474"], /* CJK Ideograph Extension A */
  ["\u3475", "\u3475"], /* CJK Ideograph Extension A */
  ["\u3476", "\u3476"], /* CJK Ideograph Extension A */
  ["\u3477", "\u3477"], /* CJK Ideograph Extension A */
  ["\u3478", "\u3478"], /* CJK Ideograph Extension A */
  ["\u3479", "\u3479"], /* CJK Ideograph Extension A */
  ["\u347A", "\u347A"], /* CJK Ideograph Extension A */
  ["\u347B", "\u347B"], /* CJK Ideograph Extension A */
  ["\u347C", "\u347C"], /* CJK Ideograph Extension A */
  ["\u347D", "\u347D"], /* CJK Ideograph Extension A */
  ["\u347E", "\u347E"], /* CJK Ideograph Extension A */
  ["\u347F", "\u347F"], /* CJK Ideograph Extension A */
  ["\u3480", "\u3480"], /* CJK Ideograph Extension A */
  ["\u3481", "\u3481"], /* CJK Ideograph Extension A */
  ["\u3482", "\u3482"], /* CJK Ideograph Extension A */
  ["\u3483", "\u3483"], /* CJK Ideograph Extension A */
  ["\u3484", "\u3484"], /* CJK Ideograph Extension A */
  ["\u3485", "\u3485"], /* CJK Ideograph Extension A */
  ["\u3486", "\u3486"], /* CJK Ideograph Extension A */
  ["\u3487", "\u3487"], /* CJK Ideograph Extension A */
  ["\u3488", "\u3488"], /* CJK Ideograph Extension A */
  ["\u3489", "\u3489"], /* CJK Ideograph Extension A */
  ["\u348A", "\u348A"], /* CJK Ideograph Extension A */
  ["\u348B", "\u348B"], /* CJK Ideograph Extension A */
  ["\u348C", "\u348C"], /* CJK Ideograph Extension A */
  ["\u348D", "\u348D"], /* CJK Ideograph Extension A */
  ["\u348E", "\u348E"], /* CJK Ideograph Extension A */
  ["\u348F", "\u348F"], /* CJK Ideograph Extension A */
  ["\u3490", "\u3490"], /* CJK Ideograph Extension A */
  ["\u3491", "\u3491"], /* CJK Ideograph Extension A */
  ["\u3492", "\u3492"], /* CJK Ideograph Extension A */
  ["\u3493", "\u3493"], /* CJK Ideograph Extension A */
  ["\u3494", "\u3494"], /* CJK Ideograph Extension A */
  ["\u3495", "\u3495"], /* CJK Ideograph Extension A */
  ["\u3496", "\u3496"], /* CJK Ideograph Extension A */
  ["\u3497", "\u3497"], /* CJK Ideograph Extension A */
  ["\u3498", "\u3498"], /* CJK Ideograph Extension A */
  ["\u3499", "\u3499"], /* CJK Ideograph Extension A */
  ["\u349A", "\u349A"], /* CJK Ideograph Extension A */
  ["\u349B", "\u349B"], /* CJK Ideograph Extension A */
  ["\u349C", "\u349C"], /* CJK Ideograph Extension A */
  ["\u349D", "\u349D"], /* CJK Ideograph Extension A */
  ["\u349E", "\u349E"], /* CJK Ideograph Extension A */
  ["\u349F", "\u349F"], /* CJK Ideograph Extension A */
  ["\u34A0", "\u34A0"], /* CJK Ideograph Extension A */
  ["\u34A1", "\u34A1"], /* CJK Ideograph Extension A */
  ["\u34A2", "\u34A2"], /* CJK Ideograph Extension A */
  ["\u34A3", "\u34A3"], /* CJK Ideograph Extension A */
  ["\u34A4", "\u34A4"], /* CJK Ideograph Extension A */
  ["\u34A5", "\u34A5"], /* CJK Ideograph Extension A */
  ["\u34A6", "\u34A6"], /* CJK Ideograph Extension A */
  ["\u34A7", "\u34A7"], /* CJK Ideograph Extension A */
  ["\u34A8", "\u34A8"], /* CJK Ideograph Extension A */
  ["\u34A9", "\u34A9"], /* CJK Ideograph Extension A */
  ["\u34AA", "\u34AA"], /* CJK Ideograph Extension A */
  ["\u34AB", "\u34AB"], /* CJK Ideograph Extension A */
  ["\u34AC", "\u34AC"], /* CJK Ideograph Extension A */
  ["\u34AD", "\u34AD"], /* CJK Ideograph Extension A */
  ["\u34AE", "\u34AE"], /* CJK Ideograph Extension A */
  ["\u34AF", "\u34AF"], /* CJK Ideograph Extension A */
  ["\u34B0", "\u34B0"], /* CJK Ideograph Extension A */
  ["\u34B1", "\u34B1"], /* CJK Ideograph Extension A */
  ["\u34B2", "\u34B2"], /* CJK Ideograph Extension A */
  ["\u34B3", "\u34B3"], /* CJK Ideograph Extension A */
  ["\u34B4", "\u34B4"], /* CJK Ideograph Extension A */
  ["\u34B5", "\u34B5"], /* CJK Ideograph Extension A */
  ["\u34B6", "\u34B6"], /* CJK Ideograph Extension A */
  ["\u34B7", "\u34B7"], /* CJK Ideograph Extension A */
  ["\u34B8", "\u34B8"], /* CJK Ideograph Extension A */
  ["\u34B9", "\u34B9"], /* CJK Ideograph Extension A */
  ["\u34BA", "\u34BA"], /* CJK Ideograph Extension A */
  ["\u34BB", "\u34BB"], /* CJK Ideograph Extension A */
  ["\u34BC", "\u34BC"], /* CJK Ideograph Extension A */
  ["\u34BD", "\u34BD"], /* CJK Ideograph Extension A */
  ["\u34BE", "\u34BE"], /* CJK Ideograph Extension A */
  ["\u34BF", "\u34BF"], /* CJK Ideograph Extension A */
  ["\u34C0", "\u34C0"], /* CJK Ideograph Extension A */
  ["\u34C1", "\u34C1"], /* CJK Ideograph Extension A */
  ["\u34C2", "\u34C2"], /* CJK Ideograph Extension A */
  ["\u34C3", "\u34C3"], /* CJK Ideograph Extension A */
  ["\u34C4", "\u34C4"], /* CJK Ideograph Extension A */
  ["\u34C5", "\u34C5"], /* CJK Ideograph Extension A */
  ["\u34C6", "\u34C6"], /* CJK Ideograph Extension A */
  ["\u34C7", "\u34C7"], /* CJK Ideograph Extension A */
  ["\u34C8", "\u34C8"], /* CJK Ideograph Extension A */
  ["\u34C9", "\u34C9"], /* CJK Ideograph Extension A */
  ["\u34CA", "\u34CA"], /* CJK Ideograph Extension A */
  ["\u34CB", "\u34CB"], /* CJK Ideograph Extension A */
  ["\u34CC", "\u34CC"], /* CJK Ideograph Extension A */
  ["\u34CD", "\u34CD"], /* CJK Ideograph Extension A */
  ["\u34CE", "\u34CE"], /* CJK Ideograph Extension A */
  ["\u34CF", "\u34CF"], /* CJK Ideograph Extension A */
  ["\u34D0", "\u34D0"], /* CJK Ideograph Extension A */
  ["\u34D1", "\u34D1"], /* CJK Ideograph Extension A */
  ["\u34D2", "\u34D2"], /* CJK Ideograph Extension A */
  ["\u34D3", "\u34D3"], /* CJK Ideograph Extension A */
  ["\u34D4", "\u34D4"], /* CJK Ideograph Extension A */
  ["\u34D5", "\u34D5"], /* CJK Ideograph Extension A */
  ["\u34D6", "\u34D6"], /* CJK Ideograph Extension A */
  ["\u34D7", "\u34D7"], /* CJK Ideograph Extension A */
  ["\u34D8", "\u34D8"], /* CJK Ideograph Extension A */
  ["\u34D9", "\u34D9"], /* CJK Ideograph Extension A */
  ["\u34DA", "\u34DA"], /* CJK Ideograph Extension A */
  ["\u34DB", "\u34DB"], /* CJK Ideograph Extension A */
  ["\u34DC", "\u34DC"], /* CJK Ideograph Extension A */
  ["\u34DD", "\u34DD"], /* CJK Ideograph Extension A */
  ["\u34DE", "\u34DE"], /* CJK Ideograph Extension A */
  ["\u34DF", "\u34DF"], /* CJK Ideograph Extension A */
  ["\u34E0", "\u34E0"], /* CJK Ideograph Extension A */
  ["\u34E1", "\u34E1"], /* CJK Ideograph Extension A */
  ["\u34E2", "\u34E2"], /* CJK Ideograph Extension A */
  ["\u34E3", "\u34E3"], /* CJK Ideograph Extension A */
  ["\u34E4", "\u34E4"], /* CJK Ideograph Extension A */
  ["\u34E5", "\u34E5"], /* CJK Ideograph Extension A */
  ["\u34E6", "\u34E6"], /* CJK Ideograph Extension A */
  ["\u34E7", "\u34E7"], /* CJK Ideograph Extension A */
  ["\u34E8", "\u34E8"], /* CJK Ideograph Extension A */
  ["\u34E9", "\u34E9"], /* CJK Ideograph Extension A */
  ["\u34EA", "\u34EA"], /* CJK Ideograph Extension A */
  ["\u34EB", "\u34EB"], /* CJK Ideograph Extension A */
  ["\u34EC", "\u34EC"], /* CJK Ideograph Extension A */
  ["\u34ED", "\u34ED"], /* CJK Ideograph Extension A */
  ["\u34EE", "\u34EE"], /* CJK Ideograph Extension A */
  ["\u34EF", "\u34EF"], /* CJK Ideograph Extension A */
  ["\u34F0", "\u34F0"], /* CJK Ideograph Extension A */
  ["\u34F1", "\u34F1"], /* CJK Ideograph Extension A */
  ["\u34F2", "\u34F2"], /* CJK Ideograph Extension A */
  ["\u34F3", "\u34F3"], /* CJK Ideograph Extension A */
  ["\u34F4", "\u34F4"], /* CJK Ideograph Extension A */
  ["\u34F5", "\u34F5"], /* CJK Ideograph Extension A */
  ["\u34F6", "\u34F6"], /* CJK Ideograph Extension A */
  ["\u34F7", "\u34F7"], /* CJK Ideograph Extension A */
  ["\u34F8", "\u34F8"], /* CJK Ideograph Extension A */
  ["\u34F9", "\u34F9"], /* CJK Ideograph Extension A */
  ["\u34FA", "\u34FA"], /* CJK Ideograph Extension A */
  ["\u34FB", "\u34FB"], /* CJK Ideograph Extension A */
  ["\u34FC", "\u34FC"], /* CJK Ideograph Extension A */
  ["\u34FD", "\u34FD"], /* CJK Ideograph Extension A */
  ["\u34FE", "\u34FE"], /* CJK Ideograph Extension A */
  ["\u34FF", "\u34FF"], /* CJK Ideograph Extension A */
  ["\u3500", "\u3500"], /* CJK Ideograph Extension A */
  ["\u3501", "\u3501"], /* CJK Ideograph Extension A */
  ["\u3502", "\u3502"], /* CJK Ideograph Extension A */
  ["\u3503", "\u3503"], /* CJK Ideograph Extension A */
  ["\u3504", "\u3504"], /* CJK Ideograph Extension A */
  ["\u3505", "\u3505"], /* CJK Ideograph Extension A */
  ["\u3506", "\u3506"], /* CJK Ideograph Extension A */
  ["\u3507", "\u3507"], /* CJK Ideograph Extension A */
  ["\u3508", "\u3508"], /* CJK Ideograph Extension A */
  ["\u3509", "\u3509"], /* CJK Ideograph Extension A */
  ["\u350A", "\u350A"], /* CJK Ideograph Extension A */
  ["\u350B", "\u350B"], /* CJK Ideograph Extension A */
  ["\u350C", "\u350C"], /* CJK Ideograph Extension A */
  ["\u350D", "\u350D"], /* CJK Ideograph Extension A */
  ["\u350E", "\u350E"], /* CJK Ideograph Extension A */
  ["\u350F", "\u350F"], /* CJK Ideograph Extension A */
  ["\u3510", "\u3510"], /* CJK Ideograph Extension A */
  ["\u3511", "\u3511"], /* CJK Ideograph Extension A */
  ["\u3512", "\u3512"], /* CJK Ideograph Extension A */
  ["\u3513", "\u3513"], /* CJK Ideograph Extension A */
  ["\u3514", "\u3514"], /* CJK Ideograph Extension A */
  ["\u3515", "\u3515"], /* CJK Ideograph Extension A */
  ["\u3516", "\u3516"], /* CJK Ideograph Extension A */
  ["\u3517", "\u3517"], /* CJK Ideograph Extension A */
  ["\u3518", "\u3518"], /* CJK Ideograph Extension A */
  ["\u3519", "\u3519"], /* CJK Ideograph Extension A */
  ["\u351A", "\u351A"], /* CJK Ideograph Extension A */
  ["\u351B", "\u351B"], /* CJK Ideograph Extension A */
  ["\u351C", "\u351C"], /* CJK Ideograph Extension A */
  ["\u351D", "\u351D"], /* CJK Ideograph Extension A */
  ["\u351E", "\u351E"], /* CJK Ideograph Extension A */
  ["\u351F", "\u351F"], /* CJK Ideograph Extension A */
  ["\u3520", "\u3520"], /* CJK Ideograph Extension A */
  ["\u3521", "\u3521"], /* CJK Ideograph Extension A */
  ["\u3522", "\u3522"], /* CJK Ideograph Extension A */
  ["\u3523", "\u3523"], /* CJK Ideograph Extension A */
  ["\u3524", "\u3524"], /* CJK Ideograph Extension A */
  ["\u3525", "\u3525"], /* CJK Ideograph Extension A */
  ["\u3526", "\u3526"], /* CJK Ideograph Extension A */
  ["\u3527", "\u3527"], /* CJK Ideograph Extension A */
  ["\u3528", "\u3528"], /* CJK Ideograph Extension A */
  ["\u3529", "\u3529"], /* CJK Ideograph Extension A */
  ["\u352A", "\u352A"], /* CJK Ideograph Extension A */
  ["\u352B", "\u352B"], /* CJK Ideograph Extension A */
  ["\u352C", "\u352C"], /* CJK Ideograph Extension A */
  ["\u352D", "\u352D"], /* CJK Ideograph Extension A */
  ["\u352E", "\u352E"], /* CJK Ideograph Extension A */
  ["\u352F", "\u352F"], /* CJK Ideograph Extension A */
  ["\u3530", "\u3530"], /* CJK Ideograph Extension A */
  ["\u3531", "\u3531"], /* CJK Ideograph Extension A */
  ["\u3532", "\u3532"], /* CJK Ideograph Extension A */
  ["\u3533", "\u3533"], /* CJK Ideograph Extension A */
  ["\u3534", "\u3534"], /* CJK Ideograph Extension A */
  ["\u3535", "\u3535"], /* CJK Ideograph Extension A */
  ["\u3536", "\u3536"], /* CJK Ideograph Extension A */
  ["\u3537", "\u3537"], /* CJK Ideograph Extension A */
  ["\u3538", "\u3538"], /* CJK Ideograph Extension A */
  ["\u3539", "\u3539"], /* CJK Ideograph Extension A */
  ["\u353A", "\u353A"], /* CJK Ideograph Extension A */
  ["\u353B", "\u353B"], /* CJK Ideograph Extension A */
  ["\u353C", "\u353C"], /* CJK Ideograph Extension A */
  ["\u353D", "\u353D"], /* CJK Ideograph Extension A */
  ["\u353E", "\u353E"], /* CJK Ideograph Extension A */
  ["\u353F", "\u353F"], /* CJK Ideograph Extension A */
  ["\u3540", "\u3540"], /* CJK Ideograph Extension A */
  ["\u3541", "\u3541"], /* CJK Ideograph Extension A */
  ["\u3542", "\u3542"], /* CJK Ideograph Extension A */
  ["\u3543", "\u3543"], /* CJK Ideograph Extension A */
  ["\u3544", "\u3544"], /* CJK Ideograph Extension A */
  ["\u3545", "\u3545"], /* CJK Ideograph Extension A */
  ["\u3546", "\u3546"], /* CJK Ideograph Extension A */
  ["\u3547", "\u3547"], /* CJK Ideograph Extension A */
  ["\u3548", "\u3548"], /* CJK Ideograph Extension A */
  ["\u3549", "\u3549"], /* CJK Ideograph Extension A */
  ["\u354A", "\u354A"], /* CJK Ideograph Extension A */
  ["\u354B", "\u354B"], /* CJK Ideograph Extension A */
  ["\u354C", "\u354C"], /* CJK Ideograph Extension A */
  ["\u354D", "\u354D"], /* CJK Ideograph Extension A */
  ["\u354E", "\u354E"], /* CJK Ideograph Extension A */
  ["\u354F", "\u354F"], /* CJK Ideograph Extension A */
  ["\u3550", "\u3550"], /* CJK Ideograph Extension A */
  ["\u3551", "\u3551"], /* CJK Ideograph Extension A */
  ["\u3552", "\u3552"], /* CJK Ideograph Extension A */
  ["\u3553", "\u3553"], /* CJK Ideograph Extension A */
  ["\u3554", "\u3554"], /* CJK Ideograph Extension A */
  ["\u3555", "\u3555"], /* CJK Ideograph Extension A */
  ["\u3556", "\u3556"], /* CJK Ideograph Extension A */
  ["\u3557", "\u3557"], /* CJK Ideograph Extension A */
  ["\u3558", "\u3558"], /* CJK Ideograph Extension A */
  ["\u3559", "\u3559"], /* CJK Ideograph Extension A */
  ["\u355A", "\u355A"], /* CJK Ideograph Extension A */
  ["\u355B", "\u355B"], /* CJK Ideograph Extension A */
  ["\u355C", "\u355C"], /* CJK Ideograph Extension A */
  ["\u355D", "\u355D"], /* CJK Ideograph Extension A */
  ["\u355E", "\u355E"], /* CJK Ideograph Extension A */
  ["\u355F", "\u355F"], /* CJK Ideograph Extension A */
  ["\u3560", "\u3560"], /* CJK Ideograph Extension A */
  ["\u3561", "\u3561"], /* CJK Ideograph Extension A */
  ["\u3562", "\u3562"], /* CJK Ideograph Extension A */
  ["\u3563", "\u3563"], /* CJK Ideograph Extension A */
  ["\u3564", "\u3564"], /* CJK Ideograph Extension A */
  ["\u3565", "\u3565"], /* CJK Ideograph Extension A */
  ["\u3566", "\u3566"], /* CJK Ideograph Extension A */
  ["\u3567", "\u3567"], /* CJK Ideograph Extension A */
  ["\u3568", "\u3568"], /* CJK Ideograph Extension A */
  ["\u3569", "\u3569"], /* CJK Ideograph Extension A */
  ["\u356A", "\u356A"], /* CJK Ideograph Extension A */
  ["\u356B", "\u356B"], /* CJK Ideograph Extension A */
  ["\u356C", "\u356C"], /* CJK Ideograph Extension A */
  ["\u356D", "\u356D"], /* CJK Ideograph Extension A */
  ["\u356E", "\u356E"], /* CJK Ideograph Extension A */
  ["\u356F", "\u356F"], /* CJK Ideograph Extension A */
  ["\u3570", "\u3570"], /* CJK Ideograph Extension A */
  ["\u3571", "\u3571"], /* CJK Ideograph Extension A */
  ["\u3572", "\u3572"], /* CJK Ideograph Extension A */
  ["\u3573", "\u3573"], /* CJK Ideograph Extension A */
  ["\u3574", "\u3574"], /* CJK Ideograph Extension A */
  ["\u3575", "\u3575"], /* CJK Ideograph Extension A */
  ["\u3576", "\u3576"], /* CJK Ideograph Extension A */
  ["\u3577", "\u3577"], /* CJK Ideograph Extension A */
  ["\u3578", "\u3578"], /* CJK Ideograph Extension A */
  ["\u3579", "\u3579"], /* CJK Ideograph Extension A */
  ["\u357A", "\u357A"], /* CJK Ideograph Extension A */
  ["\u357B", "\u357B"], /* CJK Ideograph Extension A */
  ["\u357C", "\u357C"], /* CJK Ideograph Extension A */
  ["\u357D", "\u357D"], /* CJK Ideograph Extension A */
  ["\u357E", "\u357E"], /* CJK Ideograph Extension A */
  ["\u357F", "\u357F"], /* CJK Ideograph Extension A */
  ["\u3580", "\u3580"], /* CJK Ideograph Extension A */
  ["\u3581", "\u3581"], /* CJK Ideograph Extension A */
  ["\u3582", "\u3582"], /* CJK Ideograph Extension A */
  ["\u3583", "\u3583"], /* CJK Ideograph Extension A */
  ["\u3584", "\u3584"], /* CJK Ideograph Extension A */
  ["\u3585", "\u3585"], /* CJK Ideograph Extension A */
  ["\u3586", "\u3586"], /* CJK Ideograph Extension A */
  ["\u3587", "\u3587"], /* CJK Ideograph Extension A */
  ["\u3588", "\u3588"], /* CJK Ideograph Extension A */
  ["\u3589", "\u3589"], /* CJK Ideograph Extension A */
  ["\u358A", "\u358A"], /* CJK Ideograph Extension A */
  ["\u358B", "\u358B"], /* CJK Ideograph Extension A */
  ["\u358C", "\u358C"], /* CJK Ideograph Extension A */
  ["\u358D", "\u358D"], /* CJK Ideograph Extension A */
  ["\u358E", "\u358E"], /* CJK Ideograph Extension A */
  ["\u358F", "\u358F"], /* CJK Ideograph Extension A */
  ["\u3590", "\u3590"], /* CJK Ideograph Extension A */
  ["\u3591", "\u3591"], /* CJK Ideograph Extension A */
  ["\u3592", "\u3592"], /* CJK Ideograph Extension A */
  ["\u3593", "\u3593"], /* CJK Ideograph Extension A */
  ["\u3594", "\u3594"], /* CJK Ideograph Extension A */
  ["\u3595", "\u3595"], /* CJK Ideograph Extension A */
  ["\u3596", "\u3596"], /* CJK Ideograph Extension A */
  ["\u3597", "\u3597"], /* CJK Ideograph Extension A */
  ["\u3598", "\u3598"], /* CJK Ideograph Extension A */
  ["\u3599", "\u3599"], /* CJK Ideograph Extension A */
  ["\u359A", "\u359A"], /* CJK Ideograph Extension A */
  ["\u359B", "\u359B"], /* CJK Ideograph Extension A */
  ["\u359C", "\u359C"], /* CJK Ideograph Extension A */
  ["\u359D", "\u359D"], /* CJK Ideograph Extension A */
  ["\u359E", "\u359E"], /* CJK Ideograph Extension A */
  ["\u359F", "\u359F"], /* CJK Ideograph Extension A */
  ["\u35A0", "\u35A0"], /* CJK Ideograph Extension A */
  ["\u35A1", "\u35A1"], /* CJK Ideograph Extension A */
  ["\u35A2", "\u35A2"], /* CJK Ideograph Extension A */
  ["\u35A3", "\u35A3"], /* CJK Ideograph Extension A */
  ["\u35A4", "\u35A4"], /* CJK Ideograph Extension A */
  ["\u35A5", "\u35A5"], /* CJK Ideograph Extension A */
  ["\u35A6", "\u35A6"], /* CJK Ideograph Extension A */
  ["\u35A7", "\u35A7"], /* CJK Ideograph Extension A */
  ["\u35A8", "\u35A8"], /* CJK Ideograph Extension A */
  ["\u35A9", "\u35A9"], /* CJK Ideograph Extension A */
  ["\u35AA", "\u35AA"], /* CJK Ideograph Extension A */
  ["\u35AB", "\u35AB"], /* CJK Ideograph Extension A */
  ["\u35AC", "\u35AC"], /* CJK Ideograph Extension A */
  ["\u35AD", "\u35AD"], /* CJK Ideograph Extension A */
  ["\u35AE", "\u35AE"], /* CJK Ideograph Extension A */
  ["\u35AF", "\u35AF"], /* CJK Ideograph Extension A */
  ["\u35B0", "\u35B0"], /* CJK Ideograph Extension A */
  ["\u35B1", "\u35B1"], /* CJK Ideograph Extension A */
  ["\u35B2", "\u35B2"], /* CJK Ideograph Extension A */
  ["\u35B3", "\u35B3"], /* CJK Ideograph Extension A */
  ["\u35B4", "\u35B4"], /* CJK Ideograph Extension A */
  ["\u35B5", "\u35B5"], /* CJK Ideograph Extension A */
  ["\u35B6", "\u35B6"], /* CJK Ideograph Extension A */
  ["\u35B7", "\u35B7"], /* CJK Ideograph Extension A */
  ["\u35B8", "\u35B8"], /* CJK Ideograph Extension A */
  ["\u35B9", "\u35B9"], /* CJK Ideograph Extension A */
  ["\u35BA", "\u35BA"], /* CJK Ideograph Extension A */
  ["\u35BB", "\u35BB"], /* CJK Ideograph Extension A */
  ["\u35BC", "\u35BC"], /* CJK Ideograph Extension A */
  ["\u35BD", "\u35BD"], /* CJK Ideograph Extension A */
  ["\u35BE", "\u35BE"], /* CJK Ideograph Extension A */
  ["\u35BF", "\u35BF"], /* CJK Ideograph Extension A */
  ["\u35C0", "\u35C0"], /* CJK Ideograph Extension A */
  ["\u35C1", "\u35C1"], /* CJK Ideograph Extension A */
  ["\u35C2", "\u35C2"], /* CJK Ideograph Extension A */
  ["\u35C3", "\u35C3"], /* CJK Ideograph Extension A */
  ["\u35C4", "\u35C4"], /* CJK Ideograph Extension A */
  ["\u35C5", "\u35C5"], /* CJK Ideograph Extension A */
  ["\u35C6", "\u35C6"], /* CJK Ideograph Extension A */
  ["\u35C7", "\u35C7"], /* CJK Ideograph Extension A */
  ["\u35C8", "\u35C8"], /* CJK Ideograph Extension A */
  ["\u35C9", "\u35C9"], /* CJK Ideograph Extension A */
  ["\u35CA", "\u35CA"], /* CJK Ideograph Extension A */
  ["\u35CB", "\u35CB"], /* CJK Ideograph Extension A */
  ["\u35CC", "\u35CC"], /* CJK Ideograph Extension A */
  ["\u35CD", "\u35CD"], /* CJK Ideograph Extension A */
  ["\u35CE", "\u35CE"], /* CJK Ideograph Extension A */
  ["\u35CF", "\u35CF"], /* CJK Ideograph Extension A */
  ["\u35D0", "\u35D0"], /* CJK Ideograph Extension A */
  ["\u35D1", "\u35D1"], /* CJK Ideograph Extension A */
  ["\u35D2", "\u35D2"], /* CJK Ideograph Extension A */
  ["\u35D3", "\u35D3"], /* CJK Ideograph Extension A */
  ["\u35D4", "\u35D4"], /* CJK Ideograph Extension A */
  ["\u35D5", "\u35D5"], /* CJK Ideograph Extension A */
  ["\u35D6", "\u35D6"], /* CJK Ideograph Extension A */
  ["\u35D7", "\u35D7"], /* CJK Ideograph Extension A */
  ["\u35D8", "\u35D8"], /* CJK Ideograph Extension A */
  ["\u35D9", "\u35D9"], /* CJK Ideograph Extension A */
  ["\u35DA", "\u35DA"], /* CJK Ideograph Extension A */
  ["\u35DB", "\u35DB"], /* CJK Ideograph Extension A */
  ["\u35DC", "\u35DC"], /* CJK Ideograph Extension A */
  ["\u35DD", "\u35DD"], /* CJK Ideograph Extension A */
  ["\u35DE", "\u35DE"], /* CJK Ideograph Extension A */
  ["\u35DF", "\u35DF"], /* CJK Ideograph Extension A */
  ["\u35E0", "\u35E0"], /* CJK Ideograph Extension A */
  ["\u35E1", "\u35E1"], /* CJK Ideograph Extension A */
  ["\u35E2", "\u35E2"], /* CJK Ideograph Extension A */
  ["\u35E3", "\u35E3"], /* CJK Ideograph Extension A */
  ["\u35E4", "\u35E4"], /* CJK Ideograph Extension A */
  ["\u35E5", "\u35E5"], /* CJK Ideograph Extension A */
  ["\u35E6", "\u35E6"], /* CJK Ideograph Extension A */
  ["\u35E7", "\u35E7"], /* CJK Ideograph Extension A */
  ["\u35E8", "\u35E8"], /* CJK Ideograph Extension A */
  ["\u35E9", "\u35E9"], /* CJK Ideograph Extension A */
  ["\u35EA", "\u35EA"], /* CJK Ideograph Extension A */
  ["\u35EB", "\u35EB"], /* CJK Ideograph Extension A */
  ["\u35EC", "\u35EC"], /* CJK Ideograph Extension A */
  ["\u35ED", "\u35ED"], /* CJK Ideograph Extension A */
  ["\u35EE", "\u35EE"], /* CJK Ideograph Extension A */
  ["\u35EF", "\u35EF"], /* CJK Ideograph Extension A */
  ["\u35F0", "\u35F0"], /* CJK Ideograph Extension A */
  ["\u35F1", "\u35F1"], /* CJK Ideograph Extension A */
  ["\u35F2", "\u35F2"], /* CJK Ideograph Extension A */
  ["\u35F3", "\u35F3"], /* CJK Ideograph Extension A */
  ["\u35F4", "\u35F4"], /* CJK Ideograph Extension A */
  ["\u35F5", "\u35F5"], /* CJK Ideograph Extension A */
  ["\u35F6", "\u35F6"], /* CJK Ideograph Extension A */
  ["\u35F7", "\u35F7"], /* CJK Ideograph Extension A */
  ["\u35F8", "\u35F8"], /* CJK Ideograph Extension A */
  ["\u35F9", "\u35F9"], /* CJK Ideograph Extension A */
  ["\u35FA", "\u35FA"], /* CJK Ideograph Extension A */
  ["\u35FB", "\u35FB"], /* CJK Ideograph Extension A */
  ["\u35FC", "\u35FC"], /* CJK Ideograph Extension A */
  ["\u35FD", "\u35FD"], /* CJK Ideograph Extension A */
  ["\u35FE", "\u35FE"], /* CJK Ideograph Extension A */
  ["\u35FF", "\u35FF"], /* CJK Ideograph Extension A */
  ["\u3600", "\u3600"], /* CJK Ideograph Extension A */
  ["\u3601", "\u3601"], /* CJK Ideograph Extension A */
  ["\u3602", "\u3602"], /* CJK Ideograph Extension A */
  ["\u3603", "\u3603"], /* CJK Ideograph Extension A */
  ["\u3604", "\u3604"], /* CJK Ideograph Extension A */
  ["\u3605", "\u3605"], /* CJK Ideograph Extension A */
  ["\u3606", "\u3606"], /* CJK Ideograph Extension A */
  ["\u3607", "\u3607"], /* CJK Ideograph Extension A */
  ["\u3608", "\u3608"], /* CJK Ideograph Extension A */
  ["\u3609", "\u3609"], /* CJK Ideograph Extension A */
  ["\u360A", "\u360A"], /* CJK Ideograph Extension A */
  ["\u360B", "\u360B"], /* CJK Ideograph Extension A */
  ["\u360C", "\u360C"], /* CJK Ideograph Extension A */
  ["\u360D", "\u360D"], /* CJK Ideograph Extension A */
  ["\u360E", "\u360E"], /* CJK Ideograph Extension A */
  ["\u360F", "\u360F"], /* CJK Ideograph Extension A */
  ["\u3610", "\u3610"], /* CJK Ideograph Extension A */
  ["\u3611", "\u3611"], /* CJK Ideograph Extension A */
  ["\u3612", "\u3612"], /* CJK Ideograph Extension A */
  ["\u3613", "\u3613"], /* CJK Ideograph Extension A */
  ["\u3614", "\u3614"], /* CJK Ideograph Extension A */
  ["\u3615", "\u3615"], /* CJK Ideograph Extension A */
  ["\u3616", "\u3616"], /* CJK Ideograph Extension A */
  ["\u3617", "\u3617"], /* CJK Ideograph Extension A */
  ["\u3618", "\u3618"], /* CJK Ideograph Extension A */
  ["\u3619", "\u3619"], /* CJK Ideograph Extension A */
  ["\u361A", "\u361A"], /* CJK Ideograph Extension A */
  ["\u361B", "\u361B"], /* CJK Ideograph Extension A */
  ["\u361C", "\u361C"], /* CJK Ideograph Extension A */
  ["\u361D", "\u361D"], /* CJK Ideograph Extension A */
  ["\u361E", "\u361E"], /* CJK Ideograph Extension A */
  ["\u361F", "\u361F"], /* CJK Ideograph Extension A */
  ["\u3620", "\u3620"], /* CJK Ideograph Extension A */
  ["\u3621", "\u3621"], /* CJK Ideograph Extension A */
  ["\u3622", "\u3622"], /* CJK Ideograph Extension A */
  ["\u3623", "\u3623"], /* CJK Ideograph Extension A */
  ["\u3624", "\u3624"], /* CJK Ideograph Extension A */
  ["\u3625", "\u3625"], /* CJK Ideograph Extension A */
  ["\u3626", "\u3626"], /* CJK Ideograph Extension A */
  ["\u3627", "\u3627"], /* CJK Ideograph Extension A */
  ["\u3628", "\u3628"], /* CJK Ideograph Extension A */
  ["\u3629", "\u3629"], /* CJK Ideograph Extension A */
  ["\u362A", "\u362A"], /* CJK Ideograph Extension A */
  ["\u362B", "\u362B"], /* CJK Ideograph Extension A */
  ["\u362C", "\u362C"], /* CJK Ideograph Extension A */
  ["\u362D", "\u362D"], /* CJK Ideograph Extension A */
  ["\u362E", "\u362E"], /* CJK Ideograph Extension A */
  ["\u362F", "\u362F"], /* CJK Ideograph Extension A */
  ["\u3630", "\u3630"], /* CJK Ideograph Extension A */
  ["\u3631", "\u3631"], /* CJK Ideograph Extension A */
  ["\u3632", "\u3632"], /* CJK Ideograph Extension A */
  ["\u3633", "\u3633"], /* CJK Ideograph Extension A */
  ["\u3634", "\u3634"], /* CJK Ideograph Extension A */
  ["\u3635", "\u3635"], /* CJK Ideograph Extension A */
  ["\u3636", "\u3636"], /* CJK Ideograph Extension A */
  ["\u3637", "\u3637"], /* CJK Ideograph Extension A */
  ["\u3638", "\u3638"], /* CJK Ideograph Extension A */
  ["\u3639", "\u3639"], /* CJK Ideograph Extension A */
  ["\u363A", "\u363A"], /* CJK Ideograph Extension A */
  ["\u363B", "\u363B"], /* CJK Ideograph Extension A */
  ["\u363C", "\u363C"], /* CJK Ideograph Extension A */
  ["\u363D", "\u363D"], /* CJK Ideograph Extension A */
  ["\u363E", "\u363E"], /* CJK Ideograph Extension A */
  ["\u363F", "\u363F"], /* CJK Ideograph Extension A */
  ["\u3640", "\u3640"], /* CJK Ideograph Extension A */
  ["\u3641", "\u3641"], /* CJK Ideograph Extension A */
  ["\u3642", "\u3642"], /* CJK Ideograph Extension A */
  ["\u3643", "\u3643"], /* CJK Ideograph Extension A */
  ["\u3644", "\u3644"], /* CJK Ideograph Extension A */
  ["\u3645", "\u3645"], /* CJK Ideograph Extension A */
  ["\u3646", "\u3646"], /* CJK Ideograph Extension A */
  ["\u3647", "\u3647"], /* CJK Ideograph Extension A */
  ["\u3648", "\u3648"], /* CJK Ideograph Extension A */
  ["\u3649", "\u3649"], /* CJK Ideograph Extension A */
  ["\u364A", "\u364A"], /* CJK Ideograph Extension A */
  ["\u364B", "\u364B"], /* CJK Ideograph Extension A */
  ["\u364C", "\u364C"], /* CJK Ideograph Extension A */
  ["\u364D", "\u364D"], /* CJK Ideograph Extension A */
  ["\u364E", "\u364E"], /* CJK Ideograph Extension A */
  ["\u364F", "\u364F"], /* CJK Ideograph Extension A */
  ["\u3650", "\u3650"], /* CJK Ideograph Extension A */
  ["\u3651", "\u3651"], /* CJK Ideograph Extension A */
  ["\u3652", "\u3652"], /* CJK Ideograph Extension A */
  ["\u3653", "\u3653"], /* CJK Ideograph Extension A */
  ["\u3654", "\u3654"], /* CJK Ideograph Extension A */
  ["\u3655", "\u3655"], /* CJK Ideograph Extension A */
  ["\u3656", "\u3656"], /* CJK Ideograph Extension A */
  ["\u3657", "\u3657"], /* CJK Ideograph Extension A */
  ["\u3658", "\u3658"], /* CJK Ideograph Extension A */
  ["\u3659", "\u3659"], /* CJK Ideograph Extension A */
  ["\u365A", "\u365A"], /* CJK Ideograph Extension A */
  ["\u365B", "\u365B"], /* CJK Ideograph Extension A */
  ["\u365C", "\u365C"], /* CJK Ideograph Extension A */
  ["\u365D", "\u365D"], /* CJK Ideograph Extension A */
  ["\u365E", "\u365E"], /* CJK Ideograph Extension A */
  ["\u365F", "\u365F"], /* CJK Ideograph Extension A */
  ["\u3660", "\u3660"], /* CJK Ideograph Extension A */
  ["\u3661", "\u3661"], /* CJK Ideograph Extension A */
  ["\u3662", "\u3662"], /* CJK Ideograph Extension A */
  ["\u3663", "\u3663"], /* CJK Ideograph Extension A */
  ["\u3664", "\u3664"], /* CJK Ideograph Extension A */
  ["\u3665", "\u3665"], /* CJK Ideograph Extension A */
  ["\u3666", "\u3666"], /* CJK Ideograph Extension A */
  ["\u3667", "\u3667"], /* CJK Ideograph Extension A */
  ["\u3668", "\u3668"], /* CJK Ideograph Extension A */
  ["\u3669", "\u3669"], /* CJK Ideograph Extension A */
  ["\u366A", "\u366A"], /* CJK Ideograph Extension A */
  ["\u366B", "\u366B"], /* CJK Ideograph Extension A */
  ["\u366C", "\u366C"], /* CJK Ideograph Extension A */
  ["\u366D", "\u366D"], /* CJK Ideograph Extension A */
  ["\u366E", "\u366E"], /* CJK Ideograph Extension A */
  ["\u366F", "\u366F"], /* CJK Ideograph Extension A */
  ["\u3670", "\u3670"], /* CJK Ideograph Extension A */
  ["\u3671", "\u3671"], /* CJK Ideograph Extension A */
  ["\u3672", "\u3672"], /* CJK Ideograph Extension A */
  ["\u3673", "\u3673"], /* CJK Ideograph Extension A */
  ["\u3674", "\u3674"], /* CJK Ideograph Extension A */
  ["\u3675", "\u3675"], /* CJK Ideograph Extension A */
  ["\u3676", "\u3676"], /* CJK Ideograph Extension A */
  ["\u3677", "\u3677"], /* CJK Ideograph Extension A */
  ["\u3678", "\u3678"], /* CJK Ideograph Extension A */
  ["\u3679", "\u3679"], /* CJK Ideograph Extension A */
  ["\u367A", "\u367A"], /* CJK Ideograph Extension A */
  ["\u367B", "\u367B"], /* CJK Ideograph Extension A */
  ["\u367C", "\u367C"], /* CJK Ideograph Extension A */
  ["\u367D", "\u367D"], /* CJK Ideograph Extension A */
  ["\u367E", "\u367E"], /* CJK Ideograph Extension A */
  ["\u367F", "\u367F"], /* CJK Ideograph Extension A */
  ["\u3680", "\u3680"], /* CJK Ideograph Extension A */
  ["\u3681", "\u3681"], /* CJK Ideograph Extension A */
  ["\u3682", "\u3682"], /* CJK Ideograph Extension A */
  ["\u3683", "\u3683"], /* CJK Ideograph Extension A */
  ["\u3684", "\u3684"], /* CJK Ideograph Extension A */
  ["\u3685", "\u3685"], /* CJK Ideograph Extension A */
  ["\u3686", "\u3686"], /* CJK Ideograph Extension A */
  ["\u3687", "\u3687"], /* CJK Ideograph Extension A */
  ["\u3688", "\u3688"], /* CJK Ideograph Extension A */
  ["\u3689", "\u3689"], /* CJK Ideograph Extension A */
  ["\u368A", "\u368A"], /* CJK Ideograph Extension A */
  ["\u368B", "\u368B"], /* CJK Ideograph Extension A */
  ["\u368C", "\u368C"], /* CJK Ideograph Extension A */
  ["\u368D", "\u368D"], /* CJK Ideograph Extension A */
  ["\u368E", "\u368E"], /* CJK Ideograph Extension A */
  ["\u368F", "\u368F"], /* CJK Ideograph Extension A */
  ["\u3690", "\u3690"], /* CJK Ideograph Extension A */
  ["\u3691", "\u3691"], /* CJK Ideograph Extension A */
  ["\u3692", "\u3692"], /* CJK Ideograph Extension A */
  ["\u3693", "\u3693"], /* CJK Ideograph Extension A */
  ["\u3694", "\u3694"], /* CJK Ideograph Extension A */
  ["\u3695", "\u3695"], /* CJK Ideograph Extension A */
  ["\u3696", "\u3696"], /* CJK Ideograph Extension A */
  ["\u3697", "\u3697"], /* CJK Ideograph Extension A */
  ["\u3698", "\u3698"], /* CJK Ideograph Extension A */
  ["\u3699", "\u3699"], /* CJK Ideograph Extension A */
  ["\u369A", "\u369A"], /* CJK Ideograph Extension A */
  ["\u369B", "\u369B"], /* CJK Ideograph Extension A */
  ["\u369C", "\u369C"], /* CJK Ideograph Extension A */
  ["\u369D", "\u369D"], /* CJK Ideograph Extension A */
  ["\u369E", "\u369E"], /* CJK Ideograph Extension A */
  ["\u369F", "\u369F"], /* CJK Ideograph Extension A */
  ["\u36A0", "\u36A0"], /* CJK Ideograph Extension A */
  ["\u36A1", "\u36A1"], /* CJK Ideograph Extension A */
  ["\u36A2", "\u36A2"], /* CJK Ideograph Extension A */
  ["\u36A3", "\u36A3"], /* CJK Ideograph Extension A */
  ["\u36A4", "\u36A4"], /* CJK Ideograph Extension A */
  ["\u36A5", "\u36A5"], /* CJK Ideograph Extension A */
  ["\u36A6", "\u36A6"], /* CJK Ideograph Extension A */
  ["\u36A7", "\u36A7"], /* CJK Ideograph Extension A */
  ["\u36A8", "\u36A8"], /* CJK Ideograph Extension A */
  ["\u36A9", "\u36A9"], /* CJK Ideograph Extension A */
  ["\u36AA", "\u36AA"], /* CJK Ideograph Extension A */
  ["\u36AB", "\u36AB"], /* CJK Ideograph Extension A */
  ["\u36AC", "\u36AC"], /* CJK Ideograph Extension A */
  ["\u36AD", "\u36AD"], /* CJK Ideograph Extension A */
  ["\u36AE", "\u36AE"], /* CJK Ideograph Extension A */
  ["\u36AF", "\u36AF"], /* CJK Ideograph Extension A */
  ["\u36B0", "\u36B0"], /* CJK Ideograph Extension A */
  ["\u36B1", "\u36B1"], /* CJK Ideograph Extension A */
  ["\u36B2", "\u36B2"], /* CJK Ideograph Extension A */
  ["\u36B3", "\u36B3"], /* CJK Ideograph Extension A */
  ["\u36B4", "\u36B4"], /* CJK Ideograph Extension A */
  ["\u36B5", "\u36B5"], /* CJK Ideograph Extension A */
  ["\u36B6", "\u36B6"], /* CJK Ideograph Extension A */
  ["\u36B7", "\u36B7"], /* CJK Ideograph Extension A */
  ["\u36B8", "\u36B8"], /* CJK Ideograph Extension A */
  ["\u36B9", "\u36B9"], /* CJK Ideograph Extension A */
  ["\u36BA", "\u36BA"], /* CJK Ideograph Extension A */
  ["\u36BB", "\u36BB"], /* CJK Ideograph Extension A */
  ["\u36BC", "\u36BC"], /* CJK Ideograph Extension A */
  ["\u36BD", "\u36BD"], /* CJK Ideograph Extension A */
  ["\u36BE", "\u36BE"], /* CJK Ideograph Extension A */
  ["\u36BF", "\u36BF"], /* CJK Ideograph Extension A */
  ["\u36C0", "\u36C0"], /* CJK Ideograph Extension A */
  ["\u36C1", "\u36C1"], /* CJK Ideograph Extension A */
  ["\u36C2", "\u36C2"], /* CJK Ideograph Extension A */
  ["\u36C3", "\u36C3"], /* CJK Ideograph Extension A */
  ["\u36C4", "\u36C4"], /* CJK Ideograph Extension A */
  ["\u36C5", "\u36C5"], /* CJK Ideograph Extension A */
  ["\u36C6", "\u36C6"], /* CJK Ideograph Extension A */
  ["\u36C7", "\u36C7"], /* CJK Ideograph Extension A */
  ["\u36C8", "\u36C8"], /* CJK Ideograph Extension A */
  ["\u36C9", "\u36C9"], /* CJK Ideograph Extension A */
  ["\u36CA", "\u36CA"], /* CJK Ideograph Extension A */
  ["\u36CB", "\u36CB"], /* CJK Ideograph Extension A */
  ["\u36CC", "\u36CC"], /* CJK Ideograph Extension A */
  ["\u36CD", "\u36CD"], /* CJK Ideograph Extension A */
  ["\u36CE", "\u36CE"], /* CJK Ideograph Extension A */
  ["\u36CF", "\u36CF"], /* CJK Ideograph Extension A */
  ["\u36D0", "\u36D0"], /* CJK Ideograph Extension A */
  ["\u36D1", "\u36D1"], /* CJK Ideograph Extension A */
  ["\u36D2", "\u36D2"], /* CJK Ideograph Extension A */
  ["\u36D3", "\u36D3"], /* CJK Ideograph Extension A */
  ["\u36D4", "\u36D4"], /* CJK Ideograph Extension A */
  ["\u36D5", "\u36D5"], /* CJK Ideograph Extension A */
  ["\u36D6", "\u36D6"], /* CJK Ideograph Extension A */
  ["\u36D7", "\u36D7"], /* CJK Ideograph Extension A */
  ["\u36D8", "\u36D8"], /* CJK Ideograph Extension A */
  ["\u36D9", "\u36D9"], /* CJK Ideograph Extension A */
  ["\u36DA", "\u36DA"], /* CJK Ideograph Extension A */
  ["\u36DB", "\u36DB"], /* CJK Ideograph Extension A */
  ["\u36DC", "\u36DC"], /* CJK Ideograph Extension A */
  ["\u36DD", "\u36DD"], /* CJK Ideograph Extension A */
  ["\u36DE", "\u36DE"], /* CJK Ideograph Extension A */
  ["\u36DF", "\u36DF"], /* CJK Ideograph Extension A */
  ["\u36E0", "\u36E0"], /* CJK Ideograph Extension A */
  ["\u36E1", "\u36E1"], /* CJK Ideograph Extension A */
  ["\u36E2", "\u36E2"], /* CJK Ideograph Extension A */
  ["\u36E3", "\u36E3"], /* CJK Ideograph Extension A */
  ["\u36E4", "\u36E4"], /* CJK Ideograph Extension A */
  ["\u36E5", "\u36E5"], /* CJK Ideograph Extension A */
  ["\u36E6", "\u36E6"], /* CJK Ideograph Extension A */
  ["\u36E7", "\u36E7"], /* CJK Ideograph Extension A */
  ["\u36E8", "\u36E8"], /* CJK Ideograph Extension A */
  ["\u36E9", "\u36E9"], /* CJK Ideograph Extension A */
  ["\u36EA", "\u36EA"], /* CJK Ideograph Extension A */
  ["\u36EB", "\u36EB"], /* CJK Ideograph Extension A */
  ["\u36EC", "\u36EC"], /* CJK Ideograph Extension A */
  ["\u36ED", "\u36ED"], /* CJK Ideograph Extension A */
  ["\u36EE", "\u36EE"], /* CJK Ideograph Extension A */
  ["\u36EF", "\u36EF"], /* CJK Ideograph Extension A */
  ["\u36F0", "\u36F0"], /* CJK Ideograph Extension A */
  ["\u36F1", "\u36F1"], /* CJK Ideograph Extension A */
  ["\u36F2", "\u36F2"], /* CJK Ideograph Extension A */
  ["\u36F3", "\u36F3"], /* CJK Ideograph Extension A */
  ["\u36F4", "\u36F4"], /* CJK Ideograph Extension A */
  ["\u36F5", "\u36F5"], /* CJK Ideograph Extension A */
  ["\u36F6", "\u36F6"], /* CJK Ideograph Extension A */
  ["\u36F7", "\u36F7"], /* CJK Ideograph Extension A */
  ["\u36F8", "\u36F8"], /* CJK Ideograph Extension A */
  ["\u36F9", "\u36F9"], /* CJK Ideograph Extension A */
  ["\u36FA", "\u36FA"], /* CJK Ideograph Extension A */
  ["\u36FB", "\u36FB"], /* CJK Ideograph Extension A */
  ["\u36FC", "\u36FC"], /* CJK Ideograph Extension A */
  ["\u36FD", "\u36FD"], /* CJK Ideograph Extension A */
  ["\u36FE", "\u36FE"], /* CJK Ideograph Extension A */
  ["\u36FF", "\u36FF"], /* CJK Ideograph Extension A */
  ["\u3700", "\u3700"], /* CJK Ideograph Extension A */
  ["\u3701", "\u3701"], /* CJK Ideograph Extension A */
  ["\u3702", "\u3702"], /* CJK Ideograph Extension A */
  ["\u3703", "\u3703"], /* CJK Ideograph Extension A */
  ["\u3704", "\u3704"], /* CJK Ideograph Extension A */
  ["\u3705", "\u3705"], /* CJK Ideograph Extension A */
  ["\u3706", "\u3706"], /* CJK Ideograph Extension A */
  ["\u3707", "\u3707"], /* CJK Ideograph Extension A */
  ["\u3708", "\u3708"], /* CJK Ideograph Extension A */
  ["\u3709", "\u3709"], /* CJK Ideograph Extension A */
  ["\u370A", "\u370A"], /* CJK Ideograph Extension A */
  ["\u370B", "\u370B"], /* CJK Ideograph Extension A */
  ["\u370C", "\u370C"], /* CJK Ideograph Extension A */
  ["\u370D", "\u370D"], /* CJK Ideograph Extension A */
  ["\u370E", "\u370E"], /* CJK Ideograph Extension A */
  ["\u370F", "\u370F"], /* CJK Ideograph Extension A */
  ["\u3710", "\u3710"], /* CJK Ideograph Extension A */
  ["\u3711", "\u3711"], /* CJK Ideograph Extension A */
  ["\u3712", "\u3712"], /* CJK Ideograph Extension A */
  ["\u3713", "\u3713"], /* CJK Ideograph Extension A */
  ["\u3714", "\u3714"], /* CJK Ideograph Extension A */
  ["\u3715", "\u3715"], /* CJK Ideograph Extension A */
  ["\u3716", "\u3716"], /* CJK Ideograph Extension A */
  ["\u3717", "\u3717"], /* CJK Ideograph Extension A */
  ["\u3718", "\u3718"], /* CJK Ideograph Extension A */
  ["\u3719", "\u3719"], /* CJK Ideograph Extension A */
  ["\u371A", "\u371A"], /* CJK Ideograph Extension A */
  ["\u371B", "\u371B"], /* CJK Ideograph Extension A */
  ["\u371C", "\u371C"], /* CJK Ideograph Extension A */
  ["\u371D", "\u371D"], /* CJK Ideograph Extension A */
  ["\u371E", "\u371E"], /* CJK Ideograph Extension A */
  ["\u371F", "\u371F"], /* CJK Ideograph Extension A */
  ["\u3720", "\u3720"], /* CJK Ideograph Extension A */
  ["\u3721", "\u3721"], /* CJK Ideograph Extension A */
  ["\u3722", "\u3722"], /* CJK Ideograph Extension A */
  ["\u3723", "\u3723"], /* CJK Ideograph Extension A */
  ["\u3724", "\u3724"], /* CJK Ideograph Extension A */
  ["\u3725", "\u3725"], /* CJK Ideograph Extension A */
  ["\u3726", "\u3726"], /* CJK Ideograph Extension A */
  ["\u3727", "\u3727"], /* CJK Ideograph Extension A */
  ["\u3728", "\u3728"], /* CJK Ideograph Extension A */
  ["\u3729", "\u3729"], /* CJK Ideograph Extension A */
  ["\u372A", "\u372A"], /* CJK Ideograph Extension A */
  ["\u372B", "\u372B"], /* CJK Ideograph Extension A */
  ["\u372C", "\u372C"], /* CJK Ideograph Extension A */
  ["\u372D", "\u372D"], /* CJK Ideograph Extension A */
  ["\u372E", "\u372E"], /* CJK Ideograph Extension A */
  ["\u372F", "\u372F"], /* CJK Ideograph Extension A */
  ["\u3730", "\u3730"], /* CJK Ideograph Extension A */
  ["\u3731", "\u3731"], /* CJK Ideograph Extension A */
  ["\u3732", "\u3732"], /* CJK Ideograph Extension A */
  ["\u3733", "\u3733"], /* CJK Ideograph Extension A */
  ["\u3734", "\u3734"], /* CJK Ideograph Extension A */
  ["\u3735", "\u3735"], /* CJK Ideograph Extension A */
  ["\u3736", "\u3736"], /* CJK Ideograph Extension A */
  ["\u3737", "\u3737"], /* CJK Ideograph Extension A */
  ["\u3738", "\u3738"], /* CJK Ideograph Extension A */
  ["\u3739", "\u3739"], /* CJK Ideograph Extension A */
  ["\u373A", "\u373A"], /* CJK Ideograph Extension A */
  ["\u373B", "\u373B"], /* CJK Ideograph Extension A */
  ["\u373C", "\u373C"], /* CJK Ideograph Extension A */
  ["\u373D", "\u373D"], /* CJK Ideograph Extension A */
  ["\u373E", "\u373E"], /* CJK Ideograph Extension A */
  ["\u373F", "\u373F"], /* CJK Ideograph Extension A */
  ["\u3740", "\u3740"], /* CJK Ideograph Extension A */
  ["\u3741", "\u3741"], /* CJK Ideograph Extension A */
  ["\u3742", "\u3742"], /* CJK Ideograph Extension A */
  ["\u3743", "\u3743"], /* CJK Ideograph Extension A */
  ["\u3744", "\u3744"], /* CJK Ideograph Extension A */
  ["\u3745", "\u3745"], /* CJK Ideograph Extension A */
  ["\u3746", "\u3746"], /* CJK Ideograph Extension A */
  ["\u3747", "\u3747"], /* CJK Ideograph Extension A */
  ["\u3748", "\u3748"], /* CJK Ideograph Extension A */
  ["\u3749", "\u3749"], /* CJK Ideograph Extension A */
  ["\u374A", "\u374A"], /* CJK Ideograph Extension A */
  ["\u374B", "\u374B"], /* CJK Ideograph Extension A */
  ["\u374C", "\u374C"], /* CJK Ideograph Extension A */
  ["\u374D", "\u374D"], /* CJK Ideograph Extension A */
  ["\u374E", "\u374E"], /* CJK Ideograph Extension A */
  ["\u374F", "\u374F"], /* CJK Ideograph Extension A */
  ["\u3750", "\u3750"], /* CJK Ideograph Extension A */
  ["\u3751", "\u3751"], /* CJK Ideograph Extension A */
  ["\u3752", "\u3752"], /* CJK Ideograph Extension A */
  ["\u3753", "\u3753"], /* CJK Ideograph Extension A */
  ["\u3754", "\u3754"], /* CJK Ideograph Extension A */
  ["\u3755", "\u3755"], /* CJK Ideograph Extension A */
  ["\u3756", "\u3756"], /* CJK Ideograph Extension A */
  ["\u3757", "\u3757"], /* CJK Ideograph Extension A */
  ["\u3758", "\u3758"], /* CJK Ideograph Extension A */
  ["\u3759", "\u3759"], /* CJK Ideograph Extension A */
  ["\u375A", "\u375A"], /* CJK Ideograph Extension A */
  ["\u375B", "\u375B"], /* CJK Ideograph Extension A */
  ["\u375C", "\u375C"], /* CJK Ideograph Extension A */
  ["\u375D", "\u375D"], /* CJK Ideograph Extension A */
  ["\u375E", "\u375E"], /* CJK Ideograph Extension A */
  ["\u375F", "\u375F"], /* CJK Ideograph Extension A */
  ["\u3760", "\u3760"], /* CJK Ideograph Extension A */
  ["\u3761", "\u3761"], /* CJK Ideograph Extension A */
  ["\u3762", "\u3762"], /* CJK Ideograph Extension A */
  ["\u3763", "\u3763"], /* CJK Ideograph Extension A */
  ["\u3764", "\u3764"], /* CJK Ideograph Extension A */
  ["\u3765", "\u3765"], /* CJK Ideograph Extension A */
  ["\u3766", "\u3766"], /* CJK Ideograph Extension A */
  ["\u3767", "\u3767"], /* CJK Ideograph Extension A */
  ["\u3768", "\u3768"], /* CJK Ideograph Extension A */
  ["\u3769", "\u3769"], /* CJK Ideograph Extension A */
  ["\u376A", "\u376A"], /* CJK Ideograph Extension A */
  ["\u376B", "\u376B"], /* CJK Ideograph Extension A */
  ["\u376C", "\u376C"], /* CJK Ideograph Extension A */
  ["\u376D", "\u376D"], /* CJK Ideograph Extension A */
  ["\u376E", "\u376E"], /* CJK Ideograph Extension A */
  ["\u376F", "\u376F"], /* CJK Ideograph Extension A */
  ["\u3770", "\u3770"], /* CJK Ideograph Extension A */
  ["\u3771", "\u3771"], /* CJK Ideograph Extension A */
  ["\u3772", "\u3772"], /* CJK Ideograph Extension A */
  ["\u3773", "\u3773"], /* CJK Ideograph Extension A */
  ["\u3774", "\u3774"], /* CJK Ideograph Extension A */
  ["\u3775", "\u3775"], /* CJK Ideograph Extension A */
  ["\u3776", "\u3776"], /* CJK Ideograph Extension A */
  ["\u3777", "\u3777"], /* CJK Ideograph Extension A */
  ["\u3778", "\u3778"], /* CJK Ideograph Extension A */
  ["\u3779", "\u3779"], /* CJK Ideograph Extension A */
  ["\u377A", "\u377A"], /* CJK Ideograph Extension A */
  ["\u377B", "\u377B"], /* CJK Ideograph Extension A */
  ["\u377C", "\u377C"], /* CJK Ideograph Extension A */
  ["\u377D", "\u377D"], /* CJK Ideograph Extension A */
  ["\u377E", "\u377E"], /* CJK Ideograph Extension A */
  ["\u377F", "\u377F"], /* CJK Ideograph Extension A */
  ["\u3780", "\u3780"], /* CJK Ideograph Extension A */
  ["\u3781", "\u3781"], /* CJK Ideograph Extension A */
  ["\u3782", "\u3782"], /* CJK Ideograph Extension A */
  ["\u3783", "\u3783"], /* CJK Ideograph Extension A */
  ["\u3784", "\u3784"], /* CJK Ideograph Extension A */
  ["\u3785", "\u3785"], /* CJK Ideograph Extension A */
  ["\u3786", "\u3786"], /* CJK Ideograph Extension A */
  ["\u3787", "\u3787"], /* CJK Ideograph Extension A */
  ["\u3788", "\u3788"], /* CJK Ideograph Extension A */
  ["\u3789", "\u3789"], /* CJK Ideograph Extension A */
  ["\u378A", "\u378A"], /* CJK Ideograph Extension A */
  ["\u378B", "\u378B"], /* CJK Ideograph Extension A */
  ["\u378C", "\u378C"], /* CJK Ideograph Extension A */
  ["\u378D", "\u378D"], /* CJK Ideograph Extension A */
  ["\u378E", "\u378E"], /* CJK Ideograph Extension A */
  ["\u378F", "\u378F"], /* CJK Ideograph Extension A */
  ["\u3790", "\u3790"], /* CJK Ideograph Extension A */
  ["\u3791", "\u3791"], /* CJK Ideograph Extension A */
  ["\u3792", "\u3792"], /* CJK Ideograph Extension A */
  ["\u3793", "\u3793"], /* CJK Ideograph Extension A */
  ["\u3794", "\u3794"], /* CJK Ideograph Extension A */
  ["\u3795", "\u3795"], /* CJK Ideograph Extension A */
  ["\u3796", "\u3796"], /* CJK Ideograph Extension A */
  ["\u3797", "\u3797"], /* CJK Ideograph Extension A */
  ["\u3798", "\u3798"], /* CJK Ideograph Extension A */
  ["\u3799", "\u3799"], /* CJK Ideograph Extension A */
  ["\u379A", "\u379A"], /* CJK Ideograph Extension A */
  ["\u379B", "\u379B"], /* CJK Ideograph Extension A */
  ["\u379C", "\u379C"], /* CJK Ideograph Extension A */
  ["\u379D", "\u379D"], /* CJK Ideograph Extension A */
  ["\u379E", "\u379E"], /* CJK Ideograph Extension A */
  ["\u379F", "\u379F"], /* CJK Ideograph Extension A */
  ["\u37A0", "\u37A0"], /* CJK Ideograph Extension A */
  ["\u37A1", "\u37A1"], /* CJK Ideograph Extension A */
  ["\u37A2", "\u37A2"], /* CJK Ideograph Extension A */
  ["\u37A3", "\u37A3"], /* CJK Ideograph Extension A */
  ["\u37A4", "\u37A4"], /* CJK Ideograph Extension A */
  ["\u37A5", "\u37A5"], /* CJK Ideograph Extension A */
  ["\u37A6", "\u37A6"], /* CJK Ideograph Extension A */
  ["\u37A7", "\u37A7"], /* CJK Ideograph Extension A */
  ["\u37A8", "\u37A8"], /* CJK Ideograph Extension A */
  ["\u37A9", "\u37A9"], /* CJK Ideograph Extension A */
  ["\u37AA", "\u37AA"], /* CJK Ideograph Extension A */
  ["\u37AB", "\u37AB"], /* CJK Ideograph Extension A */
  ["\u37AC", "\u37AC"], /* CJK Ideograph Extension A */
  ["\u37AD", "\u37AD"], /* CJK Ideograph Extension A */
  ["\u37AE", "\u37AE"], /* CJK Ideograph Extension A */
  ["\u37AF", "\u37AF"], /* CJK Ideograph Extension A */
  ["\u37B0", "\u37B0"], /* CJK Ideograph Extension A */
  ["\u37B1", "\u37B1"], /* CJK Ideograph Extension A */
  ["\u37B2", "\u37B2"], /* CJK Ideograph Extension A */
  ["\u37B3", "\u37B3"], /* CJK Ideograph Extension A */
  ["\u37B4", "\u37B4"], /* CJK Ideograph Extension A */
  ["\u37B5", "\u37B5"], /* CJK Ideograph Extension A */
  ["\u37B6", "\u37B6"], /* CJK Ideograph Extension A */
  ["\u37B7", "\u37B7"], /* CJK Ideograph Extension A */
  ["\u37B8", "\u37B8"], /* CJK Ideograph Extension A */
  ["\u37B9", "\u37B9"], /* CJK Ideograph Extension A */
  ["\u37BA", "\u37BA"], /* CJK Ideograph Extension A */
  ["\u37BB", "\u37BB"], /* CJK Ideograph Extension A */
  ["\u37BC", "\u37BC"], /* CJK Ideograph Extension A */
  ["\u37BD", "\u37BD"], /* CJK Ideograph Extension A */
  ["\u37BE", "\u37BE"], /* CJK Ideograph Extension A */
  ["\u37BF", "\u37BF"], /* CJK Ideograph Extension A */
  ["\u37C0", "\u37C0"], /* CJK Ideograph Extension A */
  ["\u37C1", "\u37C1"], /* CJK Ideograph Extension A */
  ["\u37C2", "\u37C2"], /* CJK Ideograph Extension A */
  ["\u37C3", "\u37C3"], /* CJK Ideograph Extension A */
  ["\u37C4", "\u37C4"], /* CJK Ideograph Extension A */
  ["\u37C5", "\u37C5"], /* CJK Ideograph Extension A */
  ["\u37C6", "\u37C6"], /* CJK Ideograph Extension A */
  ["\u37C7", "\u37C7"], /* CJK Ideograph Extension A */
  ["\u37C8", "\u37C8"], /* CJK Ideograph Extension A */
  ["\u37C9", "\u37C9"], /* CJK Ideograph Extension A */
  ["\u37CA", "\u37CA"], /* CJK Ideograph Extension A */
  ["\u37CB", "\u37CB"], /* CJK Ideograph Extension A */
  ["\u37CC", "\u37CC"], /* CJK Ideograph Extension A */
  ["\u37CD", "\u37CD"], /* CJK Ideograph Extension A */
  ["\u37CE", "\u37CE"], /* CJK Ideograph Extension A */
  ["\u37CF", "\u37CF"], /* CJK Ideograph Extension A */
  ["\u37D0", "\u37D0"], /* CJK Ideograph Extension A */
  ["\u37D1", "\u37D1"], /* CJK Ideograph Extension A */
  ["\u37D2", "\u37D2"], /* CJK Ideograph Extension A */
  ["\u37D3", "\u37D3"], /* CJK Ideograph Extension A */
  ["\u37D4", "\u37D4"], /* CJK Ideograph Extension A */
  ["\u37D5", "\u37D5"], /* CJK Ideograph Extension A */
  ["\u37D6", "\u37D6"], /* CJK Ideograph Extension A */
  ["\u37D7", "\u37D7"], /* CJK Ideograph Extension A */
  ["\u37D8", "\u37D8"], /* CJK Ideograph Extension A */
  ["\u37D9", "\u37D9"], /* CJK Ideograph Extension A */
  ["\u37DA", "\u37DA"], /* CJK Ideograph Extension A */
  ["\u37DB", "\u37DB"], /* CJK Ideograph Extension A */
  ["\u37DC", "\u37DC"], /* CJK Ideograph Extension A */
  ["\u37DD", "\u37DD"], /* CJK Ideograph Extension A */
  ["\u37DE", "\u37DE"], /* CJK Ideograph Extension A */
  ["\u37DF", "\u37DF"], /* CJK Ideograph Extension A */
  ["\u37E0", "\u37E0"], /* CJK Ideograph Extension A */
  ["\u37E1", "\u37E1"], /* CJK Ideograph Extension A */
  ["\u37E2", "\u37E2"], /* CJK Ideograph Extension A */
  ["\u37E3", "\u37E3"], /* CJK Ideograph Extension A */
  ["\u37E4", "\u37E4"], /* CJK Ideograph Extension A */
  ["\u37E5", "\u37E5"], /* CJK Ideograph Extension A */
  ["\u37E6", "\u37E6"], /* CJK Ideograph Extension A */
  ["\u37E7", "\u37E7"], /* CJK Ideograph Extension A */
  ["\u37E8", "\u37E8"], /* CJK Ideograph Extension A */
  ["\u37E9", "\u37E9"], /* CJK Ideograph Extension A */
  ["\u37EA", "\u37EA"], /* CJK Ideograph Extension A */
  ["\u37EB", "\u37EB"], /* CJK Ideograph Extension A */
  ["\u37EC", "\u37EC"], /* CJK Ideograph Extension A */
  ["\u37ED", "\u37ED"], /* CJK Ideograph Extension A */
  ["\u37EE", "\u37EE"], /* CJK Ideograph Extension A */
  ["\u37EF", "\u37EF"], /* CJK Ideograph Extension A */
  ["\u37F0", "\u37F0"], /* CJK Ideograph Extension A */
  ["\u37F1", "\u37F1"], /* CJK Ideograph Extension A */
  ["\u37F2", "\u37F2"], /* CJK Ideograph Extension A */
  ["\u37F3", "\u37F3"], /* CJK Ideograph Extension A */
  ["\u37F4", "\u37F4"], /* CJK Ideograph Extension A */
  ["\u37F5", "\u37F5"], /* CJK Ideograph Extension A */
  ["\u37F6", "\u37F6"], /* CJK Ideograph Extension A */
  ["\u37F7", "\u37F7"], /* CJK Ideograph Extension A */
  ["\u37F8", "\u37F8"], /* CJK Ideograph Extension A */
  ["\u37F9", "\u37F9"], /* CJK Ideograph Extension A */
  ["\u37FA", "\u37FA"], /* CJK Ideograph Extension A */
  ["\u37FB", "\u37FB"], /* CJK Ideograph Extension A */
  ["\u37FC", "\u37FC"], /* CJK Ideograph Extension A */
  ["\u37FD", "\u37FD"], /* CJK Ideograph Extension A */
  ["\u37FE", "\u37FE"], /* CJK Ideograph Extension A */
  ["\u37FF", "\u37FF"], /* CJK Ideograph Extension A */
  ["\u3800", "\u3800"], /* CJK Ideograph Extension A */
  ["\u3801", "\u3801"], /* CJK Ideograph Extension A */
  ["\u3802", "\u3802"], /* CJK Ideograph Extension A */
  ["\u3803", "\u3803"], /* CJK Ideograph Extension A */
  ["\u3804", "\u3804"], /* CJK Ideograph Extension A */
  ["\u3805", "\u3805"], /* CJK Ideograph Extension A */
  ["\u3806", "\u3806"], /* CJK Ideograph Extension A */
  ["\u3807", "\u3807"], /* CJK Ideograph Extension A */
  ["\u3808", "\u3808"], /* CJK Ideograph Extension A */
  ["\u3809", "\u3809"], /* CJK Ideograph Extension A */
  ["\u380A", "\u380A"], /* CJK Ideograph Extension A */
  ["\u380B", "\u380B"], /* CJK Ideograph Extension A */
  ["\u380C", "\u380C"], /* CJK Ideograph Extension A */
  ["\u380D", "\u380D"], /* CJK Ideograph Extension A */
  ["\u380E", "\u380E"], /* CJK Ideograph Extension A */
  ["\u380F", "\u380F"], /* CJK Ideograph Extension A */
  ["\u3810", "\u3810"], /* CJK Ideograph Extension A */
  ["\u3811", "\u3811"], /* CJK Ideograph Extension A */
  ["\u3812", "\u3812"], /* CJK Ideograph Extension A */
  ["\u3813", "\u3813"], /* CJK Ideograph Extension A */
  ["\u3814", "\u3814"], /* CJK Ideograph Extension A */
  ["\u3815", "\u3815"], /* CJK Ideograph Extension A */
  ["\u3816", "\u3816"], /* CJK Ideograph Extension A */
  ["\u3817", "\u3817"], /* CJK Ideograph Extension A */
  ["\u3818", "\u3818"], /* CJK Ideograph Extension A */
  ["\u3819", "\u3819"], /* CJK Ideograph Extension A */
  ["\u381A", "\u381A"], /* CJK Ideograph Extension A */
  ["\u381B", "\u381B"], /* CJK Ideograph Extension A */
  ["\u381C", "\u381C"], /* CJK Ideograph Extension A */
  ["\u381D", "\u381D"], /* CJK Ideograph Extension A */
  ["\u381E", "\u381E"], /* CJK Ideograph Extension A */
  ["\u381F", "\u381F"], /* CJK Ideograph Extension A */
  ["\u3820", "\u3820"], /* CJK Ideograph Extension A */
  ["\u3821", "\u3821"], /* CJK Ideograph Extension A */
  ["\u3822", "\u3822"], /* CJK Ideograph Extension A */
  ["\u3823", "\u3823"], /* CJK Ideograph Extension A */
  ["\u3824", "\u3824"], /* CJK Ideograph Extension A */
  ["\u3825", "\u3825"], /* CJK Ideograph Extension A */
  ["\u3826", "\u3826"], /* CJK Ideograph Extension A */
  ["\u3827", "\u3827"], /* CJK Ideograph Extension A */
  ["\u3828", "\u3828"], /* CJK Ideograph Extension A */
  ["\u3829", "\u3829"], /* CJK Ideograph Extension A */
  ["\u382A", "\u382A"], /* CJK Ideograph Extension A */
  ["\u382B", "\u382B"], /* CJK Ideograph Extension A */
  ["\u382C", "\u382C"], /* CJK Ideograph Extension A */
  ["\u382D", "\u382D"], /* CJK Ideograph Extension A */
  ["\u382E", "\u382E"], /* CJK Ideograph Extension A */
  ["\u382F", "\u382F"], /* CJK Ideograph Extension A */
  ["\u3830", "\u3830"], /* CJK Ideograph Extension A */
  ["\u3831", "\u3831"], /* CJK Ideograph Extension A */
  ["\u3832", "\u3832"], /* CJK Ideograph Extension A */
  ["\u3833", "\u3833"], /* CJK Ideograph Extension A */
  ["\u3834", "\u3834"], /* CJK Ideograph Extension A */
  ["\u3835", "\u3835"], /* CJK Ideograph Extension A */
  ["\u3836", "\u3836"], /* CJK Ideograph Extension A */
  ["\u3837", "\u3837"], /* CJK Ideograph Extension A */
  ["\u3838", "\u3838"], /* CJK Ideograph Extension A */
  ["\u3839", "\u3839"], /* CJK Ideograph Extension A */
  ["\u383A", "\u383A"], /* CJK Ideograph Extension A */
  ["\u383B", "\u383B"], /* CJK Ideograph Extension A */
  ["\u383C", "\u383C"], /* CJK Ideograph Extension A */
  ["\u383D", "\u383D"], /* CJK Ideograph Extension A */
  ["\u383E", "\u383E"], /* CJK Ideograph Extension A */
  ["\u383F", "\u383F"], /* CJK Ideograph Extension A */
  ["\u3840", "\u3840"], /* CJK Ideograph Extension A */
  ["\u3841", "\u3841"], /* CJK Ideograph Extension A */
  ["\u3842", "\u3842"], /* CJK Ideograph Extension A */
  ["\u3843", "\u3843"], /* CJK Ideograph Extension A */
  ["\u3844", "\u3844"], /* CJK Ideograph Extension A */
  ["\u3845", "\u3845"], /* CJK Ideograph Extension A */
  ["\u3846", "\u3846"], /* CJK Ideograph Extension A */
  ["\u3847", "\u3847"], /* CJK Ideograph Extension A */
  ["\u3848", "\u3848"], /* CJK Ideograph Extension A */
  ["\u3849", "\u3849"], /* CJK Ideograph Extension A */
  ["\u384A", "\u384A"], /* CJK Ideograph Extension A */
  ["\u384B", "\u384B"], /* CJK Ideograph Extension A */
  ["\u384C", "\u384C"], /* CJK Ideograph Extension A */
  ["\u384D", "\u384D"], /* CJK Ideograph Extension A */
  ["\u384E", "\u384E"], /* CJK Ideograph Extension A */
  ["\u384F", "\u384F"], /* CJK Ideograph Extension A */
  ["\u3850", "\u3850"], /* CJK Ideograph Extension A */
  ["\u3851", "\u3851"], /* CJK Ideograph Extension A */
  ["\u3852", "\u3852"], /* CJK Ideograph Extension A */
  ["\u3853", "\u3853"], /* CJK Ideograph Extension A */
  ["\u3854", "\u3854"], /* CJK Ideograph Extension A */
  ["\u3855", "\u3855"], /* CJK Ideograph Extension A */
  ["\u3856", "\u3856"], /* CJK Ideograph Extension A */
  ["\u3857", "\u3857"], /* CJK Ideograph Extension A */
  ["\u3858", "\u3858"], /* CJK Ideograph Extension A */
  ["\u3859", "\u3859"], /* CJK Ideograph Extension A */
  ["\u385A", "\u385A"], /* CJK Ideograph Extension A */
  ["\u385B", "\u385B"], /* CJK Ideograph Extension A */
  ["\u385C", "\u385C"], /* CJK Ideograph Extension A */
  ["\u385D", "\u385D"], /* CJK Ideograph Extension A */
  ["\u385E", "\u385E"], /* CJK Ideograph Extension A */
  ["\u385F", "\u385F"], /* CJK Ideograph Extension A */
  ["\u3860", "\u3860"], /* CJK Ideograph Extension A */
  ["\u3861", "\u3861"], /* CJK Ideograph Extension A */
  ["\u3862", "\u3862"], /* CJK Ideograph Extension A */
  ["\u3863", "\u3863"], /* CJK Ideograph Extension A */
  ["\u3864", "\u3864"], /* CJK Ideograph Extension A */
  ["\u3865", "\u3865"], /* CJK Ideograph Extension A */
  ["\u3866", "\u3866"], /* CJK Ideograph Extension A */
  ["\u3867", "\u3867"], /* CJK Ideograph Extension A */
  ["\u3868", "\u3868"], /* CJK Ideograph Extension A */
  ["\u3869", "\u3869"], /* CJK Ideograph Extension A */
  ["\u386A", "\u386A"], /* CJK Ideograph Extension A */
  ["\u386B", "\u386B"], /* CJK Ideograph Extension A */
  ["\u386C", "\u386C"], /* CJK Ideograph Extension A */
  ["\u386D", "\u386D"], /* CJK Ideograph Extension A */
  ["\u386E", "\u386E"], /* CJK Ideograph Extension A */
  ["\u386F", "\u386F"], /* CJK Ideograph Extension A */
  ["\u3870", "\u3870"], /* CJK Ideograph Extension A */
  ["\u3871", "\u3871"], /* CJK Ideograph Extension A */
  ["\u3872", "\u3872"], /* CJK Ideograph Extension A */
  ["\u3873", "\u3873"], /* CJK Ideograph Extension A */
  ["\u3874", "\u3874"], /* CJK Ideograph Extension A */
  ["\u3875", "\u3875"], /* CJK Ideograph Extension A */
  ["\u3876", "\u3876"], /* CJK Ideograph Extension A */
  ["\u3877", "\u3877"], /* CJK Ideograph Extension A */
  ["\u3878", "\u3878"], /* CJK Ideograph Extension A */
  ["\u3879", "\u3879"], /* CJK Ideograph Extension A */
  ["\u387A", "\u387A"], /* CJK Ideograph Extension A */
  ["\u387B", "\u387B"], /* CJK Ideograph Extension A */
  ["\u387C", "\u387C"], /* CJK Ideograph Extension A */
  ["\u387D", "\u387D"], /* CJK Ideograph Extension A */
  ["\u387E", "\u387E"], /* CJK Ideograph Extension A */
  ["\u387F", "\u387F"], /* CJK Ideograph Extension A */
  ["\u3880", "\u3880"], /* CJK Ideograph Extension A */
  ["\u3881", "\u3881"], /* CJK Ideograph Extension A */
  ["\u3882", "\u3882"], /* CJK Ideograph Extension A */
  ["\u3883", "\u3883"], /* CJK Ideograph Extension A */
  ["\u3884", "\u3884"], /* CJK Ideograph Extension A */
  ["\u3885", "\u3885"], /* CJK Ideograph Extension A */
  ["\u3886", "\u3886"], /* CJK Ideograph Extension A */
  ["\u3887", "\u3887"], /* CJK Ideograph Extension A */
  ["\u3888", "\u3888"], /* CJK Ideograph Extension A */
  ["\u3889", "\u3889"], /* CJK Ideograph Extension A */
  ["\u388A", "\u388A"], /* CJK Ideograph Extension A */
  ["\u388B", "\u388B"], /* CJK Ideograph Extension A */
  ["\u388C", "\u388C"], /* CJK Ideograph Extension A */
  ["\u388D", "\u388D"], /* CJK Ideograph Extension A */
  ["\u388E", "\u388E"], /* CJK Ideograph Extension A */
  ["\u388F", "\u388F"], /* CJK Ideograph Extension A */
  ["\u3890", "\u3890"], /* CJK Ideograph Extension A */
  ["\u3891", "\u3891"], /* CJK Ideograph Extension A */
  ["\u3892", "\u3892"], /* CJK Ideograph Extension A */
  ["\u3893", "\u3893"], /* CJK Ideograph Extension A */
  ["\u3894", "\u3894"], /* CJK Ideograph Extension A */
  ["\u3895", "\u3895"], /* CJK Ideograph Extension A */
  ["\u3896", "\u3896"], /* CJK Ideograph Extension A */
  ["\u3897", "\u3897"], /* CJK Ideograph Extension A */
  ["\u3898", "\u3898"], /* CJK Ideograph Extension A */
  ["\u3899", "\u3899"], /* CJK Ideograph Extension A */
  ["\u389A", "\u389A"], /* CJK Ideograph Extension A */
  ["\u389B", "\u389B"], /* CJK Ideograph Extension A */
  ["\u389C", "\u389C"], /* CJK Ideograph Extension A */
  ["\u389D", "\u389D"], /* CJK Ideograph Extension A */
  ["\u389E", "\u389E"], /* CJK Ideograph Extension A */
  ["\u389F", "\u389F"], /* CJK Ideograph Extension A */
  ["\u38A0", "\u38A0"], /* CJK Ideograph Extension A */
  ["\u38A1", "\u38A1"], /* CJK Ideograph Extension A */
  ["\u38A2", "\u38A2"], /* CJK Ideograph Extension A */
  ["\u38A3", "\u38A3"], /* CJK Ideograph Extension A */
  ["\u38A4", "\u38A4"], /* CJK Ideograph Extension A */
  ["\u38A5", "\u38A5"], /* CJK Ideograph Extension A */
  ["\u38A6", "\u38A6"], /* CJK Ideograph Extension A */
  ["\u38A7", "\u38A7"], /* CJK Ideograph Extension A */
  ["\u38A8", "\u38A8"], /* CJK Ideograph Extension A */
  ["\u38A9", "\u38A9"], /* CJK Ideograph Extension A */
  ["\u38AA", "\u38AA"], /* CJK Ideograph Extension A */
  ["\u38AB", "\u38AB"], /* CJK Ideograph Extension A */
  ["\u38AC", "\u38AC"], /* CJK Ideograph Extension A */
  ["\u38AD", "\u38AD"], /* CJK Ideograph Extension A */
  ["\u38AE", "\u38AE"], /* CJK Ideograph Extension A */
  ["\u38AF", "\u38AF"], /* CJK Ideograph Extension A */
  ["\u38B0", "\u38B0"], /* CJK Ideograph Extension A */
  ["\u38B1", "\u38B1"], /* CJK Ideograph Extension A */
  ["\u38B2", "\u38B2"], /* CJK Ideograph Extension A */
  ["\u38B3", "\u38B3"], /* CJK Ideograph Extension A */
  ["\u38B4", "\u38B4"], /* CJK Ideograph Extension A */
  ["\u38B5", "\u38B5"], /* CJK Ideograph Extension A */
  ["\u38B6", "\u38B6"], /* CJK Ideograph Extension A */
  ["\u38B7", "\u38B7"], /* CJK Ideograph Extension A */
  ["\u38B8", "\u38B8"], /* CJK Ideograph Extension A */
  ["\u38B9", "\u38B9"], /* CJK Ideograph Extension A */
  ["\u38BA", "\u38BA"], /* CJK Ideograph Extension A */
  ["\u38BB", "\u38BB"], /* CJK Ideograph Extension A */
  ["\u38BC", "\u38BC"], /* CJK Ideograph Extension A */
  ["\u38BD", "\u38BD"], /* CJK Ideograph Extension A */
  ["\u38BE", "\u38BE"], /* CJK Ideograph Extension A */
  ["\u38BF", "\u38BF"], /* CJK Ideograph Extension A */
  ["\u38C0", "\u38C0"], /* CJK Ideograph Extension A */
  ["\u38C1", "\u38C1"], /* CJK Ideograph Extension A */
  ["\u38C2", "\u38C2"], /* CJK Ideograph Extension A */
  ["\u38C3", "\u38C3"], /* CJK Ideograph Extension A */
  ["\u38C4", "\u38C4"], /* CJK Ideograph Extension A */
  ["\u38C5", "\u38C5"], /* CJK Ideograph Extension A */
  ["\u38C6", "\u38C6"], /* CJK Ideograph Extension A */
  ["\u38C7", "\u38C7"], /* CJK Ideograph Extension A */
  ["\u38C8", "\u38C8"], /* CJK Ideograph Extension A */
  ["\u38C9", "\u38C9"], /* CJK Ideograph Extension A */
  ["\u38CA", "\u38CA"], /* CJK Ideograph Extension A */
  ["\u38CB", "\u38CB"], /* CJK Ideograph Extension A */
  ["\u38CC", "\u38CC"], /* CJK Ideograph Extension A */
  ["\u38CD", "\u38CD"], /* CJK Ideograph Extension A */
  ["\u38CE", "\u38CE"], /* CJK Ideograph Extension A */
  ["\u38CF", "\u38CF"], /* CJK Ideograph Extension A */
  ["\u38D0", "\u38D0"], /* CJK Ideograph Extension A */
  ["\u38D1", "\u38D1"], /* CJK Ideograph Extension A */
  ["\u38D2", "\u38D2"], /* CJK Ideograph Extension A */
  ["\u38D3", "\u38D3"], /* CJK Ideograph Extension A */
  ["\u38D4", "\u38D4"], /* CJK Ideograph Extension A */
  ["\u38D5", "\u38D5"], /* CJK Ideograph Extension A */
  ["\u38D6", "\u38D6"], /* CJK Ideograph Extension A */
  ["\u38D7", "\u38D7"], /* CJK Ideograph Extension A */
  ["\u38D8", "\u38D8"], /* CJK Ideograph Extension A */
  ["\u38D9", "\u38D9"], /* CJK Ideograph Extension A */
  ["\u38DA", "\u38DA"], /* CJK Ideograph Extension A */
  ["\u38DB", "\u38DB"], /* CJK Ideograph Extension A */
  ["\u38DC", "\u38DC"], /* CJK Ideograph Extension A */
  ["\u38DD", "\u38DD"], /* CJK Ideograph Extension A */
  ["\u38DE", "\u38DE"], /* CJK Ideograph Extension A */
  ["\u38DF", "\u38DF"], /* CJK Ideograph Extension A */
  ["\u38E0", "\u38E0"], /* CJK Ideograph Extension A */
  ["\u38E1", "\u38E1"], /* CJK Ideograph Extension A */
  ["\u38E2", "\u38E2"], /* CJK Ideograph Extension A */
  ["\u38E3", "\u38E3"], /* CJK Ideograph Extension A */
  ["\u38E4", "\u38E4"], /* CJK Ideograph Extension A */
  ["\u38E5", "\u38E5"], /* CJK Ideograph Extension A */
  ["\u38E6", "\u38E6"], /* CJK Ideograph Extension A */
  ["\u38E7", "\u38E7"], /* CJK Ideograph Extension A */
  ["\u38E8", "\u38E8"], /* CJK Ideograph Extension A */
  ["\u38E9", "\u38E9"], /* CJK Ideograph Extension A */
  ["\u38EA", "\u38EA"], /* CJK Ideograph Extension A */
  ["\u38EB", "\u38EB"], /* CJK Ideograph Extension A */
  ["\u38EC", "\u38EC"], /* CJK Ideograph Extension A */
  ["\u38ED", "\u38ED"], /* CJK Ideograph Extension A */
  ["\u38EE", "\u38EE"], /* CJK Ideograph Extension A */
  ["\u38EF", "\u38EF"], /* CJK Ideograph Extension A */
  ["\u38F0", "\u38F0"], /* CJK Ideograph Extension A */
  ["\u38F1", "\u38F1"], /* CJK Ideograph Extension A */
  ["\u38F2", "\u38F2"], /* CJK Ideograph Extension A */
  ["\u38F3", "\u38F3"], /* CJK Ideograph Extension A */
  ["\u38F4", "\u38F4"], /* CJK Ideograph Extension A */
  ["\u38F5", "\u38F5"], /* CJK Ideograph Extension A */
  ["\u38F6", "\u38F6"], /* CJK Ideograph Extension A */
  ["\u38F7", "\u38F7"], /* CJK Ideograph Extension A */
  ["\u38F8", "\u38F8"], /* CJK Ideograph Extension A */
  ["\u38F9", "\u38F9"], /* CJK Ideograph Extension A */
  ["\u38FA", "\u38FA"], /* CJK Ideograph Extension A */
  ["\u38FB", "\u38FB"], /* CJK Ideograph Extension A */
  ["\u38FC", "\u38FC"], /* CJK Ideograph Extension A */
  ["\u38FD", "\u38FD"], /* CJK Ideograph Extension A */
  ["\u38FE", "\u38FE"], /* CJK Ideograph Extension A */
  ["\u38FF", "\u38FF"], /* CJK Ideograph Extension A */
  ["\u3900", "\u3900"], /* CJK Ideograph Extension A */
  ["\u3901", "\u3901"], /* CJK Ideograph Extension A */
  ["\u3902", "\u3902"], /* CJK Ideograph Extension A */
  ["\u3903", "\u3903"], /* CJK Ideograph Extension A */
  ["\u3904", "\u3904"], /* CJK Ideograph Extension A */
  ["\u3905", "\u3905"], /* CJK Ideograph Extension A */
  ["\u3906", "\u3906"], /* CJK Ideograph Extension A */
  ["\u3907", "\u3907"], /* CJK Ideograph Extension A */
  ["\u3908", "\u3908"], /* CJK Ideograph Extension A */
  ["\u3909", "\u3909"], /* CJK Ideograph Extension A */
  ["\u390A", "\u390A"], /* CJK Ideograph Extension A */
  ["\u390B", "\u390B"], /* CJK Ideograph Extension A */
  ["\u390C", "\u390C"], /* CJK Ideograph Extension A */
  ["\u390D", "\u390D"], /* CJK Ideograph Extension A */
  ["\u390E", "\u390E"], /* CJK Ideograph Extension A */
  ["\u390F", "\u390F"], /* CJK Ideograph Extension A */
  ["\u3910", "\u3910"], /* CJK Ideograph Extension A */
  ["\u3911", "\u3911"], /* CJK Ideograph Extension A */
  ["\u3912", "\u3912"], /* CJK Ideograph Extension A */
  ["\u3913", "\u3913"], /* CJK Ideograph Extension A */
  ["\u3914", "\u3914"], /* CJK Ideograph Extension A */
  ["\u3915", "\u3915"], /* CJK Ideograph Extension A */
  ["\u3916", "\u3916"], /* CJK Ideograph Extension A */
  ["\u3917", "\u3917"], /* CJK Ideograph Extension A */
  ["\u3918", "\u3918"], /* CJK Ideograph Extension A */
  ["\u3919", "\u3919"], /* CJK Ideograph Extension A */
  ["\u391A", "\u391A"], /* CJK Ideograph Extension A */
  ["\u391B", "\u391B"], /* CJK Ideograph Extension A */
  ["\u391C", "\u391C"], /* CJK Ideograph Extension A */
  ["\u391D", "\u391D"], /* CJK Ideograph Extension A */
  ["\u391E", "\u391E"], /* CJK Ideograph Extension A */
  ["\u391F", "\u391F"], /* CJK Ideograph Extension A */
  ["\u3920", "\u3920"], /* CJK Ideograph Extension A */
  ["\u3921", "\u3921"], /* CJK Ideograph Extension A */
  ["\u3922", "\u3922"], /* CJK Ideograph Extension A */
  ["\u3923", "\u3923"], /* CJK Ideograph Extension A */
  ["\u3924", "\u3924"], /* CJK Ideograph Extension A */
  ["\u3925", "\u3925"], /* CJK Ideograph Extension A */
  ["\u3926", "\u3926"], /* CJK Ideograph Extension A */
  ["\u3927", "\u3927"], /* CJK Ideograph Extension A */
  ["\u3928", "\u3928"], /* CJK Ideograph Extension A */
  ["\u3929", "\u3929"], /* CJK Ideograph Extension A */
  ["\u392A", "\u392A"], /* CJK Ideograph Extension A */
  ["\u392B", "\u392B"], /* CJK Ideograph Extension A */
  ["\u392C", "\u392C"], /* CJK Ideograph Extension A */
  ["\u392D", "\u392D"], /* CJK Ideograph Extension A */
  ["\u392E", "\u392E"], /* CJK Ideograph Extension A */
  ["\u392F", "\u392F"], /* CJK Ideograph Extension A */
  ["\u3930", "\u3930"], /* CJK Ideograph Extension A */
  ["\u3931", "\u3931"], /* CJK Ideograph Extension A */
  ["\u3932", "\u3932"], /* CJK Ideograph Extension A */
  ["\u3933", "\u3933"], /* CJK Ideograph Extension A */
  ["\u3934", "\u3934"], /* CJK Ideograph Extension A */
  ["\u3935", "\u3935"], /* CJK Ideograph Extension A */
  ["\u3936", "\u3936"], /* CJK Ideograph Extension A */
  ["\u3937", "\u3937"], /* CJK Ideograph Extension A */
  ["\u3938", "\u3938"], /* CJK Ideograph Extension A */
  ["\u3939", "\u3939"], /* CJK Ideograph Extension A */
  ["\u393A", "\u393A"], /* CJK Ideograph Extension A */
  ["\u393B", "\u393B"], /* CJK Ideograph Extension A */
  ["\u393C", "\u393C"], /* CJK Ideograph Extension A */
  ["\u393D", "\u393D"], /* CJK Ideograph Extension A */
  ["\u393E", "\u393E"], /* CJK Ideograph Extension A */
  ["\u393F", "\u393F"], /* CJK Ideograph Extension A */
  ["\u3940", "\u3940"], /* CJK Ideograph Extension A */
  ["\u3941", "\u3941"], /* CJK Ideograph Extension A */
  ["\u3942", "\u3942"], /* CJK Ideograph Extension A */
  ["\u3943", "\u3943"], /* CJK Ideograph Extension A */
  ["\u3944", "\u3944"], /* CJK Ideograph Extension A */
  ["\u3945", "\u3945"], /* CJK Ideograph Extension A */
  ["\u3946", "\u3946"], /* CJK Ideograph Extension A */
  ["\u3947", "\u3947"], /* CJK Ideograph Extension A */
  ["\u3948", "\u3948"], /* CJK Ideograph Extension A */
  ["\u3949", "\u3949"], /* CJK Ideograph Extension A */
  ["\u394A", "\u394A"], /* CJK Ideograph Extension A */
  ["\u394B", "\u394B"], /* CJK Ideograph Extension A */
  ["\u394C", "\u394C"], /* CJK Ideograph Extension A */
  ["\u394D", "\u394D"], /* CJK Ideograph Extension A */
  ["\u394E", "\u394E"], /* CJK Ideograph Extension A */
  ["\u394F", "\u394F"], /* CJK Ideograph Extension A */
  ["\u3950", "\u3950"], /* CJK Ideograph Extension A */
  ["\u3951", "\u3951"], /* CJK Ideograph Extension A */
  ["\u3952", "\u3952"], /* CJK Ideograph Extension A */
  ["\u3953", "\u3953"], /* CJK Ideograph Extension A */
  ["\u3954", "\u3954"], /* CJK Ideograph Extension A */
  ["\u3955", "\u3955"], /* CJK Ideograph Extension A */
  ["\u3956", "\u3956"], /* CJK Ideograph Extension A */
  ["\u3957", "\u3957"], /* CJK Ideograph Extension A */
  ["\u3958", "\u3958"], /* CJK Ideograph Extension A */
  ["\u3959", "\u3959"], /* CJK Ideograph Extension A */
  ["\u395A", "\u395A"], /* CJK Ideograph Extension A */
  ["\u395B", "\u395B"], /* CJK Ideograph Extension A */
  ["\u395C", "\u395C"], /* CJK Ideograph Extension A */
  ["\u395D", "\u395D"], /* CJK Ideograph Extension A */
  ["\u395E", "\u395E"], /* CJK Ideograph Extension A */
  ["\u395F", "\u395F"], /* CJK Ideograph Extension A */
  ["\u3960", "\u3960"], /* CJK Ideograph Extension A */
  ["\u3961", "\u3961"], /* CJK Ideograph Extension A */
  ["\u3962", "\u3962"], /* CJK Ideograph Extension A */
  ["\u3963", "\u3963"], /* CJK Ideograph Extension A */
  ["\u3964", "\u3964"], /* CJK Ideograph Extension A */
  ["\u3965", "\u3965"], /* CJK Ideograph Extension A */
  ["\u3966", "\u3966"], /* CJK Ideograph Extension A */
  ["\u3967", "\u3967"], /* CJK Ideograph Extension A */
  ["\u3968", "\u3968"], /* CJK Ideograph Extension A */
  ["\u3969", "\u3969"], /* CJK Ideograph Extension A */
  ["\u396A", "\u396A"], /* CJK Ideograph Extension A */
  ["\u396B", "\u396B"], /* CJK Ideograph Extension A */
  ["\u396C", "\u396C"], /* CJK Ideograph Extension A */
  ["\u396D", "\u396D"], /* CJK Ideograph Extension A */
  ["\u396E", "\u396E"], /* CJK Ideograph Extension A */
  ["\u396F", "\u396F"], /* CJK Ideograph Extension A */
  ["\u3970", "\u3970"], /* CJK Ideograph Extension A */
  ["\u3971", "\u3971"], /* CJK Ideograph Extension A */
  ["\u3972", "\u3972"], /* CJK Ideograph Extension A */
  ["\u3973", "\u3973"], /* CJK Ideograph Extension A */
  ["\u3974", "\u3974"], /* CJK Ideograph Extension A */
  ["\u3975", "\u3975"], /* CJK Ideograph Extension A */
  ["\u3976", "\u3976"], /* CJK Ideograph Extension A */
  ["\u3977", "\u3977"], /* CJK Ideograph Extension A */
  ["\u3978", "\u3978"], /* CJK Ideograph Extension A */
  ["\u3979", "\u3979"], /* CJK Ideograph Extension A */
  ["\u397A", "\u397A"], /* CJK Ideograph Extension A */
  ["\u397B", "\u397B"], /* CJK Ideograph Extension A */
  ["\u397C", "\u397C"], /* CJK Ideograph Extension A */
  ["\u397D", "\u397D"], /* CJK Ideograph Extension A */
  ["\u397E", "\u397E"], /* CJK Ideograph Extension A */
  ["\u397F", "\u397F"], /* CJK Ideograph Extension A */
  ["\u3980", "\u3980"], /* CJK Ideograph Extension A */
  ["\u3981", "\u3981"], /* CJK Ideograph Extension A */
  ["\u3982", "\u3982"], /* CJK Ideograph Extension A */
  ["\u3983", "\u3983"], /* CJK Ideograph Extension A */
  ["\u3984", "\u3984"], /* CJK Ideograph Extension A */
  ["\u3985", "\u3985"], /* CJK Ideograph Extension A */
  ["\u3986", "\u3986"], /* CJK Ideograph Extension A */
  ["\u3987", "\u3987"], /* CJK Ideograph Extension A */
  ["\u3988", "\u3988"], /* CJK Ideograph Extension A */
  ["\u3989", "\u3989"], /* CJK Ideograph Extension A */
  ["\u398A", "\u398A"], /* CJK Ideograph Extension A */
  ["\u398B", "\u398B"], /* CJK Ideograph Extension A */
  ["\u398C", "\u398C"], /* CJK Ideograph Extension A */
  ["\u398D", "\u398D"], /* CJK Ideograph Extension A */
  ["\u398E", "\u398E"], /* CJK Ideograph Extension A */
  ["\u398F", "\u398F"], /* CJK Ideograph Extension A */
  ["\u3990", "\u3990"], /* CJK Ideograph Extension A */
  ["\u3991", "\u3991"], /* CJK Ideograph Extension A */
  ["\u3992", "\u3992"], /* CJK Ideograph Extension A */
  ["\u3993", "\u3993"], /* CJK Ideograph Extension A */
  ["\u3994", "\u3994"], /* CJK Ideograph Extension A */
  ["\u3995", "\u3995"], /* CJK Ideograph Extension A */
  ["\u3996", "\u3996"], /* CJK Ideograph Extension A */
  ["\u3997", "\u3997"], /* CJK Ideograph Extension A */
  ["\u3998", "\u3998"], /* CJK Ideograph Extension A */
  ["\u3999", "\u3999"], /* CJK Ideograph Extension A */
  ["\u399A", "\u399A"], /* CJK Ideograph Extension A */
  ["\u399B", "\u399B"], /* CJK Ideograph Extension A */
  ["\u399C", "\u399C"], /* CJK Ideograph Extension A */
  ["\u399D", "\u399D"], /* CJK Ideograph Extension A */
  ["\u399E", "\u399E"], /* CJK Ideograph Extension A */
  ["\u399F", "\u399F"], /* CJK Ideograph Extension A */
  ["\u39A0", "\u39A0"], /* CJK Ideograph Extension A */
  ["\u39A1", "\u39A1"], /* CJK Ideograph Extension A */
  ["\u39A2", "\u39A2"], /* CJK Ideograph Extension A */
  ["\u39A3", "\u39A3"], /* CJK Ideograph Extension A */
  ["\u39A4", "\u39A4"], /* CJK Ideograph Extension A */
  ["\u39A5", "\u39A5"], /* CJK Ideograph Extension A */
  ["\u39A6", "\u39A6"], /* CJK Ideograph Extension A */
  ["\u39A7", "\u39A7"], /* CJK Ideograph Extension A */
  ["\u39A8", "\u39A8"], /* CJK Ideograph Extension A */
  ["\u39A9", "\u39A9"], /* CJK Ideograph Extension A */
  ["\u39AA", "\u39AA"], /* CJK Ideograph Extension A */
  ["\u39AB", "\u39AB"], /* CJK Ideograph Extension A */
  ["\u39AC", "\u39AC"], /* CJK Ideograph Extension A */
  ["\u39AD", "\u39AD"], /* CJK Ideograph Extension A */
  ["\u39AE", "\u39AE"], /* CJK Ideograph Extension A */
  ["\u39AF", "\u39AF"], /* CJK Ideograph Extension A */
  ["\u39B0", "\u39B0"], /* CJK Ideograph Extension A */
  ["\u39B1", "\u39B1"], /* CJK Ideograph Extension A */
  ["\u39B2", "\u39B2"], /* CJK Ideograph Extension A */
  ["\u39B3", "\u39B3"], /* CJK Ideograph Extension A */
  ["\u39B4", "\u39B4"], /* CJK Ideograph Extension A */
  ["\u39B5", "\u39B5"], /* CJK Ideograph Extension A */
  ["\u39B6", "\u39B6"], /* CJK Ideograph Extension A */
  ["\u39B7", "\u39B7"], /* CJK Ideograph Extension A */
  ["\u39B8", "\u39B8"], /* CJK Ideograph Extension A */
  ["\u39B9", "\u39B9"], /* CJK Ideograph Extension A */
  ["\u39BA", "\u39BA"], /* CJK Ideograph Extension A */
  ["\u39BB", "\u39BB"], /* CJK Ideograph Extension A */
  ["\u39BC", "\u39BC"], /* CJK Ideograph Extension A */
  ["\u39BD", "\u39BD"], /* CJK Ideograph Extension A */
  ["\u39BE", "\u39BE"], /* CJK Ideograph Extension A */
  ["\u39BF", "\u39BF"], /* CJK Ideograph Extension A */
  ["\u39C0", "\u39C0"], /* CJK Ideograph Extension A */
  ["\u39C1", "\u39C1"], /* CJK Ideograph Extension A */
  ["\u39C2", "\u39C2"], /* CJK Ideograph Extension A */
  ["\u39C3", "\u39C3"], /* CJK Ideograph Extension A */
  ["\u39C4", "\u39C4"], /* CJK Ideograph Extension A */
  ["\u39C5", "\u39C5"], /* CJK Ideograph Extension A */
  ["\u39C6", "\u39C6"], /* CJK Ideograph Extension A */
  ["\u39C7", "\u39C7"], /* CJK Ideograph Extension A */
  ["\u39C8", "\u39C8"], /* CJK Ideograph Extension A */
  ["\u39C9", "\u39C9"], /* CJK Ideograph Extension A */
  ["\u39CA", "\u39CA"], /* CJK Ideograph Extension A */
  ["\u39CB", "\u39CB"], /* CJK Ideograph Extension A */
  ["\u39CC", "\u39CC"], /* CJK Ideograph Extension A */
  ["\u39CD", "\u39CD"], /* CJK Ideograph Extension A */
  ["\u39CE", "\u39CE"], /* CJK Ideograph Extension A */
  ["\u39CF", "\u39CF"], /* CJK Ideograph Extension A */
  ["\u39D0", "\u39D0"], /* CJK Ideograph Extension A */
  ["\u39D1", "\u39D1"], /* CJK Ideograph Extension A */
  ["\u39D2", "\u39D2"], /* CJK Ideograph Extension A */
  ["\u39D3", "\u39D3"], /* CJK Ideograph Extension A */
  ["\u39D4", "\u39D4"], /* CJK Ideograph Extension A */
  ["\u39D5", "\u39D5"], /* CJK Ideograph Extension A */
  ["\u39D6", "\u39D6"], /* CJK Ideograph Extension A */
  ["\u39D7", "\u39D7"], /* CJK Ideograph Extension A */
  ["\u39D8", "\u39D8"], /* CJK Ideograph Extension A */
  ["\u39D9", "\u39D9"], /* CJK Ideograph Extension A */
  ["\u39DA", "\u39DA"], /* CJK Ideograph Extension A */
  ["\u39DB", "\u39DB"], /* CJK Ideograph Extension A */
  ["\u39DC", "\u39DC"], /* CJK Ideograph Extension A */
  ["\u39DD", "\u39DD"], /* CJK Ideograph Extension A */
  ["\u39DE", "\u39DE"], /* CJK Ideograph Extension A */
  ["\u39DF", "\u39DF"], /* CJK Ideograph Extension A */
  ["\u39E0", "\u39E0"], /* CJK Ideograph Extension A */
  ["\u39E1", "\u39E1"], /* CJK Ideograph Extension A */
  ["\u39E2", "\u39E2"], /* CJK Ideograph Extension A */
  ["\u39E3", "\u39E3"], /* CJK Ideograph Extension A */
  ["\u39E4", "\u39E4"], /* CJK Ideograph Extension A */
  ["\u39E5", "\u39E5"], /* CJK Ideograph Extension A */
  ["\u39E6", "\u39E6"], /* CJK Ideograph Extension A */
  ["\u39E7", "\u39E7"], /* CJK Ideograph Extension A */
  ["\u39E8", "\u39E8"], /* CJK Ideograph Extension A */
  ["\u39E9", "\u39E9"], /* CJK Ideograph Extension A */
  ["\u39EA", "\u39EA"], /* CJK Ideograph Extension A */
  ["\u39EB", "\u39EB"], /* CJK Ideograph Extension A */
  ["\u39EC", "\u39EC"], /* CJK Ideograph Extension A */
  ["\u39ED", "\u39ED"], /* CJK Ideograph Extension A */
  ["\u39EE", "\u39EE"], /* CJK Ideograph Extension A */
  ["\u39EF", "\u39EF"], /* CJK Ideograph Extension A */
  ["\u39F0", "\u39F0"], /* CJK Ideograph Extension A */
  ["\u39F1", "\u39F1"], /* CJK Ideograph Extension A */
  ["\u39F2", "\u39F2"], /* CJK Ideograph Extension A */
  ["\u39F3", "\u39F3"], /* CJK Ideograph Extension A */
  ["\u39F4", "\u39F4"], /* CJK Ideograph Extension A */
  ["\u39F5", "\u39F5"], /* CJK Ideograph Extension A */
  ["\u39F6", "\u39F6"], /* CJK Ideograph Extension A */
  ["\u39F7", "\u39F7"], /* CJK Ideograph Extension A */
  ["\u39F8", "\u39F8"], /* CJK Ideograph Extension A */
  ["\u39F9", "\u39F9"], /* CJK Ideograph Extension A */
  ["\u39FA", "\u39FA"], /* CJK Ideograph Extension A */
  ["\u39FB", "\u39FB"], /* CJK Ideograph Extension A */
  ["\u39FC", "\u39FC"], /* CJK Ideograph Extension A */
  ["\u39FD", "\u39FD"], /* CJK Ideograph Extension A */
  ["\u39FE", "\u39FE"], /* CJK Ideograph Extension A */
  ["\u39FF", "\u39FF"], /* CJK Ideograph Extension A */
  ["\u3A00", "\u3A00"], /* CJK Ideograph Extension A */
  ["\u3A01", "\u3A01"], /* CJK Ideograph Extension A */
  ["\u3A02", "\u3A02"], /* CJK Ideograph Extension A */
  ["\u3A03", "\u3A03"], /* CJK Ideograph Extension A */
  ["\u3A04", "\u3A04"], /* CJK Ideograph Extension A */
  ["\u3A05", "\u3A05"], /* CJK Ideograph Extension A */
  ["\u3A06", "\u3A06"], /* CJK Ideograph Extension A */
  ["\u3A07", "\u3A07"], /* CJK Ideograph Extension A */
  ["\u3A08", "\u3A08"], /* CJK Ideograph Extension A */
  ["\u3A09", "\u3A09"], /* CJK Ideograph Extension A */
  ["\u3A0A", "\u3A0A"], /* CJK Ideograph Extension A */
  ["\u3A0B", "\u3A0B"], /* CJK Ideograph Extension A */
  ["\u3A0C", "\u3A0C"], /* CJK Ideograph Extension A */
  ["\u3A0D", "\u3A0D"], /* CJK Ideograph Extension A */
  ["\u3A0E", "\u3A0E"], /* CJK Ideograph Extension A */
  ["\u3A0F", "\u3A0F"], /* CJK Ideograph Extension A */
  ["\u3A10", "\u3A10"], /* CJK Ideograph Extension A */
  ["\u3A11", "\u3A11"], /* CJK Ideograph Extension A */
  ["\u3A12", "\u3A12"], /* CJK Ideograph Extension A */
  ["\u3A13", "\u3A13"], /* CJK Ideograph Extension A */
  ["\u3A14", "\u3A14"], /* CJK Ideograph Extension A */
  ["\u3A15", "\u3A15"], /* CJK Ideograph Extension A */
  ["\u3A16", "\u3A16"], /* CJK Ideograph Extension A */
  ["\u3A17", "\u3A17"], /* CJK Ideograph Extension A */
  ["\u3A18", "\u3A18"], /* CJK Ideograph Extension A */
  ["\u3A19", "\u3A19"], /* CJK Ideograph Extension A */
  ["\u3A1A", "\u3A1A"], /* CJK Ideograph Extension A */
  ["\u3A1B", "\u3A1B"], /* CJK Ideograph Extension A */
  ["\u3A1C", "\u3A1C"], /* CJK Ideograph Extension A */
  ["\u3A1D", "\u3A1D"], /* CJK Ideograph Extension A */
  ["\u3A1E", "\u3A1E"], /* CJK Ideograph Extension A */
  ["\u3A1F", "\u3A1F"], /* CJK Ideograph Extension A */
  ["\u3A20", "\u3A20"], /* CJK Ideograph Extension A */
  ["\u3A21", "\u3A21"], /* CJK Ideograph Extension A */
  ["\u3A22", "\u3A22"], /* CJK Ideograph Extension A */
  ["\u3A23", "\u3A23"], /* CJK Ideograph Extension A */
  ["\u3A24", "\u3A24"], /* CJK Ideograph Extension A */
  ["\u3A25", "\u3A25"], /* CJK Ideograph Extension A */
  ["\u3A26", "\u3A26"], /* CJK Ideograph Extension A */
  ["\u3A27", "\u3A27"], /* CJK Ideograph Extension A */
  ["\u3A28", "\u3A28"], /* CJK Ideograph Extension A */
  ["\u3A29", "\u3A29"], /* CJK Ideograph Extension A */
  ["\u3A2A", "\u3A2A"], /* CJK Ideograph Extension A */
  ["\u3A2B", "\u3A2B"], /* CJK Ideograph Extension A */
  ["\u3A2C", "\u3A2C"], /* CJK Ideograph Extension A */
  ["\u3A2D", "\u3A2D"], /* CJK Ideograph Extension A */
  ["\u3A2E", "\u3A2E"], /* CJK Ideograph Extension A */
  ["\u3A2F", "\u3A2F"], /* CJK Ideograph Extension A */
  ["\u3A30", "\u3A30"], /* CJK Ideograph Extension A */
  ["\u3A31", "\u3A31"], /* CJK Ideograph Extension A */
  ["\u3A32", "\u3A32"], /* CJK Ideograph Extension A */
  ["\u3A33", "\u3A33"], /* CJK Ideograph Extension A */
  ["\u3A34", "\u3A34"], /* CJK Ideograph Extension A */
  ["\u3A35", "\u3A35"], /* CJK Ideograph Extension A */
  ["\u3A36", "\u3A36"], /* CJK Ideograph Extension A */
  ["\u3A37", "\u3A37"], /* CJK Ideograph Extension A */
  ["\u3A38", "\u3A38"], /* CJK Ideograph Extension A */
  ["\u3A39", "\u3A39"], /* CJK Ideograph Extension A */
  ["\u3A3A", "\u3A3A"], /* CJK Ideograph Extension A */
  ["\u3A3B", "\u3A3B"], /* CJK Ideograph Extension A */
  ["\u3A3C", "\u3A3C"], /* CJK Ideograph Extension A */
  ["\u3A3D", "\u3A3D"], /* CJK Ideograph Extension A */
  ["\u3A3E", "\u3A3E"], /* CJK Ideograph Extension A */
  ["\u3A3F", "\u3A3F"], /* CJK Ideograph Extension A */
  ["\u3A40", "\u3A40"], /* CJK Ideograph Extension A */
  ["\u3A41", "\u3A41"], /* CJK Ideograph Extension A */
  ["\u3A42", "\u3A42"], /* CJK Ideograph Extension A */
  ["\u3A43", "\u3A43"], /* CJK Ideograph Extension A */
  ["\u3A44", "\u3A44"], /* CJK Ideograph Extension A */
  ["\u3A45", "\u3A45"], /* CJK Ideograph Extension A */
  ["\u3A46", "\u3A46"], /* CJK Ideograph Extension A */
  ["\u3A47", "\u3A47"], /* CJK Ideograph Extension A */
  ["\u3A48", "\u3A48"], /* CJK Ideograph Extension A */
  ["\u3A49", "\u3A49"], /* CJK Ideograph Extension A */
  ["\u3A4A", "\u3A4A"], /* CJK Ideograph Extension A */
  ["\u3A4B", "\u3A4B"], /* CJK Ideograph Extension A */
  ["\u3A4C", "\u3A4C"], /* CJK Ideograph Extension A */
  ["\u3A4D", "\u3A4D"], /* CJK Ideograph Extension A */
  ["\u3A4E", "\u3A4E"], /* CJK Ideograph Extension A */
  ["\u3A4F", "\u3A4F"], /* CJK Ideograph Extension A */
  ["\u3A50", "\u3A50"], /* CJK Ideograph Extension A */
  ["\u3A51", "\u3A51"], /* CJK Ideograph Extension A */
  ["\u3A52", "\u3A52"], /* CJK Ideograph Extension A */
  ["\u3A53", "\u3A53"], /* CJK Ideograph Extension A */
  ["\u3A54", "\u3A54"], /* CJK Ideograph Extension A */
  ["\u3A55", "\u3A55"], /* CJK Ideograph Extension A */
  ["\u3A56", "\u3A56"], /* CJK Ideograph Extension A */
  ["\u3A57", "\u3A57"], /* CJK Ideograph Extension A */
  ["\u3A58", "\u3A58"], /* CJK Ideograph Extension A */
  ["\u3A59", "\u3A59"], /* CJK Ideograph Extension A */
  ["\u3A5A", "\u3A5A"], /* CJK Ideograph Extension A */
  ["\u3A5B", "\u3A5B"], /* CJK Ideograph Extension A */
  ["\u3A5C", "\u3A5C"], /* CJK Ideograph Extension A */
  ["\u3A5D", "\u3A5D"], /* CJK Ideograph Extension A */
  ["\u3A5E", "\u3A5E"], /* CJK Ideograph Extension A */
  ["\u3A5F", "\u3A5F"], /* CJK Ideograph Extension A */
  ["\u3A60", "\u3A60"], /* CJK Ideograph Extension A */
  ["\u3A61", "\u3A61"], /* CJK Ideograph Extension A */
  ["\u3A62", "\u3A62"], /* CJK Ideograph Extension A */
  ["\u3A63", "\u3A63"], /* CJK Ideograph Extension A */
  ["\u3A64", "\u3A64"], /* CJK Ideograph Extension A */
  ["\u3A65", "\u3A65"], /* CJK Ideograph Extension A */
  ["\u3A66", "\u3A66"], /* CJK Ideograph Extension A */
  ["\u3A67", "\u3A67"], /* CJK Ideograph Extension A */
  ["\u3A68", "\u3A68"], /* CJK Ideograph Extension A */
  ["\u3A69", "\u3A69"], /* CJK Ideograph Extension A */
  ["\u3A6A", "\u3A6A"], /* CJK Ideograph Extension A */
  ["\u3A6B", "\u3A6B"], /* CJK Ideograph Extension A */
  ["\u3A6C", "\u3A6C"], /* CJK Ideograph Extension A */
  ["\u3A6D", "\u3A6D"], /* CJK Ideograph Extension A */
  ["\u3A6E", "\u3A6E"], /* CJK Ideograph Extension A */
  ["\u3A6F", "\u3A6F"], /* CJK Ideograph Extension A */
  ["\u3A70", "\u3A70"], /* CJK Ideograph Extension A */
  ["\u3A71", "\u3A71"], /* CJK Ideograph Extension A */
  ["\u3A72", "\u3A72"], /* CJK Ideograph Extension A */
  ["\u3A73", "\u3A73"], /* CJK Ideograph Extension A */
  ["\u3A74", "\u3A74"], /* CJK Ideograph Extension A */
  ["\u3A75", "\u3A75"], /* CJK Ideograph Extension A */
  ["\u3A76", "\u3A76"], /* CJK Ideograph Extension A */
  ["\u3A77", "\u3A77"], /* CJK Ideograph Extension A */
  ["\u3A78", "\u3A78"], /* CJK Ideograph Extension A */
  ["\u3A79", "\u3A79"], /* CJK Ideograph Extension A */
  ["\u3A7A", "\u3A7A"], /* CJK Ideograph Extension A */
  ["\u3A7B", "\u3A7B"], /* CJK Ideograph Extension A */
  ["\u3A7C", "\u3A7C"], /* CJK Ideograph Extension A */
  ["\u3A7D", "\u3A7D"], /* CJK Ideograph Extension A */
  ["\u3A7E", "\u3A7E"], /* CJK Ideograph Extension A */
  ["\u3A7F", "\u3A7F"], /* CJK Ideograph Extension A */
  ["\u3A80", "\u3A80"], /* CJK Ideograph Extension A */
  ["\u3A81", "\u3A81"], /* CJK Ideograph Extension A */
  ["\u3A82", "\u3A82"], /* CJK Ideograph Extension A */
  ["\u3A83", "\u3A83"], /* CJK Ideograph Extension A */
  ["\u3A84", "\u3A84"], /* CJK Ideograph Extension A */
  ["\u3A85", "\u3A85"], /* CJK Ideograph Extension A */
  ["\u3A86", "\u3A86"], /* CJK Ideograph Extension A */
  ["\u3A87", "\u3A87"], /* CJK Ideograph Extension A */
  ["\u3A88", "\u3A88"], /* CJK Ideograph Extension A */
  ["\u3A89", "\u3A89"], /* CJK Ideograph Extension A */
  ["\u3A8A", "\u3A8A"], /* CJK Ideograph Extension A */
  ["\u3A8B", "\u3A8B"], /* CJK Ideograph Extension A */
  ["\u3A8C", "\u3A8C"], /* CJK Ideograph Extension A */
  ["\u3A8D", "\u3A8D"], /* CJK Ideograph Extension A */
  ["\u3A8E", "\u3A8E"], /* CJK Ideograph Extension A */
  ["\u3A8F", "\u3A8F"], /* CJK Ideograph Extension A */
  ["\u3A90", "\u3A90"], /* CJK Ideograph Extension A */
  ["\u3A91", "\u3A91"], /* CJK Ideograph Extension A */
  ["\u3A92", "\u3A92"], /* CJK Ideograph Extension A */
  ["\u3A93", "\u3A93"], /* CJK Ideograph Extension A */
  ["\u3A94", "\u3A94"], /* CJK Ideograph Extension A */
  ["\u3A95", "\u3A95"], /* CJK Ideograph Extension A */
  ["\u3A96", "\u3A96"], /* CJK Ideograph Extension A */
  ["\u3A97", "\u3A97"], /* CJK Ideograph Extension A */
  ["\u3A98", "\u3A98"], /* CJK Ideograph Extension A */
  ["\u3A99", "\u3A99"], /* CJK Ideograph Extension A */
  ["\u3A9A", "\u3A9A"], /* CJK Ideograph Extension A */
  ["\u3A9B", "\u3A9B"], /* CJK Ideograph Extension A */
  ["\u3A9C", "\u3A9C"], /* CJK Ideograph Extension A */
  ["\u3A9D", "\u3A9D"], /* CJK Ideograph Extension A */
  ["\u3A9E", "\u3A9E"], /* CJK Ideograph Extension A */
  ["\u3A9F", "\u3A9F"], /* CJK Ideograph Extension A */
  ["\u3AA0", "\u3AA0"], /* CJK Ideograph Extension A */
  ["\u3AA1", "\u3AA1"], /* CJK Ideograph Extension A */
  ["\u3AA2", "\u3AA2"], /* CJK Ideograph Extension A */
  ["\u3AA3", "\u3AA3"], /* CJK Ideograph Extension A */
  ["\u3AA4", "\u3AA4"], /* CJK Ideograph Extension A */
  ["\u3AA5", "\u3AA5"], /* CJK Ideograph Extension A */
  ["\u3AA6", "\u3AA6"], /* CJK Ideograph Extension A */
  ["\u3AA7", "\u3AA7"], /* CJK Ideograph Extension A */
  ["\u3AA8", "\u3AA8"], /* CJK Ideograph Extension A */
  ["\u3AA9", "\u3AA9"], /* CJK Ideograph Extension A */
  ["\u3AAA", "\u3AAA"], /* CJK Ideograph Extension A */
  ["\u3AAB", "\u3AAB"], /* CJK Ideograph Extension A */
  ["\u3AAC", "\u3AAC"], /* CJK Ideograph Extension A */
  ["\u3AAD", "\u3AAD"], /* CJK Ideograph Extension A */
  ["\u3AAE", "\u3AAE"], /* CJK Ideograph Extension A */
  ["\u3AAF", "\u3AAF"], /* CJK Ideograph Extension A */
  ["\u3AB0", "\u3AB0"], /* CJK Ideograph Extension A */
  ["\u3AB1", "\u3AB1"], /* CJK Ideograph Extension A */
  ["\u3AB2", "\u3AB2"], /* CJK Ideograph Extension A */
  ["\u3AB3", "\u3AB3"], /* CJK Ideograph Extension A */
  ["\u3AB4", "\u3AB4"], /* CJK Ideograph Extension A */
  ["\u3AB5", "\u3AB5"], /* CJK Ideograph Extension A */
  ["\u3AB6", "\u3AB6"], /* CJK Ideograph Extension A */
  ["\u3AB7", "\u3AB7"], /* CJK Ideograph Extension A */
  ["\u3AB8", "\u3AB8"], /* CJK Ideograph Extension A */
  ["\u3AB9", "\u3AB9"], /* CJK Ideograph Extension A */
  ["\u3ABA", "\u3ABA"], /* CJK Ideograph Extension A */
  ["\u3ABB", "\u3ABB"], /* CJK Ideograph Extension A */
  ["\u3ABC", "\u3ABC"], /* CJK Ideograph Extension A */
  ["\u3ABD", "\u3ABD"], /* CJK Ideograph Extension A */
  ["\u3ABE", "\u3ABE"], /* CJK Ideograph Extension A */
  ["\u3ABF", "\u3ABF"], /* CJK Ideograph Extension A */
  ["\u3AC0", "\u3AC0"], /* CJK Ideograph Extension A */
  ["\u3AC1", "\u3AC1"], /* CJK Ideograph Extension A */
  ["\u3AC2", "\u3AC2"], /* CJK Ideograph Extension A */
  ["\u3AC3", "\u3AC3"], /* CJK Ideograph Extension A */
  ["\u3AC4", "\u3AC4"], /* CJK Ideograph Extension A */
  ["\u3AC5", "\u3AC5"], /* CJK Ideograph Extension A */
  ["\u3AC6", "\u3AC6"], /* CJK Ideograph Extension A */
  ["\u3AC7", "\u3AC7"], /* CJK Ideograph Extension A */
  ["\u3AC8", "\u3AC8"], /* CJK Ideograph Extension A */
  ["\u3AC9", "\u3AC9"], /* CJK Ideograph Extension A */
  ["\u3ACA", "\u3ACA"], /* CJK Ideograph Extension A */
  ["\u3ACB", "\u3ACB"], /* CJK Ideograph Extension A */
  ["\u3ACC", "\u3ACC"], /* CJK Ideograph Extension A */
  ["\u3ACD", "\u3ACD"], /* CJK Ideograph Extension A */
  ["\u3ACE", "\u3ACE"], /* CJK Ideograph Extension A */
  ["\u3ACF", "\u3ACF"], /* CJK Ideograph Extension A */
  ["\u3AD0", "\u3AD0"], /* CJK Ideograph Extension A */
  ["\u3AD1", "\u3AD1"], /* CJK Ideograph Extension A */
  ["\u3AD2", "\u3AD2"], /* CJK Ideograph Extension A */
  ["\u3AD3", "\u3AD3"], /* CJK Ideograph Extension A */
  ["\u3AD4", "\u3AD4"], /* CJK Ideograph Extension A */
  ["\u3AD5", "\u3AD5"], /* CJK Ideograph Extension A */
  ["\u3AD6", "\u3AD6"], /* CJK Ideograph Extension A */
  ["\u3AD7", "\u3AD7"], /* CJK Ideograph Extension A */
  ["\u3AD8", "\u3AD8"], /* CJK Ideograph Extension A */
  ["\u3AD9", "\u3AD9"], /* CJK Ideograph Extension A */
  ["\u3ADA", "\u3ADA"], /* CJK Ideograph Extension A */
  ["\u3ADB", "\u3ADB"], /* CJK Ideograph Extension A */
  ["\u3ADC", "\u3ADC"], /* CJK Ideograph Extension A */
  ["\u3ADD", "\u3ADD"], /* CJK Ideograph Extension A */
  ["\u3ADE", "\u3ADE"], /* CJK Ideograph Extension A */
  ["\u3ADF", "\u3ADF"], /* CJK Ideograph Extension A */
  ["\u3AE0", "\u3AE0"], /* CJK Ideograph Extension A */
  ["\u3AE1", "\u3AE1"], /* CJK Ideograph Extension A */
  ["\u3AE2", "\u3AE2"], /* CJK Ideograph Extension A */
  ["\u3AE3", "\u3AE3"], /* CJK Ideograph Extension A */
  ["\u3AE4", "\u3AE4"], /* CJK Ideograph Extension A */
  ["\u3AE5", "\u3AE5"], /* CJK Ideograph Extension A */
  ["\u3AE6", "\u3AE6"], /* CJK Ideograph Extension A */
  ["\u3AE7", "\u3AE7"], /* CJK Ideograph Extension A */
  ["\u3AE8", "\u3AE8"], /* CJK Ideograph Extension A */
  ["\u3AE9", "\u3AE9"], /* CJK Ideograph Extension A */
  ["\u3AEA", "\u3AEA"], /* CJK Ideograph Extension A */
  ["\u3AEB", "\u3AEB"], /* CJK Ideograph Extension A */
  ["\u3AEC", "\u3AEC"], /* CJK Ideograph Extension A */
  ["\u3AED", "\u3AED"], /* CJK Ideograph Extension A */
  ["\u3AEE", "\u3AEE"], /* CJK Ideograph Extension A */
  ["\u3AEF", "\u3AEF"], /* CJK Ideograph Extension A */
  ["\u3AF0", "\u3AF0"], /* CJK Ideograph Extension A */
  ["\u3AF1", "\u3AF1"], /* CJK Ideograph Extension A */
  ["\u3AF2", "\u3AF2"], /* CJK Ideograph Extension A */
  ["\u3AF3", "\u3AF3"], /* CJK Ideograph Extension A */
  ["\u3AF4", "\u3AF4"], /* CJK Ideograph Extension A */
  ["\u3AF5", "\u3AF5"], /* CJK Ideograph Extension A */
  ["\u3AF6", "\u3AF6"], /* CJK Ideograph Extension A */
  ["\u3AF7", "\u3AF7"], /* CJK Ideograph Extension A */
  ["\u3AF8", "\u3AF8"], /* CJK Ideograph Extension A */
  ["\u3AF9", "\u3AF9"], /* CJK Ideograph Extension A */
  ["\u3AFA", "\u3AFA"], /* CJK Ideograph Extension A */
  ["\u3AFB", "\u3AFB"], /* CJK Ideograph Extension A */
  ["\u3AFC", "\u3AFC"], /* CJK Ideograph Extension A */
  ["\u3AFD", "\u3AFD"], /* CJK Ideograph Extension A */
  ["\u3AFE", "\u3AFE"], /* CJK Ideograph Extension A */
  ["\u3AFF", "\u3AFF"], /* CJK Ideograph Extension A */
  ["\u3B00", "\u3B00"], /* CJK Ideograph Extension A */
  ["\u3B01", "\u3B01"], /* CJK Ideograph Extension A */
  ["\u3B02", "\u3B02"], /* CJK Ideograph Extension A */
  ["\u3B03", "\u3B03"], /* CJK Ideograph Extension A */
  ["\u3B04", "\u3B04"], /* CJK Ideograph Extension A */
  ["\u3B05", "\u3B05"], /* CJK Ideograph Extension A */
  ["\u3B06", "\u3B06"], /* CJK Ideograph Extension A */
  ["\u3B07", "\u3B07"], /* CJK Ideograph Extension A */
  ["\u3B08", "\u3B08"], /* CJK Ideograph Extension A */
  ["\u3B09", "\u3B09"], /* CJK Ideograph Extension A */
  ["\u3B0A", "\u3B0A"], /* CJK Ideograph Extension A */
  ["\u3B0B", "\u3B0B"], /* CJK Ideograph Extension A */
  ["\u3B0C", "\u3B0C"], /* CJK Ideograph Extension A */
  ["\u3B0D", "\u3B0D"], /* CJK Ideograph Extension A */
  ["\u3B0E", "\u3B0E"], /* CJK Ideograph Extension A */
  ["\u3B0F", "\u3B0F"], /* CJK Ideograph Extension A */
  ["\u3B10", "\u3B10"], /* CJK Ideograph Extension A */
  ["\u3B11", "\u3B11"], /* CJK Ideograph Extension A */
  ["\u3B12", "\u3B12"], /* CJK Ideograph Extension A */
  ["\u3B13", "\u3B13"], /* CJK Ideograph Extension A */
  ["\u3B14", "\u3B14"], /* CJK Ideograph Extension A */
  ["\u3B15", "\u3B15"], /* CJK Ideograph Extension A */
  ["\u3B16", "\u3B16"], /* CJK Ideograph Extension A */
  ["\u3B17", "\u3B17"], /* CJK Ideograph Extension A */
  ["\u3B18", "\u3B18"], /* CJK Ideograph Extension A */
  ["\u3B19", "\u3B19"], /* CJK Ideograph Extension A */
  ["\u3B1A", "\u3B1A"], /* CJK Ideograph Extension A */
  ["\u3B1B", "\u3B1B"], /* CJK Ideograph Extension A */
  ["\u3B1C", "\u3B1C"], /* CJK Ideograph Extension A */
  ["\u3B1D", "\u3B1D"], /* CJK Ideograph Extension A */
  ["\u3B1E", "\u3B1E"], /* CJK Ideograph Extension A */
  ["\u3B1F", "\u3B1F"], /* CJK Ideograph Extension A */
  ["\u3B20", "\u3B20"], /* CJK Ideograph Extension A */
  ["\u3B21", "\u3B21"], /* CJK Ideograph Extension A */
  ["\u3B22", "\u3B22"], /* CJK Ideograph Extension A */
  ["\u3B23", "\u3B23"], /* CJK Ideograph Extension A */
  ["\u3B24", "\u3B24"], /* CJK Ideograph Extension A */
  ["\u3B25", "\u3B25"], /* CJK Ideograph Extension A */
  ["\u3B26", "\u3B26"], /* CJK Ideograph Extension A */
  ["\u3B27", "\u3B27"], /* CJK Ideograph Extension A */
  ["\u3B28", "\u3B28"], /* CJK Ideograph Extension A */
  ["\u3B29", "\u3B29"], /* CJK Ideograph Extension A */
  ["\u3B2A", "\u3B2A"], /* CJK Ideograph Extension A */
  ["\u3B2B", "\u3B2B"], /* CJK Ideograph Extension A */
  ["\u3B2C", "\u3B2C"], /* CJK Ideograph Extension A */
  ["\u3B2D", "\u3B2D"], /* CJK Ideograph Extension A */
  ["\u3B2E", "\u3B2E"], /* CJK Ideograph Extension A */
  ["\u3B2F", "\u3B2F"], /* CJK Ideograph Extension A */
  ["\u3B30", "\u3B30"], /* CJK Ideograph Extension A */
  ["\u3B31", "\u3B31"], /* CJK Ideograph Extension A */
  ["\u3B32", "\u3B32"], /* CJK Ideograph Extension A */
  ["\u3B33", "\u3B33"], /* CJK Ideograph Extension A */
  ["\u3B34", "\u3B34"], /* CJK Ideograph Extension A */
  ["\u3B35", "\u3B35"], /* CJK Ideograph Extension A */
  ["\u3B36", "\u3B36"], /* CJK Ideograph Extension A */
  ["\u3B37", "\u3B37"], /* CJK Ideograph Extension A */
  ["\u3B38", "\u3B38"], /* CJK Ideograph Extension A */
  ["\u3B39", "\u3B39"], /* CJK Ideograph Extension A */
  ["\u3B3A", "\u3B3A"], /* CJK Ideograph Extension A */
  ["\u3B3B", "\u3B3B"], /* CJK Ideograph Extension A */
  ["\u3B3C", "\u3B3C"], /* CJK Ideograph Extension A */
  ["\u3B3D", "\u3B3D"], /* CJK Ideograph Extension A */
  ["\u3B3E", "\u3B3E"], /* CJK Ideograph Extension A */
  ["\u3B3F", "\u3B3F"], /* CJK Ideograph Extension A */
  ["\u3B40", "\u3B40"], /* CJK Ideograph Extension A */
  ["\u3B41", "\u3B41"], /* CJK Ideograph Extension A */
  ["\u3B42", "\u3B42"], /* CJK Ideograph Extension A */
  ["\u3B43", "\u3B43"], /* CJK Ideograph Extension A */
  ["\u3B44", "\u3B44"], /* CJK Ideograph Extension A */
  ["\u3B45", "\u3B45"], /* CJK Ideograph Extension A */
  ["\u3B46", "\u3B46"], /* CJK Ideograph Extension A */
  ["\u3B47", "\u3B47"], /* CJK Ideograph Extension A */
  ["\u3B48", "\u3B48"], /* CJK Ideograph Extension A */
  ["\u3B49", "\u3B49"], /* CJK Ideograph Extension A */
  ["\u3B4A", "\u3B4A"], /* CJK Ideograph Extension A */
  ["\u3B4B", "\u3B4B"], /* CJK Ideograph Extension A */
  ["\u3B4C", "\u3B4C"], /* CJK Ideograph Extension A */
  ["\u3B4D", "\u3B4D"], /* CJK Ideograph Extension A */
  ["\u3B4E", "\u3B4E"], /* CJK Ideograph Extension A */
  ["\u3B4F", "\u3B4F"], /* CJK Ideograph Extension A */
  ["\u3B50", "\u3B50"], /* CJK Ideograph Extension A */
  ["\u3B51", "\u3B51"], /* CJK Ideograph Extension A */
  ["\u3B52", "\u3B52"], /* CJK Ideograph Extension A */
  ["\u3B53", "\u3B53"], /* CJK Ideograph Extension A */
  ["\u3B54", "\u3B54"], /* CJK Ideograph Extension A */
  ["\u3B55", "\u3B55"], /* CJK Ideograph Extension A */
  ["\u3B56", "\u3B56"], /* CJK Ideograph Extension A */
  ["\u3B57", "\u3B57"], /* CJK Ideograph Extension A */
  ["\u3B58", "\u3B58"], /* CJK Ideograph Extension A */
  ["\u3B59", "\u3B59"], /* CJK Ideograph Extension A */
  ["\u3B5A", "\u3B5A"], /* CJK Ideograph Extension A */
  ["\u3B5B", "\u3B5B"], /* CJK Ideograph Extension A */
  ["\u3B5C", "\u3B5C"], /* CJK Ideograph Extension A */
  ["\u3B5D", "\u3B5D"], /* CJK Ideograph Extension A */
  ["\u3B5E", "\u3B5E"], /* CJK Ideograph Extension A */
  ["\u3B5F", "\u3B5F"], /* CJK Ideograph Extension A */
  ["\u3B60", "\u3B60"], /* CJK Ideograph Extension A */
  ["\u3B61", "\u3B61"], /* CJK Ideograph Extension A */
  ["\u3B62", "\u3B62"], /* CJK Ideograph Extension A */
  ["\u3B63", "\u3B63"], /* CJK Ideograph Extension A */
  ["\u3B64", "\u3B64"], /* CJK Ideograph Extension A */
  ["\u3B65", "\u3B65"], /* CJK Ideograph Extension A */
  ["\u3B66", "\u3B66"], /* CJK Ideograph Extension A */
  ["\u3B67", "\u3B67"], /* CJK Ideograph Extension A */
  ["\u3B68", "\u3B68"], /* CJK Ideograph Extension A */
  ["\u3B69", "\u3B69"], /* CJK Ideograph Extension A */
  ["\u3B6A", "\u3B6A"], /* CJK Ideograph Extension A */
  ["\u3B6B", "\u3B6B"], /* CJK Ideograph Extension A */
  ["\u3B6C", "\u3B6C"], /* CJK Ideograph Extension A */
  ["\u3B6D", "\u3B6D"], /* CJK Ideograph Extension A */
  ["\u3B6E", "\u3B6E"], /* CJK Ideograph Extension A */
  ["\u3B6F", "\u3B6F"], /* CJK Ideograph Extension A */
  ["\u3B70", "\u3B70"], /* CJK Ideograph Extension A */
  ["\u3B71", "\u3B71"], /* CJK Ideograph Extension A */
  ["\u3B72", "\u3B72"], /* CJK Ideograph Extension A */
  ["\u3B73", "\u3B73"], /* CJK Ideograph Extension A */
  ["\u3B74", "\u3B74"], /* CJK Ideograph Extension A */
  ["\u3B75", "\u3B75"], /* CJK Ideograph Extension A */
  ["\u3B76", "\u3B76"], /* CJK Ideograph Extension A */
  ["\u3B77", "\u3B77"], /* CJK Ideograph Extension A */
  ["\u3B78", "\u3B78"], /* CJK Ideograph Extension A */
  ["\u3B79", "\u3B79"], /* CJK Ideograph Extension A */
  ["\u3B7A", "\u3B7A"], /* CJK Ideograph Extension A */
  ["\u3B7B", "\u3B7B"], /* CJK Ideograph Extension A */
  ["\u3B7C", "\u3B7C"], /* CJK Ideograph Extension A */
  ["\u3B7D", "\u3B7D"], /* CJK Ideograph Extension A */
  ["\u3B7E", "\u3B7E"], /* CJK Ideograph Extension A */
  ["\u3B7F", "\u3B7F"], /* CJK Ideograph Extension A */
  ["\u3B80", "\u3B80"], /* CJK Ideograph Extension A */
  ["\u3B81", "\u3B81"], /* CJK Ideograph Extension A */
  ["\u3B82", "\u3B82"], /* CJK Ideograph Extension A */
  ["\u3B83", "\u3B83"], /* CJK Ideograph Extension A */
  ["\u3B84", "\u3B84"], /* CJK Ideograph Extension A */
  ["\u3B85", "\u3B85"], /* CJK Ideograph Extension A */
  ["\u3B86", "\u3B86"], /* CJK Ideograph Extension A */
  ["\u3B87", "\u3B87"], /* CJK Ideograph Extension A */
  ["\u3B88", "\u3B88"], /* CJK Ideograph Extension A */
  ["\u3B89", "\u3B89"], /* CJK Ideograph Extension A */
  ["\u3B8A", "\u3B8A"], /* CJK Ideograph Extension A */
  ["\u3B8B", "\u3B8B"], /* CJK Ideograph Extension A */
  ["\u3B8C", "\u3B8C"], /* CJK Ideograph Extension A */
  ["\u3B8D", "\u3B8D"], /* CJK Ideograph Extension A */
  ["\u3B8E", "\u3B8E"], /* CJK Ideograph Extension A */
  ["\u3B8F", "\u3B8F"], /* CJK Ideograph Extension A */
  ["\u3B90", "\u3B90"], /* CJK Ideograph Extension A */
  ["\u3B91", "\u3B91"], /* CJK Ideograph Extension A */
  ["\u3B92", "\u3B92"], /* CJK Ideograph Extension A */
  ["\u3B93", "\u3B93"], /* CJK Ideograph Extension A */
  ["\u3B94", "\u3B94"], /* CJK Ideograph Extension A */
  ["\u3B95", "\u3B95"], /* CJK Ideograph Extension A */
  ["\u3B96", "\u3B96"], /* CJK Ideograph Extension A */
  ["\u3B97", "\u3B97"], /* CJK Ideograph Extension A */
  ["\u3B98", "\u3B98"], /* CJK Ideograph Extension A */
  ["\u3B99", "\u3B99"], /* CJK Ideograph Extension A */
  ["\u3B9A", "\u3B9A"], /* CJK Ideograph Extension A */
  ["\u3B9B", "\u3B9B"], /* CJK Ideograph Extension A */
  ["\u3B9C", "\u3B9C"], /* CJK Ideograph Extension A */
  ["\u3B9D", "\u3B9D"], /* CJK Ideograph Extension A */
  ["\u3B9E", "\u3B9E"], /* CJK Ideograph Extension A */
  ["\u3B9F", "\u3B9F"], /* CJK Ideograph Extension A */
  ["\u3BA0", "\u3BA0"], /* CJK Ideograph Extension A */
  ["\u3BA1", "\u3BA1"], /* CJK Ideograph Extension A */
  ["\u3BA2", "\u3BA2"], /* CJK Ideograph Extension A */
  ["\u3BA3", "\u3BA3"], /* CJK Ideograph Extension A */
  ["\u3BA4", "\u3BA4"], /* CJK Ideograph Extension A */
  ["\u3BA5", "\u3BA5"], /* CJK Ideograph Extension A */
  ["\u3BA6", "\u3BA6"], /* CJK Ideograph Extension A */
  ["\u3BA7", "\u3BA7"], /* CJK Ideograph Extension A */
  ["\u3BA8", "\u3BA8"], /* CJK Ideograph Extension A */
  ["\u3BA9", "\u3BA9"], /* CJK Ideograph Extension A */
  ["\u3BAA", "\u3BAA"], /* CJK Ideograph Extension A */
  ["\u3BAB", "\u3BAB"], /* CJK Ideograph Extension A */
  ["\u3BAC", "\u3BAC"], /* CJK Ideograph Extension A */
  ["\u3BAD", "\u3BAD"], /* CJK Ideograph Extension A */
  ["\u3BAE", "\u3BAE"], /* CJK Ideograph Extension A */
  ["\u3BAF", "\u3BAF"], /* CJK Ideograph Extension A */
  ["\u3BB0", "\u3BB0"], /* CJK Ideograph Extension A */
  ["\u3BB1", "\u3BB1"], /* CJK Ideograph Extension A */
  ["\u3BB2", "\u3BB2"], /* CJK Ideograph Extension A */
  ["\u3BB3", "\u3BB3"], /* CJK Ideograph Extension A */
  ["\u3BB4", "\u3BB4"], /* CJK Ideograph Extension A */
  ["\u3BB5", "\u3BB5"], /* CJK Ideograph Extension A */
  ["\u3BB6", "\u3BB6"], /* CJK Ideograph Extension A */
  ["\u3BB7", "\u3BB7"], /* CJK Ideograph Extension A */
  ["\u3BB8", "\u3BB8"], /* CJK Ideograph Extension A */
  ["\u3BB9", "\u3BB9"], /* CJK Ideograph Extension A */
  ["\u3BBA", "\u3BBA"], /* CJK Ideograph Extension A */
  ["\u3BBB", "\u3BBB"], /* CJK Ideograph Extension A */
  ["\u3BBC", "\u3BBC"], /* CJK Ideograph Extension A */
  ["\u3BBD", "\u3BBD"], /* CJK Ideograph Extension A */
  ["\u3BBE", "\u3BBE"], /* CJK Ideograph Extension A */
  ["\u3BBF", "\u3BBF"], /* CJK Ideograph Extension A */
  ["\u3BC0", "\u3BC0"], /* CJK Ideograph Extension A */
  ["\u3BC1", "\u3BC1"], /* CJK Ideograph Extension A */
  ["\u3BC2", "\u3BC2"], /* CJK Ideograph Extension A */
  ["\u3BC3", "\u3BC3"], /* CJK Ideograph Extension A */
  ["\u3BC4", "\u3BC4"], /* CJK Ideograph Extension A */
  ["\u3BC5", "\u3BC5"], /* CJK Ideograph Extension A */
  ["\u3BC6", "\u3BC6"], /* CJK Ideograph Extension A */
  ["\u3BC7", "\u3BC7"], /* CJK Ideograph Extension A */
  ["\u3BC8", "\u3BC8"], /* CJK Ideograph Extension A */
  ["\u3BC9", "\u3BC9"], /* CJK Ideograph Extension A */
  ["\u3BCA", "\u3BCA"], /* CJK Ideograph Extension A */
  ["\u3BCB", "\u3BCB"], /* CJK Ideograph Extension A */
  ["\u3BCC", "\u3BCC"], /* CJK Ideograph Extension A */
  ["\u3BCD", "\u3BCD"], /* CJK Ideograph Extension A */
  ["\u3BCE", "\u3BCE"], /* CJK Ideograph Extension A */
  ["\u3BCF", "\u3BCF"], /* CJK Ideograph Extension A */
  ["\u3BD0", "\u3BD0"], /* CJK Ideograph Extension A */
  ["\u3BD1", "\u3BD1"], /* CJK Ideograph Extension A */
  ["\u3BD2", "\u3BD2"], /* CJK Ideograph Extension A */
  ["\u3BD3", "\u3BD3"], /* CJK Ideograph Extension A */
  ["\u3BD4", "\u3BD4"], /* CJK Ideograph Extension A */
  ["\u3BD5", "\u3BD5"], /* CJK Ideograph Extension A */
  ["\u3BD6", "\u3BD6"], /* CJK Ideograph Extension A */
  ["\u3BD7", "\u3BD7"], /* CJK Ideograph Extension A */
  ["\u3BD8", "\u3BD8"], /* CJK Ideograph Extension A */
  ["\u3BD9", "\u3BD9"], /* CJK Ideograph Extension A */
  ["\u3BDA", "\u3BDA"], /* CJK Ideograph Extension A */
  ["\u3BDB", "\u3BDB"], /* CJK Ideograph Extension A */
  ["\u3BDC", "\u3BDC"], /* CJK Ideograph Extension A */
  ["\u3BDD", "\u3BDD"], /* CJK Ideograph Extension A */
  ["\u3BDE", "\u3BDE"], /* CJK Ideograph Extension A */
  ["\u3BDF", "\u3BDF"], /* CJK Ideograph Extension A */
  ["\u3BE0", "\u3BE0"], /* CJK Ideograph Extension A */
  ["\u3BE1", "\u3BE1"], /* CJK Ideograph Extension A */
  ["\u3BE2", "\u3BE2"], /* CJK Ideograph Extension A */
  ["\u3BE3", "\u3BE3"], /* CJK Ideograph Extension A */
  ["\u3BE4", "\u3BE4"], /* CJK Ideograph Extension A */
  ["\u3BE5", "\u3BE5"], /* CJK Ideograph Extension A */
  ["\u3BE6", "\u3BE6"], /* CJK Ideograph Extension A */
  ["\u3BE7", "\u3BE7"], /* CJK Ideograph Extension A */
  ["\u3BE8", "\u3BE8"], /* CJK Ideograph Extension A */
  ["\u3BE9", "\u3BE9"], /* CJK Ideograph Extension A */
  ["\u3BEA", "\u3BEA"], /* CJK Ideograph Extension A */
  ["\u3BEB", "\u3BEB"], /* CJK Ideograph Extension A */
  ["\u3BEC", "\u3BEC"], /* CJK Ideograph Extension A */
  ["\u3BED", "\u3BED"], /* CJK Ideograph Extension A */
  ["\u3BEE", "\u3BEE"], /* CJK Ideograph Extension A */
  ["\u3BEF", "\u3BEF"], /* CJK Ideograph Extension A */
  ["\u3BF0", "\u3BF0"], /* CJK Ideograph Extension A */
  ["\u3BF1", "\u3BF1"], /* CJK Ideograph Extension A */
  ["\u3BF2", "\u3BF2"], /* CJK Ideograph Extension A */
  ["\u3BF3", "\u3BF3"], /* CJK Ideograph Extension A */
  ["\u3BF4", "\u3BF4"], /* CJK Ideograph Extension A */
  ["\u3BF5", "\u3BF5"], /* CJK Ideograph Extension A */
  ["\u3BF6", "\u3BF6"], /* CJK Ideograph Extension A */
  ["\u3BF7", "\u3BF7"], /* CJK Ideograph Extension A */
  ["\u3BF8", "\u3BF8"], /* CJK Ideograph Extension A */
  ["\u3BF9", "\u3BF9"], /* CJK Ideograph Extension A */
  ["\u3BFA", "\u3BFA"], /* CJK Ideograph Extension A */
  ["\u3BFB", "\u3BFB"], /* CJK Ideograph Extension A */
  ["\u3BFC", "\u3BFC"], /* CJK Ideograph Extension A */
  ["\u3BFD", "\u3BFD"], /* CJK Ideograph Extension A */
  ["\u3BFE", "\u3BFE"], /* CJK Ideograph Extension A */
  ["\u3BFF", "\u3BFF"], /* CJK Ideograph Extension A */
  ["\u3C00", "\u3C00"], /* CJK Ideograph Extension A */
  ["\u3C01", "\u3C01"], /* CJK Ideograph Extension A */
  ["\u3C02", "\u3C02"], /* CJK Ideograph Extension A */
  ["\u3C03", "\u3C03"], /* CJK Ideograph Extension A */
  ["\u3C04", "\u3C04"], /* CJK Ideograph Extension A */
  ["\u3C05", "\u3C05"], /* CJK Ideograph Extension A */
  ["\u3C06", "\u3C06"], /* CJK Ideograph Extension A */
  ["\u3C07", "\u3C07"], /* CJK Ideograph Extension A */
  ["\u3C08", "\u3C08"], /* CJK Ideograph Extension A */
  ["\u3C09", "\u3C09"], /* CJK Ideograph Extension A */
  ["\u3C0A", "\u3C0A"], /* CJK Ideograph Extension A */
  ["\u3C0B", "\u3C0B"], /* CJK Ideograph Extension A */
  ["\u3C0C", "\u3C0C"], /* CJK Ideograph Extension A */
  ["\u3C0D", "\u3C0D"], /* CJK Ideograph Extension A */
  ["\u3C0E", "\u3C0E"], /* CJK Ideograph Extension A */
  ["\u3C0F", "\u3C0F"], /* CJK Ideograph Extension A */
  ["\u3C10", "\u3C10"], /* CJK Ideograph Extension A */
  ["\u3C11", "\u3C11"], /* CJK Ideograph Extension A */
  ["\u3C12", "\u3C12"], /* CJK Ideograph Extension A */
  ["\u3C13", "\u3C13"], /* CJK Ideograph Extension A */
  ["\u3C14", "\u3C14"], /* CJK Ideograph Extension A */
  ["\u3C15", "\u3C15"], /* CJK Ideograph Extension A */
  ["\u3C16", "\u3C16"], /* CJK Ideograph Extension A */
  ["\u3C17", "\u3C17"], /* CJK Ideograph Extension A */
  ["\u3C18", "\u3C18"], /* CJK Ideograph Extension A */
  ["\u3C19", "\u3C19"], /* CJK Ideograph Extension A */
  ["\u3C1A", "\u3C1A"], /* CJK Ideograph Extension A */
  ["\u3C1B", "\u3C1B"], /* CJK Ideograph Extension A */
  ["\u3C1C", "\u3C1C"], /* CJK Ideograph Extension A */
  ["\u3C1D", "\u3C1D"], /* CJK Ideograph Extension A */
  ["\u3C1E", "\u3C1E"], /* CJK Ideograph Extension A */
  ["\u3C1F", "\u3C1F"], /* CJK Ideograph Extension A */
  ["\u3C20", "\u3C20"], /* CJK Ideograph Extension A */
  ["\u3C21", "\u3C21"], /* CJK Ideograph Extension A */
  ["\u3C22", "\u3C22"], /* CJK Ideograph Extension A */
  ["\u3C23", "\u3C23"], /* CJK Ideograph Extension A */
  ["\u3C24", "\u3C24"], /* CJK Ideograph Extension A */
  ["\u3C25", "\u3C25"], /* CJK Ideograph Extension A */
  ["\u3C26", "\u3C26"], /* CJK Ideograph Extension A */
  ["\u3C27", "\u3C27"], /* CJK Ideograph Extension A */
  ["\u3C28", "\u3C28"], /* CJK Ideograph Extension A */
  ["\u3C29", "\u3C29"], /* CJK Ideograph Extension A */
  ["\u3C2A", "\u3C2A"], /* CJK Ideograph Extension A */
  ["\u3C2B", "\u3C2B"], /* CJK Ideograph Extension A */
  ["\u3C2C", "\u3C2C"], /* CJK Ideograph Extension A */
  ["\u3C2D", "\u3C2D"], /* CJK Ideograph Extension A */
  ["\u3C2E", "\u3C2E"], /* CJK Ideograph Extension A */
  ["\u3C2F", "\u3C2F"], /* CJK Ideograph Extension A */
  ["\u3C30", "\u3C30"], /* CJK Ideograph Extension A */
  ["\u3C31", "\u3C31"], /* CJK Ideograph Extension A */
  ["\u3C32", "\u3C32"], /* CJK Ideograph Extension A */
  ["\u3C33", "\u3C33"], /* CJK Ideograph Extension A */
  ["\u3C34", "\u3C34"], /* CJK Ideograph Extension A */
  ["\u3C35", "\u3C35"], /* CJK Ideograph Extension A */
  ["\u3C36", "\u3C36"], /* CJK Ideograph Extension A */
  ["\u3C37", "\u3C37"], /* CJK Ideograph Extension A */
  ["\u3C38", "\u3C38"], /* CJK Ideograph Extension A */
  ["\u3C39", "\u3C39"], /* CJK Ideograph Extension A */
  ["\u3C3A", "\u3C3A"], /* CJK Ideograph Extension A */
  ["\u3C3B", "\u3C3B"], /* CJK Ideograph Extension A */
  ["\u3C3C", "\u3C3C"], /* CJK Ideograph Extension A */
  ["\u3C3D", "\u3C3D"], /* CJK Ideograph Extension A */
  ["\u3C3E", "\u3C3E"], /* CJK Ideograph Extension A */
  ["\u3C3F", "\u3C3F"], /* CJK Ideograph Extension A */
  ["\u3C40", "\u3C40"], /* CJK Ideograph Extension A */
  ["\u3C41", "\u3C41"], /* CJK Ideograph Extension A */
  ["\u3C42", "\u3C42"], /* CJK Ideograph Extension A */
  ["\u3C43", "\u3C43"], /* CJK Ideograph Extension A */
  ["\u3C44", "\u3C44"], /* CJK Ideograph Extension A */
  ["\u3C45", "\u3C45"], /* CJK Ideograph Extension A */
  ["\u3C46", "\u3C46"], /* CJK Ideograph Extension A */
  ["\u3C47", "\u3C47"], /* CJK Ideograph Extension A */
  ["\u3C48", "\u3C48"], /* CJK Ideograph Extension A */
  ["\u3C49", "\u3C49"], /* CJK Ideograph Extension A */
  ["\u3C4A", "\u3C4A"], /* CJK Ideograph Extension A */
  ["\u3C4B", "\u3C4B"], /* CJK Ideograph Extension A */
  ["\u3C4C", "\u3C4C"], /* CJK Ideograph Extension A */
  ["\u3C4D", "\u3C4D"], /* CJK Ideograph Extension A */
  ["\u3C4E", "\u3C4E"], /* CJK Ideograph Extension A */
  ["\u3C4F", "\u3C4F"], /* CJK Ideograph Extension A */
  ["\u3C50", "\u3C50"], /* CJK Ideograph Extension A */
  ["\u3C51", "\u3C51"], /* CJK Ideograph Extension A */
  ["\u3C52", "\u3C52"], /* CJK Ideograph Extension A */
  ["\u3C53", "\u3C53"], /* CJK Ideograph Extension A */
  ["\u3C54", "\u3C54"], /* CJK Ideograph Extension A */
  ["\u3C55", "\u3C55"], /* CJK Ideograph Extension A */
  ["\u3C56", "\u3C56"], /* CJK Ideograph Extension A */
  ["\u3C57", "\u3C57"], /* CJK Ideograph Extension A */
  ["\u3C58", "\u3C58"], /* CJK Ideograph Extension A */
  ["\u3C59", "\u3C59"], /* CJK Ideograph Extension A */
  ["\u3C5A", "\u3C5A"], /* CJK Ideograph Extension A */
  ["\u3C5B", "\u3C5B"], /* CJK Ideograph Extension A */
  ["\u3C5C", "\u3C5C"], /* CJK Ideograph Extension A */
  ["\u3C5D", "\u3C5D"], /* CJK Ideograph Extension A */
  ["\u3C5E", "\u3C5E"], /* CJK Ideograph Extension A */
  ["\u3C5F", "\u3C5F"], /* CJK Ideograph Extension A */
  ["\u3C60", "\u3C60"], /* CJK Ideograph Extension A */
  ["\u3C61", "\u3C61"], /* CJK Ideograph Extension A */
  ["\u3C62", "\u3C62"], /* CJK Ideograph Extension A */
  ["\u3C63", "\u3C63"], /* CJK Ideograph Extension A */
  ["\u3C64", "\u3C64"], /* CJK Ideograph Extension A */
  ["\u3C65", "\u3C65"], /* CJK Ideograph Extension A */
  ["\u3C66", "\u3C66"], /* CJK Ideograph Extension A */
  ["\u3C67", "\u3C67"], /* CJK Ideograph Extension A */
  ["\u3C68", "\u3C68"], /* CJK Ideograph Extension A */
  ["\u3C69", "\u3C69"], /* CJK Ideograph Extension A */
  ["\u3C6A", "\u3C6A"], /* CJK Ideograph Extension A */
  ["\u3C6B", "\u3C6B"], /* CJK Ideograph Extension A */
  ["\u3C6C", "\u3C6C"], /* CJK Ideograph Extension A */
  ["\u3C6D", "\u3C6D"], /* CJK Ideograph Extension A */
  ["\u3C6E", "\u3C6E"], /* CJK Ideograph Extension A */
  ["\u3C6F", "\u3C6F"], /* CJK Ideograph Extension A */
  ["\u3C70", "\u3C70"], /* CJK Ideograph Extension A */
  ["\u3C71", "\u3C71"], /* CJK Ideograph Extension A */
  ["\u3C72", "\u3C72"], /* CJK Ideograph Extension A */
  ["\u3C73", "\u3C73"], /* CJK Ideograph Extension A */
  ["\u3C74", "\u3C74"], /* CJK Ideograph Extension A */
  ["\u3C75", "\u3C75"], /* CJK Ideograph Extension A */
  ["\u3C76", "\u3C76"], /* CJK Ideograph Extension A */
  ["\u3C77", "\u3C77"], /* CJK Ideograph Extension A */
  ["\u3C78", "\u3C78"], /* CJK Ideograph Extension A */
  ["\u3C79", "\u3C79"], /* CJK Ideograph Extension A */
  ["\u3C7A", "\u3C7A"], /* CJK Ideograph Extension A */
  ["\u3C7B", "\u3C7B"], /* CJK Ideograph Extension A */
  ["\u3C7C", "\u3C7C"], /* CJK Ideograph Extension A */
  ["\u3C7D", "\u3C7D"], /* CJK Ideograph Extension A */
  ["\u3C7E", "\u3C7E"], /* CJK Ideograph Extension A */
  ["\u3C7F", "\u3C7F"], /* CJK Ideograph Extension A */
  ["\u3C80", "\u3C80"], /* CJK Ideograph Extension A */
  ["\u3C81", "\u3C81"], /* CJK Ideograph Extension A */
  ["\u3C82", "\u3C82"], /* CJK Ideograph Extension A */
  ["\u3C83", "\u3C83"], /* CJK Ideograph Extension A */
  ["\u3C84", "\u3C84"], /* CJK Ideograph Extension A */
  ["\u3C85", "\u3C85"], /* CJK Ideograph Extension A */
  ["\u3C86", "\u3C86"], /* CJK Ideograph Extension A */
  ["\u3C87", "\u3C87"], /* CJK Ideograph Extension A */
  ["\u3C88", "\u3C88"], /* CJK Ideograph Extension A */
  ["\u3C89", "\u3C89"], /* CJK Ideograph Extension A */
  ["\u3C8A", "\u3C8A"], /* CJK Ideograph Extension A */
  ["\u3C8B", "\u3C8B"], /* CJK Ideograph Extension A */
  ["\u3C8C", "\u3C8C"], /* CJK Ideograph Extension A */
  ["\u3C8D", "\u3C8D"], /* CJK Ideograph Extension A */
  ["\u3C8E", "\u3C8E"], /* CJK Ideograph Extension A */
  ["\u3C8F", "\u3C8F"], /* CJK Ideograph Extension A */
  ["\u3C90", "\u3C90"], /* CJK Ideograph Extension A */
  ["\u3C91", "\u3C91"], /* CJK Ideograph Extension A */
  ["\u3C92", "\u3C92"], /* CJK Ideograph Extension A */
  ["\u3C93", "\u3C93"], /* CJK Ideograph Extension A */
  ["\u3C94", "\u3C94"], /* CJK Ideograph Extension A */
  ["\u3C95", "\u3C95"], /* CJK Ideograph Extension A */
  ["\u3C96", "\u3C96"], /* CJK Ideograph Extension A */
  ["\u3C97", "\u3C97"], /* CJK Ideograph Extension A */
  ["\u3C98", "\u3C98"], /* CJK Ideograph Extension A */
  ["\u3C99", "\u3C99"], /* CJK Ideograph Extension A */
  ["\u3C9A", "\u3C9A"], /* CJK Ideograph Extension A */
  ["\u3C9B", "\u3C9B"], /* CJK Ideograph Extension A */
  ["\u3C9C", "\u3C9C"], /* CJK Ideograph Extension A */
  ["\u3C9D", "\u3C9D"], /* CJK Ideograph Extension A */
  ["\u3C9E", "\u3C9E"], /* CJK Ideograph Extension A */
  ["\u3C9F", "\u3C9F"], /* CJK Ideograph Extension A */
  ["\u3CA0", "\u3CA0"], /* CJK Ideograph Extension A */
  ["\u3CA1", "\u3CA1"], /* CJK Ideograph Extension A */
  ["\u3CA2", "\u3CA2"], /* CJK Ideograph Extension A */
  ["\u3CA3", "\u3CA3"], /* CJK Ideograph Extension A */
  ["\u3CA4", "\u3CA4"], /* CJK Ideograph Extension A */
  ["\u3CA5", "\u3CA5"], /* CJK Ideograph Extension A */
  ["\u3CA6", "\u3CA6"], /* CJK Ideograph Extension A */
  ["\u3CA7", "\u3CA7"], /* CJK Ideograph Extension A */
  ["\u3CA8", "\u3CA8"], /* CJK Ideograph Extension A */
  ["\u3CA9", "\u3CA9"], /* CJK Ideograph Extension A */
  ["\u3CAA", "\u3CAA"], /* CJK Ideograph Extension A */
  ["\u3CAB", "\u3CAB"], /* CJK Ideograph Extension A */
  ["\u3CAC", "\u3CAC"], /* CJK Ideograph Extension A */
  ["\u3CAD", "\u3CAD"], /* CJK Ideograph Extension A */
  ["\u3CAE", "\u3CAE"], /* CJK Ideograph Extension A */
  ["\u3CAF", "\u3CAF"], /* CJK Ideograph Extension A */
  ["\u3CB0", "\u3CB0"], /* CJK Ideograph Extension A */
  ["\u3CB1", "\u3CB1"], /* CJK Ideograph Extension A */
  ["\u3CB2", "\u3CB2"], /* CJK Ideograph Extension A */
  ["\u3CB3", "\u3CB3"], /* CJK Ideograph Extension A */
  ["\u3CB4", "\u3CB4"], /* CJK Ideograph Extension A */
  ["\u3CB5", "\u3CB5"], /* CJK Ideograph Extension A */
  ["\u3CB6", "\u3CB6"], /* CJK Ideograph Extension A */
  ["\u3CB7", "\u3CB7"], /* CJK Ideograph Extension A */
  ["\u3CB8", "\u3CB8"], /* CJK Ideograph Extension A */
  ["\u3CB9", "\u3CB9"], /* CJK Ideograph Extension A */
  ["\u3CBA", "\u3CBA"], /* CJK Ideograph Extension A */
  ["\u3CBB", "\u3CBB"], /* CJK Ideograph Extension A */
  ["\u3CBC", "\u3CBC"], /* CJK Ideograph Extension A */
  ["\u3CBD", "\u3CBD"], /* CJK Ideograph Extension A */
  ["\u3CBE", "\u3CBE"], /* CJK Ideograph Extension A */
  ["\u3CBF", "\u3CBF"], /* CJK Ideograph Extension A */
  ["\u3CC0", "\u3CC0"], /* CJK Ideograph Extension A */
  ["\u3CC1", "\u3CC1"], /* CJK Ideograph Extension A */
  ["\u3CC2", "\u3CC2"], /* CJK Ideograph Extension A */
  ["\u3CC3", "\u3CC3"], /* CJK Ideograph Extension A */
  ["\u3CC4", "\u3CC4"], /* CJK Ideograph Extension A */
  ["\u3CC5", "\u3CC5"], /* CJK Ideograph Extension A */
  ["\u3CC6", "\u3CC6"], /* CJK Ideograph Extension A */
  ["\u3CC7", "\u3CC7"], /* CJK Ideograph Extension A */
  ["\u3CC8", "\u3CC8"], /* CJK Ideograph Extension A */
  ["\u3CC9", "\u3CC9"], /* CJK Ideograph Extension A */
  ["\u3CCA", "\u3CCA"], /* CJK Ideograph Extension A */
  ["\u3CCB", "\u3CCB"], /* CJK Ideograph Extension A */
  ["\u3CCC", "\u3CCC"], /* CJK Ideograph Extension A */
  ["\u3CCD", "\u3CCD"], /* CJK Ideograph Extension A */
  ["\u3CCE", "\u3CCE"], /* CJK Ideograph Extension A */
  ["\u3CCF", "\u3CCF"], /* CJK Ideograph Extension A */
  ["\u3CD0", "\u3CD0"], /* CJK Ideograph Extension A */
  ["\u3CD1", "\u3CD1"], /* CJK Ideograph Extension A */
  ["\u3CD2", "\u3CD2"], /* CJK Ideograph Extension A */
  ["\u3CD3", "\u3CD3"], /* CJK Ideograph Extension A */
  ["\u3CD4", "\u3CD4"], /* CJK Ideograph Extension A */
  ["\u3CD5", "\u3CD5"], /* CJK Ideograph Extension A */
  ["\u3CD6", "\u3CD6"], /* CJK Ideograph Extension A */
  ["\u3CD7", "\u3CD7"], /* CJK Ideograph Extension A */
  ["\u3CD8", "\u3CD8"], /* CJK Ideograph Extension A */
  ["\u3CD9", "\u3CD9"], /* CJK Ideograph Extension A */
  ["\u3CDA", "\u3CDA"], /* CJK Ideograph Extension A */
  ["\u3CDB", "\u3CDB"], /* CJK Ideograph Extension A */
  ["\u3CDC", "\u3CDC"], /* CJK Ideograph Extension A */
  ["\u3CDD", "\u3CDD"], /* CJK Ideograph Extension A */
  ["\u3CDE", "\u3CDE"], /* CJK Ideograph Extension A */
  ["\u3CDF", "\u3CDF"], /* CJK Ideograph Extension A */
  ["\u3CE0", "\u3CE0"], /* CJK Ideograph Extension A */
  ["\u3CE1", "\u3CE1"], /* CJK Ideograph Extension A */
  ["\u3CE2", "\u3CE2"], /* CJK Ideograph Extension A */
  ["\u3CE3", "\u3CE3"], /* CJK Ideograph Extension A */
  ["\u3CE4", "\u3CE4"], /* CJK Ideograph Extension A */
  ["\u3CE5", "\u3CE5"], /* CJK Ideograph Extension A */
  ["\u3CE6", "\u3CE6"], /* CJK Ideograph Extension A */
  ["\u3CE7", "\u3CE7"], /* CJK Ideograph Extension A */
  ["\u3CE8", "\u3CE8"], /* CJK Ideograph Extension A */
  ["\u3CE9", "\u3CE9"], /* CJK Ideograph Extension A */
  ["\u3CEA", "\u3CEA"], /* CJK Ideograph Extension A */
  ["\u3CEB", "\u3CEB"], /* CJK Ideograph Extension A */
  ["\u3CEC", "\u3CEC"], /* CJK Ideograph Extension A */
  ["\u3CED", "\u3CED"], /* CJK Ideograph Extension A */
  ["\u3CEE", "\u3CEE"], /* CJK Ideograph Extension A */
  ["\u3CEF", "\u3CEF"], /* CJK Ideograph Extension A */
  ["\u3CF0", "\u3CF0"], /* CJK Ideograph Extension A */
  ["\u3CF1", "\u3CF1"], /* CJK Ideograph Extension A */
  ["\u3CF2", "\u3CF2"], /* CJK Ideograph Extension A */
  ["\u3CF3", "\u3CF3"], /* CJK Ideograph Extension A */
  ["\u3CF4", "\u3CF4"], /* CJK Ideograph Extension A */
  ["\u3CF5", "\u3CF5"], /* CJK Ideograph Extension A */
  ["\u3CF6", "\u3CF6"], /* CJK Ideograph Extension A */
  ["\u3CF7", "\u3CF7"], /* CJK Ideograph Extension A */
  ["\u3CF8", "\u3CF8"], /* CJK Ideograph Extension A */
  ["\u3CF9", "\u3CF9"], /* CJK Ideograph Extension A */
  ["\u3CFA", "\u3CFA"], /* CJK Ideograph Extension A */
  ["\u3CFB", "\u3CFB"], /* CJK Ideograph Extension A */
  ["\u3CFC", "\u3CFC"], /* CJK Ideograph Extension A */
  ["\u3CFD", "\u3CFD"], /* CJK Ideograph Extension A */
  ["\u3CFE", "\u3CFE"], /* CJK Ideograph Extension A */
  ["\u3CFF", "\u3CFF"], /* CJK Ideograph Extension A */
  ["\u3D00", "\u3D00"], /* CJK Ideograph Extension A */
  ["\u3D01", "\u3D01"], /* CJK Ideograph Extension A */
  ["\u3D02", "\u3D02"], /* CJK Ideograph Extension A */
  ["\u3D03", "\u3D03"], /* CJK Ideograph Extension A */
  ["\u3D04", "\u3D04"], /* CJK Ideograph Extension A */
  ["\u3D05", "\u3D05"], /* CJK Ideograph Extension A */
  ["\u3D06", "\u3D06"], /* CJK Ideograph Extension A */
  ["\u3D07", "\u3D07"], /* CJK Ideograph Extension A */
  ["\u3D08", "\u3D08"], /* CJK Ideograph Extension A */
  ["\u3D09", "\u3D09"], /* CJK Ideograph Extension A */
  ["\u3D0A", "\u3D0A"], /* CJK Ideograph Extension A */
  ["\u3D0B", "\u3D0B"], /* CJK Ideograph Extension A */
  ["\u3D0C", "\u3D0C"], /* CJK Ideograph Extension A */
  ["\u3D0D", "\u3D0D"], /* CJK Ideograph Extension A */
  ["\u3D0E", "\u3D0E"], /* CJK Ideograph Extension A */
  ["\u3D0F", "\u3D0F"], /* CJK Ideograph Extension A */
  ["\u3D10", "\u3D10"], /* CJK Ideograph Extension A */
  ["\u3D11", "\u3D11"], /* CJK Ideograph Extension A */
  ["\u3D12", "\u3D12"], /* CJK Ideograph Extension A */
  ["\u3D13", "\u3D13"], /* CJK Ideograph Extension A */
  ["\u3D14", "\u3D14"], /* CJK Ideograph Extension A */
  ["\u3D15", "\u3D15"], /* CJK Ideograph Extension A */
  ["\u3D16", "\u3D16"], /* CJK Ideograph Extension A */
  ["\u3D17", "\u3D17"], /* CJK Ideograph Extension A */
  ["\u3D18", "\u3D18"], /* CJK Ideograph Extension A */
  ["\u3D19", "\u3D19"], /* CJK Ideograph Extension A */
  ["\u3D1A", "\u3D1A"], /* CJK Ideograph Extension A */
  ["\u3D1B", "\u3D1B"], /* CJK Ideograph Extension A */
  ["\u3D1C", "\u3D1C"], /* CJK Ideograph Extension A */
  ["\u3D1D", "\u3D1D"], /* CJK Ideograph Extension A */
  ["\u3D1E", "\u3D1E"], /* CJK Ideograph Extension A */
  ["\u3D1F", "\u3D1F"], /* CJK Ideograph Extension A */
  ["\u3D20", "\u3D20"], /* CJK Ideograph Extension A */
  ["\u3D21", "\u3D21"], /* CJK Ideograph Extension A */
  ["\u3D22", "\u3D22"], /* CJK Ideograph Extension A */
  ["\u3D23", "\u3D23"], /* CJK Ideograph Extension A */
  ["\u3D24", "\u3D24"], /* CJK Ideograph Extension A */
  ["\u3D25", "\u3D25"], /* CJK Ideograph Extension A */
  ["\u3D26", "\u3D26"], /* CJK Ideograph Extension A */
  ["\u3D27", "\u3D27"], /* CJK Ideograph Extension A */
  ["\u3D28", "\u3D28"], /* CJK Ideograph Extension A */
  ["\u3D29", "\u3D29"], /* CJK Ideograph Extension A */
  ["\u3D2A", "\u3D2A"], /* CJK Ideograph Extension A */
  ["\u3D2B", "\u3D2B"], /* CJK Ideograph Extension A */
  ["\u3D2C", "\u3D2C"], /* CJK Ideograph Extension A */
  ["\u3D2D", "\u3D2D"], /* CJK Ideograph Extension A */
  ["\u3D2E", "\u3D2E"], /* CJK Ideograph Extension A */
  ["\u3D2F", "\u3D2F"], /* CJK Ideograph Extension A */
  ["\u3D30", "\u3D30"], /* CJK Ideograph Extension A */
  ["\u3D31", "\u3D31"], /* CJK Ideograph Extension A */
  ["\u3D32", "\u3D32"], /* CJK Ideograph Extension A */
  ["\u3D33", "\u3D33"], /* CJK Ideograph Extension A */
  ["\u3D34", "\u3D34"], /* CJK Ideograph Extension A */
  ["\u3D35", "\u3D35"], /* CJK Ideograph Extension A */
  ["\u3D36", "\u3D36"], /* CJK Ideograph Extension A */
  ["\u3D37", "\u3D37"], /* CJK Ideograph Extension A */
  ["\u3D38", "\u3D38"], /* CJK Ideograph Extension A */
  ["\u3D39", "\u3D39"], /* CJK Ideograph Extension A */
  ["\u3D3A", "\u3D3A"], /* CJK Ideograph Extension A */
  ["\u3D3B", "\u3D3B"], /* CJK Ideograph Extension A */
  ["\u3D3C", "\u3D3C"], /* CJK Ideograph Extension A */
  ["\u3D3D", "\u3D3D"], /* CJK Ideograph Extension A */
  ["\u3D3E", "\u3D3E"], /* CJK Ideograph Extension A */
  ["\u3D3F", "\u3D3F"], /* CJK Ideograph Extension A */
  ["\u3D40", "\u3D40"], /* CJK Ideograph Extension A */
  ["\u3D41", "\u3D41"], /* CJK Ideograph Extension A */
  ["\u3D42", "\u3D42"], /* CJK Ideograph Extension A */
  ["\u3D43", "\u3D43"], /* CJK Ideograph Extension A */
  ["\u3D44", "\u3D44"], /* CJK Ideograph Extension A */
  ["\u3D45", "\u3D45"], /* CJK Ideograph Extension A */
  ["\u3D46", "\u3D46"], /* CJK Ideograph Extension A */
  ["\u3D47", "\u3D47"], /* CJK Ideograph Extension A */
  ["\u3D48", "\u3D48"], /* CJK Ideograph Extension A */
  ["\u3D49", "\u3D49"], /* CJK Ideograph Extension A */
  ["\u3D4A", "\u3D4A"], /* CJK Ideograph Extension A */
  ["\u3D4B", "\u3D4B"], /* CJK Ideograph Extension A */
  ["\u3D4C", "\u3D4C"], /* CJK Ideograph Extension A */
  ["\u3D4D", "\u3D4D"], /* CJK Ideograph Extension A */
  ["\u3D4E", "\u3D4E"], /* CJK Ideograph Extension A */
  ["\u3D4F", "\u3D4F"], /* CJK Ideograph Extension A */
  ["\u3D50", "\u3D50"], /* CJK Ideograph Extension A */
  ["\u3D51", "\u3D51"], /* CJK Ideograph Extension A */
  ["\u3D52", "\u3D52"], /* CJK Ideograph Extension A */
  ["\u3D53", "\u3D53"], /* CJK Ideograph Extension A */
  ["\u3D54", "\u3D54"], /* CJK Ideograph Extension A */
  ["\u3D55", "\u3D55"], /* CJK Ideograph Extension A */
  ["\u3D56", "\u3D56"], /* CJK Ideograph Extension A */
  ["\u3D57", "\u3D57"], /* CJK Ideograph Extension A */
  ["\u3D58", "\u3D58"], /* CJK Ideograph Extension A */
  ["\u3D59", "\u3D59"], /* CJK Ideograph Extension A */
  ["\u3D5A", "\u3D5A"], /* CJK Ideograph Extension A */
  ["\u3D5B", "\u3D5B"], /* CJK Ideograph Extension A */
  ["\u3D5C", "\u3D5C"], /* CJK Ideograph Extension A */
  ["\u3D5D", "\u3D5D"], /* CJK Ideograph Extension A */
  ["\u3D5E", "\u3D5E"], /* CJK Ideograph Extension A */
  ["\u3D5F", "\u3D5F"], /* CJK Ideograph Extension A */
  ["\u3D60", "\u3D60"], /* CJK Ideograph Extension A */
  ["\u3D61", "\u3D61"], /* CJK Ideograph Extension A */
  ["\u3D62", "\u3D62"], /* CJK Ideograph Extension A */
  ["\u3D63", "\u3D63"], /* CJK Ideograph Extension A */
  ["\u3D64", "\u3D64"], /* CJK Ideograph Extension A */
  ["\u3D65", "\u3D65"], /* CJK Ideograph Extension A */
  ["\u3D66", "\u3D66"], /* CJK Ideograph Extension A */
  ["\u3D67", "\u3D67"], /* CJK Ideograph Extension A */
  ["\u3D68", "\u3D68"], /* CJK Ideograph Extension A */
  ["\u3D69", "\u3D69"], /* CJK Ideograph Extension A */
  ["\u3D6A", "\u3D6A"], /* CJK Ideograph Extension A */
  ["\u3D6B", "\u3D6B"], /* CJK Ideograph Extension A */
  ["\u3D6C", "\u3D6C"], /* CJK Ideograph Extension A */
  ["\u3D6D", "\u3D6D"], /* CJK Ideograph Extension A */
  ["\u3D6E", "\u3D6E"], /* CJK Ideograph Extension A */
  ["\u3D6F", "\u3D6F"], /* CJK Ideograph Extension A */
  ["\u3D70", "\u3D70"], /* CJK Ideograph Extension A */
  ["\u3D71", "\u3D71"], /* CJK Ideograph Extension A */
  ["\u3D72", "\u3D72"], /* CJK Ideograph Extension A */
  ["\u3D73", "\u3D73"], /* CJK Ideograph Extension A */
  ["\u3D74", "\u3D74"], /* CJK Ideograph Extension A */
  ["\u3D75", "\u3D75"], /* CJK Ideograph Extension A */
  ["\u3D76", "\u3D76"], /* CJK Ideograph Extension A */
  ["\u3D77", "\u3D77"], /* CJK Ideograph Extension A */
  ["\u3D78", "\u3D78"], /* CJK Ideograph Extension A */
  ["\u3D79", "\u3D79"], /* CJK Ideograph Extension A */
  ["\u3D7A", "\u3D7A"], /* CJK Ideograph Extension A */
  ["\u3D7B", "\u3D7B"], /* CJK Ideograph Extension A */
  ["\u3D7C", "\u3D7C"], /* CJK Ideograph Extension A */
  ["\u3D7D", "\u3D7D"], /* CJK Ideograph Extension A */
  ["\u3D7E", "\u3D7E"], /* CJK Ideograph Extension A */
  ["\u3D7F", "\u3D7F"], /* CJK Ideograph Extension A */
  ["\u3D80", "\u3D80"], /* CJK Ideograph Extension A */
  ["\u3D81", "\u3D81"], /* CJK Ideograph Extension A */
  ["\u3D82", "\u3D82"], /* CJK Ideograph Extension A */
  ["\u3D83", "\u3D83"], /* CJK Ideograph Extension A */
  ["\u3D84", "\u3D84"], /* CJK Ideograph Extension A */
  ["\u3D85", "\u3D85"], /* CJK Ideograph Extension A */
  ["\u3D86", "\u3D86"], /* CJK Ideograph Extension A */
  ["\u3D87", "\u3D87"], /* CJK Ideograph Extension A */
  ["\u3D88", "\u3D88"], /* CJK Ideograph Extension A */
  ["\u3D89", "\u3D89"], /* CJK Ideograph Extension A */
  ["\u3D8A", "\u3D8A"], /* CJK Ideograph Extension A */
  ["\u3D8B", "\u3D8B"], /* CJK Ideograph Extension A */
  ["\u3D8C", "\u3D8C"], /* CJK Ideograph Extension A */
  ["\u3D8D", "\u3D8D"], /* CJK Ideograph Extension A */
  ["\u3D8E", "\u3D8E"], /* CJK Ideograph Extension A */
  ["\u3D8F", "\u3D8F"], /* CJK Ideograph Extension A */
  ["\u3D90", "\u3D90"], /* CJK Ideograph Extension A */
  ["\u3D91", "\u3D91"], /* CJK Ideograph Extension A */
  ["\u3D92", "\u3D92"], /* CJK Ideograph Extension A */
  ["\u3D93", "\u3D93"], /* CJK Ideograph Extension A */
  ["\u3D94", "\u3D94"], /* CJK Ideograph Extension A */
  ["\u3D95", "\u3D95"], /* CJK Ideograph Extension A */
  ["\u3D96", "\u3D96"], /* CJK Ideograph Extension A */
  ["\u3D97", "\u3D97"], /* CJK Ideograph Extension A */
  ["\u3D98", "\u3D98"], /* CJK Ideograph Extension A */
  ["\u3D99", "\u3D99"], /* CJK Ideograph Extension A */
  ["\u3D9A", "\u3D9A"], /* CJK Ideograph Extension A */
  ["\u3D9B", "\u3D9B"], /* CJK Ideograph Extension A */
  ["\u3D9C", "\u3D9C"], /* CJK Ideograph Extension A */
  ["\u3D9D", "\u3D9D"], /* CJK Ideograph Extension A */
  ["\u3D9E", "\u3D9E"], /* CJK Ideograph Extension A */
  ["\u3D9F", "\u3D9F"], /* CJK Ideograph Extension A */
  ["\u3DA0", "\u3DA0"], /* CJK Ideograph Extension A */
  ["\u3DA1", "\u3DA1"], /* CJK Ideograph Extension A */
  ["\u3DA2", "\u3DA2"], /* CJK Ideograph Extension A */
  ["\u3DA3", "\u3DA3"], /* CJK Ideograph Extension A */
  ["\u3DA4", "\u3DA4"], /* CJK Ideograph Extension A */
  ["\u3DA5", "\u3DA5"], /* CJK Ideograph Extension A */
  ["\u3DA6", "\u3DA6"], /* CJK Ideograph Extension A */
  ["\u3DA7", "\u3DA7"], /* CJK Ideograph Extension A */
  ["\u3DA8", "\u3DA8"], /* CJK Ideograph Extension A */
  ["\u3DA9", "\u3DA9"], /* CJK Ideograph Extension A */
  ["\u3DAA", "\u3DAA"], /* CJK Ideograph Extension A */
  ["\u3DAB", "\u3DAB"], /* CJK Ideograph Extension A */
  ["\u3DAC", "\u3DAC"], /* CJK Ideograph Extension A */
  ["\u3DAD", "\u3DAD"], /* CJK Ideograph Extension A */
  ["\u3DAE", "\u3DAE"], /* CJK Ideograph Extension A */
  ["\u3DAF", "\u3DAF"], /* CJK Ideograph Extension A */
  ["\u3DB0", "\u3DB0"], /* CJK Ideograph Extension A */
  ["\u3DB1", "\u3DB1"], /* CJK Ideograph Extension A */
  ["\u3DB2", "\u3DB2"], /* CJK Ideograph Extension A */
  ["\u3DB3", "\u3DB3"], /* CJK Ideograph Extension A */
  ["\u3DB4", "\u3DB4"], /* CJK Ideograph Extension A */
  ["\u3DB5", "\u3DB5"], /* CJK Ideograph Extension A */
  ["\u3DB6", "\u3DB6"], /* CJK Ideograph Extension A */
  ["\u3DB7", "\u3DB7"], /* CJK Ideograph Extension A */
  ["\u3DB8", "\u3DB8"], /* CJK Ideograph Extension A */
  ["\u3DB9", "\u3DB9"], /* CJK Ideograph Extension A */
  ["\u3DBA", "\u3DBA"], /* CJK Ideograph Extension A */
  ["\u3DBB", "\u3DBB"], /* CJK Ideograph Extension A */
  ["\u3DBC", "\u3DBC"], /* CJK Ideograph Extension A */
  ["\u3DBD", "\u3DBD"], /* CJK Ideograph Extension A */
  ["\u3DBE", "\u3DBE"], /* CJK Ideograph Extension A */
  ["\u3DBF", "\u3DBF"], /* CJK Ideograph Extension A */
  ["\u3DC0", "\u3DC0"], /* CJK Ideograph Extension A */
  ["\u3DC1", "\u3DC1"], /* CJK Ideograph Extension A */
  ["\u3DC2", "\u3DC2"], /* CJK Ideograph Extension A */
  ["\u3DC3", "\u3DC3"], /* CJK Ideograph Extension A */
  ["\u3DC4", "\u3DC4"], /* CJK Ideograph Extension A */
  ["\u3DC5", "\u3DC5"], /* CJK Ideograph Extension A */
  ["\u3DC6", "\u3DC6"], /* CJK Ideograph Extension A */
  ["\u3DC7", "\u3DC7"], /* CJK Ideograph Extension A */
  ["\u3DC8", "\u3DC8"], /* CJK Ideograph Extension A */
  ["\u3DC9", "\u3DC9"], /* CJK Ideograph Extension A */
  ["\u3DCA", "\u3DCA"], /* CJK Ideograph Extension A */
  ["\u3DCB", "\u3DCB"], /* CJK Ideograph Extension A */
  ["\u3DCC", "\u3DCC"], /* CJK Ideograph Extension A */
  ["\u3DCD", "\u3DCD"], /* CJK Ideograph Extension A */
  ["\u3DCE", "\u3DCE"], /* CJK Ideograph Extension A */
  ["\u3DCF", "\u3DCF"], /* CJK Ideograph Extension A */
  ["\u3DD0", "\u3DD0"], /* CJK Ideograph Extension A */
  ["\u3DD1", "\u3DD1"], /* CJK Ideograph Extension A */
  ["\u3DD2", "\u3DD2"], /* CJK Ideograph Extension A */
  ["\u3DD3", "\u3DD3"], /* CJK Ideograph Extension A */
  ["\u3DD4", "\u3DD4"], /* CJK Ideograph Extension A */
  ["\u3DD5", "\u3DD5"], /* CJK Ideograph Extension A */
  ["\u3DD6", "\u3DD6"], /* CJK Ideograph Extension A */
  ["\u3DD7", "\u3DD7"], /* CJK Ideograph Extension A */
  ["\u3DD8", "\u3DD8"], /* CJK Ideograph Extension A */
  ["\u3DD9", "\u3DD9"], /* CJK Ideograph Extension A */
  ["\u3DDA", "\u3DDA"], /* CJK Ideograph Extension A */
  ["\u3DDB", "\u3DDB"], /* CJK Ideograph Extension A */
  ["\u3DDC", "\u3DDC"], /* CJK Ideograph Extension A */
  ["\u3DDD", "\u3DDD"], /* CJK Ideograph Extension A */
  ["\u3DDE", "\u3DDE"], /* CJK Ideograph Extension A */
  ["\u3DDF", "\u3DDF"], /* CJK Ideograph Extension A */
  ["\u3DE0", "\u3DE0"], /* CJK Ideograph Extension A */
  ["\u3DE1", "\u3DE1"], /* CJK Ideograph Extension A */
  ["\u3DE2", "\u3DE2"], /* CJK Ideograph Extension A */
  ["\u3DE3", "\u3DE3"], /* CJK Ideograph Extension A */
  ["\u3DE4", "\u3DE4"], /* CJK Ideograph Extension A */
  ["\u3DE5", "\u3DE5"], /* CJK Ideograph Extension A */
  ["\u3DE6", "\u3DE6"], /* CJK Ideograph Extension A */
  ["\u3DE7", "\u3DE7"], /* CJK Ideograph Extension A */
  ["\u3DE8", "\u3DE8"], /* CJK Ideograph Extension A */
  ["\u3DE9", "\u3DE9"], /* CJK Ideograph Extension A */
  ["\u3DEA", "\u3DEA"], /* CJK Ideograph Extension A */
  ["\u3DEB", "\u3DEB"], /* CJK Ideograph Extension A */
  ["\u3DEC", "\u3DEC"], /* CJK Ideograph Extension A */
  ["\u3DED", "\u3DED"], /* CJK Ideograph Extension A */
  ["\u3DEE", "\u3DEE"], /* CJK Ideograph Extension A */
  ["\u3DEF", "\u3DEF"], /* CJK Ideograph Extension A */
  ["\u3DF0", "\u3DF0"], /* CJK Ideograph Extension A */
  ["\u3DF1", "\u3DF1"], /* CJK Ideograph Extension A */
  ["\u3DF2", "\u3DF2"], /* CJK Ideograph Extension A */
  ["\u3DF3", "\u3DF3"], /* CJK Ideograph Extension A */
  ["\u3DF4", "\u3DF4"], /* CJK Ideograph Extension A */
  ["\u3DF5", "\u3DF5"], /* CJK Ideograph Extension A */
  ["\u3DF6", "\u3DF6"], /* CJK Ideograph Extension A */
  ["\u3DF7", "\u3DF7"], /* CJK Ideograph Extension A */
  ["\u3DF8", "\u3DF8"], /* CJK Ideograph Extension A */
  ["\u3DF9", "\u3DF9"], /* CJK Ideograph Extension A */
  ["\u3DFA", "\u3DFA"], /* CJK Ideograph Extension A */
  ["\u3DFB", "\u3DFB"], /* CJK Ideograph Extension A */
  ["\u3DFC", "\u3DFC"], /* CJK Ideograph Extension A */
  ["\u3DFD", "\u3DFD"], /* CJK Ideograph Extension A */
  ["\u3DFE", "\u3DFE"], /* CJK Ideograph Extension A */
  ["\u3DFF", "\u3DFF"], /* CJK Ideograph Extension A */
  ["\u3E00", "\u3E00"], /* CJK Ideograph Extension A */
  ["\u3E01", "\u3E01"], /* CJK Ideograph Extension A */
  ["\u3E02", "\u3E02"], /* CJK Ideograph Extension A */
  ["\u3E03", "\u3E03"], /* CJK Ideograph Extension A */
  ["\u3E04", "\u3E04"], /* CJK Ideograph Extension A */
  ["\u3E05", "\u3E05"], /* CJK Ideograph Extension A */
  ["\u3E06", "\u3E06"], /* CJK Ideograph Extension A */
  ["\u3E07", "\u3E07"], /* CJK Ideograph Extension A */
  ["\u3E08", "\u3E08"], /* CJK Ideograph Extension A */
  ["\u3E09", "\u3E09"], /* CJK Ideograph Extension A */
  ["\u3E0A", "\u3E0A"], /* CJK Ideograph Extension A */
  ["\u3E0B", "\u3E0B"], /* CJK Ideograph Extension A */
  ["\u3E0C", "\u3E0C"], /* CJK Ideograph Extension A */
  ["\u3E0D", "\u3E0D"], /* CJK Ideograph Extension A */
  ["\u3E0E", "\u3E0E"], /* CJK Ideograph Extension A */
  ["\u3E0F", "\u3E0F"], /* CJK Ideograph Extension A */
  ["\u3E10", "\u3E10"], /* CJK Ideograph Extension A */
  ["\u3E11", "\u3E11"], /* CJK Ideograph Extension A */
  ["\u3E12", "\u3E12"], /* CJK Ideograph Extension A */
  ["\u3E13", "\u3E13"], /* CJK Ideograph Extension A */
  ["\u3E14", "\u3E14"], /* CJK Ideograph Extension A */
  ["\u3E15", "\u3E15"], /* CJK Ideograph Extension A */
  ["\u3E16", "\u3E16"], /* CJK Ideograph Extension A */
  ["\u3E17", "\u3E17"], /* CJK Ideograph Extension A */
  ["\u3E18", "\u3E18"], /* CJK Ideograph Extension A */
  ["\u3E19", "\u3E19"], /* CJK Ideograph Extension A */
  ["\u3E1A", "\u3E1A"], /* CJK Ideograph Extension A */
  ["\u3E1B", "\u3E1B"], /* CJK Ideograph Extension A */
  ["\u3E1C", "\u3E1C"], /* CJK Ideograph Extension A */
  ["\u3E1D", "\u3E1D"], /* CJK Ideograph Extension A */
  ["\u3E1E", "\u3E1E"], /* CJK Ideograph Extension A */
  ["\u3E1F", "\u3E1F"], /* CJK Ideograph Extension A */
  ["\u3E20", "\u3E20"], /* CJK Ideograph Extension A */
  ["\u3E21", "\u3E21"], /* CJK Ideograph Extension A */
  ["\u3E22", "\u3E22"], /* CJK Ideograph Extension A */
  ["\u3E23", "\u3E23"], /* CJK Ideograph Extension A */
  ["\u3E24", "\u3E24"], /* CJK Ideograph Extension A */
  ["\u3E25", "\u3E25"], /* CJK Ideograph Extension A */
  ["\u3E26", "\u3E26"], /* CJK Ideograph Extension A */
  ["\u3E27", "\u3E27"], /* CJK Ideograph Extension A */
  ["\u3E28", "\u3E28"], /* CJK Ideograph Extension A */
  ["\u3E29", "\u3E29"], /* CJK Ideograph Extension A */
  ["\u3E2A", "\u3E2A"], /* CJK Ideograph Extension A */
  ["\u3E2B", "\u3E2B"], /* CJK Ideograph Extension A */
  ["\u3E2C", "\u3E2C"], /* CJK Ideograph Extension A */
  ["\u3E2D", "\u3E2D"], /* CJK Ideograph Extension A */
  ["\u3E2E", "\u3E2E"], /* CJK Ideograph Extension A */
  ["\u3E2F", "\u3E2F"], /* CJK Ideograph Extension A */
  ["\u3E30", "\u3E30"], /* CJK Ideograph Extension A */
  ["\u3E31", "\u3E31"], /* CJK Ideograph Extension A */
  ["\u3E32", "\u3E32"], /* CJK Ideograph Extension A */
  ["\u3E33", "\u3E33"], /* CJK Ideograph Extension A */
  ["\u3E34", "\u3E34"], /* CJK Ideograph Extension A */
  ["\u3E35", "\u3E35"], /* CJK Ideograph Extension A */
  ["\u3E36", "\u3E36"], /* CJK Ideograph Extension A */
  ["\u3E37", "\u3E37"], /* CJK Ideograph Extension A */
  ["\u3E38", "\u3E38"], /* CJK Ideograph Extension A */
  ["\u3E39", "\u3E39"], /* CJK Ideograph Extension A */
  ["\u3E3A", "\u3E3A"], /* CJK Ideograph Extension A */
  ["\u3E3B", "\u3E3B"], /* CJK Ideograph Extension A */
  ["\u3E3C", "\u3E3C"], /* CJK Ideograph Extension A */
  ["\u3E3D", "\u3E3D"], /* CJK Ideograph Extension A */
  ["\u3E3E", "\u3E3E"], /* CJK Ideograph Extension A */
  ["\u3E3F", "\u3E3F"], /* CJK Ideograph Extension A */
  ["\u3E40", "\u3E40"], /* CJK Ideograph Extension A */
  ["\u3E41", "\u3E41"], /* CJK Ideograph Extension A */
  ["\u3E42", "\u3E42"], /* CJK Ideograph Extension A */
  ["\u3E43", "\u3E43"], /* CJK Ideograph Extension A */
  ["\u3E44", "\u3E44"], /* CJK Ideograph Extension A */
  ["\u3E45", "\u3E45"], /* CJK Ideograph Extension A */
  ["\u3E46", "\u3E46"], /* CJK Ideograph Extension A */
  ["\u3E47", "\u3E47"], /* CJK Ideograph Extension A */
  ["\u3E48", "\u3E48"], /* CJK Ideograph Extension A */
  ["\u3E49", "\u3E49"], /* CJK Ideograph Extension A */
  ["\u3E4A", "\u3E4A"], /* CJK Ideograph Extension A */
  ["\u3E4B", "\u3E4B"], /* CJK Ideograph Extension A */
  ["\u3E4C", "\u3E4C"], /* CJK Ideograph Extension A */
  ["\u3E4D", "\u3E4D"], /* CJK Ideograph Extension A */
  ["\u3E4E", "\u3E4E"], /* CJK Ideograph Extension A */
  ["\u3E4F", "\u3E4F"], /* CJK Ideograph Extension A */
  ["\u3E50", "\u3E50"], /* CJK Ideograph Extension A */
  ["\u3E51", "\u3E51"], /* CJK Ideograph Extension A */
  ["\u3E52", "\u3E52"], /* CJK Ideograph Extension A */
  ["\u3E53", "\u3E53"], /* CJK Ideograph Extension A */
  ["\u3E54", "\u3E54"], /* CJK Ideograph Extension A */
  ["\u3E55", "\u3E55"], /* CJK Ideograph Extension A */
  ["\u3E56", "\u3E56"], /* CJK Ideograph Extension A */
  ["\u3E57", "\u3E57"], /* CJK Ideograph Extension A */
  ["\u3E58", "\u3E58"], /* CJK Ideograph Extension A */
  ["\u3E59", "\u3E59"], /* CJK Ideograph Extension A */
  ["\u3E5A", "\u3E5A"], /* CJK Ideograph Extension A */
  ["\u3E5B", "\u3E5B"], /* CJK Ideograph Extension A */
  ["\u3E5C", "\u3E5C"], /* CJK Ideograph Extension A */
  ["\u3E5D", "\u3E5D"], /* CJK Ideograph Extension A */
  ["\u3E5E", "\u3E5E"], /* CJK Ideograph Extension A */
  ["\u3E5F", "\u3E5F"], /* CJK Ideograph Extension A */
  ["\u3E60", "\u3E60"], /* CJK Ideograph Extension A */
  ["\u3E61", "\u3E61"], /* CJK Ideograph Extension A */
  ["\u3E62", "\u3E62"], /* CJK Ideograph Extension A */
  ["\u3E63", "\u3E63"], /* CJK Ideograph Extension A */
  ["\u3E64", "\u3E64"], /* CJK Ideograph Extension A */
  ["\u3E65", "\u3E65"], /* CJK Ideograph Extension A */
  ["\u3E66", "\u3E66"], /* CJK Ideograph Extension A */
  ["\u3E67", "\u3E67"], /* CJK Ideograph Extension A */
  ["\u3E68", "\u3E68"], /* CJK Ideograph Extension A */
  ["\u3E69", "\u3E69"], /* CJK Ideograph Extension A */
  ["\u3E6A", "\u3E6A"], /* CJK Ideograph Extension A */
  ["\u3E6B", "\u3E6B"], /* CJK Ideograph Extension A */
  ["\u3E6C", "\u3E6C"], /* CJK Ideograph Extension A */
  ["\u3E6D", "\u3E6D"], /* CJK Ideograph Extension A */
  ["\u3E6E", "\u3E6E"], /* CJK Ideograph Extension A */
  ["\u3E6F", "\u3E6F"], /* CJK Ideograph Extension A */
  ["\u3E70", "\u3E70"], /* CJK Ideograph Extension A */
  ["\u3E71", "\u3E71"], /* CJK Ideograph Extension A */
  ["\u3E72", "\u3E72"], /* CJK Ideograph Extension A */
  ["\u3E73", "\u3E73"], /* CJK Ideograph Extension A */
  ["\u3E74", "\u3E74"], /* CJK Ideograph Extension A */
  ["\u3E75", "\u3E75"], /* CJK Ideograph Extension A */
  ["\u3E76", "\u3E76"], /* CJK Ideograph Extension A */
  ["\u3E77", "\u3E77"], /* CJK Ideograph Extension A */
  ["\u3E78", "\u3E78"], /* CJK Ideograph Extension A */
  ["\u3E79", "\u3E79"], /* CJK Ideograph Extension A */
  ["\u3E7A", "\u3E7A"], /* CJK Ideograph Extension A */
  ["\u3E7B", "\u3E7B"], /* CJK Ideograph Extension A */
  ["\u3E7C", "\u3E7C"], /* CJK Ideograph Extension A */
  ["\u3E7D", "\u3E7D"], /* CJK Ideograph Extension A */
  ["\u3E7E", "\u3E7E"], /* CJK Ideograph Extension A */
  ["\u3E7F", "\u3E7F"], /* CJK Ideograph Extension A */
  ["\u3E80", "\u3E80"], /* CJK Ideograph Extension A */
  ["\u3E81", "\u3E81"], /* CJK Ideograph Extension A */
  ["\u3E82", "\u3E82"], /* CJK Ideograph Extension A */
  ["\u3E83", "\u3E83"], /* CJK Ideograph Extension A */
  ["\u3E84", "\u3E84"], /* CJK Ideograph Extension A */
  ["\u3E85", "\u3E85"], /* CJK Ideograph Extension A */
  ["\u3E86", "\u3E86"], /* CJK Ideograph Extension A */
  ["\u3E87", "\u3E87"], /* CJK Ideograph Extension A */
  ["\u3E88", "\u3E88"], /* CJK Ideograph Extension A */
  ["\u3E89", "\u3E89"], /* CJK Ideograph Extension A */
  ["\u3E8A", "\u3E8A"], /* CJK Ideograph Extension A */
  ["\u3E8B", "\u3E8B"], /* CJK Ideograph Extension A */
  ["\u3E8C", "\u3E8C"], /* CJK Ideograph Extension A */
  ["\u3E8D", "\u3E8D"], /* CJK Ideograph Extension A */
  ["\u3E8E", "\u3E8E"], /* CJK Ideograph Extension A */
  ["\u3E8F", "\u3E8F"], /* CJK Ideograph Extension A */
  ["\u3E90", "\u3E90"], /* CJK Ideograph Extension A */
  ["\u3E91", "\u3E91"], /* CJK Ideograph Extension A */
  ["\u3E92", "\u3E92"], /* CJK Ideograph Extension A */
  ["\u3E93", "\u3E93"], /* CJK Ideograph Extension A */
  ["\u3E94", "\u3E94"], /* CJK Ideograph Extension A */
  ["\u3E95", "\u3E95"], /* CJK Ideograph Extension A */
  ["\u3E96", "\u3E96"], /* CJK Ideograph Extension A */
  ["\u3E97", "\u3E97"], /* CJK Ideograph Extension A */
  ["\u3E98", "\u3E98"], /* CJK Ideograph Extension A */
  ["\u3E99", "\u3E99"], /* CJK Ideograph Extension A */
  ["\u3E9A", "\u3E9A"], /* CJK Ideograph Extension A */
  ["\u3E9B", "\u3E9B"], /* CJK Ideograph Extension A */
  ["\u3E9C", "\u3E9C"], /* CJK Ideograph Extension A */
  ["\u3E9D", "\u3E9D"], /* CJK Ideograph Extension A */
  ["\u3E9E", "\u3E9E"], /* CJK Ideograph Extension A */
  ["\u3E9F", "\u3E9F"], /* CJK Ideograph Extension A */
  ["\u3EA0", "\u3EA0"], /* CJK Ideograph Extension A */
  ["\u3EA1", "\u3EA1"], /* CJK Ideograph Extension A */
  ["\u3EA2", "\u3EA2"], /* CJK Ideograph Extension A */
  ["\u3EA3", "\u3EA3"], /* CJK Ideograph Extension A */
  ["\u3EA4", "\u3EA4"], /* CJK Ideograph Extension A */
  ["\u3EA5", "\u3EA5"], /* CJK Ideograph Extension A */
  ["\u3EA6", "\u3EA6"], /* CJK Ideograph Extension A */
  ["\u3EA7", "\u3EA7"], /* CJK Ideograph Extension A */
  ["\u3EA8", "\u3EA8"], /* CJK Ideograph Extension A */
  ["\u3EA9", "\u3EA9"], /* CJK Ideograph Extension A */
  ["\u3EAA", "\u3EAA"], /* CJK Ideograph Extension A */
  ["\u3EAB", "\u3EAB"], /* CJK Ideograph Extension A */
  ["\u3EAC", "\u3EAC"], /* CJK Ideograph Extension A */
  ["\u3EAD", "\u3EAD"], /* CJK Ideograph Extension A */
  ["\u3EAE", "\u3EAE"], /* CJK Ideograph Extension A */
  ["\u3EAF", "\u3EAF"], /* CJK Ideograph Extension A */
  ["\u3EB0", "\u3EB0"], /* CJK Ideograph Extension A */
  ["\u3EB1", "\u3EB1"], /* CJK Ideograph Extension A */
  ["\u3EB2", "\u3EB2"], /* CJK Ideograph Extension A */
  ["\u3EB3", "\u3EB3"], /* CJK Ideograph Extension A */
  ["\u3EB4", "\u3EB4"], /* CJK Ideograph Extension A */
  ["\u3EB5", "\u3EB5"], /* CJK Ideograph Extension A */
  ["\u3EB6", "\u3EB6"], /* CJK Ideograph Extension A */
  ["\u3EB7", "\u3EB7"], /* CJK Ideograph Extension A */
  ["\u3EB8", "\u3EB8"], /* CJK Ideograph Extension A */
  ["\u3EB9", "\u3EB9"], /* CJK Ideograph Extension A */
  ["\u3EBA", "\u3EBA"], /* CJK Ideograph Extension A */
  ["\u3EBB", "\u3EBB"], /* CJK Ideograph Extension A */
  ["\u3EBC", "\u3EBC"], /* CJK Ideograph Extension A */
  ["\u3EBD", "\u3EBD"], /* CJK Ideograph Extension A */
  ["\u3EBE", "\u3EBE"], /* CJK Ideograph Extension A */
  ["\u3EBF", "\u3EBF"], /* CJK Ideograph Extension A */
  ["\u3EC0", "\u3EC0"], /* CJK Ideograph Extension A */
  ["\u3EC1", "\u3EC1"], /* CJK Ideograph Extension A */
  ["\u3EC2", "\u3EC2"], /* CJK Ideograph Extension A */
  ["\u3EC3", "\u3EC3"], /* CJK Ideograph Extension A */
  ["\u3EC4", "\u3EC4"], /* CJK Ideograph Extension A */
  ["\u3EC5", "\u3EC5"], /* CJK Ideograph Extension A */
  ["\u3EC6", "\u3EC6"], /* CJK Ideograph Extension A */
  ["\u3EC7", "\u3EC7"], /* CJK Ideograph Extension A */
  ["\u3EC8", "\u3EC8"], /* CJK Ideograph Extension A */
  ["\u3EC9", "\u3EC9"], /* CJK Ideograph Extension A */
  ["\u3ECA", "\u3ECA"], /* CJK Ideograph Extension A */
  ["\u3ECB", "\u3ECB"], /* CJK Ideograph Extension A */
  ["\u3ECC", "\u3ECC"], /* CJK Ideograph Extension A */
  ["\u3ECD", "\u3ECD"], /* CJK Ideograph Extension A */
  ["\u3ECE", "\u3ECE"], /* CJK Ideograph Extension A */
  ["\u3ECF", "\u3ECF"], /* CJK Ideograph Extension A */
  ["\u3ED0", "\u3ED0"], /* CJK Ideograph Extension A */
  ["\u3ED1", "\u3ED1"], /* CJK Ideograph Extension A */
  ["\u3ED2", "\u3ED2"], /* CJK Ideograph Extension A */
  ["\u3ED3", "\u3ED3"], /* CJK Ideograph Extension A */
  ["\u3ED4", "\u3ED4"], /* CJK Ideograph Extension A */
  ["\u3ED5", "\u3ED5"], /* CJK Ideograph Extension A */
  ["\u3ED6", "\u3ED6"], /* CJK Ideograph Extension A */
  ["\u3ED7", "\u3ED7"], /* CJK Ideograph Extension A */
  ["\u3ED8", "\u3ED8"], /* CJK Ideograph Extension A */
  ["\u3ED9", "\u3ED9"], /* CJK Ideograph Extension A */
  ["\u3EDA", "\u3EDA"], /* CJK Ideograph Extension A */
  ["\u3EDB", "\u3EDB"], /* CJK Ideograph Extension A */
  ["\u3EDC", "\u3EDC"], /* CJK Ideograph Extension A */
  ["\u3EDD", "\u3EDD"], /* CJK Ideograph Extension A */
  ["\u3EDE", "\u3EDE"], /* CJK Ideograph Extension A */
  ["\u3EDF", "\u3EDF"], /* CJK Ideograph Extension A */
  ["\u3EE0", "\u3EE0"], /* CJK Ideograph Extension A */
  ["\u3EE1", "\u3EE1"], /* CJK Ideograph Extension A */
  ["\u3EE2", "\u3EE2"], /* CJK Ideograph Extension A */
  ["\u3EE3", "\u3EE3"], /* CJK Ideograph Extension A */
  ["\u3EE4", "\u3EE4"], /* CJK Ideograph Extension A */
  ["\u3EE5", "\u3EE5"], /* CJK Ideograph Extension A */
  ["\u3EE6", "\u3EE6"], /* CJK Ideograph Extension A */
  ["\u3EE7", "\u3EE7"], /* CJK Ideograph Extension A */
  ["\u3EE8", "\u3EE8"], /* CJK Ideograph Extension A */
  ["\u3EE9", "\u3EE9"], /* CJK Ideograph Extension A */
  ["\u3EEA", "\u3EEA"], /* CJK Ideograph Extension A */
  ["\u3EEB", "\u3EEB"], /* CJK Ideograph Extension A */
  ["\u3EEC", "\u3EEC"], /* CJK Ideograph Extension A */
  ["\u3EED", "\u3EED"], /* CJK Ideograph Extension A */
  ["\u3EEE", "\u3EEE"], /* CJK Ideograph Extension A */
  ["\u3EEF", "\u3EEF"], /* CJK Ideograph Extension A */
  ["\u3EF0", "\u3EF0"], /* CJK Ideograph Extension A */
  ["\u3EF1", "\u3EF1"], /* CJK Ideograph Extension A */
  ["\u3EF2", "\u3EF2"], /* CJK Ideograph Extension A */
  ["\u3EF3", "\u3EF3"], /* CJK Ideograph Extension A */
  ["\u3EF4", "\u3EF4"], /* CJK Ideograph Extension A */
  ["\u3EF5", "\u3EF5"], /* CJK Ideograph Extension A */
  ["\u3EF6", "\u3EF6"], /* CJK Ideograph Extension A */
  ["\u3EF7", "\u3EF7"], /* CJK Ideograph Extension A */
  ["\u3EF8", "\u3EF8"], /* CJK Ideograph Extension A */
  ["\u3EF9", "\u3EF9"], /* CJK Ideograph Extension A */
  ["\u3EFA", "\u3EFA"], /* CJK Ideograph Extension A */
  ["\u3EFB", "\u3EFB"], /* CJK Ideograph Extension A */
  ["\u3EFC", "\u3EFC"], /* CJK Ideograph Extension A */
  ["\u3EFD", "\u3EFD"], /* CJK Ideograph Extension A */
  ["\u3EFE", "\u3EFE"], /* CJK Ideograph Extension A */
  ["\u3EFF", "\u3EFF"], /* CJK Ideograph Extension A */
  ["\u3F00", "\u3F00"], /* CJK Ideograph Extension A */
  ["\u3F01", "\u3F01"], /* CJK Ideograph Extension A */
  ["\u3F02", "\u3F02"], /* CJK Ideograph Extension A */
  ["\u3F03", "\u3F03"], /* CJK Ideograph Extension A */
  ["\u3F04", "\u3F04"], /* CJK Ideograph Extension A */
  ["\u3F05", "\u3F05"], /* CJK Ideograph Extension A */
  ["\u3F06", "\u3F06"], /* CJK Ideograph Extension A */
  ["\u3F07", "\u3F07"], /* CJK Ideograph Extension A */
  ["\u3F08", "\u3F08"], /* CJK Ideograph Extension A */
  ["\u3F09", "\u3F09"], /* CJK Ideograph Extension A */
  ["\u3F0A", "\u3F0A"], /* CJK Ideograph Extension A */
  ["\u3F0B", "\u3F0B"], /* CJK Ideograph Extension A */
  ["\u3F0C", "\u3F0C"], /* CJK Ideograph Extension A */
  ["\u3F0D", "\u3F0D"], /* CJK Ideograph Extension A */
  ["\u3F0E", "\u3F0E"], /* CJK Ideograph Extension A */
  ["\u3F0F", "\u3F0F"], /* CJK Ideograph Extension A */
  ["\u3F10", "\u3F10"], /* CJK Ideograph Extension A */
  ["\u3F11", "\u3F11"], /* CJK Ideograph Extension A */
  ["\u3F12", "\u3F12"], /* CJK Ideograph Extension A */
  ["\u3F13", "\u3F13"], /* CJK Ideograph Extension A */
  ["\u3F14", "\u3F14"], /* CJK Ideograph Extension A */
  ["\u3F15", "\u3F15"], /* CJK Ideograph Extension A */
  ["\u3F16", "\u3F16"], /* CJK Ideograph Extension A */
  ["\u3F17", "\u3F17"], /* CJK Ideograph Extension A */
  ["\u3F18", "\u3F18"], /* CJK Ideograph Extension A */
  ["\u3F19", "\u3F19"], /* CJK Ideograph Extension A */
  ["\u3F1A", "\u3F1A"], /* CJK Ideograph Extension A */
  ["\u3F1B", "\u3F1B"], /* CJK Ideograph Extension A */
  ["\u3F1C", "\u3F1C"], /* CJK Ideograph Extension A */
  ["\u3F1D", "\u3F1D"], /* CJK Ideograph Extension A */
  ["\u3F1E", "\u3F1E"], /* CJK Ideograph Extension A */
  ["\u3F1F", "\u3F1F"], /* CJK Ideograph Extension A */
  ["\u3F20", "\u3F20"], /* CJK Ideograph Extension A */
  ["\u3F21", "\u3F21"], /* CJK Ideograph Extension A */
  ["\u3F22", "\u3F22"], /* CJK Ideograph Extension A */
  ["\u3F23", "\u3F23"], /* CJK Ideograph Extension A */
  ["\u3F24", "\u3F24"], /* CJK Ideograph Extension A */
  ["\u3F25", "\u3F25"], /* CJK Ideograph Extension A */
  ["\u3F26", "\u3F26"], /* CJK Ideograph Extension A */
  ["\u3F27", "\u3F27"], /* CJK Ideograph Extension A */
  ["\u3F28", "\u3F28"], /* CJK Ideograph Extension A */
  ["\u3F29", "\u3F29"], /* CJK Ideograph Extension A */
  ["\u3F2A", "\u3F2A"], /* CJK Ideograph Extension A */
  ["\u3F2B", "\u3F2B"], /* CJK Ideograph Extension A */
  ["\u3F2C", "\u3F2C"], /* CJK Ideograph Extension A */
  ["\u3F2D", "\u3F2D"], /* CJK Ideograph Extension A */
  ["\u3F2E", "\u3F2E"], /* CJK Ideograph Extension A */
  ["\u3F2F", "\u3F2F"], /* CJK Ideograph Extension A */
  ["\u3F30", "\u3F30"], /* CJK Ideograph Extension A */
  ["\u3F31", "\u3F31"], /* CJK Ideograph Extension A */
  ["\u3F32", "\u3F32"], /* CJK Ideograph Extension A */
  ["\u3F33", "\u3F33"], /* CJK Ideograph Extension A */
  ["\u3F34", "\u3F34"], /* CJK Ideograph Extension A */
  ["\u3F35", "\u3F35"], /* CJK Ideograph Extension A */
  ["\u3F36", "\u3F36"], /* CJK Ideograph Extension A */
  ["\u3F37", "\u3F37"], /* CJK Ideograph Extension A */
  ["\u3F38", "\u3F38"], /* CJK Ideograph Extension A */
  ["\u3F39", "\u3F39"], /* CJK Ideograph Extension A */
  ["\u3F3A", "\u3F3A"], /* CJK Ideograph Extension A */
  ["\u3F3B", "\u3F3B"], /* CJK Ideograph Extension A */
  ["\u3F3C", "\u3F3C"], /* CJK Ideograph Extension A */
  ["\u3F3D", "\u3F3D"], /* CJK Ideograph Extension A */
  ["\u3F3E", "\u3F3E"], /* CJK Ideograph Extension A */
  ["\u3F3F", "\u3F3F"], /* CJK Ideograph Extension A */
  ["\u3F40", "\u3F40"], /* CJK Ideograph Extension A */
  ["\u3F41", "\u3F41"], /* CJK Ideograph Extension A */
  ["\u3F42", "\u3F42"], /* CJK Ideograph Extension A */
  ["\u3F43", "\u3F43"], /* CJK Ideograph Extension A */
  ["\u3F44", "\u3F44"], /* CJK Ideograph Extension A */
  ["\u3F45", "\u3F45"], /* CJK Ideograph Extension A */
  ["\u3F46", "\u3F46"], /* CJK Ideograph Extension A */
  ["\u3F47", "\u3F47"], /* CJK Ideograph Extension A */
  ["\u3F48", "\u3F48"], /* CJK Ideograph Extension A */
  ["\u3F49", "\u3F49"], /* CJK Ideograph Extension A */
  ["\u3F4A", "\u3F4A"], /* CJK Ideograph Extension A */
  ["\u3F4B", "\u3F4B"], /* CJK Ideograph Extension A */
  ["\u3F4C", "\u3F4C"], /* CJK Ideograph Extension A */
  ["\u3F4D", "\u3F4D"], /* CJK Ideograph Extension A */
  ["\u3F4E", "\u3F4E"], /* CJK Ideograph Extension A */
  ["\u3F4F", "\u3F4F"], /* CJK Ideograph Extension A */
  ["\u3F50", "\u3F50"], /* CJK Ideograph Extension A */
  ["\u3F51", "\u3F51"], /* CJK Ideograph Extension A */
  ["\u3F52", "\u3F52"], /* CJK Ideograph Extension A */
  ["\u3F53", "\u3F53"], /* CJK Ideograph Extension A */
  ["\u3F54", "\u3F54"], /* CJK Ideograph Extension A */
  ["\u3F55", "\u3F55"], /* CJK Ideograph Extension A */
  ["\u3F56", "\u3F56"], /* CJK Ideograph Extension A */
  ["\u3F57", "\u3F57"], /* CJK Ideograph Extension A */
  ["\u3F58", "\u3F58"], /* CJK Ideograph Extension A */
  ["\u3F59", "\u3F59"], /* CJK Ideograph Extension A */
  ["\u3F5A", "\u3F5A"], /* CJK Ideograph Extension A */
  ["\u3F5B", "\u3F5B"], /* CJK Ideograph Extension A */
  ["\u3F5C", "\u3F5C"], /* CJK Ideograph Extension A */
  ["\u3F5D", "\u3F5D"], /* CJK Ideograph Extension A */
  ["\u3F5E", "\u3F5E"], /* CJK Ideograph Extension A */
  ["\u3F5F", "\u3F5F"], /* CJK Ideograph Extension A */
  ["\u3F60", "\u3F60"], /* CJK Ideograph Extension A */
  ["\u3F61", "\u3F61"], /* CJK Ideograph Extension A */
  ["\u3F62", "\u3F62"], /* CJK Ideograph Extension A */
  ["\u3F63", "\u3F63"], /* CJK Ideograph Extension A */
  ["\u3F64", "\u3F64"], /* CJK Ideograph Extension A */
  ["\u3F65", "\u3F65"], /* CJK Ideograph Extension A */
  ["\u3F66", "\u3F66"], /* CJK Ideograph Extension A */
  ["\u3F67", "\u3F67"], /* CJK Ideograph Extension A */
  ["\u3F68", "\u3F68"], /* CJK Ideograph Extension A */
  ["\u3F69", "\u3F69"], /* CJK Ideograph Extension A */
  ["\u3F6A", "\u3F6A"], /* CJK Ideograph Extension A */
  ["\u3F6B", "\u3F6B"], /* CJK Ideograph Extension A */
  ["\u3F6C", "\u3F6C"], /* CJK Ideograph Extension A */
  ["\u3F6D", "\u3F6D"], /* CJK Ideograph Extension A */
  ["\u3F6E", "\u3F6E"], /* CJK Ideograph Extension A */
  ["\u3F6F", "\u3F6F"], /* CJK Ideograph Extension A */
  ["\u3F70", "\u3F70"], /* CJK Ideograph Extension A */
  ["\u3F71", "\u3F71"], /* CJK Ideograph Extension A */
  ["\u3F72", "\u3F72"], /* CJK Ideograph Extension A */
  ["\u3F73", "\u3F73"], /* CJK Ideograph Extension A */
  ["\u3F74", "\u3F74"], /* CJK Ideograph Extension A */
  ["\u3F75", "\u3F75"], /* CJK Ideograph Extension A */
  ["\u3F76", "\u3F76"], /* CJK Ideograph Extension A */
  ["\u3F77", "\u3F77"], /* CJK Ideograph Extension A */
  ["\u3F78", "\u3F78"], /* CJK Ideograph Extension A */
  ["\u3F79", "\u3F79"], /* CJK Ideograph Extension A */
  ["\u3F7A", "\u3F7A"], /* CJK Ideograph Extension A */
  ["\u3F7B", "\u3F7B"], /* CJK Ideograph Extension A */
  ["\u3F7C", "\u3F7C"], /* CJK Ideograph Extension A */
  ["\u3F7D", "\u3F7D"], /* CJK Ideograph Extension A */
  ["\u3F7E", "\u3F7E"], /* CJK Ideograph Extension A */
  ["\u3F7F", "\u3F7F"], /* CJK Ideograph Extension A */
  ["\u3F80", "\u3F80"], /* CJK Ideograph Extension A */
  ["\u3F81", "\u3F81"], /* CJK Ideograph Extension A */
  ["\u3F82", "\u3F82"], /* CJK Ideograph Extension A */
  ["\u3F83", "\u3F83"], /* CJK Ideograph Extension A */
  ["\u3F84", "\u3F84"], /* CJK Ideograph Extension A */
  ["\u3F85", "\u3F85"], /* CJK Ideograph Extension A */
  ["\u3F86", "\u3F86"], /* CJK Ideograph Extension A */
  ["\u3F87", "\u3F87"], /* CJK Ideograph Extension A */
  ["\u3F88", "\u3F88"], /* CJK Ideograph Extension A */
  ["\u3F89", "\u3F89"], /* CJK Ideograph Extension A */
  ["\u3F8A", "\u3F8A"], /* CJK Ideograph Extension A */
  ["\u3F8B", "\u3F8B"], /* CJK Ideograph Extension A */
  ["\u3F8C", "\u3F8C"], /* CJK Ideograph Extension A */
  ["\u3F8D", "\u3F8D"], /* CJK Ideograph Extension A */
  ["\u3F8E", "\u3F8E"], /* CJK Ideograph Extension A */
  ["\u3F8F", "\u3F8F"], /* CJK Ideograph Extension A */
  ["\u3F90", "\u3F90"], /* CJK Ideograph Extension A */
  ["\u3F91", "\u3F91"], /* CJK Ideograph Extension A */
  ["\u3F92", "\u3F92"], /* CJK Ideograph Extension A */
  ["\u3F93", "\u3F93"], /* CJK Ideograph Extension A */
  ["\u3F94", "\u3F94"], /* CJK Ideograph Extension A */
  ["\u3F95", "\u3F95"], /* CJK Ideograph Extension A */
  ["\u3F96", "\u3F96"], /* CJK Ideograph Extension A */
  ["\u3F97", "\u3F97"], /* CJK Ideograph Extension A */
  ["\u3F98", "\u3F98"], /* CJK Ideograph Extension A */
  ["\u3F99", "\u3F99"], /* CJK Ideograph Extension A */
  ["\u3F9A", "\u3F9A"], /* CJK Ideograph Extension A */
  ["\u3F9B", "\u3F9B"], /* CJK Ideograph Extension A */
  ["\u3F9C", "\u3F9C"], /* CJK Ideograph Extension A */
  ["\u3F9D", "\u3F9D"], /* CJK Ideograph Extension A */
  ["\u3F9E", "\u3F9E"], /* CJK Ideograph Extension A */
  ["\u3F9F", "\u3F9F"], /* CJK Ideograph Extension A */
  ["\u3FA0", "\u3FA0"], /* CJK Ideograph Extension A */
  ["\u3FA1", "\u3FA1"], /* CJK Ideograph Extension A */
  ["\u3FA2", "\u3FA2"], /* CJK Ideograph Extension A */
  ["\u3FA3", "\u3FA3"], /* CJK Ideograph Extension A */
  ["\u3FA4", "\u3FA4"], /* CJK Ideograph Extension A */
  ["\u3FA5", "\u3FA5"], /* CJK Ideograph Extension A */
  ["\u3FA6", "\u3FA6"], /* CJK Ideograph Extension A */
  ["\u3FA7", "\u3FA7"], /* CJK Ideograph Extension A */
  ["\u3FA8", "\u3FA8"], /* CJK Ideograph Extension A */
  ["\u3FA9", "\u3FA9"], /* CJK Ideograph Extension A */
  ["\u3FAA", "\u3FAA"], /* CJK Ideograph Extension A */
  ["\u3FAB", "\u3FAB"], /* CJK Ideograph Extension A */
  ["\u3FAC", "\u3FAC"], /* CJK Ideograph Extension A */
  ["\u3FAD", "\u3FAD"], /* CJK Ideograph Extension A */
  ["\u3FAE", "\u3FAE"], /* CJK Ideograph Extension A */
  ["\u3FAF", "\u3FAF"], /* CJK Ideograph Extension A */
  ["\u3FB0", "\u3FB0"], /* CJK Ideograph Extension A */
  ["\u3FB1", "\u3FB1"], /* CJK Ideograph Extension A */
  ["\u3FB2", "\u3FB2"], /* CJK Ideograph Extension A */
  ["\u3FB3", "\u3FB3"], /* CJK Ideograph Extension A */
  ["\u3FB4", "\u3FB4"], /* CJK Ideograph Extension A */
  ["\u3FB5", "\u3FB5"], /* CJK Ideograph Extension A */
  ["\u3FB6", "\u3FB6"], /* CJK Ideograph Extension A */
  ["\u3FB7", "\u3FB7"], /* CJK Ideograph Extension A */
  ["\u3FB8", "\u3FB8"], /* CJK Ideograph Extension A */
  ["\u3FB9", "\u3FB9"], /* CJK Ideograph Extension A */
  ["\u3FBA", "\u3FBA"], /* CJK Ideograph Extension A */
  ["\u3FBB", "\u3FBB"], /* CJK Ideograph Extension A */
  ["\u3FBC", "\u3FBC"], /* CJK Ideograph Extension A */
  ["\u3FBD", "\u3FBD"], /* CJK Ideograph Extension A */
  ["\u3FBE", "\u3FBE"], /* CJK Ideograph Extension A */
  ["\u3FBF", "\u3FBF"], /* CJK Ideograph Extension A */
  ["\u3FC0", "\u3FC0"], /* CJK Ideograph Extension A */
  ["\u3FC1", "\u3FC1"], /* CJK Ideograph Extension A */
  ["\u3FC2", "\u3FC2"], /* CJK Ideograph Extension A */
  ["\u3FC3", "\u3FC3"], /* CJK Ideograph Extension A */
  ["\u3FC4", "\u3FC4"], /* CJK Ideograph Extension A */
  ["\u3FC5", "\u3FC5"], /* CJK Ideograph Extension A */
  ["\u3FC6", "\u3FC6"], /* CJK Ideograph Extension A */
  ["\u3FC7", "\u3FC7"], /* CJK Ideograph Extension A */
  ["\u3FC8", "\u3FC8"], /* CJK Ideograph Extension A */
  ["\u3FC9", "\u3FC9"], /* CJK Ideograph Extension A */
  ["\u3FCA", "\u3FCA"], /* CJK Ideograph Extension A */
  ["\u3FCB", "\u3FCB"], /* CJK Ideograph Extension A */
  ["\u3FCC", "\u3FCC"], /* CJK Ideograph Extension A */
  ["\u3FCD", "\u3FCD"], /* CJK Ideograph Extension A */
  ["\u3FCE", "\u3FCE"], /* CJK Ideograph Extension A */
  ["\u3FCF", "\u3FCF"], /* CJK Ideograph Extension A */
  ["\u3FD0", "\u3FD0"], /* CJK Ideograph Extension A */
  ["\u3FD1", "\u3FD1"], /* CJK Ideograph Extension A */
  ["\u3FD2", "\u3FD2"], /* CJK Ideograph Extension A */
  ["\u3FD3", "\u3FD3"], /* CJK Ideograph Extension A */
  ["\u3FD4", "\u3FD4"], /* CJK Ideograph Extension A */
  ["\u3FD5", "\u3FD5"], /* CJK Ideograph Extension A */
  ["\u3FD6", "\u3FD6"], /* CJK Ideograph Extension A */
  ["\u3FD7", "\u3FD7"], /* CJK Ideograph Extension A */
  ["\u3FD8", "\u3FD8"], /* CJK Ideograph Extension A */
  ["\u3FD9", "\u3FD9"], /* CJK Ideograph Extension A */
  ["\u3FDA", "\u3FDA"], /* CJK Ideograph Extension A */
  ["\u3FDB", "\u3FDB"], /* CJK Ideograph Extension A */
  ["\u3FDC", "\u3FDC"], /* CJK Ideograph Extension A */
  ["\u3FDD", "\u3FDD"], /* CJK Ideograph Extension A */
  ["\u3FDE", "\u3FDE"], /* CJK Ideograph Extension A */
  ["\u3FDF", "\u3FDF"], /* CJK Ideograph Extension A */
  ["\u3FE0", "\u3FE0"], /* CJK Ideograph Extension A */
  ["\u3FE1", "\u3FE1"], /* CJK Ideograph Extension A */
  ["\u3FE2", "\u3FE2"], /* CJK Ideograph Extension A */
  ["\u3FE3", "\u3FE3"], /* CJK Ideograph Extension A */
  ["\u3FE4", "\u3FE4"], /* CJK Ideograph Extension A */
  ["\u3FE5", "\u3FE5"], /* CJK Ideograph Extension A */
  ["\u3FE6", "\u3FE6"], /* CJK Ideograph Extension A */
  ["\u3FE7", "\u3FE7"], /* CJK Ideograph Extension A */
  ["\u3FE8", "\u3FE8"], /* CJK Ideograph Extension A */
  ["\u3FE9", "\u3FE9"], /* CJK Ideograph Extension A */
  ["\u3FEA", "\u3FEA"], /* CJK Ideograph Extension A */
  ["\u3FEB", "\u3FEB"], /* CJK Ideograph Extension A */
  ["\u3FEC", "\u3FEC"], /* CJK Ideograph Extension A */
  ["\u3FED", "\u3FED"], /* CJK Ideograph Extension A */
  ["\u3FEE", "\u3FEE"], /* CJK Ideograph Extension A */
  ["\u3FEF", "\u3FEF"], /* CJK Ideograph Extension A */
  ["\u3FF0", "\u3FF0"], /* CJK Ideograph Extension A */
  ["\u3FF1", "\u3FF1"], /* CJK Ideograph Extension A */
  ["\u3FF2", "\u3FF2"], /* CJK Ideograph Extension A */
  ["\u3FF3", "\u3FF3"], /* CJK Ideograph Extension A */
  ["\u3FF4", "\u3FF4"], /* CJK Ideograph Extension A */
  ["\u3FF5", "\u3FF5"], /* CJK Ideograph Extension A */
  ["\u3FF6", "\u3FF6"], /* CJK Ideograph Extension A */
  ["\u3FF7", "\u3FF7"], /* CJK Ideograph Extension A */
  ["\u3FF8", "\u3FF8"], /* CJK Ideograph Extension A */
  ["\u3FF9", "\u3FF9"], /* CJK Ideograph Extension A */
  ["\u3FFA", "\u3FFA"], /* CJK Ideograph Extension A */
  ["\u3FFB", "\u3FFB"], /* CJK Ideograph Extension A */
  ["\u3FFC", "\u3FFC"], /* CJK Ideograph Extension A */
  ["\u3FFD", "\u3FFD"], /* CJK Ideograph Extension A */
  ["\u3FFE", "\u3FFE"], /* CJK Ideograph Extension A */
  ["\u3FFF", "\u3FFF"], /* CJK Ideograph Extension A */
  ["\u4000", "\u4000"], /* CJK Ideograph Extension A */
  ["\u4001", "\u4001"], /* CJK Ideograph Extension A */
  ["\u4002", "\u4002"], /* CJK Ideograph Extension A */
  ["\u4003", "\u4003"], /* CJK Ideograph Extension A */
  ["\u4004", "\u4004"], /* CJK Ideograph Extension A */
  ["\u4005", "\u4005"], /* CJK Ideograph Extension A */
  ["\u4006", "\u4006"], /* CJK Ideograph Extension A */
  ["\u4007", "\u4007"], /* CJK Ideograph Extension A */
  ["\u4008", "\u4008"], /* CJK Ideograph Extension A */
  ["\u4009", "\u4009"], /* CJK Ideograph Extension A */
  ["\u400A", "\u400A"], /* CJK Ideograph Extension A */
  ["\u400B", "\u400B"], /* CJK Ideograph Extension A */
  ["\u400C", "\u400C"], /* CJK Ideograph Extension A */
  ["\u400D", "\u400D"], /* CJK Ideograph Extension A */
  ["\u400E", "\u400E"], /* CJK Ideograph Extension A */
  ["\u400F", "\u400F"], /* CJK Ideograph Extension A */
  ["\u4010", "\u4010"], /* CJK Ideograph Extension A */
  ["\u4011", "\u4011"], /* CJK Ideograph Extension A */
  ["\u4012", "\u4012"], /* CJK Ideograph Extension A */
  ["\u4013", "\u4013"], /* CJK Ideograph Extension A */
  ["\u4014", "\u4014"], /* CJK Ideograph Extension A */
  ["\u4015", "\u4015"], /* CJK Ideograph Extension A */
  ["\u4016", "\u4016"], /* CJK Ideograph Extension A */
  ["\u4017", "\u4017"], /* CJK Ideograph Extension A */
  ["\u4018", "\u4018"], /* CJK Ideograph Extension A */
  ["\u4019", "\u4019"], /* CJK Ideograph Extension A */
  ["\u401A", "\u401A"], /* CJK Ideograph Extension A */
  ["\u401B", "\u401B"], /* CJK Ideograph Extension A */
  ["\u401C", "\u401C"], /* CJK Ideograph Extension A */
  ["\u401D", "\u401D"], /* CJK Ideograph Extension A */
  ["\u401E", "\u401E"], /* CJK Ideograph Extension A */
  ["\u401F", "\u401F"], /* CJK Ideograph Extension A */
  ["\u4020", "\u4020"], /* CJK Ideograph Extension A */
  ["\u4021", "\u4021"], /* CJK Ideograph Extension A */
  ["\u4022", "\u4022"], /* CJK Ideograph Extension A */
  ["\u4023", "\u4023"], /* CJK Ideograph Extension A */
  ["\u4024", "\u4024"], /* CJK Ideograph Extension A */
  ["\u4025", "\u4025"], /* CJK Ideograph Extension A */
  ["\u4026", "\u4026"], /* CJK Ideograph Extension A */
  ["\u4027", "\u4027"], /* CJK Ideograph Extension A */
  ["\u4028", "\u4028"], /* CJK Ideograph Extension A */
  ["\u4029", "\u4029"], /* CJK Ideograph Extension A */
  ["\u402A", "\u402A"], /* CJK Ideograph Extension A */
  ["\u402B", "\u402B"], /* CJK Ideograph Extension A */
  ["\u402C", "\u402C"], /* CJK Ideograph Extension A */
  ["\u402D", "\u402D"], /* CJK Ideograph Extension A */
  ["\u402E", "\u402E"], /* CJK Ideograph Extension A */
  ["\u402F", "\u402F"], /* CJK Ideograph Extension A */
  ["\u4030", "\u4030"], /* CJK Ideograph Extension A */
  ["\u4031", "\u4031"], /* CJK Ideograph Extension A */
  ["\u4032", "\u4032"], /* CJK Ideograph Extension A */
  ["\u4033", "\u4033"], /* CJK Ideograph Extension A */
  ["\u4034", "\u4034"], /* CJK Ideograph Extension A */
  ["\u4035", "\u4035"], /* CJK Ideograph Extension A */
  ["\u4036", "\u4036"], /* CJK Ideograph Extension A */
  ["\u4037", "\u4037"], /* CJK Ideograph Extension A */
  ["\u4038", "\u4038"], /* CJK Ideograph Extension A */
  ["\u4039", "\u4039"], /* CJK Ideograph Extension A */
  ["\u403A", "\u403A"], /* CJK Ideograph Extension A */
  ["\u403B", "\u403B"], /* CJK Ideograph Extension A */
  ["\u403C", "\u403C"], /* CJK Ideograph Extension A */
  ["\u403D", "\u403D"], /* CJK Ideograph Extension A */
  ["\u403E", "\u403E"], /* CJK Ideograph Extension A */
  ["\u403F", "\u403F"], /* CJK Ideograph Extension A */
  ["\u4040", "\u4040"], /* CJK Ideograph Extension A */
  ["\u4041", "\u4041"], /* CJK Ideograph Extension A */
  ["\u4042", "\u4042"], /* CJK Ideograph Extension A */
  ["\u4043", "\u4043"], /* CJK Ideograph Extension A */
  ["\u4044", "\u4044"], /* CJK Ideograph Extension A */
  ["\u4045", "\u4045"], /* CJK Ideograph Extension A */
  ["\u4046", "\u4046"], /* CJK Ideograph Extension A */
  ["\u4047", "\u4047"], /* CJK Ideograph Extension A */
  ["\u4048", "\u4048"], /* CJK Ideograph Extension A */
  ["\u4049", "\u4049"], /* CJK Ideograph Extension A */
  ["\u404A", "\u404A"], /* CJK Ideograph Extension A */
  ["\u404B", "\u404B"], /* CJK Ideograph Extension A */
  ["\u404C", "\u404C"], /* CJK Ideograph Extension A */
  ["\u404D", "\u404D"], /* CJK Ideograph Extension A */
  ["\u404E", "\u404E"], /* CJK Ideograph Extension A */
  ["\u404F", "\u404F"], /* CJK Ideograph Extension A */
  ["\u4050", "\u4050"], /* CJK Ideograph Extension A */
  ["\u4051", "\u4051"], /* CJK Ideograph Extension A */
  ["\u4052", "\u4052"], /* CJK Ideograph Extension A */
  ["\u4053", "\u4053"], /* CJK Ideograph Extension A */
  ["\u4054", "\u4054"], /* CJK Ideograph Extension A */
  ["\u4055", "\u4055"], /* CJK Ideograph Extension A */
  ["\u4056", "\u4056"], /* CJK Ideograph Extension A */
  ["\u4057", "\u4057"], /* CJK Ideograph Extension A */
  ["\u4058", "\u4058"], /* CJK Ideograph Extension A */
  ["\u4059", "\u4059"], /* CJK Ideograph Extension A */
  ["\u405A", "\u405A"], /* CJK Ideograph Extension A */
  ["\u405B", "\u405B"], /* CJK Ideograph Extension A */
  ["\u405C", "\u405C"], /* CJK Ideograph Extension A */
  ["\u405D", "\u405D"], /* CJK Ideograph Extension A */
  ["\u405E", "\u405E"], /* CJK Ideograph Extension A */
  ["\u405F", "\u405F"], /* CJK Ideograph Extension A */
  ["\u4060", "\u4060"], /* CJK Ideograph Extension A */
  ["\u4061", "\u4061"], /* CJK Ideograph Extension A */
  ["\u4062", "\u4062"], /* CJK Ideograph Extension A */
  ["\u4063", "\u4063"], /* CJK Ideograph Extension A */
  ["\u4064", "\u4064"], /* CJK Ideograph Extension A */
  ["\u4065", "\u4065"], /* CJK Ideograph Extension A */
  ["\u4066", "\u4066"], /* CJK Ideograph Extension A */
  ["\u4067", "\u4067"], /* CJK Ideograph Extension A */
  ["\u4068", "\u4068"], /* CJK Ideograph Extension A */
  ["\u4069", "\u4069"], /* CJK Ideograph Extension A */
  ["\u406A", "\u406A"], /* CJK Ideograph Extension A */
  ["\u406B", "\u406B"], /* CJK Ideograph Extension A */
  ["\u406C", "\u406C"], /* CJK Ideograph Extension A */
  ["\u406D", "\u406D"], /* CJK Ideograph Extension A */
  ["\u406E", "\u406E"], /* CJK Ideograph Extension A */
  ["\u406F", "\u406F"], /* CJK Ideograph Extension A */
  ["\u4070", "\u4070"], /* CJK Ideograph Extension A */
  ["\u4071", "\u4071"], /* CJK Ideograph Extension A */
  ["\u4072", "\u4072"], /* CJK Ideograph Extension A */
  ["\u4073", "\u4073"], /* CJK Ideograph Extension A */
  ["\u4074", "\u4074"], /* CJK Ideograph Extension A */
  ["\u4075", "\u4075"], /* CJK Ideograph Extension A */
  ["\u4076", "\u4076"], /* CJK Ideograph Extension A */
  ["\u4077", "\u4077"], /* CJK Ideograph Extension A */
  ["\u4078", "\u4078"], /* CJK Ideograph Extension A */
  ["\u4079", "\u4079"], /* CJK Ideograph Extension A */
  ["\u407A", "\u407A"], /* CJK Ideograph Extension A */
  ["\u407B", "\u407B"], /* CJK Ideograph Extension A */
  ["\u407C", "\u407C"], /* CJK Ideograph Extension A */
  ["\u407D", "\u407D"], /* CJK Ideograph Extension A */
  ["\u407E", "\u407E"], /* CJK Ideograph Extension A */
  ["\u407F", "\u407F"], /* CJK Ideograph Extension A */
  ["\u4080", "\u4080"], /* CJK Ideograph Extension A */
  ["\u4081", "\u4081"], /* CJK Ideograph Extension A */
  ["\u4082", "\u4082"], /* CJK Ideograph Extension A */
  ["\u4083", "\u4083"], /* CJK Ideograph Extension A */
  ["\u4084", "\u4084"], /* CJK Ideograph Extension A */
  ["\u4085", "\u4085"], /* CJK Ideograph Extension A */
  ["\u4086", "\u4086"], /* CJK Ideograph Extension A */
  ["\u4087", "\u4087"], /* CJK Ideograph Extension A */
  ["\u4088", "\u4088"], /* CJK Ideograph Extension A */
  ["\u4089", "\u4089"], /* CJK Ideograph Extension A */
  ["\u408A", "\u408A"], /* CJK Ideograph Extension A */
  ["\u408B", "\u408B"], /* CJK Ideograph Extension A */
  ["\u408C", "\u408C"], /* CJK Ideograph Extension A */
  ["\u408D", "\u408D"], /* CJK Ideograph Extension A */
  ["\u408E", "\u408E"], /* CJK Ideograph Extension A */
  ["\u408F", "\u408F"], /* CJK Ideograph Extension A */
  ["\u4090", "\u4090"], /* CJK Ideograph Extension A */
  ["\u4091", "\u4091"], /* CJK Ideograph Extension A */
  ["\u4092", "\u4092"], /* CJK Ideograph Extension A */
  ["\u4093", "\u4093"], /* CJK Ideograph Extension A */
  ["\u4094", "\u4094"], /* CJK Ideograph Extension A */
  ["\u4095", "\u4095"], /* CJK Ideograph Extension A */
  ["\u4096", "\u4096"], /* CJK Ideograph Extension A */
  ["\u4097", "\u4097"], /* CJK Ideograph Extension A */
  ["\u4098", "\u4098"], /* CJK Ideograph Extension A */
  ["\u4099", "\u4099"], /* CJK Ideograph Extension A */
  ["\u409A", "\u409A"], /* CJK Ideograph Extension A */
  ["\u409B", "\u409B"], /* CJK Ideograph Extension A */
  ["\u409C", "\u409C"], /* CJK Ideograph Extension A */
  ["\u409D", "\u409D"], /* CJK Ideograph Extension A */
  ["\u409E", "\u409E"], /* CJK Ideograph Extension A */
  ["\u409F", "\u409F"], /* CJK Ideograph Extension A */
  ["\u40A0", "\u40A0"], /* CJK Ideograph Extension A */
  ["\u40A1", "\u40A1"], /* CJK Ideograph Extension A */
  ["\u40A2", "\u40A2"], /* CJK Ideograph Extension A */
  ["\u40A3", "\u40A3"], /* CJK Ideograph Extension A */
  ["\u40A4", "\u40A4"], /* CJK Ideograph Extension A */
  ["\u40A5", "\u40A5"], /* CJK Ideograph Extension A */
  ["\u40A6", "\u40A6"], /* CJK Ideograph Extension A */
  ["\u40A7", "\u40A7"], /* CJK Ideograph Extension A */
  ["\u40A8", "\u40A8"], /* CJK Ideograph Extension A */
  ["\u40A9", "\u40A9"], /* CJK Ideograph Extension A */
  ["\u40AA", "\u40AA"], /* CJK Ideograph Extension A */
  ["\u40AB", "\u40AB"], /* CJK Ideograph Extension A */
  ["\u40AC", "\u40AC"], /* CJK Ideograph Extension A */
  ["\u40AD", "\u40AD"], /* CJK Ideograph Extension A */
  ["\u40AE", "\u40AE"], /* CJK Ideograph Extension A */
  ["\u40AF", "\u40AF"], /* CJK Ideograph Extension A */
  ["\u40B0", "\u40B0"], /* CJK Ideograph Extension A */
  ["\u40B1", "\u40B1"], /* CJK Ideograph Extension A */
  ["\u40B2", "\u40B2"], /* CJK Ideograph Extension A */
  ["\u40B3", "\u40B3"], /* CJK Ideograph Extension A */
  ["\u40B4", "\u40B4"], /* CJK Ideograph Extension A */
  ["\u40B5", "\u40B5"], /* CJK Ideograph Extension A */
  ["\u40B6", "\u40B6"], /* CJK Ideograph Extension A */
  ["\u40B7", "\u40B7"], /* CJK Ideograph Extension A */
  ["\u40B8", "\u40B8"], /* CJK Ideograph Extension A */
  ["\u40B9", "\u40B9"], /* CJK Ideograph Extension A */
  ["\u40BA", "\u40BA"], /* CJK Ideograph Extension A */
  ["\u40BB", "\u40BB"], /* CJK Ideograph Extension A */
  ["\u40BC", "\u40BC"], /* CJK Ideograph Extension A */
  ["\u40BD", "\u40BD"], /* CJK Ideograph Extension A */
  ["\u40BE", "\u40BE"], /* CJK Ideograph Extension A */
  ["\u40BF", "\u40BF"], /* CJK Ideograph Extension A */
  ["\u40C0", "\u40C0"], /* CJK Ideograph Extension A */
  ["\u40C1", "\u40C1"], /* CJK Ideograph Extension A */
  ["\u40C2", "\u40C2"], /* CJK Ideograph Extension A */
  ["\u40C3", "\u40C3"], /* CJK Ideograph Extension A */
  ["\u40C4", "\u40C4"], /* CJK Ideograph Extension A */
  ["\u40C5", "\u40C5"], /* CJK Ideograph Extension A */
  ["\u40C6", "\u40C6"], /* CJK Ideograph Extension A */
  ["\u40C7", "\u40C7"], /* CJK Ideograph Extension A */
  ["\u40C8", "\u40C8"], /* CJK Ideograph Extension A */
  ["\u40C9", "\u40C9"], /* CJK Ideograph Extension A */
  ["\u40CA", "\u40CA"], /* CJK Ideograph Extension A */
  ["\u40CB", "\u40CB"], /* CJK Ideograph Extension A */
  ["\u40CC", "\u40CC"], /* CJK Ideograph Extension A */
  ["\u40CD", "\u40CD"], /* CJK Ideograph Extension A */
  ["\u40CE", "\u40CE"], /* CJK Ideograph Extension A */
  ["\u40CF", "\u40CF"], /* CJK Ideograph Extension A */
  ["\u40D0", "\u40D0"], /* CJK Ideograph Extension A */
  ["\u40D1", "\u40D1"], /* CJK Ideograph Extension A */
  ["\u40D2", "\u40D2"], /* CJK Ideograph Extension A */
  ["\u40D3", "\u40D3"], /* CJK Ideograph Extension A */
  ["\u40D4", "\u40D4"], /* CJK Ideograph Extension A */
  ["\u40D5", "\u40D5"], /* CJK Ideograph Extension A */
  ["\u40D6", "\u40D6"], /* CJK Ideograph Extension A */
  ["\u40D7", "\u40D7"], /* CJK Ideograph Extension A */
  ["\u40D8", "\u40D8"], /* CJK Ideograph Extension A */
  ["\u40D9", "\u40D9"], /* CJK Ideograph Extension A */
  ["\u40DA", "\u40DA"], /* CJK Ideograph Extension A */
  ["\u40DB", "\u40DB"], /* CJK Ideograph Extension A */
  ["\u40DC", "\u40DC"], /* CJK Ideograph Extension A */
  ["\u40DD", "\u40DD"], /* CJK Ideograph Extension A */
  ["\u40DE", "\u40DE"], /* CJK Ideograph Extension A */
  ["\u40DF", "\u40DF"], /* CJK Ideograph Extension A */
  ["\u40E0", "\u40E0"], /* CJK Ideograph Extension A */
  ["\u40E1", "\u40E1"], /* CJK Ideograph Extension A */
  ["\u40E2", "\u40E2"], /* CJK Ideograph Extension A */
  ["\u40E3", "\u40E3"], /* CJK Ideograph Extension A */
  ["\u40E4", "\u40E4"], /* CJK Ideograph Extension A */
  ["\u40E5", "\u40E5"], /* CJK Ideograph Extension A */
  ["\u40E6", "\u40E6"], /* CJK Ideograph Extension A */
  ["\u40E7", "\u40E7"], /* CJK Ideograph Extension A */
  ["\u40E8", "\u40E8"], /* CJK Ideograph Extension A */
  ["\u40E9", "\u40E9"], /* CJK Ideograph Extension A */
  ["\u40EA", "\u40EA"], /* CJK Ideograph Extension A */
  ["\u40EB", "\u40EB"], /* CJK Ideograph Extension A */
  ["\u40EC", "\u40EC"], /* CJK Ideograph Extension A */
  ["\u40ED", "\u40ED"], /* CJK Ideograph Extension A */
  ["\u40EE", "\u40EE"], /* CJK Ideograph Extension A */
  ["\u40EF", "\u40EF"], /* CJK Ideograph Extension A */
  ["\u40F0", "\u40F0"], /* CJK Ideograph Extension A */
  ["\u40F1", "\u40F1"], /* CJK Ideograph Extension A */
  ["\u40F2", "\u40F2"], /* CJK Ideograph Extension A */
  ["\u40F3", "\u40F3"], /* CJK Ideograph Extension A */
  ["\u40F4", "\u40F4"], /* CJK Ideograph Extension A */
  ["\u40F5", "\u40F5"], /* CJK Ideograph Extension A */
  ["\u40F6", "\u40F6"], /* CJK Ideograph Extension A */
  ["\u40F7", "\u40F7"], /* CJK Ideograph Extension A */
  ["\u40F8", "\u40F8"], /* CJK Ideograph Extension A */
  ["\u40F9", "\u40F9"], /* CJK Ideograph Extension A */
  ["\u40FA", "\u40FA"], /* CJK Ideograph Extension A */
  ["\u40FB", "\u40FB"], /* CJK Ideograph Extension A */
  ["\u40FC", "\u40FC"], /* CJK Ideograph Extension A */
  ["\u40FD", "\u40FD"], /* CJK Ideograph Extension A */
  ["\u40FE", "\u40FE"], /* CJK Ideograph Extension A */
  ["\u40FF", "\u40FF"], /* CJK Ideograph Extension A */
  ["\u4100", "\u4100"], /* CJK Ideograph Extension A */
  ["\u4101", "\u4101"], /* CJK Ideograph Extension A */
  ["\u4102", "\u4102"], /* CJK Ideograph Extension A */
  ["\u4103", "\u4103"], /* CJK Ideograph Extension A */
  ["\u4104", "\u4104"], /* CJK Ideograph Extension A */
  ["\u4105", "\u4105"], /* CJK Ideograph Extension A */
  ["\u4106", "\u4106"], /* CJK Ideograph Extension A */
  ["\u4107", "\u4107"], /* CJK Ideograph Extension A */
  ["\u4108", "\u4108"], /* CJK Ideograph Extension A */
  ["\u4109", "\u4109"], /* CJK Ideograph Extension A */
  ["\u410A", "\u410A"], /* CJK Ideograph Extension A */
  ["\u410B", "\u410B"], /* CJK Ideograph Extension A */
  ["\u410C", "\u410C"], /* CJK Ideograph Extension A */
  ["\u410D", "\u410D"], /* CJK Ideograph Extension A */
  ["\u410E", "\u410E"], /* CJK Ideograph Extension A */
  ["\u410F", "\u410F"], /* CJK Ideograph Extension A */
  ["\u4110", "\u4110"], /* CJK Ideograph Extension A */
  ["\u4111", "\u4111"], /* CJK Ideograph Extension A */
  ["\u4112", "\u4112"], /* CJK Ideograph Extension A */
  ["\u4113", "\u4113"], /* CJK Ideograph Extension A */
  ["\u4114", "\u4114"], /* CJK Ideograph Extension A */
  ["\u4115", "\u4115"], /* CJK Ideograph Extension A */
  ["\u4116", "\u4116"], /* CJK Ideograph Extension A */
  ["\u4117", "\u4117"], /* CJK Ideograph Extension A */
  ["\u4118", "\u4118"], /* CJK Ideograph Extension A */
  ["\u4119", "\u4119"], /* CJK Ideograph Extension A */
  ["\u411A", "\u411A"], /* CJK Ideograph Extension A */
  ["\u411B", "\u411B"], /* CJK Ideograph Extension A */
  ["\u411C", "\u411C"], /* CJK Ideograph Extension A */
  ["\u411D", "\u411D"], /* CJK Ideograph Extension A */
  ["\u411E", "\u411E"], /* CJK Ideograph Extension A */
  ["\u411F", "\u411F"], /* CJK Ideograph Extension A */
  ["\u4120", "\u4120"], /* CJK Ideograph Extension A */
  ["\u4121", "\u4121"], /* CJK Ideograph Extension A */
  ["\u4122", "\u4122"], /* CJK Ideograph Extension A */
  ["\u4123", "\u4123"], /* CJK Ideograph Extension A */
  ["\u4124", "\u4124"], /* CJK Ideograph Extension A */
  ["\u4125", "\u4125"], /* CJK Ideograph Extension A */
  ["\u4126", "\u4126"], /* CJK Ideograph Extension A */
  ["\u4127", "\u4127"], /* CJK Ideograph Extension A */
  ["\u4128", "\u4128"], /* CJK Ideograph Extension A */
  ["\u4129", "\u4129"], /* CJK Ideograph Extension A */
  ["\u412A", "\u412A"], /* CJK Ideograph Extension A */
  ["\u412B", "\u412B"], /* CJK Ideograph Extension A */
  ["\u412C", "\u412C"], /* CJK Ideograph Extension A */
  ["\u412D", "\u412D"], /* CJK Ideograph Extension A */
  ["\u412E", "\u412E"], /* CJK Ideograph Extension A */
  ["\u412F", "\u412F"], /* CJK Ideograph Extension A */
  ["\u4130", "\u4130"], /* CJK Ideograph Extension A */
  ["\u4131", "\u4131"], /* CJK Ideograph Extension A */
  ["\u4132", "\u4132"], /* CJK Ideograph Extension A */
  ["\u4133", "\u4133"], /* CJK Ideograph Extension A */
  ["\u4134", "\u4134"], /* CJK Ideograph Extension A */
  ["\u4135", "\u4135"], /* CJK Ideograph Extension A */
  ["\u4136", "\u4136"], /* CJK Ideograph Extension A */
  ["\u4137", "\u4137"], /* CJK Ideograph Extension A */
  ["\u4138", "\u4138"], /* CJK Ideograph Extension A */
  ["\u4139", "\u4139"], /* CJK Ideograph Extension A */
  ["\u413A", "\u413A"], /* CJK Ideograph Extension A */
  ["\u413B", "\u413B"], /* CJK Ideograph Extension A */
  ["\u413C", "\u413C"], /* CJK Ideograph Extension A */
  ["\u413D", "\u413D"], /* CJK Ideograph Extension A */
  ["\u413E", "\u413E"], /* CJK Ideograph Extension A */
  ["\u413F", "\u413F"], /* CJK Ideograph Extension A */
  ["\u4140", "\u4140"], /* CJK Ideograph Extension A */
  ["\u4141", "\u4141"], /* CJK Ideograph Extension A */
  ["\u4142", "\u4142"], /* CJK Ideograph Extension A */
  ["\u4143", "\u4143"], /* CJK Ideograph Extension A */
  ["\u4144", "\u4144"], /* CJK Ideograph Extension A */
  ["\u4145", "\u4145"], /* CJK Ideograph Extension A */
  ["\u4146", "\u4146"], /* CJK Ideograph Extension A */
  ["\u4147", "\u4147"], /* CJK Ideograph Extension A */
  ["\u4148", "\u4148"], /* CJK Ideograph Extension A */
  ["\u4149", "\u4149"], /* CJK Ideograph Extension A */
  ["\u414A", "\u414A"], /* CJK Ideograph Extension A */
  ["\u414B", "\u414B"], /* CJK Ideograph Extension A */
  ["\u414C", "\u414C"], /* CJK Ideograph Extension A */
  ["\u414D", "\u414D"], /* CJK Ideograph Extension A */
  ["\u414E", "\u414E"], /* CJK Ideograph Extension A */
  ["\u414F", "\u414F"], /* CJK Ideograph Extension A */
  ["\u4150", "\u4150"], /* CJK Ideograph Extension A */
  ["\u4151", "\u4151"], /* CJK Ideograph Extension A */
  ["\u4152", "\u4152"], /* CJK Ideograph Extension A */
  ["\u4153", "\u4153"], /* CJK Ideograph Extension A */
  ["\u4154", "\u4154"], /* CJK Ideograph Extension A */
  ["\u4155", "\u4155"], /* CJK Ideograph Extension A */
  ["\u4156", "\u4156"], /* CJK Ideograph Extension A */
  ["\u4157", "\u4157"], /* CJK Ideograph Extension A */
  ["\u4158", "\u4158"], /* CJK Ideograph Extension A */
  ["\u4159", "\u4159"], /* CJK Ideograph Extension A */
  ["\u415A", "\u415A"], /* CJK Ideograph Extension A */
  ["\u415B", "\u415B"], /* CJK Ideograph Extension A */
  ["\u415C", "\u415C"], /* CJK Ideograph Extension A */
  ["\u415D", "\u415D"], /* CJK Ideograph Extension A */
  ["\u415E", "\u415E"], /* CJK Ideograph Extension A */
  ["\u415F", "\u415F"], /* CJK Ideograph Extension A */
  ["\u4160", "\u4160"], /* CJK Ideograph Extension A */
  ["\u4161", "\u4161"], /* CJK Ideograph Extension A */
  ["\u4162", "\u4162"], /* CJK Ideograph Extension A */
  ["\u4163", "\u4163"], /* CJK Ideograph Extension A */
  ["\u4164", "\u4164"], /* CJK Ideograph Extension A */
  ["\u4165", "\u4165"], /* CJK Ideograph Extension A */
  ["\u4166", "\u4166"], /* CJK Ideograph Extension A */
  ["\u4167", "\u4167"], /* CJK Ideograph Extension A */
  ["\u4168", "\u4168"], /* CJK Ideograph Extension A */
  ["\u4169", "\u4169"], /* CJK Ideograph Extension A */
  ["\u416A", "\u416A"], /* CJK Ideograph Extension A */
  ["\u416B", "\u416B"], /* CJK Ideograph Extension A */
  ["\u416C", "\u416C"], /* CJK Ideograph Extension A */
  ["\u416D", "\u416D"], /* CJK Ideograph Extension A */
  ["\u416E", "\u416E"], /* CJK Ideograph Extension A */
  ["\u416F", "\u416F"], /* CJK Ideograph Extension A */
  ["\u4170", "\u4170"], /* CJK Ideograph Extension A */
  ["\u4171", "\u4171"], /* CJK Ideograph Extension A */
  ["\u4172", "\u4172"], /* CJK Ideograph Extension A */
  ["\u4173", "\u4173"], /* CJK Ideograph Extension A */
  ["\u4174", "\u4174"], /* CJK Ideograph Extension A */
  ["\u4175", "\u4175"], /* CJK Ideograph Extension A */
  ["\u4176", "\u4176"], /* CJK Ideograph Extension A */
  ["\u4177", "\u4177"], /* CJK Ideograph Extension A */
  ["\u4178", "\u4178"], /* CJK Ideograph Extension A */
  ["\u4179", "\u4179"], /* CJK Ideograph Extension A */
  ["\u417A", "\u417A"], /* CJK Ideograph Extension A */
  ["\u417B", "\u417B"], /* CJK Ideograph Extension A */
  ["\u417C", "\u417C"], /* CJK Ideograph Extension A */
  ["\u417D", "\u417D"], /* CJK Ideograph Extension A */
  ["\u417E", "\u417E"], /* CJK Ideograph Extension A */
  ["\u417F", "\u417F"], /* CJK Ideograph Extension A */
  ["\u4180", "\u4180"], /* CJK Ideograph Extension A */
  ["\u4181", "\u4181"], /* CJK Ideograph Extension A */
  ["\u4182", "\u4182"], /* CJK Ideograph Extension A */
  ["\u4183", "\u4183"], /* CJK Ideograph Extension A */
  ["\u4184", "\u4184"], /* CJK Ideograph Extension A */
  ["\u4185", "\u4185"], /* CJK Ideograph Extension A */
  ["\u4186", "\u4186"], /* CJK Ideograph Extension A */
  ["\u4187", "\u4187"], /* CJK Ideograph Extension A */
  ["\u4188", "\u4188"], /* CJK Ideograph Extension A */
  ["\u4189", "\u4189"], /* CJK Ideograph Extension A */
  ["\u418A", "\u418A"], /* CJK Ideograph Extension A */
  ["\u418B", "\u418B"], /* CJK Ideograph Extension A */
  ["\u418C", "\u418C"], /* CJK Ideograph Extension A */
  ["\u418D", "\u418D"], /* CJK Ideograph Extension A */
  ["\u418E", "\u418E"], /* CJK Ideograph Extension A */
  ["\u418F", "\u418F"], /* CJK Ideograph Extension A */
  ["\u4190", "\u4190"], /* CJK Ideograph Extension A */
  ["\u4191", "\u4191"], /* CJK Ideograph Extension A */
  ["\u4192", "\u4192"], /* CJK Ideograph Extension A */
  ["\u4193", "\u4193"], /* CJK Ideograph Extension A */
  ["\u4194", "\u4194"], /* CJK Ideograph Extension A */
  ["\u4195", "\u4195"], /* CJK Ideograph Extension A */
  ["\u4196", "\u4196"], /* CJK Ideograph Extension A */
  ["\u4197", "\u4197"], /* CJK Ideograph Extension A */
  ["\u4198", "\u4198"], /* CJK Ideograph Extension A */
  ["\u4199", "\u4199"], /* CJK Ideograph Extension A */
  ["\u419A", "\u419A"], /* CJK Ideograph Extension A */
  ["\u419B", "\u419B"], /* CJK Ideograph Extension A */
  ["\u419C", "\u419C"], /* CJK Ideograph Extension A */
  ["\u419D", "\u419D"], /* CJK Ideograph Extension A */
  ["\u419E", "\u419E"], /* CJK Ideograph Extension A */
  ["\u419F", "\u419F"], /* CJK Ideograph Extension A */
  ["\u41A0", "\u41A0"], /* CJK Ideograph Extension A */
  ["\u41A1", "\u41A1"], /* CJK Ideograph Extension A */
  ["\u41A2", "\u41A2"], /* CJK Ideograph Extension A */
  ["\u41A3", "\u41A3"], /* CJK Ideograph Extension A */
  ["\u41A4", "\u41A4"], /* CJK Ideograph Extension A */
  ["\u41A5", "\u41A5"], /* CJK Ideograph Extension A */
  ["\u41A6", "\u41A6"], /* CJK Ideograph Extension A */
  ["\u41A7", "\u41A7"], /* CJK Ideograph Extension A */
  ["\u41A8", "\u41A8"], /* CJK Ideograph Extension A */
  ["\u41A9", "\u41A9"], /* CJK Ideograph Extension A */
  ["\u41AA", "\u41AA"], /* CJK Ideograph Extension A */
  ["\u41AB", "\u41AB"], /* CJK Ideograph Extension A */
  ["\u41AC", "\u41AC"], /* CJK Ideograph Extension A */
  ["\u41AD", "\u41AD"], /* CJK Ideograph Extension A */
  ["\u41AE", "\u41AE"], /* CJK Ideograph Extension A */
  ["\u41AF", "\u41AF"], /* CJK Ideograph Extension A */
  ["\u41B0", "\u41B0"], /* CJK Ideograph Extension A */
  ["\u41B1", "\u41B1"], /* CJK Ideograph Extension A */
  ["\u41B2", "\u41B2"], /* CJK Ideograph Extension A */
  ["\u41B3", "\u41B3"], /* CJK Ideograph Extension A */
  ["\u41B4", "\u41B4"], /* CJK Ideograph Extension A */
  ["\u41B5", "\u41B5"], /* CJK Ideograph Extension A */
  ["\u41B6", "\u41B6"], /* CJK Ideograph Extension A */
  ["\u41B7", "\u41B7"], /* CJK Ideograph Extension A */
  ["\u41B8", "\u41B8"], /* CJK Ideograph Extension A */
  ["\u41B9", "\u41B9"], /* CJK Ideograph Extension A */
  ["\u41BA", "\u41BA"], /* CJK Ideograph Extension A */
  ["\u41BB", "\u41BB"], /* CJK Ideograph Extension A */
  ["\u41BC", "\u41BC"], /* CJK Ideograph Extension A */
  ["\u41BD", "\u41BD"], /* CJK Ideograph Extension A */
  ["\u41BE", "\u41BE"], /* CJK Ideograph Extension A */
  ["\u41BF", "\u41BF"], /* CJK Ideograph Extension A */
  ["\u41C0", "\u41C0"], /* CJK Ideograph Extension A */
  ["\u41C1", "\u41C1"], /* CJK Ideograph Extension A */
  ["\u41C2", "\u41C2"], /* CJK Ideograph Extension A */
  ["\u41C3", "\u41C3"], /* CJK Ideograph Extension A */
  ["\u41C4", "\u41C4"], /* CJK Ideograph Extension A */
  ["\u41C5", "\u41C5"], /* CJK Ideograph Extension A */
  ["\u41C6", "\u41C6"], /* CJK Ideograph Extension A */
  ["\u41C7", "\u41C7"], /* CJK Ideograph Extension A */
  ["\u41C8", "\u41C8"], /* CJK Ideograph Extension A */
  ["\u41C9", "\u41C9"], /* CJK Ideograph Extension A */
  ["\u41CA", "\u41CA"], /* CJK Ideograph Extension A */
  ["\u41CB", "\u41CB"], /* CJK Ideograph Extension A */
  ["\u41CC", "\u41CC"], /* CJK Ideograph Extension A */
  ["\u41CD", "\u41CD"], /* CJK Ideograph Extension A */
  ["\u41CE", "\u41CE"], /* CJK Ideograph Extension A */
  ["\u41CF", "\u41CF"], /* CJK Ideograph Extension A */
  ["\u41D0", "\u41D0"], /* CJK Ideograph Extension A */
  ["\u41D1", "\u41D1"], /* CJK Ideograph Extension A */
  ["\u41D2", "\u41D2"], /* CJK Ideograph Extension A */
  ["\u41D3", "\u41D3"], /* CJK Ideograph Extension A */
  ["\u41D4", "\u41D4"], /* CJK Ideograph Extension A */
  ["\u41D5", "\u41D5"], /* CJK Ideograph Extension A */
  ["\u41D6", "\u41D6"], /* CJK Ideograph Extension A */
  ["\u41D7", "\u41D7"], /* CJK Ideograph Extension A */
  ["\u41D8", "\u41D8"], /* CJK Ideograph Extension A */
  ["\u41D9", "\u41D9"], /* CJK Ideograph Extension A */
  ["\u41DA", "\u41DA"], /* CJK Ideograph Extension A */
  ["\u41DB", "\u41DB"], /* CJK Ideograph Extension A */
  ["\u41DC", "\u41DC"], /* CJK Ideograph Extension A */
  ["\u41DD", "\u41DD"], /* CJK Ideograph Extension A */
  ["\u41DE", "\u41DE"], /* CJK Ideograph Extension A */
  ["\u41DF", "\u41DF"], /* CJK Ideograph Extension A */
  ["\u41E0", "\u41E0"], /* CJK Ideograph Extension A */
  ["\u41E1", "\u41E1"], /* CJK Ideograph Extension A */
  ["\u41E2", "\u41E2"], /* CJK Ideograph Extension A */
  ["\u41E3", "\u41E3"], /* CJK Ideograph Extension A */
  ["\u41E4", "\u41E4"], /* CJK Ideograph Extension A */
  ["\u41E5", "\u41E5"], /* CJK Ideograph Extension A */
  ["\u41E6", "\u41E6"], /* CJK Ideograph Extension A */
  ["\u41E7", "\u41E7"], /* CJK Ideograph Extension A */
  ["\u41E8", "\u41E8"], /* CJK Ideograph Extension A */
  ["\u41E9", "\u41E9"], /* CJK Ideograph Extension A */
  ["\u41EA", "\u41EA"], /* CJK Ideograph Extension A */
  ["\u41EB", "\u41EB"], /* CJK Ideograph Extension A */
  ["\u41EC", "\u41EC"], /* CJK Ideograph Extension A */
  ["\u41ED", "\u41ED"], /* CJK Ideograph Extension A */
  ["\u41EE", "\u41EE"], /* CJK Ideograph Extension A */
  ["\u41EF", "\u41EF"], /* CJK Ideograph Extension A */
  ["\u41F0", "\u41F0"], /* CJK Ideograph Extension A */
  ["\u41F1", "\u41F1"], /* CJK Ideograph Extension A */
  ["\u41F2", "\u41F2"], /* CJK Ideograph Extension A */
  ["\u41F3", "\u41F3"], /* CJK Ideograph Extension A */
  ["\u41F4", "\u41F4"], /* CJK Ideograph Extension A */
  ["\u41F5", "\u41F5"], /* CJK Ideograph Extension A */
  ["\u41F6", "\u41F6"], /* CJK Ideograph Extension A */
  ["\u41F7", "\u41F7"], /* CJK Ideograph Extension A */
  ["\u41F8", "\u41F8"], /* CJK Ideograph Extension A */
  ["\u41F9", "\u41F9"], /* CJK Ideograph Extension A */
  ["\u41FA", "\u41FA"], /* CJK Ideograph Extension A */
  ["\u41FB", "\u41FB"], /* CJK Ideograph Extension A */
  ["\u41FC", "\u41FC"], /* CJK Ideograph Extension A */
  ["\u41FD", "\u41FD"], /* CJK Ideograph Extension A */
  ["\u41FE", "\u41FE"], /* CJK Ideograph Extension A */
  ["\u41FF", "\u41FF"], /* CJK Ideograph Extension A */
  ["\u4200", "\u4200"], /* CJK Ideograph Extension A */
  ["\u4201", "\u4201"], /* CJK Ideograph Extension A */
  ["\u4202", "\u4202"], /* CJK Ideograph Extension A */
  ["\u4203", "\u4203"], /* CJK Ideograph Extension A */
  ["\u4204", "\u4204"], /* CJK Ideograph Extension A */
  ["\u4205", "\u4205"], /* CJK Ideograph Extension A */
  ["\u4206", "\u4206"], /* CJK Ideograph Extension A */
  ["\u4207", "\u4207"], /* CJK Ideograph Extension A */
  ["\u4208", "\u4208"], /* CJK Ideograph Extension A */
  ["\u4209", "\u4209"], /* CJK Ideograph Extension A */
  ["\u420A", "\u420A"], /* CJK Ideograph Extension A */
  ["\u420B", "\u420B"], /* CJK Ideograph Extension A */
  ["\u420C", "\u420C"], /* CJK Ideograph Extension A */
  ["\u420D", "\u420D"], /* CJK Ideograph Extension A */
  ["\u420E", "\u420E"], /* CJK Ideograph Extension A */
  ["\u420F", "\u420F"], /* CJK Ideograph Extension A */
  ["\u4210", "\u4210"], /* CJK Ideograph Extension A */
  ["\u4211", "\u4211"], /* CJK Ideograph Extension A */
  ["\u4212", "\u4212"], /* CJK Ideograph Extension A */
  ["\u4213", "\u4213"], /* CJK Ideograph Extension A */
  ["\u4214", "\u4214"], /* CJK Ideograph Extension A */
  ["\u4215", "\u4215"], /* CJK Ideograph Extension A */
  ["\u4216", "\u4216"], /* CJK Ideograph Extension A */
  ["\u4217", "\u4217"], /* CJK Ideograph Extension A */
  ["\u4218", "\u4218"], /* CJK Ideograph Extension A */
  ["\u4219", "\u4219"], /* CJK Ideograph Extension A */
  ["\u421A", "\u421A"], /* CJK Ideograph Extension A */
  ["\u421B", "\u421B"], /* CJK Ideograph Extension A */
  ["\u421C", "\u421C"], /* CJK Ideograph Extension A */
  ["\u421D", "\u421D"], /* CJK Ideograph Extension A */
  ["\u421E", "\u421E"], /* CJK Ideograph Extension A */
  ["\u421F", "\u421F"], /* CJK Ideograph Extension A */
  ["\u4220", "\u4220"], /* CJK Ideograph Extension A */
  ["\u4221", "\u4221"], /* CJK Ideograph Extension A */
  ["\u4222", "\u4222"], /* CJK Ideograph Extension A */
  ["\u4223", "\u4223"], /* CJK Ideograph Extension A */
  ["\u4224", "\u4224"], /* CJK Ideograph Extension A */
  ["\u4225", "\u4225"], /* CJK Ideograph Extension A */
  ["\u4226", "\u4226"], /* CJK Ideograph Extension A */
  ["\u4227", "\u4227"], /* CJK Ideograph Extension A */
  ["\u4228", "\u4228"], /* CJK Ideograph Extension A */
  ["\u4229", "\u4229"], /* CJK Ideograph Extension A */
  ["\u422A", "\u422A"], /* CJK Ideograph Extension A */
  ["\u422B", "\u422B"], /* CJK Ideograph Extension A */
  ["\u422C", "\u422C"], /* CJK Ideograph Extension A */
  ["\u422D", "\u422D"], /* CJK Ideograph Extension A */
  ["\u422E", "\u422E"], /* CJK Ideograph Extension A */
  ["\u422F", "\u422F"], /* CJK Ideograph Extension A */
  ["\u4230", "\u4230"], /* CJK Ideograph Extension A */
  ["\u4231", "\u4231"], /* CJK Ideograph Extension A */
  ["\u4232", "\u4232"], /* CJK Ideograph Extension A */
  ["\u4233", "\u4233"], /* CJK Ideograph Extension A */
  ["\u4234", "\u4234"], /* CJK Ideograph Extension A */
  ["\u4235", "\u4235"], /* CJK Ideograph Extension A */
  ["\u4236", "\u4236"], /* CJK Ideograph Extension A */
  ["\u4237", "\u4237"], /* CJK Ideograph Extension A */
  ["\u4238", "\u4238"], /* CJK Ideograph Extension A */
  ["\u4239", "\u4239"], /* CJK Ideograph Extension A */
  ["\u423A", "\u423A"], /* CJK Ideograph Extension A */
  ["\u423B", "\u423B"], /* CJK Ideograph Extension A */
  ["\u423C", "\u423C"], /* CJK Ideograph Extension A */
  ["\u423D", "\u423D"], /* CJK Ideograph Extension A */
  ["\u423E", "\u423E"], /* CJK Ideograph Extension A */
  ["\u423F", "\u423F"], /* CJK Ideograph Extension A */
  ["\u4240", "\u4240"], /* CJK Ideograph Extension A */
  ["\u4241", "\u4241"], /* CJK Ideograph Extension A */
  ["\u4242", "\u4242"], /* CJK Ideograph Extension A */
  ["\u4243", "\u4243"], /* CJK Ideograph Extension A */
  ["\u4244", "\u4244"], /* CJK Ideograph Extension A */
  ["\u4245", "\u4245"], /* CJK Ideograph Extension A */
  ["\u4246", "\u4246"], /* CJK Ideograph Extension A */
  ["\u4247", "\u4247"], /* CJK Ideograph Extension A */
  ["\u4248", "\u4248"], /* CJK Ideograph Extension A */
  ["\u4249", "\u4249"], /* CJK Ideograph Extension A */
  ["\u424A", "\u424A"], /* CJK Ideograph Extension A */
  ["\u424B", "\u424B"], /* CJK Ideograph Extension A */
  ["\u424C", "\u424C"], /* CJK Ideograph Extension A */
  ["\u424D", "\u424D"], /* CJK Ideograph Extension A */
  ["\u424E", "\u424E"], /* CJK Ideograph Extension A */
  ["\u424F", "\u424F"], /* CJK Ideograph Extension A */
  ["\u4250", "\u4250"], /* CJK Ideograph Extension A */
  ["\u4251", "\u4251"], /* CJK Ideograph Extension A */
  ["\u4252", "\u4252"], /* CJK Ideograph Extension A */
  ["\u4253", "\u4253"], /* CJK Ideograph Extension A */
  ["\u4254", "\u4254"], /* CJK Ideograph Extension A */
  ["\u4255", "\u4255"], /* CJK Ideograph Extension A */
  ["\u4256", "\u4256"], /* CJK Ideograph Extension A */
  ["\u4257", "\u4257"], /* CJK Ideograph Extension A */
  ["\u4258", "\u4258"], /* CJK Ideograph Extension A */
  ["\u4259", "\u4259"], /* CJK Ideograph Extension A */
  ["\u425A", "\u425A"], /* CJK Ideograph Extension A */
  ["\u425B", "\u425B"], /* CJK Ideograph Extension A */
  ["\u425C", "\u425C"], /* CJK Ideograph Extension A */
  ["\u425D", "\u425D"], /* CJK Ideograph Extension A */
  ["\u425E", "\u425E"], /* CJK Ideograph Extension A */
  ["\u425F", "\u425F"], /* CJK Ideograph Extension A */
  ["\u4260", "\u4260"], /* CJK Ideograph Extension A */
  ["\u4261", "\u4261"], /* CJK Ideograph Extension A */
  ["\u4262", "\u4262"], /* CJK Ideograph Extension A */
  ["\u4263", "\u4263"], /* CJK Ideograph Extension A */
  ["\u4264", "\u4264"], /* CJK Ideograph Extension A */
  ["\u4265", "\u4265"], /* CJK Ideograph Extension A */
  ["\u4266", "\u4266"], /* CJK Ideograph Extension A */
  ["\u4267", "\u4267"], /* CJK Ideograph Extension A */
  ["\u4268", "\u4268"], /* CJK Ideograph Extension A */
  ["\u4269", "\u4269"], /* CJK Ideograph Extension A */
  ["\u426A", "\u426A"], /* CJK Ideograph Extension A */
  ["\u426B", "\u426B"], /* CJK Ideograph Extension A */
  ["\u426C", "\u426C"], /* CJK Ideograph Extension A */
  ["\u426D", "\u426D"], /* CJK Ideograph Extension A */
  ["\u426E", "\u426E"], /* CJK Ideograph Extension A */
  ["\u426F", "\u426F"], /* CJK Ideograph Extension A */
  ["\u4270", "\u4270"], /* CJK Ideograph Extension A */
  ["\u4271", "\u4271"], /* CJK Ideograph Extension A */
  ["\u4272", "\u4272"], /* CJK Ideograph Extension A */
  ["\u4273", "\u4273"], /* CJK Ideograph Extension A */
  ["\u4274", "\u4274"], /* CJK Ideograph Extension A */
  ["\u4275", "\u4275"], /* CJK Ideograph Extension A */
  ["\u4276", "\u4276"], /* CJK Ideograph Extension A */
  ["\u4277", "\u4277"], /* CJK Ideograph Extension A */
  ["\u4278", "\u4278"], /* CJK Ideograph Extension A */
  ["\u4279", "\u4279"], /* CJK Ideograph Extension A */
  ["\u427A", "\u427A"], /* CJK Ideograph Extension A */
  ["\u427B", "\u427B"], /* CJK Ideograph Extension A */
  ["\u427C", "\u427C"], /* CJK Ideograph Extension A */
  ["\u427D", "\u427D"], /* CJK Ideograph Extension A */
  ["\u427E", "\u427E"], /* CJK Ideograph Extension A */
  ["\u427F", "\u427F"], /* CJK Ideograph Extension A */
  ["\u4280", "\u4280"], /* CJK Ideograph Extension A */
  ["\u4281", "\u4281"], /* CJK Ideograph Extension A */
  ["\u4282", "\u4282"], /* CJK Ideograph Extension A */
  ["\u4283", "\u4283"], /* CJK Ideograph Extension A */
  ["\u4284", "\u4284"], /* CJK Ideograph Extension A */
  ["\u4285", "\u4285"], /* CJK Ideograph Extension A */
  ["\u4286", "\u4286"], /* CJK Ideograph Extension A */
  ["\u4287", "\u4287"], /* CJK Ideograph Extension A */
  ["\u4288", "\u4288"], /* CJK Ideograph Extension A */
  ["\u4289", "\u4289"], /* CJK Ideograph Extension A */
  ["\u428A", "\u428A"], /* CJK Ideograph Extension A */
  ["\u428B", "\u428B"], /* CJK Ideograph Extension A */
  ["\u428C", "\u428C"], /* CJK Ideograph Extension A */
  ["\u428D", "\u428D"], /* CJK Ideograph Extension A */
  ["\u428E", "\u428E"], /* CJK Ideograph Extension A */
  ["\u428F", "\u428F"], /* CJK Ideograph Extension A */
  ["\u4290", "\u4290"], /* CJK Ideograph Extension A */
  ["\u4291", "\u4291"], /* CJK Ideograph Extension A */
  ["\u4292", "\u4292"], /* CJK Ideograph Extension A */
  ["\u4293", "\u4293"], /* CJK Ideograph Extension A */
  ["\u4294", "\u4294"], /* CJK Ideograph Extension A */
  ["\u4295", "\u4295"], /* CJK Ideograph Extension A */
  ["\u4296", "\u4296"], /* CJK Ideograph Extension A */
  ["\u4297", "\u4297"], /* CJK Ideograph Extension A */
  ["\u4298", "\u4298"], /* CJK Ideograph Extension A */
  ["\u4299", "\u4299"], /* CJK Ideograph Extension A */
  ["\u429A", "\u429A"], /* CJK Ideograph Extension A */
  ["\u429B", "\u429B"], /* CJK Ideograph Extension A */
  ["\u429C", "\u429C"], /* CJK Ideograph Extension A */
  ["\u429D", "\u429D"], /* CJK Ideograph Extension A */
  ["\u429E", "\u429E"], /* CJK Ideograph Extension A */
  ["\u429F", "\u429F"], /* CJK Ideograph Extension A */
  ["\u42A0", "\u42A0"], /* CJK Ideograph Extension A */
  ["\u42A1", "\u42A1"], /* CJK Ideograph Extension A */
  ["\u42A2", "\u42A2"], /* CJK Ideograph Extension A */
  ["\u42A3", "\u42A3"], /* CJK Ideograph Extension A */
  ["\u42A4", "\u42A4"], /* CJK Ideograph Extension A */
  ["\u42A5", "\u42A5"], /* CJK Ideograph Extension A */
  ["\u42A6", "\u42A6"], /* CJK Ideograph Extension A */
  ["\u42A7", "\u42A7"], /* CJK Ideograph Extension A */
  ["\u42A8", "\u42A8"], /* CJK Ideograph Extension A */
  ["\u42A9", "\u42A9"], /* CJK Ideograph Extension A */
  ["\u42AA", "\u42AA"], /* CJK Ideograph Extension A */
  ["\u42AB", "\u42AB"], /* CJK Ideograph Extension A */
  ["\u42AC", "\u42AC"], /* CJK Ideograph Extension A */
  ["\u42AD", "\u42AD"], /* CJK Ideograph Extension A */
  ["\u42AE", "\u42AE"], /* CJK Ideograph Extension A */
  ["\u42AF", "\u42AF"], /* CJK Ideograph Extension A */
  ["\u42B0", "\u42B0"], /* CJK Ideograph Extension A */
  ["\u42B1", "\u42B1"], /* CJK Ideograph Extension A */
  ["\u42B2", "\u42B2"], /* CJK Ideograph Extension A */
  ["\u42B3", "\u42B3"], /* CJK Ideograph Extension A */
  ["\u42B4", "\u42B4"], /* CJK Ideograph Extension A */
  ["\u42B5", "\u42B5"], /* CJK Ideograph Extension A */
  ["\u42B6", "\u42B6"], /* CJK Ideograph Extension A */
  ["\u42B7", "\u42B7"], /* CJK Ideograph Extension A */
  ["\u42B8", "\u42B8"], /* CJK Ideograph Extension A */
  ["\u42B9", "\u42B9"], /* CJK Ideograph Extension A */
  ["\u42BA", "\u42BA"], /* CJK Ideograph Extension A */
  ["\u42BB", "\u42BB"], /* CJK Ideograph Extension A */
  ["\u42BC", "\u42BC"], /* CJK Ideograph Extension A */
  ["\u42BD", "\u42BD"], /* CJK Ideograph Extension A */
  ["\u42BE", "\u42BE"], /* CJK Ideograph Extension A */
  ["\u42BF", "\u42BF"], /* CJK Ideograph Extension A */
  ["\u42C0", "\u42C0"], /* CJK Ideograph Extension A */
  ["\u42C1", "\u42C1"], /* CJK Ideograph Extension A */
  ["\u42C2", "\u42C2"], /* CJK Ideograph Extension A */
  ["\u42C3", "\u42C3"], /* CJK Ideograph Extension A */
  ["\u42C4", "\u42C4"], /* CJK Ideograph Extension A */
  ["\u42C5", "\u42C5"], /* CJK Ideograph Extension A */
  ["\u42C6", "\u42C6"], /* CJK Ideograph Extension A */
  ["\u42C7", "\u42C7"], /* CJK Ideograph Extension A */
  ["\u42C8", "\u42C8"], /* CJK Ideograph Extension A */
  ["\u42C9", "\u42C9"], /* CJK Ideograph Extension A */
  ["\u42CA", "\u42CA"], /* CJK Ideograph Extension A */
  ["\u42CB", "\u42CB"], /* CJK Ideograph Extension A */
  ["\u42CC", "\u42CC"], /* CJK Ideograph Extension A */
  ["\u42CD", "\u42CD"], /* CJK Ideograph Extension A */
  ["\u42CE", "\u42CE"], /* CJK Ideograph Extension A */
  ["\u42CF", "\u42CF"], /* CJK Ideograph Extension A */
  ["\u42D0", "\u42D0"], /* CJK Ideograph Extension A */
  ["\u42D1", "\u42D1"], /* CJK Ideograph Extension A */
  ["\u42D2", "\u42D2"], /* CJK Ideograph Extension A */
  ["\u42D3", "\u42D3"], /* CJK Ideograph Extension A */
  ["\u42D4", "\u42D4"], /* CJK Ideograph Extension A */
  ["\u42D5", "\u42D5"], /* CJK Ideograph Extension A */
  ["\u42D6", "\u42D6"], /* CJK Ideograph Extension A */
  ["\u42D7", "\u42D7"], /* CJK Ideograph Extension A */
  ["\u42D8", "\u42D8"], /* CJK Ideograph Extension A */
  ["\u42D9", "\u42D9"], /* CJK Ideograph Extension A */
  ["\u42DA", "\u42DA"], /* CJK Ideograph Extension A */
  ["\u42DB", "\u42DB"], /* CJK Ideograph Extension A */
  ["\u42DC", "\u42DC"], /* CJK Ideograph Extension A */
  ["\u42DD", "\u42DD"], /* CJK Ideograph Extension A */
  ["\u42DE", "\u42DE"], /* CJK Ideograph Extension A */
  ["\u42DF", "\u42DF"], /* CJK Ideograph Extension A */
  ["\u42E0", "\u42E0"], /* CJK Ideograph Extension A */
  ["\u42E1", "\u42E1"], /* CJK Ideograph Extension A */
  ["\u42E2", "\u42E2"], /* CJK Ideograph Extension A */
  ["\u42E3", "\u42E3"], /* CJK Ideograph Extension A */
  ["\u42E4", "\u42E4"], /* CJK Ideograph Extension A */
  ["\u42E5", "\u42E5"], /* CJK Ideograph Extension A */
  ["\u42E6", "\u42E6"], /* CJK Ideograph Extension A */
  ["\u42E7", "\u42E7"], /* CJK Ideograph Extension A */
  ["\u42E8", "\u42E8"], /* CJK Ideograph Extension A */
  ["\u42E9", "\u42E9"], /* CJK Ideograph Extension A */
  ["\u42EA", "\u42EA"], /* CJK Ideograph Extension A */
  ["\u42EB", "\u42EB"], /* CJK Ideograph Extension A */
  ["\u42EC", "\u42EC"], /* CJK Ideograph Extension A */
  ["\u42ED", "\u42ED"], /* CJK Ideograph Extension A */
  ["\u42EE", "\u42EE"], /* CJK Ideograph Extension A */
  ["\u42EF", "\u42EF"], /* CJK Ideograph Extension A */
  ["\u42F0", "\u42F0"], /* CJK Ideograph Extension A */
  ["\u42F1", "\u42F1"], /* CJK Ideograph Extension A */
  ["\u42F2", "\u42F2"], /* CJK Ideograph Extension A */
  ["\u42F3", "\u42F3"], /* CJK Ideograph Extension A */
  ["\u42F4", "\u42F4"], /* CJK Ideograph Extension A */
  ["\u42F5", "\u42F5"], /* CJK Ideograph Extension A */
  ["\u42F6", "\u42F6"], /* CJK Ideograph Extension A */
  ["\u42F7", "\u42F7"], /* CJK Ideograph Extension A */
  ["\u42F8", "\u42F8"], /* CJK Ideograph Extension A */
  ["\u42F9", "\u42F9"], /* CJK Ideograph Extension A */
  ["\u42FA", "\u42FA"], /* CJK Ideograph Extension A */
  ["\u42FB", "\u42FB"], /* CJK Ideograph Extension A */
  ["\u42FC", "\u42FC"], /* CJK Ideograph Extension A */
  ["\u42FD", "\u42FD"], /* CJK Ideograph Extension A */
  ["\u42FE", "\u42FE"], /* CJK Ideograph Extension A */
  ["\u42FF", "\u42FF"], /* CJK Ideograph Extension A */
  ["\u4300", "\u4300"], /* CJK Ideograph Extension A */
  ["\u4301", "\u4301"], /* CJK Ideograph Extension A */
  ["\u4302", "\u4302"], /* CJK Ideograph Extension A */
  ["\u4303", "\u4303"], /* CJK Ideograph Extension A */
  ["\u4304", "\u4304"], /* CJK Ideograph Extension A */
  ["\u4305", "\u4305"], /* CJK Ideograph Extension A */
  ["\u4306", "\u4306"], /* CJK Ideograph Extension A */
  ["\u4307", "\u4307"], /* CJK Ideograph Extension A */
  ["\u4308", "\u4308"], /* CJK Ideograph Extension A */
  ["\u4309", "\u4309"], /* CJK Ideograph Extension A */
  ["\u430A", "\u430A"], /* CJK Ideograph Extension A */
  ["\u430B", "\u430B"], /* CJK Ideograph Extension A */
  ["\u430C", "\u430C"], /* CJK Ideograph Extension A */
  ["\u430D", "\u430D"], /* CJK Ideograph Extension A */
  ["\u430E", "\u430E"], /* CJK Ideograph Extension A */
  ["\u430F", "\u430F"], /* CJK Ideograph Extension A */
  ["\u4310", "\u4310"], /* CJK Ideograph Extension A */
  ["\u4311", "\u4311"], /* CJK Ideograph Extension A */
  ["\u4312", "\u4312"], /* CJK Ideograph Extension A */
  ["\u4313", "\u4313"], /* CJK Ideograph Extension A */
  ["\u4314", "\u4314"], /* CJK Ideograph Extension A */
  ["\u4315", "\u4315"], /* CJK Ideograph Extension A */
  ["\u4316", "\u4316"], /* CJK Ideograph Extension A */
  ["\u4317", "\u4317"], /* CJK Ideograph Extension A */
  ["\u4318", "\u4318"], /* CJK Ideograph Extension A */
  ["\u4319", "\u4319"], /* CJK Ideograph Extension A */
  ["\u431A", "\u431A"], /* CJK Ideograph Extension A */
  ["\u431B", "\u431B"], /* CJK Ideograph Extension A */
  ["\u431C", "\u431C"], /* CJK Ideograph Extension A */
  ["\u431D", "\u431D"], /* CJK Ideograph Extension A */
  ["\u431E", "\u431E"], /* CJK Ideograph Extension A */
  ["\u431F", "\u431F"], /* CJK Ideograph Extension A */
  ["\u4320", "\u4320"], /* CJK Ideograph Extension A */
  ["\u4321", "\u4321"], /* CJK Ideograph Extension A */
  ["\u4322", "\u4322"], /* CJK Ideograph Extension A */
  ["\u4323", "\u4323"], /* CJK Ideograph Extension A */
  ["\u4324", "\u4324"], /* CJK Ideograph Extension A */
  ["\u4325", "\u4325"], /* CJK Ideograph Extension A */
  ["\u4326", "\u4326"], /* CJK Ideograph Extension A */
  ["\u4327", "\u4327"], /* CJK Ideograph Extension A */
  ["\u4328", "\u4328"], /* CJK Ideograph Extension A */
  ["\u4329", "\u4329"], /* CJK Ideograph Extension A */
  ["\u432A", "\u432A"], /* CJK Ideograph Extension A */
  ["\u432B", "\u432B"], /* CJK Ideograph Extension A */
  ["\u432C", "\u432C"], /* CJK Ideograph Extension A */
  ["\u432D", "\u432D"], /* CJK Ideograph Extension A */
  ["\u432E", "\u432E"], /* CJK Ideograph Extension A */
  ["\u432F", "\u432F"], /* CJK Ideograph Extension A */
  ["\u4330", "\u4330"], /* CJK Ideograph Extension A */
  ["\u4331", "\u4331"], /* CJK Ideograph Extension A */
  ["\u4332", "\u4332"], /* CJK Ideograph Extension A */
  ["\u4333", "\u4333"], /* CJK Ideograph Extension A */
  ["\u4334", "\u4334"], /* CJK Ideograph Extension A */
  ["\u4335", "\u4335"], /* CJK Ideograph Extension A */
  ["\u4336", "\u4336"], /* CJK Ideograph Extension A */
  ["\u4337", "\u4337"], /* CJK Ideograph Extension A */
  ["\u4338", "\u4338"], /* CJK Ideograph Extension A */
  ["\u4339", "\u4339"], /* CJK Ideograph Extension A */
  ["\u433A", "\u433A"], /* CJK Ideograph Extension A */
  ["\u433B", "\u433B"], /* CJK Ideograph Extension A */
  ["\u433C", "\u433C"], /* CJK Ideograph Extension A */
  ["\u433D", "\u433D"], /* CJK Ideograph Extension A */
  ["\u433E", "\u433E"], /* CJK Ideograph Extension A */
  ["\u433F", "\u433F"], /* CJK Ideograph Extension A */
  ["\u4340", "\u4340"], /* CJK Ideograph Extension A */
  ["\u4341", "\u4341"], /* CJK Ideograph Extension A */
  ["\u4342", "\u4342"], /* CJK Ideograph Extension A */
  ["\u4343", "\u4343"], /* CJK Ideograph Extension A */
  ["\u4344", "\u4344"], /* CJK Ideograph Extension A */
  ["\u4345", "\u4345"], /* CJK Ideograph Extension A */
  ["\u4346", "\u4346"], /* CJK Ideograph Extension A */
  ["\u4347", "\u4347"], /* CJK Ideograph Extension A */
  ["\u4348", "\u4348"], /* CJK Ideograph Extension A */
  ["\u4349", "\u4349"], /* CJK Ideograph Extension A */
  ["\u434A", "\u434A"], /* CJK Ideograph Extension A */
  ["\u434B", "\u434B"], /* CJK Ideograph Extension A */
  ["\u434C", "\u434C"], /* CJK Ideograph Extension A */
  ["\u434D", "\u434D"], /* CJK Ideograph Extension A */
  ["\u434E", "\u434E"], /* CJK Ideograph Extension A */
  ["\u434F", "\u434F"], /* CJK Ideograph Extension A */
  ["\u4350", "\u4350"], /* CJK Ideograph Extension A */
  ["\u4351", "\u4351"], /* CJK Ideograph Extension A */
  ["\u4352", "\u4352"], /* CJK Ideograph Extension A */
  ["\u4353", "\u4353"], /* CJK Ideograph Extension A */
  ["\u4354", "\u4354"], /* CJK Ideograph Extension A */
  ["\u4355", "\u4355"], /* CJK Ideograph Extension A */
  ["\u4356", "\u4356"], /* CJK Ideograph Extension A */
  ["\u4357", "\u4357"], /* CJK Ideograph Extension A */
  ["\u4358", "\u4358"], /* CJK Ideograph Extension A */
  ["\u4359", "\u4359"], /* CJK Ideograph Extension A */
  ["\u435A", "\u435A"], /* CJK Ideograph Extension A */
  ["\u435B", "\u435B"], /* CJK Ideograph Extension A */
  ["\u435C", "\u435C"], /* CJK Ideograph Extension A */
  ["\u435D", "\u435D"], /* CJK Ideograph Extension A */
  ["\u435E", "\u435E"], /* CJK Ideograph Extension A */
  ["\u435F", "\u435F"], /* CJK Ideograph Extension A */
  ["\u4360", "\u4360"], /* CJK Ideograph Extension A */
  ["\u4361", "\u4361"], /* CJK Ideograph Extension A */
  ["\u4362", "\u4362"], /* CJK Ideograph Extension A */
  ["\u4363", "\u4363"], /* CJK Ideograph Extension A */
  ["\u4364", "\u4364"], /* CJK Ideograph Extension A */
  ["\u4365", "\u4365"], /* CJK Ideograph Extension A */
  ["\u4366", "\u4366"], /* CJK Ideograph Extension A */
  ["\u4367", "\u4367"], /* CJK Ideograph Extension A */
  ["\u4368", "\u4368"], /* CJK Ideograph Extension A */
  ["\u4369", "\u4369"], /* CJK Ideograph Extension A */
  ["\u436A", "\u436A"], /* CJK Ideograph Extension A */
  ["\u436B", "\u436B"], /* CJK Ideograph Extension A */
  ["\u436C", "\u436C"], /* CJK Ideograph Extension A */
  ["\u436D", "\u436D"], /* CJK Ideograph Extension A */
  ["\u436E", "\u436E"], /* CJK Ideograph Extension A */
  ["\u436F", "\u436F"], /* CJK Ideograph Extension A */
  ["\u4370", "\u4370"], /* CJK Ideograph Extension A */
  ["\u4371", "\u4371"], /* CJK Ideograph Extension A */
  ["\u4372", "\u4372"], /* CJK Ideograph Extension A */
  ["\u4373", "\u4373"], /* CJK Ideograph Extension A */
  ["\u4374", "\u4374"], /* CJK Ideograph Extension A */
  ["\u4375", "\u4375"], /* CJK Ideograph Extension A */
  ["\u4376", "\u4376"], /* CJK Ideograph Extension A */
  ["\u4377", "\u4377"], /* CJK Ideograph Extension A */
  ["\u4378", "\u4378"], /* CJK Ideograph Extension A */
  ["\u4379", "\u4379"], /* CJK Ideograph Extension A */
  ["\u437A", "\u437A"], /* CJK Ideograph Extension A */
  ["\u437B", "\u437B"], /* CJK Ideograph Extension A */
  ["\u437C", "\u437C"], /* CJK Ideograph Extension A */
  ["\u437D", "\u437D"], /* CJK Ideograph Extension A */
  ["\u437E", "\u437E"], /* CJK Ideograph Extension A */
  ["\u437F", "\u437F"], /* CJK Ideograph Extension A */
  ["\u4380", "\u4380"], /* CJK Ideograph Extension A */
  ["\u4381", "\u4381"], /* CJK Ideograph Extension A */
  ["\u4382", "\u4382"], /* CJK Ideograph Extension A */
  ["\u4383", "\u4383"], /* CJK Ideograph Extension A */
  ["\u4384", "\u4384"], /* CJK Ideograph Extension A */
  ["\u4385", "\u4385"], /* CJK Ideograph Extension A */
  ["\u4386", "\u4386"], /* CJK Ideograph Extension A */
  ["\u4387", "\u4387"], /* CJK Ideograph Extension A */
  ["\u4388", "\u4388"], /* CJK Ideograph Extension A */
  ["\u4389", "\u4389"], /* CJK Ideograph Extension A */
  ["\u438A", "\u438A"], /* CJK Ideograph Extension A */
  ["\u438B", "\u438B"], /* CJK Ideograph Extension A */
  ["\u438C", "\u438C"], /* CJK Ideograph Extension A */
  ["\u438D", "\u438D"], /* CJK Ideograph Extension A */
  ["\u438E", "\u438E"], /* CJK Ideograph Extension A */
  ["\u438F", "\u438F"], /* CJK Ideograph Extension A */
  ["\u4390", "\u4390"], /* CJK Ideograph Extension A */
  ["\u4391", "\u4391"], /* CJK Ideograph Extension A */
  ["\u4392", "\u4392"], /* CJK Ideograph Extension A */
  ["\u4393", "\u4393"], /* CJK Ideograph Extension A */
  ["\u4394", "\u4394"], /* CJK Ideograph Extension A */
  ["\u4395", "\u4395"], /* CJK Ideograph Extension A */
  ["\u4396", "\u4396"], /* CJK Ideograph Extension A */
  ["\u4397", "\u4397"], /* CJK Ideograph Extension A */
  ["\u4398", "\u4398"], /* CJK Ideograph Extension A */
  ["\u4399", "\u4399"], /* CJK Ideograph Extension A */
  ["\u439A", "\u439A"], /* CJK Ideograph Extension A */
  ["\u439B", "\u439B"], /* CJK Ideograph Extension A */
  ["\u439C", "\u439C"], /* CJK Ideograph Extension A */
  ["\u439D", "\u439D"], /* CJK Ideograph Extension A */
  ["\u439E", "\u439E"], /* CJK Ideograph Extension A */
  ["\u439F", "\u439F"], /* CJK Ideograph Extension A */
  ["\u43A0", "\u43A0"], /* CJK Ideograph Extension A */
  ["\u43A1", "\u43A1"], /* CJK Ideograph Extension A */
  ["\u43A2", "\u43A2"], /* CJK Ideograph Extension A */
  ["\u43A3", "\u43A3"], /* CJK Ideograph Extension A */
  ["\u43A4", "\u43A4"], /* CJK Ideograph Extension A */
  ["\u43A5", "\u43A5"], /* CJK Ideograph Extension A */
  ["\u43A6", "\u43A6"], /* CJK Ideograph Extension A */
  ["\u43A7", "\u43A7"], /* CJK Ideograph Extension A */
  ["\u43A8", "\u43A8"], /* CJK Ideograph Extension A */
  ["\u43A9", "\u43A9"], /* CJK Ideograph Extension A */
  ["\u43AA", "\u43AA"], /* CJK Ideograph Extension A */
  ["\u43AB", "\u43AB"], /* CJK Ideograph Extension A */
  ["\u43AC", "\u43AC"], /* CJK Ideograph Extension A */
  ["\u43AD", "\u43AD"], /* CJK Ideograph Extension A */
  ["\u43AE", "\u43AE"], /* CJK Ideograph Extension A */
  ["\u43AF", "\u43AF"], /* CJK Ideograph Extension A */
  ["\u43B0", "\u43B0"], /* CJK Ideograph Extension A */
  ["\u43B1", "\u43B1"], /* CJK Ideograph Extension A */
  ["\u43B2", "\u43B2"], /* CJK Ideograph Extension A */
  ["\u43B3", "\u43B3"], /* CJK Ideograph Extension A */
  ["\u43B4", "\u43B4"], /* CJK Ideograph Extension A */
  ["\u43B5", "\u43B5"], /* CJK Ideograph Extension A */
  ["\u43B6", "\u43B6"], /* CJK Ideograph Extension A */
  ["\u43B7", "\u43B7"], /* CJK Ideograph Extension A */
  ["\u43B8", "\u43B8"], /* CJK Ideograph Extension A */
  ["\u43B9", "\u43B9"], /* CJK Ideograph Extension A */
  ["\u43BA", "\u43BA"], /* CJK Ideograph Extension A */
  ["\u43BB", "\u43BB"], /* CJK Ideograph Extension A */
  ["\u43BC", "\u43BC"], /* CJK Ideograph Extension A */
  ["\u43BD", "\u43BD"], /* CJK Ideograph Extension A */
  ["\u43BE", "\u43BE"], /* CJK Ideograph Extension A */
  ["\u43BF", "\u43BF"], /* CJK Ideograph Extension A */
  ["\u43C0", "\u43C0"], /* CJK Ideograph Extension A */
  ["\u43C1", "\u43C1"], /* CJK Ideograph Extension A */
  ["\u43C2", "\u43C2"], /* CJK Ideograph Extension A */
  ["\u43C3", "\u43C3"], /* CJK Ideograph Extension A */
  ["\u43C4", "\u43C4"], /* CJK Ideograph Extension A */
  ["\u43C5", "\u43C5"], /* CJK Ideograph Extension A */
  ["\u43C6", "\u43C6"], /* CJK Ideograph Extension A */
  ["\u43C7", "\u43C7"], /* CJK Ideograph Extension A */
  ["\u43C8", "\u43C8"], /* CJK Ideograph Extension A */
  ["\u43C9", "\u43C9"], /* CJK Ideograph Extension A */
  ["\u43CA", "\u43CA"], /* CJK Ideograph Extension A */
  ["\u43CB", "\u43CB"], /* CJK Ideograph Extension A */
  ["\u43CC", "\u43CC"], /* CJK Ideograph Extension A */
  ["\u43CD", "\u43CD"], /* CJK Ideograph Extension A */
  ["\u43CE", "\u43CE"], /* CJK Ideograph Extension A */
  ["\u43CF", "\u43CF"], /* CJK Ideograph Extension A */
  ["\u43D0", "\u43D0"], /* CJK Ideograph Extension A */
  ["\u43D1", "\u43D1"], /* CJK Ideograph Extension A */
  ["\u43D2", "\u43D2"], /* CJK Ideograph Extension A */
  ["\u43D3", "\u43D3"], /* CJK Ideograph Extension A */
  ["\u43D4", "\u43D4"], /* CJK Ideograph Extension A */
  ["\u43D5", "\u43D5"], /* CJK Ideograph Extension A */
  ["\u43D6", "\u43D6"], /* CJK Ideograph Extension A */
  ["\u43D7", "\u43D7"], /* CJK Ideograph Extension A */
  ["\u43D8", "\u43D8"], /* CJK Ideograph Extension A */
  ["\u43D9", "\u43D9"], /* CJK Ideograph Extension A */
  ["\u43DA", "\u43DA"], /* CJK Ideograph Extension A */
  ["\u43DB", "\u43DB"], /* CJK Ideograph Extension A */
  ["\u43DC", "\u43DC"], /* CJK Ideograph Extension A */
  ["\u43DD", "\u43DD"], /* CJK Ideograph Extension A */
  ["\u43DE", "\u43DE"], /* CJK Ideograph Extension A */
  ["\u43DF", "\u43DF"], /* CJK Ideograph Extension A */
  ["\u43E0", "\u43E0"], /* CJK Ideograph Extension A */
  ["\u43E1", "\u43E1"], /* CJK Ideograph Extension A */
  ["\u43E2", "\u43E2"], /* CJK Ideograph Extension A */
  ["\u43E3", "\u43E3"], /* CJK Ideograph Extension A */
  ["\u43E4", "\u43E4"], /* CJK Ideograph Extension A */
  ["\u43E5", "\u43E5"], /* CJK Ideograph Extension A */
  ["\u43E6", "\u43E6"], /* CJK Ideograph Extension A */
  ["\u43E7", "\u43E7"], /* CJK Ideograph Extension A */
  ["\u43E8", "\u43E8"], /* CJK Ideograph Extension A */
  ["\u43E9", "\u43E9"], /* CJK Ideograph Extension A */
  ["\u43EA", "\u43EA"], /* CJK Ideograph Extension A */
  ["\u43EB", "\u43EB"], /* CJK Ideograph Extension A */
  ["\u43EC", "\u43EC"], /* CJK Ideograph Extension A */
  ["\u43ED", "\u43ED"], /* CJK Ideograph Extension A */
  ["\u43EE", "\u43EE"], /* CJK Ideograph Extension A */
  ["\u43EF", "\u43EF"], /* CJK Ideograph Extension A */
  ["\u43F0", "\u43F0"], /* CJK Ideograph Extension A */
  ["\u43F1", "\u43F1"], /* CJK Ideograph Extension A */
  ["\u43F2", "\u43F2"], /* CJK Ideograph Extension A */
  ["\u43F3", "\u43F3"], /* CJK Ideograph Extension A */
  ["\u43F4", "\u43F4"], /* CJK Ideograph Extension A */
  ["\u43F5", "\u43F5"], /* CJK Ideograph Extension A */
  ["\u43F6", "\u43F6"], /* CJK Ideograph Extension A */
  ["\u43F7", "\u43F7"], /* CJK Ideograph Extension A */
  ["\u43F8", "\u43F8"], /* CJK Ideograph Extension A */
  ["\u43F9", "\u43F9"], /* CJK Ideograph Extension A */
  ["\u43FA", "\u43FA"], /* CJK Ideograph Extension A */
  ["\u43FB", "\u43FB"], /* CJK Ideograph Extension A */
  ["\u43FC", "\u43FC"], /* CJK Ideograph Extension A */
  ["\u43FD", "\u43FD"], /* CJK Ideograph Extension A */
  ["\u43FE", "\u43FE"], /* CJK Ideograph Extension A */
  ["\u43FF", "\u43FF"], /* CJK Ideograph Extension A */
  ["\u4400", "\u4400"], /* CJK Ideograph Extension A */
  ["\u4401", "\u4401"], /* CJK Ideograph Extension A */
  ["\u4402", "\u4402"], /* CJK Ideograph Extension A */
  ["\u4403", "\u4403"], /* CJK Ideograph Extension A */
  ["\u4404", "\u4404"], /* CJK Ideograph Extension A */
  ["\u4405", "\u4405"], /* CJK Ideograph Extension A */
  ["\u4406", "\u4406"], /* CJK Ideograph Extension A */
  ["\u4407", "\u4407"], /* CJK Ideograph Extension A */
  ["\u4408", "\u4408"], /* CJK Ideograph Extension A */
  ["\u4409", "\u4409"], /* CJK Ideograph Extension A */
  ["\u440A", "\u440A"], /* CJK Ideograph Extension A */
  ["\u440B", "\u440B"], /* CJK Ideograph Extension A */
  ["\u440C", "\u440C"], /* CJK Ideograph Extension A */
  ["\u440D", "\u440D"], /* CJK Ideograph Extension A */
  ["\u440E", "\u440E"], /* CJK Ideograph Extension A */
  ["\u440F", "\u440F"], /* CJK Ideograph Extension A */
  ["\u4410", "\u4410"], /* CJK Ideograph Extension A */
  ["\u4411", "\u4411"], /* CJK Ideograph Extension A */
  ["\u4412", "\u4412"], /* CJK Ideograph Extension A */
  ["\u4413", "\u4413"], /* CJK Ideograph Extension A */
  ["\u4414", "\u4414"], /* CJK Ideograph Extension A */
  ["\u4415", "\u4415"], /* CJK Ideograph Extension A */
  ["\u4416", "\u4416"], /* CJK Ideograph Extension A */
  ["\u4417", "\u4417"], /* CJK Ideograph Extension A */
  ["\u4418", "\u4418"], /* CJK Ideograph Extension A */
  ["\u4419", "\u4419"], /* CJK Ideograph Extension A */
  ["\u441A", "\u441A"], /* CJK Ideograph Extension A */
  ["\u441B", "\u441B"], /* CJK Ideograph Extension A */
  ["\u441C", "\u441C"], /* CJK Ideograph Extension A */
  ["\u441D", "\u441D"], /* CJK Ideograph Extension A */
  ["\u441E", "\u441E"], /* CJK Ideograph Extension A */
  ["\u441F", "\u441F"], /* CJK Ideograph Extension A */
  ["\u4420", "\u4420"], /* CJK Ideograph Extension A */
  ["\u4421", "\u4421"], /* CJK Ideograph Extension A */
  ["\u4422", "\u4422"], /* CJK Ideograph Extension A */
  ["\u4423", "\u4423"], /* CJK Ideograph Extension A */
  ["\u4424", "\u4424"], /* CJK Ideograph Extension A */
  ["\u4425", "\u4425"], /* CJK Ideograph Extension A */
  ["\u4426", "\u4426"], /* CJK Ideograph Extension A */
  ["\u4427", "\u4427"], /* CJK Ideograph Extension A */
  ["\u4428", "\u4428"], /* CJK Ideograph Extension A */
  ["\u4429", "\u4429"], /* CJK Ideograph Extension A */
  ["\u442A", "\u442A"], /* CJK Ideograph Extension A */
  ["\u442B", "\u442B"], /* CJK Ideograph Extension A */
  ["\u442C", "\u442C"], /* CJK Ideograph Extension A */
  ["\u442D", "\u442D"], /* CJK Ideograph Extension A */
  ["\u442E", "\u442E"], /* CJK Ideograph Extension A */
  ["\u442F", "\u442F"], /* CJK Ideograph Extension A */
  ["\u4430", "\u4430"], /* CJK Ideograph Extension A */
  ["\u4431", "\u4431"], /* CJK Ideograph Extension A */
  ["\u4432", "\u4432"], /* CJK Ideograph Extension A */
  ["\u4433", "\u4433"], /* CJK Ideograph Extension A */
  ["\u4434", "\u4434"], /* CJK Ideograph Extension A */
  ["\u4435", "\u4435"], /* CJK Ideograph Extension A */
  ["\u4436", "\u4436"], /* CJK Ideograph Extension A */
  ["\u4437", "\u4437"], /* CJK Ideograph Extension A */
  ["\u4438", "\u4438"], /* CJK Ideograph Extension A */
  ["\u4439", "\u4439"], /* CJK Ideograph Extension A */
  ["\u443A", "\u443A"], /* CJK Ideograph Extension A */
  ["\u443B", "\u443B"], /* CJK Ideograph Extension A */
  ["\u443C", "\u443C"], /* CJK Ideograph Extension A */
  ["\u443D", "\u443D"], /* CJK Ideograph Extension A */
  ["\u443E", "\u443E"], /* CJK Ideograph Extension A */
  ["\u443F", "\u443F"], /* CJK Ideograph Extension A */
  ["\u4440", "\u4440"], /* CJK Ideograph Extension A */
  ["\u4441", "\u4441"], /* CJK Ideograph Extension A */
  ["\u4442", "\u4442"], /* CJK Ideograph Extension A */
  ["\u4443", "\u4443"], /* CJK Ideograph Extension A */
  ["\u4444", "\u4444"], /* CJK Ideograph Extension A */
  ["\u4445", "\u4445"], /* CJK Ideograph Extension A */
  ["\u4446", "\u4446"], /* CJK Ideograph Extension A */
  ["\u4447", "\u4447"], /* CJK Ideograph Extension A */
  ["\u4448", "\u4448"], /* CJK Ideograph Extension A */
  ["\u4449", "\u4449"], /* CJK Ideograph Extension A */
  ["\u444A", "\u444A"], /* CJK Ideograph Extension A */
  ["\u444B", "\u444B"], /* CJK Ideograph Extension A */
  ["\u444C", "\u444C"], /* CJK Ideograph Extension A */
  ["\u444D", "\u444D"], /* CJK Ideograph Extension A */
  ["\u444E", "\u444E"], /* CJK Ideograph Extension A */
  ["\u444F", "\u444F"], /* CJK Ideograph Extension A */
  ["\u4450", "\u4450"], /* CJK Ideograph Extension A */
  ["\u4451", "\u4451"], /* CJK Ideograph Extension A */
  ["\u4452", "\u4452"], /* CJK Ideograph Extension A */
  ["\u4453", "\u4453"], /* CJK Ideograph Extension A */
  ["\u4454", "\u4454"], /* CJK Ideograph Extension A */
  ["\u4455", "\u4455"], /* CJK Ideograph Extension A */
  ["\u4456", "\u4456"], /* CJK Ideograph Extension A */
  ["\u4457", "\u4457"], /* CJK Ideograph Extension A */
  ["\u4458", "\u4458"], /* CJK Ideograph Extension A */
  ["\u4459", "\u4459"], /* CJK Ideograph Extension A */
  ["\u445A", "\u445A"], /* CJK Ideograph Extension A */
  ["\u445B", "\u445B"], /* CJK Ideograph Extension A */
  ["\u445C", "\u445C"], /* CJK Ideograph Extension A */
  ["\u445D", "\u445D"], /* CJK Ideograph Extension A */
  ["\u445E", "\u445E"], /* CJK Ideograph Extension A */
  ["\u445F", "\u445F"], /* CJK Ideograph Extension A */
  ["\u4460", "\u4460"], /* CJK Ideograph Extension A */
  ["\u4461", "\u4461"], /* CJK Ideograph Extension A */
  ["\u4462", "\u4462"], /* CJK Ideograph Extension A */
  ["\u4463", "\u4463"], /* CJK Ideograph Extension A */
  ["\u4464", "\u4464"], /* CJK Ideograph Extension A */
  ["\u4465", "\u4465"], /* CJK Ideograph Extension A */
  ["\u4466", "\u4466"], /* CJK Ideograph Extension A */
  ["\u4467", "\u4467"], /* CJK Ideograph Extension A */
  ["\u4468", "\u4468"], /* CJK Ideograph Extension A */
  ["\u4469", "\u4469"], /* CJK Ideograph Extension A */
  ["\u446A", "\u446A"], /* CJK Ideograph Extension A */
  ["\u446B", "\u446B"], /* CJK Ideograph Extension A */
  ["\u446C", "\u446C"], /* CJK Ideograph Extension A */
  ["\u446D", "\u446D"], /* CJK Ideograph Extension A */
  ["\u446E", "\u446E"], /* CJK Ideograph Extension A */
  ["\u446F", "\u446F"], /* CJK Ideograph Extension A */
  ["\u4470", "\u4470"], /* CJK Ideograph Extension A */
  ["\u4471", "\u4471"], /* CJK Ideograph Extension A */
  ["\u4472", "\u4472"], /* CJK Ideograph Extension A */
  ["\u4473", "\u4473"], /* CJK Ideograph Extension A */
  ["\u4474", "\u4474"], /* CJK Ideograph Extension A */
  ["\u4475", "\u4475"], /* CJK Ideograph Extension A */
  ["\u4476", "\u4476"], /* CJK Ideograph Extension A */
  ["\u4477", "\u4477"], /* CJK Ideograph Extension A */
  ["\u4478", "\u4478"], /* CJK Ideograph Extension A */
  ["\u4479", "\u4479"], /* CJK Ideograph Extension A */
  ["\u447A", "\u447A"], /* CJK Ideograph Extension A */
  ["\u447B", "\u447B"], /* CJK Ideograph Extension A */
  ["\u447C", "\u447C"], /* CJK Ideograph Extension A */
  ["\u447D", "\u447D"], /* CJK Ideograph Extension A */
  ["\u447E", "\u447E"], /* CJK Ideograph Extension A */
  ["\u447F", "\u447F"], /* CJK Ideograph Extension A */
  ["\u4480", "\u4480"], /* CJK Ideograph Extension A */
  ["\u4481", "\u4481"], /* CJK Ideograph Extension A */
  ["\u4482", "\u4482"], /* CJK Ideograph Extension A */
  ["\u4483", "\u4483"], /* CJK Ideograph Extension A */
  ["\u4484", "\u4484"], /* CJK Ideograph Extension A */
  ["\u4485", "\u4485"], /* CJK Ideograph Extension A */
  ["\u4486", "\u4486"], /* CJK Ideograph Extension A */
  ["\u4487", "\u4487"], /* CJK Ideograph Extension A */
  ["\u4488", "\u4488"], /* CJK Ideograph Extension A */
  ["\u4489", "\u4489"], /* CJK Ideograph Extension A */
  ["\u448A", "\u448A"], /* CJK Ideograph Extension A */
  ["\u448B", "\u448B"], /* CJK Ideograph Extension A */
  ["\u448C", "\u448C"], /* CJK Ideograph Extension A */
  ["\u448D", "\u448D"], /* CJK Ideograph Extension A */
  ["\u448E", "\u448E"], /* CJK Ideograph Extension A */
  ["\u448F", "\u448F"], /* CJK Ideograph Extension A */
  ["\u4490", "\u4490"], /* CJK Ideograph Extension A */
  ["\u4491", "\u4491"], /* CJK Ideograph Extension A */
  ["\u4492", "\u4492"], /* CJK Ideograph Extension A */
  ["\u4493", "\u4493"], /* CJK Ideograph Extension A */
  ["\u4494", "\u4494"], /* CJK Ideograph Extension A */
  ["\u4495", "\u4495"], /* CJK Ideograph Extension A */
  ["\u4496", "\u4496"], /* CJK Ideograph Extension A */
  ["\u4497", "\u4497"], /* CJK Ideograph Extension A */
  ["\u4498", "\u4498"], /* CJK Ideograph Extension A */
  ["\u4499", "\u4499"], /* CJK Ideograph Extension A */
  ["\u449A", "\u449A"], /* CJK Ideograph Extension A */
  ["\u449B", "\u449B"], /* CJK Ideograph Extension A */
  ["\u449C", "\u449C"], /* CJK Ideograph Extension A */
  ["\u449D", "\u449D"], /* CJK Ideograph Extension A */
  ["\u449E", "\u449E"], /* CJK Ideograph Extension A */
  ["\u449F", "\u449F"], /* CJK Ideograph Extension A */
  ["\u44A0", "\u44A0"], /* CJK Ideograph Extension A */
  ["\u44A1", "\u44A1"], /* CJK Ideograph Extension A */
  ["\u44A2", "\u44A2"], /* CJK Ideograph Extension A */
  ["\u44A3", "\u44A3"], /* CJK Ideograph Extension A */
  ["\u44A4", "\u44A4"], /* CJK Ideograph Extension A */
  ["\u44A5", "\u44A5"], /* CJK Ideograph Extension A */
  ["\u44A6", "\u44A6"], /* CJK Ideograph Extension A */
  ["\u44A7", "\u44A7"], /* CJK Ideograph Extension A */
  ["\u44A8", "\u44A8"], /* CJK Ideograph Extension A */
  ["\u44A9", "\u44A9"], /* CJK Ideograph Extension A */
  ["\u44AA", "\u44AA"], /* CJK Ideograph Extension A */
  ["\u44AB", "\u44AB"], /* CJK Ideograph Extension A */
  ["\u44AC", "\u44AC"], /* CJK Ideograph Extension A */
  ["\u44AD", "\u44AD"], /* CJK Ideograph Extension A */
  ["\u44AE", "\u44AE"], /* CJK Ideograph Extension A */
  ["\u44AF", "\u44AF"], /* CJK Ideograph Extension A */
  ["\u44B0", "\u44B0"], /* CJK Ideograph Extension A */
  ["\u44B1", "\u44B1"], /* CJK Ideograph Extension A */
  ["\u44B2", "\u44B2"], /* CJK Ideograph Extension A */
  ["\u44B3", "\u44B3"], /* CJK Ideograph Extension A */
  ["\u44B4", "\u44B4"], /* CJK Ideograph Extension A */
  ["\u44B5", "\u44B5"], /* CJK Ideograph Extension A */
  ["\u44B6", "\u44B6"], /* CJK Ideograph Extension A */
  ["\u44B7", "\u44B7"], /* CJK Ideograph Extension A */
  ["\u44B8", "\u44B8"], /* CJK Ideograph Extension A */
  ["\u44B9", "\u44B9"], /* CJK Ideograph Extension A */
  ["\u44BA", "\u44BA"], /* CJK Ideograph Extension A */
  ["\u44BB", "\u44BB"], /* CJK Ideograph Extension A */
  ["\u44BC", "\u44BC"], /* CJK Ideograph Extension A */
  ["\u44BD", "\u44BD"], /* CJK Ideograph Extension A */
  ["\u44BE", "\u44BE"], /* CJK Ideograph Extension A */
  ["\u44BF", "\u44BF"], /* CJK Ideograph Extension A */
  ["\u44C0", "\u44C0"], /* CJK Ideograph Extension A */
  ["\u44C1", "\u44C1"], /* CJK Ideograph Extension A */
  ["\u44C2", "\u44C2"], /* CJK Ideograph Extension A */
  ["\u44C3", "\u44C3"], /* CJK Ideograph Extension A */
  ["\u44C4", "\u44C4"], /* CJK Ideograph Extension A */
  ["\u44C5", "\u44C5"], /* CJK Ideograph Extension A */
  ["\u44C6", "\u44C6"], /* CJK Ideograph Extension A */
  ["\u44C7", "\u44C7"], /* CJK Ideograph Extension A */
  ["\u44C8", "\u44C8"], /* CJK Ideograph Extension A */
  ["\u44C9", "\u44C9"], /* CJK Ideograph Extension A */
  ["\u44CA", "\u44CA"], /* CJK Ideograph Extension A */
  ["\u44CB", "\u44CB"], /* CJK Ideograph Extension A */
  ["\u44CC", "\u44CC"], /* CJK Ideograph Extension A */
  ["\u44CD", "\u44CD"], /* CJK Ideograph Extension A */
  ["\u44CE", "\u44CE"], /* CJK Ideograph Extension A */
  ["\u44CF", "\u44CF"], /* CJK Ideograph Extension A */
  ["\u44D0", "\u44D0"], /* CJK Ideograph Extension A */
  ["\u44D1", "\u44D1"], /* CJK Ideograph Extension A */
  ["\u44D2", "\u44D2"], /* CJK Ideograph Extension A */
  ["\u44D3", "\u44D3"], /* CJK Ideograph Extension A */
  ["\u44D4", "\u44D4"], /* CJK Ideograph Extension A */
  ["\u44D5", "\u44D5"], /* CJK Ideograph Extension A */
  ["\u44D6", "\u44D6"], /* CJK Ideograph Extension A */
  ["\u44D7", "\u44D7"], /* CJK Ideograph Extension A */
  ["\u44D8", "\u44D8"], /* CJK Ideograph Extension A */
  ["\u44D9", "\u44D9"], /* CJK Ideograph Extension A */
  ["\u44DA", "\u44DA"], /* CJK Ideograph Extension A */
  ["\u44DB", "\u44DB"], /* CJK Ideograph Extension A */
  ["\u44DC", "\u44DC"], /* CJK Ideograph Extension A */
  ["\u44DD", "\u44DD"], /* CJK Ideograph Extension A */
  ["\u44DE", "\u44DE"], /* CJK Ideograph Extension A */
  ["\u44DF", "\u44DF"], /* CJK Ideograph Extension A */
  ["\u44E0", "\u44E0"], /* CJK Ideograph Extension A */
  ["\u44E1", "\u44E1"], /* CJK Ideograph Extension A */
  ["\u44E2", "\u44E2"], /* CJK Ideograph Extension A */
  ["\u44E3", "\u44E3"], /* CJK Ideograph Extension A */
  ["\u44E4", "\u44E4"], /* CJK Ideograph Extension A */
  ["\u44E5", "\u44E5"], /* CJK Ideograph Extension A */
  ["\u44E6", "\u44E6"], /* CJK Ideograph Extension A */
  ["\u44E7", "\u44E7"], /* CJK Ideograph Extension A */
  ["\u44E8", "\u44E8"], /* CJK Ideograph Extension A */
  ["\u44E9", "\u44E9"], /* CJK Ideograph Extension A */
  ["\u44EA", "\u44EA"], /* CJK Ideograph Extension A */
  ["\u44EB", "\u44EB"], /* CJK Ideograph Extension A */
  ["\u44EC", "\u44EC"], /* CJK Ideograph Extension A */
  ["\u44ED", "\u44ED"], /* CJK Ideograph Extension A */
  ["\u44EE", "\u44EE"], /* CJK Ideograph Extension A */
  ["\u44EF", "\u44EF"], /* CJK Ideograph Extension A */
  ["\u44F0", "\u44F0"], /* CJK Ideograph Extension A */
  ["\u44F1", "\u44F1"], /* CJK Ideograph Extension A */
  ["\u44F2", "\u44F2"], /* CJK Ideograph Extension A */
  ["\u44F3", "\u44F3"], /* CJK Ideograph Extension A */
  ["\u44F4", "\u44F4"], /* CJK Ideograph Extension A */
  ["\u44F5", "\u44F5"], /* CJK Ideograph Extension A */
  ["\u44F6", "\u44F6"], /* CJK Ideograph Extension A */
  ["\u44F7", "\u44F7"], /* CJK Ideograph Extension A */
  ["\u44F8", "\u44F8"], /* CJK Ideograph Extension A */
  ["\u44F9", "\u44F9"], /* CJK Ideograph Extension A */
  ["\u44FA", "\u44FA"], /* CJK Ideograph Extension A */
  ["\u44FB", "\u44FB"], /* CJK Ideograph Extension A */
  ["\u44FC", "\u44FC"], /* CJK Ideograph Extension A */
  ["\u44FD", "\u44FD"], /* CJK Ideograph Extension A */
  ["\u44FE", "\u44FE"], /* CJK Ideograph Extension A */
  ["\u44FF", "\u44FF"], /* CJK Ideograph Extension A */
  ["\u4500", "\u4500"], /* CJK Ideograph Extension A */
  ["\u4501", "\u4501"], /* CJK Ideograph Extension A */
  ["\u4502", "\u4502"], /* CJK Ideograph Extension A */
  ["\u4503", "\u4503"], /* CJK Ideograph Extension A */
  ["\u4504", "\u4504"], /* CJK Ideograph Extension A */
  ["\u4505", "\u4505"], /* CJK Ideograph Extension A */
  ["\u4506", "\u4506"], /* CJK Ideograph Extension A */
  ["\u4507", "\u4507"], /* CJK Ideograph Extension A */
  ["\u4508", "\u4508"], /* CJK Ideograph Extension A */
  ["\u4509", "\u4509"], /* CJK Ideograph Extension A */
  ["\u450A", "\u450A"], /* CJK Ideograph Extension A */
  ["\u450B", "\u450B"], /* CJK Ideograph Extension A */
  ["\u450C", "\u450C"], /* CJK Ideograph Extension A */
  ["\u450D", "\u450D"], /* CJK Ideograph Extension A */
  ["\u450E", "\u450E"], /* CJK Ideograph Extension A */
  ["\u450F", "\u450F"], /* CJK Ideograph Extension A */
  ["\u4510", "\u4510"], /* CJK Ideograph Extension A */
  ["\u4511", "\u4511"], /* CJK Ideograph Extension A */
  ["\u4512", "\u4512"], /* CJK Ideograph Extension A */
  ["\u4513", "\u4513"], /* CJK Ideograph Extension A */
  ["\u4514", "\u4514"], /* CJK Ideograph Extension A */
  ["\u4515", "\u4515"], /* CJK Ideograph Extension A */
  ["\u4516", "\u4516"], /* CJK Ideograph Extension A */
  ["\u4517", "\u4517"], /* CJK Ideograph Extension A */
  ["\u4518", "\u4518"], /* CJK Ideograph Extension A */
  ["\u4519", "\u4519"], /* CJK Ideograph Extension A */
  ["\u451A", "\u451A"], /* CJK Ideograph Extension A */
  ["\u451B", "\u451B"], /* CJK Ideograph Extension A */
  ["\u451C", "\u451C"], /* CJK Ideograph Extension A */
  ["\u451D", "\u451D"], /* CJK Ideograph Extension A */
  ["\u451E", "\u451E"], /* CJK Ideograph Extension A */
  ["\u451F", "\u451F"], /* CJK Ideograph Extension A */
  ["\u4520", "\u4520"], /* CJK Ideograph Extension A */
  ["\u4521", "\u4521"], /* CJK Ideograph Extension A */
  ["\u4522", "\u4522"], /* CJK Ideograph Extension A */
  ["\u4523", "\u4523"], /* CJK Ideograph Extension A */
  ["\u4524", "\u4524"], /* CJK Ideograph Extension A */
  ["\u4525", "\u4525"], /* CJK Ideograph Extension A */
  ["\u4526", "\u4526"], /* CJK Ideograph Extension A */
  ["\u4527", "\u4527"], /* CJK Ideograph Extension A */
  ["\u4528", "\u4528"], /* CJK Ideograph Extension A */
  ["\u4529", "\u4529"], /* CJK Ideograph Extension A */
  ["\u452A", "\u452A"], /* CJK Ideograph Extension A */
  ["\u452B", "\u452B"], /* CJK Ideograph Extension A */
  ["\u452C", "\u452C"], /* CJK Ideograph Extension A */
  ["\u452D", "\u452D"], /* CJK Ideograph Extension A */
  ["\u452E", "\u452E"], /* CJK Ideograph Extension A */
  ["\u452F", "\u452F"], /* CJK Ideograph Extension A */
  ["\u4530", "\u4530"], /* CJK Ideograph Extension A */
  ["\u4531", "\u4531"], /* CJK Ideograph Extension A */
  ["\u4532", "\u4532"], /* CJK Ideograph Extension A */
  ["\u4533", "\u4533"], /* CJK Ideograph Extension A */
  ["\u4534", "\u4534"], /* CJK Ideograph Extension A */
  ["\u4535", "\u4535"], /* CJK Ideograph Extension A */
  ["\u4536", "\u4536"], /* CJK Ideograph Extension A */
  ["\u4537", "\u4537"], /* CJK Ideograph Extension A */
  ["\u4538", "\u4538"], /* CJK Ideograph Extension A */
  ["\u4539", "\u4539"], /* CJK Ideograph Extension A */
  ["\u453A", "\u453A"], /* CJK Ideograph Extension A */
  ["\u453B", "\u453B"], /* CJK Ideograph Extension A */
  ["\u453C", "\u453C"], /* CJK Ideograph Extension A */
  ["\u453D", "\u453D"], /* CJK Ideograph Extension A */
  ["\u453E", "\u453E"], /* CJK Ideograph Extension A */
  ["\u453F", "\u453F"], /* CJK Ideograph Extension A */
  ["\u4540", "\u4540"], /* CJK Ideograph Extension A */
  ["\u4541", "\u4541"], /* CJK Ideograph Extension A */
  ["\u4542", "\u4542"], /* CJK Ideograph Extension A */
  ["\u4543", "\u4543"], /* CJK Ideograph Extension A */
  ["\u4544", "\u4544"], /* CJK Ideograph Extension A */
  ["\u4545", "\u4545"], /* CJK Ideograph Extension A */
  ["\u4546", "\u4546"], /* CJK Ideograph Extension A */
  ["\u4547", "\u4547"], /* CJK Ideograph Extension A */
  ["\u4548", "\u4548"], /* CJK Ideograph Extension A */
  ["\u4549", "\u4549"], /* CJK Ideograph Extension A */
  ["\u454A", "\u454A"], /* CJK Ideograph Extension A */
  ["\u454B", "\u454B"], /* CJK Ideograph Extension A */
  ["\u454C", "\u454C"], /* CJK Ideograph Extension A */
  ["\u454D", "\u454D"], /* CJK Ideograph Extension A */
  ["\u454E", "\u454E"], /* CJK Ideograph Extension A */
  ["\u454F", "\u454F"], /* CJK Ideograph Extension A */
  ["\u4550", "\u4550"], /* CJK Ideograph Extension A */
  ["\u4551", "\u4551"], /* CJK Ideograph Extension A */
  ["\u4552", "\u4552"], /* CJK Ideograph Extension A */
  ["\u4553", "\u4553"], /* CJK Ideograph Extension A */
  ["\u4554", "\u4554"], /* CJK Ideograph Extension A */
  ["\u4555", "\u4555"], /* CJK Ideograph Extension A */
  ["\u4556", "\u4556"], /* CJK Ideograph Extension A */
  ["\u4557", "\u4557"], /* CJK Ideograph Extension A */
  ["\u4558", "\u4558"], /* CJK Ideograph Extension A */
  ["\u4559", "\u4559"], /* CJK Ideograph Extension A */
  ["\u455A", "\u455A"], /* CJK Ideograph Extension A */
  ["\u455B", "\u455B"], /* CJK Ideograph Extension A */
  ["\u455C", "\u455C"], /* CJK Ideograph Extension A */
  ["\u455D", "\u455D"], /* CJK Ideograph Extension A */
  ["\u455E", "\u455E"], /* CJK Ideograph Extension A */
  ["\u455F", "\u455F"], /* CJK Ideograph Extension A */
  ["\u4560", "\u4560"], /* CJK Ideograph Extension A */
  ["\u4561", "\u4561"], /* CJK Ideograph Extension A */
  ["\u4562", "\u4562"], /* CJK Ideograph Extension A */
  ["\u4563", "\u4563"], /* CJK Ideograph Extension A */
  ["\u4564", "\u4564"], /* CJK Ideograph Extension A */
  ["\u4565", "\u4565"], /* CJK Ideograph Extension A */
  ["\u4566", "\u4566"], /* CJK Ideograph Extension A */
  ["\u4567", "\u4567"], /* CJK Ideograph Extension A */
  ["\u4568", "\u4568"], /* CJK Ideograph Extension A */
  ["\u4569", "\u4569"], /* CJK Ideograph Extension A */
  ["\u456A", "\u456A"], /* CJK Ideograph Extension A */
  ["\u456B", "\u456B"], /* CJK Ideograph Extension A */
  ["\u456C", "\u456C"], /* CJK Ideograph Extension A */
  ["\u456D", "\u456D"], /* CJK Ideograph Extension A */
  ["\u456E", "\u456E"], /* CJK Ideograph Extension A */
  ["\u456F", "\u456F"], /* CJK Ideograph Extension A */
  ["\u4570", "\u4570"], /* CJK Ideograph Extension A */
  ["\u4571", "\u4571"], /* CJK Ideograph Extension A */
  ["\u4572", "\u4572"], /* CJK Ideograph Extension A */
  ["\u4573", "\u4573"], /* CJK Ideograph Extension A */
  ["\u4574", "\u4574"], /* CJK Ideograph Extension A */
  ["\u4575", "\u4575"], /* CJK Ideograph Extension A */
  ["\u4576", "\u4576"], /* CJK Ideograph Extension A */
  ["\u4577", "\u4577"], /* CJK Ideograph Extension A */
  ["\u4578", "\u4578"], /* CJK Ideograph Extension A */
  ["\u4579", "\u4579"], /* CJK Ideograph Extension A */
  ["\u457A", "\u457A"], /* CJK Ideograph Extension A */
  ["\u457B", "\u457B"], /* CJK Ideograph Extension A */
  ["\u457C", "\u457C"], /* CJK Ideograph Extension A */
  ["\u457D", "\u457D"], /* CJK Ideograph Extension A */
  ["\u457E", "\u457E"], /* CJK Ideograph Extension A */
  ["\u457F", "\u457F"], /* CJK Ideograph Extension A */
  ["\u4580", "\u4580"], /* CJK Ideograph Extension A */
  ["\u4581", "\u4581"], /* CJK Ideograph Extension A */
  ["\u4582", "\u4582"], /* CJK Ideograph Extension A */
  ["\u4583", "\u4583"], /* CJK Ideograph Extension A */
  ["\u4584", "\u4584"], /* CJK Ideograph Extension A */
  ["\u4585", "\u4585"], /* CJK Ideograph Extension A */
  ["\u4586", "\u4586"], /* CJK Ideograph Extension A */
  ["\u4587", "\u4587"], /* CJK Ideograph Extension A */
  ["\u4588", "\u4588"], /* CJK Ideograph Extension A */
  ["\u4589", "\u4589"], /* CJK Ideograph Extension A */
  ["\u458A", "\u458A"], /* CJK Ideograph Extension A */
  ["\u458B", "\u458B"], /* CJK Ideograph Extension A */
  ["\u458C", "\u458C"], /* CJK Ideograph Extension A */
  ["\u458D", "\u458D"], /* CJK Ideograph Extension A */
  ["\u458E", "\u458E"], /* CJK Ideograph Extension A */
  ["\u458F", "\u458F"], /* CJK Ideograph Extension A */
  ["\u4590", "\u4590"], /* CJK Ideograph Extension A */
  ["\u4591", "\u4591"], /* CJK Ideograph Extension A */
  ["\u4592", "\u4592"], /* CJK Ideograph Extension A */
  ["\u4593", "\u4593"], /* CJK Ideograph Extension A */
  ["\u4594", "\u4594"], /* CJK Ideograph Extension A */
  ["\u4595", "\u4595"], /* CJK Ideograph Extension A */
  ["\u4596", "\u4596"], /* CJK Ideograph Extension A */
  ["\u4597", "\u4597"], /* CJK Ideograph Extension A */
  ["\u4598", "\u4598"], /* CJK Ideograph Extension A */
  ["\u4599", "\u4599"], /* CJK Ideograph Extension A */
  ["\u459A", "\u459A"], /* CJK Ideograph Extension A */
  ["\u459B", "\u459B"], /* CJK Ideograph Extension A */
  ["\u459C", "\u459C"], /* CJK Ideograph Extension A */
  ["\u459D", "\u459D"], /* CJK Ideograph Extension A */
  ["\u459E", "\u459E"], /* CJK Ideograph Extension A */
  ["\u459F", "\u459F"], /* CJK Ideograph Extension A */
  ["\u45A0", "\u45A0"], /* CJK Ideograph Extension A */
  ["\u45A1", "\u45A1"], /* CJK Ideograph Extension A */
  ["\u45A2", "\u45A2"], /* CJK Ideograph Extension A */
  ["\u45A3", "\u45A3"], /* CJK Ideograph Extension A */
  ["\u45A4", "\u45A4"], /* CJK Ideograph Extension A */
  ["\u45A5", "\u45A5"], /* CJK Ideograph Extension A */
  ["\u45A6", "\u45A6"], /* CJK Ideograph Extension A */
  ["\u45A7", "\u45A7"], /* CJK Ideograph Extension A */
  ["\u45A8", "\u45A8"], /* CJK Ideograph Extension A */
  ["\u45A9", "\u45A9"], /* CJK Ideograph Extension A */
  ["\u45AA", "\u45AA"], /* CJK Ideograph Extension A */
  ["\u45AB", "\u45AB"], /* CJK Ideograph Extension A */
  ["\u45AC", "\u45AC"], /* CJK Ideograph Extension A */
  ["\u45AD", "\u45AD"], /* CJK Ideograph Extension A */
  ["\u45AE", "\u45AE"], /* CJK Ideograph Extension A */
  ["\u45AF", "\u45AF"], /* CJK Ideograph Extension A */
  ["\u45B0", "\u45B0"], /* CJK Ideograph Extension A */
  ["\u45B1", "\u45B1"], /* CJK Ideograph Extension A */
  ["\u45B2", "\u45B2"], /* CJK Ideograph Extension A */
  ["\u45B3", "\u45B3"], /* CJK Ideograph Extension A */
  ["\u45B4", "\u45B4"], /* CJK Ideograph Extension A */
  ["\u45B5", "\u45B5"], /* CJK Ideograph Extension A */
  ["\u45B6", "\u45B6"], /* CJK Ideograph Extension A */
  ["\u45B7", "\u45B7"], /* CJK Ideograph Extension A */
  ["\u45B8", "\u45B8"], /* CJK Ideograph Extension A */
  ["\u45B9", "\u45B9"], /* CJK Ideograph Extension A */
  ["\u45BA", "\u45BA"], /* CJK Ideograph Extension A */
  ["\u45BB", "\u45BB"], /* CJK Ideograph Extension A */
  ["\u45BC", "\u45BC"], /* CJK Ideograph Extension A */
  ["\u45BD", "\u45BD"], /* CJK Ideograph Extension A */
  ["\u45BE", "\u45BE"], /* CJK Ideograph Extension A */
  ["\u45BF", "\u45BF"], /* CJK Ideograph Extension A */
  ["\u45C0", "\u45C0"], /* CJK Ideograph Extension A */
  ["\u45C1", "\u45C1"], /* CJK Ideograph Extension A */
  ["\u45C2", "\u45C2"], /* CJK Ideograph Extension A */
  ["\u45C3", "\u45C3"], /* CJK Ideograph Extension A */
  ["\u45C4", "\u45C4"], /* CJK Ideograph Extension A */
  ["\u45C5", "\u45C5"], /* CJK Ideograph Extension A */
  ["\u45C6", "\u45C6"], /* CJK Ideograph Extension A */
  ["\u45C7", "\u45C7"], /* CJK Ideograph Extension A */
  ["\u45C8", "\u45C8"], /* CJK Ideograph Extension A */
  ["\u45C9", "\u45C9"], /* CJK Ideograph Extension A */
  ["\u45CA", "\u45CA"], /* CJK Ideograph Extension A */
  ["\u45CB", "\u45CB"], /* CJK Ideograph Extension A */
  ["\u45CC", "\u45CC"], /* CJK Ideograph Extension A */
  ["\u45CD", "\u45CD"], /* CJK Ideograph Extension A */
  ["\u45CE", "\u45CE"], /* CJK Ideograph Extension A */
  ["\u45CF", "\u45CF"], /* CJK Ideograph Extension A */
  ["\u45D0", "\u45D0"], /* CJK Ideograph Extension A */
  ["\u45D1", "\u45D1"], /* CJK Ideograph Extension A */
  ["\u45D2", "\u45D2"], /* CJK Ideograph Extension A */
  ["\u45D3", "\u45D3"], /* CJK Ideograph Extension A */
  ["\u45D4", "\u45D4"], /* CJK Ideograph Extension A */
  ["\u45D5", "\u45D5"], /* CJK Ideograph Extension A */
  ["\u45D6", "\u45D6"], /* CJK Ideograph Extension A */
  ["\u45D7", "\u45D7"], /* CJK Ideograph Extension A */
  ["\u45D8", "\u45D8"], /* CJK Ideograph Extension A */
  ["\u45D9", "\u45D9"], /* CJK Ideograph Extension A */
  ["\u45DA", "\u45DA"], /* CJK Ideograph Extension A */
  ["\u45DB", "\u45DB"], /* CJK Ideograph Extension A */
  ["\u45DC", "\u45DC"], /* CJK Ideograph Extension A */
  ["\u45DD", "\u45DD"], /* CJK Ideograph Extension A */
  ["\u45DE", "\u45DE"], /* CJK Ideograph Extension A */
  ["\u45DF", "\u45DF"], /* CJK Ideograph Extension A */
  ["\u45E0", "\u45E0"], /* CJK Ideograph Extension A */
  ["\u45E1", "\u45E1"], /* CJK Ideograph Extension A */
  ["\u45E2", "\u45E2"], /* CJK Ideograph Extension A */
  ["\u45E3", "\u45E3"], /* CJK Ideograph Extension A */
  ["\u45E4", "\u45E4"], /* CJK Ideograph Extension A */
  ["\u45E5", "\u45E5"], /* CJK Ideograph Extension A */
  ["\u45E6", "\u45E6"], /* CJK Ideograph Extension A */
  ["\u45E7", "\u45E7"], /* CJK Ideograph Extension A */
  ["\u45E8", "\u45E8"], /* CJK Ideograph Extension A */
  ["\u45E9", "\u45E9"], /* CJK Ideograph Extension A */
  ["\u45EA", "\u45EA"], /* CJK Ideograph Extension A */
  ["\u45EB", "\u45EB"], /* CJK Ideograph Extension A */
  ["\u45EC", "\u45EC"], /* CJK Ideograph Extension A */
  ["\u45ED", "\u45ED"], /* CJK Ideograph Extension A */
  ["\u45EE", "\u45EE"], /* CJK Ideograph Extension A */
  ["\u45EF", "\u45EF"], /* CJK Ideograph Extension A */
  ["\u45F0", "\u45F0"], /* CJK Ideograph Extension A */
  ["\u45F1", "\u45F1"], /* CJK Ideograph Extension A */
  ["\u45F2", "\u45F2"], /* CJK Ideograph Extension A */
  ["\u45F3", "\u45F3"], /* CJK Ideograph Extension A */
  ["\u45F4", "\u45F4"], /* CJK Ideograph Extension A */
  ["\u45F5", "\u45F5"], /* CJK Ideograph Extension A */
  ["\u45F6", "\u45F6"], /* CJK Ideograph Extension A */
  ["\u45F7", "\u45F7"], /* CJK Ideograph Extension A */
  ["\u45F8", "\u45F8"], /* CJK Ideograph Extension A */
  ["\u45F9", "\u45F9"], /* CJK Ideograph Extension A */
  ["\u45FA", "\u45FA"], /* CJK Ideograph Extension A */
  ["\u45FB", "\u45FB"], /* CJK Ideograph Extension A */
  ["\u45FC", "\u45FC"], /* CJK Ideograph Extension A */
  ["\u45FD", "\u45FD"], /* CJK Ideograph Extension A */
  ["\u45FE", "\u45FE"], /* CJK Ideograph Extension A */
  ["\u45FF", "\u45FF"], /* CJK Ideograph Extension A */
  ["\u4600", "\u4600"], /* CJK Ideograph Extension A */
  ["\u4601", "\u4601"], /* CJK Ideograph Extension A */
  ["\u4602", "\u4602"], /* CJK Ideograph Extension A */
  ["\u4603", "\u4603"], /* CJK Ideograph Extension A */
  ["\u4604", "\u4604"], /* CJK Ideograph Extension A */
  ["\u4605", "\u4605"], /* CJK Ideograph Extension A */
  ["\u4606", "\u4606"], /* CJK Ideograph Extension A */
  ["\u4607", "\u4607"], /* CJK Ideograph Extension A */
  ["\u4608", "\u4608"], /* CJK Ideograph Extension A */
  ["\u4609", "\u4609"], /* CJK Ideograph Extension A */
  ["\u460A", "\u460A"], /* CJK Ideograph Extension A */
  ["\u460B", "\u460B"], /* CJK Ideograph Extension A */
  ["\u460C", "\u460C"], /* CJK Ideograph Extension A */
  ["\u460D", "\u460D"], /* CJK Ideograph Extension A */
  ["\u460E", "\u460E"], /* CJK Ideograph Extension A */
  ["\u460F", "\u460F"], /* CJK Ideograph Extension A */
  ["\u4610", "\u4610"], /* CJK Ideograph Extension A */
  ["\u4611", "\u4611"], /* CJK Ideograph Extension A */
  ["\u4612", "\u4612"], /* CJK Ideograph Extension A */
  ["\u4613", "\u4613"], /* CJK Ideograph Extension A */
  ["\u4614", "\u4614"], /* CJK Ideograph Extension A */
  ["\u4615", "\u4615"], /* CJK Ideograph Extension A */
  ["\u4616", "\u4616"], /* CJK Ideograph Extension A */
  ["\u4617", "\u4617"], /* CJK Ideograph Extension A */
  ["\u4618", "\u4618"], /* CJK Ideograph Extension A */
  ["\u4619", "\u4619"], /* CJK Ideograph Extension A */
  ["\u461A", "\u461A"], /* CJK Ideograph Extension A */
  ["\u461B", "\u461B"], /* CJK Ideograph Extension A */
  ["\u461C", "\u461C"], /* CJK Ideograph Extension A */
  ["\u461D", "\u461D"], /* CJK Ideograph Extension A */
  ["\u461E", "\u461E"], /* CJK Ideograph Extension A */
  ["\u461F", "\u461F"], /* CJK Ideograph Extension A */
  ["\u4620", "\u4620"], /* CJK Ideograph Extension A */
  ["\u4621", "\u4621"], /* CJK Ideograph Extension A */
  ["\u4622", "\u4622"], /* CJK Ideograph Extension A */
  ["\u4623", "\u4623"], /* CJK Ideograph Extension A */
  ["\u4624", "\u4624"], /* CJK Ideograph Extension A */
  ["\u4625", "\u4625"], /* CJK Ideograph Extension A */
  ["\u4626", "\u4626"], /* CJK Ideograph Extension A */
  ["\u4627", "\u4627"], /* CJK Ideograph Extension A */
  ["\u4628", "\u4628"], /* CJK Ideograph Extension A */
  ["\u4629", "\u4629"], /* CJK Ideograph Extension A */
  ["\u462A", "\u462A"], /* CJK Ideograph Extension A */
  ["\u462B", "\u462B"], /* CJK Ideograph Extension A */
  ["\u462C", "\u462C"], /* CJK Ideograph Extension A */
  ["\u462D", "\u462D"], /* CJK Ideograph Extension A */
  ["\u462E", "\u462E"], /* CJK Ideograph Extension A */
  ["\u462F", "\u462F"], /* CJK Ideograph Extension A */
  ["\u4630", "\u4630"], /* CJK Ideograph Extension A */
  ["\u4631", "\u4631"], /* CJK Ideograph Extension A */
  ["\u4632", "\u4632"], /* CJK Ideograph Extension A */
  ["\u4633", "\u4633"], /* CJK Ideograph Extension A */
  ["\u4634", "\u4634"], /* CJK Ideograph Extension A */
  ["\u4635", "\u4635"], /* CJK Ideograph Extension A */
  ["\u4636", "\u4636"], /* CJK Ideograph Extension A */
  ["\u4637", "\u4637"], /* CJK Ideograph Extension A */
  ["\u4638", "\u4638"], /* CJK Ideograph Extension A */
  ["\u4639", "\u4639"], /* CJK Ideograph Extension A */
  ["\u463A", "\u463A"], /* CJK Ideograph Extension A */
  ["\u463B", "\u463B"], /* CJK Ideograph Extension A */
  ["\u463C", "\u463C"], /* CJK Ideograph Extension A */
  ["\u463D", "\u463D"], /* CJK Ideograph Extension A */
  ["\u463E", "\u463E"], /* CJK Ideograph Extension A */
  ["\u463F", "\u463F"], /* CJK Ideograph Extension A */
  ["\u4640", "\u4640"], /* CJK Ideograph Extension A */
  ["\u4641", "\u4641"], /* CJK Ideograph Extension A */
  ["\u4642", "\u4642"], /* CJK Ideograph Extension A */
  ["\u4643", "\u4643"], /* CJK Ideograph Extension A */
  ["\u4644", "\u4644"], /* CJK Ideograph Extension A */
  ["\u4645", "\u4645"], /* CJK Ideograph Extension A */
  ["\u4646", "\u4646"], /* CJK Ideograph Extension A */
  ["\u4647", "\u4647"], /* CJK Ideograph Extension A */
  ["\u4648", "\u4648"], /* CJK Ideograph Extension A */
  ["\u4649", "\u4649"], /* CJK Ideograph Extension A */
  ["\u464A", "\u464A"], /* CJK Ideograph Extension A */
  ["\u464B", "\u464B"], /* CJK Ideograph Extension A */
  ["\u464C", "\u464C"], /* CJK Ideograph Extension A */
  ["\u464D", "\u464D"], /* CJK Ideograph Extension A */
  ["\u464E", "\u464E"], /* CJK Ideograph Extension A */
  ["\u464F", "\u464F"], /* CJK Ideograph Extension A */
  ["\u4650", "\u4650"], /* CJK Ideograph Extension A */
  ["\u4651", "\u4651"], /* CJK Ideograph Extension A */
  ["\u4652", "\u4652"], /* CJK Ideograph Extension A */
  ["\u4653", "\u4653"], /* CJK Ideograph Extension A */
  ["\u4654", "\u4654"], /* CJK Ideograph Extension A */
  ["\u4655", "\u4655"], /* CJK Ideograph Extension A */
  ["\u4656", "\u4656"], /* CJK Ideograph Extension A */
  ["\u4657", "\u4657"], /* CJK Ideograph Extension A */
  ["\u4658", "\u4658"], /* CJK Ideograph Extension A */
  ["\u4659", "\u4659"], /* CJK Ideograph Extension A */
  ["\u465A", "\u465A"], /* CJK Ideograph Extension A */
  ["\u465B", "\u465B"], /* CJK Ideograph Extension A */
  ["\u465C", "\u465C"], /* CJK Ideograph Extension A */
  ["\u465D", "\u465D"], /* CJK Ideograph Extension A */
  ["\u465E", "\u465E"], /* CJK Ideograph Extension A */
  ["\u465F", "\u465F"], /* CJK Ideograph Extension A */
  ["\u4660", "\u4660"], /* CJK Ideograph Extension A */
  ["\u4661", "\u4661"], /* CJK Ideograph Extension A */
  ["\u4662", "\u4662"], /* CJK Ideograph Extension A */
  ["\u4663", "\u4663"], /* CJK Ideograph Extension A */
  ["\u4664", "\u4664"], /* CJK Ideograph Extension A */
  ["\u4665", "\u4665"], /* CJK Ideograph Extension A */
  ["\u4666", "\u4666"], /* CJK Ideograph Extension A */
  ["\u4667", "\u4667"], /* CJK Ideograph Extension A */
  ["\u4668", "\u4668"], /* CJK Ideograph Extension A */
  ["\u4669", "\u4669"], /* CJK Ideograph Extension A */
  ["\u466A", "\u466A"], /* CJK Ideograph Extension A */
  ["\u466B", "\u466B"], /* CJK Ideograph Extension A */
  ["\u466C", "\u466C"], /* CJK Ideograph Extension A */
  ["\u466D", "\u466D"], /* CJK Ideograph Extension A */
  ["\u466E", "\u466E"], /* CJK Ideograph Extension A */
  ["\u466F", "\u466F"], /* CJK Ideograph Extension A */
  ["\u4670", "\u4670"], /* CJK Ideograph Extension A */
  ["\u4671", "\u4671"], /* CJK Ideograph Extension A */
  ["\u4672", "\u4672"], /* CJK Ideograph Extension A */
  ["\u4673", "\u4673"], /* CJK Ideograph Extension A */
  ["\u4674", "\u4674"], /* CJK Ideograph Extension A */
  ["\u4675", "\u4675"], /* CJK Ideograph Extension A */
  ["\u4676", "\u4676"], /* CJK Ideograph Extension A */
  ["\u4677", "\u4677"], /* CJK Ideograph Extension A */
  ["\u4678", "\u4678"], /* CJK Ideograph Extension A */
  ["\u4679", "\u4679"], /* CJK Ideograph Extension A */
  ["\u467A", "\u467A"], /* CJK Ideograph Extension A */
  ["\u467B", "\u467B"], /* CJK Ideograph Extension A */
  ["\u467C", "\u467C"], /* CJK Ideograph Extension A */
  ["\u467D", "\u467D"], /* CJK Ideograph Extension A */
  ["\u467E", "\u467E"], /* CJK Ideograph Extension A */
  ["\u467F", "\u467F"], /* CJK Ideograph Extension A */
  ["\u4680", "\u4680"], /* CJK Ideograph Extension A */
  ["\u4681", "\u4681"], /* CJK Ideograph Extension A */
  ["\u4682", "\u4682"], /* CJK Ideograph Extension A */
  ["\u4683", "\u4683"], /* CJK Ideograph Extension A */
  ["\u4684", "\u4684"], /* CJK Ideograph Extension A */
  ["\u4685", "\u4685"], /* CJK Ideograph Extension A */
  ["\u4686", "\u4686"], /* CJK Ideograph Extension A */
  ["\u4687", "\u4687"], /* CJK Ideograph Extension A */
  ["\u4688", "\u4688"], /* CJK Ideograph Extension A */
  ["\u4689", "\u4689"], /* CJK Ideograph Extension A */
  ["\u468A", "\u468A"], /* CJK Ideograph Extension A */
  ["\u468B", "\u468B"], /* CJK Ideograph Extension A */
  ["\u468C", "\u468C"], /* CJK Ideograph Extension A */
  ["\u468D", "\u468D"], /* CJK Ideograph Extension A */
  ["\u468E", "\u468E"], /* CJK Ideograph Extension A */
  ["\u468F", "\u468F"], /* CJK Ideograph Extension A */
  ["\u4690", "\u4690"], /* CJK Ideograph Extension A */
  ["\u4691", "\u4691"], /* CJK Ideograph Extension A */
  ["\u4692", "\u4692"], /* CJK Ideograph Extension A */
  ["\u4693", "\u4693"], /* CJK Ideograph Extension A */
  ["\u4694", "\u4694"], /* CJK Ideograph Extension A */
  ["\u4695", "\u4695"], /* CJK Ideograph Extension A */
  ["\u4696", "\u4696"], /* CJK Ideograph Extension A */
  ["\u4697", "\u4697"], /* CJK Ideograph Extension A */
  ["\u4698", "\u4698"], /* CJK Ideograph Extension A */
  ["\u4699", "\u4699"], /* CJK Ideograph Extension A */
  ["\u469A", "\u469A"], /* CJK Ideograph Extension A */
  ["\u469B", "\u469B"], /* CJK Ideograph Extension A */
  ["\u469C", "\u469C"], /* CJK Ideograph Extension A */
  ["\u469D", "\u469D"], /* CJK Ideograph Extension A */
  ["\u469E", "\u469E"], /* CJK Ideograph Extension A */
  ["\u469F", "\u469F"], /* CJK Ideograph Extension A */
  ["\u46A0", "\u46A0"], /* CJK Ideograph Extension A */
  ["\u46A1", "\u46A1"], /* CJK Ideograph Extension A */
  ["\u46A2", "\u46A2"], /* CJK Ideograph Extension A */
  ["\u46A3", "\u46A3"], /* CJK Ideograph Extension A */
  ["\u46A4", "\u46A4"], /* CJK Ideograph Extension A */
  ["\u46A5", "\u46A5"], /* CJK Ideograph Extension A */
  ["\u46A6", "\u46A6"], /* CJK Ideograph Extension A */
  ["\u46A7", "\u46A7"], /* CJK Ideograph Extension A */
  ["\u46A8", "\u46A8"], /* CJK Ideograph Extension A */
  ["\u46A9", "\u46A9"], /* CJK Ideograph Extension A */
  ["\u46AA", "\u46AA"], /* CJK Ideograph Extension A */
  ["\u46AB", "\u46AB"], /* CJK Ideograph Extension A */
  ["\u46AC", "\u46AC"], /* CJK Ideograph Extension A */
  ["\u46AD", "\u46AD"], /* CJK Ideograph Extension A */
  ["\u46AE", "\u46AE"], /* CJK Ideograph Extension A */
  ["\u46AF", "\u46AF"], /* CJK Ideograph Extension A */
  ["\u46B0", "\u46B0"], /* CJK Ideograph Extension A */
  ["\u46B1", "\u46B1"], /* CJK Ideograph Extension A */
  ["\u46B2", "\u46B2"], /* CJK Ideograph Extension A */
  ["\u46B3", "\u46B3"], /* CJK Ideograph Extension A */
  ["\u46B4", "\u46B4"], /* CJK Ideograph Extension A */
  ["\u46B5", "\u46B5"], /* CJK Ideograph Extension A */
  ["\u46B6", "\u46B6"], /* CJK Ideograph Extension A */
  ["\u46B7", "\u46B7"], /* CJK Ideograph Extension A */
  ["\u46B8", "\u46B8"], /* CJK Ideograph Extension A */
  ["\u46B9", "\u46B9"], /* CJK Ideograph Extension A */
  ["\u46BA", "\u46BA"], /* CJK Ideograph Extension A */
  ["\u46BB", "\u46BB"], /* CJK Ideograph Extension A */
  ["\u46BC", "\u46BC"], /* CJK Ideograph Extension A */
  ["\u46BD", "\u46BD"], /* CJK Ideograph Extension A */
  ["\u46BE", "\u46BE"], /* CJK Ideograph Extension A */
  ["\u46BF", "\u46BF"], /* CJK Ideograph Extension A */
  ["\u46C0", "\u46C0"], /* CJK Ideograph Extension A */
  ["\u46C1", "\u46C1"], /* CJK Ideograph Extension A */
  ["\u46C2", "\u46C2"], /* CJK Ideograph Extension A */
  ["\u46C3", "\u46C3"], /* CJK Ideograph Extension A */
  ["\u46C4", "\u46C4"], /* CJK Ideograph Extension A */
  ["\u46C5", "\u46C5"], /* CJK Ideograph Extension A */
  ["\u46C6", "\u46C6"], /* CJK Ideograph Extension A */
  ["\u46C7", "\u46C7"], /* CJK Ideograph Extension A */
  ["\u46C8", "\u46C8"], /* CJK Ideograph Extension A */
  ["\u46C9", "\u46C9"], /* CJK Ideograph Extension A */
  ["\u46CA", "\u46CA"], /* CJK Ideograph Extension A */
  ["\u46CB", "\u46CB"], /* CJK Ideograph Extension A */
  ["\u46CC", "\u46CC"], /* CJK Ideograph Extension A */
  ["\u46CD", "\u46CD"], /* CJK Ideograph Extension A */
  ["\u46CE", "\u46CE"], /* CJK Ideograph Extension A */
  ["\u46CF", "\u46CF"], /* CJK Ideograph Extension A */
  ["\u46D0", "\u46D0"], /* CJK Ideograph Extension A */
  ["\u46D1", "\u46D1"], /* CJK Ideograph Extension A */
  ["\u46D2", "\u46D2"], /* CJK Ideograph Extension A */
  ["\u46D3", "\u46D3"], /* CJK Ideograph Extension A */
  ["\u46D4", "\u46D4"], /* CJK Ideograph Extension A */
  ["\u46D5", "\u46D5"], /* CJK Ideograph Extension A */
  ["\u46D6", "\u46D6"], /* CJK Ideograph Extension A */
  ["\u46D7", "\u46D7"], /* CJK Ideograph Extension A */
  ["\u46D8", "\u46D8"], /* CJK Ideograph Extension A */
  ["\u46D9", "\u46D9"], /* CJK Ideograph Extension A */
  ["\u46DA", "\u46DA"], /* CJK Ideograph Extension A */
  ["\u46DB", "\u46DB"], /* CJK Ideograph Extension A */
  ["\u46DC", "\u46DC"], /* CJK Ideograph Extension A */
  ["\u46DD", "\u46DD"], /* CJK Ideograph Extension A */
  ["\u46DE", "\u46DE"], /* CJK Ideograph Extension A */
  ["\u46DF", "\u46DF"], /* CJK Ideograph Extension A */
  ["\u46E0", "\u46E0"], /* CJK Ideograph Extension A */
  ["\u46E1", "\u46E1"], /* CJK Ideograph Extension A */
  ["\u46E2", "\u46E2"], /* CJK Ideograph Extension A */
  ["\u46E3", "\u46E3"], /* CJK Ideograph Extension A */
  ["\u46E4", "\u46E4"], /* CJK Ideograph Extension A */
  ["\u46E5", "\u46E5"], /* CJK Ideograph Extension A */
  ["\u46E6", "\u46E6"], /* CJK Ideograph Extension A */
  ["\u46E7", "\u46E7"], /* CJK Ideograph Extension A */
  ["\u46E8", "\u46E8"], /* CJK Ideograph Extension A */
  ["\u46E9", "\u46E9"], /* CJK Ideograph Extension A */
  ["\u46EA", "\u46EA"], /* CJK Ideograph Extension A */
  ["\u46EB", "\u46EB"], /* CJK Ideograph Extension A */
  ["\u46EC", "\u46EC"], /* CJK Ideograph Extension A */
  ["\u46ED", "\u46ED"], /* CJK Ideograph Extension A */
  ["\u46EE", "\u46EE"], /* CJK Ideograph Extension A */
  ["\u46EF", "\u46EF"], /* CJK Ideograph Extension A */
  ["\u46F0", "\u46F0"], /* CJK Ideograph Extension A */
  ["\u46F1", "\u46F1"], /* CJK Ideograph Extension A */
  ["\u46F2", "\u46F2"], /* CJK Ideograph Extension A */
  ["\u46F3", "\u46F3"], /* CJK Ideograph Extension A */
  ["\u46F4", "\u46F4"], /* CJK Ideograph Extension A */
  ["\u46F5", "\u46F5"], /* CJK Ideograph Extension A */
  ["\u46F6", "\u46F6"], /* CJK Ideograph Extension A */
  ["\u46F7", "\u46F7"], /* CJK Ideograph Extension A */
  ["\u46F8", "\u46F8"], /* CJK Ideograph Extension A */
  ["\u46F9", "\u46F9"], /* CJK Ideograph Extension A */
  ["\u46FA", "\u46FA"], /* CJK Ideograph Extension A */
  ["\u46FB", "\u46FB"], /* CJK Ideograph Extension A */
  ["\u46FC", "\u46FC"], /* CJK Ideograph Extension A */
  ["\u46FD", "\u46FD"], /* CJK Ideograph Extension A */
  ["\u46FE", "\u46FE"], /* CJK Ideograph Extension A */
  ["\u46FF", "\u46FF"], /* CJK Ideograph Extension A */
  ["\u4700", "\u4700"], /* CJK Ideograph Extension A */
  ["\u4701", "\u4701"], /* CJK Ideograph Extension A */
  ["\u4702", "\u4702"], /* CJK Ideograph Extension A */
  ["\u4703", "\u4703"], /* CJK Ideograph Extension A */
  ["\u4704", "\u4704"], /* CJK Ideograph Extension A */
  ["\u4705", "\u4705"], /* CJK Ideograph Extension A */
  ["\u4706", "\u4706"], /* CJK Ideograph Extension A */
  ["\u4707", "\u4707"], /* CJK Ideograph Extension A */
  ["\u4708", "\u4708"], /* CJK Ideograph Extension A */
  ["\u4709", "\u4709"], /* CJK Ideograph Extension A */
  ["\u470A", "\u470A"], /* CJK Ideograph Extension A */
  ["\u470B", "\u470B"], /* CJK Ideograph Extension A */
  ["\u470C", "\u470C"], /* CJK Ideograph Extension A */
  ["\u470D", "\u470D"], /* CJK Ideograph Extension A */
  ["\u470E", "\u470E"], /* CJK Ideograph Extension A */
  ["\u470F", "\u470F"], /* CJK Ideograph Extension A */
  ["\u4710", "\u4710"], /* CJK Ideograph Extension A */
  ["\u4711", "\u4711"], /* CJK Ideograph Extension A */
  ["\u4712", "\u4712"], /* CJK Ideograph Extension A */
  ["\u4713", "\u4713"], /* CJK Ideograph Extension A */
  ["\u4714", "\u4714"], /* CJK Ideograph Extension A */
  ["\u4715", "\u4715"], /* CJK Ideograph Extension A */
  ["\u4716", "\u4716"], /* CJK Ideograph Extension A */
  ["\u4717", "\u4717"], /* CJK Ideograph Extension A */
  ["\u4718", "\u4718"], /* CJK Ideograph Extension A */
  ["\u4719", "\u4719"], /* CJK Ideograph Extension A */
  ["\u471A", "\u471A"], /* CJK Ideograph Extension A */
  ["\u471B", "\u471B"], /* CJK Ideograph Extension A */
  ["\u471C", "\u471C"], /* CJK Ideograph Extension A */
  ["\u471D", "\u471D"], /* CJK Ideograph Extension A */
  ["\u471E", "\u471E"], /* CJK Ideograph Extension A */
  ["\u471F", "\u471F"], /* CJK Ideograph Extension A */
  ["\u4720", "\u4720"], /* CJK Ideograph Extension A */
  ["\u4721", "\u4721"], /* CJK Ideograph Extension A */
  ["\u4722", "\u4722"], /* CJK Ideograph Extension A */
  ["\u4723", "\u4723"], /* CJK Ideograph Extension A */
  ["\u4724", "\u4724"], /* CJK Ideograph Extension A */
  ["\u4725", "\u4725"], /* CJK Ideograph Extension A */
  ["\u4726", "\u4726"], /* CJK Ideograph Extension A */
  ["\u4727", "\u4727"], /* CJK Ideograph Extension A */
  ["\u4728", "\u4728"], /* CJK Ideograph Extension A */
  ["\u4729", "\u4729"], /* CJK Ideograph Extension A */
  ["\u472A", "\u472A"], /* CJK Ideograph Extension A */
  ["\u472B", "\u472B"], /* CJK Ideograph Extension A */
  ["\u472C", "\u472C"], /* CJK Ideograph Extension A */
  ["\u472D", "\u472D"], /* CJK Ideograph Extension A */
  ["\u472E", "\u472E"], /* CJK Ideograph Extension A */
  ["\u472F", "\u472F"], /* CJK Ideograph Extension A */
  ["\u4730", "\u4730"], /* CJK Ideograph Extension A */
  ["\u4731", "\u4731"], /* CJK Ideograph Extension A */
  ["\u4732", "\u4732"], /* CJK Ideograph Extension A */
  ["\u4733", "\u4733"], /* CJK Ideograph Extension A */
  ["\u4734", "\u4734"], /* CJK Ideograph Extension A */
  ["\u4735", "\u4735"], /* CJK Ideograph Extension A */
  ["\u4736", "\u4736"], /* CJK Ideograph Extension A */
  ["\u4737", "\u4737"], /* CJK Ideograph Extension A */
  ["\u4738", "\u4738"], /* CJK Ideograph Extension A */
  ["\u4739", "\u4739"], /* CJK Ideograph Extension A */
  ["\u473A", "\u473A"], /* CJK Ideograph Extension A */
  ["\u473B", "\u473B"], /* CJK Ideograph Extension A */
  ["\u473C", "\u473C"], /* CJK Ideograph Extension A */
  ["\u473D", "\u473D"], /* CJK Ideograph Extension A */
  ["\u473E", "\u473E"], /* CJK Ideograph Extension A */
  ["\u473F", "\u473F"], /* CJK Ideograph Extension A */
  ["\u4740", "\u4740"], /* CJK Ideograph Extension A */
  ["\u4741", "\u4741"], /* CJK Ideograph Extension A */
  ["\u4742", "\u4742"], /* CJK Ideograph Extension A */
  ["\u4743", "\u4743"], /* CJK Ideograph Extension A */
  ["\u4744", "\u4744"], /* CJK Ideograph Extension A */
  ["\u4745", "\u4745"], /* CJK Ideograph Extension A */
  ["\u4746", "\u4746"], /* CJK Ideograph Extension A */
  ["\u4747", "\u4747"], /* CJK Ideograph Extension A */
  ["\u4748", "\u4748"], /* CJK Ideograph Extension A */
  ["\u4749", "\u4749"], /* CJK Ideograph Extension A */
  ["\u474A", "\u474A"], /* CJK Ideograph Extension A */
  ["\u474B", "\u474B"], /* CJK Ideograph Extension A */
  ["\u474C", "\u474C"], /* CJK Ideograph Extension A */
  ["\u474D", "\u474D"], /* CJK Ideograph Extension A */
  ["\u474E", "\u474E"], /* CJK Ideograph Extension A */
  ["\u474F", "\u474F"], /* CJK Ideograph Extension A */
  ["\u4750", "\u4750"], /* CJK Ideograph Extension A */
  ["\u4751", "\u4751"], /* CJK Ideograph Extension A */
  ["\u4752", "\u4752"], /* CJK Ideograph Extension A */
  ["\u4753", "\u4753"], /* CJK Ideograph Extension A */
  ["\u4754", "\u4754"], /* CJK Ideograph Extension A */
  ["\u4755", "\u4755"], /* CJK Ideograph Extension A */
  ["\u4756", "\u4756"], /* CJK Ideograph Extension A */
  ["\u4757", "\u4757"], /* CJK Ideograph Extension A */
  ["\u4758", "\u4758"], /* CJK Ideograph Extension A */
  ["\u4759", "\u4759"], /* CJK Ideograph Extension A */
  ["\u475A", "\u475A"], /* CJK Ideograph Extension A */
  ["\u475B", "\u475B"], /* CJK Ideograph Extension A */
  ["\u475C", "\u475C"], /* CJK Ideograph Extension A */
  ["\u475D", "\u475D"], /* CJK Ideograph Extension A */
  ["\u475E", "\u475E"], /* CJK Ideograph Extension A */
  ["\u475F", "\u475F"], /* CJK Ideograph Extension A */
  ["\u4760", "\u4760"], /* CJK Ideograph Extension A */
  ["\u4761", "\u4761"], /* CJK Ideograph Extension A */
  ["\u4762", "\u4762"], /* CJK Ideograph Extension A */
  ["\u4763", "\u4763"], /* CJK Ideograph Extension A */
  ["\u4764", "\u4764"], /* CJK Ideograph Extension A */
  ["\u4765", "\u4765"], /* CJK Ideograph Extension A */
  ["\u4766", "\u4766"], /* CJK Ideograph Extension A */
  ["\u4767", "\u4767"], /* CJK Ideograph Extension A */
  ["\u4768", "\u4768"], /* CJK Ideograph Extension A */
  ["\u4769", "\u4769"], /* CJK Ideograph Extension A */
  ["\u476A", "\u476A"], /* CJK Ideograph Extension A */
  ["\u476B", "\u476B"], /* CJK Ideograph Extension A */
  ["\u476C", "\u476C"], /* CJK Ideograph Extension A */
  ["\u476D", "\u476D"], /* CJK Ideograph Extension A */
  ["\u476E", "\u476E"], /* CJK Ideograph Extension A */
  ["\u476F", "\u476F"], /* CJK Ideograph Extension A */
  ["\u4770", "\u4770"], /* CJK Ideograph Extension A */
  ["\u4771", "\u4771"], /* CJK Ideograph Extension A */
  ["\u4772", "\u4772"], /* CJK Ideograph Extension A */
  ["\u4773", "\u4773"], /* CJK Ideograph Extension A */
  ["\u4774", "\u4774"], /* CJK Ideograph Extension A */
  ["\u4775", "\u4775"], /* CJK Ideograph Extension A */
  ["\u4776", "\u4776"], /* CJK Ideograph Extension A */
  ["\u4777", "\u4777"], /* CJK Ideograph Extension A */
  ["\u4778", "\u4778"], /* CJK Ideograph Extension A */
  ["\u4779", "\u4779"], /* CJK Ideograph Extension A */
  ["\u477A", "\u477A"], /* CJK Ideograph Extension A */
  ["\u477B", "\u477B"], /* CJK Ideograph Extension A */
  ["\u477C", "\u477C"], /* CJK Ideograph Extension A */
  ["\u477D", "\u477D"], /* CJK Ideograph Extension A */
  ["\u477E", "\u477E"], /* CJK Ideograph Extension A */
  ["\u477F", "\u477F"], /* CJK Ideograph Extension A */
  ["\u4780", "\u4780"], /* CJK Ideograph Extension A */
  ["\u4781", "\u4781"], /* CJK Ideograph Extension A */
  ["\u4782", "\u4782"], /* CJK Ideograph Extension A */
  ["\u4783", "\u4783"], /* CJK Ideograph Extension A */
  ["\u4784", "\u4784"], /* CJK Ideograph Extension A */
  ["\u4785", "\u4785"], /* CJK Ideograph Extension A */
  ["\u4786", "\u4786"], /* CJK Ideograph Extension A */
  ["\u4787", "\u4787"], /* CJK Ideograph Extension A */
  ["\u4788", "\u4788"], /* CJK Ideograph Extension A */
  ["\u4789", "\u4789"], /* CJK Ideograph Extension A */
  ["\u478A", "\u478A"], /* CJK Ideograph Extension A */
  ["\u478B", "\u478B"], /* CJK Ideograph Extension A */
  ["\u478C", "\u478C"], /* CJK Ideograph Extension A */
  ["\u478D", "\u478D"], /* CJK Ideograph Extension A */
  ["\u478E", "\u478E"], /* CJK Ideograph Extension A */
  ["\u478F", "\u478F"], /* CJK Ideograph Extension A */
  ["\u4790", "\u4790"], /* CJK Ideograph Extension A */
  ["\u4791", "\u4791"], /* CJK Ideograph Extension A */
  ["\u4792", "\u4792"], /* CJK Ideograph Extension A */
  ["\u4793", "\u4793"], /* CJK Ideograph Extension A */
  ["\u4794", "\u4794"], /* CJK Ideograph Extension A */
  ["\u4795", "\u4795"], /* CJK Ideograph Extension A */
  ["\u4796", "\u4796"], /* CJK Ideograph Extension A */
  ["\u4797", "\u4797"], /* CJK Ideograph Extension A */
  ["\u4798", "\u4798"], /* CJK Ideograph Extension A */
  ["\u4799", "\u4799"], /* CJK Ideograph Extension A */
  ["\u479A", "\u479A"], /* CJK Ideograph Extension A */
  ["\u479B", "\u479B"], /* CJK Ideograph Extension A */
  ["\u479C", "\u479C"], /* CJK Ideograph Extension A */
  ["\u479D", "\u479D"], /* CJK Ideograph Extension A */
  ["\u479E", "\u479E"], /* CJK Ideograph Extension A */
  ["\u479F", "\u479F"], /* CJK Ideograph Extension A */
  ["\u47A0", "\u47A0"], /* CJK Ideograph Extension A */
  ["\u47A1", "\u47A1"], /* CJK Ideograph Extension A */
  ["\u47A2", "\u47A2"], /* CJK Ideograph Extension A */
  ["\u47A3", "\u47A3"], /* CJK Ideograph Extension A */
  ["\u47A4", "\u47A4"], /* CJK Ideograph Extension A */
  ["\u47A5", "\u47A5"], /* CJK Ideograph Extension A */
  ["\u47A6", "\u47A6"], /* CJK Ideograph Extension A */
  ["\u47A7", "\u47A7"], /* CJK Ideograph Extension A */
  ["\u47A8", "\u47A8"], /* CJK Ideograph Extension A */
  ["\u47A9", "\u47A9"], /* CJK Ideograph Extension A */
  ["\u47AA", "\u47AA"], /* CJK Ideograph Extension A */
  ["\u47AB", "\u47AB"], /* CJK Ideograph Extension A */
  ["\u47AC", "\u47AC"], /* CJK Ideograph Extension A */
  ["\u47AD", "\u47AD"], /* CJK Ideograph Extension A */
  ["\u47AE", "\u47AE"], /* CJK Ideograph Extension A */
  ["\u47AF", "\u47AF"], /* CJK Ideograph Extension A */
  ["\u47B0", "\u47B0"], /* CJK Ideograph Extension A */
  ["\u47B1", "\u47B1"], /* CJK Ideograph Extension A */
  ["\u47B2", "\u47B2"], /* CJK Ideograph Extension A */
  ["\u47B3", "\u47B3"], /* CJK Ideograph Extension A */
  ["\u47B4", "\u47B4"], /* CJK Ideograph Extension A */
  ["\u47B5", "\u47B5"], /* CJK Ideograph Extension A */
  ["\u47B6", "\u47B6"], /* CJK Ideograph Extension A */
  ["\u47B7", "\u47B7"], /* CJK Ideograph Extension A */
  ["\u47B8", "\u47B8"], /* CJK Ideograph Extension A */
  ["\u47B9", "\u47B9"], /* CJK Ideograph Extension A */
  ["\u47BA", "\u47BA"], /* CJK Ideograph Extension A */
  ["\u47BB", "\u47BB"], /* CJK Ideograph Extension A */
  ["\u47BC", "\u47BC"], /* CJK Ideograph Extension A */
  ["\u47BD", "\u47BD"], /* CJK Ideograph Extension A */
  ["\u47BE", "\u47BE"], /* CJK Ideograph Extension A */
  ["\u47BF", "\u47BF"], /* CJK Ideograph Extension A */
  ["\u47C0", "\u47C0"], /* CJK Ideograph Extension A */
  ["\u47C1", "\u47C1"], /* CJK Ideograph Extension A */
  ["\u47C2", "\u47C2"], /* CJK Ideograph Extension A */
  ["\u47C3", "\u47C3"], /* CJK Ideograph Extension A */
  ["\u47C4", "\u47C4"], /* CJK Ideograph Extension A */
  ["\u47C5", "\u47C5"], /* CJK Ideograph Extension A */
  ["\u47C6", "\u47C6"], /* CJK Ideograph Extension A */
  ["\u47C7", "\u47C7"], /* CJK Ideograph Extension A */
  ["\u47C8", "\u47C8"], /* CJK Ideograph Extension A */
  ["\u47C9", "\u47C9"], /* CJK Ideograph Extension A */
  ["\u47CA", "\u47CA"], /* CJK Ideograph Extension A */
  ["\u47CB", "\u47CB"], /* CJK Ideograph Extension A */
  ["\u47CC", "\u47CC"], /* CJK Ideograph Extension A */
  ["\u47CD", "\u47CD"], /* CJK Ideograph Extension A */
  ["\u47CE", "\u47CE"], /* CJK Ideograph Extension A */
  ["\u47CF", "\u47CF"], /* CJK Ideograph Extension A */
  ["\u47D0", "\u47D0"], /* CJK Ideograph Extension A */
  ["\u47D1", "\u47D1"], /* CJK Ideograph Extension A */
  ["\u47D2", "\u47D2"], /* CJK Ideograph Extension A */
  ["\u47D3", "\u47D3"], /* CJK Ideograph Extension A */
  ["\u47D4", "\u47D4"], /* CJK Ideograph Extension A */
  ["\u47D5", "\u47D5"], /* CJK Ideograph Extension A */
  ["\u47D6", "\u47D6"], /* CJK Ideograph Extension A */
  ["\u47D7", "\u47D7"], /* CJK Ideograph Extension A */
  ["\u47D8", "\u47D8"], /* CJK Ideograph Extension A */
  ["\u47D9", "\u47D9"], /* CJK Ideograph Extension A */
  ["\u47DA", "\u47DA"], /* CJK Ideograph Extension A */
  ["\u47DB", "\u47DB"], /* CJK Ideograph Extension A */
  ["\u47DC", "\u47DC"], /* CJK Ideograph Extension A */
  ["\u47DD", "\u47DD"], /* CJK Ideograph Extension A */
  ["\u47DE", "\u47DE"], /* CJK Ideograph Extension A */
  ["\u47DF", "\u47DF"], /* CJK Ideograph Extension A */
  ["\u47E0", "\u47E0"], /* CJK Ideograph Extension A */
  ["\u47E1", "\u47E1"], /* CJK Ideograph Extension A */
  ["\u47E2", "\u47E2"], /* CJK Ideograph Extension A */
  ["\u47E3", "\u47E3"], /* CJK Ideograph Extension A */
  ["\u47E4", "\u47E4"], /* CJK Ideograph Extension A */
  ["\u47E5", "\u47E5"], /* CJK Ideograph Extension A */
  ["\u47E6", "\u47E6"], /* CJK Ideograph Extension A */
  ["\u47E7", "\u47E7"], /* CJK Ideograph Extension A */
  ["\u47E8", "\u47E8"], /* CJK Ideograph Extension A */
  ["\u47E9", "\u47E9"], /* CJK Ideograph Extension A */
  ["\u47EA", "\u47EA"], /* CJK Ideograph Extension A */
  ["\u47EB", "\u47EB"], /* CJK Ideograph Extension A */
  ["\u47EC", "\u47EC"], /* CJK Ideograph Extension A */
  ["\u47ED", "\u47ED"], /* CJK Ideograph Extension A */
  ["\u47EE", "\u47EE"], /* CJK Ideograph Extension A */
  ["\u47EF", "\u47EF"], /* CJK Ideograph Extension A */
  ["\u47F0", "\u47F0"], /* CJK Ideograph Extension A */
  ["\u47F1", "\u47F1"], /* CJK Ideograph Extension A */
  ["\u47F2", "\u47F2"], /* CJK Ideograph Extension A */
  ["\u47F3", "\u47F3"], /* CJK Ideograph Extension A */
  ["\u47F4", "\u47F4"], /* CJK Ideograph Extension A */
  ["\u47F5", "\u47F5"], /* CJK Ideograph Extension A */
  ["\u47F6", "\u47F6"], /* CJK Ideograph Extension A */
  ["\u47F7", "\u47F7"], /* CJK Ideograph Extension A */
  ["\u47F8", "\u47F8"], /* CJK Ideograph Extension A */
  ["\u47F9", "\u47F9"], /* CJK Ideograph Extension A */
  ["\u47FA", "\u47FA"], /* CJK Ideograph Extension A */
  ["\u47FB", "\u47FB"], /* CJK Ideograph Extension A */
  ["\u47FC", "\u47FC"], /* CJK Ideograph Extension A */
  ["\u47FD", "\u47FD"], /* CJK Ideograph Extension A */
  ["\u47FE", "\u47FE"], /* CJK Ideograph Extension A */
  ["\u47FF", "\u47FF"], /* CJK Ideograph Extension A */
  ["\u4800", "\u4800"], /* CJK Ideograph Extension A */
  ["\u4801", "\u4801"], /* CJK Ideograph Extension A */
  ["\u4802", "\u4802"], /* CJK Ideograph Extension A */
  ["\u4803", "\u4803"], /* CJK Ideograph Extension A */
  ["\u4804", "\u4804"], /* CJK Ideograph Extension A */
  ["\u4805", "\u4805"], /* CJK Ideograph Extension A */
  ["\u4806", "\u4806"], /* CJK Ideograph Extension A */
  ["\u4807", "\u4807"], /* CJK Ideograph Extension A */
  ["\u4808", "\u4808"], /* CJK Ideograph Extension A */
  ["\u4809", "\u4809"], /* CJK Ideograph Extension A */
  ["\u480A", "\u480A"], /* CJK Ideograph Extension A */
  ["\u480B", "\u480B"], /* CJK Ideograph Extension A */
  ["\u480C", "\u480C"], /* CJK Ideograph Extension A */
  ["\u480D", "\u480D"], /* CJK Ideograph Extension A */
  ["\u480E", "\u480E"], /* CJK Ideograph Extension A */
  ["\u480F", "\u480F"], /* CJK Ideograph Extension A */
  ["\u4810", "\u4810"], /* CJK Ideograph Extension A */
  ["\u4811", "\u4811"], /* CJK Ideograph Extension A */
  ["\u4812", "\u4812"], /* CJK Ideograph Extension A */
  ["\u4813", "\u4813"], /* CJK Ideograph Extension A */
  ["\u4814", "\u4814"], /* CJK Ideograph Extension A */
  ["\u4815", "\u4815"], /* CJK Ideograph Extension A */
  ["\u4816", "\u4816"], /* CJK Ideograph Extension A */
  ["\u4817", "\u4817"], /* CJK Ideograph Extension A */
  ["\u4818", "\u4818"], /* CJK Ideograph Extension A */
  ["\u4819", "\u4819"], /* CJK Ideograph Extension A */
  ["\u481A", "\u481A"], /* CJK Ideograph Extension A */
  ["\u481B", "\u481B"], /* CJK Ideograph Extension A */
  ["\u481C", "\u481C"], /* CJK Ideograph Extension A */
  ["\u481D", "\u481D"], /* CJK Ideograph Extension A */
  ["\u481E", "\u481E"], /* CJK Ideograph Extension A */
  ["\u481F", "\u481F"], /* CJK Ideograph Extension A */
  ["\u4820", "\u4820"], /* CJK Ideograph Extension A */
  ["\u4821", "\u4821"], /* CJK Ideograph Extension A */
  ["\u4822", "\u4822"], /* CJK Ideograph Extension A */
  ["\u4823", "\u4823"], /* CJK Ideograph Extension A */
  ["\u4824", "\u4824"], /* CJK Ideograph Extension A */
  ["\u4825", "\u4825"], /* CJK Ideograph Extension A */
  ["\u4826", "\u4826"], /* CJK Ideograph Extension A */
  ["\u4827", "\u4827"], /* CJK Ideograph Extension A */
  ["\u4828", "\u4828"], /* CJK Ideograph Extension A */
  ["\u4829", "\u4829"], /* CJK Ideograph Extension A */
  ["\u482A", "\u482A"], /* CJK Ideograph Extension A */
  ["\u482B", "\u482B"], /* CJK Ideograph Extension A */
  ["\u482C", "\u482C"], /* CJK Ideograph Extension A */
  ["\u482D", "\u482D"], /* CJK Ideograph Extension A */
  ["\u482E", "\u482E"], /* CJK Ideograph Extension A */
  ["\u482F", "\u482F"], /* CJK Ideograph Extension A */
  ["\u4830", "\u4830"], /* CJK Ideograph Extension A */
  ["\u4831", "\u4831"], /* CJK Ideograph Extension A */
  ["\u4832", "\u4832"], /* CJK Ideograph Extension A */
  ["\u4833", "\u4833"], /* CJK Ideograph Extension A */
  ["\u4834", "\u4834"], /* CJK Ideograph Extension A */
  ["\u4835", "\u4835"], /* CJK Ideograph Extension A */
  ["\u4836", "\u4836"], /* CJK Ideograph Extension A */
  ["\u4837", "\u4837"], /* CJK Ideograph Extension A */
  ["\u4838", "\u4838"], /* CJK Ideograph Extension A */
  ["\u4839", "\u4839"], /* CJK Ideograph Extension A */
  ["\u483A", "\u483A"], /* CJK Ideograph Extension A */
  ["\u483B", "\u483B"], /* CJK Ideograph Extension A */
  ["\u483C", "\u483C"], /* CJK Ideograph Extension A */
  ["\u483D", "\u483D"], /* CJK Ideograph Extension A */
  ["\u483E", "\u483E"], /* CJK Ideograph Extension A */
  ["\u483F", "\u483F"], /* CJK Ideograph Extension A */
  ["\u4840", "\u4840"], /* CJK Ideograph Extension A */
  ["\u4841", "\u4841"], /* CJK Ideograph Extension A */
  ["\u4842", "\u4842"], /* CJK Ideograph Extension A */
  ["\u4843", "\u4843"], /* CJK Ideograph Extension A */
  ["\u4844", "\u4844"], /* CJK Ideograph Extension A */
  ["\u4845", "\u4845"], /* CJK Ideograph Extension A */
  ["\u4846", "\u4846"], /* CJK Ideograph Extension A */
  ["\u4847", "\u4847"], /* CJK Ideograph Extension A */
  ["\u4848", "\u4848"], /* CJK Ideograph Extension A */
  ["\u4849", "\u4849"], /* CJK Ideograph Extension A */
  ["\u484A", "\u484A"], /* CJK Ideograph Extension A */
  ["\u484B", "\u484B"], /* CJK Ideograph Extension A */
  ["\u484C", "\u484C"], /* CJK Ideograph Extension A */
  ["\u484D", "\u484D"], /* CJK Ideograph Extension A */
  ["\u484E", "\u484E"], /* CJK Ideograph Extension A */
  ["\u484F", "\u484F"], /* CJK Ideograph Extension A */
  ["\u4850", "\u4850"], /* CJK Ideograph Extension A */
  ["\u4851", "\u4851"], /* CJK Ideograph Extension A */
  ["\u4852", "\u4852"], /* CJK Ideograph Extension A */
  ["\u4853", "\u4853"], /* CJK Ideograph Extension A */
  ["\u4854", "\u4854"], /* CJK Ideograph Extension A */
  ["\u4855", "\u4855"], /* CJK Ideograph Extension A */
  ["\u4856", "\u4856"], /* CJK Ideograph Extension A */
  ["\u4857", "\u4857"], /* CJK Ideograph Extension A */
  ["\u4858", "\u4858"], /* CJK Ideograph Extension A */
  ["\u4859", "\u4859"], /* CJK Ideograph Extension A */
  ["\u485A", "\u485A"], /* CJK Ideograph Extension A */
  ["\u485B", "\u485B"], /* CJK Ideograph Extension A */
  ["\u485C", "\u485C"], /* CJK Ideograph Extension A */
  ["\u485D", "\u485D"], /* CJK Ideograph Extension A */
  ["\u485E", "\u485E"], /* CJK Ideograph Extension A */
  ["\u485F", "\u485F"], /* CJK Ideograph Extension A */
  ["\u4860", "\u4860"], /* CJK Ideograph Extension A */
  ["\u4861", "\u4861"], /* CJK Ideograph Extension A */
  ["\u4862", "\u4862"], /* CJK Ideograph Extension A */
  ["\u4863", "\u4863"], /* CJK Ideograph Extension A */
  ["\u4864", "\u4864"], /* CJK Ideograph Extension A */
  ["\u4865", "\u4865"], /* CJK Ideograph Extension A */
  ["\u4866", "\u4866"], /* CJK Ideograph Extension A */
  ["\u4867", "\u4867"], /* CJK Ideograph Extension A */
  ["\u4868", "\u4868"], /* CJK Ideograph Extension A */
  ["\u4869", "\u4869"], /* CJK Ideograph Extension A */
  ["\u486A", "\u486A"], /* CJK Ideograph Extension A */
  ["\u486B", "\u486B"], /* CJK Ideograph Extension A */
  ["\u486C", "\u486C"], /* CJK Ideograph Extension A */
  ["\u486D", "\u486D"], /* CJK Ideograph Extension A */
  ["\u486E", "\u486E"], /* CJK Ideograph Extension A */
  ["\u486F", "\u486F"], /* CJK Ideograph Extension A */
  ["\u4870", "\u4870"], /* CJK Ideograph Extension A */
  ["\u4871", "\u4871"], /* CJK Ideograph Extension A */
  ["\u4872", "\u4872"], /* CJK Ideograph Extension A */
  ["\u4873", "\u4873"], /* CJK Ideograph Extension A */
  ["\u4874", "\u4874"], /* CJK Ideograph Extension A */
  ["\u4875", "\u4875"], /* CJK Ideograph Extension A */
  ["\u4876", "\u4876"], /* CJK Ideograph Extension A */
  ["\u4877", "\u4877"], /* CJK Ideograph Extension A */
  ["\u4878", "\u4878"], /* CJK Ideograph Extension A */
  ["\u4879", "\u4879"], /* CJK Ideograph Extension A */
  ["\u487A", "\u487A"], /* CJK Ideograph Extension A */
  ["\u487B", "\u487B"], /* CJK Ideograph Extension A */
  ["\u487C", "\u487C"], /* CJK Ideograph Extension A */
  ["\u487D", "\u487D"], /* CJK Ideograph Extension A */
  ["\u487E", "\u487E"], /* CJK Ideograph Extension A */
  ["\u487F", "\u487F"], /* CJK Ideograph Extension A */
  ["\u4880", "\u4880"], /* CJK Ideograph Extension A */
  ["\u4881", "\u4881"], /* CJK Ideograph Extension A */
  ["\u4882", "\u4882"], /* CJK Ideograph Extension A */
  ["\u4883", "\u4883"], /* CJK Ideograph Extension A */
  ["\u4884", "\u4884"], /* CJK Ideograph Extension A */
  ["\u4885", "\u4885"], /* CJK Ideograph Extension A */
  ["\u4886", "\u4886"], /* CJK Ideograph Extension A */
  ["\u4887", "\u4887"], /* CJK Ideograph Extension A */
  ["\u4888", "\u4888"], /* CJK Ideograph Extension A */
  ["\u4889", "\u4889"], /* CJK Ideograph Extension A */
  ["\u488A", "\u488A"], /* CJK Ideograph Extension A */
  ["\u488B", "\u488B"], /* CJK Ideograph Extension A */
  ["\u488C", "\u488C"], /* CJK Ideograph Extension A */
  ["\u488D", "\u488D"], /* CJK Ideograph Extension A */
  ["\u488E", "\u488E"], /* CJK Ideograph Extension A */
  ["\u488F", "\u488F"], /* CJK Ideograph Extension A */
  ["\u4890", "\u4890"], /* CJK Ideograph Extension A */
  ["\u4891", "\u4891"], /* CJK Ideograph Extension A */
  ["\u4892", "\u4892"], /* CJK Ideograph Extension A */
  ["\u4893", "\u4893"], /* CJK Ideograph Extension A */
  ["\u4894", "\u4894"], /* CJK Ideograph Extension A */
  ["\u4895", "\u4895"], /* CJK Ideograph Extension A */
  ["\u4896", "\u4896"], /* CJK Ideograph Extension A */
  ["\u4897", "\u4897"], /* CJK Ideograph Extension A */
  ["\u4898", "\u4898"], /* CJK Ideograph Extension A */
  ["\u4899", "\u4899"], /* CJK Ideograph Extension A */
  ["\u489A", "\u489A"], /* CJK Ideograph Extension A */
  ["\u489B", "\u489B"], /* CJK Ideograph Extension A */
  ["\u489C", "\u489C"], /* CJK Ideograph Extension A */
  ["\u489D", "\u489D"], /* CJK Ideograph Extension A */
  ["\u489E", "\u489E"], /* CJK Ideograph Extension A */
  ["\u489F", "\u489F"], /* CJK Ideograph Extension A */
  ["\u48A0", "\u48A0"], /* CJK Ideograph Extension A */
  ["\u48A1", "\u48A1"], /* CJK Ideograph Extension A */
  ["\u48A2", "\u48A2"], /* CJK Ideograph Extension A */
  ["\u48A3", "\u48A3"], /* CJK Ideograph Extension A */
  ["\u48A4", "\u48A4"], /* CJK Ideograph Extension A */
  ["\u48A5", "\u48A5"], /* CJK Ideograph Extension A */
  ["\u48A6", "\u48A6"], /* CJK Ideograph Extension A */
  ["\u48A7", "\u48A7"], /* CJK Ideograph Extension A */
  ["\u48A8", "\u48A8"], /* CJK Ideograph Extension A */
  ["\u48A9", "\u48A9"], /* CJK Ideograph Extension A */
  ["\u48AA", "\u48AA"], /* CJK Ideograph Extension A */
  ["\u48AB", "\u48AB"], /* CJK Ideograph Extension A */
  ["\u48AC", "\u48AC"], /* CJK Ideograph Extension A */
  ["\u48AD", "\u48AD"], /* CJK Ideograph Extension A */
  ["\u48AE", "\u48AE"], /* CJK Ideograph Extension A */
  ["\u48AF", "\u48AF"], /* CJK Ideograph Extension A */
  ["\u48B0", "\u48B0"], /* CJK Ideograph Extension A */
  ["\u48B1", "\u48B1"], /* CJK Ideograph Extension A */
  ["\u48B2", "\u48B2"], /* CJK Ideograph Extension A */
  ["\u48B3", "\u48B3"], /* CJK Ideograph Extension A */
  ["\u48B4", "\u48B4"], /* CJK Ideograph Extension A */
  ["\u48B5", "\u48B5"], /* CJK Ideograph Extension A */
  ["\u48B6", "\u48B6"], /* CJK Ideograph Extension A */
  ["\u48B7", "\u48B7"], /* CJK Ideograph Extension A */
  ["\u48B8", "\u48B8"], /* CJK Ideograph Extension A */
  ["\u48B9", "\u48B9"], /* CJK Ideograph Extension A */
  ["\u48BA", "\u48BA"], /* CJK Ideograph Extension A */
  ["\u48BB", "\u48BB"], /* CJK Ideograph Extension A */
  ["\u48BC", "\u48BC"], /* CJK Ideograph Extension A */
  ["\u48BD", "\u48BD"], /* CJK Ideograph Extension A */
  ["\u48BE", "\u48BE"], /* CJK Ideograph Extension A */
  ["\u48BF", "\u48BF"], /* CJK Ideograph Extension A */
  ["\u48C0", "\u48C0"], /* CJK Ideograph Extension A */
  ["\u48C1", "\u48C1"], /* CJK Ideograph Extension A */
  ["\u48C2", "\u48C2"], /* CJK Ideograph Extension A */
  ["\u48C3", "\u48C3"], /* CJK Ideograph Extension A */
  ["\u48C4", "\u48C4"], /* CJK Ideograph Extension A */
  ["\u48C5", "\u48C5"], /* CJK Ideograph Extension A */
  ["\u48C6", "\u48C6"], /* CJK Ideograph Extension A */
  ["\u48C7", "\u48C7"], /* CJK Ideograph Extension A */
  ["\u48C8", "\u48C8"], /* CJK Ideograph Extension A */
  ["\u48C9", "\u48C9"], /* CJK Ideograph Extension A */
  ["\u48CA", "\u48CA"], /* CJK Ideograph Extension A */
  ["\u48CB", "\u48CB"], /* CJK Ideograph Extension A */
  ["\u48CC", "\u48CC"], /* CJK Ideograph Extension A */
  ["\u48CD", "\u48CD"], /* CJK Ideograph Extension A */
  ["\u48CE", "\u48CE"], /* CJK Ideograph Extension A */
  ["\u48CF", "\u48CF"], /* CJK Ideograph Extension A */
  ["\u48D0", "\u48D0"], /* CJK Ideograph Extension A */
  ["\u48D1", "\u48D1"], /* CJK Ideograph Extension A */
  ["\u48D2", "\u48D2"], /* CJK Ideograph Extension A */
  ["\u48D3", "\u48D3"], /* CJK Ideograph Extension A */
  ["\u48D4", "\u48D4"], /* CJK Ideograph Extension A */
  ["\u48D5", "\u48D5"], /* CJK Ideograph Extension A */
  ["\u48D6", "\u48D6"], /* CJK Ideograph Extension A */
  ["\u48D7", "\u48D7"], /* CJK Ideograph Extension A */
  ["\u48D8", "\u48D8"], /* CJK Ideograph Extension A */
  ["\u48D9", "\u48D9"], /* CJK Ideograph Extension A */
  ["\u48DA", "\u48DA"], /* CJK Ideograph Extension A */
  ["\u48DB", "\u48DB"], /* CJK Ideograph Extension A */
  ["\u48DC", "\u48DC"], /* CJK Ideograph Extension A */
  ["\u48DD", "\u48DD"], /* CJK Ideograph Extension A */
  ["\u48DE", "\u48DE"], /* CJK Ideograph Extension A */
  ["\u48DF", "\u48DF"], /* CJK Ideograph Extension A */
  ["\u48E0", "\u48E0"], /* CJK Ideograph Extension A */
  ["\u48E1", "\u48E1"], /* CJK Ideograph Extension A */
  ["\u48E2", "\u48E2"], /* CJK Ideograph Extension A */
  ["\u48E3", "\u48E3"], /* CJK Ideograph Extension A */
  ["\u48E4", "\u48E4"], /* CJK Ideograph Extension A */
  ["\u48E5", "\u48E5"], /* CJK Ideograph Extension A */
  ["\u48E6", "\u48E6"], /* CJK Ideograph Extension A */
  ["\u48E7", "\u48E7"], /* CJK Ideograph Extension A */
  ["\u48E8", "\u48E8"], /* CJK Ideograph Extension A */
  ["\u48E9", "\u48E9"], /* CJK Ideograph Extension A */
  ["\u48EA", "\u48EA"], /* CJK Ideograph Extension A */
  ["\u48EB", "\u48EB"], /* CJK Ideograph Extension A */
  ["\u48EC", "\u48EC"], /* CJK Ideograph Extension A */
  ["\u48ED", "\u48ED"], /* CJK Ideograph Extension A */
  ["\u48EE", "\u48EE"], /* CJK Ideograph Extension A */
  ["\u48EF", "\u48EF"], /* CJK Ideograph Extension A */
  ["\u48F0", "\u48F0"], /* CJK Ideograph Extension A */
  ["\u48F1", "\u48F1"], /* CJK Ideograph Extension A */
  ["\u48F2", "\u48F2"], /* CJK Ideograph Extension A */
  ["\u48F3", "\u48F3"], /* CJK Ideograph Extension A */
  ["\u48F4", "\u48F4"], /* CJK Ideograph Extension A */
  ["\u48F5", "\u48F5"], /* CJK Ideograph Extension A */
  ["\u48F6", "\u48F6"], /* CJK Ideograph Extension A */
  ["\u48F7", "\u48F7"], /* CJK Ideograph Extension A */
  ["\u48F8", "\u48F8"], /* CJK Ideograph Extension A */
  ["\u48F9", "\u48F9"], /* CJK Ideograph Extension A */
  ["\u48FA", "\u48FA"], /* CJK Ideograph Extension A */
  ["\u48FB", "\u48FB"], /* CJK Ideograph Extension A */
  ["\u48FC", "\u48FC"], /* CJK Ideograph Extension A */
  ["\u48FD", "\u48FD"], /* CJK Ideograph Extension A */
  ["\u48FE", "\u48FE"], /* CJK Ideograph Extension A */
  ["\u48FF", "\u48FF"], /* CJK Ideograph Extension A */
  ["\u4900", "\u4900"], /* CJK Ideograph Extension A */
  ["\u4901", "\u4901"], /* CJK Ideograph Extension A */
  ["\u4902", "\u4902"], /* CJK Ideograph Extension A */
  ["\u4903", "\u4903"], /* CJK Ideograph Extension A */
  ["\u4904", "\u4904"], /* CJK Ideograph Extension A */
  ["\u4905", "\u4905"], /* CJK Ideograph Extension A */
  ["\u4906", "\u4906"], /* CJK Ideograph Extension A */
  ["\u4907", "\u4907"], /* CJK Ideograph Extension A */
  ["\u4908", "\u4908"], /* CJK Ideograph Extension A */
  ["\u4909", "\u4909"], /* CJK Ideograph Extension A */
  ["\u490A", "\u490A"], /* CJK Ideograph Extension A */
  ["\u490B", "\u490B"], /* CJK Ideograph Extension A */
  ["\u490C", "\u490C"], /* CJK Ideograph Extension A */
  ["\u490D", "\u490D"], /* CJK Ideograph Extension A */
  ["\u490E", "\u490E"], /* CJK Ideograph Extension A */
  ["\u490F", "\u490F"], /* CJK Ideograph Extension A */
  ["\u4910", "\u4910"], /* CJK Ideograph Extension A */
  ["\u4911", "\u4911"], /* CJK Ideograph Extension A */
  ["\u4912", "\u4912"], /* CJK Ideograph Extension A */
  ["\u4913", "\u4913"], /* CJK Ideograph Extension A */
  ["\u4914", "\u4914"], /* CJK Ideograph Extension A */
  ["\u4915", "\u4915"], /* CJK Ideograph Extension A */
  ["\u4916", "\u4916"], /* CJK Ideograph Extension A */
  ["\u4917", "\u4917"], /* CJK Ideograph Extension A */
  ["\u4918", "\u4918"], /* CJK Ideograph Extension A */
  ["\u4919", "\u4919"], /* CJK Ideograph Extension A */
  ["\u491A", "\u491A"], /* CJK Ideograph Extension A */
  ["\u491B", "\u491B"], /* CJK Ideograph Extension A */
  ["\u491C", "\u491C"], /* CJK Ideograph Extension A */
  ["\u491D", "\u491D"], /* CJK Ideograph Extension A */
  ["\u491E", "\u491E"], /* CJK Ideograph Extension A */
  ["\u491F", "\u491F"], /* CJK Ideograph Extension A */
  ["\u4920", "\u4920"], /* CJK Ideograph Extension A */
  ["\u4921", "\u4921"], /* CJK Ideograph Extension A */
  ["\u4922", "\u4922"], /* CJK Ideograph Extension A */
  ["\u4923", "\u4923"], /* CJK Ideograph Extension A */
  ["\u4924", "\u4924"], /* CJK Ideograph Extension A */
  ["\u4925", "\u4925"], /* CJK Ideograph Extension A */
  ["\u4926", "\u4926"], /* CJK Ideograph Extension A */
  ["\u4927", "\u4927"], /* CJK Ideograph Extension A */
  ["\u4928", "\u4928"], /* CJK Ideograph Extension A */
  ["\u4929", "\u4929"], /* CJK Ideograph Extension A */
  ["\u492A", "\u492A"], /* CJK Ideograph Extension A */
  ["\u492B", "\u492B"], /* CJK Ideograph Extension A */
  ["\u492C", "\u492C"], /* CJK Ideograph Extension A */
  ["\u492D", "\u492D"], /* CJK Ideograph Extension A */
  ["\u492E", "\u492E"], /* CJK Ideograph Extension A */
  ["\u492F", "\u492F"], /* CJK Ideograph Extension A */
  ["\u4930", "\u4930"], /* CJK Ideograph Extension A */
  ["\u4931", "\u4931"], /* CJK Ideograph Extension A */
  ["\u4932", "\u4932"], /* CJK Ideograph Extension A */
  ["\u4933", "\u4933"], /* CJK Ideograph Extension A */
  ["\u4934", "\u4934"], /* CJK Ideograph Extension A */
  ["\u4935", "\u4935"], /* CJK Ideograph Extension A */
  ["\u4936", "\u4936"], /* CJK Ideograph Extension A */
  ["\u4937", "\u4937"], /* CJK Ideograph Extension A */
  ["\u4938", "\u4938"], /* CJK Ideograph Extension A */
  ["\u4939", "\u4939"], /* CJK Ideograph Extension A */
  ["\u493A", "\u493A"], /* CJK Ideograph Extension A */
  ["\u493B", "\u493B"], /* CJK Ideograph Extension A */
  ["\u493C", "\u493C"], /* CJK Ideograph Extension A */
  ["\u493D", "\u493D"], /* CJK Ideograph Extension A */
  ["\u493E", "\u493E"], /* CJK Ideograph Extension A */
  ["\u493F", "\u493F"], /* CJK Ideograph Extension A */
  ["\u4940", "\u4940"], /* CJK Ideograph Extension A */
  ["\u4941", "\u4941"], /* CJK Ideograph Extension A */
  ["\u4942", "\u4942"], /* CJK Ideograph Extension A */
  ["\u4943", "\u4943"], /* CJK Ideograph Extension A */
  ["\u4944", "\u4944"], /* CJK Ideograph Extension A */
  ["\u4945", "\u4945"], /* CJK Ideograph Extension A */
  ["\u4946", "\u4946"], /* CJK Ideograph Extension A */
  ["\u4947", "\u4947"], /* CJK Ideograph Extension A */
  ["\u4948", "\u4948"], /* CJK Ideograph Extension A */
  ["\u4949", "\u4949"], /* CJK Ideograph Extension A */
  ["\u494A", "\u494A"], /* CJK Ideograph Extension A */
  ["\u494B", "\u494B"], /* CJK Ideograph Extension A */
  ["\u494C", "\u494C"], /* CJK Ideograph Extension A */
  ["\u494D", "\u494D"], /* CJK Ideograph Extension A */
  ["\u494E", "\u494E"], /* CJK Ideograph Extension A */
  ["\u494F", "\u494F"], /* CJK Ideograph Extension A */
  ["\u4950", "\u4950"], /* CJK Ideograph Extension A */
  ["\u4951", "\u4951"], /* CJK Ideograph Extension A */
  ["\u4952", "\u4952"], /* CJK Ideograph Extension A */
  ["\u4953", "\u4953"], /* CJK Ideograph Extension A */
  ["\u4954", "\u4954"], /* CJK Ideograph Extension A */
  ["\u4955", "\u4955"], /* CJK Ideograph Extension A */
  ["\u4956", "\u4956"], /* CJK Ideograph Extension A */
  ["\u4957", "\u4957"], /* CJK Ideograph Extension A */
  ["\u4958", "\u4958"], /* CJK Ideograph Extension A */
  ["\u4959", "\u4959"], /* CJK Ideograph Extension A */
  ["\u495A", "\u495A"], /* CJK Ideograph Extension A */
  ["\u495B", "\u495B"], /* CJK Ideograph Extension A */
  ["\u495C", "\u495C"], /* CJK Ideograph Extension A */
  ["\u495D", "\u495D"], /* CJK Ideograph Extension A */
  ["\u495E", "\u495E"], /* CJK Ideograph Extension A */
  ["\u495F", "\u495F"], /* CJK Ideograph Extension A */
  ["\u4960", "\u4960"], /* CJK Ideograph Extension A */
  ["\u4961", "\u4961"], /* CJK Ideograph Extension A */
  ["\u4962", "\u4962"], /* CJK Ideograph Extension A */
  ["\u4963", "\u4963"], /* CJK Ideograph Extension A */
  ["\u4964", "\u4964"], /* CJK Ideograph Extension A */
  ["\u4965", "\u4965"], /* CJK Ideograph Extension A */
  ["\u4966", "\u4966"], /* CJK Ideograph Extension A */
  ["\u4967", "\u4967"], /* CJK Ideograph Extension A */
  ["\u4968", "\u4968"], /* CJK Ideograph Extension A */
  ["\u4969", "\u4969"], /* CJK Ideograph Extension A */
  ["\u496A", "\u496A"], /* CJK Ideograph Extension A */
  ["\u496B", "\u496B"], /* CJK Ideograph Extension A */
  ["\u496C", "\u496C"], /* CJK Ideograph Extension A */
  ["\u496D", "\u496D"], /* CJK Ideograph Extension A */
  ["\u496E", "\u496E"], /* CJK Ideograph Extension A */
  ["\u496F", "\u496F"], /* CJK Ideograph Extension A */
  ["\u4970", "\u4970"], /* CJK Ideograph Extension A */
  ["\u4971", "\u4971"], /* CJK Ideograph Extension A */
  ["\u4972", "\u4972"], /* CJK Ideograph Extension A */
  ["\u4973", "\u4973"], /* CJK Ideograph Extension A */
  ["\u4974", "\u4974"], /* CJK Ideograph Extension A */
  ["\u4975", "\u4975"], /* CJK Ideograph Extension A */
  ["\u4976", "\u4976"], /* CJK Ideograph Extension A */
  ["\u4977", "\u4977"], /* CJK Ideograph Extension A */
  ["\u4978", "\u4978"], /* CJK Ideograph Extension A */
  ["\u4979", "\u4979"], /* CJK Ideograph Extension A */
  ["\u497A", "\u497A"], /* CJK Ideograph Extension A */
  ["\u497B", "\u497B"], /* CJK Ideograph Extension A */
  ["\u497C", "\u497C"], /* CJK Ideograph Extension A */
  ["\u497D", "\u497D"], /* CJK Ideograph Extension A */
  ["\u497E", "\u497E"], /* CJK Ideograph Extension A */
  ["\u497F", "\u497F"], /* CJK Ideograph Extension A */
  ["\u4980", "\u4980"], /* CJK Ideograph Extension A */
  ["\u4981", "\u4981"], /* CJK Ideograph Extension A */
  ["\u4982", "\u4982"], /* CJK Ideograph Extension A */
  ["\u4983", "\u4983"], /* CJK Ideograph Extension A */
  ["\u4984", "\u4984"], /* CJK Ideograph Extension A */
  ["\u4985", "\u4985"], /* CJK Ideograph Extension A */
  ["\u4986", "\u4986"], /* CJK Ideograph Extension A */
  ["\u4987", "\u4987"], /* CJK Ideograph Extension A */
  ["\u4988", "\u4988"], /* CJK Ideograph Extension A */
  ["\u4989", "\u4989"], /* CJK Ideograph Extension A */
  ["\u498A", "\u498A"], /* CJK Ideograph Extension A */
  ["\u498B", "\u498B"], /* CJK Ideograph Extension A */
  ["\u498C", "\u498C"], /* CJK Ideograph Extension A */
  ["\u498D", "\u498D"], /* CJK Ideograph Extension A */
  ["\u498E", "\u498E"], /* CJK Ideograph Extension A */
  ["\u498F", "\u498F"], /* CJK Ideograph Extension A */
  ["\u4990", "\u4990"], /* CJK Ideograph Extension A */
  ["\u4991", "\u4991"], /* CJK Ideograph Extension A */
  ["\u4992", "\u4992"], /* CJK Ideograph Extension A */
  ["\u4993", "\u4993"], /* CJK Ideograph Extension A */
  ["\u4994", "\u4994"], /* CJK Ideograph Extension A */
  ["\u4995", "\u4995"], /* CJK Ideograph Extension A */
  ["\u4996", "\u4996"], /* CJK Ideograph Extension A */
  ["\u4997", "\u4997"], /* CJK Ideograph Extension A */
  ["\u4998", "\u4998"], /* CJK Ideograph Extension A */
  ["\u4999", "\u4999"], /* CJK Ideograph Extension A */
  ["\u499A", "\u499A"], /* CJK Ideograph Extension A */
  ["\u499B", "\u499B"], /* CJK Ideograph Extension A */
  ["\u499C", "\u499C"], /* CJK Ideograph Extension A */
  ["\u499D", "\u499D"], /* CJK Ideograph Extension A */
  ["\u499E", "\u499E"], /* CJK Ideograph Extension A */
  ["\u499F", "\u499F"], /* CJK Ideograph Extension A */
  ["\u49A0", "\u49A0"], /* CJK Ideograph Extension A */
  ["\u49A1", "\u49A1"], /* CJK Ideograph Extension A */
  ["\u49A2", "\u49A2"], /* CJK Ideograph Extension A */
  ["\u49A3", "\u49A3"], /* CJK Ideograph Extension A */
  ["\u49A4", "\u49A4"], /* CJK Ideograph Extension A */
  ["\u49A5", "\u49A5"], /* CJK Ideograph Extension A */
  ["\u49A6", "\u49A6"], /* CJK Ideograph Extension A */
  ["\u49A7", "\u49A7"], /* CJK Ideograph Extension A */
  ["\u49A8", "\u49A8"], /* CJK Ideograph Extension A */
  ["\u49A9", "\u49A9"], /* CJK Ideograph Extension A */
  ["\u49AA", "\u49AA"], /* CJK Ideograph Extension A */
  ["\u49AB", "\u49AB"], /* CJK Ideograph Extension A */
  ["\u49AC", "\u49AC"], /* CJK Ideograph Extension A */
  ["\u49AD", "\u49AD"], /* CJK Ideograph Extension A */
  ["\u49AE", "\u49AE"], /* CJK Ideograph Extension A */
  ["\u49AF", "\u49AF"], /* CJK Ideograph Extension A */
  ["\u49B0", "\u49B0"], /* CJK Ideograph Extension A */
  ["\u49B1", "\u49B1"], /* CJK Ideograph Extension A */
  ["\u49B2", "\u49B2"], /* CJK Ideograph Extension A */
  ["\u49B3", "\u49B3"], /* CJK Ideograph Extension A */
  ["\u49B4", "\u49B4"], /* CJK Ideograph Extension A */
  ["\u49B5", "\u49B5"], /* CJK Ideograph Extension A */
  ["\u49B6", "\u49B6"], /* CJK Ideograph Extension A */
  ["\u49B7", "\u49B7"], /* CJK Ideograph Extension A */
  ["\u49B8", "\u49B8"], /* CJK Ideograph Extension A */
  ["\u49B9", "\u49B9"], /* CJK Ideograph Extension A */
  ["\u49BA", "\u49BA"], /* CJK Ideograph Extension A */
  ["\u49BB", "\u49BB"], /* CJK Ideograph Extension A */
  ["\u49BC", "\u49BC"], /* CJK Ideograph Extension A */
  ["\u49BD", "\u49BD"], /* CJK Ideograph Extension A */
  ["\u49BE", "\u49BE"], /* CJK Ideograph Extension A */
  ["\u49BF", "\u49BF"], /* CJK Ideograph Extension A */
  ["\u49C0", "\u49C0"], /* CJK Ideograph Extension A */
  ["\u49C1", "\u49C1"], /* CJK Ideograph Extension A */
  ["\u49C2", "\u49C2"], /* CJK Ideograph Extension A */
  ["\u49C3", "\u49C3"], /* CJK Ideograph Extension A */
  ["\u49C4", "\u49C4"], /* CJK Ideograph Extension A */
  ["\u49C5", "\u49C5"], /* CJK Ideograph Extension A */
  ["\u49C6", "\u49C6"], /* CJK Ideograph Extension A */
  ["\u49C7", "\u49C7"], /* CJK Ideograph Extension A */
  ["\u49C8", "\u49C8"], /* CJK Ideograph Extension A */
  ["\u49C9", "\u49C9"], /* CJK Ideograph Extension A */
  ["\u49CA", "\u49CA"], /* CJK Ideograph Extension A */
  ["\u49CB", "\u49CB"], /* CJK Ideograph Extension A */
  ["\u49CC", "\u49CC"], /* CJK Ideograph Extension A */
  ["\u49CD", "\u49CD"], /* CJK Ideograph Extension A */
  ["\u49CE", "\u49CE"], /* CJK Ideograph Extension A */
  ["\u49CF", "\u49CF"], /* CJK Ideograph Extension A */
  ["\u49D0", "\u49D0"], /* CJK Ideograph Extension A */
  ["\u49D1", "\u49D1"], /* CJK Ideograph Extension A */
  ["\u49D2", "\u49D2"], /* CJK Ideograph Extension A */
  ["\u49D3", "\u49D3"], /* CJK Ideograph Extension A */
  ["\u49D4", "\u49D4"], /* CJK Ideograph Extension A */
  ["\u49D5", "\u49D5"], /* CJK Ideograph Extension A */
  ["\u49D6", "\u49D6"], /* CJK Ideograph Extension A */
  ["\u49D7", "\u49D7"], /* CJK Ideograph Extension A */
  ["\u49D8", "\u49D8"], /* CJK Ideograph Extension A */
  ["\u49D9", "\u49D9"], /* CJK Ideograph Extension A */
  ["\u49DA", "\u49DA"], /* CJK Ideograph Extension A */
  ["\u49DB", "\u49DB"], /* CJK Ideograph Extension A */
  ["\u49DC", "\u49DC"], /* CJK Ideograph Extension A */
  ["\u49DD", "\u49DD"], /* CJK Ideograph Extension A */
  ["\u49DE", "\u49DE"], /* CJK Ideograph Extension A */
  ["\u49DF", "\u49DF"], /* CJK Ideograph Extension A */
  ["\u49E0", "\u49E0"], /* CJK Ideograph Extension A */
  ["\u49E1", "\u49E1"], /* CJK Ideograph Extension A */
  ["\u49E2", "\u49E2"], /* CJK Ideograph Extension A */
  ["\u49E3", "\u49E3"], /* CJK Ideograph Extension A */
  ["\u49E4", "\u49E4"], /* CJK Ideograph Extension A */
  ["\u49E5", "\u49E5"], /* CJK Ideograph Extension A */
  ["\u49E6", "\u49E6"], /* CJK Ideograph Extension A */
  ["\u49E7", "\u49E7"], /* CJK Ideograph Extension A */
  ["\u49E8", "\u49E8"], /* CJK Ideograph Extension A */
  ["\u49E9", "\u49E9"], /* CJK Ideograph Extension A */
  ["\u49EA", "\u49EA"], /* CJK Ideograph Extension A */
  ["\u49EB", "\u49EB"], /* CJK Ideograph Extension A */
  ["\u49EC", "\u49EC"], /* CJK Ideograph Extension A */
  ["\u49ED", "\u49ED"], /* CJK Ideograph Extension A */
  ["\u49EE", "\u49EE"], /* CJK Ideograph Extension A */
  ["\u49EF", "\u49EF"], /* CJK Ideograph Extension A */
  ["\u49F0", "\u49F0"], /* CJK Ideograph Extension A */
  ["\u49F1", "\u49F1"], /* CJK Ideograph Extension A */
  ["\u49F2", "\u49F2"], /* CJK Ideograph Extension A */
  ["\u49F3", "\u49F3"], /* CJK Ideograph Extension A */
  ["\u49F4", "\u49F4"], /* CJK Ideograph Extension A */
  ["\u49F5", "\u49F5"], /* CJK Ideograph Extension A */
  ["\u49F6", "\u49F6"], /* CJK Ideograph Extension A */
  ["\u49F7", "\u49F7"], /* CJK Ideograph Extension A */
  ["\u49F8", "\u49F8"], /* CJK Ideograph Extension A */
  ["\u49F9", "\u49F9"], /* CJK Ideograph Extension A */
  ["\u49FA", "\u49FA"], /* CJK Ideograph Extension A */
  ["\u49FB", "\u49FB"], /* CJK Ideograph Extension A */
  ["\u49FC", "\u49FC"], /* CJK Ideograph Extension A */
  ["\u49FD", "\u49FD"], /* CJK Ideograph Extension A */
  ["\u49FE", "\u49FE"], /* CJK Ideograph Extension A */
  ["\u49FF", "\u49FF"], /* CJK Ideograph Extension A */
  ["\u4A00", "\u4A00"], /* CJK Ideograph Extension A */
  ["\u4A01", "\u4A01"], /* CJK Ideograph Extension A */
  ["\u4A02", "\u4A02"], /* CJK Ideograph Extension A */
  ["\u4A03", "\u4A03"], /* CJK Ideograph Extension A */
  ["\u4A04", "\u4A04"], /* CJK Ideograph Extension A */
  ["\u4A05", "\u4A05"], /* CJK Ideograph Extension A */
  ["\u4A06", "\u4A06"], /* CJK Ideograph Extension A */
  ["\u4A07", "\u4A07"], /* CJK Ideograph Extension A */
  ["\u4A08", "\u4A08"], /* CJK Ideograph Extension A */
  ["\u4A09", "\u4A09"], /* CJK Ideograph Extension A */
  ["\u4A0A", "\u4A0A"], /* CJK Ideograph Extension A */
  ["\u4A0B", "\u4A0B"], /* CJK Ideograph Extension A */
  ["\u4A0C", "\u4A0C"], /* CJK Ideograph Extension A */
  ["\u4A0D", "\u4A0D"], /* CJK Ideograph Extension A */
  ["\u4A0E", "\u4A0E"], /* CJK Ideograph Extension A */
  ["\u4A0F", "\u4A0F"], /* CJK Ideograph Extension A */
  ["\u4A10", "\u4A10"], /* CJK Ideograph Extension A */
  ["\u4A11", "\u4A11"], /* CJK Ideograph Extension A */
  ["\u4A12", "\u4A12"], /* CJK Ideograph Extension A */
  ["\u4A13", "\u4A13"], /* CJK Ideograph Extension A */
  ["\u4A14", "\u4A14"], /* CJK Ideograph Extension A */
  ["\u4A15", "\u4A15"], /* CJK Ideograph Extension A */
  ["\u4A16", "\u4A16"], /* CJK Ideograph Extension A */
  ["\u4A17", "\u4A17"], /* CJK Ideograph Extension A */
  ["\u4A18", "\u4A18"], /* CJK Ideograph Extension A */
  ["\u4A19", "\u4A19"], /* CJK Ideograph Extension A */
  ["\u4A1A", "\u4A1A"], /* CJK Ideograph Extension A */
  ["\u4A1B", "\u4A1B"], /* CJK Ideograph Extension A */
  ["\u4A1C", "\u4A1C"], /* CJK Ideograph Extension A */
  ["\u4A1D", "\u4A1D"], /* CJK Ideograph Extension A */
  ["\u4A1E", "\u4A1E"], /* CJK Ideograph Extension A */
  ["\u4A1F", "\u4A1F"], /* CJK Ideograph Extension A */
  ["\u4A20", "\u4A20"], /* CJK Ideograph Extension A */
  ["\u4A21", "\u4A21"], /* CJK Ideograph Extension A */
  ["\u4A22", "\u4A22"], /* CJK Ideograph Extension A */
  ["\u4A23", "\u4A23"], /* CJK Ideograph Extension A */
  ["\u4A24", "\u4A24"], /* CJK Ideograph Extension A */
  ["\u4A25", "\u4A25"], /* CJK Ideograph Extension A */
  ["\u4A26", "\u4A26"], /* CJK Ideograph Extension A */
  ["\u4A27", "\u4A27"], /* CJK Ideograph Extension A */
  ["\u4A28", "\u4A28"], /* CJK Ideograph Extension A */
  ["\u4A29", "\u4A29"], /* CJK Ideograph Extension A */
  ["\u4A2A", "\u4A2A"], /* CJK Ideograph Extension A */
  ["\u4A2B", "\u4A2B"], /* CJK Ideograph Extension A */
  ["\u4A2C", "\u4A2C"], /* CJK Ideograph Extension A */
  ["\u4A2D", "\u4A2D"], /* CJK Ideograph Extension A */
  ["\u4A2E", "\u4A2E"], /* CJK Ideograph Extension A */
  ["\u4A2F", "\u4A2F"], /* CJK Ideograph Extension A */
  ["\u4A30", "\u4A30"], /* CJK Ideograph Extension A */
  ["\u4A31", "\u4A31"], /* CJK Ideograph Extension A */
  ["\u4A32", "\u4A32"], /* CJK Ideograph Extension A */
  ["\u4A33", "\u4A33"], /* CJK Ideograph Extension A */
  ["\u4A34", "\u4A34"], /* CJK Ideograph Extension A */
  ["\u4A35", "\u4A35"], /* CJK Ideograph Extension A */
  ["\u4A36", "\u4A36"], /* CJK Ideograph Extension A */
  ["\u4A37", "\u4A37"], /* CJK Ideograph Extension A */
  ["\u4A38", "\u4A38"], /* CJK Ideograph Extension A */
  ["\u4A39", "\u4A39"], /* CJK Ideograph Extension A */
  ["\u4A3A", "\u4A3A"], /* CJK Ideograph Extension A */
  ["\u4A3B", "\u4A3B"], /* CJK Ideograph Extension A */
  ["\u4A3C", "\u4A3C"], /* CJK Ideograph Extension A */
  ["\u4A3D", "\u4A3D"], /* CJK Ideograph Extension A */
  ["\u4A3E", "\u4A3E"], /* CJK Ideograph Extension A */
  ["\u4A3F", "\u4A3F"], /* CJK Ideograph Extension A */
  ["\u4A40", "\u4A40"], /* CJK Ideograph Extension A */
  ["\u4A41", "\u4A41"], /* CJK Ideograph Extension A */
  ["\u4A42", "\u4A42"], /* CJK Ideograph Extension A */
  ["\u4A43", "\u4A43"], /* CJK Ideograph Extension A */
  ["\u4A44", "\u4A44"], /* CJK Ideograph Extension A */
  ["\u4A45", "\u4A45"], /* CJK Ideograph Extension A */
  ["\u4A46", "\u4A46"], /* CJK Ideograph Extension A */
  ["\u4A47", "\u4A47"], /* CJK Ideograph Extension A */
  ["\u4A48", "\u4A48"], /* CJK Ideograph Extension A */
  ["\u4A49", "\u4A49"], /* CJK Ideograph Extension A */
  ["\u4A4A", "\u4A4A"], /* CJK Ideograph Extension A */
  ["\u4A4B", "\u4A4B"], /* CJK Ideograph Extension A */
  ["\u4A4C", "\u4A4C"], /* CJK Ideograph Extension A */
  ["\u4A4D", "\u4A4D"], /* CJK Ideograph Extension A */
  ["\u4A4E", "\u4A4E"], /* CJK Ideograph Extension A */
  ["\u4A4F", "\u4A4F"], /* CJK Ideograph Extension A */
  ["\u4A50", "\u4A50"], /* CJK Ideograph Extension A */
  ["\u4A51", "\u4A51"], /* CJK Ideograph Extension A */
  ["\u4A52", "\u4A52"], /* CJK Ideograph Extension A */
  ["\u4A53", "\u4A53"], /* CJK Ideograph Extension A */
  ["\u4A54", "\u4A54"], /* CJK Ideograph Extension A */
  ["\u4A55", "\u4A55"], /* CJK Ideograph Extension A */
  ["\u4A56", "\u4A56"], /* CJK Ideograph Extension A */
  ["\u4A57", "\u4A57"], /* CJK Ideograph Extension A */
  ["\u4A58", "\u4A58"], /* CJK Ideograph Extension A */
  ["\u4A59", "\u4A59"], /* CJK Ideograph Extension A */
  ["\u4A5A", "\u4A5A"], /* CJK Ideograph Extension A */
  ["\u4A5B", "\u4A5B"], /* CJK Ideograph Extension A */
  ["\u4A5C", "\u4A5C"], /* CJK Ideograph Extension A */
  ["\u4A5D", "\u4A5D"], /* CJK Ideograph Extension A */
  ["\u4A5E", "\u4A5E"], /* CJK Ideograph Extension A */
  ["\u4A5F", "\u4A5F"], /* CJK Ideograph Extension A */
  ["\u4A60", "\u4A60"], /* CJK Ideograph Extension A */
  ["\u4A61", "\u4A61"], /* CJK Ideograph Extension A */
  ["\u4A62", "\u4A62"], /* CJK Ideograph Extension A */
  ["\u4A63", "\u4A63"], /* CJK Ideograph Extension A */
  ["\u4A64", "\u4A64"], /* CJK Ideograph Extension A */
  ["\u4A65", "\u4A65"], /* CJK Ideograph Extension A */
  ["\u4A66", "\u4A66"], /* CJK Ideograph Extension A */
  ["\u4A67", "\u4A67"], /* CJK Ideograph Extension A */
  ["\u4A68", "\u4A68"], /* CJK Ideograph Extension A */
  ["\u4A69", "\u4A69"], /* CJK Ideograph Extension A */
  ["\u4A6A", "\u4A6A"], /* CJK Ideograph Extension A */
  ["\u4A6B", "\u4A6B"], /* CJK Ideograph Extension A */
  ["\u4A6C", "\u4A6C"], /* CJK Ideograph Extension A */
  ["\u4A6D", "\u4A6D"], /* CJK Ideograph Extension A */
  ["\u4A6E", "\u4A6E"], /* CJK Ideograph Extension A */
  ["\u4A6F", "\u4A6F"], /* CJK Ideograph Extension A */
  ["\u4A70", "\u4A70"], /* CJK Ideograph Extension A */
  ["\u4A71", "\u4A71"], /* CJK Ideograph Extension A */
  ["\u4A72", "\u4A72"], /* CJK Ideograph Extension A */
  ["\u4A73", "\u4A73"], /* CJK Ideograph Extension A */
  ["\u4A74", "\u4A74"], /* CJK Ideograph Extension A */
  ["\u4A75", "\u4A75"], /* CJK Ideograph Extension A */
  ["\u4A76", "\u4A76"], /* CJK Ideograph Extension A */
  ["\u4A77", "\u4A77"], /* CJK Ideograph Extension A */
  ["\u4A78", "\u4A78"], /* CJK Ideograph Extension A */
  ["\u4A79", "\u4A79"], /* CJK Ideograph Extension A */
  ["\u4A7A", "\u4A7A"], /* CJK Ideograph Extension A */
  ["\u4A7B", "\u4A7B"], /* CJK Ideograph Extension A */
  ["\u4A7C", "\u4A7C"], /* CJK Ideograph Extension A */
  ["\u4A7D", "\u4A7D"], /* CJK Ideograph Extension A */
  ["\u4A7E", "\u4A7E"], /* CJK Ideograph Extension A */
  ["\u4A7F", "\u4A7F"], /* CJK Ideograph Extension A */
  ["\u4A80", "\u4A80"], /* CJK Ideograph Extension A */
  ["\u4A81", "\u4A81"], /* CJK Ideograph Extension A */
  ["\u4A82", "\u4A82"], /* CJK Ideograph Extension A */
  ["\u4A83", "\u4A83"], /* CJK Ideograph Extension A */
  ["\u4A84", "\u4A84"], /* CJK Ideograph Extension A */
  ["\u4A85", "\u4A85"], /* CJK Ideograph Extension A */
  ["\u4A86", "\u4A86"], /* CJK Ideograph Extension A */
  ["\u4A87", "\u4A87"], /* CJK Ideograph Extension A */
  ["\u4A88", "\u4A88"], /* CJK Ideograph Extension A */
  ["\u4A89", "\u4A89"], /* CJK Ideograph Extension A */
  ["\u4A8A", "\u4A8A"], /* CJK Ideograph Extension A */
  ["\u4A8B", "\u4A8B"], /* CJK Ideograph Extension A */
  ["\u4A8C", "\u4A8C"], /* CJK Ideograph Extension A */
  ["\u4A8D", "\u4A8D"], /* CJK Ideograph Extension A */
  ["\u4A8E", "\u4A8E"], /* CJK Ideograph Extension A */
  ["\u4A8F", "\u4A8F"], /* CJK Ideograph Extension A */
  ["\u4A90", "\u4A90"], /* CJK Ideograph Extension A */
  ["\u4A91", "\u4A91"], /* CJK Ideograph Extension A */
  ["\u4A92", "\u4A92"], /* CJK Ideograph Extension A */
  ["\u4A93", "\u4A93"], /* CJK Ideograph Extension A */
  ["\u4A94", "\u4A94"], /* CJK Ideograph Extension A */
  ["\u4A95", "\u4A95"], /* CJK Ideograph Extension A */
  ["\u4A96", "\u4A96"], /* CJK Ideograph Extension A */
  ["\u4A97", "\u4A97"], /* CJK Ideograph Extension A */
  ["\u4A98", "\u4A98"], /* CJK Ideograph Extension A */
  ["\u4A99", "\u4A99"], /* CJK Ideograph Extension A */
  ["\u4A9A", "\u4A9A"], /* CJK Ideograph Extension A */
  ["\u4A9B", "\u4A9B"], /* CJK Ideograph Extension A */
  ["\u4A9C", "\u4A9C"], /* CJK Ideograph Extension A */
  ["\u4A9D", "\u4A9D"], /* CJK Ideograph Extension A */
  ["\u4A9E", "\u4A9E"], /* CJK Ideograph Extension A */
  ["\u4A9F", "\u4A9F"], /* CJK Ideograph Extension A */
  ["\u4AA0", "\u4AA0"], /* CJK Ideograph Extension A */
  ["\u4AA1", "\u4AA1"], /* CJK Ideograph Extension A */
  ["\u4AA2", "\u4AA2"], /* CJK Ideograph Extension A */
  ["\u4AA3", "\u4AA3"], /* CJK Ideograph Extension A */
  ["\u4AA4", "\u4AA4"], /* CJK Ideograph Extension A */
  ["\u4AA5", "\u4AA5"], /* CJK Ideograph Extension A */
  ["\u4AA6", "\u4AA6"], /* CJK Ideograph Extension A */
  ["\u4AA7", "\u4AA7"], /* CJK Ideograph Extension A */
  ["\u4AA8", "\u4AA8"], /* CJK Ideograph Extension A */
  ["\u4AA9", "\u4AA9"], /* CJK Ideograph Extension A */
  ["\u4AAA", "\u4AAA"], /* CJK Ideograph Extension A */
  ["\u4AAB", "\u4AAB"], /* CJK Ideograph Extension A */
  ["\u4AAC", "\u4AAC"], /* CJK Ideograph Extension A */
  ["\u4AAD", "\u4AAD"], /* CJK Ideograph Extension A */
  ["\u4AAE", "\u4AAE"], /* CJK Ideograph Extension A */
  ["\u4AAF", "\u4AAF"], /* CJK Ideograph Extension A */
  ["\u4AB0", "\u4AB0"], /* CJK Ideograph Extension A */
  ["\u4AB1", "\u4AB1"], /* CJK Ideograph Extension A */
  ["\u4AB2", "\u4AB2"], /* CJK Ideograph Extension A */
  ["\u4AB3", "\u4AB3"], /* CJK Ideograph Extension A */
  ["\u4AB4", "\u4AB4"], /* CJK Ideograph Extension A */
  ["\u4AB5", "\u4AB5"], /* CJK Ideograph Extension A */
  ["\u4AB6", "\u4AB6"], /* CJK Ideograph Extension A */
  ["\u4AB7", "\u4AB7"], /* CJK Ideograph Extension A */
  ["\u4AB8", "\u4AB8"], /* CJK Ideograph Extension A */
  ["\u4AB9", "\u4AB9"], /* CJK Ideograph Extension A */
  ["\u4ABA", "\u4ABA"], /* CJK Ideograph Extension A */
  ["\u4ABB", "\u4ABB"], /* CJK Ideograph Extension A */
  ["\u4ABC", "\u4ABC"], /* CJK Ideograph Extension A */
  ["\u4ABD", "\u4ABD"], /* CJK Ideograph Extension A */
  ["\u4ABE", "\u4ABE"], /* CJK Ideograph Extension A */
  ["\u4ABF", "\u4ABF"], /* CJK Ideograph Extension A */
  ["\u4AC0", "\u4AC0"], /* CJK Ideograph Extension A */
  ["\u4AC1", "\u4AC1"], /* CJK Ideograph Extension A */
  ["\u4AC2", "\u4AC2"], /* CJK Ideograph Extension A */
  ["\u4AC3", "\u4AC3"], /* CJK Ideograph Extension A */
  ["\u4AC4", "\u4AC4"], /* CJK Ideograph Extension A */
  ["\u4AC5", "\u4AC5"], /* CJK Ideograph Extension A */
  ["\u4AC6", "\u4AC6"], /* CJK Ideograph Extension A */
  ["\u4AC7", "\u4AC7"], /* CJK Ideograph Extension A */
  ["\u4AC8", "\u4AC8"], /* CJK Ideograph Extension A */
  ["\u4AC9", "\u4AC9"], /* CJK Ideograph Extension A */
  ["\u4ACA", "\u4ACA"], /* CJK Ideograph Extension A */
  ["\u4ACB", "\u4ACB"], /* CJK Ideograph Extension A */
  ["\u4ACC", "\u4ACC"], /* CJK Ideograph Extension A */
  ["\u4ACD", "\u4ACD"], /* CJK Ideograph Extension A */
  ["\u4ACE", "\u4ACE"], /* CJK Ideograph Extension A */
  ["\u4ACF", "\u4ACF"], /* CJK Ideograph Extension A */
  ["\u4AD0", "\u4AD0"], /* CJK Ideograph Extension A */
  ["\u4AD1", "\u4AD1"], /* CJK Ideograph Extension A */
  ["\u4AD2", "\u4AD2"], /* CJK Ideograph Extension A */
  ["\u4AD3", "\u4AD3"], /* CJK Ideograph Extension A */
  ["\u4AD4", "\u4AD4"], /* CJK Ideograph Extension A */
  ["\u4AD5", "\u4AD5"], /* CJK Ideograph Extension A */
  ["\u4AD6", "\u4AD6"], /* CJK Ideograph Extension A */
  ["\u4AD7", "\u4AD7"], /* CJK Ideograph Extension A */
  ["\u4AD8", "\u4AD8"], /* CJK Ideograph Extension A */
  ["\u4AD9", "\u4AD9"], /* CJK Ideograph Extension A */
  ["\u4ADA", "\u4ADA"], /* CJK Ideograph Extension A */
  ["\u4ADB", "\u4ADB"], /* CJK Ideograph Extension A */
  ["\u4ADC", "\u4ADC"], /* CJK Ideograph Extension A */
  ["\u4ADD", "\u4ADD"], /* CJK Ideograph Extension A */
  ["\u4ADE", "\u4ADE"], /* CJK Ideograph Extension A */
  ["\u4ADF", "\u4ADF"], /* CJK Ideograph Extension A */
  ["\u4AE0", "\u4AE0"], /* CJK Ideograph Extension A */
  ["\u4AE1", "\u4AE1"], /* CJK Ideograph Extension A */
  ["\u4AE2", "\u4AE2"], /* CJK Ideograph Extension A */
  ["\u4AE3", "\u4AE3"], /* CJK Ideograph Extension A */
  ["\u4AE4", "\u4AE4"], /* CJK Ideograph Extension A */
  ["\u4AE5", "\u4AE5"], /* CJK Ideograph Extension A */
  ["\u4AE6", "\u4AE6"], /* CJK Ideograph Extension A */
  ["\u4AE7", "\u4AE7"], /* CJK Ideograph Extension A */
  ["\u4AE8", "\u4AE8"], /* CJK Ideograph Extension A */
  ["\u4AE9", "\u4AE9"], /* CJK Ideograph Extension A */
  ["\u4AEA", "\u4AEA"], /* CJK Ideograph Extension A */
  ["\u4AEB", "\u4AEB"], /* CJK Ideograph Extension A */
  ["\u4AEC", "\u4AEC"], /* CJK Ideograph Extension A */
  ["\u4AED", "\u4AED"], /* CJK Ideograph Extension A */
  ["\u4AEE", "\u4AEE"], /* CJK Ideograph Extension A */
  ["\u4AEF", "\u4AEF"], /* CJK Ideograph Extension A */
  ["\u4AF0", "\u4AF0"], /* CJK Ideograph Extension A */
  ["\u4AF1", "\u4AF1"], /* CJK Ideograph Extension A */
  ["\u4AF2", "\u4AF2"], /* CJK Ideograph Extension A */
  ["\u4AF3", "\u4AF3"], /* CJK Ideograph Extension A */
  ["\u4AF4", "\u4AF4"], /* CJK Ideograph Extension A */
  ["\u4AF5", "\u4AF5"], /* CJK Ideograph Extension A */
  ["\u4AF6", "\u4AF6"], /* CJK Ideograph Extension A */
  ["\u4AF7", "\u4AF7"], /* CJK Ideograph Extension A */
  ["\u4AF8", "\u4AF8"], /* CJK Ideograph Extension A */
  ["\u4AF9", "\u4AF9"], /* CJK Ideograph Extension A */
  ["\u4AFA", "\u4AFA"], /* CJK Ideograph Extension A */
  ["\u4AFB", "\u4AFB"], /* CJK Ideograph Extension A */
  ["\u4AFC", "\u4AFC"], /* CJK Ideograph Extension A */
  ["\u4AFD", "\u4AFD"], /* CJK Ideograph Extension A */
  ["\u4AFE", "\u4AFE"], /* CJK Ideograph Extension A */
  ["\u4AFF", "\u4AFF"], /* CJK Ideograph Extension A */
  ["\u4B00", "\u4B00"], /* CJK Ideograph Extension A */
  ["\u4B01", "\u4B01"], /* CJK Ideograph Extension A */
  ["\u4B02", "\u4B02"], /* CJK Ideograph Extension A */
  ["\u4B03", "\u4B03"], /* CJK Ideograph Extension A */
  ["\u4B04", "\u4B04"], /* CJK Ideograph Extension A */
  ["\u4B05", "\u4B05"], /* CJK Ideograph Extension A */
  ["\u4B06", "\u4B06"], /* CJK Ideograph Extension A */
  ["\u4B07", "\u4B07"], /* CJK Ideograph Extension A */
  ["\u4B08", "\u4B08"], /* CJK Ideograph Extension A */
  ["\u4B09", "\u4B09"], /* CJK Ideograph Extension A */
  ["\u4B0A", "\u4B0A"], /* CJK Ideograph Extension A */
  ["\u4B0B", "\u4B0B"], /* CJK Ideograph Extension A */
  ["\u4B0C", "\u4B0C"], /* CJK Ideograph Extension A */
  ["\u4B0D", "\u4B0D"], /* CJK Ideograph Extension A */
  ["\u4B0E", "\u4B0E"], /* CJK Ideograph Extension A */
  ["\u4B0F", "\u4B0F"], /* CJK Ideograph Extension A */
  ["\u4B10", "\u4B10"], /* CJK Ideograph Extension A */
  ["\u4B11", "\u4B11"], /* CJK Ideograph Extension A */
  ["\u4B12", "\u4B12"], /* CJK Ideograph Extension A */
  ["\u4B13", "\u4B13"], /* CJK Ideograph Extension A */
  ["\u4B14", "\u4B14"], /* CJK Ideograph Extension A */
  ["\u4B15", "\u4B15"], /* CJK Ideograph Extension A */
  ["\u4B16", "\u4B16"], /* CJK Ideograph Extension A */
  ["\u4B17", "\u4B17"], /* CJK Ideograph Extension A */
  ["\u4B18", "\u4B18"], /* CJK Ideograph Extension A */
  ["\u4B19", "\u4B19"], /* CJK Ideograph Extension A */
  ["\u4B1A", "\u4B1A"], /* CJK Ideograph Extension A */
  ["\u4B1B", "\u4B1B"], /* CJK Ideograph Extension A */
  ["\u4B1C", "\u4B1C"], /* CJK Ideograph Extension A */
  ["\u4B1D", "\u4B1D"], /* CJK Ideograph Extension A */
  ["\u4B1E", "\u4B1E"], /* CJK Ideograph Extension A */
  ["\u4B1F", "\u4B1F"], /* CJK Ideograph Extension A */
  ["\u4B20", "\u4B20"], /* CJK Ideograph Extension A */
  ["\u4B21", "\u4B21"], /* CJK Ideograph Extension A */
  ["\u4B22", "\u4B22"], /* CJK Ideograph Extension A */
  ["\u4B23", "\u4B23"], /* CJK Ideograph Extension A */
  ["\u4B24", "\u4B24"], /* CJK Ideograph Extension A */
  ["\u4B25", "\u4B25"], /* CJK Ideograph Extension A */
  ["\u4B26", "\u4B26"], /* CJK Ideograph Extension A */
  ["\u4B27", "\u4B27"], /* CJK Ideograph Extension A */
  ["\u4B28", "\u4B28"], /* CJK Ideograph Extension A */
  ["\u4B29", "\u4B29"], /* CJK Ideograph Extension A */
  ["\u4B2A", "\u4B2A"], /* CJK Ideograph Extension A */
  ["\u4B2B", "\u4B2B"], /* CJK Ideograph Extension A */
  ["\u4B2C", "\u4B2C"], /* CJK Ideograph Extension A */
  ["\u4B2D", "\u4B2D"], /* CJK Ideograph Extension A */
  ["\u4B2E", "\u4B2E"], /* CJK Ideograph Extension A */
  ["\u4B2F", "\u4B2F"], /* CJK Ideograph Extension A */
  ["\u4B30", "\u4B30"], /* CJK Ideograph Extension A */
  ["\u4B31", "\u4B31"], /* CJK Ideograph Extension A */
  ["\u4B32", "\u4B32"], /* CJK Ideograph Extension A */
  ["\u4B33", "\u4B33"], /* CJK Ideograph Extension A */
  ["\u4B34", "\u4B34"], /* CJK Ideograph Extension A */
  ["\u4B35", "\u4B35"], /* CJK Ideograph Extension A */
  ["\u4B36", "\u4B36"], /* CJK Ideograph Extension A */
  ["\u4B37", "\u4B37"], /* CJK Ideograph Extension A */
  ["\u4B38", "\u4B38"], /* CJK Ideograph Extension A */
  ["\u4B39", "\u4B39"], /* CJK Ideograph Extension A */
  ["\u4B3A", "\u4B3A"], /* CJK Ideograph Extension A */
  ["\u4B3B", "\u4B3B"], /* CJK Ideograph Extension A */
  ["\u4B3C", "\u4B3C"], /* CJK Ideograph Extension A */
  ["\u4B3D", "\u4B3D"], /* CJK Ideograph Extension A */
  ["\u4B3E", "\u4B3E"], /* CJK Ideograph Extension A */
  ["\u4B3F", "\u4B3F"], /* CJK Ideograph Extension A */
  ["\u4B40", "\u4B40"], /* CJK Ideograph Extension A */
  ["\u4B41", "\u4B41"], /* CJK Ideograph Extension A */
  ["\u4B42", "\u4B42"], /* CJK Ideograph Extension A */
  ["\u4B43", "\u4B43"], /* CJK Ideograph Extension A */
  ["\u4B44", "\u4B44"], /* CJK Ideograph Extension A */
  ["\u4B45", "\u4B45"], /* CJK Ideograph Extension A */
  ["\u4B46", "\u4B46"], /* CJK Ideograph Extension A */
  ["\u4B47", "\u4B47"], /* CJK Ideograph Extension A */
  ["\u4B48", "\u4B48"], /* CJK Ideograph Extension A */
  ["\u4B49", "\u4B49"], /* CJK Ideograph Extension A */
  ["\u4B4A", "\u4B4A"], /* CJK Ideograph Extension A */
  ["\u4B4B", "\u4B4B"], /* CJK Ideograph Extension A */
  ["\u4B4C", "\u4B4C"], /* CJK Ideograph Extension A */
  ["\u4B4D", "\u4B4D"], /* CJK Ideograph Extension A */
  ["\u4B4E", "\u4B4E"], /* CJK Ideograph Extension A */
  ["\u4B4F", "\u4B4F"], /* CJK Ideograph Extension A */
  ["\u4B50", "\u4B50"], /* CJK Ideograph Extension A */
  ["\u4B51", "\u4B51"], /* CJK Ideograph Extension A */
  ["\u4B52", "\u4B52"], /* CJK Ideograph Extension A */
  ["\u4B53", "\u4B53"], /* CJK Ideograph Extension A */
  ["\u4B54", "\u4B54"], /* CJK Ideograph Extension A */
  ["\u4B55", "\u4B55"], /* CJK Ideograph Extension A */
  ["\u4B56", "\u4B56"], /* CJK Ideograph Extension A */
  ["\u4B57", "\u4B57"], /* CJK Ideograph Extension A */
  ["\u4B58", "\u4B58"], /* CJK Ideograph Extension A */
  ["\u4B59", "\u4B59"], /* CJK Ideograph Extension A */
  ["\u4B5A", "\u4B5A"], /* CJK Ideograph Extension A */
  ["\u4B5B", "\u4B5B"], /* CJK Ideograph Extension A */
  ["\u4B5C", "\u4B5C"], /* CJK Ideograph Extension A */
  ["\u4B5D", "\u4B5D"], /* CJK Ideograph Extension A */
  ["\u4B5E", "\u4B5E"], /* CJK Ideograph Extension A */
  ["\u4B5F", "\u4B5F"], /* CJK Ideograph Extension A */
  ["\u4B60", "\u4B60"], /* CJK Ideograph Extension A */
  ["\u4B61", "\u4B61"], /* CJK Ideograph Extension A */
  ["\u4B62", "\u4B62"], /* CJK Ideograph Extension A */
  ["\u4B63", "\u4B63"], /* CJK Ideograph Extension A */
  ["\u4B64", "\u4B64"], /* CJK Ideograph Extension A */
  ["\u4B65", "\u4B65"], /* CJK Ideograph Extension A */
  ["\u4B66", "\u4B66"], /* CJK Ideograph Extension A */
  ["\u4B67", "\u4B67"], /* CJK Ideograph Extension A */
  ["\u4B68", "\u4B68"], /* CJK Ideograph Extension A */
  ["\u4B69", "\u4B69"], /* CJK Ideograph Extension A */
  ["\u4B6A", "\u4B6A"], /* CJK Ideograph Extension A */
  ["\u4B6B", "\u4B6B"], /* CJK Ideograph Extension A */
  ["\u4B6C", "\u4B6C"], /* CJK Ideograph Extension A */
  ["\u4B6D", "\u4B6D"], /* CJK Ideograph Extension A */
  ["\u4B6E", "\u4B6E"], /* CJK Ideograph Extension A */
  ["\u4B6F", "\u4B6F"], /* CJK Ideograph Extension A */
  ["\u4B70", "\u4B70"], /* CJK Ideograph Extension A */
  ["\u4B71", "\u4B71"], /* CJK Ideograph Extension A */
  ["\u4B72", "\u4B72"], /* CJK Ideograph Extension A */
  ["\u4B73", "\u4B73"], /* CJK Ideograph Extension A */
  ["\u4B74", "\u4B74"], /* CJK Ideograph Extension A */
  ["\u4B75", "\u4B75"], /* CJK Ideograph Extension A */
  ["\u4B76", "\u4B76"], /* CJK Ideograph Extension A */
  ["\u4B77", "\u4B77"], /* CJK Ideograph Extension A */
  ["\u4B78", "\u4B78"], /* CJK Ideograph Extension A */
  ["\u4B79", "\u4B79"], /* CJK Ideograph Extension A */
  ["\u4B7A", "\u4B7A"], /* CJK Ideograph Extension A */
  ["\u4B7B", "\u4B7B"], /* CJK Ideograph Extension A */
  ["\u4B7C", "\u4B7C"], /* CJK Ideograph Extension A */
  ["\u4B7D", "\u4B7D"], /* CJK Ideograph Extension A */
  ["\u4B7E", "\u4B7E"], /* CJK Ideograph Extension A */
  ["\u4B7F", "\u4B7F"], /* CJK Ideograph Extension A */
  ["\u4B80", "\u4B80"], /* CJK Ideograph Extension A */
  ["\u4B81", "\u4B81"], /* CJK Ideograph Extension A */
  ["\u4B82", "\u4B82"], /* CJK Ideograph Extension A */
  ["\u4B83", "\u4B83"], /* CJK Ideograph Extension A */
  ["\u4B84", "\u4B84"], /* CJK Ideograph Extension A */
  ["\u4B85", "\u4B85"], /* CJK Ideograph Extension A */
  ["\u4B86", "\u4B86"], /* CJK Ideograph Extension A */
  ["\u4B87", "\u4B87"], /* CJK Ideograph Extension A */
  ["\u4B88", "\u4B88"], /* CJK Ideograph Extension A */
  ["\u4B89", "\u4B89"], /* CJK Ideograph Extension A */
  ["\u4B8A", "\u4B8A"], /* CJK Ideograph Extension A */
  ["\u4B8B", "\u4B8B"], /* CJK Ideograph Extension A */
  ["\u4B8C", "\u4B8C"], /* CJK Ideograph Extension A */
  ["\u4B8D", "\u4B8D"], /* CJK Ideograph Extension A */
  ["\u4B8E", "\u4B8E"], /* CJK Ideograph Extension A */
  ["\u4B8F", "\u4B8F"], /* CJK Ideograph Extension A */
  ["\u4B90", "\u4B90"], /* CJK Ideograph Extension A */
  ["\u4B91", "\u4B91"], /* CJK Ideograph Extension A */
  ["\u4B92", "\u4B92"], /* CJK Ideograph Extension A */
  ["\u4B93", "\u4B93"], /* CJK Ideograph Extension A */
  ["\u4B94", "\u4B94"], /* CJK Ideograph Extension A */
  ["\u4B95", "\u4B95"], /* CJK Ideograph Extension A */
  ["\u4B96", "\u4B96"], /* CJK Ideograph Extension A */
  ["\u4B97", "\u4B97"], /* CJK Ideograph Extension A */
  ["\u4B98", "\u4B98"], /* CJK Ideograph Extension A */
  ["\u4B99", "\u4B99"], /* CJK Ideograph Extension A */
  ["\u4B9A", "\u4B9A"], /* CJK Ideograph Extension A */
  ["\u4B9B", "\u4B9B"], /* CJK Ideograph Extension A */
  ["\u4B9C", "\u4B9C"], /* CJK Ideograph Extension A */
  ["\u4B9D", "\u4B9D"], /* CJK Ideograph Extension A */
  ["\u4B9E", "\u4B9E"], /* CJK Ideograph Extension A */
  ["\u4B9F", "\u4B9F"], /* CJK Ideograph Extension A */
  ["\u4BA0", "\u4BA0"], /* CJK Ideograph Extension A */
  ["\u4BA1", "\u4BA1"], /* CJK Ideograph Extension A */
  ["\u4BA2", "\u4BA2"], /* CJK Ideograph Extension A */
  ["\u4BA3", "\u4BA3"], /* CJK Ideograph Extension A */
  ["\u4BA4", "\u4BA4"], /* CJK Ideograph Extension A */
  ["\u4BA5", "\u4BA5"], /* CJK Ideograph Extension A */
  ["\u4BA6", "\u4BA6"], /* CJK Ideograph Extension A */
  ["\u4BA7", "\u4BA7"], /* CJK Ideograph Extension A */
  ["\u4BA8", "\u4BA8"], /* CJK Ideograph Extension A */
  ["\u4BA9", "\u4BA9"], /* CJK Ideograph Extension A */
  ["\u4BAA", "\u4BAA"], /* CJK Ideograph Extension A */
  ["\u4BAB", "\u4BAB"], /* CJK Ideograph Extension A */
  ["\u4BAC", "\u4BAC"], /* CJK Ideograph Extension A */
  ["\u4BAD", "\u4BAD"], /* CJK Ideograph Extension A */
  ["\u4BAE", "\u4BAE"], /* CJK Ideograph Extension A */
  ["\u4BAF", "\u4BAF"], /* CJK Ideograph Extension A */
  ["\u4BB0", "\u4BB0"], /* CJK Ideograph Extension A */
  ["\u4BB1", "\u4BB1"], /* CJK Ideograph Extension A */
  ["\u4BB2", "\u4BB2"], /* CJK Ideograph Extension A */
  ["\u4BB3", "\u4BB3"], /* CJK Ideograph Extension A */
  ["\u4BB4", "\u4BB4"], /* CJK Ideograph Extension A */
  ["\u4BB5", "\u4BB5"], /* CJK Ideograph Extension A */
  ["\u4BB6", "\u4BB6"], /* CJK Ideograph Extension A */
  ["\u4BB7", "\u4BB7"], /* CJK Ideograph Extension A */
  ["\u4BB8", "\u4BB8"], /* CJK Ideograph Extension A */
  ["\u4BB9", "\u4BB9"], /* CJK Ideograph Extension A */
  ["\u4BBA", "\u4BBA"], /* CJK Ideograph Extension A */
  ["\u4BBB", "\u4BBB"], /* CJK Ideograph Extension A */
  ["\u4BBC", "\u4BBC"], /* CJK Ideograph Extension A */
  ["\u4BBD", "\u4BBD"], /* CJK Ideograph Extension A */
  ["\u4BBE", "\u4BBE"], /* CJK Ideograph Extension A */
  ["\u4BBF", "\u4BBF"], /* CJK Ideograph Extension A */
  ["\u4BC0", "\u4BC0"], /* CJK Ideograph Extension A */
  ["\u4BC1", "\u4BC1"], /* CJK Ideograph Extension A */
  ["\u4BC2", "\u4BC2"], /* CJK Ideograph Extension A */
  ["\u4BC3", "\u4BC3"], /* CJK Ideograph Extension A */
  ["\u4BC4", "\u4BC4"], /* CJK Ideograph Extension A */
  ["\u4BC5", "\u4BC5"], /* CJK Ideograph Extension A */
  ["\u4BC6", "\u4BC6"], /* CJK Ideograph Extension A */
  ["\u4BC7", "\u4BC7"], /* CJK Ideograph Extension A */
  ["\u4BC8", "\u4BC8"], /* CJK Ideograph Extension A */
  ["\u4BC9", "\u4BC9"], /* CJK Ideograph Extension A */
  ["\u4BCA", "\u4BCA"], /* CJK Ideograph Extension A */
  ["\u4BCB", "\u4BCB"], /* CJK Ideograph Extension A */
  ["\u4BCC", "\u4BCC"], /* CJK Ideograph Extension A */
  ["\u4BCD", "\u4BCD"], /* CJK Ideograph Extension A */
  ["\u4BCE", "\u4BCE"], /* CJK Ideograph Extension A */
  ["\u4BCF", "\u4BCF"], /* CJK Ideograph Extension A */
  ["\u4BD0", "\u4BD0"], /* CJK Ideograph Extension A */
  ["\u4BD1", "\u4BD1"], /* CJK Ideograph Extension A */
  ["\u4BD2", "\u4BD2"], /* CJK Ideograph Extension A */
  ["\u4BD3", "\u4BD3"], /* CJK Ideograph Extension A */
  ["\u4BD4", "\u4BD4"], /* CJK Ideograph Extension A */
  ["\u4BD5", "\u4BD5"], /* CJK Ideograph Extension A */
  ["\u4BD6", "\u4BD6"], /* CJK Ideograph Extension A */
  ["\u4BD7", "\u4BD7"], /* CJK Ideograph Extension A */
  ["\u4BD8", "\u4BD8"], /* CJK Ideograph Extension A */
  ["\u4BD9", "\u4BD9"], /* CJK Ideograph Extension A */
  ["\u4BDA", "\u4BDA"], /* CJK Ideograph Extension A */
  ["\u4BDB", "\u4BDB"], /* CJK Ideograph Extension A */
  ["\u4BDC", "\u4BDC"], /* CJK Ideograph Extension A */
  ["\u4BDD", "\u4BDD"], /* CJK Ideograph Extension A */
  ["\u4BDE", "\u4BDE"], /* CJK Ideograph Extension A */
  ["\u4BDF", "\u4BDF"], /* CJK Ideograph Extension A */
  ["\u4BE0", "\u4BE0"], /* CJK Ideograph Extension A */
  ["\u4BE1", "\u4BE1"], /* CJK Ideograph Extension A */
  ["\u4BE2", "\u4BE2"], /* CJK Ideograph Extension A */
  ["\u4BE3", "\u4BE3"], /* CJK Ideograph Extension A */
  ["\u4BE4", "\u4BE4"], /* CJK Ideograph Extension A */
  ["\u4BE5", "\u4BE5"], /* CJK Ideograph Extension A */
  ["\u4BE6", "\u4BE6"], /* CJK Ideograph Extension A */
  ["\u4BE7", "\u4BE7"], /* CJK Ideograph Extension A */
  ["\u4BE8", "\u4BE8"], /* CJK Ideograph Extension A */
  ["\u4BE9", "\u4BE9"], /* CJK Ideograph Extension A */
  ["\u4BEA", "\u4BEA"], /* CJK Ideograph Extension A */
  ["\u4BEB", "\u4BEB"], /* CJK Ideograph Extension A */
  ["\u4BEC", "\u4BEC"], /* CJK Ideograph Extension A */
  ["\u4BED", "\u4BED"], /* CJK Ideograph Extension A */
  ["\u4BEE", "\u4BEE"], /* CJK Ideograph Extension A */
  ["\u4BEF", "\u4BEF"], /* CJK Ideograph Extension A */
  ["\u4BF0", "\u4BF0"], /* CJK Ideograph Extension A */
  ["\u4BF1", "\u4BF1"], /* CJK Ideograph Extension A */
  ["\u4BF2", "\u4BF2"], /* CJK Ideograph Extension A */
  ["\u4BF3", "\u4BF3"], /* CJK Ideograph Extension A */
  ["\u4BF4", "\u4BF4"], /* CJK Ideograph Extension A */
  ["\u4BF5", "\u4BF5"], /* CJK Ideograph Extension A */
  ["\u4BF6", "\u4BF6"], /* CJK Ideograph Extension A */
  ["\u4BF7", "\u4BF7"], /* CJK Ideograph Extension A */
  ["\u4BF8", "\u4BF8"], /* CJK Ideograph Extension A */
  ["\u4BF9", "\u4BF9"], /* CJK Ideograph Extension A */
  ["\u4BFA", "\u4BFA"], /* CJK Ideograph Extension A */
  ["\u4BFB", "\u4BFB"], /* CJK Ideograph Extension A */
  ["\u4BFC", "\u4BFC"], /* CJK Ideograph Extension A */
  ["\u4BFD", "\u4BFD"], /* CJK Ideograph Extension A */
  ["\u4BFE", "\u4BFE"], /* CJK Ideograph Extension A */
  ["\u4BFF", "\u4BFF"], /* CJK Ideograph Extension A */
  ["\u4C00", "\u4C00"], /* CJK Ideograph Extension A */
  ["\u4C01", "\u4C01"], /* CJK Ideograph Extension A */
  ["\u4C02", "\u4C02"], /* CJK Ideograph Extension A */
  ["\u4C03", "\u4C03"], /* CJK Ideograph Extension A */
  ["\u4C04", "\u4C04"], /* CJK Ideograph Extension A */
  ["\u4C05", "\u4C05"], /* CJK Ideograph Extension A */
  ["\u4C06", "\u4C06"], /* CJK Ideograph Extension A */
  ["\u4C07", "\u4C07"], /* CJK Ideograph Extension A */
  ["\u4C08", "\u4C08"], /* CJK Ideograph Extension A */
  ["\u4C09", "\u4C09"], /* CJK Ideograph Extension A */
  ["\u4C0A", "\u4C0A"], /* CJK Ideograph Extension A */
  ["\u4C0B", "\u4C0B"], /* CJK Ideograph Extension A */
  ["\u4C0C", "\u4C0C"], /* CJK Ideograph Extension A */
  ["\u4C0D", "\u4C0D"], /* CJK Ideograph Extension A */
  ["\u4C0E", "\u4C0E"], /* CJK Ideograph Extension A */
  ["\u4C0F", "\u4C0F"], /* CJK Ideograph Extension A */
  ["\u4C10", "\u4C10"], /* CJK Ideograph Extension A */
  ["\u4C11", "\u4C11"], /* CJK Ideograph Extension A */
  ["\u4C12", "\u4C12"], /* CJK Ideograph Extension A */
  ["\u4C13", "\u4C13"], /* CJK Ideograph Extension A */
  ["\u4C14", "\u4C14"], /* CJK Ideograph Extension A */
  ["\u4C15", "\u4C15"], /* CJK Ideograph Extension A */
  ["\u4C16", "\u4C16"], /* CJK Ideograph Extension A */
  ["\u4C17", "\u4C17"], /* CJK Ideograph Extension A */
  ["\u4C18", "\u4C18"], /* CJK Ideograph Extension A */
  ["\u4C19", "\u4C19"], /* CJK Ideograph Extension A */
  ["\u4C1A", "\u4C1A"], /* CJK Ideograph Extension A */
  ["\u4C1B", "\u4C1B"], /* CJK Ideograph Extension A */
  ["\u4C1C", "\u4C1C"], /* CJK Ideograph Extension A */
  ["\u4C1D", "\u4C1D"], /* CJK Ideograph Extension A */
  ["\u4C1E", "\u4C1E"], /* CJK Ideograph Extension A */
  ["\u4C1F", "\u4C1F"], /* CJK Ideograph Extension A */
  ["\u4C20", "\u4C20"], /* CJK Ideograph Extension A */
  ["\u4C21", "\u4C21"], /* CJK Ideograph Extension A */
  ["\u4C22", "\u4C22"], /* CJK Ideograph Extension A */
  ["\u4C23", "\u4C23"], /* CJK Ideograph Extension A */
  ["\u4C24", "\u4C24"], /* CJK Ideograph Extension A */
  ["\u4C25", "\u4C25"], /* CJK Ideograph Extension A */
  ["\u4C26", "\u4C26"], /* CJK Ideograph Extension A */
  ["\u4C27", "\u4C27"], /* CJK Ideograph Extension A */
  ["\u4C28", "\u4C28"], /* CJK Ideograph Extension A */
  ["\u4C29", "\u4C29"], /* CJK Ideograph Extension A */
  ["\u4C2A", "\u4C2A"], /* CJK Ideograph Extension A */
  ["\u4C2B", "\u4C2B"], /* CJK Ideograph Extension A */
  ["\u4C2C", "\u4C2C"], /* CJK Ideograph Extension A */
  ["\u4C2D", "\u4C2D"], /* CJK Ideograph Extension A */
  ["\u4C2E", "\u4C2E"], /* CJK Ideograph Extension A */
  ["\u4C2F", "\u4C2F"], /* CJK Ideograph Extension A */
  ["\u4C30", "\u4C30"], /* CJK Ideograph Extension A */
  ["\u4C31", "\u4C31"], /* CJK Ideograph Extension A */
  ["\u4C32", "\u4C32"], /* CJK Ideograph Extension A */
  ["\u4C33", "\u4C33"], /* CJK Ideograph Extension A */
  ["\u4C34", "\u4C34"], /* CJK Ideograph Extension A */
  ["\u4C35", "\u4C35"], /* CJK Ideograph Extension A */
  ["\u4C36", "\u4C36"], /* CJK Ideograph Extension A */
  ["\u4C37", "\u4C37"], /* CJK Ideograph Extension A */
  ["\u4C38", "\u4C38"], /* CJK Ideograph Extension A */
  ["\u4C39", "\u4C39"], /* CJK Ideograph Extension A */
  ["\u4C3A", "\u4C3A"], /* CJK Ideograph Extension A */
  ["\u4C3B", "\u4C3B"], /* CJK Ideograph Extension A */
  ["\u4C3C", "\u4C3C"], /* CJK Ideograph Extension A */
  ["\u4C3D", "\u4C3D"], /* CJK Ideograph Extension A */
  ["\u4C3E", "\u4C3E"], /* CJK Ideograph Extension A */
  ["\u4C3F", "\u4C3F"], /* CJK Ideograph Extension A */
  ["\u4C40", "\u4C40"], /* CJK Ideograph Extension A */
  ["\u4C41", "\u4C41"], /* CJK Ideograph Extension A */
  ["\u4C42", "\u4C42"], /* CJK Ideograph Extension A */
  ["\u4C43", "\u4C43"], /* CJK Ideograph Extension A */
  ["\u4C44", "\u4C44"], /* CJK Ideograph Extension A */
  ["\u4C45", "\u4C45"], /* CJK Ideograph Extension A */
  ["\u4C46", "\u4C46"], /* CJK Ideograph Extension A */
  ["\u4C47", "\u4C47"], /* CJK Ideograph Extension A */
  ["\u4C48", "\u4C48"], /* CJK Ideograph Extension A */
  ["\u4C49", "\u4C49"], /* CJK Ideograph Extension A */
  ["\u4C4A", "\u4C4A"], /* CJK Ideograph Extension A */
  ["\u4C4B", "\u4C4B"], /* CJK Ideograph Extension A */
  ["\u4C4C", "\u4C4C"], /* CJK Ideograph Extension A */
  ["\u4C4D", "\u4C4D"], /* CJK Ideograph Extension A */
  ["\u4C4E", "\u4C4E"], /* CJK Ideograph Extension A */
  ["\u4C4F", "\u4C4F"], /* CJK Ideograph Extension A */
  ["\u4C50", "\u4C50"], /* CJK Ideograph Extension A */
  ["\u4C51", "\u4C51"], /* CJK Ideograph Extension A */
  ["\u4C52", "\u4C52"], /* CJK Ideograph Extension A */
  ["\u4C53", "\u4C53"], /* CJK Ideograph Extension A */
  ["\u4C54", "\u4C54"], /* CJK Ideograph Extension A */
  ["\u4C55", "\u4C55"], /* CJK Ideograph Extension A */
  ["\u4C56", "\u4C56"], /* CJK Ideograph Extension A */
  ["\u4C57", "\u4C57"], /* CJK Ideograph Extension A */
  ["\u4C58", "\u4C58"], /* CJK Ideograph Extension A */
  ["\u4C59", "\u4C59"], /* CJK Ideograph Extension A */
  ["\u4C5A", "\u4C5A"], /* CJK Ideograph Extension A */
  ["\u4C5B", "\u4C5B"], /* CJK Ideograph Extension A */
  ["\u4C5C", "\u4C5C"], /* CJK Ideograph Extension A */
  ["\u4C5D", "\u4C5D"], /* CJK Ideograph Extension A */
  ["\u4C5E", "\u4C5E"], /* CJK Ideograph Extension A */
  ["\u4C5F", "\u4C5F"], /* CJK Ideograph Extension A */
  ["\u4C60", "\u4C60"], /* CJK Ideograph Extension A */
  ["\u4C61", "\u4C61"], /* CJK Ideograph Extension A */
  ["\u4C62", "\u4C62"], /* CJK Ideograph Extension A */
  ["\u4C63", "\u4C63"], /* CJK Ideograph Extension A */
  ["\u4C64", "\u4C64"], /* CJK Ideograph Extension A */
  ["\u4C65", "\u4C65"], /* CJK Ideograph Extension A */
  ["\u4C66", "\u4C66"], /* CJK Ideograph Extension A */
  ["\u4C67", "\u4C67"], /* CJK Ideograph Extension A */
  ["\u4C68", "\u4C68"], /* CJK Ideograph Extension A */
  ["\u4C69", "\u4C69"], /* CJK Ideograph Extension A */
  ["\u4C6A", "\u4C6A"], /* CJK Ideograph Extension A */
  ["\u4C6B", "\u4C6B"], /* CJK Ideograph Extension A */
  ["\u4C6C", "\u4C6C"], /* CJK Ideograph Extension A */
  ["\u4C6D", "\u4C6D"], /* CJK Ideograph Extension A */
  ["\u4C6E", "\u4C6E"], /* CJK Ideograph Extension A */
  ["\u4C6F", "\u4C6F"], /* CJK Ideograph Extension A */
  ["\u4C70", "\u4C70"], /* CJK Ideograph Extension A */
  ["\u4C71", "\u4C71"], /* CJK Ideograph Extension A */
  ["\u4C72", "\u4C72"], /* CJK Ideograph Extension A */
  ["\u4C73", "\u4C73"], /* CJK Ideograph Extension A */
  ["\u4C74", "\u4C74"], /* CJK Ideograph Extension A */
  ["\u4C75", "\u4C75"], /* CJK Ideograph Extension A */
  ["\u4C76", "\u4C76"], /* CJK Ideograph Extension A */
  ["\u4C77", "\u4C77"], /* CJK Ideograph Extension A */
  ["\u4C78", "\u4C78"], /* CJK Ideograph Extension A */
  ["\u4C79", "\u4C79"], /* CJK Ideograph Extension A */
  ["\u4C7A", "\u4C7A"], /* CJK Ideograph Extension A */
  ["\u4C7B", "\u4C7B"], /* CJK Ideograph Extension A */
  ["\u4C7C", "\u4C7C"], /* CJK Ideograph Extension A */
  ["\u4C7D", "\u4C7D"], /* CJK Ideograph Extension A */
  ["\u4C7E", "\u4C7E"], /* CJK Ideograph Extension A */
  ["\u4C7F", "\u4C7F"], /* CJK Ideograph Extension A */
  ["\u4C80", "\u4C80"], /* CJK Ideograph Extension A */
  ["\u4C81", "\u4C81"], /* CJK Ideograph Extension A */
  ["\u4C82", "\u4C82"], /* CJK Ideograph Extension A */
  ["\u4C83", "\u4C83"], /* CJK Ideograph Extension A */
  ["\u4C84", "\u4C84"], /* CJK Ideograph Extension A */
  ["\u4C85", "\u4C85"], /* CJK Ideograph Extension A */
  ["\u4C86", "\u4C86"], /* CJK Ideograph Extension A */
  ["\u4C87", "\u4C87"], /* CJK Ideograph Extension A */
  ["\u4C88", "\u4C88"], /* CJK Ideograph Extension A */
  ["\u4C89", "\u4C89"], /* CJK Ideograph Extension A */
  ["\u4C8A", "\u4C8A"], /* CJK Ideograph Extension A */
  ["\u4C8B", "\u4C8B"], /* CJK Ideograph Extension A */
  ["\u4C8C", "\u4C8C"], /* CJK Ideograph Extension A */
  ["\u4C8D", "\u4C8D"], /* CJK Ideograph Extension A */
  ["\u4C8E", "\u4C8E"], /* CJK Ideograph Extension A */
  ["\u4C8F", "\u4C8F"], /* CJK Ideograph Extension A */
  ["\u4C90", "\u4C90"], /* CJK Ideograph Extension A */
  ["\u4C91", "\u4C91"], /* CJK Ideograph Extension A */
  ["\u4C92", "\u4C92"], /* CJK Ideograph Extension A */
  ["\u4C93", "\u4C93"], /* CJK Ideograph Extension A */
  ["\u4C94", "\u4C94"], /* CJK Ideograph Extension A */
  ["\u4C95", "\u4C95"], /* CJK Ideograph Extension A */
  ["\u4C96", "\u4C96"], /* CJK Ideograph Extension A */
  ["\u4C97", "\u4C97"], /* CJK Ideograph Extension A */
  ["\u4C98", "\u4C98"], /* CJK Ideograph Extension A */
  ["\u4C99", "\u4C99"], /* CJK Ideograph Extension A */
  ["\u4C9A", "\u4C9A"], /* CJK Ideograph Extension A */
  ["\u4C9B", "\u4C9B"], /* CJK Ideograph Extension A */
  ["\u4C9C", "\u4C9C"], /* CJK Ideograph Extension A */
  ["\u4C9D", "\u4C9D"], /* CJK Ideograph Extension A */
  ["\u4C9E", "\u4C9E"], /* CJK Ideograph Extension A */
  ["\u4C9F", "\u4C9F"], /* CJK Ideograph Extension A */
  ["\u4CA0", "\u4CA0"], /* CJK Ideograph Extension A */
  ["\u4CA1", "\u4CA1"], /* CJK Ideograph Extension A */
  ["\u4CA2", "\u4CA2"], /* CJK Ideograph Extension A */
  ["\u4CA3", "\u4CA3"], /* CJK Ideograph Extension A */
  ["\u4CA4", "\u4CA4"], /* CJK Ideograph Extension A */
  ["\u4CA5", "\u4CA5"], /* CJK Ideograph Extension A */
  ["\u4CA6", "\u4CA6"], /* CJK Ideograph Extension A */
  ["\u4CA7", "\u4CA7"], /* CJK Ideograph Extension A */
  ["\u4CA8", "\u4CA8"], /* CJK Ideograph Extension A */
  ["\u4CA9", "\u4CA9"], /* CJK Ideograph Extension A */
  ["\u4CAA", "\u4CAA"], /* CJK Ideograph Extension A */
  ["\u4CAB", "\u4CAB"], /* CJK Ideograph Extension A */
  ["\u4CAC", "\u4CAC"], /* CJK Ideograph Extension A */
  ["\u4CAD", "\u4CAD"], /* CJK Ideograph Extension A */
  ["\u4CAE", "\u4CAE"], /* CJK Ideograph Extension A */
  ["\u4CAF", "\u4CAF"], /* CJK Ideograph Extension A */
  ["\u4CB0", "\u4CB0"], /* CJK Ideograph Extension A */
  ["\u4CB1", "\u4CB1"], /* CJK Ideograph Extension A */
  ["\u4CB2", "\u4CB2"], /* CJK Ideograph Extension A */
  ["\u4CB3", "\u4CB3"], /* CJK Ideograph Extension A */
  ["\u4CB4", "\u4CB4"], /* CJK Ideograph Extension A */
  ["\u4CB5", "\u4CB5"], /* CJK Ideograph Extension A */
  ["\u4CB6", "\u4CB6"], /* CJK Ideograph Extension A */
  ["\u4CB7", "\u4CB7"], /* CJK Ideograph Extension A */
  ["\u4CB8", "\u4CB8"], /* CJK Ideograph Extension A */
  ["\u4CB9", "\u4CB9"], /* CJK Ideograph Extension A */
  ["\u4CBA", "\u4CBA"], /* CJK Ideograph Extension A */
  ["\u4CBB", "\u4CBB"], /* CJK Ideograph Extension A */
  ["\u4CBC", "\u4CBC"], /* CJK Ideograph Extension A */
  ["\u4CBD", "\u4CBD"], /* CJK Ideograph Extension A */
  ["\u4CBE", "\u4CBE"], /* CJK Ideograph Extension A */
  ["\u4CBF", "\u4CBF"], /* CJK Ideograph Extension A */
  ["\u4CC0", "\u4CC0"], /* CJK Ideograph Extension A */
  ["\u4CC1", "\u4CC1"], /* CJK Ideograph Extension A */
  ["\u4CC2", "\u4CC2"], /* CJK Ideograph Extension A */
  ["\u4CC3", "\u4CC3"], /* CJK Ideograph Extension A */
  ["\u4CC4", "\u4CC4"], /* CJK Ideograph Extension A */
  ["\u4CC5", "\u4CC5"], /* CJK Ideograph Extension A */
  ["\u4CC6", "\u4CC6"], /* CJK Ideograph Extension A */
  ["\u4CC7", "\u4CC7"], /* CJK Ideograph Extension A */
  ["\u4CC8", "\u4CC8"], /* CJK Ideograph Extension A */
  ["\u4CC9", "\u4CC9"], /* CJK Ideograph Extension A */
  ["\u4CCA", "\u4CCA"], /* CJK Ideograph Extension A */
  ["\u4CCB", "\u4CCB"], /* CJK Ideograph Extension A */
  ["\u4CCC", "\u4CCC"], /* CJK Ideograph Extension A */
  ["\u4CCD", "\u4CCD"], /* CJK Ideograph Extension A */
  ["\u4CCE", "\u4CCE"], /* CJK Ideograph Extension A */
  ["\u4CCF", "\u4CCF"], /* CJK Ideograph Extension A */
  ["\u4CD0", "\u4CD0"], /* CJK Ideograph Extension A */
  ["\u4CD1", "\u4CD1"], /* CJK Ideograph Extension A */
  ["\u4CD2", "\u4CD2"], /* CJK Ideograph Extension A */
  ["\u4CD3", "\u4CD3"], /* CJK Ideograph Extension A */
  ["\u4CD4", "\u4CD4"], /* CJK Ideograph Extension A */
  ["\u4CD5", "\u4CD5"], /* CJK Ideograph Extension A */
  ["\u4CD6", "\u4CD6"], /* CJK Ideograph Extension A */
  ["\u4CD7", "\u4CD7"], /* CJK Ideograph Extension A */
  ["\u4CD8", "\u4CD8"], /* CJK Ideograph Extension A */
  ["\u4CD9", "\u4CD9"], /* CJK Ideograph Extension A */
  ["\u4CDA", "\u4CDA"], /* CJK Ideograph Extension A */
  ["\u4CDB", "\u4CDB"], /* CJK Ideograph Extension A */
  ["\u4CDC", "\u4CDC"], /* CJK Ideograph Extension A */
  ["\u4CDD", "\u4CDD"], /* CJK Ideograph Extension A */
  ["\u4CDE", "\u4CDE"], /* CJK Ideograph Extension A */
  ["\u4CDF", "\u4CDF"], /* CJK Ideograph Extension A */
  ["\u4CE0", "\u4CE0"], /* CJK Ideograph Extension A */
  ["\u4CE1", "\u4CE1"], /* CJK Ideograph Extension A */
  ["\u4CE2", "\u4CE2"], /* CJK Ideograph Extension A */
  ["\u4CE3", "\u4CE3"], /* CJK Ideograph Extension A */
  ["\u4CE4", "\u4CE4"], /* CJK Ideograph Extension A */
  ["\u4CE5", "\u4CE5"], /* CJK Ideograph Extension A */
  ["\u4CE6", "\u4CE6"], /* CJK Ideograph Extension A */
  ["\u4CE7", "\u4CE7"], /* CJK Ideograph Extension A */
  ["\u4CE8", "\u4CE8"], /* CJK Ideograph Extension A */
  ["\u4CE9", "\u4CE9"], /* CJK Ideograph Extension A */
  ["\u4CEA", "\u4CEA"], /* CJK Ideograph Extension A */
  ["\u4CEB", "\u4CEB"], /* CJK Ideograph Extension A */
  ["\u4CEC", "\u4CEC"], /* CJK Ideograph Extension A */
  ["\u4CED", "\u4CED"], /* CJK Ideograph Extension A */
  ["\u4CEE", "\u4CEE"], /* CJK Ideograph Extension A */
  ["\u4CEF", "\u4CEF"], /* CJK Ideograph Extension A */
  ["\u4CF0", "\u4CF0"], /* CJK Ideograph Extension A */
  ["\u4CF1", "\u4CF1"], /* CJK Ideograph Extension A */
  ["\u4CF2", "\u4CF2"], /* CJK Ideograph Extension A */
  ["\u4CF3", "\u4CF3"], /* CJK Ideograph Extension A */
  ["\u4CF4", "\u4CF4"], /* CJK Ideograph Extension A */
  ["\u4CF5", "\u4CF5"], /* CJK Ideograph Extension A */
  ["\u4CF6", "\u4CF6"], /* CJK Ideograph Extension A */
  ["\u4CF7", "\u4CF7"], /* CJK Ideograph Extension A */
  ["\u4CF8", "\u4CF8"], /* CJK Ideograph Extension A */
  ["\u4CF9", "\u4CF9"], /* CJK Ideograph Extension A */
  ["\u4CFA", "\u4CFA"], /* CJK Ideograph Extension A */
  ["\u4CFB", "\u4CFB"], /* CJK Ideograph Extension A */
  ["\u4CFC", "\u4CFC"], /* CJK Ideograph Extension A */
  ["\u4CFD", "\u4CFD"], /* CJK Ideograph Extension A */
  ["\u4CFE", "\u4CFE"], /* CJK Ideograph Extension A */
  ["\u4CFF", "\u4CFF"], /* CJK Ideograph Extension A */
  ["\u4D00", "\u4D00"], /* CJK Ideograph Extension A */
  ["\u4D01", "\u4D01"], /* CJK Ideograph Extension A */
  ["\u4D02", "\u4D02"], /* CJK Ideograph Extension A */
  ["\u4D03", "\u4D03"], /* CJK Ideograph Extension A */
  ["\u4D04", "\u4D04"], /* CJK Ideograph Extension A */
  ["\u4D05", "\u4D05"], /* CJK Ideograph Extension A */
  ["\u4D06", "\u4D06"], /* CJK Ideograph Extension A */
  ["\u4D07", "\u4D07"], /* CJK Ideograph Extension A */
  ["\u4D08", "\u4D08"], /* CJK Ideograph Extension A */
  ["\u4D09", "\u4D09"], /* CJK Ideograph Extension A */
  ["\u4D0A", "\u4D0A"], /* CJK Ideograph Extension A */
  ["\u4D0B", "\u4D0B"], /* CJK Ideograph Extension A */
  ["\u4D0C", "\u4D0C"], /* CJK Ideograph Extension A */
  ["\u4D0D", "\u4D0D"], /* CJK Ideograph Extension A */
  ["\u4D0E", "\u4D0E"], /* CJK Ideograph Extension A */
  ["\u4D0F", "\u4D0F"], /* CJK Ideograph Extension A */
  ["\u4D10", "\u4D10"], /* CJK Ideograph Extension A */
  ["\u4D11", "\u4D11"], /* CJK Ideograph Extension A */
  ["\u4D12", "\u4D12"], /* CJK Ideograph Extension A */
  ["\u4D13", "\u4D13"], /* CJK Ideograph Extension A */
  ["\u4D14", "\u4D14"], /* CJK Ideograph Extension A */
  ["\u4D15", "\u4D15"], /* CJK Ideograph Extension A */
  ["\u4D16", "\u4D16"], /* CJK Ideograph Extension A */
  ["\u4D17", "\u4D17"], /* CJK Ideograph Extension A */
  ["\u4D18", "\u4D18"], /* CJK Ideograph Extension A */
  ["\u4D19", "\u4D19"], /* CJK Ideograph Extension A */
  ["\u4D1A", "\u4D1A"], /* CJK Ideograph Extension A */
  ["\u4D1B", "\u4D1B"], /* CJK Ideograph Extension A */
  ["\u4D1C", "\u4D1C"], /* CJK Ideograph Extension A */
  ["\u4D1D", "\u4D1D"], /* CJK Ideograph Extension A */
  ["\u4D1E", "\u4D1E"], /* CJK Ideograph Extension A */
  ["\u4D1F", "\u4D1F"], /* CJK Ideograph Extension A */
  ["\u4D20", "\u4D20"], /* CJK Ideograph Extension A */
  ["\u4D21", "\u4D21"], /* CJK Ideograph Extension A */
  ["\u4D22", "\u4D22"], /* CJK Ideograph Extension A */
  ["\u4D23", "\u4D23"], /* CJK Ideograph Extension A */
  ["\u4D24", "\u4D24"], /* CJK Ideograph Extension A */
  ["\u4D25", "\u4D25"], /* CJK Ideograph Extension A */
  ["\u4D26", "\u4D26"], /* CJK Ideograph Extension A */
  ["\u4D27", "\u4D27"], /* CJK Ideograph Extension A */
  ["\u4D28", "\u4D28"], /* CJK Ideograph Extension A */
  ["\u4D29", "\u4D29"], /* CJK Ideograph Extension A */
  ["\u4D2A", "\u4D2A"], /* CJK Ideograph Extension A */
  ["\u4D2B", "\u4D2B"], /* CJK Ideograph Extension A */
  ["\u4D2C", "\u4D2C"], /* CJK Ideograph Extension A */
  ["\u4D2D", "\u4D2D"], /* CJK Ideograph Extension A */
  ["\u4D2E", "\u4D2E"], /* CJK Ideograph Extension A */
  ["\u4D2F", "\u4D2F"], /* CJK Ideograph Extension A */
  ["\u4D30", "\u4D30"], /* CJK Ideograph Extension A */
  ["\u4D31", "\u4D31"], /* CJK Ideograph Extension A */
  ["\u4D32", "\u4D32"], /* CJK Ideograph Extension A */
  ["\u4D33", "\u4D33"], /* CJK Ideograph Extension A */
  ["\u4D34", "\u4D34"], /* CJK Ideograph Extension A */
  ["\u4D35", "\u4D35"], /* CJK Ideograph Extension A */
  ["\u4D36", "\u4D36"], /* CJK Ideograph Extension A */
  ["\u4D37", "\u4D37"], /* CJK Ideograph Extension A */
  ["\u4D38", "\u4D38"], /* CJK Ideograph Extension A */
  ["\u4D39", "\u4D39"], /* CJK Ideograph Extension A */
  ["\u4D3A", "\u4D3A"], /* CJK Ideograph Extension A */
  ["\u4D3B", "\u4D3B"], /* CJK Ideograph Extension A */
  ["\u4D3C", "\u4D3C"], /* CJK Ideograph Extension A */
  ["\u4D3D", "\u4D3D"], /* CJK Ideograph Extension A */
  ["\u4D3E", "\u4D3E"], /* CJK Ideograph Extension A */
  ["\u4D3F", "\u4D3F"], /* CJK Ideograph Extension A */
  ["\u4D40", "\u4D40"], /* CJK Ideograph Extension A */
  ["\u4D41", "\u4D41"], /* CJK Ideograph Extension A */
  ["\u4D42", "\u4D42"], /* CJK Ideograph Extension A */
  ["\u4D43", "\u4D43"], /* CJK Ideograph Extension A */
  ["\u4D44", "\u4D44"], /* CJK Ideograph Extension A */
  ["\u4D45", "\u4D45"], /* CJK Ideograph Extension A */
  ["\u4D46", "\u4D46"], /* CJK Ideograph Extension A */
  ["\u4D47", "\u4D47"], /* CJK Ideograph Extension A */
  ["\u4D48", "\u4D48"], /* CJK Ideograph Extension A */
  ["\u4D49", "\u4D49"], /* CJK Ideograph Extension A */
  ["\u4D4A", "\u4D4A"], /* CJK Ideograph Extension A */
  ["\u4D4B", "\u4D4B"], /* CJK Ideograph Extension A */
  ["\u4D4C", "\u4D4C"], /* CJK Ideograph Extension A */
  ["\u4D4D", "\u4D4D"], /* CJK Ideograph Extension A */
  ["\u4D4E", "\u4D4E"], /* CJK Ideograph Extension A */
  ["\u4D4F", "\u4D4F"], /* CJK Ideograph Extension A */
  ["\u4D50", "\u4D50"], /* CJK Ideograph Extension A */
  ["\u4D51", "\u4D51"], /* CJK Ideograph Extension A */
  ["\u4D52", "\u4D52"], /* CJK Ideograph Extension A */
  ["\u4D53", "\u4D53"], /* CJK Ideograph Extension A */
  ["\u4D54", "\u4D54"], /* CJK Ideograph Extension A */
  ["\u4D55", "\u4D55"], /* CJK Ideograph Extension A */
  ["\u4D56", "\u4D56"], /* CJK Ideograph Extension A */
  ["\u4D57", "\u4D57"], /* CJK Ideograph Extension A */
  ["\u4D58", "\u4D58"], /* CJK Ideograph Extension A */
  ["\u4D59", "\u4D59"], /* CJK Ideograph Extension A */
  ["\u4D5A", "\u4D5A"], /* CJK Ideograph Extension A */
  ["\u4D5B", "\u4D5B"], /* CJK Ideograph Extension A */
  ["\u4D5C", "\u4D5C"], /* CJK Ideograph Extension A */
  ["\u4D5D", "\u4D5D"], /* CJK Ideograph Extension A */
  ["\u4D5E", "\u4D5E"], /* CJK Ideograph Extension A */
  ["\u4D5F", "\u4D5F"], /* CJK Ideograph Extension A */
  ["\u4D60", "\u4D60"], /* CJK Ideograph Extension A */
  ["\u4D61", "\u4D61"], /* CJK Ideograph Extension A */
  ["\u4D62", "\u4D62"], /* CJK Ideograph Extension A */
  ["\u4D63", "\u4D63"], /* CJK Ideograph Extension A */
  ["\u4D64", "\u4D64"], /* CJK Ideograph Extension A */
  ["\u4D65", "\u4D65"], /* CJK Ideograph Extension A */
  ["\u4D66", "\u4D66"], /* CJK Ideograph Extension A */
  ["\u4D67", "\u4D67"], /* CJK Ideograph Extension A */
  ["\u4D68", "\u4D68"], /* CJK Ideograph Extension A */
  ["\u4D69", "\u4D69"], /* CJK Ideograph Extension A */
  ["\u4D6A", "\u4D6A"], /* CJK Ideograph Extension A */
  ["\u4D6B", "\u4D6B"], /* CJK Ideograph Extension A */
  ["\u4D6C", "\u4D6C"], /* CJK Ideograph Extension A */
  ["\u4D6D", "\u4D6D"], /* CJK Ideograph Extension A */
  ["\u4D6E", "\u4D6E"], /* CJK Ideograph Extension A */
  ["\u4D6F", "\u4D6F"], /* CJK Ideograph Extension A */
  ["\u4D70", "\u4D70"], /* CJK Ideograph Extension A */
  ["\u4D71", "\u4D71"], /* CJK Ideograph Extension A */
  ["\u4D72", "\u4D72"], /* CJK Ideograph Extension A */
  ["\u4D73", "\u4D73"], /* CJK Ideograph Extension A */
  ["\u4D74", "\u4D74"], /* CJK Ideograph Extension A */
  ["\u4D75", "\u4D75"], /* CJK Ideograph Extension A */
  ["\u4D76", "\u4D76"], /* CJK Ideograph Extension A */
  ["\u4D77", "\u4D77"], /* CJK Ideograph Extension A */
  ["\u4D78", "\u4D78"], /* CJK Ideograph Extension A */
  ["\u4D79", "\u4D79"], /* CJK Ideograph Extension A */
  ["\u4D7A", "\u4D7A"], /* CJK Ideograph Extension A */
  ["\u4D7B", "\u4D7B"], /* CJK Ideograph Extension A */
  ["\u4D7C", "\u4D7C"], /* CJK Ideograph Extension A */
  ["\u4D7D", "\u4D7D"], /* CJK Ideograph Extension A */
  ["\u4D7E", "\u4D7E"], /* CJK Ideograph Extension A */
  ["\u4D7F", "\u4D7F"], /* CJK Ideograph Extension A */
  ["\u4D80", "\u4D80"], /* CJK Ideograph Extension A */
  ["\u4D81", "\u4D81"], /* CJK Ideograph Extension A */
  ["\u4D82", "\u4D82"], /* CJK Ideograph Extension A */
  ["\u4D83", "\u4D83"], /* CJK Ideograph Extension A */
  ["\u4D84", "\u4D84"], /* CJK Ideograph Extension A */
  ["\u4D85", "\u4D85"], /* CJK Ideograph Extension A */
  ["\u4D86", "\u4D86"], /* CJK Ideograph Extension A */
  ["\u4D87", "\u4D87"], /* CJK Ideograph Extension A */
  ["\u4D88", "\u4D88"], /* CJK Ideograph Extension A */
  ["\u4D89", "\u4D89"], /* CJK Ideograph Extension A */
  ["\u4D8A", "\u4D8A"], /* CJK Ideograph Extension A */
  ["\u4D8B", "\u4D8B"], /* CJK Ideograph Extension A */
  ["\u4D8C", "\u4D8C"], /* CJK Ideograph Extension A */
  ["\u4D8D", "\u4D8D"], /* CJK Ideograph Extension A */
  ["\u4D8E", "\u4D8E"], /* CJK Ideograph Extension A */
  ["\u4D8F", "\u4D8F"], /* CJK Ideograph Extension A */
  ["\u4D90", "\u4D90"], /* CJK Ideograph Extension A */
  ["\u4D91", "\u4D91"], /* CJK Ideograph Extension A */
  ["\u4D92", "\u4D92"], /* CJK Ideograph Extension A */
  ["\u4D93", "\u4D93"], /* CJK Ideograph Extension A */
  ["\u4D94", "\u4D94"], /* CJK Ideograph Extension A */
  ["\u4D95", "\u4D95"], /* CJK Ideograph Extension A */
  ["\u4D96", "\u4D96"], /* CJK Ideograph Extension A */
  ["\u4D97", "\u4D97"], /* CJK Ideograph Extension A */
  ["\u4D98", "\u4D98"], /* CJK Ideograph Extension A */
  ["\u4D99", "\u4D99"], /* CJK Ideograph Extension A */
  ["\u4D9A", "\u4D9A"], /* CJK Ideograph Extension A */
  ["\u4D9B", "\u4D9B"], /* CJK Ideograph Extension A */
  ["\u4D9C", "\u4D9C"], /* CJK Ideograph Extension A */
  ["\u4D9D", "\u4D9D"], /* CJK Ideograph Extension A */
  ["\u4D9E", "\u4D9E"], /* CJK Ideograph Extension A */
  ["\u4D9F", "\u4D9F"], /* CJK Ideograph Extension A */
  ["\u4DA0", "\u4DA0"], /* CJK Ideograph Extension A */
  ["\u4DA1", "\u4DA1"], /* CJK Ideograph Extension A */
  ["\u4DA2", "\u4DA2"], /* CJK Ideograph Extension A */
  ["\u4DA3", "\u4DA3"], /* CJK Ideograph Extension A */
  ["\u4DA4", "\u4DA4"], /* CJK Ideograph Extension A */
  ["\u4DA5", "\u4DA5"], /* CJK Ideograph Extension A */
  ["\u4DA6", "\u4DA6"], /* CJK Ideograph Extension A */
  ["\u4DA7", "\u4DA7"], /* CJK Ideograph Extension A */
  ["\u4DA8", "\u4DA8"], /* CJK Ideograph Extension A */
  ["\u4DA9", "\u4DA9"], /* CJK Ideograph Extension A */
  ["\u4DAA", "\u4DAA"], /* CJK Ideograph Extension A */
  ["\u4DAB", "\u4DAB"], /* CJK Ideograph Extension A */
  ["\u4DAC", "\u4DAC"], /* CJK Ideograph Extension A */
  ["\u4DAD", "\u4DAD"], /* CJK Ideograph Extension A */
  ["\u4DAE", "\u4DAE"], /* CJK Ideograph Extension A */
  ["\u4DAF", "\u4DAF"], /* CJK Ideograph Extension A */
  ["\u4DB0", "\u4DB0"], /* CJK Ideograph Extension A */
  ["\u4DB1", "\u4DB1"], /* CJK Ideograph Extension A */
  ["\u4DB2", "\u4DB2"], /* CJK Ideograph Extension A */
  ["\u4DB3", "\u4DB3"], /* CJK Ideograph Extension A */
  ["\u4DB4", "\u4DB4"], /* CJK Ideograph Extension A */
  ["\u4DB5", "\u4DB5"], /* CJK Ideograph Extension A */
  ["\u4DB6", "\u4DB6"],
  ["\u4DB7", "\u4DB7"],
  ["\u4DB8", "\u4DB8"],
  ["\u4DB9", "\u4DB9"],
  ["\u4DBA", "\u4DBA"],
  ["\u4DBB", "\u4DBB"],
  ["\u4DBC", "\u4DBC"],
  ["\u4DBD", "\u4DBD"],
  ["\u4DBE", "\u4DBE"],
  ["\u4DBF", "\u4DBF"],
  ["\u4DC0", "\u4DC0"], /* HEXAGRAM FOR THE CREATIVE HEAVEN */
  ["\u4DC1", "\u4DC1"], /* HEXAGRAM FOR THE RECEPTIVE EARTH */
  ["\u4DC2", "\u4DC2"], /* HEXAGRAM FOR DIFFICULTY AT THE BEGINNING */
  ["\u4DC3", "\u4DC3"], /* HEXAGRAM FOR YOUTHFUL FOLLY */
  ["\u4DC4", "\u4DC4"], /* HEXAGRAM FOR WAITING */
  ["\u4DC5", "\u4DC5"], /* HEXAGRAM FOR CONFLICT */
  ["\u4DC6", "\u4DC6"], /* HEXAGRAM FOR THE ARMY */
  ["\u4DC7", "\u4DC7"], /* HEXAGRAM FOR HOLDING TOGETHER */
  ["\u4DC8", "\u4DC8"], /* HEXAGRAM FOR SMALL TAMING */
  ["\u4DC9", "\u4DC9"], /* HEXAGRAM FOR TREADING */
  ["\u4DCA", "\u4DCA"], /* HEXAGRAM FOR PEACE */
  ["\u4DCB", "\u4DCB"], /* HEXAGRAM FOR STANDSTILL */
  ["\u4DCC", "\u4DCC"], /* HEXAGRAM FOR FELLOWSHIP */
  ["\u4DCD", "\u4DCD"], /* HEXAGRAM FOR GREAT POSSESSION */
  ["\u4DCE", "\u4DCE"], /* HEXAGRAM FOR MODESTY */
  ["\u4DCF", "\u4DCF"], /* HEXAGRAM FOR ENTHUSIASM */
  ["\u4DD0", "\u4DD0"], /* HEXAGRAM FOR FOLLOWING */
  ["\u4DD1", "\u4DD1"], /* HEXAGRAM FOR WORK ON THE DECAYED */
  ["\u4DD2", "\u4DD2"], /* HEXAGRAM FOR APPROACH */
  ["\u4DD3", "\u4DD3"], /* HEXAGRAM FOR CONTEMPLATION */
  ["\u4DD4", "\u4DD4"], /* HEXAGRAM FOR BITING THROUGH */
  ["\u4DD5", "\u4DD5"], /* HEXAGRAM FOR GRACE */
  ["\u4DD6", "\u4DD6"], /* HEXAGRAM FOR SPLITTING APART */
  ["\u4DD7", "\u4DD7"], /* HEXAGRAM FOR RETURN */
  ["\u4DD8", "\u4DD8"], /* HEXAGRAM FOR INNOCENCE */
  ["\u4DD9", "\u4DD9"], /* HEXAGRAM FOR GREAT TAMING */
  ["\u4DDA", "\u4DDA"], /* HEXAGRAM FOR MOUTH CORNERS */
  ["\u4DDB", "\u4DDB"], /* HEXAGRAM FOR GREAT PREPONDERANCE */
  ["\u4DDC", "\u4DDC"], /* HEXAGRAM FOR THE ABYSMAL WATER */
  ["\u4DDD", "\u4DDD"], /* HEXAGRAM FOR THE CLINGING FIRE */
  ["\u4DDE", "\u4DDE"], /* HEXAGRAM FOR INFLUENCE */
  ["\u4DDF", "\u4DDF"], /* HEXAGRAM FOR DURATION */
  ["\u4DE0", "\u4DE0"], /* HEXAGRAM FOR RETREAT */
  ["\u4DE1", "\u4DE1"], /* HEXAGRAM FOR GREAT POWER */
  ["\u4DE2", "\u4DE2"], /* HEXAGRAM FOR PROGRESS */
  ["\u4DE3", "\u4DE3"], /* HEXAGRAM FOR DARKENING OF THE LIGHT */
  ["\u4DE4", "\u4DE4"], /* HEXAGRAM FOR THE FAMILY */
  ["\u4DE5", "\u4DE5"], /* HEXAGRAM FOR OPPOSITION */
  ["\u4DE6", "\u4DE6"], /* HEXAGRAM FOR OBSTRUCTION */
  ["\u4DE7", "\u4DE7"], /* HEXAGRAM FOR DELIVERANCE */
  ["\u4DE8", "\u4DE8"], /* HEXAGRAM FOR DECREASE */
  ["\u4DE9", "\u4DE9"], /* HEXAGRAM FOR INCREASE */
  ["\u4DEA", "\u4DEA"], /* HEXAGRAM FOR BREAKTHROUGH */
  ["\u4DEB", "\u4DEB"], /* HEXAGRAM FOR COMING TO MEET */
  ["\u4DEC", "\u4DEC"], /* HEXAGRAM FOR GATHERING TOGETHER */
  ["\u4DED", "\u4DED"], /* HEXAGRAM FOR PUSHING UPWARD */
  ["\u4DEE", "\u4DEE"], /* HEXAGRAM FOR OPPRESSION */
  ["\u4DEF", "\u4DEF"], /* HEXAGRAM FOR THE WELL */
  ["\u4DF0", "\u4DF0"], /* HEXAGRAM FOR REVOLUTION */
  ["\u4DF1", "\u4DF1"], /* HEXAGRAM FOR THE CAULDRON */
  ["\u4DF2", "\u4DF2"], /* HEXAGRAM FOR THE AROUSING THUNDER */
  ["\u4DF3", "\u4DF3"], /* HEXAGRAM FOR THE KEEPING STILL MOUNTAIN */
  ["\u4DF4", "\u4DF4"], /* HEXAGRAM FOR DEVELOPMENT */
  ["\u4DF5", "\u4DF5"], /* HEXAGRAM FOR THE MARRYING MAIDEN */
  ["\u4DF6", "\u4DF6"], /* HEXAGRAM FOR ABUNDANCE */
  ["\u4DF7", "\u4DF7"], /* HEXAGRAM FOR THE WANDERER */
  ["\u4DF8", "\u4DF8"], /* HEXAGRAM FOR THE GENTLE WIND */
  ["\u4DF9", "\u4DF9"], /* HEXAGRAM FOR THE JOYOUS LAKE */
  ["\u4DFA", "\u4DFA"], /* HEXAGRAM FOR DISPERSION */
  ["\u4DFB", "\u4DFB"], /* HEXAGRAM FOR LIMITATION */
  ["\u4DFC", "\u4DFC"], /* HEXAGRAM FOR INNER TRUTH */
  ["\u4DFD", "\u4DFD"], /* HEXAGRAM FOR SMALL PREPONDERANCE */
  ["\u4DFE", "\u4DFE"], /* HEXAGRAM FOR AFTER COMPLETION */
  ["\u4DFF", "\u4DFF"], /* HEXAGRAM FOR BEFORE COMPLETION */
  ["\u4E00", "\u4E00"], /* CJK Ideograph */
  ["\u4E01", "\u4E01"], /* CJK Ideograph */
  ["\u4E02", "\u4E02"], /* CJK Ideograph */
  ["\u4E03", "\u4E03"], /* CJK Ideograph */
  ["\u4E04", "\u4E04"], /* CJK Ideograph */
  ["\u4E05", "\u4E05"], /* CJK Ideograph */
  ["\u4E06", "\u4E06"], /* CJK Ideograph */
  ["\u4E07", "\u4E07"], /* CJK Ideograph */
  ["\u4E08", "\u4E08"], /* CJK Ideograph */
  ["\u4E09", "\u4E09"], /* CJK Ideograph */
  ["\u4E0A", "\u4E0A"], /* CJK Ideograph */
  ["\u4E0B", "\u4E0B"], /* CJK Ideograph */
  ["\u4E0C", "\u4E0C"], /* CJK Ideograph */
  ["\u4E0D", "\u4E0D"], /* CJK Ideograph */
  ["\u4E0E", "\u4E0E"], /* CJK Ideograph */
  ["\u4E0F", "\u4E0F"], /* CJK Ideograph */
  ["\u4E10", "\u4E10"], /* CJK Ideograph */
  ["\u4E11", "\u4E11"], /* CJK Ideograph */
  ["\u4E12", "\u4E12"], /* CJK Ideograph */
  ["\u4E13", "\u4E13"], /* CJK Ideograph */
  ["\u4E14", "\u4E14"], /* CJK Ideograph */
  ["\u4E15", "\u4E15"], /* CJK Ideograph */
  ["\u4E16", "\u4E16"], /* CJK Ideograph */
  ["\u4E17", "\u4E17"], /* CJK Ideograph */
  ["\u4E18", "\u4E18"], /* CJK Ideograph */
  ["\u4E19", "\u4E19"], /* CJK Ideograph */
  ["\u4E1A", "\u4E1A"], /* CJK Ideograph */
  ["\u4E1B", "\u4E1B"], /* CJK Ideograph */
  ["\u4E1C", "\u4E1C"], /* CJK Ideograph */
  ["\u4E1D", "\u4E1D"], /* CJK Ideograph */
  ["\u4E1E", "\u4E1E"], /* CJK Ideograph */
  ["\u4E1F", "\u4E1F"], /* CJK Ideograph */
  ["\u4E20", "\u4E20"], /* CJK Ideograph */
  ["\u4E21", "\u4E21"], /* CJK Ideograph */
  ["\u4E22", "\u4E22"], /* CJK Ideograph */
  ["\u4E23", "\u4E23"], /* CJK Ideograph */
  ["\u4E24", "\u4E24"], /* CJK Ideograph */
  ["\u4E25", "\u4E25"], /* CJK Ideograph */
  ["\u4E26", "\u4E26"], /* CJK Ideograph */
  ["\u4E27", "\u4E27"], /* CJK Ideograph */
  ["\u4E28", "\u4E28"], /* CJK Ideograph */
  ["\u4E29", "\u4E29"], /* CJK Ideograph */
  ["\u4E2A", "\u4E2A"], /* CJK Ideograph */
  ["\u4E2B", "\u4E2B"], /* CJK Ideograph */
  ["\u4E2C", "\u4E2C"], /* CJK Ideograph */
  ["\u4E2D", "\u4E2D"], /* CJK Ideograph */
  ["\u4E2E", "\u4E2E"], /* CJK Ideograph */
  ["\u4E2F", "\u4E2F"], /* CJK Ideograph */
  ["\u4E30", "\u4E30"], /* CJK Ideograph */
  ["\u4E31", "\u4E31"], /* CJK Ideograph */
  ["\u4E32", "\u4E32"], /* CJK Ideograph */
  ["\u4E33", "\u4E33"], /* CJK Ideograph */
  ["\u4E34", "\u4E34"], /* CJK Ideograph */
  ["\u4E35", "\u4E35"], /* CJK Ideograph */
  ["\u4E36", "\u4E36"], /* CJK Ideograph */
  ["\u4E37", "\u4E37"], /* CJK Ideograph */
  ["\u4E38", "\u4E38"], /* CJK Ideograph */
  ["\u4E39", "\u4E39"], /* CJK Ideograph */
  ["\u4E3A", "\u4E3A"], /* CJK Ideograph */
  ["\u4E3B", "\u4E3B"], /* CJK Ideograph */
  ["\u4E3C", "\u4E3C"], /* CJK Ideograph */
  ["\u4E3D", "\u4E3D"], /* CJK Ideograph */
  ["\u4E3E", "\u4E3E"], /* CJK Ideograph */
  ["\u4E3F", "\u4E3F"], /* CJK Ideograph */
  ["\u4E40", "\u4E40"], /* CJK Ideograph */
  ["\u4E41", "\u4E41"], /* CJK Ideograph */
  ["\u4E42", "\u4E42"], /* CJK Ideograph */
  ["\u4E43", "\u4E43"], /* CJK Ideograph */
  ["\u4E44", "\u4E44"], /* CJK Ideograph */
  ["\u4E45", "\u4E45"], /* CJK Ideograph */
  ["\u4E46", "\u4E46"], /* CJK Ideograph */
  ["\u4E47", "\u4E47"], /* CJK Ideograph */
  ["\u4E48", "\u4E48"], /* CJK Ideograph */
  ["\u4E49", "\u4E49"], /* CJK Ideograph */
  ["\u4E4A", "\u4E4A"], /* CJK Ideograph */
  ["\u4E4B", "\u4E4B"], /* CJK Ideograph */
  ["\u4E4C", "\u4E4C"], /* CJK Ideograph */
  ["\u4E4D", "\u4E4D"], /* CJK Ideograph */
  ["\u4E4E", "\u4E4E"], /* CJK Ideograph */
  ["\u4E4F", "\u4E4F"], /* CJK Ideograph */
  ["\u4E50", "\u4E50"], /* CJK Ideograph */
  ["\u4E51", "\u4E51"], /* CJK Ideograph */
  ["\u4E52", "\u4E52"], /* CJK Ideograph */
  ["\u4E53", "\u4E53"], /* CJK Ideograph */
  ["\u4E54", "\u4E54"], /* CJK Ideograph */
  ["\u4E55", "\u4E55"], /* CJK Ideograph */
  ["\u4E56", "\u4E56"], /* CJK Ideograph */
  ["\u4E57", "\u4E57"], /* CJK Ideograph */
  ["\u4E58", "\u4E58"], /* CJK Ideograph */
  ["\u4E59", "\u4E59"], /* CJK Ideograph */
  ["\u4E5A", "\u4E5A"], /* CJK Ideograph */
  ["\u4E5B", "\u4E5B"], /* CJK Ideograph */
  ["\u4E5C", "\u4E5C"], /* CJK Ideograph */
  ["\u4E5D", "\u4E5D"], /* CJK Ideograph */
  ["\u4E5E", "\u4E5E"], /* CJK Ideograph */
  ["\u4E5F", "\u4E5F"], /* CJK Ideograph */
  ["\u4E60", "\u4E60"], /* CJK Ideograph */
  ["\u4E61", "\u4E61"], /* CJK Ideograph */
  ["\u4E62", "\u4E62"], /* CJK Ideograph */
  ["\u4E63", "\u4E63"], /* CJK Ideograph */
  ["\u4E64", "\u4E64"], /* CJK Ideograph */
  ["\u4E65", "\u4E65"], /* CJK Ideograph */
  ["\u4E66", "\u4E66"], /* CJK Ideograph */
  ["\u4E67", "\u4E67"], /* CJK Ideograph */
  ["\u4E68", "\u4E68"], /* CJK Ideograph */
  ["\u4E69", "\u4E69"], /* CJK Ideograph */
  ["\u4E6A", "\u4E6A"], /* CJK Ideograph */
  ["\u4E6B", "\u4E6B"], /* CJK Ideograph */
  ["\u4E6C", "\u4E6C"], /* CJK Ideograph */
  ["\u4E6D", "\u4E6D"], /* CJK Ideograph */
  ["\u4E6E", "\u4E6E"], /* CJK Ideograph */
  ["\u4E6F", "\u4E6F"], /* CJK Ideograph */
  ["\u4E70", "\u4E70"], /* CJK Ideograph */
  ["\u4E71", "\u4E71"], /* CJK Ideograph */
  ["\u4E72", "\u4E72"], /* CJK Ideograph */
  ["\u4E73", "\u4E73"], /* CJK Ideograph */
  ["\u4E74", "\u4E74"], /* CJK Ideograph */
  ["\u4E75", "\u4E75"], /* CJK Ideograph */
  ["\u4E76", "\u4E76"], /* CJK Ideograph */
  ["\u4E77", "\u4E77"], /* CJK Ideograph */
  ["\u4E78", "\u4E78"], /* CJK Ideograph */
  ["\u4E79", "\u4E79"], /* CJK Ideograph */
  ["\u4E7A", "\u4E7A"], /* CJK Ideograph */
  ["\u4E7B", "\u4E7B"], /* CJK Ideograph */
  ["\u4E7C", "\u4E7C"], /* CJK Ideograph */
  ["\u4E7D", "\u4E7D"], /* CJK Ideograph */
  ["\u4E7E", "\u4E7E"], /* CJK Ideograph */
  ["\u4E7F", "\u4E7F"], /* CJK Ideograph */
  ["\u4E80", "\u4E80"], /* CJK Ideograph */
  ["\u4E81", "\u4E81"], /* CJK Ideograph */
  ["\u4E82", "\u4E82"], /* CJK Ideograph */
  ["\u4E83", "\u4E83"], /* CJK Ideograph */
  ["\u4E84", "\u4E84"], /* CJK Ideograph */
  ["\u4E85", "\u4E85"], /* CJK Ideograph */
  ["\u4E86", "\u4E86"], /* CJK Ideograph */
  ["\u4E87", "\u4E87"], /* CJK Ideograph */
  ["\u4E88", "\u4E88"], /* CJK Ideograph */
  ["\u4E89", "\u4E89"], /* CJK Ideograph */
  ["\u4E8A", "\u4E8A"], /* CJK Ideograph */
  ["\u4E8B", "\u4E8B"], /* CJK Ideograph */
  ["\u4E8C", "\u4E8C"], /* CJK Ideograph */
  ["\u4E8D", "\u4E8D"], /* CJK Ideograph */
  ["\u4E8E", "\u4E8E"], /* CJK Ideograph */
  ["\u4E8F", "\u4E8F"], /* CJK Ideograph */
  ["\u4E90", "\u4E90"], /* CJK Ideograph */
  ["\u4E91", "\u4E91"], /* CJK Ideograph */
  ["\u4E92", "\u4E92"], /* CJK Ideograph */
  ["\u4E93", "\u4E93"], /* CJK Ideograph */
  ["\u4E94", "\u4E94"], /* CJK Ideograph */
  ["\u4E95", "\u4E95"], /* CJK Ideograph */
  ["\u4E96", "\u4E96"], /* CJK Ideograph */
  ["\u4E97", "\u4E97"], /* CJK Ideograph */
  ["\u4E98", "\u4E98"], /* CJK Ideograph */
  ["\u4E99", "\u4E99"], /* CJK Ideograph */
  ["\u4E9A", "\u4E9A"], /* CJK Ideograph */
  ["\u4E9B", "\u4E9B"], /* CJK Ideograph */
  ["\u4E9C", "\u4E9C"], /* CJK Ideograph */
  ["\u4E9D", "\u4E9D"], /* CJK Ideograph */
  ["\u4E9E", "\u4E9E"], /* CJK Ideograph */
  ["\u4E9F", "\u4E9F"], /* CJK Ideograph */
  ["\u4EA0", "\u4EA0"], /* CJK Ideograph */
  ["\u4EA1", "\u4EA1"], /* CJK Ideograph */
  ["\u4EA2", "\u4EA2"], /* CJK Ideograph */
  ["\u4EA3", "\u4EA3"], /* CJK Ideograph */
  ["\u4EA4", "\u4EA4"], /* CJK Ideograph */
  ["\u4EA5", "\u4EA5"], /* CJK Ideograph */
  ["\u4EA6", "\u4EA6"], /* CJK Ideograph */
  ["\u4EA7", "\u4EA7"], /* CJK Ideograph */
  ["\u4EA8", "\u4EA8"], /* CJK Ideograph */
  ["\u4EA9", "\u4EA9"], /* CJK Ideograph */
  ["\u4EAA", "\u4EAA"], /* CJK Ideograph */
  ["\u4EAB", "\u4EAB"], /* CJK Ideograph */
  ["\u4EAC", "\u4EAC"], /* CJK Ideograph */
  ["\u4EAD", "\u4EAD"], /* CJK Ideograph */
  ["\u4EAE", "\u4EAE"], /* CJK Ideograph */
  ["\u4EAF", "\u4EAF"], /* CJK Ideograph */
  ["\u4EB0", "\u4EB0"], /* CJK Ideograph */
  ["\u4EB1", "\u4EB1"], /* CJK Ideograph */
  ["\u4EB2", "\u4EB2"], /* CJK Ideograph */
  ["\u4EB3", "\u4EB3"], /* CJK Ideograph */
  ["\u4EB4", "\u4EB4"], /* CJK Ideograph */
  ["\u4EB5", "\u4EB5"], /* CJK Ideograph */
  ["\u4EB6", "\u4EB6"], /* CJK Ideograph */
  ["\u4EB7", "\u4EB7"], /* CJK Ideograph */
  ["\u4EB8", "\u4EB8"], /* CJK Ideograph */
  ["\u4EB9", "\u4EB9"], /* CJK Ideograph */
  ["\u4EBA", "\u4EBA"], /* CJK Ideograph */
  ["\u4EBB", "\u4EBB"], /* CJK Ideograph */
  ["\u4EBC", "\u4EBC"], /* CJK Ideograph */
  ["\u4EBD", "\u4EBD"], /* CJK Ideograph */
  ["\u4EBE", "\u4EBE"], /* CJK Ideograph */
  ["\u4EBF", "\u4EBF"], /* CJK Ideograph */
  ["\u4EC0", "\u4EC0"], /* CJK Ideograph */
  ["\u4EC1", "\u4EC1"], /* CJK Ideograph */
  ["\u4EC2", "\u4EC2"], /* CJK Ideograph */
  ["\u4EC3", "\u4EC3"], /* CJK Ideograph */
  ["\u4EC4", "\u4EC4"], /* CJK Ideograph */
  ["\u4EC5", "\u4EC5"], /* CJK Ideograph */
  ["\u4EC6", "\u4EC6"], /* CJK Ideograph */
  ["\u4EC7", "\u4EC7"], /* CJK Ideograph */
  ["\u4EC8", "\u4EC8"], /* CJK Ideograph */
  ["\u4EC9", "\u4EC9"], /* CJK Ideograph */
  ["\u4ECA", "\u4ECA"], /* CJK Ideograph */
  ["\u4ECB", "\u4ECB"], /* CJK Ideograph */
  ["\u4ECC", "\u4ECC"], /* CJK Ideograph */
  ["\u4ECD", "\u4ECD"], /* CJK Ideograph */
  ["\u4ECE", "\u4ECE"], /* CJK Ideograph */
  ["\u4ECF", "\u4ECF"], /* CJK Ideograph */
  ["\u4ED0", "\u4ED0"], /* CJK Ideograph */
  ["\u4ED1", "\u4ED1"], /* CJK Ideograph */
  ["\u4ED2", "\u4ED2"], /* CJK Ideograph */
  ["\u4ED3", "\u4ED3"], /* CJK Ideograph */
  ["\u4ED4", "\u4ED4"], /* CJK Ideograph */
  ["\u4ED5", "\u4ED5"], /* CJK Ideograph */
  ["\u4ED6", "\u4ED6"], /* CJK Ideograph */
  ["\u4ED7", "\u4ED7"], /* CJK Ideograph */
  ["\u4ED8", "\u4ED8"], /* CJK Ideograph */
  ["\u4ED9", "\u4ED9"], /* CJK Ideograph */
  ["\u4EDA", "\u4EDA"], /* CJK Ideograph */
  ["\u4EDB", "\u4EDB"], /* CJK Ideograph */
  ["\u4EDC", "\u4EDC"], /* CJK Ideograph */
  ["\u4EDD", "\u4EDD"], /* CJK Ideograph */
  ["\u4EDE", "\u4EDE"], /* CJK Ideograph */
  ["\u4EDF", "\u4EDF"], /* CJK Ideograph */
  ["\u4EE0", "\u4EE0"], /* CJK Ideograph */
  ["\u4EE1", "\u4EE1"], /* CJK Ideograph */
  ["\u4EE2", "\u4EE2"], /* CJK Ideograph */
  ["\u4EE3", "\u4EE3"], /* CJK Ideograph */
  ["\u4EE4", "\u4EE4"], /* CJK Ideograph */
  ["\u4EE5", "\u4EE5"], /* CJK Ideograph */
  ["\u4EE6", "\u4EE6"], /* CJK Ideograph */
  ["\u4EE7", "\u4EE7"], /* CJK Ideograph */
  ["\u4EE8", "\u4EE8"], /* CJK Ideograph */
  ["\u4EE9", "\u4EE9"], /* CJK Ideograph */
  ["\u4EEA", "\u4EEA"], /* CJK Ideograph */
  ["\u4EEB", "\u4EEB"], /* CJK Ideograph */
  ["\u4EEC", "\u4EEC"], /* CJK Ideograph */
  ["\u4EED", "\u4EED"], /* CJK Ideograph */
  ["\u4EEE", "\u4EEE"], /* CJK Ideograph */
  ["\u4EEF", "\u4EEF"], /* CJK Ideograph */
  ["\u4EF0", "\u4EF0"], /* CJK Ideograph */
  ["\u4EF1", "\u4EF1"], /* CJK Ideograph */
  ["\u4EF2", "\u4EF2"], /* CJK Ideograph */
  ["\u4EF3", "\u4EF3"], /* CJK Ideograph */
  ["\u4EF4", "\u4EF4"], /* CJK Ideograph */
  ["\u4EF5", "\u4EF5"], /* CJK Ideograph */
  ["\u4EF6", "\u4EF6"], /* CJK Ideograph */
  ["\u4EF7", "\u4EF7"], /* CJK Ideograph */
  ["\u4EF8", "\u4EF8"], /* CJK Ideograph */
  ["\u4EF9", "\u4EF9"], /* CJK Ideograph */
  ["\u4EFA", "\u4EFA"], /* CJK Ideograph */
  ["\u4EFB", "\u4EFB"], /* CJK Ideograph */
  ["\u4EFC", "\u4EFC"], /* CJK Ideograph */
  ["\u4EFD", "\u4EFD"], /* CJK Ideograph */
  ["\u4EFE", "\u4EFE"], /* CJK Ideograph */
  ["\u4EFF", "\u4EFF"], /* CJK Ideograph */
  ["\u4F00", "\u4F00"], /* CJK Ideograph */
  ["\u4F01", "\u4F01"], /* CJK Ideograph */
  ["\u4F02", "\u4F02"], /* CJK Ideograph */
  ["\u4F03", "\u4F03"], /* CJK Ideograph */
  ["\u4F04", "\u4F04"], /* CJK Ideograph */
  ["\u4F05", "\u4F05"], /* CJK Ideograph */
  ["\u4F06", "\u4F06"], /* CJK Ideograph */
  ["\u4F07", "\u4F07"], /* CJK Ideograph */
  ["\u4F08", "\u4F08"], /* CJK Ideograph */
  ["\u4F09", "\u4F09"], /* CJK Ideograph */
  ["\u4F0A", "\u4F0A"], /* CJK Ideograph */
  ["\u4F0B", "\u4F0B"], /* CJK Ideograph */
  ["\u4F0C", "\u4F0C"], /* CJK Ideograph */
  ["\u4F0D", "\u4F0D"], /* CJK Ideograph */
  ["\u4F0E", "\u4F0E"], /* CJK Ideograph */
  ["\u4F0F", "\u4F0F"], /* CJK Ideograph */
  ["\u4F10", "\u4F10"], /* CJK Ideograph */
  ["\u4F11", "\u4F11"], /* CJK Ideograph */
  ["\u4F12", "\u4F12"], /* CJK Ideograph */
  ["\u4F13", "\u4F13"], /* CJK Ideograph */
  ["\u4F14", "\u4F14"], /* CJK Ideograph */
  ["\u4F15", "\u4F15"], /* CJK Ideograph */
  ["\u4F16", "\u4F16"], /* CJK Ideograph */
  ["\u4F17", "\u4F17"], /* CJK Ideograph */
  ["\u4F18", "\u4F18"], /* CJK Ideograph */
  ["\u4F19", "\u4F19"], /* CJK Ideograph */
  ["\u4F1A", "\u4F1A"], /* CJK Ideograph */
  ["\u4F1B", "\u4F1B"], /* CJK Ideograph */
  ["\u4F1C", "\u4F1C"], /* CJK Ideograph */
  ["\u4F1D", "\u4F1D"], /* CJK Ideograph */
  ["\u4F1E", "\u4F1E"], /* CJK Ideograph */
  ["\u4F1F", "\u4F1F"], /* CJK Ideograph */
  ["\u4F20", "\u4F20"], /* CJK Ideograph */
  ["\u4F21", "\u4F21"], /* CJK Ideograph */
  ["\u4F22", "\u4F22"], /* CJK Ideograph */
  ["\u4F23", "\u4F23"], /* CJK Ideograph */
  ["\u4F24", "\u4F24"], /* CJK Ideograph */
  ["\u4F25", "\u4F25"], /* CJK Ideograph */
  ["\u4F26", "\u4F26"], /* CJK Ideograph */
  ["\u4F27", "\u4F27"], /* CJK Ideograph */
  ["\u4F28", "\u4F28"], /* CJK Ideograph */
  ["\u4F29", "\u4F29"], /* CJK Ideograph */
  ["\u4F2A", "\u4F2A"], /* CJK Ideograph */
  ["\u4F2B", "\u4F2B"], /* CJK Ideograph */
  ["\u4F2C", "\u4F2C"], /* CJK Ideograph */
  ["\u4F2D", "\u4F2D"], /* CJK Ideograph */
  ["\u4F2E", "\u4F2E"], /* CJK Ideograph */
  ["\u4F2F", "\u4F2F"], /* CJK Ideograph */
  ["\u4F30", "\u4F30"], /* CJK Ideograph */
  ["\u4F31", "\u4F31"], /* CJK Ideograph */
  ["\u4F32", "\u4F32"], /* CJK Ideograph */
  ["\u4F33", "\u4F33"], /* CJK Ideograph */
  ["\u4F34", "\u4F34"], /* CJK Ideograph */
  ["\u4F35", "\u4F35"], /* CJK Ideograph */
  ["\u4F36", "\u4F36"], /* CJK Ideograph */
  ["\u4F37", "\u4F37"], /* CJK Ideograph */
  ["\u4F38", "\u4F38"], /* CJK Ideograph */
  ["\u4F39", "\u4F39"], /* CJK Ideograph */
  ["\u4F3A", "\u4F3A"], /* CJK Ideograph */
  ["\u4F3B", "\u4F3B"], /* CJK Ideograph */
  ["\u4F3C", "\u4F3C"], /* CJK Ideograph */
  ["\u4F3D", "\u4F3D"], /* CJK Ideograph */
  ["\u4F3E", "\u4F3E"], /* CJK Ideograph */
  ["\u4F3F", "\u4F3F"], /* CJK Ideograph */
  ["\u4F40", "\u4F40"], /* CJK Ideograph */
  ["\u4F41", "\u4F41"], /* CJK Ideograph */
  ["\u4F42", "\u4F42"], /* CJK Ideograph */
  ["\u4F43", "\u4F43"], /* CJK Ideograph */
  ["\u4F44", "\u4F44"], /* CJK Ideograph */
  ["\u4F45", "\u4F45"], /* CJK Ideograph */
  ["\u4F46", "\u4F46"], /* CJK Ideograph */
  ["\u4F47", "\u4F47"], /* CJK Ideograph */
  ["\u4F48", "\u4F48"], /* CJK Ideograph */
  ["\u4F49", "\u4F49"], /* CJK Ideograph */
  ["\u4F4A", "\u4F4A"], /* CJK Ideograph */
  ["\u4F4B", "\u4F4B"], /* CJK Ideograph */
  ["\u4F4C", "\u4F4C"], /* CJK Ideograph */
  ["\u4F4D", "\u4F4D"], /* CJK Ideograph */
  ["\u4F4E", "\u4F4E"], /* CJK Ideograph */
  ["\u4F4F", "\u4F4F"], /* CJK Ideograph */
  ["\u4F50", "\u4F50"], /* CJK Ideograph */
  ["\u4F51", "\u4F51"], /* CJK Ideograph */
  ["\u4F52", "\u4F52"], /* CJK Ideograph */
  ["\u4F53", "\u4F53"], /* CJK Ideograph */
  ["\u4F54", "\u4F54"], /* CJK Ideograph */
  ["\u4F55", "\u4F55"], /* CJK Ideograph */
  ["\u4F56", "\u4F56"], /* CJK Ideograph */
  ["\u4F57", "\u4F57"], /* CJK Ideograph */
  ["\u4F58", "\u4F58"], /* CJK Ideograph */
  ["\u4F59", "\u4F59"], /* CJK Ideograph */
  ["\u4F5A", "\u4F5A"], /* CJK Ideograph */
  ["\u4F5B", "\u4F5B"], /* CJK Ideograph */
  ["\u4F5C", "\u4F5C"], /* CJK Ideograph */
  ["\u4F5D", "\u4F5D"], /* CJK Ideograph */
  ["\u4F5E", "\u4F5E"], /* CJK Ideograph */
  ["\u4F5F", "\u4F5F"], /* CJK Ideograph */
  ["\u4F60", "\u4F60"], /* CJK Ideograph */
  ["\u4F61", "\u4F61"], /* CJK Ideograph */
  ["\u4F62", "\u4F62"], /* CJK Ideograph */
  ["\u4F63", "\u4F63"], /* CJK Ideograph */
  ["\u4F64", "\u4F64"], /* CJK Ideograph */
  ["\u4F65", "\u4F65"], /* CJK Ideograph */
  ["\u4F66", "\u4F66"], /* CJK Ideograph */
  ["\u4F67", "\u4F67"], /* CJK Ideograph */
  ["\u4F68", "\u4F68"], /* CJK Ideograph */
  ["\u4F69", "\u4F69"], /* CJK Ideograph */
  ["\u4F6A", "\u4F6A"], /* CJK Ideograph */
  ["\u4F6B", "\u4F6B"], /* CJK Ideograph */
  ["\u4F6C", "\u4F6C"], /* CJK Ideograph */
  ["\u4F6D", "\u4F6D"], /* CJK Ideograph */
  ["\u4F6E", "\u4F6E"], /* CJK Ideograph */
  ["\u4F6F", "\u4F6F"], /* CJK Ideograph */
  ["\u4F70", "\u4F70"], /* CJK Ideograph */
  ["\u4F71", "\u4F71"], /* CJK Ideograph */
  ["\u4F72", "\u4F72"], /* CJK Ideograph */
  ["\u4F73", "\u4F73"], /* CJK Ideograph */
  ["\u4F74", "\u4F74"], /* CJK Ideograph */
  ["\u4F75", "\u4F75"], /* CJK Ideograph */
  ["\u4F76", "\u4F76"], /* CJK Ideograph */
  ["\u4F77", "\u4F77"], /* CJK Ideograph */
  ["\u4F78", "\u4F78"], /* CJK Ideograph */
  ["\u4F79", "\u4F79"], /* CJK Ideograph */
  ["\u4F7A", "\u4F7A"], /* CJK Ideograph */
  ["\u4F7B", "\u4F7B"], /* CJK Ideograph */
  ["\u4F7C", "\u4F7C"], /* CJK Ideograph */
  ["\u4F7D", "\u4F7D"], /* CJK Ideograph */
  ["\u4F7E", "\u4F7E"], /* CJK Ideograph */
  ["\u4F7F", "\u4F7F"], /* CJK Ideograph */
  ["\u4F80", "\u4F80"], /* CJK Ideograph */
  ["\u4F81", "\u4F81"], /* CJK Ideograph */
  ["\u4F82", "\u4F82"], /* CJK Ideograph */
  ["\u4F83", "\u4F83"], /* CJK Ideograph */
  ["\u4F84", "\u4F84"], /* CJK Ideograph */
  ["\u4F85", "\u4F85"], /* CJK Ideograph */
  ["\u4F86", "\u4F86"], /* CJK Ideograph */
  ["\u4F87", "\u4F87"], /* CJK Ideograph */
  ["\u4F88", "\u4F88"], /* CJK Ideograph */
  ["\u4F89", "\u4F89"], /* CJK Ideograph */
  ["\u4F8A", "\u4F8A"], /* CJK Ideograph */
  ["\u4F8B", "\u4F8B"], /* CJK Ideograph */
  ["\u4F8C", "\u4F8C"], /* CJK Ideograph */
  ["\u4F8D", "\u4F8D"], /* CJK Ideograph */
  ["\u4F8E", "\u4F8E"], /* CJK Ideograph */
  ["\u4F8F", "\u4F8F"], /* CJK Ideograph */
  ["\u4F90", "\u4F90"], /* CJK Ideograph */
  ["\u4F91", "\u4F91"], /* CJK Ideograph */
  ["\u4F92", "\u4F92"], /* CJK Ideograph */
  ["\u4F93", "\u4F93"], /* CJK Ideograph */
  ["\u4F94", "\u4F94"], /* CJK Ideograph */
  ["\u4F95", "\u4F95"], /* CJK Ideograph */
  ["\u4F96", "\u4F96"], /* CJK Ideograph */
  ["\u4F97", "\u4F97"], /* CJK Ideograph */
  ["\u4F98", "\u4F98"], /* CJK Ideograph */
  ["\u4F99", "\u4F99"], /* CJK Ideograph */
  ["\u4F9A", "\u4F9A"], /* CJK Ideograph */
  ["\u4F9B", "\u4F9B"], /* CJK Ideograph */
  ["\u4F9C", "\u4F9C"], /* CJK Ideograph */
  ["\u4F9D", "\u4F9D"], /* CJK Ideograph */
  ["\u4F9E", "\u4F9E"], /* CJK Ideograph */
  ["\u4F9F", "\u4F9F"], /* CJK Ideograph */
  ["\u4FA0", "\u4FA0"], /* CJK Ideograph */
  ["\u4FA1", "\u4FA1"], /* CJK Ideograph */
  ["\u4FA2", "\u4FA2"], /* CJK Ideograph */
  ["\u4FA3", "\u4FA3"], /* CJK Ideograph */
  ["\u4FA4", "\u4FA4"], /* CJK Ideograph */
  ["\u4FA5", "\u4FA5"], /* CJK Ideograph */
  ["\u4FA6", "\u4FA6"], /* CJK Ideograph */
  ["\u4FA7", "\u4FA7"], /* CJK Ideograph */
  ["\u4FA8", "\u4FA8"], /* CJK Ideograph */
  ["\u4FA9", "\u4FA9"], /* CJK Ideograph */
  ["\u4FAA", "\u4FAA"], /* CJK Ideograph */
  ["\u4FAB", "\u4FAB"], /* CJK Ideograph */
  ["\u4FAC", "\u4FAC"], /* CJK Ideograph */
  ["\u4FAD", "\u4FAD"], /* CJK Ideograph */
  ["\u4FAE", "\u4FAE"], /* CJK Ideograph */
  ["\u4FAF", "\u4FAF"], /* CJK Ideograph */
  ["\u4FB0", "\u4FB0"], /* CJK Ideograph */
  ["\u4FB1", "\u4FB1"], /* CJK Ideograph */
  ["\u4FB2", "\u4FB2"], /* CJK Ideograph */
  ["\u4FB3", "\u4FB3"], /* CJK Ideograph */
  ["\u4FB4", "\u4FB4"], /* CJK Ideograph */
  ["\u4FB5", "\u4FB5"], /* CJK Ideograph */
  ["\u4FB6", "\u4FB6"], /* CJK Ideograph */
  ["\u4FB7", "\u4FB7"], /* CJK Ideograph */
  ["\u4FB8", "\u4FB8"], /* CJK Ideograph */
  ["\u4FB9", "\u4FB9"], /* CJK Ideograph */
  ["\u4FBA", "\u4FBA"], /* CJK Ideograph */
  ["\u4FBB", "\u4FBB"], /* CJK Ideograph */
  ["\u4FBC", "\u4FBC"], /* CJK Ideograph */
  ["\u4FBD", "\u4FBD"], /* CJK Ideograph */
  ["\u4FBE", "\u4FBE"], /* CJK Ideograph */
  ["\u4FBF", "\u4FBF"], /* CJK Ideograph */
  ["\u4FC0", "\u4FC0"], /* CJK Ideograph */
  ["\u4FC1", "\u4FC1"], /* CJK Ideograph */
  ["\u4FC2", "\u4FC2"], /* CJK Ideograph */
  ["\u4FC3", "\u4FC3"], /* CJK Ideograph */
  ["\u4FC4", "\u4FC4"], /* CJK Ideograph */
  ["\u4FC5", "\u4FC5"], /* CJK Ideograph */
  ["\u4FC6", "\u4FC6"], /* CJK Ideograph */
  ["\u4FC7", "\u4FC7"], /* CJK Ideograph */
  ["\u4FC8", "\u4FC8"], /* CJK Ideograph */
  ["\u4FC9", "\u4FC9"], /* CJK Ideograph */
  ["\u4FCA", "\u4FCA"], /* CJK Ideograph */
  ["\u4FCB", "\u4FCB"], /* CJK Ideograph */
  ["\u4FCC", "\u4FCC"], /* CJK Ideograph */
  ["\u4FCD", "\u4FCD"], /* CJK Ideograph */
  ["\u4FCE", "\u4FCE"], /* CJK Ideograph */
  ["\u4FCF", "\u4FCF"], /* CJK Ideograph */
  ["\u4FD0", "\u4FD0"], /* CJK Ideograph */
  ["\u4FD1", "\u4FD1"], /* CJK Ideograph */
  ["\u4FD2", "\u4FD2"], /* CJK Ideograph */
  ["\u4FD3", "\u4FD3"], /* CJK Ideograph */
  ["\u4FD4", "\u4FD4"], /* CJK Ideograph */
  ["\u4FD5", "\u4FD5"], /* CJK Ideograph */
  ["\u4FD6", "\u4FD6"], /* CJK Ideograph */
  ["\u4FD7", "\u4FD7"], /* CJK Ideograph */
  ["\u4FD8", "\u4FD8"], /* CJK Ideograph */
  ["\u4FD9", "\u4FD9"], /* CJK Ideograph */
  ["\u4FDA", "\u4FDA"], /* CJK Ideograph */
  ["\u4FDB", "\u4FDB"], /* CJK Ideograph */
  ["\u4FDC", "\u4FDC"], /* CJK Ideograph */
  ["\u4FDD", "\u4FDD"], /* CJK Ideograph */
  ["\u4FDE", "\u4FDE"], /* CJK Ideograph */
  ["\u4FDF", "\u4FDF"], /* CJK Ideograph */
  ["\u4FE0", "\u4FE0"], /* CJK Ideograph */
  ["\u4FE1", "\u4FE1"], /* CJK Ideograph */
  ["\u4FE2", "\u4FE2"], /* CJK Ideograph */
  ["\u4FE3", "\u4FE3"], /* CJK Ideograph */
  ["\u4FE4", "\u4FE4"], /* CJK Ideograph */
  ["\u4FE5", "\u4FE5"], /* CJK Ideograph */
  ["\u4FE6", "\u4FE6"], /* CJK Ideograph */
  ["\u4FE7", "\u4FE7"], /* CJK Ideograph */
  ["\u4FE8", "\u4FE8"], /* CJK Ideograph */
  ["\u4FE9", "\u4FE9"], /* CJK Ideograph */
  ["\u4FEA", "\u4FEA"], /* CJK Ideograph */
  ["\u4FEB", "\u4FEB"], /* CJK Ideograph */
  ["\u4FEC", "\u4FEC"], /* CJK Ideograph */
  ["\u4FED", "\u4FED"], /* CJK Ideograph */
  ["\u4FEE", "\u4FEE"], /* CJK Ideograph */
  ["\u4FEF", "\u4FEF"], /* CJK Ideograph */
  ["\u4FF0", "\u4FF0"], /* CJK Ideograph */
  ["\u4FF1", "\u4FF1"], /* CJK Ideograph */
  ["\u4FF2", "\u4FF2"], /* CJK Ideograph */
  ["\u4FF3", "\u4FF3"], /* CJK Ideograph */
  ["\u4FF4", "\u4FF4"], /* CJK Ideograph */
  ["\u4FF5", "\u4FF5"], /* CJK Ideograph */
  ["\u4FF6", "\u4FF6"], /* CJK Ideograph */
  ["\u4FF7", "\u4FF7"], /* CJK Ideograph */
  ["\u4FF8", "\u4FF8"], /* CJK Ideograph */
  ["\u4FF9", "\u4FF9"], /* CJK Ideograph */
  ["\u4FFA", "\u4FFA"], /* CJK Ideograph */
  ["\u4FFB", "\u4FFB"], /* CJK Ideograph */
  ["\u4FFC", "\u4FFC"], /* CJK Ideograph */
  ["\u4FFD", "\u4FFD"], /* CJK Ideograph */
  ["\u4FFE", "\u4FFE"], /* CJK Ideograph */
  ["\u4FFF", "\u4FFF"], /* CJK Ideograph */
  ["\u5000", "\u5000"], /* CJK Ideograph */
  ["\u5001", "\u5001"], /* CJK Ideograph */
  ["\u5002", "\u5002"], /* CJK Ideograph */
  ["\u5003", "\u5003"], /* CJK Ideograph */
  ["\u5004", "\u5004"], /* CJK Ideograph */
  ["\u5005", "\u5005"], /* CJK Ideograph */
  ["\u5006", "\u5006"], /* CJK Ideograph */
  ["\u5007", "\u5007"], /* CJK Ideograph */
  ["\u5008", "\u5008"], /* CJK Ideograph */
  ["\u5009", "\u5009"], /* CJK Ideograph */
  ["\u500A", "\u500A"], /* CJK Ideograph */
  ["\u500B", "\u500B"], /* CJK Ideograph */
  ["\u500C", "\u500C"], /* CJK Ideograph */
  ["\u500D", "\u500D"], /* CJK Ideograph */
  ["\u500E", "\u500E"], /* CJK Ideograph */
  ["\u500F", "\u500F"], /* CJK Ideograph */
  ["\u5010", "\u5010"], /* CJK Ideograph */
  ["\u5011", "\u5011"], /* CJK Ideograph */
  ["\u5012", "\u5012"], /* CJK Ideograph */
  ["\u5013", "\u5013"], /* CJK Ideograph */
  ["\u5014", "\u5014"], /* CJK Ideograph */
  ["\u5015", "\u5015"], /* CJK Ideograph */
  ["\u5016", "\u5016"], /* CJK Ideograph */
  ["\u5017", "\u5017"], /* CJK Ideograph */
  ["\u5018", "\u5018"], /* CJK Ideograph */
  ["\u5019", "\u5019"], /* CJK Ideograph */
  ["\u501A", "\u501A"], /* CJK Ideograph */
  ["\u501B", "\u501B"], /* CJK Ideograph */
  ["\u501C", "\u501C"], /* CJK Ideograph */
  ["\u501D", "\u501D"], /* CJK Ideograph */
  ["\u501E", "\u501E"], /* CJK Ideograph */
  ["\u501F", "\u501F"], /* CJK Ideograph */
  ["\u5020", "\u5020"], /* CJK Ideograph */
  ["\u5021", "\u5021"], /* CJK Ideograph */
  ["\u5022", "\u5022"], /* CJK Ideograph */
  ["\u5023", "\u5023"], /* CJK Ideograph */
  ["\u5024", "\u5024"], /* CJK Ideograph */
  ["\u5025", "\u5025"], /* CJK Ideograph */
  ["\u5026", "\u5026"], /* CJK Ideograph */
  ["\u5027", "\u5027"], /* CJK Ideograph */
  ["\u5028", "\u5028"], /* CJK Ideograph */
  ["\u5029", "\u5029"], /* CJK Ideograph */
  ["\u502A", "\u502A"], /* CJK Ideograph */
  ["\u502B", "\u502B"], /* CJK Ideograph */
  ["\u502C", "\u502C"], /* CJK Ideograph */
  ["\u502D", "\u502D"], /* CJK Ideograph */
  ["\u502E", "\u502E"], /* CJK Ideograph */
  ["\u502F", "\u502F"], /* CJK Ideograph */
  ["\u5030", "\u5030"], /* CJK Ideograph */
  ["\u5031", "\u5031"], /* CJK Ideograph */
  ["\u5032", "\u5032"], /* CJK Ideograph */
  ["\u5033", "\u5033"], /* CJK Ideograph */
  ["\u5034", "\u5034"], /* CJK Ideograph */
  ["\u5035", "\u5035"], /* CJK Ideograph */
  ["\u5036", "\u5036"], /* CJK Ideograph */
  ["\u5037", "\u5037"], /* CJK Ideograph */
  ["\u5038", "\u5038"], /* CJK Ideograph */
  ["\u5039", "\u5039"], /* CJK Ideograph */
  ["\u503A", "\u503A"], /* CJK Ideograph */
  ["\u503B", "\u503B"], /* CJK Ideograph */
  ["\u503C", "\u503C"], /* CJK Ideograph */
  ["\u503D", "\u503D"], /* CJK Ideograph */
  ["\u503E", "\u503E"], /* CJK Ideograph */
  ["\u503F", "\u503F"], /* CJK Ideograph */
  ["\u5040", "\u5040"], /* CJK Ideograph */
  ["\u5041", "\u5041"], /* CJK Ideograph */
  ["\u5042", "\u5042"], /* CJK Ideograph */
  ["\u5043", "\u5043"], /* CJK Ideograph */
  ["\u5044", "\u5044"], /* CJK Ideograph */
  ["\u5045", "\u5045"], /* CJK Ideograph */
  ["\u5046", "\u5046"], /* CJK Ideograph */
  ["\u5047", "\u5047"], /* CJK Ideograph */
  ["\u5048", "\u5048"], /* CJK Ideograph */
  ["\u5049", "\u5049"], /* CJK Ideograph */
  ["\u504A", "\u504A"], /* CJK Ideograph */
  ["\u504B", "\u504B"], /* CJK Ideograph */
  ["\u504C", "\u504C"], /* CJK Ideograph */
  ["\u504D", "\u504D"], /* CJK Ideograph */
  ["\u504E", "\u504E"], /* CJK Ideograph */
  ["\u504F", "\u504F"], /* CJK Ideograph */
  ["\u5050", "\u5050"], /* CJK Ideograph */
  ["\u5051", "\u5051"], /* CJK Ideograph */
  ["\u5052", "\u5052"], /* CJK Ideograph */
  ["\u5053", "\u5053"], /* CJK Ideograph */
  ["\u5054", "\u5054"], /* CJK Ideograph */
  ["\u5055", "\u5055"], /* CJK Ideograph */
  ["\u5056", "\u5056"], /* CJK Ideograph */
  ["\u5057", "\u5057"], /* CJK Ideograph */
  ["\u5058", "\u5058"], /* CJK Ideograph */
  ["\u5059", "\u5059"], /* CJK Ideograph */
  ["\u505A", "\u505A"], /* CJK Ideograph */
  ["\u505B", "\u505B"], /* CJK Ideograph */
  ["\u505C", "\u505C"], /* CJK Ideograph */
  ["\u505D", "\u505D"], /* CJK Ideograph */
  ["\u505E", "\u505E"], /* CJK Ideograph */
  ["\u505F", "\u505F"], /* CJK Ideograph */
  ["\u5060", "\u5060"], /* CJK Ideograph */
  ["\u5061", "\u5061"], /* CJK Ideograph */
  ["\u5062", "\u5062"], /* CJK Ideograph */
  ["\u5063", "\u5063"], /* CJK Ideograph */
  ["\u5064", "\u5064"], /* CJK Ideograph */
  ["\u5065", "\u5065"], /* CJK Ideograph */
  ["\u5066", "\u5066"], /* CJK Ideograph */
  ["\u5067", "\u5067"], /* CJK Ideograph */
  ["\u5068", "\u5068"], /* CJK Ideograph */
  ["\u5069", "\u5069"], /* CJK Ideograph */
  ["\u506A", "\u506A"], /* CJK Ideograph */
  ["\u506B", "\u506B"], /* CJK Ideograph */
  ["\u506C", "\u506C"], /* CJK Ideograph */
  ["\u506D", "\u506D"], /* CJK Ideograph */
  ["\u506E", "\u506E"], /* CJK Ideograph */
  ["\u506F", "\u506F"], /* CJK Ideograph */
  ["\u5070", "\u5070"], /* CJK Ideograph */
  ["\u5071", "\u5071"], /* CJK Ideograph */
  ["\u5072", "\u5072"], /* CJK Ideograph */
  ["\u5073", "\u5073"], /* CJK Ideograph */
  ["\u5074", "\u5074"], /* CJK Ideograph */
  ["\u5075", "\u5075"], /* CJK Ideograph */
  ["\u5076", "\u5076"], /* CJK Ideograph */
  ["\u5077", "\u5077"], /* CJK Ideograph */
  ["\u5078", "\u5078"], /* CJK Ideograph */
  ["\u5079", "\u5079"], /* CJK Ideograph */
  ["\u507A", "\u507A"], /* CJK Ideograph */
  ["\u507B", "\u507B"], /* CJK Ideograph */
  ["\u507C", "\u507C"], /* CJK Ideograph */
  ["\u507D", "\u507D"], /* CJK Ideograph */
  ["\u507E", "\u507E"], /* CJK Ideograph */
  ["\u507F", "\u507F"], /* CJK Ideograph */
  ["\u5080", "\u5080"], /* CJK Ideograph */
  ["\u5081", "\u5081"], /* CJK Ideograph */
  ["\u5082", "\u5082"], /* CJK Ideograph */
  ["\u5083", "\u5083"], /* CJK Ideograph */
  ["\u5084", "\u5084"], /* CJK Ideograph */
  ["\u5085", "\u5085"], /* CJK Ideograph */
  ["\u5086", "\u5086"], /* CJK Ideograph */
  ["\u5087", "\u5087"], /* CJK Ideograph */
  ["\u5088", "\u5088"], /* CJK Ideograph */
  ["\u5089", "\u5089"], /* CJK Ideograph */
  ["\u508A", "\u508A"], /* CJK Ideograph */
  ["\u508B", "\u508B"], /* CJK Ideograph */
  ["\u508C", "\u508C"], /* CJK Ideograph */
  ["\u508D", "\u508D"], /* CJK Ideograph */
  ["\u508E", "\u508E"], /* CJK Ideograph */
  ["\u508F", "\u508F"], /* CJK Ideograph */
  ["\u5090", "\u5090"], /* CJK Ideograph */
  ["\u5091", "\u5091"], /* CJK Ideograph */
  ["\u5092", "\u5092"], /* CJK Ideograph */
  ["\u5093", "\u5093"], /* CJK Ideograph */
  ["\u5094", "\u5094"], /* CJK Ideograph */
  ["\u5095", "\u5095"], /* CJK Ideograph */
  ["\u5096", "\u5096"], /* CJK Ideograph */
  ["\u5097", "\u5097"], /* CJK Ideograph */
  ["\u5098", "\u5098"], /* CJK Ideograph */
  ["\u5099", "\u5099"], /* CJK Ideograph */
  ["\u509A", "\u509A"], /* CJK Ideograph */
  ["\u509B", "\u509B"], /* CJK Ideograph */
  ["\u509C", "\u509C"], /* CJK Ideograph */
  ["\u509D", "\u509D"], /* CJK Ideograph */
  ["\u509E", "\u509E"], /* CJK Ideograph */
  ["\u509F", "\u509F"], /* CJK Ideograph */
  ["\u50A0", "\u50A0"], /* CJK Ideograph */
  ["\u50A1", "\u50A1"], /* CJK Ideograph */
  ["\u50A2", "\u50A2"], /* CJK Ideograph */
  ["\u50A3", "\u50A3"], /* CJK Ideograph */
  ["\u50A4", "\u50A4"], /* CJK Ideograph */
  ["\u50A5", "\u50A5"], /* CJK Ideograph */
  ["\u50A6", "\u50A6"], /* CJK Ideograph */
  ["\u50A7", "\u50A7"], /* CJK Ideograph */
  ["\u50A8", "\u50A8"], /* CJK Ideograph */
  ["\u50A9", "\u50A9"], /* CJK Ideograph */
  ["\u50AA", "\u50AA"], /* CJK Ideograph */
  ["\u50AB", "\u50AB"], /* CJK Ideograph */
  ["\u50AC", "\u50AC"], /* CJK Ideograph */
  ["\u50AD", "\u50AD"], /* CJK Ideograph */
  ["\u50AE", "\u50AE"], /* CJK Ideograph */
  ["\u50AF", "\u50AF"], /* CJK Ideograph */
  ["\u50B0", "\u50B0"], /* CJK Ideograph */
  ["\u50B1", "\u50B1"], /* CJK Ideograph */
  ["\u50B2", "\u50B2"], /* CJK Ideograph */
  ["\u50B3", "\u50B3"], /* CJK Ideograph */
  ["\u50B4", "\u50B4"], /* CJK Ideograph */
  ["\u50B5", "\u50B5"], /* CJK Ideograph */
  ["\u50B6", "\u50B6"], /* CJK Ideograph */
  ["\u50B7", "\u50B7"], /* CJK Ideograph */
  ["\u50B8", "\u50B8"], /* CJK Ideograph */
  ["\u50B9", "\u50B9"], /* CJK Ideograph */
  ["\u50BA", "\u50BA"], /* CJK Ideograph */
  ["\u50BB", "\u50BB"], /* CJK Ideograph */
  ["\u50BC", "\u50BC"], /* CJK Ideograph */
  ["\u50BD", "\u50BD"], /* CJK Ideograph */
  ["\u50BE", "\u50BE"], /* CJK Ideograph */
  ["\u50BF", "\u50BF"], /* CJK Ideograph */
  ["\u50C0", "\u50C0"], /* CJK Ideograph */
  ["\u50C1", "\u50C1"], /* CJK Ideograph */
  ["\u50C2", "\u50C2"], /* CJK Ideograph */
  ["\u50C3", "\u50C3"], /* CJK Ideograph */
  ["\u50C4", "\u50C4"], /* CJK Ideograph */
  ["\u50C5", "\u50C5"], /* CJK Ideograph */
  ["\u50C6", "\u50C6"], /* CJK Ideograph */
  ["\u50C7", "\u50C7"], /* CJK Ideograph */
  ["\u50C8", "\u50C8"], /* CJK Ideograph */
  ["\u50C9", "\u50C9"], /* CJK Ideograph */
  ["\u50CA", "\u50CA"], /* CJK Ideograph */
  ["\u50CB", "\u50CB"], /* CJK Ideograph */
  ["\u50CC", "\u50CC"], /* CJK Ideograph */
  ["\u50CD", "\u50CD"], /* CJK Ideograph */
  ["\u50CE", "\u50CE"], /* CJK Ideograph */
  ["\u50CF", "\u50CF"], /* CJK Ideograph */
  ["\u50D0", "\u50D0"], /* CJK Ideograph */
  ["\u50D1", "\u50D1"], /* CJK Ideograph */
  ["\u50D2", "\u50D2"], /* CJK Ideograph */
  ["\u50D3", "\u50D3"], /* CJK Ideograph */
  ["\u50D4", "\u50D4"], /* CJK Ideograph */
  ["\u50D5", "\u50D5"], /* CJK Ideograph */
  ["\u50D6", "\u50D6"], /* CJK Ideograph */
  ["\u50D7", "\u50D7"], /* CJK Ideograph */
  ["\u50D8", "\u50D8"], /* CJK Ideograph */
  ["\u50D9", "\u50D9"], /* CJK Ideograph */
  ["\u50DA", "\u50DA"], /* CJK Ideograph */
  ["\u50DB", "\u50DB"], /* CJK Ideograph */
  ["\u50DC", "\u50DC"], /* CJK Ideograph */
  ["\u50DD", "\u50DD"], /* CJK Ideograph */
  ["\u50DE", "\u50DE"], /* CJK Ideograph */
  ["\u50DF", "\u50DF"], /* CJK Ideograph */
  ["\u50E0", "\u50E0"], /* CJK Ideograph */
  ["\u50E1", "\u50E1"], /* CJK Ideograph */
  ["\u50E2", "\u50E2"], /* CJK Ideograph */
  ["\u50E3", "\u50E3"], /* CJK Ideograph */
  ["\u50E4", "\u50E4"], /* CJK Ideograph */
  ["\u50E5", "\u50E5"], /* CJK Ideograph */
  ["\u50E6", "\u50E6"], /* CJK Ideograph */
  ["\u50E7", "\u50E7"], /* CJK Ideograph */
  ["\u50E8", "\u50E8"], /* CJK Ideograph */
  ["\u50E9", "\u50E9"], /* CJK Ideograph */
  ["\u50EA", "\u50EA"], /* CJK Ideograph */
  ["\u50EB", "\u50EB"], /* CJK Ideograph */
  ["\u50EC", "\u50EC"], /* CJK Ideograph */
  ["\u50ED", "\u50ED"], /* CJK Ideograph */
  ["\u50EE", "\u50EE"], /* CJK Ideograph */
  ["\u50EF", "\u50EF"], /* CJK Ideograph */
  ["\u50F0", "\u50F0"], /* CJK Ideograph */
  ["\u50F1", "\u50F1"], /* CJK Ideograph */
  ["\u50F2", "\u50F2"], /* CJK Ideograph */
  ["\u50F3", "\u50F3"], /* CJK Ideograph */
  ["\u50F4", "\u50F4"], /* CJK Ideograph */
  ["\u50F5", "\u50F5"], /* CJK Ideograph */
  ["\u50F6", "\u50F6"], /* CJK Ideograph */
  ["\u50F7", "\u50F7"], /* CJK Ideograph */
  ["\u50F8", "\u50F8"], /* CJK Ideograph */
  ["\u50F9", "\u50F9"], /* CJK Ideograph */
  ["\u50FA", "\u50FA"], /* CJK Ideograph */
  ["\u50FB", "\u50FB"], /* CJK Ideograph */
  ["\u50FC", "\u50FC"], /* CJK Ideograph */
  ["\u50FD", "\u50FD"], /* CJK Ideograph */
  ["\u50FE", "\u50FE"], /* CJK Ideograph */
  ["\u50FF", "\u50FF"], /* CJK Ideograph */
  ["\u5100", "\u5100"], /* CJK Ideograph */
  ["\u5101", "\u5101"], /* CJK Ideograph */
  ["\u5102", "\u5102"], /* CJK Ideograph */
  ["\u5103", "\u5103"], /* CJK Ideograph */
  ["\u5104", "\u5104"], /* CJK Ideograph */
  ["\u5105", "\u5105"], /* CJK Ideograph */
  ["\u5106", "\u5106"], /* CJK Ideograph */
  ["\u5107", "\u5107"], /* CJK Ideograph */
  ["\u5108", "\u5108"], /* CJK Ideograph */
  ["\u5109", "\u5109"], /* CJK Ideograph */
  ["\u510A", "\u510A"], /* CJK Ideograph */
  ["\u510B", "\u510B"], /* CJK Ideograph */
  ["\u510C", "\u510C"], /* CJK Ideograph */
  ["\u510D", "\u510D"], /* CJK Ideograph */
  ["\u510E", "\u510E"], /* CJK Ideograph */
  ["\u510F", "\u510F"], /* CJK Ideograph */
  ["\u5110", "\u5110"], /* CJK Ideograph */
  ["\u5111", "\u5111"], /* CJK Ideograph */
  ["\u5112", "\u5112"], /* CJK Ideograph */
  ["\u5113", "\u5113"], /* CJK Ideograph */
  ["\u5114", "\u5114"], /* CJK Ideograph */
  ["\u5115", "\u5115"], /* CJK Ideograph */
  ["\u5116", "\u5116"], /* CJK Ideograph */
  ["\u5117", "\u5117"], /* CJK Ideograph */
  ["\u5118", "\u5118"], /* CJK Ideograph */
  ["\u5119", "\u5119"], /* CJK Ideograph */
  ["\u511A", "\u511A"], /* CJK Ideograph */
  ["\u511B", "\u511B"], /* CJK Ideograph */
  ["\u511C", "\u511C"], /* CJK Ideograph */
  ["\u511D", "\u511D"], /* CJK Ideograph */
  ["\u511E", "\u511E"], /* CJK Ideograph */
  ["\u511F", "\u511F"], /* CJK Ideograph */
  ["\u5120", "\u5120"], /* CJK Ideograph */
  ["\u5121", "\u5121"], /* CJK Ideograph */
  ["\u5122", "\u5122"], /* CJK Ideograph */
  ["\u5123", "\u5123"], /* CJK Ideograph */
  ["\u5124", "\u5124"], /* CJK Ideograph */
  ["\u5125", "\u5125"], /* CJK Ideograph */
  ["\u5126", "\u5126"], /* CJK Ideograph */
  ["\u5127", "\u5127"], /* CJK Ideograph */
  ["\u5128", "\u5128"], /* CJK Ideograph */
  ["\u5129", "\u5129"], /* CJK Ideograph */
  ["\u512A", "\u512A"], /* CJK Ideograph */
  ["\u512B", "\u512B"], /* CJK Ideograph */
  ["\u512C", "\u512C"], /* CJK Ideograph */
  ["\u512D", "\u512D"], /* CJK Ideograph */
  ["\u512E", "\u512E"], /* CJK Ideograph */
  ["\u512F", "\u512F"], /* CJK Ideograph */
  ["\u5130", "\u5130"], /* CJK Ideograph */
  ["\u5131", "\u5131"], /* CJK Ideograph */
  ["\u5132", "\u5132"], /* CJK Ideograph */
  ["\u5133", "\u5133"], /* CJK Ideograph */
  ["\u5134", "\u5134"], /* CJK Ideograph */
  ["\u5135", "\u5135"], /* CJK Ideograph */
  ["\u5136", "\u5136"], /* CJK Ideograph */
  ["\u5137", "\u5137"], /* CJK Ideograph */
  ["\u5138", "\u5138"], /* CJK Ideograph */
  ["\u5139", "\u5139"], /* CJK Ideograph */
  ["\u513A", "\u513A"], /* CJK Ideograph */
  ["\u513B", "\u513B"], /* CJK Ideograph */
  ["\u513C", "\u513C"], /* CJK Ideograph */
  ["\u513D", "\u513D"], /* CJK Ideograph */
  ["\u513E", "\u513E"], /* CJK Ideograph */
  ["\u513F", "\u513F"], /* CJK Ideograph */
  ["\u5140", "\u5140"], /* CJK Ideograph */
  ["\u5141", "\u5141"], /* CJK Ideograph */
  ["\u5142", "\u5142"], /* CJK Ideograph */
  ["\u5143", "\u5143"], /* CJK Ideograph */
  ["\u5144", "\u5144"], /* CJK Ideograph */
  ["\u5145", "\u5145"], /* CJK Ideograph */
  ["\u5146", "\u5146"], /* CJK Ideograph */
  ["\u5147", "\u5147"], /* CJK Ideograph */
  ["\u5148", "\u5148"], /* CJK Ideograph */
  ["\u5149", "\u5149"], /* CJK Ideograph */
  ["\u514A", "\u514A"], /* CJK Ideograph */
  ["\u514B", "\u514B"], /* CJK Ideograph */
  ["\u514C", "\u514C"], /* CJK Ideograph */
  ["\u514D", "\u514D"], /* CJK Ideograph */
  ["\u514E", "\u514E"], /* CJK Ideograph */
  ["\u514F", "\u514F"], /* CJK Ideograph */
  ["\u5150", "\u5150"], /* CJK Ideograph */
  ["\u5151", "\u5151"], /* CJK Ideograph */
  ["\u5152", "\u5152"], /* CJK Ideograph */
  ["\u5153", "\u5153"], /* CJK Ideograph */
  ["\u5154", "\u5154"], /* CJK Ideograph */
  ["\u5155", "\u5155"], /* CJK Ideograph */
  ["\u5156", "\u5156"], /* CJK Ideograph */
  ["\u5157", "\u5157"], /* CJK Ideograph */
  ["\u5158", "\u5158"], /* CJK Ideograph */
  ["\u5159", "\u5159"], /* CJK Ideograph */
  ["\u515A", "\u515A"], /* CJK Ideograph */
  ["\u515B", "\u515B"], /* CJK Ideograph */
  ["\u515C", "\u515C"], /* CJK Ideograph */
  ["\u515D", "\u515D"], /* CJK Ideograph */
  ["\u515E", "\u515E"], /* CJK Ideograph */
  ["\u515F", "\u515F"], /* CJK Ideograph */
  ["\u5160", "\u5160"], /* CJK Ideograph */
  ["\u5161", "\u5161"], /* CJK Ideograph */
  ["\u5162", "\u5162"], /* CJK Ideograph */
  ["\u5163", "\u5163"], /* CJK Ideograph */
  ["\u5164", "\u5164"], /* CJK Ideograph */
  ["\u5165", "\u5165"], /* CJK Ideograph */
  ["\u5166", "\u5166"], /* CJK Ideograph */
  ["\u5167", "\u5167"], /* CJK Ideograph */
  ["\u5168", "\u5168"], /* CJK Ideograph */
  ["\u5169", "\u5169"], /* CJK Ideograph */
  ["\u516A", "\u516A"], /* CJK Ideograph */
  ["\u516B", "\u516B"], /* CJK Ideograph */
  ["\u516C", "\u516C"], /* CJK Ideograph */
  ["\u516D", "\u516D"], /* CJK Ideograph */
  ["\u516E", "\u516E"], /* CJK Ideograph */
  ["\u516F", "\u516F"], /* CJK Ideograph */
  ["\u5170", "\u5170"], /* CJK Ideograph */
  ["\u5171", "\u5171"], /* CJK Ideograph */
  ["\u5172", "\u5172"], /* CJK Ideograph */
  ["\u5173", "\u5173"], /* CJK Ideograph */
  ["\u5174", "\u5174"], /* CJK Ideograph */
  ["\u5175", "\u5175"], /* CJK Ideograph */
  ["\u5176", "\u5176"], /* CJK Ideograph */
  ["\u5177", "\u5177"], /* CJK Ideograph */
  ["\u5178", "\u5178"], /* CJK Ideograph */
  ["\u5179", "\u5179"], /* CJK Ideograph */
  ["\u517A", "\u517A"], /* CJK Ideograph */
  ["\u517B", "\u517B"], /* CJK Ideograph */
  ["\u517C", "\u517C"], /* CJK Ideograph */
  ["\u517D", "\u517D"], /* CJK Ideograph */
  ["\u517E", "\u517E"], /* CJK Ideograph */
  ["\u517F", "\u517F"], /* CJK Ideograph */
  ["\u5180", "\u5180"], /* CJK Ideograph */
  ["\u5181", "\u5181"], /* CJK Ideograph */
  ["\u5182", "\u5182"], /* CJK Ideograph */
  ["\u5183", "\u5183"], /* CJK Ideograph */
  ["\u5184", "\u5184"], /* CJK Ideograph */
  ["\u5185", "\u5185"], /* CJK Ideograph */
  ["\u5186", "\u5186"], /* CJK Ideograph */
  ["\u5187", "\u5187"], /* CJK Ideograph */
  ["\u5188", "\u5188"], /* CJK Ideograph */
  ["\u5189", "\u5189"], /* CJK Ideograph */
  ["\u518A", "\u518A"], /* CJK Ideograph */
  ["\u518B", "\u518B"], /* CJK Ideograph */
  ["\u518C", "\u518C"], /* CJK Ideograph */
  ["\u518D", "\u518D"], /* CJK Ideograph */
  ["\u518E", "\u518E"], /* CJK Ideograph */
  ["\u518F", "\u518F"], /* CJK Ideograph */
  ["\u5190", "\u5190"], /* CJK Ideograph */
  ["\u5191", "\u5191"], /* CJK Ideograph */
  ["\u5192", "\u5192"], /* CJK Ideograph */
  ["\u5193", "\u5193"], /* CJK Ideograph */
  ["\u5194", "\u5194"], /* CJK Ideograph */
  ["\u5195", "\u5195"], /* CJK Ideograph */
  ["\u5196", "\u5196"], /* CJK Ideograph */
  ["\u5197", "\u5197"], /* CJK Ideograph */
  ["\u5198", "\u5198"], /* CJK Ideograph */
  ["\u5199", "\u5199"], /* CJK Ideograph */
  ["\u519A", "\u519A"], /* CJK Ideograph */
  ["\u519B", "\u519B"], /* CJK Ideograph */
  ["\u519C", "\u519C"], /* CJK Ideograph */
  ["\u519D", "\u519D"], /* CJK Ideograph */
  ["\u519E", "\u519E"], /* CJK Ideograph */
  ["\u519F", "\u519F"], /* CJK Ideograph */
  ["\u51A0", "\u51A0"], /* CJK Ideograph */
  ["\u51A1", "\u51A1"], /* CJK Ideograph */
  ["\u51A2", "\u51A2"], /* CJK Ideograph */
  ["\u51A3", "\u51A3"], /* CJK Ideograph */
  ["\u51A4", "\u51A4"], /* CJK Ideograph */
  ["\u51A5", "\u51A5"], /* CJK Ideograph */
  ["\u51A6", "\u51A6"], /* CJK Ideograph */
  ["\u51A7", "\u51A7"], /* CJK Ideograph */
  ["\u51A8", "\u51A8"], /* CJK Ideograph */
  ["\u51A9", "\u51A9"], /* CJK Ideograph */
  ["\u51AA", "\u51AA"], /* CJK Ideograph */
  ["\u51AB", "\u51AB"], /* CJK Ideograph */
  ["\u51AC", "\u51AC"], /* CJK Ideograph */
  ["\u51AD", "\u51AD"], /* CJK Ideograph */
  ["\u51AE", "\u51AE"], /* CJK Ideograph */
  ["\u51AF", "\u51AF"], /* CJK Ideograph */
  ["\u51B0", "\u51B0"], /* CJK Ideograph */
  ["\u51B1", "\u51B1"], /* CJK Ideograph */
  ["\u51B2", "\u51B2"], /* CJK Ideograph */
  ["\u51B3", "\u51B3"], /* CJK Ideograph */
  ["\u51B4", "\u51B4"], /* CJK Ideograph */
  ["\u51B5", "\u51B5"], /* CJK Ideograph */
  ["\u51B6", "\u51B6"], /* CJK Ideograph */
  ["\u51B7", "\u51B7"], /* CJK Ideograph */
  ["\u51B8", "\u51B8"], /* CJK Ideograph */
  ["\u51B9", "\u51B9"], /* CJK Ideograph */
  ["\u51BA", "\u51BA"], /* CJK Ideograph */
  ["\u51BB", "\u51BB"], /* CJK Ideograph */
  ["\u51BC", "\u51BC"], /* CJK Ideograph */
  ["\u51BD", "\u51BD"], /* CJK Ideograph */
  ["\u51BE", "\u51BE"], /* CJK Ideograph */
  ["\u51BF", "\u51BF"], /* CJK Ideograph */
  ["\u51C0", "\u51C0"], /* CJK Ideograph */
  ["\u51C1", "\u51C1"], /* CJK Ideograph */
  ["\u51C2", "\u51C2"], /* CJK Ideograph */
  ["\u51C3", "\u51C3"], /* CJK Ideograph */
  ["\u51C4", "\u51C4"], /* CJK Ideograph */
  ["\u51C5", "\u51C5"], /* CJK Ideograph */
  ["\u51C6", "\u51C6"], /* CJK Ideograph */
  ["\u51C7", "\u51C7"], /* CJK Ideograph */
  ["\u51C8", "\u51C8"], /* CJK Ideograph */
  ["\u51C9", "\u51C9"], /* CJK Ideograph */
  ["\u51CA", "\u51CA"], /* CJK Ideograph */
  ["\u51CB", "\u51CB"], /* CJK Ideograph */
  ["\u51CC", "\u51CC"], /* CJK Ideograph */
  ["\u51CD", "\u51CD"], /* CJK Ideograph */
  ["\u51CE", "\u51CE"], /* CJK Ideograph */
  ["\u51CF", "\u51CF"], /* CJK Ideograph */
  ["\u51D0", "\u51D0"], /* CJK Ideograph */
  ["\u51D1", "\u51D1"], /* CJK Ideograph */
  ["\u51D2", "\u51D2"], /* CJK Ideograph */
  ["\u51D3", "\u51D3"], /* CJK Ideograph */
  ["\u51D4", "\u51D4"], /* CJK Ideograph */
  ["\u51D5", "\u51D5"], /* CJK Ideograph */
  ["\u51D6", "\u51D6"], /* CJK Ideograph */
  ["\u51D7", "\u51D7"], /* CJK Ideograph */
  ["\u51D8", "\u51D8"], /* CJK Ideograph */
  ["\u51D9", "\u51D9"], /* CJK Ideograph */
  ["\u51DA", "\u51DA"], /* CJK Ideograph */
  ["\u51DB", "\u51DB"], /* CJK Ideograph */
  ["\u51DC", "\u51DC"], /* CJK Ideograph */
  ["\u51DD", "\u51DD"], /* CJK Ideograph */
  ["\u51DE", "\u51DE"], /* CJK Ideograph */
  ["\u51DF", "\u51DF"], /* CJK Ideograph */
  ["\u51E0", "\u51E0"], /* CJK Ideograph */
  ["\u51E1", "\u51E1"], /* CJK Ideograph */
  ["\u51E2", "\u51E2"], /* CJK Ideograph */
  ["\u51E3", "\u51E3"], /* CJK Ideograph */
  ["\u51E4", "\u51E4"], /* CJK Ideograph */
  ["\u51E5", "\u51E5"], /* CJK Ideograph */
  ["\u51E6", "\u51E6"], /* CJK Ideograph */
  ["\u51E7", "\u51E7"], /* CJK Ideograph */
  ["\u51E8", "\u51E8"], /* CJK Ideograph */
  ["\u51E9", "\u51E9"], /* CJK Ideograph */
  ["\u51EA", "\u51EA"], /* CJK Ideograph */
  ["\u51EB", "\u51EB"], /* CJK Ideograph */
  ["\u51EC", "\u51EC"], /* CJK Ideograph */
  ["\u51ED", "\u51ED"], /* CJK Ideograph */
  ["\u51EE", "\u51EE"], /* CJK Ideograph */
  ["\u51EF", "\u51EF"], /* CJK Ideograph */
  ["\u51F0", "\u51F0"], /* CJK Ideograph */
  ["\u51F1", "\u51F1"], /* CJK Ideograph */
  ["\u51F2", "\u51F2"], /* CJK Ideograph */
  ["\u51F3", "\u51F3"], /* CJK Ideograph */
  ["\u51F4", "\u51F4"], /* CJK Ideograph */
  ["\u51F5", "\u51F5"], /* CJK Ideograph */
  ["\u51F6", "\u51F6"], /* CJK Ideograph */
  ["\u51F7", "\u51F7"], /* CJK Ideograph */
  ["\u51F8", "\u51F8"], /* CJK Ideograph */
  ["\u51F9", "\u51F9"], /* CJK Ideograph */
  ["\u51FA", "\u51FA"], /* CJK Ideograph */
  ["\u51FB", "\u51FB"], /* CJK Ideograph */
  ["\u51FC", "\u51FC"], /* CJK Ideograph */
  ["\u51FD", "\u51FD"], /* CJK Ideograph */
  ["\u51FE", "\u51FE"], /* CJK Ideograph */
  ["\u51FF", "\u51FF"], /* CJK Ideograph */
  ["\u5200", "\u5200"], /* CJK Ideograph */
  ["\u5201", "\u5201"], /* CJK Ideograph */
  ["\u5202", "\u5202"], /* CJK Ideograph */
  ["\u5203", "\u5203"], /* CJK Ideograph */
  ["\u5204", "\u5204"], /* CJK Ideograph */
  ["\u5205", "\u5205"], /* CJK Ideograph */
  ["\u5206", "\u5206"], /* CJK Ideograph */
  ["\u5207", "\u5207"], /* CJK Ideograph */
  ["\u5208", "\u5208"], /* CJK Ideograph */
  ["\u5209", "\u5209"], /* CJK Ideograph */
  ["\u520A", "\u520A"], /* CJK Ideograph */
  ["\u520B", "\u520B"], /* CJK Ideograph */
  ["\u520C", "\u520C"], /* CJK Ideograph */
  ["\u520D", "\u520D"], /* CJK Ideograph */
  ["\u520E", "\u520E"], /* CJK Ideograph */
  ["\u520F", "\u520F"], /* CJK Ideograph */
  ["\u5210", "\u5210"], /* CJK Ideograph */
  ["\u5211", "\u5211"], /* CJK Ideograph */
  ["\u5212", "\u5212"], /* CJK Ideograph */
  ["\u5213", "\u5213"], /* CJK Ideograph */
  ["\u5214", "\u5214"], /* CJK Ideograph */
  ["\u5215", "\u5215"], /* CJK Ideograph */
  ["\u5216", "\u5216"], /* CJK Ideograph */
  ["\u5217", "\u5217"], /* CJK Ideograph */
  ["\u5218", "\u5218"], /* CJK Ideograph */
  ["\u5219", "\u5219"], /* CJK Ideograph */
  ["\u521A", "\u521A"], /* CJK Ideograph */
  ["\u521B", "\u521B"], /* CJK Ideograph */
  ["\u521C", "\u521C"], /* CJK Ideograph */
  ["\u521D", "\u521D"], /* CJK Ideograph */
  ["\u521E", "\u521E"], /* CJK Ideograph */
  ["\u521F", "\u521F"], /* CJK Ideograph */
  ["\u5220", "\u5220"], /* CJK Ideograph */
  ["\u5221", "\u5221"], /* CJK Ideograph */
  ["\u5222", "\u5222"], /* CJK Ideograph */
  ["\u5223", "\u5223"], /* CJK Ideograph */
  ["\u5224", "\u5224"], /* CJK Ideograph */
  ["\u5225", "\u5225"], /* CJK Ideograph */
  ["\u5226", "\u5226"], /* CJK Ideograph */
  ["\u5227", "\u5227"], /* CJK Ideograph */
  ["\u5228", "\u5228"], /* CJK Ideograph */
  ["\u5229", "\u5229"], /* CJK Ideograph */
  ["\u522A", "\u522A"], /* CJK Ideograph */
  ["\u522B", "\u522B"], /* CJK Ideograph */
  ["\u522C", "\u522C"], /* CJK Ideograph */
  ["\u522D", "\u522D"], /* CJK Ideograph */
  ["\u522E", "\u522E"], /* CJK Ideograph */
  ["\u522F", "\u522F"], /* CJK Ideograph */
  ["\u5230", "\u5230"], /* CJK Ideograph */
  ["\u5231", "\u5231"], /* CJK Ideograph */
  ["\u5232", "\u5232"], /* CJK Ideograph */
  ["\u5233", "\u5233"], /* CJK Ideograph */
  ["\u5234", "\u5234"], /* CJK Ideograph */
  ["\u5235", "\u5235"], /* CJK Ideograph */
  ["\u5236", "\u5236"], /* CJK Ideograph */
  ["\u5237", "\u5237"], /* CJK Ideograph */
  ["\u5238", "\u5238"], /* CJK Ideograph */
  ["\u5239", "\u5239"], /* CJK Ideograph */
  ["\u523A", "\u523A"], /* CJK Ideograph */
  ["\u523B", "\u523B"], /* CJK Ideograph */
  ["\u523C", "\u523C"], /* CJK Ideograph */
  ["\u523D", "\u523D"], /* CJK Ideograph */
  ["\u523E", "\u523E"], /* CJK Ideograph */
  ["\u523F", "\u523F"], /* CJK Ideograph */
  ["\u5240", "\u5240"], /* CJK Ideograph */
  ["\u5241", "\u5241"], /* CJK Ideograph */
  ["\u5242", "\u5242"], /* CJK Ideograph */
  ["\u5243", "\u5243"], /* CJK Ideograph */
  ["\u5244", "\u5244"], /* CJK Ideograph */
  ["\u5245", "\u5245"], /* CJK Ideograph */
  ["\u5246", "\u5246"], /* CJK Ideograph */
  ["\u5247", "\u5247"], /* CJK Ideograph */
  ["\u5248", "\u5248"], /* CJK Ideograph */
  ["\u5249", "\u5249"], /* CJK Ideograph */
  ["\u524A", "\u524A"], /* CJK Ideograph */
  ["\u524B", "\u524B"], /* CJK Ideograph */
  ["\u524C", "\u524C"], /* CJK Ideograph */
  ["\u524D", "\u524D"], /* CJK Ideograph */
  ["\u524E", "\u524E"], /* CJK Ideograph */
  ["\u524F", "\u524F"], /* CJK Ideograph */
  ["\u5250", "\u5250"], /* CJK Ideograph */
  ["\u5251", "\u5251"], /* CJK Ideograph */
  ["\u5252", "\u5252"], /* CJK Ideograph */
  ["\u5253", "\u5253"], /* CJK Ideograph */
  ["\u5254", "\u5254"], /* CJK Ideograph */
  ["\u5255", "\u5255"], /* CJK Ideograph */
  ["\u5256", "\u5256"], /* CJK Ideograph */
  ["\u5257", "\u5257"], /* CJK Ideograph */
  ["\u5258", "\u5258"], /* CJK Ideograph */
  ["\u5259", "\u5259"], /* CJK Ideograph */
  ["\u525A", "\u525A"], /* CJK Ideograph */
  ["\u525B", "\u525B"], /* CJK Ideograph */
  ["\u525C", "\u525C"], /* CJK Ideograph */
  ["\u525D", "\u525D"], /* CJK Ideograph */
  ["\u525E", "\u525E"], /* CJK Ideograph */
  ["\u525F", "\u525F"], /* CJK Ideograph */
  ["\u5260", "\u5260"], /* CJK Ideograph */
  ["\u5261", "\u5261"], /* CJK Ideograph */
  ["\u5262", "\u5262"], /* CJK Ideograph */
  ["\u5263", "\u5263"], /* CJK Ideograph */
  ["\u5264", "\u5264"], /* CJK Ideograph */
  ["\u5265", "\u5265"], /* CJK Ideograph */
  ["\u5266", "\u5266"], /* CJK Ideograph */
  ["\u5267", "\u5267"], /* CJK Ideograph */
  ["\u5268", "\u5268"], /* CJK Ideograph */
  ["\u5269", "\u5269"], /* CJK Ideograph */
  ["\u526A", "\u526A"], /* CJK Ideograph */
  ["\u526B", "\u526B"], /* CJK Ideograph */
  ["\u526C", "\u526C"], /* CJK Ideograph */
  ["\u526D", "\u526D"], /* CJK Ideograph */
  ["\u526E", "\u526E"], /* CJK Ideograph */
  ["\u526F", "\u526F"], /* CJK Ideograph */
  ["\u5270", "\u5270"], /* CJK Ideograph */
  ["\u5271", "\u5271"], /* CJK Ideograph */
  ["\u5272", "\u5272"], /* CJK Ideograph */
  ["\u5273", "\u5273"], /* CJK Ideograph */
  ["\u5274", "\u5274"], /* CJK Ideograph */
  ["\u5275", "\u5275"], /* CJK Ideograph */
  ["\u5276", "\u5276"], /* CJK Ideograph */
  ["\u5277", "\u5277"], /* CJK Ideograph */
  ["\u5278", "\u5278"], /* CJK Ideograph */
  ["\u5279", "\u5279"], /* CJK Ideograph */
  ["\u527A", "\u527A"], /* CJK Ideograph */
  ["\u527B", "\u527B"], /* CJK Ideograph */
  ["\u527C", "\u527C"], /* CJK Ideograph */
  ["\u527D", "\u527D"], /* CJK Ideograph */
  ["\u527E", "\u527E"], /* CJK Ideograph */
  ["\u527F", "\u527F"], /* CJK Ideograph */
  ["\u5280", "\u5280"], /* CJK Ideograph */
  ["\u5281", "\u5281"], /* CJK Ideograph */
  ["\u5282", "\u5282"], /* CJK Ideograph */
  ["\u5283", "\u5283"], /* CJK Ideograph */
  ["\u5284", "\u5284"], /* CJK Ideograph */
  ["\u5285", "\u5285"], /* CJK Ideograph */
  ["\u5286", "\u5286"], /* CJK Ideograph */
  ["\u5287", "\u5287"], /* CJK Ideograph */
  ["\u5288", "\u5288"], /* CJK Ideograph */
  ["\u5289", "\u5289"], /* CJK Ideograph */
  ["\u528A", "\u528A"], /* CJK Ideograph */
  ["\u528B", "\u528B"], /* CJK Ideograph */
  ["\u528C", "\u528C"], /* CJK Ideograph */
  ["\u528D", "\u528D"], /* CJK Ideograph */
  ["\u528E", "\u528E"], /* CJK Ideograph */
  ["\u528F", "\u528F"], /* CJK Ideograph */
  ["\u5290", "\u5290"], /* CJK Ideograph */
  ["\u5291", "\u5291"], /* CJK Ideograph */
  ["\u5292", "\u5292"], /* CJK Ideograph */
  ["\u5293", "\u5293"], /* CJK Ideograph */
  ["\u5294", "\u5294"], /* CJK Ideograph */
  ["\u5295", "\u5295"], /* CJK Ideograph */
  ["\u5296", "\u5296"], /* CJK Ideograph */
  ["\u5297", "\u5297"], /* CJK Ideograph */
  ["\u5298", "\u5298"], /* CJK Ideograph */
  ["\u5299", "\u5299"], /* CJK Ideograph */
  ["\u529A", "\u529A"], /* CJK Ideograph */
  ["\u529B", "\u529B"], /* CJK Ideograph */
  ["\u529C", "\u529C"], /* CJK Ideograph */
  ["\u529D", "\u529D"], /* CJK Ideograph */
  ["\u529E", "\u529E"], /* CJK Ideograph */
  ["\u529F", "\u529F"], /* CJK Ideograph */
  ["\u52A0", "\u52A0"], /* CJK Ideograph */
  ["\u52A1", "\u52A1"], /* CJK Ideograph */
  ["\u52A2", "\u52A2"], /* CJK Ideograph */
  ["\u52A3", "\u52A3"], /* CJK Ideograph */
  ["\u52A4", "\u52A4"], /* CJK Ideograph */
  ["\u52A5", "\u52A5"], /* CJK Ideograph */
  ["\u52A6", "\u52A6"], /* CJK Ideograph */
  ["\u52A7", "\u52A7"], /* CJK Ideograph */
  ["\u52A8", "\u52A8"], /* CJK Ideograph */
  ["\u52A9", "\u52A9"], /* CJK Ideograph */
  ["\u52AA", "\u52AA"], /* CJK Ideograph */
  ["\u52AB", "\u52AB"], /* CJK Ideograph */
  ["\u52AC", "\u52AC"], /* CJK Ideograph */
  ["\u52AD", "\u52AD"], /* CJK Ideograph */
  ["\u52AE", "\u52AE"], /* CJK Ideograph */
  ["\u52AF", "\u52AF"], /* CJK Ideograph */
  ["\u52B0", "\u52B0"], /* CJK Ideograph */
  ["\u52B1", "\u52B1"], /* CJK Ideograph */
  ["\u52B2", "\u52B2"], /* CJK Ideograph */
  ["\u52B3", "\u52B3"], /* CJK Ideograph */
  ["\u52B4", "\u52B4"], /* CJK Ideograph */
  ["\u52B5", "\u52B5"], /* CJK Ideograph */
  ["\u52B6", "\u52B6"], /* CJK Ideograph */
  ["\u52B7", "\u52B7"], /* CJK Ideograph */
  ["\u52B8", "\u52B8"], /* CJK Ideograph */
  ["\u52B9", "\u52B9"], /* CJK Ideograph */
  ["\u52BA", "\u52BA"], /* CJK Ideograph */
  ["\u52BB", "\u52BB"], /* CJK Ideograph */
  ["\u52BC", "\u52BC"], /* CJK Ideograph */
  ["\u52BD", "\u52BD"], /* CJK Ideograph */
  ["\u52BE", "\u52BE"], /* CJK Ideograph */
  ["\u52BF", "\u52BF"], /* CJK Ideograph */
  ["\u52C0", "\u52C0"], /* CJK Ideograph */
  ["\u52C1", "\u52C1"], /* CJK Ideograph */
  ["\u52C2", "\u52C2"], /* CJK Ideograph */
  ["\u52C3", "\u52C3"], /* CJK Ideograph */
  ["\u52C4", "\u52C4"], /* CJK Ideograph */
  ["\u52C5", "\u52C5"], /* CJK Ideograph */
  ["\u52C6", "\u52C6"], /* CJK Ideograph */
  ["\u52C7", "\u52C7"], /* CJK Ideograph */
  ["\u52C8", "\u52C8"], /* CJK Ideograph */
  ["\u52C9", "\u52C9"], /* CJK Ideograph */
  ["\u52CA", "\u52CA"], /* CJK Ideograph */
  ["\u52CB", "\u52CB"], /* CJK Ideograph */
  ["\u52CC", "\u52CC"], /* CJK Ideograph */
  ["\u52CD", "\u52CD"], /* CJK Ideograph */
  ["\u52CE", "\u52CE"], /* CJK Ideograph */
  ["\u52CF", "\u52CF"], /* CJK Ideograph */
  ["\u52D0", "\u52D0"], /* CJK Ideograph */
  ["\u52D1", "\u52D1"], /* CJK Ideograph */
  ["\u52D2", "\u52D2"], /* CJK Ideograph */
  ["\u52D3", "\u52D3"], /* CJK Ideograph */
  ["\u52D4", "\u52D4"], /* CJK Ideograph */
  ["\u52D5", "\u52D5"], /* CJK Ideograph */
  ["\u52D6", "\u52D6"], /* CJK Ideograph */
  ["\u52D7", "\u52D7"], /* CJK Ideograph */
  ["\u52D8", "\u52D8"], /* CJK Ideograph */
  ["\u52D9", "\u52D9"], /* CJK Ideograph */
  ["\u52DA", "\u52DA"], /* CJK Ideograph */
  ["\u52DB", "\u52DB"], /* CJK Ideograph */
  ["\u52DC", "\u52DC"], /* CJK Ideograph */
  ["\u52DD", "\u52DD"], /* CJK Ideograph */
  ["\u52DE", "\u52DE"], /* CJK Ideograph */
  ["\u52DF", "\u52DF"], /* CJK Ideograph */
  ["\u52E0", "\u52E0"], /* CJK Ideograph */
  ["\u52E1", "\u52E1"], /* CJK Ideograph */
  ["\u52E2", "\u52E2"], /* CJK Ideograph */
  ["\u52E3", "\u52E3"], /* CJK Ideograph */
  ["\u52E4", "\u52E4"], /* CJK Ideograph */
  ["\u52E5", "\u52E5"], /* CJK Ideograph */
  ["\u52E6", "\u52E6"], /* CJK Ideograph */
  ["\u52E7", "\u52E7"], /* CJK Ideograph */
  ["\u52E8", "\u52E8"], /* CJK Ideograph */
  ["\u52E9", "\u52E9"], /* CJK Ideograph */
  ["\u52EA", "\u52EA"], /* CJK Ideograph */
  ["\u52EB", "\u52EB"], /* CJK Ideograph */
  ["\u52EC", "\u52EC"], /* CJK Ideograph */
  ["\u52ED", "\u52ED"], /* CJK Ideograph */
  ["\u52EE", "\u52EE"], /* CJK Ideograph */
  ["\u52EF", "\u52EF"], /* CJK Ideograph */
  ["\u52F0", "\u52F0"], /* CJK Ideograph */
  ["\u52F1", "\u52F1"], /* CJK Ideograph */
  ["\u52F2", "\u52F2"], /* CJK Ideograph */
  ["\u52F3", "\u52F3"], /* CJK Ideograph */
  ["\u52F4", "\u52F4"], /* CJK Ideograph */
  ["\u52F5", "\u52F5"], /* CJK Ideograph */
  ["\u52F6", "\u52F6"], /* CJK Ideograph */
  ["\u52F7", "\u52F7"], /* CJK Ideograph */
  ["\u52F8", "\u52F8"], /* CJK Ideograph */
  ["\u52F9", "\u52F9"], /* CJK Ideograph */
  ["\u52FA", "\u52FA"], /* CJK Ideograph */
  ["\u52FB", "\u52FB"], /* CJK Ideograph */
  ["\u52FC", "\u52FC"], /* CJK Ideograph */
  ["\u52FD", "\u52FD"], /* CJK Ideograph */
  ["\u52FE", "\u52FE"], /* CJK Ideograph */
  ["\u52FF", "\u52FF"], /* CJK Ideograph */
  ["\u5300", "\u5300"], /* CJK Ideograph */
  ["\u5301", "\u5301"], /* CJK Ideograph */
  ["\u5302", "\u5302"], /* CJK Ideograph */
  ["\u5303", "\u5303"], /* CJK Ideograph */
  ["\u5304", "\u5304"], /* CJK Ideograph */
  ["\u5305", "\u5305"], /* CJK Ideograph */
  ["\u5306", "\u5306"], /* CJK Ideograph */
  ["\u5307", "\u5307"], /* CJK Ideograph */
  ["\u5308", "\u5308"], /* CJK Ideograph */
  ["\u5309", "\u5309"], /* CJK Ideograph */
  ["\u530A", "\u530A"], /* CJK Ideograph */
  ["\u530B", "\u530B"], /* CJK Ideograph */
  ["\u530C", "\u530C"], /* CJK Ideograph */
  ["\u530D", "\u530D"], /* CJK Ideograph */
  ["\u530E", "\u530E"], /* CJK Ideograph */
  ["\u530F", "\u530F"], /* CJK Ideograph */
  ["\u5310", "\u5310"], /* CJK Ideograph */
  ["\u5311", "\u5311"], /* CJK Ideograph */
  ["\u5312", "\u5312"], /* CJK Ideograph */
  ["\u5313", "\u5313"], /* CJK Ideograph */
  ["\u5314", "\u5314"], /* CJK Ideograph */
  ["\u5315", "\u5315"], /* CJK Ideograph */
  ["\u5316", "\u5316"], /* CJK Ideograph */
  ["\u5317", "\u5317"], /* CJK Ideograph */
  ["\u5318", "\u5318"], /* CJK Ideograph */
  ["\u5319", "\u5319"], /* CJK Ideograph */
  ["\u531A", "\u531A"], /* CJK Ideograph */
  ["\u531B", "\u531B"], /* CJK Ideograph */
  ["\u531C", "\u531C"], /* CJK Ideograph */
  ["\u531D", "\u531D"], /* CJK Ideograph */
  ["\u531E", "\u531E"], /* CJK Ideograph */
  ["\u531F", "\u531F"], /* CJK Ideograph */
  ["\u5320", "\u5320"], /* CJK Ideograph */
  ["\u5321", "\u5321"], /* CJK Ideograph */
  ["\u5322", "\u5322"], /* CJK Ideograph */
  ["\u5323", "\u5323"], /* CJK Ideograph */
  ["\u5324", "\u5324"], /* CJK Ideograph */
  ["\u5325", "\u5325"], /* CJK Ideograph */
  ["\u5326", "\u5326"], /* CJK Ideograph */
  ["\u5327", "\u5327"], /* CJK Ideograph */
  ["\u5328", "\u5328"], /* CJK Ideograph */
  ["\u5329", "\u5329"], /* CJK Ideograph */
  ["\u532A", "\u532A"], /* CJK Ideograph */
  ["\u532B", "\u532B"], /* CJK Ideograph */
  ["\u532C", "\u532C"], /* CJK Ideograph */
  ["\u532D", "\u532D"], /* CJK Ideograph */
  ["\u532E", "\u532E"], /* CJK Ideograph */
  ["\u532F", "\u532F"], /* CJK Ideograph */
  ["\u5330", "\u5330"], /* CJK Ideograph */
  ["\u5331", "\u5331"], /* CJK Ideograph */
  ["\u5332", "\u5332"], /* CJK Ideograph */
  ["\u5333", "\u5333"], /* CJK Ideograph */
  ["\u5334", "\u5334"], /* CJK Ideograph */
  ["\u5335", "\u5335"], /* CJK Ideograph */
  ["\u5336", "\u5336"], /* CJK Ideograph */
  ["\u5337", "\u5337"], /* CJK Ideograph */
  ["\u5338", "\u5338"], /* CJK Ideograph */
  ["\u5339", "\u5339"], /* CJK Ideograph */
  ["\u533A", "\u533A"], /* CJK Ideograph */
  ["\u533B", "\u533B"], /* CJK Ideograph */
  ["\u533C", "\u533C"], /* CJK Ideograph */
  ["\u533D", "\u533D"], /* CJK Ideograph */
  ["\u533E", "\u533E"], /* CJK Ideograph */
  ["\u533F", "\u533F"], /* CJK Ideograph */
  ["\u5340", "\u5340"], /* CJK Ideograph */
  ["\u5341", "\u5341"], /* CJK Ideograph */
  ["\u5342", "\u5342"], /* CJK Ideograph */
  ["\u5343", "\u5343"], /* CJK Ideograph */
  ["\u5344", "\u5344"], /* CJK Ideograph */
  ["\u5345", "\u5345"], /* CJK Ideograph */
  ["\u5346", "\u5346"], /* CJK Ideograph */
  ["\u5347", "\u5347"], /* CJK Ideograph */
  ["\u5348", "\u5348"], /* CJK Ideograph */
  ["\u5349", "\u5349"], /* CJK Ideograph */
  ["\u534A", "\u534A"], /* CJK Ideograph */
  ["\u534B", "\u534B"], /* CJK Ideograph */
  ["\u534C", "\u534C"], /* CJK Ideograph */
  ["\u534D", "\u534D"], /* CJK Ideograph */
  ["\u534E", "\u534E"], /* CJK Ideograph */
  ["\u534F", "\u534F"], /* CJK Ideograph */
  ["\u5350", "\u5350"], /* CJK Ideograph */
  ["\u5351", "\u5351"], /* CJK Ideograph */
  ["\u5352", "\u5352"], /* CJK Ideograph */
  ["\u5353", "\u5353"], /* CJK Ideograph */
  ["\u5354", "\u5354"], /* CJK Ideograph */
  ["\u5355", "\u5355"], /* CJK Ideograph */
  ["\u5356", "\u5356"], /* CJK Ideograph */
  ["\u5357", "\u5357"], /* CJK Ideograph */
  ["\u5358", "\u5358"], /* CJK Ideograph */
  ["\u5359", "\u5359"], /* CJK Ideograph */
  ["\u535A", "\u535A"], /* CJK Ideograph */
  ["\u535B", "\u535B"], /* CJK Ideograph */
  ["\u535C", "\u535C"], /* CJK Ideograph */
  ["\u535D", "\u535D"], /* CJK Ideograph */
  ["\u535E", "\u535E"], /* CJK Ideograph */
  ["\u535F", "\u535F"], /* CJK Ideograph */
  ["\u5360", "\u5360"], /* CJK Ideograph */
  ["\u5361", "\u5361"], /* CJK Ideograph */
  ["\u5362", "\u5362"], /* CJK Ideograph */
  ["\u5363", "\u5363"], /* CJK Ideograph */
  ["\u5364", "\u5364"], /* CJK Ideograph */
  ["\u5365", "\u5365"], /* CJK Ideograph */
  ["\u5366", "\u5366"], /* CJK Ideograph */
  ["\u5367", "\u5367"], /* CJK Ideograph */
  ["\u5368", "\u5368"], /* CJK Ideograph */
  ["\u5369", "\u5369"], /* CJK Ideograph */
  ["\u536A", "\u536A"], /* CJK Ideograph */
  ["\u536B", "\u536B"], /* CJK Ideograph */
  ["\u536C", "\u536C"], /* CJK Ideograph */
  ["\u536D", "\u536D"], /* CJK Ideograph */
  ["\u536E", "\u536E"], /* CJK Ideograph */
  ["\u536F", "\u536F"], /* CJK Ideograph */
  ["\u5370", "\u5370"], /* CJK Ideograph */
  ["\u5371", "\u5371"], /* CJK Ideograph */
  ["\u5372", "\u5372"], /* CJK Ideograph */
  ["\u5373", "\u5373"], /* CJK Ideograph */
  ["\u5374", "\u5374"], /* CJK Ideograph */
  ["\u5375", "\u5375"], /* CJK Ideograph */
  ["\u5376", "\u5376"], /* CJK Ideograph */
  ["\u5377", "\u5377"], /* CJK Ideograph */
  ["\u5378", "\u5378"], /* CJK Ideograph */
  ["\u5379", "\u5379"], /* CJK Ideograph */
  ["\u537A", "\u537A"], /* CJK Ideograph */
  ["\u537B", "\u537B"], /* CJK Ideograph */
  ["\u537C", "\u537C"], /* CJK Ideograph */
  ["\u537D", "\u537D"], /* CJK Ideograph */
  ["\u537E", "\u537E"], /* CJK Ideograph */
  ["\u537F", "\u537F"], /* CJK Ideograph */
  ["\u5380", "\u5380"], /* CJK Ideograph */
  ["\u5381", "\u5381"], /* CJK Ideograph */
  ["\u5382", "\u5382"], /* CJK Ideograph */
  ["\u5383", "\u5383"], /* CJK Ideograph */
  ["\u5384", "\u5384"], /* CJK Ideograph */
  ["\u5385", "\u5385"], /* CJK Ideograph */
  ["\u5386", "\u5386"], /* CJK Ideograph */
  ["\u5387", "\u5387"], /* CJK Ideograph */
  ["\u5388", "\u5388"], /* CJK Ideograph */
  ["\u5389", "\u5389"], /* CJK Ideograph */
  ["\u538A", "\u538A"], /* CJK Ideograph */
  ["\u538B", "\u538B"], /* CJK Ideograph */
  ["\u538C", "\u538C"], /* CJK Ideograph */
  ["\u538D", "\u538D"], /* CJK Ideograph */
  ["\u538E", "\u538E"], /* CJK Ideograph */
  ["\u538F", "\u538F"], /* CJK Ideograph */
  ["\u5390", "\u5390"], /* CJK Ideograph */
  ["\u5391", "\u5391"], /* CJK Ideograph */
  ["\u5392", "\u5392"], /* CJK Ideograph */
  ["\u5393", "\u5393"], /* CJK Ideograph */
  ["\u5394", "\u5394"], /* CJK Ideograph */
  ["\u5395", "\u5395"], /* CJK Ideograph */
  ["\u5396", "\u5396"], /* CJK Ideograph */
  ["\u5397", "\u5397"], /* CJK Ideograph */
  ["\u5398", "\u5398"], /* CJK Ideograph */
  ["\u5399", "\u5399"], /* CJK Ideograph */
  ["\u539A", "\u539A"], /* CJK Ideograph */
  ["\u539B", "\u539B"], /* CJK Ideograph */
  ["\u539C", "\u539C"], /* CJK Ideograph */
  ["\u539D", "\u539D"], /* CJK Ideograph */
  ["\u539E", "\u539E"], /* CJK Ideograph */
  ["\u539F", "\u539F"], /* CJK Ideograph */
  ["\u53A0", "\u53A0"], /* CJK Ideograph */
  ["\u53A1", "\u53A1"], /* CJK Ideograph */
  ["\u53A2", "\u53A2"], /* CJK Ideograph */
  ["\u53A3", "\u53A3"], /* CJK Ideograph */
  ["\u53A4", "\u53A4"], /* CJK Ideograph */
  ["\u53A5", "\u53A5"], /* CJK Ideograph */
  ["\u53A6", "\u53A6"], /* CJK Ideograph */
  ["\u53A7", "\u53A7"], /* CJK Ideograph */
  ["\u53A8", "\u53A8"], /* CJK Ideograph */
  ["\u53A9", "\u53A9"], /* CJK Ideograph */
  ["\u53AA", "\u53AA"], /* CJK Ideograph */
  ["\u53AB", "\u53AB"], /* CJK Ideograph */
  ["\u53AC", "\u53AC"], /* CJK Ideograph */
  ["\u53AD", "\u53AD"], /* CJK Ideograph */
  ["\u53AE", "\u53AE"], /* CJK Ideograph */
  ["\u53AF", "\u53AF"], /* CJK Ideograph */
  ["\u53B0", "\u53B0"], /* CJK Ideograph */
  ["\u53B1", "\u53B1"], /* CJK Ideograph */
  ["\u53B2", "\u53B2"], /* CJK Ideograph */
  ["\u53B3", "\u53B3"], /* CJK Ideograph */
  ["\u53B4", "\u53B4"], /* CJK Ideograph */
  ["\u53B5", "\u53B5"], /* CJK Ideograph */
  ["\u53B6", "\u53B6"], /* CJK Ideograph */
  ["\u53B7", "\u53B7"], /* CJK Ideograph */
  ["\u53B8", "\u53B8"], /* CJK Ideograph */
  ["\u53B9", "\u53B9"], /* CJK Ideograph */
  ["\u53BA", "\u53BA"], /* CJK Ideograph */
  ["\u53BB", "\u53BB"], /* CJK Ideograph */
  ["\u53BC", "\u53BC"], /* CJK Ideograph */
  ["\u53BD", "\u53BD"], /* CJK Ideograph */
  ["\u53BE", "\u53BE"], /* CJK Ideograph */
  ["\u53BF", "\u53BF"], /* CJK Ideograph */
  ["\u53C0", "\u53C0"], /* CJK Ideograph */
  ["\u53C1", "\u53C1"], /* CJK Ideograph */
  ["\u53C2", "\u53C2"], /* CJK Ideograph */
  ["\u53C3", "\u53C3"], /* CJK Ideograph */
  ["\u53C4", "\u53C4"], /* CJK Ideograph */
  ["\u53C5", "\u53C5"], /* CJK Ideograph */
  ["\u53C6", "\u53C6"], /* CJK Ideograph */
  ["\u53C7", "\u53C7"], /* CJK Ideograph */
  ["\u53C8", "\u53C8"], /* CJK Ideograph */
  ["\u53C9", "\u53C9"], /* CJK Ideograph */
  ["\u53CA", "\u53CA"], /* CJK Ideograph */
  ["\u53CB", "\u53CB"], /* CJK Ideograph */
  ["\u53CC", "\u53CC"], /* CJK Ideograph */
  ["\u53CD", "\u53CD"], /* CJK Ideograph */
  ["\u53CE", "\u53CE"], /* CJK Ideograph */
  ["\u53CF", "\u53CF"], /* CJK Ideograph */
  ["\u53D0", "\u53D0"], /* CJK Ideograph */
  ["\u53D1", "\u53D1"], /* CJK Ideograph */
  ["\u53D2", "\u53D2"], /* CJK Ideograph */
  ["\u53D3", "\u53D3"], /* CJK Ideograph */
  ["\u53D4", "\u53D4"], /* CJK Ideograph */
  ["\u53D5", "\u53D5"], /* CJK Ideograph */
  ["\u53D6", "\u53D6"], /* CJK Ideograph */
  ["\u53D7", "\u53D7"], /* CJK Ideograph */
  ["\u53D8", "\u53D8"], /* CJK Ideograph */
  ["\u53D9", "\u53D9"], /* CJK Ideograph */
  ["\u53DA", "\u53DA"], /* CJK Ideograph */
  ["\u53DB", "\u53DB"], /* CJK Ideograph */
  ["\u53DC", "\u53DC"], /* CJK Ideograph */
  ["\u53DD", "\u53DD"], /* CJK Ideograph */
  ["\u53DE", "\u53DE"], /* CJK Ideograph */
  ["\u53DF", "\u53DF"], /* CJK Ideograph */
  ["\u53E0", "\u53E0"], /* CJK Ideograph */
  ["\u53E1", "\u53E1"], /* CJK Ideograph */
  ["\u53E2", "\u53E2"], /* CJK Ideograph */
  ["\u53E3", "\u53E3"], /* CJK Ideograph */
  ["\u53E4", "\u53E4"], /* CJK Ideograph */
  ["\u53E5", "\u53E5"], /* CJK Ideograph */
  ["\u53E6", "\u53E6"], /* CJK Ideograph */
  ["\u53E7", "\u53E7"], /* CJK Ideograph */
  ["\u53E8", "\u53E8"], /* CJK Ideograph */
  ["\u53E9", "\u53E9"], /* CJK Ideograph */
  ["\u53EA", "\u53EA"], /* CJK Ideograph */
  ["\u53EB", "\u53EB"], /* CJK Ideograph */
  ["\u53EC", "\u53EC"], /* CJK Ideograph */
  ["\u53ED", "\u53ED"], /* CJK Ideograph */
  ["\u53EE", "\u53EE"], /* CJK Ideograph */
  ["\u53EF", "\u53EF"], /* CJK Ideograph */
  ["\u53F0", "\u53F0"], /* CJK Ideograph */
  ["\u53F1", "\u53F1"], /* CJK Ideograph */
  ["\u53F2", "\u53F2"], /* CJK Ideograph */
  ["\u53F3", "\u53F3"], /* CJK Ideograph */
  ["\u53F4", "\u53F4"], /* CJK Ideograph */
  ["\u53F5", "\u53F5"], /* CJK Ideograph */
  ["\u53F6", "\u53F6"], /* CJK Ideograph */
  ["\u53F7", "\u53F7"], /* CJK Ideograph */
  ["\u53F8", "\u53F8"], /* CJK Ideograph */
  ["\u53F9", "\u53F9"], /* CJK Ideograph */
  ["\u53FA", "\u53FA"], /* CJK Ideograph */
  ["\u53FB", "\u53FB"], /* CJK Ideograph */
  ["\u53FC", "\u53FC"], /* CJK Ideograph */
  ["\u53FD", "\u53FD"], /* CJK Ideograph */
  ["\u53FE", "\u53FE"], /* CJK Ideograph */
  ["\u53FF", "\u53FF"], /* CJK Ideograph */
  ["\u5400", "\u5400"], /* CJK Ideograph */
  ["\u5401", "\u5401"], /* CJK Ideograph */
  ["\u5402", "\u5402"], /* CJK Ideograph */
  ["\u5403", "\u5403"], /* CJK Ideograph */
  ["\u5404", "\u5404"], /* CJK Ideograph */
  ["\u5405", "\u5405"], /* CJK Ideograph */
  ["\u5406", "\u5406"], /* CJK Ideograph */
  ["\u5407", "\u5407"], /* CJK Ideograph */
  ["\u5408", "\u5408"], /* CJK Ideograph */
  ["\u5409", "\u5409"], /* CJK Ideograph */
  ["\u540A", "\u540A"], /* CJK Ideograph */
  ["\u540B", "\u540B"], /* CJK Ideograph */
  ["\u540C", "\u540C"], /* CJK Ideograph */
  ["\u540D", "\u540D"], /* CJK Ideograph */
  ["\u540E", "\u540E"], /* CJK Ideograph */
  ["\u540F", "\u540F"], /* CJK Ideograph */
  ["\u5410", "\u5410"], /* CJK Ideograph */
  ["\u5411", "\u5411"], /* CJK Ideograph */
  ["\u5412", "\u5412"], /* CJK Ideograph */
  ["\u5413", "\u5413"], /* CJK Ideograph */
  ["\u5414", "\u5414"], /* CJK Ideograph */
  ["\u5415", "\u5415"], /* CJK Ideograph */
  ["\u5416", "\u5416"], /* CJK Ideograph */
  ["\u5417", "\u5417"], /* CJK Ideograph */
  ["\u5418", "\u5418"], /* CJK Ideograph */
  ["\u5419", "\u5419"], /* CJK Ideograph */
  ["\u541A", "\u541A"], /* CJK Ideograph */
  ["\u541B", "\u541B"], /* CJK Ideograph */
  ["\u541C", "\u541C"], /* CJK Ideograph */
  ["\u541D", "\u541D"], /* CJK Ideograph */
  ["\u541E", "\u541E"], /* CJK Ideograph */
  ["\u541F", "\u541F"], /* CJK Ideograph */
  ["\u5420", "\u5420"], /* CJK Ideograph */
  ["\u5421", "\u5421"], /* CJK Ideograph */
  ["\u5422", "\u5422"], /* CJK Ideograph */
  ["\u5423", "\u5423"], /* CJK Ideograph */
  ["\u5424", "\u5424"], /* CJK Ideograph */
  ["\u5425", "\u5425"], /* CJK Ideograph */
  ["\u5426", "\u5426"], /* CJK Ideograph */
  ["\u5427", "\u5427"], /* CJK Ideograph */
  ["\u5428", "\u5428"], /* CJK Ideograph */
  ["\u5429", "\u5429"], /* CJK Ideograph */
  ["\u542A", "\u542A"], /* CJK Ideograph */
  ["\u542B", "\u542B"], /* CJK Ideograph */
  ["\u542C", "\u542C"], /* CJK Ideograph */
  ["\u542D", "\u542D"], /* CJK Ideograph */
  ["\u542E", "\u542E"], /* CJK Ideograph */
  ["\u542F", "\u542F"], /* CJK Ideograph */
  ["\u5430", "\u5430"], /* CJK Ideograph */
  ["\u5431", "\u5431"], /* CJK Ideograph */
  ["\u5432", "\u5432"], /* CJK Ideograph */
  ["\u5433", "\u5433"], /* CJK Ideograph */
  ["\u5434", "\u5434"], /* CJK Ideograph */
  ["\u5435", "\u5435"], /* CJK Ideograph */
  ["\u5436", "\u5436"], /* CJK Ideograph */
  ["\u5437", "\u5437"], /* CJK Ideograph */
  ["\u5438", "\u5438"], /* CJK Ideograph */
  ["\u5439", "\u5439"], /* CJK Ideograph */
  ["\u543A", "\u543A"], /* CJK Ideograph */
  ["\u543B", "\u543B"], /* CJK Ideograph */
  ["\u543C", "\u543C"], /* CJK Ideograph */
  ["\u543D", "\u543D"], /* CJK Ideograph */
  ["\u543E", "\u543E"], /* CJK Ideograph */
  ["\u543F", "\u543F"], /* CJK Ideograph */
  ["\u5440", "\u5440"], /* CJK Ideograph */
  ["\u5441", "\u5441"], /* CJK Ideograph */
  ["\u5442", "\u5442"], /* CJK Ideograph */
  ["\u5443", "\u5443"], /* CJK Ideograph */
  ["\u5444", "\u5444"], /* CJK Ideograph */
  ["\u5445", "\u5445"], /* CJK Ideograph */
  ["\u5446", "\u5446"], /* CJK Ideograph */
  ["\u5447", "\u5447"], /* CJK Ideograph */
  ["\u5448", "\u5448"], /* CJK Ideograph */
  ["\u5449", "\u5449"], /* CJK Ideograph */
  ["\u544A", "\u544A"], /* CJK Ideograph */
  ["\u544B", "\u544B"], /* CJK Ideograph */
  ["\u544C", "\u544C"], /* CJK Ideograph */
  ["\u544D", "\u544D"], /* CJK Ideograph */
  ["\u544E", "\u544E"], /* CJK Ideograph */
  ["\u544F", "\u544F"], /* CJK Ideograph */
  ["\u5450", "\u5450"], /* CJK Ideograph */
  ["\u5451", "\u5451"], /* CJK Ideograph */
  ["\u5452", "\u5452"], /* CJK Ideograph */
  ["\u5453", "\u5453"], /* CJK Ideograph */
  ["\u5454", "\u5454"], /* CJK Ideograph */
  ["\u5455", "\u5455"], /* CJK Ideograph */
  ["\u5456", "\u5456"], /* CJK Ideograph */
  ["\u5457", "\u5457"], /* CJK Ideograph */
  ["\u5458", "\u5458"], /* CJK Ideograph */
  ["\u5459", "\u5459"], /* CJK Ideograph */
  ["\u545A", "\u545A"], /* CJK Ideograph */
  ["\u545B", "\u545B"], /* CJK Ideograph */
  ["\u545C", "\u545C"], /* CJK Ideograph */
  ["\u545D", "\u545D"], /* CJK Ideograph */
  ["\u545E", "\u545E"], /* CJK Ideograph */
  ["\u545F", "\u545F"], /* CJK Ideograph */
  ["\u5460", "\u5460"], /* CJK Ideograph */
  ["\u5461", "\u5461"], /* CJK Ideograph */
  ["\u5462", "\u5462"], /* CJK Ideograph */
  ["\u5463", "\u5463"], /* CJK Ideograph */
  ["\u5464", "\u5464"], /* CJK Ideograph */
  ["\u5465", "\u5465"], /* CJK Ideograph */
  ["\u5466", "\u5466"], /* CJK Ideograph */
  ["\u5467", "\u5467"], /* CJK Ideograph */
  ["\u5468", "\u5468"], /* CJK Ideograph */
  ["\u5469", "\u5469"], /* CJK Ideograph */
  ["\u546A", "\u546A"], /* CJK Ideograph */
  ["\u546B", "\u546B"], /* CJK Ideograph */
  ["\u546C", "\u546C"], /* CJK Ideograph */
  ["\u546D", "\u546D"], /* CJK Ideograph */
  ["\u546E", "\u546E"], /* CJK Ideograph */
  ["\u546F", "\u546F"], /* CJK Ideograph */
  ["\u5470", "\u5470"], /* CJK Ideograph */
  ["\u5471", "\u5471"], /* CJK Ideograph */
  ["\u5472", "\u5472"], /* CJK Ideograph */
  ["\u5473", "\u5473"], /* CJK Ideograph */
  ["\u5474", "\u5474"], /* CJK Ideograph */
  ["\u5475", "\u5475"], /* CJK Ideograph */
  ["\u5476", "\u5476"], /* CJK Ideograph */
  ["\u5477", "\u5477"], /* CJK Ideograph */
  ["\u5478", "\u5478"], /* CJK Ideograph */
  ["\u5479", "\u5479"], /* CJK Ideograph */
  ["\u547A", "\u547A"], /* CJK Ideograph */
  ["\u547B", "\u547B"], /* CJK Ideograph */
  ["\u547C", "\u547C"], /* CJK Ideograph */
  ["\u547D", "\u547D"], /* CJK Ideograph */
  ["\u547E", "\u547E"], /* CJK Ideograph */
  ["\u547F", "\u547F"], /* CJK Ideograph */
  ["\u5480", "\u5480"], /* CJK Ideograph */
  ["\u5481", "\u5481"], /* CJK Ideograph */
  ["\u5482", "\u5482"], /* CJK Ideograph */
  ["\u5483", "\u5483"], /* CJK Ideograph */
  ["\u5484", "\u5484"], /* CJK Ideograph */
  ["\u5485", "\u5485"], /* CJK Ideograph */
  ["\u5486", "\u5486"], /* CJK Ideograph */
  ["\u5487", "\u5487"], /* CJK Ideograph */
  ["\u5488", "\u5488"], /* CJK Ideograph */
  ["\u5489", "\u5489"], /* CJK Ideograph */
  ["\u548A", "\u548A"], /* CJK Ideograph */
  ["\u548B", "\u548B"], /* CJK Ideograph */
  ["\u548C", "\u548C"], /* CJK Ideograph */
  ["\u548D", "\u548D"], /* CJK Ideograph */
  ["\u548E", "\u548E"], /* CJK Ideograph */
  ["\u548F", "\u548F"], /* CJK Ideograph */
  ["\u5490", "\u5490"], /* CJK Ideograph */
  ["\u5491", "\u5491"], /* CJK Ideograph */
  ["\u5492", "\u5492"], /* CJK Ideograph */
  ["\u5493", "\u5493"], /* CJK Ideograph */
  ["\u5494", "\u5494"], /* CJK Ideograph */
  ["\u5495", "\u5495"], /* CJK Ideograph */
  ["\u5496", "\u5496"], /* CJK Ideograph */
  ["\u5497", "\u5497"], /* CJK Ideograph */
  ["\u5498", "\u5498"], /* CJK Ideograph */
  ["\u5499", "\u5499"], /* CJK Ideograph */
  ["\u549A", "\u549A"], /* CJK Ideograph */
  ["\u549B", "\u549B"], /* CJK Ideograph */
  ["\u549C", "\u549C"], /* CJK Ideograph */
  ["\u549D", "\u549D"], /* CJK Ideograph */
  ["\u549E", "\u549E"], /* CJK Ideograph */
  ["\u549F", "\u549F"], /* CJK Ideograph */
  ["\u54A0", "\u54A0"], /* CJK Ideograph */
  ["\u54A1", "\u54A1"], /* CJK Ideograph */
  ["\u54A2", "\u54A2"], /* CJK Ideograph */
  ["\u54A3", "\u54A3"], /* CJK Ideograph */
  ["\u54A4", "\u54A4"], /* CJK Ideograph */
  ["\u54A5", "\u54A5"], /* CJK Ideograph */
  ["\u54A6", "\u54A6"], /* CJK Ideograph */
  ["\u54A7", "\u54A7"], /* CJK Ideograph */
  ["\u54A8", "\u54A8"], /* CJK Ideograph */
  ["\u54A9", "\u54A9"], /* CJK Ideograph */
  ["\u54AA", "\u54AA"], /* CJK Ideograph */
  ["\u54AB", "\u54AB"], /* CJK Ideograph */
  ["\u54AC", "\u54AC"], /* CJK Ideograph */
  ["\u54AD", "\u54AD"], /* CJK Ideograph */
  ["\u54AE", "\u54AE"], /* CJK Ideograph */
  ["\u54AF", "\u54AF"], /* CJK Ideograph */
  ["\u54B0", "\u54B0"], /* CJK Ideograph */
  ["\u54B1", "\u54B1"], /* CJK Ideograph */
  ["\u54B2", "\u54B2"], /* CJK Ideograph */
  ["\u54B3", "\u54B3"], /* CJK Ideograph */
  ["\u54B4", "\u54B4"], /* CJK Ideograph */
  ["\u54B5", "\u54B5"], /* CJK Ideograph */
  ["\u54B6", "\u54B6"], /* CJK Ideograph */
  ["\u54B7", "\u54B7"], /* CJK Ideograph */
  ["\u54B8", "\u54B8"], /* CJK Ideograph */
  ["\u54B9", "\u54B9"], /* CJK Ideograph */
  ["\u54BA", "\u54BA"], /* CJK Ideograph */
  ["\u54BB", "\u54BB"], /* CJK Ideograph */
  ["\u54BC", "\u54BC"], /* CJK Ideograph */
  ["\u54BD", "\u54BD"], /* CJK Ideograph */
  ["\u54BE", "\u54BE"], /* CJK Ideograph */
  ["\u54BF", "\u54BF"], /* CJK Ideograph */
  ["\u54C0", "\u54C0"], /* CJK Ideograph */
  ["\u54C1", "\u54C1"], /* CJK Ideograph */
  ["\u54C2", "\u54C2"], /* CJK Ideograph */
  ["\u54C3", "\u54C3"], /* CJK Ideograph */
  ["\u54C4", "\u54C4"], /* CJK Ideograph */
  ["\u54C5", "\u54C5"], /* CJK Ideograph */
  ["\u54C6", "\u54C6"], /* CJK Ideograph */
  ["\u54C7", "\u54C7"], /* CJK Ideograph */
  ["\u54C8", "\u54C8"], /* CJK Ideograph */
  ["\u54C9", "\u54C9"], /* CJK Ideograph */
  ["\u54CA", "\u54CA"], /* CJK Ideograph */
  ["\u54CB", "\u54CB"], /* CJK Ideograph */
  ["\u54CC", "\u54CC"], /* CJK Ideograph */
  ["\u54CD", "\u54CD"], /* CJK Ideograph */
  ["\u54CE", "\u54CE"], /* CJK Ideograph */
  ["\u54CF", "\u54CF"], /* CJK Ideograph */
  ["\u54D0", "\u54D0"], /* CJK Ideograph */
  ["\u54D1", "\u54D1"], /* CJK Ideograph */
  ["\u54D2", "\u54D2"], /* CJK Ideograph */
  ["\u54D3", "\u54D3"], /* CJK Ideograph */
  ["\u54D4", "\u54D4"], /* CJK Ideograph */
  ["\u54D5", "\u54D5"], /* CJK Ideograph */
  ["\u54D6", "\u54D6"], /* CJK Ideograph */
  ["\u54D7", "\u54D7"], /* CJK Ideograph */
  ["\u54D8", "\u54D8"], /* CJK Ideograph */
  ["\u54D9", "\u54D9"], /* CJK Ideograph */
  ["\u54DA", "\u54DA"], /* CJK Ideograph */
  ["\u54DB", "\u54DB"], /* CJK Ideograph */
  ["\u54DC", "\u54DC"], /* CJK Ideograph */
  ["\u54DD", "\u54DD"], /* CJK Ideograph */
  ["\u54DE", "\u54DE"], /* CJK Ideograph */
  ["\u54DF", "\u54DF"], /* CJK Ideograph */
  ["\u54E0", "\u54E0"], /* CJK Ideograph */
  ["\u54E1", "\u54E1"], /* CJK Ideograph */
  ["\u54E2", "\u54E2"], /* CJK Ideograph */
  ["\u54E3", "\u54E3"], /* CJK Ideograph */
  ["\u54E4", "\u54E4"], /* CJK Ideograph */
  ["\u54E5", "\u54E5"], /* CJK Ideograph */
  ["\u54E6", "\u54E6"], /* CJK Ideograph */
  ["\u54E7", "\u54E7"], /* CJK Ideograph */
  ["\u54E8", "\u54E8"], /* CJK Ideograph */
  ["\u54E9", "\u54E9"], /* CJK Ideograph */
  ["\u54EA", "\u54EA"], /* CJK Ideograph */
  ["\u54EB", "\u54EB"], /* CJK Ideograph */
  ["\u54EC", "\u54EC"], /* CJK Ideograph */
  ["\u54ED", "\u54ED"], /* CJK Ideograph */
  ["\u54EE", "\u54EE"], /* CJK Ideograph */
  ["\u54EF", "\u54EF"], /* CJK Ideograph */
  ["\u54F0", "\u54F0"], /* CJK Ideograph */
  ["\u54F1", "\u54F1"], /* CJK Ideograph */
  ["\u54F2", "\u54F2"], /* CJK Ideograph */
  ["\u54F3", "\u54F3"], /* CJK Ideograph */
  ["\u54F4", "\u54F4"], /* CJK Ideograph */
  ["\u54F5", "\u54F5"], /* CJK Ideograph */
  ["\u54F6", "\u54F6"], /* CJK Ideograph */
  ["\u54F7", "\u54F7"], /* CJK Ideograph */
  ["\u54F8", "\u54F8"], /* CJK Ideograph */
  ["\u54F9", "\u54F9"], /* CJK Ideograph */
  ["\u54FA", "\u54FA"], /* CJK Ideograph */
  ["\u54FB", "\u54FB"], /* CJK Ideograph */
  ["\u54FC", "\u54FC"], /* CJK Ideograph */
  ["\u54FD", "\u54FD"], /* CJK Ideograph */
  ["\u54FE", "\u54FE"], /* CJK Ideograph */
  ["\u54FF", "\u54FF"], /* CJK Ideograph */
  ["\u5500", "\u5500"], /* CJK Ideograph */
  ["\u5501", "\u5501"], /* CJK Ideograph */
  ["\u5502", "\u5502"], /* CJK Ideograph */
  ["\u5503", "\u5503"], /* CJK Ideograph */
  ["\u5504", "\u5504"], /* CJK Ideograph */
  ["\u5505", "\u5505"], /* CJK Ideograph */
  ["\u5506", "\u5506"], /* CJK Ideograph */
  ["\u5507", "\u5507"], /* CJK Ideograph */
  ["\u5508", "\u5508"], /* CJK Ideograph */
  ["\u5509", "\u5509"], /* CJK Ideograph */
  ["\u550A", "\u550A"], /* CJK Ideograph */
  ["\u550B", "\u550B"], /* CJK Ideograph */
  ["\u550C", "\u550C"], /* CJK Ideograph */
  ["\u550D", "\u550D"], /* CJK Ideograph */
  ["\u550E", "\u550E"], /* CJK Ideograph */
  ["\u550F", "\u550F"], /* CJK Ideograph */
  ["\u5510", "\u5510"], /* CJK Ideograph */
  ["\u5511", "\u5511"], /* CJK Ideograph */
  ["\u5512", "\u5512"], /* CJK Ideograph */
  ["\u5513", "\u5513"], /* CJK Ideograph */
  ["\u5514", "\u5514"], /* CJK Ideograph */
  ["\u5515", "\u5515"], /* CJK Ideograph */
  ["\u5516", "\u5516"], /* CJK Ideograph */
  ["\u5517", "\u5517"], /* CJK Ideograph */
  ["\u5518", "\u5518"], /* CJK Ideograph */
  ["\u5519", "\u5519"], /* CJK Ideograph */
  ["\u551A", "\u551A"], /* CJK Ideograph */
  ["\u551B", "\u551B"], /* CJK Ideograph */
  ["\u551C", "\u551C"], /* CJK Ideograph */
  ["\u551D", "\u551D"], /* CJK Ideograph */
  ["\u551E", "\u551E"], /* CJK Ideograph */
  ["\u551F", "\u551F"], /* CJK Ideograph */
  ["\u5520", "\u5520"], /* CJK Ideograph */
  ["\u5521", "\u5521"], /* CJK Ideograph */
  ["\u5522", "\u5522"], /* CJK Ideograph */
  ["\u5523", "\u5523"], /* CJK Ideograph */
  ["\u5524", "\u5524"], /* CJK Ideograph */
  ["\u5525", "\u5525"], /* CJK Ideograph */
  ["\u5526", "\u5526"], /* CJK Ideograph */
  ["\u5527", "\u5527"], /* CJK Ideograph */
  ["\u5528", "\u5528"], /* CJK Ideograph */
  ["\u5529", "\u5529"], /* CJK Ideograph */
  ["\u552A", "\u552A"], /* CJK Ideograph */
  ["\u552B", "\u552B"], /* CJK Ideograph */
  ["\u552C", "\u552C"], /* CJK Ideograph */
  ["\u552D", "\u552D"], /* CJK Ideograph */
  ["\u552E", "\u552E"], /* CJK Ideograph */
  ["\u552F", "\u552F"], /* CJK Ideograph */
  ["\u5530", "\u5530"], /* CJK Ideograph */
  ["\u5531", "\u5531"], /* CJK Ideograph */
  ["\u5532", "\u5532"], /* CJK Ideograph */
  ["\u5533", "\u5533"], /* CJK Ideograph */
  ["\u5534", "\u5534"], /* CJK Ideograph */
  ["\u5535", "\u5535"], /* CJK Ideograph */
  ["\u5536", "\u5536"], /* CJK Ideograph */
  ["\u5537", "\u5537"], /* CJK Ideograph */
  ["\u5538", "\u5538"], /* CJK Ideograph */
  ["\u5539", "\u5539"], /* CJK Ideograph */
  ["\u553A", "\u553A"], /* CJK Ideograph */
  ["\u553B", "\u553B"], /* CJK Ideograph */
  ["\u553C", "\u553C"], /* CJK Ideograph */
  ["\u553D", "\u553D"], /* CJK Ideograph */
  ["\u553E", "\u553E"], /* CJK Ideograph */
  ["\u553F", "\u553F"], /* CJK Ideograph */
  ["\u5540", "\u5540"], /* CJK Ideograph */
  ["\u5541", "\u5541"], /* CJK Ideograph */
  ["\u5542", "\u5542"], /* CJK Ideograph */
  ["\u5543", "\u5543"], /* CJK Ideograph */
  ["\u5544", "\u5544"], /* CJK Ideograph */
  ["\u5545", "\u5545"], /* CJK Ideograph */
  ["\u5546", "\u5546"], /* CJK Ideograph */
  ["\u5547", "\u5547"], /* CJK Ideograph */
  ["\u5548", "\u5548"], /* CJK Ideograph */
  ["\u5549", "\u5549"], /* CJK Ideograph */
  ["\u554A", "\u554A"], /* CJK Ideograph */
  ["\u554B", "\u554B"], /* CJK Ideograph */
  ["\u554C", "\u554C"], /* CJK Ideograph */
  ["\u554D", "\u554D"], /* CJK Ideograph */
  ["\u554E", "\u554E"], /* CJK Ideograph */
  ["\u554F", "\u554F"], /* CJK Ideograph */
  ["\u5550", "\u5550"], /* CJK Ideograph */
  ["\u5551", "\u5551"], /* CJK Ideograph */
  ["\u5552", "\u5552"], /* CJK Ideograph */
  ["\u5553", "\u5553"], /* CJK Ideograph */
  ["\u5554", "\u5554"], /* CJK Ideograph */
  ["\u5555", "\u5555"], /* CJK Ideograph */
  ["\u5556", "\u5556"], /* CJK Ideograph */
  ["\u5557", "\u5557"], /* CJK Ideograph */
  ["\u5558", "\u5558"], /* CJK Ideograph */
  ["\u5559", "\u5559"], /* CJK Ideograph */
  ["\u555A", "\u555A"], /* CJK Ideograph */
  ["\u555B", "\u555B"], /* CJK Ideograph */
  ["\u555C", "\u555C"], /* CJK Ideograph */
  ["\u555D", "\u555D"], /* CJK Ideograph */
  ["\u555E", "\u555E"], /* CJK Ideograph */
  ["\u555F", "\u555F"], /* CJK Ideograph */
  ["\u5560", "\u5560"], /* CJK Ideograph */
  ["\u5561", "\u5561"], /* CJK Ideograph */
  ["\u5562", "\u5562"], /* CJK Ideograph */
  ["\u5563", "\u5563"], /* CJK Ideograph */
  ["\u5564", "\u5564"], /* CJK Ideograph */
  ["\u5565", "\u5565"], /* CJK Ideograph */
  ["\u5566", "\u5566"], /* CJK Ideograph */
  ["\u5567", "\u5567"], /* CJK Ideograph */
  ["\u5568", "\u5568"], /* CJK Ideograph */
  ["\u5569", "\u5569"], /* CJK Ideograph */
  ["\u556A", "\u556A"], /* CJK Ideograph */
  ["\u556B", "\u556B"], /* CJK Ideograph */
  ["\u556C", "\u556C"], /* CJK Ideograph */
  ["\u556D", "\u556D"], /* CJK Ideograph */
  ["\u556E", "\u556E"], /* CJK Ideograph */
  ["\u556F", "\u556F"], /* CJK Ideograph */
  ["\u5570", "\u5570"], /* CJK Ideograph */
  ["\u5571", "\u5571"], /* CJK Ideograph */
  ["\u5572", "\u5572"], /* CJK Ideograph */
  ["\u5573", "\u5573"], /* CJK Ideograph */
  ["\u5574", "\u5574"], /* CJK Ideograph */
  ["\u5575", "\u5575"], /* CJK Ideograph */
  ["\u5576", "\u5576"], /* CJK Ideograph */
  ["\u5577", "\u5577"], /* CJK Ideograph */
  ["\u5578", "\u5578"], /* CJK Ideograph */
  ["\u5579", "\u5579"], /* CJK Ideograph */
  ["\u557A", "\u557A"], /* CJK Ideograph */
  ["\u557B", "\u557B"], /* CJK Ideograph */
  ["\u557C", "\u557C"], /* CJK Ideograph */
  ["\u557D", "\u557D"], /* CJK Ideograph */
  ["\u557E", "\u557E"], /* CJK Ideograph */
  ["\u557F", "\u557F"], /* CJK Ideograph */
  ["\u5580", "\u5580"], /* CJK Ideograph */
  ["\u5581", "\u5581"], /* CJK Ideograph */
  ["\u5582", "\u5582"], /* CJK Ideograph */
  ["\u5583", "\u5583"], /* CJK Ideograph */
  ["\u5584", "\u5584"], /* CJK Ideograph */
  ["\u5585", "\u5585"], /* CJK Ideograph */
  ["\u5586", "\u5586"], /* CJK Ideograph */
  ["\u5587", "\u5587"], /* CJK Ideograph */
  ["\u5588", "\u5588"], /* CJK Ideograph */
  ["\u5589", "\u5589"], /* CJK Ideograph */
  ["\u558A", "\u558A"], /* CJK Ideograph */
  ["\u558B", "\u558B"], /* CJK Ideograph */
  ["\u558C", "\u558C"], /* CJK Ideograph */
  ["\u558D", "\u558D"], /* CJK Ideograph */
  ["\u558E", "\u558E"], /* CJK Ideograph */
  ["\u558F", "\u558F"], /* CJK Ideograph */
  ["\u5590", "\u5590"], /* CJK Ideograph */
  ["\u5591", "\u5591"], /* CJK Ideograph */
  ["\u5592", "\u5592"], /* CJK Ideograph */
  ["\u5593", "\u5593"], /* CJK Ideograph */
  ["\u5594", "\u5594"], /* CJK Ideograph */
  ["\u5595", "\u5595"], /* CJK Ideograph */
  ["\u5596", "\u5596"], /* CJK Ideograph */
  ["\u5597", "\u5597"], /* CJK Ideograph */
  ["\u5598", "\u5598"], /* CJK Ideograph */
  ["\u5599", "\u5599"], /* CJK Ideograph */
  ["\u559A", "\u559A"], /* CJK Ideograph */
  ["\u559B", "\u559B"], /* CJK Ideograph */
  ["\u559C", "\u559C"], /* CJK Ideograph */
  ["\u559D", "\u559D"], /* CJK Ideograph */
  ["\u559E", "\u559E"], /* CJK Ideograph */
  ["\u559F", "\u559F"], /* CJK Ideograph */
  ["\u55A0", "\u55A0"], /* CJK Ideograph */
  ["\u55A1", "\u55A1"], /* CJK Ideograph */
  ["\u55A2", "\u55A2"], /* CJK Ideograph */
  ["\u55A3", "\u55A3"], /* CJK Ideograph */
  ["\u55A4", "\u55A4"], /* CJK Ideograph */
  ["\u55A5", "\u55A5"], /* CJK Ideograph */
  ["\u55A6", "\u55A6"], /* CJK Ideograph */
  ["\u55A7", "\u55A7"], /* CJK Ideograph */
  ["\u55A8", "\u55A8"], /* CJK Ideograph */
  ["\u55A9", "\u55A9"], /* CJK Ideograph */
  ["\u55AA", "\u55AA"], /* CJK Ideograph */
  ["\u55AB", "\u55AB"], /* CJK Ideograph */
  ["\u55AC", "\u55AC"], /* CJK Ideograph */
  ["\u55AD", "\u55AD"], /* CJK Ideograph */
  ["\u55AE", "\u55AE"], /* CJK Ideograph */
  ["\u55AF", "\u55AF"], /* CJK Ideograph */
  ["\u55B0", "\u55B0"], /* CJK Ideograph */
  ["\u55B1", "\u55B1"], /* CJK Ideograph */
  ["\u55B2", "\u55B2"], /* CJK Ideograph */
  ["\u55B3", "\u55B3"], /* CJK Ideograph */
  ["\u55B4", "\u55B4"], /* CJK Ideograph */
  ["\u55B5", "\u55B5"], /* CJK Ideograph */
  ["\u55B6", "\u55B6"], /* CJK Ideograph */
  ["\u55B7", "\u55B7"], /* CJK Ideograph */
  ["\u55B8", "\u55B8"], /* CJK Ideograph */
  ["\u55B9", "\u55B9"], /* CJK Ideograph */
  ["\u55BA", "\u55BA"], /* CJK Ideograph */
  ["\u55BB", "\u55BB"], /* CJK Ideograph */
  ["\u55BC", "\u55BC"], /* CJK Ideograph */
  ["\u55BD", "\u55BD"], /* CJK Ideograph */
  ["\u55BE", "\u55BE"], /* CJK Ideograph */
  ["\u55BF", "\u55BF"], /* CJK Ideograph */
  ["\u55C0", "\u55C0"], /* CJK Ideograph */
  ["\u55C1", "\u55C1"], /* CJK Ideograph */
  ["\u55C2", "\u55C2"], /* CJK Ideograph */
  ["\u55C3", "\u55C3"], /* CJK Ideograph */
  ["\u55C4", "\u55C4"], /* CJK Ideograph */
  ["\u55C5", "\u55C5"], /* CJK Ideograph */
  ["\u55C6", "\u55C6"], /* CJK Ideograph */
  ["\u55C7", "\u55C7"], /* CJK Ideograph */
  ["\u55C8", "\u55C8"], /* CJK Ideograph */
  ["\u55C9", "\u55C9"], /* CJK Ideograph */
  ["\u55CA", "\u55CA"], /* CJK Ideograph */
  ["\u55CB", "\u55CB"], /* CJK Ideograph */
  ["\u55CC", "\u55CC"], /* CJK Ideograph */
  ["\u55CD", "\u55CD"], /* CJK Ideograph */
  ["\u55CE", "\u55CE"], /* CJK Ideograph */
  ["\u55CF", "\u55CF"], /* CJK Ideograph */
  ["\u55D0", "\u55D0"], /* CJK Ideograph */
  ["\u55D1", "\u55D1"], /* CJK Ideograph */
  ["\u55D2", "\u55D2"], /* CJK Ideograph */
  ["\u55D3", "\u55D3"], /* CJK Ideograph */
  ["\u55D4", "\u55D4"], /* CJK Ideograph */
  ["\u55D5", "\u55D5"], /* CJK Ideograph */
  ["\u55D6", "\u55D6"], /* CJK Ideograph */
  ["\u55D7", "\u55D7"], /* CJK Ideograph */
  ["\u55D8", "\u55D8"], /* CJK Ideograph */
  ["\u55D9", "\u55D9"], /* CJK Ideograph */
  ["\u55DA", "\u55DA"], /* CJK Ideograph */
  ["\u55DB", "\u55DB"], /* CJK Ideograph */
  ["\u55DC", "\u55DC"], /* CJK Ideograph */
  ["\u55DD", "\u55DD"], /* CJK Ideograph */
  ["\u55DE", "\u55DE"], /* CJK Ideograph */
  ["\u55DF", "\u55DF"], /* CJK Ideograph */
  ["\u55E0", "\u55E0"], /* CJK Ideograph */
  ["\u55E1", "\u55E1"], /* CJK Ideograph */
  ["\u55E2", "\u55E2"], /* CJK Ideograph */
  ["\u55E3", "\u55E3"], /* CJK Ideograph */
  ["\u55E4", "\u55E4"], /* CJK Ideograph */
  ["\u55E5", "\u55E5"], /* CJK Ideograph */
  ["\u55E6", "\u55E6"], /* CJK Ideograph */
  ["\u55E7", "\u55E7"], /* CJK Ideograph */
  ["\u55E8", "\u55E8"], /* CJK Ideograph */
  ["\u55E9", "\u55E9"], /* CJK Ideograph */
  ["\u55EA", "\u55EA"], /* CJK Ideograph */
  ["\u55EB", "\u55EB"], /* CJK Ideograph */
  ["\u55EC", "\u55EC"], /* CJK Ideograph */
  ["\u55ED", "\u55ED"], /* CJK Ideograph */
  ["\u55EE", "\u55EE"], /* CJK Ideograph */
  ["\u55EF", "\u55EF"], /* CJK Ideograph */
  ["\u55F0", "\u55F0"], /* CJK Ideograph */
  ["\u55F1", "\u55F1"], /* CJK Ideograph */
  ["\u55F2", "\u55F2"], /* CJK Ideograph */
  ["\u55F3", "\u55F3"], /* CJK Ideograph */
  ["\u55F4", "\u55F4"], /* CJK Ideograph */
  ["\u55F5", "\u55F5"], /* CJK Ideograph */
  ["\u55F6", "\u55F6"], /* CJK Ideograph */
  ["\u55F7", "\u55F7"], /* CJK Ideograph */
  ["\u55F8", "\u55F8"], /* CJK Ideograph */
  ["\u55F9", "\u55F9"], /* CJK Ideograph */
  ["\u55FA", "\u55FA"], /* CJK Ideograph */
  ["\u55FB", "\u55FB"], /* CJK Ideograph */
  ["\u55FC", "\u55FC"], /* CJK Ideograph */
  ["\u55FD", "\u55FD"], /* CJK Ideograph */
  ["\u55FE", "\u55FE"], /* CJK Ideograph */
  ["\u55FF", "\u55FF"], /* CJK Ideograph */
  ["\u5600", "\u5600"], /* CJK Ideograph */
  ["\u5601", "\u5601"], /* CJK Ideograph */
  ["\u5602", "\u5602"], /* CJK Ideograph */
  ["\u5603", "\u5603"], /* CJK Ideograph */
  ["\u5604", "\u5604"], /* CJK Ideograph */
  ["\u5605", "\u5605"], /* CJK Ideograph */
  ["\u5606", "\u5606"], /* CJK Ideograph */
  ["\u5607", "\u5607"], /* CJK Ideograph */
  ["\u5608", "\u5608"], /* CJK Ideograph */
  ["\u5609", "\u5609"], /* CJK Ideograph */
  ["\u560A", "\u560A"], /* CJK Ideograph */
  ["\u560B", "\u560B"], /* CJK Ideograph */
  ["\u560C", "\u560C"], /* CJK Ideograph */
  ["\u560D", "\u560D"], /* CJK Ideograph */
  ["\u560E", "\u560E"], /* CJK Ideograph */
  ["\u560F", "\u560F"], /* CJK Ideograph */
  ["\u5610", "\u5610"], /* CJK Ideograph */
  ["\u5611", "\u5611"], /* CJK Ideograph */
  ["\u5612", "\u5612"], /* CJK Ideograph */
  ["\u5613", "\u5613"], /* CJK Ideograph */
  ["\u5614", "\u5614"], /* CJK Ideograph */
  ["\u5615", "\u5615"], /* CJK Ideograph */
  ["\u5616", "\u5616"], /* CJK Ideograph */
  ["\u5617", "\u5617"], /* CJK Ideograph */
  ["\u5618", "\u5618"], /* CJK Ideograph */
  ["\u5619", "\u5619"], /* CJK Ideograph */
  ["\u561A", "\u561A"], /* CJK Ideograph */
  ["\u561B", "\u561B"], /* CJK Ideograph */
  ["\u561C", "\u561C"], /* CJK Ideograph */
  ["\u561D", "\u561D"], /* CJK Ideograph */
  ["\u561E", "\u561E"], /* CJK Ideograph */
  ["\u561F", "\u561F"], /* CJK Ideograph */
  ["\u5620", "\u5620"], /* CJK Ideograph */
  ["\u5621", "\u5621"], /* CJK Ideograph */
  ["\u5622", "\u5622"], /* CJK Ideograph */
  ["\u5623", "\u5623"], /* CJK Ideograph */
  ["\u5624", "\u5624"], /* CJK Ideograph */
  ["\u5625", "\u5625"], /* CJK Ideograph */
  ["\u5626", "\u5626"], /* CJK Ideograph */
  ["\u5627", "\u5627"], /* CJK Ideograph */
  ["\u5628", "\u5628"], /* CJK Ideograph */
  ["\u5629", "\u5629"], /* CJK Ideograph */
  ["\u562A", "\u562A"], /* CJK Ideograph */
  ["\u562B", "\u562B"], /* CJK Ideograph */
  ["\u562C", "\u562C"], /* CJK Ideograph */
  ["\u562D", "\u562D"], /* CJK Ideograph */
  ["\u562E", "\u562E"], /* CJK Ideograph */
  ["\u562F", "\u562F"], /* CJK Ideograph */
  ["\u5630", "\u5630"], /* CJK Ideograph */
  ["\u5631", "\u5631"], /* CJK Ideograph */
  ["\u5632", "\u5632"], /* CJK Ideograph */
  ["\u5633", "\u5633"], /* CJK Ideograph */
  ["\u5634", "\u5634"], /* CJK Ideograph */
  ["\u5635", "\u5635"], /* CJK Ideograph */
  ["\u5636", "\u5636"], /* CJK Ideograph */
  ["\u5637", "\u5637"], /* CJK Ideograph */
  ["\u5638", "\u5638"], /* CJK Ideograph */
  ["\u5639", "\u5639"], /* CJK Ideograph */
  ["\u563A", "\u563A"], /* CJK Ideograph */
  ["\u563B", "\u563B"], /* CJK Ideograph */
  ["\u563C", "\u563C"], /* CJK Ideograph */
  ["\u563D", "\u563D"], /* CJK Ideograph */
  ["\u563E", "\u563E"], /* CJK Ideograph */
  ["\u563F", "\u563F"], /* CJK Ideograph */
  ["\u5640", "\u5640"], /* CJK Ideograph */
  ["\u5641", "\u5641"], /* CJK Ideograph */
  ["\u5642", "\u5642"], /* CJK Ideograph */
  ["\u5643", "\u5643"], /* CJK Ideograph */
  ["\u5644", "\u5644"], /* CJK Ideograph */
  ["\u5645", "\u5645"], /* CJK Ideograph */
  ["\u5646", "\u5646"], /* CJK Ideograph */
  ["\u5647", "\u5647"], /* CJK Ideograph */
  ["\u5648", "\u5648"], /* CJK Ideograph */
  ["\u5649", "\u5649"], /* CJK Ideograph */
  ["\u564A", "\u564A"], /* CJK Ideograph */
  ["\u564B", "\u564B"], /* CJK Ideograph */
  ["\u564C", "\u564C"], /* CJK Ideograph */
  ["\u564D", "\u564D"], /* CJK Ideograph */
  ["\u564E", "\u564E"], /* CJK Ideograph */
  ["\u564F", "\u564F"], /* CJK Ideograph */
  ["\u5650", "\u5650"], /* CJK Ideograph */
  ["\u5651", "\u5651"], /* CJK Ideograph */
  ["\u5652", "\u5652"], /* CJK Ideograph */
  ["\u5653", "\u5653"], /* CJK Ideograph */
  ["\u5654", "\u5654"], /* CJK Ideograph */
  ["\u5655", "\u5655"], /* CJK Ideograph */
  ["\u5656", "\u5656"], /* CJK Ideograph */
  ["\u5657", "\u5657"], /* CJK Ideograph */
  ["\u5658", "\u5658"], /* CJK Ideograph */
  ["\u5659", "\u5659"], /* CJK Ideograph */
  ["\u565A", "\u565A"], /* CJK Ideograph */
  ["\u565B", "\u565B"], /* CJK Ideograph */
  ["\u565C", "\u565C"], /* CJK Ideograph */
  ["\u565D", "\u565D"], /* CJK Ideograph */
  ["\u565E", "\u565E"], /* CJK Ideograph */
  ["\u565F", "\u565F"], /* CJK Ideograph */
  ["\u5660", "\u5660"], /* CJK Ideograph */
  ["\u5661", "\u5661"], /* CJK Ideograph */
  ["\u5662", "\u5662"], /* CJK Ideograph */
  ["\u5663", "\u5663"], /* CJK Ideograph */
  ["\u5664", "\u5664"], /* CJK Ideograph */
  ["\u5665", "\u5665"], /* CJK Ideograph */
  ["\u5666", "\u5666"], /* CJK Ideograph */
  ["\u5667", "\u5667"], /* CJK Ideograph */
  ["\u5668", "\u5668"], /* CJK Ideograph */
  ["\u5669", "\u5669"], /* CJK Ideograph */
  ["\u566A", "\u566A"], /* CJK Ideograph */
  ["\u566B", "\u566B"], /* CJK Ideograph */
  ["\u566C", "\u566C"], /* CJK Ideograph */
  ["\u566D", "\u566D"], /* CJK Ideograph */
  ["\u566E", "\u566E"], /* CJK Ideograph */
  ["\u566F", "\u566F"], /* CJK Ideograph */
  ["\u5670", "\u5670"], /* CJK Ideograph */
  ["\u5671", "\u5671"], /* CJK Ideograph */
  ["\u5672", "\u5672"], /* CJK Ideograph */
  ["\u5673", "\u5673"], /* CJK Ideograph */
  ["\u5674", "\u5674"], /* CJK Ideograph */
  ["\u5675", "\u5675"], /* CJK Ideograph */
  ["\u5676", "\u5676"], /* CJK Ideograph */
  ["\u5677", "\u5677"], /* CJK Ideograph */
  ["\u5678", "\u5678"], /* CJK Ideograph */
  ["\u5679", "\u5679"], /* CJK Ideograph */
  ["\u567A", "\u567A"], /* CJK Ideograph */
  ["\u567B", "\u567B"], /* CJK Ideograph */
  ["\u567C", "\u567C"], /* CJK Ideograph */
  ["\u567D", "\u567D"], /* CJK Ideograph */
  ["\u567E", "\u567E"], /* CJK Ideograph */
  ["\u567F", "\u567F"], /* CJK Ideograph */
  ["\u5680", "\u5680"], /* CJK Ideograph */
  ["\u5681", "\u5681"], /* CJK Ideograph */
  ["\u5682", "\u5682"], /* CJK Ideograph */
  ["\u5683", "\u5683"], /* CJK Ideograph */
  ["\u5684", "\u5684"], /* CJK Ideograph */
  ["\u5685", "\u5685"], /* CJK Ideograph */
  ["\u5686", "\u5686"], /* CJK Ideograph */
  ["\u5687", "\u5687"], /* CJK Ideograph */
  ["\u5688", "\u5688"], /* CJK Ideograph */
  ["\u5689", "\u5689"], /* CJK Ideograph */
  ["\u568A", "\u568A"], /* CJK Ideograph */
  ["\u568B", "\u568B"], /* CJK Ideograph */
  ["\u568C", "\u568C"], /* CJK Ideograph */
  ["\u568D", "\u568D"], /* CJK Ideograph */
  ["\u568E", "\u568E"], /* CJK Ideograph */
  ["\u568F", "\u568F"], /* CJK Ideograph */
  ["\u5690", "\u5690"], /* CJK Ideograph */
  ["\u5691", "\u5691"], /* CJK Ideograph */
  ["\u5692", "\u5692"], /* CJK Ideograph */
  ["\u5693", "\u5693"], /* CJK Ideograph */
  ["\u5694", "\u5694"], /* CJK Ideograph */
  ["\u5695", "\u5695"], /* CJK Ideograph */
  ["\u5696", "\u5696"], /* CJK Ideograph */
  ["\u5697", "\u5697"], /* CJK Ideograph */
  ["\u5698", "\u5698"], /* CJK Ideograph */
  ["\u5699", "\u5699"], /* CJK Ideograph */
  ["\u569A", "\u569A"], /* CJK Ideograph */
  ["\u569B", "\u569B"], /* CJK Ideograph */
  ["\u569C", "\u569C"], /* CJK Ideograph */
  ["\u569D", "\u569D"], /* CJK Ideograph */
  ["\u569E", "\u569E"], /* CJK Ideograph */
  ["\u569F", "\u569F"], /* CJK Ideograph */
  ["\u56A0", "\u56A0"], /* CJK Ideograph */
  ["\u56A1", "\u56A1"], /* CJK Ideograph */
  ["\u56A2", "\u56A2"], /* CJK Ideograph */
  ["\u56A3", "\u56A3"], /* CJK Ideograph */
  ["\u56A4", "\u56A4"], /* CJK Ideograph */
  ["\u56A5", "\u56A5"], /* CJK Ideograph */
  ["\u56A6", "\u56A6"], /* CJK Ideograph */
  ["\u56A7", "\u56A7"], /* CJK Ideograph */
  ["\u56A8", "\u56A8"], /* CJK Ideograph */
  ["\u56A9", "\u56A9"], /* CJK Ideograph */
  ["\u56AA", "\u56AA"], /* CJK Ideograph */
  ["\u56AB", "\u56AB"], /* CJK Ideograph */
  ["\u56AC", "\u56AC"], /* CJK Ideograph */
  ["\u56AD", "\u56AD"], /* CJK Ideograph */
  ["\u56AE", "\u56AE"], /* CJK Ideograph */
  ["\u56AF", "\u56AF"], /* CJK Ideograph */
  ["\u56B0", "\u56B0"], /* CJK Ideograph */
  ["\u56B1", "\u56B1"], /* CJK Ideograph */
  ["\u56B2", "\u56B2"], /* CJK Ideograph */
  ["\u56B3", "\u56B3"], /* CJK Ideograph */
  ["\u56B4", "\u56B4"], /* CJK Ideograph */
  ["\u56B5", "\u56B5"], /* CJK Ideograph */
  ["\u56B6", "\u56B6"], /* CJK Ideograph */
  ["\u56B7", "\u56B7"], /* CJK Ideograph */
  ["\u56B8", "\u56B8"], /* CJK Ideograph */
  ["\u56B9", "\u56B9"], /* CJK Ideograph */
  ["\u56BA", "\u56BA"], /* CJK Ideograph */
  ["\u56BB", "\u56BB"], /* CJK Ideograph */
  ["\u56BC", "\u56BC"], /* CJK Ideograph */
  ["\u56BD", "\u56BD"], /* CJK Ideograph */
  ["\u56BE", "\u56BE"], /* CJK Ideograph */
  ["\u56BF", "\u56BF"], /* CJK Ideograph */
  ["\u56C0", "\u56C0"], /* CJK Ideograph */
  ["\u56C1", "\u56C1"], /* CJK Ideograph */
  ["\u56C2", "\u56C2"], /* CJK Ideograph */
  ["\u56C3", "\u56C3"], /* CJK Ideograph */
  ["\u56C4", "\u56C4"], /* CJK Ideograph */
  ["\u56C5", "\u56C5"], /* CJK Ideograph */
  ["\u56C6", "\u56C6"], /* CJK Ideograph */
  ["\u56C7", "\u56C7"], /* CJK Ideograph */
  ["\u56C8", "\u56C8"], /* CJK Ideograph */
  ["\u56C9", "\u56C9"], /* CJK Ideograph */
  ["\u56CA", "\u56CA"], /* CJK Ideograph */
  ["\u56CB", "\u56CB"], /* CJK Ideograph */
  ["\u56CC", "\u56CC"], /* CJK Ideograph */
  ["\u56CD", "\u56CD"], /* CJK Ideograph */
  ["\u56CE", "\u56CE"], /* CJK Ideograph */
  ["\u56CF", "\u56CF"], /* CJK Ideograph */
  ["\u56D0", "\u56D0"], /* CJK Ideograph */
  ["\u56D1", "\u56D1"], /* CJK Ideograph */
  ["\u56D2", "\u56D2"], /* CJK Ideograph */
  ["\u56D3", "\u56D3"], /* CJK Ideograph */
  ["\u56D4", "\u56D4"], /* CJK Ideograph */
  ["\u56D5", "\u56D5"], /* CJK Ideograph */
  ["\u56D6", "\u56D6"], /* CJK Ideograph */
  ["\u56D7", "\u56D7"], /* CJK Ideograph */
  ["\u56D8", "\u56D8"], /* CJK Ideograph */
  ["\u56D9", "\u56D9"], /* CJK Ideograph */
  ["\u56DA", "\u56DA"], /* CJK Ideograph */
  ["\u56DB", "\u56DB"], /* CJK Ideograph */
  ["\u56DC", "\u56DC"], /* CJK Ideograph */
  ["\u56DD", "\u56DD"], /* CJK Ideograph */
  ["\u56DE", "\u56DE"], /* CJK Ideograph */
  ["\u56DF", "\u56DF"], /* CJK Ideograph */
  ["\u56E0", "\u56E0"], /* CJK Ideograph */
  ["\u56E1", "\u56E1"], /* CJK Ideograph */
  ["\u56E2", "\u56E2"], /* CJK Ideograph */
  ["\u56E3", "\u56E3"], /* CJK Ideograph */
  ["\u56E4", "\u56E4"], /* CJK Ideograph */
  ["\u56E5", "\u56E5"], /* CJK Ideograph */
  ["\u56E6", "\u56E6"], /* CJK Ideograph */
  ["\u56E7", "\u56E7"], /* CJK Ideograph */
  ["\u56E8", "\u56E8"], /* CJK Ideograph */
  ["\u56E9", "\u56E9"], /* CJK Ideograph */
  ["\u56EA", "\u56EA"], /* CJK Ideograph */
  ["\u56EB", "\u56EB"], /* CJK Ideograph */
  ["\u56EC", "\u56EC"], /* CJK Ideograph */
  ["\u56ED", "\u56ED"], /* CJK Ideograph */
  ["\u56EE", "\u56EE"], /* CJK Ideograph */
  ["\u56EF", "\u56EF"], /* CJK Ideograph */
  ["\u56F0", "\u56F0"], /* CJK Ideograph */
  ["\u56F1", "\u56F1"], /* CJK Ideograph */
  ["\u56F2", "\u56F2"], /* CJK Ideograph */
  ["\u56F3", "\u56F3"], /* CJK Ideograph */
  ["\u56F4", "\u56F4"], /* CJK Ideograph */
  ["\u56F5", "\u56F5"], /* CJK Ideograph */
  ["\u56F6", "\u56F6"], /* CJK Ideograph */
  ["\u56F7", "\u56F7"], /* CJK Ideograph */
  ["\u56F8", "\u56F8"], /* CJK Ideograph */
  ["\u56F9", "\u56F9"], /* CJK Ideograph */
  ["\u56FA", "\u56FA"], /* CJK Ideograph */
  ["\u56FB", "\u56FB"], /* CJK Ideograph */
  ["\u56FC", "\u56FC"], /* CJK Ideograph */
  ["\u56FD", "\u56FD"], /* CJK Ideograph */
  ["\u56FE", "\u56FE"], /* CJK Ideograph */
  ["\u56FF", "\u56FF"], /* CJK Ideograph */
  ["\u5700", "\u5700"], /* CJK Ideograph */
  ["\u5701", "\u5701"], /* CJK Ideograph */
  ["\u5702", "\u5702"], /* CJK Ideograph */
  ["\u5703", "\u5703"], /* CJK Ideograph */
  ["\u5704", "\u5704"], /* CJK Ideograph */
  ["\u5705", "\u5705"], /* CJK Ideograph */
  ["\u5706", "\u5706"], /* CJK Ideograph */
  ["\u5707", "\u5707"], /* CJK Ideograph */
  ["\u5708", "\u5708"], /* CJK Ideograph */
  ["\u5709", "\u5709"], /* CJK Ideograph */
  ["\u570A", "\u570A"], /* CJK Ideograph */
  ["\u570B", "\u570B"], /* CJK Ideograph */
  ["\u570C", "\u570C"], /* CJK Ideograph */
  ["\u570D", "\u570D"], /* CJK Ideograph */
  ["\u570E", "\u570E"], /* CJK Ideograph */
  ["\u570F", "\u570F"], /* CJK Ideograph */
  ["\u5710", "\u5710"], /* CJK Ideograph */
  ["\u5711", "\u5711"], /* CJK Ideograph */
  ["\u5712", "\u5712"], /* CJK Ideograph */
  ["\u5713", "\u5713"], /* CJK Ideograph */
  ["\u5714", "\u5714"], /* CJK Ideograph */
  ["\u5715", "\u5715"], /* CJK Ideograph */
  ["\u5716", "\u5716"], /* CJK Ideograph */
  ["\u5717", "\u5717"], /* CJK Ideograph */
  ["\u5718", "\u5718"], /* CJK Ideograph */
  ["\u5719", "\u5719"], /* CJK Ideograph */
  ["\u571A", "\u571A"], /* CJK Ideograph */
  ["\u571B", "\u571B"], /* CJK Ideograph */
  ["\u571C", "\u571C"], /* CJK Ideograph */
  ["\u571D", "\u571D"], /* CJK Ideograph */
  ["\u571E", "\u571E"], /* CJK Ideograph */
  ["\u571F", "\u571F"], /* CJK Ideograph */
  ["\u5720", "\u5720"], /* CJK Ideograph */
  ["\u5721", "\u5721"], /* CJK Ideograph */
  ["\u5722", "\u5722"], /* CJK Ideograph */
  ["\u5723", "\u5723"], /* CJK Ideograph */
  ["\u5724", "\u5724"], /* CJK Ideograph */
  ["\u5725", "\u5725"], /* CJK Ideograph */
  ["\u5726", "\u5726"], /* CJK Ideograph */
  ["\u5727", "\u5727"], /* CJK Ideograph */
  ["\u5728", "\u5728"], /* CJK Ideograph */
  ["\u5729", "\u5729"], /* CJK Ideograph */
  ["\u572A", "\u572A"], /* CJK Ideograph */
  ["\u572B", "\u572B"], /* CJK Ideograph */
  ["\u572C", "\u572C"], /* CJK Ideograph */
  ["\u572D", "\u572D"], /* CJK Ideograph */
  ["\u572E", "\u572E"], /* CJK Ideograph */
  ["\u572F", "\u572F"], /* CJK Ideograph */
  ["\u5730", "\u5730"], /* CJK Ideograph */
  ["\u5731", "\u5731"], /* CJK Ideograph */
  ["\u5732", "\u5732"], /* CJK Ideograph */
  ["\u5733", "\u5733"], /* CJK Ideograph */
  ["\u5734", "\u5734"], /* CJK Ideograph */
  ["\u5735", "\u5735"], /* CJK Ideograph */
  ["\u5736", "\u5736"], /* CJK Ideograph */
  ["\u5737", "\u5737"], /* CJK Ideograph */
  ["\u5738", "\u5738"], /* CJK Ideograph */
  ["\u5739", "\u5739"], /* CJK Ideograph */
  ["\u573A", "\u573A"], /* CJK Ideograph */
  ["\u573B", "\u573B"], /* CJK Ideograph */
  ["\u573C", "\u573C"], /* CJK Ideograph */
  ["\u573D", "\u573D"], /* CJK Ideograph */
  ["\u573E", "\u573E"], /* CJK Ideograph */
  ["\u573F", "\u573F"], /* CJK Ideograph */
  ["\u5740", "\u5740"], /* CJK Ideograph */
  ["\u5741", "\u5741"], /* CJK Ideograph */
  ["\u5742", "\u5742"], /* CJK Ideograph */
  ["\u5743", "\u5743"], /* CJK Ideograph */
  ["\u5744", "\u5744"], /* CJK Ideograph */
  ["\u5745", "\u5745"], /* CJK Ideograph */
  ["\u5746", "\u5746"], /* CJK Ideograph */
  ["\u5747", "\u5747"], /* CJK Ideograph */
  ["\u5748", "\u5748"], /* CJK Ideograph */
  ["\u5749", "\u5749"], /* CJK Ideograph */
  ["\u574A", "\u574A"], /* CJK Ideograph */
  ["\u574B", "\u574B"], /* CJK Ideograph */
  ["\u574C", "\u574C"], /* CJK Ideograph */
  ["\u574D", "\u574D"], /* CJK Ideograph */
  ["\u574E", "\u574E"], /* CJK Ideograph */
  ["\u574F", "\u574F"], /* CJK Ideograph */
  ["\u5750", "\u5750"], /* CJK Ideograph */
  ["\u5751", "\u5751"], /* CJK Ideograph */
  ["\u5752", "\u5752"], /* CJK Ideograph */
  ["\u5753", "\u5753"], /* CJK Ideograph */
  ["\u5754", "\u5754"], /* CJK Ideograph */
  ["\u5755", "\u5755"], /* CJK Ideograph */
  ["\u5756", "\u5756"], /* CJK Ideograph */
  ["\u5757", "\u5757"], /* CJK Ideograph */
  ["\u5758", "\u5758"], /* CJK Ideograph */
  ["\u5759", "\u5759"], /* CJK Ideograph */
  ["\u575A", "\u575A"], /* CJK Ideograph */
  ["\u575B", "\u575B"], /* CJK Ideograph */
  ["\u575C", "\u575C"], /* CJK Ideograph */
  ["\u575D", "\u575D"], /* CJK Ideograph */
  ["\u575E", "\u575E"], /* CJK Ideograph */
  ["\u575F", "\u575F"], /* CJK Ideograph */
  ["\u5760", "\u5760"], /* CJK Ideograph */
  ["\u5761", "\u5761"], /* CJK Ideograph */
  ["\u5762", "\u5762"], /* CJK Ideograph */
  ["\u5763", "\u5763"], /* CJK Ideograph */
  ["\u5764", "\u5764"], /* CJK Ideograph */
  ["\u5765", "\u5765"], /* CJK Ideograph */
  ["\u5766", "\u5766"], /* CJK Ideograph */
  ["\u5767", "\u5767"], /* CJK Ideograph */
  ["\u5768", "\u5768"], /* CJK Ideograph */
  ["\u5769", "\u5769"], /* CJK Ideograph */
  ["\u576A", "\u576A"], /* CJK Ideograph */
  ["\u576B", "\u576B"], /* CJK Ideograph */
  ["\u576C", "\u576C"], /* CJK Ideograph */
  ["\u576D", "\u576D"], /* CJK Ideograph */
  ["\u576E", "\u576E"], /* CJK Ideograph */
  ["\u576F", "\u576F"], /* CJK Ideograph */
  ["\u5770", "\u5770"], /* CJK Ideograph */
  ["\u5771", "\u5771"], /* CJK Ideograph */
  ["\u5772", "\u5772"], /* CJK Ideograph */
  ["\u5773", "\u5773"], /* CJK Ideograph */
  ["\u5774", "\u5774"], /* CJK Ideograph */
  ["\u5775", "\u5775"], /* CJK Ideograph */
  ["\u5776", "\u5776"], /* CJK Ideograph */
  ["\u5777", "\u5777"], /* CJK Ideograph */
  ["\u5778", "\u5778"], /* CJK Ideograph */
  ["\u5779", "\u5779"], /* CJK Ideograph */
  ["\u577A", "\u577A"], /* CJK Ideograph */
  ["\u577B", "\u577B"], /* CJK Ideograph */
  ["\u577C", "\u577C"], /* CJK Ideograph */
  ["\u577D", "\u577D"], /* CJK Ideograph */
  ["\u577E", "\u577E"], /* CJK Ideograph */
  ["\u577F", "\u577F"], /* CJK Ideograph */
  ["\u5780", "\u5780"], /* CJK Ideograph */
  ["\u5781", "\u5781"], /* CJK Ideograph */
  ["\u5782", "\u5782"], /* CJK Ideograph */
  ["\u5783", "\u5783"], /* CJK Ideograph */
  ["\u5784", "\u5784"], /* CJK Ideograph */
  ["\u5785", "\u5785"], /* CJK Ideograph */
  ["\u5786", "\u5786"], /* CJK Ideograph */
  ["\u5787", "\u5787"], /* CJK Ideograph */
  ["\u5788", "\u5788"], /* CJK Ideograph */
  ["\u5789", "\u5789"], /* CJK Ideograph */
  ["\u578A", "\u578A"], /* CJK Ideograph */
  ["\u578B", "\u578B"], /* CJK Ideograph */
  ["\u578C", "\u578C"], /* CJK Ideograph */
  ["\u578D", "\u578D"], /* CJK Ideograph */
  ["\u578E", "\u578E"], /* CJK Ideograph */
  ["\u578F", "\u578F"], /* CJK Ideograph */
  ["\u5790", "\u5790"], /* CJK Ideograph */
  ["\u5791", "\u5791"], /* CJK Ideograph */
  ["\u5792", "\u5792"], /* CJK Ideograph */
  ["\u5793", "\u5793"], /* CJK Ideograph */
  ["\u5794", "\u5794"], /* CJK Ideograph */
  ["\u5795", "\u5795"], /* CJK Ideograph */
  ["\u5796", "\u5796"], /* CJK Ideograph */
  ["\u5797", "\u5797"], /* CJK Ideograph */
  ["\u5798", "\u5798"], /* CJK Ideograph */
  ["\u5799", "\u5799"], /* CJK Ideograph */
  ["\u579A", "\u579A"], /* CJK Ideograph */
  ["\u579B", "\u579B"], /* CJK Ideograph */
  ["\u579C", "\u579C"], /* CJK Ideograph */
  ["\u579D", "\u579D"], /* CJK Ideograph */
  ["\u579E", "\u579E"], /* CJK Ideograph */
  ["\u579F", "\u579F"], /* CJK Ideograph */
  ["\u57A0", "\u57A0"], /* CJK Ideograph */
  ["\u57A1", "\u57A1"], /* CJK Ideograph */
  ["\u57A2", "\u57A2"], /* CJK Ideograph */
  ["\u57A3", "\u57A3"], /* CJK Ideograph */
  ["\u57A4", "\u57A4"], /* CJK Ideograph */
  ["\u57A5", "\u57A5"], /* CJK Ideograph */
  ["\u57A6", "\u57A6"], /* CJK Ideograph */
  ["\u57A7", "\u57A7"], /* CJK Ideograph */
  ["\u57A8", "\u57A8"], /* CJK Ideograph */
  ["\u57A9", "\u57A9"], /* CJK Ideograph */
  ["\u57AA", "\u57AA"], /* CJK Ideograph */
  ["\u57AB", "\u57AB"], /* CJK Ideograph */
  ["\u57AC", "\u57AC"], /* CJK Ideograph */
  ["\u57AD", "\u57AD"], /* CJK Ideograph */
  ["\u57AE", "\u57AE"], /* CJK Ideograph */
  ["\u57AF", "\u57AF"], /* CJK Ideograph */
  ["\u57B0", "\u57B0"], /* CJK Ideograph */
  ["\u57B1", "\u57B1"], /* CJK Ideograph */
  ["\u57B2", "\u57B2"], /* CJK Ideograph */
  ["\u57B3", "\u57B3"], /* CJK Ideograph */
  ["\u57B4", "\u57B4"], /* CJK Ideograph */
  ["\u57B5", "\u57B5"], /* CJK Ideograph */
  ["\u57B6", "\u57B6"], /* CJK Ideograph */
  ["\u57B7", "\u57B7"], /* CJK Ideograph */
  ["\u57B8", "\u57B8"], /* CJK Ideograph */
  ["\u57B9", "\u57B9"], /* CJK Ideograph */
  ["\u57BA", "\u57BA"], /* CJK Ideograph */
  ["\u57BB", "\u57BB"], /* CJK Ideograph */
  ["\u57BC", "\u57BC"], /* CJK Ideograph */
  ["\u57BD", "\u57BD"], /* CJK Ideograph */
  ["\u57BE", "\u57BE"], /* CJK Ideograph */
  ["\u57BF", "\u57BF"], /* CJK Ideograph */
  ["\u57C0", "\u57C0"], /* CJK Ideograph */
  ["\u57C1", "\u57C1"], /* CJK Ideograph */
  ["\u57C2", "\u57C2"], /* CJK Ideograph */
  ["\u57C3", "\u57C3"], /* CJK Ideograph */
  ["\u57C4", "\u57C4"], /* CJK Ideograph */
  ["\u57C5", "\u57C5"], /* CJK Ideograph */
  ["\u57C6", "\u57C6"], /* CJK Ideograph */
  ["\u57C7", "\u57C7"], /* CJK Ideograph */
  ["\u57C8", "\u57C8"], /* CJK Ideograph */
  ["\u57C9", "\u57C9"], /* CJK Ideograph */
  ["\u57CA", "\u57CA"], /* CJK Ideograph */
  ["\u57CB", "\u57CB"], /* CJK Ideograph */
  ["\u57CC", "\u57CC"], /* CJK Ideograph */
  ["\u57CD", "\u57CD"], /* CJK Ideograph */
  ["\u57CE", "\u57CE"], /* CJK Ideograph */
  ["\u57CF", "\u57CF"], /* CJK Ideograph */
  ["\u57D0", "\u57D0"], /* CJK Ideograph */
  ["\u57D1", "\u57D1"], /* CJK Ideograph */
  ["\u57D2", "\u57D2"], /* CJK Ideograph */
  ["\u57D3", "\u57D3"], /* CJK Ideograph */
  ["\u57D4", "\u57D4"], /* CJK Ideograph */
  ["\u57D5", "\u57D5"], /* CJK Ideograph */
  ["\u57D6", "\u57D6"], /* CJK Ideograph */
  ["\u57D7", "\u57D7"], /* CJK Ideograph */
  ["\u57D8", "\u57D8"], /* CJK Ideograph */
  ["\u57D9", "\u57D9"], /* CJK Ideograph */
  ["\u57DA", "\u57DA"], /* CJK Ideograph */
  ["\u57DB", "\u57DB"], /* CJK Ideograph */
  ["\u57DC", "\u57DC"], /* CJK Ideograph */
  ["\u57DD", "\u57DD"], /* CJK Ideograph */
  ["\u57DE", "\u57DE"], /* CJK Ideograph */
  ["\u57DF", "\u57DF"], /* CJK Ideograph */
  ["\u57E0", "\u57E0"], /* CJK Ideograph */
  ["\u57E1", "\u57E1"], /* CJK Ideograph */
  ["\u57E2", "\u57E2"], /* CJK Ideograph */
  ["\u57E3", "\u57E3"], /* CJK Ideograph */
  ["\u57E4", "\u57E4"], /* CJK Ideograph */
  ["\u57E5", "\u57E5"], /* CJK Ideograph */
  ["\u57E6", "\u57E6"], /* CJK Ideograph */
  ["\u57E7", "\u57E7"], /* CJK Ideograph */
  ["\u57E8", "\u57E8"], /* CJK Ideograph */
  ["\u57E9", "\u57E9"], /* CJK Ideograph */
  ["\u57EA", "\u57EA"], /* CJK Ideograph */
  ["\u57EB", "\u57EB"], /* CJK Ideograph */
  ["\u57EC", "\u57EC"], /* CJK Ideograph */
  ["\u57ED", "\u57ED"], /* CJK Ideograph */
  ["\u57EE", "\u57EE"], /* CJK Ideograph */
  ["\u57EF", "\u57EF"], /* CJK Ideograph */
  ["\u57F0", "\u57F0"], /* CJK Ideograph */
  ["\u57F1", "\u57F1"], /* CJK Ideograph */
  ["\u57F2", "\u57F2"], /* CJK Ideograph */
  ["\u57F3", "\u57F3"], /* CJK Ideograph */
  ["\u57F4", "\u57F4"], /* CJK Ideograph */
  ["\u57F5", "\u57F5"], /* CJK Ideograph */
  ["\u57F6", "\u57F6"], /* CJK Ideograph */
  ["\u57F7", "\u57F7"], /* CJK Ideograph */
  ["\u57F8", "\u57F8"], /* CJK Ideograph */
  ["\u57F9", "\u57F9"], /* CJK Ideograph */
  ["\u57FA", "\u57FA"], /* CJK Ideograph */
  ["\u57FB", "\u57FB"], /* CJK Ideograph */
  ["\u57FC", "\u57FC"], /* CJK Ideograph */
  ["\u57FD", "\u57FD"], /* CJK Ideograph */
  ["\u57FE", "\u57FE"], /* CJK Ideograph */
  ["\u57FF", "\u57FF"], /* CJK Ideograph */
  ["\u5800", "\u5800"], /* CJK Ideograph */
  ["\u5801", "\u5801"], /* CJK Ideograph */
  ["\u5802", "\u5802"], /* CJK Ideograph */
  ["\u5803", "\u5803"], /* CJK Ideograph */
  ["\u5804", "\u5804"], /* CJK Ideograph */
  ["\u5805", "\u5805"], /* CJK Ideograph */
  ["\u5806", "\u5806"], /* CJK Ideograph */
  ["\u5807", "\u5807"], /* CJK Ideograph */
  ["\u5808", "\u5808"], /* CJK Ideograph */
  ["\u5809", "\u5809"], /* CJK Ideograph */
  ["\u580A", "\u580A"], /* CJK Ideograph */
  ["\u580B", "\u580B"], /* CJK Ideograph */
  ["\u580C", "\u580C"], /* CJK Ideograph */
  ["\u580D", "\u580D"], /* CJK Ideograph */
  ["\u580E", "\u580E"], /* CJK Ideograph */
  ["\u580F", "\u580F"], /* CJK Ideograph */
  ["\u5810", "\u5810"], /* CJK Ideograph */
  ["\u5811", "\u5811"], /* CJK Ideograph */
  ["\u5812", "\u5812"], /* CJK Ideograph */
  ["\u5813", "\u5813"], /* CJK Ideograph */
  ["\u5814", "\u5814"], /* CJK Ideograph */
  ["\u5815", "\u5815"], /* CJK Ideograph */
  ["\u5816", "\u5816"], /* CJK Ideograph */
  ["\u5817", "\u5817"], /* CJK Ideograph */
  ["\u5818", "\u5818"], /* CJK Ideograph */
  ["\u5819", "\u5819"], /* CJK Ideograph */
  ["\u581A", "\u581A"], /* CJK Ideograph */
  ["\u581B", "\u581B"], /* CJK Ideograph */
  ["\u581C", "\u581C"], /* CJK Ideograph */
  ["\u581D", "\u581D"], /* CJK Ideograph */
  ["\u581E", "\u581E"], /* CJK Ideograph */
  ["\u581F", "\u581F"], /* CJK Ideograph */
  ["\u5820", "\u5820"], /* CJK Ideograph */
  ["\u5821", "\u5821"], /* CJK Ideograph */
  ["\u5822", "\u5822"], /* CJK Ideograph */
  ["\u5823", "\u5823"], /* CJK Ideograph */
  ["\u5824", "\u5824"], /* CJK Ideograph */
  ["\u5825", "\u5825"], /* CJK Ideograph */
  ["\u5826", "\u5826"], /* CJK Ideograph */
  ["\u5827", "\u5827"], /* CJK Ideograph */
  ["\u5828", "\u5828"], /* CJK Ideograph */
  ["\u5829", "\u5829"], /* CJK Ideograph */
  ["\u582A", "\u582A"], /* CJK Ideograph */
  ["\u582B", "\u582B"], /* CJK Ideograph */
  ["\u582C", "\u582C"], /* CJK Ideograph */
  ["\u582D", "\u582D"], /* CJK Ideograph */
  ["\u582E", "\u582E"], /* CJK Ideograph */
  ["\u582F", "\u582F"], /* CJK Ideograph */
  ["\u5830", "\u5830"], /* CJK Ideograph */
  ["\u5831", "\u5831"], /* CJK Ideograph */
  ["\u5832", "\u5832"], /* CJK Ideograph */
  ["\u5833", "\u5833"], /* CJK Ideograph */
  ["\u5834", "\u5834"], /* CJK Ideograph */
  ["\u5835", "\u5835"], /* CJK Ideograph */
  ["\u5836", "\u5836"], /* CJK Ideograph */
  ["\u5837", "\u5837"], /* CJK Ideograph */
  ["\u5838", "\u5838"], /* CJK Ideograph */
  ["\u5839", "\u5839"], /* CJK Ideograph */
  ["\u583A", "\u583A"], /* CJK Ideograph */
  ["\u583B", "\u583B"], /* CJK Ideograph */
  ["\u583C", "\u583C"], /* CJK Ideograph */
  ["\u583D", "\u583D"], /* CJK Ideograph */
  ["\u583E", "\u583E"], /* CJK Ideograph */
  ["\u583F", "\u583F"], /* CJK Ideograph */
  ["\u5840", "\u5840"], /* CJK Ideograph */
  ["\u5841", "\u5841"], /* CJK Ideograph */
  ["\u5842", "\u5842"], /* CJK Ideograph */
  ["\u5843", "\u5843"], /* CJK Ideograph */
  ["\u5844", "\u5844"], /* CJK Ideograph */
  ["\u5845", "\u5845"], /* CJK Ideograph */
  ["\u5846", "\u5846"], /* CJK Ideograph */
  ["\u5847", "\u5847"], /* CJK Ideograph */
  ["\u5848", "\u5848"], /* CJK Ideograph */
  ["\u5849", "\u5849"], /* CJK Ideograph */
  ["\u584A", "\u584A"], /* CJK Ideograph */
  ["\u584B", "\u584B"], /* CJK Ideograph */
  ["\u584C", "\u584C"], /* CJK Ideograph */
  ["\u584D", "\u584D"], /* CJK Ideograph */
  ["\u584E", "\u584E"], /* CJK Ideograph */
  ["\u584F", "\u584F"], /* CJK Ideograph */
  ["\u5850", "\u5850"], /* CJK Ideograph */
  ["\u5851", "\u5851"], /* CJK Ideograph */
  ["\u5852", "\u5852"], /* CJK Ideograph */
  ["\u5853", "\u5853"], /* CJK Ideograph */
  ["\u5854", "\u5854"], /* CJK Ideograph */
  ["\u5855", "\u5855"], /* CJK Ideograph */
  ["\u5856", "\u5856"], /* CJK Ideograph */
  ["\u5857", "\u5857"], /* CJK Ideograph */
  ["\u5858", "\u5858"], /* CJK Ideograph */
  ["\u5859", "\u5859"], /* CJK Ideograph */
  ["\u585A", "\u585A"], /* CJK Ideograph */
  ["\u585B", "\u585B"], /* CJK Ideograph */
  ["\u585C", "\u585C"], /* CJK Ideograph */
  ["\u585D", "\u585D"], /* CJK Ideograph */
  ["\u585E", "\u585E"], /* CJK Ideograph */
  ["\u585F", "\u585F"], /* CJK Ideograph */
  ["\u5860", "\u5860"], /* CJK Ideograph */
  ["\u5861", "\u5861"], /* CJK Ideograph */
  ["\u5862", "\u5862"], /* CJK Ideograph */
  ["\u5863", "\u5863"], /* CJK Ideograph */
  ["\u5864", "\u5864"], /* CJK Ideograph */
  ["\u5865", "\u5865"], /* CJK Ideograph */
  ["\u5866", "\u5866"], /* CJK Ideograph */
  ["\u5867", "\u5867"], /* CJK Ideograph */
  ["\u5868", "\u5868"], /* CJK Ideograph */
  ["\u5869", "\u5869"], /* CJK Ideograph */
  ["\u586A", "\u586A"], /* CJK Ideograph */
  ["\u586B", "\u586B"], /* CJK Ideograph */
  ["\u586C", "\u586C"], /* CJK Ideograph */
  ["\u586D", "\u586D"], /* CJK Ideograph */
  ["\u586E", "\u586E"], /* CJK Ideograph */
  ["\u586F", "\u586F"], /* CJK Ideograph */
  ["\u5870", "\u5870"], /* CJK Ideograph */
  ["\u5871", "\u5871"], /* CJK Ideograph */
  ["\u5872", "\u5872"], /* CJK Ideograph */
  ["\u5873", "\u5873"], /* CJK Ideograph */
  ["\u5874", "\u5874"], /* CJK Ideograph */
  ["\u5875", "\u5875"], /* CJK Ideograph */
  ["\u5876", "\u5876"], /* CJK Ideograph */
  ["\u5877", "\u5877"], /* CJK Ideograph */
  ["\u5878", "\u5878"], /* CJK Ideograph */
  ["\u5879", "\u5879"], /* CJK Ideograph */
  ["\u587A", "\u587A"], /* CJK Ideograph */
  ["\u587B", "\u587B"], /* CJK Ideograph */
  ["\u587C", "\u587C"], /* CJK Ideograph */
  ["\u587D", "\u587D"], /* CJK Ideograph */
  ["\u587E", "\u587E"], /* CJK Ideograph */
  ["\u587F", "\u587F"], /* CJK Ideograph */
  ["\u5880", "\u5880"], /* CJK Ideograph */
  ["\u5881", "\u5881"], /* CJK Ideograph */
  ["\u5882", "\u5882"], /* CJK Ideograph */
  ["\u5883", "\u5883"], /* CJK Ideograph */
  ["\u5884", "\u5884"], /* CJK Ideograph */
  ["\u5885", "\u5885"], /* CJK Ideograph */
  ["\u5886", "\u5886"], /* CJK Ideograph */
  ["\u5887", "\u5887"], /* CJK Ideograph */
  ["\u5888", "\u5888"], /* CJK Ideograph */
  ["\u5889", "\u5889"], /* CJK Ideograph */
  ["\u588A", "\u588A"], /* CJK Ideograph */
  ["\u588B", "\u588B"], /* CJK Ideograph */
  ["\u588C", "\u588C"], /* CJK Ideograph */
  ["\u588D", "\u588D"], /* CJK Ideograph */
  ["\u588E", "\u588E"], /* CJK Ideograph */
  ["\u588F", "\u588F"], /* CJK Ideograph */
  ["\u5890", "\u5890"], /* CJK Ideograph */
  ["\u5891", "\u5891"], /* CJK Ideograph */
  ["\u5892", "\u5892"], /* CJK Ideograph */
  ["\u5893", "\u5893"], /* CJK Ideograph */
  ["\u5894", "\u5894"], /* CJK Ideograph */
  ["\u5895", "\u5895"], /* CJK Ideograph */
  ["\u5896", "\u5896"], /* CJK Ideograph */
  ["\u5897", "\u5897"], /* CJK Ideograph */
  ["\u5898", "\u5898"], /* CJK Ideograph */
  ["\u5899", "\u5899"], /* CJK Ideograph */
  ["\u589A", "\u589A"], /* CJK Ideograph */
  ["\u589B", "\u589B"], /* CJK Ideograph */
  ["\u589C", "\u589C"], /* CJK Ideograph */
  ["\u589D", "\u589D"], /* CJK Ideograph */
  ["\u589E", "\u589E"], /* CJK Ideograph */
  ["\u589F", "\u589F"], /* CJK Ideograph */
  ["\u58A0", "\u58A0"], /* CJK Ideograph */
  ["\u58A1", "\u58A1"], /* CJK Ideograph */
  ["\u58A2", "\u58A2"], /* CJK Ideograph */
  ["\u58A3", "\u58A3"], /* CJK Ideograph */
  ["\u58A4", "\u58A4"], /* CJK Ideograph */
  ["\u58A5", "\u58A5"], /* CJK Ideograph */
  ["\u58A6", "\u58A6"], /* CJK Ideograph */
  ["\u58A7", "\u58A7"], /* CJK Ideograph */
  ["\u58A8", "\u58A8"], /* CJK Ideograph */
  ["\u58A9", "\u58A9"], /* CJK Ideograph */
  ["\u58AA", "\u58AA"], /* CJK Ideograph */
  ["\u58AB", "\u58AB"], /* CJK Ideograph */
  ["\u58AC", "\u58AC"], /* CJK Ideograph */
  ["\u58AD", "\u58AD"], /* CJK Ideograph */
  ["\u58AE", "\u58AE"], /* CJK Ideograph */
  ["\u58AF", "\u58AF"], /* CJK Ideograph */
  ["\u58B0", "\u58B0"], /* CJK Ideograph */
  ["\u58B1", "\u58B1"], /* CJK Ideograph */
  ["\u58B2", "\u58B2"], /* CJK Ideograph */
  ["\u58B3", "\u58B3"], /* CJK Ideograph */
  ["\u58B4", "\u58B4"], /* CJK Ideograph */
  ["\u58B5", "\u58B5"], /* CJK Ideograph */
  ["\u58B6", "\u58B6"], /* CJK Ideograph */
  ["\u58B7", "\u58B7"], /* CJK Ideograph */
  ["\u58B8", "\u58B8"], /* CJK Ideograph */
  ["\u58B9", "\u58B9"], /* CJK Ideograph */
  ["\u58BA", "\u58BA"], /* CJK Ideograph */
  ["\u58BB", "\u58BB"], /* CJK Ideograph */
  ["\u58BC", "\u58BC"], /* CJK Ideograph */
  ["\u58BD", "\u58BD"], /* CJK Ideograph */
  ["\u58BE", "\u58BE"], /* CJK Ideograph */
  ["\u58BF", "\u58BF"], /* CJK Ideograph */
  ["\u58C0", "\u58C0"], /* CJK Ideograph */
  ["\u58C1", "\u58C1"], /* CJK Ideograph */
  ["\u58C2", "\u58C2"], /* CJK Ideograph */
  ["\u58C3", "\u58C3"], /* CJK Ideograph */
  ["\u58C4", "\u58C4"], /* CJK Ideograph */
  ["\u58C5", "\u58C5"], /* CJK Ideograph */
  ["\u58C6", "\u58C6"], /* CJK Ideograph */
  ["\u58C7", "\u58C7"], /* CJK Ideograph */
  ["\u58C8", "\u58C8"], /* CJK Ideograph */
  ["\u58C9", "\u58C9"], /* CJK Ideograph */
  ["\u58CA", "\u58CA"], /* CJK Ideograph */
  ["\u58CB", "\u58CB"], /* CJK Ideograph */
  ["\u58CC", "\u58CC"], /* CJK Ideograph */
  ["\u58CD", "\u58CD"], /* CJK Ideograph */
  ["\u58CE", "\u58CE"], /* CJK Ideograph */
  ["\u58CF", "\u58CF"], /* CJK Ideograph */
  ["\u58D0", "\u58D0"], /* CJK Ideograph */
  ["\u58D1", "\u58D1"], /* CJK Ideograph */
  ["\u58D2", "\u58D2"], /* CJK Ideograph */
  ["\u58D3", "\u58D3"], /* CJK Ideograph */
  ["\u58D4", "\u58D4"], /* CJK Ideograph */
  ["\u58D5", "\u58D5"], /* CJK Ideograph */
  ["\u58D6", "\u58D6"], /* CJK Ideograph */
  ["\u58D7", "\u58D7"], /* CJK Ideograph */
  ["\u58D8", "\u58D8"], /* CJK Ideograph */
  ["\u58D9", "\u58D9"], /* CJK Ideograph */
  ["\u58DA", "\u58DA"], /* CJK Ideograph */
  ["\u58DB", "\u58DB"], /* CJK Ideograph */
  ["\u58DC", "\u58DC"], /* CJK Ideograph */
  ["\u58DD", "\u58DD"], /* CJK Ideograph */
  ["\u58DE", "\u58DE"], /* CJK Ideograph */
  ["\u58DF", "\u58DF"], /* CJK Ideograph */
  ["\u58E0", "\u58E0"], /* CJK Ideograph */
  ["\u58E1", "\u58E1"], /* CJK Ideograph */
  ["\u58E2", "\u58E2"], /* CJK Ideograph */
  ["\u58E3", "\u58E3"], /* CJK Ideograph */
  ["\u58E4", "\u58E4"], /* CJK Ideograph */
  ["\u58E5", "\u58E5"], /* CJK Ideograph */
  ["\u58E6", "\u58E6"], /* CJK Ideograph */
  ["\u58E7", "\u58E7"], /* CJK Ideograph */
  ["\u58E8", "\u58E8"], /* CJK Ideograph */
  ["\u58E9", "\u58E9"], /* CJK Ideograph */
  ["\u58EA", "\u58EA"], /* CJK Ideograph */
  ["\u58EB", "\u58EB"], /* CJK Ideograph */
  ["\u58EC", "\u58EC"], /* CJK Ideograph */
  ["\u58ED", "\u58ED"], /* CJK Ideograph */
  ["\u58EE", "\u58EE"], /* CJK Ideograph */
  ["\u58EF", "\u58EF"], /* CJK Ideograph */
  ["\u58F0", "\u58F0"], /* CJK Ideograph */
  ["\u58F1", "\u58F1"], /* CJK Ideograph */
  ["\u58F2", "\u58F2"], /* CJK Ideograph */
  ["\u58F3", "\u58F3"], /* CJK Ideograph */
  ["\u58F4", "\u58F4"], /* CJK Ideograph */
  ["\u58F5", "\u58F5"], /* CJK Ideograph */
  ["\u58F6", "\u58F6"], /* CJK Ideograph */
  ["\u58F7", "\u58F7"], /* CJK Ideograph */
  ["\u58F8", "\u58F8"], /* CJK Ideograph */
  ["\u58F9", "\u58F9"], /* CJK Ideograph */
  ["\u58FA", "\u58FA"], /* CJK Ideograph */
  ["\u58FB", "\u58FB"], /* CJK Ideograph */
  ["\u58FC", "\u58FC"], /* CJK Ideograph */
  ["\u58FD", "\u58FD"], /* CJK Ideograph */
  ["\u58FE", "\u58FE"], /* CJK Ideograph */
  ["\u58FF", "\u58FF"], /* CJK Ideograph */
  ["\u5900", "\u5900"], /* CJK Ideograph */
  ["\u5901", "\u5901"], /* CJK Ideograph */
  ["\u5902", "\u5902"], /* CJK Ideograph */
  ["\u5903", "\u5903"], /* CJK Ideograph */
  ["\u5904", "\u5904"], /* CJK Ideograph */
  ["\u5905", "\u5905"], /* CJK Ideograph */
  ["\u5906", "\u5906"], /* CJK Ideograph */
  ["\u5907", "\u5907"], /* CJK Ideograph */
  ["\u5908", "\u5908"], /* CJK Ideograph */
  ["\u5909", "\u5909"], /* CJK Ideograph */
  ["\u590A", "\u590A"], /* CJK Ideograph */
  ["\u590B", "\u590B"], /* CJK Ideograph */
  ["\u590C", "\u590C"], /* CJK Ideograph */
  ["\u590D", "\u590D"], /* CJK Ideograph */
  ["\u590E", "\u590E"], /* CJK Ideograph */
  ["\u590F", "\u590F"], /* CJK Ideograph */
  ["\u5910", "\u5910"], /* CJK Ideograph */
  ["\u5911", "\u5911"], /* CJK Ideograph */
  ["\u5912", "\u5912"], /* CJK Ideograph */
  ["\u5913", "\u5913"], /* CJK Ideograph */
  ["\u5914", "\u5914"], /* CJK Ideograph */
  ["\u5915", "\u5915"], /* CJK Ideograph */
  ["\u5916", "\u5916"], /* CJK Ideograph */
  ["\u5917", "\u5917"], /* CJK Ideograph */
  ["\u5918", "\u5918"], /* CJK Ideograph */
  ["\u5919", "\u5919"], /* CJK Ideograph */
  ["\u591A", "\u591A"], /* CJK Ideograph */
  ["\u591B", "\u591B"], /* CJK Ideograph */
  ["\u591C", "\u591C"], /* CJK Ideograph */
  ["\u591D", "\u591D"], /* CJK Ideograph */
  ["\u591E", "\u591E"], /* CJK Ideograph */
  ["\u591F", "\u591F"], /* CJK Ideograph */
  ["\u5920", "\u5920"], /* CJK Ideograph */
  ["\u5921", "\u5921"], /* CJK Ideograph */
  ["\u5922", "\u5922"], /* CJK Ideograph */
  ["\u5923", "\u5923"], /* CJK Ideograph */
  ["\u5924", "\u5924"], /* CJK Ideograph */
  ["\u5925", "\u5925"], /* CJK Ideograph */
  ["\u5926", "\u5926"], /* CJK Ideograph */
  ["\u5927", "\u5927"], /* CJK Ideograph */
  ["\u5928", "\u5928"], /* CJK Ideograph */
  ["\u5929", "\u5929"], /* CJK Ideograph */
  ["\u592A", "\u592A"], /* CJK Ideograph */
  ["\u592B", "\u592B"], /* CJK Ideograph */
  ["\u592C", "\u592C"], /* CJK Ideograph */
  ["\u592D", "\u592D"], /* CJK Ideograph */
  ["\u592E", "\u592E"], /* CJK Ideograph */
  ["\u592F", "\u592F"], /* CJK Ideograph */
  ["\u5930", "\u5930"], /* CJK Ideograph */
  ["\u5931", "\u5931"], /* CJK Ideograph */
  ["\u5932", "\u5932"], /* CJK Ideograph */
  ["\u5933", "\u5933"], /* CJK Ideograph */
  ["\u5934", "\u5934"], /* CJK Ideograph */
  ["\u5935", "\u5935"], /* CJK Ideograph */
  ["\u5936", "\u5936"], /* CJK Ideograph */
  ["\u5937", "\u5937"], /* CJK Ideograph */
  ["\u5938", "\u5938"], /* CJK Ideograph */
  ["\u5939", "\u5939"], /* CJK Ideograph */
  ["\u593A", "\u593A"], /* CJK Ideograph */
  ["\u593B", "\u593B"], /* CJK Ideograph */
  ["\u593C", "\u593C"], /* CJK Ideograph */
  ["\u593D", "\u593D"], /* CJK Ideograph */
  ["\u593E", "\u593E"], /* CJK Ideograph */
  ["\u593F", "\u593F"], /* CJK Ideograph */
  ["\u5940", "\u5940"], /* CJK Ideograph */
  ["\u5941", "\u5941"], /* CJK Ideograph */
  ["\u5942", "\u5942"], /* CJK Ideograph */
  ["\u5943", "\u5943"], /* CJK Ideograph */
  ["\u5944", "\u5944"], /* CJK Ideograph */
  ["\u5945", "\u5945"], /* CJK Ideograph */
  ["\u5946", "\u5946"], /* CJK Ideograph */
  ["\u5947", "\u5947"], /* CJK Ideograph */
  ["\u5948", "\u5948"], /* CJK Ideograph */
  ["\u5949", "\u5949"], /* CJK Ideograph */
  ["\u594A", "\u594A"], /* CJK Ideograph */
  ["\u594B", "\u594B"], /* CJK Ideograph */
  ["\u594C", "\u594C"], /* CJK Ideograph */
  ["\u594D", "\u594D"], /* CJK Ideograph */
  ["\u594E", "\u594E"], /* CJK Ideograph */
  ["\u594F", "\u594F"], /* CJK Ideograph */
  ["\u5950", "\u5950"], /* CJK Ideograph */
  ["\u5951", "\u5951"], /* CJK Ideograph */
  ["\u5952", "\u5952"], /* CJK Ideograph */
  ["\u5953", "\u5953"], /* CJK Ideograph */
  ["\u5954", "\u5954"], /* CJK Ideograph */
  ["\u5955", "\u5955"], /* CJK Ideograph */
  ["\u5956", "\u5956"], /* CJK Ideograph */
  ["\u5957", "\u5957"], /* CJK Ideograph */
  ["\u5958", "\u5958"], /* CJK Ideograph */
  ["\u5959", "\u5959"], /* CJK Ideograph */
  ["\u595A", "\u595A"], /* CJK Ideograph */
  ["\u595B", "\u595B"], /* CJK Ideograph */
  ["\u595C", "\u595C"], /* CJK Ideograph */
  ["\u595D", "\u595D"], /* CJK Ideograph */
  ["\u595E", "\u595E"], /* CJK Ideograph */
  ["\u595F", "\u595F"], /* CJK Ideograph */
  ["\u5960", "\u5960"], /* CJK Ideograph */
  ["\u5961", "\u5961"], /* CJK Ideograph */
  ["\u5962", "\u5962"], /* CJK Ideograph */
  ["\u5963", "\u5963"], /* CJK Ideograph */
  ["\u5964", "\u5964"], /* CJK Ideograph */
  ["\u5965", "\u5965"], /* CJK Ideograph */
  ["\u5966", "\u5966"], /* CJK Ideograph */
  ["\u5967", "\u5967"], /* CJK Ideograph */
  ["\u5968", "\u5968"], /* CJK Ideograph */
  ["\u5969", "\u5969"], /* CJK Ideograph */
  ["\u596A", "\u596A"], /* CJK Ideograph */
  ["\u596B", "\u596B"], /* CJK Ideograph */
  ["\u596C", "\u596C"], /* CJK Ideograph */
  ["\u596D", "\u596D"], /* CJK Ideograph */
  ["\u596E", "\u596E"], /* CJK Ideograph */
  ["\u596F", "\u596F"], /* CJK Ideograph */
  ["\u5970", "\u5970"], /* CJK Ideograph */
  ["\u5971", "\u5971"], /* CJK Ideograph */
  ["\u5972", "\u5972"], /* CJK Ideograph */
  ["\u5973", "\u5973"], /* CJK Ideograph */
  ["\u5974", "\u5974"], /* CJK Ideograph */
  ["\u5975", "\u5975"], /* CJK Ideograph */
  ["\u5976", "\u5976"], /* CJK Ideograph */
  ["\u5977", "\u5977"], /* CJK Ideograph */
  ["\u5978", "\u5978"], /* CJK Ideograph */
  ["\u5979", "\u5979"], /* CJK Ideograph */
  ["\u597A", "\u597A"], /* CJK Ideograph */
  ["\u597B", "\u597B"], /* CJK Ideograph */
  ["\u597C", "\u597C"], /* CJK Ideograph */
  ["\u597D", "\u597D"], /* CJK Ideograph */
  ["\u597E", "\u597E"], /* CJK Ideograph */
  ["\u597F", "\u597F"], /* CJK Ideograph */
  ["\u5980", "\u5980"], /* CJK Ideograph */
  ["\u5981", "\u5981"], /* CJK Ideograph */
  ["\u5982", "\u5982"], /* CJK Ideograph */
  ["\u5983", "\u5983"], /* CJK Ideograph */
  ["\u5984", "\u5984"], /* CJK Ideograph */
  ["\u5985", "\u5985"], /* CJK Ideograph */
  ["\u5986", "\u5986"], /* CJK Ideograph */
  ["\u5987", "\u5987"], /* CJK Ideograph */
  ["\u5988", "\u5988"], /* CJK Ideograph */
  ["\u5989", "\u5989"], /* CJK Ideograph */
  ["\u598A", "\u598A"], /* CJK Ideograph */
  ["\u598B", "\u598B"], /* CJK Ideograph */
  ["\u598C", "\u598C"], /* CJK Ideograph */
  ["\u598D", "\u598D"], /* CJK Ideograph */
  ["\u598E", "\u598E"], /* CJK Ideograph */
  ["\u598F", "\u598F"], /* CJK Ideograph */
  ["\u5990", "\u5990"], /* CJK Ideograph */
  ["\u5991", "\u5991"], /* CJK Ideograph */
  ["\u5992", "\u5992"], /* CJK Ideograph */
  ["\u5993", "\u5993"], /* CJK Ideograph */
  ["\u5994", "\u5994"], /* CJK Ideograph */
  ["\u5995", "\u5995"], /* CJK Ideograph */
  ["\u5996", "\u5996"], /* CJK Ideograph */
  ["\u5997", "\u5997"], /* CJK Ideograph */
  ["\u5998", "\u5998"], /* CJK Ideograph */
  ["\u5999", "\u5999"], /* CJK Ideograph */
  ["\u599A", "\u599A"], /* CJK Ideograph */
  ["\u599B", "\u599B"], /* CJK Ideograph */
  ["\u599C", "\u599C"], /* CJK Ideograph */
  ["\u599D", "\u599D"], /* CJK Ideograph */
  ["\u599E", "\u599E"], /* CJK Ideograph */
  ["\u599F", "\u599F"], /* CJK Ideograph */
  ["\u59A0", "\u59A0"], /* CJK Ideograph */
  ["\u59A1", "\u59A1"], /* CJK Ideograph */
  ["\u59A2", "\u59A2"], /* CJK Ideograph */
  ["\u59A3", "\u59A3"], /* CJK Ideograph */
  ["\u59A4", "\u59A4"], /* CJK Ideograph */
  ["\u59A5", "\u59A5"], /* CJK Ideograph */
  ["\u59A6", "\u59A6"], /* CJK Ideograph */
  ["\u59A7", "\u59A7"], /* CJK Ideograph */
  ["\u59A8", "\u59A8"], /* CJK Ideograph */
  ["\u59A9", "\u59A9"], /* CJK Ideograph */
  ["\u59AA", "\u59AA"], /* CJK Ideograph */
  ["\u59AB", "\u59AB"], /* CJK Ideograph */
  ["\u59AC", "\u59AC"], /* CJK Ideograph */
  ["\u59AD", "\u59AD"], /* CJK Ideograph */
  ["\u59AE", "\u59AE"], /* CJK Ideograph */
  ["\u59AF", "\u59AF"], /* CJK Ideograph */
  ["\u59B0", "\u59B0"], /* CJK Ideograph */
  ["\u59B1", "\u59B1"], /* CJK Ideograph */
  ["\u59B2", "\u59B2"], /* CJK Ideograph */
  ["\u59B3", "\u59B3"], /* CJK Ideograph */
  ["\u59B4", "\u59B4"], /* CJK Ideograph */
  ["\u59B5", "\u59B5"], /* CJK Ideograph */
  ["\u59B6", "\u59B6"], /* CJK Ideograph */
  ["\u59B7", "\u59B7"], /* CJK Ideograph */
  ["\u59B8", "\u59B8"], /* CJK Ideograph */
  ["\u59B9", "\u59B9"], /* CJK Ideograph */
  ["\u59BA", "\u59BA"], /* CJK Ideograph */
  ["\u59BB", "\u59BB"], /* CJK Ideograph */
  ["\u59BC", "\u59BC"], /* CJK Ideograph */
  ["\u59BD", "\u59BD"], /* CJK Ideograph */
  ["\u59BE", "\u59BE"], /* CJK Ideograph */
  ["\u59BF", "\u59BF"], /* CJK Ideograph */
  ["\u59C0", "\u59C0"], /* CJK Ideograph */
  ["\u59C1", "\u59C1"], /* CJK Ideograph */
  ["\u59C2", "\u59C2"], /* CJK Ideograph */
  ["\u59C3", "\u59C3"], /* CJK Ideograph */
  ["\u59C4", "\u59C4"], /* CJK Ideograph */
  ["\u59C5", "\u59C5"], /* CJK Ideograph */
  ["\u59C6", "\u59C6"], /* CJK Ideograph */
  ["\u59C7", "\u59C7"], /* CJK Ideograph */
  ["\u59C8", "\u59C8"], /* CJK Ideograph */
  ["\u59C9", "\u59C9"], /* CJK Ideograph */
  ["\u59CA", "\u59CA"], /* CJK Ideograph */
  ["\u59CB", "\u59CB"], /* CJK Ideograph */
  ["\u59CC", "\u59CC"], /* CJK Ideograph */
  ["\u59CD", "\u59CD"], /* CJK Ideograph */
  ["\u59CE", "\u59CE"], /* CJK Ideograph */
  ["\u59CF", "\u59CF"], /* CJK Ideograph */
  ["\u59D0", "\u59D0"], /* CJK Ideograph */
  ["\u59D1", "\u59D1"], /* CJK Ideograph */
  ["\u59D2", "\u59D2"], /* CJK Ideograph */
  ["\u59D3", "\u59D3"], /* CJK Ideograph */
  ["\u59D4", "\u59D4"], /* CJK Ideograph */
  ["\u59D5", "\u59D5"], /* CJK Ideograph */
  ["\u59D6", "\u59D6"], /* CJK Ideograph */
  ["\u59D7", "\u59D7"], /* CJK Ideograph */
  ["\u59D8", "\u59D8"], /* CJK Ideograph */
  ["\u59D9", "\u59D9"], /* CJK Ideograph */
  ["\u59DA", "\u59DA"], /* CJK Ideograph */
  ["\u59DB", "\u59DB"], /* CJK Ideograph */
  ["\u59DC", "\u59DC"], /* CJK Ideograph */
  ["\u59DD", "\u59DD"], /* CJK Ideograph */
  ["\u59DE", "\u59DE"], /* CJK Ideograph */
  ["\u59DF", "\u59DF"], /* CJK Ideograph */
  ["\u59E0", "\u59E0"], /* CJK Ideograph */
  ["\u59E1", "\u59E1"], /* CJK Ideograph */
  ["\u59E2", "\u59E2"], /* CJK Ideograph */
  ["\u59E3", "\u59E3"], /* CJK Ideograph */
  ["\u59E4", "\u59E4"], /* CJK Ideograph */
  ["\u59E5", "\u59E5"], /* CJK Ideograph */
  ["\u59E6", "\u59E6"], /* CJK Ideograph */
  ["\u59E7", "\u59E7"], /* CJK Ideograph */
  ["\u59E8", "\u59E8"], /* CJK Ideograph */
  ["\u59E9", "\u59E9"], /* CJK Ideograph */
  ["\u59EA", "\u59EA"], /* CJK Ideograph */
  ["\u59EB", "\u59EB"], /* CJK Ideograph */
  ["\u59EC", "\u59EC"], /* CJK Ideograph */
  ["\u59ED", "\u59ED"], /* CJK Ideograph */
  ["\u59EE", "\u59EE"], /* CJK Ideograph */
  ["\u59EF", "\u59EF"], /* CJK Ideograph */
  ["\u59F0", "\u59F0"], /* CJK Ideograph */
  ["\u59F1", "\u59F1"], /* CJK Ideograph */
  ["\u59F2", "\u59F2"], /* CJK Ideograph */
  ["\u59F3", "\u59F3"], /* CJK Ideograph */
  ["\u59F4", "\u59F4"], /* CJK Ideograph */
  ["\u59F5", "\u59F5"], /* CJK Ideograph */
  ["\u59F6", "\u59F6"], /* CJK Ideograph */
  ["\u59F7", "\u59F7"], /* CJK Ideograph */
  ["\u59F8", "\u59F8"], /* CJK Ideograph */
  ["\u59F9", "\u59F9"], /* CJK Ideograph */
  ["\u59FA", "\u59FA"], /* CJK Ideograph */
  ["\u59FB", "\u59FB"], /* CJK Ideograph */
  ["\u59FC", "\u59FC"], /* CJK Ideograph */
  ["\u59FD", "\u59FD"], /* CJK Ideograph */
  ["\u59FE", "\u59FE"], /* CJK Ideograph */
  ["\u59FF", "\u59FF"], /* CJK Ideograph */
  ["\u5A00", "\u5A00"], /* CJK Ideograph */
  ["\u5A01", "\u5A01"], /* CJK Ideograph */
  ["\u5A02", "\u5A02"], /* CJK Ideograph */
  ["\u5A03", "\u5A03"], /* CJK Ideograph */
  ["\u5A04", "\u5A04"], /* CJK Ideograph */
  ["\u5A05", "\u5A05"], /* CJK Ideograph */
  ["\u5A06", "\u5A06"], /* CJK Ideograph */
  ["\u5A07", "\u5A07"], /* CJK Ideograph */
  ["\u5A08", "\u5A08"], /* CJK Ideograph */
  ["\u5A09", "\u5A09"], /* CJK Ideograph */
  ["\u5A0A", "\u5A0A"], /* CJK Ideograph */
  ["\u5A0B", "\u5A0B"], /* CJK Ideograph */
  ["\u5A0C", "\u5A0C"], /* CJK Ideograph */
  ["\u5A0D", "\u5A0D"], /* CJK Ideograph */
  ["\u5A0E", "\u5A0E"], /* CJK Ideograph */
  ["\u5A0F", "\u5A0F"], /* CJK Ideograph */
  ["\u5A10", "\u5A10"], /* CJK Ideograph */
  ["\u5A11", "\u5A11"], /* CJK Ideograph */
  ["\u5A12", "\u5A12"], /* CJK Ideograph */
  ["\u5A13", "\u5A13"], /* CJK Ideograph */
  ["\u5A14", "\u5A14"], /* CJK Ideograph */
  ["\u5A15", "\u5A15"], /* CJK Ideograph */
  ["\u5A16", "\u5A16"], /* CJK Ideograph */
  ["\u5A17", "\u5A17"], /* CJK Ideograph */
  ["\u5A18", "\u5A18"], /* CJK Ideograph */
  ["\u5A19", "\u5A19"], /* CJK Ideograph */
  ["\u5A1A", "\u5A1A"], /* CJK Ideograph */
  ["\u5A1B", "\u5A1B"], /* CJK Ideograph */
  ["\u5A1C", "\u5A1C"], /* CJK Ideograph */
  ["\u5A1D", "\u5A1D"], /* CJK Ideograph */
  ["\u5A1E", "\u5A1E"], /* CJK Ideograph */
  ["\u5A1F", "\u5A1F"], /* CJK Ideograph */
  ["\u5A20", "\u5A20"], /* CJK Ideograph */
  ["\u5A21", "\u5A21"], /* CJK Ideograph */
  ["\u5A22", "\u5A22"], /* CJK Ideograph */
  ["\u5A23", "\u5A23"], /* CJK Ideograph */
  ["\u5A24", "\u5A24"], /* CJK Ideograph */
  ["\u5A25", "\u5A25"], /* CJK Ideograph */
  ["\u5A26", "\u5A26"], /* CJK Ideograph */
  ["\u5A27", "\u5A27"], /* CJK Ideograph */
  ["\u5A28", "\u5A28"], /* CJK Ideograph */
  ["\u5A29", "\u5A29"], /* CJK Ideograph */
  ["\u5A2A", "\u5A2A"], /* CJK Ideograph */
  ["\u5A2B", "\u5A2B"], /* CJK Ideograph */
  ["\u5A2C", "\u5A2C"], /* CJK Ideograph */
  ["\u5A2D", "\u5A2D"], /* CJK Ideograph */
  ["\u5A2E", "\u5A2E"], /* CJK Ideograph */
  ["\u5A2F", "\u5A2F"], /* CJK Ideograph */
  ["\u5A30", "\u5A30"], /* CJK Ideograph */
  ["\u5A31", "\u5A31"], /* CJK Ideograph */
  ["\u5A32", "\u5A32"], /* CJK Ideograph */
  ["\u5A33", "\u5A33"], /* CJK Ideograph */
  ["\u5A34", "\u5A34"], /* CJK Ideograph */
  ["\u5A35", "\u5A35"], /* CJK Ideograph */
  ["\u5A36", "\u5A36"], /* CJK Ideograph */
  ["\u5A37", "\u5A37"], /* CJK Ideograph */
  ["\u5A38", "\u5A38"], /* CJK Ideograph */
  ["\u5A39", "\u5A39"], /* CJK Ideograph */
  ["\u5A3A", "\u5A3A"], /* CJK Ideograph */
  ["\u5A3B", "\u5A3B"], /* CJK Ideograph */
  ["\u5A3C", "\u5A3C"], /* CJK Ideograph */
  ["\u5A3D", "\u5A3D"], /* CJK Ideograph */
  ["\u5A3E", "\u5A3E"], /* CJK Ideograph */
  ["\u5A3F", "\u5A3F"], /* CJK Ideograph */
  ["\u5A40", "\u5A40"], /* CJK Ideograph */
  ["\u5A41", "\u5A41"], /* CJK Ideograph */
  ["\u5A42", "\u5A42"], /* CJK Ideograph */
  ["\u5A43", "\u5A43"], /* CJK Ideograph */
  ["\u5A44", "\u5A44"], /* CJK Ideograph */
  ["\u5A45", "\u5A45"], /* CJK Ideograph */
  ["\u5A46", "\u5A46"], /* CJK Ideograph */
  ["\u5A47", "\u5A47"], /* CJK Ideograph */
  ["\u5A48", "\u5A48"], /* CJK Ideograph */
  ["\u5A49", "\u5A49"], /* CJK Ideograph */
  ["\u5A4A", "\u5A4A"], /* CJK Ideograph */
  ["\u5A4B", "\u5A4B"], /* CJK Ideograph */
  ["\u5A4C", "\u5A4C"], /* CJK Ideograph */
  ["\u5A4D", "\u5A4D"], /* CJK Ideograph */
  ["\u5A4E", "\u5A4E"], /* CJK Ideograph */
  ["\u5A4F", "\u5A4F"], /* CJK Ideograph */
  ["\u5A50", "\u5A50"], /* CJK Ideograph */
  ["\u5A51", "\u5A51"], /* CJK Ideograph */
  ["\u5A52", "\u5A52"], /* CJK Ideograph */
  ["\u5A53", "\u5A53"], /* CJK Ideograph */
  ["\u5A54", "\u5A54"], /* CJK Ideograph */
  ["\u5A55", "\u5A55"], /* CJK Ideograph */
  ["\u5A56", "\u5A56"], /* CJK Ideograph */
  ["\u5A57", "\u5A57"], /* CJK Ideograph */
  ["\u5A58", "\u5A58"], /* CJK Ideograph */
  ["\u5A59", "\u5A59"], /* CJK Ideograph */
  ["\u5A5A", "\u5A5A"], /* CJK Ideograph */
  ["\u5A5B", "\u5A5B"], /* CJK Ideograph */
  ["\u5A5C", "\u5A5C"], /* CJK Ideograph */
  ["\u5A5D", "\u5A5D"], /* CJK Ideograph */
  ["\u5A5E", "\u5A5E"], /* CJK Ideograph */
  ["\u5A5F", "\u5A5F"], /* CJK Ideograph */
  ["\u5A60", "\u5A60"], /* CJK Ideograph */
  ["\u5A61", "\u5A61"], /* CJK Ideograph */
  ["\u5A62", "\u5A62"], /* CJK Ideograph */
  ["\u5A63", "\u5A63"], /* CJK Ideograph */
  ["\u5A64", "\u5A64"], /* CJK Ideograph */
  ["\u5A65", "\u5A65"], /* CJK Ideograph */
  ["\u5A66", "\u5A66"], /* CJK Ideograph */
  ["\u5A67", "\u5A67"], /* CJK Ideograph */
  ["\u5A68", "\u5A68"], /* CJK Ideograph */
  ["\u5A69", "\u5A69"], /* CJK Ideograph */
  ["\u5A6A", "\u5A6A"], /* CJK Ideograph */
  ["\u5A6B", "\u5A6B"], /* CJK Ideograph */
  ["\u5A6C", "\u5A6C"], /* CJK Ideograph */
  ["\u5A6D", "\u5A6D"], /* CJK Ideograph */
  ["\u5A6E", "\u5A6E"], /* CJK Ideograph */
  ["\u5A6F", "\u5A6F"], /* CJK Ideograph */
  ["\u5A70", "\u5A70"], /* CJK Ideograph */
  ["\u5A71", "\u5A71"], /* CJK Ideograph */
  ["\u5A72", "\u5A72"], /* CJK Ideograph */
  ["\u5A73", "\u5A73"], /* CJK Ideograph */
  ["\u5A74", "\u5A74"], /* CJK Ideograph */
  ["\u5A75", "\u5A75"], /* CJK Ideograph */
  ["\u5A76", "\u5A76"], /* CJK Ideograph */
  ["\u5A77", "\u5A77"], /* CJK Ideograph */
  ["\u5A78", "\u5A78"], /* CJK Ideograph */
  ["\u5A79", "\u5A79"], /* CJK Ideograph */
  ["\u5A7A", "\u5A7A"], /* CJK Ideograph */
  ["\u5A7B", "\u5A7B"], /* CJK Ideograph */
  ["\u5A7C", "\u5A7C"], /* CJK Ideograph */
  ["\u5A7D", "\u5A7D"], /* CJK Ideograph */
  ["\u5A7E", "\u5A7E"], /* CJK Ideograph */
  ["\u5A7F", "\u5A7F"], /* CJK Ideograph */
  ["\u5A80", "\u5A80"], /* CJK Ideograph */
  ["\u5A81", "\u5A81"], /* CJK Ideograph */
  ["\u5A82", "\u5A82"], /* CJK Ideograph */
  ["\u5A83", "\u5A83"], /* CJK Ideograph */
  ["\u5A84", "\u5A84"], /* CJK Ideograph */
  ["\u5A85", "\u5A85"], /* CJK Ideograph */
  ["\u5A86", "\u5A86"], /* CJK Ideograph */
  ["\u5A87", "\u5A87"], /* CJK Ideograph */
  ["\u5A88", "\u5A88"], /* CJK Ideograph */
  ["\u5A89", "\u5A89"], /* CJK Ideograph */
  ["\u5A8A", "\u5A8A"], /* CJK Ideograph */
  ["\u5A8B", "\u5A8B"], /* CJK Ideograph */
  ["\u5A8C", "\u5A8C"], /* CJK Ideograph */
  ["\u5A8D", "\u5A8D"], /* CJK Ideograph */
  ["\u5A8E", "\u5A8E"], /* CJK Ideograph */
  ["\u5A8F", "\u5A8F"], /* CJK Ideograph */
  ["\u5A90", "\u5A90"], /* CJK Ideograph */
  ["\u5A91", "\u5A91"], /* CJK Ideograph */
  ["\u5A92", "\u5A92"], /* CJK Ideograph */
  ["\u5A93", "\u5A93"], /* CJK Ideograph */
  ["\u5A94", "\u5A94"], /* CJK Ideograph */
  ["\u5A95", "\u5A95"], /* CJK Ideograph */
  ["\u5A96", "\u5A96"], /* CJK Ideograph */
  ["\u5A97", "\u5A97"], /* CJK Ideograph */
  ["\u5A98", "\u5A98"], /* CJK Ideograph */
  ["\u5A99", "\u5A99"], /* CJK Ideograph */
  ["\u5A9A", "\u5A9A"], /* CJK Ideograph */
  ["\u5A9B", "\u5A9B"], /* CJK Ideograph */
  ["\u5A9C", "\u5A9C"], /* CJK Ideograph */
  ["\u5A9D", "\u5A9D"], /* CJK Ideograph */
  ["\u5A9E", "\u5A9E"], /* CJK Ideograph */
  ["\u5A9F", "\u5A9F"], /* CJK Ideograph */
  ["\u5AA0", "\u5AA0"], /* CJK Ideograph */
  ["\u5AA1", "\u5AA1"], /* CJK Ideograph */
  ["\u5AA2", "\u5AA2"], /* CJK Ideograph */
  ["\u5AA3", "\u5AA3"], /* CJK Ideograph */
  ["\u5AA4", "\u5AA4"], /* CJK Ideograph */
  ["\u5AA5", "\u5AA5"], /* CJK Ideograph */
  ["\u5AA6", "\u5AA6"], /* CJK Ideograph */
  ["\u5AA7", "\u5AA7"], /* CJK Ideograph */
  ["\u5AA8", "\u5AA8"], /* CJK Ideograph */
  ["\u5AA9", "\u5AA9"], /* CJK Ideograph */
  ["\u5AAA", "\u5AAA"], /* CJK Ideograph */
  ["\u5AAB", "\u5AAB"], /* CJK Ideograph */
  ["\u5AAC", "\u5AAC"], /* CJK Ideograph */
  ["\u5AAD", "\u5AAD"], /* CJK Ideograph */
  ["\u5AAE", "\u5AAE"], /* CJK Ideograph */
  ["\u5AAF", "\u5AAF"], /* CJK Ideograph */
  ["\u5AB0", "\u5AB0"], /* CJK Ideograph */
  ["\u5AB1", "\u5AB1"], /* CJK Ideograph */
  ["\u5AB2", "\u5AB2"], /* CJK Ideograph */
  ["\u5AB3", "\u5AB3"], /* CJK Ideograph */
  ["\u5AB4", "\u5AB4"], /* CJK Ideograph */
  ["\u5AB5", "\u5AB5"], /* CJK Ideograph */
  ["\u5AB6", "\u5AB6"], /* CJK Ideograph */
  ["\u5AB7", "\u5AB7"], /* CJK Ideograph */
  ["\u5AB8", "\u5AB8"], /* CJK Ideograph */
  ["\u5AB9", "\u5AB9"], /* CJK Ideograph */
  ["\u5ABA", "\u5ABA"], /* CJK Ideograph */
  ["\u5ABB", "\u5ABB"], /* CJK Ideograph */
  ["\u5ABC", "\u5ABC"], /* CJK Ideograph */
  ["\u5ABD", "\u5ABD"], /* CJK Ideograph */
  ["\u5ABE", "\u5ABE"], /* CJK Ideograph */
  ["\u5ABF", "\u5ABF"], /* CJK Ideograph */
  ["\u5AC0", "\u5AC0"], /* CJK Ideograph */
  ["\u5AC1", "\u5AC1"], /* CJK Ideograph */
  ["\u5AC2", "\u5AC2"], /* CJK Ideograph */
  ["\u5AC3", "\u5AC3"], /* CJK Ideograph */
  ["\u5AC4", "\u5AC4"], /* CJK Ideograph */
  ["\u5AC5", "\u5AC5"], /* CJK Ideograph */
  ["\u5AC6", "\u5AC6"], /* CJK Ideograph */
  ["\u5AC7", "\u5AC7"], /* CJK Ideograph */
  ["\u5AC8", "\u5AC8"], /* CJK Ideograph */
  ["\u5AC9", "\u5AC9"], /* CJK Ideograph */
  ["\u5ACA", "\u5ACA"], /* CJK Ideograph */
  ["\u5ACB", "\u5ACB"], /* CJK Ideograph */
  ["\u5ACC", "\u5ACC"], /* CJK Ideograph */
  ["\u5ACD", "\u5ACD"], /* CJK Ideograph */
  ["\u5ACE", "\u5ACE"], /* CJK Ideograph */
  ["\u5ACF", "\u5ACF"], /* CJK Ideograph */
  ["\u5AD0", "\u5AD0"], /* CJK Ideograph */
  ["\u5AD1", "\u5AD1"], /* CJK Ideograph */
  ["\u5AD2", "\u5AD2"], /* CJK Ideograph */
  ["\u5AD3", "\u5AD3"], /* CJK Ideograph */
  ["\u5AD4", "\u5AD4"], /* CJK Ideograph */
  ["\u5AD5", "\u5AD5"], /* CJK Ideograph */
  ["\u5AD6", "\u5AD6"], /* CJK Ideograph */
  ["\u5AD7", "\u5AD7"], /* CJK Ideograph */
  ["\u5AD8", "\u5AD8"], /* CJK Ideograph */
  ["\u5AD9", "\u5AD9"], /* CJK Ideograph */
  ["\u5ADA", "\u5ADA"], /* CJK Ideograph */
  ["\u5ADB", "\u5ADB"], /* CJK Ideograph */
  ["\u5ADC", "\u5ADC"], /* CJK Ideograph */
  ["\u5ADD", "\u5ADD"], /* CJK Ideograph */
  ["\u5ADE", "\u5ADE"], /* CJK Ideograph */
  ["\u5ADF", "\u5ADF"], /* CJK Ideograph */
  ["\u5AE0", "\u5AE0"], /* CJK Ideograph */
  ["\u5AE1", "\u5AE1"], /* CJK Ideograph */
  ["\u5AE2", "\u5AE2"], /* CJK Ideograph */
  ["\u5AE3", "\u5AE3"], /* CJK Ideograph */
  ["\u5AE4", "\u5AE4"], /* CJK Ideograph */
  ["\u5AE5", "\u5AE5"], /* CJK Ideograph */
  ["\u5AE6", "\u5AE6"], /* CJK Ideograph */
  ["\u5AE7", "\u5AE7"], /* CJK Ideograph */
  ["\u5AE8", "\u5AE8"], /* CJK Ideograph */
  ["\u5AE9", "\u5AE9"], /* CJK Ideograph */
  ["\u5AEA", "\u5AEA"], /* CJK Ideograph */
  ["\u5AEB", "\u5AEB"], /* CJK Ideograph */
  ["\u5AEC", "\u5AEC"], /* CJK Ideograph */
  ["\u5AED", "\u5AED"], /* CJK Ideograph */
  ["\u5AEE", "\u5AEE"], /* CJK Ideograph */
  ["\u5AEF", "\u5AEF"], /* CJK Ideograph */
  ["\u5AF0", "\u5AF0"], /* CJK Ideograph */
  ["\u5AF1", "\u5AF1"], /* CJK Ideograph */
  ["\u5AF2", "\u5AF2"], /* CJK Ideograph */
  ["\u5AF3", "\u5AF3"], /* CJK Ideograph */
  ["\u5AF4", "\u5AF4"], /* CJK Ideograph */
  ["\u5AF5", "\u5AF5"], /* CJK Ideograph */
  ["\u5AF6", "\u5AF6"], /* CJK Ideograph */
  ["\u5AF7", "\u5AF7"], /* CJK Ideograph */
  ["\u5AF8", "\u5AF8"], /* CJK Ideograph */
  ["\u5AF9", "\u5AF9"], /* CJK Ideograph */
  ["\u5AFA", "\u5AFA"], /* CJK Ideograph */
  ["\u5AFB", "\u5AFB"], /* CJK Ideograph */
  ["\u5AFC", "\u5AFC"], /* CJK Ideograph */
  ["\u5AFD", "\u5AFD"], /* CJK Ideograph */
  ["\u5AFE", "\u5AFE"], /* CJK Ideograph */
  ["\u5AFF", "\u5AFF"], /* CJK Ideograph */
  ["\u5B00", "\u5B00"], /* CJK Ideograph */
  ["\u5B01", "\u5B01"], /* CJK Ideograph */
  ["\u5B02", "\u5B02"], /* CJK Ideograph */
  ["\u5B03", "\u5B03"], /* CJK Ideograph */
  ["\u5B04", "\u5B04"], /* CJK Ideograph */
  ["\u5B05", "\u5B05"], /* CJK Ideograph */
  ["\u5B06", "\u5B06"], /* CJK Ideograph */
  ["\u5B07", "\u5B07"], /* CJK Ideograph */
  ["\u5B08", "\u5B08"], /* CJK Ideograph */
  ["\u5B09", "\u5B09"], /* CJK Ideograph */
  ["\u5B0A", "\u5B0A"], /* CJK Ideograph */
  ["\u5B0B", "\u5B0B"], /* CJK Ideograph */
  ["\u5B0C", "\u5B0C"], /* CJK Ideograph */
  ["\u5B0D", "\u5B0D"], /* CJK Ideograph */
  ["\u5B0E", "\u5B0E"], /* CJK Ideograph */
  ["\u5B0F", "\u5B0F"], /* CJK Ideograph */
  ["\u5B10", "\u5B10"], /* CJK Ideograph */
  ["\u5B11", "\u5B11"], /* CJK Ideograph */
  ["\u5B12", "\u5B12"], /* CJK Ideograph */
  ["\u5B13", "\u5B13"], /* CJK Ideograph */
  ["\u5B14", "\u5B14"], /* CJK Ideograph */
  ["\u5B15", "\u5B15"], /* CJK Ideograph */
  ["\u5B16", "\u5B16"], /* CJK Ideograph */
  ["\u5B17", "\u5B17"], /* CJK Ideograph */
  ["\u5B18", "\u5B18"], /* CJK Ideograph */
  ["\u5B19", "\u5B19"], /* CJK Ideograph */
  ["\u5B1A", "\u5B1A"], /* CJK Ideograph */
  ["\u5B1B", "\u5B1B"], /* CJK Ideograph */
  ["\u5B1C", "\u5B1C"], /* CJK Ideograph */
  ["\u5B1D", "\u5B1D"], /* CJK Ideograph */
  ["\u5B1E", "\u5B1E"], /* CJK Ideograph */
  ["\u5B1F", "\u5B1F"], /* CJK Ideograph */
  ["\u5B20", "\u5B20"], /* CJK Ideograph */
  ["\u5B21", "\u5B21"], /* CJK Ideograph */
  ["\u5B22", "\u5B22"], /* CJK Ideograph */
  ["\u5B23", "\u5B23"], /* CJK Ideograph */
  ["\u5B24", "\u5B24"], /* CJK Ideograph */
  ["\u5B25", "\u5B25"], /* CJK Ideograph */
  ["\u5B26", "\u5B26"], /* CJK Ideograph */
  ["\u5B27", "\u5B27"], /* CJK Ideograph */
  ["\u5B28", "\u5B28"], /* CJK Ideograph */
  ["\u5B29", "\u5B29"], /* CJK Ideograph */
  ["\u5B2A", "\u5B2A"], /* CJK Ideograph */
  ["\u5B2B", "\u5B2B"], /* CJK Ideograph */
  ["\u5B2C", "\u5B2C"], /* CJK Ideograph */
  ["\u5B2D", "\u5B2D"], /* CJK Ideograph */
  ["\u5B2E", "\u5B2E"], /* CJK Ideograph */
  ["\u5B2F", "\u5B2F"], /* CJK Ideograph */
  ["\u5B30", "\u5B30"], /* CJK Ideograph */
  ["\u5B31", "\u5B31"], /* CJK Ideograph */
  ["\u5B32", "\u5B32"], /* CJK Ideograph */
  ["\u5B33", "\u5B33"], /* CJK Ideograph */
  ["\u5B34", "\u5B34"], /* CJK Ideograph */
  ["\u5B35", "\u5B35"], /* CJK Ideograph */
  ["\u5B36", "\u5B36"], /* CJK Ideograph */
  ["\u5B37", "\u5B37"], /* CJK Ideograph */
  ["\u5B38", "\u5B38"], /* CJK Ideograph */
  ["\u5B39", "\u5B39"], /* CJK Ideograph */
  ["\u5B3A", "\u5B3A"], /* CJK Ideograph */
  ["\u5B3B", "\u5B3B"], /* CJK Ideograph */
  ["\u5B3C", "\u5B3C"], /* CJK Ideograph */
  ["\u5B3D", "\u5B3D"], /* CJK Ideograph */
  ["\u5B3E", "\u5B3E"], /* CJK Ideograph */
  ["\u5B3F", "\u5B3F"], /* CJK Ideograph */
  ["\u5B40", "\u5B40"], /* CJK Ideograph */
  ["\u5B41", "\u5B41"], /* CJK Ideograph */
  ["\u5B42", "\u5B42"], /* CJK Ideograph */
  ["\u5B43", "\u5B43"], /* CJK Ideograph */
  ["\u5B44", "\u5B44"], /* CJK Ideograph */
  ["\u5B45", "\u5B45"], /* CJK Ideograph */
  ["\u5B46", "\u5B46"], /* CJK Ideograph */
  ["\u5B47", "\u5B47"], /* CJK Ideograph */
  ["\u5B48", "\u5B48"], /* CJK Ideograph */
  ["\u5B49", "\u5B49"], /* CJK Ideograph */
  ["\u5B4A", "\u5B4A"], /* CJK Ideograph */
  ["\u5B4B", "\u5B4B"], /* CJK Ideograph */
  ["\u5B4C", "\u5B4C"], /* CJK Ideograph */
  ["\u5B4D", "\u5B4D"], /* CJK Ideograph */
  ["\u5B4E", "\u5B4E"], /* CJK Ideograph */
  ["\u5B4F", "\u5B4F"], /* CJK Ideograph */
  ["\u5B50", "\u5B50"], /* CJK Ideograph */
  ["\u5B51", "\u5B51"], /* CJK Ideograph */
  ["\u5B52", "\u5B52"], /* CJK Ideograph */
  ["\u5B53", "\u5B53"], /* CJK Ideograph */
  ["\u5B54", "\u5B54"], /* CJK Ideograph */
  ["\u5B55", "\u5B55"], /* CJK Ideograph */
  ["\u5B56", "\u5B56"], /* CJK Ideograph */
  ["\u5B57", "\u5B57"], /* CJK Ideograph */
  ["\u5B58", "\u5B58"], /* CJK Ideograph */
  ["\u5B59", "\u5B59"], /* CJK Ideograph */
  ["\u5B5A", "\u5B5A"], /* CJK Ideograph */
  ["\u5B5B", "\u5B5B"], /* CJK Ideograph */
  ["\u5B5C", "\u5B5C"], /* CJK Ideograph */
  ["\u5B5D", "\u5B5D"], /* CJK Ideograph */
  ["\u5B5E", "\u5B5E"], /* CJK Ideograph */
  ["\u5B5F", "\u5B5F"], /* CJK Ideograph */
  ["\u5B60", "\u5B60"], /* CJK Ideograph */
  ["\u5B61", "\u5B61"], /* CJK Ideograph */
  ["\u5B62", "\u5B62"], /* CJK Ideograph */
  ["\u5B63", "\u5B63"], /* CJK Ideograph */
  ["\u5B64", "\u5B64"], /* CJK Ideograph */
  ["\u5B65", "\u5B65"], /* CJK Ideograph */
  ["\u5B66", "\u5B66"], /* CJK Ideograph */
  ["\u5B67", "\u5B67"], /* CJK Ideograph */
  ["\u5B68", "\u5B68"], /* CJK Ideograph */
  ["\u5B69", "\u5B69"], /* CJK Ideograph */
  ["\u5B6A", "\u5B6A"], /* CJK Ideograph */
  ["\u5B6B", "\u5B6B"], /* CJK Ideograph */
  ["\u5B6C", "\u5B6C"], /* CJK Ideograph */
  ["\u5B6D", "\u5B6D"], /* CJK Ideograph */
  ["\u5B6E", "\u5B6E"], /* CJK Ideograph */
  ["\u5B6F", "\u5B6F"], /* CJK Ideograph */
  ["\u5B70", "\u5B70"], /* CJK Ideograph */
  ["\u5B71", "\u5B71"], /* CJK Ideograph */
  ["\u5B72", "\u5B72"], /* CJK Ideograph */
  ["\u5B73", "\u5B73"], /* CJK Ideograph */
  ["\u5B74", "\u5B74"], /* CJK Ideograph */
  ["\u5B75", "\u5B75"], /* CJK Ideograph */
  ["\u5B76", "\u5B76"], /* CJK Ideograph */
  ["\u5B77", "\u5B77"], /* CJK Ideograph */
  ["\u5B78", "\u5B78"], /* CJK Ideograph */
  ["\u5B79", "\u5B79"], /* CJK Ideograph */
  ["\u5B7A", "\u5B7A"], /* CJK Ideograph */
  ["\u5B7B", "\u5B7B"], /* CJK Ideograph */
  ["\u5B7C", "\u5B7C"], /* CJK Ideograph */
  ["\u5B7D", "\u5B7D"], /* CJK Ideograph */
  ["\u5B7E", "\u5B7E"], /* CJK Ideograph */
  ["\u5B7F", "\u5B7F"], /* CJK Ideograph */
  ["\u5B80", "\u5B80"], /* CJK Ideograph */
  ["\u5B81", "\u5B81"], /* CJK Ideograph */
  ["\u5B82", "\u5B82"], /* CJK Ideograph */
  ["\u5B83", "\u5B83"], /* CJK Ideograph */
  ["\u5B84", "\u5B84"], /* CJK Ideograph */
  ["\u5B85", "\u5B85"], /* CJK Ideograph */
  ["\u5B86", "\u5B86"], /* CJK Ideograph */
  ["\u5B87", "\u5B87"], /* CJK Ideograph */
  ["\u5B88", "\u5B88"], /* CJK Ideograph */
  ["\u5B89", "\u5B89"], /* CJK Ideograph */
  ["\u5B8A", "\u5B8A"], /* CJK Ideograph */
  ["\u5B8B", "\u5B8B"], /* CJK Ideograph */
  ["\u5B8C", "\u5B8C"], /* CJK Ideograph */
  ["\u5B8D", "\u5B8D"], /* CJK Ideograph */
  ["\u5B8E", "\u5B8E"], /* CJK Ideograph */
  ["\u5B8F", "\u5B8F"], /* CJK Ideograph */
  ["\u5B90", "\u5B90"], /* CJK Ideograph */
  ["\u5B91", "\u5B91"], /* CJK Ideograph */
  ["\u5B92", "\u5B92"], /* CJK Ideograph */
  ["\u5B93", "\u5B93"], /* CJK Ideograph */
  ["\u5B94", "\u5B94"], /* CJK Ideograph */
  ["\u5B95", "\u5B95"], /* CJK Ideograph */
  ["\u5B96", "\u5B96"], /* CJK Ideograph */
  ["\u5B97", "\u5B97"], /* CJK Ideograph */
  ["\u5B98", "\u5B98"], /* CJK Ideograph */
  ["\u5B99", "\u5B99"], /* CJK Ideograph */
  ["\u5B9A", "\u5B9A"], /* CJK Ideograph */
  ["\u5B9B", "\u5B9B"], /* CJK Ideograph */
  ["\u5B9C", "\u5B9C"], /* CJK Ideograph */
  ["\u5B9D", "\u5B9D"], /* CJK Ideograph */
  ["\u5B9E", "\u5B9E"], /* CJK Ideograph */
  ["\u5B9F", "\u5B9F"], /* CJK Ideograph */
  ["\u5BA0", "\u5BA0"], /* CJK Ideograph */
  ["\u5BA1", "\u5BA1"], /* CJK Ideograph */
  ["\u5BA2", "\u5BA2"], /* CJK Ideograph */
  ["\u5BA3", "\u5BA3"], /* CJK Ideograph */
  ["\u5BA4", "\u5BA4"], /* CJK Ideograph */
  ["\u5BA5", "\u5BA5"], /* CJK Ideograph */
  ["\u5BA6", "\u5BA6"], /* CJK Ideograph */
  ["\u5BA7", "\u5BA7"], /* CJK Ideograph */
  ["\u5BA8", "\u5BA8"], /* CJK Ideograph */
  ["\u5BA9", "\u5BA9"], /* CJK Ideograph */
  ["\u5BAA", "\u5BAA"], /* CJK Ideograph */
  ["\u5BAB", "\u5BAB"], /* CJK Ideograph */
  ["\u5BAC", "\u5BAC"], /* CJK Ideograph */
  ["\u5BAD", "\u5BAD"], /* CJK Ideograph */
  ["\u5BAE", "\u5BAE"], /* CJK Ideograph */
  ["\u5BAF", "\u5BAF"], /* CJK Ideograph */
  ["\u5BB0", "\u5BB0"], /* CJK Ideograph */
  ["\u5BB1", "\u5BB1"], /* CJK Ideograph */
  ["\u5BB2", "\u5BB2"], /* CJK Ideograph */
  ["\u5BB3", "\u5BB3"], /* CJK Ideograph */
  ["\u5BB4", "\u5BB4"], /* CJK Ideograph */
  ["\u5BB5", "\u5BB5"], /* CJK Ideograph */
  ["\u5BB6", "\u5BB6"], /* CJK Ideograph */
  ["\u5BB7", "\u5BB7"], /* CJK Ideograph */
  ["\u5BB8", "\u5BB8"], /* CJK Ideograph */
  ["\u5BB9", "\u5BB9"], /* CJK Ideograph */
  ["\u5BBA", "\u5BBA"], /* CJK Ideograph */
  ["\u5BBB", "\u5BBB"], /* CJK Ideograph */
  ["\u5BBC", "\u5BBC"], /* CJK Ideograph */
  ["\u5BBD", "\u5BBD"], /* CJK Ideograph */
  ["\u5BBE", "\u5BBE"], /* CJK Ideograph */
  ["\u5BBF", "\u5BBF"], /* CJK Ideograph */
  ["\u5BC0", "\u5BC0"], /* CJK Ideograph */
  ["\u5BC1", "\u5BC1"], /* CJK Ideograph */
  ["\u5BC2", "\u5BC2"], /* CJK Ideograph */
  ["\u5BC3", "\u5BC3"], /* CJK Ideograph */
  ["\u5BC4", "\u5BC4"], /* CJK Ideograph */
  ["\u5BC5", "\u5BC5"], /* CJK Ideograph */
  ["\u5BC6", "\u5BC6"], /* CJK Ideograph */
  ["\u5BC7", "\u5BC7"], /* CJK Ideograph */
  ["\u5BC8", "\u5BC8"], /* CJK Ideograph */
  ["\u5BC9", "\u5BC9"], /* CJK Ideograph */
  ["\u5BCA", "\u5BCA"], /* CJK Ideograph */
  ["\u5BCB", "\u5BCB"], /* CJK Ideograph */
  ["\u5BCC", "\u5BCC"], /* CJK Ideograph */
  ["\u5BCD", "\u5BCD"], /* CJK Ideograph */
  ["\u5BCE", "\u5BCE"], /* CJK Ideograph */
  ["\u5BCF", "\u5BCF"], /* CJK Ideograph */
  ["\u5BD0", "\u5BD0"], /* CJK Ideograph */
  ["\u5BD1", "\u5BD1"], /* CJK Ideograph */
  ["\u5BD2", "\u5BD2"], /* CJK Ideograph */
  ["\u5BD3", "\u5BD3"], /* CJK Ideograph */
  ["\u5BD4", "\u5BD4"], /* CJK Ideograph */
  ["\u5BD5", "\u5BD5"], /* CJK Ideograph */
  ["\u5BD6", "\u5BD6"], /* CJK Ideograph */
  ["\u5BD7", "\u5BD7"], /* CJK Ideograph */
  ["\u5BD8", "\u5BD8"], /* CJK Ideograph */
  ["\u5BD9", "\u5BD9"], /* CJK Ideograph */
  ["\u5BDA", "\u5BDA"], /* CJK Ideograph */
  ["\u5BDB", "\u5BDB"], /* CJK Ideograph */
  ["\u5BDC", "\u5BDC"], /* CJK Ideograph */
  ["\u5BDD", "\u5BDD"], /* CJK Ideograph */
  ["\u5BDE", "\u5BDE"], /* CJK Ideograph */
  ["\u5BDF", "\u5BDF"], /* CJK Ideograph */
  ["\u5BE0", "\u5BE0"], /* CJK Ideograph */
  ["\u5BE1", "\u5BE1"], /* CJK Ideograph */
  ["\u5BE2", "\u5BE2"], /* CJK Ideograph */
  ["\u5BE3", "\u5BE3"], /* CJK Ideograph */
  ["\u5BE4", "\u5BE4"], /* CJK Ideograph */
  ["\u5BE5", "\u5BE5"], /* CJK Ideograph */
  ["\u5BE6", "\u5BE6"], /* CJK Ideograph */
  ["\u5BE7", "\u5BE7"], /* CJK Ideograph */
  ["\u5BE8", "\u5BE8"], /* CJK Ideograph */
  ["\u5BE9", "\u5BE9"], /* CJK Ideograph */
  ["\u5BEA", "\u5BEA"], /* CJK Ideograph */
  ["\u5BEB", "\u5BEB"], /* CJK Ideograph */
  ["\u5BEC", "\u5BEC"], /* CJK Ideograph */
  ["\u5BED", "\u5BED"], /* CJK Ideograph */
  ["\u5BEE", "\u5BEE"], /* CJK Ideograph */
  ["\u5BEF", "\u5BEF"], /* CJK Ideograph */
  ["\u5BF0", "\u5BF0"], /* CJK Ideograph */
  ["\u5BF1", "\u5BF1"], /* CJK Ideograph */
  ["\u5BF2", "\u5BF2"], /* CJK Ideograph */
  ["\u5BF3", "\u5BF3"], /* CJK Ideograph */
  ["\u5BF4", "\u5BF4"], /* CJK Ideograph */
  ["\u5BF5", "\u5BF5"], /* CJK Ideograph */
  ["\u5BF6", "\u5BF6"], /* CJK Ideograph */
  ["\u5BF7", "\u5BF7"], /* CJK Ideograph */
  ["\u5BF8", "\u5BF8"], /* CJK Ideograph */
  ["\u5BF9", "\u5BF9"], /* CJK Ideograph */
  ["\u5BFA", "\u5BFA"], /* CJK Ideograph */
  ["\u5BFB", "\u5BFB"], /* CJK Ideograph */
  ["\u5BFC", "\u5BFC"], /* CJK Ideograph */
  ["\u5BFD", "\u5BFD"], /* CJK Ideograph */
  ["\u5BFE", "\u5BFE"], /* CJK Ideograph */
  ["\u5BFF", "\u5BFF"], /* CJK Ideograph */
  ["\u5C00", "\u5C00"], /* CJK Ideograph */
  ["\u5C01", "\u5C01"], /* CJK Ideograph */
  ["\u5C02", "\u5C02"], /* CJK Ideograph */
  ["\u5C03", "\u5C03"], /* CJK Ideograph */
  ["\u5C04", "\u5C04"], /* CJK Ideograph */
  ["\u5C05", "\u5C05"], /* CJK Ideograph */
  ["\u5C06", "\u5C06"], /* CJK Ideograph */
  ["\u5C07", "\u5C07"], /* CJK Ideograph */
  ["\u5C08", "\u5C08"], /* CJK Ideograph */
  ["\u5C09", "\u5C09"], /* CJK Ideograph */
  ["\u5C0A", "\u5C0A"], /* CJK Ideograph */
  ["\u5C0B", "\u5C0B"], /* CJK Ideograph */
  ["\u5C0C", "\u5C0C"], /* CJK Ideograph */
  ["\u5C0D", "\u5C0D"], /* CJK Ideograph */
  ["\u5C0E", "\u5C0E"], /* CJK Ideograph */
  ["\u5C0F", "\u5C0F"], /* CJK Ideograph */
  ["\u5C10", "\u5C10"], /* CJK Ideograph */
  ["\u5C11", "\u5C11"], /* CJK Ideograph */
  ["\u5C12", "\u5C12"], /* CJK Ideograph */
  ["\u5C13", "\u5C13"], /* CJK Ideograph */
  ["\u5C14", "\u5C14"], /* CJK Ideograph */
  ["\u5C15", "\u5C15"], /* CJK Ideograph */
  ["\u5C16", "\u5C16"], /* CJK Ideograph */
  ["\u5C17", "\u5C17"], /* CJK Ideograph */
  ["\u5C18", "\u5C18"], /* CJK Ideograph */
  ["\u5C19", "\u5C19"], /* CJK Ideograph */
  ["\u5C1A", "\u5C1A"], /* CJK Ideograph */
  ["\u5C1B", "\u5C1B"], /* CJK Ideograph */
  ["\u5C1C", "\u5C1C"], /* CJK Ideograph */
  ["\u5C1D", "\u5C1D"], /* CJK Ideograph */
  ["\u5C1E", "\u5C1E"], /* CJK Ideograph */
  ["\u5C1F", "\u5C1F"], /* CJK Ideograph */
  ["\u5C20", "\u5C20"], /* CJK Ideograph */
  ["\u5C21", "\u5C21"], /* CJK Ideograph */
  ["\u5C22", "\u5C22"], /* CJK Ideograph */
  ["\u5C23", "\u5C23"], /* CJK Ideograph */
  ["\u5C24", "\u5C24"], /* CJK Ideograph */
  ["\u5C25", "\u5C25"], /* CJK Ideograph */
  ["\u5C26", "\u5C26"], /* CJK Ideograph */
  ["\u5C27", "\u5C27"], /* CJK Ideograph */
  ["\u5C28", "\u5C28"], /* CJK Ideograph */
  ["\u5C29", "\u5C29"], /* CJK Ideograph */
  ["\u5C2A", "\u5C2A"], /* CJK Ideograph */
  ["\u5C2B", "\u5C2B"], /* CJK Ideograph */
  ["\u5C2C", "\u5C2C"], /* CJK Ideograph */
  ["\u5C2D", "\u5C2D"], /* CJK Ideograph */
  ["\u5C2E", "\u5C2E"], /* CJK Ideograph */
  ["\u5C2F", "\u5C2F"], /* CJK Ideograph */
  ["\u5C30", "\u5C30"], /* CJK Ideograph */
  ["\u5C31", "\u5C31"], /* CJK Ideograph */
  ["\u5C32", "\u5C32"], /* CJK Ideograph */
  ["\u5C33", "\u5C33"], /* CJK Ideograph */
  ["\u5C34", "\u5C34"], /* CJK Ideograph */
  ["\u5C35", "\u5C35"], /* CJK Ideograph */
  ["\u5C36", "\u5C36"], /* CJK Ideograph */
  ["\u5C37", "\u5C37"], /* CJK Ideograph */
  ["\u5C38", "\u5C38"], /* CJK Ideograph */
  ["\u5C39", "\u5C39"], /* CJK Ideograph */
  ["\u5C3A", "\u5C3A"], /* CJK Ideograph */
  ["\u5C3B", "\u5C3B"], /* CJK Ideograph */
  ["\u5C3C", "\u5C3C"], /* CJK Ideograph */
  ["\u5C3D", "\u5C3D"], /* CJK Ideograph */
  ["\u5C3E", "\u5C3E"], /* CJK Ideograph */
  ["\u5C3F", "\u5C3F"], /* CJK Ideograph */
  ["\u5C40", "\u5C40"], /* CJK Ideograph */
  ["\u5C41", "\u5C41"], /* CJK Ideograph */
  ["\u5C42", "\u5C42"], /* CJK Ideograph */
  ["\u5C43", "\u5C43"], /* CJK Ideograph */
  ["\u5C44", "\u5C44"], /* CJK Ideograph */
  ["\u5C45", "\u5C45"], /* CJK Ideograph */
  ["\u5C46", "\u5C46"], /* CJK Ideograph */
  ["\u5C47", "\u5C47"], /* CJK Ideograph */
  ["\u5C48", "\u5C48"], /* CJK Ideograph */
  ["\u5C49", "\u5C49"], /* CJK Ideograph */
  ["\u5C4A", "\u5C4A"], /* CJK Ideograph */
  ["\u5C4B", "\u5C4B"], /* CJK Ideograph */
  ["\u5C4C", "\u5C4C"], /* CJK Ideograph */
  ["\u5C4D", "\u5C4D"], /* CJK Ideograph */
  ["\u5C4E", "\u5C4E"], /* CJK Ideograph */
  ["\u5C4F", "\u5C4F"], /* CJK Ideograph */
  ["\u5C50", "\u5C50"], /* CJK Ideograph */
  ["\u5C51", "\u5C51"], /* CJK Ideograph */
  ["\u5C52", "\u5C52"], /* CJK Ideograph */
  ["\u5C53", "\u5C53"], /* CJK Ideograph */
  ["\u5C54", "\u5C54"], /* CJK Ideograph */
  ["\u5C55", "\u5C55"], /* CJK Ideograph */
  ["\u5C56", "\u5C56"], /* CJK Ideograph */
  ["\u5C57", "\u5C57"], /* CJK Ideograph */
  ["\u5C58", "\u5C58"], /* CJK Ideograph */
  ["\u5C59", "\u5C59"], /* CJK Ideograph */
  ["\u5C5A", "\u5C5A"], /* CJK Ideograph */
  ["\u5C5B", "\u5C5B"], /* CJK Ideograph */
  ["\u5C5C", "\u5C5C"], /* CJK Ideograph */
  ["\u5C5D", "\u5C5D"], /* CJK Ideograph */
  ["\u5C5E", "\u5C5E"], /* CJK Ideograph */
  ["\u5C5F", "\u5C5F"], /* CJK Ideograph */
  ["\u5C60", "\u5C60"], /* CJK Ideograph */
  ["\u5C61", "\u5C61"], /* CJK Ideograph */
  ["\u5C62", "\u5C62"], /* CJK Ideograph */
  ["\u5C63", "\u5C63"], /* CJK Ideograph */
  ["\u5C64", "\u5C64"], /* CJK Ideograph */
  ["\u5C65", "\u5C65"], /* CJK Ideograph */
  ["\u5C66", "\u5C66"], /* CJK Ideograph */
  ["\u5C67", "\u5C67"], /* CJK Ideograph */
  ["\u5C68", "\u5C68"], /* CJK Ideograph */
  ["\u5C69", "\u5C69"], /* CJK Ideograph */
  ["\u5C6A", "\u5C6A"], /* CJK Ideograph */
  ["\u5C6B", "\u5C6B"], /* CJK Ideograph */
  ["\u5C6C", "\u5C6C"], /* CJK Ideograph */
  ["\u5C6D", "\u5C6D"], /* CJK Ideograph */
  ["\u5C6E", "\u5C6E"], /* CJK Ideograph */
  ["\u5C6F", "\u5C6F"], /* CJK Ideograph */
  ["\u5C70", "\u5C70"], /* CJK Ideograph */
  ["\u5C71", "\u5C71"], /* CJK Ideograph */
  ["\u5C72", "\u5C72"], /* CJK Ideograph */
  ["\u5C73", "\u5C73"], /* CJK Ideograph */
  ["\u5C74", "\u5C74"], /* CJK Ideograph */
  ["\u5C75", "\u5C75"], /* CJK Ideograph */
  ["\u5C76", "\u5C76"], /* CJK Ideograph */
  ["\u5C77", "\u5C77"], /* CJK Ideograph */
  ["\u5C78", "\u5C78"], /* CJK Ideograph */
  ["\u5C79", "\u5C79"], /* CJK Ideograph */
  ["\u5C7A", "\u5C7A"], /* CJK Ideograph */
  ["\u5C7B", "\u5C7B"], /* CJK Ideograph */
  ["\u5C7C", "\u5C7C"], /* CJK Ideograph */
  ["\u5C7D", "\u5C7D"], /* CJK Ideograph */
  ["\u5C7E", "\u5C7E"], /* CJK Ideograph */
  ["\u5C7F", "\u5C7F"], /* CJK Ideograph */
  ["\u5C80", "\u5C80"], /* CJK Ideograph */
  ["\u5C81", "\u5C81"], /* CJK Ideograph */
  ["\u5C82", "\u5C82"], /* CJK Ideograph */
  ["\u5C83", "\u5C83"], /* CJK Ideograph */
  ["\u5C84", "\u5C84"], /* CJK Ideograph */
  ["\u5C85", "\u5C85"], /* CJK Ideograph */
  ["\u5C86", "\u5C86"], /* CJK Ideograph */
  ["\u5C87", "\u5C87"], /* CJK Ideograph */
  ["\u5C88", "\u5C88"], /* CJK Ideograph */
  ["\u5C89", "\u5C89"], /* CJK Ideograph */
  ["\u5C8A", "\u5C8A"], /* CJK Ideograph */
  ["\u5C8B", "\u5C8B"], /* CJK Ideograph */
  ["\u5C8C", "\u5C8C"], /* CJK Ideograph */
  ["\u5C8D", "\u5C8D"], /* CJK Ideograph */
  ["\u5C8E", "\u5C8E"], /* CJK Ideograph */
  ["\u5C8F", "\u5C8F"], /* CJK Ideograph */
  ["\u5C90", "\u5C90"], /* CJK Ideograph */
  ["\u5C91", "\u5C91"], /* CJK Ideograph */
  ["\u5C92", "\u5C92"], /* CJK Ideograph */
  ["\u5C93", "\u5C93"], /* CJK Ideograph */
  ["\u5C94", "\u5C94"], /* CJK Ideograph */
  ["\u5C95", "\u5C95"], /* CJK Ideograph */
  ["\u5C96", "\u5C96"], /* CJK Ideograph */
  ["\u5C97", "\u5C97"], /* CJK Ideograph */
  ["\u5C98", "\u5C98"], /* CJK Ideograph */
  ["\u5C99", "\u5C99"], /* CJK Ideograph */
  ["\u5C9A", "\u5C9A"], /* CJK Ideograph */
  ["\u5C9B", "\u5C9B"], /* CJK Ideograph */
  ["\u5C9C", "\u5C9C"], /* CJK Ideograph */
  ["\u5C9D", "\u5C9D"], /* CJK Ideograph */
  ["\u5C9E", "\u5C9E"], /* CJK Ideograph */
  ["\u5C9F", "\u5C9F"], /* CJK Ideograph */
  ["\u5CA0", "\u5CA0"], /* CJK Ideograph */
  ["\u5CA1", "\u5CA1"], /* CJK Ideograph */
  ["\u5CA2", "\u5CA2"], /* CJK Ideograph */
  ["\u5CA3", "\u5CA3"], /* CJK Ideograph */
  ["\u5CA4", "\u5CA4"], /* CJK Ideograph */
  ["\u5CA5", "\u5CA5"], /* CJK Ideograph */
  ["\u5CA6", "\u5CA6"], /* CJK Ideograph */
  ["\u5CA7", "\u5CA7"], /* CJK Ideograph */
  ["\u5CA8", "\u5CA8"], /* CJK Ideograph */
  ["\u5CA9", "\u5CA9"], /* CJK Ideograph */
  ["\u5CAA", "\u5CAA"], /* CJK Ideograph */
  ["\u5CAB", "\u5CAB"], /* CJK Ideograph */
  ["\u5CAC", "\u5CAC"], /* CJK Ideograph */
  ["\u5CAD", "\u5CAD"], /* CJK Ideograph */
  ["\u5CAE", "\u5CAE"], /* CJK Ideograph */
  ["\u5CAF", "\u5CAF"], /* CJK Ideograph */
  ["\u5CB0", "\u5CB0"], /* CJK Ideograph */
  ["\u5CB1", "\u5CB1"], /* CJK Ideograph */
  ["\u5CB2", "\u5CB2"], /* CJK Ideograph */
  ["\u5CB3", "\u5CB3"], /* CJK Ideograph */
  ["\u5CB4", "\u5CB4"], /* CJK Ideograph */
  ["\u5CB5", "\u5CB5"], /* CJK Ideograph */
  ["\u5CB6", "\u5CB6"], /* CJK Ideograph */
  ["\u5CB7", "\u5CB7"], /* CJK Ideograph */
  ["\u5CB8", "\u5CB8"], /* CJK Ideograph */
  ["\u5CB9", "\u5CB9"], /* CJK Ideograph */
  ["\u5CBA", "\u5CBA"], /* CJK Ideograph */
  ["\u5CBB", "\u5CBB"], /* CJK Ideograph */
  ["\u5CBC", "\u5CBC"], /* CJK Ideograph */
  ["\u5CBD", "\u5CBD"], /* CJK Ideograph */
  ["\u5CBE", "\u5CBE"], /* CJK Ideograph */
  ["\u5CBF", "\u5CBF"], /* CJK Ideograph */
  ["\u5CC0", "\u5CC0"], /* CJK Ideograph */
  ["\u5CC1", "\u5CC1"], /* CJK Ideograph */
  ["\u5CC2", "\u5CC2"], /* CJK Ideograph */
  ["\u5CC3", "\u5CC3"], /* CJK Ideograph */
  ["\u5CC4", "\u5CC4"], /* CJK Ideograph */
  ["\u5CC5", "\u5CC5"], /* CJK Ideograph */
  ["\u5CC6", "\u5CC6"], /* CJK Ideograph */
  ["\u5CC7", "\u5CC7"], /* CJK Ideograph */
  ["\u5CC8", "\u5CC8"], /* CJK Ideograph */
  ["\u5CC9", "\u5CC9"], /* CJK Ideograph */
  ["\u5CCA", "\u5CCA"], /* CJK Ideograph */
  ["\u5CCB", "\u5CCB"], /* CJK Ideograph */
  ["\u5CCC", "\u5CCC"], /* CJK Ideograph */
  ["\u5CCD", "\u5CCD"], /* CJK Ideograph */
  ["\u5CCE", "\u5CCE"], /* CJK Ideograph */
  ["\u5CCF", "\u5CCF"], /* CJK Ideograph */
  ["\u5CD0", "\u5CD0"], /* CJK Ideograph */
  ["\u5CD1", "\u5CD1"], /* CJK Ideograph */
  ["\u5CD2", "\u5CD2"], /* CJK Ideograph */
  ["\u5CD3", "\u5CD3"], /* CJK Ideograph */
  ["\u5CD4", "\u5CD4"], /* CJK Ideograph */
  ["\u5CD5", "\u5CD5"], /* CJK Ideograph */
  ["\u5CD6", "\u5CD6"], /* CJK Ideograph */
  ["\u5CD7", "\u5CD7"], /* CJK Ideograph */
  ["\u5CD8", "\u5CD8"], /* CJK Ideograph */
  ["\u5CD9", "\u5CD9"], /* CJK Ideograph */
  ["\u5CDA", "\u5CDA"], /* CJK Ideograph */
  ["\u5CDB", "\u5CDB"], /* CJK Ideograph */
  ["\u5CDC", "\u5CDC"], /* CJK Ideograph */
  ["\u5CDD", "\u5CDD"], /* CJK Ideograph */
  ["\u5CDE", "\u5CDE"], /* CJK Ideograph */
  ["\u5CDF", "\u5CDF"], /* CJK Ideograph */
  ["\u5CE0", "\u5CE0"], /* CJK Ideograph */
  ["\u5CE1", "\u5CE1"], /* CJK Ideograph */
  ["\u5CE2", "\u5CE2"], /* CJK Ideograph */
  ["\u5CE3", "\u5CE3"], /* CJK Ideograph */
  ["\u5CE4", "\u5CE4"], /* CJK Ideograph */
  ["\u5CE5", "\u5CE5"], /* CJK Ideograph */
  ["\u5CE6", "\u5CE6"], /* CJK Ideograph */
  ["\u5CE7", "\u5CE7"], /* CJK Ideograph */
  ["\u5CE8", "\u5CE8"], /* CJK Ideograph */
  ["\u5CE9", "\u5CE9"], /* CJK Ideograph */
  ["\u5CEA", "\u5CEA"], /* CJK Ideograph */
  ["\u5CEB", "\u5CEB"], /* CJK Ideograph */
  ["\u5CEC", "\u5CEC"], /* CJK Ideograph */
  ["\u5CED", "\u5CED"], /* CJK Ideograph */
  ["\u5CEE", "\u5CEE"], /* CJK Ideograph */
  ["\u5CEF", "\u5CEF"], /* CJK Ideograph */
  ["\u5CF0", "\u5CF0"], /* CJK Ideograph */
  ["\u5CF1", "\u5CF1"], /* CJK Ideograph */
  ["\u5CF2", "\u5CF2"], /* CJK Ideograph */
  ["\u5CF3", "\u5CF3"], /* CJK Ideograph */
  ["\u5CF4", "\u5CF4"], /* CJK Ideograph */
  ["\u5CF5", "\u5CF5"], /* CJK Ideograph */
  ["\u5CF6", "\u5CF6"], /* CJK Ideograph */
  ["\u5CF7", "\u5CF7"], /* CJK Ideograph */
  ["\u5CF8", "\u5CF8"], /* CJK Ideograph */
  ["\u5CF9", "\u5CF9"], /* CJK Ideograph */
  ["\u5CFA", "\u5CFA"], /* CJK Ideograph */
  ["\u5CFB", "\u5CFB"], /* CJK Ideograph */
  ["\u5CFC", "\u5CFC"], /* CJK Ideograph */
  ["\u5CFD", "\u5CFD"], /* CJK Ideograph */
  ["\u5CFE", "\u5CFE"], /* CJK Ideograph */
  ["\u5CFF", "\u5CFF"], /* CJK Ideograph */
  ["\u5D00", "\u5D00"], /* CJK Ideograph */
  ["\u5D01", "\u5D01"], /* CJK Ideograph */
  ["\u5D02", "\u5D02"], /* CJK Ideograph */
  ["\u5D03", "\u5D03"], /* CJK Ideograph */
  ["\u5D04", "\u5D04"], /* CJK Ideograph */
  ["\u5D05", "\u5D05"], /* CJK Ideograph */
  ["\u5D06", "\u5D06"], /* CJK Ideograph */
  ["\u5D07", "\u5D07"], /* CJK Ideograph */
  ["\u5D08", "\u5D08"], /* CJK Ideograph */
  ["\u5D09", "\u5D09"], /* CJK Ideograph */
  ["\u5D0A", "\u5D0A"], /* CJK Ideograph */
  ["\u5D0B", "\u5D0B"], /* CJK Ideograph */
  ["\u5D0C", "\u5D0C"], /* CJK Ideograph */
  ["\u5D0D", "\u5D0D"], /* CJK Ideograph */
  ["\u5D0E", "\u5D0E"], /* CJK Ideograph */
  ["\u5D0F", "\u5D0F"], /* CJK Ideograph */
  ["\u5D10", "\u5D10"], /* CJK Ideograph */
  ["\u5D11", "\u5D11"], /* CJK Ideograph */
  ["\u5D12", "\u5D12"], /* CJK Ideograph */
  ["\u5D13", "\u5D13"], /* CJK Ideograph */
  ["\u5D14", "\u5D14"], /* CJK Ideograph */
  ["\u5D15", "\u5D15"], /* CJK Ideograph */
  ["\u5D16", "\u5D16"], /* CJK Ideograph */
  ["\u5D17", "\u5D17"], /* CJK Ideograph */
  ["\u5D18", "\u5D18"], /* CJK Ideograph */
  ["\u5D19", "\u5D19"], /* CJK Ideograph */
  ["\u5D1A", "\u5D1A"], /* CJK Ideograph */
  ["\u5D1B", "\u5D1B"], /* CJK Ideograph */
  ["\u5D1C", "\u5D1C"], /* CJK Ideograph */
  ["\u5D1D", "\u5D1D"], /* CJK Ideograph */
  ["\u5D1E", "\u5D1E"], /* CJK Ideograph */
  ["\u5D1F", "\u5D1F"], /* CJK Ideograph */
  ["\u5D20", "\u5D20"], /* CJK Ideograph */
  ["\u5D21", "\u5D21"], /* CJK Ideograph */
  ["\u5D22", "\u5D22"], /* CJK Ideograph */
  ["\u5D23", "\u5D23"], /* CJK Ideograph */
  ["\u5D24", "\u5D24"], /* CJK Ideograph */
  ["\u5D25", "\u5D25"], /* CJK Ideograph */
  ["\u5D26", "\u5D26"], /* CJK Ideograph */
  ["\u5D27", "\u5D27"], /* CJK Ideograph */
  ["\u5D28", "\u5D28"], /* CJK Ideograph */
  ["\u5D29", "\u5D29"], /* CJK Ideograph */
  ["\u5D2A", "\u5D2A"], /* CJK Ideograph */
  ["\u5D2B", "\u5D2B"], /* CJK Ideograph */
  ["\u5D2C", "\u5D2C"], /* CJK Ideograph */
  ["\u5D2D", "\u5D2D"], /* CJK Ideograph */
  ["\u5D2E", "\u5D2E"], /* CJK Ideograph */
  ["\u5D2F", "\u5D2F"], /* CJK Ideograph */
  ["\u5D30", "\u5D30"], /* CJK Ideograph */
  ["\u5D31", "\u5D31"], /* CJK Ideograph */
  ["\u5D32", "\u5D32"], /* CJK Ideograph */
  ["\u5D33", "\u5D33"], /* CJK Ideograph */
  ["\u5D34", "\u5D34"], /* CJK Ideograph */
  ["\u5D35", "\u5D35"], /* CJK Ideograph */
  ["\u5D36", "\u5D36"], /* CJK Ideograph */
  ["\u5D37", "\u5D37"], /* CJK Ideograph */
  ["\u5D38", "\u5D38"], /* CJK Ideograph */
  ["\u5D39", "\u5D39"], /* CJK Ideograph */
  ["\u5D3A", "\u5D3A"], /* CJK Ideograph */
  ["\u5D3B", "\u5D3B"], /* CJK Ideograph */
  ["\u5D3C", "\u5D3C"], /* CJK Ideograph */
  ["\u5D3D", "\u5D3D"], /* CJK Ideograph */
  ["\u5D3E", "\u5D3E"], /* CJK Ideograph */
  ["\u5D3F", "\u5D3F"], /* CJK Ideograph */
  ["\u5D40", "\u5D40"], /* CJK Ideograph */
  ["\u5D41", "\u5D41"], /* CJK Ideograph */
  ["\u5D42", "\u5D42"], /* CJK Ideograph */
  ["\u5D43", "\u5D43"], /* CJK Ideograph */
  ["\u5D44", "\u5D44"], /* CJK Ideograph */
  ["\u5D45", "\u5D45"], /* CJK Ideograph */
  ["\u5D46", "\u5D46"], /* CJK Ideograph */
  ["\u5D47", "\u5D47"], /* CJK Ideograph */
  ["\u5D48", "\u5D48"], /* CJK Ideograph */
  ["\u5D49", "\u5D49"], /* CJK Ideograph */
  ["\u5D4A", "\u5D4A"], /* CJK Ideograph */
  ["\u5D4B", "\u5D4B"], /* CJK Ideograph */
  ["\u5D4C", "\u5D4C"], /* CJK Ideograph */
  ["\u5D4D", "\u5D4D"], /* CJK Ideograph */
  ["\u5D4E", "\u5D4E"], /* CJK Ideograph */
  ["\u5D4F", "\u5D4F"], /* CJK Ideograph */
  ["\u5D50", "\u5D50"], /* CJK Ideograph */
  ["\u5D51", "\u5D51"], /* CJK Ideograph */
  ["\u5D52", "\u5D52"], /* CJK Ideograph */
  ["\u5D53", "\u5D53"], /* CJK Ideograph */
  ["\u5D54", "\u5D54"], /* CJK Ideograph */
  ["\u5D55", "\u5D55"], /* CJK Ideograph */
  ["\u5D56", "\u5D56"], /* CJK Ideograph */
  ["\u5D57", "\u5D57"], /* CJK Ideograph */
  ["\u5D58", "\u5D58"], /* CJK Ideograph */
  ["\u5D59", "\u5D59"], /* CJK Ideograph */
  ["\u5D5A", "\u5D5A"], /* CJK Ideograph */
  ["\u5D5B", "\u5D5B"], /* CJK Ideograph */
  ["\u5D5C", "\u5D5C"], /* CJK Ideograph */
  ["\u5D5D", "\u5D5D"], /* CJK Ideograph */
  ["\u5D5E", "\u5D5E"], /* CJK Ideograph */
  ["\u5D5F", "\u5D5F"], /* CJK Ideograph */
  ["\u5D60", "\u5D60"], /* CJK Ideograph */
  ["\u5D61", "\u5D61"], /* CJK Ideograph */
  ["\u5D62", "\u5D62"], /* CJK Ideograph */
  ["\u5D63", "\u5D63"], /* CJK Ideograph */
  ["\u5D64", "\u5D64"], /* CJK Ideograph */
  ["\u5D65", "\u5D65"], /* CJK Ideograph */
  ["\u5D66", "\u5D66"], /* CJK Ideograph */
  ["\u5D67", "\u5D67"], /* CJK Ideograph */
  ["\u5D68", "\u5D68"], /* CJK Ideograph */
  ["\u5D69", "\u5D69"], /* CJK Ideograph */
  ["\u5D6A", "\u5D6A"], /* CJK Ideograph */
  ["\u5D6B", "\u5D6B"], /* CJK Ideograph */
  ["\u5D6C", "\u5D6C"], /* CJK Ideograph */
  ["\u5D6D", "\u5D6D"], /* CJK Ideograph */
  ["\u5D6E", "\u5D6E"], /* CJK Ideograph */
  ["\u5D6F", "\u5D6F"], /* CJK Ideograph */
  ["\u5D70", "\u5D70"], /* CJK Ideograph */
  ["\u5D71", "\u5D71"], /* CJK Ideograph */
  ["\u5D72", "\u5D72"], /* CJK Ideograph */
  ["\u5D73", "\u5D73"], /* CJK Ideograph */
  ["\u5D74", "\u5D74"], /* CJK Ideograph */
  ["\u5D75", "\u5D75"], /* CJK Ideograph */
  ["\u5D76", "\u5D76"], /* CJK Ideograph */
  ["\u5D77", "\u5D77"], /* CJK Ideograph */
  ["\u5D78", "\u5D78"], /* CJK Ideograph */
  ["\u5D79", "\u5D79"], /* CJK Ideograph */
  ["\u5D7A", "\u5D7A"], /* CJK Ideograph */
  ["\u5D7B", "\u5D7B"], /* CJK Ideograph */
  ["\u5D7C", "\u5D7C"], /* CJK Ideograph */
  ["\u5D7D", "\u5D7D"], /* CJK Ideograph */
  ["\u5D7E", "\u5D7E"], /* CJK Ideograph */
  ["\u5D7F", "\u5D7F"], /* CJK Ideograph */
  ["\u5D80", "\u5D80"], /* CJK Ideograph */
  ["\u5D81", "\u5D81"], /* CJK Ideograph */
  ["\u5D82", "\u5D82"], /* CJK Ideograph */
  ["\u5D83", "\u5D83"], /* CJK Ideograph */
  ["\u5D84", "\u5D84"], /* CJK Ideograph */
  ["\u5D85", "\u5D85"], /* CJK Ideograph */
  ["\u5D86", "\u5D86"], /* CJK Ideograph */
  ["\u5D87", "\u5D87"], /* CJK Ideograph */
  ["\u5D88", "\u5D88"], /* CJK Ideograph */
  ["\u5D89", "\u5D89"], /* CJK Ideograph */
  ["\u5D8A", "\u5D8A"], /* CJK Ideograph */
  ["\u5D8B", "\u5D8B"], /* CJK Ideograph */
  ["\u5D8C", "\u5D8C"], /* CJK Ideograph */
  ["\u5D8D", "\u5D8D"], /* CJK Ideograph */
  ["\u5D8E", "\u5D8E"], /* CJK Ideograph */
  ["\u5D8F", "\u5D8F"], /* CJK Ideograph */
  ["\u5D90", "\u5D90"], /* CJK Ideograph */
  ["\u5D91", "\u5D91"], /* CJK Ideograph */
  ["\u5D92", "\u5D92"], /* CJK Ideograph */
  ["\u5D93", "\u5D93"], /* CJK Ideograph */
  ["\u5D94", "\u5D94"], /* CJK Ideograph */
  ["\u5D95", "\u5D95"], /* CJK Ideograph */
  ["\u5D96", "\u5D96"], /* CJK Ideograph */
  ["\u5D97", "\u5D97"], /* CJK Ideograph */
  ["\u5D98", "\u5D98"], /* CJK Ideograph */
  ["\u5D99", "\u5D99"], /* CJK Ideograph */
  ["\u5D9A", "\u5D9A"], /* CJK Ideograph */
  ["\u5D9B", "\u5D9B"], /* CJK Ideograph */
  ["\u5D9C", "\u5D9C"], /* CJK Ideograph */
  ["\u5D9D", "\u5D9D"], /* CJK Ideograph */
  ["\u5D9E", "\u5D9E"], /* CJK Ideograph */
  ["\u5D9F", "\u5D9F"], /* CJK Ideograph */
  ["\u5DA0", "\u5DA0"], /* CJK Ideograph */
  ["\u5DA1", "\u5DA1"], /* CJK Ideograph */
  ["\u5DA2", "\u5DA2"], /* CJK Ideograph */
  ["\u5DA3", "\u5DA3"], /* CJK Ideograph */
  ["\u5DA4", "\u5DA4"], /* CJK Ideograph */
  ["\u5DA5", "\u5DA5"], /* CJK Ideograph */
  ["\u5DA6", "\u5DA6"], /* CJK Ideograph */
  ["\u5DA7", "\u5DA7"], /* CJK Ideograph */
  ["\u5DA8", "\u5DA8"], /* CJK Ideograph */
  ["\u5DA9", "\u5DA9"], /* CJK Ideograph */
  ["\u5DAA", "\u5DAA"], /* CJK Ideograph */
  ["\u5DAB", "\u5DAB"], /* CJK Ideograph */
  ["\u5DAC", "\u5DAC"], /* CJK Ideograph */
  ["\u5DAD", "\u5DAD"], /* CJK Ideograph */
  ["\u5DAE", "\u5DAE"], /* CJK Ideograph */
  ["\u5DAF", "\u5DAF"], /* CJK Ideograph */
  ["\u5DB0", "\u5DB0"], /* CJK Ideograph */
  ["\u5DB1", "\u5DB1"], /* CJK Ideograph */
  ["\u5DB2", "\u5DB2"], /* CJK Ideograph */
  ["\u5DB3", "\u5DB3"], /* CJK Ideograph */
  ["\u5DB4", "\u5DB4"], /* CJK Ideograph */
  ["\u5DB5", "\u5DB5"], /* CJK Ideograph */
  ["\u5DB6", "\u5DB6"], /* CJK Ideograph */
  ["\u5DB7", "\u5DB7"], /* CJK Ideograph */
  ["\u5DB8", "\u5DB8"], /* CJK Ideograph */
  ["\u5DB9", "\u5DB9"], /* CJK Ideograph */
  ["\u5DBA", "\u5DBA"], /* CJK Ideograph */
  ["\u5DBB", "\u5DBB"], /* CJK Ideograph */
  ["\u5DBC", "\u5DBC"], /* CJK Ideograph */
  ["\u5DBD", "\u5DBD"], /* CJK Ideograph */
  ["\u5DBE", "\u5DBE"], /* CJK Ideograph */
  ["\u5DBF", "\u5DBF"], /* CJK Ideograph */
  ["\u5DC0", "\u5DC0"], /* CJK Ideograph */
  ["\u5DC1", "\u5DC1"], /* CJK Ideograph */
  ["\u5DC2", "\u5DC2"], /* CJK Ideograph */
  ["\u5DC3", "\u5DC3"], /* CJK Ideograph */
  ["\u5DC4", "\u5DC4"], /* CJK Ideograph */
  ["\u5DC5", "\u5DC5"], /* CJK Ideograph */
  ["\u5DC6", "\u5DC6"], /* CJK Ideograph */
  ["\u5DC7", "\u5DC7"], /* CJK Ideograph */
  ["\u5DC8", "\u5DC8"], /* CJK Ideograph */
  ["\u5DC9", "\u5DC9"], /* CJK Ideograph */
  ["\u5DCA", "\u5DCA"], /* CJK Ideograph */
  ["\u5DCB", "\u5DCB"], /* CJK Ideograph */
  ["\u5DCC", "\u5DCC"], /* CJK Ideograph */
  ["\u5DCD", "\u5DCD"], /* CJK Ideograph */
  ["\u5DCE", "\u5DCE"], /* CJK Ideograph */
  ["\u5DCF", "\u5DCF"], /* CJK Ideograph */
  ["\u5DD0", "\u5DD0"], /* CJK Ideograph */
  ["\u5DD1", "\u5DD1"], /* CJK Ideograph */
  ["\u5DD2", "\u5DD2"], /* CJK Ideograph */
  ["\u5DD3", "\u5DD3"], /* CJK Ideograph */
  ["\u5DD4", "\u5DD4"], /* CJK Ideograph */
  ["\u5DD5", "\u5DD5"], /* CJK Ideograph */
  ["\u5DD6", "\u5DD6"], /* CJK Ideograph */
  ["\u5DD7", "\u5DD7"], /* CJK Ideograph */
  ["\u5DD8", "\u5DD8"], /* CJK Ideograph */
  ["\u5DD9", "\u5DD9"], /* CJK Ideograph */
  ["\u5DDA", "\u5DDA"], /* CJK Ideograph */
  ["\u5DDB", "\u5DDB"], /* CJK Ideograph */
  ["\u5DDC", "\u5DDC"], /* CJK Ideograph */
  ["\u5DDD", "\u5DDD"], /* CJK Ideograph */
  ["\u5DDE", "\u5DDE"], /* CJK Ideograph */
  ["\u5DDF", "\u5DDF"], /* CJK Ideograph */
  ["\u5DE0", "\u5DE0"], /* CJK Ideograph */
  ["\u5DE1", "\u5DE1"], /* CJK Ideograph */
  ["\u5DE2", "\u5DE2"], /* CJK Ideograph */
  ["\u5DE3", "\u5DE3"], /* CJK Ideograph */
  ["\u5DE4", "\u5DE4"], /* CJK Ideograph */
  ["\u5DE5", "\u5DE5"], /* CJK Ideograph */
  ["\u5DE6", "\u5DE6"], /* CJK Ideograph */
  ["\u5DE7", "\u5DE7"], /* CJK Ideograph */
  ["\u5DE8", "\u5DE8"], /* CJK Ideograph */
  ["\u5DE9", "\u5DE9"], /* CJK Ideograph */
  ["\u5DEA", "\u5DEA"], /* CJK Ideograph */
  ["\u5DEB", "\u5DEB"], /* CJK Ideograph */
  ["\u5DEC", "\u5DEC"], /* CJK Ideograph */
  ["\u5DED", "\u5DED"], /* CJK Ideograph */
  ["\u5DEE", "\u5DEE"], /* CJK Ideograph */
  ["\u5DEF", "\u5DEF"], /* CJK Ideograph */
  ["\u5DF0", "\u5DF0"], /* CJK Ideograph */
  ["\u5DF1", "\u5DF1"], /* CJK Ideograph */
  ["\u5DF2", "\u5DF2"], /* CJK Ideograph */
  ["\u5DF3", "\u5DF3"], /* CJK Ideograph */
  ["\u5DF4", "\u5DF4"], /* CJK Ideograph */
  ["\u5DF5", "\u5DF5"], /* CJK Ideograph */
  ["\u5DF6", "\u5DF6"], /* CJK Ideograph */
  ["\u5DF7", "\u5DF7"], /* CJK Ideograph */
  ["\u5DF8", "\u5DF8"], /* CJK Ideograph */
  ["\u5DF9", "\u5DF9"], /* CJK Ideograph */
  ["\u5DFA", "\u5DFA"], /* CJK Ideograph */
  ["\u5DFB", "\u5DFB"], /* CJK Ideograph */
  ["\u5DFC", "\u5DFC"], /* CJK Ideograph */
  ["\u5DFD", "\u5DFD"], /* CJK Ideograph */
  ["\u5DFE", "\u5DFE"], /* CJK Ideograph */
  ["\u5DFF", "\u5DFF"], /* CJK Ideograph */
  ["\u5E00", "\u5E00"], /* CJK Ideograph */
  ["\u5E01", "\u5E01"], /* CJK Ideograph */
  ["\u5E02", "\u5E02"], /* CJK Ideograph */
  ["\u5E03", "\u5E03"], /* CJK Ideograph */
  ["\u5E04", "\u5E04"], /* CJK Ideograph */
  ["\u5E05", "\u5E05"], /* CJK Ideograph */
  ["\u5E06", "\u5E06"], /* CJK Ideograph */
  ["\u5E07", "\u5E07"], /* CJK Ideograph */
  ["\u5E08", "\u5E08"], /* CJK Ideograph */
  ["\u5E09", "\u5E09"], /* CJK Ideograph */
  ["\u5E0A", "\u5E0A"], /* CJK Ideograph */
  ["\u5E0B", "\u5E0B"], /* CJK Ideograph */
  ["\u5E0C", "\u5E0C"], /* CJK Ideograph */
  ["\u5E0D", "\u5E0D"], /* CJK Ideograph */
  ["\u5E0E", "\u5E0E"], /* CJK Ideograph */
  ["\u5E0F", "\u5E0F"], /* CJK Ideograph */
  ["\u5E10", "\u5E10"], /* CJK Ideograph */
  ["\u5E11", "\u5E11"], /* CJK Ideograph */
  ["\u5E12", "\u5E12"], /* CJK Ideograph */
  ["\u5E13", "\u5E13"], /* CJK Ideograph */
  ["\u5E14", "\u5E14"], /* CJK Ideograph */
  ["\u5E15", "\u5E15"], /* CJK Ideograph */
  ["\u5E16", "\u5E16"], /* CJK Ideograph */
  ["\u5E17", "\u5E17"], /* CJK Ideograph */
  ["\u5E18", "\u5E18"], /* CJK Ideograph */
  ["\u5E19", "\u5E19"], /* CJK Ideograph */
  ["\u5E1A", "\u5E1A"], /* CJK Ideograph */
  ["\u5E1B", "\u5E1B"], /* CJK Ideograph */
  ["\u5E1C", "\u5E1C"], /* CJK Ideograph */
  ["\u5E1D", "\u5E1D"], /* CJK Ideograph */
  ["\u5E1E", "\u5E1E"], /* CJK Ideograph */
  ["\u5E1F", "\u5E1F"], /* CJK Ideograph */
  ["\u5E20", "\u5E20"], /* CJK Ideograph */
  ["\u5E21", "\u5E21"], /* CJK Ideograph */
  ["\u5E22", "\u5E22"], /* CJK Ideograph */
  ["\u5E23", "\u5E23"], /* CJK Ideograph */
  ["\u5E24", "\u5E24"], /* CJK Ideograph */
  ["\u5E25", "\u5E25"], /* CJK Ideograph */
  ["\u5E26", "\u5E26"], /* CJK Ideograph */
  ["\u5E27", "\u5E27"], /* CJK Ideograph */
  ["\u5E28", "\u5E28"], /* CJK Ideograph */
  ["\u5E29", "\u5E29"], /* CJK Ideograph */
  ["\u5E2A", "\u5E2A"], /* CJK Ideograph */
  ["\u5E2B", "\u5E2B"], /* CJK Ideograph */
  ["\u5E2C", "\u5E2C"], /* CJK Ideograph */
  ["\u5E2D", "\u5E2D"], /* CJK Ideograph */
  ["\u5E2E", "\u5E2E"], /* CJK Ideograph */
  ["\u5E2F", "\u5E2F"], /* CJK Ideograph */
  ["\u5E30", "\u5E30"], /* CJK Ideograph */
  ["\u5E31", "\u5E31"], /* CJK Ideograph */
  ["\u5E32", "\u5E32"], /* CJK Ideograph */
  ["\u5E33", "\u5E33"], /* CJK Ideograph */
  ["\u5E34", "\u5E34"], /* CJK Ideograph */
  ["\u5E35", "\u5E35"], /* CJK Ideograph */
  ["\u5E36", "\u5E36"], /* CJK Ideograph */
  ["\u5E37", "\u5E37"], /* CJK Ideograph */
  ["\u5E38", "\u5E38"], /* CJK Ideograph */
  ["\u5E39", "\u5E39"], /* CJK Ideograph */
  ["\u5E3A", "\u5E3A"], /* CJK Ideograph */
  ["\u5E3B", "\u5E3B"], /* CJK Ideograph */
  ["\u5E3C", "\u5E3C"], /* CJK Ideograph */
  ["\u5E3D", "\u5E3D"], /* CJK Ideograph */
  ["\u5E3E", "\u5E3E"], /* CJK Ideograph */
  ["\u5E3F", "\u5E3F"], /* CJK Ideograph */
  ["\u5E40", "\u5E40"], /* CJK Ideograph */
  ["\u5E41", "\u5E41"], /* CJK Ideograph */
  ["\u5E42", "\u5E42"], /* CJK Ideograph */
  ["\u5E43", "\u5E43"], /* CJK Ideograph */
  ["\u5E44", "\u5E44"], /* CJK Ideograph */
  ["\u5E45", "\u5E45"], /* CJK Ideograph */
  ["\u5E46", "\u5E46"], /* CJK Ideograph */
  ["\u5E47", "\u5E47"], /* CJK Ideograph */
  ["\u5E48", "\u5E48"], /* CJK Ideograph */
  ["\u5E49", "\u5E49"], /* CJK Ideograph */
  ["\u5E4A", "\u5E4A"], /* CJK Ideograph */
  ["\u5E4B", "\u5E4B"], /* CJK Ideograph */
  ["\u5E4C", "\u5E4C"], /* CJK Ideograph */
  ["\u5E4D", "\u5E4D"], /* CJK Ideograph */
  ["\u5E4E", "\u5E4E"], /* CJK Ideograph */
  ["\u5E4F", "\u5E4F"], /* CJK Ideograph */
  ["\u5E50", "\u5E50"], /* CJK Ideograph */
  ["\u5E51", "\u5E51"], /* CJK Ideograph */
  ["\u5E52", "\u5E52"], /* CJK Ideograph */
  ["\u5E53", "\u5E53"], /* CJK Ideograph */
  ["\u5E54", "\u5E54"], /* CJK Ideograph */
  ["\u5E55", "\u5E55"], /* CJK Ideograph */
  ["\u5E56", "\u5E56"], /* CJK Ideograph */
  ["\u5E57", "\u5E57"], /* CJK Ideograph */
  ["\u5E58", "\u5E58"], /* CJK Ideograph */
  ["\u5E59", "\u5E59"], /* CJK Ideograph */
  ["\u5E5A", "\u5E5A"], /* CJK Ideograph */
  ["\u5E5B", "\u5E5B"], /* CJK Ideograph */
  ["\u5E5C", "\u5E5C"], /* CJK Ideograph */
  ["\u5E5D", "\u5E5D"], /* CJK Ideograph */
  ["\u5E5E", "\u5E5E"], /* CJK Ideograph */
  ["\u5E5F", "\u5E5F"], /* CJK Ideograph */
  ["\u5E60", "\u5E60"], /* CJK Ideograph */
  ["\u5E61", "\u5E61"], /* CJK Ideograph */
  ["\u5E62", "\u5E62"], /* CJK Ideograph */
  ["\u5E63", "\u5E63"], /* CJK Ideograph */
  ["\u5E64", "\u5E64"], /* CJK Ideograph */
  ["\u5E65", "\u5E65"], /* CJK Ideograph */
  ["\u5E66", "\u5E66"], /* CJK Ideograph */
  ["\u5E67", "\u5E67"], /* CJK Ideograph */
  ["\u5E68", "\u5E68"], /* CJK Ideograph */
  ["\u5E69", "\u5E69"], /* CJK Ideograph */
  ["\u5E6A", "\u5E6A"], /* CJK Ideograph */
  ["\u5E6B", "\u5E6B"], /* CJK Ideograph */
  ["\u5E6C", "\u5E6C"], /* CJK Ideograph */
  ["\u5E6D", "\u5E6D"], /* CJK Ideograph */
  ["\u5E6E", "\u5E6E"], /* CJK Ideograph */
  ["\u5E6F", "\u5E6F"], /* CJK Ideograph */
  ["\u5E70", "\u5E70"], /* CJK Ideograph */
  ["\u5E71", "\u5E71"], /* CJK Ideograph */
  ["\u5E72", "\u5E72"], /* CJK Ideograph */
  ["\u5E73", "\u5E73"], /* CJK Ideograph */
  ["\u5E74", "\u5E74"], /* CJK Ideograph */
  ["\u5E75", "\u5E75"], /* CJK Ideograph */
  ["\u5E76", "\u5E76"], /* CJK Ideograph */
  ["\u5E77", "\u5E77"], /* CJK Ideograph */
  ["\u5E78", "\u5E78"], /* CJK Ideograph */
  ["\u5E79", "\u5E79"], /* CJK Ideograph */
  ["\u5E7A", "\u5E7A"], /* CJK Ideograph */
  ["\u5E7B", "\u5E7B"], /* CJK Ideograph */
  ["\u5E7C", "\u5E7C"], /* CJK Ideograph */
  ["\u5E7D", "\u5E7D"], /* CJK Ideograph */
  ["\u5E7E", "\u5E7E"], /* CJK Ideograph */
  ["\u5E7F", "\u5E7F"], /* CJK Ideograph */
  ["\u5E80", "\u5E80"], /* CJK Ideograph */
  ["\u5E81", "\u5E81"], /* CJK Ideograph */
  ["\u5E82", "\u5E82"], /* CJK Ideograph */
  ["\u5E83", "\u5E83"], /* CJK Ideograph */
  ["\u5E84", "\u5E84"], /* CJK Ideograph */
  ["\u5E85", "\u5E85"], /* CJK Ideograph */
  ["\u5E86", "\u5E86"], /* CJK Ideograph */
  ["\u5E87", "\u5E87"], /* CJK Ideograph */
  ["\u5E88", "\u5E88"], /* CJK Ideograph */
  ["\u5E89", "\u5E89"], /* CJK Ideograph */
  ["\u5E8A", "\u5E8A"], /* CJK Ideograph */
  ["\u5E8B", "\u5E8B"], /* CJK Ideograph */
  ["\u5E8C", "\u5E8C"], /* CJK Ideograph */
  ["\u5E8D", "\u5E8D"], /* CJK Ideograph */
  ["\u5E8E", "\u5E8E"], /* CJK Ideograph */
  ["\u5E8F", "\u5E8F"], /* CJK Ideograph */
  ["\u5E90", "\u5E90"], /* CJK Ideograph */
  ["\u5E91", "\u5E91"], /* CJK Ideograph */
  ["\u5E92", "\u5E92"], /* CJK Ideograph */
  ["\u5E93", "\u5E93"], /* CJK Ideograph */
  ["\u5E94", "\u5E94"], /* CJK Ideograph */
  ["\u5E95", "\u5E95"], /* CJK Ideograph */
  ["\u5E96", "\u5E96"], /* CJK Ideograph */
  ["\u5E97", "\u5E97"], /* CJK Ideograph */
  ["\u5E98", "\u5E98"], /* CJK Ideograph */
  ["\u5E99", "\u5E99"], /* CJK Ideograph */
  ["\u5E9A", "\u5E9A"], /* CJK Ideograph */
  ["\u5E9B", "\u5E9B"], /* CJK Ideograph */
  ["\u5E9C", "\u5E9C"], /* CJK Ideograph */
  ["\u5E9D", "\u5E9D"], /* CJK Ideograph */
  ["\u5E9E", "\u5E9E"], /* CJK Ideograph */
  ["\u5E9F", "\u5E9F"], /* CJK Ideograph */
  ["\u5EA0", "\u5EA0"], /* CJK Ideograph */
  ["\u5EA1", "\u5EA1"], /* CJK Ideograph */
  ["\u5EA2", "\u5EA2"], /* CJK Ideograph */
  ["\u5EA3", "\u5EA3"], /* CJK Ideograph */
  ["\u5EA4", "\u5EA4"], /* CJK Ideograph */
  ["\u5EA5", "\u5EA5"], /* CJK Ideograph */
  ["\u5EA6", "\u5EA6"], /* CJK Ideograph */
  ["\u5EA7", "\u5EA7"], /* CJK Ideograph */
  ["\u5EA8", "\u5EA8"], /* CJK Ideograph */
  ["\u5EA9", "\u5EA9"], /* CJK Ideograph */
  ["\u5EAA", "\u5EAA"], /* CJK Ideograph */
  ["\u5EAB", "\u5EAB"], /* CJK Ideograph */
  ["\u5EAC", "\u5EAC"], /* CJK Ideograph */
  ["\u5EAD", "\u5EAD"], /* CJK Ideograph */
  ["\u5EAE", "\u5EAE"], /* CJK Ideograph */
  ["\u5EAF", "\u5EAF"], /* CJK Ideograph */
  ["\u5EB0", "\u5EB0"], /* CJK Ideograph */
  ["\u5EB1", "\u5EB1"], /* CJK Ideograph */
  ["\u5EB2", "\u5EB2"], /* CJK Ideograph */
  ["\u5EB3", "\u5EB3"], /* CJK Ideograph */
  ["\u5EB4", "\u5EB4"], /* CJK Ideograph */
  ["\u5EB5", "\u5EB5"], /* CJK Ideograph */
  ["\u5EB6", "\u5EB6"], /* CJK Ideograph */
  ["\u5EB7", "\u5EB7"], /* CJK Ideograph */
  ["\u5EB8", "\u5EB8"], /* CJK Ideograph */
  ["\u5EB9", "\u5EB9"], /* CJK Ideograph */
  ["\u5EBA", "\u5EBA"], /* CJK Ideograph */
  ["\u5EBB", "\u5EBB"], /* CJK Ideograph */
  ["\u5EBC", "\u5EBC"], /* CJK Ideograph */
  ["\u5EBD", "\u5EBD"], /* CJK Ideograph */
  ["\u5EBE", "\u5EBE"], /* CJK Ideograph */
  ["\u5EBF", "\u5EBF"], /* CJK Ideograph */
  ["\u5EC0", "\u5EC0"], /* CJK Ideograph */
  ["\u5EC1", "\u5EC1"], /* CJK Ideograph */
  ["\u5EC2", "\u5EC2"], /* CJK Ideograph */
  ["\u5EC3", "\u5EC3"], /* CJK Ideograph */
  ["\u5EC4", "\u5EC4"], /* CJK Ideograph */
  ["\u5EC5", "\u5EC5"], /* CJK Ideograph */
  ["\u5EC6", "\u5EC6"], /* CJK Ideograph */
  ["\u5EC7", "\u5EC7"], /* CJK Ideograph */
  ["\u5EC8", "\u5EC8"], /* CJK Ideograph */
  ["\u5EC9", "\u5EC9"], /* CJK Ideograph */
  ["\u5ECA", "\u5ECA"], /* CJK Ideograph */
  ["\u5ECB", "\u5ECB"], /* CJK Ideograph */
  ["\u5ECC", "\u5ECC"], /* CJK Ideograph */
  ["\u5ECD", "\u5ECD"], /* CJK Ideograph */
  ["\u5ECE", "\u5ECE"], /* CJK Ideograph */
  ["\u5ECF", "\u5ECF"], /* CJK Ideograph */
  ["\u5ED0", "\u5ED0"], /* CJK Ideograph */
  ["\u5ED1", "\u5ED1"], /* CJK Ideograph */
  ["\u5ED2", "\u5ED2"], /* CJK Ideograph */
  ["\u5ED3", "\u5ED3"], /* CJK Ideograph */
  ["\u5ED4", "\u5ED4"], /* CJK Ideograph */
  ["\u5ED5", "\u5ED5"], /* CJK Ideograph */
  ["\u5ED6", "\u5ED6"], /* CJK Ideograph */
  ["\u5ED7", "\u5ED7"], /* CJK Ideograph */
  ["\u5ED8", "\u5ED8"], /* CJK Ideograph */
  ["\u5ED9", "\u5ED9"], /* CJK Ideograph */
  ["\u5EDA", "\u5EDA"], /* CJK Ideograph */
  ["\u5EDB", "\u5EDB"], /* CJK Ideograph */
  ["\u5EDC", "\u5EDC"], /* CJK Ideograph */
  ["\u5EDD", "\u5EDD"], /* CJK Ideograph */
  ["\u5EDE", "\u5EDE"], /* CJK Ideograph */
  ["\u5EDF", "\u5EDF"], /* CJK Ideograph */
  ["\u5EE0", "\u5EE0"], /* CJK Ideograph */
  ["\u5EE1", "\u5EE1"], /* CJK Ideograph */
  ["\u5EE2", "\u5EE2"], /* CJK Ideograph */
  ["\u5EE3", "\u5EE3"], /* CJK Ideograph */
  ["\u5EE4", "\u5EE4"], /* CJK Ideograph */
  ["\u5EE5", "\u5EE5"], /* CJK Ideograph */
  ["\u5EE6", "\u5EE6"], /* CJK Ideograph */
  ["\u5EE7", "\u5EE7"], /* CJK Ideograph */
  ["\u5EE8", "\u5EE8"], /* CJK Ideograph */
  ["\u5EE9", "\u5EE9"], /* CJK Ideograph */
  ["\u5EEA", "\u5EEA"], /* CJK Ideograph */
  ["\u5EEB", "\u5EEB"], /* CJK Ideograph */
  ["\u5EEC", "\u5EEC"], /* CJK Ideograph */
  ["\u5EED", "\u5EED"], /* CJK Ideograph */
  ["\u5EEE", "\u5EEE"], /* CJK Ideograph */
  ["\u5EEF", "\u5EEF"], /* CJK Ideograph */
  ["\u5EF0", "\u5EF0"], /* CJK Ideograph */
  ["\u5EF1", "\u5EF1"], /* CJK Ideograph */
  ["\u5EF2", "\u5EF2"], /* CJK Ideograph */
  ["\u5EF3", "\u5EF3"], /* CJK Ideograph */
  ["\u5EF4", "\u5EF4"], /* CJK Ideograph */
  ["\u5EF5", "\u5EF5"], /* CJK Ideograph */
  ["\u5EF6", "\u5EF6"], /* CJK Ideograph */
  ["\u5EF7", "\u5EF7"], /* CJK Ideograph */
  ["\u5EF8", "\u5EF8"], /* CJK Ideograph */
  ["\u5EF9", "\u5EF9"], /* CJK Ideograph */
  ["\u5EFA", "\u5EFA"], /* CJK Ideograph */
  ["\u5EFB", "\u5EFB"], /* CJK Ideograph */
  ["\u5EFC", "\u5EFC"], /* CJK Ideograph */
  ["\u5EFD", "\u5EFD"], /* CJK Ideograph */
  ["\u5EFE", "\u5EFE"], /* CJK Ideograph */
  ["\u5EFF", "\u5EFF"], /* CJK Ideograph */
  ["\u5F00", "\u5F00"], /* CJK Ideograph */
  ["\u5F01", "\u5F01"], /* CJK Ideograph */
  ["\u5F02", "\u5F02"], /* CJK Ideograph */
  ["\u5F03", "\u5F03"], /* CJK Ideograph */
  ["\u5F04", "\u5F04"], /* CJK Ideograph */
  ["\u5F05", "\u5F05"], /* CJK Ideograph */
  ["\u5F06", "\u5F06"], /* CJK Ideograph */
  ["\u5F07", "\u5F07"], /* CJK Ideograph */
  ["\u5F08", "\u5F08"], /* CJK Ideograph */
  ["\u5F09", "\u5F09"], /* CJK Ideograph */
  ["\u5F0A", "\u5F0A"], /* CJK Ideograph */
  ["\u5F0B", "\u5F0B"], /* CJK Ideograph */
  ["\u5F0C", "\u5F0C"], /* CJK Ideograph */
  ["\u5F0D", "\u5F0D"], /* CJK Ideograph */
  ["\u5F0E", "\u5F0E"], /* CJK Ideograph */
  ["\u5F0F", "\u5F0F"], /* CJK Ideograph */
  ["\u5F10", "\u5F10"], /* CJK Ideograph */
  ["\u5F11", "\u5F11"], /* CJK Ideograph */
  ["\u5F12", "\u5F12"], /* CJK Ideograph */
  ["\u5F13", "\u5F13"], /* CJK Ideograph */
  ["\u5F14", "\u5F14"], /* CJK Ideograph */
  ["\u5F15", "\u5F15"], /* CJK Ideograph */
  ["\u5F16", "\u5F16"], /* CJK Ideograph */
  ["\u5F17", "\u5F17"], /* CJK Ideograph */
  ["\u5F18", "\u5F18"], /* CJK Ideograph */
  ["\u5F19", "\u5F19"], /* CJK Ideograph */
  ["\u5F1A", "\u5F1A"], /* CJK Ideograph */
  ["\u5F1B", "\u5F1B"], /* CJK Ideograph */
  ["\u5F1C", "\u5F1C"], /* CJK Ideograph */
  ["\u5F1D", "\u5F1D"], /* CJK Ideograph */
  ["\u5F1E", "\u5F1E"], /* CJK Ideograph */
  ["\u5F1F", "\u5F1F"], /* CJK Ideograph */
  ["\u5F20", "\u5F20"], /* CJK Ideograph */
  ["\u5F21", "\u5F21"], /* CJK Ideograph */
  ["\u5F22", "\u5F22"], /* CJK Ideograph */
  ["\u5F23", "\u5F23"], /* CJK Ideograph */
  ["\u5F24", "\u5F24"], /* CJK Ideograph */
  ["\u5F25", "\u5F25"], /* CJK Ideograph */
  ["\u5F26", "\u5F26"], /* CJK Ideograph */
  ["\u5F27", "\u5F27"], /* CJK Ideograph */
  ["\u5F28", "\u5F28"], /* CJK Ideograph */
  ["\u5F29", "\u5F29"], /* CJK Ideograph */
  ["\u5F2A", "\u5F2A"], /* CJK Ideograph */
  ["\u5F2B", "\u5F2B"], /* CJK Ideograph */
  ["\u5F2C", "\u5F2C"], /* CJK Ideograph */
  ["\u5F2D", "\u5F2D"], /* CJK Ideograph */
  ["\u5F2E", "\u5F2E"], /* CJK Ideograph */
  ["\u5F2F", "\u5F2F"], /* CJK Ideograph */
  ["\u5F30", "\u5F30"], /* CJK Ideograph */
  ["\u5F31", "\u5F31"], /* CJK Ideograph */
  ["\u5F32", "\u5F32"], /* CJK Ideograph */
  ["\u5F33", "\u5F33"], /* CJK Ideograph */
  ["\u5F34", "\u5F34"], /* CJK Ideograph */
  ["\u5F35", "\u5F35"], /* CJK Ideograph */
  ["\u5F36", "\u5F36"], /* CJK Ideograph */
  ["\u5F37", "\u5F37"], /* CJK Ideograph */
  ["\u5F38", "\u5F38"], /* CJK Ideograph */
  ["\u5F39", "\u5F39"], /* CJK Ideograph */
  ["\u5F3A", "\u5F3A"], /* CJK Ideograph */
  ["\u5F3B", "\u5F3B"], /* CJK Ideograph */
  ["\u5F3C", "\u5F3C"], /* CJK Ideograph */
  ["\u5F3D", "\u5F3D"], /* CJK Ideograph */
  ["\u5F3E", "\u5F3E"], /* CJK Ideograph */
  ["\u5F3F", "\u5F3F"], /* CJK Ideograph */
  ["\u5F40", "\u5F40"], /* CJK Ideograph */
  ["\u5F41", "\u5F41"], /* CJK Ideograph */
  ["\u5F42", "\u5F42"], /* CJK Ideograph */
  ["\u5F43", "\u5F43"], /* CJK Ideograph */
  ["\u5F44", "\u5F44"], /* CJK Ideograph */
  ["\u5F45", "\u5F45"], /* CJK Ideograph */
  ["\u5F46", "\u5F46"], /* CJK Ideograph */
  ["\u5F47", "\u5F47"], /* CJK Ideograph */
  ["\u5F48", "\u5F48"], /* CJK Ideograph */
  ["\u5F49", "\u5F49"], /* CJK Ideograph */
  ["\u5F4A", "\u5F4A"], /* CJK Ideograph */
  ["\u5F4B", "\u5F4B"], /* CJK Ideograph */
  ["\u5F4C", "\u5F4C"], /* CJK Ideograph */
  ["\u5F4D", "\u5F4D"], /* CJK Ideograph */
  ["\u5F4E", "\u5F4E"], /* CJK Ideograph */
  ["\u5F4F", "\u5F4F"], /* CJK Ideograph */
  ["\u5F50", "\u5F50"], /* CJK Ideograph */
  ["\u5F51", "\u5F51"], /* CJK Ideograph */
  ["\u5F52", "\u5F52"], /* CJK Ideograph */
  ["\u5F53", "\u5F53"], /* CJK Ideograph */
  ["\u5F54", "\u5F54"], /* CJK Ideograph */
  ["\u5F55", "\u5F55"], /* CJK Ideograph */
  ["\u5F56", "\u5F56"], /* CJK Ideograph */
  ["\u5F57", "\u5F57"], /* CJK Ideograph */
  ["\u5F58", "\u5F58"], /* CJK Ideograph */
  ["\u5F59", "\u5F59"], /* CJK Ideograph */
  ["\u5F5A", "\u5F5A"], /* CJK Ideograph */
  ["\u5F5B", "\u5F5B"], /* CJK Ideograph */
  ["\u5F5C", "\u5F5C"], /* CJK Ideograph */
  ["\u5F5D", "\u5F5D"], /* CJK Ideograph */
  ["\u5F5E", "\u5F5E"], /* CJK Ideograph */
  ["\u5F5F", "\u5F5F"], /* CJK Ideograph */
  ["\u5F60", "\u5F60"], /* CJK Ideograph */
  ["\u5F61", "\u5F61"], /* CJK Ideograph */
  ["\u5F62", "\u5F62"], /* CJK Ideograph */
  ["\u5F63", "\u5F63"], /* CJK Ideograph */
  ["\u5F64", "\u5F64"], /* CJK Ideograph */
  ["\u5F65", "\u5F65"], /* CJK Ideograph */
  ["\u5F66", "\u5F66"], /* CJK Ideograph */
  ["\u5F67", "\u5F67"], /* CJK Ideograph */
  ["\u5F68", "\u5F68"], /* CJK Ideograph */
  ["\u5F69", "\u5F69"], /* CJK Ideograph */
  ["\u5F6A", "\u5F6A"], /* CJK Ideograph */
  ["\u5F6B", "\u5F6B"], /* CJK Ideograph */
  ["\u5F6C", "\u5F6C"], /* CJK Ideograph */
  ["\u5F6D", "\u5F6D"], /* CJK Ideograph */
  ["\u5F6E", "\u5F6E"], /* CJK Ideograph */
  ["\u5F6F", "\u5F6F"], /* CJK Ideograph */
  ["\u5F70", "\u5F70"], /* CJK Ideograph */
  ["\u5F71", "\u5F71"], /* CJK Ideograph */
  ["\u5F72", "\u5F72"], /* CJK Ideograph */
  ["\u5F73", "\u5F73"], /* CJK Ideograph */
  ["\u5F74", "\u5F74"], /* CJK Ideograph */
  ["\u5F75", "\u5F75"], /* CJK Ideograph */
  ["\u5F76", "\u5F76"], /* CJK Ideograph */
  ["\u5F77", "\u5F77"], /* CJK Ideograph */
  ["\u5F78", "\u5F78"], /* CJK Ideograph */
  ["\u5F79", "\u5F79"], /* CJK Ideograph */
  ["\u5F7A", "\u5F7A"], /* CJK Ideograph */
  ["\u5F7B", "\u5F7B"], /* CJK Ideograph */
  ["\u5F7C", "\u5F7C"], /* CJK Ideograph */
  ["\u5F7D", "\u5F7D"], /* CJK Ideograph */
  ["\u5F7E", "\u5F7E"], /* CJK Ideograph */
  ["\u5F7F", "\u5F7F"], /* CJK Ideograph */
  ["\u5F80", "\u5F80"], /* CJK Ideograph */
  ["\u5F81", "\u5F81"], /* CJK Ideograph */
  ["\u5F82", "\u5F82"], /* CJK Ideograph */
  ["\u5F83", "\u5F83"], /* CJK Ideograph */
  ["\u5F84", "\u5F84"], /* CJK Ideograph */
  ["\u5F85", "\u5F85"], /* CJK Ideograph */
  ["\u5F86", "\u5F86"], /* CJK Ideograph */
  ["\u5F87", "\u5F87"], /* CJK Ideograph */
  ["\u5F88", "\u5F88"], /* CJK Ideograph */
  ["\u5F89", "\u5F89"], /* CJK Ideograph */
  ["\u5F8A", "\u5F8A"], /* CJK Ideograph */
  ["\u5F8B", "\u5F8B"], /* CJK Ideograph */
  ["\u5F8C", "\u5F8C"], /* CJK Ideograph */
  ["\u5F8D", "\u5F8D"], /* CJK Ideograph */
  ["\u5F8E", "\u5F8E"], /* CJK Ideograph */
  ["\u5F8F", "\u5F8F"], /* CJK Ideograph */
  ["\u5F90", "\u5F90"], /* CJK Ideograph */
  ["\u5F91", "\u5F91"], /* CJK Ideograph */
  ["\u5F92", "\u5F92"], /* CJK Ideograph */
  ["\u5F93", "\u5F93"], /* CJK Ideograph */
  ["\u5F94", "\u5F94"], /* CJK Ideograph */
  ["\u5F95", "\u5F95"], /* CJK Ideograph */
  ["\u5F96", "\u5F96"], /* CJK Ideograph */
  ["\u5F97", "\u5F97"], /* CJK Ideograph */
  ["\u5F98", "\u5F98"], /* CJK Ideograph */
  ["\u5F99", "\u5F99"], /* CJK Ideograph */
  ["\u5F9A", "\u5F9A"], /* CJK Ideograph */
  ["\u5F9B", "\u5F9B"], /* CJK Ideograph */
  ["\u5F9C", "\u5F9C"], /* CJK Ideograph */
  ["\u5F9D", "\u5F9D"], /* CJK Ideograph */
  ["\u5F9E", "\u5F9E"], /* CJK Ideograph */
  ["\u5F9F", "\u5F9F"], /* CJK Ideograph */
  ["\u5FA0", "\u5FA0"], /* CJK Ideograph */
  ["\u5FA1", "\u5FA1"], /* CJK Ideograph */
  ["\u5FA2", "\u5FA2"], /* CJK Ideograph */
  ["\u5FA3", "\u5FA3"], /* CJK Ideograph */
  ["\u5FA4", "\u5FA4"], /* CJK Ideograph */
  ["\u5FA5", "\u5FA5"], /* CJK Ideograph */
  ["\u5FA6", "\u5FA6"], /* CJK Ideograph */
  ["\u5FA7", "\u5FA7"], /* CJK Ideograph */
  ["\u5FA8", "\u5FA8"], /* CJK Ideograph */
  ["\u5FA9", "\u5FA9"], /* CJK Ideograph */
  ["\u5FAA", "\u5FAA"], /* CJK Ideograph */
  ["\u5FAB", "\u5FAB"], /* CJK Ideograph */
  ["\u5FAC", "\u5FAC"], /* CJK Ideograph */
  ["\u5FAD", "\u5FAD"], /* CJK Ideograph */
  ["\u5FAE", "\u5FAE"], /* CJK Ideograph */
  ["\u5FAF", "\u5FAF"], /* CJK Ideograph */
  ["\u5FB0", "\u5FB0"], /* CJK Ideograph */
  ["\u5FB1", "\u5FB1"], /* CJK Ideograph */
  ["\u5FB2", "\u5FB2"], /* CJK Ideograph */
  ["\u5FB3", "\u5FB3"], /* CJK Ideograph */
  ["\u5FB4", "\u5FB4"], /* CJK Ideograph */
  ["\u5FB5", "\u5FB5"], /* CJK Ideograph */
  ["\u5FB6", "\u5FB6"], /* CJK Ideograph */
  ["\u5FB7", "\u5FB7"], /* CJK Ideograph */
  ["\u5FB8", "\u5FB8"], /* CJK Ideograph */
  ["\u5FB9", "\u5FB9"], /* CJK Ideograph */
  ["\u5FBA", "\u5FBA"], /* CJK Ideograph */
  ["\u5FBB", "\u5FBB"], /* CJK Ideograph */
  ["\u5FBC", "\u5FBC"], /* CJK Ideograph */
  ["\u5FBD", "\u5FBD"], /* CJK Ideograph */
  ["\u5FBE", "\u5FBE"], /* CJK Ideograph */
  ["\u5FBF", "\u5FBF"], /* CJK Ideograph */
  ["\u5FC0", "\u5FC0"], /* CJK Ideograph */
  ["\u5FC1", "\u5FC1"], /* CJK Ideograph */
  ["\u5FC2", "\u5FC2"], /* CJK Ideograph */
  ["\u5FC3", "\u5FC3"], /* CJK Ideograph */
  ["\u5FC4", "\u5FC4"], /* CJK Ideograph */
  ["\u5FC5", "\u5FC5"], /* CJK Ideograph */
  ["\u5FC6", "\u5FC6"], /* CJK Ideograph */
  ["\u5FC7", "\u5FC7"], /* CJK Ideograph */
  ["\u5FC8", "\u5FC8"], /* CJK Ideograph */
  ["\u5FC9", "\u5FC9"], /* CJK Ideograph */
  ["\u5FCA", "\u5FCA"], /* CJK Ideograph */
  ["\u5FCB", "\u5FCB"], /* CJK Ideograph */
  ["\u5FCC", "\u5FCC"], /* CJK Ideograph */
  ["\u5FCD", "\u5FCD"], /* CJK Ideograph */
  ["\u5FCE", "\u5FCE"], /* CJK Ideograph */
  ["\u5FCF", "\u5FCF"], /* CJK Ideograph */
  ["\u5FD0", "\u5FD0"], /* CJK Ideograph */
  ["\u5FD1", "\u5FD1"], /* CJK Ideograph */
  ["\u5FD2", "\u5FD2"], /* CJK Ideograph */
  ["\u5FD3", "\u5FD3"], /* CJK Ideograph */
  ["\u5FD4", "\u5FD4"], /* CJK Ideograph */
  ["\u5FD5", "\u5FD5"], /* CJK Ideograph */
  ["\u5FD6", "\u5FD6"], /* CJK Ideograph */
  ["\u5FD7", "\u5FD7"], /* CJK Ideograph */
  ["\u5FD8", "\u5FD8"], /* CJK Ideograph */
  ["\u5FD9", "\u5FD9"], /* CJK Ideograph */
  ["\u5FDA", "\u5FDA"], /* CJK Ideograph */
  ["\u5FDB", "\u5FDB"], /* CJK Ideograph */
  ["\u5FDC", "\u5FDC"], /* CJK Ideograph */
  ["\u5FDD", "\u5FDD"], /* CJK Ideograph */
  ["\u5FDE", "\u5FDE"], /* CJK Ideograph */
  ["\u5FDF", "\u5FDF"], /* CJK Ideograph */
  ["\u5FE0", "\u5FE0"], /* CJK Ideograph */
  ["\u5FE1", "\u5FE1"], /* CJK Ideograph */
  ["\u5FE2", "\u5FE2"], /* CJK Ideograph */
  ["\u5FE3", "\u5FE3"], /* CJK Ideograph */
  ["\u5FE4", "\u5FE4"], /* CJK Ideograph */
  ["\u5FE5", "\u5FE5"], /* CJK Ideograph */
  ["\u5FE6", "\u5FE6"], /* CJK Ideograph */
  ["\u5FE7", "\u5FE7"], /* CJK Ideograph */
  ["\u5FE8", "\u5FE8"], /* CJK Ideograph */
  ["\u5FE9", "\u5FE9"], /* CJK Ideograph */
  ["\u5FEA", "\u5FEA"], /* CJK Ideograph */
  ["\u5FEB", "\u5FEB"], /* CJK Ideograph */
  ["\u5FEC", "\u5FEC"], /* CJK Ideograph */
  ["\u5FED", "\u5FED"], /* CJK Ideograph */
  ["\u5FEE", "\u5FEE"], /* CJK Ideograph */
  ["\u5FEF", "\u5FEF"], /* CJK Ideograph */
  ["\u5FF0", "\u5FF0"], /* CJK Ideograph */
  ["\u5FF1", "\u5FF1"], /* CJK Ideograph */
  ["\u5FF2", "\u5FF2"], /* CJK Ideograph */
  ["\u5FF3", "\u5FF3"], /* CJK Ideograph */
  ["\u5FF4", "\u5FF4"], /* CJK Ideograph */
  ["\u5FF5", "\u5FF5"], /* CJK Ideograph */
  ["\u5FF6", "\u5FF6"], /* CJK Ideograph */
  ["\u5FF7", "\u5FF7"], /* CJK Ideograph */
  ["\u5FF8", "\u5FF8"], /* CJK Ideograph */
  ["\u5FF9", "\u5FF9"], /* CJK Ideograph */
  ["\u5FFA", "\u5FFA"], /* CJK Ideograph */
  ["\u5FFB", "\u5FFB"], /* CJK Ideograph */
  ["\u5FFC", "\u5FFC"], /* CJK Ideograph */
  ["\u5FFD", "\u5FFD"], /* CJK Ideograph */
  ["\u5FFE", "\u5FFE"], /* CJK Ideograph */
  ["\u5FFF", "\u5FFF"], /* CJK Ideograph */
  ["\u6000", "\u6000"], /* CJK Ideograph */
  ["\u6001", "\u6001"], /* CJK Ideograph */
  ["\u6002", "\u6002"], /* CJK Ideograph */
  ["\u6003", "\u6003"], /* CJK Ideograph */
  ["\u6004", "\u6004"], /* CJK Ideograph */
  ["\u6005", "\u6005"], /* CJK Ideograph */
  ["\u6006", "\u6006"], /* CJK Ideograph */
  ["\u6007", "\u6007"], /* CJK Ideograph */
  ["\u6008", "\u6008"], /* CJK Ideograph */
  ["\u6009", "\u6009"], /* CJK Ideograph */
  ["\u600A", "\u600A"], /* CJK Ideograph */
  ["\u600B", "\u600B"], /* CJK Ideograph */
  ["\u600C", "\u600C"], /* CJK Ideograph */
  ["\u600D", "\u600D"], /* CJK Ideograph */
  ["\u600E", "\u600E"], /* CJK Ideograph */
  ["\u600F", "\u600F"], /* CJK Ideograph */
  ["\u6010", "\u6010"], /* CJK Ideograph */
  ["\u6011", "\u6011"], /* CJK Ideograph */
  ["\u6012", "\u6012"], /* CJK Ideograph */
  ["\u6013", "\u6013"], /* CJK Ideograph */
  ["\u6014", "\u6014"], /* CJK Ideograph */
  ["\u6015", "\u6015"], /* CJK Ideograph */
  ["\u6016", "\u6016"], /* CJK Ideograph */
  ["\u6017", "\u6017"], /* CJK Ideograph */
  ["\u6018", "\u6018"], /* CJK Ideograph */
  ["\u6019", "\u6019"], /* CJK Ideograph */
  ["\u601A", "\u601A"], /* CJK Ideograph */
  ["\u601B", "\u601B"], /* CJK Ideograph */
  ["\u601C", "\u601C"], /* CJK Ideograph */
  ["\u601D", "\u601D"], /* CJK Ideograph */
  ["\u601E", "\u601E"], /* CJK Ideograph */
  ["\u601F", "\u601F"], /* CJK Ideograph */
  ["\u6020", "\u6020"], /* CJK Ideograph */
  ["\u6021", "\u6021"], /* CJK Ideograph */
  ["\u6022", "\u6022"], /* CJK Ideograph */
  ["\u6023", "\u6023"], /* CJK Ideograph */
  ["\u6024", "\u6024"], /* CJK Ideograph */
  ["\u6025", "\u6025"], /* CJK Ideograph */
  ["\u6026", "\u6026"], /* CJK Ideograph */
  ["\u6027", "\u6027"], /* CJK Ideograph */
  ["\u6028", "\u6028"], /* CJK Ideograph */
  ["\u6029", "\u6029"], /* CJK Ideograph */
  ["\u602A", "\u602A"], /* CJK Ideograph */
  ["\u602B", "\u602B"], /* CJK Ideograph */
  ["\u602C", "\u602C"], /* CJK Ideograph */
  ["\u602D", "\u602D"], /* CJK Ideograph */
  ["\u602E", "\u602E"], /* CJK Ideograph */
  ["\u602F", "\u602F"], /* CJK Ideograph */
  ["\u6030", "\u6030"], /* CJK Ideograph */
  ["\u6031", "\u6031"], /* CJK Ideograph */
  ["\u6032", "\u6032"], /* CJK Ideograph */
  ["\u6033", "\u6033"], /* CJK Ideograph */
  ["\u6034", "\u6034"], /* CJK Ideograph */
  ["\u6035", "\u6035"], /* CJK Ideograph */
  ["\u6036", "\u6036"], /* CJK Ideograph */
  ["\u6037", "\u6037"], /* CJK Ideograph */
  ["\u6038", "\u6038"], /* CJK Ideograph */
  ["\u6039", "\u6039"], /* CJK Ideograph */
  ["\u603A", "\u603A"], /* CJK Ideograph */
  ["\u603B", "\u603B"], /* CJK Ideograph */
  ["\u603C", "\u603C"], /* CJK Ideograph */
  ["\u603D", "\u603D"], /* CJK Ideograph */
  ["\u603E", "\u603E"], /* CJK Ideograph */
  ["\u603F", "\u603F"], /* CJK Ideograph */
  ["\u6040", "\u6040"], /* CJK Ideograph */
  ["\u6041", "\u6041"], /* CJK Ideograph */
  ["\u6042", "\u6042"], /* CJK Ideograph */
  ["\u6043", "\u6043"], /* CJK Ideograph */
  ["\u6044", "\u6044"], /* CJK Ideograph */
  ["\u6045", "\u6045"], /* CJK Ideograph */
  ["\u6046", "\u6046"], /* CJK Ideograph */
  ["\u6047", "\u6047"], /* CJK Ideograph */
  ["\u6048", "\u6048"], /* CJK Ideograph */
  ["\u6049", "\u6049"], /* CJK Ideograph */
  ["\u604A", "\u604A"], /* CJK Ideograph */
  ["\u604B", "\u604B"], /* CJK Ideograph */
  ["\u604C", "\u604C"], /* CJK Ideograph */
  ["\u604D", "\u604D"], /* CJK Ideograph */
  ["\u604E", "\u604E"], /* CJK Ideograph */
  ["\u604F", "\u604F"], /* CJK Ideograph */
  ["\u6050", "\u6050"], /* CJK Ideograph */
  ["\u6051", "\u6051"], /* CJK Ideograph */
  ["\u6052", "\u6052"], /* CJK Ideograph */
  ["\u6053", "\u6053"], /* CJK Ideograph */
  ["\u6054", "\u6054"], /* CJK Ideograph */
  ["\u6055", "\u6055"], /* CJK Ideograph */
  ["\u6056", "\u6056"], /* CJK Ideograph */
  ["\u6057", "\u6057"], /* CJK Ideograph */
  ["\u6058", "\u6058"], /* CJK Ideograph */
  ["\u6059", "\u6059"], /* CJK Ideograph */
  ["\u605A", "\u605A"], /* CJK Ideograph */
  ["\u605B", "\u605B"], /* CJK Ideograph */
  ["\u605C", "\u605C"], /* CJK Ideograph */
  ["\u605D", "\u605D"], /* CJK Ideograph */
  ["\u605E", "\u605E"], /* CJK Ideograph */
  ["\u605F", "\u605F"], /* CJK Ideograph */
  ["\u6060", "\u6060"], /* CJK Ideograph */
  ["\u6061", "\u6061"], /* CJK Ideograph */
  ["\u6062", "\u6062"], /* CJK Ideograph */
  ["\u6063", "\u6063"], /* CJK Ideograph */
  ["\u6064", "\u6064"], /* CJK Ideograph */
  ["\u6065", "\u6065"], /* CJK Ideograph */
  ["\u6066", "\u6066"], /* CJK Ideograph */
  ["\u6067", "\u6067"], /* CJK Ideograph */
  ["\u6068", "\u6068"], /* CJK Ideograph */
  ["\u6069", "\u6069"], /* CJK Ideograph */
  ["\u606A", "\u606A"], /* CJK Ideograph */
  ["\u606B", "\u606B"], /* CJK Ideograph */
  ["\u606C", "\u606C"], /* CJK Ideograph */
  ["\u606D", "\u606D"], /* CJK Ideograph */
  ["\u606E", "\u606E"], /* CJK Ideograph */
  ["\u606F", "\u606F"], /* CJK Ideograph */
  ["\u6070", "\u6070"], /* CJK Ideograph */
  ["\u6071", "\u6071"], /* CJK Ideograph */
  ["\u6072", "\u6072"], /* CJK Ideograph */
  ["\u6073", "\u6073"], /* CJK Ideograph */
  ["\u6074", "\u6074"], /* CJK Ideograph */
  ["\u6075", "\u6075"], /* CJK Ideograph */
  ["\u6076", "\u6076"], /* CJK Ideograph */
  ["\u6077", "\u6077"], /* CJK Ideograph */
  ["\u6078", "\u6078"], /* CJK Ideograph */
  ["\u6079", "\u6079"], /* CJK Ideograph */
  ["\u607A", "\u607A"], /* CJK Ideograph */
  ["\u607B", "\u607B"], /* CJK Ideograph */
  ["\u607C", "\u607C"], /* CJK Ideograph */
  ["\u607D", "\u607D"], /* CJK Ideograph */
  ["\u607E", "\u607E"], /* CJK Ideograph */
  ["\u607F", "\u607F"], /* CJK Ideograph */
  ["\u6080", "\u6080"], /* CJK Ideograph */
  ["\u6081", "\u6081"], /* CJK Ideograph */
  ["\u6082", "\u6082"], /* CJK Ideograph */
  ["\u6083", "\u6083"], /* CJK Ideograph */
  ["\u6084", "\u6084"], /* CJK Ideograph */
  ["\u6085", "\u6085"], /* CJK Ideograph */
  ["\u6086", "\u6086"], /* CJK Ideograph */
  ["\u6087", "\u6087"], /* CJK Ideograph */
  ["\u6088", "\u6088"], /* CJK Ideograph */
  ["\u6089", "\u6089"], /* CJK Ideograph */
  ["\u608A", "\u608A"], /* CJK Ideograph */
  ["\u608B", "\u608B"], /* CJK Ideograph */
  ["\u608C", "\u608C"], /* CJK Ideograph */
  ["\u608D", "\u608D"], /* CJK Ideograph */
  ["\u608E", "\u608E"], /* CJK Ideograph */
  ["\u608F", "\u608F"], /* CJK Ideograph */
  ["\u6090", "\u6090"], /* CJK Ideograph */
  ["\u6091", "\u6091"], /* CJK Ideograph */
  ["\u6092", "\u6092"], /* CJK Ideograph */
  ["\u6093", "\u6093"], /* CJK Ideograph */
  ["\u6094", "\u6094"], /* CJK Ideograph */
  ["\u6095", "\u6095"], /* CJK Ideograph */
  ["\u6096", "\u6096"], /* CJK Ideograph */
  ["\u6097", "\u6097"], /* CJK Ideograph */
  ["\u6098", "\u6098"], /* CJK Ideograph */
  ["\u6099", "\u6099"], /* CJK Ideograph */
  ["\u609A", "\u609A"], /* CJK Ideograph */
  ["\u609B", "\u609B"], /* CJK Ideograph */
  ["\u609C", "\u609C"], /* CJK Ideograph */
  ["\u609D", "\u609D"], /* CJK Ideograph */
  ["\u609E", "\u609E"], /* CJK Ideograph */
  ["\u609F", "\u609F"], /* CJK Ideograph */
  ["\u60A0", "\u60A0"], /* CJK Ideograph */
  ["\u60A1", "\u60A1"], /* CJK Ideograph */
  ["\u60A2", "\u60A2"], /* CJK Ideograph */
  ["\u60A3", "\u60A3"], /* CJK Ideograph */
  ["\u60A4", "\u60A4"], /* CJK Ideograph */
  ["\u60A5", "\u60A5"], /* CJK Ideograph */
  ["\u60A6", "\u60A6"], /* CJK Ideograph */
  ["\u60A7", "\u60A7"], /* CJK Ideograph */
  ["\u60A8", "\u60A8"], /* CJK Ideograph */
  ["\u60A9", "\u60A9"], /* CJK Ideograph */
  ["\u60AA", "\u60AA"], /* CJK Ideograph */
  ["\u60AB", "\u60AB"], /* CJK Ideograph */
  ["\u60AC", "\u60AC"], /* CJK Ideograph */
  ["\u60AD", "\u60AD"], /* CJK Ideograph */
  ["\u60AE", "\u60AE"], /* CJK Ideograph */
  ["\u60AF", "\u60AF"], /* CJK Ideograph */
  ["\u60B0", "\u60B0"], /* CJK Ideograph */
  ["\u60B1", "\u60B1"], /* CJK Ideograph */
  ["\u60B2", "\u60B2"], /* CJK Ideograph */
  ["\u60B3", "\u60B3"], /* CJK Ideograph */
  ["\u60B4", "\u60B4"], /* CJK Ideograph */
  ["\u60B5", "\u60B5"], /* CJK Ideograph */
  ["\u60B6", "\u60B6"], /* CJK Ideograph */
  ["\u60B7", "\u60B7"], /* CJK Ideograph */
  ["\u60B8", "\u60B8"], /* CJK Ideograph */
  ["\u60B9", "\u60B9"], /* CJK Ideograph */
  ["\u60BA", "\u60BA"], /* CJK Ideograph */
  ["\u60BB", "\u60BB"], /* CJK Ideograph */
  ["\u60BC", "\u60BC"], /* CJK Ideograph */
  ["\u60BD", "\u60BD"], /* CJK Ideograph */
  ["\u60BE", "\u60BE"], /* CJK Ideograph */
  ["\u60BF", "\u60BF"], /* CJK Ideograph */
  ["\u60C0", "\u60C0"], /* CJK Ideograph */
  ["\u60C1", "\u60C1"], /* CJK Ideograph */
  ["\u60C2", "\u60C2"], /* CJK Ideograph */
  ["\u60C3", "\u60C3"], /* CJK Ideograph */
  ["\u60C4", "\u60C4"], /* CJK Ideograph */
  ["\u60C5", "\u60C5"], /* CJK Ideograph */
  ["\u60C6", "\u60C6"], /* CJK Ideograph */
  ["\u60C7", "\u60C7"], /* CJK Ideograph */
  ["\u60C8", "\u60C8"], /* CJK Ideograph */
  ["\u60C9", "\u60C9"], /* CJK Ideograph */
  ["\u60CA", "\u60CA"], /* CJK Ideograph */
  ["\u60CB", "\u60CB"], /* CJK Ideograph */
  ["\u60CC", "\u60CC"], /* CJK Ideograph */
  ["\u60CD", "\u60CD"], /* CJK Ideograph */
  ["\u60CE", "\u60CE"], /* CJK Ideograph */
  ["\u60CF", "\u60CF"], /* CJK Ideograph */
  ["\u60D0", "\u60D0"], /* CJK Ideograph */
  ["\u60D1", "\u60D1"], /* CJK Ideograph */
  ["\u60D2", "\u60D2"], /* CJK Ideograph */
  ["\u60D3", "\u60D3"], /* CJK Ideograph */
  ["\u60D4", "\u60D4"], /* CJK Ideograph */
  ["\u60D5", "\u60D5"], /* CJK Ideograph */
  ["\u60D6", "\u60D6"], /* CJK Ideograph */
  ["\u60D7", "\u60D7"], /* CJK Ideograph */
  ["\u60D8", "\u60D8"], /* CJK Ideograph */
  ["\u60D9", "\u60D9"], /* CJK Ideograph */
  ["\u60DA", "\u60DA"], /* CJK Ideograph */
  ["\u60DB", "\u60DB"], /* CJK Ideograph */
  ["\u60DC", "\u60DC"], /* CJK Ideograph */
  ["\u60DD", "\u60DD"], /* CJK Ideograph */
  ["\u60DE", "\u60DE"], /* CJK Ideograph */
  ["\u60DF", "\u60DF"], /* CJK Ideograph */
  ["\u60E0", "\u60E0"], /* CJK Ideograph */
  ["\u60E1", "\u60E1"], /* CJK Ideograph */
  ["\u60E2", "\u60E2"], /* CJK Ideograph */
  ["\u60E3", "\u60E3"], /* CJK Ideograph */
  ["\u60E4", "\u60E4"], /* CJK Ideograph */
  ["\u60E5", "\u60E5"], /* CJK Ideograph */
  ["\u60E6", "\u60E6"], /* CJK Ideograph */
  ["\u60E7", "\u60E7"], /* CJK Ideograph */
  ["\u60E8", "\u60E8"], /* CJK Ideograph */
  ["\u60E9", "\u60E9"], /* CJK Ideograph */
  ["\u60EA", "\u60EA"], /* CJK Ideograph */
  ["\u60EB", "\u60EB"], /* CJK Ideograph */
  ["\u60EC", "\u60EC"], /* CJK Ideograph */
  ["\u60ED", "\u60ED"], /* CJK Ideograph */
  ["\u60EE", "\u60EE"], /* CJK Ideograph */
  ["\u60EF", "\u60EF"], /* CJK Ideograph */
  ["\u60F0", "\u60F0"], /* CJK Ideograph */
  ["\u60F1", "\u60F1"], /* CJK Ideograph */
  ["\u60F2", "\u60F2"], /* CJK Ideograph */
  ["\u60F3", "\u60F3"], /* CJK Ideograph */
  ["\u60F4", "\u60F4"], /* CJK Ideograph */
  ["\u60F5", "\u60F5"], /* CJK Ideograph */
  ["\u60F6", "\u60F6"], /* CJK Ideograph */
  ["\u60F7", "\u60F7"], /* CJK Ideograph */
  ["\u60F8", "\u60F8"], /* CJK Ideograph */
  ["\u60F9", "\u60F9"], /* CJK Ideograph */
  ["\u60FA", "\u60FA"], /* CJK Ideograph */
  ["\u60FB", "\u60FB"], /* CJK Ideograph */
  ["\u60FC", "\u60FC"], /* CJK Ideograph */
  ["\u60FD", "\u60FD"], /* CJK Ideograph */
  ["\u60FE", "\u60FE"], /* CJK Ideograph */
  ["\u60FF", "\u60FF"], /* CJK Ideograph */
  ["\u6100", "\u6100"], /* CJK Ideograph */
  ["\u6101", "\u6101"], /* CJK Ideograph */
  ["\u6102", "\u6102"], /* CJK Ideograph */
  ["\u6103", "\u6103"], /* CJK Ideograph */
  ["\u6104", "\u6104"], /* CJK Ideograph */
  ["\u6105", "\u6105"], /* CJK Ideograph */
  ["\u6106", "\u6106"], /* CJK Ideograph */
  ["\u6107", "\u6107"], /* CJK Ideograph */
  ["\u6108", "\u6108"], /* CJK Ideograph */
  ["\u6109", "\u6109"], /* CJK Ideograph */
  ["\u610A", "\u610A"], /* CJK Ideograph */
  ["\u610B", "\u610B"], /* CJK Ideograph */
  ["\u610C", "\u610C"], /* CJK Ideograph */
  ["\u610D", "\u610D"], /* CJK Ideograph */
  ["\u610E", "\u610E"], /* CJK Ideograph */
  ["\u610F", "\u610F"], /* CJK Ideograph */
  ["\u6110", "\u6110"], /* CJK Ideograph */
  ["\u6111", "\u6111"], /* CJK Ideograph */
  ["\u6112", "\u6112"], /* CJK Ideograph */
  ["\u6113", "\u6113"], /* CJK Ideograph */
  ["\u6114", "\u6114"], /* CJK Ideograph */
  ["\u6115", "\u6115"], /* CJK Ideograph */
  ["\u6116", "\u6116"], /* CJK Ideograph */
  ["\u6117", "\u6117"], /* CJK Ideograph */
  ["\u6118", "\u6118"], /* CJK Ideograph */
  ["\u6119", "\u6119"], /* CJK Ideograph */
  ["\u611A", "\u611A"], /* CJK Ideograph */
  ["\u611B", "\u611B"], /* CJK Ideograph */
  ["\u611C", "\u611C"], /* CJK Ideograph */
  ["\u611D", "\u611D"], /* CJK Ideograph */
  ["\u611E", "\u611E"], /* CJK Ideograph */
  ["\u611F", "\u611F"], /* CJK Ideograph */
  ["\u6120", "\u6120"], /* CJK Ideograph */
  ["\u6121", "\u6121"], /* CJK Ideograph */
  ["\u6122", "\u6122"], /* CJK Ideograph */
  ["\u6123", "\u6123"], /* CJK Ideograph */
  ["\u6124", "\u6124"], /* CJK Ideograph */
  ["\u6125", "\u6125"], /* CJK Ideograph */
  ["\u6126", "\u6126"], /* CJK Ideograph */
  ["\u6127", "\u6127"], /* CJK Ideograph */
  ["\u6128", "\u6128"], /* CJK Ideograph */
  ["\u6129", "\u6129"], /* CJK Ideograph */
  ["\u612A", "\u612A"], /* CJK Ideograph */
  ["\u612B", "\u612B"], /* CJK Ideograph */
  ["\u612C", "\u612C"], /* CJK Ideograph */
  ["\u612D", "\u612D"], /* CJK Ideograph */
  ["\u612E", "\u612E"], /* CJK Ideograph */
  ["\u612F", "\u612F"], /* CJK Ideograph */
  ["\u6130", "\u6130"], /* CJK Ideograph */
  ["\u6131", "\u6131"], /* CJK Ideograph */
  ["\u6132", "\u6132"], /* CJK Ideograph */
  ["\u6133", "\u6133"], /* CJK Ideograph */
  ["\u6134", "\u6134"], /* CJK Ideograph */
  ["\u6135", "\u6135"], /* CJK Ideograph */
  ["\u6136", "\u6136"], /* CJK Ideograph */
  ["\u6137", "\u6137"], /* CJK Ideograph */
  ["\u6138", "\u6138"], /* CJK Ideograph */
  ["\u6139", "\u6139"], /* CJK Ideograph */
  ["\u613A", "\u613A"], /* CJK Ideograph */
  ["\u613B", "\u613B"], /* CJK Ideograph */
  ["\u613C", "\u613C"], /* CJK Ideograph */
  ["\u613D", "\u613D"], /* CJK Ideograph */
  ["\u613E", "\u613E"], /* CJK Ideograph */
  ["\u613F", "\u613F"], /* CJK Ideograph */
  ["\u6140", "\u6140"], /* CJK Ideograph */
  ["\u6141", "\u6141"], /* CJK Ideograph */
  ["\u6142", "\u6142"], /* CJK Ideograph */
  ["\u6143", "\u6143"], /* CJK Ideograph */
  ["\u6144", "\u6144"], /* CJK Ideograph */
  ["\u6145", "\u6145"], /* CJK Ideograph */
  ["\u6146", "\u6146"], /* CJK Ideograph */
  ["\u6147", "\u6147"], /* CJK Ideograph */
  ["\u6148", "\u6148"], /* CJK Ideograph */
  ["\u6149", "\u6149"], /* CJK Ideograph */
  ["\u614A", "\u614A"], /* CJK Ideograph */
  ["\u614B", "\u614B"], /* CJK Ideograph */
  ["\u614C", "\u614C"], /* CJK Ideograph */
  ["\u614D", "\u614D"], /* CJK Ideograph */
  ["\u614E", "\u614E"], /* CJK Ideograph */
  ["\u614F", "\u614F"], /* CJK Ideograph */
  ["\u6150", "\u6150"], /* CJK Ideograph */
  ["\u6151", "\u6151"], /* CJK Ideograph */
  ["\u6152", "\u6152"], /* CJK Ideograph */
  ["\u6153", "\u6153"], /* CJK Ideograph */
  ["\u6154", "\u6154"], /* CJK Ideograph */
  ["\u6155", "\u6155"], /* CJK Ideograph */
  ["\u6156", "\u6156"], /* CJK Ideograph */
  ["\u6157", "\u6157"], /* CJK Ideograph */
  ["\u6158", "\u6158"], /* CJK Ideograph */
  ["\u6159", "\u6159"], /* CJK Ideograph */
  ["\u615A", "\u615A"], /* CJK Ideograph */
  ["\u615B", "\u615B"], /* CJK Ideograph */
  ["\u615C", "\u615C"], /* CJK Ideograph */
  ["\u615D", "\u615D"], /* CJK Ideograph */
  ["\u615E", "\u615E"], /* CJK Ideograph */
  ["\u615F", "\u615F"], /* CJK Ideograph */
  ["\u6160", "\u6160"], /* CJK Ideograph */
  ["\u6161", "\u6161"], /* CJK Ideograph */
  ["\u6162", "\u6162"], /* CJK Ideograph */
  ["\u6163", "\u6163"], /* CJK Ideograph */
  ["\u6164", "\u6164"], /* CJK Ideograph */
  ["\u6165", "\u6165"], /* CJK Ideograph */
  ["\u6166", "\u6166"], /* CJK Ideograph */
  ["\u6167", "\u6167"], /* CJK Ideograph */
  ["\u6168", "\u6168"], /* CJK Ideograph */
  ["\u6169", "\u6169"], /* CJK Ideograph */
  ["\u616A", "\u616A"], /* CJK Ideograph */
  ["\u616B", "\u616B"], /* CJK Ideograph */
  ["\u616C", "\u616C"], /* CJK Ideograph */
  ["\u616D", "\u616D"], /* CJK Ideograph */
  ["\u616E", "\u616E"], /* CJK Ideograph */
  ["\u616F", "\u616F"], /* CJK Ideograph */
  ["\u6170", "\u6170"], /* CJK Ideograph */
  ["\u6171", "\u6171"], /* CJK Ideograph */
  ["\u6172", "\u6172"], /* CJK Ideograph */
  ["\u6173", "\u6173"], /* CJK Ideograph */
  ["\u6174", "\u6174"], /* CJK Ideograph */
  ["\u6175", "\u6175"], /* CJK Ideograph */
  ["\u6176", "\u6176"], /* CJK Ideograph */
  ["\u6177", "\u6177"], /* CJK Ideograph */
  ["\u6178", "\u6178"], /* CJK Ideograph */
  ["\u6179", "\u6179"], /* CJK Ideograph */
  ["\u617A", "\u617A"], /* CJK Ideograph */
  ["\u617B", "\u617B"], /* CJK Ideograph */
  ["\u617C", "\u617C"], /* CJK Ideograph */
  ["\u617D", "\u617D"], /* CJK Ideograph */
  ["\u617E", "\u617E"], /* CJK Ideograph */
  ["\u617F", "\u617F"], /* CJK Ideograph */
  ["\u6180", "\u6180"], /* CJK Ideograph */
  ["\u6181", "\u6181"], /* CJK Ideograph */
  ["\u6182", "\u6182"], /* CJK Ideograph */
  ["\u6183", "\u6183"], /* CJK Ideograph */
  ["\u6184", "\u6184"], /* CJK Ideograph */
  ["\u6185", "\u6185"], /* CJK Ideograph */
  ["\u6186", "\u6186"], /* CJK Ideograph */
  ["\u6187", "\u6187"], /* CJK Ideograph */
  ["\u6188", "\u6188"], /* CJK Ideograph */
  ["\u6189", "\u6189"], /* CJK Ideograph */
  ["\u618A", "\u618A"], /* CJK Ideograph */
  ["\u618B", "\u618B"], /* CJK Ideograph */
  ["\u618C", "\u618C"], /* CJK Ideograph */
  ["\u618D", "\u618D"], /* CJK Ideograph */
  ["\u618E", "\u618E"], /* CJK Ideograph */
  ["\u618F", "\u618F"], /* CJK Ideograph */
  ["\u6190", "\u6190"], /* CJK Ideograph */
  ["\u6191", "\u6191"], /* CJK Ideograph */
  ["\u6192", "\u6192"], /* CJK Ideograph */
  ["\u6193", "\u6193"], /* CJK Ideograph */
  ["\u6194", "\u6194"], /* CJK Ideograph */
  ["\u6195", "\u6195"], /* CJK Ideograph */
  ["\u6196", "\u6196"], /* CJK Ideograph */
  ["\u6197", "\u6197"], /* CJK Ideograph */
  ["\u6198", "\u6198"], /* CJK Ideograph */
  ["\u6199", "\u6199"], /* CJK Ideograph */
  ["\u619A", "\u619A"], /* CJK Ideograph */
  ["\u619B", "\u619B"], /* CJK Ideograph */
  ["\u619C", "\u619C"], /* CJK Ideograph */
  ["\u619D", "\u619D"], /* CJK Ideograph */
  ["\u619E", "\u619E"], /* CJK Ideograph */
  ["\u619F", "\u619F"], /* CJK Ideograph */
  ["\u61A0", "\u61A0"], /* CJK Ideograph */
  ["\u61A1", "\u61A1"], /* CJK Ideograph */
  ["\u61A2", "\u61A2"], /* CJK Ideograph */
  ["\u61A3", "\u61A3"], /* CJK Ideograph */
  ["\u61A4", "\u61A4"], /* CJK Ideograph */
  ["\u61A5", "\u61A5"], /* CJK Ideograph */
  ["\u61A6", "\u61A6"], /* CJK Ideograph */
  ["\u61A7", "\u61A7"], /* CJK Ideograph */
  ["\u61A8", "\u61A8"], /* CJK Ideograph */
  ["\u61A9", "\u61A9"], /* CJK Ideograph */
  ["\u61AA", "\u61AA"], /* CJK Ideograph */
  ["\u61AB", "\u61AB"], /* CJK Ideograph */
  ["\u61AC", "\u61AC"], /* CJK Ideograph */
  ["\u61AD", "\u61AD"], /* CJK Ideograph */
  ["\u61AE", "\u61AE"], /* CJK Ideograph */
  ["\u61AF", "\u61AF"], /* CJK Ideograph */
  ["\u61B0", "\u61B0"], /* CJK Ideograph */
  ["\u61B1", "\u61B1"], /* CJK Ideograph */
  ["\u61B2", "\u61B2"], /* CJK Ideograph */
  ["\u61B3", "\u61B3"], /* CJK Ideograph */
  ["\u61B4", "\u61B4"], /* CJK Ideograph */
  ["\u61B5", "\u61B5"], /* CJK Ideograph */
  ["\u61B6", "\u61B6"], /* CJK Ideograph */
  ["\u61B7", "\u61B7"], /* CJK Ideograph */
  ["\u61B8", "\u61B8"], /* CJK Ideograph */
  ["\u61B9", "\u61B9"], /* CJK Ideograph */
  ["\u61BA", "\u61BA"], /* CJK Ideograph */
  ["\u61BB", "\u61BB"], /* CJK Ideograph */
  ["\u61BC", "\u61BC"], /* CJK Ideograph */
  ["\u61BD", "\u61BD"], /* CJK Ideograph */
  ["\u61BE", "\u61BE"], /* CJK Ideograph */
  ["\u61BF", "\u61BF"], /* CJK Ideograph */
  ["\u61C0", "\u61C0"], /* CJK Ideograph */
  ["\u61C1", "\u61C1"], /* CJK Ideograph */
  ["\u61C2", "\u61C2"], /* CJK Ideograph */
  ["\u61C3", "\u61C3"], /* CJK Ideograph */
  ["\u61C4", "\u61C4"], /* CJK Ideograph */
  ["\u61C5", "\u61C5"], /* CJK Ideograph */
  ["\u61C6", "\u61C6"], /* CJK Ideograph */
  ["\u61C7", "\u61C7"], /* CJK Ideograph */
  ["\u61C8", "\u61C8"], /* CJK Ideograph */
  ["\u61C9", "\u61C9"], /* CJK Ideograph */
  ["\u61CA", "\u61CA"], /* CJK Ideograph */
  ["\u61CB", "\u61CB"], /* CJK Ideograph */
  ["\u61CC", "\u61CC"], /* CJK Ideograph */
  ["\u61CD", "\u61CD"], /* CJK Ideograph */
  ["\u61CE", "\u61CE"], /* CJK Ideograph */
  ["\u61CF", "\u61CF"], /* CJK Ideograph */
  ["\u61D0", "\u61D0"], /* CJK Ideograph */
  ["\u61D1", "\u61D1"], /* CJK Ideograph */
  ["\u61D2", "\u61D2"], /* CJK Ideograph */
  ["\u61D3", "\u61D3"], /* CJK Ideograph */
  ["\u61D4", "\u61D4"], /* CJK Ideograph */
  ["\u61D5", "\u61D5"], /* CJK Ideograph */
  ["\u61D6", "\u61D6"], /* CJK Ideograph */
  ["\u61D7", "\u61D7"], /* CJK Ideograph */
  ["\u61D8", "\u61D8"], /* CJK Ideograph */
  ["\u61D9", "\u61D9"], /* CJK Ideograph */
  ["\u61DA", "\u61DA"], /* CJK Ideograph */
  ["\u61DB", "\u61DB"], /* CJK Ideograph */
  ["\u61DC", "\u61DC"], /* CJK Ideograph */
  ["\u61DD", "\u61DD"], /* CJK Ideograph */
  ["\u61DE", "\u61DE"], /* CJK Ideograph */
  ["\u61DF", "\u61DF"], /* CJK Ideograph */
  ["\u61E0", "\u61E0"], /* CJK Ideograph */
  ["\u61E1", "\u61E1"], /* CJK Ideograph */
  ["\u61E2", "\u61E2"], /* CJK Ideograph */
  ["\u61E3", "\u61E3"], /* CJK Ideograph */
  ["\u61E4", "\u61E4"], /* CJK Ideograph */
  ["\u61E5", "\u61E5"], /* CJK Ideograph */
  ["\u61E6", "\u61E6"], /* CJK Ideograph */
  ["\u61E7", "\u61E7"], /* CJK Ideograph */
  ["\u61E8", "\u61E8"], /* CJK Ideograph */
  ["\u61E9", "\u61E9"], /* CJK Ideograph */
  ["\u61EA", "\u61EA"], /* CJK Ideograph */
  ["\u61EB", "\u61EB"], /* CJK Ideograph */
  ["\u61EC", "\u61EC"], /* CJK Ideograph */
  ["\u61ED", "\u61ED"], /* CJK Ideograph */
  ["\u61EE", "\u61EE"], /* CJK Ideograph */
  ["\u61EF", "\u61EF"], /* CJK Ideograph */
  ["\u61F0", "\u61F0"], /* CJK Ideograph */
  ["\u61F1", "\u61F1"], /* CJK Ideograph */
  ["\u61F2", "\u61F2"], /* CJK Ideograph */
  ["\u61F3", "\u61F3"], /* CJK Ideograph */
  ["\u61F4", "\u61F4"], /* CJK Ideograph */
  ["\u61F5", "\u61F5"], /* CJK Ideograph */
  ["\u61F6", "\u61F6"], /* CJK Ideograph */
  ["\u61F7", "\u61F7"], /* CJK Ideograph */
  ["\u61F8", "\u61F8"], /* CJK Ideograph */
  ["\u61F9", "\u61F9"], /* CJK Ideograph */
  ["\u61FA", "\u61FA"], /* CJK Ideograph */
  ["\u61FB", "\u61FB"], /* CJK Ideograph */
  ["\u61FC", "\u61FC"], /* CJK Ideograph */
  ["\u61FD", "\u61FD"], /* CJK Ideograph */
  ["\u61FE", "\u61FE"], /* CJK Ideograph */
  ["\u61FF", "\u61FF"], /* CJK Ideograph */
  ["\u6200", "\u6200"], /* CJK Ideograph */
  ["\u6201", "\u6201"], /* CJK Ideograph */
  ["\u6202", "\u6202"], /* CJK Ideograph */
  ["\u6203", "\u6203"], /* CJK Ideograph */
  ["\u6204", "\u6204"], /* CJK Ideograph */
  ["\u6205", "\u6205"], /* CJK Ideograph */
  ["\u6206", "\u6206"], /* CJK Ideograph */
  ["\u6207", "\u6207"], /* CJK Ideograph */
  ["\u6208", "\u6208"], /* CJK Ideograph */
  ["\u6209", "\u6209"], /* CJK Ideograph */
  ["\u620A", "\u620A"], /* CJK Ideograph */
  ["\u620B", "\u620B"], /* CJK Ideograph */
  ["\u620C", "\u620C"], /* CJK Ideograph */
  ["\u620D", "\u620D"], /* CJK Ideograph */
  ["\u620E", "\u620E"], /* CJK Ideograph */
  ["\u620F", "\u620F"], /* CJK Ideograph */
  ["\u6210", "\u6210"], /* CJK Ideograph */
  ["\u6211", "\u6211"], /* CJK Ideograph */
  ["\u6212", "\u6212"], /* CJK Ideograph */
  ["\u6213", "\u6213"], /* CJK Ideograph */
  ["\u6214", "\u6214"], /* CJK Ideograph */
  ["\u6215", "\u6215"], /* CJK Ideograph */
  ["\u6216", "\u6216"], /* CJK Ideograph */
  ["\u6217", "\u6217"], /* CJK Ideograph */
  ["\u6218", "\u6218"], /* CJK Ideograph */
  ["\u6219", "\u6219"], /* CJK Ideograph */
  ["\u621A", "\u621A"], /* CJK Ideograph */
  ["\u621B", "\u621B"], /* CJK Ideograph */
  ["\u621C", "\u621C"], /* CJK Ideograph */
  ["\u621D", "\u621D"], /* CJK Ideograph */
  ["\u621E", "\u621E"], /* CJK Ideograph */
  ["\u621F", "\u621F"], /* CJK Ideograph */
  ["\u6220", "\u6220"], /* CJK Ideograph */
  ["\u6221", "\u6221"], /* CJK Ideograph */
  ["\u6222", "\u6222"], /* CJK Ideograph */
  ["\u6223", "\u6223"], /* CJK Ideograph */
  ["\u6224", "\u6224"], /* CJK Ideograph */
  ["\u6225", "\u6225"], /* CJK Ideograph */
  ["\u6226", "\u6226"], /* CJK Ideograph */
  ["\u6227", "\u6227"], /* CJK Ideograph */
  ["\u6228", "\u6228"], /* CJK Ideograph */
  ["\u6229", "\u6229"], /* CJK Ideograph */
  ["\u622A", "\u622A"], /* CJK Ideograph */
  ["\u622B", "\u622B"], /* CJK Ideograph */
  ["\u622C", "\u622C"], /* CJK Ideograph */
  ["\u622D", "\u622D"], /* CJK Ideograph */
  ["\u622E", "\u622E"], /* CJK Ideograph */
  ["\u622F", "\u622F"], /* CJK Ideograph */
  ["\u6230", "\u6230"], /* CJK Ideograph */
  ["\u6231", "\u6231"], /* CJK Ideograph */
  ["\u6232", "\u6232"], /* CJK Ideograph */
  ["\u6233", "\u6233"], /* CJK Ideograph */
  ["\u6234", "\u6234"], /* CJK Ideograph */
  ["\u6235", "\u6235"], /* CJK Ideograph */
  ["\u6236", "\u6236"], /* CJK Ideograph */
  ["\u6237", "\u6237"], /* CJK Ideograph */
  ["\u6238", "\u6238"], /* CJK Ideograph */
  ["\u6239", "\u6239"], /* CJK Ideograph */
  ["\u623A", "\u623A"], /* CJK Ideograph */
  ["\u623B", "\u623B"], /* CJK Ideograph */
  ["\u623C", "\u623C"], /* CJK Ideograph */
  ["\u623D", "\u623D"], /* CJK Ideograph */
  ["\u623E", "\u623E"], /* CJK Ideograph */
  ["\u623F", "\u623F"], /* CJK Ideograph */
  ["\u6240", "\u6240"], /* CJK Ideograph */
  ["\u6241", "\u6241"], /* CJK Ideograph */
  ["\u6242", "\u6242"], /* CJK Ideograph */
  ["\u6243", "\u6243"], /* CJK Ideograph */
  ["\u6244", "\u6244"], /* CJK Ideograph */
  ["\u6245", "\u6245"], /* CJK Ideograph */
  ["\u6246", "\u6246"], /* CJK Ideograph */
  ["\u6247", "\u6247"], /* CJK Ideograph */
  ["\u6248", "\u6248"], /* CJK Ideograph */
  ["\u6249", "\u6249"], /* CJK Ideograph */
  ["\u624A", "\u624A"], /* CJK Ideograph */
  ["\u624B", "\u624B"], /* CJK Ideograph */
  ["\u624C", "\u624C"], /* CJK Ideograph */
  ["\u624D", "\u624D"], /* CJK Ideograph */
  ["\u624E", "\u624E"], /* CJK Ideograph */
  ["\u624F", "\u624F"], /* CJK Ideograph */
  ["\u6250", "\u6250"], /* CJK Ideograph */
  ["\u6251", "\u6251"], /* CJK Ideograph */
  ["\u6252", "\u6252"], /* CJK Ideograph */
  ["\u6253", "\u6253"], /* CJK Ideograph */
  ["\u6254", "\u6254"], /* CJK Ideograph */
  ["\u6255", "\u6255"], /* CJK Ideograph */
  ["\u6256", "\u6256"], /* CJK Ideograph */
  ["\u6257", "\u6257"], /* CJK Ideograph */
  ["\u6258", "\u6258"], /* CJK Ideograph */
  ["\u6259", "\u6259"], /* CJK Ideograph */
  ["\u625A", "\u625A"], /* CJK Ideograph */
  ["\u625B", "\u625B"], /* CJK Ideograph */
  ["\u625C", "\u625C"], /* CJK Ideograph */
  ["\u625D", "\u625D"], /* CJK Ideograph */
  ["\u625E", "\u625E"], /* CJK Ideograph */
  ["\u625F", "\u625F"], /* CJK Ideograph */
  ["\u6260", "\u6260"], /* CJK Ideograph */
  ["\u6261", "\u6261"], /* CJK Ideograph */
  ["\u6262", "\u6262"], /* CJK Ideograph */
  ["\u6263", "\u6263"], /* CJK Ideograph */
  ["\u6264", "\u6264"], /* CJK Ideograph */
  ["\u6265", "\u6265"], /* CJK Ideograph */
  ["\u6266", "\u6266"], /* CJK Ideograph */
  ["\u6267", "\u6267"], /* CJK Ideograph */
  ["\u6268", "\u6268"], /* CJK Ideograph */
  ["\u6269", "\u6269"], /* CJK Ideograph */
  ["\u626A", "\u626A"], /* CJK Ideograph */
  ["\u626B", "\u626B"], /* CJK Ideograph */
  ["\u626C", "\u626C"], /* CJK Ideograph */
  ["\u626D", "\u626D"], /* CJK Ideograph */
  ["\u626E", "\u626E"], /* CJK Ideograph */
  ["\u626F", "\u626F"], /* CJK Ideograph */
  ["\u6270", "\u6270"], /* CJK Ideograph */
  ["\u6271", "\u6271"], /* CJK Ideograph */
  ["\u6272", "\u6272"], /* CJK Ideograph */
  ["\u6273", "\u6273"], /* CJK Ideograph */
  ["\u6274", "\u6274"], /* CJK Ideograph */
  ["\u6275", "\u6275"], /* CJK Ideograph */
  ["\u6276", "\u6276"], /* CJK Ideograph */
  ["\u6277", "\u6277"], /* CJK Ideograph */
  ["\u6278", "\u6278"], /* CJK Ideograph */
  ["\u6279", "\u6279"], /* CJK Ideograph */
  ["\u627A", "\u627A"], /* CJK Ideograph */
  ["\u627B", "\u627B"], /* CJK Ideograph */
  ["\u627C", "\u627C"], /* CJK Ideograph */
  ["\u627D", "\u627D"], /* CJK Ideograph */
  ["\u627E", "\u627E"], /* CJK Ideograph */
  ["\u627F", "\u627F"], /* CJK Ideograph */
  ["\u6280", "\u6280"], /* CJK Ideograph */
  ["\u6281", "\u6281"], /* CJK Ideograph */
  ["\u6282", "\u6282"], /* CJK Ideograph */
  ["\u6283", "\u6283"], /* CJK Ideograph */
  ["\u6284", "\u6284"], /* CJK Ideograph */
  ["\u6285", "\u6285"], /* CJK Ideograph */
  ["\u6286", "\u6286"], /* CJK Ideograph */
  ["\u6287", "\u6287"], /* CJK Ideograph */
  ["\u6288", "\u6288"], /* CJK Ideograph */
  ["\u6289", "\u6289"], /* CJK Ideograph */
  ["\u628A", "\u628A"], /* CJK Ideograph */
  ["\u628B", "\u628B"], /* CJK Ideograph */
  ["\u628C", "\u628C"], /* CJK Ideograph */
  ["\u628D", "\u628D"], /* CJK Ideograph */
  ["\u628E", "\u628E"], /* CJK Ideograph */
  ["\u628F", "\u628F"], /* CJK Ideograph */
  ["\u6290", "\u6290"], /* CJK Ideograph */
  ["\u6291", "\u6291"], /* CJK Ideograph */
  ["\u6292", "\u6292"], /* CJK Ideograph */
  ["\u6293", "\u6293"], /* CJK Ideograph */
  ["\u6294", "\u6294"], /* CJK Ideograph */
  ["\u6295", "\u6295"], /* CJK Ideograph */
  ["\u6296", "\u6296"], /* CJK Ideograph */
  ["\u6297", "\u6297"], /* CJK Ideograph */
  ["\u6298", "\u6298"], /* CJK Ideograph */
  ["\u6299", "\u6299"], /* CJK Ideograph */
  ["\u629A", "\u629A"], /* CJK Ideograph */
  ["\u629B", "\u629B"], /* CJK Ideograph */
  ["\u629C", "\u629C"], /* CJK Ideograph */
  ["\u629D", "\u629D"], /* CJK Ideograph */
  ["\u629E", "\u629E"], /* CJK Ideograph */
  ["\u629F", "\u629F"], /* CJK Ideograph */
  ["\u62A0", "\u62A0"], /* CJK Ideograph */
  ["\u62A1", "\u62A1"], /* CJK Ideograph */
  ["\u62A2", "\u62A2"], /* CJK Ideograph */
  ["\u62A3", "\u62A3"], /* CJK Ideograph */
  ["\u62A4", "\u62A4"], /* CJK Ideograph */
  ["\u62A5", "\u62A5"], /* CJK Ideograph */
  ["\u62A6", "\u62A6"], /* CJK Ideograph */
  ["\u62A7", "\u62A7"], /* CJK Ideograph */
  ["\u62A8", "\u62A8"], /* CJK Ideograph */
  ["\u62A9", "\u62A9"], /* CJK Ideograph */
  ["\u62AA", "\u62AA"], /* CJK Ideograph */
  ["\u62AB", "\u62AB"], /* CJK Ideograph */
  ["\u62AC", "\u62AC"], /* CJK Ideograph */
  ["\u62AD", "\u62AD"], /* CJK Ideograph */
  ["\u62AE", "\u62AE"], /* CJK Ideograph */
  ["\u62AF", "\u62AF"], /* CJK Ideograph */
  ["\u62B0", "\u62B0"], /* CJK Ideograph */
  ["\u62B1", "\u62B1"], /* CJK Ideograph */
  ["\u62B2", "\u62B2"], /* CJK Ideograph */
  ["\u62B3", "\u62B3"], /* CJK Ideograph */
  ["\u62B4", "\u62B4"], /* CJK Ideograph */
  ["\u62B5", "\u62B5"], /* CJK Ideograph */
  ["\u62B6", "\u62B6"], /* CJK Ideograph */
  ["\u62B7", "\u62B7"], /* CJK Ideograph */
  ["\u62B8", "\u62B8"], /* CJK Ideograph */
  ["\u62B9", "\u62B9"], /* CJK Ideograph */
  ["\u62BA", "\u62BA"], /* CJK Ideograph */
  ["\u62BB", "\u62BB"], /* CJK Ideograph */
  ["\u62BC", "\u62BC"], /* CJK Ideograph */
  ["\u62BD", "\u62BD"], /* CJK Ideograph */
  ["\u62BE", "\u62BE"], /* CJK Ideograph */
  ["\u62BF", "\u62BF"], /* CJK Ideograph */
  ["\u62C0", "\u62C0"], /* CJK Ideograph */
  ["\u62C1", "\u62C1"], /* CJK Ideograph */
  ["\u62C2", "\u62C2"], /* CJK Ideograph */
  ["\u62C3", "\u62C3"], /* CJK Ideograph */
  ["\u62C4", "\u62C4"], /* CJK Ideograph */
  ["\u62C5", "\u62C5"], /* CJK Ideograph */
  ["\u62C6", "\u62C6"], /* CJK Ideograph */
  ["\u62C7", "\u62C7"], /* CJK Ideograph */
  ["\u62C8", "\u62C8"], /* CJK Ideograph */
  ["\u62C9", "\u62C9"], /* CJK Ideograph */
  ["\u62CA", "\u62CA"], /* CJK Ideograph */
  ["\u62CB", "\u62CB"], /* CJK Ideograph */
  ["\u62CC", "\u62CC"], /* CJK Ideograph */
  ["\u62CD", "\u62CD"], /* CJK Ideograph */
  ["\u62CE", "\u62CE"], /* CJK Ideograph */
  ["\u62CF", "\u62CF"], /* CJK Ideograph */
  ["\u62D0", "\u62D0"], /* CJK Ideograph */
  ["\u62D1", "\u62D1"], /* CJK Ideograph */
  ["\u62D2", "\u62D2"], /* CJK Ideograph */
  ["\u62D3", "\u62D3"], /* CJK Ideograph */
  ["\u62D4", "\u62D4"], /* CJK Ideograph */
  ["\u62D5", "\u62D5"], /* CJK Ideograph */
  ["\u62D6", "\u62D6"], /* CJK Ideograph */
  ["\u62D7", "\u62D7"], /* CJK Ideograph */
  ["\u62D8", "\u62D8"], /* CJK Ideograph */
  ["\u62D9", "\u62D9"], /* CJK Ideograph */
  ["\u62DA", "\u62DA"], /* CJK Ideograph */
  ["\u62DB", "\u62DB"], /* CJK Ideograph */
  ["\u62DC", "\u62DC"], /* CJK Ideograph */
  ["\u62DD", "\u62DD"], /* CJK Ideograph */
  ["\u62DE", "\u62DE"], /* CJK Ideograph */
  ["\u62DF", "\u62DF"], /* CJK Ideograph */
  ["\u62E0", "\u62E0"], /* CJK Ideograph */
  ["\u62E1", "\u62E1"], /* CJK Ideograph */
  ["\u62E2", "\u62E2"], /* CJK Ideograph */
  ["\u62E3", "\u62E3"], /* CJK Ideograph */
  ["\u62E4", "\u62E4"], /* CJK Ideograph */
  ["\u62E5", "\u62E5"], /* CJK Ideograph */
  ["\u62E6", "\u62E6"], /* CJK Ideograph */
  ["\u62E7", "\u62E7"], /* CJK Ideograph */
  ["\u62E8", "\u62E8"], /* CJK Ideograph */
  ["\u62E9", "\u62E9"], /* CJK Ideograph */
  ["\u62EA", "\u62EA"], /* CJK Ideograph */
  ["\u62EB", "\u62EB"], /* CJK Ideograph */
  ["\u62EC", "\u62EC"], /* CJK Ideograph */
  ["\u62ED", "\u62ED"], /* CJK Ideograph */
  ["\u62EE", "\u62EE"], /* CJK Ideograph */
  ["\u62EF", "\u62EF"], /* CJK Ideograph */
  ["\u62F0", "\u62F0"], /* CJK Ideograph */
  ["\u62F1", "\u62F1"], /* CJK Ideograph */
  ["\u62F2", "\u62F2"], /* CJK Ideograph */
  ["\u62F3", "\u62F3"], /* CJK Ideograph */
  ["\u62F4", "\u62F4"], /* CJK Ideograph */
  ["\u62F5", "\u62F5"], /* CJK Ideograph */
  ["\u62F6", "\u62F6"], /* CJK Ideograph */
  ["\u62F7", "\u62F7"], /* CJK Ideograph */
  ["\u62F8", "\u62F8"], /* CJK Ideograph */
  ["\u62F9", "\u62F9"], /* CJK Ideograph */
  ["\u62FA", "\u62FA"], /* CJK Ideograph */
  ["\u62FB", "\u62FB"], /* CJK Ideograph */
  ["\u62FC", "\u62FC"], /* CJK Ideograph */
  ["\u62FD", "\u62FD"], /* CJK Ideograph */
  ["\u62FE", "\u62FE"], /* CJK Ideograph */
  ["\u62FF", "\u62FF"], /* CJK Ideograph */
  ["\u6300", "\u6300"], /* CJK Ideograph */
  ["\u6301", "\u6301"], /* CJK Ideograph */
  ["\u6302", "\u6302"], /* CJK Ideograph */
  ["\u6303", "\u6303"], /* CJK Ideograph */
  ["\u6304", "\u6304"], /* CJK Ideograph */
  ["\u6305", "\u6305"], /* CJK Ideograph */
  ["\u6306", "\u6306"], /* CJK Ideograph */
  ["\u6307", "\u6307"], /* CJK Ideograph */
  ["\u6308", "\u6308"], /* CJK Ideograph */
  ["\u6309", "\u6309"], /* CJK Ideograph */
  ["\u630A", "\u630A"], /* CJK Ideograph */
  ["\u630B", "\u630B"], /* CJK Ideograph */
  ["\u630C", "\u630C"], /* CJK Ideograph */
  ["\u630D", "\u630D"], /* CJK Ideograph */
  ["\u630E", "\u630E"], /* CJK Ideograph */
  ["\u630F", "\u630F"], /* CJK Ideograph */
  ["\u6310", "\u6310"], /* CJK Ideograph */
  ["\u6311", "\u6311"], /* CJK Ideograph */
  ["\u6312", "\u6312"], /* CJK Ideograph */
  ["\u6313", "\u6313"], /* CJK Ideograph */
  ["\u6314", "\u6314"], /* CJK Ideograph */
  ["\u6315", "\u6315"], /* CJK Ideograph */
  ["\u6316", "\u6316"], /* CJK Ideograph */
  ["\u6317", "\u6317"], /* CJK Ideograph */
  ["\u6318", "\u6318"], /* CJK Ideograph */
  ["\u6319", "\u6319"], /* CJK Ideograph */
  ["\u631A", "\u631A"], /* CJK Ideograph */
  ["\u631B", "\u631B"], /* CJK Ideograph */
  ["\u631C", "\u631C"], /* CJK Ideograph */
  ["\u631D", "\u631D"], /* CJK Ideograph */
  ["\u631E", "\u631E"], /* CJK Ideograph */
  ["\u631F", "\u631F"], /* CJK Ideograph */
  ["\u6320", "\u6320"], /* CJK Ideograph */
  ["\u6321", "\u6321"], /* CJK Ideograph */
  ["\u6322", "\u6322"], /* CJK Ideograph */
  ["\u6323", "\u6323"], /* CJK Ideograph */
  ["\u6324", "\u6324"], /* CJK Ideograph */
  ["\u6325", "\u6325"], /* CJK Ideograph */
  ["\u6326", "\u6326"], /* CJK Ideograph */
  ["\u6327", "\u6327"], /* CJK Ideograph */
  ["\u6328", "\u6328"], /* CJK Ideograph */
  ["\u6329", "\u6329"], /* CJK Ideograph */
  ["\u632A", "\u632A"], /* CJK Ideograph */
  ["\u632B", "\u632B"], /* CJK Ideograph */
  ["\u632C", "\u632C"], /* CJK Ideograph */
  ["\u632D", "\u632D"], /* CJK Ideograph */
  ["\u632E", "\u632E"], /* CJK Ideograph */
  ["\u632F", "\u632F"], /* CJK Ideograph */
  ["\u6330", "\u6330"], /* CJK Ideograph */
  ["\u6331", "\u6331"], /* CJK Ideograph */
  ["\u6332", "\u6332"], /* CJK Ideograph */
  ["\u6333", "\u6333"], /* CJK Ideograph */
  ["\u6334", "\u6334"], /* CJK Ideograph */
  ["\u6335", "\u6335"], /* CJK Ideograph */
  ["\u6336", "\u6336"], /* CJK Ideograph */
  ["\u6337", "\u6337"], /* CJK Ideograph */
  ["\u6338", "\u6338"], /* CJK Ideograph */
  ["\u6339", "\u6339"], /* CJK Ideograph */
  ["\u633A", "\u633A"], /* CJK Ideograph */
  ["\u633B", "\u633B"], /* CJK Ideograph */
  ["\u633C", "\u633C"], /* CJK Ideograph */
  ["\u633D", "\u633D"], /* CJK Ideograph */
  ["\u633E", "\u633E"], /* CJK Ideograph */
  ["\u633F", "\u633F"], /* CJK Ideograph */
  ["\u6340", "\u6340"], /* CJK Ideograph */
  ["\u6341", "\u6341"], /* CJK Ideograph */
  ["\u6342", "\u6342"], /* CJK Ideograph */
  ["\u6343", "\u6343"], /* CJK Ideograph */
  ["\u6344", "\u6344"], /* CJK Ideograph */
  ["\u6345", "\u6345"], /* CJK Ideograph */
  ["\u6346", "\u6346"], /* CJK Ideograph */
  ["\u6347", "\u6347"], /* CJK Ideograph */
  ["\u6348", "\u6348"], /* CJK Ideograph */
  ["\u6349", "\u6349"], /* CJK Ideograph */
  ["\u634A", "\u634A"], /* CJK Ideograph */
  ["\u634B", "\u634B"], /* CJK Ideograph */
  ["\u634C", "\u634C"], /* CJK Ideograph */
  ["\u634D", "\u634D"], /* CJK Ideograph */
  ["\u634E", "\u634E"], /* CJK Ideograph */
  ["\u634F", "\u634F"], /* CJK Ideograph */
  ["\u6350", "\u6350"], /* CJK Ideograph */
  ["\u6351", "\u6351"], /* CJK Ideograph */
  ["\u6352", "\u6352"], /* CJK Ideograph */
  ["\u6353", "\u6353"], /* CJK Ideograph */
  ["\u6354", "\u6354"], /* CJK Ideograph */
  ["\u6355", "\u6355"], /* CJK Ideograph */
  ["\u6356", "\u6356"], /* CJK Ideograph */
  ["\u6357", "\u6357"], /* CJK Ideograph */
  ["\u6358", "\u6358"], /* CJK Ideograph */
  ["\u6359", "\u6359"], /* CJK Ideograph */
  ["\u635A", "\u635A"], /* CJK Ideograph */
  ["\u635B", "\u635B"], /* CJK Ideograph */
  ["\u635C", "\u635C"], /* CJK Ideograph */
  ["\u635D", "\u635D"], /* CJK Ideograph */
  ["\u635E", "\u635E"], /* CJK Ideograph */
  ["\u635F", "\u635F"], /* CJK Ideograph */
  ["\u6360", "\u6360"], /* CJK Ideograph */
  ["\u6361", "\u6361"], /* CJK Ideograph */
  ["\u6362", "\u6362"], /* CJK Ideograph */
  ["\u6363", "\u6363"], /* CJK Ideograph */
  ["\u6364", "\u6364"], /* CJK Ideograph */
  ["\u6365", "\u6365"], /* CJK Ideograph */
  ["\u6366", "\u6366"], /* CJK Ideograph */
  ["\u6367", "\u6367"], /* CJK Ideograph */
  ["\u6368", "\u6368"], /* CJK Ideograph */
  ["\u6369", "\u6369"], /* CJK Ideograph */
  ["\u636A", "\u636A"], /* CJK Ideograph */
  ["\u636B", "\u636B"], /* CJK Ideograph */
  ["\u636C", "\u636C"], /* CJK Ideograph */
  ["\u636D", "\u636D"], /* CJK Ideograph */
  ["\u636E", "\u636E"], /* CJK Ideograph */
  ["\u636F", "\u636F"], /* CJK Ideograph */
  ["\u6370", "\u6370"], /* CJK Ideograph */
  ["\u6371", "\u6371"], /* CJK Ideograph */
  ["\u6372", "\u6372"], /* CJK Ideograph */
  ["\u6373", "\u6373"], /* CJK Ideograph */
  ["\u6374", "\u6374"], /* CJK Ideograph */
  ["\u6375", "\u6375"], /* CJK Ideograph */
  ["\u6376", "\u6376"], /* CJK Ideograph */
  ["\u6377", "\u6377"], /* CJK Ideograph */
  ["\u6378", "\u6378"], /* CJK Ideograph */
  ["\u6379", "\u6379"], /* CJK Ideograph */
  ["\u637A", "\u637A"], /* CJK Ideograph */
  ["\u637B", "\u637B"], /* CJK Ideograph */
  ["\u637C", "\u637C"], /* CJK Ideograph */
  ["\u637D", "\u637D"], /* CJK Ideograph */
  ["\u637E", "\u637E"], /* CJK Ideograph */
  ["\u637F", "\u637F"], /* CJK Ideograph */
  ["\u6380", "\u6380"], /* CJK Ideograph */
  ["\u6381", "\u6381"], /* CJK Ideograph */
  ["\u6382", "\u6382"], /* CJK Ideograph */
  ["\u6383", "\u6383"], /* CJK Ideograph */
  ["\u6384", "\u6384"], /* CJK Ideograph */
  ["\u6385", "\u6385"], /* CJK Ideograph */
  ["\u6386", "\u6386"], /* CJK Ideograph */
  ["\u6387", "\u6387"], /* CJK Ideograph */
  ["\u6388", "\u6388"], /* CJK Ideograph */
  ["\u6389", "\u6389"], /* CJK Ideograph */
  ["\u638A", "\u638A"], /* CJK Ideograph */
  ["\u638B", "\u638B"], /* CJK Ideograph */
  ["\u638C", "\u638C"], /* CJK Ideograph */
  ["\u638D", "\u638D"], /* CJK Ideograph */
  ["\u638E", "\u638E"], /* CJK Ideograph */
  ["\u638F", "\u638F"], /* CJK Ideograph */
  ["\u6390", "\u6390"], /* CJK Ideograph */
  ["\u6391", "\u6391"], /* CJK Ideograph */
  ["\u6392", "\u6392"], /* CJK Ideograph */
  ["\u6393", "\u6393"], /* CJK Ideograph */
  ["\u6394", "\u6394"], /* CJK Ideograph */
  ["\u6395", "\u6395"], /* CJK Ideograph */
  ["\u6396", "\u6396"], /* CJK Ideograph */
  ["\u6397", "\u6397"], /* CJK Ideograph */
  ["\u6398", "\u6398"], /* CJK Ideograph */
  ["\u6399", "\u6399"], /* CJK Ideograph */
  ["\u639A", "\u639A"], /* CJK Ideograph */
  ["\u639B", "\u639B"], /* CJK Ideograph */
  ["\u639C", "\u639C"], /* CJK Ideograph */
  ["\u639D", "\u639D"], /* CJK Ideograph */
  ["\u639E", "\u639E"], /* CJK Ideograph */
  ["\u639F", "\u639F"], /* CJK Ideograph */
  ["\u63A0", "\u63A0"], /* CJK Ideograph */
  ["\u63A1", "\u63A1"], /* CJK Ideograph */
  ["\u63A2", "\u63A2"], /* CJK Ideograph */
  ["\u63A3", "\u63A3"], /* CJK Ideograph */
  ["\u63A4", "\u63A4"], /* CJK Ideograph */
  ["\u63A5", "\u63A5"], /* CJK Ideograph */
  ["\u63A6", "\u63A6"], /* CJK Ideograph */
  ["\u63A7", "\u63A7"], /* CJK Ideograph */
  ["\u63A8", "\u63A8"], /* CJK Ideograph */
  ["\u63A9", "\u63A9"], /* CJK Ideograph */
  ["\u63AA", "\u63AA"], /* CJK Ideograph */
  ["\u63AB", "\u63AB"], /* CJK Ideograph */
  ["\u63AC", "\u63AC"], /* CJK Ideograph */
  ["\u63AD", "\u63AD"], /* CJK Ideograph */
  ["\u63AE", "\u63AE"], /* CJK Ideograph */
  ["\u63AF", "\u63AF"], /* CJK Ideograph */
  ["\u63B0", "\u63B0"], /* CJK Ideograph */
  ["\u63B1", "\u63B1"], /* CJK Ideograph */
  ["\u63B2", "\u63B2"], /* CJK Ideograph */
  ["\u63B3", "\u63B3"], /* CJK Ideograph */
  ["\u63B4", "\u63B4"], /* CJK Ideograph */
  ["\u63B5", "\u63B5"], /* CJK Ideograph */
  ["\u63B6", "\u63B6"], /* CJK Ideograph */
  ["\u63B7", "\u63B7"], /* CJK Ideograph */
  ["\u63B8", "\u63B8"], /* CJK Ideograph */
  ["\u63B9", "\u63B9"], /* CJK Ideograph */
  ["\u63BA", "\u63BA"], /* CJK Ideograph */
  ["\u63BB", "\u63BB"], /* CJK Ideograph */
  ["\u63BC", "\u63BC"], /* CJK Ideograph */
  ["\u63BD", "\u63BD"], /* CJK Ideograph */
  ["\u63BE", "\u63BE"], /* CJK Ideograph */
  ["\u63BF", "\u63BF"], /* CJK Ideograph */
  ["\u63C0", "\u63C0"], /* CJK Ideograph */
  ["\u63C1", "\u63C1"], /* CJK Ideograph */
  ["\u63C2", "\u63C2"], /* CJK Ideograph */
  ["\u63C3", "\u63C3"], /* CJK Ideograph */
  ["\u63C4", "\u63C4"], /* CJK Ideograph */
  ["\u63C5", "\u63C5"], /* CJK Ideograph */
  ["\u63C6", "\u63C6"], /* CJK Ideograph */
  ["\u63C7", "\u63C7"], /* CJK Ideograph */
  ["\u63C8", "\u63C8"], /* CJK Ideograph */
  ["\u63C9", "\u63C9"], /* CJK Ideograph */
  ["\u63CA", "\u63CA"], /* CJK Ideograph */
  ["\u63CB", "\u63CB"], /* CJK Ideograph */
  ["\u63CC", "\u63CC"], /* CJK Ideograph */
  ["\u63CD", "\u63CD"], /* CJK Ideograph */
  ["\u63CE", "\u63CE"], /* CJK Ideograph */
  ["\u63CF", "\u63CF"], /* CJK Ideograph */
  ["\u63D0", "\u63D0"], /* CJK Ideograph */
  ["\u63D1", "\u63D1"], /* CJK Ideograph */
  ["\u63D2", "\u63D2"], /* CJK Ideograph */
  ["\u63D3", "\u63D3"], /* CJK Ideograph */
  ["\u63D4", "\u63D4"], /* CJK Ideograph */
  ["\u63D5", "\u63D5"], /* CJK Ideograph */
  ["\u63D6", "\u63D6"], /* CJK Ideograph */
  ["\u63D7", "\u63D7"], /* CJK Ideograph */
  ["\u63D8", "\u63D8"], /* CJK Ideograph */
  ["\u63D9", "\u63D9"], /* CJK Ideograph */
  ["\u63DA", "\u63DA"], /* CJK Ideograph */
  ["\u63DB", "\u63DB"], /* CJK Ideograph */
  ["\u63DC", "\u63DC"], /* CJK Ideograph */
  ["\u63DD", "\u63DD"], /* CJK Ideograph */
  ["\u63DE", "\u63DE"], /* CJK Ideograph */
  ["\u63DF", "\u63DF"], /* CJK Ideograph */
  ["\u63E0", "\u63E0"], /* CJK Ideograph */
  ["\u63E1", "\u63E1"], /* CJK Ideograph */
  ["\u63E2", "\u63E2"], /* CJK Ideograph */
  ["\u63E3", "\u63E3"], /* CJK Ideograph */
  ["\u63E4", "\u63E4"], /* CJK Ideograph */
  ["\u63E5", "\u63E5"], /* CJK Ideograph */
  ["\u63E6", "\u63E6"], /* CJK Ideograph */
  ["\u63E7", "\u63E7"], /* CJK Ideograph */
  ["\u63E8", "\u63E8"], /* CJK Ideograph */
  ["\u63E9", "\u63E9"], /* CJK Ideograph */
  ["\u63EA", "\u63EA"], /* CJK Ideograph */
  ["\u63EB", "\u63EB"], /* CJK Ideograph */
  ["\u63EC", "\u63EC"], /* CJK Ideograph */
  ["\u63ED", "\u63ED"], /* CJK Ideograph */
  ["\u63EE", "\u63EE"], /* CJK Ideograph */
  ["\u63EF", "\u63EF"], /* CJK Ideograph */
  ["\u63F0", "\u63F0"], /* CJK Ideograph */
  ["\u63F1", "\u63F1"], /* CJK Ideograph */
  ["\u63F2", "\u63F2"], /* CJK Ideograph */
  ["\u63F3", "\u63F3"], /* CJK Ideograph */
  ["\u63F4", "\u63F4"], /* CJK Ideograph */
  ["\u63F5", "\u63F5"], /* CJK Ideograph */
  ["\u63F6", "\u63F6"], /* CJK Ideograph */
  ["\u63F7", "\u63F7"], /* CJK Ideograph */
  ["\u63F8", "\u63F8"], /* CJK Ideograph */
  ["\u63F9", "\u63F9"], /* CJK Ideograph */
  ["\u63FA", "\u63FA"], /* CJK Ideograph */
  ["\u63FB", "\u63FB"], /* CJK Ideograph */
  ["\u63FC", "\u63FC"], /* CJK Ideograph */
  ["\u63FD", "\u63FD"], /* CJK Ideograph */
  ["\u63FE", "\u63FE"], /* CJK Ideograph */
  ["\u63FF", "\u63FF"], /* CJK Ideograph */
  ["\u6400", "\u6400"], /* CJK Ideograph */
  ["\u6401", "\u6401"], /* CJK Ideograph */
  ["\u6402", "\u6402"], /* CJK Ideograph */
  ["\u6403", "\u6403"], /* CJK Ideograph */
  ["\u6404", "\u6404"], /* CJK Ideograph */
  ["\u6405", "\u6405"], /* CJK Ideograph */
  ["\u6406", "\u6406"], /* CJK Ideograph */
  ["\u6407", "\u6407"], /* CJK Ideograph */
  ["\u6408", "\u6408"], /* CJK Ideograph */
  ["\u6409", "\u6409"], /* CJK Ideograph */
  ["\u640A", "\u640A"], /* CJK Ideograph */
  ["\u640B", "\u640B"], /* CJK Ideograph */
  ["\u640C", "\u640C"], /* CJK Ideograph */
  ["\u640D", "\u640D"], /* CJK Ideograph */
  ["\u640E", "\u640E"], /* CJK Ideograph */
  ["\u640F", "\u640F"], /* CJK Ideograph */
  ["\u6410", "\u6410"], /* CJK Ideograph */
  ["\u6411", "\u6411"], /* CJK Ideograph */
  ["\u6412", "\u6412"], /* CJK Ideograph */
  ["\u6413", "\u6413"], /* CJK Ideograph */
  ["\u6414", "\u6414"], /* CJK Ideograph */
  ["\u6415", "\u6415"], /* CJK Ideograph */
  ["\u6416", "\u6416"], /* CJK Ideograph */
  ["\u6417", "\u6417"], /* CJK Ideograph */
  ["\u6418", "\u6418"], /* CJK Ideograph */
  ["\u6419", "\u6419"], /* CJK Ideograph */
  ["\u641A", "\u641A"], /* CJK Ideograph */
  ["\u641B", "\u641B"], /* CJK Ideograph */
  ["\u641C", "\u641C"], /* CJK Ideograph */
  ["\u641D", "\u641D"], /* CJK Ideograph */
  ["\u641E", "\u641E"], /* CJK Ideograph */
  ["\u641F", "\u641F"], /* CJK Ideograph */
  ["\u6420", "\u6420"], /* CJK Ideograph */
  ["\u6421", "\u6421"], /* CJK Ideograph */
  ["\u6422", "\u6422"], /* CJK Ideograph */
  ["\u6423", "\u6423"], /* CJK Ideograph */
  ["\u6424", "\u6424"], /* CJK Ideograph */
  ["\u6425", "\u6425"], /* CJK Ideograph */
  ["\u6426", "\u6426"], /* CJK Ideograph */
  ["\u6427", "\u6427"], /* CJK Ideograph */
  ["\u6428", "\u6428"], /* CJK Ideograph */
  ["\u6429", "\u6429"], /* CJK Ideograph */
  ["\u642A", "\u642A"], /* CJK Ideograph */
  ["\u642B", "\u642B"], /* CJK Ideograph */
  ["\u642C", "\u642C"], /* CJK Ideograph */
  ["\u642D", "\u642D"], /* CJK Ideograph */
  ["\u642E", "\u642E"], /* CJK Ideograph */
  ["\u642F", "\u642F"], /* CJK Ideograph */
  ["\u6430", "\u6430"], /* CJK Ideograph */
  ["\u6431", "\u6431"], /* CJK Ideograph */
  ["\u6432", "\u6432"], /* CJK Ideograph */
  ["\u6433", "\u6433"], /* CJK Ideograph */
  ["\u6434", "\u6434"], /* CJK Ideograph */
  ["\u6435", "\u6435"], /* CJK Ideograph */
  ["\u6436", "\u6436"], /* CJK Ideograph */
  ["\u6437", "\u6437"], /* CJK Ideograph */
  ["\u6438", "\u6438"], /* CJK Ideograph */
  ["\u6439", "\u6439"], /* CJK Ideograph */
  ["\u643A", "\u643A"], /* CJK Ideograph */
  ["\u643B", "\u643B"], /* CJK Ideograph */
  ["\u643C", "\u643C"], /* CJK Ideograph */
  ["\u643D", "\u643D"], /* CJK Ideograph */
  ["\u643E", "\u643E"], /* CJK Ideograph */
  ["\u643F", "\u643F"], /* CJK Ideograph */
  ["\u6440", "\u6440"], /* CJK Ideograph */
  ["\u6441", "\u6441"], /* CJK Ideograph */
  ["\u6442", "\u6442"], /* CJK Ideograph */
  ["\u6443", "\u6443"], /* CJK Ideograph */
  ["\u6444", "\u6444"], /* CJK Ideograph */
  ["\u6445", "\u6445"], /* CJK Ideograph */
  ["\u6446", "\u6446"], /* CJK Ideograph */
  ["\u6447", "\u6447"], /* CJK Ideograph */
  ["\u6448", "\u6448"], /* CJK Ideograph */
  ["\u6449", "\u6449"], /* CJK Ideograph */
  ["\u644A", "\u644A"], /* CJK Ideograph */
  ["\u644B", "\u644B"], /* CJK Ideograph */
  ["\u644C", "\u644C"], /* CJK Ideograph */
  ["\u644D", "\u644D"], /* CJK Ideograph */
  ["\u644E", "\u644E"], /* CJK Ideograph */
  ["\u644F", "\u644F"], /* CJK Ideograph */
  ["\u6450", "\u6450"], /* CJK Ideograph */
  ["\u6451", "\u6451"], /* CJK Ideograph */
  ["\u6452", "\u6452"], /* CJK Ideograph */
  ["\u6453", "\u6453"], /* CJK Ideograph */
  ["\u6454", "\u6454"], /* CJK Ideograph */
  ["\u6455", "\u6455"], /* CJK Ideograph */
  ["\u6456", "\u6456"], /* CJK Ideograph */
  ["\u6457", "\u6457"], /* CJK Ideograph */
  ["\u6458", "\u6458"], /* CJK Ideograph */
  ["\u6459", "\u6459"], /* CJK Ideograph */
  ["\u645A", "\u645A"], /* CJK Ideograph */
  ["\u645B", "\u645B"], /* CJK Ideograph */
  ["\u645C", "\u645C"], /* CJK Ideograph */
  ["\u645D", "\u645D"], /* CJK Ideograph */
  ["\u645E", "\u645E"], /* CJK Ideograph */
  ["\u645F", "\u645F"], /* CJK Ideograph */
  ["\u6460", "\u6460"], /* CJK Ideograph */
  ["\u6461", "\u6461"], /* CJK Ideograph */
  ["\u6462", "\u6462"], /* CJK Ideograph */
  ["\u6463", "\u6463"], /* CJK Ideograph */
  ["\u6464", "\u6464"], /* CJK Ideograph */
  ["\u6465", "\u6465"], /* CJK Ideograph */
  ["\u6466", "\u6466"], /* CJK Ideograph */
  ["\u6467", "\u6467"], /* CJK Ideograph */
  ["\u6468", "\u6468"], /* CJK Ideograph */
  ["\u6469", "\u6469"], /* CJK Ideograph */
  ["\u646A", "\u646A"], /* CJK Ideograph */
  ["\u646B", "\u646B"], /* CJK Ideograph */
  ["\u646C", "\u646C"], /* CJK Ideograph */
  ["\u646D", "\u646D"], /* CJK Ideograph */
  ["\u646E", "\u646E"], /* CJK Ideograph */
  ["\u646F", "\u646F"], /* CJK Ideograph */
  ["\u6470", "\u6470"], /* CJK Ideograph */
  ["\u6471", "\u6471"], /* CJK Ideograph */
  ["\u6472", "\u6472"], /* CJK Ideograph */
  ["\u6473", "\u6473"], /* CJK Ideograph */
  ["\u6474", "\u6474"], /* CJK Ideograph */
  ["\u6475", "\u6475"], /* CJK Ideograph */
  ["\u6476", "\u6476"], /* CJK Ideograph */
  ["\u6477", "\u6477"], /* CJK Ideograph */
  ["\u6478", "\u6478"], /* CJK Ideograph */
  ["\u6479", "\u6479"], /* CJK Ideograph */
  ["\u647A", "\u647A"], /* CJK Ideograph */
  ["\u647B", "\u647B"], /* CJK Ideograph */
  ["\u647C", "\u647C"], /* CJK Ideograph */
  ["\u647D", "\u647D"], /* CJK Ideograph */
  ["\u647E", "\u647E"], /* CJK Ideograph */
  ["\u647F", "\u647F"], /* CJK Ideograph */
  ["\u6480", "\u6480"], /* CJK Ideograph */
  ["\u6481", "\u6481"], /* CJK Ideograph */
  ["\u6482", "\u6482"], /* CJK Ideograph */
  ["\u6483", "\u6483"], /* CJK Ideograph */
  ["\u6484", "\u6484"], /* CJK Ideograph */
  ["\u6485", "\u6485"], /* CJK Ideograph */
  ["\u6486", "\u6486"], /* CJK Ideograph */
  ["\u6487", "\u6487"], /* CJK Ideograph */
  ["\u6488", "\u6488"], /* CJK Ideograph */
  ["\u6489", "\u6489"], /* CJK Ideograph */
  ["\u648A", "\u648A"], /* CJK Ideograph */
  ["\u648B", "\u648B"], /* CJK Ideograph */
  ["\u648C", "\u648C"], /* CJK Ideograph */
  ["\u648D", "\u648D"], /* CJK Ideograph */
  ["\u648E", "\u648E"], /* CJK Ideograph */
  ["\u648F", "\u648F"], /* CJK Ideograph */
  ["\u6490", "\u6490"], /* CJK Ideograph */
  ["\u6491", "\u6491"], /* CJK Ideograph */
  ["\u6492", "\u6492"], /* CJK Ideograph */
  ["\u6493", "\u6493"], /* CJK Ideograph */
  ["\u6494", "\u6494"], /* CJK Ideograph */
  ["\u6495", "\u6495"], /* CJK Ideograph */
  ["\u6496", "\u6496"], /* CJK Ideograph */
  ["\u6497", "\u6497"], /* CJK Ideograph */
  ["\u6498", "\u6498"], /* CJK Ideograph */
  ["\u6499", "\u6499"], /* CJK Ideograph */
  ["\u649A", "\u649A"], /* CJK Ideograph */
  ["\u649B", "\u649B"], /* CJK Ideograph */
  ["\u649C", "\u649C"], /* CJK Ideograph */
  ["\u649D", "\u649D"], /* CJK Ideograph */
  ["\u649E", "\u649E"], /* CJK Ideograph */
  ["\u649F", "\u649F"], /* CJK Ideograph */
  ["\u64A0", "\u64A0"], /* CJK Ideograph */
  ["\u64A1", "\u64A1"], /* CJK Ideograph */
  ["\u64A2", "\u64A2"], /* CJK Ideograph */
  ["\u64A3", "\u64A3"], /* CJK Ideograph */
  ["\u64A4", "\u64A4"], /* CJK Ideograph */
  ["\u64A5", "\u64A5"], /* CJK Ideograph */
  ["\u64A6", "\u64A6"], /* CJK Ideograph */
  ["\u64A7", "\u64A7"], /* CJK Ideograph */
  ["\u64A8", "\u64A8"], /* CJK Ideograph */
  ["\u64A9", "\u64A9"], /* CJK Ideograph */
  ["\u64AA", "\u64AA"], /* CJK Ideograph */
  ["\u64AB", "\u64AB"], /* CJK Ideograph */
  ["\u64AC", "\u64AC"], /* CJK Ideograph */
  ["\u64AD", "\u64AD"], /* CJK Ideograph */
  ["\u64AE", "\u64AE"], /* CJK Ideograph */
  ["\u64AF", "\u64AF"], /* CJK Ideograph */
  ["\u64B0", "\u64B0"], /* CJK Ideograph */
  ["\u64B1", "\u64B1"], /* CJK Ideograph */
  ["\u64B2", "\u64B2"], /* CJK Ideograph */
  ["\u64B3", "\u64B3"], /* CJK Ideograph */
  ["\u64B4", "\u64B4"], /* CJK Ideograph */
  ["\u64B5", "\u64B5"], /* CJK Ideograph */
  ["\u64B6", "\u64B6"], /* CJK Ideograph */
  ["\u64B7", "\u64B7"], /* CJK Ideograph */
  ["\u64B8", "\u64B8"], /* CJK Ideograph */
  ["\u64B9", "\u64B9"], /* CJK Ideograph */
  ["\u64BA", "\u64BA"], /* CJK Ideograph */
  ["\u64BB", "\u64BB"], /* CJK Ideograph */
  ["\u64BC", "\u64BC"], /* CJK Ideograph */
  ["\u64BD", "\u64BD"], /* CJK Ideograph */
  ["\u64BE", "\u64BE"], /* CJK Ideograph */
  ["\u64BF", "\u64BF"], /* CJK Ideograph */
  ["\u64C0", "\u64C0"], /* CJK Ideograph */
  ["\u64C1", "\u64C1"], /* CJK Ideograph */
  ["\u64C2", "\u64C2"], /* CJK Ideograph */
  ["\u64C3", "\u64C3"], /* CJK Ideograph */
  ["\u64C4", "\u64C4"], /* CJK Ideograph */
  ["\u64C5", "\u64C5"], /* CJK Ideograph */
  ["\u64C6", "\u64C6"], /* CJK Ideograph */
  ["\u64C7", "\u64C7"], /* CJK Ideograph */
  ["\u64C8", "\u64C8"], /* CJK Ideograph */
  ["\u64C9", "\u64C9"], /* CJK Ideograph */
  ["\u64CA", "\u64CA"], /* CJK Ideograph */
  ["\u64CB", "\u64CB"], /* CJK Ideograph */
  ["\u64CC", "\u64CC"], /* CJK Ideograph */
  ["\u64CD", "\u64CD"], /* CJK Ideograph */
  ["\u64CE", "\u64CE"], /* CJK Ideograph */
  ["\u64CF", "\u64CF"], /* CJK Ideograph */
  ["\u64D0", "\u64D0"], /* CJK Ideograph */
  ["\u64D1", "\u64D1"], /* CJK Ideograph */
  ["\u64D2", "\u64D2"], /* CJK Ideograph */
  ["\u64D3", "\u64D3"], /* CJK Ideograph */
  ["\u64D4", "\u64D4"], /* CJK Ideograph */
  ["\u64D5", "\u64D5"], /* CJK Ideograph */
  ["\u64D6", "\u64D6"], /* CJK Ideograph */
  ["\u64D7", "\u64D7"], /* CJK Ideograph */
  ["\u64D8", "\u64D8"], /* CJK Ideograph */
  ["\u64D9", "\u64D9"], /* CJK Ideograph */
  ["\u64DA", "\u64DA"], /* CJK Ideograph */
  ["\u64DB", "\u64DB"], /* CJK Ideograph */
  ["\u64DC", "\u64DC"], /* CJK Ideograph */
  ["\u64DD", "\u64DD"], /* CJK Ideograph */
  ["\u64DE", "\u64DE"], /* CJK Ideograph */
  ["\u64DF", "\u64DF"], /* CJK Ideograph */
  ["\u64E0", "\u64E0"], /* CJK Ideograph */
  ["\u64E1", "\u64E1"], /* CJK Ideograph */
  ["\u64E2", "\u64E2"], /* CJK Ideograph */
  ["\u64E3", "\u64E3"], /* CJK Ideograph */
  ["\u64E4", "\u64E4"], /* CJK Ideograph */
  ["\u64E5", "\u64E5"], /* CJK Ideograph */
  ["\u64E6", "\u64E6"], /* CJK Ideograph */
  ["\u64E7", "\u64E7"], /* CJK Ideograph */
  ["\u64E8", "\u64E8"], /* CJK Ideograph */
  ["\u64E9", "\u64E9"], /* CJK Ideograph */
  ["\u64EA", "\u64EA"], /* CJK Ideograph */
  ["\u64EB", "\u64EB"], /* CJK Ideograph */
  ["\u64EC", "\u64EC"], /* CJK Ideograph */
  ["\u64ED", "\u64ED"], /* CJK Ideograph */
  ["\u64EE", "\u64EE"], /* CJK Ideograph */
  ["\u64EF", "\u64EF"], /* CJK Ideograph */
  ["\u64F0", "\u64F0"], /* CJK Ideograph */
  ["\u64F1", "\u64F1"], /* CJK Ideograph */
  ["\u64F2", "\u64F2"], /* CJK Ideograph */
  ["\u64F3", "\u64F3"], /* CJK Ideograph */
  ["\u64F4", "\u64F4"], /* CJK Ideograph */
  ["\u64F5", "\u64F5"], /* CJK Ideograph */
  ["\u64F6", "\u64F6"], /* CJK Ideograph */
  ["\u64F7", "\u64F7"], /* CJK Ideograph */
  ["\u64F8", "\u64F8"], /* CJK Ideograph */
  ["\u64F9", "\u64F9"], /* CJK Ideograph */
  ["\u64FA", "\u64FA"], /* CJK Ideograph */
  ["\u64FB", "\u64FB"], /* CJK Ideograph */
  ["\u64FC", "\u64FC"], /* CJK Ideograph */
  ["\u64FD", "\u64FD"], /* CJK Ideograph */
  ["\u64FE", "\u64FE"], /* CJK Ideograph */
  ["\u64FF", "\u64FF"], /* CJK Ideograph */
  ["\u6500", "\u6500"], /* CJK Ideograph */
  ["\u6501", "\u6501"], /* CJK Ideograph */
  ["\u6502", "\u6502"], /* CJK Ideograph */
  ["\u6503", "\u6503"], /* CJK Ideograph */
  ["\u6504", "\u6504"], /* CJK Ideograph */
  ["\u6505", "\u6505"], /* CJK Ideograph */
  ["\u6506", "\u6506"], /* CJK Ideograph */
  ["\u6507", "\u6507"], /* CJK Ideograph */
  ["\u6508", "\u6508"], /* CJK Ideograph */
  ["\u6509", "\u6509"], /* CJK Ideograph */
  ["\u650A", "\u650A"], /* CJK Ideograph */
  ["\u650B", "\u650B"], /* CJK Ideograph */
  ["\u650C", "\u650C"], /* CJK Ideograph */
  ["\u650D", "\u650D"], /* CJK Ideograph */
  ["\u650E", "\u650E"], /* CJK Ideograph */
  ["\u650F", "\u650F"], /* CJK Ideograph */
  ["\u6510", "\u6510"], /* CJK Ideograph */
  ["\u6511", "\u6511"], /* CJK Ideograph */
  ["\u6512", "\u6512"], /* CJK Ideograph */
  ["\u6513", "\u6513"], /* CJK Ideograph */
  ["\u6514", "\u6514"], /* CJK Ideograph */
  ["\u6515", "\u6515"], /* CJK Ideograph */
  ["\u6516", "\u6516"], /* CJK Ideograph */
  ["\u6517", "\u6517"], /* CJK Ideograph */
  ["\u6518", "\u6518"], /* CJK Ideograph */
  ["\u6519", "\u6519"], /* CJK Ideograph */
  ["\u651A", "\u651A"], /* CJK Ideograph */
  ["\u651B", "\u651B"], /* CJK Ideograph */
  ["\u651C", "\u651C"], /* CJK Ideograph */
  ["\u651D", "\u651D"], /* CJK Ideograph */
  ["\u651E", "\u651E"], /* CJK Ideograph */
  ["\u651F", "\u651F"], /* CJK Ideograph */
  ["\u6520", "\u6520"], /* CJK Ideograph */
  ["\u6521", "\u6521"], /* CJK Ideograph */
  ["\u6522", "\u6522"], /* CJK Ideograph */
  ["\u6523", "\u6523"], /* CJK Ideograph */
  ["\u6524", "\u6524"], /* CJK Ideograph */
  ["\u6525", "\u6525"], /* CJK Ideograph */
  ["\u6526", "\u6526"], /* CJK Ideograph */
  ["\u6527", "\u6527"], /* CJK Ideograph */
  ["\u6528", "\u6528"], /* CJK Ideograph */
  ["\u6529", "\u6529"], /* CJK Ideograph */
  ["\u652A", "\u652A"], /* CJK Ideograph */
  ["\u652B", "\u652B"], /* CJK Ideograph */
  ["\u652C", "\u652C"], /* CJK Ideograph */
  ["\u652D", "\u652D"], /* CJK Ideograph */
  ["\u652E", "\u652E"], /* CJK Ideograph */
  ["\u652F", "\u652F"], /* CJK Ideograph */
  ["\u6530", "\u6530"], /* CJK Ideograph */
  ["\u6531", "\u6531"], /* CJK Ideograph */
  ["\u6532", "\u6532"], /* CJK Ideograph */
  ["\u6533", "\u6533"], /* CJK Ideograph */
  ["\u6534", "\u6534"], /* CJK Ideograph */
  ["\u6535", "\u6535"], /* CJK Ideograph */
  ["\u6536", "\u6536"], /* CJK Ideograph */
  ["\u6537", "\u6537"], /* CJK Ideograph */
  ["\u6538", "\u6538"], /* CJK Ideograph */
  ["\u6539", "\u6539"], /* CJK Ideograph */
  ["\u653A", "\u653A"], /* CJK Ideograph */
  ["\u653B", "\u653B"], /* CJK Ideograph */
  ["\u653C", "\u653C"], /* CJK Ideograph */
  ["\u653D", "\u653D"], /* CJK Ideograph */
  ["\u653E", "\u653E"], /* CJK Ideograph */
  ["\u653F", "\u653F"], /* CJK Ideograph */
  ["\u6540", "\u6540"], /* CJK Ideograph */
  ["\u6541", "\u6541"], /* CJK Ideograph */
  ["\u6542", "\u6542"], /* CJK Ideograph */
  ["\u6543", "\u6543"], /* CJK Ideograph */
  ["\u6544", "\u6544"], /* CJK Ideograph */
  ["\u6545", "\u6545"], /* CJK Ideograph */
  ["\u6546", "\u6546"], /* CJK Ideograph */
  ["\u6547", "\u6547"], /* CJK Ideograph */
  ["\u6548", "\u6548"], /* CJK Ideograph */
  ["\u6549", "\u6549"], /* CJK Ideograph */
  ["\u654A", "\u654A"], /* CJK Ideograph */
  ["\u654B", "\u654B"], /* CJK Ideograph */
  ["\u654C", "\u654C"], /* CJK Ideograph */
  ["\u654D", "\u654D"], /* CJK Ideograph */
  ["\u654E", "\u654E"], /* CJK Ideograph */
  ["\u654F", "\u654F"], /* CJK Ideograph */
  ["\u6550", "\u6550"], /* CJK Ideograph */
  ["\u6551", "\u6551"], /* CJK Ideograph */
  ["\u6552", "\u6552"], /* CJK Ideograph */
  ["\u6553", "\u6553"], /* CJK Ideograph */
  ["\u6554", "\u6554"], /* CJK Ideograph */
  ["\u6555", "\u6555"], /* CJK Ideograph */
  ["\u6556", "\u6556"], /* CJK Ideograph */
  ["\u6557", "\u6557"], /* CJK Ideograph */
  ["\u6558", "\u6558"], /* CJK Ideograph */
  ["\u6559", "\u6559"], /* CJK Ideograph */
  ["\u655A", "\u655A"], /* CJK Ideograph */
  ["\u655B", "\u655B"], /* CJK Ideograph */
  ["\u655C", "\u655C"], /* CJK Ideograph */
  ["\u655D", "\u655D"], /* CJK Ideograph */
  ["\u655E", "\u655E"], /* CJK Ideograph */
  ["\u655F", "\u655F"], /* CJK Ideograph */
  ["\u6560", "\u6560"], /* CJK Ideograph */
  ["\u6561", "\u6561"], /* CJK Ideograph */
  ["\u6562", "\u6562"], /* CJK Ideograph */
  ["\u6563", "\u6563"], /* CJK Ideograph */
  ["\u6564", "\u6564"], /* CJK Ideograph */
  ["\u6565", "\u6565"], /* CJK Ideograph */
  ["\u6566", "\u6566"], /* CJK Ideograph */
  ["\u6567", "\u6567"], /* CJK Ideograph */
  ["\u6568", "\u6568"], /* CJK Ideograph */
  ["\u6569", "\u6569"], /* CJK Ideograph */
  ["\u656A", "\u656A"], /* CJK Ideograph */
  ["\u656B", "\u656B"], /* CJK Ideograph */
  ["\u656C", "\u656C"], /* CJK Ideograph */
  ["\u656D", "\u656D"], /* CJK Ideograph */
  ["\u656E", "\u656E"], /* CJK Ideograph */
  ["\u656F", "\u656F"], /* CJK Ideograph */
  ["\u6570", "\u6570"], /* CJK Ideograph */
  ["\u6571", "\u6571"], /* CJK Ideograph */
  ["\u6572", "\u6572"], /* CJK Ideograph */
  ["\u6573", "\u6573"], /* CJK Ideograph */
  ["\u6574", "\u6574"], /* CJK Ideograph */
  ["\u6575", "\u6575"], /* CJK Ideograph */
  ["\u6576", "\u6576"], /* CJK Ideograph */
  ["\u6577", "\u6577"], /* CJK Ideograph */
  ["\u6578", "\u6578"], /* CJK Ideograph */
  ["\u6579", "\u6579"], /* CJK Ideograph */
  ["\u657A", "\u657A"], /* CJK Ideograph */
  ["\u657B", "\u657B"], /* CJK Ideograph */
  ["\u657C", "\u657C"], /* CJK Ideograph */
  ["\u657D", "\u657D"], /* CJK Ideograph */
  ["\u657E", "\u657E"], /* CJK Ideograph */
  ["\u657F", "\u657F"], /* CJK Ideograph */
  ["\u6580", "\u6580"], /* CJK Ideograph */
  ["\u6581", "\u6581"], /* CJK Ideograph */
  ["\u6582", "\u6582"], /* CJK Ideograph */
  ["\u6583", "\u6583"], /* CJK Ideograph */
  ["\u6584", "\u6584"], /* CJK Ideograph */
  ["\u6585", "\u6585"], /* CJK Ideograph */
  ["\u6586", "\u6586"], /* CJK Ideograph */
  ["\u6587", "\u6587"], /* CJK Ideograph */
  ["\u6588", "\u6588"], /* CJK Ideograph */
  ["\u6589", "\u6589"], /* CJK Ideograph */
  ["\u658A", "\u658A"], /* CJK Ideograph */
  ["\u658B", "\u658B"], /* CJK Ideograph */
  ["\u658C", "\u658C"], /* CJK Ideograph */
  ["\u658D", "\u658D"], /* CJK Ideograph */
  ["\u658E", "\u658E"], /* CJK Ideograph */
  ["\u658F", "\u658F"], /* CJK Ideograph */
  ["\u6590", "\u6590"], /* CJK Ideograph */
  ["\u6591", "\u6591"], /* CJK Ideograph */
  ["\u6592", "\u6592"], /* CJK Ideograph */
  ["\u6593", "\u6593"], /* CJK Ideograph */
  ["\u6594", "\u6594"], /* CJK Ideograph */
  ["\u6595", "\u6595"], /* CJK Ideograph */
  ["\u6596", "\u6596"], /* CJK Ideograph */
  ["\u6597", "\u6597"], /* CJK Ideograph */
  ["\u6598", "\u6598"], /* CJK Ideograph */
  ["\u6599", "\u6599"], /* CJK Ideograph */
  ["\u659A", "\u659A"], /* CJK Ideograph */
  ["\u659B", "\u659B"], /* CJK Ideograph */
  ["\u659C", "\u659C"], /* CJK Ideograph */
  ["\u659D", "\u659D"], /* CJK Ideograph */
  ["\u659E", "\u659E"], /* CJK Ideograph */
  ["\u659F", "\u659F"], /* CJK Ideograph */
  ["\u65A0", "\u65A0"], /* CJK Ideograph */
  ["\u65A1", "\u65A1"], /* CJK Ideograph */
  ["\u65A2", "\u65A2"], /* CJK Ideograph */
  ["\u65A3", "\u65A3"], /* CJK Ideograph */
  ["\u65A4", "\u65A4"], /* CJK Ideograph */
  ["\u65A5", "\u65A5"], /* CJK Ideograph */
  ["\u65A6", "\u65A6"], /* CJK Ideograph */
  ["\u65A7", "\u65A7"], /* CJK Ideograph */
  ["\u65A8", "\u65A8"], /* CJK Ideograph */
  ["\u65A9", "\u65A9"], /* CJK Ideograph */
  ["\u65AA", "\u65AA"], /* CJK Ideograph */
  ["\u65AB", "\u65AB"], /* CJK Ideograph */
  ["\u65AC", "\u65AC"], /* CJK Ideograph */
  ["\u65AD", "\u65AD"], /* CJK Ideograph */
  ["\u65AE", "\u65AE"], /* CJK Ideograph */
  ["\u65AF", "\u65AF"], /* CJK Ideograph */
  ["\u65B0", "\u65B0"], /* CJK Ideograph */
  ["\u65B1", "\u65B1"], /* CJK Ideograph */
  ["\u65B2", "\u65B2"], /* CJK Ideograph */
  ["\u65B3", "\u65B3"], /* CJK Ideograph */
  ["\u65B4", "\u65B4"], /* CJK Ideograph */
  ["\u65B5", "\u65B5"], /* CJK Ideograph */
  ["\u65B6", "\u65B6"], /* CJK Ideograph */
  ["\u65B7", "\u65B7"], /* CJK Ideograph */
  ["\u65B8", "\u65B8"], /* CJK Ideograph */
  ["\u65B9", "\u65B9"], /* CJK Ideograph */
  ["\u65BA", "\u65BA"], /* CJK Ideograph */
  ["\u65BB", "\u65BB"], /* CJK Ideograph */
  ["\u65BC", "\u65BC"], /* CJK Ideograph */
  ["\u65BD", "\u65BD"], /* CJK Ideograph */
  ["\u65BE", "\u65BE"], /* CJK Ideograph */
  ["\u65BF", "\u65BF"], /* CJK Ideograph */
  ["\u65C0", "\u65C0"], /* CJK Ideograph */
  ["\u65C1", "\u65C1"], /* CJK Ideograph */
  ["\u65C2", "\u65C2"], /* CJK Ideograph */
  ["\u65C3", "\u65C3"], /* CJK Ideograph */
  ["\u65C4", "\u65C4"], /* CJK Ideograph */
  ["\u65C5", "\u65C5"], /* CJK Ideograph */
  ["\u65C6", "\u65C6"], /* CJK Ideograph */
  ["\u65C7", "\u65C7"], /* CJK Ideograph */
  ["\u65C8", "\u65C8"], /* CJK Ideograph */
  ["\u65C9", "\u65C9"], /* CJK Ideograph */
  ["\u65CA", "\u65CA"], /* CJK Ideograph */
  ["\u65CB", "\u65CB"], /* CJK Ideograph */
  ["\u65CC", "\u65CC"], /* CJK Ideograph */
  ["\u65CD", "\u65CD"], /* CJK Ideograph */
  ["\u65CE", "\u65CE"], /* CJK Ideograph */
  ["\u65CF", "\u65CF"], /* CJK Ideograph */
  ["\u65D0", "\u65D0"], /* CJK Ideograph */
  ["\u65D1", "\u65D1"], /* CJK Ideograph */
  ["\u65D2", "\u65D2"], /* CJK Ideograph */
  ["\u65D3", "\u65D3"], /* CJK Ideograph */
  ["\u65D4", "\u65D4"], /* CJK Ideograph */
  ["\u65D5", "\u65D5"], /* CJK Ideograph */
  ["\u65D6", "\u65D6"], /* CJK Ideograph */
  ["\u65D7", "\u65D7"], /* CJK Ideograph */
  ["\u65D8", "\u65D8"], /* CJK Ideograph */
  ["\u65D9", "\u65D9"], /* CJK Ideograph */
  ["\u65DA", "\u65DA"], /* CJK Ideograph */
  ["\u65DB", "\u65DB"], /* CJK Ideograph */
  ["\u65DC", "\u65DC"], /* CJK Ideograph */
  ["\u65DD", "\u65DD"], /* CJK Ideograph */
  ["\u65DE", "\u65DE"], /* CJK Ideograph */
  ["\u65DF", "\u65DF"], /* CJK Ideograph */
  ["\u65E0", "\u65E0"], /* CJK Ideograph */
  ["\u65E1", "\u65E1"], /* CJK Ideograph */
  ["\u65E2", "\u65E2"], /* CJK Ideograph */
  ["\u65E3", "\u65E3"], /* CJK Ideograph */
  ["\u65E4", "\u65E4"], /* CJK Ideograph */
  ["\u65E5", "\u65E5"], /* CJK Ideograph */
  ["\u65E6", "\u65E6"], /* CJK Ideograph */
  ["\u65E7", "\u65E7"], /* CJK Ideograph */
  ["\u65E8", "\u65E8"], /* CJK Ideograph */
  ["\u65E9", "\u65E9"], /* CJK Ideograph */
  ["\u65EA", "\u65EA"], /* CJK Ideograph */
  ["\u65EB", "\u65EB"], /* CJK Ideograph */
  ["\u65EC", "\u65EC"], /* CJK Ideograph */
  ["\u65ED", "\u65ED"], /* CJK Ideograph */
  ["\u65EE", "\u65EE"], /* CJK Ideograph */
  ["\u65EF", "\u65EF"], /* CJK Ideograph */
  ["\u65F0", "\u65F0"], /* CJK Ideograph */
  ["\u65F1", "\u65F1"], /* CJK Ideograph */
  ["\u65F2", "\u65F2"], /* CJK Ideograph */
  ["\u65F3", "\u65F3"], /* CJK Ideograph */
  ["\u65F4", "\u65F4"], /* CJK Ideograph */
  ["\u65F5", "\u65F5"], /* CJK Ideograph */
  ["\u65F6", "\u65F6"], /* CJK Ideograph */
  ["\u65F7", "\u65F7"], /* CJK Ideograph */
  ["\u65F8", "\u65F8"], /* CJK Ideograph */
  ["\u65F9", "\u65F9"], /* CJK Ideograph */
  ["\u65FA", "\u65FA"], /* CJK Ideograph */
  ["\u65FB", "\u65FB"], /* CJK Ideograph */
  ["\u65FC", "\u65FC"], /* CJK Ideograph */
  ["\u65FD", "\u65FD"], /* CJK Ideograph */
  ["\u65FE", "\u65FE"], /* CJK Ideograph */
  ["\u65FF", "\u65FF"], /* CJK Ideograph */
  ["\u6600", "\u6600"], /* CJK Ideograph */
  ["\u6601", "\u6601"], /* CJK Ideograph */
  ["\u6602", "\u6602"], /* CJK Ideograph */
  ["\u6603", "\u6603"], /* CJK Ideograph */
  ["\u6604", "\u6604"], /* CJK Ideograph */
  ["\u6605", "\u6605"], /* CJK Ideograph */
  ["\u6606", "\u6606"], /* CJK Ideograph */
  ["\u6607", "\u6607"], /* CJK Ideograph */
  ["\u6608", "\u6608"], /* CJK Ideograph */
  ["\u6609", "\u6609"], /* CJK Ideograph */
  ["\u660A", "\u660A"], /* CJK Ideograph */
  ["\u660B", "\u660B"], /* CJK Ideograph */
  ["\u660C", "\u660C"], /* CJK Ideograph */
  ["\u660D", "\u660D"], /* CJK Ideograph */
  ["\u660E", "\u660E"], /* CJK Ideograph */
  ["\u660F", "\u660F"], /* CJK Ideograph */
  ["\u6610", "\u6610"], /* CJK Ideograph */
  ["\u6611", "\u6611"], /* CJK Ideograph */
  ["\u6612", "\u6612"], /* CJK Ideograph */
  ["\u6613", "\u6613"], /* CJK Ideograph */
  ["\u6614", "\u6614"], /* CJK Ideograph */
  ["\u6615", "\u6615"], /* CJK Ideograph */
  ["\u6616", "\u6616"], /* CJK Ideograph */
  ["\u6617", "\u6617"], /* CJK Ideograph */
  ["\u6618", "\u6618"], /* CJK Ideograph */
  ["\u6619", "\u6619"], /* CJK Ideograph */
  ["\u661A", "\u661A"], /* CJK Ideograph */
  ["\u661B", "\u661B"], /* CJK Ideograph */
  ["\u661C", "\u661C"], /* CJK Ideograph */
  ["\u661D", "\u661D"], /* CJK Ideograph */
  ["\u661E", "\u661E"], /* CJK Ideograph */
  ["\u661F", "\u661F"], /* CJK Ideograph */
  ["\u6620", "\u6620"], /* CJK Ideograph */
  ["\u6621", "\u6621"], /* CJK Ideograph */
  ["\u6622", "\u6622"], /* CJK Ideograph */
  ["\u6623", "\u6623"], /* CJK Ideograph */
  ["\u6624", "\u6624"], /* CJK Ideograph */
  ["\u6625", "\u6625"], /* CJK Ideograph */
  ["\u6626", "\u6626"], /* CJK Ideograph */
  ["\u6627", "\u6627"], /* CJK Ideograph */
  ["\u6628", "\u6628"], /* CJK Ideograph */
  ["\u6629", "\u6629"], /* CJK Ideograph */
  ["\u662A", "\u662A"], /* CJK Ideograph */
  ["\u662B", "\u662B"], /* CJK Ideograph */
  ["\u662C", "\u662C"], /* CJK Ideograph */
  ["\u662D", "\u662D"], /* CJK Ideograph */
  ["\u662E", "\u662E"], /* CJK Ideograph */
  ["\u662F", "\u662F"], /* CJK Ideograph */
  ["\u6630", "\u6630"], /* CJK Ideograph */
  ["\u6631", "\u6631"], /* CJK Ideograph */
  ["\u6632", "\u6632"], /* CJK Ideograph */
  ["\u6633", "\u6633"], /* CJK Ideograph */
  ["\u6634", "\u6634"], /* CJK Ideograph */
  ["\u6635", "\u6635"], /* CJK Ideograph */
  ["\u6636", "\u6636"], /* CJK Ideograph */
  ["\u6637", "\u6637"], /* CJK Ideograph */
  ["\u6638", "\u6638"], /* CJK Ideograph */
  ["\u6639", "\u6639"], /* CJK Ideograph */
  ["\u663A", "\u663A"], /* CJK Ideograph */
  ["\u663B", "\u663B"], /* CJK Ideograph */
  ["\u663C", "\u663C"], /* CJK Ideograph */
  ["\u663D", "\u663D"], /* CJK Ideograph */
  ["\u663E", "\u663E"], /* CJK Ideograph */
  ["\u663F", "\u663F"], /* CJK Ideograph */
  ["\u6640", "\u6640"], /* CJK Ideograph */
  ["\u6641", "\u6641"], /* CJK Ideograph */
  ["\u6642", "\u6642"], /* CJK Ideograph */
  ["\u6643", "\u6643"], /* CJK Ideograph */
  ["\u6644", "\u6644"], /* CJK Ideograph */
  ["\u6645", "\u6645"], /* CJK Ideograph */
  ["\u6646", "\u6646"], /* CJK Ideograph */
  ["\u6647", "\u6647"], /* CJK Ideograph */
  ["\u6648", "\u6648"], /* CJK Ideograph */
  ["\u6649", "\u6649"], /* CJK Ideograph */
  ["\u664A", "\u664A"], /* CJK Ideograph */
  ["\u664B", "\u664B"], /* CJK Ideograph */
  ["\u664C", "\u664C"], /* CJK Ideograph */
  ["\u664D", "\u664D"], /* CJK Ideograph */
  ["\u664E", "\u664E"], /* CJK Ideograph */
  ["\u664F", "\u664F"], /* CJK Ideograph */
  ["\u6650", "\u6650"], /* CJK Ideograph */
  ["\u6651", "\u6651"], /* CJK Ideograph */
  ["\u6652", "\u6652"], /* CJK Ideograph */
  ["\u6653", "\u6653"], /* CJK Ideograph */
  ["\u6654", "\u6654"], /* CJK Ideograph */
  ["\u6655", "\u6655"], /* CJK Ideograph */
  ["\u6656", "\u6656"], /* CJK Ideograph */
  ["\u6657", "\u6657"], /* CJK Ideograph */
  ["\u6658", "\u6658"], /* CJK Ideograph */
  ["\u6659", "\u6659"], /* CJK Ideograph */
  ["\u665A", "\u665A"], /* CJK Ideograph */
  ["\u665B", "\u665B"], /* CJK Ideograph */
  ["\u665C", "\u665C"], /* CJK Ideograph */
  ["\u665D", "\u665D"], /* CJK Ideograph */
  ["\u665E", "\u665E"], /* CJK Ideograph */
  ["\u665F", "\u665F"], /* CJK Ideograph */
  ["\u6660", "\u6660"], /* CJK Ideograph */
  ["\u6661", "\u6661"], /* CJK Ideograph */
  ["\u6662", "\u6662"], /* CJK Ideograph */
  ["\u6663", "\u6663"], /* CJK Ideograph */
  ["\u6664", "\u6664"], /* CJK Ideograph */
  ["\u6665", "\u6665"], /* CJK Ideograph */
  ["\u6666", "\u6666"], /* CJK Ideograph */
  ["\u6667", "\u6667"], /* CJK Ideograph */
  ["\u6668", "\u6668"], /* CJK Ideograph */
  ["\u6669", "\u6669"], /* CJK Ideograph */
  ["\u666A", "\u666A"], /* CJK Ideograph */
  ["\u666B", "\u666B"], /* CJK Ideograph */
  ["\u666C", "\u666C"], /* CJK Ideograph */
  ["\u666D", "\u666D"], /* CJK Ideograph */
  ["\u666E", "\u666E"], /* CJK Ideograph */
  ["\u666F", "\u666F"], /* CJK Ideograph */
  ["\u6670", "\u6670"], /* CJK Ideograph */
  ["\u6671", "\u6671"], /* CJK Ideograph */
  ["\u6672", "\u6672"], /* CJK Ideograph */
  ["\u6673", "\u6673"], /* CJK Ideograph */
  ["\u6674", "\u6674"], /* CJK Ideograph */
  ["\u6675", "\u6675"], /* CJK Ideograph */
  ["\u6676", "\u6676"], /* CJK Ideograph */
  ["\u6677", "\u6677"], /* CJK Ideograph */
  ["\u6678", "\u6678"], /* CJK Ideograph */
  ["\u6679", "\u6679"], /* CJK Ideograph */
  ["\u667A", "\u667A"], /* CJK Ideograph */
  ["\u667B", "\u667B"], /* CJK Ideograph */
  ["\u667C", "\u667C"], /* CJK Ideograph */
  ["\u667D", "\u667D"], /* CJK Ideograph */
  ["\u667E", "\u667E"], /* CJK Ideograph */
  ["\u667F", "\u667F"], /* CJK Ideograph */
  ["\u6680", "\u6680"], /* CJK Ideograph */
  ["\u6681", "\u6681"], /* CJK Ideograph */
  ["\u6682", "\u6682"], /* CJK Ideograph */
  ["\u6683", "\u6683"], /* CJK Ideograph */
  ["\u6684", "\u6684"], /* CJK Ideograph */
  ["\u6685", "\u6685"], /* CJK Ideograph */
  ["\u6686", "\u6686"], /* CJK Ideograph */
  ["\u6687", "\u6687"], /* CJK Ideograph */
  ["\u6688", "\u6688"], /* CJK Ideograph */
  ["\u6689", "\u6689"], /* CJK Ideograph */
  ["\u668A", "\u668A"], /* CJK Ideograph */
  ["\u668B", "\u668B"], /* CJK Ideograph */
  ["\u668C", "\u668C"], /* CJK Ideograph */
  ["\u668D", "\u668D"], /* CJK Ideograph */
  ["\u668E", "\u668E"], /* CJK Ideograph */
  ["\u668F", "\u668F"], /* CJK Ideograph */
  ["\u6690", "\u6690"], /* CJK Ideograph */
  ["\u6691", "\u6691"], /* CJK Ideograph */
  ["\u6692", "\u6692"], /* CJK Ideograph */
  ["\u6693", "\u6693"], /* CJK Ideograph */
  ["\u6694", "\u6694"], /* CJK Ideograph */
  ["\u6695", "\u6695"], /* CJK Ideograph */
  ["\u6696", "\u6696"], /* CJK Ideograph */
  ["\u6697", "\u6697"], /* CJK Ideograph */
  ["\u6698", "\u6698"], /* CJK Ideograph */
  ["\u6699", "\u6699"], /* CJK Ideograph */
  ["\u669A", "\u669A"], /* CJK Ideograph */
  ["\u669B", "\u669B"], /* CJK Ideograph */
  ["\u669C", "\u669C"], /* CJK Ideograph */
  ["\u669D", "\u669D"], /* CJK Ideograph */
  ["\u669E", "\u669E"], /* CJK Ideograph */
  ["\u669F", "\u669F"], /* CJK Ideograph */
  ["\u66A0", "\u66A0"], /* CJK Ideograph */
  ["\u66A1", "\u66A1"], /* CJK Ideograph */
  ["\u66A2", "\u66A2"], /* CJK Ideograph */
  ["\u66A3", "\u66A3"], /* CJK Ideograph */
  ["\u66A4", "\u66A4"], /* CJK Ideograph */
  ["\u66A5", "\u66A5"], /* CJK Ideograph */
  ["\u66A6", "\u66A6"], /* CJK Ideograph */
  ["\u66A7", "\u66A7"], /* CJK Ideograph */
  ["\u66A8", "\u66A8"], /* CJK Ideograph */
  ["\u66A9", "\u66A9"], /* CJK Ideograph */
  ["\u66AA", "\u66AA"], /* CJK Ideograph */
  ["\u66AB", "\u66AB"], /* CJK Ideograph */
  ["\u66AC", "\u66AC"], /* CJK Ideograph */
  ["\u66AD", "\u66AD"], /* CJK Ideograph */
  ["\u66AE", "\u66AE"], /* CJK Ideograph */
  ["\u66AF", "\u66AF"], /* CJK Ideograph */
  ["\u66B0", "\u66B0"], /* CJK Ideograph */
  ["\u66B1", "\u66B1"], /* CJK Ideograph */
  ["\u66B2", "\u66B2"], /* CJK Ideograph */
  ["\u66B3", "\u66B3"], /* CJK Ideograph */
  ["\u66B4", "\u66B4"], /* CJK Ideograph */
  ["\u66B5", "\u66B5"], /* CJK Ideograph */
  ["\u66B6", "\u66B6"], /* CJK Ideograph */
  ["\u66B7", "\u66B7"], /* CJK Ideograph */
  ["\u66B8", "\u66B8"], /* CJK Ideograph */
  ["\u66B9", "\u66B9"], /* CJK Ideograph */
  ["\u66BA", "\u66BA"], /* CJK Ideograph */
  ["\u66BB", "\u66BB"], /* CJK Ideograph */
  ["\u66BC", "\u66BC"], /* CJK Ideograph */
  ["\u66BD", "\u66BD"], /* CJK Ideograph */
  ["\u66BE", "\u66BE"], /* CJK Ideograph */
  ["\u66BF", "\u66BF"], /* CJK Ideograph */
  ["\u66C0", "\u66C0"], /* CJK Ideograph */
  ["\u66C1", "\u66C1"], /* CJK Ideograph */
  ["\u66C2", "\u66C2"], /* CJK Ideograph */
  ["\u66C3", "\u66C3"], /* CJK Ideograph */
  ["\u66C4", "\u66C4"], /* CJK Ideograph */
  ["\u66C5", "\u66C5"], /* CJK Ideograph */
  ["\u66C6", "\u66C6"], /* CJK Ideograph */
  ["\u66C7", "\u66C7"], /* CJK Ideograph */
  ["\u66C8", "\u66C8"], /* CJK Ideograph */
  ["\u66C9", "\u66C9"], /* CJK Ideograph */
  ["\u66CA", "\u66CA"], /* CJK Ideograph */
  ["\u66CB", "\u66CB"], /* CJK Ideograph */
  ["\u66CC", "\u66CC"], /* CJK Ideograph */
  ["\u66CD", "\u66CD"], /* CJK Ideograph */
  ["\u66CE", "\u66CE"], /* CJK Ideograph */
  ["\u66CF", "\u66CF"], /* CJK Ideograph */
  ["\u66D0", "\u66D0"], /* CJK Ideograph */
  ["\u66D1", "\u66D1"], /* CJK Ideograph */
  ["\u66D2", "\u66D2"], /* CJK Ideograph */
  ["\u66D3", "\u66D3"], /* CJK Ideograph */
  ["\u66D4", "\u66D4"], /* CJK Ideograph */
  ["\u66D5", "\u66D5"], /* CJK Ideograph */
  ["\u66D6", "\u66D6"], /* CJK Ideograph */
  ["\u66D7", "\u66D7"], /* CJK Ideograph */
  ["\u66D8", "\u66D8"], /* CJK Ideograph */
  ["\u66D9", "\u66D9"], /* CJK Ideograph */
  ["\u66DA", "\u66DA"], /* CJK Ideograph */
  ["\u66DB", "\u66DB"], /* CJK Ideograph */
  ["\u66DC", "\u66DC"], /* CJK Ideograph */
  ["\u66DD", "\u66DD"], /* CJK Ideograph */
  ["\u66DE", "\u66DE"], /* CJK Ideograph */
  ["\u66DF", "\u66DF"], /* CJK Ideograph */
  ["\u66E0", "\u66E0"], /* CJK Ideograph */
  ["\u66E1", "\u66E1"], /* CJK Ideograph */
  ["\u66E2", "\u66E2"], /* CJK Ideograph */
  ["\u66E3", "\u66E3"], /* CJK Ideograph */
  ["\u66E4", "\u66E4"], /* CJK Ideograph */
  ["\u66E5", "\u66E5"], /* CJK Ideograph */
  ["\u66E6", "\u66E6"], /* CJK Ideograph */
  ["\u66E7", "\u66E7"], /* CJK Ideograph */
  ["\u66E8", "\u66E8"], /* CJK Ideograph */
  ["\u66E9", "\u66E9"], /* CJK Ideograph */
  ["\u66EA", "\u66EA"], /* CJK Ideograph */
  ["\u66EB", "\u66EB"], /* CJK Ideograph */
  ["\u66EC", "\u66EC"], /* CJK Ideograph */
  ["\u66ED", "\u66ED"], /* CJK Ideograph */
  ["\u66EE", "\u66EE"], /* CJK Ideograph */
  ["\u66EF", "\u66EF"], /* CJK Ideograph */
  ["\u66F0", "\u66F0"], /* CJK Ideograph */
  ["\u66F1", "\u66F1"], /* CJK Ideograph */
  ["\u66F2", "\u66F2"], /* CJK Ideograph */
  ["\u66F3", "\u66F3"], /* CJK Ideograph */
  ["\u66F4", "\u66F4"], /* CJK Ideograph */
  ["\u66F5", "\u66F5"], /* CJK Ideograph */
  ["\u66F6", "\u66F6"], /* CJK Ideograph */
  ["\u66F7", "\u66F7"], /* CJK Ideograph */
  ["\u66F8", "\u66F8"], /* CJK Ideograph */
  ["\u66F9", "\u66F9"], /* CJK Ideograph */
  ["\u66FA", "\u66FA"], /* CJK Ideograph */
  ["\u66FB", "\u66FB"], /* CJK Ideograph */
  ["\u66FC", "\u66FC"], /* CJK Ideograph */
  ["\u66FD", "\u66FD"], /* CJK Ideograph */
  ["\u66FE", "\u66FE"], /* CJK Ideograph */
  ["\u66FF", "\u66FF"], /* CJK Ideograph */
  ["\u6700", "\u6700"], /* CJK Ideograph */
  ["\u6701", "\u6701"], /* CJK Ideograph */
  ["\u6702", "\u6702"], /* CJK Ideograph */
  ["\u6703", "\u6703"], /* CJK Ideograph */
  ["\u6704", "\u6704"], /* CJK Ideograph */
  ["\u6705", "\u6705"], /* CJK Ideograph */
  ["\u6706", "\u6706"], /* CJK Ideograph */
  ["\u6707", "\u6707"], /* CJK Ideograph */
  ["\u6708", "\u6708"], /* CJK Ideograph */
  ["\u6709", "\u6709"], /* CJK Ideograph */
  ["\u670A", "\u670A"], /* CJK Ideograph */
  ["\u670B", "\u670B"], /* CJK Ideograph */
  ["\u670C", "\u670C"], /* CJK Ideograph */
  ["\u670D", "\u670D"], /* CJK Ideograph */
  ["\u670E", "\u670E"], /* CJK Ideograph */
  ["\u670F", "\u670F"], /* CJK Ideograph */
  ["\u6710", "\u6710"], /* CJK Ideograph */
  ["\u6711", "\u6711"], /* CJK Ideograph */
  ["\u6712", "\u6712"], /* CJK Ideograph */
  ["\u6713", "\u6713"], /* CJK Ideograph */
  ["\u6714", "\u6714"], /* CJK Ideograph */
  ["\u6715", "\u6715"], /* CJK Ideograph */
  ["\u6716", "\u6716"], /* CJK Ideograph */
  ["\u6717", "\u6717"], /* CJK Ideograph */
  ["\u6718", "\u6718"], /* CJK Ideograph */
  ["\u6719", "\u6719"], /* CJK Ideograph */
  ["\u671A", "\u671A"], /* CJK Ideograph */
  ["\u671B", "\u671B"], /* CJK Ideograph */
  ["\u671C", "\u671C"], /* CJK Ideograph */
  ["\u671D", "\u671D"], /* CJK Ideograph */
  ["\u671E", "\u671E"], /* CJK Ideograph */
  ["\u671F", "\u671F"], /* CJK Ideograph */
  ["\u6720", "\u6720"], /* CJK Ideograph */
  ["\u6721", "\u6721"], /* CJK Ideograph */
  ["\u6722", "\u6722"], /* CJK Ideograph */
  ["\u6723", "\u6723"], /* CJK Ideograph */
  ["\u6724", "\u6724"], /* CJK Ideograph */
  ["\u6725", "\u6725"], /* CJK Ideograph */
  ["\u6726", "\u6726"], /* CJK Ideograph */
  ["\u6727", "\u6727"], /* CJK Ideograph */
  ["\u6728", "\u6728"], /* CJK Ideograph */
  ["\u6729", "\u6729"], /* CJK Ideograph */
  ["\u672A", "\u672A"], /* CJK Ideograph */
  ["\u672B", "\u672B"], /* CJK Ideograph */
  ["\u672C", "\u672C"], /* CJK Ideograph */
  ["\u672D", "\u672D"], /* CJK Ideograph */
  ["\u672E", "\u672E"], /* CJK Ideograph */
  ["\u672F", "\u672F"], /* CJK Ideograph */
  ["\u6730", "\u6730"], /* CJK Ideograph */
  ["\u6731", "\u6731"], /* CJK Ideograph */
  ["\u6732", "\u6732"], /* CJK Ideograph */
  ["\u6733", "\u6733"], /* CJK Ideograph */
  ["\u6734", "\u6734"], /* CJK Ideograph */
  ["\u6735", "\u6735"], /* CJK Ideograph */
  ["\u6736", "\u6736"], /* CJK Ideograph */
  ["\u6737", "\u6737"], /* CJK Ideograph */
  ["\u6738", "\u6738"], /* CJK Ideograph */
  ["\u6739", "\u6739"], /* CJK Ideograph */
  ["\u673A", "\u673A"], /* CJK Ideograph */
  ["\u673B", "\u673B"], /* CJK Ideograph */
  ["\u673C", "\u673C"], /* CJK Ideograph */
  ["\u673D", "\u673D"], /* CJK Ideograph */
  ["\u673E", "\u673E"], /* CJK Ideograph */
  ["\u673F", "\u673F"], /* CJK Ideograph */
  ["\u6740", "\u6740"], /* CJK Ideograph */
  ["\u6741", "\u6741"], /* CJK Ideograph */
  ["\u6742", "\u6742"], /* CJK Ideograph */
  ["\u6743", "\u6743"], /* CJK Ideograph */
  ["\u6744", "\u6744"], /* CJK Ideograph */
  ["\u6745", "\u6745"], /* CJK Ideograph */
  ["\u6746", "\u6746"], /* CJK Ideograph */
  ["\u6747", "\u6747"], /* CJK Ideograph */
  ["\u6748", "\u6748"], /* CJK Ideograph */
  ["\u6749", "\u6749"], /* CJK Ideograph */
  ["\u674A", "\u674A"], /* CJK Ideograph */
  ["\u674B", "\u674B"], /* CJK Ideograph */
  ["\u674C", "\u674C"], /* CJK Ideograph */
  ["\u674D", "\u674D"], /* CJK Ideograph */
  ["\u674E", "\u674E"], /* CJK Ideograph */
  ["\u674F", "\u674F"], /* CJK Ideograph */
  ["\u6750", "\u6750"], /* CJK Ideograph */
  ["\u6751", "\u6751"], /* CJK Ideograph */
  ["\u6752", "\u6752"], /* CJK Ideograph */
  ["\u6753", "\u6753"], /* CJK Ideograph */
  ["\u6754", "\u6754"], /* CJK Ideograph */
  ["\u6755", "\u6755"], /* CJK Ideograph */
  ["\u6756", "\u6756"], /* CJK Ideograph */
  ["\u6757", "\u6757"], /* CJK Ideograph */
  ["\u6758", "\u6758"], /* CJK Ideograph */
  ["\u6759", "\u6759"], /* CJK Ideograph */
  ["\u675A", "\u675A"], /* CJK Ideograph */
  ["\u675B", "\u675B"], /* CJK Ideograph */
  ["\u675C", "\u675C"], /* CJK Ideograph */
  ["\u675D", "\u675D"], /* CJK Ideograph */
  ["\u675E", "\u675E"], /* CJK Ideograph */
  ["\u675F", "\u675F"], /* CJK Ideograph */
  ["\u6760", "\u6760"], /* CJK Ideograph */
  ["\u6761", "\u6761"], /* CJK Ideograph */
  ["\u6762", "\u6762"], /* CJK Ideograph */
  ["\u6763", "\u6763"], /* CJK Ideograph */
  ["\u6764", "\u6764"], /* CJK Ideograph */
  ["\u6765", "\u6765"], /* CJK Ideograph */
  ["\u6766", "\u6766"], /* CJK Ideograph */
  ["\u6767", "\u6767"], /* CJK Ideograph */
  ["\u6768", "\u6768"], /* CJK Ideograph */
  ["\u6769", "\u6769"], /* CJK Ideograph */
  ["\u676A", "\u676A"], /* CJK Ideograph */
  ["\u676B", "\u676B"], /* CJK Ideograph */
  ["\u676C", "\u676C"], /* CJK Ideograph */
  ["\u676D", "\u676D"], /* CJK Ideograph */
  ["\u676E", "\u676E"], /* CJK Ideograph */
  ["\u676F", "\u676F"], /* CJK Ideograph */
  ["\u6770", "\u6770"], /* CJK Ideograph */
  ["\u6771", "\u6771"], /* CJK Ideograph */
  ["\u6772", "\u6772"], /* CJK Ideograph */
  ["\u6773", "\u6773"], /* CJK Ideograph */
  ["\u6774", "\u6774"], /* CJK Ideograph */
  ["\u6775", "\u6775"], /* CJK Ideograph */
  ["\u6776", "\u6776"], /* CJK Ideograph */
  ["\u6777", "\u6777"], /* CJK Ideograph */
  ["\u6778", "\u6778"], /* CJK Ideograph */
  ["\u6779", "\u6779"], /* CJK Ideograph */
  ["\u677A", "\u677A"], /* CJK Ideograph */
  ["\u677B", "\u677B"], /* CJK Ideograph */
  ["\u677C", "\u677C"], /* CJK Ideograph */
  ["\u677D", "\u677D"], /* CJK Ideograph */
  ["\u677E", "\u677E"], /* CJK Ideograph */
  ["\u677F", "\u677F"], /* CJK Ideograph */
  ["\u6780", "\u6780"], /* CJK Ideograph */
  ["\u6781", "\u6781"], /* CJK Ideograph */
  ["\u6782", "\u6782"], /* CJK Ideograph */
  ["\u6783", "\u6783"], /* CJK Ideograph */
  ["\u6784", "\u6784"], /* CJK Ideograph */
  ["\u6785", "\u6785"], /* CJK Ideograph */
  ["\u6786", "\u6786"], /* CJK Ideograph */
  ["\u6787", "\u6787"], /* CJK Ideograph */
  ["\u6788", "\u6788"], /* CJK Ideograph */
  ["\u6789", "\u6789"], /* CJK Ideograph */
  ["\u678A", "\u678A"], /* CJK Ideograph */
  ["\u678B", "\u678B"], /* CJK Ideograph */
  ["\u678C", "\u678C"], /* CJK Ideograph */
  ["\u678D", "\u678D"], /* CJK Ideograph */
  ["\u678E", "\u678E"], /* CJK Ideograph */
  ["\u678F", "\u678F"], /* CJK Ideograph */
  ["\u6790", "\u6790"], /* CJK Ideograph */
  ["\u6791", "\u6791"], /* CJK Ideograph */
  ["\u6792", "\u6792"], /* CJK Ideograph */
  ["\u6793", "\u6793"], /* CJK Ideograph */
  ["\u6794", "\u6794"], /* CJK Ideograph */
  ["\u6795", "\u6795"], /* CJK Ideograph */
  ["\u6796", "\u6796"], /* CJK Ideograph */
  ["\u6797", "\u6797"], /* CJK Ideograph */
  ["\u6798", "\u6798"], /* CJK Ideograph */
  ["\u6799", "\u6799"], /* CJK Ideograph */
  ["\u679A", "\u679A"], /* CJK Ideograph */
  ["\u679B", "\u679B"], /* CJK Ideograph */
  ["\u679C", "\u679C"], /* CJK Ideograph */
  ["\u679D", "\u679D"], /* CJK Ideograph */
  ["\u679E", "\u679E"], /* CJK Ideograph */
  ["\u679F", "\u679F"], /* CJK Ideograph */
  ["\u67A0", "\u67A0"], /* CJK Ideograph */
  ["\u67A1", "\u67A1"], /* CJK Ideograph */
  ["\u67A2", "\u67A2"], /* CJK Ideograph */
  ["\u67A3", "\u67A3"], /* CJK Ideograph */
  ["\u67A4", "\u67A4"], /* CJK Ideograph */
  ["\u67A5", "\u67A5"], /* CJK Ideograph */
  ["\u67A6", "\u67A6"], /* CJK Ideograph */
  ["\u67A7", "\u67A7"], /* CJK Ideograph */
  ["\u67A8", "\u67A8"], /* CJK Ideograph */
  ["\u67A9", "\u67A9"], /* CJK Ideograph */
  ["\u67AA", "\u67AA"], /* CJK Ideograph */
  ["\u67AB", "\u67AB"], /* CJK Ideograph */
  ["\u67AC", "\u67AC"], /* CJK Ideograph */
  ["\u67AD", "\u67AD"], /* CJK Ideograph */
  ["\u67AE", "\u67AE"], /* CJK Ideograph */
  ["\u67AF", "\u67AF"], /* CJK Ideograph */
  ["\u67B0", "\u67B0"], /* CJK Ideograph */
  ["\u67B1", "\u67B1"], /* CJK Ideograph */
  ["\u67B2", "\u67B2"], /* CJK Ideograph */
  ["\u67B3", "\u67B3"], /* CJK Ideograph */
  ["\u67B4", "\u67B4"], /* CJK Ideograph */
  ["\u67B5", "\u67B5"], /* CJK Ideograph */
  ["\u67B6", "\u67B6"], /* CJK Ideograph */
  ["\u67B7", "\u67B7"], /* CJK Ideograph */
  ["\u67B8", "\u67B8"], /* CJK Ideograph */
  ["\u67B9", "\u67B9"], /* CJK Ideograph */
  ["\u67BA", "\u67BA"], /* CJK Ideograph */
  ["\u67BB", "\u67BB"], /* CJK Ideograph */
  ["\u67BC", "\u67BC"], /* CJK Ideograph */
  ["\u67BD", "\u67BD"], /* CJK Ideograph */
  ["\u67BE", "\u67BE"], /* CJK Ideograph */
  ["\u67BF", "\u67BF"], /* CJK Ideograph */
  ["\u67C0", "\u67C0"], /* CJK Ideograph */
  ["\u67C1", "\u67C1"], /* CJK Ideograph */
  ["\u67C2", "\u67C2"], /* CJK Ideograph */
  ["\u67C3", "\u67C3"], /* CJK Ideograph */
  ["\u67C4", "\u67C4"], /* CJK Ideograph */
  ["\u67C5", "\u67C5"], /* CJK Ideograph */
  ["\u67C6", "\u67C6"], /* CJK Ideograph */
  ["\u67C7", "\u67C7"], /* CJK Ideograph */
  ["\u67C8", "\u67C8"], /* CJK Ideograph */
  ["\u67C9", "\u67C9"], /* CJK Ideograph */
  ["\u67CA", "\u67CA"], /* CJK Ideograph */
  ["\u67CB", "\u67CB"], /* CJK Ideograph */
  ["\u67CC", "\u67CC"], /* CJK Ideograph */
  ["\u67CD", "\u67CD"], /* CJK Ideograph */
  ["\u67CE", "\u67CE"], /* CJK Ideograph */
  ["\u67CF", "\u67CF"], /* CJK Ideograph */
  ["\u67D0", "\u67D0"], /* CJK Ideograph */
  ["\u67D1", "\u67D1"], /* CJK Ideograph */
  ["\u67D2", "\u67D2"], /* CJK Ideograph */
  ["\u67D3", "\u67D3"], /* CJK Ideograph */
  ["\u67D4", "\u67D4"], /* CJK Ideograph */
  ["\u67D5", "\u67D5"], /* CJK Ideograph */
  ["\u67D6", "\u67D6"], /* CJK Ideograph */
  ["\u67D7", "\u67D7"], /* CJK Ideograph */
  ["\u67D8", "\u67D8"], /* CJK Ideograph */
  ["\u67D9", "\u67D9"], /* CJK Ideograph */
  ["\u67DA", "\u67DA"], /* CJK Ideograph */
  ["\u67DB", "\u67DB"], /* CJK Ideograph */
  ["\u67DC", "\u67DC"], /* CJK Ideograph */
  ["\u67DD", "\u67DD"], /* CJK Ideograph */
  ["\u67DE", "\u67DE"], /* CJK Ideograph */
  ["\u67DF", "\u67DF"], /* CJK Ideograph */
  ["\u67E0", "\u67E0"], /* CJK Ideograph */
  ["\u67E1", "\u67E1"], /* CJK Ideograph */
  ["\u67E2", "\u67E2"], /* CJK Ideograph */
  ["\u67E3", "\u67E3"], /* CJK Ideograph */
  ["\u67E4", "\u67E4"], /* CJK Ideograph */
  ["\u67E5", "\u67E5"], /* CJK Ideograph */
  ["\u67E6", "\u67E6"], /* CJK Ideograph */
  ["\u67E7", "\u67E7"], /* CJK Ideograph */
  ["\u67E8", "\u67E8"], /* CJK Ideograph */
  ["\u67E9", "\u67E9"], /* CJK Ideograph */
  ["\u67EA", "\u67EA"], /* CJK Ideograph */
  ["\u67EB", "\u67EB"], /* CJK Ideograph */
  ["\u67EC", "\u67EC"], /* CJK Ideograph */
  ["\u67ED", "\u67ED"], /* CJK Ideograph */
  ["\u67EE", "\u67EE"], /* CJK Ideograph */
  ["\u67EF", "\u67EF"], /* CJK Ideograph */
  ["\u67F0", "\u67F0"], /* CJK Ideograph */
  ["\u67F1", "\u67F1"], /* CJK Ideograph */
  ["\u67F2", "\u67F2"], /* CJK Ideograph */
  ["\u67F3", "\u67F3"], /* CJK Ideograph */
  ["\u67F4", "\u67F4"], /* CJK Ideograph */
  ["\u67F5", "\u67F5"], /* CJK Ideograph */
  ["\u67F6", "\u67F6"], /* CJK Ideograph */
  ["\u67F7", "\u67F7"], /* CJK Ideograph */
  ["\u67F8", "\u67F8"], /* CJK Ideograph */
  ["\u67F9", "\u67F9"], /* CJK Ideograph */
  ["\u67FA", "\u67FA"], /* CJK Ideograph */
  ["\u67FB", "\u67FB"], /* CJK Ideograph */
  ["\u67FC", "\u67FC"], /* CJK Ideograph */
  ["\u67FD", "\u67FD"], /* CJK Ideograph */
  ["\u67FE", "\u67FE"], /* CJK Ideograph */
  ["\u67FF", "\u67FF"], /* CJK Ideograph */
  ["\u6800", "\u6800"], /* CJK Ideograph */
  ["\u6801", "\u6801"], /* CJK Ideograph */
  ["\u6802", "\u6802"], /* CJK Ideograph */
  ["\u6803", "\u6803"], /* CJK Ideograph */
  ["\u6804", "\u6804"], /* CJK Ideograph */
  ["\u6805", "\u6805"], /* CJK Ideograph */
  ["\u6806", "\u6806"], /* CJK Ideograph */
  ["\u6807", "\u6807"], /* CJK Ideograph */
  ["\u6808", "\u6808"], /* CJK Ideograph */
  ["\u6809", "\u6809"], /* CJK Ideograph */
  ["\u680A", "\u680A"], /* CJK Ideograph */
  ["\u680B", "\u680B"], /* CJK Ideograph */
  ["\u680C", "\u680C"], /* CJK Ideograph */
  ["\u680D", "\u680D"], /* CJK Ideograph */
  ["\u680E", "\u680E"], /* CJK Ideograph */
  ["\u680F", "\u680F"], /* CJK Ideograph */
  ["\u6810", "\u6810"], /* CJK Ideograph */
  ["\u6811", "\u6811"], /* CJK Ideograph */
  ["\u6812", "\u6812"], /* CJK Ideograph */
  ["\u6813", "\u6813"], /* CJK Ideograph */
  ["\u6814", "\u6814"], /* CJK Ideograph */
  ["\u6815", "\u6815"], /* CJK Ideograph */
  ["\u6816", "\u6816"], /* CJK Ideograph */
  ["\u6817", "\u6817"], /* CJK Ideograph */
  ["\u6818", "\u6818"], /* CJK Ideograph */
  ["\u6819", "\u6819"], /* CJK Ideograph */
  ["\u681A", "\u681A"], /* CJK Ideograph */
  ["\u681B", "\u681B"], /* CJK Ideograph */
  ["\u681C", "\u681C"], /* CJK Ideograph */
  ["\u681D", "\u681D"], /* CJK Ideograph */
  ["\u681E", "\u681E"], /* CJK Ideograph */
  ["\u681F", "\u681F"], /* CJK Ideograph */
  ["\u6820", "\u6820"], /* CJK Ideograph */
  ["\u6821", "\u6821"], /* CJK Ideograph */
  ["\u6822", "\u6822"], /* CJK Ideograph */
  ["\u6823", "\u6823"], /* CJK Ideograph */
  ["\u6824", "\u6824"], /* CJK Ideograph */
  ["\u6825", "\u6825"], /* CJK Ideograph */
  ["\u6826", "\u6826"], /* CJK Ideograph */
  ["\u6827", "\u6827"], /* CJK Ideograph */
  ["\u6828", "\u6828"], /* CJK Ideograph */
  ["\u6829", "\u6829"], /* CJK Ideograph */
  ["\u682A", "\u682A"], /* CJK Ideograph */
  ["\u682B", "\u682B"], /* CJK Ideograph */
  ["\u682C", "\u682C"], /* CJK Ideograph */
  ["\u682D", "\u682D"], /* CJK Ideograph */
  ["\u682E", "\u682E"], /* CJK Ideograph */
  ["\u682F", "\u682F"], /* CJK Ideograph */
  ["\u6830", "\u6830"], /* CJK Ideograph */
  ["\u6831", "\u6831"], /* CJK Ideograph */
  ["\u6832", "\u6832"], /* CJK Ideograph */
  ["\u6833", "\u6833"], /* CJK Ideograph */
  ["\u6834", "\u6834"], /* CJK Ideograph */
  ["\u6835", "\u6835"], /* CJK Ideograph */
  ["\u6836", "\u6836"], /* CJK Ideograph */
  ["\u6837", "\u6837"], /* CJK Ideograph */
  ["\u6838", "\u6838"], /* CJK Ideograph */
  ["\u6839", "\u6839"], /* CJK Ideograph */
  ["\u683A", "\u683A"], /* CJK Ideograph */
  ["\u683B", "\u683B"], /* CJK Ideograph */
  ["\u683C", "\u683C"], /* CJK Ideograph */
  ["\u683D", "\u683D"], /* CJK Ideograph */
  ["\u683E", "\u683E"], /* CJK Ideograph */
  ["\u683F", "\u683F"], /* CJK Ideograph */
  ["\u6840", "\u6840"], /* CJK Ideograph */
  ["\u6841", "\u6841"], /* CJK Ideograph */
  ["\u6842", "\u6842"], /* CJK Ideograph */
  ["\u6843", "\u6843"], /* CJK Ideograph */
  ["\u6844", "\u6844"], /* CJK Ideograph */
  ["\u6845", "\u6845"], /* CJK Ideograph */
  ["\u6846", "\u6846"], /* CJK Ideograph */
  ["\u6847", "\u6847"], /* CJK Ideograph */
  ["\u6848", "\u6848"], /* CJK Ideograph */
  ["\u6849", "\u6849"], /* CJK Ideograph */
  ["\u684A", "\u684A"], /* CJK Ideograph */
  ["\u684B", "\u684B"], /* CJK Ideograph */
  ["\u684C", "\u684C"], /* CJK Ideograph */
  ["\u684D", "\u684D"], /* CJK Ideograph */
  ["\u684E", "\u684E"], /* CJK Ideograph */
  ["\u684F", "\u684F"], /* CJK Ideograph */
  ["\u6850", "\u6850"], /* CJK Ideograph */
  ["\u6851", "\u6851"], /* CJK Ideograph */
  ["\u6852", "\u6852"], /* CJK Ideograph */
  ["\u6853", "\u6853"], /* CJK Ideograph */
  ["\u6854", "\u6854"], /* CJK Ideograph */
  ["\u6855", "\u6855"], /* CJK Ideograph */
  ["\u6856", "\u6856"], /* CJK Ideograph */
  ["\u6857", "\u6857"], /* CJK Ideograph */
  ["\u6858", "\u6858"], /* CJK Ideograph */
  ["\u6859", "\u6859"], /* CJK Ideograph */
  ["\u685A", "\u685A"], /* CJK Ideograph */
  ["\u685B", "\u685B"], /* CJK Ideograph */
  ["\u685C", "\u685C"], /* CJK Ideograph */
  ["\u685D", "\u685D"], /* CJK Ideograph */
  ["\u685E", "\u685E"], /* CJK Ideograph */
  ["\u685F", "\u685F"], /* CJK Ideograph */
  ["\u6860", "\u6860"], /* CJK Ideograph */
  ["\u6861", "\u6861"], /* CJK Ideograph */
  ["\u6862", "\u6862"], /* CJK Ideograph */
  ["\u6863", "\u6863"], /* CJK Ideograph */
  ["\u6864", "\u6864"], /* CJK Ideograph */
  ["\u6865", "\u6865"], /* CJK Ideograph */
  ["\u6866", "\u6866"], /* CJK Ideograph */
  ["\u6867", "\u6867"], /* CJK Ideograph */
  ["\u6868", "\u6868"], /* CJK Ideograph */
  ["\u6869", "\u6869"], /* CJK Ideograph */
  ["\u686A", "\u686A"], /* CJK Ideograph */
  ["\u686B", "\u686B"], /* CJK Ideograph */
  ["\u686C", "\u686C"], /* CJK Ideograph */
  ["\u686D", "\u686D"], /* CJK Ideograph */
  ["\u686E", "\u686E"], /* CJK Ideograph */
  ["\u686F", "\u686F"], /* CJK Ideograph */
  ["\u6870", "\u6870"], /* CJK Ideograph */
  ["\u6871", "\u6871"], /* CJK Ideograph */
  ["\u6872", "\u6872"], /* CJK Ideograph */
  ["\u6873", "\u6873"], /* CJK Ideograph */
  ["\u6874", "\u6874"], /* CJK Ideograph */
  ["\u6875", "\u6875"], /* CJK Ideograph */
  ["\u6876", "\u6876"], /* CJK Ideograph */
  ["\u6877", "\u6877"], /* CJK Ideograph */
  ["\u6878", "\u6878"], /* CJK Ideograph */
  ["\u6879", "\u6879"], /* CJK Ideograph */
  ["\u687A", "\u687A"], /* CJK Ideograph */
  ["\u687B", "\u687B"], /* CJK Ideograph */
  ["\u687C", "\u687C"], /* CJK Ideograph */
  ["\u687D", "\u687D"], /* CJK Ideograph */
  ["\u687E", "\u687E"], /* CJK Ideograph */
  ["\u687F", "\u687F"], /* CJK Ideograph */
  ["\u6880", "\u6880"], /* CJK Ideograph */
  ["\u6881", "\u6881"], /* CJK Ideograph */
  ["\u6882", "\u6882"], /* CJK Ideograph */
  ["\u6883", "\u6883"], /* CJK Ideograph */
  ["\u6884", "\u6884"], /* CJK Ideograph */
  ["\u6885", "\u6885"], /* CJK Ideograph */
  ["\u6886", "\u6886"], /* CJK Ideograph */
  ["\u6887", "\u6887"], /* CJK Ideograph */
  ["\u6888", "\u6888"], /* CJK Ideograph */
  ["\u6889", "\u6889"], /* CJK Ideograph */
  ["\u688A", "\u688A"], /* CJK Ideograph */
  ["\u688B", "\u688B"], /* CJK Ideograph */
  ["\u688C", "\u688C"], /* CJK Ideograph */
  ["\u688D", "\u688D"], /* CJK Ideograph */
  ["\u688E", "\u688E"], /* CJK Ideograph */
  ["\u688F", "\u688F"], /* CJK Ideograph */
  ["\u6890", "\u6890"], /* CJK Ideograph */
  ["\u6891", "\u6891"], /* CJK Ideograph */
  ["\u6892", "\u6892"], /* CJK Ideograph */
  ["\u6893", "\u6893"], /* CJK Ideograph */
  ["\u6894", "\u6894"], /* CJK Ideograph */
  ["\u6895", "\u6895"], /* CJK Ideograph */
  ["\u6896", "\u6896"], /* CJK Ideograph */
  ["\u6897", "\u6897"], /* CJK Ideograph */
  ["\u6898", "\u6898"], /* CJK Ideograph */
  ["\u6899", "\u6899"], /* CJK Ideograph */
  ["\u689A", "\u689A"], /* CJK Ideograph */
  ["\u689B", "\u689B"], /* CJK Ideograph */
  ["\u689C", "\u689C"], /* CJK Ideograph */
  ["\u689D", "\u689D"], /* CJK Ideograph */
  ["\u689E", "\u689E"], /* CJK Ideograph */
  ["\u689F", "\u689F"], /* CJK Ideograph */
  ["\u68A0", "\u68A0"], /* CJK Ideograph */
  ["\u68A1", "\u68A1"], /* CJK Ideograph */
  ["\u68A2", "\u68A2"], /* CJK Ideograph */
  ["\u68A3", "\u68A3"], /* CJK Ideograph */
  ["\u68A4", "\u68A4"], /* CJK Ideograph */
  ["\u68A5", "\u68A5"], /* CJK Ideograph */
  ["\u68A6", "\u68A6"], /* CJK Ideograph */
  ["\u68A7", "\u68A7"], /* CJK Ideograph */
  ["\u68A8", "\u68A8"], /* CJK Ideograph */
  ["\u68A9", "\u68A9"], /* CJK Ideograph */
  ["\u68AA", "\u68AA"], /* CJK Ideograph */
  ["\u68AB", "\u68AB"], /* CJK Ideograph */
  ["\u68AC", "\u68AC"], /* CJK Ideograph */
  ["\u68AD", "\u68AD"], /* CJK Ideograph */
  ["\u68AE", "\u68AE"], /* CJK Ideograph */
  ["\u68AF", "\u68AF"], /* CJK Ideograph */
  ["\u68B0", "\u68B0"], /* CJK Ideograph */
  ["\u68B1", "\u68B1"], /* CJK Ideograph */
  ["\u68B2", "\u68B2"], /* CJK Ideograph */
  ["\u68B3", "\u68B3"], /* CJK Ideograph */
  ["\u68B4", "\u68B4"], /* CJK Ideograph */
  ["\u68B5", "\u68B5"], /* CJK Ideograph */
  ["\u68B6", "\u68B6"], /* CJK Ideograph */
  ["\u68B7", "\u68B7"], /* CJK Ideograph */
  ["\u68B8", "\u68B8"], /* CJK Ideograph */
  ["\u68B9", "\u68B9"], /* CJK Ideograph */
  ["\u68BA", "\u68BA"], /* CJK Ideograph */
  ["\u68BB", "\u68BB"], /* CJK Ideograph */
  ["\u68BC", "\u68BC"], /* CJK Ideograph */
  ["\u68BD", "\u68BD"], /* CJK Ideograph */
  ["\u68BE", "\u68BE"], /* CJK Ideograph */
  ["\u68BF", "\u68BF"], /* CJK Ideograph */
  ["\u68C0", "\u68C0"], /* CJK Ideograph */
  ["\u68C1", "\u68C1"], /* CJK Ideograph */
  ["\u68C2", "\u68C2"], /* CJK Ideograph */
  ["\u68C3", "\u68C3"], /* CJK Ideograph */
  ["\u68C4", "\u68C4"], /* CJK Ideograph */
  ["\u68C5", "\u68C5"], /* CJK Ideograph */
  ["\u68C6", "\u68C6"], /* CJK Ideograph */
  ["\u68C7", "\u68C7"], /* CJK Ideograph */
  ["\u68C8", "\u68C8"], /* CJK Ideograph */
  ["\u68C9", "\u68C9"], /* CJK Ideograph */
  ["\u68CA", "\u68CA"], /* CJK Ideograph */
  ["\u68CB", "\u68CB"], /* CJK Ideograph */
  ["\u68CC", "\u68CC"], /* CJK Ideograph */
  ["\u68CD", "\u68CD"], /* CJK Ideograph */
  ["\u68CE", "\u68CE"], /* CJK Ideograph */
  ["\u68CF", "\u68CF"], /* CJK Ideograph */
  ["\u68D0", "\u68D0"], /* CJK Ideograph */
  ["\u68D1", "\u68D1"], /* CJK Ideograph */
  ["\u68D2", "\u68D2"], /* CJK Ideograph */
  ["\u68D3", "\u68D3"], /* CJK Ideograph */
  ["\u68D4", "\u68D4"], /* CJK Ideograph */
  ["\u68D5", "\u68D5"], /* CJK Ideograph */
  ["\u68D6", "\u68D6"], /* CJK Ideograph */
  ["\u68D7", "\u68D7"], /* CJK Ideograph */
  ["\u68D8", "\u68D8"], /* CJK Ideograph */
  ["\u68D9", "\u68D9"], /* CJK Ideograph */
  ["\u68DA", "\u68DA"], /* CJK Ideograph */
  ["\u68DB", "\u68DB"], /* CJK Ideograph */
  ["\u68DC", "\u68DC"], /* CJK Ideograph */
  ["\u68DD", "\u68DD"], /* CJK Ideograph */
  ["\u68DE", "\u68DE"], /* CJK Ideograph */
  ["\u68DF", "\u68DF"], /* CJK Ideograph */
  ["\u68E0", "\u68E0"], /* CJK Ideograph */
  ["\u68E1", "\u68E1"], /* CJK Ideograph */
  ["\u68E2", "\u68E2"], /* CJK Ideograph */
  ["\u68E3", "\u68E3"], /* CJK Ideograph */
  ["\u68E4", "\u68E4"], /* CJK Ideograph */
  ["\u68E5", "\u68E5"], /* CJK Ideograph */
  ["\u68E6", "\u68E6"], /* CJK Ideograph */
  ["\u68E7", "\u68E7"], /* CJK Ideograph */
  ["\u68E8", "\u68E8"], /* CJK Ideograph */
  ["\u68E9", "\u68E9"], /* CJK Ideograph */
  ["\u68EA", "\u68EA"], /* CJK Ideograph */
  ["\u68EB", "\u68EB"], /* CJK Ideograph */
  ["\u68EC", "\u68EC"], /* CJK Ideograph */
  ["\u68ED", "\u68ED"], /* CJK Ideograph */
  ["\u68EE", "\u68EE"], /* CJK Ideograph */
  ["\u68EF", "\u68EF"], /* CJK Ideograph */
  ["\u68F0", "\u68F0"], /* CJK Ideograph */
  ["\u68F1", "\u68F1"], /* CJK Ideograph */
  ["\u68F2", "\u68F2"], /* CJK Ideograph */
  ["\u68F3", "\u68F3"], /* CJK Ideograph */
  ["\u68F4", "\u68F4"], /* CJK Ideograph */
  ["\u68F5", "\u68F5"], /* CJK Ideograph */
  ["\u68F6", "\u68F6"], /* CJK Ideograph */
  ["\u68F7", "\u68F7"], /* CJK Ideograph */
  ["\u68F8", "\u68F8"], /* CJK Ideograph */
  ["\u68F9", "\u68F9"], /* CJK Ideograph */
  ["\u68FA", "\u68FA"], /* CJK Ideograph */
  ["\u68FB", "\u68FB"], /* CJK Ideograph */
  ["\u68FC", "\u68FC"], /* CJK Ideograph */
  ["\u68FD", "\u68FD"], /* CJK Ideograph */
  ["\u68FE", "\u68FE"], /* CJK Ideograph */
  ["\u68FF", "\u68FF"], /* CJK Ideograph */
  ["\u6900", "\u6900"], /* CJK Ideograph */
  ["\u6901", "\u6901"], /* CJK Ideograph */
  ["\u6902", "\u6902"], /* CJK Ideograph */
  ["\u6903", "\u6903"], /* CJK Ideograph */
  ["\u6904", "\u6904"], /* CJK Ideograph */
  ["\u6905", "\u6905"], /* CJK Ideograph */
  ["\u6906", "\u6906"], /* CJK Ideograph */
  ["\u6907", "\u6907"], /* CJK Ideograph */
  ["\u6908", "\u6908"], /* CJK Ideograph */
  ["\u6909", "\u6909"], /* CJK Ideograph */
  ["\u690A", "\u690A"], /* CJK Ideograph */
  ["\u690B", "\u690B"], /* CJK Ideograph */
  ["\u690C", "\u690C"], /* CJK Ideograph */
  ["\u690D", "\u690D"], /* CJK Ideograph */
  ["\u690E", "\u690E"], /* CJK Ideograph */
  ["\u690F", "\u690F"], /* CJK Ideograph */
  ["\u6910", "\u6910"], /* CJK Ideograph */
  ["\u6911", "\u6911"], /* CJK Ideograph */
  ["\u6912", "\u6912"], /* CJK Ideograph */
  ["\u6913", "\u6913"], /* CJK Ideograph */
  ["\u6914", "\u6914"], /* CJK Ideograph */
  ["\u6915", "\u6915"], /* CJK Ideograph */
  ["\u6916", "\u6916"], /* CJK Ideograph */
  ["\u6917", "\u6917"], /* CJK Ideograph */
  ["\u6918", "\u6918"], /* CJK Ideograph */
  ["\u6919", "\u6919"], /* CJK Ideograph */
  ["\u691A", "\u691A"], /* CJK Ideograph */
  ["\u691B", "\u691B"], /* CJK Ideograph */
  ["\u691C", "\u691C"], /* CJK Ideograph */
  ["\u691D", "\u691D"], /* CJK Ideograph */
  ["\u691E", "\u691E"], /* CJK Ideograph */
  ["\u691F", "\u691F"], /* CJK Ideograph */
  ["\u6920", "\u6920"], /* CJK Ideograph */
  ["\u6921", "\u6921"], /* CJK Ideograph */
  ["\u6922", "\u6922"], /* CJK Ideograph */
  ["\u6923", "\u6923"], /* CJK Ideograph */
  ["\u6924", "\u6924"], /* CJK Ideograph */
  ["\u6925", "\u6925"], /* CJK Ideograph */
  ["\u6926", "\u6926"], /* CJK Ideograph */
  ["\u6927", "\u6927"], /* CJK Ideograph */
  ["\u6928", "\u6928"], /* CJK Ideograph */
  ["\u6929", "\u6929"], /* CJK Ideograph */
  ["\u692A", "\u692A"], /* CJK Ideograph */
  ["\u692B", "\u692B"], /* CJK Ideograph */
  ["\u692C", "\u692C"], /* CJK Ideograph */
  ["\u692D", "\u692D"], /* CJK Ideograph */
  ["\u692E", "\u692E"], /* CJK Ideograph */
  ["\u692F", "\u692F"], /* CJK Ideograph */
  ["\u6930", "\u6930"], /* CJK Ideograph */
  ["\u6931", "\u6931"], /* CJK Ideograph */
  ["\u6932", "\u6932"], /* CJK Ideograph */
  ["\u6933", "\u6933"], /* CJK Ideograph */
  ["\u6934", "\u6934"], /* CJK Ideograph */
  ["\u6935", "\u6935"], /* CJK Ideograph */
  ["\u6936", "\u6936"], /* CJK Ideograph */
  ["\u6937", "\u6937"], /* CJK Ideograph */
  ["\u6938", "\u6938"], /* CJK Ideograph */
  ["\u6939", "\u6939"], /* CJK Ideograph */
  ["\u693A", "\u693A"], /* CJK Ideograph */
  ["\u693B", "\u693B"], /* CJK Ideograph */
  ["\u693C", "\u693C"], /* CJK Ideograph */
  ["\u693D", "\u693D"], /* CJK Ideograph */
  ["\u693E", "\u693E"], /* CJK Ideograph */
  ["\u693F", "\u693F"], /* CJK Ideograph */
  ["\u6940", "\u6940"], /* CJK Ideograph */
  ["\u6941", "\u6941"], /* CJK Ideograph */
  ["\u6942", "\u6942"], /* CJK Ideograph */
  ["\u6943", "\u6943"], /* CJK Ideograph */
  ["\u6944", "\u6944"], /* CJK Ideograph */
  ["\u6945", "\u6945"], /* CJK Ideograph */
  ["\u6946", "\u6946"], /* CJK Ideograph */
  ["\u6947", "\u6947"], /* CJK Ideograph */
  ["\u6948", "\u6948"], /* CJK Ideograph */
  ["\u6949", "\u6949"], /* CJK Ideograph */
  ["\u694A", "\u694A"], /* CJK Ideograph */
  ["\u694B", "\u694B"], /* CJK Ideograph */
  ["\u694C", "\u694C"], /* CJK Ideograph */
  ["\u694D", "\u694D"], /* CJK Ideograph */
  ["\u694E", "\u694E"], /* CJK Ideograph */
  ["\u694F", "\u694F"], /* CJK Ideograph */
  ["\u6950", "\u6950"], /* CJK Ideograph */
  ["\u6951", "\u6951"], /* CJK Ideograph */
  ["\u6952", "\u6952"], /* CJK Ideograph */
  ["\u6953", "\u6953"], /* CJK Ideograph */
  ["\u6954", "\u6954"], /* CJK Ideograph */
  ["\u6955", "\u6955"], /* CJK Ideograph */
  ["\u6956", "\u6956"], /* CJK Ideograph */
  ["\u6957", "\u6957"], /* CJK Ideograph */
  ["\u6958", "\u6958"], /* CJK Ideograph */
  ["\u6959", "\u6959"], /* CJK Ideograph */
  ["\u695A", "\u695A"], /* CJK Ideograph */
  ["\u695B", "\u695B"], /* CJK Ideograph */
  ["\u695C", "\u695C"], /* CJK Ideograph */
  ["\u695D", "\u695D"], /* CJK Ideograph */
  ["\u695E", "\u695E"], /* CJK Ideograph */
  ["\u695F", "\u695F"], /* CJK Ideograph */
  ["\u6960", "\u6960"], /* CJK Ideograph */
  ["\u6961", "\u6961"], /* CJK Ideograph */
  ["\u6962", "\u6962"], /* CJK Ideograph */
  ["\u6963", "\u6963"], /* CJK Ideograph */
  ["\u6964", "\u6964"], /* CJK Ideograph */
  ["\u6965", "\u6965"], /* CJK Ideograph */
  ["\u6966", "\u6966"], /* CJK Ideograph */
  ["\u6967", "\u6967"], /* CJK Ideograph */
  ["\u6968", "\u6968"], /* CJK Ideograph */
  ["\u6969", "\u6969"], /* CJK Ideograph */
  ["\u696A", "\u696A"], /* CJK Ideograph */
  ["\u696B", "\u696B"], /* CJK Ideograph */
  ["\u696C", "\u696C"], /* CJK Ideograph */
  ["\u696D", "\u696D"], /* CJK Ideograph */
  ["\u696E", "\u696E"], /* CJK Ideograph */
  ["\u696F", "\u696F"], /* CJK Ideograph */
  ["\u6970", "\u6970"], /* CJK Ideograph */
  ["\u6971", "\u6971"], /* CJK Ideograph */
  ["\u6972", "\u6972"], /* CJK Ideograph */
  ["\u6973", "\u6973"], /* CJK Ideograph */
  ["\u6974", "\u6974"], /* CJK Ideograph */
  ["\u6975", "\u6975"], /* CJK Ideograph */
  ["\u6976", "\u6976"], /* CJK Ideograph */
  ["\u6977", "\u6977"], /* CJK Ideograph */
  ["\u6978", "\u6978"], /* CJK Ideograph */
  ["\u6979", "\u6979"], /* CJK Ideograph */
  ["\u697A", "\u697A"], /* CJK Ideograph */
  ["\u697B", "\u697B"], /* CJK Ideograph */
  ["\u697C", "\u697C"], /* CJK Ideograph */
  ["\u697D", "\u697D"], /* CJK Ideograph */
  ["\u697E", "\u697E"], /* CJK Ideograph */
  ["\u697F", "\u697F"], /* CJK Ideograph */
  ["\u6980", "\u6980"], /* CJK Ideograph */
  ["\u6981", "\u6981"], /* CJK Ideograph */
  ["\u6982", "\u6982"], /* CJK Ideograph */
  ["\u6983", "\u6983"], /* CJK Ideograph */
  ["\u6984", "\u6984"], /* CJK Ideograph */
  ["\u6985", "\u6985"], /* CJK Ideograph */
  ["\u6986", "\u6986"], /* CJK Ideograph */
  ["\u6987", "\u6987"], /* CJK Ideograph */
  ["\u6988", "\u6988"], /* CJK Ideograph */
  ["\u6989", "\u6989"], /* CJK Ideograph */
  ["\u698A", "\u698A"], /* CJK Ideograph */
  ["\u698B", "\u698B"], /* CJK Ideograph */
  ["\u698C", "\u698C"], /* CJK Ideograph */
  ["\u698D", "\u698D"], /* CJK Ideograph */
  ["\u698E", "\u698E"], /* CJK Ideograph */
  ["\u698F", "\u698F"], /* CJK Ideograph */
  ["\u6990", "\u6990"], /* CJK Ideograph */
  ["\u6991", "\u6991"], /* CJK Ideograph */
  ["\u6992", "\u6992"], /* CJK Ideograph */
  ["\u6993", "\u6993"], /* CJK Ideograph */
  ["\u6994", "\u6994"], /* CJK Ideograph */
  ["\u6995", "\u6995"], /* CJK Ideograph */
  ["\u6996", "\u6996"], /* CJK Ideograph */
  ["\u6997", "\u6997"], /* CJK Ideograph */
  ["\u6998", "\u6998"], /* CJK Ideograph */
  ["\u6999", "\u6999"], /* CJK Ideograph */
  ["\u699A", "\u699A"], /* CJK Ideograph */
  ["\u699B", "\u699B"], /* CJK Ideograph */
  ["\u699C", "\u699C"], /* CJK Ideograph */
  ["\u699D", "\u699D"], /* CJK Ideograph */
  ["\u699E", "\u699E"], /* CJK Ideograph */
  ["\u699F", "\u699F"], /* CJK Ideograph */
  ["\u69A0", "\u69A0"], /* CJK Ideograph */
  ["\u69A1", "\u69A1"], /* CJK Ideograph */
  ["\u69A2", "\u69A2"], /* CJK Ideograph */
  ["\u69A3", "\u69A3"], /* CJK Ideograph */
  ["\u69A4", "\u69A4"], /* CJK Ideograph */
  ["\u69A5", "\u69A5"], /* CJK Ideograph */
  ["\u69A6", "\u69A6"], /* CJK Ideograph */
  ["\u69A7", "\u69A7"], /* CJK Ideograph */
  ["\u69A8", "\u69A8"], /* CJK Ideograph */
  ["\u69A9", "\u69A9"], /* CJK Ideograph */
  ["\u69AA", "\u69AA"], /* CJK Ideograph */
  ["\u69AB", "\u69AB"], /* CJK Ideograph */
  ["\u69AC", "\u69AC"], /* CJK Ideograph */
  ["\u69AD", "\u69AD"], /* CJK Ideograph */
  ["\u69AE", "\u69AE"], /* CJK Ideograph */
  ["\u69AF", "\u69AF"], /* CJK Ideograph */
  ["\u69B0", "\u69B0"], /* CJK Ideograph */
  ["\u69B1", "\u69B1"], /* CJK Ideograph */
  ["\u69B2", "\u69B2"], /* CJK Ideograph */
  ["\u69B3", "\u69B3"], /* CJK Ideograph */
  ["\u69B4", "\u69B4"], /* CJK Ideograph */
  ["\u69B5", "\u69B5"], /* CJK Ideograph */
  ["\u69B6", "\u69B6"], /* CJK Ideograph */
  ["\u69B7", "\u69B7"], /* CJK Ideograph */
  ["\u69B8", "\u69B8"], /* CJK Ideograph */
  ["\u69B9", "\u69B9"], /* CJK Ideograph */
  ["\u69BA", "\u69BA"], /* CJK Ideograph */
  ["\u69BB", "\u69BB"], /* CJK Ideograph */
  ["\u69BC", "\u69BC"], /* CJK Ideograph */
  ["\u69BD", "\u69BD"], /* CJK Ideograph */
  ["\u69BE", "\u69BE"], /* CJK Ideograph */
  ["\u69BF", "\u69BF"], /* CJK Ideograph */
  ["\u69C0", "\u69C0"], /* CJK Ideograph */
  ["\u69C1", "\u69C1"], /* CJK Ideograph */
  ["\u69C2", "\u69C2"], /* CJK Ideograph */
  ["\u69C3", "\u69C3"], /* CJK Ideograph */
  ["\u69C4", "\u69C4"], /* CJK Ideograph */
  ["\u69C5", "\u69C5"], /* CJK Ideograph */
  ["\u69C6", "\u69C6"], /* CJK Ideograph */
  ["\u69C7", "\u69C7"], /* CJK Ideograph */
  ["\u69C8", "\u69C8"], /* CJK Ideograph */
  ["\u69C9", "\u69C9"], /* CJK Ideograph */
  ["\u69CA", "\u69CA"], /* CJK Ideograph */
  ["\u69CB", "\u69CB"], /* CJK Ideograph */
  ["\u69CC", "\u69CC"], /* CJK Ideograph */
  ["\u69CD", "\u69CD"], /* CJK Ideograph */
  ["\u69CE", "\u69CE"], /* CJK Ideograph */
  ["\u69CF", "\u69CF"], /* CJK Ideograph */
  ["\u69D0", "\u69D0"], /* CJK Ideograph */
  ["\u69D1", "\u69D1"], /* CJK Ideograph */
  ["\u69D2", "\u69D2"], /* CJK Ideograph */
  ["\u69D3", "\u69D3"], /* CJK Ideograph */
  ["\u69D4", "\u69D4"], /* CJK Ideograph */
  ["\u69D5", "\u69D5"], /* CJK Ideograph */
  ["\u69D6", "\u69D6"], /* CJK Ideograph */
  ["\u69D7", "\u69D7"], /* CJK Ideograph */
  ["\u69D8", "\u69D8"], /* CJK Ideograph */
  ["\u69D9", "\u69D9"], /* CJK Ideograph */
  ["\u69DA", "\u69DA"], /* CJK Ideograph */
  ["\u69DB", "\u69DB"], /* CJK Ideograph */
  ["\u69DC", "\u69DC"], /* CJK Ideograph */
  ["\u69DD", "\u69DD"], /* CJK Ideograph */
  ["\u69DE", "\u69DE"], /* CJK Ideograph */
  ["\u69DF", "\u69DF"], /* CJK Ideograph */
  ["\u69E0", "\u69E0"], /* CJK Ideograph */
  ["\u69E1", "\u69E1"], /* CJK Ideograph */
  ["\u69E2", "\u69E2"], /* CJK Ideograph */
  ["\u69E3", "\u69E3"], /* CJK Ideograph */
  ["\u69E4", "\u69E4"], /* CJK Ideograph */
  ["\u69E5", "\u69E5"], /* CJK Ideograph */
  ["\u69E6", "\u69E6"], /* CJK Ideograph */
  ["\u69E7", "\u69E7"], /* CJK Ideograph */
  ["\u69E8", "\u69E8"], /* CJK Ideograph */
  ["\u69E9", "\u69E9"], /* CJK Ideograph */
  ["\u69EA", "\u69EA"], /* CJK Ideograph */
  ["\u69EB", "\u69EB"], /* CJK Ideograph */
  ["\u69EC", "\u69EC"], /* CJK Ideograph */
  ["\u69ED", "\u69ED"], /* CJK Ideograph */
  ["\u69EE", "\u69EE"], /* CJK Ideograph */
  ["\u69EF", "\u69EF"], /* CJK Ideograph */
  ["\u69F0", "\u69F0"], /* CJK Ideograph */
  ["\u69F1", "\u69F1"], /* CJK Ideograph */
  ["\u69F2", "\u69F2"], /* CJK Ideograph */
  ["\u69F3", "\u69F3"], /* CJK Ideograph */
  ["\u69F4", "\u69F4"], /* CJK Ideograph */
  ["\u69F5", "\u69F5"], /* CJK Ideograph */
  ["\u69F6", "\u69F6"], /* CJK Ideograph */
  ["\u69F7", "\u69F7"], /* CJK Ideograph */
  ["\u69F8", "\u69F8"], /* CJK Ideograph */
  ["\u69F9", "\u69F9"], /* CJK Ideograph */
  ["\u69FA", "\u69FA"], /* CJK Ideograph */
  ["\u69FB", "\u69FB"], /* CJK Ideograph */
  ["\u69FC", "\u69FC"], /* CJK Ideograph */
  ["\u69FD", "\u69FD"], /* CJK Ideograph */
  ["\u69FE", "\u69FE"], /* CJK Ideograph */
  ["\u69FF", "\u69FF"], /* CJK Ideograph */
  ["\u6A00", "\u6A00"], /* CJK Ideograph */
  ["\u6A01", "\u6A01"], /* CJK Ideograph */
  ["\u6A02", "\u6A02"], /* CJK Ideograph */
  ["\u6A03", "\u6A03"], /* CJK Ideograph */
  ["\u6A04", "\u6A04"], /* CJK Ideograph */
  ["\u6A05", "\u6A05"], /* CJK Ideograph */
  ["\u6A06", "\u6A06"], /* CJK Ideograph */
  ["\u6A07", "\u6A07"], /* CJK Ideograph */
  ["\u6A08", "\u6A08"], /* CJK Ideograph */
  ["\u6A09", "\u6A09"], /* CJK Ideograph */
  ["\u6A0A", "\u6A0A"], /* CJK Ideograph */
  ["\u6A0B", "\u6A0B"], /* CJK Ideograph */
  ["\u6A0C", "\u6A0C"], /* CJK Ideograph */
  ["\u6A0D", "\u6A0D"], /* CJK Ideograph */
  ["\u6A0E", "\u6A0E"], /* CJK Ideograph */
  ["\u6A0F", "\u6A0F"], /* CJK Ideograph */
  ["\u6A10", "\u6A10"], /* CJK Ideograph */
  ["\u6A11", "\u6A11"], /* CJK Ideograph */
  ["\u6A12", "\u6A12"], /* CJK Ideograph */
  ["\u6A13", "\u6A13"], /* CJK Ideograph */
  ["\u6A14", "\u6A14"], /* CJK Ideograph */
  ["\u6A15", "\u6A15"], /* CJK Ideograph */
  ["\u6A16", "\u6A16"], /* CJK Ideograph */
  ["\u6A17", "\u6A17"], /* CJK Ideograph */
  ["\u6A18", "\u6A18"], /* CJK Ideograph */
  ["\u6A19", "\u6A19"], /* CJK Ideograph */
  ["\u6A1A", "\u6A1A"], /* CJK Ideograph */
  ["\u6A1B", "\u6A1B"], /* CJK Ideograph */
  ["\u6A1C", "\u6A1C"], /* CJK Ideograph */
  ["\u6A1D", "\u6A1D"], /* CJK Ideograph */
  ["\u6A1E", "\u6A1E"], /* CJK Ideograph */
  ["\u6A1F", "\u6A1F"], /* CJK Ideograph */
  ["\u6A20", "\u6A20"], /* CJK Ideograph */
  ["\u6A21", "\u6A21"], /* CJK Ideograph */
  ["\u6A22", "\u6A22"], /* CJK Ideograph */
  ["\u6A23", "\u6A23"], /* CJK Ideograph */
  ["\u6A24", "\u6A24"], /* CJK Ideograph */
  ["\u6A25", "\u6A25"], /* CJK Ideograph */
  ["\u6A26", "\u6A26"], /* CJK Ideograph */
  ["\u6A27", "\u6A27"], /* CJK Ideograph */
  ["\u6A28", "\u6A28"], /* CJK Ideograph */
  ["\u6A29", "\u6A29"], /* CJK Ideograph */
  ["\u6A2A", "\u6A2A"], /* CJK Ideograph */
  ["\u6A2B", "\u6A2B"], /* CJK Ideograph */
  ["\u6A2C", "\u6A2C"], /* CJK Ideograph */
  ["\u6A2D", "\u6A2D"], /* CJK Ideograph */
  ["\u6A2E", "\u6A2E"], /* CJK Ideograph */
  ["\u6A2F", "\u6A2F"], /* CJK Ideograph */
  ["\u6A30", "\u6A30"], /* CJK Ideograph */
  ["\u6A31", "\u6A31"], /* CJK Ideograph */
  ["\u6A32", "\u6A32"], /* CJK Ideograph */
  ["\u6A33", "\u6A33"], /* CJK Ideograph */
  ["\u6A34", "\u6A34"], /* CJK Ideograph */
  ["\u6A35", "\u6A35"], /* CJK Ideograph */
  ["\u6A36", "\u6A36"], /* CJK Ideograph */
  ["\u6A37", "\u6A37"], /* CJK Ideograph */
  ["\u6A38", "\u6A38"], /* CJK Ideograph */
  ["\u6A39", "\u6A39"], /* CJK Ideograph */
  ["\u6A3A", "\u6A3A"], /* CJK Ideograph */
  ["\u6A3B", "\u6A3B"], /* CJK Ideograph */
  ["\u6A3C", "\u6A3C"], /* CJK Ideograph */
  ["\u6A3D", "\u6A3D"], /* CJK Ideograph */
  ["\u6A3E", "\u6A3E"], /* CJK Ideograph */
  ["\u6A3F", "\u6A3F"], /* CJK Ideograph */
  ["\u6A40", "\u6A40"], /* CJK Ideograph */
  ["\u6A41", "\u6A41"], /* CJK Ideograph */
  ["\u6A42", "\u6A42"], /* CJK Ideograph */
  ["\u6A43", "\u6A43"], /* CJK Ideograph */
  ["\u6A44", "\u6A44"], /* CJK Ideograph */
  ["\u6A45", "\u6A45"], /* CJK Ideograph */
  ["\u6A46", "\u6A46"], /* CJK Ideograph */
  ["\u6A47", "\u6A47"], /* CJK Ideograph */
  ["\u6A48", "\u6A48"], /* CJK Ideograph */
  ["\u6A49", "\u6A49"], /* CJK Ideograph */
  ["\u6A4A", "\u6A4A"], /* CJK Ideograph */
  ["\u6A4B", "\u6A4B"], /* CJK Ideograph */
  ["\u6A4C", "\u6A4C"], /* CJK Ideograph */
  ["\u6A4D", "\u6A4D"], /* CJK Ideograph */
  ["\u6A4E", "\u6A4E"], /* CJK Ideograph */
  ["\u6A4F", "\u6A4F"], /* CJK Ideograph */
  ["\u6A50", "\u6A50"], /* CJK Ideograph */
  ["\u6A51", "\u6A51"], /* CJK Ideograph */
  ["\u6A52", "\u6A52"], /* CJK Ideograph */
  ["\u6A53", "\u6A53"], /* CJK Ideograph */
  ["\u6A54", "\u6A54"], /* CJK Ideograph */
  ["\u6A55", "\u6A55"], /* CJK Ideograph */
  ["\u6A56", "\u6A56"], /* CJK Ideograph */
  ["\u6A57", "\u6A57"], /* CJK Ideograph */
  ["\u6A58", "\u6A58"], /* CJK Ideograph */
  ["\u6A59", "\u6A59"], /* CJK Ideograph */
  ["\u6A5A", "\u6A5A"], /* CJK Ideograph */
  ["\u6A5B", "\u6A5B"], /* CJK Ideograph */
  ["\u6A5C", "\u6A5C"], /* CJK Ideograph */
  ["\u6A5D", "\u6A5D"], /* CJK Ideograph */
  ["\u6A5E", "\u6A5E"], /* CJK Ideograph */
  ["\u6A5F", "\u6A5F"], /* CJK Ideograph */
  ["\u6A60", "\u6A60"], /* CJK Ideograph */
  ["\u6A61", "\u6A61"], /* CJK Ideograph */
  ["\u6A62", "\u6A62"], /* CJK Ideograph */
  ["\u6A63", "\u6A63"], /* CJK Ideograph */
  ["\u6A64", "\u6A64"], /* CJK Ideograph */
  ["\u6A65", "\u6A65"], /* CJK Ideograph */
  ["\u6A66", "\u6A66"], /* CJK Ideograph */
  ["\u6A67", "\u6A67"], /* CJK Ideograph */
  ["\u6A68", "\u6A68"], /* CJK Ideograph */
  ["\u6A69", "\u6A69"], /* CJK Ideograph */
  ["\u6A6A", "\u6A6A"], /* CJK Ideograph */
  ["\u6A6B", "\u6A6B"], /* CJK Ideograph */
  ["\u6A6C", "\u6A6C"], /* CJK Ideograph */
  ["\u6A6D", "\u6A6D"], /* CJK Ideograph */
  ["\u6A6E", "\u6A6E"], /* CJK Ideograph */
  ["\u6A6F", "\u6A6F"], /* CJK Ideograph */
  ["\u6A70", "\u6A70"], /* CJK Ideograph */
  ["\u6A71", "\u6A71"], /* CJK Ideograph */
  ["\u6A72", "\u6A72"], /* CJK Ideograph */
  ["\u6A73", "\u6A73"], /* CJK Ideograph */
  ["\u6A74", "\u6A74"], /* CJK Ideograph */
  ["\u6A75", "\u6A75"], /* CJK Ideograph */
  ["\u6A76", "\u6A76"], /* CJK Ideograph */
  ["\u6A77", "\u6A77"], /* CJK Ideograph */
  ["\u6A78", "\u6A78"], /* CJK Ideograph */
  ["\u6A79", "\u6A79"], /* CJK Ideograph */
  ["\u6A7A", "\u6A7A"], /* CJK Ideograph */
  ["\u6A7B", "\u6A7B"], /* CJK Ideograph */
  ["\u6A7C", "\u6A7C"], /* CJK Ideograph */
  ["\u6A7D", "\u6A7D"], /* CJK Ideograph */
  ["\u6A7E", "\u6A7E"], /* CJK Ideograph */
  ["\u6A7F", "\u6A7F"], /* CJK Ideograph */
  ["\u6A80", "\u6A80"], /* CJK Ideograph */
  ["\u6A81", "\u6A81"], /* CJK Ideograph */
  ["\u6A82", "\u6A82"], /* CJK Ideograph */
  ["\u6A83", "\u6A83"], /* CJK Ideograph */
  ["\u6A84", "\u6A84"], /* CJK Ideograph */
  ["\u6A85", "\u6A85"], /* CJK Ideograph */
  ["\u6A86", "\u6A86"], /* CJK Ideograph */
  ["\u6A87", "\u6A87"], /* CJK Ideograph */
  ["\u6A88", "\u6A88"], /* CJK Ideograph */
  ["\u6A89", "\u6A89"], /* CJK Ideograph */
  ["\u6A8A", "\u6A8A"], /* CJK Ideograph */
  ["\u6A8B", "\u6A8B"], /* CJK Ideograph */
  ["\u6A8C", "\u6A8C"], /* CJK Ideograph */
  ["\u6A8D", "\u6A8D"], /* CJK Ideograph */
  ["\u6A8E", "\u6A8E"], /* CJK Ideograph */
  ["\u6A8F", "\u6A8F"], /* CJK Ideograph */
  ["\u6A90", "\u6A90"], /* CJK Ideograph */
  ["\u6A91", "\u6A91"], /* CJK Ideograph */
  ["\u6A92", "\u6A92"], /* CJK Ideograph */
  ["\u6A93", "\u6A93"], /* CJK Ideograph */
  ["\u6A94", "\u6A94"], /* CJK Ideograph */
  ["\u6A95", "\u6A95"], /* CJK Ideograph */
  ["\u6A96", "\u6A96"], /* CJK Ideograph */
  ["\u6A97", "\u6A97"], /* CJK Ideograph */
  ["\u6A98", "\u6A98"], /* CJK Ideograph */
  ["\u6A99", "\u6A99"], /* CJK Ideograph */
  ["\u6A9A", "\u6A9A"], /* CJK Ideograph */
  ["\u6A9B", "\u6A9B"], /* CJK Ideograph */
  ["\u6A9C", "\u6A9C"], /* CJK Ideograph */
  ["\u6A9D", "\u6A9D"], /* CJK Ideograph */
  ["\u6A9E", "\u6A9E"], /* CJK Ideograph */
  ["\u6A9F", "\u6A9F"], /* CJK Ideograph */
  ["\u6AA0", "\u6AA0"], /* CJK Ideograph */
  ["\u6AA1", "\u6AA1"], /* CJK Ideograph */
  ["\u6AA2", "\u6AA2"], /* CJK Ideograph */
  ["\u6AA3", "\u6AA3"], /* CJK Ideograph */
  ["\u6AA4", "\u6AA4"], /* CJK Ideograph */
  ["\u6AA5", "\u6AA5"], /* CJK Ideograph */
  ["\u6AA6", "\u6AA6"], /* CJK Ideograph */
  ["\u6AA7", "\u6AA7"], /* CJK Ideograph */
  ["\u6AA8", "\u6AA8"], /* CJK Ideograph */
  ["\u6AA9", "\u6AA9"], /* CJK Ideograph */
  ["\u6AAA", "\u6AAA"], /* CJK Ideograph */
  ["\u6AAB", "\u6AAB"], /* CJK Ideograph */
  ["\u6AAC", "\u6AAC"], /* CJK Ideograph */
  ["\u6AAD", "\u6AAD"], /* CJK Ideograph */
  ["\u6AAE", "\u6AAE"], /* CJK Ideograph */
  ["\u6AAF", "\u6AAF"], /* CJK Ideograph */
  ["\u6AB0", "\u6AB0"], /* CJK Ideograph */
  ["\u6AB1", "\u6AB1"], /* CJK Ideograph */
  ["\u6AB2", "\u6AB2"], /* CJK Ideograph */
  ["\u6AB3", "\u6AB3"], /* CJK Ideograph */
  ["\u6AB4", "\u6AB4"], /* CJK Ideograph */
  ["\u6AB5", "\u6AB5"], /* CJK Ideograph */
  ["\u6AB6", "\u6AB6"], /* CJK Ideograph */
  ["\u6AB7", "\u6AB7"], /* CJK Ideograph */
  ["\u6AB8", "\u6AB8"], /* CJK Ideograph */
  ["\u6AB9", "\u6AB9"], /* CJK Ideograph */
  ["\u6ABA", "\u6ABA"], /* CJK Ideograph */
  ["\u6ABB", "\u6ABB"], /* CJK Ideograph */
  ["\u6ABC", "\u6ABC"], /* CJK Ideograph */
  ["\u6ABD", "\u6ABD"], /* CJK Ideograph */
  ["\u6ABE", "\u6ABE"], /* CJK Ideograph */
  ["\u6ABF", "\u6ABF"], /* CJK Ideograph */
  ["\u6AC0", "\u6AC0"], /* CJK Ideograph */
  ["\u6AC1", "\u6AC1"], /* CJK Ideograph */
  ["\u6AC2", "\u6AC2"], /* CJK Ideograph */
  ["\u6AC3", "\u6AC3"], /* CJK Ideograph */
  ["\u6AC4", "\u6AC4"], /* CJK Ideograph */
  ["\u6AC5", "\u6AC5"], /* CJK Ideograph */
  ["\u6AC6", "\u6AC6"], /* CJK Ideograph */
  ["\u6AC7", "\u6AC7"], /* CJK Ideograph */
  ["\u6AC8", "\u6AC8"], /* CJK Ideograph */
  ["\u6AC9", "\u6AC9"], /* CJK Ideograph */
  ["\u6ACA", "\u6ACA"], /* CJK Ideograph */
  ["\u6ACB", "\u6ACB"], /* CJK Ideograph */
  ["\u6ACC", "\u6ACC"], /* CJK Ideograph */
  ["\u6ACD", "\u6ACD"], /* CJK Ideograph */
  ["\u6ACE", "\u6ACE"], /* CJK Ideograph */
  ["\u6ACF", "\u6ACF"], /* CJK Ideograph */
  ["\u6AD0", "\u6AD0"], /* CJK Ideograph */
  ["\u6AD1", "\u6AD1"], /* CJK Ideograph */
  ["\u6AD2", "\u6AD2"], /* CJK Ideograph */
  ["\u6AD3", "\u6AD3"], /* CJK Ideograph */
  ["\u6AD4", "\u6AD4"], /* CJK Ideograph */
  ["\u6AD5", "\u6AD5"], /* CJK Ideograph */
  ["\u6AD6", "\u6AD6"], /* CJK Ideograph */
  ["\u6AD7", "\u6AD7"], /* CJK Ideograph */
  ["\u6AD8", "\u6AD8"], /* CJK Ideograph */
  ["\u6AD9", "\u6AD9"], /* CJK Ideograph */
  ["\u6ADA", "\u6ADA"], /* CJK Ideograph */
  ["\u6ADB", "\u6ADB"], /* CJK Ideograph */
  ["\u6ADC", "\u6ADC"], /* CJK Ideograph */
  ["\u6ADD", "\u6ADD"], /* CJK Ideograph */
  ["\u6ADE", "\u6ADE"], /* CJK Ideograph */
  ["\u6ADF", "\u6ADF"], /* CJK Ideograph */
  ["\u6AE0", "\u6AE0"], /* CJK Ideograph */
  ["\u6AE1", "\u6AE1"], /* CJK Ideograph */
  ["\u6AE2", "\u6AE2"], /* CJK Ideograph */
  ["\u6AE3", "\u6AE3"], /* CJK Ideograph */
  ["\u6AE4", "\u6AE4"], /* CJK Ideograph */
  ["\u6AE5", "\u6AE5"], /* CJK Ideograph */
  ["\u6AE6", "\u6AE6"], /* CJK Ideograph */
  ["\u6AE7", "\u6AE7"], /* CJK Ideograph */
  ["\u6AE8", "\u6AE8"], /* CJK Ideograph */
  ["\u6AE9", "\u6AE9"], /* CJK Ideograph */
  ["\u6AEA", "\u6AEA"], /* CJK Ideograph */
  ["\u6AEB", "\u6AEB"], /* CJK Ideograph */
  ["\u6AEC", "\u6AEC"], /* CJK Ideograph */
  ["\u6AED", "\u6AED"], /* CJK Ideograph */
  ["\u6AEE", "\u6AEE"], /* CJK Ideograph */
  ["\u6AEF", "\u6AEF"], /* CJK Ideograph */
  ["\u6AF0", "\u6AF0"], /* CJK Ideograph */
  ["\u6AF1", "\u6AF1"], /* CJK Ideograph */
  ["\u6AF2", "\u6AF2"], /* CJK Ideograph */
  ["\u6AF3", "\u6AF3"], /* CJK Ideograph */
  ["\u6AF4", "\u6AF4"], /* CJK Ideograph */
  ["\u6AF5", "\u6AF5"], /* CJK Ideograph */
  ["\u6AF6", "\u6AF6"], /* CJK Ideograph */
  ["\u6AF7", "\u6AF7"], /* CJK Ideograph */
  ["\u6AF8", "\u6AF8"], /* CJK Ideograph */
  ["\u6AF9", "\u6AF9"], /* CJK Ideograph */
  ["\u6AFA", "\u6AFA"], /* CJK Ideograph */
  ["\u6AFB", "\u6AFB"], /* CJK Ideograph */
  ["\u6AFC", "\u6AFC"], /* CJK Ideograph */
  ["\u6AFD", "\u6AFD"], /* CJK Ideograph */
  ["\u6AFE", "\u6AFE"], /* CJK Ideograph */
  ["\u6AFF", "\u6AFF"], /* CJK Ideograph */
  ["\u6B00", "\u6B00"], /* CJK Ideograph */
  ["\u6B01", "\u6B01"], /* CJK Ideograph */
  ["\u6B02", "\u6B02"], /* CJK Ideograph */
  ["\u6B03", "\u6B03"], /* CJK Ideograph */
  ["\u6B04", "\u6B04"], /* CJK Ideograph */
  ["\u6B05", "\u6B05"], /* CJK Ideograph */
  ["\u6B06", "\u6B06"], /* CJK Ideograph */
  ["\u6B07", "\u6B07"], /* CJK Ideograph */
  ["\u6B08", "\u6B08"], /* CJK Ideograph */
  ["\u6B09", "\u6B09"], /* CJK Ideograph */
  ["\u6B0A", "\u6B0A"], /* CJK Ideograph */
  ["\u6B0B", "\u6B0B"], /* CJK Ideograph */
  ["\u6B0C", "\u6B0C"], /* CJK Ideograph */
  ["\u6B0D", "\u6B0D"], /* CJK Ideograph */
  ["\u6B0E", "\u6B0E"], /* CJK Ideograph */
  ["\u6B0F", "\u6B0F"], /* CJK Ideograph */
  ["\u6B10", "\u6B10"], /* CJK Ideograph */
  ["\u6B11", "\u6B11"], /* CJK Ideograph */
  ["\u6B12", "\u6B12"], /* CJK Ideograph */
  ["\u6B13", "\u6B13"], /* CJK Ideograph */
  ["\u6B14", "\u6B14"], /* CJK Ideograph */
  ["\u6B15", "\u6B15"], /* CJK Ideograph */
  ["\u6B16", "\u6B16"], /* CJK Ideograph */
  ["\u6B17", "\u6B17"], /* CJK Ideograph */
  ["\u6B18", "\u6B18"], /* CJK Ideograph */
  ["\u6B19", "\u6B19"], /* CJK Ideograph */
  ["\u6B1A", "\u6B1A"], /* CJK Ideograph */
  ["\u6B1B", "\u6B1B"], /* CJK Ideograph */
  ["\u6B1C", "\u6B1C"], /* CJK Ideograph */
  ["\u6B1D", "\u6B1D"], /* CJK Ideograph */
  ["\u6B1E", "\u6B1E"], /* CJK Ideograph */
  ["\u6B1F", "\u6B1F"], /* CJK Ideograph */
  ["\u6B20", "\u6B20"], /* CJK Ideograph */
  ["\u6B21", "\u6B21"], /* CJK Ideograph */
  ["\u6B22", "\u6B22"], /* CJK Ideograph */
  ["\u6B23", "\u6B23"], /* CJK Ideograph */
  ["\u6B24", "\u6B24"], /* CJK Ideograph */
  ["\u6B25", "\u6B25"], /* CJK Ideograph */
  ["\u6B26", "\u6B26"], /* CJK Ideograph */
  ["\u6B27", "\u6B27"], /* CJK Ideograph */
  ["\u6B28", "\u6B28"], /* CJK Ideograph */
  ["\u6B29", "\u6B29"], /* CJK Ideograph */
  ["\u6B2A", "\u6B2A"], /* CJK Ideograph */
  ["\u6B2B", "\u6B2B"], /* CJK Ideograph */
  ["\u6B2C", "\u6B2C"], /* CJK Ideograph */
  ["\u6B2D", "\u6B2D"], /* CJK Ideograph */
  ["\u6B2E", "\u6B2E"], /* CJK Ideograph */
  ["\u6B2F", "\u6B2F"], /* CJK Ideograph */
  ["\u6B30", "\u6B30"], /* CJK Ideograph */
  ["\u6B31", "\u6B31"], /* CJK Ideograph */
  ["\u6B32", "\u6B32"], /* CJK Ideograph */
  ["\u6B33", "\u6B33"], /* CJK Ideograph */
  ["\u6B34", "\u6B34"], /* CJK Ideograph */
  ["\u6B35", "\u6B35"], /* CJK Ideograph */
  ["\u6B36", "\u6B36"], /* CJK Ideograph */
  ["\u6B37", "\u6B37"], /* CJK Ideograph */
  ["\u6B38", "\u6B38"], /* CJK Ideograph */
  ["\u6B39", "\u6B39"], /* CJK Ideograph */
  ["\u6B3A", "\u6B3A"], /* CJK Ideograph */
  ["\u6B3B", "\u6B3B"], /* CJK Ideograph */
  ["\u6B3C", "\u6B3C"], /* CJK Ideograph */
  ["\u6B3D", "\u6B3D"], /* CJK Ideograph */
  ["\u6B3E", "\u6B3E"], /* CJK Ideograph */
  ["\u6B3F", "\u6B3F"], /* CJK Ideograph */
  ["\u6B40", "\u6B40"], /* CJK Ideograph */
  ["\u6B41", "\u6B41"], /* CJK Ideograph */
  ["\u6B42", "\u6B42"], /* CJK Ideograph */
  ["\u6B43", "\u6B43"], /* CJK Ideograph */
  ["\u6B44", "\u6B44"], /* CJK Ideograph */
  ["\u6B45", "\u6B45"], /* CJK Ideograph */
  ["\u6B46", "\u6B46"], /* CJK Ideograph */
  ["\u6B47", "\u6B47"], /* CJK Ideograph */
  ["\u6B48", "\u6B48"], /* CJK Ideograph */
  ["\u6B49", "\u6B49"], /* CJK Ideograph */
  ["\u6B4A", "\u6B4A"], /* CJK Ideograph */
  ["\u6B4B", "\u6B4B"], /* CJK Ideograph */
  ["\u6B4C", "\u6B4C"], /* CJK Ideograph */
  ["\u6B4D", "\u6B4D"], /* CJK Ideograph */
  ["\u6B4E", "\u6B4E"], /* CJK Ideograph */
  ["\u6B4F", "\u6B4F"], /* CJK Ideograph */
  ["\u6B50", "\u6B50"], /* CJK Ideograph */
  ["\u6B51", "\u6B51"], /* CJK Ideograph */
  ["\u6B52", "\u6B52"], /* CJK Ideograph */
  ["\u6B53", "\u6B53"], /* CJK Ideograph */
  ["\u6B54", "\u6B54"], /* CJK Ideograph */
  ["\u6B55", "\u6B55"], /* CJK Ideograph */
  ["\u6B56", "\u6B56"], /* CJK Ideograph */
  ["\u6B57", "\u6B57"], /* CJK Ideograph */
  ["\u6B58", "\u6B58"], /* CJK Ideograph */
  ["\u6B59", "\u6B59"], /* CJK Ideograph */
  ["\u6B5A", "\u6B5A"], /* CJK Ideograph */
  ["\u6B5B", "\u6B5B"], /* CJK Ideograph */
  ["\u6B5C", "\u6B5C"], /* CJK Ideograph */
  ["\u6B5D", "\u6B5D"], /* CJK Ideograph */
  ["\u6B5E", "\u6B5E"], /* CJK Ideograph */
  ["\u6B5F", "\u6B5F"], /* CJK Ideograph */
  ["\u6B60", "\u6B60"], /* CJK Ideograph */
  ["\u6B61", "\u6B61"], /* CJK Ideograph */
  ["\u6B62", "\u6B62"], /* CJK Ideograph */
  ["\u6B63", "\u6B63"], /* CJK Ideograph */
  ["\u6B64", "\u6B64"], /* CJK Ideograph */
  ["\u6B65", "\u6B65"], /* CJK Ideograph */
  ["\u6B66", "\u6B66"], /* CJK Ideograph */
  ["\u6B67", "\u6B67"], /* CJK Ideograph */
  ["\u6B68", "\u6B68"], /* CJK Ideograph */
  ["\u6B69", "\u6B69"], /* CJK Ideograph */
  ["\u6B6A", "\u6B6A"], /* CJK Ideograph */
  ["\u6B6B", "\u6B6B"], /* CJK Ideograph */
  ["\u6B6C", "\u6B6C"], /* CJK Ideograph */
  ["\u6B6D", "\u6B6D"], /* CJK Ideograph */
  ["\u6B6E", "\u6B6E"], /* CJK Ideograph */
  ["\u6B6F", "\u6B6F"], /* CJK Ideograph */
  ["\u6B70", "\u6B70"], /* CJK Ideograph */
  ["\u6B71", "\u6B71"], /* CJK Ideograph */
  ["\u6B72", "\u6B72"], /* CJK Ideograph */
  ["\u6B73", "\u6B73"], /* CJK Ideograph */
  ["\u6B74", "\u6B74"], /* CJK Ideograph */
  ["\u6B75", "\u6B75"], /* CJK Ideograph */
  ["\u6B76", "\u6B76"], /* CJK Ideograph */
  ["\u6B77", "\u6B77"], /* CJK Ideograph */
  ["\u6B78", "\u6B78"], /* CJK Ideograph */
  ["\u6B79", "\u6B79"], /* CJK Ideograph */
  ["\u6B7A", "\u6B7A"], /* CJK Ideograph */
  ["\u6B7B", "\u6B7B"], /* CJK Ideograph */
  ["\u6B7C", "\u6B7C"], /* CJK Ideograph */
  ["\u6B7D", "\u6B7D"], /* CJK Ideograph */
  ["\u6B7E", "\u6B7E"], /* CJK Ideograph */
  ["\u6B7F", "\u6B7F"], /* CJK Ideograph */
  ["\u6B80", "\u6B80"], /* CJK Ideograph */
  ["\u6B81", "\u6B81"], /* CJK Ideograph */
  ["\u6B82", "\u6B82"], /* CJK Ideograph */
  ["\u6B83", "\u6B83"], /* CJK Ideograph */
  ["\u6B84", "\u6B84"], /* CJK Ideograph */
  ["\u6B85", "\u6B85"], /* CJK Ideograph */
  ["\u6B86", "\u6B86"], /* CJK Ideograph */
  ["\u6B87", "\u6B87"], /* CJK Ideograph */
  ["\u6B88", "\u6B88"], /* CJK Ideograph */
  ["\u6B89", "\u6B89"], /* CJK Ideograph */
  ["\u6B8A", "\u6B8A"], /* CJK Ideograph */
  ["\u6B8B", "\u6B8B"], /* CJK Ideograph */
  ["\u6B8C", "\u6B8C"], /* CJK Ideograph */
  ["\u6B8D", "\u6B8D"], /* CJK Ideograph */
  ["\u6B8E", "\u6B8E"], /* CJK Ideograph */
  ["\u6B8F", "\u6B8F"], /* CJK Ideograph */
  ["\u6B90", "\u6B90"], /* CJK Ideograph */
  ["\u6B91", "\u6B91"], /* CJK Ideograph */
  ["\u6B92", "\u6B92"], /* CJK Ideograph */
  ["\u6B93", "\u6B93"], /* CJK Ideograph */
  ["\u6B94", "\u6B94"], /* CJK Ideograph */
  ["\u6B95", "\u6B95"], /* CJK Ideograph */
  ["\u6B96", "\u6B96"], /* CJK Ideograph */
  ["\u6B97", "\u6B97"], /* CJK Ideograph */
  ["\u6B98", "\u6B98"], /* CJK Ideograph */
  ["\u6B99", "\u6B99"], /* CJK Ideograph */
  ["\u6B9A", "\u6B9A"], /* CJK Ideograph */
  ["\u6B9B", "\u6B9B"], /* CJK Ideograph */
  ["\u6B9C", "\u6B9C"], /* CJK Ideograph */
  ["\u6B9D", "\u6B9D"], /* CJK Ideograph */
  ["\u6B9E", "\u6B9E"], /* CJK Ideograph */
  ["\u6B9F", "\u6B9F"], /* CJK Ideograph */
  ["\u6BA0", "\u6BA0"], /* CJK Ideograph */
  ["\u6BA1", "\u6BA1"], /* CJK Ideograph */
  ["\u6BA2", "\u6BA2"], /* CJK Ideograph */
  ["\u6BA3", "\u6BA3"], /* CJK Ideograph */
  ["\u6BA4", "\u6BA4"], /* CJK Ideograph */
  ["\u6BA5", "\u6BA5"], /* CJK Ideograph */
  ["\u6BA6", "\u6BA6"], /* CJK Ideograph */
  ["\u6BA7", "\u6BA7"], /* CJK Ideograph */
  ["\u6BA8", "\u6BA8"], /* CJK Ideograph */
  ["\u6BA9", "\u6BA9"], /* CJK Ideograph */
  ["\u6BAA", "\u6BAA"], /* CJK Ideograph */
  ["\u6BAB", "\u6BAB"], /* CJK Ideograph */
  ["\u6BAC", "\u6BAC"], /* CJK Ideograph */
  ["\u6BAD", "\u6BAD"], /* CJK Ideograph */
  ["\u6BAE", "\u6BAE"], /* CJK Ideograph */
  ["\u6BAF", "\u6BAF"], /* CJK Ideograph */
  ["\u6BB0", "\u6BB0"], /* CJK Ideograph */
  ["\u6BB1", "\u6BB1"], /* CJK Ideograph */
  ["\u6BB2", "\u6BB2"], /* CJK Ideograph */
  ["\u6BB3", "\u6BB3"], /* CJK Ideograph */
  ["\u6BB4", "\u6BB4"], /* CJK Ideograph */
  ["\u6BB5", "\u6BB5"], /* CJK Ideograph */
  ["\u6BB6", "\u6BB6"], /* CJK Ideograph */
  ["\u6BB7", "\u6BB7"], /* CJK Ideograph */
  ["\u6BB8", "\u6BB8"], /* CJK Ideograph */
  ["\u6BB9", "\u6BB9"], /* CJK Ideograph */
  ["\u6BBA", "\u6BBA"], /* CJK Ideograph */
  ["\u6BBB", "\u6BBB"], /* CJK Ideograph */
  ["\u6BBC", "\u6BBC"], /* CJK Ideograph */
  ["\u6BBD", "\u6BBD"], /* CJK Ideograph */
  ["\u6BBE", "\u6BBE"], /* CJK Ideograph */
  ["\u6BBF", "\u6BBF"], /* CJK Ideograph */
  ["\u6BC0", "\u6BC0"], /* CJK Ideograph */
  ["\u6BC1", "\u6BC1"], /* CJK Ideograph */
  ["\u6BC2", "\u6BC2"], /* CJK Ideograph */
  ["\u6BC3", "\u6BC3"], /* CJK Ideograph */
  ["\u6BC4", "\u6BC4"], /* CJK Ideograph */
  ["\u6BC5", "\u6BC5"], /* CJK Ideograph */
  ["\u6BC6", "\u6BC6"], /* CJK Ideograph */
  ["\u6BC7", "\u6BC7"], /* CJK Ideograph */
  ["\u6BC8", "\u6BC8"], /* CJK Ideograph */
  ["\u6BC9", "\u6BC9"], /* CJK Ideograph */
  ["\u6BCA", "\u6BCA"], /* CJK Ideograph */
  ["\u6BCB", "\u6BCB"], /* CJK Ideograph */
  ["\u6BCC", "\u6BCC"], /* CJK Ideograph */
  ["\u6BCD", "\u6BCD"], /* CJK Ideograph */
  ["\u6BCE", "\u6BCE"], /* CJK Ideograph */
  ["\u6BCF", "\u6BCF"], /* CJK Ideograph */
  ["\u6BD0", "\u6BD0"], /* CJK Ideograph */
  ["\u6BD1", "\u6BD1"], /* CJK Ideograph */
  ["\u6BD2", "\u6BD2"], /* CJK Ideograph */
  ["\u6BD3", "\u6BD3"], /* CJK Ideograph */
  ["\u6BD4", "\u6BD4"], /* CJK Ideograph */
  ["\u6BD5", "\u6BD5"], /* CJK Ideograph */
  ["\u6BD6", "\u6BD6"], /* CJK Ideograph */
  ["\u6BD7", "\u6BD7"], /* CJK Ideograph */
  ["\u6BD8", "\u6BD8"], /* CJK Ideograph */
  ["\u6BD9", "\u6BD9"], /* CJK Ideograph */
  ["\u6BDA", "\u6BDA"], /* CJK Ideograph */
  ["\u6BDB", "\u6BDB"], /* CJK Ideograph */
  ["\u6BDC", "\u6BDC"], /* CJK Ideograph */
  ["\u6BDD", "\u6BDD"], /* CJK Ideograph */
  ["\u6BDE", "\u6BDE"], /* CJK Ideograph */
  ["\u6BDF", "\u6BDF"], /* CJK Ideograph */
  ["\u6BE0", "\u6BE0"], /* CJK Ideograph */
  ["\u6BE1", "\u6BE1"], /* CJK Ideograph */
  ["\u6BE2", "\u6BE2"], /* CJK Ideograph */
  ["\u6BE3", "\u6BE3"], /* CJK Ideograph */
  ["\u6BE4", "\u6BE4"], /* CJK Ideograph */
  ["\u6BE5", "\u6BE5"], /* CJK Ideograph */
  ["\u6BE6", "\u6BE6"], /* CJK Ideograph */
  ["\u6BE7", "\u6BE7"], /* CJK Ideograph */
  ["\u6BE8", "\u6BE8"], /* CJK Ideograph */
  ["\u6BE9", "\u6BE9"], /* CJK Ideograph */
  ["\u6BEA", "\u6BEA"], /* CJK Ideograph */
  ["\u6BEB", "\u6BEB"], /* CJK Ideograph */
  ["\u6BEC", "\u6BEC"], /* CJK Ideograph */
  ["\u6BED", "\u6BED"], /* CJK Ideograph */
  ["\u6BEE", "\u6BEE"], /* CJK Ideograph */
  ["\u6BEF", "\u6BEF"], /* CJK Ideograph */
  ["\u6BF0", "\u6BF0"], /* CJK Ideograph */
  ["\u6BF1", "\u6BF1"], /* CJK Ideograph */
  ["\u6BF2", "\u6BF2"], /* CJK Ideograph */
  ["\u6BF3", "\u6BF3"], /* CJK Ideograph */
  ["\u6BF4", "\u6BF4"], /* CJK Ideograph */
  ["\u6BF5", "\u6BF5"], /* CJK Ideograph */
  ["\u6BF6", "\u6BF6"], /* CJK Ideograph */
  ["\u6BF7", "\u6BF7"], /* CJK Ideograph */
  ["\u6BF8", "\u6BF8"], /* CJK Ideograph */
  ["\u6BF9", "\u6BF9"], /* CJK Ideograph */
  ["\u6BFA", "\u6BFA"], /* CJK Ideograph */
  ["\u6BFB", "\u6BFB"], /* CJK Ideograph */
  ["\u6BFC", "\u6BFC"], /* CJK Ideograph */
  ["\u6BFD", "\u6BFD"], /* CJK Ideograph */
  ["\u6BFE", "\u6BFE"], /* CJK Ideograph */
  ["\u6BFF", "\u6BFF"], /* CJK Ideograph */
  ["\u6C00", "\u6C00"], /* CJK Ideograph */
  ["\u6C01", "\u6C01"], /* CJK Ideograph */
  ["\u6C02", "\u6C02"], /* CJK Ideograph */
  ["\u6C03", "\u6C03"], /* CJK Ideograph */
  ["\u6C04", "\u6C04"], /* CJK Ideograph */
  ["\u6C05", "\u6C05"], /* CJK Ideograph */
  ["\u6C06", "\u6C06"], /* CJK Ideograph */
  ["\u6C07", "\u6C07"], /* CJK Ideograph */
  ["\u6C08", "\u6C08"], /* CJK Ideograph */
  ["\u6C09", "\u6C09"], /* CJK Ideograph */
  ["\u6C0A", "\u6C0A"], /* CJK Ideograph */
  ["\u6C0B", "\u6C0B"], /* CJK Ideograph */
  ["\u6C0C", "\u6C0C"], /* CJK Ideograph */
  ["\u6C0D", "\u6C0D"], /* CJK Ideograph */
  ["\u6C0E", "\u6C0E"], /* CJK Ideograph */
  ["\u6C0F", "\u6C0F"], /* CJK Ideograph */
  ["\u6C10", "\u6C10"], /* CJK Ideograph */
  ["\u6C11", "\u6C11"], /* CJK Ideograph */
  ["\u6C12", "\u6C12"], /* CJK Ideograph */
  ["\u6C13", "\u6C13"], /* CJK Ideograph */
  ["\u6C14", "\u6C14"], /* CJK Ideograph */
  ["\u6C15", "\u6C15"], /* CJK Ideograph */
  ["\u6C16", "\u6C16"], /* CJK Ideograph */
  ["\u6C17", "\u6C17"], /* CJK Ideograph */
  ["\u6C18", "\u6C18"], /* CJK Ideograph */
  ["\u6C19", "\u6C19"], /* CJK Ideograph */
  ["\u6C1A", "\u6C1A"], /* CJK Ideograph */
  ["\u6C1B", "\u6C1B"], /* CJK Ideograph */
  ["\u6C1C", "\u6C1C"], /* CJK Ideograph */
  ["\u6C1D", "\u6C1D"], /* CJK Ideograph */
  ["\u6C1E", "\u6C1E"], /* CJK Ideograph */
  ["\u6C1F", "\u6C1F"], /* CJK Ideograph */
  ["\u6C20", "\u6C20"], /* CJK Ideograph */
  ["\u6C21", "\u6C21"], /* CJK Ideograph */
  ["\u6C22", "\u6C22"], /* CJK Ideograph */
  ["\u6C23", "\u6C23"], /* CJK Ideograph */
  ["\u6C24", "\u6C24"], /* CJK Ideograph */
  ["\u6C25", "\u6C25"], /* CJK Ideograph */
  ["\u6C26", "\u6C26"], /* CJK Ideograph */
  ["\u6C27", "\u6C27"], /* CJK Ideograph */
  ["\u6C28", "\u6C28"], /* CJK Ideograph */
  ["\u6C29", "\u6C29"], /* CJK Ideograph */
  ["\u6C2A", "\u6C2A"], /* CJK Ideograph */
  ["\u6C2B", "\u6C2B"], /* CJK Ideograph */
  ["\u6C2C", "\u6C2C"], /* CJK Ideograph */
  ["\u6C2D", "\u6C2D"], /* CJK Ideograph */
  ["\u6C2E", "\u6C2E"], /* CJK Ideograph */
  ["\u6C2F", "\u6C2F"], /* CJK Ideograph */
  ["\u6C30", "\u6C30"], /* CJK Ideograph */
  ["\u6C31", "\u6C31"], /* CJK Ideograph */
  ["\u6C32", "\u6C32"], /* CJK Ideograph */
  ["\u6C33", "\u6C33"], /* CJK Ideograph */
  ["\u6C34", "\u6C34"], /* CJK Ideograph */
  ["\u6C35", "\u6C35"], /* CJK Ideograph */
  ["\u6C36", "\u6C36"], /* CJK Ideograph */
  ["\u6C37", "\u6C37"], /* CJK Ideograph */
  ["\u6C38", "\u6C38"], /* CJK Ideograph */
  ["\u6C39", "\u6C39"], /* CJK Ideograph */
  ["\u6C3A", "\u6C3A"], /* CJK Ideograph */
  ["\u6C3B", "\u6C3B"], /* CJK Ideograph */
  ["\u6C3C", "\u6C3C"], /* CJK Ideograph */
  ["\u6C3D", "\u6C3D"], /* CJK Ideograph */
  ["\u6C3E", "\u6C3E"], /* CJK Ideograph */
  ["\u6C3F", "\u6C3F"], /* CJK Ideograph */
  ["\u6C40", "\u6C40"], /* CJK Ideograph */
  ["\u6C41", "\u6C41"], /* CJK Ideograph */
  ["\u6C42", "\u6C42"], /* CJK Ideograph */
  ["\u6C43", "\u6C43"], /* CJK Ideograph */
  ["\u6C44", "\u6C44"], /* CJK Ideograph */
  ["\u6C45", "\u6C45"], /* CJK Ideograph */
  ["\u6C46", "\u6C46"], /* CJK Ideograph */
  ["\u6C47", "\u6C47"], /* CJK Ideograph */
  ["\u6C48", "\u6C48"], /* CJK Ideograph */
  ["\u6C49", "\u6C49"], /* CJK Ideograph */
  ["\u6C4A", "\u6C4A"], /* CJK Ideograph */
  ["\u6C4B", "\u6C4B"], /* CJK Ideograph */
  ["\u6C4C", "\u6C4C"], /* CJK Ideograph */
  ["\u6C4D", "\u6C4D"], /* CJK Ideograph */
  ["\u6C4E", "\u6C4E"], /* CJK Ideograph */
  ["\u6C4F", "\u6C4F"], /* CJK Ideograph */
  ["\u6C50", "\u6C50"], /* CJK Ideograph */
  ["\u6C51", "\u6C51"], /* CJK Ideograph */
  ["\u6C52", "\u6C52"], /* CJK Ideograph */
  ["\u6C53", "\u6C53"], /* CJK Ideograph */
  ["\u6C54", "\u6C54"], /* CJK Ideograph */
  ["\u6C55", "\u6C55"], /* CJK Ideograph */
  ["\u6C56", "\u6C56"], /* CJK Ideograph */
  ["\u6C57", "\u6C57"], /* CJK Ideograph */
  ["\u6C58", "\u6C58"], /* CJK Ideograph */
  ["\u6C59", "\u6C59"], /* CJK Ideograph */
  ["\u6C5A", "\u6C5A"], /* CJK Ideograph */
  ["\u6C5B", "\u6C5B"], /* CJK Ideograph */
  ["\u6C5C", "\u6C5C"], /* CJK Ideograph */
  ["\u6C5D", "\u6C5D"], /* CJK Ideograph */
  ["\u6C5E", "\u6C5E"], /* CJK Ideograph */
  ["\u6C5F", "\u6C5F"], /* CJK Ideograph */
  ["\u6C60", "\u6C60"], /* CJK Ideograph */
  ["\u6C61", "\u6C61"], /* CJK Ideograph */
  ["\u6C62", "\u6C62"], /* CJK Ideograph */
  ["\u6C63", "\u6C63"], /* CJK Ideograph */
  ["\u6C64", "\u6C64"], /* CJK Ideograph */
  ["\u6C65", "\u6C65"], /* CJK Ideograph */
  ["\u6C66", "\u6C66"], /* CJK Ideograph */
  ["\u6C67", "\u6C67"], /* CJK Ideograph */
  ["\u6C68", "\u6C68"], /* CJK Ideograph */
  ["\u6C69", "\u6C69"], /* CJK Ideograph */
  ["\u6C6A", "\u6C6A"], /* CJK Ideograph */
  ["\u6C6B", "\u6C6B"], /* CJK Ideograph */
  ["\u6C6C", "\u6C6C"], /* CJK Ideograph */
  ["\u6C6D", "\u6C6D"], /* CJK Ideograph */
  ["\u6C6E", "\u6C6E"], /* CJK Ideograph */
  ["\u6C6F", "\u6C6F"], /* CJK Ideograph */
  ["\u6C70", "\u6C70"], /* CJK Ideograph */
  ["\u6C71", "\u6C71"], /* CJK Ideograph */
  ["\u6C72", "\u6C72"], /* CJK Ideograph */
  ["\u6C73", "\u6C73"], /* CJK Ideograph */
  ["\u6C74", "\u6C74"], /* CJK Ideograph */
  ["\u6C75", "\u6C75"], /* CJK Ideograph */
  ["\u6C76", "\u6C76"], /* CJK Ideograph */
  ["\u6C77", "\u6C77"], /* CJK Ideograph */
  ["\u6C78", "\u6C78"], /* CJK Ideograph */
  ["\u6C79", "\u6C79"], /* CJK Ideograph */
  ["\u6C7A", "\u6C7A"], /* CJK Ideograph */
  ["\u6C7B", "\u6C7B"], /* CJK Ideograph */
  ["\u6C7C", "\u6C7C"], /* CJK Ideograph */
  ["\u6C7D", "\u6C7D"], /* CJK Ideograph */
  ["\u6C7E", "\u6C7E"], /* CJK Ideograph */
  ["\u6C7F", "\u6C7F"], /* CJK Ideograph */
  ["\u6C80", "\u6C80"], /* CJK Ideograph */
  ["\u6C81", "\u6C81"], /* CJK Ideograph */
  ["\u6C82", "\u6C82"], /* CJK Ideograph */
  ["\u6C83", "\u6C83"], /* CJK Ideograph */
  ["\u6C84", "\u6C84"], /* CJK Ideograph */
  ["\u6C85", "\u6C85"], /* CJK Ideograph */
  ["\u6C86", "\u6C86"], /* CJK Ideograph */
  ["\u6C87", "\u6C87"], /* CJK Ideograph */
  ["\u6C88", "\u6C88"], /* CJK Ideograph */
  ["\u6C89", "\u6C89"], /* CJK Ideograph */
  ["\u6C8A", "\u6C8A"], /* CJK Ideograph */
  ["\u6C8B", "\u6C8B"], /* CJK Ideograph */
  ["\u6C8C", "\u6C8C"], /* CJK Ideograph */
  ["\u6C8D", "\u6C8D"], /* CJK Ideograph */
  ["\u6C8E", "\u6C8E"], /* CJK Ideograph */
  ["\u6C8F", "\u6C8F"], /* CJK Ideograph */
  ["\u6C90", "\u6C90"], /* CJK Ideograph */
  ["\u6C91", "\u6C91"], /* CJK Ideograph */
  ["\u6C92", "\u6C92"], /* CJK Ideograph */
  ["\u6C93", "\u6C93"], /* CJK Ideograph */
  ["\u6C94", "\u6C94"], /* CJK Ideograph */
  ["\u6C95", "\u6C95"], /* CJK Ideograph */
  ["\u6C96", "\u6C96"], /* CJK Ideograph */
  ["\u6C97", "\u6C97"], /* CJK Ideograph */
  ["\u6C98", "\u6C98"], /* CJK Ideograph */
  ["\u6C99", "\u6C99"], /* CJK Ideograph */
  ["\u6C9A", "\u6C9A"], /* CJK Ideograph */
  ["\u6C9B", "\u6C9B"], /* CJK Ideograph */
  ["\u6C9C", "\u6C9C"], /* CJK Ideograph */
  ["\u6C9D", "\u6C9D"], /* CJK Ideograph */
  ["\u6C9E", "\u6C9E"], /* CJK Ideograph */
  ["\u6C9F", "\u6C9F"], /* CJK Ideograph */
  ["\u6CA0", "\u6CA0"], /* CJK Ideograph */
  ["\u6CA1", "\u6CA1"], /* CJK Ideograph */
  ["\u6CA2", "\u6CA2"], /* CJK Ideograph */
  ["\u6CA3", "\u6CA3"], /* CJK Ideograph */
  ["\u6CA4", "\u6CA4"], /* CJK Ideograph */
  ["\u6CA5", "\u6CA5"], /* CJK Ideograph */
  ["\u6CA6", "\u6CA6"], /* CJK Ideograph */
  ["\u6CA7", "\u6CA7"], /* CJK Ideograph */
  ["\u6CA8", "\u6CA8"], /* CJK Ideograph */
  ["\u6CA9", "\u6CA9"], /* CJK Ideograph */
  ["\u6CAA", "\u6CAA"], /* CJK Ideograph */
  ["\u6CAB", "\u6CAB"], /* CJK Ideograph */
  ["\u6CAC", "\u6CAC"], /* CJK Ideograph */
  ["\u6CAD", "\u6CAD"], /* CJK Ideograph */
  ["\u6CAE", "\u6CAE"], /* CJK Ideograph */
  ["\u6CAF", "\u6CAF"], /* CJK Ideograph */
  ["\u6CB0", "\u6CB0"], /* CJK Ideograph */
  ["\u6CB1", "\u6CB1"], /* CJK Ideograph */
  ["\u6CB2", "\u6CB2"], /* CJK Ideograph */
  ["\u6CB3", "\u6CB3"], /* CJK Ideograph */
  ["\u6CB4", "\u6CB4"], /* CJK Ideograph */
  ["\u6CB5", "\u6CB5"], /* CJK Ideograph */
  ["\u6CB6", "\u6CB6"], /* CJK Ideograph */
  ["\u6CB7", "\u6CB7"], /* CJK Ideograph */
  ["\u6CB8", "\u6CB8"], /* CJK Ideograph */
  ["\u6CB9", "\u6CB9"], /* CJK Ideograph */
  ["\u6CBA", "\u6CBA"], /* CJK Ideograph */
  ["\u6CBB", "\u6CBB"], /* CJK Ideograph */
  ["\u6CBC", "\u6CBC"], /* CJK Ideograph */
  ["\u6CBD", "\u6CBD"], /* CJK Ideograph */
  ["\u6CBE", "\u6CBE"], /* CJK Ideograph */
  ["\u6CBF", "\u6CBF"], /* CJK Ideograph */
  ["\u6CC0", "\u6CC0"], /* CJK Ideograph */
  ["\u6CC1", "\u6CC1"], /* CJK Ideograph */
  ["\u6CC2", "\u6CC2"], /* CJK Ideograph */
  ["\u6CC3", "\u6CC3"], /* CJK Ideograph */
  ["\u6CC4", "\u6CC4"], /* CJK Ideograph */
  ["\u6CC5", "\u6CC5"], /* CJK Ideograph */
  ["\u6CC6", "\u6CC6"], /* CJK Ideograph */
  ["\u6CC7", "\u6CC7"], /* CJK Ideograph */
  ["\u6CC8", "\u6CC8"], /* CJK Ideograph */
  ["\u6CC9", "\u6CC9"], /* CJK Ideograph */
  ["\u6CCA", "\u6CCA"], /* CJK Ideograph */
  ["\u6CCB", "\u6CCB"], /* CJK Ideograph */
  ["\u6CCC", "\u6CCC"], /* CJK Ideograph */
  ["\u6CCD", "\u6CCD"], /* CJK Ideograph */
  ["\u6CCE", "\u6CCE"], /* CJK Ideograph */
  ["\u6CCF", "\u6CCF"], /* CJK Ideograph */
  ["\u6CD0", "\u6CD0"], /* CJK Ideograph */
  ["\u6CD1", "\u6CD1"], /* CJK Ideograph */
  ["\u6CD2", "\u6CD2"], /* CJK Ideograph */
  ["\u6CD3", "\u6CD3"], /* CJK Ideograph */
  ["\u6CD4", "\u6CD4"], /* CJK Ideograph */
  ["\u6CD5", "\u6CD5"], /* CJK Ideograph */
  ["\u6CD6", "\u6CD6"], /* CJK Ideograph */
  ["\u6CD7", "\u6CD7"], /* CJK Ideograph */
  ["\u6CD8", "\u6CD8"], /* CJK Ideograph */
  ["\u6CD9", "\u6CD9"], /* CJK Ideograph */
  ["\u6CDA", "\u6CDA"], /* CJK Ideograph */
  ["\u6CDB", "\u6CDB"], /* CJK Ideograph */
  ["\u6CDC", "\u6CDC"], /* CJK Ideograph */
  ["\u6CDD", "\u6CDD"], /* CJK Ideograph */
  ["\u6CDE", "\u6CDE"], /* CJK Ideograph */
  ["\u6CDF", "\u6CDF"], /* CJK Ideograph */
  ["\u6CE0", "\u6CE0"], /* CJK Ideograph */
  ["\u6CE1", "\u6CE1"], /* CJK Ideograph */
  ["\u6CE2", "\u6CE2"], /* CJK Ideograph */
  ["\u6CE3", "\u6CE3"], /* CJK Ideograph */
  ["\u6CE4", "\u6CE4"], /* CJK Ideograph */
  ["\u6CE5", "\u6CE5"], /* CJK Ideograph */
  ["\u6CE6", "\u6CE6"], /* CJK Ideograph */
  ["\u6CE7", "\u6CE7"], /* CJK Ideograph */
  ["\u6CE8", "\u6CE8"], /* CJK Ideograph */
  ["\u6CE9", "\u6CE9"], /* CJK Ideograph */
  ["\u6CEA", "\u6CEA"], /* CJK Ideograph */
  ["\u6CEB", "\u6CEB"], /* CJK Ideograph */
  ["\u6CEC", "\u6CEC"], /* CJK Ideograph */
  ["\u6CED", "\u6CED"], /* CJK Ideograph */
  ["\u6CEE", "\u6CEE"], /* CJK Ideograph */
  ["\u6CEF", "\u6CEF"], /* CJK Ideograph */
  ["\u6CF0", "\u6CF0"], /* CJK Ideograph */
  ["\u6CF1", "\u6CF1"], /* CJK Ideograph */
  ["\u6CF2", "\u6CF2"], /* CJK Ideograph */
  ["\u6CF3", "\u6CF3"], /* CJK Ideograph */
  ["\u6CF4", "\u6CF4"], /* CJK Ideograph */
  ["\u6CF5", "\u6CF5"], /* CJK Ideograph */
  ["\u6CF6", "\u6CF6"], /* CJK Ideograph */
  ["\u6CF7", "\u6CF7"], /* CJK Ideograph */
  ["\u6CF8", "\u6CF8"], /* CJK Ideograph */
  ["\u6CF9", "\u6CF9"], /* CJK Ideograph */
  ["\u6CFA", "\u6CFA"], /* CJK Ideograph */
  ["\u6CFB", "\u6CFB"], /* CJK Ideograph */
  ["\u6CFC", "\u6CFC"], /* CJK Ideograph */
  ["\u6CFD", "\u6CFD"], /* CJK Ideograph */
  ["\u6CFE", "\u6CFE"], /* CJK Ideograph */
  ["\u6CFF", "\u6CFF"], /* CJK Ideograph */
  ["\u6D00", "\u6D00"], /* CJK Ideograph */
  ["\u6D01", "\u6D01"], /* CJK Ideograph */
  ["\u6D02", "\u6D02"], /* CJK Ideograph */
  ["\u6D03", "\u6D03"], /* CJK Ideograph */
  ["\u6D04", "\u6D04"], /* CJK Ideograph */
  ["\u6D05", "\u6D05"], /* CJK Ideograph */
  ["\u6D06", "\u6D06"], /* CJK Ideograph */
  ["\u6D07", "\u6D07"], /* CJK Ideograph */
  ["\u6D08", "\u6D08"], /* CJK Ideograph */
  ["\u6D09", "\u6D09"], /* CJK Ideograph */
  ["\u6D0A", "\u6D0A"], /* CJK Ideograph */
  ["\u6D0B", "\u6D0B"], /* CJK Ideograph */
  ["\u6D0C", "\u6D0C"], /* CJK Ideograph */
  ["\u6D0D", "\u6D0D"], /* CJK Ideograph */
  ["\u6D0E", "\u6D0E"], /* CJK Ideograph */
  ["\u6D0F", "\u6D0F"], /* CJK Ideograph */
  ["\u6D10", "\u6D10"], /* CJK Ideograph */
  ["\u6D11", "\u6D11"], /* CJK Ideograph */
  ["\u6D12", "\u6D12"], /* CJK Ideograph */
  ["\u6D13", "\u6D13"], /* CJK Ideograph */
  ["\u6D14", "\u6D14"], /* CJK Ideograph */
  ["\u6D15", "\u6D15"], /* CJK Ideograph */
  ["\u6D16", "\u6D16"], /* CJK Ideograph */
  ["\u6D17", "\u6D17"], /* CJK Ideograph */
  ["\u6D18", "\u6D18"], /* CJK Ideograph */
  ["\u6D19", "\u6D19"], /* CJK Ideograph */
  ["\u6D1A", "\u6D1A"], /* CJK Ideograph */
  ["\u6D1B", "\u6D1B"], /* CJK Ideograph */
  ["\u6D1C", "\u6D1C"], /* CJK Ideograph */
  ["\u6D1D", "\u6D1D"], /* CJK Ideograph */
  ["\u6D1E", "\u6D1E"], /* CJK Ideograph */
  ["\u6D1F", "\u6D1F"], /* CJK Ideograph */
  ["\u6D20", "\u6D20"], /* CJK Ideograph */
  ["\u6D21", "\u6D21"], /* CJK Ideograph */
  ["\u6D22", "\u6D22"], /* CJK Ideograph */
  ["\u6D23", "\u6D23"], /* CJK Ideograph */
  ["\u6D24", "\u6D24"], /* CJK Ideograph */
  ["\u6D25", "\u6D25"], /* CJK Ideograph */
  ["\u6D26", "\u6D26"], /* CJK Ideograph */
  ["\u6D27", "\u6D27"], /* CJK Ideograph */
  ["\u6D28", "\u6D28"], /* CJK Ideograph */
  ["\u6D29", "\u6D29"], /* CJK Ideograph */
  ["\u6D2A", "\u6D2A"], /* CJK Ideograph */
  ["\u6D2B", "\u6D2B"], /* CJK Ideograph */
  ["\u6D2C", "\u6D2C"], /* CJK Ideograph */
  ["\u6D2D", "\u6D2D"], /* CJK Ideograph */
  ["\u6D2E", "\u6D2E"], /* CJK Ideograph */
  ["\u6D2F", "\u6D2F"], /* CJK Ideograph */
  ["\u6D30", "\u6D30"], /* CJK Ideograph */
  ["\u6D31", "\u6D31"], /* CJK Ideograph */
  ["\u6D32", "\u6D32"], /* CJK Ideograph */
  ["\u6D33", "\u6D33"], /* CJK Ideograph */
  ["\u6D34", "\u6D34"], /* CJK Ideograph */
  ["\u6D35", "\u6D35"], /* CJK Ideograph */
  ["\u6D36", "\u6D36"], /* CJK Ideograph */
  ["\u6D37", "\u6D37"], /* CJK Ideograph */
  ["\u6D38", "\u6D38"], /* CJK Ideograph */
  ["\u6D39", "\u6D39"], /* CJK Ideograph */
  ["\u6D3A", "\u6D3A"], /* CJK Ideograph */
  ["\u6D3B", "\u6D3B"], /* CJK Ideograph */
  ["\u6D3C", "\u6D3C"], /* CJK Ideograph */
  ["\u6D3D", "\u6D3D"], /* CJK Ideograph */
  ["\u6D3E", "\u6D3E"], /* CJK Ideograph */
  ["\u6D3F", "\u6D3F"], /* CJK Ideograph */
  ["\u6D40", "\u6D40"], /* CJK Ideograph */
  ["\u6D41", "\u6D41"], /* CJK Ideograph */
  ["\u6D42", "\u6D42"], /* CJK Ideograph */
  ["\u6D43", "\u6D43"], /* CJK Ideograph */
  ["\u6D44", "\u6D44"], /* CJK Ideograph */
  ["\u6D45", "\u6D45"], /* CJK Ideograph */
  ["\u6D46", "\u6D46"], /* CJK Ideograph */
  ["\u6D47", "\u6D47"], /* CJK Ideograph */
  ["\u6D48", "\u6D48"], /* CJK Ideograph */
  ["\u6D49", "\u6D49"], /* CJK Ideograph */
  ["\u6D4A", "\u6D4A"], /* CJK Ideograph */
  ["\u6D4B", "\u6D4B"], /* CJK Ideograph */
  ["\u6D4C", "\u6D4C"], /* CJK Ideograph */
  ["\u6D4D", "\u6D4D"], /* CJK Ideograph */
  ["\u6D4E", "\u6D4E"], /* CJK Ideograph */
  ["\u6D4F", "\u6D4F"], /* CJK Ideograph */
  ["\u6D50", "\u6D50"], /* CJK Ideograph */
  ["\u6D51", "\u6D51"], /* CJK Ideograph */
  ["\u6D52", "\u6D52"], /* CJK Ideograph */
  ["\u6D53", "\u6D53"], /* CJK Ideograph */
  ["\u6D54", "\u6D54"], /* CJK Ideograph */
  ["\u6D55", "\u6D55"], /* CJK Ideograph */
  ["\u6D56", "\u6D56"], /* CJK Ideograph */
  ["\u6D57", "\u6D57"], /* CJK Ideograph */
  ["\u6D58", "\u6D58"], /* CJK Ideograph */
  ["\u6D59", "\u6D59"], /* CJK Ideograph */
  ["\u6D5A", "\u6D5A"], /* CJK Ideograph */
  ["\u6D5B", "\u6D5B"], /* CJK Ideograph */
  ["\u6D5C", "\u6D5C"], /* CJK Ideograph */
  ["\u6D5D", "\u6D5D"], /* CJK Ideograph */
  ["\u6D5E", "\u6D5E"], /* CJK Ideograph */
  ["\u6D5F", "\u6D5F"], /* CJK Ideograph */
  ["\u6D60", "\u6D60"], /* CJK Ideograph */
  ["\u6D61", "\u6D61"], /* CJK Ideograph */
  ["\u6D62", "\u6D62"], /* CJK Ideograph */
  ["\u6D63", "\u6D63"], /* CJK Ideograph */
  ["\u6D64", "\u6D64"], /* CJK Ideograph */
  ["\u6D65", "\u6D65"], /* CJK Ideograph */
  ["\u6D66", "\u6D66"], /* CJK Ideograph */
  ["\u6D67", "\u6D67"], /* CJK Ideograph */
  ["\u6D68", "\u6D68"], /* CJK Ideograph */
  ["\u6D69", "\u6D69"], /* CJK Ideograph */
  ["\u6D6A", "\u6D6A"], /* CJK Ideograph */
  ["\u6D6B", "\u6D6B"], /* CJK Ideograph */
  ["\u6D6C", "\u6D6C"], /* CJK Ideograph */
  ["\u6D6D", "\u6D6D"], /* CJK Ideograph */
  ["\u6D6E", "\u6D6E"], /* CJK Ideograph */
  ["\u6D6F", "\u6D6F"], /* CJK Ideograph */
  ["\u6D70", "\u6D70"], /* CJK Ideograph */
  ["\u6D71", "\u6D71"], /* CJK Ideograph */
  ["\u6D72", "\u6D72"], /* CJK Ideograph */
  ["\u6D73", "\u6D73"], /* CJK Ideograph */
  ["\u6D74", "\u6D74"], /* CJK Ideograph */
  ["\u6D75", "\u6D75"], /* CJK Ideograph */
  ["\u6D76", "\u6D76"], /* CJK Ideograph */
  ["\u6D77", "\u6D77"], /* CJK Ideograph */
  ["\u6D78", "\u6D78"], /* CJK Ideograph */
  ["\u6D79", "\u6D79"], /* CJK Ideograph */
  ["\u6D7A", "\u6D7A"], /* CJK Ideograph */
  ["\u6D7B", "\u6D7B"], /* CJK Ideograph */
  ["\u6D7C", "\u6D7C"], /* CJK Ideograph */
  ["\u6D7D", "\u6D7D"], /* CJK Ideograph */
  ["\u6D7E", "\u6D7E"], /* CJK Ideograph */
  ["\u6D7F", "\u6D7F"], /* CJK Ideograph */
  ["\u6D80", "\u6D80"], /* CJK Ideograph */
  ["\u6D81", "\u6D81"], /* CJK Ideograph */
  ["\u6D82", "\u6D82"], /* CJK Ideograph */
  ["\u6D83", "\u6D83"], /* CJK Ideograph */
  ["\u6D84", "\u6D84"], /* CJK Ideograph */
  ["\u6D85", "\u6D85"], /* CJK Ideograph */
  ["\u6D86", "\u6D86"], /* CJK Ideograph */
  ["\u6D87", "\u6D87"], /* CJK Ideograph */
  ["\u6D88", "\u6D88"], /* CJK Ideograph */
  ["\u6D89", "\u6D89"], /* CJK Ideograph */
  ["\u6D8A", "\u6D8A"], /* CJK Ideograph */
  ["\u6D8B", "\u6D8B"], /* CJK Ideograph */
  ["\u6D8C", "\u6D8C"], /* CJK Ideograph */
  ["\u6D8D", "\u6D8D"], /* CJK Ideograph */
  ["\u6D8E", "\u6D8E"], /* CJK Ideograph */
  ["\u6D8F", "\u6D8F"], /* CJK Ideograph */
  ["\u6D90", "\u6D90"], /* CJK Ideograph */
  ["\u6D91", "\u6D91"], /* CJK Ideograph */
  ["\u6D92", "\u6D92"], /* CJK Ideograph */
  ["\u6D93", "\u6D93"], /* CJK Ideograph */
  ["\u6D94", "\u6D94"], /* CJK Ideograph */
  ["\u6D95", "\u6D95"], /* CJK Ideograph */
  ["\u6D96", "\u6D96"], /* CJK Ideograph */
  ["\u6D97", "\u6D97"], /* CJK Ideograph */
  ["\u6D98", "\u6D98"], /* CJK Ideograph */
  ["\u6D99", "\u6D99"], /* CJK Ideograph */
  ["\u6D9A", "\u6D9A"], /* CJK Ideograph */
  ["\u6D9B", "\u6D9B"], /* CJK Ideograph */
  ["\u6D9C", "\u6D9C"], /* CJK Ideograph */
  ["\u6D9D", "\u6D9D"], /* CJK Ideograph */
  ["\u6D9E", "\u6D9E"], /* CJK Ideograph */
  ["\u6D9F", "\u6D9F"], /* CJK Ideograph */
  ["\u6DA0", "\u6DA0"], /* CJK Ideograph */
  ["\u6DA1", "\u6DA1"], /* CJK Ideograph */
  ["\u6DA2", "\u6DA2"], /* CJK Ideograph */
  ["\u6DA3", "\u6DA3"], /* CJK Ideograph */
  ["\u6DA4", "\u6DA4"], /* CJK Ideograph */
  ["\u6DA5", "\u6DA5"], /* CJK Ideograph */
  ["\u6DA6", "\u6DA6"], /* CJK Ideograph */
  ["\u6DA7", "\u6DA7"], /* CJK Ideograph */
  ["\u6DA8", "\u6DA8"], /* CJK Ideograph */
  ["\u6DA9", "\u6DA9"], /* CJK Ideograph */
  ["\u6DAA", "\u6DAA"], /* CJK Ideograph */
  ["\u6DAB", "\u6DAB"], /* CJK Ideograph */
  ["\u6DAC", "\u6DAC"], /* CJK Ideograph */
  ["\u6DAD", "\u6DAD"], /* CJK Ideograph */
  ["\u6DAE", "\u6DAE"], /* CJK Ideograph */
  ["\u6DAF", "\u6DAF"], /* CJK Ideograph */
  ["\u6DB0", "\u6DB0"], /* CJK Ideograph */
  ["\u6DB1", "\u6DB1"], /* CJK Ideograph */
  ["\u6DB2", "\u6DB2"], /* CJK Ideograph */
  ["\u6DB3", "\u6DB3"], /* CJK Ideograph */
  ["\u6DB4", "\u6DB4"], /* CJK Ideograph */
  ["\u6DB5", "\u6DB5"], /* CJK Ideograph */
  ["\u6DB6", "\u6DB6"], /* CJK Ideograph */
  ["\u6DB7", "\u6DB7"], /* CJK Ideograph */
  ["\u6DB8", "\u6DB8"], /* CJK Ideograph */
  ["\u6DB9", "\u6DB9"], /* CJK Ideograph */
  ["\u6DBA", "\u6DBA"], /* CJK Ideograph */
  ["\u6DBB", "\u6DBB"], /* CJK Ideograph */
  ["\u6DBC", "\u6DBC"], /* CJK Ideograph */
  ["\u6DBD", "\u6DBD"], /* CJK Ideograph */
  ["\u6DBE", "\u6DBE"], /* CJK Ideograph */
  ["\u6DBF", "\u6DBF"], /* CJK Ideograph */
  ["\u6DC0", "\u6DC0"], /* CJK Ideograph */
  ["\u6DC1", "\u6DC1"], /* CJK Ideograph */
  ["\u6DC2", "\u6DC2"], /* CJK Ideograph */
  ["\u6DC3", "\u6DC3"], /* CJK Ideograph */
  ["\u6DC4", "\u6DC4"], /* CJK Ideograph */
  ["\u6DC5", "\u6DC5"], /* CJK Ideograph */
  ["\u6DC6", "\u6DC6"], /* CJK Ideograph */
  ["\u6DC7", "\u6DC7"], /* CJK Ideograph */
  ["\u6DC8", "\u6DC8"], /* CJK Ideograph */
  ["\u6DC9", "\u6DC9"], /* CJK Ideograph */
  ["\u6DCA", "\u6DCA"], /* CJK Ideograph */
  ["\u6DCB", "\u6DCB"], /* CJK Ideograph */
  ["\u6DCC", "\u6DCC"], /* CJK Ideograph */
  ["\u6DCD", "\u6DCD"], /* CJK Ideograph */
  ["\u6DCE", "\u6DCE"], /* CJK Ideograph */
  ["\u6DCF", "\u6DCF"], /* CJK Ideograph */
  ["\u6DD0", "\u6DD0"], /* CJK Ideograph */
  ["\u6DD1", "\u6DD1"], /* CJK Ideograph */
  ["\u6DD2", "\u6DD2"], /* CJK Ideograph */
  ["\u6DD3", "\u6DD3"], /* CJK Ideograph */
  ["\u6DD4", "\u6DD4"], /* CJK Ideograph */
  ["\u6DD5", "\u6DD5"], /* CJK Ideograph */
  ["\u6DD6", "\u6DD6"], /* CJK Ideograph */
  ["\u6DD7", "\u6DD7"], /* CJK Ideograph */
  ["\u6DD8", "\u6DD8"], /* CJK Ideograph */
  ["\u6DD9", "\u6DD9"], /* CJK Ideograph */
  ["\u6DDA", "\u6DDA"], /* CJK Ideograph */
  ["\u6DDB", "\u6DDB"], /* CJK Ideograph */
  ["\u6DDC", "\u6DDC"], /* CJK Ideograph */
  ["\u6DDD", "\u6DDD"], /* CJK Ideograph */
  ["\u6DDE", "\u6DDE"], /* CJK Ideograph */
  ["\u6DDF", "\u6DDF"], /* CJK Ideograph */
  ["\u6DE0", "\u6DE0"], /* CJK Ideograph */
  ["\u6DE1", "\u6DE1"], /* CJK Ideograph */
  ["\u6DE2", "\u6DE2"], /* CJK Ideograph */
  ["\u6DE3", "\u6DE3"], /* CJK Ideograph */
  ["\u6DE4", "\u6DE4"], /* CJK Ideograph */
  ["\u6DE5", "\u6DE5"], /* CJK Ideograph */
  ["\u6DE6", "\u6DE6"], /* CJK Ideograph */
  ["\u6DE7", "\u6DE7"], /* CJK Ideograph */
  ["\u6DE8", "\u6DE8"], /* CJK Ideograph */
  ["\u6DE9", "\u6DE9"], /* CJK Ideograph */
  ["\u6DEA", "\u6DEA"], /* CJK Ideograph */
  ["\u6DEB", "\u6DEB"], /* CJK Ideograph */
  ["\u6DEC", "\u6DEC"], /* CJK Ideograph */
  ["\u6DED", "\u6DED"], /* CJK Ideograph */
  ["\u6DEE", "\u6DEE"], /* CJK Ideograph */
  ["\u6DEF", "\u6DEF"], /* CJK Ideograph */
  ["\u6DF0", "\u6DF0"], /* CJK Ideograph */
  ["\u6DF1", "\u6DF1"], /* CJK Ideograph */
  ["\u6DF2", "\u6DF2"], /* CJK Ideograph */
  ["\u6DF3", "\u6DF3"], /* CJK Ideograph */
  ["\u6DF4", "\u6DF4"], /* CJK Ideograph */
  ["\u6DF5", "\u6DF5"], /* CJK Ideograph */
  ["\u6DF6", "\u6DF6"], /* CJK Ideograph */
  ["\u6DF7", "\u6DF7"], /* CJK Ideograph */
  ["\u6DF8", "\u6DF8"], /* CJK Ideograph */
  ["\u6DF9", "\u6DF9"], /* CJK Ideograph */
  ["\u6DFA", "\u6DFA"], /* CJK Ideograph */
  ["\u6DFB", "\u6DFB"], /* CJK Ideograph */
  ["\u6DFC", "\u6DFC"], /* CJK Ideograph */
  ["\u6DFD", "\u6DFD"], /* CJK Ideograph */
  ["\u6DFE", "\u6DFE"], /* CJK Ideograph */
  ["\u6DFF", "\u6DFF"], /* CJK Ideograph */
  ["\u6E00", "\u6E00"], /* CJK Ideograph */
  ["\u6E01", "\u6E01"], /* CJK Ideograph */
  ["\u6E02", "\u6E02"], /* CJK Ideograph */
  ["\u6E03", "\u6E03"], /* CJK Ideograph */
  ["\u6E04", "\u6E04"], /* CJK Ideograph */
  ["\u6E05", "\u6E05"], /* CJK Ideograph */
  ["\u6E06", "\u6E06"], /* CJK Ideograph */
  ["\u6E07", "\u6E07"], /* CJK Ideograph */
  ["\u6E08", "\u6E08"], /* CJK Ideograph */
  ["\u6E09", "\u6E09"], /* CJK Ideograph */
  ["\u6E0A", "\u6E0A"], /* CJK Ideograph */
  ["\u6E0B", "\u6E0B"], /* CJK Ideograph */
  ["\u6E0C", "\u6E0C"], /* CJK Ideograph */
  ["\u6E0D", "\u6E0D"], /* CJK Ideograph */
  ["\u6E0E", "\u6E0E"], /* CJK Ideograph */
  ["\u6E0F", "\u6E0F"], /* CJK Ideograph */
  ["\u6E10", "\u6E10"], /* CJK Ideograph */
  ["\u6E11", "\u6E11"], /* CJK Ideograph */
  ["\u6E12", "\u6E12"], /* CJK Ideograph */
  ["\u6E13", "\u6E13"], /* CJK Ideograph */
  ["\u6E14", "\u6E14"], /* CJK Ideograph */
  ["\u6E15", "\u6E15"], /* CJK Ideograph */
  ["\u6E16", "\u6E16"], /* CJK Ideograph */
  ["\u6E17", "\u6E17"], /* CJK Ideograph */
  ["\u6E18", "\u6E18"], /* CJK Ideograph */
  ["\u6E19", "\u6E19"], /* CJK Ideograph */
  ["\u6E1A", "\u6E1A"], /* CJK Ideograph */
  ["\u6E1B", "\u6E1B"], /* CJK Ideograph */
  ["\u6E1C", "\u6E1C"], /* CJK Ideograph */
  ["\u6E1D", "\u6E1D"], /* CJK Ideograph */
  ["\u6E1E", "\u6E1E"], /* CJK Ideograph */
  ["\u6E1F", "\u6E1F"], /* CJK Ideograph */
  ["\u6E20", "\u6E20"], /* CJK Ideograph */
  ["\u6E21", "\u6E21"], /* CJK Ideograph */
  ["\u6E22", "\u6E22"], /* CJK Ideograph */
  ["\u6E23", "\u6E23"], /* CJK Ideograph */
  ["\u6E24", "\u6E24"], /* CJK Ideograph */
  ["\u6E25", "\u6E25"], /* CJK Ideograph */
  ["\u6E26", "\u6E26"], /* CJK Ideograph */
  ["\u6E27", "\u6E27"], /* CJK Ideograph */
  ["\u6E28", "\u6E28"], /* CJK Ideograph */
  ["\u6E29", "\u6E29"], /* CJK Ideograph */
  ["\u6E2A", "\u6E2A"], /* CJK Ideograph */
  ["\u6E2B", "\u6E2B"], /* CJK Ideograph */
  ["\u6E2C", "\u6E2C"], /* CJK Ideograph */
  ["\u6E2D", "\u6E2D"], /* CJK Ideograph */
  ["\u6E2E", "\u6E2E"], /* CJK Ideograph */
  ["\u6E2F", "\u6E2F"], /* CJK Ideograph */
  ["\u6E30", "\u6E30"], /* CJK Ideograph */
  ["\u6E31", "\u6E31"], /* CJK Ideograph */
  ["\u6E32", "\u6E32"], /* CJK Ideograph */
  ["\u6E33", "\u6E33"], /* CJK Ideograph */
  ["\u6E34", "\u6E34"], /* CJK Ideograph */
  ["\u6E35", "\u6E35"], /* CJK Ideograph */
  ["\u6E36", "\u6E36"], /* CJK Ideograph */
  ["\u6E37", "\u6E37"], /* CJK Ideograph */
  ["\u6E38", "\u6E38"], /* CJK Ideograph */
  ["\u6E39", "\u6E39"], /* CJK Ideograph */
  ["\u6E3A", "\u6E3A"], /* CJK Ideograph */
  ["\u6E3B", "\u6E3B"], /* CJK Ideograph */
  ["\u6E3C", "\u6E3C"], /* CJK Ideograph */
  ["\u6E3D", "\u6E3D"], /* CJK Ideograph */
  ["\u6E3E", "\u6E3E"], /* CJK Ideograph */
  ["\u6E3F", "\u6E3F"], /* CJK Ideograph */
  ["\u6E40", "\u6E40"], /* CJK Ideograph */
  ["\u6E41", "\u6E41"], /* CJK Ideograph */
  ["\u6E42", "\u6E42"], /* CJK Ideograph */
  ["\u6E43", "\u6E43"], /* CJK Ideograph */
  ["\u6E44", "\u6E44"], /* CJK Ideograph */
  ["\u6E45", "\u6E45"], /* CJK Ideograph */
  ["\u6E46", "\u6E46"], /* CJK Ideograph */
  ["\u6E47", "\u6E47"], /* CJK Ideograph */
  ["\u6E48", "\u6E48"], /* CJK Ideograph */
  ["\u6E49", "\u6E49"], /* CJK Ideograph */
  ["\u6E4A", "\u6E4A"], /* CJK Ideograph */
  ["\u6E4B", "\u6E4B"], /* CJK Ideograph */
  ["\u6E4C", "\u6E4C"], /* CJK Ideograph */
  ["\u6E4D", "\u6E4D"], /* CJK Ideograph */
  ["\u6E4E", "\u6E4E"], /* CJK Ideograph */
  ["\u6E4F", "\u6E4F"], /* CJK Ideograph */
  ["\u6E50", "\u6E50"], /* CJK Ideograph */
  ["\u6E51", "\u6E51"], /* CJK Ideograph */
  ["\u6E52", "\u6E52"], /* CJK Ideograph */
  ["\u6E53", "\u6E53"], /* CJK Ideograph */
  ["\u6E54", "\u6E54"], /* CJK Ideograph */
  ["\u6E55", "\u6E55"], /* CJK Ideograph */
  ["\u6E56", "\u6E56"], /* CJK Ideograph */
  ["\u6E57", "\u6E57"], /* CJK Ideograph */
  ["\u6E58", "\u6E58"], /* CJK Ideograph */
  ["\u6E59", "\u6E59"], /* CJK Ideograph */
  ["\u6E5A", "\u6E5A"], /* CJK Ideograph */
  ["\u6E5B", "\u6E5B"], /* CJK Ideograph */
  ["\u6E5C", "\u6E5C"], /* CJK Ideograph */
  ["\u6E5D", "\u6E5D"], /* CJK Ideograph */
  ["\u6E5E", "\u6E5E"], /* CJK Ideograph */
  ["\u6E5F", "\u6E5F"], /* CJK Ideograph */
  ["\u6E60", "\u6E60"], /* CJK Ideograph */
  ["\u6E61", "\u6E61"], /* CJK Ideograph */
  ["\u6E62", "\u6E62"], /* CJK Ideograph */
  ["\u6E63", "\u6E63"], /* CJK Ideograph */
  ["\u6E64", "\u6E64"], /* CJK Ideograph */
  ["\u6E65", "\u6E65"], /* CJK Ideograph */
  ["\u6E66", "\u6E66"], /* CJK Ideograph */
  ["\u6E67", "\u6E67"], /* CJK Ideograph */
  ["\u6E68", "\u6E68"], /* CJK Ideograph */
  ["\u6E69", "\u6E69"], /* CJK Ideograph */
  ["\u6E6A", "\u6E6A"], /* CJK Ideograph */
  ["\u6E6B", "\u6E6B"], /* CJK Ideograph */
  ["\u6E6C", "\u6E6C"], /* CJK Ideograph */
  ["\u6E6D", "\u6E6D"], /* CJK Ideograph */
  ["\u6E6E", "\u6E6E"], /* CJK Ideograph */
  ["\u6E6F", "\u6E6F"], /* CJK Ideograph */
  ["\u6E70", "\u6E70"], /* CJK Ideograph */
  ["\u6E71", "\u6E71"], /* CJK Ideograph */
  ["\u6E72", "\u6E72"], /* CJK Ideograph */
  ["\u6E73", "\u6E73"], /* CJK Ideograph */
  ["\u6E74", "\u6E74"], /* CJK Ideograph */
  ["\u6E75", "\u6E75"], /* CJK Ideograph */
  ["\u6E76", "\u6E76"], /* CJK Ideograph */
  ["\u6E77", "\u6E77"], /* CJK Ideograph */
  ["\u6E78", "\u6E78"], /* CJK Ideograph */
  ["\u6E79", "\u6E79"], /* CJK Ideograph */
  ["\u6E7A", "\u6E7A"], /* CJK Ideograph */
  ["\u6E7B", "\u6E7B"], /* CJK Ideograph */
  ["\u6E7C", "\u6E7C"], /* CJK Ideograph */
  ["\u6E7D", "\u6E7D"], /* CJK Ideograph */
  ["\u6E7E", "\u6E7E"], /* CJK Ideograph */
  ["\u6E7F", "\u6E7F"], /* CJK Ideograph */
  ["\u6E80", "\u6E80"], /* CJK Ideograph */
  ["\u6E81", "\u6E81"], /* CJK Ideograph */
  ["\u6E82", "\u6E82"], /* CJK Ideograph */
  ["\u6E83", "\u6E83"], /* CJK Ideograph */
  ["\u6E84", "\u6E84"], /* CJK Ideograph */
  ["\u6E85", "\u6E85"], /* CJK Ideograph */
  ["\u6E86", "\u6E86"], /* CJK Ideograph */
  ["\u6E87", "\u6E87"], /* CJK Ideograph */
  ["\u6E88", "\u6E88"], /* CJK Ideograph */
  ["\u6E89", "\u6E89"], /* CJK Ideograph */
  ["\u6E8A", "\u6E8A"], /* CJK Ideograph */
  ["\u6E8B", "\u6E8B"], /* CJK Ideograph */
  ["\u6E8C", "\u6E8C"], /* CJK Ideograph */
  ["\u6E8D", "\u6E8D"], /* CJK Ideograph */
  ["\u6E8E", "\u6E8E"], /* CJK Ideograph */
  ["\u6E8F", "\u6E8F"], /* CJK Ideograph */
  ["\u6E90", "\u6E90"], /* CJK Ideograph */
  ["\u6E91", "\u6E91"], /* CJK Ideograph */
  ["\u6E92", "\u6E92"], /* CJK Ideograph */
  ["\u6E93", "\u6E93"], /* CJK Ideograph */
  ["\u6E94", "\u6E94"], /* CJK Ideograph */
  ["\u6E95", "\u6E95"], /* CJK Ideograph */
  ["\u6E96", "\u6E96"], /* CJK Ideograph */
  ["\u6E97", "\u6E97"], /* CJK Ideograph */
  ["\u6E98", "\u6E98"], /* CJK Ideograph */
  ["\u6E99", "\u6E99"], /* CJK Ideograph */
  ["\u6E9A", "\u6E9A"], /* CJK Ideograph */
  ["\u6E9B", "\u6E9B"], /* CJK Ideograph */
  ["\u6E9C", "\u6E9C"], /* CJK Ideograph */
  ["\u6E9D", "\u6E9D"], /* CJK Ideograph */
  ["\u6E9E", "\u6E9E"], /* CJK Ideograph */
  ["\u6E9F", "\u6E9F"], /* CJK Ideograph */
  ["\u6EA0", "\u6EA0"], /* CJK Ideograph */
  ["\u6EA1", "\u6EA1"], /* CJK Ideograph */
  ["\u6EA2", "\u6EA2"], /* CJK Ideograph */
  ["\u6EA3", "\u6EA3"], /* CJK Ideograph */
  ["\u6EA4", "\u6EA4"], /* CJK Ideograph */
  ["\u6EA5", "\u6EA5"], /* CJK Ideograph */
  ["\u6EA6", "\u6EA6"], /* CJK Ideograph */
  ["\u6EA7", "\u6EA7"], /* CJK Ideograph */
  ["\u6EA8", "\u6EA8"], /* CJK Ideograph */
  ["\u6EA9", "\u6EA9"], /* CJK Ideograph */
  ["\u6EAA", "\u6EAA"], /* CJK Ideograph */
  ["\u6EAB", "\u6EAB"], /* CJK Ideograph */
  ["\u6EAC", "\u6EAC"], /* CJK Ideograph */
  ["\u6EAD", "\u6EAD"], /* CJK Ideograph */
  ["\u6EAE", "\u6EAE"], /* CJK Ideograph */
  ["\u6EAF", "\u6EAF"], /* CJK Ideograph */
  ["\u6EB0", "\u6EB0"], /* CJK Ideograph */
  ["\u6EB1", "\u6EB1"], /* CJK Ideograph */
  ["\u6EB2", "\u6EB2"], /* CJK Ideograph */
  ["\u6EB3", "\u6EB3"], /* CJK Ideograph */
  ["\u6EB4", "\u6EB4"], /* CJK Ideograph */
  ["\u6EB5", "\u6EB5"], /* CJK Ideograph */
  ["\u6EB6", "\u6EB6"], /* CJK Ideograph */
  ["\u6EB7", "\u6EB7"], /* CJK Ideograph */
  ["\u6EB8", "\u6EB8"], /* CJK Ideograph */
  ["\u6EB9", "\u6EB9"], /* CJK Ideograph */
  ["\u6EBA", "\u6EBA"], /* CJK Ideograph */
  ["\u6EBB", "\u6EBB"], /* CJK Ideograph */
  ["\u6EBC", "\u6EBC"], /* CJK Ideograph */
  ["\u6EBD", "\u6EBD"], /* CJK Ideograph */
  ["\u6EBE", "\u6EBE"], /* CJK Ideograph */
  ["\u6EBF", "\u6EBF"], /* CJK Ideograph */
  ["\u6EC0", "\u6EC0"], /* CJK Ideograph */
  ["\u6EC1", "\u6EC1"], /* CJK Ideograph */
  ["\u6EC2", "\u6EC2"], /* CJK Ideograph */
  ["\u6EC3", "\u6EC3"], /* CJK Ideograph */
  ["\u6EC4", "\u6EC4"], /* CJK Ideograph */
  ["\u6EC5", "\u6EC5"], /* CJK Ideograph */
  ["\u6EC6", "\u6EC6"], /* CJK Ideograph */
  ["\u6EC7", "\u6EC7"], /* CJK Ideograph */
  ["\u6EC8", "\u6EC8"], /* CJK Ideograph */
  ["\u6EC9", "\u6EC9"], /* CJK Ideograph */
  ["\u6ECA", "\u6ECA"], /* CJK Ideograph */
  ["\u6ECB", "\u6ECB"], /* CJK Ideograph */
  ["\u6ECC", "\u6ECC"], /* CJK Ideograph */
  ["\u6ECD", "\u6ECD"], /* CJK Ideograph */
  ["\u6ECE", "\u6ECE"], /* CJK Ideograph */
  ["\u6ECF", "\u6ECF"], /* CJK Ideograph */
  ["\u6ED0", "\u6ED0"], /* CJK Ideograph */
  ["\u6ED1", "\u6ED1"], /* CJK Ideograph */
  ["\u6ED2", "\u6ED2"], /* CJK Ideograph */
  ["\u6ED3", "\u6ED3"], /* CJK Ideograph */
  ["\u6ED4", "\u6ED4"], /* CJK Ideograph */
  ["\u6ED5", "\u6ED5"], /* CJK Ideograph */
  ["\u6ED6", "\u6ED6"], /* CJK Ideograph */
  ["\u6ED7", "\u6ED7"], /* CJK Ideograph */
  ["\u6ED8", "\u6ED8"], /* CJK Ideograph */
  ["\u6ED9", "\u6ED9"], /* CJK Ideograph */
  ["\u6EDA", "\u6EDA"], /* CJK Ideograph */
  ["\u6EDB", "\u6EDB"], /* CJK Ideograph */
  ["\u6EDC", "\u6EDC"], /* CJK Ideograph */
  ["\u6EDD", "\u6EDD"], /* CJK Ideograph */
  ["\u6EDE", "\u6EDE"], /* CJK Ideograph */
  ["\u6EDF", "\u6EDF"], /* CJK Ideograph */
  ["\u6EE0", "\u6EE0"], /* CJK Ideograph */
  ["\u6EE1", "\u6EE1"], /* CJK Ideograph */
  ["\u6EE2", "\u6EE2"], /* CJK Ideograph */
  ["\u6EE3", "\u6EE3"], /* CJK Ideograph */
  ["\u6EE4", "\u6EE4"], /* CJK Ideograph */
  ["\u6EE5", "\u6EE5"], /* CJK Ideograph */
  ["\u6EE6", "\u6EE6"], /* CJK Ideograph */
  ["\u6EE7", "\u6EE7"], /* CJK Ideograph */
  ["\u6EE8", "\u6EE8"], /* CJK Ideograph */
  ["\u6EE9", "\u6EE9"], /* CJK Ideograph */
  ["\u6EEA", "\u6EEA"], /* CJK Ideograph */
  ["\u6EEB", "\u6EEB"], /* CJK Ideograph */
  ["\u6EEC", "\u6EEC"], /* CJK Ideograph */
  ["\u6EED", "\u6EED"], /* CJK Ideograph */
  ["\u6EEE", "\u6EEE"], /* CJK Ideograph */
  ["\u6EEF", "\u6EEF"], /* CJK Ideograph */
  ["\u6EF0", "\u6EF0"], /* CJK Ideograph */
  ["\u6EF1", "\u6EF1"], /* CJK Ideograph */
  ["\u6EF2", "\u6EF2"], /* CJK Ideograph */
  ["\u6EF3", "\u6EF3"], /* CJK Ideograph */
  ["\u6EF4", "\u6EF4"], /* CJK Ideograph */
  ["\u6EF5", "\u6EF5"], /* CJK Ideograph */
  ["\u6EF6", "\u6EF6"], /* CJK Ideograph */
  ["\u6EF7", "\u6EF7"], /* CJK Ideograph */
  ["\u6EF8", "\u6EF8"], /* CJK Ideograph */
  ["\u6EF9", "\u6EF9"], /* CJK Ideograph */
  ["\u6EFA", "\u6EFA"], /* CJK Ideograph */
  ["\u6EFB", "\u6EFB"], /* CJK Ideograph */
  ["\u6EFC", "\u6EFC"], /* CJK Ideograph */
  ["\u6EFD", "\u6EFD"], /* CJK Ideograph */
  ["\u6EFE", "\u6EFE"], /* CJK Ideograph */
  ["\u6EFF", "\u6EFF"], /* CJK Ideograph */
  ["\u6F00", "\u6F00"], /* CJK Ideograph */
  ["\u6F01", "\u6F01"], /* CJK Ideograph */
  ["\u6F02", "\u6F02"], /* CJK Ideograph */
  ["\u6F03", "\u6F03"], /* CJK Ideograph */
  ["\u6F04", "\u6F04"], /* CJK Ideograph */
  ["\u6F05", "\u6F05"], /* CJK Ideograph */
  ["\u6F06", "\u6F06"], /* CJK Ideograph */
  ["\u6F07", "\u6F07"], /* CJK Ideograph */
  ["\u6F08", "\u6F08"], /* CJK Ideograph */
  ["\u6F09", "\u6F09"], /* CJK Ideograph */
  ["\u6F0A", "\u6F0A"], /* CJK Ideograph */
  ["\u6F0B", "\u6F0B"], /* CJK Ideograph */
  ["\u6F0C", "\u6F0C"], /* CJK Ideograph */
  ["\u6F0D", "\u6F0D"], /* CJK Ideograph */
  ["\u6F0E", "\u6F0E"], /* CJK Ideograph */
  ["\u6F0F", "\u6F0F"], /* CJK Ideograph */
  ["\u6F10", "\u6F10"], /* CJK Ideograph */
  ["\u6F11", "\u6F11"], /* CJK Ideograph */
  ["\u6F12", "\u6F12"], /* CJK Ideograph */
  ["\u6F13", "\u6F13"], /* CJK Ideograph */
  ["\u6F14", "\u6F14"], /* CJK Ideograph */
  ["\u6F15", "\u6F15"], /* CJK Ideograph */
  ["\u6F16", "\u6F16"], /* CJK Ideograph */
  ["\u6F17", "\u6F17"], /* CJK Ideograph */
  ["\u6F18", "\u6F18"], /* CJK Ideograph */
  ["\u6F19", "\u6F19"], /* CJK Ideograph */
  ["\u6F1A", "\u6F1A"], /* CJK Ideograph */
  ["\u6F1B", "\u6F1B"], /* CJK Ideograph */
  ["\u6F1C", "\u6F1C"], /* CJK Ideograph */
  ["\u6F1D", "\u6F1D"], /* CJK Ideograph */
  ["\u6F1E", "\u6F1E"], /* CJK Ideograph */
  ["\u6F1F", "\u6F1F"], /* CJK Ideograph */
  ["\u6F20", "\u6F20"], /* CJK Ideograph */
  ["\u6F21", "\u6F21"], /* CJK Ideograph */
  ["\u6F22", "\u6F22"], /* CJK Ideograph */
  ["\u6F23", "\u6F23"], /* CJK Ideograph */
  ["\u6F24", "\u6F24"], /* CJK Ideograph */
  ["\u6F25", "\u6F25"], /* CJK Ideograph */
  ["\u6F26", "\u6F26"], /* CJK Ideograph */
  ["\u6F27", "\u6F27"], /* CJK Ideograph */
  ["\u6F28", "\u6F28"], /* CJK Ideograph */
  ["\u6F29", "\u6F29"], /* CJK Ideograph */
  ["\u6F2A", "\u6F2A"], /* CJK Ideograph */
  ["\u6F2B", "\u6F2B"], /* CJK Ideograph */
  ["\u6F2C", "\u6F2C"], /* CJK Ideograph */
  ["\u6F2D", "\u6F2D"], /* CJK Ideograph */
  ["\u6F2E", "\u6F2E"], /* CJK Ideograph */
  ["\u6F2F", "\u6F2F"], /* CJK Ideograph */
  ["\u6F30", "\u6F30"], /* CJK Ideograph */
  ["\u6F31", "\u6F31"], /* CJK Ideograph */
  ["\u6F32", "\u6F32"], /* CJK Ideograph */
  ["\u6F33", "\u6F33"], /* CJK Ideograph */
  ["\u6F34", "\u6F34"], /* CJK Ideograph */
  ["\u6F35", "\u6F35"], /* CJK Ideograph */
  ["\u6F36", "\u6F36"], /* CJK Ideograph */
  ["\u6F37", "\u6F37"], /* CJK Ideograph */
  ["\u6F38", "\u6F38"], /* CJK Ideograph */
  ["\u6F39", "\u6F39"], /* CJK Ideograph */
  ["\u6F3A", "\u6F3A"], /* CJK Ideograph */
  ["\u6F3B", "\u6F3B"], /* CJK Ideograph */
  ["\u6F3C", "\u6F3C"], /* CJK Ideograph */
  ["\u6F3D", "\u6F3D"], /* CJK Ideograph */
  ["\u6F3E", "\u6F3E"], /* CJK Ideograph */
  ["\u6F3F", "\u6F3F"], /* CJK Ideograph */
  ["\u6F40", "\u6F40"], /* CJK Ideograph */
  ["\u6F41", "\u6F41"], /* CJK Ideograph */
  ["\u6F42", "\u6F42"], /* CJK Ideograph */
  ["\u6F43", "\u6F43"], /* CJK Ideograph */
  ["\u6F44", "\u6F44"], /* CJK Ideograph */
  ["\u6F45", "\u6F45"], /* CJK Ideograph */
  ["\u6F46", "\u6F46"], /* CJK Ideograph */
  ["\u6F47", "\u6F47"], /* CJK Ideograph */
  ["\u6F48", "\u6F48"], /* CJK Ideograph */
  ["\u6F49", "\u6F49"], /* CJK Ideograph */
  ["\u6F4A", "\u6F4A"], /* CJK Ideograph */
  ["\u6F4B", "\u6F4B"], /* CJK Ideograph */
  ["\u6F4C", "\u6F4C"], /* CJK Ideograph */
  ["\u6F4D", "\u6F4D"], /* CJK Ideograph */
  ["\u6F4E", "\u6F4E"], /* CJK Ideograph */
  ["\u6F4F", "\u6F4F"], /* CJK Ideograph */
  ["\u6F50", "\u6F50"], /* CJK Ideograph */
  ["\u6F51", "\u6F51"], /* CJK Ideograph */
  ["\u6F52", "\u6F52"], /* CJK Ideograph */
  ["\u6F53", "\u6F53"], /* CJK Ideograph */
  ["\u6F54", "\u6F54"], /* CJK Ideograph */
  ["\u6F55", "\u6F55"], /* CJK Ideograph */
  ["\u6F56", "\u6F56"], /* CJK Ideograph */
  ["\u6F57", "\u6F57"], /* CJK Ideograph */
  ["\u6F58", "\u6F58"], /* CJK Ideograph */
  ["\u6F59", "\u6F59"], /* CJK Ideograph */
  ["\u6F5A", "\u6F5A"], /* CJK Ideograph */
  ["\u6F5B", "\u6F5B"], /* CJK Ideograph */
  ["\u6F5C", "\u6F5C"], /* CJK Ideograph */
  ["\u6F5D", "\u6F5D"], /* CJK Ideograph */
  ["\u6F5E", "\u6F5E"], /* CJK Ideograph */
  ["\u6F5F", "\u6F5F"], /* CJK Ideograph */
  ["\u6F60", "\u6F60"], /* CJK Ideograph */
  ["\u6F61", "\u6F61"], /* CJK Ideograph */
  ["\u6F62", "\u6F62"], /* CJK Ideograph */
  ["\u6F63", "\u6F63"], /* CJK Ideograph */
  ["\u6F64", "\u6F64"], /* CJK Ideograph */
  ["\u6F65", "\u6F65"], /* CJK Ideograph */
  ["\u6F66", "\u6F66"], /* CJK Ideograph */
  ["\u6F67", "\u6F67"], /* CJK Ideograph */
  ["\u6F68", "\u6F68"], /* CJK Ideograph */
  ["\u6F69", "\u6F69"], /* CJK Ideograph */
  ["\u6F6A", "\u6F6A"], /* CJK Ideograph */
  ["\u6F6B", "\u6F6B"], /* CJK Ideograph */
  ["\u6F6C", "\u6F6C"], /* CJK Ideograph */
  ["\u6F6D", "\u6F6D"], /* CJK Ideograph */
  ["\u6F6E", "\u6F6E"], /* CJK Ideograph */
  ["\u6F6F", "\u6F6F"], /* CJK Ideograph */
  ["\u6F70", "\u6F70"], /* CJK Ideograph */
  ["\u6F71", "\u6F71"], /* CJK Ideograph */
  ["\u6F72", "\u6F72"], /* CJK Ideograph */
  ["\u6F73", "\u6F73"], /* CJK Ideograph */
  ["\u6F74", "\u6F74"], /* CJK Ideograph */
  ["\u6F75", "\u6F75"], /* CJK Ideograph */
  ["\u6F76", "\u6F76"], /* CJK Ideograph */
  ["\u6F77", "\u6F77"], /* CJK Ideograph */
  ["\u6F78", "\u6F78"], /* CJK Ideograph */
  ["\u6F79", "\u6F79"], /* CJK Ideograph */
  ["\u6F7A", "\u6F7A"], /* CJK Ideograph */
  ["\u6F7B", "\u6F7B"], /* CJK Ideograph */
  ["\u6F7C", "\u6F7C"], /* CJK Ideograph */
  ["\u6F7D", "\u6F7D"], /* CJK Ideograph */
  ["\u6F7E", "\u6F7E"], /* CJK Ideograph */
  ["\u6F7F", "\u6F7F"], /* CJK Ideograph */
  ["\u6F80", "\u6F80"], /* CJK Ideograph */
  ["\u6F81", "\u6F81"], /* CJK Ideograph */
  ["\u6F82", "\u6F82"], /* CJK Ideograph */
  ["\u6F83", "\u6F83"], /* CJK Ideograph */
  ["\u6F84", "\u6F84"], /* CJK Ideograph */
  ["\u6F85", "\u6F85"], /* CJK Ideograph */
  ["\u6F86", "\u6F86"], /* CJK Ideograph */
  ["\u6F87", "\u6F87"], /* CJK Ideograph */
  ["\u6F88", "\u6F88"], /* CJK Ideograph */
  ["\u6F89", "\u6F89"], /* CJK Ideograph */
  ["\u6F8A", "\u6F8A"], /* CJK Ideograph */
  ["\u6F8B", "\u6F8B"], /* CJK Ideograph */
  ["\u6F8C", "\u6F8C"], /* CJK Ideograph */
  ["\u6F8D", "\u6F8D"], /* CJK Ideograph */
  ["\u6F8E", "\u6F8E"], /* CJK Ideograph */
  ["\u6F8F", "\u6F8F"], /* CJK Ideograph */
  ["\u6F90", "\u6F90"], /* CJK Ideograph */
  ["\u6F91", "\u6F91"], /* CJK Ideograph */
  ["\u6F92", "\u6F92"], /* CJK Ideograph */
  ["\u6F93", "\u6F93"], /* CJK Ideograph */
  ["\u6F94", "\u6F94"], /* CJK Ideograph */
  ["\u6F95", "\u6F95"], /* CJK Ideograph */
  ["\u6F96", "\u6F96"], /* CJK Ideograph */
  ["\u6F97", "\u6F97"], /* CJK Ideograph */
  ["\u6F98", "\u6F98"], /* CJK Ideograph */
  ["\u6F99", "\u6F99"], /* CJK Ideograph */
  ["\u6F9A", "\u6F9A"], /* CJK Ideograph */
  ["\u6F9B", "\u6F9B"], /* CJK Ideograph */
  ["\u6F9C", "\u6F9C"], /* CJK Ideograph */
  ["\u6F9D", "\u6F9D"], /* CJK Ideograph */
  ["\u6F9E", "\u6F9E"], /* CJK Ideograph */
  ["\u6F9F", "\u6F9F"], /* CJK Ideograph */
  ["\u6FA0", "\u6FA0"], /* CJK Ideograph */
  ["\u6FA1", "\u6FA1"], /* CJK Ideograph */
  ["\u6FA2", "\u6FA2"], /* CJK Ideograph */
  ["\u6FA3", "\u6FA3"], /* CJK Ideograph */
  ["\u6FA4", "\u6FA4"], /* CJK Ideograph */
  ["\u6FA5", "\u6FA5"], /* CJK Ideograph */
  ["\u6FA6", "\u6FA6"], /* CJK Ideograph */
  ["\u6FA7", "\u6FA7"], /* CJK Ideograph */
  ["\u6FA8", "\u6FA8"], /* CJK Ideograph */
  ["\u6FA9", "\u6FA9"], /* CJK Ideograph */
  ["\u6FAA", "\u6FAA"], /* CJK Ideograph */
  ["\u6FAB", "\u6FAB"], /* CJK Ideograph */
  ["\u6FAC", "\u6FAC"], /* CJK Ideograph */
  ["\u6FAD", "\u6FAD"], /* CJK Ideograph */
  ["\u6FAE", "\u6FAE"], /* CJK Ideograph */
  ["\u6FAF", "\u6FAF"], /* CJK Ideograph */
  ["\u6FB0", "\u6FB0"], /* CJK Ideograph */
  ["\u6FB1", "\u6FB1"], /* CJK Ideograph */
  ["\u6FB2", "\u6FB2"], /* CJK Ideograph */
  ["\u6FB3", "\u6FB3"], /* CJK Ideograph */
  ["\u6FB4", "\u6FB4"], /* CJK Ideograph */
  ["\u6FB5", "\u6FB5"], /* CJK Ideograph */
  ["\u6FB6", "\u6FB6"], /* CJK Ideograph */
  ["\u6FB7", "\u6FB7"], /* CJK Ideograph */
  ["\u6FB8", "\u6FB8"], /* CJK Ideograph */
  ["\u6FB9", "\u6FB9"], /* CJK Ideograph */
  ["\u6FBA", "\u6FBA"], /* CJK Ideograph */
  ["\u6FBB", "\u6FBB"], /* CJK Ideograph */
  ["\u6FBC", "\u6FBC"], /* CJK Ideograph */
  ["\u6FBD", "\u6FBD"], /* CJK Ideograph */
  ["\u6FBE", "\u6FBE"], /* CJK Ideograph */
  ["\u6FBF", "\u6FBF"], /* CJK Ideograph */
  ["\u6FC0", "\u6FC0"], /* CJK Ideograph */
  ["\u6FC1", "\u6FC1"], /* CJK Ideograph */
  ["\u6FC2", "\u6FC2"], /* CJK Ideograph */
  ["\u6FC3", "\u6FC3"], /* CJK Ideograph */
  ["\u6FC4", "\u6FC4"], /* CJK Ideograph */
  ["\u6FC5", "\u6FC5"], /* CJK Ideograph */
  ["\u6FC6", "\u6FC6"], /* CJK Ideograph */
  ["\u6FC7", "\u6FC7"], /* CJK Ideograph */
  ["\u6FC8", "\u6FC8"], /* CJK Ideograph */
  ["\u6FC9", "\u6FC9"], /* CJK Ideograph */
  ["\u6FCA", "\u6FCA"], /* CJK Ideograph */
  ["\u6FCB", "\u6FCB"], /* CJK Ideograph */
  ["\u6FCC", "\u6FCC"], /* CJK Ideograph */
  ["\u6FCD", "\u6FCD"], /* CJK Ideograph */
  ["\u6FCE", "\u6FCE"], /* CJK Ideograph */
  ["\u6FCF", "\u6FCF"], /* CJK Ideograph */
  ["\u6FD0", "\u6FD0"], /* CJK Ideograph */
  ["\u6FD1", "\u6FD1"], /* CJK Ideograph */
  ["\u6FD2", "\u6FD2"], /* CJK Ideograph */
  ["\u6FD3", "\u6FD3"], /* CJK Ideograph */
  ["\u6FD4", "\u6FD4"], /* CJK Ideograph */
  ["\u6FD5", "\u6FD5"], /* CJK Ideograph */
  ["\u6FD6", "\u6FD6"], /* CJK Ideograph */
  ["\u6FD7", "\u6FD7"], /* CJK Ideograph */
  ["\u6FD8", "\u6FD8"], /* CJK Ideograph */
  ["\u6FD9", "\u6FD9"], /* CJK Ideograph */
  ["\u6FDA", "\u6FDA"], /* CJK Ideograph */
  ["\u6FDB", "\u6FDB"], /* CJK Ideograph */
  ["\u6FDC", "\u6FDC"], /* CJK Ideograph */
  ["\u6FDD", "\u6FDD"], /* CJK Ideograph */
  ["\u6FDE", "\u6FDE"], /* CJK Ideograph */
  ["\u6FDF", "\u6FDF"], /* CJK Ideograph */
  ["\u6FE0", "\u6FE0"], /* CJK Ideograph */
  ["\u6FE1", "\u6FE1"], /* CJK Ideograph */
  ["\u6FE2", "\u6FE2"], /* CJK Ideograph */
  ["\u6FE3", "\u6FE3"], /* CJK Ideograph */
  ["\u6FE4", "\u6FE4"], /* CJK Ideograph */
  ["\u6FE5", "\u6FE5"], /* CJK Ideograph */
  ["\u6FE6", "\u6FE6"], /* CJK Ideograph */
  ["\u6FE7", "\u6FE7"], /* CJK Ideograph */
  ["\u6FE8", "\u6FE8"], /* CJK Ideograph */
  ["\u6FE9", "\u6FE9"], /* CJK Ideograph */
  ["\u6FEA", "\u6FEA"], /* CJK Ideograph */
  ["\u6FEB", "\u6FEB"], /* CJK Ideograph */
  ["\u6FEC", "\u6FEC"], /* CJK Ideograph */
  ["\u6FED", "\u6FED"], /* CJK Ideograph */
  ["\u6FEE", "\u6FEE"], /* CJK Ideograph */
  ["\u6FEF", "\u6FEF"], /* CJK Ideograph */
  ["\u6FF0", "\u6FF0"], /* CJK Ideograph */
  ["\u6FF1", "\u6FF1"], /* CJK Ideograph */
  ["\u6FF2", "\u6FF2"], /* CJK Ideograph */
  ["\u6FF3", "\u6FF3"], /* CJK Ideograph */
  ["\u6FF4", "\u6FF4"], /* CJK Ideograph */
  ["\u6FF5", "\u6FF5"], /* CJK Ideograph */
  ["\u6FF6", "\u6FF6"], /* CJK Ideograph */
  ["\u6FF7", "\u6FF7"], /* CJK Ideograph */
  ["\u6FF8", "\u6FF8"], /* CJK Ideograph */
  ["\u6FF9", "\u6FF9"], /* CJK Ideograph */
  ["\u6FFA", "\u6FFA"], /* CJK Ideograph */
  ["\u6FFB", "\u6FFB"], /* CJK Ideograph */
  ["\u6FFC", "\u6FFC"], /* CJK Ideograph */
  ["\u6FFD", "\u6FFD"], /* CJK Ideograph */
  ["\u6FFE", "\u6FFE"], /* CJK Ideograph */
  ["\u6FFF", "\u6FFF"], /* CJK Ideograph */
  ["\u7000", "\u7000"], /* CJK Ideograph */
  ["\u7001", "\u7001"], /* CJK Ideograph */
  ["\u7002", "\u7002"], /* CJK Ideograph */
  ["\u7003", "\u7003"], /* CJK Ideograph */
  ["\u7004", "\u7004"], /* CJK Ideograph */
  ["\u7005", "\u7005"], /* CJK Ideograph */
  ["\u7006", "\u7006"], /* CJK Ideograph */
  ["\u7007", "\u7007"], /* CJK Ideograph */
  ["\u7008", "\u7008"], /* CJK Ideograph */
  ["\u7009", "\u7009"], /* CJK Ideograph */
  ["\u700A", "\u700A"], /* CJK Ideograph */
  ["\u700B", "\u700B"], /* CJK Ideograph */
  ["\u700C", "\u700C"], /* CJK Ideograph */
  ["\u700D", "\u700D"], /* CJK Ideograph */
  ["\u700E", "\u700E"], /* CJK Ideograph */
  ["\u700F", "\u700F"], /* CJK Ideograph */
  ["\u7010", "\u7010"], /* CJK Ideograph */
  ["\u7011", "\u7011"], /* CJK Ideograph */
  ["\u7012", "\u7012"], /* CJK Ideograph */
  ["\u7013", "\u7013"], /* CJK Ideograph */
  ["\u7014", "\u7014"], /* CJK Ideograph */
  ["\u7015", "\u7015"], /* CJK Ideograph */
  ["\u7016", "\u7016"], /* CJK Ideograph */
  ["\u7017", "\u7017"], /* CJK Ideograph */
  ["\u7018", "\u7018"], /* CJK Ideograph */
  ["\u7019", "\u7019"], /* CJK Ideograph */
  ["\u701A", "\u701A"], /* CJK Ideograph */
  ["\u701B", "\u701B"], /* CJK Ideograph */
  ["\u701C", "\u701C"], /* CJK Ideograph */
  ["\u701D", "\u701D"], /* CJK Ideograph */
  ["\u701E", "\u701E"], /* CJK Ideograph */
  ["\u701F", "\u701F"], /* CJK Ideograph */
  ["\u7020", "\u7020"], /* CJK Ideograph */
  ["\u7021", "\u7021"], /* CJK Ideograph */
  ["\u7022", "\u7022"], /* CJK Ideograph */
  ["\u7023", "\u7023"], /* CJK Ideograph */
  ["\u7024", "\u7024"], /* CJK Ideograph */
  ["\u7025", "\u7025"], /* CJK Ideograph */
  ["\u7026", "\u7026"], /* CJK Ideograph */
  ["\u7027", "\u7027"], /* CJK Ideograph */
  ["\u7028", "\u7028"], /* CJK Ideograph */
  ["\u7029", "\u7029"], /* CJK Ideograph */
  ["\u702A", "\u702A"], /* CJK Ideograph */
  ["\u702B", "\u702B"], /* CJK Ideograph */
  ["\u702C", "\u702C"], /* CJK Ideograph */
  ["\u702D", "\u702D"], /* CJK Ideograph */
  ["\u702E", "\u702E"], /* CJK Ideograph */
  ["\u702F", "\u702F"], /* CJK Ideograph */
  ["\u7030", "\u7030"], /* CJK Ideograph */
  ["\u7031", "\u7031"], /* CJK Ideograph */
  ["\u7032", "\u7032"], /* CJK Ideograph */
  ["\u7033", "\u7033"], /* CJK Ideograph */
  ["\u7034", "\u7034"], /* CJK Ideograph */
  ["\u7035", "\u7035"], /* CJK Ideograph */
  ["\u7036", "\u7036"], /* CJK Ideograph */
  ["\u7037", "\u7037"], /* CJK Ideograph */
  ["\u7038", "\u7038"], /* CJK Ideograph */
  ["\u7039", "\u7039"], /* CJK Ideograph */
  ["\u703A", "\u703A"], /* CJK Ideograph */
  ["\u703B", "\u703B"], /* CJK Ideograph */
  ["\u703C", "\u703C"], /* CJK Ideograph */
  ["\u703D", "\u703D"], /* CJK Ideograph */
  ["\u703E", "\u703E"], /* CJK Ideograph */
  ["\u703F", "\u703F"], /* CJK Ideograph */
  ["\u7040", "\u7040"], /* CJK Ideograph */
  ["\u7041", "\u7041"], /* CJK Ideograph */
  ["\u7042", "\u7042"], /* CJK Ideograph */
  ["\u7043", "\u7043"], /* CJK Ideograph */
  ["\u7044", "\u7044"], /* CJK Ideograph */
  ["\u7045", "\u7045"], /* CJK Ideograph */
  ["\u7046", "\u7046"], /* CJK Ideograph */
  ["\u7047", "\u7047"], /* CJK Ideograph */
  ["\u7048", "\u7048"], /* CJK Ideograph */
  ["\u7049", "\u7049"], /* CJK Ideograph */
  ["\u704A", "\u704A"], /* CJK Ideograph */
  ["\u704B", "\u704B"], /* CJK Ideograph */
  ["\u704C", "\u704C"], /* CJK Ideograph */
  ["\u704D", "\u704D"], /* CJK Ideograph */
  ["\u704E", "\u704E"], /* CJK Ideograph */
  ["\u704F", "\u704F"], /* CJK Ideograph */
  ["\u7050", "\u7050"], /* CJK Ideograph */
  ["\u7051", "\u7051"], /* CJK Ideograph */
  ["\u7052", "\u7052"], /* CJK Ideograph */
  ["\u7053", "\u7053"], /* CJK Ideograph */
  ["\u7054", "\u7054"], /* CJK Ideograph */
  ["\u7055", "\u7055"], /* CJK Ideograph */
  ["\u7056", "\u7056"], /* CJK Ideograph */
  ["\u7057", "\u7057"], /* CJK Ideograph */
  ["\u7058", "\u7058"], /* CJK Ideograph */
  ["\u7059", "\u7059"], /* CJK Ideograph */
  ["\u705A", "\u705A"], /* CJK Ideograph */
  ["\u705B", "\u705B"], /* CJK Ideograph */
  ["\u705C", "\u705C"], /* CJK Ideograph */
  ["\u705D", "\u705D"], /* CJK Ideograph */
  ["\u705E", "\u705E"], /* CJK Ideograph */
  ["\u705F", "\u705F"], /* CJK Ideograph */
  ["\u7060", "\u7060"], /* CJK Ideograph */
  ["\u7061", "\u7061"], /* CJK Ideograph */
  ["\u7062", "\u7062"], /* CJK Ideograph */
  ["\u7063", "\u7063"], /* CJK Ideograph */
  ["\u7064", "\u7064"], /* CJK Ideograph */
  ["\u7065", "\u7065"], /* CJK Ideograph */
  ["\u7066", "\u7066"], /* CJK Ideograph */
  ["\u7067", "\u7067"], /* CJK Ideograph */
  ["\u7068", "\u7068"], /* CJK Ideograph */
  ["\u7069", "\u7069"], /* CJK Ideograph */
  ["\u706A", "\u706A"], /* CJK Ideograph */
  ["\u706B", "\u706B"], /* CJK Ideograph */
  ["\u706C", "\u706C"], /* CJK Ideograph */
  ["\u706D", "\u706D"], /* CJK Ideograph */
  ["\u706E", "\u706E"], /* CJK Ideograph */
  ["\u706F", "\u706F"], /* CJK Ideograph */
  ["\u7070", "\u7070"], /* CJK Ideograph */
  ["\u7071", "\u7071"], /* CJK Ideograph */
  ["\u7072", "\u7072"], /* CJK Ideograph */
  ["\u7073", "\u7073"], /* CJK Ideograph */
  ["\u7074", "\u7074"], /* CJK Ideograph */
  ["\u7075", "\u7075"], /* CJK Ideograph */
  ["\u7076", "\u7076"], /* CJK Ideograph */
  ["\u7077", "\u7077"], /* CJK Ideograph */
  ["\u7078", "\u7078"], /* CJK Ideograph */
  ["\u7079", "\u7079"], /* CJK Ideograph */
  ["\u707A", "\u707A"], /* CJK Ideograph */
  ["\u707B", "\u707B"], /* CJK Ideograph */
  ["\u707C", "\u707C"], /* CJK Ideograph */
  ["\u707D", "\u707D"], /* CJK Ideograph */
  ["\u707E", "\u707E"], /* CJK Ideograph */
  ["\u707F", "\u707F"], /* CJK Ideograph */
  ["\u7080", "\u7080"], /* CJK Ideograph */
  ["\u7081", "\u7081"], /* CJK Ideograph */
  ["\u7082", "\u7082"], /* CJK Ideograph */
  ["\u7083", "\u7083"], /* CJK Ideograph */
  ["\u7084", "\u7084"], /* CJK Ideograph */
  ["\u7085", "\u7085"], /* CJK Ideograph */
  ["\u7086", "\u7086"], /* CJK Ideograph */
  ["\u7087", "\u7087"], /* CJK Ideograph */
  ["\u7088", "\u7088"], /* CJK Ideograph */
  ["\u7089", "\u7089"], /* CJK Ideograph */
  ["\u708A", "\u708A"], /* CJK Ideograph */
  ["\u708B", "\u708B"], /* CJK Ideograph */
  ["\u708C", "\u708C"], /* CJK Ideograph */
  ["\u708D", "\u708D"], /* CJK Ideograph */
  ["\u708E", "\u708E"], /* CJK Ideograph */
  ["\u708F", "\u708F"], /* CJK Ideograph */
  ["\u7090", "\u7090"], /* CJK Ideograph */
  ["\u7091", "\u7091"], /* CJK Ideograph */
  ["\u7092", "\u7092"], /* CJK Ideograph */
  ["\u7093", "\u7093"], /* CJK Ideograph */
  ["\u7094", "\u7094"], /* CJK Ideograph */
  ["\u7095", "\u7095"], /* CJK Ideograph */
  ["\u7096", "\u7096"], /* CJK Ideograph */
  ["\u7097", "\u7097"], /* CJK Ideograph */
  ["\u7098", "\u7098"], /* CJK Ideograph */
  ["\u7099", "\u7099"], /* CJK Ideograph */
  ["\u709A", "\u709A"], /* CJK Ideograph */
  ["\u709B", "\u709B"], /* CJK Ideograph */
  ["\u709C", "\u709C"], /* CJK Ideograph */
  ["\u709D", "\u709D"], /* CJK Ideograph */
  ["\u709E", "\u709E"], /* CJK Ideograph */
  ["\u709F", "\u709F"], /* CJK Ideograph */
  ["\u70A0", "\u70A0"], /* CJK Ideograph */
  ["\u70A1", "\u70A1"], /* CJK Ideograph */
  ["\u70A2", "\u70A2"], /* CJK Ideograph */
  ["\u70A3", "\u70A3"], /* CJK Ideograph */
  ["\u70A4", "\u70A4"], /* CJK Ideograph */
  ["\u70A5", "\u70A5"], /* CJK Ideograph */
  ["\u70A6", "\u70A6"], /* CJK Ideograph */
  ["\u70A7", "\u70A7"], /* CJK Ideograph */
  ["\u70A8", "\u70A8"], /* CJK Ideograph */
  ["\u70A9", "\u70A9"], /* CJK Ideograph */
  ["\u70AA", "\u70AA"], /* CJK Ideograph */
  ["\u70AB", "\u70AB"], /* CJK Ideograph */
  ["\u70AC", "\u70AC"], /* CJK Ideograph */
  ["\u70AD", "\u70AD"], /* CJK Ideograph */
  ["\u70AE", "\u70AE"], /* CJK Ideograph */
  ["\u70AF", "\u70AF"], /* CJK Ideograph */
  ["\u70B0", "\u70B0"], /* CJK Ideograph */
  ["\u70B1", "\u70B1"], /* CJK Ideograph */
  ["\u70B2", "\u70B2"], /* CJK Ideograph */
  ["\u70B3", "\u70B3"], /* CJK Ideograph */
  ["\u70B4", "\u70B4"], /* CJK Ideograph */
  ["\u70B5", "\u70B5"], /* CJK Ideograph */
  ["\u70B6", "\u70B6"], /* CJK Ideograph */
  ["\u70B7", "\u70B7"], /* CJK Ideograph */
  ["\u70B8", "\u70B8"], /* CJK Ideograph */
  ["\u70B9", "\u70B9"], /* CJK Ideograph */
  ["\u70BA", "\u70BA"], /* CJK Ideograph */
  ["\u70BB", "\u70BB"], /* CJK Ideograph */
  ["\u70BC", "\u70BC"], /* CJK Ideograph */
  ["\u70BD", "\u70BD"], /* CJK Ideograph */
  ["\u70BE", "\u70BE"], /* CJK Ideograph */
  ["\u70BF", "\u70BF"], /* CJK Ideograph */
  ["\u70C0", "\u70C0"], /* CJK Ideograph */
  ["\u70C1", "\u70C1"], /* CJK Ideograph */
  ["\u70C2", "\u70C2"], /* CJK Ideograph */
  ["\u70C3", "\u70C3"], /* CJK Ideograph */
  ["\u70C4", "\u70C4"], /* CJK Ideograph */
  ["\u70C5", "\u70C5"], /* CJK Ideograph */
  ["\u70C6", "\u70C6"], /* CJK Ideograph */
  ["\u70C7", "\u70C7"], /* CJK Ideograph */
  ["\u70C8", "\u70C8"], /* CJK Ideograph */
  ["\u70C9", "\u70C9"], /* CJK Ideograph */
  ["\u70CA", "\u70CA"], /* CJK Ideograph */
  ["\u70CB", "\u70CB"], /* CJK Ideograph */
  ["\u70CC", "\u70CC"], /* CJK Ideograph */
  ["\u70CD", "\u70CD"], /* CJK Ideograph */
  ["\u70CE", "\u70CE"], /* CJK Ideograph */
  ["\u70CF", "\u70CF"], /* CJK Ideograph */
  ["\u70D0", "\u70D0"], /* CJK Ideograph */
  ["\u70D1", "\u70D1"], /* CJK Ideograph */
  ["\u70D2", "\u70D2"], /* CJK Ideograph */
  ["\u70D3", "\u70D3"], /* CJK Ideograph */
  ["\u70D4", "\u70D4"], /* CJK Ideograph */
  ["\u70D5", "\u70D5"], /* CJK Ideograph */
  ["\u70D6", "\u70D6"], /* CJK Ideograph */
  ["\u70D7", "\u70D7"], /* CJK Ideograph */
  ["\u70D8", "\u70D8"], /* CJK Ideograph */
  ["\u70D9", "\u70D9"], /* CJK Ideograph */
  ["\u70DA", "\u70DA"], /* CJK Ideograph */
  ["\u70DB", "\u70DB"], /* CJK Ideograph */
  ["\u70DC", "\u70DC"], /* CJK Ideograph */
  ["\u70DD", "\u70DD"], /* CJK Ideograph */
  ["\u70DE", "\u70DE"], /* CJK Ideograph */
  ["\u70DF", "\u70DF"], /* CJK Ideograph */
  ["\u70E0", "\u70E0"], /* CJK Ideograph */
  ["\u70E1", "\u70E1"], /* CJK Ideograph */
  ["\u70E2", "\u70E2"], /* CJK Ideograph */
  ["\u70E3", "\u70E3"], /* CJK Ideograph */
  ["\u70E4", "\u70E4"], /* CJK Ideograph */
  ["\u70E5", "\u70E5"], /* CJK Ideograph */
  ["\u70E6", "\u70E6"], /* CJK Ideograph */
  ["\u70E7", "\u70E7"], /* CJK Ideograph */
  ["\u70E8", "\u70E8"], /* CJK Ideograph */
  ["\u70E9", "\u70E9"], /* CJK Ideograph */
  ["\u70EA", "\u70EA"], /* CJK Ideograph */
  ["\u70EB", "\u70EB"], /* CJK Ideograph */
  ["\u70EC", "\u70EC"], /* CJK Ideograph */
  ["\u70ED", "\u70ED"], /* CJK Ideograph */
  ["\u70EE", "\u70EE"], /* CJK Ideograph */
  ["\u70EF", "\u70EF"], /* CJK Ideograph */
  ["\u70F0", "\u70F0"], /* CJK Ideograph */
  ["\u70F1", "\u70F1"], /* CJK Ideograph */
  ["\u70F2", "\u70F2"], /* CJK Ideograph */
  ["\u70F3", "\u70F3"], /* CJK Ideograph */
  ["\u70F4", "\u70F4"], /* CJK Ideograph */
  ["\u70F5", "\u70F5"], /* CJK Ideograph */
  ["\u70F6", "\u70F6"], /* CJK Ideograph */
  ["\u70F7", "\u70F7"], /* CJK Ideograph */
  ["\u70F8", "\u70F8"], /* CJK Ideograph */
  ["\u70F9", "\u70F9"], /* CJK Ideograph */
  ["\u70FA", "\u70FA"], /* CJK Ideograph */
  ["\u70FB", "\u70FB"], /* CJK Ideograph */
  ["\u70FC", "\u70FC"], /* CJK Ideograph */
  ["\u70FD", "\u70FD"], /* CJK Ideograph */
  ["\u70FE", "\u70FE"], /* CJK Ideograph */
  ["\u70FF", "\u70FF"], /* CJK Ideograph */
  ["\u7100", "\u7100"], /* CJK Ideograph */
  ["\u7101", "\u7101"], /* CJK Ideograph */
  ["\u7102", "\u7102"], /* CJK Ideograph */
  ["\u7103", "\u7103"], /* CJK Ideograph */
  ["\u7104", "\u7104"], /* CJK Ideograph */
  ["\u7105", "\u7105"], /* CJK Ideograph */
  ["\u7106", "\u7106"], /* CJK Ideograph */
  ["\u7107", "\u7107"], /* CJK Ideograph */
  ["\u7108", "\u7108"], /* CJK Ideograph */
  ["\u7109", "\u7109"], /* CJK Ideograph */
  ["\u710A", "\u710A"], /* CJK Ideograph */
  ["\u710B", "\u710B"], /* CJK Ideograph */
  ["\u710C", "\u710C"], /* CJK Ideograph */
  ["\u710D", "\u710D"], /* CJK Ideograph */
  ["\u710E", "\u710E"], /* CJK Ideograph */
  ["\u710F", "\u710F"], /* CJK Ideograph */
  ["\u7110", "\u7110"], /* CJK Ideograph */
  ["\u7111", "\u7111"], /* CJK Ideograph */
  ["\u7112", "\u7112"], /* CJK Ideograph */
  ["\u7113", "\u7113"], /* CJK Ideograph */
  ["\u7114", "\u7114"], /* CJK Ideograph */
  ["\u7115", "\u7115"], /* CJK Ideograph */
  ["\u7116", "\u7116"], /* CJK Ideograph */
  ["\u7117", "\u7117"], /* CJK Ideograph */
  ["\u7118", "\u7118"], /* CJK Ideograph */
  ["\u7119", "\u7119"], /* CJK Ideograph */
  ["\u711A", "\u711A"], /* CJK Ideograph */
  ["\u711B", "\u711B"], /* CJK Ideograph */
  ["\u711C", "\u711C"], /* CJK Ideograph */
  ["\u711D", "\u711D"], /* CJK Ideograph */
  ["\u711E", "\u711E"], /* CJK Ideograph */
  ["\u711F", "\u711F"], /* CJK Ideograph */
  ["\u7120", "\u7120"], /* CJK Ideograph */
  ["\u7121", "\u7121"], /* CJK Ideograph */
  ["\u7122", "\u7122"], /* CJK Ideograph */
  ["\u7123", "\u7123"], /* CJK Ideograph */
  ["\u7124", "\u7124"], /* CJK Ideograph */
  ["\u7125", "\u7125"], /* CJK Ideograph */
  ["\u7126", "\u7126"], /* CJK Ideograph */
  ["\u7127", "\u7127"], /* CJK Ideograph */
  ["\u7128", "\u7128"], /* CJK Ideograph */
  ["\u7129", "\u7129"], /* CJK Ideograph */
  ["\u712A", "\u712A"], /* CJK Ideograph */
  ["\u712B", "\u712B"], /* CJK Ideograph */
  ["\u712C", "\u712C"], /* CJK Ideograph */
  ["\u712D", "\u712D"], /* CJK Ideograph */
  ["\u712E", "\u712E"], /* CJK Ideograph */
  ["\u712F", "\u712F"], /* CJK Ideograph */
  ["\u7130", "\u7130"], /* CJK Ideograph */
  ["\u7131", "\u7131"], /* CJK Ideograph */
  ["\u7132", "\u7132"], /* CJK Ideograph */
  ["\u7133", "\u7133"], /* CJK Ideograph */
  ["\u7134", "\u7134"], /* CJK Ideograph */
  ["\u7135", "\u7135"], /* CJK Ideograph */
  ["\u7136", "\u7136"], /* CJK Ideograph */
  ["\u7137", "\u7137"], /* CJK Ideograph */
  ["\u7138", "\u7138"], /* CJK Ideograph */
  ["\u7139", "\u7139"], /* CJK Ideograph */
  ["\u713A", "\u713A"], /* CJK Ideograph */
  ["\u713B", "\u713B"], /* CJK Ideograph */
  ["\u713C", "\u713C"], /* CJK Ideograph */
  ["\u713D", "\u713D"], /* CJK Ideograph */
  ["\u713E", "\u713E"], /* CJK Ideograph */
  ["\u713F", "\u713F"], /* CJK Ideograph */
  ["\u7140", "\u7140"], /* CJK Ideograph */
  ["\u7141", "\u7141"], /* CJK Ideograph */
  ["\u7142", "\u7142"], /* CJK Ideograph */
  ["\u7143", "\u7143"], /* CJK Ideograph */
  ["\u7144", "\u7144"], /* CJK Ideograph */
  ["\u7145", "\u7145"], /* CJK Ideograph */
  ["\u7146", "\u7146"], /* CJK Ideograph */
  ["\u7147", "\u7147"], /* CJK Ideograph */
  ["\u7148", "\u7148"], /* CJK Ideograph */
  ["\u7149", "\u7149"], /* CJK Ideograph */
  ["\u714A", "\u714A"], /* CJK Ideograph */
  ["\u714B", "\u714B"], /* CJK Ideograph */
  ["\u714C", "\u714C"], /* CJK Ideograph */
  ["\u714D", "\u714D"], /* CJK Ideograph */
  ["\u714E", "\u714E"], /* CJK Ideograph */
  ["\u714F", "\u714F"], /* CJK Ideograph */
  ["\u7150", "\u7150"], /* CJK Ideograph */
  ["\u7151", "\u7151"], /* CJK Ideograph */
  ["\u7152", "\u7152"], /* CJK Ideograph */
  ["\u7153", "\u7153"], /* CJK Ideograph */
  ["\u7154", "\u7154"], /* CJK Ideograph */
  ["\u7155", "\u7155"], /* CJK Ideograph */
  ["\u7156", "\u7156"], /* CJK Ideograph */
  ["\u7157", "\u7157"], /* CJK Ideograph */
  ["\u7158", "\u7158"], /* CJK Ideograph */
  ["\u7159", "\u7159"], /* CJK Ideograph */
  ["\u715A", "\u715A"], /* CJK Ideograph */
  ["\u715B", "\u715B"], /* CJK Ideograph */
  ["\u715C", "\u715C"], /* CJK Ideograph */
  ["\u715D", "\u715D"], /* CJK Ideograph */
  ["\u715E", "\u715E"], /* CJK Ideograph */
  ["\u715F", "\u715F"], /* CJK Ideograph */
  ["\u7160", "\u7160"], /* CJK Ideograph */
  ["\u7161", "\u7161"], /* CJK Ideograph */
  ["\u7162", "\u7162"], /* CJK Ideograph */
  ["\u7163", "\u7163"], /* CJK Ideograph */
  ["\u7164", "\u7164"], /* CJK Ideograph */
  ["\u7165", "\u7165"], /* CJK Ideograph */
  ["\u7166", "\u7166"], /* CJK Ideograph */
  ["\u7167", "\u7167"], /* CJK Ideograph */
  ["\u7168", "\u7168"], /* CJK Ideograph */
  ["\u7169", "\u7169"], /* CJK Ideograph */
  ["\u716A", "\u716A"], /* CJK Ideograph */
  ["\u716B", "\u716B"], /* CJK Ideograph */
  ["\u716C", "\u716C"], /* CJK Ideograph */
  ["\u716D", "\u716D"], /* CJK Ideograph */
  ["\u716E", "\u716E"], /* CJK Ideograph */
  ["\u716F", "\u716F"], /* CJK Ideograph */
  ["\u7170", "\u7170"], /* CJK Ideograph */
  ["\u7171", "\u7171"], /* CJK Ideograph */
  ["\u7172", "\u7172"], /* CJK Ideograph */
  ["\u7173", "\u7173"], /* CJK Ideograph */
  ["\u7174", "\u7174"], /* CJK Ideograph */
  ["\u7175", "\u7175"], /* CJK Ideograph */
  ["\u7176", "\u7176"], /* CJK Ideograph */
  ["\u7177", "\u7177"], /* CJK Ideograph */
  ["\u7178", "\u7178"], /* CJK Ideograph */
  ["\u7179", "\u7179"], /* CJK Ideograph */
  ["\u717A", "\u717A"], /* CJK Ideograph */
  ["\u717B", "\u717B"], /* CJK Ideograph */
  ["\u717C", "\u717C"], /* CJK Ideograph */
  ["\u717D", "\u717D"], /* CJK Ideograph */
  ["\u717E", "\u717E"], /* CJK Ideograph */
  ["\u717F", "\u717F"], /* CJK Ideograph */
  ["\u7180", "\u7180"], /* CJK Ideograph */
  ["\u7181", "\u7181"], /* CJK Ideograph */
  ["\u7182", "\u7182"], /* CJK Ideograph */
  ["\u7183", "\u7183"], /* CJK Ideograph */
  ["\u7184", "\u7184"], /* CJK Ideograph */
  ["\u7185", "\u7185"], /* CJK Ideograph */
  ["\u7186", "\u7186"], /* CJK Ideograph */
  ["\u7187", "\u7187"], /* CJK Ideograph */
  ["\u7188", "\u7188"], /* CJK Ideograph */
  ["\u7189", "\u7189"], /* CJK Ideograph */
  ["\u718A", "\u718A"], /* CJK Ideograph */
  ["\u718B", "\u718B"], /* CJK Ideograph */
  ["\u718C", "\u718C"], /* CJK Ideograph */
  ["\u718D", "\u718D"], /* CJK Ideograph */
  ["\u718E", "\u718E"], /* CJK Ideograph */
  ["\u718F", "\u718F"], /* CJK Ideograph */
  ["\u7190", "\u7190"], /* CJK Ideograph */
  ["\u7191", "\u7191"], /* CJK Ideograph */
  ["\u7192", "\u7192"], /* CJK Ideograph */
  ["\u7193", "\u7193"], /* CJK Ideograph */
  ["\u7194", "\u7194"], /* CJK Ideograph */
  ["\u7195", "\u7195"], /* CJK Ideograph */
  ["\u7196", "\u7196"], /* CJK Ideograph */
  ["\u7197", "\u7197"], /* CJK Ideograph */
  ["\u7198", "\u7198"], /* CJK Ideograph */
  ["\u7199", "\u7199"], /* CJK Ideograph */
  ["\u719A", "\u719A"], /* CJK Ideograph */
  ["\u719B", "\u719B"], /* CJK Ideograph */
  ["\u719C", "\u719C"], /* CJK Ideograph */
  ["\u719D", "\u719D"], /* CJK Ideograph */
  ["\u719E", "\u719E"], /* CJK Ideograph */
  ["\u719F", "\u719F"], /* CJK Ideograph */
  ["\u71A0", "\u71A0"], /* CJK Ideograph */
  ["\u71A1", "\u71A1"], /* CJK Ideograph */
  ["\u71A2", "\u71A2"], /* CJK Ideograph */
  ["\u71A3", "\u71A3"], /* CJK Ideograph */
  ["\u71A4", "\u71A4"], /* CJK Ideograph */
  ["\u71A5", "\u71A5"], /* CJK Ideograph */
  ["\u71A6", "\u71A6"], /* CJK Ideograph */
  ["\u71A7", "\u71A7"], /* CJK Ideograph */
  ["\u71A8", "\u71A8"], /* CJK Ideograph */
  ["\u71A9", "\u71A9"], /* CJK Ideograph */
  ["\u71AA", "\u71AA"], /* CJK Ideograph */
  ["\u71AB", "\u71AB"], /* CJK Ideograph */
  ["\u71AC", "\u71AC"], /* CJK Ideograph */
  ["\u71AD", "\u71AD"], /* CJK Ideograph */
  ["\u71AE", "\u71AE"], /* CJK Ideograph */
  ["\u71AF", "\u71AF"], /* CJK Ideograph */
  ["\u71B0", "\u71B0"], /* CJK Ideograph */
  ["\u71B1", "\u71B1"], /* CJK Ideograph */
  ["\u71B2", "\u71B2"], /* CJK Ideograph */
  ["\u71B3", "\u71B3"], /* CJK Ideograph */
  ["\u71B4", "\u71B4"], /* CJK Ideograph */
  ["\u71B5", "\u71B5"], /* CJK Ideograph */
  ["\u71B6", "\u71B6"], /* CJK Ideograph */
  ["\u71B7", "\u71B7"], /* CJK Ideograph */
  ["\u71B8", "\u71B8"], /* CJK Ideograph */
  ["\u71B9", "\u71B9"], /* CJK Ideograph */
  ["\u71BA", "\u71BA"], /* CJK Ideograph */
  ["\u71BB", "\u71BB"], /* CJK Ideograph */
  ["\u71BC", "\u71BC"], /* CJK Ideograph */
  ["\u71BD", "\u71BD"], /* CJK Ideograph */
  ["\u71BE", "\u71BE"], /* CJK Ideograph */
  ["\u71BF", "\u71BF"], /* CJK Ideograph */
  ["\u71C0", "\u71C0"], /* CJK Ideograph */
  ["\u71C1", "\u71C1"], /* CJK Ideograph */
  ["\u71C2", "\u71C2"], /* CJK Ideograph */
  ["\u71C3", "\u71C3"], /* CJK Ideograph */
  ["\u71C4", "\u71C4"], /* CJK Ideograph */
  ["\u71C5", "\u71C5"], /* CJK Ideograph */
  ["\u71C6", "\u71C6"], /* CJK Ideograph */
  ["\u71C7", "\u71C7"], /* CJK Ideograph */
  ["\u71C8", "\u71C8"], /* CJK Ideograph */
  ["\u71C9", "\u71C9"], /* CJK Ideograph */
  ["\u71CA", "\u71CA"], /* CJK Ideograph */
  ["\u71CB", "\u71CB"], /* CJK Ideograph */
  ["\u71CC", "\u71CC"], /* CJK Ideograph */
  ["\u71CD", "\u71CD"], /* CJK Ideograph */
  ["\u71CE", "\u71CE"], /* CJK Ideograph */
  ["\u71CF", "\u71CF"], /* CJK Ideograph */
  ["\u71D0", "\u71D0"], /* CJK Ideograph */
  ["\u71D1", "\u71D1"], /* CJK Ideograph */
  ["\u71D2", "\u71D2"], /* CJK Ideograph */
  ["\u71D3", "\u71D3"], /* CJK Ideograph */
  ["\u71D4", "\u71D4"], /* CJK Ideograph */
  ["\u71D5", "\u71D5"], /* CJK Ideograph */
  ["\u71D6", "\u71D6"], /* CJK Ideograph */
  ["\u71D7", "\u71D7"], /* CJK Ideograph */
  ["\u71D8", "\u71D8"], /* CJK Ideograph */
  ["\u71D9", "\u71D9"], /* CJK Ideograph */
  ["\u71DA", "\u71DA"], /* CJK Ideograph */
  ["\u71DB", "\u71DB"], /* CJK Ideograph */
  ["\u71DC", "\u71DC"], /* CJK Ideograph */
  ["\u71DD", "\u71DD"], /* CJK Ideograph */
  ["\u71DE", "\u71DE"], /* CJK Ideograph */
  ["\u71DF", "\u71DF"], /* CJK Ideograph */
  ["\u71E0", "\u71E0"], /* CJK Ideograph */
  ["\u71E1", "\u71E1"], /* CJK Ideograph */
  ["\u71E2", "\u71E2"], /* CJK Ideograph */
  ["\u71E3", "\u71E3"], /* CJK Ideograph */
  ["\u71E4", "\u71E4"], /* CJK Ideograph */
  ["\u71E5", "\u71E5"], /* CJK Ideograph */
  ["\u71E6", "\u71E6"], /* CJK Ideograph */
  ["\u71E7", "\u71E7"], /* CJK Ideograph */
  ["\u71E8", "\u71E8"], /* CJK Ideograph */
  ["\u71E9", "\u71E9"], /* CJK Ideograph */
  ["\u71EA", "\u71EA"], /* CJK Ideograph */
  ["\u71EB", "\u71EB"], /* CJK Ideograph */
  ["\u71EC", "\u71EC"], /* CJK Ideograph */
  ["\u71ED", "\u71ED"], /* CJK Ideograph */
  ["\u71EE", "\u71EE"], /* CJK Ideograph */
  ["\u71EF", "\u71EF"], /* CJK Ideograph */
  ["\u71F0", "\u71F0"], /* CJK Ideograph */
  ["\u71F1", "\u71F1"], /* CJK Ideograph */
  ["\u71F2", "\u71F2"], /* CJK Ideograph */
  ["\u71F3", "\u71F3"], /* CJK Ideograph */
  ["\u71F4", "\u71F4"], /* CJK Ideograph */
  ["\u71F5", "\u71F5"], /* CJK Ideograph */
  ["\u71F6", "\u71F6"], /* CJK Ideograph */
  ["\u71F7", "\u71F7"], /* CJK Ideograph */
  ["\u71F8", "\u71F8"], /* CJK Ideograph */
  ["\u71F9", "\u71F9"], /* CJK Ideograph */
  ["\u71FA", "\u71FA"], /* CJK Ideograph */
  ["\u71FB", "\u71FB"], /* CJK Ideograph */
  ["\u71FC", "\u71FC"], /* CJK Ideograph */
  ["\u71FD", "\u71FD"], /* CJK Ideograph */
  ["\u71FE", "\u71FE"], /* CJK Ideograph */
  ["\u71FF", "\u71FF"], /* CJK Ideograph */
  ["\u7200", "\u7200"], /* CJK Ideograph */
  ["\u7201", "\u7201"], /* CJK Ideograph */
  ["\u7202", "\u7202"], /* CJK Ideograph */
  ["\u7203", "\u7203"], /* CJK Ideograph */
  ["\u7204", "\u7204"], /* CJK Ideograph */
  ["\u7205", "\u7205"], /* CJK Ideograph */
  ["\u7206", "\u7206"], /* CJK Ideograph */
  ["\u7207", "\u7207"], /* CJK Ideograph */
  ["\u7208", "\u7208"], /* CJK Ideograph */
  ["\u7209", "\u7209"], /* CJK Ideograph */
  ["\u720A", "\u720A"], /* CJK Ideograph */
  ["\u720B", "\u720B"], /* CJK Ideograph */
  ["\u720C", "\u720C"], /* CJK Ideograph */
  ["\u720D", "\u720D"], /* CJK Ideograph */
  ["\u720E", "\u720E"], /* CJK Ideograph */
  ["\u720F", "\u720F"], /* CJK Ideograph */
  ["\u7210", "\u7210"], /* CJK Ideograph */
  ["\u7211", "\u7211"], /* CJK Ideograph */
  ["\u7212", "\u7212"], /* CJK Ideograph */
  ["\u7213", "\u7213"], /* CJK Ideograph */
  ["\u7214", "\u7214"], /* CJK Ideograph */
  ["\u7215", "\u7215"], /* CJK Ideograph */
  ["\u7216", "\u7216"], /* CJK Ideograph */
  ["\u7217", "\u7217"], /* CJK Ideograph */
  ["\u7218", "\u7218"], /* CJK Ideograph */
  ["\u7219", "\u7219"], /* CJK Ideograph */
  ["\u721A", "\u721A"], /* CJK Ideograph */
  ["\u721B", "\u721B"], /* CJK Ideograph */
  ["\u721C", "\u721C"], /* CJK Ideograph */
  ["\u721D", "\u721D"], /* CJK Ideograph */
  ["\u721E", "\u721E"], /* CJK Ideograph */
  ["\u721F", "\u721F"], /* CJK Ideograph */
  ["\u7220", "\u7220"], /* CJK Ideograph */
  ["\u7221", "\u7221"], /* CJK Ideograph */
  ["\u7222", "\u7222"], /* CJK Ideograph */
  ["\u7223", "\u7223"], /* CJK Ideograph */
  ["\u7224", "\u7224"], /* CJK Ideograph */
  ["\u7225", "\u7225"], /* CJK Ideograph */
  ["\u7226", "\u7226"], /* CJK Ideograph */
  ["\u7227", "\u7227"], /* CJK Ideograph */
  ["\u7228", "\u7228"], /* CJK Ideograph */
  ["\u7229", "\u7229"], /* CJK Ideograph */
  ["\u722A", "\u722A"], /* CJK Ideograph */
  ["\u722B", "\u722B"], /* CJK Ideograph */
  ["\u722C", "\u722C"], /* CJK Ideograph */
  ["\u722D", "\u722D"], /* CJK Ideograph */
  ["\u722E", "\u722E"], /* CJK Ideograph */
  ["\u722F", "\u722F"], /* CJK Ideograph */
  ["\u7230", "\u7230"], /* CJK Ideograph */
  ["\u7231", "\u7231"], /* CJK Ideograph */
  ["\u7232", "\u7232"], /* CJK Ideograph */
  ["\u7233", "\u7233"], /* CJK Ideograph */
  ["\u7234", "\u7234"], /* CJK Ideograph */
  ["\u7235", "\u7235"], /* CJK Ideograph */
  ["\u7236", "\u7236"], /* CJK Ideograph */
  ["\u7237", "\u7237"], /* CJK Ideograph */
  ["\u7238", "\u7238"], /* CJK Ideograph */
  ["\u7239", "\u7239"], /* CJK Ideograph */
  ["\u723A", "\u723A"], /* CJK Ideograph */
  ["\u723B", "\u723B"], /* CJK Ideograph */
  ["\u723C", "\u723C"], /* CJK Ideograph */
  ["\u723D", "\u723D"], /* CJK Ideograph */
  ["\u723E", "\u723E"], /* CJK Ideograph */
  ["\u723F", "\u723F"], /* CJK Ideograph */
  ["\u7240", "\u7240"], /* CJK Ideograph */
  ["\u7241", "\u7241"], /* CJK Ideograph */
  ["\u7242", "\u7242"], /* CJK Ideograph */
  ["\u7243", "\u7243"], /* CJK Ideograph */
  ["\u7244", "\u7244"], /* CJK Ideograph */
  ["\u7245", "\u7245"], /* CJK Ideograph */
  ["\u7246", "\u7246"], /* CJK Ideograph */
  ["\u7247", "\u7247"], /* CJK Ideograph */
  ["\u7248", "\u7248"], /* CJK Ideograph */
  ["\u7249", "\u7249"], /* CJK Ideograph */
  ["\u724A", "\u724A"], /* CJK Ideograph */
  ["\u724B", "\u724B"], /* CJK Ideograph */
  ["\u724C", "\u724C"], /* CJK Ideograph */
  ["\u724D", "\u724D"], /* CJK Ideograph */
  ["\u724E", "\u724E"], /* CJK Ideograph */
  ["\u724F", "\u724F"], /* CJK Ideograph */
  ["\u7250", "\u7250"], /* CJK Ideograph */
  ["\u7251", "\u7251"], /* CJK Ideograph */
  ["\u7252", "\u7252"], /* CJK Ideograph */
  ["\u7253", "\u7253"], /* CJK Ideograph */
  ["\u7254", "\u7254"], /* CJK Ideograph */
  ["\u7255", "\u7255"], /* CJK Ideograph */
  ["\u7256", "\u7256"], /* CJK Ideograph */
  ["\u7257", "\u7257"], /* CJK Ideograph */
  ["\u7258", "\u7258"], /* CJK Ideograph */
  ["\u7259", "\u7259"], /* CJK Ideograph */
  ["\u725A", "\u725A"], /* CJK Ideograph */
  ["\u725B", "\u725B"], /* CJK Ideograph */
  ["\u725C", "\u725C"], /* CJK Ideograph */
  ["\u725D", "\u725D"], /* CJK Ideograph */
  ["\u725E", "\u725E"], /* CJK Ideograph */
  ["\u725F", "\u725F"], /* CJK Ideograph */
  ["\u7260", "\u7260"], /* CJK Ideograph */
  ["\u7261", "\u7261"], /* CJK Ideograph */
  ["\u7262", "\u7262"], /* CJK Ideograph */
  ["\u7263", "\u7263"], /* CJK Ideograph */
  ["\u7264", "\u7264"], /* CJK Ideograph */
  ["\u7265", "\u7265"], /* CJK Ideograph */
  ["\u7266", "\u7266"], /* CJK Ideograph */
  ["\u7267", "\u7267"], /* CJK Ideograph */
  ["\u7268", "\u7268"], /* CJK Ideograph */
  ["\u7269", "\u7269"], /* CJK Ideograph */
  ["\u726A", "\u726A"], /* CJK Ideograph */
  ["\u726B", "\u726B"], /* CJK Ideograph */
  ["\u726C", "\u726C"], /* CJK Ideograph */
  ["\u726D", "\u726D"], /* CJK Ideograph */
  ["\u726E", "\u726E"], /* CJK Ideograph */
  ["\u726F", "\u726F"], /* CJK Ideograph */
  ["\u7270", "\u7270"], /* CJK Ideograph */
  ["\u7271", "\u7271"], /* CJK Ideograph */
  ["\u7272", "\u7272"], /* CJK Ideograph */
  ["\u7273", "\u7273"], /* CJK Ideograph */
  ["\u7274", "\u7274"], /* CJK Ideograph */
  ["\u7275", "\u7275"], /* CJK Ideograph */
  ["\u7276", "\u7276"], /* CJK Ideograph */
  ["\u7277", "\u7277"], /* CJK Ideograph */
  ["\u7278", "\u7278"], /* CJK Ideograph */
  ["\u7279", "\u7279"], /* CJK Ideograph */
  ["\u727A", "\u727A"], /* CJK Ideograph */
  ["\u727B", "\u727B"], /* CJK Ideograph */
  ["\u727C", "\u727C"], /* CJK Ideograph */
  ["\u727D", "\u727D"], /* CJK Ideograph */
  ["\u727E", "\u727E"], /* CJK Ideograph */
  ["\u727F", "\u727F"], /* CJK Ideograph */
  ["\u7280", "\u7280"], /* CJK Ideograph */
  ["\u7281", "\u7281"], /* CJK Ideograph */
  ["\u7282", "\u7282"], /* CJK Ideograph */
  ["\u7283", "\u7283"], /* CJK Ideograph */
  ["\u7284", "\u7284"], /* CJK Ideograph */
  ["\u7285", "\u7285"], /* CJK Ideograph */
  ["\u7286", "\u7286"], /* CJK Ideograph */
  ["\u7287", "\u7287"], /* CJK Ideograph */
  ["\u7288", "\u7288"], /* CJK Ideograph */
  ["\u7289", "\u7289"], /* CJK Ideograph */
  ["\u728A", "\u728A"], /* CJK Ideograph */
  ["\u728B", "\u728B"], /* CJK Ideograph */
  ["\u728C", "\u728C"], /* CJK Ideograph */
  ["\u728D", "\u728D"], /* CJK Ideograph */
  ["\u728E", "\u728E"], /* CJK Ideograph */
  ["\u728F", "\u728F"], /* CJK Ideograph */
  ["\u7290", "\u7290"], /* CJK Ideograph */
  ["\u7291", "\u7291"], /* CJK Ideograph */
  ["\u7292", "\u7292"], /* CJK Ideograph */
  ["\u7293", "\u7293"], /* CJK Ideograph */
  ["\u7294", "\u7294"], /* CJK Ideograph */
  ["\u7295", "\u7295"], /* CJK Ideograph */
  ["\u7296", "\u7296"], /* CJK Ideograph */
  ["\u7297", "\u7297"], /* CJK Ideograph */
  ["\u7298", "\u7298"], /* CJK Ideograph */
  ["\u7299", "\u7299"], /* CJK Ideograph */
  ["\u729A", "\u729A"], /* CJK Ideograph */
  ["\u729B", "\u729B"], /* CJK Ideograph */
  ["\u729C", "\u729C"], /* CJK Ideograph */
  ["\u729D", "\u729D"], /* CJK Ideograph */
  ["\u729E", "\u729E"], /* CJK Ideograph */
  ["\u729F", "\u729F"], /* CJK Ideograph */
  ["\u72A0", "\u72A0"], /* CJK Ideograph */
  ["\u72A1", "\u72A1"], /* CJK Ideograph */
  ["\u72A2", "\u72A2"], /* CJK Ideograph */
  ["\u72A3", "\u72A3"], /* CJK Ideograph */
  ["\u72A4", "\u72A4"], /* CJK Ideograph */
  ["\u72A5", "\u72A5"], /* CJK Ideograph */
  ["\u72A6", "\u72A6"], /* CJK Ideograph */
  ["\u72A7", "\u72A7"], /* CJK Ideograph */
  ["\u72A8", "\u72A8"], /* CJK Ideograph */
  ["\u72A9", "\u72A9"], /* CJK Ideograph */
  ["\u72AA", "\u72AA"], /* CJK Ideograph */
  ["\u72AB", "\u72AB"], /* CJK Ideograph */
  ["\u72AC", "\u72AC"], /* CJK Ideograph */
  ["\u72AD", "\u72AD"], /* CJK Ideograph */
  ["\u72AE", "\u72AE"], /* CJK Ideograph */
  ["\u72AF", "\u72AF"], /* CJK Ideograph */
  ["\u72B0", "\u72B0"], /* CJK Ideograph */
  ["\u72B1", "\u72B1"], /* CJK Ideograph */
  ["\u72B2", "\u72B2"], /* CJK Ideograph */
  ["\u72B3", "\u72B3"], /* CJK Ideograph */
  ["\u72B4", "\u72B4"], /* CJK Ideograph */
  ["\u72B5", "\u72B5"], /* CJK Ideograph */
  ["\u72B6", "\u72B6"], /* CJK Ideograph */
  ["\u72B7", "\u72B7"], /* CJK Ideograph */
  ["\u72B8", "\u72B8"], /* CJK Ideograph */
  ["\u72B9", "\u72B9"], /* CJK Ideograph */
  ["\u72BA", "\u72BA"], /* CJK Ideograph */
  ["\u72BB", "\u72BB"], /* CJK Ideograph */
  ["\u72BC", "\u72BC"], /* CJK Ideograph */
  ["\u72BD", "\u72BD"], /* CJK Ideograph */
  ["\u72BE", "\u72BE"], /* CJK Ideograph */
  ["\u72BF", "\u72BF"], /* CJK Ideograph */
  ["\u72C0", "\u72C0"], /* CJK Ideograph */
  ["\u72C1", "\u72C1"], /* CJK Ideograph */
  ["\u72C2", "\u72C2"], /* CJK Ideograph */
  ["\u72C3", "\u72C3"], /* CJK Ideograph */
  ["\u72C4", "\u72C4"], /* CJK Ideograph */
  ["\u72C5", "\u72C5"], /* CJK Ideograph */
  ["\u72C6", "\u72C6"], /* CJK Ideograph */
  ["\u72C7", "\u72C7"], /* CJK Ideograph */
  ["\u72C8", "\u72C8"], /* CJK Ideograph */
  ["\u72C9", "\u72C9"], /* CJK Ideograph */
  ["\u72CA", "\u72CA"], /* CJK Ideograph */
  ["\u72CB", "\u72CB"], /* CJK Ideograph */
  ["\u72CC", "\u72CC"], /* CJK Ideograph */
  ["\u72CD", "\u72CD"], /* CJK Ideograph */
  ["\u72CE", "\u72CE"], /* CJK Ideograph */
  ["\u72CF", "\u72CF"], /* CJK Ideograph */
  ["\u72D0", "\u72D0"], /* CJK Ideograph */
  ["\u72D1", "\u72D1"], /* CJK Ideograph */
  ["\u72D2", "\u72D2"], /* CJK Ideograph */
  ["\u72D3", "\u72D3"], /* CJK Ideograph */
  ["\u72D4", "\u72D4"], /* CJK Ideograph */
  ["\u72D5", "\u72D5"], /* CJK Ideograph */
  ["\u72D6", "\u72D6"], /* CJK Ideograph */
  ["\u72D7", "\u72D7"], /* CJK Ideograph */
  ["\u72D8", "\u72D8"], /* CJK Ideograph */
  ["\u72D9", "\u72D9"], /* CJK Ideograph */
  ["\u72DA", "\u72DA"], /* CJK Ideograph */
  ["\u72DB", "\u72DB"], /* CJK Ideograph */
  ["\u72DC", "\u72DC"], /* CJK Ideograph */
  ["\u72DD", "\u72DD"], /* CJK Ideograph */
  ["\u72DE", "\u72DE"], /* CJK Ideograph */
  ["\u72DF", "\u72DF"], /* CJK Ideograph */
  ["\u72E0", "\u72E0"], /* CJK Ideograph */
  ["\u72E1", "\u72E1"], /* CJK Ideograph */
  ["\u72E2", "\u72E2"], /* CJK Ideograph */
  ["\u72E3", "\u72E3"], /* CJK Ideograph */
  ["\u72E4", "\u72E4"], /* CJK Ideograph */
  ["\u72E5", "\u72E5"], /* CJK Ideograph */
  ["\u72E6", "\u72E6"], /* CJK Ideograph */
  ["\u72E7", "\u72E7"], /* CJK Ideograph */
  ["\u72E8", "\u72E8"], /* CJK Ideograph */
  ["\u72E9", "\u72E9"], /* CJK Ideograph */
  ["\u72EA", "\u72EA"], /* CJK Ideograph */
  ["\u72EB", "\u72EB"], /* CJK Ideograph */
  ["\u72EC", "\u72EC"], /* CJK Ideograph */
  ["\u72ED", "\u72ED"], /* CJK Ideograph */
  ["\u72EE", "\u72EE"], /* CJK Ideograph */
  ["\u72EF", "\u72EF"], /* CJK Ideograph */
  ["\u72F0", "\u72F0"], /* CJK Ideograph */
  ["\u72F1", "\u72F1"], /* CJK Ideograph */
  ["\u72F2", "\u72F2"], /* CJK Ideograph */
  ["\u72F3", "\u72F3"], /* CJK Ideograph */
  ["\u72F4", "\u72F4"], /* CJK Ideograph */
  ["\u72F5", "\u72F5"], /* CJK Ideograph */
  ["\u72F6", "\u72F6"], /* CJK Ideograph */
  ["\u72F7", "\u72F7"], /* CJK Ideograph */
  ["\u72F8", "\u72F8"], /* CJK Ideograph */
  ["\u72F9", "\u72F9"], /* CJK Ideograph */
  ["\u72FA", "\u72FA"], /* CJK Ideograph */
  ["\u72FB", "\u72FB"], /* CJK Ideograph */
  ["\u72FC", "\u72FC"], /* CJK Ideograph */
  ["\u72FD", "\u72FD"], /* CJK Ideograph */
  ["\u72FE", "\u72FE"], /* CJK Ideograph */
  ["\u72FF", "\u72FF"], /* CJK Ideograph */
  ["\u7300", "\u7300"], /* CJK Ideograph */
  ["\u7301", "\u7301"], /* CJK Ideograph */
  ["\u7302", "\u7302"], /* CJK Ideograph */
  ["\u7303", "\u7303"], /* CJK Ideograph */
  ["\u7304", "\u7304"], /* CJK Ideograph */
  ["\u7305", "\u7305"], /* CJK Ideograph */
  ["\u7306", "\u7306"], /* CJK Ideograph */
  ["\u7307", "\u7307"], /* CJK Ideograph */
  ["\u7308", "\u7308"], /* CJK Ideograph */
  ["\u7309", "\u7309"], /* CJK Ideograph */
  ["\u730A", "\u730A"], /* CJK Ideograph */
  ["\u730B", "\u730B"], /* CJK Ideograph */
  ["\u730C", "\u730C"], /* CJK Ideograph */
  ["\u730D", "\u730D"], /* CJK Ideograph */
  ["\u730E", "\u730E"], /* CJK Ideograph */
  ["\u730F", "\u730F"], /* CJK Ideograph */
  ["\u7310", "\u7310"], /* CJK Ideograph */
  ["\u7311", "\u7311"], /* CJK Ideograph */
  ["\u7312", "\u7312"], /* CJK Ideograph */
  ["\u7313", "\u7313"], /* CJK Ideograph */
  ["\u7314", "\u7314"], /* CJK Ideograph */
  ["\u7315", "\u7315"], /* CJK Ideograph */
  ["\u7316", "\u7316"], /* CJK Ideograph */
  ["\u7317", "\u7317"], /* CJK Ideograph */
  ["\u7318", "\u7318"], /* CJK Ideograph */
  ["\u7319", "\u7319"], /* CJK Ideograph */
  ["\u731A", "\u731A"], /* CJK Ideograph */
  ["\u731B", "\u731B"], /* CJK Ideograph */
  ["\u731C", "\u731C"], /* CJK Ideograph */
  ["\u731D", "\u731D"], /* CJK Ideograph */
  ["\u731E", "\u731E"], /* CJK Ideograph */
  ["\u731F", "\u731F"], /* CJK Ideograph */
  ["\u7320", "\u7320"], /* CJK Ideograph */
  ["\u7321", "\u7321"], /* CJK Ideograph */
  ["\u7322", "\u7322"], /* CJK Ideograph */
  ["\u7323", "\u7323"], /* CJK Ideograph */
  ["\u7324", "\u7324"], /* CJK Ideograph */
  ["\u7325", "\u7325"], /* CJK Ideograph */
  ["\u7326", "\u7326"], /* CJK Ideograph */
  ["\u7327", "\u7327"], /* CJK Ideograph */
  ["\u7328", "\u7328"], /* CJK Ideograph */
  ["\u7329", "\u7329"], /* CJK Ideograph */
  ["\u732A", "\u732A"], /* CJK Ideograph */
  ["\u732B", "\u732B"], /* CJK Ideograph */
  ["\u732C", "\u732C"], /* CJK Ideograph */
  ["\u732D", "\u732D"], /* CJK Ideograph */
  ["\u732E", "\u732E"], /* CJK Ideograph */
  ["\u732F", "\u732F"], /* CJK Ideograph */
  ["\u7330", "\u7330"], /* CJK Ideograph */
  ["\u7331", "\u7331"], /* CJK Ideograph */
  ["\u7332", "\u7332"], /* CJK Ideograph */
  ["\u7333", "\u7333"], /* CJK Ideograph */
  ["\u7334", "\u7334"], /* CJK Ideograph */
  ["\u7335", "\u7335"], /* CJK Ideograph */
  ["\u7336", "\u7336"], /* CJK Ideograph */
  ["\u7337", "\u7337"], /* CJK Ideograph */
  ["\u7338", "\u7338"], /* CJK Ideograph */
  ["\u7339", "\u7339"], /* CJK Ideograph */
  ["\u733A", "\u733A"], /* CJK Ideograph */
  ["\u733B", "\u733B"], /* CJK Ideograph */
  ["\u733C", "\u733C"], /* CJK Ideograph */
  ["\u733D", "\u733D"], /* CJK Ideograph */
  ["\u733E", "\u733E"], /* CJK Ideograph */
  ["\u733F", "\u733F"], /* CJK Ideograph */
  ["\u7340", "\u7340"], /* CJK Ideograph */
  ["\u7341", "\u7341"], /* CJK Ideograph */
  ["\u7342", "\u7342"], /* CJK Ideograph */
  ["\u7343", "\u7343"], /* CJK Ideograph */
  ["\u7344", "\u7344"], /* CJK Ideograph */
  ["\u7345", "\u7345"], /* CJK Ideograph */
  ["\u7346", "\u7346"], /* CJK Ideograph */
  ["\u7347", "\u7347"], /* CJK Ideograph */
  ["\u7348", "\u7348"], /* CJK Ideograph */
  ["\u7349", "\u7349"], /* CJK Ideograph */
  ["\u734A", "\u734A"], /* CJK Ideograph */
  ["\u734B", "\u734B"], /* CJK Ideograph */
  ["\u734C", "\u734C"], /* CJK Ideograph */
  ["\u734D", "\u734D"], /* CJK Ideograph */
  ["\u734E", "\u734E"], /* CJK Ideograph */
  ["\u734F", "\u734F"], /* CJK Ideograph */
  ["\u7350", "\u7350"], /* CJK Ideograph */
  ["\u7351", "\u7351"], /* CJK Ideograph */
  ["\u7352", "\u7352"], /* CJK Ideograph */
  ["\u7353", "\u7353"], /* CJK Ideograph */
  ["\u7354", "\u7354"], /* CJK Ideograph */
  ["\u7355", "\u7355"], /* CJK Ideograph */
  ["\u7356", "\u7356"], /* CJK Ideograph */
  ["\u7357", "\u7357"], /* CJK Ideograph */
  ["\u7358", "\u7358"], /* CJK Ideograph */
  ["\u7359", "\u7359"], /* CJK Ideograph */
  ["\u735A", "\u735A"], /* CJK Ideograph */
  ["\u735B", "\u735B"], /* CJK Ideograph */
  ["\u735C", "\u735C"], /* CJK Ideograph */
  ["\u735D", "\u735D"], /* CJK Ideograph */
  ["\u735E", "\u735E"], /* CJK Ideograph */
  ["\u735F", "\u735F"], /* CJK Ideograph */
  ["\u7360", "\u7360"], /* CJK Ideograph */
  ["\u7361", "\u7361"], /* CJK Ideograph */
  ["\u7362", "\u7362"], /* CJK Ideograph */
  ["\u7363", "\u7363"], /* CJK Ideograph */
  ["\u7364", "\u7364"], /* CJK Ideograph */
  ["\u7365", "\u7365"], /* CJK Ideograph */
  ["\u7366", "\u7366"], /* CJK Ideograph */
  ["\u7367", "\u7367"], /* CJK Ideograph */
  ["\u7368", "\u7368"], /* CJK Ideograph */
  ["\u7369", "\u7369"], /* CJK Ideograph */
  ["\u736A", "\u736A"], /* CJK Ideograph */
  ["\u736B", "\u736B"], /* CJK Ideograph */
  ["\u736C", "\u736C"], /* CJK Ideograph */
  ["\u736D", "\u736D"], /* CJK Ideograph */
  ["\u736E", "\u736E"], /* CJK Ideograph */
  ["\u736F", "\u736F"], /* CJK Ideograph */
  ["\u7370", "\u7370"], /* CJK Ideograph */
  ["\u7371", "\u7371"], /* CJK Ideograph */
  ["\u7372", "\u7372"], /* CJK Ideograph */
  ["\u7373", "\u7373"], /* CJK Ideograph */
  ["\u7374", "\u7374"], /* CJK Ideograph */
  ["\u7375", "\u7375"], /* CJK Ideograph */
  ["\u7376", "\u7376"], /* CJK Ideograph */
  ["\u7377", "\u7377"], /* CJK Ideograph */
  ["\u7378", "\u7378"], /* CJK Ideograph */
  ["\u7379", "\u7379"], /* CJK Ideograph */
  ["\u737A", "\u737A"], /* CJK Ideograph */
  ["\u737B", "\u737B"], /* CJK Ideograph */
  ["\u737C", "\u737C"], /* CJK Ideograph */
  ["\u737D", "\u737D"], /* CJK Ideograph */
  ["\u737E", "\u737E"], /* CJK Ideograph */
  ["\u737F", "\u737F"], /* CJK Ideograph */
  ["\u7380", "\u7380"], /* CJK Ideograph */
  ["\u7381", "\u7381"], /* CJK Ideograph */
  ["\u7382", "\u7382"], /* CJK Ideograph */
  ["\u7383", "\u7383"], /* CJK Ideograph */
  ["\u7384", "\u7384"], /* CJK Ideograph */
  ["\u7385", "\u7385"], /* CJK Ideograph */
  ["\u7386", "\u7386"], /* CJK Ideograph */
  ["\u7387", "\u7387"], /* CJK Ideograph */
  ["\u7388", "\u7388"], /* CJK Ideograph */
  ["\u7389", "\u7389"], /* CJK Ideograph */
  ["\u738A", "\u738A"], /* CJK Ideograph */
  ["\u738B", "\u738B"], /* CJK Ideograph */
  ["\u738C", "\u738C"], /* CJK Ideograph */
  ["\u738D", "\u738D"], /* CJK Ideograph */
  ["\u738E", "\u738E"], /* CJK Ideograph */
  ["\u738F", "\u738F"], /* CJK Ideograph */
  ["\u7390", "\u7390"], /* CJK Ideograph */
  ["\u7391", "\u7391"], /* CJK Ideograph */
  ["\u7392", "\u7392"], /* CJK Ideograph */
  ["\u7393", "\u7393"], /* CJK Ideograph */
  ["\u7394", "\u7394"], /* CJK Ideograph */
  ["\u7395", "\u7395"], /* CJK Ideograph */
  ["\u7396", "\u7396"], /* CJK Ideograph */
  ["\u7397", "\u7397"], /* CJK Ideograph */
  ["\u7398", "\u7398"], /* CJK Ideograph */
  ["\u7399", "\u7399"], /* CJK Ideograph */
  ["\u739A", "\u739A"], /* CJK Ideograph */
  ["\u739B", "\u739B"], /* CJK Ideograph */
  ["\u739C", "\u739C"], /* CJK Ideograph */
  ["\u739D", "\u739D"], /* CJK Ideograph */
  ["\u739E", "\u739E"], /* CJK Ideograph */
  ["\u739F", "\u739F"], /* CJK Ideograph */
  ["\u73A0", "\u73A0"], /* CJK Ideograph */
  ["\u73A1", "\u73A1"], /* CJK Ideograph */
  ["\u73A2", "\u73A2"], /* CJK Ideograph */
  ["\u73A3", "\u73A3"], /* CJK Ideograph */
  ["\u73A4", "\u73A4"], /* CJK Ideograph */
  ["\u73A5", "\u73A5"], /* CJK Ideograph */
  ["\u73A6", "\u73A6"], /* CJK Ideograph */
  ["\u73A7", "\u73A7"], /* CJK Ideograph */
  ["\u73A8", "\u73A8"], /* CJK Ideograph */
  ["\u73A9", "\u73A9"], /* CJK Ideograph */
  ["\u73AA", "\u73AA"], /* CJK Ideograph */
  ["\u73AB", "\u73AB"], /* CJK Ideograph */
  ["\u73AC", "\u73AC"], /* CJK Ideograph */
  ["\u73AD", "\u73AD"], /* CJK Ideograph */
  ["\u73AE", "\u73AE"], /* CJK Ideograph */
  ["\u73AF", "\u73AF"], /* CJK Ideograph */
  ["\u73B0", "\u73B0"], /* CJK Ideograph */
  ["\u73B1", "\u73B1"], /* CJK Ideograph */
  ["\u73B2", "\u73B2"], /* CJK Ideograph */
  ["\u73B3", "\u73B3"], /* CJK Ideograph */
  ["\u73B4", "\u73B4"], /* CJK Ideograph */
  ["\u73B5", "\u73B5"], /* CJK Ideograph */
  ["\u73B6", "\u73B6"], /* CJK Ideograph */
  ["\u73B7", "\u73B7"], /* CJK Ideograph */
  ["\u73B8", "\u73B8"], /* CJK Ideograph */
  ["\u73B9", "\u73B9"], /* CJK Ideograph */
  ["\u73BA", "\u73BA"], /* CJK Ideograph */
  ["\u73BB", "\u73BB"], /* CJK Ideograph */
  ["\u73BC", "\u73BC"], /* CJK Ideograph */
  ["\u73BD", "\u73BD"], /* CJK Ideograph */
  ["\u73BE", "\u73BE"], /* CJK Ideograph */
  ["\u73BF", "\u73BF"], /* CJK Ideograph */
  ["\u73C0", "\u73C0"], /* CJK Ideograph */
  ["\u73C1", "\u73C1"], /* CJK Ideograph */
  ["\u73C2", "\u73C2"], /* CJK Ideograph */
  ["\u73C3", "\u73C3"], /* CJK Ideograph */
  ["\u73C4", "\u73C4"], /* CJK Ideograph */
  ["\u73C5", "\u73C5"], /* CJK Ideograph */
  ["\u73C6", "\u73C6"], /* CJK Ideograph */
  ["\u73C7", "\u73C7"], /* CJK Ideograph */
  ["\u73C8", "\u73C8"], /* CJK Ideograph */
  ["\u73C9", "\u73C9"], /* CJK Ideograph */
  ["\u73CA", "\u73CA"], /* CJK Ideograph */
  ["\u73CB", "\u73CB"], /* CJK Ideograph */
  ["\u73CC", "\u73CC"], /* CJK Ideograph */
  ["\u73CD", "\u73CD"], /* CJK Ideograph */
  ["\u73CE", "\u73CE"], /* CJK Ideograph */
  ["\u73CF", "\u73CF"], /* CJK Ideograph */
  ["\u73D0", "\u73D0"], /* CJK Ideograph */
  ["\u73D1", "\u73D1"], /* CJK Ideograph */
  ["\u73D2", "\u73D2"], /* CJK Ideograph */
  ["\u73D3", "\u73D3"], /* CJK Ideograph */
  ["\u73D4", "\u73D4"], /* CJK Ideograph */
  ["\u73D5", "\u73D5"], /* CJK Ideograph */
  ["\u73D6", "\u73D6"], /* CJK Ideograph */
  ["\u73D7", "\u73D7"], /* CJK Ideograph */
  ["\u73D8", "\u73D8"], /* CJK Ideograph */
  ["\u73D9", "\u73D9"], /* CJK Ideograph */
  ["\u73DA", "\u73DA"], /* CJK Ideograph */
  ["\u73DB", "\u73DB"], /* CJK Ideograph */
  ["\u73DC", "\u73DC"], /* CJK Ideograph */
  ["\u73DD", "\u73DD"], /* CJK Ideograph */
  ["\u73DE", "\u73DE"], /* CJK Ideograph */
  ["\u73DF", "\u73DF"], /* CJK Ideograph */
  ["\u73E0", "\u73E0"], /* CJK Ideograph */
  ["\u73E1", "\u73E1"], /* CJK Ideograph */
  ["\u73E2", "\u73E2"], /* CJK Ideograph */
  ["\u73E3", "\u73E3"], /* CJK Ideograph */
  ["\u73E4", "\u73E4"], /* CJK Ideograph */
  ["\u73E5", "\u73E5"], /* CJK Ideograph */
  ["\u73E6", "\u73E6"], /* CJK Ideograph */
  ["\u73E7", "\u73E7"], /* CJK Ideograph */
  ["\u73E8", "\u73E8"], /* CJK Ideograph */
  ["\u73E9", "\u73E9"], /* CJK Ideograph */
  ["\u73EA", "\u73EA"], /* CJK Ideograph */
  ["\u73EB", "\u73EB"], /* CJK Ideograph */
  ["\u73EC", "\u73EC"], /* CJK Ideograph */
  ["\u73ED", "\u73ED"], /* CJK Ideograph */
  ["\u73EE", "\u73EE"], /* CJK Ideograph */
  ["\u73EF", "\u73EF"], /* CJK Ideograph */
  ["\u73F0", "\u73F0"], /* CJK Ideograph */
  ["\u73F1", "\u73F1"], /* CJK Ideograph */
  ["\u73F2", "\u73F2"], /* CJK Ideograph */
  ["\u73F3", "\u73F3"], /* CJK Ideograph */
  ["\u73F4", "\u73F4"], /* CJK Ideograph */
  ["\u73F5", "\u73F5"], /* CJK Ideograph */
  ["\u73F6", "\u73F6"], /* CJK Ideograph */
  ["\u73F7", "\u73F7"], /* CJK Ideograph */
  ["\u73F8", "\u73F8"], /* CJK Ideograph */
  ["\u73F9", "\u73F9"], /* CJK Ideograph */
  ["\u73FA", "\u73FA"], /* CJK Ideograph */
  ["\u73FB", "\u73FB"], /* CJK Ideograph */
  ["\u73FC", "\u73FC"], /* CJK Ideograph */
  ["\u73FD", "\u73FD"], /* CJK Ideograph */
  ["\u73FE", "\u73FE"], /* CJK Ideograph */
  ["\u73FF", "\u73FF"], /* CJK Ideograph */
  ["\u7400", "\u7400"], /* CJK Ideograph */
  ["\u7401", "\u7401"], /* CJK Ideograph */
  ["\u7402", "\u7402"], /* CJK Ideograph */
  ["\u7403", "\u7403"], /* CJK Ideograph */
  ["\u7404", "\u7404"], /* CJK Ideograph */
  ["\u7405", "\u7405"], /* CJK Ideograph */
  ["\u7406", "\u7406"], /* CJK Ideograph */
  ["\u7407", "\u7407"], /* CJK Ideograph */
  ["\u7408", "\u7408"], /* CJK Ideograph */
  ["\u7409", "\u7409"], /* CJK Ideograph */
  ["\u740A", "\u740A"], /* CJK Ideograph */
  ["\u740B", "\u740B"], /* CJK Ideograph */
  ["\u740C", "\u740C"], /* CJK Ideograph */
  ["\u740D", "\u740D"], /* CJK Ideograph */
  ["\u740E", "\u740E"], /* CJK Ideograph */
  ["\u740F", "\u740F"], /* CJK Ideograph */
  ["\u7410", "\u7410"], /* CJK Ideograph */
  ["\u7411", "\u7411"], /* CJK Ideograph */
  ["\u7412", "\u7412"], /* CJK Ideograph */
  ["\u7413", "\u7413"], /* CJK Ideograph */
  ["\u7414", "\u7414"], /* CJK Ideograph */
  ["\u7415", "\u7415"], /* CJK Ideograph */
  ["\u7416", "\u7416"], /* CJK Ideograph */
  ["\u7417", "\u7417"], /* CJK Ideograph */
  ["\u7418", "\u7418"], /* CJK Ideograph */
  ["\u7419", "\u7419"], /* CJK Ideograph */
  ["\u741A", "\u741A"], /* CJK Ideograph */
  ["\u741B", "\u741B"], /* CJK Ideograph */
  ["\u741C", "\u741C"], /* CJK Ideograph */
  ["\u741D", "\u741D"], /* CJK Ideograph */
  ["\u741E", "\u741E"], /* CJK Ideograph */
  ["\u741F", "\u741F"], /* CJK Ideograph */
  ["\u7420", "\u7420"], /* CJK Ideograph */
  ["\u7421", "\u7421"], /* CJK Ideograph */
  ["\u7422", "\u7422"], /* CJK Ideograph */
  ["\u7423", "\u7423"], /* CJK Ideograph */
  ["\u7424", "\u7424"], /* CJK Ideograph */
  ["\u7425", "\u7425"], /* CJK Ideograph */
  ["\u7426", "\u7426"], /* CJK Ideograph */
  ["\u7427", "\u7427"], /* CJK Ideograph */
  ["\u7428", "\u7428"], /* CJK Ideograph */
  ["\u7429", "\u7429"], /* CJK Ideograph */
  ["\u742A", "\u742A"], /* CJK Ideograph */
  ["\u742B", "\u742B"], /* CJK Ideograph */
  ["\u742C", "\u742C"], /* CJK Ideograph */
  ["\u742D", "\u742D"], /* CJK Ideograph */
  ["\u742E", "\u742E"], /* CJK Ideograph */
  ["\u742F", "\u742F"], /* CJK Ideograph */
  ["\u7430", "\u7430"], /* CJK Ideograph */
  ["\u7431", "\u7431"], /* CJK Ideograph */
  ["\u7432", "\u7432"], /* CJK Ideograph */
  ["\u7433", "\u7433"], /* CJK Ideograph */
  ["\u7434", "\u7434"], /* CJK Ideograph */
  ["\u7435", "\u7435"], /* CJK Ideograph */
  ["\u7436", "\u7436"], /* CJK Ideograph */
  ["\u7437", "\u7437"], /* CJK Ideograph */
  ["\u7438", "\u7438"], /* CJK Ideograph */
  ["\u7439", "\u7439"], /* CJK Ideograph */
  ["\u743A", "\u743A"], /* CJK Ideograph */
  ["\u743B", "\u743B"], /* CJK Ideograph */
  ["\u743C", "\u743C"], /* CJK Ideograph */
  ["\u743D", "\u743D"], /* CJK Ideograph */
  ["\u743E", "\u743E"], /* CJK Ideograph */
  ["\u743F", "\u743F"], /* CJK Ideograph */
  ["\u7440", "\u7440"], /* CJK Ideograph */
  ["\u7441", "\u7441"], /* CJK Ideograph */
  ["\u7442", "\u7442"], /* CJK Ideograph */
  ["\u7443", "\u7443"], /* CJK Ideograph */
  ["\u7444", "\u7444"], /* CJK Ideograph */
  ["\u7445", "\u7445"], /* CJK Ideograph */
  ["\u7446", "\u7446"], /* CJK Ideograph */
  ["\u7447", "\u7447"], /* CJK Ideograph */
  ["\u7448", "\u7448"], /* CJK Ideograph */
  ["\u7449", "\u7449"], /* CJK Ideograph */
  ["\u744A", "\u744A"], /* CJK Ideograph */
  ["\u744B", "\u744B"], /* CJK Ideograph */
  ["\u744C", "\u744C"], /* CJK Ideograph */
  ["\u744D", "\u744D"], /* CJK Ideograph */
  ["\u744E", "\u744E"], /* CJK Ideograph */
  ["\u744F", "\u744F"], /* CJK Ideograph */
  ["\u7450", "\u7450"], /* CJK Ideograph */
  ["\u7451", "\u7451"], /* CJK Ideograph */
  ["\u7452", "\u7452"], /* CJK Ideograph */
  ["\u7453", "\u7453"], /* CJK Ideograph */
  ["\u7454", "\u7454"], /* CJK Ideograph */
  ["\u7455", "\u7455"], /* CJK Ideograph */
  ["\u7456", "\u7456"], /* CJK Ideograph */
  ["\u7457", "\u7457"], /* CJK Ideograph */
  ["\u7458", "\u7458"], /* CJK Ideograph */
  ["\u7459", "\u7459"], /* CJK Ideograph */
  ["\u745A", "\u745A"], /* CJK Ideograph */
  ["\u745B", "\u745B"], /* CJK Ideograph */
  ["\u745C", "\u745C"], /* CJK Ideograph */
  ["\u745D", "\u745D"], /* CJK Ideograph */
  ["\u745E", "\u745E"], /* CJK Ideograph */
  ["\u745F", "\u745F"], /* CJK Ideograph */
  ["\u7460", "\u7460"], /* CJK Ideograph */
  ["\u7461", "\u7461"], /* CJK Ideograph */
  ["\u7462", "\u7462"], /* CJK Ideograph */
  ["\u7463", "\u7463"], /* CJK Ideograph */
  ["\u7464", "\u7464"], /* CJK Ideograph */
  ["\u7465", "\u7465"], /* CJK Ideograph */
  ["\u7466", "\u7466"], /* CJK Ideograph */
  ["\u7467", "\u7467"], /* CJK Ideograph */
  ["\u7468", "\u7468"], /* CJK Ideograph */
  ["\u7469", "\u7469"], /* CJK Ideograph */
  ["\u746A", "\u746A"], /* CJK Ideograph */
  ["\u746B", "\u746B"], /* CJK Ideograph */
  ["\u746C", "\u746C"], /* CJK Ideograph */
  ["\u746D", "\u746D"], /* CJK Ideograph */
  ["\u746E", "\u746E"], /* CJK Ideograph */
  ["\u746F", "\u746F"], /* CJK Ideograph */
  ["\u7470", "\u7470"], /* CJK Ideograph */
  ["\u7471", "\u7471"], /* CJK Ideograph */
  ["\u7472", "\u7472"], /* CJK Ideograph */
  ["\u7473", "\u7473"], /* CJK Ideograph */
  ["\u7474", "\u7474"], /* CJK Ideograph */
  ["\u7475", "\u7475"], /* CJK Ideograph */
  ["\u7476", "\u7476"], /* CJK Ideograph */
  ["\u7477", "\u7477"], /* CJK Ideograph */
  ["\u7478", "\u7478"], /* CJK Ideograph */
  ["\u7479", "\u7479"], /* CJK Ideograph */
  ["\u747A", "\u747A"], /* CJK Ideograph */
  ["\u747B", "\u747B"], /* CJK Ideograph */
  ["\u747C", "\u747C"], /* CJK Ideograph */
  ["\u747D", "\u747D"], /* CJK Ideograph */
  ["\u747E", "\u747E"], /* CJK Ideograph */
  ["\u747F", "\u747F"], /* CJK Ideograph */
  ["\u7480", "\u7480"], /* CJK Ideograph */
  ["\u7481", "\u7481"], /* CJK Ideograph */
  ["\u7482", "\u7482"], /* CJK Ideograph */
  ["\u7483", "\u7483"], /* CJK Ideograph */
  ["\u7484", "\u7484"], /* CJK Ideograph */
  ["\u7485", "\u7485"], /* CJK Ideograph */
  ["\u7486", "\u7486"], /* CJK Ideograph */
  ["\u7487", "\u7487"], /* CJK Ideograph */
  ["\u7488", "\u7488"], /* CJK Ideograph */
  ["\u7489", "\u7489"], /* CJK Ideograph */
  ["\u748A", "\u748A"], /* CJK Ideograph */
  ["\u748B", "\u748B"], /* CJK Ideograph */
  ["\u748C", "\u748C"], /* CJK Ideograph */
  ["\u748D", "\u748D"], /* CJK Ideograph */
  ["\u748E", "\u748E"], /* CJK Ideograph */
  ["\u748F", "\u748F"], /* CJK Ideograph */
  ["\u7490", "\u7490"], /* CJK Ideograph */
  ["\u7491", "\u7491"], /* CJK Ideograph */
  ["\u7492", "\u7492"], /* CJK Ideograph */
  ["\u7493", "\u7493"], /* CJK Ideograph */
  ["\u7494", "\u7494"], /* CJK Ideograph */
  ["\u7495", "\u7495"], /* CJK Ideograph */
  ["\u7496", "\u7496"], /* CJK Ideograph */
  ["\u7497", "\u7497"], /* CJK Ideograph */
  ["\u7498", "\u7498"], /* CJK Ideograph */
  ["\u7499", "\u7499"], /* CJK Ideograph */
  ["\u749A", "\u749A"], /* CJK Ideograph */
  ["\u749B", "\u749B"], /* CJK Ideograph */
  ["\u749C", "\u749C"], /* CJK Ideograph */
  ["\u749D", "\u749D"], /* CJK Ideograph */
  ["\u749E", "\u749E"], /* CJK Ideograph */
  ["\u749F", "\u749F"], /* CJK Ideograph */
  ["\u74A0", "\u74A0"], /* CJK Ideograph */
  ["\u74A1", "\u74A1"], /* CJK Ideograph */
  ["\u74A2", "\u74A2"], /* CJK Ideograph */
  ["\u74A3", "\u74A3"], /* CJK Ideograph */
  ["\u74A4", "\u74A4"], /* CJK Ideograph */
  ["\u74A5", "\u74A5"], /* CJK Ideograph */
  ["\u74A6", "\u74A6"], /* CJK Ideograph */
  ["\u74A7", "\u74A7"], /* CJK Ideograph */
  ["\u74A8", "\u74A8"], /* CJK Ideograph */
  ["\u74A9", "\u74A9"], /* CJK Ideograph */
  ["\u74AA", "\u74AA"], /* CJK Ideograph */
  ["\u74AB", "\u74AB"], /* CJK Ideograph */
  ["\u74AC", "\u74AC"], /* CJK Ideograph */
  ["\u74AD", "\u74AD"], /* CJK Ideograph */
  ["\u74AE", "\u74AE"], /* CJK Ideograph */
  ["\u74AF", "\u74AF"], /* CJK Ideograph */
  ["\u74B0", "\u74B0"], /* CJK Ideograph */
  ["\u74B1", "\u74B1"], /* CJK Ideograph */
  ["\u74B2", "\u74B2"], /* CJK Ideograph */
  ["\u74B3", "\u74B3"], /* CJK Ideograph */
  ["\u74B4", "\u74B4"], /* CJK Ideograph */
  ["\u74B5", "\u74B5"], /* CJK Ideograph */
  ["\u74B6", "\u74B6"], /* CJK Ideograph */
  ["\u74B7", "\u74B7"], /* CJK Ideograph */
  ["\u74B8", "\u74B8"], /* CJK Ideograph */
  ["\u74B9", "\u74B9"], /* CJK Ideograph */
  ["\u74BA", "\u74BA"], /* CJK Ideograph */
  ["\u74BB", "\u74BB"], /* CJK Ideograph */
  ["\u74BC", "\u74BC"], /* CJK Ideograph */
  ["\u74BD", "\u74BD"], /* CJK Ideograph */
  ["\u74BE", "\u74BE"], /* CJK Ideograph */
  ["\u74BF", "\u74BF"], /* CJK Ideograph */
  ["\u74C0", "\u74C0"], /* CJK Ideograph */
  ["\u74C1", "\u74C1"], /* CJK Ideograph */
  ["\u74C2", "\u74C2"], /* CJK Ideograph */
  ["\u74C3", "\u74C3"], /* CJK Ideograph */
  ["\u74C4", "\u74C4"], /* CJK Ideograph */
  ["\u74C5", "\u74C5"], /* CJK Ideograph */
  ["\u74C6", "\u74C6"], /* CJK Ideograph */
  ["\u74C7", "\u74C7"], /* CJK Ideograph */
  ["\u74C8", "\u74C8"], /* CJK Ideograph */
  ["\u74C9", "\u74C9"], /* CJK Ideograph */
  ["\u74CA", "\u74CA"], /* CJK Ideograph */
  ["\u74CB", "\u74CB"], /* CJK Ideograph */
  ["\u74CC", "\u74CC"], /* CJK Ideograph */
  ["\u74CD", "\u74CD"], /* CJK Ideograph */
  ["\u74CE", "\u74CE"], /* CJK Ideograph */
  ["\u74CF", "\u74CF"], /* CJK Ideograph */
  ["\u74D0", "\u74D0"], /* CJK Ideograph */
  ["\u74D1", "\u74D1"], /* CJK Ideograph */
  ["\u74D2", "\u74D2"], /* CJK Ideograph */
  ["\u74D3", "\u74D3"], /* CJK Ideograph */
  ["\u74D4", "\u74D4"], /* CJK Ideograph */
  ["\u74D5", "\u74D5"], /* CJK Ideograph */
  ["\u74D6", "\u74D6"], /* CJK Ideograph */
  ["\u74D7", "\u74D7"], /* CJK Ideograph */
  ["\u74D8", "\u74D8"], /* CJK Ideograph */
  ["\u74D9", "\u74D9"], /* CJK Ideograph */
  ["\u74DA", "\u74DA"], /* CJK Ideograph */
  ["\u74DB", "\u74DB"], /* CJK Ideograph */
  ["\u74DC", "\u74DC"], /* CJK Ideograph */
  ["\u74DD", "\u74DD"], /* CJK Ideograph */
  ["\u74DE", "\u74DE"], /* CJK Ideograph */
  ["\u74DF", "\u74DF"], /* CJK Ideograph */
  ["\u74E0", "\u74E0"], /* CJK Ideograph */
  ["\u74E1", "\u74E1"], /* CJK Ideograph */
  ["\u74E2", "\u74E2"], /* CJK Ideograph */
  ["\u74E3", "\u74E3"], /* CJK Ideograph */
  ["\u74E4", "\u74E4"], /* CJK Ideograph */
  ["\u74E5", "\u74E5"], /* CJK Ideograph */
  ["\u74E6", "\u74E6"], /* CJK Ideograph */
  ["\u74E7", "\u74E7"], /* CJK Ideograph */
  ["\u74E8", "\u74E8"], /* CJK Ideograph */
  ["\u74E9", "\u74E9"], /* CJK Ideograph */
  ["\u74EA", "\u74EA"], /* CJK Ideograph */
  ["\u74EB", "\u74EB"], /* CJK Ideograph */
  ["\u74EC", "\u74EC"], /* CJK Ideograph */
  ["\u74ED", "\u74ED"], /* CJK Ideograph */
  ["\u74EE", "\u74EE"], /* CJK Ideograph */
  ["\u74EF", "\u74EF"], /* CJK Ideograph */
  ["\u74F0", "\u74F0"], /* CJK Ideograph */
  ["\u74F1", "\u74F1"], /* CJK Ideograph */
  ["\u74F2", "\u74F2"], /* CJK Ideograph */
  ["\u74F3", "\u74F3"], /* CJK Ideograph */
  ["\u74F4", "\u74F4"], /* CJK Ideograph */
  ["\u74F5", "\u74F5"], /* CJK Ideograph */
  ["\u74F6", "\u74F6"], /* CJK Ideograph */
  ["\u74F7", "\u74F7"], /* CJK Ideograph */
  ["\u74F8", "\u74F8"], /* CJK Ideograph */
  ["\u74F9", "\u74F9"], /* CJK Ideograph */
  ["\u74FA", "\u74FA"], /* CJK Ideograph */
  ["\u74FB", "\u74FB"], /* CJK Ideograph */
  ["\u74FC", "\u74FC"], /* CJK Ideograph */
  ["\u74FD", "\u74FD"], /* CJK Ideograph */
  ["\u74FE", "\u74FE"], /* CJK Ideograph */
  ["\u74FF", "\u74FF"], /* CJK Ideograph */
  ["\u7500", "\u7500"], /* CJK Ideograph */
  ["\u7501", "\u7501"], /* CJK Ideograph */
  ["\u7502", "\u7502"], /* CJK Ideograph */
  ["\u7503", "\u7503"], /* CJK Ideograph */
  ["\u7504", "\u7504"], /* CJK Ideograph */
  ["\u7505", "\u7505"], /* CJK Ideograph */
  ["\u7506", "\u7506"], /* CJK Ideograph */
  ["\u7507", "\u7507"], /* CJK Ideograph */
  ["\u7508", "\u7508"], /* CJK Ideograph */
  ["\u7509", "\u7509"], /* CJK Ideograph */
  ["\u750A", "\u750A"], /* CJK Ideograph */
  ["\u750B", "\u750B"], /* CJK Ideograph */
  ["\u750C", "\u750C"], /* CJK Ideograph */
  ["\u750D", "\u750D"], /* CJK Ideograph */
  ["\u750E", "\u750E"], /* CJK Ideograph */
  ["\u750F", "\u750F"], /* CJK Ideograph */
  ["\u7510", "\u7510"], /* CJK Ideograph */
  ["\u7511", "\u7511"], /* CJK Ideograph */
  ["\u7512", "\u7512"], /* CJK Ideograph */
  ["\u7513", "\u7513"], /* CJK Ideograph */
  ["\u7514", "\u7514"], /* CJK Ideograph */
  ["\u7515", "\u7515"], /* CJK Ideograph */
  ["\u7516", "\u7516"], /* CJK Ideograph */
  ["\u7517", "\u7517"], /* CJK Ideograph */
  ["\u7518", "\u7518"], /* CJK Ideograph */
  ["\u7519", "\u7519"], /* CJK Ideograph */
  ["\u751A", "\u751A"], /* CJK Ideograph */
  ["\u751B", "\u751B"], /* CJK Ideograph */
  ["\u751C", "\u751C"], /* CJK Ideograph */
  ["\u751D", "\u751D"], /* CJK Ideograph */
  ["\u751E", "\u751E"], /* CJK Ideograph */
  ["\u751F", "\u751F"], /* CJK Ideograph */
  ["\u7520", "\u7520"], /* CJK Ideograph */
  ["\u7521", "\u7521"], /* CJK Ideograph */
  ["\u7522", "\u7522"], /* CJK Ideograph */
  ["\u7523", "\u7523"], /* CJK Ideograph */
  ["\u7524", "\u7524"], /* CJK Ideograph */
  ["\u7525", "\u7525"], /* CJK Ideograph */
  ["\u7526", "\u7526"], /* CJK Ideograph */
  ["\u7527", "\u7527"], /* CJK Ideograph */
  ["\u7528", "\u7528"], /* CJK Ideograph */
  ["\u7529", "\u7529"], /* CJK Ideograph */
  ["\u752A", "\u752A"], /* CJK Ideograph */
  ["\u752B", "\u752B"], /* CJK Ideograph */
  ["\u752C", "\u752C"], /* CJK Ideograph */
  ["\u752D", "\u752D"], /* CJK Ideograph */
  ["\u752E", "\u752E"], /* CJK Ideograph */
  ["\u752F", "\u752F"], /* CJK Ideograph */
  ["\u7530", "\u7530"], /* CJK Ideograph */
  ["\u7531", "\u7531"], /* CJK Ideograph */
  ["\u7532", "\u7532"], /* CJK Ideograph */
  ["\u7533", "\u7533"], /* CJK Ideograph */
  ["\u7534", "\u7534"], /* CJK Ideograph */
  ["\u7535", "\u7535"], /* CJK Ideograph */
  ["\u7536", "\u7536"], /* CJK Ideograph */
  ["\u7537", "\u7537"], /* CJK Ideograph */
  ["\u7538", "\u7538"], /* CJK Ideograph */
  ["\u7539", "\u7539"], /* CJK Ideograph */
  ["\u753A", "\u753A"], /* CJK Ideograph */
  ["\u753B", "\u753B"], /* CJK Ideograph */
  ["\u753C", "\u753C"], /* CJK Ideograph */
  ["\u753D", "\u753D"], /* CJK Ideograph */
  ["\u753E", "\u753E"], /* CJK Ideograph */
  ["\u753F", "\u753F"], /* CJK Ideograph */
  ["\u7540", "\u7540"], /* CJK Ideograph */
  ["\u7541", "\u7541"], /* CJK Ideograph */
  ["\u7542", "\u7542"], /* CJK Ideograph */
  ["\u7543", "\u7543"], /* CJK Ideograph */
  ["\u7544", "\u7544"], /* CJK Ideograph */
  ["\u7545", "\u7545"], /* CJK Ideograph */
  ["\u7546", "\u7546"], /* CJK Ideograph */
  ["\u7547", "\u7547"], /* CJK Ideograph */
  ["\u7548", "\u7548"], /* CJK Ideograph */
  ["\u7549", "\u7549"], /* CJK Ideograph */
  ["\u754A", "\u754A"], /* CJK Ideograph */
  ["\u754B", "\u754B"], /* CJK Ideograph */
  ["\u754C", "\u754C"], /* CJK Ideograph */
  ["\u754D", "\u754D"], /* CJK Ideograph */
  ["\u754E", "\u754E"], /* CJK Ideograph */
  ["\u754F", "\u754F"], /* CJK Ideograph */
  ["\u7550", "\u7550"], /* CJK Ideograph */
  ["\u7551", "\u7551"], /* CJK Ideograph */
  ["\u7552", "\u7552"], /* CJK Ideograph */
  ["\u7553", "\u7553"], /* CJK Ideograph */
  ["\u7554", "\u7554"], /* CJK Ideograph */
  ["\u7555", "\u7555"], /* CJK Ideograph */
  ["\u7556", "\u7556"], /* CJK Ideograph */
  ["\u7557", "\u7557"], /* CJK Ideograph */
  ["\u7558", "\u7558"], /* CJK Ideograph */
  ["\u7559", "\u7559"], /* CJK Ideograph */
  ["\u755A", "\u755A"], /* CJK Ideograph */
  ["\u755B", "\u755B"], /* CJK Ideograph */
  ["\u755C", "\u755C"], /* CJK Ideograph */
  ["\u755D", "\u755D"], /* CJK Ideograph */
  ["\u755E", "\u755E"], /* CJK Ideograph */
  ["\u755F", "\u755F"], /* CJK Ideograph */
  ["\u7560", "\u7560"], /* CJK Ideograph */
  ["\u7561", "\u7561"], /* CJK Ideograph */
  ["\u7562", "\u7562"], /* CJK Ideograph */
  ["\u7563", "\u7563"], /* CJK Ideograph */
  ["\u7564", "\u7564"], /* CJK Ideograph */
  ["\u7565", "\u7565"], /* CJK Ideograph */
  ["\u7566", "\u7566"], /* CJK Ideograph */
  ["\u7567", "\u7567"], /* CJK Ideograph */
  ["\u7568", "\u7568"], /* CJK Ideograph */
  ["\u7569", "\u7569"], /* CJK Ideograph */
  ["\u756A", "\u756A"], /* CJK Ideograph */
  ["\u756B", "\u756B"], /* CJK Ideograph */
  ["\u756C", "\u756C"], /* CJK Ideograph */
  ["\u756D", "\u756D"], /* CJK Ideograph */
  ["\u756E", "\u756E"], /* CJK Ideograph */
  ["\u756F", "\u756F"], /* CJK Ideograph */
  ["\u7570", "\u7570"], /* CJK Ideograph */
  ["\u7571", "\u7571"], /* CJK Ideograph */
  ["\u7572", "\u7572"], /* CJK Ideograph */
  ["\u7573", "\u7573"], /* CJK Ideograph */
  ["\u7574", "\u7574"], /* CJK Ideograph */
  ["\u7575", "\u7575"], /* CJK Ideograph */
  ["\u7576", "\u7576"], /* CJK Ideograph */
  ["\u7577", "\u7577"], /* CJK Ideograph */
  ["\u7578", "\u7578"], /* CJK Ideograph */
  ["\u7579", "\u7579"], /* CJK Ideograph */
  ["\u757A", "\u757A"], /* CJK Ideograph */
  ["\u757B", "\u757B"], /* CJK Ideograph */
  ["\u757C", "\u757C"], /* CJK Ideograph */
  ["\u757D", "\u757D"], /* CJK Ideograph */
  ["\u757E", "\u757E"], /* CJK Ideograph */
  ["\u757F", "\u757F"], /* CJK Ideograph */
  ["\u7580", "\u7580"], /* CJK Ideograph */
  ["\u7581", "\u7581"], /* CJK Ideograph */
  ["\u7582", "\u7582"], /* CJK Ideograph */
  ["\u7583", "\u7583"], /* CJK Ideograph */
  ["\u7584", "\u7584"], /* CJK Ideograph */
  ["\u7585", "\u7585"], /* CJK Ideograph */
  ["\u7586", "\u7586"], /* CJK Ideograph */
  ["\u7587", "\u7587"], /* CJK Ideograph */
  ["\u7588", "\u7588"], /* CJK Ideograph */
  ["\u7589", "\u7589"], /* CJK Ideograph */
  ["\u758A", "\u758A"], /* CJK Ideograph */
  ["\u758B", "\u758B"], /* CJK Ideograph */
  ["\u758C", "\u758C"], /* CJK Ideograph */
  ["\u758D", "\u758D"], /* CJK Ideograph */
  ["\u758E", "\u758E"], /* CJK Ideograph */
  ["\u758F", "\u758F"], /* CJK Ideograph */
  ["\u7590", "\u7590"], /* CJK Ideograph */
  ["\u7591", "\u7591"], /* CJK Ideograph */
  ["\u7592", "\u7592"], /* CJK Ideograph */
  ["\u7593", "\u7593"], /* CJK Ideograph */
  ["\u7594", "\u7594"], /* CJK Ideograph */
  ["\u7595", "\u7595"], /* CJK Ideograph */
  ["\u7596", "\u7596"], /* CJK Ideograph */
  ["\u7597", "\u7597"], /* CJK Ideograph */
  ["\u7598", "\u7598"], /* CJK Ideograph */
  ["\u7599", "\u7599"], /* CJK Ideograph */
  ["\u759A", "\u759A"], /* CJK Ideograph */
  ["\u759B", "\u759B"], /* CJK Ideograph */
  ["\u759C", "\u759C"], /* CJK Ideograph */
  ["\u759D", "\u759D"], /* CJK Ideograph */
  ["\u759E", "\u759E"], /* CJK Ideograph */
  ["\u759F", "\u759F"], /* CJK Ideograph */
  ["\u75A0", "\u75A0"], /* CJK Ideograph */
  ["\u75A1", "\u75A1"], /* CJK Ideograph */
  ["\u75A2", "\u75A2"], /* CJK Ideograph */
  ["\u75A3", "\u75A3"], /* CJK Ideograph */
  ["\u75A4", "\u75A4"], /* CJK Ideograph */
  ["\u75A5", "\u75A5"], /* CJK Ideograph */
  ["\u75A6", "\u75A6"], /* CJK Ideograph */
  ["\u75A7", "\u75A7"], /* CJK Ideograph */
  ["\u75A8", "\u75A8"], /* CJK Ideograph */
  ["\u75A9", "\u75A9"], /* CJK Ideograph */
  ["\u75AA", "\u75AA"], /* CJK Ideograph */
  ["\u75AB", "\u75AB"], /* CJK Ideograph */
  ["\u75AC", "\u75AC"], /* CJK Ideograph */
  ["\u75AD", "\u75AD"], /* CJK Ideograph */
  ["\u75AE", "\u75AE"], /* CJK Ideograph */
  ["\u75AF", "\u75AF"], /* CJK Ideograph */
  ["\u75B0", "\u75B0"], /* CJK Ideograph */
  ["\u75B1", "\u75B1"], /* CJK Ideograph */
  ["\u75B2", "\u75B2"], /* CJK Ideograph */
  ["\u75B3", "\u75B3"], /* CJK Ideograph */
  ["\u75B4", "\u75B4"], /* CJK Ideograph */
  ["\u75B5", "\u75B5"], /* CJK Ideograph */
  ["\u75B6", "\u75B6"], /* CJK Ideograph */
  ["\u75B7", "\u75B7"], /* CJK Ideograph */
  ["\u75B8", "\u75B8"], /* CJK Ideograph */
  ["\u75B9", "\u75B9"], /* CJK Ideograph */
  ["\u75BA", "\u75BA"], /* CJK Ideograph */
  ["\u75BB", "\u75BB"], /* CJK Ideograph */
  ["\u75BC", "\u75BC"], /* CJK Ideograph */
  ["\u75BD", "\u75BD"], /* CJK Ideograph */
  ["\u75BE", "\u75BE"], /* CJK Ideograph */
  ["\u75BF", "\u75BF"], /* CJK Ideograph */
  ["\u75C0", "\u75C0"], /* CJK Ideograph */
  ["\u75C1", "\u75C1"], /* CJK Ideograph */
  ["\u75C2", "\u75C2"], /* CJK Ideograph */
  ["\u75C3", "\u75C3"], /* CJK Ideograph */
  ["\u75C4", "\u75C4"], /* CJK Ideograph */
  ["\u75C5", "\u75C5"], /* CJK Ideograph */
  ["\u75C6", "\u75C6"], /* CJK Ideograph */
  ["\u75C7", "\u75C7"], /* CJK Ideograph */
  ["\u75C8", "\u75C8"], /* CJK Ideograph */
  ["\u75C9", "\u75C9"], /* CJK Ideograph */
  ["\u75CA", "\u75CA"], /* CJK Ideograph */
  ["\u75CB", "\u75CB"], /* CJK Ideograph */
  ["\u75CC", "\u75CC"], /* CJK Ideograph */
  ["\u75CD", "\u75CD"], /* CJK Ideograph */
  ["\u75CE", "\u75CE"], /* CJK Ideograph */
  ["\u75CF", "\u75CF"], /* CJK Ideograph */
  ["\u75D0", "\u75D0"], /* CJK Ideograph */
  ["\u75D1", "\u75D1"], /* CJK Ideograph */
  ["\u75D2", "\u75D2"], /* CJK Ideograph */
  ["\u75D3", "\u75D3"], /* CJK Ideograph */
  ["\u75D4", "\u75D4"], /* CJK Ideograph */
  ["\u75D5", "\u75D5"], /* CJK Ideograph */
  ["\u75D6", "\u75D6"], /* CJK Ideograph */
  ["\u75D7", "\u75D7"], /* CJK Ideograph */
  ["\u75D8", "\u75D8"], /* CJK Ideograph */
  ["\u75D9", "\u75D9"], /* CJK Ideograph */
  ["\u75DA", "\u75DA"], /* CJK Ideograph */
  ["\u75DB", "\u75DB"], /* CJK Ideograph */
  ["\u75DC", "\u75DC"], /* CJK Ideograph */
  ["\u75DD", "\u75DD"], /* CJK Ideograph */
  ["\u75DE", "\u75DE"], /* CJK Ideograph */
  ["\u75DF", "\u75DF"], /* CJK Ideograph */
  ["\u75E0", "\u75E0"], /* CJK Ideograph */
  ["\u75E1", "\u75E1"], /* CJK Ideograph */
  ["\u75E2", "\u75E2"], /* CJK Ideograph */
  ["\u75E3", "\u75E3"], /* CJK Ideograph */
  ["\u75E4", "\u75E4"], /* CJK Ideograph */
  ["\u75E5", "\u75E5"], /* CJK Ideograph */
  ["\u75E6", "\u75E6"], /* CJK Ideograph */
  ["\u75E7", "\u75E7"], /* CJK Ideograph */
  ["\u75E8", "\u75E8"], /* CJK Ideograph */
  ["\u75E9", "\u75E9"], /* CJK Ideograph */
  ["\u75EA", "\u75EA"], /* CJK Ideograph */
  ["\u75EB", "\u75EB"], /* CJK Ideograph */
  ["\u75EC", "\u75EC"], /* CJK Ideograph */
  ["\u75ED", "\u75ED"], /* CJK Ideograph */
  ["\u75EE", "\u75EE"], /* CJK Ideograph */
  ["\u75EF", "\u75EF"], /* CJK Ideograph */
  ["\u75F0", "\u75F0"], /* CJK Ideograph */
  ["\u75F1", "\u75F1"], /* CJK Ideograph */
  ["\u75F2", "\u75F2"], /* CJK Ideograph */
  ["\u75F3", "\u75F3"], /* CJK Ideograph */
  ["\u75F4", "\u75F4"], /* CJK Ideograph */
  ["\u75F5", "\u75F5"], /* CJK Ideograph */
  ["\u75F6", "\u75F6"], /* CJK Ideograph */
  ["\u75F7", "\u75F7"], /* CJK Ideograph */
  ["\u75F8", "\u75F8"], /* CJK Ideograph */
  ["\u75F9", "\u75F9"], /* CJK Ideograph */
  ["\u75FA", "\u75FA"], /* CJK Ideograph */
  ["\u75FB", "\u75FB"], /* CJK Ideograph */
  ["\u75FC", "\u75FC"], /* CJK Ideograph */
  ["\u75FD", "\u75FD"], /* CJK Ideograph */
  ["\u75FE", "\u75FE"], /* CJK Ideograph */
  ["\u75FF", "\u75FF"], /* CJK Ideograph */
  ["\u7600", "\u7600"], /* CJK Ideograph */
  ["\u7601", "\u7601"], /* CJK Ideograph */
  ["\u7602", "\u7602"], /* CJK Ideograph */
  ["\u7603", "\u7603"], /* CJK Ideograph */
  ["\u7604", "\u7604"], /* CJK Ideograph */
  ["\u7605", "\u7605"], /* CJK Ideograph */
  ["\u7606", "\u7606"], /* CJK Ideograph */
  ["\u7607", "\u7607"], /* CJK Ideograph */
  ["\u7608", "\u7608"], /* CJK Ideograph */
  ["\u7609", "\u7609"], /* CJK Ideograph */
  ["\u760A", "\u760A"], /* CJK Ideograph */
  ["\u760B", "\u760B"], /* CJK Ideograph */
  ["\u760C", "\u760C"], /* CJK Ideograph */
  ["\u760D", "\u760D"], /* CJK Ideograph */
  ["\u760E", "\u760E"], /* CJK Ideograph */
  ["\u760F", "\u760F"], /* CJK Ideograph */
  ["\u7610", "\u7610"], /* CJK Ideograph */
  ["\u7611", "\u7611"], /* CJK Ideograph */
  ["\u7612", "\u7612"], /* CJK Ideograph */
  ["\u7613", "\u7613"], /* CJK Ideograph */
  ["\u7614", "\u7614"], /* CJK Ideograph */
  ["\u7615", "\u7615"], /* CJK Ideograph */
  ["\u7616", "\u7616"], /* CJK Ideograph */
  ["\u7617", "\u7617"], /* CJK Ideograph */
  ["\u7618", "\u7618"], /* CJK Ideograph */
  ["\u7619", "\u7619"], /* CJK Ideograph */
  ["\u761A", "\u761A"], /* CJK Ideograph */
  ["\u761B", "\u761B"], /* CJK Ideograph */
  ["\u761C", "\u761C"], /* CJK Ideograph */
  ["\u761D", "\u761D"], /* CJK Ideograph */
  ["\u761E", "\u761E"], /* CJK Ideograph */
  ["\u761F", "\u761F"], /* CJK Ideograph */
  ["\u7620", "\u7620"], /* CJK Ideograph */
  ["\u7621", "\u7621"], /* CJK Ideograph */
  ["\u7622", "\u7622"], /* CJK Ideograph */
  ["\u7623", "\u7623"], /* CJK Ideograph */
  ["\u7624", "\u7624"], /* CJK Ideograph */
  ["\u7625", "\u7625"], /* CJK Ideograph */
  ["\u7626", "\u7626"], /* CJK Ideograph */
  ["\u7627", "\u7627"], /* CJK Ideograph */
  ["\u7628", "\u7628"], /* CJK Ideograph */
  ["\u7629", "\u7629"], /* CJK Ideograph */
  ["\u762A", "\u762A"], /* CJK Ideograph */
  ["\u762B", "\u762B"], /* CJK Ideograph */
  ["\u762C", "\u762C"], /* CJK Ideograph */
  ["\u762D", "\u762D"], /* CJK Ideograph */
  ["\u762E", "\u762E"], /* CJK Ideograph */
  ["\u762F", "\u762F"], /* CJK Ideograph */
  ["\u7630", "\u7630"], /* CJK Ideograph */
  ["\u7631", "\u7631"], /* CJK Ideograph */
  ["\u7632", "\u7632"], /* CJK Ideograph */
  ["\u7633", "\u7633"], /* CJK Ideograph */
  ["\u7634", "\u7634"], /* CJK Ideograph */
  ["\u7635", "\u7635"], /* CJK Ideograph */
  ["\u7636", "\u7636"], /* CJK Ideograph */
  ["\u7637", "\u7637"], /* CJK Ideograph */
  ["\u7638", "\u7638"], /* CJK Ideograph */
  ["\u7639", "\u7639"], /* CJK Ideograph */
  ["\u763A", "\u763A"], /* CJK Ideograph */
  ["\u763B", "\u763B"], /* CJK Ideograph */
  ["\u763C", "\u763C"], /* CJK Ideograph */
  ["\u763D", "\u763D"], /* CJK Ideograph */
  ["\u763E", "\u763E"], /* CJK Ideograph */
  ["\u763F", "\u763F"], /* CJK Ideograph */
  ["\u7640", "\u7640"], /* CJK Ideograph */
  ["\u7641", "\u7641"], /* CJK Ideograph */
  ["\u7642", "\u7642"], /* CJK Ideograph */
  ["\u7643", "\u7643"], /* CJK Ideograph */
  ["\u7644", "\u7644"], /* CJK Ideograph */
  ["\u7645", "\u7645"], /* CJK Ideograph */
  ["\u7646", "\u7646"], /* CJK Ideograph */
  ["\u7647", "\u7647"], /* CJK Ideograph */
  ["\u7648", "\u7648"], /* CJK Ideograph */
  ["\u7649", "\u7649"], /* CJK Ideograph */
  ["\u764A", "\u764A"], /* CJK Ideograph */
  ["\u764B", "\u764B"], /* CJK Ideograph */
  ["\u764C", "\u764C"], /* CJK Ideograph */
  ["\u764D", "\u764D"], /* CJK Ideograph */
  ["\u764E", "\u764E"], /* CJK Ideograph */
  ["\u764F", "\u764F"], /* CJK Ideograph */
  ["\u7650", "\u7650"], /* CJK Ideograph */
  ["\u7651", "\u7651"], /* CJK Ideograph */
  ["\u7652", "\u7652"], /* CJK Ideograph */
  ["\u7653", "\u7653"], /* CJK Ideograph */
  ["\u7654", "\u7654"], /* CJK Ideograph */
  ["\u7655", "\u7655"], /* CJK Ideograph */
  ["\u7656", "\u7656"], /* CJK Ideograph */
  ["\u7657", "\u7657"], /* CJK Ideograph */
  ["\u7658", "\u7658"], /* CJK Ideograph */
  ["\u7659", "\u7659"], /* CJK Ideograph */
  ["\u765A", "\u765A"], /* CJK Ideograph */
  ["\u765B", "\u765B"], /* CJK Ideograph */
  ["\u765C", "\u765C"], /* CJK Ideograph */
  ["\u765D", "\u765D"], /* CJK Ideograph */
  ["\u765E", "\u765E"], /* CJK Ideograph */
  ["\u765F", "\u765F"], /* CJK Ideograph */
  ["\u7660", "\u7660"], /* CJK Ideograph */
  ["\u7661", "\u7661"], /* CJK Ideograph */
  ["\u7662", "\u7662"], /* CJK Ideograph */
  ["\u7663", "\u7663"], /* CJK Ideograph */
  ["\u7664", "\u7664"], /* CJK Ideograph */
  ["\u7665", "\u7665"], /* CJK Ideograph */
  ["\u7666", "\u7666"], /* CJK Ideograph */
  ["\u7667", "\u7667"], /* CJK Ideograph */
  ["\u7668", "\u7668"], /* CJK Ideograph */
  ["\u7669", "\u7669"], /* CJK Ideograph */
  ["\u766A", "\u766A"], /* CJK Ideograph */
  ["\u766B", "\u766B"], /* CJK Ideograph */
  ["\u766C", "\u766C"], /* CJK Ideograph */
  ["\u766D", "\u766D"], /* CJK Ideograph */
  ["\u766E", "\u766E"], /* CJK Ideograph */
  ["\u766F", "\u766F"], /* CJK Ideograph */
  ["\u7670", "\u7670"], /* CJK Ideograph */
  ["\u7671", "\u7671"], /* CJK Ideograph */
  ["\u7672", "\u7672"], /* CJK Ideograph */
  ["\u7673", "\u7673"], /* CJK Ideograph */
  ["\u7674", "\u7674"], /* CJK Ideograph */
  ["\u7675", "\u7675"], /* CJK Ideograph */
  ["\u7676", "\u7676"], /* CJK Ideograph */
  ["\u7677", "\u7677"], /* CJK Ideograph */
  ["\u7678", "\u7678"], /* CJK Ideograph */
  ["\u7679", "\u7679"], /* CJK Ideograph */
  ["\u767A", "\u767A"], /* CJK Ideograph */
  ["\u767B", "\u767B"], /* CJK Ideograph */
  ["\u767C", "\u767C"], /* CJK Ideograph */
  ["\u767D", "\u767D"], /* CJK Ideograph */
  ["\u767E", "\u767E"], /* CJK Ideograph */
  ["\u767F", "\u767F"], /* CJK Ideograph */
  ["\u7680", "\u7680"], /* CJK Ideograph */
  ["\u7681", "\u7681"], /* CJK Ideograph */
  ["\u7682", "\u7682"], /* CJK Ideograph */
  ["\u7683", "\u7683"], /* CJK Ideograph */
  ["\u7684", "\u7684"], /* CJK Ideograph */
  ["\u7685", "\u7685"], /* CJK Ideograph */
  ["\u7686", "\u7686"], /* CJK Ideograph */
  ["\u7687", "\u7687"], /* CJK Ideograph */
  ["\u7688", "\u7688"], /* CJK Ideograph */
  ["\u7689", "\u7689"], /* CJK Ideograph */
  ["\u768A", "\u768A"], /* CJK Ideograph */
  ["\u768B", "\u768B"], /* CJK Ideograph */
  ["\u768C", "\u768C"], /* CJK Ideograph */
  ["\u768D", "\u768D"], /* CJK Ideograph */
  ["\u768E", "\u768E"], /* CJK Ideograph */
  ["\u768F", "\u768F"], /* CJK Ideograph */
  ["\u7690", "\u7690"], /* CJK Ideograph */
  ["\u7691", "\u7691"], /* CJK Ideograph */
  ["\u7692", "\u7692"], /* CJK Ideograph */
  ["\u7693", "\u7693"], /* CJK Ideograph */
  ["\u7694", "\u7694"], /* CJK Ideograph */
  ["\u7695", "\u7695"], /* CJK Ideograph */
  ["\u7696", "\u7696"], /* CJK Ideograph */
  ["\u7697", "\u7697"], /* CJK Ideograph */
  ["\u7698", "\u7698"], /* CJK Ideograph */
  ["\u7699", "\u7699"], /* CJK Ideograph */
  ["\u769A", "\u769A"], /* CJK Ideograph */
  ["\u769B", "\u769B"], /* CJK Ideograph */
  ["\u769C", "\u769C"], /* CJK Ideograph */
  ["\u769D", "\u769D"], /* CJK Ideograph */
  ["\u769E", "\u769E"], /* CJK Ideograph */
  ["\u769F", "\u769F"], /* CJK Ideograph */
  ["\u76A0", "\u76A0"], /* CJK Ideograph */
  ["\u76A1", "\u76A1"], /* CJK Ideograph */
  ["\u76A2", "\u76A2"], /* CJK Ideograph */
  ["\u76A3", "\u76A3"], /* CJK Ideograph */
  ["\u76A4", "\u76A4"], /* CJK Ideograph */
  ["\u76A5", "\u76A5"], /* CJK Ideograph */
  ["\u76A6", "\u76A6"], /* CJK Ideograph */
  ["\u76A7", "\u76A7"], /* CJK Ideograph */
  ["\u76A8", "\u76A8"], /* CJK Ideograph */
  ["\u76A9", "\u76A9"], /* CJK Ideograph */
  ["\u76AA", "\u76AA"], /* CJK Ideograph */
  ["\u76AB", "\u76AB"], /* CJK Ideograph */
  ["\u76AC", "\u76AC"], /* CJK Ideograph */
  ["\u76AD", "\u76AD"], /* CJK Ideograph */
  ["\u76AE", "\u76AE"], /* CJK Ideograph */
  ["\u76AF", "\u76AF"], /* CJK Ideograph */
  ["\u76B0", "\u76B0"], /* CJK Ideograph */
  ["\u76B1", "\u76B1"], /* CJK Ideograph */
  ["\u76B2", "\u76B2"], /* CJK Ideograph */
  ["\u76B3", "\u76B3"], /* CJK Ideograph */
  ["\u76B4", "\u76B4"], /* CJK Ideograph */
  ["\u76B5", "\u76B5"], /* CJK Ideograph */
  ["\u76B6", "\u76B6"], /* CJK Ideograph */
  ["\u76B7", "\u76B7"], /* CJK Ideograph */
  ["\u76B8", "\u76B8"], /* CJK Ideograph */
  ["\u76B9", "\u76B9"], /* CJK Ideograph */
  ["\u76BA", "\u76BA"], /* CJK Ideograph */
  ["\u76BB", "\u76BB"], /* CJK Ideograph */
  ["\u76BC", "\u76BC"], /* CJK Ideograph */
  ["\u76BD", "\u76BD"], /* CJK Ideograph */
  ["\u76BE", "\u76BE"], /* CJK Ideograph */
  ["\u76BF", "\u76BF"], /* CJK Ideograph */
  ["\u76C0", "\u76C0"], /* CJK Ideograph */
  ["\u76C1", "\u76C1"], /* CJK Ideograph */
  ["\u76C2", "\u76C2"], /* CJK Ideograph */
  ["\u76C3", "\u76C3"], /* CJK Ideograph */
  ["\u76C4", "\u76C4"], /* CJK Ideograph */
  ["\u76C5", "\u76C5"], /* CJK Ideograph */
  ["\u76C6", "\u76C6"], /* CJK Ideograph */
  ["\u76C7", "\u76C7"], /* CJK Ideograph */
  ["\u76C8", "\u76C8"], /* CJK Ideograph */
  ["\u76C9", "\u76C9"], /* CJK Ideograph */
  ["\u76CA", "\u76CA"], /* CJK Ideograph */
  ["\u76CB", "\u76CB"], /* CJK Ideograph */
  ["\u76CC", "\u76CC"], /* CJK Ideograph */
  ["\u76CD", "\u76CD"], /* CJK Ideograph */
  ["\u76CE", "\u76CE"], /* CJK Ideograph */
  ["\u76CF", "\u76CF"], /* CJK Ideograph */
  ["\u76D0", "\u76D0"], /* CJK Ideograph */
  ["\u76D1", "\u76D1"], /* CJK Ideograph */
  ["\u76D2", "\u76D2"], /* CJK Ideograph */
  ["\u76D3", "\u76D3"], /* CJK Ideograph */
  ["\u76D4", "\u76D4"], /* CJK Ideograph */
  ["\u76D5", "\u76D5"], /* CJK Ideograph */
  ["\u76D6", "\u76D6"], /* CJK Ideograph */
  ["\u76D7", "\u76D7"], /* CJK Ideograph */
  ["\u76D8", "\u76D8"], /* CJK Ideograph */
  ["\u76D9", "\u76D9"], /* CJK Ideograph */
  ["\u76DA", "\u76DA"], /* CJK Ideograph */
  ["\u76DB", "\u76DB"], /* CJK Ideograph */
  ["\u76DC", "\u76DC"], /* CJK Ideograph */
  ["\u76DD", "\u76DD"], /* CJK Ideograph */
  ["\u76DE", "\u76DE"], /* CJK Ideograph */
  ["\u76DF", "\u76DF"], /* CJK Ideograph */
  ["\u76E0", "\u76E0"], /* CJK Ideograph */
  ["\u76E1", "\u76E1"], /* CJK Ideograph */
  ["\u76E2", "\u76E2"], /* CJK Ideograph */
  ["\u76E3", "\u76E3"], /* CJK Ideograph */
  ["\u76E4", "\u76E4"], /* CJK Ideograph */
  ["\u76E5", "\u76E5"], /* CJK Ideograph */
  ["\u76E6", "\u76E6"], /* CJK Ideograph */
  ["\u76E7", "\u76E7"], /* CJK Ideograph */
  ["\u76E8", "\u76E8"], /* CJK Ideograph */
  ["\u76E9", "\u76E9"], /* CJK Ideograph */
  ["\u76EA", "\u76EA"], /* CJK Ideograph */
  ["\u76EB", "\u76EB"], /* CJK Ideograph */
  ["\u76EC", "\u76EC"], /* CJK Ideograph */
  ["\u76ED", "\u76ED"], /* CJK Ideograph */
  ["\u76EE", "\u76EE"], /* CJK Ideograph */
  ["\u76EF", "\u76EF"], /* CJK Ideograph */
  ["\u76F0", "\u76F0"], /* CJK Ideograph */
  ["\u76F1", "\u76F1"], /* CJK Ideograph */
  ["\u76F2", "\u76F2"], /* CJK Ideograph */
  ["\u76F3", "\u76F3"], /* CJK Ideograph */
  ["\u76F4", "\u76F4"], /* CJK Ideograph */
  ["\u76F5", "\u76F5"], /* CJK Ideograph */
  ["\u76F6", "\u76F6"], /* CJK Ideograph */
  ["\u76F7", "\u76F7"], /* CJK Ideograph */
  ["\u76F8", "\u76F8"], /* CJK Ideograph */
  ["\u76F9", "\u76F9"], /* CJK Ideograph */
  ["\u76FA", "\u76FA"], /* CJK Ideograph */
  ["\u76FB", "\u76FB"], /* CJK Ideograph */
  ["\u76FC", "\u76FC"], /* CJK Ideograph */
  ["\u76FD", "\u76FD"], /* CJK Ideograph */
  ["\u76FE", "\u76FE"], /* CJK Ideograph */
  ["\u76FF", "\u76FF"], /* CJK Ideograph */
  ["\u7700", "\u7700"], /* CJK Ideograph */
  ["\u7701", "\u7701"], /* CJK Ideograph */
  ["\u7702", "\u7702"], /* CJK Ideograph */
  ["\u7703", "\u7703"], /* CJK Ideograph */
  ["\u7704", "\u7704"], /* CJK Ideograph */
  ["\u7705", "\u7705"], /* CJK Ideograph */
  ["\u7706", "\u7706"], /* CJK Ideograph */
  ["\u7707", "\u7707"], /* CJK Ideograph */
  ["\u7708", "\u7708"], /* CJK Ideograph */
  ["\u7709", "\u7709"], /* CJK Ideograph */
  ["\u770A", "\u770A"], /* CJK Ideograph */
  ["\u770B", "\u770B"], /* CJK Ideograph */
  ["\u770C", "\u770C"], /* CJK Ideograph */
  ["\u770D", "\u770D"], /* CJK Ideograph */
  ["\u770E", "\u770E"], /* CJK Ideograph */
  ["\u770F", "\u770F"], /* CJK Ideograph */
  ["\u7710", "\u7710"], /* CJK Ideograph */
  ["\u7711", "\u7711"], /* CJK Ideograph */
  ["\u7712", "\u7712"], /* CJK Ideograph */
  ["\u7713", "\u7713"], /* CJK Ideograph */
  ["\u7714", "\u7714"], /* CJK Ideograph */
  ["\u7715", "\u7715"], /* CJK Ideograph */
  ["\u7716", "\u7716"], /* CJK Ideograph */
  ["\u7717", "\u7717"], /* CJK Ideograph */
  ["\u7718", "\u7718"], /* CJK Ideograph */
  ["\u7719", "\u7719"], /* CJK Ideograph */
  ["\u771A", "\u771A"], /* CJK Ideograph */
  ["\u771B", "\u771B"], /* CJK Ideograph */
  ["\u771C", "\u771C"], /* CJK Ideograph */
  ["\u771D", "\u771D"], /* CJK Ideograph */
  ["\u771E", "\u771E"], /* CJK Ideograph */
  ["\u771F", "\u771F"], /* CJK Ideograph */
  ["\u7720", "\u7720"], /* CJK Ideograph */
  ["\u7721", "\u7721"], /* CJK Ideograph */
  ["\u7722", "\u7722"], /* CJK Ideograph */
  ["\u7723", "\u7723"], /* CJK Ideograph */
  ["\u7724", "\u7724"], /* CJK Ideograph */
  ["\u7725", "\u7725"], /* CJK Ideograph */
  ["\u7726", "\u7726"], /* CJK Ideograph */
  ["\u7727", "\u7727"], /* CJK Ideograph */
  ["\u7728", "\u7728"], /* CJK Ideograph */
  ["\u7729", "\u7729"], /* CJK Ideograph */
  ["\u772A", "\u772A"], /* CJK Ideograph */
  ["\u772B", "\u772B"], /* CJK Ideograph */
  ["\u772C", "\u772C"], /* CJK Ideograph */
  ["\u772D", "\u772D"], /* CJK Ideograph */
  ["\u772E", "\u772E"], /* CJK Ideograph */
  ["\u772F", "\u772F"], /* CJK Ideograph */
  ["\u7730", "\u7730"], /* CJK Ideograph */
  ["\u7731", "\u7731"], /* CJK Ideograph */
  ["\u7732", "\u7732"], /* CJK Ideograph */
  ["\u7733", "\u7733"], /* CJK Ideograph */
  ["\u7734", "\u7734"], /* CJK Ideograph */
  ["\u7735", "\u7735"], /* CJK Ideograph */
  ["\u7736", "\u7736"], /* CJK Ideograph */
  ["\u7737", "\u7737"], /* CJK Ideograph */
  ["\u7738", "\u7738"], /* CJK Ideograph */
  ["\u7739", "\u7739"], /* CJK Ideograph */
  ["\u773A", "\u773A"], /* CJK Ideograph */
  ["\u773B", "\u773B"], /* CJK Ideograph */
  ["\u773C", "\u773C"], /* CJK Ideograph */
  ["\u773D", "\u773D"], /* CJK Ideograph */
  ["\u773E", "\u773E"], /* CJK Ideograph */
  ["\u773F", "\u773F"], /* CJK Ideograph */
  ["\u7740", "\u7740"], /* CJK Ideograph */
  ["\u7741", "\u7741"], /* CJK Ideograph */
  ["\u7742", "\u7742"], /* CJK Ideograph */
  ["\u7743", "\u7743"], /* CJK Ideograph */
  ["\u7744", "\u7744"], /* CJK Ideograph */
  ["\u7745", "\u7745"], /* CJK Ideograph */
  ["\u7746", "\u7746"], /* CJK Ideograph */
  ["\u7747", "\u7747"], /* CJK Ideograph */
  ["\u7748", "\u7748"], /* CJK Ideograph */
  ["\u7749", "\u7749"], /* CJK Ideograph */
  ["\u774A", "\u774A"], /* CJK Ideograph */
  ["\u774B", "\u774B"], /* CJK Ideograph */
  ["\u774C", "\u774C"], /* CJK Ideograph */
  ["\u774D", "\u774D"], /* CJK Ideograph */
  ["\u774E", "\u774E"], /* CJK Ideograph */
  ["\u774F", "\u774F"], /* CJK Ideograph */
  ["\u7750", "\u7750"], /* CJK Ideograph */
  ["\u7751", "\u7751"], /* CJK Ideograph */
  ["\u7752", "\u7752"], /* CJK Ideograph */
  ["\u7753", "\u7753"], /* CJK Ideograph */
  ["\u7754", "\u7754"], /* CJK Ideograph */
  ["\u7755", "\u7755"], /* CJK Ideograph */
  ["\u7756", "\u7756"], /* CJK Ideograph */
  ["\u7757", "\u7757"], /* CJK Ideograph */
  ["\u7758", "\u7758"], /* CJK Ideograph */
  ["\u7759", "\u7759"], /* CJK Ideograph */
  ["\u775A", "\u775A"], /* CJK Ideograph */
  ["\u775B", "\u775B"], /* CJK Ideograph */
  ["\u775C", "\u775C"], /* CJK Ideograph */
  ["\u775D", "\u775D"], /* CJK Ideograph */
  ["\u775E", "\u775E"], /* CJK Ideograph */
  ["\u775F", "\u775F"], /* CJK Ideograph */
  ["\u7760", "\u7760"], /* CJK Ideograph */
  ["\u7761", "\u7761"], /* CJK Ideograph */
  ["\u7762", "\u7762"], /* CJK Ideograph */
  ["\u7763", "\u7763"], /* CJK Ideograph */
  ["\u7764", "\u7764"], /* CJK Ideograph */
  ["\u7765", "\u7765"], /* CJK Ideograph */
  ["\u7766", "\u7766"], /* CJK Ideograph */
  ["\u7767", "\u7767"], /* CJK Ideograph */
  ["\u7768", "\u7768"], /* CJK Ideograph */
  ["\u7769", "\u7769"], /* CJK Ideograph */
  ["\u776A", "\u776A"], /* CJK Ideograph */
  ["\u776B", "\u776B"], /* CJK Ideograph */
  ["\u776C", "\u776C"], /* CJK Ideograph */
  ["\u776D", "\u776D"], /* CJK Ideograph */
  ["\u776E", "\u776E"], /* CJK Ideograph */
  ["\u776F", "\u776F"], /* CJK Ideograph */
  ["\u7770", "\u7770"], /* CJK Ideograph */
  ["\u7771", "\u7771"], /* CJK Ideograph */
  ["\u7772", "\u7772"], /* CJK Ideograph */
  ["\u7773", "\u7773"], /* CJK Ideograph */
  ["\u7774", "\u7774"], /* CJK Ideograph */
  ["\u7775", "\u7775"], /* CJK Ideograph */
  ["\u7776", "\u7776"], /* CJK Ideograph */
  ["\u7777", "\u7777"], /* CJK Ideograph */
  ["\u7778", "\u7778"], /* CJK Ideograph */
  ["\u7779", "\u7779"], /* CJK Ideograph */
  ["\u777A", "\u777A"], /* CJK Ideograph */
  ["\u777B", "\u777B"], /* CJK Ideograph */
  ["\u777C", "\u777C"], /* CJK Ideograph */
  ["\u777D", "\u777D"], /* CJK Ideograph */
  ["\u777E", "\u777E"], /* CJK Ideograph */
  ["\u777F", "\u777F"], /* CJK Ideograph */
  ["\u7780", "\u7780"], /* CJK Ideograph */
  ["\u7781", "\u7781"], /* CJK Ideograph */
  ["\u7782", "\u7782"], /* CJK Ideograph */
  ["\u7783", "\u7783"], /* CJK Ideograph */
  ["\u7784", "\u7784"], /* CJK Ideograph */
  ["\u7785", "\u7785"], /* CJK Ideograph */
  ["\u7786", "\u7786"], /* CJK Ideograph */
  ["\u7787", "\u7787"], /* CJK Ideograph */
  ["\u7788", "\u7788"], /* CJK Ideograph */
  ["\u7789", "\u7789"], /* CJK Ideograph */
  ["\u778A", "\u778A"], /* CJK Ideograph */
  ["\u778B", "\u778B"], /* CJK Ideograph */
  ["\u778C", "\u778C"], /* CJK Ideograph */
  ["\u778D", "\u778D"], /* CJK Ideograph */
  ["\u778E", "\u778E"], /* CJK Ideograph */
  ["\u778F", "\u778F"], /* CJK Ideograph */
  ["\u7790", "\u7790"], /* CJK Ideograph */
  ["\u7791", "\u7791"], /* CJK Ideograph */
  ["\u7792", "\u7792"], /* CJK Ideograph */
  ["\u7793", "\u7793"], /* CJK Ideograph */
  ["\u7794", "\u7794"], /* CJK Ideograph */
  ["\u7795", "\u7795"], /* CJK Ideograph */
  ["\u7796", "\u7796"], /* CJK Ideograph */
  ["\u7797", "\u7797"], /* CJK Ideograph */
  ["\u7798", "\u7798"], /* CJK Ideograph */
  ["\u7799", "\u7799"], /* CJK Ideograph */
  ["\u779A", "\u779A"], /* CJK Ideograph */
  ["\u779B", "\u779B"], /* CJK Ideograph */
  ["\u779C", "\u779C"], /* CJK Ideograph */
  ["\u779D", "\u779D"], /* CJK Ideograph */
  ["\u779E", "\u779E"], /* CJK Ideograph */
  ["\u779F", "\u779F"], /* CJK Ideograph */
  ["\u77A0", "\u77A0"], /* CJK Ideograph */
  ["\u77A1", "\u77A1"], /* CJK Ideograph */
  ["\u77A2", "\u77A2"], /* CJK Ideograph */
  ["\u77A3", "\u77A3"], /* CJK Ideograph */
  ["\u77A4", "\u77A4"], /* CJK Ideograph */
  ["\u77A5", "\u77A5"], /* CJK Ideograph */
  ["\u77A6", "\u77A6"], /* CJK Ideograph */
  ["\u77A7", "\u77A7"], /* CJK Ideograph */
  ["\u77A8", "\u77A8"], /* CJK Ideograph */
  ["\u77A9", "\u77A9"], /* CJK Ideograph */
  ["\u77AA", "\u77AA"], /* CJK Ideograph */
  ["\u77AB", "\u77AB"], /* CJK Ideograph */
  ["\u77AC", "\u77AC"], /* CJK Ideograph */
  ["\u77AD", "\u77AD"], /* CJK Ideograph */
  ["\u77AE", "\u77AE"], /* CJK Ideograph */
  ["\u77AF", "\u77AF"], /* CJK Ideograph */
  ["\u77B0", "\u77B0"], /* CJK Ideograph */
  ["\u77B1", "\u77B1"], /* CJK Ideograph */
  ["\u77B2", "\u77B2"], /* CJK Ideograph */
  ["\u77B3", "\u77B3"], /* CJK Ideograph */
  ["\u77B4", "\u77B4"], /* CJK Ideograph */
  ["\u77B5", "\u77B5"], /* CJK Ideograph */
  ["\u77B6", "\u77B6"], /* CJK Ideograph */
  ["\u77B7", "\u77B7"], /* CJK Ideograph */
  ["\u77B8", "\u77B8"], /* CJK Ideograph */
  ["\u77B9", "\u77B9"], /* CJK Ideograph */
  ["\u77BA", "\u77BA"], /* CJK Ideograph */
  ["\u77BB", "\u77BB"], /* CJK Ideograph */
  ["\u77BC", "\u77BC"], /* CJK Ideograph */
  ["\u77BD", "\u77BD"], /* CJK Ideograph */
  ["\u77BE", "\u77BE"], /* CJK Ideograph */
  ["\u77BF", "\u77BF"], /* CJK Ideograph */
  ["\u77C0", "\u77C0"], /* CJK Ideograph */
  ["\u77C1", "\u77C1"], /* CJK Ideograph */
  ["\u77C2", "\u77C2"], /* CJK Ideograph */
  ["\u77C3", "\u77C3"], /* CJK Ideograph */
  ["\u77C4", "\u77C4"], /* CJK Ideograph */
  ["\u77C5", "\u77C5"], /* CJK Ideograph */
  ["\u77C6", "\u77C6"], /* CJK Ideograph */
  ["\u77C7", "\u77C7"], /* CJK Ideograph */
  ["\u77C8", "\u77C8"], /* CJK Ideograph */
  ["\u77C9", "\u77C9"], /* CJK Ideograph */
  ["\u77CA", "\u77CA"], /* CJK Ideograph */
  ["\u77CB", "\u77CB"], /* CJK Ideograph */
  ["\u77CC", "\u77CC"], /* CJK Ideograph */
  ["\u77CD", "\u77CD"], /* CJK Ideograph */
  ["\u77CE", "\u77CE"], /* CJK Ideograph */
  ["\u77CF", "\u77CF"], /* CJK Ideograph */
  ["\u77D0", "\u77D0"], /* CJK Ideograph */
  ["\u77D1", "\u77D1"], /* CJK Ideograph */
  ["\u77D2", "\u77D2"], /* CJK Ideograph */
  ["\u77D3", "\u77D3"], /* CJK Ideograph */
  ["\u77D4", "\u77D4"], /* CJK Ideograph */
  ["\u77D5", "\u77D5"], /* CJK Ideograph */
  ["\u77D6", "\u77D6"], /* CJK Ideograph */
  ["\u77D7", "\u77D7"], /* CJK Ideograph */
  ["\u77D8", "\u77D8"], /* CJK Ideograph */
  ["\u77D9", "\u77D9"], /* CJK Ideograph */
  ["\u77DA", "\u77DA"], /* CJK Ideograph */
  ["\u77DB", "\u77DB"], /* CJK Ideograph */
  ["\u77DC", "\u77DC"], /* CJK Ideograph */
  ["\u77DD", "\u77DD"], /* CJK Ideograph */
  ["\u77DE", "\u77DE"], /* CJK Ideograph */
  ["\u77DF", "\u77DF"], /* CJK Ideograph */
  ["\u77E0", "\u77E0"], /* CJK Ideograph */
  ["\u77E1", "\u77E1"], /* CJK Ideograph */
  ["\u77E2", "\u77E2"], /* CJK Ideograph */
  ["\u77E3", "\u77E3"], /* CJK Ideograph */
  ["\u77E4", "\u77E4"], /* CJK Ideograph */
  ["\u77E5", "\u77E5"], /* CJK Ideograph */
  ["\u77E6", "\u77E6"], /* CJK Ideograph */
  ["\u77E7", "\u77E7"], /* CJK Ideograph */
  ["\u77E8", "\u77E8"], /* CJK Ideograph */
  ["\u77E9", "\u77E9"], /* CJK Ideograph */
  ["\u77EA", "\u77EA"], /* CJK Ideograph */
  ["\u77EB", "\u77EB"], /* CJK Ideograph */
  ["\u77EC", "\u77EC"], /* CJK Ideograph */
  ["\u77ED", "\u77ED"], /* CJK Ideograph */
  ["\u77EE", "\u77EE"], /* CJK Ideograph */
  ["\u77EF", "\u77EF"], /* CJK Ideograph */
  ["\u77F0", "\u77F0"], /* CJK Ideograph */
  ["\u77F1", "\u77F1"], /* CJK Ideograph */
  ["\u77F2", "\u77F2"], /* CJK Ideograph */
  ["\u77F3", "\u77F3"], /* CJK Ideograph */
  ["\u77F4", "\u77F4"], /* CJK Ideograph */
  ["\u77F5", "\u77F5"], /* CJK Ideograph */
  ["\u77F6", "\u77F6"], /* CJK Ideograph */
  ["\u77F7", "\u77F7"], /* CJK Ideograph */
  ["\u77F8", "\u77F8"], /* CJK Ideograph */
  ["\u77F9", "\u77F9"], /* CJK Ideograph */
  ["\u77FA", "\u77FA"], /* CJK Ideograph */
  ["\u77FB", "\u77FB"], /* CJK Ideograph */
  ["\u77FC", "\u77FC"], /* CJK Ideograph */
  ["\u77FD", "\u77FD"], /* CJK Ideograph */
  ["\u77FE", "\u77FE"], /* CJK Ideograph */
  ["\u77FF", "\u77FF"], /* CJK Ideograph */
  ["\u7800", "\u7800"], /* CJK Ideograph */
  ["\u7801", "\u7801"], /* CJK Ideograph */
  ["\u7802", "\u7802"], /* CJK Ideograph */
  ["\u7803", "\u7803"], /* CJK Ideograph */
  ["\u7804", "\u7804"], /* CJK Ideograph */
  ["\u7805", "\u7805"], /* CJK Ideograph */
  ["\u7806", "\u7806"], /* CJK Ideograph */
  ["\u7807", "\u7807"], /* CJK Ideograph */
  ["\u7808", "\u7808"], /* CJK Ideograph */
  ["\u7809", "\u7809"], /* CJK Ideograph */
  ["\u780A", "\u780A"], /* CJK Ideograph */
  ["\u780B", "\u780B"], /* CJK Ideograph */
  ["\u780C", "\u780C"], /* CJK Ideograph */
  ["\u780D", "\u780D"], /* CJK Ideograph */
  ["\u780E", "\u780E"], /* CJK Ideograph */
  ["\u780F", "\u780F"], /* CJK Ideograph */
  ["\u7810", "\u7810"], /* CJK Ideograph */
  ["\u7811", "\u7811"], /* CJK Ideograph */
  ["\u7812", "\u7812"], /* CJK Ideograph */
  ["\u7813", "\u7813"], /* CJK Ideograph */
  ["\u7814", "\u7814"], /* CJK Ideograph */
  ["\u7815", "\u7815"], /* CJK Ideograph */
  ["\u7816", "\u7816"], /* CJK Ideograph */
  ["\u7817", "\u7817"], /* CJK Ideograph */
  ["\u7818", "\u7818"], /* CJK Ideograph */
  ["\u7819", "\u7819"], /* CJK Ideograph */
  ["\u781A", "\u781A"], /* CJK Ideograph */
  ["\u781B", "\u781B"], /* CJK Ideograph */
  ["\u781C", "\u781C"], /* CJK Ideograph */
  ["\u781D", "\u781D"], /* CJK Ideograph */
  ["\u781E", "\u781E"], /* CJK Ideograph */
  ["\u781F", "\u781F"], /* CJK Ideograph */
  ["\u7820", "\u7820"], /* CJK Ideograph */
  ["\u7821", "\u7821"], /* CJK Ideograph */
  ["\u7822", "\u7822"], /* CJK Ideograph */
  ["\u7823", "\u7823"], /* CJK Ideograph */
  ["\u7824", "\u7824"], /* CJK Ideograph */
  ["\u7825", "\u7825"], /* CJK Ideograph */
  ["\u7826", "\u7826"], /* CJK Ideograph */
  ["\u7827", "\u7827"], /* CJK Ideograph */
  ["\u7828", "\u7828"], /* CJK Ideograph */
  ["\u7829", "\u7829"], /* CJK Ideograph */
  ["\u782A", "\u782A"], /* CJK Ideograph */
  ["\u782B", "\u782B"], /* CJK Ideograph */
  ["\u782C", "\u782C"], /* CJK Ideograph */
  ["\u782D", "\u782D"], /* CJK Ideograph */
  ["\u782E", "\u782E"], /* CJK Ideograph */
  ["\u782F", "\u782F"], /* CJK Ideograph */
  ["\u7830", "\u7830"], /* CJK Ideograph */
  ["\u7831", "\u7831"], /* CJK Ideograph */
  ["\u7832", "\u7832"], /* CJK Ideograph */
  ["\u7833", "\u7833"], /* CJK Ideograph */
  ["\u7834", "\u7834"], /* CJK Ideograph */
  ["\u7835", "\u7835"], /* CJK Ideograph */
  ["\u7836", "\u7836"], /* CJK Ideograph */
  ["\u7837", "\u7837"], /* CJK Ideograph */
  ["\u7838", "\u7838"], /* CJK Ideograph */
  ["\u7839", "\u7839"], /* CJK Ideograph */
  ["\u783A", "\u783A"], /* CJK Ideograph */
  ["\u783B", "\u783B"], /* CJK Ideograph */
  ["\u783C", "\u783C"], /* CJK Ideograph */
  ["\u783D", "\u783D"], /* CJK Ideograph */
  ["\u783E", "\u783E"], /* CJK Ideograph */
  ["\u783F", "\u783F"], /* CJK Ideograph */
  ["\u7840", "\u7840"], /* CJK Ideograph */
  ["\u7841", "\u7841"], /* CJK Ideograph */
  ["\u7842", "\u7842"], /* CJK Ideograph */
  ["\u7843", "\u7843"], /* CJK Ideograph */
  ["\u7844", "\u7844"], /* CJK Ideograph */
  ["\u7845", "\u7845"], /* CJK Ideograph */
  ["\u7846", "\u7846"], /* CJK Ideograph */
  ["\u7847", "\u7847"], /* CJK Ideograph */
  ["\u7848", "\u7848"], /* CJK Ideograph */
  ["\u7849", "\u7849"], /* CJK Ideograph */
  ["\u784A", "\u784A"], /* CJK Ideograph */
  ["\u784B", "\u784B"], /* CJK Ideograph */
  ["\u784C", "\u784C"], /* CJK Ideograph */
  ["\u784D", "\u784D"], /* CJK Ideograph */
  ["\u784E", "\u784E"], /* CJK Ideograph */
  ["\u784F", "\u784F"], /* CJK Ideograph */
  ["\u7850", "\u7850"], /* CJK Ideograph */
  ["\u7851", "\u7851"], /* CJK Ideograph */
  ["\u7852", "\u7852"], /* CJK Ideograph */
  ["\u7853", "\u7853"], /* CJK Ideograph */
  ["\u7854", "\u7854"], /* CJK Ideograph */
  ["\u7855", "\u7855"], /* CJK Ideograph */
  ["\u7856", "\u7856"], /* CJK Ideograph */
  ["\u7857", "\u7857"], /* CJK Ideograph */
  ["\u7858", "\u7858"], /* CJK Ideograph */
  ["\u7859", "\u7859"], /* CJK Ideograph */
  ["\u785A", "\u785A"], /* CJK Ideograph */
  ["\u785B", "\u785B"], /* CJK Ideograph */
  ["\u785C", "\u785C"], /* CJK Ideograph */
  ["\u785D", "\u785D"], /* CJK Ideograph */
  ["\u785E", "\u785E"], /* CJK Ideograph */
  ["\u785F", "\u785F"], /* CJK Ideograph */
  ["\u7860", "\u7860"], /* CJK Ideograph */
  ["\u7861", "\u7861"], /* CJK Ideograph */
  ["\u7862", "\u7862"], /* CJK Ideograph */
  ["\u7863", "\u7863"], /* CJK Ideograph */
  ["\u7864", "\u7864"], /* CJK Ideograph */
  ["\u7865", "\u7865"], /* CJK Ideograph */
  ["\u7866", "\u7866"], /* CJK Ideograph */
  ["\u7867", "\u7867"], /* CJK Ideograph */
  ["\u7868", "\u7868"], /* CJK Ideograph */
  ["\u7869", "\u7869"], /* CJK Ideograph */
  ["\u786A", "\u786A"], /* CJK Ideograph */
  ["\u786B", "\u786B"], /* CJK Ideograph */
  ["\u786C", "\u786C"], /* CJK Ideograph */
  ["\u786D", "\u786D"], /* CJK Ideograph */
  ["\u786E", "\u786E"], /* CJK Ideograph */
  ["\u786F", "\u786F"], /* CJK Ideograph */
  ["\u7870", "\u7870"], /* CJK Ideograph */
  ["\u7871", "\u7871"], /* CJK Ideograph */
  ["\u7872", "\u7872"], /* CJK Ideograph */
  ["\u7873", "\u7873"], /* CJK Ideograph */
  ["\u7874", "\u7874"], /* CJK Ideograph */
  ["\u7875", "\u7875"], /* CJK Ideograph */
  ["\u7876", "\u7876"], /* CJK Ideograph */
  ["\u7877", "\u7877"], /* CJK Ideograph */
  ["\u7878", "\u7878"], /* CJK Ideograph */
  ["\u7879", "\u7879"], /* CJK Ideograph */
  ["\u787A", "\u787A"], /* CJK Ideograph */
  ["\u787B", "\u787B"], /* CJK Ideograph */
  ["\u787C", "\u787C"], /* CJK Ideograph */
  ["\u787D", "\u787D"], /* CJK Ideograph */
  ["\u787E", "\u787E"], /* CJK Ideograph */
  ["\u787F", "\u787F"], /* CJK Ideograph */
  ["\u7880", "\u7880"], /* CJK Ideograph */
  ["\u7881", "\u7881"], /* CJK Ideograph */
  ["\u7882", "\u7882"], /* CJK Ideograph */
  ["\u7883", "\u7883"], /* CJK Ideograph */
  ["\u7884", "\u7884"], /* CJK Ideograph */
  ["\u7885", "\u7885"], /* CJK Ideograph */
  ["\u7886", "\u7886"], /* CJK Ideograph */
  ["\u7887", "\u7887"], /* CJK Ideograph */
  ["\u7888", "\u7888"], /* CJK Ideograph */
  ["\u7889", "\u7889"], /* CJK Ideograph */
  ["\u788A", "\u788A"], /* CJK Ideograph */
  ["\u788B", "\u788B"], /* CJK Ideograph */
  ["\u788C", "\u788C"], /* CJK Ideograph */
  ["\u788D", "\u788D"], /* CJK Ideograph */
  ["\u788E", "\u788E"], /* CJK Ideograph */
  ["\u788F", "\u788F"], /* CJK Ideograph */
  ["\u7890", "\u7890"], /* CJK Ideograph */
  ["\u7891", "\u7891"], /* CJK Ideograph */
  ["\u7892", "\u7892"], /* CJK Ideograph */
  ["\u7893", "\u7893"], /* CJK Ideograph */
  ["\u7894", "\u7894"], /* CJK Ideograph */
  ["\u7895", "\u7895"], /* CJK Ideograph */
  ["\u7896", "\u7896"], /* CJK Ideograph */
  ["\u7897", "\u7897"], /* CJK Ideograph */
  ["\u7898", "\u7898"], /* CJK Ideograph */
  ["\u7899", "\u7899"], /* CJK Ideograph */
  ["\u789A", "\u789A"], /* CJK Ideograph */
  ["\u789B", "\u789B"], /* CJK Ideograph */
  ["\u789C", "\u789C"], /* CJK Ideograph */
  ["\u789D", "\u789D"], /* CJK Ideograph */
  ["\u789E", "\u789E"], /* CJK Ideograph */
  ["\u789F", "\u789F"], /* CJK Ideograph */
  ["\u78A0", "\u78A0"], /* CJK Ideograph */
  ["\u78A1", "\u78A1"], /* CJK Ideograph */
  ["\u78A2", "\u78A2"], /* CJK Ideograph */
  ["\u78A3", "\u78A3"], /* CJK Ideograph */
  ["\u78A4", "\u78A4"], /* CJK Ideograph */
  ["\u78A5", "\u78A5"], /* CJK Ideograph */
  ["\u78A6", "\u78A6"], /* CJK Ideograph */
  ["\u78A7", "\u78A7"], /* CJK Ideograph */
  ["\u78A8", "\u78A8"], /* CJK Ideograph */
  ["\u78A9", "\u78A9"], /* CJK Ideograph */
  ["\u78AA", "\u78AA"], /* CJK Ideograph */
  ["\u78AB", "\u78AB"], /* CJK Ideograph */
  ["\u78AC", "\u78AC"], /* CJK Ideograph */
  ["\u78AD", "\u78AD"], /* CJK Ideograph */
  ["\u78AE", "\u78AE"], /* CJK Ideograph */
  ["\u78AF", "\u78AF"], /* CJK Ideograph */
  ["\u78B0", "\u78B0"], /* CJK Ideograph */
  ["\u78B1", "\u78B1"], /* CJK Ideograph */
  ["\u78B2", "\u78B2"], /* CJK Ideograph */
  ["\u78B3", "\u78B3"], /* CJK Ideograph */
  ["\u78B4", "\u78B4"], /* CJK Ideograph */
  ["\u78B5", "\u78B5"], /* CJK Ideograph */
  ["\u78B6", "\u78B6"], /* CJK Ideograph */
  ["\u78B7", "\u78B7"], /* CJK Ideograph */
  ["\u78B8", "\u78B8"], /* CJK Ideograph */
  ["\u78B9", "\u78B9"], /* CJK Ideograph */
  ["\u78BA", "\u78BA"], /* CJK Ideograph */
  ["\u78BB", "\u78BB"], /* CJK Ideograph */
  ["\u78BC", "\u78BC"], /* CJK Ideograph */
  ["\u78BD", "\u78BD"], /* CJK Ideograph */
  ["\u78BE", "\u78BE"], /* CJK Ideograph */
  ["\u78BF", "\u78BF"], /* CJK Ideograph */
  ["\u78C0", "\u78C0"], /* CJK Ideograph */
  ["\u78C1", "\u78C1"], /* CJK Ideograph */
  ["\u78C2", "\u78C2"], /* CJK Ideograph */
  ["\u78C3", "\u78C3"], /* CJK Ideograph */
  ["\u78C4", "\u78C4"], /* CJK Ideograph */
  ["\u78C5", "\u78C5"], /* CJK Ideograph */
  ["\u78C6", "\u78C6"], /* CJK Ideograph */
  ["\u78C7", "\u78C7"], /* CJK Ideograph */
  ["\u78C8", "\u78C8"], /* CJK Ideograph */
  ["\u78C9", "\u78C9"], /* CJK Ideograph */
  ["\u78CA", "\u78CA"], /* CJK Ideograph */
  ["\u78CB", "\u78CB"], /* CJK Ideograph */
  ["\u78CC", "\u78CC"], /* CJK Ideograph */
  ["\u78CD", "\u78CD"], /* CJK Ideograph */
  ["\u78CE", "\u78CE"], /* CJK Ideograph */
  ["\u78CF", "\u78CF"], /* CJK Ideograph */
  ["\u78D0", "\u78D0"], /* CJK Ideograph */
  ["\u78D1", "\u78D1"], /* CJK Ideograph */
  ["\u78D2", "\u78D2"], /* CJK Ideograph */
  ["\u78D3", "\u78D3"], /* CJK Ideograph */
  ["\u78D4", "\u78D4"], /* CJK Ideograph */
  ["\u78D5", "\u78D5"], /* CJK Ideograph */
  ["\u78D6", "\u78D6"], /* CJK Ideograph */
  ["\u78D7", "\u78D7"], /* CJK Ideograph */
  ["\u78D8", "\u78D8"], /* CJK Ideograph */
  ["\u78D9", "\u78D9"], /* CJK Ideograph */
  ["\u78DA", "\u78DA"], /* CJK Ideograph */
  ["\u78DB", "\u78DB"], /* CJK Ideograph */
  ["\u78DC", "\u78DC"], /* CJK Ideograph */
  ["\u78DD", "\u78DD"], /* CJK Ideograph */
  ["\u78DE", "\u78DE"], /* CJK Ideograph */
  ["\u78DF", "\u78DF"], /* CJK Ideograph */
  ["\u78E0", "\u78E0"], /* CJK Ideograph */
  ["\u78E1", "\u78E1"], /* CJK Ideograph */
  ["\u78E2", "\u78E2"], /* CJK Ideograph */
  ["\u78E3", "\u78E3"], /* CJK Ideograph */
  ["\u78E4", "\u78E4"], /* CJK Ideograph */
  ["\u78E5", "\u78E5"], /* CJK Ideograph */
  ["\u78E6", "\u78E6"], /* CJK Ideograph */
  ["\u78E7", "\u78E7"], /* CJK Ideograph */
  ["\u78E8", "\u78E8"], /* CJK Ideograph */
  ["\u78E9", "\u78E9"], /* CJK Ideograph */
  ["\u78EA", "\u78EA"], /* CJK Ideograph */
  ["\u78EB", "\u78EB"], /* CJK Ideograph */
  ["\u78EC", "\u78EC"], /* CJK Ideograph */
  ["\u78ED", "\u78ED"], /* CJK Ideograph */
  ["\u78EE", "\u78EE"], /* CJK Ideograph */
  ["\u78EF", "\u78EF"], /* CJK Ideograph */
  ["\u78F0", "\u78F0"], /* CJK Ideograph */
  ["\u78F1", "\u78F1"], /* CJK Ideograph */
  ["\u78F2", "\u78F2"], /* CJK Ideograph */
  ["\u78F3", "\u78F3"], /* CJK Ideograph */
  ["\u78F4", "\u78F4"], /* CJK Ideograph */
  ["\u78F5", "\u78F5"], /* CJK Ideograph */
  ["\u78F6", "\u78F6"], /* CJK Ideograph */
  ["\u78F7", "\u78F7"], /* CJK Ideograph */
  ["\u78F8", "\u78F8"], /* CJK Ideograph */
  ["\u78F9", "\u78F9"], /* CJK Ideograph */
  ["\u78FA", "\u78FA"], /* CJK Ideograph */
  ["\u78FB", "\u78FB"], /* CJK Ideograph */
  ["\u78FC", "\u78FC"], /* CJK Ideograph */
  ["\u78FD", "\u78FD"], /* CJK Ideograph */
  ["\u78FE", "\u78FE"], /* CJK Ideograph */
  ["\u78FF", "\u78FF"], /* CJK Ideograph */
  ["\u7900", "\u7900"], /* CJK Ideograph */
  ["\u7901", "\u7901"], /* CJK Ideograph */
  ["\u7902", "\u7902"], /* CJK Ideograph */
  ["\u7903", "\u7903"], /* CJK Ideograph */
  ["\u7904", "\u7904"], /* CJK Ideograph */
  ["\u7905", "\u7905"], /* CJK Ideograph */
  ["\u7906", "\u7906"], /* CJK Ideograph */
  ["\u7907", "\u7907"], /* CJK Ideograph */
  ["\u7908", "\u7908"], /* CJK Ideograph */
  ["\u7909", "\u7909"], /* CJK Ideograph */
  ["\u790A", "\u790A"], /* CJK Ideograph */
  ["\u790B", "\u790B"], /* CJK Ideograph */
  ["\u790C", "\u790C"], /* CJK Ideograph */
  ["\u790D", "\u790D"], /* CJK Ideograph */
  ["\u790E", "\u790E"], /* CJK Ideograph */
  ["\u790F", "\u790F"], /* CJK Ideograph */
  ["\u7910", "\u7910"], /* CJK Ideograph */
  ["\u7911", "\u7911"], /* CJK Ideograph */
  ["\u7912", "\u7912"], /* CJK Ideograph */
  ["\u7913", "\u7913"], /* CJK Ideograph */
  ["\u7914", "\u7914"], /* CJK Ideograph */
  ["\u7915", "\u7915"], /* CJK Ideograph */
  ["\u7916", "\u7916"], /* CJK Ideograph */
  ["\u7917", "\u7917"], /* CJK Ideograph */
  ["\u7918", "\u7918"], /* CJK Ideograph */
  ["\u7919", "\u7919"], /* CJK Ideograph */
  ["\u791A", "\u791A"], /* CJK Ideograph */
  ["\u791B", "\u791B"], /* CJK Ideograph */
  ["\u791C", "\u791C"], /* CJK Ideograph */
  ["\u791D", "\u791D"], /* CJK Ideograph */
  ["\u791E", "\u791E"], /* CJK Ideograph */
  ["\u791F", "\u791F"], /* CJK Ideograph */
  ["\u7920", "\u7920"], /* CJK Ideograph */
  ["\u7921", "\u7921"], /* CJK Ideograph */
  ["\u7922", "\u7922"], /* CJK Ideograph */
  ["\u7923", "\u7923"], /* CJK Ideograph */
  ["\u7924", "\u7924"], /* CJK Ideograph */
  ["\u7925", "\u7925"], /* CJK Ideograph */
  ["\u7926", "\u7926"], /* CJK Ideograph */
  ["\u7927", "\u7927"], /* CJK Ideograph */
  ["\u7928", "\u7928"], /* CJK Ideograph */
  ["\u7929", "\u7929"], /* CJK Ideograph */
  ["\u792A", "\u792A"], /* CJK Ideograph */
  ["\u792B", "\u792B"], /* CJK Ideograph */
  ["\u792C", "\u792C"], /* CJK Ideograph */
  ["\u792D", "\u792D"], /* CJK Ideograph */
  ["\u792E", "\u792E"], /* CJK Ideograph */
  ["\u792F", "\u792F"], /* CJK Ideograph */
  ["\u7930", "\u7930"], /* CJK Ideograph */
  ["\u7931", "\u7931"], /* CJK Ideograph */
  ["\u7932", "\u7932"], /* CJK Ideograph */
  ["\u7933", "\u7933"], /* CJK Ideograph */
  ["\u7934", "\u7934"], /* CJK Ideograph */
  ["\u7935", "\u7935"], /* CJK Ideograph */
  ["\u7936", "\u7936"], /* CJK Ideograph */
  ["\u7937", "\u7937"], /* CJK Ideograph */
  ["\u7938", "\u7938"], /* CJK Ideograph */
  ["\u7939", "\u7939"], /* CJK Ideograph */
  ["\u793A", "\u793A"], /* CJK Ideograph */
  ["\u793B", "\u793B"], /* CJK Ideograph */
  ["\u793C", "\u793C"], /* CJK Ideograph */
  ["\u793D", "\u793D"], /* CJK Ideograph */
  ["\u793E", "\u793E"], /* CJK Ideograph */
  ["\u793F", "\u793F"], /* CJK Ideograph */
  ["\u7940", "\u7940"], /* CJK Ideograph */
  ["\u7941", "\u7941"], /* CJK Ideograph */
  ["\u7942", "\u7942"], /* CJK Ideograph */
  ["\u7943", "\u7943"], /* CJK Ideograph */
  ["\u7944", "\u7944"], /* CJK Ideograph */
  ["\u7945", "\u7945"], /* CJK Ideograph */
  ["\u7946", "\u7946"], /* CJK Ideograph */
  ["\u7947", "\u7947"], /* CJK Ideograph */
  ["\u7948", "\u7948"], /* CJK Ideograph */
  ["\u7949", "\u7949"], /* CJK Ideograph */
  ["\u794A", "\u794A"], /* CJK Ideograph */
  ["\u794B", "\u794B"], /* CJK Ideograph */
  ["\u794C", "\u794C"], /* CJK Ideograph */
  ["\u794D", "\u794D"], /* CJK Ideograph */
  ["\u794E", "\u794E"], /* CJK Ideograph */
  ["\u794F", "\u794F"], /* CJK Ideograph */
  ["\u7950", "\u7950"], /* CJK Ideograph */
  ["\u7951", "\u7951"], /* CJK Ideograph */
  ["\u7952", "\u7952"], /* CJK Ideograph */
  ["\u7953", "\u7953"], /* CJK Ideograph */
  ["\u7954", "\u7954"], /* CJK Ideograph */
  ["\u7955", "\u7955"], /* CJK Ideograph */
  ["\u7956", "\u7956"], /* CJK Ideograph */
  ["\u7957", "\u7957"], /* CJK Ideograph */
  ["\u7958", "\u7958"], /* CJK Ideograph */
  ["\u7959", "\u7959"], /* CJK Ideograph */
  ["\u795A", "\u795A"], /* CJK Ideograph */
  ["\u795B", "\u795B"], /* CJK Ideograph */
  ["\u795C", "\u795C"], /* CJK Ideograph */
  ["\u795D", "\u795D"], /* CJK Ideograph */
  ["\u795E", "\u795E"], /* CJK Ideograph */
  ["\u795F", "\u795F"], /* CJK Ideograph */
  ["\u7960", "\u7960"], /* CJK Ideograph */
  ["\u7961", "\u7961"], /* CJK Ideograph */
  ["\u7962", "\u7962"], /* CJK Ideograph */
  ["\u7963", "\u7963"], /* CJK Ideograph */
  ["\u7964", "\u7964"], /* CJK Ideograph */
  ["\u7965", "\u7965"], /* CJK Ideograph */
  ["\u7966", "\u7966"], /* CJK Ideograph */
  ["\u7967", "\u7967"], /* CJK Ideograph */
  ["\u7968", "\u7968"], /* CJK Ideograph */
  ["\u7969", "\u7969"], /* CJK Ideograph */
  ["\u796A", "\u796A"], /* CJK Ideograph */
  ["\u796B", "\u796B"], /* CJK Ideograph */
  ["\u796C", "\u796C"], /* CJK Ideograph */
  ["\u796D", "\u796D"], /* CJK Ideograph */
  ["\u796E", "\u796E"], /* CJK Ideograph */
  ["\u796F", "\u796F"], /* CJK Ideograph */
  ["\u7970", "\u7970"], /* CJK Ideograph */
  ["\u7971", "\u7971"], /* CJK Ideograph */
  ["\u7972", "\u7972"], /* CJK Ideograph */
  ["\u7973", "\u7973"], /* CJK Ideograph */
  ["\u7974", "\u7974"], /* CJK Ideograph */
  ["\u7975", "\u7975"], /* CJK Ideograph */
  ["\u7976", "\u7976"], /* CJK Ideograph */
  ["\u7977", "\u7977"], /* CJK Ideograph */
  ["\u7978", "\u7978"], /* CJK Ideograph */
  ["\u7979", "\u7979"], /* CJK Ideograph */
  ["\u797A", "\u797A"], /* CJK Ideograph */
  ["\u797B", "\u797B"], /* CJK Ideograph */
  ["\u797C", "\u797C"], /* CJK Ideograph */
  ["\u797D", "\u797D"], /* CJK Ideograph */
  ["\u797E", "\u797E"], /* CJK Ideograph */
  ["\u797F", "\u797F"], /* CJK Ideograph */
  ["\u7980", "\u7980"], /* CJK Ideograph */
  ["\u7981", "\u7981"], /* CJK Ideograph */
  ["\u7982", "\u7982"], /* CJK Ideograph */
  ["\u7983", "\u7983"], /* CJK Ideograph */
  ["\u7984", "\u7984"], /* CJK Ideograph */
  ["\u7985", "\u7985"], /* CJK Ideograph */
  ["\u7986", "\u7986"], /* CJK Ideograph */
  ["\u7987", "\u7987"], /* CJK Ideograph */
  ["\u7988", "\u7988"], /* CJK Ideograph */
  ["\u7989", "\u7989"], /* CJK Ideograph */
  ["\u798A", "\u798A"], /* CJK Ideograph */
  ["\u798B", "\u798B"], /* CJK Ideograph */
  ["\u798C", "\u798C"], /* CJK Ideograph */
  ["\u798D", "\u798D"], /* CJK Ideograph */
  ["\u798E", "\u798E"], /* CJK Ideograph */
  ["\u798F", "\u798F"], /* CJK Ideograph */
  ["\u7990", "\u7990"], /* CJK Ideograph */
  ["\u7991", "\u7991"], /* CJK Ideograph */
  ["\u7992", "\u7992"], /* CJK Ideograph */
  ["\u7993", "\u7993"], /* CJK Ideograph */
  ["\u7994", "\u7994"], /* CJK Ideograph */
  ["\u7995", "\u7995"], /* CJK Ideograph */
  ["\u7996", "\u7996"], /* CJK Ideograph */
  ["\u7997", "\u7997"], /* CJK Ideograph */
  ["\u7998", "\u7998"], /* CJK Ideograph */
  ["\u7999", "\u7999"], /* CJK Ideograph */
  ["\u799A", "\u799A"], /* CJK Ideograph */
  ["\u799B", "\u799B"], /* CJK Ideograph */
  ["\u799C", "\u799C"], /* CJK Ideograph */
  ["\u799D", "\u799D"], /* CJK Ideograph */
  ["\u799E", "\u799E"], /* CJK Ideograph */
  ["\u799F", "\u799F"], /* CJK Ideograph */
  ["\u79A0", "\u79A0"], /* CJK Ideograph */
  ["\u79A1", "\u79A1"], /* CJK Ideograph */
  ["\u79A2", "\u79A2"], /* CJK Ideograph */
  ["\u79A3", "\u79A3"], /* CJK Ideograph */
  ["\u79A4", "\u79A4"], /* CJK Ideograph */
  ["\u79A5", "\u79A5"], /* CJK Ideograph */
  ["\u79A6", "\u79A6"], /* CJK Ideograph */
  ["\u79A7", "\u79A7"], /* CJK Ideograph */
  ["\u79A8", "\u79A8"], /* CJK Ideograph */
  ["\u79A9", "\u79A9"], /* CJK Ideograph */
  ["\u79AA", "\u79AA"], /* CJK Ideograph */
  ["\u79AB", "\u79AB"], /* CJK Ideograph */
  ["\u79AC", "\u79AC"], /* CJK Ideograph */
  ["\u79AD", "\u79AD"], /* CJK Ideograph */
  ["\u79AE", "\u79AE"], /* CJK Ideograph */
  ["\u79AF", "\u79AF"], /* CJK Ideograph */
  ["\u79B0", "\u79B0"], /* CJK Ideograph */
  ["\u79B1", "\u79B1"], /* CJK Ideograph */
  ["\u79B2", "\u79B2"], /* CJK Ideograph */
  ["\u79B3", "\u79B3"], /* CJK Ideograph */
  ["\u79B4", "\u79B4"], /* CJK Ideograph */
  ["\u79B5", "\u79B5"], /* CJK Ideograph */
  ["\u79B6", "\u79B6"], /* CJK Ideograph */
  ["\u79B7", "\u79B7"], /* CJK Ideograph */
  ["\u79B8", "\u79B8"], /* CJK Ideograph */
  ["\u79B9", "\u79B9"], /* CJK Ideograph */
  ["\u79BA", "\u79BA"], /* CJK Ideograph */
  ["\u79BB", "\u79BB"], /* CJK Ideograph */
  ["\u79BC", "\u79BC"], /* CJK Ideograph */
  ["\u79BD", "\u79BD"], /* CJK Ideograph */
  ["\u79BE", "\u79BE"], /* CJK Ideograph */
  ["\u79BF", "\u79BF"], /* CJK Ideograph */
  ["\u79C0", "\u79C0"], /* CJK Ideograph */
  ["\u79C1", "\u79C1"], /* CJK Ideograph */
  ["\u79C2", "\u79C2"], /* CJK Ideograph */
  ["\u79C3", "\u79C3"], /* CJK Ideograph */
  ["\u79C4", "\u79C4"], /* CJK Ideograph */
  ["\u79C5", "\u79C5"], /* CJK Ideograph */
  ["\u79C6", "\u79C6"], /* CJK Ideograph */
  ["\u79C7", "\u79C7"], /* CJK Ideograph */
  ["\u79C8", "\u79C8"], /* CJK Ideograph */
  ["\u79C9", "\u79C9"], /* CJK Ideograph */
  ["\u79CA", "\u79CA"], /* CJK Ideograph */
  ["\u79CB", "\u79CB"], /* CJK Ideograph */
  ["\u79CC", "\u79CC"], /* CJK Ideograph */
  ["\u79CD", "\u79CD"], /* CJK Ideograph */
  ["\u79CE", "\u79CE"], /* CJK Ideograph */
  ["\u79CF", "\u79CF"], /* CJK Ideograph */
  ["\u79D0", "\u79D0"], /* CJK Ideograph */
  ["\u79D1", "\u79D1"], /* CJK Ideograph */
  ["\u79D2", "\u79D2"], /* CJK Ideograph */
  ["\u79D3", "\u79D3"], /* CJK Ideograph */
  ["\u79D4", "\u79D4"], /* CJK Ideograph */
  ["\u79D5", "\u79D5"], /* CJK Ideograph */
  ["\u79D6", "\u79D6"], /* CJK Ideograph */
  ["\u79D7", "\u79D7"], /* CJK Ideograph */
  ["\u79D8", "\u79D8"], /* CJK Ideograph */
  ["\u79D9", "\u79D9"], /* CJK Ideograph */
  ["\u79DA", "\u79DA"], /* CJK Ideograph */
  ["\u79DB", "\u79DB"], /* CJK Ideograph */
  ["\u79DC", "\u79DC"], /* CJK Ideograph */
  ["\u79DD", "\u79DD"], /* CJK Ideograph */
  ["\u79DE", "\u79DE"], /* CJK Ideograph */
  ["\u79DF", "\u79DF"], /* CJK Ideograph */
  ["\u79E0", "\u79E0"], /* CJK Ideograph */
  ["\u79E1", "\u79E1"], /* CJK Ideograph */
  ["\u79E2", "\u79E2"], /* CJK Ideograph */
  ["\u79E3", "\u79E3"], /* CJK Ideograph */
  ["\u79E4", "\u79E4"], /* CJK Ideograph */
  ["\u79E5", "\u79E5"], /* CJK Ideograph */
  ["\u79E6", "\u79E6"], /* CJK Ideograph */
  ["\u79E7", "\u79E7"], /* CJK Ideograph */
  ["\u79E8", "\u79E8"], /* CJK Ideograph */
  ["\u79E9", "\u79E9"], /* CJK Ideograph */
  ["\u79EA", "\u79EA"], /* CJK Ideograph */
  ["\u79EB", "\u79EB"], /* CJK Ideograph */
  ["\u79EC", "\u79EC"], /* CJK Ideograph */
  ["\u79ED", "\u79ED"], /* CJK Ideograph */
  ["\u79EE", "\u79EE"], /* CJK Ideograph */
  ["\u79EF", "\u79EF"], /* CJK Ideograph */
  ["\u79F0", "\u79F0"], /* CJK Ideograph */
  ["\u79F1", "\u79F1"], /* CJK Ideograph */
  ["\u79F2", "\u79F2"], /* CJK Ideograph */
  ["\u79F3", "\u79F3"], /* CJK Ideograph */
  ["\u79F4", "\u79F4"], /* CJK Ideograph */
  ["\u79F5", "\u79F5"], /* CJK Ideograph */
  ["\u79F6", "\u79F6"], /* CJK Ideograph */
  ["\u79F7", "\u79F7"], /* CJK Ideograph */
  ["\u79F8", "\u79F8"], /* CJK Ideograph */
  ["\u79F9", "\u79F9"], /* CJK Ideograph */
  ["\u79FA", "\u79FA"], /* CJK Ideograph */
  ["\u79FB", "\u79FB"], /* CJK Ideograph */
  ["\u79FC", "\u79FC"], /* CJK Ideograph */
  ["\u79FD", "\u79FD"], /* CJK Ideograph */
  ["\u79FE", "\u79FE"], /* CJK Ideograph */
  ["\u79FF", "\u79FF"], /* CJK Ideograph */
  ["\u7A00", "\u7A00"], /* CJK Ideograph */
  ["\u7A01", "\u7A01"], /* CJK Ideograph */
  ["\u7A02", "\u7A02"], /* CJK Ideograph */
  ["\u7A03", "\u7A03"], /* CJK Ideograph */
  ["\u7A04", "\u7A04"], /* CJK Ideograph */
  ["\u7A05", "\u7A05"], /* CJK Ideograph */
  ["\u7A06", "\u7A06"], /* CJK Ideograph */
  ["\u7A07", "\u7A07"], /* CJK Ideograph */
  ["\u7A08", "\u7A08"], /* CJK Ideograph */
  ["\u7A09", "\u7A09"], /* CJK Ideograph */
  ["\u7A0A", "\u7A0A"], /* CJK Ideograph */
  ["\u7A0B", "\u7A0B"], /* CJK Ideograph */
  ["\u7A0C", "\u7A0C"], /* CJK Ideograph */
  ["\u7A0D", "\u7A0D"], /* CJK Ideograph */
  ["\u7A0E", "\u7A0E"], /* CJK Ideograph */
  ["\u7A0F", "\u7A0F"], /* CJK Ideograph */
  ["\u7A10", "\u7A10"], /* CJK Ideograph */
  ["\u7A11", "\u7A11"], /* CJK Ideograph */
  ["\u7A12", "\u7A12"], /* CJK Ideograph */
  ["\u7A13", "\u7A13"], /* CJK Ideograph */
  ["\u7A14", "\u7A14"], /* CJK Ideograph */
  ["\u7A15", "\u7A15"], /* CJK Ideograph */
  ["\u7A16", "\u7A16"], /* CJK Ideograph */
  ["\u7A17", "\u7A17"], /* CJK Ideograph */
  ["\u7A18", "\u7A18"], /* CJK Ideograph */
  ["\u7A19", "\u7A19"], /* CJK Ideograph */
  ["\u7A1A", "\u7A1A"], /* CJK Ideograph */
  ["\u7A1B", "\u7A1B"], /* CJK Ideograph */
  ["\u7A1C", "\u7A1C"], /* CJK Ideograph */
  ["\u7A1D", "\u7A1D"], /* CJK Ideograph */
  ["\u7A1E", "\u7A1E"], /* CJK Ideograph */
  ["\u7A1F", "\u7A1F"], /* CJK Ideograph */
  ["\u7A20", "\u7A20"], /* CJK Ideograph */
  ["\u7A21", "\u7A21"], /* CJK Ideograph */
  ["\u7A22", "\u7A22"], /* CJK Ideograph */
  ["\u7A23", "\u7A23"], /* CJK Ideograph */
  ["\u7A24", "\u7A24"], /* CJK Ideograph */
  ["\u7A25", "\u7A25"], /* CJK Ideograph */
  ["\u7A26", "\u7A26"], /* CJK Ideograph */
  ["\u7A27", "\u7A27"], /* CJK Ideograph */
  ["\u7A28", "\u7A28"], /* CJK Ideograph */
  ["\u7A29", "\u7A29"], /* CJK Ideograph */
  ["\u7A2A", "\u7A2A"], /* CJK Ideograph */
  ["\u7A2B", "\u7A2B"], /* CJK Ideograph */
  ["\u7A2C", "\u7A2C"], /* CJK Ideograph */
  ["\u7A2D", "\u7A2D"], /* CJK Ideograph */
  ["\u7A2E", "\u7A2E"], /* CJK Ideograph */
  ["\u7A2F", "\u7A2F"], /* CJK Ideograph */
  ["\u7A30", "\u7A30"], /* CJK Ideograph */
  ["\u7A31", "\u7A31"], /* CJK Ideograph */
  ["\u7A32", "\u7A32"], /* CJK Ideograph */
  ["\u7A33", "\u7A33"], /* CJK Ideograph */
  ["\u7A34", "\u7A34"], /* CJK Ideograph */
  ["\u7A35", "\u7A35"], /* CJK Ideograph */
  ["\u7A36", "\u7A36"], /* CJK Ideograph */
  ["\u7A37", "\u7A37"], /* CJK Ideograph */
  ["\u7A38", "\u7A38"], /* CJK Ideograph */
  ["\u7A39", "\u7A39"], /* CJK Ideograph */
  ["\u7A3A", "\u7A3A"], /* CJK Ideograph */
  ["\u7A3B", "\u7A3B"], /* CJK Ideograph */
  ["\u7A3C", "\u7A3C"], /* CJK Ideograph */
  ["\u7A3D", "\u7A3D"], /* CJK Ideograph */
  ["\u7A3E", "\u7A3E"], /* CJK Ideograph */
  ["\u7A3F", "\u7A3F"], /* CJK Ideograph */
  ["\u7A40", "\u7A40"], /* CJK Ideograph */
  ["\u7A41", "\u7A41"], /* CJK Ideograph */
  ["\u7A42", "\u7A42"], /* CJK Ideograph */
  ["\u7A43", "\u7A43"], /* CJK Ideograph */
  ["\u7A44", "\u7A44"], /* CJK Ideograph */
  ["\u7A45", "\u7A45"], /* CJK Ideograph */
  ["\u7A46", "\u7A46"], /* CJK Ideograph */
  ["\u7A47", "\u7A47"], /* CJK Ideograph */
  ["\u7A48", "\u7A48"], /* CJK Ideograph */
  ["\u7A49", "\u7A49"], /* CJK Ideograph */
  ["\u7A4A", "\u7A4A"], /* CJK Ideograph */
  ["\u7A4B", "\u7A4B"], /* CJK Ideograph */
  ["\u7A4C", "\u7A4C"], /* CJK Ideograph */
  ["\u7A4D", "\u7A4D"], /* CJK Ideograph */
  ["\u7A4E", "\u7A4E"], /* CJK Ideograph */
  ["\u7A4F", "\u7A4F"], /* CJK Ideograph */
  ["\u7A50", "\u7A50"], /* CJK Ideograph */
  ["\u7A51", "\u7A51"], /* CJK Ideograph */
  ["\u7A52", "\u7A52"], /* CJK Ideograph */
  ["\u7A53", "\u7A53"], /* CJK Ideograph */
  ["\u7A54", "\u7A54"], /* CJK Ideograph */
  ["\u7A55", "\u7A55"], /* CJK Ideograph */
  ["\u7A56", "\u7A56"], /* CJK Ideograph */
  ["\u7A57", "\u7A57"], /* CJK Ideograph */
  ["\u7A58", "\u7A58"], /* CJK Ideograph */
  ["\u7A59", "\u7A59"], /* CJK Ideograph */
  ["\u7A5A", "\u7A5A"], /* CJK Ideograph */
  ["\u7A5B", "\u7A5B"], /* CJK Ideograph */
  ["\u7A5C", "\u7A5C"], /* CJK Ideograph */
  ["\u7A5D", "\u7A5D"], /* CJK Ideograph */
  ["\u7A5E", "\u7A5E"], /* CJK Ideograph */
  ["\u7A5F", "\u7A5F"], /* CJK Ideograph */
  ["\u7A60", "\u7A60"], /* CJK Ideograph */
  ["\u7A61", "\u7A61"], /* CJK Ideograph */
  ["\u7A62", "\u7A62"], /* CJK Ideograph */
  ["\u7A63", "\u7A63"], /* CJK Ideograph */
  ["\u7A64", "\u7A64"], /* CJK Ideograph */
  ["\u7A65", "\u7A65"], /* CJK Ideograph */
  ["\u7A66", "\u7A66"], /* CJK Ideograph */
  ["\u7A67", "\u7A67"], /* CJK Ideograph */
  ["\u7A68", "\u7A68"], /* CJK Ideograph */
  ["\u7A69", "\u7A69"], /* CJK Ideograph */
  ["\u7A6A", "\u7A6A"], /* CJK Ideograph */
  ["\u7A6B", "\u7A6B"], /* CJK Ideograph */
  ["\u7A6C", "\u7A6C"], /* CJK Ideograph */
  ["\u7A6D", "\u7A6D"], /* CJK Ideograph */
  ["\u7A6E", "\u7A6E"], /* CJK Ideograph */
  ["\u7A6F", "\u7A6F"], /* CJK Ideograph */
  ["\u7A70", "\u7A70"], /* CJK Ideograph */
  ["\u7A71", "\u7A71"], /* CJK Ideograph */
  ["\u7A72", "\u7A72"], /* CJK Ideograph */
  ["\u7A73", "\u7A73"], /* CJK Ideograph */
  ["\u7A74", "\u7A74"], /* CJK Ideograph */
  ["\u7A75", "\u7A75"], /* CJK Ideograph */
  ["\u7A76", "\u7A76"], /* CJK Ideograph */
  ["\u7A77", "\u7A77"], /* CJK Ideograph */
  ["\u7A78", "\u7A78"], /* CJK Ideograph */
  ["\u7A79", "\u7A79"], /* CJK Ideograph */
  ["\u7A7A", "\u7A7A"], /* CJK Ideograph */
  ["\u7A7B", "\u7A7B"], /* CJK Ideograph */
  ["\u7A7C", "\u7A7C"], /* CJK Ideograph */
  ["\u7A7D", "\u7A7D"], /* CJK Ideograph */
  ["\u7A7E", "\u7A7E"], /* CJK Ideograph */
  ["\u7A7F", "\u7A7F"], /* CJK Ideograph */
  ["\u7A80", "\u7A80"], /* CJK Ideograph */
  ["\u7A81", "\u7A81"], /* CJK Ideograph */
  ["\u7A82", "\u7A82"], /* CJK Ideograph */
  ["\u7A83", "\u7A83"], /* CJK Ideograph */
  ["\u7A84", "\u7A84"], /* CJK Ideograph */
  ["\u7A85", "\u7A85"], /* CJK Ideograph */
  ["\u7A86", "\u7A86"], /* CJK Ideograph */
  ["\u7A87", "\u7A87"], /* CJK Ideograph */
  ["\u7A88", "\u7A88"], /* CJK Ideograph */
  ["\u7A89", "\u7A89"], /* CJK Ideograph */
  ["\u7A8A", "\u7A8A"], /* CJK Ideograph */
  ["\u7A8B", "\u7A8B"], /* CJK Ideograph */
  ["\u7A8C", "\u7A8C"], /* CJK Ideograph */
  ["\u7A8D", "\u7A8D"], /* CJK Ideograph */
  ["\u7A8E", "\u7A8E"], /* CJK Ideograph */
  ["\u7A8F", "\u7A8F"], /* CJK Ideograph */
  ["\u7A90", "\u7A90"], /* CJK Ideograph */
  ["\u7A91", "\u7A91"], /* CJK Ideograph */
  ["\u7A92", "\u7A92"], /* CJK Ideograph */
  ["\u7A93", "\u7A93"], /* CJK Ideograph */
  ["\u7A94", "\u7A94"], /* CJK Ideograph */
  ["\u7A95", "\u7A95"], /* CJK Ideograph */
  ["\u7A96", "\u7A96"], /* CJK Ideograph */
  ["\u7A97", "\u7A97"], /* CJK Ideograph */
  ["\u7A98", "\u7A98"], /* CJK Ideograph */
  ["\u7A99", "\u7A99"], /* CJK Ideograph */
  ["\u7A9A", "\u7A9A"], /* CJK Ideograph */
  ["\u7A9B", "\u7A9B"], /* CJK Ideograph */
  ["\u7A9C", "\u7A9C"], /* CJK Ideograph */
  ["\u7A9D", "\u7A9D"], /* CJK Ideograph */
  ["\u7A9E", "\u7A9E"], /* CJK Ideograph */
  ["\u7A9F", "\u7A9F"], /* CJK Ideograph */
  ["\u7AA0", "\u7AA0"], /* CJK Ideograph */
  ["\u7AA1", "\u7AA1"], /* CJK Ideograph */
  ["\u7AA2", "\u7AA2"], /* CJK Ideograph */
  ["\u7AA3", "\u7AA3"], /* CJK Ideograph */
  ["\u7AA4", "\u7AA4"], /* CJK Ideograph */
  ["\u7AA5", "\u7AA5"], /* CJK Ideograph */
  ["\u7AA6", "\u7AA6"], /* CJK Ideograph */
  ["\u7AA7", "\u7AA7"], /* CJK Ideograph */
  ["\u7AA8", "\u7AA8"], /* CJK Ideograph */
  ["\u7AA9", "\u7AA9"], /* CJK Ideograph */
  ["\u7AAA", "\u7AAA"], /* CJK Ideograph */
  ["\u7AAB", "\u7AAB"], /* CJK Ideograph */
  ["\u7AAC", "\u7AAC"], /* CJK Ideograph */
  ["\u7AAD", "\u7AAD"], /* CJK Ideograph */
  ["\u7AAE", "\u7AAE"], /* CJK Ideograph */
  ["\u7AAF", "\u7AAF"], /* CJK Ideograph */
  ["\u7AB0", "\u7AB0"], /* CJK Ideograph */
  ["\u7AB1", "\u7AB1"], /* CJK Ideograph */
  ["\u7AB2", "\u7AB2"], /* CJK Ideograph */
  ["\u7AB3", "\u7AB3"], /* CJK Ideograph */
  ["\u7AB4", "\u7AB4"], /* CJK Ideograph */
  ["\u7AB5", "\u7AB5"], /* CJK Ideograph */
  ["\u7AB6", "\u7AB6"], /* CJK Ideograph */
  ["\u7AB7", "\u7AB7"], /* CJK Ideograph */
  ["\u7AB8", "\u7AB8"], /* CJK Ideograph */
  ["\u7AB9", "\u7AB9"], /* CJK Ideograph */
  ["\u7ABA", "\u7ABA"], /* CJK Ideograph */
  ["\u7ABB", "\u7ABB"], /* CJK Ideograph */
  ["\u7ABC", "\u7ABC"], /* CJK Ideograph */
  ["\u7ABD", "\u7ABD"], /* CJK Ideograph */
  ["\u7ABE", "\u7ABE"], /* CJK Ideograph */
  ["\u7ABF", "\u7ABF"], /* CJK Ideograph */
  ["\u7AC0", "\u7AC0"], /* CJK Ideograph */
  ["\u7AC1", "\u7AC1"], /* CJK Ideograph */
  ["\u7AC2", "\u7AC2"], /* CJK Ideograph */
  ["\u7AC3", "\u7AC3"], /* CJK Ideograph */
  ["\u7AC4", "\u7AC4"], /* CJK Ideograph */
  ["\u7AC5", "\u7AC5"], /* CJK Ideograph */
  ["\u7AC6", "\u7AC6"], /* CJK Ideograph */
  ["\u7AC7", "\u7AC7"], /* CJK Ideograph */
  ["\u7AC8", "\u7AC8"], /* CJK Ideograph */
  ["\u7AC9", "\u7AC9"], /* CJK Ideograph */
  ["\u7ACA", "\u7ACA"], /* CJK Ideograph */
  ["\u7ACB", "\u7ACB"], /* CJK Ideograph */
  ["\u7ACC", "\u7ACC"], /* CJK Ideograph */
  ["\u7ACD", "\u7ACD"], /* CJK Ideograph */
  ["\u7ACE", "\u7ACE"], /* CJK Ideograph */
  ["\u7ACF", "\u7ACF"], /* CJK Ideograph */
  ["\u7AD0", "\u7AD0"], /* CJK Ideograph */
  ["\u7AD1", "\u7AD1"], /* CJK Ideograph */
  ["\u7AD2", "\u7AD2"], /* CJK Ideograph */
  ["\u7AD3", "\u7AD3"], /* CJK Ideograph */
  ["\u7AD4", "\u7AD4"], /* CJK Ideograph */
  ["\u7AD5", "\u7AD5"], /* CJK Ideograph */
  ["\u7AD6", "\u7AD6"], /* CJK Ideograph */
  ["\u7AD7", "\u7AD7"], /* CJK Ideograph */
  ["\u7AD8", "\u7AD8"], /* CJK Ideograph */
  ["\u7AD9", "\u7AD9"], /* CJK Ideograph */
  ["\u7ADA", "\u7ADA"], /* CJK Ideograph */
  ["\u7ADB", "\u7ADB"], /* CJK Ideograph */
  ["\u7ADC", "\u7ADC"], /* CJK Ideograph */
  ["\u7ADD", "\u7ADD"], /* CJK Ideograph */
  ["\u7ADE", "\u7ADE"], /* CJK Ideograph */
  ["\u7ADF", "\u7ADF"], /* CJK Ideograph */
  ["\u7AE0", "\u7AE0"], /* CJK Ideograph */
  ["\u7AE1", "\u7AE1"], /* CJK Ideograph */
  ["\u7AE2", "\u7AE2"], /* CJK Ideograph */
  ["\u7AE3", "\u7AE3"], /* CJK Ideograph */
  ["\u7AE4", "\u7AE4"], /* CJK Ideograph */
  ["\u7AE5", "\u7AE5"], /* CJK Ideograph */
  ["\u7AE6", "\u7AE6"], /* CJK Ideograph */
  ["\u7AE7", "\u7AE7"], /* CJK Ideograph */
  ["\u7AE8", "\u7AE8"], /* CJK Ideograph */
  ["\u7AE9", "\u7AE9"], /* CJK Ideograph */
  ["\u7AEA", "\u7AEA"], /* CJK Ideograph */
  ["\u7AEB", "\u7AEB"], /* CJK Ideograph */
  ["\u7AEC", "\u7AEC"], /* CJK Ideograph */
  ["\u7AED", "\u7AED"], /* CJK Ideograph */
  ["\u7AEE", "\u7AEE"], /* CJK Ideograph */
  ["\u7AEF", "\u7AEF"], /* CJK Ideograph */
  ["\u7AF0", "\u7AF0"], /* CJK Ideograph */
  ["\u7AF1", "\u7AF1"], /* CJK Ideograph */
  ["\u7AF2", "\u7AF2"], /* CJK Ideograph */
  ["\u7AF3", "\u7AF3"], /* CJK Ideograph */
  ["\u7AF4", "\u7AF4"], /* CJK Ideograph */
  ["\u7AF5", "\u7AF5"], /* CJK Ideograph */
  ["\u7AF6", "\u7AF6"], /* CJK Ideograph */
  ["\u7AF7", "\u7AF7"], /* CJK Ideograph */
  ["\u7AF8", "\u7AF8"], /* CJK Ideograph */
  ["\u7AF9", "\u7AF9"], /* CJK Ideograph */
  ["\u7AFA", "\u7AFA"], /* CJK Ideograph */
  ["\u7AFB", "\u7AFB"], /* CJK Ideograph */
  ["\u7AFC", "\u7AFC"], /* CJK Ideograph */
  ["\u7AFD", "\u7AFD"], /* CJK Ideograph */
  ["\u7AFE", "\u7AFE"], /* CJK Ideograph */
  ["\u7AFF", "\u7AFF"], /* CJK Ideograph */
  ["\u7B00", "\u7B00"], /* CJK Ideograph */
  ["\u7B01", "\u7B01"], /* CJK Ideograph */
  ["\u7B02", "\u7B02"], /* CJK Ideograph */
  ["\u7B03", "\u7B03"], /* CJK Ideograph */
  ["\u7B04", "\u7B04"], /* CJK Ideograph */
  ["\u7B05", "\u7B05"], /* CJK Ideograph */
  ["\u7B06", "\u7B06"], /* CJK Ideograph */
  ["\u7B07", "\u7B07"], /* CJK Ideograph */
  ["\u7B08", "\u7B08"], /* CJK Ideograph */
  ["\u7B09", "\u7B09"], /* CJK Ideograph */
  ["\u7B0A", "\u7B0A"], /* CJK Ideograph */
  ["\u7B0B", "\u7B0B"], /* CJK Ideograph */
  ["\u7B0C", "\u7B0C"], /* CJK Ideograph */
  ["\u7B0D", "\u7B0D"], /* CJK Ideograph */
  ["\u7B0E", "\u7B0E"], /* CJK Ideograph */
  ["\u7B0F", "\u7B0F"], /* CJK Ideograph */
  ["\u7B10", "\u7B10"], /* CJK Ideograph */
  ["\u7B11", "\u7B11"], /* CJK Ideograph */
  ["\u7B12", "\u7B12"], /* CJK Ideograph */
  ["\u7B13", "\u7B13"], /* CJK Ideograph */
  ["\u7B14", "\u7B14"], /* CJK Ideograph */
  ["\u7B15", "\u7B15"], /* CJK Ideograph */
  ["\u7B16", "\u7B16"], /* CJK Ideograph */
  ["\u7B17", "\u7B17"], /* CJK Ideograph */
  ["\u7B18", "\u7B18"], /* CJK Ideograph */
  ["\u7B19", "\u7B19"], /* CJK Ideograph */
  ["\u7B1A", "\u7B1A"], /* CJK Ideograph */
  ["\u7B1B", "\u7B1B"], /* CJK Ideograph */
  ["\u7B1C", "\u7B1C"], /* CJK Ideograph */
  ["\u7B1D", "\u7B1D"], /* CJK Ideograph */
  ["\u7B1E", "\u7B1E"], /* CJK Ideograph */
  ["\u7B1F", "\u7B1F"], /* CJK Ideograph */
  ["\u7B20", "\u7B20"], /* CJK Ideograph */
  ["\u7B21", "\u7B21"], /* CJK Ideograph */
  ["\u7B22", "\u7B22"], /* CJK Ideograph */
  ["\u7B23", "\u7B23"], /* CJK Ideograph */
  ["\u7B24", "\u7B24"], /* CJK Ideograph */
  ["\u7B25", "\u7B25"], /* CJK Ideograph */
  ["\u7B26", "\u7B26"], /* CJK Ideograph */
  ["\u7B27", "\u7B27"], /* CJK Ideograph */
  ["\u7B28", "\u7B28"], /* CJK Ideograph */
  ["\u7B29", "\u7B29"], /* CJK Ideograph */
  ["\u7B2A", "\u7B2A"], /* CJK Ideograph */
  ["\u7B2B", "\u7B2B"], /* CJK Ideograph */
  ["\u7B2C", "\u7B2C"], /* CJK Ideograph */
  ["\u7B2D", "\u7B2D"], /* CJK Ideograph */
  ["\u7B2E", "\u7B2E"], /* CJK Ideograph */
  ["\u7B2F", "\u7B2F"], /* CJK Ideograph */
  ["\u7B30", "\u7B30"], /* CJK Ideograph */
  ["\u7B31", "\u7B31"], /* CJK Ideograph */
  ["\u7B32", "\u7B32"], /* CJK Ideograph */
  ["\u7B33", "\u7B33"], /* CJK Ideograph */
  ["\u7B34", "\u7B34"], /* CJK Ideograph */
  ["\u7B35", "\u7B35"], /* CJK Ideograph */
  ["\u7B36", "\u7B36"], /* CJK Ideograph */
  ["\u7B37", "\u7B37"], /* CJK Ideograph */
  ["\u7B38", "\u7B38"], /* CJK Ideograph */
  ["\u7B39", "\u7B39"], /* CJK Ideograph */
  ["\u7B3A", "\u7B3A"], /* CJK Ideograph */
  ["\u7B3B", "\u7B3B"], /* CJK Ideograph */
  ["\u7B3C", "\u7B3C"], /* CJK Ideograph */
  ["\u7B3D", "\u7B3D"], /* CJK Ideograph */
  ["\u7B3E", "\u7B3E"], /* CJK Ideograph */
  ["\u7B3F", "\u7B3F"], /* CJK Ideograph */
  ["\u7B40", "\u7B40"], /* CJK Ideograph */
  ["\u7B41", "\u7B41"], /* CJK Ideograph */
  ["\u7B42", "\u7B42"], /* CJK Ideograph */
  ["\u7B43", "\u7B43"], /* CJK Ideograph */
  ["\u7B44", "\u7B44"], /* CJK Ideograph */
  ["\u7B45", "\u7B45"], /* CJK Ideograph */
  ["\u7B46", "\u7B46"], /* CJK Ideograph */
  ["\u7B47", "\u7B47"], /* CJK Ideograph */
  ["\u7B48", "\u7B48"], /* CJK Ideograph */
  ["\u7B49", "\u7B49"], /* CJK Ideograph */
  ["\u7B4A", "\u7B4A"], /* CJK Ideograph */
  ["\u7B4B", "\u7B4B"], /* CJK Ideograph */
  ["\u7B4C", "\u7B4C"], /* CJK Ideograph */
  ["\u7B4D", "\u7B4D"], /* CJK Ideograph */
  ["\u7B4E", "\u7B4E"], /* CJK Ideograph */
  ["\u7B4F", "\u7B4F"], /* CJK Ideograph */
  ["\u7B50", "\u7B50"], /* CJK Ideograph */
  ["\u7B51", "\u7B51"], /* CJK Ideograph */
  ["\u7B52", "\u7B52"], /* CJK Ideograph */
  ["\u7B53", "\u7B53"], /* CJK Ideograph */
  ["\u7B54", "\u7B54"], /* CJK Ideograph */
  ["\u7B55", "\u7B55"], /* CJK Ideograph */
  ["\u7B56", "\u7B56"], /* CJK Ideograph */
  ["\u7B57", "\u7B57"], /* CJK Ideograph */
  ["\u7B58", "\u7B58"], /* CJK Ideograph */
  ["\u7B59", "\u7B59"], /* CJK Ideograph */
  ["\u7B5A", "\u7B5A"], /* CJK Ideograph */
  ["\u7B5B", "\u7B5B"], /* CJK Ideograph */
  ["\u7B5C", "\u7B5C"], /* CJK Ideograph */
  ["\u7B5D", "\u7B5D"], /* CJK Ideograph */
  ["\u7B5E", "\u7B5E"], /* CJK Ideograph */
  ["\u7B5F", "\u7B5F"], /* CJK Ideograph */
  ["\u7B60", "\u7B60"], /* CJK Ideograph */
  ["\u7B61", "\u7B61"], /* CJK Ideograph */
  ["\u7B62", "\u7B62"], /* CJK Ideograph */
  ["\u7B63", "\u7B63"], /* CJK Ideograph */
  ["\u7B64", "\u7B64"], /* CJK Ideograph */
  ["\u7B65", "\u7B65"], /* CJK Ideograph */
  ["\u7B66", "\u7B66"], /* CJK Ideograph */
  ["\u7B67", "\u7B67"], /* CJK Ideograph */
  ["\u7B68", "\u7B68"], /* CJK Ideograph */
  ["\u7B69", "\u7B69"], /* CJK Ideograph */
  ["\u7B6A", "\u7B6A"], /* CJK Ideograph */
  ["\u7B6B", "\u7B6B"], /* CJK Ideograph */
  ["\u7B6C", "\u7B6C"], /* CJK Ideograph */
  ["\u7B6D", "\u7B6D"], /* CJK Ideograph */
  ["\u7B6E", "\u7B6E"], /* CJK Ideograph */
  ["\u7B6F", "\u7B6F"], /* CJK Ideograph */
  ["\u7B70", "\u7B70"], /* CJK Ideograph */
  ["\u7B71", "\u7B71"], /* CJK Ideograph */
  ["\u7B72", "\u7B72"], /* CJK Ideograph */
  ["\u7B73", "\u7B73"], /* CJK Ideograph */
  ["\u7B74", "\u7B74"], /* CJK Ideograph */
  ["\u7B75", "\u7B75"], /* CJK Ideograph */
  ["\u7B76", "\u7B76"], /* CJK Ideograph */
  ["\u7B77", "\u7B77"], /* CJK Ideograph */
  ["\u7B78", "\u7B78"], /* CJK Ideograph */
  ["\u7B79", "\u7B79"], /* CJK Ideograph */
  ["\u7B7A", "\u7B7A"], /* CJK Ideograph */
  ["\u7B7B", "\u7B7B"], /* CJK Ideograph */
  ["\u7B7C", "\u7B7C"], /* CJK Ideograph */
  ["\u7B7D", "\u7B7D"], /* CJK Ideograph */
  ["\u7B7E", "\u7B7E"], /* CJK Ideograph */
  ["\u7B7F", "\u7B7F"], /* CJK Ideograph */
  ["\u7B80", "\u7B80"], /* CJK Ideograph */
  ["\u7B81", "\u7B81"], /* CJK Ideograph */
  ["\u7B82", "\u7B82"], /* CJK Ideograph */
  ["\u7B83", "\u7B83"], /* CJK Ideograph */
  ["\u7B84", "\u7B84"], /* CJK Ideograph */
  ["\u7B85", "\u7B85"], /* CJK Ideograph */
  ["\u7B86", "\u7B86"], /* CJK Ideograph */
  ["\u7B87", "\u7B87"], /* CJK Ideograph */
  ["\u7B88", "\u7B88"], /* CJK Ideograph */
  ["\u7B89", "\u7B89"], /* CJK Ideograph */
  ["\u7B8A", "\u7B8A"], /* CJK Ideograph */
  ["\u7B8B", "\u7B8B"], /* CJK Ideograph */
  ["\u7B8C", "\u7B8C"], /* CJK Ideograph */
  ["\u7B8D", "\u7B8D"], /* CJK Ideograph */
  ["\u7B8E", "\u7B8E"], /* CJK Ideograph */
  ["\u7B8F", "\u7B8F"], /* CJK Ideograph */
  ["\u7B90", "\u7B90"], /* CJK Ideograph */
  ["\u7B91", "\u7B91"], /* CJK Ideograph */
  ["\u7B92", "\u7B92"], /* CJK Ideograph */
  ["\u7B93", "\u7B93"], /* CJK Ideograph */
  ["\u7B94", "\u7B94"], /* CJK Ideograph */
  ["\u7B95", "\u7B95"], /* CJK Ideograph */
  ["\u7B96", "\u7B96"], /* CJK Ideograph */
  ["\u7B97", "\u7B97"], /* CJK Ideograph */
  ["\u7B98", "\u7B98"], /* CJK Ideograph */
  ["\u7B99", "\u7B99"], /* CJK Ideograph */
  ["\u7B9A", "\u7B9A"], /* CJK Ideograph */
  ["\u7B9B", "\u7B9B"], /* CJK Ideograph */
  ["\u7B9C", "\u7B9C"], /* CJK Ideograph */
  ["\u7B9D", "\u7B9D"], /* CJK Ideograph */
  ["\u7B9E", "\u7B9E"], /* CJK Ideograph */
  ["\u7B9F", "\u7B9F"], /* CJK Ideograph */
  ["\u7BA0", "\u7BA0"], /* CJK Ideograph */
  ["\u7BA1", "\u7BA1"], /* CJK Ideograph */
  ["\u7BA2", "\u7BA2"], /* CJK Ideograph */
  ["\u7BA3", "\u7BA3"], /* CJK Ideograph */
  ["\u7BA4", "\u7BA4"], /* CJK Ideograph */
  ["\u7BA5", "\u7BA5"], /* CJK Ideograph */
  ["\u7BA6", "\u7BA6"], /* CJK Ideograph */
  ["\u7BA7", "\u7BA7"], /* CJK Ideograph */
  ["\u7BA8", "\u7BA8"], /* CJK Ideograph */
  ["\u7BA9", "\u7BA9"], /* CJK Ideograph */
  ["\u7BAA", "\u7BAA"], /* CJK Ideograph */
  ["\u7BAB", "\u7BAB"], /* CJK Ideograph */
  ["\u7BAC", "\u7BAC"], /* CJK Ideograph */
  ["\u7BAD", "\u7BAD"], /* CJK Ideograph */
  ["\u7BAE", "\u7BAE"], /* CJK Ideograph */
  ["\u7BAF", "\u7BAF"], /* CJK Ideograph */
  ["\u7BB0", "\u7BB0"], /* CJK Ideograph */
  ["\u7BB1", "\u7BB1"], /* CJK Ideograph */
  ["\u7BB2", "\u7BB2"], /* CJK Ideograph */
  ["\u7BB3", "\u7BB3"], /* CJK Ideograph */
  ["\u7BB4", "\u7BB4"], /* CJK Ideograph */
  ["\u7BB5", "\u7BB5"], /* CJK Ideograph */
  ["\u7BB6", "\u7BB6"], /* CJK Ideograph */
  ["\u7BB7", "\u7BB7"], /* CJK Ideograph */
  ["\u7BB8", "\u7BB8"], /* CJK Ideograph */
  ["\u7BB9", "\u7BB9"], /* CJK Ideograph */
  ["\u7BBA", "\u7BBA"], /* CJK Ideograph */
  ["\u7BBB", "\u7BBB"], /* CJK Ideograph */
  ["\u7BBC", "\u7BBC"], /* CJK Ideograph */
  ["\u7BBD", "\u7BBD"], /* CJK Ideograph */
  ["\u7BBE", "\u7BBE"], /* CJK Ideograph */
  ["\u7BBF", "\u7BBF"], /* CJK Ideograph */
  ["\u7BC0", "\u7BC0"], /* CJK Ideograph */
  ["\u7BC1", "\u7BC1"], /* CJK Ideograph */
  ["\u7BC2", "\u7BC2"], /* CJK Ideograph */
  ["\u7BC3", "\u7BC3"], /* CJK Ideograph */
  ["\u7BC4", "\u7BC4"], /* CJK Ideograph */
  ["\u7BC5", "\u7BC5"], /* CJK Ideograph */
  ["\u7BC6", "\u7BC6"], /* CJK Ideograph */
  ["\u7BC7", "\u7BC7"], /* CJK Ideograph */
  ["\u7BC8", "\u7BC8"], /* CJK Ideograph */
  ["\u7BC9", "\u7BC9"], /* CJK Ideograph */
  ["\u7BCA", "\u7BCA"], /* CJK Ideograph */
  ["\u7BCB", "\u7BCB"], /* CJK Ideograph */
  ["\u7BCC", "\u7BCC"], /* CJK Ideograph */
  ["\u7BCD", "\u7BCD"], /* CJK Ideograph */
  ["\u7BCE", "\u7BCE"], /* CJK Ideograph */
  ["\u7BCF", "\u7BCF"], /* CJK Ideograph */
  ["\u7BD0", "\u7BD0"], /* CJK Ideograph */
  ["\u7BD1", "\u7BD1"], /* CJK Ideograph */
  ["\u7BD2", "\u7BD2"], /* CJK Ideograph */
  ["\u7BD3", "\u7BD3"], /* CJK Ideograph */
  ["\u7BD4", "\u7BD4"], /* CJK Ideograph */
  ["\u7BD5", "\u7BD5"], /* CJK Ideograph */
  ["\u7BD6", "\u7BD6"], /* CJK Ideograph */
  ["\u7BD7", "\u7BD7"], /* CJK Ideograph */
  ["\u7BD8", "\u7BD8"], /* CJK Ideograph */
  ["\u7BD9", "\u7BD9"], /* CJK Ideograph */
  ["\u7BDA", "\u7BDA"], /* CJK Ideograph */
  ["\u7BDB", "\u7BDB"], /* CJK Ideograph */
  ["\u7BDC", "\u7BDC"], /* CJK Ideograph */
  ["\u7BDD", "\u7BDD"], /* CJK Ideograph */
  ["\u7BDE", "\u7BDE"], /* CJK Ideograph */
  ["\u7BDF", "\u7BDF"], /* CJK Ideograph */
  ["\u7BE0", "\u7BE0"], /* CJK Ideograph */
  ["\u7BE1", "\u7BE1"], /* CJK Ideograph */
  ["\u7BE2", "\u7BE2"], /* CJK Ideograph */
  ["\u7BE3", "\u7BE3"], /* CJK Ideograph */
  ["\u7BE4", "\u7BE4"], /* CJK Ideograph */
  ["\u7BE5", "\u7BE5"], /* CJK Ideograph */
  ["\u7BE6", "\u7BE6"], /* CJK Ideograph */
  ["\u7BE7", "\u7BE7"], /* CJK Ideograph */
  ["\u7BE8", "\u7BE8"], /* CJK Ideograph */
  ["\u7BE9", "\u7BE9"], /* CJK Ideograph */
  ["\u7BEA", "\u7BEA"], /* CJK Ideograph */
  ["\u7BEB", "\u7BEB"], /* CJK Ideograph */
  ["\u7BEC", "\u7BEC"], /* CJK Ideograph */
  ["\u7BED", "\u7BED"], /* CJK Ideograph */
  ["\u7BEE", "\u7BEE"], /* CJK Ideograph */
  ["\u7BEF", "\u7BEF"], /* CJK Ideograph */
  ["\u7BF0", "\u7BF0"], /* CJK Ideograph */
  ["\u7BF1", "\u7BF1"], /* CJK Ideograph */
  ["\u7BF2", "\u7BF2"], /* CJK Ideograph */
  ["\u7BF3", "\u7BF3"], /* CJK Ideograph */
  ["\u7BF4", "\u7BF4"], /* CJK Ideograph */
  ["\u7BF5", "\u7BF5"], /* CJK Ideograph */
  ["\u7BF6", "\u7BF6"], /* CJK Ideograph */
  ["\u7BF7", "\u7BF7"], /* CJK Ideograph */
  ["\u7BF8", "\u7BF8"], /* CJK Ideograph */
  ["\u7BF9", "\u7BF9"], /* CJK Ideograph */
  ["\u7BFA", "\u7BFA"], /* CJK Ideograph */
  ["\u7BFB", "\u7BFB"], /* CJK Ideograph */
  ["\u7BFC", "\u7BFC"], /* CJK Ideograph */
  ["\u7BFD", "\u7BFD"], /* CJK Ideograph */
  ["\u7BFE", "\u7BFE"], /* CJK Ideograph */
  ["\u7BFF", "\u7BFF"], /* CJK Ideograph */
  ["\u7C00", "\u7C00"], /* CJK Ideograph */
  ["\u7C01", "\u7C01"], /* CJK Ideograph */
  ["\u7C02", "\u7C02"], /* CJK Ideograph */
  ["\u7C03", "\u7C03"], /* CJK Ideograph */
  ["\u7C04", "\u7C04"], /* CJK Ideograph */
  ["\u7C05", "\u7C05"], /* CJK Ideograph */
  ["\u7C06", "\u7C06"], /* CJK Ideograph */
  ["\u7C07", "\u7C07"], /* CJK Ideograph */
  ["\u7C08", "\u7C08"], /* CJK Ideograph */
  ["\u7C09", "\u7C09"], /* CJK Ideograph */
  ["\u7C0A", "\u7C0A"], /* CJK Ideograph */
  ["\u7C0B", "\u7C0B"], /* CJK Ideograph */
  ["\u7C0C", "\u7C0C"], /* CJK Ideograph */
  ["\u7C0D", "\u7C0D"], /* CJK Ideograph */
  ["\u7C0E", "\u7C0E"], /* CJK Ideograph */
  ["\u7C0F", "\u7C0F"], /* CJK Ideograph */
  ["\u7C10", "\u7C10"], /* CJK Ideograph */
  ["\u7C11", "\u7C11"], /* CJK Ideograph */
  ["\u7C12", "\u7C12"], /* CJK Ideograph */
  ["\u7C13", "\u7C13"], /* CJK Ideograph */
  ["\u7C14", "\u7C14"], /* CJK Ideograph */
  ["\u7C15", "\u7C15"], /* CJK Ideograph */
  ["\u7C16", "\u7C16"], /* CJK Ideograph */
  ["\u7C17", "\u7C17"], /* CJK Ideograph */
  ["\u7C18", "\u7C18"], /* CJK Ideograph */
  ["\u7C19", "\u7C19"], /* CJK Ideograph */
  ["\u7C1A", "\u7C1A"], /* CJK Ideograph */
  ["\u7C1B", "\u7C1B"], /* CJK Ideograph */
  ["\u7C1C", "\u7C1C"], /* CJK Ideograph */
  ["\u7C1D", "\u7C1D"], /* CJK Ideograph */
  ["\u7C1E", "\u7C1E"], /* CJK Ideograph */
  ["\u7C1F", "\u7C1F"], /* CJK Ideograph */
  ["\u7C20", "\u7C20"], /* CJK Ideograph */
  ["\u7C21", "\u7C21"], /* CJK Ideograph */
  ["\u7C22", "\u7C22"], /* CJK Ideograph */
  ["\u7C23", "\u7C23"], /* CJK Ideograph */
  ["\u7C24", "\u7C24"], /* CJK Ideograph */
  ["\u7C25", "\u7C25"], /* CJK Ideograph */
  ["\u7C26", "\u7C26"], /* CJK Ideograph */
  ["\u7C27", "\u7C27"], /* CJK Ideograph */
  ["\u7C28", "\u7C28"], /* CJK Ideograph */
  ["\u7C29", "\u7C29"], /* CJK Ideograph */
  ["\u7C2A", "\u7C2A"], /* CJK Ideograph */
  ["\u7C2B", "\u7C2B"], /* CJK Ideograph */
  ["\u7C2C", "\u7C2C"], /* CJK Ideograph */
  ["\u7C2D", "\u7C2D"], /* CJK Ideograph */
  ["\u7C2E", "\u7C2E"], /* CJK Ideograph */
  ["\u7C2F", "\u7C2F"], /* CJK Ideograph */
  ["\u7C30", "\u7C30"], /* CJK Ideograph */
  ["\u7C31", "\u7C31"], /* CJK Ideograph */
  ["\u7C32", "\u7C32"], /* CJK Ideograph */
  ["\u7C33", "\u7C33"], /* CJK Ideograph */
  ["\u7C34", "\u7C34"], /* CJK Ideograph */
  ["\u7C35", "\u7C35"], /* CJK Ideograph */
  ["\u7C36", "\u7C36"], /* CJK Ideograph */
  ["\u7C37", "\u7C37"], /* CJK Ideograph */
  ["\u7C38", "\u7C38"], /* CJK Ideograph */
  ["\u7C39", "\u7C39"], /* CJK Ideograph */
  ["\u7C3A", "\u7C3A"], /* CJK Ideograph */
  ["\u7C3B", "\u7C3B"], /* CJK Ideograph */
  ["\u7C3C", "\u7C3C"], /* CJK Ideograph */
  ["\u7C3D", "\u7C3D"], /* CJK Ideograph */
  ["\u7C3E", "\u7C3E"], /* CJK Ideograph */
  ["\u7C3F", "\u7C3F"], /* CJK Ideograph */
  ["\u7C40", "\u7C40"], /* CJK Ideograph */
  ["\u7C41", "\u7C41"], /* CJK Ideograph */
  ["\u7C42", "\u7C42"], /* CJK Ideograph */
  ["\u7C43", "\u7C43"], /* CJK Ideograph */
  ["\u7C44", "\u7C44"], /* CJK Ideograph */
  ["\u7C45", "\u7C45"], /* CJK Ideograph */
  ["\u7C46", "\u7C46"], /* CJK Ideograph */
  ["\u7C47", "\u7C47"], /* CJK Ideograph */
  ["\u7C48", "\u7C48"], /* CJK Ideograph */
  ["\u7C49", "\u7C49"], /* CJK Ideograph */
  ["\u7C4A", "\u7C4A"], /* CJK Ideograph */
  ["\u7C4B", "\u7C4B"], /* CJK Ideograph */
  ["\u7C4C", "\u7C4C"], /* CJK Ideograph */
  ["\u7C4D", "\u7C4D"], /* CJK Ideograph */
  ["\u7C4E", "\u7C4E"], /* CJK Ideograph */
  ["\u7C4F", "\u7C4F"], /* CJK Ideograph */
  ["\u7C50", "\u7C50"], /* CJK Ideograph */
  ["\u7C51", "\u7C51"], /* CJK Ideograph */
  ["\u7C52", "\u7C52"], /* CJK Ideograph */
  ["\u7C53", "\u7C53"], /* CJK Ideograph */
  ["\u7C54", "\u7C54"], /* CJK Ideograph */
  ["\u7C55", "\u7C55"], /* CJK Ideograph */
  ["\u7C56", "\u7C56"], /* CJK Ideograph */
  ["\u7C57", "\u7C57"], /* CJK Ideograph */
  ["\u7C58", "\u7C58"], /* CJK Ideograph */
  ["\u7C59", "\u7C59"], /* CJK Ideograph */
  ["\u7C5A", "\u7C5A"], /* CJK Ideograph */
  ["\u7C5B", "\u7C5B"], /* CJK Ideograph */
  ["\u7C5C", "\u7C5C"], /* CJK Ideograph */
  ["\u7C5D", "\u7C5D"], /* CJK Ideograph */
  ["\u7C5E", "\u7C5E"], /* CJK Ideograph */
  ["\u7C5F", "\u7C5F"], /* CJK Ideograph */
  ["\u7C60", "\u7C60"], /* CJK Ideograph */
  ["\u7C61", "\u7C61"], /* CJK Ideograph */
  ["\u7C62", "\u7C62"], /* CJK Ideograph */
  ["\u7C63", "\u7C63"], /* CJK Ideograph */
  ["\u7C64", "\u7C64"], /* CJK Ideograph */
  ["\u7C65", "\u7C65"], /* CJK Ideograph */
  ["\u7C66", "\u7C66"], /* CJK Ideograph */
  ["\u7C67", "\u7C67"], /* CJK Ideograph */
  ["\u7C68", "\u7C68"], /* CJK Ideograph */
  ["\u7C69", "\u7C69"], /* CJK Ideograph */
  ["\u7C6A", "\u7C6A"], /* CJK Ideograph */
  ["\u7C6B", "\u7C6B"], /* CJK Ideograph */
  ["\u7C6C", "\u7C6C"], /* CJK Ideograph */
  ["\u7C6D", "\u7C6D"], /* CJK Ideograph */
  ["\u7C6E", "\u7C6E"], /* CJK Ideograph */
  ["\u7C6F", "\u7C6F"], /* CJK Ideograph */
  ["\u7C70", "\u7C70"], /* CJK Ideograph */
  ["\u7C71", "\u7C71"], /* CJK Ideograph */
  ["\u7C72", "\u7C72"], /* CJK Ideograph */
  ["\u7C73", "\u7C73"], /* CJK Ideograph */
  ["\u7C74", "\u7C74"], /* CJK Ideograph */
  ["\u7C75", "\u7C75"], /* CJK Ideograph */
  ["\u7C76", "\u7C76"], /* CJK Ideograph */
  ["\u7C77", "\u7C77"], /* CJK Ideograph */
  ["\u7C78", "\u7C78"], /* CJK Ideograph */
  ["\u7C79", "\u7C79"], /* CJK Ideograph */
  ["\u7C7A", "\u7C7A"], /* CJK Ideograph */
  ["\u7C7B", "\u7C7B"], /* CJK Ideograph */
  ["\u7C7C", "\u7C7C"], /* CJK Ideograph */
  ["\u7C7D", "\u7C7D"], /* CJK Ideograph */
  ["\u7C7E", "\u7C7E"], /* CJK Ideograph */
  ["\u7C7F", "\u7C7F"], /* CJK Ideograph */
  ["\u7C80", "\u7C80"], /* CJK Ideograph */
  ["\u7C81", "\u7C81"], /* CJK Ideograph */
  ["\u7C82", "\u7C82"], /* CJK Ideograph */
  ["\u7C83", "\u7C83"], /* CJK Ideograph */
  ["\u7C84", "\u7C84"], /* CJK Ideograph */
  ["\u7C85", "\u7C85"], /* CJK Ideograph */
  ["\u7C86", "\u7C86"], /* CJK Ideograph */
  ["\u7C87", "\u7C87"], /* CJK Ideograph */
  ["\u7C88", "\u7C88"], /* CJK Ideograph */
  ["\u7C89", "\u7C89"], /* CJK Ideograph */
  ["\u7C8A", "\u7C8A"], /* CJK Ideograph */
  ["\u7C8B", "\u7C8B"], /* CJK Ideograph */
  ["\u7C8C", "\u7C8C"], /* CJK Ideograph */
  ["\u7C8D", "\u7C8D"], /* CJK Ideograph */
  ["\u7C8E", "\u7C8E"], /* CJK Ideograph */
  ["\u7C8F", "\u7C8F"], /* CJK Ideograph */
  ["\u7C90", "\u7C90"], /* CJK Ideograph */
  ["\u7C91", "\u7C91"], /* CJK Ideograph */
  ["\u7C92", "\u7C92"], /* CJK Ideograph */
  ["\u7C93", "\u7C93"], /* CJK Ideograph */
  ["\u7C94", "\u7C94"], /* CJK Ideograph */
  ["\u7C95", "\u7C95"], /* CJK Ideograph */
  ["\u7C96", "\u7C96"], /* CJK Ideograph */
  ["\u7C97", "\u7C97"], /* CJK Ideograph */
  ["\u7C98", "\u7C98"], /* CJK Ideograph */
  ["\u7C99", "\u7C99"], /* CJK Ideograph */
  ["\u7C9A", "\u7C9A"], /* CJK Ideograph */
  ["\u7C9B", "\u7C9B"], /* CJK Ideograph */
  ["\u7C9C", "\u7C9C"], /* CJK Ideograph */
  ["\u7C9D", "\u7C9D"], /* CJK Ideograph */
  ["\u7C9E", "\u7C9E"], /* CJK Ideograph */
  ["\u7C9F", "\u7C9F"], /* CJK Ideograph */
  ["\u7CA0", "\u7CA0"], /* CJK Ideograph */
  ["\u7CA1", "\u7CA1"], /* CJK Ideograph */
  ["\u7CA2", "\u7CA2"], /* CJK Ideograph */
  ["\u7CA3", "\u7CA3"], /* CJK Ideograph */
  ["\u7CA4", "\u7CA4"], /* CJK Ideograph */
  ["\u7CA5", "\u7CA5"], /* CJK Ideograph */
  ["\u7CA6", "\u7CA6"], /* CJK Ideograph */
  ["\u7CA7", "\u7CA7"], /* CJK Ideograph */
  ["\u7CA8", "\u7CA8"], /* CJK Ideograph */
  ["\u7CA9", "\u7CA9"], /* CJK Ideograph */
  ["\u7CAA", "\u7CAA"], /* CJK Ideograph */
  ["\u7CAB", "\u7CAB"], /* CJK Ideograph */
  ["\u7CAC", "\u7CAC"], /* CJK Ideograph */
  ["\u7CAD", "\u7CAD"], /* CJK Ideograph */
  ["\u7CAE", "\u7CAE"], /* CJK Ideograph */
  ["\u7CAF", "\u7CAF"], /* CJK Ideograph */
  ["\u7CB0", "\u7CB0"], /* CJK Ideograph */
  ["\u7CB1", "\u7CB1"], /* CJK Ideograph */
  ["\u7CB2", "\u7CB2"], /* CJK Ideograph */
  ["\u7CB3", "\u7CB3"], /* CJK Ideograph */
  ["\u7CB4", "\u7CB4"], /* CJK Ideograph */
  ["\u7CB5", "\u7CB5"], /* CJK Ideograph */
  ["\u7CB6", "\u7CB6"], /* CJK Ideograph */
  ["\u7CB7", "\u7CB7"], /* CJK Ideograph */
  ["\u7CB8", "\u7CB8"], /* CJK Ideograph */
  ["\u7CB9", "\u7CB9"], /* CJK Ideograph */
  ["\u7CBA", "\u7CBA"], /* CJK Ideograph */
  ["\u7CBB", "\u7CBB"], /* CJK Ideograph */
  ["\u7CBC", "\u7CBC"], /* CJK Ideograph */
  ["\u7CBD", "\u7CBD"], /* CJK Ideograph */
  ["\u7CBE", "\u7CBE"], /* CJK Ideograph */
  ["\u7CBF", "\u7CBF"], /* CJK Ideograph */
  ["\u7CC0", "\u7CC0"], /* CJK Ideograph */
  ["\u7CC1", "\u7CC1"], /* CJK Ideograph */
  ["\u7CC2", "\u7CC2"], /* CJK Ideograph */
  ["\u7CC3", "\u7CC3"], /* CJK Ideograph */
  ["\u7CC4", "\u7CC4"], /* CJK Ideograph */
  ["\u7CC5", "\u7CC5"], /* CJK Ideograph */
  ["\u7CC6", "\u7CC6"], /* CJK Ideograph */
  ["\u7CC7", "\u7CC7"], /* CJK Ideograph */
  ["\u7CC8", "\u7CC8"], /* CJK Ideograph */
  ["\u7CC9", "\u7CC9"], /* CJK Ideograph */
  ["\u7CCA", "\u7CCA"], /* CJK Ideograph */
  ["\u7CCB", "\u7CCB"], /* CJK Ideograph */
  ["\u7CCC", "\u7CCC"], /* CJK Ideograph */
  ["\u7CCD", "\u7CCD"], /* CJK Ideograph */
  ["\u7CCE", "\u7CCE"], /* CJK Ideograph */
  ["\u7CCF", "\u7CCF"], /* CJK Ideograph */
  ["\u7CD0", "\u7CD0"], /* CJK Ideograph */
  ["\u7CD1", "\u7CD1"], /* CJK Ideograph */
  ["\u7CD2", "\u7CD2"], /* CJK Ideograph */
  ["\u7CD3", "\u7CD3"], /* CJK Ideograph */
  ["\u7CD4", "\u7CD4"], /* CJK Ideograph */
  ["\u7CD5", "\u7CD5"], /* CJK Ideograph */
  ["\u7CD6", "\u7CD6"], /* CJK Ideograph */
  ["\u7CD7", "\u7CD7"], /* CJK Ideograph */
  ["\u7CD8", "\u7CD8"], /* CJK Ideograph */
  ["\u7CD9", "\u7CD9"], /* CJK Ideograph */
  ["\u7CDA", "\u7CDA"], /* CJK Ideograph */
  ["\u7CDB", "\u7CDB"], /* CJK Ideograph */
  ["\u7CDC", "\u7CDC"], /* CJK Ideograph */
  ["\u7CDD", "\u7CDD"], /* CJK Ideograph */
  ["\u7CDE", "\u7CDE"], /* CJK Ideograph */
  ["\u7CDF", "\u7CDF"], /* CJK Ideograph */
  ["\u7CE0", "\u7CE0"], /* CJK Ideograph */
  ["\u7CE1", "\u7CE1"], /* CJK Ideograph */
  ["\u7CE2", "\u7CE2"], /* CJK Ideograph */
  ["\u7CE3", "\u7CE3"], /* CJK Ideograph */
  ["\u7CE4", "\u7CE4"], /* CJK Ideograph */
  ["\u7CE5", "\u7CE5"], /* CJK Ideograph */
  ["\u7CE6", "\u7CE6"], /* CJK Ideograph */
  ["\u7CE7", "\u7CE7"], /* CJK Ideograph */
  ["\u7CE8", "\u7CE8"], /* CJK Ideograph */
  ["\u7CE9", "\u7CE9"], /* CJK Ideograph */
  ["\u7CEA", "\u7CEA"], /* CJK Ideograph */
  ["\u7CEB", "\u7CEB"], /* CJK Ideograph */
  ["\u7CEC", "\u7CEC"], /* CJK Ideograph */
  ["\u7CED", "\u7CED"], /* CJK Ideograph */
  ["\u7CEE", "\u7CEE"], /* CJK Ideograph */
  ["\u7CEF", "\u7CEF"], /* CJK Ideograph */
  ["\u7CF0", "\u7CF0"], /* CJK Ideograph */
  ["\u7CF1", "\u7CF1"], /* CJK Ideograph */
  ["\u7CF2", "\u7CF2"], /* CJK Ideograph */
  ["\u7CF3", "\u7CF3"], /* CJK Ideograph */
  ["\u7CF4", "\u7CF4"], /* CJK Ideograph */
  ["\u7CF5", "\u7CF5"], /* CJK Ideograph */
  ["\u7CF6", "\u7CF6"], /* CJK Ideograph */
  ["\u7CF7", "\u7CF7"], /* CJK Ideograph */
  ["\u7CF8", "\u7CF8"], /* CJK Ideograph */
  ["\u7CF9", "\u7CF9"], /* CJK Ideograph */
  ["\u7CFA", "\u7CFA"], /* CJK Ideograph */
  ["\u7CFB", "\u7CFB"], /* CJK Ideograph */
  ["\u7CFC", "\u7CFC"], /* CJK Ideograph */
  ["\u7CFD", "\u7CFD"], /* CJK Ideograph */
  ["\u7CFE", "\u7CFE"], /* CJK Ideograph */
  ["\u7CFF", "\u7CFF"], /* CJK Ideograph */
  ["\u7D00", "\u7D00"], /* CJK Ideograph */
  ["\u7D01", "\u7D01"], /* CJK Ideograph */
  ["\u7D02", "\u7D02"], /* CJK Ideograph */
  ["\u7D03", "\u7D03"], /* CJK Ideograph */
  ["\u7D04", "\u7D04"], /* CJK Ideograph */
  ["\u7D05", "\u7D05"], /* CJK Ideograph */
  ["\u7D06", "\u7D06"], /* CJK Ideograph */
  ["\u7D07", "\u7D07"], /* CJK Ideograph */
  ["\u7D08", "\u7D08"], /* CJK Ideograph */
  ["\u7D09", "\u7D09"], /* CJK Ideograph */
  ["\u7D0A", "\u7D0A"], /* CJK Ideograph */
  ["\u7D0B", "\u7D0B"], /* CJK Ideograph */
  ["\u7D0C", "\u7D0C"], /* CJK Ideograph */
  ["\u7D0D", "\u7D0D"], /* CJK Ideograph */
  ["\u7D0E", "\u7D0E"], /* CJK Ideograph */
  ["\u7D0F", "\u7D0F"], /* CJK Ideograph */
  ["\u7D10", "\u7D10"], /* CJK Ideograph */
  ["\u7D11", "\u7D11"], /* CJK Ideograph */
  ["\u7D12", "\u7D12"], /* CJK Ideograph */
  ["\u7D13", "\u7D13"], /* CJK Ideograph */
  ["\u7D14", "\u7D14"], /* CJK Ideograph */
  ["\u7D15", "\u7D15"], /* CJK Ideograph */
  ["\u7D16", "\u7D16"], /* CJK Ideograph */
  ["\u7D17", "\u7D17"], /* CJK Ideograph */
  ["\u7D18", "\u7D18"], /* CJK Ideograph */
  ["\u7D19", "\u7D19"], /* CJK Ideograph */
  ["\u7D1A", "\u7D1A"], /* CJK Ideograph */
  ["\u7D1B", "\u7D1B"], /* CJK Ideograph */
  ["\u7D1C", "\u7D1C"], /* CJK Ideograph */
  ["\u7D1D", "\u7D1D"], /* CJK Ideograph */
  ["\u7D1E", "\u7D1E"], /* CJK Ideograph */
  ["\u7D1F", "\u7D1F"], /* CJK Ideograph */
  ["\u7D20", "\u7D20"], /* CJK Ideograph */
  ["\u7D21", "\u7D21"], /* CJK Ideograph */
  ["\u7D22", "\u7D22"], /* CJK Ideograph */
  ["\u7D23", "\u7D23"], /* CJK Ideograph */
  ["\u7D24", "\u7D24"], /* CJK Ideograph */
  ["\u7D25", "\u7D25"], /* CJK Ideograph */
  ["\u7D26", "\u7D26"], /* CJK Ideograph */
  ["\u7D27", "\u7D27"], /* CJK Ideograph */
  ["\u7D28", "\u7D28"], /* CJK Ideograph */
  ["\u7D29", "\u7D29"], /* CJK Ideograph */
  ["\u7D2A", "\u7D2A"], /* CJK Ideograph */
  ["\u7D2B", "\u7D2B"], /* CJK Ideograph */
  ["\u7D2C", "\u7D2C"], /* CJK Ideograph */
  ["\u7D2D", "\u7D2D"], /* CJK Ideograph */
  ["\u7D2E", "\u7D2E"], /* CJK Ideograph */
  ["\u7D2F", "\u7D2F"], /* CJK Ideograph */
  ["\u7D30", "\u7D30"], /* CJK Ideograph */
  ["\u7D31", "\u7D31"], /* CJK Ideograph */
  ["\u7D32", "\u7D32"], /* CJK Ideograph */
  ["\u7D33", "\u7D33"], /* CJK Ideograph */
  ["\u7D34", "\u7D34"], /* CJK Ideograph */
  ["\u7D35", "\u7D35"], /* CJK Ideograph */
  ["\u7D36", "\u7D36"], /* CJK Ideograph */
  ["\u7D37", "\u7D37"], /* CJK Ideograph */
  ["\u7D38", "\u7D38"], /* CJK Ideograph */
  ["\u7D39", "\u7D39"], /* CJK Ideograph */
  ["\u7D3A", "\u7D3A"], /* CJK Ideograph */
  ["\u7D3B", "\u7D3B"], /* CJK Ideograph */
  ["\u7D3C", "\u7D3C"], /* CJK Ideograph */
  ["\u7D3D", "\u7D3D"], /* CJK Ideograph */
  ["\u7D3E", "\u7D3E"], /* CJK Ideograph */
  ["\u7D3F", "\u7D3F"], /* CJK Ideograph */
  ["\u7D40", "\u7D40"], /* CJK Ideograph */
  ["\u7D41", "\u7D41"], /* CJK Ideograph */
  ["\u7D42", "\u7D42"], /* CJK Ideograph */
  ["\u7D43", "\u7D43"], /* CJK Ideograph */
  ["\u7D44", "\u7D44"], /* CJK Ideograph */
  ["\u7D45", "\u7D45"], /* CJK Ideograph */
  ["\u7D46", "\u7D46"], /* CJK Ideograph */
  ["\u7D47", "\u7D47"], /* CJK Ideograph */
  ["\u7D48", "\u7D48"], /* CJK Ideograph */
  ["\u7D49", "\u7D49"], /* CJK Ideograph */
  ["\u7D4A", "\u7D4A"], /* CJK Ideograph */
  ["\u7D4B", "\u7D4B"], /* CJK Ideograph */
  ["\u7D4C", "\u7D4C"], /* CJK Ideograph */
  ["\u7D4D", "\u7D4D"], /* CJK Ideograph */
  ["\u7D4E", "\u7D4E"], /* CJK Ideograph */
  ["\u7D4F", "\u7D4F"], /* CJK Ideograph */
  ["\u7D50", "\u7D50"], /* CJK Ideograph */
  ["\u7D51", "\u7D51"], /* CJK Ideograph */
  ["\u7D52", "\u7D52"], /* CJK Ideograph */
  ["\u7D53", "\u7D53"], /* CJK Ideograph */
  ["\u7D54", "\u7D54"], /* CJK Ideograph */
  ["\u7D55", "\u7D55"], /* CJK Ideograph */
  ["\u7D56", "\u7D56"], /* CJK Ideograph */
  ["\u7D57", "\u7D57"], /* CJK Ideograph */
  ["\u7D58", "\u7D58"], /* CJK Ideograph */
  ["\u7D59", "\u7D59"], /* CJK Ideograph */
  ["\u7D5A", "\u7D5A"], /* CJK Ideograph */
  ["\u7D5B", "\u7D5B"], /* CJK Ideograph */
  ["\u7D5C", "\u7D5C"], /* CJK Ideograph */
  ["\u7D5D", "\u7D5D"], /* CJK Ideograph */
  ["\u7D5E", "\u7D5E"], /* CJK Ideograph */
  ["\u7D5F", "\u7D5F"], /* CJK Ideograph */
  ["\u7D60", "\u7D60"], /* CJK Ideograph */
  ["\u7D61", "\u7D61"], /* CJK Ideograph */
  ["\u7D62", "\u7D62"], /* CJK Ideograph */
  ["\u7D63", "\u7D63"], /* CJK Ideograph */
  ["\u7D64", "\u7D64"], /* CJK Ideograph */
  ["\u7D65", "\u7D65"], /* CJK Ideograph */
  ["\u7D66", "\u7D66"], /* CJK Ideograph */
  ["\u7D67", "\u7D67"], /* CJK Ideograph */
  ["\u7D68", "\u7D68"], /* CJK Ideograph */
  ["\u7D69", "\u7D69"], /* CJK Ideograph */
  ["\u7D6A", "\u7D6A"], /* CJK Ideograph */
  ["\u7D6B", "\u7D6B"], /* CJK Ideograph */
  ["\u7D6C", "\u7D6C"], /* CJK Ideograph */
  ["\u7D6D", "\u7D6D"], /* CJK Ideograph */
  ["\u7D6E", "\u7D6E"], /* CJK Ideograph */
  ["\u7D6F", "\u7D6F"], /* CJK Ideograph */
  ["\u7D70", "\u7D70"], /* CJK Ideograph */
  ["\u7D71", "\u7D71"], /* CJK Ideograph */
  ["\u7D72", "\u7D72"], /* CJK Ideograph */
  ["\u7D73", "\u7D73"], /* CJK Ideograph */
  ["\u7D74", "\u7D74"], /* CJK Ideograph */
  ["\u7D75", "\u7D75"], /* CJK Ideograph */
  ["\u7D76", "\u7D76"], /* CJK Ideograph */
  ["\u7D77", "\u7D77"], /* CJK Ideograph */
  ["\u7D78", "\u7D78"], /* CJK Ideograph */
  ["\u7D79", "\u7D79"], /* CJK Ideograph */
  ["\u7D7A", "\u7D7A"], /* CJK Ideograph */
  ["\u7D7B", "\u7D7B"], /* CJK Ideograph */
  ["\u7D7C", "\u7D7C"], /* CJK Ideograph */
  ["\u7D7D", "\u7D7D"], /* CJK Ideograph */
  ["\u7D7E", "\u7D7E"], /* CJK Ideograph */
  ["\u7D7F", "\u7D7F"], /* CJK Ideograph */
  ["\u7D80", "\u7D80"], /* CJK Ideograph */
  ["\u7D81", "\u7D81"], /* CJK Ideograph */
  ["\u7D82", "\u7D82"], /* CJK Ideograph */
  ["\u7D83", "\u7D83"], /* CJK Ideograph */
  ["\u7D84", "\u7D84"], /* CJK Ideograph */
  ["\u7D85", "\u7D85"], /* CJK Ideograph */
  ["\u7D86", "\u7D86"], /* CJK Ideograph */
  ["\u7D87", "\u7D87"], /* CJK Ideograph */
  ["\u7D88", "\u7D88"], /* CJK Ideograph */
  ["\u7D89", "\u7D89"], /* CJK Ideograph */
  ["\u7D8A", "\u7D8A"], /* CJK Ideograph */
  ["\u7D8B", "\u7D8B"], /* CJK Ideograph */
  ["\u7D8C", "\u7D8C"], /* CJK Ideograph */
  ["\u7D8D", "\u7D8D"], /* CJK Ideograph */
  ["\u7D8E", "\u7D8E"], /* CJK Ideograph */
  ["\u7D8F", "\u7D8F"], /* CJK Ideograph */
  ["\u7D90", "\u7D90"], /* CJK Ideograph */
  ["\u7D91", "\u7D91"], /* CJK Ideograph */
  ["\u7D92", "\u7D92"], /* CJK Ideograph */
  ["\u7D93", "\u7D93"], /* CJK Ideograph */
  ["\u7D94", "\u7D94"], /* CJK Ideograph */
  ["\u7D95", "\u7D95"], /* CJK Ideograph */
  ["\u7D96", "\u7D96"], /* CJK Ideograph */
  ["\u7D97", "\u7D97"], /* CJK Ideograph */
  ["\u7D98", "\u7D98"], /* CJK Ideograph */
  ["\u7D99", "\u7D99"], /* CJK Ideograph */
  ["\u7D9A", "\u7D9A"], /* CJK Ideograph */
  ["\u7D9B", "\u7D9B"], /* CJK Ideograph */
  ["\u7D9C", "\u7D9C"], /* CJK Ideograph */
  ["\u7D9D", "\u7D9D"], /* CJK Ideograph */
  ["\u7D9E", "\u7D9E"], /* CJK Ideograph */
  ["\u7D9F", "\u7D9F"], /* CJK Ideograph */
  ["\u7DA0", "\u7DA0"], /* CJK Ideograph */
  ["\u7DA1", "\u7DA1"], /* CJK Ideograph */
  ["\u7DA2", "\u7DA2"], /* CJK Ideograph */
  ["\u7DA3", "\u7DA3"], /* CJK Ideograph */
  ["\u7DA4", "\u7DA4"], /* CJK Ideograph */
  ["\u7DA5", "\u7DA5"], /* CJK Ideograph */
  ["\u7DA6", "\u7DA6"], /* CJK Ideograph */
  ["\u7DA7", "\u7DA7"], /* CJK Ideograph */
  ["\u7DA8", "\u7DA8"], /* CJK Ideograph */
  ["\u7DA9", "\u7DA9"], /* CJK Ideograph */
  ["\u7DAA", "\u7DAA"], /* CJK Ideograph */
  ["\u7DAB", "\u7DAB"], /* CJK Ideograph */
  ["\u7DAC", "\u7DAC"], /* CJK Ideograph */
  ["\u7DAD", "\u7DAD"], /* CJK Ideograph */
  ["\u7DAE", "\u7DAE"], /* CJK Ideograph */
  ["\u7DAF", "\u7DAF"], /* CJK Ideograph */
  ["\u7DB0", "\u7DB0"], /* CJK Ideograph */
  ["\u7DB1", "\u7DB1"], /* CJK Ideograph */
  ["\u7DB2", "\u7DB2"], /* CJK Ideograph */
  ["\u7DB3", "\u7DB3"], /* CJK Ideograph */
  ["\u7DB4", "\u7DB4"], /* CJK Ideograph */
  ["\u7DB5", "\u7DB5"], /* CJK Ideograph */
  ["\u7DB6", "\u7DB6"], /* CJK Ideograph */
  ["\u7DB7", "\u7DB7"], /* CJK Ideograph */
  ["\u7DB8", "\u7DB8"], /* CJK Ideograph */
  ["\u7DB9", "\u7DB9"], /* CJK Ideograph */
  ["\u7DBA", "\u7DBA"], /* CJK Ideograph */
  ["\u7DBB", "\u7DBB"], /* CJK Ideograph */
  ["\u7DBC", "\u7DBC"], /* CJK Ideograph */
  ["\u7DBD", "\u7DBD"], /* CJK Ideograph */
  ["\u7DBE", "\u7DBE"], /* CJK Ideograph */
  ["\u7DBF", "\u7DBF"], /* CJK Ideograph */
  ["\u7DC0", "\u7DC0"], /* CJK Ideograph */
  ["\u7DC1", "\u7DC1"], /* CJK Ideograph */
  ["\u7DC2", "\u7DC2"], /* CJK Ideograph */
  ["\u7DC3", "\u7DC3"], /* CJK Ideograph */
  ["\u7DC4", "\u7DC4"], /* CJK Ideograph */
  ["\u7DC5", "\u7DC5"], /* CJK Ideograph */
  ["\u7DC6", "\u7DC6"], /* CJK Ideograph */
  ["\u7DC7", "\u7DC7"], /* CJK Ideograph */
  ["\u7DC8", "\u7DC8"], /* CJK Ideograph */
  ["\u7DC9", "\u7DC9"], /* CJK Ideograph */
  ["\u7DCA", "\u7DCA"], /* CJK Ideograph */
  ["\u7DCB", "\u7DCB"], /* CJK Ideograph */
  ["\u7DCC", "\u7DCC"], /* CJK Ideograph */
  ["\u7DCD", "\u7DCD"], /* CJK Ideograph */
  ["\u7DCE", "\u7DCE"], /* CJK Ideograph */
  ["\u7DCF", "\u7DCF"], /* CJK Ideograph */
  ["\u7DD0", "\u7DD0"], /* CJK Ideograph */
  ["\u7DD1", "\u7DD1"], /* CJK Ideograph */
  ["\u7DD2", "\u7DD2"], /* CJK Ideograph */
  ["\u7DD3", "\u7DD3"], /* CJK Ideograph */
  ["\u7DD4", "\u7DD4"], /* CJK Ideograph */
  ["\u7DD5", "\u7DD5"], /* CJK Ideograph */
  ["\u7DD6", "\u7DD6"], /* CJK Ideograph */
  ["\u7DD7", "\u7DD7"], /* CJK Ideograph */
  ["\u7DD8", "\u7DD8"], /* CJK Ideograph */
  ["\u7DD9", "\u7DD9"], /* CJK Ideograph */
  ["\u7DDA", "\u7DDA"], /* CJK Ideograph */
  ["\u7DDB", "\u7DDB"], /* CJK Ideograph */
  ["\u7DDC", "\u7DDC"], /* CJK Ideograph */
  ["\u7DDD", "\u7DDD"], /* CJK Ideograph */
  ["\u7DDE", "\u7DDE"], /* CJK Ideograph */
  ["\u7DDF", "\u7DDF"], /* CJK Ideograph */
  ["\u7DE0", "\u7DE0"], /* CJK Ideograph */
  ["\u7DE1", "\u7DE1"], /* CJK Ideograph */
  ["\u7DE2", "\u7DE2"], /* CJK Ideograph */
  ["\u7DE3", "\u7DE3"], /* CJK Ideograph */
  ["\u7DE4", "\u7DE4"], /* CJK Ideograph */
  ["\u7DE5", "\u7DE5"], /* CJK Ideograph */
  ["\u7DE6", "\u7DE6"], /* CJK Ideograph */
  ["\u7DE7", "\u7DE7"], /* CJK Ideograph */
  ["\u7DE8", "\u7DE8"], /* CJK Ideograph */
  ["\u7DE9", "\u7DE9"], /* CJK Ideograph */
  ["\u7DEA", "\u7DEA"], /* CJK Ideograph */
  ["\u7DEB", "\u7DEB"], /* CJK Ideograph */
  ["\u7DEC", "\u7DEC"], /* CJK Ideograph */
  ["\u7DED", "\u7DED"], /* CJK Ideograph */
  ["\u7DEE", "\u7DEE"], /* CJK Ideograph */
  ["\u7DEF", "\u7DEF"], /* CJK Ideograph */
  ["\u7DF0", "\u7DF0"], /* CJK Ideograph */
  ["\u7DF1", "\u7DF1"], /* CJK Ideograph */
  ["\u7DF2", "\u7DF2"], /* CJK Ideograph */
  ["\u7DF3", "\u7DF3"], /* CJK Ideograph */
  ["\u7DF4", "\u7DF4"], /* CJK Ideograph */
  ["\u7DF5", "\u7DF5"], /* CJK Ideograph */
  ["\u7DF6", "\u7DF6"], /* CJK Ideograph */
  ["\u7DF7", "\u7DF7"], /* CJK Ideograph */
  ["\u7DF8", "\u7DF8"], /* CJK Ideograph */
  ["\u7DF9", "\u7DF9"], /* CJK Ideograph */
  ["\u7DFA", "\u7DFA"], /* CJK Ideograph */
  ["\u7DFB", "\u7DFB"], /* CJK Ideograph */
  ["\u7DFC", "\u7DFC"], /* CJK Ideograph */
  ["\u7DFD", "\u7DFD"], /* CJK Ideograph */
  ["\u7DFE", "\u7DFE"], /* CJK Ideograph */
  ["\u7DFF", "\u7DFF"], /* CJK Ideograph */
  ["\u7E00", "\u7E00"], /* CJK Ideograph */
  ["\u7E01", "\u7E01"], /* CJK Ideograph */
  ["\u7E02", "\u7E02"], /* CJK Ideograph */
  ["\u7E03", "\u7E03"], /* CJK Ideograph */
  ["\u7E04", "\u7E04"], /* CJK Ideograph */
  ["\u7E05", "\u7E05"], /* CJK Ideograph */
  ["\u7E06", "\u7E06"], /* CJK Ideograph */
  ["\u7E07", "\u7E07"], /* CJK Ideograph */
  ["\u7E08", "\u7E08"], /* CJK Ideograph */
  ["\u7E09", "\u7E09"], /* CJK Ideograph */
  ["\u7E0A", "\u7E0A"], /* CJK Ideograph */
  ["\u7E0B", "\u7E0B"], /* CJK Ideograph */
  ["\u7E0C", "\u7E0C"], /* CJK Ideograph */
  ["\u7E0D", "\u7E0D"], /* CJK Ideograph */
  ["\u7E0E", "\u7E0E"], /* CJK Ideograph */
  ["\u7E0F", "\u7E0F"], /* CJK Ideograph */
  ["\u7E10", "\u7E10"], /* CJK Ideograph */
  ["\u7E11", "\u7E11"], /* CJK Ideograph */
  ["\u7E12", "\u7E12"], /* CJK Ideograph */
  ["\u7E13", "\u7E13"], /* CJK Ideograph */
  ["\u7E14", "\u7E14"], /* CJK Ideograph */
  ["\u7E15", "\u7E15"], /* CJK Ideograph */
  ["\u7E16", "\u7E16"], /* CJK Ideograph */
  ["\u7E17", "\u7E17"], /* CJK Ideograph */
  ["\u7E18", "\u7E18"], /* CJK Ideograph */
  ["\u7E19", "\u7E19"], /* CJK Ideograph */
  ["\u7E1A", "\u7E1A"], /* CJK Ideograph */
  ["\u7E1B", "\u7E1B"], /* CJK Ideograph */
  ["\u7E1C", "\u7E1C"], /* CJK Ideograph */
  ["\u7E1D", "\u7E1D"], /* CJK Ideograph */
  ["\u7E1E", "\u7E1E"], /* CJK Ideograph */
  ["\u7E1F", "\u7E1F"], /* CJK Ideograph */
  ["\u7E20", "\u7E20"], /* CJK Ideograph */
  ["\u7E21", "\u7E21"], /* CJK Ideograph */
  ["\u7E22", "\u7E22"], /* CJK Ideograph */
  ["\u7E23", "\u7E23"], /* CJK Ideograph */
  ["\u7E24", "\u7E24"], /* CJK Ideograph */
  ["\u7E25", "\u7E25"], /* CJK Ideograph */
  ["\u7E26", "\u7E26"], /* CJK Ideograph */
  ["\u7E27", "\u7E27"], /* CJK Ideograph */
  ["\u7E28", "\u7E28"], /* CJK Ideograph */
  ["\u7E29", "\u7E29"], /* CJK Ideograph */
  ["\u7E2A", "\u7E2A"], /* CJK Ideograph */
  ["\u7E2B", "\u7E2B"], /* CJK Ideograph */
  ["\u7E2C", "\u7E2C"], /* CJK Ideograph */
  ["\u7E2D", "\u7E2D"], /* CJK Ideograph */
  ["\u7E2E", "\u7E2E"], /* CJK Ideograph */
  ["\u7E2F", "\u7E2F"], /* CJK Ideograph */
  ["\u7E30", "\u7E30"], /* CJK Ideograph */
  ["\u7E31", "\u7E31"], /* CJK Ideograph */
  ["\u7E32", "\u7E32"], /* CJK Ideograph */
  ["\u7E33", "\u7E33"], /* CJK Ideograph */
  ["\u7E34", "\u7E34"], /* CJK Ideograph */
  ["\u7E35", "\u7E35"], /* CJK Ideograph */
  ["\u7E36", "\u7E36"], /* CJK Ideograph */
  ["\u7E37", "\u7E37"], /* CJK Ideograph */
  ["\u7E38", "\u7E38"], /* CJK Ideograph */
  ["\u7E39", "\u7E39"], /* CJK Ideograph */
  ["\u7E3A", "\u7E3A"], /* CJK Ideograph */
  ["\u7E3B", "\u7E3B"], /* CJK Ideograph */
  ["\u7E3C", "\u7E3C"], /* CJK Ideograph */
  ["\u7E3D", "\u7E3D"], /* CJK Ideograph */
  ["\u7E3E", "\u7E3E"], /* CJK Ideograph */
  ["\u7E3F", "\u7E3F"], /* CJK Ideograph */
  ["\u7E40", "\u7E40"], /* CJK Ideograph */
  ["\u7E41", "\u7E41"], /* CJK Ideograph */
  ["\u7E42", "\u7E42"], /* CJK Ideograph */
  ["\u7E43", "\u7E43"], /* CJK Ideograph */
  ["\u7E44", "\u7E44"], /* CJK Ideograph */
  ["\u7E45", "\u7E45"], /* CJK Ideograph */
  ["\u7E46", "\u7E46"], /* CJK Ideograph */
  ["\u7E47", "\u7E47"], /* CJK Ideograph */
  ["\u7E48", "\u7E48"], /* CJK Ideograph */
  ["\u7E49", "\u7E49"], /* CJK Ideograph */
  ["\u7E4A", "\u7E4A"], /* CJK Ideograph */
  ["\u7E4B", "\u7E4B"], /* CJK Ideograph */
  ["\u7E4C", "\u7E4C"], /* CJK Ideograph */
  ["\u7E4D", "\u7E4D"], /* CJK Ideograph */
  ["\u7E4E", "\u7E4E"], /* CJK Ideograph */
  ["\u7E4F", "\u7E4F"], /* CJK Ideograph */
  ["\u7E50", "\u7E50"], /* CJK Ideograph */
  ["\u7E51", "\u7E51"], /* CJK Ideograph */
  ["\u7E52", "\u7E52"], /* CJK Ideograph */
  ["\u7E53", "\u7E53"], /* CJK Ideograph */
  ["\u7E54", "\u7E54"], /* CJK Ideograph */
  ["\u7E55", "\u7E55"], /* CJK Ideograph */
  ["\u7E56", "\u7E56"], /* CJK Ideograph */
  ["\u7E57", "\u7E57"], /* CJK Ideograph */
  ["\u7E58", "\u7E58"], /* CJK Ideograph */
  ["\u7E59", "\u7E59"], /* CJK Ideograph */
  ["\u7E5A", "\u7E5A"], /* CJK Ideograph */
  ["\u7E5B", "\u7E5B"], /* CJK Ideograph */
  ["\u7E5C", "\u7E5C"], /* CJK Ideograph */
  ["\u7E5D", "\u7E5D"], /* CJK Ideograph */
  ["\u7E5E", "\u7E5E"], /* CJK Ideograph */
  ["\u7E5F", "\u7E5F"], /* CJK Ideograph */
  ["\u7E60", "\u7E60"], /* CJK Ideograph */
  ["\u7E61", "\u7E61"], /* CJK Ideograph */
  ["\u7E62", "\u7E62"], /* CJK Ideograph */
  ["\u7E63", "\u7E63"], /* CJK Ideograph */
  ["\u7E64", "\u7E64"], /* CJK Ideograph */
  ["\u7E65", "\u7E65"], /* CJK Ideograph */
  ["\u7E66", "\u7E66"], /* CJK Ideograph */
  ["\u7E67", "\u7E67"], /* CJK Ideograph */
  ["\u7E68", "\u7E68"], /* CJK Ideograph */
  ["\u7E69", "\u7E69"], /* CJK Ideograph */
  ["\u7E6A", "\u7E6A"], /* CJK Ideograph */
  ["\u7E6B", "\u7E6B"], /* CJK Ideograph */
  ["\u7E6C", "\u7E6C"], /* CJK Ideograph */
  ["\u7E6D", "\u7E6D"], /* CJK Ideograph */
  ["\u7E6E", "\u7E6E"], /* CJK Ideograph */
  ["\u7E6F", "\u7E6F"], /* CJK Ideograph */
  ["\u7E70", "\u7E70"], /* CJK Ideograph */
  ["\u7E71", "\u7E71"], /* CJK Ideograph */
  ["\u7E72", "\u7E72"], /* CJK Ideograph */
  ["\u7E73", "\u7E73"], /* CJK Ideograph */
  ["\u7E74", "\u7E74"], /* CJK Ideograph */
  ["\u7E75", "\u7E75"], /* CJK Ideograph */
  ["\u7E76", "\u7E76"], /* CJK Ideograph */
  ["\u7E77", "\u7E77"], /* CJK Ideograph */
  ["\u7E78", "\u7E78"], /* CJK Ideograph */
  ["\u7E79", "\u7E79"], /* CJK Ideograph */
  ["\u7E7A", "\u7E7A"], /* CJK Ideograph */
  ["\u7E7B", "\u7E7B"], /* CJK Ideograph */
  ["\u7E7C", "\u7E7C"], /* CJK Ideograph */
  ["\u7E7D", "\u7E7D"], /* CJK Ideograph */
  ["\u7E7E", "\u7E7E"], /* CJK Ideograph */
  ["\u7E7F", "\u7E7F"], /* CJK Ideograph */
  ["\u7E80", "\u7E80"], /* CJK Ideograph */
  ["\u7E81", "\u7E81"], /* CJK Ideograph */
  ["\u7E82", "\u7E82"], /* CJK Ideograph */
  ["\u7E83", "\u7E83"], /* CJK Ideograph */
  ["\u7E84", "\u7E84"], /* CJK Ideograph */
  ["\u7E85", "\u7E85"], /* CJK Ideograph */
  ["\u7E86", "\u7E86"], /* CJK Ideograph */
  ["\u7E87", "\u7E87"], /* CJK Ideograph */
  ["\u7E88", "\u7E88"], /* CJK Ideograph */
  ["\u7E89", "\u7E89"], /* CJK Ideograph */
  ["\u7E8A", "\u7E8A"], /* CJK Ideograph */
  ["\u7E8B", "\u7E8B"], /* CJK Ideograph */
  ["\u7E8C", "\u7E8C"], /* CJK Ideograph */
  ["\u7E8D", "\u7E8D"], /* CJK Ideograph */
  ["\u7E8E", "\u7E8E"], /* CJK Ideograph */
  ["\u7E8F", "\u7E8F"], /* CJK Ideograph */
  ["\u7E90", "\u7E90"], /* CJK Ideograph */
  ["\u7E91", "\u7E91"], /* CJK Ideograph */
  ["\u7E92", "\u7E92"], /* CJK Ideograph */
  ["\u7E93", "\u7E93"], /* CJK Ideograph */
  ["\u7E94", "\u7E94"], /* CJK Ideograph */
  ["\u7E95", "\u7E95"], /* CJK Ideograph */
  ["\u7E96", "\u7E96"], /* CJK Ideograph */
  ["\u7E97", "\u7E97"], /* CJK Ideograph */
  ["\u7E98", "\u7E98"], /* CJK Ideograph */
  ["\u7E99", "\u7E99"], /* CJK Ideograph */
  ["\u7E9A", "\u7E9A"], /* CJK Ideograph */
  ["\u7E9B", "\u7E9B"], /* CJK Ideograph */
  ["\u7E9C", "\u7E9C"], /* CJK Ideograph */
  ["\u7E9D", "\u7E9D"], /* CJK Ideograph */
  ["\u7E9E", "\u7E9E"], /* CJK Ideograph */
  ["\u7E9F", "\u7E9F"], /* CJK Ideograph */
  ["\u7EA0", "\u7EA0"], /* CJK Ideograph */
  ["\u7EA1", "\u7EA1"], /* CJK Ideograph */
  ["\u7EA2", "\u7EA2"], /* CJK Ideograph */
  ["\u7EA3", "\u7EA3"], /* CJK Ideograph */
  ["\u7EA4", "\u7EA4"], /* CJK Ideograph */
  ["\u7EA5", "\u7EA5"], /* CJK Ideograph */
  ["\u7EA6", "\u7EA6"], /* CJK Ideograph */
  ["\u7EA7", "\u7EA7"], /* CJK Ideograph */
  ["\u7EA8", "\u7EA8"], /* CJK Ideograph */
  ["\u7EA9", "\u7EA9"], /* CJK Ideograph */
  ["\u7EAA", "\u7EAA"], /* CJK Ideograph */
  ["\u7EAB", "\u7EAB"], /* CJK Ideograph */
  ["\u7EAC", "\u7EAC"], /* CJK Ideograph */
  ["\u7EAD", "\u7EAD"], /* CJK Ideograph */
  ["\u7EAE", "\u7EAE"], /* CJK Ideograph */
  ["\u7EAF", "\u7EAF"], /* CJK Ideograph */
  ["\u7EB0", "\u7EB0"], /* CJK Ideograph */
  ["\u7EB1", "\u7EB1"], /* CJK Ideograph */
  ["\u7EB2", "\u7EB2"], /* CJK Ideograph */
  ["\u7EB3", "\u7EB3"], /* CJK Ideograph */
  ["\u7EB4", "\u7EB4"], /* CJK Ideograph */
  ["\u7EB5", "\u7EB5"], /* CJK Ideograph */
  ["\u7EB6", "\u7EB6"], /* CJK Ideograph */
  ["\u7EB7", "\u7EB7"], /* CJK Ideograph */
  ["\u7EB8", "\u7EB8"], /* CJK Ideograph */
  ["\u7EB9", "\u7EB9"], /* CJK Ideograph */
  ["\u7EBA", "\u7EBA"], /* CJK Ideograph */
  ["\u7EBB", "\u7EBB"], /* CJK Ideograph */
  ["\u7EBC", "\u7EBC"], /* CJK Ideograph */
  ["\u7EBD", "\u7EBD"], /* CJK Ideograph */
  ["\u7EBE", "\u7EBE"], /* CJK Ideograph */
  ["\u7EBF", "\u7EBF"], /* CJK Ideograph */
  ["\u7EC0", "\u7EC0"], /* CJK Ideograph */
  ["\u7EC1", "\u7EC1"], /* CJK Ideograph */
  ["\u7EC2", "\u7EC2"], /* CJK Ideograph */
  ["\u7EC3", "\u7EC3"], /* CJK Ideograph */
  ["\u7EC4", "\u7EC4"], /* CJK Ideograph */
  ["\u7EC5", "\u7EC5"], /* CJK Ideograph */
  ["\u7EC6", "\u7EC6"], /* CJK Ideograph */
  ["\u7EC7", "\u7EC7"], /* CJK Ideograph */
  ["\u7EC8", "\u7EC8"], /* CJK Ideograph */
  ["\u7EC9", "\u7EC9"], /* CJK Ideograph */
  ["\u7ECA", "\u7ECA"], /* CJK Ideograph */
  ["\u7ECB", "\u7ECB"], /* CJK Ideograph */
  ["\u7ECC", "\u7ECC"], /* CJK Ideograph */
  ["\u7ECD", "\u7ECD"], /* CJK Ideograph */
  ["\u7ECE", "\u7ECE"], /* CJK Ideograph */
  ["\u7ECF", "\u7ECF"], /* CJK Ideograph */
  ["\u7ED0", "\u7ED0"], /* CJK Ideograph */
  ["\u7ED1", "\u7ED1"], /* CJK Ideograph */
  ["\u7ED2", "\u7ED2"], /* CJK Ideograph */
  ["\u7ED3", "\u7ED3"], /* CJK Ideograph */
  ["\u7ED4", "\u7ED4"], /* CJK Ideograph */
  ["\u7ED5", "\u7ED5"], /* CJK Ideograph */
  ["\u7ED6", "\u7ED6"], /* CJK Ideograph */
  ["\u7ED7", "\u7ED7"], /* CJK Ideograph */
  ["\u7ED8", "\u7ED8"], /* CJK Ideograph */
  ["\u7ED9", "\u7ED9"], /* CJK Ideograph */
  ["\u7EDA", "\u7EDA"], /* CJK Ideograph */
  ["\u7EDB", "\u7EDB"], /* CJK Ideograph */
  ["\u7EDC", "\u7EDC"], /* CJK Ideograph */
  ["\u7EDD", "\u7EDD"], /* CJK Ideograph */
  ["\u7EDE", "\u7EDE"], /* CJK Ideograph */
  ["\u7EDF", "\u7EDF"], /* CJK Ideograph */
  ["\u7EE0", "\u7EE0"], /* CJK Ideograph */
  ["\u7EE1", "\u7EE1"], /* CJK Ideograph */
  ["\u7EE2", "\u7EE2"], /* CJK Ideograph */
  ["\u7EE3", "\u7EE3"], /* CJK Ideograph */
  ["\u7EE4", "\u7EE4"], /* CJK Ideograph */
  ["\u7EE5", "\u7EE5"], /* CJK Ideograph */
  ["\u7EE6", "\u7EE6"], /* CJK Ideograph */
  ["\u7EE7", "\u7EE7"], /* CJK Ideograph */
  ["\u7EE8", "\u7EE8"], /* CJK Ideograph */
  ["\u7EE9", "\u7EE9"], /* CJK Ideograph */
  ["\u7EEA", "\u7EEA"], /* CJK Ideograph */
  ["\u7EEB", "\u7EEB"], /* CJK Ideograph */
  ["\u7EEC", "\u7EEC"], /* CJK Ideograph */
  ["\u7EED", "\u7EED"], /* CJK Ideograph */
  ["\u7EEE", "\u7EEE"], /* CJK Ideograph */
  ["\u7EEF", "\u7EEF"], /* CJK Ideograph */
  ["\u7EF0", "\u7EF0"], /* CJK Ideograph */
  ["\u7EF1", "\u7EF1"], /* CJK Ideograph */
  ["\u7EF2", "\u7EF2"], /* CJK Ideograph */
  ["\u7EF3", "\u7EF3"], /* CJK Ideograph */
  ["\u7EF4", "\u7EF4"], /* CJK Ideograph */
  ["\u7EF5", "\u7EF5"], /* CJK Ideograph */
  ["\u7EF6", "\u7EF6"], /* CJK Ideograph */
  ["\u7EF7", "\u7EF7"], /* CJK Ideograph */
  ["\u7EF8", "\u7EF8"], /* CJK Ideograph */
  ["\u7EF9", "\u7EF9"], /* CJK Ideograph */
  ["\u7EFA", "\u7EFA"], /* CJK Ideograph */
  ["\u7EFB", "\u7EFB"], /* CJK Ideograph */
  ["\u7EFC", "\u7EFC"], /* CJK Ideograph */
  ["\u7EFD", "\u7EFD"], /* CJK Ideograph */
  ["\u7EFE", "\u7EFE"], /* CJK Ideograph */
  ["\u7EFF", "\u7EFF"], /* CJK Ideograph */
  ["\u7F00", "\u7F00"], /* CJK Ideograph */
  ["\u7F01", "\u7F01"], /* CJK Ideograph */
  ["\u7F02", "\u7F02"], /* CJK Ideograph */
  ["\u7F03", "\u7F03"], /* CJK Ideograph */
  ["\u7F04", "\u7F04"], /* CJK Ideograph */
  ["\u7F05", "\u7F05"], /* CJK Ideograph */
  ["\u7F06", "\u7F06"], /* CJK Ideograph */
  ["\u7F07", "\u7F07"], /* CJK Ideograph */
  ["\u7F08", "\u7F08"], /* CJK Ideograph */
  ["\u7F09", "\u7F09"], /* CJK Ideograph */
  ["\u7F0A", "\u7F0A"], /* CJK Ideograph */
  ["\u7F0B", "\u7F0B"], /* CJK Ideograph */
  ["\u7F0C", "\u7F0C"], /* CJK Ideograph */
  ["\u7F0D", "\u7F0D"], /* CJK Ideograph */
  ["\u7F0E", "\u7F0E"], /* CJK Ideograph */
  ["\u7F0F", "\u7F0F"], /* CJK Ideograph */
  ["\u7F10", "\u7F10"], /* CJK Ideograph */
  ["\u7F11", "\u7F11"], /* CJK Ideograph */
  ["\u7F12", "\u7F12"], /* CJK Ideograph */
  ["\u7F13", "\u7F13"], /* CJK Ideograph */
  ["\u7F14", "\u7F14"], /* CJK Ideograph */
  ["\u7F15", "\u7F15"], /* CJK Ideograph */
  ["\u7F16", "\u7F16"], /* CJK Ideograph */
  ["\u7F17", "\u7F17"], /* CJK Ideograph */
  ["\u7F18", "\u7F18"], /* CJK Ideograph */
  ["\u7F19", "\u7F19"], /* CJK Ideograph */
  ["\u7F1A", "\u7F1A"], /* CJK Ideograph */
  ["\u7F1B", "\u7F1B"], /* CJK Ideograph */
  ["\u7F1C", "\u7F1C"], /* CJK Ideograph */
  ["\u7F1D", "\u7F1D"], /* CJK Ideograph */
  ["\u7F1E", "\u7F1E"], /* CJK Ideograph */
  ["\u7F1F", "\u7F1F"], /* CJK Ideograph */
  ["\u7F20", "\u7F20"], /* CJK Ideograph */
  ["\u7F21", "\u7F21"], /* CJK Ideograph */
  ["\u7F22", "\u7F22"], /* CJK Ideograph */
  ["\u7F23", "\u7F23"], /* CJK Ideograph */
  ["\u7F24", "\u7F24"], /* CJK Ideograph */
  ["\u7F25", "\u7F25"], /* CJK Ideograph */
  ["\u7F26", "\u7F26"], /* CJK Ideograph */
  ["\u7F27", "\u7F27"], /* CJK Ideograph */
  ["\u7F28", "\u7F28"], /* CJK Ideograph */
  ["\u7F29", "\u7F29"], /* CJK Ideograph */
  ["\u7F2A", "\u7F2A"], /* CJK Ideograph */
  ["\u7F2B", "\u7F2B"], /* CJK Ideograph */
  ["\u7F2C", "\u7F2C"], /* CJK Ideograph */
  ["\u7F2D", "\u7F2D"], /* CJK Ideograph */
  ["\u7F2E", "\u7F2E"], /* CJK Ideograph */
  ["\u7F2F", "\u7F2F"], /* CJK Ideograph */
  ["\u7F30", "\u7F30"], /* CJK Ideograph */
  ["\u7F31", "\u7F31"], /* CJK Ideograph */
  ["\u7F32", "\u7F32"], /* CJK Ideograph */
  ["\u7F33", "\u7F33"], /* CJK Ideograph */
  ["\u7F34", "\u7F34"], /* CJK Ideograph */
  ["\u7F35", "\u7F35"], /* CJK Ideograph */
  ["\u7F36", "\u7F36"], /* CJK Ideograph */
  ["\u7F37", "\u7F37"], /* CJK Ideograph */
  ["\u7F38", "\u7F38"], /* CJK Ideograph */
  ["\u7F39", "\u7F39"], /* CJK Ideograph */
  ["\u7F3A", "\u7F3A"], /* CJK Ideograph */
  ["\u7F3B", "\u7F3B"], /* CJK Ideograph */
  ["\u7F3C", "\u7F3C"], /* CJK Ideograph */
  ["\u7F3D", "\u7F3D"], /* CJK Ideograph */
  ["\u7F3E", "\u7F3E"], /* CJK Ideograph */
  ["\u7F3F", "\u7F3F"], /* CJK Ideograph */
  ["\u7F40", "\u7F40"], /* CJK Ideograph */
  ["\u7F41", "\u7F41"], /* CJK Ideograph */
  ["\u7F42", "\u7F42"], /* CJK Ideograph */
  ["\u7F43", "\u7F43"], /* CJK Ideograph */
  ["\u7F44", "\u7F44"], /* CJK Ideograph */
  ["\u7F45", "\u7F45"], /* CJK Ideograph */
  ["\u7F46", "\u7F46"], /* CJK Ideograph */
  ["\u7F47", "\u7F47"], /* CJK Ideograph */
  ["\u7F48", "\u7F48"], /* CJK Ideograph */
  ["\u7F49", "\u7F49"], /* CJK Ideograph */
  ["\u7F4A", "\u7F4A"], /* CJK Ideograph */
  ["\u7F4B", "\u7F4B"], /* CJK Ideograph */
  ["\u7F4C", "\u7F4C"], /* CJK Ideograph */
  ["\u7F4D", "\u7F4D"], /* CJK Ideograph */
  ["\u7F4E", "\u7F4E"], /* CJK Ideograph */
  ["\u7F4F", "\u7F4F"], /* CJK Ideograph */
  ["\u7F50", "\u7F50"], /* CJK Ideograph */
  ["\u7F51", "\u7F51"], /* CJK Ideograph */
  ["\u7F52", "\u7F52"], /* CJK Ideograph */
  ["\u7F53", "\u7F53"], /* CJK Ideograph */
  ["\u7F54", "\u7F54"], /* CJK Ideograph */
  ["\u7F55", "\u7F55"], /* CJK Ideograph */
  ["\u7F56", "\u7F56"], /* CJK Ideograph */
  ["\u7F57", "\u7F57"], /* CJK Ideograph */
  ["\u7F58", "\u7F58"], /* CJK Ideograph */
  ["\u7F59", "\u7F59"], /* CJK Ideograph */
  ["\u7F5A", "\u7F5A"], /* CJK Ideograph */
  ["\u7F5B", "\u7F5B"], /* CJK Ideograph */
  ["\u7F5C", "\u7F5C"], /* CJK Ideograph */
  ["\u7F5D", "\u7F5D"], /* CJK Ideograph */
  ["\u7F5E", "\u7F5E"], /* CJK Ideograph */
  ["\u7F5F", "\u7F5F"], /* CJK Ideograph */
  ["\u7F60", "\u7F60"], /* CJK Ideograph */
  ["\u7F61", "\u7F61"], /* CJK Ideograph */
  ["\u7F62", "\u7F62"], /* CJK Ideograph */
  ["\u7F63", "\u7F63"], /* CJK Ideograph */
  ["\u7F64", "\u7F64"], /* CJK Ideograph */
  ["\u7F65", "\u7F65"], /* CJK Ideograph */
  ["\u7F66", "\u7F66"], /* CJK Ideograph */
  ["\u7F67", "\u7F67"], /* CJK Ideograph */
  ["\u7F68", "\u7F68"], /* CJK Ideograph */
  ["\u7F69", "\u7F69"], /* CJK Ideograph */
  ["\u7F6A", "\u7F6A"], /* CJK Ideograph */
  ["\u7F6B", "\u7F6B"], /* CJK Ideograph */
  ["\u7F6C", "\u7F6C"], /* CJK Ideograph */
  ["\u7F6D", "\u7F6D"], /* CJK Ideograph */
  ["\u7F6E", "\u7F6E"], /* CJK Ideograph */
  ["\u7F6F", "\u7F6F"], /* CJK Ideograph */
  ["\u7F70", "\u7F70"], /* CJK Ideograph */
  ["\u7F71", "\u7F71"], /* CJK Ideograph */
  ["\u7F72", "\u7F72"], /* CJK Ideograph */
  ["\u7F73", "\u7F73"], /* CJK Ideograph */
  ["\u7F74", "\u7F74"], /* CJK Ideograph */
  ["\u7F75", "\u7F75"], /* CJK Ideograph */
  ["\u7F76", "\u7F76"], /* CJK Ideograph */
  ["\u7F77", "\u7F77"], /* CJK Ideograph */
  ["\u7F78", "\u7F78"], /* CJK Ideograph */
  ["\u7F79", "\u7F79"], /* CJK Ideograph */
  ["\u7F7A", "\u7F7A"], /* CJK Ideograph */
  ["\u7F7B", "\u7F7B"], /* CJK Ideograph */
  ["\u7F7C", "\u7F7C"], /* CJK Ideograph */
  ["\u7F7D", "\u7F7D"], /* CJK Ideograph */
  ["\u7F7E", "\u7F7E"], /* CJK Ideograph */
  ["\u7F7F", "\u7F7F"], /* CJK Ideograph */
  ["\u7F80", "\u7F80"], /* CJK Ideograph */
  ["\u7F81", "\u7F81"], /* CJK Ideograph */
  ["\u7F82", "\u7F82"], /* CJK Ideograph */
  ["\u7F83", "\u7F83"], /* CJK Ideograph */
  ["\u7F84", "\u7F84"], /* CJK Ideograph */
  ["\u7F85", "\u7F85"], /* CJK Ideograph */
  ["\u7F86", "\u7F86"], /* CJK Ideograph */
  ["\u7F87", "\u7F87"], /* CJK Ideograph */
  ["\u7F88", "\u7F88"], /* CJK Ideograph */
  ["\u7F89", "\u7F89"], /* CJK Ideograph */
  ["\u7F8A", "\u7F8A"], /* CJK Ideograph */
  ["\u7F8B", "\u7F8B"], /* CJK Ideograph */
  ["\u7F8C", "\u7F8C"], /* CJK Ideograph */
  ["\u7F8D", "\u7F8D"], /* CJK Ideograph */
  ["\u7F8E", "\u7F8E"], /* CJK Ideograph */
  ["\u7F8F", "\u7F8F"], /* CJK Ideograph */
  ["\u7F90", "\u7F90"], /* CJK Ideograph */
  ["\u7F91", "\u7F91"], /* CJK Ideograph */
  ["\u7F92", "\u7F92"], /* CJK Ideograph */
  ["\u7F93", "\u7F93"], /* CJK Ideograph */
  ["\u7F94", "\u7F94"], /* CJK Ideograph */
  ["\u7F95", "\u7F95"], /* CJK Ideograph */
  ["\u7F96", "\u7F96"], /* CJK Ideograph */
  ["\u7F97", "\u7F97"], /* CJK Ideograph */
  ["\u7F98", "\u7F98"], /* CJK Ideograph */
  ["\u7F99", "\u7F99"], /* CJK Ideograph */
  ["\u7F9A", "\u7F9A"], /* CJK Ideograph */
  ["\u7F9B", "\u7F9B"], /* CJK Ideograph */
  ["\u7F9C", "\u7F9C"], /* CJK Ideograph */
  ["\u7F9D", "\u7F9D"], /* CJK Ideograph */
  ["\u7F9E", "\u7F9E"], /* CJK Ideograph */
  ["\u7F9F", "\u7F9F"], /* CJK Ideograph */
  ["\u7FA0", "\u7FA0"], /* CJK Ideograph */
  ["\u7FA1", "\u7FA1"], /* CJK Ideograph */
  ["\u7FA2", "\u7FA2"], /* CJK Ideograph */
  ["\u7FA3", "\u7FA3"], /* CJK Ideograph */
  ["\u7FA4", "\u7FA4"], /* CJK Ideograph */
  ["\u7FA5", "\u7FA5"], /* CJK Ideograph */
  ["\u7FA6", "\u7FA6"], /* CJK Ideograph */
  ["\u7FA7", "\u7FA7"], /* CJK Ideograph */
  ["\u7FA8", "\u7FA8"], /* CJK Ideograph */
  ["\u7FA9", "\u7FA9"], /* CJK Ideograph */
  ["\u7FAA", "\u7FAA"], /* CJK Ideograph */
  ["\u7FAB", "\u7FAB"], /* CJK Ideograph */
  ["\u7FAC", "\u7FAC"], /* CJK Ideograph */
  ["\u7FAD", "\u7FAD"], /* CJK Ideograph */
  ["\u7FAE", "\u7FAE"], /* CJK Ideograph */
  ["\u7FAF", "\u7FAF"], /* CJK Ideograph */
  ["\u7FB0", "\u7FB0"], /* CJK Ideograph */
  ["\u7FB1", "\u7FB1"], /* CJK Ideograph */
  ["\u7FB2", "\u7FB2"], /* CJK Ideograph */
  ["\u7FB3", "\u7FB3"], /* CJK Ideograph */
  ["\u7FB4", "\u7FB4"], /* CJK Ideograph */
  ["\u7FB5", "\u7FB5"], /* CJK Ideograph */
  ["\u7FB6", "\u7FB6"], /* CJK Ideograph */
  ["\u7FB7", "\u7FB7"], /* CJK Ideograph */
  ["\u7FB8", "\u7FB8"], /* CJK Ideograph */
  ["\u7FB9", "\u7FB9"], /* CJK Ideograph */
  ["\u7FBA", "\u7FBA"], /* CJK Ideograph */
  ["\u7FBB", "\u7FBB"], /* CJK Ideograph */
  ["\u7FBC", "\u7FBC"], /* CJK Ideograph */
  ["\u7FBD", "\u7FBD"], /* CJK Ideograph */
  ["\u7FBE", "\u7FBE"], /* CJK Ideograph */
  ["\u7FBF", "\u7FBF"], /* CJK Ideograph */
  ["\u7FC0", "\u7FC0"], /* CJK Ideograph */
  ["\u7FC1", "\u7FC1"], /* CJK Ideograph */
  ["\u7FC2", "\u7FC2"], /* CJK Ideograph */
  ["\u7FC3", "\u7FC3"], /* CJK Ideograph */
  ["\u7FC4", "\u7FC4"], /* CJK Ideograph */
  ["\u7FC5", "\u7FC5"], /* CJK Ideograph */
  ["\u7FC6", "\u7FC6"], /* CJK Ideograph */
  ["\u7FC7", "\u7FC7"], /* CJK Ideograph */
  ["\u7FC8", "\u7FC8"], /* CJK Ideograph */
  ["\u7FC9", "\u7FC9"], /* CJK Ideograph */
  ["\u7FCA", "\u7FCA"], /* CJK Ideograph */
  ["\u7FCB", "\u7FCB"], /* CJK Ideograph */
  ["\u7FCC", "\u7FCC"], /* CJK Ideograph */
  ["\u7FCD", "\u7FCD"], /* CJK Ideograph */
  ["\u7FCE", "\u7FCE"], /* CJK Ideograph */
  ["\u7FCF", "\u7FCF"], /* CJK Ideograph */
  ["\u7FD0", "\u7FD0"], /* CJK Ideograph */
  ["\u7FD1", "\u7FD1"], /* CJK Ideograph */
  ["\u7FD2", "\u7FD2"], /* CJK Ideograph */
  ["\u7FD3", "\u7FD3"], /* CJK Ideograph */
  ["\u7FD4", "\u7FD4"], /* CJK Ideograph */
  ["\u7FD5", "\u7FD5"], /* CJK Ideograph */
  ["\u7FD6", "\u7FD6"], /* CJK Ideograph */
  ["\u7FD7", "\u7FD7"], /* CJK Ideograph */
  ["\u7FD8", "\u7FD8"], /* CJK Ideograph */
  ["\u7FD9", "\u7FD9"], /* CJK Ideograph */
  ["\u7FDA", "\u7FDA"], /* CJK Ideograph */
  ["\u7FDB", "\u7FDB"], /* CJK Ideograph */
  ["\u7FDC", "\u7FDC"], /* CJK Ideograph */
  ["\u7FDD", "\u7FDD"], /* CJK Ideograph */
  ["\u7FDE", "\u7FDE"], /* CJK Ideograph */
  ["\u7FDF", "\u7FDF"], /* CJK Ideograph */
  ["\u7FE0", "\u7FE0"], /* CJK Ideograph */
  ["\u7FE1", "\u7FE1"], /* CJK Ideograph */
  ["\u7FE2", "\u7FE2"], /* CJK Ideograph */
  ["\u7FE3", "\u7FE3"], /* CJK Ideograph */
  ["\u7FE4", "\u7FE4"], /* CJK Ideograph */
  ["\u7FE5", "\u7FE5"], /* CJK Ideograph */
  ["\u7FE6", "\u7FE6"], /* CJK Ideograph */
  ["\u7FE7", "\u7FE7"], /* CJK Ideograph */
  ["\u7FE8", "\u7FE8"], /* CJK Ideograph */
  ["\u7FE9", "\u7FE9"], /* CJK Ideograph */
  ["\u7FEA", "\u7FEA"], /* CJK Ideograph */
  ["\u7FEB", "\u7FEB"], /* CJK Ideograph */
  ["\u7FEC", "\u7FEC"], /* CJK Ideograph */
  ["\u7FED", "\u7FED"], /* CJK Ideograph */
  ["\u7FEE", "\u7FEE"], /* CJK Ideograph */
  ["\u7FEF", "\u7FEF"], /* CJK Ideograph */
  ["\u7FF0", "\u7FF0"], /* CJK Ideograph */
  ["\u7FF1", "\u7FF1"], /* CJK Ideograph */
  ["\u7FF2", "\u7FF2"], /* CJK Ideograph */
  ["\u7FF3", "\u7FF3"], /* CJK Ideograph */
  ["\u7FF4", "\u7FF4"], /* CJK Ideograph */
  ["\u7FF5", "\u7FF5"], /* CJK Ideograph */
  ["\u7FF6", "\u7FF6"], /* CJK Ideograph */
  ["\u7FF7", "\u7FF7"], /* CJK Ideograph */
  ["\u7FF8", "\u7FF8"], /* CJK Ideograph */
  ["\u7FF9", "\u7FF9"], /* CJK Ideograph */
  ["\u7FFA", "\u7FFA"], /* CJK Ideograph */
  ["\u7FFB", "\u7FFB"], /* CJK Ideograph */
  ["\u7FFC", "\u7FFC"], /* CJK Ideograph */
  ["\u7FFD", "\u7FFD"], /* CJK Ideograph */
  ["\u7FFE", "\u7FFE"], /* CJK Ideograph */
  ["\u7FFF", "\u7FFF"], /* CJK Ideograph */
  ["\u8000", "\u8000"], /* CJK Ideograph */
  ["\u8001", "\u8001"], /* CJK Ideograph */
  ["\u8002", "\u8002"], /* CJK Ideograph */
  ["\u8003", "\u8003"], /* CJK Ideograph */
  ["\u8004", "\u8004"], /* CJK Ideograph */
  ["\u8005", "\u8005"], /* CJK Ideograph */
  ["\u8006", "\u8006"], /* CJK Ideograph */
  ["\u8007", "\u8007"], /* CJK Ideograph */
  ["\u8008", "\u8008"], /* CJK Ideograph */
  ["\u8009", "\u8009"], /* CJK Ideograph */
  ["\u800A", "\u800A"], /* CJK Ideograph */
  ["\u800B", "\u800B"], /* CJK Ideograph */
  ["\u800C", "\u800C"], /* CJK Ideograph */
  ["\u800D", "\u800D"], /* CJK Ideograph */
  ["\u800E", "\u800E"], /* CJK Ideograph */
  ["\u800F", "\u800F"], /* CJK Ideograph */
  ["\u8010", "\u8010"], /* CJK Ideograph */
  ["\u8011", "\u8011"], /* CJK Ideograph */
  ["\u8012", "\u8012"], /* CJK Ideograph */
  ["\u8013", "\u8013"], /* CJK Ideograph */
  ["\u8014", "\u8014"], /* CJK Ideograph */
  ["\u8015", "\u8015"], /* CJK Ideograph */
  ["\u8016", "\u8016"], /* CJK Ideograph */
  ["\u8017", "\u8017"], /* CJK Ideograph */
  ["\u8018", "\u8018"], /* CJK Ideograph */
  ["\u8019", "\u8019"], /* CJK Ideograph */
  ["\u801A", "\u801A"], /* CJK Ideograph */
  ["\u801B", "\u801B"], /* CJK Ideograph */
  ["\u801C", "\u801C"], /* CJK Ideograph */
  ["\u801D", "\u801D"], /* CJK Ideograph */
  ["\u801E", "\u801E"], /* CJK Ideograph */
  ["\u801F", "\u801F"], /* CJK Ideograph */
  ["\u8020", "\u8020"], /* CJK Ideograph */
  ["\u8021", "\u8021"], /* CJK Ideograph */
  ["\u8022", "\u8022"], /* CJK Ideograph */
  ["\u8023", "\u8023"], /* CJK Ideograph */
  ["\u8024", "\u8024"], /* CJK Ideograph */
  ["\u8025", "\u8025"], /* CJK Ideograph */
  ["\u8026", "\u8026"], /* CJK Ideograph */
  ["\u8027", "\u8027"], /* CJK Ideograph */
  ["\u8028", "\u8028"], /* CJK Ideograph */
  ["\u8029", "\u8029"], /* CJK Ideograph */
  ["\u802A", "\u802A"], /* CJK Ideograph */
  ["\u802B", "\u802B"], /* CJK Ideograph */
  ["\u802C", "\u802C"], /* CJK Ideograph */
  ["\u802D", "\u802D"], /* CJK Ideograph */
  ["\u802E", "\u802E"], /* CJK Ideograph */
  ["\u802F", "\u802F"], /* CJK Ideograph */
  ["\u8030", "\u8030"], /* CJK Ideograph */
  ["\u8031", "\u8031"], /* CJK Ideograph */
  ["\u8032", "\u8032"], /* CJK Ideograph */
  ["\u8033", "\u8033"], /* CJK Ideograph */
  ["\u8034", "\u8034"], /* CJK Ideograph */
  ["\u8035", "\u8035"], /* CJK Ideograph */
  ["\u8036", "\u8036"], /* CJK Ideograph */
  ["\u8037", "\u8037"], /* CJK Ideograph */
  ["\u8038", "\u8038"], /* CJK Ideograph */
  ["\u8039", "\u8039"], /* CJK Ideograph */
  ["\u803A", "\u803A"], /* CJK Ideograph */
  ["\u803B", "\u803B"], /* CJK Ideograph */
  ["\u803C", "\u803C"], /* CJK Ideograph */
  ["\u803D", "\u803D"], /* CJK Ideograph */
  ["\u803E", "\u803E"], /* CJK Ideograph */
  ["\u803F", "\u803F"], /* CJK Ideograph */
  ["\u8040", "\u8040"], /* CJK Ideograph */
  ["\u8041", "\u8041"], /* CJK Ideograph */
  ["\u8042", "\u8042"], /* CJK Ideograph */
  ["\u8043", "\u8043"], /* CJK Ideograph */
  ["\u8044", "\u8044"], /* CJK Ideograph */
  ["\u8045", "\u8045"], /* CJK Ideograph */
  ["\u8046", "\u8046"], /* CJK Ideograph */
  ["\u8047", "\u8047"], /* CJK Ideograph */
  ["\u8048", "\u8048"], /* CJK Ideograph */
  ["\u8049", "\u8049"], /* CJK Ideograph */
  ["\u804A", "\u804A"], /* CJK Ideograph */
  ["\u804B", "\u804B"], /* CJK Ideograph */
  ["\u804C", "\u804C"], /* CJK Ideograph */
  ["\u804D", "\u804D"], /* CJK Ideograph */
  ["\u804E", "\u804E"], /* CJK Ideograph */
  ["\u804F", "\u804F"], /* CJK Ideograph */
  ["\u8050", "\u8050"], /* CJK Ideograph */
  ["\u8051", "\u8051"], /* CJK Ideograph */
  ["\u8052", "\u8052"], /* CJK Ideograph */
  ["\u8053", "\u8053"], /* CJK Ideograph */
  ["\u8054", "\u8054"], /* CJK Ideograph */
  ["\u8055", "\u8055"], /* CJK Ideograph */
  ["\u8056", "\u8056"], /* CJK Ideograph */
  ["\u8057", "\u8057"], /* CJK Ideograph */
  ["\u8058", "\u8058"], /* CJK Ideograph */
  ["\u8059", "\u8059"], /* CJK Ideograph */
  ["\u805A", "\u805A"], /* CJK Ideograph */
  ["\u805B", "\u805B"], /* CJK Ideograph */
  ["\u805C", "\u805C"], /* CJK Ideograph */
  ["\u805D", "\u805D"], /* CJK Ideograph */
  ["\u805E", "\u805E"], /* CJK Ideograph */
  ["\u805F", "\u805F"], /* CJK Ideograph */
  ["\u8060", "\u8060"], /* CJK Ideograph */
  ["\u8061", "\u8061"], /* CJK Ideograph */
  ["\u8062", "\u8062"], /* CJK Ideograph */
  ["\u8063", "\u8063"], /* CJK Ideograph */
  ["\u8064", "\u8064"], /* CJK Ideograph */
  ["\u8065", "\u8065"], /* CJK Ideograph */
  ["\u8066", "\u8066"], /* CJK Ideograph */
  ["\u8067", "\u8067"], /* CJK Ideograph */
  ["\u8068", "\u8068"], /* CJK Ideograph */
  ["\u8069", "\u8069"], /* CJK Ideograph */
  ["\u806A", "\u806A"], /* CJK Ideograph */
  ["\u806B", "\u806B"], /* CJK Ideograph */
  ["\u806C", "\u806C"], /* CJK Ideograph */
  ["\u806D", "\u806D"], /* CJK Ideograph */
  ["\u806E", "\u806E"], /* CJK Ideograph */
  ["\u806F", "\u806F"], /* CJK Ideograph */
  ["\u8070", "\u8070"], /* CJK Ideograph */
  ["\u8071", "\u8071"], /* CJK Ideograph */
  ["\u8072", "\u8072"], /* CJK Ideograph */
  ["\u8073", "\u8073"], /* CJK Ideograph */
  ["\u8074", "\u8074"], /* CJK Ideograph */
  ["\u8075", "\u8075"], /* CJK Ideograph */
  ["\u8076", "\u8076"], /* CJK Ideograph */
  ["\u8077", "\u8077"], /* CJK Ideograph */
  ["\u8078", "\u8078"], /* CJK Ideograph */
  ["\u8079", "\u8079"], /* CJK Ideograph */
  ["\u807A", "\u807A"], /* CJK Ideograph */
  ["\u807B", "\u807B"], /* CJK Ideograph */
  ["\u807C", "\u807C"], /* CJK Ideograph */
  ["\u807D", "\u807D"], /* CJK Ideograph */
  ["\u807E", "\u807E"], /* CJK Ideograph */
  ["\u807F", "\u807F"], /* CJK Ideograph */
  ["\u8080", "\u8080"], /* CJK Ideograph */
  ["\u8081", "\u8081"], /* CJK Ideograph */
  ["\u8082", "\u8082"], /* CJK Ideograph */
  ["\u8083", "\u8083"], /* CJK Ideograph */
  ["\u8084", "\u8084"], /* CJK Ideograph */
  ["\u8085", "\u8085"], /* CJK Ideograph */
  ["\u8086", "\u8086"], /* CJK Ideograph */
  ["\u8087", "\u8087"], /* CJK Ideograph */
  ["\u8088", "\u8088"], /* CJK Ideograph */
  ["\u8089", "\u8089"], /* CJK Ideograph */
  ["\u808A", "\u808A"], /* CJK Ideograph */
  ["\u808B", "\u808B"], /* CJK Ideograph */
  ["\u808C", "\u808C"], /* CJK Ideograph */
  ["\u808D", "\u808D"], /* CJK Ideograph */
  ["\u808E", "\u808E"], /* CJK Ideograph */
  ["\u808F", "\u808F"], /* CJK Ideograph */
  ["\u8090", "\u8090"], /* CJK Ideograph */
  ["\u8091", "\u8091"], /* CJK Ideograph */
  ["\u8092", "\u8092"], /* CJK Ideograph */
  ["\u8093", "\u8093"], /* CJK Ideograph */
  ["\u8094", "\u8094"], /* CJK Ideograph */
  ["\u8095", "\u8095"], /* CJK Ideograph */
  ["\u8096", "\u8096"], /* CJK Ideograph */
  ["\u8097", "\u8097"], /* CJK Ideograph */
  ["\u8098", "\u8098"], /* CJK Ideograph */
  ["\u8099", "\u8099"], /* CJK Ideograph */
  ["\u809A", "\u809A"], /* CJK Ideograph */
  ["\u809B", "\u809B"], /* CJK Ideograph */
  ["\u809C", "\u809C"], /* CJK Ideograph */
  ["\u809D", "\u809D"], /* CJK Ideograph */
  ["\u809E", "\u809E"], /* CJK Ideograph */
  ["\u809F", "\u809F"], /* CJK Ideograph */
  ["\u80A0", "\u80A0"], /* CJK Ideograph */
  ["\u80A1", "\u80A1"], /* CJK Ideograph */
  ["\u80A2", "\u80A2"], /* CJK Ideograph */
  ["\u80A3", "\u80A3"], /* CJK Ideograph */
  ["\u80A4", "\u80A4"], /* CJK Ideograph */
  ["\u80A5", "\u80A5"], /* CJK Ideograph */
  ["\u80A6", "\u80A6"], /* CJK Ideograph */
  ["\u80A7", "\u80A7"], /* CJK Ideograph */
  ["\u80A8", "\u80A8"], /* CJK Ideograph */
  ["\u80A9", "\u80A9"], /* CJK Ideograph */
  ["\u80AA", "\u80AA"], /* CJK Ideograph */
  ["\u80AB", "\u80AB"], /* CJK Ideograph */
  ["\u80AC", "\u80AC"], /* CJK Ideograph */
  ["\u80AD", "\u80AD"], /* CJK Ideograph */
  ["\u80AE", "\u80AE"], /* CJK Ideograph */
  ["\u80AF", "\u80AF"], /* CJK Ideograph */
  ["\u80B0", "\u80B0"], /* CJK Ideograph */
  ["\u80B1", "\u80B1"], /* CJK Ideograph */
  ["\u80B2", "\u80B2"], /* CJK Ideograph */
  ["\u80B3", "\u80B3"], /* CJK Ideograph */
  ["\u80B4", "\u80B4"], /* CJK Ideograph */
  ["\u80B5", "\u80B5"], /* CJK Ideograph */
  ["\u80B6", "\u80B6"], /* CJK Ideograph */
  ["\u80B7", "\u80B7"], /* CJK Ideograph */
  ["\u80B8", "\u80B8"], /* CJK Ideograph */
  ["\u80B9", "\u80B9"], /* CJK Ideograph */
  ["\u80BA", "\u80BA"], /* CJK Ideograph */
  ["\u80BB", "\u80BB"], /* CJK Ideograph */
  ["\u80BC", "\u80BC"], /* CJK Ideograph */
  ["\u80BD", "\u80BD"], /* CJK Ideograph */
  ["\u80BE", "\u80BE"], /* CJK Ideograph */
  ["\u80BF", "\u80BF"], /* CJK Ideograph */
  ["\u80C0", "\u80C0"], /* CJK Ideograph */
  ["\u80C1", "\u80C1"], /* CJK Ideograph */
  ["\u80C2", "\u80C2"], /* CJK Ideograph */
  ["\u80C3", "\u80C3"], /* CJK Ideograph */
  ["\u80C4", "\u80C4"], /* CJK Ideograph */
  ["\u80C5", "\u80C5"], /* CJK Ideograph */
  ["\u80C6", "\u80C6"], /* CJK Ideograph */
  ["\u80C7", "\u80C7"], /* CJK Ideograph */
  ["\u80C8", "\u80C8"], /* CJK Ideograph */
  ["\u80C9", "\u80C9"], /* CJK Ideograph */
  ["\u80CA", "\u80CA"], /* CJK Ideograph */
  ["\u80CB", "\u80CB"], /* CJK Ideograph */
  ["\u80CC", "\u80CC"], /* CJK Ideograph */
  ["\u80CD", "\u80CD"], /* CJK Ideograph */
  ["\u80CE", "\u80CE"], /* CJK Ideograph */
  ["\u80CF", "\u80CF"], /* CJK Ideograph */
  ["\u80D0", "\u80D0"], /* CJK Ideograph */
  ["\u80D1", "\u80D1"], /* CJK Ideograph */
  ["\u80D2", "\u80D2"], /* CJK Ideograph */
  ["\u80D3", "\u80D3"], /* CJK Ideograph */
  ["\u80D4", "\u80D4"], /* CJK Ideograph */
  ["\u80D5", "\u80D5"], /* CJK Ideograph */
  ["\u80D6", "\u80D6"], /* CJK Ideograph */
  ["\u80D7", "\u80D7"], /* CJK Ideograph */
  ["\u80D8", "\u80D8"], /* CJK Ideograph */
  ["\u80D9", "\u80D9"], /* CJK Ideograph */
  ["\u80DA", "\u80DA"], /* CJK Ideograph */
  ["\u80DB", "\u80DB"], /* CJK Ideograph */
  ["\u80DC", "\u80DC"], /* CJK Ideograph */
  ["\u80DD", "\u80DD"], /* CJK Ideograph */
  ["\u80DE", "\u80DE"], /* CJK Ideograph */
  ["\u80DF", "\u80DF"], /* CJK Ideograph */
  ["\u80E0", "\u80E0"], /* CJK Ideograph */
  ["\u80E1", "\u80E1"], /* CJK Ideograph */
  ["\u80E2", "\u80E2"], /* CJK Ideograph */
  ["\u80E3", "\u80E3"], /* CJK Ideograph */
  ["\u80E4", "\u80E4"], /* CJK Ideograph */
  ["\u80E5", "\u80E5"], /* CJK Ideograph */
  ["\u80E6", "\u80E6"], /* CJK Ideograph */
  ["\u80E7", "\u80E7"], /* CJK Ideograph */
  ["\u80E8", "\u80E8"], /* CJK Ideograph */
  ["\u80E9", "\u80E9"], /* CJK Ideograph */
  ["\u80EA", "\u80EA"], /* CJK Ideograph */
  ["\u80EB", "\u80EB"], /* CJK Ideograph */
  ["\u80EC", "\u80EC"], /* CJK Ideograph */
  ["\u80ED", "\u80ED"], /* CJK Ideograph */
  ["\u80EE", "\u80EE"], /* CJK Ideograph */
  ["\u80EF", "\u80EF"], /* CJK Ideograph */
  ["\u80F0", "\u80F0"], /* CJK Ideograph */
  ["\u80F1", "\u80F1"], /* CJK Ideograph */
  ["\u80F2", "\u80F2"], /* CJK Ideograph */
  ["\u80F3", "\u80F3"], /* CJK Ideograph */
  ["\u80F4", "\u80F4"], /* CJK Ideograph */
  ["\u80F5", "\u80F5"], /* CJK Ideograph */
  ["\u80F6", "\u80F6"], /* CJK Ideograph */
  ["\u80F7", "\u80F7"], /* CJK Ideograph */
  ["\u80F8", "\u80F8"], /* CJK Ideograph */
  ["\u80F9", "\u80F9"], /* CJK Ideograph */
  ["\u80FA", "\u80FA"], /* CJK Ideograph */
  ["\u80FB", "\u80FB"], /* CJK Ideograph */
  ["\u80FC", "\u80FC"], /* CJK Ideograph */
  ["\u80FD", "\u80FD"], /* CJK Ideograph */
  ["\u80FE", "\u80FE"], /* CJK Ideograph */
  ["\u80FF", "\u80FF"], /* CJK Ideograph */
  ["\u8100", "\u8100"], /* CJK Ideograph */
  ["\u8101", "\u8101"], /* CJK Ideograph */
  ["\u8102", "\u8102"], /* CJK Ideograph */
  ["\u8103", "\u8103"], /* CJK Ideograph */
  ["\u8104", "\u8104"], /* CJK Ideograph */
  ["\u8105", "\u8105"], /* CJK Ideograph */
  ["\u8106", "\u8106"], /* CJK Ideograph */
  ["\u8107", "\u8107"], /* CJK Ideograph */
  ["\u8108", "\u8108"], /* CJK Ideograph */
  ["\u8109", "\u8109"], /* CJK Ideograph */
  ["\u810A", "\u810A"], /* CJK Ideograph */
  ["\u810B", "\u810B"], /* CJK Ideograph */
  ["\u810C", "\u810C"], /* CJK Ideograph */
  ["\u810D", "\u810D"], /* CJK Ideograph */
  ["\u810E", "\u810E"], /* CJK Ideograph */
  ["\u810F", "\u810F"], /* CJK Ideograph */
  ["\u8110", "\u8110"], /* CJK Ideograph */
  ["\u8111", "\u8111"], /* CJK Ideograph */
  ["\u8112", "\u8112"], /* CJK Ideograph */
  ["\u8113", "\u8113"], /* CJK Ideograph */
  ["\u8114", "\u8114"], /* CJK Ideograph */
  ["\u8115", "\u8115"], /* CJK Ideograph */
  ["\u8116", "\u8116"], /* CJK Ideograph */
  ["\u8117", "\u8117"], /* CJK Ideograph */
  ["\u8118", "\u8118"], /* CJK Ideograph */
  ["\u8119", "\u8119"], /* CJK Ideograph */
  ["\u811A", "\u811A"], /* CJK Ideograph */
  ["\u811B", "\u811B"], /* CJK Ideograph */
  ["\u811C", "\u811C"], /* CJK Ideograph */
  ["\u811D", "\u811D"], /* CJK Ideograph */
  ["\u811E", "\u811E"], /* CJK Ideograph */
  ["\u811F", "\u811F"], /* CJK Ideograph */
  ["\u8120", "\u8120"], /* CJK Ideograph */
  ["\u8121", "\u8121"], /* CJK Ideograph */
  ["\u8122", "\u8122"], /* CJK Ideograph */
  ["\u8123", "\u8123"], /* CJK Ideograph */
  ["\u8124", "\u8124"], /* CJK Ideograph */
  ["\u8125", "\u8125"], /* CJK Ideograph */
  ["\u8126", "\u8126"], /* CJK Ideograph */
  ["\u8127", "\u8127"], /* CJK Ideograph */
  ["\u8128", "\u8128"], /* CJK Ideograph */
  ["\u8129", "\u8129"], /* CJK Ideograph */
  ["\u812A", "\u812A"], /* CJK Ideograph */
  ["\u812B", "\u812B"], /* CJK Ideograph */
  ["\u812C", "\u812C"], /* CJK Ideograph */
  ["\u812D", "\u812D"], /* CJK Ideograph */
  ["\u812E", "\u812E"], /* CJK Ideograph */
  ["\u812F", "\u812F"], /* CJK Ideograph */
  ["\u8130", "\u8130"], /* CJK Ideograph */
  ["\u8131", "\u8131"], /* CJK Ideograph */
  ["\u8132", "\u8132"], /* CJK Ideograph */
  ["\u8133", "\u8133"], /* CJK Ideograph */
  ["\u8134", "\u8134"], /* CJK Ideograph */
  ["\u8135", "\u8135"], /* CJK Ideograph */
  ["\u8136", "\u8136"], /* CJK Ideograph */
  ["\u8137", "\u8137"], /* CJK Ideograph */
  ["\u8138", "\u8138"], /* CJK Ideograph */
  ["\u8139", "\u8139"], /* CJK Ideograph */
  ["\u813A", "\u813A"], /* CJK Ideograph */
  ["\u813B", "\u813B"], /* CJK Ideograph */
  ["\u813C", "\u813C"], /* CJK Ideograph */
  ["\u813D", "\u813D"], /* CJK Ideograph */
  ["\u813E", "\u813E"], /* CJK Ideograph */
  ["\u813F", "\u813F"], /* CJK Ideograph */
  ["\u8140", "\u8140"], /* CJK Ideograph */
  ["\u8141", "\u8141"], /* CJK Ideograph */
  ["\u8142", "\u8142"], /* CJK Ideograph */
  ["\u8143", "\u8143"], /* CJK Ideograph */
  ["\u8144", "\u8144"], /* CJK Ideograph */
  ["\u8145", "\u8145"], /* CJK Ideograph */
  ["\u8146", "\u8146"], /* CJK Ideograph */
  ["\u8147", "\u8147"], /* CJK Ideograph */
  ["\u8148", "\u8148"], /* CJK Ideograph */
  ["\u8149", "\u8149"], /* CJK Ideograph */
  ["\u814A", "\u814A"], /* CJK Ideograph */
  ["\u814B", "\u814B"], /* CJK Ideograph */
  ["\u814C", "\u814C"], /* CJK Ideograph */
  ["\u814D", "\u814D"], /* CJK Ideograph */
  ["\u814E", "\u814E"], /* CJK Ideograph */
  ["\u814F", "\u814F"], /* CJK Ideograph */
  ["\u8150", "\u8150"], /* CJK Ideograph */
  ["\u8151", "\u8151"], /* CJK Ideograph */
  ["\u8152", "\u8152"], /* CJK Ideograph */
  ["\u8153", "\u8153"], /* CJK Ideograph */
  ["\u8154", "\u8154"], /* CJK Ideograph */
  ["\u8155", "\u8155"], /* CJK Ideograph */
  ["\u8156", "\u8156"], /* CJK Ideograph */
  ["\u8157", "\u8157"], /* CJK Ideograph */
  ["\u8158", "\u8158"], /* CJK Ideograph */
  ["\u8159", "\u8159"], /* CJK Ideograph */
  ["\u815A", "\u815A"], /* CJK Ideograph */
  ["\u815B", "\u815B"], /* CJK Ideograph */
  ["\u815C", "\u815C"], /* CJK Ideograph */
  ["\u815D", "\u815D"], /* CJK Ideograph */
  ["\u815E", "\u815E"], /* CJK Ideograph */
  ["\u815F", "\u815F"], /* CJK Ideograph */
  ["\u8160", "\u8160"], /* CJK Ideograph */
  ["\u8161", "\u8161"], /* CJK Ideograph */
  ["\u8162", "\u8162"], /* CJK Ideograph */
  ["\u8163", "\u8163"], /* CJK Ideograph */
  ["\u8164", "\u8164"], /* CJK Ideograph */
  ["\u8165", "\u8165"], /* CJK Ideograph */
  ["\u8166", "\u8166"], /* CJK Ideograph */
  ["\u8167", "\u8167"], /* CJK Ideograph */
  ["\u8168", "\u8168"], /* CJK Ideograph */
  ["\u8169", "\u8169"], /* CJK Ideograph */
  ["\u816A", "\u816A"], /* CJK Ideograph */
  ["\u816B", "\u816B"], /* CJK Ideograph */
  ["\u816C", "\u816C"], /* CJK Ideograph */
  ["\u816D", "\u816D"], /* CJK Ideograph */
  ["\u816E", "\u816E"], /* CJK Ideograph */
  ["\u816F", "\u816F"], /* CJK Ideograph */
  ["\u8170", "\u8170"], /* CJK Ideograph */
  ["\u8171", "\u8171"], /* CJK Ideograph */
  ["\u8172", "\u8172"], /* CJK Ideograph */
  ["\u8173", "\u8173"], /* CJK Ideograph */
  ["\u8174", "\u8174"], /* CJK Ideograph */
  ["\u8175", "\u8175"], /* CJK Ideograph */
  ["\u8176", "\u8176"], /* CJK Ideograph */
  ["\u8177", "\u8177"], /* CJK Ideograph */
  ["\u8178", "\u8178"], /* CJK Ideograph */
  ["\u8179", "\u8179"], /* CJK Ideograph */
  ["\u817A", "\u817A"], /* CJK Ideograph */
  ["\u817B", "\u817B"], /* CJK Ideograph */
  ["\u817C", "\u817C"], /* CJK Ideograph */
  ["\u817D", "\u817D"], /* CJK Ideograph */
  ["\u817E", "\u817E"], /* CJK Ideograph */
  ["\u817F", "\u817F"], /* CJK Ideograph */
  ["\u8180", "\u8180"], /* CJK Ideograph */
  ["\u8181", "\u8181"], /* CJK Ideograph */
  ["\u8182", "\u8182"], /* CJK Ideograph */
  ["\u8183", "\u8183"], /* CJK Ideograph */
  ["\u8184", "\u8184"], /* CJK Ideograph */
  ["\u8185", "\u8185"], /* CJK Ideograph */
  ["\u8186", "\u8186"], /* CJK Ideograph */
  ["\u8187", "\u8187"], /* CJK Ideograph */
  ["\u8188", "\u8188"], /* CJK Ideograph */
  ["\u8189", "\u8189"], /* CJK Ideograph */
  ["\u818A", "\u818A"], /* CJK Ideograph */
  ["\u818B", "\u818B"], /* CJK Ideograph */
  ["\u818C", "\u818C"], /* CJK Ideograph */
  ["\u818D", "\u818D"], /* CJK Ideograph */
  ["\u818E", "\u818E"], /* CJK Ideograph */
  ["\u818F", "\u818F"], /* CJK Ideograph */
  ["\u8190", "\u8190"], /* CJK Ideograph */
  ["\u8191", "\u8191"], /* CJK Ideograph */
  ["\u8192", "\u8192"], /* CJK Ideograph */
  ["\u8193", "\u8193"], /* CJK Ideograph */
  ["\u8194", "\u8194"], /* CJK Ideograph */
  ["\u8195", "\u8195"], /* CJK Ideograph */
  ["\u8196", "\u8196"], /* CJK Ideograph */
  ["\u8197", "\u8197"], /* CJK Ideograph */
  ["\u8198", "\u8198"], /* CJK Ideograph */
  ["\u8199", "\u8199"], /* CJK Ideograph */
  ["\u819A", "\u819A"], /* CJK Ideograph */
  ["\u819B", "\u819B"], /* CJK Ideograph */
  ["\u819C", "\u819C"], /* CJK Ideograph */
  ["\u819D", "\u819D"], /* CJK Ideograph */
  ["\u819E", "\u819E"], /* CJK Ideograph */
  ["\u819F", "\u819F"], /* CJK Ideograph */
  ["\u81A0", "\u81A0"], /* CJK Ideograph */
  ["\u81A1", "\u81A1"], /* CJK Ideograph */
  ["\u81A2", "\u81A2"], /* CJK Ideograph */
  ["\u81A3", "\u81A3"], /* CJK Ideograph */
  ["\u81A4", "\u81A4"], /* CJK Ideograph */
  ["\u81A5", "\u81A5"], /* CJK Ideograph */
  ["\u81A6", "\u81A6"], /* CJK Ideograph */
  ["\u81A7", "\u81A7"], /* CJK Ideograph */
  ["\u81A8", "\u81A8"], /* CJK Ideograph */
  ["\u81A9", "\u81A9"], /* CJK Ideograph */
  ["\u81AA", "\u81AA"], /* CJK Ideograph */
  ["\u81AB", "\u81AB"], /* CJK Ideograph */
  ["\u81AC", "\u81AC"], /* CJK Ideograph */
  ["\u81AD", "\u81AD"], /* CJK Ideograph */
  ["\u81AE", "\u81AE"], /* CJK Ideograph */
  ["\u81AF", "\u81AF"], /* CJK Ideograph */
  ["\u81B0", "\u81B0"], /* CJK Ideograph */
  ["\u81B1", "\u81B1"], /* CJK Ideograph */
  ["\u81B2", "\u81B2"], /* CJK Ideograph */
  ["\u81B3", "\u81B3"], /* CJK Ideograph */
  ["\u81B4", "\u81B4"], /* CJK Ideograph */
  ["\u81B5", "\u81B5"], /* CJK Ideograph */
  ["\u81B6", "\u81B6"], /* CJK Ideograph */
  ["\u81B7", "\u81B7"], /* CJK Ideograph */
  ["\u81B8", "\u81B8"], /* CJK Ideograph */
  ["\u81B9", "\u81B9"], /* CJK Ideograph */
  ["\u81BA", "\u81BA"], /* CJK Ideograph */
  ["\u81BB", "\u81BB"], /* CJK Ideograph */
  ["\u81BC", "\u81BC"], /* CJK Ideograph */
  ["\u81BD", "\u81BD"], /* CJK Ideograph */
  ["\u81BE", "\u81BE"], /* CJK Ideograph */
  ["\u81BF", "\u81BF"], /* CJK Ideograph */
  ["\u81C0", "\u81C0"], /* CJK Ideograph */
  ["\u81C1", "\u81C1"], /* CJK Ideograph */
  ["\u81C2", "\u81C2"], /* CJK Ideograph */
  ["\u81C3", "\u81C3"], /* CJK Ideograph */
  ["\u81C4", "\u81C4"], /* CJK Ideograph */
  ["\u81C5", "\u81C5"], /* CJK Ideograph */
  ["\u81C6", "\u81C6"], /* CJK Ideograph */
  ["\u81C7", "\u81C7"], /* CJK Ideograph */
  ["\u81C8", "\u81C8"], /* CJK Ideograph */
  ["\u81C9", "\u81C9"], /* CJK Ideograph */
  ["\u81CA", "\u81CA"], /* CJK Ideograph */
  ["\u81CB", "\u81CB"], /* CJK Ideograph */
  ["\u81CC", "\u81CC"], /* CJK Ideograph */
  ["\u81CD", "\u81CD"], /* CJK Ideograph */
  ["\u81CE", "\u81CE"], /* CJK Ideograph */
  ["\u81CF", "\u81CF"], /* CJK Ideograph */
  ["\u81D0", "\u81D0"], /* CJK Ideograph */
  ["\u81D1", "\u81D1"], /* CJK Ideograph */
  ["\u81D2", "\u81D2"], /* CJK Ideograph */
  ["\u81D3", "\u81D3"], /* CJK Ideograph */
  ["\u81D4", "\u81D4"], /* CJK Ideograph */
  ["\u81D5", "\u81D5"], /* CJK Ideograph */
  ["\u81D6", "\u81D6"], /* CJK Ideograph */
  ["\u81D7", "\u81D7"], /* CJK Ideograph */
  ["\u81D8", "\u81D8"], /* CJK Ideograph */
  ["\u81D9", "\u81D9"], /* CJK Ideograph */
  ["\u81DA", "\u81DA"], /* CJK Ideograph */
  ["\u81DB", "\u81DB"], /* CJK Ideograph */
  ["\u81DC", "\u81DC"], /* CJK Ideograph */
  ["\u81DD", "\u81DD"], /* CJK Ideograph */
  ["\u81DE", "\u81DE"], /* CJK Ideograph */
  ["\u81DF", "\u81DF"], /* CJK Ideograph */
  ["\u81E0", "\u81E0"], /* CJK Ideograph */
  ["\u81E1", "\u81E1"], /* CJK Ideograph */
  ["\u81E2", "\u81E2"], /* CJK Ideograph */
  ["\u81E3", "\u81E3"], /* CJK Ideograph */
  ["\u81E4", "\u81E4"], /* CJK Ideograph */
  ["\u81E5", "\u81E5"], /* CJK Ideograph */
  ["\u81E6", "\u81E6"], /* CJK Ideograph */
  ["\u81E7", "\u81E7"], /* CJK Ideograph */
  ["\u81E8", "\u81E8"], /* CJK Ideograph */
  ["\u81E9", "\u81E9"], /* CJK Ideograph */
  ["\u81EA", "\u81EA"], /* CJK Ideograph */
  ["\u81EB", "\u81EB"], /* CJK Ideograph */
  ["\u81EC", "\u81EC"], /* CJK Ideograph */
  ["\u81ED", "\u81ED"], /* CJK Ideograph */
  ["\u81EE", "\u81EE"], /* CJK Ideograph */
  ["\u81EF", "\u81EF"], /* CJK Ideograph */
  ["\u81F0", "\u81F0"], /* CJK Ideograph */
  ["\u81F1", "\u81F1"], /* CJK Ideograph */
  ["\u81F2", "\u81F2"], /* CJK Ideograph */
  ["\u81F3", "\u81F3"], /* CJK Ideograph */
  ["\u81F4", "\u81F4"], /* CJK Ideograph */
  ["\u81F5", "\u81F5"], /* CJK Ideograph */
  ["\u81F6", "\u81F6"], /* CJK Ideograph */
  ["\u81F7", "\u81F7"], /* CJK Ideograph */
  ["\u81F8", "\u81F8"], /* CJK Ideograph */
  ["\u81F9", "\u81F9"], /* CJK Ideograph */
  ["\u81FA", "\u81FA"], /* CJK Ideograph */
  ["\u81FB", "\u81FB"], /* CJK Ideograph */
  ["\u81FC", "\u81FC"], /* CJK Ideograph */
  ["\u81FD", "\u81FD"], /* CJK Ideograph */
  ["\u81FE", "\u81FE"], /* CJK Ideograph */
  ["\u81FF", "\u81FF"], /* CJK Ideograph */
  ["\u8200", "\u8200"], /* CJK Ideograph */
  ["\u8201", "\u8201"], /* CJK Ideograph */
  ["\u8202", "\u8202"], /* CJK Ideograph */
  ["\u8203", "\u8203"], /* CJK Ideograph */
  ["\u8204", "\u8204"], /* CJK Ideograph */
  ["\u8205", "\u8205"], /* CJK Ideograph */
  ["\u8206", "\u8206"], /* CJK Ideograph */
  ["\u8207", "\u8207"], /* CJK Ideograph */
  ["\u8208", "\u8208"], /* CJK Ideograph */
  ["\u8209", "\u8209"], /* CJK Ideograph */
  ["\u820A", "\u820A"], /* CJK Ideograph */
  ["\u820B", "\u820B"], /* CJK Ideograph */
  ["\u820C", "\u820C"], /* CJK Ideograph */
  ["\u820D", "\u820D"], /* CJK Ideograph */
  ["\u820E", "\u820E"], /* CJK Ideograph */
  ["\u820F", "\u820F"], /* CJK Ideograph */
  ["\u8210", "\u8210"], /* CJK Ideograph */
  ["\u8211", "\u8211"], /* CJK Ideograph */
  ["\u8212", "\u8212"], /* CJK Ideograph */
  ["\u8213", "\u8213"], /* CJK Ideograph */
  ["\u8214", "\u8214"], /* CJK Ideograph */
  ["\u8215", "\u8215"], /* CJK Ideograph */
  ["\u8216", "\u8216"], /* CJK Ideograph */
  ["\u8217", "\u8217"], /* CJK Ideograph */
  ["\u8218", "\u8218"], /* CJK Ideograph */
  ["\u8219", "\u8219"], /* CJK Ideograph */
  ["\u821A", "\u821A"], /* CJK Ideograph */
  ["\u821B", "\u821B"], /* CJK Ideograph */
  ["\u821C", "\u821C"], /* CJK Ideograph */
  ["\u821D", "\u821D"], /* CJK Ideograph */
  ["\u821E", "\u821E"], /* CJK Ideograph */
  ["\u821F", "\u821F"], /* CJK Ideograph */
  ["\u8220", "\u8220"], /* CJK Ideograph */
  ["\u8221", "\u8221"], /* CJK Ideograph */
  ["\u8222", "\u8222"], /* CJK Ideograph */
  ["\u8223", "\u8223"], /* CJK Ideograph */
  ["\u8224", "\u8224"], /* CJK Ideograph */
  ["\u8225", "\u8225"], /* CJK Ideograph */
  ["\u8226", "\u8226"], /* CJK Ideograph */
  ["\u8227", "\u8227"], /* CJK Ideograph */
  ["\u8228", "\u8228"], /* CJK Ideograph */
  ["\u8229", "\u8229"], /* CJK Ideograph */
  ["\u822A", "\u822A"], /* CJK Ideograph */
  ["\u822B", "\u822B"], /* CJK Ideograph */
  ["\u822C", "\u822C"], /* CJK Ideograph */
  ["\u822D", "\u822D"], /* CJK Ideograph */
  ["\u822E", "\u822E"], /* CJK Ideograph */
  ["\u822F", "\u822F"], /* CJK Ideograph */
  ["\u8230", "\u8230"], /* CJK Ideograph */
  ["\u8231", "\u8231"], /* CJK Ideograph */
  ["\u8232", "\u8232"], /* CJK Ideograph */
  ["\u8233", "\u8233"], /* CJK Ideograph */
  ["\u8234", "\u8234"], /* CJK Ideograph */
  ["\u8235", "\u8235"], /* CJK Ideograph */
  ["\u8236", "\u8236"], /* CJK Ideograph */
  ["\u8237", "\u8237"], /* CJK Ideograph */
  ["\u8238", "\u8238"], /* CJK Ideograph */
  ["\u8239", "\u8239"], /* CJK Ideograph */
  ["\u823A", "\u823A"], /* CJK Ideograph */
  ["\u823B", "\u823B"], /* CJK Ideograph */
  ["\u823C", "\u823C"], /* CJK Ideograph */
  ["\u823D", "\u823D"], /* CJK Ideograph */
  ["\u823E", "\u823E"], /* CJK Ideograph */
  ["\u823F", "\u823F"], /* CJK Ideograph */
  ["\u8240", "\u8240"], /* CJK Ideograph */
  ["\u8241", "\u8241"], /* CJK Ideograph */
  ["\u8242", "\u8242"], /* CJK Ideograph */
  ["\u8243", "\u8243"], /* CJK Ideograph */
  ["\u8244", "\u8244"], /* CJK Ideograph */
  ["\u8245", "\u8245"], /* CJK Ideograph */
  ["\u8246", "\u8246"], /* CJK Ideograph */
  ["\u8247", "\u8247"], /* CJK Ideograph */
  ["\u8248", "\u8248"], /* CJK Ideograph */
  ["\u8249", "\u8249"], /* CJK Ideograph */
  ["\u824A", "\u824A"], /* CJK Ideograph */
  ["\u824B", "\u824B"], /* CJK Ideograph */
  ["\u824C", "\u824C"], /* CJK Ideograph */
  ["\u824D", "\u824D"], /* CJK Ideograph */
  ["\u824E", "\u824E"], /* CJK Ideograph */
  ["\u824F", "\u824F"], /* CJK Ideograph */
  ["\u8250", "\u8250"], /* CJK Ideograph */
  ["\u8251", "\u8251"], /* CJK Ideograph */
  ["\u8252", "\u8252"], /* CJK Ideograph */
  ["\u8253", "\u8253"], /* CJK Ideograph */
  ["\u8254", "\u8254"], /* CJK Ideograph */
  ["\u8255", "\u8255"], /* CJK Ideograph */
  ["\u8256", "\u8256"], /* CJK Ideograph */
  ["\u8257", "\u8257"], /* CJK Ideograph */
  ["\u8258", "\u8258"], /* CJK Ideograph */
  ["\u8259", "\u8259"], /* CJK Ideograph */
  ["\u825A", "\u825A"], /* CJK Ideograph */
  ["\u825B", "\u825B"], /* CJK Ideograph */
  ["\u825C", "\u825C"], /* CJK Ideograph */
  ["\u825D", "\u825D"], /* CJK Ideograph */
  ["\u825E", "\u825E"], /* CJK Ideograph */
  ["\u825F", "\u825F"], /* CJK Ideograph */
  ["\u8260", "\u8260"], /* CJK Ideograph */
  ["\u8261", "\u8261"], /* CJK Ideograph */
  ["\u8262", "\u8262"], /* CJK Ideograph */
  ["\u8263", "\u8263"], /* CJK Ideograph */
  ["\u8264", "\u8264"], /* CJK Ideograph */
  ["\u8265", "\u8265"], /* CJK Ideograph */
  ["\u8266", "\u8266"], /* CJK Ideograph */
  ["\u8267", "\u8267"], /* CJK Ideograph */
  ["\u8268", "\u8268"], /* CJK Ideograph */
  ["\u8269", "\u8269"], /* CJK Ideograph */
  ["\u826A", "\u826A"], /* CJK Ideograph */
  ["\u826B", "\u826B"], /* CJK Ideograph */
  ["\u826C", "\u826C"], /* CJK Ideograph */
  ["\u826D", "\u826D"], /* CJK Ideograph */
  ["\u826E", "\u826E"], /* CJK Ideograph */
  ["\u826F", "\u826F"], /* CJK Ideograph */
  ["\u8270", "\u8270"], /* CJK Ideograph */
  ["\u8271", "\u8271"], /* CJK Ideograph */
  ["\u8272", "\u8272"], /* CJK Ideograph */
  ["\u8273", "\u8273"], /* CJK Ideograph */
  ["\u8274", "\u8274"], /* CJK Ideograph */
  ["\u8275", "\u8275"], /* CJK Ideograph */
  ["\u8276", "\u8276"], /* CJK Ideograph */
  ["\u8277", "\u8277"], /* CJK Ideograph */
  ["\u8278", "\u8278"], /* CJK Ideograph */
  ["\u8279", "\u8279"], /* CJK Ideograph */
  ["\u827A", "\u827A"], /* CJK Ideograph */
  ["\u827B", "\u827B"], /* CJK Ideograph */
  ["\u827C", "\u827C"], /* CJK Ideograph */
  ["\u827D", "\u827D"], /* CJK Ideograph */
  ["\u827E", "\u827E"], /* CJK Ideograph */
  ["\u827F", "\u827F"], /* CJK Ideograph */
  ["\u8280", "\u8280"], /* CJK Ideograph */
  ["\u8281", "\u8281"], /* CJK Ideograph */
  ["\u8282", "\u8282"], /* CJK Ideograph */
  ["\u8283", "\u8283"], /* CJK Ideograph */
  ["\u8284", "\u8284"], /* CJK Ideograph */
  ["\u8285", "\u8285"], /* CJK Ideograph */
  ["\u8286", "\u8286"], /* CJK Ideograph */
  ["\u8287", "\u8287"], /* CJK Ideograph */
  ["\u8288", "\u8288"], /* CJK Ideograph */
  ["\u8289", "\u8289"], /* CJK Ideograph */
  ["\u828A", "\u828A"], /* CJK Ideograph */
  ["\u828B", "\u828B"], /* CJK Ideograph */
  ["\u828C", "\u828C"], /* CJK Ideograph */
  ["\u828D", "\u828D"], /* CJK Ideograph */
  ["\u828E", "\u828E"], /* CJK Ideograph */
  ["\u828F", "\u828F"], /* CJK Ideograph */
  ["\u8290", "\u8290"], /* CJK Ideograph */
  ["\u8291", "\u8291"], /* CJK Ideograph */
  ["\u8292", "\u8292"], /* CJK Ideograph */
  ["\u8293", "\u8293"], /* CJK Ideograph */
  ["\u8294", "\u8294"], /* CJK Ideograph */
  ["\u8295", "\u8295"], /* CJK Ideograph */
  ["\u8296", "\u8296"], /* CJK Ideograph */
  ["\u8297", "\u8297"], /* CJK Ideograph */
  ["\u8298", "\u8298"], /* CJK Ideograph */
  ["\u8299", "\u8299"], /* CJK Ideograph */
  ["\u829A", "\u829A"], /* CJK Ideograph */
  ["\u829B", "\u829B"], /* CJK Ideograph */
  ["\u829C", "\u829C"], /* CJK Ideograph */
  ["\u829D", "\u829D"], /* CJK Ideograph */
  ["\u829E", "\u829E"], /* CJK Ideograph */
  ["\u829F", "\u829F"], /* CJK Ideograph */
  ["\u82A0", "\u82A0"], /* CJK Ideograph */
  ["\u82A1", "\u82A1"], /* CJK Ideograph */
  ["\u82A2", "\u82A2"], /* CJK Ideograph */
  ["\u82A3", "\u82A3"], /* CJK Ideograph */
  ["\u82A4", "\u82A4"], /* CJK Ideograph */
  ["\u82A5", "\u82A5"], /* CJK Ideograph */
  ["\u82A6", "\u82A6"], /* CJK Ideograph */
  ["\u82A7", "\u82A7"], /* CJK Ideograph */
  ["\u82A8", "\u82A8"], /* CJK Ideograph */
  ["\u82A9", "\u82A9"], /* CJK Ideograph */
  ["\u82AA", "\u82AA"], /* CJK Ideograph */
  ["\u82AB", "\u82AB"], /* CJK Ideograph */
  ["\u82AC", "\u82AC"], /* CJK Ideograph */
  ["\u82AD", "\u82AD"], /* CJK Ideograph */
  ["\u82AE", "\u82AE"], /* CJK Ideograph */
  ["\u82AF", "\u82AF"], /* CJK Ideograph */
  ["\u82B0", "\u82B0"], /* CJK Ideograph */
  ["\u82B1", "\u82B1"], /* CJK Ideograph */
  ["\u82B2", "\u82B2"], /* CJK Ideograph */
  ["\u82B3", "\u82B3"], /* CJK Ideograph */
  ["\u82B4", "\u82B4"], /* CJK Ideograph */
  ["\u82B5", "\u82B5"], /* CJK Ideograph */
  ["\u82B6", "\u82B6"], /* CJK Ideograph */
  ["\u82B7", "\u82B7"], /* CJK Ideograph */
  ["\u82B8", "\u82B8"], /* CJK Ideograph */
  ["\u82B9", "\u82B9"], /* CJK Ideograph */
  ["\u82BA", "\u82BA"], /* CJK Ideograph */
  ["\u82BB", "\u82BB"], /* CJK Ideograph */
  ["\u82BC", "\u82BC"], /* CJK Ideograph */
  ["\u82BD", "\u82BD"], /* CJK Ideograph */
  ["\u82BE", "\u82BE"], /* CJK Ideograph */
  ["\u82BF", "\u82BF"], /* CJK Ideograph */
  ["\u82C0", "\u82C0"], /* CJK Ideograph */
  ["\u82C1", "\u82C1"], /* CJK Ideograph */
  ["\u82C2", "\u82C2"], /* CJK Ideograph */
  ["\u82C3", "\u82C3"], /* CJK Ideograph */
  ["\u82C4", "\u82C4"], /* CJK Ideograph */
  ["\u82C5", "\u82C5"], /* CJK Ideograph */
  ["\u82C6", "\u82C6"], /* CJK Ideograph */
  ["\u82C7", "\u82C7"], /* CJK Ideograph */
  ["\u82C8", "\u82C8"], /* CJK Ideograph */
  ["\u82C9", "\u82C9"], /* CJK Ideograph */
  ["\u82CA", "\u82CA"], /* CJK Ideograph */
  ["\u82CB", "\u82CB"], /* CJK Ideograph */
  ["\u82CC", "\u82CC"], /* CJK Ideograph */
  ["\u82CD", "\u82CD"], /* CJK Ideograph */
  ["\u82CE", "\u82CE"], /* CJK Ideograph */
  ["\u82CF", "\u82CF"], /* CJK Ideograph */
  ["\u82D0", "\u82D0"], /* CJK Ideograph */
  ["\u82D1", "\u82D1"], /* CJK Ideograph */
  ["\u82D2", "\u82D2"], /* CJK Ideograph */
  ["\u82D3", "\u82D3"], /* CJK Ideograph */
  ["\u82D4", "\u82D4"], /* CJK Ideograph */
  ["\u82D5", "\u82D5"], /* CJK Ideograph */
  ["\u82D6", "\u82D6"], /* CJK Ideograph */
  ["\u82D7", "\u82D7"], /* CJK Ideograph */
  ["\u82D8", "\u82D8"], /* CJK Ideograph */
  ["\u82D9", "\u82D9"], /* CJK Ideograph */
  ["\u82DA", "\u82DA"], /* CJK Ideograph */
  ["\u82DB", "\u82DB"], /* CJK Ideograph */
  ["\u82DC", "\u82DC"], /* CJK Ideograph */
  ["\u82DD", "\u82DD"], /* CJK Ideograph */
  ["\u82DE", "\u82DE"], /* CJK Ideograph */
  ["\u82DF", "\u82DF"], /* CJK Ideograph */
  ["\u82E0", "\u82E0"], /* CJK Ideograph */
  ["\u82E1", "\u82E1"], /* CJK Ideograph */
  ["\u82E2", "\u82E2"], /* CJK Ideograph */
  ["\u82E3", "\u82E3"], /* CJK Ideograph */
  ["\u82E4", "\u82E4"], /* CJK Ideograph */
  ["\u82E5", "\u82E5"], /* CJK Ideograph */
  ["\u82E6", "\u82E6"], /* CJK Ideograph */
  ["\u82E7", "\u82E7"], /* CJK Ideograph */
  ["\u82E8", "\u82E8"], /* CJK Ideograph */
  ["\u82E9", "\u82E9"], /* CJK Ideograph */
  ["\u82EA", "\u82EA"], /* CJK Ideograph */
  ["\u82EB", "\u82EB"], /* CJK Ideograph */
  ["\u82EC", "\u82EC"], /* CJK Ideograph */
  ["\u82ED", "\u82ED"], /* CJK Ideograph */
  ["\u82EE", "\u82EE"], /* CJK Ideograph */
  ["\u82EF", "\u82EF"], /* CJK Ideograph */
  ["\u82F0", "\u82F0"], /* CJK Ideograph */
  ["\u82F1", "\u82F1"], /* CJK Ideograph */
  ["\u82F2", "\u82F2"], /* CJK Ideograph */
  ["\u82F3", "\u82F3"], /* CJK Ideograph */
  ["\u82F4", "\u82F4"], /* CJK Ideograph */
  ["\u82F5", "\u82F5"], /* CJK Ideograph */
  ["\u82F6", "\u82F6"], /* CJK Ideograph */
  ["\u82F7", "\u82F7"], /* CJK Ideograph */
  ["\u82F8", "\u82F8"], /* CJK Ideograph */
  ["\u82F9", "\u82F9"], /* CJK Ideograph */
  ["\u82FA", "\u82FA"], /* CJK Ideograph */
  ["\u82FB", "\u82FB"], /* CJK Ideograph */
  ["\u82FC", "\u82FC"], /* CJK Ideograph */
  ["\u82FD", "\u82FD"], /* CJK Ideograph */
  ["\u82FE", "\u82FE"], /* CJK Ideograph */
  ["\u82FF", "\u82FF"], /* CJK Ideograph */
  ["\u8300", "\u8300"], /* CJK Ideograph */
  ["\u8301", "\u8301"], /* CJK Ideograph */
  ["\u8302", "\u8302"], /* CJK Ideograph */
  ["\u8303", "\u8303"], /* CJK Ideograph */
  ["\u8304", "\u8304"], /* CJK Ideograph */
  ["\u8305", "\u8305"], /* CJK Ideograph */
  ["\u8306", "\u8306"], /* CJK Ideograph */
  ["\u8307", "\u8307"], /* CJK Ideograph */
  ["\u8308", "\u8308"], /* CJK Ideograph */
  ["\u8309", "\u8309"], /* CJK Ideograph */
  ["\u830A", "\u830A"], /* CJK Ideograph */
  ["\u830B", "\u830B"], /* CJK Ideograph */
  ["\u830C", "\u830C"], /* CJK Ideograph */
  ["\u830D", "\u830D"], /* CJK Ideograph */
  ["\u830E", "\u830E"], /* CJK Ideograph */
  ["\u830F", "\u830F"], /* CJK Ideograph */
  ["\u8310", "\u8310"], /* CJK Ideograph */
  ["\u8311", "\u8311"], /* CJK Ideograph */
  ["\u8312", "\u8312"], /* CJK Ideograph */
  ["\u8313", "\u8313"], /* CJK Ideograph */
  ["\u8314", "\u8314"], /* CJK Ideograph */
  ["\u8315", "\u8315"], /* CJK Ideograph */
  ["\u8316", "\u8316"], /* CJK Ideograph */
  ["\u8317", "\u8317"], /* CJK Ideograph */
  ["\u8318", "\u8318"], /* CJK Ideograph */
  ["\u8319", "\u8319"], /* CJK Ideograph */
  ["\u831A", "\u831A"], /* CJK Ideograph */
  ["\u831B", "\u831B"], /* CJK Ideograph */
  ["\u831C", "\u831C"], /* CJK Ideograph */
  ["\u831D", "\u831D"], /* CJK Ideograph */
  ["\u831E", "\u831E"], /* CJK Ideograph */
  ["\u831F", "\u831F"], /* CJK Ideograph */
  ["\u8320", "\u8320"], /* CJK Ideograph */
  ["\u8321", "\u8321"], /* CJK Ideograph */
  ["\u8322", "\u8322"], /* CJK Ideograph */
  ["\u8323", "\u8323"], /* CJK Ideograph */
  ["\u8324", "\u8324"], /* CJK Ideograph */
  ["\u8325", "\u8325"], /* CJK Ideograph */
  ["\u8326", "\u8326"], /* CJK Ideograph */
  ["\u8327", "\u8327"], /* CJK Ideograph */
  ["\u8328", "\u8328"], /* CJK Ideograph */
  ["\u8329", "\u8329"], /* CJK Ideograph */
  ["\u832A", "\u832A"], /* CJK Ideograph */
  ["\u832B", "\u832B"], /* CJK Ideograph */
  ["\u832C", "\u832C"], /* CJK Ideograph */
  ["\u832D", "\u832D"], /* CJK Ideograph */
  ["\u832E", "\u832E"], /* CJK Ideograph */
  ["\u832F", "\u832F"], /* CJK Ideograph */
  ["\u8330", "\u8330"], /* CJK Ideograph */
  ["\u8331", "\u8331"], /* CJK Ideograph */
  ["\u8332", "\u8332"], /* CJK Ideograph */
  ["\u8333", "\u8333"], /* CJK Ideograph */
  ["\u8334", "\u8334"], /* CJK Ideograph */
  ["\u8335", "\u8335"], /* CJK Ideograph */
  ["\u8336", "\u8336"], /* CJK Ideograph */
  ["\u8337", "\u8337"], /* CJK Ideograph */
  ["\u8338", "\u8338"], /* CJK Ideograph */
  ["\u8339", "\u8339"], /* CJK Ideograph */
  ["\u833A", "\u833A"], /* CJK Ideograph */
  ["\u833B", "\u833B"], /* CJK Ideograph */
  ["\u833C", "\u833C"], /* CJK Ideograph */
  ["\u833D", "\u833D"], /* CJK Ideograph */
  ["\u833E", "\u833E"], /* CJK Ideograph */
  ["\u833F", "\u833F"], /* CJK Ideograph */
  ["\u8340", "\u8340"], /* CJK Ideograph */
  ["\u8341", "\u8341"], /* CJK Ideograph */
  ["\u8342", "\u8342"], /* CJK Ideograph */
  ["\u8343", "\u8343"], /* CJK Ideograph */
  ["\u8344", "\u8344"], /* CJK Ideograph */
  ["\u8345", "\u8345"], /* CJK Ideograph */
  ["\u8346", "\u8346"], /* CJK Ideograph */
  ["\u8347", "\u8347"], /* CJK Ideograph */
  ["\u8348", "\u8348"], /* CJK Ideograph */
  ["\u8349", "\u8349"], /* CJK Ideograph */
  ["\u834A", "\u834A"], /* CJK Ideograph */
  ["\u834B", "\u834B"], /* CJK Ideograph */
  ["\u834C", "\u834C"], /* CJK Ideograph */
  ["\u834D", "\u834D"], /* CJK Ideograph */
  ["\u834E", "\u834E"], /* CJK Ideograph */
  ["\u834F", "\u834F"], /* CJK Ideograph */
  ["\u8350", "\u8350"], /* CJK Ideograph */
  ["\u8351", "\u8351"], /* CJK Ideograph */
  ["\u8352", "\u8352"], /* CJK Ideograph */
  ["\u8353", "\u8353"], /* CJK Ideograph */
  ["\u8354", "\u8354"], /* CJK Ideograph */
  ["\u8355", "\u8355"], /* CJK Ideograph */
  ["\u8356", "\u8356"], /* CJK Ideograph */
  ["\u8357", "\u8357"], /* CJK Ideograph */
  ["\u8358", "\u8358"], /* CJK Ideograph */
  ["\u8359", "\u8359"], /* CJK Ideograph */
  ["\u835A", "\u835A"], /* CJK Ideograph */
  ["\u835B", "\u835B"], /* CJK Ideograph */
  ["\u835C", "\u835C"], /* CJK Ideograph */
  ["\u835D", "\u835D"], /* CJK Ideograph */
  ["\u835E", "\u835E"], /* CJK Ideograph */
  ["\u835F", "\u835F"], /* CJK Ideograph */
  ["\u8360", "\u8360"], /* CJK Ideograph */
  ["\u8361", "\u8361"], /* CJK Ideograph */
  ["\u8362", "\u8362"], /* CJK Ideograph */
  ["\u8363", "\u8363"], /* CJK Ideograph */
  ["\u8364", "\u8364"], /* CJK Ideograph */
  ["\u8365", "\u8365"], /* CJK Ideograph */
  ["\u8366", "\u8366"], /* CJK Ideograph */
  ["\u8367", "\u8367"], /* CJK Ideograph */
  ["\u8368", "\u8368"], /* CJK Ideograph */
  ["\u8369", "\u8369"], /* CJK Ideograph */
  ["\u836A", "\u836A"], /* CJK Ideograph */
  ["\u836B", "\u836B"], /* CJK Ideograph */
  ["\u836C", "\u836C"], /* CJK Ideograph */
  ["\u836D", "\u836D"], /* CJK Ideograph */
  ["\u836E", "\u836E"], /* CJK Ideograph */
  ["\u836F", "\u836F"], /* CJK Ideograph */
  ["\u8370", "\u8370"], /* CJK Ideograph */
  ["\u8371", "\u8371"], /* CJK Ideograph */
  ["\u8372", "\u8372"], /* CJK Ideograph */
  ["\u8373", "\u8373"], /* CJK Ideograph */
  ["\u8374", "\u8374"], /* CJK Ideograph */
  ["\u8375", "\u8375"], /* CJK Ideograph */
  ["\u8376", "\u8376"], /* CJK Ideograph */
  ["\u8377", "\u8377"], /* CJK Ideograph */
  ["\u8378", "\u8378"], /* CJK Ideograph */
  ["\u8379", "\u8379"], /* CJK Ideograph */
  ["\u837A", "\u837A"], /* CJK Ideograph */
  ["\u837B", "\u837B"], /* CJK Ideograph */
  ["\u837C", "\u837C"], /* CJK Ideograph */
  ["\u837D", "\u837D"], /* CJK Ideograph */
  ["\u837E", "\u837E"], /* CJK Ideograph */
  ["\u837F", "\u837F"], /* CJK Ideograph */
  ["\u8380", "\u8380"], /* CJK Ideograph */
  ["\u8381", "\u8381"], /* CJK Ideograph */
  ["\u8382", "\u8382"], /* CJK Ideograph */
  ["\u8383", "\u8383"], /* CJK Ideograph */
  ["\u8384", "\u8384"], /* CJK Ideograph */
  ["\u8385", "\u8385"], /* CJK Ideograph */
  ["\u8386", "\u8386"], /* CJK Ideograph */
  ["\u8387", "\u8387"], /* CJK Ideograph */
  ["\u8388", "\u8388"], /* CJK Ideograph */
  ["\u8389", "\u8389"], /* CJK Ideograph */
  ["\u838A", "\u838A"], /* CJK Ideograph */
  ["\u838B", "\u838B"], /* CJK Ideograph */
  ["\u838C", "\u838C"], /* CJK Ideograph */
  ["\u838D", "\u838D"], /* CJK Ideograph */
  ["\u838E", "\u838E"], /* CJK Ideograph */
  ["\u838F", "\u838F"], /* CJK Ideograph */
  ["\u8390", "\u8390"], /* CJK Ideograph */
  ["\u8391", "\u8391"], /* CJK Ideograph */
  ["\u8392", "\u8392"], /* CJK Ideograph */
  ["\u8393", "\u8393"], /* CJK Ideograph */
  ["\u8394", "\u8394"], /* CJK Ideograph */
  ["\u8395", "\u8395"], /* CJK Ideograph */
  ["\u8396", "\u8396"], /* CJK Ideograph */
  ["\u8397", "\u8397"], /* CJK Ideograph */
  ["\u8398", "\u8398"], /* CJK Ideograph */
  ["\u8399", "\u8399"], /* CJK Ideograph */
  ["\u839A", "\u839A"], /* CJK Ideograph */
  ["\u839B", "\u839B"], /* CJK Ideograph */
  ["\u839C", "\u839C"], /* CJK Ideograph */
  ["\u839D", "\u839D"], /* CJK Ideograph */
  ["\u839E", "\u839E"], /* CJK Ideograph */
  ["\u839F", "\u839F"], /* CJK Ideograph */
  ["\u83A0", "\u83A0"], /* CJK Ideograph */
  ["\u83A1", "\u83A1"], /* CJK Ideograph */
  ["\u83A2", "\u83A2"], /* CJK Ideograph */
  ["\u83A3", "\u83A3"], /* CJK Ideograph */
  ["\u83A4", "\u83A4"], /* CJK Ideograph */
  ["\u83A5", "\u83A5"], /* CJK Ideograph */
  ["\u83A6", "\u83A6"], /* CJK Ideograph */
  ["\u83A7", "\u83A7"], /* CJK Ideograph */
  ["\u83A8", "\u83A8"], /* CJK Ideograph */
  ["\u83A9", "\u83A9"], /* CJK Ideograph */
  ["\u83AA", "\u83AA"], /* CJK Ideograph */
  ["\u83AB", "\u83AB"], /* CJK Ideograph */
  ["\u83AC", "\u83AC"], /* CJK Ideograph */
  ["\u83AD", "\u83AD"], /* CJK Ideograph */
  ["\u83AE", "\u83AE"], /* CJK Ideograph */
  ["\u83AF", "\u83AF"], /* CJK Ideograph */
  ["\u83B0", "\u83B0"], /* CJK Ideograph */
  ["\u83B1", "\u83B1"], /* CJK Ideograph */
  ["\u83B2", "\u83B2"], /* CJK Ideograph */
  ["\u83B3", "\u83B3"], /* CJK Ideograph */
  ["\u83B4", "\u83B4"], /* CJK Ideograph */
  ["\u83B5", "\u83B5"], /* CJK Ideograph */
  ["\u83B6", "\u83B6"], /* CJK Ideograph */
  ["\u83B7", "\u83B7"], /* CJK Ideograph */
  ["\u83B8", "\u83B8"], /* CJK Ideograph */
  ["\u83B9", "\u83B9"], /* CJK Ideograph */
  ["\u83BA", "\u83BA"], /* CJK Ideograph */
  ["\u83BB", "\u83BB"], /* CJK Ideograph */
  ["\u83BC", "\u83BC"], /* CJK Ideograph */
  ["\u83BD", "\u83BD"], /* CJK Ideograph */
  ["\u83BE", "\u83BE"], /* CJK Ideograph */
  ["\u83BF", "\u83BF"], /* CJK Ideograph */
  ["\u83C0", "\u83C0"], /* CJK Ideograph */
  ["\u83C1", "\u83C1"], /* CJK Ideograph */
  ["\u83C2", "\u83C2"], /* CJK Ideograph */
  ["\u83C3", "\u83C3"], /* CJK Ideograph */
  ["\u83C4", "\u83C4"], /* CJK Ideograph */
  ["\u83C5", "\u83C5"], /* CJK Ideograph */
  ["\u83C6", "\u83C6"], /* CJK Ideograph */
  ["\u83C7", "\u83C7"], /* CJK Ideograph */
  ["\u83C8", "\u83C8"], /* CJK Ideograph */
  ["\u83C9", "\u83C9"], /* CJK Ideograph */
  ["\u83CA", "\u83CA"], /* CJK Ideograph */
  ["\u83CB", "\u83CB"], /* CJK Ideograph */
  ["\u83CC", "\u83CC"], /* CJK Ideograph */
  ["\u83CD", "\u83CD"], /* CJK Ideograph */
  ["\u83CE", "\u83CE"], /* CJK Ideograph */
  ["\u83CF", "\u83CF"], /* CJK Ideograph */
  ["\u83D0", "\u83D0"], /* CJK Ideograph */
  ["\u83D1", "\u83D1"], /* CJK Ideograph */
  ["\u83D2", "\u83D2"], /* CJK Ideograph */
  ["\u83D3", "\u83D3"], /* CJK Ideograph */
  ["\u83D4", "\u83D4"], /* CJK Ideograph */
  ["\u83D5", "\u83D5"], /* CJK Ideograph */
  ["\u83D6", "\u83D6"], /* CJK Ideograph */
  ["\u83D7", "\u83D7"], /* CJK Ideograph */
  ["\u83D8", "\u83D8"], /* CJK Ideograph */
  ["\u83D9", "\u83D9"], /* CJK Ideograph */
  ["\u83DA", "\u83DA"], /* CJK Ideograph */
  ["\u83DB", "\u83DB"], /* CJK Ideograph */
  ["\u83DC", "\u83DC"], /* CJK Ideograph */
  ["\u83DD", "\u83DD"], /* CJK Ideograph */
  ["\u83DE", "\u83DE"], /* CJK Ideograph */
  ["\u83DF", "\u83DF"], /* CJK Ideograph */
  ["\u83E0", "\u83E0"], /* CJK Ideograph */
  ["\u83E1", "\u83E1"], /* CJK Ideograph */
  ["\u83E2", "\u83E2"], /* CJK Ideograph */
  ["\u83E3", "\u83E3"], /* CJK Ideograph */
  ["\u83E4", "\u83E4"], /* CJK Ideograph */
  ["\u83E5", "\u83E5"], /* CJK Ideograph */
  ["\u83E6", "\u83E6"], /* CJK Ideograph */
  ["\u83E7", "\u83E7"], /* CJK Ideograph */
  ["\u83E8", "\u83E8"], /* CJK Ideograph */
  ["\u83E9", "\u83E9"], /* CJK Ideograph */
  ["\u83EA", "\u83EA"], /* CJK Ideograph */
  ["\u83EB", "\u83EB"], /* CJK Ideograph */
  ["\u83EC", "\u83EC"], /* CJK Ideograph */
  ["\u83ED", "\u83ED"], /* CJK Ideograph */
  ["\u83EE", "\u83EE"], /* CJK Ideograph */
  ["\u83EF", "\u83EF"], /* CJK Ideograph */
  ["\u83F0", "\u83F0"], /* CJK Ideograph */
  ["\u83F1", "\u83F1"], /* CJK Ideograph */
  ["\u83F2", "\u83F2"], /* CJK Ideograph */
  ["\u83F3", "\u83F3"], /* CJK Ideograph */
  ["\u83F4", "\u83F4"], /* CJK Ideograph */
  ["\u83F5", "\u83F5"], /* CJK Ideograph */
  ["\u83F6", "\u83F6"], /* CJK Ideograph */
  ["\u83F7", "\u83F7"], /* CJK Ideograph */
  ["\u83F8", "\u83F8"], /* CJK Ideograph */
  ["\u83F9", "\u83F9"], /* CJK Ideograph */
  ["\u83FA", "\u83FA"], /* CJK Ideograph */
  ["\u83FB", "\u83FB"], /* CJK Ideograph */
  ["\u83FC", "\u83FC"], /* CJK Ideograph */
  ["\u83FD", "\u83FD"], /* CJK Ideograph */
  ["\u83FE", "\u83FE"], /* CJK Ideograph */
  ["\u83FF", "\u83FF"], /* CJK Ideograph */
  ["\u8400", "\u8400"], /* CJK Ideograph */
  ["\u8401", "\u8401"], /* CJK Ideograph */
  ["\u8402", "\u8402"], /* CJK Ideograph */
  ["\u8403", "\u8403"], /* CJK Ideograph */
  ["\u8404", "\u8404"], /* CJK Ideograph */
  ["\u8405", "\u8405"], /* CJK Ideograph */
  ["\u8406", "\u8406"], /* CJK Ideograph */
  ["\u8407", "\u8407"], /* CJK Ideograph */
  ["\u8408", "\u8408"], /* CJK Ideograph */
  ["\u8409", "\u8409"], /* CJK Ideograph */
  ["\u840A", "\u840A"], /* CJK Ideograph */
  ["\u840B", "\u840B"], /* CJK Ideograph */
  ["\u840C", "\u840C"], /* CJK Ideograph */
  ["\u840D", "\u840D"], /* CJK Ideograph */
  ["\u840E", "\u840E"], /* CJK Ideograph */
  ["\u840F", "\u840F"], /* CJK Ideograph */
  ["\u8410", "\u8410"], /* CJK Ideograph */
  ["\u8411", "\u8411"], /* CJK Ideograph */
  ["\u8412", "\u8412"], /* CJK Ideograph */
  ["\u8413", "\u8413"], /* CJK Ideograph */
  ["\u8414", "\u8414"], /* CJK Ideograph */
  ["\u8415", "\u8415"], /* CJK Ideograph */
  ["\u8416", "\u8416"], /* CJK Ideograph */
  ["\u8417", "\u8417"], /* CJK Ideograph */
  ["\u8418", "\u8418"], /* CJK Ideograph */
  ["\u8419", "\u8419"], /* CJK Ideograph */
  ["\u841A", "\u841A"], /* CJK Ideograph */
  ["\u841B", "\u841B"], /* CJK Ideograph */
  ["\u841C", "\u841C"], /* CJK Ideograph */
  ["\u841D", "\u841D"], /* CJK Ideograph */
  ["\u841E", "\u841E"], /* CJK Ideograph */
  ["\u841F", "\u841F"], /* CJK Ideograph */
  ["\u8420", "\u8420"], /* CJK Ideograph */
  ["\u8421", "\u8421"], /* CJK Ideograph */
  ["\u8422", "\u8422"], /* CJK Ideograph */
  ["\u8423", "\u8423"], /* CJK Ideograph */
  ["\u8424", "\u8424"], /* CJK Ideograph */
  ["\u8425", "\u8425"], /* CJK Ideograph */
  ["\u8426", "\u8426"], /* CJK Ideograph */
  ["\u8427", "\u8427"], /* CJK Ideograph */
  ["\u8428", "\u8428"], /* CJK Ideograph */
  ["\u8429", "\u8429"], /* CJK Ideograph */
  ["\u842A", "\u842A"], /* CJK Ideograph */
  ["\u842B", "\u842B"], /* CJK Ideograph */
  ["\u842C", "\u842C"], /* CJK Ideograph */
  ["\u842D", "\u842D"], /* CJK Ideograph */
  ["\u842E", "\u842E"], /* CJK Ideograph */
  ["\u842F", "\u842F"], /* CJK Ideograph */
  ["\u8430", "\u8430"], /* CJK Ideograph */
  ["\u8431", "\u8431"], /* CJK Ideograph */
  ["\u8432", "\u8432"], /* CJK Ideograph */
  ["\u8433", "\u8433"], /* CJK Ideograph */
  ["\u8434", "\u8434"], /* CJK Ideograph */
  ["\u8435", "\u8435"], /* CJK Ideograph */
  ["\u8436", "\u8436"], /* CJK Ideograph */
  ["\u8437", "\u8437"], /* CJK Ideograph */
  ["\u8438", "\u8438"], /* CJK Ideograph */
  ["\u8439", "\u8439"], /* CJK Ideograph */
  ["\u843A", "\u843A"], /* CJK Ideograph */
  ["\u843B", "\u843B"], /* CJK Ideograph */
  ["\u843C", "\u843C"], /* CJK Ideograph */
  ["\u843D", "\u843D"], /* CJK Ideograph */
  ["\u843E", "\u843E"], /* CJK Ideograph */
  ["\u843F", "\u843F"], /* CJK Ideograph */
  ["\u8440", "\u8440"], /* CJK Ideograph */
  ["\u8441", "\u8441"], /* CJK Ideograph */
  ["\u8442", "\u8442"], /* CJK Ideograph */
  ["\u8443", "\u8443"], /* CJK Ideograph */
  ["\u8444", "\u8444"], /* CJK Ideograph */
  ["\u8445", "\u8445"], /* CJK Ideograph */
  ["\u8446", "\u8446"], /* CJK Ideograph */
  ["\u8447", "\u8447"], /* CJK Ideograph */
  ["\u8448", "\u8448"], /* CJK Ideograph */
  ["\u8449", "\u8449"], /* CJK Ideograph */
  ["\u844A", "\u844A"], /* CJK Ideograph */
  ["\u844B", "\u844B"], /* CJK Ideograph */
  ["\u844C", "\u844C"], /* CJK Ideograph */
  ["\u844D", "\u844D"], /* CJK Ideograph */
  ["\u844E", "\u844E"], /* CJK Ideograph */
  ["\u844F", "\u844F"], /* CJK Ideograph */
  ["\u8450", "\u8450"], /* CJK Ideograph */
  ["\u8451", "\u8451"], /* CJK Ideograph */
  ["\u8452", "\u8452"], /* CJK Ideograph */
  ["\u8453", "\u8453"], /* CJK Ideograph */
  ["\u8454", "\u8454"], /* CJK Ideograph */
  ["\u8455", "\u8455"], /* CJK Ideograph */
  ["\u8456", "\u8456"], /* CJK Ideograph */
  ["\u8457", "\u8457"], /* CJK Ideograph */
  ["\u8458", "\u8458"], /* CJK Ideograph */
  ["\u8459", "\u8459"], /* CJK Ideograph */
  ["\u845A", "\u845A"], /* CJK Ideograph */
  ["\u845B", "\u845B"], /* CJK Ideograph */
  ["\u845C", "\u845C"], /* CJK Ideograph */
  ["\u845D", "\u845D"], /* CJK Ideograph */
  ["\u845E", "\u845E"], /* CJK Ideograph */
  ["\u845F", "\u845F"], /* CJK Ideograph */
  ["\u8460", "\u8460"], /* CJK Ideograph */
  ["\u8461", "\u8461"], /* CJK Ideograph */
  ["\u8462", "\u8462"], /* CJK Ideograph */
  ["\u8463", "\u8463"], /* CJK Ideograph */
  ["\u8464", "\u8464"], /* CJK Ideograph */
  ["\u8465", "\u8465"], /* CJK Ideograph */
  ["\u8466", "\u8466"], /* CJK Ideograph */
  ["\u8467", "\u8467"], /* CJK Ideograph */
  ["\u8468", "\u8468"], /* CJK Ideograph */
  ["\u8469", "\u8469"], /* CJK Ideograph */
  ["\u846A", "\u846A"], /* CJK Ideograph */
  ["\u846B", "\u846B"], /* CJK Ideograph */
  ["\u846C", "\u846C"], /* CJK Ideograph */
  ["\u846D", "\u846D"], /* CJK Ideograph */
  ["\u846E", "\u846E"], /* CJK Ideograph */
  ["\u846F", "\u846F"], /* CJK Ideograph */
  ["\u8470", "\u8470"], /* CJK Ideograph */
  ["\u8471", "\u8471"], /* CJK Ideograph */
  ["\u8472", "\u8472"], /* CJK Ideograph */
  ["\u8473", "\u8473"], /* CJK Ideograph */
  ["\u8474", "\u8474"], /* CJK Ideograph */
  ["\u8475", "\u8475"], /* CJK Ideograph */
  ["\u8476", "\u8476"], /* CJK Ideograph */
  ["\u8477", "\u8477"], /* CJK Ideograph */
  ["\u8478", "\u8478"], /* CJK Ideograph */
  ["\u8479", "\u8479"], /* CJK Ideograph */
  ["\u847A", "\u847A"], /* CJK Ideograph */
  ["\u847B", "\u847B"], /* CJK Ideograph */
  ["\u847C", "\u847C"], /* CJK Ideograph */
  ["\u847D", "\u847D"], /* CJK Ideograph */
  ["\u847E", "\u847E"], /* CJK Ideograph */
  ["\u847F", "\u847F"], /* CJK Ideograph */
  ["\u8480", "\u8480"], /* CJK Ideograph */
  ["\u8481", "\u8481"], /* CJK Ideograph */
  ["\u8482", "\u8482"], /* CJK Ideograph */
  ["\u8483", "\u8483"], /* CJK Ideograph */
  ["\u8484", "\u8484"], /* CJK Ideograph */
  ["\u8485", "\u8485"], /* CJK Ideograph */
  ["\u8486", "\u8486"], /* CJK Ideograph */
  ["\u8487", "\u8487"], /* CJK Ideograph */
  ["\u8488", "\u8488"], /* CJK Ideograph */
  ["\u8489", "\u8489"], /* CJK Ideograph */
  ["\u848A", "\u848A"], /* CJK Ideograph */
  ["\u848B", "\u848B"], /* CJK Ideograph */
  ["\u848C", "\u848C"], /* CJK Ideograph */
  ["\u848D", "\u848D"], /* CJK Ideograph */
  ["\u848E", "\u848E"], /* CJK Ideograph */
  ["\u848F", "\u848F"], /* CJK Ideograph */
  ["\u8490", "\u8490"], /* CJK Ideograph */
  ["\u8491", "\u8491"], /* CJK Ideograph */
  ["\u8492", "\u8492"], /* CJK Ideograph */
  ["\u8493", "\u8493"], /* CJK Ideograph */
  ["\u8494", "\u8494"], /* CJK Ideograph */
  ["\u8495", "\u8495"], /* CJK Ideograph */
  ["\u8496", "\u8496"], /* CJK Ideograph */
  ["\u8497", "\u8497"], /* CJK Ideograph */
  ["\u8498", "\u8498"], /* CJK Ideograph */
  ["\u8499", "\u8499"], /* CJK Ideograph */
  ["\u849A", "\u849A"], /* CJK Ideograph */
  ["\u849B", "\u849B"], /* CJK Ideograph */
  ["\u849C", "\u849C"], /* CJK Ideograph */
  ["\u849D", "\u849D"], /* CJK Ideograph */
  ["\u849E", "\u849E"], /* CJK Ideograph */
  ["\u849F", "\u849F"], /* CJK Ideograph */
  ["\u84A0", "\u84A0"], /* CJK Ideograph */
  ["\u84A1", "\u84A1"], /* CJK Ideograph */
  ["\u84A2", "\u84A2"], /* CJK Ideograph */
  ["\u84A3", "\u84A3"], /* CJK Ideograph */
  ["\u84A4", "\u84A4"], /* CJK Ideograph */
  ["\u84A5", "\u84A5"], /* CJK Ideograph */
  ["\u84A6", "\u84A6"], /* CJK Ideograph */
  ["\u84A7", "\u84A7"], /* CJK Ideograph */
  ["\u84A8", "\u84A8"], /* CJK Ideograph */
  ["\u84A9", "\u84A9"], /* CJK Ideograph */
  ["\u84AA", "\u84AA"], /* CJK Ideograph */
  ["\u84AB", "\u84AB"], /* CJK Ideograph */
  ["\u84AC", "\u84AC"], /* CJK Ideograph */
  ["\u84AD", "\u84AD"], /* CJK Ideograph */
  ["\u84AE", "\u84AE"], /* CJK Ideograph */
  ["\u84AF", "\u84AF"], /* CJK Ideograph */
  ["\u84B0", "\u84B0"], /* CJK Ideograph */
  ["\u84B1", "\u84B1"], /* CJK Ideograph */
  ["\u84B2", "\u84B2"], /* CJK Ideograph */
  ["\u84B3", "\u84B3"], /* CJK Ideograph */
  ["\u84B4", "\u84B4"], /* CJK Ideograph */
  ["\u84B5", "\u84B5"], /* CJK Ideograph */
  ["\u84B6", "\u84B6"], /* CJK Ideograph */
  ["\u84B7", "\u84B7"], /* CJK Ideograph */
  ["\u84B8", "\u84B8"], /* CJK Ideograph */
  ["\u84B9", "\u84B9"], /* CJK Ideograph */
  ["\u84BA", "\u84BA"], /* CJK Ideograph */
  ["\u84BB", "\u84BB"], /* CJK Ideograph */
  ["\u84BC", "\u84BC"], /* CJK Ideograph */
  ["\u84BD", "\u84BD"], /* CJK Ideograph */
  ["\u84BE", "\u84BE"], /* CJK Ideograph */
  ["\u84BF", "\u84BF"], /* CJK Ideograph */
  ["\u84C0", "\u84C0"], /* CJK Ideograph */
  ["\u84C1", "\u84C1"], /* CJK Ideograph */
  ["\u84C2", "\u84C2"], /* CJK Ideograph */
  ["\u84C3", "\u84C3"], /* CJK Ideograph */
  ["\u84C4", "\u84C4"], /* CJK Ideograph */
  ["\u84C5", "\u84C5"], /* CJK Ideograph */
  ["\u84C6", "\u84C6"], /* CJK Ideograph */
  ["\u84C7", "\u84C7"], /* CJK Ideograph */
  ["\u84C8", "\u84C8"], /* CJK Ideograph */
  ["\u84C9", "\u84C9"], /* CJK Ideograph */
  ["\u84CA", "\u84CA"], /* CJK Ideograph */
  ["\u84CB", "\u84CB"], /* CJK Ideograph */
  ["\u84CC", "\u84CC"], /* CJK Ideograph */
  ["\u84CD", "\u84CD"], /* CJK Ideograph */
  ["\u84CE", "\u84CE"], /* CJK Ideograph */
  ["\u84CF", "\u84CF"], /* CJK Ideograph */
  ["\u84D0", "\u84D0"], /* CJK Ideograph */
  ["\u84D1", "\u84D1"], /* CJK Ideograph */
  ["\u84D2", "\u84D2"], /* CJK Ideograph */
  ["\u84D3", "\u84D3"], /* CJK Ideograph */
  ["\u84D4", "\u84D4"], /* CJK Ideograph */
  ["\u84D5", "\u84D5"], /* CJK Ideograph */
  ["\u84D6", "\u84D6"], /* CJK Ideograph */
  ["\u84D7", "\u84D7"], /* CJK Ideograph */
  ["\u84D8", "\u84D8"], /* CJK Ideograph */
  ["\u84D9", "\u84D9"], /* CJK Ideograph */
  ["\u84DA", "\u84DA"], /* CJK Ideograph */
  ["\u84DB", "\u84DB"], /* CJK Ideograph */
  ["\u84DC", "\u84DC"], /* CJK Ideograph */
  ["\u84DD", "\u84DD"], /* CJK Ideograph */
  ["\u84DE", "\u84DE"], /* CJK Ideograph */
  ["\u84DF", "\u84DF"], /* CJK Ideograph */
  ["\u84E0", "\u84E0"], /* CJK Ideograph */
  ["\u84E1", "\u84E1"], /* CJK Ideograph */
  ["\u84E2", "\u84E2"], /* CJK Ideograph */
  ["\u84E3", "\u84E3"], /* CJK Ideograph */
  ["\u84E4", "\u84E4"], /* CJK Ideograph */
  ["\u84E5", "\u84E5"], /* CJK Ideograph */
  ["\u84E6", "\u84E6"], /* CJK Ideograph */
  ["\u84E7", "\u84E7"], /* CJK Ideograph */
  ["\u84E8", "\u84E8"], /* CJK Ideograph */
  ["\u84E9", "\u84E9"], /* CJK Ideograph */
  ["\u84EA", "\u84EA"], /* CJK Ideograph */
  ["\u84EB", "\u84EB"], /* CJK Ideograph */
  ["\u84EC", "\u84EC"], /* CJK Ideograph */
  ["\u84ED", "\u84ED"], /* CJK Ideograph */
  ["\u84EE", "\u84EE"], /* CJK Ideograph */
  ["\u84EF", "\u84EF"], /* CJK Ideograph */
  ["\u84F0", "\u84F0"], /* CJK Ideograph */
  ["\u84F1", "\u84F1"], /* CJK Ideograph */
  ["\u84F2", "\u84F2"], /* CJK Ideograph */
  ["\u84F3", "\u84F3"], /* CJK Ideograph */
  ["\u84F4", "\u84F4"], /* CJK Ideograph */
  ["\u84F5", "\u84F5"], /* CJK Ideograph */
  ["\u84F6", "\u84F6"], /* CJK Ideograph */
  ["\u84F7", "\u84F7"], /* CJK Ideograph */
  ["\u84F8", "\u84F8"], /* CJK Ideograph */
  ["\u84F9", "\u84F9"], /* CJK Ideograph */
  ["\u84FA", "\u84FA"], /* CJK Ideograph */
  ["\u84FB", "\u84FB"], /* CJK Ideograph */
  ["\u84FC", "\u84FC"], /* CJK Ideograph */
  ["\u84FD", "\u84FD"], /* CJK Ideograph */
  ["\u84FE", "\u84FE"], /* CJK Ideograph */
  ["\u84FF", "\u84FF"], /* CJK Ideograph */
  ["\u8500", "\u8500"], /* CJK Ideograph */
  ["\u8501", "\u8501"], /* CJK Ideograph */
  ["\u8502", "\u8502"], /* CJK Ideograph */
  ["\u8503", "\u8503"], /* CJK Ideograph */
  ["\u8504", "\u8504"], /* CJK Ideograph */
  ["\u8505", "\u8505"], /* CJK Ideograph */
  ["\u8506", "\u8506"], /* CJK Ideograph */
  ["\u8507", "\u8507"], /* CJK Ideograph */
  ["\u8508", "\u8508"], /* CJK Ideograph */
  ["\u8509", "\u8509"], /* CJK Ideograph */
  ["\u850A", "\u850A"], /* CJK Ideograph */
  ["\u850B", "\u850B"], /* CJK Ideograph */
  ["\u850C", "\u850C"], /* CJK Ideograph */
  ["\u850D", "\u850D"], /* CJK Ideograph */
  ["\u850E", "\u850E"], /* CJK Ideograph */
  ["\u850F", "\u850F"], /* CJK Ideograph */
  ["\u8510", "\u8510"], /* CJK Ideograph */
  ["\u8511", "\u8511"], /* CJK Ideograph */
  ["\u8512", "\u8512"], /* CJK Ideograph */
  ["\u8513", "\u8513"], /* CJK Ideograph */
  ["\u8514", "\u8514"], /* CJK Ideograph */
  ["\u8515", "\u8515"], /* CJK Ideograph */
  ["\u8516", "\u8516"], /* CJK Ideograph */
  ["\u8517", "\u8517"], /* CJK Ideograph */
  ["\u8518", "\u8518"], /* CJK Ideograph */
  ["\u8519", "\u8519"], /* CJK Ideograph */
  ["\u851A", "\u851A"], /* CJK Ideograph */
  ["\u851B", "\u851B"], /* CJK Ideograph */
  ["\u851C", "\u851C"], /* CJK Ideograph */
  ["\u851D", "\u851D"], /* CJK Ideograph */
  ["\u851E", "\u851E"], /* CJK Ideograph */
  ["\u851F", "\u851F"], /* CJK Ideograph */
  ["\u8520", "\u8520"], /* CJK Ideograph */
  ["\u8521", "\u8521"], /* CJK Ideograph */
  ["\u8522", "\u8522"], /* CJK Ideograph */
  ["\u8523", "\u8523"], /* CJK Ideograph */
  ["\u8524", "\u8524"], /* CJK Ideograph */
  ["\u8525", "\u8525"], /* CJK Ideograph */
  ["\u8526", "\u8526"], /* CJK Ideograph */
  ["\u8527", "\u8527"], /* CJK Ideograph */
  ["\u8528", "\u8528"], /* CJK Ideograph */
  ["\u8529", "\u8529"], /* CJK Ideograph */
  ["\u852A", "\u852A"], /* CJK Ideograph */
  ["\u852B", "\u852B"], /* CJK Ideograph */
  ["\u852C", "\u852C"], /* CJK Ideograph */
  ["\u852D", "\u852D"], /* CJK Ideograph */
  ["\u852E", "\u852E"], /* CJK Ideograph */
  ["\u852F", "\u852F"], /* CJK Ideograph */
  ["\u8530", "\u8530"], /* CJK Ideograph */
  ["\u8531", "\u8531"], /* CJK Ideograph */
  ["\u8532", "\u8532"], /* CJK Ideograph */
  ["\u8533", "\u8533"], /* CJK Ideograph */
  ["\u8534", "\u8534"], /* CJK Ideograph */
  ["\u8535", "\u8535"], /* CJK Ideograph */
  ["\u8536", "\u8536"], /* CJK Ideograph */
  ["\u8537", "\u8537"], /* CJK Ideograph */
  ["\u8538", "\u8538"], /* CJK Ideograph */
  ["\u8539", "\u8539"], /* CJK Ideograph */
  ["\u853A", "\u853A"], /* CJK Ideograph */
  ["\u853B", "\u853B"], /* CJK Ideograph */
  ["\u853C", "\u853C"], /* CJK Ideograph */
  ["\u853D", "\u853D"], /* CJK Ideograph */
  ["\u853E", "\u853E"], /* CJK Ideograph */
  ["\u853F", "\u853F"], /* CJK Ideograph */
  ["\u8540", "\u8540"], /* CJK Ideograph */
  ["\u8541", "\u8541"], /* CJK Ideograph */
  ["\u8542", "\u8542"], /* CJK Ideograph */
  ["\u8543", "\u8543"], /* CJK Ideograph */
  ["\u8544", "\u8544"], /* CJK Ideograph */
  ["\u8545", "\u8545"], /* CJK Ideograph */
  ["\u8546", "\u8546"], /* CJK Ideograph */
  ["\u8547", "\u8547"], /* CJK Ideograph */
  ["\u8548", "\u8548"], /* CJK Ideograph */
  ["\u8549", "\u8549"], /* CJK Ideograph */
  ["\u854A", "\u854A"], /* CJK Ideograph */
  ["\u854B", "\u854B"], /* CJK Ideograph */
  ["\u854C", "\u854C"], /* CJK Ideograph */
  ["\u854D", "\u854D"], /* CJK Ideograph */
  ["\u854E", "\u854E"], /* CJK Ideograph */
  ["\u854F", "\u854F"], /* CJK Ideograph */
  ["\u8550", "\u8550"], /* CJK Ideograph */
  ["\u8551", "\u8551"], /* CJK Ideograph */
  ["\u8552", "\u8552"], /* CJK Ideograph */
  ["\u8553", "\u8553"], /* CJK Ideograph */
  ["\u8554", "\u8554"], /* CJK Ideograph */
  ["\u8555", "\u8555"], /* CJK Ideograph */
  ["\u8556", "\u8556"], /* CJK Ideograph */
  ["\u8557", "\u8557"], /* CJK Ideograph */
  ["\u8558", "\u8558"], /* CJK Ideograph */
  ["\u8559", "\u8559"], /* CJK Ideograph */
  ["\u855A", "\u855A"], /* CJK Ideograph */
  ["\u855B", "\u855B"], /* CJK Ideograph */
  ["\u855C", "\u855C"], /* CJK Ideograph */
  ["\u855D", "\u855D"], /* CJK Ideograph */
  ["\u855E", "\u855E"], /* CJK Ideograph */
  ["\u855F", "\u855F"], /* CJK Ideograph */
  ["\u8560", "\u8560"], /* CJK Ideograph */
  ["\u8561", "\u8561"], /* CJK Ideograph */
  ["\u8562", "\u8562"], /* CJK Ideograph */
  ["\u8563", "\u8563"], /* CJK Ideograph */
  ["\u8564", "\u8564"], /* CJK Ideograph */
  ["\u8565", "\u8565"], /* CJK Ideograph */
  ["\u8566", "\u8566"], /* CJK Ideograph */
  ["\u8567", "\u8567"], /* CJK Ideograph */
  ["\u8568", "\u8568"], /* CJK Ideograph */
  ["\u8569", "\u8569"], /* CJK Ideograph */
  ["\u856A", "\u856A"], /* CJK Ideograph */
  ["\u856B", "\u856B"], /* CJK Ideograph */
  ["\u856C", "\u856C"], /* CJK Ideograph */
  ["\u856D", "\u856D"], /* CJK Ideograph */
  ["\u856E", "\u856E"], /* CJK Ideograph */
  ["\u856F", "\u856F"], /* CJK Ideograph */
  ["\u8570", "\u8570"], /* CJK Ideograph */
  ["\u8571", "\u8571"], /* CJK Ideograph */
  ["\u8572", "\u8572"], /* CJK Ideograph */
  ["\u8573", "\u8573"], /* CJK Ideograph */
  ["\u8574", "\u8574"], /* CJK Ideograph */
  ["\u8575", "\u8575"], /* CJK Ideograph */
  ["\u8576", "\u8576"], /* CJK Ideograph */
  ["\u8577", "\u8577"], /* CJK Ideograph */
  ["\u8578", "\u8578"], /* CJK Ideograph */
  ["\u8579", "\u8579"], /* CJK Ideograph */
  ["\u857A", "\u857A"], /* CJK Ideograph */
  ["\u857B", "\u857B"], /* CJK Ideograph */
  ["\u857C", "\u857C"], /* CJK Ideograph */
  ["\u857D", "\u857D"], /* CJK Ideograph */
  ["\u857E", "\u857E"], /* CJK Ideograph */
  ["\u857F", "\u857F"], /* CJK Ideograph */
  ["\u8580", "\u8580"], /* CJK Ideograph */
  ["\u8581", "\u8581"], /* CJK Ideograph */
  ["\u8582", "\u8582"], /* CJK Ideograph */
  ["\u8583", "\u8583"], /* CJK Ideograph */
  ["\u8584", "\u8584"], /* CJK Ideograph */
  ["\u8585", "\u8585"], /* CJK Ideograph */
  ["\u8586", "\u8586"], /* CJK Ideograph */
  ["\u8587", "\u8587"], /* CJK Ideograph */
  ["\u8588", "\u8588"], /* CJK Ideograph */
  ["\u8589", "\u8589"], /* CJK Ideograph */
  ["\u858A", "\u858A"], /* CJK Ideograph */
  ["\u858B", "\u858B"], /* CJK Ideograph */
  ["\u858C", "\u858C"], /* CJK Ideograph */
  ["\u858D", "\u858D"], /* CJK Ideograph */
  ["\u858E", "\u858E"], /* CJK Ideograph */
  ["\u858F", "\u858F"], /* CJK Ideograph */
  ["\u8590", "\u8590"], /* CJK Ideograph */
  ["\u8591", "\u8591"], /* CJK Ideograph */
  ["\u8592", "\u8592"], /* CJK Ideograph */
  ["\u8593", "\u8593"], /* CJK Ideograph */
  ["\u8594", "\u8594"], /* CJK Ideograph */
  ["\u8595", "\u8595"], /* CJK Ideograph */
  ["\u8596", "\u8596"], /* CJK Ideograph */
  ["\u8597", "\u8597"], /* CJK Ideograph */
  ["\u8598", "\u8598"], /* CJK Ideograph */
  ["\u8599", "\u8599"], /* CJK Ideograph */
  ["\u859A", "\u859A"], /* CJK Ideograph */
  ["\u859B", "\u859B"], /* CJK Ideograph */
  ["\u859C", "\u859C"], /* CJK Ideograph */
  ["\u859D", "\u859D"], /* CJK Ideograph */
  ["\u859E", "\u859E"], /* CJK Ideograph */
  ["\u859F", "\u859F"], /* CJK Ideograph */
  ["\u85A0", "\u85A0"], /* CJK Ideograph */
  ["\u85A1", "\u85A1"], /* CJK Ideograph */
  ["\u85A2", "\u85A2"], /* CJK Ideograph */
  ["\u85A3", "\u85A3"], /* CJK Ideograph */
  ["\u85A4", "\u85A4"], /* CJK Ideograph */
  ["\u85A5", "\u85A5"], /* CJK Ideograph */
  ["\u85A6", "\u85A6"], /* CJK Ideograph */
  ["\u85A7", "\u85A7"], /* CJK Ideograph */
  ["\u85A8", "\u85A8"], /* CJK Ideograph */
  ["\u85A9", "\u85A9"], /* CJK Ideograph */
  ["\u85AA", "\u85AA"], /* CJK Ideograph */
  ["\u85AB", "\u85AB"], /* CJK Ideograph */
  ["\u85AC", "\u85AC"], /* CJK Ideograph */
  ["\u85AD", "\u85AD"], /* CJK Ideograph */
  ["\u85AE", "\u85AE"], /* CJK Ideograph */
  ["\u85AF", "\u85AF"], /* CJK Ideograph */
  ["\u85B0", "\u85B0"], /* CJK Ideograph */
  ["\u85B1", "\u85B1"], /* CJK Ideograph */
  ["\u85B2", "\u85B2"], /* CJK Ideograph */
  ["\u85B3", "\u85B3"], /* CJK Ideograph */
  ["\u85B4", "\u85B4"], /* CJK Ideograph */
  ["\u85B5", "\u85B5"], /* CJK Ideograph */
  ["\u85B6", "\u85B6"], /* CJK Ideograph */
  ["\u85B7", "\u85B7"], /* CJK Ideograph */
  ["\u85B8", "\u85B8"], /* CJK Ideograph */
  ["\u85B9", "\u85B9"], /* CJK Ideograph */
  ["\u85BA", "\u85BA"], /* CJK Ideograph */
  ["\u85BB", "\u85BB"], /* CJK Ideograph */
  ["\u85BC", "\u85BC"], /* CJK Ideograph */
  ["\u85BD", "\u85BD"], /* CJK Ideograph */
  ["\u85BE", "\u85BE"], /* CJK Ideograph */
  ["\u85BF", "\u85BF"], /* CJK Ideograph */
  ["\u85C0", "\u85C0"], /* CJK Ideograph */
  ["\u85C1", "\u85C1"], /* CJK Ideograph */
  ["\u85C2", "\u85C2"], /* CJK Ideograph */
  ["\u85C3", "\u85C3"], /* CJK Ideograph */
  ["\u85C4", "\u85C4"], /* CJK Ideograph */
  ["\u85C5", "\u85C5"], /* CJK Ideograph */
  ["\u85C6", "\u85C6"], /* CJK Ideograph */
  ["\u85C7", "\u85C7"], /* CJK Ideograph */
  ["\u85C8", "\u85C8"], /* CJK Ideograph */
  ["\u85C9", "\u85C9"], /* CJK Ideograph */
  ["\u85CA", "\u85CA"], /* CJK Ideograph */
  ["\u85CB", "\u85CB"], /* CJK Ideograph */
  ["\u85CC", "\u85CC"], /* CJK Ideograph */
  ["\u85CD", "\u85CD"], /* CJK Ideograph */
  ["\u85CE", "\u85CE"], /* CJK Ideograph */
  ["\u85CF", "\u85CF"], /* CJK Ideograph */
  ["\u85D0", "\u85D0"], /* CJK Ideograph */
  ["\u85D1", "\u85D1"], /* CJK Ideograph */
  ["\u85D2", "\u85D2"], /* CJK Ideograph */
  ["\u85D3", "\u85D3"], /* CJK Ideograph */
  ["\u85D4", "\u85D4"], /* CJK Ideograph */
  ["\u85D5", "\u85D5"], /* CJK Ideograph */
  ["\u85D6", "\u85D6"], /* CJK Ideograph */
  ["\u85D7", "\u85D7"], /* CJK Ideograph */
  ["\u85D8", "\u85D8"], /* CJK Ideograph */
  ["\u85D9", "\u85D9"], /* CJK Ideograph */
  ["\u85DA", "\u85DA"], /* CJK Ideograph */
  ["\u85DB", "\u85DB"], /* CJK Ideograph */
  ["\u85DC", "\u85DC"], /* CJK Ideograph */
  ["\u85DD", "\u85DD"], /* CJK Ideograph */
  ["\u85DE", "\u85DE"], /* CJK Ideograph */
  ["\u85DF", "\u85DF"], /* CJK Ideograph */
  ["\u85E0", "\u85E0"], /* CJK Ideograph */
  ["\u85E1", "\u85E1"], /* CJK Ideograph */
  ["\u85E2", "\u85E2"], /* CJK Ideograph */
  ["\u85E3", "\u85E3"], /* CJK Ideograph */
  ["\u85E4", "\u85E4"], /* CJK Ideograph */
  ["\u85E5", "\u85E5"], /* CJK Ideograph */
  ["\u85E6", "\u85E6"], /* CJK Ideograph */
  ["\u85E7", "\u85E7"], /* CJK Ideograph */
  ["\u85E8", "\u85E8"], /* CJK Ideograph */
  ["\u85E9", "\u85E9"], /* CJK Ideograph */
  ["\u85EA", "\u85EA"], /* CJK Ideograph */
  ["\u85EB", "\u85EB"], /* CJK Ideograph */
  ["\u85EC", "\u85EC"], /* CJK Ideograph */
  ["\u85ED", "\u85ED"], /* CJK Ideograph */
  ["\u85EE", "\u85EE"], /* CJK Ideograph */
  ["\u85EF", "\u85EF"], /* CJK Ideograph */
  ["\u85F0", "\u85F0"], /* CJK Ideograph */
  ["\u85F1", "\u85F1"], /* CJK Ideograph */
  ["\u85F2", "\u85F2"], /* CJK Ideograph */
  ["\u85F3", "\u85F3"], /* CJK Ideograph */
  ["\u85F4", "\u85F4"], /* CJK Ideograph */
  ["\u85F5", "\u85F5"], /* CJK Ideograph */
  ["\u85F6", "\u85F6"], /* CJK Ideograph */
  ["\u85F7", "\u85F7"], /* CJK Ideograph */
  ["\u85F8", "\u85F8"], /* CJK Ideograph */
  ["\u85F9", "\u85F9"], /* CJK Ideograph */
  ["\u85FA", "\u85FA"], /* CJK Ideograph */
  ["\u85FB", "\u85FB"], /* CJK Ideograph */
  ["\u85FC", "\u85FC"], /* CJK Ideograph */
  ["\u85FD", "\u85FD"], /* CJK Ideograph */
  ["\u85FE", "\u85FE"], /* CJK Ideograph */
  ["\u85FF", "\u85FF"], /* CJK Ideograph */
  ["\u8600", "\u8600"], /* CJK Ideograph */
  ["\u8601", "\u8601"], /* CJK Ideograph */
  ["\u8602", "\u8602"], /* CJK Ideograph */
  ["\u8603", "\u8603"], /* CJK Ideograph */
  ["\u8604", "\u8604"], /* CJK Ideograph */
  ["\u8605", "\u8605"], /* CJK Ideograph */
  ["\u8606", "\u8606"], /* CJK Ideograph */
  ["\u8607", "\u8607"], /* CJK Ideograph */
  ["\u8608", "\u8608"], /* CJK Ideograph */
  ["\u8609", "\u8609"], /* CJK Ideograph */
  ["\u860A", "\u860A"], /* CJK Ideograph */
  ["\u860B", "\u860B"], /* CJK Ideograph */
  ["\u860C", "\u860C"], /* CJK Ideograph */
  ["\u860D", "\u860D"], /* CJK Ideograph */
  ["\u860E", "\u860E"], /* CJK Ideograph */
  ["\u860F", "\u860F"], /* CJK Ideograph */
  ["\u8610", "\u8610"], /* CJK Ideograph */
  ["\u8611", "\u8611"], /* CJK Ideograph */
  ["\u8612", "\u8612"], /* CJK Ideograph */
  ["\u8613", "\u8613"], /* CJK Ideograph */
  ["\u8614", "\u8614"], /* CJK Ideograph */
  ["\u8615", "\u8615"], /* CJK Ideograph */
  ["\u8616", "\u8616"], /* CJK Ideograph */
  ["\u8617", "\u8617"], /* CJK Ideograph */
  ["\u8618", "\u8618"], /* CJK Ideograph */
  ["\u8619", "\u8619"], /* CJK Ideograph */
  ["\u861A", "\u861A"], /* CJK Ideograph */
  ["\u861B", "\u861B"], /* CJK Ideograph */
  ["\u861C", "\u861C"], /* CJK Ideograph */
  ["\u861D", "\u861D"], /* CJK Ideograph */
  ["\u861E", "\u861E"], /* CJK Ideograph */
  ["\u861F", "\u861F"], /* CJK Ideograph */
  ["\u8620", "\u8620"], /* CJK Ideograph */
  ["\u8621", "\u8621"], /* CJK Ideograph */
  ["\u8622", "\u8622"], /* CJK Ideograph */
  ["\u8623", "\u8623"], /* CJK Ideograph */
  ["\u8624", "\u8624"], /* CJK Ideograph */
  ["\u8625", "\u8625"], /* CJK Ideograph */
  ["\u8626", "\u8626"], /* CJK Ideograph */
  ["\u8627", "\u8627"], /* CJK Ideograph */
  ["\u8628", "\u8628"], /* CJK Ideograph */
  ["\u8629", "\u8629"], /* CJK Ideograph */
  ["\u862A", "\u862A"], /* CJK Ideograph */
  ["\u862B", "\u862B"], /* CJK Ideograph */
  ["\u862C", "\u862C"], /* CJK Ideograph */
  ["\u862D", "\u862D"], /* CJK Ideograph */
  ["\u862E", "\u862E"], /* CJK Ideograph */
  ["\u862F", "\u862F"], /* CJK Ideograph */
  ["\u8630", "\u8630"], /* CJK Ideograph */
  ["\u8631", "\u8631"], /* CJK Ideograph */
  ["\u8632", "\u8632"], /* CJK Ideograph */
  ["\u8633", "\u8633"], /* CJK Ideograph */
  ["\u8634", "\u8634"], /* CJK Ideograph */
  ["\u8635", "\u8635"], /* CJK Ideograph */
  ["\u8636", "\u8636"], /* CJK Ideograph */
  ["\u8637", "\u8637"], /* CJK Ideograph */
  ["\u8638", "\u8638"], /* CJK Ideograph */
  ["\u8639", "\u8639"], /* CJK Ideograph */
  ["\u863A", "\u863A"], /* CJK Ideograph */
  ["\u863B", "\u863B"], /* CJK Ideograph */
  ["\u863C", "\u863C"], /* CJK Ideograph */
  ["\u863D", "\u863D"], /* CJK Ideograph */
  ["\u863E", "\u863E"], /* CJK Ideograph */
  ["\u863F", "\u863F"], /* CJK Ideograph */
  ["\u8640", "\u8640"], /* CJK Ideograph */
  ["\u8641", "\u8641"], /* CJK Ideograph */
  ["\u8642", "\u8642"], /* CJK Ideograph */
  ["\u8643", "\u8643"], /* CJK Ideograph */
  ["\u8644", "\u8644"], /* CJK Ideograph */
  ["\u8645", "\u8645"], /* CJK Ideograph */
  ["\u8646", "\u8646"], /* CJK Ideograph */
  ["\u8647", "\u8647"], /* CJK Ideograph */
  ["\u8648", "\u8648"], /* CJK Ideograph */
  ["\u8649", "\u8649"], /* CJK Ideograph */
  ["\u864A", "\u864A"], /* CJK Ideograph */
  ["\u864B", "\u864B"], /* CJK Ideograph */
  ["\u864C", "\u864C"], /* CJK Ideograph */
  ["\u864D", "\u864D"], /* CJK Ideograph */
  ["\u864E", "\u864E"], /* CJK Ideograph */
  ["\u864F", "\u864F"], /* CJK Ideograph */
  ["\u8650", "\u8650"], /* CJK Ideograph */
  ["\u8651", "\u8651"], /* CJK Ideograph */
  ["\u8652", "\u8652"], /* CJK Ideograph */
  ["\u8653", "\u8653"], /* CJK Ideograph */
  ["\u8654", "\u8654"], /* CJK Ideograph */
  ["\u8655", "\u8655"], /* CJK Ideograph */
  ["\u8656", "\u8656"], /* CJK Ideograph */
  ["\u8657", "\u8657"], /* CJK Ideograph */
  ["\u8658", "\u8658"], /* CJK Ideograph */
  ["\u8659", "\u8659"], /* CJK Ideograph */
  ["\u865A", "\u865A"], /* CJK Ideograph */
  ["\u865B", "\u865B"], /* CJK Ideograph */
  ["\u865C", "\u865C"], /* CJK Ideograph */
  ["\u865D", "\u865D"], /* CJK Ideograph */
  ["\u865E", "\u865E"], /* CJK Ideograph */
  ["\u865F", "\u865F"], /* CJK Ideograph */
  ["\u8660", "\u8660"], /* CJK Ideograph */
  ["\u8661", "\u8661"], /* CJK Ideograph */
  ["\u8662", "\u8662"], /* CJK Ideograph */
  ["\u8663", "\u8663"], /* CJK Ideograph */
  ["\u8664", "\u8664"], /* CJK Ideograph */
  ["\u8665", "\u8665"], /* CJK Ideograph */
  ["\u8666", "\u8666"], /* CJK Ideograph */
  ["\u8667", "\u8667"], /* CJK Ideograph */
  ["\u8668", "\u8668"], /* CJK Ideograph */
  ["\u8669", "\u8669"], /* CJK Ideograph */
  ["\u866A", "\u866A"], /* CJK Ideograph */
  ["\u866B", "\u866B"], /* CJK Ideograph */
  ["\u866C", "\u866C"], /* CJK Ideograph */
  ["\u866D", "\u866D"], /* CJK Ideograph */
  ["\u866E", "\u866E"], /* CJK Ideograph */
  ["\u866F", "\u866F"], /* CJK Ideograph */
  ["\u8670", "\u8670"], /* CJK Ideograph */
  ["\u8671", "\u8671"], /* CJK Ideograph */
  ["\u8672", "\u8672"], /* CJK Ideograph */
  ["\u8673", "\u8673"], /* CJK Ideograph */
  ["\u8674", "\u8674"], /* CJK Ideograph */
  ["\u8675", "\u8675"], /* CJK Ideograph */
  ["\u8676", "\u8676"], /* CJK Ideograph */
  ["\u8677", "\u8677"], /* CJK Ideograph */
  ["\u8678", "\u8678"], /* CJK Ideograph */
  ["\u8679", "\u8679"], /* CJK Ideograph */
  ["\u867A", "\u867A"], /* CJK Ideograph */
  ["\u867B", "\u867B"], /* CJK Ideograph */
  ["\u867C", "\u867C"], /* CJK Ideograph */
  ["\u867D", "\u867D"], /* CJK Ideograph */
  ["\u867E", "\u867E"], /* CJK Ideograph */
  ["\u867F", "\u867F"], /* CJK Ideograph */
  ["\u8680", "\u8680"], /* CJK Ideograph */
  ["\u8681", "\u8681"], /* CJK Ideograph */
  ["\u8682", "\u8682"], /* CJK Ideograph */
  ["\u8683", "\u8683"], /* CJK Ideograph */
  ["\u8684", "\u8684"], /* CJK Ideograph */
  ["\u8685", "\u8685"], /* CJK Ideograph */
  ["\u8686", "\u8686"], /* CJK Ideograph */
  ["\u8687", "\u8687"], /* CJK Ideograph */
  ["\u8688", "\u8688"], /* CJK Ideograph */
  ["\u8689", "\u8689"], /* CJK Ideograph */
  ["\u868A", "\u868A"], /* CJK Ideograph */
  ["\u868B", "\u868B"], /* CJK Ideograph */
  ["\u868C", "\u868C"], /* CJK Ideograph */
  ["\u868D", "\u868D"], /* CJK Ideograph */
  ["\u868E", "\u868E"], /* CJK Ideograph */
  ["\u868F", "\u868F"], /* CJK Ideograph */
  ["\u8690", "\u8690"], /* CJK Ideograph */
  ["\u8691", "\u8691"], /* CJK Ideograph */
  ["\u8692", "\u8692"], /* CJK Ideograph */
  ["\u8693", "\u8693"], /* CJK Ideograph */
  ["\u8694", "\u8694"], /* CJK Ideograph */
  ["\u8695", "\u8695"], /* CJK Ideograph */
  ["\u8696", "\u8696"], /* CJK Ideograph */
  ["\u8697", "\u8697"], /* CJK Ideograph */
  ["\u8698", "\u8698"], /* CJK Ideograph */
  ["\u8699", "\u8699"], /* CJK Ideograph */
  ["\u869A", "\u869A"], /* CJK Ideograph */
  ["\u869B", "\u869B"], /* CJK Ideograph */
  ["\u869C", "\u869C"], /* CJK Ideograph */
  ["\u869D", "\u869D"], /* CJK Ideograph */
  ["\u869E", "\u869E"], /* CJK Ideograph */
  ["\u869F", "\u869F"], /* CJK Ideograph */
  ["\u86A0", "\u86A0"], /* CJK Ideograph */
  ["\u86A1", "\u86A1"], /* CJK Ideograph */
  ["\u86A2", "\u86A2"], /* CJK Ideograph */
  ["\u86A3", "\u86A3"], /* CJK Ideograph */
  ["\u86A4", "\u86A4"], /* CJK Ideograph */
  ["\u86A5", "\u86A5"], /* CJK Ideograph */
  ["\u86A6", "\u86A6"], /* CJK Ideograph */
  ["\u86A7", "\u86A7"], /* CJK Ideograph */
  ["\u86A8", "\u86A8"], /* CJK Ideograph */
  ["\u86A9", "\u86A9"], /* CJK Ideograph */
  ["\u86AA", "\u86AA"], /* CJK Ideograph */
  ["\u86AB", "\u86AB"], /* CJK Ideograph */
  ["\u86AC", "\u86AC"], /* CJK Ideograph */
  ["\u86AD", "\u86AD"], /* CJK Ideograph */
  ["\u86AE", "\u86AE"], /* CJK Ideograph */
  ["\u86AF", "\u86AF"], /* CJK Ideograph */
  ["\u86B0", "\u86B0"], /* CJK Ideograph */
  ["\u86B1", "\u86B1"], /* CJK Ideograph */
  ["\u86B2", "\u86B2"], /* CJK Ideograph */
  ["\u86B3", "\u86B3"], /* CJK Ideograph */
  ["\u86B4", "\u86B4"], /* CJK Ideograph */
  ["\u86B5", "\u86B5"], /* CJK Ideograph */
  ["\u86B6", "\u86B6"], /* CJK Ideograph */
  ["\u86B7", "\u86B7"], /* CJK Ideograph */
  ["\u86B8", "\u86B8"], /* CJK Ideograph */
  ["\u86B9", "\u86B9"], /* CJK Ideograph */
  ["\u86BA", "\u86BA"], /* CJK Ideograph */
  ["\u86BB", "\u86BB"], /* CJK Ideograph */
  ["\u86BC", "\u86BC"], /* CJK Ideograph */
  ["\u86BD", "\u86BD"], /* CJK Ideograph */
  ["\u86BE", "\u86BE"], /* CJK Ideograph */
  ["\u86BF", "\u86BF"], /* CJK Ideograph */
  ["\u86C0", "\u86C0"], /* CJK Ideograph */
  ["\u86C1", "\u86C1"], /* CJK Ideograph */
  ["\u86C2", "\u86C2"], /* CJK Ideograph */
  ["\u86C3", "\u86C3"], /* CJK Ideograph */
  ["\u86C4", "\u86C4"], /* CJK Ideograph */
  ["\u86C5", "\u86C5"], /* CJK Ideograph */
  ["\u86C6", "\u86C6"], /* CJK Ideograph */
  ["\u86C7", "\u86C7"], /* CJK Ideograph */
  ["\u86C8", "\u86C8"], /* CJK Ideograph */
  ["\u86C9", "\u86C9"], /* CJK Ideograph */
  ["\u86CA", "\u86CA"], /* CJK Ideograph */
  ["\u86CB", "\u86CB"], /* CJK Ideograph */
  ["\u86CC", "\u86CC"], /* CJK Ideograph */
  ["\u86CD", "\u86CD"], /* CJK Ideograph */
  ["\u86CE", "\u86CE"], /* CJK Ideograph */
  ["\u86CF", "\u86CF"], /* CJK Ideograph */
  ["\u86D0", "\u86D0"], /* CJK Ideograph */
  ["\u86D1", "\u86D1"], /* CJK Ideograph */
  ["\u86D2", "\u86D2"], /* CJK Ideograph */
  ["\u86D3", "\u86D3"], /* CJK Ideograph */
  ["\u86D4", "\u86D4"], /* CJK Ideograph */
  ["\u86D5", "\u86D5"], /* CJK Ideograph */
  ["\u86D6", "\u86D6"], /* CJK Ideograph */
  ["\u86D7", "\u86D7"], /* CJK Ideograph */
  ["\u86D8", "\u86D8"], /* CJK Ideograph */
  ["\u86D9", "\u86D9"], /* CJK Ideograph */
  ["\u86DA", "\u86DA"], /* CJK Ideograph */
  ["\u86DB", "\u86DB"], /* CJK Ideograph */
  ["\u86DC", "\u86DC"], /* CJK Ideograph */
  ["\u86DD", "\u86DD"], /* CJK Ideograph */
  ["\u86DE", "\u86DE"], /* CJK Ideograph */
  ["\u86DF", "\u86DF"], /* CJK Ideograph */
  ["\u86E0", "\u86E0"], /* CJK Ideograph */
  ["\u86E1", "\u86E1"], /* CJK Ideograph */
  ["\u86E2", "\u86E2"], /* CJK Ideograph */
  ["\u86E3", "\u86E3"], /* CJK Ideograph */
  ["\u86E4", "\u86E4"], /* CJK Ideograph */
  ["\u86E5", "\u86E5"], /* CJK Ideograph */
  ["\u86E6", "\u86E6"], /* CJK Ideograph */
  ["\u86E7", "\u86E7"], /* CJK Ideograph */
  ["\u86E8", "\u86E8"], /* CJK Ideograph */
  ["\u86E9", "\u86E9"], /* CJK Ideograph */
  ["\u86EA", "\u86EA"], /* CJK Ideograph */
  ["\u86EB", "\u86EB"], /* CJK Ideograph */
  ["\u86EC", "\u86EC"], /* CJK Ideograph */
  ["\u86ED", "\u86ED"], /* CJK Ideograph */
  ["\u86EE", "\u86EE"], /* CJK Ideograph */
  ["\u86EF", "\u86EF"], /* CJK Ideograph */
  ["\u86F0", "\u86F0"], /* CJK Ideograph */
  ["\u86F1", "\u86F1"], /* CJK Ideograph */
  ["\u86F2", "\u86F2"], /* CJK Ideograph */
  ["\u86F3", "\u86F3"], /* CJK Ideograph */
  ["\u86F4", "\u86F4"], /* CJK Ideograph */
  ["\u86F5", "\u86F5"], /* CJK Ideograph */
  ["\u86F6", "\u86F6"], /* CJK Ideograph */
  ["\u86F7", "\u86F7"], /* CJK Ideograph */
  ["\u86F8", "\u86F8"], /* CJK Ideograph */
  ["\u86F9", "\u86F9"], /* CJK Ideograph */
  ["\u86FA", "\u86FA"], /* CJK Ideograph */
  ["\u86FB", "\u86FB"], /* CJK Ideograph */
  ["\u86FC", "\u86FC"], /* CJK Ideograph */
  ["\u86FD", "\u86FD"], /* CJK Ideograph */
  ["\u86FE", "\u86FE"], /* CJK Ideograph */
  ["\u86FF", "\u86FF"], /* CJK Ideograph */
  ["\u8700", "\u8700"], /* CJK Ideograph */
  ["\u8701", "\u8701"], /* CJK Ideograph */
  ["\u8702", "\u8702"], /* CJK Ideograph */
  ["\u8703", "\u8703"], /* CJK Ideograph */
  ["\u8704", "\u8704"], /* CJK Ideograph */
  ["\u8705", "\u8705"], /* CJK Ideograph */
  ["\u8706", "\u8706"], /* CJK Ideograph */
  ["\u8707", "\u8707"], /* CJK Ideograph */
  ["\u8708", "\u8708"], /* CJK Ideograph */
  ["\u8709", "\u8709"], /* CJK Ideograph */
  ["\u870A", "\u870A"], /* CJK Ideograph */
  ["\u870B", "\u870B"], /* CJK Ideograph */
  ["\u870C", "\u870C"], /* CJK Ideograph */
  ["\u870D", "\u870D"], /* CJK Ideograph */
  ["\u870E", "\u870E"], /* CJK Ideograph */
  ["\u870F", "\u870F"], /* CJK Ideograph */
  ["\u8710", "\u8710"], /* CJK Ideograph */
  ["\u8711", "\u8711"], /* CJK Ideograph */
  ["\u8712", "\u8712"], /* CJK Ideograph */
  ["\u8713", "\u8713"], /* CJK Ideograph */
  ["\u8714", "\u8714"], /* CJK Ideograph */
  ["\u8715", "\u8715"], /* CJK Ideograph */
  ["\u8716", "\u8716"], /* CJK Ideograph */
  ["\u8717", "\u8717"], /* CJK Ideograph */
  ["\u8718", "\u8718"], /* CJK Ideograph */
  ["\u8719", "\u8719"], /* CJK Ideograph */
  ["\u871A", "\u871A"], /* CJK Ideograph */
  ["\u871B", "\u871B"], /* CJK Ideograph */
  ["\u871C", "\u871C"], /* CJK Ideograph */
  ["\u871D", "\u871D"], /* CJK Ideograph */
  ["\u871E", "\u871E"], /* CJK Ideograph */
  ["\u871F", "\u871F"], /* CJK Ideograph */
  ["\u8720", "\u8720"], /* CJK Ideograph */
  ["\u8721", "\u8721"], /* CJK Ideograph */
  ["\u8722", "\u8722"], /* CJK Ideograph */
  ["\u8723", "\u8723"], /* CJK Ideograph */
  ["\u8724", "\u8724"], /* CJK Ideograph */
  ["\u8725", "\u8725"], /* CJK Ideograph */
  ["\u8726", "\u8726"], /* CJK Ideograph */
  ["\u8727", "\u8727"], /* CJK Ideograph */
  ["\u8728", "\u8728"], /* CJK Ideograph */
  ["\u8729", "\u8729"], /* CJK Ideograph */
  ["\u872A", "\u872A"], /* CJK Ideograph */
  ["\u872B", "\u872B"], /* CJK Ideograph */
  ["\u872C", "\u872C"], /* CJK Ideograph */
  ["\u872D", "\u872D"], /* CJK Ideograph */
  ["\u872E", "\u872E"], /* CJK Ideograph */
  ["\u872F", "\u872F"], /* CJK Ideograph */
  ["\u8730", "\u8730"], /* CJK Ideograph */
  ["\u8731", "\u8731"], /* CJK Ideograph */
  ["\u8732", "\u8732"], /* CJK Ideograph */
  ["\u8733", "\u8733"], /* CJK Ideograph */
  ["\u8734", "\u8734"], /* CJK Ideograph */
  ["\u8735", "\u8735"], /* CJK Ideograph */
  ["\u8736", "\u8736"], /* CJK Ideograph */
  ["\u8737", "\u8737"], /* CJK Ideograph */
  ["\u8738", "\u8738"], /* CJK Ideograph */
  ["\u8739", "\u8739"], /* CJK Ideograph */
  ["\u873A", "\u873A"], /* CJK Ideograph */
  ["\u873B", "\u873B"], /* CJK Ideograph */
  ["\u873C", "\u873C"], /* CJK Ideograph */
  ["\u873D", "\u873D"], /* CJK Ideograph */
  ["\u873E", "\u873E"], /* CJK Ideograph */
  ["\u873F", "\u873F"], /* CJK Ideograph */
  ["\u8740", "\u8740"], /* CJK Ideograph */
  ["\u8741", "\u8741"], /* CJK Ideograph */
  ["\u8742", "\u8742"], /* CJK Ideograph */
  ["\u8743", "\u8743"], /* CJK Ideograph */
  ["\u8744", "\u8744"], /* CJK Ideograph */
  ["\u8745", "\u8745"], /* CJK Ideograph */
  ["\u8746", "\u8746"], /* CJK Ideograph */
  ["\u8747", "\u8747"], /* CJK Ideograph */
  ["\u8748", "\u8748"], /* CJK Ideograph */
  ["\u8749", "\u8749"], /* CJK Ideograph */
  ["\u874A", "\u874A"], /* CJK Ideograph */
  ["\u874B", "\u874B"], /* CJK Ideograph */
  ["\u874C", "\u874C"], /* CJK Ideograph */
  ["\u874D", "\u874D"], /* CJK Ideograph */
  ["\u874E", "\u874E"], /* CJK Ideograph */
  ["\u874F", "\u874F"], /* CJK Ideograph */
  ["\u8750", "\u8750"], /* CJK Ideograph */
  ["\u8751", "\u8751"], /* CJK Ideograph */
  ["\u8752", "\u8752"], /* CJK Ideograph */
  ["\u8753", "\u8753"], /* CJK Ideograph */
  ["\u8754", "\u8754"], /* CJK Ideograph */
  ["\u8755", "\u8755"], /* CJK Ideograph */
  ["\u8756", "\u8756"], /* CJK Ideograph */
  ["\u8757", "\u8757"], /* CJK Ideograph */
  ["\u8758", "\u8758"], /* CJK Ideograph */
  ["\u8759", "\u8759"], /* CJK Ideograph */
  ["\u875A", "\u875A"], /* CJK Ideograph */
  ["\u875B", "\u875B"], /* CJK Ideograph */
  ["\u875C", "\u875C"], /* CJK Ideograph */
  ["\u875D", "\u875D"], /* CJK Ideograph */
  ["\u875E", "\u875E"], /* CJK Ideograph */
  ["\u875F", "\u875F"], /* CJK Ideograph */
  ["\u8760", "\u8760"], /* CJK Ideograph */
  ["\u8761", "\u8761"], /* CJK Ideograph */
  ["\u8762", "\u8762"], /* CJK Ideograph */
  ["\u8763", "\u8763"], /* CJK Ideograph */
  ["\u8764", "\u8764"], /* CJK Ideograph */
  ["\u8765", "\u8765"], /* CJK Ideograph */
  ["\u8766", "\u8766"], /* CJK Ideograph */
  ["\u8767", "\u8767"], /* CJK Ideograph */
  ["\u8768", "\u8768"], /* CJK Ideograph */
  ["\u8769", "\u8769"], /* CJK Ideograph */
  ["\u876A", "\u876A"], /* CJK Ideograph */
  ["\u876B", "\u876B"], /* CJK Ideograph */
  ["\u876C", "\u876C"], /* CJK Ideograph */
  ["\u876D", "\u876D"], /* CJK Ideograph */
  ["\u876E", "\u876E"], /* CJK Ideograph */
  ["\u876F", "\u876F"], /* CJK Ideograph */
  ["\u8770", "\u8770"], /* CJK Ideograph */
  ["\u8771", "\u8771"], /* CJK Ideograph */
  ["\u8772", "\u8772"], /* CJK Ideograph */
  ["\u8773", "\u8773"], /* CJK Ideograph */
  ["\u8774", "\u8774"], /* CJK Ideograph */
  ["\u8775", "\u8775"], /* CJK Ideograph */
  ["\u8776", "\u8776"], /* CJK Ideograph */
  ["\u8777", "\u8777"], /* CJK Ideograph */
  ["\u8778", "\u8778"], /* CJK Ideograph */
  ["\u8779", "\u8779"], /* CJK Ideograph */
  ["\u877A", "\u877A"], /* CJK Ideograph */
  ["\u877B", "\u877B"], /* CJK Ideograph */
  ["\u877C", "\u877C"], /* CJK Ideograph */
  ["\u877D", "\u877D"], /* CJK Ideograph */
  ["\u877E", "\u877E"], /* CJK Ideograph */
  ["\u877F", "\u877F"], /* CJK Ideograph */
  ["\u8780", "\u8780"], /* CJK Ideograph */
  ["\u8781", "\u8781"], /* CJK Ideograph */
  ["\u8782", "\u8782"], /* CJK Ideograph */
  ["\u8783", "\u8783"], /* CJK Ideograph */
  ["\u8784", "\u8784"], /* CJK Ideograph */
  ["\u8785", "\u8785"], /* CJK Ideograph */
  ["\u8786", "\u8786"], /* CJK Ideograph */
  ["\u8787", "\u8787"], /* CJK Ideograph */
  ["\u8788", "\u8788"], /* CJK Ideograph */
  ["\u8789", "\u8789"], /* CJK Ideograph */
  ["\u878A", "\u878A"], /* CJK Ideograph */
  ["\u878B", "\u878B"], /* CJK Ideograph */
  ["\u878C", "\u878C"], /* CJK Ideograph */
  ["\u878D", "\u878D"], /* CJK Ideograph */
  ["\u878E", "\u878E"], /* CJK Ideograph */
  ["\u878F", "\u878F"], /* CJK Ideograph */
  ["\u8790", "\u8790"], /* CJK Ideograph */
  ["\u8791", "\u8791"], /* CJK Ideograph */
  ["\u8792", "\u8792"], /* CJK Ideograph */
  ["\u8793", "\u8793"], /* CJK Ideograph */
  ["\u8794", "\u8794"], /* CJK Ideograph */
  ["\u8795", "\u8795"], /* CJK Ideograph */
  ["\u8796", "\u8796"], /* CJK Ideograph */
  ["\u8797", "\u8797"], /* CJK Ideograph */
  ["\u8798", "\u8798"], /* CJK Ideograph */
  ["\u8799", "\u8799"], /* CJK Ideograph */
  ["\u879A", "\u879A"], /* CJK Ideograph */
  ["\u879B", "\u879B"], /* CJK Ideograph */
  ["\u879C", "\u879C"], /* CJK Ideograph */
  ["\u879D", "\u879D"], /* CJK Ideograph */
  ["\u879E", "\u879E"], /* CJK Ideograph */
  ["\u879F", "\u879F"], /* CJK Ideograph */
  ["\u87A0", "\u87A0"], /* CJK Ideograph */
  ["\u87A1", "\u87A1"], /* CJK Ideograph */
  ["\u87A2", "\u87A2"], /* CJK Ideograph */
  ["\u87A3", "\u87A3"], /* CJK Ideograph */
  ["\u87A4", "\u87A4"], /* CJK Ideograph */
  ["\u87A5", "\u87A5"], /* CJK Ideograph */
  ["\u87A6", "\u87A6"], /* CJK Ideograph */
  ["\u87A7", "\u87A7"], /* CJK Ideograph */
  ["\u87A8", "\u87A8"], /* CJK Ideograph */
  ["\u87A9", "\u87A9"], /* CJK Ideograph */
  ["\u87AA", "\u87AA"], /* CJK Ideograph */
  ["\u87AB", "\u87AB"], /* CJK Ideograph */
  ["\u87AC", "\u87AC"], /* CJK Ideograph */
  ["\u87AD", "\u87AD"], /* CJK Ideograph */
  ["\u87AE", "\u87AE"], /* CJK Ideograph */
  ["\u87AF", "\u87AF"], /* CJK Ideograph */
  ["\u87B0", "\u87B0"], /* CJK Ideograph */
  ["\u87B1", "\u87B1"], /* CJK Ideograph */
  ["\u87B2", "\u87B2"], /* CJK Ideograph */
  ["\u87B3", "\u87B3"], /* CJK Ideograph */
  ["\u87B4", "\u87B4"], /* CJK Ideograph */
  ["\u87B5", "\u87B5"], /* CJK Ideograph */
  ["\u87B6", "\u87B6"], /* CJK Ideograph */
  ["\u87B7", "\u87B7"], /* CJK Ideograph */
  ["\u87B8", "\u87B8"], /* CJK Ideograph */
  ["\u87B9", "\u87B9"], /* CJK Ideograph */
  ["\u87BA", "\u87BA"], /* CJK Ideograph */
  ["\u87BB", "\u87BB"], /* CJK Ideograph */
  ["\u87BC", "\u87BC"], /* CJK Ideograph */
  ["\u87BD", "\u87BD"], /* CJK Ideograph */
  ["\u87BE", "\u87BE"], /* CJK Ideograph */
  ["\u87BF", "\u87BF"], /* CJK Ideograph */
  ["\u87C0", "\u87C0"], /* CJK Ideograph */
  ["\u87C1", "\u87C1"], /* CJK Ideograph */
  ["\u87C2", "\u87C2"], /* CJK Ideograph */
  ["\u87C3", "\u87C3"], /* CJK Ideograph */
  ["\u87C4", "\u87C4"], /* CJK Ideograph */
  ["\u87C5", "\u87C5"], /* CJK Ideograph */
  ["\u87C6", "\u87C6"], /* CJK Ideograph */
  ["\u87C7", "\u87C7"], /* CJK Ideograph */
  ["\u87C8", "\u87C8"], /* CJK Ideograph */
  ["\u87C9", "\u87C9"], /* CJK Ideograph */
  ["\u87CA", "\u87CA"], /* CJK Ideograph */
  ["\u87CB", "\u87CB"], /* CJK Ideograph */
  ["\u87CC", "\u87CC"], /* CJK Ideograph */
  ["\u87CD", "\u87CD"], /* CJK Ideograph */
  ["\u87CE", "\u87CE"], /* CJK Ideograph */
  ["\u87CF", "\u87CF"], /* CJK Ideograph */
  ["\u87D0", "\u87D0"], /* CJK Ideograph */
  ["\u87D1", "\u87D1"], /* CJK Ideograph */
  ["\u87D2", "\u87D2"], /* CJK Ideograph */
  ["\u87D3", "\u87D3"], /* CJK Ideograph */
  ["\u87D4", "\u87D4"], /* CJK Ideograph */
  ["\u87D5", "\u87D5"], /* CJK Ideograph */
  ["\u87D6", "\u87D6"], /* CJK Ideograph */
  ["\u87D7", "\u87D7"], /* CJK Ideograph */
  ["\u87D8", "\u87D8"], /* CJK Ideograph */
  ["\u87D9", "\u87D9"], /* CJK Ideograph */
  ["\u87DA", "\u87DA"], /* CJK Ideograph */
  ["\u87DB", "\u87DB"], /* CJK Ideograph */
  ["\u87DC", "\u87DC"], /* CJK Ideograph */
  ["\u87DD", "\u87DD"], /* CJK Ideograph */
  ["\u87DE", "\u87DE"], /* CJK Ideograph */
  ["\u87DF", "\u87DF"], /* CJK Ideograph */
  ["\u87E0", "\u87E0"], /* CJK Ideograph */
  ["\u87E1", "\u87E1"], /* CJK Ideograph */
  ["\u87E2", "\u87E2"], /* CJK Ideograph */
  ["\u87E3", "\u87E3"], /* CJK Ideograph */
  ["\u87E4", "\u87E4"], /* CJK Ideograph */
  ["\u87E5", "\u87E5"], /* CJK Ideograph */
  ["\u87E6", "\u87E6"], /* CJK Ideograph */
  ["\u87E7", "\u87E7"], /* CJK Ideograph */
  ["\u87E8", "\u87E8"], /* CJK Ideograph */
  ["\u87E9", "\u87E9"], /* CJK Ideograph */
  ["\u87EA", "\u87EA"], /* CJK Ideograph */
  ["\u87EB", "\u87EB"], /* CJK Ideograph */
  ["\u87EC", "\u87EC"], /* CJK Ideograph */
  ["\u87ED", "\u87ED"], /* CJK Ideograph */
  ["\u87EE", "\u87EE"], /* CJK Ideograph */
  ["\u87EF", "\u87EF"], /* CJK Ideograph */
  ["\u87F0", "\u87F0"], /* CJK Ideograph */
  ["\u87F1", "\u87F1"], /* CJK Ideograph */
  ["\u87F2", "\u87F2"], /* CJK Ideograph */
  ["\u87F3", "\u87F3"], /* CJK Ideograph */
  ["\u87F4", "\u87F4"], /* CJK Ideograph */
  ["\u87F5", "\u87F5"], /* CJK Ideograph */
  ["\u87F6", "\u87F6"], /* CJK Ideograph */
  ["\u87F7", "\u87F7"], /* CJK Ideograph */
  ["\u87F8", "\u87F8"], /* CJK Ideograph */
  ["\u87F9", "\u87F9"], /* CJK Ideograph */
  ["\u87FA", "\u87FA"], /* CJK Ideograph */
  ["\u87FB", "\u87FB"], /* CJK Ideograph */
  ["\u87FC", "\u87FC"], /* CJK Ideograph */
  ["\u87FD", "\u87FD"], /* CJK Ideograph */
  ["\u87FE", "\u87FE"], /* CJK Ideograph */
  ["\u87FF", "\u87FF"], /* CJK Ideograph */
  ["\u8800", "\u8800"], /* CJK Ideograph */
  ["\u8801", "\u8801"], /* CJK Ideograph */
  ["\u8802", "\u8802"], /* CJK Ideograph */
  ["\u8803", "\u8803"], /* CJK Ideograph */
  ["\u8804", "\u8804"], /* CJK Ideograph */
  ["\u8805", "\u8805"], /* CJK Ideograph */
  ["\u8806", "\u8806"], /* CJK Ideograph */
  ["\u8807", "\u8807"], /* CJK Ideograph */
  ["\u8808", "\u8808"], /* CJK Ideograph */
  ["\u8809", "\u8809"], /* CJK Ideograph */
  ["\u880A", "\u880A"], /* CJK Ideograph */
  ["\u880B", "\u880B"], /* CJK Ideograph */
  ["\u880C", "\u880C"], /* CJK Ideograph */
  ["\u880D", "\u880D"], /* CJK Ideograph */
  ["\u880E", "\u880E"], /* CJK Ideograph */
  ["\u880F", "\u880F"], /* CJK Ideograph */
  ["\u8810", "\u8810"], /* CJK Ideograph */
  ["\u8811", "\u8811"], /* CJK Ideograph */
  ["\u8812", "\u8812"], /* CJK Ideograph */
  ["\u8813", "\u8813"], /* CJK Ideograph */
  ["\u8814", "\u8814"], /* CJK Ideograph */
  ["\u8815", "\u8815"], /* CJK Ideograph */
  ["\u8816", "\u8816"], /* CJK Ideograph */
  ["\u8817", "\u8817"], /* CJK Ideograph */
  ["\u8818", "\u8818"], /* CJK Ideograph */
  ["\u8819", "\u8819"], /* CJK Ideograph */
  ["\u881A", "\u881A"], /* CJK Ideograph */
  ["\u881B", "\u881B"], /* CJK Ideograph */
  ["\u881C", "\u881C"], /* CJK Ideograph */
  ["\u881D", "\u881D"], /* CJK Ideograph */
  ["\u881E", "\u881E"], /* CJK Ideograph */
  ["\u881F", "\u881F"], /* CJK Ideograph */
  ["\u8820", "\u8820"], /* CJK Ideograph */
  ["\u8821", "\u8821"], /* CJK Ideograph */
  ["\u8822", "\u8822"], /* CJK Ideograph */
  ["\u8823", "\u8823"], /* CJK Ideograph */
  ["\u8824", "\u8824"], /* CJK Ideograph */
  ["\u8825", "\u8825"], /* CJK Ideograph */
  ["\u8826", "\u8826"], /* CJK Ideograph */
  ["\u8827", "\u8827"], /* CJK Ideograph */
  ["\u8828", "\u8828"], /* CJK Ideograph */
  ["\u8829", "\u8829"], /* CJK Ideograph */
  ["\u882A", "\u882A"], /* CJK Ideograph */
  ["\u882B", "\u882B"], /* CJK Ideograph */
  ["\u882C", "\u882C"], /* CJK Ideograph */
  ["\u882D", "\u882D"], /* CJK Ideograph */
  ["\u882E", "\u882E"], /* CJK Ideograph */
  ["\u882F", "\u882F"], /* CJK Ideograph */
  ["\u8830", "\u8830"], /* CJK Ideograph */
  ["\u8831", "\u8831"], /* CJK Ideograph */
  ["\u8832", "\u8832"], /* CJK Ideograph */
  ["\u8833", "\u8833"], /* CJK Ideograph */
  ["\u8834", "\u8834"], /* CJK Ideograph */
  ["\u8835", "\u8835"], /* CJK Ideograph */
  ["\u8836", "\u8836"], /* CJK Ideograph */
  ["\u8837", "\u8837"], /* CJK Ideograph */
  ["\u8838", "\u8838"], /* CJK Ideograph */
  ["\u8839", "\u8839"], /* CJK Ideograph */
  ["\u883A", "\u883A"], /* CJK Ideograph */
  ["\u883B", "\u883B"], /* CJK Ideograph */
  ["\u883C", "\u883C"], /* CJK Ideograph */
  ["\u883D", "\u883D"], /* CJK Ideograph */
  ["\u883E", "\u883E"], /* CJK Ideograph */
  ["\u883F", "\u883F"], /* CJK Ideograph */
  ["\u8840", "\u8840"], /* CJK Ideograph */
  ["\u8841", "\u8841"], /* CJK Ideograph */
  ["\u8842", "\u8842"], /* CJK Ideograph */
  ["\u8843", "\u8843"], /* CJK Ideograph */
  ["\u8844", "\u8844"], /* CJK Ideograph */
  ["\u8845", "\u8845"], /* CJK Ideograph */
  ["\u8846", "\u8846"], /* CJK Ideograph */
  ["\u8847", "\u8847"], /* CJK Ideograph */
  ["\u8848", "\u8848"], /* CJK Ideograph */
  ["\u8849", "\u8849"], /* CJK Ideograph */
  ["\u884A", "\u884A"], /* CJK Ideograph */
  ["\u884B", "\u884B"], /* CJK Ideograph */
  ["\u884C", "\u884C"], /* CJK Ideograph */
  ["\u884D", "\u884D"], /* CJK Ideograph */
  ["\u884E", "\u884E"], /* CJK Ideograph */
  ["\u884F", "\u884F"], /* CJK Ideograph */
  ["\u8850", "\u8850"], /* CJK Ideograph */
  ["\u8851", "\u8851"], /* CJK Ideograph */
  ["\u8852", "\u8852"], /* CJK Ideograph */
  ["\u8853", "\u8853"], /* CJK Ideograph */
  ["\u8854", "\u8854"], /* CJK Ideograph */
  ["\u8855", "\u8855"], /* CJK Ideograph */
  ["\u8856", "\u8856"], /* CJK Ideograph */
  ["\u8857", "\u8857"], /* CJK Ideograph */
  ["\u8858", "\u8858"], /* CJK Ideograph */
  ["\u8859", "\u8859"], /* CJK Ideograph */
  ["\u885A", "\u885A"], /* CJK Ideograph */
  ["\u885B", "\u885B"], /* CJK Ideograph */
  ["\u885C", "\u885C"], /* CJK Ideograph */
  ["\u885D", "\u885D"], /* CJK Ideograph */
  ["\u885E", "\u885E"], /* CJK Ideograph */
  ["\u885F", "\u885F"], /* CJK Ideograph */
  ["\u8860", "\u8860"], /* CJK Ideograph */
  ["\u8861", "\u8861"], /* CJK Ideograph */
  ["\u8862", "\u8862"], /* CJK Ideograph */
  ["\u8863", "\u8863"], /* CJK Ideograph */
  ["\u8864", "\u8864"], /* CJK Ideograph */
  ["\u8865", "\u8865"], /* CJK Ideograph */
  ["\u8866", "\u8866"], /* CJK Ideograph */
  ["\u8867", "\u8867"], /* CJK Ideograph */
  ["\u8868", "\u8868"], /* CJK Ideograph */
  ["\u8869", "\u8869"], /* CJK Ideograph */
  ["\u886A", "\u886A"], /* CJK Ideograph */
  ["\u886B", "\u886B"], /* CJK Ideograph */
  ["\u886C", "\u886C"], /* CJK Ideograph */
  ["\u886D", "\u886D"], /* CJK Ideograph */
  ["\u886E", "\u886E"], /* CJK Ideograph */
  ["\u886F", "\u886F"], /* CJK Ideograph */
  ["\u8870", "\u8870"], /* CJK Ideograph */
  ["\u8871", "\u8871"], /* CJK Ideograph */
  ["\u8872", "\u8872"], /* CJK Ideograph */
  ["\u8873", "\u8873"], /* CJK Ideograph */
  ["\u8874", "\u8874"], /* CJK Ideograph */
  ["\u8875", "\u8875"], /* CJK Ideograph */
  ["\u8876", "\u8876"], /* CJK Ideograph */
  ["\u8877", "\u8877"], /* CJK Ideograph */
  ["\u8878", "\u8878"], /* CJK Ideograph */
  ["\u8879", "\u8879"], /* CJK Ideograph */
  ["\u887A", "\u887A"], /* CJK Ideograph */
  ["\u887B", "\u887B"], /* CJK Ideograph */
  ["\u887C", "\u887C"], /* CJK Ideograph */
  ["\u887D", "\u887D"], /* CJK Ideograph */
  ["\u887E", "\u887E"], /* CJK Ideograph */
  ["\u887F", "\u887F"], /* CJK Ideograph */
  ["\u8880", "\u8880"], /* CJK Ideograph */
  ["\u8881", "\u8881"], /* CJK Ideograph */
  ["\u8882", "\u8882"], /* CJK Ideograph */
  ["\u8883", "\u8883"], /* CJK Ideograph */
  ["\u8884", "\u8884"], /* CJK Ideograph */
  ["\u8885", "\u8885"], /* CJK Ideograph */
  ["\u8886", "\u8886"], /* CJK Ideograph */
  ["\u8887", "\u8887"], /* CJK Ideograph */
  ["\u8888", "\u8888"], /* CJK Ideograph */
  ["\u8889", "\u8889"], /* CJK Ideograph */
  ["\u888A", "\u888A"], /* CJK Ideograph */
  ["\u888B", "\u888B"], /* CJK Ideograph */
  ["\u888C", "\u888C"], /* CJK Ideograph */
  ["\u888D", "\u888D"], /* CJK Ideograph */
  ["\u888E", "\u888E"], /* CJK Ideograph */
  ["\u888F", "\u888F"], /* CJK Ideograph */
  ["\u8890", "\u8890"], /* CJK Ideograph */
  ["\u8891", "\u8891"], /* CJK Ideograph */
  ["\u8892", "\u8892"], /* CJK Ideograph */
  ["\u8893", "\u8893"], /* CJK Ideograph */
  ["\u8894", "\u8894"], /* CJK Ideograph */
  ["\u8895", "\u8895"], /* CJK Ideograph */
  ["\u8896", "\u8896"], /* CJK Ideograph */
  ["\u8897", "\u8897"], /* CJK Ideograph */
  ["\u8898", "\u8898"], /* CJK Ideograph */
  ["\u8899", "\u8899"], /* CJK Ideograph */
  ["\u889A", "\u889A"], /* CJK Ideograph */
  ["\u889B", "\u889B"], /* CJK Ideograph */
  ["\u889C", "\u889C"], /* CJK Ideograph */
  ["\u889D", "\u889D"], /* CJK Ideograph */
  ["\u889E", "\u889E"], /* CJK Ideograph */
  ["\u889F", "\u889F"], /* CJK Ideograph */
  ["\u88A0", "\u88A0"], /* CJK Ideograph */
  ["\u88A1", "\u88A1"], /* CJK Ideograph */
  ["\u88A2", "\u88A2"], /* CJK Ideograph */
  ["\u88A3", "\u88A3"], /* CJK Ideograph */
  ["\u88A4", "\u88A4"], /* CJK Ideograph */
  ["\u88A5", "\u88A5"], /* CJK Ideograph */
  ["\u88A6", "\u88A6"], /* CJK Ideograph */
  ["\u88A7", "\u88A7"], /* CJK Ideograph */
  ["\u88A8", "\u88A8"], /* CJK Ideograph */
  ["\u88A9", "\u88A9"], /* CJK Ideograph */
  ["\u88AA", "\u88AA"], /* CJK Ideograph */
  ["\u88AB", "\u88AB"], /* CJK Ideograph */
  ["\u88AC", "\u88AC"], /* CJK Ideograph */
  ["\u88AD", "\u88AD"], /* CJK Ideograph */
  ["\u88AE", "\u88AE"], /* CJK Ideograph */
  ["\u88AF", "\u88AF"], /* CJK Ideograph */
  ["\u88B0", "\u88B0"], /* CJK Ideograph */
  ["\u88B1", "\u88B1"], /* CJK Ideograph */
  ["\u88B2", "\u88B2"], /* CJK Ideograph */
  ["\u88B3", "\u88B3"], /* CJK Ideograph */
  ["\u88B4", "\u88B4"], /* CJK Ideograph */
  ["\u88B5", "\u88B5"], /* CJK Ideograph */
  ["\u88B6", "\u88B6"], /* CJK Ideograph */
  ["\u88B7", "\u88B7"], /* CJK Ideograph */
  ["\u88B8", "\u88B8"], /* CJK Ideograph */
  ["\u88B9", "\u88B9"], /* CJK Ideograph */
  ["\u88BA", "\u88BA"], /* CJK Ideograph */
  ["\u88BB", "\u88BB"], /* CJK Ideograph */
  ["\u88BC", "\u88BC"], /* CJK Ideograph */
  ["\u88BD", "\u88BD"], /* CJK Ideograph */
  ["\u88BE", "\u88BE"], /* CJK Ideograph */
  ["\u88BF", "\u88BF"], /* CJK Ideograph */
  ["\u88C0", "\u88C0"], /* CJK Ideograph */
  ["\u88C1", "\u88C1"], /* CJK Ideograph */
  ["\u88C2", "\u88C2"], /* CJK Ideograph */
  ["\u88C3", "\u88C3"], /* CJK Ideograph */
  ["\u88C4", "\u88C4"], /* CJK Ideograph */
  ["\u88C5", "\u88C5"], /* CJK Ideograph */
  ["\u88C6", "\u88C6"], /* CJK Ideograph */
  ["\u88C7", "\u88C7"], /* CJK Ideograph */
  ["\u88C8", "\u88C8"], /* CJK Ideograph */
  ["\u88C9", "\u88C9"], /* CJK Ideograph */
  ["\u88CA", "\u88CA"], /* CJK Ideograph */
  ["\u88CB", "\u88CB"], /* CJK Ideograph */
  ["\u88CC", "\u88CC"], /* CJK Ideograph */
  ["\u88CD", "\u88CD"], /* CJK Ideograph */
  ["\u88CE", "\u88CE"], /* CJK Ideograph */
  ["\u88CF", "\u88CF"], /* CJK Ideograph */
  ["\u88D0", "\u88D0"], /* CJK Ideograph */
  ["\u88D1", "\u88D1"], /* CJK Ideograph */
  ["\u88D2", "\u88D2"], /* CJK Ideograph */
  ["\u88D3", "\u88D3"], /* CJK Ideograph */
  ["\u88D4", "\u88D4"], /* CJK Ideograph */
  ["\u88D5", "\u88D5"], /* CJK Ideograph */
  ["\u88D6", "\u88D6"], /* CJK Ideograph */
  ["\u88D7", "\u88D7"], /* CJK Ideograph */
  ["\u88D8", "\u88D8"], /* CJK Ideograph */
  ["\u88D9", "\u88D9"], /* CJK Ideograph */
  ["\u88DA", "\u88DA"], /* CJK Ideograph */
  ["\u88DB", "\u88DB"], /* CJK Ideograph */
  ["\u88DC", "\u88DC"], /* CJK Ideograph */
  ["\u88DD", "\u88DD"], /* CJK Ideograph */
  ["\u88DE", "\u88DE"], /* CJK Ideograph */
  ["\u88DF", "\u88DF"], /* CJK Ideograph */
  ["\u88E0", "\u88E0"], /* CJK Ideograph */
  ["\u88E1", "\u88E1"], /* CJK Ideograph */
  ["\u88E2", "\u88E2"], /* CJK Ideograph */
  ["\u88E3", "\u88E3"], /* CJK Ideograph */
  ["\u88E4", "\u88E4"], /* CJK Ideograph */
  ["\u88E5", "\u88E5"], /* CJK Ideograph */
  ["\u88E6", "\u88E6"], /* CJK Ideograph */
  ["\u88E7", "\u88E7"], /* CJK Ideograph */
  ["\u88E8", "\u88E8"], /* CJK Ideograph */
  ["\u88E9", "\u88E9"], /* CJK Ideograph */
  ["\u88EA", "\u88EA"], /* CJK Ideograph */
  ["\u88EB", "\u88EB"], /* CJK Ideograph */
  ["\u88EC", "\u88EC"], /* CJK Ideograph */
  ["\u88ED", "\u88ED"], /* CJK Ideograph */
  ["\u88EE", "\u88EE"], /* CJK Ideograph */
  ["\u88EF", "\u88EF"], /* CJK Ideograph */
  ["\u88F0", "\u88F0"], /* CJK Ideograph */
  ["\u88F1", "\u88F1"], /* CJK Ideograph */
  ["\u88F2", "\u88F2"], /* CJK Ideograph */
  ["\u88F3", "\u88F3"], /* CJK Ideograph */
  ["\u88F4", "\u88F4"], /* CJK Ideograph */
  ["\u88F5", "\u88F5"], /* CJK Ideograph */
  ["\u88F6", "\u88F6"], /* CJK Ideograph */
  ["\u88F7", "\u88F7"], /* CJK Ideograph */
  ["\u88F8", "\u88F8"], /* CJK Ideograph */
  ["\u88F9", "\u88F9"], /* CJK Ideograph */
  ["\u88FA", "\u88FA"], /* CJK Ideograph */
  ["\u88FB", "\u88FB"], /* CJK Ideograph */
  ["\u88FC", "\u88FC"], /* CJK Ideograph */
  ["\u88FD", "\u88FD"], /* CJK Ideograph */
  ["\u88FE", "\u88FE"], /* CJK Ideograph */
  ["\u88FF", "\u88FF"], /* CJK Ideograph */
  ["\u8900", "\u8900"], /* CJK Ideograph */
  ["\u8901", "\u8901"], /* CJK Ideograph */
  ["\u8902", "\u8902"], /* CJK Ideograph */
  ["\u8903", "\u8903"], /* CJK Ideograph */
  ["\u8904", "\u8904"], /* CJK Ideograph */
  ["\u8905", "\u8905"], /* CJK Ideograph */
  ["\u8906", "\u8906"], /* CJK Ideograph */
  ["\u8907", "\u8907"], /* CJK Ideograph */
  ["\u8908", "\u8908"], /* CJK Ideograph */
  ["\u8909", "\u8909"], /* CJK Ideograph */
  ["\u890A", "\u890A"], /* CJK Ideograph */
  ["\u890B", "\u890B"], /* CJK Ideograph */
  ["\u890C", "\u890C"], /* CJK Ideograph */
  ["\u890D", "\u890D"], /* CJK Ideograph */
  ["\u890E", "\u890E"], /* CJK Ideograph */
  ["\u890F", "\u890F"], /* CJK Ideograph */
  ["\u8910", "\u8910"], /* CJK Ideograph */
  ["\u8911", "\u8911"], /* CJK Ideograph */
  ["\u8912", "\u8912"], /* CJK Ideograph */
  ["\u8913", "\u8913"], /* CJK Ideograph */
  ["\u8914", "\u8914"], /* CJK Ideograph */
  ["\u8915", "\u8915"], /* CJK Ideograph */
  ["\u8916", "\u8916"], /* CJK Ideograph */
  ["\u8917", "\u8917"], /* CJK Ideograph */
  ["\u8918", "\u8918"], /* CJK Ideograph */
  ["\u8919", "\u8919"], /* CJK Ideograph */
  ["\u891A", "\u891A"], /* CJK Ideograph */
  ["\u891B", "\u891B"], /* CJK Ideograph */
  ["\u891C", "\u891C"], /* CJK Ideograph */
  ["\u891D", "\u891D"], /* CJK Ideograph */
  ["\u891E", "\u891E"], /* CJK Ideograph */
  ["\u891F", "\u891F"], /* CJK Ideograph */
  ["\u8920", "\u8920"], /* CJK Ideograph */
  ["\u8921", "\u8921"], /* CJK Ideograph */
  ["\u8922", "\u8922"], /* CJK Ideograph */
  ["\u8923", "\u8923"], /* CJK Ideograph */
  ["\u8924", "\u8924"], /* CJK Ideograph */
  ["\u8925", "\u8925"], /* CJK Ideograph */
  ["\u8926", "\u8926"], /* CJK Ideograph */
  ["\u8927", "\u8927"], /* CJK Ideograph */
  ["\u8928", "\u8928"], /* CJK Ideograph */
  ["\u8929", "\u8929"], /* CJK Ideograph */
  ["\u892A", "\u892A"], /* CJK Ideograph */
  ["\u892B", "\u892B"], /* CJK Ideograph */
  ["\u892C", "\u892C"], /* CJK Ideograph */
  ["\u892D", "\u892D"], /* CJK Ideograph */
  ["\u892E", "\u892E"], /* CJK Ideograph */
  ["\u892F", "\u892F"], /* CJK Ideograph */
  ["\u8930", "\u8930"], /* CJK Ideograph */
  ["\u8931", "\u8931"], /* CJK Ideograph */
  ["\u8932", "\u8932"], /* CJK Ideograph */
  ["\u8933", "\u8933"], /* CJK Ideograph */
  ["\u8934", "\u8934"], /* CJK Ideograph */
  ["\u8935", "\u8935"], /* CJK Ideograph */
  ["\u8936", "\u8936"], /* CJK Ideograph */
  ["\u8937", "\u8937"], /* CJK Ideograph */
  ["\u8938", "\u8938"], /* CJK Ideograph */
  ["\u8939", "\u8939"], /* CJK Ideograph */
  ["\u893A", "\u893A"], /* CJK Ideograph */
  ["\u893B", "\u893B"], /* CJK Ideograph */
  ["\u893C", "\u893C"], /* CJK Ideograph */
  ["\u893D", "\u893D"], /* CJK Ideograph */
  ["\u893E", "\u893E"], /* CJK Ideograph */
  ["\u893F", "\u893F"], /* CJK Ideograph */
  ["\u8940", "\u8940"], /* CJK Ideograph */
  ["\u8941", "\u8941"], /* CJK Ideograph */
  ["\u8942", "\u8942"], /* CJK Ideograph */
  ["\u8943", "\u8943"], /* CJK Ideograph */
  ["\u8944", "\u8944"], /* CJK Ideograph */
  ["\u8945", "\u8945"], /* CJK Ideograph */
  ["\u8946", "\u8946"], /* CJK Ideograph */
  ["\u8947", "\u8947"], /* CJK Ideograph */
  ["\u8948", "\u8948"], /* CJK Ideograph */
  ["\u8949", "\u8949"], /* CJK Ideograph */
  ["\u894A", "\u894A"], /* CJK Ideograph */
  ["\u894B", "\u894B"], /* CJK Ideograph */
  ["\u894C", "\u894C"], /* CJK Ideograph */
  ["\u894D", "\u894D"], /* CJK Ideograph */
  ["\u894E", "\u894E"], /* CJK Ideograph */
  ["\u894F", "\u894F"], /* CJK Ideograph */
  ["\u8950", "\u8950"], /* CJK Ideograph */
  ["\u8951", "\u8951"], /* CJK Ideograph */
  ["\u8952", "\u8952"], /* CJK Ideograph */
  ["\u8953", "\u8953"], /* CJK Ideograph */
  ["\u8954", "\u8954"], /* CJK Ideograph */
  ["\u8955", "\u8955"], /* CJK Ideograph */
  ["\u8956", "\u8956"], /* CJK Ideograph */
  ["\u8957", "\u8957"], /* CJK Ideograph */
  ["\u8958", "\u8958"], /* CJK Ideograph */
  ["\u8959", "\u8959"], /* CJK Ideograph */
  ["\u895A", "\u895A"], /* CJK Ideograph */
  ["\u895B", "\u895B"], /* CJK Ideograph */
  ["\u895C", "\u895C"], /* CJK Ideograph */
  ["\u895D", "\u895D"], /* CJK Ideograph */
  ["\u895E", "\u895E"], /* CJK Ideograph */
  ["\u895F", "\u895F"], /* CJK Ideograph */
  ["\u8960", "\u8960"], /* CJK Ideograph */
  ["\u8961", "\u8961"], /* CJK Ideograph */
  ["\u8962", "\u8962"], /* CJK Ideograph */
  ["\u8963", "\u8963"], /* CJK Ideograph */
  ["\u8964", "\u8964"], /* CJK Ideograph */
  ["\u8965", "\u8965"], /* CJK Ideograph */
  ["\u8966", "\u8966"], /* CJK Ideograph */
  ["\u8967", "\u8967"], /* CJK Ideograph */
  ["\u8968", "\u8968"], /* CJK Ideograph */
  ["\u8969", "\u8969"], /* CJK Ideograph */
  ["\u896A", "\u896A"], /* CJK Ideograph */
  ["\u896B", "\u896B"], /* CJK Ideograph */
  ["\u896C", "\u896C"], /* CJK Ideograph */
  ["\u896D", "\u896D"], /* CJK Ideograph */
  ["\u896E", "\u896E"], /* CJK Ideograph */
  ["\u896F", "\u896F"], /* CJK Ideograph */
  ["\u8970", "\u8970"], /* CJK Ideograph */
  ["\u8971", "\u8971"], /* CJK Ideograph */
  ["\u8972", "\u8972"], /* CJK Ideograph */
  ["\u8973", "\u8973"], /* CJK Ideograph */
  ["\u8974", "\u8974"], /* CJK Ideograph */
  ["\u8975", "\u8975"], /* CJK Ideograph */
  ["\u8976", "\u8976"], /* CJK Ideograph */
  ["\u8977", "\u8977"], /* CJK Ideograph */
  ["\u8978", "\u8978"], /* CJK Ideograph */
  ["\u8979", "\u8979"], /* CJK Ideograph */
  ["\u897A", "\u897A"], /* CJK Ideograph */
  ["\u897B", "\u897B"], /* CJK Ideograph */
  ["\u897C", "\u897C"], /* CJK Ideograph */
  ["\u897D", "\u897D"], /* CJK Ideograph */
  ["\u897E", "\u897E"], /* CJK Ideograph */
  ["\u897F", "\u897F"], /* CJK Ideograph */
  ["\u8980", "\u8980"], /* CJK Ideograph */
  ["\u8981", "\u8981"], /* CJK Ideograph */
  ["\u8982", "\u8982"], /* CJK Ideograph */
  ["\u8983", "\u8983"], /* CJK Ideograph */
  ["\u8984", "\u8984"], /* CJK Ideograph */
  ["\u8985", "\u8985"], /* CJK Ideograph */
  ["\u8986", "\u8986"], /* CJK Ideograph */
  ["\u8987", "\u8987"], /* CJK Ideograph */
  ["\u8988", "\u8988"], /* CJK Ideograph */
  ["\u8989", "\u8989"], /* CJK Ideograph */
  ["\u898A", "\u898A"], /* CJK Ideograph */
  ["\u898B", "\u898B"], /* CJK Ideograph */
  ["\u898C", "\u898C"], /* CJK Ideograph */
  ["\u898D", "\u898D"], /* CJK Ideograph */
  ["\u898E", "\u898E"], /* CJK Ideograph */
  ["\u898F", "\u898F"], /* CJK Ideograph */
  ["\u8990", "\u8990"], /* CJK Ideograph */
  ["\u8991", "\u8991"], /* CJK Ideograph */
  ["\u8992", "\u8992"], /* CJK Ideograph */
  ["\u8993", "\u8993"], /* CJK Ideograph */
  ["\u8994", "\u8994"], /* CJK Ideograph */
  ["\u8995", "\u8995"], /* CJK Ideograph */
  ["\u8996", "\u8996"], /* CJK Ideograph */
  ["\u8997", "\u8997"], /* CJK Ideograph */
  ["\u8998", "\u8998"], /* CJK Ideograph */
  ["\u8999", "\u8999"], /* CJK Ideograph */
  ["\u899A", "\u899A"], /* CJK Ideograph */
  ["\u899B", "\u899B"], /* CJK Ideograph */
  ["\u899C", "\u899C"], /* CJK Ideograph */
  ["\u899D", "\u899D"], /* CJK Ideograph */
  ["\u899E", "\u899E"], /* CJK Ideograph */
  ["\u899F", "\u899F"], /* CJK Ideograph */
  ["\u89A0", "\u89A0"], /* CJK Ideograph */
  ["\u89A1", "\u89A1"], /* CJK Ideograph */
  ["\u89A2", "\u89A2"], /* CJK Ideograph */
  ["\u89A3", "\u89A3"], /* CJK Ideograph */
  ["\u89A4", "\u89A4"], /* CJK Ideograph */
  ["\u89A5", "\u89A5"], /* CJK Ideograph */
  ["\u89A6", "\u89A6"], /* CJK Ideograph */
  ["\u89A7", "\u89A7"], /* CJK Ideograph */
  ["\u89A8", "\u89A8"], /* CJK Ideograph */
  ["\u89A9", "\u89A9"], /* CJK Ideograph */
  ["\u89AA", "\u89AA"], /* CJK Ideograph */
  ["\u89AB", "\u89AB"], /* CJK Ideograph */
  ["\u89AC", "\u89AC"], /* CJK Ideograph */
  ["\u89AD", "\u89AD"], /* CJK Ideograph */
  ["\u89AE", "\u89AE"], /* CJK Ideograph */
  ["\u89AF", "\u89AF"], /* CJK Ideograph */
  ["\u89B0", "\u89B0"], /* CJK Ideograph */
  ["\u89B1", "\u89B1"], /* CJK Ideograph */
  ["\u89B2", "\u89B2"], /* CJK Ideograph */
  ["\u89B3", "\u89B3"], /* CJK Ideograph */
  ["\u89B4", "\u89B4"], /* CJK Ideograph */
  ["\u89B5", "\u89B5"], /* CJK Ideograph */
  ["\u89B6", "\u89B6"], /* CJK Ideograph */
  ["\u89B7", "\u89B7"], /* CJK Ideograph */
  ["\u89B8", "\u89B8"], /* CJK Ideograph */
  ["\u89B9", "\u89B9"], /* CJK Ideograph */
  ["\u89BA", "\u89BA"], /* CJK Ideograph */
  ["\u89BB", "\u89BB"], /* CJK Ideograph */
  ["\u89BC", "\u89BC"], /* CJK Ideograph */
  ["\u89BD", "\u89BD"], /* CJK Ideograph */
  ["\u89BE", "\u89BE"], /* CJK Ideograph */
  ["\u89BF", "\u89BF"], /* CJK Ideograph */
  ["\u89C0", "\u89C0"], /* CJK Ideograph */
  ["\u89C1", "\u89C1"], /* CJK Ideograph */
  ["\u89C2", "\u89C2"], /* CJK Ideograph */
  ["\u89C3", "\u89C3"], /* CJK Ideograph */
  ["\u89C4", "\u89C4"], /* CJK Ideograph */
  ["\u89C5", "\u89C5"], /* CJK Ideograph */
  ["\u89C6", "\u89C6"], /* CJK Ideograph */
  ["\u89C7", "\u89C7"], /* CJK Ideograph */
  ["\u89C8", "\u89C8"], /* CJK Ideograph */
  ["\u89C9", "\u89C9"], /* CJK Ideograph */
  ["\u89CA", "\u89CA"], /* CJK Ideograph */
  ["\u89CB", "\u89CB"], /* CJK Ideograph */
  ["\u89CC", "\u89CC"], /* CJK Ideograph */
  ["\u89CD", "\u89CD"], /* CJK Ideograph */
  ["\u89CE", "\u89CE"], /* CJK Ideograph */
  ["\u89CF", "\u89CF"], /* CJK Ideograph */
  ["\u89D0", "\u89D0"], /* CJK Ideograph */
  ["\u89D1", "\u89D1"], /* CJK Ideograph */
  ["\u89D2", "\u89D2"], /* CJK Ideograph */
  ["\u89D3", "\u89D3"], /* CJK Ideograph */
  ["\u89D4", "\u89D4"], /* CJK Ideograph */
  ["\u89D5", "\u89D5"], /* CJK Ideograph */
  ["\u89D6", "\u89D6"], /* CJK Ideograph */
  ["\u89D7", "\u89D7"], /* CJK Ideograph */
  ["\u89D8", "\u89D8"], /* CJK Ideograph */
  ["\u89D9", "\u89D9"], /* CJK Ideograph */
  ["\u89DA", "\u89DA"], /* CJK Ideograph */
  ["\u89DB", "\u89DB"], /* CJK Ideograph */
  ["\u89DC", "\u89DC"], /* CJK Ideograph */
  ["\u89DD", "\u89DD"], /* CJK Ideograph */
  ["\u89DE", "\u89DE"], /* CJK Ideograph */
  ["\u89DF", "\u89DF"], /* CJK Ideograph */
  ["\u89E0", "\u89E0"], /* CJK Ideograph */
  ["\u89E1", "\u89E1"], /* CJK Ideograph */
  ["\u89E2", "\u89E2"], /* CJK Ideograph */
  ["\u89E3", "\u89E3"], /* CJK Ideograph */
  ["\u89E4", "\u89E4"], /* CJK Ideograph */
  ["\u89E5", "\u89E5"], /* CJK Ideograph */
  ["\u89E6", "\u89E6"], /* CJK Ideograph */
  ["\u89E7", "\u89E7"], /* CJK Ideograph */
  ["\u89E8", "\u89E8"], /* CJK Ideograph */
  ["\u89E9", "\u89E9"], /* CJK Ideograph */
  ["\u89EA", "\u89EA"], /* CJK Ideograph */
  ["\u89EB", "\u89EB"], /* CJK Ideograph */
  ["\u89EC", "\u89EC"], /* CJK Ideograph */
  ["\u89ED", "\u89ED"], /* CJK Ideograph */
  ["\u89EE", "\u89EE"], /* CJK Ideograph */
  ["\u89EF", "\u89EF"], /* CJK Ideograph */
  ["\u89F0", "\u89F0"], /* CJK Ideograph */
  ["\u89F1", "\u89F1"], /* CJK Ideograph */
  ["\u89F2", "\u89F2"], /* CJK Ideograph */
  ["\u89F3", "\u89F3"], /* CJK Ideograph */
  ["\u89F4", "\u89F4"], /* CJK Ideograph */
  ["\u89F5", "\u89F5"], /* CJK Ideograph */
  ["\u89F6", "\u89F6"], /* CJK Ideograph */
  ["\u89F7", "\u89F7"], /* CJK Ideograph */
  ["\u89F8", "\u89F8"], /* CJK Ideograph */
  ["\u89F9", "\u89F9"], /* CJK Ideograph */
  ["\u89FA", "\u89FA"], /* CJK Ideograph */
  ["\u89FB", "\u89FB"], /* CJK Ideograph */
  ["\u89FC", "\u89FC"], /* CJK Ideograph */
  ["\u89FD", "\u89FD"], /* CJK Ideograph */
  ["\u89FE", "\u89FE"], /* CJK Ideograph */
  ["\u89FF", "\u89FF"], /* CJK Ideograph */
  ["\u8A00", "\u8A00"], /* CJK Ideograph */
  ["\u8A01", "\u8A01"], /* CJK Ideograph */
  ["\u8A02", "\u8A02"], /* CJK Ideograph */
  ["\u8A03", "\u8A03"], /* CJK Ideograph */
  ["\u8A04", "\u8A04"], /* CJK Ideograph */
  ["\u8A05", "\u8A05"], /* CJK Ideograph */
  ["\u8A06", "\u8A06"], /* CJK Ideograph */
  ["\u8A07", "\u8A07"], /* CJK Ideograph */
  ["\u8A08", "\u8A08"], /* CJK Ideograph */
  ["\u8A09", "\u8A09"], /* CJK Ideograph */
  ["\u8A0A", "\u8A0A"], /* CJK Ideograph */
  ["\u8A0B", "\u8A0B"], /* CJK Ideograph */
  ["\u8A0C", "\u8A0C"], /* CJK Ideograph */
  ["\u8A0D", "\u8A0D"], /* CJK Ideograph */
  ["\u8A0E", "\u8A0E"], /* CJK Ideograph */
  ["\u8A0F", "\u8A0F"], /* CJK Ideograph */
  ["\u8A10", "\u8A10"], /* CJK Ideograph */
  ["\u8A11", "\u8A11"], /* CJK Ideograph */
  ["\u8A12", "\u8A12"], /* CJK Ideograph */
  ["\u8A13", "\u8A13"], /* CJK Ideograph */
  ["\u8A14", "\u8A14"], /* CJK Ideograph */
  ["\u8A15", "\u8A15"], /* CJK Ideograph */
  ["\u8A16", "\u8A16"], /* CJK Ideograph */
  ["\u8A17", "\u8A17"], /* CJK Ideograph */
  ["\u8A18", "\u8A18"], /* CJK Ideograph */
  ["\u8A19", "\u8A19"], /* CJK Ideograph */
  ["\u8A1A", "\u8A1A"], /* CJK Ideograph */
  ["\u8A1B", "\u8A1B"], /* CJK Ideograph */
  ["\u8A1C", "\u8A1C"], /* CJK Ideograph */
  ["\u8A1D", "\u8A1D"], /* CJK Ideograph */
  ["\u8A1E", "\u8A1E"], /* CJK Ideograph */
  ["\u8A1F", "\u8A1F"], /* CJK Ideograph */
  ["\u8A20", "\u8A20"], /* CJK Ideograph */
  ["\u8A21", "\u8A21"], /* CJK Ideograph */
  ["\u8A22", "\u8A22"], /* CJK Ideograph */
  ["\u8A23", "\u8A23"], /* CJK Ideograph */
  ["\u8A24", "\u8A24"], /* CJK Ideograph */
  ["\u8A25", "\u8A25"], /* CJK Ideograph */
  ["\u8A26", "\u8A26"], /* CJK Ideograph */
  ["\u8A27", "\u8A27"], /* CJK Ideograph */
  ["\u8A28", "\u8A28"], /* CJK Ideograph */
  ["\u8A29", "\u8A29"], /* CJK Ideograph */
  ["\u8A2A", "\u8A2A"], /* CJK Ideograph */
  ["\u8A2B", "\u8A2B"], /* CJK Ideograph */
  ["\u8A2C", "\u8A2C"], /* CJK Ideograph */
  ["\u8A2D", "\u8A2D"], /* CJK Ideograph */
  ["\u8A2E", "\u8A2E"], /* CJK Ideograph */
  ["\u8A2F", "\u8A2F"], /* CJK Ideograph */
  ["\u8A30", "\u8A30"], /* CJK Ideograph */
  ["\u8A31", "\u8A31"], /* CJK Ideograph */
  ["\u8A32", "\u8A32"], /* CJK Ideograph */
  ["\u8A33", "\u8A33"], /* CJK Ideograph */
  ["\u8A34", "\u8A34"], /* CJK Ideograph */
  ["\u8A35", "\u8A35"], /* CJK Ideograph */
  ["\u8A36", "\u8A36"], /* CJK Ideograph */
  ["\u8A37", "\u8A37"], /* CJK Ideograph */
  ["\u8A38", "\u8A38"], /* CJK Ideograph */
  ["\u8A39", "\u8A39"], /* CJK Ideograph */
  ["\u8A3A", "\u8A3A"], /* CJK Ideograph */
  ["\u8A3B", "\u8A3B"], /* CJK Ideograph */
  ["\u8A3C", "\u8A3C"], /* CJK Ideograph */
  ["\u8A3D", "\u8A3D"], /* CJK Ideograph */
  ["\u8A3E", "\u8A3E"], /* CJK Ideograph */
  ["\u8A3F", "\u8A3F"], /* CJK Ideograph */
  ["\u8A40", "\u8A40"], /* CJK Ideograph */
  ["\u8A41", "\u8A41"], /* CJK Ideograph */
  ["\u8A42", "\u8A42"], /* CJK Ideograph */
  ["\u8A43", "\u8A43"], /* CJK Ideograph */
  ["\u8A44", "\u8A44"], /* CJK Ideograph */
  ["\u8A45", "\u8A45"], /* CJK Ideograph */
  ["\u8A46", "\u8A46"], /* CJK Ideograph */
  ["\u8A47", "\u8A47"], /* CJK Ideograph */
  ["\u8A48", "\u8A48"], /* CJK Ideograph */
  ["\u8A49", "\u8A49"], /* CJK Ideograph */
  ["\u8A4A", "\u8A4A"], /* CJK Ideograph */
  ["\u8A4B", "\u8A4B"], /* CJK Ideograph */
  ["\u8A4C", "\u8A4C"], /* CJK Ideograph */
  ["\u8A4D", "\u8A4D"], /* CJK Ideograph */
  ["\u8A4E", "\u8A4E"], /* CJK Ideograph */
  ["\u8A4F", "\u8A4F"], /* CJK Ideograph */
  ["\u8A50", "\u8A50"], /* CJK Ideograph */
  ["\u8A51", "\u8A51"], /* CJK Ideograph */
  ["\u8A52", "\u8A52"], /* CJK Ideograph */
  ["\u8A53", "\u8A53"], /* CJK Ideograph */
  ["\u8A54", "\u8A54"], /* CJK Ideograph */
  ["\u8A55", "\u8A55"], /* CJK Ideograph */
  ["\u8A56", "\u8A56"], /* CJK Ideograph */
  ["\u8A57", "\u8A57"], /* CJK Ideograph */
  ["\u8A58", "\u8A58"], /* CJK Ideograph */
  ["\u8A59", "\u8A59"], /* CJK Ideograph */
  ["\u8A5A", "\u8A5A"], /* CJK Ideograph */
  ["\u8A5B", "\u8A5B"], /* CJK Ideograph */
  ["\u8A5C", "\u8A5C"], /* CJK Ideograph */
  ["\u8A5D", "\u8A5D"], /* CJK Ideograph */
  ["\u8A5E", "\u8A5E"], /* CJK Ideograph */
  ["\u8A5F", "\u8A5F"], /* CJK Ideograph */
  ["\u8A60", "\u8A60"], /* CJK Ideograph */
  ["\u8A61", "\u8A61"], /* CJK Ideograph */
  ["\u8A62", "\u8A62"], /* CJK Ideograph */
  ["\u8A63", "\u8A63"], /* CJK Ideograph */
  ["\u8A64", "\u8A64"], /* CJK Ideograph */
  ["\u8A65", "\u8A65"], /* CJK Ideograph */
  ["\u8A66", "\u8A66"], /* CJK Ideograph */
  ["\u8A67", "\u8A67"], /* CJK Ideograph */
  ["\u8A68", "\u8A68"], /* CJK Ideograph */
  ["\u8A69", "\u8A69"], /* CJK Ideograph */
  ["\u8A6A", "\u8A6A"], /* CJK Ideograph */
  ["\u8A6B", "\u8A6B"], /* CJK Ideograph */
  ["\u8A6C", "\u8A6C"], /* CJK Ideograph */
  ["\u8A6D", "\u8A6D"], /* CJK Ideograph */
  ["\u8A6E", "\u8A6E"], /* CJK Ideograph */
  ["\u8A6F", "\u8A6F"], /* CJK Ideograph */
  ["\u8A70", "\u8A70"], /* CJK Ideograph */
  ["\u8A71", "\u8A71"], /* CJK Ideograph */
  ["\u8A72", "\u8A72"], /* CJK Ideograph */
  ["\u8A73", "\u8A73"], /* CJK Ideograph */
  ["\u8A74", "\u8A74"], /* CJK Ideograph */
  ["\u8A75", "\u8A75"], /* CJK Ideograph */
  ["\u8A76", "\u8A76"], /* CJK Ideograph */
  ["\u8A77", "\u8A77"], /* CJK Ideograph */
  ["\u8A78", "\u8A78"], /* CJK Ideograph */
  ["\u8A79", "\u8A79"], /* CJK Ideograph */
  ["\u8A7A", "\u8A7A"], /* CJK Ideograph */
  ["\u8A7B", "\u8A7B"], /* CJK Ideograph */
  ["\u8A7C", "\u8A7C"], /* CJK Ideograph */
  ["\u8A7D", "\u8A7D"], /* CJK Ideograph */
  ["\u8A7E", "\u8A7E"], /* CJK Ideograph */
  ["\u8A7F", "\u8A7F"], /* CJK Ideograph */
  ["\u8A80", "\u8A80"], /* CJK Ideograph */
  ["\u8A81", "\u8A81"], /* CJK Ideograph */
  ["\u8A82", "\u8A82"], /* CJK Ideograph */
  ["\u8A83", "\u8A83"], /* CJK Ideograph */
  ["\u8A84", "\u8A84"], /* CJK Ideograph */
  ["\u8A85", "\u8A85"], /* CJK Ideograph */
  ["\u8A86", "\u8A86"], /* CJK Ideograph */
  ["\u8A87", "\u8A87"], /* CJK Ideograph */
  ["\u8A88", "\u8A88"], /* CJK Ideograph */
  ["\u8A89", "\u8A89"], /* CJK Ideograph */
  ["\u8A8A", "\u8A8A"], /* CJK Ideograph */
  ["\u8A8B", "\u8A8B"], /* CJK Ideograph */
  ["\u8A8C", "\u8A8C"], /* CJK Ideograph */
  ["\u8A8D", "\u8A8D"], /* CJK Ideograph */
  ["\u8A8E", "\u8A8E"], /* CJK Ideograph */
  ["\u8A8F", "\u8A8F"], /* CJK Ideograph */
  ["\u8A90", "\u8A90"], /* CJK Ideograph */
  ["\u8A91", "\u8A91"], /* CJK Ideograph */
  ["\u8A92", "\u8A92"], /* CJK Ideograph */
  ["\u8A93", "\u8A93"], /* CJK Ideograph */
  ["\u8A94", "\u8A94"], /* CJK Ideograph */
  ["\u8A95", "\u8A95"], /* CJK Ideograph */
  ["\u8A96", "\u8A96"], /* CJK Ideograph */
  ["\u8A97", "\u8A97"], /* CJK Ideograph */
  ["\u8A98", "\u8A98"], /* CJK Ideograph */
  ["\u8A99", "\u8A99"], /* CJK Ideograph */
  ["\u8A9A", "\u8A9A"], /* CJK Ideograph */
  ["\u8A9B", "\u8A9B"], /* CJK Ideograph */
  ["\u8A9C", "\u8A9C"], /* CJK Ideograph */
  ["\u8A9D", "\u8A9D"], /* CJK Ideograph */
  ["\u8A9E", "\u8A9E"], /* CJK Ideograph */
  ["\u8A9F", "\u8A9F"], /* CJK Ideograph */
  ["\u8AA0", "\u8AA0"], /* CJK Ideograph */
  ["\u8AA1", "\u8AA1"], /* CJK Ideograph */
  ["\u8AA2", "\u8AA2"], /* CJK Ideograph */
  ["\u8AA3", "\u8AA3"], /* CJK Ideograph */
  ["\u8AA4", "\u8AA4"], /* CJK Ideograph */
  ["\u8AA5", "\u8AA5"], /* CJK Ideograph */
  ["\u8AA6", "\u8AA6"], /* CJK Ideograph */
  ["\u8AA7", "\u8AA7"], /* CJK Ideograph */
  ["\u8AA8", "\u8AA8"], /* CJK Ideograph */
  ["\u8AA9", "\u8AA9"], /* CJK Ideograph */
  ["\u8AAA", "\u8AAA"], /* CJK Ideograph */
  ["\u8AAB", "\u8AAB"], /* CJK Ideograph */
  ["\u8AAC", "\u8AAC"], /* CJK Ideograph */
  ["\u8AAD", "\u8AAD"], /* CJK Ideograph */
  ["\u8AAE", "\u8AAE"], /* CJK Ideograph */
  ["\u8AAF", "\u8AAF"], /* CJK Ideograph */
  ["\u8AB0", "\u8AB0"], /* CJK Ideograph */
  ["\u8AB1", "\u8AB1"], /* CJK Ideograph */
  ["\u8AB2", "\u8AB2"], /* CJK Ideograph */
  ["\u8AB3", "\u8AB3"], /* CJK Ideograph */
  ["\u8AB4", "\u8AB4"], /* CJK Ideograph */
  ["\u8AB5", "\u8AB5"], /* CJK Ideograph */
  ["\u8AB6", "\u8AB6"], /* CJK Ideograph */
  ["\u8AB7", "\u8AB7"], /* CJK Ideograph */
  ["\u8AB8", "\u8AB8"], /* CJK Ideograph */
  ["\u8AB9", "\u8AB9"], /* CJK Ideograph */
  ["\u8ABA", "\u8ABA"], /* CJK Ideograph */
  ["\u8ABB", "\u8ABB"], /* CJK Ideograph */
  ["\u8ABC", "\u8ABC"], /* CJK Ideograph */
  ["\u8ABD", "\u8ABD"], /* CJK Ideograph */
  ["\u8ABE", "\u8ABE"], /* CJK Ideograph */
  ["\u8ABF", "\u8ABF"], /* CJK Ideograph */
  ["\u8AC0", "\u8AC0"], /* CJK Ideograph */
  ["\u8AC1", "\u8AC1"], /* CJK Ideograph */
  ["\u8AC2", "\u8AC2"], /* CJK Ideograph */
  ["\u8AC3", "\u8AC3"], /* CJK Ideograph */
  ["\u8AC4", "\u8AC4"], /* CJK Ideograph */
  ["\u8AC5", "\u8AC5"], /* CJK Ideograph */
  ["\u8AC6", "\u8AC6"], /* CJK Ideograph */
  ["\u8AC7", "\u8AC7"], /* CJK Ideograph */
  ["\u8AC8", "\u8AC8"], /* CJK Ideograph */
  ["\u8AC9", "\u8AC9"], /* CJK Ideograph */
  ["\u8ACA", "\u8ACA"], /* CJK Ideograph */
  ["\u8ACB", "\u8ACB"], /* CJK Ideograph */
  ["\u8ACC", "\u8ACC"], /* CJK Ideograph */
  ["\u8ACD", "\u8ACD"], /* CJK Ideograph */
  ["\u8ACE", "\u8ACE"], /* CJK Ideograph */
  ["\u8ACF", "\u8ACF"], /* CJK Ideograph */
  ["\u8AD0", "\u8AD0"], /* CJK Ideograph */
  ["\u8AD1", "\u8AD1"], /* CJK Ideograph */
  ["\u8AD2", "\u8AD2"], /* CJK Ideograph */
  ["\u8AD3", "\u8AD3"], /* CJK Ideograph */
  ["\u8AD4", "\u8AD4"], /* CJK Ideograph */
  ["\u8AD5", "\u8AD5"], /* CJK Ideograph */
  ["\u8AD6", "\u8AD6"], /* CJK Ideograph */
  ["\u8AD7", "\u8AD7"], /* CJK Ideograph */
  ["\u8AD8", "\u8AD8"], /* CJK Ideograph */
  ["\u8AD9", "\u8AD9"], /* CJK Ideograph */
  ["\u8ADA", "\u8ADA"], /* CJK Ideograph */
  ["\u8ADB", "\u8ADB"], /* CJK Ideograph */
  ["\u8ADC", "\u8ADC"], /* CJK Ideograph */
  ["\u8ADD", "\u8ADD"], /* CJK Ideograph */
  ["\u8ADE", "\u8ADE"], /* CJK Ideograph */
  ["\u8ADF", "\u8ADF"], /* CJK Ideograph */
  ["\u8AE0", "\u8AE0"], /* CJK Ideograph */
  ["\u8AE1", "\u8AE1"], /* CJK Ideograph */
  ["\u8AE2", "\u8AE2"], /* CJK Ideograph */
  ["\u8AE3", "\u8AE3"], /* CJK Ideograph */
  ["\u8AE4", "\u8AE4"], /* CJK Ideograph */
  ["\u8AE5", "\u8AE5"], /* CJK Ideograph */
  ["\u8AE6", "\u8AE6"], /* CJK Ideograph */
  ["\u8AE7", "\u8AE7"], /* CJK Ideograph */
  ["\u8AE8", "\u8AE8"], /* CJK Ideograph */
  ["\u8AE9", "\u8AE9"], /* CJK Ideograph */
  ["\u8AEA", "\u8AEA"], /* CJK Ideograph */
  ["\u8AEB", "\u8AEB"], /* CJK Ideograph */
  ["\u8AEC", "\u8AEC"], /* CJK Ideograph */
  ["\u8AED", "\u8AED"], /* CJK Ideograph */
  ["\u8AEE", "\u8AEE"], /* CJK Ideograph */
  ["\u8AEF", "\u8AEF"], /* CJK Ideograph */
  ["\u8AF0", "\u8AF0"], /* CJK Ideograph */
  ["\u8AF1", "\u8AF1"], /* CJK Ideograph */
  ["\u8AF2", "\u8AF2"], /* CJK Ideograph */
  ["\u8AF3", "\u8AF3"], /* CJK Ideograph */
  ["\u8AF4", "\u8AF4"], /* CJK Ideograph */
  ["\u8AF5", "\u8AF5"], /* CJK Ideograph */
  ["\u8AF6", "\u8AF6"], /* CJK Ideograph */
  ["\u8AF7", "\u8AF7"], /* CJK Ideograph */
  ["\u8AF8", "\u8AF8"], /* CJK Ideograph */
  ["\u8AF9", "\u8AF9"], /* CJK Ideograph */
  ["\u8AFA", "\u8AFA"], /* CJK Ideograph */
  ["\u8AFB", "\u8AFB"], /* CJK Ideograph */
  ["\u8AFC", "\u8AFC"], /* CJK Ideograph */
  ["\u8AFD", "\u8AFD"], /* CJK Ideograph */
  ["\u8AFE", "\u8AFE"], /* CJK Ideograph */
  ["\u8AFF", "\u8AFF"], /* CJK Ideograph */
  ["\u8B00", "\u8B00"], /* CJK Ideograph */
  ["\u8B01", "\u8B01"], /* CJK Ideograph */
  ["\u8B02", "\u8B02"], /* CJK Ideograph */
  ["\u8B03", "\u8B03"], /* CJK Ideograph */
  ["\u8B04", "\u8B04"], /* CJK Ideograph */
  ["\u8B05", "\u8B05"], /* CJK Ideograph */
  ["\u8B06", "\u8B06"], /* CJK Ideograph */
  ["\u8B07", "\u8B07"], /* CJK Ideograph */
  ["\u8B08", "\u8B08"], /* CJK Ideograph */
  ["\u8B09", "\u8B09"], /* CJK Ideograph */
  ["\u8B0A", "\u8B0A"], /* CJK Ideograph */
  ["\u8B0B", "\u8B0B"], /* CJK Ideograph */
  ["\u8B0C", "\u8B0C"], /* CJK Ideograph */
  ["\u8B0D", "\u8B0D"], /* CJK Ideograph */
  ["\u8B0E", "\u8B0E"], /* CJK Ideograph */
  ["\u8B0F", "\u8B0F"], /* CJK Ideograph */
  ["\u8B10", "\u8B10"], /* CJK Ideograph */
  ["\u8B11", "\u8B11"], /* CJK Ideograph */
  ["\u8B12", "\u8B12"], /* CJK Ideograph */
  ["\u8B13", "\u8B13"], /* CJK Ideograph */
  ["\u8B14", "\u8B14"], /* CJK Ideograph */
  ["\u8B15", "\u8B15"], /* CJK Ideograph */
  ["\u8B16", "\u8B16"], /* CJK Ideograph */
  ["\u8B17", "\u8B17"], /* CJK Ideograph */
  ["\u8B18", "\u8B18"], /* CJK Ideograph */
  ["\u8B19", "\u8B19"], /* CJK Ideograph */
  ["\u8B1A", "\u8B1A"], /* CJK Ideograph */
  ["\u8B1B", "\u8B1B"], /* CJK Ideograph */
  ["\u8B1C", "\u8B1C"], /* CJK Ideograph */
  ["\u8B1D", "\u8B1D"], /* CJK Ideograph */
  ["\u8B1E", "\u8B1E"], /* CJK Ideograph */
  ["\u8B1F", "\u8B1F"], /* CJK Ideograph */
  ["\u8B20", "\u8B20"], /* CJK Ideograph */
  ["\u8B21", "\u8B21"], /* CJK Ideograph */
  ["\u8B22", "\u8B22"], /* CJK Ideograph */
  ["\u8B23", "\u8B23"], /* CJK Ideograph */
  ["\u8B24", "\u8B24"], /* CJK Ideograph */
  ["\u8B25", "\u8B25"], /* CJK Ideograph */
  ["\u8B26", "\u8B26"], /* CJK Ideograph */
  ["\u8B27", "\u8B27"], /* CJK Ideograph */
  ["\u8B28", "\u8B28"], /* CJK Ideograph */
  ["\u8B29", "\u8B29"], /* CJK Ideograph */
  ["\u8B2A", "\u8B2A"], /* CJK Ideograph */
  ["\u8B2B", "\u8B2B"], /* CJK Ideograph */
  ["\u8B2C", "\u8B2C"], /* CJK Ideograph */
  ["\u8B2D", "\u8B2D"], /* CJK Ideograph */
  ["\u8B2E", "\u8B2E"], /* CJK Ideograph */
  ["\u8B2F", "\u8B2F"], /* CJK Ideograph */
  ["\u8B30", "\u8B30"], /* CJK Ideograph */
  ["\u8B31", "\u8B31"], /* CJK Ideograph */
  ["\u8B32", "\u8B32"], /* CJK Ideograph */
  ["\u8B33", "\u8B33"], /* CJK Ideograph */
  ["\u8B34", "\u8B34"], /* CJK Ideograph */
  ["\u8B35", "\u8B35"], /* CJK Ideograph */
  ["\u8B36", "\u8B36"], /* CJK Ideograph */
  ["\u8B37", "\u8B37"], /* CJK Ideograph */
  ["\u8B38", "\u8B38"], /* CJK Ideograph */
  ["\u8B39", "\u8B39"], /* CJK Ideograph */
  ["\u8B3A", "\u8B3A"], /* CJK Ideograph */
  ["\u8B3B", "\u8B3B"], /* CJK Ideograph */
  ["\u8B3C", "\u8B3C"], /* CJK Ideograph */
  ["\u8B3D", "\u8B3D"], /* CJK Ideograph */
  ["\u8B3E", "\u8B3E"], /* CJK Ideograph */
  ["\u8B3F", "\u8B3F"], /* CJK Ideograph */
  ["\u8B40", "\u8B40"], /* CJK Ideograph */
  ["\u8B41", "\u8B41"], /* CJK Ideograph */
  ["\u8B42", "\u8B42"], /* CJK Ideograph */
  ["\u8B43", "\u8B43"], /* CJK Ideograph */
  ["\u8B44", "\u8B44"], /* CJK Ideograph */
  ["\u8B45", "\u8B45"], /* CJK Ideograph */
  ["\u8B46", "\u8B46"], /* CJK Ideograph */
  ["\u8B47", "\u8B47"], /* CJK Ideograph */
  ["\u8B48", "\u8B48"], /* CJK Ideograph */
  ["\u8B49", "\u8B49"], /* CJK Ideograph */
  ["\u8B4A", "\u8B4A"], /* CJK Ideograph */
  ["\u8B4B", "\u8B4B"], /* CJK Ideograph */
  ["\u8B4C", "\u8B4C"], /* CJK Ideograph */
  ["\u8B4D", "\u8B4D"], /* CJK Ideograph */
  ["\u8B4E", "\u8B4E"], /* CJK Ideograph */
  ["\u8B4F", "\u8B4F"], /* CJK Ideograph */
  ["\u8B50", "\u8B50"], /* CJK Ideograph */
  ["\u8B51", "\u8B51"], /* CJK Ideograph */
  ["\u8B52", "\u8B52"], /* CJK Ideograph */
  ["\u8B53", "\u8B53"], /* CJK Ideograph */
  ["\u8B54", "\u8B54"], /* CJK Ideograph */
  ["\u8B55", "\u8B55"], /* CJK Ideograph */
  ["\u8B56", "\u8B56"], /* CJK Ideograph */
  ["\u8B57", "\u8B57"], /* CJK Ideograph */
  ["\u8B58", "\u8B58"], /* CJK Ideograph */
  ["\u8B59", "\u8B59"], /* CJK Ideograph */
  ["\u8B5A", "\u8B5A"], /* CJK Ideograph */
  ["\u8B5B", "\u8B5B"], /* CJK Ideograph */
  ["\u8B5C", "\u8B5C"], /* CJK Ideograph */
  ["\u8B5D", "\u8B5D"], /* CJK Ideograph */
  ["\u8B5E", "\u8B5E"], /* CJK Ideograph */
  ["\u8B5F", "\u8B5F"], /* CJK Ideograph */
  ["\u8B60", "\u8B60"], /* CJK Ideograph */
  ["\u8B61", "\u8B61"], /* CJK Ideograph */
  ["\u8B62", "\u8B62"], /* CJK Ideograph */
  ["\u8B63", "\u8B63"], /* CJK Ideograph */
  ["\u8B64", "\u8B64"], /* CJK Ideograph */
  ["\u8B65", "\u8B65"], /* CJK Ideograph */
  ["\u8B66", "\u8B66"], /* CJK Ideograph */
  ["\u8B67", "\u8B67"], /* CJK Ideograph */
  ["\u8B68", "\u8B68"], /* CJK Ideograph */
  ["\u8B69", "\u8B69"], /* CJK Ideograph */
  ["\u8B6A", "\u8B6A"], /* CJK Ideograph */
  ["\u8B6B", "\u8B6B"], /* CJK Ideograph */
  ["\u8B6C", "\u8B6C"], /* CJK Ideograph */
  ["\u8B6D", "\u8B6D"], /* CJK Ideograph */
  ["\u8B6E", "\u8B6E"], /* CJK Ideograph */
  ["\u8B6F", "\u8B6F"], /* CJK Ideograph */
  ["\u8B70", "\u8B70"], /* CJK Ideograph */
  ["\u8B71", "\u8B71"], /* CJK Ideograph */
  ["\u8B72", "\u8B72"], /* CJK Ideograph */
  ["\u8B73", "\u8B73"], /* CJK Ideograph */
  ["\u8B74", "\u8B74"], /* CJK Ideograph */
  ["\u8B75", "\u8B75"], /* CJK Ideograph */
  ["\u8B76", "\u8B76"], /* CJK Ideograph */
  ["\u8B77", "\u8B77"], /* CJK Ideograph */
  ["\u8B78", "\u8B78"], /* CJK Ideograph */
  ["\u8B79", "\u8B79"], /* CJK Ideograph */
  ["\u8B7A", "\u8B7A"], /* CJK Ideograph */
  ["\u8B7B", "\u8B7B"], /* CJK Ideograph */
  ["\u8B7C", "\u8B7C"], /* CJK Ideograph */
  ["\u8B7D", "\u8B7D"], /* CJK Ideograph */
  ["\u8B7E", "\u8B7E"], /* CJK Ideograph */
  ["\u8B7F", "\u8B7F"], /* CJK Ideograph */
  ["\u8B80", "\u8B80"], /* CJK Ideograph */
  ["\u8B81", "\u8B81"], /* CJK Ideograph */
  ["\u8B82", "\u8B82"], /* CJK Ideograph */
  ["\u8B83", "\u8B83"], /* CJK Ideograph */
  ["\u8B84", "\u8B84"], /* CJK Ideograph */
  ["\u8B85", "\u8B85"], /* CJK Ideograph */
  ["\u8B86", "\u8B86"], /* CJK Ideograph */
  ["\u8B87", "\u8B87"], /* CJK Ideograph */
  ["\u8B88", "\u8B88"], /* CJK Ideograph */
  ["\u8B89", "\u8B89"], /* CJK Ideograph */
  ["\u8B8A", "\u8B8A"], /* CJK Ideograph */
  ["\u8B8B", "\u8B8B"], /* CJK Ideograph */
  ["\u8B8C", "\u8B8C"], /* CJK Ideograph */
  ["\u8B8D", "\u8B8D"], /* CJK Ideograph */
  ["\u8B8E", "\u8B8E"], /* CJK Ideograph */
  ["\u8B8F", "\u8B8F"], /* CJK Ideograph */
  ["\u8B90", "\u8B90"], /* CJK Ideograph */
  ["\u8B91", "\u8B91"], /* CJK Ideograph */
  ["\u8B92", "\u8B92"], /* CJK Ideograph */
  ["\u8B93", "\u8B93"], /* CJK Ideograph */
  ["\u8B94", "\u8B94"], /* CJK Ideograph */
  ["\u8B95", "\u8B95"], /* CJK Ideograph */
  ["\u8B96", "\u8B96"], /* CJK Ideograph */
  ["\u8B97", "\u8B97"], /* CJK Ideograph */
  ["\u8B98", "\u8B98"], /* CJK Ideograph */
  ["\u8B99", "\u8B99"], /* CJK Ideograph */
  ["\u8B9A", "\u8B9A"], /* CJK Ideograph */
  ["\u8B9B", "\u8B9B"], /* CJK Ideograph */
  ["\u8B9C", "\u8B9C"], /* CJK Ideograph */
  ["\u8B9D", "\u8B9D"], /* CJK Ideograph */
  ["\u8B9E", "\u8B9E"], /* CJK Ideograph */
  ["\u8B9F", "\u8B9F"], /* CJK Ideograph */
  ["\u8BA0", "\u8BA0"], /* CJK Ideograph */
  ["\u8BA1", "\u8BA1"], /* CJK Ideograph */
  ["\u8BA2", "\u8BA2"], /* CJK Ideograph */
  ["\u8BA3", "\u8BA3"], /* CJK Ideograph */
  ["\u8BA4", "\u8BA4"], /* CJK Ideograph */
  ["\u8BA5", "\u8BA5"], /* CJK Ideograph */
  ["\u8BA6", "\u8BA6"], /* CJK Ideograph */
  ["\u8BA7", "\u8BA7"], /* CJK Ideograph */
  ["\u8BA8", "\u8BA8"], /* CJK Ideograph */
  ["\u8BA9", "\u8BA9"], /* CJK Ideograph */
  ["\u8BAA", "\u8BAA"], /* CJK Ideograph */
  ["\u8BAB", "\u8BAB"], /* CJK Ideograph */
  ["\u8BAC", "\u8BAC"], /* CJK Ideograph */
  ["\u8BAD", "\u8BAD"], /* CJK Ideograph */
  ["\u8BAE", "\u8BAE"], /* CJK Ideograph */
  ["\u8BAF", "\u8BAF"], /* CJK Ideograph */
  ["\u8BB0", "\u8BB0"], /* CJK Ideograph */
  ["\u8BB1", "\u8BB1"], /* CJK Ideograph */
  ["\u8BB2", "\u8BB2"], /* CJK Ideograph */
  ["\u8BB3", "\u8BB3"], /* CJK Ideograph */
  ["\u8BB4", "\u8BB4"], /* CJK Ideograph */
  ["\u8BB5", "\u8BB5"], /* CJK Ideograph */
  ["\u8BB6", "\u8BB6"], /* CJK Ideograph */
  ["\u8BB7", "\u8BB7"], /* CJK Ideograph */
  ["\u8BB8", "\u8BB8"], /* CJK Ideograph */
  ["\u8BB9", "\u8BB9"], /* CJK Ideograph */
  ["\u8BBA", "\u8BBA"], /* CJK Ideograph */
  ["\u8BBB", "\u8BBB"], /* CJK Ideograph */
  ["\u8BBC", "\u8BBC"], /* CJK Ideograph */
  ["\u8BBD", "\u8BBD"], /* CJK Ideograph */
  ["\u8BBE", "\u8BBE"], /* CJK Ideograph */
  ["\u8BBF", "\u8BBF"], /* CJK Ideograph */
  ["\u8BC0", "\u8BC0"], /* CJK Ideograph */
  ["\u8BC1", "\u8BC1"], /* CJK Ideograph */
  ["\u8BC2", "\u8BC2"], /* CJK Ideograph */
  ["\u8BC3", "\u8BC3"], /* CJK Ideograph */
  ["\u8BC4", "\u8BC4"], /* CJK Ideograph */
  ["\u8BC5", "\u8BC5"], /* CJK Ideograph */
  ["\u8BC6", "\u8BC6"], /* CJK Ideograph */
  ["\u8BC7", "\u8BC7"], /* CJK Ideograph */
  ["\u8BC8", "\u8BC8"], /* CJK Ideograph */
  ["\u8BC9", "\u8BC9"], /* CJK Ideograph */
  ["\u8BCA", "\u8BCA"], /* CJK Ideograph */
  ["\u8BCB", "\u8BCB"], /* CJK Ideograph */
  ["\u8BCC", "\u8BCC"], /* CJK Ideograph */
  ["\u8BCD", "\u8BCD"], /* CJK Ideograph */
  ["\u8BCE", "\u8BCE"], /* CJK Ideograph */
  ["\u8BCF", "\u8BCF"], /* CJK Ideograph */
  ["\u8BD0", "\u8BD0"], /* CJK Ideograph */
  ["\u8BD1", "\u8BD1"], /* CJK Ideograph */
  ["\u8BD2", "\u8BD2"], /* CJK Ideograph */
  ["\u8BD3", "\u8BD3"], /* CJK Ideograph */
  ["\u8BD4", "\u8BD4"], /* CJK Ideograph */
  ["\u8BD5", "\u8BD5"], /* CJK Ideograph */
  ["\u8BD6", "\u8BD6"], /* CJK Ideograph */
  ["\u8BD7", "\u8BD7"], /* CJK Ideograph */
  ["\u8BD8", "\u8BD8"], /* CJK Ideograph */
  ["\u8BD9", "\u8BD9"], /* CJK Ideograph */
  ["\u8BDA", "\u8BDA"], /* CJK Ideograph */
  ["\u8BDB", "\u8BDB"], /* CJK Ideograph */
  ["\u8BDC", "\u8BDC"], /* CJK Ideograph */
  ["\u8BDD", "\u8BDD"], /* CJK Ideograph */
  ["\u8BDE", "\u8BDE"], /* CJK Ideograph */
  ["\u8BDF", "\u8BDF"], /* CJK Ideograph */
  ["\u8BE0", "\u8BE0"], /* CJK Ideograph */
  ["\u8BE1", "\u8BE1"], /* CJK Ideograph */
  ["\u8BE2", "\u8BE2"], /* CJK Ideograph */
  ["\u8BE3", "\u8BE3"], /* CJK Ideograph */
  ["\u8BE4", "\u8BE4"], /* CJK Ideograph */
  ["\u8BE5", "\u8BE5"], /* CJK Ideograph */
  ["\u8BE6", "\u8BE6"], /* CJK Ideograph */
  ["\u8BE7", "\u8BE7"], /* CJK Ideograph */
  ["\u8BE8", "\u8BE8"], /* CJK Ideograph */
  ["\u8BE9", "\u8BE9"], /* CJK Ideograph */
  ["\u8BEA", "\u8BEA"], /* CJK Ideograph */
  ["\u8BEB", "\u8BEB"], /* CJK Ideograph */
  ["\u8BEC", "\u8BEC"], /* CJK Ideograph */
  ["\u8BED", "\u8BED"], /* CJK Ideograph */
  ["\u8BEE", "\u8BEE"], /* CJK Ideograph */
  ["\u8BEF", "\u8BEF"], /* CJK Ideograph */
  ["\u8BF0", "\u8BF0"], /* CJK Ideograph */
  ["\u8BF1", "\u8BF1"], /* CJK Ideograph */
  ["\u8BF2", "\u8BF2"], /* CJK Ideograph */
  ["\u8BF3", "\u8BF3"], /* CJK Ideograph */
  ["\u8BF4", "\u8BF4"], /* CJK Ideograph */
  ["\u8BF5", "\u8BF5"], /* CJK Ideograph */
  ["\u8BF6", "\u8BF6"], /* CJK Ideograph */
  ["\u8BF7", "\u8BF7"], /* CJK Ideograph */
  ["\u8BF8", "\u8BF8"], /* CJK Ideograph */
  ["\u8BF9", "\u8BF9"], /* CJK Ideograph */
  ["\u8BFA", "\u8BFA"], /* CJK Ideograph */
  ["\u8BFB", "\u8BFB"], /* CJK Ideograph */
  ["\u8BFC", "\u8BFC"], /* CJK Ideograph */
  ["\u8BFD", "\u8BFD"], /* CJK Ideograph */
  ["\u8BFE", "\u8BFE"], /* CJK Ideograph */
  ["\u8BFF", "\u8BFF"], /* CJK Ideograph */
  ["\u8C00", "\u8C00"], /* CJK Ideograph */
  ["\u8C01", "\u8C01"], /* CJK Ideograph */
  ["\u8C02", "\u8C02"], /* CJK Ideograph */
  ["\u8C03", "\u8C03"], /* CJK Ideograph */
  ["\u8C04", "\u8C04"], /* CJK Ideograph */
  ["\u8C05", "\u8C05"], /* CJK Ideograph */
  ["\u8C06", "\u8C06"], /* CJK Ideograph */
  ["\u8C07", "\u8C07"], /* CJK Ideograph */
  ["\u8C08", "\u8C08"], /* CJK Ideograph */
  ["\u8C09", "\u8C09"], /* CJK Ideograph */
  ["\u8C0A", "\u8C0A"], /* CJK Ideograph */
  ["\u8C0B", "\u8C0B"], /* CJK Ideograph */
  ["\u8C0C", "\u8C0C"], /* CJK Ideograph */
  ["\u8C0D", "\u8C0D"], /* CJK Ideograph */
  ["\u8C0E", "\u8C0E"], /* CJK Ideograph */
  ["\u8C0F", "\u8C0F"], /* CJK Ideograph */
  ["\u8C10", "\u8C10"], /* CJK Ideograph */
  ["\u8C11", "\u8C11"], /* CJK Ideograph */
  ["\u8C12", "\u8C12"], /* CJK Ideograph */
  ["\u8C13", "\u8C13"], /* CJK Ideograph */
  ["\u8C14", "\u8C14"], /* CJK Ideograph */
  ["\u8C15", "\u8C15"], /* CJK Ideograph */
  ["\u8C16", "\u8C16"], /* CJK Ideograph */
  ["\u8C17", "\u8C17"], /* CJK Ideograph */
  ["\u8C18", "\u8C18"], /* CJK Ideograph */
  ["\u8C19", "\u8C19"], /* CJK Ideograph */
  ["\u8C1A", "\u8C1A"], /* CJK Ideograph */
  ["\u8C1B", "\u8C1B"], /* CJK Ideograph */
  ["\u8C1C", "\u8C1C"], /* CJK Ideograph */
  ["\u8C1D", "\u8C1D"], /* CJK Ideograph */
  ["\u8C1E", "\u8C1E"], /* CJK Ideograph */
  ["\u8C1F", "\u8C1F"], /* CJK Ideograph */
  ["\u8C20", "\u8C20"], /* CJK Ideograph */
  ["\u8C21", "\u8C21"], /* CJK Ideograph */
  ["\u8C22", "\u8C22"], /* CJK Ideograph */
  ["\u8C23", "\u8C23"], /* CJK Ideograph */
  ["\u8C24", "\u8C24"], /* CJK Ideograph */
  ["\u8C25", "\u8C25"], /* CJK Ideograph */
  ["\u8C26", "\u8C26"], /* CJK Ideograph */
  ["\u8C27", "\u8C27"], /* CJK Ideograph */
  ["\u8C28", "\u8C28"], /* CJK Ideograph */
  ["\u8C29", "\u8C29"], /* CJK Ideograph */
  ["\u8C2A", "\u8C2A"], /* CJK Ideograph */
  ["\u8C2B", "\u8C2B"], /* CJK Ideograph */
  ["\u8C2C", "\u8C2C"], /* CJK Ideograph */
  ["\u8C2D", "\u8C2D"], /* CJK Ideograph */
  ["\u8C2E", "\u8C2E"], /* CJK Ideograph */
  ["\u8C2F", "\u8C2F"], /* CJK Ideograph */
  ["\u8C30", "\u8C30"], /* CJK Ideograph */
  ["\u8C31", "\u8C31"], /* CJK Ideograph */
  ["\u8C32", "\u8C32"], /* CJK Ideograph */
  ["\u8C33", "\u8C33"], /* CJK Ideograph */
  ["\u8C34", "\u8C34"], /* CJK Ideograph */
  ["\u8C35", "\u8C35"], /* CJK Ideograph */
  ["\u8C36", "\u8C36"], /* CJK Ideograph */
  ["\u8C37", "\u8C37"], /* CJK Ideograph */
  ["\u8C38", "\u8C38"], /* CJK Ideograph */
  ["\u8C39", "\u8C39"], /* CJK Ideograph */
  ["\u8C3A", "\u8C3A"], /* CJK Ideograph */
  ["\u8C3B", "\u8C3B"], /* CJK Ideograph */
  ["\u8C3C", "\u8C3C"], /* CJK Ideograph */
  ["\u8C3D", "\u8C3D"], /* CJK Ideograph */
  ["\u8C3E", "\u8C3E"], /* CJK Ideograph */
  ["\u8C3F", "\u8C3F"], /* CJK Ideograph */
  ["\u8C40", "\u8C40"], /* CJK Ideograph */
  ["\u8C41", "\u8C41"], /* CJK Ideograph */
  ["\u8C42", "\u8C42"], /* CJK Ideograph */
  ["\u8C43", "\u8C43"], /* CJK Ideograph */
  ["\u8C44", "\u8C44"], /* CJK Ideograph */
  ["\u8C45", "\u8C45"], /* CJK Ideograph */
  ["\u8C46", "\u8C46"], /* CJK Ideograph */
  ["\u8C47", "\u8C47"], /* CJK Ideograph */
  ["\u8C48", "\u8C48"], /* CJK Ideograph */
  ["\u8C49", "\u8C49"], /* CJK Ideograph */
  ["\u8C4A", "\u8C4A"], /* CJK Ideograph */
  ["\u8C4B", "\u8C4B"], /* CJK Ideograph */
  ["\u8C4C", "\u8C4C"], /* CJK Ideograph */
  ["\u8C4D", "\u8C4D"], /* CJK Ideograph */
  ["\u8C4E", "\u8C4E"], /* CJK Ideograph */
  ["\u8C4F", "\u8C4F"], /* CJK Ideograph */
  ["\u8C50", "\u8C50"], /* CJK Ideograph */
  ["\u8C51", "\u8C51"], /* CJK Ideograph */
  ["\u8C52", "\u8C52"], /* CJK Ideograph */
  ["\u8C53", "\u8C53"], /* CJK Ideograph */
  ["\u8C54", "\u8C54"], /* CJK Ideograph */
  ["\u8C55", "\u8C55"], /* CJK Ideograph */
  ["\u8C56", "\u8C56"], /* CJK Ideograph */
  ["\u8C57", "\u8C57"], /* CJK Ideograph */
  ["\u8C58", "\u8C58"], /* CJK Ideograph */
  ["\u8C59", "\u8C59"], /* CJK Ideograph */
  ["\u8C5A", "\u8C5A"], /* CJK Ideograph */
  ["\u8C5B", "\u8C5B"], /* CJK Ideograph */
  ["\u8C5C", "\u8C5C"], /* CJK Ideograph */
  ["\u8C5D", "\u8C5D"], /* CJK Ideograph */
  ["\u8C5E", "\u8C5E"], /* CJK Ideograph */
  ["\u8C5F", "\u8C5F"], /* CJK Ideograph */
  ["\u8C60", "\u8C60"], /* CJK Ideograph */
  ["\u8C61", "\u8C61"], /* CJK Ideograph */
  ["\u8C62", "\u8C62"], /* CJK Ideograph */
  ["\u8C63", "\u8C63"], /* CJK Ideograph */
  ["\u8C64", "\u8C64"], /* CJK Ideograph */
  ["\u8C65", "\u8C65"], /* CJK Ideograph */
  ["\u8C66", "\u8C66"], /* CJK Ideograph */
  ["\u8C67", "\u8C67"], /* CJK Ideograph */
  ["\u8C68", "\u8C68"], /* CJK Ideograph */
  ["\u8C69", "\u8C69"], /* CJK Ideograph */
  ["\u8C6A", "\u8C6A"], /* CJK Ideograph */
  ["\u8C6B", "\u8C6B"], /* CJK Ideograph */
  ["\u8C6C", "\u8C6C"], /* CJK Ideograph */
  ["\u8C6D", "\u8C6D"], /* CJK Ideograph */
  ["\u8C6E", "\u8C6E"], /* CJK Ideograph */
  ["\u8C6F", "\u8C6F"], /* CJK Ideograph */
  ["\u8C70", "\u8C70"], /* CJK Ideograph */
  ["\u8C71", "\u8C71"], /* CJK Ideograph */
  ["\u8C72", "\u8C72"], /* CJK Ideograph */
  ["\u8C73", "\u8C73"], /* CJK Ideograph */
  ["\u8C74", "\u8C74"], /* CJK Ideograph */
  ["\u8C75", "\u8C75"], /* CJK Ideograph */
  ["\u8C76", "\u8C76"], /* CJK Ideograph */
  ["\u8C77", "\u8C77"], /* CJK Ideograph */
  ["\u8C78", "\u8C78"], /* CJK Ideograph */
  ["\u8C79", "\u8C79"], /* CJK Ideograph */
  ["\u8C7A", "\u8C7A"], /* CJK Ideograph */
  ["\u8C7B", "\u8C7B"], /* CJK Ideograph */
  ["\u8C7C", "\u8C7C"], /* CJK Ideograph */
  ["\u8C7D", "\u8C7D"], /* CJK Ideograph */
  ["\u8C7E", "\u8C7E"], /* CJK Ideograph */
  ["\u8C7F", "\u8C7F"], /* CJK Ideograph */
  ["\u8C80", "\u8C80"], /* CJK Ideograph */
  ["\u8C81", "\u8C81"], /* CJK Ideograph */
  ["\u8C82", "\u8C82"], /* CJK Ideograph */
  ["\u8C83", "\u8C83"], /* CJK Ideograph */
  ["\u8C84", "\u8C84"], /* CJK Ideograph */
  ["\u8C85", "\u8C85"], /* CJK Ideograph */
  ["\u8C86", "\u8C86"], /* CJK Ideograph */
  ["\u8C87", "\u8C87"], /* CJK Ideograph */
  ["\u8C88", "\u8C88"], /* CJK Ideograph */
  ["\u8C89", "\u8C89"], /* CJK Ideograph */
  ["\u8C8A", "\u8C8A"], /* CJK Ideograph */
  ["\u8C8B", "\u8C8B"], /* CJK Ideograph */
  ["\u8C8C", "\u8C8C"], /* CJK Ideograph */
  ["\u8C8D", "\u8C8D"], /* CJK Ideograph */
  ["\u8C8E", "\u8C8E"], /* CJK Ideograph */
  ["\u8C8F", "\u8C8F"], /* CJK Ideograph */
  ["\u8C90", "\u8C90"], /* CJK Ideograph */
  ["\u8C91", "\u8C91"], /* CJK Ideograph */
  ["\u8C92", "\u8C92"], /* CJK Ideograph */
  ["\u8C93", "\u8C93"], /* CJK Ideograph */
  ["\u8C94", "\u8C94"], /* CJK Ideograph */
  ["\u8C95", "\u8C95"], /* CJK Ideograph */
  ["\u8C96", "\u8C96"], /* CJK Ideograph */
  ["\u8C97", "\u8C97"], /* CJK Ideograph */
  ["\u8C98", "\u8C98"], /* CJK Ideograph */
  ["\u8C99", "\u8C99"], /* CJK Ideograph */
  ["\u8C9A", "\u8C9A"], /* CJK Ideograph */
  ["\u8C9B", "\u8C9B"], /* CJK Ideograph */
  ["\u8C9C", "\u8C9C"], /* CJK Ideograph */
  ["\u8C9D", "\u8C9D"], /* CJK Ideograph */
  ["\u8C9E", "\u8C9E"], /* CJK Ideograph */
  ["\u8C9F", "\u8C9F"], /* CJK Ideograph */
  ["\u8CA0", "\u8CA0"], /* CJK Ideograph */
  ["\u8CA1", "\u8CA1"], /* CJK Ideograph */
  ["\u8CA2", "\u8CA2"], /* CJK Ideograph */
  ["\u8CA3", "\u8CA3"], /* CJK Ideograph */
  ["\u8CA4", "\u8CA4"], /* CJK Ideograph */
  ["\u8CA5", "\u8CA5"], /* CJK Ideograph */
  ["\u8CA6", "\u8CA6"], /* CJK Ideograph */
  ["\u8CA7", "\u8CA7"], /* CJK Ideograph */
  ["\u8CA8", "\u8CA8"], /* CJK Ideograph */
  ["\u8CA9", "\u8CA9"], /* CJK Ideograph */
  ["\u8CAA", "\u8CAA"], /* CJK Ideograph */
  ["\u8CAB", "\u8CAB"], /* CJK Ideograph */
  ["\u8CAC", "\u8CAC"], /* CJK Ideograph */
  ["\u8CAD", "\u8CAD"], /* CJK Ideograph */
  ["\u8CAE", "\u8CAE"], /* CJK Ideograph */
  ["\u8CAF", "\u8CAF"], /* CJK Ideograph */
  ["\u8CB0", "\u8CB0"], /* CJK Ideograph */
  ["\u8CB1", "\u8CB1"], /* CJK Ideograph */
  ["\u8CB2", "\u8CB2"], /* CJK Ideograph */
  ["\u8CB3", "\u8CB3"], /* CJK Ideograph */
  ["\u8CB4", "\u8CB4"], /* CJK Ideograph */
  ["\u8CB5", "\u8CB5"], /* CJK Ideograph */
  ["\u8CB6", "\u8CB6"], /* CJK Ideograph */
  ["\u8CB7", "\u8CB7"], /* CJK Ideograph */
  ["\u8CB8", "\u8CB8"], /* CJK Ideograph */
  ["\u8CB9", "\u8CB9"], /* CJK Ideograph */
  ["\u8CBA", "\u8CBA"], /* CJK Ideograph */
  ["\u8CBB", "\u8CBB"], /* CJK Ideograph */
  ["\u8CBC", "\u8CBC"], /* CJK Ideograph */
  ["\u8CBD", "\u8CBD"], /* CJK Ideograph */
  ["\u8CBE", "\u8CBE"], /* CJK Ideograph */
  ["\u8CBF", "\u8CBF"], /* CJK Ideograph */
  ["\u8CC0", "\u8CC0"], /* CJK Ideograph */
  ["\u8CC1", "\u8CC1"], /* CJK Ideograph */
  ["\u8CC2", "\u8CC2"], /* CJK Ideograph */
  ["\u8CC3", "\u8CC3"], /* CJK Ideograph */
  ["\u8CC4", "\u8CC4"], /* CJK Ideograph */
  ["\u8CC5", "\u8CC5"], /* CJK Ideograph */
  ["\u8CC6", "\u8CC6"], /* CJK Ideograph */
  ["\u8CC7", "\u8CC7"], /* CJK Ideograph */
  ["\u8CC8", "\u8CC8"], /* CJK Ideograph */
  ["\u8CC9", "\u8CC9"], /* CJK Ideograph */
  ["\u8CCA", "\u8CCA"], /* CJK Ideograph */
  ["\u8CCB", "\u8CCB"], /* CJK Ideograph */
  ["\u8CCC", "\u8CCC"], /* CJK Ideograph */
  ["\u8CCD", "\u8CCD"], /* CJK Ideograph */
  ["\u8CCE", "\u8CCE"], /* CJK Ideograph */
  ["\u8CCF", "\u8CCF"], /* CJK Ideograph */
  ["\u8CD0", "\u8CD0"], /* CJK Ideograph */
  ["\u8CD1", "\u8CD1"], /* CJK Ideograph */
  ["\u8CD2", "\u8CD2"], /* CJK Ideograph */
  ["\u8CD3", "\u8CD3"], /* CJK Ideograph */
  ["\u8CD4", "\u8CD4"], /* CJK Ideograph */
  ["\u8CD5", "\u8CD5"], /* CJK Ideograph */
  ["\u8CD6", "\u8CD6"], /* CJK Ideograph */
  ["\u8CD7", "\u8CD7"], /* CJK Ideograph */
  ["\u8CD8", "\u8CD8"], /* CJK Ideograph */
  ["\u8CD9", "\u8CD9"], /* CJK Ideograph */
  ["\u8CDA", "\u8CDA"], /* CJK Ideograph */
  ["\u8CDB", "\u8CDB"], /* CJK Ideograph */
  ["\u8CDC", "\u8CDC"], /* CJK Ideograph */
  ["\u8CDD", "\u8CDD"], /* CJK Ideograph */
  ["\u8CDE", "\u8CDE"], /* CJK Ideograph */
  ["\u8CDF", "\u8CDF"], /* CJK Ideograph */
  ["\u8CE0", "\u8CE0"], /* CJK Ideograph */
  ["\u8CE1", "\u8CE1"], /* CJK Ideograph */
  ["\u8CE2", "\u8CE2"], /* CJK Ideograph */
  ["\u8CE3", "\u8CE3"], /* CJK Ideograph */
  ["\u8CE4", "\u8CE4"], /* CJK Ideograph */
  ["\u8CE5", "\u8CE5"], /* CJK Ideograph */
  ["\u8CE6", "\u8CE6"], /* CJK Ideograph */
  ["\u8CE7", "\u8CE7"], /* CJK Ideograph */
  ["\u8CE8", "\u8CE8"], /* CJK Ideograph */
  ["\u8CE9", "\u8CE9"], /* CJK Ideograph */
  ["\u8CEA", "\u8CEA"], /* CJK Ideograph */
  ["\u8CEB", "\u8CEB"], /* CJK Ideograph */
  ["\u8CEC", "\u8CEC"], /* CJK Ideograph */
  ["\u8CED", "\u8CED"], /* CJK Ideograph */
  ["\u8CEE", "\u8CEE"], /* CJK Ideograph */
  ["\u8CEF", "\u8CEF"], /* CJK Ideograph */
  ["\u8CF0", "\u8CF0"], /* CJK Ideograph */
  ["\u8CF1", "\u8CF1"], /* CJK Ideograph */
  ["\u8CF2", "\u8CF2"], /* CJK Ideograph */
  ["\u8CF3", "\u8CF3"], /* CJK Ideograph */
  ["\u8CF4", "\u8CF4"], /* CJK Ideograph */
  ["\u8CF5", "\u8CF5"], /* CJK Ideograph */
  ["\u8CF6", "\u8CF6"], /* CJK Ideograph */
  ["\u8CF7", "\u8CF7"], /* CJK Ideograph */
  ["\u8CF8", "\u8CF8"], /* CJK Ideograph */
  ["\u8CF9", "\u8CF9"], /* CJK Ideograph */
  ["\u8CFA", "\u8CFA"], /* CJK Ideograph */
  ["\u8CFB", "\u8CFB"], /* CJK Ideograph */
  ["\u8CFC", "\u8CFC"], /* CJK Ideograph */
  ["\u8CFD", "\u8CFD"], /* CJK Ideograph */
  ["\u8CFE", "\u8CFE"], /* CJK Ideograph */
  ["\u8CFF", "\u8CFF"], /* CJK Ideograph */
  ["\u8D00", "\u8D00"], /* CJK Ideograph */
  ["\u8D01", "\u8D01"], /* CJK Ideograph */
  ["\u8D02", "\u8D02"], /* CJK Ideograph */
  ["\u8D03", "\u8D03"], /* CJK Ideograph */
  ["\u8D04", "\u8D04"], /* CJK Ideograph */
  ["\u8D05", "\u8D05"], /* CJK Ideograph */
  ["\u8D06", "\u8D06"], /* CJK Ideograph */
  ["\u8D07", "\u8D07"], /* CJK Ideograph */
  ["\u8D08", "\u8D08"], /* CJK Ideograph */
  ["\u8D09", "\u8D09"], /* CJK Ideograph */
  ["\u8D0A", "\u8D0A"], /* CJK Ideograph */
  ["\u8D0B", "\u8D0B"], /* CJK Ideograph */
  ["\u8D0C", "\u8D0C"], /* CJK Ideograph */
  ["\u8D0D", "\u8D0D"], /* CJK Ideograph */
  ["\u8D0E", "\u8D0E"], /* CJK Ideograph */
  ["\u8D0F", "\u8D0F"], /* CJK Ideograph */
  ["\u8D10", "\u8D10"], /* CJK Ideograph */
  ["\u8D11", "\u8D11"], /* CJK Ideograph */
  ["\u8D12", "\u8D12"], /* CJK Ideograph */
  ["\u8D13", "\u8D13"], /* CJK Ideograph */
  ["\u8D14", "\u8D14"], /* CJK Ideograph */
  ["\u8D15", "\u8D15"], /* CJK Ideograph */
  ["\u8D16", "\u8D16"], /* CJK Ideograph */
  ["\u8D17", "\u8D17"], /* CJK Ideograph */
  ["\u8D18", "\u8D18"], /* CJK Ideograph */
  ["\u8D19", "\u8D19"], /* CJK Ideograph */
  ["\u8D1A", "\u8D1A"], /* CJK Ideograph */
  ["\u8D1B", "\u8D1B"], /* CJK Ideograph */
  ["\u8D1C", "\u8D1C"], /* CJK Ideograph */
  ["\u8D1D", "\u8D1D"], /* CJK Ideograph */
  ["\u8D1E", "\u8D1E"], /* CJK Ideograph */
  ["\u8D1F", "\u8D1F"], /* CJK Ideograph */
  ["\u8D20", "\u8D20"], /* CJK Ideograph */
  ["\u8D21", "\u8D21"], /* CJK Ideograph */
  ["\u8D22", "\u8D22"], /* CJK Ideograph */
  ["\u8D23", "\u8D23"], /* CJK Ideograph */
  ["\u8D24", "\u8D24"], /* CJK Ideograph */
  ["\u8D25", "\u8D25"], /* CJK Ideograph */
  ["\u8D26", "\u8D26"], /* CJK Ideograph */
  ["\u8D27", "\u8D27"], /* CJK Ideograph */
  ["\u8D28", "\u8D28"], /* CJK Ideograph */
  ["\u8D29", "\u8D29"], /* CJK Ideograph */
  ["\u8D2A", "\u8D2A"], /* CJK Ideograph */
  ["\u8D2B", "\u8D2B"], /* CJK Ideograph */
  ["\u8D2C", "\u8D2C"], /* CJK Ideograph */
  ["\u8D2D", "\u8D2D"], /* CJK Ideograph */
  ["\u8D2E", "\u8D2E"], /* CJK Ideograph */
  ["\u8D2F", "\u8D2F"], /* CJK Ideograph */
  ["\u8D30", "\u8D30"], /* CJK Ideograph */
  ["\u8D31", "\u8D31"], /* CJK Ideograph */
  ["\u8D32", "\u8D32"], /* CJK Ideograph */
  ["\u8D33", "\u8D33"], /* CJK Ideograph */
  ["\u8D34", "\u8D34"], /* CJK Ideograph */
  ["\u8D35", "\u8D35"], /* CJK Ideograph */
  ["\u8D36", "\u8D36"], /* CJK Ideograph */
  ["\u8D37", "\u8D37"], /* CJK Ideograph */
  ["\u8D38", "\u8D38"], /* CJK Ideograph */
  ["\u8D39", "\u8D39"], /* CJK Ideograph */
  ["\u8D3A", "\u8D3A"], /* CJK Ideograph */
  ["\u8D3B", "\u8D3B"], /* CJK Ideograph */
  ["\u8D3C", "\u8D3C"], /* CJK Ideograph */
  ["\u8D3D", "\u8D3D"], /* CJK Ideograph */
  ["\u8D3E", "\u8D3E"], /* CJK Ideograph */
  ["\u8D3F", "\u8D3F"], /* CJK Ideograph */
  ["\u8D40", "\u8D40"], /* CJK Ideograph */
  ["\u8D41", "\u8D41"], /* CJK Ideograph */
  ["\u8D42", "\u8D42"], /* CJK Ideograph */
  ["\u8D43", "\u8D43"], /* CJK Ideograph */
  ["\u8D44", "\u8D44"], /* CJK Ideograph */
  ["\u8D45", "\u8D45"], /* CJK Ideograph */
  ["\u8D46", "\u8D46"], /* CJK Ideograph */
  ["\u8D47", "\u8D47"], /* CJK Ideograph */
  ["\u8D48", "\u8D48"], /* CJK Ideograph */
  ["\u8D49", "\u8D49"], /* CJK Ideograph */
  ["\u8D4A", "\u8D4A"], /* CJK Ideograph */
  ["\u8D4B", "\u8D4B"], /* CJK Ideograph */
  ["\u8D4C", "\u8D4C"], /* CJK Ideograph */
  ["\u8D4D", "\u8D4D"], /* CJK Ideograph */
  ["\u8D4E", "\u8D4E"], /* CJK Ideograph */
  ["\u8D4F", "\u8D4F"], /* CJK Ideograph */
  ["\u8D50", "\u8D50"], /* CJK Ideograph */
  ["\u8D51", "\u8D51"], /* CJK Ideograph */
  ["\u8D52", "\u8D52"], /* CJK Ideograph */
  ["\u8D53", "\u8D53"], /* CJK Ideograph */
  ["\u8D54", "\u8D54"], /* CJK Ideograph */
  ["\u8D55", "\u8D55"], /* CJK Ideograph */
  ["\u8D56", "\u8D56"], /* CJK Ideograph */
  ["\u8D57", "\u8D57"], /* CJK Ideograph */
  ["\u8D58", "\u8D58"], /* CJK Ideograph */
  ["\u8D59", "\u8D59"], /* CJK Ideograph */
  ["\u8D5A", "\u8D5A"], /* CJK Ideograph */
  ["\u8D5B", "\u8D5B"], /* CJK Ideograph */
  ["\u8D5C", "\u8D5C"], /* CJK Ideograph */
  ["\u8D5D", "\u8D5D"], /* CJK Ideograph */
  ["\u8D5E", "\u8D5E"], /* CJK Ideograph */
  ["\u8D5F", "\u8D5F"], /* CJK Ideograph */
  ["\u8D60", "\u8D60"], /* CJK Ideograph */
  ["\u8D61", "\u8D61"], /* CJK Ideograph */
  ["\u8D62", "\u8D62"], /* CJK Ideograph */
  ["\u8D63", "\u8D63"], /* CJK Ideograph */
  ["\u8D64", "\u8D64"], /* CJK Ideograph */
  ["\u8D65", "\u8D65"], /* CJK Ideograph */
  ["\u8D66", "\u8D66"], /* CJK Ideograph */
  ["\u8D67", "\u8D67"], /* CJK Ideograph */
  ["\u8D68", "\u8D68"], /* CJK Ideograph */
  ["\u8D69", "\u8D69"], /* CJK Ideograph */
  ["\u8D6A", "\u8D6A"], /* CJK Ideograph */
  ["\u8D6B", "\u8D6B"], /* CJK Ideograph */
  ["\u8D6C", "\u8D6C"], /* CJK Ideograph */
  ["\u8D6D", "\u8D6D"], /* CJK Ideograph */
  ["\u8D6E", "\u8D6E"], /* CJK Ideograph */
  ["\u8D6F", "\u8D6F"], /* CJK Ideograph */
  ["\u8D70", "\u8D70"], /* CJK Ideograph */
  ["\u8D71", "\u8D71"], /* CJK Ideograph */
  ["\u8D72", "\u8D72"], /* CJK Ideograph */
  ["\u8D73", "\u8D73"], /* CJK Ideograph */
  ["\u8D74", "\u8D74"], /* CJK Ideograph */
  ["\u8D75", "\u8D75"], /* CJK Ideograph */
  ["\u8D76", "\u8D76"], /* CJK Ideograph */
  ["\u8D77", "\u8D77"], /* CJK Ideograph */
  ["\u8D78", "\u8D78"], /* CJK Ideograph */
  ["\u8D79", "\u8D79"], /* CJK Ideograph */
  ["\u8D7A", "\u8D7A"], /* CJK Ideograph */
  ["\u8D7B", "\u8D7B"], /* CJK Ideograph */
  ["\u8D7C", "\u8D7C"], /* CJK Ideograph */
  ["\u8D7D", "\u8D7D"], /* CJK Ideograph */
  ["\u8D7E", "\u8D7E"], /* CJK Ideograph */
  ["\u8D7F", "\u8D7F"], /* CJK Ideograph */
  ["\u8D80", "\u8D80"], /* CJK Ideograph */
  ["\u8D81", "\u8D81"], /* CJK Ideograph */
  ["\u8D82", "\u8D82"], /* CJK Ideograph */
  ["\u8D83", "\u8D83"], /* CJK Ideograph */
  ["\u8D84", "\u8D84"], /* CJK Ideograph */
  ["\u8D85", "\u8D85"], /* CJK Ideograph */
  ["\u8D86", "\u8D86"], /* CJK Ideograph */
  ["\u8D87", "\u8D87"], /* CJK Ideograph */
  ["\u8D88", "\u8D88"], /* CJK Ideograph */
  ["\u8D89", "\u8D89"], /* CJK Ideograph */
  ["\u8D8A", "\u8D8A"], /* CJK Ideograph */
  ["\u8D8B", "\u8D8B"], /* CJK Ideograph */
  ["\u8D8C", "\u8D8C"], /* CJK Ideograph */
  ["\u8D8D", "\u8D8D"], /* CJK Ideograph */
  ["\u8D8E", "\u8D8E"], /* CJK Ideograph */
  ["\u8D8F", "\u8D8F"], /* CJK Ideograph */
  ["\u8D90", "\u8D90"], /* CJK Ideograph */
  ["\u8D91", "\u8D91"], /* CJK Ideograph */
  ["\u8D92", "\u8D92"], /* CJK Ideograph */
  ["\u8D93", "\u8D93"], /* CJK Ideograph */
  ["\u8D94", "\u8D94"], /* CJK Ideograph */
  ["\u8D95", "\u8D95"], /* CJK Ideograph */
  ["\u8D96", "\u8D96"], /* CJK Ideograph */
  ["\u8D97", "\u8D97"], /* CJK Ideograph */
  ["\u8D98", "\u8D98"], /* CJK Ideograph */
  ["\u8D99", "\u8D99"], /* CJK Ideograph */
  ["\u8D9A", "\u8D9A"], /* CJK Ideograph */
  ["\u8D9B", "\u8D9B"], /* CJK Ideograph */
  ["\u8D9C", "\u8D9C"], /* CJK Ideograph */
  ["\u8D9D", "\u8D9D"], /* CJK Ideograph */
  ["\u8D9E", "\u8D9E"], /* CJK Ideograph */
  ["\u8D9F", "\u8D9F"], /* CJK Ideograph */
  ["\u8DA0", "\u8DA0"], /* CJK Ideograph */
  ["\u8DA1", "\u8DA1"], /* CJK Ideograph */
  ["\u8DA2", "\u8DA2"], /* CJK Ideograph */
  ["\u8DA3", "\u8DA3"], /* CJK Ideograph */
  ["\u8DA4", "\u8DA4"], /* CJK Ideograph */
  ["\u8DA5", "\u8DA5"], /* CJK Ideograph */
  ["\u8DA6", "\u8DA6"], /* CJK Ideograph */
  ["\u8DA7", "\u8DA7"], /* CJK Ideograph */
  ["\u8DA8", "\u8DA8"], /* CJK Ideograph */
  ["\u8DA9", "\u8DA9"], /* CJK Ideograph */
  ["\u8DAA", "\u8DAA"], /* CJK Ideograph */
  ["\u8DAB", "\u8DAB"], /* CJK Ideograph */
  ["\u8DAC", "\u8DAC"], /* CJK Ideograph */
  ["\u8DAD", "\u8DAD"], /* CJK Ideograph */
  ["\u8DAE", "\u8DAE"], /* CJK Ideograph */
  ["\u8DAF", "\u8DAF"], /* CJK Ideograph */
  ["\u8DB0", "\u8DB0"], /* CJK Ideograph */
  ["\u8DB1", "\u8DB1"], /* CJK Ideograph */
  ["\u8DB2", "\u8DB2"], /* CJK Ideograph */
  ["\u8DB3", "\u8DB3"], /* CJK Ideograph */
  ["\u8DB4", "\u8DB4"], /* CJK Ideograph */
  ["\u8DB5", "\u8DB5"], /* CJK Ideograph */
  ["\u8DB6", "\u8DB6"], /* CJK Ideograph */
  ["\u8DB7", "\u8DB7"], /* CJK Ideograph */
  ["\u8DB8", "\u8DB8"], /* CJK Ideograph */
  ["\u8DB9", "\u8DB9"], /* CJK Ideograph */
  ["\u8DBA", "\u8DBA"], /* CJK Ideograph */
  ["\u8DBB", "\u8DBB"], /* CJK Ideograph */
  ["\u8DBC", "\u8DBC"], /* CJK Ideograph */
  ["\u8DBD", "\u8DBD"], /* CJK Ideograph */
  ["\u8DBE", "\u8DBE"], /* CJK Ideograph */
  ["\u8DBF", "\u8DBF"], /* CJK Ideograph */
  ["\u8DC0", "\u8DC0"], /* CJK Ideograph */
  ["\u8DC1", "\u8DC1"], /* CJK Ideograph */
  ["\u8DC2", "\u8DC2"], /* CJK Ideograph */
  ["\u8DC3", "\u8DC3"], /* CJK Ideograph */
  ["\u8DC4", "\u8DC4"], /* CJK Ideograph */
  ["\u8DC5", "\u8DC5"], /* CJK Ideograph */
  ["\u8DC6", "\u8DC6"], /* CJK Ideograph */
  ["\u8DC7", "\u8DC7"], /* CJK Ideograph */
  ["\u8DC8", "\u8DC8"], /* CJK Ideograph */
  ["\u8DC9", "\u8DC9"], /* CJK Ideograph */
  ["\u8DCA", "\u8DCA"], /* CJK Ideograph */
  ["\u8DCB", "\u8DCB"], /* CJK Ideograph */
  ["\u8DCC", "\u8DCC"], /* CJK Ideograph */
  ["\u8DCD", "\u8DCD"], /* CJK Ideograph */
  ["\u8DCE", "\u8DCE"], /* CJK Ideograph */
  ["\u8DCF", "\u8DCF"], /* CJK Ideograph */
  ["\u8DD0", "\u8DD0"], /* CJK Ideograph */
  ["\u8DD1", "\u8DD1"], /* CJK Ideograph */
  ["\u8DD2", "\u8DD2"], /* CJK Ideograph */
  ["\u8DD3", "\u8DD3"], /* CJK Ideograph */
  ["\u8DD4", "\u8DD4"], /* CJK Ideograph */
  ["\u8DD5", "\u8DD5"], /* CJK Ideograph */
  ["\u8DD6", "\u8DD6"], /* CJK Ideograph */
  ["\u8DD7", "\u8DD7"], /* CJK Ideograph */
  ["\u8DD8", "\u8DD8"], /* CJK Ideograph */
  ["\u8DD9", "\u8DD9"], /* CJK Ideograph */
  ["\u8DDA", "\u8DDA"], /* CJK Ideograph */
  ["\u8DDB", "\u8DDB"], /* CJK Ideograph */
  ["\u8DDC", "\u8DDC"], /* CJK Ideograph */
  ["\u8DDD", "\u8DDD"], /* CJK Ideograph */
  ["\u8DDE", "\u8DDE"], /* CJK Ideograph */
  ["\u8DDF", "\u8DDF"], /* CJK Ideograph */
  ["\u8DE0", "\u8DE0"], /* CJK Ideograph */
  ["\u8DE1", "\u8DE1"], /* CJK Ideograph */
  ["\u8DE2", "\u8DE2"], /* CJK Ideograph */
  ["\u8DE3", "\u8DE3"], /* CJK Ideograph */
  ["\u8DE4", "\u8DE4"], /* CJK Ideograph */
  ["\u8DE5", "\u8DE5"], /* CJK Ideograph */
  ["\u8DE6", "\u8DE6"], /* CJK Ideograph */
  ["\u8DE7", "\u8DE7"], /* CJK Ideograph */
  ["\u8DE8", "\u8DE8"], /* CJK Ideograph */
  ["\u8DE9", "\u8DE9"], /* CJK Ideograph */
  ["\u8DEA", "\u8DEA"], /* CJK Ideograph */
  ["\u8DEB", "\u8DEB"], /* CJK Ideograph */
  ["\u8DEC", "\u8DEC"], /* CJK Ideograph */
  ["\u8DED", "\u8DED"], /* CJK Ideograph */
  ["\u8DEE", "\u8DEE"], /* CJK Ideograph */
  ["\u8DEF", "\u8DEF"], /* CJK Ideograph */
  ["\u8DF0", "\u8DF0"], /* CJK Ideograph */
  ["\u8DF1", "\u8DF1"], /* CJK Ideograph */
  ["\u8DF2", "\u8DF2"], /* CJK Ideograph */
  ["\u8DF3", "\u8DF3"], /* CJK Ideograph */
  ["\u8DF4", "\u8DF4"], /* CJK Ideograph */
  ["\u8DF5", "\u8DF5"], /* CJK Ideograph */
  ["\u8DF6", "\u8DF6"], /* CJK Ideograph */
  ["\u8DF7", "\u8DF7"], /* CJK Ideograph */
  ["\u8DF8", "\u8DF8"], /* CJK Ideograph */
  ["\u8DF9", "\u8DF9"], /* CJK Ideograph */
  ["\u8DFA", "\u8DFA"], /* CJK Ideograph */
  ["\u8DFB", "\u8DFB"], /* CJK Ideograph */
  ["\u8DFC", "\u8DFC"], /* CJK Ideograph */
  ["\u8DFD", "\u8DFD"], /* CJK Ideograph */
  ["\u8DFE", "\u8DFE"], /* CJK Ideograph */
  ["\u8DFF", "\u8DFF"], /* CJK Ideograph */
  ["\u8E00", "\u8E00"], /* CJK Ideograph */
  ["\u8E01", "\u8E01"], /* CJK Ideograph */
  ["\u8E02", "\u8E02"], /* CJK Ideograph */
  ["\u8E03", "\u8E03"], /* CJK Ideograph */
  ["\u8E04", "\u8E04"], /* CJK Ideograph */
  ["\u8E05", "\u8E05"], /* CJK Ideograph */
  ["\u8E06", "\u8E06"], /* CJK Ideograph */
  ["\u8E07", "\u8E07"], /* CJK Ideograph */
  ["\u8E08", "\u8E08"], /* CJK Ideograph */
  ["\u8E09", "\u8E09"], /* CJK Ideograph */
  ["\u8E0A", "\u8E0A"], /* CJK Ideograph */
  ["\u8E0B", "\u8E0B"], /* CJK Ideograph */
  ["\u8E0C", "\u8E0C"], /* CJK Ideograph */
  ["\u8E0D", "\u8E0D"], /* CJK Ideograph */
  ["\u8E0E", "\u8E0E"], /* CJK Ideograph */
  ["\u8E0F", "\u8E0F"], /* CJK Ideograph */
  ["\u8E10", "\u8E10"], /* CJK Ideograph */
  ["\u8E11", "\u8E11"], /* CJK Ideograph */
  ["\u8E12", "\u8E12"], /* CJK Ideograph */
  ["\u8E13", "\u8E13"], /* CJK Ideograph */
  ["\u8E14", "\u8E14"], /* CJK Ideograph */
  ["\u8E15", "\u8E15"], /* CJK Ideograph */
  ["\u8E16", "\u8E16"], /* CJK Ideograph */
  ["\u8E17", "\u8E17"], /* CJK Ideograph */
  ["\u8E18", "\u8E18"], /* CJK Ideograph */
  ["\u8E19", "\u8E19"], /* CJK Ideograph */
  ["\u8E1A", "\u8E1A"], /* CJK Ideograph */
  ["\u8E1B", "\u8E1B"], /* CJK Ideograph */
  ["\u8E1C", "\u8E1C"], /* CJK Ideograph */
  ["\u8E1D", "\u8E1D"], /* CJK Ideograph */
  ["\u8E1E", "\u8E1E"], /* CJK Ideograph */
  ["\u8E1F", "\u8E1F"], /* CJK Ideograph */
  ["\u8E20", "\u8E20"], /* CJK Ideograph */
  ["\u8E21", "\u8E21"], /* CJK Ideograph */
  ["\u8E22", "\u8E22"], /* CJK Ideograph */
  ["\u8E23", "\u8E23"], /* CJK Ideograph */
  ["\u8E24", "\u8E24"], /* CJK Ideograph */
  ["\u8E25", "\u8E25"], /* CJK Ideograph */
  ["\u8E26", "\u8E26"], /* CJK Ideograph */
  ["\u8E27", "\u8E27"], /* CJK Ideograph */
  ["\u8E28", "\u8E28"], /* CJK Ideograph */
  ["\u8E29", "\u8E29"], /* CJK Ideograph */
  ["\u8E2A", "\u8E2A"], /* CJK Ideograph */
  ["\u8E2B", "\u8E2B"], /* CJK Ideograph */
  ["\u8E2C", "\u8E2C"], /* CJK Ideograph */
  ["\u8E2D", "\u8E2D"], /* CJK Ideograph */
  ["\u8E2E", "\u8E2E"], /* CJK Ideograph */
  ["\u8E2F", "\u8E2F"], /* CJK Ideograph */
  ["\u8E30", "\u8E30"], /* CJK Ideograph */
  ["\u8E31", "\u8E31"], /* CJK Ideograph */
  ["\u8E32", "\u8E32"], /* CJK Ideograph */
  ["\u8E33", "\u8E33"], /* CJK Ideograph */
  ["\u8E34", "\u8E34"], /* CJK Ideograph */
  ["\u8E35", "\u8E35"], /* CJK Ideograph */
  ["\u8E36", "\u8E36"], /* CJK Ideograph */
  ["\u8E37", "\u8E37"], /* CJK Ideograph */
  ["\u8E38", "\u8E38"], /* CJK Ideograph */
  ["\u8E39", "\u8E39"], /* CJK Ideograph */
  ["\u8E3A", "\u8E3A"], /* CJK Ideograph */
  ["\u8E3B", "\u8E3B"], /* CJK Ideograph */
  ["\u8E3C", "\u8E3C"], /* CJK Ideograph */
  ["\u8E3D", "\u8E3D"], /* CJK Ideograph */
  ["\u8E3E", "\u8E3E"], /* CJK Ideograph */
  ["\u8E3F", "\u8E3F"], /* CJK Ideograph */
  ["\u8E40", "\u8E40"], /* CJK Ideograph */
  ["\u8E41", "\u8E41"], /* CJK Ideograph */
  ["\u8E42", "\u8E42"], /* CJK Ideograph */
  ["\u8E43", "\u8E43"], /* CJK Ideograph */
  ["\u8E44", "\u8E44"], /* CJK Ideograph */
  ["\u8E45", "\u8E45"], /* CJK Ideograph */
  ["\u8E46", "\u8E46"], /* CJK Ideograph */
  ["\u8E47", "\u8E47"], /* CJK Ideograph */
  ["\u8E48", "\u8E48"], /* CJK Ideograph */
  ["\u8E49", "\u8E49"], /* CJK Ideograph */
  ["\u8E4A", "\u8E4A"], /* CJK Ideograph */
  ["\u8E4B", "\u8E4B"], /* CJK Ideograph */
  ["\u8E4C", "\u8E4C"], /* CJK Ideograph */
  ["\u8E4D", "\u8E4D"], /* CJK Ideograph */
  ["\u8E4E", "\u8E4E"], /* CJK Ideograph */
  ["\u8E4F", "\u8E4F"], /* CJK Ideograph */
  ["\u8E50", "\u8E50"], /* CJK Ideograph */
  ["\u8E51", "\u8E51"], /* CJK Ideograph */
  ["\u8E52", "\u8E52"], /* CJK Ideograph */
  ["\u8E53", "\u8E53"], /* CJK Ideograph */
  ["\u8E54", "\u8E54"], /* CJK Ideograph */
  ["\u8E55", "\u8E55"], /* CJK Ideograph */
  ["\u8E56", "\u8E56"], /* CJK Ideograph */
  ["\u8E57", "\u8E57"], /* CJK Ideograph */
  ["\u8E58", "\u8E58"], /* CJK Ideograph */
  ["\u8E59", "\u8E59"], /* CJK Ideograph */
  ["\u8E5A", "\u8E5A"], /* CJK Ideograph */
  ["\u8E5B", "\u8E5B"], /* CJK Ideograph */
  ["\u8E5C", "\u8E5C"], /* CJK Ideograph */
  ["\u8E5D", "\u8E5D"], /* CJK Ideograph */
  ["\u8E5E", "\u8E5E"], /* CJK Ideograph */
  ["\u8E5F", "\u8E5F"], /* CJK Ideograph */
  ["\u8E60", "\u8E60"], /* CJK Ideograph */
  ["\u8E61", "\u8E61"], /* CJK Ideograph */
  ["\u8E62", "\u8E62"], /* CJK Ideograph */
  ["\u8E63", "\u8E63"], /* CJK Ideograph */
  ["\u8E64", "\u8E64"], /* CJK Ideograph */
  ["\u8E65", "\u8E65"], /* CJK Ideograph */
  ["\u8E66", "\u8E66"], /* CJK Ideograph */
  ["\u8E67", "\u8E67"], /* CJK Ideograph */
  ["\u8E68", "\u8E68"], /* CJK Ideograph */
  ["\u8E69", "\u8E69"], /* CJK Ideograph */
  ["\u8E6A", "\u8E6A"], /* CJK Ideograph */
  ["\u8E6B", "\u8E6B"], /* CJK Ideograph */
  ["\u8E6C", "\u8E6C"], /* CJK Ideograph */
  ["\u8E6D", "\u8E6D"], /* CJK Ideograph */
  ["\u8E6E", "\u8E6E"], /* CJK Ideograph */
  ["\u8E6F", "\u8E6F"], /* CJK Ideograph */
  ["\u8E70", "\u8E70"], /* CJK Ideograph */
  ["\u8E71", "\u8E71"], /* CJK Ideograph */
  ["\u8E72", "\u8E72"], /* CJK Ideograph */
  ["\u8E73", "\u8E73"], /* CJK Ideograph */
  ["\u8E74", "\u8E74"], /* CJK Ideograph */
  ["\u8E75", "\u8E75"], /* CJK Ideograph */
  ["\u8E76", "\u8E76"], /* CJK Ideograph */
  ["\u8E77", "\u8E77"], /* CJK Ideograph */
  ["\u8E78", "\u8E78"], /* CJK Ideograph */
  ["\u8E79", "\u8E79"], /* CJK Ideograph */
  ["\u8E7A", "\u8E7A"], /* CJK Ideograph */
  ["\u8E7B", "\u8E7B"], /* CJK Ideograph */
  ["\u8E7C", "\u8E7C"], /* CJK Ideograph */
  ["\u8E7D", "\u8E7D"], /* CJK Ideograph */
  ["\u8E7E", "\u8E7E"], /* CJK Ideograph */
  ["\u8E7F", "\u8E7F"], /* CJK Ideograph */
  ["\u8E80", "\u8E80"], /* CJK Ideograph */
  ["\u8E81", "\u8E81"], /* CJK Ideograph */
  ["\u8E82", "\u8E82"], /* CJK Ideograph */
  ["\u8E83", "\u8E83"], /* CJK Ideograph */
  ["\u8E84", "\u8E84"], /* CJK Ideograph */
  ["\u8E85", "\u8E85"], /* CJK Ideograph */
  ["\u8E86", "\u8E86"], /* CJK Ideograph */
  ["\u8E87", "\u8E87"], /* CJK Ideograph */
  ["\u8E88", "\u8E88"], /* CJK Ideograph */
  ["\u8E89", "\u8E89"], /* CJK Ideograph */
  ["\u8E8A", "\u8E8A"], /* CJK Ideograph */
  ["\u8E8B", "\u8E8B"], /* CJK Ideograph */
  ["\u8E8C", "\u8E8C"], /* CJK Ideograph */
  ["\u8E8D", "\u8E8D"], /* CJK Ideograph */
  ["\u8E8E", "\u8E8E"], /* CJK Ideograph */
  ["\u8E8F", "\u8E8F"], /* CJK Ideograph */
  ["\u8E90", "\u8E90"], /* CJK Ideograph */
  ["\u8E91", "\u8E91"], /* CJK Ideograph */
  ["\u8E92", "\u8E92"], /* CJK Ideograph */
  ["\u8E93", "\u8E93"], /* CJK Ideograph */
  ["\u8E94", "\u8E94"], /* CJK Ideograph */
  ["\u8E95", "\u8E95"], /* CJK Ideograph */
  ["\u8E96", "\u8E96"], /* CJK Ideograph */
  ["\u8E97", "\u8E97"], /* CJK Ideograph */
  ["\u8E98", "\u8E98"], /* CJK Ideograph */
  ["\u8E99", "\u8E99"], /* CJK Ideograph */
  ["\u8E9A", "\u8E9A"], /* CJK Ideograph */
  ["\u8E9B", "\u8E9B"], /* CJK Ideograph */
  ["\u8E9C", "\u8E9C"], /* CJK Ideograph */
  ["\u8E9D", "\u8E9D"], /* CJK Ideograph */
  ["\u8E9E", "\u8E9E"], /* CJK Ideograph */
  ["\u8E9F", "\u8E9F"], /* CJK Ideograph */
  ["\u8EA0", "\u8EA0"], /* CJK Ideograph */
  ["\u8EA1", "\u8EA1"], /* CJK Ideograph */
  ["\u8EA2", "\u8EA2"], /* CJK Ideograph */
  ["\u8EA3", "\u8EA3"], /* CJK Ideograph */
  ["\u8EA4", "\u8EA4"], /* CJK Ideograph */
  ["\u8EA5", "\u8EA5"], /* CJK Ideograph */
  ["\u8EA6", "\u8EA6"], /* CJK Ideograph */
  ["\u8EA7", "\u8EA7"], /* CJK Ideograph */
  ["\u8EA8", "\u8EA8"], /* CJK Ideograph */
  ["\u8EA9", "\u8EA9"], /* CJK Ideograph */
  ["\u8EAA", "\u8EAA"], /* CJK Ideograph */
  ["\u8EAB", "\u8EAB"], /* CJK Ideograph */
  ["\u8EAC", "\u8EAC"], /* CJK Ideograph */
  ["\u8EAD", "\u8EAD"], /* CJK Ideograph */
  ["\u8EAE", "\u8EAE"], /* CJK Ideograph */
  ["\u8EAF", "\u8EAF"], /* CJK Ideograph */
  ["\u8EB0", "\u8EB0"], /* CJK Ideograph */
  ["\u8EB1", "\u8EB1"], /* CJK Ideograph */
  ["\u8EB2", "\u8EB2"], /* CJK Ideograph */
  ["\u8EB3", "\u8EB3"], /* CJK Ideograph */
  ["\u8EB4", "\u8EB4"], /* CJK Ideograph */
  ["\u8EB5", "\u8EB5"], /* CJK Ideograph */
  ["\u8EB6", "\u8EB6"], /* CJK Ideograph */
  ["\u8EB7", "\u8EB7"], /* CJK Ideograph */
  ["\u8EB8", "\u8EB8"], /* CJK Ideograph */
  ["\u8EB9", "\u8EB9"], /* CJK Ideograph */
  ["\u8EBA", "\u8EBA"], /* CJK Ideograph */
  ["\u8EBB", "\u8EBB"], /* CJK Ideograph */
  ["\u8EBC", "\u8EBC"], /* CJK Ideograph */
  ["\u8EBD", "\u8EBD"], /* CJK Ideograph */
  ["\u8EBE", "\u8EBE"], /* CJK Ideograph */
  ["\u8EBF", "\u8EBF"], /* CJK Ideograph */
  ["\u8EC0", "\u8EC0"], /* CJK Ideograph */
  ["\u8EC1", "\u8EC1"], /* CJK Ideograph */
  ["\u8EC2", "\u8EC2"], /* CJK Ideograph */
  ["\u8EC3", "\u8EC3"], /* CJK Ideograph */
  ["\u8EC4", "\u8EC4"], /* CJK Ideograph */
  ["\u8EC5", "\u8EC5"], /* CJK Ideograph */
  ["\u8EC6", "\u8EC6"], /* CJK Ideograph */
  ["\u8EC7", "\u8EC7"], /* CJK Ideograph */
  ["\u8EC8", "\u8EC8"], /* CJK Ideograph */
  ["\u8EC9", "\u8EC9"], /* CJK Ideograph */
  ["\u8ECA", "\u8ECA"], /* CJK Ideograph */
  ["\u8ECB", "\u8ECB"], /* CJK Ideograph */
  ["\u8ECC", "\u8ECC"], /* CJK Ideograph */
  ["\u8ECD", "\u8ECD"], /* CJK Ideograph */
  ["\u8ECE", "\u8ECE"], /* CJK Ideograph */
  ["\u8ECF", "\u8ECF"], /* CJK Ideograph */
  ["\u8ED0", "\u8ED0"], /* CJK Ideograph */
  ["\u8ED1", "\u8ED1"], /* CJK Ideograph */
  ["\u8ED2", "\u8ED2"], /* CJK Ideograph */
  ["\u8ED3", "\u8ED3"], /* CJK Ideograph */
  ["\u8ED4", "\u8ED4"], /* CJK Ideograph */
  ["\u8ED5", "\u8ED5"], /* CJK Ideograph */
  ["\u8ED6", "\u8ED6"], /* CJK Ideograph */
  ["\u8ED7", "\u8ED7"], /* CJK Ideograph */
  ["\u8ED8", "\u8ED8"], /* CJK Ideograph */
  ["\u8ED9", "\u8ED9"], /* CJK Ideograph */
  ["\u8EDA", "\u8EDA"], /* CJK Ideograph */
  ["\u8EDB", "\u8EDB"], /* CJK Ideograph */
  ["\u8EDC", "\u8EDC"], /* CJK Ideograph */
  ["\u8EDD", "\u8EDD"], /* CJK Ideograph */
  ["\u8EDE", "\u8EDE"], /* CJK Ideograph */
  ["\u8EDF", "\u8EDF"], /* CJK Ideograph */
  ["\u8EE0", "\u8EE0"], /* CJK Ideograph */
  ["\u8EE1", "\u8EE1"], /* CJK Ideograph */
  ["\u8EE2", "\u8EE2"], /* CJK Ideograph */
  ["\u8EE3", "\u8EE3"], /* CJK Ideograph */
  ["\u8EE4", "\u8EE4"], /* CJK Ideograph */
  ["\u8EE5", "\u8EE5"], /* CJK Ideograph */
  ["\u8EE6", "\u8EE6"], /* CJK Ideograph */
  ["\u8EE7", "\u8EE7"], /* CJK Ideograph */
  ["\u8EE8", "\u8EE8"], /* CJK Ideograph */
  ["\u8EE9", "\u8EE9"], /* CJK Ideograph */
  ["\u8EEA", "\u8EEA"], /* CJK Ideograph */
  ["\u8EEB", "\u8EEB"], /* CJK Ideograph */
  ["\u8EEC", "\u8EEC"], /* CJK Ideograph */
  ["\u8EED", "\u8EED"], /* CJK Ideograph */
  ["\u8EEE", "\u8EEE"], /* CJK Ideograph */
  ["\u8EEF", "\u8EEF"], /* CJK Ideograph */
  ["\u8EF0", "\u8EF0"], /* CJK Ideograph */
  ["\u8EF1", "\u8EF1"], /* CJK Ideograph */
  ["\u8EF2", "\u8EF2"], /* CJK Ideograph */
  ["\u8EF3", "\u8EF3"], /* CJK Ideograph */
  ["\u8EF4", "\u8EF4"], /* CJK Ideograph */
  ["\u8EF5", "\u8EF5"], /* CJK Ideograph */
  ["\u8EF6", "\u8EF6"], /* CJK Ideograph */
  ["\u8EF7", "\u8EF7"], /* CJK Ideograph */
  ["\u8EF8", "\u8EF8"], /* CJK Ideograph */
  ["\u8EF9", "\u8EF9"], /* CJK Ideograph */
  ["\u8EFA", "\u8EFA"], /* CJK Ideograph */
  ["\u8EFB", "\u8EFB"], /* CJK Ideograph */
  ["\u8EFC", "\u8EFC"], /* CJK Ideograph */
  ["\u8EFD", "\u8EFD"], /* CJK Ideograph */
  ["\u8EFE", "\u8EFE"], /* CJK Ideograph */
  ["\u8EFF", "\u8EFF"], /* CJK Ideograph */
  ["\u8F00", "\u8F00"], /* CJK Ideograph */
  ["\u8F01", "\u8F01"], /* CJK Ideograph */
  ["\u8F02", "\u8F02"], /* CJK Ideograph */
  ["\u8F03", "\u8F03"], /* CJK Ideograph */
  ["\u8F04", "\u8F04"], /* CJK Ideograph */
  ["\u8F05", "\u8F05"], /* CJK Ideograph */
  ["\u8F06", "\u8F06"], /* CJK Ideograph */
  ["\u8F07", "\u8F07"], /* CJK Ideograph */
  ["\u8F08", "\u8F08"], /* CJK Ideograph */
  ["\u8F09", "\u8F09"], /* CJK Ideograph */
  ["\u8F0A", "\u8F0A"], /* CJK Ideograph */
  ["\u8F0B", "\u8F0B"], /* CJK Ideograph */
  ["\u8F0C", "\u8F0C"], /* CJK Ideograph */
  ["\u8F0D", "\u8F0D"], /* CJK Ideograph */
  ["\u8F0E", "\u8F0E"], /* CJK Ideograph */
  ["\u8F0F", "\u8F0F"], /* CJK Ideograph */
  ["\u8F10", "\u8F10"], /* CJK Ideograph */
  ["\u8F11", "\u8F11"], /* CJK Ideograph */
  ["\u8F12", "\u8F12"], /* CJK Ideograph */
  ["\u8F13", "\u8F13"], /* CJK Ideograph */
  ["\u8F14", "\u8F14"], /* CJK Ideograph */
  ["\u8F15", "\u8F15"], /* CJK Ideograph */
  ["\u8F16", "\u8F16"], /* CJK Ideograph */
  ["\u8F17", "\u8F17"], /* CJK Ideograph */
  ["\u8F18", "\u8F18"], /* CJK Ideograph */
  ["\u8F19", "\u8F19"], /* CJK Ideograph */
  ["\u8F1A", "\u8F1A"], /* CJK Ideograph */
  ["\u8F1B", "\u8F1B"], /* CJK Ideograph */
  ["\u8F1C", "\u8F1C"], /* CJK Ideograph */
  ["\u8F1D", "\u8F1D"], /* CJK Ideograph */
  ["\u8F1E", "\u8F1E"], /* CJK Ideograph */
  ["\u8F1F", "\u8F1F"], /* CJK Ideograph */
  ["\u8F20", "\u8F20"], /* CJK Ideograph */
  ["\u8F21", "\u8F21"], /* CJK Ideograph */
  ["\u8F22", "\u8F22"], /* CJK Ideograph */
  ["\u8F23", "\u8F23"], /* CJK Ideograph */
  ["\u8F24", "\u8F24"], /* CJK Ideograph */
  ["\u8F25", "\u8F25"], /* CJK Ideograph */
  ["\u8F26", "\u8F26"], /* CJK Ideograph */
  ["\u8F27", "\u8F27"], /* CJK Ideograph */
  ["\u8F28", "\u8F28"], /* CJK Ideograph */
  ["\u8F29", "\u8F29"], /* CJK Ideograph */
  ["\u8F2A", "\u8F2A"], /* CJK Ideograph */
  ["\u8F2B", "\u8F2B"], /* CJK Ideograph */
  ["\u8F2C", "\u8F2C"], /* CJK Ideograph */
  ["\u8F2D", "\u8F2D"], /* CJK Ideograph */
  ["\u8F2E", "\u8F2E"], /* CJK Ideograph */
  ["\u8F2F", "\u8F2F"], /* CJK Ideograph */
  ["\u8F30", "\u8F30"], /* CJK Ideograph */
  ["\u8F31", "\u8F31"], /* CJK Ideograph */
  ["\u8F32", "\u8F32"], /* CJK Ideograph */
  ["\u8F33", "\u8F33"], /* CJK Ideograph */
  ["\u8F34", "\u8F34"], /* CJK Ideograph */
  ["\u8F35", "\u8F35"], /* CJK Ideograph */
  ["\u8F36", "\u8F36"], /* CJK Ideograph */
  ["\u8F37", "\u8F37"], /* CJK Ideograph */
  ["\u8F38", "\u8F38"], /* CJK Ideograph */
  ["\u8F39", "\u8F39"], /* CJK Ideograph */
  ["\u8F3A", "\u8F3A"], /* CJK Ideograph */
  ["\u8F3B", "\u8F3B"], /* CJK Ideograph */
  ["\u8F3C", "\u8F3C"], /* CJK Ideograph */
  ["\u8F3D", "\u8F3D"], /* CJK Ideograph */
  ["\u8F3E", "\u8F3E"], /* CJK Ideograph */
  ["\u8F3F", "\u8F3F"], /* CJK Ideograph */
  ["\u8F40", "\u8F40"], /* CJK Ideograph */
  ["\u8F41", "\u8F41"], /* CJK Ideograph */
  ["\u8F42", "\u8F42"], /* CJK Ideograph */
  ["\u8F43", "\u8F43"], /* CJK Ideograph */
  ["\u8F44", "\u8F44"], /* CJK Ideograph */
  ["\u8F45", "\u8F45"], /* CJK Ideograph */
  ["\u8F46", "\u8F46"], /* CJK Ideograph */
  ["\u8F47", "\u8F47"], /* CJK Ideograph */
  ["\u8F48", "\u8F48"], /* CJK Ideograph */
  ["\u8F49", "\u8F49"], /* CJK Ideograph */
  ["\u8F4A", "\u8F4A"], /* CJK Ideograph */
  ["\u8F4B", "\u8F4B"], /* CJK Ideograph */
  ["\u8F4C", "\u8F4C"], /* CJK Ideograph */
  ["\u8F4D", "\u8F4D"], /* CJK Ideograph */
  ["\u8F4E", "\u8F4E"], /* CJK Ideograph */
  ["\u8F4F", "\u8F4F"], /* CJK Ideograph */
  ["\u8F50", "\u8F50"], /* CJK Ideograph */
  ["\u8F51", "\u8F51"], /* CJK Ideograph */
  ["\u8F52", "\u8F52"], /* CJK Ideograph */
  ["\u8F53", "\u8F53"], /* CJK Ideograph */
  ["\u8F54", "\u8F54"], /* CJK Ideograph */
  ["\u8F55", "\u8F55"], /* CJK Ideograph */
  ["\u8F56", "\u8F56"], /* CJK Ideograph */
  ["\u8F57", "\u8F57"], /* CJK Ideograph */
  ["\u8F58", "\u8F58"], /* CJK Ideograph */
  ["\u8F59", "\u8F59"], /* CJK Ideograph */
  ["\u8F5A", "\u8F5A"], /* CJK Ideograph */
  ["\u8F5B", "\u8F5B"], /* CJK Ideograph */
  ["\u8F5C", "\u8F5C"], /* CJK Ideograph */
  ["\u8F5D", "\u8F5D"], /* CJK Ideograph */
  ["\u8F5E", "\u8F5E"], /* CJK Ideograph */
  ["\u8F5F", "\u8F5F"], /* CJK Ideograph */
  ["\u8F60", "\u8F60"], /* CJK Ideograph */
  ["\u8F61", "\u8F61"], /* CJK Ideograph */
  ["\u8F62", "\u8F62"], /* CJK Ideograph */
  ["\u8F63", "\u8F63"], /* CJK Ideograph */
  ["\u8F64", "\u8F64"], /* CJK Ideograph */
  ["\u8F65", "\u8F65"], /* CJK Ideograph */
  ["\u8F66", "\u8F66"], /* CJK Ideograph */
  ["\u8F67", "\u8F67"], /* CJK Ideograph */
  ["\u8F68", "\u8F68"], /* CJK Ideograph */
  ["\u8F69", "\u8F69"], /* CJK Ideograph */
  ["\u8F6A", "\u8F6A"], /* CJK Ideograph */
  ["\u8F6B", "\u8F6B"], /* CJK Ideograph */
  ["\u8F6C", "\u8F6C"], /* CJK Ideograph */
  ["\u8F6D", "\u8F6D"], /* CJK Ideograph */
  ["\u8F6E", "\u8F6E"], /* CJK Ideograph */
  ["\u8F6F", "\u8F6F"], /* CJK Ideograph */
  ["\u8F70", "\u8F70"], /* CJK Ideograph */
  ["\u8F71", "\u8F71"], /* CJK Ideograph */
  ["\u8F72", "\u8F72"], /* CJK Ideograph */
  ["\u8F73", "\u8F73"], /* CJK Ideograph */
  ["\u8F74", "\u8F74"], /* CJK Ideograph */
  ["\u8F75", "\u8F75"], /* CJK Ideograph */
  ["\u8F76", "\u8F76"], /* CJK Ideograph */
  ["\u8F77", "\u8F77"], /* CJK Ideograph */
  ["\u8F78", "\u8F78"], /* CJK Ideograph */
  ["\u8F79", "\u8F79"], /* CJK Ideograph */
  ["\u8F7A", "\u8F7A"], /* CJK Ideograph */
  ["\u8F7B", "\u8F7B"], /* CJK Ideograph */
  ["\u8F7C", "\u8F7C"], /* CJK Ideograph */
  ["\u8F7D", "\u8F7D"], /* CJK Ideograph */
  ["\u8F7E", "\u8F7E"], /* CJK Ideograph */
  ["\u8F7F", "\u8F7F"], /* CJK Ideograph */
  ["\u8F80", "\u8F80"], /* CJK Ideograph */
  ["\u8F81", "\u8F81"], /* CJK Ideograph */
  ["\u8F82", "\u8F82"], /* CJK Ideograph */
  ["\u8F83", "\u8F83"], /* CJK Ideograph */
  ["\u8F84", "\u8F84"], /* CJK Ideograph */
  ["\u8F85", "\u8F85"], /* CJK Ideograph */
  ["\u8F86", "\u8F86"], /* CJK Ideograph */
  ["\u8F87", "\u8F87"], /* CJK Ideograph */
  ["\u8F88", "\u8F88"], /* CJK Ideograph */
  ["\u8F89", "\u8F89"], /* CJK Ideograph */
  ["\u8F8A", "\u8F8A"], /* CJK Ideograph */
  ["\u8F8B", "\u8F8B"], /* CJK Ideograph */
  ["\u8F8C", "\u8F8C"], /* CJK Ideograph */
  ["\u8F8D", "\u8F8D"], /* CJK Ideograph */
  ["\u8F8E", "\u8F8E"], /* CJK Ideograph */
  ["\u8F8F", "\u8F8F"], /* CJK Ideograph */
  ["\u8F90", "\u8F90"], /* CJK Ideograph */
  ["\u8F91", "\u8F91"], /* CJK Ideograph */
  ["\u8F92", "\u8F92"], /* CJK Ideograph */
  ["\u8F93", "\u8F93"], /* CJK Ideograph */
  ["\u8F94", "\u8F94"], /* CJK Ideograph */
  ["\u8F95", "\u8F95"], /* CJK Ideograph */
  ["\u8F96", "\u8F96"], /* CJK Ideograph */
  ["\u8F97", "\u8F97"], /* CJK Ideograph */
  ["\u8F98", "\u8F98"], /* CJK Ideograph */
  ["\u8F99", "\u8F99"], /* CJK Ideograph */
  ["\u8F9A", "\u8F9A"], /* CJK Ideograph */
  ["\u8F9B", "\u8F9B"], /* CJK Ideograph */
  ["\u8F9C", "\u8F9C"], /* CJK Ideograph */
  ["\u8F9D", "\u8F9D"], /* CJK Ideograph */
  ["\u8F9E", "\u8F9E"], /* CJK Ideograph */
  ["\u8F9F", "\u8F9F"], /* CJK Ideograph */
  ["\u8FA0", "\u8FA0"], /* CJK Ideograph */
  ["\u8FA1", "\u8FA1"], /* CJK Ideograph */
  ["\u8FA2", "\u8FA2"], /* CJK Ideograph */
  ["\u8FA3", "\u8FA3"], /* CJK Ideograph */
  ["\u8FA4", "\u8FA4"], /* CJK Ideograph */
  ["\u8FA5", "\u8FA5"], /* CJK Ideograph */
  ["\u8FA6", "\u8FA6"], /* CJK Ideograph */
  ["\u8FA7", "\u8FA7"], /* CJK Ideograph */
  ["\u8FA8", "\u8FA8"], /* CJK Ideograph */
  ["\u8FA9", "\u8FA9"], /* CJK Ideograph */
  ["\u8FAA", "\u8FAA"], /* CJK Ideograph */
  ["\u8FAB", "\u8FAB"], /* CJK Ideograph */
  ["\u8FAC", "\u8FAC"], /* CJK Ideograph */
  ["\u8FAD", "\u8FAD"], /* CJK Ideograph */
  ["\u8FAE", "\u8FAE"], /* CJK Ideograph */
  ["\u8FAF", "\u8FAF"], /* CJK Ideograph */
  ["\u8FB0", "\u8FB0"], /* CJK Ideograph */
  ["\u8FB1", "\u8FB1"], /* CJK Ideograph */
  ["\u8FB2", "\u8FB2"], /* CJK Ideograph */
  ["\u8FB3", "\u8FB3"], /* CJK Ideograph */
  ["\u8FB4", "\u8FB4"], /* CJK Ideograph */
  ["\u8FB5", "\u8FB5"], /* CJK Ideograph */
  ["\u8FB6", "\u8FB6"], /* CJK Ideograph */
  ["\u8FB7", "\u8FB7"], /* CJK Ideograph */
  ["\u8FB8", "\u8FB8"], /* CJK Ideograph */
  ["\u8FB9", "\u8FB9"], /* CJK Ideograph */
  ["\u8FBA", "\u8FBA"], /* CJK Ideograph */
  ["\u8FBB", "\u8FBB"], /* CJK Ideograph */
  ["\u8FBC", "\u8FBC"], /* CJK Ideograph */
  ["\u8FBD", "\u8FBD"], /* CJK Ideograph */
  ["\u8FBE", "\u8FBE"], /* CJK Ideograph */
  ["\u8FBF", "\u8FBF"], /* CJK Ideograph */
  ["\u8FC0", "\u8FC0"], /* CJK Ideograph */
  ["\u8FC1", "\u8FC1"], /* CJK Ideograph */
  ["\u8FC2", "\u8FC2"], /* CJK Ideograph */
  ["\u8FC3", "\u8FC3"], /* CJK Ideograph */
  ["\u8FC4", "\u8FC4"], /* CJK Ideograph */
  ["\u8FC5", "\u8FC5"], /* CJK Ideograph */
  ["\u8FC6", "\u8FC6"], /* CJK Ideograph */
  ["\u8FC7", "\u8FC7"], /* CJK Ideograph */
  ["\u8FC8", "\u8FC8"], /* CJK Ideograph */
  ["\u8FC9", "\u8FC9"], /* CJK Ideograph */
  ["\u8FCA", "\u8FCA"], /* CJK Ideograph */
  ["\u8FCB", "\u8FCB"], /* CJK Ideograph */
  ["\u8FCC", "\u8FCC"], /* CJK Ideograph */
  ["\u8FCD", "\u8FCD"], /* CJK Ideograph */
  ["\u8FCE", "\u8FCE"], /* CJK Ideograph */
  ["\u8FCF", "\u8FCF"], /* CJK Ideograph */
  ["\u8FD0", "\u8FD0"], /* CJK Ideograph */
  ["\u8FD1", "\u8FD1"], /* CJK Ideograph */
  ["\u8FD2", "\u8FD2"], /* CJK Ideograph */
  ["\u8FD3", "\u8FD3"], /* CJK Ideograph */
  ["\u8FD4", "\u8FD4"], /* CJK Ideograph */
  ["\u8FD5", "\u8FD5"], /* CJK Ideograph */
  ["\u8FD6", "\u8FD6"], /* CJK Ideograph */
  ["\u8FD7", "\u8FD7"], /* CJK Ideograph */
  ["\u8FD8", "\u8FD8"], /* CJK Ideograph */
  ["\u8FD9", "\u8FD9"], /* CJK Ideograph */
  ["\u8FDA", "\u8FDA"], /* CJK Ideograph */
  ["\u8FDB", "\u8FDB"], /* CJK Ideograph */
  ["\u8FDC", "\u8FDC"], /* CJK Ideograph */
  ["\u8FDD", "\u8FDD"], /* CJK Ideograph */
  ["\u8FDE", "\u8FDE"], /* CJK Ideograph */
  ["\u8FDF", "\u8FDF"], /* CJK Ideograph */
  ["\u8FE0", "\u8FE0"], /* CJK Ideograph */
  ["\u8FE1", "\u8FE1"], /* CJK Ideograph */
  ["\u8FE2", "\u8FE2"], /* CJK Ideograph */
  ["\u8FE3", "\u8FE3"], /* CJK Ideograph */
  ["\u8FE4", "\u8FE4"], /* CJK Ideograph */
  ["\u8FE5", "\u8FE5"], /* CJK Ideograph */
  ["\u8FE6", "\u8FE6"], /* CJK Ideograph */
  ["\u8FE7", "\u8FE7"], /* CJK Ideograph */
  ["\u8FE8", "\u8FE8"], /* CJK Ideograph */
  ["\u8FE9", "\u8FE9"], /* CJK Ideograph */
  ["\u8FEA", "\u8FEA"], /* CJK Ideograph */
  ["\u8FEB", "\u8FEB"], /* CJK Ideograph */
  ["\u8FEC", "\u8FEC"], /* CJK Ideograph */
  ["\u8FED", "\u8FED"], /* CJK Ideograph */
  ["\u8FEE", "\u8FEE"], /* CJK Ideograph */
  ["\u8FEF", "\u8FEF"], /* CJK Ideograph */
  ["\u8FF0", "\u8FF0"], /* CJK Ideograph */
  ["\u8FF1", "\u8FF1"], /* CJK Ideograph */
  ["\u8FF2", "\u8FF2"], /* CJK Ideograph */
  ["\u8FF3", "\u8FF3"], /* CJK Ideograph */
  ["\u8FF4", "\u8FF4"], /* CJK Ideograph */
  ["\u8FF5", "\u8FF5"], /* CJK Ideograph */
  ["\u8FF6", "\u8FF6"], /* CJK Ideograph */
  ["\u8FF7", "\u8FF7"], /* CJK Ideograph */
  ["\u8FF8", "\u8FF8"], /* CJK Ideograph */
  ["\u8FF9", "\u8FF9"], /* CJK Ideograph */
  ["\u8FFA", "\u8FFA"], /* CJK Ideograph */
  ["\u8FFB", "\u8FFB"], /* CJK Ideograph */
  ["\u8FFC", "\u8FFC"], /* CJK Ideograph */
  ["\u8FFD", "\u8FFD"], /* CJK Ideograph */
  ["\u8FFE", "\u8FFE"], /* CJK Ideograph */
  ["\u8FFF", "\u8FFF"], /* CJK Ideograph */
  ["\u9000", "\u9000"], /* CJK Ideograph */
  ["\u9001", "\u9001"], /* CJK Ideograph */
  ["\u9002", "\u9002"], /* CJK Ideograph */
  ["\u9003", "\u9003"], /* CJK Ideograph */
  ["\u9004", "\u9004"], /* CJK Ideograph */
  ["\u9005", "\u9005"], /* CJK Ideograph */
  ["\u9006", "\u9006"], /* CJK Ideograph */
  ["\u9007", "\u9007"], /* CJK Ideograph */
  ["\u9008", "\u9008"], /* CJK Ideograph */
  ["\u9009", "\u9009"], /* CJK Ideograph */
  ["\u900A", "\u900A"], /* CJK Ideograph */
  ["\u900B", "\u900B"], /* CJK Ideograph */
  ["\u900C", "\u900C"], /* CJK Ideograph */
  ["\u900D", "\u900D"], /* CJK Ideograph */
  ["\u900E", "\u900E"], /* CJK Ideograph */
  ["\u900F", "\u900F"], /* CJK Ideograph */
  ["\u9010", "\u9010"], /* CJK Ideograph */
  ["\u9011", "\u9011"], /* CJK Ideograph */
  ["\u9012", "\u9012"], /* CJK Ideograph */
  ["\u9013", "\u9013"], /* CJK Ideograph */
  ["\u9014", "\u9014"], /* CJK Ideograph */
  ["\u9015", "\u9015"], /* CJK Ideograph */
  ["\u9016", "\u9016"], /* CJK Ideograph */
  ["\u9017", "\u9017"], /* CJK Ideograph */
  ["\u9018", "\u9018"], /* CJK Ideograph */
  ["\u9019", "\u9019"], /* CJK Ideograph */
  ["\u901A", "\u901A"], /* CJK Ideograph */
  ["\u901B", "\u901B"], /* CJK Ideograph */
  ["\u901C", "\u901C"], /* CJK Ideograph */
  ["\u901D", "\u901D"], /* CJK Ideograph */
  ["\u901E", "\u901E"], /* CJK Ideograph */
  ["\u901F", "\u901F"], /* CJK Ideograph */
  ["\u9020", "\u9020"], /* CJK Ideograph */
  ["\u9021", "\u9021"], /* CJK Ideograph */
  ["\u9022", "\u9022"], /* CJK Ideograph */
  ["\u9023", "\u9023"], /* CJK Ideograph */
  ["\u9024", "\u9024"], /* CJK Ideograph */
  ["\u9025", "\u9025"], /* CJK Ideograph */
  ["\u9026", "\u9026"], /* CJK Ideograph */
  ["\u9027", "\u9027"], /* CJK Ideograph */
  ["\u9028", "\u9028"], /* CJK Ideograph */
  ["\u9029", "\u9029"], /* CJK Ideograph */
  ["\u902A", "\u902A"], /* CJK Ideograph */
  ["\u902B", "\u902B"], /* CJK Ideograph */
  ["\u902C", "\u902C"], /* CJK Ideograph */
  ["\u902D", "\u902D"], /* CJK Ideograph */
  ["\u902E", "\u902E"], /* CJK Ideograph */
  ["\u902F", "\u902F"], /* CJK Ideograph */
  ["\u9030", "\u9030"], /* CJK Ideograph */
  ["\u9031", "\u9031"], /* CJK Ideograph */
  ["\u9032", "\u9032"], /* CJK Ideograph */
  ["\u9033", "\u9033"], /* CJK Ideograph */
  ["\u9034", "\u9034"], /* CJK Ideograph */
  ["\u9035", "\u9035"], /* CJK Ideograph */
  ["\u9036", "\u9036"], /* CJK Ideograph */
  ["\u9037", "\u9037"], /* CJK Ideograph */
  ["\u9038", "\u9038"], /* CJK Ideograph */
  ["\u9039", "\u9039"], /* CJK Ideograph */
  ["\u903A", "\u903A"], /* CJK Ideograph */
  ["\u903B", "\u903B"], /* CJK Ideograph */
  ["\u903C", "\u903C"], /* CJK Ideograph */
  ["\u903D", "\u903D"], /* CJK Ideograph */
  ["\u903E", "\u903E"], /* CJK Ideograph */
  ["\u903F", "\u903F"], /* CJK Ideograph */
  ["\u9040", "\u9040"], /* CJK Ideograph */
  ["\u9041", "\u9041"], /* CJK Ideograph */
  ["\u9042", "\u9042"], /* CJK Ideograph */
  ["\u9043", "\u9043"], /* CJK Ideograph */
  ["\u9044", "\u9044"], /* CJK Ideograph */
  ["\u9045", "\u9045"], /* CJK Ideograph */
  ["\u9046", "\u9046"], /* CJK Ideograph */
  ["\u9047", "\u9047"], /* CJK Ideograph */
  ["\u9048", "\u9048"], /* CJK Ideograph */
  ["\u9049", "\u9049"], /* CJK Ideograph */
  ["\u904A", "\u904A"], /* CJK Ideograph */
  ["\u904B", "\u904B"], /* CJK Ideograph */
  ["\u904C", "\u904C"], /* CJK Ideograph */
  ["\u904D", "\u904D"], /* CJK Ideograph */
  ["\u904E", "\u904E"], /* CJK Ideograph */
  ["\u904F", "\u904F"], /* CJK Ideograph */
  ["\u9050", "\u9050"], /* CJK Ideograph */
  ["\u9051", "\u9051"], /* CJK Ideograph */
  ["\u9052", "\u9052"], /* CJK Ideograph */
  ["\u9053", "\u9053"], /* CJK Ideograph */
  ["\u9054", "\u9054"], /* CJK Ideograph */
  ["\u9055", "\u9055"], /* CJK Ideograph */
  ["\u9056", "\u9056"], /* CJK Ideograph */
  ["\u9057", "\u9057"], /* CJK Ideograph */
  ["\u9058", "\u9058"], /* CJK Ideograph */
  ["\u9059", "\u9059"], /* CJK Ideograph */
  ["\u905A", "\u905A"], /* CJK Ideograph */
  ["\u905B", "\u905B"], /* CJK Ideograph */
  ["\u905C", "\u905C"], /* CJK Ideograph */
  ["\u905D", "\u905D"], /* CJK Ideograph */
  ["\u905E", "\u905E"], /* CJK Ideograph */
  ["\u905F", "\u905F"], /* CJK Ideograph */
  ["\u9060", "\u9060"], /* CJK Ideograph */
  ["\u9061", "\u9061"], /* CJK Ideograph */
  ["\u9062", "\u9062"], /* CJK Ideograph */
  ["\u9063", "\u9063"], /* CJK Ideograph */
  ["\u9064", "\u9064"], /* CJK Ideograph */
  ["\u9065", "\u9065"], /* CJK Ideograph */
  ["\u9066", "\u9066"], /* CJK Ideograph */
  ["\u9067", "\u9067"], /* CJK Ideograph */
  ["\u9068", "\u9068"], /* CJK Ideograph */
  ["\u9069", "\u9069"], /* CJK Ideograph */
  ["\u906A", "\u906A"], /* CJK Ideograph */
  ["\u906B", "\u906B"], /* CJK Ideograph */
  ["\u906C", "\u906C"], /* CJK Ideograph */
  ["\u906D", "\u906D"], /* CJK Ideograph */
  ["\u906E", "\u906E"], /* CJK Ideograph */
  ["\u906F", "\u906F"], /* CJK Ideograph */
  ["\u9070", "\u9070"], /* CJK Ideograph */
  ["\u9071", "\u9071"], /* CJK Ideograph */
  ["\u9072", "\u9072"], /* CJK Ideograph */
  ["\u9073", "\u9073"], /* CJK Ideograph */
  ["\u9074", "\u9074"], /* CJK Ideograph */
  ["\u9075", "\u9075"], /* CJK Ideograph */
  ["\u9076", "\u9076"], /* CJK Ideograph */
  ["\u9077", "\u9077"], /* CJK Ideograph */
  ["\u9078", "\u9078"], /* CJK Ideograph */
  ["\u9079", "\u9079"], /* CJK Ideograph */
  ["\u907A", "\u907A"], /* CJK Ideograph */
  ["\u907B", "\u907B"], /* CJK Ideograph */
  ["\u907C", "\u907C"], /* CJK Ideograph */
  ["\u907D", "\u907D"], /* CJK Ideograph */
  ["\u907E", "\u907E"], /* CJK Ideograph */
  ["\u907F", "\u907F"], /* CJK Ideograph */
  ["\u9080", "\u9080"], /* CJK Ideograph */
  ["\u9081", "\u9081"], /* CJK Ideograph */
  ["\u9082", "\u9082"], /* CJK Ideograph */
  ["\u9083", "\u9083"], /* CJK Ideograph */
  ["\u9084", "\u9084"], /* CJK Ideograph */
  ["\u9085", "\u9085"], /* CJK Ideograph */
  ["\u9086", "\u9086"], /* CJK Ideograph */
  ["\u9087", "\u9087"], /* CJK Ideograph */
  ["\u9088", "\u9088"], /* CJK Ideograph */
  ["\u9089", "\u9089"], /* CJK Ideograph */
  ["\u908A", "\u908A"], /* CJK Ideograph */
  ["\u908B", "\u908B"], /* CJK Ideograph */
  ["\u908C", "\u908C"], /* CJK Ideograph */
  ["\u908D", "\u908D"], /* CJK Ideograph */
  ["\u908E", "\u908E"], /* CJK Ideograph */
  ["\u908F", "\u908F"], /* CJK Ideograph */
  ["\u9090", "\u9090"], /* CJK Ideograph */
  ["\u9091", "\u9091"], /* CJK Ideograph */
  ["\u9092", "\u9092"], /* CJK Ideograph */
  ["\u9093", "\u9093"], /* CJK Ideograph */
  ["\u9094", "\u9094"], /* CJK Ideograph */
  ["\u9095", "\u9095"], /* CJK Ideograph */
  ["\u9096", "\u9096"], /* CJK Ideograph */
  ["\u9097", "\u9097"], /* CJK Ideograph */
  ["\u9098", "\u9098"], /* CJK Ideograph */
  ["\u9099", "\u9099"], /* CJK Ideograph */
  ["\u909A", "\u909A"], /* CJK Ideograph */
  ["\u909B", "\u909B"], /* CJK Ideograph */
  ["\u909C", "\u909C"], /* CJK Ideograph */
  ["\u909D", "\u909D"], /* CJK Ideograph */
  ["\u909E", "\u909E"], /* CJK Ideograph */
  ["\u909F", "\u909F"], /* CJK Ideograph */
  ["\u90A0", "\u90A0"], /* CJK Ideograph */
  ["\u90A1", "\u90A1"], /* CJK Ideograph */
  ["\u90A2", "\u90A2"], /* CJK Ideograph */
  ["\u90A3", "\u90A3"], /* CJK Ideograph */
  ["\u90A4", "\u90A4"], /* CJK Ideograph */
  ["\u90A5", "\u90A5"], /* CJK Ideograph */
  ["\u90A6", "\u90A6"], /* CJK Ideograph */
  ["\u90A7", "\u90A7"], /* CJK Ideograph */
  ["\u90A8", "\u90A8"], /* CJK Ideograph */
  ["\u90A9", "\u90A9"], /* CJK Ideograph */
  ["\u90AA", "\u90AA"], /* CJK Ideograph */
  ["\u90AB", "\u90AB"], /* CJK Ideograph */
  ["\u90AC", "\u90AC"], /* CJK Ideograph */
  ["\u90AD", "\u90AD"], /* CJK Ideograph */
  ["\u90AE", "\u90AE"], /* CJK Ideograph */
  ["\u90AF", "\u90AF"], /* CJK Ideograph */
  ["\u90B0", "\u90B0"], /* CJK Ideograph */
  ["\u90B1", "\u90B1"], /* CJK Ideograph */
  ["\u90B2", "\u90B2"], /* CJK Ideograph */
  ["\u90B3", "\u90B3"], /* CJK Ideograph */
  ["\u90B4", "\u90B4"], /* CJK Ideograph */
  ["\u90B5", "\u90B5"], /* CJK Ideograph */
  ["\u90B6", "\u90B6"], /* CJK Ideograph */
  ["\u90B7", "\u90B7"], /* CJK Ideograph */
  ["\u90B8", "\u90B8"], /* CJK Ideograph */
  ["\u90B9", "\u90B9"], /* CJK Ideograph */
  ["\u90BA", "\u90BA"], /* CJK Ideograph */
  ["\u90BB", "\u90BB"], /* CJK Ideograph */
  ["\u90BC", "\u90BC"], /* CJK Ideograph */
  ["\u90BD", "\u90BD"], /* CJK Ideograph */
  ["\u90BE", "\u90BE"], /* CJK Ideograph */
  ["\u90BF", "\u90BF"], /* CJK Ideograph */
  ["\u90C0", "\u90C0"], /* CJK Ideograph */
  ["\u90C1", "\u90C1"], /* CJK Ideograph */
  ["\u90C2", "\u90C2"], /* CJK Ideograph */
  ["\u90C3", "\u90C3"], /* CJK Ideograph */
  ["\u90C4", "\u90C4"], /* CJK Ideograph */
  ["\u90C5", "\u90C5"], /* CJK Ideograph */
  ["\u90C6", "\u90C6"], /* CJK Ideograph */
  ["\u90C7", "\u90C7"], /* CJK Ideograph */
  ["\u90C8", "\u90C8"], /* CJK Ideograph */
  ["\u90C9", "\u90C9"], /* CJK Ideograph */
  ["\u90CA", "\u90CA"], /* CJK Ideograph */
  ["\u90CB", "\u90CB"], /* CJK Ideograph */
  ["\u90CC", "\u90CC"], /* CJK Ideograph */
  ["\u90CD", "\u90CD"], /* CJK Ideograph */
  ["\u90CE", "\u90CE"], /* CJK Ideograph */
  ["\u90CF", "\u90CF"], /* CJK Ideograph */
  ["\u90D0", "\u90D0"], /* CJK Ideograph */
  ["\u90D1", "\u90D1"], /* CJK Ideograph */
  ["\u90D2", "\u90D2"], /* CJK Ideograph */
  ["\u90D3", "\u90D3"], /* CJK Ideograph */
  ["\u90D4", "\u90D4"], /* CJK Ideograph */
  ["\u90D5", "\u90D5"], /* CJK Ideograph */
  ["\u90D6", "\u90D6"], /* CJK Ideograph */
  ["\u90D7", "\u90D7"], /* CJK Ideograph */
  ["\u90D8", "\u90D8"], /* CJK Ideograph */
  ["\u90D9", "\u90D9"], /* CJK Ideograph */
  ["\u90DA", "\u90DA"], /* CJK Ideograph */
  ["\u90DB", "\u90DB"], /* CJK Ideograph */
  ["\u90DC", "\u90DC"], /* CJK Ideograph */
  ["\u90DD", "\u90DD"], /* CJK Ideograph */
  ["\u90DE", "\u90DE"], /* CJK Ideograph */
  ["\u90DF", "\u90DF"], /* CJK Ideograph */
  ["\u90E0", "\u90E0"], /* CJK Ideograph */
  ["\u90E1", "\u90E1"], /* CJK Ideograph */
  ["\u90E2", "\u90E2"], /* CJK Ideograph */
  ["\u90E3", "\u90E3"], /* CJK Ideograph */
  ["\u90E4", "\u90E4"], /* CJK Ideograph */
  ["\u90E5", "\u90E5"], /* CJK Ideograph */
  ["\u90E6", "\u90E6"], /* CJK Ideograph */
  ["\u90E7", "\u90E7"], /* CJK Ideograph */
  ["\u90E8", "\u90E8"], /* CJK Ideograph */
  ["\u90E9", "\u90E9"], /* CJK Ideograph */
  ["\u90EA", "\u90EA"], /* CJK Ideograph */
  ["\u90EB", "\u90EB"], /* CJK Ideograph */
  ["\u90EC", "\u90EC"], /* CJK Ideograph */
  ["\u90ED", "\u90ED"], /* CJK Ideograph */
  ["\u90EE", "\u90EE"], /* CJK Ideograph */
  ["\u90EF", "\u90EF"], /* CJK Ideograph */
  ["\u90F0", "\u90F0"], /* CJK Ideograph */
  ["\u90F1", "\u90F1"], /* CJK Ideograph */
  ["\u90F2", "\u90F2"], /* CJK Ideograph */
  ["\u90F3", "\u90F3"], /* CJK Ideograph */
  ["\u90F4", "\u90F4"], /* CJK Ideograph */
  ["\u90F5", "\u90F5"], /* CJK Ideograph */
  ["\u90F6", "\u90F6"], /* CJK Ideograph */
  ["\u90F7", "\u90F7"], /* CJK Ideograph */
  ["\u90F8", "\u90F8"], /* CJK Ideograph */
  ["\u90F9", "\u90F9"], /* CJK Ideograph */
  ["\u90FA", "\u90FA"], /* CJK Ideograph */
  ["\u90FB", "\u90FB"], /* CJK Ideograph */
  ["\u90FC", "\u90FC"], /* CJK Ideograph */
  ["\u90FD", "\u90FD"], /* CJK Ideograph */
  ["\u90FE", "\u90FE"], /* CJK Ideograph */
  ["\u90FF", "\u90FF"], /* CJK Ideograph */
  ["\u9100", "\u9100"], /* CJK Ideograph */
  ["\u9101", "\u9101"], /* CJK Ideograph */
  ["\u9102", "\u9102"], /* CJK Ideograph */
  ["\u9103", "\u9103"], /* CJK Ideograph */
  ["\u9104", "\u9104"], /* CJK Ideograph */
  ["\u9105", "\u9105"], /* CJK Ideograph */
  ["\u9106", "\u9106"], /* CJK Ideograph */
  ["\u9107", "\u9107"], /* CJK Ideograph */
  ["\u9108", "\u9108"], /* CJK Ideograph */
  ["\u9109", "\u9109"], /* CJK Ideograph */
  ["\u910A", "\u910A"], /* CJK Ideograph */
  ["\u910B", "\u910B"], /* CJK Ideograph */
  ["\u910C", "\u910C"], /* CJK Ideograph */
  ["\u910D", "\u910D"], /* CJK Ideograph */
  ["\u910E", "\u910E"], /* CJK Ideograph */
  ["\u910F", "\u910F"], /* CJK Ideograph */
  ["\u9110", "\u9110"], /* CJK Ideograph */
  ["\u9111", "\u9111"], /* CJK Ideograph */
  ["\u9112", "\u9112"], /* CJK Ideograph */
  ["\u9113", "\u9113"], /* CJK Ideograph */
  ["\u9114", "\u9114"], /* CJK Ideograph */
  ["\u9115", "\u9115"], /* CJK Ideograph */
  ["\u9116", "\u9116"], /* CJK Ideograph */
  ["\u9117", "\u9117"], /* CJK Ideograph */
  ["\u9118", "\u9118"], /* CJK Ideograph */
  ["\u9119", "\u9119"], /* CJK Ideograph */
  ["\u911A", "\u911A"], /* CJK Ideograph */
  ["\u911B", "\u911B"], /* CJK Ideograph */
  ["\u911C", "\u911C"], /* CJK Ideograph */
  ["\u911D", "\u911D"], /* CJK Ideograph */
  ["\u911E", "\u911E"], /* CJK Ideograph */
  ["\u911F", "\u911F"], /* CJK Ideograph */
  ["\u9120", "\u9120"], /* CJK Ideograph */
  ["\u9121", "\u9121"], /* CJK Ideograph */
  ["\u9122", "\u9122"], /* CJK Ideograph */
  ["\u9123", "\u9123"], /* CJK Ideograph */
  ["\u9124", "\u9124"], /* CJK Ideograph */
  ["\u9125", "\u9125"], /* CJK Ideograph */
  ["\u9126", "\u9126"], /* CJK Ideograph */
  ["\u9127", "\u9127"], /* CJK Ideograph */
  ["\u9128", "\u9128"], /* CJK Ideograph */
  ["\u9129", "\u9129"], /* CJK Ideograph */
  ["\u912A", "\u912A"], /* CJK Ideograph */
  ["\u912B", "\u912B"], /* CJK Ideograph */
  ["\u912C", "\u912C"], /* CJK Ideograph */
  ["\u912D", "\u912D"], /* CJK Ideograph */
  ["\u912E", "\u912E"], /* CJK Ideograph */
  ["\u912F", "\u912F"], /* CJK Ideograph */
  ["\u9130", "\u9130"], /* CJK Ideograph */
  ["\u9131", "\u9131"], /* CJK Ideograph */
  ["\u9132", "\u9132"], /* CJK Ideograph */
  ["\u9133", "\u9133"], /* CJK Ideograph */
  ["\u9134", "\u9134"], /* CJK Ideograph */
  ["\u9135", "\u9135"], /* CJK Ideograph */
  ["\u9136", "\u9136"], /* CJK Ideograph */
  ["\u9137", "\u9137"], /* CJK Ideograph */
  ["\u9138", "\u9138"], /* CJK Ideograph */
  ["\u9139", "\u9139"], /* CJK Ideograph */
  ["\u913A", "\u913A"], /* CJK Ideograph */
  ["\u913B", "\u913B"], /* CJK Ideograph */
  ["\u913C", "\u913C"], /* CJK Ideograph */
  ["\u913D", "\u913D"], /* CJK Ideograph */
  ["\u913E", "\u913E"], /* CJK Ideograph */
  ["\u913F", "\u913F"], /* CJK Ideograph */
  ["\u9140", "\u9140"], /* CJK Ideograph */
  ["\u9141", "\u9141"], /* CJK Ideograph */
  ["\u9142", "\u9142"], /* CJK Ideograph */
  ["\u9143", "\u9143"], /* CJK Ideograph */
  ["\u9144", "\u9144"], /* CJK Ideograph */
  ["\u9145", "\u9145"], /* CJK Ideograph */
  ["\u9146", "\u9146"], /* CJK Ideograph */
  ["\u9147", "\u9147"], /* CJK Ideograph */
  ["\u9148", "\u9148"], /* CJK Ideograph */
  ["\u9149", "\u9149"], /* CJK Ideograph */
  ["\u914A", "\u914A"], /* CJK Ideograph */
  ["\u914B", "\u914B"], /* CJK Ideograph */
  ["\u914C", "\u914C"], /* CJK Ideograph */
  ["\u914D", "\u914D"], /* CJK Ideograph */
  ["\u914E", "\u914E"], /* CJK Ideograph */
  ["\u914F", "\u914F"], /* CJK Ideograph */
  ["\u9150", "\u9150"], /* CJK Ideograph */
  ["\u9151", "\u9151"], /* CJK Ideograph */
  ["\u9152", "\u9152"], /* CJK Ideograph */
  ["\u9153", "\u9153"], /* CJK Ideograph */
  ["\u9154", "\u9154"], /* CJK Ideograph */
  ["\u9155", "\u9155"], /* CJK Ideograph */
  ["\u9156", "\u9156"], /* CJK Ideograph */
  ["\u9157", "\u9157"], /* CJK Ideograph */
  ["\u9158", "\u9158"], /* CJK Ideograph */
  ["\u9159", "\u9159"], /* CJK Ideograph */
  ["\u915A", "\u915A"], /* CJK Ideograph */
  ["\u915B", "\u915B"], /* CJK Ideograph */
  ["\u915C", "\u915C"], /* CJK Ideograph */
  ["\u915D", "\u915D"], /* CJK Ideograph */
  ["\u915E", "\u915E"], /* CJK Ideograph */
  ["\u915F", "\u915F"], /* CJK Ideograph */
  ["\u9160", "\u9160"], /* CJK Ideograph */
  ["\u9161", "\u9161"], /* CJK Ideograph */
  ["\u9162", "\u9162"], /* CJK Ideograph */
  ["\u9163", "\u9163"], /* CJK Ideograph */
  ["\u9164", "\u9164"], /* CJK Ideograph */
  ["\u9165", "\u9165"], /* CJK Ideograph */
  ["\u9166", "\u9166"], /* CJK Ideograph */
  ["\u9167", "\u9167"], /* CJK Ideograph */
  ["\u9168", "\u9168"], /* CJK Ideograph */
  ["\u9169", "\u9169"], /* CJK Ideograph */
  ["\u916A", "\u916A"], /* CJK Ideograph */
  ["\u916B", "\u916B"], /* CJK Ideograph */
  ["\u916C", "\u916C"], /* CJK Ideograph */
  ["\u916D", "\u916D"], /* CJK Ideograph */
  ["\u916E", "\u916E"], /* CJK Ideograph */
  ["\u916F", "\u916F"], /* CJK Ideograph */
  ["\u9170", "\u9170"], /* CJK Ideograph */
  ["\u9171", "\u9171"], /* CJK Ideograph */
  ["\u9172", "\u9172"], /* CJK Ideograph */
  ["\u9173", "\u9173"], /* CJK Ideograph */
  ["\u9174", "\u9174"], /* CJK Ideograph */
  ["\u9175", "\u9175"], /* CJK Ideograph */
  ["\u9176", "\u9176"], /* CJK Ideograph */
  ["\u9177", "\u9177"], /* CJK Ideograph */
  ["\u9178", "\u9178"], /* CJK Ideograph */
  ["\u9179", "\u9179"], /* CJK Ideograph */
  ["\u917A", "\u917A"], /* CJK Ideograph */
  ["\u917B", "\u917B"], /* CJK Ideograph */
  ["\u917C", "\u917C"], /* CJK Ideograph */
  ["\u917D", "\u917D"], /* CJK Ideograph */
  ["\u917E", "\u917E"], /* CJK Ideograph */
  ["\u917F", "\u917F"], /* CJK Ideograph */
  ["\u9180", "\u9180"], /* CJK Ideograph */
  ["\u9181", "\u9181"], /* CJK Ideograph */
  ["\u9182", "\u9182"], /* CJK Ideograph */
  ["\u9183", "\u9183"], /* CJK Ideograph */
  ["\u9184", "\u9184"], /* CJK Ideograph */
  ["\u9185", "\u9185"], /* CJK Ideograph */
  ["\u9186", "\u9186"], /* CJK Ideograph */
  ["\u9187", "\u9187"], /* CJK Ideograph */
  ["\u9188", "\u9188"], /* CJK Ideograph */
  ["\u9189", "\u9189"], /* CJK Ideograph */
  ["\u918A", "\u918A"], /* CJK Ideograph */
  ["\u918B", "\u918B"], /* CJK Ideograph */
  ["\u918C", "\u918C"], /* CJK Ideograph */
  ["\u918D", "\u918D"], /* CJK Ideograph */
  ["\u918E", "\u918E"], /* CJK Ideograph */
  ["\u918F", "\u918F"], /* CJK Ideograph */
  ["\u9190", "\u9190"], /* CJK Ideograph */
  ["\u9191", "\u9191"], /* CJK Ideograph */
  ["\u9192", "\u9192"], /* CJK Ideograph */
  ["\u9193", "\u9193"], /* CJK Ideograph */
  ["\u9194", "\u9194"], /* CJK Ideograph */
  ["\u9195", "\u9195"], /* CJK Ideograph */
  ["\u9196", "\u9196"], /* CJK Ideograph */
  ["\u9197", "\u9197"], /* CJK Ideograph */
  ["\u9198", "\u9198"], /* CJK Ideograph */
  ["\u9199", "\u9199"], /* CJK Ideograph */
  ["\u919A", "\u919A"], /* CJK Ideograph */
  ["\u919B", "\u919B"], /* CJK Ideograph */
  ["\u919C", "\u919C"], /* CJK Ideograph */
  ["\u919D", "\u919D"], /* CJK Ideograph */
  ["\u919E", "\u919E"], /* CJK Ideograph */
  ["\u919F", "\u919F"], /* CJK Ideograph */
  ["\u91A0", "\u91A0"], /* CJK Ideograph */
  ["\u91A1", "\u91A1"], /* CJK Ideograph */
  ["\u91A2", "\u91A2"], /* CJK Ideograph */
  ["\u91A3", "\u91A3"], /* CJK Ideograph */
  ["\u91A4", "\u91A4"], /* CJK Ideograph */
  ["\u91A5", "\u91A5"], /* CJK Ideograph */
  ["\u91A6", "\u91A6"], /* CJK Ideograph */
  ["\u91A7", "\u91A7"], /* CJK Ideograph */
  ["\u91A8", "\u91A8"], /* CJK Ideograph */
  ["\u91A9", "\u91A9"], /* CJK Ideograph */
  ["\u91AA", "\u91AA"], /* CJK Ideograph */
  ["\u91AB", "\u91AB"], /* CJK Ideograph */
  ["\u91AC", "\u91AC"], /* CJK Ideograph */
  ["\u91AD", "\u91AD"], /* CJK Ideograph */
  ["\u91AE", "\u91AE"], /* CJK Ideograph */
  ["\u91AF", "\u91AF"], /* CJK Ideograph */
  ["\u91B0", "\u91B0"], /* CJK Ideograph */
  ["\u91B1", "\u91B1"], /* CJK Ideograph */
  ["\u91B2", "\u91B2"], /* CJK Ideograph */
  ["\u91B3", "\u91B3"], /* CJK Ideograph */
  ["\u91B4", "\u91B4"], /* CJK Ideograph */
  ["\u91B5", "\u91B5"], /* CJK Ideograph */
  ["\u91B6", "\u91B6"], /* CJK Ideograph */
  ["\u91B7", "\u91B7"], /* CJK Ideograph */
  ["\u91B8", "\u91B8"], /* CJK Ideograph */
  ["\u91B9", "\u91B9"], /* CJK Ideograph */
  ["\u91BA", "\u91BA"], /* CJK Ideograph */
  ["\u91BB", "\u91BB"], /* CJK Ideograph */
  ["\u91BC", "\u91BC"], /* CJK Ideograph */
  ["\u91BD", "\u91BD"], /* CJK Ideograph */
  ["\u91BE", "\u91BE"], /* CJK Ideograph */
  ["\u91BF", "\u91BF"], /* CJK Ideograph */
  ["\u91C0", "\u91C0"], /* CJK Ideograph */
  ["\u91C1", "\u91C1"], /* CJK Ideograph */
  ["\u91C2", "\u91C2"], /* CJK Ideograph */
  ["\u91C3", "\u91C3"], /* CJK Ideograph */
  ["\u91C4", "\u91C4"], /* CJK Ideograph */
  ["\u91C5", "\u91C5"], /* CJK Ideograph */
  ["\u91C6", "\u91C6"], /* CJK Ideograph */
  ["\u91C7", "\u91C7"], /* CJK Ideograph */
  ["\u91C8", "\u91C8"], /* CJK Ideograph */
  ["\u91C9", "\u91C9"], /* CJK Ideograph */
  ["\u91CA", "\u91CA"], /* CJK Ideograph */
  ["\u91CB", "\u91CB"], /* CJK Ideograph */
  ["\u91CC", "\u91CC"], /* CJK Ideograph */
  ["\u91CD", "\u91CD"], /* CJK Ideograph */
  ["\u91CE", "\u91CE"], /* CJK Ideograph */
  ["\u91CF", "\u91CF"], /* CJK Ideograph */
  ["\u91D0", "\u91D0"], /* CJK Ideograph */
  ["\u91D1", "\u91D1"], /* CJK Ideograph */
  ["\u91D2", "\u91D2"], /* CJK Ideograph */
  ["\u91D3", "\u91D3"], /* CJK Ideograph */
  ["\u91D4", "\u91D4"], /* CJK Ideograph */
  ["\u91D5", "\u91D5"], /* CJK Ideograph */
  ["\u91D6", "\u91D6"], /* CJK Ideograph */
  ["\u91D7", "\u91D7"], /* CJK Ideograph */
  ["\u91D8", "\u91D8"], /* CJK Ideograph */
  ["\u91D9", "\u91D9"], /* CJK Ideograph */
  ["\u91DA", "\u91DA"], /* CJK Ideograph */
  ["\u91DB", "\u91DB"], /* CJK Ideograph */
  ["\u91DC", "\u91DC"], /* CJK Ideograph */
  ["\u91DD", "\u91DD"], /* CJK Ideograph */
  ["\u91DE", "\u91DE"], /* CJK Ideograph */
  ["\u91DF", "\u91DF"], /* CJK Ideograph */
  ["\u91E0", "\u91E0"], /* CJK Ideograph */
  ["\u91E1", "\u91E1"], /* CJK Ideograph */
  ["\u91E2", "\u91E2"], /* CJK Ideograph */
  ["\u91E3", "\u91E3"], /* CJK Ideograph */
  ["\u91E4", "\u91E4"], /* CJK Ideograph */
  ["\u91E5", "\u91E5"], /* CJK Ideograph */
  ["\u91E6", "\u91E6"], /* CJK Ideograph */
  ["\u91E7", "\u91E7"], /* CJK Ideograph */
  ["\u91E8", "\u91E8"], /* CJK Ideograph */
  ["\u91E9", "\u91E9"], /* CJK Ideograph */
  ["\u91EA", "\u91EA"], /* CJK Ideograph */
  ["\u91EB", "\u91EB"], /* CJK Ideograph */
  ["\u91EC", "\u91EC"], /* CJK Ideograph */
  ["\u91ED", "\u91ED"], /* CJK Ideograph */
  ["\u91EE", "\u91EE"], /* CJK Ideograph */
  ["\u91EF", "\u91EF"], /* CJK Ideograph */
  ["\u91F0", "\u91F0"], /* CJK Ideograph */
  ["\u91F1", "\u91F1"], /* CJK Ideograph */
  ["\u91F2", "\u91F2"], /* CJK Ideograph */
  ["\u91F3", "\u91F3"], /* CJK Ideograph */
  ["\u91F4", "\u91F4"], /* CJK Ideograph */
  ["\u91F5", "\u91F5"], /* CJK Ideograph */
  ["\u91F6", "\u91F6"], /* CJK Ideograph */
  ["\u91F7", "\u91F7"], /* CJK Ideograph */
  ["\u91F8", "\u91F8"], /* CJK Ideograph */
  ["\u91F9", "\u91F9"], /* CJK Ideograph */
  ["\u91FA", "\u91FA"], /* CJK Ideograph */
  ["\u91FB", "\u91FB"], /* CJK Ideograph */
  ["\u91FC", "\u91FC"], /* CJK Ideograph */
  ["\u91FD", "\u91FD"], /* CJK Ideograph */
  ["\u91FE", "\u91FE"], /* CJK Ideograph */
  ["\u91FF", "\u91FF"], /* CJK Ideograph */
  ["\u9200", "\u9200"], /* CJK Ideograph */
  ["\u9201", "\u9201"], /* CJK Ideograph */
  ["\u9202", "\u9202"], /* CJK Ideograph */
  ["\u9203", "\u9203"], /* CJK Ideograph */
  ["\u9204", "\u9204"], /* CJK Ideograph */
  ["\u9205", "\u9205"], /* CJK Ideograph */
  ["\u9206", "\u9206"], /* CJK Ideograph */
  ["\u9207", "\u9207"], /* CJK Ideograph */
  ["\u9208", "\u9208"], /* CJK Ideograph */
  ["\u9209", "\u9209"], /* CJK Ideograph */
  ["\u920A", "\u920A"], /* CJK Ideograph */
  ["\u920B", "\u920B"], /* CJK Ideograph */
  ["\u920C", "\u920C"], /* CJK Ideograph */
  ["\u920D", "\u920D"], /* CJK Ideograph */
  ["\u920E", "\u920E"], /* CJK Ideograph */
  ["\u920F", "\u920F"], /* CJK Ideograph */
  ["\u9210", "\u9210"], /* CJK Ideograph */
  ["\u9211", "\u9211"], /* CJK Ideograph */
  ["\u9212", "\u9212"], /* CJK Ideograph */
  ["\u9213", "\u9213"], /* CJK Ideograph */
  ["\u9214", "\u9214"], /* CJK Ideograph */
  ["\u9215", "\u9215"], /* CJK Ideograph */
  ["\u9216", "\u9216"], /* CJK Ideograph */
  ["\u9217", "\u9217"], /* CJK Ideograph */
  ["\u9218", "\u9218"], /* CJK Ideograph */
  ["\u9219", "\u9219"], /* CJK Ideograph */
  ["\u921A", "\u921A"], /* CJK Ideograph */
  ["\u921B", "\u921B"], /* CJK Ideograph */
  ["\u921C", "\u921C"], /* CJK Ideograph */
  ["\u921D", "\u921D"], /* CJK Ideograph */
  ["\u921E", "\u921E"], /* CJK Ideograph */
  ["\u921F", "\u921F"], /* CJK Ideograph */
  ["\u9220", "\u9220"], /* CJK Ideograph */
  ["\u9221", "\u9221"], /* CJK Ideograph */
  ["\u9222", "\u9222"], /* CJK Ideograph */
  ["\u9223", "\u9223"], /* CJK Ideograph */
  ["\u9224", "\u9224"], /* CJK Ideograph */
  ["\u9225", "\u9225"], /* CJK Ideograph */
  ["\u9226", "\u9226"], /* CJK Ideograph */
  ["\u9227", "\u9227"], /* CJK Ideograph */
  ["\u9228", "\u9228"], /* CJK Ideograph */
  ["\u9229", "\u9229"], /* CJK Ideograph */
  ["\u922A", "\u922A"], /* CJK Ideograph */
  ["\u922B", "\u922B"], /* CJK Ideograph */
  ["\u922C", "\u922C"], /* CJK Ideograph */
  ["\u922D", "\u922D"], /* CJK Ideograph */
  ["\u922E", "\u922E"], /* CJK Ideograph */
  ["\u922F", "\u922F"], /* CJK Ideograph */
  ["\u9230", "\u9230"], /* CJK Ideograph */
  ["\u9231", "\u9231"], /* CJK Ideograph */
  ["\u9232", "\u9232"], /* CJK Ideograph */
  ["\u9233", "\u9233"], /* CJK Ideograph */
  ["\u9234", "\u9234"], /* CJK Ideograph */
  ["\u9235", "\u9235"], /* CJK Ideograph */
  ["\u9236", "\u9236"], /* CJK Ideograph */
  ["\u9237", "\u9237"], /* CJK Ideograph */
  ["\u9238", "\u9238"], /* CJK Ideograph */
  ["\u9239", "\u9239"], /* CJK Ideograph */
  ["\u923A", "\u923A"], /* CJK Ideograph */
  ["\u923B", "\u923B"], /* CJK Ideograph */
  ["\u923C", "\u923C"], /* CJK Ideograph */
  ["\u923D", "\u923D"], /* CJK Ideograph */
  ["\u923E", "\u923E"], /* CJK Ideograph */
  ["\u923F", "\u923F"], /* CJK Ideograph */
  ["\u9240", "\u9240"], /* CJK Ideograph */
  ["\u9241", "\u9241"], /* CJK Ideograph */
  ["\u9242", "\u9242"], /* CJK Ideograph */
  ["\u9243", "\u9243"], /* CJK Ideograph */
  ["\u9244", "\u9244"], /* CJK Ideograph */
  ["\u9245", "\u9245"], /* CJK Ideograph */
  ["\u9246", "\u9246"], /* CJK Ideograph */
  ["\u9247", "\u9247"], /* CJK Ideograph */
  ["\u9248", "\u9248"], /* CJK Ideograph */
  ["\u9249", "\u9249"], /* CJK Ideograph */
  ["\u924A", "\u924A"], /* CJK Ideograph */
  ["\u924B", "\u924B"], /* CJK Ideograph */
  ["\u924C", "\u924C"], /* CJK Ideograph */
  ["\u924D", "\u924D"], /* CJK Ideograph */
  ["\u924E", "\u924E"], /* CJK Ideograph */
  ["\u924F", "\u924F"], /* CJK Ideograph */
  ["\u9250", "\u9250"], /* CJK Ideograph */
  ["\u9251", "\u9251"], /* CJK Ideograph */
  ["\u9252", "\u9252"], /* CJK Ideograph */
  ["\u9253", "\u9253"], /* CJK Ideograph */
  ["\u9254", "\u9254"], /* CJK Ideograph */
  ["\u9255", "\u9255"], /* CJK Ideograph */
  ["\u9256", "\u9256"], /* CJK Ideograph */
  ["\u9257", "\u9257"], /* CJK Ideograph */
  ["\u9258", "\u9258"], /* CJK Ideograph */
  ["\u9259", "\u9259"], /* CJK Ideograph */
  ["\u925A", "\u925A"], /* CJK Ideograph */
  ["\u925B", "\u925B"], /* CJK Ideograph */
  ["\u925C", "\u925C"], /* CJK Ideograph */
  ["\u925D", "\u925D"], /* CJK Ideograph */
  ["\u925E", "\u925E"], /* CJK Ideograph */
  ["\u925F", "\u925F"], /* CJK Ideograph */
  ["\u9260", "\u9260"], /* CJK Ideograph */
  ["\u9261", "\u9261"], /* CJK Ideograph */
  ["\u9262", "\u9262"], /* CJK Ideograph */
  ["\u9263", "\u9263"], /* CJK Ideograph */
  ["\u9264", "\u9264"], /* CJK Ideograph */
  ["\u9265", "\u9265"], /* CJK Ideograph */
  ["\u9266", "\u9266"], /* CJK Ideograph */
  ["\u9267", "\u9267"], /* CJK Ideograph */
  ["\u9268", "\u9268"], /* CJK Ideograph */
  ["\u9269", "\u9269"], /* CJK Ideograph */
  ["\u926A", "\u926A"], /* CJK Ideograph */
  ["\u926B", "\u926B"], /* CJK Ideograph */
  ["\u926C", "\u926C"], /* CJK Ideograph */
  ["\u926D", "\u926D"], /* CJK Ideograph */
  ["\u926E", "\u926E"], /* CJK Ideograph */
  ["\u926F", "\u926F"], /* CJK Ideograph */
  ["\u9270", "\u9270"], /* CJK Ideograph */
  ["\u9271", "\u9271"], /* CJK Ideograph */
  ["\u9272", "\u9272"], /* CJK Ideograph */
  ["\u9273", "\u9273"], /* CJK Ideograph */
  ["\u9274", "\u9274"], /* CJK Ideograph */
  ["\u9275", "\u9275"], /* CJK Ideograph */
  ["\u9276", "\u9276"], /* CJK Ideograph */
  ["\u9277", "\u9277"], /* CJK Ideograph */
  ["\u9278", "\u9278"], /* CJK Ideograph */
  ["\u9279", "\u9279"], /* CJK Ideograph */
  ["\u927A", "\u927A"], /* CJK Ideograph */
  ["\u927B", "\u927B"], /* CJK Ideograph */
  ["\u927C", "\u927C"], /* CJK Ideograph */
  ["\u927D", "\u927D"], /* CJK Ideograph */
  ["\u927E", "\u927E"], /* CJK Ideograph */
  ["\u927F", "\u927F"], /* CJK Ideograph */
  ["\u9280", "\u9280"], /* CJK Ideograph */
  ["\u9281", "\u9281"], /* CJK Ideograph */
  ["\u9282", "\u9282"], /* CJK Ideograph */
  ["\u9283", "\u9283"], /* CJK Ideograph */
  ["\u9284", "\u9284"], /* CJK Ideograph */
  ["\u9285", "\u9285"], /* CJK Ideograph */
  ["\u9286", "\u9286"], /* CJK Ideograph */
  ["\u9287", "\u9287"], /* CJK Ideograph */
  ["\u9288", "\u9288"], /* CJK Ideograph */
  ["\u9289", "\u9289"], /* CJK Ideograph */
  ["\u928A", "\u928A"], /* CJK Ideograph */
  ["\u928B", "\u928B"], /* CJK Ideograph */
  ["\u928C", "\u928C"], /* CJK Ideograph */
  ["\u928D", "\u928D"], /* CJK Ideograph */
  ["\u928E", "\u928E"], /* CJK Ideograph */
  ["\u928F", "\u928F"], /* CJK Ideograph */
  ["\u9290", "\u9290"], /* CJK Ideograph */
  ["\u9291", "\u9291"], /* CJK Ideograph */
  ["\u9292", "\u9292"], /* CJK Ideograph */
  ["\u9293", "\u9293"], /* CJK Ideograph */
  ["\u9294", "\u9294"], /* CJK Ideograph */
  ["\u9295", "\u9295"], /* CJK Ideograph */
  ["\u9296", "\u9296"], /* CJK Ideograph */
  ["\u9297", "\u9297"], /* CJK Ideograph */
  ["\u9298", "\u9298"], /* CJK Ideograph */
  ["\u9299", "\u9299"], /* CJK Ideograph */
  ["\u929A", "\u929A"], /* CJK Ideograph */
  ["\u929B", "\u929B"], /* CJK Ideograph */
  ["\u929C", "\u929C"], /* CJK Ideograph */
  ["\u929D", "\u929D"], /* CJK Ideograph */
  ["\u929E", "\u929E"], /* CJK Ideograph */
  ["\u929F", "\u929F"], /* CJK Ideograph */
  ["\u92A0", "\u92A0"], /* CJK Ideograph */
  ["\u92A1", "\u92A1"], /* CJK Ideograph */
  ["\u92A2", "\u92A2"], /* CJK Ideograph */
  ["\u92A3", "\u92A3"], /* CJK Ideograph */
  ["\u92A4", "\u92A4"], /* CJK Ideograph */
  ["\u92A5", "\u92A5"], /* CJK Ideograph */
  ["\u92A6", "\u92A6"], /* CJK Ideograph */
  ["\u92A7", "\u92A7"], /* CJK Ideograph */
  ["\u92A8", "\u92A8"], /* CJK Ideograph */
  ["\u92A9", "\u92A9"], /* CJK Ideograph */
  ["\u92AA", "\u92AA"], /* CJK Ideograph */
  ["\u92AB", "\u92AB"], /* CJK Ideograph */
  ["\u92AC", "\u92AC"], /* CJK Ideograph */
  ["\u92AD", "\u92AD"], /* CJK Ideograph */
  ["\u92AE", "\u92AE"], /* CJK Ideograph */
  ["\u92AF", "\u92AF"], /* CJK Ideograph */
  ["\u92B0", "\u92B0"], /* CJK Ideograph */
  ["\u92B1", "\u92B1"], /* CJK Ideograph */
  ["\u92B2", "\u92B2"], /* CJK Ideograph */
  ["\u92B3", "\u92B3"], /* CJK Ideograph */
  ["\u92B4", "\u92B4"], /* CJK Ideograph */
  ["\u92B5", "\u92B5"], /* CJK Ideograph */
  ["\u92B6", "\u92B6"], /* CJK Ideograph */
  ["\u92B7", "\u92B7"], /* CJK Ideograph */
  ["\u92B8", "\u92B8"], /* CJK Ideograph */
  ["\u92B9", "\u92B9"], /* CJK Ideograph */
  ["\u92BA", "\u92BA"], /* CJK Ideograph */
  ["\u92BB", "\u92BB"], /* CJK Ideograph */
  ["\u92BC", "\u92BC"], /* CJK Ideograph */
  ["\u92BD", "\u92BD"], /* CJK Ideograph */
  ["\u92BE", "\u92BE"], /* CJK Ideograph */
  ["\u92BF", "\u92BF"], /* CJK Ideograph */
  ["\u92C0", "\u92C0"], /* CJK Ideograph */
  ["\u92C1", "\u92C1"], /* CJK Ideograph */
  ["\u92C2", "\u92C2"], /* CJK Ideograph */
  ["\u92C3", "\u92C3"], /* CJK Ideograph */
  ["\u92C4", "\u92C4"], /* CJK Ideograph */
  ["\u92C5", "\u92C5"], /* CJK Ideograph */
  ["\u92C6", "\u92C6"], /* CJK Ideograph */
  ["\u92C7", "\u92C7"], /* CJK Ideograph */
  ["\u92C8", "\u92C8"], /* CJK Ideograph */
  ["\u92C9", "\u92C9"], /* CJK Ideograph */
  ["\u92CA", "\u92CA"], /* CJK Ideograph */
  ["\u92CB", "\u92CB"], /* CJK Ideograph */
  ["\u92CC", "\u92CC"], /* CJK Ideograph */
  ["\u92CD", "\u92CD"], /* CJK Ideograph */
  ["\u92CE", "\u92CE"], /* CJK Ideograph */
  ["\u92CF", "\u92CF"], /* CJK Ideograph */
  ["\u92D0", "\u92D0"], /* CJK Ideograph */
  ["\u92D1", "\u92D1"], /* CJK Ideograph */
  ["\u92D2", "\u92D2"], /* CJK Ideograph */
  ["\u92D3", "\u92D3"], /* CJK Ideograph */
  ["\u92D4", "\u92D4"], /* CJK Ideograph */
  ["\u92D5", "\u92D5"], /* CJK Ideograph */
  ["\u92D6", "\u92D6"], /* CJK Ideograph */
  ["\u92D7", "\u92D7"], /* CJK Ideograph */
  ["\u92D8", "\u92D8"], /* CJK Ideograph */
  ["\u92D9", "\u92D9"], /* CJK Ideograph */
  ["\u92DA", "\u92DA"], /* CJK Ideograph */
  ["\u92DB", "\u92DB"], /* CJK Ideograph */
  ["\u92DC", "\u92DC"], /* CJK Ideograph */
  ["\u92DD", "\u92DD"], /* CJK Ideograph */
  ["\u92DE", "\u92DE"], /* CJK Ideograph */
  ["\u92DF", "\u92DF"], /* CJK Ideograph */
  ["\u92E0", "\u92E0"], /* CJK Ideograph */
  ["\u92E1", "\u92E1"], /* CJK Ideograph */
  ["\u92E2", "\u92E2"], /* CJK Ideograph */
  ["\u92E3", "\u92E3"], /* CJK Ideograph */
  ["\u92E4", "\u92E4"], /* CJK Ideograph */
  ["\u92E5", "\u92E5"], /* CJK Ideograph */
  ["\u92E6", "\u92E6"], /* CJK Ideograph */
  ["\u92E7", "\u92E7"], /* CJK Ideograph */
  ["\u92E8", "\u92E8"], /* CJK Ideograph */
  ["\u92E9", "\u92E9"], /* CJK Ideograph */
  ["\u92EA", "\u92EA"], /* CJK Ideograph */
  ["\u92EB", "\u92EB"], /* CJK Ideograph */
  ["\u92EC", "\u92EC"], /* CJK Ideograph */
  ["\u92ED", "\u92ED"], /* CJK Ideograph */
  ["\u92EE", "\u92EE"], /* CJK Ideograph */
  ["\u92EF", "\u92EF"], /* CJK Ideograph */
  ["\u92F0", "\u92F0"], /* CJK Ideograph */
  ["\u92F1", "\u92F1"], /* CJK Ideograph */
  ["\u92F2", "\u92F2"], /* CJK Ideograph */
  ["\u92F3", "\u92F3"], /* CJK Ideograph */
  ["\u92F4", "\u92F4"], /* CJK Ideograph */
  ["\u92F5", "\u92F5"], /* CJK Ideograph */
  ["\u92F6", "\u92F6"], /* CJK Ideograph */
  ["\u92F7", "\u92F7"], /* CJK Ideograph */
  ["\u92F8", "\u92F8"], /* CJK Ideograph */
  ["\u92F9", "\u92F9"], /* CJK Ideograph */
  ["\u92FA", "\u92FA"], /* CJK Ideograph */
  ["\u92FB", "\u92FB"], /* CJK Ideograph */
  ["\u92FC", "\u92FC"], /* CJK Ideograph */
  ["\u92FD", "\u92FD"], /* CJK Ideograph */
  ["\u92FE", "\u92FE"], /* CJK Ideograph */
  ["\u92FF", "\u92FF"], /* CJK Ideograph */
  ["\u9300", "\u9300"], /* CJK Ideograph */
  ["\u9301", "\u9301"], /* CJK Ideograph */
  ["\u9302", "\u9302"], /* CJK Ideograph */
  ["\u9303", "\u9303"], /* CJK Ideograph */
  ["\u9304", "\u9304"], /* CJK Ideograph */
  ["\u9305", "\u9305"], /* CJK Ideograph */
  ["\u9306", "\u9306"], /* CJK Ideograph */
  ["\u9307", "\u9307"], /* CJK Ideograph */
  ["\u9308", "\u9308"], /* CJK Ideograph */
  ["\u9309", "\u9309"], /* CJK Ideograph */
  ["\u930A", "\u930A"], /* CJK Ideograph */
  ["\u930B", "\u930B"], /* CJK Ideograph */
  ["\u930C", "\u930C"], /* CJK Ideograph */
  ["\u930D", "\u930D"], /* CJK Ideograph */
  ["\u930E", "\u930E"], /* CJK Ideograph */
  ["\u930F", "\u930F"], /* CJK Ideograph */
  ["\u9310", "\u9310"], /* CJK Ideograph */
  ["\u9311", "\u9311"], /* CJK Ideograph */
  ["\u9312", "\u9312"], /* CJK Ideograph */
  ["\u9313", "\u9313"], /* CJK Ideograph */
  ["\u9314", "\u9314"], /* CJK Ideograph */
  ["\u9315", "\u9315"], /* CJK Ideograph */
  ["\u9316", "\u9316"], /* CJK Ideograph */
  ["\u9317", "\u9317"], /* CJK Ideograph */
  ["\u9318", "\u9318"], /* CJK Ideograph */
  ["\u9319", "\u9319"], /* CJK Ideograph */
  ["\u931A", "\u931A"], /* CJK Ideograph */
  ["\u931B", "\u931B"], /* CJK Ideograph */
  ["\u931C", "\u931C"], /* CJK Ideograph */
  ["\u931D", "\u931D"], /* CJK Ideograph */
  ["\u931E", "\u931E"], /* CJK Ideograph */
  ["\u931F", "\u931F"], /* CJK Ideograph */
  ["\u9320", "\u9320"], /* CJK Ideograph */
  ["\u9321", "\u9321"], /* CJK Ideograph */
  ["\u9322", "\u9322"], /* CJK Ideograph */
  ["\u9323", "\u9323"], /* CJK Ideograph */
  ["\u9324", "\u9324"], /* CJK Ideograph */
  ["\u9325", "\u9325"], /* CJK Ideograph */
  ["\u9326", "\u9326"], /* CJK Ideograph */
  ["\u9327", "\u9327"], /* CJK Ideograph */
  ["\u9328", "\u9328"], /* CJK Ideograph */
  ["\u9329", "\u9329"], /* CJK Ideograph */
  ["\u932A", "\u932A"], /* CJK Ideograph */
  ["\u932B", "\u932B"], /* CJK Ideograph */
  ["\u932C", "\u932C"], /* CJK Ideograph */
  ["\u932D", "\u932D"], /* CJK Ideograph */
  ["\u932E", "\u932E"], /* CJK Ideograph */
  ["\u932F", "\u932F"], /* CJK Ideograph */
  ["\u9330", "\u9330"], /* CJK Ideograph */
  ["\u9331", "\u9331"], /* CJK Ideograph */
  ["\u9332", "\u9332"], /* CJK Ideograph */
  ["\u9333", "\u9333"], /* CJK Ideograph */
  ["\u9334", "\u9334"], /* CJK Ideograph */
  ["\u9335", "\u9335"], /* CJK Ideograph */
  ["\u9336", "\u9336"], /* CJK Ideograph */
  ["\u9337", "\u9337"], /* CJK Ideograph */
  ["\u9338", "\u9338"], /* CJK Ideograph */
  ["\u9339", "\u9339"], /* CJK Ideograph */
  ["\u933A", "\u933A"], /* CJK Ideograph */
  ["\u933B", "\u933B"], /* CJK Ideograph */
  ["\u933C", "\u933C"], /* CJK Ideograph */
  ["\u933D", "\u933D"], /* CJK Ideograph */
  ["\u933E", "\u933E"], /* CJK Ideograph */
  ["\u933F", "\u933F"], /* CJK Ideograph */
  ["\u9340", "\u9340"], /* CJK Ideograph */
  ["\u9341", "\u9341"], /* CJK Ideograph */
  ["\u9342", "\u9342"], /* CJK Ideograph */
  ["\u9343", "\u9343"], /* CJK Ideograph */
  ["\u9344", "\u9344"], /* CJK Ideograph */
  ["\u9345", "\u9345"], /* CJK Ideograph */
  ["\u9346", "\u9346"], /* CJK Ideograph */
  ["\u9347", "\u9347"], /* CJK Ideograph */
  ["\u9348", "\u9348"], /* CJK Ideograph */
  ["\u9349", "\u9349"], /* CJK Ideograph */
  ["\u934A", "\u934A"], /* CJK Ideograph */
  ["\u934B", "\u934B"], /* CJK Ideograph */
  ["\u934C", "\u934C"], /* CJK Ideograph */
  ["\u934D", "\u934D"], /* CJK Ideograph */
  ["\u934E", "\u934E"], /* CJK Ideograph */
  ["\u934F", "\u934F"], /* CJK Ideograph */
  ["\u9350", "\u9350"], /* CJK Ideograph */
  ["\u9351", "\u9351"], /* CJK Ideograph */
  ["\u9352", "\u9352"], /* CJK Ideograph */
  ["\u9353", "\u9353"], /* CJK Ideograph */
  ["\u9354", "\u9354"], /* CJK Ideograph */
  ["\u9355", "\u9355"], /* CJK Ideograph */
  ["\u9356", "\u9356"], /* CJK Ideograph */
  ["\u9357", "\u9357"], /* CJK Ideograph */
  ["\u9358", "\u9358"], /* CJK Ideograph */
  ["\u9359", "\u9359"], /* CJK Ideograph */
  ["\u935A", "\u935A"], /* CJK Ideograph */
  ["\u935B", "\u935B"], /* CJK Ideograph */
  ["\u935C", "\u935C"], /* CJK Ideograph */
  ["\u935D", "\u935D"], /* CJK Ideograph */
  ["\u935E", "\u935E"], /* CJK Ideograph */
  ["\u935F", "\u935F"], /* CJK Ideograph */
  ["\u9360", "\u9360"], /* CJK Ideograph */
  ["\u9361", "\u9361"], /* CJK Ideograph */
  ["\u9362", "\u9362"], /* CJK Ideograph */
  ["\u9363", "\u9363"], /* CJK Ideograph */
  ["\u9364", "\u9364"], /* CJK Ideograph */
  ["\u9365", "\u9365"], /* CJK Ideograph */
  ["\u9366", "\u9366"], /* CJK Ideograph */
  ["\u9367", "\u9367"], /* CJK Ideograph */
  ["\u9368", "\u9368"], /* CJK Ideograph */
  ["\u9369", "\u9369"], /* CJK Ideograph */
  ["\u936A", "\u936A"], /* CJK Ideograph */
  ["\u936B", "\u936B"], /* CJK Ideograph */
  ["\u936C", "\u936C"], /* CJK Ideograph */
  ["\u936D", "\u936D"], /* CJK Ideograph */
  ["\u936E", "\u936E"], /* CJK Ideograph */
  ["\u936F", "\u936F"], /* CJK Ideograph */
  ["\u9370", "\u9370"], /* CJK Ideograph */
  ["\u9371", "\u9371"], /* CJK Ideograph */
  ["\u9372", "\u9372"], /* CJK Ideograph */
  ["\u9373", "\u9373"], /* CJK Ideograph */
  ["\u9374", "\u9374"], /* CJK Ideograph */
  ["\u9375", "\u9375"], /* CJK Ideograph */
  ["\u9376", "\u9376"], /* CJK Ideograph */
  ["\u9377", "\u9377"], /* CJK Ideograph */
  ["\u9378", "\u9378"], /* CJK Ideograph */
  ["\u9379", "\u9379"], /* CJK Ideograph */
  ["\u937A", "\u937A"], /* CJK Ideograph */
  ["\u937B", "\u937B"], /* CJK Ideograph */
  ["\u937C", "\u937C"], /* CJK Ideograph */
  ["\u937D", "\u937D"], /* CJK Ideograph */
  ["\u937E", "\u937E"], /* CJK Ideograph */
  ["\u937F", "\u937F"], /* CJK Ideograph */
  ["\u9380", "\u9380"], /* CJK Ideograph */
  ["\u9381", "\u9381"], /* CJK Ideograph */
  ["\u9382", "\u9382"], /* CJK Ideograph */
  ["\u9383", "\u9383"], /* CJK Ideograph */
  ["\u9384", "\u9384"], /* CJK Ideograph */
  ["\u9385", "\u9385"], /* CJK Ideograph */
  ["\u9386", "\u9386"], /* CJK Ideograph */
  ["\u9387", "\u9387"], /* CJK Ideograph */
  ["\u9388", "\u9388"], /* CJK Ideograph */
  ["\u9389", "\u9389"], /* CJK Ideograph */
  ["\u938A", "\u938A"], /* CJK Ideograph */
  ["\u938B", "\u938B"], /* CJK Ideograph */
  ["\u938C", "\u938C"], /* CJK Ideograph */
  ["\u938D", "\u938D"], /* CJK Ideograph */
  ["\u938E", "\u938E"], /* CJK Ideograph */
  ["\u938F", "\u938F"], /* CJK Ideograph */
  ["\u9390", "\u9390"], /* CJK Ideograph */
  ["\u9391", "\u9391"], /* CJK Ideograph */
  ["\u9392", "\u9392"], /* CJK Ideograph */
  ["\u9393", "\u9393"], /* CJK Ideograph */
  ["\u9394", "\u9394"], /* CJK Ideograph */
  ["\u9395", "\u9395"], /* CJK Ideograph */
  ["\u9396", "\u9396"], /* CJK Ideograph */
  ["\u9397", "\u9397"], /* CJK Ideograph */
  ["\u9398", "\u9398"], /* CJK Ideograph */
  ["\u9399", "\u9399"], /* CJK Ideograph */
  ["\u939A", "\u939A"], /* CJK Ideograph */
  ["\u939B", "\u939B"], /* CJK Ideograph */
  ["\u939C", "\u939C"], /* CJK Ideograph */
  ["\u939D", "\u939D"], /* CJK Ideograph */
  ["\u939E", "\u939E"], /* CJK Ideograph */
  ["\u939F", "\u939F"], /* CJK Ideograph */
  ["\u93A0", "\u93A0"], /* CJK Ideograph */
  ["\u93A1", "\u93A1"], /* CJK Ideograph */
  ["\u93A2", "\u93A2"], /* CJK Ideograph */
  ["\u93A3", "\u93A3"], /* CJK Ideograph */
  ["\u93A4", "\u93A4"], /* CJK Ideograph */
  ["\u93A5", "\u93A5"], /* CJK Ideograph */
  ["\u93A6", "\u93A6"], /* CJK Ideograph */
  ["\u93A7", "\u93A7"], /* CJK Ideograph */
  ["\u93A8", "\u93A8"], /* CJK Ideograph */
  ["\u93A9", "\u93A9"], /* CJK Ideograph */
  ["\u93AA", "\u93AA"], /* CJK Ideograph */
  ["\u93AB", "\u93AB"], /* CJK Ideograph */
  ["\u93AC", "\u93AC"], /* CJK Ideograph */
  ["\u93AD", "\u93AD"], /* CJK Ideograph */
  ["\u93AE", "\u93AE"], /* CJK Ideograph */
  ["\u93AF", "\u93AF"], /* CJK Ideograph */
  ["\u93B0", "\u93B0"], /* CJK Ideograph */
  ["\u93B1", "\u93B1"], /* CJK Ideograph */
  ["\u93B2", "\u93B2"], /* CJK Ideograph */
  ["\u93B3", "\u93B3"], /* CJK Ideograph */
  ["\u93B4", "\u93B4"], /* CJK Ideograph */
  ["\u93B5", "\u93B5"], /* CJK Ideograph */
  ["\u93B6", "\u93B6"], /* CJK Ideograph */
  ["\u93B7", "\u93B7"], /* CJK Ideograph */
  ["\u93B8", "\u93B8"], /* CJK Ideograph */
  ["\u93B9", "\u93B9"], /* CJK Ideograph */
  ["\u93BA", "\u93BA"], /* CJK Ideograph */
  ["\u93BB", "\u93BB"], /* CJK Ideograph */
  ["\u93BC", "\u93BC"], /* CJK Ideograph */
  ["\u93BD", "\u93BD"], /* CJK Ideograph */
  ["\u93BE", "\u93BE"], /* CJK Ideograph */
  ["\u93BF", "\u93BF"], /* CJK Ideograph */
  ["\u93C0", "\u93C0"], /* CJK Ideograph */
  ["\u93C1", "\u93C1"], /* CJK Ideograph */
  ["\u93C2", "\u93C2"], /* CJK Ideograph */
  ["\u93C3", "\u93C3"], /* CJK Ideograph */
  ["\u93C4", "\u93C4"], /* CJK Ideograph */
  ["\u93C5", "\u93C5"], /* CJK Ideograph */
  ["\u93C6", "\u93C6"], /* CJK Ideograph */
  ["\u93C7", "\u93C7"], /* CJK Ideograph */
  ["\u93C8", "\u93C8"], /* CJK Ideograph */
  ["\u93C9", "\u93C9"], /* CJK Ideograph */
  ["\u93CA", "\u93CA"], /* CJK Ideograph */
  ["\u93CB", "\u93CB"], /* CJK Ideograph */
  ["\u93CC", "\u93CC"], /* CJK Ideograph */
  ["\u93CD", "\u93CD"], /* CJK Ideograph */
  ["\u93CE", "\u93CE"], /* CJK Ideograph */
  ["\u93CF", "\u93CF"], /* CJK Ideograph */
  ["\u93D0", "\u93D0"], /* CJK Ideograph */
  ["\u93D1", "\u93D1"], /* CJK Ideograph */
  ["\u93D2", "\u93D2"], /* CJK Ideograph */
  ["\u93D3", "\u93D3"], /* CJK Ideograph */
  ["\u93D4", "\u93D4"], /* CJK Ideograph */
  ["\u93D5", "\u93D5"], /* CJK Ideograph */
  ["\u93D6", "\u93D6"], /* CJK Ideograph */
  ["\u93D7", "\u93D7"], /* CJK Ideograph */
  ["\u93D8", "\u93D8"], /* CJK Ideograph */
  ["\u93D9", "\u93D9"], /* CJK Ideograph */
  ["\u93DA", "\u93DA"], /* CJK Ideograph */
  ["\u93DB", "\u93DB"], /* CJK Ideograph */
  ["\u93DC", "\u93DC"], /* CJK Ideograph */
  ["\u93DD", "\u93DD"], /* CJK Ideograph */
  ["\u93DE", "\u93DE"], /* CJK Ideograph */
  ["\u93DF", "\u93DF"], /* CJK Ideograph */
  ["\u93E0", "\u93E0"], /* CJK Ideograph */
  ["\u93E1", "\u93E1"], /* CJK Ideograph */
  ["\u93E2", "\u93E2"], /* CJK Ideograph */
  ["\u93E3", "\u93E3"], /* CJK Ideograph */
  ["\u93E4", "\u93E4"], /* CJK Ideograph */
  ["\u93E5", "\u93E5"], /* CJK Ideograph */
  ["\u93E6", "\u93E6"], /* CJK Ideograph */
  ["\u93E7", "\u93E7"], /* CJK Ideograph */
  ["\u93E8", "\u93E8"], /* CJK Ideograph */
  ["\u93E9", "\u93E9"], /* CJK Ideograph */
  ["\u93EA", "\u93EA"], /* CJK Ideograph */
  ["\u93EB", "\u93EB"], /* CJK Ideograph */
  ["\u93EC", "\u93EC"], /* CJK Ideograph */
  ["\u93ED", "\u93ED"], /* CJK Ideograph */
  ["\u93EE", "\u93EE"], /* CJK Ideograph */
  ["\u93EF", "\u93EF"], /* CJK Ideograph */
  ["\u93F0", "\u93F0"], /* CJK Ideograph */
  ["\u93F1", "\u93F1"], /* CJK Ideograph */
  ["\u93F2", "\u93F2"], /* CJK Ideograph */
  ["\u93F3", "\u93F3"], /* CJK Ideograph */
  ["\u93F4", "\u93F4"], /* CJK Ideograph */
  ["\u93F5", "\u93F5"], /* CJK Ideograph */
  ["\u93F6", "\u93F6"], /* CJK Ideograph */
  ["\u93F7", "\u93F7"], /* CJK Ideograph */
  ["\u93F8", "\u93F8"], /* CJK Ideograph */
  ["\u93F9", "\u93F9"], /* CJK Ideograph */
  ["\u93FA", "\u93FA"], /* CJK Ideograph */
  ["\u93FB", "\u93FB"], /* CJK Ideograph */
  ["\u93FC", "\u93FC"], /* CJK Ideograph */
  ["\u93FD", "\u93FD"], /* CJK Ideograph */
  ["\u93FE", "\u93FE"], /* CJK Ideograph */
  ["\u93FF", "\u93FF"], /* CJK Ideograph */
  ["\u9400", "\u9400"], /* CJK Ideograph */
  ["\u9401", "\u9401"], /* CJK Ideograph */
  ["\u9402", "\u9402"], /* CJK Ideograph */
  ["\u9403", "\u9403"], /* CJK Ideograph */
  ["\u9404", "\u9404"], /* CJK Ideograph */
  ["\u9405", "\u9405"], /* CJK Ideograph */
  ["\u9406", "\u9406"], /* CJK Ideograph */
  ["\u9407", "\u9407"], /* CJK Ideograph */
  ["\u9408", "\u9408"], /* CJK Ideograph */
  ["\u9409", "\u9409"], /* CJK Ideograph */
  ["\u940A", "\u940A"], /* CJK Ideograph */
  ["\u940B", "\u940B"], /* CJK Ideograph */
  ["\u940C", "\u940C"], /* CJK Ideograph */
  ["\u940D", "\u940D"], /* CJK Ideograph */
  ["\u940E", "\u940E"], /* CJK Ideograph */
  ["\u940F", "\u940F"], /* CJK Ideograph */
  ["\u9410", "\u9410"], /* CJK Ideograph */
  ["\u9411", "\u9411"], /* CJK Ideograph */
  ["\u9412", "\u9412"], /* CJK Ideograph */
  ["\u9413", "\u9413"], /* CJK Ideograph */
  ["\u9414", "\u9414"], /* CJK Ideograph */
  ["\u9415", "\u9415"], /* CJK Ideograph */
  ["\u9416", "\u9416"], /* CJK Ideograph */
  ["\u9417", "\u9417"], /* CJK Ideograph */
  ["\u9418", "\u9418"], /* CJK Ideograph */
  ["\u9419", "\u9419"], /* CJK Ideograph */
  ["\u941A", "\u941A"], /* CJK Ideograph */
  ["\u941B", "\u941B"], /* CJK Ideograph */
  ["\u941C", "\u941C"], /* CJK Ideograph */
  ["\u941D", "\u941D"], /* CJK Ideograph */
  ["\u941E", "\u941E"], /* CJK Ideograph */
  ["\u941F", "\u941F"], /* CJK Ideograph */
  ["\u9420", "\u9420"], /* CJK Ideograph */
  ["\u9421", "\u9421"], /* CJK Ideograph */
  ["\u9422", "\u9422"], /* CJK Ideograph */
  ["\u9423", "\u9423"], /* CJK Ideograph */
  ["\u9424", "\u9424"], /* CJK Ideograph */
  ["\u9425", "\u9425"], /* CJK Ideograph */
  ["\u9426", "\u9426"], /* CJK Ideograph */
  ["\u9427", "\u9427"], /* CJK Ideograph */
  ["\u9428", "\u9428"], /* CJK Ideograph */
  ["\u9429", "\u9429"], /* CJK Ideograph */
  ["\u942A", "\u942A"], /* CJK Ideograph */
  ["\u942B", "\u942B"], /* CJK Ideograph */
  ["\u942C", "\u942C"], /* CJK Ideograph */
  ["\u942D", "\u942D"], /* CJK Ideograph */
  ["\u942E", "\u942E"], /* CJK Ideograph */
  ["\u942F", "\u942F"], /* CJK Ideograph */
  ["\u9430", "\u9430"], /* CJK Ideograph */
  ["\u9431", "\u9431"], /* CJK Ideograph */
  ["\u9432", "\u9432"], /* CJK Ideograph */
  ["\u9433", "\u9433"], /* CJK Ideograph */
  ["\u9434", "\u9434"], /* CJK Ideograph */
  ["\u9435", "\u9435"], /* CJK Ideograph */
  ["\u9436", "\u9436"], /* CJK Ideograph */
  ["\u9437", "\u9437"], /* CJK Ideograph */
  ["\u9438", "\u9438"], /* CJK Ideograph */
  ["\u9439", "\u9439"], /* CJK Ideograph */
  ["\u943A", "\u943A"], /* CJK Ideograph */
  ["\u943B", "\u943B"], /* CJK Ideograph */
  ["\u943C", "\u943C"], /* CJK Ideograph */
  ["\u943D", "\u943D"], /* CJK Ideograph */
  ["\u943E", "\u943E"], /* CJK Ideograph */
  ["\u943F", "\u943F"], /* CJK Ideograph */
  ["\u9440", "\u9440"], /* CJK Ideograph */
  ["\u9441", "\u9441"], /* CJK Ideograph */
  ["\u9442", "\u9442"], /* CJK Ideograph */
  ["\u9443", "\u9443"], /* CJK Ideograph */
  ["\u9444", "\u9444"], /* CJK Ideograph */
  ["\u9445", "\u9445"], /* CJK Ideograph */
  ["\u9446", "\u9446"], /* CJK Ideograph */
  ["\u9447", "\u9447"], /* CJK Ideograph */
  ["\u9448", "\u9448"], /* CJK Ideograph */
  ["\u9449", "\u9449"], /* CJK Ideograph */
  ["\u944A", "\u944A"], /* CJK Ideograph */
  ["\u944B", "\u944B"], /* CJK Ideograph */
  ["\u944C", "\u944C"], /* CJK Ideograph */
  ["\u944D", "\u944D"], /* CJK Ideograph */
  ["\u944E", "\u944E"], /* CJK Ideograph */
  ["\u944F", "\u944F"], /* CJK Ideograph */
  ["\u9450", "\u9450"], /* CJK Ideograph */
  ["\u9451", "\u9451"], /* CJK Ideograph */
  ["\u9452", "\u9452"], /* CJK Ideograph */
  ["\u9453", "\u9453"], /* CJK Ideograph */
  ["\u9454", "\u9454"], /* CJK Ideograph */
  ["\u9455", "\u9455"], /* CJK Ideograph */
  ["\u9456", "\u9456"], /* CJK Ideograph */
  ["\u9457", "\u9457"], /* CJK Ideograph */
  ["\u9458", "\u9458"], /* CJK Ideograph */
  ["\u9459", "\u9459"], /* CJK Ideograph */
  ["\u945A", "\u945A"], /* CJK Ideograph */
  ["\u945B", "\u945B"], /* CJK Ideograph */
  ["\u945C", "\u945C"], /* CJK Ideograph */
  ["\u945D", "\u945D"], /* CJK Ideograph */
  ["\u945E", "\u945E"], /* CJK Ideograph */
  ["\u945F", "\u945F"], /* CJK Ideograph */
  ["\u9460", "\u9460"], /* CJK Ideograph */
  ["\u9461", "\u9461"], /* CJK Ideograph */
  ["\u9462", "\u9462"], /* CJK Ideograph */
  ["\u9463", "\u9463"], /* CJK Ideograph */
  ["\u9464", "\u9464"], /* CJK Ideograph */
  ["\u9465", "\u9465"], /* CJK Ideograph */
  ["\u9466", "\u9466"], /* CJK Ideograph */
  ["\u9467", "\u9467"], /* CJK Ideograph */
  ["\u9468", "\u9468"], /* CJK Ideograph */
  ["\u9469", "\u9469"], /* CJK Ideograph */
  ["\u946A", "\u946A"], /* CJK Ideograph */
  ["\u946B", "\u946B"], /* CJK Ideograph */
  ["\u946C", "\u946C"], /* CJK Ideograph */
  ["\u946D", "\u946D"], /* CJK Ideograph */
  ["\u946E", "\u946E"], /* CJK Ideograph */
  ["\u946F", "\u946F"], /* CJK Ideograph */
  ["\u9470", "\u9470"], /* CJK Ideograph */
  ["\u9471", "\u9471"], /* CJK Ideograph */
  ["\u9472", "\u9472"], /* CJK Ideograph */
  ["\u9473", "\u9473"], /* CJK Ideograph */
  ["\u9474", "\u9474"], /* CJK Ideograph */
  ["\u9475", "\u9475"], /* CJK Ideograph */
  ["\u9476", "\u9476"], /* CJK Ideograph */
  ["\u9477", "\u9477"], /* CJK Ideograph */
  ["\u9478", "\u9478"], /* CJK Ideograph */
  ["\u9479", "\u9479"], /* CJK Ideograph */
  ["\u947A", "\u947A"], /* CJK Ideograph */
  ["\u947B", "\u947B"], /* CJK Ideograph */
  ["\u947C", "\u947C"], /* CJK Ideograph */
  ["\u947D", "\u947D"], /* CJK Ideograph */
  ["\u947E", "\u947E"], /* CJK Ideograph */
  ["\u947F", "\u947F"], /* CJK Ideograph */
  ["\u9480", "\u9480"], /* CJK Ideograph */
  ["\u9481", "\u9481"], /* CJK Ideograph */
  ["\u9482", "\u9482"], /* CJK Ideograph */
  ["\u9483", "\u9483"], /* CJK Ideograph */
  ["\u9484", "\u9484"], /* CJK Ideograph */
  ["\u9485", "\u9485"], /* CJK Ideograph */
  ["\u9486", "\u9486"], /* CJK Ideograph */
  ["\u9487", "\u9487"], /* CJK Ideograph */
  ["\u9488", "\u9488"], /* CJK Ideograph */
  ["\u9489", "\u9489"], /* CJK Ideograph */
  ["\u948A", "\u948A"], /* CJK Ideograph */
  ["\u948B", "\u948B"], /* CJK Ideograph */
  ["\u948C", "\u948C"], /* CJK Ideograph */
  ["\u948D", "\u948D"], /* CJK Ideograph */
  ["\u948E", "\u948E"], /* CJK Ideograph */
  ["\u948F", "\u948F"], /* CJK Ideograph */
  ["\u9490", "\u9490"], /* CJK Ideograph */
  ["\u9491", "\u9491"], /* CJK Ideograph */
  ["\u9492", "\u9492"], /* CJK Ideograph */
  ["\u9493", "\u9493"], /* CJK Ideograph */
  ["\u9494", "\u9494"], /* CJK Ideograph */
  ["\u9495", "\u9495"], /* CJK Ideograph */
  ["\u9496", "\u9496"], /* CJK Ideograph */
  ["\u9497", "\u9497"], /* CJK Ideograph */
  ["\u9498", "\u9498"], /* CJK Ideograph */
  ["\u9499", "\u9499"], /* CJK Ideograph */
  ["\u949A", "\u949A"], /* CJK Ideograph */
  ["\u949B", "\u949B"], /* CJK Ideograph */
  ["\u949C", "\u949C"], /* CJK Ideograph */
  ["\u949D", "\u949D"], /* CJK Ideograph */
  ["\u949E", "\u949E"], /* CJK Ideograph */
  ["\u949F", "\u949F"], /* CJK Ideograph */
  ["\u94A0", "\u94A0"], /* CJK Ideograph */
  ["\u94A1", "\u94A1"], /* CJK Ideograph */
  ["\u94A2", "\u94A2"], /* CJK Ideograph */
  ["\u94A3", "\u94A3"], /* CJK Ideograph */
  ["\u94A4", "\u94A4"], /* CJK Ideograph */
  ["\u94A5", "\u94A5"], /* CJK Ideograph */
  ["\u94A6", "\u94A6"], /* CJK Ideograph */
  ["\u94A7", "\u94A7"], /* CJK Ideograph */
  ["\u94A8", "\u94A8"], /* CJK Ideograph */
  ["\u94A9", "\u94A9"], /* CJK Ideograph */
  ["\u94AA", "\u94AA"], /* CJK Ideograph */
  ["\u94AB", "\u94AB"], /* CJK Ideograph */
  ["\u94AC", "\u94AC"], /* CJK Ideograph */
  ["\u94AD", "\u94AD"], /* CJK Ideograph */
  ["\u94AE", "\u94AE"], /* CJK Ideograph */
  ["\u94AF", "\u94AF"], /* CJK Ideograph */
  ["\u94B0", "\u94B0"], /* CJK Ideograph */
  ["\u94B1", "\u94B1"], /* CJK Ideograph */
  ["\u94B2", "\u94B2"], /* CJK Ideograph */
  ["\u94B3", "\u94B3"], /* CJK Ideograph */
  ["\u94B4", "\u94B4"], /* CJK Ideograph */
  ["\u94B5", "\u94B5"], /* CJK Ideograph */
  ["\u94B6", "\u94B6"], /* CJK Ideograph */
  ["\u94B7", "\u94B7"], /* CJK Ideograph */
  ["\u94B8", "\u94B8"], /* CJK Ideograph */
  ["\u94B9", "\u94B9"], /* CJK Ideograph */
  ["\u94BA", "\u94BA"], /* CJK Ideograph */
  ["\u94BB", "\u94BB"], /* CJK Ideograph */
  ["\u94BC", "\u94BC"], /* CJK Ideograph */
  ["\u94BD", "\u94BD"], /* CJK Ideograph */
  ["\u94BE", "\u94BE"], /* CJK Ideograph */
  ["\u94BF", "\u94BF"], /* CJK Ideograph */
  ["\u94C0", "\u94C0"], /* CJK Ideograph */
  ["\u94C1", "\u94C1"], /* CJK Ideograph */
  ["\u94C2", "\u94C2"], /* CJK Ideograph */
  ["\u94C3", "\u94C3"], /* CJK Ideograph */
  ["\u94C4", "\u94C4"], /* CJK Ideograph */
  ["\u94C5", "\u94C5"], /* CJK Ideograph */
  ["\u94C6", "\u94C6"], /* CJK Ideograph */
  ["\u94C7", "\u94C7"], /* CJK Ideograph */
  ["\u94C8", "\u94C8"], /* CJK Ideograph */
  ["\u94C9", "\u94C9"], /* CJK Ideograph */
  ["\u94CA", "\u94CA"], /* CJK Ideograph */
  ["\u94CB", "\u94CB"], /* CJK Ideograph */
  ["\u94CC", "\u94CC"], /* CJK Ideograph */
  ["\u94CD", "\u94CD"], /* CJK Ideograph */
  ["\u94CE", "\u94CE"], /* CJK Ideograph */
  ["\u94CF", "\u94CF"], /* CJK Ideograph */
  ["\u94D0", "\u94D0"], /* CJK Ideograph */
  ["\u94D1", "\u94D1"], /* CJK Ideograph */
  ["\u94D2", "\u94D2"], /* CJK Ideograph */
  ["\u94D3", "\u94D3"], /* CJK Ideograph */
  ["\u94D4", "\u94D4"], /* CJK Ideograph */
  ["\u94D5", "\u94D5"], /* CJK Ideograph */
  ["\u94D6", "\u94D6"], /* CJK Ideograph */
  ["\u94D7", "\u94D7"], /* CJK Ideograph */
  ["\u94D8", "\u94D8"], /* CJK Ideograph */
  ["\u94D9", "\u94D9"], /* CJK Ideograph */
  ["\u94DA", "\u94DA"], /* CJK Ideograph */
  ["\u94DB", "\u94DB"], /* CJK Ideograph */
  ["\u94DC", "\u94DC"], /* CJK Ideograph */
  ["\u94DD", "\u94DD"], /* CJK Ideograph */
  ["\u94DE", "\u94DE"], /* CJK Ideograph */
  ["\u94DF", "\u94DF"], /* CJK Ideograph */
  ["\u94E0", "\u94E0"], /* CJK Ideograph */
  ["\u94E1", "\u94E1"], /* CJK Ideograph */
  ["\u94E2", "\u94E2"], /* CJK Ideograph */
  ["\u94E3", "\u94E3"], /* CJK Ideograph */
  ["\u94E4", "\u94E4"], /* CJK Ideograph */
  ["\u94E5", "\u94E5"], /* CJK Ideograph */
  ["\u94E6", "\u94E6"], /* CJK Ideograph */
  ["\u94E7", "\u94E7"], /* CJK Ideograph */
  ["\u94E8", "\u94E8"], /* CJK Ideograph */
  ["\u94E9", "\u94E9"], /* CJK Ideograph */
  ["\u94EA", "\u94EA"], /* CJK Ideograph */
  ["\u94EB", "\u94EB"], /* CJK Ideograph */
  ["\u94EC", "\u94EC"], /* CJK Ideograph */
  ["\u94ED", "\u94ED"], /* CJK Ideograph */
  ["\u94EE", "\u94EE"], /* CJK Ideograph */
  ["\u94EF", "\u94EF"], /* CJK Ideograph */
  ["\u94F0", "\u94F0"], /* CJK Ideograph */
  ["\u94F1", "\u94F1"], /* CJK Ideograph */
  ["\u94F2", "\u94F2"], /* CJK Ideograph */
  ["\u94F3", "\u94F3"], /* CJK Ideograph */
  ["\u94F4", "\u94F4"], /* CJK Ideograph */
  ["\u94F5", "\u94F5"], /* CJK Ideograph */
  ["\u94F6", "\u94F6"], /* CJK Ideograph */
  ["\u94F7", "\u94F7"], /* CJK Ideograph */
  ["\u94F8", "\u94F8"], /* CJK Ideograph */
  ["\u94F9", "\u94F9"], /* CJK Ideograph */
  ["\u94FA", "\u94FA"], /* CJK Ideograph */
  ["\u94FB", "\u94FB"], /* CJK Ideograph */
  ["\u94FC", "\u94FC"], /* CJK Ideograph */
  ["\u94FD", "\u94FD"], /* CJK Ideograph */
  ["\u94FE", "\u94FE"], /* CJK Ideograph */
  ["\u94FF", "\u94FF"], /* CJK Ideograph */
  ["\u9500", "\u9500"], /* CJK Ideograph */
  ["\u9501", "\u9501"], /* CJK Ideograph */
  ["\u9502", "\u9502"], /* CJK Ideograph */
  ["\u9503", "\u9503"], /* CJK Ideograph */
  ["\u9504", "\u9504"], /* CJK Ideograph */
  ["\u9505", "\u9505"], /* CJK Ideograph */
  ["\u9506", "\u9506"], /* CJK Ideograph */
  ["\u9507", "\u9507"], /* CJK Ideograph */
  ["\u9508", "\u9508"], /* CJK Ideograph */
  ["\u9509", "\u9509"], /* CJK Ideograph */
  ["\u950A", "\u950A"], /* CJK Ideograph */
  ["\u950B", "\u950B"], /* CJK Ideograph */
  ["\u950C", "\u950C"], /* CJK Ideograph */
  ["\u950D", "\u950D"], /* CJK Ideograph */
  ["\u950E", "\u950E"], /* CJK Ideograph */
  ["\u950F", "\u950F"], /* CJK Ideograph */
  ["\u9510", "\u9510"], /* CJK Ideograph */
  ["\u9511", "\u9511"], /* CJK Ideograph */
  ["\u9512", "\u9512"], /* CJK Ideograph */
  ["\u9513", "\u9513"], /* CJK Ideograph */
  ["\u9514", "\u9514"], /* CJK Ideograph */
  ["\u9515", "\u9515"], /* CJK Ideograph */
  ["\u9516", "\u9516"], /* CJK Ideograph */
  ["\u9517", "\u9517"], /* CJK Ideograph */
  ["\u9518", "\u9518"], /* CJK Ideograph */
  ["\u9519", "\u9519"], /* CJK Ideograph */
  ["\u951A", "\u951A"], /* CJK Ideograph */
  ["\u951B", "\u951B"], /* CJK Ideograph */
  ["\u951C", "\u951C"], /* CJK Ideograph */
  ["\u951D", "\u951D"], /* CJK Ideograph */
  ["\u951E", "\u951E"], /* CJK Ideograph */
  ["\u951F", "\u951F"], /* CJK Ideograph */
  ["\u9520", "\u9520"], /* CJK Ideograph */
  ["\u9521", "\u9521"], /* CJK Ideograph */
  ["\u9522", "\u9522"], /* CJK Ideograph */
  ["\u9523", "\u9523"], /* CJK Ideograph */
  ["\u9524", "\u9524"], /* CJK Ideograph */
  ["\u9525", "\u9525"], /* CJK Ideograph */
  ["\u9526", "\u9526"], /* CJK Ideograph */
  ["\u9527", "\u9527"], /* CJK Ideograph */
  ["\u9528", "\u9528"], /* CJK Ideograph */
  ["\u9529", "\u9529"], /* CJK Ideograph */
  ["\u952A", "\u952A"], /* CJK Ideograph */
  ["\u952B", "\u952B"], /* CJK Ideograph */
  ["\u952C", "\u952C"], /* CJK Ideograph */
  ["\u952D", "\u952D"], /* CJK Ideograph */
  ["\u952E", "\u952E"], /* CJK Ideograph */
  ["\u952F", "\u952F"], /* CJK Ideograph */
  ["\u9530", "\u9530"], /* CJK Ideograph */
  ["\u9531", "\u9531"], /* CJK Ideograph */
  ["\u9532", "\u9532"], /* CJK Ideograph */
  ["\u9533", "\u9533"], /* CJK Ideograph */
  ["\u9534", "\u9534"], /* CJK Ideograph */
  ["\u9535", "\u9535"], /* CJK Ideograph */
  ["\u9536", "\u9536"], /* CJK Ideograph */
  ["\u9537", "\u9537"], /* CJK Ideograph */
  ["\u9538", "\u9538"], /* CJK Ideograph */
  ["\u9539", "\u9539"], /* CJK Ideograph */
  ["\u953A", "\u953A"], /* CJK Ideograph */
  ["\u953B", "\u953B"], /* CJK Ideograph */
  ["\u953C", "\u953C"], /* CJK Ideograph */
  ["\u953D", "\u953D"], /* CJK Ideograph */
  ["\u953E", "\u953E"], /* CJK Ideograph */
  ["\u953F", "\u953F"], /* CJK Ideograph */
  ["\u9540", "\u9540"], /* CJK Ideograph */
  ["\u9541", "\u9541"], /* CJK Ideograph */
  ["\u9542", "\u9542"], /* CJK Ideograph */
  ["\u9543", "\u9543"], /* CJK Ideograph */
  ["\u9544", "\u9544"], /* CJK Ideograph */
  ["\u9545", "\u9545"], /* CJK Ideograph */
  ["\u9546", "\u9546"], /* CJK Ideograph */
  ["\u9547", "\u9547"], /* CJK Ideograph */
  ["\u9548", "\u9548"], /* CJK Ideograph */
  ["\u9549", "\u9549"], /* CJK Ideograph */
  ["\u954A", "\u954A"], /* CJK Ideograph */
  ["\u954B", "\u954B"], /* CJK Ideograph */
  ["\u954C", "\u954C"], /* CJK Ideograph */
  ["\u954D", "\u954D"], /* CJK Ideograph */
  ["\u954E", "\u954E"], /* CJK Ideograph */
  ["\u954F", "\u954F"], /* CJK Ideograph */
  ["\u9550", "\u9550"], /* CJK Ideograph */
  ["\u9551", "\u9551"], /* CJK Ideograph */
  ["\u9552", "\u9552"], /* CJK Ideograph */
  ["\u9553", "\u9553"], /* CJK Ideograph */
  ["\u9554", "\u9554"], /* CJK Ideograph */
  ["\u9555", "\u9555"], /* CJK Ideograph */
  ["\u9556", "\u9556"], /* CJK Ideograph */
  ["\u9557", "\u9557"], /* CJK Ideograph */
  ["\u9558", "\u9558"], /* CJK Ideograph */
  ["\u9559", "\u9559"], /* CJK Ideograph */
  ["\u955A", "\u955A"], /* CJK Ideograph */
  ["\u955B", "\u955B"], /* CJK Ideograph */
  ["\u955C", "\u955C"], /* CJK Ideograph */
  ["\u955D", "\u955D"], /* CJK Ideograph */
  ["\u955E", "\u955E"], /* CJK Ideograph */
  ["\u955F", "\u955F"], /* CJK Ideograph */
  ["\u9560", "\u9560"], /* CJK Ideograph */
  ["\u9561", "\u9561"], /* CJK Ideograph */
  ["\u9562", "\u9562"], /* CJK Ideograph */
  ["\u9563", "\u9563"], /* CJK Ideograph */
  ["\u9564", "\u9564"], /* CJK Ideograph */
  ["\u9565", "\u9565"], /* CJK Ideograph */
  ["\u9566", "\u9566"], /* CJK Ideograph */
  ["\u9567", "\u9567"], /* CJK Ideograph */
  ["\u9568", "\u9568"], /* CJK Ideograph */
  ["\u9569", "\u9569"], /* CJK Ideograph */
  ["\u956A", "\u956A"], /* CJK Ideograph */
  ["\u956B", "\u956B"], /* CJK Ideograph */
  ["\u956C", "\u956C"], /* CJK Ideograph */
  ["\u956D", "\u956D"], /* CJK Ideograph */
  ["\u956E", "\u956E"], /* CJK Ideograph */
  ["\u956F", "\u956F"], /* CJK Ideograph */
  ["\u9570", "\u9570"], /* CJK Ideograph */
  ["\u9571", "\u9571"], /* CJK Ideograph */
  ["\u9572", "\u9572"], /* CJK Ideograph */
  ["\u9573", "\u9573"], /* CJK Ideograph */
  ["\u9574", "\u9574"], /* CJK Ideograph */
  ["\u9575", "\u9575"], /* CJK Ideograph */
  ["\u9576", "\u9576"], /* CJK Ideograph */
  ["\u9577", "\u9577"], /* CJK Ideograph */
  ["\u9578", "\u9578"], /* CJK Ideograph */
  ["\u9579", "\u9579"], /* CJK Ideograph */
  ["\u957A", "\u957A"], /* CJK Ideograph */
  ["\u957B", "\u957B"], /* CJK Ideograph */
  ["\u957C", "\u957C"], /* CJK Ideograph */
  ["\u957D", "\u957D"], /* CJK Ideograph */
  ["\u957E", "\u957E"], /* CJK Ideograph */
  ["\u957F", "\u957F"], /* CJK Ideograph */
  ["\u9580", "\u9580"], /* CJK Ideograph */
  ["\u9581", "\u9581"], /* CJK Ideograph */
  ["\u9582", "\u9582"], /* CJK Ideograph */
  ["\u9583", "\u9583"], /* CJK Ideograph */
  ["\u9584", "\u9584"], /* CJK Ideograph */
  ["\u9585", "\u9585"], /* CJK Ideograph */
  ["\u9586", "\u9586"], /* CJK Ideograph */
  ["\u9587", "\u9587"], /* CJK Ideograph */
  ["\u9588", "\u9588"], /* CJK Ideograph */
  ["\u9589", "\u9589"], /* CJK Ideograph */
  ["\u958A", "\u958A"], /* CJK Ideograph */
  ["\u958B", "\u958B"], /* CJK Ideograph */
  ["\u958C", "\u958C"], /* CJK Ideograph */
  ["\u958D", "\u958D"], /* CJK Ideograph */
  ["\u958E", "\u958E"], /* CJK Ideograph */
  ["\u958F", "\u958F"], /* CJK Ideograph */
  ["\u9590", "\u9590"], /* CJK Ideograph */
  ["\u9591", "\u9591"], /* CJK Ideograph */
  ["\u9592", "\u9592"], /* CJK Ideograph */
  ["\u9593", "\u9593"], /* CJK Ideograph */
  ["\u9594", "\u9594"], /* CJK Ideograph */
  ["\u9595", "\u9595"], /* CJK Ideograph */
  ["\u9596", "\u9596"], /* CJK Ideograph */
  ["\u9597", "\u9597"], /* CJK Ideograph */
  ["\u9598", "\u9598"], /* CJK Ideograph */
  ["\u9599", "\u9599"], /* CJK Ideograph */
  ["\u959A", "\u959A"], /* CJK Ideograph */
  ["\u959B", "\u959B"], /* CJK Ideograph */
  ["\u959C", "\u959C"], /* CJK Ideograph */
  ["\u959D", "\u959D"], /* CJK Ideograph */
  ["\u959E", "\u959E"], /* CJK Ideograph */
  ["\u959F", "\u959F"], /* CJK Ideograph */
  ["\u95A0", "\u95A0"], /* CJK Ideograph */
  ["\u95A1", "\u95A1"], /* CJK Ideograph */
  ["\u95A2", "\u95A2"], /* CJK Ideograph */
  ["\u95A3", "\u95A3"], /* CJK Ideograph */
  ["\u95A4", "\u95A4"], /* CJK Ideograph */
  ["\u95A5", "\u95A5"], /* CJK Ideograph */
  ["\u95A6", "\u95A6"], /* CJK Ideograph */
  ["\u95A7", "\u95A7"], /* CJK Ideograph */
  ["\u95A8", "\u95A8"], /* CJK Ideograph */
  ["\u95A9", "\u95A9"], /* CJK Ideograph */
  ["\u95AA", "\u95AA"], /* CJK Ideograph */
  ["\u95AB", "\u95AB"], /* CJK Ideograph */
  ["\u95AC", "\u95AC"], /* CJK Ideograph */
  ["\u95AD", "\u95AD"], /* CJK Ideograph */
  ["\u95AE", "\u95AE"], /* CJK Ideograph */
  ["\u95AF", "\u95AF"], /* CJK Ideograph */
  ["\u95B0", "\u95B0"], /* CJK Ideograph */
  ["\u95B1", "\u95B1"], /* CJK Ideograph */
  ["\u95B2", "\u95B2"], /* CJK Ideograph */
  ["\u95B3", "\u95B3"], /* CJK Ideograph */
  ["\u95B4", "\u95B4"], /* CJK Ideograph */
  ["\u95B5", "\u95B5"], /* CJK Ideograph */
  ["\u95B6", "\u95B6"], /* CJK Ideograph */
  ["\u95B7", "\u95B7"], /* CJK Ideograph */
  ["\u95B8", "\u95B8"], /* CJK Ideograph */
  ["\u95B9", "\u95B9"], /* CJK Ideograph */
  ["\u95BA", "\u95BA"], /* CJK Ideograph */
  ["\u95BB", "\u95BB"], /* CJK Ideograph */
  ["\u95BC", "\u95BC"], /* CJK Ideograph */
  ["\u95BD", "\u95BD"], /* CJK Ideograph */
  ["\u95BE", "\u95BE"], /* CJK Ideograph */
  ["\u95BF", "\u95BF"], /* CJK Ideograph */
  ["\u95C0", "\u95C0"], /* CJK Ideograph */
  ["\u95C1", "\u95C1"], /* CJK Ideograph */
  ["\u95C2", "\u95C2"], /* CJK Ideograph */
  ["\u95C3", "\u95C3"], /* CJK Ideograph */
  ["\u95C4", "\u95C4"], /* CJK Ideograph */
  ["\u95C5", "\u95C5"], /* CJK Ideograph */
  ["\u95C6", "\u95C6"], /* CJK Ideograph */
  ["\u95C7", "\u95C7"], /* CJK Ideograph */
  ["\u95C8", "\u95C8"], /* CJK Ideograph */
  ["\u95C9", "\u95C9"], /* CJK Ideograph */
  ["\u95CA", "\u95CA"], /* CJK Ideograph */
  ["\u95CB", "\u95CB"], /* CJK Ideograph */
  ["\u95CC", "\u95CC"], /* CJK Ideograph */
  ["\u95CD", "\u95CD"], /* CJK Ideograph */
  ["\u95CE", "\u95CE"], /* CJK Ideograph */
  ["\u95CF", "\u95CF"], /* CJK Ideograph */
  ["\u95D0", "\u95D0"], /* CJK Ideograph */
  ["\u95D1", "\u95D1"], /* CJK Ideograph */
  ["\u95D2", "\u95D2"], /* CJK Ideograph */
  ["\u95D3", "\u95D3"], /* CJK Ideograph */
  ["\u95D4", "\u95D4"], /* CJK Ideograph */
  ["\u95D5", "\u95D5"], /* CJK Ideograph */
  ["\u95D6", "\u95D6"], /* CJK Ideograph */
  ["\u95D7", "\u95D7"], /* CJK Ideograph */
  ["\u95D8", "\u95D8"], /* CJK Ideograph */
  ["\u95D9", "\u95D9"], /* CJK Ideograph */
  ["\u95DA", "\u95DA"], /* CJK Ideograph */
  ["\u95DB", "\u95DB"], /* CJK Ideograph */
  ["\u95DC", "\u95DC"], /* CJK Ideograph */
  ["\u95DD", "\u95DD"], /* CJK Ideograph */
  ["\u95DE", "\u95DE"], /* CJK Ideograph */
  ["\u95DF", "\u95DF"], /* CJK Ideograph */
  ["\u95E0", "\u95E0"], /* CJK Ideograph */
  ["\u95E1", "\u95E1"], /* CJK Ideograph */
  ["\u95E2", "\u95E2"], /* CJK Ideograph */
  ["\u95E3", "\u95E3"], /* CJK Ideograph */
  ["\u95E4", "\u95E4"], /* CJK Ideograph */
  ["\u95E5", "\u95E5"], /* CJK Ideograph */
  ["\u95E6", "\u95E6"], /* CJK Ideograph */
  ["\u95E7", "\u95E7"], /* CJK Ideograph */
  ["\u95E8", "\u95E8"], /* CJK Ideograph */
  ["\u95E9", "\u95E9"], /* CJK Ideograph */
  ["\u95EA", "\u95EA"], /* CJK Ideograph */
  ["\u95EB", "\u95EB"], /* CJK Ideograph */
  ["\u95EC", "\u95EC"], /* CJK Ideograph */
  ["\u95ED", "\u95ED"], /* CJK Ideograph */
  ["\u95EE", "\u95EE"], /* CJK Ideograph */
  ["\u95EF", "\u95EF"], /* CJK Ideograph */
  ["\u95F0", "\u95F0"], /* CJK Ideograph */
  ["\u95F1", "\u95F1"], /* CJK Ideograph */
  ["\u95F2", "\u95F2"], /* CJK Ideograph */
  ["\u95F3", "\u95F3"], /* CJK Ideograph */
  ["\u95F4", "\u95F4"], /* CJK Ideograph */
  ["\u95F5", "\u95F5"], /* CJK Ideograph */
  ["\u95F6", "\u95F6"], /* CJK Ideograph */
  ["\u95F7", "\u95F7"], /* CJK Ideograph */
  ["\u95F8", "\u95F8"], /* CJK Ideograph */
  ["\u95F9", "\u95F9"], /* CJK Ideograph */
  ["\u95FA", "\u95FA"], /* CJK Ideograph */
  ["\u95FB", "\u95FB"], /* CJK Ideograph */
  ["\u95FC", "\u95FC"], /* CJK Ideograph */
  ["\u95FD", "\u95FD"], /* CJK Ideograph */
  ["\u95FE", "\u95FE"], /* CJK Ideograph */
  ["\u95FF", "\u95FF"], /* CJK Ideograph */
  ["\u9600", "\u9600"], /* CJK Ideograph */
  ["\u9601", "\u9601"], /* CJK Ideograph */
  ["\u9602", "\u9602"], /* CJK Ideograph */
  ["\u9603", "\u9603"], /* CJK Ideograph */
  ["\u9604", "\u9604"], /* CJK Ideograph */
  ["\u9605", "\u9605"], /* CJK Ideograph */
  ["\u9606", "\u9606"], /* CJK Ideograph */
  ["\u9607", "\u9607"], /* CJK Ideograph */
  ["\u9608", "\u9608"], /* CJK Ideograph */
  ["\u9609", "\u9609"], /* CJK Ideograph */
  ["\u960A", "\u960A"], /* CJK Ideograph */
  ["\u960B", "\u960B"], /* CJK Ideograph */
  ["\u960C", "\u960C"], /* CJK Ideograph */
  ["\u960D", "\u960D"], /* CJK Ideograph */
  ["\u960E", "\u960E"], /* CJK Ideograph */
  ["\u960F", "\u960F"], /* CJK Ideograph */
  ["\u9610", "\u9610"], /* CJK Ideograph */
  ["\u9611", "\u9611"], /* CJK Ideograph */
  ["\u9612", "\u9612"], /* CJK Ideograph */
  ["\u9613", "\u9613"], /* CJK Ideograph */
  ["\u9614", "\u9614"], /* CJK Ideograph */
  ["\u9615", "\u9615"], /* CJK Ideograph */
  ["\u9616", "\u9616"], /* CJK Ideograph */
  ["\u9617", "\u9617"], /* CJK Ideograph */
  ["\u9618", "\u9618"], /* CJK Ideograph */
  ["\u9619", "\u9619"], /* CJK Ideograph */
  ["\u961A", "\u961A"], /* CJK Ideograph */
  ["\u961B", "\u961B"], /* CJK Ideograph */
  ["\u961C", "\u961C"], /* CJK Ideograph */
  ["\u961D", "\u961D"], /* CJK Ideograph */
  ["\u961E", "\u961E"], /* CJK Ideograph */
  ["\u961F", "\u961F"], /* CJK Ideograph */
  ["\u9620", "\u9620"], /* CJK Ideograph */
  ["\u9621", "\u9621"], /* CJK Ideograph */
  ["\u9622", "\u9622"], /* CJK Ideograph */
  ["\u9623", "\u9623"], /* CJK Ideograph */
  ["\u9624", "\u9624"], /* CJK Ideograph */
  ["\u9625", "\u9625"], /* CJK Ideograph */
  ["\u9626", "\u9626"], /* CJK Ideograph */
  ["\u9627", "\u9627"], /* CJK Ideograph */
  ["\u9628", "\u9628"], /* CJK Ideograph */
  ["\u9629", "\u9629"], /* CJK Ideograph */
  ["\u962A", "\u962A"], /* CJK Ideograph */
  ["\u962B", "\u962B"], /* CJK Ideograph */
  ["\u962C", "\u962C"], /* CJK Ideograph */
  ["\u962D", "\u962D"], /* CJK Ideograph */
  ["\u962E", "\u962E"], /* CJK Ideograph */
  ["\u962F", "\u962F"], /* CJK Ideograph */
  ["\u9630", "\u9630"], /* CJK Ideograph */
  ["\u9631", "\u9631"], /* CJK Ideograph */
  ["\u9632", "\u9632"], /* CJK Ideograph */
  ["\u9633", "\u9633"], /* CJK Ideograph */
  ["\u9634", "\u9634"], /* CJK Ideograph */
  ["\u9635", "\u9635"], /* CJK Ideograph */
  ["\u9636", "\u9636"], /* CJK Ideograph */
  ["\u9637", "\u9637"], /* CJK Ideograph */
  ["\u9638", "\u9638"], /* CJK Ideograph */
  ["\u9639", "\u9639"], /* CJK Ideograph */
  ["\u963A", "\u963A"], /* CJK Ideograph */
  ["\u963B", "\u963B"], /* CJK Ideograph */
  ["\u963C", "\u963C"], /* CJK Ideograph */
  ["\u963D", "\u963D"], /* CJK Ideograph */
  ["\u963E", "\u963E"], /* CJK Ideograph */
  ["\u963F", "\u963F"], /* CJK Ideograph */
  ["\u9640", "\u9640"], /* CJK Ideograph */
  ["\u9641", "\u9641"], /* CJK Ideograph */
  ["\u9642", "\u9642"], /* CJK Ideograph */
  ["\u9643", "\u9643"], /* CJK Ideograph */
  ["\u9644", "\u9644"], /* CJK Ideograph */
  ["\u9645", "\u9645"], /* CJK Ideograph */
  ["\u9646", "\u9646"], /* CJK Ideograph */
  ["\u9647", "\u9647"], /* CJK Ideograph */
  ["\u9648", "\u9648"], /* CJK Ideograph */
  ["\u9649", "\u9649"], /* CJK Ideograph */
  ["\u964A", "\u964A"], /* CJK Ideograph */
  ["\u964B", "\u964B"], /* CJK Ideograph */
  ["\u964C", "\u964C"], /* CJK Ideograph */
  ["\u964D", "\u964D"], /* CJK Ideograph */
  ["\u964E", "\u964E"], /* CJK Ideograph */
  ["\u964F", "\u964F"], /* CJK Ideograph */
  ["\u9650", "\u9650"], /* CJK Ideograph */
  ["\u9651", "\u9651"], /* CJK Ideograph */
  ["\u9652", "\u9652"], /* CJK Ideograph */
  ["\u9653", "\u9653"], /* CJK Ideograph */
  ["\u9654", "\u9654"], /* CJK Ideograph */
  ["\u9655", "\u9655"], /* CJK Ideograph */
  ["\u9656", "\u9656"], /* CJK Ideograph */
  ["\u9657", "\u9657"], /* CJK Ideograph */
  ["\u9658", "\u9658"], /* CJK Ideograph */
  ["\u9659", "\u9659"], /* CJK Ideograph */
  ["\u965A", "\u965A"], /* CJK Ideograph */
  ["\u965B", "\u965B"], /* CJK Ideograph */
  ["\u965C", "\u965C"], /* CJK Ideograph */
  ["\u965D", "\u965D"], /* CJK Ideograph */
  ["\u965E", "\u965E"], /* CJK Ideograph */
  ["\u965F", "\u965F"], /* CJK Ideograph */
  ["\u9660", "\u9660"], /* CJK Ideograph */
  ["\u9661", "\u9661"], /* CJK Ideograph */
  ["\u9662", "\u9662"], /* CJK Ideograph */
  ["\u9663", "\u9663"], /* CJK Ideograph */
  ["\u9664", "\u9664"], /* CJK Ideograph */
  ["\u9665", "\u9665"], /* CJK Ideograph */
  ["\u9666", "\u9666"], /* CJK Ideograph */
  ["\u9667", "\u9667"], /* CJK Ideograph */
  ["\u9668", "\u9668"], /* CJK Ideograph */
  ["\u9669", "\u9669"], /* CJK Ideograph */
  ["\u966A", "\u966A"], /* CJK Ideograph */
  ["\u966B", "\u966B"], /* CJK Ideograph */
  ["\u966C", "\u966C"], /* CJK Ideograph */
  ["\u966D", "\u966D"], /* CJK Ideograph */
  ["\u966E", "\u966E"], /* CJK Ideograph */
  ["\u966F", "\u966F"], /* CJK Ideograph */
  ["\u9670", "\u9670"], /* CJK Ideograph */
  ["\u9671", "\u9671"], /* CJK Ideograph */
  ["\u9672", "\u9672"], /* CJK Ideograph */
  ["\u9673", "\u9673"], /* CJK Ideograph */
  ["\u9674", "\u9674"], /* CJK Ideograph */
  ["\u9675", "\u9675"], /* CJK Ideograph */
  ["\u9676", "\u9676"], /* CJK Ideograph */
  ["\u9677", "\u9677"], /* CJK Ideograph */
  ["\u9678", "\u9678"], /* CJK Ideograph */
  ["\u9679", "\u9679"], /* CJK Ideograph */
  ["\u967A", "\u967A"], /* CJK Ideograph */
  ["\u967B", "\u967B"], /* CJK Ideograph */
  ["\u967C", "\u967C"], /* CJK Ideograph */
  ["\u967D", "\u967D"], /* CJK Ideograph */
  ["\u967E", "\u967E"], /* CJK Ideograph */
  ["\u967F", "\u967F"], /* CJK Ideograph */
  ["\u9680", "\u9680"], /* CJK Ideograph */
  ["\u9681", "\u9681"], /* CJK Ideograph */
  ["\u9682", "\u9682"], /* CJK Ideograph */
  ["\u9683", "\u9683"], /* CJK Ideograph */
  ["\u9684", "\u9684"], /* CJK Ideograph */
  ["\u9685", "\u9685"], /* CJK Ideograph */
  ["\u9686", "\u9686"], /* CJK Ideograph */
  ["\u9687", "\u9687"], /* CJK Ideograph */
  ["\u9688", "\u9688"], /* CJK Ideograph */
  ["\u9689", "\u9689"], /* CJK Ideograph */
  ["\u968A", "\u968A"], /* CJK Ideograph */
  ["\u968B", "\u968B"], /* CJK Ideograph */
  ["\u968C", "\u968C"], /* CJK Ideograph */
  ["\u968D", "\u968D"], /* CJK Ideograph */
  ["\u968E", "\u968E"], /* CJK Ideograph */
  ["\u968F", "\u968F"], /* CJK Ideograph */
  ["\u9690", "\u9690"], /* CJK Ideograph */
  ["\u9691", "\u9691"], /* CJK Ideograph */
  ["\u9692", "\u9692"], /* CJK Ideograph */
  ["\u9693", "\u9693"], /* CJK Ideograph */
  ["\u9694", "\u9694"], /* CJK Ideograph */
  ["\u9695", "\u9695"], /* CJK Ideograph */
  ["\u9696", "\u9696"], /* CJK Ideograph */
  ["\u9697", "\u9697"], /* CJK Ideograph */
  ["\u9698", "\u9698"], /* CJK Ideograph */
  ["\u9699", "\u9699"], /* CJK Ideograph */
  ["\u969A", "\u969A"], /* CJK Ideograph */
  ["\u969B", "\u969B"], /* CJK Ideograph */
  ["\u969C", "\u969C"], /* CJK Ideograph */
  ["\u969D", "\u969D"], /* CJK Ideograph */
  ["\u969E", "\u969E"], /* CJK Ideograph */
  ["\u969F", "\u969F"], /* CJK Ideograph */
  ["\u96A0", "\u96A0"], /* CJK Ideograph */
  ["\u96A1", "\u96A1"], /* CJK Ideograph */
  ["\u96A2", "\u96A2"], /* CJK Ideograph */
  ["\u96A3", "\u96A3"], /* CJK Ideograph */
  ["\u96A4", "\u96A4"], /* CJK Ideograph */
  ["\u96A5", "\u96A5"], /* CJK Ideograph */
  ["\u96A6", "\u96A6"], /* CJK Ideograph */
  ["\u96A7", "\u96A7"], /* CJK Ideograph */
  ["\u96A8", "\u96A8"], /* CJK Ideograph */
  ["\u96A9", "\u96A9"], /* CJK Ideograph */
  ["\u96AA", "\u96AA"], /* CJK Ideograph */
  ["\u96AB", "\u96AB"], /* CJK Ideograph */
  ["\u96AC", "\u96AC"], /* CJK Ideograph */
  ["\u96AD", "\u96AD"], /* CJK Ideograph */
  ["\u96AE", "\u96AE"], /* CJK Ideograph */
  ["\u96AF", "\u96AF"], /* CJK Ideograph */
  ["\u96B0", "\u96B0"], /* CJK Ideograph */
  ["\u96B1", "\u96B1"], /* CJK Ideograph */
  ["\u96B2", "\u96B2"], /* CJK Ideograph */
  ["\u96B3", "\u96B3"], /* CJK Ideograph */
  ["\u96B4", "\u96B4"], /* CJK Ideograph */
  ["\u96B5", "\u96B5"], /* CJK Ideograph */
  ["\u96B6", "\u96B6"], /* CJK Ideograph */
  ["\u96B7", "\u96B7"], /* CJK Ideograph */
  ["\u96B8", "\u96B8"], /* CJK Ideograph */
  ["\u96B9", "\u96B9"], /* CJK Ideograph */
  ["\u96BA", "\u96BA"], /* CJK Ideograph */
  ["\u96BB", "\u96BB"], /* CJK Ideograph */
  ["\u96BC", "\u96BC"], /* CJK Ideograph */
  ["\u96BD", "\u96BD"], /* CJK Ideograph */
  ["\u96BE", "\u96BE"], /* CJK Ideograph */
  ["\u96BF", "\u96BF"], /* CJK Ideograph */
  ["\u96C0", "\u96C0"], /* CJK Ideograph */
  ["\u96C1", "\u96C1"], /* CJK Ideograph */
  ["\u96C2", "\u96C2"], /* CJK Ideograph */
  ["\u96C3", "\u96C3"], /* CJK Ideograph */
  ["\u96C4", "\u96C4"], /* CJK Ideograph */
  ["\u96C5", "\u96C5"], /* CJK Ideograph */
  ["\u96C6", "\u96C6"], /* CJK Ideograph */
  ["\u96C7", "\u96C7"], /* CJK Ideograph */
  ["\u96C8", "\u96C8"], /* CJK Ideograph */
  ["\u96C9", "\u96C9"], /* CJK Ideograph */
  ["\u96CA", "\u96CA"], /* CJK Ideograph */
  ["\u96CB", "\u96CB"], /* CJK Ideograph */
  ["\u96CC", "\u96CC"], /* CJK Ideograph */
  ["\u96CD", "\u96CD"], /* CJK Ideograph */
  ["\u96CE", "\u96CE"], /* CJK Ideograph */
  ["\u96CF", "\u96CF"], /* CJK Ideograph */
  ["\u96D0", "\u96D0"], /* CJK Ideograph */
  ["\u96D1", "\u96D1"], /* CJK Ideograph */
  ["\u96D2", "\u96D2"], /* CJK Ideograph */
  ["\u96D3", "\u96D3"], /* CJK Ideograph */
  ["\u96D4", "\u96D4"], /* CJK Ideograph */
  ["\u96D5", "\u96D5"], /* CJK Ideograph */
  ["\u96D6", "\u96D6"], /* CJK Ideograph */
  ["\u96D7", "\u96D7"], /* CJK Ideograph */
  ["\u96D8", "\u96D8"], /* CJK Ideograph */
  ["\u96D9", "\u96D9"], /* CJK Ideograph */
  ["\u96DA", "\u96DA"], /* CJK Ideograph */
  ["\u96DB", "\u96DB"], /* CJK Ideograph */
  ["\u96DC", "\u96DC"], /* CJK Ideograph */
  ["\u96DD", "\u96DD"], /* CJK Ideograph */
  ["\u96DE", "\u96DE"], /* CJK Ideograph */
  ["\u96DF", "\u96DF"], /* CJK Ideograph */
  ["\u96E0", "\u96E0"], /* CJK Ideograph */
  ["\u96E1", "\u96E1"], /* CJK Ideograph */
  ["\u96E2", "\u96E2"], /* CJK Ideograph */
  ["\u96E3", "\u96E3"], /* CJK Ideograph */
  ["\u96E4", "\u96E4"], /* CJK Ideograph */
  ["\u96E5", "\u96E5"], /* CJK Ideograph */
  ["\u96E6", "\u96E6"], /* CJK Ideograph */
  ["\u96E7", "\u96E7"], /* CJK Ideograph */
  ["\u96E8", "\u96E8"], /* CJK Ideograph */
  ["\u96E9", "\u96E9"], /* CJK Ideograph */
  ["\u96EA", "\u96EA"], /* CJK Ideograph */
  ["\u96EB", "\u96EB"], /* CJK Ideograph */
  ["\u96EC", "\u96EC"], /* CJK Ideograph */
  ["\u96ED", "\u96ED"], /* CJK Ideograph */
  ["\u96EE", "\u96EE"], /* CJK Ideograph */
  ["\u96EF", "\u96EF"], /* CJK Ideograph */
  ["\u96F0", "\u96F0"], /* CJK Ideograph */
  ["\u96F1", "\u96F1"], /* CJK Ideograph */
  ["\u96F2", "\u96F2"], /* CJK Ideograph */
  ["\u96F3", "\u96F3"], /* CJK Ideograph */
  ["\u96F4", "\u96F4"], /* CJK Ideograph */
  ["\u96F5", "\u96F5"], /* CJK Ideograph */
  ["\u96F6", "\u96F6"], /* CJK Ideograph */
  ["\u96F7", "\u96F7"], /* CJK Ideograph */
  ["\u96F8", "\u96F8"], /* CJK Ideograph */
  ["\u96F9", "\u96F9"], /* CJK Ideograph */
  ["\u96FA", "\u96FA"], /* CJK Ideograph */
  ["\u96FB", "\u96FB"], /* CJK Ideograph */
  ["\u96FC", "\u96FC"], /* CJK Ideograph */
  ["\u96FD", "\u96FD"], /* CJK Ideograph */
  ["\u96FE", "\u96FE"], /* CJK Ideograph */
  ["\u96FF", "\u96FF"], /* CJK Ideograph */
  ["\u9700", "\u9700"], /* CJK Ideograph */
  ["\u9701", "\u9701"], /* CJK Ideograph */
  ["\u9702", "\u9702"], /* CJK Ideograph */
  ["\u9703", "\u9703"], /* CJK Ideograph */
  ["\u9704", "\u9704"], /* CJK Ideograph */
  ["\u9705", "\u9705"], /* CJK Ideograph */
  ["\u9706", "\u9706"], /* CJK Ideograph */
  ["\u9707", "\u9707"], /* CJK Ideograph */
  ["\u9708", "\u9708"], /* CJK Ideograph */
  ["\u9709", "\u9709"], /* CJK Ideograph */
  ["\u970A", "\u970A"], /* CJK Ideograph */
  ["\u970B", "\u970B"], /* CJK Ideograph */
  ["\u970C", "\u970C"], /* CJK Ideograph */
  ["\u970D", "\u970D"], /* CJK Ideograph */
  ["\u970E", "\u970E"], /* CJK Ideograph */
  ["\u970F", "\u970F"], /* CJK Ideograph */
  ["\u9710", "\u9710"], /* CJK Ideograph */
  ["\u9711", "\u9711"], /* CJK Ideograph */
  ["\u9712", "\u9712"], /* CJK Ideograph */
  ["\u9713", "\u9713"], /* CJK Ideograph */
  ["\u9714", "\u9714"], /* CJK Ideograph */
  ["\u9715", "\u9715"], /* CJK Ideograph */
  ["\u9716", "\u9716"], /* CJK Ideograph */
  ["\u9717", "\u9717"], /* CJK Ideograph */
  ["\u9718", "\u9718"], /* CJK Ideograph */
  ["\u9719", "\u9719"], /* CJK Ideograph */
  ["\u971A", "\u971A"], /* CJK Ideograph */
  ["\u971B", "\u971B"], /* CJK Ideograph */
  ["\u971C", "\u971C"], /* CJK Ideograph */
  ["\u971D", "\u971D"], /* CJK Ideograph */
  ["\u971E", "\u971E"], /* CJK Ideograph */
  ["\u971F", "\u971F"], /* CJK Ideograph */
  ["\u9720", "\u9720"], /* CJK Ideograph */
  ["\u9721", "\u9721"], /* CJK Ideograph */
  ["\u9722", "\u9722"], /* CJK Ideograph */
  ["\u9723", "\u9723"], /* CJK Ideograph */
  ["\u9724", "\u9724"], /* CJK Ideograph */
  ["\u9725", "\u9725"], /* CJK Ideograph */
  ["\u9726", "\u9726"], /* CJK Ideograph */
  ["\u9727", "\u9727"], /* CJK Ideograph */
  ["\u9728", "\u9728"], /* CJK Ideograph */
  ["\u9729", "\u9729"], /* CJK Ideograph */
  ["\u972A", "\u972A"], /* CJK Ideograph */
  ["\u972B", "\u972B"], /* CJK Ideograph */
  ["\u972C", "\u972C"], /* CJK Ideograph */
  ["\u972D", "\u972D"], /* CJK Ideograph */
  ["\u972E", "\u972E"], /* CJK Ideograph */
  ["\u972F", "\u972F"], /* CJK Ideograph */
  ["\u9730", "\u9730"], /* CJK Ideograph */
  ["\u9731", "\u9731"], /* CJK Ideograph */
  ["\u9732", "\u9732"], /* CJK Ideograph */
  ["\u9733", "\u9733"], /* CJK Ideograph */
  ["\u9734", "\u9734"], /* CJK Ideograph */
  ["\u9735", "\u9735"], /* CJK Ideograph */
  ["\u9736", "\u9736"], /* CJK Ideograph */
  ["\u9737", "\u9737"], /* CJK Ideograph */
  ["\u9738", "\u9738"], /* CJK Ideograph */
  ["\u9739", "\u9739"], /* CJK Ideograph */
  ["\u973A", "\u973A"], /* CJK Ideograph */
  ["\u973B", "\u973B"], /* CJK Ideograph */
  ["\u973C", "\u973C"], /* CJK Ideograph */
  ["\u973D", "\u973D"], /* CJK Ideograph */
  ["\u973E", "\u973E"], /* CJK Ideograph */
  ["\u973F", "\u973F"], /* CJK Ideograph */
  ["\u9740", "\u9740"], /* CJK Ideograph */
  ["\u9741", "\u9741"], /* CJK Ideograph */
  ["\u9742", "\u9742"], /* CJK Ideograph */
  ["\u9743", "\u9743"], /* CJK Ideograph */
  ["\u9744", "\u9744"], /* CJK Ideograph */
  ["\u9745", "\u9745"], /* CJK Ideograph */
  ["\u9746", "\u9746"], /* CJK Ideograph */
  ["\u9747", "\u9747"], /* CJK Ideograph */
  ["\u9748", "\u9748"], /* CJK Ideograph */
  ["\u9749", "\u9749"], /* CJK Ideograph */
  ["\u974A", "\u974A"], /* CJK Ideograph */
  ["\u974B", "\u974B"], /* CJK Ideograph */
  ["\u974C", "\u974C"], /* CJK Ideograph */
  ["\u974D", "\u974D"], /* CJK Ideograph */
  ["\u974E", "\u974E"], /* CJK Ideograph */
  ["\u974F", "\u974F"], /* CJK Ideograph */
  ["\u9750", "\u9750"], /* CJK Ideograph */
  ["\u9751", "\u9751"], /* CJK Ideograph */
  ["\u9752", "\u9752"], /* CJK Ideograph */
  ["\u9753", "\u9753"], /* CJK Ideograph */
  ["\u9754", "\u9754"], /* CJK Ideograph */
  ["\u9755", "\u9755"], /* CJK Ideograph */
  ["\u9756", "\u9756"], /* CJK Ideograph */
  ["\u9757", "\u9757"], /* CJK Ideograph */
  ["\u9758", "\u9758"], /* CJK Ideograph */
  ["\u9759", "\u9759"], /* CJK Ideograph */
  ["\u975A", "\u975A"], /* CJK Ideograph */
  ["\u975B", "\u975B"], /* CJK Ideograph */
  ["\u975C", "\u975C"], /* CJK Ideograph */
  ["\u975D", "\u975D"], /* CJK Ideograph */
  ["\u975E", "\u975E"], /* CJK Ideograph */
  ["\u975F", "\u975F"], /* CJK Ideograph */
  ["\u9760", "\u9760"], /* CJK Ideograph */
  ["\u9761", "\u9761"], /* CJK Ideograph */
  ["\u9762", "\u9762"], /* CJK Ideograph */
  ["\u9763", "\u9763"], /* CJK Ideograph */
  ["\u9764", "\u9764"], /* CJK Ideograph */
  ["\u9765", "\u9765"], /* CJK Ideograph */
  ["\u9766", "\u9766"], /* CJK Ideograph */
  ["\u9767", "\u9767"], /* CJK Ideograph */
  ["\u9768", "\u9768"], /* CJK Ideograph */
  ["\u9769", "\u9769"], /* CJK Ideograph */
  ["\u976A", "\u976A"], /* CJK Ideograph */
  ["\u976B", "\u976B"], /* CJK Ideograph */
  ["\u976C", "\u976C"], /* CJK Ideograph */
  ["\u976D", "\u976D"], /* CJK Ideograph */
  ["\u976E", "\u976E"], /* CJK Ideograph */
  ["\u976F", "\u976F"], /* CJK Ideograph */
  ["\u9770", "\u9770"], /* CJK Ideograph */
  ["\u9771", "\u9771"], /* CJK Ideograph */
  ["\u9772", "\u9772"], /* CJK Ideograph */
  ["\u9773", "\u9773"], /* CJK Ideograph */
  ["\u9774", "\u9774"], /* CJK Ideograph */
  ["\u9775", "\u9775"], /* CJK Ideograph */
  ["\u9776", "\u9776"], /* CJK Ideograph */
  ["\u9777", "\u9777"], /* CJK Ideograph */
  ["\u9778", "\u9778"], /* CJK Ideograph */
  ["\u9779", "\u9779"], /* CJK Ideograph */
  ["\u977A", "\u977A"], /* CJK Ideograph */
  ["\u977B", "\u977B"], /* CJK Ideograph */
  ["\u977C", "\u977C"], /* CJK Ideograph */
  ["\u977D", "\u977D"], /* CJK Ideograph */
  ["\u977E", "\u977E"], /* CJK Ideograph */
  ["\u977F", "\u977F"], /* CJK Ideograph */
  ["\u9780", "\u9780"], /* CJK Ideograph */
  ["\u9781", "\u9781"], /* CJK Ideograph */
  ["\u9782", "\u9782"], /* CJK Ideograph */
  ["\u9783", "\u9783"], /* CJK Ideograph */
  ["\u9784", "\u9784"], /* CJK Ideograph */
  ["\u9785", "\u9785"], /* CJK Ideograph */
  ["\u9786", "\u9786"], /* CJK Ideograph */
  ["\u9787", "\u9787"], /* CJK Ideograph */
  ["\u9788", "\u9788"], /* CJK Ideograph */
  ["\u9789", "\u9789"], /* CJK Ideograph */
  ["\u978A", "\u978A"], /* CJK Ideograph */
  ["\u978B", "\u978B"], /* CJK Ideograph */
  ["\u978C", "\u978C"], /* CJK Ideograph */
  ["\u978D", "\u978D"], /* CJK Ideograph */
  ["\u978E", "\u978E"], /* CJK Ideograph */
  ["\u978F", "\u978F"], /* CJK Ideograph */
  ["\u9790", "\u9790"], /* CJK Ideograph */
  ["\u9791", "\u9791"], /* CJK Ideograph */
  ["\u9792", "\u9792"], /* CJK Ideograph */
  ["\u9793", "\u9793"], /* CJK Ideograph */
  ["\u9794", "\u9794"], /* CJK Ideograph */
  ["\u9795", "\u9795"], /* CJK Ideograph */
  ["\u9796", "\u9796"], /* CJK Ideograph */
  ["\u9797", "\u9797"], /* CJK Ideograph */
  ["\u9798", "\u9798"], /* CJK Ideograph */
  ["\u9799", "\u9799"], /* CJK Ideograph */
  ["\u979A", "\u979A"], /* CJK Ideograph */
  ["\u979B", "\u979B"], /* CJK Ideograph */
  ["\u979C", "\u979C"], /* CJK Ideograph */
  ["\u979D", "\u979D"], /* CJK Ideograph */
  ["\u979E", "\u979E"], /* CJK Ideograph */
  ["\u979F", "\u979F"], /* CJK Ideograph */
  ["\u97A0", "\u97A0"], /* CJK Ideograph */
  ["\u97A1", "\u97A1"], /* CJK Ideograph */
  ["\u97A2", "\u97A2"], /* CJK Ideograph */
  ["\u97A3", "\u97A3"], /* CJK Ideograph */
  ["\u97A4", "\u97A4"], /* CJK Ideograph */
  ["\u97A5", "\u97A5"], /* CJK Ideograph */
  ["\u97A6", "\u97A6"], /* CJK Ideograph */
  ["\u97A7", "\u97A7"], /* CJK Ideograph */
  ["\u97A8", "\u97A8"], /* CJK Ideograph */
  ["\u97A9", "\u97A9"], /* CJK Ideograph */
  ["\u97AA", "\u97AA"], /* CJK Ideograph */
  ["\u97AB", "\u97AB"], /* CJK Ideograph */
  ["\u97AC", "\u97AC"], /* CJK Ideograph */
  ["\u97AD", "\u97AD"], /* CJK Ideograph */
  ["\u97AE", "\u97AE"], /* CJK Ideograph */
  ["\u97AF", "\u97AF"], /* CJK Ideograph */
  ["\u97B0", "\u97B0"], /* CJK Ideograph */
  ["\u97B1", "\u97B1"], /* CJK Ideograph */
  ["\u97B2", "\u97B2"], /* CJK Ideograph */
  ["\u97B3", "\u97B3"], /* CJK Ideograph */
  ["\u97B4", "\u97B4"], /* CJK Ideograph */
  ["\u97B5", "\u97B5"], /* CJK Ideograph */
  ["\u97B6", "\u97B6"], /* CJK Ideograph */
  ["\u97B7", "\u97B7"], /* CJK Ideograph */
  ["\u97B8", "\u97B8"], /* CJK Ideograph */
  ["\u97B9", "\u97B9"], /* CJK Ideograph */
  ["\u97BA", "\u97BA"], /* CJK Ideograph */
  ["\u97BB", "\u97BB"], /* CJK Ideograph */
  ["\u97BC", "\u97BC"], /* CJK Ideograph */
  ["\u97BD", "\u97BD"], /* CJK Ideograph */
  ["\u97BE", "\u97BE"], /* CJK Ideograph */
  ["\u97BF", "\u97BF"], /* CJK Ideograph */
  ["\u97C0", "\u97C0"], /* CJK Ideograph */
  ["\u97C1", "\u97C1"], /* CJK Ideograph */
  ["\u97C2", "\u97C2"], /* CJK Ideograph */
  ["\u97C3", "\u97C3"], /* CJK Ideograph */
  ["\u97C4", "\u97C4"], /* CJK Ideograph */
  ["\u97C5", "\u97C5"], /* CJK Ideograph */
  ["\u97C6", "\u97C6"], /* CJK Ideograph */
  ["\u97C7", "\u97C7"], /* CJK Ideograph */
  ["\u97C8", "\u97C8"], /* CJK Ideograph */
  ["\u97C9", "\u97C9"], /* CJK Ideograph */
  ["\u97CA", "\u97CA"], /* CJK Ideograph */
  ["\u97CB", "\u97CB"], /* CJK Ideograph */
  ["\u97CC", "\u97CC"], /* CJK Ideograph */
  ["\u97CD", "\u97CD"], /* CJK Ideograph */
  ["\u97CE", "\u97CE"], /* CJK Ideograph */
  ["\u97CF", "\u97CF"], /* CJK Ideograph */
  ["\u97D0", "\u97D0"], /* CJK Ideograph */
  ["\u97D1", "\u97D1"], /* CJK Ideograph */
  ["\u97D2", "\u97D2"], /* CJK Ideograph */
  ["\u97D3", "\u97D3"], /* CJK Ideograph */
  ["\u97D4", "\u97D4"], /* CJK Ideograph */
  ["\u97D5", "\u97D5"], /* CJK Ideograph */
  ["\u97D6", "\u97D6"], /* CJK Ideograph */
  ["\u97D7", "\u97D7"], /* CJK Ideograph */
  ["\u97D8", "\u97D8"], /* CJK Ideograph */
  ["\u97D9", "\u97D9"], /* CJK Ideograph */
  ["\u97DA", "\u97DA"], /* CJK Ideograph */
  ["\u97DB", "\u97DB"], /* CJK Ideograph */
  ["\u97DC", "\u97DC"], /* CJK Ideograph */
  ["\u97DD", "\u97DD"], /* CJK Ideograph */
  ["\u97DE", "\u97DE"], /* CJK Ideograph */
  ["\u97DF", "\u97DF"], /* CJK Ideograph */
  ["\u97E0", "\u97E0"], /* CJK Ideograph */
  ["\u97E1", "\u97E1"], /* CJK Ideograph */
  ["\u97E2", "\u97E2"], /* CJK Ideograph */
  ["\u97E3", "\u97E3"], /* CJK Ideograph */
  ["\u97E4", "\u97E4"], /* CJK Ideograph */
  ["\u97E5", "\u97E5"], /* CJK Ideograph */
  ["\u97E6", "\u97E6"], /* CJK Ideograph */
  ["\u97E7", "\u97E7"], /* CJK Ideograph */
  ["\u97E8", "\u97E8"], /* CJK Ideograph */
  ["\u97E9", "\u97E9"], /* CJK Ideograph */
  ["\u97EA", "\u97EA"], /* CJK Ideograph */
  ["\u97EB", "\u97EB"], /* CJK Ideograph */
  ["\u97EC", "\u97EC"], /* CJK Ideograph */
  ["\u97ED", "\u97ED"], /* CJK Ideograph */
  ["\u97EE", "\u97EE"], /* CJK Ideograph */
  ["\u97EF", "\u97EF"], /* CJK Ideograph */
  ["\u97F0", "\u97F0"], /* CJK Ideograph */
  ["\u97F1", "\u97F1"], /* CJK Ideograph */
  ["\u97F2", "\u97F2"], /* CJK Ideograph */
  ["\u97F3", "\u97F3"], /* CJK Ideograph */
  ["\u97F4", "\u97F4"], /* CJK Ideograph */
  ["\u97F5", "\u97F5"], /* CJK Ideograph */
  ["\u97F6", "\u97F6"], /* CJK Ideograph */
  ["\u97F7", "\u97F7"], /* CJK Ideograph */
  ["\u97F8", "\u97F8"], /* CJK Ideograph */
  ["\u97F9", "\u97F9"], /* CJK Ideograph */
  ["\u97FA", "\u97FA"], /* CJK Ideograph */
  ["\u97FB", "\u97FB"], /* CJK Ideograph */
  ["\u97FC", "\u97FC"], /* CJK Ideograph */
  ["\u97FD", "\u97FD"], /* CJK Ideograph */
  ["\u97FE", "\u97FE"], /* CJK Ideograph */
  ["\u97FF", "\u97FF"], /* CJK Ideograph */
  ["\u9800", "\u9800"], /* CJK Ideograph */
  ["\u9801", "\u9801"], /* CJK Ideograph */
  ["\u9802", "\u9802"], /* CJK Ideograph */
  ["\u9803", "\u9803"], /* CJK Ideograph */
  ["\u9804", "\u9804"], /* CJK Ideograph */
  ["\u9805", "\u9805"], /* CJK Ideograph */
  ["\u9806", "\u9806"], /* CJK Ideograph */
  ["\u9807", "\u9807"], /* CJK Ideograph */
  ["\u9808", "\u9808"], /* CJK Ideograph */
  ["\u9809", "\u9809"], /* CJK Ideograph */
  ["\u980A", "\u980A"], /* CJK Ideograph */
  ["\u980B", "\u980B"], /* CJK Ideograph */
  ["\u980C", "\u980C"], /* CJK Ideograph */
  ["\u980D", "\u980D"], /* CJK Ideograph */
  ["\u980E", "\u980E"], /* CJK Ideograph */
  ["\u980F", "\u980F"], /* CJK Ideograph */
  ["\u9810", "\u9810"], /* CJK Ideograph */
  ["\u9811", "\u9811"], /* CJK Ideograph */
  ["\u9812", "\u9812"], /* CJK Ideograph */
  ["\u9813", "\u9813"], /* CJK Ideograph */
  ["\u9814", "\u9814"], /* CJK Ideograph */
  ["\u9815", "\u9815"], /* CJK Ideograph */
  ["\u9816", "\u9816"], /* CJK Ideograph */
  ["\u9817", "\u9817"], /* CJK Ideograph */
  ["\u9818", "\u9818"], /* CJK Ideograph */
  ["\u9819", "\u9819"], /* CJK Ideograph */
  ["\u981A", "\u981A"], /* CJK Ideograph */
  ["\u981B", "\u981B"], /* CJK Ideograph */
  ["\u981C", "\u981C"], /* CJK Ideograph */
  ["\u981D", "\u981D"], /* CJK Ideograph */
  ["\u981E", "\u981E"], /* CJK Ideograph */
  ["\u981F", "\u981F"], /* CJK Ideograph */
  ["\u9820", "\u9820"], /* CJK Ideograph */
  ["\u9821", "\u9821"], /* CJK Ideograph */
  ["\u9822", "\u9822"], /* CJK Ideograph */
  ["\u9823", "\u9823"], /* CJK Ideograph */
  ["\u9824", "\u9824"], /* CJK Ideograph */
  ["\u9825", "\u9825"], /* CJK Ideograph */
  ["\u9826", "\u9826"], /* CJK Ideograph */
  ["\u9827", "\u9827"], /* CJK Ideograph */
  ["\u9828", "\u9828"], /* CJK Ideograph */
  ["\u9829", "\u9829"], /* CJK Ideograph */
  ["\u982A", "\u982A"], /* CJK Ideograph */
  ["\u982B", "\u982B"], /* CJK Ideograph */
  ["\u982C", "\u982C"], /* CJK Ideograph */
  ["\u982D", "\u982D"], /* CJK Ideograph */
  ["\u982E", "\u982E"], /* CJK Ideograph */
  ["\u982F", "\u982F"], /* CJK Ideograph */
  ["\u9830", "\u9830"], /* CJK Ideograph */
  ["\u9831", "\u9831"], /* CJK Ideograph */
  ["\u9832", "\u9832"], /* CJK Ideograph */
  ["\u9833", "\u9833"], /* CJK Ideograph */
  ["\u9834", "\u9834"], /* CJK Ideograph */
  ["\u9835", "\u9835"], /* CJK Ideograph */
  ["\u9836", "\u9836"], /* CJK Ideograph */
  ["\u9837", "\u9837"], /* CJK Ideograph */
  ["\u9838", "\u9838"], /* CJK Ideograph */
  ["\u9839", "\u9839"], /* CJK Ideograph */
  ["\u983A", "\u983A"], /* CJK Ideograph */
  ["\u983B", "\u983B"], /* CJK Ideograph */
  ["\u983C", "\u983C"], /* CJK Ideograph */
  ["\u983D", "\u983D"], /* CJK Ideograph */
  ["\u983E", "\u983E"], /* CJK Ideograph */
  ["\u983F", "\u983F"], /* CJK Ideograph */
  ["\u9840", "\u9840"], /* CJK Ideograph */
  ["\u9841", "\u9841"], /* CJK Ideograph */
  ["\u9842", "\u9842"], /* CJK Ideograph */
  ["\u9843", "\u9843"], /* CJK Ideograph */
  ["\u9844", "\u9844"], /* CJK Ideograph */
  ["\u9845", "\u9845"], /* CJK Ideograph */
  ["\u9846", "\u9846"], /* CJK Ideograph */
  ["\u9847", "\u9847"], /* CJK Ideograph */
  ["\u9848", "\u9848"], /* CJK Ideograph */
  ["\u9849", "\u9849"], /* CJK Ideograph */
  ["\u984A", "\u984A"], /* CJK Ideograph */
  ["\u984B", "\u984B"], /* CJK Ideograph */
  ["\u984C", "\u984C"], /* CJK Ideograph */
  ["\u984D", "\u984D"], /* CJK Ideograph */
  ["\u984E", "\u984E"], /* CJK Ideograph */
  ["\u984F", "\u984F"], /* CJK Ideograph */
  ["\u9850", "\u9850"], /* CJK Ideograph */
  ["\u9851", "\u9851"], /* CJK Ideograph */
  ["\u9852", "\u9852"], /* CJK Ideograph */
  ["\u9853", "\u9853"], /* CJK Ideograph */
  ["\u9854", "\u9854"], /* CJK Ideograph */
  ["\u9855", "\u9855"], /* CJK Ideograph */
  ["\u9856", "\u9856"], /* CJK Ideograph */
  ["\u9857", "\u9857"], /* CJK Ideograph */
  ["\u9858", "\u9858"], /* CJK Ideograph */
  ["\u9859", "\u9859"], /* CJK Ideograph */
  ["\u985A", "\u985A"], /* CJK Ideograph */
  ["\u985B", "\u985B"], /* CJK Ideograph */
  ["\u985C", "\u985C"], /* CJK Ideograph */
  ["\u985D", "\u985D"], /* CJK Ideograph */
  ["\u985E", "\u985E"], /* CJK Ideograph */
  ["\u985F", "\u985F"], /* CJK Ideograph */
  ["\u9860", "\u9860"], /* CJK Ideograph */
  ["\u9861", "\u9861"], /* CJK Ideograph */
  ["\u9862", "\u9862"], /* CJK Ideograph */
  ["\u9863", "\u9863"], /* CJK Ideograph */
  ["\u9864", "\u9864"], /* CJK Ideograph */
  ["\u9865", "\u9865"], /* CJK Ideograph */
  ["\u9866", "\u9866"], /* CJK Ideograph */
  ["\u9867", "\u9867"], /* CJK Ideograph */
  ["\u9868", "\u9868"], /* CJK Ideograph */
  ["\u9869", "\u9869"], /* CJK Ideograph */
  ["\u986A", "\u986A"], /* CJK Ideograph */
  ["\u986B", "\u986B"], /* CJK Ideograph */
  ["\u986C", "\u986C"], /* CJK Ideograph */
  ["\u986D", "\u986D"], /* CJK Ideograph */
  ["\u986E", "\u986E"], /* CJK Ideograph */
  ["\u986F", "\u986F"], /* CJK Ideograph */
  ["\u9870", "\u9870"], /* CJK Ideograph */
  ["\u9871", "\u9871"], /* CJK Ideograph */
  ["\u9872", "\u9872"], /* CJK Ideograph */
  ["\u9873", "\u9873"], /* CJK Ideograph */
  ["\u9874", "\u9874"], /* CJK Ideograph */
  ["\u9875", "\u9875"], /* CJK Ideograph */
  ["\u9876", "\u9876"], /* CJK Ideograph */
  ["\u9877", "\u9877"], /* CJK Ideograph */
  ["\u9878", "\u9878"], /* CJK Ideograph */
  ["\u9879", "\u9879"], /* CJK Ideograph */
  ["\u987A", "\u987A"], /* CJK Ideograph */
  ["\u987B", "\u987B"], /* CJK Ideograph */
  ["\u987C", "\u987C"], /* CJK Ideograph */
  ["\u987D", "\u987D"], /* CJK Ideograph */
  ["\u987E", "\u987E"], /* CJK Ideograph */
  ["\u987F", "\u987F"], /* CJK Ideograph */
  ["\u9880", "\u9880"], /* CJK Ideograph */
  ["\u9881", "\u9881"], /* CJK Ideograph */
  ["\u9882", "\u9882"], /* CJK Ideograph */
  ["\u9883", "\u9883"], /* CJK Ideograph */
  ["\u9884", "\u9884"], /* CJK Ideograph */
  ["\u9885", "\u9885"], /* CJK Ideograph */
  ["\u9886", "\u9886"], /* CJK Ideograph */
  ["\u9887", "\u9887"], /* CJK Ideograph */
  ["\u9888", "\u9888"], /* CJK Ideograph */
  ["\u9889", "\u9889"], /* CJK Ideograph */
  ["\u988A", "\u988A"], /* CJK Ideograph */
  ["\u988B", "\u988B"], /* CJK Ideograph */
  ["\u988C", "\u988C"], /* CJK Ideograph */
  ["\u988D", "\u988D"], /* CJK Ideograph */
  ["\u988E", "\u988E"], /* CJK Ideograph */
  ["\u988F", "\u988F"], /* CJK Ideograph */
  ["\u9890", "\u9890"], /* CJK Ideograph */
  ["\u9891", "\u9891"], /* CJK Ideograph */
  ["\u9892", "\u9892"], /* CJK Ideograph */
  ["\u9893", "\u9893"], /* CJK Ideograph */
  ["\u9894", "\u9894"], /* CJK Ideograph */
  ["\u9895", "\u9895"], /* CJK Ideograph */
  ["\u9896", "\u9896"], /* CJK Ideograph */
  ["\u9897", "\u9897"], /* CJK Ideograph */
  ["\u9898", "\u9898"], /* CJK Ideograph */
  ["\u9899", "\u9899"], /* CJK Ideograph */
  ["\u989A", "\u989A"], /* CJK Ideograph */
  ["\u989B", "\u989B"], /* CJK Ideograph */
  ["\u989C", "\u989C"], /* CJK Ideograph */
  ["\u989D", "\u989D"], /* CJK Ideograph */
  ["\u989E", "\u989E"], /* CJK Ideograph */
  ["\u989F", "\u989F"], /* CJK Ideograph */
  ["\u98A0", "\u98A0"], /* CJK Ideograph */
  ["\u98A1", "\u98A1"], /* CJK Ideograph */
  ["\u98A2", "\u98A2"], /* CJK Ideograph */
  ["\u98A3", "\u98A3"], /* CJK Ideograph */
  ["\u98A4", "\u98A4"], /* CJK Ideograph */
  ["\u98A5", "\u98A5"], /* CJK Ideograph */
  ["\u98A6", "\u98A6"], /* CJK Ideograph */
  ["\u98A7", "\u98A7"], /* CJK Ideograph */
  ["\u98A8", "\u98A8"], /* CJK Ideograph */
  ["\u98A9", "\u98A9"], /* CJK Ideograph */
  ["\u98AA", "\u98AA"], /* CJK Ideograph */
  ["\u98AB", "\u98AB"], /* CJK Ideograph */
  ["\u98AC", "\u98AC"], /* CJK Ideograph */
  ["\u98AD", "\u98AD"], /* CJK Ideograph */
  ["\u98AE", "\u98AE"], /* CJK Ideograph */
  ["\u98AF", "\u98AF"], /* CJK Ideograph */
  ["\u98B0", "\u98B0"], /* CJK Ideograph */
  ["\u98B1", "\u98B1"], /* CJK Ideograph */
  ["\u98B2", "\u98B2"], /* CJK Ideograph */
  ["\u98B3", "\u98B3"], /* CJK Ideograph */
  ["\u98B4", "\u98B4"], /* CJK Ideograph */
  ["\u98B5", "\u98B5"], /* CJK Ideograph */
  ["\u98B6", "\u98B6"], /* CJK Ideograph */
  ["\u98B7", "\u98B7"], /* CJK Ideograph */
  ["\u98B8", "\u98B8"], /* CJK Ideograph */
  ["\u98B9", "\u98B9"], /* CJK Ideograph */
  ["\u98BA", "\u98BA"], /* CJK Ideograph */
  ["\u98BB", "\u98BB"], /* CJK Ideograph */
  ["\u98BC", "\u98BC"], /* CJK Ideograph */
  ["\u98BD", "\u98BD"], /* CJK Ideograph */
  ["\u98BE", "\u98BE"], /* CJK Ideograph */
  ["\u98BF", "\u98BF"], /* CJK Ideograph */
  ["\u98C0", "\u98C0"], /* CJK Ideograph */
  ["\u98C1", "\u98C1"], /* CJK Ideograph */
  ["\u98C2", "\u98C2"], /* CJK Ideograph */
  ["\u98C3", "\u98C3"], /* CJK Ideograph */
  ["\u98C4", "\u98C4"], /* CJK Ideograph */
  ["\u98C5", "\u98C5"], /* CJK Ideograph */
  ["\u98C6", "\u98C6"], /* CJK Ideograph */
  ["\u98C7", "\u98C7"], /* CJK Ideograph */
  ["\u98C8", "\u98C8"], /* CJK Ideograph */
  ["\u98C9", "\u98C9"], /* CJK Ideograph */
  ["\u98CA", "\u98CA"], /* CJK Ideograph */
  ["\u98CB", "\u98CB"], /* CJK Ideograph */
  ["\u98CC", "\u98CC"], /* CJK Ideograph */
  ["\u98CD", "\u98CD"], /* CJK Ideograph */
  ["\u98CE", "\u98CE"], /* CJK Ideograph */
  ["\u98CF", "\u98CF"], /* CJK Ideograph */
  ["\u98D0", "\u98D0"], /* CJK Ideograph */
  ["\u98D1", "\u98D1"], /* CJK Ideograph */
  ["\u98D2", "\u98D2"], /* CJK Ideograph */
  ["\u98D3", "\u98D3"], /* CJK Ideograph */
  ["\u98D4", "\u98D4"], /* CJK Ideograph */
  ["\u98D5", "\u98D5"], /* CJK Ideograph */
  ["\u98D6", "\u98D6"], /* CJK Ideograph */
  ["\u98D7", "\u98D7"], /* CJK Ideograph */
  ["\u98D8", "\u98D8"], /* CJK Ideograph */
  ["\u98D9", "\u98D9"], /* CJK Ideograph */
  ["\u98DA", "\u98DA"], /* CJK Ideograph */
  ["\u98DB", "\u98DB"], /* CJK Ideograph */
  ["\u98DC", "\u98DC"], /* CJK Ideograph */
  ["\u98DD", "\u98DD"], /* CJK Ideograph */
  ["\u98DE", "\u98DE"], /* CJK Ideograph */
  ["\u98DF", "\u98DF"], /* CJK Ideograph */
  ["\u98E0", "\u98E0"], /* CJK Ideograph */
  ["\u98E1", "\u98E1"], /* CJK Ideograph */
  ["\u98E2", "\u98E2"], /* CJK Ideograph */
  ["\u98E3", "\u98E3"], /* CJK Ideograph */
  ["\u98E4", "\u98E4"], /* CJK Ideograph */
  ["\u98E5", "\u98E5"], /* CJK Ideograph */
  ["\u98E6", "\u98E6"], /* CJK Ideograph */
  ["\u98E7", "\u98E7"], /* CJK Ideograph */
  ["\u98E8", "\u98E8"], /* CJK Ideograph */
  ["\u98E9", "\u98E9"], /* CJK Ideograph */
  ["\u98EA", "\u98EA"], /* CJK Ideograph */
  ["\u98EB", "\u98EB"], /* CJK Ideograph */
  ["\u98EC", "\u98EC"], /* CJK Ideograph */
  ["\u98ED", "\u98ED"], /* CJK Ideograph */
  ["\u98EE", "\u98EE"], /* CJK Ideograph */
  ["\u98EF", "\u98EF"], /* CJK Ideograph */
  ["\u98F0", "\u98F0"], /* CJK Ideograph */
  ["\u98F1", "\u98F1"], /* CJK Ideograph */
  ["\u98F2", "\u98F2"], /* CJK Ideograph */
  ["\u98F3", "\u98F3"], /* CJK Ideograph */
  ["\u98F4", "\u98F4"], /* CJK Ideograph */
  ["\u98F5", "\u98F5"], /* CJK Ideograph */
  ["\u98F6", "\u98F6"], /* CJK Ideograph */
  ["\u98F7", "\u98F7"], /* CJK Ideograph */
  ["\u98F8", "\u98F8"], /* CJK Ideograph */
  ["\u98F9", "\u98F9"], /* CJK Ideograph */
  ["\u98FA", "\u98FA"], /* CJK Ideograph */
  ["\u98FB", "\u98FB"], /* CJK Ideograph */
  ["\u98FC", "\u98FC"], /* CJK Ideograph */
  ["\u98FD", "\u98FD"], /* CJK Ideograph */
  ["\u98FE", "\u98FE"], /* CJK Ideograph */
  ["\u98FF", "\u98FF"], /* CJK Ideograph */
  ["\u9900", "\u9900"], /* CJK Ideograph */
  ["\u9901", "\u9901"], /* CJK Ideograph */
  ["\u9902", "\u9902"], /* CJK Ideograph */
  ["\u9903", "\u9903"], /* CJK Ideograph */
  ["\u9904", "\u9904"], /* CJK Ideograph */
  ["\u9905", "\u9905"], /* CJK Ideograph */
  ["\u9906", "\u9906"], /* CJK Ideograph */
  ["\u9907", "\u9907"], /* CJK Ideograph */
  ["\u9908", "\u9908"], /* CJK Ideograph */
  ["\u9909", "\u9909"], /* CJK Ideograph */
  ["\u990A", "\u990A"], /* CJK Ideograph */
  ["\u990B", "\u990B"], /* CJK Ideograph */
  ["\u990C", "\u990C"], /* CJK Ideograph */
  ["\u990D", "\u990D"], /* CJK Ideograph */
  ["\u990E", "\u990E"], /* CJK Ideograph */
  ["\u990F", "\u990F"], /* CJK Ideograph */
  ["\u9910", "\u9910"], /* CJK Ideograph */
  ["\u9911", "\u9911"], /* CJK Ideograph */
  ["\u9912", "\u9912"], /* CJK Ideograph */
  ["\u9913", "\u9913"], /* CJK Ideograph */
  ["\u9914", "\u9914"], /* CJK Ideograph */
  ["\u9915", "\u9915"], /* CJK Ideograph */
  ["\u9916", "\u9916"], /* CJK Ideograph */
  ["\u9917", "\u9917"], /* CJK Ideograph */
  ["\u9918", "\u9918"], /* CJK Ideograph */
  ["\u9919", "\u9919"], /* CJK Ideograph */
  ["\u991A", "\u991A"], /* CJK Ideograph */
  ["\u991B", "\u991B"], /* CJK Ideograph */
  ["\u991C", "\u991C"], /* CJK Ideograph */
  ["\u991D", "\u991D"], /* CJK Ideograph */
  ["\u991E", "\u991E"], /* CJK Ideograph */
  ["\u991F", "\u991F"], /* CJK Ideograph */
  ["\u9920", "\u9920"], /* CJK Ideograph */
  ["\u9921", "\u9921"], /* CJK Ideograph */
  ["\u9922", "\u9922"], /* CJK Ideograph */
  ["\u9923", "\u9923"], /* CJK Ideograph */
  ["\u9924", "\u9924"], /* CJK Ideograph */
  ["\u9925", "\u9925"], /* CJK Ideograph */
  ["\u9926", "\u9926"], /* CJK Ideograph */
  ["\u9927", "\u9927"], /* CJK Ideograph */
  ["\u9928", "\u9928"], /* CJK Ideograph */
  ["\u9929", "\u9929"], /* CJK Ideograph */
  ["\u992A", "\u992A"], /* CJK Ideograph */
  ["\u992B", "\u992B"], /* CJK Ideograph */
  ["\u992C", "\u992C"], /* CJK Ideograph */
  ["\u992D", "\u992D"], /* CJK Ideograph */
  ["\u992E", "\u992E"], /* CJK Ideograph */
  ["\u992F", "\u992F"], /* CJK Ideograph */
  ["\u9930", "\u9930"], /* CJK Ideograph */
  ["\u9931", "\u9931"], /* CJK Ideograph */
  ["\u9932", "\u9932"], /* CJK Ideograph */
  ["\u9933", "\u9933"], /* CJK Ideograph */
  ["\u9934", "\u9934"], /* CJK Ideograph */
  ["\u9935", "\u9935"], /* CJK Ideograph */
  ["\u9936", "\u9936"], /* CJK Ideograph */
  ["\u9937", "\u9937"], /* CJK Ideograph */
  ["\u9938", "\u9938"], /* CJK Ideograph */
  ["\u9939", "\u9939"], /* CJK Ideograph */
  ["\u993A", "\u993A"], /* CJK Ideograph */
  ["\u993B", "\u993B"], /* CJK Ideograph */
  ["\u993C", "\u993C"], /* CJK Ideograph */
  ["\u993D", "\u993D"], /* CJK Ideograph */
  ["\u993E", "\u993E"], /* CJK Ideograph */
  ["\u993F", "\u993F"], /* CJK Ideograph */
  ["\u9940", "\u9940"], /* CJK Ideograph */
  ["\u9941", "\u9941"], /* CJK Ideograph */
  ["\u9942", "\u9942"], /* CJK Ideograph */
  ["\u9943", "\u9943"], /* CJK Ideograph */
  ["\u9944", "\u9944"], /* CJK Ideograph */
  ["\u9945", "\u9945"], /* CJK Ideograph */
  ["\u9946", "\u9946"], /* CJK Ideograph */
  ["\u9947", "\u9947"], /* CJK Ideograph */
  ["\u9948", "\u9948"], /* CJK Ideograph */
  ["\u9949", "\u9949"], /* CJK Ideograph */
  ["\u994A", "\u994A"], /* CJK Ideograph */
  ["\u994B", "\u994B"], /* CJK Ideograph */
  ["\u994C", "\u994C"], /* CJK Ideograph */
  ["\u994D", "\u994D"], /* CJK Ideograph */
  ["\u994E", "\u994E"], /* CJK Ideograph */
  ["\u994F", "\u994F"], /* CJK Ideograph */
  ["\u9950", "\u9950"], /* CJK Ideograph */
  ["\u9951", "\u9951"], /* CJK Ideograph */
  ["\u9952", "\u9952"], /* CJK Ideograph */
  ["\u9953", "\u9953"], /* CJK Ideograph */
  ["\u9954", "\u9954"], /* CJK Ideograph */
  ["\u9955", "\u9955"], /* CJK Ideograph */
  ["\u9956", "\u9956"], /* CJK Ideograph */
  ["\u9957", "\u9957"], /* CJK Ideograph */
  ["\u9958", "\u9958"], /* CJK Ideograph */
  ["\u9959", "\u9959"], /* CJK Ideograph */
  ["\u995A", "\u995A"], /* CJK Ideograph */
  ["\u995B", "\u995B"], /* CJK Ideograph */
  ["\u995C", "\u995C"], /* CJK Ideograph */
  ["\u995D", "\u995D"], /* CJK Ideograph */
  ["\u995E", "\u995E"], /* CJK Ideograph */
  ["\u995F", "\u995F"], /* CJK Ideograph */
  ["\u9960", "\u9960"], /* CJK Ideograph */
  ["\u9961", "\u9961"], /* CJK Ideograph */
  ["\u9962", "\u9962"], /* CJK Ideograph */
  ["\u9963", "\u9963"], /* CJK Ideograph */
  ["\u9964", "\u9964"], /* CJK Ideograph */
  ["\u9965", "\u9965"], /* CJK Ideograph */
  ["\u9966", "\u9966"], /* CJK Ideograph */
  ["\u9967", "\u9967"], /* CJK Ideograph */
  ["\u9968", "\u9968"], /* CJK Ideograph */
  ["\u9969", "\u9969"], /* CJK Ideograph */
  ["\u996A", "\u996A"], /* CJK Ideograph */
  ["\u996B", "\u996B"], /* CJK Ideograph */
  ["\u996C", "\u996C"], /* CJK Ideograph */
  ["\u996D", "\u996D"], /* CJK Ideograph */
  ["\u996E", "\u996E"], /* CJK Ideograph */
  ["\u996F", "\u996F"], /* CJK Ideograph */
  ["\u9970", "\u9970"], /* CJK Ideograph */
  ["\u9971", "\u9971"], /* CJK Ideograph */
  ["\u9972", "\u9972"], /* CJK Ideograph */
  ["\u9973", "\u9973"], /* CJK Ideograph */
  ["\u9974", "\u9974"], /* CJK Ideograph */
  ["\u9975", "\u9975"], /* CJK Ideograph */
  ["\u9976", "\u9976"], /* CJK Ideograph */
  ["\u9977", "\u9977"], /* CJK Ideograph */
  ["\u9978", "\u9978"], /* CJK Ideograph */
  ["\u9979", "\u9979"], /* CJK Ideograph */
  ["\u997A", "\u997A"], /* CJK Ideograph */
  ["\u997B", "\u997B"], /* CJK Ideograph */
  ["\u997C", "\u997C"], /* CJK Ideograph */
  ["\u997D", "\u997D"], /* CJK Ideograph */
  ["\u997E", "\u997E"], /* CJK Ideograph */
  ["\u997F", "\u997F"], /* CJK Ideograph */
  ["\u9980", "\u9980"], /* CJK Ideograph */
  ["\u9981", "\u9981"], /* CJK Ideograph */
  ["\u9982", "\u9982"], /* CJK Ideograph */
  ["\u9983", "\u9983"], /* CJK Ideograph */
  ["\u9984", "\u9984"], /* CJK Ideograph */
  ["\u9985", "\u9985"], /* CJK Ideograph */
  ["\u9986", "\u9986"], /* CJK Ideograph */
  ["\u9987", "\u9987"], /* CJK Ideograph */
  ["\u9988", "\u9988"], /* CJK Ideograph */
  ["\u9989", "\u9989"], /* CJK Ideograph */
  ["\u998A", "\u998A"], /* CJK Ideograph */
  ["\u998B", "\u998B"], /* CJK Ideograph */
  ["\u998C", "\u998C"], /* CJK Ideograph */
  ["\u998D", "\u998D"], /* CJK Ideograph */
  ["\u998E", "\u998E"], /* CJK Ideograph */
  ["\u998F", "\u998F"], /* CJK Ideograph */
  ["\u9990", "\u9990"], /* CJK Ideograph */
  ["\u9991", "\u9991"], /* CJK Ideograph */
  ["\u9992", "\u9992"], /* CJK Ideograph */
  ["\u9993", "\u9993"], /* CJK Ideograph */
  ["\u9994", "\u9994"], /* CJK Ideograph */
  ["\u9995", "\u9995"], /* CJK Ideograph */
  ["\u9996", "\u9996"], /* CJK Ideograph */
  ["\u9997", "\u9997"], /* CJK Ideograph */
  ["\u9998", "\u9998"], /* CJK Ideograph */
  ["\u9999", "\u9999"], /* CJK Ideograph */
  ["\u999A", "\u999A"], /* CJK Ideograph */
  ["\u999B", "\u999B"], /* CJK Ideograph */
  ["\u999C", "\u999C"], /* CJK Ideograph */
  ["\u999D", "\u999D"], /* CJK Ideograph */
  ["\u999E", "\u999E"], /* CJK Ideograph */
  ["\u999F", "\u999F"], /* CJK Ideograph */
  ["\u99A0", "\u99A0"], /* CJK Ideograph */
  ["\u99A1", "\u99A1"], /* CJK Ideograph */
  ["\u99A2", "\u99A2"], /* CJK Ideograph */
  ["\u99A3", "\u99A3"], /* CJK Ideograph */
  ["\u99A4", "\u99A4"], /* CJK Ideograph */
  ["\u99A5", "\u99A5"], /* CJK Ideograph */
  ["\u99A6", "\u99A6"], /* CJK Ideograph */
  ["\u99A7", "\u99A7"], /* CJK Ideograph */
  ["\u99A8", "\u99A8"], /* CJK Ideograph */
  ["\u99A9", "\u99A9"], /* CJK Ideograph */
  ["\u99AA", "\u99AA"], /* CJK Ideograph */
  ["\u99AB", "\u99AB"], /* CJK Ideograph */
  ["\u99AC", "\u99AC"], /* CJK Ideograph */
  ["\u99AD", "\u99AD"], /* CJK Ideograph */
  ["\u99AE", "\u99AE"], /* CJK Ideograph */
  ["\u99AF", "\u99AF"], /* CJK Ideograph */
  ["\u99B0", "\u99B0"], /* CJK Ideograph */
  ["\u99B1", "\u99B1"], /* CJK Ideograph */
  ["\u99B2", "\u99B2"], /* CJK Ideograph */
  ["\u99B3", "\u99B3"], /* CJK Ideograph */
  ["\u99B4", "\u99B4"], /* CJK Ideograph */
  ["\u99B5", "\u99B5"], /* CJK Ideograph */
  ["\u99B6", "\u99B6"], /* CJK Ideograph */
  ["\u99B7", "\u99B7"], /* CJK Ideograph */
  ["\u99B8", "\u99B8"], /* CJK Ideograph */
  ["\u99B9", "\u99B9"], /* CJK Ideograph */
  ["\u99BA", "\u99BA"], /* CJK Ideograph */
  ["\u99BB", "\u99BB"], /* CJK Ideograph */
  ["\u99BC", "\u99BC"], /* CJK Ideograph */
  ["\u99BD", "\u99BD"], /* CJK Ideograph */
  ["\u99BE", "\u99BE"], /* CJK Ideograph */
  ["\u99BF", "\u99BF"], /* CJK Ideograph */
  ["\u99C0", "\u99C0"], /* CJK Ideograph */
  ["\u99C1", "\u99C1"], /* CJK Ideograph */
  ["\u99C2", "\u99C2"], /* CJK Ideograph */
  ["\u99C3", "\u99C3"], /* CJK Ideograph */
  ["\u99C4", "\u99C4"], /* CJK Ideograph */
  ["\u99C5", "\u99C5"], /* CJK Ideograph */
  ["\u99C6", "\u99C6"], /* CJK Ideograph */
  ["\u99C7", "\u99C7"], /* CJK Ideograph */
  ["\u99C8", "\u99C8"], /* CJK Ideograph */
  ["\u99C9", "\u99C9"], /* CJK Ideograph */
  ["\u99CA", "\u99CA"], /* CJK Ideograph */
  ["\u99CB", "\u99CB"], /* CJK Ideograph */
  ["\u99CC", "\u99CC"], /* CJK Ideograph */
  ["\u99CD", "\u99CD"], /* CJK Ideograph */
  ["\u99CE", "\u99CE"], /* CJK Ideograph */
  ["\u99CF", "\u99CF"], /* CJK Ideograph */
  ["\u99D0", "\u99D0"], /* CJK Ideograph */
  ["\u99D1", "\u99D1"], /* CJK Ideograph */
  ["\u99D2", "\u99D2"], /* CJK Ideograph */
  ["\u99D3", "\u99D3"], /* CJK Ideograph */
  ["\u99D4", "\u99D4"], /* CJK Ideograph */
  ["\u99D5", "\u99D5"], /* CJK Ideograph */
  ["\u99D6", "\u99D6"], /* CJK Ideograph */
  ["\u99D7", "\u99D7"], /* CJK Ideograph */
  ["\u99D8", "\u99D8"], /* CJK Ideograph */
  ["\u99D9", "\u99D9"], /* CJK Ideograph */
  ["\u99DA", "\u99DA"], /* CJK Ideograph */
  ["\u99DB", "\u99DB"], /* CJK Ideograph */
  ["\u99DC", "\u99DC"], /* CJK Ideograph */
  ["\u99DD", "\u99DD"], /* CJK Ideograph */
  ["\u99DE", "\u99DE"], /* CJK Ideograph */
  ["\u99DF", "\u99DF"], /* CJK Ideograph */
  ["\u99E0", "\u99E0"], /* CJK Ideograph */
  ["\u99E1", "\u99E1"], /* CJK Ideograph */
  ["\u99E2", "\u99E2"], /* CJK Ideograph */
  ["\u99E3", "\u99E3"], /* CJK Ideograph */
  ["\u99E4", "\u99E4"], /* CJK Ideograph */
  ["\u99E5", "\u99E5"], /* CJK Ideograph */
  ["\u99E6", "\u99E6"], /* CJK Ideograph */
  ["\u99E7", "\u99E7"], /* CJK Ideograph */
  ["\u99E8", "\u99E8"], /* CJK Ideograph */
  ["\u99E9", "\u99E9"], /* CJK Ideograph */
  ["\u99EA", "\u99EA"], /* CJK Ideograph */
  ["\u99EB", "\u99EB"], /* CJK Ideograph */
  ["\u99EC", "\u99EC"], /* CJK Ideograph */
  ["\u99ED", "\u99ED"], /* CJK Ideograph */
  ["\u99EE", "\u99EE"], /* CJK Ideograph */
  ["\u99EF", "\u99EF"], /* CJK Ideograph */
  ["\u99F0", "\u99F0"], /* CJK Ideograph */
  ["\u99F1", "\u99F1"], /* CJK Ideograph */
  ["\u99F2", "\u99F2"], /* CJK Ideograph */
  ["\u99F3", "\u99F3"], /* CJK Ideograph */
  ["\u99F4", "\u99F4"], /* CJK Ideograph */
  ["\u99F5", "\u99F5"], /* CJK Ideograph */
  ["\u99F6", "\u99F6"], /* CJK Ideograph */
  ["\u99F7", "\u99F7"], /* CJK Ideograph */
  ["\u99F8", "\u99F8"], /* CJK Ideograph */
  ["\u99F9", "\u99F9"], /* CJK Ideograph */
  ["\u99FA", "\u99FA"], /* CJK Ideograph */
  ["\u99FB", "\u99FB"], /* CJK Ideograph */
  ["\u99FC", "\u99FC"], /* CJK Ideograph */
  ["\u99FD", "\u99FD"], /* CJK Ideograph */
  ["\u99FE", "\u99FE"], /* CJK Ideograph */
  ["\u99FF", "\u99FF"], /* CJK Ideograph */
  ["\u9A00", "\u9A00"], /* CJK Ideograph */
  ["\u9A01", "\u9A01"], /* CJK Ideograph */
  ["\u9A02", "\u9A02"], /* CJK Ideograph */
  ["\u9A03", "\u9A03"], /* CJK Ideograph */
  ["\u9A04", "\u9A04"], /* CJK Ideograph */
  ["\u9A05", "\u9A05"], /* CJK Ideograph */
  ["\u9A06", "\u9A06"], /* CJK Ideograph */
  ["\u9A07", "\u9A07"], /* CJK Ideograph */
  ["\u9A08", "\u9A08"], /* CJK Ideograph */
  ["\u9A09", "\u9A09"], /* CJK Ideograph */
  ["\u9A0A", "\u9A0A"], /* CJK Ideograph */
  ["\u9A0B", "\u9A0B"], /* CJK Ideograph */
  ["\u9A0C", "\u9A0C"], /* CJK Ideograph */
  ["\u9A0D", "\u9A0D"], /* CJK Ideograph */
  ["\u9A0E", "\u9A0E"], /* CJK Ideograph */
  ["\u9A0F", "\u9A0F"], /* CJK Ideograph */
  ["\u9A10", "\u9A10"], /* CJK Ideograph */
  ["\u9A11", "\u9A11"], /* CJK Ideograph */
  ["\u9A12", "\u9A12"], /* CJK Ideograph */
  ["\u9A13", "\u9A13"], /* CJK Ideograph */
  ["\u9A14", "\u9A14"], /* CJK Ideograph */
  ["\u9A15", "\u9A15"], /* CJK Ideograph */
  ["\u9A16", "\u9A16"], /* CJK Ideograph */
  ["\u9A17", "\u9A17"], /* CJK Ideograph */
  ["\u9A18", "\u9A18"], /* CJK Ideograph */
  ["\u9A19", "\u9A19"], /* CJK Ideograph */
  ["\u9A1A", "\u9A1A"], /* CJK Ideograph */
  ["\u9A1B", "\u9A1B"], /* CJK Ideograph */
  ["\u9A1C", "\u9A1C"], /* CJK Ideograph */
  ["\u9A1D", "\u9A1D"], /* CJK Ideograph */
  ["\u9A1E", "\u9A1E"], /* CJK Ideograph */
  ["\u9A1F", "\u9A1F"], /* CJK Ideograph */
  ["\u9A20", "\u9A20"], /* CJK Ideograph */
  ["\u9A21", "\u9A21"], /* CJK Ideograph */
  ["\u9A22", "\u9A22"], /* CJK Ideograph */
  ["\u9A23", "\u9A23"], /* CJK Ideograph */
  ["\u9A24", "\u9A24"], /* CJK Ideograph */
  ["\u9A25", "\u9A25"], /* CJK Ideograph */
  ["\u9A26", "\u9A26"], /* CJK Ideograph */
  ["\u9A27", "\u9A27"], /* CJK Ideograph */
  ["\u9A28", "\u9A28"], /* CJK Ideograph */
  ["\u9A29", "\u9A29"], /* CJK Ideograph */
  ["\u9A2A", "\u9A2A"], /* CJK Ideograph */
  ["\u9A2B", "\u9A2B"], /* CJK Ideograph */
  ["\u9A2C", "\u9A2C"], /* CJK Ideograph */
  ["\u9A2D", "\u9A2D"], /* CJK Ideograph */
  ["\u9A2E", "\u9A2E"], /* CJK Ideograph */
  ["\u9A2F", "\u9A2F"], /* CJK Ideograph */
  ["\u9A30", "\u9A30"], /* CJK Ideograph */
  ["\u9A31", "\u9A31"], /* CJK Ideograph */
  ["\u9A32", "\u9A32"], /* CJK Ideograph */
  ["\u9A33", "\u9A33"], /* CJK Ideograph */
  ["\u9A34", "\u9A34"], /* CJK Ideograph */
  ["\u9A35", "\u9A35"], /* CJK Ideograph */
  ["\u9A36", "\u9A36"], /* CJK Ideograph */
  ["\u9A37", "\u9A37"], /* CJK Ideograph */
  ["\u9A38", "\u9A38"], /* CJK Ideograph */
  ["\u9A39", "\u9A39"], /* CJK Ideograph */
  ["\u9A3A", "\u9A3A"], /* CJK Ideograph */
  ["\u9A3B", "\u9A3B"], /* CJK Ideograph */
  ["\u9A3C", "\u9A3C"], /* CJK Ideograph */
  ["\u9A3D", "\u9A3D"], /* CJK Ideograph */
  ["\u9A3E", "\u9A3E"], /* CJK Ideograph */
  ["\u9A3F", "\u9A3F"], /* CJK Ideograph */
  ["\u9A40", "\u9A40"], /* CJK Ideograph */
  ["\u9A41", "\u9A41"], /* CJK Ideograph */
  ["\u9A42", "\u9A42"], /* CJK Ideograph */
  ["\u9A43", "\u9A43"], /* CJK Ideograph */
  ["\u9A44", "\u9A44"], /* CJK Ideograph */
  ["\u9A45", "\u9A45"], /* CJK Ideograph */
  ["\u9A46", "\u9A46"], /* CJK Ideograph */
  ["\u9A47", "\u9A47"], /* CJK Ideograph */
  ["\u9A48", "\u9A48"], /* CJK Ideograph */
  ["\u9A49", "\u9A49"], /* CJK Ideograph */
  ["\u9A4A", "\u9A4A"], /* CJK Ideograph */
  ["\u9A4B", "\u9A4B"], /* CJK Ideograph */
  ["\u9A4C", "\u9A4C"], /* CJK Ideograph */
  ["\u9A4D", "\u9A4D"], /* CJK Ideograph */
  ["\u9A4E", "\u9A4E"], /* CJK Ideograph */
  ["\u9A4F", "\u9A4F"], /* CJK Ideograph */
  ["\u9A50", "\u9A50"], /* CJK Ideograph */
  ["\u9A51", "\u9A51"], /* CJK Ideograph */
  ["\u9A52", "\u9A52"], /* CJK Ideograph */
  ["\u9A53", "\u9A53"], /* CJK Ideograph */
  ["\u9A54", "\u9A54"], /* CJK Ideograph */
  ["\u9A55", "\u9A55"], /* CJK Ideograph */
  ["\u9A56", "\u9A56"], /* CJK Ideograph */
  ["\u9A57", "\u9A57"], /* CJK Ideograph */
  ["\u9A58", "\u9A58"], /* CJK Ideograph */
  ["\u9A59", "\u9A59"], /* CJK Ideograph */
  ["\u9A5A", "\u9A5A"], /* CJK Ideograph */
  ["\u9A5B", "\u9A5B"], /* CJK Ideograph */
  ["\u9A5C", "\u9A5C"], /* CJK Ideograph */
  ["\u9A5D", "\u9A5D"], /* CJK Ideograph */
  ["\u9A5E", "\u9A5E"], /* CJK Ideograph */
  ["\u9A5F", "\u9A5F"], /* CJK Ideograph */
  ["\u9A60", "\u9A60"], /* CJK Ideograph */
  ["\u9A61", "\u9A61"], /* CJK Ideograph */
  ["\u9A62", "\u9A62"], /* CJK Ideograph */
  ["\u9A63", "\u9A63"], /* CJK Ideograph */
  ["\u9A64", "\u9A64"], /* CJK Ideograph */
  ["\u9A65", "\u9A65"], /* CJK Ideograph */
  ["\u9A66", "\u9A66"], /* CJK Ideograph */
  ["\u9A67", "\u9A67"], /* CJK Ideograph */
  ["\u9A68", "\u9A68"], /* CJK Ideograph */
  ["\u9A69", "\u9A69"], /* CJK Ideograph */
  ["\u9A6A", "\u9A6A"], /* CJK Ideograph */
  ["\u9A6B", "\u9A6B"], /* CJK Ideograph */
  ["\u9A6C", "\u9A6C"], /* CJK Ideograph */
  ["\u9A6D", "\u9A6D"], /* CJK Ideograph */
  ["\u9A6E", "\u9A6E"], /* CJK Ideograph */
  ["\u9A6F", "\u9A6F"], /* CJK Ideograph */
  ["\u9A70", "\u9A70"], /* CJK Ideograph */
  ["\u9A71", "\u9A71"], /* CJK Ideograph */
  ["\u9A72", "\u9A72"], /* CJK Ideograph */
  ["\u9A73", "\u9A73"], /* CJK Ideograph */
  ["\u9A74", "\u9A74"], /* CJK Ideograph */
  ["\u9A75", "\u9A75"], /* CJK Ideograph */
  ["\u9A76", "\u9A76"], /* CJK Ideograph */
  ["\u9A77", "\u9A77"], /* CJK Ideograph */
  ["\u9A78", "\u9A78"], /* CJK Ideograph */
  ["\u9A79", "\u9A79"], /* CJK Ideograph */
  ["\u9A7A", "\u9A7A"], /* CJK Ideograph */
  ["\u9A7B", "\u9A7B"], /* CJK Ideograph */
  ["\u9A7C", "\u9A7C"], /* CJK Ideograph */
  ["\u9A7D", "\u9A7D"], /* CJK Ideograph */
  ["\u9A7E", "\u9A7E"], /* CJK Ideograph */
  ["\u9A7F", "\u9A7F"], /* CJK Ideograph */
  ["\u9A80", "\u9A80"], /* CJK Ideograph */
  ["\u9A81", "\u9A81"], /* CJK Ideograph */
  ["\u9A82", "\u9A82"], /* CJK Ideograph */
  ["\u9A83", "\u9A83"], /* CJK Ideograph */
  ["\u9A84", "\u9A84"], /* CJK Ideograph */
  ["\u9A85", "\u9A85"], /* CJK Ideograph */
  ["\u9A86", "\u9A86"], /* CJK Ideograph */
  ["\u9A87", "\u9A87"], /* CJK Ideograph */
  ["\u9A88", "\u9A88"], /* CJK Ideograph */
  ["\u9A89", "\u9A89"], /* CJK Ideograph */
  ["\u9A8A", "\u9A8A"], /* CJK Ideograph */
  ["\u9A8B", "\u9A8B"], /* CJK Ideograph */
  ["\u9A8C", "\u9A8C"], /* CJK Ideograph */
  ["\u9A8D", "\u9A8D"], /* CJK Ideograph */
  ["\u9A8E", "\u9A8E"], /* CJK Ideograph */
  ["\u9A8F", "\u9A8F"], /* CJK Ideograph */
  ["\u9A90", "\u9A90"], /* CJK Ideograph */
  ["\u9A91", "\u9A91"], /* CJK Ideograph */
  ["\u9A92", "\u9A92"], /* CJK Ideograph */
  ["\u9A93", "\u9A93"], /* CJK Ideograph */
  ["\u9A94", "\u9A94"], /* CJK Ideograph */
  ["\u9A95", "\u9A95"], /* CJK Ideograph */
  ["\u9A96", "\u9A96"], /* CJK Ideograph */
  ["\u9A97", "\u9A97"], /* CJK Ideograph */
  ["\u9A98", "\u9A98"], /* CJK Ideograph */
  ["\u9A99", "\u9A99"], /* CJK Ideograph */
  ["\u9A9A", "\u9A9A"], /* CJK Ideograph */
  ["\u9A9B", "\u9A9B"], /* CJK Ideograph */
  ["\u9A9C", "\u9A9C"], /* CJK Ideograph */
  ["\u9A9D", "\u9A9D"], /* CJK Ideograph */
  ["\u9A9E", "\u9A9E"], /* CJK Ideograph */
  ["\u9A9F", "\u9A9F"], /* CJK Ideograph */
  ["\u9AA0", "\u9AA0"], /* CJK Ideograph */
  ["\u9AA1", "\u9AA1"], /* CJK Ideograph */
  ["\u9AA2", "\u9AA2"], /* CJK Ideograph */
  ["\u9AA3", "\u9AA3"], /* CJK Ideograph */
  ["\u9AA4", "\u9AA4"], /* CJK Ideograph */
  ["\u9AA5", "\u9AA5"], /* CJK Ideograph */
  ["\u9AA6", "\u9AA6"], /* CJK Ideograph */
  ["\u9AA7", "\u9AA7"], /* CJK Ideograph */
  ["\u9AA8", "\u9AA8"], /* CJK Ideograph */
  ["\u9AA9", "\u9AA9"], /* CJK Ideograph */
  ["\u9AAA", "\u9AAA"], /* CJK Ideograph */
  ["\u9AAB", "\u9AAB"], /* CJK Ideograph */
  ["\u9AAC", "\u9AAC"], /* CJK Ideograph */
  ["\u9AAD", "\u9AAD"], /* CJK Ideograph */
  ["\u9AAE", "\u9AAE"], /* CJK Ideograph */
  ["\u9AAF", "\u9AAF"], /* CJK Ideograph */
  ["\u9AB0", "\u9AB0"], /* CJK Ideograph */
  ["\u9AB1", "\u9AB1"], /* CJK Ideograph */
  ["\u9AB2", "\u9AB2"], /* CJK Ideograph */
  ["\u9AB3", "\u9AB3"], /* CJK Ideograph */
  ["\u9AB4", "\u9AB4"], /* CJK Ideograph */
  ["\u9AB5", "\u9AB5"], /* CJK Ideograph */
  ["\u9AB6", "\u9AB6"], /* CJK Ideograph */
  ["\u9AB7", "\u9AB7"], /* CJK Ideograph */
  ["\u9AB8", "\u9AB8"], /* CJK Ideograph */
  ["\u9AB9", "\u9AB9"], /* CJK Ideograph */
  ["\u9ABA", "\u9ABA"], /* CJK Ideograph */
  ["\u9ABB", "\u9ABB"], /* CJK Ideograph */
  ["\u9ABC", "\u9ABC"], /* CJK Ideograph */
  ["\u9ABD", "\u9ABD"], /* CJK Ideograph */
  ["\u9ABE", "\u9ABE"], /* CJK Ideograph */
  ["\u9ABF", "\u9ABF"], /* CJK Ideograph */
  ["\u9AC0", "\u9AC0"], /* CJK Ideograph */
  ["\u9AC1", "\u9AC1"], /* CJK Ideograph */
  ["\u9AC2", "\u9AC2"], /* CJK Ideograph */
  ["\u9AC3", "\u9AC3"], /* CJK Ideograph */
  ["\u9AC4", "\u9AC4"], /* CJK Ideograph */
  ["\u9AC5", "\u9AC5"], /* CJK Ideograph */
  ["\u9AC6", "\u9AC6"], /* CJK Ideograph */
  ["\u9AC7", "\u9AC7"], /* CJK Ideograph */
  ["\u9AC8", "\u9AC8"], /* CJK Ideograph */
  ["\u9AC9", "\u9AC9"], /* CJK Ideograph */
  ["\u9ACA", "\u9ACA"], /* CJK Ideograph */
  ["\u9ACB", "\u9ACB"], /* CJK Ideograph */
  ["\u9ACC", "\u9ACC"], /* CJK Ideograph */
  ["\u9ACD", "\u9ACD"], /* CJK Ideograph */
  ["\u9ACE", "\u9ACE"], /* CJK Ideograph */
  ["\u9ACF", "\u9ACF"], /* CJK Ideograph */
  ["\u9AD0", "\u9AD0"], /* CJK Ideograph */
  ["\u9AD1", "\u9AD1"], /* CJK Ideograph */
  ["\u9AD2", "\u9AD2"], /* CJK Ideograph */
  ["\u9AD3", "\u9AD3"], /* CJK Ideograph */
  ["\u9AD4", "\u9AD4"], /* CJK Ideograph */
  ["\u9AD5", "\u9AD5"], /* CJK Ideograph */
  ["\u9AD6", "\u9AD6"], /* CJK Ideograph */
  ["\u9AD7", "\u9AD7"], /* CJK Ideograph */
  ["\u9AD8", "\u9AD8"], /* CJK Ideograph */
  ["\u9AD9", "\u9AD9"], /* CJK Ideograph */
  ["\u9ADA", "\u9ADA"], /* CJK Ideograph */
  ["\u9ADB", "\u9ADB"], /* CJK Ideograph */
  ["\u9ADC", "\u9ADC"], /* CJK Ideograph */
  ["\u9ADD", "\u9ADD"], /* CJK Ideograph */
  ["\u9ADE", "\u9ADE"], /* CJK Ideograph */
  ["\u9ADF", "\u9ADF"], /* CJK Ideograph */
  ["\u9AE0", "\u9AE0"], /* CJK Ideograph */
  ["\u9AE1", "\u9AE1"], /* CJK Ideograph */
  ["\u9AE2", "\u9AE2"], /* CJK Ideograph */
  ["\u9AE3", "\u9AE3"], /* CJK Ideograph */
  ["\u9AE4", "\u9AE4"], /* CJK Ideograph */
  ["\u9AE5", "\u9AE5"], /* CJK Ideograph */
  ["\u9AE6", "\u9AE6"], /* CJK Ideograph */
  ["\u9AE7", "\u9AE7"], /* CJK Ideograph */
  ["\u9AE8", "\u9AE8"], /* CJK Ideograph */
  ["\u9AE9", "\u9AE9"], /* CJK Ideograph */
  ["\u9AEA", "\u9AEA"], /* CJK Ideograph */
  ["\u9AEB", "\u9AEB"], /* CJK Ideograph */
  ["\u9AEC", "\u9AEC"], /* CJK Ideograph */
  ["\u9AED", "\u9AED"], /* CJK Ideograph */
  ["\u9AEE", "\u9AEE"], /* CJK Ideograph */
  ["\u9AEF", "\u9AEF"], /* CJK Ideograph */
  ["\u9AF0", "\u9AF0"], /* CJK Ideograph */
  ["\u9AF1", "\u9AF1"], /* CJK Ideograph */
  ["\u9AF2", "\u9AF2"], /* CJK Ideograph */
  ["\u9AF3", "\u9AF3"], /* CJK Ideograph */
  ["\u9AF4", "\u9AF4"], /* CJK Ideograph */
  ["\u9AF5", "\u9AF5"], /* CJK Ideograph */
  ["\u9AF6", "\u9AF6"], /* CJK Ideograph */
  ["\u9AF7", "\u9AF7"], /* CJK Ideograph */
  ["\u9AF8", "\u9AF8"], /* CJK Ideograph */
  ["\u9AF9", "\u9AF9"], /* CJK Ideograph */
  ["\u9AFA", "\u9AFA"], /* CJK Ideograph */
  ["\u9AFB", "\u9AFB"], /* CJK Ideograph */
  ["\u9AFC", "\u9AFC"], /* CJK Ideograph */
  ["\u9AFD", "\u9AFD"], /* CJK Ideograph */
  ["\u9AFE", "\u9AFE"], /* CJK Ideograph */
  ["\u9AFF", "\u9AFF"], /* CJK Ideograph */
  ["\u9B00", "\u9B00"], /* CJK Ideograph */
  ["\u9B01", "\u9B01"], /* CJK Ideograph */
  ["\u9B02", "\u9B02"], /* CJK Ideograph */
  ["\u9B03", "\u9B03"], /* CJK Ideograph */
  ["\u9B04", "\u9B04"], /* CJK Ideograph */
  ["\u9B05", "\u9B05"], /* CJK Ideograph */
  ["\u9B06", "\u9B06"], /* CJK Ideograph */
  ["\u9B07", "\u9B07"], /* CJK Ideograph */
  ["\u9B08", "\u9B08"], /* CJK Ideograph */
  ["\u9B09", "\u9B09"], /* CJK Ideograph */
  ["\u9B0A", "\u9B0A"], /* CJK Ideograph */
  ["\u9B0B", "\u9B0B"], /* CJK Ideograph */
  ["\u9B0C", "\u9B0C"], /* CJK Ideograph */
  ["\u9B0D", "\u9B0D"], /* CJK Ideograph */
  ["\u9B0E", "\u9B0E"], /* CJK Ideograph */
  ["\u9B0F", "\u9B0F"], /* CJK Ideograph */
  ["\u9B10", "\u9B10"], /* CJK Ideograph */
  ["\u9B11", "\u9B11"], /* CJK Ideograph */
  ["\u9B12", "\u9B12"], /* CJK Ideograph */
  ["\u9B13", "\u9B13"], /* CJK Ideograph */
  ["\u9B14", "\u9B14"], /* CJK Ideograph */
  ["\u9B15", "\u9B15"], /* CJK Ideograph */
  ["\u9B16", "\u9B16"], /* CJK Ideograph */
  ["\u9B17", "\u9B17"], /* CJK Ideograph */
  ["\u9B18", "\u9B18"], /* CJK Ideograph */
  ["\u9B19", "\u9B19"], /* CJK Ideograph */
  ["\u9B1A", "\u9B1A"], /* CJK Ideograph */
  ["\u9B1B", "\u9B1B"], /* CJK Ideograph */
  ["\u9B1C", "\u9B1C"], /* CJK Ideograph */
  ["\u9B1D", "\u9B1D"], /* CJK Ideograph */
  ["\u9B1E", "\u9B1E"], /* CJK Ideograph */
  ["\u9B1F", "\u9B1F"], /* CJK Ideograph */
  ["\u9B20", "\u9B20"], /* CJK Ideograph */
  ["\u9B21", "\u9B21"], /* CJK Ideograph */
  ["\u9B22", "\u9B22"], /* CJK Ideograph */
  ["\u9B23", "\u9B23"], /* CJK Ideograph */
  ["\u9B24", "\u9B24"], /* CJK Ideograph */
  ["\u9B25", "\u9B25"], /* CJK Ideograph */
  ["\u9B26", "\u9B26"], /* CJK Ideograph */
  ["\u9B27", "\u9B27"], /* CJK Ideograph */
  ["\u9B28", "\u9B28"], /* CJK Ideograph */
  ["\u9B29", "\u9B29"], /* CJK Ideograph */
  ["\u9B2A", "\u9B2A"], /* CJK Ideograph */
  ["\u9B2B", "\u9B2B"], /* CJK Ideograph */
  ["\u9B2C", "\u9B2C"], /* CJK Ideograph */
  ["\u9B2D", "\u9B2D"], /* CJK Ideograph */
  ["\u9B2E", "\u9B2E"], /* CJK Ideograph */
  ["\u9B2F", "\u9B2F"], /* CJK Ideograph */
  ["\u9B30", "\u9B30"], /* CJK Ideograph */
  ["\u9B31", "\u9B31"], /* CJK Ideograph */
  ["\u9B32", "\u9B32"], /* CJK Ideograph */
  ["\u9B33", "\u9B33"], /* CJK Ideograph */
  ["\u9B34", "\u9B34"], /* CJK Ideograph */
  ["\u9B35", "\u9B35"], /* CJK Ideograph */
  ["\u9B36", "\u9B36"], /* CJK Ideograph */
  ["\u9B37", "\u9B37"], /* CJK Ideograph */
  ["\u9B38", "\u9B38"], /* CJK Ideograph */
  ["\u9B39", "\u9B39"], /* CJK Ideograph */
  ["\u9B3A", "\u9B3A"], /* CJK Ideograph */
  ["\u9B3B", "\u9B3B"], /* CJK Ideograph */
  ["\u9B3C", "\u9B3C"], /* CJK Ideograph */
  ["\u9B3D", "\u9B3D"], /* CJK Ideograph */
  ["\u9B3E", "\u9B3E"], /* CJK Ideograph */
  ["\u9B3F", "\u9B3F"], /* CJK Ideograph */
  ["\u9B40", "\u9B40"], /* CJK Ideograph */
  ["\u9B41", "\u9B41"], /* CJK Ideograph */
  ["\u9B42", "\u9B42"], /* CJK Ideograph */
  ["\u9B43", "\u9B43"], /* CJK Ideograph */
  ["\u9B44", "\u9B44"], /* CJK Ideograph */
  ["\u9B45", "\u9B45"], /* CJK Ideograph */
  ["\u9B46", "\u9B46"], /* CJK Ideograph */
  ["\u9B47", "\u9B47"], /* CJK Ideograph */
  ["\u9B48", "\u9B48"], /* CJK Ideograph */
  ["\u9B49", "\u9B49"], /* CJK Ideograph */
  ["\u9B4A", "\u9B4A"], /* CJK Ideograph */
  ["\u9B4B", "\u9B4B"], /* CJK Ideograph */
  ["\u9B4C", "\u9B4C"], /* CJK Ideograph */
  ["\u9B4D", "\u9B4D"], /* CJK Ideograph */
  ["\u9B4E", "\u9B4E"], /* CJK Ideograph */
  ["\u9B4F", "\u9B4F"], /* CJK Ideograph */
  ["\u9B50", "\u9B50"], /* CJK Ideograph */
  ["\u9B51", "\u9B51"], /* CJK Ideograph */
  ["\u9B52", "\u9B52"], /* CJK Ideograph */
  ["\u9B53", "\u9B53"], /* CJK Ideograph */
  ["\u9B54", "\u9B54"], /* CJK Ideograph */
  ["\u9B55", "\u9B55"], /* CJK Ideograph */
  ["\u9B56", "\u9B56"], /* CJK Ideograph */
  ["\u9B57", "\u9B57"], /* CJK Ideograph */
  ["\u9B58", "\u9B58"], /* CJK Ideograph */
  ["\u9B59", "\u9B59"], /* CJK Ideograph */
  ["\u9B5A", "\u9B5A"], /* CJK Ideograph */
  ["\u9B5B", "\u9B5B"], /* CJK Ideograph */
  ["\u9B5C", "\u9B5C"], /* CJK Ideograph */
  ["\u9B5D", "\u9B5D"], /* CJK Ideograph */
  ["\u9B5E", "\u9B5E"], /* CJK Ideograph */
  ["\u9B5F", "\u9B5F"], /* CJK Ideograph */
  ["\u9B60", "\u9B60"], /* CJK Ideograph */
  ["\u9B61", "\u9B61"], /* CJK Ideograph */
  ["\u9B62", "\u9B62"], /* CJK Ideograph */
  ["\u9B63", "\u9B63"], /* CJK Ideograph */
  ["\u9B64", "\u9B64"], /* CJK Ideograph */
  ["\u9B65", "\u9B65"], /* CJK Ideograph */
  ["\u9B66", "\u9B66"], /* CJK Ideograph */
  ["\u9B67", "\u9B67"], /* CJK Ideograph */
  ["\u9B68", "\u9B68"], /* CJK Ideograph */
  ["\u9B69", "\u9B69"], /* CJK Ideograph */
  ["\u9B6A", "\u9B6A"], /* CJK Ideograph */
  ["\u9B6B", "\u9B6B"], /* CJK Ideograph */
  ["\u9B6C", "\u9B6C"], /* CJK Ideograph */
  ["\u9B6D", "\u9B6D"], /* CJK Ideograph */
  ["\u9B6E", "\u9B6E"], /* CJK Ideograph */
  ["\u9B6F", "\u9B6F"], /* CJK Ideograph */
  ["\u9B70", "\u9B70"], /* CJK Ideograph */
  ["\u9B71", "\u9B71"], /* CJK Ideograph */
  ["\u9B72", "\u9B72"], /* CJK Ideograph */
  ["\u9B73", "\u9B73"], /* CJK Ideograph */
  ["\u9B74", "\u9B74"], /* CJK Ideograph */
  ["\u9B75", "\u9B75"], /* CJK Ideograph */
  ["\u9B76", "\u9B76"], /* CJK Ideograph */
  ["\u9B77", "\u9B77"], /* CJK Ideograph */
  ["\u9B78", "\u9B78"], /* CJK Ideograph */
  ["\u9B79", "\u9B79"], /* CJK Ideograph */
  ["\u9B7A", "\u9B7A"], /* CJK Ideograph */
  ["\u9B7B", "\u9B7B"], /* CJK Ideograph */
  ["\u9B7C", "\u9B7C"], /* CJK Ideograph */
  ["\u9B7D", "\u9B7D"], /* CJK Ideograph */
  ["\u9B7E", "\u9B7E"], /* CJK Ideograph */
  ["\u9B7F", "\u9B7F"], /* CJK Ideograph */
  ["\u9B80", "\u9B80"], /* CJK Ideograph */
  ["\u9B81", "\u9B81"], /* CJK Ideograph */
  ["\u9B82", "\u9B82"], /* CJK Ideograph */
  ["\u9B83", "\u9B83"], /* CJK Ideograph */
  ["\u9B84", "\u9B84"], /* CJK Ideograph */
  ["\u9B85", "\u9B85"], /* CJK Ideograph */
  ["\u9B86", "\u9B86"], /* CJK Ideograph */
  ["\u9B87", "\u9B87"], /* CJK Ideograph */
  ["\u9B88", "\u9B88"], /* CJK Ideograph */
  ["\u9B89", "\u9B89"], /* CJK Ideograph */
  ["\u9B8A", "\u9B8A"], /* CJK Ideograph */
  ["\u9B8B", "\u9B8B"], /* CJK Ideograph */
  ["\u9B8C", "\u9B8C"], /* CJK Ideograph */
  ["\u9B8D", "\u9B8D"], /* CJK Ideograph */
  ["\u9B8E", "\u9B8E"], /* CJK Ideograph */
  ["\u9B8F", "\u9B8F"], /* CJK Ideograph */
  ["\u9B90", "\u9B90"], /* CJK Ideograph */
  ["\u9B91", "\u9B91"], /* CJK Ideograph */
  ["\u9B92", "\u9B92"], /* CJK Ideograph */
  ["\u9B93", "\u9B93"], /* CJK Ideograph */
  ["\u9B94", "\u9B94"], /* CJK Ideograph */
  ["\u9B95", "\u9B95"], /* CJK Ideograph */
  ["\u9B96", "\u9B96"], /* CJK Ideograph */
  ["\u9B97", "\u9B97"], /* CJK Ideograph */
  ["\u9B98", "\u9B98"], /* CJK Ideograph */
  ["\u9B99", "\u9B99"], /* CJK Ideograph */
  ["\u9B9A", "\u9B9A"], /* CJK Ideograph */
  ["\u9B9B", "\u9B9B"], /* CJK Ideograph */
  ["\u9B9C", "\u9B9C"], /* CJK Ideograph */
  ["\u9B9D", "\u9B9D"], /* CJK Ideograph */
  ["\u9B9E", "\u9B9E"], /* CJK Ideograph */
  ["\u9B9F", "\u9B9F"], /* CJK Ideograph */
  ["\u9BA0", "\u9BA0"], /* CJK Ideograph */
  ["\u9BA1", "\u9BA1"], /* CJK Ideograph */
  ["\u9BA2", "\u9BA2"], /* CJK Ideograph */
  ["\u9BA3", "\u9BA3"], /* CJK Ideograph */
  ["\u9BA4", "\u9BA4"], /* CJK Ideograph */
  ["\u9BA5", "\u9BA5"], /* CJK Ideograph */
  ["\u9BA6", "\u9BA6"], /* CJK Ideograph */
  ["\u9BA7", "\u9BA7"], /* CJK Ideograph */
  ["\u9BA8", "\u9BA8"], /* CJK Ideograph */
  ["\u9BA9", "\u9BA9"], /* CJK Ideograph */
  ["\u9BAA", "\u9BAA"], /* CJK Ideograph */
  ["\u9BAB", "\u9BAB"], /* CJK Ideograph */
  ["\u9BAC", "\u9BAC"], /* CJK Ideograph */
  ["\u9BAD", "\u9BAD"], /* CJK Ideograph */
  ["\u9BAE", "\u9BAE"], /* CJK Ideograph */
  ["\u9BAF", "\u9BAF"], /* CJK Ideograph */
  ["\u9BB0", "\u9BB0"], /* CJK Ideograph */
  ["\u9BB1", "\u9BB1"], /* CJK Ideograph */
  ["\u9BB2", "\u9BB2"], /* CJK Ideograph */
  ["\u9BB3", "\u9BB3"], /* CJK Ideograph */
  ["\u9BB4", "\u9BB4"], /* CJK Ideograph */
  ["\u9BB5", "\u9BB5"], /* CJK Ideograph */
  ["\u9BB6", "\u9BB6"], /* CJK Ideograph */
  ["\u9BB7", "\u9BB7"], /* CJK Ideograph */
  ["\u9BB8", "\u9BB8"], /* CJK Ideograph */
  ["\u9BB9", "\u9BB9"], /* CJK Ideograph */
  ["\u9BBA", "\u9BBA"], /* CJK Ideograph */
  ["\u9BBB", "\u9BBB"], /* CJK Ideograph */
  ["\u9BBC", "\u9BBC"], /* CJK Ideograph */
  ["\u9BBD", "\u9BBD"], /* CJK Ideograph */
  ["\u9BBE", "\u9BBE"], /* CJK Ideograph */
  ["\u9BBF", "\u9BBF"], /* CJK Ideograph */
  ["\u9BC0", "\u9BC0"], /* CJK Ideograph */
  ["\u9BC1", "\u9BC1"], /* CJK Ideograph */
  ["\u9BC2", "\u9BC2"], /* CJK Ideograph */
  ["\u9BC3", "\u9BC3"], /* CJK Ideograph */
  ["\u9BC4", "\u9BC4"], /* CJK Ideograph */
  ["\u9BC5", "\u9BC5"], /* CJK Ideograph */
  ["\u9BC6", "\u9BC6"], /* CJK Ideograph */
  ["\u9BC7", "\u9BC7"], /* CJK Ideograph */
  ["\u9BC8", "\u9BC8"], /* CJK Ideograph */
  ["\u9BC9", "\u9BC9"], /* CJK Ideograph */
  ["\u9BCA", "\u9BCA"], /* CJK Ideograph */
  ["\u9BCB", "\u9BCB"], /* CJK Ideograph */
  ["\u9BCC", "\u9BCC"], /* CJK Ideograph */
  ["\u9BCD", "\u9BCD"], /* CJK Ideograph */
  ["\u9BCE", "\u9BCE"], /* CJK Ideograph */
  ["\u9BCF", "\u9BCF"], /* CJK Ideograph */
  ["\u9BD0", "\u9BD0"], /* CJK Ideograph */
  ["\u9BD1", "\u9BD1"], /* CJK Ideograph */
  ["\u9BD2", "\u9BD2"], /* CJK Ideograph */
  ["\u9BD3", "\u9BD3"], /* CJK Ideograph */
  ["\u9BD4", "\u9BD4"], /* CJK Ideograph */
  ["\u9BD5", "\u9BD5"], /* CJK Ideograph */
  ["\u9BD6", "\u9BD6"], /* CJK Ideograph */
  ["\u9BD7", "\u9BD7"], /* CJK Ideograph */
  ["\u9BD8", "\u9BD8"], /* CJK Ideograph */
  ["\u9BD9", "\u9BD9"], /* CJK Ideograph */
  ["\u9BDA", "\u9BDA"], /* CJK Ideograph */
  ["\u9BDB", "\u9BDB"], /* CJK Ideograph */
  ["\u9BDC", "\u9BDC"], /* CJK Ideograph */
  ["\u9BDD", "\u9BDD"], /* CJK Ideograph */
  ["\u9BDE", "\u9BDE"], /* CJK Ideograph */
  ["\u9BDF", "\u9BDF"], /* CJK Ideograph */
  ["\u9BE0", "\u9BE0"], /* CJK Ideograph */
  ["\u9BE1", "\u9BE1"], /* CJK Ideograph */
  ["\u9BE2", "\u9BE2"], /* CJK Ideograph */
  ["\u9BE3", "\u9BE3"], /* CJK Ideograph */
  ["\u9BE4", "\u9BE4"], /* CJK Ideograph */
  ["\u9BE5", "\u9BE5"], /* CJK Ideograph */
  ["\u9BE6", "\u9BE6"], /* CJK Ideograph */
  ["\u9BE7", "\u9BE7"], /* CJK Ideograph */
  ["\u9BE8", "\u9BE8"], /* CJK Ideograph */
  ["\u9BE9", "\u9BE9"], /* CJK Ideograph */
  ["\u9BEA", "\u9BEA"], /* CJK Ideograph */
  ["\u9BEB", "\u9BEB"], /* CJK Ideograph */
  ["\u9BEC", "\u9BEC"], /* CJK Ideograph */
  ["\u9BED", "\u9BED"], /* CJK Ideograph */
  ["\u9BEE", "\u9BEE"], /* CJK Ideograph */
  ["\u9BEF", "\u9BEF"], /* CJK Ideograph */
  ["\u9BF0", "\u9BF0"], /* CJK Ideograph */
  ["\u9BF1", "\u9BF1"], /* CJK Ideograph */
  ["\u9BF2", "\u9BF2"], /* CJK Ideograph */
  ["\u9BF3", "\u9BF3"], /* CJK Ideograph */
  ["\u9BF4", "\u9BF4"], /* CJK Ideograph */
  ["\u9BF5", "\u9BF5"], /* CJK Ideograph */
  ["\u9BF6", "\u9BF6"], /* CJK Ideograph */
  ["\u9BF7", "\u9BF7"], /* CJK Ideograph */
  ["\u9BF8", "\u9BF8"], /* CJK Ideograph */
  ["\u9BF9", "\u9BF9"], /* CJK Ideograph */
  ["\u9BFA", "\u9BFA"], /* CJK Ideograph */
  ["\u9BFB", "\u9BFB"], /* CJK Ideograph */
  ["\u9BFC", "\u9BFC"], /* CJK Ideograph */
  ["\u9BFD", "\u9BFD"], /* CJK Ideograph */
  ["\u9BFE", "\u9BFE"], /* CJK Ideograph */
  ["\u9BFF", "\u9BFF"], /* CJK Ideograph */
  ["\u9C00", "\u9C00"], /* CJK Ideograph */
  ["\u9C01", "\u9C01"], /* CJK Ideograph */
  ["\u9C02", "\u9C02"], /* CJK Ideograph */
  ["\u9C03", "\u9C03"], /* CJK Ideograph */
  ["\u9C04", "\u9C04"], /* CJK Ideograph */
  ["\u9C05", "\u9C05"], /* CJK Ideograph */
  ["\u9C06", "\u9C06"], /* CJK Ideograph */
  ["\u9C07", "\u9C07"], /* CJK Ideograph */
  ["\u9C08", "\u9C08"], /* CJK Ideograph */
  ["\u9C09", "\u9C09"], /* CJK Ideograph */
  ["\u9C0A", "\u9C0A"], /* CJK Ideograph */
  ["\u9C0B", "\u9C0B"], /* CJK Ideograph */
  ["\u9C0C", "\u9C0C"], /* CJK Ideograph */
  ["\u9C0D", "\u9C0D"], /* CJK Ideograph */
  ["\u9C0E", "\u9C0E"], /* CJK Ideograph */
  ["\u9C0F", "\u9C0F"], /* CJK Ideograph */
  ["\u9C10", "\u9C10"], /* CJK Ideograph */
  ["\u9C11", "\u9C11"], /* CJK Ideograph */
  ["\u9C12", "\u9C12"], /* CJK Ideograph */
  ["\u9C13", "\u9C13"], /* CJK Ideograph */
  ["\u9C14", "\u9C14"], /* CJK Ideograph */
  ["\u9C15", "\u9C15"], /* CJK Ideograph */
  ["\u9C16", "\u9C16"], /* CJK Ideograph */
  ["\u9C17", "\u9C17"], /* CJK Ideograph */
  ["\u9C18", "\u9C18"], /* CJK Ideograph */
  ["\u9C19", "\u9C19"], /* CJK Ideograph */
  ["\u9C1A", "\u9C1A"], /* CJK Ideograph */
  ["\u9C1B", "\u9C1B"], /* CJK Ideograph */
  ["\u9C1C", "\u9C1C"], /* CJK Ideograph */
  ["\u9C1D", "\u9C1D"], /* CJK Ideograph */
  ["\u9C1E", "\u9C1E"], /* CJK Ideograph */
  ["\u9C1F", "\u9C1F"], /* CJK Ideograph */
  ["\u9C20", "\u9C20"], /* CJK Ideograph */
  ["\u9C21", "\u9C21"], /* CJK Ideograph */
  ["\u9C22", "\u9C22"], /* CJK Ideograph */
  ["\u9C23", "\u9C23"], /* CJK Ideograph */
  ["\u9C24", "\u9C24"], /* CJK Ideograph */
  ["\u9C25", "\u9C25"], /* CJK Ideograph */
  ["\u9C26", "\u9C26"], /* CJK Ideograph */
  ["\u9C27", "\u9C27"], /* CJK Ideograph */
  ["\u9C28", "\u9C28"], /* CJK Ideograph */
  ["\u9C29", "\u9C29"], /* CJK Ideograph */
  ["\u9C2A", "\u9C2A"], /* CJK Ideograph */
  ["\u9C2B", "\u9C2B"], /* CJK Ideograph */
  ["\u9C2C", "\u9C2C"], /* CJK Ideograph */
  ["\u9C2D", "\u9C2D"], /* CJK Ideograph */
  ["\u9C2E", "\u9C2E"], /* CJK Ideograph */
  ["\u9C2F", "\u9C2F"], /* CJK Ideograph */
  ["\u9C30", "\u9C30"], /* CJK Ideograph */
  ["\u9C31", "\u9C31"], /* CJK Ideograph */
  ["\u9C32", "\u9C32"], /* CJK Ideograph */
  ["\u9C33", "\u9C33"], /* CJK Ideograph */
  ["\u9C34", "\u9C34"], /* CJK Ideograph */
  ["\u9C35", "\u9C35"], /* CJK Ideograph */
  ["\u9C36", "\u9C36"], /* CJK Ideograph */
  ["\u9C37", "\u9C37"], /* CJK Ideograph */
  ["\u9C38", "\u9C38"], /* CJK Ideograph */
  ["\u9C39", "\u9C39"], /* CJK Ideograph */
  ["\u9C3A", "\u9C3A"], /* CJK Ideograph */
  ["\u9C3B", "\u9C3B"], /* CJK Ideograph */
  ["\u9C3C", "\u9C3C"], /* CJK Ideograph */
  ["\u9C3D", "\u9C3D"], /* CJK Ideograph */
  ["\u9C3E", "\u9C3E"], /* CJK Ideograph */
  ["\u9C3F", "\u9C3F"], /* CJK Ideograph */
  ["\u9C40", "\u9C40"], /* CJK Ideograph */
  ["\u9C41", "\u9C41"], /* CJK Ideograph */
  ["\u9C42", "\u9C42"], /* CJK Ideograph */
  ["\u9C43", "\u9C43"], /* CJK Ideograph */
  ["\u9C44", "\u9C44"], /* CJK Ideograph */
  ["\u9C45", "\u9C45"], /* CJK Ideograph */
  ["\u9C46", "\u9C46"], /* CJK Ideograph */
  ["\u9C47", "\u9C47"], /* CJK Ideograph */
  ["\u9C48", "\u9C48"], /* CJK Ideograph */
  ["\u9C49", "\u9C49"], /* CJK Ideograph */
  ["\u9C4A", "\u9C4A"], /* CJK Ideograph */
  ["\u9C4B", "\u9C4B"], /* CJK Ideograph */
  ["\u9C4C", "\u9C4C"], /* CJK Ideograph */
  ["\u9C4D", "\u9C4D"], /* CJK Ideograph */
  ["\u9C4E", "\u9C4E"], /* CJK Ideograph */
  ["\u9C4F", "\u9C4F"], /* CJK Ideograph */
  ["\u9C50", "\u9C50"], /* CJK Ideograph */
  ["\u9C51", "\u9C51"], /* CJK Ideograph */
  ["\u9C52", "\u9C52"], /* CJK Ideograph */
  ["\u9C53", "\u9C53"], /* CJK Ideograph */
  ["\u9C54", "\u9C54"], /* CJK Ideograph */
  ["\u9C55", "\u9C55"], /* CJK Ideograph */
  ["\u9C56", "\u9C56"], /* CJK Ideograph */
  ["\u9C57", "\u9C57"], /* CJK Ideograph */
  ["\u9C58", "\u9C58"], /* CJK Ideograph */
  ["\u9C59", "\u9C59"], /* CJK Ideograph */
  ["\u9C5A", "\u9C5A"], /* CJK Ideograph */
  ["\u9C5B", "\u9C5B"], /* CJK Ideograph */
  ["\u9C5C", "\u9C5C"], /* CJK Ideograph */
  ["\u9C5D", "\u9C5D"], /* CJK Ideograph */
  ["\u9C5E", "\u9C5E"], /* CJK Ideograph */
  ["\u9C5F", "\u9C5F"], /* CJK Ideograph */
  ["\u9C60", "\u9C60"], /* CJK Ideograph */
  ["\u9C61", "\u9C61"], /* CJK Ideograph */
  ["\u9C62", "\u9C62"], /* CJK Ideograph */
  ["\u9C63", "\u9C63"], /* CJK Ideograph */
  ["\u9C64", "\u9C64"], /* CJK Ideograph */
  ["\u9C65", "\u9C65"], /* CJK Ideograph */
  ["\u9C66", "\u9C66"], /* CJK Ideograph */
  ["\u9C67", "\u9C67"], /* CJK Ideograph */
  ["\u9C68", "\u9C68"], /* CJK Ideograph */
  ["\u9C69", "\u9C69"], /* CJK Ideograph */
  ["\u9C6A", "\u9C6A"], /* CJK Ideograph */
  ["\u9C6B", "\u9C6B"], /* CJK Ideograph */
  ["\u9C6C", "\u9C6C"], /* CJK Ideograph */
  ["\u9C6D", "\u9C6D"], /* CJK Ideograph */
  ["\u9C6E", "\u9C6E"], /* CJK Ideograph */
  ["\u9C6F", "\u9C6F"], /* CJK Ideograph */
  ["\u9C70", "\u9C70"], /* CJK Ideograph */
  ["\u9C71", "\u9C71"], /* CJK Ideograph */
  ["\u9C72", "\u9C72"], /* CJK Ideograph */
  ["\u9C73", "\u9C73"], /* CJK Ideograph */
  ["\u9C74", "\u9C74"], /* CJK Ideograph */
  ["\u9C75", "\u9C75"], /* CJK Ideograph */
  ["\u9C76", "\u9C76"], /* CJK Ideograph */
  ["\u9C77", "\u9C77"], /* CJK Ideograph */
  ["\u9C78", "\u9C78"], /* CJK Ideograph */
  ["\u9C79", "\u9C79"], /* CJK Ideograph */
  ["\u9C7A", "\u9C7A"], /* CJK Ideograph */
  ["\u9C7B", "\u9C7B"], /* CJK Ideograph */
  ["\u9C7C", "\u9C7C"], /* CJK Ideograph */
  ["\u9C7D", "\u9C7D"], /* CJK Ideograph */
  ["\u9C7E", "\u9C7E"], /* CJK Ideograph */
  ["\u9C7F", "\u9C7F"], /* CJK Ideograph */
  ["\u9C80", "\u9C80"], /* CJK Ideograph */
  ["\u9C81", "\u9C81"], /* CJK Ideograph */
  ["\u9C82", "\u9C82"], /* CJK Ideograph */
  ["\u9C83", "\u9C83"], /* CJK Ideograph */
  ["\u9C84", "\u9C84"], /* CJK Ideograph */
  ["\u9C85", "\u9C85"], /* CJK Ideograph */
  ["\u9C86", "\u9C86"], /* CJK Ideograph */
  ["\u9C87", "\u9C87"], /* CJK Ideograph */
  ["\u9C88", "\u9C88"], /* CJK Ideograph */
  ["\u9C89", "\u9C89"], /* CJK Ideograph */
  ["\u9C8A", "\u9C8A"], /* CJK Ideograph */
  ["\u9C8B", "\u9C8B"], /* CJK Ideograph */
  ["\u9C8C", "\u9C8C"], /* CJK Ideograph */
  ["\u9C8D", "\u9C8D"], /* CJK Ideograph */
  ["\u9C8E", "\u9C8E"], /* CJK Ideograph */
  ["\u9C8F", "\u9C8F"], /* CJK Ideograph */
  ["\u9C90", "\u9C90"], /* CJK Ideograph */
  ["\u9C91", "\u9C91"], /* CJK Ideograph */
  ["\u9C92", "\u9C92"], /* CJK Ideograph */
  ["\u9C93", "\u9C93"], /* CJK Ideograph */
  ["\u9C94", "\u9C94"], /* CJK Ideograph */
  ["\u9C95", "\u9C95"], /* CJK Ideograph */
  ["\u9C96", "\u9C96"], /* CJK Ideograph */
  ["\u9C97", "\u9C97"], /* CJK Ideograph */
  ["\u9C98", "\u9C98"], /* CJK Ideograph */
  ["\u9C99", "\u9C99"], /* CJK Ideograph */
  ["\u9C9A", "\u9C9A"], /* CJK Ideograph */
  ["\u9C9B", "\u9C9B"], /* CJK Ideograph */
  ["\u9C9C", "\u9C9C"], /* CJK Ideograph */
  ["\u9C9D", "\u9C9D"], /* CJK Ideograph */
  ["\u9C9E", "\u9C9E"], /* CJK Ideograph */
  ["\u9C9F", "\u9C9F"], /* CJK Ideograph */
  ["\u9CA0", "\u9CA0"], /* CJK Ideograph */
  ["\u9CA1", "\u9CA1"], /* CJK Ideograph */
  ["\u9CA2", "\u9CA2"], /* CJK Ideograph */
  ["\u9CA3", "\u9CA3"], /* CJK Ideograph */
  ["\u9CA4", "\u9CA4"], /* CJK Ideograph */
  ["\u9CA5", "\u9CA5"], /* CJK Ideograph */
  ["\u9CA6", "\u9CA6"], /* CJK Ideograph */
  ["\u9CA7", "\u9CA7"], /* CJK Ideograph */
  ["\u9CA8", "\u9CA8"], /* CJK Ideograph */
  ["\u9CA9", "\u9CA9"], /* CJK Ideograph */
  ["\u9CAA", "\u9CAA"], /* CJK Ideograph */
  ["\u9CAB", "\u9CAB"], /* CJK Ideograph */
  ["\u9CAC", "\u9CAC"], /* CJK Ideograph */
  ["\u9CAD", "\u9CAD"], /* CJK Ideograph */
  ["\u9CAE", "\u9CAE"], /* CJK Ideograph */
  ["\u9CAF", "\u9CAF"], /* CJK Ideograph */
  ["\u9CB0", "\u9CB0"], /* CJK Ideograph */
  ["\u9CB1", "\u9CB1"], /* CJK Ideograph */
  ["\u9CB2", "\u9CB2"], /* CJK Ideograph */
  ["\u9CB3", "\u9CB3"], /* CJK Ideograph */
  ["\u9CB4", "\u9CB4"], /* CJK Ideograph */
  ["\u9CB5", "\u9CB5"], /* CJK Ideograph */
  ["\u9CB6", "\u9CB6"], /* CJK Ideograph */
  ["\u9CB7", "\u9CB7"], /* CJK Ideograph */
  ["\u9CB8", "\u9CB8"], /* CJK Ideograph */
  ["\u9CB9", "\u9CB9"], /* CJK Ideograph */
  ["\u9CBA", "\u9CBA"], /* CJK Ideograph */
  ["\u9CBB", "\u9CBB"], /* CJK Ideograph */
  ["\u9CBC", "\u9CBC"], /* CJK Ideograph */
  ["\u9CBD", "\u9CBD"], /* CJK Ideograph */
  ["\u9CBE", "\u9CBE"], /* CJK Ideograph */
  ["\u9CBF", "\u9CBF"], /* CJK Ideograph */
  ["\u9CC0", "\u9CC0"], /* CJK Ideograph */
  ["\u9CC1", "\u9CC1"], /* CJK Ideograph */
  ["\u9CC2", "\u9CC2"], /* CJK Ideograph */
  ["\u9CC3", "\u9CC3"], /* CJK Ideograph */
  ["\u9CC4", "\u9CC4"], /* CJK Ideograph */
  ["\u9CC5", "\u9CC5"], /* CJK Ideograph */
  ["\u9CC6", "\u9CC6"], /* CJK Ideograph */
  ["\u9CC7", "\u9CC7"], /* CJK Ideograph */
  ["\u9CC8", "\u9CC8"], /* CJK Ideograph */
  ["\u9CC9", "\u9CC9"], /* CJK Ideograph */
  ["\u9CCA", "\u9CCA"], /* CJK Ideograph */
  ["\u9CCB", "\u9CCB"], /* CJK Ideograph */
  ["\u9CCC", "\u9CCC"], /* CJK Ideograph */
  ["\u9CCD", "\u9CCD"], /* CJK Ideograph */
  ["\u9CCE", "\u9CCE"], /* CJK Ideograph */
  ["\u9CCF", "\u9CCF"], /* CJK Ideograph */
  ["\u9CD0", "\u9CD0"], /* CJK Ideograph */
  ["\u9CD1", "\u9CD1"], /* CJK Ideograph */
  ["\u9CD2", "\u9CD2"], /* CJK Ideograph */
  ["\u9CD3", "\u9CD3"], /* CJK Ideograph */
  ["\u9CD4", "\u9CD4"], /* CJK Ideograph */
  ["\u9CD5", "\u9CD5"], /* CJK Ideograph */
  ["\u9CD6", "\u9CD6"], /* CJK Ideograph */
  ["\u9CD7", "\u9CD7"], /* CJK Ideograph */
  ["\u9CD8", "\u9CD8"], /* CJK Ideograph */
  ["\u9CD9", "\u9CD9"], /* CJK Ideograph */
  ["\u9CDA", "\u9CDA"], /* CJK Ideograph */
  ["\u9CDB", "\u9CDB"], /* CJK Ideograph */
  ["\u9CDC", "\u9CDC"], /* CJK Ideograph */
  ["\u9CDD", "\u9CDD"], /* CJK Ideograph */
  ["\u9CDE", "\u9CDE"], /* CJK Ideograph */
  ["\u9CDF", "\u9CDF"], /* CJK Ideograph */
  ["\u9CE0", "\u9CE0"], /* CJK Ideograph */
  ["\u9CE1", "\u9CE1"], /* CJK Ideograph */
  ["\u9CE2", "\u9CE2"], /* CJK Ideograph */
  ["\u9CE3", "\u9CE3"], /* CJK Ideograph */
  ["\u9CE4", "\u9CE4"], /* CJK Ideograph */
  ["\u9CE5", "\u9CE5"], /* CJK Ideograph */
  ["\u9CE6", "\u9CE6"], /* CJK Ideograph */
  ["\u9CE7", "\u9CE7"], /* CJK Ideograph */
  ["\u9CE8", "\u9CE8"], /* CJK Ideograph */
  ["\u9CE9", "\u9CE9"], /* CJK Ideograph */
  ["\u9CEA", "\u9CEA"], /* CJK Ideograph */
  ["\u9CEB", "\u9CEB"], /* CJK Ideograph */
  ["\u9CEC", "\u9CEC"], /* CJK Ideograph */
  ["\u9CED", "\u9CED"], /* CJK Ideograph */
  ["\u9CEE", "\u9CEE"], /* CJK Ideograph */
  ["\u9CEF", "\u9CEF"], /* CJK Ideograph */
  ["\u9CF0", "\u9CF0"], /* CJK Ideograph */
  ["\u9CF1", "\u9CF1"], /* CJK Ideograph */
  ["\u9CF2", "\u9CF2"], /* CJK Ideograph */
  ["\u9CF3", "\u9CF3"], /* CJK Ideograph */
  ["\u9CF4", "\u9CF4"], /* CJK Ideograph */
  ["\u9CF5", "\u9CF5"], /* CJK Ideograph */
  ["\u9CF6", "\u9CF6"], /* CJK Ideograph */
  ["\u9CF7", "\u9CF7"], /* CJK Ideograph */
  ["\u9CF8", "\u9CF8"], /* CJK Ideograph */
  ["\u9CF9", "\u9CF9"], /* CJK Ideograph */
  ["\u9CFA", "\u9CFA"], /* CJK Ideograph */
  ["\u9CFB", "\u9CFB"], /* CJK Ideograph */
  ["\u9CFC", "\u9CFC"], /* CJK Ideograph */
  ["\u9CFD", "\u9CFD"], /* CJK Ideograph */
  ["\u9CFE", "\u9CFE"], /* CJK Ideograph */
  ["\u9CFF", "\u9CFF"], /* CJK Ideograph */
  ["\u9D00", "\u9D00"], /* CJK Ideograph */
  ["\u9D01", "\u9D01"], /* CJK Ideograph */
  ["\u9D02", "\u9D02"], /* CJK Ideograph */
  ["\u9D03", "\u9D03"], /* CJK Ideograph */
  ["\u9D04", "\u9D04"], /* CJK Ideograph */
  ["\u9D05", "\u9D05"], /* CJK Ideograph */
  ["\u9D06", "\u9D06"], /* CJK Ideograph */
  ["\u9D07", "\u9D07"], /* CJK Ideograph */
  ["\u9D08", "\u9D08"], /* CJK Ideograph */
  ["\u9D09", "\u9D09"], /* CJK Ideograph */
  ["\u9D0A", "\u9D0A"], /* CJK Ideograph */
  ["\u9D0B", "\u9D0B"], /* CJK Ideograph */
  ["\u9D0C", "\u9D0C"], /* CJK Ideograph */
  ["\u9D0D", "\u9D0D"], /* CJK Ideograph */
  ["\u9D0E", "\u9D0E"], /* CJK Ideograph */
  ["\u9D0F", "\u9D0F"], /* CJK Ideograph */
  ["\u9D10", "\u9D10"], /* CJK Ideograph */
  ["\u9D11", "\u9D11"], /* CJK Ideograph */
  ["\u9D12", "\u9D12"], /* CJK Ideograph */
  ["\u9D13", "\u9D13"], /* CJK Ideograph */
  ["\u9D14", "\u9D14"], /* CJK Ideograph */
  ["\u9D15", "\u9D15"], /* CJK Ideograph */
  ["\u9D16", "\u9D16"], /* CJK Ideograph */
  ["\u9D17", "\u9D17"], /* CJK Ideograph */
  ["\u9D18", "\u9D18"], /* CJK Ideograph */
  ["\u9D19", "\u9D19"], /* CJK Ideograph */
  ["\u9D1A", "\u9D1A"], /* CJK Ideograph */
  ["\u9D1B", "\u9D1B"], /* CJK Ideograph */
  ["\u9D1C", "\u9D1C"], /* CJK Ideograph */
  ["\u9D1D", "\u9D1D"], /* CJK Ideograph */
  ["\u9D1E", "\u9D1E"], /* CJK Ideograph */
  ["\u9D1F", "\u9D1F"], /* CJK Ideograph */
  ["\u9D20", "\u9D20"], /* CJK Ideograph */
  ["\u9D21", "\u9D21"], /* CJK Ideograph */
  ["\u9D22", "\u9D22"], /* CJK Ideograph */
  ["\u9D23", "\u9D23"], /* CJK Ideograph */
  ["\u9D24", "\u9D24"], /* CJK Ideograph */
  ["\u9D25", "\u9D25"], /* CJK Ideograph */
  ["\u9D26", "\u9D26"], /* CJK Ideograph */
  ["\u9D27", "\u9D27"], /* CJK Ideograph */
  ["\u9D28", "\u9D28"], /* CJK Ideograph */
  ["\u9D29", "\u9D29"], /* CJK Ideograph */
  ["\u9D2A", "\u9D2A"], /* CJK Ideograph */
  ["\u9D2B", "\u9D2B"], /* CJK Ideograph */
  ["\u9D2C", "\u9D2C"], /* CJK Ideograph */
  ["\u9D2D", "\u9D2D"], /* CJK Ideograph */
  ["\u9D2E", "\u9D2E"], /* CJK Ideograph */
  ["\u9D2F", "\u9D2F"], /* CJK Ideograph */
  ["\u9D30", "\u9D30"], /* CJK Ideograph */
  ["\u9D31", "\u9D31"], /* CJK Ideograph */
  ["\u9D32", "\u9D32"], /* CJK Ideograph */
  ["\u9D33", "\u9D33"], /* CJK Ideograph */
  ["\u9D34", "\u9D34"], /* CJK Ideograph */
  ["\u9D35", "\u9D35"], /* CJK Ideograph */
  ["\u9D36", "\u9D36"], /* CJK Ideograph */
  ["\u9D37", "\u9D37"], /* CJK Ideograph */
  ["\u9D38", "\u9D38"], /* CJK Ideograph */
  ["\u9D39", "\u9D39"], /* CJK Ideograph */
  ["\u9D3A", "\u9D3A"], /* CJK Ideograph */
  ["\u9D3B", "\u9D3B"], /* CJK Ideograph */
  ["\u9D3C", "\u9D3C"], /* CJK Ideograph */
  ["\u9D3D", "\u9D3D"], /* CJK Ideograph */
  ["\u9D3E", "\u9D3E"], /* CJK Ideograph */
  ["\u9D3F", "\u9D3F"], /* CJK Ideograph */
  ["\u9D40", "\u9D40"], /* CJK Ideograph */
  ["\u9D41", "\u9D41"], /* CJK Ideograph */
  ["\u9D42", "\u9D42"], /* CJK Ideograph */
  ["\u9D43", "\u9D43"], /* CJK Ideograph */
  ["\u9D44", "\u9D44"], /* CJK Ideograph */
  ["\u9D45", "\u9D45"], /* CJK Ideograph */
  ["\u9D46", "\u9D46"], /* CJK Ideograph */
  ["\u9D47", "\u9D47"], /* CJK Ideograph */
  ["\u9D48", "\u9D48"], /* CJK Ideograph */
  ["\u9D49", "\u9D49"], /* CJK Ideograph */
  ["\u9D4A", "\u9D4A"], /* CJK Ideograph */
  ["\u9D4B", "\u9D4B"], /* CJK Ideograph */
  ["\u9D4C", "\u9D4C"], /* CJK Ideograph */
  ["\u9D4D", "\u9D4D"], /* CJK Ideograph */
  ["\u9D4E", "\u9D4E"], /* CJK Ideograph */
  ["\u9D4F", "\u9D4F"], /* CJK Ideograph */
  ["\u9D50", "\u9D50"], /* CJK Ideograph */
  ["\u9D51", "\u9D51"], /* CJK Ideograph */
  ["\u9D52", "\u9D52"], /* CJK Ideograph */
  ["\u9D53", "\u9D53"], /* CJK Ideograph */
  ["\u9D54", "\u9D54"], /* CJK Ideograph */
  ["\u9D55", "\u9D55"], /* CJK Ideograph */
  ["\u9D56", "\u9D56"], /* CJK Ideograph */
  ["\u9D57", "\u9D57"], /* CJK Ideograph */
  ["\u9D58", "\u9D58"], /* CJK Ideograph */
  ["\u9D59", "\u9D59"], /* CJK Ideograph */
  ["\u9D5A", "\u9D5A"], /* CJK Ideograph */
  ["\u9D5B", "\u9D5B"], /* CJK Ideograph */
  ["\u9D5C", "\u9D5C"], /* CJK Ideograph */
  ["\u9D5D", "\u9D5D"], /* CJK Ideograph */
  ["\u9D5E", "\u9D5E"], /* CJK Ideograph */
  ["\u9D5F", "\u9D5F"], /* CJK Ideograph */
  ["\u9D60", "\u9D60"], /* CJK Ideograph */
  ["\u9D61", "\u9D61"], /* CJK Ideograph */
  ["\u9D62", "\u9D62"], /* CJK Ideograph */
  ["\u9D63", "\u9D63"], /* CJK Ideograph */
  ["\u9D64", "\u9D64"], /* CJK Ideograph */
  ["\u9D65", "\u9D65"], /* CJK Ideograph */
  ["\u9D66", "\u9D66"], /* CJK Ideograph */
  ["\u9D67", "\u9D67"], /* CJK Ideograph */
  ["\u9D68", "\u9D68"], /* CJK Ideograph */
  ["\u9D69", "\u9D69"], /* CJK Ideograph */
  ["\u9D6A", "\u9D6A"], /* CJK Ideograph */
  ["\u9D6B", "\u9D6B"], /* CJK Ideograph */
  ["\u9D6C", "\u9D6C"], /* CJK Ideograph */
  ["\u9D6D", "\u9D6D"], /* CJK Ideograph */
  ["\u9D6E", "\u9D6E"], /* CJK Ideograph */
  ["\u9D6F", "\u9D6F"], /* CJK Ideograph */
  ["\u9D70", "\u9D70"], /* CJK Ideograph */
  ["\u9D71", "\u9D71"], /* CJK Ideograph */
  ["\u9D72", "\u9D72"], /* CJK Ideograph */
  ["\u9D73", "\u9D73"], /* CJK Ideograph */
  ["\u9D74", "\u9D74"], /* CJK Ideograph */
  ["\u9D75", "\u9D75"], /* CJK Ideograph */
  ["\u9D76", "\u9D76"], /* CJK Ideograph */
  ["\u9D77", "\u9D77"], /* CJK Ideograph */
  ["\u9D78", "\u9D78"], /* CJK Ideograph */
  ["\u9D79", "\u9D79"], /* CJK Ideograph */
  ["\u9D7A", "\u9D7A"], /* CJK Ideograph */
  ["\u9D7B", "\u9D7B"], /* CJK Ideograph */
  ["\u9D7C", "\u9D7C"], /* CJK Ideograph */
  ["\u9D7D", "\u9D7D"], /* CJK Ideograph */
  ["\u9D7E", "\u9D7E"], /* CJK Ideograph */
  ["\u9D7F", "\u9D7F"], /* CJK Ideograph */
  ["\u9D80", "\u9D80"], /* CJK Ideograph */
  ["\u9D81", "\u9D81"], /* CJK Ideograph */
  ["\u9D82", "\u9D82"], /* CJK Ideograph */
  ["\u9D83", "\u9D83"], /* CJK Ideograph */
  ["\u9D84", "\u9D84"], /* CJK Ideograph */
  ["\u9D85", "\u9D85"], /* CJK Ideograph */
  ["\u9D86", "\u9D86"], /* CJK Ideograph */
  ["\u9D87", "\u9D87"], /* CJK Ideograph */
  ["\u9D88", "\u9D88"], /* CJK Ideograph */
  ["\u9D89", "\u9D89"], /* CJK Ideograph */
  ["\u9D8A", "\u9D8A"], /* CJK Ideograph */
  ["\u9D8B", "\u9D8B"], /* CJK Ideograph */
  ["\u9D8C", "\u9D8C"], /* CJK Ideograph */
  ["\u9D8D", "\u9D8D"], /* CJK Ideograph */
  ["\u9D8E", "\u9D8E"], /* CJK Ideograph */
  ["\u9D8F", "\u9D8F"], /* CJK Ideograph */
  ["\u9D90", "\u9D90"], /* CJK Ideograph */
  ["\u9D91", "\u9D91"], /* CJK Ideograph */
  ["\u9D92", "\u9D92"], /* CJK Ideograph */
  ["\u9D93", "\u9D93"], /* CJK Ideograph */
  ["\u9D94", "\u9D94"], /* CJK Ideograph */
  ["\u9D95", "\u9D95"], /* CJK Ideograph */
  ["\u9D96", "\u9D96"], /* CJK Ideograph */
  ["\u9D97", "\u9D97"], /* CJK Ideograph */
  ["\u9D98", "\u9D98"], /* CJK Ideograph */
  ["\u9D99", "\u9D99"], /* CJK Ideograph */
  ["\u9D9A", "\u9D9A"], /* CJK Ideograph */
  ["\u9D9B", "\u9D9B"], /* CJK Ideograph */
  ["\u9D9C", "\u9D9C"], /* CJK Ideograph */
  ["\u9D9D", "\u9D9D"], /* CJK Ideograph */
  ["\u9D9E", "\u9D9E"], /* CJK Ideograph */
  ["\u9D9F", "\u9D9F"], /* CJK Ideograph */
  ["\u9DA0", "\u9DA0"], /* CJK Ideograph */
  ["\u9DA1", "\u9DA1"], /* CJK Ideograph */
  ["\u9DA2", "\u9DA2"], /* CJK Ideograph */
  ["\u9DA3", "\u9DA3"], /* CJK Ideograph */
  ["\u9DA4", "\u9DA4"], /* CJK Ideograph */
  ["\u9DA5", "\u9DA5"], /* CJK Ideograph */
  ["\u9DA6", "\u9DA6"], /* CJK Ideograph */
  ["\u9DA7", "\u9DA7"], /* CJK Ideograph */
  ["\u9DA8", "\u9DA8"], /* CJK Ideograph */
  ["\u9DA9", "\u9DA9"], /* CJK Ideograph */
  ["\u9DAA", "\u9DAA"], /* CJK Ideograph */
  ["\u9DAB", "\u9DAB"], /* CJK Ideograph */
  ["\u9DAC", "\u9DAC"], /* CJK Ideograph */
  ["\u9DAD", "\u9DAD"], /* CJK Ideograph */
  ["\u9DAE", "\u9DAE"], /* CJK Ideograph */
  ["\u9DAF", "\u9DAF"], /* CJK Ideograph */
  ["\u9DB0", "\u9DB0"], /* CJK Ideograph */
  ["\u9DB1", "\u9DB1"], /* CJK Ideograph */
  ["\u9DB2", "\u9DB2"], /* CJK Ideograph */
  ["\u9DB3", "\u9DB3"], /* CJK Ideograph */
  ["\u9DB4", "\u9DB4"], /* CJK Ideograph */
  ["\u9DB5", "\u9DB5"], /* CJK Ideograph */
  ["\u9DB6", "\u9DB6"], /* CJK Ideograph */
  ["\u9DB7", "\u9DB7"], /* CJK Ideograph */
  ["\u9DB8", "\u9DB8"], /* CJK Ideograph */
  ["\u9DB9", "\u9DB9"], /* CJK Ideograph */
  ["\u9DBA", "\u9DBA"], /* CJK Ideograph */
  ["\u9DBB", "\u9DBB"], /* CJK Ideograph */
  ["\u9DBC", "\u9DBC"], /* CJK Ideograph */
  ["\u9DBD", "\u9DBD"], /* CJK Ideograph */
  ["\u9DBE", "\u9DBE"], /* CJK Ideograph */
  ["\u9DBF", "\u9DBF"], /* CJK Ideograph */
  ["\u9DC0", "\u9DC0"], /* CJK Ideograph */
  ["\u9DC1", "\u9DC1"], /* CJK Ideograph */
  ["\u9DC2", "\u9DC2"], /* CJK Ideograph */
  ["\u9DC3", "\u9DC3"], /* CJK Ideograph */
  ["\u9DC4", "\u9DC4"], /* CJK Ideograph */
  ["\u9DC5", "\u9DC5"], /* CJK Ideograph */
  ["\u9DC6", "\u9DC6"], /* CJK Ideograph */
  ["\u9DC7", "\u9DC7"], /* CJK Ideograph */
  ["\u9DC8", "\u9DC8"], /* CJK Ideograph */
  ["\u9DC9", "\u9DC9"], /* CJK Ideograph */
  ["\u9DCA", "\u9DCA"], /* CJK Ideograph */
  ["\u9DCB", "\u9DCB"], /* CJK Ideograph */
  ["\u9DCC", "\u9DCC"], /* CJK Ideograph */
  ["\u9DCD", "\u9DCD"], /* CJK Ideograph */
  ["\u9DCE", "\u9DCE"], /* CJK Ideograph */
  ["\u9DCF", "\u9DCF"], /* CJK Ideograph */
  ["\u9DD0", "\u9DD0"], /* CJK Ideograph */
  ["\u9DD1", "\u9DD1"], /* CJK Ideograph */
  ["\u9DD2", "\u9DD2"], /* CJK Ideograph */
  ["\u9DD3", "\u9DD3"], /* CJK Ideograph */
  ["\u9DD4", "\u9DD4"], /* CJK Ideograph */
  ["\u9DD5", "\u9DD5"], /* CJK Ideograph */
  ["\u9DD6", "\u9DD6"], /* CJK Ideograph */
  ["\u9DD7", "\u9DD7"], /* CJK Ideograph */
  ["\u9DD8", "\u9DD8"], /* CJK Ideograph */
  ["\u9DD9", "\u9DD9"], /* CJK Ideograph */
  ["\u9DDA", "\u9DDA"], /* CJK Ideograph */
  ["\u9DDB", "\u9DDB"], /* CJK Ideograph */
  ["\u9DDC", "\u9DDC"], /* CJK Ideograph */
  ["\u9DDD", "\u9DDD"], /* CJK Ideograph */
  ["\u9DDE", "\u9DDE"], /* CJK Ideograph */
  ["\u9DDF", "\u9DDF"], /* CJK Ideograph */
  ["\u9DE0", "\u9DE0"], /* CJK Ideograph */
  ["\u9DE1", "\u9DE1"], /* CJK Ideograph */
  ["\u9DE2", "\u9DE2"], /* CJK Ideograph */
  ["\u9DE3", "\u9DE3"], /* CJK Ideograph */
  ["\u9DE4", "\u9DE4"], /* CJK Ideograph */
  ["\u9DE5", "\u9DE5"], /* CJK Ideograph */
  ["\u9DE6", "\u9DE6"], /* CJK Ideograph */
  ["\u9DE7", "\u9DE7"], /* CJK Ideograph */
  ["\u9DE8", "\u9DE8"], /* CJK Ideograph */
  ["\u9DE9", "\u9DE9"], /* CJK Ideograph */
  ["\u9DEA", "\u9DEA"], /* CJK Ideograph */
  ["\u9DEB", "\u9DEB"], /* CJK Ideograph */
  ["\u9DEC", "\u9DEC"], /* CJK Ideograph */
  ["\u9DED", "\u9DED"], /* CJK Ideograph */
  ["\u9DEE", "\u9DEE"], /* CJK Ideograph */
  ["\u9DEF", "\u9DEF"], /* CJK Ideograph */
  ["\u9DF0", "\u9DF0"], /* CJK Ideograph */
  ["\u9DF1", "\u9DF1"], /* CJK Ideograph */
  ["\u9DF2", "\u9DF2"], /* CJK Ideograph */
  ["\u9DF3", "\u9DF3"], /* CJK Ideograph */
  ["\u9DF4", "\u9DF4"], /* CJK Ideograph */
  ["\u9DF5", "\u9DF5"], /* CJK Ideograph */
  ["\u9DF6", "\u9DF6"], /* CJK Ideograph */
  ["\u9DF7", "\u9DF7"], /* CJK Ideograph */
  ["\u9DF8", "\u9DF8"], /* CJK Ideograph */
  ["\u9DF9", "\u9DF9"], /* CJK Ideograph */
  ["\u9DFA", "\u9DFA"], /* CJK Ideograph */
  ["\u9DFB", "\u9DFB"], /* CJK Ideograph */
  ["\u9DFC", "\u9DFC"], /* CJK Ideograph */
  ["\u9DFD", "\u9DFD"], /* CJK Ideograph */
  ["\u9DFE", "\u9DFE"], /* CJK Ideograph */
  ["\u9DFF", "\u9DFF"], /* CJK Ideograph */
  ["\u9E00", "\u9E00"], /* CJK Ideograph */
  ["\u9E01", "\u9E01"], /* CJK Ideograph */
  ["\u9E02", "\u9E02"], /* CJK Ideograph */
  ["\u9E03", "\u9E03"], /* CJK Ideograph */
  ["\u9E04", "\u9E04"], /* CJK Ideograph */
  ["\u9E05", "\u9E05"], /* CJK Ideograph */
  ["\u9E06", "\u9E06"], /* CJK Ideograph */
  ["\u9E07", "\u9E07"], /* CJK Ideograph */
  ["\u9E08", "\u9E08"], /* CJK Ideograph */
  ["\u9E09", "\u9E09"], /* CJK Ideograph */
  ["\u9E0A", "\u9E0A"], /* CJK Ideograph */
  ["\u9E0B", "\u9E0B"], /* CJK Ideograph */
  ["\u9E0C", "\u9E0C"], /* CJK Ideograph */
  ["\u9E0D", "\u9E0D"], /* CJK Ideograph */
  ["\u9E0E", "\u9E0E"], /* CJK Ideograph */
  ["\u9E0F", "\u9E0F"], /* CJK Ideograph */
  ["\u9E10", "\u9E10"], /* CJK Ideograph */
  ["\u9E11", "\u9E11"], /* CJK Ideograph */
  ["\u9E12", "\u9E12"], /* CJK Ideograph */
  ["\u9E13", "\u9E13"], /* CJK Ideograph */
  ["\u9E14", "\u9E14"], /* CJK Ideograph */
  ["\u9E15", "\u9E15"], /* CJK Ideograph */
  ["\u9E16", "\u9E16"], /* CJK Ideograph */
  ["\u9E17", "\u9E17"], /* CJK Ideograph */
  ["\u9E18", "\u9E18"], /* CJK Ideograph */
  ["\u9E19", "\u9E19"], /* CJK Ideograph */
  ["\u9E1A", "\u9E1A"], /* CJK Ideograph */
  ["\u9E1B", "\u9E1B"], /* CJK Ideograph */
  ["\u9E1C", "\u9E1C"], /* CJK Ideograph */
  ["\u9E1D", "\u9E1D"], /* CJK Ideograph */
  ["\u9E1E", "\u9E1E"], /* CJK Ideograph */
  ["\u9E1F", "\u9E1F"], /* CJK Ideograph */
  ["\u9E20", "\u9E20"], /* CJK Ideograph */
  ["\u9E21", "\u9E21"], /* CJK Ideograph */
  ["\u9E22", "\u9E22"], /* CJK Ideograph */
  ["\u9E23", "\u9E23"], /* CJK Ideograph */
  ["\u9E24", "\u9E24"], /* CJK Ideograph */
  ["\u9E25", "\u9E25"], /* CJK Ideograph */
  ["\u9E26", "\u9E26"], /* CJK Ideograph */
  ["\u9E27", "\u9E27"], /* CJK Ideograph */
  ["\u9E28", "\u9E28"], /* CJK Ideograph */
  ["\u9E29", "\u9E29"], /* CJK Ideograph */
  ["\u9E2A", "\u9E2A"], /* CJK Ideograph */
  ["\u9E2B", "\u9E2B"], /* CJK Ideograph */
  ["\u9E2C", "\u9E2C"], /* CJK Ideograph */
  ["\u9E2D", "\u9E2D"], /* CJK Ideograph */
  ["\u9E2E", "\u9E2E"], /* CJK Ideograph */
  ["\u9E2F", "\u9E2F"], /* CJK Ideograph */
  ["\u9E30", "\u9E30"], /* CJK Ideograph */
  ["\u9E31", "\u9E31"], /* CJK Ideograph */
  ["\u9E32", "\u9E32"], /* CJK Ideograph */
  ["\u9E33", "\u9E33"], /* CJK Ideograph */
  ["\u9E34", "\u9E34"], /* CJK Ideograph */
  ["\u9E35", "\u9E35"], /* CJK Ideograph */
  ["\u9E36", "\u9E36"], /* CJK Ideograph */
  ["\u9E37", "\u9E37"], /* CJK Ideograph */
  ["\u9E38", "\u9E38"], /* CJK Ideograph */
  ["\u9E39", "\u9E39"], /* CJK Ideograph */
  ["\u9E3A", "\u9E3A"], /* CJK Ideograph */
  ["\u9E3B", "\u9E3B"], /* CJK Ideograph */
  ["\u9E3C", "\u9E3C"], /* CJK Ideograph */
  ["\u9E3D", "\u9E3D"], /* CJK Ideograph */
  ["\u9E3E", "\u9E3E"], /* CJK Ideograph */
  ["\u9E3F", "\u9E3F"], /* CJK Ideograph */
  ["\u9E40", "\u9E40"], /* CJK Ideograph */
  ["\u9E41", "\u9E41"], /* CJK Ideograph */
  ["\u9E42", "\u9E42"], /* CJK Ideograph */
  ["\u9E43", "\u9E43"], /* CJK Ideograph */
  ["\u9E44", "\u9E44"], /* CJK Ideograph */
  ["\u9E45", "\u9E45"], /* CJK Ideograph */
  ["\u9E46", "\u9E46"], /* CJK Ideograph */
  ["\u9E47", "\u9E47"], /* CJK Ideograph */
  ["\u9E48", "\u9E48"], /* CJK Ideograph */
  ["\u9E49", "\u9E49"], /* CJK Ideograph */
  ["\u9E4A", "\u9E4A"], /* CJK Ideograph */
  ["\u9E4B", "\u9E4B"], /* CJK Ideograph */
  ["\u9E4C", "\u9E4C"], /* CJK Ideograph */
  ["\u9E4D", "\u9E4D"], /* CJK Ideograph */
  ["\u9E4E", "\u9E4E"], /* CJK Ideograph */
  ["\u9E4F", "\u9E4F"], /* CJK Ideograph */
  ["\u9E50", "\u9E50"], /* CJK Ideograph */
  ["\u9E51", "\u9E51"], /* CJK Ideograph */
  ["\u9E52", "\u9E52"], /* CJK Ideograph */
  ["\u9E53", "\u9E53"], /* CJK Ideograph */
  ["\u9E54", "\u9E54"], /* CJK Ideograph */
  ["\u9E55", "\u9E55"], /* CJK Ideograph */
  ["\u9E56", "\u9E56"], /* CJK Ideograph */
  ["\u9E57", "\u9E57"], /* CJK Ideograph */
  ["\u9E58", "\u9E58"], /* CJK Ideograph */
  ["\u9E59", "\u9E59"], /* CJK Ideograph */
  ["\u9E5A", "\u9E5A"], /* CJK Ideograph */
  ["\u9E5B", "\u9E5B"], /* CJK Ideograph */
  ["\u9E5C", "\u9E5C"], /* CJK Ideograph */
  ["\u9E5D", "\u9E5D"], /* CJK Ideograph */
  ["\u9E5E", "\u9E5E"], /* CJK Ideograph */
  ["\u9E5F", "\u9E5F"], /* CJK Ideograph */
  ["\u9E60", "\u9E60"], /* CJK Ideograph */
  ["\u9E61", "\u9E61"], /* CJK Ideograph */
  ["\u9E62", "\u9E62"], /* CJK Ideograph */
  ["\u9E63", "\u9E63"], /* CJK Ideograph */
  ["\u9E64", "\u9E64"], /* CJK Ideograph */
  ["\u9E65", "\u9E65"], /* CJK Ideograph */
  ["\u9E66", "\u9E66"], /* CJK Ideograph */
  ["\u9E67", "\u9E67"], /* CJK Ideograph */
  ["\u9E68", "\u9E68"], /* CJK Ideograph */
  ["\u9E69", "\u9E69"], /* CJK Ideograph */
  ["\u9E6A", "\u9E6A"], /* CJK Ideograph */
  ["\u9E6B", "\u9E6B"], /* CJK Ideograph */
  ["\u9E6C", "\u9E6C"], /* CJK Ideograph */
  ["\u9E6D", "\u9E6D"], /* CJK Ideograph */
  ["\u9E6E", "\u9E6E"], /* CJK Ideograph */
  ["\u9E6F", "\u9E6F"], /* CJK Ideograph */
  ["\u9E70", "\u9E70"], /* CJK Ideograph */
  ["\u9E71", "\u9E71"], /* CJK Ideograph */
  ["\u9E72", "\u9E72"], /* CJK Ideograph */
  ["\u9E73", "\u9E73"], /* CJK Ideograph */
  ["\u9E74", "\u9E74"], /* CJK Ideograph */
  ["\u9E75", "\u9E75"], /* CJK Ideograph */
  ["\u9E76", "\u9E76"], /* CJK Ideograph */
  ["\u9E77", "\u9E77"], /* CJK Ideograph */
  ["\u9E78", "\u9E78"], /* CJK Ideograph */
  ["\u9E79", "\u9E79"], /* CJK Ideograph */
  ["\u9E7A", "\u9E7A"], /* CJK Ideograph */
  ["\u9E7B", "\u9E7B"], /* CJK Ideograph */
  ["\u9E7C", "\u9E7C"], /* CJK Ideograph */
  ["\u9E7D", "\u9E7D"], /* CJK Ideograph */
  ["\u9E7E", "\u9E7E"], /* CJK Ideograph */
  ["\u9E7F", "\u9E7F"], /* CJK Ideograph */
  ["\u9E80", "\u9E80"], /* CJK Ideograph */
  ["\u9E81", "\u9E81"], /* CJK Ideograph */
  ["\u9E82", "\u9E82"], /* CJK Ideograph */
  ["\u9E83", "\u9E83"], /* CJK Ideograph */
  ["\u9E84", "\u9E84"], /* CJK Ideograph */
  ["\u9E85", "\u9E85"], /* CJK Ideograph */
  ["\u9E86", "\u9E86"], /* CJK Ideograph */
  ["\u9E87", "\u9E87"], /* CJK Ideograph */
  ["\u9E88", "\u9E88"], /* CJK Ideograph */
  ["\u9E89", "\u9E89"], /* CJK Ideograph */
  ["\u9E8A", "\u9E8A"], /* CJK Ideograph */
  ["\u9E8B", "\u9E8B"], /* CJK Ideograph */
  ["\u9E8C", "\u9E8C"], /* CJK Ideograph */
  ["\u9E8D", "\u9E8D"], /* CJK Ideograph */
  ["\u9E8E", "\u9E8E"], /* CJK Ideograph */
  ["\u9E8F", "\u9E8F"], /* CJK Ideograph */
  ["\u9E90", "\u9E90"], /* CJK Ideograph */
  ["\u9E91", "\u9E91"], /* CJK Ideograph */
  ["\u9E92", "\u9E92"], /* CJK Ideograph */
  ["\u9E93", "\u9E93"], /* CJK Ideograph */
  ["\u9E94", "\u9E94"], /* CJK Ideograph */
  ["\u9E95", "\u9E95"], /* CJK Ideograph */
  ["\u9E96", "\u9E96"], /* CJK Ideograph */
  ["\u9E97", "\u9E97"], /* CJK Ideograph */
  ["\u9E98", "\u9E98"], /* CJK Ideograph */
  ["\u9E99", "\u9E99"], /* CJK Ideograph */
  ["\u9E9A", "\u9E9A"], /* CJK Ideograph */
  ["\u9E9B", "\u9E9B"], /* CJK Ideograph */
  ["\u9E9C", "\u9E9C"], /* CJK Ideograph */
  ["\u9E9D", "\u9E9D"], /* CJK Ideograph */
  ["\u9E9E", "\u9E9E"], /* CJK Ideograph */
  ["\u9E9F", "\u9E9F"], /* CJK Ideograph */
  ["\u9EA0", "\u9EA0"], /* CJK Ideograph */
  ["\u9EA1", "\u9EA1"], /* CJK Ideograph */
  ["\u9EA2", "\u9EA2"], /* CJK Ideograph */
  ["\u9EA3", "\u9EA3"], /* CJK Ideograph */
  ["\u9EA4", "\u9EA4"], /* CJK Ideograph */
  ["\u9EA5", "\u9EA5"], /* CJK Ideograph */
  ["\u9EA6", "\u9EA6"], /* CJK Ideograph */
  ["\u9EA7", "\u9EA7"], /* CJK Ideograph */
  ["\u9EA8", "\u9EA8"], /* CJK Ideograph */
  ["\u9EA9", "\u9EA9"], /* CJK Ideograph */
  ["\u9EAA", "\u9EAA"], /* CJK Ideograph */
  ["\u9EAB", "\u9EAB"], /* CJK Ideograph */
  ["\u9EAC", "\u9EAC"], /* CJK Ideograph */
  ["\u9EAD", "\u9EAD"], /* CJK Ideograph */
  ["\u9EAE", "\u9EAE"], /* CJK Ideograph */
  ["\u9EAF", "\u9EAF"], /* CJK Ideograph */
  ["\u9EB0", "\u9EB0"], /* CJK Ideograph */
  ["\u9EB1", "\u9EB1"], /* CJK Ideograph */
  ["\u9EB2", "\u9EB2"], /* CJK Ideograph */
  ["\u9EB3", "\u9EB3"], /* CJK Ideograph */
  ["\u9EB4", "\u9EB4"], /* CJK Ideograph */
  ["\u9EB5", "\u9EB5"], /* CJK Ideograph */
  ["\u9EB6", "\u9EB6"], /* CJK Ideograph */
  ["\u9EB7", "\u9EB7"], /* CJK Ideograph */
  ["\u9EB8", "\u9EB8"], /* CJK Ideograph */
  ["\u9EB9", "\u9EB9"], /* CJK Ideograph */
  ["\u9EBA", "\u9EBA"], /* CJK Ideograph */
  ["\u9EBB", "\u9EBB"], /* CJK Ideograph */
  ["\u9EBC", "\u9EBC"], /* CJK Ideograph */
  ["\u9EBD", "\u9EBD"], /* CJK Ideograph */
  ["\u9EBE", "\u9EBE"], /* CJK Ideograph */
  ["\u9EBF", "\u9EBF"], /* CJK Ideograph */
  ["\u9EC0", "\u9EC0"], /* CJK Ideograph */
  ["\u9EC1", "\u9EC1"], /* CJK Ideograph */
  ["\u9EC2", "\u9EC2"], /* CJK Ideograph */
  ["\u9EC3", "\u9EC3"], /* CJK Ideograph */
  ["\u9EC4", "\u9EC4"], /* CJK Ideograph */
  ["\u9EC5", "\u9EC5"], /* CJK Ideograph */
  ["\u9EC6", "\u9EC6"], /* CJK Ideograph */
  ["\u9EC7", "\u9EC7"], /* CJK Ideograph */
  ["\u9EC8", "\u9EC8"], /* CJK Ideograph */
  ["\u9EC9", "\u9EC9"], /* CJK Ideograph */
  ["\u9ECA", "\u9ECA"], /* CJK Ideograph */
  ["\u9ECB", "\u9ECB"], /* CJK Ideograph */
  ["\u9ECC", "\u9ECC"], /* CJK Ideograph */
  ["\u9ECD", "\u9ECD"], /* CJK Ideograph */
  ["\u9ECE", "\u9ECE"], /* CJK Ideograph */
  ["\u9ECF", "\u9ECF"], /* CJK Ideograph */
  ["\u9ED0", "\u9ED0"], /* CJK Ideograph */
  ["\u9ED1", "\u9ED1"], /* CJK Ideograph */
  ["\u9ED2", "\u9ED2"], /* CJK Ideograph */
  ["\u9ED3", "\u9ED3"], /* CJK Ideograph */
  ["\u9ED4", "\u9ED4"], /* CJK Ideograph */
  ["\u9ED5", "\u9ED5"], /* CJK Ideograph */
  ["\u9ED6", "\u9ED6"], /* CJK Ideograph */
  ["\u9ED7", "\u9ED7"], /* CJK Ideograph */
  ["\u9ED8", "\u9ED8"], /* CJK Ideograph */
  ["\u9ED9", "\u9ED9"], /* CJK Ideograph */
  ["\u9EDA", "\u9EDA"], /* CJK Ideograph */
  ["\u9EDB", "\u9EDB"], /* CJK Ideograph */
  ["\u9EDC", "\u9EDC"], /* CJK Ideograph */
  ["\u9EDD", "\u9EDD"], /* CJK Ideograph */
  ["\u9EDE", "\u9EDE"], /* CJK Ideograph */
  ["\u9EDF", "\u9EDF"], /* CJK Ideograph */
  ["\u9EE0", "\u9EE0"], /* CJK Ideograph */
  ["\u9EE1", "\u9EE1"], /* CJK Ideograph */
  ["\u9EE2", "\u9EE2"], /* CJK Ideograph */
  ["\u9EE3", "\u9EE3"], /* CJK Ideograph */
  ["\u9EE4", "\u9EE4"], /* CJK Ideograph */
  ["\u9EE5", "\u9EE5"], /* CJK Ideograph */
  ["\u9EE6", "\u9EE6"], /* CJK Ideograph */
  ["\u9EE7", "\u9EE7"], /* CJK Ideograph */
  ["\u9EE8", "\u9EE8"], /* CJK Ideograph */
  ["\u9EE9", "\u9EE9"], /* CJK Ideograph */
  ["\u9EEA", "\u9EEA"], /* CJK Ideograph */
  ["\u9EEB", "\u9EEB"], /* CJK Ideograph */
  ["\u9EEC", "\u9EEC"], /* CJK Ideograph */
  ["\u9EED", "\u9EED"], /* CJK Ideograph */
  ["\u9EEE", "\u9EEE"], /* CJK Ideograph */
  ["\u9EEF", "\u9EEF"], /* CJK Ideograph */
  ["\u9EF0", "\u9EF0"], /* CJK Ideograph */
  ["\u9EF1", "\u9EF1"], /* CJK Ideograph */
  ["\u9EF2", "\u9EF2"], /* CJK Ideograph */
  ["\u9EF3", "\u9EF3"], /* CJK Ideograph */
  ["\u9EF4", "\u9EF4"], /* CJK Ideograph */
  ["\u9EF5", "\u9EF5"], /* CJK Ideograph */
  ["\u9EF6", "\u9EF6"], /* CJK Ideograph */
  ["\u9EF7", "\u9EF7"], /* CJK Ideograph */
  ["\u9EF8", "\u9EF8"], /* CJK Ideograph */
  ["\u9EF9", "\u9EF9"], /* CJK Ideograph */
  ["\u9EFA", "\u9EFA"], /* CJK Ideograph */
  ["\u9EFB", "\u9EFB"], /* CJK Ideograph */
  ["\u9EFC", "\u9EFC"], /* CJK Ideograph */
  ["\u9EFD", "\u9EFD"], /* CJK Ideograph */
  ["\u9EFE", "\u9EFE"], /* CJK Ideograph */
  ["\u9EFF", "\u9EFF"], /* CJK Ideograph */
  ["\u9F00", "\u9F00"], /* CJK Ideograph */
  ["\u9F01", "\u9F01"], /* CJK Ideograph */
  ["\u9F02", "\u9F02"], /* CJK Ideograph */
  ["\u9F03", "\u9F03"], /* CJK Ideograph */
  ["\u9F04", "\u9F04"], /* CJK Ideograph */
  ["\u9F05", "\u9F05"], /* CJK Ideograph */
  ["\u9F06", "\u9F06"], /* CJK Ideograph */
  ["\u9F07", "\u9F07"], /* CJK Ideograph */
  ["\u9F08", "\u9F08"], /* CJK Ideograph */
  ["\u9F09", "\u9F09"], /* CJK Ideograph */
  ["\u9F0A", "\u9F0A"], /* CJK Ideograph */
  ["\u9F0B", "\u9F0B"], /* CJK Ideograph */
  ["\u9F0C", "\u9F0C"], /* CJK Ideograph */
  ["\u9F0D", "\u9F0D"], /* CJK Ideograph */
  ["\u9F0E", "\u9F0E"], /* CJK Ideograph */
  ["\u9F0F", "\u9F0F"], /* CJK Ideograph */
  ["\u9F10", "\u9F10"], /* CJK Ideograph */
  ["\u9F11", "\u9F11"], /* CJK Ideograph */
  ["\u9F12", "\u9F12"], /* CJK Ideograph */
  ["\u9F13", "\u9F13"], /* CJK Ideograph */
  ["\u9F14", "\u9F14"], /* CJK Ideograph */
  ["\u9F15", "\u9F15"], /* CJK Ideograph */
  ["\u9F16", "\u9F16"], /* CJK Ideograph */
  ["\u9F17", "\u9F17"], /* CJK Ideograph */
  ["\u9F18", "\u9F18"], /* CJK Ideograph */
  ["\u9F19", "\u9F19"], /* CJK Ideograph */
  ["\u9F1A", "\u9F1A"], /* CJK Ideograph */
  ["\u9F1B", "\u9F1B"], /* CJK Ideograph */
  ["\u9F1C", "\u9F1C"], /* CJK Ideograph */
  ["\u9F1D", "\u9F1D"], /* CJK Ideograph */
  ["\u9F1E", "\u9F1E"], /* CJK Ideograph */
  ["\u9F1F", "\u9F1F"], /* CJK Ideograph */
  ["\u9F20", "\u9F20"], /* CJK Ideograph */
  ["\u9F21", "\u9F21"], /* CJK Ideograph */
  ["\u9F22", "\u9F22"], /* CJK Ideograph */
  ["\u9F23", "\u9F23"], /* CJK Ideograph */
  ["\u9F24", "\u9F24"], /* CJK Ideograph */
  ["\u9F25", "\u9F25"], /* CJK Ideograph */
  ["\u9F26", "\u9F26"], /* CJK Ideograph */
  ["\u9F27", "\u9F27"], /* CJK Ideograph */
  ["\u9F28", "\u9F28"], /* CJK Ideograph */
  ["\u9F29", "\u9F29"], /* CJK Ideograph */
  ["\u9F2A", "\u9F2A"], /* CJK Ideograph */
  ["\u9F2B", "\u9F2B"], /* CJK Ideograph */
  ["\u9F2C", "\u9F2C"], /* CJK Ideograph */
  ["\u9F2D", "\u9F2D"], /* CJK Ideograph */
  ["\u9F2E", "\u9F2E"], /* CJK Ideograph */
  ["\u9F2F", "\u9F2F"], /* CJK Ideograph */
  ["\u9F30", "\u9F30"], /* CJK Ideograph */
  ["\u9F31", "\u9F31"], /* CJK Ideograph */
  ["\u9F32", "\u9F32"], /* CJK Ideograph */
  ["\u9F33", "\u9F33"], /* CJK Ideograph */
  ["\u9F34", "\u9F34"], /* CJK Ideograph */
  ["\u9F35", "\u9F35"], /* CJK Ideograph */
  ["\u9F36", "\u9F36"], /* CJK Ideograph */
  ["\u9F37", "\u9F37"], /* CJK Ideograph */
  ["\u9F38", "\u9F38"], /* CJK Ideograph */
  ["\u9F39", "\u9F39"], /* CJK Ideograph */
  ["\u9F3A", "\u9F3A"], /* CJK Ideograph */
  ["\u9F3B", "\u9F3B"], /* CJK Ideograph */
  ["\u9F3C", "\u9F3C"], /* CJK Ideograph */
  ["\u9F3D", "\u9F3D"], /* CJK Ideograph */
  ["\u9F3E", "\u9F3E"], /* CJK Ideograph */
  ["\u9F3F", "\u9F3F"], /* CJK Ideograph */
  ["\u9F40", "\u9F40"], /* CJK Ideograph */
  ["\u9F41", "\u9F41"], /* CJK Ideograph */
  ["\u9F42", "\u9F42"], /* CJK Ideograph */
  ["\u9F43", "\u9F43"], /* CJK Ideograph */
  ["\u9F44", "\u9F44"], /* CJK Ideograph */
  ["\u9F45", "\u9F45"], /* CJK Ideograph */
  ["\u9F46", "\u9F46"], /* CJK Ideograph */
  ["\u9F47", "\u9F47"], /* CJK Ideograph */
  ["\u9F48", "\u9F48"], /* CJK Ideograph */
  ["\u9F49", "\u9F49"], /* CJK Ideograph */
  ["\u9F4A", "\u9F4A"], /* CJK Ideograph */
  ["\u9F4B", "\u9F4B"], /* CJK Ideograph */
  ["\u9F4C", "\u9F4C"], /* CJK Ideograph */
  ["\u9F4D", "\u9F4D"], /* CJK Ideograph */
  ["\u9F4E", "\u9F4E"], /* CJK Ideograph */
  ["\u9F4F", "\u9F4F"], /* CJK Ideograph */
  ["\u9F50", "\u9F50"], /* CJK Ideograph */
  ["\u9F51", "\u9F51"], /* CJK Ideograph */
  ["\u9F52", "\u9F52"], /* CJK Ideograph */
  ["\u9F53", "\u9F53"], /* CJK Ideograph */
  ["\u9F54", "\u9F54"], /* CJK Ideograph */
  ["\u9F55", "\u9F55"], /* CJK Ideograph */
  ["\u9F56", "\u9F56"], /* CJK Ideograph */
  ["\u9F57", "\u9F57"], /* CJK Ideograph */
  ["\u9F58", "\u9F58"], /* CJK Ideograph */
  ["\u9F59", "\u9F59"], /* CJK Ideograph */
  ["\u9F5A", "\u9F5A"], /* CJK Ideograph */
  ["\u9F5B", "\u9F5B"], /* CJK Ideograph */
  ["\u9F5C", "\u9F5C"], /* CJK Ideograph */
  ["\u9F5D", "\u9F5D"], /* CJK Ideograph */
  ["\u9F5E", "\u9F5E"], /* CJK Ideograph */
  ["\u9F5F", "\u9F5F"], /* CJK Ideograph */
  ["\u9F60", "\u9F60"], /* CJK Ideograph */
  ["\u9F61", "\u9F61"], /* CJK Ideograph */
  ["\u9F62", "\u9F62"], /* CJK Ideograph */
  ["\u9F63", "\u9F63"], /* CJK Ideograph */
  ["\u9F64", "\u9F64"], /* CJK Ideograph */
  ["\u9F65", "\u9F65"], /* CJK Ideograph */
  ["\u9F66", "\u9F66"], /* CJK Ideograph */
  ["\u9F67", "\u9F67"], /* CJK Ideograph */
  ["\u9F68", "\u9F68"], /* CJK Ideograph */
  ["\u9F69", "\u9F69"], /* CJK Ideograph */
  ["\u9F6A", "\u9F6A"], /* CJK Ideograph */
  ["\u9F6B", "\u9F6B"], /* CJK Ideograph */
  ["\u9F6C", "\u9F6C"], /* CJK Ideograph */
  ["\u9F6D", "\u9F6D"], /* CJK Ideograph */
  ["\u9F6E", "\u9F6E"], /* CJK Ideograph */
  ["\u9F6F", "\u9F6F"], /* CJK Ideograph */
  ["\u9F70", "\u9F70"], /* CJK Ideograph */
  ["\u9F71", "\u9F71"], /* CJK Ideograph */
  ["\u9F72", "\u9F72"], /* CJK Ideograph */
  ["\u9F73", "\u9F73"], /* CJK Ideograph */
  ["\u9F74", "\u9F74"], /* CJK Ideograph */
  ["\u9F75", "\u9F75"], /* CJK Ideograph */
  ["\u9F76", "\u9F76"], /* CJK Ideograph */
  ["\u9F77", "\u9F77"], /* CJK Ideograph */
  ["\u9F78", "\u9F78"], /* CJK Ideograph */
  ["\u9F79", "\u9F79"], /* CJK Ideograph */
  ["\u9F7A", "\u9F7A"], /* CJK Ideograph */
  ["\u9F7B", "\u9F7B"], /* CJK Ideograph */
  ["\u9F7C", "\u9F7C"], /* CJK Ideograph */
  ["\u9F7D", "\u9F7D"], /* CJK Ideograph */
  ["\u9F7E", "\u9F7E"], /* CJK Ideograph */
  ["\u9F7F", "\u9F7F"], /* CJK Ideograph */
  ["\u9F80", "\u9F80"], /* CJK Ideograph */
  ["\u9F81", "\u9F81"], /* CJK Ideograph */
  ["\u9F82", "\u9F82"], /* CJK Ideograph */
  ["\u9F83", "\u9F83"], /* CJK Ideograph */
  ["\u9F84", "\u9F84"], /* CJK Ideograph */
  ["\u9F85", "\u9F85"], /* CJK Ideograph */
  ["\u9F86", "\u9F86"], /* CJK Ideograph */
  ["\u9F87", "\u9F87"], /* CJK Ideograph */
  ["\u9F88", "\u9F88"], /* CJK Ideograph */
  ["\u9F89", "\u9F89"], /* CJK Ideograph */
  ["\u9F8A", "\u9F8A"], /* CJK Ideograph */
  ["\u9F8B", "\u9F8B"], /* CJK Ideograph */
  ["\u9F8C", "\u9F8C"], /* CJK Ideograph */
  ["\u9F8D", "\u9F8D"], /* CJK Ideograph */
  ["\u9F8E", "\u9F8E"], /* CJK Ideograph */
  ["\u9F8F", "\u9F8F"], /* CJK Ideograph */
  ["\u9F90", "\u9F90"], /* CJK Ideograph */
  ["\u9F91", "\u9F91"], /* CJK Ideograph */
  ["\u9F92", "\u9F92"], /* CJK Ideograph */
  ["\u9F93", "\u9F93"], /* CJK Ideograph */
  ["\u9F94", "\u9F94"], /* CJK Ideograph */
  ["\u9F95", "\u9F95"], /* CJK Ideograph */
  ["\u9F96", "\u9F96"], /* CJK Ideograph */
  ["\u9F97", "\u9F97"], /* CJK Ideograph */
  ["\u9F98", "\u9F98"], /* CJK Ideograph */
  ["\u9F99", "\u9F99"], /* CJK Ideograph */
  ["\u9F9A", "\u9F9A"], /* CJK Ideograph */
  ["\u9F9B", "\u9F9B"], /* CJK Ideograph */
  ["\u9F9C", "\u9F9C"], /* CJK Ideograph */
  ["\u9F9D", "\u9F9D"], /* CJK Ideograph */
  ["\u9F9E", "\u9F9E"], /* CJK Ideograph */
  ["\u9F9F", "\u9F9F"], /* CJK Ideograph */
  ["\u9FA0", "\u9FA0"], /* CJK Ideograph */
  ["\u9FA1", "\u9FA1"], /* CJK Ideograph */
  ["\u9FA2", "\u9FA2"], /* CJK Ideograph */
  ["\u9FA3", "\u9FA3"], /* CJK Ideograph */
  ["\u9FA4", "\u9FA4"], /* CJK Ideograph */
  ["\u9FA5", "\u9FA5"], /* CJK Ideograph */
  ["\u9FA6", "\u9FA6"], /* CJK Ideograph */
  ["\u9FA7", "\u9FA7"], /* CJK Ideograph */
  ["\u9FA8", "\u9FA8"], /* CJK Ideograph */
  ["\u9FA9", "\u9FA9"], /* CJK Ideograph */
  ["\u9FAA", "\u9FAA"], /* CJK Ideograph */
  ["\u9FAB", "\u9FAB"], /* CJK Ideograph */
  ["\u9FAC", "\u9FAC"], /* CJK Ideograph */
  ["\u9FAD", "\u9FAD"], /* CJK Ideograph */
  ["\u9FAE", "\u9FAE"], /* CJK Ideograph */
  ["\u9FAF", "\u9FAF"], /* CJK Ideograph */
  ["\u9FB0", "\u9FB0"], /* CJK Ideograph */
  ["\u9FB1", "\u9FB1"], /* CJK Ideograph */
  ["\u9FB2", "\u9FB2"], /* CJK Ideograph */
  ["\u9FB3", "\u9FB3"], /* CJK Ideograph */
  ["\u9FB4", "\u9FB4"], /* CJK Ideograph */
  ["\u9FB5", "\u9FB5"], /* CJK Ideograph */
  ["\u9FB6", "\u9FB6"], /* CJK Ideograph */
  ["\u9FB7", "\u9FB7"], /* CJK Ideograph */
  ["\u9FB8", "\u9FB8"], /* CJK Ideograph */
  ["\u9FB9", "\u9FB9"], /* CJK Ideograph */
  ["\u9FBA", "\u9FBA"], /* CJK Ideograph */
  ["\u9FBB", "\u9FBB"], /* CJK Ideograph */
  ["\u9FBC", "\u9FBC"], /* CJK Ideograph */
  ["\u9FBD", "\u9FBD"], /* CJK Ideograph */
  ["\u9FBE", "\u9FBE"], /* CJK Ideograph */
  ["\u9FBF", "\u9FBF"], /* CJK Ideograph */
  ["\u9FC0", "\u9FC0"], /* CJK Ideograph */
  ["\u9FC1", "\u9FC1"], /* CJK Ideograph */
  ["\u9FC2", "\u9FC2"], /* CJK Ideograph */
  ["\u9FC3", "\u9FC3"], /* CJK Ideograph */
  ["\u9FC4", "\u9FC4"], /* CJK Ideograph */
  ["\u9FC5", "\u9FC5"], /* CJK Ideograph */
  ["\u9FC6", "\u9FC6"], /* CJK Ideograph */
  ["\u9FC7", "\u9FC7"], /* CJK Ideograph */
  ["\u9FC8", "\u9FC8"], /* CJK Ideograph */
  ["\u9FC9", "\u9FC9"], /* CJK Ideograph */
  ["\u9FCA", "\u9FCA"], /* CJK Ideograph */
  ["\u9FCB", "\u9FCB"], /* CJK Ideograph */
  ["\u9FCC", "\u9FCC"], /* CJK Ideograph */
  ["\u9FCD", "\u9FCD"], /* CJK Ideograph */
  ["\u9FCE", "\u9FCE"], /* CJK Ideograph */
  ["\u9FCF", "\u9FCF"], /* CJK Ideograph */
  ["\u9FD0", "\u9FD0"], /* CJK Ideograph */
  ["\u9FD1", "\u9FD1"], /* CJK Ideograph */
  ["\u9FD2", "\u9FD2"], /* CJK Ideograph */
  ["\u9FD3", "\u9FD3"], /* CJK Ideograph */
  ["\u9FD4", "\u9FD4"], /* CJK Ideograph */
  ["\u9FD5", "\u9FD5"], /* CJK Ideograph */
  ["\u9FD6", "\u9FD6"], /* CJK Ideograph */
  ["\u9FD7", "\u9FD7"], /* CJK Ideograph */
  ["\u9FD8", "\u9FD8"], /* CJK Ideograph */
  ["\u9FD9", "\u9FD9"], /* CJK Ideograph */
  ["\u9FDA", "\u9FDA"], /* CJK Ideograph */
  ["\u9FDB", "\u9FDB"], /* CJK Ideograph */
  ["\u9FDC", "\u9FDC"], /* CJK Ideograph */
  ["\u9FDD", "\u9FDD"], /* CJK Ideograph */
  ["\u9FDE", "\u9FDE"], /* CJK Ideograph */
  ["\u9FDF", "\u9FDF"], /* CJK Ideograph */
  ["\u9FE0", "\u9FE0"], /* CJK Ideograph */
  ["\u9FE1", "\u9FE1"], /* CJK Ideograph */
  ["\u9FE2", "\u9FE2"], /* CJK Ideograph */
  ["\u9FE3", "\u9FE3"], /* CJK Ideograph */
  ["\u9FE4", "\u9FE4"], /* CJK Ideograph */
  ["\u9FE5", "\u9FE5"], /* CJK Ideograph */
  ["\u9FE6", "\u9FE6"], /* CJK Ideograph */
  ["\u9FE7", "\u9FE7"], /* CJK Ideograph */
  ["\u9FE8", "\u9FE8"], /* CJK Ideograph */
  ["\u9FE9", "\u9FE9"], /* CJK Ideograph */
  ["\u9FEA", "\u9FEA"], /* CJK Ideograph */
  ["\u9FEB", "\u9FEB"], /* CJK Ideograph */
  ["\u9FEC", "\u9FEC"], /* CJK Ideograph */
  ["\u9FED", "\u9FED"], /* CJK Ideograph */
  ["\u9FEE", "\u9FEE"], /* CJK Ideograph */
  ["\u9FEF", "\u9FEF"], /* CJK Ideograph */
  ["\u9FF0", "\u9FF0"],
  ["\u9FF1", "\u9FF1"],
  ["\u9FF2", "\u9FF2"],
  ["\u9FF3", "\u9FF3"],
  ["\u9FF4", "\u9FF4"],
  ["\u9FF5", "\u9FF5"],
  ["\u9FF6", "\u9FF6"],
  ["\u9FF7", "\u9FF7"],
  ["\u9FF8", "\u9FF8"],
  ["\u9FF9", "\u9FF9"],
  ["\u9FFA", "\u9FFA"],
  ["\u9FFB", "\u9FFB"],
  ["\u9FFC", "\u9FFC"],
  ["\u9FFD", "\u9FFD"],
  ["\u9FFE", "\u9FFE"],
  ["\u9FFF", "\u9FFF"],
  ["\uA000", "\uA000"], /* YI SYLLABLE IT */
  ["\uA001", "\uA001"], /* YI SYLLABLE IX */
  ["\uA002", "\uA002"], /* YI SYLLABLE I */
  ["\uA003", "\uA003"], /* YI SYLLABLE IP */
  ["\uA004", "\uA004"], /* YI SYLLABLE IET */
  ["\uA005", "\uA005"], /* YI SYLLABLE IEX */
  ["\uA006", "\uA006"], /* YI SYLLABLE IE */
  ["\uA007", "\uA007"], /* YI SYLLABLE IEP */
  ["\uA008", "\uA008"], /* YI SYLLABLE AT */
  ["\uA009", "\uA009"], /* YI SYLLABLE AX */
  ["\uA00A", "\uA00A"], /* YI SYLLABLE A */
  ["\uA00B", "\uA00B"], /* YI SYLLABLE AP */
  ["\uA00C", "\uA00C"], /* YI SYLLABLE UOX */
  ["\uA00D", "\uA00D"], /* YI SYLLABLE UO */
  ["\uA00E", "\uA00E"], /* YI SYLLABLE UOP */
  ["\uA00F", "\uA00F"], /* YI SYLLABLE OT */
  ["\uA010", "\uA010"], /* YI SYLLABLE OX */
  ["\uA011", "\uA011"], /* YI SYLLABLE O */
  ["\uA012", "\uA012"], /* YI SYLLABLE OP */
  ["\uA013", "\uA013"], /* YI SYLLABLE EX */
  ["\uA014", "\uA014"], /* YI SYLLABLE E */
  ["\uA015", "\uA015"], /* YI SYLLABLE WU */
  ["\uA016", "\uA016"], /* YI SYLLABLE BIT */
  ["\uA017", "\uA017"], /* YI SYLLABLE BIX */
  ["\uA018", "\uA018"], /* YI SYLLABLE BI */
  ["\uA019", "\uA019"], /* YI SYLLABLE BIP */
  ["\uA01A", "\uA01A"], /* YI SYLLABLE BIET */
  ["\uA01B", "\uA01B"], /* YI SYLLABLE BIEX */
  ["\uA01C", "\uA01C"], /* YI SYLLABLE BIE */
  ["\uA01D", "\uA01D"], /* YI SYLLABLE BIEP */
  ["\uA01E", "\uA01E"], /* YI SYLLABLE BAT */
  ["\uA01F", "\uA01F"], /* YI SYLLABLE BAX */
  ["\uA020", "\uA020"], /* YI SYLLABLE BA */
  ["\uA021", "\uA021"], /* YI SYLLABLE BAP */
  ["\uA022", "\uA022"], /* YI SYLLABLE BUOX */
  ["\uA023", "\uA023"], /* YI SYLLABLE BUO */
  ["\uA024", "\uA024"], /* YI SYLLABLE BUOP */
  ["\uA025", "\uA025"], /* YI SYLLABLE BOT */
  ["\uA026", "\uA026"], /* YI SYLLABLE BOX */
  ["\uA027", "\uA027"], /* YI SYLLABLE BO */
  ["\uA028", "\uA028"], /* YI SYLLABLE BOP */
  ["\uA029", "\uA029"], /* YI SYLLABLE BEX */
  ["\uA02A", "\uA02A"], /* YI SYLLABLE BE */
  ["\uA02B", "\uA02B"], /* YI SYLLABLE BEP */
  ["\uA02C", "\uA02C"], /* YI SYLLABLE BUT */
  ["\uA02D", "\uA02D"], /* YI SYLLABLE BUX */
  ["\uA02E", "\uA02E"], /* YI SYLLABLE BU */
  ["\uA02F", "\uA02F"], /* YI SYLLABLE BUP */
  ["\uA030", "\uA030"], /* YI SYLLABLE BURX */
  ["\uA031", "\uA031"], /* YI SYLLABLE BUR */
  ["\uA032", "\uA032"], /* YI SYLLABLE BYT */
  ["\uA033", "\uA033"], /* YI SYLLABLE BYX */
  ["\uA034", "\uA034"], /* YI SYLLABLE BY */
  ["\uA035", "\uA035"], /* YI SYLLABLE BYP */
  ["\uA036", "\uA036"], /* YI SYLLABLE BYRX */
  ["\uA037", "\uA037"], /* YI SYLLABLE BYR */
  ["\uA038", "\uA038"], /* YI SYLLABLE PIT */
  ["\uA039", "\uA039"], /* YI SYLLABLE PIX */
  ["\uA03A", "\uA03A"], /* YI SYLLABLE PI */
  ["\uA03B", "\uA03B"], /* YI SYLLABLE PIP */
  ["\uA03C", "\uA03C"], /* YI SYLLABLE PIEX */
  ["\uA03D", "\uA03D"], /* YI SYLLABLE PIE */
  ["\uA03E", "\uA03E"], /* YI SYLLABLE PIEP */
  ["\uA03F", "\uA03F"], /* YI SYLLABLE PAT */
  ["\uA040", "\uA040"], /* YI SYLLABLE PAX */
  ["\uA041", "\uA041"], /* YI SYLLABLE PA */
  ["\uA042", "\uA042"], /* YI SYLLABLE PAP */
  ["\uA043", "\uA043"], /* YI SYLLABLE PUOX */
  ["\uA044", "\uA044"], /* YI SYLLABLE PUO */
  ["\uA045", "\uA045"], /* YI SYLLABLE PUOP */
  ["\uA046", "\uA046"], /* YI SYLLABLE POT */
  ["\uA047", "\uA047"], /* YI SYLLABLE POX */
  ["\uA048", "\uA048"], /* YI SYLLABLE PO */
  ["\uA049", "\uA049"], /* YI SYLLABLE POP */
  ["\uA04A", "\uA04A"], /* YI SYLLABLE PUT */
  ["\uA04B", "\uA04B"], /* YI SYLLABLE PUX */
  ["\uA04C", "\uA04C"], /* YI SYLLABLE PU */
  ["\uA04D", "\uA04D"], /* YI SYLLABLE PUP */
  ["\uA04E", "\uA04E"], /* YI SYLLABLE PURX */
  ["\uA04F", "\uA04F"], /* YI SYLLABLE PUR */
  ["\uA050", "\uA050"], /* YI SYLLABLE PYT */
  ["\uA051", "\uA051"], /* YI SYLLABLE PYX */
  ["\uA052", "\uA052"], /* YI SYLLABLE PY */
  ["\uA053", "\uA053"], /* YI SYLLABLE PYP */
  ["\uA054", "\uA054"], /* YI SYLLABLE PYRX */
  ["\uA055", "\uA055"], /* YI SYLLABLE PYR */
  ["\uA056", "\uA056"], /* YI SYLLABLE BBIT */
  ["\uA057", "\uA057"], /* YI SYLLABLE BBIX */
  ["\uA058", "\uA058"], /* YI SYLLABLE BBI */
  ["\uA059", "\uA059"], /* YI SYLLABLE BBIP */
  ["\uA05A", "\uA05A"], /* YI SYLLABLE BBIET */
  ["\uA05B", "\uA05B"], /* YI SYLLABLE BBIEX */
  ["\uA05C", "\uA05C"], /* YI SYLLABLE BBIE */
  ["\uA05D", "\uA05D"], /* YI SYLLABLE BBIEP */
  ["\uA05E", "\uA05E"], /* YI SYLLABLE BBAT */
  ["\uA05F", "\uA05F"], /* YI SYLLABLE BBAX */
  ["\uA060", "\uA060"], /* YI SYLLABLE BBA */
  ["\uA061", "\uA061"], /* YI SYLLABLE BBAP */
  ["\uA062", "\uA062"], /* YI SYLLABLE BBUOX */
  ["\uA063", "\uA063"], /* YI SYLLABLE BBUO */
  ["\uA064", "\uA064"], /* YI SYLLABLE BBUOP */
  ["\uA065", "\uA065"], /* YI SYLLABLE BBOT */
  ["\uA066", "\uA066"], /* YI SYLLABLE BBOX */
  ["\uA067", "\uA067"], /* YI SYLLABLE BBO */
  ["\uA068", "\uA068"], /* YI SYLLABLE BBOP */
  ["\uA069", "\uA069"], /* YI SYLLABLE BBEX */
  ["\uA06A", "\uA06A"], /* YI SYLLABLE BBE */
  ["\uA06B", "\uA06B"], /* YI SYLLABLE BBEP */
  ["\uA06C", "\uA06C"], /* YI SYLLABLE BBUT */
  ["\uA06D", "\uA06D"], /* YI SYLLABLE BBUX */
  ["\uA06E", "\uA06E"], /* YI SYLLABLE BBU */
  ["\uA06F", "\uA06F"], /* YI SYLLABLE BBUP */
  ["\uA070", "\uA070"], /* YI SYLLABLE BBURX */
  ["\uA071", "\uA071"], /* YI SYLLABLE BBUR */
  ["\uA072", "\uA072"], /* YI SYLLABLE BBYT */
  ["\uA073", "\uA073"], /* YI SYLLABLE BBYX */
  ["\uA074", "\uA074"], /* YI SYLLABLE BBY */
  ["\uA075", "\uA075"], /* YI SYLLABLE BBYP */
  ["\uA076", "\uA076"], /* YI SYLLABLE NBIT */
  ["\uA077", "\uA077"], /* YI SYLLABLE NBIX */
  ["\uA078", "\uA078"], /* YI SYLLABLE NBI */
  ["\uA079", "\uA079"], /* YI SYLLABLE NBIP */
  ["\uA07A", "\uA07A"], /* YI SYLLABLE NBIEX */
  ["\uA07B", "\uA07B"], /* YI SYLLABLE NBIE */
  ["\uA07C", "\uA07C"], /* YI SYLLABLE NBIEP */
  ["\uA07D", "\uA07D"], /* YI SYLLABLE NBAT */
  ["\uA07E", "\uA07E"], /* YI SYLLABLE NBAX */
  ["\uA07F", "\uA07F"], /* YI SYLLABLE NBA */
  ["\uA080", "\uA080"], /* YI SYLLABLE NBAP */
  ["\uA081", "\uA081"], /* YI SYLLABLE NBOT */
  ["\uA082", "\uA082"], /* YI SYLLABLE NBOX */
  ["\uA083", "\uA083"], /* YI SYLLABLE NBO */
  ["\uA084", "\uA084"], /* YI SYLLABLE NBOP */
  ["\uA085", "\uA085"], /* YI SYLLABLE NBUT */
  ["\uA086", "\uA086"], /* YI SYLLABLE NBUX */
  ["\uA087", "\uA087"], /* YI SYLLABLE NBU */
  ["\uA088", "\uA088"], /* YI SYLLABLE NBUP */
  ["\uA089", "\uA089"], /* YI SYLLABLE NBURX */
  ["\uA08A", "\uA08A"], /* YI SYLLABLE NBUR */
  ["\uA08B", "\uA08B"], /* YI SYLLABLE NBYT */
  ["\uA08C", "\uA08C"], /* YI SYLLABLE NBYX */
  ["\uA08D", "\uA08D"], /* YI SYLLABLE NBY */
  ["\uA08E", "\uA08E"], /* YI SYLLABLE NBYP */
  ["\uA08F", "\uA08F"], /* YI SYLLABLE NBYRX */
  ["\uA090", "\uA090"], /* YI SYLLABLE NBYR */
  ["\uA091", "\uA091"], /* YI SYLLABLE HMIT */
  ["\uA092", "\uA092"], /* YI SYLLABLE HMIX */
  ["\uA093", "\uA093"], /* YI SYLLABLE HMI */
  ["\uA094", "\uA094"], /* YI SYLLABLE HMIP */
  ["\uA095", "\uA095"], /* YI SYLLABLE HMIEX */
  ["\uA096", "\uA096"], /* YI SYLLABLE HMIE */
  ["\uA097", "\uA097"], /* YI SYLLABLE HMIEP */
  ["\uA098", "\uA098"], /* YI SYLLABLE HMAT */
  ["\uA099", "\uA099"], /* YI SYLLABLE HMAX */
  ["\uA09A", "\uA09A"], /* YI SYLLABLE HMA */
  ["\uA09B", "\uA09B"], /* YI SYLLABLE HMAP */
  ["\uA09C", "\uA09C"], /* YI SYLLABLE HMUOX */
  ["\uA09D", "\uA09D"], /* YI SYLLABLE HMUO */
  ["\uA09E", "\uA09E"], /* YI SYLLABLE HMUOP */
  ["\uA09F", "\uA09F"], /* YI SYLLABLE HMOT */
  ["\uA0A0", "\uA0A0"], /* YI SYLLABLE HMOX */
  ["\uA0A1", "\uA0A1"], /* YI SYLLABLE HMO */
  ["\uA0A2", "\uA0A2"], /* YI SYLLABLE HMOP */
  ["\uA0A3", "\uA0A3"], /* YI SYLLABLE HMUT */
  ["\uA0A4", "\uA0A4"], /* YI SYLLABLE HMUX */
  ["\uA0A5", "\uA0A5"], /* YI SYLLABLE HMU */
  ["\uA0A6", "\uA0A6"], /* YI SYLLABLE HMUP */
  ["\uA0A7", "\uA0A7"], /* YI SYLLABLE HMURX */
  ["\uA0A8", "\uA0A8"], /* YI SYLLABLE HMUR */
  ["\uA0A9", "\uA0A9"], /* YI SYLLABLE HMYX */
  ["\uA0AA", "\uA0AA"], /* YI SYLLABLE HMY */
  ["\uA0AB", "\uA0AB"], /* YI SYLLABLE HMYP */
  ["\uA0AC", "\uA0AC"], /* YI SYLLABLE HMYRX */
  ["\uA0AD", "\uA0AD"], /* YI SYLLABLE HMYR */
  ["\uA0AE", "\uA0AE"], /* YI SYLLABLE MIT */
  ["\uA0AF", "\uA0AF"], /* YI SYLLABLE MIX */
  ["\uA0B0", "\uA0B0"], /* YI SYLLABLE MI */
  ["\uA0B1", "\uA0B1"], /* YI SYLLABLE MIP */
  ["\uA0B2", "\uA0B2"], /* YI SYLLABLE MIEX */
  ["\uA0B3", "\uA0B3"], /* YI SYLLABLE MIE */
  ["\uA0B4", "\uA0B4"], /* YI SYLLABLE MIEP */
  ["\uA0B5", "\uA0B5"], /* YI SYLLABLE MAT */
  ["\uA0B6", "\uA0B6"], /* YI SYLLABLE MAX */
  ["\uA0B7", "\uA0B7"], /* YI SYLLABLE MA */
  ["\uA0B8", "\uA0B8"], /* YI SYLLABLE MAP */
  ["\uA0B9", "\uA0B9"], /* YI SYLLABLE MUOT */
  ["\uA0BA", "\uA0BA"], /* YI SYLLABLE MUOX */
  ["\uA0BB", "\uA0BB"], /* YI SYLLABLE MUO */
  ["\uA0BC", "\uA0BC"], /* YI SYLLABLE MUOP */
  ["\uA0BD", "\uA0BD"], /* YI SYLLABLE MOT */
  ["\uA0BE", "\uA0BE"], /* YI SYLLABLE MOX */
  ["\uA0BF", "\uA0BF"], /* YI SYLLABLE MO */
  ["\uA0C0", "\uA0C0"], /* YI SYLLABLE MOP */
  ["\uA0C1", "\uA0C1"], /* YI SYLLABLE MEX */
  ["\uA0C2", "\uA0C2"], /* YI SYLLABLE ME */
  ["\uA0C3", "\uA0C3"], /* YI SYLLABLE MUT */
  ["\uA0C4", "\uA0C4"], /* YI SYLLABLE MUX */
  ["\uA0C5", "\uA0C5"], /* YI SYLLABLE MU */
  ["\uA0C6", "\uA0C6"], /* YI SYLLABLE MUP */
  ["\uA0C7", "\uA0C7"], /* YI SYLLABLE MURX */
  ["\uA0C8", "\uA0C8"], /* YI SYLLABLE MUR */
  ["\uA0C9", "\uA0C9"], /* YI SYLLABLE MYT */
  ["\uA0CA", "\uA0CA"], /* YI SYLLABLE MYX */
  ["\uA0CB", "\uA0CB"], /* YI SYLLABLE MY */
  ["\uA0CC", "\uA0CC"], /* YI SYLLABLE MYP */
  ["\uA0CD", "\uA0CD"], /* YI SYLLABLE FIT */
  ["\uA0CE", "\uA0CE"], /* YI SYLLABLE FIX */
  ["\uA0CF", "\uA0CF"], /* YI SYLLABLE FI */
  ["\uA0D0", "\uA0D0"], /* YI SYLLABLE FIP */
  ["\uA0D1", "\uA0D1"], /* YI SYLLABLE FAT */
  ["\uA0D2", "\uA0D2"], /* YI SYLLABLE FAX */
  ["\uA0D3", "\uA0D3"], /* YI SYLLABLE FA */
  ["\uA0D4", "\uA0D4"], /* YI SYLLABLE FAP */
  ["\uA0D5", "\uA0D5"], /* YI SYLLABLE FOX */
  ["\uA0D6", "\uA0D6"], /* YI SYLLABLE FO */
  ["\uA0D7", "\uA0D7"], /* YI SYLLABLE FOP */
  ["\uA0D8", "\uA0D8"], /* YI SYLLABLE FUT */
  ["\uA0D9", "\uA0D9"], /* YI SYLLABLE FUX */
  ["\uA0DA", "\uA0DA"], /* YI SYLLABLE FU */
  ["\uA0DB", "\uA0DB"], /* YI SYLLABLE FUP */
  ["\uA0DC", "\uA0DC"], /* YI SYLLABLE FURX */
  ["\uA0DD", "\uA0DD"], /* YI SYLLABLE FUR */
  ["\uA0DE", "\uA0DE"], /* YI SYLLABLE FYT */
  ["\uA0DF", "\uA0DF"], /* YI SYLLABLE FYX */
  ["\uA0E0", "\uA0E0"], /* YI SYLLABLE FY */
  ["\uA0E1", "\uA0E1"], /* YI SYLLABLE FYP */
  ["\uA0E2", "\uA0E2"], /* YI SYLLABLE VIT */
  ["\uA0E3", "\uA0E3"], /* YI SYLLABLE VIX */
  ["\uA0E4", "\uA0E4"], /* YI SYLLABLE VI */
  ["\uA0E5", "\uA0E5"], /* YI SYLLABLE VIP */
  ["\uA0E6", "\uA0E6"], /* YI SYLLABLE VIET */
  ["\uA0E7", "\uA0E7"], /* YI SYLLABLE VIEX */
  ["\uA0E8", "\uA0E8"], /* YI SYLLABLE VIE */
  ["\uA0E9", "\uA0E9"], /* YI SYLLABLE VIEP */
  ["\uA0EA", "\uA0EA"], /* YI SYLLABLE VAT */
  ["\uA0EB", "\uA0EB"], /* YI SYLLABLE VAX */
  ["\uA0EC", "\uA0EC"], /* YI SYLLABLE VA */
  ["\uA0ED", "\uA0ED"], /* YI SYLLABLE VAP */
  ["\uA0EE", "\uA0EE"], /* YI SYLLABLE VOT */
  ["\uA0EF", "\uA0EF"], /* YI SYLLABLE VOX */
  ["\uA0F0", "\uA0F0"], /* YI SYLLABLE VO */
  ["\uA0F1", "\uA0F1"], /* YI SYLLABLE VOP */
  ["\uA0F2", "\uA0F2"], /* YI SYLLABLE VEX */
  ["\uA0F3", "\uA0F3"], /* YI SYLLABLE VEP */
  ["\uA0F4", "\uA0F4"], /* YI SYLLABLE VUT */
  ["\uA0F5", "\uA0F5"], /* YI SYLLABLE VUX */
  ["\uA0F6", "\uA0F6"], /* YI SYLLABLE VU */
  ["\uA0F7", "\uA0F7"], /* YI SYLLABLE VUP */
  ["\uA0F8", "\uA0F8"], /* YI SYLLABLE VURX */
  ["\uA0F9", "\uA0F9"], /* YI SYLLABLE VUR */
  ["\uA0FA", "\uA0FA"], /* YI SYLLABLE VYT */
  ["\uA0FB", "\uA0FB"], /* YI SYLLABLE VYX */
  ["\uA0FC", "\uA0FC"], /* YI SYLLABLE VY */
  ["\uA0FD", "\uA0FD"], /* YI SYLLABLE VYP */
  ["\uA0FE", "\uA0FE"], /* YI SYLLABLE VYRX */
  ["\uA0FF", "\uA0FF"], /* YI SYLLABLE VYR */
  ["\uA100", "\uA100"], /* YI SYLLABLE DIT */
  ["\uA101", "\uA101"], /* YI SYLLABLE DIX */
  ["\uA102", "\uA102"], /* YI SYLLABLE DI */
  ["\uA103", "\uA103"], /* YI SYLLABLE DIP */
  ["\uA104", "\uA104"], /* YI SYLLABLE DIEX */
  ["\uA105", "\uA105"], /* YI SYLLABLE DIE */
  ["\uA106", "\uA106"], /* YI SYLLABLE DIEP */
  ["\uA107", "\uA107"], /* YI SYLLABLE DAT */
  ["\uA108", "\uA108"], /* YI SYLLABLE DAX */
  ["\uA109", "\uA109"], /* YI SYLLABLE DA */
  ["\uA10A", "\uA10A"], /* YI SYLLABLE DAP */
  ["\uA10B", "\uA10B"], /* YI SYLLABLE DUOX */
  ["\uA10C", "\uA10C"], /* YI SYLLABLE DUO */
  ["\uA10D", "\uA10D"], /* YI SYLLABLE DOT */
  ["\uA10E", "\uA10E"], /* YI SYLLABLE DOX */
  ["\uA10F", "\uA10F"], /* YI SYLLABLE DO */
  ["\uA110", "\uA110"], /* YI SYLLABLE DOP */
  ["\uA111", "\uA111"], /* YI SYLLABLE DEX */
  ["\uA112", "\uA112"], /* YI SYLLABLE DE */
  ["\uA113", "\uA113"], /* YI SYLLABLE DEP */
  ["\uA114", "\uA114"], /* YI SYLLABLE DUT */
  ["\uA115", "\uA115"], /* YI SYLLABLE DUX */
  ["\uA116", "\uA116"], /* YI SYLLABLE DU */
  ["\uA117", "\uA117"], /* YI SYLLABLE DUP */
  ["\uA118", "\uA118"], /* YI SYLLABLE DURX */
  ["\uA119", "\uA119"], /* YI SYLLABLE DUR */
  ["\uA11A", "\uA11A"], /* YI SYLLABLE TIT */
  ["\uA11B", "\uA11B"], /* YI SYLLABLE TIX */
  ["\uA11C", "\uA11C"], /* YI SYLLABLE TI */
  ["\uA11D", "\uA11D"], /* YI SYLLABLE TIP */
  ["\uA11E", "\uA11E"], /* YI SYLLABLE TIEX */
  ["\uA11F", "\uA11F"], /* YI SYLLABLE TIE */
  ["\uA120", "\uA120"], /* YI SYLLABLE TIEP */
  ["\uA121", "\uA121"], /* YI SYLLABLE TAT */
  ["\uA122", "\uA122"], /* YI SYLLABLE TAX */
  ["\uA123", "\uA123"], /* YI SYLLABLE TA */
  ["\uA124", "\uA124"], /* YI SYLLABLE TAP */
  ["\uA125", "\uA125"], /* YI SYLLABLE TUOT */
  ["\uA126", "\uA126"], /* YI SYLLABLE TUOX */
  ["\uA127", "\uA127"], /* YI SYLLABLE TUO */
  ["\uA128", "\uA128"], /* YI SYLLABLE TUOP */
  ["\uA129", "\uA129"], /* YI SYLLABLE TOT */
  ["\uA12A", "\uA12A"], /* YI SYLLABLE TOX */
  ["\uA12B", "\uA12B"], /* YI SYLLABLE TO */
  ["\uA12C", "\uA12C"], /* YI SYLLABLE TOP */
  ["\uA12D", "\uA12D"], /* YI SYLLABLE TEX */
  ["\uA12E", "\uA12E"], /* YI SYLLABLE TE */
  ["\uA12F", "\uA12F"], /* YI SYLLABLE TEP */
  ["\uA130", "\uA130"], /* YI SYLLABLE TUT */
  ["\uA131", "\uA131"], /* YI SYLLABLE TUX */
  ["\uA132", "\uA132"], /* YI SYLLABLE TU */
  ["\uA133", "\uA133"], /* YI SYLLABLE TUP */
  ["\uA134", "\uA134"], /* YI SYLLABLE TURX */
  ["\uA135", "\uA135"], /* YI SYLLABLE TUR */
  ["\uA136", "\uA136"], /* YI SYLLABLE DDIT */
  ["\uA137", "\uA137"], /* YI SYLLABLE DDIX */
  ["\uA138", "\uA138"], /* YI SYLLABLE DDI */
  ["\uA139", "\uA139"], /* YI SYLLABLE DDIP */
  ["\uA13A", "\uA13A"], /* YI SYLLABLE DDIEX */
  ["\uA13B", "\uA13B"], /* YI SYLLABLE DDIE */
  ["\uA13C", "\uA13C"], /* YI SYLLABLE DDIEP */
  ["\uA13D", "\uA13D"], /* YI SYLLABLE DDAT */
  ["\uA13E", "\uA13E"], /* YI SYLLABLE DDAX */
  ["\uA13F", "\uA13F"], /* YI SYLLABLE DDA */
  ["\uA140", "\uA140"], /* YI SYLLABLE DDAP */
  ["\uA141", "\uA141"], /* YI SYLLABLE DDUOX */
  ["\uA142", "\uA142"], /* YI SYLLABLE DDUO */
  ["\uA143", "\uA143"], /* YI SYLLABLE DDUOP */
  ["\uA144", "\uA144"], /* YI SYLLABLE DDOT */
  ["\uA145", "\uA145"], /* YI SYLLABLE DDOX */
  ["\uA146", "\uA146"], /* YI SYLLABLE DDO */
  ["\uA147", "\uA147"], /* YI SYLLABLE DDOP */
  ["\uA148", "\uA148"], /* YI SYLLABLE DDEX */
  ["\uA149", "\uA149"], /* YI SYLLABLE DDE */
  ["\uA14A", "\uA14A"], /* YI SYLLABLE DDEP */
  ["\uA14B", "\uA14B"], /* YI SYLLABLE DDUT */
  ["\uA14C", "\uA14C"], /* YI SYLLABLE DDUX */
  ["\uA14D", "\uA14D"], /* YI SYLLABLE DDU */
  ["\uA14E", "\uA14E"], /* YI SYLLABLE DDUP */
  ["\uA14F", "\uA14F"], /* YI SYLLABLE DDURX */
  ["\uA150", "\uA150"], /* YI SYLLABLE DDUR */
  ["\uA151", "\uA151"], /* YI SYLLABLE NDIT */
  ["\uA152", "\uA152"], /* YI SYLLABLE NDIX */
  ["\uA153", "\uA153"], /* YI SYLLABLE NDI */
  ["\uA154", "\uA154"], /* YI SYLLABLE NDIP */
  ["\uA155", "\uA155"], /* YI SYLLABLE NDIEX */
  ["\uA156", "\uA156"], /* YI SYLLABLE NDIE */
  ["\uA157", "\uA157"], /* YI SYLLABLE NDAT */
  ["\uA158", "\uA158"], /* YI SYLLABLE NDAX */
  ["\uA159", "\uA159"], /* YI SYLLABLE NDA */
  ["\uA15A", "\uA15A"], /* YI SYLLABLE NDAP */
  ["\uA15B", "\uA15B"], /* YI SYLLABLE NDOT */
  ["\uA15C", "\uA15C"], /* YI SYLLABLE NDOX */
  ["\uA15D", "\uA15D"], /* YI SYLLABLE NDO */
  ["\uA15E", "\uA15E"], /* YI SYLLABLE NDOP */
  ["\uA15F", "\uA15F"], /* YI SYLLABLE NDEX */
  ["\uA160", "\uA160"], /* YI SYLLABLE NDE */
  ["\uA161", "\uA161"], /* YI SYLLABLE NDEP */
  ["\uA162", "\uA162"], /* YI SYLLABLE NDUT */
  ["\uA163", "\uA163"], /* YI SYLLABLE NDUX */
  ["\uA164", "\uA164"], /* YI SYLLABLE NDU */
  ["\uA165", "\uA165"], /* YI SYLLABLE NDUP */
  ["\uA166", "\uA166"], /* YI SYLLABLE NDURX */
  ["\uA167", "\uA167"], /* YI SYLLABLE NDUR */
  ["\uA168", "\uA168"], /* YI SYLLABLE HNIT */
  ["\uA169", "\uA169"], /* YI SYLLABLE HNIX */
  ["\uA16A", "\uA16A"], /* YI SYLLABLE HNI */
  ["\uA16B", "\uA16B"], /* YI SYLLABLE HNIP */
  ["\uA16C", "\uA16C"], /* YI SYLLABLE HNIET */
  ["\uA16D", "\uA16D"], /* YI SYLLABLE HNIEX */
  ["\uA16E", "\uA16E"], /* YI SYLLABLE HNIE */
  ["\uA16F", "\uA16F"], /* YI SYLLABLE HNIEP */
  ["\uA170", "\uA170"], /* YI SYLLABLE HNAT */
  ["\uA171", "\uA171"], /* YI SYLLABLE HNAX */
  ["\uA172", "\uA172"], /* YI SYLLABLE HNA */
  ["\uA173", "\uA173"], /* YI SYLLABLE HNAP */
  ["\uA174", "\uA174"], /* YI SYLLABLE HNUOX */
  ["\uA175", "\uA175"], /* YI SYLLABLE HNUO */
  ["\uA176", "\uA176"], /* YI SYLLABLE HNOT */
  ["\uA177", "\uA177"], /* YI SYLLABLE HNOX */
  ["\uA178", "\uA178"], /* YI SYLLABLE HNOP */
  ["\uA179", "\uA179"], /* YI SYLLABLE HNEX */
  ["\uA17A", "\uA17A"], /* YI SYLLABLE HNE */
  ["\uA17B", "\uA17B"], /* YI SYLLABLE HNEP */
  ["\uA17C", "\uA17C"], /* YI SYLLABLE HNUT */
  ["\uA17D", "\uA17D"], /* YI SYLLABLE NIT */
  ["\uA17E", "\uA17E"], /* YI SYLLABLE NIX */
  ["\uA17F", "\uA17F"], /* YI SYLLABLE NI */
  ["\uA180", "\uA180"], /* YI SYLLABLE NIP */
  ["\uA181", "\uA181"], /* YI SYLLABLE NIEX */
  ["\uA182", "\uA182"], /* YI SYLLABLE NIE */
  ["\uA183", "\uA183"], /* YI SYLLABLE NIEP */
  ["\uA184", "\uA184"], /* YI SYLLABLE NAX */
  ["\uA185", "\uA185"], /* YI SYLLABLE NA */
  ["\uA186", "\uA186"], /* YI SYLLABLE NAP */
  ["\uA187", "\uA187"], /* YI SYLLABLE NUOX */
  ["\uA188", "\uA188"], /* YI SYLLABLE NUO */
  ["\uA189", "\uA189"], /* YI SYLLABLE NUOP */
  ["\uA18A", "\uA18A"], /* YI SYLLABLE NOT */
  ["\uA18B", "\uA18B"], /* YI SYLLABLE NOX */
  ["\uA18C", "\uA18C"], /* YI SYLLABLE NO */
  ["\uA18D", "\uA18D"], /* YI SYLLABLE NOP */
  ["\uA18E", "\uA18E"], /* YI SYLLABLE NEX */
  ["\uA18F", "\uA18F"], /* YI SYLLABLE NE */
  ["\uA190", "\uA190"], /* YI SYLLABLE NEP */
  ["\uA191", "\uA191"], /* YI SYLLABLE NUT */
  ["\uA192", "\uA192"], /* YI SYLLABLE NUX */
  ["\uA193", "\uA193"], /* YI SYLLABLE NU */
  ["\uA194", "\uA194"], /* YI SYLLABLE NUP */
  ["\uA195", "\uA195"], /* YI SYLLABLE NURX */
  ["\uA196", "\uA196"], /* YI SYLLABLE NUR */
  ["\uA197", "\uA197"], /* YI SYLLABLE HLIT */
  ["\uA198", "\uA198"], /* YI SYLLABLE HLIX */
  ["\uA199", "\uA199"], /* YI SYLLABLE HLI */
  ["\uA19A", "\uA19A"], /* YI SYLLABLE HLIP */
  ["\uA19B", "\uA19B"], /* YI SYLLABLE HLIEX */
  ["\uA19C", "\uA19C"], /* YI SYLLABLE HLIE */
  ["\uA19D", "\uA19D"], /* YI SYLLABLE HLIEP */
  ["\uA19E", "\uA19E"], /* YI SYLLABLE HLAT */
  ["\uA19F", "\uA19F"], /* YI SYLLABLE HLAX */
  ["\uA1A0", "\uA1A0"], /* YI SYLLABLE HLA */
  ["\uA1A1", "\uA1A1"], /* YI SYLLABLE HLAP */
  ["\uA1A2", "\uA1A2"], /* YI SYLLABLE HLUOX */
  ["\uA1A3", "\uA1A3"], /* YI SYLLABLE HLUO */
  ["\uA1A4", "\uA1A4"], /* YI SYLLABLE HLUOP */
  ["\uA1A5", "\uA1A5"], /* YI SYLLABLE HLOX */
  ["\uA1A6", "\uA1A6"], /* YI SYLLABLE HLO */
  ["\uA1A7", "\uA1A7"], /* YI SYLLABLE HLOP */
  ["\uA1A8", "\uA1A8"], /* YI SYLLABLE HLEX */
  ["\uA1A9", "\uA1A9"], /* YI SYLLABLE HLE */
  ["\uA1AA", "\uA1AA"], /* YI SYLLABLE HLEP */
  ["\uA1AB", "\uA1AB"], /* YI SYLLABLE HLUT */
  ["\uA1AC", "\uA1AC"], /* YI SYLLABLE HLUX */
  ["\uA1AD", "\uA1AD"], /* YI SYLLABLE HLU */
  ["\uA1AE", "\uA1AE"], /* YI SYLLABLE HLUP */
  ["\uA1AF", "\uA1AF"], /* YI SYLLABLE HLURX */
  ["\uA1B0", "\uA1B0"], /* YI SYLLABLE HLUR */
  ["\uA1B1", "\uA1B1"], /* YI SYLLABLE HLYT */
  ["\uA1B2", "\uA1B2"], /* YI SYLLABLE HLYX */
  ["\uA1B3", "\uA1B3"], /* YI SYLLABLE HLY */
  ["\uA1B4", "\uA1B4"], /* YI SYLLABLE HLYP */
  ["\uA1B5", "\uA1B5"], /* YI SYLLABLE HLYRX */
  ["\uA1B6", "\uA1B6"], /* YI SYLLABLE HLYR */
  ["\uA1B7", "\uA1B7"], /* YI SYLLABLE LIT */
  ["\uA1B8", "\uA1B8"], /* YI SYLLABLE LIX */
  ["\uA1B9", "\uA1B9"], /* YI SYLLABLE LI */
  ["\uA1BA", "\uA1BA"], /* YI SYLLABLE LIP */
  ["\uA1BB", "\uA1BB"], /* YI SYLLABLE LIET */
  ["\uA1BC", "\uA1BC"], /* YI SYLLABLE LIEX */
  ["\uA1BD", "\uA1BD"], /* YI SYLLABLE LIE */
  ["\uA1BE", "\uA1BE"], /* YI SYLLABLE LIEP */
  ["\uA1BF", "\uA1BF"], /* YI SYLLABLE LAT */
  ["\uA1C0", "\uA1C0"], /* YI SYLLABLE LAX */
  ["\uA1C1", "\uA1C1"], /* YI SYLLABLE LA */
  ["\uA1C2", "\uA1C2"], /* YI SYLLABLE LAP */
  ["\uA1C3", "\uA1C3"], /* YI SYLLABLE LUOT */
  ["\uA1C4", "\uA1C4"], /* YI SYLLABLE LUOX */
  ["\uA1C5", "\uA1C5"], /* YI SYLLABLE LUO */
  ["\uA1C6", "\uA1C6"], /* YI SYLLABLE LUOP */
  ["\uA1C7", "\uA1C7"], /* YI SYLLABLE LOT */
  ["\uA1C8", "\uA1C8"], /* YI SYLLABLE LOX */
  ["\uA1C9", "\uA1C9"], /* YI SYLLABLE LO */
  ["\uA1CA", "\uA1CA"], /* YI SYLLABLE LOP */
  ["\uA1CB", "\uA1CB"], /* YI SYLLABLE LEX */
  ["\uA1CC", "\uA1CC"], /* YI SYLLABLE LE */
  ["\uA1CD", "\uA1CD"], /* YI SYLLABLE LEP */
  ["\uA1CE", "\uA1CE"], /* YI SYLLABLE LUT */
  ["\uA1CF", "\uA1CF"], /* YI SYLLABLE LUX */
  ["\uA1D0", "\uA1D0"], /* YI SYLLABLE LU */
  ["\uA1D1", "\uA1D1"], /* YI SYLLABLE LUP */
  ["\uA1D2", "\uA1D2"], /* YI SYLLABLE LURX */
  ["\uA1D3", "\uA1D3"], /* YI SYLLABLE LUR */
  ["\uA1D4", "\uA1D4"], /* YI SYLLABLE LYT */
  ["\uA1D5", "\uA1D5"], /* YI SYLLABLE LYX */
  ["\uA1D6", "\uA1D6"], /* YI SYLLABLE LY */
  ["\uA1D7", "\uA1D7"], /* YI SYLLABLE LYP */
  ["\uA1D8", "\uA1D8"], /* YI SYLLABLE LYRX */
  ["\uA1D9", "\uA1D9"], /* YI SYLLABLE LYR */
  ["\uA1DA", "\uA1DA"], /* YI SYLLABLE GIT */
  ["\uA1DB", "\uA1DB"], /* YI SYLLABLE GIX */
  ["\uA1DC", "\uA1DC"], /* YI SYLLABLE GI */
  ["\uA1DD", "\uA1DD"], /* YI SYLLABLE GIP */
  ["\uA1DE", "\uA1DE"], /* YI SYLLABLE GIET */
  ["\uA1DF", "\uA1DF"], /* YI SYLLABLE GIEX */
  ["\uA1E0", "\uA1E0"], /* YI SYLLABLE GIE */
  ["\uA1E1", "\uA1E1"], /* YI SYLLABLE GIEP */
  ["\uA1E2", "\uA1E2"], /* YI SYLLABLE GAT */
  ["\uA1E3", "\uA1E3"], /* YI SYLLABLE GAX */
  ["\uA1E4", "\uA1E4"], /* YI SYLLABLE GA */
  ["\uA1E5", "\uA1E5"], /* YI SYLLABLE GAP */
  ["\uA1E6", "\uA1E6"], /* YI SYLLABLE GUOT */
  ["\uA1E7", "\uA1E7"], /* YI SYLLABLE GUOX */
  ["\uA1E8", "\uA1E8"], /* YI SYLLABLE GUO */
  ["\uA1E9", "\uA1E9"], /* YI SYLLABLE GUOP */
  ["\uA1EA", "\uA1EA"], /* YI SYLLABLE GOT */
  ["\uA1EB", "\uA1EB"], /* YI SYLLABLE GOX */
  ["\uA1EC", "\uA1EC"], /* YI SYLLABLE GO */
  ["\uA1ED", "\uA1ED"], /* YI SYLLABLE GOP */
  ["\uA1EE", "\uA1EE"], /* YI SYLLABLE GET */
  ["\uA1EF", "\uA1EF"], /* YI SYLLABLE GEX */
  ["\uA1F0", "\uA1F0"], /* YI SYLLABLE GE */
  ["\uA1F1", "\uA1F1"], /* YI SYLLABLE GEP */
  ["\uA1F2", "\uA1F2"], /* YI SYLLABLE GUT */
  ["\uA1F3", "\uA1F3"], /* YI SYLLABLE GUX */
  ["\uA1F4", "\uA1F4"], /* YI SYLLABLE GU */
  ["\uA1F5", "\uA1F5"], /* YI SYLLABLE GUP */
  ["\uA1F6", "\uA1F6"], /* YI SYLLABLE GURX */
  ["\uA1F7", "\uA1F7"], /* YI SYLLABLE GUR */
  ["\uA1F8", "\uA1F8"], /* YI SYLLABLE KIT */
  ["\uA1F9", "\uA1F9"], /* YI SYLLABLE KIX */
  ["\uA1FA", "\uA1FA"], /* YI SYLLABLE KI */
  ["\uA1FB", "\uA1FB"], /* YI SYLLABLE KIP */
  ["\uA1FC", "\uA1FC"], /* YI SYLLABLE KIEX */
  ["\uA1FD", "\uA1FD"], /* YI SYLLABLE KIE */
  ["\uA1FE", "\uA1FE"], /* YI SYLLABLE KIEP */
  ["\uA1FF", "\uA1FF"], /* YI SYLLABLE KAT */
  ["\uA200", "\uA200"], /* YI SYLLABLE KAX */
  ["\uA201", "\uA201"], /* YI SYLLABLE KA */
  ["\uA202", "\uA202"], /* YI SYLLABLE KAP */
  ["\uA203", "\uA203"], /* YI SYLLABLE KUOX */
  ["\uA204", "\uA204"], /* YI SYLLABLE KUO */
  ["\uA205", "\uA205"], /* YI SYLLABLE KUOP */
  ["\uA206", "\uA206"], /* YI SYLLABLE KOT */
  ["\uA207", "\uA207"], /* YI SYLLABLE KOX */
  ["\uA208", "\uA208"], /* YI SYLLABLE KO */
  ["\uA209", "\uA209"], /* YI SYLLABLE KOP */
  ["\uA20A", "\uA20A"], /* YI SYLLABLE KET */
  ["\uA20B", "\uA20B"], /* YI SYLLABLE KEX */
  ["\uA20C", "\uA20C"], /* YI SYLLABLE KE */
  ["\uA20D", "\uA20D"], /* YI SYLLABLE KEP */
  ["\uA20E", "\uA20E"], /* YI SYLLABLE KUT */
  ["\uA20F", "\uA20F"], /* YI SYLLABLE KUX */
  ["\uA210", "\uA210"], /* YI SYLLABLE KU */
  ["\uA211", "\uA211"], /* YI SYLLABLE KUP */
  ["\uA212", "\uA212"], /* YI SYLLABLE KURX */
  ["\uA213", "\uA213"], /* YI SYLLABLE KUR */
  ["\uA214", "\uA214"], /* YI SYLLABLE GGIT */
  ["\uA215", "\uA215"], /* YI SYLLABLE GGIX */
  ["\uA216", "\uA216"], /* YI SYLLABLE GGI */
  ["\uA217", "\uA217"], /* YI SYLLABLE GGIEX */
  ["\uA218", "\uA218"], /* YI SYLLABLE GGIE */
  ["\uA219", "\uA219"], /* YI SYLLABLE GGIEP */
  ["\uA21A", "\uA21A"], /* YI SYLLABLE GGAT */
  ["\uA21B", "\uA21B"], /* YI SYLLABLE GGAX */
  ["\uA21C", "\uA21C"], /* YI SYLLABLE GGA */
  ["\uA21D", "\uA21D"], /* YI SYLLABLE GGAP */
  ["\uA21E", "\uA21E"], /* YI SYLLABLE GGUOT */
  ["\uA21F", "\uA21F"], /* YI SYLLABLE GGUOX */
  ["\uA220", "\uA220"], /* YI SYLLABLE GGUO */
  ["\uA221", "\uA221"], /* YI SYLLABLE GGUOP */
  ["\uA222", "\uA222"], /* YI SYLLABLE GGOT */
  ["\uA223", "\uA223"], /* YI SYLLABLE GGOX */
  ["\uA224", "\uA224"], /* YI SYLLABLE GGO */
  ["\uA225", "\uA225"], /* YI SYLLABLE GGOP */
  ["\uA226", "\uA226"], /* YI SYLLABLE GGET */
  ["\uA227", "\uA227"], /* YI SYLLABLE GGEX */
  ["\uA228", "\uA228"], /* YI SYLLABLE GGE */
  ["\uA229", "\uA229"], /* YI SYLLABLE GGEP */
  ["\uA22A", "\uA22A"], /* YI SYLLABLE GGUT */
  ["\uA22B", "\uA22B"], /* YI SYLLABLE GGUX */
  ["\uA22C", "\uA22C"], /* YI SYLLABLE GGU */
  ["\uA22D", "\uA22D"], /* YI SYLLABLE GGUP */
  ["\uA22E", "\uA22E"], /* YI SYLLABLE GGURX */
  ["\uA22F", "\uA22F"], /* YI SYLLABLE GGUR */
  ["\uA230", "\uA230"], /* YI SYLLABLE MGIEX */
  ["\uA231", "\uA231"], /* YI SYLLABLE MGIE */
  ["\uA232", "\uA232"], /* YI SYLLABLE MGAT */
  ["\uA233", "\uA233"], /* YI SYLLABLE MGAX */
  ["\uA234", "\uA234"], /* YI SYLLABLE MGA */
  ["\uA235", "\uA235"], /* YI SYLLABLE MGAP */
  ["\uA236", "\uA236"], /* YI SYLLABLE MGUOX */
  ["\uA237", "\uA237"], /* YI SYLLABLE MGUO */
  ["\uA238", "\uA238"], /* YI SYLLABLE MGUOP */
  ["\uA239", "\uA239"], /* YI SYLLABLE MGOT */
  ["\uA23A", "\uA23A"], /* YI SYLLABLE MGOX */
  ["\uA23B", "\uA23B"], /* YI SYLLABLE MGO */
  ["\uA23C", "\uA23C"], /* YI SYLLABLE MGOP */
  ["\uA23D", "\uA23D"], /* YI SYLLABLE MGEX */
  ["\uA23E", "\uA23E"], /* YI SYLLABLE MGE */
  ["\uA23F", "\uA23F"], /* YI SYLLABLE MGEP */
  ["\uA240", "\uA240"], /* YI SYLLABLE MGUT */
  ["\uA241", "\uA241"], /* YI SYLLABLE MGUX */
  ["\uA242", "\uA242"], /* YI SYLLABLE MGU */
  ["\uA243", "\uA243"], /* YI SYLLABLE MGUP */
  ["\uA244", "\uA244"], /* YI SYLLABLE MGURX */
  ["\uA245", "\uA245"], /* YI SYLLABLE MGUR */
  ["\uA246", "\uA246"], /* YI SYLLABLE HXIT */
  ["\uA247", "\uA247"], /* YI SYLLABLE HXIX */
  ["\uA248", "\uA248"], /* YI SYLLABLE HXI */
  ["\uA249", "\uA249"], /* YI SYLLABLE HXIP */
  ["\uA24A", "\uA24A"], /* YI SYLLABLE HXIET */
  ["\uA24B", "\uA24B"], /* YI SYLLABLE HXIEX */
  ["\uA24C", "\uA24C"], /* YI SYLLABLE HXIE */
  ["\uA24D", "\uA24D"], /* YI SYLLABLE HXIEP */
  ["\uA24E", "\uA24E"], /* YI SYLLABLE HXAT */
  ["\uA24F", "\uA24F"], /* YI SYLLABLE HXAX */
  ["\uA250", "\uA250"], /* YI SYLLABLE HXA */
  ["\uA251", "\uA251"], /* YI SYLLABLE HXAP */
  ["\uA252", "\uA252"], /* YI SYLLABLE HXUOT */
  ["\uA253", "\uA253"], /* YI SYLLABLE HXUOX */
  ["\uA254", "\uA254"], /* YI SYLLABLE HXUO */
  ["\uA255", "\uA255"], /* YI SYLLABLE HXUOP */
  ["\uA256", "\uA256"], /* YI SYLLABLE HXOT */
  ["\uA257", "\uA257"], /* YI SYLLABLE HXOX */
  ["\uA258", "\uA258"], /* YI SYLLABLE HXO */
  ["\uA259", "\uA259"], /* YI SYLLABLE HXOP */
  ["\uA25A", "\uA25A"], /* YI SYLLABLE HXEX */
  ["\uA25B", "\uA25B"], /* YI SYLLABLE HXE */
  ["\uA25C", "\uA25C"], /* YI SYLLABLE HXEP */
  ["\uA25D", "\uA25D"], /* YI SYLLABLE NGIEX */
  ["\uA25E", "\uA25E"], /* YI SYLLABLE NGIE */
  ["\uA25F", "\uA25F"], /* YI SYLLABLE NGIEP */
  ["\uA260", "\uA260"], /* YI SYLLABLE NGAT */
  ["\uA261", "\uA261"], /* YI SYLLABLE NGAX */
  ["\uA262", "\uA262"], /* YI SYLLABLE NGA */
  ["\uA263", "\uA263"], /* YI SYLLABLE NGAP */
  ["\uA264", "\uA264"], /* YI SYLLABLE NGUOT */
  ["\uA265", "\uA265"], /* YI SYLLABLE NGUOX */
  ["\uA266", "\uA266"], /* YI SYLLABLE NGUO */
  ["\uA267", "\uA267"], /* YI SYLLABLE NGOT */
  ["\uA268", "\uA268"], /* YI SYLLABLE NGOX */
  ["\uA269", "\uA269"], /* YI SYLLABLE NGO */
  ["\uA26A", "\uA26A"], /* YI SYLLABLE NGOP */
  ["\uA26B", "\uA26B"], /* YI SYLLABLE NGEX */
  ["\uA26C", "\uA26C"], /* YI SYLLABLE NGE */
  ["\uA26D", "\uA26D"], /* YI SYLLABLE NGEP */
  ["\uA26E", "\uA26E"], /* YI SYLLABLE HIT */
  ["\uA26F", "\uA26F"], /* YI SYLLABLE HIEX */
  ["\uA270", "\uA270"], /* YI SYLLABLE HIE */
  ["\uA271", "\uA271"], /* YI SYLLABLE HAT */
  ["\uA272", "\uA272"], /* YI SYLLABLE HAX */
  ["\uA273", "\uA273"], /* YI SYLLABLE HA */
  ["\uA274", "\uA274"], /* YI SYLLABLE HAP */
  ["\uA275", "\uA275"], /* YI SYLLABLE HUOT */
  ["\uA276", "\uA276"], /* YI SYLLABLE HUOX */
  ["\uA277", "\uA277"], /* YI SYLLABLE HUO */
  ["\uA278", "\uA278"], /* YI SYLLABLE HUOP */
  ["\uA279", "\uA279"], /* YI SYLLABLE HOT */
  ["\uA27A", "\uA27A"], /* YI SYLLABLE HOX */
  ["\uA27B", "\uA27B"], /* YI SYLLABLE HO */
  ["\uA27C", "\uA27C"], /* YI SYLLABLE HOP */
  ["\uA27D", "\uA27D"], /* YI SYLLABLE HEX */
  ["\uA27E", "\uA27E"], /* YI SYLLABLE HE */
  ["\uA27F", "\uA27F"], /* YI SYLLABLE HEP */
  ["\uA280", "\uA280"], /* YI SYLLABLE WAT */
  ["\uA281", "\uA281"], /* YI SYLLABLE WAX */
  ["\uA282", "\uA282"], /* YI SYLLABLE WA */
  ["\uA283", "\uA283"], /* YI SYLLABLE WAP */
  ["\uA284", "\uA284"], /* YI SYLLABLE WUOX */
  ["\uA285", "\uA285"], /* YI SYLLABLE WUO */
  ["\uA286", "\uA286"], /* YI SYLLABLE WUOP */
  ["\uA287", "\uA287"], /* YI SYLLABLE WOX */
  ["\uA288", "\uA288"], /* YI SYLLABLE WO */
  ["\uA289", "\uA289"], /* YI SYLLABLE WOP */
  ["\uA28A", "\uA28A"], /* YI SYLLABLE WEX */
  ["\uA28B", "\uA28B"], /* YI SYLLABLE WE */
  ["\uA28C", "\uA28C"], /* YI SYLLABLE WEP */
  ["\uA28D", "\uA28D"], /* YI SYLLABLE ZIT */
  ["\uA28E", "\uA28E"], /* YI SYLLABLE ZIX */
  ["\uA28F", "\uA28F"], /* YI SYLLABLE ZI */
  ["\uA290", "\uA290"], /* YI SYLLABLE ZIP */
  ["\uA291", "\uA291"], /* YI SYLLABLE ZIEX */
  ["\uA292", "\uA292"], /* YI SYLLABLE ZIE */
  ["\uA293", "\uA293"], /* YI SYLLABLE ZIEP */
  ["\uA294", "\uA294"], /* YI SYLLABLE ZAT */
  ["\uA295", "\uA295"], /* YI SYLLABLE ZAX */
  ["\uA296", "\uA296"], /* YI SYLLABLE ZA */
  ["\uA297", "\uA297"], /* YI SYLLABLE ZAP */
  ["\uA298", "\uA298"], /* YI SYLLABLE ZUOX */
  ["\uA299", "\uA299"], /* YI SYLLABLE ZUO */
  ["\uA29A", "\uA29A"], /* YI SYLLABLE ZUOP */
  ["\uA29B", "\uA29B"], /* YI SYLLABLE ZOT */
  ["\uA29C", "\uA29C"], /* YI SYLLABLE ZOX */
  ["\uA29D", "\uA29D"], /* YI SYLLABLE ZO */
  ["\uA29E", "\uA29E"], /* YI SYLLABLE ZOP */
  ["\uA29F", "\uA29F"], /* YI SYLLABLE ZEX */
  ["\uA2A0", "\uA2A0"], /* YI SYLLABLE ZE */
  ["\uA2A1", "\uA2A1"], /* YI SYLLABLE ZEP */
  ["\uA2A2", "\uA2A2"], /* YI SYLLABLE ZUT */
  ["\uA2A3", "\uA2A3"], /* YI SYLLABLE ZUX */
  ["\uA2A4", "\uA2A4"], /* YI SYLLABLE ZU */
  ["\uA2A5", "\uA2A5"], /* YI SYLLABLE ZUP */
  ["\uA2A6", "\uA2A6"], /* YI SYLLABLE ZURX */
  ["\uA2A7", "\uA2A7"], /* YI SYLLABLE ZUR */
  ["\uA2A8", "\uA2A8"], /* YI SYLLABLE ZYT */
  ["\uA2A9", "\uA2A9"], /* YI SYLLABLE ZYX */
  ["\uA2AA", "\uA2AA"], /* YI SYLLABLE ZY */
  ["\uA2AB", "\uA2AB"], /* YI SYLLABLE ZYP */
  ["\uA2AC", "\uA2AC"], /* YI SYLLABLE ZYRX */
  ["\uA2AD", "\uA2AD"], /* YI SYLLABLE ZYR */
  ["\uA2AE", "\uA2AE"], /* YI SYLLABLE CIT */
  ["\uA2AF", "\uA2AF"], /* YI SYLLABLE CIX */
  ["\uA2B0", "\uA2B0"], /* YI SYLLABLE CI */
  ["\uA2B1", "\uA2B1"], /* YI SYLLABLE CIP */
  ["\uA2B2", "\uA2B2"], /* YI SYLLABLE CIET */
  ["\uA2B3", "\uA2B3"], /* YI SYLLABLE CIEX */
  ["\uA2B4", "\uA2B4"], /* YI SYLLABLE CIE */
  ["\uA2B5", "\uA2B5"], /* YI SYLLABLE CIEP */
  ["\uA2B6", "\uA2B6"], /* YI SYLLABLE CAT */
  ["\uA2B7", "\uA2B7"], /* YI SYLLABLE CAX */
  ["\uA2B8", "\uA2B8"], /* YI SYLLABLE CA */
  ["\uA2B9", "\uA2B9"], /* YI SYLLABLE CAP */
  ["\uA2BA", "\uA2BA"], /* YI SYLLABLE CUOX */
  ["\uA2BB", "\uA2BB"], /* YI SYLLABLE CUO */
  ["\uA2BC", "\uA2BC"], /* YI SYLLABLE CUOP */
  ["\uA2BD", "\uA2BD"], /* YI SYLLABLE COT */
  ["\uA2BE", "\uA2BE"], /* YI SYLLABLE COX */
  ["\uA2BF", "\uA2BF"], /* YI SYLLABLE CO */
  ["\uA2C0", "\uA2C0"], /* YI SYLLABLE COP */
  ["\uA2C1", "\uA2C1"], /* YI SYLLABLE CEX */
  ["\uA2C2", "\uA2C2"], /* YI SYLLABLE CE */
  ["\uA2C3", "\uA2C3"], /* YI SYLLABLE CEP */
  ["\uA2C4", "\uA2C4"], /* YI SYLLABLE CUT */
  ["\uA2C5", "\uA2C5"], /* YI SYLLABLE CUX */
  ["\uA2C6", "\uA2C6"], /* YI SYLLABLE CU */
  ["\uA2C7", "\uA2C7"], /* YI SYLLABLE CUP */
  ["\uA2C8", "\uA2C8"], /* YI SYLLABLE CURX */
  ["\uA2C9", "\uA2C9"], /* YI SYLLABLE CUR */
  ["\uA2CA", "\uA2CA"], /* YI SYLLABLE CYT */
  ["\uA2CB", "\uA2CB"], /* YI SYLLABLE CYX */
  ["\uA2CC", "\uA2CC"], /* YI SYLLABLE CY */
  ["\uA2CD", "\uA2CD"], /* YI SYLLABLE CYP */
  ["\uA2CE", "\uA2CE"], /* YI SYLLABLE CYRX */
  ["\uA2CF", "\uA2CF"], /* YI SYLLABLE CYR */
  ["\uA2D0", "\uA2D0"], /* YI SYLLABLE ZZIT */
  ["\uA2D1", "\uA2D1"], /* YI SYLLABLE ZZIX */
  ["\uA2D2", "\uA2D2"], /* YI SYLLABLE ZZI */
  ["\uA2D3", "\uA2D3"], /* YI SYLLABLE ZZIP */
  ["\uA2D4", "\uA2D4"], /* YI SYLLABLE ZZIET */
  ["\uA2D5", "\uA2D5"], /* YI SYLLABLE ZZIEX */
  ["\uA2D6", "\uA2D6"], /* YI SYLLABLE ZZIE */
  ["\uA2D7", "\uA2D7"], /* YI SYLLABLE ZZIEP */
  ["\uA2D8", "\uA2D8"], /* YI SYLLABLE ZZAT */
  ["\uA2D9", "\uA2D9"], /* YI SYLLABLE ZZAX */
  ["\uA2DA", "\uA2DA"], /* YI SYLLABLE ZZA */
  ["\uA2DB", "\uA2DB"], /* YI SYLLABLE ZZAP */
  ["\uA2DC", "\uA2DC"], /* YI SYLLABLE ZZOX */
  ["\uA2DD", "\uA2DD"], /* YI SYLLABLE ZZO */
  ["\uA2DE", "\uA2DE"], /* YI SYLLABLE ZZOP */
  ["\uA2DF", "\uA2DF"], /* YI SYLLABLE ZZEX */
  ["\uA2E0", "\uA2E0"], /* YI SYLLABLE ZZE */
  ["\uA2E1", "\uA2E1"], /* YI SYLLABLE ZZEP */
  ["\uA2E2", "\uA2E2"], /* YI SYLLABLE ZZUX */
  ["\uA2E3", "\uA2E3"], /* YI SYLLABLE ZZU */
  ["\uA2E4", "\uA2E4"], /* YI SYLLABLE ZZUP */
  ["\uA2E5", "\uA2E5"], /* YI SYLLABLE ZZURX */
  ["\uA2E6", "\uA2E6"], /* YI SYLLABLE ZZUR */
  ["\uA2E7", "\uA2E7"], /* YI SYLLABLE ZZYT */
  ["\uA2E8", "\uA2E8"], /* YI SYLLABLE ZZYX */
  ["\uA2E9", "\uA2E9"], /* YI SYLLABLE ZZY */
  ["\uA2EA", "\uA2EA"], /* YI SYLLABLE ZZYP */
  ["\uA2EB", "\uA2EB"], /* YI SYLLABLE ZZYRX */
  ["\uA2EC", "\uA2EC"], /* YI SYLLABLE ZZYR */
  ["\uA2ED", "\uA2ED"], /* YI SYLLABLE NZIT */
  ["\uA2EE", "\uA2EE"], /* YI SYLLABLE NZIX */
  ["\uA2EF", "\uA2EF"], /* YI SYLLABLE NZI */
  ["\uA2F0", "\uA2F0"], /* YI SYLLABLE NZIP */
  ["\uA2F1", "\uA2F1"], /* YI SYLLABLE NZIEX */
  ["\uA2F2", "\uA2F2"], /* YI SYLLABLE NZIE */
  ["\uA2F3", "\uA2F3"], /* YI SYLLABLE NZIEP */
  ["\uA2F4", "\uA2F4"], /* YI SYLLABLE NZAT */
  ["\uA2F5", "\uA2F5"], /* YI SYLLABLE NZAX */
  ["\uA2F6", "\uA2F6"], /* YI SYLLABLE NZA */
  ["\uA2F7", "\uA2F7"], /* YI SYLLABLE NZAP */
  ["\uA2F8", "\uA2F8"], /* YI SYLLABLE NZUOX */
  ["\uA2F9", "\uA2F9"], /* YI SYLLABLE NZUO */
  ["\uA2FA", "\uA2FA"], /* YI SYLLABLE NZOX */
  ["\uA2FB", "\uA2FB"], /* YI SYLLABLE NZOP */
  ["\uA2FC", "\uA2FC"], /* YI SYLLABLE NZEX */
  ["\uA2FD", "\uA2FD"], /* YI SYLLABLE NZE */
  ["\uA2FE", "\uA2FE"], /* YI SYLLABLE NZUX */
  ["\uA2FF", "\uA2FF"], /* YI SYLLABLE NZU */
  ["\uA300", "\uA300"], /* YI SYLLABLE NZUP */
  ["\uA301", "\uA301"], /* YI SYLLABLE NZURX */
  ["\uA302", "\uA302"], /* YI SYLLABLE NZUR */
  ["\uA303", "\uA303"], /* YI SYLLABLE NZYT */
  ["\uA304", "\uA304"], /* YI SYLLABLE NZYX */
  ["\uA305", "\uA305"], /* YI SYLLABLE NZY */
  ["\uA306", "\uA306"], /* YI SYLLABLE NZYP */
  ["\uA307", "\uA307"], /* YI SYLLABLE NZYRX */
  ["\uA308", "\uA308"], /* YI SYLLABLE NZYR */
  ["\uA309", "\uA309"], /* YI SYLLABLE SIT */
  ["\uA30A", "\uA30A"], /* YI SYLLABLE SIX */
  ["\uA30B", "\uA30B"], /* YI SYLLABLE SI */
  ["\uA30C", "\uA30C"], /* YI SYLLABLE SIP */
  ["\uA30D", "\uA30D"], /* YI SYLLABLE SIEX */
  ["\uA30E", "\uA30E"], /* YI SYLLABLE SIE */
  ["\uA30F", "\uA30F"], /* YI SYLLABLE SIEP */
  ["\uA310", "\uA310"], /* YI SYLLABLE SAT */
  ["\uA311", "\uA311"], /* YI SYLLABLE SAX */
  ["\uA312", "\uA312"], /* YI SYLLABLE SA */
  ["\uA313", "\uA313"], /* YI SYLLABLE SAP */
  ["\uA314", "\uA314"], /* YI SYLLABLE SUOX */
  ["\uA315", "\uA315"], /* YI SYLLABLE SUO */
  ["\uA316", "\uA316"], /* YI SYLLABLE SUOP */
  ["\uA317", "\uA317"], /* YI SYLLABLE SOT */
  ["\uA318", "\uA318"], /* YI SYLLABLE SOX */
  ["\uA319", "\uA319"], /* YI SYLLABLE SO */
  ["\uA31A", "\uA31A"], /* YI SYLLABLE SOP */
  ["\uA31B", "\uA31B"], /* YI SYLLABLE SEX */
  ["\uA31C", "\uA31C"], /* YI SYLLABLE SE */
  ["\uA31D", "\uA31D"], /* YI SYLLABLE SEP */
  ["\uA31E", "\uA31E"], /* YI SYLLABLE SUT */
  ["\uA31F", "\uA31F"], /* YI SYLLABLE SUX */
  ["\uA320", "\uA320"], /* YI SYLLABLE SU */
  ["\uA321", "\uA321"], /* YI SYLLABLE SUP */
  ["\uA322", "\uA322"], /* YI SYLLABLE SURX */
  ["\uA323", "\uA323"], /* YI SYLLABLE SUR */
  ["\uA324", "\uA324"], /* YI SYLLABLE SYT */
  ["\uA325", "\uA325"], /* YI SYLLABLE SYX */
  ["\uA326", "\uA326"], /* YI SYLLABLE SY */
  ["\uA327", "\uA327"], /* YI SYLLABLE SYP */
  ["\uA328", "\uA328"], /* YI SYLLABLE SYRX */
  ["\uA329", "\uA329"], /* YI SYLLABLE SYR */
  ["\uA32A", "\uA32A"], /* YI SYLLABLE SSIT */
  ["\uA32B", "\uA32B"], /* YI SYLLABLE SSIX */
  ["\uA32C", "\uA32C"], /* YI SYLLABLE SSI */
  ["\uA32D", "\uA32D"], /* YI SYLLABLE SSIP */
  ["\uA32E", "\uA32E"], /* YI SYLLABLE SSIEX */
  ["\uA32F", "\uA32F"], /* YI SYLLABLE SSIE */
  ["\uA330", "\uA330"], /* YI SYLLABLE SSIEP */
  ["\uA331", "\uA331"], /* YI SYLLABLE SSAT */
  ["\uA332", "\uA332"], /* YI SYLLABLE SSAX */
  ["\uA333", "\uA333"], /* YI SYLLABLE SSA */
  ["\uA334", "\uA334"], /* YI SYLLABLE SSAP */
  ["\uA335", "\uA335"], /* YI SYLLABLE SSOT */
  ["\uA336", "\uA336"], /* YI SYLLABLE SSOX */
  ["\uA337", "\uA337"], /* YI SYLLABLE SSO */
  ["\uA338", "\uA338"], /* YI SYLLABLE SSOP */
  ["\uA339", "\uA339"], /* YI SYLLABLE SSEX */
  ["\uA33A", "\uA33A"], /* YI SYLLABLE SSE */
  ["\uA33B", "\uA33B"], /* YI SYLLABLE SSEP */
  ["\uA33C", "\uA33C"], /* YI SYLLABLE SSUT */
  ["\uA33D", "\uA33D"], /* YI SYLLABLE SSUX */
  ["\uA33E", "\uA33E"], /* YI SYLLABLE SSU */
  ["\uA33F", "\uA33F"], /* YI SYLLABLE SSUP */
  ["\uA340", "\uA340"], /* YI SYLLABLE SSYT */
  ["\uA341", "\uA341"], /* YI SYLLABLE SSYX */
  ["\uA342", "\uA342"], /* YI SYLLABLE SSY */
  ["\uA343", "\uA343"], /* YI SYLLABLE SSYP */
  ["\uA344", "\uA344"], /* YI SYLLABLE SSYRX */
  ["\uA345", "\uA345"], /* YI SYLLABLE SSYR */
  ["\uA346", "\uA346"], /* YI SYLLABLE ZHAT */
  ["\uA347", "\uA347"], /* YI SYLLABLE ZHAX */
  ["\uA348", "\uA348"], /* YI SYLLABLE ZHA */
  ["\uA349", "\uA349"], /* YI SYLLABLE ZHAP */
  ["\uA34A", "\uA34A"], /* YI SYLLABLE ZHUOX */
  ["\uA34B", "\uA34B"], /* YI SYLLABLE ZHUO */
  ["\uA34C", "\uA34C"], /* YI SYLLABLE ZHUOP */
  ["\uA34D", "\uA34D"], /* YI SYLLABLE ZHOT */
  ["\uA34E", "\uA34E"], /* YI SYLLABLE ZHOX */
  ["\uA34F", "\uA34F"], /* YI SYLLABLE ZHO */
  ["\uA350", "\uA350"], /* YI SYLLABLE ZHOP */
  ["\uA351", "\uA351"], /* YI SYLLABLE ZHET */
  ["\uA352", "\uA352"], /* YI SYLLABLE ZHEX */
  ["\uA353", "\uA353"], /* YI SYLLABLE ZHE */
  ["\uA354", "\uA354"], /* YI SYLLABLE ZHEP */
  ["\uA355", "\uA355"], /* YI SYLLABLE ZHUT */
  ["\uA356", "\uA356"], /* YI SYLLABLE ZHUX */
  ["\uA357", "\uA357"], /* YI SYLLABLE ZHU */
  ["\uA358", "\uA358"], /* YI SYLLABLE ZHUP */
  ["\uA359", "\uA359"], /* YI SYLLABLE ZHURX */
  ["\uA35A", "\uA35A"], /* YI SYLLABLE ZHUR */
  ["\uA35B", "\uA35B"], /* YI SYLLABLE ZHYT */
  ["\uA35C", "\uA35C"], /* YI SYLLABLE ZHYX */
  ["\uA35D", "\uA35D"], /* YI SYLLABLE ZHY */
  ["\uA35E", "\uA35E"], /* YI SYLLABLE ZHYP */
  ["\uA35F", "\uA35F"], /* YI SYLLABLE ZHYRX */
  ["\uA360", "\uA360"], /* YI SYLLABLE ZHYR */
  ["\uA361", "\uA361"], /* YI SYLLABLE CHAT */
  ["\uA362", "\uA362"], /* YI SYLLABLE CHAX */
  ["\uA363", "\uA363"], /* YI SYLLABLE CHA */
  ["\uA364", "\uA364"], /* YI SYLLABLE CHAP */
  ["\uA365", "\uA365"], /* YI SYLLABLE CHUOT */
  ["\uA366", "\uA366"], /* YI SYLLABLE CHUOX */
  ["\uA367", "\uA367"], /* YI SYLLABLE CHUO */
  ["\uA368", "\uA368"], /* YI SYLLABLE CHUOP */
  ["\uA369", "\uA369"], /* YI SYLLABLE CHOT */
  ["\uA36A", "\uA36A"], /* YI SYLLABLE CHOX */
  ["\uA36B", "\uA36B"], /* YI SYLLABLE CHO */
  ["\uA36C", "\uA36C"], /* YI SYLLABLE CHOP */
  ["\uA36D", "\uA36D"], /* YI SYLLABLE CHET */
  ["\uA36E", "\uA36E"], /* YI SYLLABLE CHEX */
  ["\uA36F", "\uA36F"], /* YI SYLLABLE CHE */
  ["\uA370", "\uA370"], /* YI SYLLABLE CHEP */
  ["\uA371", "\uA371"], /* YI SYLLABLE CHUX */
  ["\uA372", "\uA372"], /* YI SYLLABLE CHU */
  ["\uA373", "\uA373"], /* YI SYLLABLE CHUP */
  ["\uA374", "\uA374"], /* YI SYLLABLE CHURX */
  ["\uA375", "\uA375"], /* YI SYLLABLE CHUR */
  ["\uA376", "\uA376"], /* YI SYLLABLE CHYT */
  ["\uA377", "\uA377"], /* YI SYLLABLE CHYX */
  ["\uA378", "\uA378"], /* YI SYLLABLE CHY */
  ["\uA379", "\uA379"], /* YI SYLLABLE CHYP */
  ["\uA37A", "\uA37A"], /* YI SYLLABLE CHYRX */
  ["\uA37B", "\uA37B"], /* YI SYLLABLE CHYR */
  ["\uA37C", "\uA37C"], /* YI SYLLABLE RRAX */
  ["\uA37D", "\uA37D"], /* YI SYLLABLE RRA */
  ["\uA37E", "\uA37E"], /* YI SYLLABLE RRUOX */
  ["\uA37F", "\uA37F"], /* YI SYLLABLE RRUO */
  ["\uA380", "\uA380"], /* YI SYLLABLE RROT */
  ["\uA381", "\uA381"], /* YI SYLLABLE RROX */
  ["\uA382", "\uA382"], /* YI SYLLABLE RRO */
  ["\uA383", "\uA383"], /* YI SYLLABLE RROP */
  ["\uA384", "\uA384"], /* YI SYLLABLE RRET */
  ["\uA385", "\uA385"], /* YI SYLLABLE RREX */
  ["\uA386", "\uA386"], /* YI SYLLABLE RRE */
  ["\uA387", "\uA387"], /* YI SYLLABLE RREP */
  ["\uA388", "\uA388"], /* YI SYLLABLE RRUT */
  ["\uA389", "\uA389"], /* YI SYLLABLE RRUX */
  ["\uA38A", "\uA38A"], /* YI SYLLABLE RRU */
  ["\uA38B", "\uA38B"], /* YI SYLLABLE RRUP */
  ["\uA38C", "\uA38C"], /* YI SYLLABLE RRURX */
  ["\uA38D", "\uA38D"], /* YI SYLLABLE RRUR */
  ["\uA38E", "\uA38E"], /* YI SYLLABLE RRYT */
  ["\uA38F", "\uA38F"], /* YI SYLLABLE RRYX */
  ["\uA390", "\uA390"], /* YI SYLLABLE RRY */
  ["\uA391", "\uA391"], /* YI SYLLABLE RRYP */
  ["\uA392", "\uA392"], /* YI SYLLABLE RRYRX */
  ["\uA393", "\uA393"], /* YI SYLLABLE RRYR */
  ["\uA394", "\uA394"], /* YI SYLLABLE NRAT */
  ["\uA395", "\uA395"], /* YI SYLLABLE NRAX */
  ["\uA396", "\uA396"], /* YI SYLLABLE NRA */
  ["\uA397", "\uA397"], /* YI SYLLABLE NRAP */
  ["\uA398", "\uA398"], /* YI SYLLABLE NROX */
  ["\uA399", "\uA399"], /* YI SYLLABLE NRO */
  ["\uA39A", "\uA39A"], /* YI SYLLABLE NROP */
  ["\uA39B", "\uA39B"], /* YI SYLLABLE NRET */
  ["\uA39C", "\uA39C"], /* YI SYLLABLE NREX */
  ["\uA39D", "\uA39D"], /* YI SYLLABLE NRE */
  ["\uA39E", "\uA39E"], /* YI SYLLABLE NREP */
  ["\uA39F", "\uA39F"], /* YI SYLLABLE NRUT */
  ["\uA3A0", "\uA3A0"], /* YI SYLLABLE NRUX */
  ["\uA3A1", "\uA3A1"], /* YI SYLLABLE NRU */
  ["\uA3A2", "\uA3A2"], /* YI SYLLABLE NRUP */
  ["\uA3A3", "\uA3A3"], /* YI SYLLABLE NRURX */
  ["\uA3A4", "\uA3A4"], /* YI SYLLABLE NRUR */
  ["\uA3A5", "\uA3A5"], /* YI SYLLABLE NRYT */
  ["\uA3A6", "\uA3A6"], /* YI SYLLABLE NRYX */
  ["\uA3A7", "\uA3A7"], /* YI SYLLABLE NRY */
  ["\uA3A8", "\uA3A8"], /* YI SYLLABLE NRYP */
  ["\uA3A9", "\uA3A9"], /* YI SYLLABLE NRYRX */
  ["\uA3AA", "\uA3AA"], /* YI SYLLABLE NRYR */
  ["\uA3AB", "\uA3AB"], /* YI SYLLABLE SHAT */
  ["\uA3AC", "\uA3AC"], /* YI SYLLABLE SHAX */
  ["\uA3AD", "\uA3AD"], /* YI SYLLABLE SHA */
  ["\uA3AE", "\uA3AE"], /* YI SYLLABLE SHAP */
  ["\uA3AF", "\uA3AF"], /* YI SYLLABLE SHUOX */
  ["\uA3B0", "\uA3B0"], /* YI SYLLABLE SHUO */
  ["\uA3B1", "\uA3B1"], /* YI SYLLABLE SHUOP */
  ["\uA3B2", "\uA3B2"], /* YI SYLLABLE SHOT */
  ["\uA3B3", "\uA3B3"], /* YI SYLLABLE SHOX */
  ["\uA3B4", "\uA3B4"], /* YI SYLLABLE SHO */
  ["\uA3B5", "\uA3B5"], /* YI SYLLABLE SHOP */
  ["\uA3B6", "\uA3B6"], /* YI SYLLABLE SHET */
  ["\uA3B7", "\uA3B7"], /* YI SYLLABLE SHEX */
  ["\uA3B8", "\uA3B8"], /* YI SYLLABLE SHE */
  ["\uA3B9", "\uA3B9"], /* YI SYLLABLE SHEP */
  ["\uA3BA", "\uA3BA"], /* YI SYLLABLE SHUT */
  ["\uA3BB", "\uA3BB"], /* YI SYLLABLE SHUX */
  ["\uA3BC", "\uA3BC"], /* YI SYLLABLE SHU */
  ["\uA3BD", "\uA3BD"], /* YI SYLLABLE SHUP */
  ["\uA3BE", "\uA3BE"], /* YI SYLLABLE SHURX */
  ["\uA3BF", "\uA3BF"], /* YI SYLLABLE SHUR */
  ["\uA3C0", "\uA3C0"], /* YI SYLLABLE SHYT */
  ["\uA3C1", "\uA3C1"], /* YI SYLLABLE SHYX */
  ["\uA3C2", "\uA3C2"], /* YI SYLLABLE SHY */
  ["\uA3C3", "\uA3C3"], /* YI SYLLABLE SHYP */
  ["\uA3C4", "\uA3C4"], /* YI SYLLABLE SHYRX */
  ["\uA3C5", "\uA3C5"], /* YI SYLLABLE SHYR */
  ["\uA3C6", "\uA3C6"], /* YI SYLLABLE RAT */
  ["\uA3C7", "\uA3C7"], /* YI SYLLABLE RAX */
  ["\uA3C8", "\uA3C8"], /* YI SYLLABLE RA */
  ["\uA3C9", "\uA3C9"], /* YI SYLLABLE RAP */
  ["\uA3CA", "\uA3CA"], /* YI SYLLABLE RUOX */
  ["\uA3CB", "\uA3CB"], /* YI SYLLABLE RUO */
  ["\uA3CC", "\uA3CC"], /* YI SYLLABLE RUOP */
  ["\uA3CD", "\uA3CD"], /* YI SYLLABLE ROT */
  ["\uA3CE", "\uA3CE"], /* YI SYLLABLE ROX */
  ["\uA3CF", "\uA3CF"], /* YI SYLLABLE RO */
  ["\uA3D0", "\uA3D0"], /* YI SYLLABLE ROP */
  ["\uA3D1", "\uA3D1"], /* YI SYLLABLE REX */
  ["\uA3D2", "\uA3D2"], /* YI SYLLABLE RE */
  ["\uA3D3", "\uA3D3"], /* YI SYLLABLE REP */
  ["\uA3D4", "\uA3D4"], /* YI SYLLABLE RUT */
  ["\uA3D5", "\uA3D5"], /* YI SYLLABLE RUX */
  ["\uA3D6", "\uA3D6"], /* YI SYLLABLE RU */
  ["\uA3D7", "\uA3D7"], /* YI SYLLABLE RUP */
  ["\uA3D8", "\uA3D8"], /* YI SYLLABLE RURX */
  ["\uA3D9", "\uA3D9"], /* YI SYLLABLE RUR */
  ["\uA3DA", "\uA3DA"], /* YI SYLLABLE RYT */
  ["\uA3DB", "\uA3DB"], /* YI SYLLABLE RYX */
  ["\uA3DC", "\uA3DC"], /* YI SYLLABLE RY */
  ["\uA3DD", "\uA3DD"], /* YI SYLLABLE RYP */
  ["\uA3DE", "\uA3DE"], /* YI SYLLABLE RYRX */
  ["\uA3DF", "\uA3DF"], /* YI SYLLABLE RYR */
  ["\uA3E0", "\uA3E0"], /* YI SYLLABLE JIT */
  ["\uA3E1", "\uA3E1"], /* YI SYLLABLE JIX */
  ["\uA3E2", "\uA3E2"], /* YI SYLLABLE JI */
  ["\uA3E3", "\uA3E3"], /* YI SYLLABLE JIP */
  ["\uA3E4", "\uA3E4"], /* YI SYLLABLE JIET */
  ["\uA3E5", "\uA3E5"], /* YI SYLLABLE JIEX */
  ["\uA3E6", "\uA3E6"], /* YI SYLLABLE JIE */
  ["\uA3E7", "\uA3E7"], /* YI SYLLABLE JIEP */
  ["\uA3E8", "\uA3E8"], /* YI SYLLABLE JUOT */
  ["\uA3E9", "\uA3E9"], /* YI SYLLABLE JUOX */
  ["\uA3EA", "\uA3EA"], /* YI SYLLABLE JUO */
  ["\uA3EB", "\uA3EB"], /* YI SYLLABLE JUOP */
  ["\uA3EC", "\uA3EC"], /* YI SYLLABLE JOT */
  ["\uA3ED", "\uA3ED"], /* YI SYLLABLE JOX */
  ["\uA3EE", "\uA3EE"], /* YI SYLLABLE JO */
  ["\uA3EF", "\uA3EF"], /* YI SYLLABLE JOP */
  ["\uA3F0", "\uA3F0"], /* YI SYLLABLE JUT */
  ["\uA3F1", "\uA3F1"], /* YI SYLLABLE JUX */
  ["\uA3F2", "\uA3F2"], /* YI SYLLABLE JU */
  ["\uA3F3", "\uA3F3"], /* YI SYLLABLE JUP */
  ["\uA3F4", "\uA3F4"], /* YI SYLLABLE JURX */
  ["\uA3F5", "\uA3F5"], /* YI SYLLABLE JUR */
  ["\uA3F6", "\uA3F6"], /* YI SYLLABLE JYT */
  ["\uA3F7", "\uA3F7"], /* YI SYLLABLE JYX */
  ["\uA3F8", "\uA3F8"], /* YI SYLLABLE JY */
  ["\uA3F9", "\uA3F9"], /* YI SYLLABLE JYP */
  ["\uA3FA", "\uA3FA"], /* YI SYLLABLE JYRX */
  ["\uA3FB", "\uA3FB"], /* YI SYLLABLE JYR */
  ["\uA3FC", "\uA3FC"], /* YI SYLLABLE QIT */
  ["\uA3FD", "\uA3FD"], /* YI SYLLABLE QIX */
  ["\uA3FE", "\uA3FE"], /* YI SYLLABLE QI */
  ["\uA3FF", "\uA3FF"], /* YI SYLLABLE QIP */
  ["\uA400", "\uA400"], /* YI SYLLABLE QIET */
  ["\uA401", "\uA401"], /* YI SYLLABLE QIEX */
  ["\uA402", "\uA402"], /* YI SYLLABLE QIE */
  ["\uA403", "\uA403"], /* YI SYLLABLE QIEP */
  ["\uA404", "\uA404"], /* YI SYLLABLE QUOT */
  ["\uA405", "\uA405"], /* YI SYLLABLE QUOX */
  ["\uA406", "\uA406"], /* YI SYLLABLE QUO */
  ["\uA407", "\uA407"], /* YI SYLLABLE QUOP */
  ["\uA408", "\uA408"], /* YI SYLLABLE QOT */
  ["\uA409", "\uA409"], /* YI SYLLABLE QOX */
  ["\uA40A", "\uA40A"], /* YI SYLLABLE QO */
  ["\uA40B", "\uA40B"], /* YI SYLLABLE QOP */
  ["\uA40C", "\uA40C"], /* YI SYLLABLE QUT */
  ["\uA40D", "\uA40D"], /* YI SYLLABLE QUX */
  ["\uA40E", "\uA40E"], /* YI SYLLABLE QU */
  ["\uA40F", "\uA40F"], /* YI SYLLABLE QUP */
  ["\uA410", "\uA410"], /* YI SYLLABLE QURX */
  ["\uA411", "\uA411"], /* YI SYLLABLE QUR */
  ["\uA412", "\uA412"], /* YI SYLLABLE QYT */
  ["\uA413", "\uA413"], /* YI SYLLABLE QYX */
  ["\uA414", "\uA414"], /* YI SYLLABLE QY */
  ["\uA415", "\uA415"], /* YI SYLLABLE QYP */
  ["\uA416", "\uA416"], /* YI SYLLABLE QYRX */
  ["\uA417", "\uA417"], /* YI SYLLABLE QYR */
  ["\uA418", "\uA418"], /* YI SYLLABLE JJIT */
  ["\uA419", "\uA419"], /* YI SYLLABLE JJIX */
  ["\uA41A", "\uA41A"], /* YI SYLLABLE JJI */
  ["\uA41B", "\uA41B"], /* YI SYLLABLE JJIP */
  ["\uA41C", "\uA41C"], /* YI SYLLABLE JJIET */
  ["\uA41D", "\uA41D"], /* YI SYLLABLE JJIEX */
  ["\uA41E", "\uA41E"], /* YI SYLLABLE JJIE */
  ["\uA41F", "\uA41F"], /* YI SYLLABLE JJIEP */
  ["\uA420", "\uA420"], /* YI SYLLABLE JJUOX */
  ["\uA421", "\uA421"], /* YI SYLLABLE JJUO */
  ["\uA422", "\uA422"], /* YI SYLLABLE JJUOP */
  ["\uA423", "\uA423"], /* YI SYLLABLE JJOT */
  ["\uA424", "\uA424"], /* YI SYLLABLE JJOX */
  ["\uA425", "\uA425"], /* YI SYLLABLE JJO */
  ["\uA426", "\uA426"], /* YI SYLLABLE JJOP */
  ["\uA427", "\uA427"], /* YI SYLLABLE JJUT */
  ["\uA428", "\uA428"], /* YI SYLLABLE JJUX */
  ["\uA429", "\uA429"], /* YI SYLLABLE JJU */
  ["\uA42A", "\uA42A"], /* YI SYLLABLE JJUP */
  ["\uA42B", "\uA42B"], /* YI SYLLABLE JJURX */
  ["\uA42C", "\uA42C"], /* YI SYLLABLE JJUR */
  ["\uA42D", "\uA42D"], /* YI SYLLABLE JJYT */
  ["\uA42E", "\uA42E"], /* YI SYLLABLE JJYX */
  ["\uA42F", "\uA42F"], /* YI SYLLABLE JJY */
  ["\uA430", "\uA430"], /* YI SYLLABLE JJYP */
  ["\uA431", "\uA431"], /* YI SYLLABLE NJIT */
  ["\uA432", "\uA432"], /* YI SYLLABLE NJIX */
  ["\uA433", "\uA433"], /* YI SYLLABLE NJI */
  ["\uA434", "\uA434"], /* YI SYLLABLE NJIP */
  ["\uA435", "\uA435"], /* YI SYLLABLE NJIET */
  ["\uA436", "\uA436"], /* YI SYLLABLE NJIEX */
  ["\uA437", "\uA437"], /* YI SYLLABLE NJIE */
  ["\uA438", "\uA438"], /* YI SYLLABLE NJIEP */
  ["\uA439", "\uA439"], /* YI SYLLABLE NJUOX */
  ["\uA43A", "\uA43A"], /* YI SYLLABLE NJUO */
  ["\uA43B", "\uA43B"], /* YI SYLLABLE NJOT */
  ["\uA43C", "\uA43C"], /* YI SYLLABLE NJOX */
  ["\uA43D", "\uA43D"], /* YI SYLLABLE NJO */
  ["\uA43E", "\uA43E"], /* YI SYLLABLE NJOP */
  ["\uA43F", "\uA43F"], /* YI SYLLABLE NJUX */
  ["\uA440", "\uA440"], /* YI SYLLABLE NJU */
  ["\uA441", "\uA441"], /* YI SYLLABLE NJUP */
  ["\uA442", "\uA442"], /* YI SYLLABLE NJURX */
  ["\uA443", "\uA443"], /* YI SYLLABLE NJUR */
  ["\uA444", "\uA444"], /* YI SYLLABLE NJYT */
  ["\uA445", "\uA445"], /* YI SYLLABLE NJYX */
  ["\uA446", "\uA446"], /* YI SYLLABLE NJY */
  ["\uA447", "\uA447"], /* YI SYLLABLE NJYP */
  ["\uA448", "\uA448"], /* YI SYLLABLE NJYRX */
  ["\uA449", "\uA449"], /* YI SYLLABLE NJYR */
  ["\uA44A", "\uA44A"], /* YI SYLLABLE NYIT */
  ["\uA44B", "\uA44B"], /* YI SYLLABLE NYIX */
  ["\uA44C", "\uA44C"], /* YI SYLLABLE NYI */
  ["\uA44D", "\uA44D"], /* YI SYLLABLE NYIP */
  ["\uA44E", "\uA44E"], /* YI SYLLABLE NYIET */
  ["\uA44F", "\uA44F"], /* YI SYLLABLE NYIEX */
  ["\uA450", "\uA450"], /* YI SYLLABLE NYIE */
  ["\uA451", "\uA451"], /* YI SYLLABLE NYIEP */
  ["\uA452", "\uA452"], /* YI SYLLABLE NYUOX */
  ["\uA453", "\uA453"], /* YI SYLLABLE NYUO */
  ["\uA454", "\uA454"], /* YI SYLLABLE NYUOP */
  ["\uA455", "\uA455"], /* YI SYLLABLE NYOT */
  ["\uA456", "\uA456"], /* YI SYLLABLE NYOX */
  ["\uA457", "\uA457"], /* YI SYLLABLE NYO */
  ["\uA458", "\uA458"], /* YI SYLLABLE NYOP */
  ["\uA459", "\uA459"], /* YI SYLLABLE NYUT */
  ["\uA45A", "\uA45A"], /* YI SYLLABLE NYUX */
  ["\uA45B", "\uA45B"], /* YI SYLLABLE NYU */
  ["\uA45C", "\uA45C"], /* YI SYLLABLE NYUP */
  ["\uA45D", "\uA45D"], /* YI SYLLABLE XIT */
  ["\uA45E", "\uA45E"], /* YI SYLLABLE XIX */
  ["\uA45F", "\uA45F"], /* YI SYLLABLE XI */
  ["\uA460", "\uA460"], /* YI SYLLABLE XIP */
  ["\uA461", "\uA461"], /* YI SYLLABLE XIET */
  ["\uA462", "\uA462"], /* YI SYLLABLE XIEX */
  ["\uA463", "\uA463"], /* YI SYLLABLE XIE */
  ["\uA464", "\uA464"], /* YI SYLLABLE XIEP */
  ["\uA465", "\uA465"], /* YI SYLLABLE XUOX */
  ["\uA466", "\uA466"], /* YI SYLLABLE XUO */
  ["\uA467", "\uA467"], /* YI SYLLABLE XOT */
  ["\uA468", "\uA468"], /* YI SYLLABLE XOX */
  ["\uA469", "\uA469"], /* YI SYLLABLE XO */
  ["\uA46A", "\uA46A"], /* YI SYLLABLE XOP */
  ["\uA46B", "\uA46B"], /* YI SYLLABLE XYT */
  ["\uA46C", "\uA46C"], /* YI SYLLABLE XYX */
  ["\uA46D", "\uA46D"], /* YI SYLLABLE XY */
  ["\uA46E", "\uA46E"], /* YI SYLLABLE XYP */
  ["\uA46F", "\uA46F"], /* YI SYLLABLE XYRX */
  ["\uA470", "\uA470"], /* YI SYLLABLE XYR */
  ["\uA471", "\uA471"], /* YI SYLLABLE YIT */
  ["\uA472", "\uA472"], /* YI SYLLABLE YIX */
  ["\uA473", "\uA473"], /* YI SYLLABLE YI */
  ["\uA474", "\uA474"], /* YI SYLLABLE YIP */
  ["\uA475", "\uA475"], /* YI SYLLABLE YIET */
  ["\uA476", "\uA476"], /* YI SYLLABLE YIEX */
  ["\uA477", "\uA477"], /* YI SYLLABLE YIE */
  ["\uA478", "\uA478"], /* YI SYLLABLE YIEP */
  ["\uA479", "\uA479"], /* YI SYLLABLE YUOT */
  ["\uA47A", "\uA47A"], /* YI SYLLABLE YUOX */
  ["\uA47B", "\uA47B"], /* YI SYLLABLE YUO */
  ["\uA47C", "\uA47C"], /* YI SYLLABLE YUOP */
  ["\uA47D", "\uA47D"], /* YI SYLLABLE YOT */
  ["\uA47E", "\uA47E"], /* YI SYLLABLE YOX */
  ["\uA47F", "\uA47F"], /* YI SYLLABLE YO */
  ["\uA480", "\uA480"], /* YI SYLLABLE YOP */
  ["\uA481", "\uA481"], /* YI SYLLABLE YUT */
  ["\uA482", "\uA482"], /* YI SYLLABLE YUX */
  ["\uA483", "\uA483"], /* YI SYLLABLE YU */
  ["\uA484", "\uA484"], /* YI SYLLABLE YUP */
  ["\uA485", "\uA485"], /* YI SYLLABLE YURX */
  ["\uA486", "\uA486"], /* YI SYLLABLE YUR */
  ["\uA487", "\uA487"], /* YI SYLLABLE YYT */
  ["\uA488", "\uA488"], /* YI SYLLABLE YYX */
  ["\uA489", "\uA489"], /* YI SYLLABLE YY */
  ["\uA48A", "\uA48A"], /* YI SYLLABLE YYP */
  ["\uA48B", "\uA48B"], /* YI SYLLABLE YYRX */
  ["\uA48C", "\uA48C"], /* YI SYLLABLE YYR */
  ["\uA48D", "\uA48D"],
  ["\uA48E", "\uA48E"],
  ["\uA48F", "\uA48F"],
  ["\uA490", "\uA490"], /* YI RADICAL QOT */
  ["\uA491", "\uA491"], /* YI RADICAL LI */
  ["\uA492", "\uA492"], /* YI RADICAL KIT */
  ["\uA493", "\uA493"], /* YI RADICAL NYIP */
  ["\uA494", "\uA494"], /* YI RADICAL CYP */
  ["\uA495", "\uA495"], /* YI RADICAL SSI */
  ["\uA496", "\uA496"], /* YI RADICAL GGOP */
  ["\uA497", "\uA497"], /* YI RADICAL GEP */
  ["\uA498", "\uA498"], /* YI RADICAL MI */
  ["\uA499", "\uA499"], /* YI RADICAL HXIT */
  ["\uA49A", "\uA49A"], /* YI RADICAL LYR */
  ["\uA49B", "\uA49B"], /* YI RADICAL BBUT */
  ["\uA49C", "\uA49C"], /* YI RADICAL MOP */
  ["\uA49D", "\uA49D"], /* YI RADICAL YO */
  ["\uA49E", "\uA49E"], /* YI RADICAL PUT */
  ["\uA49F", "\uA49F"], /* YI RADICAL HXUO */
  ["\uA4A0", "\uA4A0"], /* YI RADICAL TAT */
  ["\uA4A1", "\uA4A1"], /* YI RADICAL GA */
  ["\uA4A2", "\uA4A2"], /* YI RADICAL ZUP */
  ["\uA4A3", "\uA4A3"], /* YI RADICAL CYT */
  ["\uA4A4", "\uA4A4"], /* YI RADICAL DDUR */
  ["\uA4A5", "\uA4A5"], /* YI RADICAL BUR */
  ["\uA4A6", "\uA4A6"], /* YI RADICAL GGUO */
  ["\uA4A7", "\uA4A7"], /* YI RADICAL NYOP */
  ["\uA4A8", "\uA4A8"], /* YI RADICAL TU */
  ["\uA4A9", "\uA4A9"], /* YI RADICAL OP */
  ["\uA4AA", "\uA4AA"], /* YI RADICAL JJUT */
  ["\uA4AB", "\uA4AB"], /* YI RADICAL ZOT */
  ["\uA4AC", "\uA4AC"], /* YI RADICAL PYT */
  ["\uA4AD", "\uA4AD"], /* YI RADICAL HMO */
  ["\uA4AE", "\uA4AE"], /* YI RADICAL YIT */
  ["\uA4AF", "\uA4AF"], /* YI RADICAL VUR */
  ["\uA4B0", "\uA4B0"], /* YI RADICAL SHY */
  ["\uA4B1", "\uA4B1"], /* YI RADICAL VEP */
  ["\uA4B2", "\uA4B2"], /* YI RADICAL ZA */
  ["\uA4B3", "\uA4B3"], /* YI RADICAL JO */
  ["\uA4B4", "\uA4B4"], /* YI RADICAL NZUP */
  ["\uA4B5", "\uA4B5"], /* YI RADICAL JJY */
  ["\uA4B6", "\uA4B6"], /* YI RADICAL GOT */
  ["\uA4B7", "\uA4B7"], /* YI RADICAL JJIE */
  ["\uA4B8", "\uA4B8"], /* YI RADICAL WO */
  ["\uA4B9", "\uA4B9"], /* YI RADICAL DU */
  ["\uA4BA", "\uA4BA"], /* YI RADICAL SHUR */
  ["\uA4BB", "\uA4BB"], /* YI RADICAL LIE */
  ["\uA4BC", "\uA4BC"], /* YI RADICAL CY */
  ["\uA4BD", "\uA4BD"], /* YI RADICAL CUOP */
  ["\uA4BE", "\uA4BE"], /* YI RADICAL CIP */
  ["\uA4BF", "\uA4BF"], /* YI RADICAL HXOP */
  ["\uA4C0", "\uA4C0"], /* YI RADICAL SHAT */
  ["\uA4C1", "\uA4C1"], /* YI RADICAL ZUR */
  ["\uA4C2", "\uA4C2"], /* YI RADICAL SHOP */
  ["\uA4C3", "\uA4C3"], /* YI RADICAL CHE */
  ["\uA4C4", "\uA4C4"], /* YI RADICAL ZZIET */
  ["\uA4C5", "\uA4C5"], /* YI RADICAL NBIE */
  ["\uA4C6", "\uA4C6"], /* YI RADICAL KE */
  ["\uA4C7", "\uA4C7"],
  ["\uA4C8", "\uA4C8"],
  ["\uA4C9", "\uA4C9"],
  ["\uA4CA", "\uA4CA"],
  ["\uA4CB", "\uA4CB"],
  ["\uA4CC", "\uA4CC"],
  ["\uA4CD", "\uA4CD"],
  ["\uA4CE", "\uA4CE"],
  ["\uA4CF", "\uA4CF"],
  ["\uA4D0", "\uA4D0"], /* LISU LETTER BA */
  ["\uA4D1", "\uA4D1"], /* LISU LETTER PA */
  ["\uA4D2", "\uA4D2"], /* LISU LETTER PHA */
  ["\uA4D3", "\uA4D3"], /* LISU LETTER DA */
  ["\uA4D4", "\uA4D4"], /* LISU LETTER TA */
  ["\uA4D5", "\uA4D5"], /* LISU LETTER THA */
  ["\uA4D6", "\uA4D6"], /* LISU LETTER GA */
  ["\uA4D7", "\uA4D7"], /* LISU LETTER KA */
  ["\uA4D8", "\uA4D8"], /* LISU LETTER KHA */
  ["\uA4D9", "\uA4D9"], /* LISU LETTER JA */
  ["\uA4DA", "\uA4DA"], /* LISU LETTER CA */
  ["\uA4DB", "\uA4DB"], /* LISU LETTER CHA */
  ["\uA4DC", "\uA4DC"], /* LISU LETTER DZA */
  ["\uA4DD", "\uA4DD"], /* LISU LETTER TSA */
  ["\uA4DE", "\uA4DE"], /* LISU LETTER TSHA */
  ["\uA4DF", "\uA4DF"], /* LISU LETTER MA */
  ["\uA4E0", "\uA4E0"], /* LISU LETTER NA */
  ["\uA4E1", "\uA4E1"], /* LISU LETTER LA */
  ["\uA4E2", "\uA4E2"], /* LISU LETTER SA */
  ["\uA4E3", "\uA4E3"], /* LISU LETTER ZHA */
  ["\uA4E4", "\uA4E4"], /* LISU LETTER ZA */
  ["\uA4E5", "\uA4E5"], /* LISU LETTER NGA */
  ["\uA4E6", "\uA4E6"], /* LISU LETTER HA */
  ["\uA4E7", "\uA4E7"], /* LISU LETTER XA */
  ["\uA4E8", "\uA4E8"], /* LISU LETTER HHA */
  ["\uA4E9", "\uA4E9"], /* LISU LETTER FA */
  ["\uA4EA", "\uA4EA"], /* LISU LETTER WA */
  ["\uA4EB", "\uA4EB"], /* LISU LETTER SHA */
  ["\uA4EC", "\uA4EC"], /* LISU LETTER YA */
  ["\uA4ED", "\uA4ED"], /* LISU LETTER GHA */
  ["\uA4EE", "\uA4EE"], /* LISU LETTER A */
  ["\uA4EF", "\uA4EF"], /* LISU LETTER AE */
  ["\uA4F0", "\uA4F0"], /* LISU LETTER E */
  ["\uA4F1", "\uA4F1"], /* LISU LETTER EU */
  ["\uA4F2", "\uA4F2"], /* LISU LETTER I */
  ["\uA4F3", "\uA4F3"], /* LISU LETTER O */
  ["\uA4F4", "\uA4F4"], /* LISU LETTER U */
  ["\uA4F5", "\uA4F5"], /* LISU LETTER UE */
  ["\uA4F6", "\uA4F6"], /* LISU LETTER UH */
  ["\uA4F7", "\uA4F7"], /* LISU LETTER OE */
  ["\uA4F8", "\uA4F8"], /* LISU LETTER TONE MYA TI */
  ["\uA4F9", "\uA4F9"], /* LISU LETTER TONE NA PO */
  ["\uA4FA", "\uA4FA"], /* LISU LETTER TONE MYA CYA */
  ["\uA4FB", "\uA4FB"], /* LISU LETTER TONE MYA BO */
  ["\uA4FC", "\uA4FC"], /* LISU LETTER TONE MYA NA */
  ["\uA4FD", "\uA4FD"], /* LISU LETTER TONE MYA JEU */
  ["\uA4FE", "\uA4FE"], /* LISU PUNCTUATION COMMA */
  ["\uA4FF", "\uA4FF"], /* LISU PUNCTUATION FULL STOP */
  ["\uA500", "\uA500"], /* VAI SYLLABLE EE */
  ["\uA501", "\uA501"], /* VAI SYLLABLE EEN */
  ["\uA502", "\uA502"], /* VAI SYLLABLE HEE */
  ["\uA503", "\uA503"], /* VAI SYLLABLE WEE */
  ["\uA504", "\uA504"], /* VAI SYLLABLE WEEN */
  ["\uA505", "\uA505"], /* VAI SYLLABLE PEE */
  ["\uA506", "\uA506"], /* VAI SYLLABLE BHEE */
  ["\uA507", "\uA507"], /* VAI SYLLABLE BEE */
  ["\uA508", "\uA508"], /* VAI SYLLABLE MBEE */
  ["\uA509", "\uA509"], /* VAI SYLLABLE KPEE */
  ["\uA50A", "\uA50A"], /* VAI SYLLABLE MGBEE */
  ["\uA50B", "\uA50B"], /* VAI SYLLABLE GBEE */
  ["\uA50C", "\uA50C"], /* VAI SYLLABLE FEE */
  ["\uA50D", "\uA50D"], /* VAI SYLLABLE VEE */
  ["\uA50E", "\uA50E"], /* VAI SYLLABLE TEE */
  ["\uA50F", "\uA50F"], /* VAI SYLLABLE THEE */
  ["\uA510", "\uA510"], /* VAI SYLLABLE DHEE */
  ["\uA511", "\uA511"], /* VAI SYLLABLE DHHEE */
  ["\uA512", "\uA512"], /* VAI SYLLABLE LEE */
  ["\uA513", "\uA513"], /* VAI SYLLABLE REE */
  ["\uA514", "\uA514"], /* VAI SYLLABLE DEE */
  ["\uA515", "\uA515"], /* VAI SYLLABLE NDEE */
  ["\uA516", "\uA516"], /* VAI SYLLABLE SEE */
  ["\uA517", "\uA517"], /* VAI SYLLABLE SHEE */
  ["\uA518", "\uA518"], /* VAI SYLLABLE ZEE */
  ["\uA519", "\uA519"], /* VAI SYLLABLE ZHEE */
  ["\uA51A", "\uA51A"], /* VAI SYLLABLE CEE */
  ["\uA51B", "\uA51B"], /* VAI SYLLABLE JEE */
  ["\uA51C", "\uA51C"], /* VAI SYLLABLE NJEE */
  ["\uA51D", "\uA51D"], /* VAI SYLLABLE YEE */
  ["\uA51E", "\uA51E"], /* VAI SYLLABLE KEE */
  ["\uA51F", "\uA51F"], /* VAI SYLLABLE NGGEE */
  ["\uA520", "\uA520"], /* VAI SYLLABLE GEE */
  ["\uA521", "\uA521"], /* VAI SYLLABLE MEE */
  ["\uA522", "\uA522"], /* VAI SYLLABLE NEE */
  ["\uA523", "\uA523"], /* VAI SYLLABLE NYEE */
  ["\uA524", "\uA524"], /* VAI SYLLABLE I */
  ["\uA525", "\uA525"], /* VAI SYLLABLE IN */
  ["\uA526", "\uA526"], /* VAI SYLLABLE HI */
  ["\uA527", "\uA527"], /* VAI SYLLABLE HIN */
  ["\uA528", "\uA528"], /* VAI SYLLABLE WI */
  ["\uA529", "\uA529"], /* VAI SYLLABLE WIN */
  ["\uA52A", "\uA52A"], /* VAI SYLLABLE PI */
  ["\uA52B", "\uA52B"], /* VAI SYLLABLE BHI */
  ["\uA52C", "\uA52C"], /* VAI SYLLABLE BI */
  ["\uA52D", "\uA52D"], /* VAI SYLLABLE MBI */
  ["\uA52E", "\uA52E"], /* VAI SYLLABLE KPI */
  ["\uA52F", "\uA52F"], /* VAI SYLLABLE MGBI */
  ["\uA530", "\uA530"], /* VAI SYLLABLE GBI */
  ["\uA531", "\uA531"], /* VAI SYLLABLE FI */
  ["\uA532", "\uA532"], /* VAI SYLLABLE VI */
  ["\uA533", "\uA533"], /* VAI SYLLABLE TI */
  ["\uA534", "\uA534"], /* VAI SYLLABLE THI */
  ["\uA535", "\uA535"], /* VAI SYLLABLE DHI */
  ["\uA536", "\uA536"], /* VAI SYLLABLE DHHI */
  ["\uA537", "\uA537"], /* VAI SYLLABLE LI */
  ["\uA538", "\uA538"], /* VAI SYLLABLE RI */
  ["\uA539", "\uA539"], /* VAI SYLLABLE DI */
  ["\uA53A", "\uA53A"], /* VAI SYLLABLE NDI */
  ["\uA53B", "\uA53B"], /* VAI SYLLABLE SI */
  ["\uA53C", "\uA53C"], /* VAI SYLLABLE SHI */
  ["\uA53D", "\uA53D"], /* VAI SYLLABLE ZI */
  ["\uA53E", "\uA53E"], /* VAI SYLLABLE ZHI */
  ["\uA53F", "\uA53F"], /* VAI SYLLABLE CI */
  ["\uA540", "\uA540"], /* VAI SYLLABLE JI */
  ["\uA541", "\uA541"], /* VAI SYLLABLE NJI */
  ["\uA542", "\uA542"], /* VAI SYLLABLE YI */
  ["\uA543", "\uA543"], /* VAI SYLLABLE KI */
  ["\uA544", "\uA544"], /* VAI SYLLABLE NGGI */
  ["\uA545", "\uA545"], /* VAI SYLLABLE GI */
  ["\uA546", "\uA546"], /* VAI SYLLABLE MI */
  ["\uA547", "\uA547"], /* VAI SYLLABLE NI */
  ["\uA548", "\uA548"], /* VAI SYLLABLE NYI */
  ["\uA549", "\uA549"], /* VAI SYLLABLE A */
  ["\uA54A", "\uA54A"], /* VAI SYLLABLE AN */
  ["\uA54B", "\uA54B"], /* VAI SYLLABLE NGAN */
  ["\uA54C", "\uA54C"], /* VAI SYLLABLE HA */
  ["\uA54D", "\uA54D"], /* VAI SYLLABLE HAN */
  ["\uA54E", "\uA54E"], /* VAI SYLLABLE WA */
  ["\uA54F", "\uA54F"], /* VAI SYLLABLE WAN */
  ["\uA550", "\uA550"], /* VAI SYLLABLE PA */
  ["\uA551", "\uA551"], /* VAI SYLLABLE BHA */
  ["\uA552", "\uA552"], /* VAI SYLLABLE BA */
  ["\uA553", "\uA553"], /* VAI SYLLABLE MBA */
  ["\uA554", "\uA554"], /* VAI SYLLABLE KPA */
  ["\uA555", "\uA555"], /* VAI SYLLABLE KPAN */
  ["\uA556", "\uA556"], /* VAI SYLLABLE MGBA */
  ["\uA557", "\uA557"], /* VAI SYLLABLE GBA */
  ["\uA558", "\uA558"], /* VAI SYLLABLE FA */
  ["\uA559", "\uA559"], /* VAI SYLLABLE VA */
  ["\uA55A", "\uA55A"], /* VAI SYLLABLE TA */
  ["\uA55B", "\uA55B"], /* VAI SYLLABLE THA */
  ["\uA55C", "\uA55C"], /* VAI SYLLABLE DHA */
  ["\uA55D", "\uA55D"], /* VAI SYLLABLE DHHA */
  ["\uA55E", "\uA55E"], /* VAI SYLLABLE LA */
  ["\uA55F", "\uA55F"], /* VAI SYLLABLE RA */
  ["\uA560", "\uA560"], /* VAI SYLLABLE DA */
  ["\uA561", "\uA561"], /* VAI SYLLABLE NDA */
  ["\uA562", "\uA562"], /* VAI SYLLABLE SA */
  ["\uA563", "\uA563"], /* VAI SYLLABLE SHA */
  ["\uA564", "\uA564"], /* VAI SYLLABLE ZA */
  ["\uA565", "\uA565"], /* VAI SYLLABLE ZHA */
  ["\uA566", "\uA566"], /* VAI SYLLABLE CA */
  ["\uA567", "\uA567"], /* VAI SYLLABLE JA */
  ["\uA568", "\uA568"], /* VAI SYLLABLE NJA */
  ["\uA569", "\uA569"], /* VAI SYLLABLE YA */
  ["\uA56A", "\uA56A"], /* VAI SYLLABLE KA */
  ["\uA56B", "\uA56B"], /* VAI SYLLABLE KAN */
  ["\uA56C", "\uA56C"], /* VAI SYLLABLE NGGA */
  ["\uA56D", "\uA56D"], /* VAI SYLLABLE GA */
  ["\uA56E", "\uA56E"], /* VAI SYLLABLE MA */
  ["\uA56F", "\uA56F"], /* VAI SYLLABLE NA */
  ["\uA570", "\uA570"], /* VAI SYLLABLE NYA */
  ["\uA571", "\uA571"], /* VAI SYLLABLE OO */
  ["\uA572", "\uA572"], /* VAI SYLLABLE OON */
  ["\uA573", "\uA573"], /* VAI SYLLABLE HOO */
  ["\uA574", "\uA574"], /* VAI SYLLABLE WOO */
  ["\uA575", "\uA575"], /* VAI SYLLABLE WOON */
  ["\uA576", "\uA576"], /* VAI SYLLABLE POO */
  ["\uA577", "\uA577"], /* VAI SYLLABLE BHOO */
  ["\uA578", "\uA578"], /* VAI SYLLABLE BOO */
  ["\uA579", "\uA579"], /* VAI SYLLABLE MBOO */
  ["\uA57A", "\uA57A"], /* VAI SYLLABLE KPOO */
  ["\uA57B", "\uA57B"], /* VAI SYLLABLE MGBOO */
  ["\uA57C", "\uA57C"], /* VAI SYLLABLE GBOO */
  ["\uA57D", "\uA57D"], /* VAI SYLLABLE FOO */
  ["\uA57E", "\uA57E"], /* VAI SYLLABLE VOO */
  ["\uA57F", "\uA57F"], /* VAI SYLLABLE TOO */
  ["\uA580", "\uA580"], /* VAI SYLLABLE THOO */
  ["\uA581", "\uA581"], /* VAI SYLLABLE DHOO */
  ["\uA582", "\uA582"], /* VAI SYLLABLE DHHOO */
  ["\uA583", "\uA583"], /* VAI SYLLABLE LOO */
  ["\uA584", "\uA584"], /* VAI SYLLABLE ROO */
  ["\uA585", "\uA585"], /* VAI SYLLABLE DOO */
  ["\uA586", "\uA586"], /* VAI SYLLABLE NDOO */
  ["\uA587", "\uA587"], /* VAI SYLLABLE SOO */
  ["\uA588", "\uA588"], /* VAI SYLLABLE SHOO */
  ["\uA589", "\uA589"], /* VAI SYLLABLE ZOO */
  ["\uA58A", "\uA58A"], /* VAI SYLLABLE ZHOO */
  ["\uA58B", "\uA58B"], /* VAI SYLLABLE COO */
  ["\uA58C", "\uA58C"], /* VAI SYLLABLE JOO */
  ["\uA58D", "\uA58D"], /* VAI SYLLABLE NJOO */
  ["\uA58E", "\uA58E"], /* VAI SYLLABLE YOO */
  ["\uA58F", "\uA58F"], /* VAI SYLLABLE KOO */
  ["\uA590", "\uA590"], /* VAI SYLLABLE NGGOO */
  ["\uA591", "\uA591"], /* VAI SYLLABLE GOO */
  ["\uA592", "\uA592"], /* VAI SYLLABLE MOO */
  ["\uA593", "\uA593"], /* VAI SYLLABLE NOO */
  ["\uA594", "\uA594"], /* VAI SYLLABLE NYOO */
  ["\uA595", "\uA595"], /* VAI SYLLABLE U */
  ["\uA596", "\uA596"], /* VAI SYLLABLE UN */
  ["\uA597", "\uA597"], /* VAI SYLLABLE HU */
  ["\uA598", "\uA598"], /* VAI SYLLABLE HUN */
  ["\uA599", "\uA599"], /* VAI SYLLABLE WU */
  ["\uA59A", "\uA59A"], /* VAI SYLLABLE WUN */
  ["\uA59B", "\uA59B"], /* VAI SYLLABLE PU */
  ["\uA59C", "\uA59C"], /* VAI SYLLABLE BHU */
  ["\uA59D", "\uA59D"], /* VAI SYLLABLE BU */
  ["\uA59E", "\uA59E"], /* VAI SYLLABLE MBU */
  ["\uA59F", "\uA59F"], /* VAI SYLLABLE KPU */
  ["\uA5A0", "\uA5A0"], /* VAI SYLLABLE MGBU */
  ["\uA5A1", "\uA5A1"], /* VAI SYLLABLE GBU */
  ["\uA5A2", "\uA5A2"], /* VAI SYLLABLE FU */
  ["\uA5A3", "\uA5A3"], /* VAI SYLLABLE VU */
  ["\uA5A4", "\uA5A4"], /* VAI SYLLABLE TU */
  ["\uA5A5", "\uA5A5"], /* VAI SYLLABLE THU */
  ["\uA5A6", "\uA5A6"], /* VAI SYLLABLE DHU */
  ["\uA5A7", "\uA5A7"], /* VAI SYLLABLE DHHU */
  ["\uA5A8", "\uA5A8"], /* VAI SYLLABLE LU */
  ["\uA5A9", "\uA5A9"], /* VAI SYLLABLE RU */
  ["\uA5AA", "\uA5AA"], /* VAI SYLLABLE DU */
  ["\uA5AB", "\uA5AB"], /* VAI SYLLABLE NDU */
  ["\uA5AC", "\uA5AC"], /* VAI SYLLABLE SU */
  ["\uA5AD", "\uA5AD"], /* VAI SYLLABLE SHU */
  ["\uA5AE", "\uA5AE"], /* VAI SYLLABLE ZU */
  ["\uA5AF", "\uA5AF"], /* VAI SYLLABLE ZHU */
  ["\uA5B0", "\uA5B0"], /* VAI SYLLABLE CU */
  ["\uA5B1", "\uA5B1"], /* VAI SYLLABLE JU */
  ["\uA5B2", "\uA5B2"], /* VAI SYLLABLE NJU */
  ["\uA5B3", "\uA5B3"], /* VAI SYLLABLE YU */
  ["\uA5B4", "\uA5B4"], /* VAI SYLLABLE KU */
  ["\uA5B5", "\uA5B5"], /* VAI SYLLABLE NGGU */
  ["\uA5B6", "\uA5B6"], /* VAI SYLLABLE GU */
  ["\uA5B7", "\uA5B7"], /* VAI SYLLABLE MU */
  ["\uA5B8", "\uA5B8"], /* VAI SYLLABLE NU */
  ["\uA5B9", "\uA5B9"], /* VAI SYLLABLE NYU */
  ["\uA5BA", "\uA5BA"], /* VAI SYLLABLE O */
  ["\uA5BB", "\uA5BB"], /* VAI SYLLABLE ON */
  ["\uA5BC", "\uA5BC"], /* VAI SYLLABLE NGON */
  ["\uA5BD", "\uA5BD"], /* VAI SYLLABLE HO */
  ["\uA5BE", "\uA5BE"], /* VAI SYLLABLE HON */
  ["\uA5BF", "\uA5BF"], /* VAI SYLLABLE WO */
  ["\uA5C0", "\uA5C0"], /* VAI SYLLABLE WON */
  ["\uA5C1", "\uA5C1"], /* VAI SYLLABLE PO */
  ["\uA5C2", "\uA5C2"], /* VAI SYLLABLE BHO */
  ["\uA5C3", "\uA5C3"], /* VAI SYLLABLE BO */
  ["\uA5C4", "\uA5C4"], /* VAI SYLLABLE MBO */
  ["\uA5C5", "\uA5C5"], /* VAI SYLLABLE KPO */
  ["\uA5C6", "\uA5C6"], /* VAI SYLLABLE MGBO */
  ["\uA5C7", "\uA5C7"], /* VAI SYLLABLE GBO */
  ["\uA5C8", "\uA5C8"], /* VAI SYLLABLE GBON */
  ["\uA5C9", "\uA5C9"], /* VAI SYLLABLE FO */
  ["\uA5CA", "\uA5CA"], /* VAI SYLLABLE VO */
  ["\uA5CB", "\uA5CB"], /* VAI SYLLABLE TO */
  ["\uA5CC", "\uA5CC"], /* VAI SYLLABLE THO */
  ["\uA5CD", "\uA5CD"], /* VAI SYLLABLE DHO */
  ["\uA5CE", "\uA5CE"], /* VAI SYLLABLE DHHO */
  ["\uA5CF", "\uA5CF"], /* VAI SYLLABLE LO */
  ["\uA5D0", "\uA5D0"], /* VAI SYLLABLE RO */
  ["\uA5D1", "\uA5D1"], /* VAI SYLLABLE DO */
  ["\uA5D2", "\uA5D2"], /* VAI SYLLABLE NDO */
  ["\uA5D3", "\uA5D3"], /* VAI SYLLABLE SO */
  ["\uA5D4", "\uA5D4"], /* VAI SYLLABLE SHO */
  ["\uA5D5", "\uA5D5"], /* VAI SYLLABLE ZO */
  ["\uA5D6", "\uA5D6"], /* VAI SYLLABLE ZHO */
  ["\uA5D7", "\uA5D7"], /* VAI SYLLABLE CO */
  ["\uA5D8", "\uA5D8"], /* VAI SYLLABLE JO */
  ["\uA5D9", "\uA5D9"], /* VAI SYLLABLE NJO */
  ["\uA5DA", "\uA5DA"], /* VAI SYLLABLE YO */
  ["\uA5DB", "\uA5DB"], /* VAI SYLLABLE KO */
  ["\uA5DC", "\uA5DC"], /* VAI SYLLABLE NGGO */
  ["\uA5DD", "\uA5DD"], /* VAI SYLLABLE GO */
  ["\uA5DE", "\uA5DE"], /* VAI SYLLABLE MO */
  ["\uA5DF", "\uA5DF"], /* VAI SYLLABLE NO */
  ["\uA5E0", "\uA5E0"], /* VAI SYLLABLE NYO */
  ["\uA5E1", "\uA5E1"], /* VAI SYLLABLE E */
  ["\uA5E2", "\uA5E2"], /* VAI SYLLABLE EN */
  ["\uA5E3", "\uA5E3"], /* VAI SYLLABLE NGEN */
  ["\uA5E4", "\uA5E4"], /* VAI SYLLABLE HE */
  ["\uA5E5", "\uA5E5"], /* VAI SYLLABLE HEN */
  ["\uA5E6", "\uA5E6"], /* VAI SYLLABLE WE */
  ["\uA5E7", "\uA5E7"], /* VAI SYLLABLE WEN */
  ["\uA5E8", "\uA5E8"], /* VAI SYLLABLE PE */
  ["\uA5E9", "\uA5E9"], /* VAI SYLLABLE BHE */
  ["\uA5EA", "\uA5EA"], /* VAI SYLLABLE BE */
  ["\uA5EB", "\uA5EB"], /* VAI SYLLABLE MBE */
  ["\uA5EC", "\uA5EC"], /* VAI SYLLABLE KPE */
  ["\uA5ED", "\uA5ED"], /* VAI SYLLABLE KPEN */
  ["\uA5EE", "\uA5EE"], /* VAI SYLLABLE MGBE */
  ["\uA5EF", "\uA5EF"], /* VAI SYLLABLE GBE */
  ["\uA5F0", "\uA5F0"], /* VAI SYLLABLE GBEN */
  ["\uA5F1", "\uA5F1"], /* VAI SYLLABLE FE */
  ["\uA5F2", "\uA5F2"], /* VAI SYLLABLE VE */
  ["\uA5F3", "\uA5F3"], /* VAI SYLLABLE TE */
  ["\uA5F4", "\uA5F4"], /* VAI SYLLABLE THE */
  ["\uA5F5", "\uA5F5"], /* VAI SYLLABLE DHE */
  ["\uA5F6", "\uA5F6"], /* VAI SYLLABLE DHHE */
  ["\uA5F7", "\uA5F7"], /* VAI SYLLABLE LE */
  ["\uA5F8", "\uA5F8"], /* VAI SYLLABLE RE */
  ["\uA5F9", "\uA5F9"], /* VAI SYLLABLE DE */
  ["\uA5FA", "\uA5FA"], /* VAI SYLLABLE NDE */
  ["\uA5FB", "\uA5FB"], /* VAI SYLLABLE SE */
  ["\uA5FC", "\uA5FC"], /* VAI SYLLABLE SHE */
  ["\uA5FD", "\uA5FD"], /* VAI SYLLABLE ZE */
  ["\uA5FE", "\uA5FE"], /* VAI SYLLABLE ZHE */
  ["\uA5FF", "\uA5FF"], /* VAI SYLLABLE CE */
  ["\uA600", "\uA600"], /* VAI SYLLABLE JE */
  ["\uA601", "\uA601"], /* VAI SYLLABLE NJE */
  ["\uA602", "\uA602"], /* VAI SYLLABLE YE */
  ["\uA603", "\uA603"], /* VAI SYLLABLE KE */
  ["\uA604", "\uA604"], /* VAI SYLLABLE NGGE */
  ["\uA605", "\uA605"], /* VAI SYLLABLE NGGEN */
  ["\uA606", "\uA606"], /* VAI SYLLABLE GE */
  ["\uA607", "\uA607"], /* VAI SYLLABLE GEN */
  ["\uA608", "\uA608"], /* VAI SYLLABLE ME */
  ["\uA609", "\uA609"], /* VAI SYLLABLE NE */
  ["\uA60A", "\uA60A"], /* VAI SYLLABLE NYE */
  ["\uA60B", "\uA60B"], /* VAI SYLLABLE NG */
  ["\uA60C", "\uA60C"], /* VAI SYLLABLE LENGTHENER */
  ["\uA60D", "\uA60D"], /* VAI COMMA */
  ["\uA60E", "\uA60E"], /* VAI FULL STOP */
  ["\uA60F", "\uA60F"], /* VAI QUESTION MARK */
  ["\uA610", "\uA610"], /* VAI SYLLABLE NDOLE FA */
  ["\uA611", "\uA611"], /* VAI SYLLABLE NDOLE KA */
  ["\uA612", "\uA612"], /* VAI SYLLABLE NDOLE SOO */
  ["\uA613", "\uA613"], /* VAI SYMBOL FEENG */
  ["\uA614", "\uA614"], /* VAI SYMBOL KEENG */
  ["\uA615", "\uA615"], /* VAI SYMBOL TING */
  ["\uA616", "\uA616"], /* VAI SYMBOL NII */
  ["\uA617", "\uA617"], /* VAI SYMBOL BANG */
  ["\uA618", "\uA618"], /* VAI SYMBOL FAA */
  ["\uA619", "\uA619"], /* VAI SYMBOL TAA */
  ["\uA61A", "\uA61A"], /* VAI SYMBOL DANG */
  ["\uA61B", "\uA61B"], /* VAI SYMBOL DOONG */
  ["\uA61C", "\uA61C"], /* VAI SYMBOL KUNG */
  ["\uA61D", "\uA61D"], /* VAI SYMBOL TONG */
  ["\uA61E", "\uA61E"], /* VAI SYMBOL DO-O */
  ["\uA61F", "\uA61F"], /* VAI SYMBOL JONG */
  ["\uA620", "\uA620"], /* VAI DIGIT ZERO */
  ["\uA621", "\uA621"], /* VAI DIGIT ONE */
  ["\uA622", "\uA622"], /* VAI DIGIT TWO */
  ["\uA623", "\uA623"], /* VAI DIGIT THREE */
  ["\uA624", "\uA624"], /* VAI DIGIT FOUR */
  ["\uA625", "\uA625"], /* VAI DIGIT FIVE */
  ["\uA626", "\uA626"], /* VAI DIGIT SIX */
  ["\uA627", "\uA627"], /* VAI DIGIT SEVEN */
  ["\uA628", "\uA628"], /* VAI DIGIT EIGHT */
  ["\uA629", "\uA629"], /* VAI DIGIT NINE */
  ["\uA62A", "\uA62A"], /* VAI SYLLABLE NDOLE MA */
  ["\uA62B", "\uA62B"], /* VAI SYLLABLE NDOLE DO */
  ["\uA62C", "\uA62C"],
  ["\uA62D", "\uA62D"],
  ["\uA62E", "\uA62E"],
  ["\uA62F", "\uA62F"],
  ["\uA630", "\uA630"],
  ["\uA631", "\uA631"],
  ["\uA632", "\uA632"],
  ["\uA633", "\uA633"],
  ["\uA634", "\uA634"],
  ["\uA635", "\uA635"],
  ["\uA636", "\uA636"],
  ["\uA637", "\uA637"],
  ["\uA638", "\uA638"],
  ["\uA639", "\uA639"],
  ["\uA63A", "\uA63A"],
  ["\uA63B", "\uA63B"],
  ["\uA63C", "\uA63C"],
  ["\uA63D", "\uA63D"],
  ["\uA63E", "\uA63E"],
  ["\uA63F", "\uA63F"],
  ["\uA640", "\uA641"], /* CYRILLIC CAPITAL LETTER ZEMLYA */
  ["\uA640", "\uA641"], /* CYRILLIC SMALL LETTER ZEMLYA */
  ["\uA642", "\uA643"], /* CYRILLIC CAPITAL LETTER DZELO */
  ["\uA642", "\uA643"], /* CYRILLIC SMALL LETTER DZELO */
  ["\uA644", "\uA645"], /* CYRILLIC CAPITAL LETTER REVERSED DZE */
  ["\uA644", "\uA645"], /* CYRILLIC SMALL LETTER REVERSED DZE */
  ["\uA646", "\uA647"], /* CYRILLIC CAPITAL LETTER IOTA */
  ["\uA646", "\uA647"], /* CYRILLIC SMALL LETTER IOTA */
  ["\uA648", "\uA649"], /* CYRILLIC CAPITAL LETTER DJERV */
  ["\uA648", "\uA649"], /* CYRILLIC SMALL LETTER DJERV */
  ["\uA64A", "\uA64B"], /* CYRILLIC CAPITAL LETTER MONOGRAPH UK */
  ["\uA64A", "\uA64B"], /* CYRILLIC SMALL LETTER MONOGRAPH UK */
  ["\uA64C", "\uA64D"], /* CYRILLIC CAPITAL LETTER BROAD OMEGA */
  ["\uA64C", "\uA64D"], /* CYRILLIC SMALL LETTER BROAD OMEGA */
  ["\uA64E", "\uA64F"], /* CYRILLIC CAPITAL LETTER NEUTRAL YER */
  ["\uA64E", "\uA64F"], /* CYRILLIC SMALL LETTER NEUTRAL YER */
  ["\uA650", "\uA651"], /* CYRILLIC CAPITAL LETTER YERU WITH BACK YER */
  ["\uA650", "\uA651"], /* CYRILLIC SMALL LETTER YERU WITH BACK YER */
  ["\uA652", "\uA653"], /* CYRILLIC CAPITAL LETTER IOTIFIED YAT */
  ["\uA652", "\uA653"], /* CYRILLIC SMALL LETTER IOTIFIED YAT */
  ["\uA654", "\uA655"], /* CYRILLIC CAPITAL LETTER REVERSED YU */
  ["\uA654", "\uA655"], /* CYRILLIC SMALL LETTER REVERSED YU */
  ["\uA656", "\uA657"], /* CYRILLIC CAPITAL LETTER IOTIFIED A */
  ["\uA656", "\uA657"], /* CYRILLIC SMALL LETTER IOTIFIED A */
  ["\uA658", "\uA659"], /* CYRILLIC CAPITAL LETTER CLOSED LITTLE YUS */
  ["\uA658", "\uA659"], /* CYRILLIC SMALL LETTER CLOSED LITTLE YUS */
  ["\uA65A", "\uA65B"], /* CYRILLIC CAPITAL LETTER BLENDED YUS */
  ["\uA65A", "\uA65B"], /* CYRILLIC SMALL LETTER BLENDED YUS */
  ["\uA65C", "\uA65D"], /* CYRILLIC CAPITAL LETTER IOTIFIED CLOSED LITTLE YUS */
  ["\uA65C", "\uA65D"], /* CYRILLIC SMALL LETTER IOTIFIED CLOSED LITTLE YUS */
  ["\uA65E", "\uA65F"], /* CYRILLIC CAPITAL LETTER YN */
  ["\uA65E", "\uA65F"], /* CYRILLIC SMALL LETTER YN */
  ["\uA660", "\uA661"], /* CYRILLIC CAPITAL LETTER REVERSED TSE */
  ["\uA660", "\uA661"], /* CYRILLIC SMALL LETTER REVERSED TSE */
  ["\uA662", "\uA663"], /* CYRILLIC CAPITAL LETTER SOFT DE */
  ["\uA662", "\uA663"], /* CYRILLIC SMALL LETTER SOFT DE */
  ["\uA664", "\uA665"], /* CYRILLIC CAPITAL LETTER SOFT EL */
  ["\uA664", "\uA665"], /* CYRILLIC SMALL LETTER SOFT EL */
  ["\uA666", "\uA667"], /* CYRILLIC CAPITAL LETTER SOFT EM */
  ["\uA666", "\uA667"], /* CYRILLIC SMALL LETTER SOFT EM */
  ["\uA668", "\uA669"], /* CYRILLIC CAPITAL LETTER MONOCULAR O */
  ["\uA668", "\uA669"], /* CYRILLIC SMALL LETTER MONOCULAR O */
  ["\uA66A", "\uA66B"], /* CYRILLIC CAPITAL LETTER BINOCULAR O */
  ["\uA66A", "\uA66B"], /* CYRILLIC SMALL LETTER BINOCULAR O */
  ["\uA66C", "\uA66D"], /* CYRILLIC CAPITAL LETTER DOUBLE MONOCULAR O */
  ["\uA66C", "\uA66D"], /* CYRILLIC SMALL LETTER DOUBLE MONOCULAR O */
  ["\uA66E", "\uA66E"], /* CYRILLIC LETTER MULTIOCULAR O */
  ["\uA66F", "\uA66F"], /* COMBINING CYRILLIC VZMET */
  ["\uA670", "\uA670"], /* COMBINING CYRILLIC TEN MILLIONS SIGN */
  ["\uA671", "\uA671"], /* COMBINING CYRILLIC HUNDRED MILLIONS SIGN */
  ["\uA672", "\uA672"], /* COMBINING CYRILLIC THOUSAND MILLIONS SIGN */
  ["\uA673", "\uA673"], /* SLAVONIC ASTERISK */
  ["\uA674", "\uA674"], /* COMBINING CYRILLIC LETTER UKRAINIAN IE */
  ["\uA675", "\uA675"], /* COMBINING CYRILLIC LETTER I */
  ["\uA676", "\uA676"], /* COMBINING CYRILLIC LETTER YI */
  ["\uA677", "\uA677"], /* COMBINING CYRILLIC LETTER U */
  ["\uA678", "\uA678"], /* COMBINING CYRILLIC LETTER HARD SIGN */
  ["\uA679", "\uA679"], /* COMBINING CYRILLIC LETTER YERU */
  ["\uA67A", "\uA67A"], /* COMBINING CYRILLIC LETTER SOFT SIGN */
  ["\uA67B", "\uA67B"], /* COMBINING CYRILLIC LETTER OMEGA */
  ["\uA67C", "\uA67C"], /* COMBINING CYRILLIC KAVYKA */
  ["\uA67D", "\uA67D"], /* COMBINING CYRILLIC PAYEROK */
  ["\uA67E", "\uA67E"], /* CYRILLIC KAVYKA */
  ["\uA67F", "\uA67F"], /* CYRILLIC PAYEROK */
  ["\uA680", "\uA681"], /* CYRILLIC CAPITAL LETTER DWE */
  ["\uA680", "\uA681"], /* CYRILLIC SMALL LETTER DWE */
  ["\uA682", "\uA683"], /* CYRILLIC CAPITAL LETTER DZWE */
  ["\uA682", "\uA683"], /* CYRILLIC SMALL LETTER DZWE */
  ["\uA684", "\uA685"], /* CYRILLIC CAPITAL LETTER ZHWE */
  ["\uA684", "\uA685"], /* CYRILLIC SMALL LETTER ZHWE */
  ["\uA686", "\uA687"], /* CYRILLIC CAPITAL LETTER CCHE */
  ["\uA686", "\uA687"], /* CYRILLIC SMALL LETTER CCHE */
  ["\uA688", "\uA689"], /* CYRILLIC CAPITAL LETTER DZZE */
  ["\uA688", "\uA689"], /* CYRILLIC SMALL LETTER DZZE */
  ["\uA68A", "\uA68B"], /* CYRILLIC CAPITAL LETTER TE WITH MIDDLE HOOK */
  ["\uA68A", "\uA68B"], /* CYRILLIC SMALL LETTER TE WITH MIDDLE HOOK */
  ["\uA68C", "\uA68D"], /* CYRILLIC CAPITAL LETTER TWE */
  ["\uA68C", "\uA68D"], /* CYRILLIC SMALL LETTER TWE */
  ["\uA68E", "\uA68F"], /* CYRILLIC CAPITAL LETTER TSWE */
  ["\uA68E", "\uA68F"], /* CYRILLIC SMALL LETTER TSWE */
  ["\uA690", "\uA691"], /* CYRILLIC CAPITAL LETTER TSSE */
  ["\uA690", "\uA691"], /* CYRILLIC SMALL LETTER TSSE */
  ["\uA692", "\uA693"], /* CYRILLIC CAPITAL LETTER TCHE */
  ["\uA692", "\uA693"], /* CYRILLIC SMALL LETTER TCHE */
  ["\uA694", "\uA695"], /* CYRILLIC CAPITAL LETTER HWE */
  ["\uA694", "\uA695"], /* CYRILLIC SMALL LETTER HWE */
  ["\uA696", "\uA697"], /* CYRILLIC CAPITAL LETTER SHWE */
  ["\uA696", "\uA697"], /* CYRILLIC SMALL LETTER SHWE */
  ["\uA698", "\uA699"], /* CYRILLIC CAPITAL LETTER DOUBLE O */
  ["\uA698", "\uA699"], /* CYRILLIC SMALL LETTER DOUBLE O */
  ["\uA69A", "\uA69B"], /* CYRILLIC CAPITAL LETTER CROSSED O */
  ["\uA69A", "\uA69B"], /* CYRILLIC SMALL LETTER CROSSED O */
  ["\uA69C", "\uA69C"], /* MODIFIER LETTER CYRILLIC HARD SIGN */
  ["\uA69D", "\uA69D"], /* MODIFIER LETTER CYRILLIC SOFT SIGN */
  ["\uA69E", "\uA69E"], /* COMBINING CYRILLIC LETTER EF */
  ["\uA69F", "\uA69F"], /* COMBINING CYRILLIC LETTER IOTIFIED E */
  ["\uA6A0", "\uA6A0"], /* BAMUM LETTER A */
  ["\uA6A1", "\uA6A1"], /* BAMUM LETTER KA */
  ["\uA6A2", "\uA6A2"], /* BAMUM LETTER U */
  ["\uA6A3", "\uA6A3"], /* BAMUM LETTER KU */
  ["\uA6A4", "\uA6A4"], /* BAMUM LETTER EE */
  ["\uA6A5", "\uA6A5"], /* BAMUM LETTER REE */
  ["\uA6A6", "\uA6A6"], /* BAMUM LETTER TAE */
  ["\uA6A7", "\uA6A7"], /* BAMUM LETTER O */
  ["\uA6A8", "\uA6A8"], /* BAMUM LETTER NYI */
  ["\uA6A9", "\uA6A9"], /* BAMUM LETTER I */
  ["\uA6AA", "\uA6AA"], /* BAMUM LETTER LA */
  ["\uA6AB", "\uA6AB"], /* BAMUM LETTER PA */
  ["\uA6AC", "\uA6AC"], /* BAMUM LETTER RII */
  ["\uA6AD", "\uA6AD"], /* BAMUM LETTER RIEE */
  ["\uA6AE", "\uA6AE"], /* BAMUM LETTER LEEEE */
  ["\uA6AF", "\uA6AF"], /* BAMUM LETTER MEEEE */
  ["\uA6B0", "\uA6B0"], /* BAMUM LETTER TAA */
  ["\uA6B1", "\uA6B1"], /* BAMUM LETTER NDAA */
  ["\uA6B2", "\uA6B2"], /* BAMUM LETTER NJAEM */
  ["\uA6B3", "\uA6B3"], /* BAMUM LETTER M */
  ["\uA6B4", "\uA6B4"], /* BAMUM LETTER SUU */
  ["\uA6B5", "\uA6B5"], /* BAMUM LETTER MU */
  ["\uA6B6", "\uA6B6"], /* BAMUM LETTER SHII */
  ["\uA6B7", "\uA6B7"], /* BAMUM LETTER SI */
  ["\uA6B8", "\uA6B8"], /* BAMUM LETTER SHEUX */
  ["\uA6B9", "\uA6B9"], /* BAMUM LETTER SEUX */
  ["\uA6BA", "\uA6BA"], /* BAMUM LETTER KYEE */
  ["\uA6BB", "\uA6BB"], /* BAMUM LETTER KET */
  ["\uA6BC", "\uA6BC"], /* BAMUM LETTER NUAE */
  ["\uA6BD", "\uA6BD"], /* BAMUM LETTER NU */
  ["\uA6BE", "\uA6BE"], /* BAMUM LETTER NJUAE */
  ["\uA6BF", "\uA6BF"], /* BAMUM LETTER YOQ */
  ["\uA6C0", "\uA6C0"], /* BAMUM LETTER SHU */
  ["\uA6C1", "\uA6C1"], /* BAMUM LETTER YUQ */
  ["\uA6C2", "\uA6C2"], /* BAMUM LETTER YA */
  ["\uA6C3", "\uA6C3"], /* BAMUM LETTER NSHA */
  ["\uA6C4", "\uA6C4"], /* BAMUM LETTER KEUX */
  ["\uA6C5", "\uA6C5"], /* BAMUM LETTER PEUX */
  ["\uA6C6", "\uA6C6"], /* BAMUM LETTER NJEE */
  ["\uA6C7", "\uA6C7"], /* BAMUM LETTER NTEE */
  ["\uA6C8", "\uA6C8"], /* BAMUM LETTER PUE */
  ["\uA6C9", "\uA6C9"], /* BAMUM LETTER WUE */
  ["\uA6CA", "\uA6CA"], /* BAMUM LETTER PEE */
  ["\uA6CB", "\uA6CB"], /* BAMUM LETTER FEE */
  ["\uA6CC", "\uA6CC"], /* BAMUM LETTER RU */
  ["\uA6CD", "\uA6CD"], /* BAMUM LETTER LU */
  ["\uA6CE", "\uA6CE"], /* BAMUM LETTER MI */
  ["\uA6CF", "\uA6CF"], /* BAMUM LETTER NI */
  ["\uA6D0", "\uA6D0"], /* BAMUM LETTER REUX */
  ["\uA6D1", "\uA6D1"], /* BAMUM LETTER RAE */
  ["\uA6D2", "\uA6D2"], /* BAMUM LETTER KEN */
  ["\uA6D3", "\uA6D3"], /* BAMUM LETTER NGKWAEN */
  ["\uA6D4", "\uA6D4"], /* BAMUM LETTER NGGA */
  ["\uA6D5", "\uA6D5"], /* BAMUM LETTER NGA */
  ["\uA6D6", "\uA6D6"], /* BAMUM LETTER SHO */
  ["\uA6D7", "\uA6D7"], /* BAMUM LETTER PUAE */
  ["\uA6D8", "\uA6D8"], /* BAMUM LETTER FU */
  ["\uA6D9", "\uA6D9"], /* BAMUM LETTER FOM */
  ["\uA6DA", "\uA6DA"], /* BAMUM LETTER WA */
  ["\uA6DB", "\uA6DB"], /* BAMUM LETTER NA */
  ["\uA6DC", "\uA6DC"], /* BAMUM LETTER LI */
  ["\uA6DD", "\uA6DD"], /* BAMUM LETTER PI */
  ["\uA6DE", "\uA6DE"], /* BAMUM LETTER LOQ */
  ["\uA6DF", "\uA6DF"], /* BAMUM LETTER KO */
  ["\uA6E0", "\uA6E0"], /* BAMUM LETTER MBEN */
  ["\uA6E1", "\uA6E1"], /* BAMUM LETTER REN */
  ["\uA6E2", "\uA6E2"], /* BAMUM LETTER MEN */
  ["\uA6E3", "\uA6E3"], /* BAMUM LETTER MA */
  ["\uA6E4", "\uA6E4"], /* BAMUM LETTER TI */
  ["\uA6E5", "\uA6E5"], /* BAMUM LETTER KI */
  ["\uA6E6", "\uA6E6"], /* BAMUM LETTER MO */
  ["\uA6E7", "\uA6E7"], /* BAMUM LETTER MBAA */
  ["\uA6E8", "\uA6E8"], /* BAMUM LETTER TET */
  ["\uA6E9", "\uA6E9"], /* BAMUM LETTER KPA */
  ["\uA6EA", "\uA6EA"], /* BAMUM LETTER TEN */
  ["\uA6EB", "\uA6EB"], /* BAMUM LETTER NTUU */
  ["\uA6EC", "\uA6EC"], /* BAMUM LETTER SAMBA */
  ["\uA6ED", "\uA6ED"], /* BAMUM LETTER FAAMAE */
  ["\uA6EE", "\uA6EE"], /* BAMUM LETTER KOVUU */
  ["\uA6EF", "\uA6EF"], /* BAMUM LETTER KOGHOM */
  ["\uA6F0", "\uA6F0"], /* BAMUM COMBINING MARK KOQNDON */
  ["\uA6F1", "\uA6F1"], /* BAMUM COMBINING MARK TUKWENTIS */
  ["\uA6F2", "\uA6F2"], /* BAMUM NJAEMLI */
  ["\uA6F3", "\uA6F3"], /* BAMUM FULL STOP */
  ["\uA6F4", "\uA6F4"], /* BAMUM COLON */
  ["\uA6F5", "\uA6F5"], /* BAMUM COMMA */
  ["\uA6F6", "\uA6F6"], /* BAMUM SEMICOLON */
  ["\uA6F7", "\uA6F7"], /* BAMUM QUESTION MARK */
  ["\uA6F8", "\uA6F8"],
  ["\uA6F9", "\uA6F9"],
  ["\uA6FA", "\uA6FA"],
  ["\uA6FB", "\uA6FB"],
  ["\uA6FC", "\uA6FC"],
  ["\uA6FD", "\uA6FD"],
  ["\uA6FE", "\uA6FE"],
  ["\uA6FF", "\uA6FF"],
  ["\uA700", "\uA700"], /* MODIFIER LETTER CHINESE TONE YIN PING */
  ["\uA701", "\uA701"], /* MODIFIER LETTER CHINESE TONE YANG PING */
  ["\uA702", "\uA702"], /* MODIFIER LETTER CHINESE TONE YIN SHANG */
  ["\uA703", "\uA703"], /* MODIFIER LETTER CHINESE TONE YANG SHANG */
  ["\uA704", "\uA704"], /* MODIFIER LETTER CHINESE TONE YIN QU */
  ["\uA705", "\uA705"], /* MODIFIER LETTER CHINESE TONE YANG QU */
  ["\uA706", "\uA706"], /* MODIFIER LETTER CHINESE TONE YIN RU */
  ["\uA707", "\uA707"], /* MODIFIER LETTER CHINESE TONE YANG RU */
  ["\uA708", "\uA708"], /* MODIFIER LETTER EXTRA-HIGH DOTTED TONE BAR */
  ["\uA709", "\uA709"], /* MODIFIER LETTER HIGH DOTTED TONE BAR */
  ["\uA70A", "\uA70A"], /* MODIFIER LETTER MID DOTTED TONE BAR */
  ["\uA70B", "\uA70B"], /* MODIFIER LETTER LOW DOTTED TONE BAR */
  ["\uA70C", "\uA70C"], /* MODIFIER LETTER EXTRA-LOW DOTTED TONE BAR */
  ["\uA70D", "\uA70D"], /* MODIFIER LETTER EXTRA-HIGH DOTTED LEFT-STEM TONE BAR */
  ["\uA70E", "\uA70E"], /* MODIFIER LETTER HIGH DOTTED LEFT-STEM TONE BAR */
  ["\uA70F", "\uA70F"], /* MODIFIER LETTER MID DOTTED LEFT-STEM TONE BAR */
  ["\uA710", "\uA710"], /* MODIFIER LETTER LOW DOTTED LEFT-STEM TONE BAR */
  ["\uA711", "\uA711"], /* MODIFIER LETTER EXTRA-LOW DOTTED LEFT-STEM TONE BAR */
  ["\uA712", "\uA712"], /* MODIFIER LETTER EXTRA-HIGH LEFT-STEM TONE BAR */
  ["\uA713", "\uA713"], /* MODIFIER LETTER HIGH LEFT-STEM TONE BAR */
  ["\uA714", "\uA714"], /* MODIFIER LETTER MID LEFT-STEM TONE BAR */
  ["\uA715", "\uA715"], /* MODIFIER LETTER LOW LEFT-STEM TONE BAR */
  ["\uA716", "\uA716"], /* MODIFIER LETTER EXTRA-LOW LEFT-STEM TONE BAR */
  ["\uA717", "\uA717"], /* MODIFIER LETTER DOT VERTICAL BAR */
  ["\uA718", "\uA718"], /* MODIFIER LETTER DOT SLASH */
  ["\uA719", "\uA719"], /* MODIFIER LETTER DOT HORIZONTAL BAR */
  ["\uA71A", "\uA71A"], /* MODIFIER LETTER LOWER RIGHT CORNER ANGLE */
  ["\uA71B", "\uA71B"], /* MODIFIER LETTER RAISED UP ARROW */
  ["\uA71C", "\uA71C"], /* MODIFIER LETTER RAISED DOWN ARROW */
  ["\uA71D", "\uA71D"], /* MODIFIER LETTER RAISED EXCLAMATION MARK */
  ["\uA71E", "\uA71E"], /* MODIFIER LETTER RAISED INVERTED EXCLAMATION MARK */
  ["\uA71F", "\uA71F"], /* MODIFIER LETTER LOW INVERTED EXCLAMATION MARK */
  ["\uA720", "\uA720"], /* MODIFIER LETTER STRESS AND HIGH TONE */
  ["\uA721", "\uA721"], /* MODIFIER LETTER STRESS AND LOW TONE */
  ["\uA722", "\uA723"], /* LATIN CAPITAL LETTER EGYPTOLOGICAL ALEF */
  ["\uA722", "\uA723"], /* LATIN SMALL LETTER EGYPTOLOGICAL ALEF */
  ["\uA724", "\uA725"], /* LATIN CAPITAL LETTER EGYPTOLOGICAL AIN */
  ["\uA724", "\uA725"], /* LATIN SMALL LETTER EGYPTOLOGICAL AIN */
  ["\uA726", "\uA727"], /* LATIN CAPITAL LETTER HENG */
  ["\uA726", "\uA727"], /* LATIN SMALL LETTER HENG */
  ["\uA728", "\uA729"], /* LATIN CAPITAL LETTER TZ */
  ["\uA728", "\uA729"], /* LATIN SMALL LETTER TZ */
  ["\uA72A", "\uA72B"], /* LATIN CAPITAL LETTER TRESILLO */
  ["\uA72A", "\uA72B"], /* LATIN SMALL LETTER TRESILLO */
  ["\uA72C", "\uA72D"], /* LATIN CAPITAL LETTER CUATRILLO */
  ["\uA72C", "\uA72D"], /* LATIN SMALL LETTER CUATRILLO */
  ["\uA72E", "\uA72F"], /* LATIN CAPITAL LETTER CUATRILLO WITH COMMA */
  ["\uA72E", "\uA72F"], /* LATIN SMALL LETTER CUATRILLO WITH COMMA */
  ["\uA730", "\uA730"], /* LATIN LETTER SMALL CAPITAL F */
  ["\uA731", "\uA731"], /* LATIN LETTER SMALL CAPITAL S */
  ["\uA732", "\uA733"], /* LATIN CAPITAL LETTER AA */
  ["\uA732", "\uA733"], /* LATIN SMALL LETTER AA */
  ["\uA734", "\uA735"], /* LATIN CAPITAL LETTER AO */
  ["\uA734", "\uA735"], /* LATIN SMALL LETTER AO */
  ["\uA736", "\uA737"], /* LATIN CAPITAL LETTER AU */
  ["\uA736", "\uA737"], /* LATIN SMALL LETTER AU */
  ["\uA738", "\uA739"], /* LATIN CAPITAL LETTER AV */
  ["\uA738", "\uA739"], /* LATIN SMALL LETTER AV */
  ["\uA73A", "\uA73B"], /* LATIN CAPITAL LETTER AV WITH HORIZONTAL BAR */
  ["\uA73A", "\uA73B"], /* LATIN SMALL LETTER AV WITH HORIZONTAL BAR */
  ["\uA73C", "\uA73D"], /* LATIN CAPITAL LETTER AY */
  ["\uA73C", "\uA73D"], /* LATIN SMALL LETTER AY */
  ["\uA73E", "\uA73F"], /* LATIN CAPITAL LETTER REVERSED C WITH DOT */
  ["\uA73E", "\uA73F"], /* LATIN SMALL LETTER REVERSED C WITH DOT */
  ["\uA740", "\uA741"], /* LATIN CAPITAL LETTER K WITH STROKE */
  ["\uA740", "\uA741"], /* LATIN SMALL LETTER K WITH STROKE */
  ["\uA742", "\uA743"], /* LATIN CAPITAL LETTER K WITH DIAGONAL STROKE */
  ["\uA742", "\uA743"], /* LATIN SMALL LETTER K WITH DIAGONAL STROKE */
  ["\uA744", "\uA745"], /* LATIN CAPITAL LETTER K WITH STROKE AND DIAGONAL STROKE */
  ["\uA744", "\uA745"], /* LATIN SMALL LETTER K WITH STROKE AND DIAGONAL STROKE */
  ["\uA746", "\uA747"], /* LATIN CAPITAL LETTER BROKEN L */
  ["\uA746", "\uA747"], /* LATIN SMALL LETTER BROKEN L */
  ["\uA748", "\uA749"], /* LATIN CAPITAL LETTER L WITH HIGH STROKE */
  ["\uA748", "\uA749"], /* LATIN SMALL LETTER L WITH HIGH STROKE */
  ["\uA74A", "\uA74B"], /* LATIN CAPITAL LETTER O WITH LONG STROKE OVERLAY */
  ["\uA74A", "\uA74B"], /* LATIN SMALL LETTER O WITH LONG STROKE OVERLAY */
  ["\uA74C", "\uA74D"], /* LATIN CAPITAL LETTER O WITH LOOP */
  ["\uA74C", "\uA74D"], /* LATIN SMALL LETTER O WITH LOOP */
  ["\uA74E", "\uA74F"], /* LATIN CAPITAL LETTER OO */
  ["\uA74E", "\uA74F"], /* LATIN SMALL LETTER OO */
  ["\uA750", "\uA751"], /* LATIN CAPITAL LETTER P WITH STROKE THROUGH DESCENDER */
  ["\uA750", "\uA751"], /* LATIN SMALL LETTER P WITH STROKE THROUGH DESCENDER */
  ["\uA752", "\uA753"], /* LATIN CAPITAL LETTER P WITH FLOURISH */
  ["\uA752", "\uA753"], /* LATIN SMALL LETTER P WITH FLOURISH */
  ["\uA754", "\uA755"], /* LATIN CAPITAL LETTER P WITH SQUIRREL TAIL */
  ["\uA754", "\uA755"], /* LATIN SMALL LETTER P WITH SQUIRREL TAIL */
  ["\uA756", "\uA757"], /* LATIN CAPITAL LETTER Q WITH STROKE THROUGH DESCENDER */
  ["\uA756", "\uA757"], /* LATIN SMALL LETTER Q WITH STROKE THROUGH DESCENDER */
  ["\uA758", "\uA759"], /* LATIN CAPITAL LETTER Q WITH DIAGONAL STROKE */
  ["\uA758", "\uA759"], /* LATIN SMALL LETTER Q WITH DIAGONAL STROKE */
  ["\uA75A", "\uA75B"], /* LATIN CAPITAL LETTER R ROTUNDA */
  ["\uA75A", "\uA75B"], /* LATIN SMALL LETTER R ROTUNDA */
  ["\uA75C", "\uA75D"], /* LATIN CAPITAL LETTER RUM ROTUNDA */
  ["\uA75C", "\uA75D"], /* LATIN SMALL LETTER RUM ROTUNDA */
  ["\uA75E", "\uA75F"], /* LATIN CAPITAL LETTER V WITH DIAGONAL STROKE */
  ["\uA75E", "\uA75F"], /* LATIN SMALL LETTER V WITH DIAGONAL STROKE */
  ["\uA760", "\uA761"], /* LATIN CAPITAL LETTER VY */
  ["\uA760", "\uA761"], /* LATIN SMALL LETTER VY */
  ["\uA762", "\uA763"], /* LATIN CAPITAL LETTER VISIGOTHIC Z */
  ["\uA762", "\uA763"], /* LATIN SMALL LETTER VISIGOTHIC Z */
  ["\uA764", "\uA765"], /* LATIN CAPITAL LETTER THORN WITH STROKE */
  ["\uA764", "\uA765"], /* LATIN SMALL LETTER THORN WITH STROKE */
  ["\uA766", "\uA767"], /* LATIN CAPITAL LETTER THORN WITH STROKE THROUGH DESCENDER */
  ["\uA766", "\uA767"], /* LATIN SMALL LETTER THORN WITH STROKE THROUGH DESCENDER */
  ["\uA768", "\uA769"], /* LATIN CAPITAL LETTER VEND */
  ["\uA768", "\uA769"], /* LATIN SMALL LETTER VEND */
  ["\uA76A", "\uA76B"], /* LATIN CAPITAL LETTER ET */
  ["\uA76A", "\uA76B"], /* LATIN SMALL LETTER ET */
  ["\uA76C", "\uA76D"], /* LATIN CAPITAL LETTER IS */
  ["\uA76C", "\uA76D"], /* LATIN SMALL LETTER IS */
  ["\uA76E", "\uA76F"], /* LATIN CAPITAL LETTER CON */
  ["\uA76E", "\uA76F"], /* LATIN SMALL LETTER CON */
  ["\uA770", "\uA770"], /* MODIFIER LETTER US */
  ["\uA771", "\uA771"], /* LATIN SMALL LETTER DUM */
  ["\uA772", "\uA772"], /* LATIN SMALL LETTER LUM */
  ["\uA773", "\uA773"], /* LATIN SMALL LETTER MUM */
  ["\uA774", "\uA774"], /* LATIN SMALL LETTER NUM */
  ["\uA775", "\uA775"], /* LATIN SMALL LETTER RUM */
  ["\uA776", "\uA776"], /* LATIN LETTER SMALL CAPITAL RUM */
  ["\uA777", "\uA777"], /* LATIN SMALL LETTER TUM */
  ["\uA778", "\uA778"], /* LATIN SMALL LETTER UM */
  ["\uA779", "\uA77A"], /* LATIN CAPITAL LETTER INSULAR D */
  ["\uA779", "\uA77A"], /* LATIN SMALL LETTER INSULAR D */
  ["\uA77B", "\uA77C"], /* LATIN CAPITAL LETTER INSULAR F */
  ["\uA77B", "\uA77C"], /* LATIN SMALL LETTER INSULAR F */
  ["\uA77D", "\u1D79"], /* LATIN CAPITAL LETTER INSULAR G */
  ["\uA77E", "\uA77F"], /* LATIN CAPITAL LETTER TURNED INSULAR G */
  ["\uA77E", "\uA77F"], /* LATIN SMALL LETTER TURNED INSULAR G */
  ["\uA780", "\uA781"], /* LATIN CAPITAL LETTER TURNED L */
  ["\uA780", "\uA781"], /* LATIN SMALL LETTER TURNED L */
  ["\uA782", "\uA783"], /* LATIN CAPITAL LETTER INSULAR R */
  ["\uA782", "\uA783"], /* LATIN SMALL LETTER INSULAR R */
  ["\uA784", "\uA785"], /* LATIN CAPITAL LETTER INSULAR S */
  ["\uA784", "\uA785"], /* LATIN SMALL LETTER INSULAR S */
  ["\uA786", "\uA787"], /* LATIN CAPITAL LETTER INSULAR T */
  ["\uA786", "\uA787"], /* LATIN SMALL LETTER INSULAR T */
  ["\uA788", "\uA788"], /* MODIFIER LETTER LOW CIRCUMFLEX ACCENT */
  ["\uA789", "\uA789"], /* MODIFIER LETTER COLON */
  ["\uA78A", "\uA78A"], /* MODIFIER LETTER SHORT EQUALS SIGN */
  ["\uA78B", "\uA78C"], /* LATIN CAPITAL LETTER SALTILLO */
  ["\uA78B", "\uA78C"], /* LATIN SMALL LETTER SALTILLO */
  ["\uA78D", "\u0265"], /* LATIN CAPITAL LETTER TURNED H */
  ["\uA78E", "\uA78E"], /* LATIN SMALL LETTER L WITH RETROFLEX HOOK AND BELT */
  ["\uA78F", "\uA78F"], /* LATIN LETTER SINOLOGICAL DOT */
  ["\uA790", "\uA791"], /* LATIN CAPITAL LETTER N WITH DESCENDER */
  ["\uA790", "\uA791"], /* LATIN SMALL LETTER N WITH DESCENDER */
  ["\uA792", "\uA793"], /* LATIN CAPITAL LETTER C WITH BAR */
  ["\uA792", "\uA793"], /* LATIN SMALL LETTER C WITH BAR */
  ["\uA7C4", "\uA794"], /* LATIN SMALL LETTER C WITH PALATAL HOOK */
  ["\uA795", "\uA795"], /* LATIN SMALL LETTER H WITH PALATAL HOOK */
  ["\uA796", "\uA797"], /* LATIN CAPITAL LETTER B WITH FLOURISH */
  ["\uA796", "\uA797"], /* LATIN SMALL LETTER B WITH FLOURISH */
  ["\uA798", "\uA799"], /* LATIN CAPITAL LETTER F WITH STROKE */
  ["\uA798", "\uA799"], /* LATIN SMALL LETTER F WITH STROKE */
  ["\uA79A", "\uA79B"], /* LATIN CAPITAL LETTER VOLAPUK AE */
  ["\uA79A", "\uA79B"], /* LATIN SMALL LETTER VOLAPUK AE */
  ["\uA79C", "\uA79D"], /* LATIN CAPITAL LETTER VOLAPUK OE */
  ["\uA79C", "\uA79D"], /* LATIN SMALL LETTER VOLAPUK OE */
  ["\uA79E", "\uA79F"], /* LATIN CAPITAL LETTER VOLAPUK UE */
  ["\uA79E", "\uA79F"], /* LATIN SMALL LETTER VOLAPUK UE */
  ["\uA7A0", "\uA7A1"], /* LATIN CAPITAL LETTER G WITH OBLIQUE STROKE */
  ["\uA7A0", "\uA7A1"], /* LATIN SMALL LETTER G WITH OBLIQUE STROKE */
  ["\uA7A2", "\uA7A3"], /* LATIN CAPITAL LETTER K WITH OBLIQUE STROKE */
  ["\uA7A2", "\uA7A3"], /* LATIN SMALL LETTER K WITH OBLIQUE STROKE */
  ["\uA7A4", "\uA7A5"], /* LATIN CAPITAL LETTER N WITH OBLIQUE STROKE */
  ["\uA7A4", "\uA7A5"], /* LATIN SMALL LETTER N WITH OBLIQUE STROKE */
  ["\uA7A6", "\uA7A7"], /* LATIN CAPITAL LETTER R WITH OBLIQUE STROKE */
  ["\uA7A6", "\uA7A7"], /* LATIN SMALL LETTER R WITH OBLIQUE STROKE */
  ["\uA7A8", "\uA7A9"], /* LATIN CAPITAL LETTER S WITH OBLIQUE STROKE */
  ["\uA7A8", "\uA7A9"], /* LATIN SMALL LETTER S WITH OBLIQUE STROKE */
  ["\uA7AA", "\u0266"], /* LATIN CAPITAL LETTER H WITH HOOK */
  ["\uA7AB", "\u025C"], /* LATIN CAPITAL LETTER REVERSED OPEN E */
  ["\uA7AC", "\u0261"], /* LATIN CAPITAL LETTER SCRIPT G */
  ["\uA7AD", "\u026C"], /* LATIN CAPITAL LETTER L WITH BELT */
  ["\uA7AE", "\u026A"], /* LATIN CAPITAL LETTER SMALL CAPITAL I */
  ["\uA7AF", "\uA7AF"], /* LATIN LETTER SMALL CAPITAL Q */
  ["\uA7B0", "\u029E"], /* LATIN CAPITAL LETTER TURNED K */
  ["\uA7B1", "\u0287"], /* LATIN CAPITAL LETTER TURNED T */
  ["\uA7B2", "\u029D"], /* LATIN CAPITAL LETTER J WITH CROSSED-TAIL */
  ["\uA7B3", "\uAB53"], /* LATIN CAPITAL LETTER CHI */
  ["\uA7B4", "\uA7B5"], /* LATIN CAPITAL LETTER BETA */
  ["\uA7B4", "\uA7B5"], /* LATIN SMALL LETTER BETA */
  ["\uA7B6", "\uA7B7"], /* LATIN CAPITAL LETTER OMEGA */
  ["\uA7B6", "\uA7B7"], /* LATIN SMALL LETTER OMEGA */
  ["\uA7B8", "\uA7B9"], /* LATIN CAPITAL LETTER U WITH STROKE */
  ["\uA7B8", "\uA7B9"], /* LATIN SMALL LETTER U WITH STROKE */
  ["\uA7BA", "\uA7BB"], /* LATIN CAPITAL LETTER GLOTTAL A */
  ["\uA7BA", "\uA7BB"], /* LATIN SMALL LETTER GLOTTAL A */
  ["\uA7BC", "\uA7BD"], /* LATIN CAPITAL LETTER GLOTTAL I */
  ["\uA7BC", "\uA7BD"], /* LATIN SMALL LETTER GLOTTAL I */
  ["\uA7BE", "\uA7BF"], /* LATIN CAPITAL LETTER GLOTTAL U */
  ["\uA7BE", "\uA7BF"], /* LATIN SMALL LETTER GLOTTAL U */
  ["\uA7C0", "\uA7C0"],
  ["\uA7C1", "\uA7C1"],
  ["\uA7C2", "\uA7C3"], /* LATIN CAPITAL LETTER ANGLICANA W */
  ["\uA7C2", "\uA7C3"], /* LATIN SMALL LETTER ANGLICANA W */
  ["\uA7C4", "\uA794"], /* LATIN CAPITAL LETTER C WITH PALATAL HOOK */
  ["\uA7C5", "\u0282"], /* LATIN CAPITAL LETTER S WITH HOOK */
  ["\uA7C6", "\u1D8E"], /* LATIN CAPITAL LETTER Z WITH PALATAL HOOK */
  ["\uA7C7", "\uA7C7"],
  ["\uA7C8", "\uA7C8"],
  ["\uA7C9", "\uA7C9"],
  ["\uA7CA", "\uA7CA"],
  ["\uA7CB", "\uA7CB"],
  ["\uA7CC", "\uA7CC"],
  ["\uA7CD", "\uA7CD"],
  ["\uA7CE", "\uA7CE"],
  ["\uA7CF", "\uA7CF"],
  ["\uA7D0", "\uA7D0"],
  ["\uA7D1", "\uA7D1"],
  ["\uA7D2", "\uA7D2"],
  ["\uA7D3", "\uA7D3"],
  ["\uA7D4", "\uA7D4"],
  ["\uA7D5", "\uA7D5"],
  ["\uA7D6", "\uA7D6"],
  ["\uA7D7", "\uA7D7"],
  ["\uA7D8", "\uA7D8"],
  ["\uA7D9", "\uA7D9"],
  ["\uA7DA", "\uA7DA"],
  ["\uA7DB", "\uA7DB"],
  ["\uA7DC", "\uA7DC"],
  ["\uA7DD", "\uA7DD"],
  ["\uA7DE", "\uA7DE"],
  ["\uA7DF", "\uA7DF"],
  ["\uA7E0", "\uA7E0"],
  ["\uA7E1", "\uA7E1"],
  ["\uA7E2", "\uA7E2"],
  ["\uA7E3", "\uA7E3"],
  ["\uA7E4", "\uA7E4"],
  ["\uA7E5", "\uA7E5"],
  ["\uA7E6", "\uA7E6"],
  ["\uA7E7", "\uA7E7"],
  ["\uA7E8", "\uA7E8"],
  ["\uA7E9", "\uA7E9"],
  ["\uA7EA", "\uA7EA"],
  ["\uA7EB", "\uA7EB"],
  ["\uA7EC", "\uA7EC"],
  ["\uA7ED", "\uA7ED"],
  ["\uA7EE", "\uA7EE"],
  ["\uA7EF", "\uA7EF"],
  ["\uA7F0", "\uA7F0"],
  ["\uA7F1", "\uA7F1"],
  ["\uA7F2", "\uA7F2"],
  ["\uA7F3", "\uA7F3"],
  ["\uA7F4", "\uA7F4"],
  ["\uA7F5", "\uA7F5"],
  ["\uA7F6", "\uA7F6"],
  ["\uA7F7", "\uA7F7"], /* LATIN EPIGRAPHIC LETTER SIDEWAYS I */
  ["\uA7F8", "\uA7F8"], /* MODIFIER LETTER CAPITAL H WITH STROKE */
  ["\uA7F9", "\uA7F9"], /* MODIFIER LETTER SMALL LIGATURE OE */
  ["\uA7FA", "\uA7FA"], /* LATIN LETTER SMALL CAPITAL TURNED M */
  ["\uA7FB", "\uA7FB"], /* LATIN EPIGRAPHIC LETTER REVERSED F */
  ["\uA7FC", "\uA7FC"], /* LATIN EPIGRAPHIC LETTER REVERSED P */
  ["\uA7FD", "\uA7FD"], /* LATIN EPIGRAPHIC LETTER INVERTED M */
  ["\uA7FE", "\uA7FE"], /* LATIN EPIGRAPHIC LETTER I LONGA */
  ["\uA7FF", "\uA7FF"], /* LATIN EPIGRAPHIC LETTER ARCHAIC M */
  ["\uA800", "\uA800"], /* SYLOTI NAGRI LETTER A */
  ["\uA801", "\uA801"], /* SYLOTI NAGRI LETTER I */
  ["\uA802", "\uA802"], /* SYLOTI NAGRI SIGN DVISVARA */
  ["\uA803", "\uA803"], /* SYLOTI NAGRI LETTER U */
  ["\uA804", "\uA804"], /* SYLOTI NAGRI LETTER E */
  ["\uA805", "\uA805"], /* SYLOTI NAGRI LETTER O */
  ["\uA806", "\uA806"], /* SYLOTI NAGRI SIGN HASANTA */
  ["\uA807", "\uA807"], /* SYLOTI NAGRI LETTER KO */
  ["\uA808", "\uA808"], /* SYLOTI NAGRI LETTER KHO */
  ["\uA809", "\uA809"], /* SYLOTI NAGRI LETTER GO */
  ["\uA80A", "\uA80A"], /* SYLOTI NAGRI LETTER GHO */
  ["\uA80B", "\uA80B"], /* SYLOTI NAGRI SIGN ANUSVARA */
  ["\uA80C", "\uA80C"], /* SYLOTI NAGRI LETTER CO */
  ["\uA80D", "\uA80D"], /* SYLOTI NAGRI LETTER CHO */
  ["\uA80E", "\uA80E"], /* SYLOTI NAGRI LETTER JO */
  ["\uA80F", "\uA80F"], /* SYLOTI NAGRI LETTER JHO */
  ["\uA810", "\uA810"], /* SYLOTI NAGRI LETTER TTO */
  ["\uA811", "\uA811"], /* SYLOTI NAGRI LETTER TTHO */
  ["\uA812", "\uA812"], /* SYLOTI NAGRI LETTER DDO */
  ["\uA813", "\uA813"], /* SYLOTI NAGRI LETTER DDHO */
  ["\uA814", "\uA814"], /* SYLOTI NAGRI LETTER TO */
  ["\uA815", "\uA815"], /* SYLOTI NAGRI LETTER THO */
  ["\uA816", "\uA816"], /* SYLOTI NAGRI LETTER DO */
  ["\uA817", "\uA817"], /* SYLOTI NAGRI LETTER DHO */
  ["\uA818", "\uA818"], /* SYLOTI NAGRI LETTER NO */
  ["\uA819", "\uA819"], /* SYLOTI NAGRI LETTER PO */
  ["\uA81A", "\uA81A"], /* SYLOTI NAGRI LETTER PHO */
  ["\uA81B", "\uA81B"], /* SYLOTI NAGRI LETTER BO */
  ["\uA81C", "\uA81C"], /* SYLOTI NAGRI LETTER BHO */
  ["\uA81D", "\uA81D"], /* SYLOTI NAGRI LETTER MO */
  ["\uA81E", "\uA81E"], /* SYLOTI NAGRI LETTER RO */
  ["\uA81F", "\uA81F"], /* SYLOTI NAGRI LETTER LO */
  ["\uA820", "\uA820"], /* SYLOTI NAGRI LETTER RRO */
  ["\uA821", "\uA821"], /* SYLOTI NAGRI LETTER SO */
  ["\uA822", "\uA822"], /* SYLOTI NAGRI LETTER HO */
  ["\uA823", "\uA823"], /* SYLOTI NAGRI VOWEL SIGN A */
  ["\uA824", "\uA824"], /* SYLOTI NAGRI VOWEL SIGN I */
  ["\uA825", "\uA825"], /* SYLOTI NAGRI VOWEL SIGN U */
  ["\uA826", "\uA826"], /* SYLOTI NAGRI VOWEL SIGN E */
  ["\uA827", "\uA827"], /* SYLOTI NAGRI VOWEL SIGN OO */
  ["\uA828", "\uA828"], /* SYLOTI NAGRI POETRY MARK-1 */
  ["\uA829", "\uA829"], /* SYLOTI NAGRI POETRY MARK-2 */
  ["\uA82A", "\uA82A"], /* SYLOTI NAGRI POETRY MARK-3 */
  ["\uA82B", "\uA82B"], /* SYLOTI NAGRI POETRY MARK-4 */
  ["\uA82C", "\uA82C"],
  ["\uA82D", "\uA82D"],
  ["\uA82E", "\uA82E"],
  ["\uA82F", "\uA82F"],
  ["\uA830", "\uA830"], /* NORTH INDIC FRACTION ONE QUARTER */
  ["\uA831", "\uA831"], /* NORTH INDIC FRACTION ONE HALF */
  ["\uA832", "\uA832"], /* NORTH INDIC FRACTION THREE QUARTERS */
  ["\uA833", "\uA833"], /* NORTH INDIC FRACTION ONE SIXTEENTH */
  ["\uA834", "\uA834"], /* NORTH INDIC FRACTION ONE EIGHTH */
  ["\uA835", "\uA835"], /* NORTH INDIC FRACTION THREE SIXTEENTHS */
  ["\uA836", "\uA836"], /* NORTH INDIC QUARTER MARK */
  ["\uA837", "\uA837"], /* NORTH INDIC PLACEHOLDER MARK */
  ["\uA838", "\uA838"], /* NORTH INDIC RUPEE MARK */
  ["\uA839", "\uA839"], /* NORTH INDIC QUANTITY MARK */
  ["\uA83A", "\uA83A"],
  ["\uA83B", "\uA83B"],
  ["\uA83C", "\uA83C"],
  ["\uA83D", "\uA83D"],
  ["\uA83E", "\uA83E"],
  ["\uA83F", "\uA83F"],
  ["\uA840", "\uA840"], /* PHAGS-PA LETTER KA */
  ["\uA841", "\uA841"], /* PHAGS-PA LETTER KHA */
  ["\uA842", "\uA842"], /* PHAGS-PA LETTER GA */
  ["\uA843", "\uA843"], /* PHAGS-PA LETTER NGA */
  ["\uA844", "\uA844"], /* PHAGS-PA LETTER CA */
  ["\uA845", "\uA845"], /* PHAGS-PA LETTER CHA */
  ["\uA846", "\uA846"], /* PHAGS-PA LETTER JA */
  ["\uA847", "\uA847"], /* PHAGS-PA LETTER NYA */
  ["\uA848", "\uA848"], /* PHAGS-PA LETTER TA */
  ["\uA849", "\uA849"], /* PHAGS-PA LETTER THA */
  ["\uA84A", "\uA84A"], /* PHAGS-PA LETTER DA */
  ["\uA84B", "\uA84B"], /* PHAGS-PA LETTER NA */
  ["\uA84C", "\uA84C"], /* PHAGS-PA LETTER PA */
  ["\uA84D", "\uA84D"], /* PHAGS-PA LETTER PHA */
  ["\uA84E", "\uA84E"], /* PHAGS-PA LETTER BA */
  ["\uA84F", "\uA84F"], /* PHAGS-PA LETTER MA */
  ["\uA850", "\uA850"], /* PHAGS-PA LETTER TSA */
  ["\uA851", "\uA851"], /* PHAGS-PA LETTER TSHA */
  ["\uA852", "\uA852"], /* PHAGS-PA LETTER DZA */
  ["\uA853", "\uA853"], /* PHAGS-PA LETTER WA */
  ["\uA854", "\uA854"], /* PHAGS-PA LETTER ZHA */
  ["\uA855", "\uA855"], /* PHAGS-PA LETTER ZA */
  ["\uA856", "\uA856"], /* PHAGS-PA LETTER SMALL A */
  ["\uA857", "\uA857"], /* PHAGS-PA LETTER YA */
  ["\uA858", "\uA858"], /* PHAGS-PA LETTER RA */
  ["\uA859", "\uA859"], /* PHAGS-PA LETTER LA */
  ["\uA85A", "\uA85A"], /* PHAGS-PA LETTER SHA */
  ["\uA85B", "\uA85B"], /* PHAGS-PA LETTER SA */
  ["\uA85C", "\uA85C"], /* PHAGS-PA LETTER HA */
  ["\uA85D", "\uA85D"], /* PHAGS-PA LETTER A */
  ["\uA85E", "\uA85E"], /* PHAGS-PA LETTER I */
  ["\uA85F", "\uA85F"], /* PHAGS-PA LETTER U */
  ["\uA860", "\uA860"], /* PHAGS-PA LETTER E */
  ["\uA861", "\uA861"], /* PHAGS-PA LETTER O */
  ["\uA862", "\uA862"], /* PHAGS-PA LETTER QA */
  ["\uA863", "\uA863"], /* PHAGS-PA LETTER XA */
  ["\uA864", "\uA864"], /* PHAGS-PA LETTER FA */
  ["\uA865", "\uA865"], /* PHAGS-PA LETTER GGA */
  ["\uA866", "\uA866"], /* PHAGS-PA LETTER EE */
  ["\uA867", "\uA867"], /* PHAGS-PA SUBJOINED LETTER WA */
  ["\uA868", "\uA868"], /* PHAGS-PA SUBJOINED LETTER YA */
  ["\uA869", "\uA869"], /* PHAGS-PA LETTER TTA */
  ["\uA86A", "\uA86A"], /* PHAGS-PA LETTER TTHA */
  ["\uA86B", "\uA86B"], /* PHAGS-PA LETTER DDA */
  ["\uA86C", "\uA86C"], /* PHAGS-PA LETTER NNA */
  ["\uA86D", "\uA86D"], /* PHAGS-PA LETTER ALTERNATE YA */
  ["\uA86E", "\uA86E"], /* PHAGS-PA LETTER VOICELESS SHA */
  ["\uA86F", "\uA86F"], /* PHAGS-PA LETTER VOICED HA */
  ["\uA870", "\uA870"], /* PHAGS-PA LETTER ASPIRATED FA */
  ["\uA871", "\uA871"], /* PHAGS-PA SUBJOINED LETTER RA */
  ["\uA872", "\uA872"], /* PHAGS-PA SUPERFIXED LETTER RA */
  ["\uA873", "\uA873"], /* PHAGS-PA LETTER CANDRABINDU */
  ["\uA874", "\uA874"], /* PHAGS-PA SINGLE HEAD MARK */
  ["\uA875", "\uA875"], /* PHAGS-PA DOUBLE HEAD MARK */
  ["\uA876", "\uA876"], /* PHAGS-PA MARK SHAD */
  ["\uA877", "\uA877"], /* PHAGS-PA MARK DOUBLE SHAD */
  ["\uA878", "\uA878"],
  ["\uA879", "\uA879"],
  ["\uA87A", "\uA87A"],
  ["\uA87B", "\uA87B"],
  ["\uA87C", "\uA87C"],
  ["\uA87D", "\uA87D"],
  ["\uA87E", "\uA87E"],
  ["\uA87F", "\uA87F"],
  ["\uA880", "\uA880"], /* SAURASHTRA SIGN ANUSVARA */
  ["\uA881", "\uA881"], /* SAURASHTRA SIGN VISARGA */
  ["\uA882", "\uA882"], /* SAURASHTRA LETTER A */
  ["\uA883", "\uA883"], /* SAURASHTRA LETTER AA */
  ["\uA884", "\uA884"], /* SAURASHTRA LETTER I */
  ["\uA885", "\uA885"], /* SAURASHTRA LETTER II */
  ["\uA886", "\uA886"], /* SAURASHTRA LETTER U */
  ["\uA887", "\uA887"], /* SAURASHTRA LETTER UU */
  ["\uA888", "\uA888"], /* SAURASHTRA LETTER VOCALIC R */
  ["\uA889", "\uA889"], /* SAURASHTRA LETTER VOCALIC RR */
  ["\uA88A", "\uA88A"], /* SAURASHTRA LETTER VOCALIC L */
  ["\uA88B", "\uA88B"], /* SAURASHTRA LETTER VOCALIC LL */
  ["\uA88C", "\uA88C"], /* SAURASHTRA LETTER E */
  ["\uA88D", "\uA88D"], /* SAURASHTRA LETTER EE */
  ["\uA88E", "\uA88E"], /* SAURASHTRA LETTER AI */
  ["\uA88F", "\uA88F"], /* SAURASHTRA LETTER O */
  ["\uA890", "\uA890"], /* SAURASHTRA LETTER OO */
  ["\uA891", "\uA891"], /* SAURASHTRA LETTER AU */
  ["\uA892", "\uA892"], /* SAURASHTRA LETTER KA */
  ["\uA893", "\uA893"], /* SAURASHTRA LETTER KHA */
  ["\uA894", "\uA894"], /* SAURASHTRA LETTER GA */
  ["\uA895", "\uA895"], /* SAURASHTRA LETTER GHA */
  ["\uA896", "\uA896"], /* SAURASHTRA LETTER NGA */
  ["\uA897", "\uA897"], /* SAURASHTRA LETTER CA */
  ["\uA898", "\uA898"], /* SAURASHTRA LETTER CHA */
  ["\uA899", "\uA899"], /* SAURASHTRA LETTER JA */
  ["\uA89A", "\uA89A"], /* SAURASHTRA LETTER JHA */
  ["\uA89B", "\uA89B"], /* SAURASHTRA LETTER NYA */
  ["\uA89C", "\uA89C"], /* SAURASHTRA LETTER TTA */
  ["\uA89D", "\uA89D"], /* SAURASHTRA LETTER TTHA */
  ["\uA89E", "\uA89E"], /* SAURASHTRA LETTER DDA */
  ["\uA89F", "\uA89F"], /* SAURASHTRA LETTER DDHA */
  ["\uA8A0", "\uA8A0"], /* SAURASHTRA LETTER NNA */
  ["\uA8A1", "\uA8A1"], /* SAURASHTRA LETTER TA */
  ["\uA8A2", "\uA8A2"], /* SAURASHTRA LETTER THA */
  ["\uA8A3", "\uA8A3"], /* SAURASHTRA LETTER DA */
  ["\uA8A4", "\uA8A4"], /* SAURASHTRA LETTER DHA */
  ["\uA8A5", "\uA8A5"], /* SAURASHTRA LETTER NA */
  ["\uA8A6", "\uA8A6"], /* SAURASHTRA LETTER PA */
  ["\uA8A7", "\uA8A7"], /* SAURASHTRA LETTER PHA */
  ["\uA8A8", "\uA8A8"], /* SAURASHTRA LETTER BA */
  ["\uA8A9", "\uA8A9"], /* SAURASHTRA LETTER BHA */
  ["\uA8AA", "\uA8AA"], /* SAURASHTRA LETTER MA */
  ["\uA8AB", "\uA8AB"], /* SAURASHTRA LETTER YA */
  ["\uA8AC", "\uA8AC"], /* SAURASHTRA LETTER RA */
  ["\uA8AD", "\uA8AD"], /* SAURASHTRA LETTER LA */
  ["\uA8AE", "\uA8AE"], /* SAURASHTRA LETTER VA */
  ["\uA8AF", "\uA8AF"], /* SAURASHTRA LETTER SHA */
  ["\uA8B0", "\uA8B0"], /* SAURASHTRA LETTER SSA */
  ["\uA8B1", "\uA8B1"], /* SAURASHTRA LETTER SA */
  ["\uA8B2", "\uA8B2"], /* SAURASHTRA LETTER HA */
  ["\uA8B3", "\uA8B3"], /* SAURASHTRA LETTER LLA */
  ["\uA8B4", "\uA8B4"], /* SAURASHTRA CONSONANT SIGN HAARU */
  ["\uA8B5", "\uA8B5"], /* SAURASHTRA VOWEL SIGN AA */
  ["\uA8B6", "\uA8B6"], /* SAURASHTRA VOWEL SIGN I */
  ["\uA8B7", "\uA8B7"], /* SAURASHTRA VOWEL SIGN II */
  ["\uA8B8", "\uA8B8"], /* SAURASHTRA VOWEL SIGN U */
  ["\uA8B9", "\uA8B9"], /* SAURASHTRA VOWEL SIGN UU */
  ["\uA8BA", "\uA8BA"], /* SAURASHTRA VOWEL SIGN VOCALIC R */
  ["\uA8BB", "\uA8BB"], /* SAURASHTRA VOWEL SIGN VOCALIC RR */
  ["\uA8BC", "\uA8BC"], /* SAURASHTRA VOWEL SIGN VOCALIC L */
  ["\uA8BD", "\uA8BD"], /* SAURASHTRA VOWEL SIGN VOCALIC LL */
  ["\uA8BE", "\uA8BE"], /* SAURASHTRA VOWEL SIGN E */
  ["\uA8BF", "\uA8BF"], /* SAURASHTRA VOWEL SIGN EE */
  ["\uA8C0", "\uA8C0"], /* SAURASHTRA VOWEL SIGN AI */
  ["\uA8C1", "\uA8C1"], /* SAURASHTRA VOWEL SIGN O */
  ["\uA8C2", "\uA8C2"], /* SAURASHTRA VOWEL SIGN OO */
  ["\uA8C3", "\uA8C3"], /* SAURASHTRA VOWEL SIGN AU */
  ["\uA8C4", "\uA8C4"], /* SAURASHTRA SIGN VIRAMA */
  ["\uA8C5", "\uA8C5"], /* SAURASHTRA SIGN CANDRABINDU */
  ["\uA8C6", "\uA8C6"],
  ["\uA8C7", "\uA8C7"],
  ["\uA8C8", "\uA8C8"],
  ["\uA8C9", "\uA8C9"],
  ["\uA8CA", "\uA8CA"],
  ["\uA8CB", "\uA8CB"],
  ["\uA8CC", "\uA8CC"],
  ["\uA8CD", "\uA8CD"],
  ["\uA8CE", "\uA8CE"], /* SAURASHTRA DANDA */
  ["\uA8CF", "\uA8CF"], /* SAURASHTRA DOUBLE DANDA */
  ["\uA8D0", "\uA8D0"], /* SAURASHTRA DIGIT ZERO */
  ["\uA8D1", "\uA8D1"], /* SAURASHTRA DIGIT ONE */
  ["\uA8D2", "\uA8D2"], /* SAURASHTRA DIGIT TWO */
  ["\uA8D3", "\uA8D3"], /* SAURASHTRA DIGIT THREE */
  ["\uA8D4", "\uA8D4"], /* SAURASHTRA DIGIT FOUR */
  ["\uA8D5", "\uA8D5"], /* SAURASHTRA DIGIT FIVE */
  ["\uA8D6", "\uA8D6"], /* SAURASHTRA DIGIT SIX */
  ["\uA8D7", "\uA8D7"], /* SAURASHTRA DIGIT SEVEN */
  ["\uA8D8", "\uA8D8"], /* SAURASHTRA DIGIT EIGHT */
  ["\uA8D9", "\uA8D9"], /* SAURASHTRA DIGIT NINE */
  ["\uA8DA", "\uA8DA"],
  ["\uA8DB", "\uA8DB"],
  ["\uA8DC", "\uA8DC"],
  ["\uA8DD", "\uA8DD"],
  ["\uA8DE", "\uA8DE"],
  ["\uA8DF", "\uA8DF"],
  ["\uA8E0", "\uA8E0"], /* COMBINING DEVANAGARI DIGIT ZERO */
  ["\uA8E1", "\uA8E1"], /* COMBINING DEVANAGARI DIGIT ONE */
  ["\uA8E2", "\uA8E2"], /* COMBINING DEVANAGARI DIGIT TWO */
  ["\uA8E3", "\uA8E3"], /* COMBINING DEVANAGARI DIGIT THREE */
  ["\uA8E4", "\uA8E4"], /* COMBINING DEVANAGARI DIGIT FOUR */
  ["\uA8E5", "\uA8E5"], /* COMBINING DEVANAGARI DIGIT FIVE */
  ["\uA8E6", "\uA8E6"], /* COMBINING DEVANAGARI DIGIT SIX */
  ["\uA8E7", "\uA8E7"], /* COMBINING DEVANAGARI DIGIT SEVEN */
  ["\uA8E8", "\uA8E8"], /* COMBINING DEVANAGARI DIGIT EIGHT */
  ["\uA8E9", "\uA8E9"], /* COMBINING DEVANAGARI DIGIT NINE */
  ["\uA8EA", "\uA8EA"], /* COMBINING DEVANAGARI LETTER A */
  ["\uA8EB", "\uA8EB"], /* COMBINING DEVANAGARI LETTER U */
  ["\uA8EC", "\uA8EC"], /* COMBINING DEVANAGARI LETTER KA */
  ["\uA8ED", "\uA8ED"], /* COMBINING DEVANAGARI LETTER NA */
  ["\uA8EE", "\uA8EE"], /* COMBINING DEVANAGARI LETTER PA */
  ["\uA8EF", "\uA8EF"], /* COMBINING DEVANAGARI LETTER RA */
  ["\uA8F0", "\uA8F0"], /* COMBINING DEVANAGARI LETTER VI */
  ["\uA8F1", "\uA8F1"], /* COMBINING DEVANAGARI SIGN AVAGRAHA */
  ["\uA8F2", "\uA8F2"], /* DEVANAGARI SIGN SPACING CANDRABINDU */
  ["\uA8F3", "\uA8F3"], /* DEVANAGARI SIGN CANDRABINDU VIRAMA */
  ["\uA8F4", "\uA8F4"], /* DEVANAGARI SIGN DOUBLE CANDRABINDU VIRAMA */
  ["\uA8F5", "\uA8F5"], /* DEVANAGARI SIGN CANDRABINDU TWO */
  ["\uA8F6", "\uA8F6"], /* DEVANAGARI SIGN CANDRABINDU THREE */
  ["\uA8F7", "\uA8F7"], /* DEVANAGARI SIGN CANDRABINDU AVAGRAHA */
  ["\uA8F8", "\uA8F8"], /* DEVANAGARI SIGN PUSHPIKA */
  ["\uA8F9", "\uA8F9"], /* DEVANAGARI GAP FILLER */
  ["\uA8FA", "\uA8FA"], /* DEVANAGARI CARET */
  ["\uA8FB", "\uA8FB"], /* DEVANAGARI HEADSTROKE */
  ["\uA8FC", "\uA8FC"], /* DEVANAGARI SIGN SIDDHAM */
  ["\uA8FD", "\uA8FD"], /* DEVANAGARI JAIN OM */
  ["\uA8FE", "\uA8FE"], /* DEVANAGARI LETTER AY */
  ["\uA8FF", "\uA8FF"], /* DEVANAGARI VOWEL SIGN AY */
  ["\uA900", "\uA900"], /* KAYAH LI DIGIT ZERO */
  ["\uA901", "\uA901"], /* KAYAH LI DIGIT ONE */
  ["\uA902", "\uA902"], /* KAYAH LI DIGIT TWO */
  ["\uA903", "\uA903"], /* KAYAH LI DIGIT THREE */
  ["\uA904", "\uA904"], /* KAYAH LI DIGIT FOUR */
  ["\uA905", "\uA905"], /* KAYAH LI DIGIT FIVE */
  ["\uA906", "\uA906"], /* KAYAH LI DIGIT SIX */
  ["\uA907", "\uA907"], /* KAYAH LI DIGIT SEVEN */
  ["\uA908", "\uA908"], /* KAYAH LI DIGIT EIGHT */
  ["\uA909", "\uA909"], /* KAYAH LI DIGIT NINE */
  ["\uA90A", "\uA90A"], /* KAYAH LI LETTER KA */
  ["\uA90B", "\uA90B"], /* KAYAH LI LETTER KHA */
  ["\uA90C", "\uA90C"], /* KAYAH LI LETTER GA */
  ["\uA90D", "\uA90D"], /* KAYAH LI LETTER NGA */
  ["\uA90E", "\uA90E"], /* KAYAH LI LETTER SA */
  ["\uA90F", "\uA90F"], /* KAYAH LI LETTER SHA */
  ["\uA910", "\uA910"], /* KAYAH LI LETTER ZA */
  ["\uA911", "\uA911"], /* KAYAH LI LETTER NYA */
  ["\uA912", "\uA912"], /* KAYAH LI LETTER TA */
  ["\uA913", "\uA913"], /* KAYAH LI LETTER HTA */
  ["\uA914", "\uA914"], /* KAYAH LI LETTER NA */
  ["\uA915", "\uA915"], /* KAYAH LI LETTER PA */
  ["\uA916", "\uA916"], /* KAYAH LI LETTER PHA */
  ["\uA917", "\uA917"], /* KAYAH LI LETTER MA */
  ["\uA918", "\uA918"], /* KAYAH LI LETTER DA */
  ["\uA919", "\uA919"], /* KAYAH LI LETTER BA */
  ["\uA91A", "\uA91A"], /* KAYAH LI LETTER RA */
  ["\uA91B", "\uA91B"], /* KAYAH LI LETTER YA */
  ["\uA91C", "\uA91C"], /* KAYAH LI LETTER LA */
  ["\uA91D", "\uA91D"], /* KAYAH LI LETTER WA */
  ["\uA91E", "\uA91E"], /* KAYAH LI LETTER THA */
  ["\uA91F", "\uA91F"], /* KAYAH LI LETTER HA */
  ["\uA920", "\uA920"], /* KAYAH LI LETTER VA */
  ["\uA921", "\uA921"], /* KAYAH LI LETTER CA */
  ["\uA922", "\uA922"], /* KAYAH LI LETTER A */
  ["\uA923", "\uA923"], /* KAYAH LI LETTER OE */
  ["\uA924", "\uA924"], /* KAYAH LI LETTER I */
  ["\uA925", "\uA925"], /* KAYAH LI LETTER OO */
  ["\uA926", "\uA926"], /* KAYAH LI VOWEL UE */
  ["\uA927", "\uA927"], /* KAYAH LI VOWEL E */
  ["\uA928", "\uA928"], /* KAYAH LI VOWEL U */
  ["\uA929", "\uA929"], /* KAYAH LI VOWEL EE */
  ["\uA92A", "\uA92A"], /* KAYAH LI VOWEL O */
  ["\uA92B", "\uA92B"], /* KAYAH LI TONE PLOPHU */
  ["\uA92C", "\uA92C"], /* KAYAH LI TONE CALYA */
  ["\uA92D", "\uA92D"], /* KAYAH LI TONE CALYA PLOPHU */
  ["\uA92E", "\uA92E"], /* KAYAH LI SIGN CWI */
  ["\uA92F", "\uA92F"], /* KAYAH LI SIGN SHYA */
  ["\uA930", "\uA930"], /* REJANG LETTER KA */
  ["\uA931", "\uA931"], /* REJANG LETTER GA */
  ["\uA932", "\uA932"], /* REJANG LETTER NGA */
  ["\uA933", "\uA933"], /* REJANG LETTER TA */
  ["\uA934", "\uA934"], /* REJANG LETTER DA */
  ["\uA935", "\uA935"], /* REJANG LETTER NA */
  ["\uA936", "\uA936"], /* REJANG LETTER PA */
  ["\uA937", "\uA937"], /* REJANG LETTER BA */
  ["\uA938", "\uA938"], /* REJANG LETTER MA */
  ["\uA939", "\uA939"], /* REJANG LETTER CA */
  ["\uA93A", "\uA93A"], /* REJANG LETTER JA */
  ["\uA93B", "\uA93B"], /* REJANG LETTER NYA */
  ["\uA93C", "\uA93C"], /* REJANG LETTER SA */
  ["\uA93D", "\uA93D"], /* REJANG LETTER RA */
  ["\uA93E", "\uA93E"], /* REJANG LETTER LA */
  ["\uA93F", "\uA93F"], /* REJANG LETTER YA */
  ["\uA940", "\uA940"], /* REJANG LETTER WA */
  ["\uA941", "\uA941"], /* REJANG LETTER HA */
  ["\uA942", "\uA942"], /* REJANG LETTER MBA */
  ["\uA943", "\uA943"], /* REJANG LETTER NGGA */
  ["\uA944", "\uA944"], /* REJANG LETTER NDA */
  ["\uA945", "\uA945"], /* REJANG LETTER NYJA */
  ["\uA946", "\uA946"], /* REJANG LETTER A */
  ["\uA947", "\uA947"], /* REJANG VOWEL SIGN I */
  ["\uA948", "\uA948"], /* REJANG VOWEL SIGN U */
  ["\uA949", "\uA949"], /* REJANG VOWEL SIGN E */
  ["\uA94A", "\uA94A"], /* REJANG VOWEL SIGN AI */
  ["\uA94B", "\uA94B"], /* REJANG VOWEL SIGN O */
  ["\uA94C", "\uA94C"], /* REJANG VOWEL SIGN AU */
  ["\uA94D", "\uA94D"], /* REJANG VOWEL SIGN EU */
  ["\uA94E", "\uA94E"], /* REJANG VOWEL SIGN EA */
  ["\uA94F", "\uA94F"], /* REJANG CONSONANT SIGN NG */
  ["\uA950", "\uA950"], /* REJANG CONSONANT SIGN N */
  ["\uA951", "\uA951"], /* REJANG CONSONANT SIGN R */
  ["\uA952", "\uA952"], /* REJANG CONSONANT SIGN H */
  ["\uA953", "\uA953"], /* REJANG VIRAMA */
  ["\uA954", "\uA954"],
  ["\uA955", "\uA955"],
  ["\uA956", "\uA956"],
  ["\uA957", "\uA957"],
  ["\uA958", "\uA958"],
  ["\uA959", "\uA959"],
  ["\uA95A", "\uA95A"],
  ["\uA95B", "\uA95B"],
  ["\uA95C", "\uA95C"],
  ["\uA95D", "\uA95D"],
  ["\uA95E", "\uA95E"],
  ["\uA95F", "\uA95F"], /* REJANG SECTION MARK */
  ["\uA960", "\uA960"], /* HANGUL CHOSEONG TIKEUT-MIEUM */
  ["\uA961", "\uA961"], /* HANGUL CHOSEONG TIKEUT-PIEUP */
  ["\uA962", "\uA962"], /* HANGUL CHOSEONG TIKEUT-SIOS */
  ["\uA963", "\uA963"], /* HANGUL CHOSEONG TIKEUT-CIEUC */
  ["\uA964", "\uA964"], /* HANGUL CHOSEONG RIEUL-KIYEOK */
  ["\uA965", "\uA965"], /* HANGUL CHOSEONG RIEUL-SSANGKIYEOK */
  ["\uA966", "\uA966"], /* HANGUL CHOSEONG RIEUL-TIKEUT */
  ["\uA967", "\uA967"], /* HANGUL CHOSEONG RIEUL-SSANGTIKEUT */
  ["\uA968", "\uA968"], /* HANGUL CHOSEONG RIEUL-MIEUM */
  ["\uA969", "\uA969"], /* HANGUL CHOSEONG RIEUL-PIEUP */
  ["\uA96A", "\uA96A"], /* HANGUL CHOSEONG RIEUL-SSANGPIEUP */
  ["\uA96B", "\uA96B"], /* HANGUL CHOSEONG RIEUL-KAPYEOUNPIEUP */
  ["\uA96C", "\uA96C"], /* HANGUL CHOSEONG RIEUL-SIOS */
  ["\uA96D", "\uA96D"], /* HANGUL CHOSEONG RIEUL-CIEUC */
  ["\uA96E", "\uA96E"], /* HANGUL CHOSEONG RIEUL-KHIEUKH */
  ["\uA96F", "\uA96F"], /* HANGUL CHOSEONG MIEUM-KIYEOK */
  ["\uA970", "\uA970"], /* HANGUL CHOSEONG MIEUM-TIKEUT */
  ["\uA971", "\uA971"], /* HANGUL CHOSEONG MIEUM-SIOS */
  ["\uA972", "\uA972"], /* HANGUL CHOSEONG PIEUP-SIOS-THIEUTH */
  ["\uA973", "\uA973"], /* HANGUL CHOSEONG PIEUP-KHIEUKH */
  ["\uA974", "\uA974"], /* HANGUL CHOSEONG PIEUP-HIEUH */
  ["\uA975", "\uA975"], /* HANGUL CHOSEONG SSANGSIOS-PIEUP */
  ["\uA976", "\uA976"], /* HANGUL CHOSEONG IEUNG-RIEUL */
  ["\uA977", "\uA977"], /* HANGUL CHOSEONG IEUNG-HIEUH */
  ["\uA978", "\uA978"], /* HANGUL CHOSEONG SSANGCIEUC-HIEUH */
  ["\uA979", "\uA979"], /* HANGUL CHOSEONG SSANGTHIEUTH */
  ["\uA97A", "\uA97A"], /* HANGUL CHOSEONG PHIEUPH-HIEUH */
  ["\uA97B", "\uA97B"], /* HANGUL CHOSEONG HIEUH-SIOS */
  ["\uA97C", "\uA97C"], /* HANGUL CHOSEONG SSANGYEORINHIEUH */
  ["\uA97D", "\uA97D"],
  ["\uA97E", "\uA97E"],
  ["\uA97F", "\uA97F"],
  ["\uA980", "\uA980"], /* JAVANESE SIGN PANYANGGA */
  ["\uA981", "\uA981"], /* JAVANESE SIGN CECAK */
  ["\uA982", "\uA982"], /* JAVANESE SIGN LAYAR */
  ["\uA983", "\uA983"], /* JAVANESE SIGN WIGNYAN */
  ["\uA984", "\uA984"], /* JAVANESE LETTER A */
  ["\uA985", "\uA985"], /* JAVANESE LETTER I KAWI */
  ["\uA986", "\uA986"], /* JAVANESE LETTER I */
  ["\uA987", "\uA987"], /* JAVANESE LETTER II */
  ["\uA988", "\uA988"], /* JAVANESE LETTER U */
  ["\uA989", "\uA989"], /* JAVANESE LETTER PA CEREK */
  ["\uA98A", "\uA98A"], /* JAVANESE LETTER NGA LELET */
  ["\uA98B", "\uA98B"], /* JAVANESE LETTER NGA LELET RASWADI */
  ["\uA98C", "\uA98C"], /* JAVANESE LETTER E */
  ["\uA98D", "\uA98D"], /* JAVANESE LETTER AI */
  ["\uA98E", "\uA98E"], /* JAVANESE LETTER O */
  ["\uA98F", "\uA98F"], /* JAVANESE LETTER KA */
  ["\uA990", "\uA990"], /* JAVANESE LETTER KA SASAK */
  ["\uA991", "\uA991"], /* JAVANESE LETTER KA MURDA */
  ["\uA992", "\uA992"], /* JAVANESE LETTER GA */
  ["\uA993", "\uA993"], /* JAVANESE LETTER GA MURDA */
  ["\uA994", "\uA994"], /* JAVANESE LETTER NGA */
  ["\uA995", "\uA995"], /* JAVANESE LETTER CA */
  ["\uA996", "\uA996"], /* JAVANESE LETTER CA MURDA */
  ["\uA997", "\uA997"], /* JAVANESE LETTER JA */
  ["\uA998", "\uA998"], /* JAVANESE LETTER NYA MURDA */
  ["\uA999", "\uA999"], /* JAVANESE LETTER JA MAHAPRANA */
  ["\uA99A", "\uA99A"], /* JAVANESE LETTER NYA */
  ["\uA99B", "\uA99B"], /* JAVANESE LETTER TTA */
  ["\uA99C", "\uA99C"], /* JAVANESE LETTER TTA MAHAPRANA */
  ["\uA99D", "\uA99D"], /* JAVANESE LETTER DDA */
  ["\uA99E", "\uA99E"], /* JAVANESE LETTER DDA MAHAPRANA */
  ["\uA99F", "\uA99F"], /* JAVANESE LETTER NA MURDA */
  ["\uA9A0", "\uA9A0"], /* JAVANESE LETTER TA */
  ["\uA9A1", "\uA9A1"], /* JAVANESE LETTER TA MURDA */
  ["\uA9A2", "\uA9A2"], /* JAVANESE LETTER DA */
  ["\uA9A3", "\uA9A3"], /* JAVANESE LETTER DA MAHAPRANA */
  ["\uA9A4", "\uA9A4"], /* JAVANESE LETTER NA */
  ["\uA9A5", "\uA9A5"], /* JAVANESE LETTER PA */
  ["\uA9A6", "\uA9A6"], /* JAVANESE LETTER PA MURDA */
  ["\uA9A7", "\uA9A7"], /* JAVANESE LETTER BA */
  ["\uA9A8", "\uA9A8"], /* JAVANESE LETTER BA MURDA */
  ["\uA9A9", "\uA9A9"], /* JAVANESE LETTER MA */
  ["\uA9AA", "\uA9AA"], /* JAVANESE LETTER YA */
  ["\uA9AB", "\uA9AB"], /* JAVANESE LETTER RA */
  ["\uA9AC", "\uA9AC"], /* JAVANESE LETTER RA AGUNG */
  ["\uA9AD", "\uA9AD"], /* JAVANESE LETTER LA */
  ["\uA9AE", "\uA9AE"], /* JAVANESE LETTER WA */
  ["\uA9AF", "\uA9AF"], /* JAVANESE LETTER SA MURDA */
  ["\uA9B0", "\uA9B0"], /* JAVANESE LETTER SA MAHAPRANA */
  ["\uA9B1", "\uA9B1"], /* JAVANESE LETTER SA */
  ["\uA9B2", "\uA9B2"], /* JAVANESE LETTER HA */
  ["\uA9B3", "\uA9B3"], /* JAVANESE SIGN CECAK TELU */
  ["\uA9B4", "\uA9B4"], /* JAVANESE VOWEL SIGN TARUNG */
  ["\uA9B5", "\uA9B5"], /* JAVANESE VOWEL SIGN TOLONG */
  ["\uA9B6", "\uA9B6"], /* JAVANESE VOWEL SIGN WULU */
  ["\uA9B7", "\uA9B7"], /* JAVANESE VOWEL SIGN WULU MELIK */
  ["\uA9B8", "\uA9B8"], /* JAVANESE VOWEL SIGN SUKU */
  ["\uA9B9", "\uA9B9"], /* JAVANESE VOWEL SIGN SUKU MENDUT */
  ["\uA9BA", "\uA9BA"], /* JAVANESE VOWEL SIGN TALING */
  ["\uA9BB", "\uA9BB"], /* JAVANESE VOWEL SIGN DIRGA MURE */
  ["\uA9BC", "\uA9BC"], /* JAVANESE VOWEL SIGN PEPET */
  ["\uA9BD", "\uA9BD"], /* JAVANESE CONSONANT SIGN KERET */
  ["\uA9BE", "\uA9BE"], /* JAVANESE CONSONANT SIGN PENGKAL */
  ["\uA9BF", "\uA9BF"], /* JAVANESE CONSONANT SIGN CAKRA */
  ["\uA9C0", "\uA9C0"], /* JAVANESE PANGKON */
  ["\uA9C1", "\uA9C1"], /* JAVANESE LEFT RERENGGAN */
  ["\uA9C2", "\uA9C2"], /* JAVANESE RIGHT RERENGGAN */
  ["\uA9C3", "\uA9C3"], /* JAVANESE PADA ANDAP */
  ["\uA9C4", "\uA9C4"], /* JAVANESE PADA MADYA */
  ["\uA9C5", "\uA9C5"], /* JAVANESE PADA LUHUR */
  ["\uA9C6", "\uA9C6"], /* JAVANESE PADA WINDU */
  ["\uA9C7", "\uA9C7"], /* JAVANESE PADA PANGKAT */
  ["\uA9C8", "\uA9C8"], /* JAVANESE PADA LINGSA */
  ["\uA9C9", "\uA9C9"], /* JAVANESE PADA LUNGSI */
  ["\uA9CA", "\uA9CA"], /* JAVANESE PADA ADEG */
  ["\uA9CB", "\uA9CB"], /* JAVANESE PADA ADEG ADEG */
  ["\uA9CC", "\uA9CC"], /* JAVANESE PADA PISELEH */
  ["\uA9CD", "\uA9CD"], /* JAVANESE TURNED PADA PISELEH */
  ["\uA9CE", "\uA9CE"],
  ["\uA9CF", "\uA9CF"], /* JAVANESE PANGRANGKEP */
  ["\uA9D0", "\uA9D0"], /* JAVANESE DIGIT ZERO */
  ["\uA9D1", "\uA9D1"], /* JAVANESE DIGIT ONE */
  ["\uA9D2", "\uA9D2"], /* JAVANESE DIGIT TWO */
  ["\uA9D3", "\uA9D3"], /* JAVANESE DIGIT THREE */
  ["\uA9D4", "\uA9D4"], /* JAVANESE DIGIT FOUR */
  ["\uA9D5", "\uA9D5"], /* JAVANESE DIGIT FIVE */
  ["\uA9D6", "\uA9D6"], /* JAVANESE DIGIT SIX */
  ["\uA9D7", "\uA9D7"], /* JAVANESE DIGIT SEVEN */
  ["\uA9D8", "\uA9D8"], /* JAVANESE DIGIT EIGHT */
  ["\uA9D9", "\uA9D9"], /* JAVANESE DIGIT NINE */
  ["\uA9DA", "\uA9DA"],
  ["\uA9DB", "\uA9DB"],
  ["\uA9DC", "\uA9DC"],
  ["\uA9DD", "\uA9DD"],
  ["\uA9DE", "\uA9DE"], /* JAVANESE PADA TIRTA TUMETES */
  ["\uA9DF", "\uA9DF"], /* JAVANESE PADA ISEN-ISEN */
  ["\uA9E0", "\uA9E0"], /* MYANMAR LETTER SHAN GHA */
  ["\uA9E1", "\uA9E1"], /* MYANMAR LETTER SHAN CHA */
  ["\uA9E2", "\uA9E2"], /* MYANMAR LETTER SHAN JHA */
  ["\uA9E3", "\uA9E3"], /* MYANMAR LETTER SHAN NNA */
  ["\uA9E4", "\uA9E4"], /* MYANMAR LETTER SHAN BHA */
  ["\uA9E5", "\uA9E5"], /* MYANMAR SIGN SHAN SAW */
  ["\uA9E6", "\uA9E6"], /* MYANMAR MODIFIER LETTER SHAN REDUPLICATION */
  ["\uA9E7", "\uA9E7"], /* MYANMAR LETTER TAI LAING NYA */
  ["\uA9E8", "\uA9E8"], /* MYANMAR LETTER TAI LAING FA */
  ["\uA9E9", "\uA9E9"], /* MYANMAR LETTER TAI LAING GA */
  ["\uA9EA", "\uA9EA"], /* MYANMAR LETTER TAI LAING GHA */
  ["\uA9EB", "\uA9EB"], /* MYANMAR LETTER TAI LAING JA */
  ["\uA9EC", "\uA9EC"], /* MYANMAR LETTER TAI LAING JHA */
  ["\uA9ED", "\uA9ED"], /* MYANMAR LETTER TAI LAING DDA */
  ["\uA9EE", "\uA9EE"], /* MYANMAR LETTER TAI LAING DDHA */
  ["\uA9EF", "\uA9EF"], /* MYANMAR LETTER TAI LAING NNA */
  ["\uA9F0", "\uA9F0"], /* MYANMAR TAI LAING DIGIT ZERO */
  ["\uA9F1", "\uA9F1"], /* MYANMAR TAI LAING DIGIT ONE */
  ["\uA9F2", "\uA9F2"], /* MYANMAR TAI LAING DIGIT TWO */
  ["\uA9F3", "\uA9F3"], /* MYANMAR TAI LAING DIGIT THREE */
  ["\uA9F4", "\uA9F4"], /* MYANMAR TAI LAING DIGIT FOUR */
  ["\uA9F5", "\uA9F5"], /* MYANMAR TAI LAING DIGIT FIVE */
  ["\uA9F6", "\uA9F6"], /* MYANMAR TAI LAING DIGIT SIX */
  ["\uA9F7", "\uA9F7"], /* MYANMAR TAI LAING DIGIT SEVEN */
  ["\uA9F8", "\uA9F8"], /* MYANMAR TAI LAING DIGIT EIGHT */
  ["\uA9F9", "\uA9F9"], /* MYANMAR TAI LAING DIGIT NINE */
  ["\uA9FA", "\uA9FA"], /* MYANMAR LETTER TAI LAING LLA */
  ["\uA9FB", "\uA9FB"], /* MYANMAR LETTER TAI LAING DA */
  ["\uA9FC", "\uA9FC"], /* MYANMAR LETTER TAI LAING DHA */
  ["\uA9FD", "\uA9FD"], /* MYANMAR LETTER TAI LAING BA */
  ["\uA9FE", "\uA9FE"], /* MYANMAR LETTER TAI LAING BHA */
  ["\uA9FF", "\uA9FF"],
  ["\uAA00", "\uAA00"], /* CHAM LETTER A */
  ["\uAA01", "\uAA01"], /* CHAM LETTER I */
  ["\uAA02", "\uAA02"], /* CHAM LETTER U */
  ["\uAA03", "\uAA03"], /* CHAM LETTER E */
  ["\uAA04", "\uAA04"], /* CHAM LETTER AI */
  ["\uAA05", "\uAA05"], /* CHAM LETTER O */
  ["\uAA06", "\uAA06"], /* CHAM LETTER KA */
  ["\uAA07", "\uAA07"], /* CHAM LETTER KHA */
  ["\uAA08", "\uAA08"], /* CHAM LETTER GA */
  ["\uAA09", "\uAA09"], /* CHAM LETTER GHA */
  ["\uAA0A", "\uAA0A"], /* CHAM LETTER NGUE */
  ["\uAA0B", "\uAA0B"], /* CHAM LETTER NGA */
  ["\uAA0C", "\uAA0C"], /* CHAM LETTER CHA */
  ["\uAA0D", "\uAA0D"], /* CHAM LETTER CHHA */
  ["\uAA0E", "\uAA0E"], /* CHAM LETTER JA */
  ["\uAA0F", "\uAA0F"], /* CHAM LETTER JHA */
  ["\uAA10", "\uAA10"], /* CHAM LETTER NHUE */
  ["\uAA11", "\uAA11"], /* CHAM LETTER NHA */
  ["\uAA12", "\uAA12"], /* CHAM LETTER NHJA */
  ["\uAA13", "\uAA13"], /* CHAM LETTER TA */
  ["\uAA14", "\uAA14"], /* CHAM LETTER THA */
  ["\uAA15", "\uAA15"], /* CHAM LETTER DA */
  ["\uAA16", "\uAA16"], /* CHAM LETTER DHA */
  ["\uAA17", "\uAA17"], /* CHAM LETTER NUE */
  ["\uAA18", "\uAA18"], /* CHAM LETTER NA */
  ["\uAA19", "\uAA19"], /* CHAM LETTER DDA */
  ["\uAA1A", "\uAA1A"], /* CHAM LETTER PA */
  ["\uAA1B", "\uAA1B"], /* CHAM LETTER PPA */
  ["\uAA1C", "\uAA1C"], /* CHAM LETTER PHA */
  ["\uAA1D", "\uAA1D"], /* CHAM LETTER BA */
  ["\uAA1E", "\uAA1E"], /* CHAM LETTER BHA */
  ["\uAA1F", "\uAA1F"], /* CHAM LETTER MUE */
  ["\uAA20", "\uAA20"], /* CHAM LETTER MA */
  ["\uAA21", "\uAA21"], /* CHAM LETTER BBA */
  ["\uAA22", "\uAA22"], /* CHAM LETTER YA */
  ["\uAA23", "\uAA23"], /* CHAM LETTER RA */
  ["\uAA24", "\uAA24"], /* CHAM LETTER LA */
  ["\uAA25", "\uAA25"], /* CHAM LETTER VA */
  ["\uAA26", "\uAA26"], /* CHAM LETTER SSA */
  ["\uAA27", "\uAA27"], /* CHAM LETTER SA */
  ["\uAA28", "\uAA28"], /* CHAM LETTER HA */
  ["\uAA29", "\uAA29"], /* CHAM VOWEL SIGN AA */
  ["\uAA2A", "\uAA2A"], /* CHAM VOWEL SIGN I */
  ["\uAA2B", "\uAA2B"], /* CHAM VOWEL SIGN II */
  ["\uAA2C", "\uAA2C"], /* CHAM VOWEL SIGN EI */
  ["\uAA2D", "\uAA2D"], /* CHAM VOWEL SIGN U */
  ["\uAA2E", "\uAA2E"], /* CHAM VOWEL SIGN OE */
  ["\uAA2F", "\uAA2F"], /* CHAM VOWEL SIGN O */
  ["\uAA30", "\uAA30"], /* CHAM VOWEL SIGN AI */
  ["\uAA31", "\uAA31"], /* CHAM VOWEL SIGN AU */
  ["\uAA32", "\uAA32"], /* CHAM VOWEL SIGN UE */
  ["\uAA33", "\uAA33"], /* CHAM CONSONANT SIGN YA */
  ["\uAA34", "\uAA34"], /* CHAM CONSONANT SIGN RA */
  ["\uAA35", "\uAA35"], /* CHAM CONSONANT SIGN LA */
  ["\uAA36", "\uAA36"], /* CHAM CONSONANT SIGN WA */
  ["\uAA37", "\uAA37"],
  ["\uAA38", "\uAA38"],
  ["\uAA39", "\uAA39"],
  ["\uAA3A", "\uAA3A"],
  ["\uAA3B", "\uAA3B"],
  ["\uAA3C", "\uAA3C"],
  ["\uAA3D", "\uAA3D"],
  ["\uAA3E", "\uAA3E"],
  ["\uAA3F", "\uAA3F"],
  ["\uAA40", "\uAA40"], /* CHAM LETTER FINAL K */
  ["\uAA41", "\uAA41"], /* CHAM LETTER FINAL G */
  ["\uAA42", "\uAA42"], /* CHAM LETTER FINAL NG */
  ["\uAA43", "\uAA43"], /* CHAM CONSONANT SIGN FINAL NG */
  ["\uAA44", "\uAA44"], /* CHAM LETTER FINAL CH */
  ["\uAA45", "\uAA45"], /* CHAM LETTER FINAL T */
  ["\uAA46", "\uAA46"], /* CHAM LETTER FINAL N */
  ["\uAA47", "\uAA47"], /* CHAM LETTER FINAL P */
  ["\uAA48", "\uAA48"], /* CHAM LETTER FINAL Y */
  ["\uAA49", "\uAA49"], /* CHAM LETTER FINAL R */
  ["\uAA4A", "\uAA4A"], /* CHAM LETTER FINAL L */
  ["\uAA4B", "\uAA4B"], /* CHAM LETTER FINAL SS */
  ["\uAA4C", "\uAA4C"], /* CHAM CONSONANT SIGN FINAL M */
  ["\uAA4D", "\uAA4D"], /* CHAM CONSONANT SIGN FINAL H */
  ["\uAA4E", "\uAA4E"],
  ["\uAA4F", "\uAA4F"],
  ["\uAA50", "\uAA50"], /* CHAM DIGIT ZERO */
  ["\uAA51", "\uAA51"], /* CHAM DIGIT ONE */
  ["\uAA52", "\uAA52"], /* CHAM DIGIT TWO */
  ["\uAA53", "\uAA53"], /* CHAM DIGIT THREE */
  ["\uAA54", "\uAA54"], /* CHAM DIGIT FOUR */
  ["\uAA55", "\uAA55"], /* CHAM DIGIT FIVE */
  ["\uAA56", "\uAA56"], /* CHAM DIGIT SIX */
  ["\uAA57", "\uAA57"], /* CHAM DIGIT SEVEN */
  ["\uAA58", "\uAA58"], /* CHAM DIGIT EIGHT */
  ["\uAA59", "\uAA59"], /* CHAM DIGIT NINE */
  ["\uAA5A", "\uAA5A"],
  ["\uAA5B", "\uAA5B"],
  ["\uAA5C", "\uAA5C"], /* CHAM PUNCTUATION SPIRAL */
  ["\uAA5D", "\uAA5D"], /* CHAM PUNCTUATION DANDA */
  ["\uAA5E", "\uAA5E"], /* CHAM PUNCTUATION DOUBLE DANDA */
  ["\uAA5F", "\uAA5F"], /* CHAM PUNCTUATION TRIPLE DANDA */
  ["\uAA60", "\uAA60"], /* MYANMAR LETTER KHAMTI GA */
  ["\uAA61", "\uAA61"], /* MYANMAR LETTER KHAMTI CA */
  ["\uAA62", "\uAA62"], /* MYANMAR LETTER KHAMTI CHA */
  ["\uAA63", "\uAA63"], /* MYANMAR LETTER KHAMTI JA */
  ["\uAA64", "\uAA64"], /* MYANMAR LETTER KHAMTI JHA */
  ["\uAA65", "\uAA65"], /* MYANMAR LETTER KHAMTI NYA */
  ["\uAA66", "\uAA66"], /* MYANMAR LETTER KHAMTI TTA */
  ["\uAA67", "\uAA67"], /* MYANMAR LETTER KHAMTI TTHA */
  ["\uAA68", "\uAA68"], /* MYANMAR LETTER KHAMTI DDA */
  ["\uAA69", "\uAA69"], /* MYANMAR LETTER KHAMTI DDHA */
  ["\uAA6A", "\uAA6A"], /* MYANMAR LETTER KHAMTI DHA */
  ["\uAA6B", "\uAA6B"], /* MYANMAR LETTER KHAMTI NA */
  ["\uAA6C", "\uAA6C"], /* MYANMAR LETTER KHAMTI SA */
  ["\uAA6D", "\uAA6D"], /* MYANMAR LETTER KHAMTI HA */
  ["\uAA6E", "\uAA6E"], /* MYANMAR LETTER KHAMTI HHA */
  ["\uAA6F", "\uAA6F"], /* MYANMAR LETTER KHAMTI FA */
  ["\uAA70", "\uAA70"], /* MYANMAR MODIFIER LETTER KHAMTI REDUPLICATION */
  ["\uAA71", "\uAA71"], /* MYANMAR LETTER KHAMTI XA */
  ["\uAA72", "\uAA72"], /* MYANMAR LETTER KHAMTI ZA */
  ["\uAA73", "\uAA73"], /* MYANMAR LETTER KHAMTI RA */
  ["\uAA74", "\uAA74"], /* MYANMAR LOGOGRAM KHAMTI OAY */
  ["\uAA75", "\uAA75"], /* MYANMAR LOGOGRAM KHAMTI QN */
  ["\uAA76", "\uAA76"], /* MYANMAR LOGOGRAM KHAMTI HM */
  ["\uAA77", "\uAA77"], /* MYANMAR SYMBOL AITON EXCLAMATION */
  ["\uAA78", "\uAA78"], /* MYANMAR SYMBOL AITON ONE */
  ["\uAA79", "\uAA79"], /* MYANMAR SYMBOL AITON TWO */
  ["\uAA7A", "\uAA7A"], /* MYANMAR LETTER AITON RA */
  ["\uAA7B", "\uAA7B"], /* MYANMAR SIGN PAO KAREN TONE */
  ["\uAA7C", "\uAA7C"], /* MYANMAR SIGN TAI LAING TONE-2 */
  ["\uAA7D", "\uAA7D"], /* MYANMAR SIGN TAI LAING TONE-5 */
  ["\uAA7E", "\uAA7E"], /* MYANMAR LETTER SHWE PALAUNG CHA */
  ["\uAA7F", "\uAA7F"], /* MYANMAR LETTER SHWE PALAUNG SHA */
  ["\uAA80", "\uAA80"], /* TAI VIET LETTER LOW KO */
  ["\uAA81", "\uAA81"], /* TAI VIET LETTER HIGH KO */
  ["\uAA82", "\uAA82"], /* TAI VIET LETTER LOW KHO */
  ["\uAA83", "\uAA83"], /* TAI VIET LETTER HIGH KHO */
  ["\uAA84", "\uAA84"], /* TAI VIET LETTER LOW KHHO */
  ["\uAA85", "\uAA85"], /* TAI VIET LETTER HIGH KHHO */
  ["\uAA86", "\uAA86"], /* TAI VIET LETTER LOW GO */
  ["\uAA87", "\uAA87"], /* TAI VIET LETTER HIGH GO */
  ["\uAA88", "\uAA88"], /* TAI VIET LETTER LOW NGO */
  ["\uAA89", "\uAA89"], /* TAI VIET LETTER HIGH NGO */
  ["\uAA8A", "\uAA8A"], /* TAI VIET LETTER LOW CO */
  ["\uAA8B", "\uAA8B"], /* TAI VIET LETTER HIGH CO */
  ["\uAA8C", "\uAA8C"], /* TAI VIET LETTER LOW CHO */
  ["\uAA8D", "\uAA8D"], /* TAI VIET LETTER HIGH CHO */
  ["\uAA8E", "\uAA8E"], /* TAI VIET LETTER LOW SO */
  ["\uAA8F", "\uAA8F"], /* TAI VIET LETTER HIGH SO */
  ["\uAA90", "\uAA90"], /* TAI VIET LETTER LOW NYO */
  ["\uAA91", "\uAA91"], /* TAI VIET LETTER HIGH NYO */
  ["\uAA92", "\uAA92"], /* TAI VIET LETTER LOW DO */
  ["\uAA93", "\uAA93"], /* TAI VIET LETTER HIGH DO */
  ["\uAA94", "\uAA94"], /* TAI VIET LETTER LOW TO */
  ["\uAA95", "\uAA95"], /* TAI VIET LETTER HIGH TO */
  ["\uAA96", "\uAA96"], /* TAI VIET LETTER LOW THO */
  ["\uAA97", "\uAA97"], /* TAI VIET LETTER HIGH THO */
  ["\uAA98", "\uAA98"], /* TAI VIET LETTER LOW NO */
  ["\uAA99", "\uAA99"], /* TAI VIET LETTER HIGH NO */
  ["\uAA9A", "\uAA9A"], /* TAI VIET LETTER LOW BO */
  ["\uAA9B", "\uAA9B"], /* TAI VIET LETTER HIGH BO */
  ["\uAA9C", "\uAA9C"], /* TAI VIET LETTER LOW PO */
  ["\uAA9D", "\uAA9D"], /* TAI VIET LETTER HIGH PO */
  ["\uAA9E", "\uAA9E"], /* TAI VIET LETTER LOW PHO */
  ["\uAA9F", "\uAA9F"], /* TAI VIET LETTER HIGH PHO */
  ["\uAAA0", "\uAAA0"], /* TAI VIET LETTER LOW FO */
  ["\uAAA1", "\uAAA1"], /* TAI VIET LETTER HIGH FO */
  ["\uAAA2", "\uAAA2"], /* TAI VIET LETTER LOW MO */
  ["\uAAA3", "\uAAA3"], /* TAI VIET LETTER HIGH MO */
  ["\uAAA4", "\uAAA4"], /* TAI VIET LETTER LOW YO */
  ["\uAAA5", "\uAAA5"], /* TAI VIET LETTER HIGH YO */
  ["\uAAA6", "\uAAA6"], /* TAI VIET LETTER LOW RO */
  ["\uAAA7", "\uAAA7"], /* TAI VIET LETTER HIGH RO */
  ["\uAAA8", "\uAAA8"], /* TAI VIET LETTER LOW LO */
  ["\uAAA9", "\uAAA9"], /* TAI VIET LETTER HIGH LO */
  ["\uAAAA", "\uAAAA"], /* TAI VIET LETTER LOW VO */
  ["\uAAAB", "\uAAAB"], /* TAI VIET LETTER HIGH VO */
  ["\uAAAC", "\uAAAC"], /* TAI VIET LETTER LOW HO */
  ["\uAAAD", "\uAAAD"], /* TAI VIET LETTER HIGH HO */
  ["\uAAAE", "\uAAAE"], /* TAI VIET LETTER LOW O */
  ["\uAAAF", "\uAAAF"], /* TAI VIET LETTER HIGH O */
  ["\uAAB0", "\uAAB0"], /* TAI VIET MAI KANG */
  ["\uAAB1", "\uAAB1"], /* TAI VIET VOWEL AA */
  ["\uAAB2", "\uAAB2"], /* TAI VIET VOWEL I */
  ["\uAAB3", "\uAAB3"], /* TAI VIET VOWEL UE */
  ["\uAAB4", "\uAAB4"], /* TAI VIET VOWEL U */
  ["\uAAB5", "\uAAB5"], /* TAI VIET VOWEL E */
  ["\uAAB6", "\uAAB6"], /* TAI VIET VOWEL O */
  ["\uAAB7", "\uAAB7"], /* TAI VIET MAI KHIT */
  ["\uAAB8", "\uAAB8"], /* TAI VIET VOWEL IA */
  ["\uAAB9", "\uAAB9"], /* TAI VIET VOWEL UEA */
  ["\uAABA", "\uAABA"], /* TAI VIET VOWEL UA */
  ["\uAABB", "\uAABB"], /* TAI VIET VOWEL AUE */
  ["\uAABC", "\uAABC"], /* TAI VIET VOWEL AY */
  ["\uAABD", "\uAABD"], /* TAI VIET VOWEL AN */
  ["\uAABE", "\uAABE"], /* TAI VIET VOWEL AM */
  ["\uAABF", "\uAABF"], /* TAI VIET TONE MAI EK */
  ["\uAAC0", "\uAAC0"], /* TAI VIET TONE MAI NUENG */
  ["\uAAC1", "\uAAC1"], /* TAI VIET TONE MAI THO */
  ["\uAAC2", "\uAAC2"], /* TAI VIET TONE MAI SONG */
  ["\uAAC3", "\uAAC3"],
  ["\uAAC4", "\uAAC4"],
  ["\uAAC5", "\uAAC5"],
  ["\uAAC6", "\uAAC6"],
  ["\uAAC7", "\uAAC7"],
  ["\uAAC8", "\uAAC8"],
  ["\uAAC9", "\uAAC9"],
  ["\uAACA", "\uAACA"],
  ["\uAACB", "\uAACB"],
  ["\uAACC", "\uAACC"],
  ["\uAACD", "\uAACD"],
  ["\uAACE", "\uAACE"],
  ["\uAACF", "\uAACF"],
  ["\uAAD0", "\uAAD0"],
  ["\uAAD1", "\uAAD1"],
  ["\uAAD2", "\uAAD2"],
  ["\uAAD3", "\uAAD3"],
  ["\uAAD4", "\uAAD4"],
  ["\uAAD5", "\uAAD5"],
  ["\uAAD6", "\uAAD6"],
  ["\uAAD7", "\uAAD7"],
  ["\uAAD8", "\uAAD8"],
  ["\uAAD9", "\uAAD9"],
  ["\uAADA", "\uAADA"],
  ["\uAADB", "\uAADB"], /* TAI VIET SYMBOL KON */
  ["\uAADC", "\uAADC"], /* TAI VIET SYMBOL NUENG */
  ["\uAADD", "\uAADD"], /* TAI VIET SYMBOL SAM */
  ["\uAADE", "\uAADE"], /* TAI VIET SYMBOL HO HOI */
  ["\uAADF", "\uAADF"], /* TAI VIET SYMBOL KOI KOI */
  ["\uAAE0", "\uAAE0"], /* MEETEI MAYEK LETTER E */
  ["\uAAE1", "\uAAE1"], /* MEETEI MAYEK LETTER O */
  ["\uAAE2", "\uAAE2"], /* MEETEI MAYEK LETTER CHA */
  ["\uAAE3", "\uAAE3"], /* MEETEI MAYEK LETTER NYA */
  ["\uAAE4", "\uAAE4"], /* MEETEI MAYEK LETTER TTA */
  ["\uAAE5", "\uAAE5"], /* MEETEI MAYEK LETTER TTHA */
  ["\uAAE6", "\uAAE6"], /* MEETEI MAYEK LETTER DDA */
  ["\uAAE7", "\uAAE7"], /* MEETEI MAYEK LETTER DDHA */
  ["\uAAE8", "\uAAE8"], /* MEETEI MAYEK LETTER NNA */
  ["\uAAE9", "\uAAE9"], /* MEETEI MAYEK LETTER SHA */
  ["\uAAEA", "\uAAEA"], /* MEETEI MAYEK LETTER SSA */
  ["\uAAEB", "\uAAEB"], /* MEETEI MAYEK VOWEL SIGN II */
  ["\uAAEC", "\uAAEC"], /* MEETEI MAYEK VOWEL SIGN UU */
  ["\uAAED", "\uAAED"], /* MEETEI MAYEK VOWEL SIGN AAI */
  ["\uAAEE", "\uAAEE"], /* MEETEI MAYEK VOWEL SIGN AU */
  ["\uAAEF", "\uAAEF"], /* MEETEI MAYEK VOWEL SIGN AAU */
  ["\uAAF0", "\uAAF0"], /* MEETEI MAYEK CHEIKHAN */
  ["\uAAF1", "\uAAF1"], /* MEETEI MAYEK AHANG KHUDAM */
  ["\uAAF2", "\uAAF2"], /* MEETEI MAYEK ANJI */
  ["\uAAF3", "\uAAF3"], /* MEETEI MAYEK SYLLABLE REPETITION MARK */
  ["\uAAF4", "\uAAF4"], /* MEETEI MAYEK WORD REPETITION MARK */
  ["\uAAF5", "\uAAF5"], /* MEETEI MAYEK VOWEL SIGN VISARGA */
  ["\uAAF6", "\uAAF6"], /* MEETEI MAYEK VIRAMA */
  ["\uAAF7", "\uAAF7"],
  ["\uAAF8", "\uAAF8"],
  ["\uAAF9", "\uAAF9"],
  ["\uAAFA", "\uAAFA"],
  ["\uAAFB", "\uAAFB"],
  ["\uAAFC", "\uAAFC"],
  ["\uAAFD", "\uAAFD"],
  ["\uAAFE", "\uAAFE"],
  ["\uAAFF", "\uAAFF"],
  ["\uAB00", "\uAB00"],
  ["\uAB01", "\uAB01"], /* ETHIOPIC SYLLABLE TTHU */
  ["\uAB02", "\uAB02"], /* ETHIOPIC SYLLABLE TTHI */
  ["\uAB03", "\uAB03"], /* ETHIOPIC SYLLABLE TTHAA */
  ["\uAB04", "\uAB04"], /* ETHIOPIC SYLLABLE TTHEE */
  ["\uAB05", "\uAB05"], /* ETHIOPIC SYLLABLE TTHE */
  ["\uAB06", "\uAB06"], /* ETHIOPIC SYLLABLE TTHO */
  ["\uAB07", "\uAB07"],
  ["\uAB08", "\uAB08"],
  ["\uAB09", "\uAB09"], /* ETHIOPIC SYLLABLE DDHU */
  ["\uAB0A", "\uAB0A"], /* ETHIOPIC SYLLABLE DDHI */
  ["\uAB0B", "\uAB0B"], /* ETHIOPIC SYLLABLE DDHAA */
  ["\uAB0C", "\uAB0C"], /* ETHIOPIC SYLLABLE DDHEE */
  ["\uAB0D", "\uAB0D"], /* ETHIOPIC SYLLABLE DDHE */
  ["\uAB0E", "\uAB0E"], /* ETHIOPIC SYLLABLE DDHO */
  ["\uAB0F", "\uAB0F"],
  ["\uAB10", "\uAB10"],
  ["\uAB11", "\uAB11"], /* ETHIOPIC SYLLABLE DZU */
  ["\uAB12", "\uAB12"], /* ETHIOPIC SYLLABLE DZI */
  ["\uAB13", "\uAB13"], /* ETHIOPIC SYLLABLE DZAA */
  ["\uAB14", "\uAB14"], /* ETHIOPIC SYLLABLE DZEE */
  ["\uAB15", "\uAB15"], /* ETHIOPIC SYLLABLE DZE */
  ["\uAB16", "\uAB16"], /* ETHIOPIC SYLLABLE DZO */
  ["\uAB17", "\uAB17"],
  ["\uAB18", "\uAB18"],
  ["\uAB19", "\uAB19"],
  ["\uAB1A", "\uAB1A"],
  ["\uAB1B", "\uAB1B"],
  ["\uAB1C", "\uAB1C"],
  ["\uAB1D", "\uAB1D"],
  ["\uAB1E", "\uAB1E"],
  ["\uAB1F", "\uAB1F"],
  ["\uAB20", "\uAB20"], /* ETHIOPIC SYLLABLE CCHHA */
  ["\uAB21", "\uAB21"], /* ETHIOPIC SYLLABLE CCHHU */
  ["\uAB22", "\uAB22"], /* ETHIOPIC SYLLABLE CCHHI */
  ["\uAB23", "\uAB23"], /* ETHIOPIC SYLLABLE CCHHAA */
  ["\uAB24", "\uAB24"], /* ETHIOPIC SYLLABLE CCHHEE */
  ["\uAB25", "\uAB25"], /* ETHIOPIC SYLLABLE CCHHE */
  ["\uAB26", "\uAB26"], /* ETHIOPIC SYLLABLE CCHHO */
  ["\uAB27", "\uAB27"],
  ["\uAB28", "\uAB28"], /* ETHIOPIC SYLLABLE BBA */
  ["\uAB29", "\uAB29"], /* ETHIOPIC SYLLABLE BBU */
  ["\uAB2A", "\uAB2A"], /* ETHIOPIC SYLLABLE BBI */
  ["\uAB2B", "\uAB2B"], /* ETHIOPIC SYLLABLE BBAA */
  ["\uAB2C", "\uAB2C"], /* ETHIOPIC SYLLABLE BBEE */
  ["\uAB2D", "\uAB2D"], /* ETHIOPIC SYLLABLE BBE */
  ["\uAB2E", "\uAB2E"], /* ETHIOPIC SYLLABLE BBO */
  ["\uAB2F", "\uAB2F"],
  ["\uAB30", "\uAB30"], /* LATIN SMALL LETTER BARRED ALPHA */
  ["\uAB31", "\uAB31"], /* LATIN SMALL LETTER A REVERSED-SCHWA */
  ["\uAB32", "\uAB32"], /* LATIN SMALL LETTER BLACKLETTER E */
  ["\uAB33", "\uAB33"], /* LATIN SMALL LETTER BARRED E */
  ["\uAB34", "\uAB34"], /* LATIN SMALL LETTER E WITH FLOURISH */
  ["\uAB35", "\uAB35"], /* LATIN SMALL LETTER LENIS F */
  ["\uAB36", "\uAB36"], /* LATIN SMALL LETTER SCRIPT G WITH CROSSED-TAIL */
  ["\uAB37", "\uAB37"], /* LATIN SMALL LETTER L WITH INVERTED LAZY S */
  ["\uAB38", "\uAB38"], /* LATIN SMALL LETTER L WITH DOUBLE MIDDLE TILDE */
  ["\uAB39", "\uAB39"], /* LATIN SMALL LETTER L WITH MIDDLE RING */
  ["\uAB3A", "\uAB3A"], /* LATIN SMALL LETTER M WITH CROSSED-TAIL */
  ["\uAB3B", "\uAB3B"], /* LATIN SMALL LETTER N WITH CROSSED-TAIL */
  ["\uAB3C", "\uAB3C"], /* LATIN SMALL LETTER ENG WITH CROSSED-TAIL */
  ["\uAB3D", "\uAB3D"], /* LATIN SMALL LETTER BLACKLETTER O */
  ["\uAB3E", "\uAB3E"], /* LATIN SMALL LETTER BLACKLETTER O WITH STROKE */
  ["\uAB3F", "\uAB3F"], /* LATIN SMALL LETTER OPEN O WITH STROKE */
  ["\uAB40", "\uAB40"], /* LATIN SMALL LETTER INVERTED OE */
  ["\uAB41", "\uAB41"], /* LATIN SMALL LETTER TURNED OE WITH STROKE */
  ["\uAB42", "\uAB42"], /* LATIN SMALL LETTER TURNED OE WITH HORIZONTAL STROKE */
  ["\uAB43", "\uAB43"], /* LATIN SMALL LETTER TURNED O OPEN-O */
  ["\uAB44", "\uAB44"], /* LATIN SMALL LETTER TURNED O OPEN-O WITH STROKE */
  ["\uAB45", "\uAB45"], /* LATIN SMALL LETTER STIRRUP R */
  ["\uAB46", "\uAB46"], /* LATIN LETTER SMALL CAPITAL R WITH RIGHT LEG */
  ["\uAB47", "\uAB47"], /* LATIN SMALL LETTER R WITHOUT HANDLE */
  ["\uAB48", "\uAB48"], /* LATIN SMALL LETTER DOUBLE R */
  ["\uAB49", "\uAB49"], /* LATIN SMALL LETTER R WITH CROSSED-TAIL */
  ["\uAB4A", "\uAB4A"], /* LATIN SMALL LETTER DOUBLE R WITH CROSSED-TAIL */
  ["\uAB4B", "\uAB4B"], /* LATIN SMALL LETTER SCRIPT R */
  ["\uAB4C", "\uAB4C"], /* LATIN SMALL LETTER SCRIPT R WITH RING */
  ["\uAB4D", "\uAB4D"], /* LATIN SMALL LETTER BASELINE ESH */
  ["\uAB4E", "\uAB4E"], /* LATIN SMALL LETTER U WITH SHORT RIGHT LEG */
  ["\uAB4F", "\uAB4F"], /* LATIN SMALL LETTER U BAR WITH SHORT RIGHT LEG */
  ["\uAB50", "\uAB50"], /* LATIN SMALL LETTER UI */
  ["\uAB51", "\uAB51"], /* LATIN SMALL LETTER TURNED UI */
  ["\uAB52", "\uAB52"], /* LATIN SMALL LETTER U WITH LEFT HOOK */
  ["\uA7B3", "\uAB53"], /* LATIN SMALL LETTER CHI */
  ["\uAB54", "\uAB54"], /* LATIN SMALL LETTER CHI WITH LOW RIGHT RING */
  ["\uAB55", "\uAB55"], /* LATIN SMALL LETTER CHI WITH LOW LEFT SERIF */
  ["\uAB56", "\uAB56"], /* LATIN SMALL LETTER X WITH LOW RIGHT RING */
  ["\uAB57", "\uAB57"], /* LATIN SMALL LETTER X WITH LONG LEFT LEG */
  ["\uAB58", "\uAB58"], /* LATIN SMALL LETTER X WITH LONG LEFT LEG AND LOW RIGHT RING */
  ["\uAB59", "\uAB59"], /* LATIN SMALL LETTER X WITH LONG LEFT LEG WITH SERIF */
  ["\uAB5A", "\uAB5A"], /* LATIN SMALL LETTER Y WITH SHORT RIGHT LEG */
  ["\uAB5B", "\uAB5B"], /* MODIFIER BREVE WITH INVERTED BREVE */
  ["\uAB5C", "\uAB5C"], /* MODIFIER LETTER SMALL HENG */
  ["\uAB5D", "\uAB5D"], /* MODIFIER LETTER SMALL L WITH INVERTED LAZY S */
  ["\uAB5E", "\uAB5E"], /* MODIFIER LETTER SMALL L WITH MIDDLE TILDE */
  ["\uAB5F", "\uAB5F"], /* MODIFIER LETTER SMALL U WITH LEFT HOOK */
  ["\uAB60", "\uAB60"], /* LATIN SMALL LETTER SAKHA YAT */
  ["\uAB61", "\uAB61"], /* LATIN SMALL LETTER IOTIFIED E */
  ["\uAB62", "\uAB62"], /* LATIN SMALL LETTER OPEN OE */
  ["\uAB63", "\uAB63"], /* LATIN SMALL LETTER UO */
  ["\uAB64", "\uAB64"], /* LATIN SMALL LETTER INVERTED ALPHA */
  ["\uAB65", "\uAB65"], /* GREEK LETTER SMALL CAPITAL OMEGA */
  ["\uAB66", "\uAB66"], /* LATIN SMALL LETTER DZ DIGRAPH WITH RETROFLEX HOOK */
  ["\uAB67", "\uAB67"], /* LATIN SMALL LETTER TS DIGRAPH WITH RETROFLEX HOOK */
  ["\uAB68", "\uAB68"],
  ["\uAB69", "\uAB69"],
  ["\uAB6A", "\uAB6A"],
  ["\uAB6B", "\uAB6B"],
  ["\uAB6C", "\uAB6C"],
  ["\uAB6D", "\uAB6D"],
  ["\uAB6E", "\uAB6E"],
  ["\uAB6F", "\uAB6F"],
  ["\u13A0", "\uAB70"], /* CHEROKEE SMALL LETTER A */
  ["\u13A1", "\uAB71"], /* CHEROKEE SMALL LETTER E */
  ["\u13A2", "\uAB72"], /* CHEROKEE SMALL LETTER I */
  ["\u13A3", "\uAB73"], /* CHEROKEE SMALL LETTER O */
  ["\u13A4", "\uAB74"], /* CHEROKEE SMALL LETTER U */
  ["\u13A5", "\uAB75"], /* CHEROKEE SMALL LETTER V */
  ["\u13A6", "\uAB76"], /* CHEROKEE SMALL LETTER GA */
  ["\u13A7", "\uAB77"], /* CHEROKEE SMALL LETTER KA */
  ["\u13A8", "\uAB78"], /* CHEROKEE SMALL LETTER GE */
  ["\u13A9", "\uAB79"], /* CHEROKEE SMALL LETTER GI */
  ["\u13AA", "\uAB7A"], /* CHEROKEE SMALL LETTER GO */
  ["\u13AB", "\uAB7B"], /* CHEROKEE SMALL LETTER GU */
  ["\u13AC", "\uAB7C"], /* CHEROKEE SMALL LETTER GV */
  ["\u13AD", "\uAB7D"], /* CHEROKEE SMALL LETTER HA */
  ["\u13AE", "\uAB7E"], /* CHEROKEE SMALL LETTER HE */
  ["\u13AF", "\uAB7F"], /* CHEROKEE SMALL LETTER HI */
  ["\u13B0", "\uAB80"], /* CHEROKEE SMALL LETTER HO */
  ["\u13B1", "\uAB81"], /* CHEROKEE SMALL LETTER HU */
  ["\u13B2", "\uAB82"], /* CHEROKEE SMALL LETTER HV */
  ["\u13B3", "\uAB83"], /* CHEROKEE SMALL LETTER LA */
  ["\u13B4", "\uAB84"], /* CHEROKEE SMALL LETTER LE */
  ["\u13B5", "\uAB85"], /* CHEROKEE SMALL LETTER LI */
  ["\u13B6", "\uAB86"], /* CHEROKEE SMALL LETTER LO */
  ["\u13B7", "\uAB87"], /* CHEROKEE SMALL LETTER LU */
  ["\u13B8", "\uAB88"], /* CHEROKEE SMALL LETTER LV */
  ["\u13B9", "\uAB89"], /* CHEROKEE SMALL LETTER MA */
  ["\u13BA", "\uAB8A"], /* CHEROKEE SMALL LETTER ME */
  ["\u13BB", "\uAB8B"], /* CHEROKEE SMALL LETTER MI */
  ["\u13BC", "\uAB8C"], /* CHEROKEE SMALL LETTER MO */
  ["\u13BD", "\uAB8D"], /* CHEROKEE SMALL LETTER MU */
  ["\u13BE", "\uAB8E"], /* CHEROKEE SMALL LETTER NA */
  ["\u13BF", "\uAB8F"], /* CHEROKEE SMALL LETTER HNA */
  ["\u13C0", "\uAB90"], /* CHEROKEE SMALL LETTER NAH */
  ["\u13C1", "\uAB91"], /* CHEROKEE SMALL LETTER NE */
  ["\u13C2", "\uAB92"], /* CHEROKEE SMALL LETTER NI */
  ["\u13C3", "\uAB93"], /* CHEROKEE SMALL LETTER NO */
  ["\u13C4", "\uAB94"], /* CHEROKEE SMALL LETTER NU */
  ["\u13C5", "\uAB95"], /* CHEROKEE SMALL LETTER NV */
  ["\u13C6", "\uAB96"], /* CHEROKEE SMALL LETTER QUA */
  ["\u13C7", "\uAB97"], /* CHEROKEE SMALL LETTER QUE */
  ["\u13C8", "\uAB98"], /* CHEROKEE SMALL LETTER QUI */
  ["\u13C9", "\uAB99"], /* CHEROKEE SMALL LETTER QUO */
  ["\u13CA", "\uAB9A"], /* CHEROKEE SMALL LETTER QUU */
  ["\u13CB", "\uAB9B"], /* CHEROKEE SMALL LETTER QUV */
  ["\u13CC", "\uAB9C"], /* CHEROKEE SMALL LETTER SA */
  ["\u13CD", "\uAB9D"], /* CHEROKEE SMALL LETTER S */
  ["\u13CE", "\uAB9E"], /* CHEROKEE SMALL LETTER SE */
  ["\u13CF", "\uAB9F"], /* CHEROKEE SMALL LETTER SI */
  ["\u13D0", "\uABA0"], /* CHEROKEE SMALL LETTER SO */
  ["\u13D1", "\uABA1"], /* CHEROKEE SMALL LETTER SU */
  ["\u13D2", "\uABA2"], /* CHEROKEE SMALL LETTER SV */
  ["\u13D3", "\uABA3"], /* CHEROKEE SMALL LETTER DA */
  ["\u13D4", "\uABA4"], /* CHEROKEE SMALL LETTER TA */
  ["\u13D5", "\uABA5"], /* CHEROKEE SMALL LETTER DE */
  ["\u13D6", "\uABA6"], /* CHEROKEE SMALL LETTER TE */
  ["\u13D7", "\uABA7"], /* CHEROKEE SMALL LETTER DI */
  ["\u13D8", "\uABA8"], /* CHEROKEE SMALL LETTER TI */
  ["\u13D9", "\uABA9"], /* CHEROKEE SMALL LETTER DO */
  ["\u13DA", "\uABAA"], /* CHEROKEE SMALL LETTER DU */
  ["\u13DB", "\uABAB"], /* CHEROKEE SMALL LETTER DV */
  ["\u13DC", "\uABAC"], /* CHEROKEE SMALL LETTER DLA */
  ["\u13DD", "\uABAD"], /* CHEROKEE SMALL LETTER TLA */
  ["\u13DE", "\uABAE"], /* CHEROKEE SMALL LETTER TLE */
  ["\u13DF", "\uABAF"], /* CHEROKEE SMALL LETTER TLI */
  ["\u13E0", "\uABB0"], /* CHEROKEE SMALL LETTER TLO */
  ["\u13E1", "\uABB1"], /* CHEROKEE SMALL LETTER TLU */
  ["\u13E2", "\uABB2"], /* CHEROKEE SMALL LETTER TLV */
  ["\u13E3", "\uABB3"], /* CHEROKEE SMALL LETTER TSA */
  ["\u13E4", "\uABB4"], /* CHEROKEE SMALL LETTER TSE */
  ["\u13E5", "\uABB5"], /* CHEROKEE SMALL LETTER TSI */
  ["\u13E6", "\uABB6"], /* CHEROKEE SMALL LETTER TSO */
  ["\u13E7", "\uABB7"], /* CHEROKEE SMALL LETTER TSU */
  ["\u13E8", "\uABB8"], /* CHEROKEE SMALL LETTER TSV */
  ["\u13E9", "\uABB9"], /* CHEROKEE SMALL LETTER WA */
  ["\u13EA", "\uABBA"], /* CHEROKEE SMALL LETTER WE */
  ["\u13EB", "\uABBB"], /* CHEROKEE SMALL LETTER WI */
  ["\u13EC", "\uABBC"], /* CHEROKEE SMALL LETTER WO */
  ["\u13ED", "\uABBD"], /* CHEROKEE SMALL LETTER WU */
  ["\u13EE", "\uABBE"], /* CHEROKEE SMALL LETTER WV */
  ["\u13EF", "\uABBF"], /* CHEROKEE SMALL LETTER YA */
  ["\uABC0", "\uABC0"], /* MEETEI MAYEK LETTER KOK */
  ["\uABC1", "\uABC1"], /* MEETEI MAYEK LETTER SAM */
  ["\uABC2", "\uABC2"], /* MEETEI MAYEK LETTER LAI */
  ["\uABC3", "\uABC3"], /* MEETEI MAYEK LETTER MIT */
  ["\uABC4", "\uABC4"], /* MEETEI MAYEK LETTER PA */
  ["\uABC5", "\uABC5"], /* MEETEI MAYEK LETTER NA */
  ["\uABC6", "\uABC6"], /* MEETEI MAYEK LETTER CHIL */
  ["\uABC7", "\uABC7"], /* MEETEI MAYEK LETTER TIL */
  ["\uABC8", "\uABC8"], /* MEETEI MAYEK LETTER KHOU */
  ["\uABC9", "\uABC9"], /* MEETEI MAYEK LETTER NGOU */
  ["\uABCA", "\uABCA"], /* MEETEI MAYEK LETTER THOU */
  ["\uABCB", "\uABCB"], /* MEETEI MAYEK LETTER WAI */
  ["\uABCC", "\uABCC"], /* MEETEI MAYEK LETTER YANG */
  ["\uABCD", "\uABCD"], /* MEETEI MAYEK LETTER HUK */
  ["\uABCE", "\uABCE"], /* MEETEI MAYEK LETTER UN */
  ["\uABCF", "\uABCF"], /* MEETEI MAYEK LETTER I */
  ["\uABD0", "\uABD0"], /* MEETEI MAYEK LETTER PHAM */
  ["\uABD1", "\uABD1"], /* MEETEI MAYEK LETTER ATIYA */
  ["\uABD2", "\uABD2"], /* MEETEI MAYEK LETTER GOK */
  ["\uABD3", "\uABD3"], /* MEETEI MAYEK LETTER JHAM */
  ["\uABD4", "\uABD4"], /* MEETEI MAYEK LETTER RAI */
  ["\uABD5", "\uABD5"], /* MEETEI MAYEK LETTER BA */
  ["\uABD6", "\uABD6"], /* MEETEI MAYEK LETTER JIL */
  ["\uABD7", "\uABD7"], /* MEETEI MAYEK LETTER DIL */
  ["\uABD8", "\uABD8"], /* MEETEI MAYEK LETTER GHOU */
  ["\uABD9", "\uABD9"], /* MEETEI MAYEK LETTER DHOU */
  ["\uABDA", "\uABDA"], /* MEETEI MAYEK LETTER BHAM */
  ["\uABDB", "\uABDB"], /* MEETEI MAYEK LETTER KOK LONSUM */
  ["\uABDC", "\uABDC"], /* MEETEI MAYEK LETTER LAI LONSUM */
  ["\uABDD", "\uABDD"], /* MEETEI MAYEK LETTER MIT LONSUM */
  ["\uABDE", "\uABDE"], /* MEETEI MAYEK LETTER PA LONSUM */
  ["\uABDF", "\uABDF"], /* MEETEI MAYEK LETTER NA LONSUM */
  ["\uABE0", "\uABE0"], /* MEETEI MAYEK LETTER TIL LONSUM */
  ["\uABE1", "\uABE1"], /* MEETEI MAYEK LETTER NGOU LONSUM */
  ["\uABE2", "\uABE2"], /* MEETEI MAYEK LETTER I LONSUM */
  ["\uABE3", "\uABE3"], /* MEETEI MAYEK VOWEL SIGN ONAP */
  ["\uABE4", "\uABE4"], /* MEETEI MAYEK VOWEL SIGN INAP */
  ["\uABE5", "\uABE5"], /* MEETEI MAYEK VOWEL SIGN ANAP */
  ["\uABE6", "\uABE6"], /* MEETEI MAYEK VOWEL SIGN YENAP */
  ["\uABE7", "\uABE7"], /* MEETEI MAYEK VOWEL SIGN SOUNAP */
  ["\uABE8", "\uABE8"], /* MEETEI MAYEK VOWEL SIGN UNAP */
  ["\uABE9", "\uABE9"], /* MEETEI MAYEK VOWEL SIGN CHEINAP */
  ["\uABEA", "\uABEA"], /* MEETEI MAYEK VOWEL SIGN NUNG */
  ["\uABEB", "\uABEB"], /* MEETEI MAYEK CHEIKHEI */
  ["\uABEC", "\uABEC"], /* MEETEI MAYEK LUM IYEK */
  ["\uABED", "\uABED"], /* MEETEI MAYEK APUN IYEK */
  ["\uABEE", "\uABEE"],
  ["\uABEF", "\uABEF"],
  ["\uABF0", "\uABF0"], /* MEETEI MAYEK DIGIT ZERO */
  ["\uABF1", "\uABF1"], /* MEETEI MAYEK DIGIT ONE */
  ["\uABF2", "\uABF2"], /* MEETEI MAYEK DIGIT TWO */
  ["\uABF3", "\uABF3"], /* MEETEI MAYEK DIGIT THREE */
  ["\uABF4", "\uABF4"], /* MEETEI MAYEK DIGIT FOUR */
  ["\uABF5", "\uABF5"], /* MEETEI MAYEK DIGIT FIVE */
  ["\uABF6", "\uABF6"], /* MEETEI MAYEK DIGIT SIX */
  ["\uABF7", "\uABF7"], /* MEETEI MAYEK DIGIT SEVEN */
  ["\uABF8", "\uABF8"], /* MEETEI MAYEK DIGIT EIGHT */
  ["\uABF9", "\uABF9"], /* MEETEI MAYEK DIGIT NINE */
  ["\uABFA", "\uABFA"],
  ["\uABFB", "\uABFB"],
  ["\uABFC", "\uABFC"],
  ["\uABFD", "\uABFD"],
  ["\uABFE", "\uABFE"],
  ["\uABFF", "\uABFF"],
  ["\uAC00", "\uAC00"], /* Hangul Syllable */
  ["\uAC01", "\uAC01"], /* Hangul Syllable */
  ["\uAC02", "\uAC02"], /* Hangul Syllable */
  ["\uAC03", "\uAC03"], /* Hangul Syllable */
  ["\uAC04", "\uAC04"], /* Hangul Syllable */
  ["\uAC05", "\uAC05"], /* Hangul Syllable */
  ["\uAC06", "\uAC06"], /* Hangul Syllable */
  ["\uAC07", "\uAC07"], /* Hangul Syllable */
  ["\uAC08", "\uAC08"], /* Hangul Syllable */
  ["\uAC09", "\uAC09"], /* Hangul Syllable */
  ["\uAC0A", "\uAC0A"], /* Hangul Syllable */
  ["\uAC0B", "\uAC0B"], /* Hangul Syllable */
  ["\uAC0C", "\uAC0C"], /* Hangul Syllable */
  ["\uAC0D", "\uAC0D"], /* Hangul Syllable */
  ["\uAC0E", "\uAC0E"], /* Hangul Syllable */
  ["\uAC0F", "\uAC0F"], /* Hangul Syllable */
  ["\uAC10", "\uAC10"], /* Hangul Syllable */
  ["\uAC11", "\uAC11"], /* Hangul Syllable */
  ["\uAC12", "\uAC12"], /* Hangul Syllable */
  ["\uAC13", "\uAC13"], /* Hangul Syllable */
  ["\uAC14", "\uAC14"], /* Hangul Syllable */
  ["\uAC15", "\uAC15"], /* Hangul Syllable */
  ["\uAC16", "\uAC16"], /* Hangul Syllable */
  ["\uAC17", "\uAC17"], /* Hangul Syllable */
  ["\uAC18", "\uAC18"], /* Hangul Syllable */
  ["\uAC19", "\uAC19"], /* Hangul Syllable */
  ["\uAC1A", "\uAC1A"], /* Hangul Syllable */
  ["\uAC1B", "\uAC1B"], /* Hangul Syllable */
  ["\uAC1C", "\uAC1C"], /* Hangul Syllable */
  ["\uAC1D", "\uAC1D"], /* Hangul Syllable */
  ["\uAC1E", "\uAC1E"], /* Hangul Syllable */
  ["\uAC1F", "\uAC1F"], /* Hangul Syllable */
  ["\uAC20", "\uAC20"], /* Hangul Syllable */
  ["\uAC21", "\uAC21"], /* Hangul Syllable */
  ["\uAC22", "\uAC22"], /* Hangul Syllable */
  ["\uAC23", "\uAC23"], /* Hangul Syllable */
  ["\uAC24", "\uAC24"], /* Hangul Syllable */
  ["\uAC25", "\uAC25"], /* Hangul Syllable */
  ["\uAC26", "\uAC26"], /* Hangul Syllable */
  ["\uAC27", "\uAC27"], /* Hangul Syllable */
  ["\uAC28", "\uAC28"], /* Hangul Syllable */
  ["\uAC29", "\uAC29"], /* Hangul Syllable */
  ["\uAC2A", "\uAC2A"], /* Hangul Syllable */
  ["\uAC2B", "\uAC2B"], /* Hangul Syllable */
  ["\uAC2C", "\uAC2C"], /* Hangul Syllable */
  ["\uAC2D", "\uAC2D"], /* Hangul Syllable */
  ["\uAC2E", "\uAC2E"], /* Hangul Syllable */
  ["\uAC2F", "\uAC2F"], /* Hangul Syllable */
  ["\uAC30", "\uAC30"], /* Hangul Syllable */
  ["\uAC31", "\uAC31"], /* Hangul Syllable */
  ["\uAC32", "\uAC32"], /* Hangul Syllable */
  ["\uAC33", "\uAC33"], /* Hangul Syllable */
  ["\uAC34", "\uAC34"], /* Hangul Syllable */
  ["\uAC35", "\uAC35"], /* Hangul Syllable */
  ["\uAC36", "\uAC36"], /* Hangul Syllable */
  ["\uAC37", "\uAC37"], /* Hangul Syllable */
  ["\uAC38", "\uAC38"], /* Hangul Syllable */
  ["\uAC39", "\uAC39"], /* Hangul Syllable */
  ["\uAC3A", "\uAC3A"], /* Hangul Syllable */
  ["\uAC3B", "\uAC3B"], /* Hangul Syllable */
  ["\uAC3C", "\uAC3C"], /* Hangul Syllable */
  ["\uAC3D", "\uAC3D"], /* Hangul Syllable */
  ["\uAC3E", "\uAC3E"], /* Hangul Syllable */
  ["\uAC3F", "\uAC3F"], /* Hangul Syllable */
  ["\uAC40", "\uAC40"], /* Hangul Syllable */
  ["\uAC41", "\uAC41"], /* Hangul Syllable */
  ["\uAC42", "\uAC42"], /* Hangul Syllable */
  ["\uAC43", "\uAC43"], /* Hangul Syllable */
  ["\uAC44", "\uAC44"], /* Hangul Syllable */
  ["\uAC45", "\uAC45"], /* Hangul Syllable */
  ["\uAC46", "\uAC46"], /* Hangul Syllable */
  ["\uAC47", "\uAC47"], /* Hangul Syllable */
  ["\uAC48", "\uAC48"], /* Hangul Syllable */
  ["\uAC49", "\uAC49"], /* Hangul Syllable */
  ["\uAC4A", "\uAC4A"], /* Hangul Syllable */
  ["\uAC4B", "\uAC4B"], /* Hangul Syllable */
  ["\uAC4C", "\uAC4C"], /* Hangul Syllable */
  ["\uAC4D", "\uAC4D"], /* Hangul Syllable */
  ["\uAC4E", "\uAC4E"], /* Hangul Syllable */
  ["\uAC4F", "\uAC4F"], /* Hangul Syllable */
  ["\uAC50", "\uAC50"], /* Hangul Syllable */
  ["\uAC51", "\uAC51"], /* Hangul Syllable */
  ["\uAC52", "\uAC52"], /* Hangul Syllable */
  ["\uAC53", "\uAC53"], /* Hangul Syllable */
  ["\uAC54", "\uAC54"], /* Hangul Syllable */
  ["\uAC55", "\uAC55"], /* Hangul Syllable */
  ["\uAC56", "\uAC56"], /* Hangul Syllable */
  ["\uAC57", "\uAC57"], /* Hangul Syllable */
  ["\uAC58", "\uAC58"], /* Hangul Syllable */
  ["\uAC59", "\uAC59"], /* Hangul Syllable */
  ["\uAC5A", "\uAC5A"], /* Hangul Syllable */
  ["\uAC5B", "\uAC5B"], /* Hangul Syllable */
  ["\uAC5C", "\uAC5C"], /* Hangul Syllable */
  ["\uAC5D", "\uAC5D"], /* Hangul Syllable */
  ["\uAC5E", "\uAC5E"], /* Hangul Syllable */
  ["\uAC5F", "\uAC5F"], /* Hangul Syllable */
  ["\uAC60", "\uAC60"], /* Hangul Syllable */
  ["\uAC61", "\uAC61"], /* Hangul Syllable */
  ["\uAC62", "\uAC62"], /* Hangul Syllable */
  ["\uAC63", "\uAC63"], /* Hangul Syllable */
  ["\uAC64", "\uAC64"], /* Hangul Syllable */
  ["\uAC65", "\uAC65"], /* Hangul Syllable */
  ["\uAC66", "\uAC66"], /* Hangul Syllable */
  ["\uAC67", "\uAC67"], /* Hangul Syllable */
  ["\uAC68", "\uAC68"], /* Hangul Syllable */
  ["\uAC69", "\uAC69"], /* Hangul Syllable */
  ["\uAC6A", "\uAC6A"], /* Hangul Syllable */
  ["\uAC6B", "\uAC6B"], /* Hangul Syllable */
  ["\uAC6C", "\uAC6C"], /* Hangul Syllable */
  ["\uAC6D", "\uAC6D"], /* Hangul Syllable */
  ["\uAC6E", "\uAC6E"], /* Hangul Syllable */
  ["\uAC6F", "\uAC6F"], /* Hangul Syllable */
  ["\uAC70", "\uAC70"], /* Hangul Syllable */
  ["\uAC71", "\uAC71"], /* Hangul Syllable */
  ["\uAC72", "\uAC72"], /* Hangul Syllable */
  ["\uAC73", "\uAC73"], /* Hangul Syllable */
  ["\uAC74", "\uAC74"], /* Hangul Syllable */
  ["\uAC75", "\uAC75"], /* Hangul Syllable */
  ["\uAC76", "\uAC76"], /* Hangul Syllable */
  ["\uAC77", "\uAC77"], /* Hangul Syllable */
  ["\uAC78", "\uAC78"], /* Hangul Syllable */
  ["\uAC79", "\uAC79"], /* Hangul Syllable */
  ["\uAC7A", "\uAC7A"], /* Hangul Syllable */
  ["\uAC7B", "\uAC7B"], /* Hangul Syllable */
  ["\uAC7C", "\uAC7C"], /* Hangul Syllable */
  ["\uAC7D", "\uAC7D"], /* Hangul Syllable */
  ["\uAC7E", "\uAC7E"], /* Hangul Syllable */
  ["\uAC7F", "\uAC7F"], /* Hangul Syllable */
  ["\uAC80", "\uAC80"], /* Hangul Syllable */
  ["\uAC81", "\uAC81"], /* Hangul Syllable */
  ["\uAC82", "\uAC82"], /* Hangul Syllable */
  ["\uAC83", "\uAC83"], /* Hangul Syllable */
  ["\uAC84", "\uAC84"], /* Hangul Syllable */
  ["\uAC85", "\uAC85"], /* Hangul Syllable */
  ["\uAC86", "\uAC86"], /* Hangul Syllable */
  ["\uAC87", "\uAC87"], /* Hangul Syllable */
  ["\uAC88", "\uAC88"], /* Hangul Syllable */
  ["\uAC89", "\uAC89"], /* Hangul Syllable */
  ["\uAC8A", "\uAC8A"], /* Hangul Syllable */
  ["\uAC8B", "\uAC8B"], /* Hangul Syllable */
  ["\uAC8C", "\uAC8C"], /* Hangul Syllable */
  ["\uAC8D", "\uAC8D"], /* Hangul Syllable */
  ["\uAC8E", "\uAC8E"], /* Hangul Syllable */
  ["\uAC8F", "\uAC8F"], /* Hangul Syllable */
  ["\uAC90", "\uAC90"], /* Hangul Syllable */
  ["\uAC91", "\uAC91"], /* Hangul Syllable */
  ["\uAC92", "\uAC92"], /* Hangul Syllable */
  ["\uAC93", "\uAC93"], /* Hangul Syllable */
  ["\uAC94", "\uAC94"], /* Hangul Syllable */
  ["\uAC95", "\uAC95"], /* Hangul Syllable */
  ["\uAC96", "\uAC96"], /* Hangul Syllable */
  ["\uAC97", "\uAC97"], /* Hangul Syllable */
  ["\uAC98", "\uAC98"], /* Hangul Syllable */
  ["\uAC99", "\uAC99"], /* Hangul Syllable */
  ["\uAC9A", "\uAC9A"], /* Hangul Syllable */
  ["\uAC9B", "\uAC9B"], /* Hangul Syllable */
  ["\uAC9C", "\uAC9C"], /* Hangul Syllable */
  ["\uAC9D", "\uAC9D"], /* Hangul Syllable */
  ["\uAC9E", "\uAC9E"], /* Hangul Syllable */
  ["\uAC9F", "\uAC9F"], /* Hangul Syllable */
  ["\uACA0", "\uACA0"], /* Hangul Syllable */
  ["\uACA1", "\uACA1"], /* Hangul Syllable */
  ["\uACA2", "\uACA2"], /* Hangul Syllable */
  ["\uACA3", "\uACA3"], /* Hangul Syllable */
  ["\uACA4", "\uACA4"], /* Hangul Syllable */
  ["\uACA5", "\uACA5"], /* Hangul Syllable */
  ["\uACA6", "\uACA6"], /* Hangul Syllable */
  ["\uACA7", "\uACA7"], /* Hangul Syllable */
  ["\uACA8", "\uACA8"], /* Hangul Syllable */
  ["\uACA9", "\uACA9"], /* Hangul Syllable */
  ["\uACAA", "\uACAA"], /* Hangul Syllable */
  ["\uACAB", "\uACAB"], /* Hangul Syllable */
  ["\uACAC", "\uACAC"], /* Hangul Syllable */
  ["\uACAD", "\uACAD"], /* Hangul Syllable */
  ["\uACAE", "\uACAE"], /* Hangul Syllable */
  ["\uACAF", "\uACAF"], /* Hangul Syllable */
  ["\uACB0", "\uACB0"], /* Hangul Syllable */
  ["\uACB1", "\uACB1"], /* Hangul Syllable */
  ["\uACB2", "\uACB2"], /* Hangul Syllable */
  ["\uACB3", "\uACB3"], /* Hangul Syllable */
  ["\uACB4", "\uACB4"], /* Hangul Syllable */
  ["\uACB5", "\uACB5"], /* Hangul Syllable */
  ["\uACB6", "\uACB6"], /* Hangul Syllable */
  ["\uACB7", "\uACB7"], /* Hangul Syllable */
  ["\uACB8", "\uACB8"], /* Hangul Syllable */
  ["\uACB9", "\uACB9"], /* Hangul Syllable */
  ["\uACBA", "\uACBA"], /* Hangul Syllable */
  ["\uACBB", "\uACBB"], /* Hangul Syllable */
  ["\uACBC", "\uACBC"], /* Hangul Syllable */
  ["\uACBD", "\uACBD"], /* Hangul Syllable */
  ["\uACBE", "\uACBE"], /* Hangul Syllable */
  ["\uACBF", "\uACBF"], /* Hangul Syllable */
  ["\uACC0", "\uACC0"], /* Hangul Syllable */
  ["\uACC1", "\uACC1"], /* Hangul Syllable */
  ["\uACC2", "\uACC2"], /* Hangul Syllable */
  ["\uACC3", "\uACC3"], /* Hangul Syllable */
  ["\uACC4", "\uACC4"], /* Hangul Syllable */
  ["\uACC5", "\uACC5"], /* Hangul Syllable */
  ["\uACC6", "\uACC6"], /* Hangul Syllable */
  ["\uACC7", "\uACC7"], /* Hangul Syllable */
  ["\uACC8", "\uACC8"], /* Hangul Syllable */
  ["\uACC9", "\uACC9"], /* Hangul Syllable */
  ["\uACCA", "\uACCA"], /* Hangul Syllable */
  ["\uACCB", "\uACCB"], /* Hangul Syllable */
  ["\uACCC", "\uACCC"], /* Hangul Syllable */
  ["\uACCD", "\uACCD"], /* Hangul Syllable */
  ["\uACCE", "\uACCE"], /* Hangul Syllable */
  ["\uACCF", "\uACCF"], /* Hangul Syllable */
  ["\uACD0", "\uACD0"], /* Hangul Syllable */
  ["\uACD1", "\uACD1"], /* Hangul Syllable */
  ["\uACD2", "\uACD2"], /* Hangul Syllable */
  ["\uACD3", "\uACD3"], /* Hangul Syllable */
  ["\uACD4", "\uACD4"], /* Hangul Syllable */
  ["\uACD5", "\uACD5"], /* Hangul Syllable */
  ["\uACD6", "\uACD6"], /* Hangul Syllable */
  ["\uACD7", "\uACD7"], /* Hangul Syllable */
  ["\uACD8", "\uACD8"], /* Hangul Syllable */
  ["\uACD9", "\uACD9"], /* Hangul Syllable */
  ["\uACDA", "\uACDA"], /* Hangul Syllable */
  ["\uACDB", "\uACDB"], /* Hangul Syllable */
  ["\uACDC", "\uACDC"], /* Hangul Syllable */
  ["\uACDD", "\uACDD"], /* Hangul Syllable */
  ["\uACDE", "\uACDE"], /* Hangul Syllable */
  ["\uACDF", "\uACDF"], /* Hangul Syllable */
  ["\uACE0", "\uACE0"], /* Hangul Syllable */
  ["\uACE1", "\uACE1"], /* Hangul Syllable */
  ["\uACE2", "\uACE2"], /* Hangul Syllable */
  ["\uACE3", "\uACE3"], /* Hangul Syllable */
  ["\uACE4", "\uACE4"], /* Hangul Syllable */
  ["\uACE5", "\uACE5"], /* Hangul Syllable */
  ["\uACE6", "\uACE6"], /* Hangul Syllable */
  ["\uACE7", "\uACE7"], /* Hangul Syllable */
  ["\uACE8", "\uACE8"], /* Hangul Syllable */
  ["\uACE9", "\uACE9"], /* Hangul Syllable */
  ["\uACEA", "\uACEA"], /* Hangul Syllable */
  ["\uACEB", "\uACEB"], /* Hangul Syllable */
  ["\uACEC", "\uACEC"], /* Hangul Syllable */
  ["\uACED", "\uACED"], /* Hangul Syllable */
  ["\uACEE", "\uACEE"], /* Hangul Syllable */
  ["\uACEF", "\uACEF"], /* Hangul Syllable */
  ["\uACF0", "\uACF0"], /* Hangul Syllable */
  ["\uACF1", "\uACF1"], /* Hangul Syllable */
  ["\uACF2", "\uACF2"], /* Hangul Syllable */
  ["\uACF3", "\uACF3"], /* Hangul Syllable */
  ["\uACF4", "\uACF4"], /* Hangul Syllable */
  ["\uACF5", "\uACF5"], /* Hangul Syllable */
  ["\uACF6", "\uACF6"], /* Hangul Syllable */
  ["\uACF7", "\uACF7"], /* Hangul Syllable */
  ["\uACF8", "\uACF8"], /* Hangul Syllable */
  ["\uACF9", "\uACF9"], /* Hangul Syllable */
  ["\uACFA", "\uACFA"], /* Hangul Syllable */
  ["\uACFB", "\uACFB"], /* Hangul Syllable */
  ["\uACFC", "\uACFC"], /* Hangul Syllable */
  ["\uACFD", "\uACFD"], /* Hangul Syllable */
  ["\uACFE", "\uACFE"], /* Hangul Syllable */
  ["\uACFF", "\uACFF"], /* Hangul Syllable */
  ["\uAD00", "\uAD00"], /* Hangul Syllable */
  ["\uAD01", "\uAD01"], /* Hangul Syllable */
  ["\uAD02", "\uAD02"], /* Hangul Syllable */
  ["\uAD03", "\uAD03"], /* Hangul Syllable */
  ["\uAD04", "\uAD04"], /* Hangul Syllable */
  ["\uAD05", "\uAD05"], /* Hangul Syllable */
  ["\uAD06", "\uAD06"], /* Hangul Syllable */
  ["\uAD07", "\uAD07"], /* Hangul Syllable */
  ["\uAD08", "\uAD08"], /* Hangul Syllable */
  ["\uAD09", "\uAD09"], /* Hangul Syllable */
  ["\uAD0A", "\uAD0A"], /* Hangul Syllable */
  ["\uAD0B", "\uAD0B"], /* Hangul Syllable */
  ["\uAD0C", "\uAD0C"], /* Hangul Syllable */
  ["\uAD0D", "\uAD0D"], /* Hangul Syllable */
  ["\uAD0E", "\uAD0E"], /* Hangul Syllable */
  ["\uAD0F", "\uAD0F"], /* Hangul Syllable */
  ["\uAD10", "\uAD10"], /* Hangul Syllable */
  ["\uAD11", "\uAD11"], /* Hangul Syllable */
  ["\uAD12", "\uAD12"], /* Hangul Syllable */
  ["\uAD13", "\uAD13"], /* Hangul Syllable */
  ["\uAD14", "\uAD14"], /* Hangul Syllable */
  ["\uAD15", "\uAD15"], /* Hangul Syllable */
  ["\uAD16", "\uAD16"], /* Hangul Syllable */
  ["\uAD17", "\uAD17"], /* Hangul Syllable */
  ["\uAD18", "\uAD18"], /* Hangul Syllable */
  ["\uAD19", "\uAD19"], /* Hangul Syllable */
  ["\uAD1A", "\uAD1A"], /* Hangul Syllable */
  ["\uAD1B", "\uAD1B"], /* Hangul Syllable */
  ["\uAD1C", "\uAD1C"], /* Hangul Syllable */
  ["\uAD1D", "\uAD1D"], /* Hangul Syllable */
  ["\uAD1E", "\uAD1E"], /* Hangul Syllable */
  ["\uAD1F", "\uAD1F"], /* Hangul Syllable */
  ["\uAD20", "\uAD20"], /* Hangul Syllable */
  ["\uAD21", "\uAD21"], /* Hangul Syllable */
  ["\uAD22", "\uAD22"], /* Hangul Syllable */
  ["\uAD23", "\uAD23"], /* Hangul Syllable */
  ["\uAD24", "\uAD24"], /* Hangul Syllable */
  ["\uAD25", "\uAD25"], /* Hangul Syllable */
  ["\uAD26", "\uAD26"], /* Hangul Syllable */
  ["\uAD27", "\uAD27"], /* Hangul Syllable */
  ["\uAD28", "\uAD28"], /* Hangul Syllable */
  ["\uAD29", "\uAD29"], /* Hangul Syllable */
  ["\uAD2A", "\uAD2A"], /* Hangul Syllable */
  ["\uAD2B", "\uAD2B"], /* Hangul Syllable */
  ["\uAD2C", "\uAD2C"], /* Hangul Syllable */
  ["\uAD2D", "\uAD2D"], /* Hangul Syllable */
  ["\uAD2E", "\uAD2E"], /* Hangul Syllable */
  ["\uAD2F", "\uAD2F"], /* Hangul Syllable */
  ["\uAD30", "\uAD30"], /* Hangul Syllable */
  ["\uAD31", "\uAD31"], /* Hangul Syllable */
  ["\uAD32", "\uAD32"], /* Hangul Syllable */
  ["\uAD33", "\uAD33"], /* Hangul Syllable */
  ["\uAD34", "\uAD34"], /* Hangul Syllable */
  ["\uAD35", "\uAD35"], /* Hangul Syllable */
  ["\uAD36", "\uAD36"], /* Hangul Syllable */
  ["\uAD37", "\uAD37"], /* Hangul Syllable */
  ["\uAD38", "\uAD38"], /* Hangul Syllable */
  ["\uAD39", "\uAD39"], /* Hangul Syllable */
  ["\uAD3A", "\uAD3A"], /* Hangul Syllable */
  ["\uAD3B", "\uAD3B"], /* Hangul Syllable */
  ["\uAD3C", "\uAD3C"], /* Hangul Syllable */
  ["\uAD3D", "\uAD3D"], /* Hangul Syllable */
  ["\uAD3E", "\uAD3E"], /* Hangul Syllable */
  ["\uAD3F", "\uAD3F"], /* Hangul Syllable */
  ["\uAD40", "\uAD40"], /* Hangul Syllable */
  ["\uAD41", "\uAD41"], /* Hangul Syllable */
  ["\uAD42", "\uAD42"], /* Hangul Syllable */
  ["\uAD43", "\uAD43"], /* Hangul Syllable */
  ["\uAD44", "\uAD44"], /* Hangul Syllable */
  ["\uAD45", "\uAD45"], /* Hangul Syllable */
  ["\uAD46", "\uAD46"], /* Hangul Syllable */
  ["\uAD47", "\uAD47"], /* Hangul Syllable */
  ["\uAD48", "\uAD48"], /* Hangul Syllable */
  ["\uAD49", "\uAD49"], /* Hangul Syllable */
  ["\uAD4A", "\uAD4A"], /* Hangul Syllable */
  ["\uAD4B", "\uAD4B"], /* Hangul Syllable */
  ["\uAD4C", "\uAD4C"], /* Hangul Syllable */
  ["\uAD4D", "\uAD4D"], /* Hangul Syllable */
  ["\uAD4E", "\uAD4E"], /* Hangul Syllable */
  ["\uAD4F", "\uAD4F"], /* Hangul Syllable */
  ["\uAD50", "\uAD50"], /* Hangul Syllable */
  ["\uAD51", "\uAD51"], /* Hangul Syllable */
  ["\uAD52", "\uAD52"], /* Hangul Syllable */
  ["\uAD53", "\uAD53"], /* Hangul Syllable */
  ["\uAD54", "\uAD54"], /* Hangul Syllable */
  ["\uAD55", "\uAD55"], /* Hangul Syllable */
  ["\uAD56", "\uAD56"], /* Hangul Syllable */
  ["\uAD57", "\uAD57"], /* Hangul Syllable */
  ["\uAD58", "\uAD58"], /* Hangul Syllable */
  ["\uAD59", "\uAD59"], /* Hangul Syllable */
  ["\uAD5A", "\uAD5A"], /* Hangul Syllable */
  ["\uAD5B", "\uAD5B"], /* Hangul Syllable */
  ["\uAD5C", "\uAD5C"], /* Hangul Syllable */
  ["\uAD5D", "\uAD5D"], /* Hangul Syllable */
  ["\uAD5E", "\uAD5E"], /* Hangul Syllable */
  ["\uAD5F", "\uAD5F"], /* Hangul Syllable */
  ["\uAD60", "\uAD60"], /* Hangul Syllable */
  ["\uAD61", "\uAD61"], /* Hangul Syllable */
  ["\uAD62", "\uAD62"], /* Hangul Syllable */
  ["\uAD63", "\uAD63"], /* Hangul Syllable */
  ["\uAD64", "\uAD64"], /* Hangul Syllable */
  ["\uAD65", "\uAD65"], /* Hangul Syllable */
  ["\uAD66", "\uAD66"], /* Hangul Syllable */
  ["\uAD67", "\uAD67"], /* Hangul Syllable */
  ["\uAD68", "\uAD68"], /* Hangul Syllable */
  ["\uAD69", "\uAD69"], /* Hangul Syllable */
  ["\uAD6A", "\uAD6A"], /* Hangul Syllable */
  ["\uAD6B", "\uAD6B"], /* Hangul Syllable */
  ["\uAD6C", "\uAD6C"], /* Hangul Syllable */
  ["\uAD6D", "\uAD6D"], /* Hangul Syllable */
  ["\uAD6E", "\uAD6E"], /* Hangul Syllable */
  ["\uAD6F", "\uAD6F"], /* Hangul Syllable */
  ["\uAD70", "\uAD70"], /* Hangul Syllable */
  ["\uAD71", "\uAD71"], /* Hangul Syllable */
  ["\uAD72", "\uAD72"], /* Hangul Syllable */
  ["\uAD73", "\uAD73"], /* Hangul Syllable */
  ["\uAD74", "\uAD74"], /* Hangul Syllable */
  ["\uAD75", "\uAD75"], /* Hangul Syllable */
  ["\uAD76", "\uAD76"], /* Hangul Syllable */
  ["\uAD77", "\uAD77"], /* Hangul Syllable */
  ["\uAD78", "\uAD78"], /* Hangul Syllable */
  ["\uAD79", "\uAD79"], /* Hangul Syllable */
  ["\uAD7A", "\uAD7A"], /* Hangul Syllable */
  ["\uAD7B", "\uAD7B"], /* Hangul Syllable */
  ["\uAD7C", "\uAD7C"], /* Hangul Syllable */
  ["\uAD7D", "\uAD7D"], /* Hangul Syllable */
  ["\uAD7E", "\uAD7E"], /* Hangul Syllable */
  ["\uAD7F", "\uAD7F"], /* Hangul Syllable */
  ["\uAD80", "\uAD80"], /* Hangul Syllable */
  ["\uAD81", "\uAD81"], /* Hangul Syllable */
  ["\uAD82", "\uAD82"], /* Hangul Syllable */
  ["\uAD83", "\uAD83"], /* Hangul Syllable */
  ["\uAD84", "\uAD84"], /* Hangul Syllable */
  ["\uAD85", "\uAD85"], /* Hangul Syllable */
  ["\uAD86", "\uAD86"], /* Hangul Syllable */
  ["\uAD87", "\uAD87"], /* Hangul Syllable */
  ["\uAD88", "\uAD88"], /* Hangul Syllable */
  ["\uAD89", "\uAD89"], /* Hangul Syllable */
  ["\uAD8A", "\uAD8A"], /* Hangul Syllable */
  ["\uAD8B", "\uAD8B"], /* Hangul Syllable */
  ["\uAD8C", "\uAD8C"], /* Hangul Syllable */
  ["\uAD8D", "\uAD8D"], /* Hangul Syllable */
  ["\uAD8E", "\uAD8E"], /* Hangul Syllable */
  ["\uAD8F", "\uAD8F"], /* Hangul Syllable */
  ["\uAD90", "\uAD90"], /* Hangul Syllable */
  ["\uAD91", "\uAD91"], /* Hangul Syllable */
  ["\uAD92", "\uAD92"], /* Hangul Syllable */
  ["\uAD93", "\uAD93"], /* Hangul Syllable */
  ["\uAD94", "\uAD94"], /* Hangul Syllable */
  ["\uAD95", "\uAD95"], /* Hangul Syllable */
  ["\uAD96", "\uAD96"], /* Hangul Syllable */
  ["\uAD97", "\uAD97"], /* Hangul Syllable */
  ["\uAD98", "\uAD98"], /* Hangul Syllable */
  ["\uAD99", "\uAD99"], /* Hangul Syllable */
  ["\uAD9A", "\uAD9A"], /* Hangul Syllable */
  ["\uAD9B", "\uAD9B"], /* Hangul Syllable */
  ["\uAD9C", "\uAD9C"], /* Hangul Syllable */
  ["\uAD9D", "\uAD9D"], /* Hangul Syllable */
  ["\uAD9E", "\uAD9E"], /* Hangul Syllable */
  ["\uAD9F", "\uAD9F"], /* Hangul Syllable */
  ["\uADA0", "\uADA0"], /* Hangul Syllable */
  ["\uADA1", "\uADA1"], /* Hangul Syllable */
  ["\uADA2", "\uADA2"], /* Hangul Syllable */
  ["\uADA3", "\uADA3"], /* Hangul Syllable */
  ["\uADA4", "\uADA4"], /* Hangul Syllable */
  ["\uADA5", "\uADA5"], /* Hangul Syllable */
  ["\uADA6", "\uADA6"], /* Hangul Syllable */
  ["\uADA7", "\uADA7"], /* Hangul Syllable */
  ["\uADA8", "\uADA8"], /* Hangul Syllable */
  ["\uADA9", "\uADA9"], /* Hangul Syllable */
  ["\uADAA", "\uADAA"], /* Hangul Syllable */
  ["\uADAB", "\uADAB"], /* Hangul Syllable */
  ["\uADAC", "\uADAC"], /* Hangul Syllable */
  ["\uADAD", "\uADAD"], /* Hangul Syllable */
  ["\uADAE", "\uADAE"], /* Hangul Syllable */
  ["\uADAF", "\uADAF"], /* Hangul Syllable */
  ["\uADB0", "\uADB0"], /* Hangul Syllable */
  ["\uADB1", "\uADB1"], /* Hangul Syllable */
  ["\uADB2", "\uADB2"], /* Hangul Syllable */
  ["\uADB3", "\uADB3"], /* Hangul Syllable */
  ["\uADB4", "\uADB4"], /* Hangul Syllable */
  ["\uADB5", "\uADB5"], /* Hangul Syllable */
  ["\uADB6", "\uADB6"], /* Hangul Syllable */
  ["\uADB7", "\uADB7"], /* Hangul Syllable */
  ["\uADB8", "\uADB8"], /* Hangul Syllable */
  ["\uADB9", "\uADB9"], /* Hangul Syllable */
  ["\uADBA", "\uADBA"], /* Hangul Syllable */
  ["\uADBB", "\uADBB"], /* Hangul Syllable */
  ["\uADBC", "\uADBC"], /* Hangul Syllable */
  ["\uADBD", "\uADBD"], /* Hangul Syllable */
  ["\uADBE", "\uADBE"], /* Hangul Syllable */
  ["\uADBF", "\uADBF"], /* Hangul Syllable */
  ["\uADC0", "\uADC0"], /* Hangul Syllable */
  ["\uADC1", "\uADC1"], /* Hangul Syllable */
  ["\uADC2", "\uADC2"], /* Hangul Syllable */
  ["\uADC3", "\uADC3"], /* Hangul Syllable */
  ["\uADC4", "\uADC4"], /* Hangul Syllable */
  ["\uADC5", "\uADC5"], /* Hangul Syllable */
  ["\uADC6", "\uADC6"], /* Hangul Syllable */
  ["\uADC7", "\uADC7"], /* Hangul Syllable */
  ["\uADC8", "\uADC8"], /* Hangul Syllable */
  ["\uADC9", "\uADC9"], /* Hangul Syllable */
  ["\uADCA", "\uADCA"], /* Hangul Syllable */
  ["\uADCB", "\uADCB"], /* Hangul Syllable */
  ["\uADCC", "\uADCC"], /* Hangul Syllable */
  ["\uADCD", "\uADCD"], /* Hangul Syllable */
  ["\uADCE", "\uADCE"], /* Hangul Syllable */
  ["\uADCF", "\uADCF"], /* Hangul Syllable */
  ["\uADD0", "\uADD0"], /* Hangul Syllable */
  ["\uADD1", "\uADD1"], /* Hangul Syllable */
  ["\uADD2", "\uADD2"], /* Hangul Syllable */
  ["\uADD3", "\uADD3"], /* Hangul Syllable */
  ["\uADD4", "\uADD4"], /* Hangul Syllable */
  ["\uADD5", "\uADD5"], /* Hangul Syllable */
  ["\uADD6", "\uADD6"], /* Hangul Syllable */
  ["\uADD7", "\uADD7"], /* Hangul Syllable */
  ["\uADD8", "\uADD8"], /* Hangul Syllable */
  ["\uADD9", "\uADD9"], /* Hangul Syllable */
  ["\uADDA", "\uADDA"], /* Hangul Syllable */
  ["\uADDB", "\uADDB"], /* Hangul Syllable */
  ["\uADDC", "\uADDC"], /* Hangul Syllable */
  ["\uADDD", "\uADDD"], /* Hangul Syllable */
  ["\uADDE", "\uADDE"], /* Hangul Syllable */
  ["\uADDF", "\uADDF"], /* Hangul Syllable */
  ["\uADE0", "\uADE0"], /* Hangul Syllable */
  ["\uADE1", "\uADE1"], /* Hangul Syllable */
  ["\uADE2", "\uADE2"], /* Hangul Syllable */
  ["\uADE3", "\uADE3"], /* Hangul Syllable */
  ["\uADE4", "\uADE4"], /* Hangul Syllable */
  ["\uADE5", "\uADE5"], /* Hangul Syllable */
  ["\uADE6", "\uADE6"], /* Hangul Syllable */
  ["\uADE7", "\uADE7"], /* Hangul Syllable */
  ["\uADE8", "\uADE8"], /* Hangul Syllable */
  ["\uADE9", "\uADE9"], /* Hangul Syllable */
  ["\uADEA", "\uADEA"], /* Hangul Syllable */
  ["\uADEB", "\uADEB"], /* Hangul Syllable */
  ["\uADEC", "\uADEC"], /* Hangul Syllable */
  ["\uADED", "\uADED"], /* Hangul Syllable */
  ["\uADEE", "\uADEE"], /* Hangul Syllable */
  ["\uADEF", "\uADEF"], /* Hangul Syllable */
  ["\uADF0", "\uADF0"], /* Hangul Syllable */
  ["\uADF1", "\uADF1"], /* Hangul Syllable */
  ["\uADF2", "\uADF2"], /* Hangul Syllable */
  ["\uADF3", "\uADF3"], /* Hangul Syllable */
  ["\uADF4", "\uADF4"], /* Hangul Syllable */
  ["\uADF5", "\uADF5"], /* Hangul Syllable */
  ["\uADF6", "\uADF6"], /* Hangul Syllable */
  ["\uADF7", "\uADF7"], /* Hangul Syllable */
  ["\uADF8", "\uADF8"], /* Hangul Syllable */
  ["\uADF9", "\uADF9"], /* Hangul Syllable */
  ["\uADFA", "\uADFA"], /* Hangul Syllable */
  ["\uADFB", "\uADFB"], /* Hangul Syllable */
  ["\uADFC", "\uADFC"], /* Hangul Syllable */
  ["\uADFD", "\uADFD"], /* Hangul Syllable */
  ["\uADFE", "\uADFE"], /* Hangul Syllable */
  ["\uADFF", "\uADFF"], /* Hangul Syllable */
  ["\uAE00", "\uAE00"], /* Hangul Syllable */
  ["\uAE01", "\uAE01"], /* Hangul Syllable */
  ["\uAE02", "\uAE02"], /* Hangul Syllable */
  ["\uAE03", "\uAE03"], /* Hangul Syllable */
  ["\uAE04", "\uAE04"], /* Hangul Syllable */
  ["\uAE05", "\uAE05"], /* Hangul Syllable */
  ["\uAE06", "\uAE06"], /* Hangul Syllable */
  ["\uAE07", "\uAE07"], /* Hangul Syllable */
  ["\uAE08", "\uAE08"], /* Hangul Syllable */
  ["\uAE09", "\uAE09"], /* Hangul Syllable */
  ["\uAE0A", "\uAE0A"], /* Hangul Syllable */
  ["\uAE0B", "\uAE0B"], /* Hangul Syllable */
  ["\uAE0C", "\uAE0C"], /* Hangul Syllable */
  ["\uAE0D", "\uAE0D"], /* Hangul Syllable */
  ["\uAE0E", "\uAE0E"], /* Hangul Syllable */
  ["\uAE0F", "\uAE0F"], /* Hangul Syllable */
  ["\uAE10", "\uAE10"], /* Hangul Syllable */
  ["\uAE11", "\uAE11"], /* Hangul Syllable */
  ["\uAE12", "\uAE12"], /* Hangul Syllable */
  ["\uAE13", "\uAE13"], /* Hangul Syllable */
  ["\uAE14", "\uAE14"], /* Hangul Syllable */
  ["\uAE15", "\uAE15"], /* Hangul Syllable */
  ["\uAE16", "\uAE16"], /* Hangul Syllable */
  ["\uAE17", "\uAE17"], /* Hangul Syllable */
  ["\uAE18", "\uAE18"], /* Hangul Syllable */
  ["\uAE19", "\uAE19"], /* Hangul Syllable */
  ["\uAE1A", "\uAE1A"], /* Hangul Syllable */
  ["\uAE1B", "\uAE1B"], /* Hangul Syllable */
  ["\uAE1C", "\uAE1C"], /* Hangul Syllable */
  ["\uAE1D", "\uAE1D"], /* Hangul Syllable */
  ["\uAE1E", "\uAE1E"], /* Hangul Syllable */
  ["\uAE1F", "\uAE1F"], /* Hangul Syllable */
  ["\uAE20", "\uAE20"], /* Hangul Syllable */
  ["\uAE21", "\uAE21"], /* Hangul Syllable */
  ["\uAE22", "\uAE22"], /* Hangul Syllable */
  ["\uAE23", "\uAE23"], /* Hangul Syllable */
  ["\uAE24", "\uAE24"], /* Hangul Syllable */
  ["\uAE25", "\uAE25"], /* Hangul Syllable */
  ["\uAE26", "\uAE26"], /* Hangul Syllable */
  ["\uAE27", "\uAE27"], /* Hangul Syllable */
  ["\uAE28", "\uAE28"], /* Hangul Syllable */
  ["\uAE29", "\uAE29"], /* Hangul Syllable */
  ["\uAE2A", "\uAE2A"], /* Hangul Syllable */
  ["\uAE2B", "\uAE2B"], /* Hangul Syllable */
  ["\uAE2C", "\uAE2C"], /* Hangul Syllable */
  ["\uAE2D", "\uAE2D"], /* Hangul Syllable */
  ["\uAE2E", "\uAE2E"], /* Hangul Syllable */
  ["\uAE2F", "\uAE2F"], /* Hangul Syllable */
  ["\uAE30", "\uAE30"], /* Hangul Syllable */
  ["\uAE31", "\uAE31"], /* Hangul Syllable */
  ["\uAE32", "\uAE32"], /* Hangul Syllable */
  ["\uAE33", "\uAE33"], /* Hangul Syllable */
  ["\uAE34", "\uAE34"], /* Hangul Syllable */
  ["\uAE35", "\uAE35"], /* Hangul Syllable */
  ["\uAE36", "\uAE36"], /* Hangul Syllable */
  ["\uAE37", "\uAE37"], /* Hangul Syllable */
  ["\uAE38", "\uAE38"], /* Hangul Syllable */
  ["\uAE39", "\uAE39"], /* Hangul Syllable */
  ["\uAE3A", "\uAE3A"], /* Hangul Syllable */
  ["\uAE3B", "\uAE3B"], /* Hangul Syllable */
  ["\uAE3C", "\uAE3C"], /* Hangul Syllable */
  ["\uAE3D", "\uAE3D"], /* Hangul Syllable */
  ["\uAE3E", "\uAE3E"], /* Hangul Syllable */
  ["\uAE3F", "\uAE3F"], /* Hangul Syllable */
  ["\uAE40", "\uAE40"], /* Hangul Syllable */
  ["\uAE41", "\uAE41"], /* Hangul Syllable */
  ["\uAE42", "\uAE42"], /* Hangul Syllable */
  ["\uAE43", "\uAE43"], /* Hangul Syllable */
  ["\uAE44", "\uAE44"], /* Hangul Syllable */
  ["\uAE45", "\uAE45"], /* Hangul Syllable */
  ["\uAE46", "\uAE46"], /* Hangul Syllable */
  ["\uAE47", "\uAE47"], /* Hangul Syllable */
  ["\uAE48", "\uAE48"], /* Hangul Syllable */
  ["\uAE49", "\uAE49"], /* Hangul Syllable */
  ["\uAE4A", "\uAE4A"], /* Hangul Syllable */
  ["\uAE4B", "\uAE4B"], /* Hangul Syllable */
  ["\uAE4C", "\uAE4C"], /* Hangul Syllable */
  ["\uAE4D", "\uAE4D"], /* Hangul Syllable */
  ["\uAE4E", "\uAE4E"], /* Hangul Syllable */
  ["\uAE4F", "\uAE4F"], /* Hangul Syllable */
  ["\uAE50", "\uAE50"], /* Hangul Syllable */
  ["\uAE51", "\uAE51"], /* Hangul Syllable */
  ["\uAE52", "\uAE52"], /* Hangul Syllable */
  ["\uAE53", "\uAE53"], /* Hangul Syllable */
  ["\uAE54", "\uAE54"], /* Hangul Syllable */
  ["\uAE55", "\uAE55"], /* Hangul Syllable */
  ["\uAE56", "\uAE56"], /* Hangul Syllable */
  ["\uAE57", "\uAE57"], /* Hangul Syllable */
  ["\uAE58", "\uAE58"], /* Hangul Syllable */
  ["\uAE59", "\uAE59"], /* Hangul Syllable */
  ["\uAE5A", "\uAE5A"], /* Hangul Syllable */
  ["\uAE5B", "\uAE5B"], /* Hangul Syllable */
  ["\uAE5C", "\uAE5C"], /* Hangul Syllable */
  ["\uAE5D", "\uAE5D"], /* Hangul Syllable */
  ["\uAE5E", "\uAE5E"], /* Hangul Syllable */
  ["\uAE5F", "\uAE5F"], /* Hangul Syllable */
  ["\uAE60", "\uAE60"], /* Hangul Syllable */
  ["\uAE61", "\uAE61"], /* Hangul Syllable */
  ["\uAE62", "\uAE62"], /* Hangul Syllable */
  ["\uAE63", "\uAE63"], /* Hangul Syllable */
  ["\uAE64", "\uAE64"], /* Hangul Syllable */
  ["\uAE65", "\uAE65"], /* Hangul Syllable */
  ["\uAE66", "\uAE66"], /* Hangul Syllable */
  ["\uAE67", "\uAE67"], /* Hangul Syllable */
  ["\uAE68", "\uAE68"], /* Hangul Syllable */
  ["\uAE69", "\uAE69"], /* Hangul Syllable */
  ["\uAE6A", "\uAE6A"], /* Hangul Syllable */
  ["\uAE6B", "\uAE6B"], /* Hangul Syllable */
  ["\uAE6C", "\uAE6C"], /* Hangul Syllable */
  ["\uAE6D", "\uAE6D"], /* Hangul Syllable */
  ["\uAE6E", "\uAE6E"], /* Hangul Syllable */
  ["\uAE6F", "\uAE6F"], /* Hangul Syllable */
  ["\uAE70", "\uAE70"], /* Hangul Syllable */
  ["\uAE71", "\uAE71"], /* Hangul Syllable */
  ["\uAE72", "\uAE72"], /* Hangul Syllable */
  ["\uAE73", "\uAE73"], /* Hangul Syllable */
  ["\uAE74", "\uAE74"], /* Hangul Syllable */
  ["\uAE75", "\uAE75"], /* Hangul Syllable */
  ["\uAE76", "\uAE76"], /* Hangul Syllable */
  ["\uAE77", "\uAE77"], /* Hangul Syllable */
  ["\uAE78", "\uAE78"], /* Hangul Syllable */
  ["\uAE79", "\uAE79"], /* Hangul Syllable */
  ["\uAE7A", "\uAE7A"], /* Hangul Syllable */
  ["\uAE7B", "\uAE7B"], /* Hangul Syllable */
  ["\uAE7C", "\uAE7C"], /* Hangul Syllable */
  ["\uAE7D", "\uAE7D"], /* Hangul Syllable */
  ["\uAE7E", "\uAE7E"], /* Hangul Syllable */
  ["\uAE7F", "\uAE7F"], /* Hangul Syllable */
  ["\uAE80", "\uAE80"], /* Hangul Syllable */
  ["\uAE81", "\uAE81"], /* Hangul Syllable */
  ["\uAE82", "\uAE82"], /* Hangul Syllable */
  ["\uAE83", "\uAE83"], /* Hangul Syllable */
  ["\uAE84", "\uAE84"], /* Hangul Syllable */
  ["\uAE85", "\uAE85"], /* Hangul Syllable */
  ["\uAE86", "\uAE86"], /* Hangul Syllable */
  ["\uAE87", "\uAE87"], /* Hangul Syllable */
  ["\uAE88", "\uAE88"], /* Hangul Syllable */
  ["\uAE89", "\uAE89"], /* Hangul Syllable */
  ["\uAE8A", "\uAE8A"], /* Hangul Syllable */
  ["\uAE8B", "\uAE8B"], /* Hangul Syllable */
  ["\uAE8C", "\uAE8C"], /* Hangul Syllable */
  ["\uAE8D", "\uAE8D"], /* Hangul Syllable */
  ["\uAE8E", "\uAE8E"], /* Hangul Syllable */
  ["\uAE8F", "\uAE8F"], /* Hangul Syllable */
  ["\uAE90", "\uAE90"], /* Hangul Syllable */
  ["\uAE91", "\uAE91"], /* Hangul Syllable */
  ["\uAE92", "\uAE92"], /* Hangul Syllable */
  ["\uAE93", "\uAE93"], /* Hangul Syllable */
  ["\uAE94", "\uAE94"], /* Hangul Syllable */
  ["\uAE95", "\uAE95"], /* Hangul Syllable */
  ["\uAE96", "\uAE96"], /* Hangul Syllable */
  ["\uAE97", "\uAE97"], /* Hangul Syllable */
  ["\uAE98", "\uAE98"], /* Hangul Syllable */
  ["\uAE99", "\uAE99"], /* Hangul Syllable */
  ["\uAE9A", "\uAE9A"], /* Hangul Syllable */
  ["\uAE9B", "\uAE9B"], /* Hangul Syllable */
  ["\uAE9C", "\uAE9C"], /* Hangul Syllable */
  ["\uAE9D", "\uAE9D"], /* Hangul Syllable */
  ["\uAE9E", "\uAE9E"], /* Hangul Syllable */
  ["\uAE9F", "\uAE9F"], /* Hangul Syllable */
  ["\uAEA0", "\uAEA0"], /* Hangul Syllable */
  ["\uAEA1", "\uAEA1"], /* Hangul Syllable */
  ["\uAEA2", "\uAEA2"], /* Hangul Syllable */
  ["\uAEA3", "\uAEA3"], /* Hangul Syllable */
  ["\uAEA4", "\uAEA4"], /* Hangul Syllable */
  ["\uAEA5", "\uAEA5"], /* Hangul Syllable */
  ["\uAEA6", "\uAEA6"], /* Hangul Syllable */
  ["\uAEA7", "\uAEA7"], /* Hangul Syllable */
  ["\uAEA8", "\uAEA8"], /* Hangul Syllable */
  ["\uAEA9", "\uAEA9"], /* Hangul Syllable */
  ["\uAEAA", "\uAEAA"], /* Hangul Syllable */
  ["\uAEAB", "\uAEAB"], /* Hangul Syllable */
  ["\uAEAC", "\uAEAC"], /* Hangul Syllable */
  ["\uAEAD", "\uAEAD"], /* Hangul Syllable */
  ["\uAEAE", "\uAEAE"], /* Hangul Syllable */
  ["\uAEAF", "\uAEAF"], /* Hangul Syllable */
  ["\uAEB0", "\uAEB0"], /* Hangul Syllable */
  ["\uAEB1", "\uAEB1"], /* Hangul Syllable */
  ["\uAEB2", "\uAEB2"], /* Hangul Syllable */
  ["\uAEB3", "\uAEB3"], /* Hangul Syllable */
  ["\uAEB4", "\uAEB4"], /* Hangul Syllable */
  ["\uAEB5", "\uAEB5"], /* Hangul Syllable */
  ["\uAEB6", "\uAEB6"], /* Hangul Syllable */
  ["\uAEB7", "\uAEB7"], /* Hangul Syllable */
  ["\uAEB8", "\uAEB8"], /* Hangul Syllable */
  ["\uAEB9", "\uAEB9"], /* Hangul Syllable */
  ["\uAEBA", "\uAEBA"], /* Hangul Syllable */
  ["\uAEBB", "\uAEBB"], /* Hangul Syllable */
  ["\uAEBC", "\uAEBC"], /* Hangul Syllable */
  ["\uAEBD", "\uAEBD"], /* Hangul Syllable */
  ["\uAEBE", "\uAEBE"], /* Hangul Syllable */
  ["\uAEBF", "\uAEBF"], /* Hangul Syllable */
  ["\uAEC0", "\uAEC0"], /* Hangul Syllable */
  ["\uAEC1", "\uAEC1"], /* Hangul Syllable */
  ["\uAEC2", "\uAEC2"], /* Hangul Syllable */
  ["\uAEC3", "\uAEC3"], /* Hangul Syllable */
  ["\uAEC4", "\uAEC4"], /* Hangul Syllable */
  ["\uAEC5", "\uAEC5"], /* Hangul Syllable */
  ["\uAEC6", "\uAEC6"], /* Hangul Syllable */
  ["\uAEC7", "\uAEC7"], /* Hangul Syllable */
  ["\uAEC8", "\uAEC8"], /* Hangul Syllable */
  ["\uAEC9", "\uAEC9"], /* Hangul Syllable */
  ["\uAECA", "\uAECA"], /* Hangul Syllable */
  ["\uAECB", "\uAECB"], /* Hangul Syllable */
  ["\uAECC", "\uAECC"], /* Hangul Syllable */
  ["\uAECD", "\uAECD"], /* Hangul Syllable */
  ["\uAECE", "\uAECE"], /* Hangul Syllable */
  ["\uAECF", "\uAECF"], /* Hangul Syllable */
  ["\uAED0", "\uAED0"], /* Hangul Syllable */
  ["\uAED1", "\uAED1"], /* Hangul Syllable */
  ["\uAED2", "\uAED2"], /* Hangul Syllable */
  ["\uAED3", "\uAED3"], /* Hangul Syllable */
  ["\uAED4", "\uAED4"], /* Hangul Syllable */
  ["\uAED5", "\uAED5"], /* Hangul Syllable */
  ["\uAED6", "\uAED6"], /* Hangul Syllable */
  ["\uAED7", "\uAED7"], /* Hangul Syllable */
  ["\uAED8", "\uAED8"], /* Hangul Syllable */
  ["\uAED9", "\uAED9"], /* Hangul Syllable */
  ["\uAEDA", "\uAEDA"], /* Hangul Syllable */
  ["\uAEDB", "\uAEDB"], /* Hangul Syllable */
  ["\uAEDC", "\uAEDC"], /* Hangul Syllable */
  ["\uAEDD", "\uAEDD"], /* Hangul Syllable */
  ["\uAEDE", "\uAEDE"], /* Hangul Syllable */
  ["\uAEDF", "\uAEDF"], /* Hangul Syllable */
  ["\uAEE0", "\uAEE0"], /* Hangul Syllable */
  ["\uAEE1", "\uAEE1"], /* Hangul Syllable */
  ["\uAEE2", "\uAEE2"], /* Hangul Syllable */
  ["\uAEE3", "\uAEE3"], /* Hangul Syllable */
  ["\uAEE4", "\uAEE4"], /* Hangul Syllable */
  ["\uAEE5", "\uAEE5"], /* Hangul Syllable */
  ["\uAEE6", "\uAEE6"], /* Hangul Syllable */
  ["\uAEE7", "\uAEE7"], /* Hangul Syllable */
  ["\uAEE8", "\uAEE8"], /* Hangul Syllable */
  ["\uAEE9", "\uAEE9"], /* Hangul Syllable */
  ["\uAEEA", "\uAEEA"], /* Hangul Syllable */
  ["\uAEEB", "\uAEEB"], /* Hangul Syllable */
  ["\uAEEC", "\uAEEC"], /* Hangul Syllable */
  ["\uAEED", "\uAEED"], /* Hangul Syllable */
  ["\uAEEE", "\uAEEE"], /* Hangul Syllable */
  ["\uAEEF", "\uAEEF"], /* Hangul Syllable */
  ["\uAEF0", "\uAEF0"], /* Hangul Syllable */
  ["\uAEF1", "\uAEF1"], /* Hangul Syllable */
  ["\uAEF2", "\uAEF2"], /* Hangul Syllable */
  ["\uAEF3", "\uAEF3"], /* Hangul Syllable */
  ["\uAEF4", "\uAEF4"], /* Hangul Syllable */
  ["\uAEF5", "\uAEF5"], /* Hangul Syllable */
  ["\uAEF6", "\uAEF6"], /* Hangul Syllable */
  ["\uAEF7", "\uAEF7"], /* Hangul Syllable */
  ["\uAEF8", "\uAEF8"], /* Hangul Syllable */
  ["\uAEF9", "\uAEF9"], /* Hangul Syllable */
  ["\uAEFA", "\uAEFA"], /* Hangul Syllable */
  ["\uAEFB", "\uAEFB"], /* Hangul Syllable */
  ["\uAEFC", "\uAEFC"], /* Hangul Syllable */
  ["\uAEFD", "\uAEFD"], /* Hangul Syllable */
  ["\uAEFE", "\uAEFE"], /* Hangul Syllable */
  ["\uAEFF", "\uAEFF"], /* Hangul Syllable */
  ["\uAF00", "\uAF00"], /* Hangul Syllable */
  ["\uAF01", "\uAF01"], /* Hangul Syllable */
  ["\uAF02", "\uAF02"], /* Hangul Syllable */
  ["\uAF03", "\uAF03"], /* Hangul Syllable */
  ["\uAF04", "\uAF04"], /* Hangul Syllable */
  ["\uAF05", "\uAF05"], /* Hangul Syllable */
  ["\uAF06", "\uAF06"], /* Hangul Syllable */
  ["\uAF07", "\uAF07"], /* Hangul Syllable */
  ["\uAF08", "\uAF08"], /* Hangul Syllable */
  ["\uAF09", "\uAF09"], /* Hangul Syllable */
  ["\uAF0A", "\uAF0A"], /* Hangul Syllable */
  ["\uAF0B", "\uAF0B"], /* Hangul Syllable */
  ["\uAF0C", "\uAF0C"], /* Hangul Syllable */
  ["\uAF0D", "\uAF0D"], /* Hangul Syllable */
  ["\uAF0E", "\uAF0E"], /* Hangul Syllable */
  ["\uAF0F", "\uAF0F"], /* Hangul Syllable */
  ["\uAF10", "\uAF10"], /* Hangul Syllable */
  ["\uAF11", "\uAF11"], /* Hangul Syllable */
  ["\uAF12", "\uAF12"], /* Hangul Syllable */
  ["\uAF13", "\uAF13"], /* Hangul Syllable */
  ["\uAF14", "\uAF14"], /* Hangul Syllable */
  ["\uAF15", "\uAF15"], /* Hangul Syllable */
  ["\uAF16", "\uAF16"], /* Hangul Syllable */
  ["\uAF17", "\uAF17"], /* Hangul Syllable */
  ["\uAF18", "\uAF18"], /* Hangul Syllable */
  ["\uAF19", "\uAF19"], /* Hangul Syllable */
  ["\uAF1A", "\uAF1A"], /* Hangul Syllable */
  ["\uAF1B", "\uAF1B"], /* Hangul Syllable */
  ["\uAF1C", "\uAF1C"], /* Hangul Syllable */
  ["\uAF1D", "\uAF1D"], /* Hangul Syllable */
  ["\uAF1E", "\uAF1E"], /* Hangul Syllable */
  ["\uAF1F", "\uAF1F"], /* Hangul Syllable */
  ["\uAF20", "\uAF20"], /* Hangul Syllable */
  ["\uAF21", "\uAF21"], /* Hangul Syllable */
  ["\uAF22", "\uAF22"], /* Hangul Syllable */
  ["\uAF23", "\uAF23"], /* Hangul Syllable */
  ["\uAF24", "\uAF24"], /* Hangul Syllable */
  ["\uAF25", "\uAF25"], /* Hangul Syllable */
  ["\uAF26", "\uAF26"], /* Hangul Syllable */
  ["\uAF27", "\uAF27"], /* Hangul Syllable */
  ["\uAF28", "\uAF28"], /* Hangul Syllable */
  ["\uAF29", "\uAF29"], /* Hangul Syllable */
  ["\uAF2A", "\uAF2A"], /* Hangul Syllable */
  ["\uAF2B", "\uAF2B"], /* Hangul Syllable */
  ["\uAF2C", "\uAF2C"], /* Hangul Syllable */
  ["\uAF2D", "\uAF2D"], /* Hangul Syllable */
  ["\uAF2E", "\uAF2E"], /* Hangul Syllable */
  ["\uAF2F", "\uAF2F"], /* Hangul Syllable */
  ["\uAF30", "\uAF30"], /* Hangul Syllable */
  ["\uAF31", "\uAF31"], /* Hangul Syllable */
  ["\uAF32", "\uAF32"], /* Hangul Syllable */
  ["\uAF33", "\uAF33"], /* Hangul Syllable */
  ["\uAF34", "\uAF34"], /* Hangul Syllable */
  ["\uAF35", "\uAF35"], /* Hangul Syllable */
  ["\uAF36", "\uAF36"], /* Hangul Syllable */
  ["\uAF37", "\uAF37"], /* Hangul Syllable */
  ["\uAF38", "\uAF38"], /* Hangul Syllable */
  ["\uAF39", "\uAF39"], /* Hangul Syllable */
  ["\uAF3A", "\uAF3A"], /* Hangul Syllable */
  ["\uAF3B", "\uAF3B"], /* Hangul Syllable */
  ["\uAF3C", "\uAF3C"], /* Hangul Syllable */
  ["\uAF3D", "\uAF3D"], /* Hangul Syllable */
  ["\uAF3E", "\uAF3E"], /* Hangul Syllable */
  ["\uAF3F", "\uAF3F"], /* Hangul Syllable */
  ["\uAF40", "\uAF40"], /* Hangul Syllable */
  ["\uAF41", "\uAF41"], /* Hangul Syllable */
  ["\uAF42", "\uAF42"], /* Hangul Syllable */
  ["\uAF43", "\uAF43"], /* Hangul Syllable */
  ["\uAF44", "\uAF44"], /* Hangul Syllable */
  ["\uAF45", "\uAF45"], /* Hangul Syllable */
  ["\uAF46", "\uAF46"], /* Hangul Syllable */
  ["\uAF47", "\uAF47"], /* Hangul Syllable */
  ["\uAF48", "\uAF48"], /* Hangul Syllable */
  ["\uAF49", "\uAF49"], /* Hangul Syllable */
  ["\uAF4A", "\uAF4A"], /* Hangul Syllable */
  ["\uAF4B", "\uAF4B"], /* Hangul Syllable */
  ["\uAF4C", "\uAF4C"], /* Hangul Syllable */
  ["\uAF4D", "\uAF4D"], /* Hangul Syllable */
  ["\uAF4E", "\uAF4E"], /* Hangul Syllable */
  ["\uAF4F", "\uAF4F"], /* Hangul Syllable */
  ["\uAF50", "\uAF50"], /* Hangul Syllable */
  ["\uAF51", "\uAF51"], /* Hangul Syllable */
  ["\uAF52", "\uAF52"], /* Hangul Syllable */
  ["\uAF53", "\uAF53"], /* Hangul Syllable */
  ["\uAF54", "\uAF54"], /* Hangul Syllable */
  ["\uAF55", "\uAF55"], /* Hangul Syllable */
  ["\uAF56", "\uAF56"], /* Hangul Syllable */
  ["\uAF57", "\uAF57"], /* Hangul Syllable */
  ["\uAF58", "\uAF58"], /* Hangul Syllable */
  ["\uAF59", "\uAF59"], /* Hangul Syllable */
  ["\uAF5A", "\uAF5A"], /* Hangul Syllable */
  ["\uAF5B", "\uAF5B"], /* Hangul Syllable */
  ["\uAF5C", "\uAF5C"], /* Hangul Syllable */
  ["\uAF5D", "\uAF5D"], /* Hangul Syllable */
  ["\uAF5E", "\uAF5E"], /* Hangul Syllable */
  ["\uAF5F", "\uAF5F"], /* Hangul Syllable */
  ["\uAF60", "\uAF60"], /* Hangul Syllable */
  ["\uAF61", "\uAF61"], /* Hangul Syllable */
  ["\uAF62", "\uAF62"], /* Hangul Syllable */
  ["\uAF63", "\uAF63"], /* Hangul Syllable */
  ["\uAF64", "\uAF64"], /* Hangul Syllable */
  ["\uAF65", "\uAF65"], /* Hangul Syllable */
  ["\uAF66", "\uAF66"], /* Hangul Syllable */
  ["\uAF67", "\uAF67"], /* Hangul Syllable */
  ["\uAF68", "\uAF68"], /* Hangul Syllable */
  ["\uAF69", "\uAF69"], /* Hangul Syllable */
  ["\uAF6A", "\uAF6A"], /* Hangul Syllable */
  ["\uAF6B", "\uAF6B"], /* Hangul Syllable */
  ["\uAF6C", "\uAF6C"], /* Hangul Syllable */
  ["\uAF6D", "\uAF6D"], /* Hangul Syllable */
  ["\uAF6E", "\uAF6E"], /* Hangul Syllable */
  ["\uAF6F", "\uAF6F"], /* Hangul Syllable */
  ["\uAF70", "\uAF70"], /* Hangul Syllable */
  ["\uAF71", "\uAF71"], /* Hangul Syllable */
  ["\uAF72", "\uAF72"], /* Hangul Syllable */
  ["\uAF73", "\uAF73"], /* Hangul Syllable */
  ["\uAF74", "\uAF74"], /* Hangul Syllable */
  ["\uAF75", "\uAF75"], /* Hangul Syllable */
  ["\uAF76", "\uAF76"], /* Hangul Syllable */
  ["\uAF77", "\uAF77"], /* Hangul Syllable */
  ["\uAF78", "\uAF78"], /* Hangul Syllable */
  ["\uAF79", "\uAF79"], /* Hangul Syllable */
  ["\uAF7A", "\uAF7A"], /* Hangul Syllable */
  ["\uAF7B", "\uAF7B"], /* Hangul Syllable */
  ["\uAF7C", "\uAF7C"], /* Hangul Syllable */
  ["\uAF7D", "\uAF7D"], /* Hangul Syllable */
  ["\uAF7E", "\uAF7E"], /* Hangul Syllable */
  ["\uAF7F", "\uAF7F"], /* Hangul Syllable */
  ["\uAF80", "\uAF80"], /* Hangul Syllable */
  ["\uAF81", "\uAF81"], /* Hangul Syllable */
  ["\uAF82", "\uAF82"], /* Hangul Syllable */
  ["\uAF83", "\uAF83"], /* Hangul Syllable */
  ["\uAF84", "\uAF84"], /* Hangul Syllable */
  ["\uAF85", "\uAF85"], /* Hangul Syllable */
  ["\uAF86", "\uAF86"], /* Hangul Syllable */
  ["\uAF87", "\uAF87"], /* Hangul Syllable */
  ["\uAF88", "\uAF88"], /* Hangul Syllable */
  ["\uAF89", "\uAF89"], /* Hangul Syllable */
  ["\uAF8A", "\uAF8A"], /* Hangul Syllable */
  ["\uAF8B", "\uAF8B"], /* Hangul Syllable */
  ["\uAF8C", "\uAF8C"], /* Hangul Syllable */
  ["\uAF8D", "\uAF8D"], /* Hangul Syllable */
  ["\uAF8E", "\uAF8E"], /* Hangul Syllable */
  ["\uAF8F", "\uAF8F"], /* Hangul Syllable */
  ["\uAF90", "\uAF90"], /* Hangul Syllable */
  ["\uAF91", "\uAF91"], /* Hangul Syllable */
  ["\uAF92", "\uAF92"], /* Hangul Syllable */
  ["\uAF93", "\uAF93"], /* Hangul Syllable */
  ["\uAF94", "\uAF94"], /* Hangul Syllable */
  ["\uAF95", "\uAF95"], /* Hangul Syllable */
  ["\uAF96", "\uAF96"], /* Hangul Syllable */
  ["\uAF97", "\uAF97"], /* Hangul Syllable */
  ["\uAF98", "\uAF98"], /* Hangul Syllable */
  ["\uAF99", "\uAF99"], /* Hangul Syllable */
  ["\uAF9A", "\uAF9A"], /* Hangul Syllable */
  ["\uAF9B", "\uAF9B"], /* Hangul Syllable */
  ["\uAF9C", "\uAF9C"], /* Hangul Syllable */
  ["\uAF9D", "\uAF9D"], /* Hangul Syllable */
  ["\uAF9E", "\uAF9E"], /* Hangul Syllable */
  ["\uAF9F", "\uAF9F"], /* Hangul Syllable */
  ["\uAFA0", "\uAFA0"], /* Hangul Syllable */
  ["\uAFA1", "\uAFA1"], /* Hangul Syllable */
  ["\uAFA2", "\uAFA2"], /* Hangul Syllable */
  ["\uAFA3", "\uAFA3"], /* Hangul Syllable */
  ["\uAFA4", "\uAFA4"], /* Hangul Syllable */
  ["\uAFA5", "\uAFA5"], /* Hangul Syllable */
  ["\uAFA6", "\uAFA6"], /* Hangul Syllable */
  ["\uAFA7", "\uAFA7"], /* Hangul Syllable */
  ["\uAFA8", "\uAFA8"], /* Hangul Syllable */
  ["\uAFA9", "\uAFA9"], /* Hangul Syllable */
  ["\uAFAA", "\uAFAA"], /* Hangul Syllable */
  ["\uAFAB", "\uAFAB"], /* Hangul Syllable */
  ["\uAFAC", "\uAFAC"], /* Hangul Syllable */
  ["\uAFAD", "\uAFAD"], /* Hangul Syllable */
  ["\uAFAE", "\uAFAE"], /* Hangul Syllable */
  ["\uAFAF", "\uAFAF"], /* Hangul Syllable */
  ["\uAFB0", "\uAFB0"], /* Hangul Syllable */
  ["\uAFB1", "\uAFB1"], /* Hangul Syllable */
  ["\uAFB2", "\uAFB2"], /* Hangul Syllable */
  ["\uAFB3", "\uAFB3"], /* Hangul Syllable */
  ["\uAFB4", "\uAFB4"], /* Hangul Syllable */
  ["\uAFB5", "\uAFB5"], /* Hangul Syllable */
  ["\uAFB6", "\uAFB6"], /* Hangul Syllable */
  ["\uAFB7", "\uAFB7"], /* Hangul Syllable */
  ["\uAFB8", "\uAFB8"], /* Hangul Syllable */
  ["\uAFB9", "\uAFB9"], /* Hangul Syllable */
  ["\uAFBA", "\uAFBA"], /* Hangul Syllable */
  ["\uAFBB", "\uAFBB"], /* Hangul Syllable */
  ["\uAFBC", "\uAFBC"], /* Hangul Syllable */
  ["\uAFBD", "\uAFBD"], /* Hangul Syllable */
  ["\uAFBE", "\uAFBE"], /* Hangul Syllable */
  ["\uAFBF", "\uAFBF"], /* Hangul Syllable */
  ["\uAFC0", "\uAFC0"], /* Hangul Syllable */
  ["\uAFC1", "\uAFC1"], /* Hangul Syllable */
  ["\uAFC2", "\uAFC2"], /* Hangul Syllable */
  ["\uAFC3", "\uAFC3"], /* Hangul Syllable */
  ["\uAFC4", "\uAFC4"], /* Hangul Syllable */
  ["\uAFC5", "\uAFC5"], /* Hangul Syllable */
  ["\uAFC6", "\uAFC6"], /* Hangul Syllable */
  ["\uAFC7", "\uAFC7"], /* Hangul Syllable */
  ["\uAFC8", "\uAFC8"], /* Hangul Syllable */
  ["\uAFC9", "\uAFC9"], /* Hangul Syllable */
  ["\uAFCA", "\uAFCA"], /* Hangul Syllable */
  ["\uAFCB", "\uAFCB"], /* Hangul Syllable */
  ["\uAFCC", "\uAFCC"], /* Hangul Syllable */
  ["\uAFCD", "\uAFCD"], /* Hangul Syllable */
  ["\uAFCE", "\uAFCE"], /* Hangul Syllable */
  ["\uAFCF", "\uAFCF"], /* Hangul Syllable */
  ["\uAFD0", "\uAFD0"], /* Hangul Syllable */
  ["\uAFD1", "\uAFD1"], /* Hangul Syllable */
  ["\uAFD2", "\uAFD2"], /* Hangul Syllable */
  ["\uAFD3", "\uAFD3"], /* Hangul Syllable */
  ["\uAFD4", "\uAFD4"], /* Hangul Syllable */
  ["\uAFD5", "\uAFD5"], /* Hangul Syllable */
  ["\uAFD6", "\uAFD6"], /* Hangul Syllable */
  ["\uAFD7", "\uAFD7"], /* Hangul Syllable */
  ["\uAFD8", "\uAFD8"], /* Hangul Syllable */
  ["\uAFD9", "\uAFD9"], /* Hangul Syllable */
  ["\uAFDA", "\uAFDA"], /* Hangul Syllable */
  ["\uAFDB", "\uAFDB"], /* Hangul Syllable */
  ["\uAFDC", "\uAFDC"], /* Hangul Syllable */
  ["\uAFDD", "\uAFDD"], /* Hangul Syllable */
  ["\uAFDE", "\uAFDE"], /* Hangul Syllable */
  ["\uAFDF", "\uAFDF"], /* Hangul Syllable */
  ["\uAFE0", "\uAFE0"], /* Hangul Syllable */
  ["\uAFE1", "\uAFE1"], /* Hangul Syllable */
  ["\uAFE2", "\uAFE2"], /* Hangul Syllable */
  ["\uAFE3", "\uAFE3"], /* Hangul Syllable */
  ["\uAFE4", "\uAFE4"], /* Hangul Syllable */
  ["\uAFE5", "\uAFE5"], /* Hangul Syllable */
  ["\uAFE6", "\uAFE6"], /* Hangul Syllable */
  ["\uAFE7", "\uAFE7"], /* Hangul Syllable */
  ["\uAFE8", "\uAFE8"], /* Hangul Syllable */
  ["\uAFE9", "\uAFE9"], /* Hangul Syllable */
  ["\uAFEA", "\uAFEA"], /* Hangul Syllable */
  ["\uAFEB", "\uAFEB"], /* Hangul Syllable */
  ["\uAFEC", "\uAFEC"], /* Hangul Syllable */
  ["\uAFED", "\uAFED"], /* Hangul Syllable */
  ["\uAFEE", "\uAFEE"], /* Hangul Syllable */
  ["\uAFEF", "\uAFEF"], /* Hangul Syllable */
  ["\uAFF0", "\uAFF0"], /* Hangul Syllable */
  ["\uAFF1", "\uAFF1"], /* Hangul Syllable */
  ["\uAFF2", "\uAFF2"], /* Hangul Syllable */
  ["\uAFF3", "\uAFF3"], /* Hangul Syllable */
  ["\uAFF4", "\uAFF4"], /* Hangul Syllable */
  ["\uAFF5", "\uAFF5"], /* Hangul Syllable */
  ["\uAFF6", "\uAFF6"], /* Hangul Syllable */
  ["\uAFF7", "\uAFF7"], /* Hangul Syllable */
  ["\uAFF8", "\uAFF8"], /* Hangul Syllable */
  ["\uAFF9", "\uAFF9"], /* Hangul Syllable */
  ["\uAFFA", "\uAFFA"], /* Hangul Syllable */
  ["\uAFFB", "\uAFFB"], /* Hangul Syllable */
  ["\uAFFC", "\uAFFC"], /* Hangul Syllable */
  ["\uAFFD", "\uAFFD"], /* Hangul Syllable */
  ["\uAFFE", "\uAFFE"], /* Hangul Syllable */
  ["\uAFFF", "\uAFFF"], /* Hangul Syllable */
  ["\uB000", "\uB000"], /* Hangul Syllable */
  ["\uB001", "\uB001"], /* Hangul Syllable */
  ["\uB002", "\uB002"], /* Hangul Syllable */
  ["\uB003", "\uB003"], /* Hangul Syllable */
  ["\uB004", "\uB004"], /* Hangul Syllable */
  ["\uB005", "\uB005"], /* Hangul Syllable */
  ["\uB006", "\uB006"], /* Hangul Syllable */
  ["\uB007", "\uB007"], /* Hangul Syllable */
  ["\uB008", "\uB008"], /* Hangul Syllable */
  ["\uB009", "\uB009"], /* Hangul Syllable */
  ["\uB00A", "\uB00A"], /* Hangul Syllable */
  ["\uB00B", "\uB00B"], /* Hangul Syllable */
  ["\uB00C", "\uB00C"], /* Hangul Syllable */
  ["\uB00D", "\uB00D"], /* Hangul Syllable */
  ["\uB00E", "\uB00E"], /* Hangul Syllable */
  ["\uB00F", "\uB00F"], /* Hangul Syllable */
  ["\uB010", "\uB010"], /* Hangul Syllable */
  ["\uB011", "\uB011"], /* Hangul Syllable */
  ["\uB012", "\uB012"], /* Hangul Syllable */
  ["\uB013", "\uB013"], /* Hangul Syllable */
  ["\uB014", "\uB014"], /* Hangul Syllable */
  ["\uB015", "\uB015"], /* Hangul Syllable */
  ["\uB016", "\uB016"], /* Hangul Syllable */
  ["\uB017", "\uB017"], /* Hangul Syllable */
  ["\uB018", "\uB018"], /* Hangul Syllable */
  ["\uB019", "\uB019"], /* Hangul Syllable */
  ["\uB01A", "\uB01A"], /* Hangul Syllable */
  ["\uB01B", "\uB01B"], /* Hangul Syllable */
  ["\uB01C", "\uB01C"], /* Hangul Syllable */
  ["\uB01D", "\uB01D"], /* Hangul Syllable */
  ["\uB01E", "\uB01E"], /* Hangul Syllable */
  ["\uB01F", "\uB01F"], /* Hangul Syllable */
  ["\uB020", "\uB020"], /* Hangul Syllable */
  ["\uB021", "\uB021"], /* Hangul Syllable */
  ["\uB022", "\uB022"], /* Hangul Syllable */
  ["\uB023", "\uB023"], /* Hangul Syllable */
  ["\uB024", "\uB024"], /* Hangul Syllable */
  ["\uB025", "\uB025"], /* Hangul Syllable */
  ["\uB026", "\uB026"], /* Hangul Syllable */
  ["\uB027", "\uB027"], /* Hangul Syllable */
  ["\uB028", "\uB028"], /* Hangul Syllable */
  ["\uB029", "\uB029"], /* Hangul Syllable */
  ["\uB02A", "\uB02A"], /* Hangul Syllable */
  ["\uB02B", "\uB02B"], /* Hangul Syllable */
  ["\uB02C", "\uB02C"], /* Hangul Syllable */
  ["\uB02D", "\uB02D"], /* Hangul Syllable */
  ["\uB02E", "\uB02E"], /* Hangul Syllable */
  ["\uB02F", "\uB02F"], /* Hangul Syllable */
  ["\uB030", "\uB030"], /* Hangul Syllable */
  ["\uB031", "\uB031"], /* Hangul Syllable */
  ["\uB032", "\uB032"], /* Hangul Syllable */
  ["\uB033", "\uB033"], /* Hangul Syllable */
  ["\uB034", "\uB034"], /* Hangul Syllable */
  ["\uB035", "\uB035"], /* Hangul Syllable */
  ["\uB036", "\uB036"], /* Hangul Syllable */
  ["\uB037", "\uB037"], /* Hangul Syllable */
  ["\uB038", "\uB038"], /* Hangul Syllable */
  ["\uB039", "\uB039"], /* Hangul Syllable */
  ["\uB03A", "\uB03A"], /* Hangul Syllable */
  ["\uB03B", "\uB03B"], /* Hangul Syllable */
  ["\uB03C", "\uB03C"], /* Hangul Syllable */
  ["\uB03D", "\uB03D"], /* Hangul Syllable */
  ["\uB03E", "\uB03E"], /* Hangul Syllable */
  ["\uB03F", "\uB03F"], /* Hangul Syllable */
  ["\uB040", "\uB040"], /* Hangul Syllable */
  ["\uB041", "\uB041"], /* Hangul Syllable */
  ["\uB042", "\uB042"], /* Hangul Syllable */
  ["\uB043", "\uB043"], /* Hangul Syllable */
  ["\uB044", "\uB044"], /* Hangul Syllable */
  ["\uB045", "\uB045"], /* Hangul Syllable */
  ["\uB046", "\uB046"], /* Hangul Syllable */
  ["\uB047", "\uB047"], /* Hangul Syllable */
  ["\uB048", "\uB048"], /* Hangul Syllable */
  ["\uB049", "\uB049"], /* Hangul Syllable */
  ["\uB04A", "\uB04A"], /* Hangul Syllable */
  ["\uB04B", "\uB04B"], /* Hangul Syllable */
  ["\uB04C", "\uB04C"], /* Hangul Syllable */
  ["\uB04D", "\uB04D"], /* Hangul Syllable */
  ["\uB04E", "\uB04E"], /* Hangul Syllable */
  ["\uB04F", "\uB04F"], /* Hangul Syllable */
  ["\uB050", "\uB050"], /* Hangul Syllable */
  ["\uB051", "\uB051"], /* Hangul Syllable */
  ["\uB052", "\uB052"], /* Hangul Syllable */
  ["\uB053", "\uB053"], /* Hangul Syllable */
  ["\uB054", "\uB054"], /* Hangul Syllable */
  ["\uB055", "\uB055"], /* Hangul Syllable */
  ["\uB056", "\uB056"], /* Hangul Syllable */
  ["\uB057", "\uB057"], /* Hangul Syllable */
  ["\uB058", "\uB058"], /* Hangul Syllable */
  ["\uB059", "\uB059"], /* Hangul Syllable */
  ["\uB05A", "\uB05A"], /* Hangul Syllable */
  ["\uB05B", "\uB05B"], /* Hangul Syllable */
  ["\uB05C", "\uB05C"], /* Hangul Syllable */
  ["\uB05D", "\uB05D"], /* Hangul Syllable */
  ["\uB05E", "\uB05E"], /* Hangul Syllable */
  ["\uB05F", "\uB05F"], /* Hangul Syllable */
  ["\uB060", "\uB060"], /* Hangul Syllable */
  ["\uB061", "\uB061"], /* Hangul Syllable */
  ["\uB062", "\uB062"], /* Hangul Syllable */
  ["\uB063", "\uB063"], /* Hangul Syllable */
  ["\uB064", "\uB064"], /* Hangul Syllable */
  ["\uB065", "\uB065"], /* Hangul Syllable */
  ["\uB066", "\uB066"], /* Hangul Syllable */
  ["\uB067", "\uB067"], /* Hangul Syllable */
  ["\uB068", "\uB068"], /* Hangul Syllable */
  ["\uB069", "\uB069"], /* Hangul Syllable */
  ["\uB06A", "\uB06A"], /* Hangul Syllable */
  ["\uB06B", "\uB06B"], /* Hangul Syllable */
  ["\uB06C", "\uB06C"], /* Hangul Syllable */
  ["\uB06D", "\uB06D"], /* Hangul Syllable */
  ["\uB06E", "\uB06E"], /* Hangul Syllable */
  ["\uB06F", "\uB06F"], /* Hangul Syllable */
  ["\uB070", "\uB070"], /* Hangul Syllable */
  ["\uB071", "\uB071"], /* Hangul Syllable */
  ["\uB072", "\uB072"], /* Hangul Syllable */
  ["\uB073", "\uB073"], /* Hangul Syllable */
  ["\uB074", "\uB074"], /* Hangul Syllable */
  ["\uB075", "\uB075"], /* Hangul Syllable */
  ["\uB076", "\uB076"], /* Hangul Syllable */
  ["\uB077", "\uB077"], /* Hangul Syllable */
  ["\uB078", "\uB078"], /* Hangul Syllable */
  ["\uB079", "\uB079"], /* Hangul Syllable */
  ["\uB07A", "\uB07A"], /* Hangul Syllable */
  ["\uB07B", "\uB07B"], /* Hangul Syllable */
  ["\uB07C", "\uB07C"], /* Hangul Syllable */
  ["\uB07D", "\uB07D"], /* Hangul Syllable */
  ["\uB07E", "\uB07E"], /* Hangul Syllable */
  ["\uB07F", "\uB07F"], /* Hangul Syllable */
  ["\uB080", "\uB080"], /* Hangul Syllable */
  ["\uB081", "\uB081"], /* Hangul Syllable */
  ["\uB082", "\uB082"], /* Hangul Syllable */
  ["\uB083", "\uB083"], /* Hangul Syllable */
  ["\uB084", "\uB084"], /* Hangul Syllable */
  ["\uB085", "\uB085"], /* Hangul Syllable */
  ["\uB086", "\uB086"], /* Hangul Syllable */
  ["\uB087", "\uB087"], /* Hangul Syllable */
  ["\uB088", "\uB088"], /* Hangul Syllable */
  ["\uB089", "\uB089"], /* Hangul Syllable */
  ["\uB08A", "\uB08A"], /* Hangul Syllable */
  ["\uB08B", "\uB08B"], /* Hangul Syllable */
  ["\uB08C", "\uB08C"], /* Hangul Syllable */
  ["\uB08D", "\uB08D"], /* Hangul Syllable */
  ["\uB08E", "\uB08E"], /* Hangul Syllable */
  ["\uB08F", "\uB08F"], /* Hangul Syllable */
  ["\uB090", "\uB090"], /* Hangul Syllable */
  ["\uB091", "\uB091"], /* Hangul Syllable */
  ["\uB092", "\uB092"], /* Hangul Syllable */
  ["\uB093", "\uB093"], /* Hangul Syllable */
  ["\uB094", "\uB094"], /* Hangul Syllable */
  ["\uB095", "\uB095"], /* Hangul Syllable */
  ["\uB096", "\uB096"], /* Hangul Syllable */
  ["\uB097", "\uB097"], /* Hangul Syllable */
  ["\uB098", "\uB098"], /* Hangul Syllable */
  ["\uB099", "\uB099"], /* Hangul Syllable */
  ["\uB09A", "\uB09A"], /* Hangul Syllable */
  ["\uB09B", "\uB09B"], /* Hangul Syllable */
  ["\uB09C", "\uB09C"], /* Hangul Syllable */
  ["\uB09D", "\uB09D"], /* Hangul Syllable */
  ["\uB09E", "\uB09E"], /* Hangul Syllable */
  ["\uB09F", "\uB09F"], /* Hangul Syllable */
  ["\uB0A0", "\uB0A0"], /* Hangul Syllable */
  ["\uB0A1", "\uB0A1"], /* Hangul Syllable */
  ["\uB0A2", "\uB0A2"], /* Hangul Syllable */
  ["\uB0A3", "\uB0A3"], /* Hangul Syllable */
  ["\uB0A4", "\uB0A4"], /* Hangul Syllable */
  ["\uB0A5", "\uB0A5"], /* Hangul Syllable */
  ["\uB0A6", "\uB0A6"], /* Hangul Syllable */
  ["\uB0A7", "\uB0A7"], /* Hangul Syllable */
  ["\uB0A8", "\uB0A8"], /* Hangul Syllable */
  ["\uB0A9", "\uB0A9"], /* Hangul Syllable */
  ["\uB0AA", "\uB0AA"], /* Hangul Syllable */
  ["\uB0AB", "\uB0AB"], /* Hangul Syllable */
  ["\uB0AC", "\uB0AC"], /* Hangul Syllable */
  ["\uB0AD", "\uB0AD"], /* Hangul Syllable */
  ["\uB0AE", "\uB0AE"], /* Hangul Syllable */
  ["\uB0AF", "\uB0AF"], /* Hangul Syllable */
  ["\uB0B0", "\uB0B0"], /* Hangul Syllable */
  ["\uB0B1", "\uB0B1"], /* Hangul Syllable */
  ["\uB0B2", "\uB0B2"], /* Hangul Syllable */
  ["\uB0B3", "\uB0B3"], /* Hangul Syllable */
  ["\uB0B4", "\uB0B4"], /* Hangul Syllable */
  ["\uB0B5", "\uB0B5"], /* Hangul Syllable */
  ["\uB0B6", "\uB0B6"], /* Hangul Syllable */
  ["\uB0B7", "\uB0B7"], /* Hangul Syllable */
  ["\uB0B8", "\uB0B8"], /* Hangul Syllable */
  ["\uB0B9", "\uB0B9"], /* Hangul Syllable */
  ["\uB0BA", "\uB0BA"], /* Hangul Syllable */
  ["\uB0BB", "\uB0BB"], /* Hangul Syllable */
  ["\uB0BC", "\uB0BC"], /* Hangul Syllable */
  ["\uB0BD", "\uB0BD"], /* Hangul Syllable */
  ["\uB0BE", "\uB0BE"], /* Hangul Syllable */
  ["\uB0BF", "\uB0BF"], /* Hangul Syllable */
  ["\uB0C0", "\uB0C0"], /* Hangul Syllable */
  ["\uB0C1", "\uB0C1"], /* Hangul Syllable */
  ["\uB0C2", "\uB0C2"], /* Hangul Syllable */
  ["\uB0C3", "\uB0C3"], /* Hangul Syllable */
  ["\uB0C4", "\uB0C4"], /* Hangul Syllable */
  ["\uB0C5", "\uB0C5"], /* Hangul Syllable */
  ["\uB0C6", "\uB0C6"], /* Hangul Syllable */
  ["\uB0C7", "\uB0C7"], /* Hangul Syllable */
  ["\uB0C8", "\uB0C8"], /* Hangul Syllable */
  ["\uB0C9", "\uB0C9"], /* Hangul Syllable */
  ["\uB0CA", "\uB0CA"], /* Hangul Syllable */
  ["\uB0CB", "\uB0CB"], /* Hangul Syllable */
  ["\uB0CC", "\uB0CC"], /* Hangul Syllable */
  ["\uB0CD", "\uB0CD"], /* Hangul Syllable */
  ["\uB0CE", "\uB0CE"], /* Hangul Syllable */
  ["\uB0CF", "\uB0CF"], /* Hangul Syllable */
  ["\uB0D0", "\uB0D0"], /* Hangul Syllable */
  ["\uB0D1", "\uB0D1"], /* Hangul Syllable */
  ["\uB0D2", "\uB0D2"], /* Hangul Syllable */
  ["\uB0D3", "\uB0D3"], /* Hangul Syllable */
  ["\uB0D4", "\uB0D4"], /* Hangul Syllable */
  ["\uB0D5", "\uB0D5"], /* Hangul Syllable */
  ["\uB0D6", "\uB0D6"], /* Hangul Syllable */
  ["\uB0D7", "\uB0D7"], /* Hangul Syllable */
  ["\uB0D8", "\uB0D8"], /* Hangul Syllable */
  ["\uB0D9", "\uB0D9"], /* Hangul Syllable */
  ["\uB0DA", "\uB0DA"], /* Hangul Syllable */
  ["\uB0DB", "\uB0DB"], /* Hangul Syllable */
  ["\uB0DC", "\uB0DC"], /* Hangul Syllable */
  ["\uB0DD", "\uB0DD"], /* Hangul Syllable */
  ["\uB0DE", "\uB0DE"], /* Hangul Syllable */
  ["\uB0DF", "\uB0DF"], /* Hangul Syllable */
  ["\uB0E0", "\uB0E0"], /* Hangul Syllable */
  ["\uB0E1", "\uB0E1"], /* Hangul Syllable */
  ["\uB0E2", "\uB0E2"], /* Hangul Syllable */
  ["\uB0E3", "\uB0E3"], /* Hangul Syllable */
  ["\uB0E4", "\uB0E4"], /* Hangul Syllable */
  ["\uB0E5", "\uB0E5"], /* Hangul Syllable */
  ["\uB0E6", "\uB0E6"], /* Hangul Syllable */
  ["\uB0E7", "\uB0E7"], /* Hangul Syllable */
  ["\uB0E8", "\uB0E8"], /* Hangul Syllable */
  ["\uB0E9", "\uB0E9"], /* Hangul Syllable */
  ["\uB0EA", "\uB0EA"], /* Hangul Syllable */
  ["\uB0EB", "\uB0EB"], /* Hangul Syllable */
  ["\uB0EC", "\uB0EC"], /* Hangul Syllable */
  ["\uB0ED", "\uB0ED"], /* Hangul Syllable */
  ["\uB0EE", "\uB0EE"], /* Hangul Syllable */
  ["\uB0EF", "\uB0EF"], /* Hangul Syllable */
  ["\uB0F0", "\uB0F0"], /* Hangul Syllable */
  ["\uB0F1", "\uB0F1"], /* Hangul Syllable */
  ["\uB0F2", "\uB0F2"], /* Hangul Syllable */
  ["\uB0F3", "\uB0F3"], /* Hangul Syllable */
  ["\uB0F4", "\uB0F4"], /* Hangul Syllable */
  ["\uB0F5", "\uB0F5"], /* Hangul Syllable */
  ["\uB0F6", "\uB0F6"], /* Hangul Syllable */
  ["\uB0F7", "\uB0F7"], /* Hangul Syllable */
  ["\uB0F8", "\uB0F8"], /* Hangul Syllable */
  ["\uB0F9", "\uB0F9"], /* Hangul Syllable */
  ["\uB0FA", "\uB0FA"], /* Hangul Syllable */
  ["\uB0FB", "\uB0FB"], /* Hangul Syllable */
  ["\uB0FC", "\uB0FC"], /* Hangul Syllable */
  ["\uB0FD", "\uB0FD"], /* Hangul Syllable */
  ["\uB0FE", "\uB0FE"], /* Hangul Syllable */
  ["\uB0FF", "\uB0FF"], /* Hangul Syllable */
  ["\uB100", "\uB100"], /* Hangul Syllable */
  ["\uB101", "\uB101"], /* Hangul Syllable */
  ["\uB102", "\uB102"], /* Hangul Syllable */
  ["\uB103", "\uB103"], /* Hangul Syllable */
  ["\uB104", "\uB104"], /* Hangul Syllable */
  ["\uB105", "\uB105"], /* Hangul Syllable */
  ["\uB106", "\uB106"], /* Hangul Syllable */
  ["\uB107", "\uB107"], /* Hangul Syllable */
  ["\uB108", "\uB108"], /* Hangul Syllable */
  ["\uB109", "\uB109"], /* Hangul Syllable */
  ["\uB10A", "\uB10A"], /* Hangul Syllable */
  ["\uB10B", "\uB10B"], /* Hangul Syllable */
  ["\uB10C", "\uB10C"], /* Hangul Syllable */
  ["\uB10D", "\uB10D"], /* Hangul Syllable */
  ["\uB10E", "\uB10E"], /* Hangul Syllable */
  ["\uB10F", "\uB10F"], /* Hangul Syllable */
  ["\uB110", "\uB110"], /* Hangul Syllable */
  ["\uB111", "\uB111"], /* Hangul Syllable */
  ["\uB112", "\uB112"], /* Hangul Syllable */
  ["\uB113", "\uB113"], /* Hangul Syllable */
  ["\uB114", "\uB114"], /* Hangul Syllable */
  ["\uB115", "\uB115"], /* Hangul Syllable */
  ["\uB116", "\uB116"], /* Hangul Syllable */
  ["\uB117", "\uB117"], /* Hangul Syllable */
  ["\uB118", "\uB118"], /* Hangul Syllable */
  ["\uB119", "\uB119"], /* Hangul Syllable */
  ["\uB11A", "\uB11A"], /* Hangul Syllable */
  ["\uB11B", "\uB11B"], /* Hangul Syllable */
  ["\uB11C", "\uB11C"], /* Hangul Syllable */
  ["\uB11D", "\uB11D"], /* Hangul Syllable */
  ["\uB11E", "\uB11E"], /* Hangul Syllable */
  ["\uB11F", "\uB11F"], /* Hangul Syllable */
  ["\uB120", "\uB120"], /* Hangul Syllable */
  ["\uB121", "\uB121"], /* Hangul Syllable */
  ["\uB122", "\uB122"], /* Hangul Syllable */
  ["\uB123", "\uB123"], /* Hangul Syllable */
  ["\uB124", "\uB124"], /* Hangul Syllable */
  ["\uB125", "\uB125"], /* Hangul Syllable */
  ["\uB126", "\uB126"], /* Hangul Syllable */
  ["\uB127", "\uB127"], /* Hangul Syllable */
  ["\uB128", "\uB128"], /* Hangul Syllable */
  ["\uB129", "\uB129"], /* Hangul Syllable */
  ["\uB12A", "\uB12A"], /* Hangul Syllable */
  ["\uB12B", "\uB12B"], /* Hangul Syllable */
  ["\uB12C", "\uB12C"], /* Hangul Syllable */
  ["\uB12D", "\uB12D"], /* Hangul Syllable */
  ["\uB12E", "\uB12E"], /* Hangul Syllable */
  ["\uB12F", "\uB12F"], /* Hangul Syllable */
  ["\uB130", "\uB130"], /* Hangul Syllable */
  ["\uB131", "\uB131"], /* Hangul Syllable */
  ["\uB132", "\uB132"], /* Hangul Syllable */
  ["\uB133", "\uB133"], /* Hangul Syllable */
  ["\uB134", "\uB134"], /* Hangul Syllable */
  ["\uB135", "\uB135"], /* Hangul Syllable */
  ["\uB136", "\uB136"], /* Hangul Syllable */
  ["\uB137", "\uB137"], /* Hangul Syllable */
  ["\uB138", "\uB138"], /* Hangul Syllable */
  ["\uB139", "\uB139"], /* Hangul Syllable */
  ["\uB13A", "\uB13A"], /* Hangul Syllable */
  ["\uB13B", "\uB13B"], /* Hangul Syllable */
  ["\uB13C", "\uB13C"], /* Hangul Syllable */
  ["\uB13D", "\uB13D"], /* Hangul Syllable */
  ["\uB13E", "\uB13E"], /* Hangul Syllable */
  ["\uB13F", "\uB13F"], /* Hangul Syllable */
  ["\uB140", "\uB140"], /* Hangul Syllable */
  ["\uB141", "\uB141"], /* Hangul Syllable */
  ["\uB142", "\uB142"], /* Hangul Syllable */
  ["\uB143", "\uB143"], /* Hangul Syllable */
  ["\uB144", "\uB144"], /* Hangul Syllable */
  ["\uB145", "\uB145"], /* Hangul Syllable */
  ["\uB146", "\uB146"], /* Hangul Syllable */
  ["\uB147", "\uB147"], /* Hangul Syllable */
  ["\uB148", "\uB148"], /* Hangul Syllable */
  ["\uB149", "\uB149"], /* Hangul Syllable */
  ["\uB14A", "\uB14A"], /* Hangul Syllable */
  ["\uB14B", "\uB14B"], /* Hangul Syllable */
  ["\uB14C", "\uB14C"], /* Hangul Syllable */
  ["\uB14D", "\uB14D"], /* Hangul Syllable */
  ["\uB14E", "\uB14E"], /* Hangul Syllable */
  ["\uB14F", "\uB14F"], /* Hangul Syllable */
  ["\uB150", "\uB150"], /* Hangul Syllable */
  ["\uB151", "\uB151"], /* Hangul Syllable */
  ["\uB152", "\uB152"], /* Hangul Syllable */
  ["\uB153", "\uB153"], /* Hangul Syllable */
  ["\uB154", "\uB154"], /* Hangul Syllable */
  ["\uB155", "\uB155"], /* Hangul Syllable */
  ["\uB156", "\uB156"], /* Hangul Syllable */
  ["\uB157", "\uB157"], /* Hangul Syllable */
  ["\uB158", "\uB158"], /* Hangul Syllable */
  ["\uB159", "\uB159"], /* Hangul Syllable */
  ["\uB15A", "\uB15A"], /* Hangul Syllable */
  ["\uB15B", "\uB15B"], /* Hangul Syllable */
  ["\uB15C", "\uB15C"], /* Hangul Syllable */
  ["\uB15D", "\uB15D"], /* Hangul Syllable */
  ["\uB15E", "\uB15E"], /* Hangul Syllable */
  ["\uB15F", "\uB15F"], /* Hangul Syllable */
  ["\uB160", "\uB160"], /* Hangul Syllable */
  ["\uB161", "\uB161"], /* Hangul Syllable */
  ["\uB162", "\uB162"], /* Hangul Syllable */
  ["\uB163", "\uB163"], /* Hangul Syllable */
  ["\uB164", "\uB164"], /* Hangul Syllable */
  ["\uB165", "\uB165"], /* Hangul Syllable */
  ["\uB166", "\uB166"], /* Hangul Syllable */
  ["\uB167", "\uB167"], /* Hangul Syllable */
  ["\uB168", "\uB168"], /* Hangul Syllable */
  ["\uB169", "\uB169"], /* Hangul Syllable */
  ["\uB16A", "\uB16A"], /* Hangul Syllable */
  ["\uB16B", "\uB16B"], /* Hangul Syllable */
  ["\uB16C", "\uB16C"], /* Hangul Syllable */
  ["\uB16D", "\uB16D"], /* Hangul Syllable */
  ["\uB16E", "\uB16E"], /* Hangul Syllable */
  ["\uB16F", "\uB16F"], /* Hangul Syllable */
  ["\uB170", "\uB170"], /* Hangul Syllable */
  ["\uB171", "\uB171"], /* Hangul Syllable */
  ["\uB172", "\uB172"], /* Hangul Syllable */
  ["\uB173", "\uB173"], /* Hangul Syllable */
  ["\uB174", "\uB174"], /* Hangul Syllable */
  ["\uB175", "\uB175"], /* Hangul Syllable */
  ["\uB176", "\uB176"], /* Hangul Syllable */
  ["\uB177", "\uB177"], /* Hangul Syllable */
  ["\uB178", "\uB178"], /* Hangul Syllable */
  ["\uB179", "\uB179"], /* Hangul Syllable */
  ["\uB17A", "\uB17A"], /* Hangul Syllable */
  ["\uB17B", "\uB17B"], /* Hangul Syllable */
  ["\uB17C", "\uB17C"], /* Hangul Syllable */
  ["\uB17D", "\uB17D"], /* Hangul Syllable */
  ["\uB17E", "\uB17E"], /* Hangul Syllable */
  ["\uB17F", "\uB17F"], /* Hangul Syllable */
  ["\uB180", "\uB180"], /* Hangul Syllable */
  ["\uB181", "\uB181"], /* Hangul Syllable */
  ["\uB182", "\uB182"], /* Hangul Syllable */
  ["\uB183", "\uB183"], /* Hangul Syllable */
  ["\uB184", "\uB184"], /* Hangul Syllable */
  ["\uB185", "\uB185"], /* Hangul Syllable */
  ["\uB186", "\uB186"], /* Hangul Syllable */
  ["\uB187", "\uB187"], /* Hangul Syllable */
  ["\uB188", "\uB188"], /* Hangul Syllable */
  ["\uB189", "\uB189"], /* Hangul Syllable */
  ["\uB18A", "\uB18A"], /* Hangul Syllable */
  ["\uB18B", "\uB18B"], /* Hangul Syllable */
  ["\uB18C", "\uB18C"], /* Hangul Syllable */
  ["\uB18D", "\uB18D"], /* Hangul Syllable */
  ["\uB18E", "\uB18E"], /* Hangul Syllable */
  ["\uB18F", "\uB18F"], /* Hangul Syllable */
  ["\uB190", "\uB190"], /* Hangul Syllable */
  ["\uB191", "\uB191"], /* Hangul Syllable */
  ["\uB192", "\uB192"], /* Hangul Syllable */
  ["\uB193", "\uB193"], /* Hangul Syllable */
  ["\uB194", "\uB194"], /* Hangul Syllable */
  ["\uB195", "\uB195"], /* Hangul Syllable */
  ["\uB196", "\uB196"], /* Hangul Syllable */
  ["\uB197", "\uB197"], /* Hangul Syllable */
  ["\uB198", "\uB198"], /* Hangul Syllable */
  ["\uB199", "\uB199"], /* Hangul Syllable */
  ["\uB19A", "\uB19A"], /* Hangul Syllable */
  ["\uB19B", "\uB19B"], /* Hangul Syllable */
  ["\uB19C", "\uB19C"], /* Hangul Syllable */
  ["\uB19D", "\uB19D"], /* Hangul Syllable */
  ["\uB19E", "\uB19E"], /* Hangul Syllable */
  ["\uB19F", "\uB19F"], /* Hangul Syllable */
  ["\uB1A0", "\uB1A0"], /* Hangul Syllable */
  ["\uB1A1", "\uB1A1"], /* Hangul Syllable */
  ["\uB1A2", "\uB1A2"], /* Hangul Syllable */
  ["\uB1A3", "\uB1A3"], /* Hangul Syllable */
  ["\uB1A4", "\uB1A4"], /* Hangul Syllable */
  ["\uB1A5", "\uB1A5"], /* Hangul Syllable */
  ["\uB1A6", "\uB1A6"], /* Hangul Syllable */
  ["\uB1A7", "\uB1A7"], /* Hangul Syllable */
  ["\uB1A8", "\uB1A8"], /* Hangul Syllable */
  ["\uB1A9", "\uB1A9"], /* Hangul Syllable */
  ["\uB1AA", "\uB1AA"], /* Hangul Syllable */
  ["\uB1AB", "\uB1AB"], /* Hangul Syllable */
  ["\uB1AC", "\uB1AC"], /* Hangul Syllable */
  ["\uB1AD", "\uB1AD"], /* Hangul Syllable */
  ["\uB1AE", "\uB1AE"], /* Hangul Syllable */
  ["\uB1AF", "\uB1AF"], /* Hangul Syllable */
  ["\uB1B0", "\uB1B0"], /* Hangul Syllable */
  ["\uB1B1", "\uB1B1"], /* Hangul Syllable */
  ["\uB1B2", "\uB1B2"], /* Hangul Syllable */
  ["\uB1B3", "\uB1B3"], /* Hangul Syllable */
  ["\uB1B4", "\uB1B4"], /* Hangul Syllable */
  ["\uB1B5", "\uB1B5"], /* Hangul Syllable */
  ["\uB1B6", "\uB1B6"], /* Hangul Syllable */
  ["\uB1B7", "\uB1B7"], /* Hangul Syllable */
  ["\uB1B8", "\uB1B8"], /* Hangul Syllable */
  ["\uB1B9", "\uB1B9"], /* Hangul Syllable */
  ["\uB1BA", "\uB1BA"], /* Hangul Syllable */
  ["\uB1BB", "\uB1BB"], /* Hangul Syllable */
  ["\uB1BC", "\uB1BC"], /* Hangul Syllable */
  ["\uB1BD", "\uB1BD"], /* Hangul Syllable */
  ["\uB1BE", "\uB1BE"], /* Hangul Syllable */
  ["\uB1BF", "\uB1BF"], /* Hangul Syllable */
  ["\uB1C0", "\uB1C0"], /* Hangul Syllable */
  ["\uB1C1", "\uB1C1"], /* Hangul Syllable */
  ["\uB1C2", "\uB1C2"], /* Hangul Syllable */
  ["\uB1C3", "\uB1C3"], /* Hangul Syllable */
  ["\uB1C4", "\uB1C4"], /* Hangul Syllable */
  ["\uB1C5", "\uB1C5"], /* Hangul Syllable */
  ["\uB1C6", "\uB1C6"], /* Hangul Syllable */
  ["\uB1C7", "\uB1C7"], /* Hangul Syllable */
  ["\uB1C8", "\uB1C8"], /* Hangul Syllable */
  ["\uB1C9", "\uB1C9"], /* Hangul Syllable */
  ["\uB1CA", "\uB1CA"], /* Hangul Syllable */
  ["\uB1CB", "\uB1CB"], /* Hangul Syllable */
  ["\uB1CC", "\uB1CC"], /* Hangul Syllable */
  ["\uB1CD", "\uB1CD"], /* Hangul Syllable */
  ["\uB1CE", "\uB1CE"], /* Hangul Syllable */
  ["\uB1CF", "\uB1CF"], /* Hangul Syllable */
  ["\uB1D0", "\uB1D0"], /* Hangul Syllable */
  ["\uB1D1", "\uB1D1"], /* Hangul Syllable */
  ["\uB1D2", "\uB1D2"], /* Hangul Syllable */
  ["\uB1D3", "\uB1D3"], /* Hangul Syllable */
  ["\uB1D4", "\uB1D4"], /* Hangul Syllable */
  ["\uB1D5", "\uB1D5"], /* Hangul Syllable */
  ["\uB1D6", "\uB1D6"], /* Hangul Syllable */
  ["\uB1D7", "\uB1D7"], /* Hangul Syllable */
  ["\uB1D8", "\uB1D8"], /* Hangul Syllable */
  ["\uB1D9", "\uB1D9"], /* Hangul Syllable */
  ["\uB1DA", "\uB1DA"], /* Hangul Syllable */
  ["\uB1DB", "\uB1DB"], /* Hangul Syllable */
  ["\uB1DC", "\uB1DC"], /* Hangul Syllable */
  ["\uB1DD", "\uB1DD"], /* Hangul Syllable */
  ["\uB1DE", "\uB1DE"], /* Hangul Syllable */
  ["\uB1DF", "\uB1DF"], /* Hangul Syllable */
  ["\uB1E0", "\uB1E0"], /* Hangul Syllable */
  ["\uB1E1", "\uB1E1"], /* Hangul Syllable */
  ["\uB1E2", "\uB1E2"], /* Hangul Syllable */
  ["\uB1E3", "\uB1E3"], /* Hangul Syllable */
  ["\uB1E4", "\uB1E4"], /* Hangul Syllable */
  ["\uB1E5", "\uB1E5"], /* Hangul Syllable */
  ["\uB1E6", "\uB1E6"], /* Hangul Syllable */
  ["\uB1E7", "\uB1E7"], /* Hangul Syllable */
  ["\uB1E8", "\uB1E8"], /* Hangul Syllable */
  ["\uB1E9", "\uB1E9"], /* Hangul Syllable */
  ["\uB1EA", "\uB1EA"], /* Hangul Syllable */
  ["\uB1EB", "\uB1EB"], /* Hangul Syllable */
  ["\uB1EC", "\uB1EC"], /* Hangul Syllable */
  ["\uB1ED", "\uB1ED"], /* Hangul Syllable */
  ["\uB1EE", "\uB1EE"], /* Hangul Syllable */
  ["\uB1EF", "\uB1EF"], /* Hangul Syllable */
  ["\uB1F0", "\uB1F0"], /* Hangul Syllable */
  ["\uB1F1", "\uB1F1"], /* Hangul Syllable */
  ["\uB1F2", "\uB1F2"], /* Hangul Syllable */
  ["\uB1F3", "\uB1F3"], /* Hangul Syllable */
  ["\uB1F4", "\uB1F4"], /* Hangul Syllable */
  ["\uB1F5", "\uB1F5"], /* Hangul Syllable */
  ["\uB1F6", "\uB1F6"], /* Hangul Syllable */
  ["\uB1F7", "\uB1F7"], /* Hangul Syllable */
  ["\uB1F8", "\uB1F8"], /* Hangul Syllable */
  ["\uB1F9", "\uB1F9"], /* Hangul Syllable */
  ["\uB1FA", "\uB1FA"], /* Hangul Syllable */
  ["\uB1FB", "\uB1FB"], /* Hangul Syllable */
  ["\uB1FC", "\uB1FC"], /* Hangul Syllable */
  ["\uB1FD", "\uB1FD"], /* Hangul Syllable */
  ["\uB1FE", "\uB1FE"], /* Hangul Syllable */
  ["\uB1FF", "\uB1FF"], /* Hangul Syllable */
  ["\uB200", "\uB200"], /* Hangul Syllable */
  ["\uB201", "\uB201"], /* Hangul Syllable */
  ["\uB202", "\uB202"], /* Hangul Syllable */
  ["\uB203", "\uB203"], /* Hangul Syllable */
  ["\uB204", "\uB204"], /* Hangul Syllable */
  ["\uB205", "\uB205"], /* Hangul Syllable */
  ["\uB206", "\uB206"], /* Hangul Syllable */
  ["\uB207", "\uB207"], /* Hangul Syllable */
  ["\uB208", "\uB208"], /* Hangul Syllable */
  ["\uB209", "\uB209"], /* Hangul Syllable */
  ["\uB20A", "\uB20A"], /* Hangul Syllable */
  ["\uB20B", "\uB20B"], /* Hangul Syllable */
  ["\uB20C", "\uB20C"], /* Hangul Syllable */
  ["\uB20D", "\uB20D"], /* Hangul Syllable */
  ["\uB20E", "\uB20E"], /* Hangul Syllable */
  ["\uB20F", "\uB20F"], /* Hangul Syllable */
  ["\uB210", "\uB210"], /* Hangul Syllable */
  ["\uB211", "\uB211"], /* Hangul Syllable */
  ["\uB212", "\uB212"], /* Hangul Syllable */
  ["\uB213", "\uB213"], /* Hangul Syllable */
  ["\uB214", "\uB214"], /* Hangul Syllable */
  ["\uB215", "\uB215"], /* Hangul Syllable */
  ["\uB216", "\uB216"], /* Hangul Syllable */
  ["\uB217", "\uB217"], /* Hangul Syllable */
  ["\uB218", "\uB218"], /* Hangul Syllable */
  ["\uB219", "\uB219"], /* Hangul Syllable */
  ["\uB21A", "\uB21A"], /* Hangul Syllable */
  ["\uB21B", "\uB21B"], /* Hangul Syllable */
  ["\uB21C", "\uB21C"], /* Hangul Syllable */
  ["\uB21D", "\uB21D"], /* Hangul Syllable */
  ["\uB21E", "\uB21E"], /* Hangul Syllable */
  ["\uB21F", "\uB21F"], /* Hangul Syllable */
  ["\uB220", "\uB220"], /* Hangul Syllable */
  ["\uB221", "\uB221"], /* Hangul Syllable */
  ["\uB222", "\uB222"], /* Hangul Syllable */
  ["\uB223", "\uB223"], /* Hangul Syllable */
  ["\uB224", "\uB224"], /* Hangul Syllable */
  ["\uB225", "\uB225"], /* Hangul Syllable */
  ["\uB226", "\uB226"], /* Hangul Syllable */
  ["\uB227", "\uB227"], /* Hangul Syllable */
  ["\uB228", "\uB228"], /* Hangul Syllable */
  ["\uB229", "\uB229"], /* Hangul Syllable */
  ["\uB22A", "\uB22A"], /* Hangul Syllable */
  ["\uB22B", "\uB22B"], /* Hangul Syllable */
  ["\uB22C", "\uB22C"], /* Hangul Syllable */
  ["\uB22D", "\uB22D"], /* Hangul Syllable */
  ["\uB22E", "\uB22E"], /* Hangul Syllable */
  ["\uB22F", "\uB22F"], /* Hangul Syllable */
  ["\uB230", "\uB230"], /* Hangul Syllable */
  ["\uB231", "\uB231"], /* Hangul Syllable */
  ["\uB232", "\uB232"], /* Hangul Syllable */
  ["\uB233", "\uB233"], /* Hangul Syllable */
  ["\uB234", "\uB234"], /* Hangul Syllable */
  ["\uB235", "\uB235"], /* Hangul Syllable */
  ["\uB236", "\uB236"], /* Hangul Syllable */
  ["\uB237", "\uB237"], /* Hangul Syllable */
  ["\uB238", "\uB238"], /* Hangul Syllable */
  ["\uB239", "\uB239"], /* Hangul Syllable */
  ["\uB23A", "\uB23A"], /* Hangul Syllable */
  ["\uB23B", "\uB23B"], /* Hangul Syllable */
  ["\uB23C", "\uB23C"], /* Hangul Syllable */
  ["\uB23D", "\uB23D"], /* Hangul Syllable */
  ["\uB23E", "\uB23E"], /* Hangul Syllable */
  ["\uB23F", "\uB23F"], /* Hangul Syllable */
  ["\uB240", "\uB240"], /* Hangul Syllable */
  ["\uB241", "\uB241"], /* Hangul Syllable */
  ["\uB242", "\uB242"], /* Hangul Syllable */
  ["\uB243", "\uB243"], /* Hangul Syllable */
  ["\uB244", "\uB244"], /* Hangul Syllable */
  ["\uB245", "\uB245"], /* Hangul Syllable */
  ["\uB246", "\uB246"], /* Hangul Syllable */
  ["\uB247", "\uB247"], /* Hangul Syllable */
  ["\uB248", "\uB248"], /* Hangul Syllable */
  ["\uB249", "\uB249"], /* Hangul Syllable */
  ["\uB24A", "\uB24A"], /* Hangul Syllable */
  ["\uB24B", "\uB24B"], /* Hangul Syllable */
  ["\uB24C", "\uB24C"], /* Hangul Syllable */
  ["\uB24D", "\uB24D"], /* Hangul Syllable */
  ["\uB24E", "\uB24E"], /* Hangul Syllable */
  ["\uB24F", "\uB24F"], /* Hangul Syllable */
  ["\uB250", "\uB250"], /* Hangul Syllable */
  ["\uB251", "\uB251"], /* Hangul Syllable */
  ["\uB252", "\uB252"], /* Hangul Syllable */
  ["\uB253", "\uB253"], /* Hangul Syllable */
  ["\uB254", "\uB254"], /* Hangul Syllable */
  ["\uB255", "\uB255"], /* Hangul Syllable */
  ["\uB256", "\uB256"], /* Hangul Syllable */
  ["\uB257", "\uB257"], /* Hangul Syllable */
  ["\uB258", "\uB258"], /* Hangul Syllable */
  ["\uB259", "\uB259"], /* Hangul Syllable */
  ["\uB25A", "\uB25A"], /* Hangul Syllable */
  ["\uB25B", "\uB25B"], /* Hangul Syllable */
  ["\uB25C", "\uB25C"], /* Hangul Syllable */
  ["\uB25D", "\uB25D"], /* Hangul Syllable */
  ["\uB25E", "\uB25E"], /* Hangul Syllable */
  ["\uB25F", "\uB25F"], /* Hangul Syllable */
  ["\uB260", "\uB260"], /* Hangul Syllable */
  ["\uB261", "\uB261"], /* Hangul Syllable */
  ["\uB262", "\uB262"], /* Hangul Syllable */
  ["\uB263", "\uB263"], /* Hangul Syllable */
  ["\uB264", "\uB264"], /* Hangul Syllable */
  ["\uB265", "\uB265"], /* Hangul Syllable */
  ["\uB266", "\uB266"], /* Hangul Syllable */
  ["\uB267", "\uB267"], /* Hangul Syllable */
  ["\uB268", "\uB268"], /* Hangul Syllable */
  ["\uB269", "\uB269"], /* Hangul Syllable */
  ["\uB26A", "\uB26A"], /* Hangul Syllable */
  ["\uB26B", "\uB26B"], /* Hangul Syllable */
  ["\uB26C", "\uB26C"], /* Hangul Syllable */
  ["\uB26D", "\uB26D"], /* Hangul Syllable */
  ["\uB26E", "\uB26E"], /* Hangul Syllable */
  ["\uB26F", "\uB26F"], /* Hangul Syllable */
  ["\uB270", "\uB270"], /* Hangul Syllable */
  ["\uB271", "\uB271"], /* Hangul Syllable */
  ["\uB272", "\uB272"], /* Hangul Syllable */
  ["\uB273", "\uB273"], /* Hangul Syllable */
  ["\uB274", "\uB274"], /* Hangul Syllable */
  ["\uB275", "\uB275"], /* Hangul Syllable */
  ["\uB276", "\uB276"], /* Hangul Syllable */
  ["\uB277", "\uB277"], /* Hangul Syllable */
  ["\uB278", "\uB278"], /* Hangul Syllable */
  ["\uB279", "\uB279"], /* Hangul Syllable */
  ["\uB27A", "\uB27A"], /* Hangul Syllable */
  ["\uB27B", "\uB27B"], /* Hangul Syllable */
  ["\uB27C", "\uB27C"], /* Hangul Syllable */
  ["\uB27D", "\uB27D"], /* Hangul Syllable */
  ["\uB27E", "\uB27E"], /* Hangul Syllable */
  ["\uB27F", "\uB27F"], /* Hangul Syllable */
  ["\uB280", "\uB280"], /* Hangul Syllable */
  ["\uB281", "\uB281"], /* Hangul Syllable */
  ["\uB282", "\uB282"], /* Hangul Syllable */
  ["\uB283", "\uB283"], /* Hangul Syllable */
  ["\uB284", "\uB284"], /* Hangul Syllable */
  ["\uB285", "\uB285"], /* Hangul Syllable */
  ["\uB286", "\uB286"], /* Hangul Syllable */
  ["\uB287", "\uB287"], /* Hangul Syllable */
  ["\uB288", "\uB288"], /* Hangul Syllable */
  ["\uB289", "\uB289"], /* Hangul Syllable */
  ["\uB28A", "\uB28A"], /* Hangul Syllable */
  ["\uB28B", "\uB28B"], /* Hangul Syllable */
  ["\uB28C", "\uB28C"], /* Hangul Syllable */
  ["\uB28D", "\uB28D"], /* Hangul Syllable */
  ["\uB28E", "\uB28E"], /* Hangul Syllable */
  ["\uB28F", "\uB28F"], /* Hangul Syllable */
  ["\uB290", "\uB290"], /* Hangul Syllable */
  ["\uB291", "\uB291"], /* Hangul Syllable */
  ["\uB292", "\uB292"], /* Hangul Syllable */
  ["\uB293", "\uB293"], /* Hangul Syllable */
  ["\uB294", "\uB294"], /* Hangul Syllable */
  ["\uB295", "\uB295"], /* Hangul Syllable */
  ["\uB296", "\uB296"], /* Hangul Syllable */
  ["\uB297", "\uB297"], /* Hangul Syllable */
  ["\uB298", "\uB298"], /* Hangul Syllable */
  ["\uB299", "\uB299"], /* Hangul Syllable */
  ["\uB29A", "\uB29A"], /* Hangul Syllable */
  ["\uB29B", "\uB29B"], /* Hangul Syllable */
  ["\uB29C", "\uB29C"], /* Hangul Syllable */
  ["\uB29D", "\uB29D"], /* Hangul Syllable */
  ["\uB29E", "\uB29E"], /* Hangul Syllable */
  ["\uB29F", "\uB29F"], /* Hangul Syllable */
  ["\uB2A0", "\uB2A0"], /* Hangul Syllable */
  ["\uB2A1", "\uB2A1"], /* Hangul Syllable */
  ["\uB2A2", "\uB2A2"], /* Hangul Syllable */
  ["\uB2A3", "\uB2A3"], /* Hangul Syllable */
  ["\uB2A4", "\uB2A4"], /* Hangul Syllable */
  ["\uB2A5", "\uB2A5"], /* Hangul Syllable */
  ["\uB2A6", "\uB2A6"], /* Hangul Syllable */
  ["\uB2A7", "\uB2A7"], /* Hangul Syllable */
  ["\uB2A8", "\uB2A8"], /* Hangul Syllable */
  ["\uB2A9", "\uB2A9"], /* Hangul Syllable */
  ["\uB2AA", "\uB2AA"], /* Hangul Syllable */
  ["\uB2AB", "\uB2AB"], /* Hangul Syllable */
  ["\uB2AC", "\uB2AC"], /* Hangul Syllable */
  ["\uB2AD", "\uB2AD"], /* Hangul Syllable */
  ["\uB2AE", "\uB2AE"], /* Hangul Syllable */
  ["\uB2AF", "\uB2AF"], /* Hangul Syllable */
  ["\uB2B0", "\uB2B0"], /* Hangul Syllable */
  ["\uB2B1", "\uB2B1"], /* Hangul Syllable */
  ["\uB2B2", "\uB2B2"], /* Hangul Syllable */
  ["\uB2B3", "\uB2B3"], /* Hangul Syllable */
  ["\uB2B4", "\uB2B4"], /* Hangul Syllable */
  ["\uB2B5", "\uB2B5"], /* Hangul Syllable */
  ["\uB2B6", "\uB2B6"], /* Hangul Syllable */
  ["\uB2B7", "\uB2B7"], /* Hangul Syllable */
  ["\uB2B8", "\uB2B8"], /* Hangul Syllable */
  ["\uB2B9", "\uB2B9"], /* Hangul Syllable */
  ["\uB2BA", "\uB2BA"], /* Hangul Syllable */
  ["\uB2BB", "\uB2BB"], /* Hangul Syllable */
  ["\uB2BC", "\uB2BC"], /* Hangul Syllable */
  ["\uB2BD", "\uB2BD"], /* Hangul Syllable */
  ["\uB2BE", "\uB2BE"], /* Hangul Syllable */
  ["\uB2BF", "\uB2BF"], /* Hangul Syllable */
  ["\uB2C0", "\uB2C0"], /* Hangul Syllable */
  ["\uB2C1", "\uB2C1"], /* Hangul Syllable */
  ["\uB2C2", "\uB2C2"], /* Hangul Syllable */
  ["\uB2C3", "\uB2C3"], /* Hangul Syllable */
  ["\uB2C4", "\uB2C4"], /* Hangul Syllable */
  ["\uB2C5", "\uB2C5"], /* Hangul Syllable */
  ["\uB2C6", "\uB2C6"], /* Hangul Syllable */
  ["\uB2C7", "\uB2C7"], /* Hangul Syllable */
  ["\uB2C8", "\uB2C8"], /* Hangul Syllable */
  ["\uB2C9", "\uB2C9"], /* Hangul Syllable */
  ["\uB2CA", "\uB2CA"], /* Hangul Syllable */
  ["\uB2CB", "\uB2CB"], /* Hangul Syllable */
  ["\uB2CC", "\uB2CC"], /* Hangul Syllable */
  ["\uB2CD", "\uB2CD"], /* Hangul Syllable */
  ["\uB2CE", "\uB2CE"], /* Hangul Syllable */
  ["\uB2CF", "\uB2CF"], /* Hangul Syllable */
  ["\uB2D0", "\uB2D0"], /* Hangul Syllable */
  ["\uB2D1", "\uB2D1"], /* Hangul Syllable */
  ["\uB2D2", "\uB2D2"], /* Hangul Syllable */
  ["\uB2D3", "\uB2D3"], /* Hangul Syllable */
  ["\uB2D4", "\uB2D4"], /* Hangul Syllable */
  ["\uB2D5", "\uB2D5"], /* Hangul Syllable */
  ["\uB2D6", "\uB2D6"], /* Hangul Syllable */
  ["\uB2D7", "\uB2D7"], /* Hangul Syllable */
  ["\uB2D8", "\uB2D8"], /* Hangul Syllable */
  ["\uB2D9", "\uB2D9"], /* Hangul Syllable */
  ["\uB2DA", "\uB2DA"], /* Hangul Syllable */
  ["\uB2DB", "\uB2DB"], /* Hangul Syllable */
  ["\uB2DC", "\uB2DC"], /* Hangul Syllable */
  ["\uB2DD", "\uB2DD"], /* Hangul Syllable */
  ["\uB2DE", "\uB2DE"], /* Hangul Syllable */
  ["\uB2DF", "\uB2DF"], /* Hangul Syllable */
  ["\uB2E0", "\uB2E0"], /* Hangul Syllable */
  ["\uB2E1", "\uB2E1"], /* Hangul Syllable */
  ["\uB2E2", "\uB2E2"], /* Hangul Syllable */
  ["\uB2E3", "\uB2E3"], /* Hangul Syllable */
  ["\uB2E4", "\uB2E4"], /* Hangul Syllable */
  ["\uB2E5", "\uB2E5"], /* Hangul Syllable */
  ["\uB2E6", "\uB2E6"], /* Hangul Syllable */
  ["\uB2E7", "\uB2E7"], /* Hangul Syllable */
  ["\uB2E8", "\uB2E8"], /* Hangul Syllable */
  ["\uB2E9", "\uB2E9"], /* Hangul Syllable */
  ["\uB2EA", "\uB2EA"], /* Hangul Syllable */
  ["\uB2EB", "\uB2EB"], /* Hangul Syllable */
  ["\uB2EC", "\uB2EC"], /* Hangul Syllable */
  ["\uB2ED", "\uB2ED"], /* Hangul Syllable */
  ["\uB2EE", "\uB2EE"], /* Hangul Syllable */
  ["\uB2EF", "\uB2EF"], /* Hangul Syllable */
  ["\uB2F0", "\uB2F0"], /* Hangul Syllable */
  ["\uB2F1", "\uB2F1"], /* Hangul Syllable */
  ["\uB2F2", "\uB2F2"], /* Hangul Syllable */
  ["\uB2F3", "\uB2F3"], /* Hangul Syllable */
  ["\uB2F4", "\uB2F4"], /* Hangul Syllable */
  ["\uB2F5", "\uB2F5"], /* Hangul Syllable */
  ["\uB2F6", "\uB2F6"], /* Hangul Syllable */
  ["\uB2F7", "\uB2F7"], /* Hangul Syllable */
  ["\uB2F8", "\uB2F8"], /* Hangul Syllable */
  ["\uB2F9", "\uB2F9"], /* Hangul Syllable */
  ["\uB2FA", "\uB2FA"], /* Hangul Syllable */
  ["\uB2FB", "\uB2FB"], /* Hangul Syllable */
  ["\uB2FC", "\uB2FC"], /* Hangul Syllable */
  ["\uB2FD", "\uB2FD"], /* Hangul Syllable */
  ["\uB2FE", "\uB2FE"], /* Hangul Syllable */
  ["\uB2FF", "\uB2FF"], /* Hangul Syllable */
  ["\uB300", "\uB300"], /* Hangul Syllable */
  ["\uB301", "\uB301"], /* Hangul Syllable */
  ["\uB302", "\uB302"], /* Hangul Syllable */
  ["\uB303", "\uB303"], /* Hangul Syllable */
  ["\uB304", "\uB304"], /* Hangul Syllable */
  ["\uB305", "\uB305"], /* Hangul Syllable */
  ["\uB306", "\uB306"], /* Hangul Syllable */
  ["\uB307", "\uB307"], /* Hangul Syllable */
  ["\uB308", "\uB308"], /* Hangul Syllable */
  ["\uB309", "\uB309"], /* Hangul Syllable */
  ["\uB30A", "\uB30A"], /* Hangul Syllable */
  ["\uB30B", "\uB30B"], /* Hangul Syllable */
  ["\uB30C", "\uB30C"], /* Hangul Syllable */
  ["\uB30D", "\uB30D"], /* Hangul Syllable */
  ["\uB30E", "\uB30E"], /* Hangul Syllable */
  ["\uB30F", "\uB30F"], /* Hangul Syllable */
  ["\uB310", "\uB310"], /* Hangul Syllable */
  ["\uB311", "\uB311"], /* Hangul Syllable */
  ["\uB312", "\uB312"], /* Hangul Syllable */
  ["\uB313", "\uB313"], /* Hangul Syllable */
  ["\uB314", "\uB314"], /* Hangul Syllable */
  ["\uB315", "\uB315"], /* Hangul Syllable */
  ["\uB316", "\uB316"], /* Hangul Syllable */
  ["\uB317", "\uB317"], /* Hangul Syllable */
  ["\uB318", "\uB318"], /* Hangul Syllable */
  ["\uB319", "\uB319"], /* Hangul Syllable */
  ["\uB31A", "\uB31A"], /* Hangul Syllable */
  ["\uB31B", "\uB31B"], /* Hangul Syllable */
  ["\uB31C", "\uB31C"], /* Hangul Syllable */
  ["\uB31D", "\uB31D"], /* Hangul Syllable */
  ["\uB31E", "\uB31E"], /* Hangul Syllable */
  ["\uB31F", "\uB31F"], /* Hangul Syllable */
  ["\uB320", "\uB320"], /* Hangul Syllable */
  ["\uB321", "\uB321"], /* Hangul Syllable */
  ["\uB322", "\uB322"], /* Hangul Syllable */
  ["\uB323", "\uB323"], /* Hangul Syllable */
  ["\uB324", "\uB324"], /* Hangul Syllable */
  ["\uB325", "\uB325"], /* Hangul Syllable */
  ["\uB326", "\uB326"], /* Hangul Syllable */
  ["\uB327", "\uB327"], /* Hangul Syllable */
  ["\uB328", "\uB328"], /* Hangul Syllable */
  ["\uB329", "\uB329"], /* Hangul Syllable */
  ["\uB32A", "\uB32A"], /* Hangul Syllable */
  ["\uB32B", "\uB32B"], /* Hangul Syllable */
  ["\uB32C", "\uB32C"], /* Hangul Syllable */
  ["\uB32D", "\uB32D"], /* Hangul Syllable */
  ["\uB32E", "\uB32E"], /* Hangul Syllable */
  ["\uB32F", "\uB32F"], /* Hangul Syllable */
  ["\uB330", "\uB330"], /* Hangul Syllable */
  ["\uB331", "\uB331"], /* Hangul Syllable */
  ["\uB332", "\uB332"], /* Hangul Syllable */
  ["\uB333", "\uB333"], /* Hangul Syllable */
  ["\uB334", "\uB334"], /* Hangul Syllable */
  ["\uB335", "\uB335"], /* Hangul Syllable */
  ["\uB336", "\uB336"], /* Hangul Syllable */
  ["\uB337", "\uB337"], /* Hangul Syllable */
  ["\uB338", "\uB338"], /* Hangul Syllable */
  ["\uB339", "\uB339"], /* Hangul Syllable */
  ["\uB33A", "\uB33A"], /* Hangul Syllable */
  ["\uB33B", "\uB33B"], /* Hangul Syllable */
  ["\uB33C", "\uB33C"], /* Hangul Syllable */
  ["\uB33D", "\uB33D"], /* Hangul Syllable */
  ["\uB33E", "\uB33E"], /* Hangul Syllable */
  ["\uB33F", "\uB33F"], /* Hangul Syllable */
  ["\uB340", "\uB340"], /* Hangul Syllable */
  ["\uB341", "\uB341"], /* Hangul Syllable */
  ["\uB342", "\uB342"], /* Hangul Syllable */
  ["\uB343", "\uB343"], /* Hangul Syllable */
  ["\uB344", "\uB344"], /* Hangul Syllable */
  ["\uB345", "\uB345"], /* Hangul Syllable */
  ["\uB346", "\uB346"], /* Hangul Syllable */
  ["\uB347", "\uB347"], /* Hangul Syllable */
  ["\uB348", "\uB348"], /* Hangul Syllable */
  ["\uB349", "\uB349"], /* Hangul Syllable */
  ["\uB34A", "\uB34A"], /* Hangul Syllable */
  ["\uB34B", "\uB34B"], /* Hangul Syllable */
  ["\uB34C", "\uB34C"], /* Hangul Syllable */
  ["\uB34D", "\uB34D"], /* Hangul Syllable */
  ["\uB34E", "\uB34E"], /* Hangul Syllable */
  ["\uB34F", "\uB34F"], /* Hangul Syllable */
  ["\uB350", "\uB350"], /* Hangul Syllable */
  ["\uB351", "\uB351"], /* Hangul Syllable */
  ["\uB352", "\uB352"], /* Hangul Syllable */
  ["\uB353", "\uB353"], /* Hangul Syllable */
  ["\uB354", "\uB354"], /* Hangul Syllable */
  ["\uB355", "\uB355"], /* Hangul Syllable */
  ["\uB356", "\uB356"], /* Hangul Syllable */
  ["\uB357", "\uB357"], /* Hangul Syllable */
  ["\uB358", "\uB358"], /* Hangul Syllable */
  ["\uB359", "\uB359"], /* Hangul Syllable */
  ["\uB35A", "\uB35A"], /* Hangul Syllable */
  ["\uB35B", "\uB35B"], /* Hangul Syllable */
  ["\uB35C", "\uB35C"], /* Hangul Syllable */
  ["\uB35D", "\uB35D"], /* Hangul Syllable */
  ["\uB35E", "\uB35E"], /* Hangul Syllable */
  ["\uB35F", "\uB35F"], /* Hangul Syllable */
  ["\uB360", "\uB360"], /* Hangul Syllable */
  ["\uB361", "\uB361"], /* Hangul Syllable */
  ["\uB362", "\uB362"], /* Hangul Syllable */
  ["\uB363", "\uB363"], /* Hangul Syllable */
  ["\uB364", "\uB364"], /* Hangul Syllable */
  ["\uB365", "\uB365"], /* Hangul Syllable */
  ["\uB366", "\uB366"], /* Hangul Syllable */
  ["\uB367", "\uB367"], /* Hangul Syllable */
  ["\uB368", "\uB368"], /* Hangul Syllable */
  ["\uB369", "\uB369"], /* Hangul Syllable */
  ["\uB36A", "\uB36A"], /* Hangul Syllable */
  ["\uB36B", "\uB36B"], /* Hangul Syllable */
  ["\uB36C", "\uB36C"], /* Hangul Syllable */
  ["\uB36D", "\uB36D"], /* Hangul Syllable */
  ["\uB36E", "\uB36E"], /* Hangul Syllable */
  ["\uB36F", "\uB36F"], /* Hangul Syllable */
  ["\uB370", "\uB370"], /* Hangul Syllable */
  ["\uB371", "\uB371"], /* Hangul Syllable */
  ["\uB372", "\uB372"], /* Hangul Syllable */
  ["\uB373", "\uB373"], /* Hangul Syllable */
  ["\uB374", "\uB374"], /* Hangul Syllable */
  ["\uB375", "\uB375"], /* Hangul Syllable */
  ["\uB376", "\uB376"], /* Hangul Syllable */
  ["\uB377", "\uB377"], /* Hangul Syllable */
  ["\uB378", "\uB378"], /* Hangul Syllable */
  ["\uB379", "\uB379"], /* Hangul Syllable */
  ["\uB37A", "\uB37A"], /* Hangul Syllable */
  ["\uB37B", "\uB37B"], /* Hangul Syllable */
  ["\uB37C", "\uB37C"], /* Hangul Syllable */
  ["\uB37D", "\uB37D"], /* Hangul Syllable */
  ["\uB37E", "\uB37E"], /* Hangul Syllable */
  ["\uB37F", "\uB37F"], /* Hangul Syllable */
  ["\uB380", "\uB380"], /* Hangul Syllable */
  ["\uB381", "\uB381"], /* Hangul Syllable */
  ["\uB382", "\uB382"], /* Hangul Syllable */
  ["\uB383", "\uB383"], /* Hangul Syllable */
  ["\uB384", "\uB384"], /* Hangul Syllable */
  ["\uB385", "\uB385"], /* Hangul Syllable */
  ["\uB386", "\uB386"], /* Hangul Syllable */
  ["\uB387", "\uB387"], /* Hangul Syllable */
  ["\uB388", "\uB388"], /* Hangul Syllable */
  ["\uB389", "\uB389"], /* Hangul Syllable */
  ["\uB38A", "\uB38A"], /* Hangul Syllable */
  ["\uB38B", "\uB38B"], /* Hangul Syllable */
  ["\uB38C", "\uB38C"], /* Hangul Syllable */
  ["\uB38D", "\uB38D"], /* Hangul Syllable */
  ["\uB38E", "\uB38E"], /* Hangul Syllable */
  ["\uB38F", "\uB38F"], /* Hangul Syllable */
  ["\uB390", "\uB390"], /* Hangul Syllable */
  ["\uB391", "\uB391"], /* Hangul Syllable */
  ["\uB392", "\uB392"], /* Hangul Syllable */
  ["\uB393", "\uB393"], /* Hangul Syllable */
  ["\uB394", "\uB394"], /* Hangul Syllable */
  ["\uB395", "\uB395"], /* Hangul Syllable */
  ["\uB396", "\uB396"], /* Hangul Syllable */
  ["\uB397", "\uB397"], /* Hangul Syllable */
  ["\uB398", "\uB398"], /* Hangul Syllable */
  ["\uB399", "\uB399"], /* Hangul Syllable */
  ["\uB39A", "\uB39A"], /* Hangul Syllable */
  ["\uB39B", "\uB39B"], /* Hangul Syllable */
  ["\uB39C", "\uB39C"], /* Hangul Syllable */
  ["\uB39D", "\uB39D"], /* Hangul Syllable */
  ["\uB39E", "\uB39E"], /* Hangul Syllable */
  ["\uB39F", "\uB39F"], /* Hangul Syllable */
  ["\uB3A0", "\uB3A0"], /* Hangul Syllable */
  ["\uB3A1", "\uB3A1"], /* Hangul Syllable */
  ["\uB3A2", "\uB3A2"], /* Hangul Syllable */
  ["\uB3A3", "\uB3A3"], /* Hangul Syllable */
  ["\uB3A4", "\uB3A4"], /* Hangul Syllable */
  ["\uB3A5", "\uB3A5"], /* Hangul Syllable */
  ["\uB3A6", "\uB3A6"], /* Hangul Syllable */
  ["\uB3A7", "\uB3A7"], /* Hangul Syllable */
  ["\uB3A8", "\uB3A8"], /* Hangul Syllable */
  ["\uB3A9", "\uB3A9"], /* Hangul Syllable */
  ["\uB3AA", "\uB3AA"], /* Hangul Syllable */
  ["\uB3AB", "\uB3AB"], /* Hangul Syllable */
  ["\uB3AC", "\uB3AC"], /* Hangul Syllable */
  ["\uB3AD", "\uB3AD"], /* Hangul Syllable */
  ["\uB3AE", "\uB3AE"], /* Hangul Syllable */
  ["\uB3AF", "\uB3AF"], /* Hangul Syllable */
  ["\uB3B0", "\uB3B0"], /* Hangul Syllable */
  ["\uB3B1", "\uB3B1"], /* Hangul Syllable */
  ["\uB3B2", "\uB3B2"], /* Hangul Syllable */
  ["\uB3B3", "\uB3B3"], /* Hangul Syllable */
  ["\uB3B4", "\uB3B4"], /* Hangul Syllable */
  ["\uB3B5", "\uB3B5"], /* Hangul Syllable */
  ["\uB3B6", "\uB3B6"], /* Hangul Syllable */
  ["\uB3B7", "\uB3B7"], /* Hangul Syllable */
  ["\uB3B8", "\uB3B8"], /* Hangul Syllable */
  ["\uB3B9", "\uB3B9"], /* Hangul Syllable */
  ["\uB3BA", "\uB3BA"], /* Hangul Syllable */
  ["\uB3BB", "\uB3BB"], /* Hangul Syllable */
  ["\uB3BC", "\uB3BC"], /* Hangul Syllable */
  ["\uB3BD", "\uB3BD"], /* Hangul Syllable */
  ["\uB3BE", "\uB3BE"], /* Hangul Syllable */
  ["\uB3BF", "\uB3BF"], /* Hangul Syllable */
  ["\uB3C0", "\uB3C0"], /* Hangul Syllable */
  ["\uB3C1", "\uB3C1"], /* Hangul Syllable */
  ["\uB3C2", "\uB3C2"], /* Hangul Syllable */
  ["\uB3C3", "\uB3C3"], /* Hangul Syllable */
  ["\uB3C4", "\uB3C4"], /* Hangul Syllable */
  ["\uB3C5", "\uB3C5"], /* Hangul Syllable */
  ["\uB3C6", "\uB3C6"], /* Hangul Syllable */
  ["\uB3C7", "\uB3C7"], /* Hangul Syllable */
  ["\uB3C8", "\uB3C8"], /* Hangul Syllable */
  ["\uB3C9", "\uB3C9"], /* Hangul Syllable */
  ["\uB3CA", "\uB3CA"], /* Hangul Syllable */
  ["\uB3CB", "\uB3CB"], /* Hangul Syllable */
  ["\uB3CC", "\uB3CC"], /* Hangul Syllable */
  ["\uB3CD", "\uB3CD"], /* Hangul Syllable */
  ["\uB3CE", "\uB3CE"], /* Hangul Syllable */
  ["\uB3CF", "\uB3CF"], /* Hangul Syllable */
  ["\uB3D0", "\uB3D0"], /* Hangul Syllable */
  ["\uB3D1", "\uB3D1"], /* Hangul Syllable */
  ["\uB3D2", "\uB3D2"], /* Hangul Syllable */
  ["\uB3D3", "\uB3D3"], /* Hangul Syllable */
  ["\uB3D4", "\uB3D4"], /* Hangul Syllable */
  ["\uB3D5", "\uB3D5"], /* Hangul Syllable */
  ["\uB3D6", "\uB3D6"], /* Hangul Syllable */
  ["\uB3D7", "\uB3D7"], /* Hangul Syllable */
  ["\uB3D8", "\uB3D8"], /* Hangul Syllable */
  ["\uB3D9", "\uB3D9"], /* Hangul Syllable */
  ["\uB3DA", "\uB3DA"], /* Hangul Syllable */
  ["\uB3DB", "\uB3DB"], /* Hangul Syllable */
  ["\uB3DC", "\uB3DC"], /* Hangul Syllable */
  ["\uB3DD", "\uB3DD"], /* Hangul Syllable */
  ["\uB3DE", "\uB3DE"], /* Hangul Syllable */
  ["\uB3DF", "\uB3DF"], /* Hangul Syllable */
  ["\uB3E0", "\uB3E0"], /* Hangul Syllable */
  ["\uB3E1", "\uB3E1"], /* Hangul Syllable */
  ["\uB3E2", "\uB3E2"], /* Hangul Syllable */
  ["\uB3E3", "\uB3E3"], /* Hangul Syllable */
  ["\uB3E4", "\uB3E4"], /* Hangul Syllable */
  ["\uB3E5", "\uB3E5"], /* Hangul Syllable */
  ["\uB3E6", "\uB3E6"], /* Hangul Syllable */
  ["\uB3E7", "\uB3E7"], /* Hangul Syllable */
  ["\uB3E8", "\uB3E8"], /* Hangul Syllable */
  ["\uB3E9", "\uB3E9"], /* Hangul Syllable */
  ["\uB3EA", "\uB3EA"], /* Hangul Syllable */
  ["\uB3EB", "\uB3EB"], /* Hangul Syllable */
  ["\uB3EC", "\uB3EC"], /* Hangul Syllable */
  ["\uB3ED", "\uB3ED"], /* Hangul Syllable */
  ["\uB3EE", "\uB3EE"], /* Hangul Syllable */
  ["\uB3EF", "\uB3EF"], /* Hangul Syllable */
  ["\uB3F0", "\uB3F0"], /* Hangul Syllable */
  ["\uB3F1", "\uB3F1"], /* Hangul Syllable */
  ["\uB3F2", "\uB3F2"], /* Hangul Syllable */
  ["\uB3F3", "\uB3F3"], /* Hangul Syllable */
  ["\uB3F4", "\uB3F4"], /* Hangul Syllable */
  ["\uB3F5", "\uB3F5"], /* Hangul Syllable */
  ["\uB3F6", "\uB3F6"], /* Hangul Syllable */
  ["\uB3F7", "\uB3F7"], /* Hangul Syllable */
  ["\uB3F8", "\uB3F8"], /* Hangul Syllable */
  ["\uB3F9", "\uB3F9"], /* Hangul Syllable */
  ["\uB3FA", "\uB3FA"], /* Hangul Syllable */
  ["\uB3FB", "\uB3FB"], /* Hangul Syllable */
  ["\uB3FC", "\uB3FC"], /* Hangul Syllable */
  ["\uB3FD", "\uB3FD"], /* Hangul Syllable */
  ["\uB3FE", "\uB3FE"], /* Hangul Syllable */
  ["\uB3FF", "\uB3FF"], /* Hangul Syllable */
  ["\uB400", "\uB400"], /* Hangul Syllable */
  ["\uB401", "\uB401"], /* Hangul Syllable */
  ["\uB402", "\uB402"], /* Hangul Syllable */
  ["\uB403", "\uB403"], /* Hangul Syllable */
  ["\uB404", "\uB404"], /* Hangul Syllable */
  ["\uB405", "\uB405"], /* Hangul Syllable */
  ["\uB406", "\uB406"], /* Hangul Syllable */
  ["\uB407", "\uB407"], /* Hangul Syllable */
  ["\uB408", "\uB408"], /* Hangul Syllable */
  ["\uB409", "\uB409"], /* Hangul Syllable */
  ["\uB40A", "\uB40A"], /* Hangul Syllable */
  ["\uB40B", "\uB40B"], /* Hangul Syllable */
  ["\uB40C", "\uB40C"], /* Hangul Syllable */
  ["\uB40D", "\uB40D"], /* Hangul Syllable */
  ["\uB40E", "\uB40E"], /* Hangul Syllable */
  ["\uB40F", "\uB40F"], /* Hangul Syllable */
  ["\uB410", "\uB410"], /* Hangul Syllable */
  ["\uB411", "\uB411"], /* Hangul Syllable */
  ["\uB412", "\uB412"], /* Hangul Syllable */
  ["\uB413", "\uB413"], /* Hangul Syllable */
  ["\uB414", "\uB414"], /* Hangul Syllable */
  ["\uB415", "\uB415"], /* Hangul Syllable */
  ["\uB416", "\uB416"], /* Hangul Syllable */
  ["\uB417", "\uB417"], /* Hangul Syllable */
  ["\uB418", "\uB418"], /* Hangul Syllable */
  ["\uB419", "\uB419"], /* Hangul Syllable */
  ["\uB41A", "\uB41A"], /* Hangul Syllable */
  ["\uB41B", "\uB41B"], /* Hangul Syllable */
  ["\uB41C", "\uB41C"], /* Hangul Syllable */
  ["\uB41D", "\uB41D"], /* Hangul Syllable */
  ["\uB41E", "\uB41E"], /* Hangul Syllable */
  ["\uB41F", "\uB41F"], /* Hangul Syllable */
  ["\uB420", "\uB420"], /* Hangul Syllable */
  ["\uB421", "\uB421"], /* Hangul Syllable */
  ["\uB422", "\uB422"], /* Hangul Syllable */
  ["\uB423", "\uB423"], /* Hangul Syllable */
  ["\uB424", "\uB424"], /* Hangul Syllable */
  ["\uB425", "\uB425"], /* Hangul Syllable */
  ["\uB426", "\uB426"], /* Hangul Syllable */
  ["\uB427", "\uB427"], /* Hangul Syllable */
  ["\uB428", "\uB428"], /* Hangul Syllable */
  ["\uB429", "\uB429"], /* Hangul Syllable */
  ["\uB42A", "\uB42A"], /* Hangul Syllable */
  ["\uB42B", "\uB42B"], /* Hangul Syllable */
  ["\uB42C", "\uB42C"], /* Hangul Syllable */
  ["\uB42D", "\uB42D"], /* Hangul Syllable */
  ["\uB42E", "\uB42E"], /* Hangul Syllable */
  ["\uB42F", "\uB42F"], /* Hangul Syllable */
  ["\uB430", "\uB430"], /* Hangul Syllable */
  ["\uB431", "\uB431"], /* Hangul Syllable */
  ["\uB432", "\uB432"], /* Hangul Syllable */
  ["\uB433", "\uB433"], /* Hangul Syllable */
  ["\uB434", "\uB434"], /* Hangul Syllable */
  ["\uB435", "\uB435"], /* Hangul Syllable */
  ["\uB436", "\uB436"], /* Hangul Syllable */
  ["\uB437", "\uB437"], /* Hangul Syllable */
  ["\uB438", "\uB438"], /* Hangul Syllable */
  ["\uB439", "\uB439"], /* Hangul Syllable */
  ["\uB43A", "\uB43A"], /* Hangul Syllable */
  ["\uB43B", "\uB43B"], /* Hangul Syllable */
  ["\uB43C", "\uB43C"], /* Hangul Syllable */
  ["\uB43D", "\uB43D"], /* Hangul Syllable */
  ["\uB43E", "\uB43E"], /* Hangul Syllable */
  ["\uB43F", "\uB43F"], /* Hangul Syllable */
  ["\uB440", "\uB440"], /* Hangul Syllable */
  ["\uB441", "\uB441"], /* Hangul Syllable */
  ["\uB442", "\uB442"], /* Hangul Syllable */
  ["\uB443", "\uB443"], /* Hangul Syllable */
  ["\uB444", "\uB444"], /* Hangul Syllable */
  ["\uB445", "\uB445"], /* Hangul Syllable */
  ["\uB446", "\uB446"], /* Hangul Syllable */
  ["\uB447", "\uB447"], /* Hangul Syllable */
  ["\uB448", "\uB448"], /* Hangul Syllable */
  ["\uB449", "\uB449"], /* Hangul Syllable */
  ["\uB44A", "\uB44A"], /* Hangul Syllable */
  ["\uB44B", "\uB44B"], /* Hangul Syllable */
  ["\uB44C", "\uB44C"], /* Hangul Syllable */
  ["\uB44D", "\uB44D"], /* Hangul Syllable */
  ["\uB44E", "\uB44E"], /* Hangul Syllable */
  ["\uB44F", "\uB44F"], /* Hangul Syllable */
  ["\uB450", "\uB450"], /* Hangul Syllable */
  ["\uB451", "\uB451"], /* Hangul Syllable */
  ["\uB452", "\uB452"], /* Hangul Syllable */
  ["\uB453", "\uB453"], /* Hangul Syllable */
  ["\uB454", "\uB454"], /* Hangul Syllable */
  ["\uB455", "\uB455"], /* Hangul Syllable */
  ["\uB456", "\uB456"], /* Hangul Syllable */
  ["\uB457", "\uB457"], /* Hangul Syllable */
  ["\uB458", "\uB458"], /* Hangul Syllable */
  ["\uB459", "\uB459"], /* Hangul Syllable */
  ["\uB45A", "\uB45A"], /* Hangul Syllable */
  ["\uB45B", "\uB45B"], /* Hangul Syllable */
  ["\uB45C", "\uB45C"], /* Hangul Syllable */
  ["\uB45D", "\uB45D"], /* Hangul Syllable */
  ["\uB45E", "\uB45E"], /* Hangul Syllable */
  ["\uB45F", "\uB45F"], /* Hangul Syllable */
  ["\uB460", "\uB460"], /* Hangul Syllable */
  ["\uB461", "\uB461"], /* Hangul Syllable */
  ["\uB462", "\uB462"], /* Hangul Syllable */
  ["\uB463", "\uB463"], /* Hangul Syllable */
  ["\uB464", "\uB464"], /* Hangul Syllable */
  ["\uB465", "\uB465"], /* Hangul Syllable */
  ["\uB466", "\uB466"], /* Hangul Syllable */
  ["\uB467", "\uB467"], /* Hangul Syllable */
  ["\uB468", "\uB468"], /* Hangul Syllable */
  ["\uB469", "\uB469"], /* Hangul Syllable */
  ["\uB46A", "\uB46A"], /* Hangul Syllable */
  ["\uB46B", "\uB46B"], /* Hangul Syllable */
  ["\uB46C", "\uB46C"], /* Hangul Syllable */
  ["\uB46D", "\uB46D"], /* Hangul Syllable */
  ["\uB46E", "\uB46E"], /* Hangul Syllable */
  ["\uB46F", "\uB46F"], /* Hangul Syllable */
  ["\uB470", "\uB470"], /* Hangul Syllable */
  ["\uB471", "\uB471"], /* Hangul Syllable */
  ["\uB472", "\uB472"], /* Hangul Syllable */
  ["\uB473", "\uB473"], /* Hangul Syllable */
  ["\uB474", "\uB474"], /* Hangul Syllable */
  ["\uB475", "\uB475"], /* Hangul Syllable */
  ["\uB476", "\uB476"], /* Hangul Syllable */
  ["\uB477", "\uB477"], /* Hangul Syllable */
  ["\uB478", "\uB478"], /* Hangul Syllable */
  ["\uB479", "\uB479"], /* Hangul Syllable */
  ["\uB47A", "\uB47A"], /* Hangul Syllable */
  ["\uB47B", "\uB47B"], /* Hangul Syllable */
  ["\uB47C", "\uB47C"], /* Hangul Syllable */
  ["\uB47D", "\uB47D"], /* Hangul Syllable */
  ["\uB47E", "\uB47E"], /* Hangul Syllable */
  ["\uB47F", "\uB47F"], /* Hangul Syllable */
  ["\uB480", "\uB480"], /* Hangul Syllable */
  ["\uB481", "\uB481"], /* Hangul Syllable */
  ["\uB482", "\uB482"], /* Hangul Syllable */
  ["\uB483", "\uB483"], /* Hangul Syllable */
  ["\uB484", "\uB484"], /* Hangul Syllable */
  ["\uB485", "\uB485"], /* Hangul Syllable */
  ["\uB486", "\uB486"], /* Hangul Syllable */
  ["\uB487", "\uB487"], /* Hangul Syllable */
  ["\uB488", "\uB488"], /* Hangul Syllable */
  ["\uB489", "\uB489"], /* Hangul Syllable */
  ["\uB48A", "\uB48A"], /* Hangul Syllable */
  ["\uB48B", "\uB48B"], /* Hangul Syllable */
  ["\uB48C", "\uB48C"], /* Hangul Syllable */
  ["\uB48D", "\uB48D"], /* Hangul Syllable */
  ["\uB48E", "\uB48E"], /* Hangul Syllable */
  ["\uB48F", "\uB48F"], /* Hangul Syllable */
  ["\uB490", "\uB490"], /* Hangul Syllable */
  ["\uB491", "\uB491"], /* Hangul Syllable */
  ["\uB492", "\uB492"], /* Hangul Syllable */
  ["\uB493", "\uB493"], /* Hangul Syllable */
  ["\uB494", "\uB494"], /* Hangul Syllable */
  ["\uB495", "\uB495"], /* Hangul Syllable */
  ["\uB496", "\uB496"], /* Hangul Syllable */
  ["\uB497", "\uB497"], /* Hangul Syllable */
  ["\uB498", "\uB498"], /* Hangul Syllable */
  ["\uB499", "\uB499"], /* Hangul Syllable */
  ["\uB49A", "\uB49A"], /* Hangul Syllable */
  ["\uB49B", "\uB49B"], /* Hangul Syllable */
  ["\uB49C", "\uB49C"], /* Hangul Syllable */
  ["\uB49D", "\uB49D"], /* Hangul Syllable */
  ["\uB49E", "\uB49E"], /* Hangul Syllable */
  ["\uB49F", "\uB49F"], /* Hangul Syllable */
  ["\uB4A0", "\uB4A0"], /* Hangul Syllable */
  ["\uB4A1", "\uB4A1"], /* Hangul Syllable */
  ["\uB4A2", "\uB4A2"], /* Hangul Syllable */
  ["\uB4A3", "\uB4A3"], /* Hangul Syllable */
  ["\uB4A4", "\uB4A4"], /* Hangul Syllable */
  ["\uB4A5", "\uB4A5"], /* Hangul Syllable */
  ["\uB4A6", "\uB4A6"], /* Hangul Syllable */
  ["\uB4A7", "\uB4A7"], /* Hangul Syllable */
  ["\uB4A8", "\uB4A8"], /* Hangul Syllable */
  ["\uB4A9", "\uB4A9"], /* Hangul Syllable */
  ["\uB4AA", "\uB4AA"], /* Hangul Syllable */
  ["\uB4AB", "\uB4AB"], /* Hangul Syllable */
  ["\uB4AC", "\uB4AC"], /* Hangul Syllable */
  ["\uB4AD", "\uB4AD"], /* Hangul Syllable */
  ["\uB4AE", "\uB4AE"], /* Hangul Syllable */
  ["\uB4AF", "\uB4AF"], /* Hangul Syllable */
  ["\uB4B0", "\uB4B0"], /* Hangul Syllable */
  ["\uB4B1", "\uB4B1"], /* Hangul Syllable */
  ["\uB4B2", "\uB4B2"], /* Hangul Syllable */
  ["\uB4B3", "\uB4B3"], /* Hangul Syllable */
  ["\uB4B4", "\uB4B4"], /* Hangul Syllable */
  ["\uB4B5", "\uB4B5"], /* Hangul Syllable */
  ["\uB4B6", "\uB4B6"], /* Hangul Syllable */
  ["\uB4B7", "\uB4B7"], /* Hangul Syllable */
  ["\uB4B8", "\uB4B8"], /* Hangul Syllable */
  ["\uB4B9", "\uB4B9"], /* Hangul Syllable */
  ["\uB4BA", "\uB4BA"], /* Hangul Syllable */
  ["\uB4BB", "\uB4BB"], /* Hangul Syllable */
  ["\uB4BC", "\uB4BC"], /* Hangul Syllable */
  ["\uB4BD", "\uB4BD"], /* Hangul Syllable */
  ["\uB4BE", "\uB4BE"], /* Hangul Syllable */
  ["\uB4BF", "\uB4BF"], /* Hangul Syllable */
  ["\uB4C0", "\uB4C0"], /* Hangul Syllable */
  ["\uB4C1", "\uB4C1"], /* Hangul Syllable */
  ["\uB4C2", "\uB4C2"], /* Hangul Syllable */
  ["\uB4C3", "\uB4C3"], /* Hangul Syllable */
  ["\uB4C4", "\uB4C4"], /* Hangul Syllable */
  ["\uB4C5", "\uB4C5"], /* Hangul Syllable */
  ["\uB4C6", "\uB4C6"], /* Hangul Syllable */
  ["\uB4C7", "\uB4C7"], /* Hangul Syllable */
  ["\uB4C8", "\uB4C8"], /* Hangul Syllable */
  ["\uB4C9", "\uB4C9"], /* Hangul Syllable */
  ["\uB4CA", "\uB4CA"], /* Hangul Syllable */
  ["\uB4CB", "\uB4CB"], /* Hangul Syllable */
  ["\uB4CC", "\uB4CC"], /* Hangul Syllable */
  ["\uB4CD", "\uB4CD"], /* Hangul Syllable */
  ["\uB4CE", "\uB4CE"], /* Hangul Syllable */
  ["\uB4CF", "\uB4CF"], /* Hangul Syllable */
  ["\uB4D0", "\uB4D0"], /* Hangul Syllable */
  ["\uB4D1", "\uB4D1"], /* Hangul Syllable */
  ["\uB4D2", "\uB4D2"], /* Hangul Syllable */
  ["\uB4D3", "\uB4D3"], /* Hangul Syllable */
  ["\uB4D4", "\uB4D4"], /* Hangul Syllable */
  ["\uB4D5", "\uB4D5"], /* Hangul Syllable */
  ["\uB4D6", "\uB4D6"], /* Hangul Syllable */
  ["\uB4D7", "\uB4D7"], /* Hangul Syllable */
  ["\uB4D8", "\uB4D8"], /* Hangul Syllable */
  ["\uB4D9", "\uB4D9"], /* Hangul Syllable */
  ["\uB4DA", "\uB4DA"], /* Hangul Syllable */
  ["\uB4DB", "\uB4DB"], /* Hangul Syllable */
  ["\uB4DC", "\uB4DC"], /* Hangul Syllable */
  ["\uB4DD", "\uB4DD"], /* Hangul Syllable */
  ["\uB4DE", "\uB4DE"], /* Hangul Syllable */
  ["\uB4DF", "\uB4DF"], /* Hangul Syllable */
  ["\uB4E0", "\uB4E0"], /* Hangul Syllable */
  ["\uB4E1", "\uB4E1"], /* Hangul Syllable */
  ["\uB4E2", "\uB4E2"], /* Hangul Syllable */
  ["\uB4E3", "\uB4E3"], /* Hangul Syllable */
  ["\uB4E4", "\uB4E4"], /* Hangul Syllable */
  ["\uB4E5", "\uB4E5"], /* Hangul Syllable */
  ["\uB4E6", "\uB4E6"], /* Hangul Syllable */
  ["\uB4E7", "\uB4E7"], /* Hangul Syllable */
  ["\uB4E8", "\uB4E8"], /* Hangul Syllable */
  ["\uB4E9", "\uB4E9"], /* Hangul Syllable */
  ["\uB4EA", "\uB4EA"], /* Hangul Syllable */
  ["\uB4EB", "\uB4EB"], /* Hangul Syllable */
  ["\uB4EC", "\uB4EC"], /* Hangul Syllable */
  ["\uB4ED", "\uB4ED"], /* Hangul Syllable */
  ["\uB4EE", "\uB4EE"], /* Hangul Syllable */
  ["\uB4EF", "\uB4EF"], /* Hangul Syllable */
  ["\uB4F0", "\uB4F0"], /* Hangul Syllable */
  ["\uB4F1", "\uB4F1"], /* Hangul Syllable */
  ["\uB4F2", "\uB4F2"], /* Hangul Syllable */
  ["\uB4F3", "\uB4F3"], /* Hangul Syllable */
  ["\uB4F4", "\uB4F4"], /* Hangul Syllable */
  ["\uB4F5", "\uB4F5"], /* Hangul Syllable */
  ["\uB4F6", "\uB4F6"], /* Hangul Syllable */
  ["\uB4F7", "\uB4F7"], /* Hangul Syllable */
  ["\uB4F8", "\uB4F8"], /* Hangul Syllable */
  ["\uB4F9", "\uB4F9"], /* Hangul Syllable */
  ["\uB4FA", "\uB4FA"], /* Hangul Syllable */
  ["\uB4FB", "\uB4FB"], /* Hangul Syllable */
  ["\uB4FC", "\uB4FC"], /* Hangul Syllable */
  ["\uB4FD", "\uB4FD"], /* Hangul Syllable */
  ["\uB4FE", "\uB4FE"], /* Hangul Syllable */
  ["\uB4FF", "\uB4FF"], /* Hangul Syllable */
  ["\uB500", "\uB500"], /* Hangul Syllable */
  ["\uB501", "\uB501"], /* Hangul Syllable */
  ["\uB502", "\uB502"], /* Hangul Syllable */
  ["\uB503", "\uB503"], /* Hangul Syllable */
  ["\uB504", "\uB504"], /* Hangul Syllable */
  ["\uB505", "\uB505"], /* Hangul Syllable */
  ["\uB506", "\uB506"], /* Hangul Syllable */
  ["\uB507", "\uB507"], /* Hangul Syllable */
  ["\uB508", "\uB508"], /* Hangul Syllable */
  ["\uB509", "\uB509"], /* Hangul Syllable */
  ["\uB50A", "\uB50A"], /* Hangul Syllable */
  ["\uB50B", "\uB50B"], /* Hangul Syllable */
  ["\uB50C", "\uB50C"], /* Hangul Syllable */
  ["\uB50D", "\uB50D"], /* Hangul Syllable */
  ["\uB50E", "\uB50E"], /* Hangul Syllable */
  ["\uB50F", "\uB50F"], /* Hangul Syllable */
  ["\uB510", "\uB510"], /* Hangul Syllable */
  ["\uB511", "\uB511"], /* Hangul Syllable */
  ["\uB512", "\uB512"], /* Hangul Syllable */
  ["\uB513", "\uB513"], /* Hangul Syllable */
  ["\uB514", "\uB514"], /* Hangul Syllable */
  ["\uB515", "\uB515"], /* Hangul Syllable */
  ["\uB516", "\uB516"], /* Hangul Syllable */
  ["\uB517", "\uB517"], /* Hangul Syllable */
  ["\uB518", "\uB518"], /* Hangul Syllable */
  ["\uB519", "\uB519"], /* Hangul Syllable */
  ["\uB51A", "\uB51A"], /* Hangul Syllable */
  ["\uB51B", "\uB51B"], /* Hangul Syllable */
  ["\uB51C", "\uB51C"], /* Hangul Syllable */
  ["\uB51D", "\uB51D"], /* Hangul Syllable */
  ["\uB51E", "\uB51E"], /* Hangul Syllable */
  ["\uB51F", "\uB51F"], /* Hangul Syllable */
  ["\uB520", "\uB520"], /* Hangul Syllable */
  ["\uB521", "\uB521"], /* Hangul Syllable */
  ["\uB522", "\uB522"], /* Hangul Syllable */
  ["\uB523", "\uB523"], /* Hangul Syllable */
  ["\uB524", "\uB524"], /* Hangul Syllable */
  ["\uB525", "\uB525"], /* Hangul Syllable */
  ["\uB526", "\uB526"], /* Hangul Syllable */
  ["\uB527", "\uB527"], /* Hangul Syllable */
  ["\uB528", "\uB528"], /* Hangul Syllable */
  ["\uB529", "\uB529"], /* Hangul Syllable */
  ["\uB52A", "\uB52A"], /* Hangul Syllable */
  ["\uB52B", "\uB52B"], /* Hangul Syllable */
  ["\uB52C", "\uB52C"], /* Hangul Syllable */
  ["\uB52D", "\uB52D"], /* Hangul Syllable */
  ["\uB52E", "\uB52E"], /* Hangul Syllable */
  ["\uB52F", "\uB52F"], /* Hangul Syllable */
  ["\uB530", "\uB530"], /* Hangul Syllable */
  ["\uB531", "\uB531"], /* Hangul Syllable */
  ["\uB532", "\uB532"], /* Hangul Syllable */
  ["\uB533", "\uB533"], /* Hangul Syllable */
  ["\uB534", "\uB534"], /* Hangul Syllable */
  ["\uB535", "\uB535"], /* Hangul Syllable */
  ["\uB536", "\uB536"], /* Hangul Syllable */
  ["\uB537", "\uB537"], /* Hangul Syllable */
  ["\uB538", "\uB538"], /* Hangul Syllable */
  ["\uB539", "\uB539"], /* Hangul Syllable */
  ["\uB53A", "\uB53A"], /* Hangul Syllable */
  ["\uB53B", "\uB53B"], /* Hangul Syllable */
  ["\uB53C", "\uB53C"], /* Hangul Syllable */
  ["\uB53D", "\uB53D"], /* Hangul Syllable */
  ["\uB53E", "\uB53E"], /* Hangul Syllable */
  ["\uB53F", "\uB53F"], /* Hangul Syllable */
  ["\uB540", "\uB540"], /* Hangul Syllable */
  ["\uB541", "\uB541"], /* Hangul Syllable */
  ["\uB542", "\uB542"], /* Hangul Syllable */
  ["\uB543", "\uB543"], /* Hangul Syllable */
  ["\uB544", "\uB544"], /* Hangul Syllable */
  ["\uB545", "\uB545"], /* Hangul Syllable */
  ["\uB546", "\uB546"], /* Hangul Syllable */
  ["\uB547", "\uB547"], /* Hangul Syllable */
  ["\uB548", "\uB548"], /* Hangul Syllable */
  ["\uB549", "\uB549"], /* Hangul Syllable */
  ["\uB54A", "\uB54A"], /* Hangul Syllable */
  ["\uB54B", "\uB54B"], /* Hangul Syllable */
  ["\uB54C", "\uB54C"], /* Hangul Syllable */
  ["\uB54D", "\uB54D"], /* Hangul Syllable */
  ["\uB54E", "\uB54E"], /* Hangul Syllable */
  ["\uB54F", "\uB54F"], /* Hangul Syllable */
  ["\uB550", "\uB550"], /* Hangul Syllable */
  ["\uB551", "\uB551"], /* Hangul Syllable */
  ["\uB552", "\uB552"], /* Hangul Syllable */
  ["\uB553", "\uB553"], /* Hangul Syllable */
  ["\uB554", "\uB554"], /* Hangul Syllable */
  ["\uB555", "\uB555"], /* Hangul Syllable */
  ["\uB556", "\uB556"], /* Hangul Syllable */
  ["\uB557", "\uB557"], /* Hangul Syllable */
  ["\uB558", "\uB558"], /* Hangul Syllable */
  ["\uB559", "\uB559"], /* Hangul Syllable */
  ["\uB55A", "\uB55A"], /* Hangul Syllable */
  ["\uB55B", "\uB55B"], /* Hangul Syllable */
  ["\uB55C", "\uB55C"], /* Hangul Syllable */
  ["\uB55D", "\uB55D"], /* Hangul Syllable */
  ["\uB55E", "\uB55E"], /* Hangul Syllable */
  ["\uB55F", "\uB55F"], /* Hangul Syllable */
  ["\uB560", "\uB560"], /* Hangul Syllable */
  ["\uB561", "\uB561"], /* Hangul Syllable */
  ["\uB562", "\uB562"], /* Hangul Syllable */
  ["\uB563", "\uB563"], /* Hangul Syllable */
  ["\uB564", "\uB564"], /* Hangul Syllable */
  ["\uB565", "\uB565"], /* Hangul Syllable */
  ["\uB566", "\uB566"], /* Hangul Syllable */
  ["\uB567", "\uB567"], /* Hangul Syllable */
  ["\uB568", "\uB568"], /* Hangul Syllable */
  ["\uB569", "\uB569"], /* Hangul Syllable */
  ["\uB56A", "\uB56A"], /* Hangul Syllable */
  ["\uB56B", "\uB56B"], /* Hangul Syllable */
  ["\uB56C", "\uB56C"], /* Hangul Syllable */
  ["\uB56D", "\uB56D"], /* Hangul Syllable */
  ["\uB56E", "\uB56E"], /* Hangul Syllable */
  ["\uB56F", "\uB56F"], /* Hangul Syllable */
  ["\uB570", "\uB570"], /* Hangul Syllable */
  ["\uB571", "\uB571"], /* Hangul Syllable */
  ["\uB572", "\uB572"], /* Hangul Syllable */
  ["\uB573", "\uB573"], /* Hangul Syllable */
  ["\uB574", "\uB574"], /* Hangul Syllable */
  ["\uB575", "\uB575"], /* Hangul Syllable */
  ["\uB576", "\uB576"], /* Hangul Syllable */
  ["\uB577", "\uB577"], /* Hangul Syllable */
  ["\uB578", "\uB578"], /* Hangul Syllable */
  ["\uB579", "\uB579"], /* Hangul Syllable */
  ["\uB57A", "\uB57A"], /* Hangul Syllable */
  ["\uB57B", "\uB57B"], /* Hangul Syllable */
  ["\uB57C", "\uB57C"], /* Hangul Syllable */
  ["\uB57D", "\uB57D"], /* Hangul Syllable */
  ["\uB57E", "\uB57E"], /* Hangul Syllable */
  ["\uB57F", "\uB57F"], /* Hangul Syllable */
  ["\uB580", "\uB580"], /* Hangul Syllable */
  ["\uB581", "\uB581"], /* Hangul Syllable */
  ["\uB582", "\uB582"], /* Hangul Syllable */
  ["\uB583", "\uB583"], /* Hangul Syllable */
  ["\uB584", "\uB584"], /* Hangul Syllable */
  ["\uB585", "\uB585"], /* Hangul Syllable */
  ["\uB586", "\uB586"], /* Hangul Syllable */
  ["\uB587", "\uB587"], /* Hangul Syllable */
  ["\uB588", "\uB588"], /* Hangul Syllable */
  ["\uB589", "\uB589"], /* Hangul Syllable */
  ["\uB58A", "\uB58A"], /* Hangul Syllable */
  ["\uB58B", "\uB58B"], /* Hangul Syllable */
  ["\uB58C", "\uB58C"], /* Hangul Syllable */
  ["\uB58D", "\uB58D"], /* Hangul Syllable */
  ["\uB58E", "\uB58E"], /* Hangul Syllable */
  ["\uB58F", "\uB58F"], /* Hangul Syllable */
  ["\uB590", "\uB590"], /* Hangul Syllable */
  ["\uB591", "\uB591"], /* Hangul Syllable */
  ["\uB592", "\uB592"], /* Hangul Syllable */
  ["\uB593", "\uB593"], /* Hangul Syllable */
  ["\uB594", "\uB594"], /* Hangul Syllable */
  ["\uB595", "\uB595"], /* Hangul Syllable */
  ["\uB596", "\uB596"], /* Hangul Syllable */
  ["\uB597", "\uB597"], /* Hangul Syllable */
  ["\uB598", "\uB598"], /* Hangul Syllable */
  ["\uB599", "\uB599"], /* Hangul Syllable */
  ["\uB59A", "\uB59A"], /* Hangul Syllable */
  ["\uB59B", "\uB59B"], /* Hangul Syllable */
  ["\uB59C", "\uB59C"], /* Hangul Syllable */
  ["\uB59D", "\uB59D"], /* Hangul Syllable */
  ["\uB59E", "\uB59E"], /* Hangul Syllable */
  ["\uB59F", "\uB59F"], /* Hangul Syllable */
  ["\uB5A0", "\uB5A0"], /* Hangul Syllable */
  ["\uB5A1", "\uB5A1"], /* Hangul Syllable */
  ["\uB5A2", "\uB5A2"], /* Hangul Syllable */
  ["\uB5A3", "\uB5A3"], /* Hangul Syllable */
  ["\uB5A4", "\uB5A4"], /* Hangul Syllable */
  ["\uB5A5", "\uB5A5"], /* Hangul Syllable */
  ["\uB5A6", "\uB5A6"], /* Hangul Syllable */
  ["\uB5A7", "\uB5A7"], /* Hangul Syllable */
  ["\uB5A8", "\uB5A8"], /* Hangul Syllable */
  ["\uB5A9", "\uB5A9"], /* Hangul Syllable */
  ["\uB5AA", "\uB5AA"], /* Hangul Syllable */
  ["\uB5AB", "\uB5AB"], /* Hangul Syllable */
  ["\uB5AC", "\uB5AC"], /* Hangul Syllable */
  ["\uB5AD", "\uB5AD"], /* Hangul Syllable */
  ["\uB5AE", "\uB5AE"], /* Hangul Syllable */
  ["\uB5AF", "\uB5AF"], /* Hangul Syllable */
  ["\uB5B0", "\uB5B0"], /* Hangul Syllable */
  ["\uB5B1", "\uB5B1"], /* Hangul Syllable */
  ["\uB5B2", "\uB5B2"], /* Hangul Syllable */
  ["\uB5B3", "\uB5B3"], /* Hangul Syllable */
  ["\uB5B4", "\uB5B4"], /* Hangul Syllable */
  ["\uB5B5", "\uB5B5"], /* Hangul Syllable */
  ["\uB5B6", "\uB5B6"], /* Hangul Syllable */
  ["\uB5B7", "\uB5B7"], /* Hangul Syllable */
  ["\uB5B8", "\uB5B8"], /* Hangul Syllable */
  ["\uB5B9", "\uB5B9"], /* Hangul Syllable */
  ["\uB5BA", "\uB5BA"], /* Hangul Syllable */
  ["\uB5BB", "\uB5BB"], /* Hangul Syllable */
  ["\uB5BC", "\uB5BC"], /* Hangul Syllable */
  ["\uB5BD", "\uB5BD"], /* Hangul Syllable */
  ["\uB5BE", "\uB5BE"], /* Hangul Syllable */
  ["\uB5BF", "\uB5BF"], /* Hangul Syllable */
  ["\uB5C0", "\uB5C0"], /* Hangul Syllable */
  ["\uB5C1", "\uB5C1"], /* Hangul Syllable */
  ["\uB5C2", "\uB5C2"], /* Hangul Syllable */
  ["\uB5C3", "\uB5C3"], /* Hangul Syllable */
  ["\uB5C4", "\uB5C4"], /* Hangul Syllable */
  ["\uB5C5", "\uB5C5"], /* Hangul Syllable */
  ["\uB5C6", "\uB5C6"], /* Hangul Syllable */
  ["\uB5C7", "\uB5C7"], /* Hangul Syllable */
  ["\uB5C8", "\uB5C8"], /* Hangul Syllable */
  ["\uB5C9", "\uB5C9"], /* Hangul Syllable */
  ["\uB5CA", "\uB5CA"], /* Hangul Syllable */
  ["\uB5CB", "\uB5CB"], /* Hangul Syllable */
  ["\uB5CC", "\uB5CC"], /* Hangul Syllable */
  ["\uB5CD", "\uB5CD"], /* Hangul Syllable */
  ["\uB5CE", "\uB5CE"], /* Hangul Syllable */
  ["\uB5CF", "\uB5CF"], /* Hangul Syllable */
  ["\uB5D0", "\uB5D0"], /* Hangul Syllable */
  ["\uB5D1", "\uB5D1"], /* Hangul Syllable */
  ["\uB5D2", "\uB5D2"], /* Hangul Syllable */
  ["\uB5D3", "\uB5D3"], /* Hangul Syllable */
  ["\uB5D4", "\uB5D4"], /* Hangul Syllable */
  ["\uB5D5", "\uB5D5"], /* Hangul Syllable */
  ["\uB5D6", "\uB5D6"], /* Hangul Syllable */
  ["\uB5D7", "\uB5D7"], /* Hangul Syllable */
  ["\uB5D8", "\uB5D8"], /* Hangul Syllable */
  ["\uB5D9", "\uB5D9"], /* Hangul Syllable */
  ["\uB5DA", "\uB5DA"], /* Hangul Syllable */
  ["\uB5DB", "\uB5DB"], /* Hangul Syllable */
  ["\uB5DC", "\uB5DC"], /* Hangul Syllable */
  ["\uB5DD", "\uB5DD"], /* Hangul Syllable */
  ["\uB5DE", "\uB5DE"], /* Hangul Syllable */
  ["\uB5DF", "\uB5DF"], /* Hangul Syllable */
  ["\uB5E0", "\uB5E0"], /* Hangul Syllable */
  ["\uB5E1", "\uB5E1"], /* Hangul Syllable */
  ["\uB5E2", "\uB5E2"], /* Hangul Syllable */
  ["\uB5E3", "\uB5E3"], /* Hangul Syllable */
  ["\uB5E4", "\uB5E4"], /* Hangul Syllable */
  ["\uB5E5", "\uB5E5"], /* Hangul Syllable */
  ["\uB5E6", "\uB5E6"], /* Hangul Syllable */
  ["\uB5E7", "\uB5E7"], /* Hangul Syllable */
  ["\uB5E8", "\uB5E8"], /* Hangul Syllable */
  ["\uB5E9", "\uB5E9"], /* Hangul Syllable */
  ["\uB5EA", "\uB5EA"], /* Hangul Syllable */
  ["\uB5EB", "\uB5EB"], /* Hangul Syllable */
  ["\uB5EC", "\uB5EC"], /* Hangul Syllable */
  ["\uB5ED", "\uB5ED"], /* Hangul Syllable */
  ["\uB5EE", "\uB5EE"], /* Hangul Syllable */
  ["\uB5EF", "\uB5EF"], /* Hangul Syllable */
  ["\uB5F0", "\uB5F0"], /* Hangul Syllable */
  ["\uB5F1", "\uB5F1"], /* Hangul Syllable */
  ["\uB5F2", "\uB5F2"], /* Hangul Syllable */
  ["\uB5F3", "\uB5F3"], /* Hangul Syllable */
  ["\uB5F4", "\uB5F4"], /* Hangul Syllable */
  ["\uB5F5", "\uB5F5"], /* Hangul Syllable */
  ["\uB5F6", "\uB5F6"], /* Hangul Syllable */
  ["\uB5F7", "\uB5F7"], /* Hangul Syllable */
  ["\uB5F8", "\uB5F8"], /* Hangul Syllable */
  ["\uB5F9", "\uB5F9"], /* Hangul Syllable */
  ["\uB5FA", "\uB5FA"], /* Hangul Syllable */
  ["\uB5FB", "\uB5FB"], /* Hangul Syllable */
  ["\uB5FC", "\uB5FC"], /* Hangul Syllable */
  ["\uB5FD", "\uB5FD"], /* Hangul Syllable */
  ["\uB5FE", "\uB5FE"], /* Hangul Syllable */
  ["\uB5FF", "\uB5FF"], /* Hangul Syllable */
  ["\uB600", "\uB600"], /* Hangul Syllable */
  ["\uB601", "\uB601"], /* Hangul Syllable */
  ["\uB602", "\uB602"], /* Hangul Syllable */
  ["\uB603", "\uB603"], /* Hangul Syllable */
  ["\uB604", "\uB604"], /* Hangul Syllable */
  ["\uB605", "\uB605"], /* Hangul Syllable */
  ["\uB606", "\uB606"], /* Hangul Syllable */
  ["\uB607", "\uB607"], /* Hangul Syllable */
  ["\uB608", "\uB608"], /* Hangul Syllable */
  ["\uB609", "\uB609"], /* Hangul Syllable */
  ["\uB60A", "\uB60A"], /* Hangul Syllable */
  ["\uB60B", "\uB60B"], /* Hangul Syllable */
  ["\uB60C", "\uB60C"], /* Hangul Syllable */
  ["\uB60D", "\uB60D"], /* Hangul Syllable */
  ["\uB60E", "\uB60E"], /* Hangul Syllable */
  ["\uB60F", "\uB60F"], /* Hangul Syllable */
  ["\uB610", "\uB610"], /* Hangul Syllable */
  ["\uB611", "\uB611"], /* Hangul Syllable */
  ["\uB612", "\uB612"], /* Hangul Syllable */
  ["\uB613", "\uB613"], /* Hangul Syllable */
  ["\uB614", "\uB614"], /* Hangul Syllable */
  ["\uB615", "\uB615"], /* Hangul Syllable */
  ["\uB616", "\uB616"], /* Hangul Syllable */
  ["\uB617", "\uB617"], /* Hangul Syllable */
  ["\uB618", "\uB618"], /* Hangul Syllable */
  ["\uB619", "\uB619"], /* Hangul Syllable */
  ["\uB61A", "\uB61A"], /* Hangul Syllable */
  ["\uB61B", "\uB61B"], /* Hangul Syllable */
  ["\uB61C", "\uB61C"], /* Hangul Syllable */
  ["\uB61D", "\uB61D"], /* Hangul Syllable */
  ["\uB61E", "\uB61E"], /* Hangul Syllable */
  ["\uB61F", "\uB61F"], /* Hangul Syllable */
  ["\uB620", "\uB620"], /* Hangul Syllable */
  ["\uB621", "\uB621"], /* Hangul Syllable */
  ["\uB622", "\uB622"], /* Hangul Syllable */
  ["\uB623", "\uB623"], /* Hangul Syllable */
  ["\uB624", "\uB624"], /* Hangul Syllable */
  ["\uB625", "\uB625"], /* Hangul Syllable */
  ["\uB626", "\uB626"], /* Hangul Syllable */
  ["\uB627", "\uB627"], /* Hangul Syllable */
  ["\uB628", "\uB628"], /* Hangul Syllable */
  ["\uB629", "\uB629"], /* Hangul Syllable */
  ["\uB62A", "\uB62A"], /* Hangul Syllable */
  ["\uB62B", "\uB62B"], /* Hangul Syllable */
  ["\uB62C", "\uB62C"], /* Hangul Syllable */
  ["\uB62D", "\uB62D"], /* Hangul Syllable */
  ["\uB62E", "\uB62E"], /* Hangul Syllable */
  ["\uB62F", "\uB62F"], /* Hangul Syllable */
  ["\uB630", "\uB630"], /* Hangul Syllable */
  ["\uB631", "\uB631"], /* Hangul Syllable */
  ["\uB632", "\uB632"], /* Hangul Syllable */
  ["\uB633", "\uB633"], /* Hangul Syllable */
  ["\uB634", "\uB634"], /* Hangul Syllable */
  ["\uB635", "\uB635"], /* Hangul Syllable */
  ["\uB636", "\uB636"], /* Hangul Syllable */
  ["\uB637", "\uB637"], /* Hangul Syllable */
  ["\uB638", "\uB638"], /* Hangul Syllable */
  ["\uB639", "\uB639"], /* Hangul Syllable */
  ["\uB63A", "\uB63A"], /* Hangul Syllable */
  ["\uB63B", "\uB63B"], /* Hangul Syllable */
  ["\uB63C", "\uB63C"], /* Hangul Syllable */
  ["\uB63D", "\uB63D"], /* Hangul Syllable */
  ["\uB63E", "\uB63E"], /* Hangul Syllable */
  ["\uB63F", "\uB63F"], /* Hangul Syllable */
  ["\uB640", "\uB640"], /* Hangul Syllable */
  ["\uB641", "\uB641"], /* Hangul Syllable */
  ["\uB642", "\uB642"], /* Hangul Syllable */
  ["\uB643", "\uB643"], /* Hangul Syllable */
  ["\uB644", "\uB644"], /* Hangul Syllable */
  ["\uB645", "\uB645"], /* Hangul Syllable */
  ["\uB646", "\uB646"], /* Hangul Syllable */
  ["\uB647", "\uB647"], /* Hangul Syllable */
  ["\uB648", "\uB648"], /* Hangul Syllable */
  ["\uB649", "\uB649"], /* Hangul Syllable */
  ["\uB64A", "\uB64A"], /* Hangul Syllable */
  ["\uB64B", "\uB64B"], /* Hangul Syllable */
  ["\uB64C", "\uB64C"], /* Hangul Syllable */
  ["\uB64D", "\uB64D"], /* Hangul Syllable */
  ["\uB64E", "\uB64E"], /* Hangul Syllable */
  ["\uB64F", "\uB64F"], /* Hangul Syllable */
  ["\uB650", "\uB650"], /* Hangul Syllable */
  ["\uB651", "\uB651"], /* Hangul Syllable */
  ["\uB652", "\uB652"], /* Hangul Syllable */
  ["\uB653", "\uB653"], /* Hangul Syllable */
  ["\uB654", "\uB654"], /* Hangul Syllable */
  ["\uB655", "\uB655"], /* Hangul Syllable */
  ["\uB656", "\uB656"], /* Hangul Syllable */
  ["\uB657", "\uB657"], /* Hangul Syllable */
  ["\uB658", "\uB658"], /* Hangul Syllable */
  ["\uB659", "\uB659"], /* Hangul Syllable */
  ["\uB65A", "\uB65A"], /* Hangul Syllable */
  ["\uB65B", "\uB65B"], /* Hangul Syllable */
  ["\uB65C", "\uB65C"], /* Hangul Syllable */
  ["\uB65D", "\uB65D"], /* Hangul Syllable */
  ["\uB65E", "\uB65E"], /* Hangul Syllable */
  ["\uB65F", "\uB65F"], /* Hangul Syllable */
  ["\uB660", "\uB660"], /* Hangul Syllable */
  ["\uB661", "\uB661"], /* Hangul Syllable */
  ["\uB662", "\uB662"], /* Hangul Syllable */
  ["\uB663", "\uB663"], /* Hangul Syllable */
  ["\uB664", "\uB664"], /* Hangul Syllable */
  ["\uB665", "\uB665"], /* Hangul Syllable */
  ["\uB666", "\uB666"], /* Hangul Syllable */
  ["\uB667", "\uB667"], /* Hangul Syllable */
  ["\uB668", "\uB668"], /* Hangul Syllable */
  ["\uB669", "\uB669"], /* Hangul Syllable */
  ["\uB66A", "\uB66A"], /* Hangul Syllable */
  ["\uB66B", "\uB66B"], /* Hangul Syllable */
  ["\uB66C", "\uB66C"], /* Hangul Syllable */
  ["\uB66D", "\uB66D"], /* Hangul Syllable */
  ["\uB66E", "\uB66E"], /* Hangul Syllable */
  ["\uB66F", "\uB66F"], /* Hangul Syllable */
  ["\uB670", "\uB670"], /* Hangul Syllable */
  ["\uB671", "\uB671"], /* Hangul Syllable */
  ["\uB672", "\uB672"], /* Hangul Syllable */
  ["\uB673", "\uB673"], /* Hangul Syllable */
  ["\uB674", "\uB674"], /* Hangul Syllable */
  ["\uB675", "\uB675"], /* Hangul Syllable */
  ["\uB676", "\uB676"], /* Hangul Syllable */
  ["\uB677", "\uB677"], /* Hangul Syllable */
  ["\uB678", "\uB678"], /* Hangul Syllable */
  ["\uB679", "\uB679"], /* Hangul Syllable */
  ["\uB67A", "\uB67A"], /* Hangul Syllable */
  ["\uB67B", "\uB67B"], /* Hangul Syllable */
  ["\uB67C", "\uB67C"], /* Hangul Syllable */
  ["\uB67D", "\uB67D"], /* Hangul Syllable */
  ["\uB67E", "\uB67E"], /* Hangul Syllable */
  ["\uB67F", "\uB67F"], /* Hangul Syllable */
  ["\uB680", "\uB680"], /* Hangul Syllable */
  ["\uB681", "\uB681"], /* Hangul Syllable */
  ["\uB682", "\uB682"], /* Hangul Syllable */
  ["\uB683", "\uB683"], /* Hangul Syllable */
  ["\uB684", "\uB684"], /* Hangul Syllable */
  ["\uB685", "\uB685"], /* Hangul Syllable */
  ["\uB686", "\uB686"], /* Hangul Syllable */
  ["\uB687", "\uB687"], /* Hangul Syllable */
  ["\uB688", "\uB688"], /* Hangul Syllable */
  ["\uB689", "\uB689"], /* Hangul Syllable */
  ["\uB68A", "\uB68A"], /* Hangul Syllable */
  ["\uB68B", "\uB68B"], /* Hangul Syllable */
  ["\uB68C", "\uB68C"], /* Hangul Syllable */
  ["\uB68D", "\uB68D"], /* Hangul Syllable */
  ["\uB68E", "\uB68E"], /* Hangul Syllable */
  ["\uB68F", "\uB68F"], /* Hangul Syllable */
  ["\uB690", "\uB690"], /* Hangul Syllable */
  ["\uB691", "\uB691"], /* Hangul Syllable */
  ["\uB692", "\uB692"], /* Hangul Syllable */
  ["\uB693", "\uB693"], /* Hangul Syllable */
  ["\uB694", "\uB694"], /* Hangul Syllable */
  ["\uB695", "\uB695"], /* Hangul Syllable */
  ["\uB696", "\uB696"], /* Hangul Syllable */
  ["\uB697", "\uB697"], /* Hangul Syllable */
  ["\uB698", "\uB698"], /* Hangul Syllable */
  ["\uB699", "\uB699"], /* Hangul Syllable */
  ["\uB69A", "\uB69A"], /* Hangul Syllable */
  ["\uB69B", "\uB69B"], /* Hangul Syllable */
  ["\uB69C", "\uB69C"], /* Hangul Syllable */
  ["\uB69D", "\uB69D"], /* Hangul Syllable */
  ["\uB69E", "\uB69E"], /* Hangul Syllable */
  ["\uB69F", "\uB69F"], /* Hangul Syllable */
  ["\uB6A0", "\uB6A0"], /* Hangul Syllable */
  ["\uB6A1", "\uB6A1"], /* Hangul Syllable */
  ["\uB6A2", "\uB6A2"], /* Hangul Syllable */
  ["\uB6A3", "\uB6A3"], /* Hangul Syllable */
  ["\uB6A4", "\uB6A4"], /* Hangul Syllable */
  ["\uB6A5", "\uB6A5"], /* Hangul Syllable */
  ["\uB6A6", "\uB6A6"], /* Hangul Syllable */
  ["\uB6A7", "\uB6A7"], /* Hangul Syllable */
  ["\uB6A8", "\uB6A8"], /* Hangul Syllable */
  ["\uB6A9", "\uB6A9"], /* Hangul Syllable */
  ["\uB6AA", "\uB6AA"], /* Hangul Syllable */
  ["\uB6AB", "\uB6AB"], /* Hangul Syllable */
  ["\uB6AC", "\uB6AC"], /* Hangul Syllable */
  ["\uB6AD", "\uB6AD"], /* Hangul Syllable */
  ["\uB6AE", "\uB6AE"], /* Hangul Syllable */
  ["\uB6AF", "\uB6AF"], /* Hangul Syllable */
  ["\uB6B0", "\uB6B0"], /* Hangul Syllable */
  ["\uB6B1", "\uB6B1"], /* Hangul Syllable */
  ["\uB6B2", "\uB6B2"], /* Hangul Syllable */
  ["\uB6B3", "\uB6B3"], /* Hangul Syllable */
  ["\uB6B4", "\uB6B4"], /* Hangul Syllable */
  ["\uB6B5", "\uB6B5"], /* Hangul Syllable */
  ["\uB6B6", "\uB6B6"], /* Hangul Syllable */
  ["\uB6B7", "\uB6B7"], /* Hangul Syllable */
  ["\uB6B8", "\uB6B8"], /* Hangul Syllable */
  ["\uB6B9", "\uB6B9"], /* Hangul Syllable */
  ["\uB6BA", "\uB6BA"], /* Hangul Syllable */
  ["\uB6BB", "\uB6BB"], /* Hangul Syllable */
  ["\uB6BC", "\uB6BC"], /* Hangul Syllable */
  ["\uB6BD", "\uB6BD"], /* Hangul Syllable */
  ["\uB6BE", "\uB6BE"], /* Hangul Syllable */
  ["\uB6BF", "\uB6BF"], /* Hangul Syllable */
  ["\uB6C0", "\uB6C0"], /* Hangul Syllable */
  ["\uB6C1", "\uB6C1"], /* Hangul Syllable */
  ["\uB6C2", "\uB6C2"], /* Hangul Syllable */
  ["\uB6C3", "\uB6C3"], /* Hangul Syllable */
  ["\uB6C4", "\uB6C4"], /* Hangul Syllable */
  ["\uB6C5", "\uB6C5"], /* Hangul Syllable */
  ["\uB6C6", "\uB6C6"], /* Hangul Syllable */
  ["\uB6C7", "\uB6C7"], /* Hangul Syllable */
  ["\uB6C8", "\uB6C8"], /* Hangul Syllable */
  ["\uB6C9", "\uB6C9"], /* Hangul Syllable */
  ["\uB6CA", "\uB6CA"], /* Hangul Syllable */
  ["\uB6CB", "\uB6CB"], /* Hangul Syllable */
  ["\uB6CC", "\uB6CC"], /* Hangul Syllable */
  ["\uB6CD", "\uB6CD"], /* Hangul Syllable */
  ["\uB6CE", "\uB6CE"], /* Hangul Syllable */
  ["\uB6CF", "\uB6CF"], /* Hangul Syllable */
  ["\uB6D0", "\uB6D0"], /* Hangul Syllable */
  ["\uB6D1", "\uB6D1"], /* Hangul Syllable */
  ["\uB6D2", "\uB6D2"], /* Hangul Syllable */
  ["\uB6D3", "\uB6D3"], /* Hangul Syllable */
  ["\uB6D4", "\uB6D4"], /* Hangul Syllable */
  ["\uB6D5", "\uB6D5"], /* Hangul Syllable */
  ["\uB6D6", "\uB6D6"], /* Hangul Syllable */
  ["\uB6D7", "\uB6D7"], /* Hangul Syllable */
  ["\uB6D8", "\uB6D8"], /* Hangul Syllable */
  ["\uB6D9", "\uB6D9"], /* Hangul Syllable */
  ["\uB6DA", "\uB6DA"], /* Hangul Syllable */
  ["\uB6DB", "\uB6DB"], /* Hangul Syllable */
  ["\uB6DC", "\uB6DC"], /* Hangul Syllable */
  ["\uB6DD", "\uB6DD"], /* Hangul Syllable */
  ["\uB6DE", "\uB6DE"], /* Hangul Syllable */
  ["\uB6DF", "\uB6DF"], /* Hangul Syllable */
  ["\uB6E0", "\uB6E0"], /* Hangul Syllable */
  ["\uB6E1", "\uB6E1"], /* Hangul Syllable */
  ["\uB6E2", "\uB6E2"], /* Hangul Syllable */
  ["\uB6E3", "\uB6E3"], /* Hangul Syllable */
  ["\uB6E4", "\uB6E4"], /* Hangul Syllable */
  ["\uB6E5", "\uB6E5"], /* Hangul Syllable */
  ["\uB6E6", "\uB6E6"], /* Hangul Syllable */
  ["\uB6E7", "\uB6E7"], /* Hangul Syllable */
  ["\uB6E8", "\uB6E8"], /* Hangul Syllable */
  ["\uB6E9", "\uB6E9"], /* Hangul Syllable */
  ["\uB6EA", "\uB6EA"], /* Hangul Syllable */
  ["\uB6EB", "\uB6EB"], /* Hangul Syllable */
  ["\uB6EC", "\uB6EC"], /* Hangul Syllable */
  ["\uB6ED", "\uB6ED"], /* Hangul Syllable */
  ["\uB6EE", "\uB6EE"], /* Hangul Syllable */
  ["\uB6EF", "\uB6EF"], /* Hangul Syllable */
  ["\uB6F0", "\uB6F0"], /* Hangul Syllable */
  ["\uB6F1", "\uB6F1"], /* Hangul Syllable */
  ["\uB6F2", "\uB6F2"], /* Hangul Syllable */
  ["\uB6F3", "\uB6F3"], /* Hangul Syllable */
  ["\uB6F4", "\uB6F4"], /* Hangul Syllable */
  ["\uB6F5", "\uB6F5"], /* Hangul Syllable */
  ["\uB6F6", "\uB6F6"], /* Hangul Syllable */
  ["\uB6F7", "\uB6F7"], /* Hangul Syllable */
  ["\uB6F8", "\uB6F8"], /* Hangul Syllable */
  ["\uB6F9", "\uB6F9"], /* Hangul Syllable */
  ["\uB6FA", "\uB6FA"], /* Hangul Syllable */
  ["\uB6FB", "\uB6FB"], /* Hangul Syllable */
  ["\uB6FC", "\uB6FC"], /* Hangul Syllable */
  ["\uB6FD", "\uB6FD"], /* Hangul Syllable */
  ["\uB6FE", "\uB6FE"], /* Hangul Syllable */
  ["\uB6FF", "\uB6FF"], /* Hangul Syllable */
  ["\uB700", "\uB700"], /* Hangul Syllable */
  ["\uB701", "\uB701"], /* Hangul Syllable */
  ["\uB702", "\uB702"], /* Hangul Syllable */
  ["\uB703", "\uB703"], /* Hangul Syllable */
  ["\uB704", "\uB704"], /* Hangul Syllable */
  ["\uB705", "\uB705"], /* Hangul Syllable */
  ["\uB706", "\uB706"], /* Hangul Syllable */
  ["\uB707", "\uB707"], /* Hangul Syllable */
  ["\uB708", "\uB708"], /* Hangul Syllable */
  ["\uB709", "\uB709"], /* Hangul Syllable */
  ["\uB70A", "\uB70A"], /* Hangul Syllable */
  ["\uB70B", "\uB70B"], /* Hangul Syllable */
  ["\uB70C", "\uB70C"], /* Hangul Syllable */
  ["\uB70D", "\uB70D"], /* Hangul Syllable */
  ["\uB70E", "\uB70E"], /* Hangul Syllable */
  ["\uB70F", "\uB70F"], /* Hangul Syllable */
  ["\uB710", "\uB710"], /* Hangul Syllable */
  ["\uB711", "\uB711"], /* Hangul Syllable */
  ["\uB712", "\uB712"], /* Hangul Syllable */
  ["\uB713", "\uB713"], /* Hangul Syllable */
  ["\uB714", "\uB714"], /* Hangul Syllable */
  ["\uB715", "\uB715"], /* Hangul Syllable */
  ["\uB716", "\uB716"], /* Hangul Syllable */
  ["\uB717", "\uB717"], /* Hangul Syllable */
  ["\uB718", "\uB718"], /* Hangul Syllable */
  ["\uB719", "\uB719"], /* Hangul Syllable */
  ["\uB71A", "\uB71A"], /* Hangul Syllable */
  ["\uB71B", "\uB71B"], /* Hangul Syllable */
  ["\uB71C", "\uB71C"], /* Hangul Syllable */
  ["\uB71D", "\uB71D"], /* Hangul Syllable */
  ["\uB71E", "\uB71E"], /* Hangul Syllable */
  ["\uB71F", "\uB71F"], /* Hangul Syllable */
  ["\uB720", "\uB720"], /* Hangul Syllable */
  ["\uB721", "\uB721"], /* Hangul Syllable */
  ["\uB722", "\uB722"], /* Hangul Syllable */
  ["\uB723", "\uB723"], /* Hangul Syllable */
  ["\uB724", "\uB724"], /* Hangul Syllable */
  ["\uB725", "\uB725"], /* Hangul Syllable */
  ["\uB726", "\uB726"], /* Hangul Syllable */
  ["\uB727", "\uB727"], /* Hangul Syllable */
  ["\uB728", "\uB728"], /* Hangul Syllable */
  ["\uB729", "\uB729"], /* Hangul Syllable */
  ["\uB72A", "\uB72A"], /* Hangul Syllable */
  ["\uB72B", "\uB72B"], /* Hangul Syllable */
  ["\uB72C", "\uB72C"], /* Hangul Syllable */
  ["\uB72D", "\uB72D"], /* Hangul Syllable */
  ["\uB72E", "\uB72E"], /* Hangul Syllable */
  ["\uB72F", "\uB72F"], /* Hangul Syllable */
  ["\uB730", "\uB730"], /* Hangul Syllable */
  ["\uB731", "\uB731"], /* Hangul Syllable */
  ["\uB732", "\uB732"], /* Hangul Syllable */
  ["\uB733", "\uB733"], /* Hangul Syllable */
  ["\uB734", "\uB734"], /* Hangul Syllable */
  ["\uB735", "\uB735"], /* Hangul Syllable */
  ["\uB736", "\uB736"], /* Hangul Syllable */
  ["\uB737", "\uB737"], /* Hangul Syllable */
  ["\uB738", "\uB738"], /* Hangul Syllable */
  ["\uB739", "\uB739"], /* Hangul Syllable */
  ["\uB73A", "\uB73A"], /* Hangul Syllable */
  ["\uB73B", "\uB73B"], /* Hangul Syllable */
  ["\uB73C", "\uB73C"], /* Hangul Syllable */
  ["\uB73D", "\uB73D"], /* Hangul Syllable */
  ["\uB73E", "\uB73E"], /* Hangul Syllable */
  ["\uB73F", "\uB73F"], /* Hangul Syllable */
  ["\uB740", "\uB740"], /* Hangul Syllable */
  ["\uB741", "\uB741"], /* Hangul Syllable */
  ["\uB742", "\uB742"], /* Hangul Syllable */
  ["\uB743", "\uB743"], /* Hangul Syllable */
  ["\uB744", "\uB744"], /* Hangul Syllable */
  ["\uB745", "\uB745"], /* Hangul Syllable */
  ["\uB746", "\uB746"], /* Hangul Syllable */
  ["\uB747", "\uB747"], /* Hangul Syllable */
  ["\uB748", "\uB748"], /* Hangul Syllable */
  ["\uB749", "\uB749"], /* Hangul Syllable */
  ["\uB74A", "\uB74A"], /* Hangul Syllable */
  ["\uB74B", "\uB74B"], /* Hangul Syllable */
  ["\uB74C", "\uB74C"], /* Hangul Syllable */
  ["\uB74D", "\uB74D"], /* Hangul Syllable */
  ["\uB74E", "\uB74E"], /* Hangul Syllable */
  ["\uB74F", "\uB74F"], /* Hangul Syllable */
  ["\uB750", "\uB750"], /* Hangul Syllable */
  ["\uB751", "\uB751"], /* Hangul Syllable */
  ["\uB752", "\uB752"], /* Hangul Syllable */
  ["\uB753", "\uB753"], /* Hangul Syllable */
  ["\uB754", "\uB754"], /* Hangul Syllable */
  ["\uB755", "\uB755"], /* Hangul Syllable */
  ["\uB756", "\uB756"], /* Hangul Syllable */
  ["\uB757", "\uB757"], /* Hangul Syllable */
  ["\uB758", "\uB758"], /* Hangul Syllable */
  ["\uB759", "\uB759"], /* Hangul Syllable */
  ["\uB75A", "\uB75A"], /* Hangul Syllable */
  ["\uB75B", "\uB75B"], /* Hangul Syllable */
  ["\uB75C", "\uB75C"], /* Hangul Syllable */
  ["\uB75D", "\uB75D"], /* Hangul Syllable */
  ["\uB75E", "\uB75E"], /* Hangul Syllable */
  ["\uB75F", "\uB75F"], /* Hangul Syllable */
  ["\uB760", "\uB760"], /* Hangul Syllable */
  ["\uB761", "\uB761"], /* Hangul Syllable */
  ["\uB762", "\uB762"], /* Hangul Syllable */
  ["\uB763", "\uB763"], /* Hangul Syllable */
  ["\uB764", "\uB764"], /* Hangul Syllable */
  ["\uB765", "\uB765"], /* Hangul Syllable */
  ["\uB766", "\uB766"], /* Hangul Syllable */
  ["\uB767", "\uB767"], /* Hangul Syllable */
  ["\uB768", "\uB768"], /* Hangul Syllable */
  ["\uB769", "\uB769"], /* Hangul Syllable */
  ["\uB76A", "\uB76A"], /* Hangul Syllable */
  ["\uB76B", "\uB76B"], /* Hangul Syllable */
  ["\uB76C", "\uB76C"], /* Hangul Syllable */
  ["\uB76D", "\uB76D"], /* Hangul Syllable */
  ["\uB76E", "\uB76E"], /* Hangul Syllable */
  ["\uB76F", "\uB76F"], /* Hangul Syllable */
  ["\uB770", "\uB770"], /* Hangul Syllable */
  ["\uB771", "\uB771"], /* Hangul Syllable */
  ["\uB772", "\uB772"], /* Hangul Syllable */
  ["\uB773", "\uB773"], /* Hangul Syllable */
  ["\uB774", "\uB774"], /* Hangul Syllable */
  ["\uB775", "\uB775"], /* Hangul Syllable */
  ["\uB776", "\uB776"], /* Hangul Syllable */
  ["\uB777", "\uB777"], /* Hangul Syllable */
  ["\uB778", "\uB778"], /* Hangul Syllable */
  ["\uB779", "\uB779"], /* Hangul Syllable */
  ["\uB77A", "\uB77A"], /* Hangul Syllable */
  ["\uB77B", "\uB77B"], /* Hangul Syllable */
  ["\uB77C", "\uB77C"], /* Hangul Syllable */
  ["\uB77D", "\uB77D"], /* Hangul Syllable */
  ["\uB77E", "\uB77E"], /* Hangul Syllable */
  ["\uB77F", "\uB77F"], /* Hangul Syllable */
  ["\uB780", "\uB780"], /* Hangul Syllable */
  ["\uB781", "\uB781"], /* Hangul Syllable */
  ["\uB782", "\uB782"], /* Hangul Syllable */
  ["\uB783", "\uB783"], /* Hangul Syllable */
  ["\uB784", "\uB784"], /* Hangul Syllable */
  ["\uB785", "\uB785"], /* Hangul Syllable */
  ["\uB786", "\uB786"], /* Hangul Syllable */
  ["\uB787", "\uB787"], /* Hangul Syllable */
  ["\uB788", "\uB788"], /* Hangul Syllable */
  ["\uB789", "\uB789"], /* Hangul Syllable */
  ["\uB78A", "\uB78A"], /* Hangul Syllable */
  ["\uB78B", "\uB78B"], /* Hangul Syllable */
  ["\uB78C", "\uB78C"], /* Hangul Syllable */
  ["\uB78D", "\uB78D"], /* Hangul Syllable */
  ["\uB78E", "\uB78E"], /* Hangul Syllable */
  ["\uB78F", "\uB78F"], /* Hangul Syllable */
  ["\uB790", "\uB790"], /* Hangul Syllable */
  ["\uB791", "\uB791"], /* Hangul Syllable */
  ["\uB792", "\uB792"], /* Hangul Syllable */
  ["\uB793", "\uB793"], /* Hangul Syllable */
  ["\uB794", "\uB794"], /* Hangul Syllable */
  ["\uB795", "\uB795"], /* Hangul Syllable */
  ["\uB796", "\uB796"], /* Hangul Syllable */
  ["\uB797", "\uB797"], /* Hangul Syllable */
  ["\uB798", "\uB798"], /* Hangul Syllable */
  ["\uB799", "\uB799"], /* Hangul Syllable */
  ["\uB79A", "\uB79A"], /* Hangul Syllable */
  ["\uB79B", "\uB79B"], /* Hangul Syllable */
  ["\uB79C", "\uB79C"], /* Hangul Syllable */
  ["\uB79D", "\uB79D"], /* Hangul Syllable */
  ["\uB79E", "\uB79E"], /* Hangul Syllable */
  ["\uB79F", "\uB79F"], /* Hangul Syllable */
  ["\uB7A0", "\uB7A0"], /* Hangul Syllable */
  ["\uB7A1", "\uB7A1"], /* Hangul Syllable */
  ["\uB7A2", "\uB7A2"], /* Hangul Syllable */
  ["\uB7A3", "\uB7A3"], /* Hangul Syllable */
  ["\uB7A4", "\uB7A4"], /* Hangul Syllable */
  ["\uB7A5", "\uB7A5"], /* Hangul Syllable */
  ["\uB7A6", "\uB7A6"], /* Hangul Syllable */
  ["\uB7A7", "\uB7A7"], /* Hangul Syllable */
  ["\uB7A8", "\uB7A8"], /* Hangul Syllable */
  ["\uB7A9", "\uB7A9"], /* Hangul Syllable */
  ["\uB7AA", "\uB7AA"], /* Hangul Syllable */
  ["\uB7AB", "\uB7AB"], /* Hangul Syllable */
  ["\uB7AC", "\uB7AC"], /* Hangul Syllable */
  ["\uB7AD", "\uB7AD"], /* Hangul Syllable */
  ["\uB7AE", "\uB7AE"], /* Hangul Syllable */
  ["\uB7AF", "\uB7AF"], /* Hangul Syllable */
  ["\uB7B0", "\uB7B0"], /* Hangul Syllable */
  ["\uB7B1", "\uB7B1"], /* Hangul Syllable */
  ["\uB7B2", "\uB7B2"], /* Hangul Syllable */
  ["\uB7B3", "\uB7B3"], /* Hangul Syllable */
  ["\uB7B4", "\uB7B4"], /* Hangul Syllable */
  ["\uB7B5", "\uB7B5"], /* Hangul Syllable */
  ["\uB7B6", "\uB7B6"], /* Hangul Syllable */
  ["\uB7B7", "\uB7B7"], /* Hangul Syllable */
  ["\uB7B8", "\uB7B8"], /* Hangul Syllable */
  ["\uB7B9", "\uB7B9"], /* Hangul Syllable */
  ["\uB7BA", "\uB7BA"], /* Hangul Syllable */
  ["\uB7BB", "\uB7BB"], /* Hangul Syllable */
  ["\uB7BC", "\uB7BC"], /* Hangul Syllable */
  ["\uB7BD", "\uB7BD"], /* Hangul Syllable */
  ["\uB7BE", "\uB7BE"], /* Hangul Syllable */
  ["\uB7BF", "\uB7BF"], /* Hangul Syllable */
  ["\uB7C0", "\uB7C0"], /* Hangul Syllable */
  ["\uB7C1", "\uB7C1"], /* Hangul Syllable */
  ["\uB7C2", "\uB7C2"], /* Hangul Syllable */
  ["\uB7C3", "\uB7C3"], /* Hangul Syllable */
  ["\uB7C4", "\uB7C4"], /* Hangul Syllable */
  ["\uB7C5", "\uB7C5"], /* Hangul Syllable */
  ["\uB7C6", "\uB7C6"], /* Hangul Syllable */
  ["\uB7C7", "\uB7C7"], /* Hangul Syllable */
  ["\uB7C8", "\uB7C8"], /* Hangul Syllable */
  ["\uB7C9", "\uB7C9"], /* Hangul Syllable */
  ["\uB7CA", "\uB7CA"], /* Hangul Syllable */
  ["\uB7CB", "\uB7CB"], /* Hangul Syllable */
  ["\uB7CC", "\uB7CC"], /* Hangul Syllable */
  ["\uB7CD", "\uB7CD"], /* Hangul Syllable */
  ["\uB7CE", "\uB7CE"], /* Hangul Syllable */
  ["\uB7CF", "\uB7CF"], /* Hangul Syllable */
  ["\uB7D0", "\uB7D0"], /* Hangul Syllable */
  ["\uB7D1", "\uB7D1"], /* Hangul Syllable */
  ["\uB7D2", "\uB7D2"], /* Hangul Syllable */
  ["\uB7D3", "\uB7D3"], /* Hangul Syllable */
  ["\uB7D4", "\uB7D4"], /* Hangul Syllable */
  ["\uB7D5", "\uB7D5"], /* Hangul Syllable */
  ["\uB7D6", "\uB7D6"], /* Hangul Syllable */
  ["\uB7D7", "\uB7D7"], /* Hangul Syllable */
  ["\uB7D8", "\uB7D8"], /* Hangul Syllable */
  ["\uB7D9", "\uB7D9"], /* Hangul Syllable */
  ["\uB7DA", "\uB7DA"], /* Hangul Syllable */
  ["\uB7DB", "\uB7DB"], /* Hangul Syllable */
  ["\uB7DC", "\uB7DC"], /* Hangul Syllable */
  ["\uB7DD", "\uB7DD"], /* Hangul Syllable */
  ["\uB7DE", "\uB7DE"], /* Hangul Syllable */
  ["\uB7DF", "\uB7DF"], /* Hangul Syllable */
  ["\uB7E0", "\uB7E0"], /* Hangul Syllable */
  ["\uB7E1", "\uB7E1"], /* Hangul Syllable */
  ["\uB7E2", "\uB7E2"], /* Hangul Syllable */
  ["\uB7E3", "\uB7E3"], /* Hangul Syllable */
  ["\uB7E4", "\uB7E4"], /* Hangul Syllable */
  ["\uB7E5", "\uB7E5"], /* Hangul Syllable */
  ["\uB7E6", "\uB7E6"], /* Hangul Syllable */
  ["\uB7E7", "\uB7E7"], /* Hangul Syllable */
  ["\uB7E8", "\uB7E8"], /* Hangul Syllable */
  ["\uB7E9", "\uB7E9"], /* Hangul Syllable */
  ["\uB7EA", "\uB7EA"], /* Hangul Syllable */
  ["\uB7EB", "\uB7EB"], /* Hangul Syllable */
  ["\uB7EC", "\uB7EC"], /* Hangul Syllable */
  ["\uB7ED", "\uB7ED"], /* Hangul Syllable */
  ["\uB7EE", "\uB7EE"], /* Hangul Syllable */
  ["\uB7EF", "\uB7EF"], /* Hangul Syllable */
  ["\uB7F0", "\uB7F0"], /* Hangul Syllable */
  ["\uB7F1", "\uB7F1"], /* Hangul Syllable */
  ["\uB7F2", "\uB7F2"], /* Hangul Syllable */
  ["\uB7F3", "\uB7F3"], /* Hangul Syllable */
  ["\uB7F4", "\uB7F4"], /* Hangul Syllable */
  ["\uB7F5", "\uB7F5"], /* Hangul Syllable */
  ["\uB7F6", "\uB7F6"], /* Hangul Syllable */
  ["\uB7F7", "\uB7F7"], /* Hangul Syllable */
  ["\uB7F8", "\uB7F8"], /* Hangul Syllable */
  ["\uB7F9", "\uB7F9"], /* Hangul Syllable */
  ["\uB7FA", "\uB7FA"], /* Hangul Syllable */
  ["\uB7FB", "\uB7FB"], /* Hangul Syllable */
  ["\uB7FC", "\uB7FC"], /* Hangul Syllable */
  ["\uB7FD", "\uB7FD"], /* Hangul Syllable */
  ["\uB7FE", "\uB7FE"], /* Hangul Syllable */
  ["\uB7FF", "\uB7FF"], /* Hangul Syllable */
  ["\uB800", "\uB800"], /* Hangul Syllable */
  ["\uB801", "\uB801"], /* Hangul Syllable */
  ["\uB802", "\uB802"], /* Hangul Syllable */
  ["\uB803", "\uB803"], /* Hangul Syllable */
  ["\uB804", "\uB804"], /* Hangul Syllable */
  ["\uB805", "\uB805"], /* Hangul Syllable */
  ["\uB806", "\uB806"], /* Hangul Syllable */
  ["\uB807", "\uB807"], /* Hangul Syllable */
  ["\uB808", "\uB808"], /* Hangul Syllable */
  ["\uB809", "\uB809"], /* Hangul Syllable */
  ["\uB80A", "\uB80A"], /* Hangul Syllable */
  ["\uB80B", "\uB80B"], /* Hangul Syllable */
  ["\uB80C", "\uB80C"], /* Hangul Syllable */
  ["\uB80D", "\uB80D"], /* Hangul Syllable */
  ["\uB80E", "\uB80E"], /* Hangul Syllable */
  ["\uB80F", "\uB80F"], /* Hangul Syllable */
  ["\uB810", "\uB810"], /* Hangul Syllable */
  ["\uB811", "\uB811"], /* Hangul Syllable */
  ["\uB812", "\uB812"], /* Hangul Syllable */
  ["\uB813", "\uB813"], /* Hangul Syllable */
  ["\uB814", "\uB814"], /* Hangul Syllable */
  ["\uB815", "\uB815"], /* Hangul Syllable */
  ["\uB816", "\uB816"], /* Hangul Syllable */
  ["\uB817", "\uB817"], /* Hangul Syllable */
  ["\uB818", "\uB818"], /* Hangul Syllable */
  ["\uB819", "\uB819"], /* Hangul Syllable */
  ["\uB81A", "\uB81A"], /* Hangul Syllable */
  ["\uB81B", "\uB81B"], /* Hangul Syllable */
  ["\uB81C", "\uB81C"], /* Hangul Syllable */
  ["\uB81D", "\uB81D"], /* Hangul Syllable */
  ["\uB81E", "\uB81E"], /* Hangul Syllable */
  ["\uB81F", "\uB81F"], /* Hangul Syllable */
  ["\uB820", "\uB820"], /* Hangul Syllable */
  ["\uB821", "\uB821"], /* Hangul Syllable */
  ["\uB822", "\uB822"], /* Hangul Syllable */
  ["\uB823", "\uB823"], /* Hangul Syllable */
  ["\uB824", "\uB824"], /* Hangul Syllable */
  ["\uB825", "\uB825"], /* Hangul Syllable */
  ["\uB826", "\uB826"], /* Hangul Syllable */
  ["\uB827", "\uB827"], /* Hangul Syllable */
  ["\uB828", "\uB828"], /* Hangul Syllable */
  ["\uB829", "\uB829"], /* Hangul Syllable */
  ["\uB82A", "\uB82A"], /* Hangul Syllable */
  ["\uB82B", "\uB82B"], /* Hangul Syllable */
  ["\uB82C", "\uB82C"], /* Hangul Syllable */
  ["\uB82D", "\uB82D"], /* Hangul Syllable */
  ["\uB82E", "\uB82E"], /* Hangul Syllable */
  ["\uB82F", "\uB82F"], /* Hangul Syllable */
  ["\uB830", "\uB830"], /* Hangul Syllable */
  ["\uB831", "\uB831"], /* Hangul Syllable */
  ["\uB832", "\uB832"], /* Hangul Syllable */
  ["\uB833", "\uB833"], /* Hangul Syllable */
  ["\uB834", "\uB834"], /* Hangul Syllable */
  ["\uB835", "\uB835"], /* Hangul Syllable */
  ["\uB836", "\uB836"], /* Hangul Syllable */
  ["\uB837", "\uB837"], /* Hangul Syllable */
  ["\uB838", "\uB838"], /* Hangul Syllable */
  ["\uB839", "\uB839"], /* Hangul Syllable */
  ["\uB83A", "\uB83A"], /* Hangul Syllable */
  ["\uB83B", "\uB83B"], /* Hangul Syllable */
  ["\uB83C", "\uB83C"], /* Hangul Syllable */
  ["\uB83D", "\uB83D"], /* Hangul Syllable */
  ["\uB83E", "\uB83E"], /* Hangul Syllable */
  ["\uB83F", "\uB83F"], /* Hangul Syllable */
  ["\uB840", "\uB840"], /* Hangul Syllable */
  ["\uB841", "\uB841"], /* Hangul Syllable */
  ["\uB842", "\uB842"], /* Hangul Syllable */
  ["\uB843", "\uB843"], /* Hangul Syllable */
  ["\uB844", "\uB844"], /* Hangul Syllable */
  ["\uB845", "\uB845"], /* Hangul Syllable */
  ["\uB846", "\uB846"], /* Hangul Syllable */
  ["\uB847", "\uB847"], /* Hangul Syllable */
  ["\uB848", "\uB848"], /* Hangul Syllable */
  ["\uB849", "\uB849"], /* Hangul Syllable */
  ["\uB84A", "\uB84A"], /* Hangul Syllable */
  ["\uB84B", "\uB84B"], /* Hangul Syllable */
  ["\uB84C", "\uB84C"], /* Hangul Syllable */
  ["\uB84D", "\uB84D"], /* Hangul Syllable */
  ["\uB84E", "\uB84E"], /* Hangul Syllable */
  ["\uB84F", "\uB84F"], /* Hangul Syllable */
  ["\uB850", "\uB850"], /* Hangul Syllable */
  ["\uB851", "\uB851"], /* Hangul Syllable */
  ["\uB852", "\uB852"], /* Hangul Syllable */
  ["\uB853", "\uB853"], /* Hangul Syllable */
  ["\uB854", "\uB854"], /* Hangul Syllable */
  ["\uB855", "\uB855"], /* Hangul Syllable */
  ["\uB856", "\uB856"], /* Hangul Syllable */
  ["\uB857", "\uB857"], /* Hangul Syllable */
  ["\uB858", "\uB858"], /* Hangul Syllable */
  ["\uB859", "\uB859"], /* Hangul Syllable */
  ["\uB85A", "\uB85A"], /* Hangul Syllable */
  ["\uB85B", "\uB85B"], /* Hangul Syllable */
  ["\uB85C", "\uB85C"], /* Hangul Syllable */
  ["\uB85D", "\uB85D"], /* Hangul Syllable */
  ["\uB85E", "\uB85E"], /* Hangul Syllable */
  ["\uB85F", "\uB85F"], /* Hangul Syllable */
  ["\uB860", "\uB860"], /* Hangul Syllable */
  ["\uB861", "\uB861"], /* Hangul Syllable */
  ["\uB862", "\uB862"], /* Hangul Syllable */
  ["\uB863", "\uB863"], /* Hangul Syllable */
  ["\uB864", "\uB864"], /* Hangul Syllable */
  ["\uB865", "\uB865"], /* Hangul Syllable */
  ["\uB866", "\uB866"], /* Hangul Syllable */
  ["\uB867", "\uB867"], /* Hangul Syllable */
  ["\uB868", "\uB868"], /* Hangul Syllable */
  ["\uB869", "\uB869"], /* Hangul Syllable */
  ["\uB86A", "\uB86A"], /* Hangul Syllable */
  ["\uB86B", "\uB86B"], /* Hangul Syllable */
  ["\uB86C", "\uB86C"], /* Hangul Syllable */
  ["\uB86D", "\uB86D"], /* Hangul Syllable */
  ["\uB86E", "\uB86E"], /* Hangul Syllable */
  ["\uB86F", "\uB86F"], /* Hangul Syllable */
  ["\uB870", "\uB870"], /* Hangul Syllable */
  ["\uB871", "\uB871"], /* Hangul Syllable */
  ["\uB872", "\uB872"], /* Hangul Syllable */
  ["\uB873", "\uB873"], /* Hangul Syllable */
  ["\uB874", "\uB874"], /* Hangul Syllable */
  ["\uB875", "\uB875"], /* Hangul Syllable */
  ["\uB876", "\uB876"], /* Hangul Syllable */
  ["\uB877", "\uB877"], /* Hangul Syllable */
  ["\uB878", "\uB878"], /* Hangul Syllable */
  ["\uB879", "\uB879"], /* Hangul Syllable */
  ["\uB87A", "\uB87A"], /* Hangul Syllable */
  ["\uB87B", "\uB87B"], /* Hangul Syllable */
  ["\uB87C", "\uB87C"], /* Hangul Syllable */
  ["\uB87D", "\uB87D"], /* Hangul Syllable */
  ["\uB87E", "\uB87E"], /* Hangul Syllable */
  ["\uB87F", "\uB87F"], /* Hangul Syllable */
  ["\uB880", "\uB880"], /* Hangul Syllable */
  ["\uB881", "\uB881"], /* Hangul Syllable */
  ["\uB882", "\uB882"], /* Hangul Syllable */
  ["\uB883", "\uB883"], /* Hangul Syllable */
  ["\uB884", "\uB884"], /* Hangul Syllable */
  ["\uB885", "\uB885"], /* Hangul Syllable */
  ["\uB886", "\uB886"], /* Hangul Syllable */
  ["\uB887", "\uB887"], /* Hangul Syllable */
  ["\uB888", "\uB888"], /* Hangul Syllable */
  ["\uB889", "\uB889"], /* Hangul Syllable */
  ["\uB88A", "\uB88A"], /* Hangul Syllable */
  ["\uB88B", "\uB88B"], /* Hangul Syllable */
  ["\uB88C", "\uB88C"], /* Hangul Syllable */
  ["\uB88D", "\uB88D"], /* Hangul Syllable */
  ["\uB88E", "\uB88E"], /* Hangul Syllable */
  ["\uB88F", "\uB88F"], /* Hangul Syllable */
  ["\uB890", "\uB890"], /* Hangul Syllable */
  ["\uB891", "\uB891"], /* Hangul Syllable */
  ["\uB892", "\uB892"], /* Hangul Syllable */
  ["\uB893", "\uB893"], /* Hangul Syllable */
  ["\uB894", "\uB894"], /* Hangul Syllable */
  ["\uB895", "\uB895"], /* Hangul Syllable */
  ["\uB896", "\uB896"], /* Hangul Syllable */
  ["\uB897", "\uB897"], /* Hangul Syllable */
  ["\uB898", "\uB898"], /* Hangul Syllable */
  ["\uB899", "\uB899"], /* Hangul Syllable */
  ["\uB89A", "\uB89A"], /* Hangul Syllable */
  ["\uB89B", "\uB89B"], /* Hangul Syllable */
  ["\uB89C", "\uB89C"], /* Hangul Syllable */
  ["\uB89D", "\uB89D"], /* Hangul Syllable */
  ["\uB89E", "\uB89E"], /* Hangul Syllable */
  ["\uB89F", "\uB89F"], /* Hangul Syllable */
  ["\uB8A0", "\uB8A0"], /* Hangul Syllable */
  ["\uB8A1", "\uB8A1"], /* Hangul Syllable */
  ["\uB8A2", "\uB8A2"], /* Hangul Syllable */
  ["\uB8A3", "\uB8A3"], /* Hangul Syllable */
  ["\uB8A4", "\uB8A4"], /* Hangul Syllable */
  ["\uB8A5", "\uB8A5"], /* Hangul Syllable */
  ["\uB8A6", "\uB8A6"], /* Hangul Syllable */
  ["\uB8A7", "\uB8A7"], /* Hangul Syllable */
  ["\uB8A8", "\uB8A8"], /* Hangul Syllable */
  ["\uB8A9", "\uB8A9"], /* Hangul Syllable */
  ["\uB8AA", "\uB8AA"], /* Hangul Syllable */
  ["\uB8AB", "\uB8AB"], /* Hangul Syllable */
  ["\uB8AC", "\uB8AC"], /* Hangul Syllable */
  ["\uB8AD", "\uB8AD"], /* Hangul Syllable */
  ["\uB8AE", "\uB8AE"], /* Hangul Syllable */
  ["\uB8AF", "\uB8AF"], /* Hangul Syllable */
  ["\uB8B0", "\uB8B0"], /* Hangul Syllable */
  ["\uB8B1", "\uB8B1"], /* Hangul Syllable */
  ["\uB8B2", "\uB8B2"], /* Hangul Syllable */
  ["\uB8B3", "\uB8B3"], /* Hangul Syllable */
  ["\uB8B4", "\uB8B4"], /* Hangul Syllable */
  ["\uB8B5", "\uB8B5"], /* Hangul Syllable */
  ["\uB8B6", "\uB8B6"], /* Hangul Syllable */
  ["\uB8B7", "\uB8B7"], /* Hangul Syllable */
  ["\uB8B8", "\uB8B8"], /* Hangul Syllable */
  ["\uB8B9", "\uB8B9"], /* Hangul Syllable */
  ["\uB8BA", "\uB8BA"], /* Hangul Syllable */
  ["\uB8BB", "\uB8BB"], /* Hangul Syllable */
  ["\uB8BC", "\uB8BC"], /* Hangul Syllable */
  ["\uB8BD", "\uB8BD"], /* Hangul Syllable */
  ["\uB8BE", "\uB8BE"], /* Hangul Syllable */
  ["\uB8BF", "\uB8BF"], /* Hangul Syllable */
  ["\uB8C0", "\uB8C0"], /* Hangul Syllable */
  ["\uB8C1", "\uB8C1"], /* Hangul Syllable */
  ["\uB8C2", "\uB8C2"], /* Hangul Syllable */
  ["\uB8C3", "\uB8C3"], /* Hangul Syllable */
  ["\uB8C4", "\uB8C4"], /* Hangul Syllable */
  ["\uB8C5", "\uB8C5"], /* Hangul Syllable */
  ["\uB8C6", "\uB8C6"], /* Hangul Syllable */
  ["\uB8C7", "\uB8C7"], /* Hangul Syllable */
  ["\uB8C8", "\uB8C8"], /* Hangul Syllable */
  ["\uB8C9", "\uB8C9"], /* Hangul Syllable */
  ["\uB8CA", "\uB8CA"], /* Hangul Syllable */
  ["\uB8CB", "\uB8CB"], /* Hangul Syllable */
  ["\uB8CC", "\uB8CC"], /* Hangul Syllable */
  ["\uB8CD", "\uB8CD"], /* Hangul Syllable */
  ["\uB8CE", "\uB8CE"], /* Hangul Syllable */
  ["\uB8CF", "\uB8CF"], /* Hangul Syllable */
  ["\uB8D0", "\uB8D0"], /* Hangul Syllable */
  ["\uB8D1", "\uB8D1"], /* Hangul Syllable */
  ["\uB8D2", "\uB8D2"], /* Hangul Syllable */
  ["\uB8D3", "\uB8D3"], /* Hangul Syllable */
  ["\uB8D4", "\uB8D4"], /* Hangul Syllable */
  ["\uB8D5", "\uB8D5"], /* Hangul Syllable */
  ["\uB8D6", "\uB8D6"], /* Hangul Syllable */
  ["\uB8D7", "\uB8D7"], /* Hangul Syllable */
  ["\uB8D8", "\uB8D8"], /* Hangul Syllable */
  ["\uB8D9", "\uB8D9"], /* Hangul Syllable */
  ["\uB8DA", "\uB8DA"], /* Hangul Syllable */
  ["\uB8DB", "\uB8DB"], /* Hangul Syllable */
  ["\uB8DC", "\uB8DC"], /* Hangul Syllable */
  ["\uB8DD", "\uB8DD"], /* Hangul Syllable */
  ["\uB8DE", "\uB8DE"], /* Hangul Syllable */
  ["\uB8DF", "\uB8DF"], /* Hangul Syllable */
  ["\uB8E0", "\uB8E0"], /* Hangul Syllable */
  ["\uB8E1", "\uB8E1"], /* Hangul Syllable */
  ["\uB8E2", "\uB8E2"], /* Hangul Syllable */
  ["\uB8E3", "\uB8E3"], /* Hangul Syllable */
  ["\uB8E4", "\uB8E4"], /* Hangul Syllable */
  ["\uB8E5", "\uB8E5"], /* Hangul Syllable */
  ["\uB8E6", "\uB8E6"], /* Hangul Syllable */
  ["\uB8E7", "\uB8E7"], /* Hangul Syllable */
  ["\uB8E8", "\uB8E8"], /* Hangul Syllable */
  ["\uB8E9", "\uB8E9"], /* Hangul Syllable */
  ["\uB8EA", "\uB8EA"], /* Hangul Syllable */
  ["\uB8EB", "\uB8EB"], /* Hangul Syllable */
  ["\uB8EC", "\uB8EC"], /* Hangul Syllable */
  ["\uB8ED", "\uB8ED"], /* Hangul Syllable */
  ["\uB8EE", "\uB8EE"], /* Hangul Syllable */
  ["\uB8EF", "\uB8EF"], /* Hangul Syllable */
  ["\uB8F0", "\uB8F0"], /* Hangul Syllable */
  ["\uB8F1", "\uB8F1"], /* Hangul Syllable */
  ["\uB8F2", "\uB8F2"], /* Hangul Syllable */
  ["\uB8F3", "\uB8F3"], /* Hangul Syllable */
  ["\uB8F4", "\uB8F4"], /* Hangul Syllable */
  ["\uB8F5", "\uB8F5"], /* Hangul Syllable */
  ["\uB8F6", "\uB8F6"], /* Hangul Syllable */
  ["\uB8F7", "\uB8F7"], /* Hangul Syllable */
  ["\uB8F8", "\uB8F8"], /* Hangul Syllable */
  ["\uB8F9", "\uB8F9"], /* Hangul Syllable */
  ["\uB8FA", "\uB8FA"], /* Hangul Syllable */
  ["\uB8FB", "\uB8FB"], /* Hangul Syllable */
  ["\uB8FC", "\uB8FC"], /* Hangul Syllable */
  ["\uB8FD", "\uB8FD"], /* Hangul Syllable */
  ["\uB8FE", "\uB8FE"], /* Hangul Syllable */
  ["\uB8FF", "\uB8FF"], /* Hangul Syllable */
  ["\uB900", "\uB900"], /* Hangul Syllable */
  ["\uB901", "\uB901"], /* Hangul Syllable */
  ["\uB902", "\uB902"], /* Hangul Syllable */
  ["\uB903", "\uB903"], /* Hangul Syllable */
  ["\uB904", "\uB904"], /* Hangul Syllable */
  ["\uB905", "\uB905"], /* Hangul Syllable */
  ["\uB906", "\uB906"], /* Hangul Syllable */
  ["\uB907", "\uB907"], /* Hangul Syllable */
  ["\uB908", "\uB908"], /* Hangul Syllable */
  ["\uB909", "\uB909"], /* Hangul Syllable */
  ["\uB90A", "\uB90A"], /* Hangul Syllable */
  ["\uB90B", "\uB90B"], /* Hangul Syllable */
  ["\uB90C", "\uB90C"], /* Hangul Syllable */
  ["\uB90D", "\uB90D"], /* Hangul Syllable */
  ["\uB90E", "\uB90E"], /* Hangul Syllable */
  ["\uB90F", "\uB90F"], /* Hangul Syllable */
  ["\uB910", "\uB910"], /* Hangul Syllable */
  ["\uB911", "\uB911"], /* Hangul Syllable */
  ["\uB912", "\uB912"], /* Hangul Syllable */
  ["\uB913", "\uB913"], /* Hangul Syllable */
  ["\uB914", "\uB914"], /* Hangul Syllable */
  ["\uB915", "\uB915"], /* Hangul Syllable */
  ["\uB916", "\uB916"], /* Hangul Syllable */
  ["\uB917", "\uB917"], /* Hangul Syllable */
  ["\uB918", "\uB918"], /* Hangul Syllable */
  ["\uB919", "\uB919"], /* Hangul Syllable */
  ["\uB91A", "\uB91A"], /* Hangul Syllable */
  ["\uB91B", "\uB91B"], /* Hangul Syllable */
  ["\uB91C", "\uB91C"], /* Hangul Syllable */
  ["\uB91D", "\uB91D"], /* Hangul Syllable */
  ["\uB91E", "\uB91E"], /* Hangul Syllable */
  ["\uB91F", "\uB91F"], /* Hangul Syllable */
  ["\uB920", "\uB920"], /* Hangul Syllable */
  ["\uB921", "\uB921"], /* Hangul Syllable */
  ["\uB922", "\uB922"], /* Hangul Syllable */
  ["\uB923", "\uB923"], /* Hangul Syllable */
  ["\uB924", "\uB924"], /* Hangul Syllable */
  ["\uB925", "\uB925"], /* Hangul Syllable */
  ["\uB926", "\uB926"], /* Hangul Syllable */
  ["\uB927", "\uB927"], /* Hangul Syllable */
  ["\uB928", "\uB928"], /* Hangul Syllable */
  ["\uB929", "\uB929"], /* Hangul Syllable */
  ["\uB92A", "\uB92A"], /* Hangul Syllable */
  ["\uB92B", "\uB92B"], /* Hangul Syllable */
  ["\uB92C", "\uB92C"], /* Hangul Syllable */
  ["\uB92D", "\uB92D"], /* Hangul Syllable */
  ["\uB92E", "\uB92E"], /* Hangul Syllable */
  ["\uB92F", "\uB92F"], /* Hangul Syllable */
  ["\uB930", "\uB930"], /* Hangul Syllable */
  ["\uB931", "\uB931"], /* Hangul Syllable */
  ["\uB932", "\uB932"], /* Hangul Syllable */
  ["\uB933", "\uB933"], /* Hangul Syllable */
  ["\uB934", "\uB934"], /* Hangul Syllable */
  ["\uB935", "\uB935"], /* Hangul Syllable */
  ["\uB936", "\uB936"], /* Hangul Syllable */
  ["\uB937", "\uB937"], /* Hangul Syllable */
  ["\uB938", "\uB938"], /* Hangul Syllable */
  ["\uB939", "\uB939"], /* Hangul Syllable */
  ["\uB93A", "\uB93A"], /* Hangul Syllable */
  ["\uB93B", "\uB93B"], /* Hangul Syllable */
  ["\uB93C", "\uB93C"], /* Hangul Syllable */
  ["\uB93D", "\uB93D"], /* Hangul Syllable */
  ["\uB93E", "\uB93E"], /* Hangul Syllable */
  ["\uB93F", "\uB93F"], /* Hangul Syllable */
  ["\uB940", "\uB940"], /* Hangul Syllable */
  ["\uB941", "\uB941"], /* Hangul Syllable */
  ["\uB942", "\uB942"], /* Hangul Syllable */
  ["\uB943", "\uB943"], /* Hangul Syllable */
  ["\uB944", "\uB944"], /* Hangul Syllable */
  ["\uB945", "\uB945"], /* Hangul Syllable */
  ["\uB946", "\uB946"], /* Hangul Syllable */
  ["\uB947", "\uB947"], /* Hangul Syllable */
  ["\uB948", "\uB948"], /* Hangul Syllable */
  ["\uB949", "\uB949"], /* Hangul Syllable */
  ["\uB94A", "\uB94A"], /* Hangul Syllable */
  ["\uB94B", "\uB94B"], /* Hangul Syllable */
  ["\uB94C", "\uB94C"], /* Hangul Syllable */
  ["\uB94D", "\uB94D"], /* Hangul Syllable */
  ["\uB94E", "\uB94E"], /* Hangul Syllable */
  ["\uB94F", "\uB94F"], /* Hangul Syllable */
  ["\uB950", "\uB950"], /* Hangul Syllable */
  ["\uB951", "\uB951"], /* Hangul Syllable */
  ["\uB952", "\uB952"], /* Hangul Syllable */
  ["\uB953", "\uB953"], /* Hangul Syllable */
  ["\uB954", "\uB954"], /* Hangul Syllable */
  ["\uB955", "\uB955"], /* Hangul Syllable */
  ["\uB956", "\uB956"], /* Hangul Syllable */
  ["\uB957", "\uB957"], /* Hangul Syllable */
  ["\uB958", "\uB958"], /* Hangul Syllable */
  ["\uB959", "\uB959"], /* Hangul Syllable */
  ["\uB95A", "\uB95A"], /* Hangul Syllable */
  ["\uB95B", "\uB95B"], /* Hangul Syllable */
  ["\uB95C", "\uB95C"], /* Hangul Syllable */
  ["\uB95D", "\uB95D"], /* Hangul Syllable */
  ["\uB95E", "\uB95E"], /* Hangul Syllable */
  ["\uB95F", "\uB95F"], /* Hangul Syllable */
  ["\uB960", "\uB960"], /* Hangul Syllable */
  ["\uB961", "\uB961"], /* Hangul Syllable */
  ["\uB962", "\uB962"], /* Hangul Syllable */
  ["\uB963", "\uB963"], /* Hangul Syllable */
  ["\uB964", "\uB964"], /* Hangul Syllable */
  ["\uB965", "\uB965"], /* Hangul Syllable */
  ["\uB966", "\uB966"], /* Hangul Syllable */
  ["\uB967", "\uB967"], /* Hangul Syllable */
  ["\uB968", "\uB968"], /* Hangul Syllable */
  ["\uB969", "\uB969"], /* Hangul Syllable */
  ["\uB96A", "\uB96A"], /* Hangul Syllable */
  ["\uB96B", "\uB96B"], /* Hangul Syllable */
  ["\uB96C", "\uB96C"], /* Hangul Syllable */
  ["\uB96D", "\uB96D"], /* Hangul Syllable */
  ["\uB96E", "\uB96E"], /* Hangul Syllable */
  ["\uB96F", "\uB96F"], /* Hangul Syllable */
  ["\uB970", "\uB970"], /* Hangul Syllable */
  ["\uB971", "\uB971"], /* Hangul Syllable */
  ["\uB972", "\uB972"], /* Hangul Syllable */
  ["\uB973", "\uB973"], /* Hangul Syllable */
  ["\uB974", "\uB974"], /* Hangul Syllable */
  ["\uB975", "\uB975"], /* Hangul Syllable */
  ["\uB976", "\uB976"], /* Hangul Syllable */
  ["\uB977", "\uB977"], /* Hangul Syllable */
  ["\uB978", "\uB978"], /* Hangul Syllable */
  ["\uB979", "\uB979"], /* Hangul Syllable */
  ["\uB97A", "\uB97A"], /* Hangul Syllable */
  ["\uB97B", "\uB97B"], /* Hangul Syllable */
  ["\uB97C", "\uB97C"], /* Hangul Syllable */
  ["\uB97D", "\uB97D"], /* Hangul Syllable */
  ["\uB97E", "\uB97E"], /* Hangul Syllable */
  ["\uB97F", "\uB97F"], /* Hangul Syllable */
  ["\uB980", "\uB980"], /* Hangul Syllable */
  ["\uB981", "\uB981"], /* Hangul Syllable */
  ["\uB982", "\uB982"], /* Hangul Syllable */
  ["\uB983", "\uB983"], /* Hangul Syllable */
  ["\uB984", "\uB984"], /* Hangul Syllable */
  ["\uB985", "\uB985"], /* Hangul Syllable */
  ["\uB986", "\uB986"], /* Hangul Syllable */
  ["\uB987", "\uB987"], /* Hangul Syllable */
  ["\uB988", "\uB988"], /* Hangul Syllable */
  ["\uB989", "\uB989"], /* Hangul Syllable */
  ["\uB98A", "\uB98A"], /* Hangul Syllable */
  ["\uB98B", "\uB98B"], /* Hangul Syllable */
  ["\uB98C", "\uB98C"], /* Hangul Syllable */
  ["\uB98D", "\uB98D"], /* Hangul Syllable */
  ["\uB98E", "\uB98E"], /* Hangul Syllable */
  ["\uB98F", "\uB98F"], /* Hangul Syllable */
  ["\uB990", "\uB990"], /* Hangul Syllable */
  ["\uB991", "\uB991"], /* Hangul Syllable */
  ["\uB992", "\uB992"], /* Hangul Syllable */
  ["\uB993", "\uB993"], /* Hangul Syllable */
  ["\uB994", "\uB994"], /* Hangul Syllable */
  ["\uB995", "\uB995"], /* Hangul Syllable */
  ["\uB996", "\uB996"], /* Hangul Syllable */
  ["\uB997", "\uB997"], /* Hangul Syllable */
  ["\uB998", "\uB998"], /* Hangul Syllable */
  ["\uB999", "\uB999"], /* Hangul Syllable */
  ["\uB99A", "\uB99A"], /* Hangul Syllable */
  ["\uB99B", "\uB99B"], /* Hangul Syllable */
  ["\uB99C", "\uB99C"], /* Hangul Syllable */
  ["\uB99D", "\uB99D"], /* Hangul Syllable */
  ["\uB99E", "\uB99E"], /* Hangul Syllable */
  ["\uB99F", "\uB99F"], /* Hangul Syllable */
  ["\uB9A0", "\uB9A0"], /* Hangul Syllable */
  ["\uB9A1", "\uB9A1"], /* Hangul Syllable */
  ["\uB9A2", "\uB9A2"], /* Hangul Syllable */
  ["\uB9A3", "\uB9A3"], /* Hangul Syllable */
  ["\uB9A4", "\uB9A4"], /* Hangul Syllable */
  ["\uB9A5", "\uB9A5"], /* Hangul Syllable */
  ["\uB9A6", "\uB9A6"], /* Hangul Syllable */
  ["\uB9A7", "\uB9A7"], /* Hangul Syllable */
  ["\uB9A8", "\uB9A8"], /* Hangul Syllable */
  ["\uB9A9", "\uB9A9"], /* Hangul Syllable */
  ["\uB9AA", "\uB9AA"], /* Hangul Syllable */
  ["\uB9AB", "\uB9AB"], /* Hangul Syllable */
  ["\uB9AC", "\uB9AC"], /* Hangul Syllable */
  ["\uB9AD", "\uB9AD"], /* Hangul Syllable */
  ["\uB9AE", "\uB9AE"], /* Hangul Syllable */
  ["\uB9AF", "\uB9AF"], /* Hangul Syllable */
  ["\uB9B0", "\uB9B0"], /* Hangul Syllable */
  ["\uB9B1", "\uB9B1"], /* Hangul Syllable */
  ["\uB9B2", "\uB9B2"], /* Hangul Syllable */
  ["\uB9B3", "\uB9B3"], /* Hangul Syllable */
  ["\uB9B4", "\uB9B4"], /* Hangul Syllable */
  ["\uB9B5", "\uB9B5"], /* Hangul Syllable */
  ["\uB9B6", "\uB9B6"], /* Hangul Syllable */
  ["\uB9B7", "\uB9B7"], /* Hangul Syllable */
  ["\uB9B8", "\uB9B8"], /* Hangul Syllable */
  ["\uB9B9", "\uB9B9"], /* Hangul Syllable */
  ["\uB9BA", "\uB9BA"], /* Hangul Syllable */
  ["\uB9BB", "\uB9BB"], /* Hangul Syllable */
  ["\uB9BC", "\uB9BC"], /* Hangul Syllable */
  ["\uB9BD", "\uB9BD"], /* Hangul Syllable */
  ["\uB9BE", "\uB9BE"], /* Hangul Syllable */
  ["\uB9BF", "\uB9BF"], /* Hangul Syllable */
  ["\uB9C0", "\uB9C0"], /* Hangul Syllable */
  ["\uB9C1", "\uB9C1"], /* Hangul Syllable */
  ["\uB9C2", "\uB9C2"], /* Hangul Syllable */
  ["\uB9C3", "\uB9C3"], /* Hangul Syllable */
  ["\uB9C4", "\uB9C4"], /* Hangul Syllable */
  ["\uB9C5", "\uB9C5"], /* Hangul Syllable */
  ["\uB9C6", "\uB9C6"], /* Hangul Syllable */
  ["\uB9C7", "\uB9C7"], /* Hangul Syllable */
  ["\uB9C8", "\uB9C8"], /* Hangul Syllable */
  ["\uB9C9", "\uB9C9"], /* Hangul Syllable */
  ["\uB9CA", "\uB9CA"], /* Hangul Syllable */
  ["\uB9CB", "\uB9CB"], /* Hangul Syllable */
  ["\uB9CC", "\uB9CC"], /* Hangul Syllable */
  ["\uB9CD", "\uB9CD"], /* Hangul Syllable */
  ["\uB9CE", "\uB9CE"], /* Hangul Syllable */
  ["\uB9CF", "\uB9CF"], /* Hangul Syllable */
  ["\uB9D0", "\uB9D0"], /* Hangul Syllable */
  ["\uB9D1", "\uB9D1"], /* Hangul Syllable */
  ["\uB9D2", "\uB9D2"], /* Hangul Syllable */
  ["\uB9D3", "\uB9D3"], /* Hangul Syllable */
  ["\uB9D4", "\uB9D4"], /* Hangul Syllable */
  ["\uB9D5", "\uB9D5"], /* Hangul Syllable */
  ["\uB9D6", "\uB9D6"], /* Hangul Syllable */
  ["\uB9D7", "\uB9D7"], /* Hangul Syllable */
  ["\uB9D8", "\uB9D8"], /* Hangul Syllable */
  ["\uB9D9", "\uB9D9"], /* Hangul Syllable */
  ["\uB9DA", "\uB9DA"], /* Hangul Syllable */
  ["\uB9DB", "\uB9DB"], /* Hangul Syllable */
  ["\uB9DC", "\uB9DC"], /* Hangul Syllable */
  ["\uB9DD", "\uB9DD"], /* Hangul Syllable */
  ["\uB9DE", "\uB9DE"], /* Hangul Syllable */
  ["\uB9DF", "\uB9DF"], /* Hangul Syllable */
  ["\uB9E0", "\uB9E0"], /* Hangul Syllable */
  ["\uB9E1", "\uB9E1"], /* Hangul Syllable */
  ["\uB9E2", "\uB9E2"], /* Hangul Syllable */
  ["\uB9E3", "\uB9E3"], /* Hangul Syllable */
  ["\uB9E4", "\uB9E4"], /* Hangul Syllable */
  ["\uB9E5", "\uB9E5"], /* Hangul Syllable */
  ["\uB9E6", "\uB9E6"], /* Hangul Syllable */
  ["\uB9E7", "\uB9E7"], /* Hangul Syllable */
  ["\uB9E8", "\uB9E8"], /* Hangul Syllable */
  ["\uB9E9", "\uB9E9"], /* Hangul Syllable */
  ["\uB9EA", "\uB9EA"], /* Hangul Syllable */
  ["\uB9EB", "\uB9EB"], /* Hangul Syllable */
  ["\uB9EC", "\uB9EC"], /* Hangul Syllable */
  ["\uB9ED", "\uB9ED"], /* Hangul Syllable */
  ["\uB9EE", "\uB9EE"], /* Hangul Syllable */
  ["\uB9EF", "\uB9EF"], /* Hangul Syllable */
  ["\uB9F0", "\uB9F0"], /* Hangul Syllable */
  ["\uB9F1", "\uB9F1"], /* Hangul Syllable */
  ["\uB9F2", "\uB9F2"], /* Hangul Syllable */
  ["\uB9F3", "\uB9F3"], /* Hangul Syllable */
  ["\uB9F4", "\uB9F4"], /* Hangul Syllable */
  ["\uB9F5", "\uB9F5"], /* Hangul Syllable */
  ["\uB9F6", "\uB9F6"], /* Hangul Syllable */
  ["\uB9F7", "\uB9F7"], /* Hangul Syllable */
  ["\uB9F8", "\uB9F8"], /* Hangul Syllable */
  ["\uB9F9", "\uB9F9"], /* Hangul Syllable */
  ["\uB9FA", "\uB9FA"], /* Hangul Syllable */
  ["\uB9FB", "\uB9FB"], /* Hangul Syllable */
  ["\uB9FC", "\uB9FC"], /* Hangul Syllable */
  ["\uB9FD", "\uB9FD"], /* Hangul Syllable */
  ["\uB9FE", "\uB9FE"], /* Hangul Syllable */
  ["\uB9FF", "\uB9FF"], /* Hangul Syllable */
  ["\uBA00", "\uBA00"], /* Hangul Syllable */
  ["\uBA01", "\uBA01"], /* Hangul Syllable */
  ["\uBA02", "\uBA02"], /* Hangul Syllable */
  ["\uBA03", "\uBA03"], /* Hangul Syllable */
  ["\uBA04", "\uBA04"], /* Hangul Syllable */
  ["\uBA05", "\uBA05"], /* Hangul Syllable */
  ["\uBA06", "\uBA06"], /* Hangul Syllable */
  ["\uBA07", "\uBA07"], /* Hangul Syllable */
  ["\uBA08", "\uBA08"], /* Hangul Syllable */
  ["\uBA09", "\uBA09"], /* Hangul Syllable */
  ["\uBA0A", "\uBA0A"], /* Hangul Syllable */
  ["\uBA0B", "\uBA0B"], /* Hangul Syllable */
  ["\uBA0C", "\uBA0C"], /* Hangul Syllable */
  ["\uBA0D", "\uBA0D"], /* Hangul Syllable */
  ["\uBA0E", "\uBA0E"], /* Hangul Syllable */
  ["\uBA0F", "\uBA0F"], /* Hangul Syllable */
  ["\uBA10", "\uBA10"], /* Hangul Syllable */
  ["\uBA11", "\uBA11"], /* Hangul Syllable */
  ["\uBA12", "\uBA12"], /* Hangul Syllable */
  ["\uBA13", "\uBA13"], /* Hangul Syllable */
  ["\uBA14", "\uBA14"], /* Hangul Syllable */
  ["\uBA15", "\uBA15"], /* Hangul Syllable */
  ["\uBA16", "\uBA16"], /* Hangul Syllable */
  ["\uBA17", "\uBA17"], /* Hangul Syllable */
  ["\uBA18", "\uBA18"], /* Hangul Syllable */
  ["\uBA19", "\uBA19"], /* Hangul Syllable */
  ["\uBA1A", "\uBA1A"], /* Hangul Syllable */
  ["\uBA1B", "\uBA1B"], /* Hangul Syllable */
  ["\uBA1C", "\uBA1C"], /* Hangul Syllable */
  ["\uBA1D", "\uBA1D"], /* Hangul Syllable */
  ["\uBA1E", "\uBA1E"], /* Hangul Syllable */
  ["\uBA1F", "\uBA1F"], /* Hangul Syllable */
  ["\uBA20", "\uBA20"], /* Hangul Syllable */
  ["\uBA21", "\uBA21"], /* Hangul Syllable */
  ["\uBA22", "\uBA22"], /* Hangul Syllable */
  ["\uBA23", "\uBA23"], /* Hangul Syllable */
  ["\uBA24", "\uBA24"], /* Hangul Syllable */
  ["\uBA25", "\uBA25"], /* Hangul Syllable */
  ["\uBA26", "\uBA26"], /* Hangul Syllable */
  ["\uBA27", "\uBA27"], /* Hangul Syllable */
  ["\uBA28", "\uBA28"], /* Hangul Syllable */
  ["\uBA29", "\uBA29"], /* Hangul Syllable */
  ["\uBA2A", "\uBA2A"], /* Hangul Syllable */
  ["\uBA2B", "\uBA2B"], /* Hangul Syllable */
  ["\uBA2C", "\uBA2C"], /* Hangul Syllable */
  ["\uBA2D", "\uBA2D"], /* Hangul Syllable */
  ["\uBA2E", "\uBA2E"], /* Hangul Syllable */
  ["\uBA2F", "\uBA2F"], /* Hangul Syllable */
  ["\uBA30", "\uBA30"], /* Hangul Syllable */
  ["\uBA31", "\uBA31"], /* Hangul Syllable */
  ["\uBA32", "\uBA32"], /* Hangul Syllable */
  ["\uBA33", "\uBA33"], /* Hangul Syllable */
  ["\uBA34", "\uBA34"], /* Hangul Syllable */
  ["\uBA35", "\uBA35"], /* Hangul Syllable */
  ["\uBA36", "\uBA36"], /* Hangul Syllable */
  ["\uBA37", "\uBA37"], /* Hangul Syllable */
  ["\uBA38", "\uBA38"], /* Hangul Syllable */
  ["\uBA39", "\uBA39"], /* Hangul Syllable */
  ["\uBA3A", "\uBA3A"], /* Hangul Syllable */
  ["\uBA3B", "\uBA3B"], /* Hangul Syllable */
  ["\uBA3C", "\uBA3C"], /* Hangul Syllable */
  ["\uBA3D", "\uBA3D"], /* Hangul Syllable */
  ["\uBA3E", "\uBA3E"], /* Hangul Syllable */
  ["\uBA3F", "\uBA3F"], /* Hangul Syllable */
  ["\uBA40", "\uBA40"], /* Hangul Syllable */
  ["\uBA41", "\uBA41"], /* Hangul Syllable */
  ["\uBA42", "\uBA42"], /* Hangul Syllable */
  ["\uBA43", "\uBA43"], /* Hangul Syllable */
  ["\uBA44", "\uBA44"], /* Hangul Syllable */
  ["\uBA45", "\uBA45"], /* Hangul Syllable */
  ["\uBA46", "\uBA46"], /* Hangul Syllable */
  ["\uBA47", "\uBA47"], /* Hangul Syllable */
  ["\uBA48", "\uBA48"], /* Hangul Syllable */
  ["\uBA49", "\uBA49"], /* Hangul Syllable */
  ["\uBA4A", "\uBA4A"], /* Hangul Syllable */
  ["\uBA4B", "\uBA4B"], /* Hangul Syllable */
  ["\uBA4C", "\uBA4C"], /* Hangul Syllable */
  ["\uBA4D", "\uBA4D"], /* Hangul Syllable */
  ["\uBA4E", "\uBA4E"], /* Hangul Syllable */
  ["\uBA4F", "\uBA4F"], /* Hangul Syllable */
  ["\uBA50", "\uBA50"], /* Hangul Syllable */
  ["\uBA51", "\uBA51"], /* Hangul Syllable */
  ["\uBA52", "\uBA52"], /* Hangul Syllable */
  ["\uBA53", "\uBA53"], /* Hangul Syllable */
  ["\uBA54", "\uBA54"], /* Hangul Syllable */
  ["\uBA55", "\uBA55"], /* Hangul Syllable */
  ["\uBA56", "\uBA56"], /* Hangul Syllable */
  ["\uBA57", "\uBA57"], /* Hangul Syllable */
  ["\uBA58", "\uBA58"], /* Hangul Syllable */
  ["\uBA59", "\uBA59"], /* Hangul Syllable */
  ["\uBA5A", "\uBA5A"], /* Hangul Syllable */
  ["\uBA5B", "\uBA5B"], /* Hangul Syllable */
  ["\uBA5C", "\uBA5C"], /* Hangul Syllable */
  ["\uBA5D", "\uBA5D"], /* Hangul Syllable */
  ["\uBA5E", "\uBA5E"], /* Hangul Syllable */
  ["\uBA5F", "\uBA5F"], /* Hangul Syllable */
  ["\uBA60", "\uBA60"], /* Hangul Syllable */
  ["\uBA61", "\uBA61"], /* Hangul Syllable */
  ["\uBA62", "\uBA62"], /* Hangul Syllable */
  ["\uBA63", "\uBA63"], /* Hangul Syllable */
  ["\uBA64", "\uBA64"], /* Hangul Syllable */
  ["\uBA65", "\uBA65"], /* Hangul Syllable */
  ["\uBA66", "\uBA66"], /* Hangul Syllable */
  ["\uBA67", "\uBA67"], /* Hangul Syllable */
  ["\uBA68", "\uBA68"], /* Hangul Syllable */
  ["\uBA69", "\uBA69"], /* Hangul Syllable */
  ["\uBA6A", "\uBA6A"], /* Hangul Syllable */
  ["\uBA6B", "\uBA6B"], /* Hangul Syllable */
  ["\uBA6C", "\uBA6C"], /* Hangul Syllable */
  ["\uBA6D", "\uBA6D"], /* Hangul Syllable */
  ["\uBA6E", "\uBA6E"], /* Hangul Syllable */
  ["\uBA6F", "\uBA6F"], /* Hangul Syllable */
  ["\uBA70", "\uBA70"], /* Hangul Syllable */
  ["\uBA71", "\uBA71"], /* Hangul Syllable */
  ["\uBA72", "\uBA72"], /* Hangul Syllable */
  ["\uBA73", "\uBA73"], /* Hangul Syllable */
  ["\uBA74", "\uBA74"], /* Hangul Syllable */
  ["\uBA75", "\uBA75"], /* Hangul Syllable */
  ["\uBA76", "\uBA76"], /* Hangul Syllable */
  ["\uBA77", "\uBA77"], /* Hangul Syllable */
  ["\uBA78", "\uBA78"], /* Hangul Syllable */
  ["\uBA79", "\uBA79"], /* Hangul Syllable */
  ["\uBA7A", "\uBA7A"], /* Hangul Syllable */
  ["\uBA7B", "\uBA7B"], /* Hangul Syllable */
  ["\uBA7C", "\uBA7C"], /* Hangul Syllable */
  ["\uBA7D", "\uBA7D"], /* Hangul Syllable */
  ["\uBA7E", "\uBA7E"], /* Hangul Syllable */
  ["\uBA7F", "\uBA7F"], /* Hangul Syllable */
  ["\uBA80", "\uBA80"], /* Hangul Syllable */
  ["\uBA81", "\uBA81"], /* Hangul Syllable */
  ["\uBA82", "\uBA82"], /* Hangul Syllable */
  ["\uBA83", "\uBA83"], /* Hangul Syllable */
  ["\uBA84", "\uBA84"], /* Hangul Syllable */
  ["\uBA85", "\uBA85"], /* Hangul Syllable */
  ["\uBA86", "\uBA86"], /* Hangul Syllable */
  ["\uBA87", "\uBA87"], /* Hangul Syllable */
  ["\uBA88", "\uBA88"], /* Hangul Syllable */
  ["\uBA89", "\uBA89"], /* Hangul Syllable */
  ["\uBA8A", "\uBA8A"], /* Hangul Syllable */
  ["\uBA8B", "\uBA8B"], /* Hangul Syllable */
  ["\uBA8C", "\uBA8C"], /* Hangul Syllable */
  ["\uBA8D", "\uBA8D"], /* Hangul Syllable */
  ["\uBA8E", "\uBA8E"], /* Hangul Syllable */
  ["\uBA8F", "\uBA8F"], /* Hangul Syllable */
  ["\uBA90", "\uBA90"], /* Hangul Syllable */
  ["\uBA91", "\uBA91"], /* Hangul Syllable */
  ["\uBA92", "\uBA92"], /* Hangul Syllable */
  ["\uBA93", "\uBA93"], /* Hangul Syllable */
  ["\uBA94", "\uBA94"], /* Hangul Syllable */
  ["\uBA95", "\uBA95"], /* Hangul Syllable */
  ["\uBA96", "\uBA96"], /* Hangul Syllable */
  ["\uBA97", "\uBA97"], /* Hangul Syllable */
  ["\uBA98", "\uBA98"], /* Hangul Syllable */
  ["\uBA99", "\uBA99"], /* Hangul Syllable */
  ["\uBA9A", "\uBA9A"], /* Hangul Syllable */
  ["\uBA9B", "\uBA9B"], /* Hangul Syllable */
  ["\uBA9C", "\uBA9C"], /* Hangul Syllable */
  ["\uBA9D", "\uBA9D"], /* Hangul Syllable */
  ["\uBA9E", "\uBA9E"], /* Hangul Syllable */
  ["\uBA9F", "\uBA9F"], /* Hangul Syllable */
  ["\uBAA0", "\uBAA0"], /* Hangul Syllable */
  ["\uBAA1", "\uBAA1"], /* Hangul Syllable */
  ["\uBAA2", "\uBAA2"], /* Hangul Syllable */
  ["\uBAA3", "\uBAA3"], /* Hangul Syllable */
  ["\uBAA4", "\uBAA4"], /* Hangul Syllable */
  ["\uBAA5", "\uBAA5"], /* Hangul Syllable */
  ["\uBAA6", "\uBAA6"], /* Hangul Syllable */
  ["\uBAA7", "\uBAA7"], /* Hangul Syllable */
  ["\uBAA8", "\uBAA8"], /* Hangul Syllable */
  ["\uBAA9", "\uBAA9"], /* Hangul Syllable */
  ["\uBAAA", "\uBAAA"], /* Hangul Syllable */
  ["\uBAAB", "\uBAAB"], /* Hangul Syllable */
  ["\uBAAC", "\uBAAC"], /* Hangul Syllable */
  ["\uBAAD", "\uBAAD"], /* Hangul Syllable */
  ["\uBAAE", "\uBAAE"], /* Hangul Syllable */
  ["\uBAAF", "\uBAAF"], /* Hangul Syllable */
  ["\uBAB0", "\uBAB0"], /* Hangul Syllable */
  ["\uBAB1", "\uBAB1"], /* Hangul Syllable */
  ["\uBAB2", "\uBAB2"], /* Hangul Syllable */
  ["\uBAB3", "\uBAB3"], /* Hangul Syllable */
  ["\uBAB4", "\uBAB4"], /* Hangul Syllable */
  ["\uBAB5", "\uBAB5"], /* Hangul Syllable */
  ["\uBAB6", "\uBAB6"], /* Hangul Syllable */
  ["\uBAB7", "\uBAB7"], /* Hangul Syllable */
  ["\uBAB8", "\uBAB8"], /* Hangul Syllable */
  ["\uBAB9", "\uBAB9"], /* Hangul Syllable */
  ["\uBABA", "\uBABA"], /* Hangul Syllable */
  ["\uBABB", "\uBABB"], /* Hangul Syllable */
  ["\uBABC", "\uBABC"], /* Hangul Syllable */
  ["\uBABD", "\uBABD"], /* Hangul Syllable */
  ["\uBABE", "\uBABE"], /* Hangul Syllable */
  ["\uBABF", "\uBABF"], /* Hangul Syllable */
  ["\uBAC0", "\uBAC0"], /* Hangul Syllable */
  ["\uBAC1", "\uBAC1"], /* Hangul Syllable */
  ["\uBAC2", "\uBAC2"], /* Hangul Syllable */
  ["\uBAC3", "\uBAC3"], /* Hangul Syllable */
  ["\uBAC4", "\uBAC4"], /* Hangul Syllable */
  ["\uBAC5", "\uBAC5"], /* Hangul Syllable */
  ["\uBAC6", "\uBAC6"], /* Hangul Syllable */
  ["\uBAC7", "\uBAC7"], /* Hangul Syllable */
  ["\uBAC8", "\uBAC8"], /* Hangul Syllable */
  ["\uBAC9", "\uBAC9"], /* Hangul Syllable */
  ["\uBACA", "\uBACA"], /* Hangul Syllable */
  ["\uBACB", "\uBACB"], /* Hangul Syllable */
  ["\uBACC", "\uBACC"], /* Hangul Syllable */
  ["\uBACD", "\uBACD"], /* Hangul Syllable */
  ["\uBACE", "\uBACE"], /* Hangul Syllable */
  ["\uBACF", "\uBACF"], /* Hangul Syllable */
  ["\uBAD0", "\uBAD0"], /* Hangul Syllable */
  ["\uBAD1", "\uBAD1"], /* Hangul Syllable */
  ["\uBAD2", "\uBAD2"], /* Hangul Syllable */
  ["\uBAD3", "\uBAD3"], /* Hangul Syllable */
  ["\uBAD4", "\uBAD4"], /* Hangul Syllable */
  ["\uBAD5", "\uBAD5"], /* Hangul Syllable */
  ["\uBAD6", "\uBAD6"], /* Hangul Syllable */
  ["\uBAD7", "\uBAD7"], /* Hangul Syllable */
  ["\uBAD8", "\uBAD8"], /* Hangul Syllable */
  ["\uBAD9", "\uBAD9"], /* Hangul Syllable */
  ["\uBADA", "\uBADA"], /* Hangul Syllable */
  ["\uBADB", "\uBADB"], /* Hangul Syllable */
  ["\uBADC", "\uBADC"], /* Hangul Syllable */
  ["\uBADD", "\uBADD"], /* Hangul Syllable */
  ["\uBADE", "\uBADE"], /* Hangul Syllable */
  ["\uBADF", "\uBADF"], /* Hangul Syllable */
  ["\uBAE0", "\uBAE0"], /* Hangul Syllable */
  ["\uBAE1", "\uBAE1"], /* Hangul Syllable */
  ["\uBAE2", "\uBAE2"], /* Hangul Syllable */
  ["\uBAE3", "\uBAE3"], /* Hangul Syllable */
  ["\uBAE4", "\uBAE4"], /* Hangul Syllable */
  ["\uBAE5", "\uBAE5"], /* Hangul Syllable */
  ["\uBAE6", "\uBAE6"], /* Hangul Syllable */
  ["\uBAE7", "\uBAE7"], /* Hangul Syllable */
  ["\uBAE8", "\uBAE8"], /* Hangul Syllable */
  ["\uBAE9", "\uBAE9"], /* Hangul Syllable */
  ["\uBAEA", "\uBAEA"], /* Hangul Syllable */
  ["\uBAEB", "\uBAEB"], /* Hangul Syllable */
  ["\uBAEC", "\uBAEC"], /* Hangul Syllable */
  ["\uBAED", "\uBAED"], /* Hangul Syllable */
  ["\uBAEE", "\uBAEE"], /* Hangul Syllable */
  ["\uBAEF", "\uBAEF"], /* Hangul Syllable */
  ["\uBAF0", "\uBAF0"], /* Hangul Syllable */
  ["\uBAF1", "\uBAF1"], /* Hangul Syllable */
  ["\uBAF2", "\uBAF2"], /* Hangul Syllable */
  ["\uBAF3", "\uBAF3"], /* Hangul Syllable */
  ["\uBAF4", "\uBAF4"], /* Hangul Syllable */
  ["\uBAF5", "\uBAF5"], /* Hangul Syllable */
  ["\uBAF6", "\uBAF6"], /* Hangul Syllable */
  ["\uBAF7", "\uBAF7"], /* Hangul Syllable */
  ["\uBAF8", "\uBAF8"], /* Hangul Syllable */
  ["\uBAF9", "\uBAF9"], /* Hangul Syllable */
  ["\uBAFA", "\uBAFA"], /* Hangul Syllable */
  ["\uBAFB", "\uBAFB"], /* Hangul Syllable */
  ["\uBAFC", "\uBAFC"], /* Hangul Syllable */
  ["\uBAFD", "\uBAFD"], /* Hangul Syllable */
  ["\uBAFE", "\uBAFE"], /* Hangul Syllable */
  ["\uBAFF", "\uBAFF"], /* Hangul Syllable */
  ["\uBB00", "\uBB00"], /* Hangul Syllable */
  ["\uBB01", "\uBB01"], /* Hangul Syllable */
  ["\uBB02", "\uBB02"], /* Hangul Syllable */
  ["\uBB03", "\uBB03"], /* Hangul Syllable */
  ["\uBB04", "\uBB04"], /* Hangul Syllable */
  ["\uBB05", "\uBB05"], /* Hangul Syllable */
  ["\uBB06", "\uBB06"], /* Hangul Syllable */
  ["\uBB07", "\uBB07"], /* Hangul Syllable */
  ["\uBB08", "\uBB08"], /* Hangul Syllable */
  ["\uBB09", "\uBB09"], /* Hangul Syllable */
  ["\uBB0A", "\uBB0A"], /* Hangul Syllable */
  ["\uBB0B", "\uBB0B"], /* Hangul Syllable */
  ["\uBB0C", "\uBB0C"], /* Hangul Syllable */
  ["\uBB0D", "\uBB0D"], /* Hangul Syllable */
  ["\uBB0E", "\uBB0E"], /* Hangul Syllable */
  ["\uBB0F", "\uBB0F"], /* Hangul Syllable */
  ["\uBB10", "\uBB10"], /* Hangul Syllable */
  ["\uBB11", "\uBB11"], /* Hangul Syllable */
  ["\uBB12", "\uBB12"], /* Hangul Syllable */
  ["\uBB13", "\uBB13"], /* Hangul Syllable */
  ["\uBB14", "\uBB14"], /* Hangul Syllable */
  ["\uBB15", "\uBB15"], /* Hangul Syllable */
  ["\uBB16", "\uBB16"], /* Hangul Syllable */
  ["\uBB17", "\uBB17"], /* Hangul Syllable */
  ["\uBB18", "\uBB18"], /* Hangul Syllable */
  ["\uBB19", "\uBB19"], /* Hangul Syllable */
  ["\uBB1A", "\uBB1A"], /* Hangul Syllable */
  ["\uBB1B", "\uBB1B"], /* Hangul Syllable */
  ["\uBB1C", "\uBB1C"], /* Hangul Syllable */
  ["\uBB1D", "\uBB1D"], /* Hangul Syllable */
  ["\uBB1E", "\uBB1E"], /* Hangul Syllable */
  ["\uBB1F", "\uBB1F"], /* Hangul Syllable */
  ["\uBB20", "\uBB20"], /* Hangul Syllable */
  ["\uBB21", "\uBB21"], /* Hangul Syllable */
  ["\uBB22", "\uBB22"], /* Hangul Syllable */
  ["\uBB23", "\uBB23"], /* Hangul Syllable */
  ["\uBB24", "\uBB24"], /* Hangul Syllable */
  ["\uBB25", "\uBB25"], /* Hangul Syllable */
  ["\uBB26", "\uBB26"], /* Hangul Syllable */
  ["\uBB27", "\uBB27"], /* Hangul Syllable */
  ["\uBB28", "\uBB28"], /* Hangul Syllable */
  ["\uBB29", "\uBB29"], /* Hangul Syllable */
  ["\uBB2A", "\uBB2A"], /* Hangul Syllable */
  ["\uBB2B", "\uBB2B"], /* Hangul Syllable */
  ["\uBB2C", "\uBB2C"], /* Hangul Syllable */
  ["\uBB2D", "\uBB2D"], /* Hangul Syllable */
  ["\uBB2E", "\uBB2E"], /* Hangul Syllable */
  ["\uBB2F", "\uBB2F"], /* Hangul Syllable */
  ["\uBB30", "\uBB30"], /* Hangul Syllable */
  ["\uBB31", "\uBB31"], /* Hangul Syllable */
  ["\uBB32", "\uBB32"], /* Hangul Syllable */
  ["\uBB33", "\uBB33"], /* Hangul Syllable */
  ["\uBB34", "\uBB34"], /* Hangul Syllable */
  ["\uBB35", "\uBB35"], /* Hangul Syllable */
  ["\uBB36", "\uBB36"], /* Hangul Syllable */
  ["\uBB37", "\uBB37"], /* Hangul Syllable */
  ["\uBB38", "\uBB38"], /* Hangul Syllable */
  ["\uBB39", "\uBB39"], /* Hangul Syllable */
  ["\uBB3A", "\uBB3A"], /* Hangul Syllable */
  ["\uBB3B", "\uBB3B"], /* Hangul Syllable */
  ["\uBB3C", "\uBB3C"], /* Hangul Syllable */
  ["\uBB3D", "\uBB3D"], /* Hangul Syllable */
  ["\uBB3E", "\uBB3E"], /* Hangul Syllable */
  ["\uBB3F", "\uBB3F"], /* Hangul Syllable */
  ["\uBB40", "\uBB40"], /* Hangul Syllable */
  ["\uBB41", "\uBB41"], /* Hangul Syllable */
  ["\uBB42", "\uBB42"], /* Hangul Syllable */
  ["\uBB43", "\uBB43"], /* Hangul Syllable */
  ["\uBB44", "\uBB44"], /* Hangul Syllable */
  ["\uBB45", "\uBB45"], /* Hangul Syllable */
  ["\uBB46", "\uBB46"], /* Hangul Syllable */
  ["\uBB47", "\uBB47"], /* Hangul Syllable */
  ["\uBB48", "\uBB48"], /* Hangul Syllable */
  ["\uBB49", "\uBB49"], /* Hangul Syllable */
  ["\uBB4A", "\uBB4A"], /* Hangul Syllable */
  ["\uBB4B", "\uBB4B"], /* Hangul Syllable */
  ["\uBB4C", "\uBB4C"], /* Hangul Syllable */
  ["\uBB4D", "\uBB4D"], /* Hangul Syllable */
  ["\uBB4E", "\uBB4E"], /* Hangul Syllable */
  ["\uBB4F", "\uBB4F"], /* Hangul Syllable */
  ["\uBB50", "\uBB50"], /* Hangul Syllable */
  ["\uBB51", "\uBB51"], /* Hangul Syllable */
  ["\uBB52", "\uBB52"], /* Hangul Syllable */
  ["\uBB53", "\uBB53"], /* Hangul Syllable */
  ["\uBB54", "\uBB54"], /* Hangul Syllable */
  ["\uBB55", "\uBB55"], /* Hangul Syllable */
  ["\uBB56", "\uBB56"], /* Hangul Syllable */
  ["\uBB57", "\uBB57"], /* Hangul Syllable */
  ["\uBB58", "\uBB58"], /* Hangul Syllable */
  ["\uBB59", "\uBB59"], /* Hangul Syllable */
  ["\uBB5A", "\uBB5A"], /* Hangul Syllable */
  ["\uBB5B", "\uBB5B"], /* Hangul Syllable */
  ["\uBB5C", "\uBB5C"], /* Hangul Syllable */
  ["\uBB5D", "\uBB5D"], /* Hangul Syllable */
  ["\uBB5E", "\uBB5E"], /* Hangul Syllable */
  ["\uBB5F", "\uBB5F"], /* Hangul Syllable */
  ["\uBB60", "\uBB60"], /* Hangul Syllable */
  ["\uBB61", "\uBB61"], /* Hangul Syllable */
  ["\uBB62", "\uBB62"], /* Hangul Syllable */
  ["\uBB63", "\uBB63"], /* Hangul Syllable */
  ["\uBB64", "\uBB64"], /* Hangul Syllable */
  ["\uBB65", "\uBB65"], /* Hangul Syllable */
  ["\uBB66", "\uBB66"], /* Hangul Syllable */
  ["\uBB67", "\uBB67"], /* Hangul Syllable */
  ["\uBB68", "\uBB68"], /* Hangul Syllable */
  ["\uBB69", "\uBB69"], /* Hangul Syllable */
  ["\uBB6A", "\uBB6A"], /* Hangul Syllable */
  ["\uBB6B", "\uBB6B"], /* Hangul Syllable */
  ["\uBB6C", "\uBB6C"], /* Hangul Syllable */
  ["\uBB6D", "\uBB6D"], /* Hangul Syllable */
  ["\uBB6E", "\uBB6E"], /* Hangul Syllable */
  ["\uBB6F", "\uBB6F"], /* Hangul Syllable */
  ["\uBB70", "\uBB70"], /* Hangul Syllable */
  ["\uBB71", "\uBB71"], /* Hangul Syllable */
  ["\uBB72", "\uBB72"], /* Hangul Syllable */
  ["\uBB73", "\uBB73"], /* Hangul Syllable */
  ["\uBB74", "\uBB74"], /* Hangul Syllable */
  ["\uBB75", "\uBB75"], /* Hangul Syllable */
  ["\uBB76", "\uBB76"], /* Hangul Syllable */
  ["\uBB77", "\uBB77"], /* Hangul Syllable */
  ["\uBB78", "\uBB78"], /* Hangul Syllable */
  ["\uBB79", "\uBB79"], /* Hangul Syllable */
  ["\uBB7A", "\uBB7A"], /* Hangul Syllable */
  ["\uBB7B", "\uBB7B"], /* Hangul Syllable */
  ["\uBB7C", "\uBB7C"], /* Hangul Syllable */
  ["\uBB7D", "\uBB7D"], /* Hangul Syllable */
  ["\uBB7E", "\uBB7E"], /* Hangul Syllable */
  ["\uBB7F", "\uBB7F"], /* Hangul Syllable */
  ["\uBB80", "\uBB80"], /* Hangul Syllable */
  ["\uBB81", "\uBB81"], /* Hangul Syllable */
  ["\uBB82", "\uBB82"], /* Hangul Syllable */
  ["\uBB83", "\uBB83"], /* Hangul Syllable */
  ["\uBB84", "\uBB84"], /* Hangul Syllable */
  ["\uBB85", "\uBB85"], /* Hangul Syllable */
  ["\uBB86", "\uBB86"], /* Hangul Syllable */
  ["\uBB87", "\uBB87"], /* Hangul Syllable */
  ["\uBB88", "\uBB88"], /* Hangul Syllable */
  ["\uBB89", "\uBB89"], /* Hangul Syllable */
  ["\uBB8A", "\uBB8A"], /* Hangul Syllable */
  ["\uBB8B", "\uBB8B"], /* Hangul Syllable */
  ["\uBB8C", "\uBB8C"], /* Hangul Syllable */
  ["\uBB8D", "\uBB8D"], /* Hangul Syllable */
  ["\uBB8E", "\uBB8E"], /* Hangul Syllable */
  ["\uBB8F", "\uBB8F"], /* Hangul Syllable */
  ["\uBB90", "\uBB90"], /* Hangul Syllable */
  ["\uBB91", "\uBB91"], /* Hangul Syllable */
  ["\uBB92", "\uBB92"], /* Hangul Syllable */
  ["\uBB93", "\uBB93"], /* Hangul Syllable */
  ["\uBB94", "\uBB94"], /* Hangul Syllable */
  ["\uBB95", "\uBB95"], /* Hangul Syllable */
  ["\uBB96", "\uBB96"], /* Hangul Syllable */
  ["\uBB97", "\uBB97"], /* Hangul Syllable */
  ["\uBB98", "\uBB98"], /* Hangul Syllable */
  ["\uBB99", "\uBB99"], /* Hangul Syllable */
  ["\uBB9A", "\uBB9A"], /* Hangul Syllable */
  ["\uBB9B", "\uBB9B"], /* Hangul Syllable */
  ["\uBB9C", "\uBB9C"], /* Hangul Syllable */
  ["\uBB9D", "\uBB9D"], /* Hangul Syllable */
  ["\uBB9E", "\uBB9E"], /* Hangul Syllable */
  ["\uBB9F", "\uBB9F"], /* Hangul Syllable */
  ["\uBBA0", "\uBBA0"], /* Hangul Syllable */
  ["\uBBA1", "\uBBA1"], /* Hangul Syllable */
  ["\uBBA2", "\uBBA2"], /* Hangul Syllable */
  ["\uBBA3", "\uBBA3"], /* Hangul Syllable */
  ["\uBBA4", "\uBBA4"], /* Hangul Syllable */
  ["\uBBA5", "\uBBA5"], /* Hangul Syllable */
  ["\uBBA6", "\uBBA6"], /* Hangul Syllable */
  ["\uBBA7", "\uBBA7"], /* Hangul Syllable */
  ["\uBBA8", "\uBBA8"], /* Hangul Syllable */
  ["\uBBA9", "\uBBA9"], /* Hangul Syllable */
  ["\uBBAA", "\uBBAA"], /* Hangul Syllable */
  ["\uBBAB", "\uBBAB"], /* Hangul Syllable */
  ["\uBBAC", "\uBBAC"], /* Hangul Syllable */
  ["\uBBAD", "\uBBAD"], /* Hangul Syllable */
  ["\uBBAE", "\uBBAE"], /* Hangul Syllable */
  ["\uBBAF", "\uBBAF"], /* Hangul Syllable */
  ["\uBBB0", "\uBBB0"], /* Hangul Syllable */
  ["\uBBB1", "\uBBB1"], /* Hangul Syllable */
  ["\uBBB2", "\uBBB2"], /* Hangul Syllable */
  ["\uBBB3", "\uBBB3"], /* Hangul Syllable */
  ["\uBBB4", "\uBBB4"], /* Hangul Syllable */
  ["\uBBB5", "\uBBB5"], /* Hangul Syllable */
  ["\uBBB6", "\uBBB6"], /* Hangul Syllable */
  ["\uBBB7", "\uBBB7"], /* Hangul Syllable */
  ["\uBBB8", "\uBBB8"], /* Hangul Syllable */
  ["\uBBB9", "\uBBB9"], /* Hangul Syllable */
  ["\uBBBA", "\uBBBA"], /* Hangul Syllable */
  ["\uBBBB", "\uBBBB"], /* Hangul Syllable */
  ["\uBBBC", "\uBBBC"], /* Hangul Syllable */
  ["\uBBBD", "\uBBBD"], /* Hangul Syllable */
  ["\uBBBE", "\uBBBE"], /* Hangul Syllable */
  ["\uBBBF", "\uBBBF"], /* Hangul Syllable */
  ["\uBBC0", "\uBBC0"], /* Hangul Syllable */
  ["\uBBC1", "\uBBC1"], /* Hangul Syllable */
  ["\uBBC2", "\uBBC2"], /* Hangul Syllable */
  ["\uBBC3", "\uBBC3"], /* Hangul Syllable */
  ["\uBBC4", "\uBBC4"], /* Hangul Syllable */
  ["\uBBC5", "\uBBC5"], /* Hangul Syllable */
  ["\uBBC6", "\uBBC6"], /* Hangul Syllable */
  ["\uBBC7", "\uBBC7"], /* Hangul Syllable */
  ["\uBBC8", "\uBBC8"], /* Hangul Syllable */
  ["\uBBC9", "\uBBC9"], /* Hangul Syllable */
  ["\uBBCA", "\uBBCA"], /* Hangul Syllable */
  ["\uBBCB", "\uBBCB"], /* Hangul Syllable */
  ["\uBBCC", "\uBBCC"], /* Hangul Syllable */
  ["\uBBCD", "\uBBCD"], /* Hangul Syllable */
  ["\uBBCE", "\uBBCE"], /* Hangul Syllable */
  ["\uBBCF", "\uBBCF"], /* Hangul Syllable */
  ["\uBBD0", "\uBBD0"], /* Hangul Syllable */
  ["\uBBD1", "\uBBD1"], /* Hangul Syllable */
  ["\uBBD2", "\uBBD2"], /* Hangul Syllable */
  ["\uBBD3", "\uBBD3"], /* Hangul Syllable */
  ["\uBBD4", "\uBBD4"], /* Hangul Syllable */
  ["\uBBD5", "\uBBD5"], /* Hangul Syllable */
  ["\uBBD6", "\uBBD6"], /* Hangul Syllable */
  ["\uBBD7", "\uBBD7"], /* Hangul Syllable */
  ["\uBBD8", "\uBBD8"], /* Hangul Syllable */
  ["\uBBD9", "\uBBD9"], /* Hangul Syllable */
  ["\uBBDA", "\uBBDA"], /* Hangul Syllable */
  ["\uBBDB", "\uBBDB"], /* Hangul Syllable */
  ["\uBBDC", "\uBBDC"], /* Hangul Syllable */
  ["\uBBDD", "\uBBDD"], /* Hangul Syllable */
  ["\uBBDE", "\uBBDE"], /* Hangul Syllable */
  ["\uBBDF", "\uBBDF"], /* Hangul Syllable */
  ["\uBBE0", "\uBBE0"], /* Hangul Syllable */
  ["\uBBE1", "\uBBE1"], /* Hangul Syllable */
  ["\uBBE2", "\uBBE2"], /* Hangul Syllable */
  ["\uBBE3", "\uBBE3"], /* Hangul Syllable */
  ["\uBBE4", "\uBBE4"], /* Hangul Syllable */
  ["\uBBE5", "\uBBE5"], /* Hangul Syllable */
  ["\uBBE6", "\uBBE6"], /* Hangul Syllable */
  ["\uBBE7", "\uBBE7"], /* Hangul Syllable */
  ["\uBBE8", "\uBBE8"], /* Hangul Syllable */
  ["\uBBE9", "\uBBE9"], /* Hangul Syllable */
  ["\uBBEA", "\uBBEA"], /* Hangul Syllable */
  ["\uBBEB", "\uBBEB"], /* Hangul Syllable */
  ["\uBBEC", "\uBBEC"], /* Hangul Syllable */
  ["\uBBED", "\uBBED"], /* Hangul Syllable */
  ["\uBBEE", "\uBBEE"], /* Hangul Syllable */
  ["\uBBEF", "\uBBEF"], /* Hangul Syllable */
  ["\uBBF0", "\uBBF0"], /* Hangul Syllable */
  ["\uBBF1", "\uBBF1"], /* Hangul Syllable */
  ["\uBBF2", "\uBBF2"], /* Hangul Syllable */
  ["\uBBF3", "\uBBF3"], /* Hangul Syllable */
  ["\uBBF4", "\uBBF4"], /* Hangul Syllable */
  ["\uBBF5", "\uBBF5"], /* Hangul Syllable */
  ["\uBBF6", "\uBBF6"], /* Hangul Syllable */
  ["\uBBF7", "\uBBF7"], /* Hangul Syllable */
  ["\uBBF8", "\uBBF8"], /* Hangul Syllable */
  ["\uBBF9", "\uBBF9"], /* Hangul Syllable */
  ["\uBBFA", "\uBBFA"], /* Hangul Syllable */
  ["\uBBFB", "\uBBFB"], /* Hangul Syllable */
  ["\uBBFC", "\uBBFC"], /* Hangul Syllable */
  ["\uBBFD", "\uBBFD"], /* Hangul Syllable */
  ["\uBBFE", "\uBBFE"], /* Hangul Syllable */
  ["\uBBFF", "\uBBFF"], /* Hangul Syllable */
  ["\uBC00", "\uBC00"], /* Hangul Syllable */
  ["\uBC01", "\uBC01"], /* Hangul Syllable */
  ["\uBC02", "\uBC02"], /* Hangul Syllable */
  ["\uBC03", "\uBC03"], /* Hangul Syllable */
  ["\uBC04", "\uBC04"], /* Hangul Syllable */
  ["\uBC05", "\uBC05"], /* Hangul Syllable */
  ["\uBC06", "\uBC06"], /* Hangul Syllable */
  ["\uBC07", "\uBC07"], /* Hangul Syllable */
  ["\uBC08", "\uBC08"], /* Hangul Syllable */
  ["\uBC09", "\uBC09"], /* Hangul Syllable */
  ["\uBC0A", "\uBC0A"], /* Hangul Syllable */
  ["\uBC0B", "\uBC0B"], /* Hangul Syllable */
  ["\uBC0C", "\uBC0C"], /* Hangul Syllable */
  ["\uBC0D", "\uBC0D"], /* Hangul Syllable */
  ["\uBC0E", "\uBC0E"], /* Hangul Syllable */
  ["\uBC0F", "\uBC0F"], /* Hangul Syllable */
  ["\uBC10", "\uBC10"], /* Hangul Syllable */
  ["\uBC11", "\uBC11"], /* Hangul Syllable */
  ["\uBC12", "\uBC12"], /* Hangul Syllable */
  ["\uBC13", "\uBC13"], /* Hangul Syllable */
  ["\uBC14", "\uBC14"], /* Hangul Syllable */
  ["\uBC15", "\uBC15"], /* Hangul Syllable */
  ["\uBC16", "\uBC16"], /* Hangul Syllable */
  ["\uBC17", "\uBC17"], /* Hangul Syllable */
  ["\uBC18", "\uBC18"], /* Hangul Syllable */
  ["\uBC19", "\uBC19"], /* Hangul Syllable */
  ["\uBC1A", "\uBC1A"], /* Hangul Syllable */
  ["\uBC1B", "\uBC1B"], /* Hangul Syllable */
  ["\uBC1C", "\uBC1C"], /* Hangul Syllable */
  ["\uBC1D", "\uBC1D"], /* Hangul Syllable */
  ["\uBC1E", "\uBC1E"], /* Hangul Syllable */
  ["\uBC1F", "\uBC1F"], /* Hangul Syllable */
  ["\uBC20", "\uBC20"], /* Hangul Syllable */
  ["\uBC21", "\uBC21"], /* Hangul Syllable */
  ["\uBC22", "\uBC22"], /* Hangul Syllable */
  ["\uBC23", "\uBC23"], /* Hangul Syllable */
  ["\uBC24", "\uBC24"], /* Hangul Syllable */
  ["\uBC25", "\uBC25"], /* Hangul Syllable */
  ["\uBC26", "\uBC26"], /* Hangul Syllable */
  ["\uBC27", "\uBC27"], /* Hangul Syllable */
  ["\uBC28", "\uBC28"], /* Hangul Syllable */
  ["\uBC29", "\uBC29"], /* Hangul Syllable */
  ["\uBC2A", "\uBC2A"], /* Hangul Syllable */
  ["\uBC2B", "\uBC2B"], /* Hangul Syllable */
  ["\uBC2C", "\uBC2C"], /* Hangul Syllable */
  ["\uBC2D", "\uBC2D"], /* Hangul Syllable */
  ["\uBC2E", "\uBC2E"], /* Hangul Syllable */
  ["\uBC2F", "\uBC2F"], /* Hangul Syllable */
  ["\uBC30", "\uBC30"], /* Hangul Syllable */
  ["\uBC31", "\uBC31"], /* Hangul Syllable */
  ["\uBC32", "\uBC32"], /* Hangul Syllable */
  ["\uBC33", "\uBC33"], /* Hangul Syllable */
  ["\uBC34", "\uBC34"], /* Hangul Syllable */
  ["\uBC35", "\uBC35"], /* Hangul Syllable */
  ["\uBC36", "\uBC36"], /* Hangul Syllable */
  ["\uBC37", "\uBC37"], /* Hangul Syllable */
  ["\uBC38", "\uBC38"], /* Hangul Syllable */
  ["\uBC39", "\uBC39"], /* Hangul Syllable */
  ["\uBC3A", "\uBC3A"], /* Hangul Syllable */
  ["\uBC3B", "\uBC3B"], /* Hangul Syllable */
  ["\uBC3C", "\uBC3C"], /* Hangul Syllable */
  ["\uBC3D", "\uBC3D"], /* Hangul Syllable */
  ["\uBC3E", "\uBC3E"], /* Hangul Syllable */
  ["\uBC3F", "\uBC3F"], /* Hangul Syllable */
  ["\uBC40", "\uBC40"], /* Hangul Syllable */
  ["\uBC41", "\uBC41"], /* Hangul Syllable */
  ["\uBC42", "\uBC42"], /* Hangul Syllable */
  ["\uBC43", "\uBC43"], /* Hangul Syllable */
  ["\uBC44", "\uBC44"], /* Hangul Syllable */
  ["\uBC45", "\uBC45"], /* Hangul Syllable */
  ["\uBC46", "\uBC46"], /* Hangul Syllable */
  ["\uBC47", "\uBC47"], /* Hangul Syllable */
  ["\uBC48", "\uBC48"], /* Hangul Syllable */
  ["\uBC49", "\uBC49"], /* Hangul Syllable */
  ["\uBC4A", "\uBC4A"], /* Hangul Syllable */
  ["\uBC4B", "\uBC4B"], /* Hangul Syllable */
  ["\uBC4C", "\uBC4C"], /* Hangul Syllable */
  ["\uBC4D", "\uBC4D"], /* Hangul Syllable */
  ["\uBC4E", "\uBC4E"], /* Hangul Syllable */
  ["\uBC4F", "\uBC4F"], /* Hangul Syllable */
  ["\uBC50", "\uBC50"], /* Hangul Syllable */
  ["\uBC51", "\uBC51"], /* Hangul Syllable */
  ["\uBC52", "\uBC52"], /* Hangul Syllable */
  ["\uBC53", "\uBC53"], /* Hangul Syllable */
  ["\uBC54", "\uBC54"], /* Hangul Syllable */
  ["\uBC55", "\uBC55"], /* Hangul Syllable */
  ["\uBC56", "\uBC56"], /* Hangul Syllable */
  ["\uBC57", "\uBC57"], /* Hangul Syllable */
  ["\uBC58", "\uBC58"], /* Hangul Syllable */
  ["\uBC59", "\uBC59"], /* Hangul Syllable */
  ["\uBC5A", "\uBC5A"], /* Hangul Syllable */
  ["\uBC5B", "\uBC5B"], /* Hangul Syllable */
  ["\uBC5C", "\uBC5C"], /* Hangul Syllable */
  ["\uBC5D", "\uBC5D"], /* Hangul Syllable */
  ["\uBC5E", "\uBC5E"], /* Hangul Syllable */
  ["\uBC5F", "\uBC5F"], /* Hangul Syllable */
  ["\uBC60", "\uBC60"], /* Hangul Syllable */
  ["\uBC61", "\uBC61"], /* Hangul Syllable */
  ["\uBC62", "\uBC62"], /* Hangul Syllable */
  ["\uBC63", "\uBC63"], /* Hangul Syllable */
  ["\uBC64", "\uBC64"], /* Hangul Syllable */
  ["\uBC65", "\uBC65"], /* Hangul Syllable */
  ["\uBC66", "\uBC66"], /* Hangul Syllable */
  ["\uBC67", "\uBC67"], /* Hangul Syllable */
  ["\uBC68", "\uBC68"], /* Hangul Syllable */
  ["\uBC69", "\uBC69"], /* Hangul Syllable */
  ["\uBC6A", "\uBC6A"], /* Hangul Syllable */
  ["\uBC6B", "\uBC6B"], /* Hangul Syllable */
  ["\uBC6C", "\uBC6C"], /* Hangul Syllable */
  ["\uBC6D", "\uBC6D"], /* Hangul Syllable */
  ["\uBC6E", "\uBC6E"], /* Hangul Syllable */
  ["\uBC6F", "\uBC6F"], /* Hangul Syllable */
  ["\uBC70", "\uBC70"], /* Hangul Syllable */
  ["\uBC71", "\uBC71"], /* Hangul Syllable */
  ["\uBC72", "\uBC72"], /* Hangul Syllable */
  ["\uBC73", "\uBC73"], /* Hangul Syllable */
  ["\uBC74", "\uBC74"], /* Hangul Syllable */
  ["\uBC75", "\uBC75"], /* Hangul Syllable */
  ["\uBC76", "\uBC76"], /* Hangul Syllable */
  ["\uBC77", "\uBC77"], /* Hangul Syllable */
  ["\uBC78", "\uBC78"], /* Hangul Syllable */
  ["\uBC79", "\uBC79"], /* Hangul Syllable */
  ["\uBC7A", "\uBC7A"], /* Hangul Syllable */
  ["\uBC7B", "\uBC7B"], /* Hangul Syllable */
  ["\uBC7C", "\uBC7C"], /* Hangul Syllable */
  ["\uBC7D", "\uBC7D"], /* Hangul Syllable */
  ["\uBC7E", "\uBC7E"], /* Hangul Syllable */
  ["\uBC7F", "\uBC7F"], /* Hangul Syllable */
  ["\uBC80", "\uBC80"], /* Hangul Syllable */
  ["\uBC81", "\uBC81"], /* Hangul Syllable */
  ["\uBC82", "\uBC82"], /* Hangul Syllable */
  ["\uBC83", "\uBC83"], /* Hangul Syllable */
  ["\uBC84", "\uBC84"], /* Hangul Syllable */
  ["\uBC85", "\uBC85"], /* Hangul Syllable */
  ["\uBC86", "\uBC86"], /* Hangul Syllable */
  ["\uBC87", "\uBC87"], /* Hangul Syllable */
  ["\uBC88", "\uBC88"], /* Hangul Syllable */
  ["\uBC89", "\uBC89"], /* Hangul Syllable */
  ["\uBC8A", "\uBC8A"], /* Hangul Syllable */
  ["\uBC8B", "\uBC8B"], /* Hangul Syllable */
  ["\uBC8C", "\uBC8C"], /* Hangul Syllable */
  ["\uBC8D", "\uBC8D"], /* Hangul Syllable */
  ["\uBC8E", "\uBC8E"], /* Hangul Syllable */
  ["\uBC8F", "\uBC8F"], /* Hangul Syllable */
  ["\uBC90", "\uBC90"], /* Hangul Syllable */
  ["\uBC91", "\uBC91"], /* Hangul Syllable */
  ["\uBC92", "\uBC92"], /* Hangul Syllable */
  ["\uBC93", "\uBC93"], /* Hangul Syllable */
  ["\uBC94", "\uBC94"], /* Hangul Syllable */
  ["\uBC95", "\uBC95"], /* Hangul Syllable */
  ["\uBC96", "\uBC96"], /* Hangul Syllable */
  ["\uBC97", "\uBC97"], /* Hangul Syllable */
  ["\uBC98", "\uBC98"], /* Hangul Syllable */
  ["\uBC99", "\uBC99"], /* Hangul Syllable */
  ["\uBC9A", "\uBC9A"], /* Hangul Syllable */
  ["\uBC9B", "\uBC9B"], /* Hangul Syllable */
  ["\uBC9C", "\uBC9C"], /* Hangul Syllable */
  ["\uBC9D", "\uBC9D"], /* Hangul Syllable */
  ["\uBC9E", "\uBC9E"], /* Hangul Syllable */
  ["\uBC9F", "\uBC9F"], /* Hangul Syllable */
  ["\uBCA0", "\uBCA0"], /* Hangul Syllable */
  ["\uBCA1", "\uBCA1"], /* Hangul Syllable */
  ["\uBCA2", "\uBCA2"], /* Hangul Syllable */
  ["\uBCA3", "\uBCA3"], /* Hangul Syllable */
  ["\uBCA4", "\uBCA4"], /* Hangul Syllable */
  ["\uBCA5", "\uBCA5"], /* Hangul Syllable */
  ["\uBCA6", "\uBCA6"], /* Hangul Syllable */
  ["\uBCA7", "\uBCA7"], /* Hangul Syllable */
  ["\uBCA8", "\uBCA8"], /* Hangul Syllable */
  ["\uBCA9", "\uBCA9"], /* Hangul Syllable */
  ["\uBCAA", "\uBCAA"], /* Hangul Syllable */
  ["\uBCAB", "\uBCAB"], /* Hangul Syllable */
  ["\uBCAC", "\uBCAC"], /* Hangul Syllable */
  ["\uBCAD", "\uBCAD"], /* Hangul Syllable */
  ["\uBCAE", "\uBCAE"], /* Hangul Syllable */
  ["\uBCAF", "\uBCAF"], /* Hangul Syllable */
  ["\uBCB0", "\uBCB0"], /* Hangul Syllable */
  ["\uBCB1", "\uBCB1"], /* Hangul Syllable */
  ["\uBCB2", "\uBCB2"], /* Hangul Syllable */
  ["\uBCB3", "\uBCB3"], /* Hangul Syllable */
  ["\uBCB4", "\uBCB4"], /* Hangul Syllable */
  ["\uBCB5", "\uBCB5"], /* Hangul Syllable */
  ["\uBCB6", "\uBCB6"], /* Hangul Syllable */
  ["\uBCB7", "\uBCB7"], /* Hangul Syllable */
  ["\uBCB8", "\uBCB8"], /* Hangul Syllable */
  ["\uBCB9", "\uBCB9"], /* Hangul Syllable */
  ["\uBCBA", "\uBCBA"], /* Hangul Syllable */
  ["\uBCBB", "\uBCBB"], /* Hangul Syllable */
  ["\uBCBC", "\uBCBC"], /* Hangul Syllable */
  ["\uBCBD", "\uBCBD"], /* Hangul Syllable */
  ["\uBCBE", "\uBCBE"], /* Hangul Syllable */
  ["\uBCBF", "\uBCBF"], /* Hangul Syllable */
  ["\uBCC0", "\uBCC0"], /* Hangul Syllable */
  ["\uBCC1", "\uBCC1"], /* Hangul Syllable */
  ["\uBCC2", "\uBCC2"], /* Hangul Syllable */
  ["\uBCC3", "\uBCC3"], /* Hangul Syllable */
  ["\uBCC4", "\uBCC4"], /* Hangul Syllable */
  ["\uBCC5", "\uBCC5"], /* Hangul Syllable */
  ["\uBCC6", "\uBCC6"], /* Hangul Syllable */
  ["\uBCC7", "\uBCC7"], /* Hangul Syllable */
  ["\uBCC8", "\uBCC8"], /* Hangul Syllable */
  ["\uBCC9", "\uBCC9"], /* Hangul Syllable */
  ["\uBCCA", "\uBCCA"], /* Hangul Syllable */
  ["\uBCCB", "\uBCCB"], /* Hangul Syllable */
  ["\uBCCC", "\uBCCC"], /* Hangul Syllable */
  ["\uBCCD", "\uBCCD"], /* Hangul Syllable */
  ["\uBCCE", "\uBCCE"], /* Hangul Syllable */
  ["\uBCCF", "\uBCCF"], /* Hangul Syllable */
  ["\uBCD0", "\uBCD0"], /* Hangul Syllable */
  ["\uBCD1", "\uBCD1"], /* Hangul Syllable */
  ["\uBCD2", "\uBCD2"], /* Hangul Syllable */
  ["\uBCD3", "\uBCD3"], /* Hangul Syllable */
  ["\uBCD4", "\uBCD4"], /* Hangul Syllable */
  ["\uBCD5", "\uBCD5"], /* Hangul Syllable */
  ["\uBCD6", "\uBCD6"], /* Hangul Syllable */
  ["\uBCD7", "\uBCD7"], /* Hangul Syllable */
  ["\uBCD8", "\uBCD8"], /* Hangul Syllable */
  ["\uBCD9", "\uBCD9"], /* Hangul Syllable */
  ["\uBCDA", "\uBCDA"], /* Hangul Syllable */
  ["\uBCDB", "\uBCDB"], /* Hangul Syllable */
  ["\uBCDC", "\uBCDC"], /* Hangul Syllable */
  ["\uBCDD", "\uBCDD"], /* Hangul Syllable */
  ["\uBCDE", "\uBCDE"], /* Hangul Syllable */
  ["\uBCDF", "\uBCDF"], /* Hangul Syllable */
  ["\uBCE0", "\uBCE0"], /* Hangul Syllable */
  ["\uBCE1", "\uBCE1"], /* Hangul Syllable */
  ["\uBCE2", "\uBCE2"], /* Hangul Syllable */
  ["\uBCE3", "\uBCE3"], /* Hangul Syllable */
  ["\uBCE4", "\uBCE4"], /* Hangul Syllable */
  ["\uBCE5", "\uBCE5"], /* Hangul Syllable */
  ["\uBCE6", "\uBCE6"], /* Hangul Syllable */
  ["\uBCE7", "\uBCE7"], /* Hangul Syllable */
  ["\uBCE8", "\uBCE8"], /* Hangul Syllable */
  ["\uBCE9", "\uBCE9"], /* Hangul Syllable */
  ["\uBCEA", "\uBCEA"], /* Hangul Syllable */
  ["\uBCEB", "\uBCEB"], /* Hangul Syllable */
  ["\uBCEC", "\uBCEC"], /* Hangul Syllable */
  ["\uBCED", "\uBCED"], /* Hangul Syllable */
  ["\uBCEE", "\uBCEE"], /* Hangul Syllable */
  ["\uBCEF", "\uBCEF"], /* Hangul Syllable */
  ["\uBCF0", "\uBCF0"], /* Hangul Syllable */
  ["\uBCF1", "\uBCF1"], /* Hangul Syllable */
  ["\uBCF2", "\uBCF2"], /* Hangul Syllable */
  ["\uBCF3", "\uBCF3"], /* Hangul Syllable */
  ["\uBCF4", "\uBCF4"], /* Hangul Syllable */
  ["\uBCF5", "\uBCF5"], /* Hangul Syllable */
  ["\uBCF6", "\uBCF6"], /* Hangul Syllable */
  ["\uBCF7", "\uBCF7"], /* Hangul Syllable */
  ["\uBCF8", "\uBCF8"], /* Hangul Syllable */
  ["\uBCF9", "\uBCF9"], /* Hangul Syllable */
  ["\uBCFA", "\uBCFA"], /* Hangul Syllable */
  ["\uBCFB", "\uBCFB"], /* Hangul Syllable */
  ["\uBCFC", "\uBCFC"], /* Hangul Syllable */
  ["\uBCFD", "\uBCFD"], /* Hangul Syllable */
  ["\uBCFE", "\uBCFE"], /* Hangul Syllable */
  ["\uBCFF", "\uBCFF"], /* Hangul Syllable */
  ["\uBD00", "\uBD00"], /* Hangul Syllable */
  ["\uBD01", "\uBD01"], /* Hangul Syllable */
  ["\uBD02", "\uBD02"], /* Hangul Syllable */
  ["\uBD03", "\uBD03"], /* Hangul Syllable */
  ["\uBD04", "\uBD04"], /* Hangul Syllable */
  ["\uBD05", "\uBD05"], /* Hangul Syllable */
  ["\uBD06", "\uBD06"], /* Hangul Syllable */
  ["\uBD07", "\uBD07"], /* Hangul Syllable */
  ["\uBD08", "\uBD08"], /* Hangul Syllable */
  ["\uBD09", "\uBD09"], /* Hangul Syllable */
  ["\uBD0A", "\uBD0A"], /* Hangul Syllable */
  ["\uBD0B", "\uBD0B"], /* Hangul Syllable */
  ["\uBD0C", "\uBD0C"], /* Hangul Syllable */
  ["\uBD0D", "\uBD0D"], /* Hangul Syllable */
  ["\uBD0E", "\uBD0E"], /* Hangul Syllable */
  ["\uBD0F", "\uBD0F"], /* Hangul Syllable */
  ["\uBD10", "\uBD10"], /* Hangul Syllable */
  ["\uBD11", "\uBD11"], /* Hangul Syllable */
  ["\uBD12", "\uBD12"], /* Hangul Syllable */
  ["\uBD13", "\uBD13"], /* Hangul Syllable */
  ["\uBD14", "\uBD14"], /* Hangul Syllable */
  ["\uBD15", "\uBD15"], /* Hangul Syllable */
  ["\uBD16", "\uBD16"], /* Hangul Syllable */
  ["\uBD17", "\uBD17"], /* Hangul Syllable */
  ["\uBD18", "\uBD18"], /* Hangul Syllable */
  ["\uBD19", "\uBD19"], /* Hangul Syllable */
  ["\uBD1A", "\uBD1A"], /* Hangul Syllable */
  ["\uBD1B", "\uBD1B"], /* Hangul Syllable */
  ["\uBD1C", "\uBD1C"], /* Hangul Syllable */
  ["\uBD1D", "\uBD1D"], /* Hangul Syllable */
  ["\uBD1E", "\uBD1E"], /* Hangul Syllable */
  ["\uBD1F", "\uBD1F"], /* Hangul Syllable */
  ["\uBD20", "\uBD20"], /* Hangul Syllable */
  ["\uBD21", "\uBD21"], /* Hangul Syllable */
  ["\uBD22", "\uBD22"], /* Hangul Syllable */
  ["\uBD23", "\uBD23"], /* Hangul Syllable */
  ["\uBD24", "\uBD24"], /* Hangul Syllable */
  ["\uBD25", "\uBD25"], /* Hangul Syllable */
  ["\uBD26", "\uBD26"], /* Hangul Syllable */
  ["\uBD27", "\uBD27"], /* Hangul Syllable */
  ["\uBD28", "\uBD28"], /* Hangul Syllable */
  ["\uBD29", "\uBD29"], /* Hangul Syllable */
  ["\uBD2A", "\uBD2A"], /* Hangul Syllable */
  ["\uBD2B", "\uBD2B"], /* Hangul Syllable */
  ["\uBD2C", "\uBD2C"], /* Hangul Syllable */
  ["\uBD2D", "\uBD2D"], /* Hangul Syllable */
  ["\uBD2E", "\uBD2E"], /* Hangul Syllable */
  ["\uBD2F", "\uBD2F"], /* Hangul Syllable */
  ["\uBD30", "\uBD30"], /* Hangul Syllable */
  ["\uBD31", "\uBD31"], /* Hangul Syllable */
  ["\uBD32", "\uBD32"], /* Hangul Syllable */
  ["\uBD33", "\uBD33"], /* Hangul Syllable */
  ["\uBD34", "\uBD34"], /* Hangul Syllable */
  ["\uBD35", "\uBD35"], /* Hangul Syllable */
  ["\uBD36", "\uBD36"], /* Hangul Syllable */
  ["\uBD37", "\uBD37"], /* Hangul Syllable */
  ["\uBD38", "\uBD38"], /* Hangul Syllable */
  ["\uBD39", "\uBD39"], /* Hangul Syllable */
  ["\uBD3A", "\uBD3A"], /* Hangul Syllable */
  ["\uBD3B", "\uBD3B"], /* Hangul Syllable */
  ["\uBD3C", "\uBD3C"], /* Hangul Syllable */
  ["\uBD3D", "\uBD3D"], /* Hangul Syllable */
  ["\uBD3E", "\uBD3E"], /* Hangul Syllable */
  ["\uBD3F", "\uBD3F"], /* Hangul Syllable */
  ["\uBD40", "\uBD40"], /* Hangul Syllable */
  ["\uBD41", "\uBD41"], /* Hangul Syllable */
  ["\uBD42", "\uBD42"], /* Hangul Syllable */
  ["\uBD43", "\uBD43"], /* Hangul Syllable */
  ["\uBD44", "\uBD44"], /* Hangul Syllable */
  ["\uBD45", "\uBD45"], /* Hangul Syllable */
  ["\uBD46", "\uBD46"], /* Hangul Syllable */
  ["\uBD47", "\uBD47"], /* Hangul Syllable */
  ["\uBD48", "\uBD48"], /* Hangul Syllable */
  ["\uBD49", "\uBD49"], /* Hangul Syllable */
  ["\uBD4A", "\uBD4A"], /* Hangul Syllable */
  ["\uBD4B", "\uBD4B"], /* Hangul Syllable */
  ["\uBD4C", "\uBD4C"], /* Hangul Syllable */
  ["\uBD4D", "\uBD4D"], /* Hangul Syllable */
  ["\uBD4E", "\uBD4E"], /* Hangul Syllable */
  ["\uBD4F", "\uBD4F"], /* Hangul Syllable */
  ["\uBD50", "\uBD50"], /* Hangul Syllable */
  ["\uBD51", "\uBD51"], /* Hangul Syllable */
  ["\uBD52", "\uBD52"], /* Hangul Syllable */
  ["\uBD53", "\uBD53"], /* Hangul Syllable */
  ["\uBD54", "\uBD54"], /* Hangul Syllable */
  ["\uBD55", "\uBD55"], /* Hangul Syllable */
  ["\uBD56", "\uBD56"], /* Hangul Syllable */
  ["\uBD57", "\uBD57"], /* Hangul Syllable */
  ["\uBD58", "\uBD58"], /* Hangul Syllable */
  ["\uBD59", "\uBD59"], /* Hangul Syllable */
  ["\uBD5A", "\uBD5A"], /* Hangul Syllable */
  ["\uBD5B", "\uBD5B"], /* Hangul Syllable */
  ["\uBD5C", "\uBD5C"], /* Hangul Syllable */
  ["\uBD5D", "\uBD5D"], /* Hangul Syllable */
  ["\uBD5E", "\uBD5E"], /* Hangul Syllable */
  ["\uBD5F", "\uBD5F"], /* Hangul Syllable */
  ["\uBD60", "\uBD60"], /* Hangul Syllable */
  ["\uBD61", "\uBD61"], /* Hangul Syllable */
  ["\uBD62", "\uBD62"], /* Hangul Syllable */
  ["\uBD63", "\uBD63"], /* Hangul Syllable */
  ["\uBD64", "\uBD64"], /* Hangul Syllable */
  ["\uBD65", "\uBD65"], /* Hangul Syllable */
  ["\uBD66", "\uBD66"], /* Hangul Syllable */
  ["\uBD67", "\uBD67"], /* Hangul Syllable */
  ["\uBD68", "\uBD68"], /* Hangul Syllable */
  ["\uBD69", "\uBD69"], /* Hangul Syllable */
  ["\uBD6A", "\uBD6A"], /* Hangul Syllable */
  ["\uBD6B", "\uBD6B"], /* Hangul Syllable */
  ["\uBD6C", "\uBD6C"], /* Hangul Syllable */
  ["\uBD6D", "\uBD6D"], /* Hangul Syllable */
  ["\uBD6E", "\uBD6E"], /* Hangul Syllable */
  ["\uBD6F", "\uBD6F"], /* Hangul Syllable */
  ["\uBD70", "\uBD70"], /* Hangul Syllable */
  ["\uBD71", "\uBD71"], /* Hangul Syllable */
  ["\uBD72", "\uBD72"], /* Hangul Syllable */
  ["\uBD73", "\uBD73"], /* Hangul Syllable */
  ["\uBD74", "\uBD74"], /* Hangul Syllable */
  ["\uBD75", "\uBD75"], /* Hangul Syllable */
  ["\uBD76", "\uBD76"], /* Hangul Syllable */
  ["\uBD77", "\uBD77"], /* Hangul Syllable */
  ["\uBD78", "\uBD78"], /* Hangul Syllable */
  ["\uBD79", "\uBD79"], /* Hangul Syllable */
  ["\uBD7A", "\uBD7A"], /* Hangul Syllable */
  ["\uBD7B", "\uBD7B"], /* Hangul Syllable */
  ["\uBD7C", "\uBD7C"], /* Hangul Syllable */
  ["\uBD7D", "\uBD7D"], /* Hangul Syllable */
  ["\uBD7E", "\uBD7E"], /* Hangul Syllable */
  ["\uBD7F", "\uBD7F"], /* Hangul Syllable */
  ["\uBD80", "\uBD80"], /* Hangul Syllable */
  ["\uBD81", "\uBD81"], /* Hangul Syllable */
  ["\uBD82", "\uBD82"], /* Hangul Syllable */
  ["\uBD83", "\uBD83"], /* Hangul Syllable */
  ["\uBD84", "\uBD84"], /* Hangul Syllable */
  ["\uBD85", "\uBD85"], /* Hangul Syllable */
  ["\uBD86", "\uBD86"], /* Hangul Syllable */
  ["\uBD87", "\uBD87"], /* Hangul Syllable */
  ["\uBD88", "\uBD88"], /* Hangul Syllable */
  ["\uBD89", "\uBD89"], /* Hangul Syllable */
  ["\uBD8A", "\uBD8A"], /* Hangul Syllable */
  ["\uBD8B", "\uBD8B"], /* Hangul Syllable */
  ["\uBD8C", "\uBD8C"], /* Hangul Syllable */
  ["\uBD8D", "\uBD8D"], /* Hangul Syllable */
  ["\uBD8E", "\uBD8E"], /* Hangul Syllable */
  ["\uBD8F", "\uBD8F"], /* Hangul Syllable */
  ["\uBD90", "\uBD90"], /* Hangul Syllable */
  ["\uBD91", "\uBD91"], /* Hangul Syllable */
  ["\uBD92", "\uBD92"], /* Hangul Syllable */
  ["\uBD93", "\uBD93"], /* Hangul Syllable */
  ["\uBD94", "\uBD94"], /* Hangul Syllable */
  ["\uBD95", "\uBD95"], /* Hangul Syllable */
  ["\uBD96", "\uBD96"], /* Hangul Syllable */
  ["\uBD97", "\uBD97"], /* Hangul Syllable */
  ["\uBD98", "\uBD98"], /* Hangul Syllable */
  ["\uBD99", "\uBD99"], /* Hangul Syllable */
  ["\uBD9A", "\uBD9A"], /* Hangul Syllable */
  ["\uBD9B", "\uBD9B"], /* Hangul Syllable */
  ["\uBD9C", "\uBD9C"], /* Hangul Syllable */
  ["\uBD9D", "\uBD9D"], /* Hangul Syllable */
  ["\uBD9E", "\uBD9E"], /* Hangul Syllable */
  ["\uBD9F", "\uBD9F"], /* Hangul Syllable */
  ["\uBDA0", "\uBDA0"], /* Hangul Syllable */
  ["\uBDA1", "\uBDA1"], /* Hangul Syllable */
  ["\uBDA2", "\uBDA2"], /* Hangul Syllable */
  ["\uBDA3", "\uBDA3"], /* Hangul Syllable */
  ["\uBDA4", "\uBDA4"], /* Hangul Syllable */
  ["\uBDA5", "\uBDA5"], /* Hangul Syllable */
  ["\uBDA6", "\uBDA6"], /* Hangul Syllable */
  ["\uBDA7", "\uBDA7"], /* Hangul Syllable */
  ["\uBDA8", "\uBDA8"], /* Hangul Syllable */
  ["\uBDA9", "\uBDA9"], /* Hangul Syllable */
  ["\uBDAA", "\uBDAA"], /* Hangul Syllable */
  ["\uBDAB", "\uBDAB"], /* Hangul Syllable */
  ["\uBDAC", "\uBDAC"], /* Hangul Syllable */
  ["\uBDAD", "\uBDAD"], /* Hangul Syllable */
  ["\uBDAE", "\uBDAE"], /* Hangul Syllable */
  ["\uBDAF", "\uBDAF"], /* Hangul Syllable */
  ["\uBDB0", "\uBDB0"], /* Hangul Syllable */
  ["\uBDB1", "\uBDB1"], /* Hangul Syllable */
  ["\uBDB2", "\uBDB2"], /* Hangul Syllable */
  ["\uBDB3", "\uBDB3"], /* Hangul Syllable */
  ["\uBDB4", "\uBDB4"], /* Hangul Syllable */
  ["\uBDB5", "\uBDB5"], /* Hangul Syllable */
  ["\uBDB6", "\uBDB6"], /* Hangul Syllable */
  ["\uBDB7", "\uBDB7"], /* Hangul Syllable */
  ["\uBDB8", "\uBDB8"], /* Hangul Syllable */
  ["\uBDB9", "\uBDB9"], /* Hangul Syllable */
  ["\uBDBA", "\uBDBA"], /* Hangul Syllable */
  ["\uBDBB", "\uBDBB"], /* Hangul Syllable */
  ["\uBDBC", "\uBDBC"], /* Hangul Syllable */
  ["\uBDBD", "\uBDBD"], /* Hangul Syllable */
  ["\uBDBE", "\uBDBE"], /* Hangul Syllable */
  ["\uBDBF", "\uBDBF"], /* Hangul Syllable */
  ["\uBDC0", "\uBDC0"], /* Hangul Syllable */
  ["\uBDC1", "\uBDC1"], /* Hangul Syllable */
  ["\uBDC2", "\uBDC2"], /* Hangul Syllable */
  ["\uBDC3", "\uBDC3"], /* Hangul Syllable */
  ["\uBDC4", "\uBDC4"], /* Hangul Syllable */
  ["\uBDC5", "\uBDC5"], /* Hangul Syllable */
  ["\uBDC6", "\uBDC6"], /* Hangul Syllable */
  ["\uBDC7", "\uBDC7"], /* Hangul Syllable */
  ["\uBDC8", "\uBDC8"], /* Hangul Syllable */
  ["\uBDC9", "\uBDC9"], /* Hangul Syllable */
  ["\uBDCA", "\uBDCA"], /* Hangul Syllable */
  ["\uBDCB", "\uBDCB"], /* Hangul Syllable */
  ["\uBDCC", "\uBDCC"], /* Hangul Syllable */
  ["\uBDCD", "\uBDCD"], /* Hangul Syllable */
  ["\uBDCE", "\uBDCE"], /* Hangul Syllable */
  ["\uBDCF", "\uBDCF"], /* Hangul Syllable */
  ["\uBDD0", "\uBDD0"], /* Hangul Syllable */
  ["\uBDD1", "\uBDD1"], /* Hangul Syllable */
  ["\uBDD2", "\uBDD2"], /* Hangul Syllable */
  ["\uBDD3", "\uBDD3"], /* Hangul Syllable */
  ["\uBDD4", "\uBDD4"], /* Hangul Syllable */
  ["\uBDD5", "\uBDD5"], /* Hangul Syllable */
  ["\uBDD6", "\uBDD6"], /* Hangul Syllable */
  ["\uBDD7", "\uBDD7"], /* Hangul Syllable */
  ["\uBDD8", "\uBDD8"], /* Hangul Syllable */
  ["\uBDD9", "\uBDD9"], /* Hangul Syllable */
  ["\uBDDA", "\uBDDA"], /* Hangul Syllable */
  ["\uBDDB", "\uBDDB"], /* Hangul Syllable */
  ["\uBDDC", "\uBDDC"], /* Hangul Syllable */
  ["\uBDDD", "\uBDDD"], /* Hangul Syllable */
  ["\uBDDE", "\uBDDE"], /* Hangul Syllable */
  ["\uBDDF", "\uBDDF"], /* Hangul Syllable */
  ["\uBDE0", "\uBDE0"], /* Hangul Syllable */
  ["\uBDE1", "\uBDE1"], /* Hangul Syllable */
  ["\uBDE2", "\uBDE2"], /* Hangul Syllable */
  ["\uBDE3", "\uBDE3"], /* Hangul Syllable */
  ["\uBDE4", "\uBDE4"], /* Hangul Syllable */
  ["\uBDE5", "\uBDE5"], /* Hangul Syllable */
  ["\uBDE6", "\uBDE6"], /* Hangul Syllable */
  ["\uBDE7", "\uBDE7"], /* Hangul Syllable */
  ["\uBDE8", "\uBDE8"], /* Hangul Syllable */
  ["\uBDE9", "\uBDE9"], /* Hangul Syllable */
  ["\uBDEA", "\uBDEA"], /* Hangul Syllable */
  ["\uBDEB", "\uBDEB"], /* Hangul Syllable */
  ["\uBDEC", "\uBDEC"], /* Hangul Syllable */
  ["\uBDED", "\uBDED"], /* Hangul Syllable */
  ["\uBDEE", "\uBDEE"], /* Hangul Syllable */
  ["\uBDEF", "\uBDEF"], /* Hangul Syllable */
  ["\uBDF0", "\uBDF0"], /* Hangul Syllable */
  ["\uBDF1", "\uBDF1"], /* Hangul Syllable */
  ["\uBDF2", "\uBDF2"], /* Hangul Syllable */
  ["\uBDF3", "\uBDF3"], /* Hangul Syllable */
  ["\uBDF4", "\uBDF4"], /* Hangul Syllable */
  ["\uBDF5", "\uBDF5"], /* Hangul Syllable */
  ["\uBDF6", "\uBDF6"], /* Hangul Syllable */
  ["\uBDF7", "\uBDF7"], /* Hangul Syllable */
  ["\uBDF8", "\uBDF8"], /* Hangul Syllable */
  ["\uBDF9", "\uBDF9"], /* Hangul Syllable */
  ["\uBDFA", "\uBDFA"], /* Hangul Syllable */
  ["\uBDFB", "\uBDFB"], /* Hangul Syllable */
  ["\uBDFC", "\uBDFC"], /* Hangul Syllable */
  ["\uBDFD", "\uBDFD"], /* Hangul Syllable */
  ["\uBDFE", "\uBDFE"], /* Hangul Syllable */
  ["\uBDFF", "\uBDFF"], /* Hangul Syllable */
  ["\uBE00", "\uBE00"], /* Hangul Syllable */
  ["\uBE01", "\uBE01"], /* Hangul Syllable */
  ["\uBE02", "\uBE02"], /* Hangul Syllable */
  ["\uBE03", "\uBE03"], /* Hangul Syllable */
  ["\uBE04", "\uBE04"], /* Hangul Syllable */
  ["\uBE05", "\uBE05"], /* Hangul Syllable */
  ["\uBE06", "\uBE06"], /* Hangul Syllable */
  ["\uBE07", "\uBE07"], /* Hangul Syllable */
  ["\uBE08", "\uBE08"], /* Hangul Syllable */
  ["\uBE09", "\uBE09"], /* Hangul Syllable */
  ["\uBE0A", "\uBE0A"], /* Hangul Syllable */
  ["\uBE0B", "\uBE0B"], /* Hangul Syllable */
  ["\uBE0C", "\uBE0C"], /* Hangul Syllable */
  ["\uBE0D", "\uBE0D"], /* Hangul Syllable */
  ["\uBE0E", "\uBE0E"], /* Hangul Syllable */
  ["\uBE0F", "\uBE0F"], /* Hangul Syllable */
  ["\uBE10", "\uBE10"], /* Hangul Syllable */
  ["\uBE11", "\uBE11"], /* Hangul Syllable */
  ["\uBE12", "\uBE12"], /* Hangul Syllable */
  ["\uBE13", "\uBE13"], /* Hangul Syllable */
  ["\uBE14", "\uBE14"], /* Hangul Syllable */
  ["\uBE15", "\uBE15"], /* Hangul Syllable */
  ["\uBE16", "\uBE16"], /* Hangul Syllable */
  ["\uBE17", "\uBE17"], /* Hangul Syllable */
  ["\uBE18", "\uBE18"], /* Hangul Syllable */
  ["\uBE19", "\uBE19"], /* Hangul Syllable */
  ["\uBE1A", "\uBE1A"], /* Hangul Syllable */
  ["\uBE1B", "\uBE1B"], /* Hangul Syllable */
  ["\uBE1C", "\uBE1C"], /* Hangul Syllable */
  ["\uBE1D", "\uBE1D"], /* Hangul Syllable */
  ["\uBE1E", "\uBE1E"], /* Hangul Syllable */
  ["\uBE1F", "\uBE1F"], /* Hangul Syllable */
  ["\uBE20", "\uBE20"], /* Hangul Syllable */
  ["\uBE21", "\uBE21"], /* Hangul Syllable */
  ["\uBE22", "\uBE22"], /* Hangul Syllable */
  ["\uBE23", "\uBE23"], /* Hangul Syllable */
  ["\uBE24", "\uBE24"], /* Hangul Syllable */
  ["\uBE25", "\uBE25"], /* Hangul Syllable */
  ["\uBE26", "\uBE26"], /* Hangul Syllable */
  ["\uBE27", "\uBE27"], /* Hangul Syllable */
  ["\uBE28", "\uBE28"], /* Hangul Syllable */
  ["\uBE29", "\uBE29"], /* Hangul Syllable */
  ["\uBE2A", "\uBE2A"], /* Hangul Syllable */
  ["\uBE2B", "\uBE2B"], /* Hangul Syllable */
  ["\uBE2C", "\uBE2C"], /* Hangul Syllable */
  ["\uBE2D", "\uBE2D"], /* Hangul Syllable */
  ["\uBE2E", "\uBE2E"], /* Hangul Syllable */
  ["\uBE2F", "\uBE2F"], /* Hangul Syllable */
  ["\uBE30", "\uBE30"], /* Hangul Syllable */
  ["\uBE31", "\uBE31"], /* Hangul Syllable */
  ["\uBE32", "\uBE32"], /* Hangul Syllable */
  ["\uBE33", "\uBE33"], /* Hangul Syllable */
  ["\uBE34", "\uBE34"], /* Hangul Syllable */
  ["\uBE35", "\uBE35"], /* Hangul Syllable */
  ["\uBE36", "\uBE36"], /* Hangul Syllable */
  ["\uBE37", "\uBE37"], /* Hangul Syllable */
  ["\uBE38", "\uBE38"], /* Hangul Syllable */
  ["\uBE39", "\uBE39"], /* Hangul Syllable */
  ["\uBE3A", "\uBE3A"], /* Hangul Syllable */
  ["\uBE3B", "\uBE3B"], /* Hangul Syllable */
  ["\uBE3C", "\uBE3C"], /* Hangul Syllable */
  ["\uBE3D", "\uBE3D"], /* Hangul Syllable */
  ["\uBE3E", "\uBE3E"], /* Hangul Syllable */
  ["\uBE3F", "\uBE3F"], /* Hangul Syllable */
  ["\uBE40", "\uBE40"], /* Hangul Syllable */
  ["\uBE41", "\uBE41"], /* Hangul Syllable */
  ["\uBE42", "\uBE42"], /* Hangul Syllable */
  ["\uBE43", "\uBE43"], /* Hangul Syllable */
  ["\uBE44", "\uBE44"], /* Hangul Syllable */
  ["\uBE45", "\uBE45"], /* Hangul Syllable */
  ["\uBE46", "\uBE46"], /* Hangul Syllable */
  ["\uBE47", "\uBE47"], /* Hangul Syllable */
  ["\uBE48", "\uBE48"], /* Hangul Syllable */
  ["\uBE49", "\uBE49"], /* Hangul Syllable */
  ["\uBE4A", "\uBE4A"], /* Hangul Syllable */
  ["\uBE4B", "\uBE4B"], /* Hangul Syllable */
  ["\uBE4C", "\uBE4C"], /* Hangul Syllable */
  ["\uBE4D", "\uBE4D"], /* Hangul Syllable */
  ["\uBE4E", "\uBE4E"], /* Hangul Syllable */
  ["\uBE4F", "\uBE4F"], /* Hangul Syllable */
  ["\uBE50", "\uBE50"], /* Hangul Syllable */
  ["\uBE51", "\uBE51"], /* Hangul Syllable */
  ["\uBE52", "\uBE52"], /* Hangul Syllable */
  ["\uBE53", "\uBE53"], /* Hangul Syllable */
  ["\uBE54", "\uBE54"], /* Hangul Syllable */
  ["\uBE55", "\uBE55"], /* Hangul Syllable */
  ["\uBE56", "\uBE56"], /* Hangul Syllable */
  ["\uBE57", "\uBE57"], /* Hangul Syllable */
  ["\uBE58", "\uBE58"], /* Hangul Syllable */
  ["\uBE59", "\uBE59"], /* Hangul Syllable */
  ["\uBE5A", "\uBE5A"], /* Hangul Syllable */
  ["\uBE5B", "\uBE5B"], /* Hangul Syllable */
  ["\uBE5C", "\uBE5C"], /* Hangul Syllable */
  ["\uBE5D", "\uBE5D"], /* Hangul Syllable */
  ["\uBE5E", "\uBE5E"], /* Hangul Syllable */
  ["\uBE5F", "\uBE5F"], /* Hangul Syllable */
  ["\uBE60", "\uBE60"], /* Hangul Syllable */
  ["\uBE61", "\uBE61"], /* Hangul Syllable */
  ["\uBE62", "\uBE62"], /* Hangul Syllable */
  ["\uBE63", "\uBE63"], /* Hangul Syllable */
  ["\uBE64", "\uBE64"], /* Hangul Syllable */
  ["\uBE65", "\uBE65"], /* Hangul Syllable */
  ["\uBE66", "\uBE66"], /* Hangul Syllable */
  ["\uBE67", "\uBE67"], /* Hangul Syllable */
  ["\uBE68", "\uBE68"], /* Hangul Syllable */
  ["\uBE69", "\uBE69"], /* Hangul Syllable */
  ["\uBE6A", "\uBE6A"], /* Hangul Syllable */
  ["\uBE6B", "\uBE6B"], /* Hangul Syllable */
  ["\uBE6C", "\uBE6C"], /* Hangul Syllable */
  ["\uBE6D", "\uBE6D"], /* Hangul Syllable */
  ["\uBE6E", "\uBE6E"], /* Hangul Syllable */
  ["\uBE6F", "\uBE6F"], /* Hangul Syllable */
  ["\uBE70", "\uBE70"], /* Hangul Syllable */
  ["\uBE71", "\uBE71"], /* Hangul Syllable */
  ["\uBE72", "\uBE72"], /* Hangul Syllable */
  ["\uBE73", "\uBE73"], /* Hangul Syllable */
  ["\uBE74", "\uBE74"], /* Hangul Syllable */
  ["\uBE75", "\uBE75"], /* Hangul Syllable */
  ["\uBE76", "\uBE76"], /* Hangul Syllable */
  ["\uBE77", "\uBE77"], /* Hangul Syllable */
  ["\uBE78", "\uBE78"], /* Hangul Syllable */
  ["\uBE79", "\uBE79"], /* Hangul Syllable */
  ["\uBE7A", "\uBE7A"], /* Hangul Syllable */
  ["\uBE7B", "\uBE7B"], /* Hangul Syllable */
  ["\uBE7C", "\uBE7C"], /* Hangul Syllable */
  ["\uBE7D", "\uBE7D"], /* Hangul Syllable */
  ["\uBE7E", "\uBE7E"], /* Hangul Syllable */
  ["\uBE7F", "\uBE7F"], /* Hangul Syllable */
  ["\uBE80", "\uBE80"], /* Hangul Syllable */
  ["\uBE81", "\uBE81"], /* Hangul Syllable */
  ["\uBE82", "\uBE82"], /* Hangul Syllable */
  ["\uBE83", "\uBE83"], /* Hangul Syllable */
  ["\uBE84", "\uBE84"], /* Hangul Syllable */
  ["\uBE85", "\uBE85"], /* Hangul Syllable */
  ["\uBE86", "\uBE86"], /* Hangul Syllable */
  ["\uBE87", "\uBE87"], /* Hangul Syllable */
  ["\uBE88", "\uBE88"], /* Hangul Syllable */
  ["\uBE89", "\uBE89"], /* Hangul Syllable */
  ["\uBE8A", "\uBE8A"], /* Hangul Syllable */
  ["\uBE8B", "\uBE8B"], /* Hangul Syllable */
  ["\uBE8C", "\uBE8C"], /* Hangul Syllable */
  ["\uBE8D", "\uBE8D"], /* Hangul Syllable */
  ["\uBE8E", "\uBE8E"], /* Hangul Syllable */
  ["\uBE8F", "\uBE8F"], /* Hangul Syllable */
  ["\uBE90", "\uBE90"], /* Hangul Syllable */
  ["\uBE91", "\uBE91"], /* Hangul Syllable */
  ["\uBE92", "\uBE92"], /* Hangul Syllable */
  ["\uBE93", "\uBE93"], /* Hangul Syllable */
  ["\uBE94", "\uBE94"], /* Hangul Syllable */
  ["\uBE95", "\uBE95"], /* Hangul Syllable */
  ["\uBE96", "\uBE96"], /* Hangul Syllable */
  ["\uBE97", "\uBE97"], /* Hangul Syllable */
  ["\uBE98", "\uBE98"], /* Hangul Syllable */
  ["\uBE99", "\uBE99"], /* Hangul Syllable */
  ["\uBE9A", "\uBE9A"], /* Hangul Syllable */
  ["\uBE9B", "\uBE9B"], /* Hangul Syllable */
  ["\uBE9C", "\uBE9C"], /* Hangul Syllable */
  ["\uBE9D", "\uBE9D"], /* Hangul Syllable */
  ["\uBE9E", "\uBE9E"], /* Hangul Syllable */
  ["\uBE9F", "\uBE9F"], /* Hangul Syllable */
  ["\uBEA0", "\uBEA0"], /* Hangul Syllable */
  ["\uBEA1", "\uBEA1"], /* Hangul Syllable */
  ["\uBEA2", "\uBEA2"], /* Hangul Syllable */
  ["\uBEA3", "\uBEA3"], /* Hangul Syllable */
  ["\uBEA4", "\uBEA4"], /* Hangul Syllable */
  ["\uBEA5", "\uBEA5"], /* Hangul Syllable */
  ["\uBEA6", "\uBEA6"], /* Hangul Syllable */
  ["\uBEA7", "\uBEA7"], /* Hangul Syllable */
  ["\uBEA8", "\uBEA8"], /* Hangul Syllable */
  ["\uBEA9", "\uBEA9"], /* Hangul Syllable */
  ["\uBEAA", "\uBEAA"], /* Hangul Syllable */
  ["\uBEAB", "\uBEAB"], /* Hangul Syllable */
  ["\uBEAC", "\uBEAC"], /* Hangul Syllable */
  ["\uBEAD", "\uBEAD"], /* Hangul Syllable */
  ["\uBEAE", "\uBEAE"], /* Hangul Syllable */
  ["\uBEAF", "\uBEAF"], /* Hangul Syllable */
  ["\uBEB0", "\uBEB0"], /* Hangul Syllable */
  ["\uBEB1", "\uBEB1"], /* Hangul Syllable */
  ["\uBEB2", "\uBEB2"], /* Hangul Syllable */
  ["\uBEB3", "\uBEB3"], /* Hangul Syllable */
  ["\uBEB4", "\uBEB4"], /* Hangul Syllable */
  ["\uBEB5", "\uBEB5"], /* Hangul Syllable */
  ["\uBEB6", "\uBEB6"], /* Hangul Syllable */
  ["\uBEB7", "\uBEB7"], /* Hangul Syllable */
  ["\uBEB8", "\uBEB8"], /* Hangul Syllable */
  ["\uBEB9", "\uBEB9"], /* Hangul Syllable */
  ["\uBEBA", "\uBEBA"], /* Hangul Syllable */
  ["\uBEBB", "\uBEBB"], /* Hangul Syllable */
  ["\uBEBC", "\uBEBC"], /* Hangul Syllable */
  ["\uBEBD", "\uBEBD"], /* Hangul Syllable */
  ["\uBEBE", "\uBEBE"], /* Hangul Syllable */
  ["\uBEBF", "\uBEBF"], /* Hangul Syllable */
  ["\uBEC0", "\uBEC0"], /* Hangul Syllable */
  ["\uBEC1", "\uBEC1"], /* Hangul Syllable */
  ["\uBEC2", "\uBEC2"], /* Hangul Syllable */
  ["\uBEC3", "\uBEC3"], /* Hangul Syllable */
  ["\uBEC4", "\uBEC4"], /* Hangul Syllable */
  ["\uBEC5", "\uBEC5"], /* Hangul Syllable */
  ["\uBEC6", "\uBEC6"], /* Hangul Syllable */
  ["\uBEC7", "\uBEC7"], /* Hangul Syllable */
  ["\uBEC8", "\uBEC8"], /* Hangul Syllable */
  ["\uBEC9", "\uBEC9"], /* Hangul Syllable */
  ["\uBECA", "\uBECA"], /* Hangul Syllable */
  ["\uBECB", "\uBECB"], /* Hangul Syllable */
  ["\uBECC", "\uBECC"], /* Hangul Syllable */
  ["\uBECD", "\uBECD"], /* Hangul Syllable */
  ["\uBECE", "\uBECE"], /* Hangul Syllable */
  ["\uBECF", "\uBECF"], /* Hangul Syllable */
  ["\uBED0", "\uBED0"], /* Hangul Syllable */
  ["\uBED1", "\uBED1"], /* Hangul Syllable */
  ["\uBED2", "\uBED2"], /* Hangul Syllable */
  ["\uBED3", "\uBED3"], /* Hangul Syllable */
  ["\uBED4", "\uBED4"], /* Hangul Syllable */
  ["\uBED5", "\uBED5"], /* Hangul Syllable */
  ["\uBED6", "\uBED6"], /* Hangul Syllable */
  ["\uBED7", "\uBED7"], /* Hangul Syllable */
  ["\uBED8", "\uBED8"], /* Hangul Syllable */
  ["\uBED9", "\uBED9"], /* Hangul Syllable */
  ["\uBEDA", "\uBEDA"], /* Hangul Syllable */
  ["\uBEDB", "\uBEDB"], /* Hangul Syllable */
  ["\uBEDC", "\uBEDC"], /* Hangul Syllable */
  ["\uBEDD", "\uBEDD"], /* Hangul Syllable */
  ["\uBEDE", "\uBEDE"], /* Hangul Syllable */
  ["\uBEDF", "\uBEDF"], /* Hangul Syllable */
  ["\uBEE0", "\uBEE0"], /* Hangul Syllable */
  ["\uBEE1", "\uBEE1"], /* Hangul Syllable */
  ["\uBEE2", "\uBEE2"], /* Hangul Syllable */
  ["\uBEE3", "\uBEE3"], /* Hangul Syllable */
  ["\uBEE4", "\uBEE4"], /* Hangul Syllable */
  ["\uBEE5", "\uBEE5"], /* Hangul Syllable */
  ["\uBEE6", "\uBEE6"], /* Hangul Syllable */
  ["\uBEE7", "\uBEE7"], /* Hangul Syllable */
  ["\uBEE8", "\uBEE8"], /* Hangul Syllable */
  ["\uBEE9", "\uBEE9"], /* Hangul Syllable */
  ["\uBEEA", "\uBEEA"], /* Hangul Syllable */
  ["\uBEEB", "\uBEEB"], /* Hangul Syllable */
  ["\uBEEC", "\uBEEC"], /* Hangul Syllable */
  ["\uBEED", "\uBEED"], /* Hangul Syllable */
  ["\uBEEE", "\uBEEE"], /* Hangul Syllable */
  ["\uBEEF", "\uBEEF"], /* Hangul Syllable */
  ["\uBEF0", "\uBEF0"], /* Hangul Syllable */
  ["\uBEF1", "\uBEF1"], /* Hangul Syllable */
  ["\uBEF2", "\uBEF2"], /* Hangul Syllable */
  ["\uBEF3", "\uBEF3"], /* Hangul Syllable */
  ["\uBEF4", "\uBEF4"], /* Hangul Syllable */
  ["\uBEF5", "\uBEF5"], /* Hangul Syllable */
  ["\uBEF6", "\uBEF6"], /* Hangul Syllable */
  ["\uBEF7", "\uBEF7"], /* Hangul Syllable */
  ["\uBEF8", "\uBEF8"], /* Hangul Syllable */
  ["\uBEF9", "\uBEF9"], /* Hangul Syllable */
  ["\uBEFA", "\uBEFA"], /* Hangul Syllable */
  ["\uBEFB", "\uBEFB"], /* Hangul Syllable */
  ["\uBEFC", "\uBEFC"], /* Hangul Syllable */
  ["\uBEFD", "\uBEFD"], /* Hangul Syllable */
  ["\uBEFE", "\uBEFE"], /* Hangul Syllable */
  ["\uBEFF", "\uBEFF"], /* Hangul Syllable */
  ["\uBF00", "\uBF00"], /* Hangul Syllable */
  ["\uBF01", "\uBF01"], /* Hangul Syllable */
  ["\uBF02", "\uBF02"], /* Hangul Syllable */
  ["\uBF03", "\uBF03"], /* Hangul Syllable */
  ["\uBF04", "\uBF04"], /* Hangul Syllable */
  ["\uBF05", "\uBF05"], /* Hangul Syllable */
  ["\uBF06", "\uBF06"], /* Hangul Syllable */
  ["\uBF07", "\uBF07"], /* Hangul Syllable */
  ["\uBF08", "\uBF08"], /* Hangul Syllable */
  ["\uBF09", "\uBF09"], /* Hangul Syllable */
  ["\uBF0A", "\uBF0A"], /* Hangul Syllable */
  ["\uBF0B", "\uBF0B"], /* Hangul Syllable */
  ["\uBF0C", "\uBF0C"], /* Hangul Syllable */
  ["\uBF0D", "\uBF0D"], /* Hangul Syllable */
  ["\uBF0E", "\uBF0E"], /* Hangul Syllable */
  ["\uBF0F", "\uBF0F"], /* Hangul Syllable */
  ["\uBF10", "\uBF10"], /* Hangul Syllable */
  ["\uBF11", "\uBF11"], /* Hangul Syllable */
  ["\uBF12", "\uBF12"], /* Hangul Syllable */
  ["\uBF13", "\uBF13"], /* Hangul Syllable */
  ["\uBF14", "\uBF14"], /* Hangul Syllable */
  ["\uBF15", "\uBF15"], /* Hangul Syllable */
  ["\uBF16", "\uBF16"], /* Hangul Syllable */
  ["\uBF17", "\uBF17"], /* Hangul Syllable */
  ["\uBF18", "\uBF18"], /* Hangul Syllable */
  ["\uBF19", "\uBF19"], /* Hangul Syllable */
  ["\uBF1A", "\uBF1A"], /* Hangul Syllable */
  ["\uBF1B", "\uBF1B"], /* Hangul Syllable */
  ["\uBF1C", "\uBF1C"], /* Hangul Syllable */
  ["\uBF1D", "\uBF1D"], /* Hangul Syllable */
  ["\uBF1E", "\uBF1E"], /* Hangul Syllable */
  ["\uBF1F", "\uBF1F"], /* Hangul Syllable */
  ["\uBF20", "\uBF20"], /* Hangul Syllable */
  ["\uBF21", "\uBF21"], /* Hangul Syllable */
  ["\uBF22", "\uBF22"], /* Hangul Syllable */
  ["\uBF23", "\uBF23"], /* Hangul Syllable */
  ["\uBF24", "\uBF24"], /* Hangul Syllable */
  ["\uBF25", "\uBF25"], /* Hangul Syllable */
  ["\uBF26", "\uBF26"], /* Hangul Syllable */
  ["\uBF27", "\uBF27"], /* Hangul Syllable */
  ["\uBF28", "\uBF28"], /* Hangul Syllable */
  ["\uBF29", "\uBF29"], /* Hangul Syllable */
  ["\uBF2A", "\uBF2A"], /* Hangul Syllable */
  ["\uBF2B", "\uBF2B"], /* Hangul Syllable */
  ["\uBF2C", "\uBF2C"], /* Hangul Syllable */
  ["\uBF2D", "\uBF2D"], /* Hangul Syllable */
  ["\uBF2E", "\uBF2E"], /* Hangul Syllable */
  ["\uBF2F", "\uBF2F"], /* Hangul Syllable */
  ["\uBF30", "\uBF30"], /* Hangul Syllable */
  ["\uBF31", "\uBF31"], /* Hangul Syllable */
  ["\uBF32", "\uBF32"], /* Hangul Syllable */
  ["\uBF33", "\uBF33"], /* Hangul Syllable */
  ["\uBF34", "\uBF34"], /* Hangul Syllable */
  ["\uBF35", "\uBF35"], /* Hangul Syllable */
  ["\uBF36", "\uBF36"], /* Hangul Syllable */
  ["\uBF37", "\uBF37"], /* Hangul Syllable */
  ["\uBF38", "\uBF38"], /* Hangul Syllable */
  ["\uBF39", "\uBF39"], /* Hangul Syllable */
  ["\uBF3A", "\uBF3A"], /* Hangul Syllable */
  ["\uBF3B", "\uBF3B"], /* Hangul Syllable */
  ["\uBF3C", "\uBF3C"], /* Hangul Syllable */
  ["\uBF3D", "\uBF3D"], /* Hangul Syllable */
  ["\uBF3E", "\uBF3E"], /* Hangul Syllable */
  ["\uBF3F", "\uBF3F"], /* Hangul Syllable */
  ["\uBF40", "\uBF40"], /* Hangul Syllable */
  ["\uBF41", "\uBF41"], /* Hangul Syllable */
  ["\uBF42", "\uBF42"], /* Hangul Syllable */
  ["\uBF43", "\uBF43"], /* Hangul Syllable */
  ["\uBF44", "\uBF44"], /* Hangul Syllable */
  ["\uBF45", "\uBF45"], /* Hangul Syllable */
  ["\uBF46", "\uBF46"], /* Hangul Syllable */
  ["\uBF47", "\uBF47"], /* Hangul Syllable */
  ["\uBF48", "\uBF48"], /* Hangul Syllable */
  ["\uBF49", "\uBF49"], /* Hangul Syllable */
  ["\uBF4A", "\uBF4A"], /* Hangul Syllable */
  ["\uBF4B", "\uBF4B"], /* Hangul Syllable */
  ["\uBF4C", "\uBF4C"], /* Hangul Syllable */
  ["\uBF4D", "\uBF4D"], /* Hangul Syllable */
  ["\uBF4E", "\uBF4E"], /* Hangul Syllable */
  ["\uBF4F", "\uBF4F"], /* Hangul Syllable */
  ["\uBF50", "\uBF50"], /* Hangul Syllable */
  ["\uBF51", "\uBF51"], /* Hangul Syllable */
  ["\uBF52", "\uBF52"], /* Hangul Syllable */
  ["\uBF53", "\uBF53"], /* Hangul Syllable */
  ["\uBF54", "\uBF54"], /* Hangul Syllable */
  ["\uBF55", "\uBF55"], /* Hangul Syllable */
  ["\uBF56", "\uBF56"], /* Hangul Syllable */
  ["\uBF57", "\uBF57"], /* Hangul Syllable */
  ["\uBF58", "\uBF58"], /* Hangul Syllable */
  ["\uBF59", "\uBF59"], /* Hangul Syllable */
  ["\uBF5A", "\uBF5A"], /* Hangul Syllable */
  ["\uBF5B", "\uBF5B"], /* Hangul Syllable */
  ["\uBF5C", "\uBF5C"], /* Hangul Syllable */
  ["\uBF5D", "\uBF5D"], /* Hangul Syllable */
  ["\uBF5E", "\uBF5E"], /* Hangul Syllable */
  ["\uBF5F", "\uBF5F"], /* Hangul Syllable */
  ["\uBF60", "\uBF60"], /* Hangul Syllable */
  ["\uBF61", "\uBF61"], /* Hangul Syllable */
  ["\uBF62", "\uBF62"], /* Hangul Syllable */
  ["\uBF63", "\uBF63"], /* Hangul Syllable */
  ["\uBF64", "\uBF64"], /* Hangul Syllable */
  ["\uBF65", "\uBF65"], /* Hangul Syllable */
  ["\uBF66", "\uBF66"], /* Hangul Syllable */
  ["\uBF67", "\uBF67"], /* Hangul Syllable */
  ["\uBF68", "\uBF68"], /* Hangul Syllable */
  ["\uBF69", "\uBF69"], /* Hangul Syllable */
  ["\uBF6A", "\uBF6A"], /* Hangul Syllable */
  ["\uBF6B", "\uBF6B"], /* Hangul Syllable */
  ["\uBF6C", "\uBF6C"], /* Hangul Syllable */
  ["\uBF6D", "\uBF6D"], /* Hangul Syllable */
  ["\uBF6E", "\uBF6E"], /* Hangul Syllable */
  ["\uBF6F", "\uBF6F"], /* Hangul Syllable */
  ["\uBF70", "\uBF70"], /* Hangul Syllable */
  ["\uBF71", "\uBF71"], /* Hangul Syllable */
  ["\uBF72", "\uBF72"], /* Hangul Syllable */
  ["\uBF73", "\uBF73"], /* Hangul Syllable */
  ["\uBF74", "\uBF74"], /* Hangul Syllable */
  ["\uBF75", "\uBF75"], /* Hangul Syllable */
  ["\uBF76", "\uBF76"], /* Hangul Syllable */
  ["\uBF77", "\uBF77"], /* Hangul Syllable */
  ["\uBF78", "\uBF78"], /* Hangul Syllable */
  ["\uBF79", "\uBF79"], /* Hangul Syllable */
  ["\uBF7A", "\uBF7A"], /* Hangul Syllable */
  ["\uBF7B", "\uBF7B"], /* Hangul Syllable */
  ["\uBF7C", "\uBF7C"], /* Hangul Syllable */
  ["\uBF7D", "\uBF7D"], /* Hangul Syllable */
  ["\uBF7E", "\uBF7E"], /* Hangul Syllable */
  ["\uBF7F", "\uBF7F"], /* Hangul Syllable */
  ["\uBF80", "\uBF80"], /* Hangul Syllable */
  ["\uBF81", "\uBF81"], /* Hangul Syllable */
  ["\uBF82", "\uBF82"], /* Hangul Syllable */
  ["\uBF83", "\uBF83"], /* Hangul Syllable */
  ["\uBF84", "\uBF84"], /* Hangul Syllable */
  ["\uBF85", "\uBF85"], /* Hangul Syllable */
  ["\uBF86", "\uBF86"], /* Hangul Syllable */
  ["\uBF87", "\uBF87"], /* Hangul Syllable */
  ["\uBF88", "\uBF88"], /* Hangul Syllable */
  ["\uBF89", "\uBF89"], /* Hangul Syllable */
  ["\uBF8A", "\uBF8A"], /* Hangul Syllable */
  ["\uBF8B", "\uBF8B"], /* Hangul Syllable */
  ["\uBF8C", "\uBF8C"], /* Hangul Syllable */
  ["\uBF8D", "\uBF8D"], /* Hangul Syllable */
  ["\uBF8E", "\uBF8E"], /* Hangul Syllable */
  ["\uBF8F", "\uBF8F"], /* Hangul Syllable */
  ["\uBF90", "\uBF90"], /* Hangul Syllable */
  ["\uBF91", "\uBF91"], /* Hangul Syllable */
  ["\uBF92", "\uBF92"], /* Hangul Syllable */
  ["\uBF93", "\uBF93"], /* Hangul Syllable */
  ["\uBF94", "\uBF94"], /* Hangul Syllable */
  ["\uBF95", "\uBF95"], /* Hangul Syllable */
  ["\uBF96", "\uBF96"], /* Hangul Syllable */
  ["\uBF97", "\uBF97"], /* Hangul Syllable */
  ["\uBF98", "\uBF98"], /* Hangul Syllable */
  ["\uBF99", "\uBF99"], /* Hangul Syllable */
  ["\uBF9A", "\uBF9A"], /* Hangul Syllable */
  ["\uBF9B", "\uBF9B"], /* Hangul Syllable */
  ["\uBF9C", "\uBF9C"], /* Hangul Syllable */
  ["\uBF9D", "\uBF9D"], /* Hangul Syllable */
  ["\uBF9E", "\uBF9E"], /* Hangul Syllable */
  ["\uBF9F", "\uBF9F"], /* Hangul Syllable */
  ["\uBFA0", "\uBFA0"], /* Hangul Syllable */
  ["\uBFA1", "\uBFA1"], /* Hangul Syllable */
  ["\uBFA2", "\uBFA2"], /* Hangul Syllable */
  ["\uBFA3", "\uBFA3"], /* Hangul Syllable */
  ["\uBFA4", "\uBFA4"], /* Hangul Syllable */
  ["\uBFA5", "\uBFA5"], /* Hangul Syllable */
  ["\uBFA6", "\uBFA6"], /* Hangul Syllable */
  ["\uBFA7", "\uBFA7"], /* Hangul Syllable */
  ["\uBFA8", "\uBFA8"], /* Hangul Syllable */
  ["\uBFA9", "\uBFA9"], /* Hangul Syllable */
  ["\uBFAA", "\uBFAA"], /* Hangul Syllable */
  ["\uBFAB", "\uBFAB"], /* Hangul Syllable */
  ["\uBFAC", "\uBFAC"], /* Hangul Syllable */
  ["\uBFAD", "\uBFAD"], /* Hangul Syllable */
  ["\uBFAE", "\uBFAE"], /* Hangul Syllable */
  ["\uBFAF", "\uBFAF"], /* Hangul Syllable */
  ["\uBFB0", "\uBFB0"], /* Hangul Syllable */
  ["\uBFB1", "\uBFB1"], /* Hangul Syllable */
  ["\uBFB2", "\uBFB2"], /* Hangul Syllable */
  ["\uBFB3", "\uBFB3"], /* Hangul Syllable */
  ["\uBFB4", "\uBFB4"], /* Hangul Syllable */
  ["\uBFB5", "\uBFB5"], /* Hangul Syllable */
  ["\uBFB6", "\uBFB6"], /* Hangul Syllable */
  ["\uBFB7", "\uBFB7"], /* Hangul Syllable */
  ["\uBFB8", "\uBFB8"], /* Hangul Syllable */
  ["\uBFB9", "\uBFB9"], /* Hangul Syllable */
  ["\uBFBA", "\uBFBA"], /* Hangul Syllable */
  ["\uBFBB", "\uBFBB"], /* Hangul Syllable */
  ["\uBFBC", "\uBFBC"], /* Hangul Syllable */
  ["\uBFBD", "\uBFBD"], /* Hangul Syllable */
  ["\uBFBE", "\uBFBE"], /* Hangul Syllable */
  ["\uBFBF", "\uBFBF"], /* Hangul Syllable */
  ["\uBFC0", "\uBFC0"], /* Hangul Syllable */
  ["\uBFC1", "\uBFC1"], /* Hangul Syllable */
  ["\uBFC2", "\uBFC2"], /* Hangul Syllable */
  ["\uBFC3", "\uBFC3"], /* Hangul Syllable */
  ["\uBFC4", "\uBFC4"], /* Hangul Syllable */
  ["\uBFC5", "\uBFC5"], /* Hangul Syllable */
  ["\uBFC6", "\uBFC6"], /* Hangul Syllable */
  ["\uBFC7", "\uBFC7"], /* Hangul Syllable */
  ["\uBFC8", "\uBFC8"], /* Hangul Syllable */
  ["\uBFC9", "\uBFC9"], /* Hangul Syllable */
  ["\uBFCA", "\uBFCA"], /* Hangul Syllable */
  ["\uBFCB", "\uBFCB"], /* Hangul Syllable */
  ["\uBFCC", "\uBFCC"], /* Hangul Syllable */
  ["\uBFCD", "\uBFCD"], /* Hangul Syllable */
  ["\uBFCE", "\uBFCE"], /* Hangul Syllable */
  ["\uBFCF", "\uBFCF"], /* Hangul Syllable */
  ["\uBFD0", "\uBFD0"], /* Hangul Syllable */
  ["\uBFD1", "\uBFD1"], /* Hangul Syllable */
  ["\uBFD2", "\uBFD2"], /* Hangul Syllable */
  ["\uBFD3", "\uBFD3"], /* Hangul Syllable */
  ["\uBFD4", "\uBFD4"], /* Hangul Syllable */
  ["\uBFD5", "\uBFD5"], /* Hangul Syllable */
  ["\uBFD6", "\uBFD6"], /* Hangul Syllable */
  ["\uBFD7", "\uBFD7"], /* Hangul Syllable */
  ["\uBFD8", "\uBFD8"], /* Hangul Syllable */
  ["\uBFD9", "\uBFD9"], /* Hangul Syllable */
  ["\uBFDA", "\uBFDA"], /* Hangul Syllable */
  ["\uBFDB", "\uBFDB"], /* Hangul Syllable */
  ["\uBFDC", "\uBFDC"], /* Hangul Syllable */
  ["\uBFDD", "\uBFDD"], /* Hangul Syllable */
  ["\uBFDE", "\uBFDE"], /* Hangul Syllable */
  ["\uBFDF", "\uBFDF"], /* Hangul Syllable */
  ["\uBFE0", "\uBFE0"], /* Hangul Syllable */
  ["\uBFE1", "\uBFE1"], /* Hangul Syllable */
  ["\uBFE2", "\uBFE2"], /* Hangul Syllable */
  ["\uBFE3", "\uBFE3"], /* Hangul Syllable */
  ["\uBFE4", "\uBFE4"], /* Hangul Syllable */
  ["\uBFE5", "\uBFE5"], /* Hangul Syllable */
  ["\uBFE6", "\uBFE6"], /* Hangul Syllable */
  ["\uBFE7", "\uBFE7"], /* Hangul Syllable */
  ["\uBFE8", "\uBFE8"], /* Hangul Syllable */
  ["\uBFE9", "\uBFE9"], /* Hangul Syllable */
  ["\uBFEA", "\uBFEA"], /* Hangul Syllable */
  ["\uBFEB", "\uBFEB"], /* Hangul Syllable */
  ["\uBFEC", "\uBFEC"], /* Hangul Syllable */
  ["\uBFED", "\uBFED"], /* Hangul Syllable */
  ["\uBFEE", "\uBFEE"], /* Hangul Syllable */
  ["\uBFEF", "\uBFEF"], /* Hangul Syllable */
  ["\uBFF0", "\uBFF0"], /* Hangul Syllable */
  ["\uBFF1", "\uBFF1"], /* Hangul Syllable */
  ["\uBFF2", "\uBFF2"], /* Hangul Syllable */
  ["\uBFF3", "\uBFF3"], /* Hangul Syllable */
  ["\uBFF4", "\uBFF4"], /* Hangul Syllable */
  ["\uBFF5", "\uBFF5"], /* Hangul Syllable */
  ["\uBFF6", "\uBFF6"], /* Hangul Syllable */
  ["\uBFF7", "\uBFF7"], /* Hangul Syllable */
  ["\uBFF8", "\uBFF8"], /* Hangul Syllable */
  ["\uBFF9", "\uBFF9"], /* Hangul Syllable */
  ["\uBFFA", "\uBFFA"], /* Hangul Syllable */
  ["\uBFFB", "\uBFFB"], /* Hangul Syllable */
  ["\uBFFC", "\uBFFC"], /* Hangul Syllable */
  ["\uBFFD", "\uBFFD"], /* Hangul Syllable */
  ["\uBFFE", "\uBFFE"], /* Hangul Syllable */
  ["\uBFFF", "\uBFFF"], /* Hangul Syllable */
  ["\uC000", "\uC000"], /* Hangul Syllable */
  ["\uC001", "\uC001"], /* Hangul Syllable */
  ["\uC002", "\uC002"], /* Hangul Syllable */
  ["\uC003", "\uC003"], /* Hangul Syllable */
  ["\uC004", "\uC004"], /* Hangul Syllable */
  ["\uC005", "\uC005"], /* Hangul Syllable */
  ["\uC006", "\uC006"], /* Hangul Syllable */
  ["\uC007", "\uC007"], /* Hangul Syllable */
  ["\uC008", "\uC008"], /* Hangul Syllable */
  ["\uC009", "\uC009"], /* Hangul Syllable */
  ["\uC00A", "\uC00A"], /* Hangul Syllable */
  ["\uC00B", "\uC00B"], /* Hangul Syllable */
  ["\uC00C", "\uC00C"], /* Hangul Syllable */
  ["\uC00D", "\uC00D"], /* Hangul Syllable */
  ["\uC00E", "\uC00E"], /* Hangul Syllable */
  ["\uC00F", "\uC00F"], /* Hangul Syllable */
  ["\uC010", "\uC010"], /* Hangul Syllable */
  ["\uC011", "\uC011"], /* Hangul Syllable */
  ["\uC012", "\uC012"], /* Hangul Syllable */
  ["\uC013", "\uC013"], /* Hangul Syllable */
  ["\uC014", "\uC014"], /* Hangul Syllable */
  ["\uC015", "\uC015"], /* Hangul Syllable */
  ["\uC016", "\uC016"], /* Hangul Syllable */
  ["\uC017", "\uC017"], /* Hangul Syllable */
  ["\uC018", "\uC018"], /* Hangul Syllable */
  ["\uC019", "\uC019"], /* Hangul Syllable */
  ["\uC01A", "\uC01A"], /* Hangul Syllable */
  ["\uC01B", "\uC01B"], /* Hangul Syllable */
  ["\uC01C", "\uC01C"], /* Hangul Syllable */
  ["\uC01D", "\uC01D"], /* Hangul Syllable */
  ["\uC01E", "\uC01E"], /* Hangul Syllable */
  ["\uC01F", "\uC01F"], /* Hangul Syllable */
  ["\uC020", "\uC020"], /* Hangul Syllable */
  ["\uC021", "\uC021"], /* Hangul Syllable */
  ["\uC022", "\uC022"], /* Hangul Syllable */
  ["\uC023", "\uC023"], /* Hangul Syllable */
  ["\uC024", "\uC024"], /* Hangul Syllable */
  ["\uC025", "\uC025"], /* Hangul Syllable */
  ["\uC026", "\uC026"], /* Hangul Syllable */
  ["\uC027", "\uC027"], /* Hangul Syllable */
  ["\uC028", "\uC028"], /* Hangul Syllable */
  ["\uC029", "\uC029"], /* Hangul Syllable */
  ["\uC02A", "\uC02A"], /* Hangul Syllable */
  ["\uC02B", "\uC02B"], /* Hangul Syllable */
  ["\uC02C", "\uC02C"], /* Hangul Syllable */
  ["\uC02D", "\uC02D"], /* Hangul Syllable */
  ["\uC02E", "\uC02E"], /* Hangul Syllable */
  ["\uC02F", "\uC02F"], /* Hangul Syllable */
  ["\uC030", "\uC030"], /* Hangul Syllable */
  ["\uC031", "\uC031"], /* Hangul Syllable */
  ["\uC032", "\uC032"], /* Hangul Syllable */
  ["\uC033", "\uC033"], /* Hangul Syllable */
  ["\uC034", "\uC034"], /* Hangul Syllable */
  ["\uC035", "\uC035"], /* Hangul Syllable */
  ["\uC036", "\uC036"], /* Hangul Syllable */
  ["\uC037", "\uC037"], /* Hangul Syllable */
  ["\uC038", "\uC038"], /* Hangul Syllable */
  ["\uC039", "\uC039"], /* Hangul Syllable */
  ["\uC03A", "\uC03A"], /* Hangul Syllable */
  ["\uC03B", "\uC03B"], /* Hangul Syllable */
  ["\uC03C", "\uC03C"], /* Hangul Syllable */
  ["\uC03D", "\uC03D"], /* Hangul Syllable */
  ["\uC03E", "\uC03E"], /* Hangul Syllable */
  ["\uC03F", "\uC03F"], /* Hangul Syllable */
  ["\uC040", "\uC040"], /* Hangul Syllable */
  ["\uC041", "\uC041"], /* Hangul Syllable */
  ["\uC042", "\uC042"], /* Hangul Syllable */
  ["\uC043", "\uC043"], /* Hangul Syllable */
  ["\uC044", "\uC044"], /* Hangul Syllable */
  ["\uC045", "\uC045"], /* Hangul Syllable */
  ["\uC046", "\uC046"], /* Hangul Syllable */
  ["\uC047", "\uC047"], /* Hangul Syllable */
  ["\uC048", "\uC048"], /* Hangul Syllable */
  ["\uC049", "\uC049"], /* Hangul Syllable */
  ["\uC04A", "\uC04A"], /* Hangul Syllable */
  ["\uC04B", "\uC04B"], /* Hangul Syllable */
  ["\uC04C", "\uC04C"], /* Hangul Syllable */
  ["\uC04D", "\uC04D"], /* Hangul Syllable */
  ["\uC04E", "\uC04E"], /* Hangul Syllable */
  ["\uC04F", "\uC04F"], /* Hangul Syllable */
  ["\uC050", "\uC050"], /* Hangul Syllable */
  ["\uC051", "\uC051"], /* Hangul Syllable */
  ["\uC052", "\uC052"], /* Hangul Syllable */
  ["\uC053", "\uC053"], /* Hangul Syllable */
  ["\uC054", "\uC054"], /* Hangul Syllable */
  ["\uC055", "\uC055"], /* Hangul Syllable */
  ["\uC056", "\uC056"], /* Hangul Syllable */
  ["\uC057", "\uC057"], /* Hangul Syllable */
  ["\uC058", "\uC058"], /* Hangul Syllable */
  ["\uC059", "\uC059"], /* Hangul Syllable */
  ["\uC05A", "\uC05A"], /* Hangul Syllable */
  ["\uC05B", "\uC05B"], /* Hangul Syllable */
  ["\uC05C", "\uC05C"], /* Hangul Syllable */
  ["\uC05D", "\uC05D"], /* Hangul Syllable */
  ["\uC05E", "\uC05E"], /* Hangul Syllable */
  ["\uC05F", "\uC05F"], /* Hangul Syllable */
  ["\uC060", "\uC060"], /* Hangul Syllable */
  ["\uC061", "\uC061"], /* Hangul Syllable */
  ["\uC062", "\uC062"], /* Hangul Syllable */
  ["\uC063", "\uC063"], /* Hangul Syllable */
  ["\uC064", "\uC064"], /* Hangul Syllable */
  ["\uC065", "\uC065"], /* Hangul Syllable */
  ["\uC066", "\uC066"], /* Hangul Syllable */
  ["\uC067", "\uC067"], /* Hangul Syllable */
  ["\uC068", "\uC068"], /* Hangul Syllable */
  ["\uC069", "\uC069"], /* Hangul Syllable */
  ["\uC06A", "\uC06A"], /* Hangul Syllable */
  ["\uC06B", "\uC06B"], /* Hangul Syllable */
  ["\uC06C", "\uC06C"], /* Hangul Syllable */
  ["\uC06D", "\uC06D"], /* Hangul Syllable */
  ["\uC06E", "\uC06E"], /* Hangul Syllable */
  ["\uC06F", "\uC06F"], /* Hangul Syllable */
  ["\uC070", "\uC070"], /* Hangul Syllable */
  ["\uC071", "\uC071"], /* Hangul Syllable */
  ["\uC072", "\uC072"], /* Hangul Syllable */
  ["\uC073", "\uC073"], /* Hangul Syllable */
  ["\uC074", "\uC074"], /* Hangul Syllable */
  ["\uC075", "\uC075"], /* Hangul Syllable */
  ["\uC076", "\uC076"], /* Hangul Syllable */
  ["\uC077", "\uC077"], /* Hangul Syllable */
  ["\uC078", "\uC078"], /* Hangul Syllable */
  ["\uC079", "\uC079"], /* Hangul Syllable */
  ["\uC07A", "\uC07A"], /* Hangul Syllable */
  ["\uC07B", "\uC07B"], /* Hangul Syllable */
  ["\uC07C", "\uC07C"], /* Hangul Syllable */
  ["\uC07D", "\uC07D"], /* Hangul Syllable */
  ["\uC07E", "\uC07E"], /* Hangul Syllable */
  ["\uC07F", "\uC07F"], /* Hangul Syllable */
  ["\uC080", "\uC080"], /* Hangul Syllable */
  ["\uC081", "\uC081"], /* Hangul Syllable */
  ["\uC082", "\uC082"], /* Hangul Syllable */
  ["\uC083", "\uC083"], /* Hangul Syllable */
  ["\uC084", "\uC084"], /* Hangul Syllable */
  ["\uC085", "\uC085"], /* Hangul Syllable */
  ["\uC086", "\uC086"], /* Hangul Syllable */
  ["\uC087", "\uC087"], /* Hangul Syllable */
  ["\uC088", "\uC088"], /* Hangul Syllable */
  ["\uC089", "\uC089"], /* Hangul Syllable */
  ["\uC08A", "\uC08A"], /* Hangul Syllable */
  ["\uC08B", "\uC08B"], /* Hangul Syllable */
  ["\uC08C", "\uC08C"], /* Hangul Syllable */
  ["\uC08D", "\uC08D"], /* Hangul Syllable */
  ["\uC08E", "\uC08E"], /* Hangul Syllable */
  ["\uC08F", "\uC08F"], /* Hangul Syllable */
  ["\uC090", "\uC090"], /* Hangul Syllable */
  ["\uC091", "\uC091"], /* Hangul Syllable */
  ["\uC092", "\uC092"], /* Hangul Syllable */
  ["\uC093", "\uC093"], /* Hangul Syllable */
  ["\uC094", "\uC094"], /* Hangul Syllable */
  ["\uC095", "\uC095"], /* Hangul Syllable */
  ["\uC096", "\uC096"], /* Hangul Syllable */
  ["\uC097", "\uC097"], /* Hangul Syllable */
  ["\uC098", "\uC098"], /* Hangul Syllable */
  ["\uC099", "\uC099"], /* Hangul Syllable */
  ["\uC09A", "\uC09A"], /* Hangul Syllable */
  ["\uC09B", "\uC09B"], /* Hangul Syllable */
  ["\uC09C", "\uC09C"], /* Hangul Syllable */
  ["\uC09D", "\uC09D"], /* Hangul Syllable */
  ["\uC09E", "\uC09E"], /* Hangul Syllable */
  ["\uC09F", "\uC09F"], /* Hangul Syllable */
  ["\uC0A0", "\uC0A0"], /* Hangul Syllable */
  ["\uC0A1", "\uC0A1"], /* Hangul Syllable */
  ["\uC0A2", "\uC0A2"], /* Hangul Syllable */
  ["\uC0A3", "\uC0A3"], /* Hangul Syllable */
  ["\uC0A4", "\uC0A4"], /* Hangul Syllable */
  ["\uC0A5", "\uC0A5"], /* Hangul Syllable */
  ["\uC0A6", "\uC0A6"], /* Hangul Syllable */
  ["\uC0A7", "\uC0A7"], /* Hangul Syllable */
  ["\uC0A8", "\uC0A8"], /* Hangul Syllable */
  ["\uC0A9", "\uC0A9"], /* Hangul Syllable */
  ["\uC0AA", "\uC0AA"], /* Hangul Syllable */
  ["\uC0AB", "\uC0AB"], /* Hangul Syllable */
  ["\uC0AC", "\uC0AC"], /* Hangul Syllable */
  ["\uC0AD", "\uC0AD"], /* Hangul Syllable */
  ["\uC0AE", "\uC0AE"], /* Hangul Syllable */
  ["\uC0AF", "\uC0AF"], /* Hangul Syllable */
  ["\uC0B0", "\uC0B0"], /* Hangul Syllable */
  ["\uC0B1", "\uC0B1"], /* Hangul Syllable */
  ["\uC0B2", "\uC0B2"], /* Hangul Syllable */
  ["\uC0B3", "\uC0B3"], /* Hangul Syllable */
  ["\uC0B4", "\uC0B4"], /* Hangul Syllable */
  ["\uC0B5", "\uC0B5"], /* Hangul Syllable */
  ["\uC0B6", "\uC0B6"], /* Hangul Syllable */
  ["\uC0B7", "\uC0B7"], /* Hangul Syllable */
  ["\uC0B8", "\uC0B8"], /* Hangul Syllable */
  ["\uC0B9", "\uC0B9"], /* Hangul Syllable */
  ["\uC0BA", "\uC0BA"], /* Hangul Syllable */
  ["\uC0BB", "\uC0BB"], /* Hangul Syllable */
  ["\uC0BC", "\uC0BC"], /* Hangul Syllable */
  ["\uC0BD", "\uC0BD"], /* Hangul Syllable */
  ["\uC0BE", "\uC0BE"], /* Hangul Syllable */
  ["\uC0BF", "\uC0BF"], /* Hangul Syllable */
  ["\uC0C0", "\uC0C0"], /* Hangul Syllable */
  ["\uC0C1", "\uC0C1"], /* Hangul Syllable */
  ["\uC0C2", "\uC0C2"], /* Hangul Syllable */
  ["\uC0C3", "\uC0C3"], /* Hangul Syllable */
  ["\uC0C4", "\uC0C4"], /* Hangul Syllable */
  ["\uC0C5", "\uC0C5"], /* Hangul Syllable */
  ["\uC0C6", "\uC0C6"], /* Hangul Syllable */
  ["\uC0C7", "\uC0C7"], /* Hangul Syllable */
  ["\uC0C8", "\uC0C8"], /* Hangul Syllable */
  ["\uC0C9", "\uC0C9"], /* Hangul Syllable */
  ["\uC0CA", "\uC0CA"], /* Hangul Syllable */
  ["\uC0CB", "\uC0CB"], /* Hangul Syllable */
  ["\uC0CC", "\uC0CC"], /* Hangul Syllable */
  ["\uC0CD", "\uC0CD"], /* Hangul Syllable */
  ["\uC0CE", "\uC0CE"], /* Hangul Syllable */
  ["\uC0CF", "\uC0CF"], /* Hangul Syllable */
  ["\uC0D0", "\uC0D0"], /* Hangul Syllable */
  ["\uC0D1", "\uC0D1"], /* Hangul Syllable */
  ["\uC0D2", "\uC0D2"], /* Hangul Syllable */
  ["\uC0D3", "\uC0D3"], /* Hangul Syllable */
  ["\uC0D4", "\uC0D4"], /* Hangul Syllable */
  ["\uC0D5", "\uC0D5"], /* Hangul Syllable */
  ["\uC0D6", "\uC0D6"], /* Hangul Syllable */
  ["\uC0D7", "\uC0D7"], /* Hangul Syllable */
  ["\uC0D8", "\uC0D8"], /* Hangul Syllable */
  ["\uC0D9", "\uC0D9"], /* Hangul Syllable */
  ["\uC0DA", "\uC0DA"], /* Hangul Syllable */
  ["\uC0DB", "\uC0DB"], /* Hangul Syllable */
  ["\uC0DC", "\uC0DC"], /* Hangul Syllable */
  ["\uC0DD", "\uC0DD"], /* Hangul Syllable */
  ["\uC0DE", "\uC0DE"], /* Hangul Syllable */
  ["\uC0DF", "\uC0DF"], /* Hangul Syllable */
  ["\uC0E0", "\uC0E0"], /* Hangul Syllable */
  ["\uC0E1", "\uC0E1"], /* Hangul Syllable */
  ["\uC0E2", "\uC0E2"], /* Hangul Syllable */
  ["\uC0E3", "\uC0E3"], /* Hangul Syllable */
  ["\uC0E4", "\uC0E4"], /* Hangul Syllable */
  ["\uC0E5", "\uC0E5"], /* Hangul Syllable */
  ["\uC0E6", "\uC0E6"], /* Hangul Syllable */
  ["\uC0E7", "\uC0E7"], /* Hangul Syllable */
  ["\uC0E8", "\uC0E8"], /* Hangul Syllable */
  ["\uC0E9", "\uC0E9"], /* Hangul Syllable */
  ["\uC0EA", "\uC0EA"], /* Hangul Syllable */
  ["\uC0EB", "\uC0EB"], /* Hangul Syllable */
  ["\uC0EC", "\uC0EC"], /* Hangul Syllable */
  ["\uC0ED", "\uC0ED"], /* Hangul Syllable */
  ["\uC0EE", "\uC0EE"], /* Hangul Syllable */
  ["\uC0EF", "\uC0EF"], /* Hangul Syllable */
  ["\uC0F0", "\uC0F0"], /* Hangul Syllable */
  ["\uC0F1", "\uC0F1"], /* Hangul Syllable */
  ["\uC0F2", "\uC0F2"], /* Hangul Syllable */
  ["\uC0F3", "\uC0F3"], /* Hangul Syllable */
  ["\uC0F4", "\uC0F4"], /* Hangul Syllable */
  ["\uC0F5", "\uC0F5"], /* Hangul Syllable */
  ["\uC0F6", "\uC0F6"], /* Hangul Syllable */
  ["\uC0F7", "\uC0F7"], /* Hangul Syllable */
  ["\uC0F8", "\uC0F8"], /* Hangul Syllable */
  ["\uC0F9", "\uC0F9"], /* Hangul Syllable */
  ["\uC0FA", "\uC0FA"], /* Hangul Syllable */
  ["\uC0FB", "\uC0FB"], /* Hangul Syllable */
  ["\uC0FC", "\uC0FC"], /* Hangul Syllable */
  ["\uC0FD", "\uC0FD"], /* Hangul Syllable */
  ["\uC0FE", "\uC0FE"], /* Hangul Syllable */
  ["\uC0FF", "\uC0FF"], /* Hangul Syllable */
  ["\uC100", "\uC100"], /* Hangul Syllable */
  ["\uC101", "\uC101"], /* Hangul Syllable */
  ["\uC102", "\uC102"], /* Hangul Syllable */
  ["\uC103", "\uC103"], /* Hangul Syllable */
  ["\uC104", "\uC104"], /* Hangul Syllable */
  ["\uC105", "\uC105"], /* Hangul Syllable */
  ["\uC106", "\uC106"], /* Hangul Syllable */
  ["\uC107", "\uC107"], /* Hangul Syllable */
  ["\uC108", "\uC108"], /* Hangul Syllable */
  ["\uC109", "\uC109"], /* Hangul Syllable */
  ["\uC10A", "\uC10A"], /* Hangul Syllable */
  ["\uC10B", "\uC10B"], /* Hangul Syllable */
  ["\uC10C", "\uC10C"], /* Hangul Syllable */
  ["\uC10D", "\uC10D"], /* Hangul Syllable */
  ["\uC10E", "\uC10E"], /* Hangul Syllable */
  ["\uC10F", "\uC10F"], /* Hangul Syllable */
  ["\uC110", "\uC110"], /* Hangul Syllable */
  ["\uC111", "\uC111"], /* Hangul Syllable */
  ["\uC112", "\uC112"], /* Hangul Syllable */
  ["\uC113", "\uC113"], /* Hangul Syllable */
  ["\uC114", "\uC114"], /* Hangul Syllable */
  ["\uC115", "\uC115"], /* Hangul Syllable */
  ["\uC116", "\uC116"], /* Hangul Syllable */
  ["\uC117", "\uC117"], /* Hangul Syllable */
  ["\uC118", "\uC118"], /* Hangul Syllable */
  ["\uC119", "\uC119"], /* Hangul Syllable */
  ["\uC11A", "\uC11A"], /* Hangul Syllable */
  ["\uC11B", "\uC11B"], /* Hangul Syllable */
  ["\uC11C", "\uC11C"], /* Hangul Syllable */
  ["\uC11D", "\uC11D"], /* Hangul Syllable */
  ["\uC11E", "\uC11E"], /* Hangul Syllable */
  ["\uC11F", "\uC11F"], /* Hangul Syllable */
  ["\uC120", "\uC120"], /* Hangul Syllable */
  ["\uC121", "\uC121"], /* Hangul Syllable */
  ["\uC122", "\uC122"], /* Hangul Syllable */
  ["\uC123", "\uC123"], /* Hangul Syllable */
  ["\uC124", "\uC124"], /* Hangul Syllable */
  ["\uC125", "\uC125"], /* Hangul Syllable */
  ["\uC126", "\uC126"], /* Hangul Syllable */
  ["\uC127", "\uC127"], /* Hangul Syllable */
  ["\uC128", "\uC128"], /* Hangul Syllable */
  ["\uC129", "\uC129"], /* Hangul Syllable */
  ["\uC12A", "\uC12A"], /* Hangul Syllable */
  ["\uC12B", "\uC12B"], /* Hangul Syllable */
  ["\uC12C", "\uC12C"], /* Hangul Syllable */
  ["\uC12D", "\uC12D"], /* Hangul Syllable */
  ["\uC12E", "\uC12E"], /* Hangul Syllable */
  ["\uC12F", "\uC12F"], /* Hangul Syllable */
  ["\uC130", "\uC130"], /* Hangul Syllable */
  ["\uC131", "\uC131"], /* Hangul Syllable */
  ["\uC132", "\uC132"], /* Hangul Syllable */
  ["\uC133", "\uC133"], /* Hangul Syllable */
  ["\uC134", "\uC134"], /* Hangul Syllable */
  ["\uC135", "\uC135"], /* Hangul Syllable */
  ["\uC136", "\uC136"], /* Hangul Syllable */
  ["\uC137", "\uC137"], /* Hangul Syllable */
  ["\uC138", "\uC138"], /* Hangul Syllable */
  ["\uC139", "\uC139"], /* Hangul Syllable */
  ["\uC13A", "\uC13A"], /* Hangul Syllable */
  ["\uC13B", "\uC13B"], /* Hangul Syllable */
  ["\uC13C", "\uC13C"], /* Hangul Syllable */
  ["\uC13D", "\uC13D"], /* Hangul Syllable */
  ["\uC13E", "\uC13E"], /* Hangul Syllable */
  ["\uC13F", "\uC13F"], /* Hangul Syllable */
  ["\uC140", "\uC140"], /* Hangul Syllable */
  ["\uC141", "\uC141"], /* Hangul Syllable */
  ["\uC142", "\uC142"], /* Hangul Syllable */
  ["\uC143", "\uC143"], /* Hangul Syllable */
  ["\uC144", "\uC144"], /* Hangul Syllable */
  ["\uC145", "\uC145"], /* Hangul Syllable */
  ["\uC146", "\uC146"], /* Hangul Syllable */
  ["\uC147", "\uC147"], /* Hangul Syllable */
  ["\uC148", "\uC148"], /* Hangul Syllable */
  ["\uC149", "\uC149"], /* Hangul Syllable */
  ["\uC14A", "\uC14A"], /* Hangul Syllable */
  ["\uC14B", "\uC14B"], /* Hangul Syllable */
  ["\uC14C", "\uC14C"], /* Hangul Syllable */
  ["\uC14D", "\uC14D"], /* Hangul Syllable */
  ["\uC14E", "\uC14E"], /* Hangul Syllable */
  ["\uC14F", "\uC14F"], /* Hangul Syllable */
  ["\uC150", "\uC150"], /* Hangul Syllable */
  ["\uC151", "\uC151"], /* Hangul Syllable */
  ["\uC152", "\uC152"], /* Hangul Syllable */
  ["\uC153", "\uC153"], /* Hangul Syllable */
  ["\uC154", "\uC154"], /* Hangul Syllable */
  ["\uC155", "\uC155"], /* Hangul Syllable */
  ["\uC156", "\uC156"], /* Hangul Syllable */
  ["\uC157", "\uC157"], /* Hangul Syllable */
  ["\uC158", "\uC158"], /* Hangul Syllable */
  ["\uC159", "\uC159"], /* Hangul Syllable */
  ["\uC15A", "\uC15A"], /* Hangul Syllable */
  ["\uC15B", "\uC15B"], /* Hangul Syllable */
  ["\uC15C", "\uC15C"], /* Hangul Syllable */
  ["\uC15D", "\uC15D"], /* Hangul Syllable */
  ["\uC15E", "\uC15E"], /* Hangul Syllable */
  ["\uC15F", "\uC15F"], /* Hangul Syllable */
  ["\uC160", "\uC160"], /* Hangul Syllable */
  ["\uC161", "\uC161"], /* Hangul Syllable */
  ["\uC162", "\uC162"], /* Hangul Syllable */
  ["\uC163", "\uC163"], /* Hangul Syllable */
  ["\uC164", "\uC164"], /* Hangul Syllable */
  ["\uC165", "\uC165"], /* Hangul Syllable */
  ["\uC166", "\uC166"], /* Hangul Syllable */
  ["\uC167", "\uC167"], /* Hangul Syllable */
  ["\uC168", "\uC168"], /* Hangul Syllable */
  ["\uC169", "\uC169"], /* Hangul Syllable */
  ["\uC16A", "\uC16A"], /* Hangul Syllable */
  ["\uC16B", "\uC16B"], /* Hangul Syllable */
  ["\uC16C", "\uC16C"], /* Hangul Syllable */
  ["\uC16D", "\uC16D"], /* Hangul Syllable */
  ["\uC16E", "\uC16E"], /* Hangul Syllable */
  ["\uC16F", "\uC16F"], /* Hangul Syllable */
  ["\uC170", "\uC170"], /* Hangul Syllable */
  ["\uC171", "\uC171"], /* Hangul Syllable */
  ["\uC172", "\uC172"], /* Hangul Syllable */
  ["\uC173", "\uC173"], /* Hangul Syllable */
  ["\uC174", "\uC174"], /* Hangul Syllable */
  ["\uC175", "\uC175"], /* Hangul Syllable */
  ["\uC176", "\uC176"], /* Hangul Syllable */
  ["\uC177", "\uC177"], /* Hangul Syllable */
  ["\uC178", "\uC178"], /* Hangul Syllable */
  ["\uC179", "\uC179"], /* Hangul Syllable */
  ["\uC17A", "\uC17A"], /* Hangul Syllable */
  ["\uC17B", "\uC17B"], /* Hangul Syllable */
  ["\uC17C", "\uC17C"], /* Hangul Syllable */
  ["\uC17D", "\uC17D"], /* Hangul Syllable */
  ["\uC17E", "\uC17E"], /* Hangul Syllable */
  ["\uC17F", "\uC17F"], /* Hangul Syllable */
  ["\uC180", "\uC180"], /* Hangul Syllable */
  ["\uC181", "\uC181"], /* Hangul Syllable */
  ["\uC182", "\uC182"], /* Hangul Syllable */
  ["\uC183", "\uC183"], /* Hangul Syllable */
  ["\uC184", "\uC184"], /* Hangul Syllable */
  ["\uC185", "\uC185"], /* Hangul Syllable */
  ["\uC186", "\uC186"], /* Hangul Syllable */
  ["\uC187", "\uC187"], /* Hangul Syllable */
  ["\uC188", "\uC188"], /* Hangul Syllable */
  ["\uC189", "\uC189"], /* Hangul Syllable */
  ["\uC18A", "\uC18A"], /* Hangul Syllable */
  ["\uC18B", "\uC18B"], /* Hangul Syllable */
  ["\uC18C", "\uC18C"], /* Hangul Syllable */
  ["\uC18D", "\uC18D"], /* Hangul Syllable */
  ["\uC18E", "\uC18E"], /* Hangul Syllable */
  ["\uC18F", "\uC18F"], /* Hangul Syllable */
  ["\uC190", "\uC190"], /* Hangul Syllable */
  ["\uC191", "\uC191"], /* Hangul Syllable */
  ["\uC192", "\uC192"], /* Hangul Syllable */
  ["\uC193", "\uC193"], /* Hangul Syllable */
  ["\uC194", "\uC194"], /* Hangul Syllable */
  ["\uC195", "\uC195"], /* Hangul Syllable */
  ["\uC196", "\uC196"], /* Hangul Syllable */
  ["\uC197", "\uC197"], /* Hangul Syllable */
  ["\uC198", "\uC198"], /* Hangul Syllable */
  ["\uC199", "\uC199"], /* Hangul Syllable */
  ["\uC19A", "\uC19A"], /* Hangul Syllable */
  ["\uC19B", "\uC19B"], /* Hangul Syllable */
  ["\uC19C", "\uC19C"], /* Hangul Syllable */
  ["\uC19D", "\uC19D"], /* Hangul Syllable */
  ["\uC19E", "\uC19E"], /* Hangul Syllable */
  ["\uC19F", "\uC19F"], /* Hangul Syllable */
  ["\uC1A0", "\uC1A0"], /* Hangul Syllable */
  ["\uC1A1", "\uC1A1"], /* Hangul Syllable */
  ["\uC1A2", "\uC1A2"], /* Hangul Syllable */
  ["\uC1A3", "\uC1A3"], /* Hangul Syllable */
  ["\uC1A4", "\uC1A4"], /* Hangul Syllable */
  ["\uC1A5", "\uC1A5"], /* Hangul Syllable */
  ["\uC1A6", "\uC1A6"], /* Hangul Syllable */
  ["\uC1A7", "\uC1A7"], /* Hangul Syllable */
  ["\uC1A8", "\uC1A8"], /* Hangul Syllable */
  ["\uC1A9", "\uC1A9"], /* Hangul Syllable */
  ["\uC1AA", "\uC1AA"], /* Hangul Syllable */
  ["\uC1AB", "\uC1AB"], /* Hangul Syllable */
  ["\uC1AC", "\uC1AC"], /* Hangul Syllable */
  ["\uC1AD", "\uC1AD"], /* Hangul Syllable */
  ["\uC1AE", "\uC1AE"], /* Hangul Syllable */
  ["\uC1AF", "\uC1AF"], /* Hangul Syllable */
  ["\uC1B0", "\uC1B0"], /* Hangul Syllable */
  ["\uC1B1", "\uC1B1"], /* Hangul Syllable */
  ["\uC1B2", "\uC1B2"], /* Hangul Syllable */
  ["\uC1B3", "\uC1B3"], /* Hangul Syllable */
  ["\uC1B4", "\uC1B4"], /* Hangul Syllable */
  ["\uC1B5", "\uC1B5"], /* Hangul Syllable */
  ["\uC1B6", "\uC1B6"], /* Hangul Syllable */
  ["\uC1B7", "\uC1B7"], /* Hangul Syllable */
  ["\uC1B8", "\uC1B8"], /* Hangul Syllable */
  ["\uC1B9", "\uC1B9"], /* Hangul Syllable */
  ["\uC1BA", "\uC1BA"], /* Hangul Syllable */
  ["\uC1BB", "\uC1BB"], /* Hangul Syllable */
  ["\uC1BC", "\uC1BC"], /* Hangul Syllable */
  ["\uC1BD", "\uC1BD"], /* Hangul Syllable */
  ["\uC1BE", "\uC1BE"], /* Hangul Syllable */
  ["\uC1BF", "\uC1BF"], /* Hangul Syllable */
  ["\uC1C0", "\uC1C0"], /* Hangul Syllable */
  ["\uC1C1", "\uC1C1"], /* Hangul Syllable */
  ["\uC1C2", "\uC1C2"], /* Hangul Syllable */
  ["\uC1C3", "\uC1C3"], /* Hangul Syllable */
  ["\uC1C4", "\uC1C4"], /* Hangul Syllable */
  ["\uC1C5", "\uC1C5"], /* Hangul Syllable */
  ["\uC1C6", "\uC1C6"], /* Hangul Syllable */
  ["\uC1C7", "\uC1C7"], /* Hangul Syllable */
  ["\uC1C8", "\uC1C8"], /* Hangul Syllable */
  ["\uC1C9", "\uC1C9"], /* Hangul Syllable */
  ["\uC1CA", "\uC1CA"], /* Hangul Syllable */
  ["\uC1CB", "\uC1CB"], /* Hangul Syllable */
  ["\uC1CC", "\uC1CC"], /* Hangul Syllable */
  ["\uC1CD", "\uC1CD"], /* Hangul Syllable */
  ["\uC1CE", "\uC1CE"], /* Hangul Syllable */
  ["\uC1CF", "\uC1CF"], /* Hangul Syllable */
  ["\uC1D0", "\uC1D0"], /* Hangul Syllable */
  ["\uC1D1", "\uC1D1"], /* Hangul Syllable */
  ["\uC1D2", "\uC1D2"], /* Hangul Syllable */
  ["\uC1D3", "\uC1D3"], /* Hangul Syllable */
  ["\uC1D4", "\uC1D4"], /* Hangul Syllable */
  ["\uC1D5", "\uC1D5"], /* Hangul Syllable */
  ["\uC1D6", "\uC1D6"], /* Hangul Syllable */
  ["\uC1D7", "\uC1D7"], /* Hangul Syllable */
  ["\uC1D8", "\uC1D8"], /* Hangul Syllable */
  ["\uC1D9", "\uC1D9"], /* Hangul Syllable */
  ["\uC1DA", "\uC1DA"], /* Hangul Syllable */
  ["\uC1DB", "\uC1DB"], /* Hangul Syllable */
  ["\uC1DC", "\uC1DC"], /* Hangul Syllable */
  ["\uC1DD", "\uC1DD"], /* Hangul Syllable */
  ["\uC1DE", "\uC1DE"], /* Hangul Syllable */
  ["\uC1DF", "\uC1DF"], /* Hangul Syllable */
  ["\uC1E0", "\uC1E0"], /* Hangul Syllable */
  ["\uC1E1", "\uC1E1"], /* Hangul Syllable */
  ["\uC1E2", "\uC1E2"], /* Hangul Syllable */
  ["\uC1E3", "\uC1E3"], /* Hangul Syllable */
  ["\uC1E4", "\uC1E4"], /* Hangul Syllable */
  ["\uC1E5", "\uC1E5"], /* Hangul Syllable */
  ["\uC1E6", "\uC1E6"], /* Hangul Syllable */
  ["\uC1E7", "\uC1E7"], /* Hangul Syllable */
  ["\uC1E8", "\uC1E8"], /* Hangul Syllable */
  ["\uC1E9", "\uC1E9"], /* Hangul Syllable */
  ["\uC1EA", "\uC1EA"], /* Hangul Syllable */
  ["\uC1EB", "\uC1EB"], /* Hangul Syllable */
  ["\uC1EC", "\uC1EC"], /* Hangul Syllable */
  ["\uC1ED", "\uC1ED"], /* Hangul Syllable */
  ["\uC1EE", "\uC1EE"], /* Hangul Syllable */
  ["\uC1EF", "\uC1EF"], /* Hangul Syllable */
  ["\uC1F0", "\uC1F0"], /* Hangul Syllable */
  ["\uC1F1", "\uC1F1"], /* Hangul Syllable */
  ["\uC1F2", "\uC1F2"], /* Hangul Syllable */
  ["\uC1F3", "\uC1F3"], /* Hangul Syllable */
  ["\uC1F4", "\uC1F4"], /* Hangul Syllable */
  ["\uC1F5", "\uC1F5"], /* Hangul Syllable */
  ["\uC1F6", "\uC1F6"], /* Hangul Syllable */
  ["\uC1F7", "\uC1F7"], /* Hangul Syllable */
  ["\uC1F8", "\uC1F8"], /* Hangul Syllable */
  ["\uC1F9", "\uC1F9"], /* Hangul Syllable */
  ["\uC1FA", "\uC1FA"], /* Hangul Syllable */
  ["\uC1FB", "\uC1FB"], /* Hangul Syllable */
  ["\uC1FC", "\uC1FC"], /* Hangul Syllable */
  ["\uC1FD", "\uC1FD"], /* Hangul Syllable */
  ["\uC1FE", "\uC1FE"], /* Hangul Syllable */
  ["\uC1FF", "\uC1FF"], /* Hangul Syllable */
  ["\uC200", "\uC200"], /* Hangul Syllable */
  ["\uC201", "\uC201"], /* Hangul Syllable */
  ["\uC202", "\uC202"], /* Hangul Syllable */
  ["\uC203", "\uC203"], /* Hangul Syllable */
  ["\uC204", "\uC204"], /* Hangul Syllable */
  ["\uC205", "\uC205"], /* Hangul Syllable */
  ["\uC206", "\uC206"], /* Hangul Syllable */
  ["\uC207", "\uC207"], /* Hangul Syllable */
  ["\uC208", "\uC208"], /* Hangul Syllable */
  ["\uC209", "\uC209"], /* Hangul Syllable */
  ["\uC20A", "\uC20A"], /* Hangul Syllable */
  ["\uC20B", "\uC20B"], /* Hangul Syllable */
  ["\uC20C", "\uC20C"], /* Hangul Syllable */
  ["\uC20D", "\uC20D"], /* Hangul Syllable */
  ["\uC20E", "\uC20E"], /* Hangul Syllable */
  ["\uC20F", "\uC20F"], /* Hangul Syllable */
  ["\uC210", "\uC210"], /* Hangul Syllable */
  ["\uC211", "\uC211"], /* Hangul Syllable */
  ["\uC212", "\uC212"], /* Hangul Syllable */
  ["\uC213", "\uC213"], /* Hangul Syllable */
  ["\uC214", "\uC214"], /* Hangul Syllable */
  ["\uC215", "\uC215"], /* Hangul Syllable */
  ["\uC216", "\uC216"], /* Hangul Syllable */
  ["\uC217", "\uC217"], /* Hangul Syllable */
  ["\uC218", "\uC218"], /* Hangul Syllable */
  ["\uC219", "\uC219"], /* Hangul Syllable */
  ["\uC21A", "\uC21A"], /* Hangul Syllable */
  ["\uC21B", "\uC21B"], /* Hangul Syllable */
  ["\uC21C", "\uC21C"], /* Hangul Syllable */
  ["\uC21D", "\uC21D"], /* Hangul Syllable */
  ["\uC21E", "\uC21E"], /* Hangul Syllable */
  ["\uC21F", "\uC21F"], /* Hangul Syllable */
  ["\uC220", "\uC220"], /* Hangul Syllable */
  ["\uC221", "\uC221"], /* Hangul Syllable */
  ["\uC222", "\uC222"], /* Hangul Syllable */
  ["\uC223", "\uC223"], /* Hangul Syllable */
  ["\uC224", "\uC224"], /* Hangul Syllable */
  ["\uC225", "\uC225"], /* Hangul Syllable */
  ["\uC226", "\uC226"], /* Hangul Syllable */
  ["\uC227", "\uC227"], /* Hangul Syllable */
  ["\uC228", "\uC228"], /* Hangul Syllable */
  ["\uC229", "\uC229"], /* Hangul Syllable */
  ["\uC22A", "\uC22A"], /* Hangul Syllable */
  ["\uC22B", "\uC22B"], /* Hangul Syllable */
  ["\uC22C", "\uC22C"], /* Hangul Syllable */
  ["\uC22D", "\uC22D"], /* Hangul Syllable */
  ["\uC22E", "\uC22E"], /* Hangul Syllable */
  ["\uC22F", "\uC22F"], /* Hangul Syllable */
  ["\uC230", "\uC230"], /* Hangul Syllable */
  ["\uC231", "\uC231"], /* Hangul Syllable */
  ["\uC232", "\uC232"], /* Hangul Syllable */
  ["\uC233", "\uC233"], /* Hangul Syllable */
  ["\uC234", "\uC234"], /* Hangul Syllable */
  ["\uC235", "\uC235"], /* Hangul Syllable */
  ["\uC236", "\uC236"], /* Hangul Syllable */
  ["\uC237", "\uC237"], /* Hangul Syllable */
  ["\uC238", "\uC238"], /* Hangul Syllable */
  ["\uC239", "\uC239"], /* Hangul Syllable */
  ["\uC23A", "\uC23A"], /* Hangul Syllable */
  ["\uC23B", "\uC23B"], /* Hangul Syllable */
  ["\uC23C", "\uC23C"], /* Hangul Syllable */
  ["\uC23D", "\uC23D"], /* Hangul Syllable */
  ["\uC23E", "\uC23E"], /* Hangul Syllable */
  ["\uC23F", "\uC23F"], /* Hangul Syllable */
  ["\uC240", "\uC240"], /* Hangul Syllable */
  ["\uC241", "\uC241"], /* Hangul Syllable */
  ["\uC242", "\uC242"], /* Hangul Syllable */
  ["\uC243", "\uC243"], /* Hangul Syllable */
  ["\uC244", "\uC244"], /* Hangul Syllable */
  ["\uC245", "\uC245"], /* Hangul Syllable */
  ["\uC246", "\uC246"], /* Hangul Syllable */
  ["\uC247", "\uC247"], /* Hangul Syllable */
  ["\uC248", "\uC248"], /* Hangul Syllable */
  ["\uC249", "\uC249"], /* Hangul Syllable */
  ["\uC24A", "\uC24A"], /* Hangul Syllable */
  ["\uC24B", "\uC24B"], /* Hangul Syllable */
  ["\uC24C", "\uC24C"], /* Hangul Syllable */
  ["\uC24D", "\uC24D"], /* Hangul Syllable */
  ["\uC24E", "\uC24E"], /* Hangul Syllable */
  ["\uC24F", "\uC24F"], /* Hangul Syllable */
  ["\uC250", "\uC250"], /* Hangul Syllable */
  ["\uC251", "\uC251"], /* Hangul Syllable */
  ["\uC252", "\uC252"], /* Hangul Syllable */
  ["\uC253", "\uC253"], /* Hangul Syllable */
  ["\uC254", "\uC254"], /* Hangul Syllable */
  ["\uC255", "\uC255"], /* Hangul Syllable */
  ["\uC256", "\uC256"], /* Hangul Syllable */
  ["\uC257", "\uC257"], /* Hangul Syllable */
  ["\uC258", "\uC258"], /* Hangul Syllable */
  ["\uC259", "\uC259"], /* Hangul Syllable */
  ["\uC25A", "\uC25A"], /* Hangul Syllable */
  ["\uC25B", "\uC25B"], /* Hangul Syllable */
  ["\uC25C", "\uC25C"], /* Hangul Syllable */
  ["\uC25D", "\uC25D"], /* Hangul Syllable */
  ["\uC25E", "\uC25E"], /* Hangul Syllable */
  ["\uC25F", "\uC25F"], /* Hangul Syllable */
  ["\uC260", "\uC260"], /* Hangul Syllable */
  ["\uC261", "\uC261"], /* Hangul Syllable */
  ["\uC262", "\uC262"], /* Hangul Syllable */
  ["\uC263", "\uC263"], /* Hangul Syllable */
  ["\uC264", "\uC264"], /* Hangul Syllable */
  ["\uC265", "\uC265"], /* Hangul Syllable */
  ["\uC266", "\uC266"], /* Hangul Syllable */
  ["\uC267", "\uC267"], /* Hangul Syllable */
  ["\uC268", "\uC268"], /* Hangul Syllable */
  ["\uC269", "\uC269"], /* Hangul Syllable */
  ["\uC26A", "\uC26A"], /* Hangul Syllable */
  ["\uC26B", "\uC26B"], /* Hangul Syllable */
  ["\uC26C", "\uC26C"], /* Hangul Syllable */
  ["\uC26D", "\uC26D"], /* Hangul Syllable */
  ["\uC26E", "\uC26E"], /* Hangul Syllable */
  ["\uC26F", "\uC26F"], /* Hangul Syllable */
  ["\uC270", "\uC270"], /* Hangul Syllable */
  ["\uC271", "\uC271"], /* Hangul Syllable */
  ["\uC272", "\uC272"], /* Hangul Syllable */
  ["\uC273", "\uC273"], /* Hangul Syllable */
  ["\uC274", "\uC274"], /* Hangul Syllable */
  ["\uC275", "\uC275"], /* Hangul Syllable */
  ["\uC276", "\uC276"], /* Hangul Syllable */
  ["\uC277", "\uC277"], /* Hangul Syllable */
  ["\uC278", "\uC278"], /* Hangul Syllable */
  ["\uC279", "\uC279"], /* Hangul Syllable */
  ["\uC27A", "\uC27A"], /* Hangul Syllable */
  ["\uC27B", "\uC27B"], /* Hangul Syllable */
  ["\uC27C", "\uC27C"], /* Hangul Syllable */
  ["\uC27D", "\uC27D"], /* Hangul Syllable */
  ["\uC27E", "\uC27E"], /* Hangul Syllable */
  ["\uC27F", "\uC27F"], /* Hangul Syllable */
  ["\uC280", "\uC280"], /* Hangul Syllable */
  ["\uC281", "\uC281"], /* Hangul Syllable */
  ["\uC282", "\uC282"], /* Hangul Syllable */
  ["\uC283", "\uC283"], /* Hangul Syllable */
  ["\uC284", "\uC284"], /* Hangul Syllable */
  ["\uC285", "\uC285"], /* Hangul Syllable */
  ["\uC286", "\uC286"], /* Hangul Syllable */
  ["\uC287", "\uC287"], /* Hangul Syllable */
  ["\uC288", "\uC288"], /* Hangul Syllable */
  ["\uC289", "\uC289"], /* Hangul Syllable */
  ["\uC28A", "\uC28A"], /* Hangul Syllable */
  ["\uC28B", "\uC28B"], /* Hangul Syllable */
  ["\uC28C", "\uC28C"], /* Hangul Syllable */
  ["\uC28D", "\uC28D"], /* Hangul Syllable */
  ["\uC28E", "\uC28E"], /* Hangul Syllable */
  ["\uC28F", "\uC28F"], /* Hangul Syllable */
  ["\uC290", "\uC290"], /* Hangul Syllable */
  ["\uC291", "\uC291"], /* Hangul Syllable */
  ["\uC292", "\uC292"], /* Hangul Syllable */
  ["\uC293", "\uC293"], /* Hangul Syllable */
  ["\uC294", "\uC294"], /* Hangul Syllable */
  ["\uC295", "\uC295"], /* Hangul Syllable */
  ["\uC296", "\uC296"], /* Hangul Syllable */
  ["\uC297", "\uC297"], /* Hangul Syllable */
  ["\uC298", "\uC298"], /* Hangul Syllable */
  ["\uC299", "\uC299"], /* Hangul Syllable */
  ["\uC29A", "\uC29A"], /* Hangul Syllable */
  ["\uC29B", "\uC29B"], /* Hangul Syllable */
  ["\uC29C", "\uC29C"], /* Hangul Syllable */
  ["\uC29D", "\uC29D"], /* Hangul Syllable */
  ["\uC29E", "\uC29E"], /* Hangul Syllable */
  ["\uC29F", "\uC29F"], /* Hangul Syllable */
  ["\uC2A0", "\uC2A0"], /* Hangul Syllable */
  ["\uC2A1", "\uC2A1"], /* Hangul Syllable */
  ["\uC2A2", "\uC2A2"], /* Hangul Syllable */
  ["\uC2A3", "\uC2A3"], /* Hangul Syllable */
  ["\uC2A4", "\uC2A4"], /* Hangul Syllable */
  ["\uC2A5", "\uC2A5"], /* Hangul Syllable */
  ["\uC2A6", "\uC2A6"], /* Hangul Syllable */
  ["\uC2A7", "\uC2A7"], /* Hangul Syllable */
  ["\uC2A8", "\uC2A8"], /* Hangul Syllable */
  ["\uC2A9", "\uC2A9"], /* Hangul Syllable */
  ["\uC2AA", "\uC2AA"], /* Hangul Syllable */
  ["\uC2AB", "\uC2AB"], /* Hangul Syllable */
  ["\uC2AC", "\uC2AC"], /* Hangul Syllable */
  ["\uC2AD", "\uC2AD"], /* Hangul Syllable */
  ["\uC2AE", "\uC2AE"], /* Hangul Syllable */
  ["\uC2AF", "\uC2AF"], /* Hangul Syllable */
  ["\uC2B0", "\uC2B0"], /* Hangul Syllable */
  ["\uC2B1", "\uC2B1"], /* Hangul Syllable */
  ["\uC2B2", "\uC2B2"], /* Hangul Syllable */
  ["\uC2B3", "\uC2B3"], /* Hangul Syllable */
  ["\uC2B4", "\uC2B4"], /* Hangul Syllable */
  ["\uC2B5", "\uC2B5"], /* Hangul Syllable */
  ["\uC2B6", "\uC2B6"], /* Hangul Syllable */
  ["\uC2B7", "\uC2B7"], /* Hangul Syllable */
  ["\uC2B8", "\uC2B8"], /* Hangul Syllable */
  ["\uC2B9", "\uC2B9"], /* Hangul Syllable */
  ["\uC2BA", "\uC2BA"], /* Hangul Syllable */
  ["\uC2BB", "\uC2BB"], /* Hangul Syllable */
  ["\uC2BC", "\uC2BC"], /* Hangul Syllable */
  ["\uC2BD", "\uC2BD"], /* Hangul Syllable */
  ["\uC2BE", "\uC2BE"], /* Hangul Syllable */
  ["\uC2BF", "\uC2BF"], /* Hangul Syllable */
  ["\uC2C0", "\uC2C0"], /* Hangul Syllable */
  ["\uC2C1", "\uC2C1"], /* Hangul Syllable */
  ["\uC2C2", "\uC2C2"], /* Hangul Syllable */
  ["\uC2C3", "\uC2C3"], /* Hangul Syllable */
  ["\uC2C4", "\uC2C4"], /* Hangul Syllable */
  ["\uC2C5", "\uC2C5"], /* Hangul Syllable */
  ["\uC2C6", "\uC2C6"], /* Hangul Syllable */
  ["\uC2C7", "\uC2C7"], /* Hangul Syllable */
  ["\uC2C8", "\uC2C8"], /* Hangul Syllable */
  ["\uC2C9", "\uC2C9"], /* Hangul Syllable */
  ["\uC2CA", "\uC2CA"], /* Hangul Syllable */
  ["\uC2CB", "\uC2CB"], /* Hangul Syllable */
  ["\uC2CC", "\uC2CC"], /* Hangul Syllable */
  ["\uC2CD", "\uC2CD"], /* Hangul Syllable */
  ["\uC2CE", "\uC2CE"], /* Hangul Syllable */
  ["\uC2CF", "\uC2CF"], /* Hangul Syllable */
  ["\uC2D0", "\uC2D0"], /* Hangul Syllable */
  ["\uC2D1", "\uC2D1"], /* Hangul Syllable */
  ["\uC2D2", "\uC2D2"], /* Hangul Syllable */
  ["\uC2D3", "\uC2D3"], /* Hangul Syllable */
  ["\uC2D4", "\uC2D4"], /* Hangul Syllable */
  ["\uC2D5", "\uC2D5"], /* Hangul Syllable */
  ["\uC2D6", "\uC2D6"], /* Hangul Syllable */
  ["\uC2D7", "\uC2D7"], /* Hangul Syllable */
  ["\uC2D8", "\uC2D8"], /* Hangul Syllable */
  ["\uC2D9", "\uC2D9"], /* Hangul Syllable */
  ["\uC2DA", "\uC2DA"], /* Hangul Syllable */
  ["\uC2DB", "\uC2DB"], /* Hangul Syllable */
  ["\uC2DC", "\uC2DC"], /* Hangul Syllable */
  ["\uC2DD", "\uC2DD"], /* Hangul Syllable */
  ["\uC2DE", "\uC2DE"], /* Hangul Syllable */
  ["\uC2DF", "\uC2DF"], /* Hangul Syllable */
  ["\uC2E0", "\uC2E0"], /* Hangul Syllable */
  ["\uC2E1", "\uC2E1"], /* Hangul Syllable */
  ["\uC2E2", "\uC2E2"], /* Hangul Syllable */
  ["\uC2E3", "\uC2E3"], /* Hangul Syllable */
  ["\uC2E4", "\uC2E4"], /* Hangul Syllable */
  ["\uC2E5", "\uC2E5"], /* Hangul Syllable */
  ["\uC2E6", "\uC2E6"], /* Hangul Syllable */
  ["\uC2E7", "\uC2E7"], /* Hangul Syllable */
  ["\uC2E8", "\uC2E8"], /* Hangul Syllable */
  ["\uC2E9", "\uC2E9"], /* Hangul Syllable */
  ["\uC2EA", "\uC2EA"], /* Hangul Syllable */
  ["\uC2EB", "\uC2EB"], /* Hangul Syllable */
  ["\uC2EC", "\uC2EC"], /* Hangul Syllable */
  ["\uC2ED", "\uC2ED"], /* Hangul Syllable */
  ["\uC2EE", "\uC2EE"], /* Hangul Syllable */
  ["\uC2EF", "\uC2EF"], /* Hangul Syllable */
  ["\uC2F0", "\uC2F0"], /* Hangul Syllable */
  ["\uC2F1", "\uC2F1"], /* Hangul Syllable */
  ["\uC2F2", "\uC2F2"], /* Hangul Syllable */
  ["\uC2F3", "\uC2F3"], /* Hangul Syllable */
  ["\uC2F4", "\uC2F4"], /* Hangul Syllable */
  ["\uC2F5", "\uC2F5"], /* Hangul Syllable */
  ["\uC2F6", "\uC2F6"], /* Hangul Syllable */
  ["\uC2F7", "\uC2F7"], /* Hangul Syllable */
  ["\uC2F8", "\uC2F8"], /* Hangul Syllable */
  ["\uC2F9", "\uC2F9"], /* Hangul Syllable */
  ["\uC2FA", "\uC2FA"], /* Hangul Syllable */
  ["\uC2FB", "\uC2FB"], /* Hangul Syllable */
  ["\uC2FC", "\uC2FC"], /* Hangul Syllable */
  ["\uC2FD", "\uC2FD"], /* Hangul Syllable */
  ["\uC2FE", "\uC2FE"], /* Hangul Syllable */
  ["\uC2FF", "\uC2FF"], /* Hangul Syllable */
  ["\uC300", "\uC300"], /* Hangul Syllable */
  ["\uC301", "\uC301"], /* Hangul Syllable */
  ["\uC302", "\uC302"], /* Hangul Syllable */
  ["\uC303", "\uC303"], /* Hangul Syllable */
  ["\uC304", "\uC304"], /* Hangul Syllable */
  ["\uC305", "\uC305"], /* Hangul Syllable */
  ["\uC306", "\uC306"], /* Hangul Syllable */
  ["\uC307", "\uC307"], /* Hangul Syllable */
  ["\uC308", "\uC308"], /* Hangul Syllable */
  ["\uC309", "\uC309"], /* Hangul Syllable */
  ["\uC30A", "\uC30A"], /* Hangul Syllable */
  ["\uC30B", "\uC30B"], /* Hangul Syllable */
  ["\uC30C", "\uC30C"], /* Hangul Syllable */
  ["\uC30D", "\uC30D"], /* Hangul Syllable */
  ["\uC30E", "\uC30E"], /* Hangul Syllable */
  ["\uC30F", "\uC30F"], /* Hangul Syllable */
  ["\uC310", "\uC310"], /* Hangul Syllable */
  ["\uC311", "\uC311"], /* Hangul Syllable */
  ["\uC312", "\uC312"], /* Hangul Syllable */
  ["\uC313", "\uC313"], /* Hangul Syllable */
  ["\uC314", "\uC314"], /* Hangul Syllable */
  ["\uC315", "\uC315"], /* Hangul Syllable */
  ["\uC316", "\uC316"], /* Hangul Syllable */
  ["\uC317", "\uC317"], /* Hangul Syllable */
  ["\uC318", "\uC318"], /* Hangul Syllable */
  ["\uC319", "\uC319"], /* Hangul Syllable */
  ["\uC31A", "\uC31A"], /* Hangul Syllable */
  ["\uC31B", "\uC31B"], /* Hangul Syllable */
  ["\uC31C", "\uC31C"], /* Hangul Syllable */
  ["\uC31D", "\uC31D"], /* Hangul Syllable */
  ["\uC31E", "\uC31E"], /* Hangul Syllable */
  ["\uC31F", "\uC31F"], /* Hangul Syllable */
  ["\uC320", "\uC320"], /* Hangul Syllable */
  ["\uC321", "\uC321"], /* Hangul Syllable */
  ["\uC322", "\uC322"], /* Hangul Syllable */
  ["\uC323", "\uC323"], /* Hangul Syllable */
  ["\uC324", "\uC324"], /* Hangul Syllable */
  ["\uC325", "\uC325"], /* Hangul Syllable */
  ["\uC326", "\uC326"], /* Hangul Syllable */
  ["\uC327", "\uC327"], /* Hangul Syllable */
  ["\uC328", "\uC328"], /* Hangul Syllable */
  ["\uC329", "\uC329"], /* Hangul Syllable */
  ["\uC32A", "\uC32A"], /* Hangul Syllable */
  ["\uC32B", "\uC32B"], /* Hangul Syllable */
  ["\uC32C", "\uC32C"], /* Hangul Syllable */
  ["\uC32D", "\uC32D"], /* Hangul Syllable */
  ["\uC32E", "\uC32E"], /* Hangul Syllable */
  ["\uC32F", "\uC32F"], /* Hangul Syllable */
  ["\uC330", "\uC330"], /* Hangul Syllable */
  ["\uC331", "\uC331"], /* Hangul Syllable */
  ["\uC332", "\uC332"], /* Hangul Syllable */
  ["\uC333", "\uC333"], /* Hangul Syllable */
  ["\uC334", "\uC334"], /* Hangul Syllable */
  ["\uC335", "\uC335"], /* Hangul Syllable */
  ["\uC336", "\uC336"], /* Hangul Syllable */
  ["\uC337", "\uC337"], /* Hangul Syllable */
  ["\uC338", "\uC338"], /* Hangul Syllable */
  ["\uC339", "\uC339"], /* Hangul Syllable */
  ["\uC33A", "\uC33A"], /* Hangul Syllable */
  ["\uC33B", "\uC33B"], /* Hangul Syllable */
  ["\uC33C", "\uC33C"], /* Hangul Syllable */
  ["\uC33D", "\uC33D"], /* Hangul Syllable */
  ["\uC33E", "\uC33E"], /* Hangul Syllable */
  ["\uC33F", "\uC33F"], /* Hangul Syllable */
  ["\uC340", "\uC340"], /* Hangul Syllable */
  ["\uC341", "\uC341"], /* Hangul Syllable */
  ["\uC342", "\uC342"], /* Hangul Syllable */
  ["\uC343", "\uC343"], /* Hangul Syllable */
  ["\uC344", "\uC344"], /* Hangul Syllable */
  ["\uC345", "\uC345"], /* Hangul Syllable */
  ["\uC346", "\uC346"], /* Hangul Syllable */
  ["\uC347", "\uC347"], /* Hangul Syllable */
  ["\uC348", "\uC348"], /* Hangul Syllable */
  ["\uC349", "\uC349"], /* Hangul Syllable */
  ["\uC34A", "\uC34A"], /* Hangul Syllable */
  ["\uC34B", "\uC34B"], /* Hangul Syllable */
  ["\uC34C", "\uC34C"], /* Hangul Syllable */
  ["\uC34D", "\uC34D"], /* Hangul Syllable */
  ["\uC34E", "\uC34E"], /* Hangul Syllable */
  ["\uC34F", "\uC34F"], /* Hangul Syllable */
  ["\uC350", "\uC350"], /* Hangul Syllable */
  ["\uC351", "\uC351"], /* Hangul Syllable */
  ["\uC352", "\uC352"], /* Hangul Syllable */
  ["\uC353", "\uC353"], /* Hangul Syllable */
  ["\uC354", "\uC354"], /* Hangul Syllable */
  ["\uC355", "\uC355"], /* Hangul Syllable */
  ["\uC356", "\uC356"], /* Hangul Syllable */
  ["\uC357", "\uC357"], /* Hangul Syllable */
  ["\uC358", "\uC358"], /* Hangul Syllable */
  ["\uC359", "\uC359"], /* Hangul Syllable */
  ["\uC35A", "\uC35A"], /* Hangul Syllable */
  ["\uC35B", "\uC35B"], /* Hangul Syllable */
  ["\uC35C", "\uC35C"], /* Hangul Syllable */
  ["\uC35D", "\uC35D"], /* Hangul Syllable */
  ["\uC35E", "\uC35E"], /* Hangul Syllable */
  ["\uC35F", "\uC35F"], /* Hangul Syllable */
  ["\uC360", "\uC360"], /* Hangul Syllable */
  ["\uC361", "\uC361"], /* Hangul Syllable */
  ["\uC362", "\uC362"], /* Hangul Syllable */
  ["\uC363", "\uC363"], /* Hangul Syllable */
  ["\uC364", "\uC364"], /* Hangul Syllable */
  ["\uC365", "\uC365"], /* Hangul Syllable */
  ["\uC366", "\uC366"], /* Hangul Syllable */
  ["\uC367", "\uC367"], /* Hangul Syllable */
  ["\uC368", "\uC368"], /* Hangul Syllable */
  ["\uC369", "\uC369"], /* Hangul Syllable */
  ["\uC36A", "\uC36A"], /* Hangul Syllable */
  ["\uC36B", "\uC36B"], /* Hangul Syllable */
  ["\uC36C", "\uC36C"], /* Hangul Syllable */
  ["\uC36D", "\uC36D"], /* Hangul Syllable */
  ["\uC36E", "\uC36E"], /* Hangul Syllable */
  ["\uC36F", "\uC36F"], /* Hangul Syllable */
  ["\uC370", "\uC370"], /* Hangul Syllable */
  ["\uC371", "\uC371"], /* Hangul Syllable */
  ["\uC372", "\uC372"], /* Hangul Syllable */
  ["\uC373", "\uC373"], /* Hangul Syllable */
  ["\uC374", "\uC374"], /* Hangul Syllable */
  ["\uC375", "\uC375"], /* Hangul Syllable */
  ["\uC376", "\uC376"], /* Hangul Syllable */
  ["\uC377", "\uC377"], /* Hangul Syllable */
  ["\uC378", "\uC378"], /* Hangul Syllable */
  ["\uC379", "\uC379"], /* Hangul Syllable */
  ["\uC37A", "\uC37A"], /* Hangul Syllable */
  ["\uC37B", "\uC37B"], /* Hangul Syllable */
  ["\uC37C", "\uC37C"], /* Hangul Syllable */
  ["\uC37D", "\uC37D"], /* Hangul Syllable */
  ["\uC37E", "\uC37E"], /* Hangul Syllable */
  ["\uC37F", "\uC37F"], /* Hangul Syllable */
  ["\uC380", "\uC380"], /* Hangul Syllable */
  ["\uC381", "\uC381"], /* Hangul Syllable */
  ["\uC382", "\uC382"], /* Hangul Syllable */
  ["\uC383", "\uC383"], /* Hangul Syllable */
  ["\uC384", "\uC384"], /* Hangul Syllable */
  ["\uC385", "\uC385"], /* Hangul Syllable */
  ["\uC386", "\uC386"], /* Hangul Syllable */
  ["\uC387", "\uC387"], /* Hangul Syllable */
  ["\uC388", "\uC388"], /* Hangul Syllable */
  ["\uC389", "\uC389"], /* Hangul Syllable */
  ["\uC38A", "\uC38A"], /* Hangul Syllable */
  ["\uC38B", "\uC38B"], /* Hangul Syllable */
  ["\uC38C", "\uC38C"], /* Hangul Syllable */
  ["\uC38D", "\uC38D"], /* Hangul Syllable */
  ["\uC38E", "\uC38E"], /* Hangul Syllable */
  ["\uC38F", "\uC38F"], /* Hangul Syllable */
  ["\uC390", "\uC390"], /* Hangul Syllable */
  ["\uC391", "\uC391"], /* Hangul Syllable */
  ["\uC392", "\uC392"], /* Hangul Syllable */
  ["\uC393", "\uC393"], /* Hangul Syllable */
  ["\uC394", "\uC394"], /* Hangul Syllable */
  ["\uC395", "\uC395"], /* Hangul Syllable */
  ["\uC396", "\uC396"], /* Hangul Syllable */
  ["\uC397", "\uC397"], /* Hangul Syllable */
  ["\uC398", "\uC398"], /* Hangul Syllable */
  ["\uC399", "\uC399"], /* Hangul Syllable */
  ["\uC39A", "\uC39A"], /* Hangul Syllable */
  ["\uC39B", "\uC39B"], /* Hangul Syllable */
  ["\uC39C", "\uC39C"], /* Hangul Syllable */
  ["\uC39D", "\uC39D"], /* Hangul Syllable */
  ["\uC39E", "\uC39E"], /* Hangul Syllable */
  ["\uC39F", "\uC39F"], /* Hangul Syllable */
  ["\uC3A0", "\uC3A0"], /* Hangul Syllable */
  ["\uC3A1", "\uC3A1"], /* Hangul Syllable */
  ["\uC3A2", "\uC3A2"], /* Hangul Syllable */
  ["\uC3A3", "\uC3A3"], /* Hangul Syllable */
  ["\uC3A4", "\uC3A4"], /* Hangul Syllable */
  ["\uC3A5", "\uC3A5"], /* Hangul Syllable */
  ["\uC3A6", "\uC3A6"], /* Hangul Syllable */
  ["\uC3A7", "\uC3A7"], /* Hangul Syllable */
  ["\uC3A8", "\uC3A8"], /* Hangul Syllable */
  ["\uC3A9", "\uC3A9"], /* Hangul Syllable */
  ["\uC3AA", "\uC3AA"], /* Hangul Syllable */
  ["\uC3AB", "\uC3AB"], /* Hangul Syllable */
  ["\uC3AC", "\uC3AC"], /* Hangul Syllable */
  ["\uC3AD", "\uC3AD"], /* Hangul Syllable */
  ["\uC3AE", "\uC3AE"], /* Hangul Syllable */
  ["\uC3AF", "\uC3AF"], /* Hangul Syllable */
  ["\uC3B0", "\uC3B0"], /* Hangul Syllable */
  ["\uC3B1", "\uC3B1"], /* Hangul Syllable */
  ["\uC3B2", "\uC3B2"], /* Hangul Syllable */
  ["\uC3B3", "\uC3B3"], /* Hangul Syllable */
  ["\uC3B4", "\uC3B4"], /* Hangul Syllable */
  ["\uC3B5", "\uC3B5"], /* Hangul Syllable */
  ["\uC3B6", "\uC3B6"], /* Hangul Syllable */
  ["\uC3B7", "\uC3B7"], /* Hangul Syllable */
  ["\uC3B8", "\uC3B8"], /* Hangul Syllable */
  ["\uC3B9", "\uC3B9"], /* Hangul Syllable */
  ["\uC3BA", "\uC3BA"], /* Hangul Syllable */
  ["\uC3BB", "\uC3BB"], /* Hangul Syllable */
  ["\uC3BC", "\uC3BC"], /* Hangul Syllable */
  ["\uC3BD", "\uC3BD"], /* Hangul Syllable */
  ["\uC3BE", "\uC3BE"], /* Hangul Syllable */
  ["\uC3BF", "\uC3BF"], /* Hangul Syllable */
  ["\uC3C0", "\uC3C0"], /* Hangul Syllable */
  ["\uC3C1", "\uC3C1"], /* Hangul Syllable */
  ["\uC3C2", "\uC3C2"], /* Hangul Syllable */
  ["\uC3C3", "\uC3C3"], /* Hangul Syllable */
  ["\uC3C4", "\uC3C4"], /* Hangul Syllable */
  ["\uC3C5", "\uC3C5"], /* Hangul Syllable */
  ["\uC3C6", "\uC3C6"], /* Hangul Syllable */
  ["\uC3C7", "\uC3C7"], /* Hangul Syllable */
  ["\uC3C8", "\uC3C8"], /* Hangul Syllable */
  ["\uC3C9", "\uC3C9"], /* Hangul Syllable */
  ["\uC3CA", "\uC3CA"], /* Hangul Syllable */
  ["\uC3CB", "\uC3CB"], /* Hangul Syllable */
  ["\uC3CC", "\uC3CC"], /* Hangul Syllable */
  ["\uC3CD", "\uC3CD"], /* Hangul Syllable */
  ["\uC3CE", "\uC3CE"], /* Hangul Syllable */
  ["\uC3CF", "\uC3CF"], /* Hangul Syllable */
  ["\uC3D0", "\uC3D0"], /* Hangul Syllable */
  ["\uC3D1", "\uC3D1"], /* Hangul Syllable */
  ["\uC3D2", "\uC3D2"], /* Hangul Syllable */
  ["\uC3D3", "\uC3D3"], /* Hangul Syllable */
  ["\uC3D4", "\uC3D4"], /* Hangul Syllable */
  ["\uC3D5", "\uC3D5"], /* Hangul Syllable */
  ["\uC3D6", "\uC3D6"], /* Hangul Syllable */
  ["\uC3D7", "\uC3D7"], /* Hangul Syllable */
  ["\uC3D8", "\uC3D8"], /* Hangul Syllable */
  ["\uC3D9", "\uC3D9"], /* Hangul Syllable */
  ["\uC3DA", "\uC3DA"], /* Hangul Syllable */
  ["\uC3DB", "\uC3DB"], /* Hangul Syllable */
  ["\uC3DC", "\uC3DC"], /* Hangul Syllable */
  ["\uC3DD", "\uC3DD"], /* Hangul Syllable */
  ["\uC3DE", "\uC3DE"], /* Hangul Syllable */
  ["\uC3DF", "\uC3DF"], /* Hangul Syllable */
  ["\uC3E0", "\uC3E0"], /* Hangul Syllable */
  ["\uC3E1", "\uC3E1"], /* Hangul Syllable */
  ["\uC3E2", "\uC3E2"], /* Hangul Syllable */
  ["\uC3E3", "\uC3E3"], /* Hangul Syllable */
  ["\uC3E4", "\uC3E4"], /* Hangul Syllable */
  ["\uC3E5", "\uC3E5"], /* Hangul Syllable */
  ["\uC3E6", "\uC3E6"], /* Hangul Syllable */
  ["\uC3E7", "\uC3E7"], /* Hangul Syllable */
  ["\uC3E8", "\uC3E8"], /* Hangul Syllable */
  ["\uC3E9", "\uC3E9"], /* Hangul Syllable */
  ["\uC3EA", "\uC3EA"], /* Hangul Syllable */
  ["\uC3EB", "\uC3EB"], /* Hangul Syllable */
  ["\uC3EC", "\uC3EC"], /* Hangul Syllable */
  ["\uC3ED", "\uC3ED"], /* Hangul Syllable */
  ["\uC3EE", "\uC3EE"], /* Hangul Syllable */
  ["\uC3EF", "\uC3EF"], /* Hangul Syllable */
  ["\uC3F0", "\uC3F0"], /* Hangul Syllable */
  ["\uC3F1", "\uC3F1"], /* Hangul Syllable */
  ["\uC3F2", "\uC3F2"], /* Hangul Syllable */
  ["\uC3F3", "\uC3F3"], /* Hangul Syllable */
  ["\uC3F4", "\uC3F4"], /* Hangul Syllable */
  ["\uC3F5", "\uC3F5"], /* Hangul Syllable */
  ["\uC3F6", "\uC3F6"], /* Hangul Syllable */
  ["\uC3F7", "\uC3F7"], /* Hangul Syllable */
  ["\uC3F8", "\uC3F8"], /* Hangul Syllable */
  ["\uC3F9", "\uC3F9"], /* Hangul Syllable */
  ["\uC3FA", "\uC3FA"], /* Hangul Syllable */
  ["\uC3FB", "\uC3FB"], /* Hangul Syllable */
  ["\uC3FC", "\uC3FC"], /* Hangul Syllable */
  ["\uC3FD", "\uC3FD"], /* Hangul Syllable */
  ["\uC3FE", "\uC3FE"], /* Hangul Syllable */
  ["\uC3FF", "\uC3FF"], /* Hangul Syllable */
  ["\uC400", "\uC400"], /* Hangul Syllable */
  ["\uC401", "\uC401"], /* Hangul Syllable */
  ["\uC402", "\uC402"], /* Hangul Syllable */
  ["\uC403", "\uC403"], /* Hangul Syllable */
  ["\uC404", "\uC404"], /* Hangul Syllable */
  ["\uC405", "\uC405"], /* Hangul Syllable */
  ["\uC406", "\uC406"], /* Hangul Syllable */
  ["\uC407", "\uC407"], /* Hangul Syllable */
  ["\uC408", "\uC408"], /* Hangul Syllable */
  ["\uC409", "\uC409"], /* Hangul Syllable */
  ["\uC40A", "\uC40A"], /* Hangul Syllable */
  ["\uC40B", "\uC40B"], /* Hangul Syllable */
  ["\uC40C", "\uC40C"], /* Hangul Syllable */
  ["\uC40D", "\uC40D"], /* Hangul Syllable */
  ["\uC40E", "\uC40E"], /* Hangul Syllable */
  ["\uC40F", "\uC40F"], /* Hangul Syllable */
  ["\uC410", "\uC410"], /* Hangul Syllable */
  ["\uC411", "\uC411"], /* Hangul Syllable */
  ["\uC412", "\uC412"], /* Hangul Syllable */
  ["\uC413", "\uC413"], /* Hangul Syllable */
  ["\uC414", "\uC414"], /* Hangul Syllable */
  ["\uC415", "\uC415"], /* Hangul Syllable */
  ["\uC416", "\uC416"], /* Hangul Syllable */
  ["\uC417", "\uC417"], /* Hangul Syllable */
  ["\uC418", "\uC418"], /* Hangul Syllable */
  ["\uC419", "\uC419"], /* Hangul Syllable */
  ["\uC41A", "\uC41A"], /* Hangul Syllable */
  ["\uC41B", "\uC41B"], /* Hangul Syllable */
  ["\uC41C", "\uC41C"], /* Hangul Syllable */
  ["\uC41D", "\uC41D"], /* Hangul Syllable */
  ["\uC41E", "\uC41E"], /* Hangul Syllable */
  ["\uC41F", "\uC41F"], /* Hangul Syllable */
  ["\uC420", "\uC420"], /* Hangul Syllable */
  ["\uC421", "\uC421"], /* Hangul Syllable */
  ["\uC422", "\uC422"], /* Hangul Syllable */
  ["\uC423", "\uC423"], /* Hangul Syllable */
  ["\uC424", "\uC424"], /* Hangul Syllable */
  ["\uC425", "\uC425"], /* Hangul Syllable */
  ["\uC426", "\uC426"], /* Hangul Syllable */
  ["\uC427", "\uC427"], /* Hangul Syllable */
  ["\uC428", "\uC428"], /* Hangul Syllable */
  ["\uC429", "\uC429"], /* Hangul Syllable */
  ["\uC42A", "\uC42A"], /* Hangul Syllable */
  ["\uC42B", "\uC42B"], /* Hangul Syllable */
  ["\uC42C", "\uC42C"], /* Hangul Syllable */
  ["\uC42D", "\uC42D"], /* Hangul Syllable */
  ["\uC42E", "\uC42E"], /* Hangul Syllable */
  ["\uC42F", "\uC42F"], /* Hangul Syllable */
  ["\uC430", "\uC430"], /* Hangul Syllable */
  ["\uC431", "\uC431"], /* Hangul Syllable */
  ["\uC432", "\uC432"], /* Hangul Syllable */
  ["\uC433", "\uC433"], /* Hangul Syllable */
  ["\uC434", "\uC434"], /* Hangul Syllable */
  ["\uC435", "\uC435"], /* Hangul Syllable */
  ["\uC436", "\uC436"], /* Hangul Syllable */
  ["\uC437", "\uC437"], /* Hangul Syllable */
  ["\uC438", "\uC438"], /* Hangul Syllable */
  ["\uC439", "\uC439"], /* Hangul Syllable */
  ["\uC43A", "\uC43A"], /* Hangul Syllable */
  ["\uC43B", "\uC43B"], /* Hangul Syllable */
  ["\uC43C", "\uC43C"], /* Hangul Syllable */
  ["\uC43D", "\uC43D"], /* Hangul Syllable */
  ["\uC43E", "\uC43E"], /* Hangul Syllable */
  ["\uC43F", "\uC43F"], /* Hangul Syllable */
  ["\uC440", "\uC440"], /* Hangul Syllable */
  ["\uC441", "\uC441"], /* Hangul Syllable */
  ["\uC442", "\uC442"], /* Hangul Syllable */
  ["\uC443", "\uC443"], /* Hangul Syllable */
  ["\uC444", "\uC444"], /* Hangul Syllable */
  ["\uC445", "\uC445"], /* Hangul Syllable */
  ["\uC446", "\uC446"], /* Hangul Syllable */
  ["\uC447", "\uC447"], /* Hangul Syllable */
  ["\uC448", "\uC448"], /* Hangul Syllable */
  ["\uC449", "\uC449"], /* Hangul Syllable */
  ["\uC44A", "\uC44A"], /* Hangul Syllable */
  ["\uC44B", "\uC44B"], /* Hangul Syllable */
  ["\uC44C", "\uC44C"], /* Hangul Syllable */
  ["\uC44D", "\uC44D"], /* Hangul Syllable */
  ["\uC44E", "\uC44E"], /* Hangul Syllable */
  ["\uC44F", "\uC44F"], /* Hangul Syllable */
  ["\uC450", "\uC450"], /* Hangul Syllable */
  ["\uC451", "\uC451"], /* Hangul Syllable */
  ["\uC452", "\uC452"], /* Hangul Syllable */
  ["\uC453", "\uC453"], /* Hangul Syllable */
  ["\uC454", "\uC454"], /* Hangul Syllable */
  ["\uC455", "\uC455"], /* Hangul Syllable */
  ["\uC456", "\uC456"], /* Hangul Syllable */
  ["\uC457", "\uC457"], /* Hangul Syllable */
  ["\uC458", "\uC458"], /* Hangul Syllable */
  ["\uC459", "\uC459"], /* Hangul Syllable */
  ["\uC45A", "\uC45A"], /* Hangul Syllable */
  ["\uC45B", "\uC45B"], /* Hangul Syllable */
  ["\uC45C", "\uC45C"], /* Hangul Syllable */
  ["\uC45D", "\uC45D"], /* Hangul Syllable */
  ["\uC45E", "\uC45E"], /* Hangul Syllable */
  ["\uC45F", "\uC45F"], /* Hangul Syllable */
  ["\uC460", "\uC460"], /* Hangul Syllable */
  ["\uC461", "\uC461"], /* Hangul Syllable */
  ["\uC462", "\uC462"], /* Hangul Syllable */
  ["\uC463", "\uC463"], /* Hangul Syllable */
  ["\uC464", "\uC464"], /* Hangul Syllable */
  ["\uC465", "\uC465"], /* Hangul Syllable */
  ["\uC466", "\uC466"], /* Hangul Syllable */
  ["\uC467", "\uC467"], /* Hangul Syllable */
  ["\uC468", "\uC468"], /* Hangul Syllable */
  ["\uC469", "\uC469"], /* Hangul Syllable */
  ["\uC46A", "\uC46A"], /* Hangul Syllable */
  ["\uC46B", "\uC46B"], /* Hangul Syllable */
  ["\uC46C", "\uC46C"], /* Hangul Syllable */
  ["\uC46D", "\uC46D"], /* Hangul Syllable */
  ["\uC46E", "\uC46E"], /* Hangul Syllable */
  ["\uC46F", "\uC46F"], /* Hangul Syllable */
  ["\uC470", "\uC470"], /* Hangul Syllable */
  ["\uC471", "\uC471"], /* Hangul Syllable */
  ["\uC472", "\uC472"], /* Hangul Syllable */
  ["\uC473", "\uC473"], /* Hangul Syllable */
  ["\uC474", "\uC474"], /* Hangul Syllable */
  ["\uC475", "\uC475"], /* Hangul Syllable */
  ["\uC476", "\uC476"], /* Hangul Syllable */
  ["\uC477", "\uC477"], /* Hangul Syllable */
  ["\uC478", "\uC478"], /* Hangul Syllable */
  ["\uC479", "\uC479"], /* Hangul Syllable */
  ["\uC47A", "\uC47A"], /* Hangul Syllable */
  ["\uC47B", "\uC47B"], /* Hangul Syllable */
  ["\uC47C", "\uC47C"], /* Hangul Syllable */
  ["\uC47D", "\uC47D"], /* Hangul Syllable */
  ["\uC47E", "\uC47E"], /* Hangul Syllable */
  ["\uC47F", "\uC47F"], /* Hangul Syllable */
  ["\uC480", "\uC480"], /* Hangul Syllable */
  ["\uC481", "\uC481"], /* Hangul Syllable */
  ["\uC482", "\uC482"], /* Hangul Syllable */
  ["\uC483", "\uC483"], /* Hangul Syllable */
  ["\uC484", "\uC484"], /* Hangul Syllable */
  ["\uC485", "\uC485"], /* Hangul Syllable */
  ["\uC486", "\uC486"], /* Hangul Syllable */
  ["\uC487", "\uC487"], /* Hangul Syllable */
  ["\uC488", "\uC488"], /* Hangul Syllable */
  ["\uC489", "\uC489"], /* Hangul Syllable */
  ["\uC48A", "\uC48A"], /* Hangul Syllable */
  ["\uC48B", "\uC48B"], /* Hangul Syllable */
  ["\uC48C", "\uC48C"], /* Hangul Syllable */
  ["\uC48D", "\uC48D"], /* Hangul Syllable */
  ["\uC48E", "\uC48E"], /* Hangul Syllable */
  ["\uC48F", "\uC48F"], /* Hangul Syllable */
  ["\uC490", "\uC490"], /* Hangul Syllable */
  ["\uC491", "\uC491"], /* Hangul Syllable */
  ["\uC492", "\uC492"], /* Hangul Syllable */
  ["\uC493", "\uC493"], /* Hangul Syllable */
  ["\uC494", "\uC494"], /* Hangul Syllable */
  ["\uC495", "\uC495"], /* Hangul Syllable */
  ["\uC496", "\uC496"], /* Hangul Syllable */
  ["\uC497", "\uC497"], /* Hangul Syllable */
  ["\uC498", "\uC498"], /* Hangul Syllable */
  ["\uC499", "\uC499"], /* Hangul Syllable */
  ["\uC49A", "\uC49A"], /* Hangul Syllable */
  ["\uC49B", "\uC49B"], /* Hangul Syllable */
  ["\uC49C", "\uC49C"], /* Hangul Syllable */
  ["\uC49D", "\uC49D"], /* Hangul Syllable */
  ["\uC49E", "\uC49E"], /* Hangul Syllable */
  ["\uC49F", "\uC49F"], /* Hangul Syllable */
  ["\uC4A0", "\uC4A0"], /* Hangul Syllable */
  ["\uC4A1", "\uC4A1"], /* Hangul Syllable */
  ["\uC4A2", "\uC4A2"], /* Hangul Syllable */
  ["\uC4A3", "\uC4A3"], /* Hangul Syllable */
  ["\uC4A4", "\uC4A4"], /* Hangul Syllable */
  ["\uC4A5", "\uC4A5"], /* Hangul Syllable */
  ["\uC4A6", "\uC4A6"], /* Hangul Syllable */
  ["\uC4A7", "\uC4A7"], /* Hangul Syllable */
  ["\uC4A8", "\uC4A8"], /* Hangul Syllable */
  ["\uC4A9", "\uC4A9"], /* Hangul Syllable */
  ["\uC4AA", "\uC4AA"], /* Hangul Syllable */
  ["\uC4AB", "\uC4AB"], /* Hangul Syllable */
  ["\uC4AC", "\uC4AC"], /* Hangul Syllable */
  ["\uC4AD", "\uC4AD"], /* Hangul Syllable */
  ["\uC4AE", "\uC4AE"], /* Hangul Syllable */
  ["\uC4AF", "\uC4AF"], /* Hangul Syllable */
  ["\uC4B0", "\uC4B0"], /* Hangul Syllable */
  ["\uC4B1", "\uC4B1"], /* Hangul Syllable */
  ["\uC4B2", "\uC4B2"], /* Hangul Syllable */
  ["\uC4B3", "\uC4B3"], /* Hangul Syllable */
  ["\uC4B4", "\uC4B4"], /* Hangul Syllable */
  ["\uC4B5", "\uC4B5"], /* Hangul Syllable */
  ["\uC4B6", "\uC4B6"], /* Hangul Syllable */
  ["\uC4B7", "\uC4B7"], /* Hangul Syllable */
  ["\uC4B8", "\uC4B8"], /* Hangul Syllable */
  ["\uC4B9", "\uC4B9"], /* Hangul Syllable */
  ["\uC4BA", "\uC4BA"], /* Hangul Syllable */
  ["\uC4BB", "\uC4BB"], /* Hangul Syllable */
  ["\uC4BC", "\uC4BC"], /* Hangul Syllable */
  ["\uC4BD", "\uC4BD"], /* Hangul Syllable */
  ["\uC4BE", "\uC4BE"], /* Hangul Syllable */
  ["\uC4BF", "\uC4BF"], /* Hangul Syllable */
  ["\uC4C0", "\uC4C0"], /* Hangul Syllable */
  ["\uC4C1", "\uC4C1"], /* Hangul Syllable */
  ["\uC4C2", "\uC4C2"], /* Hangul Syllable */
  ["\uC4C3", "\uC4C3"], /* Hangul Syllable */
  ["\uC4C4", "\uC4C4"], /* Hangul Syllable */
  ["\uC4C5", "\uC4C5"], /* Hangul Syllable */
  ["\uC4C6", "\uC4C6"], /* Hangul Syllable */
  ["\uC4C7", "\uC4C7"], /* Hangul Syllable */
  ["\uC4C8", "\uC4C8"], /* Hangul Syllable */
  ["\uC4C9", "\uC4C9"], /* Hangul Syllable */
  ["\uC4CA", "\uC4CA"], /* Hangul Syllable */
  ["\uC4CB", "\uC4CB"], /* Hangul Syllable */
  ["\uC4CC", "\uC4CC"], /* Hangul Syllable */
  ["\uC4CD", "\uC4CD"], /* Hangul Syllable */
  ["\uC4CE", "\uC4CE"], /* Hangul Syllable */
  ["\uC4CF", "\uC4CF"], /* Hangul Syllable */
  ["\uC4D0", "\uC4D0"], /* Hangul Syllable */
  ["\uC4D1", "\uC4D1"], /* Hangul Syllable */
  ["\uC4D2", "\uC4D2"], /* Hangul Syllable */
  ["\uC4D3", "\uC4D3"], /* Hangul Syllable */
  ["\uC4D4", "\uC4D4"], /* Hangul Syllable */
  ["\uC4D5", "\uC4D5"], /* Hangul Syllable */
  ["\uC4D6", "\uC4D6"], /* Hangul Syllable */
  ["\uC4D7", "\uC4D7"], /* Hangul Syllable */
  ["\uC4D8", "\uC4D8"], /* Hangul Syllable */
  ["\uC4D9", "\uC4D9"], /* Hangul Syllable */
  ["\uC4DA", "\uC4DA"], /* Hangul Syllable */
  ["\uC4DB", "\uC4DB"], /* Hangul Syllable */
  ["\uC4DC", "\uC4DC"], /* Hangul Syllable */
  ["\uC4DD", "\uC4DD"], /* Hangul Syllable */
  ["\uC4DE", "\uC4DE"], /* Hangul Syllable */
  ["\uC4DF", "\uC4DF"], /* Hangul Syllable */
  ["\uC4E0", "\uC4E0"], /* Hangul Syllable */
  ["\uC4E1", "\uC4E1"], /* Hangul Syllable */
  ["\uC4E2", "\uC4E2"], /* Hangul Syllable */
  ["\uC4E3", "\uC4E3"], /* Hangul Syllable */
  ["\uC4E4", "\uC4E4"], /* Hangul Syllable */
  ["\uC4E5", "\uC4E5"], /* Hangul Syllable */
  ["\uC4E6", "\uC4E6"], /* Hangul Syllable */
  ["\uC4E7", "\uC4E7"], /* Hangul Syllable */
  ["\uC4E8", "\uC4E8"], /* Hangul Syllable */
  ["\uC4E9", "\uC4E9"], /* Hangul Syllable */
  ["\uC4EA", "\uC4EA"], /* Hangul Syllable */
  ["\uC4EB", "\uC4EB"], /* Hangul Syllable */
  ["\uC4EC", "\uC4EC"], /* Hangul Syllable */
  ["\uC4ED", "\uC4ED"], /* Hangul Syllable */
  ["\uC4EE", "\uC4EE"], /* Hangul Syllable */
  ["\uC4EF", "\uC4EF"], /* Hangul Syllable */
  ["\uC4F0", "\uC4F0"], /* Hangul Syllable */
  ["\uC4F1", "\uC4F1"], /* Hangul Syllable */
  ["\uC4F2", "\uC4F2"], /* Hangul Syllable */
  ["\uC4F3", "\uC4F3"], /* Hangul Syllable */
  ["\uC4F4", "\uC4F4"], /* Hangul Syllable */
  ["\uC4F5", "\uC4F5"], /* Hangul Syllable */
  ["\uC4F6", "\uC4F6"], /* Hangul Syllable */
  ["\uC4F7", "\uC4F7"], /* Hangul Syllable */
  ["\uC4F8", "\uC4F8"], /* Hangul Syllable */
  ["\uC4F9", "\uC4F9"], /* Hangul Syllable */
  ["\uC4FA", "\uC4FA"], /* Hangul Syllable */
  ["\uC4FB", "\uC4FB"], /* Hangul Syllable */
  ["\uC4FC", "\uC4FC"], /* Hangul Syllable */
  ["\uC4FD", "\uC4FD"], /* Hangul Syllable */
  ["\uC4FE", "\uC4FE"], /* Hangul Syllable */
  ["\uC4FF", "\uC4FF"], /* Hangul Syllable */
  ["\uC500", "\uC500"], /* Hangul Syllable */
  ["\uC501", "\uC501"], /* Hangul Syllable */
  ["\uC502", "\uC502"], /* Hangul Syllable */
  ["\uC503", "\uC503"], /* Hangul Syllable */
  ["\uC504", "\uC504"], /* Hangul Syllable */
  ["\uC505", "\uC505"], /* Hangul Syllable */
  ["\uC506", "\uC506"], /* Hangul Syllable */
  ["\uC507", "\uC507"], /* Hangul Syllable */
  ["\uC508", "\uC508"], /* Hangul Syllable */
  ["\uC509", "\uC509"], /* Hangul Syllable */
  ["\uC50A", "\uC50A"], /* Hangul Syllable */
  ["\uC50B", "\uC50B"], /* Hangul Syllable */
  ["\uC50C", "\uC50C"], /* Hangul Syllable */
  ["\uC50D", "\uC50D"], /* Hangul Syllable */
  ["\uC50E", "\uC50E"], /* Hangul Syllable */
  ["\uC50F", "\uC50F"], /* Hangul Syllable */
  ["\uC510", "\uC510"], /* Hangul Syllable */
  ["\uC511", "\uC511"], /* Hangul Syllable */
  ["\uC512", "\uC512"], /* Hangul Syllable */
  ["\uC513", "\uC513"], /* Hangul Syllable */
  ["\uC514", "\uC514"], /* Hangul Syllable */
  ["\uC515", "\uC515"], /* Hangul Syllable */
  ["\uC516", "\uC516"], /* Hangul Syllable */
  ["\uC517", "\uC517"], /* Hangul Syllable */
  ["\uC518", "\uC518"], /* Hangul Syllable */
  ["\uC519", "\uC519"], /* Hangul Syllable */
  ["\uC51A", "\uC51A"], /* Hangul Syllable */
  ["\uC51B", "\uC51B"], /* Hangul Syllable */
  ["\uC51C", "\uC51C"], /* Hangul Syllable */
  ["\uC51D", "\uC51D"], /* Hangul Syllable */
  ["\uC51E", "\uC51E"], /* Hangul Syllable */
  ["\uC51F", "\uC51F"], /* Hangul Syllable */
  ["\uC520", "\uC520"], /* Hangul Syllable */
  ["\uC521", "\uC521"], /* Hangul Syllable */
  ["\uC522", "\uC522"], /* Hangul Syllable */
  ["\uC523", "\uC523"], /* Hangul Syllable */
  ["\uC524", "\uC524"], /* Hangul Syllable */
  ["\uC525", "\uC525"], /* Hangul Syllable */
  ["\uC526", "\uC526"], /* Hangul Syllable */
  ["\uC527", "\uC527"], /* Hangul Syllable */
  ["\uC528", "\uC528"], /* Hangul Syllable */
  ["\uC529", "\uC529"], /* Hangul Syllable */
  ["\uC52A", "\uC52A"], /* Hangul Syllable */
  ["\uC52B", "\uC52B"], /* Hangul Syllable */
  ["\uC52C", "\uC52C"], /* Hangul Syllable */
  ["\uC52D", "\uC52D"], /* Hangul Syllable */
  ["\uC52E", "\uC52E"], /* Hangul Syllable */
  ["\uC52F", "\uC52F"], /* Hangul Syllable */
  ["\uC530", "\uC530"], /* Hangul Syllable */
  ["\uC531", "\uC531"], /* Hangul Syllable */
  ["\uC532", "\uC532"], /* Hangul Syllable */
  ["\uC533", "\uC533"], /* Hangul Syllable */
  ["\uC534", "\uC534"], /* Hangul Syllable */
  ["\uC535", "\uC535"], /* Hangul Syllable */
  ["\uC536", "\uC536"], /* Hangul Syllable */
  ["\uC537", "\uC537"], /* Hangul Syllable */
  ["\uC538", "\uC538"], /* Hangul Syllable */
  ["\uC539", "\uC539"], /* Hangul Syllable */
  ["\uC53A", "\uC53A"], /* Hangul Syllable */
  ["\uC53B", "\uC53B"], /* Hangul Syllable */
  ["\uC53C", "\uC53C"], /* Hangul Syllable */
  ["\uC53D", "\uC53D"], /* Hangul Syllable */
  ["\uC53E", "\uC53E"], /* Hangul Syllable */
  ["\uC53F", "\uC53F"], /* Hangul Syllable */
  ["\uC540", "\uC540"], /* Hangul Syllable */
  ["\uC541", "\uC541"], /* Hangul Syllable */
  ["\uC542", "\uC542"], /* Hangul Syllable */
  ["\uC543", "\uC543"], /* Hangul Syllable */
  ["\uC544", "\uC544"], /* Hangul Syllable */
  ["\uC545", "\uC545"], /* Hangul Syllable */
  ["\uC546", "\uC546"], /* Hangul Syllable */
  ["\uC547", "\uC547"], /* Hangul Syllable */
  ["\uC548", "\uC548"], /* Hangul Syllable */
  ["\uC549", "\uC549"], /* Hangul Syllable */
  ["\uC54A", "\uC54A"], /* Hangul Syllable */
  ["\uC54B", "\uC54B"], /* Hangul Syllable */
  ["\uC54C", "\uC54C"], /* Hangul Syllable */
  ["\uC54D", "\uC54D"], /* Hangul Syllable */
  ["\uC54E", "\uC54E"], /* Hangul Syllable */
  ["\uC54F", "\uC54F"], /* Hangul Syllable */
  ["\uC550", "\uC550"], /* Hangul Syllable */
  ["\uC551", "\uC551"], /* Hangul Syllable */
  ["\uC552", "\uC552"], /* Hangul Syllable */
  ["\uC553", "\uC553"], /* Hangul Syllable */
  ["\uC554", "\uC554"], /* Hangul Syllable */
  ["\uC555", "\uC555"], /* Hangul Syllable */
  ["\uC556", "\uC556"], /* Hangul Syllable */
  ["\uC557", "\uC557"], /* Hangul Syllable */
  ["\uC558", "\uC558"], /* Hangul Syllable */
  ["\uC559", "\uC559"], /* Hangul Syllable */
  ["\uC55A", "\uC55A"], /* Hangul Syllable */
  ["\uC55B", "\uC55B"], /* Hangul Syllable */
  ["\uC55C", "\uC55C"], /* Hangul Syllable */
  ["\uC55D", "\uC55D"], /* Hangul Syllable */
  ["\uC55E", "\uC55E"], /* Hangul Syllable */
  ["\uC55F", "\uC55F"], /* Hangul Syllable */
  ["\uC560", "\uC560"], /* Hangul Syllable */
  ["\uC561", "\uC561"], /* Hangul Syllable */
  ["\uC562", "\uC562"], /* Hangul Syllable */
  ["\uC563", "\uC563"], /* Hangul Syllable */
  ["\uC564", "\uC564"], /* Hangul Syllable */
  ["\uC565", "\uC565"], /* Hangul Syllable */
  ["\uC566", "\uC566"], /* Hangul Syllable */
  ["\uC567", "\uC567"], /* Hangul Syllable */
  ["\uC568", "\uC568"], /* Hangul Syllable */
  ["\uC569", "\uC569"], /* Hangul Syllable */
  ["\uC56A", "\uC56A"], /* Hangul Syllable */
  ["\uC56B", "\uC56B"], /* Hangul Syllable */
  ["\uC56C", "\uC56C"], /* Hangul Syllable */
  ["\uC56D", "\uC56D"], /* Hangul Syllable */
  ["\uC56E", "\uC56E"], /* Hangul Syllable */
  ["\uC56F", "\uC56F"], /* Hangul Syllable */
  ["\uC570", "\uC570"], /* Hangul Syllable */
  ["\uC571", "\uC571"], /* Hangul Syllable */
  ["\uC572", "\uC572"], /* Hangul Syllable */
  ["\uC573", "\uC573"], /* Hangul Syllable */
  ["\uC574", "\uC574"], /* Hangul Syllable */
  ["\uC575", "\uC575"], /* Hangul Syllable */
  ["\uC576", "\uC576"], /* Hangul Syllable */
  ["\uC577", "\uC577"], /* Hangul Syllable */
  ["\uC578", "\uC578"], /* Hangul Syllable */
  ["\uC579", "\uC579"], /* Hangul Syllable */
  ["\uC57A", "\uC57A"], /* Hangul Syllable */
  ["\uC57B", "\uC57B"], /* Hangul Syllable */
  ["\uC57C", "\uC57C"], /* Hangul Syllable */
  ["\uC57D", "\uC57D"], /* Hangul Syllable */
  ["\uC57E", "\uC57E"], /* Hangul Syllable */
  ["\uC57F", "\uC57F"], /* Hangul Syllable */
  ["\uC580", "\uC580"], /* Hangul Syllable */
  ["\uC581", "\uC581"], /* Hangul Syllable */
  ["\uC582", "\uC582"], /* Hangul Syllable */
  ["\uC583", "\uC583"], /* Hangul Syllable */
  ["\uC584", "\uC584"], /* Hangul Syllable */
  ["\uC585", "\uC585"], /* Hangul Syllable */
  ["\uC586", "\uC586"], /* Hangul Syllable */
  ["\uC587", "\uC587"], /* Hangul Syllable */
  ["\uC588", "\uC588"], /* Hangul Syllable */
  ["\uC589", "\uC589"], /* Hangul Syllable */
  ["\uC58A", "\uC58A"], /* Hangul Syllable */
  ["\uC58B", "\uC58B"], /* Hangul Syllable */
  ["\uC58C", "\uC58C"], /* Hangul Syllable */
  ["\uC58D", "\uC58D"], /* Hangul Syllable */
  ["\uC58E", "\uC58E"], /* Hangul Syllable */
  ["\uC58F", "\uC58F"], /* Hangul Syllable */
  ["\uC590", "\uC590"], /* Hangul Syllable */
  ["\uC591", "\uC591"], /* Hangul Syllable */
  ["\uC592", "\uC592"], /* Hangul Syllable */
  ["\uC593", "\uC593"], /* Hangul Syllable */
  ["\uC594", "\uC594"], /* Hangul Syllable */
  ["\uC595", "\uC595"], /* Hangul Syllable */
  ["\uC596", "\uC596"], /* Hangul Syllable */
  ["\uC597", "\uC597"], /* Hangul Syllable */
  ["\uC598", "\uC598"], /* Hangul Syllable */
  ["\uC599", "\uC599"], /* Hangul Syllable */
  ["\uC59A", "\uC59A"], /* Hangul Syllable */
  ["\uC59B", "\uC59B"], /* Hangul Syllable */
  ["\uC59C", "\uC59C"], /* Hangul Syllable */
  ["\uC59D", "\uC59D"], /* Hangul Syllable */
  ["\uC59E", "\uC59E"], /* Hangul Syllable */
  ["\uC59F", "\uC59F"], /* Hangul Syllable */
  ["\uC5A0", "\uC5A0"], /* Hangul Syllable */
  ["\uC5A1", "\uC5A1"], /* Hangul Syllable */
  ["\uC5A2", "\uC5A2"], /* Hangul Syllable */
  ["\uC5A3", "\uC5A3"], /* Hangul Syllable */
  ["\uC5A4", "\uC5A4"], /* Hangul Syllable */
  ["\uC5A5", "\uC5A5"], /* Hangul Syllable */
  ["\uC5A6", "\uC5A6"], /* Hangul Syllable */
  ["\uC5A7", "\uC5A7"], /* Hangul Syllable */
  ["\uC5A8", "\uC5A8"], /* Hangul Syllable */
  ["\uC5A9", "\uC5A9"], /* Hangul Syllable */
  ["\uC5AA", "\uC5AA"], /* Hangul Syllable */
  ["\uC5AB", "\uC5AB"], /* Hangul Syllable */
  ["\uC5AC", "\uC5AC"], /* Hangul Syllable */
  ["\uC5AD", "\uC5AD"], /* Hangul Syllable */
  ["\uC5AE", "\uC5AE"], /* Hangul Syllable */
  ["\uC5AF", "\uC5AF"], /* Hangul Syllable */
  ["\uC5B0", "\uC5B0"], /* Hangul Syllable */
  ["\uC5B1", "\uC5B1"], /* Hangul Syllable */
  ["\uC5B2", "\uC5B2"], /* Hangul Syllable */
  ["\uC5B3", "\uC5B3"], /* Hangul Syllable */
  ["\uC5B4", "\uC5B4"], /* Hangul Syllable */
  ["\uC5B5", "\uC5B5"], /* Hangul Syllable */
  ["\uC5B6", "\uC5B6"], /* Hangul Syllable */
  ["\uC5B7", "\uC5B7"], /* Hangul Syllable */
  ["\uC5B8", "\uC5B8"], /* Hangul Syllable */
  ["\uC5B9", "\uC5B9"], /* Hangul Syllable */
  ["\uC5BA", "\uC5BA"], /* Hangul Syllable */
  ["\uC5BB", "\uC5BB"], /* Hangul Syllable */
  ["\uC5BC", "\uC5BC"], /* Hangul Syllable */
  ["\uC5BD", "\uC5BD"], /* Hangul Syllable */
  ["\uC5BE", "\uC5BE"], /* Hangul Syllable */
  ["\uC5BF", "\uC5BF"], /* Hangul Syllable */
  ["\uC5C0", "\uC5C0"], /* Hangul Syllable */
  ["\uC5C1", "\uC5C1"], /* Hangul Syllable */
  ["\uC5C2", "\uC5C2"], /* Hangul Syllable */
  ["\uC5C3", "\uC5C3"], /* Hangul Syllable */
  ["\uC5C4", "\uC5C4"], /* Hangul Syllable */
  ["\uC5C5", "\uC5C5"], /* Hangul Syllable */
  ["\uC5C6", "\uC5C6"], /* Hangul Syllable */
  ["\uC5C7", "\uC5C7"], /* Hangul Syllable */
  ["\uC5C8", "\uC5C8"], /* Hangul Syllable */
  ["\uC5C9", "\uC5C9"], /* Hangul Syllable */
  ["\uC5CA", "\uC5CA"], /* Hangul Syllable */
  ["\uC5CB", "\uC5CB"], /* Hangul Syllable */
  ["\uC5CC", "\uC5CC"], /* Hangul Syllable */
  ["\uC5CD", "\uC5CD"], /* Hangul Syllable */
  ["\uC5CE", "\uC5CE"], /* Hangul Syllable */
  ["\uC5CF", "\uC5CF"], /* Hangul Syllable */
  ["\uC5D0", "\uC5D0"], /* Hangul Syllable */
  ["\uC5D1", "\uC5D1"], /* Hangul Syllable */
  ["\uC5D2", "\uC5D2"], /* Hangul Syllable */
  ["\uC5D3", "\uC5D3"], /* Hangul Syllable */
  ["\uC5D4", "\uC5D4"], /* Hangul Syllable */
  ["\uC5D5", "\uC5D5"], /* Hangul Syllable */
  ["\uC5D6", "\uC5D6"], /* Hangul Syllable */
  ["\uC5D7", "\uC5D7"], /* Hangul Syllable */
  ["\uC5D8", "\uC5D8"], /* Hangul Syllable */
  ["\uC5D9", "\uC5D9"], /* Hangul Syllable */
  ["\uC5DA", "\uC5DA"], /* Hangul Syllable */
  ["\uC5DB", "\uC5DB"], /* Hangul Syllable */
  ["\uC5DC", "\uC5DC"], /* Hangul Syllable */
  ["\uC5DD", "\uC5DD"], /* Hangul Syllable */
  ["\uC5DE", "\uC5DE"], /* Hangul Syllable */
  ["\uC5DF", "\uC5DF"], /* Hangul Syllable */
  ["\uC5E0", "\uC5E0"], /* Hangul Syllable */
  ["\uC5E1", "\uC5E1"], /* Hangul Syllable */
  ["\uC5E2", "\uC5E2"], /* Hangul Syllable */
  ["\uC5E3", "\uC5E3"], /* Hangul Syllable */
  ["\uC5E4", "\uC5E4"], /* Hangul Syllable */
  ["\uC5E5", "\uC5E5"], /* Hangul Syllable */
  ["\uC5E6", "\uC5E6"], /* Hangul Syllable */
  ["\uC5E7", "\uC5E7"], /* Hangul Syllable */
  ["\uC5E8", "\uC5E8"], /* Hangul Syllable */
  ["\uC5E9", "\uC5E9"], /* Hangul Syllable */
  ["\uC5EA", "\uC5EA"], /* Hangul Syllable */
  ["\uC5EB", "\uC5EB"], /* Hangul Syllable */
  ["\uC5EC", "\uC5EC"], /* Hangul Syllable */
  ["\uC5ED", "\uC5ED"], /* Hangul Syllable */
  ["\uC5EE", "\uC5EE"], /* Hangul Syllable */
  ["\uC5EF", "\uC5EF"], /* Hangul Syllable */
  ["\uC5F0", "\uC5F0"], /* Hangul Syllable */
  ["\uC5F1", "\uC5F1"], /* Hangul Syllable */
  ["\uC5F2", "\uC5F2"], /* Hangul Syllable */
  ["\uC5F3", "\uC5F3"], /* Hangul Syllable */
  ["\uC5F4", "\uC5F4"], /* Hangul Syllable */
  ["\uC5F5", "\uC5F5"], /* Hangul Syllable */
  ["\uC5F6", "\uC5F6"], /* Hangul Syllable */
  ["\uC5F7", "\uC5F7"], /* Hangul Syllable */
  ["\uC5F8", "\uC5F8"], /* Hangul Syllable */
  ["\uC5F9", "\uC5F9"], /* Hangul Syllable */
  ["\uC5FA", "\uC5FA"], /* Hangul Syllable */
  ["\uC5FB", "\uC5FB"], /* Hangul Syllable */
  ["\uC5FC", "\uC5FC"], /* Hangul Syllable */
  ["\uC5FD", "\uC5FD"], /* Hangul Syllable */
  ["\uC5FE", "\uC5FE"], /* Hangul Syllable */
  ["\uC5FF", "\uC5FF"], /* Hangul Syllable */
  ["\uC600", "\uC600"], /* Hangul Syllable */
  ["\uC601", "\uC601"], /* Hangul Syllable */
  ["\uC602", "\uC602"], /* Hangul Syllable */
  ["\uC603", "\uC603"], /* Hangul Syllable */
  ["\uC604", "\uC604"], /* Hangul Syllable */
  ["\uC605", "\uC605"], /* Hangul Syllable */
  ["\uC606", "\uC606"], /* Hangul Syllable */
  ["\uC607", "\uC607"], /* Hangul Syllable */
  ["\uC608", "\uC608"], /* Hangul Syllable */
  ["\uC609", "\uC609"], /* Hangul Syllable */
  ["\uC60A", "\uC60A"], /* Hangul Syllable */
  ["\uC60B", "\uC60B"], /* Hangul Syllable */
  ["\uC60C", "\uC60C"], /* Hangul Syllable */
  ["\uC60D", "\uC60D"], /* Hangul Syllable */
  ["\uC60E", "\uC60E"], /* Hangul Syllable */
  ["\uC60F", "\uC60F"], /* Hangul Syllable */
  ["\uC610", "\uC610"], /* Hangul Syllable */
  ["\uC611", "\uC611"], /* Hangul Syllable */
  ["\uC612", "\uC612"], /* Hangul Syllable */
  ["\uC613", "\uC613"], /* Hangul Syllable */
  ["\uC614", "\uC614"], /* Hangul Syllable */
  ["\uC615", "\uC615"], /* Hangul Syllable */
  ["\uC616", "\uC616"], /* Hangul Syllable */
  ["\uC617", "\uC617"], /* Hangul Syllable */
  ["\uC618", "\uC618"], /* Hangul Syllable */
  ["\uC619", "\uC619"], /* Hangul Syllable */
  ["\uC61A", "\uC61A"], /* Hangul Syllable */
  ["\uC61B", "\uC61B"], /* Hangul Syllable */
  ["\uC61C", "\uC61C"], /* Hangul Syllable */
  ["\uC61D", "\uC61D"], /* Hangul Syllable */
  ["\uC61E", "\uC61E"], /* Hangul Syllable */
  ["\uC61F", "\uC61F"], /* Hangul Syllable */
  ["\uC620", "\uC620"], /* Hangul Syllable */
  ["\uC621", "\uC621"], /* Hangul Syllable */
  ["\uC622", "\uC622"], /* Hangul Syllable */
  ["\uC623", "\uC623"], /* Hangul Syllable */
  ["\uC624", "\uC624"], /* Hangul Syllable */
  ["\uC625", "\uC625"], /* Hangul Syllable */
  ["\uC626", "\uC626"], /* Hangul Syllable */
  ["\uC627", "\uC627"], /* Hangul Syllable */
  ["\uC628", "\uC628"], /* Hangul Syllable */
  ["\uC629", "\uC629"], /* Hangul Syllable */
  ["\uC62A", "\uC62A"], /* Hangul Syllable */
  ["\uC62B", "\uC62B"], /* Hangul Syllable */
  ["\uC62C", "\uC62C"], /* Hangul Syllable */
  ["\uC62D", "\uC62D"], /* Hangul Syllable */
  ["\uC62E", "\uC62E"], /* Hangul Syllable */
  ["\uC62F", "\uC62F"], /* Hangul Syllable */
  ["\uC630", "\uC630"], /* Hangul Syllable */
  ["\uC631", "\uC631"], /* Hangul Syllable */
  ["\uC632", "\uC632"], /* Hangul Syllable */
  ["\uC633", "\uC633"], /* Hangul Syllable */
  ["\uC634", "\uC634"], /* Hangul Syllable */
  ["\uC635", "\uC635"], /* Hangul Syllable */
  ["\uC636", "\uC636"], /* Hangul Syllable */
  ["\uC637", "\uC637"], /* Hangul Syllable */
  ["\uC638", "\uC638"], /* Hangul Syllable */
  ["\uC639", "\uC639"], /* Hangul Syllable */
  ["\uC63A", "\uC63A"], /* Hangul Syllable */
  ["\uC63B", "\uC63B"], /* Hangul Syllable */
  ["\uC63C", "\uC63C"], /* Hangul Syllable */
  ["\uC63D", "\uC63D"], /* Hangul Syllable */
  ["\uC63E", "\uC63E"], /* Hangul Syllable */
  ["\uC63F", "\uC63F"], /* Hangul Syllable */
  ["\uC640", "\uC640"], /* Hangul Syllable */
  ["\uC641", "\uC641"], /* Hangul Syllable */
  ["\uC642", "\uC642"], /* Hangul Syllable */
  ["\uC643", "\uC643"], /* Hangul Syllable */
  ["\uC644", "\uC644"], /* Hangul Syllable */
  ["\uC645", "\uC645"], /* Hangul Syllable */
  ["\uC646", "\uC646"], /* Hangul Syllable */
  ["\uC647", "\uC647"], /* Hangul Syllable */
  ["\uC648", "\uC648"], /* Hangul Syllable */
  ["\uC649", "\uC649"], /* Hangul Syllable */
  ["\uC64A", "\uC64A"], /* Hangul Syllable */
  ["\uC64B", "\uC64B"], /* Hangul Syllable */
  ["\uC64C", "\uC64C"], /* Hangul Syllable */
  ["\uC64D", "\uC64D"], /* Hangul Syllable */
  ["\uC64E", "\uC64E"], /* Hangul Syllable */
  ["\uC64F", "\uC64F"], /* Hangul Syllable */
  ["\uC650", "\uC650"], /* Hangul Syllable */
  ["\uC651", "\uC651"], /* Hangul Syllable */
  ["\uC652", "\uC652"], /* Hangul Syllable */
  ["\uC653", "\uC653"], /* Hangul Syllable */
  ["\uC654", "\uC654"], /* Hangul Syllable */
  ["\uC655", "\uC655"], /* Hangul Syllable */
  ["\uC656", "\uC656"], /* Hangul Syllable */
  ["\uC657", "\uC657"], /* Hangul Syllable */
  ["\uC658", "\uC658"], /* Hangul Syllable */
  ["\uC659", "\uC659"], /* Hangul Syllable */
  ["\uC65A", "\uC65A"], /* Hangul Syllable */
  ["\uC65B", "\uC65B"], /* Hangul Syllable */
  ["\uC65C", "\uC65C"], /* Hangul Syllable */
  ["\uC65D", "\uC65D"], /* Hangul Syllable */
  ["\uC65E", "\uC65E"], /* Hangul Syllable */
  ["\uC65F", "\uC65F"], /* Hangul Syllable */
  ["\uC660", "\uC660"], /* Hangul Syllable */
  ["\uC661", "\uC661"], /* Hangul Syllable */
  ["\uC662", "\uC662"], /* Hangul Syllable */
  ["\uC663", "\uC663"], /* Hangul Syllable */
  ["\uC664", "\uC664"], /* Hangul Syllable */
  ["\uC665", "\uC665"], /* Hangul Syllable */
  ["\uC666", "\uC666"], /* Hangul Syllable */
  ["\uC667", "\uC667"], /* Hangul Syllable */
  ["\uC668", "\uC668"], /* Hangul Syllable */
  ["\uC669", "\uC669"], /* Hangul Syllable */
  ["\uC66A", "\uC66A"], /* Hangul Syllable */
  ["\uC66B", "\uC66B"], /* Hangul Syllable */
  ["\uC66C", "\uC66C"], /* Hangul Syllable */
  ["\uC66D", "\uC66D"], /* Hangul Syllable */
  ["\uC66E", "\uC66E"], /* Hangul Syllable */
  ["\uC66F", "\uC66F"], /* Hangul Syllable */
  ["\uC670", "\uC670"], /* Hangul Syllable */
  ["\uC671", "\uC671"], /* Hangul Syllable */
  ["\uC672", "\uC672"], /* Hangul Syllable */
  ["\uC673", "\uC673"], /* Hangul Syllable */
  ["\uC674", "\uC674"], /* Hangul Syllable */
  ["\uC675", "\uC675"], /* Hangul Syllable */
  ["\uC676", "\uC676"], /* Hangul Syllable */
  ["\uC677", "\uC677"], /* Hangul Syllable */
  ["\uC678", "\uC678"], /* Hangul Syllable */
  ["\uC679", "\uC679"], /* Hangul Syllable */
  ["\uC67A", "\uC67A"], /* Hangul Syllable */
  ["\uC67B", "\uC67B"], /* Hangul Syllable */
  ["\uC67C", "\uC67C"], /* Hangul Syllable */
  ["\uC67D", "\uC67D"], /* Hangul Syllable */
  ["\uC67E", "\uC67E"], /* Hangul Syllable */
  ["\uC67F", "\uC67F"], /* Hangul Syllable */
  ["\uC680", "\uC680"], /* Hangul Syllable */
  ["\uC681", "\uC681"], /* Hangul Syllable */
  ["\uC682", "\uC682"], /* Hangul Syllable */
  ["\uC683", "\uC683"], /* Hangul Syllable */
  ["\uC684", "\uC684"], /* Hangul Syllable */
  ["\uC685", "\uC685"], /* Hangul Syllable */
  ["\uC686", "\uC686"], /* Hangul Syllable */
  ["\uC687", "\uC687"], /* Hangul Syllable */
  ["\uC688", "\uC688"], /* Hangul Syllable */
  ["\uC689", "\uC689"], /* Hangul Syllable */
  ["\uC68A", "\uC68A"], /* Hangul Syllable */
  ["\uC68B", "\uC68B"], /* Hangul Syllable */
  ["\uC68C", "\uC68C"], /* Hangul Syllable */
  ["\uC68D", "\uC68D"], /* Hangul Syllable */
  ["\uC68E", "\uC68E"], /* Hangul Syllable */
  ["\uC68F", "\uC68F"], /* Hangul Syllable */
  ["\uC690", "\uC690"], /* Hangul Syllable */
  ["\uC691", "\uC691"], /* Hangul Syllable */
  ["\uC692", "\uC692"], /* Hangul Syllable */
  ["\uC693", "\uC693"], /* Hangul Syllable */
  ["\uC694", "\uC694"], /* Hangul Syllable */
  ["\uC695", "\uC695"], /* Hangul Syllable */
  ["\uC696", "\uC696"], /* Hangul Syllable */
  ["\uC697", "\uC697"], /* Hangul Syllable */
  ["\uC698", "\uC698"], /* Hangul Syllable */
  ["\uC699", "\uC699"], /* Hangul Syllable */
  ["\uC69A", "\uC69A"], /* Hangul Syllable */
  ["\uC69B", "\uC69B"], /* Hangul Syllable */
  ["\uC69C", "\uC69C"], /* Hangul Syllable */
  ["\uC69D", "\uC69D"], /* Hangul Syllable */
  ["\uC69E", "\uC69E"], /* Hangul Syllable */
  ["\uC69F", "\uC69F"], /* Hangul Syllable */
  ["\uC6A0", "\uC6A0"], /* Hangul Syllable */
  ["\uC6A1", "\uC6A1"], /* Hangul Syllable */
  ["\uC6A2", "\uC6A2"], /* Hangul Syllable */
  ["\uC6A3", "\uC6A3"], /* Hangul Syllable */
  ["\uC6A4", "\uC6A4"], /* Hangul Syllable */
  ["\uC6A5", "\uC6A5"], /* Hangul Syllable */
  ["\uC6A6", "\uC6A6"], /* Hangul Syllable */
  ["\uC6A7", "\uC6A7"], /* Hangul Syllable */
  ["\uC6A8", "\uC6A8"], /* Hangul Syllable */
  ["\uC6A9", "\uC6A9"], /* Hangul Syllable */
  ["\uC6AA", "\uC6AA"], /* Hangul Syllable */
  ["\uC6AB", "\uC6AB"], /* Hangul Syllable */
  ["\uC6AC", "\uC6AC"], /* Hangul Syllable */
  ["\uC6AD", "\uC6AD"], /* Hangul Syllable */
  ["\uC6AE", "\uC6AE"], /* Hangul Syllable */
  ["\uC6AF", "\uC6AF"], /* Hangul Syllable */
  ["\uC6B0", "\uC6B0"], /* Hangul Syllable */
  ["\uC6B1", "\uC6B1"], /* Hangul Syllable */
  ["\uC6B2", "\uC6B2"], /* Hangul Syllable */
  ["\uC6B3", "\uC6B3"], /* Hangul Syllable */
  ["\uC6B4", "\uC6B4"], /* Hangul Syllable */
  ["\uC6B5", "\uC6B5"], /* Hangul Syllable */
  ["\uC6B6", "\uC6B6"], /* Hangul Syllable */
  ["\uC6B7", "\uC6B7"], /* Hangul Syllable */
  ["\uC6B8", "\uC6B8"], /* Hangul Syllable */
  ["\uC6B9", "\uC6B9"], /* Hangul Syllable */
  ["\uC6BA", "\uC6BA"], /* Hangul Syllable */
  ["\uC6BB", "\uC6BB"], /* Hangul Syllable */
  ["\uC6BC", "\uC6BC"], /* Hangul Syllable */
  ["\uC6BD", "\uC6BD"], /* Hangul Syllable */
  ["\uC6BE", "\uC6BE"], /* Hangul Syllable */
  ["\uC6BF", "\uC6BF"], /* Hangul Syllable */
  ["\uC6C0", "\uC6C0"], /* Hangul Syllable */
  ["\uC6C1", "\uC6C1"], /* Hangul Syllable */
  ["\uC6C2", "\uC6C2"], /* Hangul Syllable */
  ["\uC6C3", "\uC6C3"], /* Hangul Syllable */
  ["\uC6C4", "\uC6C4"], /* Hangul Syllable */
  ["\uC6C5", "\uC6C5"], /* Hangul Syllable */
  ["\uC6C6", "\uC6C6"], /* Hangul Syllable */
  ["\uC6C7", "\uC6C7"], /* Hangul Syllable */
  ["\uC6C8", "\uC6C8"], /* Hangul Syllable */
  ["\uC6C9", "\uC6C9"], /* Hangul Syllable */
  ["\uC6CA", "\uC6CA"], /* Hangul Syllable */
  ["\uC6CB", "\uC6CB"], /* Hangul Syllable */
  ["\uC6CC", "\uC6CC"], /* Hangul Syllable */
  ["\uC6CD", "\uC6CD"], /* Hangul Syllable */
  ["\uC6CE", "\uC6CE"], /* Hangul Syllable */
  ["\uC6CF", "\uC6CF"], /* Hangul Syllable */
  ["\uC6D0", "\uC6D0"], /* Hangul Syllable */
  ["\uC6D1", "\uC6D1"], /* Hangul Syllable */
  ["\uC6D2", "\uC6D2"], /* Hangul Syllable */
  ["\uC6D3", "\uC6D3"], /* Hangul Syllable */
  ["\uC6D4", "\uC6D4"], /* Hangul Syllable */
  ["\uC6D5", "\uC6D5"], /* Hangul Syllable */
  ["\uC6D6", "\uC6D6"], /* Hangul Syllable */
  ["\uC6D7", "\uC6D7"], /* Hangul Syllable */
  ["\uC6D8", "\uC6D8"], /* Hangul Syllable */
  ["\uC6D9", "\uC6D9"], /* Hangul Syllable */
  ["\uC6DA", "\uC6DA"], /* Hangul Syllable */
  ["\uC6DB", "\uC6DB"], /* Hangul Syllable */
  ["\uC6DC", "\uC6DC"], /* Hangul Syllable */
  ["\uC6DD", "\uC6DD"], /* Hangul Syllable */
  ["\uC6DE", "\uC6DE"], /* Hangul Syllable */
  ["\uC6DF", "\uC6DF"], /* Hangul Syllable */
  ["\uC6E0", "\uC6E0"], /* Hangul Syllable */
  ["\uC6E1", "\uC6E1"], /* Hangul Syllable */
  ["\uC6E2", "\uC6E2"], /* Hangul Syllable */
  ["\uC6E3", "\uC6E3"], /* Hangul Syllable */
  ["\uC6E4", "\uC6E4"], /* Hangul Syllable */
  ["\uC6E5", "\uC6E5"], /* Hangul Syllable */
  ["\uC6E6", "\uC6E6"], /* Hangul Syllable */
  ["\uC6E7", "\uC6E7"], /* Hangul Syllable */
  ["\uC6E8", "\uC6E8"], /* Hangul Syllable */
  ["\uC6E9", "\uC6E9"], /* Hangul Syllable */
  ["\uC6EA", "\uC6EA"], /* Hangul Syllable */
  ["\uC6EB", "\uC6EB"], /* Hangul Syllable */
  ["\uC6EC", "\uC6EC"], /* Hangul Syllable */
  ["\uC6ED", "\uC6ED"], /* Hangul Syllable */
  ["\uC6EE", "\uC6EE"], /* Hangul Syllable */
  ["\uC6EF", "\uC6EF"], /* Hangul Syllable */
  ["\uC6F0", "\uC6F0"], /* Hangul Syllable */
  ["\uC6F1", "\uC6F1"], /* Hangul Syllable */
  ["\uC6F2", "\uC6F2"], /* Hangul Syllable */
  ["\uC6F3", "\uC6F3"], /* Hangul Syllable */
  ["\uC6F4", "\uC6F4"], /* Hangul Syllable */
  ["\uC6F5", "\uC6F5"], /* Hangul Syllable */
  ["\uC6F6", "\uC6F6"], /* Hangul Syllable */
  ["\uC6F7", "\uC6F7"], /* Hangul Syllable */
  ["\uC6F8", "\uC6F8"], /* Hangul Syllable */
  ["\uC6F9", "\uC6F9"], /* Hangul Syllable */
  ["\uC6FA", "\uC6FA"], /* Hangul Syllable */
  ["\uC6FB", "\uC6FB"], /* Hangul Syllable */
  ["\uC6FC", "\uC6FC"], /* Hangul Syllable */
  ["\uC6FD", "\uC6FD"], /* Hangul Syllable */
  ["\uC6FE", "\uC6FE"], /* Hangul Syllable */
  ["\uC6FF", "\uC6FF"], /* Hangul Syllable */
  ["\uC700", "\uC700"], /* Hangul Syllable */
  ["\uC701", "\uC701"], /* Hangul Syllable */
  ["\uC702", "\uC702"], /* Hangul Syllable */
  ["\uC703", "\uC703"], /* Hangul Syllable */
  ["\uC704", "\uC704"], /* Hangul Syllable */
  ["\uC705", "\uC705"], /* Hangul Syllable */
  ["\uC706", "\uC706"], /* Hangul Syllable */
  ["\uC707", "\uC707"], /* Hangul Syllable */
  ["\uC708", "\uC708"], /* Hangul Syllable */
  ["\uC709", "\uC709"], /* Hangul Syllable */
  ["\uC70A", "\uC70A"], /* Hangul Syllable */
  ["\uC70B", "\uC70B"], /* Hangul Syllable */
  ["\uC70C", "\uC70C"], /* Hangul Syllable */
  ["\uC70D", "\uC70D"], /* Hangul Syllable */
  ["\uC70E", "\uC70E"], /* Hangul Syllable */
  ["\uC70F", "\uC70F"], /* Hangul Syllable */
  ["\uC710", "\uC710"], /* Hangul Syllable */
  ["\uC711", "\uC711"], /* Hangul Syllable */
  ["\uC712", "\uC712"], /* Hangul Syllable */
  ["\uC713", "\uC713"], /* Hangul Syllable */
  ["\uC714", "\uC714"], /* Hangul Syllable */
  ["\uC715", "\uC715"], /* Hangul Syllable */
  ["\uC716", "\uC716"], /* Hangul Syllable */
  ["\uC717", "\uC717"], /* Hangul Syllable */
  ["\uC718", "\uC718"], /* Hangul Syllable */
  ["\uC719", "\uC719"], /* Hangul Syllable */
  ["\uC71A", "\uC71A"], /* Hangul Syllable */
  ["\uC71B", "\uC71B"], /* Hangul Syllable */
  ["\uC71C", "\uC71C"], /* Hangul Syllable */
  ["\uC71D", "\uC71D"], /* Hangul Syllable */
  ["\uC71E", "\uC71E"], /* Hangul Syllable */
  ["\uC71F", "\uC71F"], /* Hangul Syllable */
  ["\uC720", "\uC720"], /* Hangul Syllable */
  ["\uC721", "\uC721"], /* Hangul Syllable */
  ["\uC722", "\uC722"], /* Hangul Syllable */
  ["\uC723", "\uC723"], /* Hangul Syllable */
  ["\uC724", "\uC724"], /* Hangul Syllable */
  ["\uC725", "\uC725"], /* Hangul Syllable */
  ["\uC726", "\uC726"], /* Hangul Syllable */
  ["\uC727", "\uC727"], /* Hangul Syllable */
  ["\uC728", "\uC728"], /* Hangul Syllable */
  ["\uC729", "\uC729"], /* Hangul Syllable */
  ["\uC72A", "\uC72A"], /* Hangul Syllable */
  ["\uC72B", "\uC72B"], /* Hangul Syllable */
  ["\uC72C", "\uC72C"], /* Hangul Syllable */
  ["\uC72D", "\uC72D"], /* Hangul Syllable */
  ["\uC72E", "\uC72E"], /* Hangul Syllable */
  ["\uC72F", "\uC72F"], /* Hangul Syllable */
  ["\uC730", "\uC730"], /* Hangul Syllable */
  ["\uC731", "\uC731"], /* Hangul Syllable */
  ["\uC732", "\uC732"], /* Hangul Syllable */
  ["\uC733", "\uC733"], /* Hangul Syllable */
  ["\uC734", "\uC734"], /* Hangul Syllable */
  ["\uC735", "\uC735"], /* Hangul Syllable */
  ["\uC736", "\uC736"], /* Hangul Syllable */
  ["\uC737", "\uC737"], /* Hangul Syllable */
  ["\uC738", "\uC738"], /* Hangul Syllable */
  ["\uC739", "\uC739"], /* Hangul Syllable */
  ["\uC73A", "\uC73A"], /* Hangul Syllable */
  ["\uC73B", "\uC73B"], /* Hangul Syllable */
  ["\uC73C", "\uC73C"], /* Hangul Syllable */
  ["\uC73D", "\uC73D"], /* Hangul Syllable */
  ["\uC73E", "\uC73E"], /* Hangul Syllable */
  ["\uC73F", "\uC73F"], /* Hangul Syllable */
  ["\uC740", "\uC740"], /* Hangul Syllable */
  ["\uC741", "\uC741"], /* Hangul Syllable */
  ["\uC742", "\uC742"], /* Hangul Syllable */
  ["\uC743", "\uC743"], /* Hangul Syllable */
  ["\uC744", "\uC744"], /* Hangul Syllable */
  ["\uC745", "\uC745"], /* Hangul Syllable */
  ["\uC746", "\uC746"], /* Hangul Syllable */
  ["\uC747", "\uC747"], /* Hangul Syllable */
  ["\uC748", "\uC748"], /* Hangul Syllable */
  ["\uC749", "\uC749"], /* Hangul Syllable */
  ["\uC74A", "\uC74A"], /* Hangul Syllable */
  ["\uC74B", "\uC74B"], /* Hangul Syllable */
  ["\uC74C", "\uC74C"], /* Hangul Syllable */
  ["\uC74D", "\uC74D"], /* Hangul Syllable */
  ["\uC74E", "\uC74E"], /* Hangul Syllable */
  ["\uC74F", "\uC74F"], /* Hangul Syllable */
  ["\uC750", "\uC750"], /* Hangul Syllable */
  ["\uC751", "\uC751"], /* Hangul Syllable */
  ["\uC752", "\uC752"], /* Hangul Syllable */
  ["\uC753", "\uC753"], /* Hangul Syllable */
  ["\uC754", "\uC754"], /* Hangul Syllable */
  ["\uC755", "\uC755"], /* Hangul Syllable */
  ["\uC756", "\uC756"], /* Hangul Syllable */
  ["\uC757", "\uC757"], /* Hangul Syllable */
  ["\uC758", "\uC758"], /* Hangul Syllable */
  ["\uC759", "\uC759"], /* Hangul Syllable */
  ["\uC75A", "\uC75A"], /* Hangul Syllable */
  ["\uC75B", "\uC75B"], /* Hangul Syllable */
  ["\uC75C", "\uC75C"], /* Hangul Syllable */
  ["\uC75D", "\uC75D"], /* Hangul Syllable */
  ["\uC75E", "\uC75E"], /* Hangul Syllable */
  ["\uC75F", "\uC75F"], /* Hangul Syllable */
  ["\uC760", "\uC760"], /* Hangul Syllable */
  ["\uC761", "\uC761"], /* Hangul Syllable */
  ["\uC762", "\uC762"], /* Hangul Syllable */
  ["\uC763", "\uC763"], /* Hangul Syllable */
  ["\uC764", "\uC764"], /* Hangul Syllable */
  ["\uC765", "\uC765"], /* Hangul Syllable */
  ["\uC766", "\uC766"], /* Hangul Syllable */
  ["\uC767", "\uC767"], /* Hangul Syllable */
  ["\uC768", "\uC768"], /* Hangul Syllable */
  ["\uC769", "\uC769"], /* Hangul Syllable */
  ["\uC76A", "\uC76A"], /* Hangul Syllable */
  ["\uC76B", "\uC76B"], /* Hangul Syllable */
  ["\uC76C", "\uC76C"], /* Hangul Syllable */
  ["\uC76D", "\uC76D"], /* Hangul Syllable */
  ["\uC76E", "\uC76E"], /* Hangul Syllable */
  ["\uC76F", "\uC76F"], /* Hangul Syllable */
  ["\uC770", "\uC770"], /* Hangul Syllable */
  ["\uC771", "\uC771"], /* Hangul Syllable */
  ["\uC772", "\uC772"], /* Hangul Syllable */
  ["\uC773", "\uC773"], /* Hangul Syllable */
  ["\uC774", "\uC774"], /* Hangul Syllable */
  ["\uC775", "\uC775"], /* Hangul Syllable */
  ["\uC776", "\uC776"], /* Hangul Syllable */
  ["\uC777", "\uC777"], /* Hangul Syllable */
  ["\uC778", "\uC778"], /* Hangul Syllable */
  ["\uC779", "\uC779"], /* Hangul Syllable */
  ["\uC77A", "\uC77A"], /* Hangul Syllable */
  ["\uC77B", "\uC77B"], /* Hangul Syllable */
  ["\uC77C", "\uC77C"], /* Hangul Syllable */
  ["\uC77D", "\uC77D"], /* Hangul Syllable */
  ["\uC77E", "\uC77E"], /* Hangul Syllable */
  ["\uC77F", "\uC77F"], /* Hangul Syllable */
  ["\uC780", "\uC780"], /* Hangul Syllable */
  ["\uC781", "\uC781"], /* Hangul Syllable */
  ["\uC782", "\uC782"], /* Hangul Syllable */
  ["\uC783", "\uC783"], /* Hangul Syllable */
  ["\uC784", "\uC784"], /* Hangul Syllable */
  ["\uC785", "\uC785"], /* Hangul Syllable */
  ["\uC786", "\uC786"], /* Hangul Syllable */
  ["\uC787", "\uC787"], /* Hangul Syllable */
  ["\uC788", "\uC788"], /* Hangul Syllable */
  ["\uC789", "\uC789"], /* Hangul Syllable */
  ["\uC78A", "\uC78A"], /* Hangul Syllable */
  ["\uC78B", "\uC78B"], /* Hangul Syllable */
  ["\uC78C", "\uC78C"], /* Hangul Syllable */
  ["\uC78D", "\uC78D"], /* Hangul Syllable */
  ["\uC78E", "\uC78E"], /* Hangul Syllable */
  ["\uC78F", "\uC78F"], /* Hangul Syllable */
  ["\uC790", "\uC790"], /* Hangul Syllable */
  ["\uC791", "\uC791"], /* Hangul Syllable */
  ["\uC792", "\uC792"], /* Hangul Syllable */
  ["\uC793", "\uC793"], /* Hangul Syllable */
  ["\uC794", "\uC794"], /* Hangul Syllable */
  ["\uC795", "\uC795"], /* Hangul Syllable */
  ["\uC796", "\uC796"], /* Hangul Syllable */
  ["\uC797", "\uC797"], /* Hangul Syllable */
  ["\uC798", "\uC798"], /* Hangul Syllable */
  ["\uC799", "\uC799"], /* Hangul Syllable */
  ["\uC79A", "\uC79A"], /* Hangul Syllable */
  ["\uC79B", "\uC79B"], /* Hangul Syllable */
  ["\uC79C", "\uC79C"], /* Hangul Syllable */
  ["\uC79D", "\uC79D"], /* Hangul Syllable */
  ["\uC79E", "\uC79E"], /* Hangul Syllable */
  ["\uC79F", "\uC79F"], /* Hangul Syllable */
  ["\uC7A0", "\uC7A0"], /* Hangul Syllable */
  ["\uC7A1", "\uC7A1"], /* Hangul Syllable */
  ["\uC7A2", "\uC7A2"], /* Hangul Syllable */
  ["\uC7A3", "\uC7A3"], /* Hangul Syllable */
  ["\uC7A4", "\uC7A4"], /* Hangul Syllable */
  ["\uC7A5", "\uC7A5"], /* Hangul Syllable */
  ["\uC7A6", "\uC7A6"], /* Hangul Syllable */
  ["\uC7A7", "\uC7A7"], /* Hangul Syllable */
  ["\uC7A8", "\uC7A8"], /* Hangul Syllable */
  ["\uC7A9", "\uC7A9"], /* Hangul Syllable */
  ["\uC7AA", "\uC7AA"], /* Hangul Syllable */
  ["\uC7AB", "\uC7AB"], /* Hangul Syllable */
  ["\uC7AC", "\uC7AC"], /* Hangul Syllable */
  ["\uC7AD", "\uC7AD"], /* Hangul Syllable */
  ["\uC7AE", "\uC7AE"], /* Hangul Syllable */
  ["\uC7AF", "\uC7AF"], /* Hangul Syllable */
  ["\uC7B0", "\uC7B0"], /* Hangul Syllable */
  ["\uC7B1", "\uC7B1"], /* Hangul Syllable */
  ["\uC7B2", "\uC7B2"], /* Hangul Syllable */
  ["\uC7B3", "\uC7B3"], /* Hangul Syllable */
  ["\uC7B4", "\uC7B4"], /* Hangul Syllable */
  ["\uC7B5", "\uC7B5"], /* Hangul Syllable */
  ["\uC7B6", "\uC7B6"], /* Hangul Syllable */
  ["\uC7B7", "\uC7B7"], /* Hangul Syllable */
  ["\uC7B8", "\uC7B8"], /* Hangul Syllable */
  ["\uC7B9", "\uC7B9"], /* Hangul Syllable */
  ["\uC7BA", "\uC7BA"], /* Hangul Syllable */
  ["\uC7BB", "\uC7BB"], /* Hangul Syllable */
  ["\uC7BC", "\uC7BC"], /* Hangul Syllable */
  ["\uC7BD", "\uC7BD"], /* Hangul Syllable */
  ["\uC7BE", "\uC7BE"], /* Hangul Syllable */
  ["\uC7BF", "\uC7BF"], /* Hangul Syllable */
  ["\uC7C0", "\uC7C0"], /* Hangul Syllable */
  ["\uC7C1", "\uC7C1"], /* Hangul Syllable */
  ["\uC7C2", "\uC7C2"], /* Hangul Syllable */
  ["\uC7C3", "\uC7C3"], /* Hangul Syllable */
  ["\uC7C4", "\uC7C4"], /* Hangul Syllable */
  ["\uC7C5", "\uC7C5"], /* Hangul Syllable */
  ["\uC7C6", "\uC7C6"], /* Hangul Syllable */
  ["\uC7C7", "\uC7C7"], /* Hangul Syllable */
  ["\uC7C8", "\uC7C8"], /* Hangul Syllable */
  ["\uC7C9", "\uC7C9"], /* Hangul Syllable */
  ["\uC7CA", "\uC7CA"], /* Hangul Syllable */
  ["\uC7CB", "\uC7CB"], /* Hangul Syllable */
  ["\uC7CC", "\uC7CC"], /* Hangul Syllable */
  ["\uC7CD", "\uC7CD"], /* Hangul Syllable */
  ["\uC7CE", "\uC7CE"], /* Hangul Syllable */
  ["\uC7CF", "\uC7CF"], /* Hangul Syllable */
  ["\uC7D0", "\uC7D0"], /* Hangul Syllable */
  ["\uC7D1", "\uC7D1"], /* Hangul Syllable */
  ["\uC7D2", "\uC7D2"], /* Hangul Syllable */
  ["\uC7D3", "\uC7D3"], /* Hangul Syllable */
  ["\uC7D4", "\uC7D4"], /* Hangul Syllable */
  ["\uC7D5", "\uC7D5"], /* Hangul Syllable */
  ["\uC7D6", "\uC7D6"], /* Hangul Syllable */
  ["\uC7D7", "\uC7D7"], /* Hangul Syllable */
  ["\uC7D8", "\uC7D8"], /* Hangul Syllable */
  ["\uC7D9", "\uC7D9"], /* Hangul Syllable */
  ["\uC7DA", "\uC7DA"], /* Hangul Syllable */
  ["\uC7DB", "\uC7DB"], /* Hangul Syllable */
  ["\uC7DC", "\uC7DC"], /* Hangul Syllable */
  ["\uC7DD", "\uC7DD"], /* Hangul Syllable */
  ["\uC7DE", "\uC7DE"], /* Hangul Syllable */
  ["\uC7DF", "\uC7DF"], /* Hangul Syllable */
  ["\uC7E0", "\uC7E0"], /* Hangul Syllable */
  ["\uC7E1", "\uC7E1"], /* Hangul Syllable */
  ["\uC7E2", "\uC7E2"], /* Hangul Syllable */
  ["\uC7E3", "\uC7E3"], /* Hangul Syllable */
  ["\uC7E4", "\uC7E4"], /* Hangul Syllable */
  ["\uC7E5", "\uC7E5"], /* Hangul Syllable */
  ["\uC7E6", "\uC7E6"], /* Hangul Syllable */
  ["\uC7E7", "\uC7E7"], /* Hangul Syllable */
  ["\uC7E8", "\uC7E8"], /* Hangul Syllable */
  ["\uC7E9", "\uC7E9"], /* Hangul Syllable */
  ["\uC7EA", "\uC7EA"], /* Hangul Syllable */
  ["\uC7EB", "\uC7EB"], /* Hangul Syllable */
  ["\uC7EC", "\uC7EC"], /* Hangul Syllable */
  ["\uC7ED", "\uC7ED"], /* Hangul Syllable */
  ["\uC7EE", "\uC7EE"], /* Hangul Syllable */
  ["\uC7EF", "\uC7EF"], /* Hangul Syllable */
  ["\uC7F0", "\uC7F0"], /* Hangul Syllable */
  ["\uC7F1", "\uC7F1"], /* Hangul Syllable */
  ["\uC7F2", "\uC7F2"], /* Hangul Syllable */
  ["\uC7F3", "\uC7F3"], /* Hangul Syllable */
  ["\uC7F4", "\uC7F4"], /* Hangul Syllable */
  ["\uC7F5", "\uC7F5"], /* Hangul Syllable */
  ["\uC7F6", "\uC7F6"], /* Hangul Syllable */
  ["\uC7F7", "\uC7F7"], /* Hangul Syllable */
  ["\uC7F8", "\uC7F8"], /* Hangul Syllable */
  ["\uC7F9", "\uC7F9"], /* Hangul Syllable */
  ["\uC7FA", "\uC7FA"], /* Hangul Syllable */
  ["\uC7FB", "\uC7FB"], /* Hangul Syllable */
  ["\uC7FC", "\uC7FC"], /* Hangul Syllable */
  ["\uC7FD", "\uC7FD"], /* Hangul Syllable */
  ["\uC7FE", "\uC7FE"], /* Hangul Syllable */
  ["\uC7FF", "\uC7FF"], /* Hangul Syllable */
  ["\uC800", "\uC800"], /* Hangul Syllable */
  ["\uC801", "\uC801"], /* Hangul Syllable */
  ["\uC802", "\uC802"], /* Hangul Syllable */
  ["\uC803", "\uC803"], /* Hangul Syllable */
  ["\uC804", "\uC804"], /* Hangul Syllable */
  ["\uC805", "\uC805"], /* Hangul Syllable */
  ["\uC806", "\uC806"], /* Hangul Syllable */
  ["\uC807", "\uC807"], /* Hangul Syllable */
  ["\uC808", "\uC808"], /* Hangul Syllable */
  ["\uC809", "\uC809"], /* Hangul Syllable */
  ["\uC80A", "\uC80A"], /* Hangul Syllable */
  ["\uC80B", "\uC80B"], /* Hangul Syllable */
  ["\uC80C", "\uC80C"], /* Hangul Syllable */
  ["\uC80D", "\uC80D"], /* Hangul Syllable */
  ["\uC80E", "\uC80E"], /* Hangul Syllable */
  ["\uC80F", "\uC80F"], /* Hangul Syllable */
  ["\uC810", "\uC810"], /* Hangul Syllable */
  ["\uC811", "\uC811"], /* Hangul Syllable */
  ["\uC812", "\uC812"], /* Hangul Syllable */
  ["\uC813", "\uC813"], /* Hangul Syllable */
  ["\uC814", "\uC814"], /* Hangul Syllable */
  ["\uC815", "\uC815"], /* Hangul Syllable */
  ["\uC816", "\uC816"], /* Hangul Syllable */
  ["\uC817", "\uC817"], /* Hangul Syllable */
  ["\uC818", "\uC818"], /* Hangul Syllable */
  ["\uC819", "\uC819"], /* Hangul Syllable */
  ["\uC81A", "\uC81A"], /* Hangul Syllable */
  ["\uC81B", "\uC81B"], /* Hangul Syllable */
  ["\uC81C", "\uC81C"], /* Hangul Syllable */
  ["\uC81D", "\uC81D"], /* Hangul Syllable */
  ["\uC81E", "\uC81E"], /* Hangul Syllable */
  ["\uC81F", "\uC81F"], /* Hangul Syllable */
  ["\uC820", "\uC820"], /* Hangul Syllable */
  ["\uC821", "\uC821"], /* Hangul Syllable */
  ["\uC822", "\uC822"], /* Hangul Syllable */
  ["\uC823", "\uC823"], /* Hangul Syllable */
  ["\uC824", "\uC824"], /* Hangul Syllable */
  ["\uC825", "\uC825"], /* Hangul Syllable */
  ["\uC826", "\uC826"], /* Hangul Syllable */
  ["\uC827", "\uC827"], /* Hangul Syllable */
  ["\uC828", "\uC828"], /* Hangul Syllable */
  ["\uC829", "\uC829"], /* Hangul Syllable */
  ["\uC82A", "\uC82A"], /* Hangul Syllable */
  ["\uC82B", "\uC82B"], /* Hangul Syllable */
  ["\uC82C", "\uC82C"], /* Hangul Syllable */
  ["\uC82D", "\uC82D"], /* Hangul Syllable */
  ["\uC82E", "\uC82E"], /* Hangul Syllable */
  ["\uC82F", "\uC82F"], /* Hangul Syllable */
  ["\uC830", "\uC830"], /* Hangul Syllable */
  ["\uC831", "\uC831"], /* Hangul Syllable */
  ["\uC832", "\uC832"], /* Hangul Syllable */
  ["\uC833", "\uC833"], /* Hangul Syllable */
  ["\uC834", "\uC834"], /* Hangul Syllable */
  ["\uC835", "\uC835"], /* Hangul Syllable */
  ["\uC836", "\uC836"], /* Hangul Syllable */
  ["\uC837", "\uC837"], /* Hangul Syllable */
  ["\uC838", "\uC838"], /* Hangul Syllable */
  ["\uC839", "\uC839"], /* Hangul Syllable */
  ["\uC83A", "\uC83A"], /* Hangul Syllable */
  ["\uC83B", "\uC83B"], /* Hangul Syllable */
  ["\uC83C", "\uC83C"], /* Hangul Syllable */
  ["\uC83D", "\uC83D"], /* Hangul Syllable */
  ["\uC83E", "\uC83E"], /* Hangul Syllable */
  ["\uC83F", "\uC83F"], /* Hangul Syllable */
  ["\uC840", "\uC840"], /* Hangul Syllable */
  ["\uC841", "\uC841"], /* Hangul Syllable */
  ["\uC842", "\uC842"], /* Hangul Syllable */
  ["\uC843", "\uC843"], /* Hangul Syllable */
  ["\uC844", "\uC844"], /* Hangul Syllable */
  ["\uC845", "\uC845"], /* Hangul Syllable */
  ["\uC846", "\uC846"], /* Hangul Syllable */
  ["\uC847", "\uC847"], /* Hangul Syllable */
  ["\uC848", "\uC848"], /* Hangul Syllable */
  ["\uC849", "\uC849"], /* Hangul Syllable */
  ["\uC84A", "\uC84A"], /* Hangul Syllable */
  ["\uC84B", "\uC84B"], /* Hangul Syllable */
  ["\uC84C", "\uC84C"], /* Hangul Syllable */
  ["\uC84D", "\uC84D"], /* Hangul Syllable */
  ["\uC84E", "\uC84E"], /* Hangul Syllable */
  ["\uC84F", "\uC84F"], /* Hangul Syllable */
  ["\uC850", "\uC850"], /* Hangul Syllable */
  ["\uC851", "\uC851"], /* Hangul Syllable */
  ["\uC852", "\uC852"], /* Hangul Syllable */
  ["\uC853", "\uC853"], /* Hangul Syllable */
  ["\uC854", "\uC854"], /* Hangul Syllable */
  ["\uC855", "\uC855"], /* Hangul Syllable */
  ["\uC856", "\uC856"], /* Hangul Syllable */
  ["\uC857", "\uC857"], /* Hangul Syllable */
  ["\uC858", "\uC858"], /* Hangul Syllable */
  ["\uC859", "\uC859"], /* Hangul Syllable */
  ["\uC85A", "\uC85A"], /* Hangul Syllable */
  ["\uC85B", "\uC85B"], /* Hangul Syllable */
  ["\uC85C", "\uC85C"], /* Hangul Syllable */
  ["\uC85D", "\uC85D"], /* Hangul Syllable */
  ["\uC85E", "\uC85E"], /* Hangul Syllable */
  ["\uC85F", "\uC85F"], /* Hangul Syllable */
  ["\uC860", "\uC860"], /* Hangul Syllable */
  ["\uC861", "\uC861"], /* Hangul Syllable */
  ["\uC862", "\uC862"], /* Hangul Syllable */
  ["\uC863", "\uC863"], /* Hangul Syllable */
  ["\uC864", "\uC864"], /* Hangul Syllable */
  ["\uC865", "\uC865"], /* Hangul Syllable */
  ["\uC866", "\uC866"], /* Hangul Syllable */
  ["\uC867", "\uC867"], /* Hangul Syllable */
  ["\uC868", "\uC868"], /* Hangul Syllable */
  ["\uC869", "\uC869"], /* Hangul Syllable */
  ["\uC86A", "\uC86A"], /* Hangul Syllable */
  ["\uC86B", "\uC86B"], /* Hangul Syllable */
  ["\uC86C", "\uC86C"], /* Hangul Syllable */
  ["\uC86D", "\uC86D"], /* Hangul Syllable */
  ["\uC86E", "\uC86E"], /* Hangul Syllable */
  ["\uC86F", "\uC86F"], /* Hangul Syllable */
  ["\uC870", "\uC870"], /* Hangul Syllable */
  ["\uC871", "\uC871"], /* Hangul Syllable */
  ["\uC872", "\uC872"], /* Hangul Syllable */
  ["\uC873", "\uC873"], /* Hangul Syllable */
  ["\uC874", "\uC874"], /* Hangul Syllable */
  ["\uC875", "\uC875"], /* Hangul Syllable */
  ["\uC876", "\uC876"], /* Hangul Syllable */
  ["\uC877", "\uC877"], /* Hangul Syllable */
  ["\uC878", "\uC878"], /* Hangul Syllable */
  ["\uC879", "\uC879"], /* Hangul Syllable */
  ["\uC87A", "\uC87A"], /* Hangul Syllable */
  ["\uC87B", "\uC87B"], /* Hangul Syllable */
  ["\uC87C", "\uC87C"], /* Hangul Syllable */
  ["\uC87D", "\uC87D"], /* Hangul Syllable */
  ["\uC87E", "\uC87E"], /* Hangul Syllable */
  ["\uC87F", "\uC87F"], /* Hangul Syllable */
  ["\uC880", "\uC880"], /* Hangul Syllable */
  ["\uC881", "\uC881"], /* Hangul Syllable */
  ["\uC882", "\uC882"], /* Hangul Syllable */
  ["\uC883", "\uC883"], /* Hangul Syllable */
  ["\uC884", "\uC884"], /* Hangul Syllable */
  ["\uC885", "\uC885"], /* Hangul Syllable */
  ["\uC886", "\uC886"], /* Hangul Syllable */
  ["\uC887", "\uC887"], /* Hangul Syllable */
  ["\uC888", "\uC888"], /* Hangul Syllable */
  ["\uC889", "\uC889"], /* Hangul Syllable */
  ["\uC88A", "\uC88A"], /* Hangul Syllable */
  ["\uC88B", "\uC88B"], /* Hangul Syllable */
  ["\uC88C", "\uC88C"], /* Hangul Syllable */
  ["\uC88D", "\uC88D"], /* Hangul Syllable */
  ["\uC88E", "\uC88E"], /* Hangul Syllable */
  ["\uC88F", "\uC88F"], /* Hangul Syllable */
  ["\uC890", "\uC890"], /* Hangul Syllable */
  ["\uC891", "\uC891"], /* Hangul Syllable */
  ["\uC892", "\uC892"], /* Hangul Syllable */
  ["\uC893", "\uC893"], /* Hangul Syllable */
  ["\uC894", "\uC894"], /* Hangul Syllable */
  ["\uC895", "\uC895"], /* Hangul Syllable */
  ["\uC896", "\uC896"], /* Hangul Syllable */
  ["\uC897", "\uC897"], /* Hangul Syllable */
  ["\uC898", "\uC898"], /* Hangul Syllable */
  ["\uC899", "\uC899"], /* Hangul Syllable */
  ["\uC89A", "\uC89A"], /* Hangul Syllable */
  ["\uC89B", "\uC89B"], /* Hangul Syllable */
  ["\uC89C", "\uC89C"], /* Hangul Syllable */
  ["\uC89D", "\uC89D"], /* Hangul Syllable */
  ["\uC89E", "\uC89E"], /* Hangul Syllable */
  ["\uC89F", "\uC89F"], /* Hangul Syllable */
  ["\uC8A0", "\uC8A0"], /* Hangul Syllable */
  ["\uC8A1", "\uC8A1"], /* Hangul Syllable */
  ["\uC8A2", "\uC8A2"], /* Hangul Syllable */
  ["\uC8A3", "\uC8A3"], /* Hangul Syllable */
  ["\uC8A4", "\uC8A4"], /* Hangul Syllable */
  ["\uC8A5", "\uC8A5"], /* Hangul Syllable */
  ["\uC8A6", "\uC8A6"], /* Hangul Syllable */
  ["\uC8A7", "\uC8A7"], /* Hangul Syllable */
  ["\uC8A8", "\uC8A8"], /* Hangul Syllable */
  ["\uC8A9", "\uC8A9"], /* Hangul Syllable */
  ["\uC8AA", "\uC8AA"], /* Hangul Syllable */
  ["\uC8AB", "\uC8AB"], /* Hangul Syllable */
  ["\uC8AC", "\uC8AC"], /* Hangul Syllable */
  ["\uC8AD", "\uC8AD"], /* Hangul Syllable */
  ["\uC8AE", "\uC8AE"], /* Hangul Syllable */
  ["\uC8AF", "\uC8AF"], /* Hangul Syllable */
  ["\uC8B0", "\uC8B0"], /* Hangul Syllable */
  ["\uC8B1", "\uC8B1"], /* Hangul Syllable */
  ["\uC8B2", "\uC8B2"], /* Hangul Syllable */
  ["\uC8B3", "\uC8B3"], /* Hangul Syllable */
  ["\uC8B4", "\uC8B4"], /* Hangul Syllable */
  ["\uC8B5", "\uC8B5"], /* Hangul Syllable */
  ["\uC8B6", "\uC8B6"], /* Hangul Syllable */
  ["\uC8B7", "\uC8B7"], /* Hangul Syllable */
  ["\uC8B8", "\uC8B8"], /* Hangul Syllable */
  ["\uC8B9", "\uC8B9"], /* Hangul Syllable */
  ["\uC8BA", "\uC8BA"], /* Hangul Syllable */
  ["\uC8BB", "\uC8BB"], /* Hangul Syllable */
  ["\uC8BC", "\uC8BC"], /* Hangul Syllable */
  ["\uC8BD", "\uC8BD"], /* Hangul Syllable */
  ["\uC8BE", "\uC8BE"], /* Hangul Syllable */
  ["\uC8BF", "\uC8BF"], /* Hangul Syllable */
  ["\uC8C0", "\uC8C0"], /* Hangul Syllable */
  ["\uC8C1", "\uC8C1"], /* Hangul Syllable */
  ["\uC8C2", "\uC8C2"], /* Hangul Syllable */
  ["\uC8C3", "\uC8C3"], /* Hangul Syllable */
  ["\uC8C4", "\uC8C4"], /* Hangul Syllable */
  ["\uC8C5", "\uC8C5"], /* Hangul Syllable */
  ["\uC8C6", "\uC8C6"], /* Hangul Syllable */
  ["\uC8C7", "\uC8C7"], /* Hangul Syllable */
  ["\uC8C8", "\uC8C8"], /* Hangul Syllable */
  ["\uC8C9", "\uC8C9"], /* Hangul Syllable */
  ["\uC8CA", "\uC8CA"], /* Hangul Syllable */
  ["\uC8CB", "\uC8CB"], /* Hangul Syllable */
  ["\uC8CC", "\uC8CC"], /* Hangul Syllable */
  ["\uC8CD", "\uC8CD"], /* Hangul Syllable */
  ["\uC8CE", "\uC8CE"], /* Hangul Syllable */
  ["\uC8CF", "\uC8CF"], /* Hangul Syllable */
  ["\uC8D0", "\uC8D0"], /* Hangul Syllable */
  ["\uC8D1", "\uC8D1"], /* Hangul Syllable */
  ["\uC8D2", "\uC8D2"], /* Hangul Syllable */
  ["\uC8D3", "\uC8D3"], /* Hangul Syllable */
  ["\uC8D4", "\uC8D4"], /* Hangul Syllable */
  ["\uC8D5", "\uC8D5"], /* Hangul Syllable */
  ["\uC8D6", "\uC8D6"], /* Hangul Syllable */
  ["\uC8D7", "\uC8D7"], /* Hangul Syllable */
  ["\uC8D8", "\uC8D8"], /* Hangul Syllable */
  ["\uC8D9", "\uC8D9"], /* Hangul Syllable */
  ["\uC8DA", "\uC8DA"], /* Hangul Syllable */
  ["\uC8DB", "\uC8DB"], /* Hangul Syllable */
  ["\uC8DC", "\uC8DC"], /* Hangul Syllable */
  ["\uC8DD", "\uC8DD"], /* Hangul Syllable */
  ["\uC8DE", "\uC8DE"], /* Hangul Syllable */
  ["\uC8DF", "\uC8DF"], /* Hangul Syllable */
  ["\uC8E0", "\uC8E0"], /* Hangul Syllable */
  ["\uC8E1", "\uC8E1"], /* Hangul Syllable */
  ["\uC8E2", "\uC8E2"], /* Hangul Syllable */
  ["\uC8E3", "\uC8E3"], /* Hangul Syllable */
  ["\uC8E4", "\uC8E4"], /* Hangul Syllable */
  ["\uC8E5", "\uC8E5"], /* Hangul Syllable */
  ["\uC8E6", "\uC8E6"], /* Hangul Syllable */
  ["\uC8E7", "\uC8E7"], /* Hangul Syllable */
  ["\uC8E8", "\uC8E8"], /* Hangul Syllable */
  ["\uC8E9", "\uC8E9"], /* Hangul Syllable */
  ["\uC8EA", "\uC8EA"], /* Hangul Syllable */
  ["\uC8EB", "\uC8EB"], /* Hangul Syllable */
  ["\uC8EC", "\uC8EC"], /* Hangul Syllable */
  ["\uC8ED", "\uC8ED"], /* Hangul Syllable */
  ["\uC8EE", "\uC8EE"], /* Hangul Syllable */
  ["\uC8EF", "\uC8EF"], /* Hangul Syllable */
  ["\uC8F0", "\uC8F0"], /* Hangul Syllable */
  ["\uC8F1", "\uC8F1"], /* Hangul Syllable */
  ["\uC8F2", "\uC8F2"], /* Hangul Syllable */
  ["\uC8F3", "\uC8F3"], /* Hangul Syllable */
  ["\uC8F4", "\uC8F4"], /* Hangul Syllable */
  ["\uC8F5", "\uC8F5"], /* Hangul Syllable */
  ["\uC8F6", "\uC8F6"], /* Hangul Syllable */
  ["\uC8F7", "\uC8F7"], /* Hangul Syllable */
  ["\uC8F8", "\uC8F8"], /* Hangul Syllable */
  ["\uC8F9", "\uC8F9"], /* Hangul Syllable */
  ["\uC8FA", "\uC8FA"], /* Hangul Syllable */
  ["\uC8FB", "\uC8FB"], /* Hangul Syllable */
  ["\uC8FC", "\uC8FC"], /* Hangul Syllable */
  ["\uC8FD", "\uC8FD"], /* Hangul Syllable */
  ["\uC8FE", "\uC8FE"], /* Hangul Syllable */
  ["\uC8FF", "\uC8FF"], /* Hangul Syllable */
  ["\uC900", "\uC900"], /* Hangul Syllable */
  ["\uC901", "\uC901"], /* Hangul Syllable */
  ["\uC902", "\uC902"], /* Hangul Syllable */
  ["\uC903", "\uC903"], /* Hangul Syllable */
  ["\uC904", "\uC904"], /* Hangul Syllable */
  ["\uC905", "\uC905"], /* Hangul Syllable */
  ["\uC906", "\uC906"], /* Hangul Syllable */
  ["\uC907", "\uC907"], /* Hangul Syllable */
  ["\uC908", "\uC908"], /* Hangul Syllable */
  ["\uC909", "\uC909"], /* Hangul Syllable */
  ["\uC90A", "\uC90A"], /* Hangul Syllable */
  ["\uC90B", "\uC90B"], /* Hangul Syllable */
  ["\uC90C", "\uC90C"], /* Hangul Syllable */
  ["\uC90D", "\uC90D"], /* Hangul Syllable */
  ["\uC90E", "\uC90E"], /* Hangul Syllable */
  ["\uC90F", "\uC90F"], /* Hangul Syllable */
  ["\uC910", "\uC910"], /* Hangul Syllable */
  ["\uC911", "\uC911"], /* Hangul Syllable */
  ["\uC912", "\uC912"], /* Hangul Syllable */
  ["\uC913", "\uC913"], /* Hangul Syllable */
  ["\uC914", "\uC914"], /* Hangul Syllable */
  ["\uC915", "\uC915"], /* Hangul Syllable */
  ["\uC916", "\uC916"], /* Hangul Syllable */
  ["\uC917", "\uC917"], /* Hangul Syllable */
  ["\uC918", "\uC918"], /* Hangul Syllable */
  ["\uC919", "\uC919"], /* Hangul Syllable */
  ["\uC91A", "\uC91A"], /* Hangul Syllable */
  ["\uC91B", "\uC91B"], /* Hangul Syllable */
  ["\uC91C", "\uC91C"], /* Hangul Syllable */
  ["\uC91D", "\uC91D"], /* Hangul Syllable */
  ["\uC91E", "\uC91E"], /* Hangul Syllable */
  ["\uC91F", "\uC91F"], /* Hangul Syllable */
  ["\uC920", "\uC920"], /* Hangul Syllable */
  ["\uC921", "\uC921"], /* Hangul Syllable */
  ["\uC922", "\uC922"], /* Hangul Syllable */
  ["\uC923", "\uC923"], /* Hangul Syllable */
  ["\uC924", "\uC924"], /* Hangul Syllable */
  ["\uC925", "\uC925"], /* Hangul Syllable */
  ["\uC926", "\uC926"], /* Hangul Syllable */
  ["\uC927", "\uC927"], /* Hangul Syllable */
  ["\uC928", "\uC928"], /* Hangul Syllable */
  ["\uC929", "\uC929"], /* Hangul Syllable */
  ["\uC92A", "\uC92A"], /* Hangul Syllable */
  ["\uC92B", "\uC92B"], /* Hangul Syllable */
  ["\uC92C", "\uC92C"], /* Hangul Syllable */
  ["\uC92D", "\uC92D"], /* Hangul Syllable */
  ["\uC92E", "\uC92E"], /* Hangul Syllable */
  ["\uC92F", "\uC92F"], /* Hangul Syllable */
  ["\uC930", "\uC930"], /* Hangul Syllable */
  ["\uC931", "\uC931"], /* Hangul Syllable */
  ["\uC932", "\uC932"], /* Hangul Syllable */
  ["\uC933", "\uC933"], /* Hangul Syllable */
  ["\uC934", "\uC934"], /* Hangul Syllable */
  ["\uC935", "\uC935"], /* Hangul Syllable */
  ["\uC936", "\uC936"], /* Hangul Syllable */
  ["\uC937", "\uC937"], /* Hangul Syllable */
  ["\uC938", "\uC938"], /* Hangul Syllable */
  ["\uC939", "\uC939"], /* Hangul Syllable */
  ["\uC93A", "\uC93A"], /* Hangul Syllable */
  ["\uC93B", "\uC93B"], /* Hangul Syllable */
  ["\uC93C", "\uC93C"], /* Hangul Syllable */
  ["\uC93D", "\uC93D"], /* Hangul Syllable */
  ["\uC93E", "\uC93E"], /* Hangul Syllable */
  ["\uC93F", "\uC93F"], /* Hangul Syllable */
  ["\uC940", "\uC940"], /* Hangul Syllable */
  ["\uC941", "\uC941"], /* Hangul Syllable */
  ["\uC942", "\uC942"], /* Hangul Syllable */
  ["\uC943", "\uC943"], /* Hangul Syllable */
  ["\uC944", "\uC944"], /* Hangul Syllable */
  ["\uC945", "\uC945"], /* Hangul Syllable */
  ["\uC946", "\uC946"], /* Hangul Syllable */
  ["\uC947", "\uC947"], /* Hangul Syllable */
  ["\uC948", "\uC948"], /* Hangul Syllable */
  ["\uC949", "\uC949"], /* Hangul Syllable */
  ["\uC94A", "\uC94A"], /* Hangul Syllable */
  ["\uC94B", "\uC94B"], /* Hangul Syllable */
  ["\uC94C", "\uC94C"], /* Hangul Syllable */
  ["\uC94D", "\uC94D"], /* Hangul Syllable */
  ["\uC94E", "\uC94E"], /* Hangul Syllable */
  ["\uC94F", "\uC94F"], /* Hangul Syllable */
  ["\uC950", "\uC950"], /* Hangul Syllable */
  ["\uC951", "\uC951"], /* Hangul Syllable */
  ["\uC952", "\uC952"], /* Hangul Syllable */
  ["\uC953", "\uC953"], /* Hangul Syllable */
  ["\uC954", "\uC954"], /* Hangul Syllable */
  ["\uC955", "\uC955"], /* Hangul Syllable */
  ["\uC956", "\uC956"], /* Hangul Syllable */
  ["\uC957", "\uC957"], /* Hangul Syllable */
  ["\uC958", "\uC958"], /* Hangul Syllable */
  ["\uC959", "\uC959"], /* Hangul Syllable */
  ["\uC95A", "\uC95A"], /* Hangul Syllable */
  ["\uC95B", "\uC95B"], /* Hangul Syllable */
  ["\uC95C", "\uC95C"], /* Hangul Syllable */
  ["\uC95D", "\uC95D"], /* Hangul Syllable */
  ["\uC95E", "\uC95E"], /* Hangul Syllable */
  ["\uC95F", "\uC95F"], /* Hangul Syllable */
  ["\uC960", "\uC960"], /* Hangul Syllable */
  ["\uC961", "\uC961"], /* Hangul Syllable */
  ["\uC962", "\uC962"], /* Hangul Syllable */
  ["\uC963", "\uC963"], /* Hangul Syllable */
  ["\uC964", "\uC964"], /* Hangul Syllable */
  ["\uC965", "\uC965"], /* Hangul Syllable */
  ["\uC966", "\uC966"], /* Hangul Syllable */
  ["\uC967", "\uC967"], /* Hangul Syllable */
  ["\uC968", "\uC968"], /* Hangul Syllable */
  ["\uC969", "\uC969"], /* Hangul Syllable */
  ["\uC96A", "\uC96A"], /* Hangul Syllable */
  ["\uC96B", "\uC96B"], /* Hangul Syllable */
  ["\uC96C", "\uC96C"], /* Hangul Syllable */
  ["\uC96D", "\uC96D"], /* Hangul Syllable */
  ["\uC96E", "\uC96E"], /* Hangul Syllable */
  ["\uC96F", "\uC96F"], /* Hangul Syllable */
  ["\uC970", "\uC970"], /* Hangul Syllable */
  ["\uC971", "\uC971"], /* Hangul Syllable */
  ["\uC972", "\uC972"], /* Hangul Syllable */
  ["\uC973", "\uC973"], /* Hangul Syllable */
  ["\uC974", "\uC974"], /* Hangul Syllable */
  ["\uC975", "\uC975"], /* Hangul Syllable */
  ["\uC976", "\uC976"], /* Hangul Syllable */
  ["\uC977", "\uC977"], /* Hangul Syllable */
  ["\uC978", "\uC978"], /* Hangul Syllable */
  ["\uC979", "\uC979"], /* Hangul Syllable */
  ["\uC97A", "\uC97A"], /* Hangul Syllable */
  ["\uC97B", "\uC97B"], /* Hangul Syllable */
  ["\uC97C", "\uC97C"], /* Hangul Syllable */
  ["\uC97D", "\uC97D"], /* Hangul Syllable */
  ["\uC97E", "\uC97E"], /* Hangul Syllable */
  ["\uC97F", "\uC97F"], /* Hangul Syllable */
  ["\uC980", "\uC980"], /* Hangul Syllable */
  ["\uC981", "\uC981"], /* Hangul Syllable */
  ["\uC982", "\uC982"], /* Hangul Syllable */
  ["\uC983", "\uC983"], /* Hangul Syllable */
  ["\uC984", "\uC984"], /* Hangul Syllable */
  ["\uC985", "\uC985"], /* Hangul Syllable */
  ["\uC986", "\uC986"], /* Hangul Syllable */
  ["\uC987", "\uC987"], /* Hangul Syllable */
  ["\uC988", "\uC988"], /* Hangul Syllable */
  ["\uC989", "\uC989"], /* Hangul Syllable */
  ["\uC98A", "\uC98A"], /* Hangul Syllable */
  ["\uC98B", "\uC98B"], /* Hangul Syllable */
  ["\uC98C", "\uC98C"], /* Hangul Syllable */
  ["\uC98D", "\uC98D"], /* Hangul Syllable */
  ["\uC98E", "\uC98E"], /* Hangul Syllable */
  ["\uC98F", "\uC98F"], /* Hangul Syllable */
  ["\uC990", "\uC990"], /* Hangul Syllable */
  ["\uC991", "\uC991"], /* Hangul Syllable */
  ["\uC992", "\uC992"], /* Hangul Syllable */
  ["\uC993", "\uC993"], /* Hangul Syllable */
  ["\uC994", "\uC994"], /* Hangul Syllable */
  ["\uC995", "\uC995"], /* Hangul Syllable */
  ["\uC996", "\uC996"], /* Hangul Syllable */
  ["\uC997", "\uC997"], /* Hangul Syllable */
  ["\uC998", "\uC998"], /* Hangul Syllable */
  ["\uC999", "\uC999"], /* Hangul Syllable */
  ["\uC99A", "\uC99A"], /* Hangul Syllable */
  ["\uC99B", "\uC99B"], /* Hangul Syllable */
  ["\uC99C", "\uC99C"], /* Hangul Syllable */
  ["\uC99D", "\uC99D"], /* Hangul Syllable */
  ["\uC99E", "\uC99E"], /* Hangul Syllable */
  ["\uC99F", "\uC99F"], /* Hangul Syllable */
  ["\uC9A0", "\uC9A0"], /* Hangul Syllable */
  ["\uC9A1", "\uC9A1"], /* Hangul Syllable */
  ["\uC9A2", "\uC9A2"], /* Hangul Syllable */
  ["\uC9A3", "\uC9A3"], /* Hangul Syllable */
  ["\uC9A4", "\uC9A4"], /* Hangul Syllable */
  ["\uC9A5", "\uC9A5"], /* Hangul Syllable */
  ["\uC9A6", "\uC9A6"], /* Hangul Syllable */
  ["\uC9A7", "\uC9A7"], /* Hangul Syllable */
  ["\uC9A8", "\uC9A8"], /* Hangul Syllable */
  ["\uC9A9", "\uC9A9"], /* Hangul Syllable */
  ["\uC9AA", "\uC9AA"], /* Hangul Syllable */
  ["\uC9AB", "\uC9AB"], /* Hangul Syllable */
  ["\uC9AC", "\uC9AC"], /* Hangul Syllable */
  ["\uC9AD", "\uC9AD"], /* Hangul Syllable */
  ["\uC9AE", "\uC9AE"], /* Hangul Syllable */
  ["\uC9AF", "\uC9AF"], /* Hangul Syllable */
  ["\uC9B0", "\uC9B0"], /* Hangul Syllable */
  ["\uC9B1", "\uC9B1"], /* Hangul Syllable */
  ["\uC9B2", "\uC9B2"], /* Hangul Syllable */
  ["\uC9B3", "\uC9B3"], /* Hangul Syllable */
  ["\uC9B4", "\uC9B4"], /* Hangul Syllable */
  ["\uC9B5", "\uC9B5"], /* Hangul Syllable */
  ["\uC9B6", "\uC9B6"], /* Hangul Syllable */
  ["\uC9B7", "\uC9B7"], /* Hangul Syllable */
  ["\uC9B8", "\uC9B8"], /* Hangul Syllable */
  ["\uC9B9", "\uC9B9"], /* Hangul Syllable */
  ["\uC9BA", "\uC9BA"], /* Hangul Syllable */
  ["\uC9BB", "\uC9BB"], /* Hangul Syllable */
  ["\uC9BC", "\uC9BC"], /* Hangul Syllable */
  ["\uC9BD", "\uC9BD"], /* Hangul Syllable */
  ["\uC9BE", "\uC9BE"], /* Hangul Syllable */
  ["\uC9BF", "\uC9BF"], /* Hangul Syllable */
  ["\uC9C0", "\uC9C0"], /* Hangul Syllable */
  ["\uC9C1", "\uC9C1"], /* Hangul Syllable */
  ["\uC9C2", "\uC9C2"], /* Hangul Syllable */
  ["\uC9C3", "\uC9C3"], /* Hangul Syllable */
  ["\uC9C4", "\uC9C4"], /* Hangul Syllable */
  ["\uC9C5", "\uC9C5"], /* Hangul Syllable */
  ["\uC9C6", "\uC9C6"], /* Hangul Syllable */
  ["\uC9C7", "\uC9C7"], /* Hangul Syllable */
  ["\uC9C8", "\uC9C8"], /* Hangul Syllable */
  ["\uC9C9", "\uC9C9"], /* Hangul Syllable */
  ["\uC9CA", "\uC9CA"], /* Hangul Syllable */
  ["\uC9CB", "\uC9CB"], /* Hangul Syllable */
  ["\uC9CC", "\uC9CC"], /* Hangul Syllable */
  ["\uC9CD", "\uC9CD"], /* Hangul Syllable */
  ["\uC9CE", "\uC9CE"], /* Hangul Syllable */
  ["\uC9CF", "\uC9CF"], /* Hangul Syllable */
  ["\uC9D0", "\uC9D0"], /* Hangul Syllable */
  ["\uC9D1", "\uC9D1"], /* Hangul Syllable */
  ["\uC9D2", "\uC9D2"], /* Hangul Syllable */
  ["\uC9D3", "\uC9D3"], /* Hangul Syllable */
  ["\uC9D4", "\uC9D4"], /* Hangul Syllable */
  ["\uC9D5", "\uC9D5"], /* Hangul Syllable */
  ["\uC9D6", "\uC9D6"], /* Hangul Syllable */
  ["\uC9D7", "\uC9D7"], /* Hangul Syllable */
  ["\uC9D8", "\uC9D8"], /* Hangul Syllable */
  ["\uC9D9", "\uC9D9"], /* Hangul Syllable */
  ["\uC9DA", "\uC9DA"], /* Hangul Syllable */
  ["\uC9DB", "\uC9DB"], /* Hangul Syllable */
  ["\uC9DC", "\uC9DC"], /* Hangul Syllable */
  ["\uC9DD", "\uC9DD"], /* Hangul Syllable */
  ["\uC9DE", "\uC9DE"], /* Hangul Syllable */
  ["\uC9DF", "\uC9DF"], /* Hangul Syllable */
  ["\uC9E0", "\uC9E0"], /* Hangul Syllable */
  ["\uC9E1", "\uC9E1"], /* Hangul Syllable */
  ["\uC9E2", "\uC9E2"], /* Hangul Syllable */
  ["\uC9E3", "\uC9E3"], /* Hangul Syllable */
  ["\uC9E4", "\uC9E4"], /* Hangul Syllable */
  ["\uC9E5", "\uC9E5"], /* Hangul Syllable */
  ["\uC9E6", "\uC9E6"], /* Hangul Syllable */
  ["\uC9E7", "\uC9E7"], /* Hangul Syllable */
  ["\uC9E8", "\uC9E8"], /* Hangul Syllable */
  ["\uC9E9", "\uC9E9"], /* Hangul Syllable */
  ["\uC9EA", "\uC9EA"], /* Hangul Syllable */
  ["\uC9EB", "\uC9EB"], /* Hangul Syllable */
  ["\uC9EC", "\uC9EC"], /* Hangul Syllable */
  ["\uC9ED", "\uC9ED"], /* Hangul Syllable */
  ["\uC9EE", "\uC9EE"], /* Hangul Syllable */
  ["\uC9EF", "\uC9EF"], /* Hangul Syllable */
  ["\uC9F0", "\uC9F0"], /* Hangul Syllable */
  ["\uC9F1", "\uC9F1"], /* Hangul Syllable */
  ["\uC9F2", "\uC9F2"], /* Hangul Syllable */
  ["\uC9F3", "\uC9F3"], /* Hangul Syllable */
  ["\uC9F4", "\uC9F4"], /* Hangul Syllable */
  ["\uC9F5", "\uC9F5"], /* Hangul Syllable */
  ["\uC9F6", "\uC9F6"], /* Hangul Syllable */
  ["\uC9F7", "\uC9F7"], /* Hangul Syllable */
  ["\uC9F8", "\uC9F8"], /* Hangul Syllable */
  ["\uC9F9", "\uC9F9"], /* Hangul Syllable */
  ["\uC9FA", "\uC9FA"], /* Hangul Syllable */
  ["\uC9FB", "\uC9FB"], /* Hangul Syllable */
  ["\uC9FC", "\uC9FC"], /* Hangul Syllable */
  ["\uC9FD", "\uC9FD"], /* Hangul Syllable */
  ["\uC9FE", "\uC9FE"], /* Hangul Syllable */
  ["\uC9FF", "\uC9FF"], /* Hangul Syllable */
  ["\uCA00", "\uCA00"], /* Hangul Syllable */
  ["\uCA01", "\uCA01"], /* Hangul Syllable */
  ["\uCA02", "\uCA02"], /* Hangul Syllable */
  ["\uCA03", "\uCA03"], /* Hangul Syllable */
  ["\uCA04", "\uCA04"], /* Hangul Syllable */
  ["\uCA05", "\uCA05"], /* Hangul Syllable */
  ["\uCA06", "\uCA06"], /* Hangul Syllable */
  ["\uCA07", "\uCA07"], /* Hangul Syllable */
  ["\uCA08", "\uCA08"], /* Hangul Syllable */
  ["\uCA09", "\uCA09"], /* Hangul Syllable */
  ["\uCA0A", "\uCA0A"], /* Hangul Syllable */
  ["\uCA0B", "\uCA0B"], /* Hangul Syllable */
  ["\uCA0C", "\uCA0C"], /* Hangul Syllable */
  ["\uCA0D", "\uCA0D"], /* Hangul Syllable */
  ["\uCA0E", "\uCA0E"], /* Hangul Syllable */
  ["\uCA0F", "\uCA0F"], /* Hangul Syllable */
  ["\uCA10", "\uCA10"], /* Hangul Syllable */
  ["\uCA11", "\uCA11"], /* Hangul Syllable */
  ["\uCA12", "\uCA12"], /* Hangul Syllable */
  ["\uCA13", "\uCA13"], /* Hangul Syllable */
  ["\uCA14", "\uCA14"], /* Hangul Syllable */
  ["\uCA15", "\uCA15"], /* Hangul Syllable */
  ["\uCA16", "\uCA16"], /* Hangul Syllable */
  ["\uCA17", "\uCA17"], /* Hangul Syllable */
  ["\uCA18", "\uCA18"], /* Hangul Syllable */
  ["\uCA19", "\uCA19"], /* Hangul Syllable */
  ["\uCA1A", "\uCA1A"], /* Hangul Syllable */
  ["\uCA1B", "\uCA1B"], /* Hangul Syllable */
  ["\uCA1C", "\uCA1C"], /* Hangul Syllable */
  ["\uCA1D", "\uCA1D"], /* Hangul Syllable */
  ["\uCA1E", "\uCA1E"], /* Hangul Syllable */
  ["\uCA1F", "\uCA1F"], /* Hangul Syllable */
  ["\uCA20", "\uCA20"], /* Hangul Syllable */
  ["\uCA21", "\uCA21"], /* Hangul Syllable */
  ["\uCA22", "\uCA22"], /* Hangul Syllable */
  ["\uCA23", "\uCA23"], /* Hangul Syllable */
  ["\uCA24", "\uCA24"], /* Hangul Syllable */
  ["\uCA25", "\uCA25"], /* Hangul Syllable */
  ["\uCA26", "\uCA26"], /* Hangul Syllable */
  ["\uCA27", "\uCA27"], /* Hangul Syllable */
  ["\uCA28", "\uCA28"], /* Hangul Syllable */
  ["\uCA29", "\uCA29"], /* Hangul Syllable */
  ["\uCA2A", "\uCA2A"], /* Hangul Syllable */
  ["\uCA2B", "\uCA2B"], /* Hangul Syllable */
  ["\uCA2C", "\uCA2C"], /* Hangul Syllable */
  ["\uCA2D", "\uCA2D"], /* Hangul Syllable */
  ["\uCA2E", "\uCA2E"], /* Hangul Syllable */
  ["\uCA2F", "\uCA2F"], /* Hangul Syllable */
  ["\uCA30", "\uCA30"], /* Hangul Syllable */
  ["\uCA31", "\uCA31"], /* Hangul Syllable */
  ["\uCA32", "\uCA32"], /* Hangul Syllable */
  ["\uCA33", "\uCA33"], /* Hangul Syllable */
  ["\uCA34", "\uCA34"], /* Hangul Syllable */
  ["\uCA35", "\uCA35"], /* Hangul Syllable */
  ["\uCA36", "\uCA36"], /* Hangul Syllable */
  ["\uCA37", "\uCA37"], /* Hangul Syllable */
  ["\uCA38", "\uCA38"], /* Hangul Syllable */
  ["\uCA39", "\uCA39"], /* Hangul Syllable */
  ["\uCA3A", "\uCA3A"], /* Hangul Syllable */
  ["\uCA3B", "\uCA3B"], /* Hangul Syllable */
  ["\uCA3C", "\uCA3C"], /* Hangul Syllable */
  ["\uCA3D", "\uCA3D"], /* Hangul Syllable */
  ["\uCA3E", "\uCA3E"], /* Hangul Syllable */
  ["\uCA3F", "\uCA3F"], /* Hangul Syllable */
  ["\uCA40", "\uCA40"], /* Hangul Syllable */
  ["\uCA41", "\uCA41"], /* Hangul Syllable */
  ["\uCA42", "\uCA42"], /* Hangul Syllable */
  ["\uCA43", "\uCA43"], /* Hangul Syllable */
  ["\uCA44", "\uCA44"], /* Hangul Syllable */
  ["\uCA45", "\uCA45"], /* Hangul Syllable */
  ["\uCA46", "\uCA46"], /* Hangul Syllable */
  ["\uCA47", "\uCA47"], /* Hangul Syllable */
  ["\uCA48", "\uCA48"], /* Hangul Syllable */
  ["\uCA49", "\uCA49"], /* Hangul Syllable */
  ["\uCA4A", "\uCA4A"], /* Hangul Syllable */
  ["\uCA4B", "\uCA4B"], /* Hangul Syllable */
  ["\uCA4C", "\uCA4C"], /* Hangul Syllable */
  ["\uCA4D", "\uCA4D"], /* Hangul Syllable */
  ["\uCA4E", "\uCA4E"], /* Hangul Syllable */
  ["\uCA4F", "\uCA4F"], /* Hangul Syllable */
  ["\uCA50", "\uCA50"], /* Hangul Syllable */
  ["\uCA51", "\uCA51"], /* Hangul Syllable */
  ["\uCA52", "\uCA52"], /* Hangul Syllable */
  ["\uCA53", "\uCA53"], /* Hangul Syllable */
  ["\uCA54", "\uCA54"], /* Hangul Syllable */
  ["\uCA55", "\uCA55"], /* Hangul Syllable */
  ["\uCA56", "\uCA56"], /* Hangul Syllable */
  ["\uCA57", "\uCA57"], /* Hangul Syllable */
  ["\uCA58", "\uCA58"], /* Hangul Syllable */
  ["\uCA59", "\uCA59"], /* Hangul Syllable */
  ["\uCA5A", "\uCA5A"], /* Hangul Syllable */
  ["\uCA5B", "\uCA5B"], /* Hangul Syllable */
  ["\uCA5C", "\uCA5C"], /* Hangul Syllable */
  ["\uCA5D", "\uCA5D"], /* Hangul Syllable */
  ["\uCA5E", "\uCA5E"], /* Hangul Syllable */
  ["\uCA5F", "\uCA5F"], /* Hangul Syllable */
  ["\uCA60", "\uCA60"], /* Hangul Syllable */
  ["\uCA61", "\uCA61"], /* Hangul Syllable */
  ["\uCA62", "\uCA62"], /* Hangul Syllable */
  ["\uCA63", "\uCA63"], /* Hangul Syllable */
  ["\uCA64", "\uCA64"], /* Hangul Syllable */
  ["\uCA65", "\uCA65"], /* Hangul Syllable */
  ["\uCA66", "\uCA66"], /* Hangul Syllable */
  ["\uCA67", "\uCA67"], /* Hangul Syllable */
  ["\uCA68", "\uCA68"], /* Hangul Syllable */
  ["\uCA69", "\uCA69"], /* Hangul Syllable */
  ["\uCA6A", "\uCA6A"], /* Hangul Syllable */
  ["\uCA6B", "\uCA6B"], /* Hangul Syllable */
  ["\uCA6C", "\uCA6C"], /* Hangul Syllable */
  ["\uCA6D", "\uCA6D"], /* Hangul Syllable */
  ["\uCA6E", "\uCA6E"], /* Hangul Syllable */
  ["\uCA6F", "\uCA6F"], /* Hangul Syllable */
  ["\uCA70", "\uCA70"], /* Hangul Syllable */
  ["\uCA71", "\uCA71"], /* Hangul Syllable */
  ["\uCA72", "\uCA72"], /* Hangul Syllable */
  ["\uCA73", "\uCA73"], /* Hangul Syllable */
  ["\uCA74", "\uCA74"], /* Hangul Syllable */
  ["\uCA75", "\uCA75"], /* Hangul Syllable */
  ["\uCA76", "\uCA76"], /* Hangul Syllable */
  ["\uCA77", "\uCA77"], /* Hangul Syllable */
  ["\uCA78", "\uCA78"], /* Hangul Syllable */
  ["\uCA79", "\uCA79"], /* Hangul Syllable */
  ["\uCA7A", "\uCA7A"], /* Hangul Syllable */
  ["\uCA7B", "\uCA7B"], /* Hangul Syllable */
  ["\uCA7C", "\uCA7C"], /* Hangul Syllable */
  ["\uCA7D", "\uCA7D"], /* Hangul Syllable */
  ["\uCA7E", "\uCA7E"], /* Hangul Syllable */
  ["\uCA7F", "\uCA7F"], /* Hangul Syllable */
  ["\uCA80", "\uCA80"], /* Hangul Syllable */
  ["\uCA81", "\uCA81"], /* Hangul Syllable */
  ["\uCA82", "\uCA82"], /* Hangul Syllable */
  ["\uCA83", "\uCA83"], /* Hangul Syllable */
  ["\uCA84", "\uCA84"], /* Hangul Syllable */
  ["\uCA85", "\uCA85"], /* Hangul Syllable */
  ["\uCA86", "\uCA86"], /* Hangul Syllable */
  ["\uCA87", "\uCA87"], /* Hangul Syllable */
  ["\uCA88", "\uCA88"], /* Hangul Syllable */
  ["\uCA89", "\uCA89"], /* Hangul Syllable */
  ["\uCA8A", "\uCA8A"], /* Hangul Syllable */
  ["\uCA8B", "\uCA8B"], /* Hangul Syllable */
  ["\uCA8C", "\uCA8C"], /* Hangul Syllable */
  ["\uCA8D", "\uCA8D"], /* Hangul Syllable */
  ["\uCA8E", "\uCA8E"], /* Hangul Syllable */
  ["\uCA8F", "\uCA8F"], /* Hangul Syllable */
  ["\uCA90", "\uCA90"], /* Hangul Syllable */
  ["\uCA91", "\uCA91"], /* Hangul Syllable */
  ["\uCA92", "\uCA92"], /* Hangul Syllable */
  ["\uCA93", "\uCA93"], /* Hangul Syllable */
  ["\uCA94", "\uCA94"], /* Hangul Syllable */
  ["\uCA95", "\uCA95"], /* Hangul Syllable */
  ["\uCA96", "\uCA96"], /* Hangul Syllable */
  ["\uCA97", "\uCA97"], /* Hangul Syllable */
  ["\uCA98", "\uCA98"], /* Hangul Syllable */
  ["\uCA99", "\uCA99"], /* Hangul Syllable */
  ["\uCA9A", "\uCA9A"], /* Hangul Syllable */
  ["\uCA9B", "\uCA9B"], /* Hangul Syllable */
  ["\uCA9C", "\uCA9C"], /* Hangul Syllable */
  ["\uCA9D", "\uCA9D"], /* Hangul Syllable */
  ["\uCA9E", "\uCA9E"], /* Hangul Syllable */
  ["\uCA9F", "\uCA9F"], /* Hangul Syllable */
  ["\uCAA0", "\uCAA0"], /* Hangul Syllable */
  ["\uCAA1", "\uCAA1"], /* Hangul Syllable */
  ["\uCAA2", "\uCAA2"], /* Hangul Syllable */
  ["\uCAA3", "\uCAA3"], /* Hangul Syllable */
  ["\uCAA4", "\uCAA4"], /* Hangul Syllable */
  ["\uCAA5", "\uCAA5"], /* Hangul Syllable */
  ["\uCAA6", "\uCAA6"], /* Hangul Syllable */
  ["\uCAA7", "\uCAA7"], /* Hangul Syllable */
  ["\uCAA8", "\uCAA8"], /* Hangul Syllable */
  ["\uCAA9", "\uCAA9"], /* Hangul Syllable */
  ["\uCAAA", "\uCAAA"], /* Hangul Syllable */
  ["\uCAAB", "\uCAAB"], /* Hangul Syllable */
  ["\uCAAC", "\uCAAC"], /* Hangul Syllable */
  ["\uCAAD", "\uCAAD"], /* Hangul Syllable */
  ["\uCAAE", "\uCAAE"], /* Hangul Syllable */
  ["\uCAAF", "\uCAAF"], /* Hangul Syllable */
  ["\uCAB0", "\uCAB0"], /* Hangul Syllable */
  ["\uCAB1", "\uCAB1"], /* Hangul Syllable */
  ["\uCAB2", "\uCAB2"], /* Hangul Syllable */
  ["\uCAB3", "\uCAB3"], /* Hangul Syllable */
  ["\uCAB4", "\uCAB4"], /* Hangul Syllable */
  ["\uCAB5", "\uCAB5"], /* Hangul Syllable */
  ["\uCAB6", "\uCAB6"], /* Hangul Syllable */
  ["\uCAB7", "\uCAB7"], /* Hangul Syllable */
  ["\uCAB8", "\uCAB8"], /* Hangul Syllable */
  ["\uCAB9", "\uCAB9"], /* Hangul Syllable */
  ["\uCABA", "\uCABA"], /* Hangul Syllable */
  ["\uCABB", "\uCABB"], /* Hangul Syllable */
  ["\uCABC", "\uCABC"], /* Hangul Syllable */
  ["\uCABD", "\uCABD"], /* Hangul Syllable */
  ["\uCABE", "\uCABE"], /* Hangul Syllable */
  ["\uCABF", "\uCABF"], /* Hangul Syllable */
  ["\uCAC0", "\uCAC0"], /* Hangul Syllable */
  ["\uCAC1", "\uCAC1"], /* Hangul Syllable */
  ["\uCAC2", "\uCAC2"], /* Hangul Syllable */
  ["\uCAC3", "\uCAC3"], /* Hangul Syllable */
  ["\uCAC4", "\uCAC4"], /* Hangul Syllable */
  ["\uCAC5", "\uCAC5"], /* Hangul Syllable */
  ["\uCAC6", "\uCAC6"], /* Hangul Syllable */
  ["\uCAC7", "\uCAC7"], /* Hangul Syllable */
  ["\uCAC8", "\uCAC8"], /* Hangul Syllable */
  ["\uCAC9", "\uCAC9"], /* Hangul Syllable */
  ["\uCACA", "\uCACA"], /* Hangul Syllable */
  ["\uCACB", "\uCACB"], /* Hangul Syllable */
  ["\uCACC", "\uCACC"], /* Hangul Syllable */
  ["\uCACD", "\uCACD"], /* Hangul Syllable */
  ["\uCACE", "\uCACE"], /* Hangul Syllable */
  ["\uCACF", "\uCACF"], /* Hangul Syllable */
  ["\uCAD0", "\uCAD0"], /* Hangul Syllable */
  ["\uCAD1", "\uCAD1"], /* Hangul Syllable */
  ["\uCAD2", "\uCAD2"], /* Hangul Syllable */
  ["\uCAD3", "\uCAD3"], /* Hangul Syllable */
  ["\uCAD4", "\uCAD4"], /* Hangul Syllable */
  ["\uCAD5", "\uCAD5"], /* Hangul Syllable */
  ["\uCAD6", "\uCAD6"], /* Hangul Syllable */
  ["\uCAD7", "\uCAD7"], /* Hangul Syllable */
  ["\uCAD8", "\uCAD8"], /* Hangul Syllable */
  ["\uCAD9", "\uCAD9"], /* Hangul Syllable */
  ["\uCADA", "\uCADA"], /* Hangul Syllable */
  ["\uCADB", "\uCADB"], /* Hangul Syllable */
  ["\uCADC", "\uCADC"], /* Hangul Syllable */
  ["\uCADD", "\uCADD"], /* Hangul Syllable */
  ["\uCADE", "\uCADE"], /* Hangul Syllable */
  ["\uCADF", "\uCADF"], /* Hangul Syllable */
  ["\uCAE0", "\uCAE0"], /* Hangul Syllable */
  ["\uCAE1", "\uCAE1"], /* Hangul Syllable */
  ["\uCAE2", "\uCAE2"], /* Hangul Syllable */
  ["\uCAE3", "\uCAE3"], /* Hangul Syllable */
  ["\uCAE4", "\uCAE4"], /* Hangul Syllable */
  ["\uCAE5", "\uCAE5"], /* Hangul Syllable */
  ["\uCAE6", "\uCAE6"], /* Hangul Syllable */
  ["\uCAE7", "\uCAE7"], /* Hangul Syllable */
  ["\uCAE8", "\uCAE8"], /* Hangul Syllable */
  ["\uCAE9", "\uCAE9"], /* Hangul Syllable */
  ["\uCAEA", "\uCAEA"], /* Hangul Syllable */
  ["\uCAEB", "\uCAEB"], /* Hangul Syllable */
  ["\uCAEC", "\uCAEC"], /* Hangul Syllable */
  ["\uCAED", "\uCAED"], /* Hangul Syllable */
  ["\uCAEE", "\uCAEE"], /* Hangul Syllable */
  ["\uCAEF", "\uCAEF"], /* Hangul Syllable */
  ["\uCAF0", "\uCAF0"], /* Hangul Syllable */
  ["\uCAF1", "\uCAF1"], /* Hangul Syllable */
  ["\uCAF2", "\uCAF2"], /* Hangul Syllable */
  ["\uCAF3", "\uCAF3"], /* Hangul Syllable */
  ["\uCAF4", "\uCAF4"], /* Hangul Syllable */
  ["\uCAF5", "\uCAF5"], /* Hangul Syllable */
  ["\uCAF6", "\uCAF6"], /* Hangul Syllable */
  ["\uCAF7", "\uCAF7"], /* Hangul Syllable */
  ["\uCAF8", "\uCAF8"], /* Hangul Syllable */
  ["\uCAF9", "\uCAF9"], /* Hangul Syllable */
  ["\uCAFA", "\uCAFA"], /* Hangul Syllable */
  ["\uCAFB", "\uCAFB"], /* Hangul Syllable */
  ["\uCAFC", "\uCAFC"], /* Hangul Syllable */
  ["\uCAFD", "\uCAFD"], /* Hangul Syllable */
  ["\uCAFE", "\uCAFE"], /* Hangul Syllable */
  ["\uCAFF", "\uCAFF"], /* Hangul Syllable */
  ["\uCB00", "\uCB00"], /* Hangul Syllable */
  ["\uCB01", "\uCB01"], /* Hangul Syllable */
  ["\uCB02", "\uCB02"], /* Hangul Syllable */
  ["\uCB03", "\uCB03"], /* Hangul Syllable */
  ["\uCB04", "\uCB04"], /* Hangul Syllable */
  ["\uCB05", "\uCB05"], /* Hangul Syllable */
  ["\uCB06", "\uCB06"], /* Hangul Syllable */
  ["\uCB07", "\uCB07"], /* Hangul Syllable */
  ["\uCB08", "\uCB08"], /* Hangul Syllable */
  ["\uCB09", "\uCB09"], /* Hangul Syllable */
  ["\uCB0A", "\uCB0A"], /* Hangul Syllable */
  ["\uCB0B", "\uCB0B"], /* Hangul Syllable */
  ["\uCB0C", "\uCB0C"], /* Hangul Syllable */
  ["\uCB0D", "\uCB0D"], /* Hangul Syllable */
  ["\uCB0E", "\uCB0E"], /* Hangul Syllable */
  ["\uCB0F", "\uCB0F"], /* Hangul Syllable */
  ["\uCB10", "\uCB10"], /* Hangul Syllable */
  ["\uCB11", "\uCB11"], /* Hangul Syllable */
  ["\uCB12", "\uCB12"], /* Hangul Syllable */
  ["\uCB13", "\uCB13"], /* Hangul Syllable */
  ["\uCB14", "\uCB14"], /* Hangul Syllable */
  ["\uCB15", "\uCB15"], /* Hangul Syllable */
  ["\uCB16", "\uCB16"], /* Hangul Syllable */
  ["\uCB17", "\uCB17"], /* Hangul Syllable */
  ["\uCB18", "\uCB18"], /* Hangul Syllable */
  ["\uCB19", "\uCB19"], /* Hangul Syllable */
  ["\uCB1A", "\uCB1A"], /* Hangul Syllable */
  ["\uCB1B", "\uCB1B"], /* Hangul Syllable */
  ["\uCB1C", "\uCB1C"], /* Hangul Syllable */
  ["\uCB1D", "\uCB1D"], /* Hangul Syllable */
  ["\uCB1E", "\uCB1E"], /* Hangul Syllable */
  ["\uCB1F", "\uCB1F"], /* Hangul Syllable */
  ["\uCB20", "\uCB20"], /* Hangul Syllable */
  ["\uCB21", "\uCB21"], /* Hangul Syllable */
  ["\uCB22", "\uCB22"], /* Hangul Syllable */
  ["\uCB23", "\uCB23"], /* Hangul Syllable */
  ["\uCB24", "\uCB24"], /* Hangul Syllable */
  ["\uCB25", "\uCB25"], /* Hangul Syllable */
  ["\uCB26", "\uCB26"], /* Hangul Syllable */
  ["\uCB27", "\uCB27"], /* Hangul Syllable */
  ["\uCB28", "\uCB28"], /* Hangul Syllable */
  ["\uCB29", "\uCB29"], /* Hangul Syllable */
  ["\uCB2A", "\uCB2A"], /* Hangul Syllable */
  ["\uCB2B", "\uCB2B"], /* Hangul Syllable */
  ["\uCB2C", "\uCB2C"], /* Hangul Syllable */
  ["\uCB2D", "\uCB2D"], /* Hangul Syllable */
  ["\uCB2E", "\uCB2E"], /* Hangul Syllable */
  ["\uCB2F", "\uCB2F"], /* Hangul Syllable */
  ["\uCB30", "\uCB30"], /* Hangul Syllable */
  ["\uCB31", "\uCB31"], /* Hangul Syllable */
  ["\uCB32", "\uCB32"], /* Hangul Syllable */
  ["\uCB33", "\uCB33"], /* Hangul Syllable */
  ["\uCB34", "\uCB34"], /* Hangul Syllable */
  ["\uCB35", "\uCB35"], /* Hangul Syllable */
  ["\uCB36", "\uCB36"], /* Hangul Syllable */
  ["\uCB37", "\uCB37"], /* Hangul Syllable */
  ["\uCB38", "\uCB38"], /* Hangul Syllable */
  ["\uCB39", "\uCB39"], /* Hangul Syllable */
  ["\uCB3A", "\uCB3A"], /* Hangul Syllable */
  ["\uCB3B", "\uCB3B"], /* Hangul Syllable */
  ["\uCB3C", "\uCB3C"], /* Hangul Syllable */
  ["\uCB3D", "\uCB3D"], /* Hangul Syllable */
  ["\uCB3E", "\uCB3E"], /* Hangul Syllable */
  ["\uCB3F", "\uCB3F"], /* Hangul Syllable */
  ["\uCB40", "\uCB40"], /* Hangul Syllable */
  ["\uCB41", "\uCB41"], /* Hangul Syllable */
  ["\uCB42", "\uCB42"], /* Hangul Syllable */
  ["\uCB43", "\uCB43"], /* Hangul Syllable */
  ["\uCB44", "\uCB44"], /* Hangul Syllable */
  ["\uCB45", "\uCB45"], /* Hangul Syllable */
  ["\uCB46", "\uCB46"], /* Hangul Syllable */
  ["\uCB47", "\uCB47"], /* Hangul Syllable */
  ["\uCB48", "\uCB48"], /* Hangul Syllable */
  ["\uCB49", "\uCB49"], /* Hangul Syllable */
  ["\uCB4A", "\uCB4A"], /* Hangul Syllable */
  ["\uCB4B", "\uCB4B"], /* Hangul Syllable */
  ["\uCB4C", "\uCB4C"], /* Hangul Syllable */
  ["\uCB4D", "\uCB4D"], /* Hangul Syllable */
  ["\uCB4E", "\uCB4E"], /* Hangul Syllable */
  ["\uCB4F", "\uCB4F"], /* Hangul Syllable */
  ["\uCB50", "\uCB50"], /* Hangul Syllable */
  ["\uCB51", "\uCB51"], /* Hangul Syllable */
  ["\uCB52", "\uCB52"], /* Hangul Syllable */
  ["\uCB53", "\uCB53"], /* Hangul Syllable */
  ["\uCB54", "\uCB54"], /* Hangul Syllable */
  ["\uCB55", "\uCB55"], /* Hangul Syllable */
  ["\uCB56", "\uCB56"], /* Hangul Syllable */
  ["\uCB57", "\uCB57"], /* Hangul Syllable */
  ["\uCB58", "\uCB58"], /* Hangul Syllable */
  ["\uCB59", "\uCB59"], /* Hangul Syllable */
  ["\uCB5A", "\uCB5A"], /* Hangul Syllable */
  ["\uCB5B", "\uCB5B"], /* Hangul Syllable */
  ["\uCB5C", "\uCB5C"], /* Hangul Syllable */
  ["\uCB5D", "\uCB5D"], /* Hangul Syllable */
  ["\uCB5E", "\uCB5E"], /* Hangul Syllable */
  ["\uCB5F", "\uCB5F"], /* Hangul Syllable */
  ["\uCB60", "\uCB60"], /* Hangul Syllable */
  ["\uCB61", "\uCB61"], /* Hangul Syllable */
  ["\uCB62", "\uCB62"], /* Hangul Syllable */
  ["\uCB63", "\uCB63"], /* Hangul Syllable */
  ["\uCB64", "\uCB64"], /* Hangul Syllable */
  ["\uCB65", "\uCB65"], /* Hangul Syllable */
  ["\uCB66", "\uCB66"], /* Hangul Syllable */
  ["\uCB67", "\uCB67"], /* Hangul Syllable */
  ["\uCB68", "\uCB68"], /* Hangul Syllable */
  ["\uCB69", "\uCB69"], /* Hangul Syllable */
  ["\uCB6A", "\uCB6A"], /* Hangul Syllable */
  ["\uCB6B", "\uCB6B"], /* Hangul Syllable */
  ["\uCB6C", "\uCB6C"], /* Hangul Syllable */
  ["\uCB6D", "\uCB6D"], /* Hangul Syllable */
  ["\uCB6E", "\uCB6E"], /* Hangul Syllable */
  ["\uCB6F", "\uCB6F"], /* Hangul Syllable */
  ["\uCB70", "\uCB70"], /* Hangul Syllable */
  ["\uCB71", "\uCB71"], /* Hangul Syllable */
  ["\uCB72", "\uCB72"], /* Hangul Syllable */
  ["\uCB73", "\uCB73"], /* Hangul Syllable */
  ["\uCB74", "\uCB74"], /* Hangul Syllable */
  ["\uCB75", "\uCB75"], /* Hangul Syllable */
  ["\uCB76", "\uCB76"], /* Hangul Syllable */
  ["\uCB77", "\uCB77"], /* Hangul Syllable */
  ["\uCB78", "\uCB78"], /* Hangul Syllable */
  ["\uCB79", "\uCB79"], /* Hangul Syllable */
  ["\uCB7A", "\uCB7A"], /* Hangul Syllable */
  ["\uCB7B", "\uCB7B"], /* Hangul Syllable */
  ["\uCB7C", "\uCB7C"], /* Hangul Syllable */
  ["\uCB7D", "\uCB7D"], /* Hangul Syllable */
  ["\uCB7E", "\uCB7E"], /* Hangul Syllable */
  ["\uCB7F", "\uCB7F"], /* Hangul Syllable */
  ["\uCB80", "\uCB80"], /* Hangul Syllable */
  ["\uCB81", "\uCB81"], /* Hangul Syllable */
  ["\uCB82", "\uCB82"], /* Hangul Syllable */
  ["\uCB83", "\uCB83"], /* Hangul Syllable */
  ["\uCB84", "\uCB84"], /* Hangul Syllable */
  ["\uCB85", "\uCB85"], /* Hangul Syllable */
  ["\uCB86", "\uCB86"], /* Hangul Syllable */
  ["\uCB87", "\uCB87"], /* Hangul Syllable */
  ["\uCB88", "\uCB88"], /* Hangul Syllable */
  ["\uCB89", "\uCB89"], /* Hangul Syllable */
  ["\uCB8A", "\uCB8A"], /* Hangul Syllable */
  ["\uCB8B", "\uCB8B"], /* Hangul Syllable */
  ["\uCB8C", "\uCB8C"], /* Hangul Syllable */
  ["\uCB8D", "\uCB8D"], /* Hangul Syllable */
  ["\uCB8E", "\uCB8E"], /* Hangul Syllable */
  ["\uCB8F", "\uCB8F"], /* Hangul Syllable */
  ["\uCB90", "\uCB90"], /* Hangul Syllable */
  ["\uCB91", "\uCB91"], /* Hangul Syllable */
  ["\uCB92", "\uCB92"], /* Hangul Syllable */
  ["\uCB93", "\uCB93"], /* Hangul Syllable */
  ["\uCB94", "\uCB94"], /* Hangul Syllable */
  ["\uCB95", "\uCB95"], /* Hangul Syllable */
  ["\uCB96", "\uCB96"], /* Hangul Syllable */
  ["\uCB97", "\uCB97"], /* Hangul Syllable */
  ["\uCB98", "\uCB98"], /* Hangul Syllable */
  ["\uCB99", "\uCB99"], /* Hangul Syllable */
  ["\uCB9A", "\uCB9A"], /* Hangul Syllable */
  ["\uCB9B", "\uCB9B"], /* Hangul Syllable */
  ["\uCB9C", "\uCB9C"], /* Hangul Syllable */
  ["\uCB9D", "\uCB9D"], /* Hangul Syllable */
  ["\uCB9E", "\uCB9E"], /* Hangul Syllable */
  ["\uCB9F", "\uCB9F"], /* Hangul Syllable */
  ["\uCBA0", "\uCBA0"], /* Hangul Syllable */
  ["\uCBA1", "\uCBA1"], /* Hangul Syllable */
  ["\uCBA2", "\uCBA2"], /* Hangul Syllable */
  ["\uCBA3", "\uCBA3"], /* Hangul Syllable */
  ["\uCBA4", "\uCBA4"], /* Hangul Syllable */
  ["\uCBA5", "\uCBA5"], /* Hangul Syllable */
  ["\uCBA6", "\uCBA6"], /* Hangul Syllable */
  ["\uCBA7", "\uCBA7"], /* Hangul Syllable */
  ["\uCBA8", "\uCBA8"], /* Hangul Syllable */
  ["\uCBA9", "\uCBA9"], /* Hangul Syllable */
  ["\uCBAA", "\uCBAA"], /* Hangul Syllable */
  ["\uCBAB", "\uCBAB"], /* Hangul Syllable */
  ["\uCBAC", "\uCBAC"], /* Hangul Syllable */
  ["\uCBAD", "\uCBAD"], /* Hangul Syllable */
  ["\uCBAE", "\uCBAE"], /* Hangul Syllable */
  ["\uCBAF", "\uCBAF"], /* Hangul Syllable */
  ["\uCBB0", "\uCBB0"], /* Hangul Syllable */
  ["\uCBB1", "\uCBB1"], /* Hangul Syllable */
  ["\uCBB2", "\uCBB2"], /* Hangul Syllable */
  ["\uCBB3", "\uCBB3"], /* Hangul Syllable */
  ["\uCBB4", "\uCBB4"], /* Hangul Syllable */
  ["\uCBB5", "\uCBB5"], /* Hangul Syllable */
  ["\uCBB6", "\uCBB6"], /* Hangul Syllable */
  ["\uCBB7", "\uCBB7"], /* Hangul Syllable */
  ["\uCBB8", "\uCBB8"], /* Hangul Syllable */
  ["\uCBB9", "\uCBB9"], /* Hangul Syllable */
  ["\uCBBA", "\uCBBA"], /* Hangul Syllable */
  ["\uCBBB", "\uCBBB"], /* Hangul Syllable */
  ["\uCBBC", "\uCBBC"], /* Hangul Syllable */
  ["\uCBBD", "\uCBBD"], /* Hangul Syllable */
  ["\uCBBE", "\uCBBE"], /* Hangul Syllable */
  ["\uCBBF", "\uCBBF"], /* Hangul Syllable */
  ["\uCBC0", "\uCBC0"], /* Hangul Syllable */
  ["\uCBC1", "\uCBC1"], /* Hangul Syllable */
  ["\uCBC2", "\uCBC2"], /* Hangul Syllable */
  ["\uCBC3", "\uCBC3"], /* Hangul Syllable */
  ["\uCBC4", "\uCBC4"], /* Hangul Syllable */
  ["\uCBC5", "\uCBC5"], /* Hangul Syllable */
  ["\uCBC6", "\uCBC6"], /* Hangul Syllable */
  ["\uCBC7", "\uCBC7"], /* Hangul Syllable */
  ["\uCBC8", "\uCBC8"], /* Hangul Syllable */
  ["\uCBC9", "\uCBC9"], /* Hangul Syllable */
  ["\uCBCA", "\uCBCA"], /* Hangul Syllable */
  ["\uCBCB", "\uCBCB"], /* Hangul Syllable */
  ["\uCBCC", "\uCBCC"], /* Hangul Syllable */
  ["\uCBCD", "\uCBCD"], /* Hangul Syllable */
  ["\uCBCE", "\uCBCE"], /* Hangul Syllable */
  ["\uCBCF", "\uCBCF"], /* Hangul Syllable */
  ["\uCBD0", "\uCBD0"], /* Hangul Syllable */
  ["\uCBD1", "\uCBD1"], /* Hangul Syllable */
  ["\uCBD2", "\uCBD2"], /* Hangul Syllable */
  ["\uCBD3", "\uCBD3"], /* Hangul Syllable */
  ["\uCBD4", "\uCBD4"], /* Hangul Syllable */
  ["\uCBD5", "\uCBD5"], /* Hangul Syllable */
  ["\uCBD6", "\uCBD6"], /* Hangul Syllable */
  ["\uCBD7", "\uCBD7"], /* Hangul Syllable */
  ["\uCBD8", "\uCBD8"], /* Hangul Syllable */
  ["\uCBD9", "\uCBD9"], /* Hangul Syllable */
  ["\uCBDA", "\uCBDA"], /* Hangul Syllable */
  ["\uCBDB", "\uCBDB"], /* Hangul Syllable */
  ["\uCBDC", "\uCBDC"], /* Hangul Syllable */
  ["\uCBDD", "\uCBDD"], /* Hangul Syllable */
  ["\uCBDE", "\uCBDE"], /* Hangul Syllable */
  ["\uCBDF", "\uCBDF"], /* Hangul Syllable */
  ["\uCBE0", "\uCBE0"], /* Hangul Syllable */
  ["\uCBE1", "\uCBE1"], /* Hangul Syllable */
  ["\uCBE2", "\uCBE2"], /* Hangul Syllable */
  ["\uCBE3", "\uCBE3"], /* Hangul Syllable */
  ["\uCBE4", "\uCBE4"], /* Hangul Syllable */
  ["\uCBE5", "\uCBE5"], /* Hangul Syllable */
  ["\uCBE6", "\uCBE6"], /* Hangul Syllable */
  ["\uCBE7", "\uCBE7"], /* Hangul Syllable */
  ["\uCBE8", "\uCBE8"], /* Hangul Syllable */
  ["\uCBE9", "\uCBE9"], /* Hangul Syllable */
  ["\uCBEA", "\uCBEA"], /* Hangul Syllable */
  ["\uCBEB", "\uCBEB"], /* Hangul Syllable */
  ["\uCBEC", "\uCBEC"], /* Hangul Syllable */
  ["\uCBED", "\uCBED"], /* Hangul Syllable */
  ["\uCBEE", "\uCBEE"], /* Hangul Syllable */
  ["\uCBEF", "\uCBEF"], /* Hangul Syllable */
  ["\uCBF0", "\uCBF0"], /* Hangul Syllable */
  ["\uCBF1", "\uCBF1"], /* Hangul Syllable */
  ["\uCBF2", "\uCBF2"], /* Hangul Syllable */
  ["\uCBF3", "\uCBF3"], /* Hangul Syllable */
  ["\uCBF4", "\uCBF4"], /* Hangul Syllable */
  ["\uCBF5", "\uCBF5"], /* Hangul Syllable */
  ["\uCBF6", "\uCBF6"], /* Hangul Syllable */
  ["\uCBF7", "\uCBF7"], /* Hangul Syllable */
  ["\uCBF8", "\uCBF8"], /* Hangul Syllable */
  ["\uCBF9", "\uCBF9"], /* Hangul Syllable */
  ["\uCBFA", "\uCBFA"], /* Hangul Syllable */
  ["\uCBFB", "\uCBFB"], /* Hangul Syllable */
  ["\uCBFC", "\uCBFC"], /* Hangul Syllable */
  ["\uCBFD", "\uCBFD"], /* Hangul Syllable */
  ["\uCBFE", "\uCBFE"], /* Hangul Syllable */
  ["\uCBFF", "\uCBFF"], /* Hangul Syllable */
  ["\uCC00", "\uCC00"], /* Hangul Syllable */
  ["\uCC01", "\uCC01"], /* Hangul Syllable */
  ["\uCC02", "\uCC02"], /* Hangul Syllable */
  ["\uCC03", "\uCC03"], /* Hangul Syllable */
  ["\uCC04", "\uCC04"], /* Hangul Syllable */
  ["\uCC05", "\uCC05"], /* Hangul Syllable */
  ["\uCC06", "\uCC06"], /* Hangul Syllable */
  ["\uCC07", "\uCC07"], /* Hangul Syllable */
  ["\uCC08", "\uCC08"], /* Hangul Syllable */
  ["\uCC09", "\uCC09"], /* Hangul Syllable */
  ["\uCC0A", "\uCC0A"], /* Hangul Syllable */
  ["\uCC0B", "\uCC0B"], /* Hangul Syllable */
  ["\uCC0C", "\uCC0C"], /* Hangul Syllable */
  ["\uCC0D", "\uCC0D"], /* Hangul Syllable */
  ["\uCC0E", "\uCC0E"], /* Hangul Syllable */
  ["\uCC0F", "\uCC0F"], /* Hangul Syllable */
  ["\uCC10", "\uCC10"], /* Hangul Syllable */
  ["\uCC11", "\uCC11"], /* Hangul Syllable */
  ["\uCC12", "\uCC12"], /* Hangul Syllable */
  ["\uCC13", "\uCC13"], /* Hangul Syllable */
  ["\uCC14", "\uCC14"], /* Hangul Syllable */
  ["\uCC15", "\uCC15"], /* Hangul Syllable */
  ["\uCC16", "\uCC16"], /* Hangul Syllable */
  ["\uCC17", "\uCC17"], /* Hangul Syllable */
  ["\uCC18", "\uCC18"], /* Hangul Syllable */
  ["\uCC19", "\uCC19"], /* Hangul Syllable */
  ["\uCC1A", "\uCC1A"], /* Hangul Syllable */
  ["\uCC1B", "\uCC1B"], /* Hangul Syllable */
  ["\uCC1C", "\uCC1C"], /* Hangul Syllable */
  ["\uCC1D", "\uCC1D"], /* Hangul Syllable */
  ["\uCC1E", "\uCC1E"], /* Hangul Syllable */
  ["\uCC1F", "\uCC1F"], /* Hangul Syllable */
  ["\uCC20", "\uCC20"], /* Hangul Syllable */
  ["\uCC21", "\uCC21"], /* Hangul Syllable */
  ["\uCC22", "\uCC22"], /* Hangul Syllable */
  ["\uCC23", "\uCC23"], /* Hangul Syllable */
  ["\uCC24", "\uCC24"], /* Hangul Syllable */
  ["\uCC25", "\uCC25"], /* Hangul Syllable */
  ["\uCC26", "\uCC26"], /* Hangul Syllable */
  ["\uCC27", "\uCC27"], /* Hangul Syllable */
  ["\uCC28", "\uCC28"], /* Hangul Syllable */
  ["\uCC29", "\uCC29"], /* Hangul Syllable */
  ["\uCC2A", "\uCC2A"], /* Hangul Syllable */
  ["\uCC2B", "\uCC2B"], /* Hangul Syllable */
  ["\uCC2C", "\uCC2C"], /* Hangul Syllable */
  ["\uCC2D", "\uCC2D"], /* Hangul Syllable */
  ["\uCC2E", "\uCC2E"], /* Hangul Syllable */
  ["\uCC2F", "\uCC2F"], /* Hangul Syllable */
  ["\uCC30", "\uCC30"], /* Hangul Syllable */
  ["\uCC31", "\uCC31"], /* Hangul Syllable */
  ["\uCC32", "\uCC32"], /* Hangul Syllable */
  ["\uCC33", "\uCC33"], /* Hangul Syllable */
  ["\uCC34", "\uCC34"], /* Hangul Syllable */
  ["\uCC35", "\uCC35"], /* Hangul Syllable */
  ["\uCC36", "\uCC36"], /* Hangul Syllable */
  ["\uCC37", "\uCC37"], /* Hangul Syllable */
  ["\uCC38", "\uCC38"], /* Hangul Syllable */
  ["\uCC39", "\uCC39"], /* Hangul Syllable */
  ["\uCC3A", "\uCC3A"], /* Hangul Syllable */
  ["\uCC3B", "\uCC3B"], /* Hangul Syllable */
  ["\uCC3C", "\uCC3C"], /* Hangul Syllable */
  ["\uCC3D", "\uCC3D"], /* Hangul Syllable */
  ["\uCC3E", "\uCC3E"], /* Hangul Syllable */
  ["\uCC3F", "\uCC3F"], /* Hangul Syllable */
  ["\uCC40", "\uCC40"], /* Hangul Syllable */
  ["\uCC41", "\uCC41"], /* Hangul Syllable */
  ["\uCC42", "\uCC42"], /* Hangul Syllable */
  ["\uCC43", "\uCC43"], /* Hangul Syllable */
  ["\uCC44", "\uCC44"], /* Hangul Syllable */
  ["\uCC45", "\uCC45"], /* Hangul Syllable */
  ["\uCC46", "\uCC46"], /* Hangul Syllable */
  ["\uCC47", "\uCC47"], /* Hangul Syllable */
  ["\uCC48", "\uCC48"], /* Hangul Syllable */
  ["\uCC49", "\uCC49"], /* Hangul Syllable */
  ["\uCC4A", "\uCC4A"], /* Hangul Syllable */
  ["\uCC4B", "\uCC4B"], /* Hangul Syllable */
  ["\uCC4C", "\uCC4C"], /* Hangul Syllable */
  ["\uCC4D", "\uCC4D"], /* Hangul Syllable */
  ["\uCC4E", "\uCC4E"], /* Hangul Syllable */
  ["\uCC4F", "\uCC4F"], /* Hangul Syllable */
  ["\uCC50", "\uCC50"], /* Hangul Syllable */
  ["\uCC51", "\uCC51"], /* Hangul Syllable */
  ["\uCC52", "\uCC52"], /* Hangul Syllable */
  ["\uCC53", "\uCC53"], /* Hangul Syllable */
  ["\uCC54", "\uCC54"], /* Hangul Syllable */
  ["\uCC55", "\uCC55"], /* Hangul Syllable */
  ["\uCC56", "\uCC56"], /* Hangul Syllable */
  ["\uCC57", "\uCC57"], /* Hangul Syllable */
  ["\uCC58", "\uCC58"], /* Hangul Syllable */
  ["\uCC59", "\uCC59"], /* Hangul Syllable */
  ["\uCC5A", "\uCC5A"], /* Hangul Syllable */
  ["\uCC5B", "\uCC5B"], /* Hangul Syllable */
  ["\uCC5C", "\uCC5C"], /* Hangul Syllable */
  ["\uCC5D", "\uCC5D"], /* Hangul Syllable */
  ["\uCC5E", "\uCC5E"], /* Hangul Syllable */
  ["\uCC5F", "\uCC5F"], /* Hangul Syllable */
  ["\uCC60", "\uCC60"], /* Hangul Syllable */
  ["\uCC61", "\uCC61"], /* Hangul Syllable */
  ["\uCC62", "\uCC62"], /* Hangul Syllable */
  ["\uCC63", "\uCC63"], /* Hangul Syllable */
  ["\uCC64", "\uCC64"], /* Hangul Syllable */
  ["\uCC65", "\uCC65"], /* Hangul Syllable */
  ["\uCC66", "\uCC66"], /* Hangul Syllable */
  ["\uCC67", "\uCC67"], /* Hangul Syllable */
  ["\uCC68", "\uCC68"], /* Hangul Syllable */
  ["\uCC69", "\uCC69"], /* Hangul Syllable */
  ["\uCC6A", "\uCC6A"], /* Hangul Syllable */
  ["\uCC6B", "\uCC6B"], /* Hangul Syllable */
  ["\uCC6C", "\uCC6C"], /* Hangul Syllable */
  ["\uCC6D", "\uCC6D"], /* Hangul Syllable */
  ["\uCC6E", "\uCC6E"], /* Hangul Syllable */
  ["\uCC6F", "\uCC6F"], /* Hangul Syllable */
  ["\uCC70", "\uCC70"], /* Hangul Syllable */
  ["\uCC71", "\uCC71"], /* Hangul Syllable */
  ["\uCC72", "\uCC72"], /* Hangul Syllable */
  ["\uCC73", "\uCC73"], /* Hangul Syllable */
  ["\uCC74", "\uCC74"], /* Hangul Syllable */
  ["\uCC75", "\uCC75"], /* Hangul Syllable */
  ["\uCC76", "\uCC76"], /* Hangul Syllable */
  ["\uCC77", "\uCC77"], /* Hangul Syllable */
  ["\uCC78", "\uCC78"], /* Hangul Syllable */
  ["\uCC79", "\uCC79"], /* Hangul Syllable */
  ["\uCC7A", "\uCC7A"], /* Hangul Syllable */
  ["\uCC7B", "\uCC7B"], /* Hangul Syllable */
  ["\uCC7C", "\uCC7C"], /* Hangul Syllable */
  ["\uCC7D", "\uCC7D"], /* Hangul Syllable */
  ["\uCC7E", "\uCC7E"], /* Hangul Syllable */
  ["\uCC7F", "\uCC7F"], /* Hangul Syllable */
  ["\uCC80", "\uCC80"], /* Hangul Syllable */
  ["\uCC81", "\uCC81"], /* Hangul Syllable */
  ["\uCC82", "\uCC82"], /* Hangul Syllable */
  ["\uCC83", "\uCC83"], /* Hangul Syllable */
  ["\uCC84", "\uCC84"], /* Hangul Syllable */
  ["\uCC85", "\uCC85"], /* Hangul Syllable */
  ["\uCC86", "\uCC86"], /* Hangul Syllable */
  ["\uCC87", "\uCC87"], /* Hangul Syllable */
  ["\uCC88", "\uCC88"], /* Hangul Syllable */
  ["\uCC89", "\uCC89"], /* Hangul Syllable */
  ["\uCC8A", "\uCC8A"], /* Hangul Syllable */
  ["\uCC8B", "\uCC8B"], /* Hangul Syllable */
  ["\uCC8C", "\uCC8C"], /* Hangul Syllable */
  ["\uCC8D", "\uCC8D"], /* Hangul Syllable */
  ["\uCC8E", "\uCC8E"], /* Hangul Syllable */
  ["\uCC8F", "\uCC8F"], /* Hangul Syllable */
  ["\uCC90", "\uCC90"], /* Hangul Syllable */
  ["\uCC91", "\uCC91"], /* Hangul Syllable */
  ["\uCC92", "\uCC92"], /* Hangul Syllable */
  ["\uCC93", "\uCC93"], /* Hangul Syllable */
  ["\uCC94", "\uCC94"], /* Hangul Syllable */
  ["\uCC95", "\uCC95"], /* Hangul Syllable */
  ["\uCC96", "\uCC96"], /* Hangul Syllable */
  ["\uCC97", "\uCC97"], /* Hangul Syllable */
  ["\uCC98", "\uCC98"], /* Hangul Syllable */
  ["\uCC99", "\uCC99"], /* Hangul Syllable */
  ["\uCC9A", "\uCC9A"], /* Hangul Syllable */
  ["\uCC9B", "\uCC9B"], /* Hangul Syllable */
  ["\uCC9C", "\uCC9C"], /* Hangul Syllable */
  ["\uCC9D", "\uCC9D"], /* Hangul Syllable */
  ["\uCC9E", "\uCC9E"], /* Hangul Syllable */
  ["\uCC9F", "\uCC9F"], /* Hangul Syllable */
  ["\uCCA0", "\uCCA0"], /* Hangul Syllable */
  ["\uCCA1", "\uCCA1"], /* Hangul Syllable */
  ["\uCCA2", "\uCCA2"], /* Hangul Syllable */
  ["\uCCA3", "\uCCA3"], /* Hangul Syllable */
  ["\uCCA4", "\uCCA4"], /* Hangul Syllable */
  ["\uCCA5", "\uCCA5"], /* Hangul Syllable */
  ["\uCCA6", "\uCCA6"], /* Hangul Syllable */
  ["\uCCA7", "\uCCA7"], /* Hangul Syllable */
  ["\uCCA8", "\uCCA8"], /* Hangul Syllable */
  ["\uCCA9", "\uCCA9"], /* Hangul Syllable */
  ["\uCCAA", "\uCCAA"], /* Hangul Syllable */
  ["\uCCAB", "\uCCAB"], /* Hangul Syllable */
  ["\uCCAC", "\uCCAC"], /* Hangul Syllable */
  ["\uCCAD", "\uCCAD"], /* Hangul Syllable */
  ["\uCCAE", "\uCCAE"], /* Hangul Syllable */
  ["\uCCAF", "\uCCAF"], /* Hangul Syllable */
  ["\uCCB0", "\uCCB0"], /* Hangul Syllable */
  ["\uCCB1", "\uCCB1"], /* Hangul Syllable */
  ["\uCCB2", "\uCCB2"], /* Hangul Syllable */
  ["\uCCB3", "\uCCB3"], /* Hangul Syllable */
  ["\uCCB4", "\uCCB4"], /* Hangul Syllable */
  ["\uCCB5", "\uCCB5"], /* Hangul Syllable */
  ["\uCCB6", "\uCCB6"], /* Hangul Syllable */
  ["\uCCB7", "\uCCB7"], /* Hangul Syllable */
  ["\uCCB8", "\uCCB8"], /* Hangul Syllable */
  ["\uCCB9", "\uCCB9"], /* Hangul Syllable */
  ["\uCCBA", "\uCCBA"], /* Hangul Syllable */
  ["\uCCBB", "\uCCBB"], /* Hangul Syllable */
  ["\uCCBC", "\uCCBC"], /* Hangul Syllable */
  ["\uCCBD", "\uCCBD"], /* Hangul Syllable */
  ["\uCCBE", "\uCCBE"], /* Hangul Syllable */
  ["\uCCBF", "\uCCBF"], /* Hangul Syllable */
  ["\uCCC0", "\uCCC0"], /* Hangul Syllable */
  ["\uCCC1", "\uCCC1"], /* Hangul Syllable */
  ["\uCCC2", "\uCCC2"], /* Hangul Syllable */
  ["\uCCC3", "\uCCC3"], /* Hangul Syllable */
  ["\uCCC4", "\uCCC4"], /* Hangul Syllable */
  ["\uCCC5", "\uCCC5"], /* Hangul Syllable */
  ["\uCCC6", "\uCCC6"], /* Hangul Syllable */
  ["\uCCC7", "\uCCC7"], /* Hangul Syllable */
  ["\uCCC8", "\uCCC8"], /* Hangul Syllable */
  ["\uCCC9", "\uCCC9"], /* Hangul Syllable */
  ["\uCCCA", "\uCCCA"], /* Hangul Syllable */
  ["\uCCCB", "\uCCCB"], /* Hangul Syllable */
  ["\uCCCC", "\uCCCC"], /* Hangul Syllable */
  ["\uCCCD", "\uCCCD"], /* Hangul Syllable */
  ["\uCCCE", "\uCCCE"], /* Hangul Syllable */
  ["\uCCCF", "\uCCCF"], /* Hangul Syllable */
  ["\uCCD0", "\uCCD0"], /* Hangul Syllable */
  ["\uCCD1", "\uCCD1"], /* Hangul Syllable */
  ["\uCCD2", "\uCCD2"], /* Hangul Syllable */
  ["\uCCD3", "\uCCD3"], /* Hangul Syllable */
  ["\uCCD4", "\uCCD4"], /* Hangul Syllable */
  ["\uCCD5", "\uCCD5"], /* Hangul Syllable */
  ["\uCCD6", "\uCCD6"], /* Hangul Syllable */
  ["\uCCD7", "\uCCD7"], /* Hangul Syllable */
  ["\uCCD8", "\uCCD8"], /* Hangul Syllable */
  ["\uCCD9", "\uCCD9"], /* Hangul Syllable */
  ["\uCCDA", "\uCCDA"], /* Hangul Syllable */
  ["\uCCDB", "\uCCDB"], /* Hangul Syllable */
  ["\uCCDC", "\uCCDC"], /* Hangul Syllable */
  ["\uCCDD", "\uCCDD"], /* Hangul Syllable */
  ["\uCCDE", "\uCCDE"], /* Hangul Syllable */
  ["\uCCDF", "\uCCDF"], /* Hangul Syllable */
  ["\uCCE0", "\uCCE0"], /* Hangul Syllable */
  ["\uCCE1", "\uCCE1"], /* Hangul Syllable */
  ["\uCCE2", "\uCCE2"], /* Hangul Syllable */
  ["\uCCE3", "\uCCE3"], /* Hangul Syllable */
  ["\uCCE4", "\uCCE4"], /* Hangul Syllable */
  ["\uCCE5", "\uCCE5"], /* Hangul Syllable */
  ["\uCCE6", "\uCCE6"], /* Hangul Syllable */
  ["\uCCE7", "\uCCE7"], /* Hangul Syllable */
  ["\uCCE8", "\uCCE8"], /* Hangul Syllable */
  ["\uCCE9", "\uCCE9"], /* Hangul Syllable */
  ["\uCCEA", "\uCCEA"], /* Hangul Syllable */
  ["\uCCEB", "\uCCEB"], /* Hangul Syllable */
  ["\uCCEC", "\uCCEC"], /* Hangul Syllable */
  ["\uCCED", "\uCCED"], /* Hangul Syllable */
  ["\uCCEE", "\uCCEE"], /* Hangul Syllable */
  ["\uCCEF", "\uCCEF"], /* Hangul Syllable */
  ["\uCCF0", "\uCCF0"], /* Hangul Syllable */
  ["\uCCF1", "\uCCF1"], /* Hangul Syllable */
  ["\uCCF2", "\uCCF2"], /* Hangul Syllable */
  ["\uCCF3", "\uCCF3"], /* Hangul Syllable */
  ["\uCCF4", "\uCCF4"], /* Hangul Syllable */
  ["\uCCF5", "\uCCF5"], /* Hangul Syllable */
  ["\uCCF6", "\uCCF6"], /* Hangul Syllable */
  ["\uCCF7", "\uCCF7"], /* Hangul Syllable */
  ["\uCCF8", "\uCCF8"], /* Hangul Syllable */
  ["\uCCF9", "\uCCF9"], /* Hangul Syllable */
  ["\uCCFA", "\uCCFA"], /* Hangul Syllable */
  ["\uCCFB", "\uCCFB"], /* Hangul Syllable */
  ["\uCCFC", "\uCCFC"], /* Hangul Syllable */
  ["\uCCFD", "\uCCFD"], /* Hangul Syllable */
  ["\uCCFE", "\uCCFE"], /* Hangul Syllable */
  ["\uCCFF", "\uCCFF"], /* Hangul Syllable */
  ["\uCD00", "\uCD00"], /* Hangul Syllable */
  ["\uCD01", "\uCD01"], /* Hangul Syllable */
  ["\uCD02", "\uCD02"], /* Hangul Syllable */
  ["\uCD03", "\uCD03"], /* Hangul Syllable */
  ["\uCD04", "\uCD04"], /* Hangul Syllable */
  ["\uCD05", "\uCD05"], /* Hangul Syllable */
  ["\uCD06", "\uCD06"], /* Hangul Syllable */
  ["\uCD07", "\uCD07"], /* Hangul Syllable */
  ["\uCD08", "\uCD08"], /* Hangul Syllable */
  ["\uCD09", "\uCD09"], /* Hangul Syllable */
  ["\uCD0A", "\uCD0A"], /* Hangul Syllable */
  ["\uCD0B", "\uCD0B"], /* Hangul Syllable */
  ["\uCD0C", "\uCD0C"], /* Hangul Syllable */
  ["\uCD0D", "\uCD0D"], /* Hangul Syllable */
  ["\uCD0E", "\uCD0E"], /* Hangul Syllable */
  ["\uCD0F", "\uCD0F"], /* Hangul Syllable */
  ["\uCD10", "\uCD10"], /* Hangul Syllable */
  ["\uCD11", "\uCD11"], /* Hangul Syllable */
  ["\uCD12", "\uCD12"], /* Hangul Syllable */
  ["\uCD13", "\uCD13"], /* Hangul Syllable */
  ["\uCD14", "\uCD14"], /* Hangul Syllable */
  ["\uCD15", "\uCD15"], /* Hangul Syllable */
  ["\uCD16", "\uCD16"], /* Hangul Syllable */
  ["\uCD17", "\uCD17"], /* Hangul Syllable */
  ["\uCD18", "\uCD18"], /* Hangul Syllable */
  ["\uCD19", "\uCD19"], /* Hangul Syllable */
  ["\uCD1A", "\uCD1A"], /* Hangul Syllable */
  ["\uCD1B", "\uCD1B"], /* Hangul Syllable */
  ["\uCD1C", "\uCD1C"], /* Hangul Syllable */
  ["\uCD1D", "\uCD1D"], /* Hangul Syllable */
  ["\uCD1E", "\uCD1E"], /* Hangul Syllable */
  ["\uCD1F", "\uCD1F"], /* Hangul Syllable */
  ["\uCD20", "\uCD20"], /* Hangul Syllable */
  ["\uCD21", "\uCD21"], /* Hangul Syllable */
  ["\uCD22", "\uCD22"], /* Hangul Syllable */
  ["\uCD23", "\uCD23"], /* Hangul Syllable */
  ["\uCD24", "\uCD24"], /* Hangul Syllable */
  ["\uCD25", "\uCD25"], /* Hangul Syllable */
  ["\uCD26", "\uCD26"], /* Hangul Syllable */
  ["\uCD27", "\uCD27"], /* Hangul Syllable */
  ["\uCD28", "\uCD28"], /* Hangul Syllable */
  ["\uCD29", "\uCD29"], /* Hangul Syllable */
  ["\uCD2A", "\uCD2A"], /* Hangul Syllable */
  ["\uCD2B", "\uCD2B"], /* Hangul Syllable */
  ["\uCD2C", "\uCD2C"], /* Hangul Syllable */
  ["\uCD2D", "\uCD2D"], /* Hangul Syllable */
  ["\uCD2E", "\uCD2E"], /* Hangul Syllable */
  ["\uCD2F", "\uCD2F"], /* Hangul Syllable */
  ["\uCD30", "\uCD30"], /* Hangul Syllable */
  ["\uCD31", "\uCD31"], /* Hangul Syllable */
  ["\uCD32", "\uCD32"], /* Hangul Syllable */
  ["\uCD33", "\uCD33"], /* Hangul Syllable */
  ["\uCD34", "\uCD34"], /* Hangul Syllable */
  ["\uCD35", "\uCD35"], /* Hangul Syllable */
  ["\uCD36", "\uCD36"], /* Hangul Syllable */
  ["\uCD37", "\uCD37"], /* Hangul Syllable */
  ["\uCD38", "\uCD38"], /* Hangul Syllable */
  ["\uCD39", "\uCD39"], /* Hangul Syllable */
  ["\uCD3A", "\uCD3A"], /* Hangul Syllable */
  ["\uCD3B", "\uCD3B"], /* Hangul Syllable */
  ["\uCD3C", "\uCD3C"], /* Hangul Syllable */
  ["\uCD3D", "\uCD3D"], /* Hangul Syllable */
  ["\uCD3E", "\uCD3E"], /* Hangul Syllable */
  ["\uCD3F", "\uCD3F"], /* Hangul Syllable */
  ["\uCD40", "\uCD40"], /* Hangul Syllable */
  ["\uCD41", "\uCD41"], /* Hangul Syllable */
  ["\uCD42", "\uCD42"], /* Hangul Syllable */
  ["\uCD43", "\uCD43"], /* Hangul Syllable */
  ["\uCD44", "\uCD44"], /* Hangul Syllable */
  ["\uCD45", "\uCD45"], /* Hangul Syllable */
  ["\uCD46", "\uCD46"], /* Hangul Syllable */
  ["\uCD47", "\uCD47"], /* Hangul Syllable */
  ["\uCD48", "\uCD48"], /* Hangul Syllable */
  ["\uCD49", "\uCD49"], /* Hangul Syllable */
  ["\uCD4A", "\uCD4A"], /* Hangul Syllable */
  ["\uCD4B", "\uCD4B"], /* Hangul Syllable */
  ["\uCD4C", "\uCD4C"], /* Hangul Syllable */
  ["\uCD4D", "\uCD4D"], /* Hangul Syllable */
  ["\uCD4E", "\uCD4E"], /* Hangul Syllable */
  ["\uCD4F", "\uCD4F"], /* Hangul Syllable */
  ["\uCD50", "\uCD50"], /* Hangul Syllable */
  ["\uCD51", "\uCD51"], /* Hangul Syllable */
  ["\uCD52", "\uCD52"], /* Hangul Syllable */
  ["\uCD53", "\uCD53"], /* Hangul Syllable */
  ["\uCD54", "\uCD54"], /* Hangul Syllable */
  ["\uCD55", "\uCD55"], /* Hangul Syllable */
  ["\uCD56", "\uCD56"], /* Hangul Syllable */
  ["\uCD57", "\uCD57"], /* Hangul Syllable */
  ["\uCD58", "\uCD58"], /* Hangul Syllable */
  ["\uCD59", "\uCD59"], /* Hangul Syllable */
  ["\uCD5A", "\uCD5A"], /* Hangul Syllable */
  ["\uCD5B", "\uCD5B"], /* Hangul Syllable */
  ["\uCD5C", "\uCD5C"], /* Hangul Syllable */
  ["\uCD5D", "\uCD5D"], /* Hangul Syllable */
  ["\uCD5E", "\uCD5E"], /* Hangul Syllable */
  ["\uCD5F", "\uCD5F"], /* Hangul Syllable */
  ["\uCD60", "\uCD60"], /* Hangul Syllable */
  ["\uCD61", "\uCD61"], /* Hangul Syllable */
  ["\uCD62", "\uCD62"], /* Hangul Syllable */
  ["\uCD63", "\uCD63"], /* Hangul Syllable */
  ["\uCD64", "\uCD64"], /* Hangul Syllable */
  ["\uCD65", "\uCD65"], /* Hangul Syllable */
  ["\uCD66", "\uCD66"], /* Hangul Syllable */
  ["\uCD67", "\uCD67"], /* Hangul Syllable */
  ["\uCD68", "\uCD68"], /* Hangul Syllable */
  ["\uCD69", "\uCD69"], /* Hangul Syllable */
  ["\uCD6A", "\uCD6A"], /* Hangul Syllable */
  ["\uCD6B", "\uCD6B"], /* Hangul Syllable */
  ["\uCD6C", "\uCD6C"], /* Hangul Syllable */
  ["\uCD6D", "\uCD6D"], /* Hangul Syllable */
  ["\uCD6E", "\uCD6E"], /* Hangul Syllable */
  ["\uCD6F", "\uCD6F"], /* Hangul Syllable */
  ["\uCD70", "\uCD70"], /* Hangul Syllable */
  ["\uCD71", "\uCD71"], /* Hangul Syllable */
  ["\uCD72", "\uCD72"], /* Hangul Syllable */
  ["\uCD73", "\uCD73"], /* Hangul Syllable */
  ["\uCD74", "\uCD74"], /* Hangul Syllable */
  ["\uCD75", "\uCD75"], /* Hangul Syllable */
  ["\uCD76", "\uCD76"], /* Hangul Syllable */
  ["\uCD77", "\uCD77"], /* Hangul Syllable */
  ["\uCD78", "\uCD78"], /* Hangul Syllable */
  ["\uCD79", "\uCD79"], /* Hangul Syllable */
  ["\uCD7A", "\uCD7A"], /* Hangul Syllable */
  ["\uCD7B", "\uCD7B"], /* Hangul Syllable */
  ["\uCD7C", "\uCD7C"], /* Hangul Syllable */
  ["\uCD7D", "\uCD7D"], /* Hangul Syllable */
  ["\uCD7E", "\uCD7E"], /* Hangul Syllable */
  ["\uCD7F", "\uCD7F"], /* Hangul Syllable */
  ["\uCD80", "\uCD80"], /* Hangul Syllable */
  ["\uCD81", "\uCD81"], /* Hangul Syllable */
  ["\uCD82", "\uCD82"], /* Hangul Syllable */
  ["\uCD83", "\uCD83"], /* Hangul Syllable */
  ["\uCD84", "\uCD84"], /* Hangul Syllable */
  ["\uCD85", "\uCD85"], /* Hangul Syllable */
  ["\uCD86", "\uCD86"], /* Hangul Syllable */
  ["\uCD87", "\uCD87"], /* Hangul Syllable */
  ["\uCD88", "\uCD88"], /* Hangul Syllable */
  ["\uCD89", "\uCD89"], /* Hangul Syllable */
  ["\uCD8A", "\uCD8A"], /* Hangul Syllable */
  ["\uCD8B", "\uCD8B"], /* Hangul Syllable */
  ["\uCD8C", "\uCD8C"], /* Hangul Syllable */
  ["\uCD8D", "\uCD8D"], /* Hangul Syllable */
  ["\uCD8E", "\uCD8E"], /* Hangul Syllable */
  ["\uCD8F", "\uCD8F"], /* Hangul Syllable */
  ["\uCD90", "\uCD90"], /* Hangul Syllable */
  ["\uCD91", "\uCD91"], /* Hangul Syllable */
  ["\uCD92", "\uCD92"], /* Hangul Syllable */
  ["\uCD93", "\uCD93"], /* Hangul Syllable */
  ["\uCD94", "\uCD94"], /* Hangul Syllable */
  ["\uCD95", "\uCD95"], /* Hangul Syllable */
  ["\uCD96", "\uCD96"], /* Hangul Syllable */
  ["\uCD97", "\uCD97"], /* Hangul Syllable */
  ["\uCD98", "\uCD98"], /* Hangul Syllable */
  ["\uCD99", "\uCD99"], /* Hangul Syllable */
  ["\uCD9A", "\uCD9A"], /* Hangul Syllable */
  ["\uCD9B", "\uCD9B"], /* Hangul Syllable */
  ["\uCD9C", "\uCD9C"], /* Hangul Syllable */
  ["\uCD9D", "\uCD9D"], /* Hangul Syllable */
  ["\uCD9E", "\uCD9E"], /* Hangul Syllable */
  ["\uCD9F", "\uCD9F"], /* Hangul Syllable */
  ["\uCDA0", "\uCDA0"], /* Hangul Syllable */
  ["\uCDA1", "\uCDA1"], /* Hangul Syllable */
  ["\uCDA2", "\uCDA2"], /* Hangul Syllable */
  ["\uCDA3", "\uCDA3"], /* Hangul Syllable */
  ["\uCDA4", "\uCDA4"], /* Hangul Syllable */
  ["\uCDA5", "\uCDA5"], /* Hangul Syllable */
  ["\uCDA6", "\uCDA6"], /* Hangul Syllable */
  ["\uCDA7", "\uCDA7"], /* Hangul Syllable */
  ["\uCDA8", "\uCDA8"], /* Hangul Syllable */
  ["\uCDA9", "\uCDA9"], /* Hangul Syllable */
  ["\uCDAA", "\uCDAA"], /* Hangul Syllable */
  ["\uCDAB", "\uCDAB"], /* Hangul Syllable */
  ["\uCDAC", "\uCDAC"], /* Hangul Syllable */
  ["\uCDAD", "\uCDAD"], /* Hangul Syllable */
  ["\uCDAE", "\uCDAE"], /* Hangul Syllable */
  ["\uCDAF", "\uCDAF"], /* Hangul Syllable */
  ["\uCDB0", "\uCDB0"], /* Hangul Syllable */
  ["\uCDB1", "\uCDB1"], /* Hangul Syllable */
  ["\uCDB2", "\uCDB2"], /* Hangul Syllable */
  ["\uCDB3", "\uCDB3"], /* Hangul Syllable */
  ["\uCDB4", "\uCDB4"], /* Hangul Syllable */
  ["\uCDB5", "\uCDB5"], /* Hangul Syllable */
  ["\uCDB6", "\uCDB6"], /* Hangul Syllable */
  ["\uCDB7", "\uCDB7"], /* Hangul Syllable */
  ["\uCDB8", "\uCDB8"], /* Hangul Syllable */
  ["\uCDB9", "\uCDB9"], /* Hangul Syllable */
  ["\uCDBA", "\uCDBA"], /* Hangul Syllable */
  ["\uCDBB", "\uCDBB"], /* Hangul Syllable */
  ["\uCDBC", "\uCDBC"], /* Hangul Syllable */
  ["\uCDBD", "\uCDBD"], /* Hangul Syllable */
  ["\uCDBE", "\uCDBE"], /* Hangul Syllable */
  ["\uCDBF", "\uCDBF"], /* Hangul Syllable */
  ["\uCDC0", "\uCDC0"], /* Hangul Syllable */
  ["\uCDC1", "\uCDC1"], /* Hangul Syllable */
  ["\uCDC2", "\uCDC2"], /* Hangul Syllable */
  ["\uCDC3", "\uCDC3"], /* Hangul Syllable */
  ["\uCDC4", "\uCDC4"], /* Hangul Syllable */
  ["\uCDC5", "\uCDC5"], /* Hangul Syllable */
  ["\uCDC6", "\uCDC6"], /* Hangul Syllable */
  ["\uCDC7", "\uCDC7"], /* Hangul Syllable */
  ["\uCDC8", "\uCDC8"], /* Hangul Syllable */
  ["\uCDC9", "\uCDC9"], /* Hangul Syllable */
  ["\uCDCA", "\uCDCA"], /* Hangul Syllable */
  ["\uCDCB", "\uCDCB"], /* Hangul Syllable */
  ["\uCDCC", "\uCDCC"], /* Hangul Syllable */
  ["\uCDCD", "\uCDCD"], /* Hangul Syllable */
  ["\uCDCE", "\uCDCE"], /* Hangul Syllable */
  ["\uCDCF", "\uCDCF"], /* Hangul Syllable */
  ["\uCDD0", "\uCDD0"], /* Hangul Syllable */
  ["\uCDD1", "\uCDD1"], /* Hangul Syllable */
  ["\uCDD2", "\uCDD2"], /* Hangul Syllable */
  ["\uCDD3", "\uCDD3"], /* Hangul Syllable */
  ["\uCDD4", "\uCDD4"], /* Hangul Syllable */
  ["\uCDD5", "\uCDD5"], /* Hangul Syllable */
  ["\uCDD6", "\uCDD6"], /* Hangul Syllable */
  ["\uCDD7", "\uCDD7"], /* Hangul Syllable */
  ["\uCDD8", "\uCDD8"], /* Hangul Syllable */
  ["\uCDD9", "\uCDD9"], /* Hangul Syllable */
  ["\uCDDA", "\uCDDA"], /* Hangul Syllable */
  ["\uCDDB", "\uCDDB"], /* Hangul Syllable */
  ["\uCDDC", "\uCDDC"], /* Hangul Syllable */
  ["\uCDDD", "\uCDDD"], /* Hangul Syllable */
  ["\uCDDE", "\uCDDE"], /* Hangul Syllable */
  ["\uCDDF", "\uCDDF"], /* Hangul Syllable */
  ["\uCDE0", "\uCDE0"], /* Hangul Syllable */
  ["\uCDE1", "\uCDE1"], /* Hangul Syllable */
  ["\uCDE2", "\uCDE2"], /* Hangul Syllable */
  ["\uCDE3", "\uCDE3"], /* Hangul Syllable */
  ["\uCDE4", "\uCDE4"], /* Hangul Syllable */
  ["\uCDE5", "\uCDE5"], /* Hangul Syllable */
  ["\uCDE6", "\uCDE6"], /* Hangul Syllable */
  ["\uCDE7", "\uCDE7"], /* Hangul Syllable */
  ["\uCDE8", "\uCDE8"], /* Hangul Syllable */
  ["\uCDE9", "\uCDE9"], /* Hangul Syllable */
  ["\uCDEA", "\uCDEA"], /* Hangul Syllable */
  ["\uCDEB", "\uCDEB"], /* Hangul Syllable */
  ["\uCDEC", "\uCDEC"], /* Hangul Syllable */
  ["\uCDED", "\uCDED"], /* Hangul Syllable */
  ["\uCDEE", "\uCDEE"], /* Hangul Syllable */
  ["\uCDEF", "\uCDEF"], /* Hangul Syllable */
  ["\uCDF0", "\uCDF0"], /* Hangul Syllable */
  ["\uCDF1", "\uCDF1"], /* Hangul Syllable */
  ["\uCDF2", "\uCDF2"], /* Hangul Syllable */
  ["\uCDF3", "\uCDF3"], /* Hangul Syllable */
  ["\uCDF4", "\uCDF4"], /* Hangul Syllable */
  ["\uCDF5", "\uCDF5"], /* Hangul Syllable */
  ["\uCDF6", "\uCDF6"], /* Hangul Syllable */
  ["\uCDF7", "\uCDF7"], /* Hangul Syllable */
  ["\uCDF8", "\uCDF8"], /* Hangul Syllable */
  ["\uCDF9", "\uCDF9"], /* Hangul Syllable */
  ["\uCDFA", "\uCDFA"], /* Hangul Syllable */
  ["\uCDFB", "\uCDFB"], /* Hangul Syllable */
  ["\uCDFC", "\uCDFC"], /* Hangul Syllable */
  ["\uCDFD", "\uCDFD"], /* Hangul Syllable */
  ["\uCDFE", "\uCDFE"], /* Hangul Syllable */
  ["\uCDFF", "\uCDFF"], /* Hangul Syllable */
  ["\uCE00", "\uCE00"], /* Hangul Syllable */
  ["\uCE01", "\uCE01"], /* Hangul Syllable */
  ["\uCE02", "\uCE02"], /* Hangul Syllable */
  ["\uCE03", "\uCE03"], /* Hangul Syllable */
  ["\uCE04", "\uCE04"], /* Hangul Syllable */
  ["\uCE05", "\uCE05"], /* Hangul Syllable */
  ["\uCE06", "\uCE06"], /* Hangul Syllable */
  ["\uCE07", "\uCE07"], /* Hangul Syllable */
  ["\uCE08", "\uCE08"], /* Hangul Syllable */
  ["\uCE09", "\uCE09"], /* Hangul Syllable */
  ["\uCE0A", "\uCE0A"], /* Hangul Syllable */
  ["\uCE0B", "\uCE0B"], /* Hangul Syllable */
  ["\uCE0C", "\uCE0C"], /* Hangul Syllable */
  ["\uCE0D", "\uCE0D"], /* Hangul Syllable */
  ["\uCE0E", "\uCE0E"], /* Hangul Syllable */
  ["\uCE0F", "\uCE0F"], /* Hangul Syllable */
  ["\uCE10", "\uCE10"], /* Hangul Syllable */
  ["\uCE11", "\uCE11"], /* Hangul Syllable */
  ["\uCE12", "\uCE12"], /* Hangul Syllable */
  ["\uCE13", "\uCE13"], /* Hangul Syllable */
  ["\uCE14", "\uCE14"], /* Hangul Syllable */
  ["\uCE15", "\uCE15"], /* Hangul Syllable */
  ["\uCE16", "\uCE16"], /* Hangul Syllable */
  ["\uCE17", "\uCE17"], /* Hangul Syllable */
  ["\uCE18", "\uCE18"], /* Hangul Syllable */
  ["\uCE19", "\uCE19"], /* Hangul Syllable */
  ["\uCE1A", "\uCE1A"], /* Hangul Syllable */
  ["\uCE1B", "\uCE1B"], /* Hangul Syllable */
  ["\uCE1C", "\uCE1C"], /* Hangul Syllable */
  ["\uCE1D", "\uCE1D"], /* Hangul Syllable */
  ["\uCE1E", "\uCE1E"], /* Hangul Syllable */
  ["\uCE1F", "\uCE1F"], /* Hangul Syllable */
  ["\uCE20", "\uCE20"], /* Hangul Syllable */
  ["\uCE21", "\uCE21"], /* Hangul Syllable */
  ["\uCE22", "\uCE22"], /* Hangul Syllable */
  ["\uCE23", "\uCE23"], /* Hangul Syllable */
  ["\uCE24", "\uCE24"], /* Hangul Syllable */
  ["\uCE25", "\uCE25"], /* Hangul Syllable */
  ["\uCE26", "\uCE26"], /* Hangul Syllable */
  ["\uCE27", "\uCE27"], /* Hangul Syllable */
  ["\uCE28", "\uCE28"], /* Hangul Syllable */
  ["\uCE29", "\uCE29"], /* Hangul Syllable */
  ["\uCE2A", "\uCE2A"], /* Hangul Syllable */
  ["\uCE2B", "\uCE2B"], /* Hangul Syllable */
  ["\uCE2C", "\uCE2C"], /* Hangul Syllable */
  ["\uCE2D", "\uCE2D"], /* Hangul Syllable */
  ["\uCE2E", "\uCE2E"], /* Hangul Syllable */
  ["\uCE2F", "\uCE2F"], /* Hangul Syllable */
  ["\uCE30", "\uCE30"], /* Hangul Syllable */
  ["\uCE31", "\uCE31"], /* Hangul Syllable */
  ["\uCE32", "\uCE32"], /* Hangul Syllable */
  ["\uCE33", "\uCE33"], /* Hangul Syllable */
  ["\uCE34", "\uCE34"], /* Hangul Syllable */
  ["\uCE35", "\uCE35"], /* Hangul Syllable */
  ["\uCE36", "\uCE36"], /* Hangul Syllable */
  ["\uCE37", "\uCE37"], /* Hangul Syllable */
  ["\uCE38", "\uCE38"], /* Hangul Syllable */
  ["\uCE39", "\uCE39"], /* Hangul Syllable */
  ["\uCE3A", "\uCE3A"], /* Hangul Syllable */
  ["\uCE3B", "\uCE3B"], /* Hangul Syllable */
  ["\uCE3C", "\uCE3C"], /* Hangul Syllable */
  ["\uCE3D", "\uCE3D"], /* Hangul Syllable */
  ["\uCE3E", "\uCE3E"], /* Hangul Syllable */
  ["\uCE3F", "\uCE3F"], /* Hangul Syllable */
  ["\uCE40", "\uCE40"], /* Hangul Syllable */
  ["\uCE41", "\uCE41"], /* Hangul Syllable */
  ["\uCE42", "\uCE42"], /* Hangul Syllable */
  ["\uCE43", "\uCE43"], /* Hangul Syllable */
  ["\uCE44", "\uCE44"], /* Hangul Syllable */
  ["\uCE45", "\uCE45"], /* Hangul Syllable */
  ["\uCE46", "\uCE46"], /* Hangul Syllable */
  ["\uCE47", "\uCE47"], /* Hangul Syllable */
  ["\uCE48", "\uCE48"], /* Hangul Syllable */
  ["\uCE49", "\uCE49"], /* Hangul Syllable */
  ["\uCE4A", "\uCE4A"], /* Hangul Syllable */
  ["\uCE4B", "\uCE4B"], /* Hangul Syllable */
  ["\uCE4C", "\uCE4C"], /* Hangul Syllable */
  ["\uCE4D", "\uCE4D"], /* Hangul Syllable */
  ["\uCE4E", "\uCE4E"], /* Hangul Syllable */
  ["\uCE4F", "\uCE4F"], /* Hangul Syllable */
  ["\uCE50", "\uCE50"], /* Hangul Syllable */
  ["\uCE51", "\uCE51"], /* Hangul Syllable */
  ["\uCE52", "\uCE52"], /* Hangul Syllable */
  ["\uCE53", "\uCE53"], /* Hangul Syllable */
  ["\uCE54", "\uCE54"], /* Hangul Syllable */
  ["\uCE55", "\uCE55"], /* Hangul Syllable */
  ["\uCE56", "\uCE56"], /* Hangul Syllable */
  ["\uCE57", "\uCE57"], /* Hangul Syllable */
  ["\uCE58", "\uCE58"], /* Hangul Syllable */
  ["\uCE59", "\uCE59"], /* Hangul Syllable */
  ["\uCE5A", "\uCE5A"], /* Hangul Syllable */
  ["\uCE5B", "\uCE5B"], /* Hangul Syllable */
  ["\uCE5C", "\uCE5C"], /* Hangul Syllable */
  ["\uCE5D", "\uCE5D"], /* Hangul Syllable */
  ["\uCE5E", "\uCE5E"], /* Hangul Syllable */
  ["\uCE5F", "\uCE5F"], /* Hangul Syllable */
  ["\uCE60", "\uCE60"], /* Hangul Syllable */
  ["\uCE61", "\uCE61"], /* Hangul Syllable */
  ["\uCE62", "\uCE62"], /* Hangul Syllable */
  ["\uCE63", "\uCE63"], /* Hangul Syllable */
  ["\uCE64", "\uCE64"], /* Hangul Syllable */
  ["\uCE65", "\uCE65"], /* Hangul Syllable */
  ["\uCE66", "\uCE66"], /* Hangul Syllable */
  ["\uCE67", "\uCE67"], /* Hangul Syllable */
  ["\uCE68", "\uCE68"], /* Hangul Syllable */
  ["\uCE69", "\uCE69"], /* Hangul Syllable */
  ["\uCE6A", "\uCE6A"], /* Hangul Syllable */
  ["\uCE6B", "\uCE6B"], /* Hangul Syllable */
  ["\uCE6C", "\uCE6C"], /* Hangul Syllable */
  ["\uCE6D", "\uCE6D"], /* Hangul Syllable */
  ["\uCE6E", "\uCE6E"], /* Hangul Syllable */
  ["\uCE6F", "\uCE6F"], /* Hangul Syllable */
  ["\uCE70", "\uCE70"], /* Hangul Syllable */
  ["\uCE71", "\uCE71"], /* Hangul Syllable */
  ["\uCE72", "\uCE72"], /* Hangul Syllable */
  ["\uCE73", "\uCE73"], /* Hangul Syllable */
  ["\uCE74", "\uCE74"], /* Hangul Syllable */
  ["\uCE75", "\uCE75"], /* Hangul Syllable */
  ["\uCE76", "\uCE76"], /* Hangul Syllable */
  ["\uCE77", "\uCE77"], /* Hangul Syllable */
  ["\uCE78", "\uCE78"], /* Hangul Syllable */
  ["\uCE79", "\uCE79"], /* Hangul Syllable */
  ["\uCE7A", "\uCE7A"], /* Hangul Syllable */
  ["\uCE7B", "\uCE7B"], /* Hangul Syllable */
  ["\uCE7C", "\uCE7C"], /* Hangul Syllable */
  ["\uCE7D", "\uCE7D"], /* Hangul Syllable */
  ["\uCE7E", "\uCE7E"], /* Hangul Syllable */
  ["\uCE7F", "\uCE7F"], /* Hangul Syllable */
  ["\uCE80", "\uCE80"], /* Hangul Syllable */
  ["\uCE81", "\uCE81"], /* Hangul Syllable */
  ["\uCE82", "\uCE82"], /* Hangul Syllable */
  ["\uCE83", "\uCE83"], /* Hangul Syllable */
  ["\uCE84", "\uCE84"], /* Hangul Syllable */
  ["\uCE85", "\uCE85"], /* Hangul Syllable */
  ["\uCE86", "\uCE86"], /* Hangul Syllable */
  ["\uCE87", "\uCE87"], /* Hangul Syllable */
  ["\uCE88", "\uCE88"], /* Hangul Syllable */
  ["\uCE89", "\uCE89"], /* Hangul Syllable */
  ["\uCE8A", "\uCE8A"], /* Hangul Syllable */
  ["\uCE8B", "\uCE8B"], /* Hangul Syllable */
  ["\uCE8C", "\uCE8C"], /* Hangul Syllable */
  ["\uCE8D", "\uCE8D"], /* Hangul Syllable */
  ["\uCE8E", "\uCE8E"], /* Hangul Syllable */
  ["\uCE8F", "\uCE8F"], /* Hangul Syllable */
  ["\uCE90", "\uCE90"], /* Hangul Syllable */
  ["\uCE91", "\uCE91"], /* Hangul Syllable */
  ["\uCE92", "\uCE92"], /* Hangul Syllable */
  ["\uCE93", "\uCE93"], /* Hangul Syllable */
  ["\uCE94", "\uCE94"], /* Hangul Syllable */
  ["\uCE95", "\uCE95"], /* Hangul Syllable */
  ["\uCE96", "\uCE96"], /* Hangul Syllable */
  ["\uCE97", "\uCE97"], /* Hangul Syllable */
  ["\uCE98", "\uCE98"], /* Hangul Syllable */
  ["\uCE99", "\uCE99"], /* Hangul Syllable */
  ["\uCE9A", "\uCE9A"], /* Hangul Syllable */
  ["\uCE9B", "\uCE9B"], /* Hangul Syllable */
  ["\uCE9C", "\uCE9C"], /* Hangul Syllable */
  ["\uCE9D", "\uCE9D"], /* Hangul Syllable */
  ["\uCE9E", "\uCE9E"], /* Hangul Syllable */
  ["\uCE9F", "\uCE9F"], /* Hangul Syllable */
  ["\uCEA0", "\uCEA0"], /* Hangul Syllable */
  ["\uCEA1", "\uCEA1"], /* Hangul Syllable */
  ["\uCEA2", "\uCEA2"], /* Hangul Syllable */
  ["\uCEA3", "\uCEA3"], /* Hangul Syllable */
  ["\uCEA4", "\uCEA4"], /* Hangul Syllable */
  ["\uCEA5", "\uCEA5"], /* Hangul Syllable */
  ["\uCEA6", "\uCEA6"], /* Hangul Syllable */
  ["\uCEA7", "\uCEA7"], /* Hangul Syllable */
  ["\uCEA8", "\uCEA8"], /* Hangul Syllable */
  ["\uCEA9", "\uCEA9"], /* Hangul Syllable */
  ["\uCEAA", "\uCEAA"], /* Hangul Syllable */
  ["\uCEAB", "\uCEAB"], /* Hangul Syllable */
  ["\uCEAC", "\uCEAC"], /* Hangul Syllable */
  ["\uCEAD", "\uCEAD"], /* Hangul Syllable */
  ["\uCEAE", "\uCEAE"], /* Hangul Syllable */
  ["\uCEAF", "\uCEAF"], /* Hangul Syllable */
  ["\uCEB0", "\uCEB0"], /* Hangul Syllable */
  ["\uCEB1", "\uCEB1"], /* Hangul Syllable */
  ["\uCEB2", "\uCEB2"], /* Hangul Syllable */
  ["\uCEB3", "\uCEB3"], /* Hangul Syllable */
  ["\uCEB4", "\uCEB4"], /* Hangul Syllable */
  ["\uCEB5", "\uCEB5"], /* Hangul Syllable */
  ["\uCEB6", "\uCEB6"], /* Hangul Syllable */
  ["\uCEB7", "\uCEB7"], /* Hangul Syllable */
  ["\uCEB8", "\uCEB8"], /* Hangul Syllable */
  ["\uCEB9", "\uCEB9"], /* Hangul Syllable */
  ["\uCEBA", "\uCEBA"], /* Hangul Syllable */
  ["\uCEBB", "\uCEBB"], /* Hangul Syllable */
  ["\uCEBC", "\uCEBC"], /* Hangul Syllable */
  ["\uCEBD", "\uCEBD"], /* Hangul Syllable */
  ["\uCEBE", "\uCEBE"], /* Hangul Syllable */
  ["\uCEBF", "\uCEBF"], /* Hangul Syllable */
  ["\uCEC0", "\uCEC0"], /* Hangul Syllable */
  ["\uCEC1", "\uCEC1"], /* Hangul Syllable */
  ["\uCEC2", "\uCEC2"], /* Hangul Syllable */
  ["\uCEC3", "\uCEC3"], /* Hangul Syllable */
  ["\uCEC4", "\uCEC4"], /* Hangul Syllable */
  ["\uCEC5", "\uCEC5"], /* Hangul Syllable */
  ["\uCEC6", "\uCEC6"], /* Hangul Syllable */
  ["\uCEC7", "\uCEC7"], /* Hangul Syllable */
  ["\uCEC8", "\uCEC8"], /* Hangul Syllable */
  ["\uCEC9", "\uCEC9"], /* Hangul Syllable */
  ["\uCECA", "\uCECA"], /* Hangul Syllable */
  ["\uCECB", "\uCECB"], /* Hangul Syllable */
  ["\uCECC", "\uCECC"], /* Hangul Syllable */
  ["\uCECD", "\uCECD"], /* Hangul Syllable */
  ["\uCECE", "\uCECE"], /* Hangul Syllable */
  ["\uCECF", "\uCECF"], /* Hangul Syllable */
  ["\uCED0", "\uCED0"], /* Hangul Syllable */
  ["\uCED1", "\uCED1"], /* Hangul Syllable */
  ["\uCED2", "\uCED2"], /* Hangul Syllable */
  ["\uCED3", "\uCED3"], /* Hangul Syllable */
  ["\uCED4", "\uCED4"], /* Hangul Syllable */
  ["\uCED5", "\uCED5"], /* Hangul Syllable */
  ["\uCED6", "\uCED6"], /* Hangul Syllable */
  ["\uCED7", "\uCED7"], /* Hangul Syllable */
  ["\uCED8", "\uCED8"], /* Hangul Syllable */
  ["\uCED9", "\uCED9"], /* Hangul Syllable */
  ["\uCEDA", "\uCEDA"], /* Hangul Syllable */
  ["\uCEDB", "\uCEDB"], /* Hangul Syllable */
  ["\uCEDC", "\uCEDC"], /* Hangul Syllable */
  ["\uCEDD", "\uCEDD"], /* Hangul Syllable */
  ["\uCEDE", "\uCEDE"], /* Hangul Syllable */
  ["\uCEDF", "\uCEDF"], /* Hangul Syllable */
  ["\uCEE0", "\uCEE0"], /* Hangul Syllable */
  ["\uCEE1", "\uCEE1"], /* Hangul Syllable */
  ["\uCEE2", "\uCEE2"], /* Hangul Syllable */
  ["\uCEE3", "\uCEE3"], /* Hangul Syllable */
  ["\uCEE4", "\uCEE4"], /* Hangul Syllable */
  ["\uCEE5", "\uCEE5"], /* Hangul Syllable */
  ["\uCEE6", "\uCEE6"], /* Hangul Syllable */
  ["\uCEE7", "\uCEE7"], /* Hangul Syllable */
  ["\uCEE8", "\uCEE8"], /* Hangul Syllable */
  ["\uCEE9", "\uCEE9"], /* Hangul Syllable */
  ["\uCEEA", "\uCEEA"], /* Hangul Syllable */
  ["\uCEEB", "\uCEEB"], /* Hangul Syllable */
  ["\uCEEC", "\uCEEC"], /* Hangul Syllable */
  ["\uCEED", "\uCEED"], /* Hangul Syllable */
  ["\uCEEE", "\uCEEE"], /* Hangul Syllable */
  ["\uCEEF", "\uCEEF"], /* Hangul Syllable */
  ["\uCEF0", "\uCEF0"], /* Hangul Syllable */
  ["\uCEF1", "\uCEF1"], /* Hangul Syllable */
  ["\uCEF2", "\uCEF2"], /* Hangul Syllable */
  ["\uCEF3", "\uCEF3"], /* Hangul Syllable */
  ["\uCEF4", "\uCEF4"], /* Hangul Syllable */
  ["\uCEF5", "\uCEF5"], /* Hangul Syllable */
  ["\uCEF6", "\uCEF6"], /* Hangul Syllable */
  ["\uCEF7", "\uCEF7"], /* Hangul Syllable */
  ["\uCEF8", "\uCEF8"], /* Hangul Syllable */
  ["\uCEF9", "\uCEF9"], /* Hangul Syllable */
  ["\uCEFA", "\uCEFA"], /* Hangul Syllable */
  ["\uCEFB", "\uCEFB"], /* Hangul Syllable */
  ["\uCEFC", "\uCEFC"], /* Hangul Syllable */
  ["\uCEFD", "\uCEFD"], /* Hangul Syllable */
  ["\uCEFE", "\uCEFE"], /* Hangul Syllable */
  ["\uCEFF", "\uCEFF"], /* Hangul Syllable */
  ["\uCF00", "\uCF00"], /* Hangul Syllable */
  ["\uCF01", "\uCF01"], /* Hangul Syllable */
  ["\uCF02", "\uCF02"], /* Hangul Syllable */
  ["\uCF03", "\uCF03"], /* Hangul Syllable */
  ["\uCF04", "\uCF04"], /* Hangul Syllable */
  ["\uCF05", "\uCF05"], /* Hangul Syllable */
  ["\uCF06", "\uCF06"], /* Hangul Syllable */
  ["\uCF07", "\uCF07"], /* Hangul Syllable */
  ["\uCF08", "\uCF08"], /* Hangul Syllable */
  ["\uCF09", "\uCF09"], /* Hangul Syllable */
  ["\uCF0A", "\uCF0A"], /* Hangul Syllable */
  ["\uCF0B", "\uCF0B"], /* Hangul Syllable */
  ["\uCF0C", "\uCF0C"], /* Hangul Syllable */
  ["\uCF0D", "\uCF0D"], /* Hangul Syllable */
  ["\uCF0E", "\uCF0E"], /* Hangul Syllable */
  ["\uCF0F", "\uCF0F"], /* Hangul Syllable */
  ["\uCF10", "\uCF10"], /* Hangul Syllable */
  ["\uCF11", "\uCF11"], /* Hangul Syllable */
  ["\uCF12", "\uCF12"], /* Hangul Syllable */
  ["\uCF13", "\uCF13"], /* Hangul Syllable */
  ["\uCF14", "\uCF14"], /* Hangul Syllable */
  ["\uCF15", "\uCF15"], /* Hangul Syllable */
  ["\uCF16", "\uCF16"], /* Hangul Syllable */
  ["\uCF17", "\uCF17"], /* Hangul Syllable */
  ["\uCF18", "\uCF18"], /* Hangul Syllable */
  ["\uCF19", "\uCF19"], /* Hangul Syllable */
  ["\uCF1A", "\uCF1A"], /* Hangul Syllable */
  ["\uCF1B", "\uCF1B"], /* Hangul Syllable */
  ["\uCF1C", "\uCF1C"], /* Hangul Syllable */
  ["\uCF1D", "\uCF1D"], /* Hangul Syllable */
  ["\uCF1E", "\uCF1E"], /* Hangul Syllable */
  ["\uCF1F", "\uCF1F"], /* Hangul Syllable */
  ["\uCF20", "\uCF20"], /* Hangul Syllable */
  ["\uCF21", "\uCF21"], /* Hangul Syllable */
  ["\uCF22", "\uCF22"], /* Hangul Syllable */
  ["\uCF23", "\uCF23"], /* Hangul Syllable */
  ["\uCF24", "\uCF24"], /* Hangul Syllable */
  ["\uCF25", "\uCF25"], /* Hangul Syllable */
  ["\uCF26", "\uCF26"], /* Hangul Syllable */
  ["\uCF27", "\uCF27"], /* Hangul Syllable */
  ["\uCF28", "\uCF28"], /* Hangul Syllable */
  ["\uCF29", "\uCF29"], /* Hangul Syllable */
  ["\uCF2A", "\uCF2A"], /* Hangul Syllable */
  ["\uCF2B", "\uCF2B"], /* Hangul Syllable */
  ["\uCF2C", "\uCF2C"], /* Hangul Syllable */
  ["\uCF2D", "\uCF2D"], /* Hangul Syllable */
  ["\uCF2E", "\uCF2E"], /* Hangul Syllable */
  ["\uCF2F", "\uCF2F"], /* Hangul Syllable */
  ["\uCF30", "\uCF30"], /* Hangul Syllable */
  ["\uCF31", "\uCF31"], /* Hangul Syllable */
  ["\uCF32", "\uCF32"], /* Hangul Syllable */
  ["\uCF33", "\uCF33"], /* Hangul Syllable */
  ["\uCF34", "\uCF34"], /* Hangul Syllable */
  ["\uCF35", "\uCF35"], /* Hangul Syllable */
  ["\uCF36", "\uCF36"], /* Hangul Syllable */
  ["\uCF37", "\uCF37"], /* Hangul Syllable */
  ["\uCF38", "\uCF38"], /* Hangul Syllable */
  ["\uCF39", "\uCF39"], /* Hangul Syllable */
  ["\uCF3A", "\uCF3A"], /* Hangul Syllable */
  ["\uCF3B", "\uCF3B"], /* Hangul Syllable */
  ["\uCF3C", "\uCF3C"], /* Hangul Syllable */
  ["\uCF3D", "\uCF3D"], /* Hangul Syllable */
  ["\uCF3E", "\uCF3E"], /* Hangul Syllable */
  ["\uCF3F", "\uCF3F"], /* Hangul Syllable */
  ["\uCF40", "\uCF40"], /* Hangul Syllable */
  ["\uCF41", "\uCF41"], /* Hangul Syllable */
  ["\uCF42", "\uCF42"], /* Hangul Syllable */
  ["\uCF43", "\uCF43"], /* Hangul Syllable */
  ["\uCF44", "\uCF44"], /* Hangul Syllable */
  ["\uCF45", "\uCF45"], /* Hangul Syllable */
  ["\uCF46", "\uCF46"], /* Hangul Syllable */
  ["\uCF47", "\uCF47"], /* Hangul Syllable */
  ["\uCF48", "\uCF48"], /* Hangul Syllable */
  ["\uCF49", "\uCF49"], /* Hangul Syllable */
  ["\uCF4A", "\uCF4A"], /* Hangul Syllable */
  ["\uCF4B", "\uCF4B"], /* Hangul Syllable */
  ["\uCF4C", "\uCF4C"], /* Hangul Syllable */
  ["\uCF4D", "\uCF4D"], /* Hangul Syllable */
  ["\uCF4E", "\uCF4E"], /* Hangul Syllable */
  ["\uCF4F", "\uCF4F"], /* Hangul Syllable */
  ["\uCF50", "\uCF50"], /* Hangul Syllable */
  ["\uCF51", "\uCF51"], /* Hangul Syllable */
  ["\uCF52", "\uCF52"], /* Hangul Syllable */
  ["\uCF53", "\uCF53"], /* Hangul Syllable */
  ["\uCF54", "\uCF54"], /* Hangul Syllable */
  ["\uCF55", "\uCF55"], /* Hangul Syllable */
  ["\uCF56", "\uCF56"], /* Hangul Syllable */
  ["\uCF57", "\uCF57"], /* Hangul Syllable */
  ["\uCF58", "\uCF58"], /* Hangul Syllable */
  ["\uCF59", "\uCF59"], /* Hangul Syllable */
  ["\uCF5A", "\uCF5A"], /* Hangul Syllable */
  ["\uCF5B", "\uCF5B"], /* Hangul Syllable */
  ["\uCF5C", "\uCF5C"], /* Hangul Syllable */
  ["\uCF5D", "\uCF5D"], /* Hangul Syllable */
  ["\uCF5E", "\uCF5E"], /* Hangul Syllable */
  ["\uCF5F", "\uCF5F"], /* Hangul Syllable */
  ["\uCF60", "\uCF60"], /* Hangul Syllable */
  ["\uCF61", "\uCF61"], /* Hangul Syllable */
  ["\uCF62", "\uCF62"], /* Hangul Syllable */
  ["\uCF63", "\uCF63"], /* Hangul Syllable */
  ["\uCF64", "\uCF64"], /* Hangul Syllable */
  ["\uCF65", "\uCF65"], /* Hangul Syllable */
  ["\uCF66", "\uCF66"], /* Hangul Syllable */
  ["\uCF67", "\uCF67"], /* Hangul Syllable */
  ["\uCF68", "\uCF68"], /* Hangul Syllable */
  ["\uCF69", "\uCF69"], /* Hangul Syllable */
  ["\uCF6A", "\uCF6A"], /* Hangul Syllable */
  ["\uCF6B", "\uCF6B"], /* Hangul Syllable */
  ["\uCF6C", "\uCF6C"], /* Hangul Syllable */
  ["\uCF6D", "\uCF6D"], /* Hangul Syllable */
  ["\uCF6E", "\uCF6E"], /* Hangul Syllable */
  ["\uCF6F", "\uCF6F"], /* Hangul Syllable */
  ["\uCF70", "\uCF70"], /* Hangul Syllable */
  ["\uCF71", "\uCF71"], /* Hangul Syllable */
  ["\uCF72", "\uCF72"], /* Hangul Syllable */
  ["\uCF73", "\uCF73"], /* Hangul Syllable */
  ["\uCF74", "\uCF74"], /* Hangul Syllable */
  ["\uCF75", "\uCF75"], /* Hangul Syllable */
  ["\uCF76", "\uCF76"], /* Hangul Syllable */
  ["\uCF77", "\uCF77"], /* Hangul Syllable */
  ["\uCF78", "\uCF78"], /* Hangul Syllable */
  ["\uCF79", "\uCF79"], /* Hangul Syllable */
  ["\uCF7A", "\uCF7A"], /* Hangul Syllable */
  ["\uCF7B", "\uCF7B"], /* Hangul Syllable */
  ["\uCF7C", "\uCF7C"], /* Hangul Syllable */
  ["\uCF7D", "\uCF7D"], /* Hangul Syllable */
  ["\uCF7E", "\uCF7E"], /* Hangul Syllable */
  ["\uCF7F", "\uCF7F"], /* Hangul Syllable */
  ["\uCF80", "\uCF80"], /* Hangul Syllable */
  ["\uCF81", "\uCF81"], /* Hangul Syllable */
  ["\uCF82", "\uCF82"], /* Hangul Syllable */
  ["\uCF83", "\uCF83"], /* Hangul Syllable */
  ["\uCF84", "\uCF84"], /* Hangul Syllable */
  ["\uCF85", "\uCF85"], /* Hangul Syllable */
  ["\uCF86", "\uCF86"], /* Hangul Syllable */
  ["\uCF87", "\uCF87"], /* Hangul Syllable */
  ["\uCF88", "\uCF88"], /* Hangul Syllable */
  ["\uCF89", "\uCF89"], /* Hangul Syllable */
  ["\uCF8A", "\uCF8A"], /* Hangul Syllable */
  ["\uCF8B", "\uCF8B"], /* Hangul Syllable */
  ["\uCF8C", "\uCF8C"], /* Hangul Syllable */
  ["\uCF8D", "\uCF8D"], /* Hangul Syllable */
  ["\uCF8E", "\uCF8E"], /* Hangul Syllable */
  ["\uCF8F", "\uCF8F"], /* Hangul Syllable */
  ["\uCF90", "\uCF90"], /* Hangul Syllable */
  ["\uCF91", "\uCF91"], /* Hangul Syllable */
  ["\uCF92", "\uCF92"], /* Hangul Syllable */
  ["\uCF93", "\uCF93"], /* Hangul Syllable */
  ["\uCF94", "\uCF94"], /* Hangul Syllable */
  ["\uCF95", "\uCF95"], /* Hangul Syllable */
  ["\uCF96", "\uCF96"], /* Hangul Syllable */
  ["\uCF97", "\uCF97"], /* Hangul Syllable */
  ["\uCF98", "\uCF98"], /* Hangul Syllable */
  ["\uCF99", "\uCF99"], /* Hangul Syllable */
  ["\uCF9A", "\uCF9A"], /* Hangul Syllable */
  ["\uCF9B", "\uCF9B"], /* Hangul Syllable */
  ["\uCF9C", "\uCF9C"], /* Hangul Syllable */
  ["\uCF9D", "\uCF9D"], /* Hangul Syllable */
  ["\uCF9E", "\uCF9E"], /* Hangul Syllable */
  ["\uCF9F", "\uCF9F"], /* Hangul Syllable */
  ["\uCFA0", "\uCFA0"], /* Hangul Syllable */
  ["\uCFA1", "\uCFA1"], /* Hangul Syllable */
  ["\uCFA2", "\uCFA2"], /* Hangul Syllable */
  ["\uCFA3", "\uCFA3"], /* Hangul Syllable */
  ["\uCFA4", "\uCFA4"], /* Hangul Syllable */
  ["\uCFA5", "\uCFA5"], /* Hangul Syllable */
  ["\uCFA6", "\uCFA6"], /* Hangul Syllable */
  ["\uCFA7", "\uCFA7"], /* Hangul Syllable */
  ["\uCFA8", "\uCFA8"], /* Hangul Syllable */
  ["\uCFA9", "\uCFA9"], /* Hangul Syllable */
  ["\uCFAA", "\uCFAA"], /* Hangul Syllable */
  ["\uCFAB", "\uCFAB"], /* Hangul Syllable */
  ["\uCFAC", "\uCFAC"], /* Hangul Syllable */
  ["\uCFAD", "\uCFAD"], /* Hangul Syllable */
  ["\uCFAE", "\uCFAE"], /* Hangul Syllable */
  ["\uCFAF", "\uCFAF"], /* Hangul Syllable */
  ["\uCFB0", "\uCFB0"], /* Hangul Syllable */
  ["\uCFB1", "\uCFB1"], /* Hangul Syllable */
  ["\uCFB2", "\uCFB2"], /* Hangul Syllable */
  ["\uCFB3", "\uCFB3"], /* Hangul Syllable */
  ["\uCFB4", "\uCFB4"], /* Hangul Syllable */
  ["\uCFB5", "\uCFB5"], /* Hangul Syllable */
  ["\uCFB6", "\uCFB6"], /* Hangul Syllable */
  ["\uCFB7", "\uCFB7"], /* Hangul Syllable */
  ["\uCFB8", "\uCFB8"], /* Hangul Syllable */
  ["\uCFB9", "\uCFB9"], /* Hangul Syllable */
  ["\uCFBA", "\uCFBA"], /* Hangul Syllable */
  ["\uCFBB", "\uCFBB"], /* Hangul Syllable */
  ["\uCFBC", "\uCFBC"], /* Hangul Syllable */
  ["\uCFBD", "\uCFBD"], /* Hangul Syllable */
  ["\uCFBE", "\uCFBE"], /* Hangul Syllable */
  ["\uCFBF", "\uCFBF"], /* Hangul Syllable */
  ["\uCFC0", "\uCFC0"], /* Hangul Syllable */
  ["\uCFC1", "\uCFC1"], /* Hangul Syllable */
  ["\uCFC2", "\uCFC2"], /* Hangul Syllable */
  ["\uCFC3", "\uCFC3"], /* Hangul Syllable */
  ["\uCFC4", "\uCFC4"], /* Hangul Syllable */
  ["\uCFC5", "\uCFC5"], /* Hangul Syllable */
  ["\uCFC6", "\uCFC6"], /* Hangul Syllable */
  ["\uCFC7", "\uCFC7"], /* Hangul Syllable */
  ["\uCFC8", "\uCFC8"], /* Hangul Syllable */
  ["\uCFC9", "\uCFC9"], /* Hangul Syllable */
  ["\uCFCA", "\uCFCA"], /* Hangul Syllable */
  ["\uCFCB", "\uCFCB"], /* Hangul Syllable */
  ["\uCFCC", "\uCFCC"], /* Hangul Syllable */
  ["\uCFCD", "\uCFCD"], /* Hangul Syllable */
  ["\uCFCE", "\uCFCE"], /* Hangul Syllable */
  ["\uCFCF", "\uCFCF"], /* Hangul Syllable */
  ["\uCFD0", "\uCFD0"], /* Hangul Syllable */
  ["\uCFD1", "\uCFD1"], /* Hangul Syllable */
  ["\uCFD2", "\uCFD2"], /* Hangul Syllable */
  ["\uCFD3", "\uCFD3"], /* Hangul Syllable */
  ["\uCFD4", "\uCFD4"], /* Hangul Syllable */
  ["\uCFD5", "\uCFD5"], /* Hangul Syllable */
  ["\uCFD6", "\uCFD6"], /* Hangul Syllable */
  ["\uCFD7", "\uCFD7"], /* Hangul Syllable */
  ["\uCFD8", "\uCFD8"], /* Hangul Syllable */
  ["\uCFD9", "\uCFD9"], /* Hangul Syllable */
  ["\uCFDA", "\uCFDA"], /* Hangul Syllable */
  ["\uCFDB", "\uCFDB"], /* Hangul Syllable */
  ["\uCFDC", "\uCFDC"], /* Hangul Syllable */
  ["\uCFDD", "\uCFDD"], /* Hangul Syllable */
  ["\uCFDE", "\uCFDE"], /* Hangul Syllable */
  ["\uCFDF", "\uCFDF"], /* Hangul Syllable */
  ["\uCFE0", "\uCFE0"], /* Hangul Syllable */
  ["\uCFE1", "\uCFE1"], /* Hangul Syllable */
  ["\uCFE2", "\uCFE2"], /* Hangul Syllable */
  ["\uCFE3", "\uCFE3"], /* Hangul Syllable */
  ["\uCFE4", "\uCFE4"], /* Hangul Syllable */
  ["\uCFE5", "\uCFE5"], /* Hangul Syllable */
  ["\uCFE6", "\uCFE6"], /* Hangul Syllable */
  ["\uCFE7", "\uCFE7"], /* Hangul Syllable */
  ["\uCFE8", "\uCFE8"], /* Hangul Syllable */
  ["\uCFE9", "\uCFE9"], /* Hangul Syllable */
  ["\uCFEA", "\uCFEA"], /* Hangul Syllable */
  ["\uCFEB", "\uCFEB"], /* Hangul Syllable */
  ["\uCFEC", "\uCFEC"], /* Hangul Syllable */
  ["\uCFED", "\uCFED"], /* Hangul Syllable */
  ["\uCFEE", "\uCFEE"], /* Hangul Syllable */
  ["\uCFEF", "\uCFEF"], /* Hangul Syllable */
  ["\uCFF0", "\uCFF0"], /* Hangul Syllable */
  ["\uCFF1", "\uCFF1"], /* Hangul Syllable */
  ["\uCFF2", "\uCFF2"], /* Hangul Syllable */
  ["\uCFF3", "\uCFF3"], /* Hangul Syllable */
  ["\uCFF4", "\uCFF4"], /* Hangul Syllable */
  ["\uCFF5", "\uCFF5"], /* Hangul Syllable */
  ["\uCFF6", "\uCFF6"], /* Hangul Syllable */
  ["\uCFF7", "\uCFF7"], /* Hangul Syllable */
  ["\uCFF8", "\uCFF8"], /* Hangul Syllable */
  ["\uCFF9", "\uCFF9"], /* Hangul Syllable */
  ["\uCFFA", "\uCFFA"], /* Hangul Syllable */
  ["\uCFFB", "\uCFFB"], /* Hangul Syllable */
  ["\uCFFC", "\uCFFC"], /* Hangul Syllable */
  ["\uCFFD", "\uCFFD"], /* Hangul Syllable */
  ["\uCFFE", "\uCFFE"], /* Hangul Syllable */
  ["\uCFFF", "\uCFFF"], /* Hangul Syllable */
  ["\uD000", "\uD000"], /* Hangul Syllable */
  ["\uD001", "\uD001"], /* Hangul Syllable */
  ["\uD002", "\uD002"], /* Hangul Syllable */
  ["\uD003", "\uD003"], /* Hangul Syllable */
  ["\uD004", "\uD004"], /* Hangul Syllable */
  ["\uD005", "\uD005"], /* Hangul Syllable */
  ["\uD006", "\uD006"], /* Hangul Syllable */
  ["\uD007", "\uD007"], /* Hangul Syllable */
  ["\uD008", "\uD008"], /* Hangul Syllable */
  ["\uD009", "\uD009"], /* Hangul Syllable */
  ["\uD00A", "\uD00A"], /* Hangul Syllable */
  ["\uD00B", "\uD00B"], /* Hangul Syllable */
  ["\uD00C", "\uD00C"], /* Hangul Syllable */
  ["\uD00D", "\uD00D"], /* Hangul Syllable */
  ["\uD00E", "\uD00E"], /* Hangul Syllable */
  ["\uD00F", "\uD00F"], /* Hangul Syllable */
  ["\uD010", "\uD010"], /* Hangul Syllable */
  ["\uD011", "\uD011"], /* Hangul Syllable */
  ["\uD012", "\uD012"], /* Hangul Syllable */
  ["\uD013", "\uD013"], /* Hangul Syllable */
  ["\uD014", "\uD014"], /* Hangul Syllable */
  ["\uD015", "\uD015"], /* Hangul Syllable */
  ["\uD016", "\uD016"], /* Hangul Syllable */
  ["\uD017", "\uD017"], /* Hangul Syllable */
  ["\uD018", "\uD018"], /* Hangul Syllable */
  ["\uD019", "\uD019"], /* Hangul Syllable */
  ["\uD01A", "\uD01A"], /* Hangul Syllable */
  ["\uD01B", "\uD01B"], /* Hangul Syllable */
  ["\uD01C", "\uD01C"], /* Hangul Syllable */
  ["\uD01D", "\uD01D"], /* Hangul Syllable */
  ["\uD01E", "\uD01E"], /* Hangul Syllable */
  ["\uD01F", "\uD01F"], /* Hangul Syllable */
  ["\uD020", "\uD020"], /* Hangul Syllable */
  ["\uD021", "\uD021"], /* Hangul Syllable */
  ["\uD022", "\uD022"], /* Hangul Syllable */
  ["\uD023", "\uD023"], /* Hangul Syllable */
  ["\uD024", "\uD024"], /* Hangul Syllable */
  ["\uD025", "\uD025"], /* Hangul Syllable */
  ["\uD026", "\uD026"], /* Hangul Syllable */
  ["\uD027", "\uD027"], /* Hangul Syllable */
  ["\uD028", "\uD028"], /* Hangul Syllable */
  ["\uD029", "\uD029"], /* Hangul Syllable */
  ["\uD02A", "\uD02A"], /* Hangul Syllable */
  ["\uD02B", "\uD02B"], /* Hangul Syllable */
  ["\uD02C", "\uD02C"], /* Hangul Syllable */
  ["\uD02D", "\uD02D"], /* Hangul Syllable */
  ["\uD02E", "\uD02E"], /* Hangul Syllable */
  ["\uD02F", "\uD02F"], /* Hangul Syllable */
  ["\uD030", "\uD030"], /* Hangul Syllable */
  ["\uD031", "\uD031"], /* Hangul Syllable */
  ["\uD032", "\uD032"], /* Hangul Syllable */
  ["\uD033", "\uD033"], /* Hangul Syllable */
  ["\uD034", "\uD034"], /* Hangul Syllable */
  ["\uD035", "\uD035"], /* Hangul Syllable */
  ["\uD036", "\uD036"], /* Hangul Syllable */
  ["\uD037", "\uD037"], /* Hangul Syllable */
  ["\uD038", "\uD038"], /* Hangul Syllable */
  ["\uD039", "\uD039"], /* Hangul Syllable */
  ["\uD03A", "\uD03A"], /* Hangul Syllable */
  ["\uD03B", "\uD03B"], /* Hangul Syllable */
  ["\uD03C", "\uD03C"], /* Hangul Syllable */
  ["\uD03D", "\uD03D"], /* Hangul Syllable */
  ["\uD03E", "\uD03E"], /* Hangul Syllable */
  ["\uD03F", "\uD03F"], /* Hangul Syllable */
  ["\uD040", "\uD040"], /* Hangul Syllable */
  ["\uD041", "\uD041"], /* Hangul Syllable */
  ["\uD042", "\uD042"], /* Hangul Syllable */
  ["\uD043", "\uD043"], /* Hangul Syllable */
  ["\uD044", "\uD044"], /* Hangul Syllable */
  ["\uD045", "\uD045"], /* Hangul Syllable */
  ["\uD046", "\uD046"], /* Hangul Syllable */
  ["\uD047", "\uD047"], /* Hangul Syllable */
  ["\uD048", "\uD048"], /* Hangul Syllable */
  ["\uD049", "\uD049"], /* Hangul Syllable */
  ["\uD04A", "\uD04A"], /* Hangul Syllable */
  ["\uD04B", "\uD04B"], /* Hangul Syllable */
  ["\uD04C", "\uD04C"], /* Hangul Syllable */
  ["\uD04D", "\uD04D"], /* Hangul Syllable */
  ["\uD04E", "\uD04E"], /* Hangul Syllable */
  ["\uD04F", "\uD04F"], /* Hangul Syllable */
  ["\uD050", "\uD050"], /* Hangul Syllable */
  ["\uD051", "\uD051"], /* Hangul Syllable */
  ["\uD052", "\uD052"], /* Hangul Syllable */
  ["\uD053", "\uD053"], /* Hangul Syllable */
  ["\uD054", "\uD054"], /* Hangul Syllable */
  ["\uD055", "\uD055"], /* Hangul Syllable */
  ["\uD056", "\uD056"], /* Hangul Syllable */
  ["\uD057", "\uD057"], /* Hangul Syllable */
  ["\uD058", "\uD058"], /* Hangul Syllable */
  ["\uD059", "\uD059"], /* Hangul Syllable */
  ["\uD05A", "\uD05A"], /* Hangul Syllable */
  ["\uD05B", "\uD05B"], /* Hangul Syllable */
  ["\uD05C", "\uD05C"], /* Hangul Syllable */
  ["\uD05D", "\uD05D"], /* Hangul Syllable */
  ["\uD05E", "\uD05E"], /* Hangul Syllable */
  ["\uD05F", "\uD05F"], /* Hangul Syllable */
  ["\uD060", "\uD060"], /* Hangul Syllable */
  ["\uD061", "\uD061"], /* Hangul Syllable */
  ["\uD062", "\uD062"], /* Hangul Syllable */
  ["\uD063", "\uD063"], /* Hangul Syllable */
  ["\uD064", "\uD064"], /* Hangul Syllable */
  ["\uD065", "\uD065"], /* Hangul Syllable */
  ["\uD066", "\uD066"], /* Hangul Syllable */
  ["\uD067", "\uD067"], /* Hangul Syllable */
  ["\uD068", "\uD068"], /* Hangul Syllable */
  ["\uD069", "\uD069"], /* Hangul Syllable */
  ["\uD06A", "\uD06A"], /* Hangul Syllable */
  ["\uD06B", "\uD06B"], /* Hangul Syllable */
  ["\uD06C", "\uD06C"], /* Hangul Syllable */
  ["\uD06D", "\uD06D"], /* Hangul Syllable */
  ["\uD06E", "\uD06E"], /* Hangul Syllable */
  ["\uD06F", "\uD06F"], /* Hangul Syllable */
  ["\uD070", "\uD070"], /* Hangul Syllable */
  ["\uD071", "\uD071"], /* Hangul Syllable */
  ["\uD072", "\uD072"], /* Hangul Syllable */
  ["\uD073", "\uD073"], /* Hangul Syllable */
  ["\uD074", "\uD074"], /* Hangul Syllable */
  ["\uD075", "\uD075"], /* Hangul Syllable */
  ["\uD076", "\uD076"], /* Hangul Syllable */
  ["\uD077", "\uD077"], /* Hangul Syllable */
  ["\uD078", "\uD078"], /* Hangul Syllable */
  ["\uD079", "\uD079"], /* Hangul Syllable */
  ["\uD07A", "\uD07A"], /* Hangul Syllable */
  ["\uD07B", "\uD07B"], /* Hangul Syllable */
  ["\uD07C", "\uD07C"], /* Hangul Syllable */
  ["\uD07D", "\uD07D"], /* Hangul Syllable */
  ["\uD07E", "\uD07E"], /* Hangul Syllable */
  ["\uD07F", "\uD07F"], /* Hangul Syllable */
  ["\uD080", "\uD080"], /* Hangul Syllable */
  ["\uD081", "\uD081"], /* Hangul Syllable */
  ["\uD082", "\uD082"], /* Hangul Syllable */
  ["\uD083", "\uD083"], /* Hangul Syllable */
  ["\uD084", "\uD084"], /* Hangul Syllable */
  ["\uD085", "\uD085"], /* Hangul Syllable */
  ["\uD086", "\uD086"], /* Hangul Syllable */
  ["\uD087", "\uD087"], /* Hangul Syllable */
  ["\uD088", "\uD088"], /* Hangul Syllable */
  ["\uD089", "\uD089"], /* Hangul Syllable */
  ["\uD08A", "\uD08A"], /* Hangul Syllable */
  ["\uD08B", "\uD08B"], /* Hangul Syllable */
  ["\uD08C", "\uD08C"], /* Hangul Syllable */
  ["\uD08D", "\uD08D"], /* Hangul Syllable */
  ["\uD08E", "\uD08E"], /* Hangul Syllable */
  ["\uD08F", "\uD08F"], /* Hangul Syllable */
  ["\uD090", "\uD090"], /* Hangul Syllable */
  ["\uD091", "\uD091"], /* Hangul Syllable */
  ["\uD092", "\uD092"], /* Hangul Syllable */
  ["\uD093", "\uD093"], /* Hangul Syllable */
  ["\uD094", "\uD094"], /* Hangul Syllable */
  ["\uD095", "\uD095"], /* Hangul Syllable */
  ["\uD096", "\uD096"], /* Hangul Syllable */
  ["\uD097", "\uD097"], /* Hangul Syllable */
  ["\uD098", "\uD098"], /* Hangul Syllable */
  ["\uD099", "\uD099"], /* Hangul Syllable */
  ["\uD09A", "\uD09A"], /* Hangul Syllable */
  ["\uD09B", "\uD09B"], /* Hangul Syllable */
  ["\uD09C", "\uD09C"], /* Hangul Syllable */
  ["\uD09D", "\uD09D"], /* Hangul Syllable */
  ["\uD09E", "\uD09E"], /* Hangul Syllable */
  ["\uD09F", "\uD09F"], /* Hangul Syllable */
  ["\uD0A0", "\uD0A0"], /* Hangul Syllable */
  ["\uD0A1", "\uD0A1"], /* Hangul Syllable */
  ["\uD0A2", "\uD0A2"], /* Hangul Syllable */
  ["\uD0A3", "\uD0A3"], /* Hangul Syllable */
  ["\uD0A4", "\uD0A4"], /* Hangul Syllable */
  ["\uD0A5", "\uD0A5"], /* Hangul Syllable */
  ["\uD0A6", "\uD0A6"], /* Hangul Syllable */
  ["\uD0A7", "\uD0A7"], /* Hangul Syllable */
  ["\uD0A8", "\uD0A8"], /* Hangul Syllable */
  ["\uD0A9", "\uD0A9"], /* Hangul Syllable */
  ["\uD0AA", "\uD0AA"], /* Hangul Syllable */
  ["\uD0AB", "\uD0AB"], /* Hangul Syllable */
  ["\uD0AC", "\uD0AC"], /* Hangul Syllable */
  ["\uD0AD", "\uD0AD"], /* Hangul Syllable */
  ["\uD0AE", "\uD0AE"], /* Hangul Syllable */
  ["\uD0AF", "\uD0AF"], /* Hangul Syllable */
  ["\uD0B0", "\uD0B0"], /* Hangul Syllable */
  ["\uD0B1", "\uD0B1"], /* Hangul Syllable */
  ["\uD0B2", "\uD0B2"], /* Hangul Syllable */
  ["\uD0B3", "\uD0B3"], /* Hangul Syllable */
  ["\uD0B4", "\uD0B4"], /* Hangul Syllable */
  ["\uD0B5", "\uD0B5"], /* Hangul Syllable */
  ["\uD0B6", "\uD0B6"], /* Hangul Syllable */
  ["\uD0B7", "\uD0B7"], /* Hangul Syllable */
  ["\uD0B8", "\uD0B8"], /* Hangul Syllable */
  ["\uD0B9", "\uD0B9"], /* Hangul Syllable */
  ["\uD0BA", "\uD0BA"], /* Hangul Syllable */
  ["\uD0BB", "\uD0BB"], /* Hangul Syllable */
  ["\uD0BC", "\uD0BC"], /* Hangul Syllable */
  ["\uD0BD", "\uD0BD"], /* Hangul Syllable */
  ["\uD0BE", "\uD0BE"], /* Hangul Syllable */
  ["\uD0BF", "\uD0BF"], /* Hangul Syllable */
  ["\uD0C0", "\uD0C0"], /* Hangul Syllable */
  ["\uD0C1", "\uD0C1"], /* Hangul Syllable */
  ["\uD0C2", "\uD0C2"], /* Hangul Syllable */
  ["\uD0C3", "\uD0C3"], /* Hangul Syllable */
  ["\uD0C4", "\uD0C4"], /* Hangul Syllable */
  ["\uD0C5", "\uD0C5"], /* Hangul Syllable */
  ["\uD0C6", "\uD0C6"], /* Hangul Syllable */
  ["\uD0C7", "\uD0C7"], /* Hangul Syllable */
  ["\uD0C8", "\uD0C8"], /* Hangul Syllable */
  ["\uD0C9", "\uD0C9"], /* Hangul Syllable */
  ["\uD0CA", "\uD0CA"], /* Hangul Syllable */
  ["\uD0CB", "\uD0CB"], /* Hangul Syllable */
  ["\uD0CC", "\uD0CC"], /* Hangul Syllable */
  ["\uD0CD", "\uD0CD"], /* Hangul Syllable */
  ["\uD0CE", "\uD0CE"], /* Hangul Syllable */
  ["\uD0CF", "\uD0CF"], /* Hangul Syllable */
  ["\uD0D0", "\uD0D0"], /* Hangul Syllable */
  ["\uD0D1", "\uD0D1"], /* Hangul Syllable */
  ["\uD0D2", "\uD0D2"], /* Hangul Syllable */
  ["\uD0D3", "\uD0D3"], /* Hangul Syllable */
  ["\uD0D4", "\uD0D4"], /* Hangul Syllable */
  ["\uD0D5", "\uD0D5"], /* Hangul Syllable */
  ["\uD0D6", "\uD0D6"], /* Hangul Syllable */
  ["\uD0D7", "\uD0D7"], /* Hangul Syllable */
  ["\uD0D8", "\uD0D8"], /* Hangul Syllable */
  ["\uD0D9", "\uD0D9"], /* Hangul Syllable */
  ["\uD0DA", "\uD0DA"], /* Hangul Syllable */
  ["\uD0DB", "\uD0DB"], /* Hangul Syllable */
  ["\uD0DC", "\uD0DC"], /* Hangul Syllable */
  ["\uD0DD", "\uD0DD"], /* Hangul Syllable */
  ["\uD0DE", "\uD0DE"], /* Hangul Syllable */
  ["\uD0DF", "\uD0DF"], /* Hangul Syllable */
  ["\uD0E0", "\uD0E0"], /* Hangul Syllable */
  ["\uD0E1", "\uD0E1"], /* Hangul Syllable */
  ["\uD0E2", "\uD0E2"], /* Hangul Syllable */
  ["\uD0E3", "\uD0E3"], /* Hangul Syllable */
  ["\uD0E4", "\uD0E4"], /* Hangul Syllable */
  ["\uD0E5", "\uD0E5"], /* Hangul Syllable */
  ["\uD0E6", "\uD0E6"], /* Hangul Syllable */
  ["\uD0E7", "\uD0E7"], /* Hangul Syllable */
  ["\uD0E8", "\uD0E8"], /* Hangul Syllable */
  ["\uD0E9", "\uD0E9"], /* Hangul Syllable */
  ["\uD0EA", "\uD0EA"], /* Hangul Syllable */
  ["\uD0EB", "\uD0EB"], /* Hangul Syllable */
  ["\uD0EC", "\uD0EC"], /* Hangul Syllable */
  ["\uD0ED", "\uD0ED"], /* Hangul Syllable */
  ["\uD0EE", "\uD0EE"], /* Hangul Syllable */
  ["\uD0EF", "\uD0EF"], /* Hangul Syllable */
  ["\uD0F0", "\uD0F0"], /* Hangul Syllable */
  ["\uD0F1", "\uD0F1"], /* Hangul Syllable */
  ["\uD0F2", "\uD0F2"], /* Hangul Syllable */
  ["\uD0F3", "\uD0F3"], /* Hangul Syllable */
  ["\uD0F4", "\uD0F4"], /* Hangul Syllable */
  ["\uD0F5", "\uD0F5"], /* Hangul Syllable */
  ["\uD0F6", "\uD0F6"], /* Hangul Syllable */
  ["\uD0F7", "\uD0F7"], /* Hangul Syllable */
  ["\uD0F8", "\uD0F8"], /* Hangul Syllable */
  ["\uD0F9", "\uD0F9"], /* Hangul Syllable */
  ["\uD0FA", "\uD0FA"], /* Hangul Syllable */
  ["\uD0FB", "\uD0FB"], /* Hangul Syllable */
  ["\uD0FC", "\uD0FC"], /* Hangul Syllable */
  ["\uD0FD", "\uD0FD"], /* Hangul Syllable */
  ["\uD0FE", "\uD0FE"], /* Hangul Syllable */
  ["\uD0FF", "\uD0FF"], /* Hangul Syllable */
  ["\uD100", "\uD100"], /* Hangul Syllable */
  ["\uD101", "\uD101"], /* Hangul Syllable */
  ["\uD102", "\uD102"], /* Hangul Syllable */
  ["\uD103", "\uD103"], /* Hangul Syllable */
  ["\uD104", "\uD104"], /* Hangul Syllable */
  ["\uD105", "\uD105"], /* Hangul Syllable */
  ["\uD106", "\uD106"], /* Hangul Syllable */
  ["\uD107", "\uD107"], /* Hangul Syllable */
  ["\uD108", "\uD108"], /* Hangul Syllable */
  ["\uD109", "\uD109"], /* Hangul Syllable */
  ["\uD10A", "\uD10A"], /* Hangul Syllable */
  ["\uD10B", "\uD10B"], /* Hangul Syllable */
  ["\uD10C", "\uD10C"], /* Hangul Syllable */
  ["\uD10D", "\uD10D"], /* Hangul Syllable */
  ["\uD10E", "\uD10E"], /* Hangul Syllable */
  ["\uD10F", "\uD10F"], /* Hangul Syllable */
  ["\uD110", "\uD110"], /* Hangul Syllable */
  ["\uD111", "\uD111"], /* Hangul Syllable */
  ["\uD112", "\uD112"], /* Hangul Syllable */
  ["\uD113", "\uD113"], /* Hangul Syllable */
  ["\uD114", "\uD114"], /* Hangul Syllable */
  ["\uD115", "\uD115"], /* Hangul Syllable */
  ["\uD116", "\uD116"], /* Hangul Syllable */
  ["\uD117", "\uD117"], /* Hangul Syllable */
  ["\uD118", "\uD118"], /* Hangul Syllable */
  ["\uD119", "\uD119"], /* Hangul Syllable */
  ["\uD11A", "\uD11A"], /* Hangul Syllable */
  ["\uD11B", "\uD11B"], /* Hangul Syllable */
  ["\uD11C", "\uD11C"], /* Hangul Syllable */
  ["\uD11D", "\uD11D"], /* Hangul Syllable */
  ["\uD11E", "\uD11E"], /* Hangul Syllable */
  ["\uD11F", "\uD11F"], /* Hangul Syllable */
  ["\uD120", "\uD120"], /* Hangul Syllable */
  ["\uD121", "\uD121"], /* Hangul Syllable */
  ["\uD122", "\uD122"], /* Hangul Syllable */
  ["\uD123", "\uD123"], /* Hangul Syllable */
  ["\uD124", "\uD124"], /* Hangul Syllable */
  ["\uD125", "\uD125"], /* Hangul Syllable */
  ["\uD126", "\uD126"], /* Hangul Syllable */
  ["\uD127", "\uD127"], /* Hangul Syllable */
  ["\uD128", "\uD128"], /* Hangul Syllable */
  ["\uD129", "\uD129"], /* Hangul Syllable */
  ["\uD12A", "\uD12A"], /* Hangul Syllable */
  ["\uD12B", "\uD12B"], /* Hangul Syllable */
  ["\uD12C", "\uD12C"], /* Hangul Syllable */
  ["\uD12D", "\uD12D"], /* Hangul Syllable */
  ["\uD12E", "\uD12E"], /* Hangul Syllable */
  ["\uD12F", "\uD12F"], /* Hangul Syllable */
  ["\uD130", "\uD130"], /* Hangul Syllable */
  ["\uD131", "\uD131"], /* Hangul Syllable */
  ["\uD132", "\uD132"], /* Hangul Syllable */
  ["\uD133", "\uD133"], /* Hangul Syllable */
  ["\uD134", "\uD134"], /* Hangul Syllable */
  ["\uD135", "\uD135"], /* Hangul Syllable */
  ["\uD136", "\uD136"], /* Hangul Syllable */
  ["\uD137", "\uD137"], /* Hangul Syllable */
  ["\uD138", "\uD138"], /* Hangul Syllable */
  ["\uD139", "\uD139"], /* Hangul Syllable */
  ["\uD13A", "\uD13A"], /* Hangul Syllable */
  ["\uD13B", "\uD13B"], /* Hangul Syllable */
  ["\uD13C", "\uD13C"], /* Hangul Syllable */
  ["\uD13D", "\uD13D"], /* Hangul Syllable */
  ["\uD13E", "\uD13E"], /* Hangul Syllable */
  ["\uD13F", "\uD13F"], /* Hangul Syllable */
  ["\uD140", "\uD140"], /* Hangul Syllable */
  ["\uD141", "\uD141"], /* Hangul Syllable */
  ["\uD142", "\uD142"], /* Hangul Syllable */
  ["\uD143", "\uD143"], /* Hangul Syllable */
  ["\uD144", "\uD144"], /* Hangul Syllable */
  ["\uD145", "\uD145"], /* Hangul Syllable */
  ["\uD146", "\uD146"], /* Hangul Syllable */
  ["\uD147", "\uD147"], /* Hangul Syllable */
  ["\uD148", "\uD148"], /* Hangul Syllable */
  ["\uD149", "\uD149"], /* Hangul Syllable */
  ["\uD14A", "\uD14A"], /* Hangul Syllable */
  ["\uD14B", "\uD14B"], /* Hangul Syllable */
  ["\uD14C", "\uD14C"], /* Hangul Syllable */
  ["\uD14D", "\uD14D"], /* Hangul Syllable */
  ["\uD14E", "\uD14E"], /* Hangul Syllable */
  ["\uD14F", "\uD14F"], /* Hangul Syllable */
  ["\uD150", "\uD150"], /* Hangul Syllable */
  ["\uD151", "\uD151"], /* Hangul Syllable */
  ["\uD152", "\uD152"], /* Hangul Syllable */
  ["\uD153", "\uD153"], /* Hangul Syllable */
  ["\uD154", "\uD154"], /* Hangul Syllable */
  ["\uD155", "\uD155"], /* Hangul Syllable */
  ["\uD156", "\uD156"], /* Hangul Syllable */
  ["\uD157", "\uD157"], /* Hangul Syllable */
  ["\uD158", "\uD158"], /* Hangul Syllable */
  ["\uD159", "\uD159"], /* Hangul Syllable */
  ["\uD15A", "\uD15A"], /* Hangul Syllable */
  ["\uD15B", "\uD15B"], /* Hangul Syllable */
  ["\uD15C", "\uD15C"], /* Hangul Syllable */
  ["\uD15D", "\uD15D"], /* Hangul Syllable */
  ["\uD15E", "\uD15E"], /* Hangul Syllable */
  ["\uD15F", "\uD15F"], /* Hangul Syllable */
  ["\uD160", "\uD160"], /* Hangul Syllable */
  ["\uD161", "\uD161"], /* Hangul Syllable */
  ["\uD162", "\uD162"], /* Hangul Syllable */
  ["\uD163", "\uD163"], /* Hangul Syllable */
  ["\uD164", "\uD164"], /* Hangul Syllable */
  ["\uD165", "\uD165"], /* Hangul Syllable */
  ["\uD166", "\uD166"], /* Hangul Syllable */
  ["\uD167", "\uD167"], /* Hangul Syllable */
  ["\uD168", "\uD168"], /* Hangul Syllable */
  ["\uD169", "\uD169"], /* Hangul Syllable */
  ["\uD16A", "\uD16A"], /* Hangul Syllable */
  ["\uD16B", "\uD16B"], /* Hangul Syllable */
  ["\uD16C", "\uD16C"], /* Hangul Syllable */
  ["\uD16D", "\uD16D"], /* Hangul Syllable */
  ["\uD16E", "\uD16E"], /* Hangul Syllable */
  ["\uD16F", "\uD16F"], /* Hangul Syllable */
  ["\uD170", "\uD170"], /* Hangul Syllable */
  ["\uD171", "\uD171"], /* Hangul Syllable */
  ["\uD172", "\uD172"], /* Hangul Syllable */
  ["\uD173", "\uD173"], /* Hangul Syllable */
  ["\uD174", "\uD174"], /* Hangul Syllable */
  ["\uD175", "\uD175"], /* Hangul Syllable */
  ["\uD176", "\uD176"], /* Hangul Syllable */
  ["\uD177", "\uD177"], /* Hangul Syllable */
  ["\uD178", "\uD178"], /* Hangul Syllable */
  ["\uD179", "\uD179"], /* Hangul Syllable */
  ["\uD17A", "\uD17A"], /* Hangul Syllable */
  ["\uD17B", "\uD17B"], /* Hangul Syllable */
  ["\uD17C", "\uD17C"], /* Hangul Syllable */
  ["\uD17D", "\uD17D"], /* Hangul Syllable */
  ["\uD17E", "\uD17E"], /* Hangul Syllable */
  ["\uD17F", "\uD17F"], /* Hangul Syllable */
  ["\uD180", "\uD180"], /* Hangul Syllable */
  ["\uD181", "\uD181"], /* Hangul Syllable */
  ["\uD182", "\uD182"], /* Hangul Syllable */
  ["\uD183", "\uD183"], /* Hangul Syllable */
  ["\uD184", "\uD184"], /* Hangul Syllable */
  ["\uD185", "\uD185"], /* Hangul Syllable */
  ["\uD186", "\uD186"], /* Hangul Syllable */
  ["\uD187", "\uD187"], /* Hangul Syllable */
  ["\uD188", "\uD188"], /* Hangul Syllable */
  ["\uD189", "\uD189"], /* Hangul Syllable */
  ["\uD18A", "\uD18A"], /* Hangul Syllable */
  ["\uD18B", "\uD18B"], /* Hangul Syllable */
  ["\uD18C", "\uD18C"], /* Hangul Syllable */
  ["\uD18D", "\uD18D"], /* Hangul Syllable */
  ["\uD18E", "\uD18E"], /* Hangul Syllable */
  ["\uD18F", "\uD18F"], /* Hangul Syllable */
  ["\uD190", "\uD190"], /* Hangul Syllable */
  ["\uD191", "\uD191"], /* Hangul Syllable */
  ["\uD192", "\uD192"], /* Hangul Syllable */
  ["\uD193", "\uD193"], /* Hangul Syllable */
  ["\uD194", "\uD194"], /* Hangul Syllable */
  ["\uD195", "\uD195"], /* Hangul Syllable */
  ["\uD196", "\uD196"], /* Hangul Syllable */
  ["\uD197", "\uD197"], /* Hangul Syllable */
  ["\uD198", "\uD198"], /* Hangul Syllable */
  ["\uD199", "\uD199"], /* Hangul Syllable */
  ["\uD19A", "\uD19A"], /* Hangul Syllable */
  ["\uD19B", "\uD19B"], /* Hangul Syllable */
  ["\uD19C", "\uD19C"], /* Hangul Syllable */
  ["\uD19D", "\uD19D"], /* Hangul Syllable */
  ["\uD19E", "\uD19E"], /* Hangul Syllable */
  ["\uD19F", "\uD19F"], /* Hangul Syllable */
  ["\uD1A0", "\uD1A0"], /* Hangul Syllable */
  ["\uD1A1", "\uD1A1"], /* Hangul Syllable */
  ["\uD1A2", "\uD1A2"], /* Hangul Syllable */
  ["\uD1A3", "\uD1A3"], /* Hangul Syllable */
  ["\uD1A4", "\uD1A4"], /* Hangul Syllable */
  ["\uD1A5", "\uD1A5"], /* Hangul Syllable */
  ["\uD1A6", "\uD1A6"], /* Hangul Syllable */
  ["\uD1A7", "\uD1A7"], /* Hangul Syllable */
  ["\uD1A8", "\uD1A8"], /* Hangul Syllable */
  ["\uD1A9", "\uD1A9"], /* Hangul Syllable */
  ["\uD1AA", "\uD1AA"], /* Hangul Syllable */
  ["\uD1AB", "\uD1AB"], /* Hangul Syllable */
  ["\uD1AC", "\uD1AC"], /* Hangul Syllable */
  ["\uD1AD", "\uD1AD"], /* Hangul Syllable */
  ["\uD1AE", "\uD1AE"], /* Hangul Syllable */
  ["\uD1AF", "\uD1AF"], /* Hangul Syllable */
  ["\uD1B0", "\uD1B0"], /* Hangul Syllable */
  ["\uD1B1", "\uD1B1"], /* Hangul Syllable */
  ["\uD1B2", "\uD1B2"], /* Hangul Syllable */
  ["\uD1B3", "\uD1B3"], /* Hangul Syllable */
  ["\uD1B4", "\uD1B4"], /* Hangul Syllable */
  ["\uD1B5", "\uD1B5"], /* Hangul Syllable */
  ["\uD1B6", "\uD1B6"], /* Hangul Syllable */
  ["\uD1B7", "\uD1B7"], /* Hangul Syllable */
  ["\uD1B8", "\uD1B8"], /* Hangul Syllable */
  ["\uD1B9", "\uD1B9"], /* Hangul Syllable */
  ["\uD1BA", "\uD1BA"], /* Hangul Syllable */
  ["\uD1BB", "\uD1BB"], /* Hangul Syllable */
  ["\uD1BC", "\uD1BC"], /* Hangul Syllable */
  ["\uD1BD", "\uD1BD"], /* Hangul Syllable */
  ["\uD1BE", "\uD1BE"], /* Hangul Syllable */
  ["\uD1BF", "\uD1BF"], /* Hangul Syllable */
  ["\uD1C0", "\uD1C0"], /* Hangul Syllable */
  ["\uD1C1", "\uD1C1"], /* Hangul Syllable */
  ["\uD1C2", "\uD1C2"], /* Hangul Syllable */
  ["\uD1C3", "\uD1C3"], /* Hangul Syllable */
  ["\uD1C4", "\uD1C4"], /* Hangul Syllable */
  ["\uD1C5", "\uD1C5"], /* Hangul Syllable */
  ["\uD1C6", "\uD1C6"], /* Hangul Syllable */
  ["\uD1C7", "\uD1C7"], /* Hangul Syllable */
  ["\uD1C8", "\uD1C8"], /* Hangul Syllable */
  ["\uD1C9", "\uD1C9"], /* Hangul Syllable */
  ["\uD1CA", "\uD1CA"], /* Hangul Syllable */
  ["\uD1CB", "\uD1CB"], /* Hangul Syllable */
  ["\uD1CC", "\uD1CC"], /* Hangul Syllable */
  ["\uD1CD", "\uD1CD"], /* Hangul Syllable */
  ["\uD1CE", "\uD1CE"], /* Hangul Syllable */
  ["\uD1CF", "\uD1CF"], /* Hangul Syllable */
  ["\uD1D0", "\uD1D0"], /* Hangul Syllable */
  ["\uD1D1", "\uD1D1"], /* Hangul Syllable */
  ["\uD1D2", "\uD1D2"], /* Hangul Syllable */
  ["\uD1D3", "\uD1D3"], /* Hangul Syllable */
  ["\uD1D4", "\uD1D4"], /* Hangul Syllable */
  ["\uD1D5", "\uD1D5"], /* Hangul Syllable */
  ["\uD1D6", "\uD1D6"], /* Hangul Syllable */
  ["\uD1D7", "\uD1D7"], /* Hangul Syllable */
  ["\uD1D8", "\uD1D8"], /* Hangul Syllable */
  ["\uD1D9", "\uD1D9"], /* Hangul Syllable */
  ["\uD1DA", "\uD1DA"], /* Hangul Syllable */
  ["\uD1DB", "\uD1DB"], /* Hangul Syllable */
  ["\uD1DC", "\uD1DC"], /* Hangul Syllable */
  ["\uD1DD", "\uD1DD"], /* Hangul Syllable */
  ["\uD1DE", "\uD1DE"], /* Hangul Syllable */
  ["\uD1DF", "\uD1DF"], /* Hangul Syllable */
  ["\uD1E0", "\uD1E0"], /* Hangul Syllable */
  ["\uD1E1", "\uD1E1"], /* Hangul Syllable */
  ["\uD1E2", "\uD1E2"], /* Hangul Syllable */
  ["\uD1E3", "\uD1E3"], /* Hangul Syllable */
  ["\uD1E4", "\uD1E4"], /* Hangul Syllable */
  ["\uD1E5", "\uD1E5"], /* Hangul Syllable */
  ["\uD1E6", "\uD1E6"], /* Hangul Syllable */
  ["\uD1E7", "\uD1E7"], /* Hangul Syllable */
  ["\uD1E8", "\uD1E8"], /* Hangul Syllable */
  ["\uD1E9", "\uD1E9"], /* Hangul Syllable */
  ["\uD1EA", "\uD1EA"], /* Hangul Syllable */
  ["\uD1EB", "\uD1EB"], /* Hangul Syllable */
  ["\uD1EC", "\uD1EC"], /* Hangul Syllable */
  ["\uD1ED", "\uD1ED"], /* Hangul Syllable */
  ["\uD1EE", "\uD1EE"], /* Hangul Syllable */
  ["\uD1EF", "\uD1EF"], /* Hangul Syllable */
  ["\uD1F0", "\uD1F0"], /* Hangul Syllable */
  ["\uD1F1", "\uD1F1"], /* Hangul Syllable */
  ["\uD1F2", "\uD1F2"], /* Hangul Syllable */
  ["\uD1F3", "\uD1F3"], /* Hangul Syllable */
  ["\uD1F4", "\uD1F4"], /* Hangul Syllable */
  ["\uD1F5", "\uD1F5"], /* Hangul Syllable */
  ["\uD1F6", "\uD1F6"], /* Hangul Syllable */
  ["\uD1F7", "\uD1F7"], /* Hangul Syllable */
  ["\uD1F8", "\uD1F8"], /* Hangul Syllable */
  ["\uD1F9", "\uD1F9"], /* Hangul Syllable */
  ["\uD1FA", "\uD1FA"], /* Hangul Syllable */
  ["\uD1FB", "\uD1FB"], /* Hangul Syllable */
  ["\uD1FC", "\uD1FC"], /* Hangul Syllable */
  ["\uD1FD", "\uD1FD"], /* Hangul Syllable */
  ["\uD1FE", "\uD1FE"], /* Hangul Syllable */
  ["\uD1FF", "\uD1FF"], /* Hangul Syllable */
  ["\uD200", "\uD200"], /* Hangul Syllable */
  ["\uD201", "\uD201"], /* Hangul Syllable */
  ["\uD202", "\uD202"], /* Hangul Syllable */
  ["\uD203", "\uD203"], /* Hangul Syllable */
  ["\uD204", "\uD204"], /* Hangul Syllable */
  ["\uD205", "\uD205"], /* Hangul Syllable */
  ["\uD206", "\uD206"], /* Hangul Syllable */
  ["\uD207", "\uD207"], /* Hangul Syllable */
  ["\uD208", "\uD208"], /* Hangul Syllable */
  ["\uD209", "\uD209"], /* Hangul Syllable */
  ["\uD20A", "\uD20A"], /* Hangul Syllable */
  ["\uD20B", "\uD20B"], /* Hangul Syllable */
  ["\uD20C", "\uD20C"], /* Hangul Syllable */
  ["\uD20D", "\uD20D"], /* Hangul Syllable */
  ["\uD20E", "\uD20E"], /* Hangul Syllable */
  ["\uD20F", "\uD20F"], /* Hangul Syllable */
  ["\uD210", "\uD210"], /* Hangul Syllable */
  ["\uD211", "\uD211"], /* Hangul Syllable */
  ["\uD212", "\uD212"], /* Hangul Syllable */
  ["\uD213", "\uD213"], /* Hangul Syllable */
  ["\uD214", "\uD214"], /* Hangul Syllable */
  ["\uD215", "\uD215"], /* Hangul Syllable */
  ["\uD216", "\uD216"], /* Hangul Syllable */
  ["\uD217", "\uD217"], /* Hangul Syllable */
  ["\uD218", "\uD218"], /* Hangul Syllable */
  ["\uD219", "\uD219"], /* Hangul Syllable */
  ["\uD21A", "\uD21A"], /* Hangul Syllable */
  ["\uD21B", "\uD21B"], /* Hangul Syllable */
  ["\uD21C", "\uD21C"], /* Hangul Syllable */
  ["\uD21D", "\uD21D"], /* Hangul Syllable */
  ["\uD21E", "\uD21E"], /* Hangul Syllable */
  ["\uD21F", "\uD21F"], /* Hangul Syllable */
  ["\uD220", "\uD220"], /* Hangul Syllable */
  ["\uD221", "\uD221"], /* Hangul Syllable */
  ["\uD222", "\uD222"], /* Hangul Syllable */
  ["\uD223", "\uD223"], /* Hangul Syllable */
  ["\uD224", "\uD224"], /* Hangul Syllable */
  ["\uD225", "\uD225"], /* Hangul Syllable */
  ["\uD226", "\uD226"], /* Hangul Syllable */
  ["\uD227", "\uD227"], /* Hangul Syllable */
  ["\uD228", "\uD228"], /* Hangul Syllable */
  ["\uD229", "\uD229"], /* Hangul Syllable */
  ["\uD22A", "\uD22A"], /* Hangul Syllable */
  ["\uD22B", "\uD22B"], /* Hangul Syllable */
  ["\uD22C", "\uD22C"], /* Hangul Syllable */
  ["\uD22D", "\uD22D"], /* Hangul Syllable */
  ["\uD22E", "\uD22E"], /* Hangul Syllable */
  ["\uD22F", "\uD22F"], /* Hangul Syllable */
  ["\uD230", "\uD230"], /* Hangul Syllable */
  ["\uD231", "\uD231"], /* Hangul Syllable */
  ["\uD232", "\uD232"], /* Hangul Syllable */
  ["\uD233", "\uD233"], /* Hangul Syllable */
  ["\uD234", "\uD234"], /* Hangul Syllable */
  ["\uD235", "\uD235"], /* Hangul Syllable */
  ["\uD236", "\uD236"], /* Hangul Syllable */
  ["\uD237", "\uD237"], /* Hangul Syllable */
  ["\uD238", "\uD238"], /* Hangul Syllable */
  ["\uD239", "\uD239"], /* Hangul Syllable */
  ["\uD23A", "\uD23A"], /* Hangul Syllable */
  ["\uD23B", "\uD23B"], /* Hangul Syllable */
  ["\uD23C", "\uD23C"], /* Hangul Syllable */
  ["\uD23D", "\uD23D"], /* Hangul Syllable */
  ["\uD23E", "\uD23E"], /* Hangul Syllable */
  ["\uD23F", "\uD23F"], /* Hangul Syllable */
  ["\uD240", "\uD240"], /* Hangul Syllable */
  ["\uD241", "\uD241"], /* Hangul Syllable */
  ["\uD242", "\uD242"], /* Hangul Syllable */
  ["\uD243", "\uD243"], /* Hangul Syllable */
  ["\uD244", "\uD244"], /* Hangul Syllable */
  ["\uD245", "\uD245"], /* Hangul Syllable */
  ["\uD246", "\uD246"], /* Hangul Syllable */
  ["\uD247", "\uD247"], /* Hangul Syllable */
  ["\uD248", "\uD248"], /* Hangul Syllable */
  ["\uD249", "\uD249"], /* Hangul Syllable */
  ["\uD24A", "\uD24A"], /* Hangul Syllable */
  ["\uD24B", "\uD24B"], /* Hangul Syllable */
  ["\uD24C", "\uD24C"], /* Hangul Syllable */
  ["\uD24D", "\uD24D"], /* Hangul Syllable */
  ["\uD24E", "\uD24E"], /* Hangul Syllable */
  ["\uD24F", "\uD24F"], /* Hangul Syllable */
  ["\uD250", "\uD250"], /* Hangul Syllable */
  ["\uD251", "\uD251"], /* Hangul Syllable */
  ["\uD252", "\uD252"], /* Hangul Syllable */
  ["\uD253", "\uD253"], /* Hangul Syllable */
  ["\uD254", "\uD254"], /* Hangul Syllable */
  ["\uD255", "\uD255"], /* Hangul Syllable */
  ["\uD256", "\uD256"], /* Hangul Syllable */
  ["\uD257", "\uD257"], /* Hangul Syllable */
  ["\uD258", "\uD258"], /* Hangul Syllable */
  ["\uD259", "\uD259"], /* Hangul Syllable */
  ["\uD25A", "\uD25A"], /* Hangul Syllable */
  ["\uD25B", "\uD25B"], /* Hangul Syllable */
  ["\uD25C", "\uD25C"], /* Hangul Syllable */
  ["\uD25D", "\uD25D"], /* Hangul Syllable */
  ["\uD25E", "\uD25E"], /* Hangul Syllable */
  ["\uD25F", "\uD25F"], /* Hangul Syllable */
  ["\uD260", "\uD260"], /* Hangul Syllable */
  ["\uD261", "\uD261"], /* Hangul Syllable */
  ["\uD262", "\uD262"], /* Hangul Syllable */
  ["\uD263", "\uD263"], /* Hangul Syllable */
  ["\uD264", "\uD264"], /* Hangul Syllable */
  ["\uD265", "\uD265"], /* Hangul Syllable */
  ["\uD266", "\uD266"], /* Hangul Syllable */
  ["\uD267", "\uD267"], /* Hangul Syllable */
  ["\uD268", "\uD268"], /* Hangul Syllable */
  ["\uD269", "\uD269"], /* Hangul Syllable */
  ["\uD26A", "\uD26A"], /* Hangul Syllable */
  ["\uD26B", "\uD26B"], /* Hangul Syllable */
  ["\uD26C", "\uD26C"], /* Hangul Syllable */
  ["\uD26D", "\uD26D"], /* Hangul Syllable */
  ["\uD26E", "\uD26E"], /* Hangul Syllable */
  ["\uD26F", "\uD26F"], /* Hangul Syllable */
  ["\uD270", "\uD270"], /* Hangul Syllable */
  ["\uD271", "\uD271"], /* Hangul Syllable */
  ["\uD272", "\uD272"], /* Hangul Syllable */
  ["\uD273", "\uD273"], /* Hangul Syllable */
  ["\uD274", "\uD274"], /* Hangul Syllable */
  ["\uD275", "\uD275"], /* Hangul Syllable */
  ["\uD276", "\uD276"], /* Hangul Syllable */
  ["\uD277", "\uD277"], /* Hangul Syllable */
  ["\uD278", "\uD278"], /* Hangul Syllable */
  ["\uD279", "\uD279"], /* Hangul Syllable */
  ["\uD27A", "\uD27A"], /* Hangul Syllable */
  ["\uD27B", "\uD27B"], /* Hangul Syllable */
  ["\uD27C", "\uD27C"], /* Hangul Syllable */
  ["\uD27D", "\uD27D"], /* Hangul Syllable */
  ["\uD27E", "\uD27E"], /* Hangul Syllable */
  ["\uD27F", "\uD27F"], /* Hangul Syllable */
  ["\uD280", "\uD280"], /* Hangul Syllable */
  ["\uD281", "\uD281"], /* Hangul Syllable */
  ["\uD282", "\uD282"], /* Hangul Syllable */
  ["\uD283", "\uD283"], /* Hangul Syllable */
  ["\uD284", "\uD284"], /* Hangul Syllable */
  ["\uD285", "\uD285"], /* Hangul Syllable */
  ["\uD286", "\uD286"], /* Hangul Syllable */
  ["\uD287", "\uD287"], /* Hangul Syllable */
  ["\uD288", "\uD288"], /* Hangul Syllable */
  ["\uD289", "\uD289"], /* Hangul Syllable */
  ["\uD28A", "\uD28A"], /* Hangul Syllable */
  ["\uD28B", "\uD28B"], /* Hangul Syllable */
  ["\uD28C", "\uD28C"], /* Hangul Syllable */
  ["\uD28D", "\uD28D"], /* Hangul Syllable */
  ["\uD28E", "\uD28E"], /* Hangul Syllable */
  ["\uD28F", "\uD28F"], /* Hangul Syllable */
  ["\uD290", "\uD290"], /* Hangul Syllable */
  ["\uD291", "\uD291"], /* Hangul Syllable */
  ["\uD292", "\uD292"], /* Hangul Syllable */
  ["\uD293", "\uD293"], /* Hangul Syllable */
  ["\uD294", "\uD294"], /* Hangul Syllable */
  ["\uD295", "\uD295"], /* Hangul Syllable */
  ["\uD296", "\uD296"], /* Hangul Syllable */
  ["\uD297", "\uD297"], /* Hangul Syllable */
  ["\uD298", "\uD298"], /* Hangul Syllable */
  ["\uD299", "\uD299"], /* Hangul Syllable */
  ["\uD29A", "\uD29A"], /* Hangul Syllable */
  ["\uD29B", "\uD29B"], /* Hangul Syllable */
  ["\uD29C", "\uD29C"], /* Hangul Syllable */
  ["\uD29D", "\uD29D"], /* Hangul Syllable */
  ["\uD29E", "\uD29E"], /* Hangul Syllable */
  ["\uD29F", "\uD29F"], /* Hangul Syllable */
  ["\uD2A0", "\uD2A0"], /* Hangul Syllable */
  ["\uD2A1", "\uD2A1"], /* Hangul Syllable */
  ["\uD2A2", "\uD2A2"], /* Hangul Syllable */
  ["\uD2A3", "\uD2A3"], /* Hangul Syllable */
  ["\uD2A4", "\uD2A4"], /* Hangul Syllable */
  ["\uD2A5", "\uD2A5"], /* Hangul Syllable */
  ["\uD2A6", "\uD2A6"], /* Hangul Syllable */
  ["\uD2A7", "\uD2A7"], /* Hangul Syllable */
  ["\uD2A8", "\uD2A8"], /* Hangul Syllable */
  ["\uD2A9", "\uD2A9"], /* Hangul Syllable */
  ["\uD2AA", "\uD2AA"], /* Hangul Syllable */
  ["\uD2AB", "\uD2AB"], /* Hangul Syllable */
  ["\uD2AC", "\uD2AC"], /* Hangul Syllable */
  ["\uD2AD", "\uD2AD"], /* Hangul Syllable */
  ["\uD2AE", "\uD2AE"], /* Hangul Syllable */
  ["\uD2AF", "\uD2AF"], /* Hangul Syllable */
  ["\uD2B0", "\uD2B0"], /* Hangul Syllable */
  ["\uD2B1", "\uD2B1"], /* Hangul Syllable */
  ["\uD2B2", "\uD2B2"], /* Hangul Syllable */
  ["\uD2B3", "\uD2B3"], /* Hangul Syllable */
  ["\uD2B4", "\uD2B4"], /* Hangul Syllable */
  ["\uD2B5", "\uD2B5"], /* Hangul Syllable */
  ["\uD2B6", "\uD2B6"], /* Hangul Syllable */
  ["\uD2B7", "\uD2B7"], /* Hangul Syllable */
  ["\uD2B8", "\uD2B8"], /* Hangul Syllable */
  ["\uD2B9", "\uD2B9"], /* Hangul Syllable */
  ["\uD2BA", "\uD2BA"], /* Hangul Syllable */
  ["\uD2BB", "\uD2BB"], /* Hangul Syllable */
  ["\uD2BC", "\uD2BC"], /* Hangul Syllable */
  ["\uD2BD", "\uD2BD"], /* Hangul Syllable */
  ["\uD2BE", "\uD2BE"], /* Hangul Syllable */
  ["\uD2BF", "\uD2BF"], /* Hangul Syllable */
  ["\uD2C0", "\uD2C0"], /* Hangul Syllable */
  ["\uD2C1", "\uD2C1"], /* Hangul Syllable */
  ["\uD2C2", "\uD2C2"], /* Hangul Syllable */
  ["\uD2C3", "\uD2C3"], /* Hangul Syllable */
  ["\uD2C4", "\uD2C4"], /* Hangul Syllable */
  ["\uD2C5", "\uD2C5"], /* Hangul Syllable */
  ["\uD2C6", "\uD2C6"], /* Hangul Syllable */
  ["\uD2C7", "\uD2C7"], /* Hangul Syllable */
  ["\uD2C8", "\uD2C8"], /* Hangul Syllable */
  ["\uD2C9", "\uD2C9"], /* Hangul Syllable */
  ["\uD2CA", "\uD2CA"], /* Hangul Syllable */
  ["\uD2CB", "\uD2CB"], /* Hangul Syllable */
  ["\uD2CC", "\uD2CC"], /* Hangul Syllable */
  ["\uD2CD", "\uD2CD"], /* Hangul Syllable */
  ["\uD2CE", "\uD2CE"], /* Hangul Syllable */
  ["\uD2CF", "\uD2CF"], /* Hangul Syllable */
  ["\uD2D0", "\uD2D0"], /* Hangul Syllable */
  ["\uD2D1", "\uD2D1"], /* Hangul Syllable */
  ["\uD2D2", "\uD2D2"], /* Hangul Syllable */
  ["\uD2D3", "\uD2D3"], /* Hangul Syllable */
  ["\uD2D4", "\uD2D4"], /* Hangul Syllable */
  ["\uD2D5", "\uD2D5"], /* Hangul Syllable */
  ["\uD2D6", "\uD2D6"], /* Hangul Syllable */
  ["\uD2D7", "\uD2D7"], /* Hangul Syllable */
  ["\uD2D8", "\uD2D8"], /* Hangul Syllable */
  ["\uD2D9", "\uD2D9"], /* Hangul Syllable */
  ["\uD2DA", "\uD2DA"], /* Hangul Syllable */
  ["\uD2DB", "\uD2DB"], /* Hangul Syllable */
  ["\uD2DC", "\uD2DC"], /* Hangul Syllable */
  ["\uD2DD", "\uD2DD"], /* Hangul Syllable */
  ["\uD2DE", "\uD2DE"], /* Hangul Syllable */
  ["\uD2DF", "\uD2DF"], /* Hangul Syllable */
  ["\uD2E0", "\uD2E0"], /* Hangul Syllable */
  ["\uD2E1", "\uD2E1"], /* Hangul Syllable */
  ["\uD2E2", "\uD2E2"], /* Hangul Syllable */
  ["\uD2E3", "\uD2E3"], /* Hangul Syllable */
  ["\uD2E4", "\uD2E4"], /* Hangul Syllable */
  ["\uD2E5", "\uD2E5"], /* Hangul Syllable */
  ["\uD2E6", "\uD2E6"], /* Hangul Syllable */
  ["\uD2E7", "\uD2E7"], /* Hangul Syllable */
  ["\uD2E8", "\uD2E8"], /* Hangul Syllable */
  ["\uD2E9", "\uD2E9"], /* Hangul Syllable */
  ["\uD2EA", "\uD2EA"], /* Hangul Syllable */
  ["\uD2EB", "\uD2EB"], /* Hangul Syllable */
  ["\uD2EC", "\uD2EC"], /* Hangul Syllable */
  ["\uD2ED", "\uD2ED"], /* Hangul Syllable */
  ["\uD2EE", "\uD2EE"], /* Hangul Syllable */
  ["\uD2EF", "\uD2EF"], /* Hangul Syllable */
  ["\uD2F0", "\uD2F0"], /* Hangul Syllable */
  ["\uD2F1", "\uD2F1"], /* Hangul Syllable */
  ["\uD2F2", "\uD2F2"], /* Hangul Syllable */
  ["\uD2F3", "\uD2F3"], /* Hangul Syllable */
  ["\uD2F4", "\uD2F4"], /* Hangul Syllable */
  ["\uD2F5", "\uD2F5"], /* Hangul Syllable */
  ["\uD2F6", "\uD2F6"], /* Hangul Syllable */
  ["\uD2F7", "\uD2F7"], /* Hangul Syllable */
  ["\uD2F8", "\uD2F8"], /* Hangul Syllable */
  ["\uD2F9", "\uD2F9"], /* Hangul Syllable */
  ["\uD2FA", "\uD2FA"], /* Hangul Syllable */
  ["\uD2FB", "\uD2FB"], /* Hangul Syllable */
  ["\uD2FC", "\uD2FC"], /* Hangul Syllable */
  ["\uD2FD", "\uD2FD"], /* Hangul Syllable */
  ["\uD2FE", "\uD2FE"], /* Hangul Syllable */
  ["\uD2FF", "\uD2FF"], /* Hangul Syllable */
  ["\uD300", "\uD300"], /* Hangul Syllable */
  ["\uD301", "\uD301"], /* Hangul Syllable */
  ["\uD302", "\uD302"], /* Hangul Syllable */
  ["\uD303", "\uD303"], /* Hangul Syllable */
  ["\uD304", "\uD304"], /* Hangul Syllable */
  ["\uD305", "\uD305"], /* Hangul Syllable */
  ["\uD306", "\uD306"], /* Hangul Syllable */
  ["\uD307", "\uD307"], /* Hangul Syllable */
  ["\uD308", "\uD308"], /* Hangul Syllable */
  ["\uD309", "\uD309"], /* Hangul Syllable */
  ["\uD30A", "\uD30A"], /* Hangul Syllable */
  ["\uD30B", "\uD30B"], /* Hangul Syllable */
  ["\uD30C", "\uD30C"], /* Hangul Syllable */
  ["\uD30D", "\uD30D"], /* Hangul Syllable */
  ["\uD30E", "\uD30E"], /* Hangul Syllable */
  ["\uD30F", "\uD30F"], /* Hangul Syllable */
  ["\uD310", "\uD310"], /* Hangul Syllable */
  ["\uD311", "\uD311"], /* Hangul Syllable */
  ["\uD312", "\uD312"], /* Hangul Syllable */
  ["\uD313", "\uD313"], /* Hangul Syllable */
  ["\uD314", "\uD314"], /* Hangul Syllable */
  ["\uD315", "\uD315"], /* Hangul Syllable */
  ["\uD316", "\uD316"], /* Hangul Syllable */
  ["\uD317", "\uD317"], /* Hangul Syllable */
  ["\uD318", "\uD318"], /* Hangul Syllable */
  ["\uD319", "\uD319"], /* Hangul Syllable */
  ["\uD31A", "\uD31A"], /* Hangul Syllable */
  ["\uD31B", "\uD31B"], /* Hangul Syllable */
  ["\uD31C", "\uD31C"], /* Hangul Syllable */
  ["\uD31D", "\uD31D"], /* Hangul Syllable */
  ["\uD31E", "\uD31E"], /* Hangul Syllable */
  ["\uD31F", "\uD31F"], /* Hangul Syllable */
  ["\uD320", "\uD320"], /* Hangul Syllable */
  ["\uD321", "\uD321"], /* Hangul Syllable */
  ["\uD322", "\uD322"], /* Hangul Syllable */
  ["\uD323", "\uD323"], /* Hangul Syllable */
  ["\uD324", "\uD324"], /* Hangul Syllable */
  ["\uD325", "\uD325"], /* Hangul Syllable */
  ["\uD326", "\uD326"], /* Hangul Syllable */
  ["\uD327", "\uD327"], /* Hangul Syllable */
  ["\uD328", "\uD328"], /* Hangul Syllable */
  ["\uD329", "\uD329"], /* Hangul Syllable */
  ["\uD32A", "\uD32A"], /* Hangul Syllable */
  ["\uD32B", "\uD32B"], /* Hangul Syllable */
  ["\uD32C", "\uD32C"], /* Hangul Syllable */
  ["\uD32D", "\uD32D"], /* Hangul Syllable */
  ["\uD32E", "\uD32E"], /* Hangul Syllable */
  ["\uD32F", "\uD32F"], /* Hangul Syllable */
  ["\uD330", "\uD330"], /* Hangul Syllable */
  ["\uD331", "\uD331"], /* Hangul Syllable */
  ["\uD332", "\uD332"], /* Hangul Syllable */
  ["\uD333", "\uD333"], /* Hangul Syllable */
  ["\uD334", "\uD334"], /* Hangul Syllable */
  ["\uD335", "\uD335"], /* Hangul Syllable */
  ["\uD336", "\uD336"], /* Hangul Syllable */
  ["\uD337", "\uD337"], /* Hangul Syllable */
  ["\uD338", "\uD338"], /* Hangul Syllable */
  ["\uD339", "\uD339"], /* Hangul Syllable */
  ["\uD33A", "\uD33A"], /* Hangul Syllable */
  ["\uD33B", "\uD33B"], /* Hangul Syllable */
  ["\uD33C", "\uD33C"], /* Hangul Syllable */
  ["\uD33D", "\uD33D"], /* Hangul Syllable */
  ["\uD33E", "\uD33E"], /* Hangul Syllable */
  ["\uD33F", "\uD33F"], /* Hangul Syllable */
  ["\uD340", "\uD340"], /* Hangul Syllable */
  ["\uD341", "\uD341"], /* Hangul Syllable */
  ["\uD342", "\uD342"], /* Hangul Syllable */
  ["\uD343", "\uD343"], /* Hangul Syllable */
  ["\uD344", "\uD344"], /* Hangul Syllable */
  ["\uD345", "\uD345"], /* Hangul Syllable */
  ["\uD346", "\uD346"], /* Hangul Syllable */
  ["\uD347", "\uD347"], /* Hangul Syllable */
  ["\uD348", "\uD348"], /* Hangul Syllable */
  ["\uD349", "\uD349"], /* Hangul Syllable */
  ["\uD34A", "\uD34A"], /* Hangul Syllable */
  ["\uD34B", "\uD34B"], /* Hangul Syllable */
  ["\uD34C", "\uD34C"], /* Hangul Syllable */
  ["\uD34D", "\uD34D"], /* Hangul Syllable */
  ["\uD34E", "\uD34E"], /* Hangul Syllable */
  ["\uD34F", "\uD34F"], /* Hangul Syllable */
  ["\uD350", "\uD350"], /* Hangul Syllable */
  ["\uD351", "\uD351"], /* Hangul Syllable */
  ["\uD352", "\uD352"], /* Hangul Syllable */
  ["\uD353", "\uD353"], /* Hangul Syllable */
  ["\uD354", "\uD354"], /* Hangul Syllable */
  ["\uD355", "\uD355"], /* Hangul Syllable */
  ["\uD356", "\uD356"], /* Hangul Syllable */
  ["\uD357", "\uD357"], /* Hangul Syllable */
  ["\uD358", "\uD358"], /* Hangul Syllable */
  ["\uD359", "\uD359"], /* Hangul Syllable */
  ["\uD35A", "\uD35A"], /* Hangul Syllable */
  ["\uD35B", "\uD35B"], /* Hangul Syllable */
  ["\uD35C", "\uD35C"], /* Hangul Syllable */
  ["\uD35D", "\uD35D"], /* Hangul Syllable */
  ["\uD35E", "\uD35E"], /* Hangul Syllable */
  ["\uD35F", "\uD35F"], /* Hangul Syllable */
  ["\uD360", "\uD360"], /* Hangul Syllable */
  ["\uD361", "\uD361"], /* Hangul Syllable */
  ["\uD362", "\uD362"], /* Hangul Syllable */
  ["\uD363", "\uD363"], /* Hangul Syllable */
  ["\uD364", "\uD364"], /* Hangul Syllable */
  ["\uD365", "\uD365"], /* Hangul Syllable */
  ["\uD366", "\uD366"], /* Hangul Syllable */
  ["\uD367", "\uD367"], /* Hangul Syllable */
  ["\uD368", "\uD368"], /* Hangul Syllable */
  ["\uD369", "\uD369"], /* Hangul Syllable */
  ["\uD36A", "\uD36A"], /* Hangul Syllable */
  ["\uD36B", "\uD36B"], /* Hangul Syllable */
  ["\uD36C", "\uD36C"], /* Hangul Syllable */
  ["\uD36D", "\uD36D"], /* Hangul Syllable */
  ["\uD36E", "\uD36E"], /* Hangul Syllable */
  ["\uD36F", "\uD36F"], /* Hangul Syllable */
  ["\uD370", "\uD370"], /* Hangul Syllable */
  ["\uD371", "\uD371"], /* Hangul Syllable */
  ["\uD372", "\uD372"], /* Hangul Syllable */
  ["\uD373", "\uD373"], /* Hangul Syllable */
  ["\uD374", "\uD374"], /* Hangul Syllable */
  ["\uD375", "\uD375"], /* Hangul Syllable */
  ["\uD376", "\uD376"], /* Hangul Syllable */
  ["\uD377", "\uD377"], /* Hangul Syllable */
  ["\uD378", "\uD378"], /* Hangul Syllable */
  ["\uD379", "\uD379"], /* Hangul Syllable */
  ["\uD37A", "\uD37A"], /* Hangul Syllable */
  ["\uD37B", "\uD37B"], /* Hangul Syllable */
  ["\uD37C", "\uD37C"], /* Hangul Syllable */
  ["\uD37D", "\uD37D"], /* Hangul Syllable */
  ["\uD37E", "\uD37E"], /* Hangul Syllable */
  ["\uD37F", "\uD37F"], /* Hangul Syllable */
  ["\uD380", "\uD380"], /* Hangul Syllable */
  ["\uD381", "\uD381"], /* Hangul Syllable */
  ["\uD382", "\uD382"], /* Hangul Syllable */
  ["\uD383", "\uD383"], /* Hangul Syllable */
  ["\uD384", "\uD384"], /* Hangul Syllable */
  ["\uD385", "\uD385"], /* Hangul Syllable */
  ["\uD386", "\uD386"], /* Hangul Syllable */
  ["\uD387", "\uD387"], /* Hangul Syllable */
  ["\uD388", "\uD388"], /* Hangul Syllable */
  ["\uD389", "\uD389"], /* Hangul Syllable */
  ["\uD38A", "\uD38A"], /* Hangul Syllable */
  ["\uD38B", "\uD38B"], /* Hangul Syllable */
  ["\uD38C", "\uD38C"], /* Hangul Syllable */
  ["\uD38D", "\uD38D"], /* Hangul Syllable */
  ["\uD38E", "\uD38E"], /* Hangul Syllable */
  ["\uD38F", "\uD38F"], /* Hangul Syllable */
  ["\uD390", "\uD390"], /* Hangul Syllable */
  ["\uD391", "\uD391"], /* Hangul Syllable */
  ["\uD392", "\uD392"], /* Hangul Syllable */
  ["\uD393", "\uD393"], /* Hangul Syllable */
  ["\uD394", "\uD394"], /* Hangul Syllable */
  ["\uD395", "\uD395"], /* Hangul Syllable */
  ["\uD396", "\uD396"], /* Hangul Syllable */
  ["\uD397", "\uD397"], /* Hangul Syllable */
  ["\uD398", "\uD398"], /* Hangul Syllable */
  ["\uD399", "\uD399"], /* Hangul Syllable */
  ["\uD39A", "\uD39A"], /* Hangul Syllable */
  ["\uD39B", "\uD39B"], /* Hangul Syllable */
  ["\uD39C", "\uD39C"], /* Hangul Syllable */
  ["\uD39D", "\uD39D"], /* Hangul Syllable */
  ["\uD39E", "\uD39E"], /* Hangul Syllable */
  ["\uD39F", "\uD39F"], /* Hangul Syllable */
  ["\uD3A0", "\uD3A0"], /* Hangul Syllable */
  ["\uD3A1", "\uD3A1"], /* Hangul Syllable */
  ["\uD3A2", "\uD3A2"], /* Hangul Syllable */
  ["\uD3A3", "\uD3A3"], /* Hangul Syllable */
  ["\uD3A4", "\uD3A4"], /* Hangul Syllable */
  ["\uD3A5", "\uD3A5"], /* Hangul Syllable */
  ["\uD3A6", "\uD3A6"], /* Hangul Syllable */
  ["\uD3A7", "\uD3A7"], /* Hangul Syllable */
  ["\uD3A8", "\uD3A8"], /* Hangul Syllable */
  ["\uD3A9", "\uD3A9"], /* Hangul Syllable */
  ["\uD3AA", "\uD3AA"], /* Hangul Syllable */
  ["\uD3AB", "\uD3AB"], /* Hangul Syllable */
  ["\uD3AC", "\uD3AC"], /* Hangul Syllable */
  ["\uD3AD", "\uD3AD"], /* Hangul Syllable */
  ["\uD3AE", "\uD3AE"], /* Hangul Syllable */
  ["\uD3AF", "\uD3AF"], /* Hangul Syllable */
  ["\uD3B0", "\uD3B0"], /* Hangul Syllable */
  ["\uD3B1", "\uD3B1"], /* Hangul Syllable */
  ["\uD3B2", "\uD3B2"], /* Hangul Syllable */
  ["\uD3B3", "\uD3B3"], /* Hangul Syllable */
  ["\uD3B4", "\uD3B4"], /* Hangul Syllable */
  ["\uD3B5", "\uD3B5"], /* Hangul Syllable */
  ["\uD3B6", "\uD3B6"], /* Hangul Syllable */
  ["\uD3B7", "\uD3B7"], /* Hangul Syllable */
  ["\uD3B8", "\uD3B8"], /* Hangul Syllable */
  ["\uD3B9", "\uD3B9"], /* Hangul Syllable */
  ["\uD3BA", "\uD3BA"], /* Hangul Syllable */
  ["\uD3BB", "\uD3BB"], /* Hangul Syllable */
  ["\uD3BC", "\uD3BC"], /* Hangul Syllable */
  ["\uD3BD", "\uD3BD"], /* Hangul Syllable */
  ["\uD3BE", "\uD3BE"], /* Hangul Syllable */
  ["\uD3BF", "\uD3BF"], /* Hangul Syllable */
  ["\uD3C0", "\uD3C0"], /* Hangul Syllable */
  ["\uD3C1", "\uD3C1"], /* Hangul Syllable */
  ["\uD3C2", "\uD3C2"], /* Hangul Syllable */
  ["\uD3C3", "\uD3C3"], /* Hangul Syllable */
  ["\uD3C4", "\uD3C4"], /* Hangul Syllable */
  ["\uD3C5", "\uD3C5"], /* Hangul Syllable */
  ["\uD3C6", "\uD3C6"], /* Hangul Syllable */
  ["\uD3C7", "\uD3C7"], /* Hangul Syllable */
  ["\uD3C8", "\uD3C8"], /* Hangul Syllable */
  ["\uD3C9", "\uD3C9"], /* Hangul Syllable */
  ["\uD3CA", "\uD3CA"], /* Hangul Syllable */
  ["\uD3CB", "\uD3CB"], /* Hangul Syllable */
  ["\uD3CC", "\uD3CC"], /* Hangul Syllable */
  ["\uD3CD", "\uD3CD"], /* Hangul Syllable */
  ["\uD3CE", "\uD3CE"], /* Hangul Syllable */
  ["\uD3CF", "\uD3CF"], /* Hangul Syllable */
  ["\uD3D0", "\uD3D0"], /* Hangul Syllable */
  ["\uD3D1", "\uD3D1"], /* Hangul Syllable */
  ["\uD3D2", "\uD3D2"], /* Hangul Syllable */
  ["\uD3D3", "\uD3D3"], /* Hangul Syllable */
  ["\uD3D4", "\uD3D4"], /* Hangul Syllable */
  ["\uD3D5", "\uD3D5"], /* Hangul Syllable */
  ["\uD3D6", "\uD3D6"], /* Hangul Syllable */
  ["\uD3D7", "\uD3D7"], /* Hangul Syllable */
  ["\uD3D8", "\uD3D8"], /* Hangul Syllable */
  ["\uD3D9", "\uD3D9"], /* Hangul Syllable */
  ["\uD3DA", "\uD3DA"], /* Hangul Syllable */
  ["\uD3DB", "\uD3DB"], /* Hangul Syllable */
  ["\uD3DC", "\uD3DC"], /* Hangul Syllable */
  ["\uD3DD", "\uD3DD"], /* Hangul Syllable */
  ["\uD3DE", "\uD3DE"], /* Hangul Syllable */
  ["\uD3DF", "\uD3DF"], /* Hangul Syllable */
  ["\uD3E0", "\uD3E0"], /* Hangul Syllable */
  ["\uD3E1", "\uD3E1"], /* Hangul Syllable */
  ["\uD3E2", "\uD3E2"], /* Hangul Syllable */
  ["\uD3E3", "\uD3E3"], /* Hangul Syllable */
  ["\uD3E4", "\uD3E4"], /* Hangul Syllable */
  ["\uD3E5", "\uD3E5"], /* Hangul Syllable */
  ["\uD3E6", "\uD3E6"], /* Hangul Syllable */
  ["\uD3E7", "\uD3E7"], /* Hangul Syllable */
  ["\uD3E8", "\uD3E8"], /* Hangul Syllable */
  ["\uD3E9", "\uD3E9"], /* Hangul Syllable */
  ["\uD3EA", "\uD3EA"], /* Hangul Syllable */
  ["\uD3EB", "\uD3EB"], /* Hangul Syllable */
  ["\uD3EC", "\uD3EC"], /* Hangul Syllable */
  ["\uD3ED", "\uD3ED"], /* Hangul Syllable */
  ["\uD3EE", "\uD3EE"], /* Hangul Syllable */
  ["\uD3EF", "\uD3EF"], /* Hangul Syllable */
  ["\uD3F0", "\uD3F0"], /* Hangul Syllable */
  ["\uD3F1", "\uD3F1"], /* Hangul Syllable */
  ["\uD3F2", "\uD3F2"], /* Hangul Syllable */
  ["\uD3F3", "\uD3F3"], /* Hangul Syllable */
  ["\uD3F4", "\uD3F4"], /* Hangul Syllable */
  ["\uD3F5", "\uD3F5"], /* Hangul Syllable */
  ["\uD3F6", "\uD3F6"], /* Hangul Syllable */
  ["\uD3F7", "\uD3F7"], /* Hangul Syllable */
  ["\uD3F8", "\uD3F8"], /* Hangul Syllable */
  ["\uD3F9", "\uD3F9"], /* Hangul Syllable */
  ["\uD3FA", "\uD3FA"], /* Hangul Syllable */
  ["\uD3FB", "\uD3FB"], /* Hangul Syllable */
  ["\uD3FC", "\uD3FC"], /* Hangul Syllable */
  ["\uD3FD", "\uD3FD"], /* Hangul Syllable */
  ["\uD3FE", "\uD3FE"], /* Hangul Syllable */
  ["\uD3FF", "\uD3FF"], /* Hangul Syllable */
  ["\uD400", "\uD400"], /* Hangul Syllable */
  ["\uD401", "\uD401"], /* Hangul Syllable */
  ["\uD402", "\uD402"], /* Hangul Syllable */
  ["\uD403", "\uD403"], /* Hangul Syllable */
  ["\uD404", "\uD404"], /* Hangul Syllable */
  ["\uD405", "\uD405"], /* Hangul Syllable */
  ["\uD406", "\uD406"], /* Hangul Syllable */
  ["\uD407", "\uD407"], /* Hangul Syllable */
  ["\uD408", "\uD408"], /* Hangul Syllable */
  ["\uD409", "\uD409"], /* Hangul Syllable */
  ["\uD40A", "\uD40A"], /* Hangul Syllable */
  ["\uD40B", "\uD40B"], /* Hangul Syllable */
  ["\uD40C", "\uD40C"], /* Hangul Syllable */
  ["\uD40D", "\uD40D"], /* Hangul Syllable */
  ["\uD40E", "\uD40E"], /* Hangul Syllable */
  ["\uD40F", "\uD40F"], /* Hangul Syllable */
  ["\uD410", "\uD410"], /* Hangul Syllable */
  ["\uD411", "\uD411"], /* Hangul Syllable */
  ["\uD412", "\uD412"], /* Hangul Syllable */
  ["\uD413", "\uD413"], /* Hangul Syllable */
  ["\uD414", "\uD414"], /* Hangul Syllable */
  ["\uD415", "\uD415"], /* Hangul Syllable */
  ["\uD416", "\uD416"], /* Hangul Syllable */
  ["\uD417", "\uD417"], /* Hangul Syllable */
  ["\uD418", "\uD418"], /* Hangul Syllable */
  ["\uD419", "\uD419"], /* Hangul Syllable */
  ["\uD41A", "\uD41A"], /* Hangul Syllable */
  ["\uD41B", "\uD41B"], /* Hangul Syllable */
  ["\uD41C", "\uD41C"], /* Hangul Syllable */
  ["\uD41D", "\uD41D"], /* Hangul Syllable */
  ["\uD41E", "\uD41E"], /* Hangul Syllable */
  ["\uD41F", "\uD41F"], /* Hangul Syllable */
  ["\uD420", "\uD420"], /* Hangul Syllable */
  ["\uD421", "\uD421"], /* Hangul Syllable */
  ["\uD422", "\uD422"], /* Hangul Syllable */
  ["\uD423", "\uD423"], /* Hangul Syllable */
  ["\uD424", "\uD424"], /* Hangul Syllable */
  ["\uD425", "\uD425"], /* Hangul Syllable */
  ["\uD426", "\uD426"], /* Hangul Syllable */
  ["\uD427", "\uD427"], /* Hangul Syllable */
  ["\uD428", "\uD428"], /* Hangul Syllable */
  ["\uD429", "\uD429"], /* Hangul Syllable */
  ["\uD42A", "\uD42A"], /* Hangul Syllable */
  ["\uD42B", "\uD42B"], /* Hangul Syllable */
  ["\uD42C", "\uD42C"], /* Hangul Syllable */
  ["\uD42D", "\uD42D"], /* Hangul Syllable */
  ["\uD42E", "\uD42E"], /* Hangul Syllable */
  ["\uD42F", "\uD42F"], /* Hangul Syllable */
  ["\uD430", "\uD430"], /* Hangul Syllable */
  ["\uD431", "\uD431"], /* Hangul Syllable */
  ["\uD432", "\uD432"], /* Hangul Syllable */
  ["\uD433", "\uD433"], /* Hangul Syllable */
  ["\uD434", "\uD434"], /* Hangul Syllable */
  ["\uD435", "\uD435"], /* Hangul Syllable */
  ["\uD436", "\uD436"], /* Hangul Syllable */
  ["\uD437", "\uD437"], /* Hangul Syllable */
  ["\uD438", "\uD438"], /* Hangul Syllable */
  ["\uD439", "\uD439"], /* Hangul Syllable */
  ["\uD43A", "\uD43A"], /* Hangul Syllable */
  ["\uD43B", "\uD43B"], /* Hangul Syllable */
  ["\uD43C", "\uD43C"], /* Hangul Syllable */
  ["\uD43D", "\uD43D"], /* Hangul Syllable */
  ["\uD43E", "\uD43E"], /* Hangul Syllable */
  ["\uD43F", "\uD43F"], /* Hangul Syllable */
  ["\uD440", "\uD440"], /* Hangul Syllable */
  ["\uD441", "\uD441"], /* Hangul Syllable */
  ["\uD442", "\uD442"], /* Hangul Syllable */
  ["\uD443", "\uD443"], /* Hangul Syllable */
  ["\uD444", "\uD444"], /* Hangul Syllable */
  ["\uD445", "\uD445"], /* Hangul Syllable */
  ["\uD446", "\uD446"], /* Hangul Syllable */
  ["\uD447", "\uD447"], /* Hangul Syllable */
  ["\uD448", "\uD448"], /* Hangul Syllable */
  ["\uD449", "\uD449"], /* Hangul Syllable */
  ["\uD44A", "\uD44A"], /* Hangul Syllable */
  ["\uD44B", "\uD44B"], /* Hangul Syllable */
  ["\uD44C", "\uD44C"], /* Hangul Syllable */
  ["\uD44D", "\uD44D"], /* Hangul Syllable */
  ["\uD44E", "\uD44E"], /* Hangul Syllable */
  ["\uD44F", "\uD44F"], /* Hangul Syllable */
  ["\uD450", "\uD450"], /* Hangul Syllable */
  ["\uD451", "\uD451"], /* Hangul Syllable */
  ["\uD452", "\uD452"], /* Hangul Syllable */
  ["\uD453", "\uD453"], /* Hangul Syllable */
  ["\uD454", "\uD454"], /* Hangul Syllable */
  ["\uD455", "\uD455"], /* Hangul Syllable */
  ["\uD456", "\uD456"], /* Hangul Syllable */
  ["\uD457", "\uD457"], /* Hangul Syllable */
  ["\uD458", "\uD458"], /* Hangul Syllable */
  ["\uD459", "\uD459"], /* Hangul Syllable */
  ["\uD45A", "\uD45A"], /* Hangul Syllable */
  ["\uD45B", "\uD45B"], /* Hangul Syllable */
  ["\uD45C", "\uD45C"], /* Hangul Syllable */
  ["\uD45D", "\uD45D"], /* Hangul Syllable */
  ["\uD45E", "\uD45E"], /* Hangul Syllable */
  ["\uD45F", "\uD45F"], /* Hangul Syllable */
  ["\uD460", "\uD460"], /* Hangul Syllable */
  ["\uD461", "\uD461"], /* Hangul Syllable */
  ["\uD462", "\uD462"], /* Hangul Syllable */
  ["\uD463", "\uD463"], /* Hangul Syllable */
  ["\uD464", "\uD464"], /* Hangul Syllable */
  ["\uD465", "\uD465"], /* Hangul Syllable */
  ["\uD466", "\uD466"], /* Hangul Syllable */
  ["\uD467", "\uD467"], /* Hangul Syllable */
  ["\uD468", "\uD468"], /* Hangul Syllable */
  ["\uD469", "\uD469"], /* Hangul Syllable */
  ["\uD46A", "\uD46A"], /* Hangul Syllable */
  ["\uD46B", "\uD46B"], /* Hangul Syllable */
  ["\uD46C", "\uD46C"], /* Hangul Syllable */
  ["\uD46D", "\uD46D"], /* Hangul Syllable */
  ["\uD46E", "\uD46E"], /* Hangul Syllable */
  ["\uD46F", "\uD46F"], /* Hangul Syllable */
  ["\uD470", "\uD470"], /* Hangul Syllable */
  ["\uD471", "\uD471"], /* Hangul Syllable */
  ["\uD472", "\uD472"], /* Hangul Syllable */
  ["\uD473", "\uD473"], /* Hangul Syllable */
  ["\uD474", "\uD474"], /* Hangul Syllable */
  ["\uD475", "\uD475"], /* Hangul Syllable */
  ["\uD476", "\uD476"], /* Hangul Syllable */
  ["\uD477", "\uD477"], /* Hangul Syllable */
  ["\uD478", "\uD478"], /* Hangul Syllable */
  ["\uD479", "\uD479"], /* Hangul Syllable */
  ["\uD47A", "\uD47A"], /* Hangul Syllable */
  ["\uD47B", "\uD47B"], /* Hangul Syllable */
  ["\uD47C", "\uD47C"], /* Hangul Syllable */
  ["\uD47D", "\uD47D"], /* Hangul Syllable */
  ["\uD47E", "\uD47E"], /* Hangul Syllable */
  ["\uD47F", "\uD47F"], /* Hangul Syllable */
  ["\uD480", "\uD480"], /* Hangul Syllable */
  ["\uD481", "\uD481"], /* Hangul Syllable */
  ["\uD482", "\uD482"], /* Hangul Syllable */
  ["\uD483", "\uD483"], /* Hangul Syllable */
  ["\uD484", "\uD484"], /* Hangul Syllable */
  ["\uD485", "\uD485"], /* Hangul Syllable */
  ["\uD486", "\uD486"], /* Hangul Syllable */
  ["\uD487", "\uD487"], /* Hangul Syllable */
  ["\uD488", "\uD488"], /* Hangul Syllable */
  ["\uD489", "\uD489"], /* Hangul Syllable */
  ["\uD48A", "\uD48A"], /* Hangul Syllable */
  ["\uD48B", "\uD48B"], /* Hangul Syllable */
  ["\uD48C", "\uD48C"], /* Hangul Syllable */
  ["\uD48D", "\uD48D"], /* Hangul Syllable */
  ["\uD48E", "\uD48E"], /* Hangul Syllable */
  ["\uD48F", "\uD48F"], /* Hangul Syllable */
  ["\uD490", "\uD490"], /* Hangul Syllable */
  ["\uD491", "\uD491"], /* Hangul Syllable */
  ["\uD492", "\uD492"], /* Hangul Syllable */
  ["\uD493", "\uD493"], /* Hangul Syllable */
  ["\uD494", "\uD494"], /* Hangul Syllable */
  ["\uD495", "\uD495"], /* Hangul Syllable */
  ["\uD496", "\uD496"], /* Hangul Syllable */
  ["\uD497", "\uD497"], /* Hangul Syllable */
  ["\uD498", "\uD498"], /* Hangul Syllable */
  ["\uD499", "\uD499"], /* Hangul Syllable */
  ["\uD49A", "\uD49A"], /* Hangul Syllable */
  ["\uD49B", "\uD49B"], /* Hangul Syllable */
  ["\uD49C", "\uD49C"], /* Hangul Syllable */
  ["\uD49D", "\uD49D"], /* Hangul Syllable */
  ["\uD49E", "\uD49E"], /* Hangul Syllable */
  ["\uD49F", "\uD49F"], /* Hangul Syllable */
  ["\uD4A0", "\uD4A0"], /* Hangul Syllable */
  ["\uD4A1", "\uD4A1"], /* Hangul Syllable */
  ["\uD4A2", "\uD4A2"], /* Hangul Syllable */
  ["\uD4A3", "\uD4A3"], /* Hangul Syllable */
  ["\uD4A4", "\uD4A4"], /* Hangul Syllable */
  ["\uD4A5", "\uD4A5"], /* Hangul Syllable */
  ["\uD4A6", "\uD4A6"], /* Hangul Syllable */
  ["\uD4A7", "\uD4A7"], /* Hangul Syllable */
  ["\uD4A8", "\uD4A8"], /* Hangul Syllable */
  ["\uD4A9", "\uD4A9"], /* Hangul Syllable */
  ["\uD4AA", "\uD4AA"], /* Hangul Syllable */
  ["\uD4AB", "\uD4AB"], /* Hangul Syllable */
  ["\uD4AC", "\uD4AC"], /* Hangul Syllable */
  ["\uD4AD", "\uD4AD"], /* Hangul Syllable */
  ["\uD4AE", "\uD4AE"], /* Hangul Syllable */
  ["\uD4AF", "\uD4AF"], /* Hangul Syllable */
  ["\uD4B0", "\uD4B0"], /* Hangul Syllable */
  ["\uD4B1", "\uD4B1"], /* Hangul Syllable */
  ["\uD4B2", "\uD4B2"], /* Hangul Syllable */
  ["\uD4B3", "\uD4B3"], /* Hangul Syllable */
  ["\uD4B4", "\uD4B4"], /* Hangul Syllable */
  ["\uD4B5", "\uD4B5"], /* Hangul Syllable */
  ["\uD4B6", "\uD4B6"], /* Hangul Syllable */
  ["\uD4B7", "\uD4B7"], /* Hangul Syllable */
  ["\uD4B8", "\uD4B8"], /* Hangul Syllable */
  ["\uD4B9", "\uD4B9"], /* Hangul Syllable */
  ["\uD4BA", "\uD4BA"], /* Hangul Syllable */
  ["\uD4BB", "\uD4BB"], /* Hangul Syllable */
  ["\uD4BC", "\uD4BC"], /* Hangul Syllable */
  ["\uD4BD", "\uD4BD"], /* Hangul Syllable */
  ["\uD4BE", "\uD4BE"], /* Hangul Syllable */
  ["\uD4BF", "\uD4BF"], /* Hangul Syllable */
  ["\uD4C0", "\uD4C0"], /* Hangul Syllable */
  ["\uD4C1", "\uD4C1"], /* Hangul Syllable */
  ["\uD4C2", "\uD4C2"], /* Hangul Syllable */
  ["\uD4C3", "\uD4C3"], /* Hangul Syllable */
  ["\uD4C4", "\uD4C4"], /* Hangul Syllable */
  ["\uD4C5", "\uD4C5"], /* Hangul Syllable */
  ["\uD4C6", "\uD4C6"], /* Hangul Syllable */
  ["\uD4C7", "\uD4C7"], /* Hangul Syllable */
  ["\uD4C8", "\uD4C8"], /* Hangul Syllable */
  ["\uD4C9", "\uD4C9"], /* Hangul Syllable */
  ["\uD4CA", "\uD4CA"], /* Hangul Syllable */
  ["\uD4CB", "\uD4CB"], /* Hangul Syllable */
  ["\uD4CC", "\uD4CC"], /* Hangul Syllable */
  ["\uD4CD", "\uD4CD"], /* Hangul Syllable */
  ["\uD4CE", "\uD4CE"], /* Hangul Syllable */
  ["\uD4CF", "\uD4CF"], /* Hangul Syllable */
  ["\uD4D0", "\uD4D0"], /* Hangul Syllable */
  ["\uD4D1", "\uD4D1"], /* Hangul Syllable */
  ["\uD4D2", "\uD4D2"], /* Hangul Syllable */
  ["\uD4D3", "\uD4D3"], /* Hangul Syllable */
  ["\uD4D4", "\uD4D4"], /* Hangul Syllable */
  ["\uD4D5", "\uD4D5"], /* Hangul Syllable */
  ["\uD4D6", "\uD4D6"], /* Hangul Syllable */
  ["\uD4D7", "\uD4D7"], /* Hangul Syllable */
  ["\uD4D8", "\uD4D8"], /* Hangul Syllable */
  ["\uD4D9", "\uD4D9"], /* Hangul Syllable */
  ["\uD4DA", "\uD4DA"], /* Hangul Syllable */
  ["\uD4DB", "\uD4DB"], /* Hangul Syllable */
  ["\uD4DC", "\uD4DC"], /* Hangul Syllable */
  ["\uD4DD", "\uD4DD"], /* Hangul Syllable */
  ["\uD4DE", "\uD4DE"], /* Hangul Syllable */
  ["\uD4DF", "\uD4DF"], /* Hangul Syllable */
  ["\uD4E0", "\uD4E0"], /* Hangul Syllable */
  ["\uD4E1", "\uD4E1"], /* Hangul Syllable */
  ["\uD4E2", "\uD4E2"], /* Hangul Syllable */
  ["\uD4E3", "\uD4E3"], /* Hangul Syllable */
  ["\uD4E4", "\uD4E4"], /* Hangul Syllable */
  ["\uD4E5", "\uD4E5"], /* Hangul Syllable */
  ["\uD4E6", "\uD4E6"], /* Hangul Syllable */
  ["\uD4E7", "\uD4E7"], /* Hangul Syllable */
  ["\uD4E8", "\uD4E8"], /* Hangul Syllable */
  ["\uD4E9", "\uD4E9"], /* Hangul Syllable */
  ["\uD4EA", "\uD4EA"], /* Hangul Syllable */
  ["\uD4EB", "\uD4EB"], /* Hangul Syllable */
  ["\uD4EC", "\uD4EC"], /* Hangul Syllable */
  ["\uD4ED", "\uD4ED"], /* Hangul Syllable */
  ["\uD4EE", "\uD4EE"], /* Hangul Syllable */
  ["\uD4EF", "\uD4EF"], /* Hangul Syllable */
  ["\uD4F0", "\uD4F0"], /* Hangul Syllable */
  ["\uD4F1", "\uD4F1"], /* Hangul Syllable */
  ["\uD4F2", "\uD4F2"], /* Hangul Syllable */
  ["\uD4F3", "\uD4F3"], /* Hangul Syllable */
  ["\uD4F4", "\uD4F4"], /* Hangul Syllable */
  ["\uD4F5", "\uD4F5"], /* Hangul Syllable */
  ["\uD4F6", "\uD4F6"], /* Hangul Syllable */
  ["\uD4F7", "\uD4F7"], /* Hangul Syllable */
  ["\uD4F8", "\uD4F8"], /* Hangul Syllable */
  ["\uD4F9", "\uD4F9"], /* Hangul Syllable */
  ["\uD4FA", "\uD4FA"], /* Hangul Syllable */
  ["\uD4FB", "\uD4FB"], /* Hangul Syllable */
  ["\uD4FC", "\uD4FC"], /* Hangul Syllable */
  ["\uD4FD", "\uD4FD"], /* Hangul Syllable */
  ["\uD4FE", "\uD4FE"], /* Hangul Syllable */
  ["\uD4FF", "\uD4FF"], /* Hangul Syllable */
  ["\uD500", "\uD500"], /* Hangul Syllable */
  ["\uD501", "\uD501"], /* Hangul Syllable */
  ["\uD502", "\uD502"], /* Hangul Syllable */
  ["\uD503", "\uD503"], /* Hangul Syllable */
  ["\uD504", "\uD504"], /* Hangul Syllable */
  ["\uD505", "\uD505"], /* Hangul Syllable */
  ["\uD506", "\uD506"], /* Hangul Syllable */
  ["\uD507", "\uD507"], /* Hangul Syllable */
  ["\uD508", "\uD508"], /* Hangul Syllable */
  ["\uD509", "\uD509"], /* Hangul Syllable */
  ["\uD50A", "\uD50A"], /* Hangul Syllable */
  ["\uD50B", "\uD50B"], /* Hangul Syllable */
  ["\uD50C", "\uD50C"], /* Hangul Syllable */
  ["\uD50D", "\uD50D"], /* Hangul Syllable */
  ["\uD50E", "\uD50E"], /* Hangul Syllable */
  ["\uD50F", "\uD50F"], /* Hangul Syllable */
  ["\uD510", "\uD510"], /* Hangul Syllable */
  ["\uD511", "\uD511"], /* Hangul Syllable */
  ["\uD512", "\uD512"], /* Hangul Syllable */
  ["\uD513", "\uD513"], /* Hangul Syllable */
  ["\uD514", "\uD514"], /* Hangul Syllable */
  ["\uD515", "\uD515"], /* Hangul Syllable */
  ["\uD516", "\uD516"], /* Hangul Syllable */
  ["\uD517", "\uD517"], /* Hangul Syllable */
  ["\uD518", "\uD518"], /* Hangul Syllable */
  ["\uD519", "\uD519"], /* Hangul Syllable */
  ["\uD51A", "\uD51A"], /* Hangul Syllable */
  ["\uD51B", "\uD51B"], /* Hangul Syllable */
  ["\uD51C", "\uD51C"], /* Hangul Syllable */
  ["\uD51D", "\uD51D"], /* Hangul Syllable */
  ["\uD51E", "\uD51E"], /* Hangul Syllable */
  ["\uD51F", "\uD51F"], /* Hangul Syllable */
  ["\uD520", "\uD520"], /* Hangul Syllable */
  ["\uD521", "\uD521"], /* Hangul Syllable */
  ["\uD522", "\uD522"], /* Hangul Syllable */
  ["\uD523", "\uD523"], /* Hangul Syllable */
  ["\uD524", "\uD524"], /* Hangul Syllable */
  ["\uD525", "\uD525"], /* Hangul Syllable */
  ["\uD526", "\uD526"], /* Hangul Syllable */
  ["\uD527", "\uD527"], /* Hangul Syllable */
  ["\uD528", "\uD528"], /* Hangul Syllable */
  ["\uD529", "\uD529"], /* Hangul Syllable */
  ["\uD52A", "\uD52A"], /* Hangul Syllable */
  ["\uD52B", "\uD52B"], /* Hangul Syllable */
  ["\uD52C", "\uD52C"], /* Hangul Syllable */
  ["\uD52D", "\uD52D"], /* Hangul Syllable */
  ["\uD52E", "\uD52E"], /* Hangul Syllable */
  ["\uD52F", "\uD52F"], /* Hangul Syllable */
  ["\uD530", "\uD530"], /* Hangul Syllable */
  ["\uD531", "\uD531"], /* Hangul Syllable */
  ["\uD532", "\uD532"], /* Hangul Syllable */
  ["\uD533", "\uD533"], /* Hangul Syllable */
  ["\uD534", "\uD534"], /* Hangul Syllable */
  ["\uD535", "\uD535"], /* Hangul Syllable */
  ["\uD536", "\uD536"], /* Hangul Syllable */
  ["\uD537", "\uD537"], /* Hangul Syllable */
  ["\uD538", "\uD538"], /* Hangul Syllable */
  ["\uD539", "\uD539"], /* Hangul Syllable */
  ["\uD53A", "\uD53A"], /* Hangul Syllable */
  ["\uD53B", "\uD53B"], /* Hangul Syllable */
  ["\uD53C", "\uD53C"], /* Hangul Syllable */
  ["\uD53D", "\uD53D"], /* Hangul Syllable */
  ["\uD53E", "\uD53E"], /* Hangul Syllable */
  ["\uD53F", "\uD53F"], /* Hangul Syllable */
  ["\uD540", "\uD540"], /* Hangul Syllable */
  ["\uD541", "\uD541"], /* Hangul Syllable */
  ["\uD542", "\uD542"], /* Hangul Syllable */
  ["\uD543", "\uD543"], /* Hangul Syllable */
  ["\uD544", "\uD544"], /* Hangul Syllable */
  ["\uD545", "\uD545"], /* Hangul Syllable */
  ["\uD546", "\uD546"], /* Hangul Syllable */
  ["\uD547", "\uD547"], /* Hangul Syllable */
  ["\uD548", "\uD548"], /* Hangul Syllable */
  ["\uD549", "\uD549"], /* Hangul Syllable */
  ["\uD54A", "\uD54A"], /* Hangul Syllable */
  ["\uD54B", "\uD54B"], /* Hangul Syllable */
  ["\uD54C", "\uD54C"], /* Hangul Syllable */
  ["\uD54D", "\uD54D"], /* Hangul Syllable */
  ["\uD54E", "\uD54E"], /* Hangul Syllable */
  ["\uD54F", "\uD54F"], /* Hangul Syllable */
  ["\uD550", "\uD550"], /* Hangul Syllable */
  ["\uD551", "\uD551"], /* Hangul Syllable */
  ["\uD552", "\uD552"], /* Hangul Syllable */
  ["\uD553", "\uD553"], /* Hangul Syllable */
  ["\uD554", "\uD554"], /* Hangul Syllable */
  ["\uD555", "\uD555"], /* Hangul Syllable */
  ["\uD556", "\uD556"], /* Hangul Syllable */
  ["\uD557", "\uD557"], /* Hangul Syllable */
  ["\uD558", "\uD558"], /* Hangul Syllable */
  ["\uD559", "\uD559"], /* Hangul Syllable */
  ["\uD55A", "\uD55A"], /* Hangul Syllable */
  ["\uD55B", "\uD55B"], /* Hangul Syllable */
  ["\uD55C", "\uD55C"], /* Hangul Syllable */
  ["\uD55D", "\uD55D"], /* Hangul Syllable */
  ["\uD55E", "\uD55E"], /* Hangul Syllable */
  ["\uD55F", "\uD55F"], /* Hangul Syllable */
  ["\uD560", "\uD560"], /* Hangul Syllable */
  ["\uD561", "\uD561"], /* Hangul Syllable */
  ["\uD562", "\uD562"], /* Hangul Syllable */
  ["\uD563", "\uD563"], /* Hangul Syllable */
  ["\uD564", "\uD564"], /* Hangul Syllable */
  ["\uD565", "\uD565"], /* Hangul Syllable */
  ["\uD566", "\uD566"], /* Hangul Syllable */
  ["\uD567", "\uD567"], /* Hangul Syllable */
  ["\uD568", "\uD568"], /* Hangul Syllable */
  ["\uD569", "\uD569"], /* Hangul Syllable */
  ["\uD56A", "\uD56A"], /* Hangul Syllable */
  ["\uD56B", "\uD56B"], /* Hangul Syllable */
  ["\uD56C", "\uD56C"], /* Hangul Syllable */
  ["\uD56D", "\uD56D"], /* Hangul Syllable */
  ["\uD56E", "\uD56E"], /* Hangul Syllable */
  ["\uD56F", "\uD56F"], /* Hangul Syllable */
  ["\uD570", "\uD570"], /* Hangul Syllable */
  ["\uD571", "\uD571"], /* Hangul Syllable */
  ["\uD572", "\uD572"], /* Hangul Syllable */
  ["\uD573", "\uD573"], /* Hangul Syllable */
  ["\uD574", "\uD574"], /* Hangul Syllable */
  ["\uD575", "\uD575"], /* Hangul Syllable */
  ["\uD576", "\uD576"], /* Hangul Syllable */
  ["\uD577", "\uD577"], /* Hangul Syllable */
  ["\uD578", "\uD578"], /* Hangul Syllable */
  ["\uD579", "\uD579"], /* Hangul Syllable */
  ["\uD57A", "\uD57A"], /* Hangul Syllable */
  ["\uD57B", "\uD57B"], /* Hangul Syllable */
  ["\uD57C", "\uD57C"], /* Hangul Syllable */
  ["\uD57D", "\uD57D"], /* Hangul Syllable */
  ["\uD57E", "\uD57E"], /* Hangul Syllable */
  ["\uD57F", "\uD57F"], /* Hangul Syllable */
  ["\uD580", "\uD580"], /* Hangul Syllable */
  ["\uD581", "\uD581"], /* Hangul Syllable */
  ["\uD582", "\uD582"], /* Hangul Syllable */
  ["\uD583", "\uD583"], /* Hangul Syllable */
  ["\uD584", "\uD584"], /* Hangul Syllable */
  ["\uD585", "\uD585"], /* Hangul Syllable */
  ["\uD586", "\uD586"], /* Hangul Syllable */
  ["\uD587", "\uD587"], /* Hangul Syllable */
  ["\uD588", "\uD588"], /* Hangul Syllable */
  ["\uD589", "\uD589"], /* Hangul Syllable */
  ["\uD58A", "\uD58A"], /* Hangul Syllable */
  ["\uD58B", "\uD58B"], /* Hangul Syllable */
  ["\uD58C", "\uD58C"], /* Hangul Syllable */
  ["\uD58D", "\uD58D"], /* Hangul Syllable */
  ["\uD58E", "\uD58E"], /* Hangul Syllable */
  ["\uD58F", "\uD58F"], /* Hangul Syllable */
  ["\uD590", "\uD590"], /* Hangul Syllable */
  ["\uD591", "\uD591"], /* Hangul Syllable */
  ["\uD592", "\uD592"], /* Hangul Syllable */
  ["\uD593", "\uD593"], /* Hangul Syllable */
  ["\uD594", "\uD594"], /* Hangul Syllable */
  ["\uD595", "\uD595"], /* Hangul Syllable */
  ["\uD596", "\uD596"], /* Hangul Syllable */
  ["\uD597", "\uD597"], /* Hangul Syllable */
  ["\uD598", "\uD598"], /* Hangul Syllable */
  ["\uD599", "\uD599"], /* Hangul Syllable */
  ["\uD59A", "\uD59A"], /* Hangul Syllable */
  ["\uD59B", "\uD59B"], /* Hangul Syllable */
  ["\uD59C", "\uD59C"], /* Hangul Syllable */
  ["\uD59D", "\uD59D"], /* Hangul Syllable */
  ["\uD59E", "\uD59E"], /* Hangul Syllable */
  ["\uD59F", "\uD59F"], /* Hangul Syllable */
  ["\uD5A0", "\uD5A0"], /* Hangul Syllable */
  ["\uD5A1", "\uD5A1"], /* Hangul Syllable */
  ["\uD5A2", "\uD5A2"], /* Hangul Syllable */
  ["\uD5A3", "\uD5A3"], /* Hangul Syllable */
  ["\uD5A4", "\uD5A4"], /* Hangul Syllable */
  ["\uD5A5", "\uD5A5"], /* Hangul Syllable */
  ["\uD5A6", "\uD5A6"], /* Hangul Syllable */
  ["\uD5A7", "\uD5A7"], /* Hangul Syllable */
  ["\uD5A8", "\uD5A8"], /* Hangul Syllable */
  ["\uD5A9", "\uD5A9"], /* Hangul Syllable */
  ["\uD5AA", "\uD5AA"], /* Hangul Syllable */
  ["\uD5AB", "\uD5AB"], /* Hangul Syllable */
  ["\uD5AC", "\uD5AC"], /* Hangul Syllable */
  ["\uD5AD", "\uD5AD"], /* Hangul Syllable */
  ["\uD5AE", "\uD5AE"], /* Hangul Syllable */
  ["\uD5AF", "\uD5AF"], /* Hangul Syllable */
  ["\uD5B0", "\uD5B0"], /* Hangul Syllable */
  ["\uD5B1", "\uD5B1"], /* Hangul Syllable */
  ["\uD5B2", "\uD5B2"], /* Hangul Syllable */
  ["\uD5B3", "\uD5B3"], /* Hangul Syllable */
  ["\uD5B4", "\uD5B4"], /* Hangul Syllable */
  ["\uD5B5", "\uD5B5"], /* Hangul Syllable */
  ["\uD5B6", "\uD5B6"], /* Hangul Syllable */
  ["\uD5B7", "\uD5B7"], /* Hangul Syllable */
  ["\uD5B8", "\uD5B8"], /* Hangul Syllable */
  ["\uD5B9", "\uD5B9"], /* Hangul Syllable */
  ["\uD5BA", "\uD5BA"], /* Hangul Syllable */
  ["\uD5BB", "\uD5BB"], /* Hangul Syllable */
  ["\uD5BC", "\uD5BC"], /* Hangul Syllable */
  ["\uD5BD", "\uD5BD"], /* Hangul Syllable */
  ["\uD5BE", "\uD5BE"], /* Hangul Syllable */
  ["\uD5BF", "\uD5BF"], /* Hangul Syllable */
  ["\uD5C0", "\uD5C0"], /* Hangul Syllable */
  ["\uD5C1", "\uD5C1"], /* Hangul Syllable */
  ["\uD5C2", "\uD5C2"], /* Hangul Syllable */
  ["\uD5C3", "\uD5C3"], /* Hangul Syllable */
  ["\uD5C4", "\uD5C4"], /* Hangul Syllable */
  ["\uD5C5", "\uD5C5"], /* Hangul Syllable */
  ["\uD5C6", "\uD5C6"], /* Hangul Syllable */
  ["\uD5C7", "\uD5C7"], /* Hangul Syllable */
  ["\uD5C8", "\uD5C8"], /* Hangul Syllable */
  ["\uD5C9", "\uD5C9"], /* Hangul Syllable */
  ["\uD5CA", "\uD5CA"], /* Hangul Syllable */
  ["\uD5CB", "\uD5CB"], /* Hangul Syllable */
  ["\uD5CC", "\uD5CC"], /* Hangul Syllable */
  ["\uD5CD", "\uD5CD"], /* Hangul Syllable */
  ["\uD5CE", "\uD5CE"], /* Hangul Syllable */
  ["\uD5CF", "\uD5CF"], /* Hangul Syllable */
  ["\uD5D0", "\uD5D0"], /* Hangul Syllable */
  ["\uD5D1", "\uD5D1"], /* Hangul Syllable */
  ["\uD5D2", "\uD5D2"], /* Hangul Syllable */
  ["\uD5D3", "\uD5D3"], /* Hangul Syllable */
  ["\uD5D4", "\uD5D4"], /* Hangul Syllable */
  ["\uD5D5", "\uD5D5"], /* Hangul Syllable */
  ["\uD5D6", "\uD5D6"], /* Hangul Syllable */
  ["\uD5D7", "\uD5D7"], /* Hangul Syllable */
  ["\uD5D8", "\uD5D8"], /* Hangul Syllable */
  ["\uD5D9", "\uD5D9"], /* Hangul Syllable */
  ["\uD5DA", "\uD5DA"], /* Hangul Syllable */
  ["\uD5DB", "\uD5DB"], /* Hangul Syllable */
  ["\uD5DC", "\uD5DC"], /* Hangul Syllable */
  ["\uD5DD", "\uD5DD"], /* Hangul Syllable */
  ["\uD5DE", "\uD5DE"], /* Hangul Syllable */
  ["\uD5DF", "\uD5DF"], /* Hangul Syllable */
  ["\uD5E0", "\uD5E0"], /* Hangul Syllable */
  ["\uD5E1", "\uD5E1"], /* Hangul Syllable */
  ["\uD5E2", "\uD5E2"], /* Hangul Syllable */
  ["\uD5E3", "\uD5E3"], /* Hangul Syllable */
  ["\uD5E4", "\uD5E4"], /* Hangul Syllable */
  ["\uD5E5", "\uD5E5"], /* Hangul Syllable */
  ["\uD5E6", "\uD5E6"], /* Hangul Syllable */
  ["\uD5E7", "\uD5E7"], /* Hangul Syllable */
  ["\uD5E8", "\uD5E8"], /* Hangul Syllable */
  ["\uD5E9", "\uD5E9"], /* Hangul Syllable */
  ["\uD5EA", "\uD5EA"], /* Hangul Syllable */
  ["\uD5EB", "\uD5EB"], /* Hangul Syllable */
  ["\uD5EC", "\uD5EC"], /* Hangul Syllable */
  ["\uD5ED", "\uD5ED"], /* Hangul Syllable */
  ["\uD5EE", "\uD5EE"], /* Hangul Syllable */
  ["\uD5EF", "\uD5EF"], /* Hangul Syllable */
  ["\uD5F0", "\uD5F0"], /* Hangul Syllable */
  ["\uD5F1", "\uD5F1"], /* Hangul Syllable */
  ["\uD5F2", "\uD5F2"], /* Hangul Syllable */
  ["\uD5F3", "\uD5F3"], /* Hangul Syllable */
  ["\uD5F4", "\uD5F4"], /* Hangul Syllable */
  ["\uD5F5", "\uD5F5"], /* Hangul Syllable */
  ["\uD5F6", "\uD5F6"], /* Hangul Syllable */
  ["\uD5F7", "\uD5F7"], /* Hangul Syllable */
  ["\uD5F8", "\uD5F8"], /* Hangul Syllable */
  ["\uD5F9", "\uD5F9"], /* Hangul Syllable */
  ["\uD5FA", "\uD5FA"], /* Hangul Syllable */
  ["\uD5FB", "\uD5FB"], /* Hangul Syllable */
  ["\uD5FC", "\uD5FC"], /* Hangul Syllable */
  ["\uD5FD", "\uD5FD"], /* Hangul Syllable */
  ["\uD5FE", "\uD5FE"], /* Hangul Syllable */
  ["\uD5FF", "\uD5FF"], /* Hangul Syllable */
  ["\uD600", "\uD600"], /* Hangul Syllable */
  ["\uD601", "\uD601"], /* Hangul Syllable */
  ["\uD602", "\uD602"], /* Hangul Syllable */
  ["\uD603", "\uD603"], /* Hangul Syllable */
  ["\uD604", "\uD604"], /* Hangul Syllable */
  ["\uD605", "\uD605"], /* Hangul Syllable */
  ["\uD606", "\uD606"], /* Hangul Syllable */
  ["\uD607", "\uD607"], /* Hangul Syllable */
  ["\uD608", "\uD608"], /* Hangul Syllable */
  ["\uD609", "\uD609"], /* Hangul Syllable */
  ["\uD60A", "\uD60A"], /* Hangul Syllable */
  ["\uD60B", "\uD60B"], /* Hangul Syllable */
  ["\uD60C", "\uD60C"], /* Hangul Syllable */
  ["\uD60D", "\uD60D"], /* Hangul Syllable */
  ["\uD60E", "\uD60E"], /* Hangul Syllable */
  ["\uD60F", "\uD60F"], /* Hangul Syllable */
  ["\uD610", "\uD610"], /* Hangul Syllable */
  ["\uD611", "\uD611"], /* Hangul Syllable */
  ["\uD612", "\uD612"], /* Hangul Syllable */
  ["\uD613", "\uD613"], /* Hangul Syllable */
  ["\uD614", "\uD614"], /* Hangul Syllable */
  ["\uD615", "\uD615"], /* Hangul Syllable */
  ["\uD616", "\uD616"], /* Hangul Syllable */
  ["\uD617", "\uD617"], /* Hangul Syllable */
  ["\uD618", "\uD618"], /* Hangul Syllable */
  ["\uD619", "\uD619"], /* Hangul Syllable */
  ["\uD61A", "\uD61A"], /* Hangul Syllable */
  ["\uD61B", "\uD61B"], /* Hangul Syllable */
  ["\uD61C", "\uD61C"], /* Hangul Syllable */
  ["\uD61D", "\uD61D"], /* Hangul Syllable */
  ["\uD61E", "\uD61E"], /* Hangul Syllable */
  ["\uD61F", "\uD61F"], /* Hangul Syllable */
  ["\uD620", "\uD620"], /* Hangul Syllable */
  ["\uD621", "\uD621"], /* Hangul Syllable */
  ["\uD622", "\uD622"], /* Hangul Syllable */
  ["\uD623", "\uD623"], /* Hangul Syllable */
  ["\uD624", "\uD624"], /* Hangul Syllable */
  ["\uD625", "\uD625"], /* Hangul Syllable */
  ["\uD626", "\uD626"], /* Hangul Syllable */
  ["\uD627", "\uD627"], /* Hangul Syllable */
  ["\uD628", "\uD628"], /* Hangul Syllable */
  ["\uD629", "\uD629"], /* Hangul Syllable */
  ["\uD62A", "\uD62A"], /* Hangul Syllable */
  ["\uD62B", "\uD62B"], /* Hangul Syllable */
  ["\uD62C", "\uD62C"], /* Hangul Syllable */
  ["\uD62D", "\uD62D"], /* Hangul Syllable */
  ["\uD62E", "\uD62E"], /* Hangul Syllable */
  ["\uD62F", "\uD62F"], /* Hangul Syllable */
  ["\uD630", "\uD630"], /* Hangul Syllable */
  ["\uD631", "\uD631"], /* Hangul Syllable */
  ["\uD632", "\uD632"], /* Hangul Syllable */
  ["\uD633", "\uD633"], /* Hangul Syllable */
  ["\uD634", "\uD634"], /* Hangul Syllable */
  ["\uD635", "\uD635"], /* Hangul Syllable */
  ["\uD636", "\uD636"], /* Hangul Syllable */
  ["\uD637", "\uD637"], /* Hangul Syllable */
  ["\uD638", "\uD638"], /* Hangul Syllable */
  ["\uD639", "\uD639"], /* Hangul Syllable */
  ["\uD63A", "\uD63A"], /* Hangul Syllable */
  ["\uD63B", "\uD63B"], /* Hangul Syllable */
  ["\uD63C", "\uD63C"], /* Hangul Syllable */
  ["\uD63D", "\uD63D"], /* Hangul Syllable */
  ["\uD63E", "\uD63E"], /* Hangul Syllable */
  ["\uD63F", "\uD63F"], /* Hangul Syllable */
  ["\uD640", "\uD640"], /* Hangul Syllable */
  ["\uD641", "\uD641"], /* Hangul Syllable */
  ["\uD642", "\uD642"], /* Hangul Syllable */
  ["\uD643", "\uD643"], /* Hangul Syllable */
  ["\uD644", "\uD644"], /* Hangul Syllable */
  ["\uD645", "\uD645"], /* Hangul Syllable */
  ["\uD646", "\uD646"], /* Hangul Syllable */
  ["\uD647", "\uD647"], /* Hangul Syllable */
  ["\uD648", "\uD648"], /* Hangul Syllable */
  ["\uD649", "\uD649"], /* Hangul Syllable */
  ["\uD64A", "\uD64A"], /* Hangul Syllable */
  ["\uD64B", "\uD64B"], /* Hangul Syllable */
  ["\uD64C", "\uD64C"], /* Hangul Syllable */
  ["\uD64D", "\uD64D"], /* Hangul Syllable */
  ["\uD64E", "\uD64E"], /* Hangul Syllable */
  ["\uD64F", "\uD64F"], /* Hangul Syllable */
  ["\uD650", "\uD650"], /* Hangul Syllable */
  ["\uD651", "\uD651"], /* Hangul Syllable */
  ["\uD652", "\uD652"], /* Hangul Syllable */
  ["\uD653", "\uD653"], /* Hangul Syllable */
  ["\uD654", "\uD654"], /* Hangul Syllable */
  ["\uD655", "\uD655"], /* Hangul Syllable */
  ["\uD656", "\uD656"], /* Hangul Syllable */
  ["\uD657", "\uD657"], /* Hangul Syllable */
  ["\uD658", "\uD658"], /* Hangul Syllable */
  ["\uD659", "\uD659"], /* Hangul Syllable */
  ["\uD65A", "\uD65A"], /* Hangul Syllable */
  ["\uD65B", "\uD65B"], /* Hangul Syllable */
  ["\uD65C", "\uD65C"], /* Hangul Syllable */
  ["\uD65D", "\uD65D"], /* Hangul Syllable */
  ["\uD65E", "\uD65E"], /* Hangul Syllable */
  ["\uD65F", "\uD65F"], /* Hangul Syllable */
  ["\uD660", "\uD660"], /* Hangul Syllable */
  ["\uD661", "\uD661"], /* Hangul Syllable */
  ["\uD662", "\uD662"], /* Hangul Syllable */
  ["\uD663", "\uD663"], /* Hangul Syllable */
  ["\uD664", "\uD664"], /* Hangul Syllable */
  ["\uD665", "\uD665"], /* Hangul Syllable */
  ["\uD666", "\uD666"], /* Hangul Syllable */
  ["\uD667", "\uD667"], /* Hangul Syllable */
  ["\uD668", "\uD668"], /* Hangul Syllable */
  ["\uD669", "\uD669"], /* Hangul Syllable */
  ["\uD66A", "\uD66A"], /* Hangul Syllable */
  ["\uD66B", "\uD66B"], /* Hangul Syllable */
  ["\uD66C", "\uD66C"], /* Hangul Syllable */
  ["\uD66D", "\uD66D"], /* Hangul Syllable */
  ["\uD66E", "\uD66E"], /* Hangul Syllable */
  ["\uD66F", "\uD66F"], /* Hangul Syllable */
  ["\uD670", "\uD670"], /* Hangul Syllable */
  ["\uD671", "\uD671"], /* Hangul Syllable */
  ["\uD672", "\uD672"], /* Hangul Syllable */
  ["\uD673", "\uD673"], /* Hangul Syllable */
  ["\uD674", "\uD674"], /* Hangul Syllable */
  ["\uD675", "\uD675"], /* Hangul Syllable */
  ["\uD676", "\uD676"], /* Hangul Syllable */
  ["\uD677", "\uD677"], /* Hangul Syllable */
  ["\uD678", "\uD678"], /* Hangul Syllable */
  ["\uD679", "\uD679"], /* Hangul Syllable */
  ["\uD67A", "\uD67A"], /* Hangul Syllable */
  ["\uD67B", "\uD67B"], /* Hangul Syllable */
  ["\uD67C", "\uD67C"], /* Hangul Syllable */
  ["\uD67D", "\uD67D"], /* Hangul Syllable */
  ["\uD67E", "\uD67E"], /* Hangul Syllable */
  ["\uD67F", "\uD67F"], /* Hangul Syllable */
  ["\uD680", "\uD680"], /* Hangul Syllable */
  ["\uD681", "\uD681"], /* Hangul Syllable */
  ["\uD682", "\uD682"], /* Hangul Syllable */
  ["\uD683", "\uD683"], /* Hangul Syllable */
  ["\uD684", "\uD684"], /* Hangul Syllable */
  ["\uD685", "\uD685"], /* Hangul Syllable */
  ["\uD686", "\uD686"], /* Hangul Syllable */
  ["\uD687", "\uD687"], /* Hangul Syllable */
  ["\uD688", "\uD688"], /* Hangul Syllable */
  ["\uD689", "\uD689"], /* Hangul Syllable */
  ["\uD68A", "\uD68A"], /* Hangul Syllable */
  ["\uD68B", "\uD68B"], /* Hangul Syllable */
  ["\uD68C", "\uD68C"], /* Hangul Syllable */
  ["\uD68D", "\uD68D"], /* Hangul Syllable */
  ["\uD68E", "\uD68E"], /* Hangul Syllable */
  ["\uD68F", "\uD68F"], /* Hangul Syllable */
  ["\uD690", "\uD690"], /* Hangul Syllable */
  ["\uD691", "\uD691"], /* Hangul Syllable */
  ["\uD692", "\uD692"], /* Hangul Syllable */
  ["\uD693", "\uD693"], /* Hangul Syllable */
  ["\uD694", "\uD694"], /* Hangul Syllable */
  ["\uD695", "\uD695"], /* Hangul Syllable */
  ["\uD696", "\uD696"], /* Hangul Syllable */
  ["\uD697", "\uD697"], /* Hangul Syllable */
  ["\uD698", "\uD698"], /* Hangul Syllable */
  ["\uD699", "\uD699"], /* Hangul Syllable */
  ["\uD69A", "\uD69A"], /* Hangul Syllable */
  ["\uD69B", "\uD69B"], /* Hangul Syllable */
  ["\uD69C", "\uD69C"], /* Hangul Syllable */
  ["\uD69D", "\uD69D"], /* Hangul Syllable */
  ["\uD69E", "\uD69E"], /* Hangul Syllable */
  ["\uD69F", "\uD69F"], /* Hangul Syllable */
  ["\uD6A0", "\uD6A0"], /* Hangul Syllable */
  ["\uD6A1", "\uD6A1"], /* Hangul Syllable */
  ["\uD6A2", "\uD6A2"], /* Hangul Syllable */
  ["\uD6A3", "\uD6A3"], /* Hangul Syllable */
  ["\uD6A4", "\uD6A4"], /* Hangul Syllable */
  ["\uD6A5", "\uD6A5"], /* Hangul Syllable */
  ["\uD6A6", "\uD6A6"], /* Hangul Syllable */
  ["\uD6A7", "\uD6A7"], /* Hangul Syllable */
  ["\uD6A8", "\uD6A8"], /* Hangul Syllable */
  ["\uD6A9", "\uD6A9"], /* Hangul Syllable */
  ["\uD6AA", "\uD6AA"], /* Hangul Syllable */
  ["\uD6AB", "\uD6AB"], /* Hangul Syllable */
  ["\uD6AC", "\uD6AC"], /* Hangul Syllable */
  ["\uD6AD", "\uD6AD"], /* Hangul Syllable */
  ["\uD6AE", "\uD6AE"], /* Hangul Syllable */
  ["\uD6AF", "\uD6AF"], /* Hangul Syllable */
  ["\uD6B0", "\uD6B0"], /* Hangul Syllable */
  ["\uD6B1", "\uD6B1"], /* Hangul Syllable */
  ["\uD6B2", "\uD6B2"], /* Hangul Syllable */
  ["\uD6B3", "\uD6B3"], /* Hangul Syllable */
  ["\uD6B4", "\uD6B4"], /* Hangul Syllable */
  ["\uD6B5", "\uD6B5"], /* Hangul Syllable */
  ["\uD6B6", "\uD6B6"], /* Hangul Syllable */
  ["\uD6B7", "\uD6B7"], /* Hangul Syllable */
  ["\uD6B8", "\uD6B8"], /* Hangul Syllable */
  ["\uD6B9", "\uD6B9"], /* Hangul Syllable */
  ["\uD6BA", "\uD6BA"], /* Hangul Syllable */
  ["\uD6BB", "\uD6BB"], /* Hangul Syllable */
  ["\uD6BC", "\uD6BC"], /* Hangul Syllable */
  ["\uD6BD", "\uD6BD"], /* Hangul Syllable */
  ["\uD6BE", "\uD6BE"], /* Hangul Syllable */
  ["\uD6BF", "\uD6BF"], /* Hangul Syllable */
  ["\uD6C0", "\uD6C0"], /* Hangul Syllable */
  ["\uD6C1", "\uD6C1"], /* Hangul Syllable */
  ["\uD6C2", "\uD6C2"], /* Hangul Syllable */
  ["\uD6C3", "\uD6C3"], /* Hangul Syllable */
  ["\uD6C4", "\uD6C4"], /* Hangul Syllable */
  ["\uD6C5", "\uD6C5"], /* Hangul Syllable */
  ["\uD6C6", "\uD6C6"], /* Hangul Syllable */
  ["\uD6C7", "\uD6C7"], /* Hangul Syllable */
  ["\uD6C8", "\uD6C8"], /* Hangul Syllable */
  ["\uD6C9", "\uD6C9"], /* Hangul Syllable */
  ["\uD6CA", "\uD6CA"], /* Hangul Syllable */
  ["\uD6CB", "\uD6CB"], /* Hangul Syllable */
  ["\uD6CC", "\uD6CC"], /* Hangul Syllable */
  ["\uD6CD", "\uD6CD"], /* Hangul Syllable */
  ["\uD6CE", "\uD6CE"], /* Hangul Syllable */
  ["\uD6CF", "\uD6CF"], /* Hangul Syllable */
  ["\uD6D0", "\uD6D0"], /* Hangul Syllable */
  ["\uD6D1", "\uD6D1"], /* Hangul Syllable */
  ["\uD6D2", "\uD6D2"], /* Hangul Syllable */
  ["\uD6D3", "\uD6D3"], /* Hangul Syllable */
  ["\uD6D4", "\uD6D4"], /* Hangul Syllable */
  ["\uD6D5", "\uD6D5"], /* Hangul Syllable */
  ["\uD6D6", "\uD6D6"], /* Hangul Syllable */
  ["\uD6D7", "\uD6D7"], /* Hangul Syllable */
  ["\uD6D8", "\uD6D8"], /* Hangul Syllable */
  ["\uD6D9", "\uD6D9"], /* Hangul Syllable */
  ["\uD6DA", "\uD6DA"], /* Hangul Syllable */
  ["\uD6DB", "\uD6DB"], /* Hangul Syllable */
  ["\uD6DC", "\uD6DC"], /* Hangul Syllable */
  ["\uD6DD", "\uD6DD"], /* Hangul Syllable */
  ["\uD6DE", "\uD6DE"], /* Hangul Syllable */
  ["\uD6DF", "\uD6DF"], /* Hangul Syllable */
  ["\uD6E0", "\uD6E0"], /* Hangul Syllable */
  ["\uD6E1", "\uD6E1"], /* Hangul Syllable */
  ["\uD6E2", "\uD6E2"], /* Hangul Syllable */
  ["\uD6E3", "\uD6E3"], /* Hangul Syllable */
  ["\uD6E4", "\uD6E4"], /* Hangul Syllable */
  ["\uD6E5", "\uD6E5"], /* Hangul Syllable */
  ["\uD6E6", "\uD6E6"], /* Hangul Syllable */
  ["\uD6E7", "\uD6E7"], /* Hangul Syllable */
  ["\uD6E8", "\uD6E8"], /* Hangul Syllable */
  ["\uD6E9", "\uD6E9"], /* Hangul Syllable */
  ["\uD6EA", "\uD6EA"], /* Hangul Syllable */
  ["\uD6EB", "\uD6EB"], /* Hangul Syllable */
  ["\uD6EC", "\uD6EC"], /* Hangul Syllable */
  ["\uD6ED", "\uD6ED"], /* Hangul Syllable */
  ["\uD6EE", "\uD6EE"], /* Hangul Syllable */
  ["\uD6EF", "\uD6EF"], /* Hangul Syllable */
  ["\uD6F0", "\uD6F0"], /* Hangul Syllable */
  ["\uD6F1", "\uD6F1"], /* Hangul Syllable */
  ["\uD6F2", "\uD6F2"], /* Hangul Syllable */
  ["\uD6F3", "\uD6F3"], /* Hangul Syllable */
  ["\uD6F4", "\uD6F4"], /* Hangul Syllable */
  ["\uD6F5", "\uD6F5"], /* Hangul Syllable */
  ["\uD6F6", "\uD6F6"], /* Hangul Syllable */
  ["\uD6F7", "\uD6F7"], /* Hangul Syllable */
  ["\uD6F8", "\uD6F8"], /* Hangul Syllable */
  ["\uD6F9", "\uD6F9"], /* Hangul Syllable */
  ["\uD6FA", "\uD6FA"], /* Hangul Syllable */
  ["\uD6FB", "\uD6FB"], /* Hangul Syllable */
  ["\uD6FC", "\uD6FC"], /* Hangul Syllable */
  ["\uD6FD", "\uD6FD"], /* Hangul Syllable */
  ["\uD6FE", "\uD6FE"], /* Hangul Syllable */
  ["\uD6FF", "\uD6FF"], /* Hangul Syllable */
  ["\uD700", "\uD700"], /* Hangul Syllable */
  ["\uD701", "\uD701"], /* Hangul Syllable */
  ["\uD702", "\uD702"], /* Hangul Syllable */
  ["\uD703", "\uD703"], /* Hangul Syllable */
  ["\uD704", "\uD704"], /* Hangul Syllable */
  ["\uD705", "\uD705"], /* Hangul Syllable */
  ["\uD706", "\uD706"], /* Hangul Syllable */
  ["\uD707", "\uD707"], /* Hangul Syllable */
  ["\uD708", "\uD708"], /* Hangul Syllable */
  ["\uD709", "\uD709"], /* Hangul Syllable */
  ["\uD70A", "\uD70A"], /* Hangul Syllable */
  ["\uD70B", "\uD70B"], /* Hangul Syllable */
  ["\uD70C", "\uD70C"], /* Hangul Syllable */
  ["\uD70D", "\uD70D"], /* Hangul Syllable */
  ["\uD70E", "\uD70E"], /* Hangul Syllable */
  ["\uD70F", "\uD70F"], /* Hangul Syllable */
  ["\uD710", "\uD710"], /* Hangul Syllable */
  ["\uD711", "\uD711"], /* Hangul Syllable */
  ["\uD712", "\uD712"], /* Hangul Syllable */
  ["\uD713", "\uD713"], /* Hangul Syllable */
  ["\uD714", "\uD714"], /* Hangul Syllable */
  ["\uD715", "\uD715"], /* Hangul Syllable */
  ["\uD716", "\uD716"], /* Hangul Syllable */
  ["\uD717", "\uD717"], /* Hangul Syllable */
  ["\uD718", "\uD718"], /* Hangul Syllable */
  ["\uD719", "\uD719"], /* Hangul Syllable */
  ["\uD71A", "\uD71A"], /* Hangul Syllable */
  ["\uD71B", "\uD71B"], /* Hangul Syllable */
  ["\uD71C", "\uD71C"], /* Hangul Syllable */
  ["\uD71D", "\uD71D"], /* Hangul Syllable */
  ["\uD71E", "\uD71E"], /* Hangul Syllable */
  ["\uD71F", "\uD71F"], /* Hangul Syllable */
  ["\uD720", "\uD720"], /* Hangul Syllable */
  ["\uD721", "\uD721"], /* Hangul Syllable */
  ["\uD722", "\uD722"], /* Hangul Syllable */
  ["\uD723", "\uD723"], /* Hangul Syllable */
  ["\uD724", "\uD724"], /* Hangul Syllable */
  ["\uD725", "\uD725"], /* Hangul Syllable */
  ["\uD726", "\uD726"], /* Hangul Syllable */
  ["\uD727", "\uD727"], /* Hangul Syllable */
  ["\uD728", "\uD728"], /* Hangul Syllable */
  ["\uD729", "\uD729"], /* Hangul Syllable */
  ["\uD72A", "\uD72A"], /* Hangul Syllable */
  ["\uD72B", "\uD72B"], /* Hangul Syllable */
  ["\uD72C", "\uD72C"], /* Hangul Syllable */
  ["\uD72D", "\uD72D"], /* Hangul Syllable */
  ["\uD72E", "\uD72E"], /* Hangul Syllable */
  ["\uD72F", "\uD72F"], /* Hangul Syllable */
  ["\uD730", "\uD730"], /* Hangul Syllable */
  ["\uD731", "\uD731"], /* Hangul Syllable */
  ["\uD732", "\uD732"], /* Hangul Syllable */
  ["\uD733", "\uD733"], /* Hangul Syllable */
  ["\uD734", "\uD734"], /* Hangul Syllable */
  ["\uD735", "\uD735"], /* Hangul Syllable */
  ["\uD736", "\uD736"], /* Hangul Syllable */
  ["\uD737", "\uD737"], /* Hangul Syllable */
  ["\uD738", "\uD738"], /* Hangul Syllable */
  ["\uD739", "\uD739"], /* Hangul Syllable */
  ["\uD73A", "\uD73A"], /* Hangul Syllable */
  ["\uD73B", "\uD73B"], /* Hangul Syllable */
  ["\uD73C", "\uD73C"], /* Hangul Syllable */
  ["\uD73D", "\uD73D"], /* Hangul Syllable */
  ["\uD73E", "\uD73E"], /* Hangul Syllable */
  ["\uD73F", "\uD73F"], /* Hangul Syllable */
  ["\uD740", "\uD740"], /* Hangul Syllable */
  ["\uD741", "\uD741"], /* Hangul Syllable */
  ["\uD742", "\uD742"], /* Hangul Syllable */
  ["\uD743", "\uD743"], /* Hangul Syllable */
  ["\uD744", "\uD744"], /* Hangul Syllable */
  ["\uD745", "\uD745"], /* Hangul Syllable */
  ["\uD746", "\uD746"], /* Hangul Syllable */
  ["\uD747", "\uD747"], /* Hangul Syllable */
  ["\uD748", "\uD748"], /* Hangul Syllable */
  ["\uD749", "\uD749"], /* Hangul Syllable */
  ["\uD74A", "\uD74A"], /* Hangul Syllable */
  ["\uD74B", "\uD74B"], /* Hangul Syllable */
  ["\uD74C", "\uD74C"], /* Hangul Syllable */
  ["\uD74D", "\uD74D"], /* Hangul Syllable */
  ["\uD74E", "\uD74E"], /* Hangul Syllable */
  ["\uD74F", "\uD74F"], /* Hangul Syllable */
  ["\uD750", "\uD750"], /* Hangul Syllable */
  ["\uD751", "\uD751"], /* Hangul Syllable */
  ["\uD752", "\uD752"], /* Hangul Syllable */
  ["\uD753", "\uD753"], /* Hangul Syllable */
  ["\uD754", "\uD754"], /* Hangul Syllable */
  ["\uD755", "\uD755"], /* Hangul Syllable */
  ["\uD756", "\uD756"], /* Hangul Syllable */
  ["\uD757", "\uD757"], /* Hangul Syllable */
  ["\uD758", "\uD758"], /* Hangul Syllable */
  ["\uD759", "\uD759"], /* Hangul Syllable */
  ["\uD75A", "\uD75A"], /* Hangul Syllable */
  ["\uD75B", "\uD75B"], /* Hangul Syllable */
  ["\uD75C", "\uD75C"], /* Hangul Syllable */
  ["\uD75D", "\uD75D"], /* Hangul Syllable */
  ["\uD75E", "\uD75E"], /* Hangul Syllable */
  ["\uD75F", "\uD75F"], /* Hangul Syllable */
  ["\uD760", "\uD760"], /* Hangul Syllable */
  ["\uD761", "\uD761"], /* Hangul Syllable */
  ["\uD762", "\uD762"], /* Hangul Syllable */
  ["\uD763", "\uD763"], /* Hangul Syllable */
  ["\uD764", "\uD764"], /* Hangul Syllable */
  ["\uD765", "\uD765"], /* Hangul Syllable */
  ["\uD766", "\uD766"], /* Hangul Syllable */
  ["\uD767", "\uD767"], /* Hangul Syllable */
  ["\uD768", "\uD768"], /* Hangul Syllable */
  ["\uD769", "\uD769"], /* Hangul Syllable */
  ["\uD76A", "\uD76A"], /* Hangul Syllable */
  ["\uD76B", "\uD76B"], /* Hangul Syllable */
  ["\uD76C", "\uD76C"], /* Hangul Syllable */
  ["\uD76D", "\uD76D"], /* Hangul Syllable */
  ["\uD76E", "\uD76E"], /* Hangul Syllable */
  ["\uD76F", "\uD76F"], /* Hangul Syllable */
  ["\uD770", "\uD770"], /* Hangul Syllable */
  ["\uD771", "\uD771"], /* Hangul Syllable */
  ["\uD772", "\uD772"], /* Hangul Syllable */
  ["\uD773", "\uD773"], /* Hangul Syllable */
  ["\uD774", "\uD774"], /* Hangul Syllable */
  ["\uD775", "\uD775"], /* Hangul Syllable */
  ["\uD776", "\uD776"], /* Hangul Syllable */
  ["\uD777", "\uD777"], /* Hangul Syllable */
  ["\uD778", "\uD778"], /* Hangul Syllable */
  ["\uD779", "\uD779"], /* Hangul Syllable */
  ["\uD77A", "\uD77A"], /* Hangul Syllable */
  ["\uD77B", "\uD77B"], /* Hangul Syllable */
  ["\uD77C", "\uD77C"], /* Hangul Syllable */
  ["\uD77D", "\uD77D"], /* Hangul Syllable */
  ["\uD77E", "\uD77E"], /* Hangul Syllable */
  ["\uD77F", "\uD77F"], /* Hangul Syllable */
  ["\uD780", "\uD780"], /* Hangul Syllable */
  ["\uD781", "\uD781"], /* Hangul Syllable */
  ["\uD782", "\uD782"], /* Hangul Syllable */
  ["\uD783", "\uD783"], /* Hangul Syllable */
  ["\uD784", "\uD784"], /* Hangul Syllable */
  ["\uD785", "\uD785"], /* Hangul Syllable */
  ["\uD786", "\uD786"], /* Hangul Syllable */
  ["\uD787", "\uD787"], /* Hangul Syllable */
  ["\uD788", "\uD788"], /* Hangul Syllable */
  ["\uD789", "\uD789"], /* Hangul Syllable */
  ["\uD78A", "\uD78A"], /* Hangul Syllable */
  ["\uD78B", "\uD78B"], /* Hangul Syllable */
  ["\uD78C", "\uD78C"], /* Hangul Syllable */
  ["\uD78D", "\uD78D"], /* Hangul Syllable */
  ["\uD78E", "\uD78E"], /* Hangul Syllable */
  ["\uD78F", "\uD78F"], /* Hangul Syllable */
  ["\uD790", "\uD790"], /* Hangul Syllable */
  ["\uD791", "\uD791"], /* Hangul Syllable */
  ["\uD792", "\uD792"], /* Hangul Syllable */
  ["\uD793", "\uD793"], /* Hangul Syllable */
  ["\uD794", "\uD794"], /* Hangul Syllable */
  ["\uD795", "\uD795"], /* Hangul Syllable */
  ["\uD796", "\uD796"], /* Hangul Syllable */
  ["\uD797", "\uD797"], /* Hangul Syllable */
  ["\uD798", "\uD798"], /* Hangul Syllable */
  ["\uD799", "\uD799"], /* Hangul Syllable */
  ["\uD79A", "\uD79A"], /* Hangul Syllable */
  ["\uD79B", "\uD79B"], /* Hangul Syllable */
  ["\uD79C", "\uD79C"], /* Hangul Syllable */
  ["\uD79D", "\uD79D"], /* Hangul Syllable */
  ["\uD79E", "\uD79E"], /* Hangul Syllable */
  ["\uD79F", "\uD79F"], /* Hangul Syllable */
  ["\uD7A0", "\uD7A0"], /* Hangul Syllable */
  ["\uD7A1", "\uD7A1"], /* Hangul Syllable */
  ["\uD7A2", "\uD7A2"], /* Hangul Syllable */
  ["\uD7A3", "\uD7A3"], /* Hangul Syllable */
  ["\uD7A4", "\uD7A4"],
  ["\uD7A5", "\uD7A5"],
  ["\uD7A6", "\uD7A6"],
  ["\uD7A7", "\uD7A7"],
  ["\uD7A8", "\uD7A8"],
  ["\uD7A9", "\uD7A9"],
  ["\uD7AA", "\uD7AA"],
  ["\uD7AB", "\uD7AB"],
  ["\uD7AC", "\uD7AC"],
  ["\uD7AD", "\uD7AD"],
  ["\uD7AE", "\uD7AE"],
  ["\uD7AF", "\uD7AF"],
  ["\uD7B0", "\uD7B0"], /* HANGUL JUNGSEONG O-YEO */
  ["\uD7B1", "\uD7B1"], /* HANGUL JUNGSEONG O-O-I */
  ["\uD7B2", "\uD7B2"], /* HANGUL JUNGSEONG YO-A */
  ["\uD7B3", "\uD7B3"], /* HANGUL JUNGSEONG YO-AE */
  ["\uD7B4", "\uD7B4"], /* HANGUL JUNGSEONG YO-EO */
  ["\uD7B5", "\uD7B5"], /* HANGUL JUNGSEONG U-YEO */
  ["\uD7B6", "\uD7B6"], /* HANGUL JUNGSEONG U-I-I */
  ["\uD7B7", "\uD7B7"], /* HANGUL JUNGSEONG YU-AE */
  ["\uD7B8", "\uD7B8"], /* HANGUL JUNGSEONG YU-O */
  ["\uD7B9", "\uD7B9"], /* HANGUL JUNGSEONG EU-A */
  ["\uD7BA", "\uD7BA"], /* HANGUL JUNGSEONG EU-EO */
  ["\uD7BB", "\uD7BB"], /* HANGUL JUNGSEONG EU-E */
  ["\uD7BC", "\uD7BC"], /* HANGUL JUNGSEONG EU-O */
  ["\uD7BD", "\uD7BD"], /* HANGUL JUNGSEONG I-YA-O */
  ["\uD7BE", "\uD7BE"], /* HANGUL JUNGSEONG I-YAE */
  ["\uD7BF", "\uD7BF"], /* HANGUL JUNGSEONG I-YEO */
  ["\uD7C0", "\uD7C0"], /* HANGUL JUNGSEONG I-YE */
  ["\uD7C1", "\uD7C1"], /* HANGUL JUNGSEONG I-O-I */
  ["\uD7C2", "\uD7C2"], /* HANGUL JUNGSEONG I-YO */
  ["\uD7C3", "\uD7C3"], /* HANGUL JUNGSEONG I-YU */
  ["\uD7C4", "\uD7C4"], /* HANGUL JUNGSEONG I-I */
  ["\uD7C5", "\uD7C5"], /* HANGUL JUNGSEONG ARAEA-A */
  ["\uD7C6", "\uD7C6"], /* HANGUL JUNGSEONG ARAEA-E */
  ["\uD7C7", "\uD7C7"],
  ["\uD7C8", "\uD7C8"],
  ["\uD7C9", "\uD7C9"],
  ["\uD7CA", "\uD7CA"],
  ["\uD7CB", "\uD7CB"], /* HANGUL JONGSEONG NIEUN-RIEUL */
  ["\uD7CC", "\uD7CC"], /* HANGUL JONGSEONG NIEUN-CHIEUCH */
  ["\uD7CD", "\uD7CD"], /* HANGUL JONGSEONG SSANGTIKEUT */
  ["\uD7CE", "\uD7CE"], /* HANGUL JONGSEONG SSANGTIKEUT-PIEUP */
  ["\uD7CF", "\uD7CF"], /* HANGUL JONGSEONG TIKEUT-PIEUP */
  ["\uD7D0", "\uD7D0"], /* HANGUL JONGSEONG TIKEUT-SIOS */
  ["\uD7D1", "\uD7D1"], /* HANGUL JONGSEONG TIKEUT-SIOS-KIYEOK */
  ["\uD7D2", "\uD7D2"], /* HANGUL JONGSEONG TIKEUT-CIEUC */
  ["\uD7D3", "\uD7D3"], /* HANGUL JONGSEONG TIKEUT-CHIEUCH */
  ["\uD7D4", "\uD7D4"], /* HANGUL JONGSEONG TIKEUT-THIEUTH */
  ["\uD7D5", "\uD7D5"], /* HANGUL JONGSEONG RIEUL-SSANGKIYEOK */
  ["\uD7D6", "\uD7D6"], /* HANGUL JONGSEONG RIEUL-KIYEOK-HIEUH */
  ["\uD7D7", "\uD7D7"], /* HANGUL JONGSEONG SSANGRIEUL-KHIEUKH */
  ["\uD7D8", "\uD7D8"], /* HANGUL JONGSEONG RIEUL-MIEUM-HIEUH */
  ["\uD7D9", "\uD7D9"], /* HANGUL JONGSEONG RIEUL-PIEUP-TIKEUT */
  ["\uD7DA", "\uD7DA"], /* HANGUL JONGSEONG RIEUL-PIEUP-PHIEUPH */
  ["\uD7DB", "\uD7DB"], /* HANGUL JONGSEONG RIEUL-YESIEUNG */
  ["\uD7DC", "\uD7DC"], /* HANGUL JONGSEONG RIEUL-YEORINHIEUH-HIEUH */
  ["\uD7DD", "\uD7DD"], /* HANGUL JONGSEONG KAPYEOUNRIEUL */
  ["\uD7DE", "\uD7DE"], /* HANGUL JONGSEONG MIEUM-NIEUN */
  ["\uD7DF", "\uD7DF"], /* HANGUL JONGSEONG MIEUM-SSANGNIEUN */
  ["\uD7E0", "\uD7E0"], /* HANGUL JONGSEONG SSANGMIEUM */
  ["\uD7E1", "\uD7E1"], /* HANGUL JONGSEONG MIEUM-PIEUP-SIOS */
  ["\uD7E2", "\uD7E2"], /* HANGUL JONGSEONG MIEUM-CIEUC */
  ["\uD7E3", "\uD7E3"], /* HANGUL JONGSEONG PIEUP-TIKEUT */
  ["\uD7E4", "\uD7E4"], /* HANGUL JONGSEONG PIEUP-RIEUL-PHIEUPH */
  ["\uD7E5", "\uD7E5"], /* HANGUL JONGSEONG PIEUP-MIEUM */
  ["\uD7E6", "\uD7E6"], /* HANGUL JONGSEONG SSANGPIEUP */
  ["\uD7E7", "\uD7E7"], /* HANGUL JONGSEONG PIEUP-SIOS-TIKEUT */
  ["\uD7E8", "\uD7E8"], /* HANGUL JONGSEONG PIEUP-CIEUC */
  ["\uD7E9", "\uD7E9"], /* HANGUL JONGSEONG PIEUP-CHIEUCH */
  ["\uD7EA", "\uD7EA"], /* HANGUL JONGSEONG SIOS-MIEUM */
  ["\uD7EB", "\uD7EB"], /* HANGUL JONGSEONG SIOS-KAPYEOUNPIEUP */
  ["\uD7EC", "\uD7EC"], /* HANGUL JONGSEONG SSANGSIOS-KIYEOK */
  ["\uD7ED", "\uD7ED"], /* HANGUL JONGSEONG SSANGSIOS-TIKEUT */
  ["\uD7EE", "\uD7EE"], /* HANGUL JONGSEONG SIOS-PANSIOS */
  ["\uD7EF", "\uD7EF"], /* HANGUL JONGSEONG SIOS-CIEUC */
  ["\uD7F0", "\uD7F0"], /* HANGUL JONGSEONG SIOS-CHIEUCH */
  ["\uD7F1", "\uD7F1"], /* HANGUL JONGSEONG SIOS-THIEUTH */
  ["\uD7F2", "\uD7F2"], /* HANGUL JONGSEONG SIOS-HIEUH */
  ["\uD7F3", "\uD7F3"], /* HANGUL JONGSEONG PANSIOS-PIEUP */
  ["\uD7F4", "\uD7F4"], /* HANGUL JONGSEONG PANSIOS-KAPYEOUNPIEUP */
  ["\uD7F5", "\uD7F5"], /* HANGUL JONGSEONG YESIEUNG-MIEUM */
  ["\uD7F6", "\uD7F6"], /* HANGUL JONGSEONG YESIEUNG-HIEUH */
  ["\uD7F7", "\uD7F7"], /* HANGUL JONGSEONG CIEUC-PIEUP */
  ["\uD7F8", "\uD7F8"], /* HANGUL JONGSEONG CIEUC-SSANGPIEUP */
  ["\uD7F9", "\uD7F9"], /* HANGUL JONGSEONG SSANGCIEUC */
  ["\uD7FA", "\uD7FA"], /* HANGUL JONGSEONG PHIEUPH-SIOS */
  ["\uD7FB", "\uD7FB"], /* HANGUL JONGSEONG PHIEUPH-THIEUTH */
  ["\uD7FC", "\uD7FC"],
  ["\uD7FD", "\uD7FD"],
  ["\uD7FE", "\uD7FE"],
  ["\uD7FF", "\uD7FF"],
  ["\uD800", "\uD800"], /* Non Private Use High Surrogate */
  ["\uD801", "\uD801"], /* Non Private Use High Surrogate */
  ["\uD802", "\uD802"], /* Non Private Use High Surrogate */
  ["\uD803", "\uD803"], /* Non Private Use High Surrogate */
  ["\uD804", "\uD804"], /* Non Private Use High Surrogate */
  ["\uD805", "\uD805"], /* Non Private Use High Surrogate */
  ["\uD806", "\uD806"], /* Non Private Use High Surrogate */
  ["\uD807", "\uD807"], /* Non Private Use High Surrogate */
  ["\uD808", "\uD808"], /* Non Private Use High Surrogate */
  ["\uD809", "\uD809"], /* Non Private Use High Surrogate */
  ["\uD80A", "\uD80A"], /* Non Private Use High Surrogate */
  ["\uD80B", "\uD80B"], /* Non Private Use High Surrogate */
  ["\uD80C", "\uD80C"], /* Non Private Use High Surrogate */
  ["\uD80D", "\uD80D"], /* Non Private Use High Surrogate */
  ["\uD80E", "\uD80E"], /* Non Private Use High Surrogate */
  ["\uD80F", "\uD80F"], /* Non Private Use High Surrogate */
  ["\uD810", "\uD810"], /* Non Private Use High Surrogate */
  ["\uD811", "\uD811"], /* Non Private Use High Surrogate */
  ["\uD812", "\uD812"], /* Non Private Use High Surrogate */
  ["\uD813", "\uD813"], /* Non Private Use High Surrogate */
  ["\uD814", "\uD814"], /* Non Private Use High Surrogate */
  ["\uD815", "\uD815"], /* Non Private Use High Surrogate */
  ["\uD816", "\uD816"], /* Non Private Use High Surrogate */
  ["\uD817", "\uD817"], /* Non Private Use High Surrogate */
  ["\uD818", "\uD818"], /* Non Private Use High Surrogate */
  ["\uD819", "\uD819"], /* Non Private Use High Surrogate */
  ["\uD81A", "\uD81A"], /* Non Private Use High Surrogate */
  ["\uD81B", "\uD81B"], /* Non Private Use High Surrogate */
  ["\uD81C", "\uD81C"], /* Non Private Use High Surrogate */
  ["\uD81D", "\uD81D"], /* Non Private Use High Surrogate */
  ["\uD81E", "\uD81E"], /* Non Private Use High Surrogate */
  ["\uD81F", "\uD81F"], /* Non Private Use High Surrogate */
  ["\uD820", "\uD820"], /* Non Private Use High Surrogate */
  ["\uD821", "\uD821"], /* Non Private Use High Surrogate */
  ["\uD822", "\uD822"], /* Non Private Use High Surrogate */
  ["\uD823", "\uD823"], /* Non Private Use High Surrogate */
  ["\uD824", "\uD824"], /* Non Private Use High Surrogate */
  ["\uD825", "\uD825"], /* Non Private Use High Surrogate */
  ["\uD826", "\uD826"], /* Non Private Use High Surrogate */
  ["\uD827", "\uD827"], /* Non Private Use High Surrogate */
  ["\uD828", "\uD828"], /* Non Private Use High Surrogate */
  ["\uD829", "\uD829"], /* Non Private Use High Surrogate */
  ["\uD82A", "\uD82A"], /* Non Private Use High Surrogate */
  ["\uD82B", "\uD82B"], /* Non Private Use High Surrogate */
  ["\uD82C", "\uD82C"], /* Non Private Use High Surrogate */
  ["\uD82D", "\uD82D"], /* Non Private Use High Surrogate */
  ["\uD82E", "\uD82E"], /* Non Private Use High Surrogate */
  ["\uD82F", "\uD82F"], /* Non Private Use High Surrogate */
  ["\uD830", "\uD830"], /* Non Private Use High Surrogate */
  ["\uD831", "\uD831"], /* Non Private Use High Surrogate */
  ["\uD832", "\uD832"], /* Non Private Use High Surrogate */
  ["\uD833", "\uD833"], /* Non Private Use High Surrogate */
  ["\uD834", "\uD834"], /* Non Private Use High Surrogate */
  ["\uD835", "\uD835"], /* Non Private Use High Surrogate */
  ["\uD836", "\uD836"], /* Non Private Use High Surrogate */
  ["\uD837", "\uD837"], /* Non Private Use High Surrogate */
  ["\uD838", "\uD838"], /* Non Private Use High Surrogate */
  ["\uD839", "\uD839"], /* Non Private Use High Surrogate */
  ["\uD83A", "\uD83A"], /* Non Private Use High Surrogate */
  ["\uD83B", "\uD83B"], /* Non Private Use High Surrogate */
  ["\uD83C", "\uD83C"], /* Non Private Use High Surrogate */
  ["\uD83D", "\uD83D"], /* Non Private Use High Surrogate */
  ["\uD83E", "\uD83E"], /* Non Private Use High Surrogate */
  ["\uD83F", "\uD83F"], /* Non Private Use High Surrogate */
  ["\uD840", "\uD840"], /* Non Private Use High Surrogate */
  ["\uD841", "\uD841"], /* Non Private Use High Surrogate */
  ["\uD842", "\uD842"], /* Non Private Use High Surrogate */
  ["\uD843", "\uD843"], /* Non Private Use High Surrogate */
  ["\uD844", "\uD844"], /* Non Private Use High Surrogate */
  ["\uD845", "\uD845"], /* Non Private Use High Surrogate */
  ["\uD846", "\uD846"], /* Non Private Use High Surrogate */
  ["\uD847", "\uD847"], /* Non Private Use High Surrogate */
  ["\uD848", "\uD848"], /* Non Private Use High Surrogate */
  ["\uD849", "\uD849"], /* Non Private Use High Surrogate */
  ["\uD84A", "\uD84A"], /* Non Private Use High Surrogate */
  ["\uD84B", "\uD84B"], /* Non Private Use High Surrogate */
  ["\uD84C", "\uD84C"], /* Non Private Use High Surrogate */
  ["\uD84D", "\uD84D"], /* Non Private Use High Surrogate */
  ["\uD84E", "\uD84E"], /* Non Private Use High Surrogate */
  ["\uD84F", "\uD84F"], /* Non Private Use High Surrogate */
  ["\uD850", "\uD850"], /* Non Private Use High Surrogate */
  ["\uD851", "\uD851"], /* Non Private Use High Surrogate */
  ["\uD852", "\uD852"], /* Non Private Use High Surrogate */
  ["\uD853", "\uD853"], /* Non Private Use High Surrogate */
  ["\uD854", "\uD854"], /* Non Private Use High Surrogate */
  ["\uD855", "\uD855"], /* Non Private Use High Surrogate */
  ["\uD856", "\uD856"], /* Non Private Use High Surrogate */
  ["\uD857", "\uD857"], /* Non Private Use High Surrogate */
  ["\uD858", "\uD858"], /* Non Private Use High Surrogate */
  ["\uD859", "\uD859"], /* Non Private Use High Surrogate */
  ["\uD85A", "\uD85A"], /* Non Private Use High Surrogate */
  ["\uD85B", "\uD85B"], /* Non Private Use High Surrogate */
  ["\uD85C", "\uD85C"], /* Non Private Use High Surrogate */
  ["\uD85D", "\uD85D"], /* Non Private Use High Surrogate */
  ["\uD85E", "\uD85E"], /* Non Private Use High Surrogate */
  ["\uD85F", "\uD85F"], /* Non Private Use High Surrogate */
  ["\uD860", "\uD860"], /* Non Private Use High Surrogate */
  ["\uD861", "\uD861"], /* Non Private Use High Surrogate */
  ["\uD862", "\uD862"], /* Non Private Use High Surrogate */
  ["\uD863", "\uD863"], /* Non Private Use High Surrogate */
  ["\uD864", "\uD864"], /* Non Private Use High Surrogate */
  ["\uD865", "\uD865"], /* Non Private Use High Surrogate */
  ["\uD866", "\uD866"], /* Non Private Use High Surrogate */
  ["\uD867", "\uD867"], /* Non Private Use High Surrogate */
  ["\uD868", "\uD868"], /* Non Private Use High Surrogate */
  ["\uD869", "\uD869"], /* Non Private Use High Surrogate */
  ["\uD86A", "\uD86A"], /* Non Private Use High Surrogate */
  ["\uD86B", "\uD86B"], /* Non Private Use High Surrogate */
  ["\uD86C", "\uD86C"], /* Non Private Use High Surrogate */
  ["\uD86D", "\uD86D"], /* Non Private Use High Surrogate */
  ["\uD86E", "\uD86E"], /* Non Private Use High Surrogate */
  ["\uD86F", "\uD86F"], /* Non Private Use High Surrogate */
  ["\uD870", "\uD870"], /* Non Private Use High Surrogate */
  ["\uD871", "\uD871"], /* Non Private Use High Surrogate */
  ["\uD872", "\uD872"], /* Non Private Use High Surrogate */
  ["\uD873", "\uD873"], /* Non Private Use High Surrogate */
  ["\uD874", "\uD874"], /* Non Private Use High Surrogate */
  ["\uD875", "\uD875"], /* Non Private Use High Surrogate */
  ["\uD876", "\uD876"], /* Non Private Use High Surrogate */
  ["\uD877", "\uD877"], /* Non Private Use High Surrogate */
  ["\uD878", "\uD878"], /* Non Private Use High Surrogate */
  ["\uD879", "\uD879"], /* Non Private Use High Surrogate */
  ["\uD87A", "\uD87A"], /* Non Private Use High Surrogate */
  ["\uD87B", "\uD87B"], /* Non Private Use High Surrogate */
  ["\uD87C", "\uD87C"], /* Non Private Use High Surrogate */
  ["\uD87D", "\uD87D"], /* Non Private Use High Surrogate */
  ["\uD87E", "\uD87E"], /* Non Private Use High Surrogate */
  ["\uD87F", "\uD87F"], /* Non Private Use High Surrogate */
  ["\uD880", "\uD880"], /* Non Private Use High Surrogate */
  ["\uD881", "\uD881"], /* Non Private Use High Surrogate */
  ["\uD882", "\uD882"], /* Non Private Use High Surrogate */
  ["\uD883", "\uD883"], /* Non Private Use High Surrogate */
  ["\uD884", "\uD884"], /* Non Private Use High Surrogate */
  ["\uD885", "\uD885"], /* Non Private Use High Surrogate */
  ["\uD886", "\uD886"], /* Non Private Use High Surrogate */
  ["\uD887", "\uD887"], /* Non Private Use High Surrogate */
  ["\uD888", "\uD888"], /* Non Private Use High Surrogate */
  ["\uD889", "\uD889"], /* Non Private Use High Surrogate */
  ["\uD88A", "\uD88A"], /* Non Private Use High Surrogate */
  ["\uD88B", "\uD88B"], /* Non Private Use High Surrogate */
  ["\uD88C", "\uD88C"], /* Non Private Use High Surrogate */
  ["\uD88D", "\uD88D"], /* Non Private Use High Surrogate */
  ["\uD88E", "\uD88E"], /* Non Private Use High Surrogate */
  ["\uD88F", "\uD88F"], /* Non Private Use High Surrogate */
  ["\uD890", "\uD890"], /* Non Private Use High Surrogate */
  ["\uD891", "\uD891"], /* Non Private Use High Surrogate */
  ["\uD892", "\uD892"], /* Non Private Use High Surrogate */
  ["\uD893", "\uD893"], /* Non Private Use High Surrogate */
  ["\uD894", "\uD894"], /* Non Private Use High Surrogate */
  ["\uD895", "\uD895"], /* Non Private Use High Surrogate */
  ["\uD896", "\uD896"], /* Non Private Use High Surrogate */
  ["\uD897", "\uD897"], /* Non Private Use High Surrogate */
  ["\uD898", "\uD898"], /* Non Private Use High Surrogate */
  ["\uD899", "\uD899"], /* Non Private Use High Surrogate */
  ["\uD89A", "\uD89A"], /* Non Private Use High Surrogate */
  ["\uD89B", "\uD89B"], /* Non Private Use High Surrogate */
  ["\uD89C", "\uD89C"], /* Non Private Use High Surrogate */
  ["\uD89D", "\uD89D"], /* Non Private Use High Surrogate */
  ["\uD89E", "\uD89E"], /* Non Private Use High Surrogate */
  ["\uD89F", "\uD89F"], /* Non Private Use High Surrogate */
  ["\uD8A0", "\uD8A0"], /* Non Private Use High Surrogate */
  ["\uD8A1", "\uD8A1"], /* Non Private Use High Surrogate */
  ["\uD8A2", "\uD8A2"], /* Non Private Use High Surrogate */
  ["\uD8A3", "\uD8A3"], /* Non Private Use High Surrogate */
  ["\uD8A4", "\uD8A4"], /* Non Private Use High Surrogate */
  ["\uD8A5", "\uD8A5"], /* Non Private Use High Surrogate */
  ["\uD8A6", "\uD8A6"], /* Non Private Use High Surrogate */
  ["\uD8A7", "\uD8A7"], /* Non Private Use High Surrogate */
  ["\uD8A8", "\uD8A8"], /* Non Private Use High Surrogate */
  ["\uD8A9", "\uD8A9"], /* Non Private Use High Surrogate */
  ["\uD8AA", "\uD8AA"], /* Non Private Use High Surrogate */
  ["\uD8AB", "\uD8AB"], /* Non Private Use High Surrogate */
  ["\uD8AC", "\uD8AC"], /* Non Private Use High Surrogate */
  ["\uD8AD", "\uD8AD"], /* Non Private Use High Surrogate */
  ["\uD8AE", "\uD8AE"], /* Non Private Use High Surrogate */
  ["\uD8AF", "\uD8AF"], /* Non Private Use High Surrogate */
  ["\uD8B0", "\uD8B0"], /* Non Private Use High Surrogate */
  ["\uD8B1", "\uD8B1"], /* Non Private Use High Surrogate */
  ["\uD8B2", "\uD8B2"], /* Non Private Use High Surrogate */
  ["\uD8B3", "\uD8B3"], /* Non Private Use High Surrogate */
  ["\uD8B4", "\uD8B4"], /* Non Private Use High Surrogate */
  ["\uD8B5", "\uD8B5"], /* Non Private Use High Surrogate */
  ["\uD8B6", "\uD8B6"], /* Non Private Use High Surrogate */
  ["\uD8B7", "\uD8B7"], /* Non Private Use High Surrogate */
  ["\uD8B8", "\uD8B8"], /* Non Private Use High Surrogate */
  ["\uD8B9", "\uD8B9"], /* Non Private Use High Surrogate */
  ["\uD8BA", "\uD8BA"], /* Non Private Use High Surrogate */
  ["\uD8BB", "\uD8BB"], /* Non Private Use High Surrogate */
  ["\uD8BC", "\uD8BC"], /* Non Private Use High Surrogate */
  ["\uD8BD", "\uD8BD"], /* Non Private Use High Surrogate */
  ["\uD8BE", "\uD8BE"], /* Non Private Use High Surrogate */
  ["\uD8BF", "\uD8BF"], /* Non Private Use High Surrogate */
  ["\uD8C0", "\uD8C0"], /* Non Private Use High Surrogate */
  ["\uD8C1", "\uD8C1"], /* Non Private Use High Surrogate */
  ["\uD8C2", "\uD8C2"], /* Non Private Use High Surrogate */
  ["\uD8C3", "\uD8C3"], /* Non Private Use High Surrogate */
  ["\uD8C4", "\uD8C4"], /* Non Private Use High Surrogate */
  ["\uD8C5", "\uD8C5"], /* Non Private Use High Surrogate */
  ["\uD8C6", "\uD8C6"], /* Non Private Use High Surrogate */
  ["\uD8C7", "\uD8C7"], /* Non Private Use High Surrogate */
  ["\uD8C8", "\uD8C8"], /* Non Private Use High Surrogate */
  ["\uD8C9", "\uD8C9"], /* Non Private Use High Surrogate */
  ["\uD8CA", "\uD8CA"], /* Non Private Use High Surrogate */
  ["\uD8CB", "\uD8CB"], /* Non Private Use High Surrogate */
  ["\uD8CC", "\uD8CC"], /* Non Private Use High Surrogate */
  ["\uD8CD", "\uD8CD"], /* Non Private Use High Surrogate */
  ["\uD8CE", "\uD8CE"], /* Non Private Use High Surrogate */
  ["\uD8CF", "\uD8CF"], /* Non Private Use High Surrogate */
  ["\uD8D0", "\uD8D0"], /* Non Private Use High Surrogate */
  ["\uD8D1", "\uD8D1"], /* Non Private Use High Surrogate */
  ["\uD8D2", "\uD8D2"], /* Non Private Use High Surrogate */
  ["\uD8D3", "\uD8D3"], /* Non Private Use High Surrogate */
  ["\uD8D4", "\uD8D4"], /* Non Private Use High Surrogate */
  ["\uD8D5", "\uD8D5"], /* Non Private Use High Surrogate */
  ["\uD8D6", "\uD8D6"], /* Non Private Use High Surrogate */
  ["\uD8D7", "\uD8D7"], /* Non Private Use High Surrogate */
  ["\uD8D8", "\uD8D8"], /* Non Private Use High Surrogate */
  ["\uD8D9", "\uD8D9"], /* Non Private Use High Surrogate */
  ["\uD8DA", "\uD8DA"], /* Non Private Use High Surrogate */
  ["\uD8DB", "\uD8DB"], /* Non Private Use High Surrogate */
  ["\uD8DC", "\uD8DC"], /* Non Private Use High Surrogate */
  ["\uD8DD", "\uD8DD"], /* Non Private Use High Surrogate */
  ["\uD8DE", "\uD8DE"], /* Non Private Use High Surrogate */
  ["\uD8DF", "\uD8DF"], /* Non Private Use High Surrogate */
  ["\uD8E0", "\uD8E0"], /* Non Private Use High Surrogate */
  ["\uD8E1", "\uD8E1"], /* Non Private Use High Surrogate */
  ["\uD8E2", "\uD8E2"], /* Non Private Use High Surrogate */
  ["\uD8E3", "\uD8E3"], /* Non Private Use High Surrogate */
  ["\uD8E4", "\uD8E4"], /* Non Private Use High Surrogate */
  ["\uD8E5", "\uD8E5"], /* Non Private Use High Surrogate */
  ["\uD8E6", "\uD8E6"], /* Non Private Use High Surrogate */
  ["\uD8E7", "\uD8E7"], /* Non Private Use High Surrogate */
  ["\uD8E8", "\uD8E8"], /* Non Private Use High Surrogate */
  ["\uD8E9", "\uD8E9"], /* Non Private Use High Surrogate */
  ["\uD8EA", "\uD8EA"], /* Non Private Use High Surrogate */
  ["\uD8EB", "\uD8EB"], /* Non Private Use High Surrogate */
  ["\uD8EC", "\uD8EC"], /* Non Private Use High Surrogate */
  ["\uD8ED", "\uD8ED"], /* Non Private Use High Surrogate */
  ["\uD8EE", "\uD8EE"], /* Non Private Use High Surrogate */
  ["\uD8EF", "\uD8EF"], /* Non Private Use High Surrogate */
  ["\uD8F0", "\uD8F0"], /* Non Private Use High Surrogate */
  ["\uD8F1", "\uD8F1"], /* Non Private Use High Surrogate */
  ["\uD8F2", "\uD8F2"], /* Non Private Use High Surrogate */
  ["\uD8F3", "\uD8F3"], /* Non Private Use High Surrogate */
  ["\uD8F4", "\uD8F4"], /* Non Private Use High Surrogate */
  ["\uD8F5", "\uD8F5"], /* Non Private Use High Surrogate */
  ["\uD8F6", "\uD8F6"], /* Non Private Use High Surrogate */
  ["\uD8F7", "\uD8F7"], /* Non Private Use High Surrogate */
  ["\uD8F8", "\uD8F8"], /* Non Private Use High Surrogate */
  ["\uD8F9", "\uD8F9"], /* Non Private Use High Surrogate */
  ["\uD8FA", "\uD8FA"], /* Non Private Use High Surrogate */
  ["\uD8FB", "\uD8FB"], /* Non Private Use High Surrogate */
  ["\uD8FC", "\uD8FC"], /* Non Private Use High Surrogate */
  ["\uD8FD", "\uD8FD"], /* Non Private Use High Surrogate */
  ["\uD8FE", "\uD8FE"], /* Non Private Use High Surrogate */
  ["\uD8FF", "\uD8FF"], /* Non Private Use High Surrogate */
  ["\uD900", "\uD900"], /* Non Private Use High Surrogate */
  ["\uD901", "\uD901"], /* Non Private Use High Surrogate */
  ["\uD902", "\uD902"], /* Non Private Use High Surrogate */
  ["\uD903", "\uD903"], /* Non Private Use High Surrogate */
  ["\uD904", "\uD904"], /* Non Private Use High Surrogate */
  ["\uD905", "\uD905"], /* Non Private Use High Surrogate */
  ["\uD906", "\uD906"], /* Non Private Use High Surrogate */
  ["\uD907", "\uD907"], /* Non Private Use High Surrogate */
  ["\uD908", "\uD908"], /* Non Private Use High Surrogate */
  ["\uD909", "\uD909"], /* Non Private Use High Surrogate */
  ["\uD90A", "\uD90A"], /* Non Private Use High Surrogate */
  ["\uD90B", "\uD90B"], /* Non Private Use High Surrogate */
  ["\uD90C", "\uD90C"], /* Non Private Use High Surrogate */
  ["\uD90D", "\uD90D"], /* Non Private Use High Surrogate */
  ["\uD90E", "\uD90E"], /* Non Private Use High Surrogate */
  ["\uD90F", "\uD90F"], /* Non Private Use High Surrogate */
  ["\uD910", "\uD910"], /* Non Private Use High Surrogate */
  ["\uD911", "\uD911"], /* Non Private Use High Surrogate */
  ["\uD912", "\uD912"], /* Non Private Use High Surrogate */
  ["\uD913", "\uD913"], /* Non Private Use High Surrogate */
  ["\uD914", "\uD914"], /* Non Private Use High Surrogate */
  ["\uD915", "\uD915"], /* Non Private Use High Surrogate */
  ["\uD916", "\uD916"], /* Non Private Use High Surrogate */
  ["\uD917", "\uD917"], /* Non Private Use High Surrogate */
  ["\uD918", "\uD918"], /* Non Private Use High Surrogate */
  ["\uD919", "\uD919"], /* Non Private Use High Surrogate */
  ["\uD91A", "\uD91A"], /* Non Private Use High Surrogate */
  ["\uD91B", "\uD91B"], /* Non Private Use High Surrogate */
  ["\uD91C", "\uD91C"], /* Non Private Use High Surrogate */
  ["\uD91D", "\uD91D"], /* Non Private Use High Surrogate */
  ["\uD91E", "\uD91E"], /* Non Private Use High Surrogate */
  ["\uD91F", "\uD91F"], /* Non Private Use High Surrogate */
  ["\uD920", "\uD920"], /* Non Private Use High Surrogate */
  ["\uD921", "\uD921"], /* Non Private Use High Surrogate */
  ["\uD922", "\uD922"], /* Non Private Use High Surrogate */
  ["\uD923", "\uD923"], /* Non Private Use High Surrogate */
  ["\uD924", "\uD924"], /* Non Private Use High Surrogate */
  ["\uD925", "\uD925"], /* Non Private Use High Surrogate */
  ["\uD926", "\uD926"], /* Non Private Use High Surrogate */
  ["\uD927", "\uD927"], /* Non Private Use High Surrogate */
  ["\uD928", "\uD928"], /* Non Private Use High Surrogate */
  ["\uD929", "\uD929"], /* Non Private Use High Surrogate */
  ["\uD92A", "\uD92A"], /* Non Private Use High Surrogate */
  ["\uD92B", "\uD92B"], /* Non Private Use High Surrogate */
  ["\uD92C", "\uD92C"], /* Non Private Use High Surrogate */
  ["\uD92D", "\uD92D"], /* Non Private Use High Surrogate */
  ["\uD92E", "\uD92E"], /* Non Private Use High Surrogate */
  ["\uD92F", "\uD92F"], /* Non Private Use High Surrogate */
  ["\uD930", "\uD930"], /* Non Private Use High Surrogate */
  ["\uD931", "\uD931"], /* Non Private Use High Surrogate */
  ["\uD932", "\uD932"], /* Non Private Use High Surrogate */
  ["\uD933", "\uD933"], /* Non Private Use High Surrogate */
  ["\uD934", "\uD934"], /* Non Private Use High Surrogate */
  ["\uD935", "\uD935"], /* Non Private Use High Surrogate */
  ["\uD936", "\uD936"], /* Non Private Use High Surrogate */
  ["\uD937", "\uD937"], /* Non Private Use High Surrogate */
  ["\uD938", "\uD938"], /* Non Private Use High Surrogate */
  ["\uD939", "\uD939"], /* Non Private Use High Surrogate */
  ["\uD93A", "\uD93A"], /* Non Private Use High Surrogate */
  ["\uD93B", "\uD93B"], /* Non Private Use High Surrogate */
  ["\uD93C", "\uD93C"], /* Non Private Use High Surrogate */
  ["\uD93D", "\uD93D"], /* Non Private Use High Surrogate */
  ["\uD93E", "\uD93E"], /* Non Private Use High Surrogate */
  ["\uD93F", "\uD93F"], /* Non Private Use High Surrogate */
  ["\uD940", "\uD940"], /* Non Private Use High Surrogate */
  ["\uD941", "\uD941"], /* Non Private Use High Surrogate */
  ["\uD942", "\uD942"], /* Non Private Use High Surrogate */
  ["\uD943", "\uD943"], /* Non Private Use High Surrogate */
  ["\uD944", "\uD944"], /* Non Private Use High Surrogate */
  ["\uD945", "\uD945"], /* Non Private Use High Surrogate */
  ["\uD946", "\uD946"], /* Non Private Use High Surrogate */
  ["\uD947", "\uD947"], /* Non Private Use High Surrogate */
  ["\uD948", "\uD948"], /* Non Private Use High Surrogate */
  ["\uD949", "\uD949"], /* Non Private Use High Surrogate */
  ["\uD94A", "\uD94A"], /* Non Private Use High Surrogate */
  ["\uD94B", "\uD94B"], /* Non Private Use High Surrogate */
  ["\uD94C", "\uD94C"], /* Non Private Use High Surrogate */
  ["\uD94D", "\uD94D"], /* Non Private Use High Surrogate */
  ["\uD94E", "\uD94E"], /* Non Private Use High Surrogate */
  ["\uD94F", "\uD94F"], /* Non Private Use High Surrogate */
  ["\uD950", "\uD950"], /* Non Private Use High Surrogate */
  ["\uD951", "\uD951"], /* Non Private Use High Surrogate */
  ["\uD952", "\uD952"], /* Non Private Use High Surrogate */
  ["\uD953", "\uD953"], /* Non Private Use High Surrogate */
  ["\uD954", "\uD954"], /* Non Private Use High Surrogate */
  ["\uD955", "\uD955"], /* Non Private Use High Surrogate */
  ["\uD956", "\uD956"], /* Non Private Use High Surrogate */
  ["\uD957", "\uD957"], /* Non Private Use High Surrogate */
  ["\uD958", "\uD958"], /* Non Private Use High Surrogate */
  ["\uD959", "\uD959"], /* Non Private Use High Surrogate */
  ["\uD95A", "\uD95A"], /* Non Private Use High Surrogate */
  ["\uD95B", "\uD95B"], /* Non Private Use High Surrogate */
  ["\uD95C", "\uD95C"], /* Non Private Use High Surrogate */
  ["\uD95D", "\uD95D"], /* Non Private Use High Surrogate */
  ["\uD95E", "\uD95E"], /* Non Private Use High Surrogate */
  ["\uD95F", "\uD95F"], /* Non Private Use High Surrogate */
  ["\uD960", "\uD960"], /* Non Private Use High Surrogate */
  ["\uD961", "\uD961"], /* Non Private Use High Surrogate */
  ["\uD962", "\uD962"], /* Non Private Use High Surrogate */
  ["\uD963", "\uD963"], /* Non Private Use High Surrogate */
  ["\uD964", "\uD964"], /* Non Private Use High Surrogate */
  ["\uD965", "\uD965"], /* Non Private Use High Surrogate */
  ["\uD966", "\uD966"], /* Non Private Use High Surrogate */
  ["\uD967", "\uD967"], /* Non Private Use High Surrogate */
  ["\uD968", "\uD968"], /* Non Private Use High Surrogate */
  ["\uD969", "\uD969"], /* Non Private Use High Surrogate */
  ["\uD96A", "\uD96A"], /* Non Private Use High Surrogate */
  ["\uD96B", "\uD96B"], /* Non Private Use High Surrogate */
  ["\uD96C", "\uD96C"], /* Non Private Use High Surrogate */
  ["\uD96D", "\uD96D"], /* Non Private Use High Surrogate */
  ["\uD96E", "\uD96E"], /* Non Private Use High Surrogate */
  ["\uD96F", "\uD96F"], /* Non Private Use High Surrogate */
  ["\uD970", "\uD970"], /* Non Private Use High Surrogate */
  ["\uD971", "\uD971"], /* Non Private Use High Surrogate */
  ["\uD972", "\uD972"], /* Non Private Use High Surrogate */
  ["\uD973", "\uD973"], /* Non Private Use High Surrogate */
  ["\uD974", "\uD974"], /* Non Private Use High Surrogate */
  ["\uD975", "\uD975"], /* Non Private Use High Surrogate */
  ["\uD976", "\uD976"], /* Non Private Use High Surrogate */
  ["\uD977", "\uD977"], /* Non Private Use High Surrogate */
  ["\uD978", "\uD978"], /* Non Private Use High Surrogate */
  ["\uD979", "\uD979"], /* Non Private Use High Surrogate */
  ["\uD97A", "\uD97A"], /* Non Private Use High Surrogate */
  ["\uD97B", "\uD97B"], /* Non Private Use High Surrogate */
  ["\uD97C", "\uD97C"], /* Non Private Use High Surrogate */
  ["\uD97D", "\uD97D"], /* Non Private Use High Surrogate */
  ["\uD97E", "\uD97E"], /* Non Private Use High Surrogate */
  ["\uD97F", "\uD97F"], /* Non Private Use High Surrogate */
  ["\uD980", "\uD980"], /* Non Private Use High Surrogate */
  ["\uD981", "\uD981"], /* Non Private Use High Surrogate */
  ["\uD982", "\uD982"], /* Non Private Use High Surrogate */
  ["\uD983", "\uD983"], /* Non Private Use High Surrogate */
  ["\uD984", "\uD984"], /* Non Private Use High Surrogate */
  ["\uD985", "\uD985"], /* Non Private Use High Surrogate */
  ["\uD986", "\uD986"], /* Non Private Use High Surrogate */
  ["\uD987", "\uD987"], /* Non Private Use High Surrogate */
  ["\uD988", "\uD988"], /* Non Private Use High Surrogate */
  ["\uD989", "\uD989"], /* Non Private Use High Surrogate */
  ["\uD98A", "\uD98A"], /* Non Private Use High Surrogate */
  ["\uD98B", "\uD98B"], /* Non Private Use High Surrogate */
  ["\uD98C", "\uD98C"], /* Non Private Use High Surrogate */
  ["\uD98D", "\uD98D"], /* Non Private Use High Surrogate */
  ["\uD98E", "\uD98E"], /* Non Private Use High Surrogate */
  ["\uD98F", "\uD98F"], /* Non Private Use High Surrogate */
  ["\uD990", "\uD990"], /* Non Private Use High Surrogate */
  ["\uD991", "\uD991"], /* Non Private Use High Surrogate */
  ["\uD992", "\uD992"], /* Non Private Use High Surrogate */
  ["\uD993", "\uD993"], /* Non Private Use High Surrogate */
  ["\uD994", "\uD994"], /* Non Private Use High Surrogate */
  ["\uD995", "\uD995"], /* Non Private Use High Surrogate */
  ["\uD996", "\uD996"], /* Non Private Use High Surrogate */
  ["\uD997", "\uD997"], /* Non Private Use High Surrogate */
  ["\uD998", "\uD998"], /* Non Private Use High Surrogate */
  ["\uD999", "\uD999"], /* Non Private Use High Surrogate */
  ["\uD99A", "\uD99A"], /* Non Private Use High Surrogate */
  ["\uD99B", "\uD99B"], /* Non Private Use High Surrogate */
  ["\uD99C", "\uD99C"], /* Non Private Use High Surrogate */
  ["\uD99D", "\uD99D"], /* Non Private Use High Surrogate */
  ["\uD99E", "\uD99E"], /* Non Private Use High Surrogate */
  ["\uD99F", "\uD99F"], /* Non Private Use High Surrogate */
  ["\uD9A0", "\uD9A0"], /* Non Private Use High Surrogate */
  ["\uD9A1", "\uD9A1"], /* Non Private Use High Surrogate */
  ["\uD9A2", "\uD9A2"], /* Non Private Use High Surrogate */
  ["\uD9A3", "\uD9A3"], /* Non Private Use High Surrogate */
  ["\uD9A4", "\uD9A4"], /* Non Private Use High Surrogate */
  ["\uD9A5", "\uD9A5"], /* Non Private Use High Surrogate */
  ["\uD9A6", "\uD9A6"], /* Non Private Use High Surrogate */
  ["\uD9A7", "\uD9A7"], /* Non Private Use High Surrogate */
  ["\uD9A8", "\uD9A8"], /* Non Private Use High Surrogate */
  ["\uD9A9", "\uD9A9"], /* Non Private Use High Surrogate */
  ["\uD9AA", "\uD9AA"], /* Non Private Use High Surrogate */
  ["\uD9AB", "\uD9AB"], /* Non Private Use High Surrogate */
  ["\uD9AC", "\uD9AC"], /* Non Private Use High Surrogate */
  ["\uD9AD", "\uD9AD"], /* Non Private Use High Surrogate */
  ["\uD9AE", "\uD9AE"], /* Non Private Use High Surrogate */
  ["\uD9AF", "\uD9AF"], /* Non Private Use High Surrogate */
  ["\uD9B0", "\uD9B0"], /* Non Private Use High Surrogate */
  ["\uD9B1", "\uD9B1"], /* Non Private Use High Surrogate */
  ["\uD9B2", "\uD9B2"], /* Non Private Use High Surrogate */
  ["\uD9B3", "\uD9B3"], /* Non Private Use High Surrogate */
  ["\uD9B4", "\uD9B4"], /* Non Private Use High Surrogate */
  ["\uD9B5", "\uD9B5"], /* Non Private Use High Surrogate */
  ["\uD9B6", "\uD9B6"], /* Non Private Use High Surrogate */
  ["\uD9B7", "\uD9B7"], /* Non Private Use High Surrogate */
  ["\uD9B8", "\uD9B8"], /* Non Private Use High Surrogate */
  ["\uD9B9", "\uD9B9"], /* Non Private Use High Surrogate */
  ["\uD9BA", "\uD9BA"], /* Non Private Use High Surrogate */
  ["\uD9BB", "\uD9BB"], /* Non Private Use High Surrogate */
  ["\uD9BC", "\uD9BC"], /* Non Private Use High Surrogate */
  ["\uD9BD", "\uD9BD"], /* Non Private Use High Surrogate */
  ["\uD9BE", "\uD9BE"], /* Non Private Use High Surrogate */
  ["\uD9BF", "\uD9BF"], /* Non Private Use High Surrogate */
  ["\uD9C0", "\uD9C0"], /* Non Private Use High Surrogate */
  ["\uD9C1", "\uD9C1"], /* Non Private Use High Surrogate */
  ["\uD9C2", "\uD9C2"], /* Non Private Use High Surrogate */
  ["\uD9C3", "\uD9C3"], /* Non Private Use High Surrogate */
  ["\uD9C4", "\uD9C4"], /* Non Private Use High Surrogate */
  ["\uD9C5", "\uD9C5"], /* Non Private Use High Surrogate */
  ["\uD9C6", "\uD9C6"], /* Non Private Use High Surrogate */
  ["\uD9C7", "\uD9C7"], /* Non Private Use High Surrogate */
  ["\uD9C8", "\uD9C8"], /* Non Private Use High Surrogate */
  ["\uD9C9", "\uD9C9"], /* Non Private Use High Surrogate */
  ["\uD9CA", "\uD9CA"], /* Non Private Use High Surrogate */
  ["\uD9CB", "\uD9CB"], /* Non Private Use High Surrogate */
  ["\uD9CC", "\uD9CC"], /* Non Private Use High Surrogate */
  ["\uD9CD", "\uD9CD"], /* Non Private Use High Surrogate */
  ["\uD9CE", "\uD9CE"], /* Non Private Use High Surrogate */
  ["\uD9CF", "\uD9CF"], /* Non Private Use High Surrogate */
  ["\uD9D0", "\uD9D0"], /* Non Private Use High Surrogate */
  ["\uD9D1", "\uD9D1"], /* Non Private Use High Surrogate */
  ["\uD9D2", "\uD9D2"], /* Non Private Use High Surrogate */
  ["\uD9D3", "\uD9D3"], /* Non Private Use High Surrogate */
  ["\uD9D4", "\uD9D4"], /* Non Private Use High Surrogate */
  ["\uD9D5", "\uD9D5"], /* Non Private Use High Surrogate */
  ["\uD9D6", "\uD9D6"], /* Non Private Use High Surrogate */
  ["\uD9D7", "\uD9D7"], /* Non Private Use High Surrogate */
  ["\uD9D8", "\uD9D8"], /* Non Private Use High Surrogate */
  ["\uD9D9", "\uD9D9"], /* Non Private Use High Surrogate */
  ["\uD9DA", "\uD9DA"], /* Non Private Use High Surrogate */
  ["\uD9DB", "\uD9DB"], /* Non Private Use High Surrogate */
  ["\uD9DC", "\uD9DC"], /* Non Private Use High Surrogate */
  ["\uD9DD", "\uD9DD"], /* Non Private Use High Surrogate */
  ["\uD9DE", "\uD9DE"], /* Non Private Use High Surrogate */
  ["\uD9DF", "\uD9DF"], /* Non Private Use High Surrogate */
  ["\uD9E0", "\uD9E0"], /* Non Private Use High Surrogate */
  ["\uD9E1", "\uD9E1"], /* Non Private Use High Surrogate */
  ["\uD9E2", "\uD9E2"], /* Non Private Use High Surrogate */
  ["\uD9E3", "\uD9E3"], /* Non Private Use High Surrogate */
  ["\uD9E4", "\uD9E4"], /* Non Private Use High Surrogate */
  ["\uD9E5", "\uD9E5"], /* Non Private Use High Surrogate */
  ["\uD9E6", "\uD9E6"], /* Non Private Use High Surrogate */
  ["\uD9E7", "\uD9E7"], /* Non Private Use High Surrogate */
  ["\uD9E8", "\uD9E8"], /* Non Private Use High Surrogate */
  ["\uD9E9", "\uD9E9"], /* Non Private Use High Surrogate */
  ["\uD9EA", "\uD9EA"], /* Non Private Use High Surrogate */
  ["\uD9EB", "\uD9EB"], /* Non Private Use High Surrogate */
  ["\uD9EC", "\uD9EC"], /* Non Private Use High Surrogate */
  ["\uD9ED", "\uD9ED"], /* Non Private Use High Surrogate */
  ["\uD9EE", "\uD9EE"], /* Non Private Use High Surrogate */
  ["\uD9EF", "\uD9EF"], /* Non Private Use High Surrogate */
  ["\uD9F0", "\uD9F0"], /* Non Private Use High Surrogate */
  ["\uD9F1", "\uD9F1"], /* Non Private Use High Surrogate */
  ["\uD9F2", "\uD9F2"], /* Non Private Use High Surrogate */
  ["\uD9F3", "\uD9F3"], /* Non Private Use High Surrogate */
  ["\uD9F4", "\uD9F4"], /* Non Private Use High Surrogate */
  ["\uD9F5", "\uD9F5"], /* Non Private Use High Surrogate */
  ["\uD9F6", "\uD9F6"], /* Non Private Use High Surrogate */
  ["\uD9F7", "\uD9F7"], /* Non Private Use High Surrogate */
  ["\uD9F8", "\uD9F8"], /* Non Private Use High Surrogate */
  ["\uD9F9", "\uD9F9"], /* Non Private Use High Surrogate */
  ["\uD9FA", "\uD9FA"], /* Non Private Use High Surrogate */
  ["\uD9FB", "\uD9FB"], /* Non Private Use High Surrogate */
  ["\uD9FC", "\uD9FC"], /* Non Private Use High Surrogate */
  ["\uD9FD", "\uD9FD"], /* Non Private Use High Surrogate */
  ["\uD9FE", "\uD9FE"], /* Non Private Use High Surrogate */
  ["\uD9FF", "\uD9FF"], /* Non Private Use High Surrogate */
  ["\uDA00", "\uDA00"], /* Non Private Use High Surrogate */
  ["\uDA01", "\uDA01"], /* Non Private Use High Surrogate */
  ["\uDA02", "\uDA02"], /* Non Private Use High Surrogate */
  ["\uDA03", "\uDA03"], /* Non Private Use High Surrogate */
  ["\uDA04", "\uDA04"], /* Non Private Use High Surrogate */
  ["\uDA05", "\uDA05"], /* Non Private Use High Surrogate */
  ["\uDA06", "\uDA06"], /* Non Private Use High Surrogate */
  ["\uDA07", "\uDA07"], /* Non Private Use High Surrogate */
  ["\uDA08", "\uDA08"], /* Non Private Use High Surrogate */
  ["\uDA09", "\uDA09"], /* Non Private Use High Surrogate */
  ["\uDA0A", "\uDA0A"], /* Non Private Use High Surrogate */
  ["\uDA0B", "\uDA0B"], /* Non Private Use High Surrogate */
  ["\uDA0C", "\uDA0C"], /* Non Private Use High Surrogate */
  ["\uDA0D", "\uDA0D"], /* Non Private Use High Surrogate */
  ["\uDA0E", "\uDA0E"], /* Non Private Use High Surrogate */
  ["\uDA0F", "\uDA0F"], /* Non Private Use High Surrogate */
  ["\uDA10", "\uDA10"], /* Non Private Use High Surrogate */
  ["\uDA11", "\uDA11"], /* Non Private Use High Surrogate */
  ["\uDA12", "\uDA12"], /* Non Private Use High Surrogate */
  ["\uDA13", "\uDA13"], /* Non Private Use High Surrogate */
  ["\uDA14", "\uDA14"], /* Non Private Use High Surrogate */
  ["\uDA15", "\uDA15"], /* Non Private Use High Surrogate */
  ["\uDA16", "\uDA16"], /* Non Private Use High Surrogate */
  ["\uDA17", "\uDA17"], /* Non Private Use High Surrogate */
  ["\uDA18", "\uDA18"], /* Non Private Use High Surrogate */
  ["\uDA19", "\uDA19"], /* Non Private Use High Surrogate */
  ["\uDA1A", "\uDA1A"], /* Non Private Use High Surrogate */
  ["\uDA1B", "\uDA1B"], /* Non Private Use High Surrogate */
  ["\uDA1C", "\uDA1C"], /* Non Private Use High Surrogate */
  ["\uDA1D", "\uDA1D"], /* Non Private Use High Surrogate */
  ["\uDA1E", "\uDA1E"], /* Non Private Use High Surrogate */
  ["\uDA1F", "\uDA1F"], /* Non Private Use High Surrogate */
  ["\uDA20", "\uDA20"], /* Non Private Use High Surrogate */
  ["\uDA21", "\uDA21"], /* Non Private Use High Surrogate */
  ["\uDA22", "\uDA22"], /* Non Private Use High Surrogate */
  ["\uDA23", "\uDA23"], /* Non Private Use High Surrogate */
  ["\uDA24", "\uDA24"], /* Non Private Use High Surrogate */
  ["\uDA25", "\uDA25"], /* Non Private Use High Surrogate */
  ["\uDA26", "\uDA26"], /* Non Private Use High Surrogate */
  ["\uDA27", "\uDA27"], /* Non Private Use High Surrogate */
  ["\uDA28", "\uDA28"], /* Non Private Use High Surrogate */
  ["\uDA29", "\uDA29"], /* Non Private Use High Surrogate */
  ["\uDA2A", "\uDA2A"], /* Non Private Use High Surrogate */
  ["\uDA2B", "\uDA2B"], /* Non Private Use High Surrogate */
  ["\uDA2C", "\uDA2C"], /* Non Private Use High Surrogate */
  ["\uDA2D", "\uDA2D"], /* Non Private Use High Surrogate */
  ["\uDA2E", "\uDA2E"], /* Non Private Use High Surrogate */
  ["\uDA2F", "\uDA2F"], /* Non Private Use High Surrogate */
  ["\uDA30", "\uDA30"], /* Non Private Use High Surrogate */
  ["\uDA31", "\uDA31"], /* Non Private Use High Surrogate */
  ["\uDA32", "\uDA32"], /* Non Private Use High Surrogate */
  ["\uDA33", "\uDA33"], /* Non Private Use High Surrogate */
  ["\uDA34", "\uDA34"], /* Non Private Use High Surrogate */
  ["\uDA35", "\uDA35"], /* Non Private Use High Surrogate */
  ["\uDA36", "\uDA36"], /* Non Private Use High Surrogate */
  ["\uDA37", "\uDA37"], /* Non Private Use High Surrogate */
  ["\uDA38", "\uDA38"], /* Non Private Use High Surrogate */
  ["\uDA39", "\uDA39"], /* Non Private Use High Surrogate */
  ["\uDA3A", "\uDA3A"], /* Non Private Use High Surrogate */
  ["\uDA3B", "\uDA3B"], /* Non Private Use High Surrogate */
  ["\uDA3C", "\uDA3C"], /* Non Private Use High Surrogate */
  ["\uDA3D", "\uDA3D"], /* Non Private Use High Surrogate */
  ["\uDA3E", "\uDA3E"], /* Non Private Use High Surrogate */
  ["\uDA3F", "\uDA3F"], /* Non Private Use High Surrogate */
  ["\uDA40", "\uDA40"], /* Non Private Use High Surrogate */
  ["\uDA41", "\uDA41"], /* Non Private Use High Surrogate */
  ["\uDA42", "\uDA42"], /* Non Private Use High Surrogate */
  ["\uDA43", "\uDA43"], /* Non Private Use High Surrogate */
  ["\uDA44", "\uDA44"], /* Non Private Use High Surrogate */
  ["\uDA45", "\uDA45"], /* Non Private Use High Surrogate */
  ["\uDA46", "\uDA46"], /* Non Private Use High Surrogate */
  ["\uDA47", "\uDA47"], /* Non Private Use High Surrogate */
  ["\uDA48", "\uDA48"], /* Non Private Use High Surrogate */
  ["\uDA49", "\uDA49"], /* Non Private Use High Surrogate */
  ["\uDA4A", "\uDA4A"], /* Non Private Use High Surrogate */
  ["\uDA4B", "\uDA4B"], /* Non Private Use High Surrogate */
  ["\uDA4C", "\uDA4C"], /* Non Private Use High Surrogate */
  ["\uDA4D", "\uDA4D"], /* Non Private Use High Surrogate */
  ["\uDA4E", "\uDA4E"], /* Non Private Use High Surrogate */
  ["\uDA4F", "\uDA4F"], /* Non Private Use High Surrogate */
  ["\uDA50", "\uDA50"], /* Non Private Use High Surrogate */
  ["\uDA51", "\uDA51"], /* Non Private Use High Surrogate */
  ["\uDA52", "\uDA52"], /* Non Private Use High Surrogate */
  ["\uDA53", "\uDA53"], /* Non Private Use High Surrogate */
  ["\uDA54", "\uDA54"], /* Non Private Use High Surrogate */
  ["\uDA55", "\uDA55"], /* Non Private Use High Surrogate */
  ["\uDA56", "\uDA56"], /* Non Private Use High Surrogate */
  ["\uDA57", "\uDA57"], /* Non Private Use High Surrogate */
  ["\uDA58", "\uDA58"], /* Non Private Use High Surrogate */
  ["\uDA59", "\uDA59"], /* Non Private Use High Surrogate */
  ["\uDA5A", "\uDA5A"], /* Non Private Use High Surrogate */
  ["\uDA5B", "\uDA5B"], /* Non Private Use High Surrogate */
  ["\uDA5C", "\uDA5C"], /* Non Private Use High Surrogate */
  ["\uDA5D", "\uDA5D"], /* Non Private Use High Surrogate */
  ["\uDA5E", "\uDA5E"], /* Non Private Use High Surrogate */
  ["\uDA5F", "\uDA5F"], /* Non Private Use High Surrogate */
  ["\uDA60", "\uDA60"], /* Non Private Use High Surrogate */
  ["\uDA61", "\uDA61"], /* Non Private Use High Surrogate */
  ["\uDA62", "\uDA62"], /* Non Private Use High Surrogate */
  ["\uDA63", "\uDA63"], /* Non Private Use High Surrogate */
  ["\uDA64", "\uDA64"], /* Non Private Use High Surrogate */
  ["\uDA65", "\uDA65"], /* Non Private Use High Surrogate */
  ["\uDA66", "\uDA66"], /* Non Private Use High Surrogate */
  ["\uDA67", "\uDA67"], /* Non Private Use High Surrogate */
  ["\uDA68", "\uDA68"], /* Non Private Use High Surrogate */
  ["\uDA69", "\uDA69"], /* Non Private Use High Surrogate */
  ["\uDA6A", "\uDA6A"], /* Non Private Use High Surrogate */
  ["\uDA6B", "\uDA6B"], /* Non Private Use High Surrogate */
  ["\uDA6C", "\uDA6C"], /* Non Private Use High Surrogate */
  ["\uDA6D", "\uDA6D"], /* Non Private Use High Surrogate */
  ["\uDA6E", "\uDA6E"], /* Non Private Use High Surrogate */
  ["\uDA6F", "\uDA6F"], /* Non Private Use High Surrogate */
  ["\uDA70", "\uDA70"], /* Non Private Use High Surrogate */
  ["\uDA71", "\uDA71"], /* Non Private Use High Surrogate */
  ["\uDA72", "\uDA72"], /* Non Private Use High Surrogate */
  ["\uDA73", "\uDA73"], /* Non Private Use High Surrogate */
  ["\uDA74", "\uDA74"], /* Non Private Use High Surrogate */
  ["\uDA75", "\uDA75"], /* Non Private Use High Surrogate */
  ["\uDA76", "\uDA76"], /* Non Private Use High Surrogate */
  ["\uDA77", "\uDA77"], /* Non Private Use High Surrogate */
  ["\uDA78", "\uDA78"], /* Non Private Use High Surrogate */
  ["\uDA79", "\uDA79"], /* Non Private Use High Surrogate */
  ["\uDA7A", "\uDA7A"], /* Non Private Use High Surrogate */
  ["\uDA7B", "\uDA7B"], /* Non Private Use High Surrogate */
  ["\uDA7C", "\uDA7C"], /* Non Private Use High Surrogate */
  ["\uDA7D", "\uDA7D"], /* Non Private Use High Surrogate */
  ["\uDA7E", "\uDA7E"], /* Non Private Use High Surrogate */
  ["\uDA7F", "\uDA7F"], /* Non Private Use High Surrogate */
  ["\uDA80", "\uDA80"], /* Non Private Use High Surrogate */
  ["\uDA81", "\uDA81"], /* Non Private Use High Surrogate */
  ["\uDA82", "\uDA82"], /* Non Private Use High Surrogate */
  ["\uDA83", "\uDA83"], /* Non Private Use High Surrogate */
  ["\uDA84", "\uDA84"], /* Non Private Use High Surrogate */
  ["\uDA85", "\uDA85"], /* Non Private Use High Surrogate */
  ["\uDA86", "\uDA86"], /* Non Private Use High Surrogate */
  ["\uDA87", "\uDA87"], /* Non Private Use High Surrogate */
  ["\uDA88", "\uDA88"], /* Non Private Use High Surrogate */
  ["\uDA89", "\uDA89"], /* Non Private Use High Surrogate */
  ["\uDA8A", "\uDA8A"], /* Non Private Use High Surrogate */
  ["\uDA8B", "\uDA8B"], /* Non Private Use High Surrogate */
  ["\uDA8C", "\uDA8C"], /* Non Private Use High Surrogate */
  ["\uDA8D", "\uDA8D"], /* Non Private Use High Surrogate */
  ["\uDA8E", "\uDA8E"], /* Non Private Use High Surrogate */
  ["\uDA8F", "\uDA8F"], /* Non Private Use High Surrogate */
  ["\uDA90", "\uDA90"], /* Non Private Use High Surrogate */
  ["\uDA91", "\uDA91"], /* Non Private Use High Surrogate */
  ["\uDA92", "\uDA92"], /* Non Private Use High Surrogate */
  ["\uDA93", "\uDA93"], /* Non Private Use High Surrogate */
  ["\uDA94", "\uDA94"], /* Non Private Use High Surrogate */
  ["\uDA95", "\uDA95"], /* Non Private Use High Surrogate */
  ["\uDA96", "\uDA96"], /* Non Private Use High Surrogate */
  ["\uDA97", "\uDA97"], /* Non Private Use High Surrogate */
  ["\uDA98", "\uDA98"], /* Non Private Use High Surrogate */
  ["\uDA99", "\uDA99"], /* Non Private Use High Surrogate */
  ["\uDA9A", "\uDA9A"], /* Non Private Use High Surrogate */
  ["\uDA9B", "\uDA9B"], /* Non Private Use High Surrogate */
  ["\uDA9C", "\uDA9C"], /* Non Private Use High Surrogate */
  ["\uDA9D", "\uDA9D"], /* Non Private Use High Surrogate */
  ["\uDA9E", "\uDA9E"], /* Non Private Use High Surrogate */
  ["\uDA9F", "\uDA9F"], /* Non Private Use High Surrogate */
  ["\uDAA0", "\uDAA0"], /* Non Private Use High Surrogate */
  ["\uDAA1", "\uDAA1"], /* Non Private Use High Surrogate */
  ["\uDAA2", "\uDAA2"], /* Non Private Use High Surrogate */
  ["\uDAA3", "\uDAA3"], /* Non Private Use High Surrogate */
  ["\uDAA4", "\uDAA4"], /* Non Private Use High Surrogate */
  ["\uDAA5", "\uDAA5"], /* Non Private Use High Surrogate */
  ["\uDAA6", "\uDAA6"], /* Non Private Use High Surrogate */
  ["\uDAA7", "\uDAA7"], /* Non Private Use High Surrogate */
  ["\uDAA8", "\uDAA8"], /* Non Private Use High Surrogate */
  ["\uDAA9", "\uDAA9"], /* Non Private Use High Surrogate */
  ["\uDAAA", "\uDAAA"], /* Non Private Use High Surrogate */
  ["\uDAAB", "\uDAAB"], /* Non Private Use High Surrogate */
  ["\uDAAC", "\uDAAC"], /* Non Private Use High Surrogate */
  ["\uDAAD", "\uDAAD"], /* Non Private Use High Surrogate */
  ["\uDAAE", "\uDAAE"], /* Non Private Use High Surrogate */
  ["\uDAAF", "\uDAAF"], /* Non Private Use High Surrogate */
  ["\uDAB0", "\uDAB0"], /* Non Private Use High Surrogate */
  ["\uDAB1", "\uDAB1"], /* Non Private Use High Surrogate */
  ["\uDAB2", "\uDAB2"], /* Non Private Use High Surrogate */
  ["\uDAB3", "\uDAB3"], /* Non Private Use High Surrogate */
  ["\uDAB4", "\uDAB4"], /* Non Private Use High Surrogate */
  ["\uDAB5", "\uDAB5"], /* Non Private Use High Surrogate */
  ["\uDAB6", "\uDAB6"], /* Non Private Use High Surrogate */
  ["\uDAB7", "\uDAB7"], /* Non Private Use High Surrogate */
  ["\uDAB8", "\uDAB8"], /* Non Private Use High Surrogate */
  ["\uDAB9", "\uDAB9"], /* Non Private Use High Surrogate */
  ["\uDABA", "\uDABA"], /* Non Private Use High Surrogate */
  ["\uDABB", "\uDABB"], /* Non Private Use High Surrogate */
  ["\uDABC", "\uDABC"], /* Non Private Use High Surrogate */
  ["\uDABD", "\uDABD"], /* Non Private Use High Surrogate */
  ["\uDABE", "\uDABE"], /* Non Private Use High Surrogate */
  ["\uDABF", "\uDABF"], /* Non Private Use High Surrogate */
  ["\uDAC0", "\uDAC0"], /* Non Private Use High Surrogate */
  ["\uDAC1", "\uDAC1"], /* Non Private Use High Surrogate */
  ["\uDAC2", "\uDAC2"], /* Non Private Use High Surrogate */
  ["\uDAC3", "\uDAC3"], /* Non Private Use High Surrogate */
  ["\uDAC4", "\uDAC4"], /* Non Private Use High Surrogate */
  ["\uDAC5", "\uDAC5"], /* Non Private Use High Surrogate */
  ["\uDAC6", "\uDAC6"], /* Non Private Use High Surrogate */
  ["\uDAC7", "\uDAC7"], /* Non Private Use High Surrogate */
  ["\uDAC8", "\uDAC8"], /* Non Private Use High Surrogate */
  ["\uDAC9", "\uDAC9"], /* Non Private Use High Surrogate */
  ["\uDACA", "\uDACA"], /* Non Private Use High Surrogate */
  ["\uDACB", "\uDACB"], /* Non Private Use High Surrogate */
  ["\uDACC", "\uDACC"], /* Non Private Use High Surrogate */
  ["\uDACD", "\uDACD"], /* Non Private Use High Surrogate */
  ["\uDACE", "\uDACE"], /* Non Private Use High Surrogate */
  ["\uDACF", "\uDACF"], /* Non Private Use High Surrogate */
  ["\uDAD0", "\uDAD0"], /* Non Private Use High Surrogate */
  ["\uDAD1", "\uDAD1"], /* Non Private Use High Surrogate */
  ["\uDAD2", "\uDAD2"], /* Non Private Use High Surrogate */
  ["\uDAD3", "\uDAD3"], /* Non Private Use High Surrogate */
  ["\uDAD4", "\uDAD4"], /* Non Private Use High Surrogate */
  ["\uDAD5", "\uDAD5"], /* Non Private Use High Surrogate */
  ["\uDAD6", "\uDAD6"], /* Non Private Use High Surrogate */
  ["\uDAD7", "\uDAD7"], /* Non Private Use High Surrogate */
  ["\uDAD8", "\uDAD8"], /* Non Private Use High Surrogate */
  ["\uDAD9", "\uDAD9"], /* Non Private Use High Surrogate */
  ["\uDADA", "\uDADA"], /* Non Private Use High Surrogate */
  ["\uDADB", "\uDADB"], /* Non Private Use High Surrogate */
  ["\uDADC", "\uDADC"], /* Non Private Use High Surrogate */
  ["\uDADD", "\uDADD"], /* Non Private Use High Surrogate */
  ["\uDADE", "\uDADE"], /* Non Private Use High Surrogate */
  ["\uDADF", "\uDADF"], /* Non Private Use High Surrogate */
  ["\uDAE0", "\uDAE0"], /* Non Private Use High Surrogate */
  ["\uDAE1", "\uDAE1"], /* Non Private Use High Surrogate */
  ["\uDAE2", "\uDAE2"], /* Non Private Use High Surrogate */
  ["\uDAE3", "\uDAE3"], /* Non Private Use High Surrogate */
  ["\uDAE4", "\uDAE4"], /* Non Private Use High Surrogate */
  ["\uDAE5", "\uDAE5"], /* Non Private Use High Surrogate */
  ["\uDAE6", "\uDAE6"], /* Non Private Use High Surrogate */
  ["\uDAE7", "\uDAE7"], /* Non Private Use High Surrogate */
  ["\uDAE8", "\uDAE8"], /* Non Private Use High Surrogate */
  ["\uDAE9", "\uDAE9"], /* Non Private Use High Surrogate */
  ["\uDAEA", "\uDAEA"], /* Non Private Use High Surrogate */
  ["\uDAEB", "\uDAEB"], /* Non Private Use High Surrogate */
  ["\uDAEC", "\uDAEC"], /* Non Private Use High Surrogate */
  ["\uDAED", "\uDAED"], /* Non Private Use High Surrogate */
  ["\uDAEE", "\uDAEE"], /* Non Private Use High Surrogate */
  ["\uDAEF", "\uDAEF"], /* Non Private Use High Surrogate */
  ["\uDAF0", "\uDAF0"], /* Non Private Use High Surrogate */
  ["\uDAF1", "\uDAF1"], /* Non Private Use High Surrogate */
  ["\uDAF2", "\uDAF2"], /* Non Private Use High Surrogate */
  ["\uDAF3", "\uDAF3"], /* Non Private Use High Surrogate */
  ["\uDAF4", "\uDAF4"], /* Non Private Use High Surrogate */
  ["\uDAF5", "\uDAF5"], /* Non Private Use High Surrogate */
  ["\uDAF6", "\uDAF6"], /* Non Private Use High Surrogate */
  ["\uDAF7", "\uDAF7"], /* Non Private Use High Surrogate */
  ["\uDAF8", "\uDAF8"], /* Non Private Use High Surrogate */
  ["\uDAF9", "\uDAF9"], /* Non Private Use High Surrogate */
  ["\uDAFA", "\uDAFA"], /* Non Private Use High Surrogate */
  ["\uDAFB", "\uDAFB"], /* Non Private Use High Surrogate */
  ["\uDAFC", "\uDAFC"], /* Non Private Use High Surrogate */
  ["\uDAFD", "\uDAFD"], /* Non Private Use High Surrogate */
  ["\uDAFE", "\uDAFE"], /* Non Private Use High Surrogate */
  ["\uDAFF", "\uDAFF"], /* Non Private Use High Surrogate */
  ["\uDB00", "\uDB00"], /* Non Private Use High Surrogate */
  ["\uDB01", "\uDB01"], /* Non Private Use High Surrogate */
  ["\uDB02", "\uDB02"], /* Non Private Use High Surrogate */
  ["\uDB03", "\uDB03"], /* Non Private Use High Surrogate */
  ["\uDB04", "\uDB04"], /* Non Private Use High Surrogate */
  ["\uDB05", "\uDB05"], /* Non Private Use High Surrogate */
  ["\uDB06", "\uDB06"], /* Non Private Use High Surrogate */
  ["\uDB07", "\uDB07"], /* Non Private Use High Surrogate */
  ["\uDB08", "\uDB08"], /* Non Private Use High Surrogate */
  ["\uDB09", "\uDB09"], /* Non Private Use High Surrogate */
  ["\uDB0A", "\uDB0A"], /* Non Private Use High Surrogate */
  ["\uDB0B", "\uDB0B"], /* Non Private Use High Surrogate */
  ["\uDB0C", "\uDB0C"], /* Non Private Use High Surrogate */
  ["\uDB0D", "\uDB0D"], /* Non Private Use High Surrogate */
  ["\uDB0E", "\uDB0E"], /* Non Private Use High Surrogate */
  ["\uDB0F", "\uDB0F"], /* Non Private Use High Surrogate */
  ["\uDB10", "\uDB10"], /* Non Private Use High Surrogate */
  ["\uDB11", "\uDB11"], /* Non Private Use High Surrogate */
  ["\uDB12", "\uDB12"], /* Non Private Use High Surrogate */
  ["\uDB13", "\uDB13"], /* Non Private Use High Surrogate */
  ["\uDB14", "\uDB14"], /* Non Private Use High Surrogate */
  ["\uDB15", "\uDB15"], /* Non Private Use High Surrogate */
  ["\uDB16", "\uDB16"], /* Non Private Use High Surrogate */
  ["\uDB17", "\uDB17"], /* Non Private Use High Surrogate */
  ["\uDB18", "\uDB18"], /* Non Private Use High Surrogate */
  ["\uDB19", "\uDB19"], /* Non Private Use High Surrogate */
  ["\uDB1A", "\uDB1A"], /* Non Private Use High Surrogate */
  ["\uDB1B", "\uDB1B"], /* Non Private Use High Surrogate */
  ["\uDB1C", "\uDB1C"], /* Non Private Use High Surrogate */
  ["\uDB1D", "\uDB1D"], /* Non Private Use High Surrogate */
  ["\uDB1E", "\uDB1E"], /* Non Private Use High Surrogate */
  ["\uDB1F", "\uDB1F"], /* Non Private Use High Surrogate */
  ["\uDB20", "\uDB20"], /* Non Private Use High Surrogate */
  ["\uDB21", "\uDB21"], /* Non Private Use High Surrogate */
  ["\uDB22", "\uDB22"], /* Non Private Use High Surrogate */
  ["\uDB23", "\uDB23"], /* Non Private Use High Surrogate */
  ["\uDB24", "\uDB24"], /* Non Private Use High Surrogate */
  ["\uDB25", "\uDB25"], /* Non Private Use High Surrogate */
  ["\uDB26", "\uDB26"], /* Non Private Use High Surrogate */
  ["\uDB27", "\uDB27"], /* Non Private Use High Surrogate */
  ["\uDB28", "\uDB28"], /* Non Private Use High Surrogate */
  ["\uDB29", "\uDB29"], /* Non Private Use High Surrogate */
  ["\uDB2A", "\uDB2A"], /* Non Private Use High Surrogate */
  ["\uDB2B", "\uDB2B"], /* Non Private Use High Surrogate */
  ["\uDB2C", "\uDB2C"], /* Non Private Use High Surrogate */
  ["\uDB2D", "\uDB2D"], /* Non Private Use High Surrogate */
  ["\uDB2E", "\uDB2E"], /* Non Private Use High Surrogate */
  ["\uDB2F", "\uDB2F"], /* Non Private Use High Surrogate */
  ["\uDB30", "\uDB30"], /* Non Private Use High Surrogate */
  ["\uDB31", "\uDB31"], /* Non Private Use High Surrogate */
  ["\uDB32", "\uDB32"], /* Non Private Use High Surrogate */
  ["\uDB33", "\uDB33"], /* Non Private Use High Surrogate */
  ["\uDB34", "\uDB34"], /* Non Private Use High Surrogate */
  ["\uDB35", "\uDB35"], /* Non Private Use High Surrogate */
  ["\uDB36", "\uDB36"], /* Non Private Use High Surrogate */
  ["\uDB37", "\uDB37"], /* Non Private Use High Surrogate */
  ["\uDB38", "\uDB38"], /* Non Private Use High Surrogate */
  ["\uDB39", "\uDB39"], /* Non Private Use High Surrogate */
  ["\uDB3A", "\uDB3A"], /* Non Private Use High Surrogate */
  ["\uDB3B", "\uDB3B"], /* Non Private Use High Surrogate */
  ["\uDB3C", "\uDB3C"], /* Non Private Use High Surrogate */
  ["\uDB3D", "\uDB3D"], /* Non Private Use High Surrogate */
  ["\uDB3E", "\uDB3E"], /* Non Private Use High Surrogate */
  ["\uDB3F", "\uDB3F"], /* Non Private Use High Surrogate */
  ["\uDB40", "\uDB40"], /* Non Private Use High Surrogate */
  ["\uDB41", "\uDB41"], /* Non Private Use High Surrogate */
  ["\uDB42", "\uDB42"], /* Non Private Use High Surrogate */
  ["\uDB43", "\uDB43"], /* Non Private Use High Surrogate */
  ["\uDB44", "\uDB44"], /* Non Private Use High Surrogate */
  ["\uDB45", "\uDB45"], /* Non Private Use High Surrogate */
  ["\uDB46", "\uDB46"], /* Non Private Use High Surrogate */
  ["\uDB47", "\uDB47"], /* Non Private Use High Surrogate */
  ["\uDB48", "\uDB48"], /* Non Private Use High Surrogate */
  ["\uDB49", "\uDB49"], /* Non Private Use High Surrogate */
  ["\uDB4A", "\uDB4A"], /* Non Private Use High Surrogate */
  ["\uDB4B", "\uDB4B"], /* Non Private Use High Surrogate */
  ["\uDB4C", "\uDB4C"], /* Non Private Use High Surrogate */
  ["\uDB4D", "\uDB4D"], /* Non Private Use High Surrogate */
  ["\uDB4E", "\uDB4E"], /* Non Private Use High Surrogate */
  ["\uDB4F", "\uDB4F"], /* Non Private Use High Surrogate */
  ["\uDB50", "\uDB50"], /* Non Private Use High Surrogate */
  ["\uDB51", "\uDB51"], /* Non Private Use High Surrogate */
  ["\uDB52", "\uDB52"], /* Non Private Use High Surrogate */
  ["\uDB53", "\uDB53"], /* Non Private Use High Surrogate */
  ["\uDB54", "\uDB54"], /* Non Private Use High Surrogate */
  ["\uDB55", "\uDB55"], /* Non Private Use High Surrogate */
  ["\uDB56", "\uDB56"], /* Non Private Use High Surrogate */
  ["\uDB57", "\uDB57"], /* Non Private Use High Surrogate */
  ["\uDB58", "\uDB58"], /* Non Private Use High Surrogate */
  ["\uDB59", "\uDB59"], /* Non Private Use High Surrogate */
  ["\uDB5A", "\uDB5A"], /* Non Private Use High Surrogate */
  ["\uDB5B", "\uDB5B"], /* Non Private Use High Surrogate */
  ["\uDB5C", "\uDB5C"], /* Non Private Use High Surrogate */
  ["\uDB5D", "\uDB5D"], /* Non Private Use High Surrogate */
  ["\uDB5E", "\uDB5E"], /* Non Private Use High Surrogate */
  ["\uDB5F", "\uDB5F"], /* Non Private Use High Surrogate */
  ["\uDB60", "\uDB60"], /* Non Private Use High Surrogate */
  ["\uDB61", "\uDB61"], /* Non Private Use High Surrogate */
  ["\uDB62", "\uDB62"], /* Non Private Use High Surrogate */
  ["\uDB63", "\uDB63"], /* Non Private Use High Surrogate */
  ["\uDB64", "\uDB64"], /* Non Private Use High Surrogate */
  ["\uDB65", "\uDB65"], /* Non Private Use High Surrogate */
  ["\uDB66", "\uDB66"], /* Non Private Use High Surrogate */
  ["\uDB67", "\uDB67"], /* Non Private Use High Surrogate */
  ["\uDB68", "\uDB68"], /* Non Private Use High Surrogate */
  ["\uDB69", "\uDB69"], /* Non Private Use High Surrogate */
  ["\uDB6A", "\uDB6A"], /* Non Private Use High Surrogate */
  ["\uDB6B", "\uDB6B"], /* Non Private Use High Surrogate */
  ["\uDB6C", "\uDB6C"], /* Non Private Use High Surrogate */
  ["\uDB6D", "\uDB6D"], /* Non Private Use High Surrogate */
  ["\uDB6E", "\uDB6E"], /* Non Private Use High Surrogate */
  ["\uDB6F", "\uDB6F"], /* Non Private Use High Surrogate */
  ["\uDB70", "\uDB70"], /* Non Private Use High Surrogate */
  ["\uDB71", "\uDB71"], /* Non Private Use High Surrogate */
  ["\uDB72", "\uDB72"], /* Non Private Use High Surrogate */
  ["\uDB73", "\uDB73"], /* Non Private Use High Surrogate */
  ["\uDB74", "\uDB74"], /* Non Private Use High Surrogate */
  ["\uDB75", "\uDB75"], /* Non Private Use High Surrogate */
  ["\uDB76", "\uDB76"], /* Non Private Use High Surrogate */
  ["\uDB77", "\uDB77"], /* Non Private Use High Surrogate */
  ["\uDB78", "\uDB78"], /* Non Private Use High Surrogate */
  ["\uDB79", "\uDB79"], /* Non Private Use High Surrogate */
  ["\uDB7A", "\uDB7A"], /* Non Private Use High Surrogate */
  ["\uDB7B", "\uDB7B"], /* Non Private Use High Surrogate */
  ["\uDB7C", "\uDB7C"], /* Non Private Use High Surrogate */
  ["\uDB7D", "\uDB7D"], /* Non Private Use High Surrogate */
  ["\uDB7E", "\uDB7E"], /* Non Private Use High Surrogate */
  ["\uDB7F", "\uDB7F"], /* Non Private Use High Surrogate */
  ["\uDB80", "\uDB80"], /* Private Use High Surrogate */
  ["\uDB81", "\uDB81"], /* Private Use High Surrogate */
  ["\uDB82", "\uDB82"], /* Private Use High Surrogate */
  ["\uDB83", "\uDB83"], /* Private Use High Surrogate */
  ["\uDB84", "\uDB84"], /* Private Use High Surrogate */
  ["\uDB85", "\uDB85"], /* Private Use High Surrogate */
  ["\uDB86", "\uDB86"], /* Private Use High Surrogate */
  ["\uDB87", "\uDB87"], /* Private Use High Surrogate */
  ["\uDB88", "\uDB88"], /* Private Use High Surrogate */
  ["\uDB89", "\uDB89"], /* Private Use High Surrogate */
  ["\uDB8A", "\uDB8A"], /* Private Use High Surrogate */
  ["\uDB8B", "\uDB8B"], /* Private Use High Surrogate */
  ["\uDB8C", "\uDB8C"], /* Private Use High Surrogate */
  ["\uDB8D", "\uDB8D"], /* Private Use High Surrogate */
  ["\uDB8E", "\uDB8E"], /* Private Use High Surrogate */
  ["\uDB8F", "\uDB8F"], /* Private Use High Surrogate */
  ["\uDB90", "\uDB90"], /* Private Use High Surrogate */
  ["\uDB91", "\uDB91"], /* Private Use High Surrogate */
  ["\uDB92", "\uDB92"], /* Private Use High Surrogate */
  ["\uDB93", "\uDB93"], /* Private Use High Surrogate */
  ["\uDB94", "\uDB94"], /* Private Use High Surrogate */
  ["\uDB95", "\uDB95"], /* Private Use High Surrogate */
  ["\uDB96", "\uDB96"], /* Private Use High Surrogate */
  ["\uDB97", "\uDB97"], /* Private Use High Surrogate */
  ["\uDB98", "\uDB98"], /* Private Use High Surrogate */
  ["\uDB99", "\uDB99"], /* Private Use High Surrogate */
  ["\uDB9A", "\uDB9A"], /* Private Use High Surrogate */
  ["\uDB9B", "\uDB9B"], /* Private Use High Surrogate */
  ["\uDB9C", "\uDB9C"], /* Private Use High Surrogate */
  ["\uDB9D", "\uDB9D"], /* Private Use High Surrogate */
  ["\uDB9E", "\uDB9E"], /* Private Use High Surrogate */
  ["\uDB9F", "\uDB9F"], /* Private Use High Surrogate */
  ["\uDBA0", "\uDBA0"], /* Private Use High Surrogate */
  ["\uDBA1", "\uDBA1"], /* Private Use High Surrogate */
  ["\uDBA2", "\uDBA2"], /* Private Use High Surrogate */
  ["\uDBA3", "\uDBA3"], /* Private Use High Surrogate */
  ["\uDBA4", "\uDBA4"], /* Private Use High Surrogate */
  ["\uDBA5", "\uDBA5"], /* Private Use High Surrogate */
  ["\uDBA6", "\uDBA6"], /* Private Use High Surrogate */
  ["\uDBA7", "\uDBA7"], /* Private Use High Surrogate */
  ["\uDBA8", "\uDBA8"], /* Private Use High Surrogate */
  ["\uDBA9", "\uDBA9"], /* Private Use High Surrogate */
  ["\uDBAA", "\uDBAA"], /* Private Use High Surrogate */
  ["\uDBAB", "\uDBAB"], /* Private Use High Surrogate */
  ["\uDBAC", "\uDBAC"], /* Private Use High Surrogate */
  ["\uDBAD", "\uDBAD"], /* Private Use High Surrogate */
  ["\uDBAE", "\uDBAE"], /* Private Use High Surrogate */
  ["\uDBAF", "\uDBAF"], /* Private Use High Surrogate */
  ["\uDBB0", "\uDBB0"], /* Private Use High Surrogate */
  ["\uDBB1", "\uDBB1"], /* Private Use High Surrogate */
  ["\uDBB2", "\uDBB2"], /* Private Use High Surrogate */
  ["\uDBB3", "\uDBB3"], /* Private Use High Surrogate */
  ["\uDBB4", "\uDBB4"], /* Private Use High Surrogate */
  ["\uDBB5", "\uDBB5"], /* Private Use High Surrogate */
  ["\uDBB6", "\uDBB6"], /* Private Use High Surrogate */
  ["\uDBB7", "\uDBB7"], /* Private Use High Surrogate */
  ["\uDBB8", "\uDBB8"], /* Private Use High Surrogate */
  ["\uDBB9", "\uDBB9"], /* Private Use High Surrogate */
  ["\uDBBA", "\uDBBA"], /* Private Use High Surrogate */
  ["\uDBBB", "\uDBBB"], /* Private Use High Surrogate */
  ["\uDBBC", "\uDBBC"], /* Private Use High Surrogate */
  ["\uDBBD", "\uDBBD"], /* Private Use High Surrogate */
  ["\uDBBE", "\uDBBE"], /* Private Use High Surrogate */
  ["\uDBBF", "\uDBBF"], /* Private Use High Surrogate */
  ["\uDBC0", "\uDBC0"], /* Private Use High Surrogate */
  ["\uDBC1", "\uDBC1"], /* Private Use High Surrogate */
  ["\uDBC2", "\uDBC2"], /* Private Use High Surrogate */
  ["\uDBC3", "\uDBC3"], /* Private Use High Surrogate */
  ["\uDBC4", "\uDBC4"], /* Private Use High Surrogate */
  ["\uDBC5", "\uDBC5"], /* Private Use High Surrogate */
  ["\uDBC6", "\uDBC6"], /* Private Use High Surrogate */
  ["\uDBC7", "\uDBC7"], /* Private Use High Surrogate */
  ["\uDBC8", "\uDBC8"], /* Private Use High Surrogate */
  ["\uDBC9", "\uDBC9"], /* Private Use High Surrogate */
  ["\uDBCA", "\uDBCA"], /* Private Use High Surrogate */
  ["\uDBCB", "\uDBCB"], /* Private Use High Surrogate */
  ["\uDBCC", "\uDBCC"], /* Private Use High Surrogate */
  ["\uDBCD", "\uDBCD"], /* Private Use High Surrogate */
  ["\uDBCE", "\uDBCE"], /* Private Use High Surrogate */
  ["\uDBCF", "\uDBCF"], /* Private Use High Surrogate */
  ["\uDBD0", "\uDBD0"], /* Private Use High Surrogate */
  ["\uDBD1", "\uDBD1"], /* Private Use High Surrogate */
  ["\uDBD2", "\uDBD2"], /* Private Use High Surrogate */
  ["\uDBD3", "\uDBD3"], /* Private Use High Surrogate */
  ["\uDBD4", "\uDBD4"], /* Private Use High Surrogate */
  ["\uDBD5", "\uDBD5"], /* Private Use High Surrogate */
  ["\uDBD6", "\uDBD6"], /* Private Use High Surrogate */
  ["\uDBD7", "\uDBD7"], /* Private Use High Surrogate */
  ["\uDBD8", "\uDBD8"], /* Private Use High Surrogate */
  ["\uDBD9", "\uDBD9"], /* Private Use High Surrogate */
  ["\uDBDA", "\uDBDA"], /* Private Use High Surrogate */
  ["\uDBDB", "\uDBDB"], /* Private Use High Surrogate */
  ["\uDBDC", "\uDBDC"], /* Private Use High Surrogate */
  ["\uDBDD", "\uDBDD"], /* Private Use High Surrogate */
  ["\uDBDE", "\uDBDE"], /* Private Use High Surrogate */
  ["\uDBDF", "\uDBDF"], /* Private Use High Surrogate */
  ["\uDBE0", "\uDBE0"], /* Private Use High Surrogate */
  ["\uDBE1", "\uDBE1"], /* Private Use High Surrogate */
  ["\uDBE2", "\uDBE2"], /* Private Use High Surrogate */
  ["\uDBE3", "\uDBE3"], /* Private Use High Surrogate */
  ["\uDBE4", "\uDBE4"], /* Private Use High Surrogate */
  ["\uDBE5", "\uDBE5"], /* Private Use High Surrogate */
  ["\uDBE6", "\uDBE6"], /* Private Use High Surrogate */
  ["\uDBE7", "\uDBE7"], /* Private Use High Surrogate */
  ["\uDBE8", "\uDBE8"], /* Private Use High Surrogate */
  ["\uDBE9", "\uDBE9"], /* Private Use High Surrogate */
  ["\uDBEA", "\uDBEA"], /* Private Use High Surrogate */
  ["\uDBEB", "\uDBEB"], /* Private Use High Surrogate */
  ["\uDBEC", "\uDBEC"], /* Private Use High Surrogate */
  ["\uDBED", "\uDBED"], /* Private Use High Surrogate */
  ["\uDBEE", "\uDBEE"], /* Private Use High Surrogate */
  ["\uDBEF", "\uDBEF"], /* Private Use High Surrogate */
  ["\uDBF0", "\uDBF0"], /* Private Use High Surrogate */
  ["\uDBF1", "\uDBF1"], /* Private Use High Surrogate */
  ["\uDBF2", "\uDBF2"], /* Private Use High Surrogate */
  ["\uDBF3", "\uDBF3"], /* Private Use High Surrogate */
  ["\uDBF4", "\uDBF4"], /* Private Use High Surrogate */
  ["\uDBF5", "\uDBF5"], /* Private Use High Surrogate */
  ["\uDBF6", "\uDBF6"], /* Private Use High Surrogate */
  ["\uDBF7", "\uDBF7"], /* Private Use High Surrogate */
  ["\uDBF8", "\uDBF8"], /* Private Use High Surrogate */
  ["\uDBF9", "\uDBF9"], /* Private Use High Surrogate */
  ["\uDBFA", "\uDBFA"], /* Private Use High Surrogate */
  ["\uDBFB", "\uDBFB"], /* Private Use High Surrogate */
  ["\uDBFC", "\uDBFC"], /* Private Use High Surrogate */
  ["\uDBFD", "\uDBFD"], /* Private Use High Surrogate */
  ["\uDBFE", "\uDBFE"], /* Private Use High Surrogate */
  ["\uDBFF", "\uDBFF"], /* Private Use High Surrogate */
  ["\uDC00", "\uDC00"], /* Low Surrogate */
  ["\uDC01", "\uDC01"], /* Low Surrogate */
  ["\uDC02", "\uDC02"], /* Low Surrogate */
  ["\uDC03", "\uDC03"], /* Low Surrogate */
  ["\uDC04", "\uDC04"], /* Low Surrogate */
  ["\uDC05", "\uDC05"], /* Low Surrogate */
  ["\uDC06", "\uDC06"], /* Low Surrogate */
  ["\uDC07", "\uDC07"], /* Low Surrogate */
  ["\uDC08", "\uDC08"], /* Low Surrogate */
  ["\uDC09", "\uDC09"], /* Low Surrogate */
  ["\uDC0A", "\uDC0A"], /* Low Surrogate */
  ["\uDC0B", "\uDC0B"], /* Low Surrogate */
  ["\uDC0C", "\uDC0C"], /* Low Surrogate */
  ["\uDC0D", "\uDC0D"], /* Low Surrogate */
  ["\uDC0E", "\uDC0E"], /* Low Surrogate */
  ["\uDC0F", "\uDC0F"], /* Low Surrogate */
  ["\uDC10", "\uDC10"], /* Low Surrogate */
  ["\uDC11", "\uDC11"], /* Low Surrogate */
  ["\uDC12", "\uDC12"], /* Low Surrogate */
  ["\uDC13", "\uDC13"], /* Low Surrogate */
  ["\uDC14", "\uDC14"], /* Low Surrogate */
  ["\uDC15", "\uDC15"], /* Low Surrogate */
  ["\uDC16", "\uDC16"], /* Low Surrogate */
  ["\uDC17", "\uDC17"], /* Low Surrogate */
  ["\uDC18", "\uDC18"], /* Low Surrogate */
  ["\uDC19", "\uDC19"], /* Low Surrogate */
  ["\uDC1A", "\uDC1A"], /* Low Surrogate */
  ["\uDC1B", "\uDC1B"], /* Low Surrogate */
  ["\uDC1C", "\uDC1C"], /* Low Surrogate */
  ["\uDC1D", "\uDC1D"], /* Low Surrogate */
  ["\uDC1E", "\uDC1E"], /* Low Surrogate */
  ["\uDC1F", "\uDC1F"], /* Low Surrogate */
  ["\uDC20", "\uDC20"], /* Low Surrogate */
  ["\uDC21", "\uDC21"], /* Low Surrogate */
  ["\uDC22", "\uDC22"], /* Low Surrogate */
  ["\uDC23", "\uDC23"], /* Low Surrogate */
  ["\uDC24", "\uDC24"], /* Low Surrogate */
  ["\uDC25", "\uDC25"], /* Low Surrogate */
  ["\uDC26", "\uDC26"], /* Low Surrogate */
  ["\uDC27", "\uDC27"], /* Low Surrogate */
  ["\uDC28", "\uDC28"], /* Low Surrogate */
  ["\uDC29", "\uDC29"], /* Low Surrogate */
  ["\uDC2A", "\uDC2A"], /* Low Surrogate */
  ["\uDC2B", "\uDC2B"], /* Low Surrogate */
  ["\uDC2C", "\uDC2C"], /* Low Surrogate */
  ["\uDC2D", "\uDC2D"], /* Low Surrogate */
  ["\uDC2E", "\uDC2E"], /* Low Surrogate */
  ["\uDC2F", "\uDC2F"], /* Low Surrogate */
  ["\uDC30", "\uDC30"], /* Low Surrogate */
  ["\uDC31", "\uDC31"], /* Low Surrogate */
  ["\uDC32", "\uDC32"], /* Low Surrogate */
  ["\uDC33", "\uDC33"], /* Low Surrogate */
  ["\uDC34", "\uDC34"], /* Low Surrogate */
  ["\uDC35", "\uDC35"], /* Low Surrogate */
  ["\uDC36", "\uDC36"], /* Low Surrogate */
  ["\uDC37", "\uDC37"], /* Low Surrogate */
  ["\uDC38", "\uDC38"], /* Low Surrogate */
  ["\uDC39", "\uDC39"], /* Low Surrogate */
  ["\uDC3A", "\uDC3A"], /* Low Surrogate */
  ["\uDC3B", "\uDC3B"], /* Low Surrogate */
  ["\uDC3C", "\uDC3C"], /* Low Surrogate */
  ["\uDC3D", "\uDC3D"], /* Low Surrogate */
  ["\uDC3E", "\uDC3E"], /* Low Surrogate */
  ["\uDC3F", "\uDC3F"], /* Low Surrogate */
  ["\uDC40", "\uDC40"], /* Low Surrogate */
  ["\uDC41", "\uDC41"], /* Low Surrogate */
  ["\uDC42", "\uDC42"], /* Low Surrogate */
  ["\uDC43", "\uDC43"], /* Low Surrogate */
  ["\uDC44", "\uDC44"], /* Low Surrogate */
  ["\uDC45", "\uDC45"], /* Low Surrogate */
  ["\uDC46", "\uDC46"], /* Low Surrogate */
  ["\uDC47", "\uDC47"], /* Low Surrogate */
  ["\uDC48", "\uDC48"], /* Low Surrogate */
  ["\uDC49", "\uDC49"], /* Low Surrogate */
  ["\uDC4A", "\uDC4A"], /* Low Surrogate */
  ["\uDC4B", "\uDC4B"], /* Low Surrogate */
  ["\uDC4C", "\uDC4C"], /* Low Surrogate */
  ["\uDC4D", "\uDC4D"], /* Low Surrogate */
  ["\uDC4E", "\uDC4E"], /* Low Surrogate */
  ["\uDC4F", "\uDC4F"], /* Low Surrogate */
  ["\uDC50", "\uDC50"], /* Low Surrogate */
  ["\uDC51", "\uDC51"], /* Low Surrogate */
  ["\uDC52", "\uDC52"], /* Low Surrogate */
  ["\uDC53", "\uDC53"], /* Low Surrogate */
  ["\uDC54", "\uDC54"], /* Low Surrogate */
  ["\uDC55", "\uDC55"], /* Low Surrogate */
  ["\uDC56", "\uDC56"], /* Low Surrogate */
  ["\uDC57", "\uDC57"], /* Low Surrogate */
  ["\uDC58", "\uDC58"], /* Low Surrogate */
  ["\uDC59", "\uDC59"], /* Low Surrogate */
  ["\uDC5A", "\uDC5A"], /* Low Surrogate */
  ["\uDC5B", "\uDC5B"], /* Low Surrogate */
  ["\uDC5C", "\uDC5C"], /* Low Surrogate */
  ["\uDC5D", "\uDC5D"], /* Low Surrogate */
  ["\uDC5E", "\uDC5E"], /* Low Surrogate */
  ["\uDC5F", "\uDC5F"], /* Low Surrogate */
  ["\uDC60", "\uDC60"], /* Low Surrogate */
  ["\uDC61", "\uDC61"], /* Low Surrogate */
  ["\uDC62", "\uDC62"], /* Low Surrogate */
  ["\uDC63", "\uDC63"], /* Low Surrogate */
  ["\uDC64", "\uDC64"], /* Low Surrogate */
  ["\uDC65", "\uDC65"], /* Low Surrogate */
  ["\uDC66", "\uDC66"], /* Low Surrogate */
  ["\uDC67", "\uDC67"], /* Low Surrogate */
  ["\uDC68", "\uDC68"], /* Low Surrogate */
  ["\uDC69", "\uDC69"], /* Low Surrogate */
  ["\uDC6A", "\uDC6A"], /* Low Surrogate */
  ["\uDC6B", "\uDC6B"], /* Low Surrogate */
  ["\uDC6C", "\uDC6C"], /* Low Surrogate */
  ["\uDC6D", "\uDC6D"], /* Low Surrogate */
  ["\uDC6E", "\uDC6E"], /* Low Surrogate */
  ["\uDC6F", "\uDC6F"], /* Low Surrogate */
  ["\uDC70", "\uDC70"], /* Low Surrogate */
  ["\uDC71", "\uDC71"], /* Low Surrogate */
  ["\uDC72", "\uDC72"], /* Low Surrogate */
  ["\uDC73", "\uDC73"], /* Low Surrogate */
  ["\uDC74", "\uDC74"], /* Low Surrogate */
  ["\uDC75", "\uDC75"], /* Low Surrogate */
  ["\uDC76", "\uDC76"], /* Low Surrogate */
  ["\uDC77", "\uDC77"], /* Low Surrogate */
  ["\uDC78", "\uDC78"], /* Low Surrogate */
  ["\uDC79", "\uDC79"], /* Low Surrogate */
  ["\uDC7A", "\uDC7A"], /* Low Surrogate */
  ["\uDC7B", "\uDC7B"], /* Low Surrogate */
  ["\uDC7C", "\uDC7C"], /* Low Surrogate */
  ["\uDC7D", "\uDC7D"], /* Low Surrogate */
  ["\uDC7E", "\uDC7E"], /* Low Surrogate */
  ["\uDC7F", "\uDC7F"], /* Low Surrogate */
  ["\uDC80", "\uDC80"], /* Low Surrogate */
  ["\uDC81", "\uDC81"], /* Low Surrogate */
  ["\uDC82", "\uDC82"], /* Low Surrogate */
  ["\uDC83", "\uDC83"], /* Low Surrogate */
  ["\uDC84", "\uDC84"], /* Low Surrogate */
  ["\uDC85", "\uDC85"], /* Low Surrogate */
  ["\uDC86", "\uDC86"], /* Low Surrogate */
  ["\uDC87", "\uDC87"], /* Low Surrogate */
  ["\uDC88", "\uDC88"], /* Low Surrogate */
  ["\uDC89", "\uDC89"], /* Low Surrogate */
  ["\uDC8A", "\uDC8A"], /* Low Surrogate */
  ["\uDC8B", "\uDC8B"], /* Low Surrogate */
  ["\uDC8C", "\uDC8C"], /* Low Surrogate */
  ["\uDC8D", "\uDC8D"], /* Low Surrogate */
  ["\uDC8E", "\uDC8E"], /* Low Surrogate */
  ["\uDC8F", "\uDC8F"], /* Low Surrogate */
  ["\uDC90", "\uDC90"], /* Low Surrogate */
  ["\uDC91", "\uDC91"], /* Low Surrogate */
  ["\uDC92", "\uDC92"], /* Low Surrogate */
  ["\uDC93", "\uDC93"], /* Low Surrogate */
  ["\uDC94", "\uDC94"], /* Low Surrogate */
  ["\uDC95", "\uDC95"], /* Low Surrogate */
  ["\uDC96", "\uDC96"], /* Low Surrogate */
  ["\uDC97", "\uDC97"], /* Low Surrogate */
  ["\uDC98", "\uDC98"], /* Low Surrogate */
  ["\uDC99", "\uDC99"], /* Low Surrogate */
  ["\uDC9A", "\uDC9A"], /* Low Surrogate */
  ["\uDC9B", "\uDC9B"], /* Low Surrogate */
  ["\uDC9C", "\uDC9C"], /* Low Surrogate */
  ["\uDC9D", "\uDC9D"], /* Low Surrogate */
  ["\uDC9E", "\uDC9E"], /* Low Surrogate */
  ["\uDC9F", "\uDC9F"], /* Low Surrogate */
  ["\uDCA0", "\uDCA0"], /* Low Surrogate */
  ["\uDCA1", "\uDCA1"], /* Low Surrogate */
  ["\uDCA2", "\uDCA2"], /* Low Surrogate */
  ["\uDCA3", "\uDCA3"], /* Low Surrogate */
  ["\uDCA4", "\uDCA4"], /* Low Surrogate */
  ["\uDCA5", "\uDCA5"], /* Low Surrogate */
  ["\uDCA6", "\uDCA6"], /* Low Surrogate */
  ["\uDCA7", "\uDCA7"], /* Low Surrogate */
  ["\uDCA8", "\uDCA8"], /* Low Surrogate */
  ["\uDCA9", "\uDCA9"], /* Low Surrogate */
  ["\uDCAA", "\uDCAA"], /* Low Surrogate */
  ["\uDCAB", "\uDCAB"], /* Low Surrogate */
  ["\uDCAC", "\uDCAC"], /* Low Surrogate */
  ["\uDCAD", "\uDCAD"], /* Low Surrogate */
  ["\uDCAE", "\uDCAE"], /* Low Surrogate */
  ["\uDCAF", "\uDCAF"], /* Low Surrogate */
  ["\uDCB0", "\uDCB0"], /* Low Surrogate */
  ["\uDCB1", "\uDCB1"], /* Low Surrogate */
  ["\uDCB2", "\uDCB2"], /* Low Surrogate */
  ["\uDCB3", "\uDCB3"], /* Low Surrogate */
  ["\uDCB4", "\uDCB4"], /* Low Surrogate */
  ["\uDCB5", "\uDCB5"], /* Low Surrogate */
  ["\uDCB6", "\uDCB6"], /* Low Surrogate */
  ["\uDCB7", "\uDCB7"], /* Low Surrogate */
  ["\uDCB8", "\uDCB8"], /* Low Surrogate */
  ["\uDCB9", "\uDCB9"], /* Low Surrogate */
  ["\uDCBA", "\uDCBA"], /* Low Surrogate */
  ["\uDCBB", "\uDCBB"], /* Low Surrogate */
  ["\uDCBC", "\uDCBC"], /* Low Surrogate */
  ["\uDCBD", "\uDCBD"], /* Low Surrogate */
  ["\uDCBE", "\uDCBE"], /* Low Surrogate */
  ["\uDCBF", "\uDCBF"], /* Low Surrogate */
  ["\uDCC0", "\uDCC0"], /* Low Surrogate */
  ["\uDCC1", "\uDCC1"], /* Low Surrogate */
  ["\uDCC2", "\uDCC2"], /* Low Surrogate */
  ["\uDCC3", "\uDCC3"], /* Low Surrogate */
  ["\uDCC4", "\uDCC4"], /* Low Surrogate */
  ["\uDCC5", "\uDCC5"], /* Low Surrogate */
  ["\uDCC6", "\uDCC6"], /* Low Surrogate */
  ["\uDCC7", "\uDCC7"], /* Low Surrogate */
  ["\uDCC8", "\uDCC8"], /* Low Surrogate */
  ["\uDCC9", "\uDCC9"], /* Low Surrogate */
  ["\uDCCA", "\uDCCA"], /* Low Surrogate */
  ["\uDCCB", "\uDCCB"], /* Low Surrogate */
  ["\uDCCC", "\uDCCC"], /* Low Surrogate */
  ["\uDCCD", "\uDCCD"], /* Low Surrogate */
  ["\uDCCE", "\uDCCE"], /* Low Surrogate */
  ["\uDCCF", "\uDCCF"], /* Low Surrogate */
  ["\uDCD0", "\uDCD0"], /* Low Surrogate */
  ["\uDCD1", "\uDCD1"], /* Low Surrogate */
  ["\uDCD2", "\uDCD2"], /* Low Surrogate */
  ["\uDCD3", "\uDCD3"], /* Low Surrogate */
  ["\uDCD4", "\uDCD4"], /* Low Surrogate */
  ["\uDCD5", "\uDCD5"], /* Low Surrogate */
  ["\uDCD6", "\uDCD6"], /* Low Surrogate */
  ["\uDCD7", "\uDCD7"], /* Low Surrogate */
  ["\uDCD8", "\uDCD8"], /* Low Surrogate */
  ["\uDCD9", "\uDCD9"], /* Low Surrogate */
  ["\uDCDA", "\uDCDA"], /* Low Surrogate */
  ["\uDCDB", "\uDCDB"], /* Low Surrogate */
  ["\uDCDC", "\uDCDC"], /* Low Surrogate */
  ["\uDCDD", "\uDCDD"], /* Low Surrogate */
  ["\uDCDE", "\uDCDE"], /* Low Surrogate */
  ["\uDCDF", "\uDCDF"], /* Low Surrogate */
  ["\uDCE0", "\uDCE0"], /* Low Surrogate */
  ["\uDCE1", "\uDCE1"], /* Low Surrogate */
  ["\uDCE2", "\uDCE2"], /* Low Surrogate */
  ["\uDCE3", "\uDCE3"], /* Low Surrogate */
  ["\uDCE4", "\uDCE4"], /* Low Surrogate */
  ["\uDCE5", "\uDCE5"], /* Low Surrogate */
  ["\uDCE6", "\uDCE6"], /* Low Surrogate */
  ["\uDCE7", "\uDCE7"], /* Low Surrogate */
  ["\uDCE8", "\uDCE8"], /* Low Surrogate */
  ["\uDCE9", "\uDCE9"], /* Low Surrogate */
  ["\uDCEA", "\uDCEA"], /* Low Surrogate */
  ["\uDCEB", "\uDCEB"], /* Low Surrogate */
  ["\uDCEC", "\uDCEC"], /* Low Surrogate */
  ["\uDCED", "\uDCED"], /* Low Surrogate */
  ["\uDCEE", "\uDCEE"], /* Low Surrogate */
  ["\uDCEF", "\uDCEF"], /* Low Surrogate */
  ["\uDCF0", "\uDCF0"], /* Low Surrogate */
  ["\uDCF1", "\uDCF1"], /* Low Surrogate */
  ["\uDCF2", "\uDCF2"], /* Low Surrogate */
  ["\uDCF3", "\uDCF3"], /* Low Surrogate */
  ["\uDCF4", "\uDCF4"], /* Low Surrogate */
  ["\uDCF5", "\uDCF5"], /* Low Surrogate */
  ["\uDCF6", "\uDCF6"], /* Low Surrogate */
  ["\uDCF7", "\uDCF7"], /* Low Surrogate */
  ["\uDCF8", "\uDCF8"], /* Low Surrogate */
  ["\uDCF9", "\uDCF9"], /* Low Surrogate */
  ["\uDCFA", "\uDCFA"], /* Low Surrogate */
  ["\uDCFB", "\uDCFB"], /* Low Surrogate */
  ["\uDCFC", "\uDCFC"], /* Low Surrogate */
  ["\uDCFD", "\uDCFD"], /* Low Surrogate */
  ["\uDCFE", "\uDCFE"], /* Low Surrogate */
  ["\uDCFF", "\uDCFF"], /* Low Surrogate */
  ["\uDD00", "\uDD00"], /* Low Surrogate */
  ["\uDD01", "\uDD01"], /* Low Surrogate */
  ["\uDD02", "\uDD02"], /* Low Surrogate */
  ["\uDD03", "\uDD03"], /* Low Surrogate */
  ["\uDD04", "\uDD04"], /* Low Surrogate */
  ["\uDD05", "\uDD05"], /* Low Surrogate */
  ["\uDD06", "\uDD06"], /* Low Surrogate */
  ["\uDD07", "\uDD07"], /* Low Surrogate */
  ["\uDD08", "\uDD08"], /* Low Surrogate */
  ["\uDD09", "\uDD09"], /* Low Surrogate */
  ["\uDD0A", "\uDD0A"], /* Low Surrogate */
  ["\uDD0B", "\uDD0B"], /* Low Surrogate */
  ["\uDD0C", "\uDD0C"], /* Low Surrogate */
  ["\uDD0D", "\uDD0D"], /* Low Surrogate */
  ["\uDD0E", "\uDD0E"], /* Low Surrogate */
  ["\uDD0F", "\uDD0F"], /* Low Surrogate */
  ["\uDD10", "\uDD10"], /* Low Surrogate */
  ["\uDD11", "\uDD11"], /* Low Surrogate */
  ["\uDD12", "\uDD12"], /* Low Surrogate */
  ["\uDD13", "\uDD13"], /* Low Surrogate */
  ["\uDD14", "\uDD14"], /* Low Surrogate */
  ["\uDD15", "\uDD15"], /* Low Surrogate */
  ["\uDD16", "\uDD16"], /* Low Surrogate */
  ["\uDD17", "\uDD17"], /* Low Surrogate */
  ["\uDD18", "\uDD18"], /* Low Surrogate */
  ["\uDD19", "\uDD19"], /* Low Surrogate */
  ["\uDD1A", "\uDD1A"], /* Low Surrogate */
  ["\uDD1B", "\uDD1B"], /* Low Surrogate */
  ["\uDD1C", "\uDD1C"], /* Low Surrogate */
  ["\uDD1D", "\uDD1D"], /* Low Surrogate */
  ["\uDD1E", "\uDD1E"], /* Low Surrogate */
  ["\uDD1F", "\uDD1F"], /* Low Surrogate */
  ["\uDD20", "\uDD20"], /* Low Surrogate */
  ["\uDD21", "\uDD21"], /* Low Surrogate */
  ["\uDD22", "\uDD22"], /* Low Surrogate */
  ["\uDD23", "\uDD23"], /* Low Surrogate */
  ["\uDD24", "\uDD24"], /* Low Surrogate */
  ["\uDD25", "\uDD25"], /* Low Surrogate */
  ["\uDD26", "\uDD26"], /* Low Surrogate */
  ["\uDD27", "\uDD27"], /* Low Surrogate */
  ["\uDD28", "\uDD28"], /* Low Surrogate */
  ["\uDD29", "\uDD29"], /* Low Surrogate */
  ["\uDD2A", "\uDD2A"], /* Low Surrogate */
  ["\uDD2B", "\uDD2B"], /* Low Surrogate */
  ["\uDD2C", "\uDD2C"], /* Low Surrogate */
  ["\uDD2D", "\uDD2D"], /* Low Surrogate */
  ["\uDD2E", "\uDD2E"], /* Low Surrogate */
  ["\uDD2F", "\uDD2F"], /* Low Surrogate */
  ["\uDD30", "\uDD30"], /* Low Surrogate */
  ["\uDD31", "\uDD31"], /* Low Surrogate */
  ["\uDD32", "\uDD32"], /* Low Surrogate */
  ["\uDD33", "\uDD33"], /* Low Surrogate */
  ["\uDD34", "\uDD34"], /* Low Surrogate */
  ["\uDD35", "\uDD35"], /* Low Surrogate */
  ["\uDD36", "\uDD36"], /* Low Surrogate */
  ["\uDD37", "\uDD37"], /* Low Surrogate */
  ["\uDD38", "\uDD38"], /* Low Surrogate */
  ["\uDD39", "\uDD39"], /* Low Surrogate */
  ["\uDD3A", "\uDD3A"], /* Low Surrogate */
  ["\uDD3B", "\uDD3B"], /* Low Surrogate */
  ["\uDD3C", "\uDD3C"], /* Low Surrogate */
  ["\uDD3D", "\uDD3D"], /* Low Surrogate */
  ["\uDD3E", "\uDD3E"], /* Low Surrogate */
  ["\uDD3F", "\uDD3F"], /* Low Surrogate */
  ["\uDD40", "\uDD40"], /* Low Surrogate */
  ["\uDD41", "\uDD41"], /* Low Surrogate */
  ["\uDD42", "\uDD42"], /* Low Surrogate */
  ["\uDD43", "\uDD43"], /* Low Surrogate */
  ["\uDD44", "\uDD44"], /* Low Surrogate */
  ["\uDD45", "\uDD45"], /* Low Surrogate */
  ["\uDD46", "\uDD46"], /* Low Surrogate */
  ["\uDD47", "\uDD47"], /* Low Surrogate */
  ["\uDD48", "\uDD48"], /* Low Surrogate */
  ["\uDD49", "\uDD49"], /* Low Surrogate */
  ["\uDD4A", "\uDD4A"], /* Low Surrogate */
  ["\uDD4B", "\uDD4B"], /* Low Surrogate */
  ["\uDD4C", "\uDD4C"], /* Low Surrogate */
  ["\uDD4D", "\uDD4D"], /* Low Surrogate */
  ["\uDD4E", "\uDD4E"], /* Low Surrogate */
  ["\uDD4F", "\uDD4F"], /* Low Surrogate */
  ["\uDD50", "\uDD50"], /* Low Surrogate */
  ["\uDD51", "\uDD51"], /* Low Surrogate */
  ["\uDD52", "\uDD52"], /* Low Surrogate */
  ["\uDD53", "\uDD53"], /* Low Surrogate */
  ["\uDD54", "\uDD54"], /* Low Surrogate */
  ["\uDD55", "\uDD55"], /* Low Surrogate */
  ["\uDD56", "\uDD56"], /* Low Surrogate */
  ["\uDD57", "\uDD57"], /* Low Surrogate */
  ["\uDD58", "\uDD58"], /* Low Surrogate */
  ["\uDD59", "\uDD59"], /* Low Surrogate */
  ["\uDD5A", "\uDD5A"], /* Low Surrogate */
  ["\uDD5B", "\uDD5B"], /* Low Surrogate */
  ["\uDD5C", "\uDD5C"], /* Low Surrogate */
  ["\uDD5D", "\uDD5D"], /* Low Surrogate */
  ["\uDD5E", "\uDD5E"], /* Low Surrogate */
  ["\uDD5F", "\uDD5F"], /* Low Surrogate */
  ["\uDD60", "\uDD60"], /* Low Surrogate */
  ["\uDD61", "\uDD61"], /* Low Surrogate */
  ["\uDD62", "\uDD62"], /* Low Surrogate */
  ["\uDD63", "\uDD63"], /* Low Surrogate */
  ["\uDD64", "\uDD64"], /* Low Surrogate */
  ["\uDD65", "\uDD65"], /* Low Surrogate */
  ["\uDD66", "\uDD66"], /* Low Surrogate */
  ["\uDD67", "\uDD67"], /* Low Surrogate */
  ["\uDD68", "\uDD68"], /* Low Surrogate */
  ["\uDD69", "\uDD69"], /* Low Surrogate */
  ["\uDD6A", "\uDD6A"], /* Low Surrogate */
  ["\uDD6B", "\uDD6B"], /* Low Surrogate */
  ["\uDD6C", "\uDD6C"], /* Low Surrogate */
  ["\uDD6D", "\uDD6D"], /* Low Surrogate */
  ["\uDD6E", "\uDD6E"], /* Low Surrogate */
  ["\uDD6F", "\uDD6F"], /* Low Surrogate */
  ["\uDD70", "\uDD70"], /* Low Surrogate */
  ["\uDD71", "\uDD71"], /* Low Surrogate */
  ["\uDD72", "\uDD72"], /* Low Surrogate */
  ["\uDD73", "\uDD73"], /* Low Surrogate */
  ["\uDD74", "\uDD74"], /* Low Surrogate */
  ["\uDD75", "\uDD75"], /* Low Surrogate */
  ["\uDD76", "\uDD76"], /* Low Surrogate */
  ["\uDD77", "\uDD77"], /* Low Surrogate */
  ["\uDD78", "\uDD78"], /* Low Surrogate */
  ["\uDD79", "\uDD79"], /* Low Surrogate */
  ["\uDD7A", "\uDD7A"], /* Low Surrogate */
  ["\uDD7B", "\uDD7B"], /* Low Surrogate */
  ["\uDD7C", "\uDD7C"], /* Low Surrogate */
  ["\uDD7D", "\uDD7D"], /* Low Surrogate */
  ["\uDD7E", "\uDD7E"], /* Low Surrogate */
  ["\uDD7F", "\uDD7F"], /* Low Surrogate */
  ["\uDD80", "\uDD80"], /* Low Surrogate */
  ["\uDD81", "\uDD81"], /* Low Surrogate */
  ["\uDD82", "\uDD82"], /* Low Surrogate */
  ["\uDD83", "\uDD83"], /* Low Surrogate */
  ["\uDD84", "\uDD84"], /* Low Surrogate */
  ["\uDD85", "\uDD85"], /* Low Surrogate */
  ["\uDD86", "\uDD86"], /* Low Surrogate */
  ["\uDD87", "\uDD87"], /* Low Surrogate */
  ["\uDD88", "\uDD88"], /* Low Surrogate */
  ["\uDD89", "\uDD89"], /* Low Surrogate */
  ["\uDD8A", "\uDD8A"], /* Low Surrogate */
  ["\uDD8B", "\uDD8B"], /* Low Surrogate */
  ["\uDD8C", "\uDD8C"], /* Low Surrogate */
  ["\uDD8D", "\uDD8D"], /* Low Surrogate */
  ["\uDD8E", "\uDD8E"], /* Low Surrogate */
  ["\uDD8F", "\uDD8F"], /* Low Surrogate */
  ["\uDD90", "\uDD90"], /* Low Surrogate */
  ["\uDD91", "\uDD91"], /* Low Surrogate */
  ["\uDD92", "\uDD92"], /* Low Surrogate */
  ["\uDD93", "\uDD93"], /* Low Surrogate */
  ["\uDD94", "\uDD94"], /* Low Surrogate */
  ["\uDD95", "\uDD95"], /* Low Surrogate */
  ["\uDD96", "\uDD96"], /* Low Surrogate */
  ["\uDD97", "\uDD97"], /* Low Surrogate */
  ["\uDD98", "\uDD98"], /* Low Surrogate */
  ["\uDD99", "\uDD99"], /* Low Surrogate */
  ["\uDD9A", "\uDD9A"], /* Low Surrogate */
  ["\uDD9B", "\uDD9B"], /* Low Surrogate */
  ["\uDD9C", "\uDD9C"], /* Low Surrogate */
  ["\uDD9D", "\uDD9D"], /* Low Surrogate */
  ["\uDD9E", "\uDD9E"], /* Low Surrogate */
  ["\uDD9F", "\uDD9F"], /* Low Surrogate */
  ["\uDDA0", "\uDDA0"], /* Low Surrogate */
  ["\uDDA1", "\uDDA1"], /* Low Surrogate */
  ["\uDDA2", "\uDDA2"], /* Low Surrogate */
  ["\uDDA3", "\uDDA3"], /* Low Surrogate */
  ["\uDDA4", "\uDDA4"], /* Low Surrogate */
  ["\uDDA5", "\uDDA5"], /* Low Surrogate */
  ["\uDDA6", "\uDDA6"], /* Low Surrogate */
  ["\uDDA7", "\uDDA7"], /* Low Surrogate */
  ["\uDDA8", "\uDDA8"], /* Low Surrogate */
  ["\uDDA9", "\uDDA9"], /* Low Surrogate */
  ["\uDDAA", "\uDDAA"], /* Low Surrogate */
  ["\uDDAB", "\uDDAB"], /* Low Surrogate */
  ["\uDDAC", "\uDDAC"], /* Low Surrogate */
  ["\uDDAD", "\uDDAD"], /* Low Surrogate */
  ["\uDDAE", "\uDDAE"], /* Low Surrogate */
  ["\uDDAF", "\uDDAF"], /* Low Surrogate */
  ["\uDDB0", "\uDDB0"], /* Low Surrogate */
  ["\uDDB1", "\uDDB1"], /* Low Surrogate */
  ["\uDDB2", "\uDDB2"], /* Low Surrogate */
  ["\uDDB3", "\uDDB3"], /* Low Surrogate */
  ["\uDDB4", "\uDDB4"], /* Low Surrogate */
  ["\uDDB5", "\uDDB5"], /* Low Surrogate */
  ["\uDDB6", "\uDDB6"], /* Low Surrogate */
  ["\uDDB7", "\uDDB7"], /* Low Surrogate */
  ["\uDDB8", "\uDDB8"], /* Low Surrogate */
  ["\uDDB9", "\uDDB9"], /* Low Surrogate */
  ["\uDDBA", "\uDDBA"], /* Low Surrogate */
  ["\uDDBB", "\uDDBB"], /* Low Surrogate */
  ["\uDDBC", "\uDDBC"], /* Low Surrogate */
  ["\uDDBD", "\uDDBD"], /* Low Surrogate */
  ["\uDDBE", "\uDDBE"], /* Low Surrogate */
  ["\uDDBF", "\uDDBF"], /* Low Surrogate */
  ["\uDDC0", "\uDDC0"], /* Low Surrogate */
  ["\uDDC1", "\uDDC1"], /* Low Surrogate */
  ["\uDDC2", "\uDDC2"], /* Low Surrogate */
  ["\uDDC3", "\uDDC3"], /* Low Surrogate */
  ["\uDDC4", "\uDDC4"], /* Low Surrogate */
  ["\uDDC5", "\uDDC5"], /* Low Surrogate */
  ["\uDDC6", "\uDDC6"], /* Low Surrogate */
  ["\uDDC7", "\uDDC7"], /* Low Surrogate */
  ["\uDDC8", "\uDDC8"], /* Low Surrogate */
  ["\uDDC9", "\uDDC9"], /* Low Surrogate */
  ["\uDDCA", "\uDDCA"], /* Low Surrogate */
  ["\uDDCB", "\uDDCB"], /* Low Surrogate */
  ["\uDDCC", "\uDDCC"], /* Low Surrogate */
  ["\uDDCD", "\uDDCD"], /* Low Surrogate */
  ["\uDDCE", "\uDDCE"], /* Low Surrogate */
  ["\uDDCF", "\uDDCF"], /* Low Surrogate */
  ["\uDDD0", "\uDDD0"], /* Low Surrogate */
  ["\uDDD1", "\uDDD1"], /* Low Surrogate */
  ["\uDDD2", "\uDDD2"], /* Low Surrogate */
  ["\uDDD3", "\uDDD3"], /* Low Surrogate */
  ["\uDDD4", "\uDDD4"], /* Low Surrogate */
  ["\uDDD5", "\uDDD5"], /* Low Surrogate */
  ["\uDDD6", "\uDDD6"], /* Low Surrogate */
  ["\uDDD7", "\uDDD7"], /* Low Surrogate */
  ["\uDDD8", "\uDDD8"], /* Low Surrogate */
  ["\uDDD9", "\uDDD9"], /* Low Surrogate */
  ["\uDDDA", "\uDDDA"], /* Low Surrogate */
  ["\uDDDB", "\uDDDB"], /* Low Surrogate */
  ["\uDDDC", "\uDDDC"], /* Low Surrogate */
  ["\uDDDD", "\uDDDD"], /* Low Surrogate */
  ["\uDDDE", "\uDDDE"], /* Low Surrogate */
  ["\uDDDF", "\uDDDF"], /* Low Surrogate */
  ["\uDDE0", "\uDDE0"], /* Low Surrogate */
  ["\uDDE1", "\uDDE1"], /* Low Surrogate */
  ["\uDDE2", "\uDDE2"], /* Low Surrogate */
  ["\uDDE3", "\uDDE3"], /* Low Surrogate */
  ["\uDDE4", "\uDDE4"], /* Low Surrogate */
  ["\uDDE5", "\uDDE5"], /* Low Surrogate */
  ["\uDDE6", "\uDDE6"], /* Low Surrogate */
  ["\uDDE7", "\uDDE7"], /* Low Surrogate */
  ["\uDDE8", "\uDDE8"], /* Low Surrogate */
  ["\uDDE9", "\uDDE9"], /* Low Surrogate */
  ["\uDDEA", "\uDDEA"], /* Low Surrogate */
  ["\uDDEB", "\uDDEB"], /* Low Surrogate */
  ["\uDDEC", "\uDDEC"], /* Low Surrogate */
  ["\uDDED", "\uDDED"], /* Low Surrogate */
  ["\uDDEE", "\uDDEE"], /* Low Surrogate */
  ["\uDDEF", "\uDDEF"], /* Low Surrogate */
  ["\uDDF0", "\uDDF0"], /* Low Surrogate */
  ["\uDDF1", "\uDDF1"], /* Low Surrogate */
  ["\uDDF2", "\uDDF2"], /* Low Surrogate */
  ["\uDDF3", "\uDDF3"], /* Low Surrogate */
  ["\uDDF4", "\uDDF4"], /* Low Surrogate */
  ["\uDDF5", "\uDDF5"], /* Low Surrogate */
  ["\uDDF6", "\uDDF6"], /* Low Surrogate */
  ["\uDDF7", "\uDDF7"], /* Low Surrogate */
  ["\uDDF8", "\uDDF8"], /* Low Surrogate */
  ["\uDDF9", "\uDDF9"], /* Low Surrogate */
  ["\uDDFA", "\uDDFA"], /* Low Surrogate */
  ["\uDDFB", "\uDDFB"], /* Low Surrogate */
  ["\uDDFC", "\uDDFC"], /* Low Surrogate */
  ["\uDDFD", "\uDDFD"], /* Low Surrogate */
  ["\uDDFE", "\uDDFE"], /* Low Surrogate */
  ["\uDDFF", "\uDDFF"], /* Low Surrogate */
  ["\uDE00", "\uDE00"], /* Low Surrogate */
  ["\uDE01", "\uDE01"], /* Low Surrogate */
  ["\uDE02", "\uDE02"], /* Low Surrogate */
  ["\uDE03", "\uDE03"], /* Low Surrogate */
  ["\uDE04", "\uDE04"], /* Low Surrogate */
  ["\uDE05", "\uDE05"], /* Low Surrogate */
  ["\uDE06", "\uDE06"], /* Low Surrogate */
  ["\uDE07", "\uDE07"], /* Low Surrogate */
  ["\uDE08", "\uDE08"], /* Low Surrogate */
  ["\uDE09", "\uDE09"], /* Low Surrogate */
  ["\uDE0A", "\uDE0A"], /* Low Surrogate */
  ["\uDE0B", "\uDE0B"], /* Low Surrogate */
  ["\uDE0C", "\uDE0C"], /* Low Surrogate */
  ["\uDE0D", "\uDE0D"], /* Low Surrogate */
  ["\uDE0E", "\uDE0E"], /* Low Surrogate */
  ["\uDE0F", "\uDE0F"], /* Low Surrogate */
  ["\uDE10", "\uDE10"], /* Low Surrogate */
  ["\uDE11", "\uDE11"], /* Low Surrogate */
  ["\uDE12", "\uDE12"], /* Low Surrogate */
  ["\uDE13", "\uDE13"], /* Low Surrogate */
  ["\uDE14", "\uDE14"], /* Low Surrogate */
  ["\uDE15", "\uDE15"], /* Low Surrogate */
  ["\uDE16", "\uDE16"], /* Low Surrogate */
  ["\uDE17", "\uDE17"], /* Low Surrogate */
  ["\uDE18", "\uDE18"], /* Low Surrogate */
  ["\uDE19", "\uDE19"], /* Low Surrogate */
  ["\uDE1A", "\uDE1A"], /* Low Surrogate */
  ["\uDE1B", "\uDE1B"], /* Low Surrogate */
  ["\uDE1C", "\uDE1C"], /* Low Surrogate */
  ["\uDE1D", "\uDE1D"], /* Low Surrogate */
  ["\uDE1E", "\uDE1E"], /* Low Surrogate */
  ["\uDE1F", "\uDE1F"], /* Low Surrogate */
  ["\uDE20", "\uDE20"], /* Low Surrogate */
  ["\uDE21", "\uDE21"], /* Low Surrogate */
  ["\uDE22", "\uDE22"], /* Low Surrogate */
  ["\uDE23", "\uDE23"], /* Low Surrogate */
  ["\uDE24", "\uDE24"], /* Low Surrogate */
  ["\uDE25", "\uDE25"], /* Low Surrogate */
  ["\uDE26", "\uDE26"], /* Low Surrogate */
  ["\uDE27", "\uDE27"], /* Low Surrogate */
  ["\uDE28", "\uDE28"], /* Low Surrogate */
  ["\uDE29", "\uDE29"], /* Low Surrogate */
  ["\uDE2A", "\uDE2A"], /* Low Surrogate */
  ["\uDE2B", "\uDE2B"], /* Low Surrogate */
  ["\uDE2C", "\uDE2C"], /* Low Surrogate */
  ["\uDE2D", "\uDE2D"], /* Low Surrogate */
  ["\uDE2E", "\uDE2E"], /* Low Surrogate */
  ["\uDE2F", "\uDE2F"], /* Low Surrogate */
  ["\uDE30", "\uDE30"], /* Low Surrogate */
  ["\uDE31", "\uDE31"], /* Low Surrogate */
  ["\uDE32", "\uDE32"], /* Low Surrogate */
  ["\uDE33", "\uDE33"], /* Low Surrogate */
  ["\uDE34", "\uDE34"], /* Low Surrogate */
  ["\uDE35", "\uDE35"], /* Low Surrogate */
  ["\uDE36", "\uDE36"], /* Low Surrogate */
  ["\uDE37", "\uDE37"], /* Low Surrogate */
  ["\uDE38", "\uDE38"], /* Low Surrogate */
  ["\uDE39", "\uDE39"], /* Low Surrogate */
  ["\uDE3A", "\uDE3A"], /* Low Surrogate */
  ["\uDE3B", "\uDE3B"], /* Low Surrogate */
  ["\uDE3C", "\uDE3C"], /* Low Surrogate */
  ["\uDE3D", "\uDE3D"], /* Low Surrogate */
  ["\uDE3E", "\uDE3E"], /* Low Surrogate */
  ["\uDE3F", "\uDE3F"], /* Low Surrogate */
  ["\uDE40", "\uDE40"], /* Low Surrogate */
  ["\uDE41", "\uDE41"], /* Low Surrogate */
  ["\uDE42", "\uDE42"], /* Low Surrogate */
  ["\uDE43", "\uDE43"], /* Low Surrogate */
  ["\uDE44", "\uDE44"], /* Low Surrogate */
  ["\uDE45", "\uDE45"], /* Low Surrogate */
  ["\uDE46", "\uDE46"], /* Low Surrogate */
  ["\uDE47", "\uDE47"], /* Low Surrogate */
  ["\uDE48", "\uDE48"], /* Low Surrogate */
  ["\uDE49", "\uDE49"], /* Low Surrogate */
  ["\uDE4A", "\uDE4A"], /* Low Surrogate */
  ["\uDE4B", "\uDE4B"], /* Low Surrogate */
  ["\uDE4C", "\uDE4C"], /* Low Surrogate */
  ["\uDE4D", "\uDE4D"], /* Low Surrogate */
  ["\uDE4E", "\uDE4E"], /* Low Surrogate */
  ["\uDE4F", "\uDE4F"], /* Low Surrogate */
  ["\uDE50", "\uDE50"], /* Low Surrogate */
  ["\uDE51", "\uDE51"], /* Low Surrogate */
  ["\uDE52", "\uDE52"], /* Low Surrogate */
  ["\uDE53", "\uDE53"], /* Low Surrogate */
  ["\uDE54", "\uDE54"], /* Low Surrogate */
  ["\uDE55", "\uDE55"], /* Low Surrogate */
  ["\uDE56", "\uDE56"], /* Low Surrogate */
  ["\uDE57", "\uDE57"], /* Low Surrogate */
  ["\uDE58", "\uDE58"], /* Low Surrogate */
  ["\uDE59", "\uDE59"], /* Low Surrogate */
  ["\uDE5A", "\uDE5A"], /* Low Surrogate */
  ["\uDE5B", "\uDE5B"], /* Low Surrogate */
  ["\uDE5C", "\uDE5C"], /* Low Surrogate */
  ["\uDE5D", "\uDE5D"], /* Low Surrogate */
  ["\uDE5E", "\uDE5E"], /* Low Surrogate */
  ["\uDE5F", "\uDE5F"], /* Low Surrogate */
  ["\uDE60", "\uDE60"], /* Low Surrogate */
  ["\uDE61", "\uDE61"], /* Low Surrogate */
  ["\uDE62", "\uDE62"], /* Low Surrogate */
  ["\uDE63", "\uDE63"], /* Low Surrogate */
  ["\uDE64", "\uDE64"], /* Low Surrogate */
  ["\uDE65", "\uDE65"], /* Low Surrogate */
  ["\uDE66", "\uDE66"], /* Low Surrogate */
  ["\uDE67", "\uDE67"], /* Low Surrogate */
  ["\uDE68", "\uDE68"], /* Low Surrogate */
  ["\uDE69", "\uDE69"], /* Low Surrogate */
  ["\uDE6A", "\uDE6A"], /* Low Surrogate */
  ["\uDE6B", "\uDE6B"], /* Low Surrogate */
  ["\uDE6C", "\uDE6C"], /* Low Surrogate */
  ["\uDE6D", "\uDE6D"], /* Low Surrogate */
  ["\uDE6E", "\uDE6E"], /* Low Surrogate */
  ["\uDE6F", "\uDE6F"], /* Low Surrogate */
  ["\uDE70", "\uDE70"], /* Low Surrogate */
  ["\uDE71", "\uDE71"], /* Low Surrogate */
  ["\uDE72", "\uDE72"], /* Low Surrogate */
  ["\uDE73", "\uDE73"], /* Low Surrogate */
  ["\uDE74", "\uDE74"], /* Low Surrogate */
  ["\uDE75", "\uDE75"], /* Low Surrogate */
  ["\uDE76", "\uDE76"], /* Low Surrogate */
  ["\uDE77", "\uDE77"], /* Low Surrogate */
  ["\uDE78", "\uDE78"], /* Low Surrogate */
  ["\uDE79", "\uDE79"], /* Low Surrogate */
  ["\uDE7A", "\uDE7A"], /* Low Surrogate */
  ["\uDE7B", "\uDE7B"], /* Low Surrogate */
  ["\uDE7C", "\uDE7C"], /* Low Surrogate */
  ["\uDE7D", "\uDE7D"], /* Low Surrogate */
  ["\uDE7E", "\uDE7E"], /* Low Surrogate */
  ["\uDE7F", "\uDE7F"], /* Low Surrogate */
  ["\uDE80", "\uDE80"], /* Low Surrogate */
  ["\uDE81", "\uDE81"], /* Low Surrogate */
  ["\uDE82", "\uDE82"], /* Low Surrogate */
  ["\uDE83", "\uDE83"], /* Low Surrogate */
  ["\uDE84", "\uDE84"], /* Low Surrogate */
  ["\uDE85", "\uDE85"], /* Low Surrogate */
  ["\uDE86", "\uDE86"], /* Low Surrogate */
  ["\uDE87", "\uDE87"], /* Low Surrogate */
  ["\uDE88", "\uDE88"], /* Low Surrogate */
  ["\uDE89", "\uDE89"], /* Low Surrogate */
  ["\uDE8A", "\uDE8A"], /* Low Surrogate */
  ["\uDE8B", "\uDE8B"], /* Low Surrogate */
  ["\uDE8C", "\uDE8C"], /* Low Surrogate */
  ["\uDE8D", "\uDE8D"], /* Low Surrogate */
  ["\uDE8E", "\uDE8E"], /* Low Surrogate */
  ["\uDE8F", "\uDE8F"], /* Low Surrogate */
  ["\uDE90", "\uDE90"], /* Low Surrogate */
  ["\uDE91", "\uDE91"], /* Low Surrogate */
  ["\uDE92", "\uDE92"], /* Low Surrogate */
  ["\uDE93", "\uDE93"], /* Low Surrogate */
  ["\uDE94", "\uDE94"], /* Low Surrogate */
  ["\uDE95", "\uDE95"], /* Low Surrogate */
  ["\uDE96", "\uDE96"], /* Low Surrogate */
  ["\uDE97", "\uDE97"], /* Low Surrogate */
  ["\uDE98", "\uDE98"], /* Low Surrogate */
  ["\uDE99", "\uDE99"], /* Low Surrogate */
  ["\uDE9A", "\uDE9A"], /* Low Surrogate */
  ["\uDE9B", "\uDE9B"], /* Low Surrogate */
  ["\uDE9C", "\uDE9C"], /* Low Surrogate */
  ["\uDE9D", "\uDE9D"], /* Low Surrogate */
  ["\uDE9E", "\uDE9E"], /* Low Surrogate */
  ["\uDE9F", "\uDE9F"], /* Low Surrogate */
  ["\uDEA0", "\uDEA0"], /* Low Surrogate */
  ["\uDEA1", "\uDEA1"], /* Low Surrogate */
  ["\uDEA2", "\uDEA2"], /* Low Surrogate */
  ["\uDEA3", "\uDEA3"], /* Low Surrogate */
  ["\uDEA4", "\uDEA4"], /* Low Surrogate */
  ["\uDEA5", "\uDEA5"], /* Low Surrogate */
  ["\uDEA6", "\uDEA6"], /* Low Surrogate */
  ["\uDEA7", "\uDEA7"], /* Low Surrogate */
  ["\uDEA8", "\uDEA8"], /* Low Surrogate */
  ["\uDEA9", "\uDEA9"], /* Low Surrogate */
  ["\uDEAA", "\uDEAA"], /* Low Surrogate */
  ["\uDEAB", "\uDEAB"], /* Low Surrogate */
  ["\uDEAC", "\uDEAC"], /* Low Surrogate */
  ["\uDEAD", "\uDEAD"], /* Low Surrogate */
  ["\uDEAE", "\uDEAE"], /* Low Surrogate */
  ["\uDEAF", "\uDEAF"], /* Low Surrogate */
  ["\uDEB0", "\uDEB0"], /* Low Surrogate */
  ["\uDEB1", "\uDEB1"], /* Low Surrogate */
  ["\uDEB2", "\uDEB2"], /* Low Surrogate */
  ["\uDEB3", "\uDEB3"], /* Low Surrogate */
  ["\uDEB4", "\uDEB4"], /* Low Surrogate */
  ["\uDEB5", "\uDEB5"], /* Low Surrogate */
  ["\uDEB6", "\uDEB6"], /* Low Surrogate */
  ["\uDEB7", "\uDEB7"], /* Low Surrogate */
  ["\uDEB8", "\uDEB8"], /* Low Surrogate */
  ["\uDEB9", "\uDEB9"], /* Low Surrogate */
  ["\uDEBA", "\uDEBA"], /* Low Surrogate */
  ["\uDEBB", "\uDEBB"], /* Low Surrogate */
  ["\uDEBC", "\uDEBC"], /* Low Surrogate */
  ["\uDEBD", "\uDEBD"], /* Low Surrogate */
  ["\uDEBE", "\uDEBE"], /* Low Surrogate */
  ["\uDEBF", "\uDEBF"], /* Low Surrogate */
  ["\uDEC0", "\uDEC0"], /* Low Surrogate */
  ["\uDEC1", "\uDEC1"], /* Low Surrogate */
  ["\uDEC2", "\uDEC2"], /* Low Surrogate */
  ["\uDEC3", "\uDEC3"], /* Low Surrogate */
  ["\uDEC4", "\uDEC4"], /* Low Surrogate */
  ["\uDEC5", "\uDEC5"], /* Low Surrogate */
  ["\uDEC6", "\uDEC6"], /* Low Surrogate */
  ["\uDEC7", "\uDEC7"], /* Low Surrogate */
  ["\uDEC8", "\uDEC8"], /* Low Surrogate */
  ["\uDEC9", "\uDEC9"], /* Low Surrogate */
  ["\uDECA", "\uDECA"], /* Low Surrogate */
  ["\uDECB", "\uDECB"], /* Low Surrogate */
  ["\uDECC", "\uDECC"], /* Low Surrogate */
  ["\uDECD", "\uDECD"], /* Low Surrogate */
  ["\uDECE", "\uDECE"], /* Low Surrogate */
  ["\uDECF", "\uDECF"], /* Low Surrogate */
  ["\uDED0", "\uDED0"], /* Low Surrogate */
  ["\uDED1", "\uDED1"], /* Low Surrogate */
  ["\uDED2", "\uDED2"], /* Low Surrogate */
  ["\uDED3", "\uDED3"], /* Low Surrogate */
  ["\uDED4", "\uDED4"], /* Low Surrogate */
  ["\uDED5", "\uDED5"], /* Low Surrogate */
  ["\uDED6", "\uDED6"], /* Low Surrogate */
  ["\uDED7", "\uDED7"], /* Low Surrogate */
  ["\uDED8", "\uDED8"], /* Low Surrogate */
  ["\uDED9", "\uDED9"], /* Low Surrogate */
  ["\uDEDA", "\uDEDA"], /* Low Surrogate */
  ["\uDEDB", "\uDEDB"], /* Low Surrogate */
  ["\uDEDC", "\uDEDC"], /* Low Surrogate */
  ["\uDEDD", "\uDEDD"], /* Low Surrogate */
  ["\uDEDE", "\uDEDE"], /* Low Surrogate */
  ["\uDEDF", "\uDEDF"], /* Low Surrogate */
  ["\uDEE0", "\uDEE0"], /* Low Surrogate */
  ["\uDEE1", "\uDEE1"], /* Low Surrogate */
  ["\uDEE2", "\uDEE2"], /* Low Surrogate */
  ["\uDEE3", "\uDEE3"], /* Low Surrogate */
  ["\uDEE4", "\uDEE4"], /* Low Surrogate */
  ["\uDEE5", "\uDEE5"], /* Low Surrogate */
  ["\uDEE6", "\uDEE6"], /* Low Surrogate */
  ["\uDEE7", "\uDEE7"], /* Low Surrogate */
  ["\uDEE8", "\uDEE8"], /* Low Surrogate */
  ["\uDEE9", "\uDEE9"], /* Low Surrogate */
  ["\uDEEA", "\uDEEA"], /* Low Surrogate */
  ["\uDEEB", "\uDEEB"], /* Low Surrogate */
  ["\uDEEC", "\uDEEC"], /* Low Surrogate */
  ["\uDEED", "\uDEED"], /* Low Surrogate */
  ["\uDEEE", "\uDEEE"], /* Low Surrogate */
  ["\uDEEF", "\uDEEF"], /* Low Surrogate */
  ["\uDEF0", "\uDEF0"], /* Low Surrogate */
  ["\uDEF1", "\uDEF1"], /* Low Surrogate */
  ["\uDEF2", "\uDEF2"], /* Low Surrogate */
  ["\uDEF3", "\uDEF3"], /* Low Surrogate */
  ["\uDEF4", "\uDEF4"], /* Low Surrogate */
  ["\uDEF5", "\uDEF5"], /* Low Surrogate */
  ["\uDEF6", "\uDEF6"], /* Low Surrogate */
  ["\uDEF7", "\uDEF7"], /* Low Surrogate */
  ["\uDEF8", "\uDEF8"], /* Low Surrogate */
  ["\uDEF9", "\uDEF9"], /* Low Surrogate */
  ["\uDEFA", "\uDEFA"], /* Low Surrogate */
  ["\uDEFB", "\uDEFB"], /* Low Surrogate */
  ["\uDEFC", "\uDEFC"], /* Low Surrogate */
  ["\uDEFD", "\uDEFD"], /* Low Surrogate */
  ["\uDEFE", "\uDEFE"], /* Low Surrogate */
  ["\uDEFF", "\uDEFF"], /* Low Surrogate */
  ["\uDF00", "\uDF00"], /* Low Surrogate */
  ["\uDF01", "\uDF01"], /* Low Surrogate */
  ["\uDF02", "\uDF02"], /* Low Surrogate */
  ["\uDF03", "\uDF03"], /* Low Surrogate */
  ["\uDF04", "\uDF04"], /* Low Surrogate */
  ["\uDF05", "\uDF05"], /* Low Surrogate */
  ["\uDF06", "\uDF06"], /* Low Surrogate */
  ["\uDF07", "\uDF07"], /* Low Surrogate */
  ["\uDF08", "\uDF08"], /* Low Surrogate */
  ["\uDF09", "\uDF09"], /* Low Surrogate */
  ["\uDF0A", "\uDF0A"], /* Low Surrogate */
  ["\uDF0B", "\uDF0B"], /* Low Surrogate */
  ["\uDF0C", "\uDF0C"], /* Low Surrogate */
  ["\uDF0D", "\uDF0D"], /* Low Surrogate */
  ["\uDF0E", "\uDF0E"], /* Low Surrogate */
  ["\uDF0F", "\uDF0F"], /* Low Surrogate */
  ["\uDF10", "\uDF10"], /* Low Surrogate */
  ["\uDF11", "\uDF11"], /* Low Surrogate */
  ["\uDF12", "\uDF12"], /* Low Surrogate */
  ["\uDF13", "\uDF13"], /* Low Surrogate */
  ["\uDF14", "\uDF14"], /* Low Surrogate */
  ["\uDF15", "\uDF15"], /* Low Surrogate */
  ["\uDF16", "\uDF16"], /* Low Surrogate */
  ["\uDF17", "\uDF17"], /* Low Surrogate */
  ["\uDF18", "\uDF18"], /* Low Surrogate */
  ["\uDF19", "\uDF19"], /* Low Surrogate */
  ["\uDF1A", "\uDF1A"], /* Low Surrogate */
  ["\uDF1B", "\uDF1B"], /* Low Surrogate */
  ["\uDF1C", "\uDF1C"], /* Low Surrogate */
  ["\uDF1D", "\uDF1D"], /* Low Surrogate */
  ["\uDF1E", "\uDF1E"], /* Low Surrogate */
  ["\uDF1F", "\uDF1F"], /* Low Surrogate */
  ["\uDF20", "\uDF20"], /* Low Surrogate */
  ["\uDF21", "\uDF21"], /* Low Surrogate */
  ["\uDF22", "\uDF22"], /* Low Surrogate */
  ["\uDF23", "\uDF23"], /* Low Surrogate */
  ["\uDF24", "\uDF24"], /* Low Surrogate */
  ["\uDF25", "\uDF25"], /* Low Surrogate */
  ["\uDF26", "\uDF26"], /* Low Surrogate */
  ["\uDF27", "\uDF27"], /* Low Surrogate */
  ["\uDF28", "\uDF28"], /* Low Surrogate */
  ["\uDF29", "\uDF29"], /* Low Surrogate */
  ["\uDF2A", "\uDF2A"], /* Low Surrogate */
  ["\uDF2B", "\uDF2B"], /* Low Surrogate */
  ["\uDF2C", "\uDF2C"], /* Low Surrogate */
  ["\uDF2D", "\uDF2D"], /* Low Surrogate */
  ["\uDF2E", "\uDF2E"], /* Low Surrogate */
  ["\uDF2F", "\uDF2F"], /* Low Surrogate */
  ["\uDF30", "\uDF30"], /* Low Surrogate */
  ["\uDF31", "\uDF31"], /* Low Surrogate */
  ["\uDF32", "\uDF32"], /* Low Surrogate */
  ["\uDF33", "\uDF33"], /* Low Surrogate */
  ["\uDF34", "\uDF34"], /* Low Surrogate */
  ["\uDF35", "\uDF35"], /* Low Surrogate */
  ["\uDF36", "\uDF36"], /* Low Surrogate */
  ["\uDF37", "\uDF37"], /* Low Surrogate */
  ["\uDF38", "\uDF38"], /* Low Surrogate */
  ["\uDF39", "\uDF39"], /* Low Surrogate */
  ["\uDF3A", "\uDF3A"], /* Low Surrogate */
  ["\uDF3B", "\uDF3B"], /* Low Surrogate */
  ["\uDF3C", "\uDF3C"], /* Low Surrogate */
  ["\uDF3D", "\uDF3D"], /* Low Surrogate */
  ["\uDF3E", "\uDF3E"], /* Low Surrogate */
  ["\uDF3F", "\uDF3F"], /* Low Surrogate */
  ["\uDF40", "\uDF40"], /* Low Surrogate */
  ["\uDF41", "\uDF41"], /* Low Surrogate */
  ["\uDF42", "\uDF42"], /* Low Surrogate */
  ["\uDF43", "\uDF43"], /* Low Surrogate */
  ["\uDF44", "\uDF44"], /* Low Surrogate */
  ["\uDF45", "\uDF45"], /* Low Surrogate */
  ["\uDF46", "\uDF46"], /* Low Surrogate */
  ["\uDF47", "\uDF47"], /* Low Surrogate */
  ["\uDF48", "\uDF48"], /* Low Surrogate */
  ["\uDF49", "\uDF49"], /* Low Surrogate */
  ["\uDF4A", "\uDF4A"], /* Low Surrogate */
  ["\uDF4B", "\uDF4B"], /* Low Surrogate */
  ["\uDF4C", "\uDF4C"], /* Low Surrogate */
  ["\uDF4D", "\uDF4D"], /* Low Surrogate */
  ["\uDF4E", "\uDF4E"], /* Low Surrogate */
  ["\uDF4F", "\uDF4F"], /* Low Surrogate */
  ["\uDF50", "\uDF50"], /* Low Surrogate */
  ["\uDF51", "\uDF51"], /* Low Surrogate */
  ["\uDF52", "\uDF52"], /* Low Surrogate */
  ["\uDF53", "\uDF53"], /* Low Surrogate */
  ["\uDF54", "\uDF54"], /* Low Surrogate */
  ["\uDF55", "\uDF55"], /* Low Surrogate */
  ["\uDF56", "\uDF56"], /* Low Surrogate */
  ["\uDF57", "\uDF57"], /* Low Surrogate */
  ["\uDF58", "\uDF58"], /* Low Surrogate */
  ["\uDF59", "\uDF59"], /* Low Surrogate */
  ["\uDF5A", "\uDF5A"], /* Low Surrogate */
  ["\uDF5B", "\uDF5B"], /* Low Surrogate */
  ["\uDF5C", "\uDF5C"], /* Low Surrogate */
  ["\uDF5D", "\uDF5D"], /* Low Surrogate */
  ["\uDF5E", "\uDF5E"], /* Low Surrogate */
  ["\uDF5F", "\uDF5F"], /* Low Surrogate */
  ["\uDF60", "\uDF60"], /* Low Surrogate */
  ["\uDF61", "\uDF61"], /* Low Surrogate */
  ["\uDF62", "\uDF62"], /* Low Surrogate */
  ["\uDF63", "\uDF63"], /* Low Surrogate */
  ["\uDF64", "\uDF64"], /* Low Surrogate */
  ["\uDF65", "\uDF65"], /* Low Surrogate */
  ["\uDF66", "\uDF66"], /* Low Surrogate */
  ["\uDF67", "\uDF67"], /* Low Surrogate */
  ["\uDF68", "\uDF68"], /* Low Surrogate */
  ["\uDF69", "\uDF69"], /* Low Surrogate */
  ["\uDF6A", "\uDF6A"], /* Low Surrogate */
  ["\uDF6B", "\uDF6B"], /* Low Surrogate */
  ["\uDF6C", "\uDF6C"], /* Low Surrogate */
  ["\uDF6D", "\uDF6D"], /* Low Surrogate */
  ["\uDF6E", "\uDF6E"], /* Low Surrogate */
  ["\uDF6F", "\uDF6F"], /* Low Surrogate */
  ["\uDF70", "\uDF70"], /* Low Surrogate */
  ["\uDF71", "\uDF71"], /* Low Surrogate */
  ["\uDF72", "\uDF72"], /* Low Surrogate */
  ["\uDF73", "\uDF73"], /* Low Surrogate */
  ["\uDF74", "\uDF74"], /* Low Surrogate */
  ["\uDF75", "\uDF75"], /* Low Surrogate */
  ["\uDF76", "\uDF76"], /* Low Surrogate */
  ["\uDF77", "\uDF77"], /* Low Surrogate */
  ["\uDF78", "\uDF78"], /* Low Surrogate */
  ["\uDF79", "\uDF79"], /* Low Surrogate */
  ["\uDF7A", "\uDF7A"], /* Low Surrogate */
  ["\uDF7B", "\uDF7B"], /* Low Surrogate */
  ["\uDF7C", "\uDF7C"], /* Low Surrogate */
  ["\uDF7D", "\uDF7D"], /* Low Surrogate */
  ["\uDF7E", "\uDF7E"], /* Low Surrogate */
  ["\uDF7F", "\uDF7F"], /* Low Surrogate */
  ["\uDF80", "\uDF80"], /* Low Surrogate */
  ["\uDF81", "\uDF81"], /* Low Surrogate */
  ["\uDF82", "\uDF82"], /* Low Surrogate */
  ["\uDF83", "\uDF83"], /* Low Surrogate */
  ["\uDF84", "\uDF84"], /* Low Surrogate */
  ["\uDF85", "\uDF85"], /* Low Surrogate */
  ["\uDF86", "\uDF86"], /* Low Surrogate */
  ["\uDF87", "\uDF87"], /* Low Surrogate */
  ["\uDF88", "\uDF88"], /* Low Surrogate */
  ["\uDF89", "\uDF89"], /* Low Surrogate */
  ["\uDF8A", "\uDF8A"], /* Low Surrogate */
  ["\uDF8B", "\uDF8B"], /* Low Surrogate */
  ["\uDF8C", "\uDF8C"], /* Low Surrogate */
  ["\uDF8D", "\uDF8D"], /* Low Surrogate */
  ["\uDF8E", "\uDF8E"], /* Low Surrogate */
  ["\uDF8F", "\uDF8F"], /* Low Surrogate */
  ["\uDF90", "\uDF90"], /* Low Surrogate */
  ["\uDF91", "\uDF91"], /* Low Surrogate */
  ["\uDF92", "\uDF92"], /* Low Surrogate */
  ["\uDF93", "\uDF93"], /* Low Surrogate */
  ["\uDF94", "\uDF94"], /* Low Surrogate */
  ["\uDF95", "\uDF95"], /* Low Surrogate */
  ["\uDF96", "\uDF96"], /* Low Surrogate */
  ["\uDF97", "\uDF97"], /* Low Surrogate */
  ["\uDF98", "\uDF98"], /* Low Surrogate */
  ["\uDF99", "\uDF99"], /* Low Surrogate */
  ["\uDF9A", "\uDF9A"], /* Low Surrogate */
  ["\uDF9B", "\uDF9B"], /* Low Surrogate */
  ["\uDF9C", "\uDF9C"], /* Low Surrogate */
  ["\uDF9D", "\uDF9D"], /* Low Surrogate */
  ["\uDF9E", "\uDF9E"], /* Low Surrogate */
  ["\uDF9F", "\uDF9F"], /* Low Surrogate */
  ["\uDFA0", "\uDFA0"], /* Low Surrogate */
  ["\uDFA1", "\uDFA1"], /* Low Surrogate */
  ["\uDFA2", "\uDFA2"], /* Low Surrogate */
  ["\uDFA3", "\uDFA3"], /* Low Surrogate */
  ["\uDFA4", "\uDFA4"], /* Low Surrogate */
  ["\uDFA5", "\uDFA5"], /* Low Surrogate */
  ["\uDFA6", "\uDFA6"], /* Low Surrogate */
  ["\uDFA7", "\uDFA7"], /* Low Surrogate */
  ["\uDFA8", "\uDFA8"], /* Low Surrogate */
  ["\uDFA9", "\uDFA9"], /* Low Surrogate */
  ["\uDFAA", "\uDFAA"], /* Low Surrogate */
  ["\uDFAB", "\uDFAB"], /* Low Surrogate */
  ["\uDFAC", "\uDFAC"], /* Low Surrogate */
  ["\uDFAD", "\uDFAD"], /* Low Surrogate */
  ["\uDFAE", "\uDFAE"], /* Low Surrogate */
  ["\uDFAF", "\uDFAF"], /* Low Surrogate */
  ["\uDFB0", "\uDFB0"], /* Low Surrogate */
  ["\uDFB1", "\uDFB1"], /* Low Surrogate */
  ["\uDFB2", "\uDFB2"], /* Low Surrogate */
  ["\uDFB3", "\uDFB3"], /* Low Surrogate */
  ["\uDFB4", "\uDFB4"], /* Low Surrogate */
  ["\uDFB5", "\uDFB5"], /* Low Surrogate */
  ["\uDFB6", "\uDFB6"], /* Low Surrogate */
  ["\uDFB7", "\uDFB7"], /* Low Surrogate */
  ["\uDFB8", "\uDFB8"], /* Low Surrogate */
  ["\uDFB9", "\uDFB9"], /* Low Surrogate */
  ["\uDFBA", "\uDFBA"], /* Low Surrogate */
  ["\uDFBB", "\uDFBB"], /* Low Surrogate */
  ["\uDFBC", "\uDFBC"], /* Low Surrogate */
  ["\uDFBD", "\uDFBD"], /* Low Surrogate */
  ["\uDFBE", "\uDFBE"], /* Low Surrogate */
  ["\uDFBF", "\uDFBF"], /* Low Surrogate */
  ["\uDFC0", "\uDFC0"], /* Low Surrogate */
  ["\uDFC1", "\uDFC1"], /* Low Surrogate */
  ["\uDFC2", "\uDFC2"], /* Low Surrogate */
  ["\uDFC3", "\uDFC3"], /* Low Surrogate */
  ["\uDFC4", "\uDFC4"], /* Low Surrogate */
  ["\uDFC5", "\uDFC5"], /* Low Surrogate */
  ["\uDFC6", "\uDFC6"], /* Low Surrogate */
  ["\uDFC7", "\uDFC7"], /* Low Surrogate */
  ["\uDFC8", "\uDFC8"], /* Low Surrogate */
  ["\uDFC9", "\uDFC9"], /* Low Surrogate */
  ["\uDFCA", "\uDFCA"], /* Low Surrogate */
  ["\uDFCB", "\uDFCB"], /* Low Surrogate */
  ["\uDFCC", "\uDFCC"], /* Low Surrogate */
  ["\uDFCD", "\uDFCD"], /* Low Surrogate */
  ["\uDFCE", "\uDFCE"], /* Low Surrogate */
  ["\uDFCF", "\uDFCF"], /* Low Surrogate */
  ["\uDFD0", "\uDFD0"], /* Low Surrogate */
  ["\uDFD1", "\uDFD1"], /* Low Surrogate */
  ["\uDFD2", "\uDFD2"], /* Low Surrogate */
  ["\uDFD3", "\uDFD3"], /* Low Surrogate */
  ["\uDFD4", "\uDFD4"], /* Low Surrogate */
  ["\uDFD5", "\uDFD5"], /* Low Surrogate */
  ["\uDFD6", "\uDFD6"], /* Low Surrogate */
  ["\uDFD7", "\uDFD7"], /* Low Surrogate */
  ["\uDFD8", "\uDFD8"], /* Low Surrogate */
  ["\uDFD9", "\uDFD9"], /* Low Surrogate */
  ["\uDFDA", "\uDFDA"], /* Low Surrogate */
  ["\uDFDB", "\uDFDB"], /* Low Surrogate */
  ["\uDFDC", "\uDFDC"], /* Low Surrogate */
  ["\uDFDD", "\uDFDD"], /* Low Surrogate */
  ["\uDFDE", "\uDFDE"], /* Low Surrogate */
  ["\uDFDF", "\uDFDF"], /* Low Surrogate */
  ["\uDFE0", "\uDFE0"], /* Low Surrogate */
  ["\uDFE1", "\uDFE1"], /* Low Surrogate */
  ["\uDFE2", "\uDFE2"], /* Low Surrogate */
  ["\uDFE3", "\uDFE3"], /* Low Surrogate */
  ["\uDFE4", "\uDFE4"], /* Low Surrogate */
  ["\uDFE5", "\uDFE5"], /* Low Surrogate */
  ["\uDFE6", "\uDFE6"], /* Low Surrogate */
  ["\uDFE7", "\uDFE7"], /* Low Surrogate */
  ["\uDFE8", "\uDFE8"], /* Low Surrogate */
  ["\uDFE9", "\uDFE9"], /* Low Surrogate */
  ["\uDFEA", "\uDFEA"], /* Low Surrogate */
  ["\uDFEB", "\uDFEB"], /* Low Surrogate */
  ["\uDFEC", "\uDFEC"], /* Low Surrogate */
  ["\uDFED", "\uDFED"], /* Low Surrogate */
  ["\uDFEE", "\uDFEE"], /* Low Surrogate */
  ["\uDFEF", "\uDFEF"], /* Low Surrogate */
  ["\uDFF0", "\uDFF0"], /* Low Surrogate */
  ["\uDFF1", "\uDFF1"], /* Low Surrogate */
  ["\uDFF2", "\uDFF2"], /* Low Surrogate */
  ["\uDFF3", "\uDFF3"], /* Low Surrogate */
  ["\uDFF4", "\uDFF4"], /* Low Surrogate */
  ["\uDFF5", "\uDFF5"], /* Low Surrogate */
  ["\uDFF6", "\uDFF6"], /* Low Surrogate */
  ["\uDFF7", "\uDFF7"], /* Low Surrogate */
  ["\uDFF8", "\uDFF8"], /* Low Surrogate */
  ["\uDFF9", "\uDFF9"], /* Low Surrogate */
  ["\uDFFA", "\uDFFA"], /* Low Surrogate */
  ["\uDFFB", "\uDFFB"], /* Low Surrogate */
  ["\uDFFC", "\uDFFC"], /* Low Surrogate */
  ["\uDFFD", "\uDFFD"], /* Low Surrogate */
  ["\uDFFE", "\uDFFE"], /* Low Surrogate */
  ["\uDFFF", "\uDFFF"], /* Low Surrogate */
  ["\uE000", "\uE000"], /* Private Use */
  ["\uE001", "\uE001"], /* Private Use */
  ["\uE002", "\uE002"], /* Private Use */
  ["\uE003", "\uE003"], /* Private Use */
  ["\uE004", "\uE004"], /* Private Use */
  ["\uE005", "\uE005"], /* Private Use */
  ["\uE006", "\uE006"], /* Private Use */
  ["\uE007", "\uE007"], /* Private Use */
  ["\uE008", "\uE008"], /* Private Use */
  ["\uE009", "\uE009"], /* Private Use */
  ["\uE00A", "\uE00A"], /* Private Use */
  ["\uE00B", "\uE00B"], /* Private Use */
  ["\uE00C", "\uE00C"], /* Private Use */
  ["\uE00D", "\uE00D"], /* Private Use */
  ["\uE00E", "\uE00E"], /* Private Use */
  ["\uE00F", "\uE00F"], /* Private Use */
  ["\uE010", "\uE010"], /* Private Use */
  ["\uE011", "\uE011"], /* Private Use */
  ["\uE012", "\uE012"], /* Private Use */
  ["\uE013", "\uE013"], /* Private Use */
  ["\uE014", "\uE014"], /* Private Use */
  ["\uE015", "\uE015"], /* Private Use */
  ["\uE016", "\uE016"], /* Private Use */
  ["\uE017", "\uE017"], /* Private Use */
  ["\uE018", "\uE018"], /* Private Use */
  ["\uE019", "\uE019"], /* Private Use */
  ["\uE01A", "\uE01A"], /* Private Use */
  ["\uE01B", "\uE01B"], /* Private Use */
  ["\uE01C", "\uE01C"], /* Private Use */
  ["\uE01D", "\uE01D"], /* Private Use */
  ["\uE01E", "\uE01E"], /* Private Use */
  ["\uE01F", "\uE01F"], /* Private Use */
  ["\uE020", "\uE020"], /* Private Use */
  ["\uE021", "\uE021"], /* Private Use */
  ["\uE022", "\uE022"], /* Private Use */
  ["\uE023", "\uE023"], /* Private Use */
  ["\uE024", "\uE024"], /* Private Use */
  ["\uE025", "\uE025"], /* Private Use */
  ["\uE026", "\uE026"], /* Private Use */
  ["\uE027", "\uE027"], /* Private Use */
  ["\uE028", "\uE028"], /* Private Use */
  ["\uE029", "\uE029"], /* Private Use */
  ["\uE02A", "\uE02A"], /* Private Use */
  ["\uE02B", "\uE02B"], /* Private Use */
  ["\uE02C", "\uE02C"], /* Private Use */
  ["\uE02D", "\uE02D"], /* Private Use */
  ["\uE02E", "\uE02E"], /* Private Use */
  ["\uE02F", "\uE02F"], /* Private Use */
  ["\uE030", "\uE030"], /* Private Use */
  ["\uE031", "\uE031"], /* Private Use */
  ["\uE032", "\uE032"], /* Private Use */
  ["\uE033", "\uE033"], /* Private Use */
  ["\uE034", "\uE034"], /* Private Use */
  ["\uE035", "\uE035"], /* Private Use */
  ["\uE036", "\uE036"], /* Private Use */
  ["\uE037", "\uE037"], /* Private Use */
  ["\uE038", "\uE038"], /* Private Use */
  ["\uE039", "\uE039"], /* Private Use */
  ["\uE03A", "\uE03A"], /* Private Use */
  ["\uE03B", "\uE03B"], /* Private Use */
  ["\uE03C", "\uE03C"], /* Private Use */
  ["\uE03D", "\uE03D"], /* Private Use */
  ["\uE03E", "\uE03E"], /* Private Use */
  ["\uE03F", "\uE03F"], /* Private Use */
  ["\uE040", "\uE040"], /* Private Use */
  ["\uE041", "\uE041"], /* Private Use */
  ["\uE042", "\uE042"], /* Private Use */
  ["\uE043", "\uE043"], /* Private Use */
  ["\uE044", "\uE044"], /* Private Use */
  ["\uE045", "\uE045"], /* Private Use */
  ["\uE046", "\uE046"], /* Private Use */
  ["\uE047", "\uE047"], /* Private Use */
  ["\uE048", "\uE048"], /* Private Use */
  ["\uE049", "\uE049"], /* Private Use */
  ["\uE04A", "\uE04A"], /* Private Use */
  ["\uE04B", "\uE04B"], /* Private Use */
  ["\uE04C", "\uE04C"], /* Private Use */
  ["\uE04D", "\uE04D"], /* Private Use */
  ["\uE04E", "\uE04E"], /* Private Use */
  ["\uE04F", "\uE04F"], /* Private Use */
  ["\uE050", "\uE050"], /* Private Use */
  ["\uE051", "\uE051"], /* Private Use */
  ["\uE052", "\uE052"], /* Private Use */
  ["\uE053", "\uE053"], /* Private Use */
  ["\uE054", "\uE054"], /* Private Use */
  ["\uE055", "\uE055"], /* Private Use */
  ["\uE056", "\uE056"], /* Private Use */
  ["\uE057", "\uE057"], /* Private Use */
  ["\uE058", "\uE058"], /* Private Use */
  ["\uE059", "\uE059"], /* Private Use */
  ["\uE05A", "\uE05A"], /* Private Use */
  ["\uE05B", "\uE05B"], /* Private Use */
  ["\uE05C", "\uE05C"], /* Private Use */
  ["\uE05D", "\uE05D"], /* Private Use */
  ["\uE05E", "\uE05E"], /* Private Use */
  ["\uE05F", "\uE05F"], /* Private Use */
  ["\uE060", "\uE060"], /* Private Use */
  ["\uE061", "\uE061"], /* Private Use */
  ["\uE062", "\uE062"], /* Private Use */
  ["\uE063", "\uE063"], /* Private Use */
  ["\uE064", "\uE064"], /* Private Use */
  ["\uE065", "\uE065"], /* Private Use */
  ["\uE066", "\uE066"], /* Private Use */
  ["\uE067", "\uE067"], /* Private Use */
  ["\uE068", "\uE068"], /* Private Use */
  ["\uE069", "\uE069"], /* Private Use */
  ["\uE06A", "\uE06A"], /* Private Use */
  ["\uE06B", "\uE06B"], /* Private Use */
  ["\uE06C", "\uE06C"], /* Private Use */
  ["\uE06D", "\uE06D"], /* Private Use */
  ["\uE06E", "\uE06E"], /* Private Use */
  ["\uE06F", "\uE06F"], /* Private Use */
  ["\uE070", "\uE070"], /* Private Use */
  ["\uE071", "\uE071"], /* Private Use */
  ["\uE072", "\uE072"], /* Private Use */
  ["\uE073", "\uE073"], /* Private Use */
  ["\uE074", "\uE074"], /* Private Use */
  ["\uE075", "\uE075"], /* Private Use */
  ["\uE076", "\uE076"], /* Private Use */
  ["\uE077", "\uE077"], /* Private Use */
  ["\uE078", "\uE078"], /* Private Use */
  ["\uE079", "\uE079"], /* Private Use */
  ["\uE07A", "\uE07A"], /* Private Use */
  ["\uE07B", "\uE07B"], /* Private Use */
  ["\uE07C", "\uE07C"], /* Private Use */
  ["\uE07D", "\uE07D"], /* Private Use */
  ["\uE07E", "\uE07E"], /* Private Use */
  ["\uE07F", "\uE07F"], /* Private Use */
  ["\uE080", "\uE080"], /* Private Use */
  ["\uE081", "\uE081"], /* Private Use */
  ["\uE082", "\uE082"], /* Private Use */
  ["\uE083", "\uE083"], /* Private Use */
  ["\uE084", "\uE084"], /* Private Use */
  ["\uE085", "\uE085"], /* Private Use */
  ["\uE086", "\uE086"], /* Private Use */
  ["\uE087", "\uE087"], /* Private Use */
  ["\uE088", "\uE088"], /* Private Use */
  ["\uE089", "\uE089"], /* Private Use */
  ["\uE08A", "\uE08A"], /* Private Use */
  ["\uE08B", "\uE08B"], /* Private Use */
  ["\uE08C", "\uE08C"], /* Private Use */
  ["\uE08D", "\uE08D"], /* Private Use */
  ["\uE08E", "\uE08E"], /* Private Use */
  ["\uE08F", "\uE08F"], /* Private Use */
  ["\uE090", "\uE090"], /* Private Use */
  ["\uE091", "\uE091"], /* Private Use */
  ["\uE092", "\uE092"], /* Private Use */
  ["\uE093", "\uE093"], /* Private Use */
  ["\uE094", "\uE094"], /* Private Use */
  ["\uE095", "\uE095"], /* Private Use */
  ["\uE096", "\uE096"], /* Private Use */
  ["\uE097", "\uE097"], /* Private Use */
  ["\uE098", "\uE098"], /* Private Use */
  ["\uE099", "\uE099"], /* Private Use */
  ["\uE09A", "\uE09A"], /* Private Use */
  ["\uE09B", "\uE09B"], /* Private Use */
  ["\uE09C", "\uE09C"], /* Private Use */
  ["\uE09D", "\uE09D"], /* Private Use */
  ["\uE09E", "\uE09E"], /* Private Use */
  ["\uE09F", "\uE09F"], /* Private Use */
  ["\uE0A0", "\uE0A0"], /* Private Use */
  ["\uE0A1", "\uE0A1"], /* Private Use */
  ["\uE0A2", "\uE0A2"], /* Private Use */
  ["\uE0A3", "\uE0A3"], /* Private Use */
  ["\uE0A4", "\uE0A4"], /* Private Use */
  ["\uE0A5", "\uE0A5"], /* Private Use */
  ["\uE0A6", "\uE0A6"], /* Private Use */
  ["\uE0A7", "\uE0A7"], /* Private Use */
  ["\uE0A8", "\uE0A8"], /* Private Use */
  ["\uE0A9", "\uE0A9"], /* Private Use */
  ["\uE0AA", "\uE0AA"], /* Private Use */
  ["\uE0AB", "\uE0AB"], /* Private Use */
  ["\uE0AC", "\uE0AC"], /* Private Use */
  ["\uE0AD", "\uE0AD"], /* Private Use */
  ["\uE0AE", "\uE0AE"], /* Private Use */
  ["\uE0AF", "\uE0AF"], /* Private Use */
  ["\uE0B0", "\uE0B0"], /* Private Use */
  ["\uE0B1", "\uE0B1"], /* Private Use */
  ["\uE0B2", "\uE0B2"], /* Private Use */
  ["\uE0B3", "\uE0B3"], /* Private Use */
  ["\uE0B4", "\uE0B4"], /* Private Use */
  ["\uE0B5", "\uE0B5"], /* Private Use */
  ["\uE0B6", "\uE0B6"], /* Private Use */
  ["\uE0B7", "\uE0B7"], /* Private Use */
  ["\uE0B8", "\uE0B8"], /* Private Use */
  ["\uE0B9", "\uE0B9"], /* Private Use */
  ["\uE0BA", "\uE0BA"], /* Private Use */
  ["\uE0BB", "\uE0BB"], /* Private Use */
  ["\uE0BC", "\uE0BC"], /* Private Use */
  ["\uE0BD", "\uE0BD"], /* Private Use */
  ["\uE0BE", "\uE0BE"], /* Private Use */
  ["\uE0BF", "\uE0BF"], /* Private Use */
  ["\uE0C0", "\uE0C0"], /* Private Use */
  ["\uE0C1", "\uE0C1"], /* Private Use */
  ["\uE0C2", "\uE0C2"], /* Private Use */
  ["\uE0C3", "\uE0C3"], /* Private Use */
  ["\uE0C4", "\uE0C4"], /* Private Use */
  ["\uE0C5", "\uE0C5"], /* Private Use */
  ["\uE0C6", "\uE0C6"], /* Private Use */
  ["\uE0C7", "\uE0C7"], /* Private Use */
  ["\uE0C8", "\uE0C8"], /* Private Use */
  ["\uE0C9", "\uE0C9"], /* Private Use */
  ["\uE0CA", "\uE0CA"], /* Private Use */
  ["\uE0CB", "\uE0CB"], /* Private Use */
  ["\uE0CC", "\uE0CC"], /* Private Use */
  ["\uE0CD", "\uE0CD"], /* Private Use */
  ["\uE0CE", "\uE0CE"], /* Private Use */
  ["\uE0CF", "\uE0CF"], /* Private Use */
  ["\uE0D0", "\uE0D0"], /* Private Use */
  ["\uE0D1", "\uE0D1"], /* Private Use */
  ["\uE0D2", "\uE0D2"], /* Private Use */
  ["\uE0D3", "\uE0D3"], /* Private Use */
  ["\uE0D4", "\uE0D4"], /* Private Use */
  ["\uE0D5", "\uE0D5"], /* Private Use */
  ["\uE0D6", "\uE0D6"], /* Private Use */
  ["\uE0D7", "\uE0D7"], /* Private Use */
  ["\uE0D8", "\uE0D8"], /* Private Use */
  ["\uE0D9", "\uE0D9"], /* Private Use */
  ["\uE0DA", "\uE0DA"], /* Private Use */
  ["\uE0DB", "\uE0DB"], /* Private Use */
  ["\uE0DC", "\uE0DC"], /* Private Use */
  ["\uE0DD", "\uE0DD"], /* Private Use */
  ["\uE0DE", "\uE0DE"], /* Private Use */
  ["\uE0DF", "\uE0DF"], /* Private Use */
  ["\uE0E0", "\uE0E0"], /* Private Use */
  ["\uE0E1", "\uE0E1"], /* Private Use */
  ["\uE0E2", "\uE0E2"], /* Private Use */
  ["\uE0E3", "\uE0E3"], /* Private Use */
  ["\uE0E4", "\uE0E4"], /* Private Use */
  ["\uE0E5", "\uE0E5"], /* Private Use */
  ["\uE0E6", "\uE0E6"], /* Private Use */
  ["\uE0E7", "\uE0E7"], /* Private Use */
  ["\uE0E8", "\uE0E8"], /* Private Use */
  ["\uE0E9", "\uE0E9"], /* Private Use */
  ["\uE0EA", "\uE0EA"], /* Private Use */
  ["\uE0EB", "\uE0EB"], /* Private Use */
  ["\uE0EC", "\uE0EC"], /* Private Use */
  ["\uE0ED", "\uE0ED"], /* Private Use */
  ["\uE0EE", "\uE0EE"], /* Private Use */
  ["\uE0EF", "\uE0EF"], /* Private Use */
  ["\uE0F0", "\uE0F0"], /* Private Use */
  ["\uE0F1", "\uE0F1"], /* Private Use */
  ["\uE0F2", "\uE0F2"], /* Private Use */
  ["\uE0F3", "\uE0F3"], /* Private Use */
  ["\uE0F4", "\uE0F4"], /* Private Use */
  ["\uE0F5", "\uE0F5"], /* Private Use */
  ["\uE0F6", "\uE0F6"], /* Private Use */
  ["\uE0F7", "\uE0F7"], /* Private Use */
  ["\uE0F8", "\uE0F8"], /* Private Use */
  ["\uE0F9", "\uE0F9"], /* Private Use */
  ["\uE0FA", "\uE0FA"], /* Private Use */
  ["\uE0FB", "\uE0FB"], /* Private Use */
  ["\uE0FC", "\uE0FC"], /* Private Use */
  ["\uE0FD", "\uE0FD"], /* Private Use */
  ["\uE0FE", "\uE0FE"], /* Private Use */
  ["\uE0FF", "\uE0FF"], /* Private Use */
  ["\uE100", "\uE100"], /* Private Use */
  ["\uE101", "\uE101"], /* Private Use */
  ["\uE102", "\uE102"], /* Private Use */
  ["\uE103", "\uE103"], /* Private Use */
  ["\uE104", "\uE104"], /* Private Use */
  ["\uE105", "\uE105"], /* Private Use */
  ["\uE106", "\uE106"], /* Private Use */
  ["\uE107", "\uE107"], /* Private Use */
  ["\uE108", "\uE108"], /* Private Use */
  ["\uE109", "\uE109"], /* Private Use */
  ["\uE10A", "\uE10A"], /* Private Use */
  ["\uE10B", "\uE10B"], /* Private Use */
  ["\uE10C", "\uE10C"], /* Private Use */
  ["\uE10D", "\uE10D"], /* Private Use */
  ["\uE10E", "\uE10E"], /* Private Use */
  ["\uE10F", "\uE10F"], /* Private Use */
  ["\uE110", "\uE110"], /* Private Use */
  ["\uE111", "\uE111"], /* Private Use */
  ["\uE112", "\uE112"], /* Private Use */
  ["\uE113", "\uE113"], /* Private Use */
  ["\uE114", "\uE114"], /* Private Use */
  ["\uE115", "\uE115"], /* Private Use */
  ["\uE116", "\uE116"], /* Private Use */
  ["\uE117", "\uE117"], /* Private Use */
  ["\uE118", "\uE118"], /* Private Use */
  ["\uE119", "\uE119"], /* Private Use */
  ["\uE11A", "\uE11A"], /* Private Use */
  ["\uE11B", "\uE11B"], /* Private Use */
  ["\uE11C", "\uE11C"], /* Private Use */
  ["\uE11D", "\uE11D"], /* Private Use */
  ["\uE11E", "\uE11E"], /* Private Use */
  ["\uE11F", "\uE11F"], /* Private Use */
  ["\uE120", "\uE120"], /* Private Use */
  ["\uE121", "\uE121"], /* Private Use */
  ["\uE122", "\uE122"], /* Private Use */
  ["\uE123", "\uE123"], /* Private Use */
  ["\uE124", "\uE124"], /* Private Use */
  ["\uE125", "\uE125"], /* Private Use */
  ["\uE126", "\uE126"], /* Private Use */
  ["\uE127", "\uE127"], /* Private Use */
  ["\uE128", "\uE128"], /* Private Use */
  ["\uE129", "\uE129"], /* Private Use */
  ["\uE12A", "\uE12A"], /* Private Use */
  ["\uE12B", "\uE12B"], /* Private Use */
  ["\uE12C", "\uE12C"], /* Private Use */
  ["\uE12D", "\uE12D"], /* Private Use */
  ["\uE12E", "\uE12E"], /* Private Use */
  ["\uE12F", "\uE12F"], /* Private Use */
  ["\uE130", "\uE130"], /* Private Use */
  ["\uE131", "\uE131"], /* Private Use */
  ["\uE132", "\uE132"], /* Private Use */
  ["\uE133", "\uE133"], /* Private Use */
  ["\uE134", "\uE134"], /* Private Use */
  ["\uE135", "\uE135"], /* Private Use */
  ["\uE136", "\uE136"], /* Private Use */
  ["\uE137", "\uE137"], /* Private Use */
  ["\uE138", "\uE138"], /* Private Use */
  ["\uE139", "\uE139"], /* Private Use */
  ["\uE13A", "\uE13A"], /* Private Use */
  ["\uE13B", "\uE13B"], /* Private Use */
  ["\uE13C", "\uE13C"], /* Private Use */
  ["\uE13D", "\uE13D"], /* Private Use */
  ["\uE13E", "\uE13E"], /* Private Use */
  ["\uE13F", "\uE13F"], /* Private Use */
  ["\uE140", "\uE140"], /* Private Use */
  ["\uE141", "\uE141"], /* Private Use */
  ["\uE142", "\uE142"], /* Private Use */
  ["\uE143", "\uE143"], /* Private Use */
  ["\uE144", "\uE144"], /* Private Use */
  ["\uE145", "\uE145"], /* Private Use */
  ["\uE146", "\uE146"], /* Private Use */
  ["\uE147", "\uE147"], /* Private Use */
  ["\uE148", "\uE148"], /* Private Use */
  ["\uE149", "\uE149"], /* Private Use */
  ["\uE14A", "\uE14A"], /* Private Use */
  ["\uE14B", "\uE14B"], /* Private Use */
  ["\uE14C", "\uE14C"], /* Private Use */
  ["\uE14D", "\uE14D"], /* Private Use */
  ["\uE14E", "\uE14E"], /* Private Use */
  ["\uE14F", "\uE14F"], /* Private Use */
  ["\uE150", "\uE150"], /* Private Use */
  ["\uE151", "\uE151"], /* Private Use */
  ["\uE152", "\uE152"], /* Private Use */
  ["\uE153", "\uE153"], /* Private Use */
  ["\uE154", "\uE154"], /* Private Use */
  ["\uE155", "\uE155"], /* Private Use */
  ["\uE156", "\uE156"], /* Private Use */
  ["\uE157", "\uE157"], /* Private Use */
  ["\uE158", "\uE158"], /* Private Use */
  ["\uE159", "\uE159"], /* Private Use */
  ["\uE15A", "\uE15A"], /* Private Use */
  ["\uE15B", "\uE15B"], /* Private Use */
  ["\uE15C", "\uE15C"], /* Private Use */
  ["\uE15D", "\uE15D"], /* Private Use */
  ["\uE15E", "\uE15E"], /* Private Use */
  ["\uE15F", "\uE15F"], /* Private Use */
  ["\uE160", "\uE160"], /* Private Use */
  ["\uE161", "\uE161"], /* Private Use */
  ["\uE162", "\uE162"], /* Private Use */
  ["\uE163", "\uE163"], /* Private Use */
  ["\uE164", "\uE164"], /* Private Use */
  ["\uE165", "\uE165"], /* Private Use */
  ["\uE166", "\uE166"], /* Private Use */
  ["\uE167", "\uE167"], /* Private Use */
  ["\uE168", "\uE168"], /* Private Use */
  ["\uE169", "\uE169"], /* Private Use */
  ["\uE16A", "\uE16A"], /* Private Use */
  ["\uE16B", "\uE16B"], /* Private Use */
  ["\uE16C", "\uE16C"], /* Private Use */
  ["\uE16D", "\uE16D"], /* Private Use */
  ["\uE16E", "\uE16E"], /* Private Use */
  ["\uE16F", "\uE16F"], /* Private Use */
  ["\uE170", "\uE170"], /* Private Use */
  ["\uE171", "\uE171"], /* Private Use */
  ["\uE172", "\uE172"], /* Private Use */
  ["\uE173", "\uE173"], /* Private Use */
  ["\uE174", "\uE174"], /* Private Use */
  ["\uE175", "\uE175"], /* Private Use */
  ["\uE176", "\uE176"], /* Private Use */
  ["\uE177", "\uE177"], /* Private Use */
  ["\uE178", "\uE178"], /* Private Use */
  ["\uE179", "\uE179"], /* Private Use */
  ["\uE17A", "\uE17A"], /* Private Use */
  ["\uE17B", "\uE17B"], /* Private Use */
  ["\uE17C", "\uE17C"], /* Private Use */
  ["\uE17D", "\uE17D"], /* Private Use */
  ["\uE17E", "\uE17E"], /* Private Use */
  ["\uE17F", "\uE17F"], /* Private Use */
  ["\uE180", "\uE180"], /* Private Use */
  ["\uE181", "\uE181"], /* Private Use */
  ["\uE182", "\uE182"], /* Private Use */
  ["\uE183", "\uE183"], /* Private Use */
  ["\uE184", "\uE184"], /* Private Use */
  ["\uE185", "\uE185"], /* Private Use */
  ["\uE186", "\uE186"], /* Private Use */
  ["\uE187", "\uE187"], /* Private Use */
  ["\uE188", "\uE188"], /* Private Use */
  ["\uE189", "\uE189"], /* Private Use */
  ["\uE18A", "\uE18A"], /* Private Use */
  ["\uE18B", "\uE18B"], /* Private Use */
  ["\uE18C", "\uE18C"], /* Private Use */
  ["\uE18D", "\uE18D"], /* Private Use */
  ["\uE18E", "\uE18E"], /* Private Use */
  ["\uE18F", "\uE18F"], /* Private Use */
  ["\uE190", "\uE190"], /* Private Use */
  ["\uE191", "\uE191"], /* Private Use */
  ["\uE192", "\uE192"], /* Private Use */
  ["\uE193", "\uE193"], /* Private Use */
  ["\uE194", "\uE194"], /* Private Use */
  ["\uE195", "\uE195"], /* Private Use */
  ["\uE196", "\uE196"], /* Private Use */
  ["\uE197", "\uE197"], /* Private Use */
  ["\uE198", "\uE198"], /* Private Use */
  ["\uE199", "\uE199"], /* Private Use */
  ["\uE19A", "\uE19A"], /* Private Use */
  ["\uE19B", "\uE19B"], /* Private Use */
  ["\uE19C", "\uE19C"], /* Private Use */
  ["\uE19D", "\uE19D"], /* Private Use */
  ["\uE19E", "\uE19E"], /* Private Use */
  ["\uE19F", "\uE19F"], /* Private Use */
  ["\uE1A0", "\uE1A0"], /* Private Use */
  ["\uE1A1", "\uE1A1"], /* Private Use */
  ["\uE1A2", "\uE1A2"], /* Private Use */
  ["\uE1A3", "\uE1A3"], /* Private Use */
  ["\uE1A4", "\uE1A4"], /* Private Use */
  ["\uE1A5", "\uE1A5"], /* Private Use */
  ["\uE1A6", "\uE1A6"], /* Private Use */
  ["\uE1A7", "\uE1A7"], /* Private Use */
  ["\uE1A8", "\uE1A8"], /* Private Use */
  ["\uE1A9", "\uE1A9"], /* Private Use */
  ["\uE1AA", "\uE1AA"], /* Private Use */
  ["\uE1AB", "\uE1AB"], /* Private Use */
  ["\uE1AC", "\uE1AC"], /* Private Use */
  ["\uE1AD", "\uE1AD"], /* Private Use */
  ["\uE1AE", "\uE1AE"], /* Private Use */
  ["\uE1AF", "\uE1AF"], /* Private Use */
  ["\uE1B0", "\uE1B0"], /* Private Use */
  ["\uE1B1", "\uE1B1"], /* Private Use */
  ["\uE1B2", "\uE1B2"], /* Private Use */
  ["\uE1B3", "\uE1B3"], /* Private Use */
  ["\uE1B4", "\uE1B4"], /* Private Use */
  ["\uE1B5", "\uE1B5"], /* Private Use */
  ["\uE1B6", "\uE1B6"], /* Private Use */
  ["\uE1B7", "\uE1B7"], /* Private Use */
  ["\uE1B8", "\uE1B8"], /* Private Use */
  ["\uE1B9", "\uE1B9"], /* Private Use */
  ["\uE1BA", "\uE1BA"], /* Private Use */
  ["\uE1BB", "\uE1BB"], /* Private Use */
  ["\uE1BC", "\uE1BC"], /* Private Use */
  ["\uE1BD", "\uE1BD"], /* Private Use */
  ["\uE1BE", "\uE1BE"], /* Private Use */
  ["\uE1BF", "\uE1BF"], /* Private Use */
  ["\uE1C0", "\uE1C0"], /* Private Use */
  ["\uE1C1", "\uE1C1"], /* Private Use */
  ["\uE1C2", "\uE1C2"], /* Private Use */
  ["\uE1C3", "\uE1C3"], /* Private Use */
  ["\uE1C4", "\uE1C4"], /* Private Use */
  ["\uE1C5", "\uE1C5"], /* Private Use */
  ["\uE1C6", "\uE1C6"], /* Private Use */
  ["\uE1C7", "\uE1C7"], /* Private Use */
  ["\uE1C8", "\uE1C8"], /* Private Use */
  ["\uE1C9", "\uE1C9"], /* Private Use */
  ["\uE1CA", "\uE1CA"], /* Private Use */
  ["\uE1CB", "\uE1CB"], /* Private Use */
  ["\uE1CC", "\uE1CC"], /* Private Use */
  ["\uE1CD", "\uE1CD"], /* Private Use */
  ["\uE1CE", "\uE1CE"], /* Private Use */
  ["\uE1CF", "\uE1CF"], /* Private Use */
  ["\uE1D0", "\uE1D0"], /* Private Use */
  ["\uE1D1", "\uE1D1"], /* Private Use */
  ["\uE1D2", "\uE1D2"], /* Private Use */
  ["\uE1D3", "\uE1D3"], /* Private Use */
  ["\uE1D4", "\uE1D4"], /* Private Use */
  ["\uE1D5", "\uE1D5"], /* Private Use */
  ["\uE1D6", "\uE1D6"], /* Private Use */
  ["\uE1D7", "\uE1D7"], /* Private Use */
  ["\uE1D8", "\uE1D8"], /* Private Use */
  ["\uE1D9", "\uE1D9"], /* Private Use */
  ["\uE1DA", "\uE1DA"], /* Private Use */
  ["\uE1DB", "\uE1DB"], /* Private Use */
  ["\uE1DC", "\uE1DC"], /* Private Use */
  ["\uE1DD", "\uE1DD"], /* Private Use */
  ["\uE1DE", "\uE1DE"], /* Private Use */
  ["\uE1DF", "\uE1DF"], /* Private Use */
  ["\uE1E0", "\uE1E0"], /* Private Use */
  ["\uE1E1", "\uE1E1"], /* Private Use */
  ["\uE1E2", "\uE1E2"], /* Private Use */
  ["\uE1E3", "\uE1E3"], /* Private Use */
  ["\uE1E4", "\uE1E4"], /* Private Use */
  ["\uE1E5", "\uE1E5"], /* Private Use */
  ["\uE1E6", "\uE1E6"], /* Private Use */
  ["\uE1E7", "\uE1E7"], /* Private Use */
  ["\uE1E8", "\uE1E8"], /* Private Use */
  ["\uE1E9", "\uE1E9"], /* Private Use */
  ["\uE1EA", "\uE1EA"], /* Private Use */
  ["\uE1EB", "\uE1EB"], /* Private Use */
  ["\uE1EC", "\uE1EC"], /* Private Use */
  ["\uE1ED", "\uE1ED"], /* Private Use */
  ["\uE1EE", "\uE1EE"], /* Private Use */
  ["\uE1EF", "\uE1EF"], /* Private Use */
  ["\uE1F0", "\uE1F0"], /* Private Use */
  ["\uE1F1", "\uE1F1"], /* Private Use */
  ["\uE1F2", "\uE1F2"], /* Private Use */
  ["\uE1F3", "\uE1F3"], /* Private Use */
  ["\uE1F4", "\uE1F4"], /* Private Use */
  ["\uE1F5", "\uE1F5"], /* Private Use */
  ["\uE1F6", "\uE1F6"], /* Private Use */
  ["\uE1F7", "\uE1F7"], /* Private Use */
  ["\uE1F8", "\uE1F8"], /* Private Use */
  ["\uE1F9", "\uE1F9"], /* Private Use */
  ["\uE1FA", "\uE1FA"], /* Private Use */
  ["\uE1FB", "\uE1FB"], /* Private Use */
  ["\uE1FC", "\uE1FC"], /* Private Use */
  ["\uE1FD", "\uE1FD"], /* Private Use */
  ["\uE1FE", "\uE1FE"], /* Private Use */
  ["\uE1FF", "\uE1FF"], /* Private Use */
  ["\uE200", "\uE200"], /* Private Use */
  ["\uE201", "\uE201"], /* Private Use */
  ["\uE202", "\uE202"], /* Private Use */
  ["\uE203", "\uE203"], /* Private Use */
  ["\uE204", "\uE204"], /* Private Use */
  ["\uE205", "\uE205"], /* Private Use */
  ["\uE206", "\uE206"], /* Private Use */
  ["\uE207", "\uE207"], /* Private Use */
  ["\uE208", "\uE208"], /* Private Use */
  ["\uE209", "\uE209"], /* Private Use */
  ["\uE20A", "\uE20A"], /* Private Use */
  ["\uE20B", "\uE20B"], /* Private Use */
  ["\uE20C", "\uE20C"], /* Private Use */
  ["\uE20D", "\uE20D"], /* Private Use */
  ["\uE20E", "\uE20E"], /* Private Use */
  ["\uE20F", "\uE20F"], /* Private Use */
  ["\uE210", "\uE210"], /* Private Use */
  ["\uE211", "\uE211"], /* Private Use */
  ["\uE212", "\uE212"], /* Private Use */
  ["\uE213", "\uE213"], /* Private Use */
  ["\uE214", "\uE214"], /* Private Use */
  ["\uE215", "\uE215"], /* Private Use */
  ["\uE216", "\uE216"], /* Private Use */
  ["\uE217", "\uE217"], /* Private Use */
  ["\uE218", "\uE218"], /* Private Use */
  ["\uE219", "\uE219"], /* Private Use */
  ["\uE21A", "\uE21A"], /* Private Use */
  ["\uE21B", "\uE21B"], /* Private Use */
  ["\uE21C", "\uE21C"], /* Private Use */
  ["\uE21D", "\uE21D"], /* Private Use */
  ["\uE21E", "\uE21E"], /* Private Use */
  ["\uE21F", "\uE21F"], /* Private Use */
  ["\uE220", "\uE220"], /* Private Use */
  ["\uE221", "\uE221"], /* Private Use */
  ["\uE222", "\uE222"], /* Private Use */
  ["\uE223", "\uE223"], /* Private Use */
  ["\uE224", "\uE224"], /* Private Use */
  ["\uE225", "\uE225"], /* Private Use */
  ["\uE226", "\uE226"], /* Private Use */
  ["\uE227", "\uE227"], /* Private Use */
  ["\uE228", "\uE228"], /* Private Use */
  ["\uE229", "\uE229"], /* Private Use */
  ["\uE22A", "\uE22A"], /* Private Use */
  ["\uE22B", "\uE22B"], /* Private Use */
  ["\uE22C", "\uE22C"], /* Private Use */
  ["\uE22D", "\uE22D"], /* Private Use */
  ["\uE22E", "\uE22E"], /* Private Use */
  ["\uE22F", "\uE22F"], /* Private Use */
  ["\uE230", "\uE230"], /* Private Use */
  ["\uE231", "\uE231"], /* Private Use */
  ["\uE232", "\uE232"], /* Private Use */
  ["\uE233", "\uE233"], /* Private Use */
  ["\uE234", "\uE234"], /* Private Use */
  ["\uE235", "\uE235"], /* Private Use */
  ["\uE236", "\uE236"], /* Private Use */
  ["\uE237", "\uE237"], /* Private Use */
  ["\uE238", "\uE238"], /* Private Use */
  ["\uE239", "\uE239"], /* Private Use */
  ["\uE23A", "\uE23A"], /* Private Use */
  ["\uE23B", "\uE23B"], /* Private Use */
  ["\uE23C", "\uE23C"], /* Private Use */
  ["\uE23D", "\uE23D"], /* Private Use */
  ["\uE23E", "\uE23E"], /* Private Use */
  ["\uE23F", "\uE23F"], /* Private Use */
  ["\uE240", "\uE240"], /* Private Use */
  ["\uE241", "\uE241"], /* Private Use */
  ["\uE242", "\uE242"], /* Private Use */
  ["\uE243", "\uE243"], /* Private Use */
  ["\uE244", "\uE244"], /* Private Use */
  ["\uE245", "\uE245"], /* Private Use */
  ["\uE246", "\uE246"], /* Private Use */
  ["\uE247", "\uE247"], /* Private Use */
  ["\uE248", "\uE248"], /* Private Use */
  ["\uE249", "\uE249"], /* Private Use */
  ["\uE24A", "\uE24A"], /* Private Use */
  ["\uE24B", "\uE24B"], /* Private Use */
  ["\uE24C", "\uE24C"], /* Private Use */
  ["\uE24D", "\uE24D"], /* Private Use */
  ["\uE24E", "\uE24E"], /* Private Use */
  ["\uE24F", "\uE24F"], /* Private Use */
  ["\uE250", "\uE250"], /* Private Use */
  ["\uE251", "\uE251"], /* Private Use */
  ["\uE252", "\uE252"], /* Private Use */
  ["\uE253", "\uE253"], /* Private Use */
  ["\uE254", "\uE254"], /* Private Use */
  ["\uE255", "\uE255"], /* Private Use */
  ["\uE256", "\uE256"], /* Private Use */
  ["\uE257", "\uE257"], /* Private Use */
  ["\uE258", "\uE258"], /* Private Use */
  ["\uE259", "\uE259"], /* Private Use */
  ["\uE25A", "\uE25A"], /* Private Use */
  ["\uE25B", "\uE25B"], /* Private Use */
  ["\uE25C", "\uE25C"], /* Private Use */
  ["\uE25D", "\uE25D"], /* Private Use */
  ["\uE25E", "\uE25E"], /* Private Use */
  ["\uE25F", "\uE25F"], /* Private Use */
  ["\uE260", "\uE260"], /* Private Use */
  ["\uE261", "\uE261"], /* Private Use */
  ["\uE262", "\uE262"], /* Private Use */
  ["\uE263", "\uE263"], /* Private Use */
  ["\uE264", "\uE264"], /* Private Use */
  ["\uE265", "\uE265"], /* Private Use */
  ["\uE266", "\uE266"], /* Private Use */
  ["\uE267", "\uE267"], /* Private Use */
  ["\uE268", "\uE268"], /* Private Use */
  ["\uE269", "\uE269"], /* Private Use */
  ["\uE26A", "\uE26A"], /* Private Use */
  ["\uE26B", "\uE26B"], /* Private Use */
  ["\uE26C", "\uE26C"], /* Private Use */
  ["\uE26D", "\uE26D"], /* Private Use */
  ["\uE26E", "\uE26E"], /* Private Use */
  ["\uE26F", "\uE26F"], /* Private Use */
  ["\uE270", "\uE270"], /* Private Use */
  ["\uE271", "\uE271"], /* Private Use */
  ["\uE272", "\uE272"], /* Private Use */
  ["\uE273", "\uE273"], /* Private Use */
  ["\uE274", "\uE274"], /* Private Use */
  ["\uE275", "\uE275"], /* Private Use */
  ["\uE276", "\uE276"], /* Private Use */
  ["\uE277", "\uE277"], /* Private Use */
  ["\uE278", "\uE278"], /* Private Use */
  ["\uE279", "\uE279"], /* Private Use */
  ["\uE27A", "\uE27A"], /* Private Use */
  ["\uE27B", "\uE27B"], /* Private Use */
  ["\uE27C", "\uE27C"], /* Private Use */
  ["\uE27D", "\uE27D"], /* Private Use */
  ["\uE27E", "\uE27E"], /* Private Use */
  ["\uE27F", "\uE27F"], /* Private Use */
  ["\uE280", "\uE280"], /* Private Use */
  ["\uE281", "\uE281"], /* Private Use */
  ["\uE282", "\uE282"], /* Private Use */
  ["\uE283", "\uE283"], /* Private Use */
  ["\uE284", "\uE284"], /* Private Use */
  ["\uE285", "\uE285"], /* Private Use */
  ["\uE286", "\uE286"], /* Private Use */
  ["\uE287", "\uE287"], /* Private Use */
  ["\uE288", "\uE288"], /* Private Use */
  ["\uE289", "\uE289"], /* Private Use */
  ["\uE28A", "\uE28A"], /* Private Use */
  ["\uE28B", "\uE28B"], /* Private Use */
  ["\uE28C", "\uE28C"], /* Private Use */
  ["\uE28D", "\uE28D"], /* Private Use */
  ["\uE28E", "\uE28E"], /* Private Use */
  ["\uE28F", "\uE28F"], /* Private Use */
  ["\uE290", "\uE290"], /* Private Use */
  ["\uE291", "\uE291"], /* Private Use */
  ["\uE292", "\uE292"], /* Private Use */
  ["\uE293", "\uE293"], /* Private Use */
  ["\uE294", "\uE294"], /* Private Use */
  ["\uE295", "\uE295"], /* Private Use */
  ["\uE296", "\uE296"], /* Private Use */
  ["\uE297", "\uE297"], /* Private Use */
  ["\uE298", "\uE298"], /* Private Use */
  ["\uE299", "\uE299"], /* Private Use */
  ["\uE29A", "\uE29A"], /* Private Use */
  ["\uE29B", "\uE29B"], /* Private Use */
  ["\uE29C", "\uE29C"], /* Private Use */
  ["\uE29D", "\uE29D"], /* Private Use */
  ["\uE29E", "\uE29E"], /* Private Use */
  ["\uE29F", "\uE29F"], /* Private Use */
  ["\uE2A0", "\uE2A0"], /* Private Use */
  ["\uE2A1", "\uE2A1"], /* Private Use */
  ["\uE2A2", "\uE2A2"], /* Private Use */
  ["\uE2A3", "\uE2A3"], /* Private Use */
  ["\uE2A4", "\uE2A4"], /* Private Use */
  ["\uE2A5", "\uE2A5"], /* Private Use */
  ["\uE2A6", "\uE2A6"], /* Private Use */
  ["\uE2A7", "\uE2A7"], /* Private Use */
  ["\uE2A8", "\uE2A8"], /* Private Use */
  ["\uE2A9", "\uE2A9"], /* Private Use */
  ["\uE2AA", "\uE2AA"], /* Private Use */
  ["\uE2AB", "\uE2AB"], /* Private Use */
  ["\uE2AC", "\uE2AC"], /* Private Use */
  ["\uE2AD", "\uE2AD"], /* Private Use */
  ["\uE2AE", "\uE2AE"], /* Private Use */
  ["\uE2AF", "\uE2AF"], /* Private Use */
  ["\uE2B0", "\uE2B0"], /* Private Use */
  ["\uE2B1", "\uE2B1"], /* Private Use */
  ["\uE2B2", "\uE2B2"], /* Private Use */
  ["\uE2B3", "\uE2B3"], /* Private Use */
  ["\uE2B4", "\uE2B4"], /* Private Use */
  ["\uE2B5", "\uE2B5"], /* Private Use */
  ["\uE2B6", "\uE2B6"], /* Private Use */
  ["\uE2B7", "\uE2B7"], /* Private Use */
  ["\uE2B8", "\uE2B8"], /* Private Use */
  ["\uE2B9", "\uE2B9"], /* Private Use */
  ["\uE2BA", "\uE2BA"], /* Private Use */
  ["\uE2BB", "\uE2BB"], /* Private Use */
  ["\uE2BC", "\uE2BC"], /* Private Use */
  ["\uE2BD", "\uE2BD"], /* Private Use */
  ["\uE2BE", "\uE2BE"], /* Private Use */
  ["\uE2BF", "\uE2BF"], /* Private Use */
  ["\uE2C0", "\uE2C0"], /* Private Use */
  ["\uE2C1", "\uE2C1"], /* Private Use */
  ["\uE2C2", "\uE2C2"], /* Private Use */
  ["\uE2C3", "\uE2C3"], /* Private Use */
  ["\uE2C4", "\uE2C4"], /* Private Use */
  ["\uE2C5", "\uE2C5"], /* Private Use */
  ["\uE2C6", "\uE2C6"], /* Private Use */
  ["\uE2C7", "\uE2C7"], /* Private Use */
  ["\uE2C8", "\uE2C8"], /* Private Use */
  ["\uE2C9", "\uE2C9"], /* Private Use */
  ["\uE2CA", "\uE2CA"], /* Private Use */
  ["\uE2CB", "\uE2CB"], /* Private Use */
  ["\uE2CC", "\uE2CC"], /* Private Use */
  ["\uE2CD", "\uE2CD"], /* Private Use */
  ["\uE2CE", "\uE2CE"], /* Private Use */
  ["\uE2CF", "\uE2CF"], /* Private Use */
  ["\uE2D0", "\uE2D0"], /* Private Use */
  ["\uE2D1", "\uE2D1"], /* Private Use */
  ["\uE2D2", "\uE2D2"], /* Private Use */
  ["\uE2D3", "\uE2D3"], /* Private Use */
  ["\uE2D4", "\uE2D4"], /* Private Use */
  ["\uE2D5", "\uE2D5"], /* Private Use */
  ["\uE2D6", "\uE2D6"], /* Private Use */
  ["\uE2D7", "\uE2D7"], /* Private Use */
  ["\uE2D8", "\uE2D8"], /* Private Use */
  ["\uE2D9", "\uE2D9"], /* Private Use */
  ["\uE2DA", "\uE2DA"], /* Private Use */
  ["\uE2DB", "\uE2DB"], /* Private Use */
  ["\uE2DC", "\uE2DC"], /* Private Use */
  ["\uE2DD", "\uE2DD"], /* Private Use */
  ["\uE2DE", "\uE2DE"], /* Private Use */
  ["\uE2DF", "\uE2DF"], /* Private Use */
  ["\uE2E0", "\uE2E0"], /* Private Use */
  ["\uE2E1", "\uE2E1"], /* Private Use */
  ["\uE2E2", "\uE2E2"], /* Private Use */
  ["\uE2E3", "\uE2E3"], /* Private Use */
  ["\uE2E4", "\uE2E4"], /* Private Use */
  ["\uE2E5", "\uE2E5"], /* Private Use */
  ["\uE2E6", "\uE2E6"], /* Private Use */
  ["\uE2E7", "\uE2E7"], /* Private Use */
  ["\uE2E8", "\uE2E8"], /* Private Use */
  ["\uE2E9", "\uE2E9"], /* Private Use */
  ["\uE2EA", "\uE2EA"], /* Private Use */
  ["\uE2EB", "\uE2EB"], /* Private Use */
  ["\uE2EC", "\uE2EC"], /* Private Use */
  ["\uE2ED", "\uE2ED"], /* Private Use */
  ["\uE2EE", "\uE2EE"], /* Private Use */
  ["\uE2EF", "\uE2EF"], /* Private Use */
  ["\uE2F0", "\uE2F0"], /* Private Use */
  ["\uE2F1", "\uE2F1"], /* Private Use */
  ["\uE2F2", "\uE2F2"], /* Private Use */
  ["\uE2F3", "\uE2F3"], /* Private Use */
  ["\uE2F4", "\uE2F4"], /* Private Use */
  ["\uE2F5", "\uE2F5"], /* Private Use */
  ["\uE2F6", "\uE2F6"], /* Private Use */
  ["\uE2F7", "\uE2F7"], /* Private Use */
  ["\uE2F8", "\uE2F8"], /* Private Use */
  ["\uE2F9", "\uE2F9"], /* Private Use */
  ["\uE2FA", "\uE2FA"], /* Private Use */
  ["\uE2FB", "\uE2FB"], /* Private Use */
  ["\uE2FC", "\uE2FC"], /* Private Use */
  ["\uE2FD", "\uE2FD"], /* Private Use */
  ["\uE2FE", "\uE2FE"], /* Private Use */
  ["\uE2FF", "\uE2FF"], /* Private Use */
  ["\uE300", "\uE300"], /* Private Use */
  ["\uE301", "\uE301"], /* Private Use */
  ["\uE302", "\uE302"], /* Private Use */
  ["\uE303", "\uE303"], /* Private Use */
  ["\uE304", "\uE304"], /* Private Use */
  ["\uE305", "\uE305"], /* Private Use */
  ["\uE306", "\uE306"], /* Private Use */
  ["\uE307", "\uE307"], /* Private Use */
  ["\uE308", "\uE308"], /* Private Use */
  ["\uE309", "\uE309"], /* Private Use */
  ["\uE30A", "\uE30A"], /* Private Use */
  ["\uE30B", "\uE30B"], /* Private Use */
  ["\uE30C", "\uE30C"], /* Private Use */
  ["\uE30D", "\uE30D"], /* Private Use */
  ["\uE30E", "\uE30E"], /* Private Use */
  ["\uE30F", "\uE30F"], /* Private Use */
  ["\uE310", "\uE310"], /* Private Use */
  ["\uE311", "\uE311"], /* Private Use */
  ["\uE312", "\uE312"], /* Private Use */
  ["\uE313", "\uE313"], /* Private Use */
  ["\uE314", "\uE314"], /* Private Use */
  ["\uE315", "\uE315"], /* Private Use */
  ["\uE316", "\uE316"], /* Private Use */
  ["\uE317", "\uE317"], /* Private Use */
  ["\uE318", "\uE318"], /* Private Use */
  ["\uE319", "\uE319"], /* Private Use */
  ["\uE31A", "\uE31A"], /* Private Use */
  ["\uE31B", "\uE31B"], /* Private Use */
  ["\uE31C", "\uE31C"], /* Private Use */
  ["\uE31D", "\uE31D"], /* Private Use */
  ["\uE31E", "\uE31E"], /* Private Use */
  ["\uE31F", "\uE31F"], /* Private Use */
  ["\uE320", "\uE320"], /* Private Use */
  ["\uE321", "\uE321"], /* Private Use */
  ["\uE322", "\uE322"], /* Private Use */
  ["\uE323", "\uE323"], /* Private Use */
  ["\uE324", "\uE324"], /* Private Use */
  ["\uE325", "\uE325"], /* Private Use */
  ["\uE326", "\uE326"], /* Private Use */
  ["\uE327", "\uE327"], /* Private Use */
  ["\uE328", "\uE328"], /* Private Use */
  ["\uE329", "\uE329"], /* Private Use */
  ["\uE32A", "\uE32A"], /* Private Use */
  ["\uE32B", "\uE32B"], /* Private Use */
  ["\uE32C", "\uE32C"], /* Private Use */
  ["\uE32D", "\uE32D"], /* Private Use */
  ["\uE32E", "\uE32E"], /* Private Use */
  ["\uE32F", "\uE32F"], /* Private Use */
  ["\uE330", "\uE330"], /* Private Use */
  ["\uE331", "\uE331"], /* Private Use */
  ["\uE332", "\uE332"], /* Private Use */
  ["\uE333", "\uE333"], /* Private Use */
  ["\uE334", "\uE334"], /* Private Use */
  ["\uE335", "\uE335"], /* Private Use */
  ["\uE336", "\uE336"], /* Private Use */
  ["\uE337", "\uE337"], /* Private Use */
  ["\uE338", "\uE338"], /* Private Use */
  ["\uE339", "\uE339"], /* Private Use */
  ["\uE33A", "\uE33A"], /* Private Use */
  ["\uE33B", "\uE33B"], /* Private Use */
  ["\uE33C", "\uE33C"], /* Private Use */
  ["\uE33D", "\uE33D"], /* Private Use */
  ["\uE33E", "\uE33E"], /* Private Use */
  ["\uE33F", "\uE33F"], /* Private Use */
  ["\uE340", "\uE340"], /* Private Use */
  ["\uE341", "\uE341"], /* Private Use */
  ["\uE342", "\uE342"], /* Private Use */
  ["\uE343", "\uE343"], /* Private Use */
  ["\uE344", "\uE344"], /* Private Use */
  ["\uE345", "\uE345"], /* Private Use */
  ["\uE346", "\uE346"], /* Private Use */
  ["\uE347", "\uE347"], /* Private Use */
  ["\uE348", "\uE348"], /* Private Use */
  ["\uE349", "\uE349"], /* Private Use */
  ["\uE34A", "\uE34A"], /* Private Use */
  ["\uE34B", "\uE34B"], /* Private Use */
  ["\uE34C", "\uE34C"], /* Private Use */
  ["\uE34D", "\uE34D"], /* Private Use */
  ["\uE34E", "\uE34E"], /* Private Use */
  ["\uE34F", "\uE34F"], /* Private Use */
  ["\uE350", "\uE350"], /* Private Use */
  ["\uE351", "\uE351"], /* Private Use */
  ["\uE352", "\uE352"], /* Private Use */
  ["\uE353", "\uE353"], /* Private Use */
  ["\uE354", "\uE354"], /* Private Use */
  ["\uE355", "\uE355"], /* Private Use */
  ["\uE356", "\uE356"], /* Private Use */
  ["\uE357", "\uE357"], /* Private Use */
  ["\uE358", "\uE358"], /* Private Use */
  ["\uE359", "\uE359"], /* Private Use */
  ["\uE35A", "\uE35A"], /* Private Use */
  ["\uE35B", "\uE35B"], /* Private Use */
  ["\uE35C", "\uE35C"], /* Private Use */
  ["\uE35D", "\uE35D"], /* Private Use */
  ["\uE35E", "\uE35E"], /* Private Use */
  ["\uE35F", "\uE35F"], /* Private Use */
  ["\uE360", "\uE360"], /* Private Use */
  ["\uE361", "\uE361"], /* Private Use */
  ["\uE362", "\uE362"], /* Private Use */
  ["\uE363", "\uE363"], /* Private Use */
  ["\uE364", "\uE364"], /* Private Use */
  ["\uE365", "\uE365"], /* Private Use */
  ["\uE366", "\uE366"], /* Private Use */
  ["\uE367", "\uE367"], /* Private Use */
  ["\uE368", "\uE368"], /* Private Use */
  ["\uE369", "\uE369"], /* Private Use */
  ["\uE36A", "\uE36A"], /* Private Use */
  ["\uE36B", "\uE36B"], /* Private Use */
  ["\uE36C", "\uE36C"], /* Private Use */
  ["\uE36D", "\uE36D"], /* Private Use */
  ["\uE36E", "\uE36E"], /* Private Use */
  ["\uE36F", "\uE36F"], /* Private Use */
  ["\uE370", "\uE370"], /* Private Use */
  ["\uE371", "\uE371"], /* Private Use */
  ["\uE372", "\uE372"], /* Private Use */
  ["\uE373", "\uE373"], /* Private Use */
  ["\uE374", "\uE374"], /* Private Use */
  ["\uE375", "\uE375"], /* Private Use */
  ["\uE376", "\uE376"], /* Private Use */
  ["\uE377", "\uE377"], /* Private Use */
  ["\uE378", "\uE378"], /* Private Use */
  ["\uE379", "\uE379"], /* Private Use */
  ["\uE37A", "\uE37A"], /* Private Use */
  ["\uE37B", "\uE37B"], /* Private Use */
  ["\uE37C", "\uE37C"], /* Private Use */
  ["\uE37D", "\uE37D"], /* Private Use */
  ["\uE37E", "\uE37E"], /* Private Use */
  ["\uE37F", "\uE37F"], /* Private Use */
  ["\uE380", "\uE380"], /* Private Use */
  ["\uE381", "\uE381"], /* Private Use */
  ["\uE382", "\uE382"], /* Private Use */
  ["\uE383", "\uE383"], /* Private Use */
  ["\uE384", "\uE384"], /* Private Use */
  ["\uE385", "\uE385"], /* Private Use */
  ["\uE386", "\uE386"], /* Private Use */
  ["\uE387", "\uE387"], /* Private Use */
  ["\uE388", "\uE388"], /* Private Use */
  ["\uE389", "\uE389"], /* Private Use */
  ["\uE38A", "\uE38A"], /* Private Use */
  ["\uE38B", "\uE38B"], /* Private Use */
  ["\uE38C", "\uE38C"], /* Private Use */
  ["\uE38D", "\uE38D"], /* Private Use */
  ["\uE38E", "\uE38E"], /* Private Use */
  ["\uE38F", "\uE38F"], /* Private Use */
  ["\uE390", "\uE390"], /* Private Use */
  ["\uE391", "\uE391"], /* Private Use */
  ["\uE392", "\uE392"], /* Private Use */
  ["\uE393", "\uE393"], /* Private Use */
  ["\uE394", "\uE394"], /* Private Use */
  ["\uE395", "\uE395"], /* Private Use */
  ["\uE396", "\uE396"], /* Private Use */
  ["\uE397", "\uE397"], /* Private Use */
  ["\uE398", "\uE398"], /* Private Use */
  ["\uE399", "\uE399"], /* Private Use */
  ["\uE39A", "\uE39A"], /* Private Use */
  ["\uE39B", "\uE39B"], /* Private Use */
  ["\uE39C", "\uE39C"], /* Private Use */
  ["\uE39D", "\uE39D"], /* Private Use */
  ["\uE39E", "\uE39E"], /* Private Use */
  ["\uE39F", "\uE39F"], /* Private Use */
  ["\uE3A0", "\uE3A0"], /* Private Use */
  ["\uE3A1", "\uE3A1"], /* Private Use */
  ["\uE3A2", "\uE3A2"], /* Private Use */
  ["\uE3A3", "\uE3A3"], /* Private Use */
  ["\uE3A4", "\uE3A4"], /* Private Use */
  ["\uE3A5", "\uE3A5"], /* Private Use */
  ["\uE3A6", "\uE3A6"], /* Private Use */
  ["\uE3A7", "\uE3A7"], /* Private Use */
  ["\uE3A8", "\uE3A8"], /* Private Use */
  ["\uE3A9", "\uE3A9"], /* Private Use */
  ["\uE3AA", "\uE3AA"], /* Private Use */
  ["\uE3AB", "\uE3AB"], /* Private Use */
  ["\uE3AC", "\uE3AC"], /* Private Use */
  ["\uE3AD", "\uE3AD"], /* Private Use */
  ["\uE3AE", "\uE3AE"], /* Private Use */
  ["\uE3AF", "\uE3AF"], /* Private Use */
  ["\uE3B0", "\uE3B0"], /* Private Use */
  ["\uE3B1", "\uE3B1"], /* Private Use */
  ["\uE3B2", "\uE3B2"], /* Private Use */
  ["\uE3B3", "\uE3B3"], /* Private Use */
  ["\uE3B4", "\uE3B4"], /* Private Use */
  ["\uE3B5", "\uE3B5"], /* Private Use */
  ["\uE3B6", "\uE3B6"], /* Private Use */
  ["\uE3B7", "\uE3B7"], /* Private Use */
  ["\uE3B8", "\uE3B8"], /* Private Use */
  ["\uE3B9", "\uE3B9"], /* Private Use */
  ["\uE3BA", "\uE3BA"], /* Private Use */
  ["\uE3BB", "\uE3BB"], /* Private Use */
  ["\uE3BC", "\uE3BC"], /* Private Use */
  ["\uE3BD", "\uE3BD"], /* Private Use */
  ["\uE3BE", "\uE3BE"], /* Private Use */
  ["\uE3BF", "\uE3BF"], /* Private Use */
  ["\uE3C0", "\uE3C0"], /* Private Use */
  ["\uE3C1", "\uE3C1"], /* Private Use */
  ["\uE3C2", "\uE3C2"], /* Private Use */
  ["\uE3C3", "\uE3C3"], /* Private Use */
  ["\uE3C4", "\uE3C4"], /* Private Use */
  ["\uE3C5", "\uE3C5"], /* Private Use */
  ["\uE3C6", "\uE3C6"], /* Private Use */
  ["\uE3C7", "\uE3C7"], /* Private Use */
  ["\uE3C8", "\uE3C8"], /* Private Use */
  ["\uE3C9", "\uE3C9"], /* Private Use */
  ["\uE3CA", "\uE3CA"], /* Private Use */
  ["\uE3CB", "\uE3CB"], /* Private Use */
  ["\uE3CC", "\uE3CC"], /* Private Use */
  ["\uE3CD", "\uE3CD"], /* Private Use */
  ["\uE3CE", "\uE3CE"], /* Private Use */
  ["\uE3CF", "\uE3CF"], /* Private Use */
  ["\uE3D0", "\uE3D0"], /* Private Use */
  ["\uE3D1", "\uE3D1"], /* Private Use */
  ["\uE3D2", "\uE3D2"], /* Private Use */
  ["\uE3D3", "\uE3D3"], /* Private Use */
  ["\uE3D4", "\uE3D4"], /* Private Use */
  ["\uE3D5", "\uE3D5"], /* Private Use */
  ["\uE3D6", "\uE3D6"], /* Private Use */
  ["\uE3D7", "\uE3D7"], /* Private Use */
  ["\uE3D8", "\uE3D8"], /* Private Use */
  ["\uE3D9", "\uE3D9"], /* Private Use */
  ["\uE3DA", "\uE3DA"], /* Private Use */
  ["\uE3DB", "\uE3DB"], /* Private Use */
  ["\uE3DC", "\uE3DC"], /* Private Use */
  ["\uE3DD", "\uE3DD"], /* Private Use */
  ["\uE3DE", "\uE3DE"], /* Private Use */
  ["\uE3DF", "\uE3DF"], /* Private Use */
  ["\uE3E0", "\uE3E0"], /* Private Use */
  ["\uE3E1", "\uE3E1"], /* Private Use */
  ["\uE3E2", "\uE3E2"], /* Private Use */
  ["\uE3E3", "\uE3E3"], /* Private Use */
  ["\uE3E4", "\uE3E4"], /* Private Use */
  ["\uE3E5", "\uE3E5"], /* Private Use */
  ["\uE3E6", "\uE3E6"], /* Private Use */
  ["\uE3E7", "\uE3E7"], /* Private Use */
  ["\uE3E8", "\uE3E8"], /* Private Use */
  ["\uE3E9", "\uE3E9"], /* Private Use */
  ["\uE3EA", "\uE3EA"], /* Private Use */
  ["\uE3EB", "\uE3EB"], /* Private Use */
  ["\uE3EC", "\uE3EC"], /* Private Use */
  ["\uE3ED", "\uE3ED"], /* Private Use */
  ["\uE3EE", "\uE3EE"], /* Private Use */
  ["\uE3EF", "\uE3EF"], /* Private Use */
  ["\uE3F0", "\uE3F0"], /* Private Use */
  ["\uE3F1", "\uE3F1"], /* Private Use */
  ["\uE3F2", "\uE3F2"], /* Private Use */
  ["\uE3F3", "\uE3F3"], /* Private Use */
  ["\uE3F4", "\uE3F4"], /* Private Use */
  ["\uE3F5", "\uE3F5"], /* Private Use */
  ["\uE3F6", "\uE3F6"], /* Private Use */
  ["\uE3F7", "\uE3F7"], /* Private Use */
  ["\uE3F8", "\uE3F8"], /* Private Use */
  ["\uE3F9", "\uE3F9"], /* Private Use */
  ["\uE3FA", "\uE3FA"], /* Private Use */
  ["\uE3FB", "\uE3FB"], /* Private Use */
  ["\uE3FC", "\uE3FC"], /* Private Use */
  ["\uE3FD", "\uE3FD"], /* Private Use */
  ["\uE3FE", "\uE3FE"], /* Private Use */
  ["\uE3FF", "\uE3FF"], /* Private Use */
  ["\uE400", "\uE400"], /* Private Use */
  ["\uE401", "\uE401"], /* Private Use */
  ["\uE402", "\uE402"], /* Private Use */
  ["\uE403", "\uE403"], /* Private Use */
  ["\uE404", "\uE404"], /* Private Use */
  ["\uE405", "\uE405"], /* Private Use */
  ["\uE406", "\uE406"], /* Private Use */
  ["\uE407", "\uE407"], /* Private Use */
  ["\uE408", "\uE408"], /* Private Use */
  ["\uE409", "\uE409"], /* Private Use */
  ["\uE40A", "\uE40A"], /* Private Use */
  ["\uE40B", "\uE40B"], /* Private Use */
  ["\uE40C", "\uE40C"], /* Private Use */
  ["\uE40D", "\uE40D"], /* Private Use */
  ["\uE40E", "\uE40E"], /* Private Use */
  ["\uE40F", "\uE40F"], /* Private Use */
  ["\uE410", "\uE410"], /* Private Use */
  ["\uE411", "\uE411"], /* Private Use */
  ["\uE412", "\uE412"], /* Private Use */
  ["\uE413", "\uE413"], /* Private Use */
  ["\uE414", "\uE414"], /* Private Use */
  ["\uE415", "\uE415"], /* Private Use */
  ["\uE416", "\uE416"], /* Private Use */
  ["\uE417", "\uE417"], /* Private Use */
  ["\uE418", "\uE418"], /* Private Use */
  ["\uE419", "\uE419"], /* Private Use */
  ["\uE41A", "\uE41A"], /* Private Use */
  ["\uE41B", "\uE41B"], /* Private Use */
  ["\uE41C", "\uE41C"], /* Private Use */
  ["\uE41D", "\uE41D"], /* Private Use */
  ["\uE41E", "\uE41E"], /* Private Use */
  ["\uE41F", "\uE41F"], /* Private Use */
  ["\uE420", "\uE420"], /* Private Use */
  ["\uE421", "\uE421"], /* Private Use */
  ["\uE422", "\uE422"], /* Private Use */
  ["\uE423", "\uE423"], /* Private Use */
  ["\uE424", "\uE424"], /* Private Use */
  ["\uE425", "\uE425"], /* Private Use */
  ["\uE426", "\uE426"], /* Private Use */
  ["\uE427", "\uE427"], /* Private Use */
  ["\uE428", "\uE428"], /* Private Use */
  ["\uE429", "\uE429"], /* Private Use */
  ["\uE42A", "\uE42A"], /* Private Use */
  ["\uE42B", "\uE42B"], /* Private Use */
  ["\uE42C", "\uE42C"], /* Private Use */
  ["\uE42D", "\uE42D"], /* Private Use */
  ["\uE42E", "\uE42E"], /* Private Use */
  ["\uE42F", "\uE42F"], /* Private Use */
  ["\uE430", "\uE430"], /* Private Use */
  ["\uE431", "\uE431"], /* Private Use */
  ["\uE432", "\uE432"], /* Private Use */
  ["\uE433", "\uE433"], /* Private Use */
  ["\uE434", "\uE434"], /* Private Use */
  ["\uE435", "\uE435"], /* Private Use */
  ["\uE436", "\uE436"], /* Private Use */
  ["\uE437", "\uE437"], /* Private Use */
  ["\uE438", "\uE438"], /* Private Use */
  ["\uE439", "\uE439"], /* Private Use */
  ["\uE43A", "\uE43A"], /* Private Use */
  ["\uE43B", "\uE43B"], /* Private Use */
  ["\uE43C", "\uE43C"], /* Private Use */
  ["\uE43D", "\uE43D"], /* Private Use */
  ["\uE43E", "\uE43E"], /* Private Use */
  ["\uE43F", "\uE43F"], /* Private Use */
  ["\uE440", "\uE440"], /* Private Use */
  ["\uE441", "\uE441"], /* Private Use */
  ["\uE442", "\uE442"], /* Private Use */
  ["\uE443", "\uE443"], /* Private Use */
  ["\uE444", "\uE444"], /* Private Use */
  ["\uE445", "\uE445"], /* Private Use */
  ["\uE446", "\uE446"], /* Private Use */
  ["\uE447", "\uE447"], /* Private Use */
  ["\uE448", "\uE448"], /* Private Use */
  ["\uE449", "\uE449"], /* Private Use */
  ["\uE44A", "\uE44A"], /* Private Use */
  ["\uE44B", "\uE44B"], /* Private Use */
  ["\uE44C", "\uE44C"], /* Private Use */
  ["\uE44D", "\uE44D"], /* Private Use */
  ["\uE44E", "\uE44E"], /* Private Use */
  ["\uE44F", "\uE44F"], /* Private Use */
  ["\uE450", "\uE450"], /* Private Use */
  ["\uE451", "\uE451"], /* Private Use */
  ["\uE452", "\uE452"], /* Private Use */
  ["\uE453", "\uE453"], /* Private Use */
  ["\uE454", "\uE454"], /* Private Use */
  ["\uE455", "\uE455"], /* Private Use */
  ["\uE456", "\uE456"], /* Private Use */
  ["\uE457", "\uE457"], /* Private Use */
  ["\uE458", "\uE458"], /* Private Use */
  ["\uE459", "\uE459"], /* Private Use */
  ["\uE45A", "\uE45A"], /* Private Use */
  ["\uE45B", "\uE45B"], /* Private Use */
  ["\uE45C", "\uE45C"], /* Private Use */
  ["\uE45D", "\uE45D"], /* Private Use */
  ["\uE45E", "\uE45E"], /* Private Use */
  ["\uE45F", "\uE45F"], /* Private Use */
  ["\uE460", "\uE460"], /* Private Use */
  ["\uE461", "\uE461"], /* Private Use */
  ["\uE462", "\uE462"], /* Private Use */
  ["\uE463", "\uE463"], /* Private Use */
  ["\uE464", "\uE464"], /* Private Use */
  ["\uE465", "\uE465"], /* Private Use */
  ["\uE466", "\uE466"], /* Private Use */
  ["\uE467", "\uE467"], /* Private Use */
  ["\uE468", "\uE468"], /* Private Use */
  ["\uE469", "\uE469"], /* Private Use */
  ["\uE46A", "\uE46A"], /* Private Use */
  ["\uE46B", "\uE46B"], /* Private Use */
  ["\uE46C", "\uE46C"], /* Private Use */
  ["\uE46D", "\uE46D"], /* Private Use */
  ["\uE46E", "\uE46E"], /* Private Use */
  ["\uE46F", "\uE46F"], /* Private Use */
  ["\uE470", "\uE470"], /* Private Use */
  ["\uE471", "\uE471"], /* Private Use */
  ["\uE472", "\uE472"], /* Private Use */
  ["\uE473", "\uE473"], /* Private Use */
  ["\uE474", "\uE474"], /* Private Use */
  ["\uE475", "\uE475"], /* Private Use */
  ["\uE476", "\uE476"], /* Private Use */
  ["\uE477", "\uE477"], /* Private Use */
  ["\uE478", "\uE478"], /* Private Use */
  ["\uE479", "\uE479"], /* Private Use */
  ["\uE47A", "\uE47A"], /* Private Use */
  ["\uE47B", "\uE47B"], /* Private Use */
  ["\uE47C", "\uE47C"], /* Private Use */
  ["\uE47D", "\uE47D"], /* Private Use */
  ["\uE47E", "\uE47E"], /* Private Use */
  ["\uE47F", "\uE47F"], /* Private Use */
  ["\uE480", "\uE480"], /* Private Use */
  ["\uE481", "\uE481"], /* Private Use */
  ["\uE482", "\uE482"], /* Private Use */
  ["\uE483", "\uE483"], /* Private Use */
  ["\uE484", "\uE484"], /* Private Use */
  ["\uE485", "\uE485"], /* Private Use */
  ["\uE486", "\uE486"], /* Private Use */
  ["\uE487", "\uE487"], /* Private Use */
  ["\uE488", "\uE488"], /* Private Use */
  ["\uE489", "\uE489"], /* Private Use */
  ["\uE48A", "\uE48A"], /* Private Use */
  ["\uE48B", "\uE48B"], /* Private Use */
  ["\uE48C", "\uE48C"], /* Private Use */
  ["\uE48D", "\uE48D"], /* Private Use */
  ["\uE48E", "\uE48E"], /* Private Use */
  ["\uE48F", "\uE48F"], /* Private Use */
  ["\uE490", "\uE490"], /* Private Use */
  ["\uE491", "\uE491"], /* Private Use */
  ["\uE492", "\uE492"], /* Private Use */
  ["\uE493", "\uE493"], /* Private Use */
  ["\uE494", "\uE494"], /* Private Use */
  ["\uE495", "\uE495"], /* Private Use */
  ["\uE496", "\uE496"], /* Private Use */
  ["\uE497", "\uE497"], /* Private Use */
  ["\uE498", "\uE498"], /* Private Use */
  ["\uE499", "\uE499"], /* Private Use */
  ["\uE49A", "\uE49A"], /* Private Use */
  ["\uE49B", "\uE49B"], /* Private Use */
  ["\uE49C", "\uE49C"], /* Private Use */
  ["\uE49D", "\uE49D"], /* Private Use */
  ["\uE49E", "\uE49E"], /* Private Use */
  ["\uE49F", "\uE49F"], /* Private Use */
  ["\uE4A0", "\uE4A0"], /* Private Use */
  ["\uE4A1", "\uE4A1"], /* Private Use */
  ["\uE4A2", "\uE4A2"], /* Private Use */
  ["\uE4A3", "\uE4A3"], /* Private Use */
  ["\uE4A4", "\uE4A4"], /* Private Use */
  ["\uE4A5", "\uE4A5"], /* Private Use */
  ["\uE4A6", "\uE4A6"], /* Private Use */
  ["\uE4A7", "\uE4A7"], /* Private Use */
  ["\uE4A8", "\uE4A8"], /* Private Use */
  ["\uE4A9", "\uE4A9"], /* Private Use */
  ["\uE4AA", "\uE4AA"], /* Private Use */
  ["\uE4AB", "\uE4AB"], /* Private Use */
  ["\uE4AC", "\uE4AC"], /* Private Use */
  ["\uE4AD", "\uE4AD"], /* Private Use */
  ["\uE4AE", "\uE4AE"], /* Private Use */
  ["\uE4AF", "\uE4AF"], /* Private Use */
  ["\uE4B0", "\uE4B0"], /* Private Use */
  ["\uE4B1", "\uE4B1"], /* Private Use */
  ["\uE4B2", "\uE4B2"], /* Private Use */
  ["\uE4B3", "\uE4B3"], /* Private Use */
  ["\uE4B4", "\uE4B4"], /* Private Use */
  ["\uE4B5", "\uE4B5"], /* Private Use */
  ["\uE4B6", "\uE4B6"], /* Private Use */
  ["\uE4B7", "\uE4B7"], /* Private Use */
  ["\uE4B8", "\uE4B8"], /* Private Use */
  ["\uE4B9", "\uE4B9"], /* Private Use */
  ["\uE4BA", "\uE4BA"], /* Private Use */
  ["\uE4BB", "\uE4BB"], /* Private Use */
  ["\uE4BC", "\uE4BC"], /* Private Use */
  ["\uE4BD", "\uE4BD"], /* Private Use */
  ["\uE4BE", "\uE4BE"], /* Private Use */
  ["\uE4BF", "\uE4BF"], /* Private Use */
  ["\uE4C0", "\uE4C0"], /* Private Use */
  ["\uE4C1", "\uE4C1"], /* Private Use */
  ["\uE4C2", "\uE4C2"], /* Private Use */
  ["\uE4C3", "\uE4C3"], /* Private Use */
  ["\uE4C4", "\uE4C4"], /* Private Use */
  ["\uE4C5", "\uE4C5"], /* Private Use */
  ["\uE4C6", "\uE4C6"], /* Private Use */
  ["\uE4C7", "\uE4C7"], /* Private Use */
  ["\uE4C8", "\uE4C8"], /* Private Use */
  ["\uE4C9", "\uE4C9"], /* Private Use */
  ["\uE4CA", "\uE4CA"], /* Private Use */
  ["\uE4CB", "\uE4CB"], /* Private Use */
  ["\uE4CC", "\uE4CC"], /* Private Use */
  ["\uE4CD", "\uE4CD"], /* Private Use */
  ["\uE4CE", "\uE4CE"], /* Private Use */
  ["\uE4CF", "\uE4CF"], /* Private Use */
  ["\uE4D0", "\uE4D0"], /* Private Use */
  ["\uE4D1", "\uE4D1"], /* Private Use */
  ["\uE4D2", "\uE4D2"], /* Private Use */
  ["\uE4D3", "\uE4D3"], /* Private Use */
  ["\uE4D4", "\uE4D4"], /* Private Use */
  ["\uE4D5", "\uE4D5"], /* Private Use */
  ["\uE4D6", "\uE4D6"], /* Private Use */
  ["\uE4D7", "\uE4D7"], /* Private Use */
  ["\uE4D8", "\uE4D8"], /* Private Use */
  ["\uE4D9", "\uE4D9"], /* Private Use */
  ["\uE4DA", "\uE4DA"], /* Private Use */
  ["\uE4DB", "\uE4DB"], /* Private Use */
  ["\uE4DC", "\uE4DC"], /* Private Use */
  ["\uE4DD", "\uE4DD"], /* Private Use */
  ["\uE4DE", "\uE4DE"], /* Private Use */
  ["\uE4DF", "\uE4DF"], /* Private Use */
  ["\uE4E0", "\uE4E0"], /* Private Use */
  ["\uE4E1", "\uE4E1"], /* Private Use */
  ["\uE4E2", "\uE4E2"], /* Private Use */
  ["\uE4E3", "\uE4E3"], /* Private Use */
  ["\uE4E4", "\uE4E4"], /* Private Use */
  ["\uE4E5", "\uE4E5"], /* Private Use */
  ["\uE4E6", "\uE4E6"], /* Private Use */
  ["\uE4E7", "\uE4E7"], /* Private Use */
  ["\uE4E8", "\uE4E8"], /* Private Use */
  ["\uE4E9", "\uE4E9"], /* Private Use */
  ["\uE4EA", "\uE4EA"], /* Private Use */
  ["\uE4EB", "\uE4EB"], /* Private Use */
  ["\uE4EC", "\uE4EC"], /* Private Use */
  ["\uE4ED", "\uE4ED"], /* Private Use */
  ["\uE4EE", "\uE4EE"], /* Private Use */
  ["\uE4EF", "\uE4EF"], /* Private Use */
  ["\uE4F0", "\uE4F0"], /* Private Use */
  ["\uE4F1", "\uE4F1"], /* Private Use */
  ["\uE4F2", "\uE4F2"], /* Private Use */
  ["\uE4F3", "\uE4F3"], /* Private Use */
  ["\uE4F4", "\uE4F4"], /* Private Use */
  ["\uE4F5", "\uE4F5"], /* Private Use */
  ["\uE4F6", "\uE4F6"], /* Private Use */
  ["\uE4F7", "\uE4F7"], /* Private Use */
  ["\uE4F8", "\uE4F8"], /* Private Use */
  ["\uE4F9", "\uE4F9"], /* Private Use */
  ["\uE4FA", "\uE4FA"], /* Private Use */
  ["\uE4FB", "\uE4FB"], /* Private Use */
  ["\uE4FC", "\uE4FC"], /* Private Use */
  ["\uE4FD", "\uE4FD"], /* Private Use */
  ["\uE4FE", "\uE4FE"], /* Private Use */
  ["\uE4FF", "\uE4FF"], /* Private Use */
  ["\uE500", "\uE500"], /* Private Use */
  ["\uE501", "\uE501"], /* Private Use */
  ["\uE502", "\uE502"], /* Private Use */
  ["\uE503", "\uE503"], /* Private Use */
  ["\uE504", "\uE504"], /* Private Use */
  ["\uE505", "\uE505"], /* Private Use */
  ["\uE506", "\uE506"], /* Private Use */
  ["\uE507", "\uE507"], /* Private Use */
  ["\uE508", "\uE508"], /* Private Use */
  ["\uE509", "\uE509"], /* Private Use */
  ["\uE50A", "\uE50A"], /* Private Use */
  ["\uE50B", "\uE50B"], /* Private Use */
  ["\uE50C", "\uE50C"], /* Private Use */
  ["\uE50D", "\uE50D"], /* Private Use */
  ["\uE50E", "\uE50E"], /* Private Use */
  ["\uE50F", "\uE50F"], /* Private Use */
  ["\uE510", "\uE510"], /* Private Use */
  ["\uE511", "\uE511"], /* Private Use */
  ["\uE512", "\uE512"], /* Private Use */
  ["\uE513", "\uE513"], /* Private Use */
  ["\uE514", "\uE514"], /* Private Use */
  ["\uE515", "\uE515"], /* Private Use */
  ["\uE516", "\uE516"], /* Private Use */
  ["\uE517", "\uE517"], /* Private Use */
  ["\uE518", "\uE518"], /* Private Use */
  ["\uE519", "\uE519"], /* Private Use */
  ["\uE51A", "\uE51A"], /* Private Use */
  ["\uE51B", "\uE51B"], /* Private Use */
  ["\uE51C", "\uE51C"], /* Private Use */
  ["\uE51D", "\uE51D"], /* Private Use */
  ["\uE51E", "\uE51E"], /* Private Use */
  ["\uE51F", "\uE51F"], /* Private Use */
  ["\uE520", "\uE520"], /* Private Use */
  ["\uE521", "\uE521"], /* Private Use */
  ["\uE522", "\uE522"], /* Private Use */
  ["\uE523", "\uE523"], /* Private Use */
  ["\uE524", "\uE524"], /* Private Use */
  ["\uE525", "\uE525"], /* Private Use */
  ["\uE526", "\uE526"], /* Private Use */
  ["\uE527", "\uE527"], /* Private Use */
  ["\uE528", "\uE528"], /* Private Use */
  ["\uE529", "\uE529"], /* Private Use */
  ["\uE52A", "\uE52A"], /* Private Use */
  ["\uE52B", "\uE52B"], /* Private Use */
  ["\uE52C", "\uE52C"], /* Private Use */
  ["\uE52D", "\uE52D"], /* Private Use */
  ["\uE52E", "\uE52E"], /* Private Use */
  ["\uE52F", "\uE52F"], /* Private Use */
  ["\uE530", "\uE530"], /* Private Use */
  ["\uE531", "\uE531"], /* Private Use */
  ["\uE532", "\uE532"], /* Private Use */
  ["\uE533", "\uE533"], /* Private Use */
  ["\uE534", "\uE534"], /* Private Use */
  ["\uE535", "\uE535"], /* Private Use */
  ["\uE536", "\uE536"], /* Private Use */
  ["\uE537", "\uE537"], /* Private Use */
  ["\uE538", "\uE538"], /* Private Use */
  ["\uE539", "\uE539"], /* Private Use */
  ["\uE53A", "\uE53A"], /* Private Use */
  ["\uE53B", "\uE53B"], /* Private Use */
  ["\uE53C", "\uE53C"], /* Private Use */
  ["\uE53D", "\uE53D"], /* Private Use */
  ["\uE53E", "\uE53E"], /* Private Use */
  ["\uE53F", "\uE53F"], /* Private Use */
  ["\uE540", "\uE540"], /* Private Use */
  ["\uE541", "\uE541"], /* Private Use */
  ["\uE542", "\uE542"], /* Private Use */
  ["\uE543", "\uE543"], /* Private Use */
  ["\uE544", "\uE544"], /* Private Use */
  ["\uE545", "\uE545"], /* Private Use */
  ["\uE546", "\uE546"], /* Private Use */
  ["\uE547", "\uE547"], /* Private Use */
  ["\uE548", "\uE548"], /* Private Use */
  ["\uE549", "\uE549"], /* Private Use */
  ["\uE54A", "\uE54A"], /* Private Use */
  ["\uE54B", "\uE54B"], /* Private Use */
  ["\uE54C", "\uE54C"], /* Private Use */
  ["\uE54D", "\uE54D"], /* Private Use */
  ["\uE54E", "\uE54E"], /* Private Use */
  ["\uE54F", "\uE54F"], /* Private Use */
  ["\uE550", "\uE550"], /* Private Use */
  ["\uE551", "\uE551"], /* Private Use */
  ["\uE552", "\uE552"], /* Private Use */
  ["\uE553", "\uE553"], /* Private Use */
  ["\uE554", "\uE554"], /* Private Use */
  ["\uE555", "\uE555"], /* Private Use */
  ["\uE556", "\uE556"], /* Private Use */
  ["\uE557", "\uE557"], /* Private Use */
  ["\uE558", "\uE558"], /* Private Use */
  ["\uE559", "\uE559"], /* Private Use */
  ["\uE55A", "\uE55A"], /* Private Use */
  ["\uE55B", "\uE55B"], /* Private Use */
  ["\uE55C", "\uE55C"], /* Private Use */
  ["\uE55D", "\uE55D"], /* Private Use */
  ["\uE55E", "\uE55E"], /* Private Use */
  ["\uE55F", "\uE55F"], /* Private Use */
  ["\uE560", "\uE560"], /* Private Use */
  ["\uE561", "\uE561"], /* Private Use */
  ["\uE562", "\uE562"], /* Private Use */
  ["\uE563", "\uE563"], /* Private Use */
  ["\uE564", "\uE564"], /* Private Use */
  ["\uE565", "\uE565"], /* Private Use */
  ["\uE566", "\uE566"], /* Private Use */
  ["\uE567", "\uE567"], /* Private Use */
  ["\uE568", "\uE568"], /* Private Use */
  ["\uE569", "\uE569"], /* Private Use */
  ["\uE56A", "\uE56A"], /* Private Use */
  ["\uE56B", "\uE56B"], /* Private Use */
  ["\uE56C", "\uE56C"], /* Private Use */
  ["\uE56D", "\uE56D"], /* Private Use */
  ["\uE56E", "\uE56E"], /* Private Use */
  ["\uE56F", "\uE56F"], /* Private Use */
  ["\uE570", "\uE570"], /* Private Use */
  ["\uE571", "\uE571"], /* Private Use */
  ["\uE572", "\uE572"], /* Private Use */
  ["\uE573", "\uE573"], /* Private Use */
  ["\uE574", "\uE574"], /* Private Use */
  ["\uE575", "\uE575"], /* Private Use */
  ["\uE576", "\uE576"], /* Private Use */
  ["\uE577", "\uE577"], /* Private Use */
  ["\uE578", "\uE578"], /* Private Use */
  ["\uE579", "\uE579"], /* Private Use */
  ["\uE57A", "\uE57A"], /* Private Use */
  ["\uE57B", "\uE57B"], /* Private Use */
  ["\uE57C", "\uE57C"], /* Private Use */
  ["\uE57D", "\uE57D"], /* Private Use */
  ["\uE57E", "\uE57E"], /* Private Use */
  ["\uE57F", "\uE57F"], /* Private Use */
  ["\uE580", "\uE580"], /* Private Use */
  ["\uE581", "\uE581"], /* Private Use */
  ["\uE582", "\uE582"], /* Private Use */
  ["\uE583", "\uE583"], /* Private Use */
  ["\uE584", "\uE584"], /* Private Use */
  ["\uE585", "\uE585"], /* Private Use */
  ["\uE586", "\uE586"], /* Private Use */
  ["\uE587", "\uE587"], /* Private Use */
  ["\uE588", "\uE588"], /* Private Use */
  ["\uE589", "\uE589"], /* Private Use */
  ["\uE58A", "\uE58A"], /* Private Use */
  ["\uE58B", "\uE58B"], /* Private Use */
  ["\uE58C", "\uE58C"], /* Private Use */
  ["\uE58D", "\uE58D"], /* Private Use */
  ["\uE58E", "\uE58E"], /* Private Use */
  ["\uE58F", "\uE58F"], /* Private Use */
  ["\uE590", "\uE590"], /* Private Use */
  ["\uE591", "\uE591"], /* Private Use */
  ["\uE592", "\uE592"], /* Private Use */
  ["\uE593", "\uE593"], /* Private Use */
  ["\uE594", "\uE594"], /* Private Use */
  ["\uE595", "\uE595"], /* Private Use */
  ["\uE596", "\uE596"], /* Private Use */
  ["\uE597", "\uE597"], /* Private Use */
  ["\uE598", "\uE598"], /* Private Use */
  ["\uE599", "\uE599"], /* Private Use */
  ["\uE59A", "\uE59A"], /* Private Use */
  ["\uE59B", "\uE59B"], /* Private Use */
  ["\uE59C", "\uE59C"], /* Private Use */
  ["\uE59D", "\uE59D"], /* Private Use */
  ["\uE59E", "\uE59E"], /* Private Use */
  ["\uE59F", "\uE59F"], /* Private Use */
  ["\uE5A0", "\uE5A0"], /* Private Use */
  ["\uE5A1", "\uE5A1"], /* Private Use */
  ["\uE5A2", "\uE5A2"], /* Private Use */
  ["\uE5A3", "\uE5A3"], /* Private Use */
  ["\uE5A4", "\uE5A4"], /* Private Use */
  ["\uE5A5", "\uE5A5"], /* Private Use */
  ["\uE5A6", "\uE5A6"], /* Private Use */
  ["\uE5A7", "\uE5A7"], /* Private Use */
  ["\uE5A8", "\uE5A8"], /* Private Use */
  ["\uE5A9", "\uE5A9"], /* Private Use */
  ["\uE5AA", "\uE5AA"], /* Private Use */
  ["\uE5AB", "\uE5AB"], /* Private Use */
  ["\uE5AC", "\uE5AC"], /* Private Use */
  ["\uE5AD", "\uE5AD"], /* Private Use */
  ["\uE5AE", "\uE5AE"], /* Private Use */
  ["\uE5AF", "\uE5AF"], /* Private Use */
  ["\uE5B0", "\uE5B0"], /* Private Use */
  ["\uE5B1", "\uE5B1"], /* Private Use */
  ["\uE5B2", "\uE5B2"], /* Private Use */
  ["\uE5B3", "\uE5B3"], /* Private Use */
  ["\uE5B4", "\uE5B4"], /* Private Use */
  ["\uE5B5", "\uE5B5"], /* Private Use */
  ["\uE5B6", "\uE5B6"], /* Private Use */
  ["\uE5B7", "\uE5B7"], /* Private Use */
  ["\uE5B8", "\uE5B8"], /* Private Use */
  ["\uE5B9", "\uE5B9"], /* Private Use */
  ["\uE5BA", "\uE5BA"], /* Private Use */
  ["\uE5BB", "\uE5BB"], /* Private Use */
  ["\uE5BC", "\uE5BC"], /* Private Use */
  ["\uE5BD", "\uE5BD"], /* Private Use */
  ["\uE5BE", "\uE5BE"], /* Private Use */
  ["\uE5BF", "\uE5BF"], /* Private Use */
  ["\uE5C0", "\uE5C0"], /* Private Use */
  ["\uE5C1", "\uE5C1"], /* Private Use */
  ["\uE5C2", "\uE5C2"], /* Private Use */
  ["\uE5C3", "\uE5C3"], /* Private Use */
  ["\uE5C4", "\uE5C4"], /* Private Use */
  ["\uE5C5", "\uE5C5"], /* Private Use */
  ["\uE5C6", "\uE5C6"], /* Private Use */
  ["\uE5C7", "\uE5C7"], /* Private Use */
  ["\uE5C8", "\uE5C8"], /* Private Use */
  ["\uE5C9", "\uE5C9"], /* Private Use */
  ["\uE5CA", "\uE5CA"], /* Private Use */
  ["\uE5CB", "\uE5CB"], /* Private Use */
  ["\uE5CC", "\uE5CC"], /* Private Use */
  ["\uE5CD", "\uE5CD"], /* Private Use */
  ["\uE5CE", "\uE5CE"], /* Private Use */
  ["\uE5CF", "\uE5CF"], /* Private Use */
  ["\uE5D0", "\uE5D0"], /* Private Use */
  ["\uE5D1", "\uE5D1"], /* Private Use */
  ["\uE5D2", "\uE5D2"], /* Private Use */
  ["\uE5D3", "\uE5D3"], /* Private Use */
  ["\uE5D4", "\uE5D4"], /* Private Use */
  ["\uE5D5", "\uE5D5"], /* Private Use */
  ["\uE5D6", "\uE5D6"], /* Private Use */
  ["\uE5D7", "\uE5D7"], /* Private Use */
  ["\uE5D8", "\uE5D8"], /* Private Use */
  ["\uE5D9", "\uE5D9"], /* Private Use */
  ["\uE5DA", "\uE5DA"], /* Private Use */
  ["\uE5DB", "\uE5DB"], /* Private Use */
  ["\uE5DC", "\uE5DC"], /* Private Use */
  ["\uE5DD", "\uE5DD"], /* Private Use */
  ["\uE5DE", "\uE5DE"], /* Private Use */
  ["\uE5DF", "\uE5DF"], /* Private Use */
  ["\uE5E0", "\uE5E0"], /* Private Use */
  ["\uE5E1", "\uE5E1"], /* Private Use */
  ["\uE5E2", "\uE5E2"], /* Private Use */
  ["\uE5E3", "\uE5E3"], /* Private Use */
  ["\uE5E4", "\uE5E4"], /* Private Use */
  ["\uE5E5", "\uE5E5"], /* Private Use */
  ["\uE5E6", "\uE5E6"], /* Private Use */
  ["\uE5E7", "\uE5E7"], /* Private Use */
  ["\uE5E8", "\uE5E8"], /* Private Use */
  ["\uE5E9", "\uE5E9"], /* Private Use */
  ["\uE5EA", "\uE5EA"], /* Private Use */
  ["\uE5EB", "\uE5EB"], /* Private Use */
  ["\uE5EC", "\uE5EC"], /* Private Use */
  ["\uE5ED", "\uE5ED"], /* Private Use */
  ["\uE5EE", "\uE5EE"], /* Private Use */
  ["\uE5EF", "\uE5EF"], /* Private Use */
  ["\uE5F0", "\uE5F0"], /* Private Use */
  ["\uE5F1", "\uE5F1"], /* Private Use */
  ["\uE5F2", "\uE5F2"], /* Private Use */
  ["\uE5F3", "\uE5F3"], /* Private Use */
  ["\uE5F4", "\uE5F4"], /* Private Use */
  ["\uE5F5", "\uE5F5"], /* Private Use */
  ["\uE5F6", "\uE5F6"], /* Private Use */
  ["\uE5F7", "\uE5F7"], /* Private Use */
  ["\uE5F8", "\uE5F8"], /* Private Use */
  ["\uE5F9", "\uE5F9"], /* Private Use */
  ["\uE5FA", "\uE5FA"], /* Private Use */
  ["\uE5FB", "\uE5FB"], /* Private Use */
  ["\uE5FC", "\uE5FC"], /* Private Use */
  ["\uE5FD", "\uE5FD"], /* Private Use */
  ["\uE5FE", "\uE5FE"], /* Private Use */
  ["\uE5FF", "\uE5FF"], /* Private Use */
  ["\uE600", "\uE600"], /* Private Use */
  ["\uE601", "\uE601"], /* Private Use */
  ["\uE602", "\uE602"], /* Private Use */
  ["\uE603", "\uE603"], /* Private Use */
  ["\uE604", "\uE604"], /* Private Use */
  ["\uE605", "\uE605"], /* Private Use */
  ["\uE606", "\uE606"], /* Private Use */
  ["\uE607", "\uE607"], /* Private Use */
  ["\uE608", "\uE608"], /* Private Use */
  ["\uE609", "\uE609"], /* Private Use */
  ["\uE60A", "\uE60A"], /* Private Use */
  ["\uE60B", "\uE60B"], /* Private Use */
  ["\uE60C", "\uE60C"], /* Private Use */
  ["\uE60D", "\uE60D"], /* Private Use */
  ["\uE60E", "\uE60E"], /* Private Use */
  ["\uE60F", "\uE60F"], /* Private Use */
  ["\uE610", "\uE610"], /* Private Use */
  ["\uE611", "\uE611"], /* Private Use */
  ["\uE612", "\uE612"], /* Private Use */
  ["\uE613", "\uE613"], /* Private Use */
  ["\uE614", "\uE614"], /* Private Use */
  ["\uE615", "\uE615"], /* Private Use */
  ["\uE616", "\uE616"], /* Private Use */
  ["\uE617", "\uE617"], /* Private Use */
  ["\uE618", "\uE618"], /* Private Use */
  ["\uE619", "\uE619"], /* Private Use */
  ["\uE61A", "\uE61A"], /* Private Use */
  ["\uE61B", "\uE61B"], /* Private Use */
  ["\uE61C", "\uE61C"], /* Private Use */
  ["\uE61D", "\uE61D"], /* Private Use */
  ["\uE61E", "\uE61E"], /* Private Use */
  ["\uE61F", "\uE61F"], /* Private Use */
  ["\uE620", "\uE620"], /* Private Use */
  ["\uE621", "\uE621"], /* Private Use */
  ["\uE622", "\uE622"], /* Private Use */
  ["\uE623", "\uE623"], /* Private Use */
  ["\uE624", "\uE624"], /* Private Use */
  ["\uE625", "\uE625"], /* Private Use */
  ["\uE626", "\uE626"], /* Private Use */
  ["\uE627", "\uE627"], /* Private Use */
  ["\uE628", "\uE628"], /* Private Use */
  ["\uE629", "\uE629"], /* Private Use */
  ["\uE62A", "\uE62A"], /* Private Use */
  ["\uE62B", "\uE62B"], /* Private Use */
  ["\uE62C", "\uE62C"], /* Private Use */
  ["\uE62D", "\uE62D"], /* Private Use */
  ["\uE62E", "\uE62E"], /* Private Use */
  ["\uE62F", "\uE62F"], /* Private Use */
  ["\uE630", "\uE630"], /* Private Use */
  ["\uE631", "\uE631"], /* Private Use */
  ["\uE632", "\uE632"], /* Private Use */
  ["\uE633", "\uE633"], /* Private Use */
  ["\uE634", "\uE634"], /* Private Use */
  ["\uE635", "\uE635"], /* Private Use */
  ["\uE636", "\uE636"], /* Private Use */
  ["\uE637", "\uE637"], /* Private Use */
  ["\uE638", "\uE638"], /* Private Use */
  ["\uE639", "\uE639"], /* Private Use */
  ["\uE63A", "\uE63A"], /* Private Use */
  ["\uE63B", "\uE63B"], /* Private Use */
  ["\uE63C", "\uE63C"], /* Private Use */
  ["\uE63D", "\uE63D"], /* Private Use */
  ["\uE63E", "\uE63E"], /* Private Use */
  ["\uE63F", "\uE63F"], /* Private Use */
  ["\uE640", "\uE640"], /* Private Use */
  ["\uE641", "\uE641"], /* Private Use */
  ["\uE642", "\uE642"], /* Private Use */
  ["\uE643", "\uE643"], /* Private Use */
  ["\uE644", "\uE644"], /* Private Use */
  ["\uE645", "\uE645"], /* Private Use */
  ["\uE646", "\uE646"], /* Private Use */
  ["\uE647", "\uE647"], /* Private Use */
  ["\uE648", "\uE648"], /* Private Use */
  ["\uE649", "\uE649"], /* Private Use */
  ["\uE64A", "\uE64A"], /* Private Use */
  ["\uE64B", "\uE64B"], /* Private Use */
  ["\uE64C", "\uE64C"], /* Private Use */
  ["\uE64D", "\uE64D"], /* Private Use */
  ["\uE64E", "\uE64E"], /* Private Use */
  ["\uE64F", "\uE64F"], /* Private Use */
  ["\uE650", "\uE650"], /* Private Use */
  ["\uE651", "\uE651"], /* Private Use */
  ["\uE652", "\uE652"], /* Private Use */
  ["\uE653", "\uE653"], /* Private Use */
  ["\uE654", "\uE654"], /* Private Use */
  ["\uE655", "\uE655"], /* Private Use */
  ["\uE656", "\uE656"], /* Private Use */
  ["\uE657", "\uE657"], /* Private Use */
  ["\uE658", "\uE658"], /* Private Use */
  ["\uE659", "\uE659"], /* Private Use */
  ["\uE65A", "\uE65A"], /* Private Use */
  ["\uE65B", "\uE65B"], /* Private Use */
  ["\uE65C", "\uE65C"], /* Private Use */
  ["\uE65D", "\uE65D"], /* Private Use */
  ["\uE65E", "\uE65E"], /* Private Use */
  ["\uE65F", "\uE65F"], /* Private Use */
  ["\uE660", "\uE660"], /* Private Use */
  ["\uE661", "\uE661"], /* Private Use */
  ["\uE662", "\uE662"], /* Private Use */
  ["\uE663", "\uE663"], /* Private Use */
  ["\uE664", "\uE664"], /* Private Use */
  ["\uE665", "\uE665"], /* Private Use */
  ["\uE666", "\uE666"], /* Private Use */
  ["\uE667", "\uE667"], /* Private Use */
  ["\uE668", "\uE668"], /* Private Use */
  ["\uE669", "\uE669"], /* Private Use */
  ["\uE66A", "\uE66A"], /* Private Use */
  ["\uE66B", "\uE66B"], /* Private Use */
  ["\uE66C", "\uE66C"], /* Private Use */
  ["\uE66D", "\uE66D"], /* Private Use */
  ["\uE66E", "\uE66E"], /* Private Use */
  ["\uE66F", "\uE66F"], /* Private Use */
  ["\uE670", "\uE670"], /* Private Use */
  ["\uE671", "\uE671"], /* Private Use */
  ["\uE672", "\uE672"], /* Private Use */
  ["\uE673", "\uE673"], /* Private Use */
  ["\uE674", "\uE674"], /* Private Use */
  ["\uE675", "\uE675"], /* Private Use */
  ["\uE676", "\uE676"], /* Private Use */
  ["\uE677", "\uE677"], /* Private Use */
  ["\uE678", "\uE678"], /* Private Use */
  ["\uE679", "\uE679"], /* Private Use */
  ["\uE67A", "\uE67A"], /* Private Use */
  ["\uE67B", "\uE67B"], /* Private Use */
  ["\uE67C", "\uE67C"], /* Private Use */
  ["\uE67D", "\uE67D"], /* Private Use */
  ["\uE67E", "\uE67E"], /* Private Use */
  ["\uE67F", "\uE67F"], /* Private Use */
  ["\uE680", "\uE680"], /* Private Use */
  ["\uE681", "\uE681"], /* Private Use */
  ["\uE682", "\uE682"], /* Private Use */
  ["\uE683", "\uE683"], /* Private Use */
  ["\uE684", "\uE684"], /* Private Use */
  ["\uE685", "\uE685"], /* Private Use */
  ["\uE686", "\uE686"], /* Private Use */
  ["\uE687", "\uE687"], /* Private Use */
  ["\uE688", "\uE688"], /* Private Use */
  ["\uE689", "\uE689"], /* Private Use */
  ["\uE68A", "\uE68A"], /* Private Use */
  ["\uE68B", "\uE68B"], /* Private Use */
  ["\uE68C", "\uE68C"], /* Private Use */
  ["\uE68D", "\uE68D"], /* Private Use */
  ["\uE68E", "\uE68E"], /* Private Use */
  ["\uE68F", "\uE68F"], /* Private Use */
  ["\uE690", "\uE690"], /* Private Use */
  ["\uE691", "\uE691"], /* Private Use */
  ["\uE692", "\uE692"], /* Private Use */
  ["\uE693", "\uE693"], /* Private Use */
  ["\uE694", "\uE694"], /* Private Use */
  ["\uE695", "\uE695"], /* Private Use */
  ["\uE696", "\uE696"], /* Private Use */
  ["\uE697", "\uE697"], /* Private Use */
  ["\uE698", "\uE698"], /* Private Use */
  ["\uE699", "\uE699"], /* Private Use */
  ["\uE69A", "\uE69A"], /* Private Use */
  ["\uE69B", "\uE69B"], /* Private Use */
  ["\uE69C", "\uE69C"], /* Private Use */
  ["\uE69D", "\uE69D"], /* Private Use */
  ["\uE69E", "\uE69E"], /* Private Use */
  ["\uE69F", "\uE69F"], /* Private Use */
  ["\uE6A0", "\uE6A0"], /* Private Use */
  ["\uE6A1", "\uE6A1"], /* Private Use */
  ["\uE6A2", "\uE6A2"], /* Private Use */
  ["\uE6A3", "\uE6A3"], /* Private Use */
  ["\uE6A4", "\uE6A4"], /* Private Use */
  ["\uE6A5", "\uE6A5"], /* Private Use */
  ["\uE6A6", "\uE6A6"], /* Private Use */
  ["\uE6A7", "\uE6A7"], /* Private Use */
  ["\uE6A8", "\uE6A8"], /* Private Use */
  ["\uE6A9", "\uE6A9"], /* Private Use */
  ["\uE6AA", "\uE6AA"], /* Private Use */
  ["\uE6AB", "\uE6AB"], /* Private Use */
  ["\uE6AC", "\uE6AC"], /* Private Use */
  ["\uE6AD", "\uE6AD"], /* Private Use */
  ["\uE6AE", "\uE6AE"], /* Private Use */
  ["\uE6AF", "\uE6AF"], /* Private Use */
  ["\uE6B0", "\uE6B0"], /* Private Use */
  ["\uE6B1", "\uE6B1"], /* Private Use */
  ["\uE6B2", "\uE6B2"], /* Private Use */
  ["\uE6B3", "\uE6B3"], /* Private Use */
  ["\uE6B4", "\uE6B4"], /* Private Use */
  ["\uE6B5", "\uE6B5"], /* Private Use */
  ["\uE6B6", "\uE6B6"], /* Private Use */
  ["\uE6B7", "\uE6B7"], /* Private Use */
  ["\uE6B8", "\uE6B8"], /* Private Use */
  ["\uE6B9", "\uE6B9"], /* Private Use */
  ["\uE6BA", "\uE6BA"], /* Private Use */
  ["\uE6BB", "\uE6BB"], /* Private Use */
  ["\uE6BC", "\uE6BC"], /* Private Use */
  ["\uE6BD", "\uE6BD"], /* Private Use */
  ["\uE6BE", "\uE6BE"], /* Private Use */
  ["\uE6BF", "\uE6BF"], /* Private Use */
  ["\uE6C0", "\uE6C0"], /* Private Use */
  ["\uE6C1", "\uE6C1"], /* Private Use */
  ["\uE6C2", "\uE6C2"], /* Private Use */
  ["\uE6C3", "\uE6C3"], /* Private Use */
  ["\uE6C4", "\uE6C4"], /* Private Use */
  ["\uE6C5", "\uE6C5"], /* Private Use */
  ["\uE6C6", "\uE6C6"], /* Private Use */
  ["\uE6C7", "\uE6C7"], /* Private Use */
  ["\uE6C8", "\uE6C8"], /* Private Use */
  ["\uE6C9", "\uE6C9"], /* Private Use */
  ["\uE6CA", "\uE6CA"], /* Private Use */
  ["\uE6CB", "\uE6CB"], /* Private Use */
  ["\uE6CC", "\uE6CC"], /* Private Use */
  ["\uE6CD", "\uE6CD"], /* Private Use */
  ["\uE6CE", "\uE6CE"], /* Private Use */
  ["\uE6CF", "\uE6CF"], /* Private Use */
  ["\uE6D0", "\uE6D0"], /* Private Use */
  ["\uE6D1", "\uE6D1"], /* Private Use */
  ["\uE6D2", "\uE6D2"], /* Private Use */
  ["\uE6D3", "\uE6D3"], /* Private Use */
  ["\uE6D4", "\uE6D4"], /* Private Use */
  ["\uE6D5", "\uE6D5"], /* Private Use */
  ["\uE6D6", "\uE6D6"], /* Private Use */
  ["\uE6D7", "\uE6D7"], /* Private Use */
  ["\uE6D8", "\uE6D8"], /* Private Use */
  ["\uE6D9", "\uE6D9"], /* Private Use */
  ["\uE6DA", "\uE6DA"], /* Private Use */
  ["\uE6DB", "\uE6DB"], /* Private Use */
  ["\uE6DC", "\uE6DC"], /* Private Use */
  ["\uE6DD", "\uE6DD"], /* Private Use */
  ["\uE6DE", "\uE6DE"], /* Private Use */
  ["\uE6DF", "\uE6DF"], /* Private Use */
  ["\uE6E0", "\uE6E0"], /* Private Use */
  ["\uE6E1", "\uE6E1"], /* Private Use */
  ["\uE6E2", "\uE6E2"], /* Private Use */
  ["\uE6E3", "\uE6E3"], /* Private Use */
  ["\uE6E4", "\uE6E4"], /* Private Use */
  ["\uE6E5", "\uE6E5"], /* Private Use */
  ["\uE6E6", "\uE6E6"], /* Private Use */
  ["\uE6E7", "\uE6E7"], /* Private Use */
  ["\uE6E8", "\uE6E8"], /* Private Use */
  ["\uE6E9", "\uE6E9"], /* Private Use */
  ["\uE6EA", "\uE6EA"], /* Private Use */
  ["\uE6EB", "\uE6EB"], /* Private Use */
  ["\uE6EC", "\uE6EC"], /* Private Use */
  ["\uE6ED", "\uE6ED"], /* Private Use */
  ["\uE6EE", "\uE6EE"], /* Private Use */
  ["\uE6EF", "\uE6EF"], /* Private Use */
  ["\uE6F0", "\uE6F0"], /* Private Use */
  ["\uE6F1", "\uE6F1"], /* Private Use */
  ["\uE6F2", "\uE6F2"], /* Private Use */
  ["\uE6F3", "\uE6F3"], /* Private Use */
  ["\uE6F4", "\uE6F4"], /* Private Use */
  ["\uE6F5", "\uE6F5"], /* Private Use */
  ["\uE6F6", "\uE6F6"], /* Private Use */
  ["\uE6F7", "\uE6F7"], /* Private Use */
  ["\uE6F8", "\uE6F8"], /* Private Use */
  ["\uE6F9", "\uE6F9"], /* Private Use */
  ["\uE6FA", "\uE6FA"], /* Private Use */
  ["\uE6FB", "\uE6FB"], /* Private Use */
  ["\uE6FC", "\uE6FC"], /* Private Use */
  ["\uE6FD", "\uE6FD"], /* Private Use */
  ["\uE6FE", "\uE6FE"], /* Private Use */
  ["\uE6FF", "\uE6FF"], /* Private Use */
  ["\uE700", "\uE700"], /* Private Use */
  ["\uE701", "\uE701"], /* Private Use */
  ["\uE702", "\uE702"], /* Private Use */
  ["\uE703", "\uE703"], /* Private Use */
  ["\uE704", "\uE704"], /* Private Use */
  ["\uE705", "\uE705"], /* Private Use */
  ["\uE706", "\uE706"], /* Private Use */
  ["\uE707", "\uE707"], /* Private Use */
  ["\uE708", "\uE708"], /* Private Use */
  ["\uE709", "\uE709"], /* Private Use */
  ["\uE70A", "\uE70A"], /* Private Use */
  ["\uE70B", "\uE70B"], /* Private Use */
  ["\uE70C", "\uE70C"], /* Private Use */
  ["\uE70D", "\uE70D"], /* Private Use */
  ["\uE70E", "\uE70E"], /* Private Use */
  ["\uE70F", "\uE70F"], /* Private Use */
  ["\uE710", "\uE710"], /* Private Use */
  ["\uE711", "\uE711"], /* Private Use */
  ["\uE712", "\uE712"], /* Private Use */
  ["\uE713", "\uE713"], /* Private Use */
  ["\uE714", "\uE714"], /* Private Use */
  ["\uE715", "\uE715"], /* Private Use */
  ["\uE716", "\uE716"], /* Private Use */
  ["\uE717", "\uE717"], /* Private Use */
  ["\uE718", "\uE718"], /* Private Use */
  ["\uE719", "\uE719"], /* Private Use */
  ["\uE71A", "\uE71A"], /* Private Use */
  ["\uE71B", "\uE71B"], /* Private Use */
  ["\uE71C", "\uE71C"], /* Private Use */
  ["\uE71D", "\uE71D"], /* Private Use */
  ["\uE71E", "\uE71E"], /* Private Use */
  ["\uE71F", "\uE71F"], /* Private Use */
  ["\uE720", "\uE720"], /* Private Use */
  ["\uE721", "\uE721"], /* Private Use */
  ["\uE722", "\uE722"], /* Private Use */
  ["\uE723", "\uE723"], /* Private Use */
  ["\uE724", "\uE724"], /* Private Use */
  ["\uE725", "\uE725"], /* Private Use */
  ["\uE726", "\uE726"], /* Private Use */
  ["\uE727", "\uE727"], /* Private Use */
  ["\uE728", "\uE728"], /* Private Use */
  ["\uE729", "\uE729"], /* Private Use */
  ["\uE72A", "\uE72A"], /* Private Use */
  ["\uE72B", "\uE72B"], /* Private Use */
  ["\uE72C", "\uE72C"], /* Private Use */
  ["\uE72D", "\uE72D"], /* Private Use */
  ["\uE72E", "\uE72E"], /* Private Use */
  ["\uE72F", "\uE72F"], /* Private Use */
  ["\uE730", "\uE730"], /* Private Use */
  ["\uE731", "\uE731"], /* Private Use */
  ["\uE732", "\uE732"], /* Private Use */
  ["\uE733", "\uE733"], /* Private Use */
  ["\uE734", "\uE734"], /* Private Use */
  ["\uE735", "\uE735"], /* Private Use */
  ["\uE736", "\uE736"], /* Private Use */
  ["\uE737", "\uE737"], /* Private Use */
  ["\uE738", "\uE738"], /* Private Use */
  ["\uE739", "\uE739"], /* Private Use */
  ["\uE73A", "\uE73A"], /* Private Use */
  ["\uE73B", "\uE73B"], /* Private Use */
  ["\uE73C", "\uE73C"], /* Private Use */
  ["\uE73D", "\uE73D"], /* Private Use */
  ["\uE73E", "\uE73E"], /* Private Use */
  ["\uE73F", "\uE73F"], /* Private Use */
  ["\uE740", "\uE740"], /* Private Use */
  ["\uE741", "\uE741"], /* Private Use */
  ["\uE742", "\uE742"], /* Private Use */
  ["\uE743", "\uE743"], /* Private Use */
  ["\uE744", "\uE744"], /* Private Use */
  ["\uE745", "\uE745"], /* Private Use */
  ["\uE746", "\uE746"], /* Private Use */
  ["\uE747", "\uE747"], /* Private Use */
  ["\uE748", "\uE748"], /* Private Use */
  ["\uE749", "\uE749"], /* Private Use */
  ["\uE74A", "\uE74A"], /* Private Use */
  ["\uE74B", "\uE74B"], /* Private Use */
  ["\uE74C", "\uE74C"], /* Private Use */
  ["\uE74D", "\uE74D"], /* Private Use */
  ["\uE74E", "\uE74E"], /* Private Use */
  ["\uE74F", "\uE74F"], /* Private Use */
  ["\uE750", "\uE750"], /* Private Use */
  ["\uE751", "\uE751"], /* Private Use */
  ["\uE752", "\uE752"], /* Private Use */
  ["\uE753", "\uE753"], /* Private Use */
  ["\uE754", "\uE754"], /* Private Use */
  ["\uE755", "\uE755"], /* Private Use */
  ["\uE756", "\uE756"], /* Private Use */
  ["\uE757", "\uE757"], /* Private Use */
  ["\uE758", "\uE758"], /* Private Use */
  ["\uE759", "\uE759"], /* Private Use */
  ["\uE75A", "\uE75A"], /* Private Use */
  ["\uE75B", "\uE75B"], /* Private Use */
  ["\uE75C", "\uE75C"], /* Private Use */
  ["\uE75D", "\uE75D"], /* Private Use */
  ["\uE75E", "\uE75E"], /* Private Use */
  ["\uE75F", "\uE75F"], /* Private Use */
  ["\uE760", "\uE760"], /* Private Use */
  ["\uE761", "\uE761"], /* Private Use */
  ["\uE762", "\uE762"], /* Private Use */
  ["\uE763", "\uE763"], /* Private Use */
  ["\uE764", "\uE764"], /* Private Use */
  ["\uE765", "\uE765"], /* Private Use */
  ["\uE766", "\uE766"], /* Private Use */
  ["\uE767", "\uE767"], /* Private Use */
  ["\uE768", "\uE768"], /* Private Use */
  ["\uE769", "\uE769"], /* Private Use */
  ["\uE76A", "\uE76A"], /* Private Use */
  ["\uE76B", "\uE76B"], /* Private Use */
  ["\uE76C", "\uE76C"], /* Private Use */
  ["\uE76D", "\uE76D"], /* Private Use */
  ["\uE76E", "\uE76E"], /* Private Use */
  ["\uE76F", "\uE76F"], /* Private Use */
  ["\uE770", "\uE770"], /* Private Use */
  ["\uE771", "\uE771"], /* Private Use */
  ["\uE772", "\uE772"], /* Private Use */
  ["\uE773", "\uE773"], /* Private Use */
  ["\uE774", "\uE774"], /* Private Use */
  ["\uE775", "\uE775"], /* Private Use */
  ["\uE776", "\uE776"], /* Private Use */
  ["\uE777", "\uE777"], /* Private Use */
  ["\uE778", "\uE778"], /* Private Use */
  ["\uE779", "\uE779"], /* Private Use */
  ["\uE77A", "\uE77A"], /* Private Use */
  ["\uE77B", "\uE77B"], /* Private Use */
  ["\uE77C", "\uE77C"], /* Private Use */
  ["\uE77D", "\uE77D"], /* Private Use */
  ["\uE77E", "\uE77E"], /* Private Use */
  ["\uE77F", "\uE77F"], /* Private Use */
  ["\uE780", "\uE780"], /* Private Use */
  ["\uE781", "\uE781"], /* Private Use */
  ["\uE782", "\uE782"], /* Private Use */
  ["\uE783", "\uE783"], /* Private Use */
  ["\uE784", "\uE784"], /* Private Use */
  ["\uE785", "\uE785"], /* Private Use */
  ["\uE786", "\uE786"], /* Private Use */
  ["\uE787", "\uE787"], /* Private Use */
  ["\uE788", "\uE788"], /* Private Use */
  ["\uE789", "\uE789"], /* Private Use */
  ["\uE78A", "\uE78A"], /* Private Use */
  ["\uE78B", "\uE78B"], /* Private Use */
  ["\uE78C", "\uE78C"], /* Private Use */
  ["\uE78D", "\uE78D"], /* Private Use */
  ["\uE78E", "\uE78E"], /* Private Use */
  ["\uE78F", "\uE78F"], /* Private Use */
  ["\uE790", "\uE790"], /* Private Use */
  ["\uE791", "\uE791"], /* Private Use */
  ["\uE792", "\uE792"], /* Private Use */
  ["\uE793", "\uE793"], /* Private Use */
  ["\uE794", "\uE794"], /* Private Use */
  ["\uE795", "\uE795"], /* Private Use */
  ["\uE796", "\uE796"], /* Private Use */
  ["\uE797", "\uE797"], /* Private Use */
  ["\uE798", "\uE798"], /* Private Use */
  ["\uE799", "\uE799"], /* Private Use */
  ["\uE79A", "\uE79A"], /* Private Use */
  ["\uE79B", "\uE79B"], /* Private Use */
  ["\uE79C", "\uE79C"], /* Private Use */
  ["\uE79D", "\uE79D"], /* Private Use */
  ["\uE79E", "\uE79E"], /* Private Use */
  ["\uE79F", "\uE79F"], /* Private Use */
  ["\uE7A0", "\uE7A0"], /* Private Use */
  ["\uE7A1", "\uE7A1"], /* Private Use */
  ["\uE7A2", "\uE7A2"], /* Private Use */
  ["\uE7A3", "\uE7A3"], /* Private Use */
  ["\uE7A4", "\uE7A4"], /* Private Use */
  ["\uE7A5", "\uE7A5"], /* Private Use */
  ["\uE7A6", "\uE7A6"], /* Private Use */
  ["\uE7A7", "\uE7A7"], /* Private Use */
  ["\uE7A8", "\uE7A8"], /* Private Use */
  ["\uE7A9", "\uE7A9"], /* Private Use */
  ["\uE7AA", "\uE7AA"], /* Private Use */
  ["\uE7AB", "\uE7AB"], /* Private Use */
  ["\uE7AC", "\uE7AC"], /* Private Use */
  ["\uE7AD", "\uE7AD"], /* Private Use */
  ["\uE7AE", "\uE7AE"], /* Private Use */
  ["\uE7AF", "\uE7AF"], /* Private Use */
  ["\uE7B0", "\uE7B0"], /* Private Use */
  ["\uE7B1", "\uE7B1"], /* Private Use */
  ["\uE7B2", "\uE7B2"], /* Private Use */
  ["\uE7B3", "\uE7B3"], /* Private Use */
  ["\uE7B4", "\uE7B4"], /* Private Use */
  ["\uE7B5", "\uE7B5"], /* Private Use */
  ["\uE7B6", "\uE7B6"], /* Private Use */
  ["\uE7B7", "\uE7B7"], /* Private Use */
  ["\uE7B8", "\uE7B8"], /* Private Use */
  ["\uE7B9", "\uE7B9"], /* Private Use */
  ["\uE7BA", "\uE7BA"], /* Private Use */
  ["\uE7BB", "\uE7BB"], /* Private Use */
  ["\uE7BC", "\uE7BC"], /* Private Use */
  ["\uE7BD", "\uE7BD"], /* Private Use */
  ["\uE7BE", "\uE7BE"], /* Private Use */
  ["\uE7BF", "\uE7BF"], /* Private Use */
  ["\uE7C0", "\uE7C0"], /* Private Use */
  ["\uE7C1", "\uE7C1"], /* Private Use */
  ["\uE7C2", "\uE7C2"], /* Private Use */
  ["\uE7C3", "\uE7C3"], /* Private Use */
  ["\uE7C4", "\uE7C4"], /* Private Use */
  ["\uE7C5", "\uE7C5"], /* Private Use */
  ["\uE7C6", "\uE7C6"], /* Private Use */
  ["\uE7C7", "\uE7C7"], /* Private Use */
  ["\uE7C8", "\uE7C8"], /* Private Use */
  ["\uE7C9", "\uE7C9"], /* Private Use */
  ["\uE7CA", "\uE7CA"], /* Private Use */
  ["\uE7CB", "\uE7CB"], /* Private Use */
  ["\uE7CC", "\uE7CC"], /* Private Use */
  ["\uE7CD", "\uE7CD"], /* Private Use */
  ["\uE7CE", "\uE7CE"], /* Private Use */
  ["\uE7CF", "\uE7CF"], /* Private Use */
  ["\uE7D0", "\uE7D0"], /* Private Use */
  ["\uE7D1", "\uE7D1"], /* Private Use */
  ["\uE7D2", "\uE7D2"], /* Private Use */
  ["\uE7D3", "\uE7D3"], /* Private Use */
  ["\uE7D4", "\uE7D4"], /* Private Use */
  ["\uE7D5", "\uE7D5"], /* Private Use */
  ["\uE7D6", "\uE7D6"], /* Private Use */
  ["\uE7D7", "\uE7D7"], /* Private Use */
  ["\uE7D8", "\uE7D8"], /* Private Use */
  ["\uE7D9", "\uE7D9"], /* Private Use */
  ["\uE7DA", "\uE7DA"], /* Private Use */
  ["\uE7DB", "\uE7DB"], /* Private Use */
  ["\uE7DC", "\uE7DC"], /* Private Use */
  ["\uE7DD", "\uE7DD"], /* Private Use */
  ["\uE7DE", "\uE7DE"], /* Private Use */
  ["\uE7DF", "\uE7DF"], /* Private Use */
  ["\uE7E0", "\uE7E0"], /* Private Use */
  ["\uE7E1", "\uE7E1"], /* Private Use */
  ["\uE7E2", "\uE7E2"], /* Private Use */
  ["\uE7E3", "\uE7E3"], /* Private Use */
  ["\uE7E4", "\uE7E4"], /* Private Use */
  ["\uE7E5", "\uE7E5"], /* Private Use */
  ["\uE7E6", "\uE7E6"], /* Private Use */
  ["\uE7E7", "\uE7E7"], /* Private Use */
  ["\uE7E8", "\uE7E8"], /* Private Use */
  ["\uE7E9", "\uE7E9"], /* Private Use */
  ["\uE7EA", "\uE7EA"], /* Private Use */
  ["\uE7EB", "\uE7EB"], /* Private Use */
  ["\uE7EC", "\uE7EC"], /* Private Use */
  ["\uE7ED", "\uE7ED"], /* Private Use */
  ["\uE7EE", "\uE7EE"], /* Private Use */
  ["\uE7EF", "\uE7EF"], /* Private Use */
  ["\uE7F0", "\uE7F0"], /* Private Use */
  ["\uE7F1", "\uE7F1"], /* Private Use */
  ["\uE7F2", "\uE7F2"], /* Private Use */
  ["\uE7F3", "\uE7F3"], /* Private Use */
  ["\uE7F4", "\uE7F4"], /* Private Use */
  ["\uE7F5", "\uE7F5"], /* Private Use */
  ["\uE7F6", "\uE7F6"], /* Private Use */
  ["\uE7F7", "\uE7F7"], /* Private Use */
  ["\uE7F8", "\uE7F8"], /* Private Use */
  ["\uE7F9", "\uE7F9"], /* Private Use */
  ["\uE7FA", "\uE7FA"], /* Private Use */
  ["\uE7FB", "\uE7FB"], /* Private Use */
  ["\uE7FC", "\uE7FC"], /* Private Use */
  ["\uE7FD", "\uE7FD"], /* Private Use */
  ["\uE7FE", "\uE7FE"], /* Private Use */
  ["\uE7FF", "\uE7FF"], /* Private Use */
  ["\uE800", "\uE800"], /* Private Use */
  ["\uE801", "\uE801"], /* Private Use */
  ["\uE802", "\uE802"], /* Private Use */
  ["\uE803", "\uE803"], /* Private Use */
  ["\uE804", "\uE804"], /* Private Use */
  ["\uE805", "\uE805"], /* Private Use */
  ["\uE806", "\uE806"], /* Private Use */
  ["\uE807", "\uE807"], /* Private Use */
  ["\uE808", "\uE808"], /* Private Use */
  ["\uE809", "\uE809"], /* Private Use */
  ["\uE80A", "\uE80A"], /* Private Use */
  ["\uE80B", "\uE80B"], /* Private Use */
  ["\uE80C", "\uE80C"], /* Private Use */
  ["\uE80D", "\uE80D"], /* Private Use */
  ["\uE80E", "\uE80E"], /* Private Use */
  ["\uE80F", "\uE80F"], /* Private Use */
  ["\uE810", "\uE810"], /* Private Use */
  ["\uE811", "\uE811"], /* Private Use */
  ["\uE812", "\uE812"], /* Private Use */
  ["\uE813", "\uE813"], /* Private Use */
  ["\uE814", "\uE814"], /* Private Use */
  ["\uE815", "\uE815"], /* Private Use */
  ["\uE816", "\uE816"], /* Private Use */
  ["\uE817", "\uE817"], /* Private Use */
  ["\uE818", "\uE818"], /* Private Use */
  ["\uE819", "\uE819"], /* Private Use */
  ["\uE81A", "\uE81A"], /* Private Use */
  ["\uE81B", "\uE81B"], /* Private Use */
  ["\uE81C", "\uE81C"], /* Private Use */
  ["\uE81D", "\uE81D"], /* Private Use */
  ["\uE81E", "\uE81E"], /* Private Use */
  ["\uE81F", "\uE81F"], /* Private Use */
  ["\uE820", "\uE820"], /* Private Use */
  ["\uE821", "\uE821"], /* Private Use */
  ["\uE822", "\uE822"], /* Private Use */
  ["\uE823", "\uE823"], /* Private Use */
  ["\uE824", "\uE824"], /* Private Use */
  ["\uE825", "\uE825"], /* Private Use */
  ["\uE826", "\uE826"], /* Private Use */
  ["\uE827", "\uE827"], /* Private Use */
  ["\uE828", "\uE828"], /* Private Use */
  ["\uE829", "\uE829"], /* Private Use */
  ["\uE82A", "\uE82A"], /* Private Use */
  ["\uE82B", "\uE82B"], /* Private Use */
  ["\uE82C", "\uE82C"], /* Private Use */
  ["\uE82D", "\uE82D"], /* Private Use */
  ["\uE82E", "\uE82E"], /* Private Use */
  ["\uE82F", "\uE82F"], /* Private Use */
  ["\uE830", "\uE830"], /* Private Use */
  ["\uE831", "\uE831"], /* Private Use */
  ["\uE832", "\uE832"], /* Private Use */
  ["\uE833", "\uE833"], /* Private Use */
  ["\uE834", "\uE834"], /* Private Use */
  ["\uE835", "\uE835"], /* Private Use */
  ["\uE836", "\uE836"], /* Private Use */
  ["\uE837", "\uE837"], /* Private Use */
  ["\uE838", "\uE838"], /* Private Use */
  ["\uE839", "\uE839"], /* Private Use */
  ["\uE83A", "\uE83A"], /* Private Use */
  ["\uE83B", "\uE83B"], /* Private Use */
  ["\uE83C", "\uE83C"], /* Private Use */
  ["\uE83D", "\uE83D"], /* Private Use */
  ["\uE83E", "\uE83E"], /* Private Use */
  ["\uE83F", "\uE83F"], /* Private Use */
  ["\uE840", "\uE840"], /* Private Use */
  ["\uE841", "\uE841"], /* Private Use */
  ["\uE842", "\uE842"], /* Private Use */
  ["\uE843", "\uE843"], /* Private Use */
  ["\uE844", "\uE844"], /* Private Use */
  ["\uE845", "\uE845"], /* Private Use */
  ["\uE846", "\uE846"], /* Private Use */
  ["\uE847", "\uE847"], /* Private Use */
  ["\uE848", "\uE848"], /* Private Use */
  ["\uE849", "\uE849"], /* Private Use */
  ["\uE84A", "\uE84A"], /* Private Use */
  ["\uE84B", "\uE84B"], /* Private Use */
  ["\uE84C", "\uE84C"], /* Private Use */
  ["\uE84D", "\uE84D"], /* Private Use */
  ["\uE84E", "\uE84E"], /* Private Use */
  ["\uE84F", "\uE84F"], /* Private Use */
  ["\uE850", "\uE850"], /* Private Use */
  ["\uE851", "\uE851"], /* Private Use */
  ["\uE852", "\uE852"], /* Private Use */
  ["\uE853", "\uE853"], /* Private Use */
  ["\uE854", "\uE854"], /* Private Use */
  ["\uE855", "\uE855"], /* Private Use */
  ["\uE856", "\uE856"], /* Private Use */
  ["\uE857", "\uE857"], /* Private Use */
  ["\uE858", "\uE858"], /* Private Use */
  ["\uE859", "\uE859"], /* Private Use */
  ["\uE85A", "\uE85A"], /* Private Use */
  ["\uE85B", "\uE85B"], /* Private Use */
  ["\uE85C", "\uE85C"], /* Private Use */
  ["\uE85D", "\uE85D"], /* Private Use */
  ["\uE85E", "\uE85E"], /* Private Use */
  ["\uE85F", "\uE85F"], /* Private Use */
  ["\uE860", "\uE860"], /* Private Use */
  ["\uE861", "\uE861"], /* Private Use */
  ["\uE862", "\uE862"], /* Private Use */
  ["\uE863", "\uE863"], /* Private Use */
  ["\uE864", "\uE864"], /* Private Use */
  ["\uE865", "\uE865"], /* Private Use */
  ["\uE866", "\uE866"], /* Private Use */
  ["\uE867", "\uE867"], /* Private Use */
  ["\uE868", "\uE868"], /* Private Use */
  ["\uE869", "\uE869"], /* Private Use */
  ["\uE86A", "\uE86A"], /* Private Use */
  ["\uE86B", "\uE86B"], /* Private Use */
  ["\uE86C", "\uE86C"], /* Private Use */
  ["\uE86D", "\uE86D"], /* Private Use */
  ["\uE86E", "\uE86E"], /* Private Use */
  ["\uE86F", "\uE86F"], /* Private Use */
  ["\uE870", "\uE870"], /* Private Use */
  ["\uE871", "\uE871"], /* Private Use */
  ["\uE872", "\uE872"], /* Private Use */
  ["\uE873", "\uE873"], /* Private Use */
  ["\uE874", "\uE874"], /* Private Use */
  ["\uE875", "\uE875"], /* Private Use */
  ["\uE876", "\uE876"], /* Private Use */
  ["\uE877", "\uE877"], /* Private Use */
  ["\uE878", "\uE878"], /* Private Use */
  ["\uE879", "\uE879"], /* Private Use */
  ["\uE87A", "\uE87A"], /* Private Use */
  ["\uE87B", "\uE87B"], /* Private Use */
  ["\uE87C", "\uE87C"], /* Private Use */
  ["\uE87D", "\uE87D"], /* Private Use */
  ["\uE87E", "\uE87E"], /* Private Use */
  ["\uE87F", "\uE87F"], /* Private Use */
  ["\uE880", "\uE880"], /* Private Use */
  ["\uE881", "\uE881"], /* Private Use */
  ["\uE882", "\uE882"], /* Private Use */
  ["\uE883", "\uE883"], /* Private Use */
  ["\uE884", "\uE884"], /* Private Use */
  ["\uE885", "\uE885"], /* Private Use */
  ["\uE886", "\uE886"], /* Private Use */
  ["\uE887", "\uE887"], /* Private Use */
  ["\uE888", "\uE888"], /* Private Use */
  ["\uE889", "\uE889"], /* Private Use */
  ["\uE88A", "\uE88A"], /* Private Use */
  ["\uE88B", "\uE88B"], /* Private Use */
  ["\uE88C", "\uE88C"], /* Private Use */
  ["\uE88D", "\uE88D"], /* Private Use */
  ["\uE88E", "\uE88E"], /* Private Use */
  ["\uE88F", "\uE88F"], /* Private Use */
  ["\uE890", "\uE890"], /* Private Use */
  ["\uE891", "\uE891"], /* Private Use */
  ["\uE892", "\uE892"], /* Private Use */
  ["\uE893", "\uE893"], /* Private Use */
  ["\uE894", "\uE894"], /* Private Use */
  ["\uE895", "\uE895"], /* Private Use */
  ["\uE896", "\uE896"], /* Private Use */
  ["\uE897", "\uE897"], /* Private Use */
  ["\uE898", "\uE898"], /* Private Use */
  ["\uE899", "\uE899"], /* Private Use */
  ["\uE89A", "\uE89A"], /* Private Use */
  ["\uE89B", "\uE89B"], /* Private Use */
  ["\uE89C", "\uE89C"], /* Private Use */
  ["\uE89D", "\uE89D"], /* Private Use */
  ["\uE89E", "\uE89E"], /* Private Use */
  ["\uE89F", "\uE89F"], /* Private Use */
  ["\uE8A0", "\uE8A0"], /* Private Use */
  ["\uE8A1", "\uE8A1"], /* Private Use */
  ["\uE8A2", "\uE8A2"], /* Private Use */
  ["\uE8A3", "\uE8A3"], /* Private Use */
  ["\uE8A4", "\uE8A4"], /* Private Use */
  ["\uE8A5", "\uE8A5"], /* Private Use */
  ["\uE8A6", "\uE8A6"], /* Private Use */
  ["\uE8A7", "\uE8A7"], /* Private Use */
  ["\uE8A8", "\uE8A8"], /* Private Use */
  ["\uE8A9", "\uE8A9"], /* Private Use */
  ["\uE8AA", "\uE8AA"], /* Private Use */
  ["\uE8AB", "\uE8AB"], /* Private Use */
  ["\uE8AC", "\uE8AC"], /* Private Use */
  ["\uE8AD", "\uE8AD"], /* Private Use */
  ["\uE8AE", "\uE8AE"], /* Private Use */
  ["\uE8AF", "\uE8AF"], /* Private Use */
  ["\uE8B0", "\uE8B0"], /* Private Use */
  ["\uE8B1", "\uE8B1"], /* Private Use */
  ["\uE8B2", "\uE8B2"], /* Private Use */
  ["\uE8B3", "\uE8B3"], /* Private Use */
  ["\uE8B4", "\uE8B4"], /* Private Use */
  ["\uE8B5", "\uE8B5"], /* Private Use */
  ["\uE8B6", "\uE8B6"], /* Private Use */
  ["\uE8B7", "\uE8B7"], /* Private Use */
  ["\uE8B8", "\uE8B8"], /* Private Use */
  ["\uE8B9", "\uE8B9"], /* Private Use */
  ["\uE8BA", "\uE8BA"], /* Private Use */
  ["\uE8BB", "\uE8BB"], /* Private Use */
  ["\uE8BC", "\uE8BC"], /* Private Use */
  ["\uE8BD", "\uE8BD"], /* Private Use */
  ["\uE8BE", "\uE8BE"], /* Private Use */
  ["\uE8BF", "\uE8BF"], /* Private Use */
  ["\uE8C0", "\uE8C0"], /* Private Use */
  ["\uE8C1", "\uE8C1"], /* Private Use */
  ["\uE8C2", "\uE8C2"], /* Private Use */
  ["\uE8C3", "\uE8C3"], /* Private Use */
  ["\uE8C4", "\uE8C4"], /* Private Use */
  ["\uE8C5", "\uE8C5"], /* Private Use */
  ["\uE8C6", "\uE8C6"], /* Private Use */
  ["\uE8C7", "\uE8C7"], /* Private Use */
  ["\uE8C8", "\uE8C8"], /* Private Use */
  ["\uE8C9", "\uE8C9"], /* Private Use */
  ["\uE8CA", "\uE8CA"], /* Private Use */
  ["\uE8CB", "\uE8CB"], /* Private Use */
  ["\uE8CC", "\uE8CC"], /* Private Use */
  ["\uE8CD", "\uE8CD"], /* Private Use */
  ["\uE8CE", "\uE8CE"], /* Private Use */
  ["\uE8CF", "\uE8CF"], /* Private Use */
  ["\uE8D0", "\uE8D0"], /* Private Use */
  ["\uE8D1", "\uE8D1"], /* Private Use */
  ["\uE8D2", "\uE8D2"], /* Private Use */
  ["\uE8D3", "\uE8D3"], /* Private Use */
  ["\uE8D4", "\uE8D4"], /* Private Use */
  ["\uE8D5", "\uE8D5"], /* Private Use */
  ["\uE8D6", "\uE8D6"], /* Private Use */
  ["\uE8D7", "\uE8D7"], /* Private Use */
  ["\uE8D8", "\uE8D8"], /* Private Use */
  ["\uE8D9", "\uE8D9"], /* Private Use */
  ["\uE8DA", "\uE8DA"], /* Private Use */
  ["\uE8DB", "\uE8DB"], /* Private Use */
  ["\uE8DC", "\uE8DC"], /* Private Use */
  ["\uE8DD", "\uE8DD"], /* Private Use */
  ["\uE8DE", "\uE8DE"], /* Private Use */
  ["\uE8DF", "\uE8DF"], /* Private Use */
  ["\uE8E0", "\uE8E0"], /* Private Use */
  ["\uE8E1", "\uE8E1"], /* Private Use */
  ["\uE8E2", "\uE8E2"], /* Private Use */
  ["\uE8E3", "\uE8E3"], /* Private Use */
  ["\uE8E4", "\uE8E4"], /* Private Use */
  ["\uE8E5", "\uE8E5"], /* Private Use */
  ["\uE8E6", "\uE8E6"], /* Private Use */
  ["\uE8E7", "\uE8E7"], /* Private Use */
  ["\uE8E8", "\uE8E8"], /* Private Use */
  ["\uE8E9", "\uE8E9"], /* Private Use */
  ["\uE8EA", "\uE8EA"], /* Private Use */
  ["\uE8EB", "\uE8EB"], /* Private Use */
  ["\uE8EC", "\uE8EC"], /* Private Use */
  ["\uE8ED", "\uE8ED"], /* Private Use */
  ["\uE8EE", "\uE8EE"], /* Private Use */
  ["\uE8EF", "\uE8EF"], /* Private Use */
  ["\uE8F0", "\uE8F0"], /* Private Use */
  ["\uE8F1", "\uE8F1"], /* Private Use */
  ["\uE8F2", "\uE8F2"], /* Private Use */
  ["\uE8F3", "\uE8F3"], /* Private Use */
  ["\uE8F4", "\uE8F4"], /* Private Use */
  ["\uE8F5", "\uE8F5"], /* Private Use */
  ["\uE8F6", "\uE8F6"], /* Private Use */
  ["\uE8F7", "\uE8F7"], /* Private Use */
  ["\uE8F8", "\uE8F8"], /* Private Use */
  ["\uE8F9", "\uE8F9"], /* Private Use */
  ["\uE8FA", "\uE8FA"], /* Private Use */
  ["\uE8FB", "\uE8FB"], /* Private Use */
  ["\uE8FC", "\uE8FC"], /* Private Use */
  ["\uE8FD", "\uE8FD"], /* Private Use */
  ["\uE8FE", "\uE8FE"], /* Private Use */
  ["\uE8FF", "\uE8FF"], /* Private Use */
  ["\uE900", "\uE900"], /* Private Use */
  ["\uE901", "\uE901"], /* Private Use */
  ["\uE902", "\uE902"], /* Private Use */
  ["\uE903", "\uE903"], /* Private Use */
  ["\uE904", "\uE904"], /* Private Use */
  ["\uE905", "\uE905"], /* Private Use */
  ["\uE906", "\uE906"], /* Private Use */
  ["\uE907", "\uE907"], /* Private Use */
  ["\uE908", "\uE908"], /* Private Use */
  ["\uE909", "\uE909"], /* Private Use */
  ["\uE90A", "\uE90A"], /* Private Use */
  ["\uE90B", "\uE90B"], /* Private Use */
  ["\uE90C", "\uE90C"], /* Private Use */
  ["\uE90D", "\uE90D"], /* Private Use */
  ["\uE90E", "\uE90E"], /* Private Use */
  ["\uE90F", "\uE90F"], /* Private Use */
  ["\uE910", "\uE910"], /* Private Use */
  ["\uE911", "\uE911"], /* Private Use */
  ["\uE912", "\uE912"], /* Private Use */
  ["\uE913", "\uE913"], /* Private Use */
  ["\uE914", "\uE914"], /* Private Use */
  ["\uE915", "\uE915"], /* Private Use */
  ["\uE916", "\uE916"], /* Private Use */
  ["\uE917", "\uE917"], /* Private Use */
  ["\uE918", "\uE918"], /* Private Use */
  ["\uE919", "\uE919"], /* Private Use */
  ["\uE91A", "\uE91A"], /* Private Use */
  ["\uE91B", "\uE91B"], /* Private Use */
  ["\uE91C", "\uE91C"], /* Private Use */
  ["\uE91D", "\uE91D"], /* Private Use */
  ["\uE91E", "\uE91E"], /* Private Use */
  ["\uE91F", "\uE91F"], /* Private Use */
  ["\uE920", "\uE920"], /* Private Use */
  ["\uE921", "\uE921"], /* Private Use */
  ["\uE922", "\uE922"], /* Private Use */
  ["\uE923", "\uE923"], /* Private Use */
  ["\uE924", "\uE924"], /* Private Use */
  ["\uE925", "\uE925"], /* Private Use */
  ["\uE926", "\uE926"], /* Private Use */
  ["\uE927", "\uE927"], /* Private Use */
  ["\uE928", "\uE928"], /* Private Use */
  ["\uE929", "\uE929"], /* Private Use */
  ["\uE92A", "\uE92A"], /* Private Use */
  ["\uE92B", "\uE92B"], /* Private Use */
  ["\uE92C", "\uE92C"], /* Private Use */
  ["\uE92D", "\uE92D"], /* Private Use */
  ["\uE92E", "\uE92E"], /* Private Use */
  ["\uE92F", "\uE92F"], /* Private Use */
  ["\uE930", "\uE930"], /* Private Use */
  ["\uE931", "\uE931"], /* Private Use */
  ["\uE932", "\uE932"], /* Private Use */
  ["\uE933", "\uE933"], /* Private Use */
  ["\uE934", "\uE934"], /* Private Use */
  ["\uE935", "\uE935"], /* Private Use */
  ["\uE936", "\uE936"], /* Private Use */
  ["\uE937", "\uE937"], /* Private Use */
  ["\uE938", "\uE938"], /* Private Use */
  ["\uE939", "\uE939"], /* Private Use */
  ["\uE93A", "\uE93A"], /* Private Use */
  ["\uE93B", "\uE93B"], /* Private Use */
  ["\uE93C", "\uE93C"], /* Private Use */
  ["\uE93D", "\uE93D"], /* Private Use */
  ["\uE93E", "\uE93E"], /* Private Use */
  ["\uE93F", "\uE93F"], /* Private Use */
  ["\uE940", "\uE940"], /* Private Use */
  ["\uE941", "\uE941"], /* Private Use */
  ["\uE942", "\uE942"], /* Private Use */
  ["\uE943", "\uE943"], /* Private Use */
  ["\uE944", "\uE944"], /* Private Use */
  ["\uE945", "\uE945"], /* Private Use */
  ["\uE946", "\uE946"], /* Private Use */
  ["\uE947", "\uE947"], /* Private Use */
  ["\uE948", "\uE948"], /* Private Use */
  ["\uE949", "\uE949"], /* Private Use */
  ["\uE94A", "\uE94A"], /* Private Use */
  ["\uE94B", "\uE94B"], /* Private Use */
  ["\uE94C", "\uE94C"], /* Private Use */
  ["\uE94D", "\uE94D"], /* Private Use */
  ["\uE94E", "\uE94E"], /* Private Use */
  ["\uE94F", "\uE94F"], /* Private Use */
  ["\uE950", "\uE950"], /* Private Use */
  ["\uE951", "\uE951"], /* Private Use */
  ["\uE952", "\uE952"], /* Private Use */
  ["\uE953", "\uE953"], /* Private Use */
  ["\uE954", "\uE954"], /* Private Use */
  ["\uE955", "\uE955"], /* Private Use */
  ["\uE956", "\uE956"], /* Private Use */
  ["\uE957", "\uE957"], /* Private Use */
  ["\uE958", "\uE958"], /* Private Use */
  ["\uE959", "\uE959"], /* Private Use */
  ["\uE95A", "\uE95A"], /* Private Use */
  ["\uE95B", "\uE95B"], /* Private Use */
  ["\uE95C", "\uE95C"], /* Private Use */
  ["\uE95D", "\uE95D"], /* Private Use */
  ["\uE95E", "\uE95E"], /* Private Use */
  ["\uE95F", "\uE95F"], /* Private Use */
  ["\uE960", "\uE960"], /* Private Use */
  ["\uE961", "\uE961"], /* Private Use */
  ["\uE962", "\uE962"], /* Private Use */
  ["\uE963", "\uE963"], /* Private Use */
  ["\uE964", "\uE964"], /* Private Use */
  ["\uE965", "\uE965"], /* Private Use */
  ["\uE966", "\uE966"], /* Private Use */
  ["\uE967", "\uE967"], /* Private Use */
  ["\uE968", "\uE968"], /* Private Use */
  ["\uE969", "\uE969"], /* Private Use */
  ["\uE96A", "\uE96A"], /* Private Use */
  ["\uE96B", "\uE96B"], /* Private Use */
  ["\uE96C", "\uE96C"], /* Private Use */
  ["\uE96D", "\uE96D"], /* Private Use */
  ["\uE96E", "\uE96E"], /* Private Use */
  ["\uE96F", "\uE96F"], /* Private Use */
  ["\uE970", "\uE970"], /* Private Use */
  ["\uE971", "\uE971"], /* Private Use */
  ["\uE972", "\uE972"], /* Private Use */
  ["\uE973", "\uE973"], /* Private Use */
  ["\uE974", "\uE974"], /* Private Use */
  ["\uE975", "\uE975"], /* Private Use */
  ["\uE976", "\uE976"], /* Private Use */
  ["\uE977", "\uE977"], /* Private Use */
  ["\uE978", "\uE978"], /* Private Use */
  ["\uE979", "\uE979"], /* Private Use */
  ["\uE97A", "\uE97A"], /* Private Use */
  ["\uE97B", "\uE97B"], /* Private Use */
  ["\uE97C", "\uE97C"], /* Private Use */
  ["\uE97D", "\uE97D"], /* Private Use */
  ["\uE97E", "\uE97E"], /* Private Use */
  ["\uE97F", "\uE97F"], /* Private Use */
  ["\uE980", "\uE980"], /* Private Use */
  ["\uE981", "\uE981"], /* Private Use */
  ["\uE982", "\uE982"], /* Private Use */
  ["\uE983", "\uE983"], /* Private Use */
  ["\uE984", "\uE984"], /* Private Use */
  ["\uE985", "\uE985"], /* Private Use */
  ["\uE986", "\uE986"], /* Private Use */
  ["\uE987", "\uE987"], /* Private Use */
  ["\uE988", "\uE988"], /* Private Use */
  ["\uE989", "\uE989"], /* Private Use */
  ["\uE98A", "\uE98A"], /* Private Use */
  ["\uE98B", "\uE98B"], /* Private Use */
  ["\uE98C", "\uE98C"], /* Private Use */
  ["\uE98D", "\uE98D"], /* Private Use */
  ["\uE98E", "\uE98E"], /* Private Use */
  ["\uE98F", "\uE98F"], /* Private Use */
  ["\uE990", "\uE990"], /* Private Use */
  ["\uE991", "\uE991"], /* Private Use */
  ["\uE992", "\uE992"], /* Private Use */
  ["\uE993", "\uE993"], /* Private Use */
  ["\uE994", "\uE994"], /* Private Use */
  ["\uE995", "\uE995"], /* Private Use */
  ["\uE996", "\uE996"], /* Private Use */
  ["\uE997", "\uE997"], /* Private Use */
  ["\uE998", "\uE998"], /* Private Use */
  ["\uE999", "\uE999"], /* Private Use */
  ["\uE99A", "\uE99A"], /* Private Use */
  ["\uE99B", "\uE99B"], /* Private Use */
  ["\uE99C", "\uE99C"], /* Private Use */
  ["\uE99D", "\uE99D"], /* Private Use */
  ["\uE99E", "\uE99E"], /* Private Use */
  ["\uE99F", "\uE99F"], /* Private Use */
  ["\uE9A0", "\uE9A0"], /* Private Use */
  ["\uE9A1", "\uE9A1"], /* Private Use */
  ["\uE9A2", "\uE9A2"], /* Private Use */
  ["\uE9A3", "\uE9A3"], /* Private Use */
  ["\uE9A4", "\uE9A4"], /* Private Use */
  ["\uE9A5", "\uE9A5"], /* Private Use */
  ["\uE9A6", "\uE9A6"], /* Private Use */
  ["\uE9A7", "\uE9A7"], /* Private Use */
  ["\uE9A8", "\uE9A8"], /* Private Use */
  ["\uE9A9", "\uE9A9"], /* Private Use */
  ["\uE9AA", "\uE9AA"], /* Private Use */
  ["\uE9AB", "\uE9AB"], /* Private Use */
  ["\uE9AC", "\uE9AC"], /* Private Use */
  ["\uE9AD", "\uE9AD"], /* Private Use */
  ["\uE9AE", "\uE9AE"], /* Private Use */
  ["\uE9AF", "\uE9AF"], /* Private Use */
  ["\uE9B0", "\uE9B0"], /* Private Use */
  ["\uE9B1", "\uE9B1"], /* Private Use */
  ["\uE9B2", "\uE9B2"], /* Private Use */
  ["\uE9B3", "\uE9B3"], /* Private Use */
  ["\uE9B4", "\uE9B4"], /* Private Use */
  ["\uE9B5", "\uE9B5"], /* Private Use */
  ["\uE9B6", "\uE9B6"], /* Private Use */
  ["\uE9B7", "\uE9B7"], /* Private Use */
  ["\uE9B8", "\uE9B8"], /* Private Use */
  ["\uE9B9", "\uE9B9"], /* Private Use */
  ["\uE9BA", "\uE9BA"], /* Private Use */
  ["\uE9BB", "\uE9BB"], /* Private Use */
  ["\uE9BC", "\uE9BC"], /* Private Use */
  ["\uE9BD", "\uE9BD"], /* Private Use */
  ["\uE9BE", "\uE9BE"], /* Private Use */
  ["\uE9BF", "\uE9BF"], /* Private Use */
  ["\uE9C0", "\uE9C0"], /* Private Use */
  ["\uE9C1", "\uE9C1"], /* Private Use */
  ["\uE9C2", "\uE9C2"], /* Private Use */
  ["\uE9C3", "\uE9C3"], /* Private Use */
  ["\uE9C4", "\uE9C4"], /* Private Use */
  ["\uE9C5", "\uE9C5"], /* Private Use */
  ["\uE9C6", "\uE9C6"], /* Private Use */
  ["\uE9C7", "\uE9C7"], /* Private Use */
  ["\uE9C8", "\uE9C8"], /* Private Use */
  ["\uE9C9", "\uE9C9"], /* Private Use */
  ["\uE9CA", "\uE9CA"], /* Private Use */
  ["\uE9CB", "\uE9CB"], /* Private Use */
  ["\uE9CC", "\uE9CC"], /* Private Use */
  ["\uE9CD", "\uE9CD"], /* Private Use */
  ["\uE9CE", "\uE9CE"], /* Private Use */
  ["\uE9CF", "\uE9CF"], /* Private Use */
  ["\uE9D0", "\uE9D0"], /* Private Use */
  ["\uE9D1", "\uE9D1"], /* Private Use */
  ["\uE9D2", "\uE9D2"], /* Private Use */
  ["\uE9D3", "\uE9D3"], /* Private Use */
  ["\uE9D4", "\uE9D4"], /* Private Use */
  ["\uE9D5", "\uE9D5"], /* Private Use */
  ["\uE9D6", "\uE9D6"], /* Private Use */
  ["\uE9D7", "\uE9D7"], /* Private Use */
  ["\uE9D8", "\uE9D8"], /* Private Use */
  ["\uE9D9", "\uE9D9"], /* Private Use */
  ["\uE9DA", "\uE9DA"], /* Private Use */
  ["\uE9DB", "\uE9DB"], /* Private Use */
  ["\uE9DC", "\uE9DC"], /* Private Use */
  ["\uE9DD", "\uE9DD"], /* Private Use */
  ["\uE9DE", "\uE9DE"], /* Private Use */
  ["\uE9DF", "\uE9DF"], /* Private Use */
  ["\uE9E0", "\uE9E0"], /* Private Use */
  ["\uE9E1", "\uE9E1"], /* Private Use */
  ["\uE9E2", "\uE9E2"], /* Private Use */
  ["\uE9E3", "\uE9E3"], /* Private Use */
  ["\uE9E4", "\uE9E4"], /* Private Use */
  ["\uE9E5", "\uE9E5"], /* Private Use */
  ["\uE9E6", "\uE9E6"], /* Private Use */
  ["\uE9E7", "\uE9E7"], /* Private Use */
  ["\uE9E8", "\uE9E8"], /* Private Use */
  ["\uE9E9", "\uE9E9"], /* Private Use */
  ["\uE9EA", "\uE9EA"], /* Private Use */
  ["\uE9EB", "\uE9EB"], /* Private Use */
  ["\uE9EC", "\uE9EC"], /* Private Use */
  ["\uE9ED", "\uE9ED"], /* Private Use */
  ["\uE9EE", "\uE9EE"], /* Private Use */
  ["\uE9EF", "\uE9EF"], /* Private Use */
  ["\uE9F0", "\uE9F0"], /* Private Use */
  ["\uE9F1", "\uE9F1"], /* Private Use */
  ["\uE9F2", "\uE9F2"], /* Private Use */
  ["\uE9F3", "\uE9F3"], /* Private Use */
  ["\uE9F4", "\uE9F4"], /* Private Use */
  ["\uE9F5", "\uE9F5"], /* Private Use */
  ["\uE9F6", "\uE9F6"], /* Private Use */
  ["\uE9F7", "\uE9F7"], /* Private Use */
  ["\uE9F8", "\uE9F8"], /* Private Use */
  ["\uE9F9", "\uE9F9"], /* Private Use */
  ["\uE9FA", "\uE9FA"], /* Private Use */
  ["\uE9FB", "\uE9FB"], /* Private Use */
  ["\uE9FC", "\uE9FC"], /* Private Use */
  ["\uE9FD", "\uE9FD"], /* Private Use */
  ["\uE9FE", "\uE9FE"], /* Private Use */
  ["\uE9FF", "\uE9FF"], /* Private Use */
  ["\uEA00", "\uEA00"], /* Private Use */
  ["\uEA01", "\uEA01"], /* Private Use */
  ["\uEA02", "\uEA02"], /* Private Use */
  ["\uEA03", "\uEA03"], /* Private Use */
  ["\uEA04", "\uEA04"], /* Private Use */
  ["\uEA05", "\uEA05"], /* Private Use */
  ["\uEA06", "\uEA06"], /* Private Use */
  ["\uEA07", "\uEA07"], /* Private Use */
  ["\uEA08", "\uEA08"], /* Private Use */
  ["\uEA09", "\uEA09"], /* Private Use */
  ["\uEA0A", "\uEA0A"], /* Private Use */
  ["\uEA0B", "\uEA0B"], /* Private Use */
  ["\uEA0C", "\uEA0C"], /* Private Use */
  ["\uEA0D", "\uEA0D"], /* Private Use */
  ["\uEA0E", "\uEA0E"], /* Private Use */
  ["\uEA0F", "\uEA0F"], /* Private Use */
  ["\uEA10", "\uEA10"], /* Private Use */
  ["\uEA11", "\uEA11"], /* Private Use */
  ["\uEA12", "\uEA12"], /* Private Use */
  ["\uEA13", "\uEA13"], /* Private Use */
  ["\uEA14", "\uEA14"], /* Private Use */
  ["\uEA15", "\uEA15"], /* Private Use */
  ["\uEA16", "\uEA16"], /* Private Use */
  ["\uEA17", "\uEA17"], /* Private Use */
  ["\uEA18", "\uEA18"], /* Private Use */
  ["\uEA19", "\uEA19"], /* Private Use */
  ["\uEA1A", "\uEA1A"], /* Private Use */
  ["\uEA1B", "\uEA1B"], /* Private Use */
  ["\uEA1C", "\uEA1C"], /* Private Use */
  ["\uEA1D", "\uEA1D"], /* Private Use */
  ["\uEA1E", "\uEA1E"], /* Private Use */
  ["\uEA1F", "\uEA1F"], /* Private Use */
  ["\uEA20", "\uEA20"], /* Private Use */
  ["\uEA21", "\uEA21"], /* Private Use */
  ["\uEA22", "\uEA22"], /* Private Use */
  ["\uEA23", "\uEA23"], /* Private Use */
  ["\uEA24", "\uEA24"], /* Private Use */
  ["\uEA25", "\uEA25"], /* Private Use */
  ["\uEA26", "\uEA26"], /* Private Use */
  ["\uEA27", "\uEA27"], /* Private Use */
  ["\uEA28", "\uEA28"], /* Private Use */
  ["\uEA29", "\uEA29"], /* Private Use */
  ["\uEA2A", "\uEA2A"], /* Private Use */
  ["\uEA2B", "\uEA2B"], /* Private Use */
  ["\uEA2C", "\uEA2C"], /* Private Use */
  ["\uEA2D", "\uEA2D"], /* Private Use */
  ["\uEA2E", "\uEA2E"], /* Private Use */
  ["\uEA2F", "\uEA2F"], /* Private Use */
  ["\uEA30", "\uEA30"], /* Private Use */
  ["\uEA31", "\uEA31"], /* Private Use */
  ["\uEA32", "\uEA32"], /* Private Use */
  ["\uEA33", "\uEA33"], /* Private Use */
  ["\uEA34", "\uEA34"], /* Private Use */
  ["\uEA35", "\uEA35"], /* Private Use */
  ["\uEA36", "\uEA36"], /* Private Use */
  ["\uEA37", "\uEA37"], /* Private Use */
  ["\uEA38", "\uEA38"], /* Private Use */
  ["\uEA39", "\uEA39"], /* Private Use */
  ["\uEA3A", "\uEA3A"], /* Private Use */
  ["\uEA3B", "\uEA3B"], /* Private Use */
  ["\uEA3C", "\uEA3C"], /* Private Use */
  ["\uEA3D", "\uEA3D"], /* Private Use */
  ["\uEA3E", "\uEA3E"], /* Private Use */
  ["\uEA3F", "\uEA3F"], /* Private Use */
  ["\uEA40", "\uEA40"], /* Private Use */
  ["\uEA41", "\uEA41"], /* Private Use */
  ["\uEA42", "\uEA42"], /* Private Use */
  ["\uEA43", "\uEA43"], /* Private Use */
  ["\uEA44", "\uEA44"], /* Private Use */
  ["\uEA45", "\uEA45"], /* Private Use */
  ["\uEA46", "\uEA46"], /* Private Use */
  ["\uEA47", "\uEA47"], /* Private Use */
  ["\uEA48", "\uEA48"], /* Private Use */
  ["\uEA49", "\uEA49"], /* Private Use */
  ["\uEA4A", "\uEA4A"], /* Private Use */
  ["\uEA4B", "\uEA4B"], /* Private Use */
  ["\uEA4C", "\uEA4C"], /* Private Use */
  ["\uEA4D", "\uEA4D"], /* Private Use */
  ["\uEA4E", "\uEA4E"], /* Private Use */
  ["\uEA4F", "\uEA4F"], /* Private Use */
  ["\uEA50", "\uEA50"], /* Private Use */
  ["\uEA51", "\uEA51"], /* Private Use */
  ["\uEA52", "\uEA52"], /* Private Use */
  ["\uEA53", "\uEA53"], /* Private Use */
  ["\uEA54", "\uEA54"], /* Private Use */
  ["\uEA55", "\uEA55"], /* Private Use */
  ["\uEA56", "\uEA56"], /* Private Use */
  ["\uEA57", "\uEA57"], /* Private Use */
  ["\uEA58", "\uEA58"], /* Private Use */
  ["\uEA59", "\uEA59"], /* Private Use */
  ["\uEA5A", "\uEA5A"], /* Private Use */
  ["\uEA5B", "\uEA5B"], /* Private Use */
  ["\uEA5C", "\uEA5C"], /* Private Use */
  ["\uEA5D", "\uEA5D"], /* Private Use */
  ["\uEA5E", "\uEA5E"], /* Private Use */
  ["\uEA5F", "\uEA5F"], /* Private Use */
  ["\uEA60", "\uEA60"], /* Private Use */
  ["\uEA61", "\uEA61"], /* Private Use */
  ["\uEA62", "\uEA62"], /* Private Use */
  ["\uEA63", "\uEA63"], /* Private Use */
  ["\uEA64", "\uEA64"], /* Private Use */
  ["\uEA65", "\uEA65"], /* Private Use */
  ["\uEA66", "\uEA66"], /* Private Use */
  ["\uEA67", "\uEA67"], /* Private Use */
  ["\uEA68", "\uEA68"], /* Private Use */
  ["\uEA69", "\uEA69"], /* Private Use */
  ["\uEA6A", "\uEA6A"], /* Private Use */
  ["\uEA6B", "\uEA6B"], /* Private Use */
  ["\uEA6C", "\uEA6C"], /* Private Use */
  ["\uEA6D", "\uEA6D"], /* Private Use */
  ["\uEA6E", "\uEA6E"], /* Private Use */
  ["\uEA6F", "\uEA6F"], /* Private Use */
  ["\uEA70", "\uEA70"], /* Private Use */
  ["\uEA71", "\uEA71"], /* Private Use */
  ["\uEA72", "\uEA72"], /* Private Use */
  ["\uEA73", "\uEA73"], /* Private Use */
  ["\uEA74", "\uEA74"], /* Private Use */
  ["\uEA75", "\uEA75"], /* Private Use */
  ["\uEA76", "\uEA76"], /* Private Use */
  ["\uEA77", "\uEA77"], /* Private Use */
  ["\uEA78", "\uEA78"], /* Private Use */
  ["\uEA79", "\uEA79"], /* Private Use */
  ["\uEA7A", "\uEA7A"], /* Private Use */
  ["\uEA7B", "\uEA7B"], /* Private Use */
  ["\uEA7C", "\uEA7C"], /* Private Use */
  ["\uEA7D", "\uEA7D"], /* Private Use */
  ["\uEA7E", "\uEA7E"], /* Private Use */
  ["\uEA7F", "\uEA7F"], /* Private Use */
  ["\uEA80", "\uEA80"], /* Private Use */
  ["\uEA81", "\uEA81"], /* Private Use */
  ["\uEA82", "\uEA82"], /* Private Use */
  ["\uEA83", "\uEA83"], /* Private Use */
  ["\uEA84", "\uEA84"], /* Private Use */
  ["\uEA85", "\uEA85"], /* Private Use */
  ["\uEA86", "\uEA86"], /* Private Use */
  ["\uEA87", "\uEA87"], /* Private Use */
  ["\uEA88", "\uEA88"], /* Private Use */
  ["\uEA89", "\uEA89"], /* Private Use */
  ["\uEA8A", "\uEA8A"], /* Private Use */
  ["\uEA8B", "\uEA8B"], /* Private Use */
  ["\uEA8C", "\uEA8C"], /* Private Use */
  ["\uEA8D", "\uEA8D"], /* Private Use */
  ["\uEA8E", "\uEA8E"], /* Private Use */
  ["\uEA8F", "\uEA8F"], /* Private Use */
  ["\uEA90", "\uEA90"], /* Private Use */
  ["\uEA91", "\uEA91"], /* Private Use */
  ["\uEA92", "\uEA92"], /* Private Use */
  ["\uEA93", "\uEA93"], /* Private Use */
  ["\uEA94", "\uEA94"], /* Private Use */
  ["\uEA95", "\uEA95"], /* Private Use */
  ["\uEA96", "\uEA96"], /* Private Use */
  ["\uEA97", "\uEA97"], /* Private Use */
  ["\uEA98", "\uEA98"], /* Private Use */
  ["\uEA99", "\uEA99"], /* Private Use */
  ["\uEA9A", "\uEA9A"], /* Private Use */
  ["\uEA9B", "\uEA9B"], /* Private Use */
  ["\uEA9C", "\uEA9C"], /* Private Use */
  ["\uEA9D", "\uEA9D"], /* Private Use */
  ["\uEA9E", "\uEA9E"], /* Private Use */
  ["\uEA9F", "\uEA9F"], /* Private Use */
  ["\uEAA0", "\uEAA0"], /* Private Use */
  ["\uEAA1", "\uEAA1"], /* Private Use */
  ["\uEAA2", "\uEAA2"], /* Private Use */
  ["\uEAA3", "\uEAA3"], /* Private Use */
  ["\uEAA4", "\uEAA4"], /* Private Use */
  ["\uEAA5", "\uEAA5"], /* Private Use */
  ["\uEAA6", "\uEAA6"], /* Private Use */
  ["\uEAA7", "\uEAA7"], /* Private Use */
  ["\uEAA8", "\uEAA8"], /* Private Use */
  ["\uEAA9", "\uEAA9"], /* Private Use */
  ["\uEAAA", "\uEAAA"], /* Private Use */
  ["\uEAAB", "\uEAAB"], /* Private Use */
  ["\uEAAC", "\uEAAC"], /* Private Use */
  ["\uEAAD", "\uEAAD"], /* Private Use */
  ["\uEAAE", "\uEAAE"], /* Private Use */
  ["\uEAAF", "\uEAAF"], /* Private Use */
  ["\uEAB0", "\uEAB0"], /* Private Use */
  ["\uEAB1", "\uEAB1"], /* Private Use */
  ["\uEAB2", "\uEAB2"], /* Private Use */
  ["\uEAB3", "\uEAB3"], /* Private Use */
  ["\uEAB4", "\uEAB4"], /* Private Use */
  ["\uEAB5", "\uEAB5"], /* Private Use */
  ["\uEAB6", "\uEAB6"], /* Private Use */
  ["\uEAB7", "\uEAB7"], /* Private Use */
  ["\uEAB8", "\uEAB8"], /* Private Use */
  ["\uEAB9", "\uEAB9"], /* Private Use */
  ["\uEABA", "\uEABA"], /* Private Use */
  ["\uEABB", "\uEABB"], /* Private Use */
  ["\uEABC", "\uEABC"], /* Private Use */
  ["\uEABD", "\uEABD"], /* Private Use */
  ["\uEABE", "\uEABE"], /* Private Use */
  ["\uEABF", "\uEABF"], /* Private Use */
  ["\uEAC0", "\uEAC0"], /* Private Use */
  ["\uEAC1", "\uEAC1"], /* Private Use */
  ["\uEAC2", "\uEAC2"], /* Private Use */
  ["\uEAC3", "\uEAC3"], /* Private Use */
  ["\uEAC4", "\uEAC4"], /* Private Use */
  ["\uEAC5", "\uEAC5"], /* Private Use */
  ["\uEAC6", "\uEAC6"], /* Private Use */
  ["\uEAC7", "\uEAC7"], /* Private Use */
  ["\uEAC8", "\uEAC8"], /* Private Use */
  ["\uEAC9", "\uEAC9"], /* Private Use */
  ["\uEACA", "\uEACA"], /* Private Use */
  ["\uEACB", "\uEACB"], /* Private Use */
  ["\uEACC", "\uEACC"], /* Private Use */
  ["\uEACD", "\uEACD"], /* Private Use */
  ["\uEACE", "\uEACE"], /* Private Use */
  ["\uEACF", "\uEACF"], /* Private Use */
  ["\uEAD0", "\uEAD0"], /* Private Use */
  ["\uEAD1", "\uEAD1"], /* Private Use */
  ["\uEAD2", "\uEAD2"], /* Private Use */
  ["\uEAD3", "\uEAD3"], /* Private Use */
  ["\uEAD4", "\uEAD4"], /* Private Use */
  ["\uEAD5", "\uEAD5"], /* Private Use */
  ["\uEAD6", "\uEAD6"], /* Private Use */
  ["\uEAD7", "\uEAD7"], /* Private Use */
  ["\uEAD8", "\uEAD8"], /* Private Use */
  ["\uEAD9", "\uEAD9"], /* Private Use */
  ["\uEADA", "\uEADA"], /* Private Use */
  ["\uEADB", "\uEADB"], /* Private Use */
  ["\uEADC", "\uEADC"], /* Private Use */
  ["\uEADD", "\uEADD"], /* Private Use */
  ["\uEADE", "\uEADE"], /* Private Use */
  ["\uEADF", "\uEADF"], /* Private Use */
  ["\uEAE0", "\uEAE0"], /* Private Use */
  ["\uEAE1", "\uEAE1"], /* Private Use */
  ["\uEAE2", "\uEAE2"], /* Private Use */
  ["\uEAE3", "\uEAE3"], /* Private Use */
  ["\uEAE4", "\uEAE4"], /* Private Use */
  ["\uEAE5", "\uEAE5"], /* Private Use */
  ["\uEAE6", "\uEAE6"], /* Private Use */
  ["\uEAE7", "\uEAE7"], /* Private Use */
  ["\uEAE8", "\uEAE8"], /* Private Use */
  ["\uEAE9", "\uEAE9"], /* Private Use */
  ["\uEAEA", "\uEAEA"], /* Private Use */
  ["\uEAEB", "\uEAEB"], /* Private Use */
  ["\uEAEC", "\uEAEC"], /* Private Use */
  ["\uEAED", "\uEAED"], /* Private Use */
  ["\uEAEE", "\uEAEE"], /* Private Use */
  ["\uEAEF", "\uEAEF"], /* Private Use */
  ["\uEAF0", "\uEAF0"], /* Private Use */
  ["\uEAF1", "\uEAF1"], /* Private Use */
  ["\uEAF2", "\uEAF2"], /* Private Use */
  ["\uEAF3", "\uEAF3"], /* Private Use */
  ["\uEAF4", "\uEAF4"], /* Private Use */
  ["\uEAF5", "\uEAF5"], /* Private Use */
  ["\uEAF6", "\uEAF6"], /* Private Use */
  ["\uEAF7", "\uEAF7"], /* Private Use */
  ["\uEAF8", "\uEAF8"], /* Private Use */
  ["\uEAF9", "\uEAF9"], /* Private Use */
  ["\uEAFA", "\uEAFA"], /* Private Use */
  ["\uEAFB", "\uEAFB"], /* Private Use */
  ["\uEAFC", "\uEAFC"], /* Private Use */
  ["\uEAFD", "\uEAFD"], /* Private Use */
  ["\uEAFE", "\uEAFE"], /* Private Use */
  ["\uEAFF", "\uEAFF"], /* Private Use */
  ["\uEB00", "\uEB00"], /* Private Use */
  ["\uEB01", "\uEB01"], /* Private Use */
  ["\uEB02", "\uEB02"], /* Private Use */
  ["\uEB03", "\uEB03"], /* Private Use */
  ["\uEB04", "\uEB04"], /* Private Use */
  ["\uEB05", "\uEB05"], /* Private Use */
  ["\uEB06", "\uEB06"], /* Private Use */
  ["\uEB07", "\uEB07"], /* Private Use */
  ["\uEB08", "\uEB08"], /* Private Use */
  ["\uEB09", "\uEB09"], /* Private Use */
  ["\uEB0A", "\uEB0A"], /* Private Use */
  ["\uEB0B", "\uEB0B"], /* Private Use */
  ["\uEB0C", "\uEB0C"], /* Private Use */
  ["\uEB0D", "\uEB0D"], /* Private Use */
  ["\uEB0E", "\uEB0E"], /* Private Use */
  ["\uEB0F", "\uEB0F"], /* Private Use */
  ["\uEB10", "\uEB10"], /* Private Use */
  ["\uEB11", "\uEB11"], /* Private Use */
  ["\uEB12", "\uEB12"], /* Private Use */
  ["\uEB13", "\uEB13"], /* Private Use */
  ["\uEB14", "\uEB14"], /* Private Use */
  ["\uEB15", "\uEB15"], /* Private Use */
  ["\uEB16", "\uEB16"], /* Private Use */
  ["\uEB17", "\uEB17"], /* Private Use */
  ["\uEB18", "\uEB18"], /* Private Use */
  ["\uEB19", "\uEB19"], /* Private Use */
  ["\uEB1A", "\uEB1A"], /* Private Use */
  ["\uEB1B", "\uEB1B"], /* Private Use */
  ["\uEB1C", "\uEB1C"], /* Private Use */
  ["\uEB1D", "\uEB1D"], /* Private Use */
  ["\uEB1E", "\uEB1E"], /* Private Use */
  ["\uEB1F", "\uEB1F"], /* Private Use */
  ["\uEB20", "\uEB20"], /* Private Use */
  ["\uEB21", "\uEB21"], /* Private Use */
  ["\uEB22", "\uEB22"], /* Private Use */
  ["\uEB23", "\uEB23"], /* Private Use */
  ["\uEB24", "\uEB24"], /* Private Use */
  ["\uEB25", "\uEB25"], /* Private Use */
  ["\uEB26", "\uEB26"], /* Private Use */
  ["\uEB27", "\uEB27"], /* Private Use */
  ["\uEB28", "\uEB28"], /* Private Use */
  ["\uEB29", "\uEB29"], /* Private Use */
  ["\uEB2A", "\uEB2A"], /* Private Use */
  ["\uEB2B", "\uEB2B"], /* Private Use */
  ["\uEB2C", "\uEB2C"], /* Private Use */
  ["\uEB2D", "\uEB2D"], /* Private Use */
  ["\uEB2E", "\uEB2E"], /* Private Use */
  ["\uEB2F", "\uEB2F"], /* Private Use */
  ["\uEB30", "\uEB30"], /* Private Use */
  ["\uEB31", "\uEB31"], /* Private Use */
  ["\uEB32", "\uEB32"], /* Private Use */
  ["\uEB33", "\uEB33"], /* Private Use */
  ["\uEB34", "\uEB34"], /* Private Use */
  ["\uEB35", "\uEB35"], /* Private Use */
  ["\uEB36", "\uEB36"], /* Private Use */
  ["\uEB37", "\uEB37"], /* Private Use */
  ["\uEB38", "\uEB38"], /* Private Use */
  ["\uEB39", "\uEB39"], /* Private Use */
  ["\uEB3A", "\uEB3A"], /* Private Use */
  ["\uEB3B", "\uEB3B"], /* Private Use */
  ["\uEB3C", "\uEB3C"], /* Private Use */
  ["\uEB3D", "\uEB3D"], /* Private Use */
  ["\uEB3E", "\uEB3E"], /* Private Use */
  ["\uEB3F", "\uEB3F"], /* Private Use */
  ["\uEB40", "\uEB40"], /* Private Use */
  ["\uEB41", "\uEB41"], /* Private Use */
  ["\uEB42", "\uEB42"], /* Private Use */
  ["\uEB43", "\uEB43"], /* Private Use */
  ["\uEB44", "\uEB44"], /* Private Use */
  ["\uEB45", "\uEB45"], /* Private Use */
  ["\uEB46", "\uEB46"], /* Private Use */
  ["\uEB47", "\uEB47"], /* Private Use */
  ["\uEB48", "\uEB48"], /* Private Use */
  ["\uEB49", "\uEB49"], /* Private Use */
  ["\uEB4A", "\uEB4A"], /* Private Use */
  ["\uEB4B", "\uEB4B"], /* Private Use */
  ["\uEB4C", "\uEB4C"], /* Private Use */
  ["\uEB4D", "\uEB4D"], /* Private Use */
  ["\uEB4E", "\uEB4E"], /* Private Use */
  ["\uEB4F", "\uEB4F"], /* Private Use */
  ["\uEB50", "\uEB50"], /* Private Use */
  ["\uEB51", "\uEB51"], /* Private Use */
  ["\uEB52", "\uEB52"], /* Private Use */
  ["\uEB53", "\uEB53"], /* Private Use */
  ["\uEB54", "\uEB54"], /* Private Use */
  ["\uEB55", "\uEB55"], /* Private Use */
  ["\uEB56", "\uEB56"], /* Private Use */
  ["\uEB57", "\uEB57"], /* Private Use */
  ["\uEB58", "\uEB58"], /* Private Use */
  ["\uEB59", "\uEB59"], /* Private Use */
  ["\uEB5A", "\uEB5A"], /* Private Use */
  ["\uEB5B", "\uEB5B"], /* Private Use */
  ["\uEB5C", "\uEB5C"], /* Private Use */
  ["\uEB5D", "\uEB5D"], /* Private Use */
  ["\uEB5E", "\uEB5E"], /* Private Use */
  ["\uEB5F", "\uEB5F"], /* Private Use */
  ["\uEB60", "\uEB60"], /* Private Use */
  ["\uEB61", "\uEB61"], /* Private Use */
  ["\uEB62", "\uEB62"], /* Private Use */
  ["\uEB63", "\uEB63"], /* Private Use */
  ["\uEB64", "\uEB64"], /* Private Use */
  ["\uEB65", "\uEB65"], /* Private Use */
  ["\uEB66", "\uEB66"], /* Private Use */
  ["\uEB67", "\uEB67"], /* Private Use */
  ["\uEB68", "\uEB68"], /* Private Use */
  ["\uEB69", "\uEB69"], /* Private Use */
  ["\uEB6A", "\uEB6A"], /* Private Use */
  ["\uEB6B", "\uEB6B"], /* Private Use */
  ["\uEB6C", "\uEB6C"], /* Private Use */
  ["\uEB6D", "\uEB6D"], /* Private Use */
  ["\uEB6E", "\uEB6E"], /* Private Use */
  ["\uEB6F", "\uEB6F"], /* Private Use */
  ["\uEB70", "\uEB70"], /* Private Use */
  ["\uEB71", "\uEB71"], /* Private Use */
  ["\uEB72", "\uEB72"], /* Private Use */
  ["\uEB73", "\uEB73"], /* Private Use */
  ["\uEB74", "\uEB74"], /* Private Use */
  ["\uEB75", "\uEB75"], /* Private Use */
  ["\uEB76", "\uEB76"], /* Private Use */
  ["\uEB77", "\uEB77"], /* Private Use */
  ["\uEB78", "\uEB78"], /* Private Use */
  ["\uEB79", "\uEB79"], /* Private Use */
  ["\uEB7A", "\uEB7A"], /* Private Use */
  ["\uEB7B", "\uEB7B"], /* Private Use */
  ["\uEB7C", "\uEB7C"], /* Private Use */
  ["\uEB7D", "\uEB7D"], /* Private Use */
  ["\uEB7E", "\uEB7E"], /* Private Use */
  ["\uEB7F", "\uEB7F"], /* Private Use */
  ["\uEB80", "\uEB80"], /* Private Use */
  ["\uEB81", "\uEB81"], /* Private Use */
  ["\uEB82", "\uEB82"], /* Private Use */
  ["\uEB83", "\uEB83"], /* Private Use */
  ["\uEB84", "\uEB84"], /* Private Use */
  ["\uEB85", "\uEB85"], /* Private Use */
  ["\uEB86", "\uEB86"], /* Private Use */
  ["\uEB87", "\uEB87"], /* Private Use */
  ["\uEB88", "\uEB88"], /* Private Use */
  ["\uEB89", "\uEB89"], /* Private Use */
  ["\uEB8A", "\uEB8A"], /* Private Use */
  ["\uEB8B", "\uEB8B"], /* Private Use */
  ["\uEB8C", "\uEB8C"], /* Private Use */
  ["\uEB8D", "\uEB8D"], /* Private Use */
  ["\uEB8E", "\uEB8E"], /* Private Use */
  ["\uEB8F", "\uEB8F"], /* Private Use */
  ["\uEB90", "\uEB90"], /* Private Use */
  ["\uEB91", "\uEB91"], /* Private Use */
  ["\uEB92", "\uEB92"], /* Private Use */
  ["\uEB93", "\uEB93"], /* Private Use */
  ["\uEB94", "\uEB94"], /* Private Use */
  ["\uEB95", "\uEB95"], /* Private Use */
  ["\uEB96", "\uEB96"], /* Private Use */
  ["\uEB97", "\uEB97"], /* Private Use */
  ["\uEB98", "\uEB98"], /* Private Use */
  ["\uEB99", "\uEB99"], /* Private Use */
  ["\uEB9A", "\uEB9A"], /* Private Use */
  ["\uEB9B", "\uEB9B"], /* Private Use */
  ["\uEB9C", "\uEB9C"], /* Private Use */
  ["\uEB9D", "\uEB9D"], /* Private Use */
  ["\uEB9E", "\uEB9E"], /* Private Use */
  ["\uEB9F", "\uEB9F"], /* Private Use */
  ["\uEBA0", "\uEBA0"], /* Private Use */
  ["\uEBA1", "\uEBA1"], /* Private Use */
  ["\uEBA2", "\uEBA2"], /* Private Use */
  ["\uEBA3", "\uEBA3"], /* Private Use */
  ["\uEBA4", "\uEBA4"], /* Private Use */
  ["\uEBA5", "\uEBA5"], /* Private Use */
  ["\uEBA6", "\uEBA6"], /* Private Use */
  ["\uEBA7", "\uEBA7"], /* Private Use */
  ["\uEBA8", "\uEBA8"], /* Private Use */
  ["\uEBA9", "\uEBA9"], /* Private Use */
  ["\uEBAA", "\uEBAA"], /* Private Use */
  ["\uEBAB", "\uEBAB"], /* Private Use */
  ["\uEBAC", "\uEBAC"], /* Private Use */
  ["\uEBAD", "\uEBAD"], /* Private Use */
  ["\uEBAE", "\uEBAE"], /* Private Use */
  ["\uEBAF", "\uEBAF"], /* Private Use */
  ["\uEBB0", "\uEBB0"], /* Private Use */
  ["\uEBB1", "\uEBB1"], /* Private Use */
  ["\uEBB2", "\uEBB2"], /* Private Use */
  ["\uEBB3", "\uEBB3"], /* Private Use */
  ["\uEBB4", "\uEBB4"], /* Private Use */
  ["\uEBB5", "\uEBB5"], /* Private Use */
  ["\uEBB6", "\uEBB6"], /* Private Use */
  ["\uEBB7", "\uEBB7"], /* Private Use */
  ["\uEBB8", "\uEBB8"], /* Private Use */
  ["\uEBB9", "\uEBB9"], /* Private Use */
  ["\uEBBA", "\uEBBA"], /* Private Use */
  ["\uEBBB", "\uEBBB"], /* Private Use */
  ["\uEBBC", "\uEBBC"], /* Private Use */
  ["\uEBBD", "\uEBBD"], /* Private Use */
  ["\uEBBE", "\uEBBE"], /* Private Use */
  ["\uEBBF", "\uEBBF"], /* Private Use */
  ["\uEBC0", "\uEBC0"], /* Private Use */
  ["\uEBC1", "\uEBC1"], /* Private Use */
  ["\uEBC2", "\uEBC2"], /* Private Use */
  ["\uEBC3", "\uEBC3"], /* Private Use */
  ["\uEBC4", "\uEBC4"], /* Private Use */
  ["\uEBC5", "\uEBC5"], /* Private Use */
  ["\uEBC6", "\uEBC6"], /* Private Use */
  ["\uEBC7", "\uEBC7"], /* Private Use */
  ["\uEBC8", "\uEBC8"], /* Private Use */
  ["\uEBC9", "\uEBC9"], /* Private Use */
  ["\uEBCA", "\uEBCA"], /* Private Use */
  ["\uEBCB", "\uEBCB"], /* Private Use */
  ["\uEBCC", "\uEBCC"], /* Private Use */
  ["\uEBCD", "\uEBCD"], /* Private Use */
  ["\uEBCE", "\uEBCE"], /* Private Use */
  ["\uEBCF", "\uEBCF"], /* Private Use */
  ["\uEBD0", "\uEBD0"], /* Private Use */
  ["\uEBD1", "\uEBD1"], /* Private Use */
  ["\uEBD2", "\uEBD2"], /* Private Use */
  ["\uEBD3", "\uEBD3"], /* Private Use */
  ["\uEBD4", "\uEBD4"], /* Private Use */
  ["\uEBD5", "\uEBD5"], /* Private Use */
  ["\uEBD6", "\uEBD6"], /* Private Use */
  ["\uEBD7", "\uEBD7"], /* Private Use */
  ["\uEBD8", "\uEBD8"], /* Private Use */
  ["\uEBD9", "\uEBD9"], /* Private Use */
  ["\uEBDA", "\uEBDA"], /* Private Use */
  ["\uEBDB", "\uEBDB"], /* Private Use */
  ["\uEBDC", "\uEBDC"], /* Private Use */
  ["\uEBDD", "\uEBDD"], /* Private Use */
  ["\uEBDE", "\uEBDE"], /* Private Use */
  ["\uEBDF", "\uEBDF"], /* Private Use */
  ["\uEBE0", "\uEBE0"], /* Private Use */
  ["\uEBE1", "\uEBE1"], /* Private Use */
  ["\uEBE2", "\uEBE2"], /* Private Use */
  ["\uEBE3", "\uEBE3"], /* Private Use */
  ["\uEBE4", "\uEBE4"], /* Private Use */
  ["\uEBE5", "\uEBE5"], /* Private Use */
  ["\uEBE6", "\uEBE6"], /* Private Use */
  ["\uEBE7", "\uEBE7"], /* Private Use */
  ["\uEBE8", "\uEBE8"], /* Private Use */
  ["\uEBE9", "\uEBE9"], /* Private Use */
  ["\uEBEA", "\uEBEA"], /* Private Use */
  ["\uEBEB", "\uEBEB"], /* Private Use */
  ["\uEBEC", "\uEBEC"], /* Private Use */
  ["\uEBED", "\uEBED"], /* Private Use */
  ["\uEBEE", "\uEBEE"], /* Private Use */
  ["\uEBEF", "\uEBEF"], /* Private Use */
  ["\uEBF0", "\uEBF0"], /* Private Use */
  ["\uEBF1", "\uEBF1"], /* Private Use */
  ["\uEBF2", "\uEBF2"], /* Private Use */
  ["\uEBF3", "\uEBF3"], /* Private Use */
  ["\uEBF4", "\uEBF4"], /* Private Use */
  ["\uEBF5", "\uEBF5"], /* Private Use */
  ["\uEBF6", "\uEBF6"], /* Private Use */
  ["\uEBF7", "\uEBF7"], /* Private Use */
  ["\uEBF8", "\uEBF8"], /* Private Use */
  ["\uEBF9", "\uEBF9"], /* Private Use */
  ["\uEBFA", "\uEBFA"], /* Private Use */
  ["\uEBFB", "\uEBFB"], /* Private Use */
  ["\uEBFC", "\uEBFC"], /* Private Use */
  ["\uEBFD", "\uEBFD"], /* Private Use */
  ["\uEBFE", "\uEBFE"], /* Private Use */
  ["\uEBFF", "\uEBFF"], /* Private Use */
  ["\uEC00", "\uEC00"], /* Private Use */
  ["\uEC01", "\uEC01"], /* Private Use */
  ["\uEC02", "\uEC02"], /* Private Use */
  ["\uEC03", "\uEC03"], /* Private Use */
  ["\uEC04", "\uEC04"], /* Private Use */
  ["\uEC05", "\uEC05"], /* Private Use */
  ["\uEC06", "\uEC06"], /* Private Use */
  ["\uEC07", "\uEC07"], /* Private Use */
  ["\uEC08", "\uEC08"], /* Private Use */
  ["\uEC09", "\uEC09"], /* Private Use */
  ["\uEC0A", "\uEC0A"], /* Private Use */
  ["\uEC0B", "\uEC0B"], /* Private Use */
  ["\uEC0C", "\uEC0C"], /* Private Use */
  ["\uEC0D", "\uEC0D"], /* Private Use */
  ["\uEC0E", "\uEC0E"], /* Private Use */
  ["\uEC0F", "\uEC0F"], /* Private Use */
  ["\uEC10", "\uEC10"], /* Private Use */
  ["\uEC11", "\uEC11"], /* Private Use */
  ["\uEC12", "\uEC12"], /* Private Use */
  ["\uEC13", "\uEC13"], /* Private Use */
  ["\uEC14", "\uEC14"], /* Private Use */
  ["\uEC15", "\uEC15"], /* Private Use */
  ["\uEC16", "\uEC16"], /* Private Use */
  ["\uEC17", "\uEC17"], /* Private Use */
  ["\uEC18", "\uEC18"], /* Private Use */
  ["\uEC19", "\uEC19"], /* Private Use */
  ["\uEC1A", "\uEC1A"], /* Private Use */
  ["\uEC1B", "\uEC1B"], /* Private Use */
  ["\uEC1C", "\uEC1C"], /* Private Use */
  ["\uEC1D", "\uEC1D"], /* Private Use */
  ["\uEC1E", "\uEC1E"], /* Private Use */
  ["\uEC1F", "\uEC1F"], /* Private Use */
  ["\uEC20", "\uEC20"], /* Private Use */
  ["\uEC21", "\uEC21"], /* Private Use */
  ["\uEC22", "\uEC22"], /* Private Use */
  ["\uEC23", "\uEC23"], /* Private Use */
  ["\uEC24", "\uEC24"], /* Private Use */
  ["\uEC25", "\uEC25"], /* Private Use */
  ["\uEC26", "\uEC26"], /* Private Use */
  ["\uEC27", "\uEC27"], /* Private Use */
  ["\uEC28", "\uEC28"], /* Private Use */
  ["\uEC29", "\uEC29"], /* Private Use */
  ["\uEC2A", "\uEC2A"], /* Private Use */
  ["\uEC2B", "\uEC2B"], /* Private Use */
  ["\uEC2C", "\uEC2C"], /* Private Use */
  ["\uEC2D", "\uEC2D"], /* Private Use */
  ["\uEC2E", "\uEC2E"], /* Private Use */
  ["\uEC2F", "\uEC2F"], /* Private Use */
  ["\uEC30", "\uEC30"], /* Private Use */
  ["\uEC31", "\uEC31"], /* Private Use */
  ["\uEC32", "\uEC32"], /* Private Use */
  ["\uEC33", "\uEC33"], /* Private Use */
  ["\uEC34", "\uEC34"], /* Private Use */
  ["\uEC35", "\uEC35"], /* Private Use */
  ["\uEC36", "\uEC36"], /* Private Use */
  ["\uEC37", "\uEC37"], /* Private Use */
  ["\uEC38", "\uEC38"], /* Private Use */
  ["\uEC39", "\uEC39"], /* Private Use */
  ["\uEC3A", "\uEC3A"], /* Private Use */
  ["\uEC3B", "\uEC3B"], /* Private Use */
  ["\uEC3C", "\uEC3C"], /* Private Use */
  ["\uEC3D", "\uEC3D"], /* Private Use */
  ["\uEC3E", "\uEC3E"], /* Private Use */
  ["\uEC3F", "\uEC3F"], /* Private Use */
  ["\uEC40", "\uEC40"], /* Private Use */
  ["\uEC41", "\uEC41"], /* Private Use */
  ["\uEC42", "\uEC42"], /* Private Use */
  ["\uEC43", "\uEC43"], /* Private Use */
  ["\uEC44", "\uEC44"], /* Private Use */
  ["\uEC45", "\uEC45"], /* Private Use */
  ["\uEC46", "\uEC46"], /* Private Use */
  ["\uEC47", "\uEC47"], /* Private Use */
  ["\uEC48", "\uEC48"], /* Private Use */
  ["\uEC49", "\uEC49"], /* Private Use */
  ["\uEC4A", "\uEC4A"], /* Private Use */
  ["\uEC4B", "\uEC4B"], /* Private Use */
  ["\uEC4C", "\uEC4C"], /* Private Use */
  ["\uEC4D", "\uEC4D"], /* Private Use */
  ["\uEC4E", "\uEC4E"], /* Private Use */
  ["\uEC4F", "\uEC4F"], /* Private Use */
  ["\uEC50", "\uEC50"], /* Private Use */
  ["\uEC51", "\uEC51"], /* Private Use */
  ["\uEC52", "\uEC52"], /* Private Use */
  ["\uEC53", "\uEC53"], /* Private Use */
  ["\uEC54", "\uEC54"], /* Private Use */
  ["\uEC55", "\uEC55"], /* Private Use */
  ["\uEC56", "\uEC56"], /* Private Use */
  ["\uEC57", "\uEC57"], /* Private Use */
  ["\uEC58", "\uEC58"], /* Private Use */
  ["\uEC59", "\uEC59"], /* Private Use */
  ["\uEC5A", "\uEC5A"], /* Private Use */
  ["\uEC5B", "\uEC5B"], /* Private Use */
  ["\uEC5C", "\uEC5C"], /* Private Use */
  ["\uEC5D", "\uEC5D"], /* Private Use */
  ["\uEC5E", "\uEC5E"], /* Private Use */
  ["\uEC5F", "\uEC5F"], /* Private Use */
  ["\uEC60", "\uEC60"], /* Private Use */
  ["\uEC61", "\uEC61"], /* Private Use */
  ["\uEC62", "\uEC62"], /* Private Use */
  ["\uEC63", "\uEC63"], /* Private Use */
  ["\uEC64", "\uEC64"], /* Private Use */
  ["\uEC65", "\uEC65"], /* Private Use */
  ["\uEC66", "\uEC66"], /* Private Use */
  ["\uEC67", "\uEC67"], /* Private Use */
  ["\uEC68", "\uEC68"], /* Private Use */
  ["\uEC69", "\uEC69"], /* Private Use */
  ["\uEC6A", "\uEC6A"], /* Private Use */
  ["\uEC6B", "\uEC6B"], /* Private Use */
  ["\uEC6C", "\uEC6C"], /* Private Use */
  ["\uEC6D", "\uEC6D"], /* Private Use */
  ["\uEC6E", "\uEC6E"], /* Private Use */
  ["\uEC6F", "\uEC6F"], /* Private Use */
  ["\uEC70", "\uEC70"], /* Private Use */
  ["\uEC71", "\uEC71"], /* Private Use */
  ["\uEC72", "\uEC72"], /* Private Use */
  ["\uEC73", "\uEC73"], /* Private Use */
  ["\uEC74", "\uEC74"], /* Private Use */
  ["\uEC75", "\uEC75"], /* Private Use */
  ["\uEC76", "\uEC76"], /* Private Use */
  ["\uEC77", "\uEC77"], /* Private Use */
  ["\uEC78", "\uEC78"], /* Private Use */
  ["\uEC79", "\uEC79"], /* Private Use */
  ["\uEC7A", "\uEC7A"], /* Private Use */
  ["\uEC7B", "\uEC7B"], /* Private Use */
  ["\uEC7C", "\uEC7C"], /* Private Use */
  ["\uEC7D", "\uEC7D"], /* Private Use */
  ["\uEC7E", "\uEC7E"], /* Private Use */
  ["\uEC7F", "\uEC7F"], /* Private Use */
  ["\uEC80", "\uEC80"], /* Private Use */
  ["\uEC81", "\uEC81"], /* Private Use */
  ["\uEC82", "\uEC82"], /* Private Use */
  ["\uEC83", "\uEC83"], /* Private Use */
  ["\uEC84", "\uEC84"], /* Private Use */
  ["\uEC85", "\uEC85"], /* Private Use */
  ["\uEC86", "\uEC86"], /* Private Use */
  ["\uEC87", "\uEC87"], /* Private Use */
  ["\uEC88", "\uEC88"], /* Private Use */
  ["\uEC89", "\uEC89"], /* Private Use */
  ["\uEC8A", "\uEC8A"], /* Private Use */
  ["\uEC8B", "\uEC8B"], /* Private Use */
  ["\uEC8C", "\uEC8C"], /* Private Use */
  ["\uEC8D", "\uEC8D"], /* Private Use */
  ["\uEC8E", "\uEC8E"], /* Private Use */
  ["\uEC8F", "\uEC8F"], /* Private Use */
  ["\uEC90", "\uEC90"], /* Private Use */
  ["\uEC91", "\uEC91"], /* Private Use */
  ["\uEC92", "\uEC92"], /* Private Use */
  ["\uEC93", "\uEC93"], /* Private Use */
  ["\uEC94", "\uEC94"], /* Private Use */
  ["\uEC95", "\uEC95"], /* Private Use */
  ["\uEC96", "\uEC96"], /* Private Use */
  ["\uEC97", "\uEC97"], /* Private Use */
  ["\uEC98", "\uEC98"], /* Private Use */
  ["\uEC99", "\uEC99"], /* Private Use */
  ["\uEC9A", "\uEC9A"], /* Private Use */
  ["\uEC9B", "\uEC9B"], /* Private Use */
  ["\uEC9C", "\uEC9C"], /* Private Use */
  ["\uEC9D", "\uEC9D"], /* Private Use */
  ["\uEC9E", "\uEC9E"], /* Private Use */
  ["\uEC9F", "\uEC9F"], /* Private Use */
  ["\uECA0", "\uECA0"], /* Private Use */
  ["\uECA1", "\uECA1"], /* Private Use */
  ["\uECA2", "\uECA2"], /* Private Use */
  ["\uECA3", "\uECA3"], /* Private Use */
  ["\uECA4", "\uECA4"], /* Private Use */
  ["\uECA5", "\uECA5"], /* Private Use */
  ["\uECA6", "\uECA6"], /* Private Use */
  ["\uECA7", "\uECA7"], /* Private Use */
  ["\uECA8", "\uECA8"], /* Private Use */
  ["\uECA9", "\uECA9"], /* Private Use */
  ["\uECAA", "\uECAA"], /* Private Use */
  ["\uECAB", "\uECAB"], /* Private Use */
  ["\uECAC", "\uECAC"], /* Private Use */
  ["\uECAD", "\uECAD"], /* Private Use */
  ["\uECAE", "\uECAE"], /* Private Use */
  ["\uECAF", "\uECAF"], /* Private Use */
  ["\uECB0", "\uECB0"], /* Private Use */
  ["\uECB1", "\uECB1"], /* Private Use */
  ["\uECB2", "\uECB2"], /* Private Use */
  ["\uECB3", "\uECB3"], /* Private Use */
  ["\uECB4", "\uECB4"], /* Private Use */
  ["\uECB5", "\uECB5"], /* Private Use */
  ["\uECB6", "\uECB6"], /* Private Use */
  ["\uECB7", "\uECB7"], /* Private Use */
  ["\uECB8", "\uECB8"], /* Private Use */
  ["\uECB9", "\uECB9"], /* Private Use */
  ["\uECBA", "\uECBA"], /* Private Use */
  ["\uECBB", "\uECBB"], /* Private Use */
  ["\uECBC", "\uECBC"], /* Private Use */
  ["\uECBD", "\uECBD"], /* Private Use */
  ["\uECBE", "\uECBE"], /* Private Use */
  ["\uECBF", "\uECBF"], /* Private Use */
  ["\uECC0", "\uECC0"], /* Private Use */
  ["\uECC1", "\uECC1"], /* Private Use */
  ["\uECC2", "\uECC2"], /* Private Use */
  ["\uECC3", "\uECC3"], /* Private Use */
  ["\uECC4", "\uECC4"], /* Private Use */
  ["\uECC5", "\uECC5"], /* Private Use */
  ["\uECC6", "\uECC6"], /* Private Use */
  ["\uECC7", "\uECC7"], /* Private Use */
  ["\uECC8", "\uECC8"], /* Private Use */
  ["\uECC9", "\uECC9"], /* Private Use */
  ["\uECCA", "\uECCA"], /* Private Use */
  ["\uECCB", "\uECCB"], /* Private Use */
  ["\uECCC", "\uECCC"], /* Private Use */
  ["\uECCD", "\uECCD"], /* Private Use */
  ["\uECCE", "\uECCE"], /* Private Use */
  ["\uECCF", "\uECCF"], /* Private Use */
  ["\uECD0", "\uECD0"], /* Private Use */
  ["\uECD1", "\uECD1"], /* Private Use */
  ["\uECD2", "\uECD2"], /* Private Use */
  ["\uECD3", "\uECD3"], /* Private Use */
  ["\uECD4", "\uECD4"], /* Private Use */
  ["\uECD5", "\uECD5"], /* Private Use */
  ["\uECD6", "\uECD6"], /* Private Use */
  ["\uECD7", "\uECD7"], /* Private Use */
  ["\uECD8", "\uECD8"], /* Private Use */
  ["\uECD9", "\uECD9"], /* Private Use */
  ["\uECDA", "\uECDA"], /* Private Use */
  ["\uECDB", "\uECDB"], /* Private Use */
  ["\uECDC", "\uECDC"], /* Private Use */
  ["\uECDD", "\uECDD"], /* Private Use */
  ["\uECDE", "\uECDE"], /* Private Use */
  ["\uECDF", "\uECDF"], /* Private Use */
  ["\uECE0", "\uECE0"], /* Private Use */
  ["\uECE1", "\uECE1"], /* Private Use */
  ["\uECE2", "\uECE2"], /* Private Use */
  ["\uECE3", "\uECE3"], /* Private Use */
  ["\uECE4", "\uECE4"], /* Private Use */
  ["\uECE5", "\uECE5"], /* Private Use */
  ["\uECE6", "\uECE6"], /* Private Use */
  ["\uECE7", "\uECE7"], /* Private Use */
  ["\uECE8", "\uECE8"], /* Private Use */
  ["\uECE9", "\uECE9"], /* Private Use */
  ["\uECEA", "\uECEA"], /* Private Use */
  ["\uECEB", "\uECEB"], /* Private Use */
  ["\uECEC", "\uECEC"], /* Private Use */
  ["\uECED", "\uECED"], /* Private Use */
  ["\uECEE", "\uECEE"], /* Private Use */
  ["\uECEF", "\uECEF"], /* Private Use */
  ["\uECF0", "\uECF0"], /* Private Use */
  ["\uECF1", "\uECF1"], /* Private Use */
  ["\uECF2", "\uECF2"], /* Private Use */
  ["\uECF3", "\uECF3"], /* Private Use */
  ["\uECF4", "\uECF4"], /* Private Use */
  ["\uECF5", "\uECF5"], /* Private Use */
  ["\uECF6", "\uECF6"], /* Private Use */
  ["\uECF7", "\uECF7"], /* Private Use */
  ["\uECF8", "\uECF8"], /* Private Use */
  ["\uECF9", "\uECF9"], /* Private Use */
  ["\uECFA", "\uECFA"], /* Private Use */
  ["\uECFB", "\uECFB"], /* Private Use */
  ["\uECFC", "\uECFC"], /* Private Use */
  ["\uECFD", "\uECFD"], /* Private Use */
  ["\uECFE", "\uECFE"], /* Private Use */
  ["\uECFF", "\uECFF"], /* Private Use */
  ["\uED00", "\uED00"], /* Private Use */
  ["\uED01", "\uED01"], /* Private Use */
  ["\uED02", "\uED02"], /* Private Use */
  ["\uED03", "\uED03"], /* Private Use */
  ["\uED04", "\uED04"], /* Private Use */
  ["\uED05", "\uED05"], /* Private Use */
  ["\uED06", "\uED06"], /* Private Use */
  ["\uED07", "\uED07"], /* Private Use */
  ["\uED08", "\uED08"], /* Private Use */
  ["\uED09", "\uED09"], /* Private Use */
  ["\uED0A", "\uED0A"], /* Private Use */
  ["\uED0B", "\uED0B"], /* Private Use */
  ["\uED0C", "\uED0C"], /* Private Use */
  ["\uED0D", "\uED0D"], /* Private Use */
  ["\uED0E", "\uED0E"], /* Private Use */
  ["\uED0F", "\uED0F"], /* Private Use */
  ["\uED10", "\uED10"], /* Private Use */
  ["\uED11", "\uED11"], /* Private Use */
  ["\uED12", "\uED12"], /* Private Use */
  ["\uED13", "\uED13"], /* Private Use */
  ["\uED14", "\uED14"], /* Private Use */
  ["\uED15", "\uED15"], /* Private Use */
  ["\uED16", "\uED16"], /* Private Use */
  ["\uED17", "\uED17"], /* Private Use */
  ["\uED18", "\uED18"], /* Private Use */
  ["\uED19", "\uED19"], /* Private Use */
  ["\uED1A", "\uED1A"], /* Private Use */
  ["\uED1B", "\uED1B"], /* Private Use */
  ["\uED1C", "\uED1C"], /* Private Use */
  ["\uED1D", "\uED1D"], /* Private Use */
  ["\uED1E", "\uED1E"], /* Private Use */
  ["\uED1F", "\uED1F"], /* Private Use */
  ["\uED20", "\uED20"], /* Private Use */
  ["\uED21", "\uED21"], /* Private Use */
  ["\uED22", "\uED22"], /* Private Use */
  ["\uED23", "\uED23"], /* Private Use */
  ["\uED24", "\uED24"], /* Private Use */
  ["\uED25", "\uED25"], /* Private Use */
  ["\uED26", "\uED26"], /* Private Use */
  ["\uED27", "\uED27"], /* Private Use */
  ["\uED28", "\uED28"], /* Private Use */
  ["\uED29", "\uED29"], /* Private Use */
  ["\uED2A", "\uED2A"], /* Private Use */
  ["\uED2B", "\uED2B"], /* Private Use */
  ["\uED2C", "\uED2C"], /* Private Use */
  ["\uED2D", "\uED2D"], /* Private Use */
  ["\uED2E", "\uED2E"], /* Private Use */
  ["\uED2F", "\uED2F"], /* Private Use */
  ["\uED30", "\uED30"], /* Private Use */
  ["\uED31", "\uED31"], /* Private Use */
  ["\uED32", "\uED32"], /* Private Use */
  ["\uED33", "\uED33"], /* Private Use */
  ["\uED34", "\uED34"], /* Private Use */
  ["\uED35", "\uED35"], /* Private Use */
  ["\uED36", "\uED36"], /* Private Use */
  ["\uED37", "\uED37"], /* Private Use */
  ["\uED38", "\uED38"], /* Private Use */
  ["\uED39", "\uED39"], /* Private Use */
  ["\uED3A", "\uED3A"], /* Private Use */
  ["\uED3B", "\uED3B"], /* Private Use */
  ["\uED3C", "\uED3C"], /* Private Use */
  ["\uED3D", "\uED3D"], /* Private Use */
  ["\uED3E", "\uED3E"], /* Private Use */
  ["\uED3F", "\uED3F"], /* Private Use */
  ["\uED40", "\uED40"], /* Private Use */
  ["\uED41", "\uED41"], /* Private Use */
  ["\uED42", "\uED42"], /* Private Use */
  ["\uED43", "\uED43"], /* Private Use */
  ["\uED44", "\uED44"], /* Private Use */
  ["\uED45", "\uED45"], /* Private Use */
  ["\uED46", "\uED46"], /* Private Use */
  ["\uED47", "\uED47"], /* Private Use */
  ["\uED48", "\uED48"], /* Private Use */
  ["\uED49", "\uED49"], /* Private Use */
  ["\uED4A", "\uED4A"], /* Private Use */
  ["\uED4B", "\uED4B"], /* Private Use */
  ["\uED4C", "\uED4C"], /* Private Use */
  ["\uED4D", "\uED4D"], /* Private Use */
  ["\uED4E", "\uED4E"], /* Private Use */
  ["\uED4F", "\uED4F"], /* Private Use */
  ["\uED50", "\uED50"], /* Private Use */
  ["\uED51", "\uED51"], /* Private Use */
  ["\uED52", "\uED52"], /* Private Use */
  ["\uED53", "\uED53"], /* Private Use */
  ["\uED54", "\uED54"], /* Private Use */
  ["\uED55", "\uED55"], /* Private Use */
  ["\uED56", "\uED56"], /* Private Use */
  ["\uED57", "\uED57"], /* Private Use */
  ["\uED58", "\uED58"], /* Private Use */
  ["\uED59", "\uED59"], /* Private Use */
  ["\uED5A", "\uED5A"], /* Private Use */
  ["\uED5B", "\uED5B"], /* Private Use */
  ["\uED5C", "\uED5C"], /* Private Use */
  ["\uED5D", "\uED5D"], /* Private Use */
  ["\uED5E", "\uED5E"], /* Private Use */
  ["\uED5F", "\uED5F"], /* Private Use */
  ["\uED60", "\uED60"], /* Private Use */
  ["\uED61", "\uED61"], /* Private Use */
  ["\uED62", "\uED62"], /* Private Use */
  ["\uED63", "\uED63"], /* Private Use */
  ["\uED64", "\uED64"], /* Private Use */
  ["\uED65", "\uED65"], /* Private Use */
  ["\uED66", "\uED66"], /* Private Use */
  ["\uED67", "\uED67"], /* Private Use */
  ["\uED68", "\uED68"], /* Private Use */
  ["\uED69", "\uED69"], /* Private Use */
  ["\uED6A", "\uED6A"], /* Private Use */
  ["\uED6B", "\uED6B"], /* Private Use */
  ["\uED6C", "\uED6C"], /* Private Use */
  ["\uED6D", "\uED6D"], /* Private Use */
  ["\uED6E", "\uED6E"], /* Private Use */
  ["\uED6F", "\uED6F"], /* Private Use */
  ["\uED70", "\uED70"], /* Private Use */
  ["\uED71", "\uED71"], /* Private Use */
  ["\uED72", "\uED72"], /* Private Use */
  ["\uED73", "\uED73"], /* Private Use */
  ["\uED74", "\uED74"], /* Private Use */
  ["\uED75", "\uED75"], /* Private Use */
  ["\uED76", "\uED76"], /* Private Use */
  ["\uED77", "\uED77"], /* Private Use */
  ["\uED78", "\uED78"], /* Private Use */
  ["\uED79", "\uED79"], /* Private Use */
  ["\uED7A", "\uED7A"], /* Private Use */
  ["\uED7B", "\uED7B"], /* Private Use */
  ["\uED7C", "\uED7C"], /* Private Use */
  ["\uED7D", "\uED7D"], /* Private Use */
  ["\uED7E", "\uED7E"], /* Private Use */
  ["\uED7F", "\uED7F"], /* Private Use */
  ["\uED80", "\uED80"], /* Private Use */
  ["\uED81", "\uED81"], /* Private Use */
  ["\uED82", "\uED82"], /* Private Use */
  ["\uED83", "\uED83"], /* Private Use */
  ["\uED84", "\uED84"], /* Private Use */
  ["\uED85", "\uED85"], /* Private Use */
  ["\uED86", "\uED86"], /* Private Use */
  ["\uED87", "\uED87"], /* Private Use */
  ["\uED88", "\uED88"], /* Private Use */
  ["\uED89", "\uED89"], /* Private Use */
  ["\uED8A", "\uED8A"], /* Private Use */
  ["\uED8B", "\uED8B"], /* Private Use */
  ["\uED8C", "\uED8C"], /* Private Use */
  ["\uED8D", "\uED8D"], /* Private Use */
  ["\uED8E", "\uED8E"], /* Private Use */
  ["\uED8F", "\uED8F"], /* Private Use */
  ["\uED90", "\uED90"], /* Private Use */
  ["\uED91", "\uED91"], /* Private Use */
  ["\uED92", "\uED92"], /* Private Use */
  ["\uED93", "\uED93"], /* Private Use */
  ["\uED94", "\uED94"], /* Private Use */
  ["\uED95", "\uED95"], /* Private Use */
  ["\uED96", "\uED96"], /* Private Use */
  ["\uED97", "\uED97"], /* Private Use */
  ["\uED98", "\uED98"], /* Private Use */
  ["\uED99", "\uED99"], /* Private Use */
  ["\uED9A", "\uED9A"], /* Private Use */
  ["\uED9B", "\uED9B"], /* Private Use */
  ["\uED9C", "\uED9C"], /* Private Use */
  ["\uED9D", "\uED9D"], /* Private Use */
  ["\uED9E", "\uED9E"], /* Private Use */
  ["\uED9F", "\uED9F"], /* Private Use */
  ["\uEDA0", "\uEDA0"], /* Private Use */
  ["\uEDA1", "\uEDA1"], /* Private Use */
  ["\uEDA2", "\uEDA2"], /* Private Use */
  ["\uEDA3", "\uEDA3"], /* Private Use */
  ["\uEDA4", "\uEDA4"], /* Private Use */
  ["\uEDA5", "\uEDA5"], /* Private Use */
  ["\uEDA6", "\uEDA6"], /* Private Use */
  ["\uEDA7", "\uEDA7"], /* Private Use */
  ["\uEDA8", "\uEDA8"], /* Private Use */
  ["\uEDA9", "\uEDA9"], /* Private Use */
  ["\uEDAA", "\uEDAA"], /* Private Use */
  ["\uEDAB", "\uEDAB"], /* Private Use */
  ["\uEDAC", "\uEDAC"], /* Private Use */
  ["\uEDAD", "\uEDAD"], /* Private Use */
  ["\uEDAE", "\uEDAE"], /* Private Use */
  ["\uEDAF", "\uEDAF"], /* Private Use */
  ["\uEDB0", "\uEDB0"], /* Private Use */
  ["\uEDB1", "\uEDB1"], /* Private Use */
  ["\uEDB2", "\uEDB2"], /* Private Use */
  ["\uEDB3", "\uEDB3"], /* Private Use */
  ["\uEDB4", "\uEDB4"], /* Private Use */
  ["\uEDB5", "\uEDB5"], /* Private Use */
  ["\uEDB6", "\uEDB6"], /* Private Use */
  ["\uEDB7", "\uEDB7"], /* Private Use */
  ["\uEDB8", "\uEDB8"], /* Private Use */
  ["\uEDB9", "\uEDB9"], /* Private Use */
  ["\uEDBA", "\uEDBA"], /* Private Use */
  ["\uEDBB", "\uEDBB"], /* Private Use */
  ["\uEDBC", "\uEDBC"], /* Private Use */
  ["\uEDBD", "\uEDBD"], /* Private Use */
  ["\uEDBE", "\uEDBE"], /* Private Use */
  ["\uEDBF", "\uEDBF"], /* Private Use */
  ["\uEDC0", "\uEDC0"], /* Private Use */
  ["\uEDC1", "\uEDC1"], /* Private Use */
  ["\uEDC2", "\uEDC2"], /* Private Use */
  ["\uEDC3", "\uEDC3"], /* Private Use */
  ["\uEDC4", "\uEDC4"], /* Private Use */
  ["\uEDC5", "\uEDC5"], /* Private Use */
  ["\uEDC6", "\uEDC6"], /* Private Use */
  ["\uEDC7", "\uEDC7"], /* Private Use */
  ["\uEDC8", "\uEDC8"], /* Private Use */
  ["\uEDC9", "\uEDC9"], /* Private Use */
  ["\uEDCA", "\uEDCA"], /* Private Use */
  ["\uEDCB", "\uEDCB"], /* Private Use */
  ["\uEDCC", "\uEDCC"], /* Private Use */
  ["\uEDCD", "\uEDCD"], /* Private Use */
  ["\uEDCE", "\uEDCE"], /* Private Use */
  ["\uEDCF", "\uEDCF"], /* Private Use */
  ["\uEDD0", "\uEDD0"], /* Private Use */
  ["\uEDD1", "\uEDD1"], /* Private Use */
  ["\uEDD2", "\uEDD2"], /* Private Use */
  ["\uEDD3", "\uEDD3"], /* Private Use */
  ["\uEDD4", "\uEDD4"], /* Private Use */
  ["\uEDD5", "\uEDD5"], /* Private Use */
  ["\uEDD6", "\uEDD6"], /* Private Use */
  ["\uEDD7", "\uEDD7"], /* Private Use */
  ["\uEDD8", "\uEDD8"], /* Private Use */
  ["\uEDD9", "\uEDD9"], /* Private Use */
  ["\uEDDA", "\uEDDA"], /* Private Use */
  ["\uEDDB", "\uEDDB"], /* Private Use */
  ["\uEDDC", "\uEDDC"], /* Private Use */
  ["\uEDDD", "\uEDDD"], /* Private Use */
  ["\uEDDE", "\uEDDE"], /* Private Use */
  ["\uEDDF", "\uEDDF"], /* Private Use */
  ["\uEDE0", "\uEDE0"], /* Private Use */
  ["\uEDE1", "\uEDE1"], /* Private Use */
  ["\uEDE2", "\uEDE2"], /* Private Use */
  ["\uEDE3", "\uEDE3"], /* Private Use */
  ["\uEDE4", "\uEDE4"], /* Private Use */
  ["\uEDE5", "\uEDE5"], /* Private Use */
  ["\uEDE6", "\uEDE6"], /* Private Use */
  ["\uEDE7", "\uEDE7"], /* Private Use */
  ["\uEDE8", "\uEDE8"], /* Private Use */
  ["\uEDE9", "\uEDE9"], /* Private Use */
  ["\uEDEA", "\uEDEA"], /* Private Use */
  ["\uEDEB", "\uEDEB"], /* Private Use */
  ["\uEDEC", "\uEDEC"], /* Private Use */
  ["\uEDED", "\uEDED"], /* Private Use */
  ["\uEDEE", "\uEDEE"], /* Private Use */
  ["\uEDEF", "\uEDEF"], /* Private Use */
  ["\uEDF0", "\uEDF0"], /* Private Use */
  ["\uEDF1", "\uEDF1"], /* Private Use */
  ["\uEDF2", "\uEDF2"], /* Private Use */
  ["\uEDF3", "\uEDF3"], /* Private Use */
  ["\uEDF4", "\uEDF4"], /* Private Use */
  ["\uEDF5", "\uEDF5"], /* Private Use */
  ["\uEDF6", "\uEDF6"], /* Private Use */
  ["\uEDF7", "\uEDF7"], /* Private Use */
  ["\uEDF8", "\uEDF8"], /* Private Use */
  ["\uEDF9", "\uEDF9"], /* Private Use */
  ["\uEDFA", "\uEDFA"], /* Private Use */
  ["\uEDFB", "\uEDFB"], /* Private Use */
  ["\uEDFC", "\uEDFC"], /* Private Use */
  ["\uEDFD", "\uEDFD"], /* Private Use */
  ["\uEDFE", "\uEDFE"], /* Private Use */
  ["\uEDFF", "\uEDFF"], /* Private Use */
  ["\uEE00", "\uEE00"], /* Private Use */
  ["\uEE01", "\uEE01"], /* Private Use */
  ["\uEE02", "\uEE02"], /* Private Use */
  ["\uEE03", "\uEE03"], /* Private Use */
  ["\uEE04", "\uEE04"], /* Private Use */
  ["\uEE05", "\uEE05"], /* Private Use */
  ["\uEE06", "\uEE06"], /* Private Use */
  ["\uEE07", "\uEE07"], /* Private Use */
  ["\uEE08", "\uEE08"], /* Private Use */
  ["\uEE09", "\uEE09"], /* Private Use */
  ["\uEE0A", "\uEE0A"], /* Private Use */
  ["\uEE0B", "\uEE0B"], /* Private Use */
  ["\uEE0C", "\uEE0C"], /* Private Use */
  ["\uEE0D", "\uEE0D"], /* Private Use */
  ["\uEE0E", "\uEE0E"], /* Private Use */
  ["\uEE0F", "\uEE0F"], /* Private Use */
  ["\uEE10", "\uEE10"], /* Private Use */
  ["\uEE11", "\uEE11"], /* Private Use */
  ["\uEE12", "\uEE12"], /* Private Use */
  ["\uEE13", "\uEE13"], /* Private Use */
  ["\uEE14", "\uEE14"], /* Private Use */
  ["\uEE15", "\uEE15"], /* Private Use */
  ["\uEE16", "\uEE16"], /* Private Use */
  ["\uEE17", "\uEE17"], /* Private Use */
  ["\uEE18", "\uEE18"], /* Private Use */
  ["\uEE19", "\uEE19"], /* Private Use */
  ["\uEE1A", "\uEE1A"], /* Private Use */
  ["\uEE1B", "\uEE1B"], /* Private Use */
  ["\uEE1C", "\uEE1C"], /* Private Use */
  ["\uEE1D", "\uEE1D"], /* Private Use */
  ["\uEE1E", "\uEE1E"], /* Private Use */
  ["\uEE1F", "\uEE1F"], /* Private Use */
  ["\uEE20", "\uEE20"], /* Private Use */
  ["\uEE21", "\uEE21"], /* Private Use */
  ["\uEE22", "\uEE22"], /* Private Use */
  ["\uEE23", "\uEE23"], /* Private Use */
  ["\uEE24", "\uEE24"], /* Private Use */
  ["\uEE25", "\uEE25"], /* Private Use */
  ["\uEE26", "\uEE26"], /* Private Use */
  ["\uEE27", "\uEE27"], /* Private Use */
  ["\uEE28", "\uEE28"], /* Private Use */
  ["\uEE29", "\uEE29"], /* Private Use */
  ["\uEE2A", "\uEE2A"], /* Private Use */
  ["\uEE2B", "\uEE2B"], /* Private Use */
  ["\uEE2C", "\uEE2C"], /* Private Use */
  ["\uEE2D", "\uEE2D"], /* Private Use */
  ["\uEE2E", "\uEE2E"], /* Private Use */
  ["\uEE2F", "\uEE2F"], /* Private Use */
  ["\uEE30", "\uEE30"], /* Private Use */
  ["\uEE31", "\uEE31"], /* Private Use */
  ["\uEE32", "\uEE32"], /* Private Use */
  ["\uEE33", "\uEE33"], /* Private Use */
  ["\uEE34", "\uEE34"], /* Private Use */
  ["\uEE35", "\uEE35"], /* Private Use */
  ["\uEE36", "\uEE36"], /* Private Use */
  ["\uEE37", "\uEE37"], /* Private Use */
  ["\uEE38", "\uEE38"], /* Private Use */
  ["\uEE39", "\uEE39"], /* Private Use */
  ["\uEE3A", "\uEE3A"], /* Private Use */
  ["\uEE3B", "\uEE3B"], /* Private Use */
  ["\uEE3C", "\uEE3C"], /* Private Use */
  ["\uEE3D", "\uEE3D"], /* Private Use */
  ["\uEE3E", "\uEE3E"], /* Private Use */
  ["\uEE3F", "\uEE3F"], /* Private Use */
  ["\uEE40", "\uEE40"], /* Private Use */
  ["\uEE41", "\uEE41"], /* Private Use */
  ["\uEE42", "\uEE42"], /* Private Use */
  ["\uEE43", "\uEE43"], /* Private Use */
  ["\uEE44", "\uEE44"], /* Private Use */
  ["\uEE45", "\uEE45"], /* Private Use */
  ["\uEE46", "\uEE46"], /* Private Use */
  ["\uEE47", "\uEE47"], /* Private Use */
  ["\uEE48", "\uEE48"], /* Private Use */
  ["\uEE49", "\uEE49"], /* Private Use */
  ["\uEE4A", "\uEE4A"], /* Private Use */
  ["\uEE4B", "\uEE4B"], /* Private Use */
  ["\uEE4C", "\uEE4C"], /* Private Use */
  ["\uEE4D", "\uEE4D"], /* Private Use */
  ["\uEE4E", "\uEE4E"], /* Private Use */
  ["\uEE4F", "\uEE4F"], /* Private Use */
  ["\uEE50", "\uEE50"], /* Private Use */
  ["\uEE51", "\uEE51"], /* Private Use */
  ["\uEE52", "\uEE52"], /* Private Use */
  ["\uEE53", "\uEE53"], /* Private Use */
  ["\uEE54", "\uEE54"], /* Private Use */
  ["\uEE55", "\uEE55"], /* Private Use */
  ["\uEE56", "\uEE56"], /* Private Use */
  ["\uEE57", "\uEE57"], /* Private Use */
  ["\uEE58", "\uEE58"], /* Private Use */
  ["\uEE59", "\uEE59"], /* Private Use */
  ["\uEE5A", "\uEE5A"], /* Private Use */
  ["\uEE5B", "\uEE5B"], /* Private Use */
  ["\uEE5C", "\uEE5C"], /* Private Use */
  ["\uEE5D", "\uEE5D"], /* Private Use */
  ["\uEE5E", "\uEE5E"], /* Private Use */
  ["\uEE5F", "\uEE5F"], /* Private Use */
  ["\uEE60", "\uEE60"], /* Private Use */
  ["\uEE61", "\uEE61"], /* Private Use */
  ["\uEE62", "\uEE62"], /* Private Use */
  ["\uEE63", "\uEE63"], /* Private Use */
  ["\uEE64", "\uEE64"], /* Private Use */
  ["\uEE65", "\uEE65"], /* Private Use */
  ["\uEE66", "\uEE66"], /* Private Use */
  ["\uEE67", "\uEE67"], /* Private Use */
  ["\uEE68", "\uEE68"], /* Private Use */
  ["\uEE69", "\uEE69"], /* Private Use */
  ["\uEE6A", "\uEE6A"], /* Private Use */
  ["\uEE6B", "\uEE6B"], /* Private Use */
  ["\uEE6C", "\uEE6C"], /* Private Use */
  ["\uEE6D", "\uEE6D"], /* Private Use */
  ["\uEE6E", "\uEE6E"], /* Private Use */
  ["\uEE6F", "\uEE6F"], /* Private Use */
  ["\uEE70", "\uEE70"], /* Private Use */
  ["\uEE71", "\uEE71"], /* Private Use */
  ["\uEE72", "\uEE72"], /* Private Use */
  ["\uEE73", "\uEE73"], /* Private Use */
  ["\uEE74", "\uEE74"], /* Private Use */
  ["\uEE75", "\uEE75"], /* Private Use */
  ["\uEE76", "\uEE76"], /* Private Use */
  ["\uEE77", "\uEE77"], /* Private Use */
  ["\uEE78", "\uEE78"], /* Private Use */
  ["\uEE79", "\uEE79"], /* Private Use */
  ["\uEE7A", "\uEE7A"], /* Private Use */
  ["\uEE7B", "\uEE7B"], /* Private Use */
  ["\uEE7C", "\uEE7C"], /* Private Use */
  ["\uEE7D", "\uEE7D"], /* Private Use */
  ["\uEE7E", "\uEE7E"], /* Private Use */
  ["\uEE7F", "\uEE7F"], /* Private Use */
  ["\uEE80", "\uEE80"], /* Private Use */
  ["\uEE81", "\uEE81"], /* Private Use */
  ["\uEE82", "\uEE82"], /* Private Use */
  ["\uEE83", "\uEE83"], /* Private Use */
  ["\uEE84", "\uEE84"], /* Private Use */
  ["\uEE85", "\uEE85"], /* Private Use */
  ["\uEE86", "\uEE86"], /* Private Use */
  ["\uEE87", "\uEE87"], /* Private Use */
  ["\uEE88", "\uEE88"], /* Private Use */
  ["\uEE89", "\uEE89"], /* Private Use */
  ["\uEE8A", "\uEE8A"], /* Private Use */
  ["\uEE8B", "\uEE8B"], /* Private Use */
  ["\uEE8C", "\uEE8C"], /* Private Use */
  ["\uEE8D", "\uEE8D"], /* Private Use */
  ["\uEE8E", "\uEE8E"], /* Private Use */
  ["\uEE8F", "\uEE8F"], /* Private Use */
  ["\uEE90", "\uEE90"], /* Private Use */
  ["\uEE91", "\uEE91"], /* Private Use */
  ["\uEE92", "\uEE92"], /* Private Use */
  ["\uEE93", "\uEE93"], /* Private Use */
  ["\uEE94", "\uEE94"], /* Private Use */
  ["\uEE95", "\uEE95"], /* Private Use */
  ["\uEE96", "\uEE96"], /* Private Use */
  ["\uEE97", "\uEE97"], /* Private Use */
  ["\uEE98", "\uEE98"], /* Private Use */
  ["\uEE99", "\uEE99"], /* Private Use */
  ["\uEE9A", "\uEE9A"], /* Private Use */
  ["\uEE9B", "\uEE9B"], /* Private Use */
  ["\uEE9C", "\uEE9C"], /* Private Use */
  ["\uEE9D", "\uEE9D"], /* Private Use */
  ["\uEE9E", "\uEE9E"], /* Private Use */
  ["\uEE9F", "\uEE9F"], /* Private Use */
  ["\uEEA0", "\uEEA0"], /* Private Use */
  ["\uEEA1", "\uEEA1"], /* Private Use */
  ["\uEEA2", "\uEEA2"], /* Private Use */
  ["\uEEA3", "\uEEA3"], /* Private Use */
  ["\uEEA4", "\uEEA4"], /* Private Use */
  ["\uEEA5", "\uEEA5"], /* Private Use */
  ["\uEEA6", "\uEEA6"], /* Private Use */
  ["\uEEA7", "\uEEA7"], /* Private Use */
  ["\uEEA8", "\uEEA8"], /* Private Use */
  ["\uEEA9", "\uEEA9"], /* Private Use */
  ["\uEEAA", "\uEEAA"], /* Private Use */
  ["\uEEAB", "\uEEAB"], /* Private Use */
  ["\uEEAC", "\uEEAC"], /* Private Use */
  ["\uEEAD", "\uEEAD"], /* Private Use */
  ["\uEEAE", "\uEEAE"], /* Private Use */
  ["\uEEAF", "\uEEAF"], /* Private Use */
  ["\uEEB0", "\uEEB0"], /* Private Use */
  ["\uEEB1", "\uEEB1"], /* Private Use */
  ["\uEEB2", "\uEEB2"], /* Private Use */
  ["\uEEB3", "\uEEB3"], /* Private Use */
  ["\uEEB4", "\uEEB4"], /* Private Use */
  ["\uEEB5", "\uEEB5"], /* Private Use */
  ["\uEEB6", "\uEEB6"], /* Private Use */
  ["\uEEB7", "\uEEB7"], /* Private Use */
  ["\uEEB8", "\uEEB8"], /* Private Use */
  ["\uEEB9", "\uEEB9"], /* Private Use */
  ["\uEEBA", "\uEEBA"], /* Private Use */
  ["\uEEBB", "\uEEBB"], /* Private Use */
  ["\uEEBC", "\uEEBC"], /* Private Use */
  ["\uEEBD", "\uEEBD"], /* Private Use */
  ["\uEEBE", "\uEEBE"], /* Private Use */
  ["\uEEBF", "\uEEBF"], /* Private Use */
  ["\uEEC0", "\uEEC0"], /* Private Use */
  ["\uEEC1", "\uEEC1"], /* Private Use */
  ["\uEEC2", "\uEEC2"], /* Private Use */
  ["\uEEC3", "\uEEC3"], /* Private Use */
  ["\uEEC4", "\uEEC4"], /* Private Use */
  ["\uEEC5", "\uEEC5"], /* Private Use */
  ["\uEEC6", "\uEEC6"], /* Private Use */
  ["\uEEC7", "\uEEC7"], /* Private Use */
  ["\uEEC8", "\uEEC8"], /* Private Use */
  ["\uEEC9", "\uEEC9"], /* Private Use */
  ["\uEECA", "\uEECA"], /* Private Use */
  ["\uEECB", "\uEECB"], /* Private Use */
  ["\uEECC", "\uEECC"], /* Private Use */
  ["\uEECD", "\uEECD"], /* Private Use */
  ["\uEECE", "\uEECE"], /* Private Use */
  ["\uEECF", "\uEECF"], /* Private Use */
  ["\uEED0", "\uEED0"], /* Private Use */
  ["\uEED1", "\uEED1"], /* Private Use */
  ["\uEED2", "\uEED2"], /* Private Use */
  ["\uEED3", "\uEED3"], /* Private Use */
  ["\uEED4", "\uEED4"], /* Private Use */
  ["\uEED5", "\uEED5"], /* Private Use */
  ["\uEED6", "\uEED6"], /* Private Use */
  ["\uEED7", "\uEED7"], /* Private Use */
  ["\uEED8", "\uEED8"], /* Private Use */
  ["\uEED9", "\uEED9"], /* Private Use */
  ["\uEEDA", "\uEEDA"], /* Private Use */
  ["\uEEDB", "\uEEDB"], /* Private Use */
  ["\uEEDC", "\uEEDC"], /* Private Use */
  ["\uEEDD", "\uEEDD"], /* Private Use */
  ["\uEEDE", "\uEEDE"], /* Private Use */
  ["\uEEDF", "\uEEDF"], /* Private Use */
  ["\uEEE0", "\uEEE0"], /* Private Use */
  ["\uEEE1", "\uEEE1"], /* Private Use */
  ["\uEEE2", "\uEEE2"], /* Private Use */
  ["\uEEE3", "\uEEE3"], /* Private Use */
  ["\uEEE4", "\uEEE4"], /* Private Use */
  ["\uEEE5", "\uEEE5"], /* Private Use */
  ["\uEEE6", "\uEEE6"], /* Private Use */
  ["\uEEE7", "\uEEE7"], /* Private Use */
  ["\uEEE8", "\uEEE8"], /* Private Use */
  ["\uEEE9", "\uEEE9"], /* Private Use */
  ["\uEEEA", "\uEEEA"], /* Private Use */
  ["\uEEEB", "\uEEEB"], /* Private Use */
  ["\uEEEC", "\uEEEC"], /* Private Use */
  ["\uEEED", "\uEEED"], /* Private Use */
  ["\uEEEE", "\uEEEE"], /* Private Use */
  ["\uEEEF", "\uEEEF"], /* Private Use */
  ["\uEEF0", "\uEEF0"], /* Private Use */
  ["\uEEF1", "\uEEF1"], /* Private Use */
  ["\uEEF2", "\uEEF2"], /* Private Use */
  ["\uEEF3", "\uEEF3"], /* Private Use */
  ["\uEEF4", "\uEEF4"], /* Private Use */
  ["\uEEF5", "\uEEF5"], /* Private Use */
  ["\uEEF6", "\uEEF6"], /* Private Use */
  ["\uEEF7", "\uEEF7"], /* Private Use */
  ["\uEEF8", "\uEEF8"], /* Private Use */
  ["\uEEF9", "\uEEF9"], /* Private Use */
  ["\uEEFA", "\uEEFA"], /* Private Use */
  ["\uEEFB", "\uEEFB"], /* Private Use */
  ["\uEEFC", "\uEEFC"], /* Private Use */
  ["\uEEFD", "\uEEFD"], /* Private Use */
  ["\uEEFE", "\uEEFE"], /* Private Use */
  ["\uEEFF", "\uEEFF"], /* Private Use */
  ["\uEF00", "\uEF00"], /* Private Use */
  ["\uEF01", "\uEF01"], /* Private Use */
  ["\uEF02", "\uEF02"], /* Private Use */
  ["\uEF03", "\uEF03"], /* Private Use */
  ["\uEF04", "\uEF04"], /* Private Use */
  ["\uEF05", "\uEF05"], /* Private Use */
  ["\uEF06", "\uEF06"], /* Private Use */
  ["\uEF07", "\uEF07"], /* Private Use */
  ["\uEF08", "\uEF08"], /* Private Use */
  ["\uEF09", "\uEF09"], /* Private Use */
  ["\uEF0A", "\uEF0A"], /* Private Use */
  ["\uEF0B", "\uEF0B"], /* Private Use */
  ["\uEF0C", "\uEF0C"], /* Private Use */
  ["\uEF0D", "\uEF0D"], /* Private Use */
  ["\uEF0E", "\uEF0E"], /* Private Use */
  ["\uEF0F", "\uEF0F"], /* Private Use */
  ["\uEF10", "\uEF10"], /* Private Use */
  ["\uEF11", "\uEF11"], /* Private Use */
  ["\uEF12", "\uEF12"], /* Private Use */
  ["\uEF13", "\uEF13"], /* Private Use */
  ["\uEF14", "\uEF14"], /* Private Use */
  ["\uEF15", "\uEF15"], /* Private Use */
  ["\uEF16", "\uEF16"], /* Private Use */
  ["\uEF17", "\uEF17"], /* Private Use */
  ["\uEF18", "\uEF18"], /* Private Use */
  ["\uEF19", "\uEF19"], /* Private Use */
  ["\uEF1A", "\uEF1A"], /* Private Use */
  ["\uEF1B", "\uEF1B"], /* Private Use */
  ["\uEF1C", "\uEF1C"], /* Private Use */
  ["\uEF1D", "\uEF1D"], /* Private Use */
  ["\uEF1E", "\uEF1E"], /* Private Use */
  ["\uEF1F", "\uEF1F"], /* Private Use */
  ["\uEF20", "\uEF20"], /* Private Use */
  ["\uEF21", "\uEF21"], /* Private Use */
  ["\uEF22", "\uEF22"], /* Private Use */
  ["\uEF23", "\uEF23"], /* Private Use */
  ["\uEF24", "\uEF24"], /* Private Use */
  ["\uEF25", "\uEF25"], /* Private Use */
  ["\uEF26", "\uEF26"], /* Private Use */
  ["\uEF27", "\uEF27"], /* Private Use */
  ["\uEF28", "\uEF28"], /* Private Use */
  ["\uEF29", "\uEF29"], /* Private Use */
  ["\uEF2A", "\uEF2A"], /* Private Use */
  ["\uEF2B", "\uEF2B"], /* Private Use */
  ["\uEF2C", "\uEF2C"], /* Private Use */
  ["\uEF2D", "\uEF2D"], /* Private Use */
  ["\uEF2E", "\uEF2E"], /* Private Use */
  ["\uEF2F", "\uEF2F"], /* Private Use */
  ["\uEF30", "\uEF30"], /* Private Use */
  ["\uEF31", "\uEF31"], /* Private Use */
  ["\uEF32", "\uEF32"], /* Private Use */
  ["\uEF33", "\uEF33"], /* Private Use */
  ["\uEF34", "\uEF34"], /* Private Use */
  ["\uEF35", "\uEF35"], /* Private Use */
  ["\uEF36", "\uEF36"], /* Private Use */
  ["\uEF37", "\uEF37"], /* Private Use */
  ["\uEF38", "\uEF38"], /* Private Use */
  ["\uEF39", "\uEF39"], /* Private Use */
  ["\uEF3A", "\uEF3A"], /* Private Use */
  ["\uEF3B", "\uEF3B"], /* Private Use */
  ["\uEF3C", "\uEF3C"], /* Private Use */
  ["\uEF3D", "\uEF3D"], /* Private Use */
  ["\uEF3E", "\uEF3E"], /* Private Use */
  ["\uEF3F", "\uEF3F"], /* Private Use */
  ["\uEF40", "\uEF40"], /* Private Use */
  ["\uEF41", "\uEF41"], /* Private Use */
  ["\uEF42", "\uEF42"], /* Private Use */
  ["\uEF43", "\uEF43"], /* Private Use */
  ["\uEF44", "\uEF44"], /* Private Use */
  ["\uEF45", "\uEF45"], /* Private Use */
  ["\uEF46", "\uEF46"], /* Private Use */
  ["\uEF47", "\uEF47"], /* Private Use */
  ["\uEF48", "\uEF48"], /* Private Use */
  ["\uEF49", "\uEF49"], /* Private Use */
  ["\uEF4A", "\uEF4A"], /* Private Use */
  ["\uEF4B", "\uEF4B"], /* Private Use */
  ["\uEF4C", "\uEF4C"], /* Private Use */
  ["\uEF4D", "\uEF4D"], /* Private Use */
  ["\uEF4E", "\uEF4E"], /* Private Use */
  ["\uEF4F", "\uEF4F"], /* Private Use */
  ["\uEF50", "\uEF50"], /* Private Use */
  ["\uEF51", "\uEF51"], /* Private Use */
  ["\uEF52", "\uEF52"], /* Private Use */
  ["\uEF53", "\uEF53"], /* Private Use */
  ["\uEF54", "\uEF54"], /* Private Use */
  ["\uEF55", "\uEF55"], /* Private Use */
  ["\uEF56", "\uEF56"], /* Private Use */
  ["\uEF57", "\uEF57"], /* Private Use */
  ["\uEF58", "\uEF58"], /* Private Use */
  ["\uEF59", "\uEF59"], /* Private Use */
  ["\uEF5A", "\uEF5A"], /* Private Use */
  ["\uEF5B", "\uEF5B"], /* Private Use */
  ["\uEF5C", "\uEF5C"], /* Private Use */
  ["\uEF5D", "\uEF5D"], /* Private Use */
  ["\uEF5E", "\uEF5E"], /* Private Use */
  ["\uEF5F", "\uEF5F"], /* Private Use */
  ["\uEF60", "\uEF60"], /* Private Use */
  ["\uEF61", "\uEF61"], /* Private Use */
  ["\uEF62", "\uEF62"], /* Private Use */
  ["\uEF63", "\uEF63"], /* Private Use */
  ["\uEF64", "\uEF64"], /* Private Use */
  ["\uEF65", "\uEF65"], /* Private Use */
  ["\uEF66", "\uEF66"], /* Private Use */
  ["\uEF67", "\uEF67"], /* Private Use */
  ["\uEF68", "\uEF68"], /* Private Use */
  ["\uEF69", "\uEF69"], /* Private Use */
  ["\uEF6A", "\uEF6A"], /* Private Use */
  ["\uEF6B", "\uEF6B"], /* Private Use */
  ["\uEF6C", "\uEF6C"], /* Private Use */
  ["\uEF6D", "\uEF6D"], /* Private Use */
  ["\uEF6E", "\uEF6E"], /* Private Use */
  ["\uEF6F", "\uEF6F"], /* Private Use */
  ["\uEF70", "\uEF70"], /* Private Use */
  ["\uEF71", "\uEF71"], /* Private Use */
  ["\uEF72", "\uEF72"], /* Private Use */
  ["\uEF73", "\uEF73"], /* Private Use */
  ["\uEF74", "\uEF74"], /* Private Use */
  ["\uEF75", "\uEF75"], /* Private Use */
  ["\uEF76", "\uEF76"], /* Private Use */
  ["\uEF77", "\uEF77"], /* Private Use */
  ["\uEF78", "\uEF78"], /* Private Use */
  ["\uEF79", "\uEF79"], /* Private Use */
  ["\uEF7A", "\uEF7A"], /* Private Use */
  ["\uEF7B", "\uEF7B"], /* Private Use */
  ["\uEF7C", "\uEF7C"], /* Private Use */
  ["\uEF7D", "\uEF7D"], /* Private Use */
  ["\uEF7E", "\uEF7E"], /* Private Use */
  ["\uEF7F", "\uEF7F"], /* Private Use */
  ["\uEF80", "\uEF80"], /* Private Use */
  ["\uEF81", "\uEF81"], /* Private Use */
  ["\uEF82", "\uEF82"], /* Private Use */
  ["\uEF83", "\uEF83"], /* Private Use */
  ["\uEF84", "\uEF84"], /* Private Use */
  ["\uEF85", "\uEF85"], /* Private Use */
  ["\uEF86", "\uEF86"], /* Private Use */
  ["\uEF87", "\uEF87"], /* Private Use */
  ["\uEF88", "\uEF88"], /* Private Use */
  ["\uEF89", "\uEF89"], /* Private Use */
  ["\uEF8A", "\uEF8A"], /* Private Use */
  ["\uEF8B", "\uEF8B"], /* Private Use */
  ["\uEF8C", "\uEF8C"], /* Private Use */
  ["\uEF8D", "\uEF8D"], /* Private Use */
  ["\uEF8E", "\uEF8E"], /* Private Use */
  ["\uEF8F", "\uEF8F"], /* Private Use */
  ["\uEF90", "\uEF90"], /* Private Use */
  ["\uEF91", "\uEF91"], /* Private Use */
  ["\uEF92", "\uEF92"], /* Private Use */
  ["\uEF93", "\uEF93"], /* Private Use */
  ["\uEF94", "\uEF94"], /* Private Use */
  ["\uEF95", "\uEF95"], /* Private Use */
  ["\uEF96", "\uEF96"], /* Private Use */
  ["\uEF97", "\uEF97"], /* Private Use */
  ["\uEF98", "\uEF98"], /* Private Use */
  ["\uEF99", "\uEF99"], /* Private Use */
  ["\uEF9A", "\uEF9A"], /* Private Use */
  ["\uEF9B", "\uEF9B"], /* Private Use */
  ["\uEF9C", "\uEF9C"], /* Private Use */
  ["\uEF9D", "\uEF9D"], /* Private Use */
  ["\uEF9E", "\uEF9E"], /* Private Use */
  ["\uEF9F", "\uEF9F"], /* Private Use */
  ["\uEFA0", "\uEFA0"], /* Private Use */
  ["\uEFA1", "\uEFA1"], /* Private Use */
  ["\uEFA2", "\uEFA2"], /* Private Use */
  ["\uEFA3", "\uEFA3"], /* Private Use */
  ["\uEFA4", "\uEFA4"], /* Private Use */
  ["\uEFA5", "\uEFA5"], /* Private Use */
  ["\uEFA6", "\uEFA6"], /* Private Use */
  ["\uEFA7", "\uEFA7"], /* Private Use */
  ["\uEFA8", "\uEFA8"], /* Private Use */
  ["\uEFA9", "\uEFA9"], /* Private Use */
  ["\uEFAA", "\uEFAA"], /* Private Use */
  ["\uEFAB", "\uEFAB"], /* Private Use */
  ["\uEFAC", "\uEFAC"], /* Private Use */
  ["\uEFAD", "\uEFAD"], /* Private Use */
  ["\uEFAE", "\uEFAE"], /* Private Use */
  ["\uEFAF", "\uEFAF"], /* Private Use */
  ["\uEFB0", "\uEFB0"], /* Private Use */
  ["\uEFB1", "\uEFB1"], /* Private Use */
  ["\uEFB2", "\uEFB2"], /* Private Use */
  ["\uEFB3", "\uEFB3"], /* Private Use */
  ["\uEFB4", "\uEFB4"], /* Private Use */
  ["\uEFB5", "\uEFB5"], /* Private Use */
  ["\uEFB6", "\uEFB6"], /* Private Use */
  ["\uEFB7", "\uEFB7"], /* Private Use */
  ["\uEFB8", "\uEFB8"], /* Private Use */
  ["\uEFB9", "\uEFB9"], /* Private Use */
  ["\uEFBA", "\uEFBA"], /* Private Use */
  ["\uEFBB", "\uEFBB"], /* Private Use */
  ["\uEFBC", "\uEFBC"], /* Private Use */
  ["\uEFBD", "\uEFBD"], /* Private Use */
  ["\uEFBE", "\uEFBE"], /* Private Use */
  ["\uEFBF", "\uEFBF"], /* Private Use */
  ["\uEFC0", "\uEFC0"], /* Private Use */
  ["\uEFC1", "\uEFC1"], /* Private Use */
  ["\uEFC2", "\uEFC2"], /* Private Use */
  ["\uEFC3", "\uEFC3"], /* Private Use */
  ["\uEFC4", "\uEFC4"], /* Private Use */
  ["\uEFC5", "\uEFC5"], /* Private Use */
  ["\uEFC6", "\uEFC6"], /* Private Use */
  ["\uEFC7", "\uEFC7"], /* Private Use */
  ["\uEFC8", "\uEFC8"], /* Private Use */
  ["\uEFC9", "\uEFC9"], /* Private Use */
  ["\uEFCA", "\uEFCA"], /* Private Use */
  ["\uEFCB", "\uEFCB"], /* Private Use */
  ["\uEFCC", "\uEFCC"], /* Private Use */
  ["\uEFCD", "\uEFCD"], /* Private Use */
  ["\uEFCE", "\uEFCE"], /* Private Use */
  ["\uEFCF", "\uEFCF"], /* Private Use */
  ["\uEFD0", "\uEFD0"], /* Private Use */
  ["\uEFD1", "\uEFD1"], /* Private Use */
  ["\uEFD2", "\uEFD2"], /* Private Use */
  ["\uEFD3", "\uEFD3"], /* Private Use */
  ["\uEFD4", "\uEFD4"], /* Private Use */
  ["\uEFD5", "\uEFD5"], /* Private Use */
  ["\uEFD6", "\uEFD6"], /* Private Use */
  ["\uEFD7", "\uEFD7"], /* Private Use */
  ["\uEFD8", "\uEFD8"], /* Private Use */
  ["\uEFD9", "\uEFD9"], /* Private Use */
  ["\uEFDA", "\uEFDA"], /* Private Use */
  ["\uEFDB", "\uEFDB"], /* Private Use */
  ["\uEFDC", "\uEFDC"], /* Private Use */
  ["\uEFDD", "\uEFDD"], /* Private Use */
  ["\uEFDE", "\uEFDE"], /* Private Use */
  ["\uEFDF", "\uEFDF"], /* Private Use */
  ["\uEFE0", "\uEFE0"], /* Private Use */
  ["\uEFE1", "\uEFE1"], /* Private Use */
  ["\uEFE2", "\uEFE2"], /* Private Use */
  ["\uEFE3", "\uEFE3"], /* Private Use */
  ["\uEFE4", "\uEFE4"], /* Private Use */
  ["\uEFE5", "\uEFE5"], /* Private Use */
  ["\uEFE6", "\uEFE6"], /* Private Use */
  ["\uEFE7", "\uEFE7"], /* Private Use */
  ["\uEFE8", "\uEFE8"], /* Private Use */
  ["\uEFE9", "\uEFE9"], /* Private Use */
  ["\uEFEA", "\uEFEA"], /* Private Use */
  ["\uEFEB", "\uEFEB"], /* Private Use */
  ["\uEFEC", "\uEFEC"], /* Private Use */
  ["\uEFED", "\uEFED"], /* Private Use */
  ["\uEFEE", "\uEFEE"], /* Private Use */
  ["\uEFEF", "\uEFEF"], /* Private Use */
  ["\uEFF0", "\uEFF0"], /* Private Use */
  ["\uEFF1", "\uEFF1"], /* Private Use */
  ["\uEFF2", "\uEFF2"], /* Private Use */
  ["\uEFF3", "\uEFF3"], /* Private Use */
  ["\uEFF4", "\uEFF4"], /* Private Use */
  ["\uEFF5", "\uEFF5"], /* Private Use */
  ["\uEFF6", "\uEFF6"], /* Private Use */
  ["\uEFF7", "\uEFF7"], /* Private Use */
  ["\uEFF8", "\uEFF8"], /* Private Use */
  ["\uEFF9", "\uEFF9"], /* Private Use */
  ["\uEFFA", "\uEFFA"], /* Private Use */
  ["\uEFFB", "\uEFFB"], /* Private Use */
  ["\uEFFC", "\uEFFC"], /* Private Use */
  ["\uEFFD", "\uEFFD"], /* Private Use */
  ["\uEFFE", "\uEFFE"], /* Private Use */
  ["\uEFFF", "\uEFFF"], /* Private Use */
  ["\uF000", "\uF000"], /* Private Use */
  ["\uF001", "\uF001"], /* Private Use */
  ["\uF002", "\uF002"], /* Private Use */
  ["\uF003", "\uF003"], /* Private Use */
  ["\uF004", "\uF004"], /* Private Use */
  ["\uF005", "\uF005"], /* Private Use */
  ["\uF006", "\uF006"], /* Private Use */
  ["\uF007", "\uF007"], /* Private Use */
  ["\uF008", "\uF008"], /* Private Use */
  ["\uF009", "\uF009"], /* Private Use */
  ["\uF00A", "\uF00A"], /* Private Use */
  ["\uF00B", "\uF00B"], /* Private Use */
  ["\uF00C", "\uF00C"], /* Private Use */
  ["\uF00D", "\uF00D"], /* Private Use */
  ["\uF00E", "\uF00E"], /* Private Use */
  ["\uF00F", "\uF00F"], /* Private Use */
  ["\uF010", "\uF010"], /* Private Use */
  ["\uF011", "\uF011"], /* Private Use */
  ["\uF012", "\uF012"], /* Private Use */
  ["\uF013", "\uF013"], /* Private Use */
  ["\uF014", "\uF014"], /* Private Use */
  ["\uF015", "\uF015"], /* Private Use */
  ["\uF016", "\uF016"], /* Private Use */
  ["\uF017", "\uF017"], /* Private Use */
  ["\uF018", "\uF018"], /* Private Use */
  ["\uF019", "\uF019"], /* Private Use */
  ["\uF01A", "\uF01A"], /* Private Use */
  ["\uF01B", "\uF01B"], /* Private Use */
  ["\uF01C", "\uF01C"], /* Private Use */
  ["\uF01D", "\uF01D"], /* Private Use */
  ["\uF01E", "\uF01E"], /* Private Use */
  ["\uF01F", "\uF01F"], /* Private Use */
  ["\uF020", "\uF020"], /* Private Use */
  ["\uF021", "\uF021"], /* Private Use */
  ["\uF022", "\uF022"], /* Private Use */
  ["\uF023", "\uF023"], /* Private Use */
  ["\uF024", "\uF024"], /* Private Use */
  ["\uF025", "\uF025"], /* Private Use */
  ["\uF026", "\uF026"], /* Private Use */
  ["\uF027", "\uF027"], /* Private Use */
  ["\uF028", "\uF028"], /* Private Use */
  ["\uF029", "\uF029"], /* Private Use */
  ["\uF02A", "\uF02A"], /* Private Use */
  ["\uF02B", "\uF02B"], /* Private Use */
  ["\uF02C", "\uF02C"], /* Private Use */
  ["\uF02D", "\uF02D"], /* Private Use */
  ["\uF02E", "\uF02E"], /* Private Use */
  ["\uF02F", "\uF02F"], /* Private Use */
  ["\uF030", "\uF030"], /* Private Use */
  ["\uF031", "\uF031"], /* Private Use */
  ["\uF032", "\uF032"], /* Private Use */
  ["\uF033", "\uF033"], /* Private Use */
  ["\uF034", "\uF034"], /* Private Use */
  ["\uF035", "\uF035"], /* Private Use */
  ["\uF036", "\uF036"], /* Private Use */
  ["\uF037", "\uF037"], /* Private Use */
  ["\uF038", "\uF038"], /* Private Use */
  ["\uF039", "\uF039"], /* Private Use */
  ["\uF03A", "\uF03A"], /* Private Use */
  ["\uF03B", "\uF03B"], /* Private Use */
  ["\uF03C", "\uF03C"], /* Private Use */
  ["\uF03D", "\uF03D"], /* Private Use */
  ["\uF03E", "\uF03E"], /* Private Use */
  ["\uF03F", "\uF03F"], /* Private Use */
  ["\uF040", "\uF040"], /* Private Use */
  ["\uF041", "\uF041"], /* Private Use */
  ["\uF042", "\uF042"], /* Private Use */
  ["\uF043", "\uF043"], /* Private Use */
  ["\uF044", "\uF044"], /* Private Use */
  ["\uF045", "\uF045"], /* Private Use */
  ["\uF046", "\uF046"], /* Private Use */
  ["\uF047", "\uF047"], /* Private Use */
  ["\uF048", "\uF048"], /* Private Use */
  ["\uF049", "\uF049"], /* Private Use */
  ["\uF04A", "\uF04A"], /* Private Use */
  ["\uF04B", "\uF04B"], /* Private Use */
  ["\uF04C", "\uF04C"], /* Private Use */
  ["\uF04D", "\uF04D"], /* Private Use */
  ["\uF04E", "\uF04E"], /* Private Use */
  ["\uF04F", "\uF04F"], /* Private Use */
  ["\uF050", "\uF050"], /* Private Use */
  ["\uF051", "\uF051"], /* Private Use */
  ["\uF052", "\uF052"], /* Private Use */
  ["\uF053", "\uF053"], /* Private Use */
  ["\uF054", "\uF054"], /* Private Use */
  ["\uF055", "\uF055"], /* Private Use */
  ["\uF056", "\uF056"], /* Private Use */
  ["\uF057", "\uF057"], /* Private Use */
  ["\uF058", "\uF058"], /* Private Use */
  ["\uF059", "\uF059"], /* Private Use */
  ["\uF05A", "\uF05A"], /* Private Use */
  ["\uF05B", "\uF05B"], /* Private Use */
  ["\uF05C", "\uF05C"], /* Private Use */
  ["\uF05D", "\uF05D"], /* Private Use */
  ["\uF05E", "\uF05E"], /* Private Use */
  ["\uF05F", "\uF05F"], /* Private Use */
  ["\uF060", "\uF060"], /* Private Use */
  ["\uF061", "\uF061"], /* Private Use */
  ["\uF062", "\uF062"], /* Private Use */
  ["\uF063", "\uF063"], /* Private Use */
  ["\uF064", "\uF064"], /* Private Use */
  ["\uF065", "\uF065"], /* Private Use */
  ["\uF066", "\uF066"], /* Private Use */
  ["\uF067", "\uF067"], /* Private Use */
  ["\uF068", "\uF068"], /* Private Use */
  ["\uF069", "\uF069"], /* Private Use */
  ["\uF06A", "\uF06A"], /* Private Use */
  ["\uF06B", "\uF06B"], /* Private Use */
  ["\uF06C", "\uF06C"], /* Private Use */
  ["\uF06D", "\uF06D"], /* Private Use */
  ["\uF06E", "\uF06E"], /* Private Use */
  ["\uF06F", "\uF06F"], /* Private Use */
  ["\uF070", "\uF070"], /* Private Use */
  ["\uF071", "\uF071"], /* Private Use */
  ["\uF072", "\uF072"], /* Private Use */
  ["\uF073", "\uF073"], /* Private Use */
  ["\uF074", "\uF074"], /* Private Use */
  ["\uF075", "\uF075"], /* Private Use */
  ["\uF076", "\uF076"], /* Private Use */
  ["\uF077", "\uF077"], /* Private Use */
  ["\uF078", "\uF078"], /* Private Use */
  ["\uF079", "\uF079"], /* Private Use */
  ["\uF07A", "\uF07A"], /* Private Use */
  ["\uF07B", "\uF07B"], /* Private Use */
  ["\uF07C", "\uF07C"], /* Private Use */
  ["\uF07D", "\uF07D"], /* Private Use */
  ["\uF07E", "\uF07E"], /* Private Use */
  ["\uF07F", "\uF07F"], /* Private Use */
  ["\uF080", "\uF080"], /* Private Use */
  ["\uF081", "\uF081"], /* Private Use */
  ["\uF082", "\uF082"], /* Private Use */
  ["\uF083", "\uF083"], /* Private Use */
  ["\uF084", "\uF084"], /* Private Use */
  ["\uF085", "\uF085"], /* Private Use */
  ["\uF086", "\uF086"], /* Private Use */
  ["\uF087", "\uF087"], /* Private Use */
  ["\uF088", "\uF088"], /* Private Use */
  ["\uF089", "\uF089"], /* Private Use */
  ["\uF08A", "\uF08A"], /* Private Use */
  ["\uF08B", "\uF08B"], /* Private Use */
  ["\uF08C", "\uF08C"], /* Private Use */
  ["\uF08D", "\uF08D"], /* Private Use */
  ["\uF08E", "\uF08E"], /* Private Use */
  ["\uF08F", "\uF08F"], /* Private Use */
  ["\uF090", "\uF090"], /* Private Use */
  ["\uF091", "\uF091"], /* Private Use */
  ["\uF092", "\uF092"], /* Private Use */
  ["\uF093", "\uF093"], /* Private Use */
  ["\uF094", "\uF094"], /* Private Use */
  ["\uF095", "\uF095"], /* Private Use */
  ["\uF096", "\uF096"], /* Private Use */
  ["\uF097", "\uF097"], /* Private Use */
  ["\uF098", "\uF098"], /* Private Use */
  ["\uF099", "\uF099"], /* Private Use */
  ["\uF09A", "\uF09A"], /* Private Use */
  ["\uF09B", "\uF09B"], /* Private Use */
  ["\uF09C", "\uF09C"], /* Private Use */
  ["\uF09D", "\uF09D"], /* Private Use */
  ["\uF09E", "\uF09E"], /* Private Use */
  ["\uF09F", "\uF09F"], /* Private Use */
  ["\uF0A0", "\uF0A0"], /* Private Use */
  ["\uF0A1", "\uF0A1"], /* Private Use */
  ["\uF0A2", "\uF0A2"], /* Private Use */
  ["\uF0A3", "\uF0A3"], /* Private Use */
  ["\uF0A4", "\uF0A4"], /* Private Use */
  ["\uF0A5", "\uF0A5"], /* Private Use */
  ["\uF0A6", "\uF0A6"], /* Private Use */
  ["\uF0A7", "\uF0A7"], /* Private Use */
  ["\uF0A8", "\uF0A8"], /* Private Use */
  ["\uF0A9", "\uF0A9"], /* Private Use */
  ["\uF0AA", "\uF0AA"], /* Private Use */
  ["\uF0AB", "\uF0AB"], /* Private Use */
  ["\uF0AC", "\uF0AC"], /* Private Use */
  ["\uF0AD", "\uF0AD"], /* Private Use */
  ["\uF0AE", "\uF0AE"], /* Private Use */
  ["\uF0AF", "\uF0AF"], /* Private Use */
  ["\uF0B0", "\uF0B0"], /* Private Use */
  ["\uF0B1", "\uF0B1"], /* Private Use */
  ["\uF0B2", "\uF0B2"], /* Private Use */
  ["\uF0B3", "\uF0B3"], /* Private Use */
  ["\uF0B4", "\uF0B4"], /* Private Use */
  ["\uF0B5", "\uF0B5"], /* Private Use */
  ["\uF0B6", "\uF0B6"], /* Private Use */
  ["\uF0B7", "\uF0B7"], /* Private Use */
  ["\uF0B8", "\uF0B8"], /* Private Use */
  ["\uF0B9", "\uF0B9"], /* Private Use */
  ["\uF0BA", "\uF0BA"], /* Private Use */
  ["\uF0BB", "\uF0BB"], /* Private Use */
  ["\uF0BC", "\uF0BC"], /* Private Use */
  ["\uF0BD", "\uF0BD"], /* Private Use */
  ["\uF0BE", "\uF0BE"], /* Private Use */
  ["\uF0BF", "\uF0BF"], /* Private Use */
  ["\uF0C0", "\uF0C0"], /* Private Use */
  ["\uF0C1", "\uF0C1"], /* Private Use */
  ["\uF0C2", "\uF0C2"], /* Private Use */
  ["\uF0C3", "\uF0C3"], /* Private Use */
  ["\uF0C4", "\uF0C4"], /* Private Use */
  ["\uF0C5", "\uF0C5"], /* Private Use */
  ["\uF0C6", "\uF0C6"], /* Private Use */
  ["\uF0C7", "\uF0C7"], /* Private Use */
  ["\uF0C8", "\uF0C8"], /* Private Use */
  ["\uF0C9", "\uF0C9"], /* Private Use */
  ["\uF0CA", "\uF0CA"], /* Private Use */
  ["\uF0CB", "\uF0CB"], /* Private Use */
  ["\uF0CC", "\uF0CC"], /* Private Use */
  ["\uF0CD", "\uF0CD"], /* Private Use */
  ["\uF0CE", "\uF0CE"], /* Private Use */
  ["\uF0CF", "\uF0CF"], /* Private Use */
  ["\uF0D0", "\uF0D0"], /* Private Use */
  ["\uF0D1", "\uF0D1"], /* Private Use */
  ["\uF0D2", "\uF0D2"], /* Private Use */
  ["\uF0D3", "\uF0D3"], /* Private Use */
  ["\uF0D4", "\uF0D4"], /* Private Use */
  ["\uF0D5", "\uF0D5"], /* Private Use */
  ["\uF0D6", "\uF0D6"], /* Private Use */
  ["\uF0D7", "\uF0D7"], /* Private Use */
  ["\uF0D8", "\uF0D8"], /* Private Use */
  ["\uF0D9", "\uF0D9"], /* Private Use */
  ["\uF0DA", "\uF0DA"], /* Private Use */
  ["\uF0DB", "\uF0DB"], /* Private Use */
  ["\uF0DC", "\uF0DC"], /* Private Use */
  ["\uF0DD", "\uF0DD"], /* Private Use */
  ["\uF0DE", "\uF0DE"], /* Private Use */
  ["\uF0DF", "\uF0DF"], /* Private Use */
  ["\uF0E0", "\uF0E0"], /* Private Use */
  ["\uF0E1", "\uF0E1"], /* Private Use */
  ["\uF0E2", "\uF0E2"], /* Private Use */
  ["\uF0E3", "\uF0E3"], /* Private Use */
  ["\uF0E4", "\uF0E4"], /* Private Use */
  ["\uF0E5", "\uF0E5"], /* Private Use */
  ["\uF0E6", "\uF0E6"], /* Private Use */
  ["\uF0E7", "\uF0E7"], /* Private Use */
  ["\uF0E8", "\uF0E8"], /* Private Use */
  ["\uF0E9", "\uF0E9"], /* Private Use */
  ["\uF0EA", "\uF0EA"], /* Private Use */
  ["\uF0EB", "\uF0EB"], /* Private Use */
  ["\uF0EC", "\uF0EC"], /* Private Use */
  ["\uF0ED", "\uF0ED"], /* Private Use */
  ["\uF0EE", "\uF0EE"], /* Private Use */
  ["\uF0EF", "\uF0EF"], /* Private Use */
  ["\uF0F0", "\uF0F0"], /* Private Use */
  ["\uF0F1", "\uF0F1"], /* Private Use */
  ["\uF0F2", "\uF0F2"], /* Private Use */
  ["\uF0F3", "\uF0F3"], /* Private Use */
  ["\uF0F4", "\uF0F4"], /* Private Use */
  ["\uF0F5", "\uF0F5"], /* Private Use */
  ["\uF0F6", "\uF0F6"], /* Private Use */
  ["\uF0F7", "\uF0F7"], /* Private Use */
  ["\uF0F8", "\uF0F8"], /* Private Use */
  ["\uF0F9", "\uF0F9"], /* Private Use */
  ["\uF0FA", "\uF0FA"], /* Private Use */
  ["\uF0FB", "\uF0FB"], /* Private Use */
  ["\uF0FC", "\uF0FC"], /* Private Use */
  ["\uF0FD", "\uF0FD"], /* Private Use */
  ["\uF0FE", "\uF0FE"], /* Private Use */
  ["\uF0FF", "\uF0FF"], /* Private Use */
  ["\uF100", "\uF100"], /* Private Use */
  ["\uF101", "\uF101"], /* Private Use */
  ["\uF102", "\uF102"], /* Private Use */
  ["\uF103", "\uF103"], /* Private Use */
  ["\uF104", "\uF104"], /* Private Use */
  ["\uF105", "\uF105"], /* Private Use */
  ["\uF106", "\uF106"], /* Private Use */
  ["\uF107", "\uF107"], /* Private Use */
  ["\uF108", "\uF108"], /* Private Use */
  ["\uF109", "\uF109"], /* Private Use */
  ["\uF10A", "\uF10A"], /* Private Use */
  ["\uF10B", "\uF10B"], /* Private Use */
  ["\uF10C", "\uF10C"], /* Private Use */
  ["\uF10D", "\uF10D"], /* Private Use */
  ["\uF10E", "\uF10E"], /* Private Use */
  ["\uF10F", "\uF10F"], /* Private Use */
  ["\uF110", "\uF110"], /* Private Use */
  ["\uF111", "\uF111"], /* Private Use */
  ["\uF112", "\uF112"], /* Private Use */
  ["\uF113", "\uF113"], /* Private Use */
  ["\uF114", "\uF114"], /* Private Use */
  ["\uF115", "\uF115"], /* Private Use */
  ["\uF116", "\uF116"], /* Private Use */
  ["\uF117", "\uF117"], /* Private Use */
  ["\uF118", "\uF118"], /* Private Use */
  ["\uF119", "\uF119"], /* Private Use */
  ["\uF11A", "\uF11A"], /* Private Use */
  ["\uF11B", "\uF11B"], /* Private Use */
  ["\uF11C", "\uF11C"], /* Private Use */
  ["\uF11D", "\uF11D"], /* Private Use */
  ["\uF11E", "\uF11E"], /* Private Use */
  ["\uF11F", "\uF11F"], /* Private Use */
  ["\uF120", "\uF120"], /* Private Use */
  ["\uF121", "\uF121"], /* Private Use */
  ["\uF122", "\uF122"], /* Private Use */
  ["\uF123", "\uF123"], /* Private Use */
  ["\uF124", "\uF124"], /* Private Use */
  ["\uF125", "\uF125"], /* Private Use */
  ["\uF126", "\uF126"], /* Private Use */
  ["\uF127", "\uF127"], /* Private Use */
  ["\uF128", "\uF128"], /* Private Use */
  ["\uF129", "\uF129"], /* Private Use */
  ["\uF12A", "\uF12A"], /* Private Use */
  ["\uF12B", "\uF12B"], /* Private Use */
  ["\uF12C", "\uF12C"], /* Private Use */
  ["\uF12D", "\uF12D"], /* Private Use */
  ["\uF12E", "\uF12E"], /* Private Use */
  ["\uF12F", "\uF12F"], /* Private Use */
  ["\uF130", "\uF130"], /* Private Use */
  ["\uF131", "\uF131"], /* Private Use */
  ["\uF132", "\uF132"], /* Private Use */
  ["\uF133", "\uF133"], /* Private Use */
  ["\uF134", "\uF134"], /* Private Use */
  ["\uF135", "\uF135"], /* Private Use */
  ["\uF136", "\uF136"], /* Private Use */
  ["\uF137", "\uF137"], /* Private Use */
  ["\uF138", "\uF138"], /* Private Use */
  ["\uF139", "\uF139"], /* Private Use */
  ["\uF13A", "\uF13A"], /* Private Use */
  ["\uF13B", "\uF13B"], /* Private Use */
  ["\uF13C", "\uF13C"], /* Private Use */
  ["\uF13D", "\uF13D"], /* Private Use */
  ["\uF13E", "\uF13E"], /* Private Use */
  ["\uF13F", "\uF13F"], /* Private Use */
  ["\uF140", "\uF140"], /* Private Use */
  ["\uF141", "\uF141"], /* Private Use */
  ["\uF142", "\uF142"], /* Private Use */
  ["\uF143", "\uF143"], /* Private Use */
  ["\uF144", "\uF144"], /* Private Use */
  ["\uF145", "\uF145"], /* Private Use */
  ["\uF146", "\uF146"], /* Private Use */
  ["\uF147", "\uF147"], /* Private Use */
  ["\uF148", "\uF148"], /* Private Use */
  ["\uF149", "\uF149"], /* Private Use */
  ["\uF14A", "\uF14A"], /* Private Use */
  ["\uF14B", "\uF14B"], /* Private Use */
  ["\uF14C", "\uF14C"], /* Private Use */
  ["\uF14D", "\uF14D"], /* Private Use */
  ["\uF14E", "\uF14E"], /* Private Use */
  ["\uF14F", "\uF14F"], /* Private Use */
  ["\uF150", "\uF150"], /* Private Use */
  ["\uF151", "\uF151"], /* Private Use */
  ["\uF152", "\uF152"], /* Private Use */
  ["\uF153", "\uF153"], /* Private Use */
  ["\uF154", "\uF154"], /* Private Use */
  ["\uF155", "\uF155"], /* Private Use */
  ["\uF156", "\uF156"], /* Private Use */
  ["\uF157", "\uF157"], /* Private Use */
  ["\uF158", "\uF158"], /* Private Use */
  ["\uF159", "\uF159"], /* Private Use */
  ["\uF15A", "\uF15A"], /* Private Use */
  ["\uF15B", "\uF15B"], /* Private Use */
  ["\uF15C", "\uF15C"], /* Private Use */
  ["\uF15D", "\uF15D"], /* Private Use */
  ["\uF15E", "\uF15E"], /* Private Use */
  ["\uF15F", "\uF15F"], /* Private Use */
  ["\uF160", "\uF160"], /* Private Use */
  ["\uF161", "\uF161"], /* Private Use */
  ["\uF162", "\uF162"], /* Private Use */
  ["\uF163", "\uF163"], /* Private Use */
  ["\uF164", "\uF164"], /* Private Use */
  ["\uF165", "\uF165"], /* Private Use */
  ["\uF166", "\uF166"], /* Private Use */
  ["\uF167", "\uF167"], /* Private Use */
  ["\uF168", "\uF168"], /* Private Use */
  ["\uF169", "\uF169"], /* Private Use */
  ["\uF16A", "\uF16A"], /* Private Use */
  ["\uF16B", "\uF16B"], /* Private Use */
  ["\uF16C", "\uF16C"], /* Private Use */
  ["\uF16D", "\uF16D"], /* Private Use */
  ["\uF16E", "\uF16E"], /* Private Use */
  ["\uF16F", "\uF16F"], /* Private Use */
  ["\uF170", "\uF170"], /* Private Use */
  ["\uF171", "\uF171"], /* Private Use */
  ["\uF172", "\uF172"], /* Private Use */
  ["\uF173", "\uF173"], /* Private Use */
  ["\uF174", "\uF174"], /* Private Use */
  ["\uF175", "\uF175"], /* Private Use */
  ["\uF176", "\uF176"], /* Private Use */
  ["\uF177", "\uF177"], /* Private Use */
  ["\uF178", "\uF178"], /* Private Use */
  ["\uF179", "\uF179"], /* Private Use */
  ["\uF17A", "\uF17A"], /* Private Use */
  ["\uF17B", "\uF17B"], /* Private Use */
  ["\uF17C", "\uF17C"], /* Private Use */
  ["\uF17D", "\uF17D"], /* Private Use */
  ["\uF17E", "\uF17E"], /* Private Use */
  ["\uF17F", "\uF17F"], /* Private Use */
  ["\uF180", "\uF180"], /* Private Use */
  ["\uF181", "\uF181"], /* Private Use */
  ["\uF182", "\uF182"], /* Private Use */
  ["\uF183", "\uF183"], /* Private Use */
  ["\uF184", "\uF184"], /* Private Use */
  ["\uF185", "\uF185"], /* Private Use */
  ["\uF186", "\uF186"], /* Private Use */
  ["\uF187", "\uF187"], /* Private Use */
  ["\uF188", "\uF188"], /* Private Use */
  ["\uF189", "\uF189"], /* Private Use */
  ["\uF18A", "\uF18A"], /* Private Use */
  ["\uF18B", "\uF18B"], /* Private Use */
  ["\uF18C", "\uF18C"], /* Private Use */
  ["\uF18D", "\uF18D"], /* Private Use */
  ["\uF18E", "\uF18E"], /* Private Use */
  ["\uF18F", "\uF18F"], /* Private Use */
  ["\uF190", "\uF190"], /* Private Use */
  ["\uF191", "\uF191"], /* Private Use */
  ["\uF192", "\uF192"], /* Private Use */
  ["\uF193", "\uF193"], /* Private Use */
  ["\uF194", "\uF194"], /* Private Use */
  ["\uF195", "\uF195"], /* Private Use */
  ["\uF196", "\uF196"], /* Private Use */
  ["\uF197", "\uF197"], /* Private Use */
  ["\uF198", "\uF198"], /* Private Use */
  ["\uF199", "\uF199"], /* Private Use */
  ["\uF19A", "\uF19A"], /* Private Use */
  ["\uF19B", "\uF19B"], /* Private Use */
  ["\uF19C", "\uF19C"], /* Private Use */
  ["\uF19D", "\uF19D"], /* Private Use */
  ["\uF19E", "\uF19E"], /* Private Use */
  ["\uF19F", "\uF19F"], /* Private Use */
  ["\uF1A0", "\uF1A0"], /* Private Use */
  ["\uF1A1", "\uF1A1"], /* Private Use */
  ["\uF1A2", "\uF1A2"], /* Private Use */
  ["\uF1A3", "\uF1A3"], /* Private Use */
  ["\uF1A4", "\uF1A4"], /* Private Use */
  ["\uF1A5", "\uF1A5"], /* Private Use */
  ["\uF1A6", "\uF1A6"], /* Private Use */
  ["\uF1A7", "\uF1A7"], /* Private Use */
  ["\uF1A8", "\uF1A8"], /* Private Use */
  ["\uF1A9", "\uF1A9"], /* Private Use */
  ["\uF1AA", "\uF1AA"], /* Private Use */
  ["\uF1AB", "\uF1AB"], /* Private Use */
  ["\uF1AC", "\uF1AC"], /* Private Use */
  ["\uF1AD", "\uF1AD"], /* Private Use */
  ["\uF1AE", "\uF1AE"], /* Private Use */
  ["\uF1AF", "\uF1AF"], /* Private Use */
  ["\uF1B0", "\uF1B0"], /* Private Use */
  ["\uF1B1", "\uF1B1"], /* Private Use */
  ["\uF1B2", "\uF1B2"], /* Private Use */
  ["\uF1B3", "\uF1B3"], /* Private Use */
  ["\uF1B4", "\uF1B4"], /* Private Use */
  ["\uF1B5", "\uF1B5"], /* Private Use */
  ["\uF1B6", "\uF1B6"], /* Private Use */
  ["\uF1B7", "\uF1B7"], /* Private Use */
  ["\uF1B8", "\uF1B8"], /* Private Use */
  ["\uF1B9", "\uF1B9"], /* Private Use */
  ["\uF1BA", "\uF1BA"], /* Private Use */
  ["\uF1BB", "\uF1BB"], /* Private Use */
  ["\uF1BC", "\uF1BC"], /* Private Use */
  ["\uF1BD", "\uF1BD"], /* Private Use */
  ["\uF1BE", "\uF1BE"], /* Private Use */
  ["\uF1BF", "\uF1BF"], /* Private Use */
  ["\uF1C0", "\uF1C0"], /* Private Use */
  ["\uF1C1", "\uF1C1"], /* Private Use */
  ["\uF1C2", "\uF1C2"], /* Private Use */
  ["\uF1C3", "\uF1C3"], /* Private Use */
  ["\uF1C4", "\uF1C4"], /* Private Use */
  ["\uF1C5", "\uF1C5"], /* Private Use */
  ["\uF1C6", "\uF1C6"], /* Private Use */
  ["\uF1C7", "\uF1C7"], /* Private Use */
  ["\uF1C8", "\uF1C8"], /* Private Use */
  ["\uF1C9", "\uF1C9"], /* Private Use */
  ["\uF1CA", "\uF1CA"], /* Private Use */
  ["\uF1CB", "\uF1CB"], /* Private Use */
  ["\uF1CC", "\uF1CC"], /* Private Use */
  ["\uF1CD", "\uF1CD"], /* Private Use */
  ["\uF1CE", "\uF1CE"], /* Private Use */
  ["\uF1CF", "\uF1CF"], /* Private Use */
  ["\uF1D0", "\uF1D0"], /* Private Use */
  ["\uF1D1", "\uF1D1"], /* Private Use */
  ["\uF1D2", "\uF1D2"], /* Private Use */
  ["\uF1D3", "\uF1D3"], /* Private Use */
  ["\uF1D4", "\uF1D4"], /* Private Use */
  ["\uF1D5", "\uF1D5"], /* Private Use */
  ["\uF1D6", "\uF1D6"], /* Private Use */
  ["\uF1D7", "\uF1D7"], /* Private Use */
  ["\uF1D8", "\uF1D8"], /* Private Use */
  ["\uF1D9", "\uF1D9"], /* Private Use */
  ["\uF1DA", "\uF1DA"], /* Private Use */
  ["\uF1DB", "\uF1DB"], /* Private Use */
  ["\uF1DC", "\uF1DC"], /* Private Use */
  ["\uF1DD", "\uF1DD"], /* Private Use */
  ["\uF1DE", "\uF1DE"], /* Private Use */
  ["\uF1DF", "\uF1DF"], /* Private Use */
  ["\uF1E0", "\uF1E0"], /* Private Use */
  ["\uF1E1", "\uF1E1"], /* Private Use */
  ["\uF1E2", "\uF1E2"], /* Private Use */
  ["\uF1E3", "\uF1E3"], /* Private Use */
  ["\uF1E4", "\uF1E4"], /* Private Use */
  ["\uF1E5", "\uF1E5"], /* Private Use */
  ["\uF1E6", "\uF1E6"], /* Private Use */
  ["\uF1E7", "\uF1E7"], /* Private Use */
  ["\uF1E8", "\uF1E8"], /* Private Use */
  ["\uF1E9", "\uF1E9"], /* Private Use */
  ["\uF1EA", "\uF1EA"], /* Private Use */
  ["\uF1EB", "\uF1EB"], /* Private Use */
  ["\uF1EC", "\uF1EC"], /* Private Use */
  ["\uF1ED", "\uF1ED"], /* Private Use */
  ["\uF1EE", "\uF1EE"], /* Private Use */
  ["\uF1EF", "\uF1EF"], /* Private Use */
  ["\uF1F0", "\uF1F0"], /* Private Use */
  ["\uF1F1", "\uF1F1"], /* Private Use */
  ["\uF1F2", "\uF1F2"], /* Private Use */
  ["\uF1F3", "\uF1F3"], /* Private Use */
  ["\uF1F4", "\uF1F4"], /* Private Use */
  ["\uF1F5", "\uF1F5"], /* Private Use */
  ["\uF1F6", "\uF1F6"], /* Private Use */
  ["\uF1F7", "\uF1F7"], /* Private Use */
  ["\uF1F8", "\uF1F8"], /* Private Use */
  ["\uF1F9", "\uF1F9"], /* Private Use */
  ["\uF1FA", "\uF1FA"], /* Private Use */
  ["\uF1FB", "\uF1FB"], /* Private Use */
  ["\uF1FC", "\uF1FC"], /* Private Use */
  ["\uF1FD", "\uF1FD"], /* Private Use */
  ["\uF1FE", "\uF1FE"], /* Private Use */
  ["\uF1FF", "\uF1FF"], /* Private Use */
  ["\uF200", "\uF200"], /* Private Use */
  ["\uF201", "\uF201"], /* Private Use */
  ["\uF202", "\uF202"], /* Private Use */
  ["\uF203", "\uF203"], /* Private Use */
  ["\uF204", "\uF204"], /* Private Use */
  ["\uF205", "\uF205"], /* Private Use */
  ["\uF206", "\uF206"], /* Private Use */
  ["\uF207", "\uF207"], /* Private Use */
  ["\uF208", "\uF208"], /* Private Use */
  ["\uF209", "\uF209"], /* Private Use */
  ["\uF20A", "\uF20A"], /* Private Use */
  ["\uF20B", "\uF20B"], /* Private Use */
  ["\uF20C", "\uF20C"], /* Private Use */
  ["\uF20D", "\uF20D"], /* Private Use */
  ["\uF20E", "\uF20E"], /* Private Use */
  ["\uF20F", "\uF20F"], /* Private Use */
  ["\uF210", "\uF210"], /* Private Use */
  ["\uF211", "\uF211"], /* Private Use */
  ["\uF212", "\uF212"], /* Private Use */
  ["\uF213", "\uF213"], /* Private Use */
  ["\uF214", "\uF214"], /* Private Use */
  ["\uF215", "\uF215"], /* Private Use */
  ["\uF216", "\uF216"], /* Private Use */
  ["\uF217", "\uF217"], /* Private Use */
  ["\uF218", "\uF218"], /* Private Use */
  ["\uF219", "\uF219"], /* Private Use */
  ["\uF21A", "\uF21A"], /* Private Use */
  ["\uF21B", "\uF21B"], /* Private Use */
  ["\uF21C", "\uF21C"], /* Private Use */
  ["\uF21D", "\uF21D"], /* Private Use */
  ["\uF21E", "\uF21E"], /* Private Use */
  ["\uF21F", "\uF21F"], /* Private Use */
  ["\uF220", "\uF220"], /* Private Use */
  ["\uF221", "\uF221"], /* Private Use */
  ["\uF222", "\uF222"], /* Private Use */
  ["\uF223", "\uF223"], /* Private Use */
  ["\uF224", "\uF224"], /* Private Use */
  ["\uF225", "\uF225"], /* Private Use */
  ["\uF226", "\uF226"], /* Private Use */
  ["\uF227", "\uF227"], /* Private Use */
  ["\uF228", "\uF228"], /* Private Use */
  ["\uF229", "\uF229"], /* Private Use */
  ["\uF22A", "\uF22A"], /* Private Use */
  ["\uF22B", "\uF22B"], /* Private Use */
  ["\uF22C", "\uF22C"], /* Private Use */
  ["\uF22D", "\uF22D"], /* Private Use */
  ["\uF22E", "\uF22E"], /* Private Use */
  ["\uF22F", "\uF22F"], /* Private Use */
  ["\uF230", "\uF230"], /* Private Use */
  ["\uF231", "\uF231"], /* Private Use */
  ["\uF232", "\uF232"], /* Private Use */
  ["\uF233", "\uF233"], /* Private Use */
  ["\uF234", "\uF234"], /* Private Use */
  ["\uF235", "\uF235"], /* Private Use */
  ["\uF236", "\uF236"], /* Private Use */
  ["\uF237", "\uF237"], /* Private Use */
  ["\uF238", "\uF238"], /* Private Use */
  ["\uF239", "\uF239"], /* Private Use */
  ["\uF23A", "\uF23A"], /* Private Use */
  ["\uF23B", "\uF23B"], /* Private Use */
  ["\uF23C", "\uF23C"], /* Private Use */
  ["\uF23D", "\uF23D"], /* Private Use */
  ["\uF23E", "\uF23E"], /* Private Use */
  ["\uF23F", "\uF23F"], /* Private Use */
  ["\uF240", "\uF240"], /* Private Use */
  ["\uF241", "\uF241"], /* Private Use */
  ["\uF242", "\uF242"], /* Private Use */
  ["\uF243", "\uF243"], /* Private Use */
  ["\uF244", "\uF244"], /* Private Use */
  ["\uF245", "\uF245"], /* Private Use */
  ["\uF246", "\uF246"], /* Private Use */
  ["\uF247", "\uF247"], /* Private Use */
  ["\uF248", "\uF248"], /* Private Use */
  ["\uF249", "\uF249"], /* Private Use */
  ["\uF24A", "\uF24A"], /* Private Use */
  ["\uF24B", "\uF24B"], /* Private Use */
  ["\uF24C", "\uF24C"], /* Private Use */
  ["\uF24D", "\uF24D"], /* Private Use */
  ["\uF24E", "\uF24E"], /* Private Use */
  ["\uF24F", "\uF24F"], /* Private Use */
  ["\uF250", "\uF250"], /* Private Use */
  ["\uF251", "\uF251"], /* Private Use */
  ["\uF252", "\uF252"], /* Private Use */
  ["\uF253", "\uF253"], /* Private Use */
  ["\uF254", "\uF254"], /* Private Use */
  ["\uF255", "\uF255"], /* Private Use */
  ["\uF256", "\uF256"], /* Private Use */
  ["\uF257", "\uF257"], /* Private Use */
  ["\uF258", "\uF258"], /* Private Use */
  ["\uF259", "\uF259"], /* Private Use */
  ["\uF25A", "\uF25A"], /* Private Use */
  ["\uF25B", "\uF25B"], /* Private Use */
  ["\uF25C", "\uF25C"], /* Private Use */
  ["\uF25D", "\uF25D"], /* Private Use */
  ["\uF25E", "\uF25E"], /* Private Use */
  ["\uF25F", "\uF25F"], /* Private Use */
  ["\uF260", "\uF260"], /* Private Use */
  ["\uF261", "\uF261"], /* Private Use */
  ["\uF262", "\uF262"], /* Private Use */
  ["\uF263", "\uF263"], /* Private Use */
  ["\uF264", "\uF264"], /* Private Use */
  ["\uF265", "\uF265"], /* Private Use */
  ["\uF266", "\uF266"], /* Private Use */
  ["\uF267", "\uF267"], /* Private Use */
  ["\uF268", "\uF268"], /* Private Use */
  ["\uF269", "\uF269"], /* Private Use */
  ["\uF26A", "\uF26A"], /* Private Use */
  ["\uF26B", "\uF26B"], /* Private Use */
  ["\uF26C", "\uF26C"], /* Private Use */
  ["\uF26D", "\uF26D"], /* Private Use */
  ["\uF26E", "\uF26E"], /* Private Use */
  ["\uF26F", "\uF26F"], /* Private Use */
  ["\uF270", "\uF270"], /* Private Use */
  ["\uF271", "\uF271"], /* Private Use */
  ["\uF272", "\uF272"], /* Private Use */
  ["\uF273", "\uF273"], /* Private Use */
  ["\uF274", "\uF274"], /* Private Use */
  ["\uF275", "\uF275"], /* Private Use */
  ["\uF276", "\uF276"], /* Private Use */
  ["\uF277", "\uF277"], /* Private Use */
  ["\uF278", "\uF278"], /* Private Use */
  ["\uF279", "\uF279"], /* Private Use */
  ["\uF27A", "\uF27A"], /* Private Use */
  ["\uF27B", "\uF27B"], /* Private Use */
  ["\uF27C", "\uF27C"], /* Private Use */
  ["\uF27D", "\uF27D"], /* Private Use */
  ["\uF27E", "\uF27E"], /* Private Use */
  ["\uF27F", "\uF27F"], /* Private Use */
  ["\uF280", "\uF280"], /* Private Use */
  ["\uF281", "\uF281"], /* Private Use */
  ["\uF282", "\uF282"], /* Private Use */
  ["\uF283", "\uF283"], /* Private Use */
  ["\uF284", "\uF284"], /* Private Use */
  ["\uF285", "\uF285"], /* Private Use */
  ["\uF286", "\uF286"], /* Private Use */
  ["\uF287", "\uF287"], /* Private Use */
  ["\uF288", "\uF288"], /* Private Use */
  ["\uF289", "\uF289"], /* Private Use */
  ["\uF28A", "\uF28A"], /* Private Use */
  ["\uF28B", "\uF28B"], /* Private Use */
  ["\uF28C", "\uF28C"], /* Private Use */
  ["\uF28D", "\uF28D"], /* Private Use */
  ["\uF28E", "\uF28E"], /* Private Use */
  ["\uF28F", "\uF28F"], /* Private Use */
  ["\uF290", "\uF290"], /* Private Use */
  ["\uF291", "\uF291"], /* Private Use */
  ["\uF292", "\uF292"], /* Private Use */
  ["\uF293", "\uF293"], /* Private Use */
  ["\uF294", "\uF294"], /* Private Use */
  ["\uF295", "\uF295"], /* Private Use */
  ["\uF296", "\uF296"], /* Private Use */
  ["\uF297", "\uF297"], /* Private Use */
  ["\uF298", "\uF298"], /* Private Use */
  ["\uF299", "\uF299"], /* Private Use */
  ["\uF29A", "\uF29A"], /* Private Use */
  ["\uF29B", "\uF29B"], /* Private Use */
  ["\uF29C", "\uF29C"], /* Private Use */
  ["\uF29D", "\uF29D"], /* Private Use */
  ["\uF29E", "\uF29E"], /* Private Use */
  ["\uF29F", "\uF29F"], /* Private Use */
  ["\uF2A0", "\uF2A0"], /* Private Use */
  ["\uF2A1", "\uF2A1"], /* Private Use */
  ["\uF2A2", "\uF2A2"], /* Private Use */
  ["\uF2A3", "\uF2A3"], /* Private Use */
  ["\uF2A4", "\uF2A4"], /* Private Use */
  ["\uF2A5", "\uF2A5"], /* Private Use */
  ["\uF2A6", "\uF2A6"], /* Private Use */
  ["\uF2A7", "\uF2A7"], /* Private Use */
  ["\uF2A8", "\uF2A8"], /* Private Use */
  ["\uF2A9", "\uF2A9"], /* Private Use */
  ["\uF2AA", "\uF2AA"], /* Private Use */
  ["\uF2AB", "\uF2AB"], /* Private Use */
  ["\uF2AC", "\uF2AC"], /* Private Use */
  ["\uF2AD", "\uF2AD"], /* Private Use */
  ["\uF2AE", "\uF2AE"], /* Private Use */
  ["\uF2AF", "\uF2AF"], /* Private Use */
  ["\uF2B0", "\uF2B0"], /* Private Use */
  ["\uF2B1", "\uF2B1"], /* Private Use */
  ["\uF2B2", "\uF2B2"], /* Private Use */
  ["\uF2B3", "\uF2B3"], /* Private Use */
  ["\uF2B4", "\uF2B4"], /* Private Use */
  ["\uF2B5", "\uF2B5"], /* Private Use */
  ["\uF2B6", "\uF2B6"], /* Private Use */
  ["\uF2B7", "\uF2B7"], /* Private Use */
  ["\uF2B8", "\uF2B8"], /* Private Use */
  ["\uF2B9", "\uF2B9"], /* Private Use */
  ["\uF2BA", "\uF2BA"], /* Private Use */
  ["\uF2BB", "\uF2BB"], /* Private Use */
  ["\uF2BC", "\uF2BC"], /* Private Use */
  ["\uF2BD", "\uF2BD"], /* Private Use */
  ["\uF2BE", "\uF2BE"], /* Private Use */
  ["\uF2BF", "\uF2BF"], /* Private Use */
  ["\uF2C0", "\uF2C0"], /* Private Use */
  ["\uF2C1", "\uF2C1"], /* Private Use */
  ["\uF2C2", "\uF2C2"], /* Private Use */
  ["\uF2C3", "\uF2C3"], /* Private Use */
  ["\uF2C4", "\uF2C4"], /* Private Use */
  ["\uF2C5", "\uF2C5"], /* Private Use */
  ["\uF2C6", "\uF2C6"], /* Private Use */
  ["\uF2C7", "\uF2C7"], /* Private Use */
  ["\uF2C8", "\uF2C8"], /* Private Use */
  ["\uF2C9", "\uF2C9"], /* Private Use */
  ["\uF2CA", "\uF2CA"], /* Private Use */
  ["\uF2CB", "\uF2CB"], /* Private Use */
  ["\uF2CC", "\uF2CC"], /* Private Use */
  ["\uF2CD", "\uF2CD"], /* Private Use */
  ["\uF2CE", "\uF2CE"], /* Private Use */
  ["\uF2CF", "\uF2CF"], /* Private Use */
  ["\uF2D0", "\uF2D0"], /* Private Use */
  ["\uF2D1", "\uF2D1"], /* Private Use */
  ["\uF2D2", "\uF2D2"], /* Private Use */
  ["\uF2D3", "\uF2D3"], /* Private Use */
  ["\uF2D4", "\uF2D4"], /* Private Use */
  ["\uF2D5", "\uF2D5"], /* Private Use */
  ["\uF2D6", "\uF2D6"], /* Private Use */
  ["\uF2D7", "\uF2D7"], /* Private Use */
  ["\uF2D8", "\uF2D8"], /* Private Use */
  ["\uF2D9", "\uF2D9"], /* Private Use */
  ["\uF2DA", "\uF2DA"], /* Private Use */
  ["\uF2DB", "\uF2DB"], /* Private Use */
  ["\uF2DC", "\uF2DC"], /* Private Use */
  ["\uF2DD", "\uF2DD"], /* Private Use */
  ["\uF2DE", "\uF2DE"], /* Private Use */
  ["\uF2DF", "\uF2DF"], /* Private Use */
  ["\uF2E0", "\uF2E0"], /* Private Use */
  ["\uF2E1", "\uF2E1"], /* Private Use */
  ["\uF2E2", "\uF2E2"], /* Private Use */
  ["\uF2E3", "\uF2E3"], /* Private Use */
  ["\uF2E4", "\uF2E4"], /* Private Use */
  ["\uF2E5", "\uF2E5"], /* Private Use */
  ["\uF2E6", "\uF2E6"], /* Private Use */
  ["\uF2E7", "\uF2E7"], /* Private Use */
  ["\uF2E8", "\uF2E8"], /* Private Use */
  ["\uF2E9", "\uF2E9"], /* Private Use */
  ["\uF2EA", "\uF2EA"], /* Private Use */
  ["\uF2EB", "\uF2EB"], /* Private Use */
  ["\uF2EC", "\uF2EC"], /* Private Use */
  ["\uF2ED", "\uF2ED"], /* Private Use */
  ["\uF2EE", "\uF2EE"], /* Private Use */
  ["\uF2EF", "\uF2EF"], /* Private Use */
  ["\uF2F0", "\uF2F0"], /* Private Use */
  ["\uF2F1", "\uF2F1"], /* Private Use */
  ["\uF2F2", "\uF2F2"], /* Private Use */
  ["\uF2F3", "\uF2F3"], /* Private Use */
  ["\uF2F4", "\uF2F4"], /* Private Use */
  ["\uF2F5", "\uF2F5"], /* Private Use */
  ["\uF2F6", "\uF2F6"], /* Private Use */
  ["\uF2F7", "\uF2F7"], /* Private Use */
  ["\uF2F8", "\uF2F8"], /* Private Use */
  ["\uF2F9", "\uF2F9"], /* Private Use */
  ["\uF2FA", "\uF2FA"], /* Private Use */
  ["\uF2FB", "\uF2FB"], /* Private Use */
  ["\uF2FC", "\uF2FC"], /* Private Use */
  ["\uF2FD", "\uF2FD"], /* Private Use */
  ["\uF2FE", "\uF2FE"], /* Private Use */
  ["\uF2FF", "\uF2FF"], /* Private Use */
  ["\uF300", "\uF300"], /* Private Use */
  ["\uF301", "\uF301"], /* Private Use */
  ["\uF302", "\uF302"], /* Private Use */
  ["\uF303", "\uF303"], /* Private Use */
  ["\uF304", "\uF304"], /* Private Use */
  ["\uF305", "\uF305"], /* Private Use */
  ["\uF306", "\uF306"], /* Private Use */
  ["\uF307", "\uF307"], /* Private Use */
  ["\uF308", "\uF308"], /* Private Use */
  ["\uF309", "\uF309"], /* Private Use */
  ["\uF30A", "\uF30A"], /* Private Use */
  ["\uF30B", "\uF30B"], /* Private Use */
  ["\uF30C", "\uF30C"], /* Private Use */
  ["\uF30D", "\uF30D"], /* Private Use */
  ["\uF30E", "\uF30E"], /* Private Use */
  ["\uF30F", "\uF30F"], /* Private Use */
  ["\uF310", "\uF310"], /* Private Use */
  ["\uF311", "\uF311"], /* Private Use */
  ["\uF312", "\uF312"], /* Private Use */
  ["\uF313", "\uF313"], /* Private Use */
  ["\uF314", "\uF314"], /* Private Use */
  ["\uF315", "\uF315"], /* Private Use */
  ["\uF316", "\uF316"], /* Private Use */
  ["\uF317", "\uF317"], /* Private Use */
  ["\uF318", "\uF318"], /* Private Use */
  ["\uF319", "\uF319"], /* Private Use */
  ["\uF31A", "\uF31A"], /* Private Use */
  ["\uF31B", "\uF31B"], /* Private Use */
  ["\uF31C", "\uF31C"], /* Private Use */
  ["\uF31D", "\uF31D"], /* Private Use */
  ["\uF31E", "\uF31E"], /* Private Use */
  ["\uF31F", "\uF31F"], /* Private Use */
  ["\uF320", "\uF320"], /* Private Use */
  ["\uF321", "\uF321"], /* Private Use */
  ["\uF322", "\uF322"], /* Private Use */
  ["\uF323", "\uF323"], /* Private Use */
  ["\uF324", "\uF324"], /* Private Use */
  ["\uF325", "\uF325"], /* Private Use */
  ["\uF326", "\uF326"], /* Private Use */
  ["\uF327", "\uF327"], /* Private Use */
  ["\uF328", "\uF328"], /* Private Use */
  ["\uF329", "\uF329"], /* Private Use */
  ["\uF32A", "\uF32A"], /* Private Use */
  ["\uF32B", "\uF32B"], /* Private Use */
  ["\uF32C", "\uF32C"], /* Private Use */
  ["\uF32D", "\uF32D"], /* Private Use */
  ["\uF32E", "\uF32E"], /* Private Use */
  ["\uF32F", "\uF32F"], /* Private Use */
  ["\uF330", "\uF330"], /* Private Use */
  ["\uF331", "\uF331"], /* Private Use */
  ["\uF332", "\uF332"], /* Private Use */
  ["\uF333", "\uF333"], /* Private Use */
  ["\uF334", "\uF334"], /* Private Use */
  ["\uF335", "\uF335"], /* Private Use */
  ["\uF336", "\uF336"], /* Private Use */
  ["\uF337", "\uF337"], /* Private Use */
  ["\uF338", "\uF338"], /* Private Use */
  ["\uF339", "\uF339"], /* Private Use */
  ["\uF33A", "\uF33A"], /* Private Use */
  ["\uF33B", "\uF33B"], /* Private Use */
  ["\uF33C", "\uF33C"], /* Private Use */
  ["\uF33D", "\uF33D"], /* Private Use */
  ["\uF33E", "\uF33E"], /* Private Use */
  ["\uF33F", "\uF33F"], /* Private Use */
  ["\uF340", "\uF340"], /* Private Use */
  ["\uF341", "\uF341"], /* Private Use */
  ["\uF342", "\uF342"], /* Private Use */
  ["\uF343", "\uF343"], /* Private Use */
  ["\uF344", "\uF344"], /* Private Use */
  ["\uF345", "\uF345"], /* Private Use */
  ["\uF346", "\uF346"], /* Private Use */
  ["\uF347", "\uF347"], /* Private Use */
  ["\uF348", "\uF348"], /* Private Use */
  ["\uF349", "\uF349"], /* Private Use */
  ["\uF34A", "\uF34A"], /* Private Use */
  ["\uF34B", "\uF34B"], /* Private Use */
  ["\uF34C", "\uF34C"], /* Private Use */
  ["\uF34D", "\uF34D"], /* Private Use */
  ["\uF34E", "\uF34E"], /* Private Use */
  ["\uF34F", "\uF34F"], /* Private Use */
  ["\uF350", "\uF350"], /* Private Use */
  ["\uF351", "\uF351"], /* Private Use */
  ["\uF352", "\uF352"], /* Private Use */
  ["\uF353", "\uF353"], /* Private Use */
  ["\uF354", "\uF354"], /* Private Use */
  ["\uF355", "\uF355"], /* Private Use */
  ["\uF356", "\uF356"], /* Private Use */
  ["\uF357", "\uF357"], /* Private Use */
  ["\uF358", "\uF358"], /* Private Use */
  ["\uF359", "\uF359"], /* Private Use */
  ["\uF35A", "\uF35A"], /* Private Use */
  ["\uF35B", "\uF35B"], /* Private Use */
  ["\uF35C", "\uF35C"], /* Private Use */
  ["\uF35D", "\uF35D"], /* Private Use */
  ["\uF35E", "\uF35E"], /* Private Use */
  ["\uF35F", "\uF35F"], /* Private Use */
  ["\uF360", "\uF360"], /* Private Use */
  ["\uF361", "\uF361"], /* Private Use */
  ["\uF362", "\uF362"], /* Private Use */
  ["\uF363", "\uF363"], /* Private Use */
  ["\uF364", "\uF364"], /* Private Use */
  ["\uF365", "\uF365"], /* Private Use */
  ["\uF366", "\uF366"], /* Private Use */
  ["\uF367", "\uF367"], /* Private Use */
  ["\uF368", "\uF368"], /* Private Use */
  ["\uF369", "\uF369"], /* Private Use */
  ["\uF36A", "\uF36A"], /* Private Use */
  ["\uF36B", "\uF36B"], /* Private Use */
  ["\uF36C", "\uF36C"], /* Private Use */
  ["\uF36D", "\uF36D"], /* Private Use */
  ["\uF36E", "\uF36E"], /* Private Use */
  ["\uF36F", "\uF36F"], /* Private Use */
  ["\uF370", "\uF370"], /* Private Use */
  ["\uF371", "\uF371"], /* Private Use */
  ["\uF372", "\uF372"], /* Private Use */
  ["\uF373", "\uF373"], /* Private Use */
  ["\uF374", "\uF374"], /* Private Use */
  ["\uF375", "\uF375"], /* Private Use */
  ["\uF376", "\uF376"], /* Private Use */
  ["\uF377", "\uF377"], /* Private Use */
  ["\uF378", "\uF378"], /* Private Use */
  ["\uF379", "\uF379"], /* Private Use */
  ["\uF37A", "\uF37A"], /* Private Use */
  ["\uF37B", "\uF37B"], /* Private Use */
  ["\uF37C", "\uF37C"], /* Private Use */
  ["\uF37D", "\uF37D"], /* Private Use */
  ["\uF37E", "\uF37E"], /* Private Use */
  ["\uF37F", "\uF37F"], /* Private Use */
  ["\uF380", "\uF380"], /* Private Use */
  ["\uF381", "\uF381"], /* Private Use */
  ["\uF382", "\uF382"], /* Private Use */
  ["\uF383", "\uF383"], /* Private Use */
  ["\uF384", "\uF384"], /* Private Use */
  ["\uF385", "\uF385"], /* Private Use */
  ["\uF386", "\uF386"], /* Private Use */
  ["\uF387", "\uF387"], /* Private Use */
  ["\uF388", "\uF388"], /* Private Use */
  ["\uF389", "\uF389"], /* Private Use */
  ["\uF38A", "\uF38A"], /* Private Use */
  ["\uF38B", "\uF38B"], /* Private Use */
  ["\uF38C", "\uF38C"], /* Private Use */
  ["\uF38D", "\uF38D"], /* Private Use */
  ["\uF38E", "\uF38E"], /* Private Use */
  ["\uF38F", "\uF38F"], /* Private Use */
  ["\uF390", "\uF390"], /* Private Use */
  ["\uF391", "\uF391"], /* Private Use */
  ["\uF392", "\uF392"], /* Private Use */
  ["\uF393", "\uF393"], /* Private Use */
  ["\uF394", "\uF394"], /* Private Use */
  ["\uF395", "\uF395"], /* Private Use */
  ["\uF396", "\uF396"], /* Private Use */
  ["\uF397", "\uF397"], /* Private Use */
  ["\uF398", "\uF398"], /* Private Use */
  ["\uF399", "\uF399"], /* Private Use */
  ["\uF39A", "\uF39A"], /* Private Use */
  ["\uF39B", "\uF39B"], /* Private Use */
  ["\uF39C", "\uF39C"], /* Private Use */
  ["\uF39D", "\uF39D"], /* Private Use */
  ["\uF39E", "\uF39E"], /* Private Use */
  ["\uF39F", "\uF39F"], /* Private Use */
  ["\uF3A0", "\uF3A0"], /* Private Use */
  ["\uF3A1", "\uF3A1"], /* Private Use */
  ["\uF3A2", "\uF3A2"], /* Private Use */
  ["\uF3A3", "\uF3A3"], /* Private Use */
  ["\uF3A4", "\uF3A4"], /* Private Use */
  ["\uF3A5", "\uF3A5"], /* Private Use */
  ["\uF3A6", "\uF3A6"], /* Private Use */
  ["\uF3A7", "\uF3A7"], /* Private Use */
  ["\uF3A8", "\uF3A8"], /* Private Use */
  ["\uF3A9", "\uF3A9"], /* Private Use */
  ["\uF3AA", "\uF3AA"], /* Private Use */
  ["\uF3AB", "\uF3AB"], /* Private Use */
  ["\uF3AC", "\uF3AC"], /* Private Use */
  ["\uF3AD", "\uF3AD"], /* Private Use */
  ["\uF3AE", "\uF3AE"], /* Private Use */
  ["\uF3AF", "\uF3AF"], /* Private Use */
  ["\uF3B0", "\uF3B0"], /* Private Use */
  ["\uF3B1", "\uF3B1"], /* Private Use */
  ["\uF3B2", "\uF3B2"], /* Private Use */
  ["\uF3B3", "\uF3B3"], /* Private Use */
  ["\uF3B4", "\uF3B4"], /* Private Use */
  ["\uF3B5", "\uF3B5"], /* Private Use */
  ["\uF3B6", "\uF3B6"], /* Private Use */
  ["\uF3B7", "\uF3B7"], /* Private Use */
  ["\uF3B8", "\uF3B8"], /* Private Use */
  ["\uF3B9", "\uF3B9"], /* Private Use */
  ["\uF3BA", "\uF3BA"], /* Private Use */
  ["\uF3BB", "\uF3BB"], /* Private Use */
  ["\uF3BC", "\uF3BC"], /* Private Use */
  ["\uF3BD", "\uF3BD"], /* Private Use */
  ["\uF3BE", "\uF3BE"], /* Private Use */
  ["\uF3BF", "\uF3BF"], /* Private Use */
  ["\uF3C0", "\uF3C0"], /* Private Use */
  ["\uF3C1", "\uF3C1"], /* Private Use */
  ["\uF3C2", "\uF3C2"], /* Private Use */
  ["\uF3C3", "\uF3C3"], /* Private Use */
  ["\uF3C4", "\uF3C4"], /* Private Use */
  ["\uF3C5", "\uF3C5"], /* Private Use */
  ["\uF3C6", "\uF3C6"], /* Private Use */
  ["\uF3C7", "\uF3C7"], /* Private Use */
  ["\uF3C8", "\uF3C8"], /* Private Use */
  ["\uF3C9", "\uF3C9"], /* Private Use */
  ["\uF3CA", "\uF3CA"], /* Private Use */
  ["\uF3CB", "\uF3CB"], /* Private Use */
  ["\uF3CC", "\uF3CC"], /* Private Use */
  ["\uF3CD", "\uF3CD"], /* Private Use */
  ["\uF3CE", "\uF3CE"], /* Private Use */
  ["\uF3CF", "\uF3CF"], /* Private Use */
  ["\uF3D0", "\uF3D0"], /* Private Use */
  ["\uF3D1", "\uF3D1"], /* Private Use */
  ["\uF3D2", "\uF3D2"], /* Private Use */
  ["\uF3D3", "\uF3D3"], /* Private Use */
  ["\uF3D4", "\uF3D4"], /* Private Use */
  ["\uF3D5", "\uF3D5"], /* Private Use */
  ["\uF3D6", "\uF3D6"], /* Private Use */
  ["\uF3D7", "\uF3D7"], /* Private Use */
  ["\uF3D8", "\uF3D8"], /* Private Use */
  ["\uF3D9", "\uF3D9"], /* Private Use */
  ["\uF3DA", "\uF3DA"], /* Private Use */
  ["\uF3DB", "\uF3DB"], /* Private Use */
  ["\uF3DC", "\uF3DC"], /* Private Use */
  ["\uF3DD", "\uF3DD"], /* Private Use */
  ["\uF3DE", "\uF3DE"], /* Private Use */
  ["\uF3DF", "\uF3DF"], /* Private Use */
  ["\uF3E0", "\uF3E0"], /* Private Use */
  ["\uF3E1", "\uF3E1"], /* Private Use */
  ["\uF3E2", "\uF3E2"], /* Private Use */
  ["\uF3E3", "\uF3E3"], /* Private Use */
  ["\uF3E4", "\uF3E4"], /* Private Use */
  ["\uF3E5", "\uF3E5"], /* Private Use */
  ["\uF3E6", "\uF3E6"], /* Private Use */
  ["\uF3E7", "\uF3E7"], /* Private Use */
  ["\uF3E8", "\uF3E8"], /* Private Use */
  ["\uF3E9", "\uF3E9"], /* Private Use */
  ["\uF3EA", "\uF3EA"], /* Private Use */
  ["\uF3EB", "\uF3EB"], /* Private Use */
  ["\uF3EC", "\uF3EC"], /* Private Use */
  ["\uF3ED", "\uF3ED"], /* Private Use */
  ["\uF3EE", "\uF3EE"], /* Private Use */
  ["\uF3EF", "\uF3EF"], /* Private Use */
  ["\uF3F0", "\uF3F0"], /* Private Use */
  ["\uF3F1", "\uF3F1"], /* Private Use */
  ["\uF3F2", "\uF3F2"], /* Private Use */
  ["\uF3F3", "\uF3F3"], /* Private Use */
  ["\uF3F4", "\uF3F4"], /* Private Use */
  ["\uF3F5", "\uF3F5"], /* Private Use */
  ["\uF3F6", "\uF3F6"], /* Private Use */
  ["\uF3F7", "\uF3F7"], /* Private Use */
  ["\uF3F8", "\uF3F8"], /* Private Use */
  ["\uF3F9", "\uF3F9"], /* Private Use */
  ["\uF3FA", "\uF3FA"], /* Private Use */
  ["\uF3FB", "\uF3FB"], /* Private Use */
  ["\uF3FC", "\uF3FC"], /* Private Use */
  ["\uF3FD", "\uF3FD"], /* Private Use */
  ["\uF3FE", "\uF3FE"], /* Private Use */
  ["\uF3FF", "\uF3FF"], /* Private Use */
  ["\uF400", "\uF400"], /* Private Use */
  ["\uF401", "\uF401"], /* Private Use */
  ["\uF402", "\uF402"], /* Private Use */
  ["\uF403", "\uF403"], /* Private Use */
  ["\uF404", "\uF404"], /* Private Use */
  ["\uF405", "\uF405"], /* Private Use */
  ["\uF406", "\uF406"], /* Private Use */
  ["\uF407", "\uF407"], /* Private Use */
  ["\uF408", "\uF408"], /* Private Use */
  ["\uF409", "\uF409"], /* Private Use */
  ["\uF40A", "\uF40A"], /* Private Use */
  ["\uF40B", "\uF40B"], /* Private Use */
  ["\uF40C", "\uF40C"], /* Private Use */
  ["\uF40D", "\uF40D"], /* Private Use */
  ["\uF40E", "\uF40E"], /* Private Use */
  ["\uF40F", "\uF40F"], /* Private Use */
  ["\uF410", "\uF410"], /* Private Use */
  ["\uF411", "\uF411"], /* Private Use */
  ["\uF412", "\uF412"], /* Private Use */
  ["\uF413", "\uF413"], /* Private Use */
  ["\uF414", "\uF414"], /* Private Use */
  ["\uF415", "\uF415"], /* Private Use */
  ["\uF416", "\uF416"], /* Private Use */
  ["\uF417", "\uF417"], /* Private Use */
  ["\uF418", "\uF418"], /* Private Use */
  ["\uF419", "\uF419"], /* Private Use */
  ["\uF41A", "\uF41A"], /* Private Use */
  ["\uF41B", "\uF41B"], /* Private Use */
  ["\uF41C", "\uF41C"], /* Private Use */
  ["\uF41D", "\uF41D"], /* Private Use */
  ["\uF41E", "\uF41E"], /* Private Use */
  ["\uF41F", "\uF41F"], /* Private Use */
  ["\uF420", "\uF420"], /* Private Use */
  ["\uF421", "\uF421"], /* Private Use */
  ["\uF422", "\uF422"], /* Private Use */
  ["\uF423", "\uF423"], /* Private Use */
  ["\uF424", "\uF424"], /* Private Use */
  ["\uF425", "\uF425"], /* Private Use */
  ["\uF426", "\uF426"], /* Private Use */
  ["\uF427", "\uF427"], /* Private Use */
  ["\uF428", "\uF428"], /* Private Use */
  ["\uF429", "\uF429"], /* Private Use */
  ["\uF42A", "\uF42A"], /* Private Use */
  ["\uF42B", "\uF42B"], /* Private Use */
  ["\uF42C", "\uF42C"], /* Private Use */
  ["\uF42D", "\uF42D"], /* Private Use */
  ["\uF42E", "\uF42E"], /* Private Use */
  ["\uF42F", "\uF42F"], /* Private Use */
  ["\uF430", "\uF430"], /* Private Use */
  ["\uF431", "\uF431"], /* Private Use */
  ["\uF432", "\uF432"], /* Private Use */
  ["\uF433", "\uF433"], /* Private Use */
  ["\uF434", "\uF434"], /* Private Use */
  ["\uF435", "\uF435"], /* Private Use */
  ["\uF436", "\uF436"], /* Private Use */
  ["\uF437", "\uF437"], /* Private Use */
  ["\uF438", "\uF438"], /* Private Use */
  ["\uF439", "\uF439"], /* Private Use */
  ["\uF43A", "\uF43A"], /* Private Use */
  ["\uF43B", "\uF43B"], /* Private Use */
  ["\uF43C", "\uF43C"], /* Private Use */
  ["\uF43D", "\uF43D"], /* Private Use */
  ["\uF43E", "\uF43E"], /* Private Use */
  ["\uF43F", "\uF43F"], /* Private Use */
  ["\uF440", "\uF440"], /* Private Use */
  ["\uF441", "\uF441"], /* Private Use */
  ["\uF442", "\uF442"], /* Private Use */
  ["\uF443", "\uF443"], /* Private Use */
  ["\uF444", "\uF444"], /* Private Use */
  ["\uF445", "\uF445"], /* Private Use */
  ["\uF446", "\uF446"], /* Private Use */
  ["\uF447", "\uF447"], /* Private Use */
  ["\uF448", "\uF448"], /* Private Use */
  ["\uF449", "\uF449"], /* Private Use */
  ["\uF44A", "\uF44A"], /* Private Use */
  ["\uF44B", "\uF44B"], /* Private Use */
  ["\uF44C", "\uF44C"], /* Private Use */
  ["\uF44D", "\uF44D"], /* Private Use */
  ["\uF44E", "\uF44E"], /* Private Use */
  ["\uF44F", "\uF44F"], /* Private Use */
  ["\uF450", "\uF450"], /* Private Use */
  ["\uF451", "\uF451"], /* Private Use */
  ["\uF452", "\uF452"], /* Private Use */
  ["\uF453", "\uF453"], /* Private Use */
  ["\uF454", "\uF454"], /* Private Use */
  ["\uF455", "\uF455"], /* Private Use */
  ["\uF456", "\uF456"], /* Private Use */
  ["\uF457", "\uF457"], /* Private Use */
  ["\uF458", "\uF458"], /* Private Use */
  ["\uF459", "\uF459"], /* Private Use */
  ["\uF45A", "\uF45A"], /* Private Use */
  ["\uF45B", "\uF45B"], /* Private Use */
  ["\uF45C", "\uF45C"], /* Private Use */
  ["\uF45D", "\uF45D"], /* Private Use */
  ["\uF45E", "\uF45E"], /* Private Use */
  ["\uF45F", "\uF45F"], /* Private Use */
  ["\uF460", "\uF460"], /* Private Use */
  ["\uF461", "\uF461"], /* Private Use */
  ["\uF462", "\uF462"], /* Private Use */
  ["\uF463", "\uF463"], /* Private Use */
  ["\uF464", "\uF464"], /* Private Use */
  ["\uF465", "\uF465"], /* Private Use */
  ["\uF466", "\uF466"], /* Private Use */
  ["\uF467", "\uF467"], /* Private Use */
  ["\uF468", "\uF468"], /* Private Use */
  ["\uF469", "\uF469"], /* Private Use */
  ["\uF46A", "\uF46A"], /* Private Use */
  ["\uF46B", "\uF46B"], /* Private Use */
  ["\uF46C", "\uF46C"], /* Private Use */
  ["\uF46D", "\uF46D"], /* Private Use */
  ["\uF46E", "\uF46E"], /* Private Use */
  ["\uF46F", "\uF46F"], /* Private Use */
  ["\uF470", "\uF470"], /* Private Use */
  ["\uF471", "\uF471"], /* Private Use */
  ["\uF472", "\uF472"], /* Private Use */
  ["\uF473", "\uF473"], /* Private Use */
  ["\uF474", "\uF474"], /* Private Use */
  ["\uF475", "\uF475"], /* Private Use */
  ["\uF476", "\uF476"], /* Private Use */
  ["\uF477", "\uF477"], /* Private Use */
  ["\uF478", "\uF478"], /* Private Use */
  ["\uF479", "\uF479"], /* Private Use */
  ["\uF47A", "\uF47A"], /* Private Use */
  ["\uF47B", "\uF47B"], /* Private Use */
  ["\uF47C", "\uF47C"], /* Private Use */
  ["\uF47D", "\uF47D"], /* Private Use */
  ["\uF47E", "\uF47E"], /* Private Use */
  ["\uF47F", "\uF47F"], /* Private Use */
  ["\uF480", "\uF480"], /* Private Use */
  ["\uF481", "\uF481"], /* Private Use */
  ["\uF482", "\uF482"], /* Private Use */
  ["\uF483", "\uF483"], /* Private Use */
  ["\uF484", "\uF484"], /* Private Use */
  ["\uF485", "\uF485"], /* Private Use */
  ["\uF486", "\uF486"], /* Private Use */
  ["\uF487", "\uF487"], /* Private Use */
  ["\uF488", "\uF488"], /* Private Use */
  ["\uF489", "\uF489"], /* Private Use */
  ["\uF48A", "\uF48A"], /* Private Use */
  ["\uF48B", "\uF48B"], /* Private Use */
  ["\uF48C", "\uF48C"], /* Private Use */
  ["\uF48D", "\uF48D"], /* Private Use */
  ["\uF48E", "\uF48E"], /* Private Use */
  ["\uF48F", "\uF48F"], /* Private Use */
  ["\uF490", "\uF490"], /* Private Use */
  ["\uF491", "\uF491"], /* Private Use */
  ["\uF492", "\uF492"], /* Private Use */
  ["\uF493", "\uF493"], /* Private Use */
  ["\uF494", "\uF494"], /* Private Use */
  ["\uF495", "\uF495"], /* Private Use */
  ["\uF496", "\uF496"], /* Private Use */
  ["\uF497", "\uF497"], /* Private Use */
  ["\uF498", "\uF498"], /* Private Use */
  ["\uF499", "\uF499"], /* Private Use */
  ["\uF49A", "\uF49A"], /* Private Use */
  ["\uF49B", "\uF49B"], /* Private Use */
  ["\uF49C", "\uF49C"], /* Private Use */
  ["\uF49D", "\uF49D"], /* Private Use */
  ["\uF49E", "\uF49E"], /* Private Use */
  ["\uF49F", "\uF49F"], /* Private Use */
  ["\uF4A0", "\uF4A0"], /* Private Use */
  ["\uF4A1", "\uF4A1"], /* Private Use */
  ["\uF4A2", "\uF4A2"], /* Private Use */
  ["\uF4A3", "\uF4A3"], /* Private Use */
  ["\uF4A4", "\uF4A4"], /* Private Use */
  ["\uF4A5", "\uF4A5"], /* Private Use */
  ["\uF4A6", "\uF4A6"], /* Private Use */
  ["\uF4A7", "\uF4A7"], /* Private Use */
  ["\uF4A8", "\uF4A8"], /* Private Use */
  ["\uF4A9", "\uF4A9"], /* Private Use */
  ["\uF4AA", "\uF4AA"], /* Private Use */
  ["\uF4AB", "\uF4AB"], /* Private Use */
  ["\uF4AC", "\uF4AC"], /* Private Use */
  ["\uF4AD", "\uF4AD"], /* Private Use */
  ["\uF4AE", "\uF4AE"], /* Private Use */
  ["\uF4AF", "\uF4AF"], /* Private Use */
  ["\uF4B0", "\uF4B0"], /* Private Use */
  ["\uF4B1", "\uF4B1"], /* Private Use */
  ["\uF4B2", "\uF4B2"], /* Private Use */
  ["\uF4B3", "\uF4B3"], /* Private Use */
  ["\uF4B4", "\uF4B4"], /* Private Use */
  ["\uF4B5", "\uF4B5"], /* Private Use */
  ["\uF4B6", "\uF4B6"], /* Private Use */
  ["\uF4B7", "\uF4B7"], /* Private Use */
  ["\uF4B8", "\uF4B8"], /* Private Use */
  ["\uF4B9", "\uF4B9"], /* Private Use */
  ["\uF4BA", "\uF4BA"], /* Private Use */
  ["\uF4BB", "\uF4BB"], /* Private Use */
  ["\uF4BC", "\uF4BC"], /* Private Use */
  ["\uF4BD", "\uF4BD"], /* Private Use */
  ["\uF4BE", "\uF4BE"], /* Private Use */
  ["\uF4BF", "\uF4BF"], /* Private Use */
  ["\uF4C0", "\uF4C0"], /* Private Use */
  ["\uF4C1", "\uF4C1"], /* Private Use */
  ["\uF4C2", "\uF4C2"], /* Private Use */
  ["\uF4C3", "\uF4C3"], /* Private Use */
  ["\uF4C4", "\uF4C4"], /* Private Use */
  ["\uF4C5", "\uF4C5"], /* Private Use */
  ["\uF4C6", "\uF4C6"], /* Private Use */
  ["\uF4C7", "\uF4C7"], /* Private Use */
  ["\uF4C8", "\uF4C8"], /* Private Use */
  ["\uF4C9", "\uF4C9"], /* Private Use */
  ["\uF4CA", "\uF4CA"], /* Private Use */
  ["\uF4CB", "\uF4CB"], /* Private Use */
  ["\uF4CC", "\uF4CC"], /* Private Use */
  ["\uF4CD", "\uF4CD"], /* Private Use */
  ["\uF4CE", "\uF4CE"], /* Private Use */
  ["\uF4CF", "\uF4CF"], /* Private Use */
  ["\uF4D0", "\uF4D0"], /* Private Use */
  ["\uF4D1", "\uF4D1"], /* Private Use */
  ["\uF4D2", "\uF4D2"], /* Private Use */
  ["\uF4D3", "\uF4D3"], /* Private Use */
  ["\uF4D4", "\uF4D4"], /* Private Use */
  ["\uF4D5", "\uF4D5"], /* Private Use */
  ["\uF4D6", "\uF4D6"], /* Private Use */
  ["\uF4D7", "\uF4D7"], /* Private Use */
  ["\uF4D8", "\uF4D8"], /* Private Use */
  ["\uF4D9", "\uF4D9"], /* Private Use */
  ["\uF4DA", "\uF4DA"], /* Private Use */
  ["\uF4DB", "\uF4DB"], /* Private Use */
  ["\uF4DC", "\uF4DC"], /* Private Use */
  ["\uF4DD", "\uF4DD"], /* Private Use */
  ["\uF4DE", "\uF4DE"], /* Private Use */
  ["\uF4DF", "\uF4DF"], /* Private Use */
  ["\uF4E0", "\uF4E0"], /* Private Use */
  ["\uF4E1", "\uF4E1"], /* Private Use */
  ["\uF4E2", "\uF4E2"], /* Private Use */
  ["\uF4E3", "\uF4E3"], /* Private Use */
  ["\uF4E4", "\uF4E4"], /* Private Use */
  ["\uF4E5", "\uF4E5"], /* Private Use */
  ["\uF4E6", "\uF4E6"], /* Private Use */
  ["\uF4E7", "\uF4E7"], /* Private Use */
  ["\uF4E8", "\uF4E8"], /* Private Use */
  ["\uF4E9", "\uF4E9"], /* Private Use */
  ["\uF4EA", "\uF4EA"], /* Private Use */
  ["\uF4EB", "\uF4EB"], /* Private Use */
  ["\uF4EC", "\uF4EC"], /* Private Use */
  ["\uF4ED", "\uF4ED"], /* Private Use */
  ["\uF4EE", "\uF4EE"], /* Private Use */
  ["\uF4EF", "\uF4EF"], /* Private Use */
  ["\uF4F0", "\uF4F0"], /* Private Use */
  ["\uF4F1", "\uF4F1"], /* Private Use */
  ["\uF4F2", "\uF4F2"], /* Private Use */
  ["\uF4F3", "\uF4F3"], /* Private Use */
  ["\uF4F4", "\uF4F4"], /* Private Use */
  ["\uF4F5", "\uF4F5"], /* Private Use */
  ["\uF4F6", "\uF4F6"], /* Private Use */
  ["\uF4F7", "\uF4F7"], /* Private Use */
  ["\uF4F8", "\uF4F8"], /* Private Use */
  ["\uF4F9", "\uF4F9"], /* Private Use */
  ["\uF4FA", "\uF4FA"], /* Private Use */
  ["\uF4FB", "\uF4FB"], /* Private Use */
  ["\uF4FC", "\uF4FC"], /* Private Use */
  ["\uF4FD", "\uF4FD"], /* Private Use */
  ["\uF4FE", "\uF4FE"], /* Private Use */
  ["\uF4FF", "\uF4FF"], /* Private Use */
  ["\uF500", "\uF500"], /* Private Use */
  ["\uF501", "\uF501"], /* Private Use */
  ["\uF502", "\uF502"], /* Private Use */
  ["\uF503", "\uF503"], /* Private Use */
  ["\uF504", "\uF504"], /* Private Use */
  ["\uF505", "\uF505"], /* Private Use */
  ["\uF506", "\uF506"], /* Private Use */
  ["\uF507", "\uF507"], /* Private Use */
  ["\uF508", "\uF508"], /* Private Use */
  ["\uF509", "\uF509"], /* Private Use */
  ["\uF50A", "\uF50A"], /* Private Use */
  ["\uF50B", "\uF50B"], /* Private Use */
  ["\uF50C", "\uF50C"], /* Private Use */
  ["\uF50D", "\uF50D"], /* Private Use */
  ["\uF50E", "\uF50E"], /* Private Use */
  ["\uF50F", "\uF50F"], /* Private Use */
  ["\uF510", "\uF510"], /* Private Use */
  ["\uF511", "\uF511"], /* Private Use */
  ["\uF512", "\uF512"], /* Private Use */
  ["\uF513", "\uF513"], /* Private Use */
  ["\uF514", "\uF514"], /* Private Use */
  ["\uF515", "\uF515"], /* Private Use */
  ["\uF516", "\uF516"], /* Private Use */
  ["\uF517", "\uF517"], /* Private Use */
  ["\uF518", "\uF518"], /* Private Use */
  ["\uF519", "\uF519"], /* Private Use */
  ["\uF51A", "\uF51A"], /* Private Use */
  ["\uF51B", "\uF51B"], /* Private Use */
  ["\uF51C", "\uF51C"], /* Private Use */
  ["\uF51D", "\uF51D"], /* Private Use */
  ["\uF51E", "\uF51E"], /* Private Use */
  ["\uF51F", "\uF51F"], /* Private Use */
  ["\uF520", "\uF520"], /* Private Use */
  ["\uF521", "\uF521"], /* Private Use */
  ["\uF522", "\uF522"], /* Private Use */
  ["\uF523", "\uF523"], /* Private Use */
  ["\uF524", "\uF524"], /* Private Use */
  ["\uF525", "\uF525"], /* Private Use */
  ["\uF526", "\uF526"], /* Private Use */
  ["\uF527", "\uF527"], /* Private Use */
  ["\uF528", "\uF528"], /* Private Use */
  ["\uF529", "\uF529"], /* Private Use */
  ["\uF52A", "\uF52A"], /* Private Use */
  ["\uF52B", "\uF52B"], /* Private Use */
  ["\uF52C", "\uF52C"], /* Private Use */
  ["\uF52D", "\uF52D"], /* Private Use */
  ["\uF52E", "\uF52E"], /* Private Use */
  ["\uF52F", "\uF52F"], /* Private Use */
  ["\uF530", "\uF530"], /* Private Use */
  ["\uF531", "\uF531"], /* Private Use */
  ["\uF532", "\uF532"], /* Private Use */
  ["\uF533", "\uF533"], /* Private Use */
  ["\uF534", "\uF534"], /* Private Use */
  ["\uF535", "\uF535"], /* Private Use */
  ["\uF536", "\uF536"], /* Private Use */
  ["\uF537", "\uF537"], /* Private Use */
  ["\uF538", "\uF538"], /* Private Use */
  ["\uF539", "\uF539"], /* Private Use */
  ["\uF53A", "\uF53A"], /* Private Use */
  ["\uF53B", "\uF53B"], /* Private Use */
  ["\uF53C", "\uF53C"], /* Private Use */
  ["\uF53D", "\uF53D"], /* Private Use */
  ["\uF53E", "\uF53E"], /* Private Use */
  ["\uF53F", "\uF53F"], /* Private Use */
  ["\uF540", "\uF540"], /* Private Use */
  ["\uF541", "\uF541"], /* Private Use */
  ["\uF542", "\uF542"], /* Private Use */
  ["\uF543", "\uF543"], /* Private Use */
  ["\uF544", "\uF544"], /* Private Use */
  ["\uF545", "\uF545"], /* Private Use */
  ["\uF546", "\uF546"], /* Private Use */
  ["\uF547", "\uF547"], /* Private Use */
  ["\uF548", "\uF548"], /* Private Use */
  ["\uF549", "\uF549"], /* Private Use */
  ["\uF54A", "\uF54A"], /* Private Use */
  ["\uF54B", "\uF54B"], /* Private Use */
  ["\uF54C", "\uF54C"], /* Private Use */
  ["\uF54D", "\uF54D"], /* Private Use */
  ["\uF54E", "\uF54E"], /* Private Use */
  ["\uF54F", "\uF54F"], /* Private Use */
  ["\uF550", "\uF550"], /* Private Use */
  ["\uF551", "\uF551"], /* Private Use */
  ["\uF552", "\uF552"], /* Private Use */
  ["\uF553", "\uF553"], /* Private Use */
  ["\uF554", "\uF554"], /* Private Use */
  ["\uF555", "\uF555"], /* Private Use */
  ["\uF556", "\uF556"], /* Private Use */
  ["\uF557", "\uF557"], /* Private Use */
  ["\uF558", "\uF558"], /* Private Use */
  ["\uF559", "\uF559"], /* Private Use */
  ["\uF55A", "\uF55A"], /* Private Use */
  ["\uF55B", "\uF55B"], /* Private Use */
  ["\uF55C", "\uF55C"], /* Private Use */
  ["\uF55D", "\uF55D"], /* Private Use */
  ["\uF55E", "\uF55E"], /* Private Use */
  ["\uF55F", "\uF55F"], /* Private Use */
  ["\uF560", "\uF560"], /* Private Use */
  ["\uF561", "\uF561"], /* Private Use */
  ["\uF562", "\uF562"], /* Private Use */
  ["\uF563", "\uF563"], /* Private Use */
  ["\uF564", "\uF564"], /* Private Use */
  ["\uF565", "\uF565"], /* Private Use */
  ["\uF566", "\uF566"], /* Private Use */
  ["\uF567", "\uF567"], /* Private Use */
  ["\uF568", "\uF568"], /* Private Use */
  ["\uF569", "\uF569"], /* Private Use */
  ["\uF56A", "\uF56A"], /* Private Use */
  ["\uF56B", "\uF56B"], /* Private Use */
  ["\uF56C", "\uF56C"], /* Private Use */
  ["\uF56D", "\uF56D"], /* Private Use */
  ["\uF56E", "\uF56E"], /* Private Use */
  ["\uF56F", "\uF56F"], /* Private Use */
  ["\uF570", "\uF570"], /* Private Use */
  ["\uF571", "\uF571"], /* Private Use */
  ["\uF572", "\uF572"], /* Private Use */
  ["\uF573", "\uF573"], /* Private Use */
  ["\uF574", "\uF574"], /* Private Use */
  ["\uF575", "\uF575"], /* Private Use */
  ["\uF576", "\uF576"], /* Private Use */
  ["\uF577", "\uF577"], /* Private Use */
  ["\uF578", "\uF578"], /* Private Use */
  ["\uF579", "\uF579"], /* Private Use */
  ["\uF57A", "\uF57A"], /* Private Use */
  ["\uF57B", "\uF57B"], /* Private Use */
  ["\uF57C", "\uF57C"], /* Private Use */
  ["\uF57D", "\uF57D"], /* Private Use */
  ["\uF57E", "\uF57E"], /* Private Use */
  ["\uF57F", "\uF57F"], /* Private Use */
  ["\uF580", "\uF580"], /* Private Use */
  ["\uF581", "\uF581"], /* Private Use */
  ["\uF582", "\uF582"], /* Private Use */
  ["\uF583", "\uF583"], /* Private Use */
  ["\uF584", "\uF584"], /* Private Use */
  ["\uF585", "\uF585"], /* Private Use */
  ["\uF586", "\uF586"], /* Private Use */
  ["\uF587", "\uF587"], /* Private Use */
  ["\uF588", "\uF588"], /* Private Use */
  ["\uF589", "\uF589"], /* Private Use */
  ["\uF58A", "\uF58A"], /* Private Use */
  ["\uF58B", "\uF58B"], /* Private Use */
  ["\uF58C", "\uF58C"], /* Private Use */
  ["\uF58D", "\uF58D"], /* Private Use */
  ["\uF58E", "\uF58E"], /* Private Use */
  ["\uF58F", "\uF58F"], /* Private Use */
  ["\uF590", "\uF590"], /* Private Use */
  ["\uF591", "\uF591"], /* Private Use */
  ["\uF592", "\uF592"], /* Private Use */
  ["\uF593", "\uF593"], /* Private Use */
  ["\uF594", "\uF594"], /* Private Use */
  ["\uF595", "\uF595"], /* Private Use */
  ["\uF596", "\uF596"], /* Private Use */
  ["\uF597", "\uF597"], /* Private Use */
  ["\uF598", "\uF598"], /* Private Use */
  ["\uF599", "\uF599"], /* Private Use */
  ["\uF59A", "\uF59A"], /* Private Use */
  ["\uF59B", "\uF59B"], /* Private Use */
  ["\uF59C", "\uF59C"], /* Private Use */
  ["\uF59D", "\uF59D"], /* Private Use */
  ["\uF59E", "\uF59E"], /* Private Use */
  ["\uF59F", "\uF59F"], /* Private Use */
  ["\uF5A0", "\uF5A0"], /* Private Use */
  ["\uF5A1", "\uF5A1"], /* Private Use */
  ["\uF5A2", "\uF5A2"], /* Private Use */
  ["\uF5A3", "\uF5A3"], /* Private Use */
  ["\uF5A4", "\uF5A4"], /* Private Use */
  ["\uF5A5", "\uF5A5"], /* Private Use */
  ["\uF5A6", "\uF5A6"], /* Private Use */
  ["\uF5A7", "\uF5A7"], /* Private Use */
  ["\uF5A8", "\uF5A8"], /* Private Use */
  ["\uF5A9", "\uF5A9"], /* Private Use */
  ["\uF5AA", "\uF5AA"], /* Private Use */
  ["\uF5AB", "\uF5AB"], /* Private Use */
  ["\uF5AC", "\uF5AC"], /* Private Use */
  ["\uF5AD", "\uF5AD"], /* Private Use */
  ["\uF5AE", "\uF5AE"], /* Private Use */
  ["\uF5AF", "\uF5AF"], /* Private Use */
  ["\uF5B0", "\uF5B0"], /* Private Use */
  ["\uF5B1", "\uF5B1"], /* Private Use */
  ["\uF5B2", "\uF5B2"], /* Private Use */
  ["\uF5B3", "\uF5B3"], /* Private Use */
  ["\uF5B4", "\uF5B4"], /* Private Use */
  ["\uF5B5", "\uF5B5"], /* Private Use */
  ["\uF5B6", "\uF5B6"], /* Private Use */
  ["\uF5B7", "\uF5B7"], /* Private Use */
  ["\uF5B8", "\uF5B8"], /* Private Use */
  ["\uF5B9", "\uF5B9"], /* Private Use */
  ["\uF5BA", "\uF5BA"], /* Private Use */
  ["\uF5BB", "\uF5BB"], /* Private Use */
  ["\uF5BC", "\uF5BC"], /* Private Use */
  ["\uF5BD", "\uF5BD"], /* Private Use */
  ["\uF5BE", "\uF5BE"], /* Private Use */
  ["\uF5BF", "\uF5BF"], /* Private Use */
  ["\uF5C0", "\uF5C0"], /* Private Use */
  ["\uF5C1", "\uF5C1"], /* Private Use */
  ["\uF5C2", "\uF5C2"], /* Private Use */
  ["\uF5C3", "\uF5C3"], /* Private Use */
  ["\uF5C4", "\uF5C4"], /* Private Use */
  ["\uF5C5", "\uF5C5"], /* Private Use */
  ["\uF5C6", "\uF5C6"], /* Private Use */
  ["\uF5C7", "\uF5C7"], /* Private Use */
  ["\uF5C8", "\uF5C8"], /* Private Use */
  ["\uF5C9", "\uF5C9"], /* Private Use */
  ["\uF5CA", "\uF5CA"], /* Private Use */
  ["\uF5CB", "\uF5CB"], /* Private Use */
  ["\uF5CC", "\uF5CC"], /* Private Use */
  ["\uF5CD", "\uF5CD"], /* Private Use */
  ["\uF5CE", "\uF5CE"], /* Private Use */
  ["\uF5CF", "\uF5CF"], /* Private Use */
  ["\uF5D0", "\uF5D0"], /* Private Use */
  ["\uF5D1", "\uF5D1"], /* Private Use */
  ["\uF5D2", "\uF5D2"], /* Private Use */
  ["\uF5D3", "\uF5D3"], /* Private Use */
  ["\uF5D4", "\uF5D4"], /* Private Use */
  ["\uF5D5", "\uF5D5"], /* Private Use */
  ["\uF5D6", "\uF5D6"], /* Private Use */
  ["\uF5D7", "\uF5D7"], /* Private Use */
  ["\uF5D8", "\uF5D8"], /* Private Use */
  ["\uF5D9", "\uF5D9"], /* Private Use */
  ["\uF5DA", "\uF5DA"], /* Private Use */
  ["\uF5DB", "\uF5DB"], /* Private Use */
  ["\uF5DC", "\uF5DC"], /* Private Use */
  ["\uF5DD", "\uF5DD"], /* Private Use */
  ["\uF5DE", "\uF5DE"], /* Private Use */
  ["\uF5DF", "\uF5DF"], /* Private Use */
  ["\uF5E0", "\uF5E0"], /* Private Use */
  ["\uF5E1", "\uF5E1"], /* Private Use */
  ["\uF5E2", "\uF5E2"], /* Private Use */
  ["\uF5E3", "\uF5E3"], /* Private Use */
  ["\uF5E4", "\uF5E4"], /* Private Use */
  ["\uF5E5", "\uF5E5"], /* Private Use */
  ["\uF5E6", "\uF5E6"], /* Private Use */
  ["\uF5E7", "\uF5E7"], /* Private Use */
  ["\uF5E8", "\uF5E8"], /* Private Use */
  ["\uF5E9", "\uF5E9"], /* Private Use */
  ["\uF5EA", "\uF5EA"], /* Private Use */
  ["\uF5EB", "\uF5EB"], /* Private Use */
  ["\uF5EC", "\uF5EC"], /* Private Use */
  ["\uF5ED", "\uF5ED"], /* Private Use */
  ["\uF5EE", "\uF5EE"], /* Private Use */
  ["\uF5EF", "\uF5EF"], /* Private Use */
  ["\uF5F0", "\uF5F0"], /* Private Use */
  ["\uF5F1", "\uF5F1"], /* Private Use */
  ["\uF5F2", "\uF5F2"], /* Private Use */
  ["\uF5F3", "\uF5F3"], /* Private Use */
  ["\uF5F4", "\uF5F4"], /* Private Use */
  ["\uF5F5", "\uF5F5"], /* Private Use */
  ["\uF5F6", "\uF5F6"], /* Private Use */
  ["\uF5F7", "\uF5F7"], /* Private Use */
  ["\uF5F8", "\uF5F8"], /* Private Use */
  ["\uF5F9", "\uF5F9"], /* Private Use */
  ["\uF5FA", "\uF5FA"], /* Private Use */
  ["\uF5FB", "\uF5FB"], /* Private Use */
  ["\uF5FC", "\uF5FC"], /* Private Use */
  ["\uF5FD", "\uF5FD"], /* Private Use */
  ["\uF5FE", "\uF5FE"], /* Private Use */
  ["\uF5FF", "\uF5FF"], /* Private Use */
  ["\uF600", "\uF600"], /* Private Use */
  ["\uF601", "\uF601"], /* Private Use */
  ["\uF602", "\uF602"], /* Private Use */
  ["\uF603", "\uF603"], /* Private Use */
  ["\uF604", "\uF604"], /* Private Use */
  ["\uF605", "\uF605"], /* Private Use */
  ["\uF606", "\uF606"], /* Private Use */
  ["\uF607", "\uF607"], /* Private Use */
  ["\uF608", "\uF608"], /* Private Use */
  ["\uF609", "\uF609"], /* Private Use */
  ["\uF60A", "\uF60A"], /* Private Use */
  ["\uF60B", "\uF60B"], /* Private Use */
  ["\uF60C", "\uF60C"], /* Private Use */
  ["\uF60D", "\uF60D"], /* Private Use */
  ["\uF60E", "\uF60E"], /* Private Use */
  ["\uF60F", "\uF60F"], /* Private Use */
  ["\uF610", "\uF610"], /* Private Use */
  ["\uF611", "\uF611"], /* Private Use */
  ["\uF612", "\uF612"], /* Private Use */
  ["\uF613", "\uF613"], /* Private Use */
  ["\uF614", "\uF614"], /* Private Use */
  ["\uF615", "\uF615"], /* Private Use */
  ["\uF616", "\uF616"], /* Private Use */
  ["\uF617", "\uF617"], /* Private Use */
  ["\uF618", "\uF618"], /* Private Use */
  ["\uF619", "\uF619"], /* Private Use */
  ["\uF61A", "\uF61A"], /* Private Use */
  ["\uF61B", "\uF61B"], /* Private Use */
  ["\uF61C", "\uF61C"], /* Private Use */
  ["\uF61D", "\uF61D"], /* Private Use */
  ["\uF61E", "\uF61E"], /* Private Use */
  ["\uF61F", "\uF61F"], /* Private Use */
  ["\uF620", "\uF620"], /* Private Use */
  ["\uF621", "\uF621"], /* Private Use */
  ["\uF622", "\uF622"], /* Private Use */
  ["\uF623", "\uF623"], /* Private Use */
  ["\uF624", "\uF624"], /* Private Use */
  ["\uF625", "\uF625"], /* Private Use */
  ["\uF626", "\uF626"], /* Private Use */
  ["\uF627", "\uF627"], /* Private Use */
  ["\uF628", "\uF628"], /* Private Use */
  ["\uF629", "\uF629"], /* Private Use */
  ["\uF62A", "\uF62A"], /* Private Use */
  ["\uF62B", "\uF62B"], /* Private Use */
  ["\uF62C", "\uF62C"], /* Private Use */
  ["\uF62D", "\uF62D"], /* Private Use */
  ["\uF62E", "\uF62E"], /* Private Use */
  ["\uF62F", "\uF62F"], /* Private Use */
  ["\uF630", "\uF630"], /* Private Use */
  ["\uF631", "\uF631"], /* Private Use */
  ["\uF632", "\uF632"], /* Private Use */
  ["\uF633", "\uF633"], /* Private Use */
  ["\uF634", "\uF634"], /* Private Use */
  ["\uF635", "\uF635"], /* Private Use */
  ["\uF636", "\uF636"], /* Private Use */
  ["\uF637", "\uF637"], /* Private Use */
  ["\uF638", "\uF638"], /* Private Use */
  ["\uF639", "\uF639"], /* Private Use */
  ["\uF63A", "\uF63A"], /* Private Use */
  ["\uF63B", "\uF63B"], /* Private Use */
  ["\uF63C", "\uF63C"], /* Private Use */
  ["\uF63D", "\uF63D"], /* Private Use */
  ["\uF63E", "\uF63E"], /* Private Use */
  ["\uF63F", "\uF63F"], /* Private Use */
  ["\uF640", "\uF640"], /* Private Use */
  ["\uF641", "\uF641"], /* Private Use */
  ["\uF642", "\uF642"], /* Private Use */
  ["\uF643", "\uF643"], /* Private Use */
  ["\uF644", "\uF644"], /* Private Use */
  ["\uF645", "\uF645"], /* Private Use */
  ["\uF646", "\uF646"], /* Private Use */
  ["\uF647", "\uF647"], /* Private Use */
  ["\uF648", "\uF648"], /* Private Use */
  ["\uF649", "\uF649"], /* Private Use */
  ["\uF64A", "\uF64A"], /* Private Use */
  ["\uF64B", "\uF64B"], /* Private Use */
  ["\uF64C", "\uF64C"], /* Private Use */
  ["\uF64D", "\uF64D"], /* Private Use */
  ["\uF64E", "\uF64E"], /* Private Use */
  ["\uF64F", "\uF64F"], /* Private Use */
  ["\uF650", "\uF650"], /* Private Use */
  ["\uF651", "\uF651"], /* Private Use */
  ["\uF652", "\uF652"], /* Private Use */
  ["\uF653", "\uF653"], /* Private Use */
  ["\uF654", "\uF654"], /* Private Use */
  ["\uF655", "\uF655"], /* Private Use */
  ["\uF656", "\uF656"], /* Private Use */
  ["\uF657", "\uF657"], /* Private Use */
  ["\uF658", "\uF658"], /* Private Use */
  ["\uF659", "\uF659"], /* Private Use */
  ["\uF65A", "\uF65A"], /* Private Use */
  ["\uF65B", "\uF65B"], /* Private Use */
  ["\uF65C", "\uF65C"], /* Private Use */
  ["\uF65D", "\uF65D"], /* Private Use */
  ["\uF65E", "\uF65E"], /* Private Use */
  ["\uF65F", "\uF65F"], /* Private Use */
  ["\uF660", "\uF660"], /* Private Use */
  ["\uF661", "\uF661"], /* Private Use */
  ["\uF662", "\uF662"], /* Private Use */
  ["\uF663", "\uF663"], /* Private Use */
  ["\uF664", "\uF664"], /* Private Use */
  ["\uF665", "\uF665"], /* Private Use */
  ["\uF666", "\uF666"], /* Private Use */
  ["\uF667", "\uF667"], /* Private Use */
  ["\uF668", "\uF668"], /* Private Use */
  ["\uF669", "\uF669"], /* Private Use */
  ["\uF66A", "\uF66A"], /* Private Use */
  ["\uF66B", "\uF66B"], /* Private Use */
  ["\uF66C", "\uF66C"], /* Private Use */
  ["\uF66D", "\uF66D"], /* Private Use */
  ["\uF66E", "\uF66E"], /* Private Use */
  ["\uF66F", "\uF66F"], /* Private Use */
  ["\uF670", "\uF670"], /* Private Use */
  ["\uF671", "\uF671"], /* Private Use */
  ["\uF672", "\uF672"], /* Private Use */
  ["\uF673", "\uF673"], /* Private Use */
  ["\uF674", "\uF674"], /* Private Use */
  ["\uF675", "\uF675"], /* Private Use */
  ["\uF676", "\uF676"], /* Private Use */
  ["\uF677", "\uF677"], /* Private Use */
  ["\uF678", "\uF678"], /* Private Use */
  ["\uF679", "\uF679"], /* Private Use */
  ["\uF67A", "\uF67A"], /* Private Use */
  ["\uF67B", "\uF67B"], /* Private Use */
  ["\uF67C", "\uF67C"], /* Private Use */
  ["\uF67D", "\uF67D"], /* Private Use */
  ["\uF67E", "\uF67E"], /* Private Use */
  ["\uF67F", "\uF67F"], /* Private Use */
  ["\uF680", "\uF680"], /* Private Use */
  ["\uF681", "\uF681"], /* Private Use */
  ["\uF682", "\uF682"], /* Private Use */
  ["\uF683", "\uF683"], /* Private Use */
  ["\uF684", "\uF684"], /* Private Use */
  ["\uF685", "\uF685"], /* Private Use */
  ["\uF686", "\uF686"], /* Private Use */
  ["\uF687", "\uF687"], /* Private Use */
  ["\uF688", "\uF688"], /* Private Use */
  ["\uF689", "\uF689"], /* Private Use */
  ["\uF68A", "\uF68A"], /* Private Use */
  ["\uF68B", "\uF68B"], /* Private Use */
  ["\uF68C", "\uF68C"], /* Private Use */
  ["\uF68D", "\uF68D"], /* Private Use */
  ["\uF68E", "\uF68E"], /* Private Use */
  ["\uF68F", "\uF68F"], /* Private Use */
  ["\uF690", "\uF690"], /* Private Use */
  ["\uF691", "\uF691"], /* Private Use */
  ["\uF692", "\uF692"], /* Private Use */
  ["\uF693", "\uF693"], /* Private Use */
  ["\uF694", "\uF694"], /* Private Use */
  ["\uF695", "\uF695"], /* Private Use */
  ["\uF696", "\uF696"], /* Private Use */
  ["\uF697", "\uF697"], /* Private Use */
  ["\uF698", "\uF698"], /* Private Use */
  ["\uF699", "\uF699"], /* Private Use */
  ["\uF69A", "\uF69A"], /* Private Use */
  ["\uF69B", "\uF69B"], /* Private Use */
  ["\uF69C", "\uF69C"], /* Private Use */
  ["\uF69D", "\uF69D"], /* Private Use */
  ["\uF69E", "\uF69E"], /* Private Use */
  ["\uF69F", "\uF69F"], /* Private Use */
  ["\uF6A0", "\uF6A0"], /* Private Use */
  ["\uF6A1", "\uF6A1"], /* Private Use */
  ["\uF6A2", "\uF6A2"], /* Private Use */
  ["\uF6A3", "\uF6A3"], /* Private Use */
  ["\uF6A4", "\uF6A4"], /* Private Use */
  ["\uF6A5", "\uF6A5"], /* Private Use */
  ["\uF6A6", "\uF6A6"], /* Private Use */
  ["\uF6A7", "\uF6A7"], /* Private Use */
  ["\uF6A8", "\uF6A8"], /* Private Use */
  ["\uF6A9", "\uF6A9"], /* Private Use */
  ["\uF6AA", "\uF6AA"], /* Private Use */
  ["\uF6AB", "\uF6AB"], /* Private Use */
  ["\uF6AC", "\uF6AC"], /* Private Use */
  ["\uF6AD", "\uF6AD"], /* Private Use */
  ["\uF6AE", "\uF6AE"], /* Private Use */
  ["\uF6AF", "\uF6AF"], /* Private Use */
  ["\uF6B0", "\uF6B0"], /* Private Use */
  ["\uF6B1", "\uF6B1"], /* Private Use */
  ["\uF6B2", "\uF6B2"], /* Private Use */
  ["\uF6B3", "\uF6B3"], /* Private Use */
  ["\uF6B4", "\uF6B4"], /* Private Use */
  ["\uF6B5", "\uF6B5"], /* Private Use */
  ["\uF6B6", "\uF6B6"], /* Private Use */
  ["\uF6B7", "\uF6B7"], /* Private Use */
  ["\uF6B8", "\uF6B8"], /* Private Use */
  ["\uF6B9", "\uF6B9"], /* Private Use */
  ["\uF6BA", "\uF6BA"], /* Private Use */
  ["\uF6BB", "\uF6BB"], /* Private Use */
  ["\uF6BC", "\uF6BC"], /* Private Use */
  ["\uF6BD", "\uF6BD"], /* Private Use */
  ["\uF6BE", "\uF6BE"], /* Private Use */
  ["\uF6BF", "\uF6BF"], /* Private Use */
  ["\uF6C0", "\uF6C0"], /* Private Use */
  ["\uF6C1", "\uF6C1"], /* Private Use */
  ["\uF6C2", "\uF6C2"], /* Private Use */
  ["\uF6C3", "\uF6C3"], /* Private Use */
  ["\uF6C4", "\uF6C4"], /* Private Use */
  ["\uF6C5", "\uF6C5"], /* Private Use */
  ["\uF6C6", "\uF6C6"], /* Private Use */
  ["\uF6C7", "\uF6C7"], /* Private Use */
  ["\uF6C8", "\uF6C8"], /* Private Use */
  ["\uF6C9", "\uF6C9"], /* Private Use */
  ["\uF6CA", "\uF6CA"], /* Private Use */
  ["\uF6CB", "\uF6CB"], /* Private Use */
  ["\uF6CC", "\uF6CC"], /* Private Use */
  ["\uF6CD", "\uF6CD"], /* Private Use */
  ["\uF6CE", "\uF6CE"], /* Private Use */
  ["\uF6CF", "\uF6CF"], /* Private Use */
  ["\uF6D0", "\uF6D0"], /* Private Use */
  ["\uF6D1", "\uF6D1"], /* Private Use */
  ["\uF6D2", "\uF6D2"], /* Private Use */
  ["\uF6D3", "\uF6D3"], /* Private Use */
  ["\uF6D4", "\uF6D4"], /* Private Use */
  ["\uF6D5", "\uF6D5"], /* Private Use */
  ["\uF6D6", "\uF6D6"], /* Private Use */
  ["\uF6D7", "\uF6D7"], /* Private Use */
  ["\uF6D8", "\uF6D8"], /* Private Use */
  ["\uF6D9", "\uF6D9"], /* Private Use */
  ["\uF6DA", "\uF6DA"], /* Private Use */
  ["\uF6DB", "\uF6DB"], /* Private Use */
  ["\uF6DC", "\uF6DC"], /* Private Use */
  ["\uF6DD", "\uF6DD"], /* Private Use */
  ["\uF6DE", "\uF6DE"], /* Private Use */
  ["\uF6DF", "\uF6DF"], /* Private Use */
  ["\uF6E0", "\uF6E0"], /* Private Use */
  ["\uF6E1", "\uF6E1"], /* Private Use */
  ["\uF6E2", "\uF6E2"], /* Private Use */
  ["\uF6E3", "\uF6E3"], /* Private Use */
  ["\uF6E4", "\uF6E4"], /* Private Use */
  ["\uF6E5", "\uF6E5"], /* Private Use */
  ["\uF6E6", "\uF6E6"], /* Private Use */
  ["\uF6E7", "\uF6E7"], /* Private Use */
  ["\uF6E8", "\uF6E8"], /* Private Use */
  ["\uF6E9", "\uF6E9"], /* Private Use */
  ["\uF6EA", "\uF6EA"], /* Private Use */
  ["\uF6EB", "\uF6EB"], /* Private Use */
  ["\uF6EC", "\uF6EC"], /* Private Use */
  ["\uF6ED", "\uF6ED"], /* Private Use */
  ["\uF6EE", "\uF6EE"], /* Private Use */
  ["\uF6EF", "\uF6EF"], /* Private Use */
  ["\uF6F0", "\uF6F0"], /* Private Use */
  ["\uF6F1", "\uF6F1"], /* Private Use */
  ["\uF6F2", "\uF6F2"], /* Private Use */
  ["\uF6F3", "\uF6F3"], /* Private Use */
  ["\uF6F4", "\uF6F4"], /* Private Use */
  ["\uF6F5", "\uF6F5"], /* Private Use */
  ["\uF6F6", "\uF6F6"], /* Private Use */
  ["\uF6F7", "\uF6F7"], /* Private Use */
  ["\uF6F8", "\uF6F8"], /* Private Use */
  ["\uF6F9", "\uF6F9"], /* Private Use */
  ["\uF6FA", "\uF6FA"], /* Private Use */
  ["\uF6FB", "\uF6FB"], /* Private Use */
  ["\uF6FC", "\uF6FC"], /* Private Use */
  ["\uF6FD", "\uF6FD"], /* Private Use */
  ["\uF6FE", "\uF6FE"], /* Private Use */
  ["\uF6FF", "\uF6FF"], /* Private Use */
  ["\uF700", "\uF700"], /* Private Use */
  ["\uF701", "\uF701"], /* Private Use */
  ["\uF702", "\uF702"], /* Private Use */
  ["\uF703", "\uF703"], /* Private Use */
  ["\uF704", "\uF704"], /* Private Use */
  ["\uF705", "\uF705"], /* Private Use */
  ["\uF706", "\uF706"], /* Private Use */
  ["\uF707", "\uF707"], /* Private Use */
  ["\uF708", "\uF708"], /* Private Use */
  ["\uF709", "\uF709"], /* Private Use */
  ["\uF70A", "\uF70A"], /* Private Use */
  ["\uF70B", "\uF70B"], /* Private Use */
  ["\uF70C", "\uF70C"], /* Private Use */
  ["\uF70D", "\uF70D"], /* Private Use */
  ["\uF70E", "\uF70E"], /* Private Use */
  ["\uF70F", "\uF70F"], /* Private Use */
  ["\uF710", "\uF710"], /* Private Use */
  ["\uF711", "\uF711"], /* Private Use */
  ["\uF712", "\uF712"], /* Private Use */
  ["\uF713", "\uF713"], /* Private Use */
  ["\uF714", "\uF714"], /* Private Use */
  ["\uF715", "\uF715"], /* Private Use */
  ["\uF716", "\uF716"], /* Private Use */
  ["\uF717", "\uF717"], /* Private Use */
  ["\uF718", "\uF718"], /* Private Use */
  ["\uF719", "\uF719"], /* Private Use */
  ["\uF71A", "\uF71A"], /* Private Use */
  ["\uF71B", "\uF71B"], /* Private Use */
  ["\uF71C", "\uF71C"], /* Private Use */
  ["\uF71D", "\uF71D"], /* Private Use */
  ["\uF71E", "\uF71E"], /* Private Use */
  ["\uF71F", "\uF71F"], /* Private Use */
  ["\uF720", "\uF720"], /* Private Use */
  ["\uF721", "\uF721"], /* Private Use */
  ["\uF722", "\uF722"], /* Private Use */
  ["\uF723", "\uF723"], /* Private Use */
  ["\uF724", "\uF724"], /* Private Use */
  ["\uF725", "\uF725"], /* Private Use */
  ["\uF726", "\uF726"], /* Private Use */
  ["\uF727", "\uF727"], /* Private Use */
  ["\uF728", "\uF728"], /* Private Use */
  ["\uF729", "\uF729"], /* Private Use */
  ["\uF72A", "\uF72A"], /* Private Use */
  ["\uF72B", "\uF72B"], /* Private Use */
  ["\uF72C", "\uF72C"], /* Private Use */
  ["\uF72D", "\uF72D"], /* Private Use */
  ["\uF72E", "\uF72E"], /* Private Use */
  ["\uF72F", "\uF72F"], /* Private Use */
  ["\uF730", "\uF730"], /* Private Use */
  ["\uF731", "\uF731"], /* Private Use */
  ["\uF732", "\uF732"], /* Private Use */
  ["\uF733", "\uF733"], /* Private Use */
  ["\uF734", "\uF734"], /* Private Use */
  ["\uF735", "\uF735"], /* Private Use */
  ["\uF736", "\uF736"], /* Private Use */
  ["\uF737", "\uF737"], /* Private Use */
  ["\uF738", "\uF738"], /* Private Use */
  ["\uF739", "\uF739"], /* Private Use */
  ["\uF73A", "\uF73A"], /* Private Use */
  ["\uF73B", "\uF73B"], /* Private Use */
  ["\uF73C", "\uF73C"], /* Private Use */
  ["\uF73D", "\uF73D"], /* Private Use */
  ["\uF73E", "\uF73E"], /* Private Use */
  ["\uF73F", "\uF73F"], /* Private Use */
  ["\uF740", "\uF740"], /* Private Use */
  ["\uF741", "\uF741"], /* Private Use */
  ["\uF742", "\uF742"], /* Private Use */
  ["\uF743", "\uF743"], /* Private Use */
  ["\uF744", "\uF744"], /* Private Use */
  ["\uF745", "\uF745"], /* Private Use */
  ["\uF746", "\uF746"], /* Private Use */
  ["\uF747", "\uF747"], /* Private Use */
  ["\uF748", "\uF748"], /* Private Use */
  ["\uF749", "\uF749"], /* Private Use */
  ["\uF74A", "\uF74A"], /* Private Use */
  ["\uF74B", "\uF74B"], /* Private Use */
  ["\uF74C", "\uF74C"], /* Private Use */
  ["\uF74D", "\uF74D"], /* Private Use */
  ["\uF74E", "\uF74E"], /* Private Use */
  ["\uF74F", "\uF74F"], /* Private Use */
  ["\uF750", "\uF750"], /* Private Use */
  ["\uF751", "\uF751"], /* Private Use */
  ["\uF752", "\uF752"], /* Private Use */
  ["\uF753", "\uF753"], /* Private Use */
  ["\uF754", "\uF754"], /* Private Use */
  ["\uF755", "\uF755"], /* Private Use */
  ["\uF756", "\uF756"], /* Private Use */
  ["\uF757", "\uF757"], /* Private Use */
  ["\uF758", "\uF758"], /* Private Use */
  ["\uF759", "\uF759"], /* Private Use */
  ["\uF75A", "\uF75A"], /* Private Use */
  ["\uF75B", "\uF75B"], /* Private Use */
  ["\uF75C", "\uF75C"], /* Private Use */
  ["\uF75D", "\uF75D"], /* Private Use */
  ["\uF75E", "\uF75E"], /* Private Use */
  ["\uF75F", "\uF75F"], /* Private Use */
  ["\uF760", "\uF760"], /* Private Use */
  ["\uF761", "\uF761"], /* Private Use */
  ["\uF762", "\uF762"], /* Private Use */
  ["\uF763", "\uF763"], /* Private Use */
  ["\uF764", "\uF764"], /* Private Use */
  ["\uF765", "\uF765"], /* Private Use */
  ["\uF766", "\uF766"], /* Private Use */
  ["\uF767", "\uF767"], /* Private Use */
  ["\uF768", "\uF768"], /* Private Use */
  ["\uF769", "\uF769"], /* Private Use */
  ["\uF76A", "\uF76A"], /* Private Use */
  ["\uF76B", "\uF76B"], /* Private Use */
  ["\uF76C", "\uF76C"], /* Private Use */
  ["\uF76D", "\uF76D"], /* Private Use */
  ["\uF76E", "\uF76E"], /* Private Use */
  ["\uF76F", "\uF76F"], /* Private Use */
  ["\uF770", "\uF770"], /* Private Use */
  ["\uF771", "\uF771"], /* Private Use */
  ["\uF772", "\uF772"], /* Private Use */
  ["\uF773", "\uF773"], /* Private Use */
  ["\uF774", "\uF774"], /* Private Use */
  ["\uF775", "\uF775"], /* Private Use */
  ["\uF776", "\uF776"], /* Private Use */
  ["\uF777", "\uF777"], /* Private Use */
  ["\uF778", "\uF778"], /* Private Use */
  ["\uF779", "\uF779"], /* Private Use */
  ["\uF77A", "\uF77A"], /* Private Use */
  ["\uF77B", "\uF77B"], /* Private Use */
  ["\uF77C", "\uF77C"], /* Private Use */
  ["\uF77D", "\uF77D"], /* Private Use */
  ["\uF77E", "\uF77E"], /* Private Use */
  ["\uF77F", "\uF77F"], /* Private Use */
  ["\uF780", "\uF780"], /* Private Use */
  ["\uF781", "\uF781"], /* Private Use */
  ["\uF782", "\uF782"], /* Private Use */
  ["\uF783", "\uF783"], /* Private Use */
  ["\uF784", "\uF784"], /* Private Use */
  ["\uF785", "\uF785"], /* Private Use */
  ["\uF786", "\uF786"], /* Private Use */
  ["\uF787", "\uF787"], /* Private Use */
  ["\uF788", "\uF788"], /* Private Use */
  ["\uF789", "\uF789"], /* Private Use */
  ["\uF78A", "\uF78A"], /* Private Use */
  ["\uF78B", "\uF78B"], /* Private Use */
  ["\uF78C", "\uF78C"], /* Private Use */
  ["\uF78D", "\uF78D"], /* Private Use */
  ["\uF78E", "\uF78E"], /* Private Use */
  ["\uF78F", "\uF78F"], /* Private Use */
  ["\uF790", "\uF790"], /* Private Use */
  ["\uF791", "\uF791"], /* Private Use */
  ["\uF792", "\uF792"], /* Private Use */
  ["\uF793", "\uF793"], /* Private Use */
  ["\uF794", "\uF794"], /* Private Use */
  ["\uF795", "\uF795"], /* Private Use */
  ["\uF796", "\uF796"], /* Private Use */
  ["\uF797", "\uF797"], /* Private Use */
  ["\uF798", "\uF798"], /* Private Use */
  ["\uF799", "\uF799"], /* Private Use */
  ["\uF79A", "\uF79A"], /* Private Use */
  ["\uF79B", "\uF79B"], /* Private Use */
  ["\uF79C", "\uF79C"], /* Private Use */
  ["\uF79D", "\uF79D"], /* Private Use */
  ["\uF79E", "\uF79E"], /* Private Use */
  ["\uF79F", "\uF79F"], /* Private Use */
  ["\uF7A0", "\uF7A0"], /* Private Use */
  ["\uF7A1", "\uF7A1"], /* Private Use */
  ["\uF7A2", "\uF7A2"], /* Private Use */
  ["\uF7A3", "\uF7A3"], /* Private Use */
  ["\uF7A4", "\uF7A4"], /* Private Use */
  ["\uF7A5", "\uF7A5"], /* Private Use */
  ["\uF7A6", "\uF7A6"], /* Private Use */
  ["\uF7A7", "\uF7A7"], /* Private Use */
  ["\uF7A8", "\uF7A8"], /* Private Use */
  ["\uF7A9", "\uF7A9"], /* Private Use */
  ["\uF7AA", "\uF7AA"], /* Private Use */
  ["\uF7AB", "\uF7AB"], /* Private Use */
  ["\uF7AC", "\uF7AC"], /* Private Use */
  ["\uF7AD", "\uF7AD"], /* Private Use */
  ["\uF7AE", "\uF7AE"], /* Private Use */
  ["\uF7AF", "\uF7AF"], /* Private Use */
  ["\uF7B0", "\uF7B0"], /* Private Use */
  ["\uF7B1", "\uF7B1"], /* Private Use */
  ["\uF7B2", "\uF7B2"], /* Private Use */
  ["\uF7B3", "\uF7B3"], /* Private Use */
  ["\uF7B4", "\uF7B4"], /* Private Use */
  ["\uF7B5", "\uF7B5"], /* Private Use */
  ["\uF7B6", "\uF7B6"], /* Private Use */
  ["\uF7B7", "\uF7B7"], /* Private Use */
  ["\uF7B8", "\uF7B8"], /* Private Use */
  ["\uF7B9", "\uF7B9"], /* Private Use */
  ["\uF7BA", "\uF7BA"], /* Private Use */
  ["\uF7BB", "\uF7BB"], /* Private Use */
  ["\uF7BC", "\uF7BC"], /* Private Use */
  ["\uF7BD", "\uF7BD"], /* Private Use */
  ["\uF7BE", "\uF7BE"], /* Private Use */
  ["\uF7BF", "\uF7BF"], /* Private Use */
  ["\uF7C0", "\uF7C0"], /* Private Use */
  ["\uF7C1", "\uF7C1"], /* Private Use */
  ["\uF7C2", "\uF7C2"], /* Private Use */
  ["\uF7C3", "\uF7C3"], /* Private Use */
  ["\uF7C4", "\uF7C4"], /* Private Use */
  ["\uF7C5", "\uF7C5"], /* Private Use */
  ["\uF7C6", "\uF7C6"], /* Private Use */
  ["\uF7C7", "\uF7C7"], /* Private Use */
  ["\uF7C8", "\uF7C8"], /* Private Use */
  ["\uF7C9", "\uF7C9"], /* Private Use */
  ["\uF7CA", "\uF7CA"], /* Private Use */
  ["\uF7CB", "\uF7CB"], /* Private Use */
  ["\uF7CC", "\uF7CC"], /* Private Use */
  ["\uF7CD", "\uF7CD"], /* Private Use */
  ["\uF7CE", "\uF7CE"], /* Private Use */
  ["\uF7CF", "\uF7CF"], /* Private Use */
  ["\uF7D0", "\uF7D0"], /* Private Use */
  ["\uF7D1", "\uF7D1"], /* Private Use */
  ["\uF7D2", "\uF7D2"], /* Private Use */
  ["\uF7D3", "\uF7D3"], /* Private Use */
  ["\uF7D4", "\uF7D4"], /* Private Use */
  ["\uF7D5", "\uF7D5"], /* Private Use */
  ["\uF7D6", "\uF7D6"], /* Private Use */
  ["\uF7D7", "\uF7D7"], /* Private Use */
  ["\uF7D8", "\uF7D8"], /* Private Use */
  ["\uF7D9", "\uF7D9"], /* Private Use */
  ["\uF7DA", "\uF7DA"], /* Private Use */
  ["\uF7DB", "\uF7DB"], /* Private Use */
  ["\uF7DC", "\uF7DC"], /* Private Use */
  ["\uF7DD", "\uF7DD"], /* Private Use */
  ["\uF7DE", "\uF7DE"], /* Private Use */
  ["\uF7DF", "\uF7DF"], /* Private Use */
  ["\uF7E0", "\uF7E0"], /* Private Use */
  ["\uF7E1", "\uF7E1"], /* Private Use */
  ["\uF7E2", "\uF7E2"], /* Private Use */
  ["\uF7E3", "\uF7E3"], /* Private Use */
  ["\uF7E4", "\uF7E4"], /* Private Use */
  ["\uF7E5", "\uF7E5"], /* Private Use */
  ["\uF7E6", "\uF7E6"], /* Private Use */
  ["\uF7E7", "\uF7E7"], /* Private Use */
  ["\uF7E8", "\uF7E8"], /* Private Use */
  ["\uF7E9", "\uF7E9"], /* Private Use */
  ["\uF7EA", "\uF7EA"], /* Private Use */
  ["\uF7EB", "\uF7EB"], /* Private Use */
  ["\uF7EC", "\uF7EC"], /* Private Use */
  ["\uF7ED", "\uF7ED"], /* Private Use */
  ["\uF7EE", "\uF7EE"], /* Private Use */
  ["\uF7EF", "\uF7EF"], /* Private Use */
  ["\uF7F0", "\uF7F0"], /* Private Use */
  ["\uF7F1", "\uF7F1"], /* Private Use */
  ["\uF7F2", "\uF7F2"], /* Private Use */
  ["\uF7F3", "\uF7F3"], /* Private Use */
  ["\uF7F4", "\uF7F4"], /* Private Use */
  ["\uF7F5", "\uF7F5"], /* Private Use */
  ["\uF7F6", "\uF7F6"], /* Private Use */
  ["\uF7F7", "\uF7F7"], /* Private Use */
  ["\uF7F8", "\uF7F8"], /* Private Use */
  ["\uF7F9", "\uF7F9"], /* Private Use */
  ["\uF7FA", "\uF7FA"], /* Private Use */
  ["\uF7FB", "\uF7FB"], /* Private Use */
  ["\uF7FC", "\uF7FC"], /* Private Use */
  ["\uF7FD", "\uF7FD"], /* Private Use */
  ["\uF7FE", "\uF7FE"], /* Private Use */
  ["\uF7FF", "\uF7FF"], /* Private Use */
  ["\uF800", "\uF800"], /* Private Use */
  ["\uF801", "\uF801"], /* Private Use */
  ["\uF802", "\uF802"], /* Private Use */
  ["\uF803", "\uF803"], /* Private Use */
  ["\uF804", "\uF804"], /* Private Use */
  ["\uF805", "\uF805"], /* Private Use */
  ["\uF806", "\uF806"], /* Private Use */
  ["\uF807", "\uF807"], /* Private Use */
  ["\uF808", "\uF808"], /* Private Use */
  ["\uF809", "\uF809"], /* Private Use */
  ["\uF80A", "\uF80A"], /* Private Use */
  ["\uF80B", "\uF80B"], /* Private Use */
  ["\uF80C", "\uF80C"], /* Private Use */
  ["\uF80D", "\uF80D"], /* Private Use */
  ["\uF80E", "\uF80E"], /* Private Use */
  ["\uF80F", "\uF80F"], /* Private Use */
  ["\uF810", "\uF810"], /* Private Use */
  ["\uF811", "\uF811"], /* Private Use */
  ["\uF812", "\uF812"], /* Private Use */
  ["\uF813", "\uF813"], /* Private Use */
  ["\uF814", "\uF814"], /* Private Use */
  ["\uF815", "\uF815"], /* Private Use */
  ["\uF816", "\uF816"], /* Private Use */
  ["\uF817", "\uF817"], /* Private Use */
  ["\uF818", "\uF818"], /* Private Use */
  ["\uF819", "\uF819"], /* Private Use */
  ["\uF81A", "\uF81A"], /* Private Use */
  ["\uF81B", "\uF81B"], /* Private Use */
  ["\uF81C", "\uF81C"], /* Private Use */
  ["\uF81D", "\uF81D"], /* Private Use */
  ["\uF81E", "\uF81E"], /* Private Use */
  ["\uF81F", "\uF81F"], /* Private Use */
  ["\uF820", "\uF820"], /* Private Use */
  ["\uF821", "\uF821"], /* Private Use */
  ["\uF822", "\uF822"], /* Private Use */
  ["\uF823", "\uF823"], /* Private Use */
  ["\uF824", "\uF824"], /* Private Use */
  ["\uF825", "\uF825"], /* Private Use */
  ["\uF826", "\uF826"], /* Private Use */
  ["\uF827", "\uF827"], /* Private Use */
  ["\uF828", "\uF828"], /* Private Use */
  ["\uF829", "\uF829"], /* Private Use */
  ["\uF82A", "\uF82A"], /* Private Use */
  ["\uF82B", "\uF82B"], /* Private Use */
  ["\uF82C", "\uF82C"], /* Private Use */
  ["\uF82D", "\uF82D"], /* Private Use */
  ["\uF82E", "\uF82E"], /* Private Use */
  ["\uF82F", "\uF82F"], /* Private Use */
  ["\uF830", "\uF830"], /* Private Use */
  ["\uF831", "\uF831"], /* Private Use */
  ["\uF832", "\uF832"], /* Private Use */
  ["\uF833", "\uF833"], /* Private Use */
  ["\uF834", "\uF834"], /* Private Use */
  ["\uF835", "\uF835"], /* Private Use */
  ["\uF836", "\uF836"], /* Private Use */
  ["\uF837", "\uF837"], /* Private Use */
  ["\uF838", "\uF838"], /* Private Use */
  ["\uF839", "\uF839"], /* Private Use */
  ["\uF83A", "\uF83A"], /* Private Use */
  ["\uF83B", "\uF83B"], /* Private Use */
  ["\uF83C", "\uF83C"], /* Private Use */
  ["\uF83D", "\uF83D"], /* Private Use */
  ["\uF83E", "\uF83E"], /* Private Use */
  ["\uF83F", "\uF83F"], /* Private Use */
  ["\uF840", "\uF840"], /* Private Use */
  ["\uF841", "\uF841"], /* Private Use */
  ["\uF842", "\uF842"], /* Private Use */
  ["\uF843", "\uF843"], /* Private Use */
  ["\uF844", "\uF844"], /* Private Use */
  ["\uF845", "\uF845"], /* Private Use */
  ["\uF846", "\uF846"], /* Private Use */
  ["\uF847", "\uF847"], /* Private Use */
  ["\uF848", "\uF848"], /* Private Use */
  ["\uF849", "\uF849"], /* Private Use */
  ["\uF84A", "\uF84A"], /* Private Use */
  ["\uF84B", "\uF84B"], /* Private Use */
  ["\uF84C", "\uF84C"], /* Private Use */
  ["\uF84D", "\uF84D"], /* Private Use */
  ["\uF84E", "\uF84E"], /* Private Use */
  ["\uF84F", "\uF84F"], /* Private Use */
  ["\uF850", "\uF850"], /* Private Use */
  ["\uF851", "\uF851"], /* Private Use */
  ["\uF852", "\uF852"], /* Private Use */
  ["\uF853", "\uF853"], /* Private Use */
  ["\uF854", "\uF854"], /* Private Use */
  ["\uF855", "\uF855"], /* Private Use */
  ["\uF856", "\uF856"], /* Private Use */
  ["\uF857", "\uF857"], /* Private Use */
  ["\uF858", "\uF858"], /* Private Use */
  ["\uF859", "\uF859"], /* Private Use */
  ["\uF85A", "\uF85A"], /* Private Use */
  ["\uF85B", "\uF85B"], /* Private Use */
  ["\uF85C", "\uF85C"], /* Private Use */
  ["\uF85D", "\uF85D"], /* Private Use */
  ["\uF85E", "\uF85E"], /* Private Use */
  ["\uF85F", "\uF85F"], /* Private Use */
  ["\uF860", "\uF860"], /* Private Use */
  ["\uF861", "\uF861"], /* Private Use */
  ["\uF862", "\uF862"], /* Private Use */
  ["\uF863", "\uF863"], /* Private Use */
  ["\uF864", "\uF864"], /* Private Use */
  ["\uF865", "\uF865"], /* Private Use */
  ["\uF866", "\uF866"], /* Private Use */
  ["\uF867", "\uF867"], /* Private Use */
  ["\uF868", "\uF868"], /* Private Use */
  ["\uF869", "\uF869"], /* Private Use */
  ["\uF86A", "\uF86A"], /* Private Use */
  ["\uF86B", "\uF86B"], /* Private Use */
  ["\uF86C", "\uF86C"], /* Private Use */
  ["\uF86D", "\uF86D"], /* Private Use */
  ["\uF86E", "\uF86E"], /* Private Use */
  ["\uF86F", "\uF86F"], /* Private Use */
  ["\uF870", "\uF870"], /* Private Use */
  ["\uF871", "\uF871"], /* Private Use */
  ["\uF872", "\uF872"], /* Private Use */
  ["\uF873", "\uF873"], /* Private Use */
  ["\uF874", "\uF874"], /* Private Use */
  ["\uF875", "\uF875"], /* Private Use */
  ["\uF876", "\uF876"], /* Private Use */
  ["\uF877", "\uF877"], /* Private Use */
  ["\uF878", "\uF878"], /* Private Use */
  ["\uF879", "\uF879"], /* Private Use */
  ["\uF87A", "\uF87A"], /* Private Use */
  ["\uF87B", "\uF87B"], /* Private Use */
  ["\uF87C", "\uF87C"], /* Private Use */
  ["\uF87D", "\uF87D"], /* Private Use */
  ["\uF87E", "\uF87E"], /* Private Use */
  ["\uF87F", "\uF87F"], /* Private Use */
  ["\uF880", "\uF880"], /* Private Use */
  ["\uF881", "\uF881"], /* Private Use */
  ["\uF882", "\uF882"], /* Private Use */
  ["\uF883", "\uF883"], /* Private Use */
  ["\uF884", "\uF884"], /* Private Use */
  ["\uF885", "\uF885"], /* Private Use */
  ["\uF886", "\uF886"], /* Private Use */
  ["\uF887", "\uF887"], /* Private Use */
  ["\uF888", "\uF888"], /* Private Use */
  ["\uF889", "\uF889"], /* Private Use */
  ["\uF88A", "\uF88A"], /* Private Use */
  ["\uF88B", "\uF88B"], /* Private Use */
  ["\uF88C", "\uF88C"], /* Private Use */
  ["\uF88D", "\uF88D"], /* Private Use */
  ["\uF88E", "\uF88E"], /* Private Use */
  ["\uF88F", "\uF88F"], /* Private Use */
  ["\uF890", "\uF890"], /* Private Use */
  ["\uF891", "\uF891"], /* Private Use */
  ["\uF892", "\uF892"], /* Private Use */
  ["\uF893", "\uF893"], /* Private Use */
  ["\uF894", "\uF894"], /* Private Use */
  ["\uF895", "\uF895"], /* Private Use */
  ["\uF896", "\uF896"], /* Private Use */
  ["\uF897", "\uF897"], /* Private Use */
  ["\uF898", "\uF898"], /* Private Use */
  ["\uF899", "\uF899"], /* Private Use */
  ["\uF89A", "\uF89A"], /* Private Use */
  ["\uF89B", "\uF89B"], /* Private Use */
  ["\uF89C", "\uF89C"], /* Private Use */
  ["\uF89D", "\uF89D"], /* Private Use */
  ["\uF89E", "\uF89E"], /* Private Use */
  ["\uF89F", "\uF89F"], /* Private Use */
  ["\uF8A0", "\uF8A0"], /* Private Use */
  ["\uF8A1", "\uF8A1"], /* Private Use */
  ["\uF8A2", "\uF8A2"], /* Private Use */
  ["\uF8A3", "\uF8A3"], /* Private Use */
  ["\uF8A4", "\uF8A4"], /* Private Use */
  ["\uF8A5", "\uF8A5"], /* Private Use */
  ["\uF8A6", "\uF8A6"], /* Private Use */
  ["\uF8A7", "\uF8A7"], /* Private Use */
  ["\uF8A8", "\uF8A8"], /* Private Use */
  ["\uF8A9", "\uF8A9"], /* Private Use */
  ["\uF8AA", "\uF8AA"], /* Private Use */
  ["\uF8AB", "\uF8AB"], /* Private Use */
  ["\uF8AC", "\uF8AC"], /* Private Use */
  ["\uF8AD", "\uF8AD"], /* Private Use */
  ["\uF8AE", "\uF8AE"], /* Private Use */
  ["\uF8AF", "\uF8AF"], /* Private Use */
  ["\uF8B0", "\uF8B0"], /* Private Use */
  ["\uF8B1", "\uF8B1"], /* Private Use */
  ["\uF8B2", "\uF8B2"], /* Private Use */
  ["\uF8B3", "\uF8B3"], /* Private Use */
  ["\uF8B4", "\uF8B4"], /* Private Use */
  ["\uF8B5", "\uF8B5"], /* Private Use */
  ["\uF8B6", "\uF8B6"], /* Private Use */
  ["\uF8B7", "\uF8B7"], /* Private Use */
  ["\uF8B8", "\uF8B8"], /* Private Use */
  ["\uF8B9", "\uF8B9"], /* Private Use */
  ["\uF8BA", "\uF8BA"], /* Private Use */
  ["\uF8BB", "\uF8BB"], /* Private Use */
  ["\uF8BC", "\uF8BC"], /* Private Use */
  ["\uF8BD", "\uF8BD"], /* Private Use */
  ["\uF8BE", "\uF8BE"], /* Private Use */
  ["\uF8BF", "\uF8BF"], /* Private Use */
  ["\uF8C0", "\uF8C0"], /* Private Use */
  ["\uF8C1", "\uF8C1"], /* Private Use */
  ["\uF8C2", "\uF8C2"], /* Private Use */
  ["\uF8C3", "\uF8C3"], /* Private Use */
  ["\uF8C4", "\uF8C4"], /* Private Use */
  ["\uF8C5", "\uF8C5"], /* Private Use */
  ["\uF8C6", "\uF8C6"], /* Private Use */
  ["\uF8C7", "\uF8C7"], /* Private Use */
  ["\uF8C8", "\uF8C8"], /* Private Use */
  ["\uF8C9", "\uF8C9"], /* Private Use */
  ["\uF8CA", "\uF8CA"], /* Private Use */
  ["\uF8CB", "\uF8CB"], /* Private Use */
  ["\uF8CC", "\uF8CC"], /* Private Use */
  ["\uF8CD", "\uF8CD"], /* Private Use */
  ["\uF8CE", "\uF8CE"], /* Private Use */
  ["\uF8CF", "\uF8CF"], /* Private Use */
  ["\uF8D0", "\uF8D0"], /* Private Use */
  ["\uF8D1", "\uF8D1"], /* Private Use */
  ["\uF8D2", "\uF8D2"], /* Private Use */
  ["\uF8D3", "\uF8D3"], /* Private Use */
  ["\uF8D4", "\uF8D4"], /* Private Use */
  ["\uF8D5", "\uF8D5"], /* Private Use */
  ["\uF8D6", "\uF8D6"], /* Private Use */
  ["\uF8D7", "\uF8D7"], /* Private Use */
  ["\uF8D8", "\uF8D8"], /* Private Use */
  ["\uF8D9", "\uF8D9"], /* Private Use */
  ["\uF8DA", "\uF8DA"], /* Private Use */
  ["\uF8DB", "\uF8DB"], /* Private Use */
  ["\uF8DC", "\uF8DC"], /* Private Use */
  ["\uF8DD", "\uF8DD"], /* Private Use */
  ["\uF8DE", "\uF8DE"], /* Private Use */
  ["\uF8DF", "\uF8DF"], /* Private Use */
  ["\uF8E0", "\uF8E0"], /* Private Use */
  ["\uF8E1", "\uF8E1"], /* Private Use */
  ["\uF8E2", "\uF8E2"], /* Private Use */
  ["\uF8E3", "\uF8E3"], /* Private Use */
  ["\uF8E4", "\uF8E4"], /* Private Use */
  ["\uF8E5", "\uF8E5"], /* Private Use */
  ["\uF8E6", "\uF8E6"], /* Private Use */
  ["\uF8E7", "\uF8E7"], /* Private Use */
  ["\uF8E8", "\uF8E8"], /* Private Use */
  ["\uF8E9", "\uF8E9"], /* Private Use */
  ["\uF8EA", "\uF8EA"], /* Private Use */
  ["\uF8EB", "\uF8EB"], /* Private Use */
  ["\uF8EC", "\uF8EC"], /* Private Use */
  ["\uF8ED", "\uF8ED"], /* Private Use */
  ["\uF8EE", "\uF8EE"], /* Private Use */
  ["\uF8EF", "\uF8EF"], /* Private Use */
  ["\uF8F0", "\uF8F0"], /* Private Use */
  ["\uF8F1", "\uF8F1"], /* Private Use */
  ["\uF8F2", "\uF8F2"], /* Private Use */
  ["\uF8F3", "\uF8F3"], /* Private Use */
  ["\uF8F4", "\uF8F4"], /* Private Use */
  ["\uF8F5", "\uF8F5"], /* Private Use */
  ["\uF8F6", "\uF8F6"], /* Private Use */
  ["\uF8F7", "\uF8F7"], /* Private Use */
  ["\uF8F8", "\uF8F8"], /* Private Use */
  ["\uF8F9", "\uF8F9"], /* Private Use */
  ["\uF8FA", "\uF8FA"], /* Private Use */
  ["\uF8FB", "\uF8FB"], /* Private Use */
  ["\uF8FC", "\uF8FC"], /* Private Use */
  ["\uF8FD", "\uF8FD"], /* Private Use */
  ["\uF8FE", "\uF8FE"], /* Private Use */
  ["\uF8FF", "\uF8FF"], /* Private Use */
  ["\uF900", "\uF900"], /* CJK COMPATIBILITY IDEOGRAPH-F900 */
  ["\uF901", "\uF901"], /* CJK COMPATIBILITY IDEOGRAPH-F901 */
  ["\uF902", "\uF902"], /* CJK COMPATIBILITY IDEOGRAPH-F902 */
  ["\uF903", "\uF903"], /* CJK COMPATIBILITY IDEOGRAPH-F903 */
  ["\uF904", "\uF904"], /* CJK COMPATIBILITY IDEOGRAPH-F904 */
  ["\uF905", "\uF905"], /* CJK COMPATIBILITY IDEOGRAPH-F905 */
  ["\uF906", "\uF906"], /* CJK COMPATIBILITY IDEOGRAPH-F906 */
  ["\uF907", "\uF907"], /* CJK COMPATIBILITY IDEOGRAPH-F907 */
  ["\uF908", "\uF908"], /* CJK COMPATIBILITY IDEOGRAPH-F908 */
  ["\uF909", "\uF909"], /* CJK COMPATIBILITY IDEOGRAPH-F909 */
  ["\uF90A", "\uF90A"], /* CJK COMPATIBILITY IDEOGRAPH-F90A */
  ["\uF90B", "\uF90B"], /* CJK COMPATIBILITY IDEOGRAPH-F90B */
  ["\uF90C", "\uF90C"], /* CJK COMPATIBILITY IDEOGRAPH-F90C */
  ["\uF90D", "\uF90D"], /* CJK COMPATIBILITY IDEOGRAPH-F90D */
  ["\uF90E", "\uF90E"], /* CJK COMPATIBILITY IDEOGRAPH-F90E */
  ["\uF90F", "\uF90F"], /* CJK COMPATIBILITY IDEOGRAPH-F90F */
  ["\uF910", "\uF910"], /* CJK COMPATIBILITY IDEOGRAPH-F910 */
  ["\uF911", "\uF911"], /* CJK COMPATIBILITY IDEOGRAPH-F911 */
  ["\uF912", "\uF912"], /* CJK COMPATIBILITY IDEOGRAPH-F912 */
  ["\uF913", "\uF913"], /* CJK COMPATIBILITY IDEOGRAPH-F913 */
  ["\uF914", "\uF914"], /* CJK COMPATIBILITY IDEOGRAPH-F914 */
  ["\uF915", "\uF915"], /* CJK COMPATIBILITY IDEOGRAPH-F915 */
  ["\uF916", "\uF916"], /* CJK COMPATIBILITY IDEOGRAPH-F916 */
  ["\uF917", "\uF917"], /* CJK COMPATIBILITY IDEOGRAPH-F917 */
  ["\uF918", "\uF918"], /* CJK COMPATIBILITY IDEOGRAPH-F918 */
  ["\uF919", "\uF919"], /* CJK COMPATIBILITY IDEOGRAPH-F919 */
  ["\uF91A", "\uF91A"], /* CJK COMPATIBILITY IDEOGRAPH-F91A */
  ["\uF91B", "\uF91B"], /* CJK COMPATIBILITY IDEOGRAPH-F91B */
  ["\uF91C", "\uF91C"], /* CJK COMPATIBILITY IDEOGRAPH-F91C */
  ["\uF91D", "\uF91D"], /* CJK COMPATIBILITY IDEOGRAPH-F91D */
  ["\uF91E", "\uF91E"], /* CJK COMPATIBILITY IDEOGRAPH-F91E */
  ["\uF91F", "\uF91F"], /* CJK COMPATIBILITY IDEOGRAPH-F91F */
  ["\uF920", "\uF920"], /* CJK COMPATIBILITY IDEOGRAPH-F920 */
  ["\uF921", "\uF921"], /* CJK COMPATIBILITY IDEOGRAPH-F921 */
  ["\uF922", "\uF922"], /* CJK COMPATIBILITY IDEOGRAPH-F922 */
  ["\uF923", "\uF923"], /* CJK COMPATIBILITY IDEOGRAPH-F923 */
  ["\uF924", "\uF924"], /* CJK COMPATIBILITY IDEOGRAPH-F924 */
  ["\uF925", "\uF925"], /* CJK COMPATIBILITY IDEOGRAPH-F925 */
  ["\uF926", "\uF926"], /* CJK COMPATIBILITY IDEOGRAPH-F926 */
  ["\uF927", "\uF927"], /* CJK COMPATIBILITY IDEOGRAPH-F927 */
  ["\uF928", "\uF928"], /* CJK COMPATIBILITY IDEOGRAPH-F928 */
  ["\uF929", "\uF929"], /* CJK COMPATIBILITY IDEOGRAPH-F929 */
  ["\uF92A", "\uF92A"], /* CJK COMPATIBILITY IDEOGRAPH-F92A */
  ["\uF92B", "\uF92B"], /* CJK COMPATIBILITY IDEOGRAPH-F92B */
  ["\uF92C", "\uF92C"], /* CJK COMPATIBILITY IDEOGRAPH-F92C */
  ["\uF92D", "\uF92D"], /* CJK COMPATIBILITY IDEOGRAPH-F92D */
  ["\uF92E", "\uF92E"], /* CJK COMPATIBILITY IDEOGRAPH-F92E */
  ["\uF92F", "\uF92F"], /* CJK COMPATIBILITY IDEOGRAPH-F92F */
  ["\uF930", "\uF930"], /* CJK COMPATIBILITY IDEOGRAPH-F930 */
  ["\uF931", "\uF931"], /* CJK COMPATIBILITY IDEOGRAPH-F931 */
  ["\uF932", "\uF932"], /* CJK COMPATIBILITY IDEOGRAPH-F932 */
  ["\uF933", "\uF933"], /* CJK COMPATIBILITY IDEOGRAPH-F933 */
  ["\uF934", "\uF934"], /* CJK COMPATIBILITY IDEOGRAPH-F934 */
  ["\uF935", "\uF935"], /* CJK COMPATIBILITY IDEOGRAPH-F935 */
  ["\uF936", "\uF936"], /* CJK COMPATIBILITY IDEOGRAPH-F936 */
  ["\uF937", "\uF937"], /* CJK COMPATIBILITY IDEOGRAPH-F937 */
  ["\uF938", "\uF938"], /* CJK COMPATIBILITY IDEOGRAPH-F938 */
  ["\uF939", "\uF939"], /* CJK COMPATIBILITY IDEOGRAPH-F939 */
  ["\uF93A", "\uF93A"], /* CJK COMPATIBILITY IDEOGRAPH-F93A */
  ["\uF93B", "\uF93B"], /* CJK COMPATIBILITY IDEOGRAPH-F93B */
  ["\uF93C", "\uF93C"], /* CJK COMPATIBILITY IDEOGRAPH-F93C */
  ["\uF93D", "\uF93D"], /* CJK COMPATIBILITY IDEOGRAPH-F93D */
  ["\uF93E", "\uF93E"], /* CJK COMPATIBILITY IDEOGRAPH-F93E */
  ["\uF93F", "\uF93F"], /* CJK COMPATIBILITY IDEOGRAPH-F93F */
  ["\uF940", "\uF940"], /* CJK COMPATIBILITY IDEOGRAPH-F940 */
  ["\uF941", "\uF941"], /* CJK COMPATIBILITY IDEOGRAPH-F941 */
  ["\uF942", "\uF942"], /* CJK COMPATIBILITY IDEOGRAPH-F942 */
  ["\uF943", "\uF943"], /* CJK COMPATIBILITY IDEOGRAPH-F943 */
  ["\uF944", "\uF944"], /* CJK COMPATIBILITY IDEOGRAPH-F944 */
  ["\uF945", "\uF945"], /* CJK COMPATIBILITY IDEOGRAPH-F945 */
  ["\uF946", "\uF946"], /* CJK COMPATIBILITY IDEOGRAPH-F946 */
  ["\uF947", "\uF947"], /* CJK COMPATIBILITY IDEOGRAPH-F947 */
  ["\uF948", "\uF948"], /* CJK COMPATIBILITY IDEOGRAPH-F948 */
  ["\uF949", "\uF949"], /* CJK COMPATIBILITY IDEOGRAPH-F949 */
  ["\uF94A", "\uF94A"], /* CJK COMPATIBILITY IDEOGRAPH-F94A */
  ["\uF94B", "\uF94B"], /* CJK COMPATIBILITY IDEOGRAPH-F94B */
  ["\uF94C", "\uF94C"], /* CJK COMPATIBILITY IDEOGRAPH-F94C */
  ["\uF94D", "\uF94D"], /* CJK COMPATIBILITY IDEOGRAPH-F94D */
  ["\uF94E", "\uF94E"], /* CJK COMPATIBILITY IDEOGRAPH-F94E */
  ["\uF94F", "\uF94F"], /* CJK COMPATIBILITY IDEOGRAPH-F94F */
  ["\uF950", "\uF950"], /* CJK COMPATIBILITY IDEOGRAPH-F950 */
  ["\uF951", "\uF951"], /* CJK COMPATIBILITY IDEOGRAPH-F951 */
  ["\uF952", "\uF952"], /* CJK COMPATIBILITY IDEOGRAPH-F952 */
  ["\uF953", "\uF953"], /* CJK COMPATIBILITY IDEOGRAPH-F953 */
  ["\uF954", "\uF954"], /* CJK COMPATIBILITY IDEOGRAPH-F954 */
  ["\uF955", "\uF955"], /* CJK COMPATIBILITY IDEOGRAPH-F955 */
  ["\uF956", "\uF956"], /* CJK COMPATIBILITY IDEOGRAPH-F956 */
  ["\uF957", "\uF957"], /* CJK COMPATIBILITY IDEOGRAPH-F957 */
  ["\uF958", "\uF958"], /* CJK COMPATIBILITY IDEOGRAPH-F958 */
  ["\uF959", "\uF959"], /* CJK COMPATIBILITY IDEOGRAPH-F959 */
  ["\uF95A", "\uF95A"], /* CJK COMPATIBILITY IDEOGRAPH-F95A */
  ["\uF95B", "\uF95B"], /* CJK COMPATIBILITY IDEOGRAPH-F95B */
  ["\uF95C", "\uF95C"], /* CJK COMPATIBILITY IDEOGRAPH-F95C */
  ["\uF95D", "\uF95D"], /* CJK COMPATIBILITY IDEOGRAPH-F95D */
  ["\uF95E", "\uF95E"], /* CJK COMPATIBILITY IDEOGRAPH-F95E */
  ["\uF95F", "\uF95F"], /* CJK COMPATIBILITY IDEOGRAPH-F95F */
  ["\uF960", "\uF960"], /* CJK COMPATIBILITY IDEOGRAPH-F960 */
  ["\uF961", "\uF961"], /* CJK COMPATIBILITY IDEOGRAPH-F961 */
  ["\uF962", "\uF962"], /* CJK COMPATIBILITY IDEOGRAPH-F962 */
  ["\uF963", "\uF963"], /* CJK COMPATIBILITY IDEOGRAPH-F963 */
  ["\uF964", "\uF964"], /* CJK COMPATIBILITY IDEOGRAPH-F964 */
  ["\uF965", "\uF965"], /* CJK COMPATIBILITY IDEOGRAPH-F965 */
  ["\uF966", "\uF966"], /* CJK COMPATIBILITY IDEOGRAPH-F966 */
  ["\uF967", "\uF967"], /* CJK COMPATIBILITY IDEOGRAPH-F967 */
  ["\uF968", "\uF968"], /* CJK COMPATIBILITY IDEOGRAPH-F968 */
  ["\uF969", "\uF969"], /* CJK COMPATIBILITY IDEOGRAPH-F969 */
  ["\uF96A", "\uF96A"], /* CJK COMPATIBILITY IDEOGRAPH-F96A */
  ["\uF96B", "\uF96B"], /* CJK COMPATIBILITY IDEOGRAPH-F96B */
  ["\uF96C", "\uF96C"], /* CJK COMPATIBILITY IDEOGRAPH-F96C */
  ["\uF96D", "\uF96D"], /* CJK COMPATIBILITY IDEOGRAPH-F96D */
  ["\uF96E", "\uF96E"], /* CJK COMPATIBILITY IDEOGRAPH-F96E */
  ["\uF96F", "\uF96F"], /* CJK COMPATIBILITY IDEOGRAPH-F96F */
  ["\uF970", "\uF970"], /* CJK COMPATIBILITY IDEOGRAPH-F970 */
  ["\uF971", "\uF971"], /* CJK COMPATIBILITY IDEOGRAPH-F971 */
  ["\uF972", "\uF972"], /* CJK COMPATIBILITY IDEOGRAPH-F972 */
  ["\uF973", "\uF973"], /* CJK COMPATIBILITY IDEOGRAPH-F973 */
  ["\uF974", "\uF974"], /* CJK COMPATIBILITY IDEOGRAPH-F974 */
  ["\uF975", "\uF975"], /* CJK COMPATIBILITY IDEOGRAPH-F975 */
  ["\uF976", "\uF976"], /* CJK COMPATIBILITY IDEOGRAPH-F976 */
  ["\uF977", "\uF977"], /* CJK COMPATIBILITY IDEOGRAPH-F977 */
  ["\uF978", "\uF978"], /* CJK COMPATIBILITY IDEOGRAPH-F978 */
  ["\uF979", "\uF979"], /* CJK COMPATIBILITY IDEOGRAPH-F979 */
  ["\uF97A", "\uF97A"], /* CJK COMPATIBILITY IDEOGRAPH-F97A */
  ["\uF97B", "\uF97B"], /* CJK COMPATIBILITY IDEOGRAPH-F97B */
  ["\uF97C", "\uF97C"], /* CJK COMPATIBILITY IDEOGRAPH-F97C */
  ["\uF97D", "\uF97D"], /* CJK COMPATIBILITY IDEOGRAPH-F97D */
  ["\uF97E", "\uF97E"], /* CJK COMPATIBILITY IDEOGRAPH-F97E */
  ["\uF97F", "\uF97F"], /* CJK COMPATIBILITY IDEOGRAPH-F97F */
  ["\uF980", "\uF980"], /* CJK COMPATIBILITY IDEOGRAPH-F980 */
  ["\uF981", "\uF981"], /* CJK COMPATIBILITY IDEOGRAPH-F981 */
  ["\uF982", "\uF982"], /* CJK COMPATIBILITY IDEOGRAPH-F982 */
  ["\uF983", "\uF983"], /* CJK COMPATIBILITY IDEOGRAPH-F983 */
  ["\uF984", "\uF984"], /* CJK COMPATIBILITY IDEOGRAPH-F984 */
  ["\uF985", "\uF985"], /* CJK COMPATIBILITY IDEOGRAPH-F985 */
  ["\uF986", "\uF986"], /* CJK COMPATIBILITY IDEOGRAPH-F986 */
  ["\uF987", "\uF987"], /* CJK COMPATIBILITY IDEOGRAPH-F987 */
  ["\uF988", "\uF988"], /* CJK COMPATIBILITY IDEOGRAPH-F988 */
  ["\uF989", "\uF989"], /* CJK COMPATIBILITY IDEOGRAPH-F989 */
  ["\uF98A", "\uF98A"], /* CJK COMPATIBILITY IDEOGRAPH-F98A */
  ["\uF98B", "\uF98B"], /* CJK COMPATIBILITY IDEOGRAPH-F98B */
  ["\uF98C", "\uF98C"], /* CJK COMPATIBILITY IDEOGRAPH-F98C */
  ["\uF98D", "\uF98D"], /* CJK COMPATIBILITY IDEOGRAPH-F98D */
  ["\uF98E", "\uF98E"], /* CJK COMPATIBILITY IDEOGRAPH-F98E */
  ["\uF98F", "\uF98F"], /* CJK COMPATIBILITY IDEOGRAPH-F98F */
  ["\uF990", "\uF990"], /* CJK COMPATIBILITY IDEOGRAPH-F990 */
  ["\uF991", "\uF991"], /* CJK COMPATIBILITY IDEOGRAPH-F991 */
  ["\uF992", "\uF992"], /* CJK COMPATIBILITY IDEOGRAPH-F992 */
  ["\uF993", "\uF993"], /* CJK COMPATIBILITY IDEOGRAPH-F993 */
  ["\uF994", "\uF994"], /* CJK COMPATIBILITY IDEOGRAPH-F994 */
  ["\uF995", "\uF995"], /* CJK COMPATIBILITY IDEOGRAPH-F995 */
  ["\uF996", "\uF996"], /* CJK COMPATIBILITY IDEOGRAPH-F996 */
  ["\uF997", "\uF997"], /* CJK COMPATIBILITY IDEOGRAPH-F997 */
  ["\uF998", "\uF998"], /* CJK COMPATIBILITY IDEOGRAPH-F998 */
  ["\uF999", "\uF999"], /* CJK COMPATIBILITY IDEOGRAPH-F999 */
  ["\uF99A", "\uF99A"], /* CJK COMPATIBILITY IDEOGRAPH-F99A */
  ["\uF99B", "\uF99B"], /* CJK COMPATIBILITY IDEOGRAPH-F99B */
  ["\uF99C", "\uF99C"], /* CJK COMPATIBILITY IDEOGRAPH-F99C */
  ["\uF99D", "\uF99D"], /* CJK COMPATIBILITY IDEOGRAPH-F99D */
  ["\uF99E", "\uF99E"], /* CJK COMPATIBILITY IDEOGRAPH-F99E */
  ["\uF99F", "\uF99F"], /* CJK COMPATIBILITY IDEOGRAPH-F99F */
  ["\uF9A0", "\uF9A0"], /* CJK COMPATIBILITY IDEOGRAPH-F9A0 */
  ["\uF9A1", "\uF9A1"], /* CJK COMPATIBILITY IDEOGRAPH-F9A1 */
  ["\uF9A2", "\uF9A2"], /* CJK COMPATIBILITY IDEOGRAPH-F9A2 */
  ["\uF9A3", "\uF9A3"], /* CJK COMPATIBILITY IDEOGRAPH-F9A3 */
  ["\uF9A4", "\uF9A4"], /* CJK COMPATIBILITY IDEOGRAPH-F9A4 */
  ["\uF9A5", "\uF9A5"], /* CJK COMPATIBILITY IDEOGRAPH-F9A5 */
  ["\uF9A6", "\uF9A6"], /* CJK COMPATIBILITY IDEOGRAPH-F9A6 */
  ["\uF9A7", "\uF9A7"], /* CJK COMPATIBILITY IDEOGRAPH-F9A7 */
  ["\uF9A8", "\uF9A8"], /* CJK COMPATIBILITY IDEOGRAPH-F9A8 */
  ["\uF9A9", "\uF9A9"], /* CJK COMPATIBILITY IDEOGRAPH-F9A9 */
  ["\uF9AA", "\uF9AA"], /* CJK COMPATIBILITY IDEOGRAPH-F9AA */
  ["\uF9AB", "\uF9AB"], /* CJK COMPATIBILITY IDEOGRAPH-F9AB */
  ["\uF9AC", "\uF9AC"], /* CJK COMPATIBILITY IDEOGRAPH-F9AC */
  ["\uF9AD", "\uF9AD"], /* CJK COMPATIBILITY IDEOGRAPH-F9AD */
  ["\uF9AE", "\uF9AE"], /* CJK COMPATIBILITY IDEOGRAPH-F9AE */
  ["\uF9AF", "\uF9AF"], /* CJK COMPATIBILITY IDEOGRAPH-F9AF */
  ["\uF9B0", "\uF9B0"], /* CJK COMPATIBILITY IDEOGRAPH-F9B0 */
  ["\uF9B1", "\uF9B1"], /* CJK COMPATIBILITY IDEOGRAPH-F9B1 */
  ["\uF9B2", "\uF9B2"], /* CJK COMPATIBILITY IDEOGRAPH-F9B2 */
  ["\uF9B3", "\uF9B3"], /* CJK COMPATIBILITY IDEOGRAPH-F9B3 */
  ["\uF9B4", "\uF9B4"], /* CJK COMPATIBILITY IDEOGRAPH-F9B4 */
  ["\uF9B5", "\uF9B5"], /* CJK COMPATIBILITY IDEOGRAPH-F9B5 */
  ["\uF9B6", "\uF9B6"], /* CJK COMPATIBILITY IDEOGRAPH-F9B6 */
  ["\uF9B7", "\uF9B7"], /* CJK COMPATIBILITY IDEOGRAPH-F9B7 */
  ["\uF9B8", "\uF9B8"], /* CJK COMPATIBILITY IDEOGRAPH-F9B8 */
  ["\uF9B9", "\uF9B9"], /* CJK COMPATIBILITY IDEOGRAPH-F9B9 */
  ["\uF9BA", "\uF9BA"], /* CJK COMPATIBILITY IDEOGRAPH-F9BA */
  ["\uF9BB", "\uF9BB"], /* CJK COMPATIBILITY IDEOGRAPH-F9BB */
  ["\uF9BC", "\uF9BC"], /* CJK COMPATIBILITY IDEOGRAPH-F9BC */
  ["\uF9BD", "\uF9BD"], /* CJK COMPATIBILITY IDEOGRAPH-F9BD */
  ["\uF9BE", "\uF9BE"], /* CJK COMPATIBILITY IDEOGRAPH-F9BE */
  ["\uF9BF", "\uF9BF"], /* CJK COMPATIBILITY IDEOGRAPH-F9BF */
  ["\uF9C0", "\uF9C0"], /* CJK COMPATIBILITY IDEOGRAPH-F9C0 */
  ["\uF9C1", "\uF9C1"], /* CJK COMPATIBILITY IDEOGRAPH-F9C1 */
  ["\uF9C2", "\uF9C2"], /* CJK COMPATIBILITY IDEOGRAPH-F9C2 */
  ["\uF9C3", "\uF9C3"], /* CJK COMPATIBILITY IDEOGRAPH-F9C3 */
  ["\uF9C4", "\uF9C4"], /* CJK COMPATIBILITY IDEOGRAPH-F9C4 */
  ["\uF9C5", "\uF9C5"], /* CJK COMPATIBILITY IDEOGRAPH-F9C5 */
  ["\uF9C6", "\uF9C6"], /* CJK COMPATIBILITY IDEOGRAPH-F9C6 */
  ["\uF9C7", "\uF9C7"], /* CJK COMPATIBILITY IDEOGRAPH-F9C7 */
  ["\uF9C8", "\uF9C8"], /* CJK COMPATIBILITY IDEOGRAPH-F9C8 */
  ["\uF9C9", "\uF9C9"], /* CJK COMPATIBILITY IDEOGRAPH-F9C9 */
  ["\uF9CA", "\uF9CA"], /* CJK COMPATIBILITY IDEOGRAPH-F9CA */
  ["\uF9CB", "\uF9CB"], /* CJK COMPATIBILITY IDEOGRAPH-F9CB */
  ["\uF9CC", "\uF9CC"], /* CJK COMPATIBILITY IDEOGRAPH-F9CC */
  ["\uF9CD", "\uF9CD"], /* CJK COMPATIBILITY IDEOGRAPH-F9CD */
  ["\uF9CE", "\uF9CE"], /* CJK COMPATIBILITY IDEOGRAPH-F9CE */
  ["\uF9CF", "\uF9CF"], /* CJK COMPATIBILITY IDEOGRAPH-F9CF */
  ["\uF9D0", "\uF9D0"], /* CJK COMPATIBILITY IDEOGRAPH-F9D0 */
  ["\uF9D1", "\uF9D1"], /* CJK COMPATIBILITY IDEOGRAPH-F9D1 */
  ["\uF9D2", "\uF9D2"], /* CJK COMPATIBILITY IDEOGRAPH-F9D2 */
  ["\uF9D3", "\uF9D3"], /* CJK COMPATIBILITY IDEOGRAPH-F9D3 */
  ["\uF9D4", "\uF9D4"], /* CJK COMPATIBILITY IDEOGRAPH-F9D4 */
  ["\uF9D5", "\uF9D5"], /* CJK COMPATIBILITY IDEOGRAPH-F9D5 */
  ["\uF9D6", "\uF9D6"], /* CJK COMPATIBILITY IDEOGRAPH-F9D6 */
  ["\uF9D7", "\uF9D7"], /* CJK COMPATIBILITY IDEOGRAPH-F9D7 */
  ["\uF9D8", "\uF9D8"], /* CJK COMPATIBILITY IDEOGRAPH-F9D8 */
  ["\uF9D9", "\uF9D9"], /* CJK COMPATIBILITY IDEOGRAPH-F9D9 */
  ["\uF9DA", "\uF9DA"], /* CJK COMPATIBILITY IDEOGRAPH-F9DA */
  ["\uF9DB", "\uF9DB"], /* CJK COMPATIBILITY IDEOGRAPH-F9DB */
  ["\uF9DC", "\uF9DC"], /* CJK COMPATIBILITY IDEOGRAPH-F9DC */
  ["\uF9DD", "\uF9DD"], /* CJK COMPATIBILITY IDEOGRAPH-F9DD */
  ["\uF9DE", "\uF9DE"], /* CJK COMPATIBILITY IDEOGRAPH-F9DE */
  ["\uF9DF", "\uF9DF"], /* CJK COMPATIBILITY IDEOGRAPH-F9DF */
  ["\uF9E0", "\uF9E0"], /* CJK COMPATIBILITY IDEOGRAPH-F9E0 */
  ["\uF9E1", "\uF9E1"], /* CJK COMPATIBILITY IDEOGRAPH-F9E1 */
  ["\uF9E2", "\uF9E2"], /* CJK COMPATIBILITY IDEOGRAPH-F9E2 */
  ["\uF9E3", "\uF9E3"], /* CJK COMPATIBILITY IDEOGRAPH-F9E3 */
  ["\uF9E4", "\uF9E4"], /* CJK COMPATIBILITY IDEOGRAPH-F9E4 */
  ["\uF9E5", "\uF9E5"], /* CJK COMPATIBILITY IDEOGRAPH-F9E5 */
  ["\uF9E6", "\uF9E6"], /* CJK COMPATIBILITY IDEOGRAPH-F9E6 */
  ["\uF9E7", "\uF9E7"], /* CJK COMPATIBILITY IDEOGRAPH-F9E7 */
  ["\uF9E8", "\uF9E8"], /* CJK COMPATIBILITY IDEOGRAPH-F9E8 */
  ["\uF9E9", "\uF9E9"], /* CJK COMPATIBILITY IDEOGRAPH-F9E9 */
  ["\uF9EA", "\uF9EA"], /* CJK COMPATIBILITY IDEOGRAPH-F9EA */
  ["\uF9EB", "\uF9EB"], /* CJK COMPATIBILITY IDEOGRAPH-F9EB */
  ["\uF9EC", "\uF9EC"], /* CJK COMPATIBILITY IDEOGRAPH-F9EC */
  ["\uF9ED", "\uF9ED"], /* CJK COMPATIBILITY IDEOGRAPH-F9ED */
  ["\uF9EE", "\uF9EE"], /* CJK COMPATIBILITY IDEOGRAPH-F9EE */
  ["\uF9EF", "\uF9EF"], /* CJK COMPATIBILITY IDEOGRAPH-F9EF */
  ["\uF9F0", "\uF9F0"], /* CJK COMPATIBILITY IDEOGRAPH-F9F0 */
  ["\uF9F1", "\uF9F1"], /* CJK COMPATIBILITY IDEOGRAPH-F9F1 */
  ["\uF9F2", "\uF9F2"], /* CJK COMPATIBILITY IDEOGRAPH-F9F2 */
  ["\uF9F3", "\uF9F3"], /* CJK COMPATIBILITY IDEOGRAPH-F9F3 */
  ["\uF9F4", "\uF9F4"], /* CJK COMPATIBILITY IDEOGRAPH-F9F4 */
  ["\uF9F5", "\uF9F5"], /* CJK COMPATIBILITY IDEOGRAPH-F9F5 */
  ["\uF9F6", "\uF9F6"], /* CJK COMPATIBILITY IDEOGRAPH-F9F6 */
  ["\uF9F7", "\uF9F7"], /* CJK COMPATIBILITY IDEOGRAPH-F9F7 */
  ["\uF9F8", "\uF9F8"], /* CJK COMPATIBILITY IDEOGRAPH-F9F8 */
  ["\uF9F9", "\uF9F9"], /* CJK COMPATIBILITY IDEOGRAPH-F9F9 */
  ["\uF9FA", "\uF9FA"], /* CJK COMPATIBILITY IDEOGRAPH-F9FA */
  ["\uF9FB", "\uF9FB"], /* CJK COMPATIBILITY IDEOGRAPH-F9FB */
  ["\uF9FC", "\uF9FC"], /* CJK COMPATIBILITY IDEOGRAPH-F9FC */
  ["\uF9FD", "\uF9FD"], /* CJK COMPATIBILITY IDEOGRAPH-F9FD */
  ["\uF9FE", "\uF9FE"], /* CJK COMPATIBILITY IDEOGRAPH-F9FE */
  ["\uF9FF", "\uF9FF"], /* CJK COMPATIBILITY IDEOGRAPH-F9FF */
  ["\uFA00", "\uFA00"], /* CJK COMPATIBILITY IDEOGRAPH-FA00 */
  ["\uFA01", "\uFA01"], /* CJK COMPATIBILITY IDEOGRAPH-FA01 */
  ["\uFA02", "\uFA02"], /* CJK COMPATIBILITY IDEOGRAPH-FA02 */
  ["\uFA03", "\uFA03"], /* CJK COMPATIBILITY IDEOGRAPH-FA03 */
  ["\uFA04", "\uFA04"], /* CJK COMPATIBILITY IDEOGRAPH-FA04 */
  ["\uFA05", "\uFA05"], /* CJK COMPATIBILITY IDEOGRAPH-FA05 */
  ["\uFA06", "\uFA06"], /* CJK COMPATIBILITY IDEOGRAPH-FA06 */
  ["\uFA07", "\uFA07"], /* CJK COMPATIBILITY IDEOGRAPH-FA07 */
  ["\uFA08", "\uFA08"], /* CJK COMPATIBILITY IDEOGRAPH-FA08 */
  ["\uFA09", "\uFA09"], /* CJK COMPATIBILITY IDEOGRAPH-FA09 */
  ["\uFA0A", "\uFA0A"], /* CJK COMPATIBILITY IDEOGRAPH-FA0A */
  ["\uFA0B", "\uFA0B"], /* CJK COMPATIBILITY IDEOGRAPH-FA0B */
  ["\uFA0C", "\uFA0C"], /* CJK COMPATIBILITY IDEOGRAPH-FA0C */
  ["\uFA0D", "\uFA0D"], /* CJK COMPATIBILITY IDEOGRAPH-FA0D */
  ["\uFA0E", "\uFA0E"], /* CJK COMPATIBILITY IDEOGRAPH-FA0E */
  ["\uFA0F", "\uFA0F"], /* CJK COMPATIBILITY IDEOGRAPH-FA0F */
  ["\uFA10", "\uFA10"], /* CJK COMPATIBILITY IDEOGRAPH-FA10 */
  ["\uFA11", "\uFA11"], /* CJK COMPATIBILITY IDEOGRAPH-FA11 */
  ["\uFA12", "\uFA12"], /* CJK COMPATIBILITY IDEOGRAPH-FA12 */
  ["\uFA13", "\uFA13"], /* CJK COMPATIBILITY IDEOGRAPH-FA13 */
  ["\uFA14", "\uFA14"], /* CJK COMPATIBILITY IDEOGRAPH-FA14 */
  ["\uFA15", "\uFA15"], /* CJK COMPATIBILITY IDEOGRAPH-FA15 */
  ["\uFA16", "\uFA16"], /* CJK COMPATIBILITY IDEOGRAPH-FA16 */
  ["\uFA17", "\uFA17"], /* CJK COMPATIBILITY IDEOGRAPH-FA17 */
  ["\uFA18", "\uFA18"], /* CJK COMPATIBILITY IDEOGRAPH-FA18 */
  ["\uFA19", "\uFA19"], /* CJK COMPATIBILITY IDEOGRAPH-FA19 */
  ["\uFA1A", "\uFA1A"], /* CJK COMPATIBILITY IDEOGRAPH-FA1A */
  ["\uFA1B", "\uFA1B"], /* CJK COMPATIBILITY IDEOGRAPH-FA1B */
  ["\uFA1C", "\uFA1C"], /* CJK COMPATIBILITY IDEOGRAPH-FA1C */
  ["\uFA1D", "\uFA1D"], /* CJK COMPATIBILITY IDEOGRAPH-FA1D */
  ["\uFA1E", "\uFA1E"], /* CJK COMPATIBILITY IDEOGRAPH-FA1E */
  ["\uFA1F", "\uFA1F"], /* CJK COMPATIBILITY IDEOGRAPH-FA1F */
  ["\uFA20", "\uFA20"], /* CJK COMPATIBILITY IDEOGRAPH-FA20 */
  ["\uFA21", "\uFA21"], /* CJK COMPATIBILITY IDEOGRAPH-FA21 */
  ["\uFA22", "\uFA22"], /* CJK COMPATIBILITY IDEOGRAPH-FA22 */
  ["\uFA23", "\uFA23"], /* CJK COMPATIBILITY IDEOGRAPH-FA23 */
  ["\uFA24", "\uFA24"], /* CJK COMPATIBILITY IDEOGRAPH-FA24 */
  ["\uFA25", "\uFA25"], /* CJK COMPATIBILITY IDEOGRAPH-FA25 */
  ["\uFA26", "\uFA26"], /* CJK COMPATIBILITY IDEOGRAPH-FA26 */
  ["\uFA27", "\uFA27"], /* CJK COMPATIBILITY IDEOGRAPH-FA27 */
  ["\uFA28", "\uFA28"], /* CJK COMPATIBILITY IDEOGRAPH-FA28 */
  ["\uFA29", "\uFA29"], /* CJK COMPATIBILITY IDEOGRAPH-FA29 */
  ["\uFA2A", "\uFA2A"], /* CJK COMPATIBILITY IDEOGRAPH-FA2A */
  ["\uFA2B", "\uFA2B"], /* CJK COMPATIBILITY IDEOGRAPH-FA2B */
  ["\uFA2C", "\uFA2C"], /* CJK COMPATIBILITY IDEOGRAPH-FA2C */
  ["\uFA2D", "\uFA2D"], /* CJK COMPATIBILITY IDEOGRAPH-FA2D */
  ["\uFA2E", "\uFA2E"], /* CJK COMPATIBILITY IDEOGRAPH-FA2E */
  ["\uFA2F", "\uFA2F"], /* CJK COMPATIBILITY IDEOGRAPH-FA2F */
  ["\uFA30", "\uFA30"], /* CJK COMPATIBILITY IDEOGRAPH-FA30 */
  ["\uFA31", "\uFA31"], /* CJK COMPATIBILITY IDEOGRAPH-FA31 */
  ["\uFA32", "\uFA32"], /* CJK COMPATIBILITY IDEOGRAPH-FA32 */
  ["\uFA33", "\uFA33"], /* CJK COMPATIBILITY IDEOGRAPH-FA33 */
  ["\uFA34", "\uFA34"], /* CJK COMPATIBILITY IDEOGRAPH-FA34 */
  ["\uFA35", "\uFA35"], /* CJK COMPATIBILITY IDEOGRAPH-FA35 */
  ["\uFA36", "\uFA36"], /* CJK COMPATIBILITY IDEOGRAPH-FA36 */
  ["\uFA37", "\uFA37"], /* CJK COMPATIBILITY IDEOGRAPH-FA37 */
  ["\uFA38", "\uFA38"], /* CJK COMPATIBILITY IDEOGRAPH-FA38 */
  ["\uFA39", "\uFA39"], /* CJK COMPATIBILITY IDEOGRAPH-FA39 */
  ["\uFA3A", "\uFA3A"], /* CJK COMPATIBILITY IDEOGRAPH-FA3A */
  ["\uFA3B", "\uFA3B"], /* CJK COMPATIBILITY IDEOGRAPH-FA3B */
  ["\uFA3C", "\uFA3C"], /* CJK COMPATIBILITY IDEOGRAPH-FA3C */
  ["\uFA3D", "\uFA3D"], /* CJK COMPATIBILITY IDEOGRAPH-FA3D */
  ["\uFA3E", "\uFA3E"], /* CJK COMPATIBILITY IDEOGRAPH-FA3E */
  ["\uFA3F", "\uFA3F"], /* CJK COMPATIBILITY IDEOGRAPH-FA3F */
  ["\uFA40", "\uFA40"], /* CJK COMPATIBILITY IDEOGRAPH-FA40 */
  ["\uFA41", "\uFA41"], /* CJK COMPATIBILITY IDEOGRAPH-FA41 */
  ["\uFA42", "\uFA42"], /* CJK COMPATIBILITY IDEOGRAPH-FA42 */
  ["\uFA43", "\uFA43"], /* CJK COMPATIBILITY IDEOGRAPH-FA43 */
  ["\uFA44", "\uFA44"], /* CJK COMPATIBILITY IDEOGRAPH-FA44 */
  ["\uFA45", "\uFA45"], /* CJK COMPATIBILITY IDEOGRAPH-FA45 */
  ["\uFA46", "\uFA46"], /* CJK COMPATIBILITY IDEOGRAPH-FA46 */
  ["\uFA47", "\uFA47"], /* CJK COMPATIBILITY IDEOGRAPH-FA47 */
  ["\uFA48", "\uFA48"], /* CJK COMPATIBILITY IDEOGRAPH-FA48 */
  ["\uFA49", "\uFA49"], /* CJK COMPATIBILITY IDEOGRAPH-FA49 */
  ["\uFA4A", "\uFA4A"], /* CJK COMPATIBILITY IDEOGRAPH-FA4A */
  ["\uFA4B", "\uFA4B"], /* CJK COMPATIBILITY IDEOGRAPH-FA4B */
  ["\uFA4C", "\uFA4C"], /* CJK COMPATIBILITY IDEOGRAPH-FA4C */
  ["\uFA4D", "\uFA4D"], /* CJK COMPATIBILITY IDEOGRAPH-FA4D */
  ["\uFA4E", "\uFA4E"], /* CJK COMPATIBILITY IDEOGRAPH-FA4E */
  ["\uFA4F", "\uFA4F"], /* CJK COMPATIBILITY IDEOGRAPH-FA4F */
  ["\uFA50", "\uFA50"], /* CJK COMPATIBILITY IDEOGRAPH-FA50 */
  ["\uFA51", "\uFA51"], /* CJK COMPATIBILITY IDEOGRAPH-FA51 */
  ["\uFA52", "\uFA52"], /* CJK COMPATIBILITY IDEOGRAPH-FA52 */
  ["\uFA53", "\uFA53"], /* CJK COMPATIBILITY IDEOGRAPH-FA53 */
  ["\uFA54", "\uFA54"], /* CJK COMPATIBILITY IDEOGRAPH-FA54 */
  ["\uFA55", "\uFA55"], /* CJK COMPATIBILITY IDEOGRAPH-FA55 */
  ["\uFA56", "\uFA56"], /* CJK COMPATIBILITY IDEOGRAPH-FA56 */
  ["\uFA57", "\uFA57"], /* CJK COMPATIBILITY IDEOGRAPH-FA57 */
  ["\uFA58", "\uFA58"], /* CJK COMPATIBILITY IDEOGRAPH-FA58 */
  ["\uFA59", "\uFA59"], /* CJK COMPATIBILITY IDEOGRAPH-FA59 */
  ["\uFA5A", "\uFA5A"], /* CJK COMPATIBILITY IDEOGRAPH-FA5A */
  ["\uFA5B", "\uFA5B"], /* CJK COMPATIBILITY IDEOGRAPH-FA5B */
  ["\uFA5C", "\uFA5C"], /* CJK COMPATIBILITY IDEOGRAPH-FA5C */
  ["\uFA5D", "\uFA5D"], /* CJK COMPATIBILITY IDEOGRAPH-FA5D */
  ["\uFA5E", "\uFA5E"], /* CJK COMPATIBILITY IDEOGRAPH-FA5E */
  ["\uFA5F", "\uFA5F"], /* CJK COMPATIBILITY IDEOGRAPH-FA5F */
  ["\uFA60", "\uFA60"], /* CJK COMPATIBILITY IDEOGRAPH-FA60 */
  ["\uFA61", "\uFA61"], /* CJK COMPATIBILITY IDEOGRAPH-FA61 */
  ["\uFA62", "\uFA62"], /* CJK COMPATIBILITY IDEOGRAPH-FA62 */
  ["\uFA63", "\uFA63"], /* CJK COMPATIBILITY IDEOGRAPH-FA63 */
  ["\uFA64", "\uFA64"], /* CJK COMPATIBILITY IDEOGRAPH-FA64 */
  ["\uFA65", "\uFA65"], /* CJK COMPATIBILITY IDEOGRAPH-FA65 */
  ["\uFA66", "\uFA66"], /* CJK COMPATIBILITY IDEOGRAPH-FA66 */
  ["\uFA67", "\uFA67"], /* CJK COMPATIBILITY IDEOGRAPH-FA67 */
  ["\uFA68", "\uFA68"], /* CJK COMPATIBILITY IDEOGRAPH-FA68 */
  ["\uFA69", "\uFA69"], /* CJK COMPATIBILITY IDEOGRAPH-FA69 */
  ["\uFA6A", "\uFA6A"], /* CJK COMPATIBILITY IDEOGRAPH-FA6A */
  ["\uFA6B", "\uFA6B"], /* CJK COMPATIBILITY IDEOGRAPH-FA6B */
  ["\uFA6C", "\uFA6C"], /* CJK COMPATIBILITY IDEOGRAPH-FA6C */
  ["\uFA6D", "\uFA6D"], /* CJK COMPATIBILITY IDEOGRAPH-FA6D */
  ["\uFA6E", "\uFA6E"],
  ["\uFA6F", "\uFA6F"],
  ["\uFA70", "\uFA70"], /* CJK COMPATIBILITY IDEOGRAPH-FA70 */
  ["\uFA71", "\uFA71"], /* CJK COMPATIBILITY IDEOGRAPH-FA71 */
  ["\uFA72", "\uFA72"], /* CJK COMPATIBILITY IDEOGRAPH-FA72 */
  ["\uFA73", "\uFA73"], /* CJK COMPATIBILITY IDEOGRAPH-FA73 */
  ["\uFA74", "\uFA74"], /* CJK COMPATIBILITY IDEOGRAPH-FA74 */
  ["\uFA75", "\uFA75"], /* CJK COMPATIBILITY IDEOGRAPH-FA75 */
  ["\uFA76", "\uFA76"], /* CJK COMPATIBILITY IDEOGRAPH-FA76 */
  ["\uFA77", "\uFA77"], /* CJK COMPATIBILITY IDEOGRAPH-FA77 */
  ["\uFA78", "\uFA78"], /* CJK COMPATIBILITY IDEOGRAPH-FA78 */
  ["\uFA79", "\uFA79"], /* CJK COMPATIBILITY IDEOGRAPH-FA79 */
  ["\uFA7A", "\uFA7A"], /* CJK COMPATIBILITY IDEOGRAPH-FA7A */
  ["\uFA7B", "\uFA7B"], /* CJK COMPATIBILITY IDEOGRAPH-FA7B */
  ["\uFA7C", "\uFA7C"], /* CJK COMPATIBILITY IDEOGRAPH-FA7C */
  ["\uFA7D", "\uFA7D"], /* CJK COMPATIBILITY IDEOGRAPH-FA7D */
  ["\uFA7E", "\uFA7E"], /* CJK COMPATIBILITY IDEOGRAPH-FA7E */
  ["\uFA7F", "\uFA7F"], /* CJK COMPATIBILITY IDEOGRAPH-FA7F */
  ["\uFA80", "\uFA80"], /* CJK COMPATIBILITY IDEOGRAPH-FA80 */
  ["\uFA81", "\uFA81"], /* CJK COMPATIBILITY IDEOGRAPH-FA81 */
  ["\uFA82", "\uFA82"], /* CJK COMPATIBILITY IDEOGRAPH-FA82 */
  ["\uFA83", "\uFA83"], /* CJK COMPATIBILITY IDEOGRAPH-FA83 */
  ["\uFA84", "\uFA84"], /* CJK COMPATIBILITY IDEOGRAPH-FA84 */
  ["\uFA85", "\uFA85"], /* CJK COMPATIBILITY IDEOGRAPH-FA85 */
  ["\uFA86", "\uFA86"], /* CJK COMPATIBILITY IDEOGRAPH-FA86 */
  ["\uFA87", "\uFA87"], /* CJK COMPATIBILITY IDEOGRAPH-FA87 */
  ["\uFA88", "\uFA88"], /* CJK COMPATIBILITY IDEOGRAPH-FA88 */
  ["\uFA89", "\uFA89"], /* CJK COMPATIBILITY IDEOGRAPH-FA89 */
  ["\uFA8A", "\uFA8A"], /* CJK COMPATIBILITY IDEOGRAPH-FA8A */
  ["\uFA8B", "\uFA8B"], /* CJK COMPATIBILITY IDEOGRAPH-FA8B */
  ["\uFA8C", "\uFA8C"], /* CJK COMPATIBILITY IDEOGRAPH-FA8C */
  ["\uFA8D", "\uFA8D"], /* CJK COMPATIBILITY IDEOGRAPH-FA8D */
  ["\uFA8E", "\uFA8E"], /* CJK COMPATIBILITY IDEOGRAPH-FA8E */
  ["\uFA8F", "\uFA8F"], /* CJK COMPATIBILITY IDEOGRAPH-FA8F */
  ["\uFA90", "\uFA90"], /* CJK COMPATIBILITY IDEOGRAPH-FA90 */
  ["\uFA91", "\uFA91"], /* CJK COMPATIBILITY IDEOGRAPH-FA91 */
  ["\uFA92", "\uFA92"], /* CJK COMPATIBILITY IDEOGRAPH-FA92 */
  ["\uFA93", "\uFA93"], /* CJK COMPATIBILITY IDEOGRAPH-FA93 */
  ["\uFA94", "\uFA94"], /* CJK COMPATIBILITY IDEOGRAPH-FA94 */
  ["\uFA95", "\uFA95"], /* CJK COMPATIBILITY IDEOGRAPH-FA95 */
  ["\uFA96", "\uFA96"], /* CJK COMPATIBILITY IDEOGRAPH-FA96 */
  ["\uFA97", "\uFA97"], /* CJK COMPATIBILITY IDEOGRAPH-FA97 */
  ["\uFA98", "\uFA98"], /* CJK COMPATIBILITY IDEOGRAPH-FA98 */
  ["\uFA99", "\uFA99"], /* CJK COMPATIBILITY IDEOGRAPH-FA99 */
  ["\uFA9A", "\uFA9A"], /* CJK COMPATIBILITY IDEOGRAPH-FA9A */
  ["\uFA9B", "\uFA9B"], /* CJK COMPATIBILITY IDEOGRAPH-FA9B */
  ["\uFA9C", "\uFA9C"], /* CJK COMPATIBILITY IDEOGRAPH-FA9C */
  ["\uFA9D", "\uFA9D"], /* CJK COMPATIBILITY IDEOGRAPH-FA9D */
  ["\uFA9E", "\uFA9E"], /* CJK COMPATIBILITY IDEOGRAPH-FA9E */
  ["\uFA9F", "\uFA9F"], /* CJK COMPATIBILITY IDEOGRAPH-FA9F */
  ["\uFAA0", "\uFAA0"], /* CJK COMPATIBILITY IDEOGRAPH-FAA0 */
  ["\uFAA1", "\uFAA1"], /* CJK COMPATIBILITY IDEOGRAPH-FAA1 */
  ["\uFAA2", "\uFAA2"], /* CJK COMPATIBILITY IDEOGRAPH-FAA2 */
  ["\uFAA3", "\uFAA3"], /* CJK COMPATIBILITY IDEOGRAPH-FAA3 */
  ["\uFAA4", "\uFAA4"], /* CJK COMPATIBILITY IDEOGRAPH-FAA4 */
  ["\uFAA5", "\uFAA5"], /* CJK COMPATIBILITY IDEOGRAPH-FAA5 */
  ["\uFAA6", "\uFAA6"], /* CJK COMPATIBILITY IDEOGRAPH-FAA6 */
  ["\uFAA7", "\uFAA7"], /* CJK COMPATIBILITY IDEOGRAPH-FAA7 */
  ["\uFAA8", "\uFAA8"], /* CJK COMPATIBILITY IDEOGRAPH-FAA8 */
  ["\uFAA9", "\uFAA9"], /* CJK COMPATIBILITY IDEOGRAPH-FAA9 */
  ["\uFAAA", "\uFAAA"], /* CJK COMPATIBILITY IDEOGRAPH-FAAA */
  ["\uFAAB", "\uFAAB"], /* CJK COMPATIBILITY IDEOGRAPH-FAAB */
  ["\uFAAC", "\uFAAC"], /* CJK COMPATIBILITY IDEOGRAPH-FAAC */
  ["\uFAAD", "\uFAAD"], /* CJK COMPATIBILITY IDEOGRAPH-FAAD */
  ["\uFAAE", "\uFAAE"], /* CJK COMPATIBILITY IDEOGRAPH-FAAE */
  ["\uFAAF", "\uFAAF"], /* CJK COMPATIBILITY IDEOGRAPH-FAAF */
  ["\uFAB0", "\uFAB0"], /* CJK COMPATIBILITY IDEOGRAPH-FAB0 */
  ["\uFAB1", "\uFAB1"], /* CJK COMPATIBILITY IDEOGRAPH-FAB1 */
  ["\uFAB2", "\uFAB2"], /* CJK COMPATIBILITY IDEOGRAPH-FAB2 */
  ["\uFAB3", "\uFAB3"], /* CJK COMPATIBILITY IDEOGRAPH-FAB3 */
  ["\uFAB4", "\uFAB4"], /* CJK COMPATIBILITY IDEOGRAPH-FAB4 */
  ["\uFAB5", "\uFAB5"], /* CJK COMPATIBILITY IDEOGRAPH-FAB5 */
  ["\uFAB6", "\uFAB6"], /* CJK COMPATIBILITY IDEOGRAPH-FAB6 */
  ["\uFAB7", "\uFAB7"], /* CJK COMPATIBILITY IDEOGRAPH-FAB7 */
  ["\uFAB8", "\uFAB8"], /* CJK COMPATIBILITY IDEOGRAPH-FAB8 */
  ["\uFAB9", "\uFAB9"], /* CJK COMPATIBILITY IDEOGRAPH-FAB9 */
  ["\uFABA", "\uFABA"], /* CJK COMPATIBILITY IDEOGRAPH-FABA */
  ["\uFABB", "\uFABB"], /* CJK COMPATIBILITY IDEOGRAPH-FABB */
  ["\uFABC", "\uFABC"], /* CJK COMPATIBILITY IDEOGRAPH-FABC */
  ["\uFABD", "\uFABD"], /* CJK COMPATIBILITY IDEOGRAPH-FABD */
  ["\uFABE", "\uFABE"], /* CJK COMPATIBILITY IDEOGRAPH-FABE */
  ["\uFABF", "\uFABF"], /* CJK COMPATIBILITY IDEOGRAPH-FABF */
  ["\uFAC0", "\uFAC0"], /* CJK COMPATIBILITY IDEOGRAPH-FAC0 */
  ["\uFAC1", "\uFAC1"], /* CJK COMPATIBILITY IDEOGRAPH-FAC1 */
  ["\uFAC2", "\uFAC2"], /* CJK COMPATIBILITY IDEOGRAPH-FAC2 */
  ["\uFAC3", "\uFAC3"], /* CJK COMPATIBILITY IDEOGRAPH-FAC3 */
  ["\uFAC4", "\uFAC4"], /* CJK COMPATIBILITY IDEOGRAPH-FAC4 */
  ["\uFAC5", "\uFAC5"], /* CJK COMPATIBILITY IDEOGRAPH-FAC5 */
  ["\uFAC6", "\uFAC6"], /* CJK COMPATIBILITY IDEOGRAPH-FAC6 */
  ["\uFAC7", "\uFAC7"], /* CJK COMPATIBILITY IDEOGRAPH-FAC7 */
  ["\uFAC8", "\uFAC8"], /* CJK COMPATIBILITY IDEOGRAPH-FAC8 */
  ["\uFAC9", "\uFAC9"], /* CJK COMPATIBILITY IDEOGRAPH-FAC9 */
  ["\uFACA", "\uFACA"], /* CJK COMPATIBILITY IDEOGRAPH-FACA */
  ["\uFACB", "\uFACB"], /* CJK COMPATIBILITY IDEOGRAPH-FACB */
  ["\uFACC", "\uFACC"], /* CJK COMPATIBILITY IDEOGRAPH-FACC */
  ["\uFACD", "\uFACD"], /* CJK COMPATIBILITY IDEOGRAPH-FACD */
  ["\uFACE", "\uFACE"], /* CJK COMPATIBILITY IDEOGRAPH-FACE */
  ["\uFACF", "\uFACF"], /* CJK COMPATIBILITY IDEOGRAPH-FACF */
  ["\uFAD0", "\uFAD0"], /* CJK COMPATIBILITY IDEOGRAPH-FAD0 */
  ["\uFAD1", "\uFAD1"], /* CJK COMPATIBILITY IDEOGRAPH-FAD1 */
  ["\uFAD2", "\uFAD2"], /* CJK COMPATIBILITY IDEOGRAPH-FAD2 */
  ["\uFAD3", "\uFAD3"], /* CJK COMPATIBILITY IDEOGRAPH-FAD3 */
  ["\uFAD4", "\uFAD4"], /* CJK COMPATIBILITY IDEOGRAPH-FAD4 */
  ["\uFAD5", "\uFAD5"], /* CJK COMPATIBILITY IDEOGRAPH-FAD5 */
  ["\uFAD6", "\uFAD6"], /* CJK COMPATIBILITY IDEOGRAPH-FAD6 */
  ["\uFAD7", "\uFAD7"], /* CJK COMPATIBILITY IDEOGRAPH-FAD7 */
  ["\uFAD8", "\uFAD8"], /* CJK COMPATIBILITY IDEOGRAPH-FAD8 */
  ["\uFAD9", "\uFAD9"], /* CJK COMPATIBILITY IDEOGRAPH-FAD9 */
  ["\uFADA", "\uFADA"],
  ["\uFADB", "\uFADB"],
  ["\uFADC", "\uFADC"],
  ["\uFADD", "\uFADD"],
  ["\uFADE", "\uFADE"],
  ["\uFADF", "\uFADF"],
  ["\uFAE0", "\uFAE0"],
  ["\uFAE1", "\uFAE1"],
  ["\uFAE2", "\uFAE2"],
  ["\uFAE3", "\uFAE3"],
  ["\uFAE4", "\uFAE4"],
  ["\uFAE5", "\uFAE5"],
  ["\uFAE6", "\uFAE6"],
  ["\uFAE7", "\uFAE7"],
  ["\uFAE8", "\uFAE8"],
  ["\uFAE9", "\uFAE9"],
  ["\uFAEA", "\uFAEA"],
  ["\uFAEB", "\uFAEB"],
  ["\uFAEC", "\uFAEC"],
  ["\uFAED", "\uFAED"],
  ["\uFAEE", "\uFAEE"],
  ["\uFAEF", "\uFAEF"],
  ["\uFAF0", "\uFAF0"],
  ["\uFAF1", "\uFAF1"],
  ["\uFAF2", "\uFAF2"],
  ["\uFAF3", "\uFAF3"],
  ["\uFAF4", "\uFAF4"],
  ["\uFAF5", "\uFAF5"],
  ["\uFAF6", "\uFAF6"],
  ["\uFAF7", "\uFAF7"],
  ["\uFAF8", "\uFAF8"],
  ["\uFAF9", "\uFAF9"],
  ["\uFAFA", "\uFAFA"],
  ["\uFAFB", "\uFAFB"],
  ["\uFAFC", "\uFAFC"],
  ["\uFAFD", "\uFAFD"],
  ["\uFAFE", "\uFAFE"],
  ["\uFAFF", "\uFAFF"],
  ["\u0046\u0046", "\uFB00"], /* LATIN SMALL LIGATURE FF */
  ["\u0046\u0049", "\uFB01"], /* LATIN SMALL LIGATURE FI */
  ["\u0046\u004C", "\uFB02"], /* LATIN SMALL LIGATURE FL */
  ["\u0046\u0046\u0049", "\uFB03"], /* LATIN SMALL LIGATURE FFI */
  ["\u0046\u0046\u004C", "\uFB04"], /* LATIN SMALL LIGATURE FFL */
  ["\u0053\u0054", "\uFB05"], /* LATIN SMALL LIGATURE LONG S T */
  ["\u0053\u0054", "\uFB06"], /* LATIN SMALL LIGATURE ST */
  ["\uFB07", "\uFB07"],
  ["\uFB08", "\uFB08"],
  ["\uFB09", "\uFB09"],
  ["\uFB0A", "\uFB0A"],
  ["\uFB0B", "\uFB0B"],
  ["\uFB0C", "\uFB0C"],
  ["\uFB0D", "\uFB0D"],
  ["\uFB0E", "\uFB0E"],
  ["\uFB0F", "\uFB0F"],
  ["\uFB10", "\uFB10"],
  ["\uFB11", "\uFB11"],
  ["\uFB12", "\uFB12"],
  ["\u0544\u0546", "\uFB13"], /* ARMENIAN SMALL LIGATURE MEN NOW */
  ["\u0544\u0535", "\uFB14"], /* ARMENIAN SMALL LIGATURE MEN ECH */
  ["\u0544\u053B", "\uFB15"], /* ARMENIAN SMALL LIGATURE MEN INI */
  ["\u054E\u0546", "\uFB16"], /* ARMENIAN SMALL LIGATURE VEW NOW */
  ["\u0544\u053D", "\uFB17"], /* ARMENIAN SMALL LIGATURE MEN XEH */
  ["\uFB18", "\uFB18"],
  ["\uFB19", "\uFB19"],
  ["\uFB1A", "\uFB1A"],
  ["\uFB1B", "\uFB1B"],
  ["\uFB1C", "\uFB1C"],
  ["\uFB1D", "\uFB1D"], /* HEBREW LETTER YOD WITH HIRIQ */
  ["\uFB1E", "\uFB1E"], /* HEBREW POINT JUDEO-SPANISH VARIKA (HEBREW POINT VARIKA) */
  ["\uFB1F", "\uFB1F"], /* HEBREW LIGATURE YIDDISH YOD YOD PATAH */
  ["\uFB20", "\uFB20"], /* HEBREW LETTER ALTERNATIVE AYIN */
  ["\uFB21", "\uFB21"], /* HEBREW LETTER WIDE ALEF */
  ["\uFB22", "\uFB22"], /* HEBREW LETTER WIDE DALET */
  ["\uFB23", "\uFB23"], /* HEBREW LETTER WIDE HE */
  ["\uFB24", "\uFB24"], /* HEBREW LETTER WIDE KAF */
  ["\uFB25", "\uFB25"], /* HEBREW LETTER WIDE LAMED */
  ["\uFB26", "\uFB26"], /* HEBREW LETTER WIDE FINAL MEM */
  ["\uFB27", "\uFB27"], /* HEBREW LETTER WIDE RESH */
  ["\uFB28", "\uFB28"], /* HEBREW LETTER WIDE TAV */
  ["\uFB29", "\uFB29"], /* HEBREW LETTER ALTERNATIVE PLUS SIGN */
  ["\uFB2A", "\uFB2A"], /* HEBREW LETTER SHIN WITH SHIN DOT */
  ["\uFB2B", "\uFB2B"], /* HEBREW LETTER SHIN WITH SIN DOT */
  ["\uFB2C", "\uFB2C"], /* HEBREW LETTER SHIN WITH DAGESH AND SHIN DOT */
  ["\uFB2D", "\uFB2D"], /* HEBREW LETTER SHIN WITH DAGESH AND SIN DOT */
  ["\uFB2E", "\uFB2E"], /* HEBREW LETTER ALEF WITH PATAH */
  ["\uFB2F", "\uFB2F"], /* HEBREW LETTER ALEF WITH QAMATS */
  ["\uFB30", "\uFB30"], /* HEBREW LETTER ALEF WITH MAPIQ */
  ["\uFB31", "\uFB31"], /* HEBREW LETTER BET WITH DAGESH */
  ["\uFB32", "\uFB32"], /* HEBREW LETTER GIMEL WITH DAGESH */
  ["\uFB33", "\uFB33"], /* HEBREW LETTER DALET WITH DAGESH */
  ["\uFB34", "\uFB34"], /* HEBREW LETTER HE WITH MAPIQ */
  ["\uFB35", "\uFB35"], /* HEBREW LETTER VAV WITH DAGESH */
  ["\uFB36", "\uFB36"], /* HEBREW LETTER ZAYIN WITH DAGESH */
  ["\uFB37", "\uFB37"],
  ["\uFB38", "\uFB38"], /* HEBREW LETTER TET WITH DAGESH */
  ["\uFB39", "\uFB39"], /* HEBREW LETTER YOD WITH DAGESH */
  ["\uFB3A", "\uFB3A"], /* HEBREW LETTER FINAL KAF WITH DAGESH */
  ["\uFB3B", "\uFB3B"], /* HEBREW LETTER KAF WITH DAGESH */
  ["\uFB3C", "\uFB3C"], /* HEBREW LETTER LAMED WITH DAGESH */
  ["\uFB3D", "\uFB3D"],
  ["\uFB3E", "\uFB3E"], /* HEBREW LETTER MEM WITH DAGESH */
  ["\uFB3F", "\uFB3F"],
  ["\uFB40", "\uFB40"], /* HEBREW LETTER NUN WITH DAGESH */
  ["\uFB41", "\uFB41"], /* HEBREW LETTER SAMEKH WITH DAGESH */
  ["\uFB42", "\uFB42"],
  ["\uFB43", "\uFB43"], /* HEBREW LETTER FINAL PE WITH DAGESH */
  ["\uFB44", "\uFB44"], /* HEBREW LETTER PE WITH DAGESH */
  ["\uFB45", "\uFB45"],
  ["\uFB46", "\uFB46"], /* HEBREW LETTER TSADI WITH DAGESH */
  ["\uFB47", "\uFB47"], /* HEBREW LETTER QOF WITH DAGESH */
  ["\uFB48", "\uFB48"], /* HEBREW LETTER RESH WITH DAGESH */
  ["\uFB49", "\uFB49"], /* HEBREW LETTER SHIN WITH DAGESH */
  ["\uFB4A", "\uFB4A"], /* HEBREW LETTER TAV WITH DAGESH */
  ["\uFB4B", "\uFB4B"], /* HEBREW LETTER VAV WITH HOLAM */
  ["\uFB4C", "\uFB4C"], /* HEBREW LETTER BET WITH RAFE */
  ["\uFB4D", "\uFB4D"], /* HEBREW LETTER KAF WITH RAFE */
  ["\uFB4E", "\uFB4E"], /* HEBREW LETTER PE WITH RAFE */
  ["\uFB4F", "\uFB4F"], /* HEBREW LIGATURE ALEF LAMED */
  ["\uFB50", "\uFB50"], /* ARABIC LETTER ALEF WASLA ISOLATED FORM */
  ["\uFB51", "\uFB51"], /* ARABIC LETTER ALEF WASLA FINAL FORM */
  ["\uFB52", "\uFB52"], /* ARABIC LETTER BEEH ISOLATED FORM */
  ["\uFB53", "\uFB53"], /* ARABIC LETTER BEEH FINAL FORM */
  ["\uFB54", "\uFB54"], /* ARABIC LETTER BEEH INITIAL FORM */
  ["\uFB55", "\uFB55"], /* ARABIC LETTER BEEH MEDIAL FORM */
  ["\uFB56", "\uFB56"], /* ARABIC LETTER PEH ISOLATED FORM */
  ["\uFB57", "\uFB57"], /* ARABIC LETTER PEH FINAL FORM */
  ["\uFB58", "\uFB58"], /* ARABIC LETTER PEH INITIAL FORM */
  ["\uFB59", "\uFB59"], /* ARABIC LETTER PEH MEDIAL FORM */
  ["\uFB5A", "\uFB5A"], /* ARABIC LETTER BEHEH ISOLATED FORM */
  ["\uFB5B", "\uFB5B"], /* ARABIC LETTER BEHEH FINAL FORM */
  ["\uFB5C", "\uFB5C"], /* ARABIC LETTER BEHEH INITIAL FORM */
  ["\uFB5D", "\uFB5D"], /* ARABIC LETTER BEHEH MEDIAL FORM */
  ["\uFB5E", "\uFB5E"], /* ARABIC LETTER TTEHEH ISOLATED FORM */
  ["\uFB5F", "\uFB5F"], /* ARABIC LETTER TTEHEH FINAL FORM */
  ["\uFB60", "\uFB60"], /* ARABIC LETTER TTEHEH INITIAL FORM */
  ["\uFB61", "\uFB61"], /* ARABIC LETTER TTEHEH MEDIAL FORM */
  ["\uFB62", "\uFB62"], /* ARABIC LETTER TEHEH ISOLATED FORM */
  ["\uFB63", "\uFB63"], /* ARABIC LETTER TEHEH FINAL FORM */
  ["\uFB64", "\uFB64"], /* ARABIC LETTER TEHEH INITIAL FORM */
  ["\uFB65", "\uFB65"], /* ARABIC LETTER TEHEH MEDIAL FORM */
  ["\uFB66", "\uFB66"], /* ARABIC LETTER TTEH ISOLATED FORM */
  ["\uFB67", "\uFB67"], /* ARABIC LETTER TTEH FINAL FORM */
  ["\uFB68", "\uFB68"], /* ARABIC LETTER TTEH INITIAL FORM */
  ["\uFB69", "\uFB69"], /* ARABIC LETTER TTEH MEDIAL FORM */
  ["\uFB6A", "\uFB6A"], /* ARABIC LETTER VEH ISOLATED FORM */
  ["\uFB6B", "\uFB6B"], /* ARABIC LETTER VEH FINAL FORM */
  ["\uFB6C", "\uFB6C"], /* ARABIC LETTER VEH INITIAL FORM */
  ["\uFB6D", "\uFB6D"], /* ARABIC LETTER VEH MEDIAL FORM */
  ["\uFB6E", "\uFB6E"], /* ARABIC LETTER PEHEH ISOLATED FORM */
  ["\uFB6F", "\uFB6F"], /* ARABIC LETTER PEHEH FINAL FORM */
  ["\uFB70", "\uFB70"], /* ARABIC LETTER PEHEH INITIAL FORM */
  ["\uFB71", "\uFB71"], /* ARABIC LETTER PEHEH MEDIAL FORM */
  ["\uFB72", "\uFB72"], /* ARABIC LETTER DYEH ISOLATED FORM */
  ["\uFB73", "\uFB73"], /* ARABIC LETTER DYEH FINAL FORM */
  ["\uFB74", "\uFB74"], /* ARABIC LETTER DYEH INITIAL FORM */
  ["\uFB75", "\uFB75"], /* ARABIC LETTER DYEH MEDIAL FORM */
  ["\uFB76", "\uFB76"], /* ARABIC LETTER NYEH ISOLATED FORM */
  ["\uFB77", "\uFB77"], /* ARABIC LETTER NYEH FINAL FORM */
  ["\uFB78", "\uFB78"], /* ARABIC LETTER NYEH INITIAL FORM */
  ["\uFB79", "\uFB79"], /* ARABIC LETTER NYEH MEDIAL FORM */
  ["\uFB7A", "\uFB7A"], /* ARABIC LETTER TCHEH ISOLATED FORM */
  ["\uFB7B", "\uFB7B"], /* ARABIC LETTER TCHEH FINAL FORM */
  ["\uFB7C", "\uFB7C"], /* ARABIC LETTER TCHEH INITIAL FORM */
  ["\uFB7D", "\uFB7D"], /* ARABIC LETTER TCHEH MEDIAL FORM */
  ["\uFB7E", "\uFB7E"], /* ARABIC LETTER TCHEHEH ISOLATED FORM */
  ["\uFB7F", "\uFB7F"], /* ARABIC LETTER TCHEHEH FINAL FORM */
  ["\uFB80", "\uFB80"], /* ARABIC LETTER TCHEHEH INITIAL FORM */
  ["\uFB81", "\uFB81"], /* ARABIC LETTER TCHEHEH MEDIAL FORM */
  ["\uFB82", "\uFB82"], /* ARABIC LETTER DDAHAL ISOLATED FORM */
  ["\uFB83", "\uFB83"], /* ARABIC LETTER DDAHAL FINAL FORM */
  ["\uFB84", "\uFB84"], /* ARABIC LETTER DAHAL ISOLATED FORM */
  ["\uFB85", "\uFB85"], /* ARABIC LETTER DAHAL FINAL FORM */
  ["\uFB86", "\uFB86"], /* ARABIC LETTER DUL ISOLATED FORM */
  ["\uFB87", "\uFB87"], /* ARABIC LETTER DUL FINAL FORM */
  ["\uFB88", "\uFB88"], /* ARABIC LETTER DDAL ISOLATED FORM */
  ["\uFB89", "\uFB89"], /* ARABIC LETTER DDAL FINAL FORM */
  ["\uFB8A", "\uFB8A"], /* ARABIC LETTER JEH ISOLATED FORM */
  ["\uFB8B", "\uFB8B"], /* ARABIC LETTER JEH FINAL FORM */
  ["\uFB8C", "\uFB8C"], /* ARABIC LETTER RREH ISOLATED FORM */
  ["\uFB8D", "\uFB8D"], /* ARABIC LETTER RREH FINAL FORM */
  ["\uFB8E", "\uFB8E"], /* ARABIC LETTER KEHEH ISOLATED FORM */
  ["\uFB8F", "\uFB8F"], /* ARABIC LETTER KEHEH FINAL FORM */
  ["\uFB90", "\uFB90"], /* ARABIC LETTER KEHEH INITIAL FORM */
  ["\uFB91", "\uFB91"], /* ARABIC LETTER KEHEH MEDIAL FORM */
  ["\uFB92", "\uFB92"], /* ARABIC LETTER GAF ISOLATED FORM */
  ["\uFB93", "\uFB93"], /* ARABIC LETTER GAF FINAL FORM */
  ["\uFB94", "\uFB94"], /* ARABIC LETTER GAF INITIAL FORM */
  ["\uFB95", "\uFB95"], /* ARABIC LETTER GAF MEDIAL FORM */
  ["\uFB96", "\uFB96"], /* ARABIC LETTER GUEH ISOLATED FORM */
  ["\uFB97", "\uFB97"], /* ARABIC LETTER GUEH FINAL FORM */
  ["\uFB98", "\uFB98"], /* ARABIC LETTER GUEH INITIAL FORM */
  ["\uFB99", "\uFB99"], /* ARABIC LETTER GUEH MEDIAL FORM */
  ["\uFB9A", "\uFB9A"], /* ARABIC LETTER NGOEH ISOLATED FORM */
  ["\uFB9B", "\uFB9B"], /* ARABIC LETTER NGOEH FINAL FORM */
  ["\uFB9C", "\uFB9C"], /* ARABIC LETTER NGOEH INITIAL FORM */
  ["\uFB9D", "\uFB9D"], /* ARABIC LETTER NGOEH MEDIAL FORM */
  ["\uFB9E", "\uFB9E"], /* ARABIC LETTER NOON GHUNNA ISOLATED FORM */
  ["\uFB9F", "\uFB9F"], /* ARABIC LETTER NOON GHUNNA FINAL FORM */
  ["\uFBA0", "\uFBA0"], /* ARABIC LETTER RNOON ISOLATED FORM */
  ["\uFBA1", "\uFBA1"], /* ARABIC LETTER RNOON FINAL FORM */
  ["\uFBA2", "\uFBA2"], /* ARABIC LETTER RNOON INITIAL FORM */
  ["\uFBA3", "\uFBA3"], /* ARABIC LETTER RNOON MEDIAL FORM */
  ["\uFBA4", "\uFBA4"], /* ARABIC LETTER HEH WITH YEH ABOVE ISOLATED FORM */
  ["\uFBA5", "\uFBA5"], /* ARABIC LETTER HEH WITH YEH ABOVE FINAL FORM */
  ["\uFBA6", "\uFBA6"], /* ARABIC LETTER HEH GOAL ISOLATED FORM */
  ["\uFBA7", "\uFBA7"], /* ARABIC LETTER HEH GOAL FINAL FORM */
  ["\uFBA8", "\uFBA8"], /* ARABIC LETTER HEH GOAL INITIAL FORM */
  ["\uFBA9", "\uFBA9"], /* ARABIC LETTER HEH GOAL MEDIAL FORM */
  ["\uFBAA", "\uFBAA"], /* ARABIC LETTER HEH DOACHASHMEE ISOLATED FORM */
  ["\uFBAB", "\uFBAB"], /* ARABIC LETTER HEH DOACHASHMEE FINAL FORM */
  ["\uFBAC", "\uFBAC"], /* ARABIC LETTER HEH DOACHASHMEE INITIAL FORM */
  ["\uFBAD", "\uFBAD"], /* ARABIC LETTER HEH DOACHASHMEE MEDIAL FORM */
  ["\uFBAE", "\uFBAE"], /* ARABIC LETTER YEH BARREE ISOLATED FORM */
  ["\uFBAF", "\uFBAF"], /* ARABIC LETTER YEH BARREE FINAL FORM */
  ["\uFBB0", "\uFBB0"], /* ARABIC LETTER YEH BARREE WITH HAMZA ABOVE ISOLATED FORM */
  ["\uFBB1", "\uFBB1"], /* ARABIC LETTER YEH BARREE WITH HAMZA ABOVE FINAL FORM */
  ["\uFBB2", "\uFBB2"], /* ARABIC SYMBOL DOT ABOVE */
  ["\uFBB3", "\uFBB3"], /* ARABIC SYMBOL DOT BELOW */
  ["\uFBB4", "\uFBB4"], /* ARABIC SYMBOL TWO DOTS ABOVE */
  ["\uFBB5", "\uFBB5"], /* ARABIC SYMBOL TWO DOTS BELOW */
  ["\uFBB6", "\uFBB6"], /* ARABIC SYMBOL THREE DOTS ABOVE */
  ["\uFBB7", "\uFBB7"], /* ARABIC SYMBOL THREE DOTS BELOW */
  ["\uFBB8", "\uFBB8"], /* ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS ABOVE */
  ["\uFBB9", "\uFBB9"], /* ARABIC SYMBOL THREE DOTS POINTING DOWNWARDS BELOW */
  ["\uFBBA", "\uFBBA"], /* ARABIC SYMBOL FOUR DOTS ABOVE */
  ["\uFBBB", "\uFBBB"], /* ARABIC SYMBOL FOUR DOTS BELOW */
  ["\uFBBC", "\uFBBC"], /* ARABIC SYMBOL DOUBLE VERTICAL BAR BELOW */
  ["\uFBBD", "\uFBBD"], /* ARABIC SYMBOL TWO DOTS VERTICALLY ABOVE */
  ["\uFBBE", "\uFBBE"], /* ARABIC SYMBOL TWO DOTS VERTICALLY BELOW */
  ["\uFBBF", "\uFBBF"], /* ARABIC SYMBOL RING */
  ["\uFBC0", "\uFBC0"], /* ARABIC SYMBOL SMALL TAH ABOVE */
  ["\uFBC1", "\uFBC1"], /* ARABIC SYMBOL SMALL TAH BELOW */
  ["\uFBC2", "\uFBC2"],
  ["\uFBC3", "\uFBC3"],
  ["\uFBC4", "\uFBC4"],
  ["\uFBC5", "\uFBC5"],
  ["\uFBC6", "\uFBC6"],
  ["\uFBC7", "\uFBC7"],
  ["\uFBC8", "\uFBC8"],
  ["\uFBC9", "\uFBC9"],
  ["\uFBCA", "\uFBCA"],
  ["\uFBCB", "\uFBCB"],
  ["\uFBCC", "\uFBCC"],
  ["\uFBCD", "\uFBCD"],
  ["\uFBCE", "\uFBCE"],
  ["\uFBCF", "\uFBCF"],
  ["\uFBD0", "\uFBD0"],
  ["\uFBD1", "\uFBD1"],
  ["\uFBD2", "\uFBD2"],
  ["\uFBD3", "\uFBD3"], /* ARABIC LETTER NG ISOLATED FORM */
  ["\uFBD4", "\uFBD4"], /* ARABIC LETTER NG FINAL FORM */
  ["\uFBD5", "\uFBD5"], /* ARABIC LETTER NG INITIAL FORM */
  ["\uFBD6", "\uFBD6"], /* ARABIC LETTER NG MEDIAL FORM */
  ["\uFBD7", "\uFBD7"], /* ARABIC LETTER U ISOLATED FORM */
  ["\uFBD8", "\uFBD8"], /* ARABIC LETTER U FINAL FORM */
  ["\uFBD9", "\uFBD9"], /* ARABIC LETTER OE ISOLATED FORM */
  ["\uFBDA", "\uFBDA"], /* ARABIC LETTER OE FINAL FORM */
  ["\uFBDB", "\uFBDB"], /* ARABIC LETTER YU ISOLATED FORM */
  ["\uFBDC", "\uFBDC"], /* ARABIC LETTER YU FINAL FORM */
  ["\uFBDD", "\uFBDD"], /* ARABIC LETTER U WITH HAMZA ABOVE ISOLATED FORM */
  ["\uFBDE", "\uFBDE"], /* ARABIC LETTER VE ISOLATED FORM */
  ["\uFBDF", "\uFBDF"], /* ARABIC LETTER VE FINAL FORM */
  ["\uFBE0", "\uFBE0"], /* ARABIC LETTER KIRGHIZ OE ISOLATED FORM */
  ["\uFBE1", "\uFBE1"], /* ARABIC LETTER KIRGHIZ OE FINAL FORM */
  ["\uFBE2", "\uFBE2"], /* ARABIC LETTER KIRGHIZ YU ISOLATED FORM */
  ["\uFBE3", "\uFBE3"], /* ARABIC LETTER KIRGHIZ YU FINAL FORM */
  ["\uFBE4", "\uFBE4"], /* ARABIC LETTER E ISOLATED FORM */
  ["\uFBE5", "\uFBE5"], /* ARABIC LETTER E FINAL FORM */
  ["\uFBE6", "\uFBE6"], /* ARABIC LETTER E INITIAL FORM */
  ["\uFBE7", "\uFBE7"], /* ARABIC LETTER E MEDIAL FORM */
  ["\uFBE8", "\uFBE8"], /* ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA INITIAL FORM */
  ["\uFBE9", "\uFBE9"], /* ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA MEDIAL FORM */
  ["\uFBEA", "\uFBEA"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF ISOLATED FORM */
  ["\uFBEB", "\uFBEB"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF FINAL FORM */
  ["\uFBEC", "\uFBEC"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE ISOLATED FORM */
  ["\uFBED", "\uFBED"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH AE FINAL FORM */
  ["\uFBEE", "\uFBEE"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW ISOLATED FORM */
  ["\uFBEF", "\uFBEF"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH WAW FINAL FORM */
  ["\uFBF0", "\uFBF0"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U ISOLATED FORM */
  ["\uFBF1", "\uFBF1"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH U FINAL FORM */
  ["\uFBF2", "\uFBF2"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE ISOLATED FORM */
  ["\uFBF3", "\uFBF3"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH OE FINAL FORM */
  ["\uFBF4", "\uFBF4"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU ISOLATED FORM */
  ["\uFBF5", "\uFBF5"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YU FINAL FORM */
  ["\uFBF6", "\uFBF6"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E ISOLATED FORM */
  ["\uFBF7", "\uFBF7"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E FINAL FORM */
  ["\uFBF8", "\uFBF8"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH E INITIAL FORM */
  ["\uFBF9", "\uFBF9"], /* ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFBFA", "\uFBFA"], /* ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM */
  ["\uFBFB", "\uFBFB"], /* ARABIC LIGATURE UIGHUR KIRGHIZ YEH WITH HAMZA ABOVE WITH ALEF MAKSURA INITIAL FORM */
  ["\uFBFC", "\uFBFC"], /* ARABIC LETTER FARSI YEH ISOLATED FORM */
  ["\uFBFD", "\uFBFD"], /* ARABIC LETTER FARSI YEH FINAL FORM */
  ["\uFBFE", "\uFBFE"], /* ARABIC LETTER FARSI YEH INITIAL FORM */
  ["\uFBFF", "\uFBFF"], /* ARABIC LETTER FARSI YEH MEDIAL FORM */
  ["\uFC00", "\uFC00"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM ISOLATED FORM */
  ["\uFC01", "\uFC01"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH ISOLATED FORM */
  ["\uFC02", "\uFC02"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM ISOLATED FORM */
  ["\uFC03", "\uFC03"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC04", "\uFC04"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH ISOLATED FORM */
  ["\uFC05", "\uFC05"], /* ARABIC LIGATURE BEH WITH JEEM ISOLATED FORM */
  ["\uFC06", "\uFC06"], /* ARABIC LIGATURE BEH WITH HAH ISOLATED FORM */
  ["\uFC07", "\uFC07"], /* ARABIC LIGATURE BEH WITH KHAH ISOLATED FORM */
  ["\uFC08", "\uFC08"], /* ARABIC LIGATURE BEH WITH MEEM ISOLATED FORM */
  ["\uFC09", "\uFC09"], /* ARABIC LIGATURE BEH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC0A", "\uFC0A"], /* ARABIC LIGATURE BEH WITH YEH ISOLATED FORM */
  ["\uFC0B", "\uFC0B"], /* ARABIC LIGATURE TEH WITH JEEM ISOLATED FORM */
  ["\uFC0C", "\uFC0C"], /* ARABIC LIGATURE TEH WITH HAH ISOLATED FORM */
  ["\uFC0D", "\uFC0D"], /* ARABIC LIGATURE TEH WITH KHAH ISOLATED FORM */
  ["\uFC0E", "\uFC0E"], /* ARABIC LIGATURE TEH WITH MEEM ISOLATED FORM */
  ["\uFC0F", "\uFC0F"], /* ARABIC LIGATURE TEH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC10", "\uFC10"], /* ARABIC LIGATURE TEH WITH YEH ISOLATED FORM */
  ["\uFC11", "\uFC11"], /* ARABIC LIGATURE THEH WITH JEEM ISOLATED FORM */
  ["\uFC12", "\uFC12"], /* ARABIC LIGATURE THEH WITH MEEM ISOLATED FORM */
  ["\uFC13", "\uFC13"], /* ARABIC LIGATURE THEH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC14", "\uFC14"], /* ARABIC LIGATURE THEH WITH YEH ISOLATED FORM */
  ["\uFC15", "\uFC15"], /* ARABIC LIGATURE JEEM WITH HAH ISOLATED FORM */
  ["\uFC16", "\uFC16"], /* ARABIC LIGATURE JEEM WITH MEEM ISOLATED FORM */
  ["\uFC17", "\uFC17"], /* ARABIC LIGATURE HAH WITH JEEM ISOLATED FORM */
  ["\uFC18", "\uFC18"], /* ARABIC LIGATURE HAH WITH MEEM ISOLATED FORM */
  ["\uFC19", "\uFC19"], /* ARABIC LIGATURE KHAH WITH JEEM ISOLATED FORM */
  ["\uFC1A", "\uFC1A"], /* ARABIC LIGATURE KHAH WITH HAH ISOLATED FORM */
  ["\uFC1B", "\uFC1B"], /* ARABIC LIGATURE KHAH WITH MEEM ISOLATED FORM */
  ["\uFC1C", "\uFC1C"], /* ARABIC LIGATURE SEEN WITH JEEM ISOLATED FORM */
  ["\uFC1D", "\uFC1D"], /* ARABIC LIGATURE SEEN WITH HAH ISOLATED FORM */
  ["\uFC1E", "\uFC1E"], /* ARABIC LIGATURE SEEN WITH KHAH ISOLATED FORM */
  ["\uFC1F", "\uFC1F"], /* ARABIC LIGATURE SEEN WITH MEEM ISOLATED FORM */
  ["\uFC20", "\uFC20"], /* ARABIC LIGATURE SAD WITH HAH ISOLATED FORM */
  ["\uFC21", "\uFC21"], /* ARABIC LIGATURE SAD WITH MEEM ISOLATED FORM */
  ["\uFC22", "\uFC22"], /* ARABIC LIGATURE DAD WITH JEEM ISOLATED FORM */
  ["\uFC23", "\uFC23"], /* ARABIC LIGATURE DAD WITH HAH ISOLATED FORM */
  ["\uFC24", "\uFC24"], /* ARABIC LIGATURE DAD WITH KHAH ISOLATED FORM */
  ["\uFC25", "\uFC25"], /* ARABIC LIGATURE DAD WITH MEEM ISOLATED FORM */
  ["\uFC26", "\uFC26"], /* ARABIC LIGATURE TAH WITH HAH ISOLATED FORM */
  ["\uFC27", "\uFC27"], /* ARABIC LIGATURE TAH WITH MEEM ISOLATED FORM */
  ["\uFC28", "\uFC28"], /* ARABIC LIGATURE ZAH WITH MEEM ISOLATED FORM */
  ["\uFC29", "\uFC29"], /* ARABIC LIGATURE AIN WITH JEEM ISOLATED FORM */
  ["\uFC2A", "\uFC2A"], /* ARABIC LIGATURE AIN WITH MEEM ISOLATED FORM */
  ["\uFC2B", "\uFC2B"], /* ARABIC LIGATURE GHAIN WITH JEEM ISOLATED FORM */
  ["\uFC2C", "\uFC2C"], /* ARABIC LIGATURE GHAIN WITH MEEM ISOLATED FORM */
  ["\uFC2D", "\uFC2D"], /* ARABIC LIGATURE FEH WITH JEEM ISOLATED FORM */
  ["\uFC2E", "\uFC2E"], /* ARABIC LIGATURE FEH WITH HAH ISOLATED FORM */
  ["\uFC2F", "\uFC2F"], /* ARABIC LIGATURE FEH WITH KHAH ISOLATED FORM */
  ["\uFC30", "\uFC30"], /* ARABIC LIGATURE FEH WITH MEEM ISOLATED FORM */
  ["\uFC31", "\uFC31"], /* ARABIC LIGATURE FEH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC32", "\uFC32"], /* ARABIC LIGATURE FEH WITH YEH ISOLATED FORM */
  ["\uFC33", "\uFC33"], /* ARABIC LIGATURE QAF WITH HAH ISOLATED FORM */
  ["\uFC34", "\uFC34"], /* ARABIC LIGATURE QAF WITH MEEM ISOLATED FORM */
  ["\uFC35", "\uFC35"], /* ARABIC LIGATURE QAF WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC36", "\uFC36"], /* ARABIC LIGATURE QAF WITH YEH ISOLATED FORM */
  ["\uFC37", "\uFC37"], /* ARABIC LIGATURE KAF WITH ALEF ISOLATED FORM */
  ["\uFC38", "\uFC38"], /* ARABIC LIGATURE KAF WITH JEEM ISOLATED FORM */
  ["\uFC39", "\uFC39"], /* ARABIC LIGATURE KAF WITH HAH ISOLATED FORM */
  ["\uFC3A", "\uFC3A"], /* ARABIC LIGATURE KAF WITH KHAH ISOLATED FORM */
  ["\uFC3B", "\uFC3B"], /* ARABIC LIGATURE KAF WITH LAM ISOLATED FORM */
  ["\uFC3C", "\uFC3C"], /* ARABIC LIGATURE KAF WITH MEEM ISOLATED FORM */
  ["\uFC3D", "\uFC3D"], /* ARABIC LIGATURE KAF WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC3E", "\uFC3E"], /* ARABIC LIGATURE KAF WITH YEH ISOLATED FORM */
  ["\uFC3F", "\uFC3F"], /* ARABIC LIGATURE LAM WITH JEEM ISOLATED FORM */
  ["\uFC40", "\uFC40"], /* ARABIC LIGATURE LAM WITH HAH ISOLATED FORM */
  ["\uFC41", "\uFC41"], /* ARABIC LIGATURE LAM WITH KHAH ISOLATED FORM */
  ["\uFC42", "\uFC42"], /* ARABIC LIGATURE LAM WITH MEEM ISOLATED FORM */
  ["\uFC43", "\uFC43"], /* ARABIC LIGATURE LAM WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC44", "\uFC44"], /* ARABIC LIGATURE LAM WITH YEH ISOLATED FORM */
  ["\uFC45", "\uFC45"], /* ARABIC LIGATURE MEEM WITH JEEM ISOLATED FORM */
  ["\uFC46", "\uFC46"], /* ARABIC LIGATURE MEEM WITH HAH ISOLATED FORM */
  ["\uFC47", "\uFC47"], /* ARABIC LIGATURE MEEM WITH KHAH ISOLATED FORM */
  ["\uFC48", "\uFC48"], /* ARABIC LIGATURE MEEM WITH MEEM ISOLATED FORM */
  ["\uFC49", "\uFC49"], /* ARABIC LIGATURE MEEM WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC4A", "\uFC4A"], /* ARABIC LIGATURE MEEM WITH YEH ISOLATED FORM */
  ["\uFC4B", "\uFC4B"], /* ARABIC LIGATURE NOON WITH JEEM ISOLATED FORM */
  ["\uFC4C", "\uFC4C"], /* ARABIC LIGATURE NOON WITH HAH ISOLATED FORM */
  ["\uFC4D", "\uFC4D"], /* ARABIC LIGATURE NOON WITH KHAH ISOLATED FORM */
  ["\uFC4E", "\uFC4E"], /* ARABIC LIGATURE NOON WITH MEEM ISOLATED FORM */
  ["\uFC4F", "\uFC4F"], /* ARABIC LIGATURE NOON WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC50", "\uFC50"], /* ARABIC LIGATURE NOON WITH YEH ISOLATED FORM */
  ["\uFC51", "\uFC51"], /* ARABIC LIGATURE HEH WITH JEEM ISOLATED FORM */
  ["\uFC52", "\uFC52"], /* ARABIC LIGATURE HEH WITH MEEM ISOLATED FORM */
  ["\uFC53", "\uFC53"], /* ARABIC LIGATURE HEH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC54", "\uFC54"], /* ARABIC LIGATURE HEH WITH YEH ISOLATED FORM */
  ["\uFC55", "\uFC55"], /* ARABIC LIGATURE YEH WITH JEEM ISOLATED FORM */
  ["\uFC56", "\uFC56"], /* ARABIC LIGATURE YEH WITH HAH ISOLATED FORM */
  ["\uFC57", "\uFC57"], /* ARABIC LIGATURE YEH WITH KHAH ISOLATED FORM */
  ["\uFC58", "\uFC58"], /* ARABIC LIGATURE YEH WITH MEEM ISOLATED FORM */
  ["\uFC59", "\uFC59"], /* ARABIC LIGATURE YEH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFC5A", "\uFC5A"], /* ARABIC LIGATURE YEH WITH YEH ISOLATED FORM */
  ["\uFC5B", "\uFC5B"], /* ARABIC LIGATURE THAL WITH SUPERSCRIPT ALEF ISOLATED FORM */
  ["\uFC5C", "\uFC5C"], /* ARABIC LIGATURE REH WITH SUPERSCRIPT ALEF ISOLATED FORM */
  ["\uFC5D", "\uFC5D"], /* ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF ISOLATED FORM */
  ["\uFC5E", "\uFC5E"], /* ARABIC LIGATURE SHADDA WITH DAMMATAN ISOLATED FORM */
  ["\uFC5F", "\uFC5F"], /* ARABIC LIGATURE SHADDA WITH KASRATAN ISOLATED FORM */
  ["\uFC60", "\uFC60"], /* ARABIC LIGATURE SHADDA WITH FATHA ISOLATED FORM */
  ["\uFC61", "\uFC61"], /* ARABIC LIGATURE SHADDA WITH DAMMA ISOLATED FORM */
  ["\uFC62", "\uFC62"], /* ARABIC LIGATURE SHADDA WITH KASRA ISOLATED FORM */
  ["\uFC63", "\uFC63"], /* ARABIC LIGATURE SHADDA WITH SUPERSCRIPT ALEF ISOLATED FORM */
  ["\uFC64", "\uFC64"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH REH FINAL FORM */
  ["\uFC65", "\uFC65"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ZAIN FINAL FORM */
  ["\uFC66", "\uFC66"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM FINAL FORM */
  ["\uFC67", "\uFC67"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH NOON FINAL FORM */
  ["\uFC68", "\uFC68"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH ALEF MAKSURA FINAL FORM */
  ["\uFC69", "\uFC69"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH YEH FINAL FORM */
  ["\uFC6A", "\uFC6A"], /* ARABIC LIGATURE BEH WITH REH FINAL FORM */
  ["\uFC6B", "\uFC6B"], /* ARABIC LIGATURE BEH WITH ZAIN FINAL FORM */
  ["\uFC6C", "\uFC6C"], /* ARABIC LIGATURE BEH WITH MEEM FINAL FORM */
  ["\uFC6D", "\uFC6D"], /* ARABIC LIGATURE BEH WITH NOON FINAL FORM */
  ["\uFC6E", "\uFC6E"], /* ARABIC LIGATURE BEH WITH ALEF MAKSURA FINAL FORM */
  ["\uFC6F", "\uFC6F"], /* ARABIC LIGATURE BEH WITH YEH FINAL FORM */
  ["\uFC70", "\uFC70"], /* ARABIC LIGATURE TEH WITH REH FINAL FORM */
  ["\uFC71", "\uFC71"], /* ARABIC LIGATURE TEH WITH ZAIN FINAL FORM */
  ["\uFC72", "\uFC72"], /* ARABIC LIGATURE TEH WITH MEEM FINAL FORM */
  ["\uFC73", "\uFC73"], /* ARABIC LIGATURE TEH WITH NOON FINAL FORM */
  ["\uFC74", "\uFC74"], /* ARABIC LIGATURE TEH WITH ALEF MAKSURA FINAL FORM */
  ["\uFC75", "\uFC75"], /* ARABIC LIGATURE TEH WITH YEH FINAL FORM */
  ["\uFC76", "\uFC76"], /* ARABIC LIGATURE THEH WITH REH FINAL FORM */
  ["\uFC77", "\uFC77"], /* ARABIC LIGATURE THEH WITH ZAIN FINAL FORM */
  ["\uFC78", "\uFC78"], /* ARABIC LIGATURE THEH WITH MEEM FINAL FORM */
  ["\uFC79", "\uFC79"], /* ARABIC LIGATURE THEH WITH NOON FINAL FORM */
  ["\uFC7A", "\uFC7A"], /* ARABIC LIGATURE THEH WITH ALEF MAKSURA FINAL FORM */
  ["\uFC7B", "\uFC7B"], /* ARABIC LIGATURE THEH WITH YEH FINAL FORM */
  ["\uFC7C", "\uFC7C"], /* ARABIC LIGATURE FEH WITH ALEF MAKSURA FINAL FORM */
  ["\uFC7D", "\uFC7D"], /* ARABIC LIGATURE FEH WITH YEH FINAL FORM */
  ["\uFC7E", "\uFC7E"], /* ARABIC LIGATURE QAF WITH ALEF MAKSURA FINAL FORM */
  ["\uFC7F", "\uFC7F"], /* ARABIC LIGATURE QAF WITH YEH FINAL FORM */
  ["\uFC80", "\uFC80"], /* ARABIC LIGATURE KAF WITH ALEF FINAL FORM */
  ["\uFC81", "\uFC81"], /* ARABIC LIGATURE KAF WITH LAM FINAL FORM */
  ["\uFC82", "\uFC82"], /* ARABIC LIGATURE KAF WITH MEEM FINAL FORM */
  ["\uFC83", "\uFC83"], /* ARABIC LIGATURE KAF WITH ALEF MAKSURA FINAL FORM */
  ["\uFC84", "\uFC84"], /* ARABIC LIGATURE KAF WITH YEH FINAL FORM */
  ["\uFC85", "\uFC85"], /* ARABIC LIGATURE LAM WITH MEEM FINAL FORM */
  ["\uFC86", "\uFC86"], /* ARABIC LIGATURE LAM WITH ALEF MAKSURA FINAL FORM */
  ["\uFC87", "\uFC87"], /* ARABIC LIGATURE LAM WITH YEH FINAL FORM */
  ["\uFC88", "\uFC88"], /* ARABIC LIGATURE MEEM WITH ALEF FINAL FORM */
  ["\uFC89", "\uFC89"], /* ARABIC LIGATURE MEEM WITH MEEM FINAL FORM */
  ["\uFC8A", "\uFC8A"], /* ARABIC LIGATURE NOON WITH REH FINAL FORM */
  ["\uFC8B", "\uFC8B"], /* ARABIC LIGATURE NOON WITH ZAIN FINAL FORM */
  ["\uFC8C", "\uFC8C"], /* ARABIC LIGATURE NOON WITH MEEM FINAL FORM */
  ["\uFC8D", "\uFC8D"], /* ARABIC LIGATURE NOON WITH NOON FINAL FORM */
  ["\uFC8E", "\uFC8E"], /* ARABIC LIGATURE NOON WITH ALEF MAKSURA FINAL FORM */
  ["\uFC8F", "\uFC8F"], /* ARABIC LIGATURE NOON WITH YEH FINAL FORM */
  ["\uFC90", "\uFC90"], /* ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF FINAL FORM */
  ["\uFC91", "\uFC91"], /* ARABIC LIGATURE YEH WITH REH FINAL FORM */
  ["\uFC92", "\uFC92"], /* ARABIC LIGATURE YEH WITH ZAIN FINAL FORM */
  ["\uFC93", "\uFC93"], /* ARABIC LIGATURE YEH WITH MEEM FINAL FORM */
  ["\uFC94", "\uFC94"], /* ARABIC LIGATURE YEH WITH NOON FINAL FORM */
  ["\uFC95", "\uFC95"], /* ARABIC LIGATURE YEH WITH ALEF MAKSURA FINAL FORM */
  ["\uFC96", "\uFC96"], /* ARABIC LIGATURE YEH WITH YEH FINAL FORM */
  ["\uFC97", "\uFC97"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH JEEM INITIAL FORM */
  ["\uFC98", "\uFC98"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HAH INITIAL FORM */
  ["\uFC99", "\uFC99"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH KHAH INITIAL FORM */
  ["\uFC9A", "\uFC9A"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM INITIAL FORM */
  ["\uFC9B", "\uFC9B"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH INITIAL FORM */
  ["\uFC9C", "\uFC9C"], /* ARABIC LIGATURE BEH WITH JEEM INITIAL FORM */
  ["\uFC9D", "\uFC9D"], /* ARABIC LIGATURE BEH WITH HAH INITIAL FORM */
  ["\uFC9E", "\uFC9E"], /* ARABIC LIGATURE BEH WITH KHAH INITIAL FORM */
  ["\uFC9F", "\uFC9F"], /* ARABIC LIGATURE BEH WITH MEEM INITIAL FORM */
  ["\uFCA0", "\uFCA0"], /* ARABIC LIGATURE BEH WITH HEH INITIAL FORM */
  ["\uFCA1", "\uFCA1"], /* ARABIC LIGATURE TEH WITH JEEM INITIAL FORM */
  ["\uFCA2", "\uFCA2"], /* ARABIC LIGATURE TEH WITH HAH INITIAL FORM */
  ["\uFCA3", "\uFCA3"], /* ARABIC LIGATURE TEH WITH KHAH INITIAL FORM */
  ["\uFCA4", "\uFCA4"], /* ARABIC LIGATURE TEH WITH MEEM INITIAL FORM */
  ["\uFCA5", "\uFCA5"], /* ARABIC LIGATURE TEH WITH HEH INITIAL FORM */
  ["\uFCA6", "\uFCA6"], /* ARABIC LIGATURE THEH WITH MEEM INITIAL FORM */
  ["\uFCA7", "\uFCA7"], /* ARABIC LIGATURE JEEM WITH HAH INITIAL FORM */
  ["\uFCA8", "\uFCA8"], /* ARABIC LIGATURE JEEM WITH MEEM INITIAL FORM */
  ["\uFCA9", "\uFCA9"], /* ARABIC LIGATURE HAH WITH JEEM INITIAL FORM */
  ["\uFCAA", "\uFCAA"], /* ARABIC LIGATURE HAH WITH MEEM INITIAL FORM */
  ["\uFCAB", "\uFCAB"], /* ARABIC LIGATURE KHAH WITH JEEM INITIAL FORM */
  ["\uFCAC", "\uFCAC"], /* ARABIC LIGATURE KHAH WITH MEEM INITIAL FORM */
  ["\uFCAD", "\uFCAD"], /* ARABIC LIGATURE SEEN WITH JEEM INITIAL FORM */
  ["\uFCAE", "\uFCAE"], /* ARABIC LIGATURE SEEN WITH HAH INITIAL FORM */
  ["\uFCAF", "\uFCAF"], /* ARABIC LIGATURE SEEN WITH KHAH INITIAL FORM */
  ["\uFCB0", "\uFCB0"], /* ARABIC LIGATURE SEEN WITH MEEM INITIAL FORM */
  ["\uFCB1", "\uFCB1"], /* ARABIC LIGATURE SAD WITH HAH INITIAL FORM */
  ["\uFCB2", "\uFCB2"], /* ARABIC LIGATURE SAD WITH KHAH INITIAL FORM */
  ["\uFCB3", "\uFCB3"], /* ARABIC LIGATURE SAD WITH MEEM INITIAL FORM */
  ["\uFCB4", "\uFCB4"], /* ARABIC LIGATURE DAD WITH JEEM INITIAL FORM */
  ["\uFCB5", "\uFCB5"], /* ARABIC LIGATURE DAD WITH HAH INITIAL FORM */
  ["\uFCB6", "\uFCB6"], /* ARABIC LIGATURE DAD WITH KHAH INITIAL FORM */
  ["\uFCB7", "\uFCB7"], /* ARABIC LIGATURE DAD WITH MEEM INITIAL FORM */
  ["\uFCB8", "\uFCB8"], /* ARABIC LIGATURE TAH WITH HAH INITIAL FORM */
  ["\uFCB9", "\uFCB9"], /* ARABIC LIGATURE ZAH WITH MEEM INITIAL FORM */
  ["\uFCBA", "\uFCBA"], /* ARABIC LIGATURE AIN WITH JEEM INITIAL FORM */
  ["\uFCBB", "\uFCBB"], /* ARABIC LIGATURE AIN WITH MEEM INITIAL FORM */
  ["\uFCBC", "\uFCBC"], /* ARABIC LIGATURE GHAIN WITH JEEM INITIAL FORM */
  ["\uFCBD", "\uFCBD"], /* ARABIC LIGATURE GHAIN WITH MEEM INITIAL FORM */
  ["\uFCBE", "\uFCBE"], /* ARABIC LIGATURE FEH WITH JEEM INITIAL FORM */
  ["\uFCBF", "\uFCBF"], /* ARABIC LIGATURE FEH WITH HAH INITIAL FORM */
  ["\uFCC0", "\uFCC0"], /* ARABIC LIGATURE FEH WITH KHAH INITIAL FORM */
  ["\uFCC1", "\uFCC1"], /* ARABIC LIGATURE FEH WITH MEEM INITIAL FORM */
  ["\uFCC2", "\uFCC2"], /* ARABIC LIGATURE QAF WITH HAH INITIAL FORM */
  ["\uFCC3", "\uFCC3"], /* ARABIC LIGATURE QAF WITH MEEM INITIAL FORM */
  ["\uFCC4", "\uFCC4"], /* ARABIC LIGATURE KAF WITH JEEM INITIAL FORM */
  ["\uFCC5", "\uFCC5"], /* ARABIC LIGATURE KAF WITH HAH INITIAL FORM */
  ["\uFCC6", "\uFCC6"], /* ARABIC LIGATURE KAF WITH KHAH INITIAL FORM */
  ["\uFCC7", "\uFCC7"], /* ARABIC LIGATURE KAF WITH LAM INITIAL FORM */
  ["\uFCC8", "\uFCC8"], /* ARABIC LIGATURE KAF WITH MEEM INITIAL FORM */
  ["\uFCC9", "\uFCC9"], /* ARABIC LIGATURE LAM WITH JEEM INITIAL FORM */
  ["\uFCCA", "\uFCCA"], /* ARABIC LIGATURE LAM WITH HAH INITIAL FORM */
  ["\uFCCB", "\uFCCB"], /* ARABIC LIGATURE LAM WITH KHAH INITIAL FORM */
  ["\uFCCC", "\uFCCC"], /* ARABIC LIGATURE LAM WITH MEEM INITIAL FORM */
  ["\uFCCD", "\uFCCD"], /* ARABIC LIGATURE LAM WITH HEH INITIAL FORM */
  ["\uFCCE", "\uFCCE"], /* ARABIC LIGATURE MEEM WITH JEEM INITIAL FORM */
  ["\uFCCF", "\uFCCF"], /* ARABIC LIGATURE MEEM WITH HAH INITIAL FORM */
  ["\uFCD0", "\uFCD0"], /* ARABIC LIGATURE MEEM WITH KHAH INITIAL FORM */
  ["\uFCD1", "\uFCD1"], /* ARABIC LIGATURE MEEM WITH MEEM INITIAL FORM */
  ["\uFCD2", "\uFCD2"], /* ARABIC LIGATURE NOON WITH JEEM INITIAL FORM */
  ["\uFCD3", "\uFCD3"], /* ARABIC LIGATURE NOON WITH HAH INITIAL FORM */
  ["\uFCD4", "\uFCD4"], /* ARABIC LIGATURE NOON WITH KHAH INITIAL FORM */
  ["\uFCD5", "\uFCD5"], /* ARABIC LIGATURE NOON WITH MEEM INITIAL FORM */
  ["\uFCD6", "\uFCD6"], /* ARABIC LIGATURE NOON WITH HEH INITIAL FORM */
  ["\uFCD7", "\uFCD7"], /* ARABIC LIGATURE HEH WITH JEEM INITIAL FORM */
  ["\uFCD8", "\uFCD8"], /* ARABIC LIGATURE HEH WITH MEEM INITIAL FORM */
  ["\uFCD9", "\uFCD9"], /* ARABIC LIGATURE HEH WITH SUPERSCRIPT ALEF INITIAL FORM */
  ["\uFCDA", "\uFCDA"], /* ARABIC LIGATURE YEH WITH JEEM INITIAL FORM */
  ["\uFCDB", "\uFCDB"], /* ARABIC LIGATURE YEH WITH HAH INITIAL FORM */
  ["\uFCDC", "\uFCDC"], /* ARABIC LIGATURE YEH WITH KHAH INITIAL FORM */
  ["\uFCDD", "\uFCDD"], /* ARABIC LIGATURE YEH WITH MEEM INITIAL FORM */
  ["\uFCDE", "\uFCDE"], /* ARABIC LIGATURE YEH WITH HEH INITIAL FORM */
  ["\uFCDF", "\uFCDF"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH MEEM MEDIAL FORM */
  ["\uFCE0", "\uFCE0"], /* ARABIC LIGATURE YEH WITH HAMZA ABOVE WITH HEH MEDIAL FORM */
  ["\uFCE1", "\uFCE1"], /* ARABIC LIGATURE BEH WITH MEEM MEDIAL FORM */
  ["\uFCE2", "\uFCE2"], /* ARABIC LIGATURE BEH WITH HEH MEDIAL FORM */
  ["\uFCE3", "\uFCE3"], /* ARABIC LIGATURE TEH WITH MEEM MEDIAL FORM */
  ["\uFCE4", "\uFCE4"], /* ARABIC LIGATURE TEH WITH HEH MEDIAL FORM */
  ["\uFCE5", "\uFCE5"], /* ARABIC LIGATURE THEH WITH MEEM MEDIAL FORM */
  ["\uFCE6", "\uFCE6"], /* ARABIC LIGATURE THEH WITH HEH MEDIAL FORM */
  ["\uFCE7", "\uFCE7"], /* ARABIC LIGATURE SEEN WITH MEEM MEDIAL FORM */
  ["\uFCE8", "\uFCE8"], /* ARABIC LIGATURE SEEN WITH HEH MEDIAL FORM */
  ["\uFCE9", "\uFCE9"], /* ARABIC LIGATURE SHEEN WITH MEEM MEDIAL FORM */
  ["\uFCEA", "\uFCEA"], /* ARABIC LIGATURE SHEEN WITH HEH MEDIAL FORM */
  ["\uFCEB", "\uFCEB"], /* ARABIC LIGATURE KAF WITH LAM MEDIAL FORM */
  ["\uFCEC", "\uFCEC"], /* ARABIC LIGATURE KAF WITH MEEM MEDIAL FORM */
  ["\uFCED", "\uFCED"], /* ARABIC LIGATURE LAM WITH MEEM MEDIAL FORM */
  ["\uFCEE", "\uFCEE"], /* ARABIC LIGATURE NOON WITH MEEM MEDIAL FORM */
  ["\uFCEF", "\uFCEF"], /* ARABIC LIGATURE NOON WITH HEH MEDIAL FORM */
  ["\uFCF0", "\uFCF0"], /* ARABIC LIGATURE YEH WITH MEEM MEDIAL FORM */
  ["\uFCF1", "\uFCF1"], /* ARABIC LIGATURE YEH WITH HEH MEDIAL FORM */
  ["\uFCF2", "\uFCF2"], /* ARABIC LIGATURE SHADDA WITH FATHA MEDIAL FORM */
  ["\uFCF3", "\uFCF3"], /* ARABIC LIGATURE SHADDA WITH DAMMA MEDIAL FORM */
  ["\uFCF4", "\uFCF4"], /* ARABIC LIGATURE SHADDA WITH KASRA MEDIAL FORM */
  ["\uFCF5", "\uFCF5"], /* ARABIC LIGATURE TAH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFCF6", "\uFCF6"], /* ARABIC LIGATURE TAH WITH YEH ISOLATED FORM */
  ["\uFCF7", "\uFCF7"], /* ARABIC LIGATURE AIN WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFCF8", "\uFCF8"], /* ARABIC LIGATURE AIN WITH YEH ISOLATED FORM */
  ["\uFCF9", "\uFCF9"], /* ARABIC LIGATURE GHAIN WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFCFA", "\uFCFA"], /* ARABIC LIGATURE GHAIN WITH YEH ISOLATED FORM */
  ["\uFCFB", "\uFCFB"], /* ARABIC LIGATURE SEEN WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFCFC", "\uFCFC"], /* ARABIC LIGATURE SEEN WITH YEH ISOLATED FORM */
  ["\uFCFD", "\uFCFD"], /* ARABIC LIGATURE SHEEN WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFCFE", "\uFCFE"], /* ARABIC LIGATURE SHEEN WITH YEH ISOLATED FORM */
  ["\uFCFF", "\uFCFF"], /* ARABIC LIGATURE HAH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFD00", "\uFD00"], /* ARABIC LIGATURE HAH WITH YEH ISOLATED FORM */
  ["\uFD01", "\uFD01"], /* ARABIC LIGATURE JEEM WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFD02", "\uFD02"], /* ARABIC LIGATURE JEEM WITH YEH ISOLATED FORM */
  ["\uFD03", "\uFD03"], /* ARABIC LIGATURE KHAH WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFD04", "\uFD04"], /* ARABIC LIGATURE KHAH WITH YEH ISOLATED FORM */
  ["\uFD05", "\uFD05"], /* ARABIC LIGATURE SAD WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFD06", "\uFD06"], /* ARABIC LIGATURE SAD WITH YEH ISOLATED FORM */
  ["\uFD07", "\uFD07"], /* ARABIC LIGATURE DAD WITH ALEF MAKSURA ISOLATED FORM */
  ["\uFD08", "\uFD08"], /* ARABIC LIGATURE DAD WITH YEH ISOLATED FORM */
  ["\uFD09", "\uFD09"], /* ARABIC LIGATURE SHEEN WITH JEEM ISOLATED FORM */
  ["\uFD0A", "\uFD0A"], /* ARABIC LIGATURE SHEEN WITH HAH ISOLATED FORM */
  ["\uFD0B", "\uFD0B"], /* ARABIC LIGATURE SHEEN WITH KHAH ISOLATED FORM */
  ["\uFD0C", "\uFD0C"], /* ARABIC LIGATURE SHEEN WITH MEEM ISOLATED FORM */
  ["\uFD0D", "\uFD0D"], /* ARABIC LIGATURE SHEEN WITH REH ISOLATED FORM */
  ["\uFD0E", "\uFD0E"], /* ARABIC LIGATURE SEEN WITH REH ISOLATED FORM */
  ["\uFD0F", "\uFD0F"], /* ARABIC LIGATURE SAD WITH REH ISOLATED FORM */
  ["\uFD10", "\uFD10"], /* ARABIC LIGATURE DAD WITH REH ISOLATED FORM */
  ["\uFD11", "\uFD11"], /* ARABIC LIGATURE TAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFD12", "\uFD12"], /* ARABIC LIGATURE TAH WITH YEH FINAL FORM */
  ["\uFD13", "\uFD13"], /* ARABIC LIGATURE AIN WITH ALEF MAKSURA FINAL FORM */
  ["\uFD14", "\uFD14"], /* ARABIC LIGATURE AIN WITH YEH FINAL FORM */
  ["\uFD15", "\uFD15"], /* ARABIC LIGATURE GHAIN WITH ALEF MAKSURA FINAL FORM */
  ["\uFD16", "\uFD16"], /* ARABIC LIGATURE GHAIN WITH YEH FINAL FORM */
  ["\uFD17", "\uFD17"], /* ARABIC LIGATURE SEEN WITH ALEF MAKSURA FINAL FORM */
  ["\uFD18", "\uFD18"], /* ARABIC LIGATURE SEEN WITH YEH FINAL FORM */
  ["\uFD19", "\uFD19"], /* ARABIC LIGATURE SHEEN WITH ALEF MAKSURA FINAL FORM */
  ["\uFD1A", "\uFD1A"], /* ARABIC LIGATURE SHEEN WITH YEH FINAL FORM */
  ["\uFD1B", "\uFD1B"], /* ARABIC LIGATURE HAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFD1C", "\uFD1C"], /* ARABIC LIGATURE HAH WITH YEH FINAL FORM */
  ["\uFD1D", "\uFD1D"], /* ARABIC LIGATURE JEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD1E", "\uFD1E"], /* ARABIC LIGATURE JEEM WITH YEH FINAL FORM */
  ["\uFD1F", "\uFD1F"], /* ARABIC LIGATURE KHAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFD20", "\uFD20"], /* ARABIC LIGATURE KHAH WITH YEH FINAL FORM */
  ["\uFD21", "\uFD21"], /* ARABIC LIGATURE SAD WITH ALEF MAKSURA FINAL FORM */
  ["\uFD22", "\uFD22"], /* ARABIC LIGATURE SAD WITH YEH FINAL FORM */
  ["\uFD23", "\uFD23"], /* ARABIC LIGATURE DAD WITH ALEF MAKSURA FINAL FORM */
  ["\uFD24", "\uFD24"], /* ARABIC LIGATURE DAD WITH YEH FINAL FORM */
  ["\uFD25", "\uFD25"], /* ARABIC LIGATURE SHEEN WITH JEEM FINAL FORM */
  ["\uFD26", "\uFD26"], /* ARABIC LIGATURE SHEEN WITH HAH FINAL FORM */
  ["\uFD27", "\uFD27"], /* ARABIC LIGATURE SHEEN WITH KHAH FINAL FORM */
  ["\uFD28", "\uFD28"], /* ARABIC LIGATURE SHEEN WITH MEEM FINAL FORM */
  ["\uFD29", "\uFD29"], /* ARABIC LIGATURE SHEEN WITH REH FINAL FORM */
  ["\uFD2A", "\uFD2A"], /* ARABIC LIGATURE SEEN WITH REH FINAL FORM */
  ["\uFD2B", "\uFD2B"], /* ARABIC LIGATURE SAD WITH REH FINAL FORM */
  ["\uFD2C", "\uFD2C"], /* ARABIC LIGATURE DAD WITH REH FINAL FORM */
  ["\uFD2D", "\uFD2D"], /* ARABIC LIGATURE SHEEN WITH JEEM INITIAL FORM */
  ["\uFD2E", "\uFD2E"], /* ARABIC LIGATURE SHEEN WITH HAH INITIAL FORM */
  ["\uFD2F", "\uFD2F"], /* ARABIC LIGATURE SHEEN WITH KHAH INITIAL FORM */
  ["\uFD30", "\uFD30"], /* ARABIC LIGATURE SHEEN WITH MEEM INITIAL FORM */
  ["\uFD31", "\uFD31"], /* ARABIC LIGATURE SEEN WITH HEH INITIAL FORM */
  ["\uFD32", "\uFD32"], /* ARABIC LIGATURE SHEEN WITH HEH INITIAL FORM */
  ["\uFD33", "\uFD33"], /* ARABIC LIGATURE TAH WITH MEEM INITIAL FORM */
  ["\uFD34", "\uFD34"], /* ARABIC LIGATURE SEEN WITH JEEM MEDIAL FORM */
  ["\uFD35", "\uFD35"], /* ARABIC LIGATURE SEEN WITH HAH MEDIAL FORM */
  ["\uFD36", "\uFD36"], /* ARABIC LIGATURE SEEN WITH KHAH MEDIAL FORM */
  ["\uFD37", "\uFD37"], /* ARABIC LIGATURE SHEEN WITH JEEM MEDIAL FORM */
  ["\uFD38", "\uFD38"], /* ARABIC LIGATURE SHEEN WITH HAH MEDIAL FORM */
  ["\uFD39", "\uFD39"], /* ARABIC LIGATURE SHEEN WITH KHAH MEDIAL FORM */
  ["\uFD3A", "\uFD3A"], /* ARABIC LIGATURE TAH WITH MEEM MEDIAL FORM */
  ["\uFD3B", "\uFD3B"], /* ARABIC LIGATURE ZAH WITH MEEM MEDIAL FORM */
  ["\uFD3C", "\uFD3C"], /* ARABIC LIGATURE ALEF WITH FATHATAN FINAL FORM */
  ["\uFD3D", "\uFD3D"], /* ARABIC LIGATURE ALEF WITH FATHATAN ISOLATED FORM */
  ["\uFD3E", "\uFD3E"], /* ORNATE LEFT PARENTHESIS */
  ["\uFD3F", "\uFD3F"], /* ORNATE RIGHT PARENTHESIS */
  ["\uFD40", "\uFD40"],
  ["\uFD41", "\uFD41"],
  ["\uFD42", "\uFD42"],
  ["\uFD43", "\uFD43"],
  ["\uFD44", "\uFD44"],
  ["\uFD45", "\uFD45"],
  ["\uFD46", "\uFD46"],
  ["\uFD47", "\uFD47"],
  ["\uFD48", "\uFD48"],
  ["\uFD49", "\uFD49"],
  ["\uFD4A", "\uFD4A"],
  ["\uFD4B", "\uFD4B"],
  ["\uFD4C", "\uFD4C"],
  ["\uFD4D", "\uFD4D"],
  ["\uFD4E", "\uFD4E"],
  ["\uFD4F", "\uFD4F"],
  ["\uFD50", "\uFD50"], /* ARABIC LIGATURE TEH WITH JEEM WITH MEEM INITIAL FORM */
  ["\uFD51", "\uFD51"], /* ARABIC LIGATURE TEH WITH HAH WITH JEEM FINAL FORM */
  ["\uFD52", "\uFD52"], /* ARABIC LIGATURE TEH WITH HAH WITH JEEM INITIAL FORM */
  ["\uFD53", "\uFD53"], /* ARABIC LIGATURE TEH WITH HAH WITH MEEM INITIAL FORM */
  ["\uFD54", "\uFD54"], /* ARABIC LIGATURE TEH WITH KHAH WITH MEEM INITIAL FORM */
  ["\uFD55", "\uFD55"], /* ARABIC LIGATURE TEH WITH MEEM WITH JEEM INITIAL FORM */
  ["\uFD56", "\uFD56"], /* ARABIC LIGATURE TEH WITH MEEM WITH HAH INITIAL FORM */
  ["\uFD57", "\uFD57"], /* ARABIC LIGATURE TEH WITH MEEM WITH KHAH INITIAL FORM */
  ["\uFD58", "\uFD58"], /* ARABIC LIGATURE JEEM WITH MEEM WITH HAH FINAL FORM */
  ["\uFD59", "\uFD59"], /* ARABIC LIGATURE JEEM WITH MEEM WITH HAH INITIAL FORM */
  ["\uFD5A", "\uFD5A"], /* ARABIC LIGATURE HAH WITH MEEM WITH YEH FINAL FORM */
  ["\uFD5B", "\uFD5B"], /* ARABIC LIGATURE HAH WITH MEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD5C", "\uFD5C"], /* ARABIC LIGATURE SEEN WITH HAH WITH JEEM INITIAL FORM */
  ["\uFD5D", "\uFD5D"], /* ARABIC LIGATURE SEEN WITH JEEM WITH HAH INITIAL FORM */
  ["\uFD5E", "\uFD5E"], /* ARABIC LIGATURE SEEN WITH JEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD5F", "\uFD5F"], /* ARABIC LIGATURE SEEN WITH MEEM WITH HAH FINAL FORM */
  ["\uFD60", "\uFD60"], /* ARABIC LIGATURE SEEN WITH MEEM WITH HAH INITIAL FORM */
  ["\uFD61", "\uFD61"], /* ARABIC LIGATURE SEEN WITH MEEM WITH JEEM INITIAL FORM */
  ["\uFD62", "\uFD62"], /* ARABIC LIGATURE SEEN WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD63", "\uFD63"], /* ARABIC LIGATURE SEEN WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFD64", "\uFD64"], /* ARABIC LIGATURE SAD WITH HAH WITH HAH FINAL FORM */
  ["\uFD65", "\uFD65"], /* ARABIC LIGATURE SAD WITH HAH WITH HAH INITIAL FORM */
  ["\uFD66", "\uFD66"], /* ARABIC LIGATURE SAD WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD67", "\uFD67"], /* ARABIC LIGATURE SHEEN WITH HAH WITH MEEM FINAL FORM */
  ["\uFD68", "\uFD68"], /* ARABIC LIGATURE SHEEN WITH HAH WITH MEEM INITIAL FORM */
  ["\uFD69", "\uFD69"], /* ARABIC LIGATURE SHEEN WITH JEEM WITH YEH FINAL FORM */
  ["\uFD6A", "\uFD6A"], /* ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH FINAL FORM */
  ["\uFD6B", "\uFD6B"], /* ARABIC LIGATURE SHEEN WITH MEEM WITH KHAH INITIAL FORM */
  ["\uFD6C", "\uFD6C"], /* ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD6D", "\uFD6D"], /* ARABIC LIGATURE SHEEN WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFD6E", "\uFD6E"], /* ARABIC LIGATURE DAD WITH HAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFD6F", "\uFD6F"], /* ARABIC LIGATURE DAD WITH KHAH WITH MEEM FINAL FORM */
  ["\uFD70", "\uFD70"], /* ARABIC LIGATURE DAD WITH KHAH WITH MEEM INITIAL FORM */
  ["\uFD71", "\uFD71"], /* ARABIC LIGATURE TAH WITH MEEM WITH HAH FINAL FORM */
  ["\uFD72", "\uFD72"], /* ARABIC LIGATURE TAH WITH MEEM WITH HAH INITIAL FORM */
  ["\uFD73", "\uFD73"], /* ARABIC LIGATURE TAH WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFD74", "\uFD74"], /* ARABIC LIGATURE TAH WITH MEEM WITH YEH FINAL FORM */
  ["\uFD75", "\uFD75"], /* ARABIC LIGATURE AIN WITH JEEM WITH MEEM FINAL FORM */
  ["\uFD76", "\uFD76"], /* ARABIC LIGATURE AIN WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD77", "\uFD77"], /* ARABIC LIGATURE AIN WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFD78", "\uFD78"], /* ARABIC LIGATURE AIN WITH MEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD79", "\uFD79"], /* ARABIC LIGATURE GHAIN WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD7A", "\uFD7A"], /* ARABIC LIGATURE GHAIN WITH MEEM WITH YEH FINAL FORM */
  ["\uFD7B", "\uFD7B"], /* ARABIC LIGATURE GHAIN WITH MEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD7C", "\uFD7C"], /* ARABIC LIGATURE FEH WITH KHAH WITH MEEM FINAL FORM */
  ["\uFD7D", "\uFD7D"], /* ARABIC LIGATURE FEH WITH KHAH WITH MEEM INITIAL FORM */
  ["\uFD7E", "\uFD7E"], /* ARABIC LIGATURE QAF WITH MEEM WITH HAH FINAL FORM */
  ["\uFD7F", "\uFD7F"], /* ARABIC LIGATURE QAF WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD80", "\uFD80"], /* ARABIC LIGATURE LAM WITH HAH WITH MEEM FINAL FORM */
  ["\uFD81", "\uFD81"], /* ARABIC LIGATURE LAM WITH HAH WITH YEH FINAL FORM */
  ["\uFD82", "\uFD82"], /* ARABIC LIGATURE LAM WITH HAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFD83", "\uFD83"], /* ARABIC LIGATURE LAM WITH JEEM WITH JEEM INITIAL FORM */
  ["\uFD84", "\uFD84"], /* ARABIC LIGATURE LAM WITH JEEM WITH JEEM FINAL FORM */
  ["\uFD85", "\uFD85"], /* ARABIC LIGATURE LAM WITH KHAH WITH MEEM FINAL FORM */
  ["\uFD86", "\uFD86"], /* ARABIC LIGATURE LAM WITH KHAH WITH MEEM INITIAL FORM */
  ["\uFD87", "\uFD87"], /* ARABIC LIGATURE LAM WITH MEEM WITH HAH FINAL FORM */
  ["\uFD88", "\uFD88"], /* ARABIC LIGATURE LAM WITH MEEM WITH HAH INITIAL FORM */
  ["\uFD89", "\uFD89"], /* ARABIC LIGATURE MEEM WITH HAH WITH JEEM INITIAL FORM */
  ["\uFD8A", "\uFD8A"], /* ARABIC LIGATURE MEEM WITH HAH WITH MEEM INITIAL FORM */
  ["\uFD8B", "\uFD8B"], /* ARABIC LIGATURE MEEM WITH HAH WITH YEH FINAL FORM */
  ["\uFD8C", "\uFD8C"], /* ARABIC LIGATURE MEEM WITH JEEM WITH HAH INITIAL FORM */
  ["\uFD8D", "\uFD8D"], /* ARABIC LIGATURE MEEM WITH JEEM WITH MEEM INITIAL FORM */
  ["\uFD8E", "\uFD8E"], /* ARABIC LIGATURE MEEM WITH KHAH WITH JEEM INITIAL FORM */
  ["\uFD8F", "\uFD8F"], /* ARABIC LIGATURE MEEM WITH KHAH WITH MEEM INITIAL FORM */
  ["\uFD90", "\uFD90"],
  ["\uFD91", "\uFD91"],
  ["\uFD92", "\uFD92"], /* ARABIC LIGATURE MEEM WITH JEEM WITH KHAH INITIAL FORM */
  ["\uFD93", "\uFD93"], /* ARABIC LIGATURE HEH WITH MEEM WITH JEEM INITIAL FORM */
  ["\uFD94", "\uFD94"], /* ARABIC LIGATURE HEH WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFD95", "\uFD95"], /* ARABIC LIGATURE NOON WITH HAH WITH MEEM INITIAL FORM */
  ["\uFD96", "\uFD96"], /* ARABIC LIGATURE NOON WITH HAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFD97", "\uFD97"], /* ARABIC LIGATURE NOON WITH JEEM WITH MEEM FINAL FORM */
  ["\uFD98", "\uFD98"], /* ARABIC LIGATURE NOON WITH JEEM WITH MEEM INITIAL FORM */
  ["\uFD99", "\uFD99"], /* ARABIC LIGATURE NOON WITH JEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD9A", "\uFD9A"], /* ARABIC LIGATURE NOON WITH MEEM WITH YEH FINAL FORM */
  ["\uFD9B", "\uFD9B"], /* ARABIC LIGATURE NOON WITH MEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFD9C", "\uFD9C"], /* ARABIC LIGATURE YEH WITH MEEM WITH MEEM FINAL FORM */
  ["\uFD9D", "\uFD9D"], /* ARABIC LIGATURE YEH WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFD9E", "\uFD9E"], /* ARABIC LIGATURE BEH WITH KHAH WITH YEH FINAL FORM */
  ["\uFD9F", "\uFD9F"], /* ARABIC LIGATURE TEH WITH JEEM WITH YEH FINAL FORM */
  ["\uFDA0", "\uFDA0"], /* ARABIC LIGATURE TEH WITH JEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFDA1", "\uFDA1"], /* ARABIC LIGATURE TEH WITH KHAH WITH YEH FINAL FORM */
  ["\uFDA2", "\uFDA2"], /* ARABIC LIGATURE TEH WITH KHAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFDA3", "\uFDA3"], /* ARABIC LIGATURE TEH WITH MEEM WITH YEH FINAL FORM */
  ["\uFDA4", "\uFDA4"], /* ARABIC LIGATURE TEH WITH MEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFDA5", "\uFDA5"], /* ARABIC LIGATURE JEEM WITH MEEM WITH YEH FINAL FORM */
  ["\uFDA6", "\uFDA6"], /* ARABIC LIGATURE JEEM WITH HAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFDA7", "\uFDA7"], /* ARABIC LIGATURE JEEM WITH MEEM WITH ALEF MAKSURA FINAL FORM */
  ["\uFDA8", "\uFDA8"], /* ARABIC LIGATURE SEEN WITH KHAH WITH ALEF MAKSURA FINAL FORM */
  ["\uFDA9", "\uFDA9"], /* ARABIC LIGATURE SAD WITH HAH WITH YEH FINAL FORM */
  ["\uFDAA", "\uFDAA"], /* ARABIC LIGATURE SHEEN WITH HAH WITH YEH FINAL FORM */
  ["\uFDAB", "\uFDAB"], /* ARABIC LIGATURE DAD WITH HAH WITH YEH FINAL FORM */
  ["\uFDAC", "\uFDAC"], /* ARABIC LIGATURE LAM WITH JEEM WITH YEH FINAL FORM */
  ["\uFDAD", "\uFDAD"], /* ARABIC LIGATURE LAM WITH MEEM WITH YEH FINAL FORM */
  ["\uFDAE", "\uFDAE"], /* ARABIC LIGATURE YEH WITH HAH WITH YEH FINAL FORM */
  ["\uFDAF", "\uFDAF"], /* ARABIC LIGATURE YEH WITH JEEM WITH YEH FINAL FORM */
  ["\uFDB0", "\uFDB0"], /* ARABIC LIGATURE YEH WITH MEEM WITH YEH FINAL FORM */
  ["\uFDB1", "\uFDB1"], /* ARABIC LIGATURE MEEM WITH MEEM WITH YEH FINAL FORM */
  ["\uFDB2", "\uFDB2"], /* ARABIC LIGATURE QAF WITH MEEM WITH YEH FINAL FORM */
  ["\uFDB3", "\uFDB3"], /* ARABIC LIGATURE NOON WITH HAH WITH YEH FINAL FORM */
  ["\uFDB4", "\uFDB4"], /* ARABIC LIGATURE QAF WITH MEEM WITH HAH INITIAL FORM */
  ["\uFDB5", "\uFDB5"], /* ARABIC LIGATURE LAM WITH HAH WITH MEEM INITIAL FORM */
  ["\uFDB6", "\uFDB6"], /* ARABIC LIGATURE AIN WITH MEEM WITH YEH FINAL FORM */
  ["\uFDB7", "\uFDB7"], /* ARABIC LIGATURE KAF WITH MEEM WITH YEH FINAL FORM */
  ["\uFDB8", "\uFDB8"], /* ARABIC LIGATURE NOON WITH JEEM WITH HAH INITIAL FORM */
  ["\uFDB9", "\uFDB9"], /* ARABIC LIGATURE MEEM WITH KHAH WITH YEH FINAL FORM */
  ["\uFDBA", "\uFDBA"], /* ARABIC LIGATURE LAM WITH JEEM WITH MEEM INITIAL FORM */
  ["\uFDBB", "\uFDBB"], /* ARABIC LIGATURE KAF WITH MEEM WITH MEEM FINAL FORM */
  ["\uFDBC", "\uFDBC"], /* ARABIC LIGATURE LAM WITH JEEM WITH MEEM FINAL FORM */
  ["\uFDBD", "\uFDBD"], /* ARABIC LIGATURE NOON WITH JEEM WITH HAH FINAL FORM */
  ["\uFDBE", "\uFDBE"], /* ARABIC LIGATURE JEEM WITH HAH WITH YEH FINAL FORM */
  ["\uFDBF", "\uFDBF"], /* ARABIC LIGATURE HAH WITH JEEM WITH YEH FINAL FORM */
  ["\uFDC0", "\uFDC0"], /* ARABIC LIGATURE MEEM WITH JEEM WITH YEH FINAL FORM */
  ["\uFDC1", "\uFDC1"], /* ARABIC LIGATURE FEH WITH MEEM WITH YEH FINAL FORM */
  ["\uFDC2", "\uFDC2"], /* ARABIC LIGATURE BEH WITH HAH WITH YEH FINAL FORM */
  ["\uFDC3", "\uFDC3"], /* ARABIC LIGATURE KAF WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFDC4", "\uFDC4"], /* ARABIC LIGATURE AIN WITH JEEM WITH MEEM INITIAL FORM */
  ["\uFDC5", "\uFDC5"], /* ARABIC LIGATURE SAD WITH MEEM WITH MEEM INITIAL FORM */
  ["\uFDC6", "\uFDC6"], /* ARABIC LIGATURE SEEN WITH KHAH WITH YEH FINAL FORM */
  ["\uFDC7", "\uFDC7"], /* ARABIC LIGATURE NOON WITH JEEM WITH YEH FINAL FORM */
  ["\uFDC8", "\uFDC8"],
  ["\uFDC9", "\uFDC9"],
  ["\uFDCA", "\uFDCA"],
  ["\uFDCB", "\uFDCB"],
  ["\uFDCC", "\uFDCC"],
  ["\uFDCD", "\uFDCD"],
  ["\uFDCE", "\uFDCE"],
  ["\uFDCF", "\uFDCF"],
  ["\uFDD0", "\uFDD0"],
  ["\uFDD1", "\uFDD1"],
  ["\uFDD2", "\uFDD2"],
  ["\uFDD3", "\uFDD3"],
  ["\uFDD4", "\uFDD4"],
  ["\uFDD5", "\uFDD5"],
  ["\uFDD6", "\uFDD6"],
  ["\uFDD7", "\uFDD7"],
  ["\uFDD8", "\uFDD8"],
  ["\uFDD9", "\uFDD9"],
  ["\uFDDA", "\uFDDA"],
  ["\uFDDB", "\uFDDB"],
  ["\uFDDC", "\uFDDC"],
  ["\uFDDD", "\uFDDD"],
  ["\uFDDE", "\uFDDE"],
  ["\uFDDF", "\uFDDF"],
  ["\uFDE0", "\uFDE0"],
  ["\uFDE1", "\uFDE1"],
  ["\uFDE2", "\uFDE2"],
  ["\uFDE3", "\uFDE3"],
  ["\uFDE4", "\uFDE4"],
  ["\uFDE5", "\uFDE5"],
  ["\uFDE6", "\uFDE6"],
  ["\uFDE7", "\uFDE7"],
  ["\uFDE8", "\uFDE8"],
  ["\uFDE9", "\uFDE9"],
  ["\uFDEA", "\uFDEA"],
  ["\uFDEB", "\uFDEB"],
  ["\uFDEC", "\uFDEC"],
  ["\uFDED", "\uFDED"],
  ["\uFDEE", "\uFDEE"],
  ["\uFDEF", "\uFDEF"],
  ["\uFDF0", "\uFDF0"], /* ARABIC LIGATURE SALLA USED AS KORANIC STOP SIGN ISOLATED FORM */
  ["\uFDF1", "\uFDF1"], /* ARABIC LIGATURE QALA USED AS KORANIC STOP SIGN ISOLATED FORM */
  ["\uFDF2", "\uFDF2"], /* ARABIC LIGATURE ALLAH ISOLATED FORM */
  ["\uFDF3", "\uFDF3"], /* ARABIC LIGATURE AKBAR ISOLATED FORM */
  ["\uFDF4", "\uFDF4"], /* ARABIC LIGATURE MOHAMMAD ISOLATED FORM */
  ["\uFDF5", "\uFDF5"], /* ARABIC LIGATURE SALAM ISOLATED FORM */
  ["\uFDF6", "\uFDF6"], /* ARABIC LIGATURE RASOUL ISOLATED FORM */
  ["\uFDF7", "\uFDF7"], /* ARABIC LIGATURE ALAYHE ISOLATED FORM */
  ["\uFDF8", "\uFDF8"], /* ARABIC LIGATURE WASALLAM ISOLATED FORM */
  ["\uFDF9", "\uFDF9"], /* ARABIC LIGATURE SALLA ISOLATED FORM */
  ["\uFDFA", "\uFDFA"], /* ARABIC LIGATURE SALLALLAHOU ALAYHE WASALLAM (ARABIC LETTER SALLALLAHOU ALAYHE WASALLAM) */
  ["\uFDFB", "\uFDFB"], /* ARABIC LIGATURE JALLAJALALOUHOU (ARABIC LETTER JALLAJALALOUHOU) */
  ["\uFDFC", "\uFDFC"], /* RIAL SIGN */
  ["\uFDFD", "\uFDFD"], /* ARABIC LIGATURE BISMILLAH AR-RAHMAN AR-RAHEEM */
  ["\uFDFE", "\uFDFE"],
  ["\uFDFF", "\uFDFF"],
  ["\uFE00", "\uFE00"], /* VARIATION SELECTOR-1 */
  ["\uFE01", "\uFE01"], /* VARIATION SELECTOR-2 */
  ["\uFE02", "\uFE02"], /* VARIATION SELECTOR-3 */
  ["\uFE03", "\uFE03"], /* VARIATION SELECTOR-4 */
  ["\uFE04", "\uFE04"], /* VARIATION SELECTOR-5 */
  ["\uFE05", "\uFE05"], /* VARIATION SELECTOR-6 */
  ["\uFE06", "\uFE06"], /* VARIATION SELECTOR-7 */
  ["\uFE07", "\uFE07"], /* VARIATION SELECTOR-8 */
  ["\uFE08", "\uFE08"], /* VARIATION SELECTOR-9 */
  ["\uFE09", "\uFE09"], /* VARIATION SELECTOR-10 */
  ["\uFE0A", "\uFE0A"], /* VARIATION SELECTOR-11 */
  ["\uFE0B", "\uFE0B"], /* VARIATION SELECTOR-12 */
  ["\uFE0C", "\uFE0C"], /* VARIATION SELECTOR-13 */
  ["\uFE0D", "\uFE0D"], /* VARIATION SELECTOR-14 */
  ["\uFE0E", "\uFE0E"], /* VARIATION SELECTOR-15 */
  ["\uFE0F", "\uFE0F"], /* VARIATION SELECTOR-16 */
  ["\uFE10", "\uFE10"], /* PRESENTATION FORM FOR VERTICAL COMMA */
  ["\uFE11", "\uFE11"], /* PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC COMMA */
  ["\uFE12", "\uFE12"], /* PRESENTATION FORM FOR VERTICAL IDEOGRAPHIC FULL STOP */
  ["\uFE13", "\uFE13"], /* PRESENTATION FORM FOR VERTICAL COLON */
  ["\uFE14", "\uFE14"], /* PRESENTATION FORM FOR VERTICAL SEMICOLON */
  ["\uFE15", "\uFE15"], /* PRESENTATION FORM FOR VERTICAL EXCLAMATION MARK */
  ["\uFE16", "\uFE16"], /* PRESENTATION FORM FOR VERTICAL QUESTION MARK */
  ["\uFE17", "\uFE17"], /* PRESENTATION FORM FOR VERTICAL LEFT WHITE LENTICULAR BRACKET */
  ["\uFE18", "\uFE18"], /* PRESENTATION FORM FOR VERTICAL RIGHT WHITE LENTICULAR BRAKCET */
  ["\uFE19", "\uFE19"], /* PRESENTATION FORM FOR VERTICAL HORIZONTAL ELLIPSIS */
  ["\uFE1A", "\uFE1A"],
  ["\uFE1B", "\uFE1B"],
  ["\uFE1C", "\uFE1C"],
  ["\uFE1D", "\uFE1D"],
  ["\uFE1E", "\uFE1E"],
  ["\uFE1F", "\uFE1F"],
  ["\uFE20", "\uFE20"], /* COMBINING LIGATURE LEFT HALF */
  ["\uFE21", "\uFE21"], /* COMBINING LIGATURE RIGHT HALF */
  ["\uFE22", "\uFE22"], /* COMBINING DOUBLE TILDE LEFT HALF */
  ["\uFE23", "\uFE23"], /* COMBINING DOUBLE TILDE RIGHT HALF */
  ["\uFE24", "\uFE24"], /* COMBINING MACRON LEFT HALF */
  ["\uFE25", "\uFE25"], /* COMBINING MACRON RIGHT HALF */
  ["\uFE26", "\uFE26"], /* COMBINING CONJOINING MACRON */
  ["\uFE27", "\uFE27"], /* COMBINING LIGATURE LEFT HALF BELOW */
  ["\uFE28", "\uFE28"], /* COMBINING LIGATURE RIGHT HALF BELOW */
  ["\uFE29", "\uFE29"], /* COMBINING TILDE LEFT HALF BELOW */
  ["\uFE2A", "\uFE2A"], /* COMBINING TILDE RIGHT HALF BELOW */
  ["\uFE2B", "\uFE2B"], /* COMBINING MACRON LEFT HALF BELOW */
  ["\uFE2C", "\uFE2C"], /* COMBINING MACRON RIGHT HALF BELOW */
  ["\uFE2D", "\uFE2D"], /* COMBINING CONJOINING MACRON BELOW */
  ["\uFE2E", "\uFE2E"], /* COMBINING CYRILLIC TITLO LEFT HALF */
  ["\uFE2F", "\uFE2F"], /* COMBINING CYRILLIC TITLO RIGHT HALF */
  ["\uFE30", "\uFE30"], /* PRESENTATION FORM FOR VERTICAL TWO DOT LEADER (GLYPH FOR VERTICAL TWO DOT LEADER) */
  ["\uFE31", "\uFE31"], /* PRESENTATION FORM FOR VERTICAL EM DASH (GLYPH FOR VERTICAL EM DASH) */
  ["\uFE32", "\uFE32"], /* PRESENTATION FORM FOR VERTICAL EN DASH (GLYPH FOR VERTICAL EN DASH) */
  ["\uFE33", "\uFE33"], /* PRESENTATION FORM FOR VERTICAL LOW LINE (GLYPH FOR VERTICAL SPACING UNDERSCORE) */
  ["\uFE34", "\uFE34"], /* PRESENTATION FORM FOR VERTICAL WAVY LOW LINE (GLYPH FOR VERTICAL SPACING WAVY UNDERSCORE) */
  ["\uFE35", "\uFE35"], /* PRESENTATION FORM FOR VERTICAL LEFT PARENTHESIS (GLYPH FOR VERTICAL OPENING PARENTHESIS) */
  ["\uFE36", "\uFE36"], /* PRESENTATION FORM FOR VERTICAL RIGHT PARENTHESIS (GLYPH FOR VERTICAL CLOSING PARENTHESIS) */
  ["\uFE37", "\uFE37"], /* PRESENTATION FORM FOR VERTICAL LEFT CURLY BRACKET (GLYPH FOR VERTICAL OPENING CURLY BRACKET) */
  ["\uFE38", "\uFE38"], /* PRESENTATION FORM FOR VERTICAL RIGHT CURLY BRACKET (GLYPH FOR VERTICAL CLOSING CURLY BRACKET) */
  ["\uFE39", "\uFE39"], /* PRESENTATION FORM FOR VERTICAL LEFT TORTOISE SHELL BRACKET (GLYPH FOR VERTICAL OPENING TORTOISE SHELL BRACKET) */
  ["\uFE3A", "\uFE3A"], /* PRESENTATION FORM FOR VERTICAL RIGHT TORTOISE SHELL BRACKET (GLYPH FOR VERTICAL CLOSING TORTOISE SHELL BRACKET) */
  ["\uFE3B", "\uFE3B"], /* PRESENTATION FORM FOR VERTICAL LEFT BLACK LENTICULAR BRACKET (GLYPH FOR VERTICAL OPENING BLACK LENTICULAR BRACKET) */
  ["\uFE3C", "\uFE3C"], /* PRESENTATION FORM FOR VERTICAL RIGHT BLACK LENTICULAR BRACKET (GLYPH FOR VERTICAL CLOSING BLACK LENTICULAR BRACKET) */
  ["\uFE3D", "\uFE3D"], /* PRESENTATION FORM FOR VERTICAL LEFT DOUBLE ANGLE BRACKET (GLYPH FOR VERTICAL OPENING DOUBLE ANGLE BRACKET) */
  ["\uFE3E", "\uFE3E"], /* PRESENTATION FORM FOR VERTICAL RIGHT DOUBLE ANGLE BRACKET (GLYPH FOR VERTICAL CLOSING DOUBLE ANGLE BRACKET) */
  ["\uFE3F", "\uFE3F"], /* PRESENTATION FORM FOR VERTICAL LEFT ANGLE BRACKET (GLYPH FOR VERTICAL OPENING ANGLE BRACKET) */
  ["\uFE40", "\uFE40"], /* PRESENTATION FORM FOR VERTICAL RIGHT ANGLE BRACKET (GLYPH FOR VERTICAL CLOSING ANGLE BRACKET) */
  ["\uFE41", "\uFE41"], /* PRESENTATION FORM FOR VERTICAL LEFT CORNER BRACKET (GLYPH FOR VERTICAL OPENING CORNER BRACKET) */
  ["\uFE42", "\uFE42"], /* PRESENTATION FORM FOR VERTICAL RIGHT CORNER BRACKET (GLYPH FOR VERTICAL CLOSING CORNER BRACKET) */
  ["\uFE43", "\uFE43"], /* PRESENTATION FORM FOR VERTICAL LEFT WHITE CORNER BRACKET (GLYPH FOR VERTICAL OPENING WHITE CORNER BRACKET) */
  ["\uFE44", "\uFE44"], /* PRESENTATION FORM FOR VERTICAL RIGHT WHITE CORNER BRACKET (GLYPH FOR VERTICAL CLOSING WHITE CORNER BRACKET) */
  ["\uFE45", "\uFE45"], /* SESAME DOT */
  ["\uFE46", "\uFE46"], /* WHITE SESAME DOT */
  ["\uFE47", "\uFE47"], /* PRESENTATION FORM FOR VERTICAL LEFT SQUARE BRACKET */
  ["\uFE48", "\uFE48"], /* PRESENTATION FORM FOR VERTICAL RIGHT SQUARE BRACKET */
  ["\uFE49", "\uFE49"], /* DASHED OVERLINE (SPACING DASHED OVERSCORE) */
  ["\uFE4A", "\uFE4A"], /* CENTRELINE OVERLINE (SPACING CENTERLINE OVERSCORE) */
  ["\uFE4B", "\uFE4B"], /* WAVY OVERLINE (SPACING WAVY OVERSCORE) */
  ["\uFE4C", "\uFE4C"], /* DOUBLE WAVY OVERLINE (SPACING DOUBLE WAVY OVERSCORE) */
  ["\uFE4D", "\uFE4D"], /* DASHED LOW LINE (SPACING DASHED UNDERSCORE) */
  ["\uFE4E", "\uFE4E"], /* CENTRELINE LOW LINE (SPACING CENTERLINE UNDERSCORE) */
  ["\uFE4F", "\uFE4F"], /* WAVY LOW LINE (SPACING WAVY UNDERSCORE) */
  ["\uFE50", "\uFE50"], /* SMALL COMMA */
  ["\uFE51", "\uFE51"], /* SMALL IDEOGRAPHIC COMMA */
  ["\uFE52", "\uFE52"], /* SMALL FULL STOP (SMALL PERIOD) */
  ["\uFE53", "\uFE53"],
  ["\uFE54", "\uFE54"], /* SMALL SEMICOLON */
  ["\uFE55", "\uFE55"], /* SMALL COLON */
  ["\uFE56", "\uFE56"], /* SMALL QUESTION MARK */
  ["\uFE57", "\uFE57"], /* SMALL EXCLAMATION MARK */
  ["\uFE58", "\uFE58"], /* SMALL EM DASH */
  ["\uFE59", "\uFE59"], /* SMALL LEFT PARENTHESIS (SMALL OPENING PARENTHESIS) */
  ["\uFE5A", "\uFE5A"], /* SMALL RIGHT PARENTHESIS (SMALL CLOSING PARENTHESIS) */
  ["\uFE5B", "\uFE5B"], /* SMALL LEFT CURLY BRACKET (SMALL OPENING CURLY BRACKET) */
  ["\uFE5C", "\uFE5C"], /* SMALL RIGHT CURLY BRACKET (SMALL CLOSING CURLY BRACKET) */
  ["\uFE5D", "\uFE5D"], /* SMALL LEFT TORTOISE SHELL BRACKET (SMALL OPENING TORTOISE SHELL BRACKET) */
  ["\uFE5E", "\uFE5E"], /* SMALL RIGHT TORTOISE SHELL BRACKET (SMALL CLOSING TORTOISE SHELL BRACKET) */
  ["\uFE5F", "\uFE5F"], /* SMALL NUMBER SIGN */
  ["\uFE60", "\uFE60"], /* SMALL AMPERSAND */
  ["\uFE61", "\uFE61"], /* SMALL ASTERISK */
  ["\uFE62", "\uFE62"], /* SMALL PLUS SIGN */
  ["\uFE63", "\uFE63"], /* SMALL HYPHEN-MINUS */
  ["\uFE64", "\uFE64"], /* SMALL LESS-THAN SIGN */
  ["\uFE65", "\uFE65"], /* SMALL GREATER-THAN SIGN */
  ["\uFE66", "\uFE66"], /* SMALL EQUALS SIGN */
  ["\uFE67", "\uFE67"],
  ["\uFE68", "\uFE68"], /* SMALL REVERSE SOLIDUS (SMALL BACKSLASH) */
  ["\uFE69", "\uFE69"], /* SMALL DOLLAR SIGN */
  ["\uFE6A", "\uFE6A"], /* SMALL PERCENT SIGN */
  ["\uFE6B", "\uFE6B"], /* SMALL COMMERCIAL AT */
  ["\uFE6C", "\uFE6C"],
  ["\uFE6D", "\uFE6D"],
  ["\uFE6E", "\uFE6E"],
  ["\uFE6F", "\uFE6F"],
  ["\uFE70", "\uFE70"], /* ARABIC FATHATAN ISOLATED FORM (ARABIC SPACING FATHATAN) */
  ["\uFE71", "\uFE71"], /* ARABIC TATWEEL WITH FATHATAN ABOVE (ARABIC FATHATAN ON TATWEEL) */
  ["\uFE72", "\uFE72"], /* ARABIC DAMMATAN ISOLATED FORM (ARABIC SPACING DAMMATAN) */
  ["\uFE73", "\uFE73"], /* ARABIC TAIL FRAGMENT */
  ["\uFE74", "\uFE74"], /* ARABIC KASRATAN ISOLATED FORM (ARABIC SPACING KASRATAN) */
  ["\uFE75", "\uFE75"],
  ["\uFE76", "\uFE76"], /* ARABIC FATHA ISOLATED FORM (ARABIC SPACING FATHAH) */
  ["\uFE77", "\uFE77"], /* ARABIC FATHA MEDIAL FORM (ARABIC FATHAH ON TATWEEL) */
  ["\uFE78", "\uFE78"], /* ARABIC DAMMA ISOLATED FORM (ARABIC SPACING DAMMAH) */
  ["\uFE79", "\uFE79"], /* ARABIC DAMMA MEDIAL FORM (ARABIC DAMMAH ON TATWEEL) */
  ["\uFE7A", "\uFE7A"], /* ARABIC KASRA ISOLATED FORM (ARABIC SPACING KASRAH) */
  ["\uFE7B", "\uFE7B"], /* ARABIC KASRA MEDIAL FORM (ARABIC KASRAH ON TATWEEL) */
  ["\uFE7C", "\uFE7C"], /* ARABIC SHADDA ISOLATED FORM (ARABIC SPACING SHADDAH) */
  ["\uFE7D", "\uFE7D"], /* ARABIC SHADDA MEDIAL FORM (ARABIC SHADDAH ON TATWEEL) */
  ["\uFE7E", "\uFE7E"], /* ARABIC SUKUN ISOLATED FORM (ARABIC SPACING SUKUN) */
  ["\uFE7F", "\uFE7F"], /* ARABIC SUKUN MEDIAL FORM (ARABIC SUKUN ON TATWEEL) */
  ["\uFE80", "\uFE80"], /* ARABIC LETTER HAMZA ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH) */
  ["\uFE81", "\uFE81"], /* ARABIC LETTER ALEF WITH MADDA ABOVE ISOLATED FORM (GLYPH FOR ISOLATE ARABIC MADDAH ON ALEF) */
  ["\uFE82", "\uFE82"], /* ARABIC LETTER ALEF WITH MADDA ABOVE FINAL FORM (GLYPH FOR FINAL ARABIC MADDAH ON ALEF) */
  ["\uFE83", "\uFE83"], /* ARABIC LETTER ALEF WITH HAMZA ABOVE ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH ON ALEF) */
  ["\uFE84", "\uFE84"], /* ARABIC LETTER ALEF WITH HAMZA ABOVE FINAL FORM (GLYPH FOR FINAL ARABIC HAMZAH ON ALEF) */
  ["\uFE85", "\uFE85"], /* ARABIC LETTER WAW WITH HAMZA ABOVE ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH ON WAW) */
  ["\uFE86", "\uFE86"], /* ARABIC LETTER WAW WITH HAMZA ABOVE FINAL FORM (GLYPH FOR FINAL ARABIC HAMZAH ON WAW) */
  ["\uFE87", "\uFE87"], /* ARABIC LETTER ALEF WITH HAMZA BELOW ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH UNDER ALEF) */
  ["\uFE88", "\uFE88"], /* ARABIC LETTER ALEF WITH HAMZA BELOW FINAL FORM (GLYPH FOR FINAL ARABIC HAMZAH UNDER ALEF) */
  ["\uFE89", "\uFE89"], /* ARABIC LETTER YEH WITH HAMZA ABOVE ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH ON YA) */
  ["\uFE8A", "\uFE8A"], /* ARABIC LETTER YEH WITH HAMZA ABOVE FINAL FORM (GLYPH FOR FINAL ARABIC HAMZAH ON YA) */
  ["\uFE8B", "\uFE8B"], /* ARABIC LETTER YEH WITH HAMZA ABOVE INITIAL FORM (GLYPH FOR INITIAL ARABIC HAMZAH ON YA) */
  ["\uFE8C", "\uFE8C"], /* ARABIC LETTER YEH WITH HAMZA ABOVE MEDIAL FORM (GLYPH FOR MEDIAL ARABIC HAMZAH ON YA) */
  ["\uFE8D", "\uFE8D"], /* ARABIC LETTER ALEF ISOLATED FORM (GLYPH FOR ISOLATE ARABIC ALEF) */
  ["\uFE8E", "\uFE8E"], /* ARABIC LETTER ALEF FINAL FORM (GLYPH FOR FINAL ARABIC ALEF) */
  ["\uFE8F", "\uFE8F"], /* ARABIC LETTER BEH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC BAA) */
  ["\uFE90", "\uFE90"], /* ARABIC LETTER BEH FINAL FORM (GLYPH FOR FINAL ARABIC BAA) */
  ["\uFE91", "\uFE91"], /* ARABIC LETTER BEH INITIAL FORM (GLYPH FOR INITIAL ARABIC BAA) */
  ["\uFE92", "\uFE92"], /* ARABIC LETTER BEH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC BAA) */
  ["\uFE93", "\uFE93"], /* ARABIC LETTER TEH MARBUTA ISOLATED FORM (GLYPH FOR ISOLATE ARABIC TAA MARBUTAH) */
  ["\uFE94", "\uFE94"], /* ARABIC LETTER TEH MARBUTA FINAL FORM (GLYPH FOR FINAL ARABIC TAA MARBUTAH) */
  ["\uFE95", "\uFE95"], /* ARABIC LETTER TEH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC TAA) */
  ["\uFE96", "\uFE96"], /* ARABIC LETTER TEH FINAL FORM (GLYPH FOR FINAL ARABIC TAA) */
  ["\uFE97", "\uFE97"], /* ARABIC LETTER TEH INITIAL FORM (GLYPH FOR INITIAL ARABIC TAA) */
  ["\uFE98", "\uFE98"], /* ARABIC LETTER TEH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC TAA) */
  ["\uFE99", "\uFE99"], /* ARABIC LETTER THEH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC THAA) */
  ["\uFE9A", "\uFE9A"], /* ARABIC LETTER THEH FINAL FORM (GLYPH FOR FINAL ARABIC THAA) */
  ["\uFE9B", "\uFE9B"], /* ARABIC LETTER THEH INITIAL FORM (GLYPH FOR INITIAL ARABIC THAA) */
  ["\uFE9C", "\uFE9C"], /* ARABIC LETTER THEH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC THAA) */
  ["\uFE9D", "\uFE9D"], /* ARABIC LETTER JEEM ISOLATED FORM (GLYPH FOR ISOLATE ARABIC JEEM) */
  ["\uFE9E", "\uFE9E"], /* ARABIC LETTER JEEM FINAL FORM (GLYPH FOR FINAL ARABIC JEEM) */
  ["\uFE9F", "\uFE9F"], /* ARABIC LETTER JEEM INITIAL FORM (GLYPH FOR INITIAL ARABIC JEEM) */
  ["\uFEA0", "\uFEA0"], /* ARABIC LETTER JEEM MEDIAL FORM (GLYPH FOR MEDIAL ARABIC JEEM) */
  ["\uFEA1", "\uFEA1"], /* ARABIC LETTER HAH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAA) */
  ["\uFEA2", "\uFEA2"], /* ARABIC LETTER HAH FINAL FORM (GLYPH FOR FINAL ARABIC HAA) */
  ["\uFEA3", "\uFEA3"], /* ARABIC LETTER HAH INITIAL FORM (GLYPH FOR INITIAL ARABIC HAA) */
  ["\uFEA4", "\uFEA4"], /* ARABIC LETTER HAH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC HAA) */
  ["\uFEA5", "\uFEA5"], /* ARABIC LETTER KHAH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC KHAA) */
  ["\uFEA6", "\uFEA6"], /* ARABIC LETTER KHAH FINAL FORM (GLYPH FOR FINAL ARABIC KHAA) */
  ["\uFEA7", "\uFEA7"], /* ARABIC LETTER KHAH INITIAL FORM (GLYPH FOR INITIAL ARABIC KHAA) */
  ["\uFEA8", "\uFEA8"], /* ARABIC LETTER KHAH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC KHAA) */
  ["\uFEA9", "\uFEA9"], /* ARABIC LETTER DAL ISOLATED FORM (GLYPH FOR ISOLATE ARABIC DAL) */
  ["\uFEAA", "\uFEAA"], /* ARABIC LETTER DAL FINAL FORM (GLYPH FOR FINAL ARABIC DAL) */
  ["\uFEAB", "\uFEAB"], /* ARABIC LETTER THAL ISOLATED FORM (GLYPH FOR ISOLATE ARABIC THAL) */
  ["\uFEAC", "\uFEAC"], /* ARABIC LETTER THAL FINAL FORM (GLYPH FOR FINAL ARABIC THAL) */
  ["\uFEAD", "\uFEAD"], /* ARABIC LETTER REH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC RA) */
  ["\uFEAE", "\uFEAE"], /* ARABIC LETTER REH FINAL FORM (GLYPH FOR FINAL ARABIC RA) */
  ["\uFEAF", "\uFEAF"], /* ARABIC LETTER ZAIN ISOLATED FORM (GLYPH FOR ISOLATE ARABIC ZAIN) */
  ["\uFEB0", "\uFEB0"], /* ARABIC LETTER ZAIN FINAL FORM (GLYPH FOR FINAL ARABIC ZAIN) */
  ["\uFEB1", "\uFEB1"], /* ARABIC LETTER SEEN ISOLATED FORM (GLYPH FOR ISOLATE ARABIC SEEN) */
  ["\uFEB2", "\uFEB2"], /* ARABIC LETTER SEEN FINAL FORM (GLYPH FOR FINAL ARABIC SEEN) */
  ["\uFEB3", "\uFEB3"], /* ARABIC LETTER SEEN INITIAL FORM (GLYPH FOR INITIAL ARABIC SEEN) */
  ["\uFEB4", "\uFEB4"], /* ARABIC LETTER SEEN MEDIAL FORM (GLYPH FOR MEDIAL ARABIC SEEN) */
  ["\uFEB5", "\uFEB5"], /* ARABIC LETTER SHEEN ISOLATED FORM (GLYPH FOR ISOLATE ARABIC SHEEN) */
  ["\uFEB6", "\uFEB6"], /* ARABIC LETTER SHEEN FINAL FORM (GLYPH FOR FINAL ARABIC SHEEN) */
  ["\uFEB7", "\uFEB7"], /* ARABIC LETTER SHEEN INITIAL FORM (GLYPH FOR INITIAL ARABIC SHEEN) */
  ["\uFEB8", "\uFEB8"], /* ARABIC LETTER SHEEN MEDIAL FORM (GLYPH FOR MEDIAL ARABIC SHEEN) */
  ["\uFEB9", "\uFEB9"], /* ARABIC LETTER SAD ISOLATED FORM (GLYPH FOR ISOLATE ARABIC SAD) */
  ["\uFEBA", "\uFEBA"], /* ARABIC LETTER SAD FINAL FORM (GLYPH FOR FINAL ARABIC SAD) */
  ["\uFEBB", "\uFEBB"], /* ARABIC LETTER SAD INITIAL FORM (GLYPH FOR INITIAL ARABIC SAD) */
  ["\uFEBC", "\uFEBC"], /* ARABIC LETTER SAD MEDIAL FORM (GLYPH FOR MEDIAL ARABIC SAD) */
  ["\uFEBD", "\uFEBD"], /* ARABIC LETTER DAD ISOLATED FORM (GLYPH FOR ISOLATE ARABIC DAD) */
  ["\uFEBE", "\uFEBE"], /* ARABIC LETTER DAD FINAL FORM (GLYPH FOR FINAL ARABIC DAD) */
  ["\uFEBF", "\uFEBF"], /* ARABIC LETTER DAD INITIAL FORM (GLYPH FOR INITIAL ARABIC DAD) */
  ["\uFEC0", "\uFEC0"], /* ARABIC LETTER DAD MEDIAL FORM (GLYPH FOR MEDIAL ARABIC DAD) */
  ["\uFEC1", "\uFEC1"], /* ARABIC LETTER TAH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC TAH) */
  ["\uFEC2", "\uFEC2"], /* ARABIC LETTER TAH FINAL FORM (GLYPH FOR FINAL ARABIC TAH) */
  ["\uFEC3", "\uFEC3"], /* ARABIC LETTER TAH INITIAL FORM (GLYPH FOR INITIAL ARABIC TAH) */
  ["\uFEC4", "\uFEC4"], /* ARABIC LETTER TAH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC TAH) */
  ["\uFEC5", "\uFEC5"], /* ARABIC LETTER ZAH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC DHAH) */
  ["\uFEC6", "\uFEC6"], /* ARABIC LETTER ZAH FINAL FORM (GLYPH FOR FINAL ARABIC DHAH) */
  ["\uFEC7", "\uFEC7"], /* ARABIC LETTER ZAH INITIAL FORM (GLYPH FOR INITIAL ARABIC DHAH) */
  ["\uFEC8", "\uFEC8"], /* ARABIC LETTER ZAH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC DHAH) */
  ["\uFEC9", "\uFEC9"], /* ARABIC LETTER AIN ISOLATED FORM (GLYPH FOR ISOLATE ARABIC AIN) */
  ["\uFECA", "\uFECA"], /* ARABIC LETTER AIN FINAL FORM (GLYPH FOR FINAL ARABIC AIN) */
  ["\uFECB", "\uFECB"], /* ARABIC LETTER AIN INITIAL FORM (GLYPH FOR INITIAL ARABIC AIN) */
  ["\uFECC", "\uFECC"], /* ARABIC LETTER AIN MEDIAL FORM (GLYPH FOR MEDIAL ARABIC AIN) */
  ["\uFECD", "\uFECD"], /* ARABIC LETTER GHAIN ISOLATED FORM (GLYPH FOR ISOLATE ARABIC GHAIN) */
  ["\uFECE", "\uFECE"], /* ARABIC LETTER GHAIN FINAL FORM (GLYPH FOR FINAL ARABIC GHAIN) */
  ["\uFECF", "\uFECF"], /* ARABIC LETTER GHAIN INITIAL FORM (GLYPH FOR INITIAL ARABIC GHAIN) */
  ["\uFED0", "\uFED0"], /* ARABIC LETTER GHAIN MEDIAL FORM (GLYPH FOR MEDIAL ARABIC GHAIN) */
  ["\uFED1", "\uFED1"], /* ARABIC LETTER FEH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC FA) */
  ["\uFED2", "\uFED2"], /* ARABIC LETTER FEH FINAL FORM (GLYPH FOR FINAL ARABIC FA) */
  ["\uFED3", "\uFED3"], /* ARABIC LETTER FEH INITIAL FORM (GLYPH FOR INITIAL ARABIC FA) */
  ["\uFED4", "\uFED4"], /* ARABIC LETTER FEH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC FA) */
  ["\uFED5", "\uFED5"], /* ARABIC LETTER QAF ISOLATED FORM (GLYPH FOR ISOLATE ARABIC QAF) */
  ["\uFED6", "\uFED6"], /* ARABIC LETTER QAF FINAL FORM (GLYPH FOR FINAL ARABIC QAF) */
  ["\uFED7", "\uFED7"], /* ARABIC LETTER QAF INITIAL FORM (GLYPH FOR INITIAL ARABIC QAF) */
  ["\uFED8", "\uFED8"], /* ARABIC LETTER QAF MEDIAL FORM (GLYPH FOR MEDIAL ARABIC QAF) */
  ["\uFED9", "\uFED9"], /* ARABIC LETTER KAF ISOLATED FORM (GLYPH FOR ISOLATE ARABIC CAF) */
  ["\uFEDA", "\uFEDA"], /* ARABIC LETTER KAF FINAL FORM (GLYPH FOR FINAL ARABIC CAF) */
  ["\uFEDB", "\uFEDB"], /* ARABIC LETTER KAF INITIAL FORM (GLYPH FOR INITIAL ARABIC CAF) */
  ["\uFEDC", "\uFEDC"], /* ARABIC LETTER KAF MEDIAL FORM (GLYPH FOR MEDIAL ARABIC CAF) */
  ["\uFEDD", "\uFEDD"], /* ARABIC LETTER LAM ISOLATED FORM (GLYPH FOR ISOLATE ARABIC LAM) */
  ["\uFEDE", "\uFEDE"], /* ARABIC LETTER LAM FINAL FORM (GLYPH FOR FINAL ARABIC LAM) */
  ["\uFEDF", "\uFEDF"], /* ARABIC LETTER LAM INITIAL FORM (GLYPH FOR INITIAL ARABIC LAM) */
  ["\uFEE0", "\uFEE0"], /* ARABIC LETTER LAM MEDIAL FORM (GLYPH FOR MEDIAL ARABIC LAM) */
  ["\uFEE1", "\uFEE1"], /* ARABIC LETTER MEEM ISOLATED FORM (GLYPH FOR ISOLATE ARABIC MEEM) */
  ["\uFEE2", "\uFEE2"], /* ARABIC LETTER MEEM FINAL FORM (GLYPH FOR FINAL ARABIC MEEM) */
  ["\uFEE3", "\uFEE3"], /* ARABIC LETTER MEEM INITIAL FORM (GLYPH FOR INITIAL ARABIC MEEM) */
  ["\uFEE4", "\uFEE4"], /* ARABIC LETTER MEEM MEDIAL FORM (GLYPH FOR MEDIAL ARABIC MEEM) */
  ["\uFEE5", "\uFEE5"], /* ARABIC LETTER NOON ISOLATED FORM (GLYPH FOR ISOLATE ARABIC NOON) */
  ["\uFEE6", "\uFEE6"], /* ARABIC LETTER NOON FINAL FORM (GLYPH FOR FINAL ARABIC NOON) */
  ["\uFEE7", "\uFEE7"], /* ARABIC LETTER NOON INITIAL FORM (GLYPH FOR INITIAL ARABIC NOON) */
  ["\uFEE8", "\uFEE8"], /* ARABIC LETTER NOON MEDIAL FORM (GLYPH FOR MEDIAL ARABIC NOON) */
  ["\uFEE9", "\uFEE9"], /* ARABIC LETTER HEH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HA) */
  ["\uFEEA", "\uFEEA"], /* ARABIC LETTER HEH FINAL FORM (GLYPH FOR FINAL ARABIC HA) */
  ["\uFEEB", "\uFEEB"], /* ARABIC LETTER HEH INITIAL FORM (GLYPH FOR INITIAL ARABIC HA) */
  ["\uFEEC", "\uFEEC"], /* ARABIC LETTER HEH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC HA) */
  ["\uFEED", "\uFEED"], /* ARABIC LETTER WAW ISOLATED FORM (GLYPH FOR ISOLATE ARABIC WAW) */
  ["\uFEEE", "\uFEEE"], /* ARABIC LETTER WAW FINAL FORM (GLYPH FOR FINAL ARABIC WAW) */
  ["\uFEEF", "\uFEEF"], /* ARABIC LETTER ALEF MAKSURA ISOLATED FORM (GLYPH FOR ISOLATE ARABIC ALEF MAQSURAH) */
  ["\uFEF0", "\uFEF0"], /* ARABIC LETTER ALEF MAKSURA FINAL FORM (GLYPH FOR FINAL ARABIC ALEF MAQSURAH) */
  ["\uFEF1", "\uFEF1"], /* ARABIC LETTER YEH ISOLATED FORM (GLYPH FOR ISOLATE ARABIC YA) */
  ["\uFEF2", "\uFEF2"], /* ARABIC LETTER YEH FINAL FORM (GLYPH FOR FINAL ARABIC YA) */
  ["\uFEF3", "\uFEF3"], /* ARABIC LETTER YEH INITIAL FORM (GLYPH FOR INITIAL ARABIC YA) */
  ["\uFEF4", "\uFEF4"], /* ARABIC LETTER YEH MEDIAL FORM (GLYPH FOR MEDIAL ARABIC YA) */
  ["\uFEF5", "\uFEF5"], /* ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE ISOLATED FORM (GLYPH FOR ISOLATE ARABIC MADDAH ON LIGATURE LAM ALEF) */
  ["\uFEF6", "\uFEF6"], /* ARABIC LIGATURE LAM WITH ALEF WITH MADDA ABOVE FINAL FORM (GLYPH FOR FINAL ARABIC MADDAH ON LIGATURE LAM ALEF) */
  ["\uFEF7", "\uFEF7"], /* ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH ON LIGATURE LAM ALEF) */
  ["\uFEF8", "\uFEF8"], /* ARABIC LIGATURE LAM WITH ALEF WITH HAMZA ABOVE FINAL FORM (GLYPH FOR FINAL ARABIC HAMZAH ON LIGATURE LAM ALEF) */
  ["\uFEF9", "\uFEF9"], /* ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW ISOLATED FORM (GLYPH FOR ISOLATE ARABIC HAMZAH UNDER LIGATURE LAM ALEF) */
  ["\uFEFA", "\uFEFA"], /* ARABIC LIGATURE LAM WITH ALEF WITH HAMZA BELOW FINAL FORM (GLYPH FOR FINAL ARABIC HAMZAH UNDER LIGATURE LAM ALEF) */
  ["\uFEFB", "\uFEFB"], /* ARABIC LIGATURE LAM WITH ALEF ISOLATED FORM (GLYPH FOR ISOLATE ARABIC LIGATURE LAM ALEF) */
  ["\uFEFC", "\uFEFC"], /* ARABIC LIGATURE LAM WITH ALEF FINAL FORM (GLYPH FOR FINAL ARABIC LIGATURE LAM ALEF) */
  ["\uFEFD", "\uFEFD"],
  ["\uFEFE", "\uFEFE"],
  ["\uFEFF", "\uFEFF"], /* ZERO WIDTH NO-BREAK SPACE (BYTE ORDER MARK) */
  ["\uFF00", "\uFF00"],
  ["\uFF01", "\uFF01"], /* FULLWIDTH EXCLAMATION MARK */
  ["\uFF02", "\uFF02"], /* FULLWIDTH QUOTATION MARK */
  ["\uFF03", "\uFF03"], /* FULLWIDTH NUMBER SIGN */
  ["\uFF04", "\uFF04"], /* FULLWIDTH DOLLAR SIGN */
  ["\uFF05", "\uFF05"], /* FULLWIDTH PERCENT SIGN */
  ["\uFF06", "\uFF06"], /* FULLWIDTH AMPERSAND */
  ["\uFF07", "\uFF07"], /* FULLWIDTH APOSTROPHE */
  ["\uFF08", "\uFF08"], /* FULLWIDTH LEFT PARENTHESIS (FULLWIDTH OPENING PARENTHESIS) */
  ["\uFF09", "\uFF09"], /* FULLWIDTH RIGHT PARENTHESIS (FULLWIDTH CLOSING PARENTHESIS) */
  ["\uFF0A", "\uFF0A"], /* FULLWIDTH ASTERISK */
  ["\uFF0B", "\uFF0B"], /* FULLWIDTH PLUS SIGN */
  ["\uFF0C", "\uFF0C"], /* FULLWIDTH COMMA */
  ["\uFF0D", "\uFF0D"], /* FULLWIDTH HYPHEN-MINUS */
  ["\uFF0E", "\uFF0E"], /* FULLWIDTH FULL STOP (FULLWIDTH PERIOD) */
  ["\uFF0F", "\uFF0F"], /* FULLWIDTH SOLIDUS (FULLWIDTH SLASH) */
  ["\uFF10", "\uFF10"], /* FULLWIDTH DIGIT ZERO */
  ["\uFF11", "\uFF11"], /* FULLWIDTH DIGIT ONE */
  ["\uFF12", "\uFF12"], /* FULLWIDTH DIGIT TWO */
  ["\uFF13", "\uFF13"], /* FULLWIDTH DIGIT THREE */
  ["\uFF14", "\uFF14"], /* FULLWIDTH DIGIT FOUR */
  ["\uFF15", "\uFF15"], /* FULLWIDTH DIGIT FIVE */
  ["\uFF16", "\uFF16"], /* FULLWIDTH DIGIT SIX */
  ["\uFF17", "\uFF17"], /* FULLWIDTH DIGIT SEVEN */
  ["\uFF18", "\uFF18"], /* FULLWIDTH DIGIT EIGHT */
  ["\uFF19", "\uFF19"], /* FULLWIDTH DIGIT NINE */
  ["\uFF1A", "\uFF1A"], /* FULLWIDTH COLON */
  ["\uFF1B", "\uFF1B"], /* FULLWIDTH SEMICOLON */
  ["\uFF1C", "\uFF1C"], /* FULLWIDTH LESS-THAN SIGN */
  ["\uFF1D", "\uFF1D"], /* FULLWIDTH EQUALS SIGN */
  ["\uFF1E", "\uFF1E"], /* FULLWIDTH GREATER-THAN SIGN */
  ["\uFF1F", "\uFF1F"], /* FULLWIDTH QUESTION MARK */
  ["\uFF20", "\uFF20"], /* FULLWIDTH COMMERCIAL AT */
  ["\uFF21", "\uFF41"], /* FULLWIDTH LATIN CAPITAL LETTER A */
  ["\uFF22", "\uFF42"], /* FULLWIDTH LATIN CAPITAL LETTER B */
  ["\uFF23", "\uFF43"], /* FULLWIDTH LATIN CAPITAL LETTER C */
  ["\uFF24", "\uFF44"], /* FULLWIDTH LATIN CAPITAL LETTER D */
  ["\uFF25", "\uFF45"], /* FULLWIDTH LATIN CAPITAL LETTER E */
  ["\uFF26", "\uFF46"], /* FULLWIDTH LATIN CAPITAL LETTER F */
  ["\uFF27", "\uFF47"], /* FULLWIDTH LATIN CAPITAL LETTER G */
  ["\uFF28", "\uFF48"], /* FULLWIDTH LATIN CAPITAL LETTER H */
  ["\uFF29", "\uFF49"], /* FULLWIDTH LATIN CAPITAL LETTER I */
  ["\uFF2A", "\uFF4A"], /* FULLWIDTH LATIN CAPITAL LETTER J */
  ["\uFF2B", "\uFF4B"], /* FULLWIDTH LATIN CAPITAL LETTER K */
  ["\uFF2C", "\uFF4C"], /* FULLWIDTH LATIN CAPITAL LETTER L */
  ["\uFF2D", "\uFF4D"], /* FULLWIDTH LATIN CAPITAL LETTER M */
  ["\uFF2E", "\uFF4E"], /* FULLWIDTH LATIN CAPITAL LETTER N */
  ["\uFF2F", "\uFF4F"], /* FULLWIDTH LATIN CAPITAL LETTER O */
  ["\uFF30", "\uFF50"], /* FULLWIDTH LATIN CAPITAL LETTER P */
  ["\uFF31", "\uFF51"], /* FULLWIDTH LATIN CAPITAL LETTER Q */
  ["\uFF32", "\uFF52"], /* FULLWIDTH LATIN CAPITAL LETTER R */
  ["\uFF33", "\uFF53"], /* FULLWIDTH LATIN CAPITAL LETTER S */
  ["\uFF34", "\uFF54"], /* FULLWIDTH LATIN CAPITAL LETTER T */
  ["\uFF35", "\uFF55"], /* FULLWIDTH LATIN CAPITAL LETTER U */
  ["\uFF36", "\uFF56"], /* FULLWIDTH LATIN CAPITAL LETTER V */
  ["\uFF37", "\uFF57"], /* FULLWIDTH LATIN CAPITAL LETTER W */
  ["\uFF38", "\uFF58"], /* FULLWIDTH LATIN CAPITAL LETTER X */
  ["\uFF39", "\uFF59"], /* FULLWIDTH LATIN CAPITAL LETTER Y */
  ["\uFF3A", "\uFF5A"], /* FULLWIDTH LATIN CAPITAL LETTER Z */
  ["\uFF3B", "\uFF3B"], /* FULLWIDTH LEFT SQUARE BRACKET (FULLWIDTH OPENING SQUARE BRACKET) */
  ["\uFF3C", "\uFF3C"], /* FULLWIDTH REVERSE SOLIDUS (FULLWIDTH BACKSLASH) */
  ["\uFF3D", "\uFF3D"], /* FULLWIDTH RIGHT SQUARE BRACKET (FULLWIDTH CLOSING SQUARE BRACKET) */
  ["\uFF3E", "\uFF3E"], /* FULLWIDTH CIRCUMFLEX ACCENT (FULLWIDTH SPACING CIRCUMFLEX) */
  ["\uFF3F", "\uFF3F"], /* FULLWIDTH LOW LINE (FULLWIDTH SPACING UNDERSCORE) */
  ["\uFF40", "\uFF40"], /* FULLWIDTH GRAVE ACCENT (FULLWIDTH SPACING GRAVE) */
  ["\uFF21", "\uFF41"], /* FULLWIDTH LATIN SMALL LETTER A */
  ["\uFF22", "\uFF42"], /* FULLWIDTH LATIN SMALL LETTER B */
  ["\uFF23", "\uFF43"], /* FULLWIDTH LATIN SMALL LETTER C */
  ["\uFF24", "\uFF44"], /* FULLWIDTH LATIN SMALL LETTER D */
  ["\uFF25", "\uFF45"], /* FULLWIDTH LATIN SMALL LETTER E */
  ["\uFF26", "\uFF46"], /* FULLWIDTH LATIN SMALL LETTER F */
  ["\uFF27", "\uFF47"], /* FULLWIDTH LATIN SMALL LETTER G */
  ["\uFF28", "\uFF48"], /* FULLWIDTH LATIN SMALL LETTER H */
  ["\uFF29", "\uFF49"], /* FULLWIDTH LATIN SMALL LETTER I */
  ["\uFF2A", "\uFF4A"], /* FULLWIDTH LATIN SMALL LETTER J */
  ["\uFF2B", "\uFF4B"], /* FULLWIDTH LATIN SMALL LETTER K */
  ["\uFF2C", "\uFF4C"], /* FULLWIDTH LATIN SMALL LETTER L */
  ["\uFF2D", "\uFF4D"], /* FULLWIDTH LATIN SMALL LETTER M */
  ["\uFF2E", "\uFF4E"], /* FULLWIDTH LATIN SMALL LETTER N */
  ["\uFF2F", "\uFF4F"], /* FULLWIDTH LATIN SMALL LETTER O */
  ["\uFF30", "\uFF50"], /* FULLWIDTH LATIN SMALL LETTER P */
  ["\uFF31", "\uFF51"], /* FULLWIDTH LATIN SMALL LETTER Q */
  ["\uFF32", "\uFF52"], /* FULLWIDTH LATIN SMALL LETTER R */
  ["\uFF33", "\uFF53"], /* FULLWIDTH LATIN SMALL LETTER S */
  ["\uFF34", "\uFF54"], /* FULLWIDTH LATIN SMALL LETTER T */
  ["\uFF35", "\uFF55"], /* FULLWIDTH LATIN SMALL LETTER U */
  ["\uFF36", "\uFF56"], /* FULLWIDTH LATIN SMALL LETTER V */
  ["\uFF37", "\uFF57"], /* FULLWIDTH LATIN SMALL LETTER W */
  ["\uFF38", "\uFF58"], /* FULLWIDTH LATIN SMALL LETTER X */
  ["\uFF39", "\uFF59"], /* FULLWIDTH LATIN SMALL LETTER Y */
  ["\uFF3A", "\uFF5A"], /* FULLWIDTH LATIN SMALL LETTER Z */
  ["\uFF5B", "\uFF5B"], /* FULLWIDTH LEFT CURLY BRACKET (FULLWIDTH OPENING CURLY BRACKET) */
  ["\uFF5C", "\uFF5C"], /* FULLWIDTH VERTICAL LINE (FULLWIDTH VERTICAL BAR) */
  ["\uFF5D", "\uFF5D"], /* FULLWIDTH RIGHT CURLY BRACKET (FULLWIDTH CLOSING CURLY BRACKET) */
  ["\uFF5E", "\uFF5E"], /* FULLWIDTH TILDE (FULLWIDTH SPACING TILDE) */
  ["\uFF5F", "\uFF5F"], /* FULLWIDTH LEFT WHITE PARENTHESIS */
  ["\uFF60", "\uFF60"], /* FULLWIDTH RIGHT WHITE PARENTHESIS */
  ["\uFF61", "\uFF61"], /* HALFWIDTH IDEOGRAPHIC FULL STOP (HALFWIDTH IDEOGRAPHIC PERIOD) */
  ["\uFF62", "\uFF62"], /* HALFWIDTH LEFT CORNER BRACKET (HALFWIDTH OPENING CORNER BRACKET) */
  ["\uFF63", "\uFF63"], /* HALFWIDTH RIGHT CORNER BRACKET (HALFWIDTH CLOSING CORNER BRACKET) */
  ["\uFF64", "\uFF64"], /* HALFWIDTH IDEOGRAPHIC COMMA */
  ["\uFF65", "\uFF65"], /* HALFWIDTH KATAKANA MIDDLE DOT */
  ["\uFF66", "\uFF66"], /* HALFWIDTH KATAKANA LETTER WO */
  ["\uFF67", "\uFF67"], /* HALFWIDTH KATAKANA LETTER SMALL A */
  ["\uFF68", "\uFF68"], /* HALFWIDTH KATAKANA LETTER SMALL I */
  ["\uFF69", "\uFF69"], /* HALFWIDTH KATAKANA LETTER SMALL U */
  ["\uFF6A", "\uFF6A"], /* HALFWIDTH KATAKANA LETTER SMALL E */
  ["\uFF6B", "\uFF6B"], /* HALFWIDTH KATAKANA LETTER SMALL O */
  ["\uFF6C", "\uFF6C"], /* HALFWIDTH KATAKANA LETTER SMALL YA */
  ["\uFF6D", "\uFF6D"], /* HALFWIDTH KATAKANA LETTER SMALL YU */
  ["\uFF6E", "\uFF6E"], /* HALFWIDTH KATAKANA LETTER SMALL YO */
  ["\uFF6F", "\uFF6F"], /* HALFWIDTH KATAKANA LETTER SMALL TU */
  ["\uFF70", "\uFF70"], /* HALFWIDTH KATAKANA-HIRAGANA PROLONGED SOUND MARK */
  ["\uFF71", "\uFF71"], /* HALFWIDTH KATAKANA LETTER A */
  ["\uFF72", "\uFF72"], /* HALFWIDTH KATAKANA LETTER I */
  ["\uFF73", "\uFF73"], /* HALFWIDTH KATAKANA LETTER U */
  ["\uFF74", "\uFF74"], /* HALFWIDTH KATAKANA LETTER E */
  ["\uFF75", "\uFF75"], /* HALFWIDTH KATAKANA LETTER O */
  ["\uFF76", "\uFF76"], /* HALFWIDTH KATAKANA LETTER KA */
  ["\uFF77", "\uFF77"], /* HALFWIDTH KATAKANA LETTER KI */
  ["\uFF78", "\uFF78"], /* HALFWIDTH KATAKANA LETTER KU */
  ["\uFF79", "\uFF79"], /* HALFWIDTH KATAKANA LETTER KE */
  ["\uFF7A", "\uFF7A"], /* HALFWIDTH KATAKANA LETTER KO */
  ["\uFF7B", "\uFF7B"], /* HALFWIDTH KATAKANA LETTER SA */
  ["\uFF7C", "\uFF7C"], /* HALFWIDTH KATAKANA LETTER SI */
  ["\uFF7D", "\uFF7D"], /* HALFWIDTH KATAKANA LETTER SU */
  ["\uFF7E", "\uFF7E"], /* HALFWIDTH KATAKANA LETTER SE */
  ["\uFF7F", "\uFF7F"], /* HALFWIDTH KATAKANA LETTER SO */
  ["\uFF80", "\uFF80"], /* HALFWIDTH KATAKANA LETTER TA */
  ["\uFF81", "\uFF81"], /* HALFWIDTH KATAKANA LETTER TI */
  ["\uFF82", "\uFF82"], /* HALFWIDTH KATAKANA LETTER TU */
  ["\uFF83", "\uFF83"], /* HALFWIDTH KATAKANA LETTER TE */
  ["\uFF84", "\uFF84"], /* HALFWIDTH KATAKANA LETTER TO */
  ["\uFF85", "\uFF85"], /* HALFWIDTH KATAKANA LETTER NA */
  ["\uFF86", "\uFF86"], /* HALFWIDTH KATAKANA LETTER NI */
  ["\uFF87", "\uFF87"], /* HALFWIDTH KATAKANA LETTER NU */
  ["\uFF88", "\uFF88"], /* HALFWIDTH KATAKANA LETTER NE */
  ["\uFF89", "\uFF89"], /* HALFWIDTH KATAKANA LETTER NO */
  ["\uFF8A", "\uFF8A"], /* HALFWIDTH KATAKANA LETTER HA */
  ["\uFF8B", "\uFF8B"], /* HALFWIDTH KATAKANA LETTER HI */
  ["\uFF8C", "\uFF8C"], /* HALFWIDTH KATAKANA LETTER HU */
  ["\uFF8D", "\uFF8D"], /* HALFWIDTH KATAKANA LETTER HE */
  ["\uFF8E", "\uFF8E"], /* HALFWIDTH KATAKANA LETTER HO */
  ["\uFF8F", "\uFF8F"], /* HALFWIDTH KATAKANA LETTER MA */
  ["\uFF90", "\uFF90"], /* HALFWIDTH KATAKANA LETTER MI */
  ["\uFF91", "\uFF91"], /* HALFWIDTH KATAKANA LETTER MU */
  ["\uFF92", "\uFF92"], /* HALFWIDTH KATAKANA LETTER ME */
  ["\uFF93", "\uFF93"], /* HALFWIDTH KATAKANA LETTER MO */
  ["\uFF94", "\uFF94"], /* HALFWIDTH KATAKANA LETTER YA */
  ["\uFF95", "\uFF95"], /* HALFWIDTH KATAKANA LETTER YU */
  ["\uFF96", "\uFF96"], /* HALFWIDTH KATAKANA LETTER YO */
  ["\uFF97", "\uFF97"], /* HALFWIDTH KATAKANA LETTER RA */
  ["\uFF98", "\uFF98"], /* HALFWIDTH KATAKANA LETTER RI */
  ["\uFF99", "\uFF99"], /* HALFWIDTH KATAKANA LETTER RU */
  ["\uFF9A", "\uFF9A"], /* HALFWIDTH KATAKANA LETTER RE */
  ["\uFF9B", "\uFF9B"], /* HALFWIDTH KATAKANA LETTER RO */
  ["\uFF9C", "\uFF9C"], /* HALFWIDTH KATAKANA LETTER WA */
  ["\uFF9D", "\uFF9D"], /* HALFWIDTH KATAKANA LETTER N */
  ["\uFF9E", "\uFF9E"], /* HALFWIDTH KATAKANA VOICED SOUND MARK */
  ["\uFF9F", "\uFF9F"], /* HALFWIDTH KATAKANA SEMI-VOICED SOUND MARK */
  ["\uFFA0", "\uFFA0"], /* HALFWIDTH HANGUL FILLER (HALFWIDTH HANGUL CAE OM) */
  ["\uFFA1", "\uFFA1"], /* HALFWIDTH HANGUL LETTER KIYEOK (HALFWIDTH HANGUL LETTER GIYEOG) */
  ["\uFFA2", "\uFFA2"], /* HALFWIDTH HANGUL LETTER SSANGKIYEOK (HALFWIDTH HANGUL LETTER SSANG GIYEOG) */
  ["\uFFA3", "\uFFA3"], /* HALFWIDTH HANGUL LETTER KIYEOK-SIOS (HALFWIDTH HANGUL LETTER GIYEOG SIOS) */
  ["\uFFA4", "\uFFA4"], /* HALFWIDTH HANGUL LETTER NIEUN */
  ["\uFFA5", "\uFFA5"], /* HALFWIDTH HANGUL LETTER NIEUN-CIEUC (HALFWIDTH HANGUL LETTER NIEUN JIEUJ) */
  ["\uFFA6", "\uFFA6"], /* HALFWIDTH HANGUL LETTER NIEUN-HIEUH (HALFWIDTH HANGUL LETTER NIEUN HIEUH) */
  ["\uFFA7", "\uFFA7"], /* HALFWIDTH HANGUL LETTER TIKEUT (HALFWIDTH HANGUL LETTER DIGEUD) */
  ["\uFFA8", "\uFFA8"], /* HALFWIDTH HANGUL LETTER SSANGTIKEUT (HALFWIDTH HANGUL LETTER SSANG DIGEUD) */
  ["\uFFA9", "\uFFA9"], /* HALFWIDTH HANGUL LETTER RIEUL (HALFWIDTH HANGUL LETTER LIEUL) */
  ["\uFFAA", "\uFFAA"], /* HALFWIDTH HANGUL LETTER RIEUL-KIYEOK (HALFWIDTH HANGUL LETTER LIEUL GIYEOG) */
  ["\uFFAB", "\uFFAB"], /* HALFWIDTH HANGUL LETTER RIEUL-MIEUM (HALFWIDTH HANGUL LETTER LIEUL MIEUM) */
  ["\uFFAC", "\uFFAC"], /* HALFWIDTH HANGUL LETTER RIEUL-PIEUP (HALFWIDTH HANGUL LETTER LIEUL BIEUB) */
  ["\uFFAD", "\uFFAD"], /* HALFWIDTH HANGUL LETTER RIEUL-SIOS (HALFWIDTH HANGUL LETTER LIEUL SIOS) */
  ["\uFFAE", "\uFFAE"], /* HALFWIDTH HANGUL LETTER RIEUL-THIEUTH (HALFWIDTH HANGUL LETTER LIEUL TIEUT) */
  ["\uFFAF", "\uFFAF"], /* HALFWIDTH HANGUL LETTER RIEUL-PHIEUPH (HALFWIDTH HANGUL LETTER LIEUL PIEUP) */
  ["\uFFB0", "\uFFB0"], /* HALFWIDTH HANGUL LETTER RIEUL-HIEUH (HALFWIDTH HANGUL LETTER LIEUL HIEUH) */
  ["\uFFB1", "\uFFB1"], /* HALFWIDTH HANGUL LETTER MIEUM */
  ["\uFFB2", "\uFFB2"], /* HALFWIDTH HANGUL LETTER PIEUP (HALFWIDTH HANGUL LETTER BIEUB) */
  ["\uFFB3", "\uFFB3"], /* HALFWIDTH HANGUL LETTER SSANGPIEUP (HALFWIDTH HANGUL LETTER SSANG BIEUB) */
  ["\uFFB4", "\uFFB4"], /* HALFWIDTH HANGUL LETTER PIEUP-SIOS (HALFWIDTH HANGUL LETTER BIEUB SIOS) */
  ["\uFFB5", "\uFFB5"], /* HALFWIDTH HANGUL LETTER SIOS */
  ["\uFFB6", "\uFFB6"], /* HALFWIDTH HANGUL LETTER SSANGSIOS (HALFWIDTH HANGUL LETTER SSANG SIOS) */
  ["\uFFB7", "\uFFB7"], /* HALFWIDTH HANGUL LETTER IEUNG */
  ["\uFFB8", "\uFFB8"], /* HALFWIDTH HANGUL LETTER CIEUC (HALFWIDTH HANGUL LETTER JIEUJ) */
  ["\uFFB9", "\uFFB9"], /* HALFWIDTH HANGUL LETTER SSANGCIEUC (HALFWIDTH HANGUL LETTER SSANG JIEUJ) */
  ["\uFFBA", "\uFFBA"], /* HALFWIDTH HANGUL LETTER CHIEUCH (HALFWIDTH HANGUL LETTER CIEUC) */
  ["\uFFBB", "\uFFBB"], /* HALFWIDTH HANGUL LETTER KHIEUKH (HALFWIDTH HANGUL LETTER KIYEOK) */
  ["\uFFBC", "\uFFBC"], /* HALFWIDTH HANGUL LETTER THIEUTH (HALFWIDTH HANGUL LETTER TIEUT) */
  ["\uFFBD", "\uFFBD"], /* HALFWIDTH HANGUL LETTER PHIEUPH (HALFWIDTH HANGUL LETTER PIEUP) */
  ["\uFFBE", "\uFFBE"], /* HALFWIDTH HANGUL LETTER HIEUH */
  ["\uFFBF", "\uFFBF"],
  ["\uFFC0", "\uFFC0"],
  ["\uFFC1", "\uFFC1"],
  ["\uFFC2", "\uFFC2"], /* HALFWIDTH HANGUL LETTER A */
  ["\uFFC3", "\uFFC3"], /* HALFWIDTH HANGUL LETTER AE */
  ["\uFFC4", "\uFFC4"], /* HALFWIDTH HANGUL LETTER YA */
  ["\uFFC5", "\uFFC5"], /* HALFWIDTH HANGUL LETTER YAE */
  ["\uFFC6", "\uFFC6"], /* HALFWIDTH HANGUL LETTER EO */
  ["\uFFC7", "\uFFC7"], /* HALFWIDTH HANGUL LETTER E */
  ["\uFFC8", "\uFFC8"],
  ["\uFFC9", "\uFFC9"],
  ["\uFFCA", "\uFFCA"], /* HALFWIDTH HANGUL LETTER YEO */
  ["\uFFCB", "\uFFCB"], /* HALFWIDTH HANGUL LETTER YE */
  ["\uFFCC", "\uFFCC"], /* HALFWIDTH HANGUL LETTER O */
  ["\uFFCD", "\uFFCD"], /* HALFWIDTH HANGUL LETTER WA */
  ["\uFFCE", "\uFFCE"], /* HALFWIDTH HANGUL LETTER WAE */
  ["\uFFCF", "\uFFCF"], /* HALFWIDTH HANGUL LETTER OE */
  ["\uFFD0", "\uFFD0"],
  ["\uFFD1", "\uFFD1"],
  ["\uFFD2", "\uFFD2"], /* HALFWIDTH HANGUL LETTER YO */
  ["\uFFD3", "\uFFD3"], /* HALFWIDTH HANGUL LETTER U */
  ["\uFFD4", "\uFFD4"], /* HALFWIDTH HANGUL LETTER WEO */
  ["\uFFD5", "\uFFD5"], /* HALFWIDTH HANGUL LETTER WE */
  ["\uFFD6", "\uFFD6"], /* HALFWIDTH HANGUL LETTER WI */
  ["\uFFD7", "\uFFD7"], /* HALFWIDTH HANGUL LETTER YU */
  ["\uFFD8", "\uFFD8"],
  ["\uFFD9", "\uFFD9"],
  ["\uFFDA", "\uFFDA"], /* HALFWIDTH HANGUL LETTER EU */
  ["\uFFDB", "\uFFDB"], /* HALFWIDTH HANGUL LETTER YI */
  ["\uFFDC", "\uFFDC"], /* HALFWIDTH HANGUL LETTER I */
  ["\uFFDD", "\uFFDD"],
  ["\uFFDE", "\uFFDE"],
  ["\uFFDF", "\uFFDF"],
  ["\uFFE0", "\uFFE0"], /* FULLWIDTH CENT SIGN */
  ["\uFFE1", "\uFFE1"], /* FULLWIDTH POUND SIGN */
  ["\uFFE2", "\uFFE2"], /* FULLWIDTH NOT SIGN */
  ["\uFFE3", "\uFFE3"], /* FULLWIDTH MACRON (FULLWIDTH SPACING MACRON) */
  ["\uFFE4", "\uFFE4"], /* FULLWIDTH BROKEN BAR (FULLWIDTH BROKEN VERTICAL BAR) */
  ["\uFFE5", "\uFFE5"], /* FULLWIDTH YEN SIGN */
  ["\uFFE6", "\uFFE6"], /* FULLWIDTH WON SIGN */
  ["\uFFE7", "\uFFE7"],
  ["\uFFE8", "\uFFE8"], /* HALFWIDTH FORMS LIGHT VERTICAL */
  ["\uFFE9", "\uFFE9"], /* HALFWIDTH LEFTWARDS ARROW */
  ["\uFFEA", "\uFFEA"], /* HALFWIDTH UPWARDS ARROW */
  ["\uFFEB", "\uFFEB"], /* HALFWIDTH RIGHTWARDS ARROW */
  ["\uFFEC", "\uFFEC"], /* HALFWIDTH DOWNWARDS ARROW */
  ["\uFFED", "\uFFED"], /* HALFWIDTH BLACK SQUARE */
  ["\uFFEE", "\uFFEE"], /* HALFWIDTH WHITE CIRCLE */
  ["\uFFEF", "\uFFEF"],
  ["\uFFF0", "\uFFF0"],
  ["\uFFF1", "\uFFF1"],
  ["\uFFF2", "\uFFF2"],
  ["\uFFF3", "\uFFF3"],
  ["\uFFF4", "\uFFF4"],
  ["\uFFF5", "\uFFF5"],
  ["\uFFF6", "\uFFF6"],
  ["\uFFF7", "\uFFF7"],
  ["\uFFF8", "\uFFF8"],
  ["\uFFF9", "\uFFF9"], /* INTERLINEAR ANNOTATION ANCHOR */
  ["\uFFFA", "\uFFFA"], /* INTERLINEAR ANNOTATION SEPARATOR */
  ["\uFFFB", "\uFFFB"], /* INTERLINEAR ANNOTATION TERMINATOR */
  ["\uFFFC", "\uFFFC"], /* OBJECT REPLACEMENT CHARACTER */
  ["\uFFFD", "\uFFFD"], /* REPLACEMENT CHARACTER */
  ["\uFFFE", "\uFFFE"],
  ["\uFFFF", "\uFFFF"],
];

assertEq(mapping.length, 0x10000);
for (var i = 0; i <= 0xffff; i++) {
    var char = String.fromCharCode(i);
    var info = mapping[i];

    assertEq(char.toUpperCase(), info[0]);
    assertEq(char.toLowerCase(), info[1]);
}

if (typeof reportCompare === "function")
    reportCompare(true, true);
